FUNCTION LineByLINE, struct, $
                     TimeRange, TimeWidth, $
                     Guess, $
                     W_Lasers, W_Lasers_E, $
                     Wave, $
                     Time, $
                     WAVEE = WaveE, $
                     GUESSWIDTH = GuessWidth

;Output
;   ExpArr     = 2-dimensional array --
;                1D is radiant exposure,
;                2D specifies lineout,
;                (i.e., ExpArr[*,i] is the radiant exposure array for the ith 
;                lineout - beginning at TimeRange[0]+TimeWidth*i - to be 
;                plotted against Wave (below))
;
;Input
;   str        = structure array number of streak image or string of .pff filename
;   TimeRange  = 2-element vector giving temporal boundaries for lineouts
;   TimeWidth  = duration of time integration for lineout
;   Guess      = 2-element vector giving initial guess x positions of laser peaks 
;                (microns)
;   W_Lasers   = 2-element vector giving wavelengths of laser fiducials
;   W_Lasers_E = 2-element vector giving uncertainties of wavelengths of laser 
;                fiducials
;   Wave       = array of type double that will contain the wavelength-corrected 
;                horizontal axis corresponding to each lineout
;   Time       = vector that will contain start times in ns of each lineout
;   WaveE      = 2-dimensional vector --
;                1D will contain uncertainty of Wave,
;                2D will specify lineout
;
;Optional Input
;   GuessWidth = 2-element vector giving half width of x position range that 
;                function will search for laser peak
;                If no input, default value is [2.0e3,2.0e3]

;Handle full path name to .pff streak image or use structure # that is provided
IF DATATYPE(struct) EQ 'STR' THEN BEGIN
  OPENPFF,struct
  READSTR,str
  CLOSEPFF
ENDIF ELSE str=struct

;Create variables
nlo=FLOOR((TimeRange[1]-TimeRange[0])/TimeWidth)
XPos  = DBLARR(nlo,N_ELEMENTS(W_Lasers))
XPosE = DBLARR(nlo,N_ELEMENTS(W_Lasers))
XWid  = DBLARR(nlo,N_ELEMENTS(W_Lasers))
;XWidE = DBLARR(nlo,N_ELEMENTS(W_Lasers))
Time=DBLARR(nlo)
WDispArr=DBLARR(nlo)
WDispArrE=DBLARR(nlo)

Option=' '
ProblemInd=LONARR(nlo,N_ELEMENTS(W_Lasers))-1

IF NOT KEYWORD_SET(GuessWidth) THEN GuessWidth=[1000.,1000.]

Angstrom = '!6!sA!r!u!9 % !6!n' ;For plotting later

FOR m=0,9 DO BEGIN
IF (m GT 0) AND (STRPOS(Option,'1') GE 0) THEN BEGIN
  Guess=NGuess
  GuessWidth=NGuessWidth
ENDIF

;Determine wavelength dispersion for each lineout
;The first lineout sets the precedent for Wave
FOR i=0,nlo-1 DO BEGIN
  
  ;Create lineout from streak image
  LINSTR,str,i+300,TimeRange[0]+TimeWidth*(i+.5),WIDTH=TimeWidth,/HORIZ,/AVERAGE
  w2i,XPosArr,Exp,i+300
  
  ;Find positions (microns) by fitting Voigt profiles to laser lines
  FOR j=0,N_ELEMENTS(XPos[i,*])-1 DO BEGIN
    Temp=MIN( ABS(XPosArr-Guess[j]+GuessWidth[j]),Left )
    Temp=MIN( ABS(XPosArr-Guess[j]-GuessWidth[j]),Right )
    ;VFit=VOIGT_FIT(XPosArr[Left:Right],Exp[Left:Right],A,SigmaA)
    GFit=GAUSSFIT(XPosArr[Left:Right],Exp[Left:Right],B,SIGMA=SigmaB,NTERMS=4)
    ;Using Voigt
    ;XPos[i,j]=A[4]
    ;XPosE[i,j]=SigmaA[4]
    ;XWid[i,j]=2*SQRT(ALOG(2))*A(5);assuming that the profile is nearly Gaussian
    ;Using Gaussian
    XPos[i,j] =B[1]
    XPosE[i,j]=SigmaB[1]
    XWid[i,j] =B[2]
    ;XWidE[i,j]=SigmaB[2]
    ;IF i eq 5 and m eq 1 then stop
  ENDFOR

  ;Currently only equipped to handle a linear dispersion using two fiducials
  IF N_ELEMENTS(W_Lasers) NE 2 THEN STOP
  ;For problem lineouts, use average value for X Position
  FOR n=0,1 DO IF ProblemInd[i,n] GE 0 THEN XPos[i,n]=Meds[n]
  WDisp=(W_Lasers[1]-W_Lasers[0])/(XPos[i,1]-XPos[i,0])
  PartA =  1./(XPos[i,1]-XPos[i,0])
  PartB = -1./(XPos[i,1]-XPos[i,0])
  PartC = -(W_Lasers[1]-W_Lasers[0])/(XPos[i,1]-XPos[i,0])/(XPos[i,1]-XPos[i,0])
  PartD =  (W_Lasers[1]-W_Lasers[0])/(XPos[i,1]-XPos[i,0])/(XPos[i,1]-XPos[i,0])
  WDispE=SQRT( PartA*PartA*W_Lasers_E[1]*W_Lasers_E[1] + $
               PartB*PartB*W_Lasers_E[0]*W_Lasers_E[0] + $
               PartC*PartC*XPosE[i,1]*XPosE[i,1] + $
               PartD*PartD*XPosE[i,0]*XPosE[i,0])
  
  ;If first lineout, create Wave that will be used for all lineouts
  IF i EQ 0 THEN BEGIN
    Wave  = DBLARR(N_ELEMENTS(XPosArr),/NOZERO)
    WaveE = DBLARR(N_ELEMENTS(XPosArr),nlo,/NOZERO)
    Wave  = XPosArr*WDisp+(W_Lasers[1]-XPos[i,1]*WDisp)
    ExpArr=DBLARR(N_ELEMENTS(Exp),nlo,/NOZERO)
    ExpArr[*,i]=Exp
  ENDIF ELSE BEGIN
    ;If not first lineout, map Exp onto Wave from first lineout
    WTemp=XPosArr*WDisp+(W_Lasers[1]-XPos[i,1]*WDisp)
    ;Temp=INTERPOLATE_MAP(WTemp,Wave)
    ;ExpArr[*,i]=INTERPOLATE(Exp,Temp)
    ExpArr[*,i]=INTERPOL(Exp,WTemp,Wave)
  ENDELSE
  ;assuming XPosArr is known to infinite precision
  ;assuming these quantities are uncorrelated, which they are not
  ;neglecting uncertainty due to interpolation
  WaveE[*,i] = SQRT( (XPosArr-XPos[i,1])*(XPosArr-XPos[i,1])*WDispE*WDispE + $
                 W_Lasers_E[1]*W_Lasers_E[1] + $
                 WDisp*WDisp*XPosE[i,1]*XPosE[i,1] )
  
  Time[i]=FLOAT(TimeRange[0]+i*TimeWidth)
  WDispArr[i]=WDisp
  WDispArrE[i]=WDispE
  ;IF i eq 5 and m eq 1 then stop
ENDFOR

  ;Plot results
  LOADCT,12,/SILENT
  PLOT,Time,WDispArr,PSYM=-SYM(4),XSTY=1,$
  XTITLE='Time (ns)',YTITLE='Wavelength Dispersion ('+Angstrom+'/!7l!6m)',$
  CHARSIZE=1.5
  OPLOT,Time,WDispArrE,PSYM=-SYM(4),COLOR=200
  XYOUTS,0.7,0.25,'Uncertainty',/NORMAL,COLOR=200,CHARSIZE=1.25
  PLOTS,Time[0],SQRT(VARIANCE(WDispArr))
  PLOTS,Time[N_ELEMENTS(Time)-1],SQRT(VARIANCE(WDispArr)),/CONTINUE,COLOR=90
  XYOUTS,0.7,0.2,'Standard Deviation',/NORMAL,COLOR=90,CHARSIZE=1.25

;STOP
  
  PRINT,'What shall I do?'
  PRINT,'(1) Revise GUESS parameters and try again'
  PRINT,'(2) Specify lineouts with problems and try again'
  PRINT,'(3) Nothing.  I am satisfied.  Get me out of here!'
  Option=' '
  READ,Option
  If (STRPOS(Option,'3') GE 0) OR (Option EQ '') THEN BREAK ELSE BEGIN


  IF (STRPOS(Option,'1') GE 0) OR (STRPOS(Option,'2') GE 0) THEN BEGIN
    PLOT,XPos[*,0],Time,PSYM=-SYM(4),XSTY=1,XRA=[16000,40000],YSTY=1,$
    YTITLE='Time (ns)',XTITLE='X Position (!7l!6m)',$
    TITLE='Position of Lasers from Fits',CHARSIZE=1.5
    Meds=DBLARR(2)
    FOR jj=0,1 DO BEGIN
      OPLOT,XPos[*,jj]+XPosE[*,jj],Time,COLOR=200
      OPLOT,XPos[*,jj]-XPosE[*,jj],Time,COLOR=200
      OPLOT,XPos[*,jj],Time,PSYM=-SYM(4)
      Meds[jj]=MEDIAN(XPos[*,jj])
      PLOTS,Meds[jj],Time[0]
      PLOTS,Meds[jj],Time[N_ELEMENTS(Time)-1],/CONTINUE,COLOR=90
      ;FOR ii=0,N_ELEMENTS(Time)-1 DO BEGIN
      ;  PLOTS,1000+XPos[ii,jj]-XWid[ii,jj],Time[ii]
      ;  PLOTS,1000+XPos[ii,jj]+XWid[ii,jj],Time[ii],/CONTINUE
      ;ENDFOR
    ENDFOR
    XYOUTS,0.5,0.25,'Uncertainty',/NORMAL,COLOR=200,CHARSIZE=1.25
    XYOUTS,0.5,0.2,'Median',/NORMAL,COLOR=90,CHARSIZE=1.25
    
    PRINT,'Leftmost laser, rightmost laser, or both? (l/r/b)'
    temp=' '
    READ,temp
    GoGuess=INTARR(2)
    If STRPOS(STRUPCASE(temp),'L') GE 0 THEN GoGuess[0]=1
    If STRPOS(STRUPCASE(temp),'R') GE 0 THEN GoGuess[1]=1
    If (STRPOS(STRUPCASE(temp),'B') GE 0) OR (temp EQ '') THEN GoGuess[*]=1
  ENDIF

  ;REVISION OPTION
  IF (STRPOS(Option,'1') GE 0) THEN BEGIN
    ;Revise guess values for laser peaks
    NGuess=DBLARR(2)
    NGuessWidth=DBLARR(2)
    FOR n=0,N_ELEMENTS(Guess)-1 DO BEGIN
      IF GoGuess[n] EQ 0 THEN BEGIN
        NGuess[n]=Guess[n]
        NGuessWidth[n]=GuessWidth[n]
        CONTINUE
      ENDIF
      IF n EQ 0 THEN PRINT,'Use median value for X Position for leftmost laser? (y/n)'
      IF n EQ 1 THEN PRINT,'Use median value for X Position for rightmost laser? (y/n)'
      temp=' '
      READ,temp
      If STRPOS(STRUPCASE(temp),'N') GE 0 THEN BEGIN
        ;Exclude XPos and XWid where: 
        ;  XPos is beyond GuessWidth,
        ;  XWid is larger than 10% of the median XWid,
        ;  XWid is larger than GuessWidth
        TempInd=WHERE( (XPos[*,n] GT Guess[n]-GuessWidth[n]) AND $
                       (XPos[*,n] LT Guess[n]+GuessWidth[n]) AND $
                       (XWid[*,n] GT MEDIAN(XWid[*,n])*0.1) AND $
                       (XWid[*,n] LT GuessWidth[n]))
        NXPos=XPos[TempInd,n]
        BinSize=((MEDIAN(NXPos)+SQRT(VARIANCE(NXPos)))-(MEDIAN(NXPos)-SQRT(VARIANCE(NXPos))))/SQRT(N_ELEMENTS(NXPos))
        BinSize2=(MAX(NXPos)-MIN(NXPos))/SQRT(N_ELEMENTS(NXPos))
        IF BinSize2 LT BinSize THEN BinSize=BinSize2
        PLOTHIST,NXPos,xhist,yhist,BIN=BinSize,/NOPLOT
        Result=GAUSSFIT(xhist,yhist,A,NTERMS=4)
        NGuess[n]=A[1]
      ENDIF ELSE BEGIN
        NGuess[n]=MEDIAN(XPos[*,n])
      ENDELSE
      TempInd=WHERE( (XWid[*,n] GT MEDIAN(XWid[*,n])*0.1) AND $
                     (XWid[*,n] LT GuessWidth[n]))
      NXWid=XWid[TempInd,n]
      BinSize=((MEDIAN(NXWid)+SQRT(VARIANCE(NXWid)))-(MEDIAN(NXWid)-SQRT(VARIANCE(NXWid))))/SQRT(N_ELEMENTS(NXWid))
      BinSize2=(MAX(NXWid)-MIN(NXWid))/SQRT(N_ELEMENTS(NXWid))
      IF BinSize2 LT BinSize THEN BinSize=BinSize2
      PLOTHIST,NXWid,xhist,yhist,BIN=BinSize,/NOPLOT
      Result=GAUSSFIT(xhist,yhist,A,NTERMS=4)
      IF A[1] LT 0 THEN NGuessWidth[n]=MEDIAN(NXWid)*2.5 ELSE NGuessWidth[n]=A[1]*2.5
;STOP

    ENDFOR

    IF (NGuess[0] EQ Guess[0]) AND (NGuessWidth[0] EQ GuessWidth[0]) AND $
       (NGuess[1] EQ Guess[1]) AND (NGuessWidth[1] EQ GuessWidth[1]) THEN BREAK

    PRINT,'Previous GUESS values:'
    PRINT,Guess
    PRINT,'New GUESS values:
    PRINT,NGuess
    PRINT,'Previous GUESSWIDTH values:'
    PRINT,GuessWidth
    PRINT,'New GUESSWIDTH values:'
    PRINT,NGuessWidth
    PRINT,''

  ENDIF
  
  ;INTERPOLATE-AROUND-PROBLEM-AREAS OPTION
  IF (STRPOS(Option,'2') GE 0) THEN BEGIN
    FOR n=0,N_ELEMENTS(Guess)-1 DO BEGIN
      IF GoGuess[n] EQ 0 THEN CONTINUE ELSE BEGIN
        IF n EQ 0 THEN PRINT,'The leftmost laser:'
        IF n EQ 1 THEN PRINT,'The rightmost laser:'
        temp='go'
        WHILE (temp NE '') DO BEGIN
          PRINT,'Enter index of problem lineout.  Press ENTER to stop.'
          READ,temp
          IF Temp EQ '' THEN BREAK
          Ind=LONG(temp)
          
          PLOT,XPos[*,0],Time,PSYM=-SYM(4),XSTY=1,XRA=[16000,40000],YSTY=1,$
          YTITLE='Time (ns)',XTITLE='X Position (!7l!6m)',$
          TITLE='Position of Lasers from Fits',CHARSIZE=1.5
          Meds=DBLARR(2)
          FOR jj=0,1 DO BEGIN
            OPLOT,XPos[*,jj]+XPosE[*,jj],Time,COLOR=200
            OPLOT,XPos[*,jj]-XPosE[*,jj],Time,COLOR=200
            OPLOT,XPos[*,jj],Time,PSYM=-SYM(4)
            Meds[jj]=MEDIAN(XPos[*,jj])
            PLOTS,Meds[jj],Time[0]
            PLOTS,Meds[jj],Time[N_ELEMENTS(Time)-1],/CONTINUE,COLOR=90
          ENDFOR
          XYOUTS,0.5,0.25,'Uncertainty',/NORMAL,COLOR=200,CHARSIZE=1.25
          XYOUTS,0.5,0.2,'Median',/NORMAL,COLOR=90,CHARSIZE=1.25
          
          LOADCT,5,/SILENT
          Places=ProblemInd[*,n]
          PLOTS,XPos[WHERE(Places EQ 1),n],Time[WHERE(Places EQ 1)],PSYM=SYM(4),COLOR=200
          PLOTS,XPos[Ind,n],Time[Ind],PSYM=SYM(4),SYMSIZE=2,COLOR=200
          LOADCT,12,/SILENT
          PRINT,'This one? (y/n)'
          tempq=' '
          READ,tempq
          
          WHILE (STRPOS(STRUPCASE(tempq),'N') NE -1) DO BEGIN
            PRINT,'Enter index of problem lineout:'
            READ,temp
            IF temp EQ '' THEN BREAK
            Ind=LONG(temp)
            PLOT,XPos[*,0],Time,PSYM=-SYM(4),XSTY=1,XRA=[16000,40000],YSTY=1,$
            YTITLE='Time (ns)',XTITLE='X Position (!7l!6m)',$
            TITLE='Position of Lasers from Fits',CHARSIZE=1.5
            Meds=DBLARR(2)
            FOR jj=0,1 DO BEGIN
              OPLOT,XPos[*,jj]+XPosE[*,jj],Time,COLOR=200
              OPLOT,XPos[*,jj]-XPosE[*,jj],Time,COLOR=200
              OPLOT,XPos[*,jj],Time,PSYM=-SYM(4)
              Meds[jj]=MEDIAN(XPos[*,jj])
              PLOTS,Meds[jj],Time[0]
              PLOTS,Meds[jj],Time[N_ELEMENTS(Time)-1],/CONTINUE,COLOR=90
            ENDFOR
            XYOUTS,0.5,0.25,'Uncertainty',/NORMAL,COLOR=200,CHARSIZE=1.25
            XYOUTS,0.5,0.2,'Median',/NORMAL,COLOR=90,CHARSIZE=1.25
            
            LOADCT,5,/SILENT
            PLOTS,XPos[WHERE(Places EQ 1),n],Time[WHERE(Places EQ 1)],PSYM=SYM(4),COLOR=200
            PLOTS,XPos[Ind,n],Time[Ind],PSYM=SYM(4),SYMSIZE=2,COLOR=200
            LOADCT,12,/SILENT
            PRINT,'This one? (y/n)'
            READ,tempq
          ENDWHILE
          
          IF (STRPOS(STRUPCASE(tempq),'N') EQ -1) THEN ProblemInd[Ind,n]=1
      
        ENDWHILE
      ENDELSE
      
    ENDFOR
  ENDIF
  ENDELSE
  
;STOP
ENDFOR


;;Plot final results
PLOT,Time,WDispArr,PSYM=-SYM(4),XSTY=1,$
XTITLE='Time (ns)',YTITLE='Wavelength Dispersion ('+Angstrom+'/!7l!6m)',$
CHARSIZE=1.5
OPLOT,Time,WDispArrE,PSYM=-SYM(4),COLOR=200
XYOUTS,0.7,0.25,'Uncertainty',/NORMAL,COLOR=200,CHARSIZE=1.25
PLOTS,Time[0],SQRT(VARIANCE(WDispArr))
PLOTS,Time[N_ELEMENTS(Time)-1],SQRT(VARIANCE(WDispArr)),/CONTINUE,COLOR=90
XYOUTS,0.7,0.2,'Standard Deviation',/NORMAL,COLOR=90,CHARSIZE=1.25

LOADCT,5,/SILENT

;STOP
RETURN,ExpArr

END