;
; Copyright 1999 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
;+
;  Fit a WDF array to a polynomial using the IDL user function, SVDFIT.
;  Up to 3 different polynomials may be used.
;  Calculated fit is stored in a WDF array and , optionally, written to a file.
;  Note: Scale_regrid, with default parametes, is used to calculate values.
;
;  Format:  WDFIT3, wd1 [, wd2] [, /YLOG] [, /YTYPE] [, VALID = valid]
;                   [, FUDGE = fudge] [, /Exact_Range] [, XYDATA = xydatain]
;  where:
;    wd1, wd2 - WDF array numbers
;               if wd2 is present, the final curve is stored in wd2
;               otherwise the result is stored in MAXWDFARRAY.
;    Ytype    - If present and not zero, then a log fit in the Y direction
;               is performed.  Only points greater than zero will be used in
;               the log direction.
;    YLog     - Same as ytype.
;    VALID    - Domain of validity for the fit.
;               Default: valid = [0.95*minimum, 1.02*maximum]
;    Exact_Range - If set VALID = [minimum, maximum]
;    Fudge    - 2 element vector sent to scale_regrid (see PFIDL2/scale_regrid)
;               Used to insure that the fitted function is continuous
;               Default values are .05 of VALID
;    XYDATA   - If set to zero or a negative number, the data will be
;               interpolated to a uniform grid i fit is X-Y data.
;               If XYDATA is negative then XYDATA is the number of points in
;               the linear interpolation.  Default: NPOINT = 1024
;               Default:  XYDATA = 1, use exact data points
;
;  Discussion:
;       The first execution of the command will be performed with:
;         Number of Breaks = 2
;         Break Points =[ Minimum + range/16, Maximum - range/16]
;                       where range = maximum-minimum
;         Degree =[ 2, 5, 3]
;       In the case of a singular value degree will automatically be reduced to
;       obtain a fit.
;       Current version adds a minimum of one point past the break point
;         and will add two points if it is within 1/16 of the domain of the fit.
;
;  Examples:
;
;    Fit data points in array 2 to a polynomial multiple polynomials.
;    Save the fit in array 3.
;      WDfit3, 2, 3
;-
;
 pro wdfit3_wid_event, event
;
; Common block
@wdfit3_common.dat

on_error, 1
;  widget_control, event.id, get_value = ev
   widget_control, event.top, get_uvalue = info, /no_copy
   if event.value  lt 3 then begin
          ds = degree_save
;     info = {vmin: valid_min_id, vmax: valid_max_id, nbr: nbreak_id, $
;          br1_val: break1_id, br2_val: break2_id, pdeg1: degree_1_id, $
;          pdeg2: degree_2_id, pdeg3: degree_3_id, offset: offset_real}

          widget_control, info.vmin, get_value = vmin
          widget_control, info.vmax, get_value = vmax
          valid_save = [vmin, vmax]
          widget_control, info.nbr, get_value = nbreak_save

          widget_control, info.br1_val, get_value =  b1
          widget_control, info.br2_val, get_value =  b2
          break_save=[b1, b2]

          widget_control, info.pdeg1, get_value =  d1
          widget_control, info.pdeg2, get_value =  d2
          widget_control, info.pdeg3, get_value =  d3
          degree_save=[d1,d2, d3]

          widget_control, event.top,  tlb_get_offset= offset
          offset_save = offset+info.offset

          Calculate_fit, 0
          if lc_valid eq 0  then begin
            winformation, ['Unable to calculate fit parameters for these values.'$
                 , 'Please change and try again.']
            plo, save_save[0], /ov, /col, psym = [-8,0], /nl, ylog = lc_log
          endif else begin
            plo, save_save, /ov, /col, psym = [-8,0], /nl, ylog = lc_log
            if degree_save(0) ne d1 then $
               widget_control, info.pdeg1, set_value = degree_save(0)
            if degree_save(1) ne d2 then $
               widget_control, info.pdeg2, set_value = degree_save(1)
            if degree_save(2) ne d3 then $
               widget_control, info.pdeg3, set_value = degree_save(2)

            if event.value eq 1 then $
              zoomw, save_save, /ov, /col, psym = [-8,0], /nl, ylog = lc_log,$
              group = event.top,  /modal
            if event.value eq 2 then begin
              get_tqstring, iq, 'ENTER the File Name:', file, $
                title = 'Output File Name', group=event.top, /modal
              if iq eq 0 then begin
                openw, unit, /get, file, error= err
                if err eq 0 then begin
                  printf, unit, 'WDFIT3: ', filename_save, '
                  printf, unit, ' Calculated: ', thin_date()
                  printf, unit, ' Valid: ', vmin, vmax
                  printf, unit, ' Break: ', b1, b2
                  printf, unit, ' Exponential:', lc_log
                  printf, unit, degree_save(0)+1
                  printf, unit, c1_save, format='(e27.18)'
                  if nbreak_save gt 0 then begin
                    printf, unit, degree_save(1)+1
                    printf, unit, c2_save, format='(e27.18)'
                    if nbreak_save gt 1 then begin
                      printf, unit, degree_save(2)+1
                      printf, unit, c3_save, format='(e27.18)'
                    endif else begin
                      printf, unit, 1
                      printf, unit, 0.0d0, format='(e27.18)'
                    endelse
                  endif else begin
                    printf, unit, 1
                    printf, unit, 0.0d0, format='(e27.18)'
                    printf, unit, 1
                    printf, unit, 0.0d0, format='(e27.18)'
                  endelse
                  free_lun, unit
                endif else print, '% WDFIT3: Error opening file'
              endif
            endif
          endelse
          widget_control, event.top, set_uvalue = info, /no_copy
   endif else begin
          widget_control, event.top,  tlb_get_offset= offset
          offset_save = offset+info.offset
          widget_control, event.top, /destroy
   endelse
   END
 PRO calculate_fit, data
; Common block
@wdfit3_common.dat

 on_error, 1
      fud = 0.0625
      lc_valid = 0
      deltamin = 0.025*(valid_save(1)-valid_save(0))
      if nbreak_save eq 0 then begin
;       M is the number of coefficients
        m = (degree_save(0)+1) > 2
        sing = 1
        while (m ge 2  and sing gt 0) do begin
          c1_save = svdfit(x_save_scal, y_save, m, sing = sing, /double)
          m = m-1
        endwhile
        if sing ne 0 then return
        for i = 1, m do  c1_save(i:m) = c1_save(i:m)/xscale
        degree_save(0) = m
        c2_save = 0.0
        c3_save = 0.0
      endif else if nbreak_save eq 1 then begin
        ok = where (x_save le break_save(0), nok)
        if (nok lt 2)  or (nok ge  nel_save-2) then begin
          Print, '% WDFIT3: Break point must be shifted toward middle of data', $
            nok, nel
          return
        endif
        nok1 = nok-2
        range = max([(break_save[0]-x_save[0])*fud, deltamin])
        if (x_save[nok+1] -break_save[0] lt range) then nok = nok+1
        x1 = x_save_scal(0:nok)
        y1 = y_save(0:nok)
        n1 = nok

        range = max([ (x_save[nel_save-1] -break_save[0])*fud , deltamin])
        if (break_save[0] - x_save[nok1-1] lt range) then nok1 = nok1-1
        x2 = x_save_scal(nok1:*)
        y2 = y_save(nok1:*)
        n2 = n_elements(x2) -1
        if n2 le 3 then begin
          Print, '% WDFIT3: Break point must be shifted toward middle of data', $
             n1, n2, nel_save
          return
        endif
;       M is the number of coefficients
        m = (degree_save(0)+1) >2
        m =  m < n1
        sing = 1
        while (m ge 2  and sing gt 0) do begin
          c1_save = svdfit(x1, y1, m, sing = sing, /double)
          m = m-1
        endwhile
        if sing ne 0 then return
        m1 = m

        m = (degree_save(1)+1) >2
        m =  m < n2
        sing = 1
        while (m ge 2  and sing gt 0) do begin
          c2_save = svdfit(x2, y2, m, sing = sing, /double)
          m = m-1
        endwhile
        if sing ne 0 then return
        degree_save[0:1] = [m1, m]

        for i = 1, m1 do c1_save(i:m1) = c1_save(i:m1)/xscale
        for i = 1, m  do c2_save(i:m ) = c2_save(i:m)/xscale

        c3_save = 0.0

      endif else begin
; 2 break points
        ok = where (x_save le break_save(0), nok)
        if (nok lt 2)  or (nok ge  nel_save-5) then begin
          Print, '% WDFIT3: Break point must be shifted toward middle of data', $
            nbreak, break_save, nok, nel_save
          return
        endif
        nok1 = nok-2
        range = max([ (break_save[0]-x_save[0])*fud, deltamin])
        if (x_save[nok+1] -break_save[0] lt range) then nok = nok+1
        x1 = x_save_scal(0:nok)
        y1 = y_save(0:nok)
        n1 = nok

;       set up a range allowance for getting additional points
        range = max([ (break_save(1)-break_save(0))*fud, deltamin])
        ok = where (x_save ge break_save[0] and x_save le break_save[1], nok2)
        st = ok[0] -1> 0
        if st gt 0 then if (break_save[0]-x_save[st-1] le range) then st = st-1
        en = ok(nok2-1)+1 < nel_save -1
        if en lt nel_save -1 then $
          if (x_save[en+1]-break_save[1]) le range then en = en+1
        x2 = x_save_scal(st:en)
        y2 = y_save(st:en)
        n2 = en-st
        range = max([(x_save[nel_save-1]-break_save[1])*fud, deltamin])
        st = ok[nok2-1]-1
        if (break_save[1] - x_save[st-1] lt range) then st = st-1
        x3 = x_save_scal[st:*]
        y3 = y_save[st:*]
        n3 = n_elements(x3)-1
        if min([n1, n2, n3]) lt 3 then begin
          Print, '% WDFIT3: Break point must be shifted toward middle of data', $
             n1, n2, n3, nel_save
          return
        endif

        m = (degree_save(0)+1) >2
        m =  m < n1
        sing = 1
        while (m ge 2  and sing gt 0) do begin
          c1_save = svdfit(x1, y1, m, sing = sing, /double)
          m = m-1
        endwhile
        if sing ne 0 then return
        m1 = m

        m = (degree_save(1)+1) >2
        m =  m < n2
        sing = 1
        while (m ge 2  and sing gt 0) do begin
          c2_save = svdfit(x2, y2, m, sing = sing, /double)
          m = m-1
        endwhile
        if sing ne 0 then return
        m2 = m

        m = (degree_save(2)+1) >2
        m =  m < n3
        sing = 1
        while (m ge 2  and sing gt 0) do begin
          c3_save = svdfit(x3, y3, m, sing = sing, /double)
          m = m-1
        endwhile
        if sing ne 0 then return

        for i = 1, m1 do c1_save(i:m1) =  c1_save(i:m1)/xscale
        for i = 1, m2 do c2_save(i:m2) =  c2_save(i:m2)/xscale
        for i = 1, m  do c3_save(i:m ) =  c3_save(i:m )/xscale

        degree_save = [m1, m2, m]
      endelse
      lc_valid = 1
      del = (valid_save(1)-valid_save(0))/511.
      y = valid_save(0)+ findgen(512)*del
      x = [valid_save(0), del]
      ff = get_overlap(valid_save,break_save, nbreak_save, fudge_save)
      scale_regrid, y, fudge = ff

      put_wdfarray, [x, y], save_save[1], clab = 'Fit Values', $
        file=filename_save
      end

      pro set_br1_but, event
on_error, 1
      widget_control, event.top, get_uvalue = info, /no_copy
        cur, x, y
      widget_control, info.br1_val, set_value = x
      widget_control, event.top, set_uvalue = info, /no_copy
      end


      pro set_br2_but, event
on_error, 1
      widget_control, event.top, get_uvalue = info, /no_copy
        cur, x, y
      widget_control, info.br2_val, set_value = x
      widget_control, event.top, set_uvalue = info, /no_copy
      end

 Pro  exray_wdfit3, wdf1, wdf2, ytype = ytype, ylog = ylog, valid = valid, $
           fudge = fudge_value, exact_range = exact_range, XYDATA = xydatain
@com.dat
; Common block
@wdfit3_common.dat
on_error, 1
;
      if n_elements(initwdf) eq 0 then wdfinit
      nparams = n_params()
      if nparams lt 1 then begin
        print,'% Format: ', $
        'WDFIT3, wd1 [, wd2] [, VALID = valid ] [, /YLOG ] [, /Exact_range]'
        return
      endif
      nelf =n_elements(fudge_value)
      if nelf eq 0 then fudge_save = [0.0,0.0] else if nelf eq 1 then $
        fudge_save=[fudge_value,fudge_value] else fudge_save=fudge_value(0:1)

      if n_elements(wdf1) eq 0 then begin
        print, 'Parameters to WDfit must be defined'
        return
      endif
      if wdf1 lt 1 or wdf1 gt maxwdfarray then begin
        print, 'wdf array must be greater than 0 and </=', maxwdfarray
        return
      endif
;
      if nparams gt 1 then save = get_save_array(wdf2, maxwdfarray) else $
         save = maxwdfarray
      if save lt 1 or save gt maxwdfarray then save = maxwdfarray

      nel_save = wdfnp(wdf1-1)
      filename_save = wdffile(wdf1-1)
      if nel_save eq 0 then begin
        print, 'No valid data in the array'
        return
      endif
      save_save = [wdf1, save]
      if n_elements (xydatain) eq 0 then xydata =1 else begin
        if xydatain gt 0 then xydata =1 else begin
          xydata =0
          if xydatain lt 0 then npoint = round(-xydatain) else npoint =1024
        endelse
      endelse
      get_wdfarray, array, wdf1, xydata = xydata, npoint = npoint

      nela = n_elements(array)-2
      if nela eq nel_save or xydata eq 0 then begin
        nel_save = nela
        y_save = array(2:*)
        x_save=findgen(nel_save)*array(1)+array(0)
        xydata = 0
      endif else begin
        y_save = array(nel_save:*)
        x_save = array(0:nel_save-1)
        j = sort (x_save)
        x_save = x_save(j)
        y_save = y_save(j)
        xydata = 1
      endelse
      xscale = max(abs([x_save[nel_save-1], x_save[0]]))*0.5D0
      x_save_scal= x_save/xscale
      if keyword_set(ytype) or keyword_set(ylog) then begin
        ok = where (y_save gt 0, nok)
        if nok eq 0 then begin
          Print, '% WDFIT: Log fit not possible; all values less/equal to zero'
          return
        endif
        if nok eq nel_save then y_save = alog (y_save) else begin
          x_save = x_save(ok)
          nel_save = nok
          y_save = alog(y_save(ok))
        endelse
        lc_log = 1
      endif else lc_log = 0
      if n_elements (c1_save) eq 0 then begin
        c1_save = 0.0
        c2_save = 0.0
        c3_save = 0.0
        nbreak_save = 2
        range = (x_save[nel_save-1]-x_save[0])*0.0625
        offset_save = [10, 40]
        break_save = [ x_save[0]+range, x_save[nel_save-1]-range]
        degree_save =[2,5, 2]
        lc_valid=0
      endif
      if nbreak_save gt 0 then begin
        if break_save (0) lt x_save(2) or break_save (0) gt x_save(nel_save-1) $
           then nbreak_save = 0
      endif
      if keyword_set(exact_range) then $
       valid_save = [x_save[0],x_save[nel_save-1]] else begin
       if n_elements (valid) lt 2 then $
         valid_save = [0.95*x_save[0], 1.02*x_save[nel_save-1]] else $
         valid_save = valid [0:1]
      endelse

      sing = 0
      calculate_fit, 0
      plo, save_save[0], /ov, /col, psym = [-8,0], /nl, ylog = lc_log
      if lc_valid ne 0 then plo, save_save[1],/nl, /ns, /nogr, /ov, /co

; set up the widget
; top level base
      tlb = widget_base(/column, /map)
      frame_size = 3

; base for valid range
      base1 = widget_base(tlb, /column, frame = frame_size, /map)

        Text_Val= 'Valid Range for Fit'
        wtext = widget_label( base1,VALUE=Text_Val)
;       base for values in base 1
        base1_1 = widget_base(base1, /row, /map)

;       Minimum valid range
        valid_min_id = cw_field(base1_1, value = valid_save[0], /column, $
                /float, xsize = 15, title='Minimum')

;       Maximum valid range
        valid_max_id = cw_field(base1_1, value = valid_save[1], /column, $
                /float, xsize = 15, title='Maximum')


; base for break points
      base2 = widget_base(tlb, /column, frame = frame_size, /map)

      nbreak_id = cw_field(base2, value = nbreak_save, /row, $
            /integer, title = 'Number of Breaks (0,1,2)', xsize = 5)

      base21 = widget_base(base2, /row, frame = frame_size/2, /map)
      break1_id = cw_field(base21, value = break_save[0], /row, $
            /float, title = 'Value for Break #1', xsize = 15)
      but_b1 = widget_button(base21, /no_release, event_pro= 'set_br1_but', $
               value = 'Cursor')

      base22 = widget_base(base2, /row, frame = frame_size/2, /map)
      break2_id = cw_field(base22, value = break_save[1], /row, $
            /float, title = 'Value for Break #2', xsize = 15)
      but_b2 = widget_button(base22, /no_release, event_pro= 'set_br2_but', $
               value = 'Cursor')

; base for polynomial fit
      base3 = widget_base(tlb, /column, frame= frame_size, /map)

      degree_1_id  = cw_field(base3, value = degree_save[0], /integer, $
            row=1, title='Polynomial Degree-1', Xsize = 5)
      degree_2_id  = cw_field(base3, value = degree_save[1], /integer, $
            row=1, title='Polynomial Degree-2', Xsize = 5)
      degree_3_id  = cw_field(base3, value = degree_save[2], /integer, $
            row=1, title='Polynomial Degree-3', Xsize = 5)
; base for buttons
      base4 = widget_base( tlb, /column, frame= frame_size, /map)

      btns=[ ' Calculate ', $
             '  Z o o m  ',  $
             'Save to File', $
             ' Quit / Exit ' ]

      doit_id = cw_bgroup(base4, btns, /row, uvalue = 'Buttons')
      if n_elements(offset_save) eq 0 then offset_save=[10, 40]

      widget_control, tlb, /realize,  tlb_set_xoffset = offset_save(0), $
       tlb_set_yoffset= offset_save(1)
      widget_control, tlb, tlb_get_offset= offset_real
      offset_real = offset_save - offset_real
      info = {vmin: valid_min_id, vmax: valid_max_id, nbr: nbreak_id, $
           br1_val: break1_id, br2_val: break2_id, pdeg1: degree_1_id, $
           pdeg2: degree_2_id, pdeg3: degree_3_id, offset: offset_real}

      widget_control, tlb, set_uvalue = info, /no_copy
      xmanager, 'wdfit3_wid', tlb ,/BLOCK

      END
