;
; Copyright 1999 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
 Pro legend2, stringin, linestyle= linestyle, psymbol = psymbol, color= color, $
          position = positin, device = device, data = data, normal = normal, $
          relative = relative, thick = thick, mspace = mspace, $
          fill=symfill, symsize=symsize, charsize = charsize, $
          offset = ioffset, nextpos = nextpos
;+
;   Produce a legend for a plot
;   format: legend, string, linestyle = linestyle, psymbol = psymbol,
;           color = color, position = position, /device, /data, /normal,
;           /relative, mspace = mspace, thick = thick, nextpos = nextpos,
;           offset = offset
;   where
;         STRING    - String array with the legend text for each line in the
;                     legend.
;         LINESTYLE - Array of integers indicating the linestyle for each line
;                     Default is -1, solid line. Other values are the same as IDL
;                     -1, No LINE, 0, Solid; 1, Dotted; 2, Dashed; 3, Dash-Dot;
;                     4, Dash-Dot-Dot-Dot; 5, Long Dashes
;                     Linestyle should have one value for all curves or one
;                       value for each value of string. If there are fewer
;                       values, then they will be recycled.
;         PSYMBOL   - Array of integers indicating the symbol for each line
;                     Default is 0, no symbol.  If there are fewer values
;                     of psymbol than curves then they will be recycled.
;                     Values are the same as IDL (abs(pysmbol) is used).
;                     1, +; 2, *; 3, period(.); 4, Diamond; 5, Triangle;
;                     6, Square; 7, X; 8, UserDefined.
;         COLOR     - Color for symbols and lines for each line of the legend
;                     Default is !p.color (white for X and Black for PostScript)
;                     If there are fewer values of color than curves then they
;                     will be recycled.
;                     Values are: 1, red; 2, green; 3, blue; 4, magenta;
;                     5, orange; 6, cyan; 7, yellow; 8, white; 0, black
;         POSITION  - Position on the plot in units determined by keywords.
;                     Default is data coordinates if position is specified.
;                     If POSITION is undefined then default is at a relative
;                        position of [0.04, 0.94] of the plotting area.
;         /device, /data, /normal, /relative  - only one may be specified.
;                     Determines the coordinates for the position keywork
;                     Data refers to the plot coordinates (Default)
;                     Relative is relative the actual grid
;                     Device is in device coordinates
;                     Normal is normalized space coordinates
;         Mspace    - Multiplier for the spacing between lines
;         Thick     - parameter sent to oplot
;         Charsize  - parameter sent to xyouts; if not specified, set to
;                     1/max(!p.multi(1:2)-1)>1
;         FILL      - Indicates filled symbols - only for symbols 11 and higher
;                     (see PLO, MKSYMBOL)
;         SYMSIZE   - Symbol size multiplier
;         NEXTPOS   - Next position for the legend in data coordinates
;                     Use this keyword for successive calls to legend
;         OFFSET    - Use to offset the count on the symbols, colors, linestyles
;                     Default = 0
;         Example:
;           Label curves with numbers in the upper left hand corner of the grid
;           legend, sindgen(10), pos= [0.04, 0.94], /relative, lin = 1, $
;                    color = [2, 3, 4, 5, 6, 7, 1]
;-
  on_error, 1
;
@symbol_list.pro
      nel = n_elements(stringin)
      if nel eq 0 then begin
        print,'% Format: LEGEND, STRING_ARRAY, [, LINESTYLE = linestyle]'
        print,'    [, PSYMBOL = psymbol] [, COLOR = color] [, MSPACE = mspace]'
        print,'    [, /DEVICE] [, /DATA] [, /RELATIVE] [, /NORMAL]'
        print,'    [, POSITION = position] [, CHARSIZE= charsize]'
        print,'    [, THICK = thick] [, NEXTPOS = nextpos] [,OFFSET = offset]'
        return
      endif
;
      s = size(stringin)
      if s(s(0)+1) ne 7 then $
        message, 'Paramete to LEGEND must be a sting array'
      xcrange = !x.crange
      ycrange = !y.crange
      if !x.type eq 1 then xcrange = 10^xcrange
      if !y.type eq 1 then ycrange = 10^ycrange
      minran = convert_coord(xcrange(0), ycrange(0), /data, /to_device)
      maxran = convert_coord(xcrange(1), ycrange(1), /data, /to_device)
      xrang = maxran(0)- minran(0)
      yrang = maxran(1)- minran(1)
      if n_elements (charsize) eq 0 then charsize = 1.0
      crowd = max ((!p.multi)(1:2))
      if crowd gt 2 then chars = 0.6 *charsize  else chars = charsize
      chy = !d.y_ch_size*chars
      chx = !d.x_ch_size*chars
      offset = 0.4*chy
      deltay = 1.5*chy
      if n_elements(mspace) ne 0 then begin
        deltay = deltay*abs(mspace)
        offset = offset*abs(mspace)
      endif
      if n_elements (symfill) eq 0 then begin
         symfill = 0
         nsymfill = 1
      endif else nsymfill = n_elements (symfill)
      if n_elements (symsize) eq 0 then begin
         symsize  = 1
         nsymsize = 1
      endif else nsymsize = n_elements (symsize)
      if n_elements (positin) ge 2 then begin
        position = positin
        if keyword_set (device) then begin
          position = position
        endif else if keyword_set(normal) then begin
          position = CONVERT_COORD(position(0), position(1), /normal, /to_device)
        endif else if keyword_set(relative) then begin
          position = position(0:1) *[xrang, yrang] +minran
        endif else $
          position = CONVERT_COORD(position(0), position(1), /data, /to_device)
      endif else position = [0.04, 0.94]*[xrang, yrang] + minran
;
      ncolor = n_elements(color)
      if ncolor eq 0 then color = !p.color else loadxyct, /quiet, /freeze
      ncolor = ncolor > 1
      nsymbol = n_elements(psymbol)
      if n_elements(psymbol) gt 0 then begin
        a = size(psymbol)
        if a(a(0)+1) eq 7 then begin
          nel = a(a(0)+2)
          temp = make_array(nel, /int)
          for i = 0, nel-1 do begin
            s = strupcase(strcompress(psymbol(i), /remove_all))
            l = strlen(s)
            if l eq 1 then begin
              if s eq 'X' then s = 'BIGX' else $
              if s eq '+' then s = 'PLUS' else $
              if s eq 'O' then s = 'CIRCLE'
              l = strlen(s)
            endif
            if l eq 0 then temp(i) = -11 else begin
              templist = strmid(symbol_list, 0,l)
              ok = where (s eq templist, nok )
              if nok eq 0 then begin
                print, 'A valid symbol string must be specified - ', s
                return
              endif else if nok eq 1 then temp(i) = ok else begin
                print, ' Multiple symbols specified, using ' , s
                print, symbol_list(ok)
                temp(i) = ok(0)
                print, '   Using a symbol of ', symbol_list(ok(0))
              endelse
            endelse
          endfor
          psymbol = temp+11
        endif
      endif
      if nsymbol eq 0 then psymbol = 0 else psymbol = abs (psymbol)
      nsymbol = nsymbol > 1
      nlinest = n_elements(linestyle)
      if nlinest eq 0 then linestyle = -1
      nlinest = nlinest > 1
      if n_elements(thick) eq 0 then thick=!p.thick else $
        thick=abs(thick)*!p.thick
;
      if n_elements(ioffset) eq 0 then ioffset = 0
      x = position(0)
      xf = x + 5.0*chx
      y = position(1)
      if max (linestyle) lt 0 then begin
        xc = x+2.5*chx
        for i = ioffset, ioffset+ nel-1 do begin
          x1 = convert_coord(x, y, /device, /to_data)
          x3 = convert_coord(xc, y-offset, /device, /to_data)
          psym = psymbol(i mod nsymbol)
          if psym gt 10 then begin
            psym = psym - 11
            mksymbol, psym, fill = symfill(i mod nsymfill)
            psym = 8
          endif
          col = color (i mod ncolor)
          if psym gt 0 then $
          plots, x1(0), x1(1), symsize=1.1*chars, psym=psym, color = col, $
           thick = thick
          xyouts, x3(0), x3(1), stringin(i-ioffset), charsize=chars
          y = y -deltay
        endfor
      endif else begin
        xc = xf+2.5*chx
        for i = ioffset, ioffset+nel-1 do begin
          x1 = convert_coord(x, y, /device, /to_data)
          x2 = convert_coord(xf, y, /device, /to_data)
          x3 = convert_coord(xc, y-offset, /device, /to_data)
          ll = linestyle (i mod nlinest)
          psym = psymbol(i mod nsymbol)
          if psym gt 10 then begin
            psym = psym - 11
            mksymbol, psym, fill = symfill(i mod nsymfill)
            psym = 8
          endif
          col = color (i mod ncolor)
          ss = symsize (i mod nsymsize)
          if ss eq 0. then ss = 1
          if ll ge 0 then $
          oplot,[x1(0), x2(0)], [x1(1), x2(1)], line=ll, psym=-psym, $
            color=col, thick=2., /noclip , symsize = ss* chars $
          else if psym gt 0 then $
            plots, (x1(0)+x2(0))/2.0, x1(1),symsize=1.5*ss*chars,psym=psym, $
              color = col, thick = 2.
          xyouts,x3(0),x3(1),stringin(i-ioffset), charsize=chars
          y = y-deltay
        endfor
      endelse
      nextpos =  convert_coord(x, y, /device, /to_data)
      return
      end
