PRO EXRAY_GO,wda,exrayptr,xrange=xrange,yrange=yrange,over=over, $
              limit=limit, trex=trex,rotation=rot, parabolic=parabolic,$
              noprint=noprint, shift=xstalshift,origin=LOS,windID=winID,$
              ignore=ignore

;*******************************************************************
;* This is a program to calcuate the ray-tracing for an elliptical *
;* crystal spectrometer.  The inputs and keywords are as follows:  *
;*                                                                 *
;* Inputs:                                                         *
;* -------                                                         *
;*   wda:        -An array of wdf arrays.  wda must have as many   *
;*                elements are there are crystals defined in the   *
;*                input deck.                                      *
;*   file:       -The path/name of the EXRAY input deck. If file   *
;*                is not specified, then the user is prompted to   *
;*                open one. A sample input deck, called exray.inp, *
;*                can be found in the EXRAY program folder.        *
;*   xrange:     -The x-range of the plot window.                  *
;*   yrange:     -The y-range of the plot window.                  *
;*   rotation:   -The rotation of the coordinate system beyond that*
;*                of the default rotation (average beta).  This    *
;*                variable only has effect when keyword TREX is    *
;*                set.                                             *
;*                                                                 *
;* Keywords:                                                       *
;* ---------                                                       *
;*   over:       -Plot output over the current active plot window. *
;*   limit:      -Only calculate the diffraction for those rays    *
;*                which fall betweem the specified minimum and     *
;*                maximum crystal beta angles.                     *
;*   trex:       -Plot output in TREX coordinates, and print out a *
;*                list of crystal features and fabrication specs.  *
;*   parabolic:  -Assume the crystal to have a parabolic rather    *
;*                than an elliptical shape. (Ignore R0)            *
;*   noprint:    -Do not print the wavelength values on the plot.  *
;*   ignore:     -Ignore those rays for which no Beta exists       *
;*                (Do not notify user)                             *
;*                                                                 *
;* Output:                                                         *
;* -------                                                         *
;*   wda:        -Contains the wavelength versus film position     *
;*                curves for each crystal. If a Johann locus       *
;*                detector is specified, then the wdf arrays       *
;*                contain the Johann cross-over points vs. lambda  *
;*                for each crystal.                                *
;*******************************************************************


;***************************************************************
; Calculate where the ray strikes the crystal (xc,yc)
; and calculate where the ray strikes the film (xlambda,ylambda)
;***************************************************************
if keyword_set(limit) then limit=1 else limit=0
if keyword_set(ignore) then ignore=1 else ignore=0
if keyword_set(parabolic) then begin
   ;the user has asked for a parabolic crystal, so set ecc = 1.
   parabolic=1
   (*exrayptr).xstal(*).ecc = 1.
endif else parabolic=0
numxstals=(*exrayptr).numxstals
numsource=(*exrayptr).numsource
numorders=(*exrayptr).numorders
numlambda=(*exrayptr).numlambda
for l=0,numxstals-1 do begin
   for i=0,numsource-1 do begin
      for j=0,numorders-1 do begin
          for k=0,numlambda-1 do begin
              theta=(*exrayptr).xstal(l).theta(i,j,k)
              beta=(*exrayptr).xstal(l).beta(j,k)
              if (beta gt 0.) then begin
                 ;beta was given, so we need to calculate theta and lambda
                 exray_thetacalc,exrayptr,l,i,j,k,limit,parabolic
              endif else if (theta gt 0.) then begin
                 ;lambda was given, so we need to find beta
                 exray_betacalc,exrayptr,l,i,j,k,limit,parabolic,ignore
              endif
              theta=(*exrayptr).xstal(l).theta(i,j,k)
              if (theta gt 0.) then begin
                 ;calculate where the ray will hit the film
                 exray_filmposition,exrayptr,l,i,j,k
              endif else begin
                 ;if this lambda wasn't defined for this source, or if the lambda
                 ;won't diffract off the xstal in this order, then set xc=yc=0.
                 ;and xlambda=ylambda=dlambda=1000.
                 (*exrayptr).xstal(l).xc(i,j,k)=0.
                 (*exrayptr).xstal(l).yc(i,j,k)=0.
                 (*exrayptr).film.xlambda(l,i,j,k)=1000.
                 (*exrayptr).film.ylambda(l,i,j,k)=1000.
                 (*exrayptr).film.d(l,i,j,k)=1000.
              endelse
          endfor
      endfor
   endfor
endfor

;**********************************************************
; Plot the xstals and the film plane
;**********************************************************
if ((n_elements(xrange) eq 0) and (n_elements(trex) eq 0)) then begin
  xrange=[-10,10]
endif else if (n_elements(xrange) eq 0) then begin
   xrange=[-37,7]
endif
if ((n_elements(yrange) eq 0) and (n_elements(trex) eq 0)) then begin
   yrange=[-10,10]
endif else if (n_elements(yrange) eq 0) then begin
   yrange=[-6,36]
endif
if keyword_set(over) then over = 1 else over = 0

if keyword_set(TREX) then begin
   ;the user wants to plot in TREX coordinates and a print-
   ; out of the design parameters
   trex = 1
   if n_elements(xstalshift) eq 0 then xstalshift = 0.
;   LOS=391.+xstalshift  ;distance between TREX origin and the source focus
   if (n_elements(rot) eq 0) then rot = 0. else rot=double(rot*!pi/180.)
   exray_trex,exrayptr,rot,LOS,alpham,R0m
   ;will determine x and y shift to TREX coordinates, and print design
   ; paramters under this shift
endif else begin
   trex = 0
endelse

exray_plotprep,exrayptr,xrange,yrange,over,trex,LOS,alpham,R0m,parabolic,windID=winid

;**********************************************************
; Plot the rays from source to the crystal, and from the
; crystal to the film.
;**********************************************************
if keyword_set(noprint) then noprint=1 else noprint=0
exray_plotrays,exrayptr,over,trex,LOS,alpham,R0m,noprint,windID=winID
if ((*exrayptr).film.rf) lt 0 then $
  exray_plotJohann,exrayptr,trex,LOS,alpham,R0m,windID=winID

;**********************************************************
; Define the legend and plot it
;**********************************************************
exray_legend2,exrayptr,windID=winID

;**********************************************************
; Create the arrays of xfilm and lambdafilm (positions of
; the specified wavelengths on the film)
;  <or> the x and y positions of the johann crossover(s)
;**********************************************************
if (n_elements(wda) ne numxstals) then begin
   print,'You must define as many wdf arrays as there are xstals'
endif else begin
   for l=0,numxstals-1 do begin
      xfilm=dblarr(numsource*numorders*numlambda)
      lambdafilm=dblarr(numsource*numorders*numlambda)
      for i=0,numsource-1 do begin
         for j=0,numorders-1 do begin
             for k=0,numlambda-1 do begin
                 theta=(*exrayptr).xstal(l).theta(i,j,k)
                 if (theta gt 0.) then begin
                    if ((*exrayptr).film.rf) ge 0 then begin
                      ;return lambda vs x on the linear film
                       d=(*exrayptr).film.d(l,i,j,k)
                       lambda=(*exrayptr).source(i).lambda(j,k)
                       xfilm(i*numorders*numlambda+j*numlambda+k)=d
                       lambdafilm(i*numorders*numlambda+j*numlambda+k)=lambda
                    endif else begin
                      ;return yxover vs xxover
                       xxover=(*exrayptr).film.xxover(l,i,j,k)
                       yxover=(*exrayptr).film.yxover(l,i,j,k)
                       xfilm(i*numorders*numlambda+j*numlambda+k)=xxover
                       lambdafilm(i*numorders*numlambda+j*numlambda+k)=yxover
                    endelse
                 endif else begin
                    xfilm(i*numorders*numlambda+j*numlambda+k)=-1000.
                    lambdafilm(i*numorders*numlambda+j*numlambda+k)=-1000.
                 endelse
             endfor
         endfor
      endfor
      index=where(xfilm gt -1000.)
      if (index(0) ne -1) then begin
          xfilm=xfilm(index)
          lambdafilm=lambdafilm(index)
      endif
      index=sort(xfilm)
      xfilm=xfilm(index)
      lambdafilm=lambdafilm(index)
      if (n_elements(xfilm) ge 2) then begin
          i2w,xfilm,lambdafilm,wda(l),/xy
      endif
   endfor
endelse

;**********************************************************
; exit EXRAY_GO
;**********************************************************
return
end