;
; @(#)$Id: henke_film_resp.pro  Version 1.7  Last Modified 09/15/93 $
;
;+
; NAME:
;       henke_flm_resp  (from henke_film_resp)
; PURPOSE:
;   PRO HENKE_FILM_RESP calculates the intensity of photons incident on
;   film type FILM given the film density and the wavelength of the
;   incident radiation, and the angle of incidence theta.
;
;       Film types: -1 - Kodak 101-07
;                    0 - DEF
;                    1 - SB-392 (SB-5)
;                    2 - RAR 2492
;                3 - RAR 2495
;                    4 - RAR 2497
; REFERENCE:
;   J.Opt.Soc.Am.B/Vol.1,No.6/December 1984, Henke et al.
; CATEGORY:
;       Grating Spectroscopy, Vida
; CALLING SEQUENCE:
;       henke_flm_resp, film, wl, density, theta, photons
; INPUTS:
;       film is the film type
;       wl is the wavelength array
;       density is the corresponding film density array
;       theta is the array of angles of incidence on the film
; OUTPUTS:
;       photons is the output array
; COMMON BLOCKS:
;       none
; MODIFICATION HISTORY:
;       Written by W. E. Nelson, SEA
;       Vectorized by R. J. Dukart, 1262, April 19, 1991
;   Single wavelength corrections by W. E. Nelson, June, 1991
;-

PRO exray_henke_film_resp, film, wl, density, theta, photons,tension,modify,test

  ;+
  ; How many density points have been passed
  ;-
  nd = N_elements(density)
  photons = fltarr(nd)

  no_density = where(density le 0.001)
  if min(no_density ge 0) then density(no_density) = 0.0

  ;+
  ; Calculate absorption coefficients amu0 and amu1
  ;-
  exray_absorption_coef, wl, amu0, amu1,tension, modify

  ;+
  ; Number of wavelengths passed
  ;-
  nwl = N_elements(wl)

  if nwl eq 1 then begin
      amu0 = amu0(0)
      amu1 = amu1(0)
  endif

  IF (film lt 0) THEN BEGIN

      ; Henke film parameters for Kodak 101-07
      a1 = 1.957
      b1 = 0.3128
      t1 = 2.0

    ;+
    ; Kodak 101-07
    ;-

      ; Calculate Beta1 for Kodak 101-07
      ;   NOTE: The exponential may result in an underflow, but it does not affect
      ;          the result on the Solbourne or Sun.

      Beta1 = sin(theta)*(1. - exp(-amu1*t1/sin(theta)))

      ; Two parameter empirical relation for Kodak 101-07

      photons = density < (a1 - 0.001)
      photons = alog(a1/(a1 - photons))/(b1*Beta1)
;eqn 7 from henke josaB 1,818 (1984)
      return

  endif else begin

      ;+
      ; Henke film parameters for SB-392 and the RAR films
      ;-
      ; AgBr volume fraction
      v = [0.40, 0.20, 0.10, 0.20, 0.10]

      ; Emulsion thickness (micrometer)
      t = [13.0, 10.0, 7.0, 7.0, 7.0]

      ; AgBr grain size (micrometer)
      d = [1.6, 1.0, 0.3, 0.3, 0.3]

      ; Effective emulsion layer thickness (micrometer)
      d0 = [2.0, 1.5, 0.6, 0.6, 0.6]

      ; Supercoat thickness (micrometer)
      t0 = [1.0, 1.0, 0.3, 0.3, 0.3]

      ; Empirical film parameter (1/micrometer)
      a = [0.68, 0.285, 0.527, 0.528, 0.414]

      ; Empirical film parameter (micrometer^2)
      b = [1.69, 1.41, 0.372, 0.926, 0.454]

      ; Polyester thickness for DEF (micometer)
      tb = 185.

      ; Low energy, film coefficients
      IF (max(wl) GT 13.) THEN BEGIN
          t13    = t       ; Emulsion thickness (micrometer)
          t13(1) = 10.0
          a13    = a       ; Empirical film parameter (1/micrometer)
          a13(1) = 0.285
          b13    = b       ; Empirical film parameter (micrometer^2)
          b13(1) = 1.41
      ENDIF

      ile13 = where(wl le 13.)
      igt13 = where(wl gt 13.)


      ;+
      ; SB-392 or RAR films
      ;-

      ; Calculate the absorption coefficient amu
      amu = amu0 - (1./d(film))*alog(1. - v(film)*(1. - exp(-(amu1 - amu0)* $
                   d(film))))

      ; Calculate the photon energy dependent factors Alpha and Beta
      alpha = amu/(sin(theta)+amu*d0(film))        ; High energy aproximation

;      IF (min(igt13) ge 0) THEN begin             ; Low energy correction
;        if nwl eq 1 then alpha = alpha/(1. + alpha*d0(film)) $
;        else alpha(igt13) = alpha(igt13)/(1. + alpha(igt13)*d0(film))
;      ENDIF

      beta = (1. - exp(-amu1*d(film)))*exp(-amu0*t0(film)/sin(theta))

      ;+
      ; Two parameter equation for THICK (completely absorbing) EMULLSION
      ; response.
      ;-
      IF (min(ile13) ge 0) THEN begin
        if nwl eq 1 then photons = (exp(alpha*density/a(film)) - 1.)/(b(film)*beta) $
        else photons(ile13) = (exp(alpha(ile13)*density(ile13)/a(film)) - 1.)/$
                               (b(film)*beta(ile13))
      ENDIF

      IF (min(igt13) ge 0) THEN begin
        if nwl eq 1 then photons = (exp(alpha*density/a13(film)) - 1.)/(b13(film)*beta) $
        else photons(igt13) = (exp(alpha(igt13)*density(igt13)/a13(film)) - 1.)/$
                               (b13(film)*beta(igt13))
      ENDIF

      ;+
      ; Two parameter equation for THIN EMULLSION response.
      ;-
    ;  ilt12 = where(wl LT 12.40)
    ;  IF (min(ilt12) ge 0) THEN begin
    ;    if nwl eq 1 then photons = photons / (1. - exp(-amu*t(film)/sin(theta) + $
    ;                                alpha*density/a(film))) $
    ;    else photons(ilt12) = photons(ilt12) / $
    ;      (1. - exp(-amu(ilt12)*t(film)/sin(theta(ilt12)) + $
    ;          alpha(ilt12)*density(ilt12)/a(film)))
    ;  ENDIF

      photons = photons / (1. - exp(-amu*t(film)/sin(theta) + $
                                    alpha*density/a(film)))

      photons = photons > 0.

  endelse

  ;+
  ; Double emulsion response for DEF, uses thin emulsion response as
  ; starting guess.
  ;-
  IF (film eq 0) then begin

      ; Different alpha for high energy film
      ; (see Henke et al "High-Energy x-ray response of photographic films:
      ; models and measurement")

      alpha = amu/sin(theta)


      ; vectorized convergence routine

      photold = fltarr(nd)
      check = fltarr(nd)

      epsilon = 0.001

      aa = alpha*density/a(film)
      bb = amu*t(film)/sin(theta)
      cc = (amu0*tb + 2.*amu*t(film))/sin(theta)
      dd = (amu0*tb + amu*t(film))/sin(theta)

      ; Ignore photons less than or equal zero
      nconv = where(photons gt 0.)

      while min(nconv ge 0) do begin

          photold(nconv) = photons(nconv)

          if nwl eq 1 then $
            photons(nconv) = (exp(aa(nconv))  * $
              (1.+b(film)*beta*photold(nconv)*exp(-bb)) * $
              (1.+b(film)*beta*photold(nconv)*exp(-cc)) / $
              (1.+b(film)*beta*photold(nconv)*exp(-dd)) - 1.) / $
              (b(film)*beta) $
          else $
            photons(nconv) = (exp(aa(nconv)) * $
              (1.+b(film)*beta(nconv)*photold(nconv)*exp(-bb(nconv))) * $
              (1.+b(film)*beta(nconv)*photold(nconv)*exp(-cc(nconv))) / $
              (1.+b(film)*beta(nconv)*photold(nconv)*exp(-dd(nconv))) - 1.) / $
              (b(film)*beta(nconv))

          check(nconv) = abs(1. - photons(nconv)/photold(nconv))
          nconv = where(check gt epsilon)

      endwhile

  endif

return
end

