;
; @(#)$Id: absorption_coef.pro	Version 1.5 	Last Modified 09/14/93 $
;
;+
; NAME:
;     ABSORPTION_COEF
; PURPOSE:
;     Calculates the linear absorption coefficients mu0 and mu1
;     of AgBr versus wavelength (lambda). These are passed to the
;     calling routine for use in the film response functions.
;     Ref: J.Opt.Soc.Am.B/Vol.1,No.6/December 1984, Henke et al.
; CATEGORY:
; CALLING SEQUENCE:
;     ABSORP_COEF, LAMBDA, MU0, MU1
; INPUTS:
;     LAMBDA - Wavelength array
; OPTIONAL INPUT PARAMETERS:
;     None
; OUTPUTS:
;     MU0 - Array of absorption coefficients for polyester
;     MU1 - Array of absorption coefficients for AgBr
; OPTIONAL OUTPUT PARAMETERS:
;     None
; COMMON BLOCKS:
;     None
; SIDE EFFECTS:
;     None
; RESTRICTIONS:
;     None
; PROCEDURE:
; MODIFICATION HISTORY:
;	Initial linear procedure written by W. E. Nelson - 1990
;	Routine vectorized by R. J. Dukart, 1262 - April, 1991
;-

PRO exray_absorption_coef, lambda, mu0, mu1,tension,modify

n = N_elements(lambda)
mu0 = fltarr(n)
mu1 = fltarr(n)

amu0 = fltarr(60)
amu1 = fltarr(60)
alam = fltarr(60)

ibegin = intarr(6)
iend = intarr(6)

;+
; Pointers for arrays
;-
ibegin = [0, 7, 23, 43, 47, 51]
iend = [6, 22, 42, 46, 50, 59]

;+
; Table 9. Linear Absorption Coefficients
;-
alam(0)  = [1.24, 1.38, 1.55, 1.77, 2.06, 2.48, 3.10]
alam(7)  = [3.75, 3.87, 4.00, 4.13, 4.27, 4.42, 4.59, 4.77, $ 
            4.96, 5.16, 5.39, 5.63, 5.90, 6.19, 6.52, 6.88]
alam(23) = [8.26, 8.54, 8.85, 9.18, 9.53, 9.91, 10.32, 10.77, $
            11.26, 11.80, 12.39, 13.04, 13.77, 14.58, 15.49, $
            16.52, 17.70, 19.06, 20.65, 22.53]
alam(43) = [24.78, 26.08, 27.53, 29.15]
alam(47) = [33.04, 35.40, 38.12, 41.30]
alam(51) = [45.05, 49.56, 55.06, 61.95, 70.79, 82.59, 99.11, $
            123.89, 165.19]

amu0(0)  = [4.55e-04, 6.38e-04, 9.27e-04, 1.41e-03, 2.28e-03, 4.01e-03, $
            7.94e-03]
amu0(7)  = [1.41e-02, 1.55e-02, 1.70e-02, 1.87e-02, 2.07e-02, 2.29e-02, $
            2.55e-02, 2.85e-02, 3.19e-02, 3.59e-02, 4.06e-02, 4.62e-02, $
            5.28e-02, 6.25e-02, 7.24e-02, 8.44e-02]
amu0(23) = [1.41e-01, 1.55e-01, 1.71e-01, 1.89e-01, 2.10e-01, 2.34e-01, $
            2.62e-01, 2.94e-01, 3.32e-01, 3.76e-01, 4.29e-01, 4.93e-01, $
            5.68e-01, 6.59e-01, 7.74e-01, 9.17e-01, 1.09e+00, 1.32e+00, $
            1.61e+00, 2.00e+00]
amu0(43) = [1.27e+00, 1.44e+00, 1.64e+00, 1.87e+00]
amu0(47) = [1.92e+00, 2.28e+00, 2.74e+00, 3.34e+00]
amu0(51) = [5.40e-01, 6.82e-01, 8.83e-01, 1.17e+00, 1.61e+00, 2.30e+00, $
            3.45e+00, 5.65e+00, 1.01e+01]

amu1(0)  = [5.56e-02, 7.43e-02, 1.02e-01, 1.47e-01, 2.23e-01, 3.60e-01, $
            6.36e-01]
amu1(7)  = [4.18e-01, 4.53e-01, 4.92e-01, 5.35e-01, 5.84e-01, 6.39e-01, $
            7.01e-01, 7.72e-01, 8.53e-01, 9.46e-01, 1.05e+00, 1.18e+00, $
            1.32e+00, 1.49e+00, 1.69e+00, 1.87e+00]
amu1(23) = [1.33e+00, 1.43e+00, 1.55e+00, 1.69e+00, 1.84e+00, 2.02e+00, $
            2.22e+00, 2.44e+00, 2.71e+00, 3.01e+00, 3.36e+00, 3.76e+00, $
            4.23e+00, 4.78e+00, 5.43e+00, 6.23e+00, 7.19e+00, 7.84e+00, $
            8.60e+00, 9.42e+00]
amu1(43) = [1.02e+01, 1.06e+01, 1.07e+01, 9.10e+00]
amu1(47) = [8.26e+00, 9.00e+00, 9.88e+00, 1.09e+01]
amu1(51) = [1.21e+01, 1.29e+01, 1.37e+01, 1.47e+01, 1.46e+01, 1.48e+01, $
            1.22e+01, 2.11e+01, 4.63e+01]

;+
; Interpolate in the proper interval
; and calculate the linear absorption coefficients mu0 and mu1
;-
lam_min=min(lambda)
lam_max=max(lambda)

if n_elements(tension) eq 0 then tension = 1.0

if (lam_min le alam(iend(0))) then begin
  index = where(lambda LE alam(iend(0)))
  i1    = ibegin(0)
  i2    = iend(0)
  for k=0,N_Elements(index)-1 do begin
    mu0(index(k))   = SPLINE(alam(i1:i2), amu0(i1:i2),lambda(index(k)),tension)
    mu1(index(k))   = SPLINE(alam(i1:i2), amu1(i1:i2),lambda(index(k)),tension)
  endfor
endif

for i = 1,5 do begin
  i1    = ibegin(i)
  i2    = iend(i)
  index = where((lambda GT alam(i1)) AND (lambda LE alam(i2)))
  if (min(index) ge 0) then begin
    for k=0,n_elements(index)-1 do begin
  	mu0(index(k)) = SPLINE(alam(i1:i2), amu0(i1:i2),lambda(index(k)),tension)
  	mu1(index(k)) = SPLINE(alam(i1:i2), amu1(i1:i2),lambda(index(k)),tension)
    endfor
  endif

  Case modify of 
    0: begin   ; Cubic Spline across all edges
      i1=iend(i-1)
      i2=ibegin(i)
      index=where((lambda GT alam(i1)) AND (lambda LE alam(i2)))
      if (min(index) ge 0) then begin
          for k=0,n_elements(index)-1 do begin
              if(lambda(index(k)) GT 22.53 and lambda(index(k)) LT 33.04) then begin
                  i1=ibegin(i)
                  i2=iend(i)
                endif else begin
                  i1=ibegin(i-1)
                  i2=iend(i-1)
              endelse
              mu0(index(k)) = SPLINE(alam(i1:i2), amu0(i1:i2),lambda(index(k)),tension)
              mu1(index(k)) = SPLINE(alam(i1:i2), amu1(i1:i2),lambda(index(k)),tension)
          endfor
      endif
    end

    ; Note: this mod is for Jim and not part of Henke at all.
    1: begin   ; Linear Interpolation across all edges but spline where there is data
      i1=iend(i-1)
      i2=ibegin(i)
      index=where((lambda GT alam(i1)) AND (lambda LE alam(i2)))
      if (min(index) ge 0) then begin
          for k=0,n_elements(index)-1 do begin
              mu0(index(k)) = (lambda(index(k)) - alam(i1))/(alam(i2)-alam(i1))* $
                              (amu0(i2)-amu0(i1))+amu0(i1)
              mu1(index(k)) = (lambda(index(k)) - alam(i1))/(alam(i2)-alam(i1))* $
                              (amu1(i2)-amu1(i1))+amu1(i1)
          endfor
       endif
    end

  Endcase

endfor

index = where(lambda GT alam(iend(5)))
i1    = ibegin(5)
i2    = iend(5)
if (min(index) ge 0) then begin
  for k=0,N_Elements(index)-1 do begin
    mu0(index(k))   = SPLINE(alam(i1:i2), amu0(i1:i2),lambda(index(k)),tension)
    mu1(index(k))   = SPLINE(alam(i1:i2), amu1(i1:i2),lambda(index(k)),tension)
  endfor
endif

return
end

