PRO EXRAY_CONVEX_GO,wda,convexptr,xrange=xrange,yrange=yrange,over=over, $
              circle=circle,limit=limit,noprint=noprint,windID=winID, noplot=noplot

;*******************************************************************
;* This is a program to calcuate the ray-tracing for a convex      *
;* crystal spectrometer.  The inputs and keywords are as follows:  *
;*                                                                 *
;* Inputs:                                                         *
;* -------                                                         *
;*   wda:        -An array of wdf arrays.  wda must have as many   *
;*                elements are there are crystals defined in the   *
;*                input deck.                                      *
;*   convexptr:  -The pointer containing required input variables  *
;*   xrange:     -The x-range of the plot window.                  *
;*   yrange:     -The y-range of the plot window.                  *
;*                                                                 *
;* Keywords:                                                       *
;* ---------                                                       *
;*   over:       -Plot output over the current active plot window. *
;*   limit:      -Only calculate the diffraction for those rays    *
;*                which fall betweem the specified minimum and     *
;*                maximum of the film.                             *
;*   noprint:    -Do not print the wavelength values on the plot.  *
;*                                                                 *
;* Output:                                                         *
;* -------                                                         *
;*   wda:        -Contains the wavelength versus film position     *
;*                curves for each crystal.                         *
;*******************************************************************


;***************************************************************
; Calculate where the ray strikes the crystal (xc,yc)
; and calculate where the ray strikes the film (xlambda,ylambda)
;***************************************************************
if keyword_set(limit) then limit=1 else limit=0
numxstals=(*convexptr).numxstals
numsource=(*convexptr).numsource
numorders=(*convexptr).numorders
numlambda=(*convexptr).numlambda
help, numxstals,numsource, numorders , numlambda
for l=0,numxstals-1 do begin
   for i=0,numsource-1 do begin
      for j=0,numorders-1 do begin
          for k=0,numlambda-1 do begin
              theta=(*convexptr).xstal(l).theta(i,j,k)
              beta=(*convexptr).xstal(l).beta(j,k)
              if ((beta gt 0.) and (theta le 0.)) then begin
                 ;beta was given, so we need to calculate theta and lambda
                 exray_convex_thetacalc,convexptr,l,i,j,k,limit
                 theta=(*convexptr).xstal(l).theta(i,j,k)
              endif
              if (theta gt 0.) then begin
                 ;now calculate xc and yc (and beta)
                 exray_convex_betacalc,convexptr,l,i,j,k,limit
              endif
              theta=(*convexptr).xstal(l).theta(i,j,k)
            ; calculate where the ray will hit the film
              if (theta gt 0.) then $
                 exray_convex_filmposition,convexptr,l,i,j,k $
              else begin
                 ;if this lambda wasn't defined for this source, or if the lambda
                 ;won't diffract off the xstal in this order, then set xc=yc=0.
                 ;and xlambda=ylambda=dlambda=1000.
                 (*convexptr).xstal(l).xc(i,j,k)=0.
                 (*convexptr).xstal(l).yc(i,j,k)=0.
                 (*convexptr).film.xlambda(l,i,j,k)=1000.
                 (*convexptr).film.ylambda(l,i,j,k)=1000.
                 (*convexptr).film.d(l,i,j,k)=1000.
              endelse
          endfor
      endfor
   endfor
endfor

if not keyword_set(noplot) then begin
;**********************************************************
; Plot the xstals and the film plane
;**********************************************************
if (n_elements(xrange) eq 0) then begin
  xrange=[-20,20]
endif
if (n_elements(yrange) eq 0) then begin
   yrange=[-20,20]
endif
if keyword_set(over) then over = 1 else over = 0

exray_convex_plotprep,convexptr,xrange,yrange,over,windID=winid

;**********************************************************
; Plot the rays from source to the crystal, and from the
; crystal to the film.
;**********************************************************
if keyword_set(noprint) then noprint=1 else noprint=0
exray_convex_plotrays,convexptr,over,noprint,windID=winID

;**********************************************************
; Define the legend and plot it
;**********************************************************
exray_legend2,convexptr,windID=winID

endif ;(noplot)

;**********************************************************
; Create the arrays of xfilm and lambdafilm (positions of
; the specified wavelengths on the film)
;  <or> the x and y positions of the johann crossover(s)
;**********************************************************
if (n_elements(wda) ne numxstals) then begin
   print,'You must define as many wdf arrays as there are xstals'
endif else begin
   for l=0,numxstals-1 do begin
      xfilm=dblarr(numsource*numorders*numlambda)
      lambdafilm=dblarr(numsource*numorders*numlambda)
      for i=0,numsource-1 do begin
         for j=0,numorders-1 do begin
             for k=0,numlambda-1 do begin
                 theta=(*convexptr).xstal(l).theta(i,j,k)
                 if (theta gt 0.) then begin
                   ;return lambda vs x on the linear film
                    d=(*convexptr).film.d(l,i,j,k)
                    twod=(*convexptr).xstal(l).twod
                    p=(*convexptr).xstal(l).p
                    order=(*convexptr).source(i).n(j)
;                   calculate lambda in case betas given
                    lambda=twod*sin(theta-p)/order
                    xfilm(i*numorders*numlambda+j*numlambda+k)=d
                    lambdafilm(i*numorders*numlambda+j*numlambda+k)=lambda
                 endif else begin
                    xfilm(i*numorders*numlambda+j*numlambda+k)=-1000.
                    lambdafilm(i*numorders*numlambda+j*numlambda+k)=-1000.
                 endelse
             endfor
         endfor
      endfor
      index=where(xfilm gt -1000., nok)
      if (nok gt 2) then begin
        xfilm=xfilm(index)
        lambdafilm=lambdafilm(index)
        index=sort(xfilm)
        xfilm=xfilm(index)
        lambdafilm=lambdafilm(index)
        i2w,xfilm,lambdafilm,wda(l),/xy
      endif
   endfor
endelse

;**********************************************************
; exit EXRAY_GO
;**********************************************************
return
end
