; ******************************************************************
; Program for designing an FSSR system when given the distance from
; the source to the detector center as an input along with the
; central Bragg angle, etc.
;
; Inputs:
;     R0:     Bending radius of crystal (mm)
;     R_2d:   2d spacing of crystal (angstroms)
;     R_L:    Working area of crystal (mm)
;     a0:     Source-to-Xtal dist. (mm)
;     theta0: Bragg angle at center of crystal (radians)
;     Xtalarr: Array of x-axis values (-R_L/2 < Xtalarr < R_L/2)
; Outputs: (all functions of Xtalarr)
;     Lamarr: Lambda values (angstroms)
;     Tharr:  Theta values (radians)
;     Axarr:  Source-to-crystal values (mm)
;     Bxarr:  Crystal-to-Film values (mm)
;     Mxarr:  Magnification values
;     xGarr:  Film position values (mm) (in plane of film)
; Film scans can be synchronized to xGarr and then the other
; outputs will also be synchronized to the film scan.
; ******************************************************************
;
function FUNC, a1
  common params1,R0,theta0,strk0
  return,a1*a1 + a1*a1*R0*R0/(2*a1*sin(theta0)-R0)^2 - 2*a1*a1*R0*cos(2*(!pi/2.-theta0))/(2*a1*sin(theta0)-R0) - strk0*strk0
END
;
;
;pro FSSR2Dmodel,R0in,R_2din,R_Lin,a0in,theta0in,Xtalarr,Lamarr,Tharr,Axarr,Bxarr,Mxarr,xGarr,wdfi=wdfi
;
; ******************************************************************
pro FSSRdesign,R0in,R_2din,R_Lin,strk0in,theta0in,a0out,b0out,Range_arr,FilmRangeArr
; ******************************************************************
; BEGIN PROGRAM
; ******************************************************************
common params1,R0,theta0,strk0
; -----------------------------------------------------
; Begin by finding a0,b0 for declared FASTR params
; -----------------------------------------------------
R0    =R0in
R_2d  =R_2din
R_L   =R_Lin
strk0 =strk0in
theta0=theta0in
;
guess=[strk0,strk0+200.,strk0-200.]
a0 = FX_ROOT(guess,'FUNC',/double)
b0 = a0*R0/(2*a0*sin(theta0)-R0)
print,a0,FUNC(a0),b0
;
; -----------------------------------------------------
; Now calculate full FSSR range using FSSR2Dmodel
; -----------------------------------------------------
Xtalarr=DINDGEN(10001)
Xtalarr=(Xtalarr-5001)*R_L/10000.0
;  FSSR2Dmodel, R0,  R_2d,  R_L,  a0,  theta0,  Xtalarr,Lamarr,Tharr,Axarr,Bxarr,Mxarr,xGarr,wdfi=50
;FSSR2Dmodel,R0in,R_2din,R_Lin,a0in,theta0in,Xtalarr,Lamarr,Tharr,Axarr,Bxarr,Mxarr,xGarr,wdfi=wdfi
FSSR2Dmodel_2, R0,  R_2d,  R_L,  a0,  theta0,  a0,   theta0,   Xtalarr,Lamarr,Tharr,Axarr,Bxarr,Mxarr,mMxarr,xGarr,ThFarr,wdfi=50
;FSSR2Dmodel_2,R0in,R_2din,R_Lin,a0in,theta0in,a01in,theta01in,Xtalarr,Lamarr,Tharr,Axarr,Bxarr,Mxarr,mMxarr,xGarr,ThFarr,wdfi=wdfi
i2w,xGarr,Lamarr,10,/xydata,clab='Wavelength (angstroms) vs. Film Position (mm)'
;
; -----------------------------------------------------
; Calculate spectral range covered by 25 mm photocathode
; -----------------------------------------------------
CentPosition=get_time(10,R_2d*sin(theta0))
print,'Central position is at:',CentPosition
FilmRangeArr=[CentPosition-12.5,CentPosition+12.5]
plo,10,/xst,/yst,nchar=100;,xr=FilmRangeArr
; yarr1=[   LyA,    HeB,    HeG,    LyB,    LyG,    HeD,    HeA]
Larr1 = [3.7338, 3.3653, 3.1996, 3.1508, 2.9876, 3.1282, 3.9489]
oplot,!x.crange,[Larr1(0),Larr1(0)],col=1
oplot,!x.crange,[Larr1(1),Larr1(1)],col=1
oplot,!x.crange,[Larr1(2),Larr1(2)],col=1
oplot,!x.crange,[Larr1(3),Larr1(3)],col=1
oplot,!x.crange,[Larr1(4),Larr1(4)],col=1
oplot,!x.crange,[Larr1(5),Larr1(5)],col=1
oplot,!x.crange,[Larr1(6),Larr1(6)],col=1
oplot,!x.crange,[3.7365,3.7365],col=2
oplot,!x.crange,[3.7311,3.7311],col=2
  xyouts,CentPosition,Larr1(0),'Lyman-A'
  xyouts,CentPosition,Larr1(1),'He-B'
  xyouts,CentPosition,Larr1(2),'He-G'
  xyouts,CentPosition,Larr1(3),'Lyman-B'
  xyouts,CentPosition,Larr1(4),'Lyman-G'
  xyouts,CentPosition,Larr1(5),'He-D'
  xyouts,CentPosition,Larr1(6),'He-A'
;
Larr2 = [1.542,1.506,1.503,1.495,1.461,1.443,1.425,1.392]
oplot,!x.crange,[Larr2(0),Larr2(0)],col=1
oplot,!x.crange,[Larr2(1),Larr2(1)],col=1
oplot,!x.crange,[Larr2(2),Larr2(2)],col=1
oplot,!x.crange,[Larr2(3),Larr2(3)],col=1
oplot,!x.crange,[Larr2(4),Larr2(4)],col=1
oplot,!x.crange,[Larr2(5),Larr2(5)],col=1
oplot,!x.crange,[Larr2(6),Larr2(6)],col=1
oplot,!x.crange,[Larr2(7),Larr2(7)],col=1
  xyouts,CentPosition,Larr2(0),'Mn Ly-g'
  xyouts,CentPosition,Larr2(1),'Mn Ly-d'
  xyouts,CentPosition,Larr2(2),'Fe Ly-b'
  xyouts,CentPosition,Larr2(3),'Fe He-g'
  xyouts,CentPosition,Larr2(4),'Fe He-d'
  xyouts,CentPosition,Larr2(5),'Fe He-e'
  xyouts,CentPosition,Larr2(6),'Fe Ly-g'
  xyouts,CentPosition,Larr2(7),'Fe Ly-d'
;
Larr3 = [2.192,2.182,2.093,2.016,2.006,1.930,1.925]
oplot,!x.crange,[Larr3(0),Larr3(0)],col=1
oplot,!x.crange,[Larr3(1),Larr3(1)],col=1
oplot,!x.crange,[Larr3(2),Larr3(2)],col=1
oplot,!x.crange,[Larr3(3),Larr3(3)],col=1
oplot,!x.crange,[Larr3(4),Larr3(4)],col=1
oplot,!x.crange,[Larr3(5),Larr3(5)],col=1
oplot,!x.crange,[Larr3(6),Larr3(6)],col=1
  xyouts,CentPosition,Larr3(0),'Cr I.C.'
  xyouts,CentPosition,Larr3(1),'Cr He-a'
  xyouts,CentPosition,Larr3(2),'Cr Ly-a'
  xyouts,CentPosition,Larr3(3),'Mn I.C.'
  xyouts,CentPosition,Larr3(4),'Mn He-a'
  xyouts,CentPosition,Larr3(5),'Mn Ly-a1'
  xyouts,CentPosition,Larr3(6),'Mn Ly-a2'
;
print,'[   LyD,    HeD,    LyG,    LyB,    HeG,    HeB,    LyA,    HeA]'
print,'[2.9177, 3.1282, 2.9876, 3.1508, 3.1996, 3.3653, 3.7338, 3.9489]'
hak,/mess
Range_arr=[get_value(10,CentPosition-12.5),get_value(10,CentPosition+12.5)]
;print,'Range Array is:',Range_arr
IF Range_arr(0) EQ 0.0 THEN Range_arr(0)=mn(10,/quiet)
IF Range_arr(1) EQ 0.0 THEN Range_arr(1)=mx(10,/quiet)
;print,'Range Array is:',Range_arr
a0out=a0
b0out=b0
;
; -----------------------------------------------------
; Calculate spectral resolution of FASTR diagnostic
; -----------------------------------------------------
Sdiam=1.0         ; Assume a 13-mm diameter source
SRarr=Sdiam*mMxarr     ; Width of source projected on the film (mm)
DL_Dxarr=Deriv(xGarr,Lamarr)
SRarr=SRarr*DL_Dxarr   ; Multiply by local dispersion derivative (-->angstroms)
SRarr=Lamarr/SRarr     ; Divide into dispersion curve to get spectral resolution (L/dL)
i2w,Lamarr,SRarr,11,/xydata,clab='Spectral Resolution vs. Lambda (Analytic)'
;
; The spatial resolution of the streak camera is approximately one
; part in 400 at the output, so assume that this could be a limiting
; factor in the spectral resolution of the instrument.
MinWidth=0.0625 ; 1/400 of 25 mm
SRarr=MinWidth*DL_Dxarr ; Multiply by local dispersion derivative (-->angstroms)
SRarr=Lamarr/SRarr   ; Divide into dispersion curve to get spectral resolution (L/dL)
i2w,Lamarr,SRarr,12,/xydata,clab='Spectral Resolution vs. Lambda (Photocathode Limited)'
;
plo,[12,11],/xst,/yst,yr=[0,3000],nchar=100;,xr=Range_arr
;
ans='y'
read,PROMPT='Do you want to save the plots as PNG files?',ans
IF ans EQ 'y' THEN BEGIN
  startgif,'FASTRplot'
  plo,10,/xst,/yst,xr=FilmRangeArr,nchar=100
  oplot,!x.crange,[Larr1(0),Larr1(0)],col=1
  oplot,!x.crange,[Larr1(1),Larr1(1)],col=1
  oplot,!x.crange,[Larr1(2),Larr1(2)],col=1
  oplot,!x.crange,[Larr1(3),Larr1(3)],col=1
  oplot,!x.crange,[Larr1(4),Larr1(4)],col=1
  oplot,!x.crange,[Larr1(5),Larr1(5)],col=1
  oplot,!x.crange,[Larr1(6),Larr1(6)],col=1
  oplot,!x.crange,[3.7365,3.7365],col=2
  oplot,!x.crange,[3.7311,3.7311],col=2
  oplot,!x.crange,[Larr2(0),Larr2(0)],col=1
oplot,!x.crange,[Larr2(1),Larr2(1)],col=1
oplot,!x.crange,[Larr2(2),Larr2(2)],col=1
oplot,!x.crange,[Larr2(3),Larr2(3)],col=1
oplot,!x.crange,[Larr2(4),Larr2(4)],col=1
oplot,!x.crange,[Larr2(5),Larr2(5)],col=1
oplot,!x.crange,[Larr2(6),Larr2(6)],col=1
oplot,!x.crange,[Larr2(7),Larr2(7)],col=1
oplot,!x.crange,[Larr3(0),Larr3(0)],col=1
oplot,!x.crange,[Larr3(1),Larr3(1)],col=1
oplot,!x.crange,[Larr3(2),Larr3(2)],col=1
oplot,!x.crange,[Larr3(3),Larr3(3)],col=1
oplot,!x.crange,[Larr3(4),Larr3(4)],col=1
oplot,!x.crange,[Larr3(5),Larr3(5)],col=1
oplot,!x.crange,[Larr3(6),Larr3(6)],col=1
  xyouts,0.15,0.25,'Crystal 2d spacing: '+strtrim(R_2d,2),/normal
  xyouts,0.15,0.22,'Central Angle: '+strtrim(theta0*180./!pi,2),/normal
  xyouts,0.15,0.19,'Crystal size: '+strtrim(R_L,2),/normal
  xyouts,0.15,0.16,'Photocathode Dist.: '+strtrim(strk0,2),/normal
  ; yarr1=[   LyA,    HeB,    HeG,    LyB,    LyG,    HeD,    HeA]
  xyouts,CentPosition,Larr1(0),'Lyman-A'
  xyouts,CentPosition,Larr1(1),'He-B'
  xyouts,CentPosition,Larr1(2),'He-G'
  xyouts,CentPosition,Larr1(3),'Lyman-B'
  xyouts,CentPosition,Larr1(4),'Lyman-G'
  xyouts,CentPosition,Larr1(5),'He-D'
  xyouts,CentPosition,Larr1(6),'He-A'
  xyouts,CentPosition,Larr2(0),'Mn Ly-g'
  xyouts,CentPosition,Larr2(1),'Mn Ly-d'
  xyouts,CentPosition,Larr2(2),'Fe Ly-b'
  xyouts,CentPosition,Larr2(3),'Fe He-g'
  xyouts,CentPosition,Larr2(4),'Fe He-d'
  xyouts,CentPosition,Larr2(5),'Fe He-e'
  xyouts,CentPosition,Larr2(6),'Fe Ly-g'
  xyouts,CentPosition,Larr2(7),'Fe Ly-d'
  xyouts,CentPosition,Larr3(0),'Cr I.C.'
  xyouts,CentPosition,Larr3(1),'Cr He-a'
  xyouts,CentPosition,Larr3(2),'Cr Ly-a'
  xyouts,CentPosition,Larr3(3),'Mn I.C.'
  xyouts,CentPosition,Larr3(4),'Mn He-a'
  xyouts,CentPosition,Larr3(5),'Mn Ly-a1'
  xyouts,CentPosition,Larr3(6),'Mn Ly-a2'
  spng
  plo,[12,11],/xst,xr=Range_arr,/yst,yr=[0,3000],nchar=100
  xyouts,0.15,0.25,'Crystal 2d spacing: '+strtrim(R_2d,2),/normal
  xyouts,0.15,0.22,'Central Angle: '+strtrim(theta0*180./!pi,2),/normal
  xyouts,0.15,0.19,'Crystal size: '+strtrim(R_L,2),/normal
  xyouts,0.15,0.16,'Photocathode Dist.: '+strtrim(strk0,2),/normal
  xyouts,0.60,0.16,'If analytic curve not visible, it is >10000',/normal
  spng
  endgif
ENDIF
;
; -----------------------------------------------------
; Calculate spectral resolution of FASTR diagnostic
; -----------------------------------------------------
R1=R0/2.0
FO=SQRT( b0*b0 + 4*R1*R1 - 4*b0*R1*COS(!pi/2.0-theta0) )
FR=SQRT( b0*b0 + R1*R1 - 2*b0*R1*COS(!pi/2.0-theta0) )
Angle_FRO=ACOS( (FR*FR + R1*R1 - FO*FO)/(2*FR*R1) )
Angle_a1=!pi/2.0-ASIN( R1*SIN(Angle_FRO)/FO )
print,'FRO=',Angle_FRO*180./!pi
print,'A1=',Angle_a1*180./!pi
print,'RF=',FR
; ******************************************************************
; END PROGRAM
; ******************************************************************
print,'The FASTRdesign program is finished'
END

; ******************************************************************
; 2d spacing values for various crystals
; ******************************************************************
; Quartz 10-10: 8.512
; Quartz 10-11: 6.687
; Quartz 11-20: 4.912
; Quartz 10-12: 4.564
; Quartz 21-31: 3.082
; Quartz 20-23: 2.749
; Quartz 31-40: 2.3604
; Quartz 22-43: 2.030
; Quartz 50-52: 1.625
; Mica I      : 19.9149
; Mica II     : 9.96885
; Mica III    : 6.6473
; Mica V      : 3.98882
; ******************************************************************