FUNCTION EXTRACT_INST, Wave, ExpArr, W_Lasers, Time, $
                       FWHM_0, FWHM_1, $
                       WIDTH=Width, $
                       T_CUTOFF=T_Cutoff

;This can currently handle measuring the instrumental profile from the 
; emission lines of two lasers.
;
;Input
;   Wave       = array of type double that contains the wavelength-corrected 
;                horizontal axis corresponding to each lineout
;   ExpArr     = 2-dimensional array:
;                1D is radiant exposure,
;                2D specifies lineout
;   W_Lasers   = 2-element vector giving wavelengths of laser fiducials
;   Time       = vector that contains start times in ns of each lineout
;   FWHM_0     = 2-dimensional array:
;                1D will contain fit values to first laser fiducial for each lineout
;                2D will traverse fit values --
;                [Gaussian width, uncertainty, Lorentzian width, uncertainty, Voigt width]
;   FWHM_1     = same as FWHM_0 but for second laser fiducial               
;
;Optional Input
;   Width      = 2-element vector giving half-width about central wavelength of laser 
;                defining range over which to perform fit (Angstroms).
;                Each element corresponds to each laser fiducial.
;                If no input, default value is [35.,35.]
;   T_Cutoff   = 4-element vector giving time boundaries over which to collect values
;                from the Voigt fit (ns).
;                Elements 0 and 1 correspond to the start and end times, respectively, 
;                of the first laser fiducial.  Elements 2 and 3 correspond to that of 
;                the second laser fiducial.
;
;Note: - When determining FWHM values, width containing negative values are excluded
;      - T_Cutoff excludes FWHM values beyond T_Cutoff
;      Therefore, the second dimensions of FWHM_0 and FWHM_1 may have different lengths,
;       and these will always be less than or equal to the length of the second 
;       dimension of ExpArr.

IF NOT KEYWORD_SET(Width) THEN Width=[35.,35.]

nlo=N_ELEMENTS(ExpArr[0,*])
FWHM_G   = DBLARR(nlo,N_ELEMENTS(W_Lasers),/NOZERO)
FWHM_G_E = DBLARR(nlo,N_ELEMENTS(W_Lasers),/NOZERO)
FWHM_L   = DBLARR(nlo,N_ELEMENTS(W_Lasers),/NOZERO)
FWHM_L_E = DBLARR(nlo,N_ELEMENTS(W_Lasers),/NOZERO)

FOR i=0,nlo-1 DO BEGIN
  FOR j=0,N_ELEMENTS(W_Lasers)-1 DO BEGIN
    Temp=MIN( ABS(Wave-W_Lasers[j]+Width[j]),Left )
    Temp=MIN( ABS(Wave-W_Lasers[j]-Width[j]),Right )
    VFit=VOIGT_FIT(Wave[Left:Right],ExpArr[Left:Right,i],A,SigmaA)
    FWHM_G[i,j]=2*SQRT(ALOG(2))*A[5]
    FWHM_G_E[i,j]=SigmaA[5]
    FWHM_L[i,j]=2.*A[6]
    FWHM_L_E[i,j]=2.*SigmaA[6]
    
;    PLOT,Wave[Left:Right],ExpArr[Left:Right,i],XSTY=1
;    OPLOT,Wave[Left:Right],VFit,COLOR=200
;    Wait,0.3

  ENDFOR
ENDFOR

FWHM_V=0.5346*FWHM_L+SQRT(0.2166*FWHM_L*FWHM_L+FWHM_G*FWHM_G)

Angstrom = '!6!sA!r!u!9 % !6!n' 

Top=MAX([FWHM_G,FWHM_L])
;Bot=MIN([FWHM_G,FWHM_L])
LOADCT,12,/SILENT
;PLOT,X,FWHM_G[*,0],XSTY=1,YSTY=3,YRA=[0,Top],/NODATA
;OPLOT,X,FWHM_G[*,0],PSYM=-SYM(4),SYMSIZE=1.25,COLOR=80,LINESTY=1
;OPLOT,X,FWHM_L[*,0],PSYM=-SYM(9),SYMSIZE=1.25,COLOR=80,LINESTY=1
;OPLOT,X,FWHM_G[*,1],PSYM=-SYM(4),SYMSIZE=1.25,COLOR=200,LINESTY=1
;OPLOT,X,FWHM_L[*,1],PSYM=-SYM(9),SYMSIZE=1.25,COLOR=200,LINESTY=1

;Remove negative values
Ind0=WHERE(FWHM_G[*,0] GT 0 AND FWHM_L[*,0] GT 0)
Ind1=WHERE(FWHM_G[*,1] GT 0 AND FWHM_L[*,1] GT 0)

Time_0   =  Time[Ind0]
Time_1   =  Time[Ind1]

FWHM_G_0  =FWHM_G[Ind0,0]
FWHM_G_E_0=FWHM_G_E[Ind0,0]
FWHM_L_0  =FWHM_L[Ind0,0]
FWHM_L_E_0=FWHM_L_E[Ind0,0]
FWHM_V_0  =FWHM_V[Ind0,0]
FWHM_G_1  =FWHM_G[Ind1,1]
FWHM_G_E_1=FWHM_G_E[Ind1,1]
FWHM_L_1  =FWHM_L[Ind1,1]
FWHM_L_E_1=FWHM_L_E[Ind1,1]
FWHM_V_1  =FWHM_V[Ind1,1]

temp=' '
WHILE STRPOS(STRUPCASE(temp),'N') EQ -1 DO BEGIN

  PLOT,Time_0,FWHM_G_0,XSTY=1,YSTY=3,YRA=[0,Top],/NODATA,$
  XTITLE='!6Time (ns)',YTITLE='FWHM ('+Angstrom+')',CHARSIZE=1.25
  OPLOT,Time_0,FWHM_G_0,PSYM=-SYM(4),SYMSIZE=1.25,COLOR=80,LINESTY=1
  OPLOT,Time_0,FWHM_L_0,PSYM=-SYM(9),SYMSIZE=1.25,COLOR=80,LINESTY=1
  OPLOT,Time_1,FWHM_G_1,PSYM=-SYM(4),SYMSIZE=1.25,COLOR=200,LINESTY=1
  OPLOT,Time_1,FWHM_L_1,PSYM=-SYM(9),SYMSIZE=1.25,COLOR=200,LINESTY=1
  ;OPLOT,Time_0,FWHM_V_0,PSYM=-SYM(4),SYMSIZE=1.25
  ;OPLOT,Time_1,FWHM_V_1,PSYM=-SYM(9),SYMSIZE=1.25
  XYOUTS,0.18,0.85, 'BLUER LASER',/NORMAL,COLOR=80
  XYOUTS,0.18,0.81,'REDDER LASER',/NORMAL,COLOR=200
  XYOUTS,0.82,0.85, 'GAUSSIAN',/NORMAL
  PLOTS, 0.80,0.855,PSYM=SYM(4),SYMSIZE=1.25,/NORMAL
  XYOUTS,0.82,0.81,'LORENTZIAN',/NORMAL
  PLOTS, 0.80,0.815,PSYM=SYM(9),SYMSIZE=1.25,/NORMAL
  
  PRINT,'Would you like to remove a point? (y/n)'
  READ,temp
  If STRPOS(STRUPCASE(temp),'N') NE -1 THEN BREAK
  
  PRINT,'For the bluer or redder laser? (b/r)'
  READ,temp
  
  IF STRPOS(STRUPCASE(temp),'B') NE -1 THEN BEGIN
    
    WHILE (STRPOS(STRUPCASE(temp),'B') NE -1) OR $
    (STRPOS(STRUPCASE(temp),'N') NE -1) DO BEGIN
      PRINT,'Give index number:'
      READ,temp
      RInd=LONG(temp)
      PLOT,Time_0,FWHM_G_0,XSTY=1,YSTY=3,YRA=[0,Top],/NODATA,$
      XTITLE='!6Time (ns)',YTITLE='FWHM ('+Angstrom+')',CHARSIZE=1.25
      OPLOT,Time_0,FWHM_G_0,PSYM=-SYM(4),SYMSIZE=1.25,COLOR=80,LINESTY=1
      OPLOT,Time_0,FWHM_L_0,PSYM=-SYM(9),SYMSIZE=1.25,COLOR=80,LINESTY=1
      OPLOT,Time_1,FWHM_G_1,PSYM=-SYM(4),SYMSIZE=1.25,COLOR=200,LINESTY=1
      OPLOT,Time_1,FWHM_L_1,PSYM=-SYM(9),SYMSIZE=1.25,COLOR=200,LINESTY=1
      XYOUTS,0.18,0.85, 'BLUER LASER',/NORMAL,COLOR=80
      XYOUTS,0.18,0.81,'REDDER LASER',/NORMAL,COLOR=200
      XYOUTS,0.82,0.85, 'GAUSSIAN',/NORMAL
      PLOTS, 0.80,0.855,PSYM=SYM(4),SYMSIZE=1.25,/NORMAL
      XYOUTS,0.82,0.81,'LORENTZIAN',/NORMAL
      PLOTS, 0.80,0.815,PSYM=SYM(9),SYMSIZE=1.25,/NORMAL
      
      LOADCT,5,/SILENT
      PLOTS,Time_0[RInd],FWHM_G_0[RInd],PSYM=-SYM(4),SYMSIZE=2,COLOR=200
      PLOTS,Time_0[RInd],FWHM_L_0[RInd],PSYM=-SYM(9),SYMSIZE=2,COLOR=200
      LOADCT,12,/SILENT
      PRINT,'This one? (y/n)'
      READ,temp
    ENDWHILE
    
    ;Remove point
    IF RInd EQ 0 THEN BEGIN
      Time_0     = Time_0[1:*]
      FWHM_G_0   = FWHM_G_0[1:*]
      FWHM_G_E_0 = FWHM_G_E_0[1:*]
      FWHM_L_0   = FWHM_L_0[1:*]
      FWHM_L_E_0 = FWHM_L_E_0[1:*]
      FWHM_V_0   = FWHM_V_0[1:*]
    ENDIF
    IF RInd EQ N_ELEMENTS(Time_0)-1 THEN BEGIN
      Time_0     = Time_0[0:N_ELEMENTS(Time_0)-2]
      FWHM_G_0   = FWHM_G_0[0:N_ELEMENTS(Time_0)-2]
      FWHM_G_E_0 = FWHM_G_E_0[0:N_ELEMENTS(Time_0)-2]
      FWHM_L_0   = FWHM_L_0[0:N_ELEMENTS(Time_0)-2]
      FWHM_L_E_0 = FWHM_L_E_0[0:N_ELEMENTS(Time_0)-2]
      FWHM_V_0   = FWHM_V_0[0:N_ELEMENTS(Time_0)-2]
    ENDIF
    IF (RInd GT 0) AND (RInd LT N_ELEMENTS(Time_0)-1) THEN BEGIN
      Time_0     = [Time_0[0:RInd-1],Time_0[RInd+1:*]]
      FWHM_G_0   = [FWHM_G_0[0:RInd-1],FWHM_G_0[RInd+1:*]]
      FWHM_G_E_0 = [FWHM_G_E_0[0:RInd-1],FWHM_G_E_0[RInd+1:*]]
      FWHM_L_0   = [FWHM_L_0[0:RInd-1],FWHM_L_0[RInd+1:*]]
      FWHM_L_E_0 = [FWHM_L_E_0[0:RInd-1],FWHM_L_E_0[RInd+1:*]]
      FWHM_V_0   = [FWHM_V_0[0:RInd-1],FWHM_V_0[RInd+1:*]]
    ENDIF
    
  ENDIF ELSE BEGIN
    
    WHILE (STRPOS(STRUPCASE(temp),'R') NE -1) OR $
    (STRPOS(STRUPCASE(temp),'N') NE -1) DO BEGIN
      PRINT,'Give index number:'
      READ,temp
      RInd=LONG(temp)
      PLOT,Time_0,FWHM_G_0,XSTY=1,YSTY=3,YRA=[0,Top],/NODATA,$
      XTITLE='!6Time (ns)',YTITLE='FWHM ('+Angstrom+')',CHARSIZE=1.25
      OPLOT,Time_0,FWHM_G_0,PSYM=-SYM(4),SYMSIZE=1.25,COLOR=80,LINESTY=1
      OPLOT,Time_0,FWHM_L_0,PSYM=-SYM(9),SYMSIZE=1.25,COLOR=80,LINESTY=1
      OPLOT,Time_1,FWHM_G_1,PSYM=-SYM(4),SYMSIZE=1.25,COLOR=200,LINESTY=1
      OPLOT,Time_1,FWHM_L_1,PSYM=-SYM(9),SYMSIZE=1.25,COLOR=200,LINESTY=1
      XYOUTS,0.18,0.85, 'BLUER LASER',/NORMAL,COLOR=80
      XYOUTS,0.18,0.81,'REDDER LASER',/NORMAL,COLOR=200
      XYOUTS,0.82,0.85, 'GAUSSIAN',/NORMAL
      PLOTS, 0.80,0.855,PSYM=SYM(4),SYMSIZE=1.25,/NORMAL
      XYOUTS,0.82,0.81,'LORENTZIAN',/NORMAL
      PLOTS, 0.80,0.815,PSYM=SYM(9),SYMSIZE=1.25,/NORMAL
      
      LOADCT,5,/SILENT
      PLOTS,Time_1[RInd],FWHM_G_1[RInd],THICK=2,PSYM=-SYM(4),SYMSIZE=2,COLOR=200
      PLOTS,Time_1[RInd],FWHM_L_1[RInd],THICK=2,PSYM=-SYM(9),SYMSIZE=2,COLOR=200
      LOADCT,12,/SILENT
      PRINT,'This one? (y/n)'
      READ,temp
    ENDWHILE
    
    ;Remove point
    IF RInd EQ 0 THEN BEGIN
      Time_1     = Time_1[1:*]
      FWHM_G_1   = FWHM_G_1[1:*]
      FWHM_G_E_1 = FWHM_G_E_1[1:*]
      FWHM_L_1   = FWHM_L_1[1:*]
      FWHM_L_E_1 = FWHM_L_E_1[1:*]
      FWHM_V_1   = FWHM_V_1[1:*]
    ENDIF
    IF RInd EQ N_ELEMENTS(Time_1)-1 THEN BEGIN
      Time_1     = Time_1[0:N_ELEMENTS(Time_1)-2]
      FWHM_G_1   = FWHM_G_1[0:N_ELEMENTS(Time_1)-2]
      FWHM_G_E_1 = FWHM_G_E_1[0:N_ELEMENTS(Time_1)-2]
      FWHM_L_1   = FWHM_L_1[0:N_ELEMENTS(Time_1)-2]
      FWHM_L_E_1 = FWHM_L_E_1[0:N_ELEMENTS(Time_1)-2]
      FWHM_V_1   = FWHM_V_1[0:N_ELEMENTS(Time_1)-2]
    ENDIF
    IF (RInd GT 0) AND (RInd LT N_ELEMENTS(Time_1)-1) THEN BEGIN
      Time_1     = [Time_1[0:RInd-1],Time_1[RInd+1:*]]
      FWHM_G_1   = [FWHM_G_1[0:RInd-1],FWHM_G_1[RInd+1:*]]
      FWHM_G_E_1 = [FWHM_G_E_1[0:RInd-1],FWHM_G_E_1[RInd+1:*]]
      FWHM_L_1   = [FWHM_L_1[0:RInd-1],FWHM_L_1[RInd+1:*]]
      FWHM_L_E_1 = [FWHM_L_E_1[0:RInd-1],FWHM_L_E_1[RInd+1:*]]
      FWHM_V_1   = [FWHM_V_1[0:RInd-1],FWHM_V_1[RInd+1:*]]
    ENDIF

  ENDELSE
  
ENDWHILE


IF NOT KEYWORD_SET(T_Cutoff) THEN BEGIN
  PRINT,'Introduce time boundary for fit values? (y/n)'
  temp=' '
  READ,temp
  If STRPOS(STRUPCASE(temp),'N') EQ -1 THEN BEGIN
    T_Cutoff=FLTARR(4)
    PRINT,'Enter start time boundary for bluer laser:'
    READ,temp
    T_Cutoff[0]=FLOAT(temp)
    PRINT,'Enter end time boundary for bluer laser:'
    READ,temp
    T_Cutoff[1]=FLOAT(temp)
    PRINT,'Enter start time boundary for redder laser:'
    READ,temp
    T_Cutoff[2]=FLOAT(temp)
    PRINT,'Enter end time boundary for redder laser:'
    READ,temp
    T_Cutoff[3]=FLOAT(temp)
  ENDIF
ENDIF
IF KEYWORD_SET(T_Cutoff) THEN BEGIN
  PLOTS,T_Cutoff[0],0
  PLOTS,T_Cutoff[0],Top,/CONTINUE,THICK=2,LINESTY=2,COLOR=80
  PLOTS,T_Cutoff[1],0
  PLOTS,T_Cutoff[1],Top,/CONTINUE,THICK=2,LINESTY=2,COLOR=80
  PLOTS,T_Cutoff[2],0
  PLOTS,T_Cutoff[2],Top,/CONTINUE,THICK=2,LINESTY=2,COLOR=200
  PLOTS,T_Cutoff[3],0
  PLOTS,T_Cutoff[3],Top,/CONTINUE,THICK=2,LINESTY=2,COLOR=200
ENDIF

Num0=LONARR(2)
Num1=LONARR(2)
IF T_Cutoff NE !NULL THEN BEGIN
  Num0[0]=MAX( WHERE(Time_0 LT T_Cutoff[0]) )
  Num0[1]=MIN( WHERE(Time_0 GT T_Cutoff[1]) )-1
  IF Num0[0] LT 0 THEN Num0[0]=0
  IF Num0[0] GT N_ELEMENTS(FWHM_G_0)-1 THEN Num0[0]=N_ELEMENTS(FWHM_G_0)-1
  IF (Num0[1] LT 0) OR (Num0[1] GT N_ELEMENTS(FWHM_G_0)-1) THEN Num0[1]=N_ELEMENTS(FWHM_G_0)-1
  Num1[0]=MAX( WHERE(Time_1 LT T_Cutoff[2]) )
  Num1[1]=MIN( WHERE(Time_1 GT T_Cutoff[3]) )-1
  IF Num1[0] LT 0 THEN Num1[0]=0
  IF Num1[0] GT N_ELEMENTS(FWHM_G_1)-1 THEN Num1[0]=N_ELEMENTS(FWHM_G_1)-1
  IF (Num1[1] LT 0) OR (Num1[1] GT N_ELEMENTS(FWHM_G_1)-1) THEN Num1[1]=N_ELEMENTS(FWHM_G_1)-1
ENDIF ELSE BEGIN
  Num0[0]=0
  Num0[1]=N_ELEMENTS(FWHM_G_0)-1
  Num1[0]=0
  Num1[1]=N_ELEMENTS(FWHM_G_1)-1
ENDELSE
;Guassian,uncertainty,Lorentzian,uncertainty,Voigt
FWHM_0=DBLARR(Num0[1]-Num0[0]+1,5)
FWHM_1=DBLARR(Num1[1]-Num1[0]+1,5)

FWHM_0[*,0]=  FWHM_G_0[Num0[0]:Num0[1]]
FWHM_0[*,1]=FWHM_G_E_0[Num0[0]:Num0[1]]
FWHM_0[*,2]=  FWHM_L_0[Num0[0]:Num0[1]]
FWHM_0[*,3]=FWHM_L_E_0[Num0[0]:Num0[1]]
FWHM_0[*,4]=  FWHM_V_0[Num0[0]:Num0[1]]
FWHM_1[*,0]=  FWHM_G_1[Num1[0]:Num1[1]]
FWHM_1[*,1]=FWHM_G_E_1[Num1[0]:Num1[1]]
FWHM_1[*,2]=  FWHM_L_1[Num1[0]:Num1[1]]
FWHM_1[*,3]=FWHM_L_E_1[Num1[0]:Num1[1]]
FWHM_1[*,4]=  FWHM_V_1[Num1[0]:Num1[1]]


PRINT,''
PRINT,'  -- -- --  Laser at '+STRCOMPRESS(STRING(W_Lasers[0]),/REMOVE_ALL)+' AA  -- -- --  '
PRINT,'       Mean            Median         Standard Deviation'
PRINT,'Gaussian component FWHM: '
PRINT,MEAN(FWHM_0[*,0]),MEDIAN(FWHM_0[*,0]),SQRT(VARIANCE(FWHM_0[*,0]))
PRINT,'Lorenztian component FWHM: '
PRINT,MEAN(FWHM_0[*,2]),MEDIAN(FWHM_0[*,2]),SQRT(VARIANCE(FWHM_0[*,2]))
PRINT,'Convolved Voigt FWHM: '
PRINT,MEAN(FWHM_0[*,4]),MEDIAN(FWHM_0[*,4]),SQRT(VARIANCE(FWHM_0[*,4]))
PRINT,''
PRINT,'  -- -- --  Laser at '+STRCOMPRESS(STRING(W_Lasers[1]),/REMOVE_ALL)+' AA  -- -- --  '
PRINT,'       Mean            Median         Standard Deviation'
PRINT,'Gaussian component FWHM: '
PRINT,MEAN(FWHM_1[*,0]),MEDIAN(FWHM_1[*,0]),SQRT(VARIANCE(FWHM_1[*,0]))
PRINT,'Lorenztian component FWHM: '
PRINT,MEAN(FWHM_1[*,2]),MEDIAN(FWHM_1[*,2]),SQRT(VARIANCE(FWHM_1[*,2]))
PRINT,'Convolved Voigt FWHM: '
PRINT,MEAN(FWHM_1[*,4]),MEDIAN(FWHM_1[*,4]),SQRT(VARIANCE(FWHM_1[*,4]))

LOADCT,5,/SILENT

END