; ------------------------------------------------------------------------
; $Id: acis2cim.pro,v 1.2 2009/04/14 21:35:05 lpmix Exp $
; ------------------------------------------------------------------------
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;

pro acis2cim, geofile, strp, str=str, blks=blks, skip=skip, logfile=logfile, $
              errfile=errfile, quiet=quiet, geom=geom, conductors=conds, $
              materials=mats, cname=cname, mname=mname, cm=cm, mm=mm, $
              inches=inches, helpme=helpme
;+
; ; NAME:
;        acis2cim
;
; PURPOSE:
;       Convert an ACIS solid model to a QuickSilver CIM structure,
;       using a supplied Quicksilver grid.
;
; CALLING SEQUENCE:
;       ACIS2CIM, geofile [, STR] [, /GEOM] [, STR=str_KW] [, BLKS=blks]
;                 [, SKIP=skip] [, LOGFILE=logfile] [, ERRFILE=errfile]
;                 [, /QUIET] [, CONDUCTORS=conds] [, MATERIALS=mats]
;                 [, CNAME=cname] [, MNAME=mname] [, /HELPME]
; INPUTS:
;       geofile  - Name of the file containing the multi-block grid
;                  data from Quicksilver. By default, this is assumed
;                  to be the file produced using Quicksilver's "WRITE
;                  GRID" command. If the GEOM keyword is set, the file
;                  is assumed to be in the form written by
;                  Quicksilver's "WRITE GEOMETRY" command. In this
;                  case, the supplied grids in each block are padded
;                  with one buffer cell at both ends of each
;                  coordinate direction using a mirror of the nearest
;                  neighbor. An interleaved "half"-grid is generated,
;                  each grid-point being centered in the full-grid
;                  cell defined by the two surrounding original
;                  grid-points.
;                  DEFAULT: If GEOFILE is zero or undefined, 
;                           DIALOG_PICKFILE will be used to determine file.
;         STR:     Integer structure array number in which to store
;                  the CIM array. If not supplied, the first available
;                  unused structure number will be used, or
;                  MaxStructure-1 if all structures are already in use.
;                  
;  
; KEYWORD PARAMETERS:
;       GEOM       If set, geofile is treated as a Quicksilver
;                  GEOMETRY file rather than a Quicksilver GRID file
;                  (see description of geofile above).
;       STR        Integer structure array number in which to store
;                  the CIM array. If not supplied, the first available
;                  unused structure number will be used, or
;                  MaxStructure-1 if all structures are already in use.
;           Note:  If the parameter, STR, is present, then str_KW is ignored.
;
;       BLKS       Integer array containing the block indices (first
;                  block has index 1) of the grid blocks that are to
;                  be processed with the ACIS solid model.
;       SKIP       Integer array containing the block indices (first
;                  block has index 1) of the grid blocks that are NOT
;                  to be processed with the ACIS solid model.
;           Note:  If neither BLKS or SKIP are defined, then all
;                  blocks are processed. If both are supplied, all
;                  blocks specified by BLKS, other than any that are
;                  also specified by SKIP, are processed.
;       LOGFILE    Name of file to which the ACIS interface library's
;                  informational messages are to be routed. If not
;                  supplied or equal to zero, they are written to the
;                  terminal. If LOGFILE is not a string or zero,
;                  messages will be written to acis.log.
;       ERRFILE    Name of file to which the ACIS interface library's
;                  error messages are to be routed. If not supplied or
;                  equal to zero, they are written to the terminal. If
;                  ERRFILE is not a string or zero, messages will be
;                  written to acis.err.
;       QUIET      If set, informational messages from the ACIS
;                  interface library are supressed.
;       CONDUCTORS Array of ACIS part numbers that are to be processed
;                  as conductors.
;       MATERIALS  Array of ACIS part numbers that are to be processed
;                  as materials.
;           Note:  If neither CONDUCTORS or MATERIALS are supplied,
;                  then all ACIS parts are processed as conductors.
;                  The AHELP procedure can be used to determine the
;                  numbers of available ACIS parts.
;       CNAME      An array of names for the conductors that will be
;                  processed. Supplied names must be distinct from one
;                  another. If not supplied, the corresponding ACIS
;                  part names will be used, or if fewer names are
;                  supplied than the number of conductors to be
;                  processed, the ACIS part names will be used for
;                  the remaining conductors. If extra names are
;                  supplied, they are ignored.
;       MNAME      An array of names for the materials that will be
;                  processed. Supplied names must be distinct from one
;                  another. If not supplied, the corresponding ACIS
;                  part names will be used, or if fewer names are
;                  supplied than the number of materials to be
;                  processed, the ACIS part names will be used for
;                  the remaining materials. If extra names are
;                  supplied, they are ignored.
;       CM         If set, ACIS parts are assumed to be in centimeters
;       MM         If set, ACIS parts are assumed to be in millimeters
;       INCHES     If set, ACIS parts are assumed to be in inches
;       HELPME     If set, help message will be printed.
;
; COMMON BLOCKS:
;       ACISCOM
;       ACIS_PARTS_COM
;
; EXAMPLE: Use the multiblock grid defined in qsgrid.pff to generate a
;          CIM structure from the ACIS parts currently available. The
;          generated structure is to be stored as pfidl structure
;          2. Include all blocks from the grid except 1 and 14 when
;          constructing the CIM. Map ACIS part number 3 to a conductor
;          named "anode" and part number 1 to a conductor named
;          "cathode". Map ACIS part number 4 to a material named
;          "teflon". Direct any informational messages to the default log
;          file (acis.log) rather than to the terminal.
;
;         ACIS2CIM, 'qsgrid.pff', str=2, skip=[1,14],cond=[3,1], mat=4, 
;              cname=['anode','cathode'], mname='teflon', /log
;    or
;         ACIS2CIM, 'qsgrid.pff', 2, skip=[1,14],cond=[3,1], mat=4, 
;              cname=['anode','cathode'], mname='teflon', /log
;
; SEE ALSO:
;      READ_ACIS, AHELP, LIST_ACIS_PARTS
;
; MODIFICATION HISTORY:
;      Original version, D. B. Seidel, Sept. 26, 2008
;-
;
on_error, 1

common aciscom, initacis, acis_exe
common acis_parts_com, part_name, material_name, npart_save

if keyword_set(helpme) then begin
  print, '% Format: ACIS2CIM, geofile [, STR] [, /GEOM] [, STR=str] [, BLKS=blks]'
  print, '              [, SKIP=skip] [, LOGFILE=logfile] [, ERRFILE=errfile]'
  print, '              [, /QUIET] [, CONDUCTORS=conds] [, MATERIALS=mats]'
  print, '              [, PARTS = parts] [, FILL_VALUE = fill_value]'
  print, '              [, CNAME=cname] [, MNAME=mname] [, /HELPME]'
  return
endif

;
;     check to see if ACIS has been initialized
;
if (n_elements(initacis) eq 0) then init_acis $
else if (initacis eq 0) then init_acis
if initacis eq 0 then begin
  print, '% ACIS2CIM: ACIS libraries are not supported by this System'
  return
endif

if npart_save eq 0 then begin
  print, '% ACIS2CIM: No ACIS parts have been read'
  return
endif

openpff,geofile, error = ier
if ier ne 0 then begin
  print, '% ACIS2CIM: PFF file not opened'
  return
endif

if keyword_set(geom) then readgrd,f,2 $
else begin
  readgrd,f,1
  readgrd,h,2
endelse
closepff

coord = f.spare[0,0] - 1

maxstruct = get_maxstr()
tmpstr = maxstruct

nunits = 0
uscale = 1.0

if keyword_set(cm)     then begin & uscale = 100.0  & ++nunits & endif
if keyword_set(mm)     then begin & uscale = 1000.0 & ++nunits & endif
if keyword_set(inches) then begin & uscale = 39.37  & ++nunits & endif
if nunits gt 1 then begin
  print,'% ACIS2CIM: ', $
    'Only one ACIS-parts unit specifier (/CM, /MM, /INCHES) is allowed'
  return
endif

; print,nunits,uscale,coord

supplied = 1
if n_elements(strp) eq 0 then $
  if n_elements(str) gt 0 then strp = str else supplied = 0
 
type = size(strp,/type)

if supplied && type lt 4 then begin
  strp= strp[0]
  if strp le 0 or strp ge maxstruct then supplied = 0
endif

if supplied eq 0 then begin
  free = strfree(max = maxstruct -1, nel = nel)
  if nel eq 0 then begin
    strp = maxstruct - 1
    print,' *** No empty structures available. Using MaxStructure-1 (' + $
          strtrim(string(strp),2) + ')'
  endif else strp = free[0]
  print,'% ACIS2CIM: Output will be returned in Structure # ',strp
endif
str = strp

nblk = f.block

nconds = n_elements(conds)
nmats = n_elements(mats)
if nconds + nmats eq 0 then begin
  nparts = npart_save
  parts = 1 + indgen(nparts)
  vals = 2*parts + 1
  nconds = nparts
  conds = parts
endif else begin
  nparts = nconds + nmats
  parts = make_array(/int, nparts)
  vals  = make_array(/int, nparts)
  if nconds gt 0 then begin
    parts[0:nconds-1] = conds
    vals[0:nconds-1] = 2*indgen(nconds) + 3
  endif
  if nmats gt 0 then begin
    parts[nconds:nparts-1] = mats
    vals[nconds:nparts-1] = 2*indgen(nmats) + 2
  endif
endelse

clab = 'CIM ARRAY - '
prefix=''
if nconds gt 0 then begin
  nlab = n_elements(cname) < nconds
  for i=0,nlab-1 do begin
    clab += prefix + cname[i] + '=' + strtrim(string(i+1),2)
    prefix=';;'
  endfor
  for i=nlab,nconds-1 do begin
    clab += prefix + part_name[conds[i]-1] + '=' + strtrim(string(i+1),2)
    prefix=';;'
  endfor
endif

if nmats gt 0 then begin
  nlab = n_elements(mname) < nmats
  for i=0,nlab-1 do begin
    clab += prefix + mname[i] + '=-' + strtrim(string(i+1),2)
    prefix=';;'
  endfor
  for i=nlab,nmats-1 do begin
    clab += prefix + part_name[mats[i]-1] + '=-' + strtrim(string(i+1),2)
    prefix=';;'
  endfor
endif

print,'Ncon,Nmat:  ',nconds,nmats
print,'Parts:  ',parts
print,'values: ',vals
print,'clab: ',clab

if n_elements(blks) eq 0 then blist = make_array(nblk,/int,val=1) $
else begin
  blist = make_array(nblk,/int,val=0)
  blist(blks-1) = 1
endelse

if n_elements(skip) ne 0 then blist(skip-1) = 0

used_blks = where(blist eq 1) + 1
nused = n_elements(used_blks)
print,nused,used_blks

logstr = ''
restorestr = ''
if n_elements(logfile) gt 0 then begin
  s=size(logfile)
  ok = 1
  if s[0] ne 0 then ok = 0 $
  else if s[1] eq 7 then logstr += ",log='"+strtrim(logfile,2)+"'" $
  else if s[1]/2 eq 1 then $
    logstr += ',log='+strcompress(string(logfile),/rem) $
  else ok = 0
  if ok eq 0 then print,"% ACIS2CIM: Unrecognizable logfile value: ",logfile $
  else restorestr += ',log=0'
endif

if n_elements(errfile) gt 0 then begin
  s=size(errfile)
  ok = 1
  if s[0] ne 0 then ok = 0 $
  else if s[1] eq 7 then logstr += ",err='"+strtrim(errfile,2)+"'" $
  else if s[1]/2 eq 1 then $
    logstr += ',err='+strcompress(string(errfile),/rem) $
  else ok = 0
  if ok eq 0 then print,"% ACIS2CIM: Unrecognizable errfile value: ",errfile $
  else restorestr += ',err=0'
endif

if n_elements(quiet) gt 0 then begin
  s=size(quiet)
  ok = 1
  if s[0] ne 0 then ok = 0 $
  else if s[1]/2 eq 1 then $
    logstr += ',quiet='+strcompress(string(quiet),/rem) $
  else ok = 0
  if ok eq 0 then print,"% ACIS2CIM: Unrecognizable quiet value: ",quiet $
  else restorestr += ',quiet=0'
endif


if strlen(logstr) gt 0 then begin
  print,'log string:     "' + logstr + '"'
  print,'restore string: "' + restorestr + '"'
  commandstr = 'acis_setlog' + logstr
  r = execute(commandstr)
  if r eq 0 then begin
    print, '% ACIS2CIM: Unable to set logfile'
    print, ' Command: '+'"'+commandstr+'"'
  endif
endif

for i=0,nused-1 do begin

  blk = used_blks[i]
  if not keyword_set(quiet) then print,"Processing block ",blk

  if keyword_set(geom) then begin
    xf = get_array(f,1,blk)
    yf = get_array(f,2,blk)
    zf = get_array(f,3,blk)

    nx = n_elements(xf)
    xf = [ 2.0*xf[0] - xf[1], xf, 2.0*xf[nx-1] - xf[nx-2] ]
    ++nx
    xh = 0.5*(xf[1:nx] + xf[0:nx-1])

    ny = n_elements(yf)
    yf = [ 2.0*yf[0] - yf[1], yf, 2.0*yf[ny-1] - yf[ny-2] ]
    ++ny
    yh = 0.5*(yf[1:ny] + yf[0:ny-1])

    nz = n_elements(zf)
    zf = [ 2.0*zf[0] - zf[1], zf, 2.0*zf[nz-1] - zf[nz-2] ]
    ++nz
    zh = 0.5*(zf[1:nz] + zf[0:nz-1])
  endif else begin
    xf = get_array(f,1,blk)
    yf = get_array(f,2,blk)
    zf = get_array(f,3,blk)
    xh = get_array(h,1,blk)
    yh = get_array(h,2,blk)
    zh = get_array(h,3,blk)
    nx = n_elements(xh)
    ny = n_elements(yh)
    nz = n_elements(zh)
    if nx + 1 ne n_elements(xf) or ny + 1 ne n_elements(yf) or $
       nz + 1 ne n_elements(zf) then begin
print,'% ACIS2CIM: Full and half grids for block '+strtrim(blk, 2)+' are not consistent'
      print,'  Nxf: ',n_elements(xf),'  Nxh: ',nx
      print,'  Nyf: ',n_elements(yf),'  Nyh: ',ny
      print,'  Nzf: ',n_elements(zf),'  Nzh: ',nz
      return
    endif
  endelse

  if nunits eq 1 then begin
    xh *= uscale
    if coord lt 1 then yh *= uscale
    if coord lt 2 then zh *= uscale
    xf *= uscale
    if coord lt 1 then yf *= uscale
    if coord lt 2 then zf *= uscale
  endif

  if i eq 0 then tstr = str else tstr = tmpstr

  voxels,xh,xf,yh,yf,zh,zf,str=tstr,coord=coord,parts=parts,fill=vals

  if size(tstr,/type)/2 eq 1 then begin
    s2i,/no_copy,a,tstr
    a.spare[1] = blk
    i2s,/no_copy,a,tstr
  endif

  if i gt 0 then joinstr,str,tmpstr
  
endfor

chastr,str,clab=clab
if nunits eq 1 then begin
  uscale = 1.0/uscale
  chastr,str,mxdim=uscale
  if coord lt 1 then chastr,str,mydim=uscale
  if coord lt 2 then chastr,str,mzdim=uscale
endif


if strlen(restorestr) gt 0 then begin
  commandstr ='acis_setlog' + restorestr
  r = execute(commandstr)
  if r eq 0 then begin
    print, '% ACIS2CIM: Unable to restore logfile'
    print, ' Command: '+'"'+commandstr+'"'
  endif
endif
end
