; ------------------------------------------------------------------------ 
; $Id: makepoly.pro,v 1.2 2005/03/09 22:20:18 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2005 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
;+ 
; format: makepoly, x, y, z, ijk,  partname= partname, material = material
; Make a cylinder of a constant polygonal cross-section along a given
; coordinate axis.  (an extruded polygon)
;
;   X, Y and Z are a right handed coordinate system.
;    
;     where:
;        X, Y  - List of x, y coordinates describing the polygonal cross-section.
;                polygon is closed by ACIS ie. First point is added to the end.
;        Z     - 2 element vector.  Values are the beginning and ending points
;                in the uniform direction.
;        ijk   - Coordinate direction for the Z axis.  Must be 1, 2, or 3.
;        partname - Name of part. Default = 'Polygon_Extrusion'
;        material - Name of material. Default = 'Conductor'
; Example:
;    Make a polygon extrusion parallel to the x axis with a shape in the 
;    Y,Z plane given by yval and zval and call the part 'Feed'. Make the part
;    extend from x = -1 to x = 1.
;      makepoly, yval, zval, [-1, 1], 1, part='Feed'
;
;    Same as above but axis in the y direction.
;      makepoly, zval, xval, [-1, 1], 2, part= 'Feed'
; Note:  To have a right handed coordinate system: zvalues, xvalues, then yvalues.
;    
;    Original Version: January 6, 1999, lpm
;-
 pro makepoly, xval, yval, zval, ijk, partname =pname, material = matname
on_error, 1
  common aciscom, initacis, acis_exe
  common acis_parts_com, part_name, material_name, npart_save

  npar = n_params()
  if npar eq 0 then begin
    print, '% Format: MakePoly,  xval, yval, zval, ijk'
    print, '        [, PARTNAME = partname]  [, MATERIAL= material]'
    return
  endif

;
;     check to see if ACIS has been initialized
;
      if (n_elements(initacis) eq 0) then init_acis else $
        if (initacis eq 0) then init_acis
      if initacis eq 0 then begin
        print, '% MakePoly: ACIS libraries are not supported by this System'
        return
      endif
  if npar lt 4 then begin
    Print, '% MAKEPOLY: Four parameters are required.
    return
  endif
  

  nx = n_elements(xval)
  ny = n_elements(yval)
  nz = n_elements(zval)
  ni = n_elements(ijk)
  if nx*ny*nz*ni eq 0 then begin
    Print, '% MakePoly:  X, Y and Z values and the extrusion axis must be defined.
    return
  endif
  if nx ne ny then begin
    Print, '% MakePoly:  X, Y must have the same number of elements'
    return
  endif
  if nz lt 2 then begin
    Print, '% MakePoly:  A Minimum and Maximum Z-value must be provided.
    return
  endif
  kval = long(ijk(0))
  if kval lt 1 or kval gt 3 then begin
    Print, '% MakePoly:  Extrusion axis must be in the range 1 to 3.'
    return
  endif

  z_acis = make_array(2, /double)
  x_acis = double(xval)
  y_acis = double(yval)
  z_acis(0) = min(zval(0:1), max = mm)
  z_acis(1) = mm

  npart = -1L

  if n_elements(pname) eq 0 then pname = 'Cylinder_Part' else begin
    pname = strtrim (pname, 2)
    len = strlen(pname)
    if len eq 0 then pname = 'Polygon_Extrusion'
  endelse
  if n_elements(matname) eq 0 then matname = 'Conductor' else begin
    matname = strtrim (matname, 2)
    len = strlen(matname)
    if len eq 0 then matname = 'Conductor'
  endelse
  pnameb = [byte(pname), 0B]
  matnameb = [byte(matname), 0B]
;
      if initacis eq 1 then begin
; HP
        junk = call_external(ACIS_EXE, 'acis_poly', kval, nx, x_acis, $
          y_acis, z_acis, pnameb, matnameb, npart)
      endif else if initacis eq 2 then begin
; Sun
        junk = call_external(ACIS_EXE, 'acis_poly', kval, nx, x_acis, $
         y_acis, z_acis,  pnameb, matnameb, npart)
      endif else if initacis eq 3 then begin
; VMS
        file = string(p3)
        junk = call_external(ACIS_EXE, 'acis_poly', kval, nx, x_acis, $
          y_acis, z_acis, pnameb, matnameb, npart)
      endif else if initacis eq 4 then begin
; WIN
       junk = call_external(ACIS_EXE, 'ACIS_POLY@8', kval, nx, x_acis, $
          y_acis, z_acis, pnameb, matnameb, npart)
      endif else if initacis eq 5 then begin
; MAC
         junk = call_external(ACIS_EXE, 'acis_poly', kval, nx, x_acis,  $
          y_acis, z_acis, pnameb, matnameb, npart)
      endif

  if npart le 0 then begin
    Print, '% MakePoly: Unable to make extruded polygon'
    return
  endif
  npart_save = npart
  if npart eq 1 then begin
    part_name = pname
    material_name = matname
  endif else begin
    part_name = [part_name, pname]
    material_name = [material_name, matname]
  endelse
  return
  end


