; ------------------------------------------------------------------------
; $Id: merge_cim_parts.pro,v 1.1 2009/04/16 17:33:00 lpmix Exp $
; ------------------------------------------------------------------------
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;

pro merge_cim_parts, s1, s2, oldlist=oldlist, newname=newname, $
                     material=material
;+
; ; NAME:
;        MERGE_CIM_PARTS
;
; PURPOSE:
;       Merge multiple parts in a QuickSilver CIM structure into a
;       single part.
;
; CALLING SEQUENCE:
;       MERGE_CIM_PARTS, STR1 [, STR2] , OLDLIST=oldlist , NEWNAME=newname
;                        [, /MATERIAL]
; INPUTS:
;       STR1     - Integer structure array number or IDL CIM structure
;                  containing the parts to be merged. If S2 is not
;                  supplied, the structure is modified to reflect the
;                  requested merge.
; OUTPUTS:
;       STR2     - Integer structure array number or IDL CIM structure
;                  in which the merged CIM output is returned. 
;                  If STR1 is a structure number, STR2 shoudl be a valid
;                    structure array number.
;                  If STR1 is an IDL structure, STR2 must be a variable.
;                  If STR2 does not meet these criteria, then the output 
;                    is in STR1
;  
; KEYWORD PARAMETERS:
;       OLDLIST    Array of integer indices or string part names that
;                  are to be merged into a single part. The array must
;                  have at least two elements.
;       NEWNAME    String containing the name to be used for the new,
;                  merged part. This name must be distinct from any
;                  part name remaining after the merge.
;       MATERIAL   If set, and oldlist specifies both conductor and
;                  material parts to be merged, the merged part will
;                  be considered a material rather than a
;                  conductor. If not set, and both conductor and
;                  material parts are specified, the merged part will
;                  be considered a conductor. If the parts specified
;                  by OLDLIST are either all conductors or all
;                  materials, then the merged part is assumed to be
;                  the same type.
;
; COMMON BLOCKS: none

; EXAMPLE: Given a QS CIM structure "cim3" with two conductors and one
;          material, and whose dataset comment is "CIM ARRAY -
;          cathode=1;;anode=2;;plastic=-1", there are various ways to
;          merge the two conductors (anode and cathode) into a single
;          part (metal) and store the result in a new structure
;          "cim2". Here are two possibilities:

;         MERGE_CIM_PARTS, cim3, cim2, OLDLIST = [1,2], NEWNAME='metal'
;
;         MERGE_CIM_PARTS, cim3, cim2, OLDLIST = ['anode','cathode'],
;                          NEWNAME='metal'
;
; SEE ALSO:
;      ASIS2CIM, COMBINE_CIMS
;
; MODIFICATION HISTORY:
;      Original version, D. B. Seidel, Nov. 4, 2008
;-
;

on_error, 1


npar = n_params()
if npar lt 1 then begin
  print, '% Format:  MERGE_CIM_PARTS, S1, [S2,] OLDLIST=oldlist, NEWNAME=newname'
  return
endif

if (n_elements(oldlist) lt 2) or (n_elements(newname) ne 1) then begin
  print, '% Format:  MERGE_CIM_PARTS, S1, [S2,] OLDLIST=oldlist, NEWNAME=newname'
  print, '  OLDLIST must have two elements, and NEWNAME a single new name'
  return
endif

sout = get_valid_structure(s1, 'MERGE_CIM_PARTS', lcstr = lcstr1, ierr = ier)
if ier ne 0 then begin
  print, '% MERGE_CIM_PARTS: Valid structure must be provided'
  return
endif

type = size(oldlist,/type)
if type eq 2 or type eq 3 then oldint = 1 $
else if type eq 7 then oldint = 0 $
else begin
  print, '% MERGE_CIM_PARTS: OLDLIST must be an integer or string array'
  return
endelse

type = size(newname,/type)
if type ne 7 then begin
  print, '% MERGE_CIM_PARTS: NEWNAME must be string for the new name'
  return
endif

if npar eq 2 then begin
  if lcstr1 then begin
    lcstr2 = (arg_present(s2)) ? 1: -1
  endif else begin
    lcstr2 =0
    s2= s2[0]
    t2 = size(/type, s2)
    if (t2 gt 0) and (t2 lt 4) then begin
      if ((s2 le 0) || (s2 gt get_maxstr())) then s2 = s1[0]
    endif else s2 = s1[0]
  endelse
endif else begin
  lcstr2 = lcstr1
  if lcstr2 then lcstr2 = -1
endelse
 
clab = sout.clab

err = 0
nmrg = n_elements(oldlist)
parse_cim_str,clab,nc,nm,cnames,mnames
if oldint ne 0 then begin
  locs = where(oldlist gt 0,ncm)
  if ncm gt 0 then cmerge = oldlist[locs]
  locs = where(oldlist gt 0,nmm)
  if nmm gt 0 then mmerge = -oldlist[locs]
endif else begin
  cmerge = -1
  mmerge = -1
  for i=0,nmrg-1 do begin
    j = get_match(oldlist[i],cnames) + 1
    if j gt 0 then cmerge = [ cmerge, j ] $
    else begin
      j = get_match(oldlist[i],mnames) + 1
      if j eq 0 then begin
        print,'MERGE_CIM_PARTS: Cannot locate part "',oldlist[i],'"'
        ++err
      endif else mmerge = [mmerge, j]
    endelse
  endfor
  ncm = n_elements(cmerge) - 1
  if ncm gt 0 then cmerge = cmerge[1:ncm]
  nmm = n_elements(mmerge) - 1
  if nmm gt 0 then mmerge = mmerge[1:nmm]
endelse

if err ne 0 then begin
  print, '% MERGE_CIM_PARTS: ',err, ' errors encountered'
  return
endif

print,ncm,nmm

cloc = -1
mloc = -1
oldvals = -1
if ncm gt 0 then begin
  cmerge = cmerge[sort(cmerge)] - 1
  if nmm eq 0 or not keyword_set(material) then cloc = cmerge[0] $
  else oldvals = [oldvals, 2*cmerge[0] + 3]
  for i=1,ncm-1 do oldvals = [oldvals, 2*cmerge[i] + 3]
  print,cmerge
endif
if nmm gt 0 then begin
  mmerge = mmerge[sort(mmerge)] - 1
  if ncm eq 0 or keyword_set(material) then mloc = mmerge[0] $
  else oldvals = [oldvals, 2*mmerge[0] + 2]
  for i=1,nmm-1 do oldvals = [oldvals, 2*mmerge[i] + 2]
  print,mmerge
endif

oldvals = oldvals[1:*]

if cloc ge 0 then newval = 2*cloc + 3
if mloc ge 0 then newval = 2*mloc + 2

k = 0
for i=0,nc-1 do begin
  if i eq cloc then cnames[k++] = newname $
  else if (where(i eq cmerge))[0] eq -1 then cnames[k++] = cnames[i]
endfor
nc = k
if k gt 0 then cnames = cnames[0,k-1]

k = 0
for i=0,nm-1 do begin
  if i eq mloc then mnames[k++] = newname $
  else if (where(i eq mmerge))[0] eq -1 then mnames[k++] = mnames[i]
endfor
nm = k
if k gt 0 then mnames = mnames[0,k-1]
bld_cim_str,nclab,nc,nm,cnames,mnames
print,nclab

print,cloc,mloc,newval
print,oldvals

for i=0,n_elements(oldvals)-1 do begin
  locs = where( oldvals[i] eq sout.v1, cnt)
  if cnt gt 0 then sout.v1[locs] = newval
endfor

sout.clab = nclab

if lcstr2 then begin
  if lcstr2 gt 0 then s2 = temporary(sout) else s1 = temporary(sout)
endif else i2s,sout,s2,/no_copy
 
end
