; ------------------------------------------------------------------------ 
; $Id: readacis.pro,v 1.5 2009/04/17 17:00:37 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
;+
; Read an acis set of parts
;
; Format:
;     READACIS [, filename] [, NPART = npart] [, PART_NAME = partname]
;          [, MATERIAL = material] [, IER =ier] [, /HELPME]
; where:
;     filename  - ACIS assembly file
;     NPART     - Number of parts read
;     PART_NAME - Name of part
;     MATERIAL  - Name of the material
;     IER       - ERROR parameter -- Returns 0 if no problem.
;     HELPME    - If set a hint will be provided
;
; Example:
;     Read the file 'total.asy'
;       READACIS, 'total.asy'
;
; Modifications:
;     1.  Modified file handling and added MXNAME to a common block, LPM, April 17, 2009
;         
;-
 
 pro readacis, p1in, NPART = npart, PART_NAME = parts,  $
           MATERIAL = mats, IER =ier, HELPME = helpme, update = update

      on_error, 1
      common aciscom, initacis, acis_exe
      common acis_parts_com, part_name, material_name, npart_save
      common acis_readparts_com, mxname, oldfilter, oldpath 
;
;     check on the number of parameters
;
      nparams = n_params()
      if keyword_set(helpme)  then begin
print,'% Format: READACIS [, Filename] [, NPART = npart] [, PART_NAME = partname]'
print,'                [, MATERIAL = material] [, IER =ier] [, /HELPME]'
        return
      endif
;
;     check to see if ACIS has been initialized
;
      if (n_elements(initacis) eq 0) then init_acis else $
        if (initacis eq 0) then init_acis
      if initacis eq 0 then begin
        print, '% READACIS: ACIS libraries are not supported by this System'
        return
      endif
;
;     be sure we have a file name
;
    ier =0L
    cd, current = current
    if keyword_set(update) then begin
      filename = make_array(4, /byte, value = 0B)
      directory = current
    endif else begin
      p1 = ( n_elements(p1in) eq 0) ?  '' : strtrim(p1in, 2)
      sep = path_sep()
      if (strlen(p1) lt 1) then begin
        IF (N_ELEMENTS(PATH) EQ 0) then $
          path = (n_elements(oldpath) eq 0) ? current + sep : oldpath
        p1 = dialog_pickfile( path = path, get_path =tempath,  $
          title= 'Select ACIS file to read')
        p1 = strtrim(p1, 2)
        if strlen(p1) lt 1 then return
        if n_elements(tempath) gt 0 then begin
          oldpath = tempath
          directory = tempath
        endif
      endif
      vers = strupcase(strmid(!version.os_family, 0, 3))
      if (vers eq 'UNI') then begin
;     get system variable substitution for unix systems
        temp = p1
        FULFNAM, temp, p1
      endif
      fname = file_expand_path( p1 )
;      if !version.os eq 'linux' then resolve_it, fname
      info = file_info(fname)
      if (info.exists + info.read) ne 2b then begin
         print, '% REASC: File does not exist with read permission'
         print, '         File Name = ', fname
         print, '         File Exists =', info.exists
         print, '         Read Permission =', info.read
         ier = -2
         return
      endif
;     need to switch directories in case reading an assembly
      directory = file_dirname(fname)
      if directory ne current then cd, directory
      filename = [byte(fname), 0B]
    endelse
    NPART = 0L

AGAIN:
    partname = make_array(mxname, /byte)
    material = make_array(mxname, /byte)
      
;
    if initacis eq 1 then begin
; HP
        junk = call_external(ACIS_EXE, 'acis_read', filename, npart, $
          partname, material, mxname, ier )
    endif else if initacis eq 2 then begin
; Sun, Linux
        junk = call_external(ACIS_EXE, 'acis_read', filename, npart, $
          partname, material, mxname, ier )

    endif else if initacis eq 4 then begin
; WIN
       junk = call_external(ACIS_EXE, 'ACIS_READ@8', filename, npart, $
          partname, material, mxname, ier)
    endif else if initacis eq 5 then begin
; MAC
         junk = call_external(ACIS_EXE, 'acis_read', filename, npart, $
          partname, material, mxname, ier)
    endif
    if junk ne 0 then begin
      print,'Error encountered in ACIS_READ, CALL_EXTERNAL: ',junk
      return
    endif
;   If MXNAME is too short for the parts and materials 
;     do an update from the ACIS code
    if ier gt mxname  then begin
;     set MXNAME to a larger value to get the labels
      mxname = (ier / 255L + 1L) * 256L
      filename = make_array(4, /byte, value = 0B)
      goto, AGAIN
    endif

      if directory ne current then cd, current
      if ier ne 0 then begin
        print, '% READACIS: Error reading file: ', strtrim(filename,2)
        print, '  IER = ', strtrim(ier, 2)
        return
      endif
      if npart eq 0 then begin
        parts = ''
        mats = ''
        return
      endif
      parts  = make_array (npart, /string)
      ok =[-1L,  where (partname eq 0B)]
      for i = 0, npart-1 do begin
        if (ok(i+1) eq ok(i) +1) then  parts(i)='' else $
         parts(i) =string( partname(ok(i)+1:ok(i+1)-1))
      endfor
          
      mats = make_array (npart, /string)
      ok =[-1L,  where (material eq 0B)]
      for i = 0, npart-1 do begin
        if (ok(i+1) eq ok(i) +1) then  mats(i)='' else $
         mats(i) =string( material(ok(i)+1:ok(i+1)-1))
      endfor

      new_parts = npart-npart_save
      npart_save = npart
      part_name = parts
      material_name = mats
      if keyword_set(update) then return
      print, new_parts, ' parts read from file: ', fname
      end

