; ------------------------------------------------------------------------ 
; $Id: sliceacis.pro,v 1.3 2009/04/09 20:42:32 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2005 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
;+
; Slice an acis part or set of parts through the point P with a plane with a
;  normal vector of V.
;
; Format:
;     SLICEACIS, str, p1, p2, p3, v1, v2, v3, [, partnum = partnum], $ 
;            xnormal = x_normal, ynormal = y_normal, znormal = z_normal, $
;            clabel = clabel, ierror = ierror
; or 
;     SLICEACIS, str, P, V, [, partnum = partnum], clabel = clabel
; where:
;     str       - Valid Structure array number
;                 If not valid, an empty structure will be provided.
;                 If no empty structures, maxstructure-1 is used.
;     p1, p2, p3 are coordinates of a point in the slice plane
;     v1, v2, v3 are direction cosines for the normal vector of the slice plane
;      P   =     [p1, p2, p3]
;      V   =     [v1, v2, v3]
;
;     Note: If xnormal, ynormal or znormal is set then the V is ignored 
;           and P(0) = p1 or P = P is the only parameter which is used.
;
;     xnormal   - If not zero, the plane is normal to the X axis
;                 V1, V2 and V3 are ignored. (and other normal keywords)
;                 If the number of parameters is less than 3 then
;                   P = [p1, 0, 0] else P = [p1, p2, p3] or P = P
;                   V = [1, 0, 0]
;     ynormal   - If not zero, the plane is normal to the Y axis
;                 V1, V2 and V3 are ignored. (and other normal keywords)
;                 If the number of parameters is less than 3 then
;                   P = [0, p1, 0] else P = [p1, p2, p3] or P = P
;                   V = [0, 1, 0]
;     znormal   - If not zero, the plane is normal to the Z axis
;                 V1, V2 and V3 are ignored. (and other normal keywords)
;                 If the number of parameters is less than 3 then
;                   P = [0, 0, p1] else P = [p1, p2, p3] or P = P
;                   V = [0, 0, 1]
;     partnum   - the acis parts for the slice
;                 If unspecified or zero, all parts will be sliced.
;     clabel    - the comment label for the sliced structure.
;     ierror    - Return error status
;                 0 No error occurred.
;
;
;  Example:
;     Slice all parts normal to the y axis at y = 6.3 and put the results in 
;     structure array, 1.
;       SLICEACIS,1, 6.3, /y
;-
 

 pro sliceacis,strin, p1, p2, p3, p4, p5, p6, partnum = partnumin, $
        xnormal = xnormal, ynormal = ynormal, znormal = znormal, $
        clabel = clab, ierror = ier
;       return on an error
on_error, 1
      common aciscom, initacis, acis_exe
      common acis_parts_com, part_name, material_name, npart_save
      common slice_acis_common, maxloc, maxx, maxxt

@qsstrcom.dat
;
;     check to see if ACIS has been initialized
;
      if (n_elements(initacis) eq 0) then init_acis else $
        if (initacis eq 0) then init_acis
      if initacis eq 0 then begin
        print, '% SLICEACIS: ACIS libraries are not supported by this System'
        return
      endif

      if n_elements(npart_save) eq 0 then npart_save = 0

    if npart_save  eq 0 then begin
      print, '% SLICEACIS: No valid parts to slice'
      return
    endif
;
    ier = -1
    if n_elements (maxloc) eq 0 then begin
       maxloc = 4096L
       maxx  = 16384L
       maxxt = 16384L
    endif
    if n_elements (structinit) eq 0 then qsstruct_init
    npar = n_params()
    if npar eq 0 then begin
 print, '% Format: SLICEACIS, STR, P1, P2, P3, V1, V2, V3, PartNum = partnum,'
      print, '             /Xnormal, /Ynormal, /Znormal, Clabel = clabel '
      print, ' < or >'
 print, '% Format: SLICEACIS, STR, P, V, PartNum = partnum, CLABEL = clabel '
      return
    endif
    if npar lt 2 then begin
      Print, '% SLICEACIS: Two parameters must be provided.'
      return
    endif
;   resolve_routine, get_save_str, /is_function
    if n_elements(strin) gt 0 then str = strin(0)
    savestr = get_save_str(str, (get_maxstr())-1)
    if n_elements(p1) eq 0 then begin
      Print, '% SLICEACIS: One point must be provided'
      return
    endif
    nel = n_elements(partnumin)
    if nel gt 1 then begin
      partnum = long(partnumin(sort(partnumin)))
    endif else if nel eq 1 then begin
      if partnumin eq 0 then partnum = lindgen(npart_save)+1L $
        else partnum = long(partnumin)
    endif else partnum = lindgen(npart_save) +1L
    nelp = n_elements(partnum)
    ok = where (partnum gt 0 and partnum le npart_save, nok)
    if nok eq 0 then begin
      print,  '% SLICEACIS: No valid part numbers, Number of parts:', $
        strtrim( npart_save, 2)
      print, 'Requested parts: ', strtrim(partnum, 2)
      return
    endif
    if nok ne nelp then begin
      partnum = partnum(ok)
      nelp = nok
    endif

; set default values and change only those required
;   P's are the three components of the point 
;   C's are the direction cosines of the normal 
    px =0.0d0
    py =0.0d0
    pz =0.0d0
    cx =0.0d0
    cy =0.0d0
    cz =0.0d0
    lcnorm = 0
    if keyword_set(xnormal) then begin
      lcnorm = 1
      cx = 1.0D0
      if n_elements(p1)ge 3 then begin
        px = double(p1(0))
        py = double(p1(1))
        pz = double(p1(2))
      endif else begin
        px = double(p1(0))
        if npar ge 4 then begin
          if n_elements(p2) gt 0 and N_elements(p3) gt 0 then begin
            py = double(p2(0))
            py = double(p3(0))
          endif
        endif 
      endelse
    endif else if keyword_set(ynormal) then begin
      lcnorm = 1
      cy = 1.0D0
      if n_elements(p1)ge 3 then begin
        px = double(p1(0))
        py = double(p1(1))
        pz = double(p1(2))
      endif else begin
        if npar ge 4 then begin
          if n_elements(p2) gt 0 and N_elements(p3) gt 0 then begin
            px = double(p1(0))
            py = double(p2(0))
            pz = double(p3(0))
          endif else py = double(p1(0))
        endif else py = double(p1(0))
      endelse
    endif else if keyword_set(znormal) then begin
      lcnorm = 1
      cz = 1.0D0
      if n_elements(p1)ge 3 then begin
        px = double(p1(0))
        py = double(p1(1))
        pz = double(p1(2))
      endif else begin
        if npar ge 4 then begin
          if n_elements(p2) gt 0 and N_elements(p3) gt 0 then begin
            px = double(p1(0))
            py = double(p2(0))
            pz = double(p3(0))
          endif else pz = double(p1(0))
        endif else pz = double(p1(0))
      endelse
    endif else if npar lt 7 then begin
        if n_elements(p1) ge 3 and n_elements(p2) ge 3 then begin
            px = double(p1(0))
            py = double(p1(1))
            pz = double(p1(2))
            cx = double(p2(0))
            cy = double(p2(1))
            cz = double(p2(2))
        endif else begin
          print, '% SliceACIS: Seven parameters or a str, a point and a vector'
          print, '              must be provided if a normal is not specified'
          return
        endelse
    endif else begin
      if n_elements(p2)*n_elements(p3)*n_elements(p4)*n_elements(p5)* $
         n_elements(p6) eq 0 then begin
          print, '% SliceACIS: Seven parameters or a str, a point and a vector'
          print, '              must be provided if a normal is not specified'
          return
      endif
      px = double(p1(0))
      py = double(p2(0))
      pz = double(p3(0))
      cx = double(p4(0))
      cy = double(p5(0))
      cz = double(p6(0))
    endelse
    if lcnorm eq 0 then begin
      nor = sqrt(cx*cx + cy*cy + cz*cz)
      cx = cx/nor
      cy = cy/nor
      cz = cz/nor
    endif
; save the slice parameters
    clab = 'Slice, p=('+strtrim(float(px),2)+', '+ strtrim(float(py),2)+ $
      ', '+ strtrim(float(pz),2)+')'
    two = 0
    slice = 0.0
    if cx gt 0.99999 then begin
      two = 1
      slice = px
      clab = clab+', X-Normal'
    endif else if cy gt 0.99999 then begin 
      two = 2
      slice = py
      clab = clab+', Y-Normal'
    endif else if cz gt 0.99999 then begin 
      two = 3
      slice = pz
      clab = clab+', Z-Normal'
    endif else clab = clab +', Normal =('+strtrim(float(cx),2)+', '+ $
      strtrim(float(cy),2)+ ', '+ strtrim(float(cz),2)+')'
    Tlab = 'ACIS Slice'
    clab = clab + ' - Part Numbers: '
    for i = 0, nelp-1 do clab = clab + ' '+strtrim(partnum(i), 2)

     p = make_array(npart_save, /long)
     p (partnum-1) = 1

     locpart = make_array(/long,npart_save+1) 
     maxloct = maxloc
     loc = make_array(maxloc, /long)
     loctmp = make_array(maxloc, /long)
     x = make_array(maxx)
     y = make_array(maxx)
     z = make_array(maxx)
     xtmp = make_array(maxx)
     ytmp = make_array(maxx)
     ztmp = make_array(maxx)
     nloc = 0L
     mx   = 0L
     mloc = 0L
     junk = -1
     for i = 0, npart_save-1 do begin
       if p(i) eq 0 then locpart(i+1) = locpart(i) else begin
       part =  long(i+1)
AGAIN:
;
;    argc - number of arguements
;    argv[0] - Part number
;    argv[1] - Px- First coordinate position
;    argv[2] - Py- Second coordinate position
;    argv[3] - Pz- Third coordinate position
;    argv[4] - Vx- First direction cosine for unit normal vector
;    argv[5] - Vy- Second direction cosine for unit normal vector
;    argv[6] - Vz- Third direction cosine for unit normal vector
;    argv[7] - Loc array - pointer to the first segment of each line segment
;    argv[8] - Xarr, First coordinate points for the line segments
;    argv[9] - Yarr, Second coordinate points for the line segments
;    argv[10]- Zarr, Third coordinate points for the line segments
;    argv[11]- Maximum number of line segments
;    argv[12]- Maximum number of line segment data points
;    argv[13]- Actual number of line segments
;    argv[14]- Actual number of line segment data points
     if initacis eq 1 then begin
; HP
        junk = call_external(ACIS_EXE, 'acis_slice', $
           part, px, py, pz, cx, cy, cz, loctmp, xtmp, ytmp, ztmp, $
           maxloc, maxx, mloc, mx)
     endif else if initacis eq 2 then begin
; Sun
        junk = call_external(ACIS_EXE, 'acis_slice', $
           part, px, py, pz, cx, cy, cz, loctmp, xtmp, ytmp, ztmp, $
           maxloc, maxx, mloc, mx)
     endif else if initacis eq 3 then begin
; VMS
        junk = call_external(ACIS_EXE, 'acis_slice', $
           part, px, py, pz, cx, cy, cz, loctmp, xtmp, ytmp, ztmp, $
           maxloc, maxx, mloc, mx)
     endif else if initacis eq 4 then begin
; WIN
        junk = call_external(ACIS_EXE, 'ACIS_SLICE@8', $
           part, px, py, pz, cx, cy, cz, loctmp, xtmp, ytmp, ztmp, $
           maxloc, maxx, mloc, mx)
     endif else if initacis eq 5 then begin
; MAC
        junk = call_external(ACIS_EXE, 'acis_slice', $
           part, px, py, pz, cx, cy, cz, loctmp, xtmp, ytmp, ztmp, $
           maxloc, maxx, mloc, mx)
     endif

       if ((mx gt maxxt) or (mloc gt maxloct)) then begin
         help, maxloc, maxx, mloc, mx
         if mx gt maxxt then begin
           maxxt = (mx/64 +32L)*64L
           x = make_array(maxxt)
           y = make_array(maxxt)
           z = make_array(maxxt)
         endif
         if mloc gt maxloct then begin
           maxloct  = (maxloct/64 +32L)*64L
           loctmp = make_array(maxloct, /long)
         endif
         if junk eq i then begin
          print, ' this is not correct', i
          return
         endif
         junk = i
         goto, AGAIN
       endif
       junk = i
       if maxx  lt mx +loc(nloc) then begin
         maxx = maxx+maxx
         x = [x,x]
         y = [y,y]
         z = [z,z]
       endif
       if maxloc lt nloc+mloc  then begin
         maxloc = maxloc+ maxloc
         loc = [loc, loc]
       endif
       locpart(i+1) = locpart(i)+ mloc
       if mloc gt 0 then begin
         loc(nloc+1:nloc+mloc) = loc(nloc)+loctmp(0:mloc-1)
         x(loc(nloc): loc(nloc+mloc)-1) =xtmp(0:mx-1)
         y(loc(nloc): loc(nloc+mloc)-1) =ytmp(0:mx-1)
         z(loc(nloc): loc(nloc+mloc)-1) =ztmp(0:mx-1)
         nloc = nloc+mloc
       endif
       endelse
     endfor
     if nloc eq 0 then begin
       print, '% SliceACIS: No intersections for plane -'
       print, '            ',clab 
       return
     endif

     x = x(0:loc(nloc)-1)
     y = y(0:loc(nloc)-1)
     z = z(0:loc(nloc)-1)
     loc = loc(0:nloc)
     
     nn = locpart(npart_save)
     loc = loc(0:nn)
     range = make_array(2, 3)
     range(0, 0) = min(x, max = mm)
     range(1, 0) =  mm
     range(0, 1) = min(y, max = mm)
     range(1, 1) =  mm
     range(0, 2) = min(z, max = mm)
     range(1, 2) =  mm
 
     a = {type:7,ndim:3L, part:partnum,clab:clab, tlab:tlab, two:two, $
         npart:npart_save, slice:slice, point: [px, py, pz], $
         Normal:[cx, cy, cz], x1lab:'X axis', $
         x2lab:'Y axis', x3lab: 'Z axis', x1:x, x2:y, x3:z, $
         locx: loc, locp:locpart, range:range }
 i2s, a, savestr, /no_copy
 ier = 0
 end 
