; ------------------------------------------------------------------------ 
; $Id: voxels.pro,v 1.3 2009/02/17 18:19:07 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2005 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
 pro voxels,hx, fx, hy, fy, hz, fz, parts = partsin, fill_value = fillin, $
         structure  = str, clabel = clabel, xlabel = xlabel, ylabel = ylabel, $
         zlabel = zlabel, helpme = helpme, $
         cimonly = cimonly, coordsys = coordsys
;+
; ; NAME:
;       Voxels 
;
; PURPOSE:
;       Convert an ACIS solid model to a CIM array for QuickSilver.
;
; CALLING SEQUENCE:
;       VOXELS [, HX, FX, HY, FY, HZ, FZ], /HELPME
; INPUTS:
;       None Required  - If any of the optional parameters are missing, values
;                        from the QSGRID routine will be used.  These values 
;                        are padded by one cell using a mirror of the nearest 
;                        neighbor.
;  
; KEYWORD PARAMETERS:
;       PARTS      Part numbers of parts to be included in the CIM array
;                  Default: All parts
;       FILL_VALUE Integer value to be put into the CIM array. Number of fill
;                  values should equal the number of parts.  Default = part no
;       STRUCTURE  Structure array number to store the CIM array.
;                  Default = MaxStructure if keyword is undefined
;          else    Default = If value not valid, then structure is the lowest
;                  unused structure or MaxStructure if all structures
;                  are used.  Value will be returned in structure.
;          That is, unless a cimonly is set then the CIM will be stored.
;                  If structure is omitted, then structure = maxstructure
;                  If structure is a valid structure value then structure is used
;                  If structure number is not valid (for example <= 0) then 
;                     CIM is stored in structure = unused structure sturcture.
;                     (If all structures are in use, structure = MaxStructure)
;       CLABEL     Dataset comment, Default is 'CIM matrix'
;       XLABEL     First spatial dimension label, Default is 'Xi'
;       YLABEL     Second spatial dimension label, Default is 'Xj'
;       ZLABEL     Third spatial dimension label, Default is 'Xk'
;       COORDSYS   Coordinate system. Default is cartesian (0).
;                  {0, 1, 2} = {cartesian, cylindrical, spherical}
;       CIMONLY    If set, return the cim in structure.
;       HELPME     If set, help message will be printed.
;
; COMMON BLOCKS:
;       PFIDLTQ
;       ACISCOM
;       ACIS_PARTS_COM
;
; RESTRICTIONS:
;       Full and half cell values must be provided either with QSGRID or 
;       specified in the call
; EXAMPLES:
;       Grid parts 2, 3, 4, 7 and 9 using values from QSGRID.
;       Put the values [1, 0, 2, 3, 4] into the CIM array and store in 6
;          VOXELS, part = [2, 3, 4, 7, 9], str = 6, fill = [1, 0, 2, 3, 4]
;
; MODIFICATION HISTORY:
;      Original version, L. P. Mix, March 26, 1997
;-
;
on_error, 1

@pfidltq.com
      common aciscom, initacis, acis_exe
      common acis_parts_com, part_name, material_name, npart_save

@qsstrcom.dat
;
      if (n_elements (npart_save) eq 0) then npart_save = 0
      if npart_save eq 0 then begin
        print, '% VOXELS: No valid acis saved '
        return
      endif

;
;     check to see if ACIS has been initialized
;
      if (n_elements(initacis) eq 0) then init_acis else $
        if (initacis eq 0) then init_acis
      if initacis eq 0 then begin
        print, '% VOXELS: ACIS libraries are not supported by this System'
        return
      endif

    if n_elements (structinit) eq 0 then qsstruct_init
    npar = n_params()
    if keyword_set(helpme) then begin
 print, '% Format: VOXELS [, HX, FX, HY, FY, HZ, FZ] [, /HELPME]'
 print, '          [, PARTS = parts] [, FILL_VALUE = fill_value]'
 print, '          [, STRUCTURE = structure] [, CLABEL = clabel]'
 print, '          [, COORDSYS = coordsys ] [, /CIMonly ]'
      return
    endif


nx = n_elements(hx)
ny = n_elements(hy)
nz = n_elements(hz)
if nx*ny*nz gt 0 then begin
  if n_elements(fx) ne nx+1 or n_elements(fy) ne ny+1 $
     or n_elements(fz) ne nz+1 then begin
     print, '%VOXELS: Full and half grid values are not consistent'
     return
  endif
 
endif else begin
  nx = n_elements(gxhalf)
  ny = n_elements(gyhalf)
  nz = n_elements(gzhalf)
  if nx*ny*nz eq 0 then begin
    print, '%VOXELS: Full and half grid values are not defined in QSGRID'
    return
  endif
  if n_elements(gxfull) ne nx+1 or n_elements(gyfull) ne ny+1 $
     or n_elements(gzfull) ne nz+1 then begin
     print, '%VOXELS: Full and half grid values are not consistent in QSGRID'
     return
  endif
; pad the cells using simple reflection of the last cell an half cell
  hx = [gxfull(0)-gxhalf(0)+gxfull(0),gxhalf,gxfull(nx)-gxhalf(nx-1)+gxfull(nx)]
  hy = [gyfull(0)-gyhalf(0)+gyfull(0),gyhalf,gyfull(ny)-gyhalf(ny-1)+gyfull(ny)]
  hz = [gzfull(0)-gzhalf(0)+gzfull(0),gzhalf,gzfull(nz)-gzhalf(nz-1)+gzfull(nz)]

  fx = [gxfull(0)-gxfull(1)+gxfull(0),gxfull,gxfull(nx)-gxfull(nx-1)+gxfull(nx)]
  fy = [gyfull(0)-gyfull(1)+gyfull(0),gyfull,gyfull(ny)-gyfull(ny-1)+gyfull(ny)]
  fz = [gzfull(0)-gzfull(1)+gzfull(0),gzfull,gzfull(nz)-gzfull(nz-1)+gzfull(nz)]

  nx = nx+2L
  ny = ny+2L
  nz = nz+2L
  

;    argc - number of arguements
; Following are pointers to:
;        
;    argv[0] - Coordsystem 
;    enum coordsys {cartesian, cylindrical, spherical}; {0,1,2}
;    argv[1] - Nx- Number of half grid points in X
;    argv[2] - Ny- Number of half grid points in Y
;    argv[3] - Nz- Number of half grid points in Z
;    argv[4] - Hx- Half grid values in X
;    argv[5] - Hy- Half grid values in Y
;    argv[6] - Hz- Half grid values in Z
;    argv[7] - FX - Full grid values in X (nx+1 points)
;    argv[8] - FY - Full grid values in Y (ny+1 points)
;    argv[9] - FZ - Full grid values in Z (nz+1 points)
;    argv[10]- Number of parts in list
;    argv[11]- Parts list for voxels
;    argv[12]- Number of fill values
;    argv[13]- Fill values for the parts (Default is the part number)
;    argv[14]- CIM array (Nx x Ny x Nz)

endelse

 cim = make_array(nx, ny, nz, /long, val=0L)
 nparts = n_elements(partsin)
 if nparts eq 0 then begin
   parts = 1+lindgen(npart_save) 
   nparts = npart_save
 endif else begin
   parts = long(partsin)
   ok = where (parts gt 0 and parts le npart_save, nparts)
   if nparts eq 0 then begin
     print, '% VOXELS: No valid parts specified'
     ahelp
     return
   endif else parts = parts(ok)
 endelse

   
 nfill = n_elements(fillin)
 if nfill eq 0 then fill = parts else begin
   fill = long(fillin)
   if nfill lt nparts then fill = [fill, parts(nfill:*)]
 endelse
 nfill = nparts
 
if n_elements(coordsys) eq 0 then coord = 0L else  $
 coord = long( (coordsys > 0L) < 2L)

; help,coord, nx,ny,nz,hx,hy,hz,fx,fy,fz,nparts,parts,nfill,fill,cim
;
      if initacis eq 1 then begin
; HP
        junk = call_external(ACIS_EXE, 'acis_2_cim', coord, nx, ny, nz, $
           hx, hy, hz, fx, fy, fz, nparts, parts, nfill, fill, cim)
      endif else if initacis eq 2 then begin
; Sun
        junk = call_external(ACIS_EXE, 'acis_2_cim', coord, nx, ny, nz, $
           hx, hy, hz, fx, fy, fz, nparts, parts, nfill, fill, cim)
      endif else if initacis eq 3 then begin
 VMS;
        junk = call_external(ACIS_EXE, 'acis_2_cim', coord, nx, ny, nz, $
           hx, hy, hz, fx, fy, fz, nparts, parts, nfill, fill, cim)
      endif else if initacis eq 4 then begin
; WIN
       junk = call_external(ACIS_EXE, 'ACIS_2_CIM@8', coord, nx, ny, nz, $
           hx, hy, hz, fx, fy, fz, nparts, parts, nfill, fill, cim)
      endif else if initacis eq 5 then begin
; MAC
       junk = call_external(ACIS_EXE, 'acis_2_cim', coord, nx, ny, nz, $
           hx, hy, hz, fx, fy, fz, nparts, parts, nfill, fill, cim)
      endif

if junk ne 0 then begin
  print,'Error encountered in ACIS_2_CIM: '+ strtrim(junk, 2)
  return
endif

if keyword_set(cimonly) then begin
  str = cim
  return
endif
if n_elements(str) eq 0 then str = maxstructure else $
  str = get_save_str(str, maxstructure)
if n_elements(clabel) eq 0 then clabel = 'CIM matrix' else  $
  clabel= strtrim(clabel,2)
if n_elements(xlabel) eq 0 then xlabel = 'Xi' else xlabel= strtrim(xlabel,2)
if n_elements(ylabel) eq 0 then ylabel = 'Xj' else ylabel= strtrim(ylabel,2)
if n_elements(zlabel) eq 0 then zlabel = 'Xk' else zlabel= strtrim(zlabel,2)
arr2str, str, cim, hx,hy, hz, clab=clabel, xlab=xlabel, ylab=ylabel, $
 zlab=zlabel, spare=[coord+1,-3,-3,-3,-3]

end
