; ------------------------------------------------------------------------
; $Id: get_anim_wdf.pro,v 1.2 2007/06/06 19:39:21 lpmix Exp $
; ------------------------------------------------------------------------
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
;+
; NAME:
;       GET_ANIM_WDF
;
; PURPOSE:
;       This function returns the initial WDF array number for each waveform
;         stored by MAKE_ANIM
;       The times and the number of times may also be returned.
;       See also MAKE_ANIM, GET_ANIM_WDF
;
; CALLING SEQUENCE:
;       WDF = GET_ANIM_WDF(PARAMETER, NTIMES = ntimes, TIMES = times)
;
; INPUTS:
;       PARAMETER: One of the 20 parameters specified under OUTPUTS.
;               GET_MATCH is used to search for these searches.
;               The search strings must provide only one match.
;               Examples include: '^press', '^o*a$', '^te', 'zbar'
;               If no parameter is specified, but keywords are present,
;                 -1 is returned.
;               If no parameters and no keywords are present or some error
;                 occurs, 0 (zero) is returned.
;
; KEYWORD PARAMETERS:
;       NTIME:  Number of time steps
;       TIMES:  Value of the times
;
; OUTPUTS:
;       The number of times and the time values are returned as keywords.
;       The following WDF array initial values are returned by the function:
;          Density
;          Temperature
;          Rad-Temp
;          Pressure
;          BTheta
;          Mag Press = Btheta * Btheta/ (2*mu)
;          J_r
;          J_z
;          R_Velocity
;          Z_Velocity
;          R_Acceleration
;          Z_Acceleration
;          E_Cond
;          Ther_Cond
;          Opacity_A
;          Opacity_R
;          ZBar
;          Sound Sp
;          Art. Viscosity
;          Spec Heat
;
; EXAMPLE:
;       Plot all density curves in a spectrum plot.
;         plo, GET_ANIM_WDF('density', ntime= nt), nt, /spectrum
;
;       Plot one quarter of the specific heat curves
;         plo, GET_ANIM_WDF('^spec', ntime= nt) +indgen(nt/4)*4, /spectrum
;
;       Make an animation of jz which runs at about 3 frames/sec
;         st =  GET_ANIM_WDF('j_z', ntime= nt)
;         for i = st, st+nt-1 do begin & plo, i, ov=0 & wait, 0.2 & end
;       Note to make it continuous: (Use Control-C to end)
;     START: for i = st, st+nt-1 do begin & plo, i, ov=0 & wait, 0.2 & end & goto, start
;
; MODIFICATION HISTORY:
;       Written by:     L. P. Mix, Jr., July 19, 2005
;-

 function get_anim_wdf, value, ntimes = ntimes, times = times
@ make_anim_common.dat
 on_error, 1
 if n_params() eq 0 then begin
   if arg_present(ntimes) or arg_present(times) then begin
     ntimes = ntime
     if arg_present(times) then times = times_saved
     return, -1
   endif
   print, '% Format: WDF = GET_ANIM_WDF(VAR_STRING, ntime = ntime, times= times)'
   return, 0
 endif
 if n_elements (value) eq 0 then begin
   print, '% GET_ANIM_WDF: Variable name must be provided'
   return, 0
 endif
 i = get_match(value, labels, reg = 0, nel = nel)
 if nel eq 1 then return, var_arr[i[0]] else begin
   print, '% GET_ANIM_WDF: Search string must produce one value'
   print, '                Values Returned: ', strtrim (nel, 2)
   print, '                Search String : ', value
   return, 0
 endelse
 end
