; ------------------------------------------------------------------------ 
; $Id: get_cdf.pro,v 1.3 2007/06/06 19:39:21 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
 Function get_cdf, str, fileid = fid, maxdset = maxdset, wdfarray= wdfarray

;+
; Name: GET_CDF
; 
; PURPOSE:
;       Get CDF dataset numbers for all dataset comments containing
;          a specified string. 
; Format:
;       Map = Get_CDF(STRING [, FILEID = fileid] [, MAXDSET = maxdset]
;                      [, /WDFARRAY])
; Parameters:
;       MAP   :   Array of datasets with the desired string.
;                 -1 if no datasets found.
;       STRING:   Search string 
;                 If STRING = '' or ' ' or '*' then 
;                     Map = 1+lindgen(ndspff(fid))
;                     (Maxdset is ignored)
;                 The following special characters can be used:
;                 *  - Wild card indicating 0 to n characters
;                      To use a * in a search string use "\*"
;                 ?  - Wild card indicating 1 and only 1 characters
;                      To use a ? in a search string use "\?"
;                 ^  - If this is the first character, indicates that
;                      the string begins the dataset comment.
;                      All other times a ^ is taken literally.
;                      To begin a search string with a literal ^ use:
;                          "*^" or "\^"
;                      All search strings are assumed to begin with "*" unless
;                      a "^" is present. 
;                 $  - If this is the last character, indicates that the
;                      string ends the non-blank portion of the dataset comment.
;                      All other times a $ is taken literally.
;                      To end a search string with a literal $ use:
;                          "*$" or "\$"
;                      All search strings are assumed to end with "*" unless
;                      a "$" is present.
; OPTIONAL INPUT:
;       FILEID  : File id - Default is 0 (The current CDF file) 
;       WDF     : If set waveform arrays (global arrays) will be searched.
;                 Default are the nodal and element (cell) arrays.
;       MAXDSET : Maximum number of files desired.
;                 If more than MAXDSET files are found, then an additional
;                 search is performed over the returned datasets with the
;                 search string modified to force an exact match; ie.
;                 if not present a "^" will be added to the front of the 
;                 string and a "$" to the end of the string.
;  EXAMPLES: 
;     Find all the datasets with VDIODE in the comment.
;              id = get_cdf('vdiode')
;     Find all the datasets with the non_blank characters only VDIODE
;              id = get_cdf('^vdiode$')
;     Find all the datasets with the string VDIODE and at least one 
;       more character at the end
;              id = get_cdf('vdiode?')
;     Read all the datasets with 'pho' into structures beginning with 10
;              id =  get_cdf()
;              n = n_elements(id)
;              for i = 0, n-1 do readcdf, i+10, id(i)
;     NOTE: For WDF arrays, recdf, 1, 'string', /al will read all valid
;            arrays with 'string' in one operation.
;  
;-
; add common block for netCDF
;
@netcdfcom.dat
;
 on_error, 1
;     check to see if cdf has been initialized
;
      if n_elements(initcdf) eq 0 then begin
        print, '% GET_CDF: No netCDF files are open'
        return, 0
      endif
;
;     check on the number of parameters
;
      nparams = long( n_params())
      if nparams eq 0 then begin
        print, 'Format: ID = GET_CDF(search_string,  [, FILEID = fileid]'
        print, '       [, MAXDSET = maxdset] [, /WDFARRAY])'
        return, 0
      endif
;
      if n_elements (fid) eq 0 then fid = array_cdfid(cdf_id, 0)

      if array_cdfid(cdf_id, 0) eq fid then begin
        if lcdfvalid ne array_cdfid(cdf_id, 1) then load_cdf_data
      endif else begin
        ok = where (fid eq array_cdfid(*, 0), nok)
        if nok ne 1 then begin
          print,'% GET_CDF: ', strtrim(nok,2), ' netCDF files with a fid of ', $
            strtrim(fid, 2)
          print, '         No Dataset found'
          return, -1
        endif
        cdf_id = ok(0)
        if lcdfvalid ne array_cdfid(cdf_id, 1) then load_cdf_data
      endelse

      if keyword_set(wdfarray) then begin
        if num_glo_var(cdf_id) eq 0 then return, -1
      endif else begin
        tot1 = num_nod_var(cdf_id) 
        tot2 = num_elem_var(cdf_id)
        tot = tot1 +tot2
        if tot eq 0 then return, -1
      endelse
      if n_elements(str) eq 0 then str = '' else str = strtrim(str(0),2)
      slen = strlen (str)
      if ((slen eq 0) or ((slen eq 1) and (strmid(str,0,1) eq '*'))) then begin
        if keyword_set(wdfarray) then return, 1+lindgen(num_glo_var(cdf_id))
        return, 1+lindgen(tot)
      endif 
      if n_elements (maxdset) eq 0 then begin
        if keyword_set(wdfarray) then maxdset = num_glo_var(cdf_id) else $
          maxdset = tot
      endif else maxdset = maxdset 
      if keyword_set(wdfarray) then names = name_glo_var else begin
        if tot1*tot2 ne 0 then names=[name_nod_var, name_elem_var] else begin
          if tot1 eq 0 then names = name_elem_var else names = name_nod_var
        endelse
      endelse
      get_ncdf_string, str, names, map, nmap, maxdset, ier, lcstring
      if ier ne 0 then begin
       print,'% GET_CDF: Valid datasets not found for string: ', str
       print,'           Please change the dataset specification'
       return, -1
      endif
      return, map
      end
