; ------------------------------------------------------------------------
; $Id$
; ------------------------------------------------------------------------
;
; Copyright 2005 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
;+
; NAME:
;       OneD
;
; PURPOSE:
;       Add triangles to a cell array from a 1-D Alegra run with 
;       Time is the vertical axis.
;       If data is node data, there should be two unique Y values.
;       Top and bottom data values will be averaged.
;
; CALLING SEQUENCE:
;       OneD, STRIN [, STROUT]
;
;   Where:
;       STRIN  Structure array number or structure of the input data
;       STROUT Structure array number to save data
;
; RESTRICTIONS:
;       Input structure should be one dimensional cell data
;
; PROCEDURE:
;       Sort the data in X and provide trianglese for 
;       a new structure.
;
; EXAMPLE:
;       Convert structure 2 to a connected structure in 3
;       OneD, 2, 3
;
; MODIFICATION HISTORY:
; 	Written by:     L. P. Mix, September 20, 2004
;-

pro OneD, strin, strout
on_error, 2
;
  if n_elements (structinit) eq 0 then qsstruct_init
;

If n_elements (strin) eq 0 then begin
  print, '% OneD: Structure or structure array number must be provided'
  return
endif

str = get_valid_structure(strin, 'OneD', lcstr= lcstr, ierror = ier)
if ier ne 0 then return
x1 = str.x1
x2 = str.x2
v1 = str.v1
ss = size(x1)
if ss[0] ne 2 then begin
  print, '% OneD: Size of X is not appropriate'
  print, ss
  return
endif
 
if str.type eq 8 then begin
  nx = ss[1]
  ny = ss[2]

  ix = sort(x1[*, 0])
  ixt = sort(x1[*, ny-1])
  mm= min (ix -ixt, max= mmm)
  if mm ne 0 or mmm ne 0 then begin
    print, '% OneD: Top and bottom of X array are not in same order'
    return
  endif

  x1 = x1[ix, *]
  x2 = x2[ix, *]
  v1 = v1[ix, *]
  range = str.range
  tri = make_triangles(0, nx, ny)
  ss = size(tri)
  loctri=[0, ss[2]]
endif else begin   
  y0 = x2[*, 0]
  a = sort(y0)
  jj = uniq(y0, a)
  if n_elements(jj) ne 2 then begin
    print, '% OneD: Structure must be a type 8 structure or a type 9'
    print, '            structure with two unique Y values.'
    print, y0[jj]
    return
  endif
  nt = str.ntime
  y = temporary(x2)
  x2 = str.x4
  ok_top = where(y0 eq y0[jj[0]], complement = ok_bot, ncomplement =nokc, nok)
  if nok ne nokc then begin
    print, '% OneD: Unequal number of top and bottom elements'
    help, y0
    print, nok, nokc
    return
  endif
  xx_top = (x1[*, 0])[ok_top]
  xx_bot = (x1[*, 0])[ok_bot]
  atop = sort(xx_top)
  abot = sort(xx_bot)
  ok_top = ok_top[atop]
  ok_bot = ok_bot[abot]
  x2 = make_array(nok, value=1.0)#x2
  x1 = (x1[ok_top, *]+x1[ok_bot, *])*0.5
  v1 = (v1[ok_top, *]+v1[ok_bot, *])*0.5
  tri = make_triangles(0, nok, nt)
  ss = size(tri)
  loctri=[0, ss[2]]
  ny = nt
  nx = nok
  range = make_array(2, 2, 2)
  range(0, 0, *) = min(x1, max = mm)
  range(1, 0, *) = mm
  range(0, 1, *) = str.x4[0]
  range(1, 1, *) = str.x4[nt-1]
  
endelse
locx = make_array(2, 2, /long)
locx[*, 1]= [nx, ny]
locv =[0l, nx*ny]

str = {type: 8, ndim: 2, nvect: 1L, block:1L, two: 3, $
     ntime:1,  slice: 0.0, size: [nx, ny], range: range, $
     order: 0L, locx:locx, locv: locv, triangles: tri, $
     loctri: loctri, x1:x1, x2:x2, x3:0.0, x4:0.0, $
     v1:v1, v2:0.0, v3:0.0, v4:0.0, v5:0.0, v6:0.0, x1lab : str.x1lab, $
     x2lab : 'Time', x3lab : '', x4lab : '', $
     v1lab : 'Amplitude' , v2lab : '', v3lab : '', v4lab : '',v5lab : '', $
     v6lab : '', file : str.file, spare : str.spare, tlab : str.tlab, $
              clab:str.clab}

if N_params() eq 1 then begin
  if lcstr eq 0 then $ 
   i2s, str, strin, /no_copy else strin = temporary(str)
endif else begin
  if lcstr eq 0 then begin
    strout = get_save_str(strout, strin)
    i2s, str, strout, /no_copy
  endif else strout = temporary(str)
endelse

END
