; ------------------------------------------------------------------------ 
; $Id: opencdf.pro,v 1.7 2008/02/27 22:49:24 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
  pro opencdf, p1, p2,  path = path, filter = filter, noload = noload,  $
                fileid = fileid, ndset = ndset, help = help, quiet = quiet, $
                write = write, verbose = verbose, ierror = ierror
;+
;  Open a netCDF file.  The current file pointer is set to this file id.
;  format: OPENCDF [, File] [, Fileid ] [, NDSET =ndset] [, NOLOAD = noload]
;                  [, WRITE = write] [, IERROR = ierror]
;                  [, PATH = path] [, FILTER = filter] [, HELP = help]
;                  [, FILEID = fileid] [, QUIET = quiet] [, /VERBOSE]
;  where:
;    File   - name of the PFF file to be opened.  The default file extension
;             is ".exo". 
;             If file is omitted or undefined, PICKFILE will be used to select
;             a file. The initial path in PICKFILE is the current directory or
;             the PATH to the last file selected.
;             NOTE: PATH and FILTER may be used to set the initial path and
;                   filter for PICKFILE.
;             NOTE: FULFNAM procedure is used to handle environment variables
;                   See FULFNAM for details
;    Fileid - user specified file id 
;             (If fileid is specified, it will be used as the file id.  If 
;             fileid is not provided or is zero, a value will be 
;             provided; default value is the first unused integer greater
;             than 0.  If fileid is provided but has been used for another 
;             file the program will inform the user and terminate.
;    WRITE  - Set to open the file for both read and write
;    help   - Print a help text
;    PATH   - Path to the current file 
;             Default is the current directory in the first call to OPENCDF/pff
;             but the default path is changed to the final path selected is 
;             retained as the default in the next call to opencdf/openpff.
;    FILTER - File filter - The default may be set in USERQSINIT; if undefined
;             the default is '*'.  The default filter is set only with
;              USERQSINIT or SETDEFAULT.
;    NOLOAD - Do not read data into a data common block
;             This will save time in opening a file if the open command is not
;             followed by a read command.
;    NDSET  - Number of datasets 
;             For Exodus II: Array of [# of global, # of node, # of cell]
;             For WDF array: Number of WDF arrays
;    QUIET  - Only print error messages
;    VERBOSE- Print file information
;    IERROR - Error flag.  Returned zero if no problems encountered.
;
;  Examples:
;    Open a CDF file with the default file id
;      openpff
;    Open a CDF file, 'users/test', with the default file id
;      openpff, '/users/test'
;    Open a file to be selected from a menu with a file id of 14
;      openCDF, ' ', 14
;    Get help
;      openCDF, /h
;-
;
      on_error, 1
@netcdfcom.dat
      COMMON savecdfpicker, oldfilter, oldpath

;     common netcdfcom, initcdf, maxcdffile, cdfid, array_cdfid, $
;      cdftype, num_his, num_glo_var, num_nod_var, num_elem_var
;     common netcdfdatacom lcdfvalid, coord, dispx, dispy, time_whole, $
;       connect, name_glo_var, name_nod_var, name_elem_var

      if n_elements(initcdf) eq 0 then init_cdf 
      ierror = 1
;
;     check on the number of parameters
;
      nparams = n_params()
      if nparams eq 0 and keyword_set(help) then begin
print, '% Format: OPENCDF [, File] [, Fileid ] [, /NoLoad] [, IERROR = ierror]'
print, '                  [, PATH = path] [, FILTER = filter]'
print, '                  [, NDSET = ndset] [, FILEID = fileid]'
print, '                  [, QUIET = quiet] [, VERBOSE = verbose]'
        ierror =0
        return
      endif
;
;     check to see if pff has been initialized
;
      if (nparams eq 0) then begin
        IF (N_ELEMENTS(PATH) EQ 0 and n_elements(oldpath) eq 0) THEN BEGIN
          cd, current = dir
          if initcdf eq  1 then path = dir+'/' else path = dir
        endif else if n_elements(path) eq 0 then path = oldpath
        if n_elements(filter) eq 0 and n_elements(oldfilter) eq 0 then $
          filter = '*.exo' else if n_elements(filter) eq 0 then filter = oldfilter
        p1 = pickfile(/read, path = path, get_path = oldpath, filter = filter)
        if strlen(p1) lt 1 then return
        nparams = 1
      endif else if n_elements(p1) eq 0 then begin
        IF (N_ELEMENTS(PATH) EQ 0 and n_elements(oldpath) eq 0) THEN BEGIN
          cd, current = dir
          if initcdf eq 1 then path = dir+'/' else path = dir
        endif else if n_elements(path) eq 0 then path = oldpath
        if n_elements(filter) eq 0 and n_elements(oldfilter) eq 0 then $
          filter = '*' else if n_elements(filter) eq 0 then filter = oldfilter
        p1 = pickfile(/read, path = path, get_path = oldpath, filter = filter)
        if strlen(p1) lt 1 then return
      endif else begin
        p1 = strtrim(p1, 2)
        if strlen(p1) lt 1 then begin
          IF (N_ELEMENTS(PATH) EQ 0 and n_elements(oldpath) eq 0) THEN BEGIN
            cd, current = dir
            if initcdf eq 1 then path = dir+'/' else path = dir
          endif else if n_elements(path) eq 0 then path = oldpath
          if n_elements(filter) eq 0 and n_elements(oldfilter) eq 0 then $
            filter = '*' else if n_elements(filter) eq 0 then filter = oldfilter
          p1 = pickfile(/read, path = path, get_path = oldpath, filter = filter)
          if strlen(p1) lt 1 then return
        endif
      endelse
;
; user specified file id
      if n_elements (p2) eq 0 then begin
        if n_elements(fileid) gt 0 then p2 = long(fileid) else p2 = 0L
      endif else p2 = long(p2)
      if initcdf eq 1 then begin
;
;       get variable substitution path name for unix systems
;
        temp = p1
        FULFNAM, temp, p1
      endif
      p1 = file_expand_path(p1)
      if !version.os eq 'linux' then resolve_it, p1
      NDSET = 0L
      oldid = cdf_id
     
      cdf_id = ncdf_open(p1, write = write)
      if cdf_id lt 0  then begin
        print, '% OPENCDF: Invalid file: ', p1
        cdf_id = oldid
        return
      endif
      fid = cdf_id
      save_file_id
      
      if cdf_id lt 0 or cdf_id ge max_cdffile then begin
        ncdf_close, cdf_id
        print, '% OPENCDF: Unable to open file, Too many files open', cdf_id
        cdf_id = oldid
        return
      endif
      if p2 eq 0 then begin
        p2 = 1
        ok = where (array_cdfid(*, 0) eq p2 and array_cdfid(*,1) ne 0 , nok)
        while nok gt 0 do begin
          p2 = p2+1
          ok = where (array_cdfid(*, 0) eq p2 and array_cdfid(*,1) ne 0, nok)
        endwhile
      endif else begin
        ok = where (array_cdfid(*, 0) eq p2 and array_cdfid(*,1) ne 0, nok)
        if nok gt 0 then begin
          print, '% OPENCDF: Unable to open file with that fid.'
          print, '        Either close other file or choose anothr fid'
          ncdf_close, fid
          fcdf_id(cdf_id) = -1
          cdf_id = oldid
          return
        endif
      endelse
      array_cdfid(cdf_id, 0) = p2
      array_cdfid(cdf_id, 1) = long((systime(1)*10) mod 65536)
      if keyword_set(write) then array_cdfid(cdf_id, 2) = 1 else $
        array_cdfid(cdf_id, 2) = 0
;     cannot allow identifier to be zero.
      if  array_cdfid(cdf_id, 1) eq 0 then  array_cdfid(cdf_id, 1) = 1
      file_name(cdf_id) = p1
      glob = ncdf_inquire(fid)
      nattr = glob.ngatts
      attr = make_array(nattr, /string)
      for i = 0, nattr-1 do attr(i) = ncdf_attname(fid, i, /global)
      attr = strlowcase(attr)
      ok = where(attr eq 'title', nok)
      if nok eq  1 then begin
        ncdf_attget, fid, attr(ok(0)), title, /global
        title = strtrim(title, 2)
        file_title(cdf_id) = title
      endif else begin
        if nok eq 0 then $
          print, '% OpenCDF: No Title found in the attributes'
        Print, '% OpenCDF: Title set to NULL'
        print, attr
        title = ''
        file_title(cdf_id) = ''
      endelse
      ok = where(attr eq 'file_size', nok)
      if nok eq 0 then API_VERSION[cdf_id] =0.0 else begin
        ncdf_attget,  fid, 'file_size', value, /global
        api_version[cdf_id] = value
      endelse
      ok = where(attr eq 'pfidl_type', nok)
      if nok eq 0 then begin
        cdf_type(cdf_id) = 1
        type = 1
      endif else begin  
        ncdf_attget, fid, attr(ok(0)), type, /global
        cdf_type(cdf_id) = type
      endelse
      if keyword_set(quiet) then verbose = 0
      dimname = make_array(glob.ndims, /string)
      dimvalue = make_array(glob.ndims, /long)
;     get the names of the dimensional variables
      for i = 0, glob.ndims-1  do begin
        ncdf_diminq, fid, i, name, dimval
        dimname(i) = strlowcase(strtrim(name, 2))
        dimvalue(i) = dimval
      endfor
      if type eq 1 then begin
;       Exodus II
;
        ndset = make_array(3, /long)
        if not keyword_set(quiet) then print, $
          'Exodus II File: ', strtrim(file_name(cdf_id), 2), ' opened with ', $
           'ID = ', Strtrim(p2, 2)
        if keyword_set(verbose) then print, ' Title = ', title

        ok = where(dimname eq  'num_glo_var', nok)
        if nok eq 0 then num_glo_var(cdf_id) = 0 else begin
          num_glo_var(cdf_id) = dimvalue(ok(0))
          ndset(0) = dimvalue(ok(0))
          if keyword_set(verbose) then print, ' Number of Global Variables = ', $
            strtrim(dimvalue(ok(0)), 2)
        endelse

        ok = where(dimname eq 'num_elem_var', nok)
        if nok eq 0 then num_elem_var(cdf_id) = 0 else begin
          num_elem_var(cdf_id) = dimvalue(ok(0))
          ndset(2) = dimvalue(ok(0))
          if keyword_set(verbose) then print, ' Number of Cell Variables = ', $
            strtrim(dimvalue(ok(0)), 2)
        endelse

        ok = where(dimname eq 'num_nod_var', nok)
        if nok eq 0 then num_nod_var(cdf_id) = 0 else begin
          num_nod_var(cdf_id) = dimvalue(ok(0))
          ndset(1) = dimvalue(ok(0))
          if keyword_set(verbose) then print, ' Number of Nodal Variables = ', $
            strtrim(dimvalue(ok(0)), 2)
        endelse
        if n_elements(lcdfvalid) eq 0 then lcdfvalid = -1
        
        if keyword_set(noload) then return
        load_cdf_data, dimname, dimvalue, verbose = verbose
      endif else if type eq 2 then begin
;       Waveform data
        num_nod_var(cdf_id) = 0
        num_elem_var(cdf_id) = 0
        varid = ncdf_varid(fid, 'numb_dataset')
        if varid ge 0 then begin
          ncdf_varget, fid, varid, value
          num_glo_var(cdf_id) = value
        endif else begin
          print, '% OPENCDF: Number of datasets variable not found'
          print, '          Contact PFIDL adminsistrator'
          num_glo_var(cdf_id) = 0
        endelse
        if keyword_set(noload) then return
        load_cdf_data, dimname, dimvalue, verbose = verbose
      endif
;     ndset = Array of [# of global, # of node, # of cell]
      ndset= [num_glo_var(cdf_id), num_nod_var(cdf_id), num_elem_var(cdf_id)]
      ierror = 0
      end
