; ------------------------------------------------------------------------ 
; $Id: plotfe9.pro,v 1.6 2007/06/06 19:39:21 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
  pro plotfe9, a, time, node = node,onode = onode, index= index, $
        coption = coption, value_node = value_node, skip_node = skip_node, $
        xarray = xarray, yarray = yarray, farray = farray, $
       slice_only = slice_only, xtime = xtime, work = work, $
        xrange = xrange, yrange = yrange, zrange= zrange, $
       block = block, _extra = extrastuff
;
;+
;  Name: PLOTFE9
;
;  This routine will slice a finite element structure in time or provide a
;  time history of a particular node.  This data consists of two spatial
;  dimensions with an amplitude for a number of times.
;
;  Format:
;       PLOTFE9, STR, Time, [/XTIME] [, /INDEX ] NODE = Node [, ONODE = onode]
;         [, Value_Node = valuenode]  [, COPTION = coption]
;         [, SKIP_NODE=skip_node]
;         [, XRANGE = xrange] [, YRANGE = yrange] [, BLOCK = block]
;         [, XARRAY = xarray] [, YARRAY = yarray] [, FARRAY = farray]
;         [, SLICE_ONLY = sliceonly] [, WORK = work] [, _EXTRA = extrastuff]
;  where:
;    STR        - Structure array number or a structure array of a type 9
;                 field (2 spatial dimensions and time)
;
;  NOTE:  Either Xtime, Time or node is required, but both may be used.
;
;    XTIME      - If set, a plot of X versus Time for lagrangian nodes.
;                 This is ignored if Node is a vector of length =3.
;                 Minimum value of y to 2^(-14) is used 
;                 Note: If Xtime is lt 0 then x is X[t[0]] for all times
;                       If Xtime is gt 0 then X is X[t]
;    TIME       - Time for a slice to be taken 
;                 If keyword(INDEX) is set then the time will be taken as a 
;                 time slice and time must be in the range [1, Ntime]
;                 If time is not an exact time slice the values will be 
;                 interpolated.
;                 If time is outside the range of valid times, the nearest
;                   value will be used.
;                 This converts a type 9 array to a single block type 8 array.
;                 This will be stored in st2 = work  and the
;                 following command issued:
;                   plotfld, st2, xrange = xrange, yrange = yrange, $
;                     xarray = xarray, yarray = yarray, farray = farray, $
;                     _extra = extrastuff
;    INDEX      - If set, TIME is taken as a time index.
;                 If set and NODE has more than one value, all values will be
;                 considered as node index values.  Note: If Node has more than
;                 one value and index is set, then no time slices will be made.
;    Node       - Time history for a node or nodes.  Actual node values are 
;                 returned in Value_Nodes.  If NODE has more than one element
;                 then value of INDEX is checked.
;               Scalar Value
;                 Values must be in the range [1, Number_of_nodes]
;                 If Node is zero or negative, then a plot will be made at the
;                 first time step and the user will be asked to designate a node.
;                 Xrange, Yrange and Block will be used for the plot.
;                 If no valid blocks are specified then all blocks will be used.
;                 If node gt number_of_nodes then node is set to -1 and a warning
;                 is issued.
;               Vector Value - Two values and INDEX not set
;                 The closest [X, Y] (= node(0:1)) points at the initial time 
;                 will be selected.
;               Vextor Value - Three Values = [X1, X2, Y] and INDEX not set
;                 The closest [X1, Y] and [X2, Y] points at the initial time 
;                 will be selected and all points between these values.
;                 Y values are selected as (y1+y2)/2 with an error of 
;                 err= (abs(y1-y2)/2+ yrange/2^15  
;        
;                 EXAMPLE:
;                 readcdf, 1, /grid
;                 readcdf, 2, 'density'
;                 plotstr, 2, 0, xr=[1.25, 1.4]
;                 plotstr, 1, /ov
;                 cur, x, y
;                 cur, x2
;                 readcdf, 3, 'posx
;                 plotstr, 3, node=[x, x2, y], value = v
;                 plo, 1, n_elements(v), /over, /spectrum
;    VALUE_NODE   Value of nodes returned above.
;    SKIP_NODE    This parameter is ignored unless more than one node is output
;                 SKIP_NODE may be an integer N; in that case then total number
;                    of nodes will be reduced by N.  For example SKIP_NODE =2
;                    will output half the nodes.
;                 SKIP_NODE may be an array.  This array will be replicated to
;                    match the total number of nodes and only non-zero values 
;                    will be output.  For example: SKIP_VALUE=[1,0,0,1]
;                    will output half the nodes; 
;                    Value_node = [1,4,5,8,9,12,....]
;    Coption      Comment option for node output.  This is set by bit value.
;                 Coption =1  Output the node value
;                 Coption =2  Output the X position at t=0.
;                 Coption =4  Output the X and Y position at t=0.
;                 Default = 1
;
;    SLICE_ONLY - only slice the structure and store the slice in work
;    WORK       - Location of the slice.
;                 Default value: WORK = get_maxstr()-1 
;    Onode      - WDF output array for node value
;                 DEFAULT: ONODE =1
;              IF one node value then the following options exist.
;                 6 types of wdf arrays are possible.
;                 1. Value vs time         4. Y vs X
;                 2. X vs time             5. Value vs X
;                 3. Y vs time             6. Value vs Y
;
;                 If WDF is a scalar, then [wdf, wdf+1, wdf+2] will be saved
;                 with types 1,2,3.  Default is [1, 2, 3]
;
;          NOTE:  Additional plots can be obtained with the XPLO command.
;
;                 If WDF is a vector then up to 6 WDF arrays will be saved.
;                 A WDF array will be saved for each valid value of a wdf array.
;                 For example: onode = [1, 0,0,0,2] would store value vs time
;                 in WDF array 1 and Value vs X in array 2.
;               If more than one node values then:
;                 Only value vs time will be saved.
;                 If n_ELEMENTS (ONODE) => N_ELEMENTS(NODE) then value vs time 
;                   will be save in each waveform array specified by ONODE.
;                 If n_ELEMENTS (ONODE) < N_ELEMENTS(NODE) then value vs time 
;                   will be save in each waveform array specified by 
;                   ONODE = [ONODE, ONODE[NO-1]+1+INDGEN(NN-NO)]
;                   Where NO is the number of onode values and NN is the number
;                   of node values. If all the data cannot be saved then, no
;                   data is saved.
;    Xrange, yrange, zrange - passed to plotfld and used if node le 0 
;    extrastuff - any valid keyword for plotfld.                 
;
;     Written 11/19/97 - lpmix
;      Modified: 3/1/00 - lpmix
;-
@qscom.dat
;  Common block
@qsstrcom.dat
@com.dat
;
;       return on an error
      on_error, 1
common mkcircle, lcmkcircle, nel_symbols

      if n_elements(lcmkcircle) eq 0 then lcmkcircle = 0
      if lcmkcircle eq 0 then mkcircle

      if n_elements (structinit) eq 0 then qsstruct_init
      if n_elements (qsplotinit) eq 0 then qsinit
;
      npar = n_params()
      if npar eq 0 then begin
       print, '% Format: ', $
      'PLOTFE9, STR, Time, /XTIME [, /INDEX ] NODE = Node [, ONODE = onode]'
print, '             [, XRANGE = xrange] [, YRANGE = yrange] [ZRANGE = zrange]'
print, '             [, XARRAY = xarray] [, YARRAY = yarray] [, FARRAY = farray]'
print, '             [, BLOCK = block] [, /SLICE_ONLY] [, _EXTRA = extrastuff]'
        return
      endif
;     Set STRUCTINIT positive in case we send data to plotfld
;     If call from plotstr, structinit is negative.

      structinit = 1

      neln = n_elements(node)
      
      if (npar eq 1) and (neln eq 0) and not keyword_set(xtime) then begin
          print, '% PLOTFE9: Either a time, /xtime  or a node must be specified'
          return
      endif
      if ((neln gt 0) and (n_elements(initwdf) eq 0)) then wdfinit
      b = size(a)
      if b(b(0)+1) ne 8 then begin
        if a gt 0 and a le maxstructure then begin
          s2i, a, a
          b = size (a)
          if b(b(0)+1) ne 8 then begin
            print, '% PLOTFE9: ', $
   ' First parameter must be a structure or a structure array number'
            return
          endif
        endif  else begin
          print, '% PLOTFE9: ', $
   ' First parameter must be a structure or a valid structure array number'
          return
        endelse
      endif

      if a.type ne 9 then begin
        print, '% PLOTFE9: ', $
          'Only type 9 structures can be used with PLOTFE9'
        return
      endif
; Check for node information
      if n_elements (xrange) eq 0 then xrange = 0
      if n_elements (yrange) eq 0 then yrange = 0
    
      if neln gt 0 then begin
        value_node = node
        if neln eq 1 then begin
          if node le 0 then begin
            if n_elements (block) gt 0 then begin
              ok = where (block gt 0 and block le a.block, nok) 
              if nok gt 0 then blo = block(ok) else $
                blo = indgen(a.block)+1
            endif else blo = indgen(a.block)+1
            nok = n_elements(blo)
            if n_elements(xrange) lt 2 then begin
              xr = make_array(2)
              xr(0) = min(a.range(0, 0, blo))
              xr(1) = max(a.range(1, 0, blo))
              xst = 0
            endif else begin
              xr = xrange(0:1)
              xst = 1
            endelse
            if n_elements(yrange) lt 2 then begin
              yr = make_array(2)
              yr(0) = min(a.range(0, 1, blo))
              yr(1) = max(a.range(1, 1, blo))
              yst = 0
            endif else begin
              yr = yrange(0:1)
              yst = 1
            endelse
            plot, /nodata, xr, yr, xrange = xr, yrange = yr, yst= yst, $
              xst = xst, Title = a.clab
            for i = 0, nok-1 do begin
              st = a.locx[0, i]
              en = a.locx[0, i+1]-1
              oplot, a.x1(st:en, 0), a.x2(st:en, 0), color = (i mod 7)+1, $
                psym = 8
            endfor
            print, ' Pick the desired node'
            curp, node_pt
;           add 1 to have value 1 to (number of nodes)
            value_node = get_min_dist(node_pt,a.x1(*,0),a.x2(*, 0),/screen)+1
            if value_node lt 1 then begin
              print, '% PLOTFE9: ', $
               ' Error calculating the point closest to', p
              return
            endif
          endif
          if value_node gt a.locx(0, a.block) then value_node = -1
        endif else begin
          if not keyword_set (index) then begin
            if neln eq 2 then $
            value_node = get_min_dist(node(0:1),a.x1(*,0), a.x2(*, 0)) $
            else begin
              i1 = get_min_dist([node(0), node(2)],a.x1(*,0), a.x2(*, 0))
              i2 = get_min_dist(node(1:2),a.x1(*,0), a.x2(*, 0))
              dy = (max(a.x2, min= mm)-mm)/16384.
              xmin = min([a.x1(i1, 0), a.x1(i2, 0)], max = xmax)
              ymin = min([a.x2(i1, 0), a.x2(i2, 0)], max = ymax)

              value_node = where((a.x2 gt ymin-dy) and (a.x2 lt ymax+dy) $
                 and (a.x1 ge xmin) and (a.x1 le xmax))
              ii = sort(a.x1(value_node))
              value_node = value_node(ii) 
            endelse
          endif
;         get value_node in the range [1, number of nodes]
          value_node = value_node+1
        endelse
        ok = where (value_node gt 0 and value_node le a.locv(a.block), neln)
        if neln eq 0 then begin
          print, '% PLOTFE9:  Error calculating node positions'
          return
        endif
        if neln lt n_elements(value_node) then value_node = value_node[ok]
        if N_elements(onode) eq 0 then onode = 1
        if neln eq 1 then begin
          if N_elements(onode) eq 1 then onode = onode + indgen(3)
          mwdf = get_maxwdf()
          vv= a.v1(value_node-1, *)
          xx= a.x1(value_node-1, *)
          yy= a.x2(value_node-1, *)
          tt = a.x4
          i = strpos (a.clab, ' - ')
          if i gt 0 then vlab = strmid(a.clab, 0, i) else vlab = a.clab
          clab = 'Node '+strtrim(value_node, 2)+', ('+strtrim(xx(0), 2)+',' $
            +strtrim(yy(0), 2)+')'
          for i = 0, n_elements(onode)-1 do begin
            if onode(i) gt 0 and onode(i) le  mwdf then begin
              if i eq 0 then begin
                i2w, tt, vv, onode(i), xlabel = 'Time', ylabel = vlab, $
                  clab = vlab + ' vs t - ' + clab, file = a.file, /xy
              endif else if i eq 1 then begin
                i2w, tt, xx, onode(i), xlabel = 'Time', ylabel = a.x1lab, $
                  clab = a.x1lab + ' vs t - ' + clab, file = a.file, /xy
              endif else if i eq 2 then begin
                i2w, tt, yy, onode(i), xlabel = 'Time', ylabel = a.x2lab, $
                  clab = a.x2lab + ' vs t - ' + clab, file = a.file, /xy
              endif else if i eq 3 then begin
                i2w, xx, yy, onode(i), xlabel = a.x1lab, ylabel = a.x2lab, $
                  clab = 'Y vs X - ' + clab, file = a.file, /xy
              endif else if i eq 4 then begin
                i2w, xx, vv, onode(i), xlabel = a.x1lab, ylabel = 'Amplitude', $
                  clab = vlab+' vs X - ' + clab, file = a.file, /xy
              endif else if i eq 5 then begin
                i2w, yy, vv, onode(i), xlabel = a.x2lab, ylabel = 'Amplitude', $
                  clab = vlab+' vs Y - ' + clab, file = a.file, /xy
              endif
            endif
          endfor
        endif else begin 
          nels = n_elements(skip_node)
          if nels gt 0 then begin
            if nels eq 1 then begin
              skip = round (skip_node[0])>1
              nn = round(neln/skip)
              if (neln mod skip) gt 0 then nn = nn+1
              if nn lt 2 then begin
                value_node = value_node[[0, neln-1]]
                neln = 2
                print, '% PLOTFE9: Skip factor too large'
                print, '           First and last nodes will be output'
              endif else begin
                value_node = value_node[skip*indgen(nn)]
                neln = nn
              endelse
            endif else begin
              nrep= neln/nels
              if(neln mod nels) gt 0 then nrep=nrep+1
              skip = intarr(nels)
              ok = where( skip_node ne 0, nok)
              if nok eq 0 then begin
                print, '% PLOTFE9: No output specified'
                print, '           One node will be output with a skip of',nels
                skip[0] = 1
              endif else skip[ok] =1
              j = make_array(nels*nrep)
              for i = 0, nrep-1 do j[i*nels:(i+1)*nels-1] = skip
              j = j[0:neln-1]
              ok = where (j ne 0, neln)
              value_node= value_node[ok]
            endelse
          endif
          ok = where (onode gt 0 and onode le maxwdfarray, nok)
          if nok gt 0 then onode = onode[ok] else onode = 1
          nelo = n_elements (onode)
          if nelo lt neln then $
            onode = [onode, onode[nelo-1]+1+indgen(neln-nelo)]
          if onode[neln-1] gt maxwdfarray then begin
            print, '% PLOTFE9: ONODE value exceeds MAXWDFARRAY', $
             onode[neln-1], maxwdfarray
            Print, '  Please reduce the range of X or increase MAXWDFARRAY'
            return
          endif
          tt = a.x4
          if n_elements(coption) eq 0 then coption = 1 
          i = strpos (a.clab, ' - ')
          if i gt 0 then vlab = strmid(a.clab, 0, i) else vlab = a.clab
          for i = 0, neln-1 do i2w, tt, a.v1[value_node[i]-1, *], onode[i], $
            ylabel = vlab, xlabel = 'Time', clab = $
            get_wdf_comment(coption, value_node[i], a.x1[value_node[i]-1,0], $
            a.x2[value_node[i]-1,0]), file = a.file, /xy
          return
        endelse
      endif
      if n_elements (time) eq 0 then begin
        if keyword_set (xtime) then begin
        if xtime lt 0 then begin
          ymin = a.range(0, 1, 0)
          eps=( a.range(1, 1, 0)-a.range(0, 1, 0))/16384.
          ok = where (a.x2(*, 0) le (ymin+eps), nok) 
          xx = a.x1(ok, 0)
          ii = sort(xx)
          ok = ok(ii) 
          xx = a.x1(ok, 0)
          v1 = a.v1(ok, *)
          yy = a.x4
          ny = n_elements(yy)
          ss = size(v1)
          if (ss(1) ne nok) or (ss(2) ne ny) then begin
            print, '% PLOTFE9: Problem with array sizes'
            help, v1, xx, yy
            return
          endif
          ssize = [nok, ny]
          range = make_array(2, 2, 2)
          range(0, 0, *) = min(xx, max = m)
          range(1, 0, *) = m
          range(0, 1, *) = min(yy, max = m)
          range(1, 1, *) = m
        
          locx = reform( [0, nok, 0,ny ], 2,2)
          locv = [0, ny*nok]
          st2 = {type: 1L, ndim:2L, nvect: 1L, block:1L, two:3L, $
             ntime:1L,  slice:ymin, size: ssize, range: range, $
              order: 0L, locx:locx, locv: locv,  $
              x1:xx, x2:yy, x3:0.0, x4:0.0, v1:v1, v2:0.0, v3:0.0, v4:0.0, $
              v5:0.0, v6:0.0, x1lab : a.x1lab, x2lab :a.x4lab, x3lab : '', $
              x4lab : '', v1lab : 'Amplitude' , v2lab : '', $
              v3lab : '', v4lab : '',v5lab : '', v6lab : '', $
              file : a.file, spare : a.spare, tlab : a.tlab, $
              clab:a.clab +' - Position vs Time' }
          if not keyword_set(slice_only) then $
            plotfld, st2, xrange = xrange, yrange = yrange, zrange= zrange, $
                    xarray = xarray, yarray = yarray, farray = farray, $
                    _extra = extrastuff

          i2s, st2, maxstructure-1, /no_copy
        endif else begin
          ymin = a.range(0, 1, 0)
          eps=( a.range(1, 1, 0)-a.range(0, 1, 0))/16384.
          ok = where (a.x2(*, 0) le (ymin+eps), nok)
          xx = a.x1(ok, 0)
          ii = sort(xx)
          ok = ok(ii)
          xx = a.x1(ok, *)
          v1 = a.v1(ok, *)
          yy = make_array(nok, a.ntime)
          for i = 0, a.ntime -1 do yy[*, i] = a.x4[i]
          tri = make_triangles(0, nok, a.ntime )
          ss = size(tri)
          loctri=[0, ss[2]]

          ny = n_elements(yy)
          ssize = [ny, ny]
          range = make_array(2, 2, 2)
          range(0, 0, *) = min(xx, max = mm)
          range(1, 0, *) = mm
          range(0, 1, *) = min(yy, max = mm)
          range(1, 1, *) = mm
          st2 = {type: 8L, ndim:2L, nvect: 1L, block:1L, two:3L, $
             ntime:1L,  slice:ymin, size: ssize, range: range, $
              order: 0L, locx:a.locx, locv: a.locv, triangles :tri, $
              loctri:loctri, $
              x1:xx, x2:yy, x3:0.0, x4:0.0, v1:v1, v2:0.0, v3:0.0, v4:0.0, $
              v5:0.0, v6:0.0, x1lab : a.x1lab, x2lab :a.x4lab, x3lab : '', $
              x4lab : '', v1lab : 'Amplitude' , v2lab : '', $
              v3lab : '', v4lab : '',v5lab : '', v6lab : '', $
              file : a.file, spare : a.spare, tlab : a.tlab, $
              clab:a.clab +' - Position vs Time' }
          if not keyword_set(slice_only) then $
            plotfld, st2, xrange = xrange, yrange = yrange, zrange= zrange, $
                    xarray = xarray, yarray = yarray, farray = farray, $
                    _extra = extrastuff

          i2s, st2, maxstructure-1, /no_copy

        endelse
        endif else begin
          if neln gt 0 or keyword_set (xtime) then return
          print, '%PLOTFE9: Time or index must be provided.'
        endelse
        return
      endif
      if keyword_set (index) then begin
        time = round (time)
;  next if block was commented out for some reason?
;
        if time lt 1 or time gt a.ntime then begin
          print, '% PLOTFE9: Time index must be in the range: (1, ' + $
            strtrim(a.ntime, 2)+')'
          return
        endif
        tim = time -1
        xx = reform(a.x1(*, tim))
        yy = reform(a.x2(*, tim))
        if n_elements(a.triangles) lt 3  and $
          strpos(a.tlab,'PACT') ge 0 then begin
          ireg= a.iregion[*, *, tim]
          imat = ireg
          make_pact_triangles, imat, triangles
          ss = size(triangles)
          loctri = [0L, ss[2]]
          vv = a.v1[*,  tim]
          if strpos(a.tlab , 'PACT Node') lt 0 then begin
            vv =  reform (vv, a.size, /overwrite)
            cell2node_3d, vv, ireg, nodes
            vv = temporary(nodes)
          endif
        endif else begin
          if (size(a.v1))[0] eq 2 then vv = reform(a.v1(*, tim)) else vv = a.v1
          triangles = a.triangles
          loctri = a.loctri
        endelse
        slice = a.x4(tim)
        timlab = '; Time: '+strtrim(slice, 2)
      endif else begin
        nt = a.ntime
        slice = time > a.x4(0)
        slice = slice < a.x4(nt-1)
        mint = min(abs(slice-a.x4), ii)
;       if (ii eq 0 and slice lt a.x4(0)) or $
;         (ii eq nt-1 and slice gt a.x4(nt-1)) then begin
;     print, '% PLOTFE9: Time outside the range of the data: ',strtrim(slice,2)
;     print,  nt, ' Data values in the range: ', a.x4(0), ' to', a.x4(nt-1)
;         return
;       endif
        if n_elements(a.triangles) lt 3  and $
          strpos(a.tlab,'PACT') ge 0 then begin
          tim = (ii > 0) < (nt-1)
          ireg= a.iregion[*, *, tim]
          imat = ireg
          if (a.x4[tim] eq slice) then begin
            xx = reform(a.x1(*, tim))
            yy = reform(a.x2(*, tim))
            vv = reform(a.v1[*,  tim])
            slice = a.x4[tim]
          endif else begin
            timsav = tim
            if ii eq (nt-1) then ii = ii-1
            if slice ge a.x4(ii) then begin
              tim = [ii, ii+1]
              frac = [slice-a.x4(ii), a.x4(ii+1)-slice]/(a.x4(ii+1) - a.x4(ii))
            endif else begin
              tim = [ii-1, ii]
              frac = [slice-a.x4(ii-1), a.x4(ii)-slice]/(a.x4(ii) - a.x4(ii-1))
            endelse
;           check the regions
            if ((max(a.iregion[*, *, tim[0]]-a.iregion[*, *, tim[1]], min=mmm) $
                 -mmm) eq 0) then begin
              xx = reform(a.x1(*, tim(0))*frac(1)+a.x1(*, tim(1))*frac(0))
              yy = reform(a.x2(*, tim(0))*frac(1)+a.x2(*, tim(1))*frac(0))
              vv = reform(a.v1(*, tim(0))*frac(1)+a.v1(*, tim(1))*frac(0))
            endif else begin
              xx = reform(a.x1(*, timsav))
              yy = reform(a.x2(*, timsav))
              vv = reform(a.v1[*, timsav])
              slice = a.x4(timsav)
            endelse
          endelse
          make_pact_triangles, imat, triangles
          ss = size(triangles)
          loctri = [0L, ss[2]]
          if strpos(a.tlab , 'PACT Node') lt 0 then begin
            vv =  reform (vv, a.size, /overwrite)
            cell2node_3d, vv, ireg, nodes
            vv = temporary(nodes)
          endif
          timlab = '; Time: '+strtrim(slice, 2)
        endif else begin
          if ii eq nt -1 then ii = ii-1
          if slice ge a.x4(ii) then begin
            tim = [ii, ii+1]
            frac = [slice-a.x4(ii), a.x4(ii+1)-slice]/(a.x4(ii+1) - a.x4(ii))
          endif else begin
            tim = [ii-1, ii]
            frac = [slice-a.x4(ii-1), a.x4(ii)-slice]/(a.x4(ii) - a.x4(ii-1))
          endelse
          xx = reform(a.x1(*, tim(0))*frac(1)+a.x1(*, tim(1))*frac(0))
          yy = reform(a.x2(*, tim(0))*frac(1)+a.x2(*, tim(1))*frac(0))
          vv = reform(a.v1(*, tim(0))*frac(1)+a.v1(*, tim(1))*frac(0))
          timlab = '; Time: '+strtrim(slice, 2)
          triangles = a.triangles
          loctri = a.loctri
        endelse
      endelse
      if n_elements(block) gt 0 then begin
        ok = where (block gt 0 and block le a.block, nok)
        if nok gt 0 then begin
          is = 0
          for i = 0, nok-1 do begin
            blo = block(ok(i))
            np = a.locx(blo)-a.locx(blo-1)
            xx(is:is+np-1) = xx(a.locx(blo-1):a.locx(blo)-1) 
            yy(is:is+np-1) = yy(a.locx(blo-1):a.locx(blo)-1) 
            vv(is:is+np-1) = vv(a.locx(blo-1):a.locx(blo)-1) 
            is = is +np
          endfor
          xx = xx(0:is-1)
          yy = yy(0:is-1)
          vv = vv(0:is-1)
        endif
      endif
      nelx = n_elements(xx)
      range = make_array(2,2, 2)
      if n_elements(triangles) gt 8 then begin
        range(0, 0, *) = min(xx[triangles], max = mm)
        range(1, 0, *) = mm
        range(0, 1, *) = min(yy[triangles], max = mm)
        range(1, 1, *) = mm
      endif else begin
        range(0, 0, *) = min(xx, max = mm)
        range(1, 0, *) = mm
        range(0, 1, *) = min(yy, max = mm)
        range(1, 1, *) = mm
      endelse
      ssize = [nelx, nelx]
      locx = [0, nelx]
      locv = [0, nelx]
      st2 = {type: 8L, ndim:2L, nvect: 1L, block:1L, two:3L, $
             ntime:1L,  slice:slice, size: ssize, range: range, $
              order: 0L, locx:locx, locv: locv,triangles:triangles,  $
              loctri:loctri, $
              x1:xx, x2:yy, x3:0.0, x4:0.0, v1:vv, v2:0.0, v3:0.0, v4:0.0, $
              v5:0.0, v6:0.0, x1lab : a.x1lab, x2lab :a.x2lab, x3lab : '', $
              x4lab : '', v1lab : a.v1lab , v2lab : '', $
              v3lab : '', v4lab : '',v5lab : '', v6lab : '', $
              file : a.file, spare : a.spare, tlab : a.tlab, $
              clab:a.clab+ timlab }

      if keyword_set (work) then begin
        if work lt 1 or work gt maxstructure then work = maxstructure-1
      endif else work = maxstructure-1

      if not keyword_set(slice_only) then $
        plotfld, st2, xrange = xrange, yrange = yrange, zrange= zrange, $
            xarray = xarray, yarray = yarray, farray = farray, $
            _extra = extrastuff
      i2s, st2, work, /no_copy
      return
      end
