; ------------------------------------------------------------------------ 
; $Id: recdf.pro,v 1.5 2008/02/27 22:49:24 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
 Pro  recdf, wdfa, dataset, all=alldata, narray=narray, fileid=fid
;+
;  Reads a PFF dataset into a WDF array.  Procedure options are explained
;  below.
;
;  Format:  RECDF, wda  , dataset [, NARRAY = narray] [, ALL = all] 
;             [, BLOCK = block] [, FILEID = fileid] 
;  where:
;    wda -     WDF array number or an array of numbers
;    dataset - indicates a CDF dataset number, an array of dataset numbers,
;              or is a search string for dataset comments.  
;        Note: A '*' value of dataset will return all datasets.
;        See: GET_DSET for more details
;    all -     nonzero value indicates all datasets with the search 
;              string are to be read into WDF arrays.  If all is 
;              missing or zero, only the elements of wda will be filled.
;    fileid -  file id of the PFF file containing the data
;    narray -  number of datasets read into WDF arrays
; the parameters wda and dataset can take the following combination of values
; to produce the correspondint actions:
;  wda          dataset         action
; array(N)  zero or missing   Read current dataset and (N-1) successive
;                             datasets into the array(N) WDF arrays.
; integer      array(M)       Read array(M) datasets and store in WDF array
;                             integer and the following (M-1) WDF arrays.
; array(N)     array(M)       M must equal N; read the array(M) datasets
;                             and store in the array(N) WDF arrays.
; integer      string         Read dataset with string into WDF array, integer.
;             (all = 0)       If more than one array with string, look for
;                             string followed by spaces at the beginning of
;                             the dataset comment.
;            (all not zero)   Read all datasets with string into WDF array,
;                             integer and successive WDF arrays.
; array(N)     string         Read all datasets with string into WDF arrays,
;                             array(N).  Number of array elements must equal
;                             number of datasets with string in the dataset
;                             comment.If string occurs in only 1 dataset,
;                             then read that dataset and the next (N-1) 
;                             datasets into WDF arrays, array(N).
;  Examples:
;    Read dataset 34 into array 1 from file with file id of 4.
;      recdf, 1, 34, fil = 4
;    Read into WDF array 2,..., 10, datasets 34,..., 42 from the current file
;      recdf, indgen(9)+2, 34
;  or: redcv, 2, indgen(9)+34
;    Read into WDF arrays beginning with 2 all datasets beginning with "mspin" 
;    in the current file and indicate the number of arrays in the variable,
;    nread. 
;      recdf, 2,  '^mspin' , n=nread, /all 
;    Read into WDF arrays [2, 5, 8, 11] the datasets [3, 4, 1, 2] from file
;    with id of 2
;      recdf, [2,5,8,11], [3,4,1,2], file= 2
;    Read into WDF arrays [2,5,8, 11] the datasets having the string "vtot".
;    Note:  vtot must occur in either 1 array or 4 arrays.
;      recdf,[2,5,8,11], 'vtot'
;-
;     add common block
      on_error, 1
@com.dat
@netcdfcom.dat

;     check to see if cdf and wdf have been initialized
;
      if n_elements(initcdf) eq 0 then init_cdf
;
      if n_elements(initwdf) eq 0 then wdfinit


;
      if cdf_id  lt 0 then begin
        print, '% RECDF: No netCDF files are open.'
        return
      endif
;
      narray = 0
      nparam = n_params()
      if (nparam lt 1 ) then begin
        print,'% Format: RECDF, wda , dataset [, NARRAY = narray] [, ALL = all]'
        print,'         [, FILEID = fileid]'
	return
      endif
      if (nparam lt 2) then begin
        print,'% RECDF: 2 parameters are required'
        return
      endif
      wdfa = get_save_array(wdfa, 0)
      if wdfa(0) eq 0 then begin
        print,'% RECDF: WDF array value must be provided'
        return
      end
      wdfar = wdfa(*)
      if wdfar(0) lt 1 or wdfar(0) gt maxwdfarray then begin
        print, '% RECDF: WDF array must be greater than 0 and </=', maxwdfarray
        return
      endif

      if n_elements (alldata) eq 0 then alldata = 0
      if n_elements (dataset) eq 0 then dataset = 0
      if n_elements (fid) eq 0 then fid = array_cdfid(cdf_id, 0)

      nwdf = n_elements(wdfar)
      if alldata eq 0 then nmax = nwdf else nmax = maxwdfarray
;
      if array_cdfid(cdf_id, 0) eq fid then begin
        if lcdfvalid ne array_cdfid(cdf_id, 1) then load_cdf_data
      endif else begin
        ok = where (fid eq array_cdfid(*, 0), nok)
        if nok ne 1 then begin
          print, '% RECDF: ', strtrim(nok,2), ' netCDF files with a fid of ', $
            strtrim(fid, 2)
          print, '         No Data read'
          return
        endif
        cdf_id = ok(0)
        if lcdfvalid ne array_cdfid(cdf_id, 1) then load_cdf_data
      endelse
      if num_glo_var(cdf_id) eq 0 then begin
        print, '% RECDF: No waveform arrays in this netCDF file'
        return
      endif
      if n_elements(dataset) eq 0 then begin
        print,'% RECDF: Dataset must be defined'
        return
      end
      get_ncdf_string, dataset, name_glo_var, map, nmap, nmax, ier, lcstring
      
      if ier ne 0 then begin
         print,'Valid datasets not found. Please change the dataset specification'
         return
      endif
;
      if alldata eq 0 and nmap ne nwdf and lcstring gt 0 then begin
        print, ' The number of WDF arrays must equal the number of '+ $
           'datasets with the string:'
        print, '             ' , dataset, ' -- Current datasets are'
        dirpff, dataset
        return
      endif else if lcstring lt 0 then begin
        alldata = 1
        nmax = maxwdfarray
      endif
      if nwdf gt 1 and nmap gt 1 then begin
        if nwdf ne nmap then begin
          print, ' Number of wdf arrays is ', nwdf
          print, ' Number of datasets is ', nmap
          print, ' These must be equal if both are greater than 1'
          return
        endif
      endif else begin
        if nwdf gt 1 then begin
          if map(0) gt 0 then map = map(0) + indgen(nwdf) else $
              map= make_array(nwdf, /int)
        endif
        if nmap gt 1 then wdfar = wdfar(0)+indgen(nmap)
      endelse
      narray = max([nwdf, nmap]) < maxwdfarray
      nread = 0
      ok = where (wdfar gt 0 and wdfar le maxwdfarray, nok)
      if nok ne narray then begin
        print, '% RECDF: Some WDF arrays are not valid.'
        print, '   Only ', strtrim(nok,2) , ' arrays will be read'
        narray = nok
      endif
      for i = 0, narray-1 do begin
        data = map(i)-1
        wdf = wdfar(ok(i))
        if cdf_type(cdf_id) eq 1 then begin
          if n_elements (wdf_array) lt 4 then begin
            varid = ncdf_varid(fcdf_id[cdf_id], 'vals_glo_var')
            if varid ge 0 then begin
              ncdf_varget, fcdf_id[cdf_id], varid, wdf_array
              wdf_array= transpose(wdf_array)
            endif else begin
              print, '% RECDF: No WDF array variables found in Exodus II data'
              print, '                 Number of global Variables set to zero.'
              num_glo_var(cdf_id) = 0
              return
            endelse
          endif
         put_wdfarray,[time_whole,reform(wdf_array[*,data])], wdf, /xydata, clab = $
            name_glo_var(data)+' - '+file_title(cdf_id), file=file_name(cdf_id), $
            xlab = 'Time (s)', /no_copy
;
        endif else if cdf_type(cdf_id) eq 2 then begin
          np = coord_cell(data)
          arr = wdf_array(loc_f(data) : loc_f(data+1)-1)
          if n_elements(arr) eq np+2 then xydata = 0 else xydata = 1
          put_wdfarray,arr, wdf,  xydata = xydata,  clab = $
           name_glo_var(data)+' - '+file_title(cdf_id), file=file_name(cdf_id), $
            xlab = x1_label(data), ylab = x2_label(data), /no_copy 
        endif
      endfor
      narray = nread
      return
      end
