; ------------------------------------------------------------------------
; $Id: run_anim.pro,v 1.2 2007/06/06 19:39:21 lpmix Exp $
; ------------------------------------------------------------------------
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
;+
; NAME:
;       RUN_ANIM
;
; PURPOSE:
;       This loads animation frames and displays them with the IDL routine:
;         XINTERANIMATE
;       See also MAKE_ANIM, GET_ANIM_WDF
;
; CATEGORY:
;       Grapics
;
; CALLING SEQUENCE:
;       RUN_ANIM, ARRAY [, BEG [, STOP[, SKIP]]] [, MAXIMUM = maximum]
;
; INPUTS:
;       ARRAY:  Array returned from MAKE_ANIM.  This is a series of annimation
;               snapshots.
;       BEG:    First frame to display.
;               If zero or undefined, BEG = 1.
;       STOP:   Last frame to display.
;               If zero or undefined, STOP = NTIME 
;               (where NTIME is the number of time steps in ARRAY)
;       SKIP:   Increment on the frame number.
;               If zero or undefined, SKIP = 1 if MAXIMUM is not set.
;                                or  SKIP = maximum required to keep total
;                                           number of bytes below MAXIMUM using
;                                           BEG and STOP
;      Note:  BEG, STOP and SKIP are used in a "DO Loop" sense.  Numbering is from 
;             1 to NTIME (NTIME = Number of time steps.)  "DO I= BEG, STOP, SKIP"
;
; KEYWORD PARAMETERS:
;       MAXIMUM: Maximum number of bytes in the animation.
;                For true color this is 3 * Xsize * Ysize * NFrame
;
; OUTPUTS:
;       An XINTERANIMATE animation is generated.
;
; RESTRICTIONS:
;       The computer must have enough video memory to store the animation.
;
; EXAMPLE:
;       For the following assume that the command:
;         ARRAY = MAKE_ANIM() 
;       has been run.
;
;       Make an animation with every third frame.
;         RUN_ANIM, ARRAY, 0, 0, 3
;
;       Make an animation with over all time with less than about 64Mb
;         RUN_ANIM, ARRAY, MAXIMUM = 65.e6
;
;       Make an animation with over the first 60 frames with less than 64Mb
;         RUN_ANIM, ARRAY, 0, 60, MAXIMUM = 65.e6
;
; MODIFICATION HISTORY:
; 	Written by:	L. P. Mix, Jr., July 19, 2005
;-

pro run_anim, arr, beg, sto, skip, maximum = maximum

npar = n_params()
if npar eq 0 then begin
  print, '% Format: RUN_ANIM, ARRAY, Beginning, End, Skip, MAXIMUM = maximum'
  return
endif
ss = size(arr)
if ss[0] lt 3 then begin
  print, '% Format: RUN_ANIM, ARRAY, Beginning, End, Skip'
  print, '% RUN_ANIM: First parameter must be a 3D or 4D array'
  print, '            Array = ARRAY[nx, ny, nframe]'
  print, ' <or>       Array = ARRAY[3, nx, ny, nframe]'
  return
endif
if ss[0] eq 3 then begin
  nx = ss[1]
  ny = ss[2]
  nf = ss[3]
  true_color = 0
endif else if ss[0] eq 4 then begin
  if ss[1] ne 3 then begin
    print, '% RUN_ANIM: Bad color information'
    help, arr
    return
  endif
  nx = ss[2]
  ny = ss[3]
  nf = ss[4]
  true_color = 1
endif else begin
  print, '% RUN_ANIM: Bad array passed to Run_Anim'
  help, arr
  return
endelse


If n_elements(beg) eq 0 then beg = 1 else if beg eq 0 then beg = 1
beg = (beg > 1) < nf

If n_elements(sto) eq 0 then sto = nf else if sto eq 0 then sto = nf
sto = (sto > beg+1) < nf



If n_elements(skip) eq 0 then skip = 1 else if skip eq 0 then skip = 1
skip = (skip > 1)
frames = sto-beg
frames = 1 +(frames/skip)
if keyword_set (maximum) then begin
  current = nx*ny*frames
  if true_color gt 0 then current = current*3
  if current gt maximum then begin
    frames = long(maximum/current *frames)
    skip = (sto-beg)/(frames-1)
    print, '%  RUN_ANIM:  SKIP set to ', strtrim(skip, 2)
  endif
endif
print, '%  RUN_ANIM:  Total number of frames: ', strtrim(frames, 2)
xinteranimate, set=[nx,ny,frames]
n = 0
for i = beg, sto, skip do begin
  if true_color eq 0 then  $
    xinteranimate, frame=n, image = reform(arr[*,*, i-1]) else $
    xinteranimate, frame=n, image = reform(arr[*,*,*, i-1])
  n += 1
endfor
xinteranimate, /block
end
