; ------------------------------------------------------------------------ 
; $Id: tslice.pro,v 1.3 2007/06/06 19:39:21 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
 Function tslice, ain, tvaluein, index = index
;
;+
; NAME:
;	TSLICE
;
; PURPOSE:
;	Reduce a four dimensional field structure to three dimensions
;       Must be 4 dimensional, single block, single vector, type 1 data.
;
; CATEGORY:
;	Graphics
;
; CALLING SEQUENCE:
;
;	Structure2 = SLICE (Structure1, tvalue [, /INDEX)
;
;
; INPUTS:
;	Structure1: Input this is a multidimensional field type structure.
;                     or a structure array numbger
;       Structure2: Output this is a three dimensional field structure 
;       TVALUE:     Time value for dimension 4 or index for dimension 4.
;               Note:  Index values range from 1 to n_elements(structure1.x4)
;
;       INDEX   : If present and not zero, then the units of TVALUE will be
;                 taken as indices of of the field array.  Valid values are
;                 1 to the maximum number of values in the fourth dimension.
;
; OUTPUTS:
;	Structure is modified to produce a three dimensional structure
;
; EXAMPLE:
;	Reduce structure f to a three dimensional structure by slicing along
;         the time = 5e-9; store the result in g.
;       g = TSLICE (f,  5e-9)
;
;	Reduce structure f to a three dimensional structure by slicing along
;         the time index of 5; store the result in g.
;       g = TSLICE (f,  5, /index)
;
; MODIFICATION HISTORY:
; 	Written by:	Paul Mix, July 20, 1998
;-
; insert common blocks
;
@qscom.dat
;
@qsstrcom.dat
;
;       return on an error
      on_error, 1
;
      npar = n_params()
      if n_elements (ain) eq 0 then begin
        print, '% Format: Structure2 = SLICE (Structure1, Tvalue [, /INDEX])'
        return, 0.0
      endif
      if npar lt 2 then begin
        print, '% TSLICE: Structure and time value must be provided'
        return, 0.0
      endif
      if n_elements (ain) eq 0 then begin
        print, '% TSLICE: Structure must be defined'
        return, 0.0
      endif
      if n_elements (tvaluein) eq 0 then begin
        print, '% TSLICE: Tvalue muse be defined'
        return, 0.0
      endif else tvalue = tvaluein
        
      if n_elements (structinit) eq 0 then qsstruct_init
      if structinit lt 0 then a = temporary(ain) else a = ain 
      structinit = 1
      b = size(a)
      if b(b(0)+1) ne 8 then begin
        if a gt 0 and a le maxstructure then begin
          s2i, a, a
          b = size (a)
          if b(b(0)+1) ne 8 then begin
            print, '% TSLICE: ', $
   'First parameter must be a structure or a structure array number'
            return, 0.0
          endif
        endif  else begin
          print, '% TSLICE:-', $
   'First parameter must be a structure or a valid structure array number'
          return, 0.0
        endelse
      endif
      if a.type ne 1 then begin
        print, '% TSLICE: Only Field type structures may be sliced'
        return, 0.0
      endif
;
;
; Function tslice, ain, tvalue, index = index
      ndim = a.ndim
      if a.ndim ne 4 then begin
        print, '% TSLICE: Structure must have 4 dimensions'
        return, 0.0
      Endif
      nv= a.nvect
      nb = a.block
      if nb ne 1 then  begin
        print, '% TSLICE: Structure must have a single block'
        return, 0.0
      Endif
      v2 = 0.0
      v3 = 0.0
      v4 = 0.0
      v5 = 0.0
      v6 = 0.0
      x4 = a.x4
      nt = n_elements(x4)
      v1 = get_array(a, -1)
      if keyword_set(index) then begin
        if min(tvalue,max=mm) lt 1 or mm gt nt then begin
          print, '% TSLICE: Slice value must be greater than 1 and less than ', $
            strtrim(n_elements(x4),2)
          return, 0.0
        endif
        v1 = reduce_iarray(v1, 4, x4, tvalue, 0)
        x4 = total(x4(tvalue-1))/n_elements(tvalue)

      endif else begin
        eps = (x4(nt-1)-x4(0)) / 65536.
        if min(tvalue, max = mm) lt x4(0)-eps or mm gt x4(nt-1)+eps then begin 
          print, '% TSLICE: Slice value must be greater than ',strtrim(x4(0),2), $
            ' and less than ',strtrim(x4(nt-1),2)
          return, 0.0
        endif
        v1 = reduce_array(v1, 4, x4, tvalue , 0)
        x4 = total(x4(tvalue-1))/n_elements(tvalue)
      endelse
;

; initialise arrays
      ndim = a.ndim 
      locv = [0L, n_elements(v1)]
      if ndim eq 4 then begin
        ndim = 3
        ssize = a.size (0:2, 0)
      endif else ssize = a.size(0:1, 0)
      
      if a.two eq 0 then begin
        ok = where (ssize eq 1, nok)
        if nok eq 0 then begin
          two = 4
          slice = x4(0)
        endif else begin
          two = ok(0)+1
          slice = ([a.x1(0), a.x2(0), a.x3(0)])(ok(0))
        endelse
      endif else begin
        two = a.two
        slice = a.slice
      endelse
      a = {type:1L, ndim:ndim, nvect: 1L, block :1L, two: two, $
            slice: slice, size:ssize,range: a.range, order: a.order, $
              locx:a.locx, $
              locv: locv, x1:a.x1, x2:a.x2, x3:a.x3, x4:x4, v1:v1, v2:v2, $
              v3:v3, v4:v4, v5:v5, v6:v6,x1lab : a.x1lab, x2lab :a.x2lab, $
              x3lab : a.x3lab, x4lab :a.x4lab,v1lab : a.v1lab, v2lab :a.v2lab, $
              v3lab : a.v3lab, v4lab :a.v4lab,v5lab : a.v5lab, v6lab :a.v6lab, $
              file : a.file, spare : a.spare, tlab : a.tlab, $
              clab: a.clab + ';; Time='+strtrim(x4(0),2)}
      return, a
      end
