; ------------------------------------------------------------------------
; $Id: dircsv.pro,v 1.1 2011/01/07 18:01:45 lpmix Exp $
; ------------------------------------------------------------------------
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
;+
; NAME:
;       DIRCSV
;
; PURPOSE:
;       This routine provides tag_names to a CSV structure.
;       The names are in a directory widget.
;       The number of rows, columns and tag names are returned as keyword variables
;       
;
; CALLING SEQUENCE:
;
;       DIRCSV, STRUCTURE [, COLUMN]
;
; INPUTS:
;       STRUCTURE: CSV structure generated from READCSV
;       COLUMN:  Search string to select certain columns
;                Default: All columns are listed with the units
;
; KEYWORD PARAMETERS:
;       TAGS:    Column headings for each column in the spreadsheet
;       UNITS:   An array of units for each column heading
;       KEYS:    An array with the contents of the first column 
;       QUIET:   If set, no output to the terminal unless there is an error
;       NROWS:   Number of rows in the spreadsheet
;       NCOLUMNS: Number of columns in the spreadsheet
;       LINE:    The string used for the directory is returned in LINE
;       PAGE:    Number of lines in the directory
;                Default = directorypage (24)
;       RESET:   If set, a new directory widget will be opened
;       COLUMN:  Column numbers for selected columns
;       HELPME:  If set, print a help message
;
; OUTPUTS:
;       A scrolling widget is generated with a list of the column headings with the
;       units for the column if they are available.
;
; OPTIONAL OUTPUTS:
;       Arrays with the column headings and the first column values can be returned
;       Scalars giving the number of rows and columns for the structure can be returned
;
;
; EXAMPLE:
;       List the column headings for CSV structure, SHOTSTR
;         DIRCSV, SHOTSTR
;
;       Get a list of column headings in variable, LIST, for structure, FILTSTR
;         DIRCSV, FILTSTR, TAGS = LIST, /quiet
;         NOTE:  QUIET eliminates all other outputs
;
; MODIFICATION HISTORY:
; 	Written by:	L. P. Mix, Jr., May 19, 2009
;
;
;-

pro dircsv, str, COL, tags = tags, keys = keys, quiet = quiet, ncolumns = ncol, $
      nrow= nrow, page = page, reset = reset, helpme = helpme, units = units, $
      column = column, line = line
on_error, 1

@qscom.dat
common dircsv_common, base, list, text1, oldpage, truncate_save
;
@csvcommon.dat
if n_elements(initcsv) eq 0 then init_csv
;
      if n_elements (qsplotinit) eq 0 then qsinit

      if n_elements (truncate_save) eq 0 then begin
        truncate_save =1
        base  = 0
        text1 = 0
        text2 = 0
        list  = 0
        oldpage = directorypage > 4
      endif
      if n_elements (page) eq 0 then page = oldpage > 4 else page = page > 4
      if n_elements (truncate) gt 0 then truncate_save=truncate $
        else truncate = truncate_save
      nparams = n_params()
      if keyword_set (helpme) || (nparams eq 0)  then begin
        print,'% Format: DIRCSV,  STRUCTURE [, COLUMN] [, NROW = nrow] [, NCOLUMNS = ncol]'
        print,'          [, TAGS = tags] [, KEYS = keys]  [, PAGE = PAGE]'
        print,'          [, VARIABLE = variable]'
        print,'          [, /RESET] [, /QUIET] [, /HELPME]'
        return
      endif
      if keyword_set(reset) then base = 0
      if size(str, /type) ne 8 then begin
        print, '% DIRCSV: CSV structure must be defined.'
        return
      endif
      tags = tag_names(STR)
      ok = get_match('CVS_TYPE', tags, nel = nel)
      if nel ne 1 then begin
        print, '% DIRCSV: This structure is not a CVS_STRUCTURE'
        return
      endif
      if str.CVS_TYPE ne 200 then begin
        print, '% DIRCSV: Expected CVS_TYPE of 200; value = '+strtrim(str.cvs_type, 2)
        return
      endif
      tags = str.tag_names
     
      keys = str.C0
      filename =str.file_name
      ncol = n_elements(tags)
      nrow = n_elements(keys)
      n = 2 + ceil(alog10(ncol))
      column = 1+indgen(ncol)
      forma = '(i'+strtrim(n)+')'
      row = string(column, format= forma)
      row = row + ' - '
      line = row+tags
      units = strtrim(str.UNITS, 2)
      ok = where (strlen(units) gt 0, nok)
      if nok gt 0 then line[ok] = line[ok]+"("+ units[ok]+")"
      if n_elements(col) gt 0 then begin
        if size(/type, col) ne 7 then begin
          print, '% DIRCSV:  Column search string must be a string'
          return
        endif
        ok = get_match(col[0], tags, nelem= nelem)
        if nelem eq 0 then begin
          print, '% DIRCSV:  No columns match the search string'
          return
        endif
        tags = tags[ok]
        line = line[ok]
        column = column[ok]
        units = units[ok]
      endif
      if keyword_set(QUIET) then return
      widget_directory, base, page, list, line, base_title = 'CSV Contents', $
            xsize = xsize, text1 = text1, value1 = filename, text2 = text2, $
            value2 ='Column - Column Name' 

      end
