; ------------------------------------------------------------------------
; $Id: getcsv.pro,v 1.2 2011/12/01 21:40:09 lpmix Exp $
; ------------------------------------------------------------------------
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
;+
; NAME:
;       GETCSV
;
; PURPOSE:
;       Return a column of data or a single value from a column of a 
;         CSV structure
;
; CALLING SEQUENCE:
;
;       DATA = GETCSV (STRUCTURE [, COL_VALUE] [, KEY = key] [, /SINGLE] [, IER = ier] )
;
; OUTPUTS:
;       DATA:   Scalar or vector with the contents of the designated column 
;               in the spreadsheet or several columns from a specified row.
; INPUTS:
;       STRUCTURE: Structure from which a column is to be extracted
;       COL_VALUE: Column number (1 to NCOL) or a string used to search the
;                  column headings.  
;                  If misssing, a widget will be provided to select the column.
;                  If multiple columns are found, a widget will be used to select 1 column
;
; KEYWORD PARAMETERS:
;       KEY:    Search string or integer to designate the desired row in the 
;               spreadsheet.  If missing, all rows are returned.  
;               Key is not a row number but a value to be matched to the first row.
;               If the type for column 1 is a string, then KEY must be a string.
;               KEY may be an array or a scalar.
;               If KEY is an INTEGER or LONG, IDL will look for equal values.
;               If KEY is a STRING, GET_MATCH will be used to find matches.  
;               For example, if a value of 846 is desired, KEY may be specified as
;                 KEY = 846 or KEY ='^846$'
;               If KEY is an array, then 
;                 If KEY is a string, each value will be searched and all unique rows
;                   will be returned.
;                 If KEY is an INTEGER or LONG, then 
;                   If there are 2 values, they will be interpreted as a minimum and 
;                     maximum value for the key.
;                   If there are 3 or more values, they will be considered a list and
;                     each value will be searched.
;       SINGLE: If set, and key is not unique, a widget will be provided to 
;               insure a scalar is returned.
;               If not set, multiple rows will be returned.
;       ROWS:   Values of the key (first column) for the values returned.
;       IER:    If no error occurs, IER = 0
;
; RESTRICTIONS:
;       VALUE must be a scalar integer or long, or VALUE must be a string that 
;       provides a unique column identification.  
;
; EXAMPLE:
;       Return the neutron detector distance for detector 1 for shot 1021.
;       Assume that the column heading is 'ND 1-DIST' and the shot data has 
;       been read into shotstr.  Store the distance in DIST.
;          dist = get_value(shotstr, 'ND 1-DIST', KEY = 1021)
;
;       Return column 4 from structure, STR, into variable V4
;          V4 = GETCSV(str, 4)
;
;	Return a list of filters for shot 999, in SHOTSTR.  Assume the column
;       headings are of the form: FILT 1, Filt 2, ...  
;          dircsv, SHOTSTR, '^filt', column= column
;          nfilt = n_elements (column)
;          filter_names = make_array(nfilt, /string)
;          for i = 0, nfilt-1 do filter_names[i] = getcsv(shotstr,column[i], key= 999)
;
; MODIFICATION HISTORY:
; 	Written by:	L. P. Mix, Jr., May 20, 2009
;	July, 1994    
;
;-

Function getcsv, structure, valuein, key = key, single = single, rows = rows, ier = ier

on_error, 1

@csvcommon.dat

ier =1
if n_elements(initcsv) eq 0 then init_csv

if size(structure, /type) ne 8 then begin
  print, '% GETCSV: CSV structure must be defined.'
  return, 0
endif
tags = tag_names(structure)

ok = get_match('CVS_TYPE', tags, nel = nel)
if nel ne 1 then begin
  print, '% GETCSV: This structure is not a CVS_STRUCTURE'
  return, 0
endif
if structure.CVS_TYPE ne 200 then begin
  print, '% GETCSV: Expected CVS_TYPE of 200; value = '+strtrim(structure.cvs_type, 2)
  return, 0
endif

instr = 'Pick the column that you wish returned'
tags = structure.tag_names
ncol = structure.ncol
nrow = structure.nrow
invalid_key = [0, 0, 1, 1, 0]
typecode =[ 2, 3, 4, 5, 7 ]

if ncol gt 1 then begin
  nelv = n_elements(valuein)
  
  if nelv eq 0 then begin
;   use dircsv to get the units included if they are available
    dircsv, structure, line = line, /quiet, column = column
    pick_one, line, pick, instr=instr, Title = 'GETCSV Column Selection'
    value = column[pick]
  endif else value = valuein[0]
  
  typ_val = size(/type, value)
  ok = where(typ_val eq typecode, nok)
  if (nok eq 0) || (invalid_key[ok[0]]) then begin
    print, '% GETCSV: VALUE must be integer, long, or string'
    help, value
    return, 0
  endif

  if ok[0] gt 2 then begin
;   use dircsv to get the units included if they are available
    dircsv, structure, value[0], line = line, /quiet, column = column
    if n_elements(line) gt 1 then begin
      pick_one, line, pick, instr=instr, Title = 'GETCSV Column Selection'
      value = column[pick]
    endif else value = column[0]
  endif else begin
    if (value[0] lt 1) || (value[0] gt ncol) then begin
      print, '% GETCSV: VALUE must be in the range [1, '+strtrim(ncol, 2) + ']'
      return, 0
    endif
  endelse
  rr = value[0]-1
endif

keys = structure.C0
keystype = size (keys, /type)
data = structure.(rr)
if keyword_set(key) then begin
  typ_key = size(/typ, key)
  ok = where (typ_key eq typecode, nok)
  if (nok eq 0) || (invalid_key[ok[0]]) then begin
    print, '% GETCSV: KEY must be integer, long, or string'
    help, key
    return, 0
  endif
   if typ_key lt 7 and keystype eq 7 then begin
    print, '% GETCSV: KEY - Cannot have an integer key for a string'
    help, key
    return, 0
  endif
  nkey = n_elements(key) 
  if typ_key lt 7 then begin
    if nkey eq 2 then begin
      key = [min(key, max= m), m]
      ok = where ((key[0] le keys) and (key[1] ge keys))
    endif else begin
      ok = where (key[0] eq keys)
      if nkey gt 1 then for i = 1, nkey-1 do begin
        oka = where (key[i] eq keys, nok)
        if (nok gt 0) then ok=[ok, oka]
      endfor
    endelse
  endif else begin
    vart = (keystype eq 7) ? keys : strtrim(keys, 2)
    ok = get_match(key[0], vart)
    if nkey gt 1 then for i = 1, nkey-1 do begin
      oka = get_match(key[i], vart, nel = nel)
      if nel gt 0 then ok =[ok, oka]
    endfor
  endelse

  j = where(ok ge 0, nok)
  if nok eq 0 then begin
    print, '% GETCSV: No values found for key='+string(key)
    return, 0
  endif
  ok = ok[j]
  if nok gt 1 then ok = ok[uniq(ok, sort(ok))]
endif else ok = lindgen(nrow)
nok = n_elements(ok)
if keyword_set(single) and nok gt 1 then begin
  instr = 'Pick the row that you wish returned'
  if keystype eq 7 then $ 
    pick_one, keys[ok], pick, instr=instr, Title = 'GETCSV Row Selection' else $
    pick_one, strtrim(keys[ok], 2), pick, instr=instr, Title = 'GETCSV Row Selection'
  ok = ok[pick]
endif
ier = 0
if n_elements(ok) eq 1 then ok = ok[0]
rows = keys[ok]
return, data[ok]
END
