; ------------------------------------------------------------------------
; $Id: readcsv.pro,v 1.2 2011/03/29 15:37:11 lpmix Exp $
; ------------------------------------------------------------------------
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
;+
; NAME:
;       READCSV
;
; PURPOSE:
;       Read a comma separated value type ASCII file.
;       This code assumes that the dominant variable is in the first column.
;         This may be easily changed with a request to Paul Mix
;
;       CSV file are generated from spreadsheets such as EXCEL or OpenOffice Calc
;       The entire file or selected lines may be read.
;       The format is:
;       Row 1: Column Names - May contain commas and spaces, but do not use double
;                             quotes (").  Names should be unique, but this is not 
;                             mandatory.  Some operations will be easier with unique
;                             column names.  A VERSION or REVISION column can be 
;                             implemented if necessary to non-unique names.
;       Row 2: Data Type (STRING, INTEGER, LONG, FLOAT, DOUBLE)
;              Only the first 3 characters are checked.
;       Row 2a:  Data units (for example, Ampere, MA, MV, Volts, etc.) (May be blank)
;                The units should not contain a comma.
;       Row 3 to NRow: Data Values
;       
;       Note: For consistent operation the first column, the primary key for the row of 
;             data, should be a string, integer, or long data type.  No blanks are allowed 
;             in the first row.
;
; CALLING SEQUENCE:
;
;       READCSV, STRUCTURE [, FILE]
;
; WHERE:
;       STRUCTURE: Output structure with the desired output from the file
;               The structure has the following tag names:
;                 C0, C1, C2, P3, ... CN where the CSV file has N+1 columns
;                 TAG_NAMES: Array of strings with the column headings
;                 FILE_NAME: Complete path of the file.
;                 UNITS:     Units for parameters in each file (Format = 1)
;                 CVS_TYPE: 200
;               NOTE:  If both MATCH and VALUE keywords are not present, the entire
;                      file will be returned.
;       File:   File to be read
;               If missing or undefined, DIALOG_PICKFILE will be used.
;
;	
; KEYWORD PARAMETERS:
;       SKIP:   If the file has several lines of header information which
;               must be skipped, set skip equal to this number of lines.
;         Note: If the number of columns is less than 2, the row will be skipped.
;       INTEGER_DEF: Value for missing INTEGERS
;               Default: See SET_CSVDEF
;       LONG_DEF: Value for missing LONGS
;               Default: See SET_CSVDEF
;       FLOAT_DEF: Value for missing FLOATS and DOUBLES
;               Default: See SET_CSVDEF
;       DOUBLE_DEF: Value for missing DOUBLES
;               Default: See SET_CSVDEF
;       STRING_DEF: Value for missing STRINGS
;               Default: See SET_CSVDEF
;       Note:   Default values initially are 
;               INTEGER_DEF - Default INTEGER value= -9999
;               LONG_DEF   - Default LONG value   = -9999L
;               FLOAT_DEF  - Default FLOAT value  = -1.0E30
;               DOUBLE_DEF  - Default DOUBLE value = -1.0D30
;               STRING_DEF - Default STRING value = ''
;               These may be set by user
;       MINIMUM_ROW: Minimum row number returned.
;               Value must be GE 1 and LE the maximum number of data rows.
;               Row 1 is the first row of data.
;               DEFAULT:  MINIMUM_ROW = 1 
;       MAXIMUM_ROW: Maximum row number returned.
;               Value must be GE 1 and LE the maximum number of data rows.
;               DEFAULT:  MAXIMUM_ROW = M = Number of data rows
;       FORMAT: File format (see PURPOSE:)
;               Format = 0 (or Undefined) File contains rows 1, 2, 3, ... N
;               Format = 1 File contains rows, 1, 2, 2a, 3, ... N
;               NOTE: If first column is of the third row is blank, it is assumed
;                     to be Row 2A
;       KEY:    Return only rows with values specified by this keyword.
;               If MINIMUM_ROW or MAXIMUM_ROW are specified, only selected rows
;               are checked.
;               KEY must be an integer, long, or string variable.
;               If the type for column 1 is a string, then KEY must be a string.
;               KEY may be an array or a scalar.
;               If KEY is an INTEGER or LONG, IDL will look for equal values after
;                 converting the first column to an INTEGER or LONG.
;               If KEY is a STRING, GET_MATCH will be used to find matches.  
;               For example, if a value of 846 is desired, KEY may be specified as
;                 KEY = 846 or KEY ='^846$'
;               If KEY is an array, then 
;                 If KEY is a string, each value will be searched and all unique rows
;                   will be returned.
;                 If KEY is an INTEGER or LONG, then 
;                   If there are 2 values, they will be interpreted as a minimum and 
;                     maximum value for the key.
;                   If there are 3 or more values, they will be considered a list and
;                     each value will be searched.
;       BKEY:   Same as KEY.  Values returned will be added to KEY values and only
;               unique rows returned.       
;       CKEY:   Same as KEY.  Values returned will be added to KEY values and only
;               unique rows returned.       
;       Note:   KEY usage examples
;               To return rows with key values, 836 and 840
;                ..., KEY =[836, 840, 840]
;        or      ..., KEY = 836, BKEY = 840
;        or      ..., KEY = ['^836$', '^840$']
;                 Note: Must have 3 values to be interpreted as a list, but multiple
;                       values deleted.  Strings are always a list.
;
;               To return all rows with key in the range 850 to 869
;                ..., KEY =[850, 869]
;                ..., KEY =['85?', '86?']
;            or  ..., KEY = 850+indgen(20)
;
; OUTPUTS:
;       The output is a structure containing the CSV data from the file.
;
; PROCEDURE:
;       The specified file is opened
;       The header information is read and the number of columns determined.
;       The data is read and the number of rows determined.
;       If KEY is specified, the specified rows are determined.
;       The specified rows or all rows are stored in a structure.
;
; EXAMPLE:
;       Read the parameters for shot 344 from file 'neutrons.csv' into variable STR
;         READCSV, STR, 'neutrons.csv', c=344
;
;       Read the entire file, 'tgs.csv' into structure, allshot
;         READCSV, ALLSHOT, 'tgs.csv'
;
;       Read a file to be determined into structure, a
;         READCSV, a
;
;       Read a file containing filter parameters and obtain values for filter, 'F10V002_09'
;       Store the data in F and select the file from a widget.
;         READCSV, F, c='F10V002_09'
;
; See also (some may not exist!)jjj:
;       DIRCSV  to see a list of tag names
;       FILTCSV to filter a CSV structure - similar to a database query
;       CSV2W   to convert two CSV columns to a waveform array
;       CSVHELP to see the CSV data in a structure
;       WRITECSV to write data in a CSV structure to a file or to 
;               merge data in a CSV structure to an existing CSV file 
;               or to append data in a CSV structure to an existing CSV file
;       GETCSV  to return a column of data (or single value if only one row requested) from
;                a CSV structure
;       PUTCSV  to put values into the CSV structure
;
; MODIFICATION HISTORY:
; 	Written by:	L. P. Mix, Jr., May 12, 2009
;	July, 2009      
;
;-
;

PRO READCSV, structure, filein, skip = skip, Integer_def = integerdef, $
   long_def = longdef, float_def = floatdef, string_def = stringdef, $
   double_def = doubledef, key = key1in,  bkey = key2in, ckey = key3in, $
   ncommamax = ncommamax, format = format, minimum_row = minimum_row, $
   maximum_row = maximum_row

on_error, 1

@csvcommon.dat
if n_elements(initcsv) eq 0 then init_csv

nparams = n_params()
if nparams eq 0 then begin
  print, '% Format: READCSV, STRUCTURE [, FILE] [, SKIP = skip]'
  print, '     [, INTEGER_DEF = int_def,] [, LONG_DEF = long_def]'
  print, '     [, FLOAT_DEF = float_def] [, STRING_DEF = string_def,]'
  print, '     [, DOUBLE_DEF = doub_def]'
  print, '     [, KEY = key] [, BKEY = bkey] [, CKEY = ckey] [, FORMAT = format]'
  print, '     [, MINIMUM_ROW = minimum_row] [, MAXIMUM_ROW = maximum_row]'
  return
endif

if not arg_present(structure) then begin
  print, '% READCSV: Structure variable must be present.'
  return
endif else structure = 0

file = (n_elements(filein) eq 0) ? 0 : filein[0]
if size (file, /type) ne 7 then begin
  file = dialog_pickfile(/must_exist, /read)
  if strlen(file) eq 0 then begin
    print, '% READCSV: No file selected - RETURNING'
    return
  endif
endif
vers = strupcase(strmid(!version.os_family, 0, 3))
if (vers eq 'UNI') then begin
;       get system variable substitution for unix systems
  temp = file
  fulfnam, temp, file
endif 
file = file_expand_path(file)

info =  file_info(file)
if (info.exists + info.read) ne 2b then begin
  print, '% READCSV: File does not exist with read permission'
  print, '           File Name = ', file
  print, '           File Exists =', info.exists
  print, '           Read Permission =', info.read
  return
endif
openr,unit,file, /get_lun, error = error
if error ne 0 then begin
  print, '% READCSV: Unable to open file, Error=', error
  print, '  File: ' + file
  return
endif
ss = ''
if n_elements (skip) gt 0 then begin
  skip = round (skip[0])
  for i = 1, skip do readf, unit, ss
endif

integerdef = (n_elements(integerdef) eq 0) ? integer_def : fix(integerdef[0])
longdef = (n_elements(longdef) eq 0) ? long_def : long(longdef[0])
floatdef = (n_elements(floatdef) eq 0) ? float_def : float(floatdef[0])
doubledef = (n_elements(doubledef) eq 0) ? double_def : double(doubledef[0])
stringdef = (n_elements(stringdef) eq 0) ? string_def : string(stringdef[0])

; read the column headings
nstr = 0
while nstr lt 2 do begin
  readf, unit, ss
  str =  strsplit(ss, ',', /extract, /preserve, count = nstr)
endwhile
ncol = 0L
i = 0L
tag_nam = make_array(nstr, /string)
while i lt nstr do begin
  nam = str_fix(str, i, nstr, ier = ier)
  if ier ne 0 then begin
    print, '% READCSV: Error reading the column headings'
    free_lun, unit
    return
  endif
  tag_nam[ncol++] = nam
endwhile
if ncol lt nstr then tag_nam = tag_nam[0:ncol-1]
if ncol eq 1 then begin
  print, '% READCSV: Must have more than 1 column'
  free_lun, unit
  return
endif
vartypes =['INT', 'LON', 'FLO', 'DOU', 'STR']
typecode =[  2,     3,     4,     5,     7 ]
default ={int:integerdef, lon:longdef, flo:floatdef, dou:doubledef, str:stringdef}
types = make_array(ncol, /integer)
vardefault = make_array(ncol, /integer)
readf, unit, ss
st = byte(ss)
ok = where (st eq 34b, comp = comp, nok)
if nok gt 0 then ss = string(st[comp])
str = strsplit(ss, ',', /extract, /preserve, count = nstr)
if nstr ne ncol then begin
  print, '% READCSV: Number of columns for datatypes is: '+strtrim(nstr, 2)
  print, '% READCSV: Number of column names is: '+strtrim(ncol, 2)
  print, '           Please make them equal and try again.'
  free_lun, unit
  return
endif
for i = 0, ncol-1 do begin
  ok = where (strupcase(strmid(str[i], 0, 3)) eq vartypes, nok)
  if nok gt 0 then begin
    types[i] = typecode[ok[0]]
    vardefault[i] = ok[0]
    if (i eq 0) && ((ok[0] eq 2) or (ok[0] eq 3)) then begin
      print, '% READCSV: First column must be a string, integer or long'
      print, '  VARIABLE TYPE: '+vartypes[ok[0]]
      free_lun, unit
      return
    endif
  endif else begin
    print, '% READCSV: Unable to match variable type code in column: '+ strtrim(i+1, 2)
    print, ' Type: '+str[i]
    print, ss
    help,ss
    free_lun, unit
    return
  endelse
endfor

units = make_array(ncol, /string, value = stringdef)
readf, unit, ss
ss = strtrim (ss, 2)
if keyword_set(format) or (strpos(ss, ',') eq 0) then begin
  st = byte(ss)
  ok = where (st eq 34b, comp = comp, nok)
  if nok gt 0 then ss = string(st[comp])
  str = strsplit(ss, ',', /extract, /preserve, count = nstr) 
  if nstr eq ncol then units = str else begin
    print, '% READCSV: Number of columns for units is: '+strtrim(nstr, 2)
    print, '% READCSV: Number of column names is: '+strtrim(ncol, 2)
    print, '           Please make them equal and try again.'
    free_lun, unit
    return
  endelse
  n= 0L
endif else n=1L
nr = 1024L
input = make_array(nr, /string)
if n eq 1 then input[0] = ss
while not EOF(unit) do begin
  readf, unit, ss
  if strlen(strtrim(ss, 2)) gt 0 then input[n++]=ss
  if n eq nr then begin
    nr += 512
    input =[input, make_array(512, /string)]
  endif
endwhile
free_lun, unit

if n lt nr then input= input[0:n-1]
nrow = n

invalid_key = [0, 0, 1, 1, 0]

nkey1 =  n_elements(key1in)
if (nkey1 gt 0) then begin
  key = key1in
  typ_key = size(/type, key)
  ok = where(typ_key eq typecode, nok)
  if (nok eq 0) || (invalid_key[ok[0]]) then begin
    print, '% READCSV: KEY keyword must be integer, long, or string'
    help, key1in
    return
  endif
  key1 = (typ_key eq 7) ? strtrim(key, 2) : key
  if typ_key lt 7 and types [0] eq 7 then begin
    print, '% READCSV: KEY - Cannot have an integer key for a string'
    return
  endif
  typ_key1 = temporary(typ_key)
endif

nkey2 =  n_elements(key2in)
if (nkey2 gt 0) then begin
  key = key2in
  typ_key = size(/type, key)
  ok = where(typ_key eq typecode, nok)
  if (nok eq 0) || (invalid_key[ok[0]]) then begin
    print, '% READCSV: BKEY keyword must be integer, long, or string'
    help, key2in
    return
  endif
  key2 = (typ_key eq 7) ? strtrim(key, 2) : key
  if typ_key lt 7 and types [0] eq 7 then begin
    print, '% READCSV: BKEY - Cannot have an integer key for a string'
    return
  endif
  typ_key2 = temporary(typ_key)
endif

nkey3 =  n_elements(key3in)
if (nkey3 gt 0) then begin
  key = key3in
  typ_key = size(/type, key)
  ok = where(typ_key eq typecode, nok)
  if (nok eq 0) || (invalid_key[ok[0]]) then begin
    print, '% READCSV: CKEY keyword must be integer, long, or string'
    help, key3in
    return
  endif
  key3 = (typ_key eq 7) ? strtrim(key, 2) : key
  if typ_key lt 7 and types [0] eq 7 then begin
    print, '% READCSV: CKEY - Cannot have an integer key for a string'
    return
  endif
  typ_key3 = temporary(typ_key)
endif
if n_elements(minimum_row) eq 0 then min_row = 1 else $
  min_row = (minimum_row[0] > 1) < nrow
if n_elements(maximum_row) eq 0 then max_row = nrow else $
  max_row = (maximum_row[0] > 1) < nrow
numbr = max_row - min_row + 1

ist = make_array(numbr, /long, value =0L)
nstr = make_array(numbr, /long)
if n_elements(ncommamax) eq 0 then ncommamax = 1.15
mxstr =ceil(ncommamax*ncol) > (ncol+2)
str = make_array(mxstr, numbr, /string)
nc = 0L
var = make_array(numbr, /string)
;
lc_comma = make_array(numbr, /int)
nr= min_row-1
nrstart = nr
;
i = 0L
while nr lt max_row  do begin
  s=  strsplit(input[nr], ',', /extract, /preserve, count = ns) 
  if ns gt mxstr then begin
    print, '% READCSV: STR first dimension must be increased'
    str = ncommamax+0.5
    print, '           ADD keyword: ncommamax = '+strtrim(str, 2)
    return
  endif
  if ns gt ncol then lc_comma[i] = 1
  str[0:ns-1, i] = s
  j =0L
  if strlen(s[0]) eq 0 then nam = '' else begin
    if lc_comma[i] then begin
      nam = str_fix(s, j, ns)
      j = j-1
    endif else nam = s[0]
  endelse
  if strlen(nam) eq 0 then begin
    print, '% READCSV: First column must have a valid key value'
    return
  endif
  var[i] = nam
  ist [i]= j+1
  nstr[i++] = ns
  nr++
endwhile
; release the memory for the file
  input = 0
data_index = lindgen(numbr)
if nkey1 +nkey2+nkey3 gt 0 then begin
  ok1 = (nkey1 gt 0) ? filter_it(var, key, typ_key1, nkey1) : -1
  ok2 = (nkey2 gt 0) ? filter_it(var, key, typ_key2, nkey2) : -1
  ok3 = (nkey3 gt 0) ? filter_it(var, key, typ_key3, nkey3) : -1
  ok =[ok1, ok2, ok3]
  tok = where(ok ge 0, nok)
  if nok gt 0 then begin
    ok = ok[tok]
    oku = ok[uniq(ok, sort(ok))]
    var = var[oku]
    ist = ist[oku]
    nstr = nstr[oku]
    str = str[*,oku]
    numbr = n_elements(oku)

  endif else if nok eq 0 then begin
    print, '% READCSV: No values in column 1 match the keys'
    return
  endif
endif
nam_def ='nam_def_csv_99'
for j = 0L, ncol-1 do begin
  if j gt 0 then begin
    nr = 0L
    var = make_array(numbr, /string)
    while nr lt numbr do begin
      ii = ist[nr]
      if strlen(str[ii, nr ]) eq 0 then begin
        nam= nam_def
        ist[nr]++
      endif else if lc_comma[nr] then begin
        nam =str_fix(str[*, nr], ii, nstr[nr])
        ist[nr] = ii
        if strlen(nam) eq 0 then nam =nam_def
      endif else begin
        nam = str[ii, nr]
        if strlen(nam) eq 0 then nam =nam_def
        ist[nr]++
      endelse
      var[nr++] = nam
    endwhile
  endif
  okdef = where (var eq nam_def, nokdef)
  if nokdef gt 0 then var[okdef]='0'
  if types[j] eq 7 then begin
    ll = strlen(var)-1
    ok = where((strpos(var, '"') eq 0) and $
       (strpos(var, '"', /reverse_search) eq ll), nok)
    if nok gt 0 then for i = 0, nok-1 do $
      var[ok[i]] = strmid(var[ok[i]], 1, ll[i]-1)
  endif else if types[j] eq 2 then var = fix(var) else $
      if types[j] eq 3 then var = long(var) else $
        if types[j] eq 4 then var = float(var) else $
          var = double(var)
  if nokdef gt 0 then var[okdef]=default.(vardefault[j])
  buildstr = (j eq 0) ? create_struct('C'+strtrim(j, 2), var) : $ 
             create_struct(buildstr, 'C'+strtrim(j, 2), var)
endfor
  
endstr = {tag_names: tag_nam, NCOL: ncol, NROW: numbr, File_name: file, $
          units: units, CVS_TYPE: 200}
structure = create_struct(buildstr, endstr)
str =0
end
