; ------------------------------------------------------------------------
; $Id: writecsv.pro,v 1.1 2011/01/07 18:52:01 lpmix Exp $
; ------------------------------------------------------------------------
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
;+
; NAME:
;       WRITECSV
;
; PURPOSE:
;       Write a comma separated value type ASCII file.
;       This code assumes that the dominant variable is in the first column.
;
;       CSV file are generated from spreadsheets such as EXCEL or OpenOffice Calc
;       The data in a structure may be written to a file, the data may be 
;       appended to an existing file or the data may be merged with data in a file
;       The format is:
;       Row 1: Column Names - May contain commas and spaces, but do not use double
;                             quotes (").  Names should be unique, but this is not 
;                             mandatory.  Some operations will be easier with unique
;                             column names.  A VERSION or REVISION column can be 
;                             implemented if necessary to non-unique names.
;       Row 2: Data Type (STRING, INTEGER, LONG, FLOAT, DOUBLE)
;              Only the first 3 characters are checked.
;       Row 2a:  Data units (for example, Ampere, MA, MV, Volts, etc.) (May be blank)
;                The units should not contain a comma.
;       Row 3 to NRow: Data Values
;       
;       Note: For consistent operation the first column, the primary key for the row of 
;             data, should be a string, integer, or long data type.  No blanks are allowed 
;             in the first row.
;
; CALLING SEQUENCE:
;
;       WRITECSV, STRUCTURE, OUTPUT_FILE
;
; WHERE:
;       STRUCTURE: Output structure with the desired output for the file
;               The structure has the following tag names:
;                 C0, C1, C2, C3, ... CN where the CSV file has N+1 columns
;                 TAG_NAMES: Array of strings with the column headings
;                 FILE_NAME: Complete path of the file.
;                 UNITS:     Units for parameters in each file (Format = 1)
;                 CVS_TYPE: 200
;       OUTPUT_FILE:   File to be written
;               This file will be unique unless overwrite is set.
;	
; KEYWORD PARAMETERS:
;       OVERWRITE: If set, the output file may overwrite an existing file
;               Default: The OUTPUT_FILE will be unique.
;       REPLACE: If set equal to a file, then the "REPLACE" file is copied to 
;               OUTPUT_FILE, but rows which have key values equal to values in
;               STRUCTURE are replaced by the STRUCTURE rows.
;               If REPLACE is set but the file cannot be opened, DIALOG_PICKFILE 
;                 will be called.
;               The REPLACE file and the structure must have the same number of
;               columns or one less column than STRUCTURE.
;               The current date is added to the end of the row on replaced rows
;               unless NODATE is set.
;               Any rows which are not represented in the replace file are added
;               to the end of the file.
;       MERGE:  Same as REPLACE but both the row in the file MERGE and the row in
;               STRUCTURE are written to the file.
;               If MERGE is set but the file cannot be opened, DIALOG_PICKFILE 
;                 will be called.
;               The MERGE file and the structure must have the same number of
;               columns or one less column than STRUCTURE.
;       FORMAT: File format (see PURPOSE:)  This refers to the REPLACE or MERGE file
;               Format = 0 (or Undefined) File contains rows 1, 2, 3, ... N
;               Format = 1 File contains rows, 1, 2, 2a, 3, ... N
;               If first column of Row 2a is blank, Format = 1 is assumed.
;       SKIP:   If the MERGE or REPLACE file has several lines of header information which
;               must be skipped, set skip equal to this number of lines.
;       NODATE: If set, the date will not be added to the row in OVERWRITE and REPLACE
;               files.
;
; OUTPUTS:
;       The output is a CSV file containing the STRUCTURE values
;
; PROCEDURE:
;       The specified output file is opened
;       If MERGE or REPLACE is provided those files are opened and the number of
;         Columns checked 
;       The first 3 rows are copied from the MERGE or REPLACE files or generated
;         from STRUCTURE
;       The rows of the file are then written
;
; EXAMPLE:
;       Merge the data is STR with the file neutron.csv
;       Save the old file neutron.csv
;         WRITECSV, STR, 'neutrons.csv', merge ='neutron.csv'
;       Note:  The actual output file will be "neutrons_001.csv"
;              unless that file also exists.
;
;       Read the entire structure, str5, 'tgs.csv'
;         WRITECSV, str5, 'tgs.csv'
;
;       Replace the rows in file, 'zshots.csv', with the corresponsing rows in ss.
;       Overwrite the file zshots.csv
;         WRITECSV,ss, 'zshots.csv', replace = 'zshots.csv', /overwrite 
;       Note: The actual output file is 'zshots.csv'
;
; See also:
;       DIRCSV  to see a list of tag names
;       FILTCSV to filter a CSV structure - similar to a database query
;       CSV2W   to convert two CSV columns to a waveform array
;       CSVHELP to see the CSV data in a structure
;       WRITECSV to write data in a CSV structure to a file or to 
;               merge data in a CSV structure to an existing CSV file 
;               or to append data in a CSV structure to an existing CSV file
;       GETCSV  to return a column of data (or single value if only one row requested) from
;                a CSV structure
;       PUTCSV to put values into the CSV structure
;
; MODIFICATION HISTORY:
; 	Written by:	L. P. Mix, Jr., May 12, 2009
;	July, 2009      
;
;-
;

PRO WRITECSV, structure, filein, skip = skip, format = format, overwrite= overwrite, $
       replace = replace, merge = merge, nodate = nodate

on_error, 1

@csvcommon.dat
if n_elements(initcsv) eq 0 then init_csv

nparams = n_params()
if nparams eq 0 then begin
  print, '% Format: WRITECSV, STRUCTURE [, FILE] [, SKIP = skip]'
  print, '     [, OVERWRITE = overwrite] [, FORMAT = format] [, REPLACE = replace]'
  print, '     [, MERGE = merge] [, NODATE = nodate]'
  return
endif

if nparams lt 2 then begin
  print, '% WRITECSV: Structure and output file must be provided'
  return
endif
if size(structure, /type) ne 8 then begin
  print, '% WRITECSV: CSV structure must be defined.'
  return
endif
tags = tag_names(structure)

ok = get_match('CVS_TYPE', tags, nel = nel)
if nel ne 1 then begin
  print, '% WRITECSV: This structure is not a CVS_STRUCTURE'
  return
endif
if structure.CVS_TYPE ne 200 then begin
  print, '% WRITECSV: Expected CVS_TYPE of 200; value = '+strtrim(structure.cvs_type, 2)
  return
endif

if n_elements(filein) eq 0 then begin
  print, '% WRITECSV: Output file name must be defined'
  return
endif else file = filein[0]

vers = strupcase(strmid(!version.os_family, 0, 3))
if (vers eq 'UNI') then begin
;       get system variable substitution for unix systems
  temp = file
  fulfnam, temp, file
endif 
fileorig = file_expand_path(file)

file = find_newfile(fileorig, /quiet)

openw,unit,file, /get_lun, error = error
if error ne 0 then begin
  print, '% WRITECSV: Unable to open file, Error=', error
  print, '  File: ' + file
  return
endif
ncol = structure.ncol
nrow = structure.nrow
vartypes =['INT', 'LON', 'FLO', 'DOU', 'STR']
typecode =[  2,     3,     4,     5,     7 ]
keys = structure.(0)
dateit = (keyword_set(nodate)) ? 0 : 1
thindate = thin_date()

if keyword_set(merge) or keyword_set(replace) then begin 
  if keyword_set(merge) then begin
    if size(/type, merge) eq 7 then begin
      fileold = merge
    endif else begin
      title = 'Select file to merge'
      fileold = dialog_pickfile(/must_exist, title = title)
    endelse
  endif else begin
    if size(/type, replace) eq 7 then begin
      fileold = replace
    endif else begin
      title = 'Select file to replace'
      fileold = dialog_pickfile(/must_exist, title = title)
    endelse
  endelse
  if (vers eq 'UNI') then begin
    lcunix = 1
;   get system variable substitution for unix systems
    temp = fileold
    fulfnam, temp, fileold
  endif  
  pfile = file_expand_path(fileold)
  info = file_info(pfile)
  if (info.exists + info.read) ne 2b then begin
    print, '% REASC: File does not exist with read permission'
    print, '         File Name = ', fname
    print, '         File Exists =', info.exists
    print, '         Read Permission =', info.read
    ier = -2
    return
  endif
  openr,unitr ,pfile, /get_lun
  
  ss = ''
  if n_elements (skip) gt 0 then begin
    skip = round (skip[0])
    for i = 1, skip do begin
      readf, unitr, ss
      printf, unit, ss
    endfor
  endif
  nstr = 0
  while nstr lt 2 do begin
    readf, unitr, ss
    str =  strsplit(ss, ',', /extract, /preserve, count = nstr)
    printf, unit, ss
  endwhile
  if abs (nstr - ncol) gt 1 then begin
    print,  '% WRITECSV: Number of columns in structure must be within 1 of the basis file.'
    free_lun, unitr
    free_lun, unit
    file_delete, file
    return
  endif else ncolr = nstr
  readf, unitr, ss
  printf, unit, ss
  readf, unitr, ss
  if keyword_set(format) then begin
    printf, unit, ss
    readf, unitr, ss
  endif else begin
    str =  strsplit(ss, ',', /extract, /preserve, count = nstr)
    if nstr lt ncolr then begin
      printf, unit, ss
      readf, unitr, ss
    endif
  endelse
  lcread = make_array(nrow, value = 0)
  lc =0
   
  while not EOF(unitr) do begin
    if lc then readf, unitr, ss
    lc =1
    j = strpos(ss, ',')
    if j eq 0 then begin
      print, '% WRITECSV: Cannot have a row with no key value in column 1'
      free_lun, unitr
      free_lun, unit
      file_delete, file
      return
    endif
    k = get_match(strmid(ss, 0, j), keys, nel=nel)
    if nel gt 1 then begin
      first = '^'+strmid(ss, 0, j)+'$'
      k = get_match(strmid(ss, 0, j), keys, nel=nel)
    endif
    if nel eq 1 then begin
      if keyword_set(merge) then printf, unit, ss
      ss = str2string(structure, k[0])
      if dateit then ss = ss+','+thindate
      printf, unit, ss
      lcread[k[0]] = 1
    endif
  endwhile
  ok = where (lcread eq 0, nok)
  for i = 0, nok-1 do begin
    ss = str2string(structure, ok[i])
    if dateit then ss = ss+','+thindate
    printf, unit, ss
  endfor
  free_lun, unit
  free_lun, unitr
endif else begin
  nam = structure.tag_names
  ss = ''

  for i = 0L, ncol-1 do begin
    if i gt 0 then ss = ss+','
    var = strtrim(nam[i], 2)
    if  (strpos(var, ',') ge 0) then var = '"'+var+'"'
    ss = ss + var
  endfor
  printf, unit, ss
  ss = ''

  for i = 0L, ncol-1 do begin
    if i gt 0 then ss = ss+','
    var = size(/type, structure.(i))
    ok = where (var eq typecode,nok)
; 
    if nok eq 0 then begin
      print, '% WRITECSV: Bad variable type in CSV structure'
      help, i, structure.(i)
      free_lun, unit
      return
    endif
    ss = ss + vartypes[ok[0]] 
  endfor
  printf, unit, ss
  ss = ''
  
  nam = structure.units
  for i = 0L, ncol-1 do begin
    if i gt 0 then ss = ss+','
    var = strtrim(nam[i], 2)
    if  (strpos(var, ',') ge 0) then var = '"'+var+'"'
    ss = ss + var
  endfor
  printf, unit, ss
  for i = 0L, nrow-1 do printf, unit, str2string(structure, i)
  free_lun, unit
endelse

if keyword_set(overwrite) then if file ne fileorig then $
  file_move, file, fileorig, /overwrite

end
