; ------------------------------------------------------------------------
; $Id: fld2exo.pro,v 1.2 2009/09/02 16:32:13 lpmix Exp $
; ------------------------------------------------------------------------
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
;+
; NAME:
;       FLD2EXO
;
; PURPOSE:
;       Convert a series of QS field datasets to an exodus file
;       The exodus format will allow visualization with ParaView or Ensight
;       Two modes exist:  
;       1.  A group of PFF datasets with identical space/time parameters
;           are converted to a Exodus file
;       2.  All datasets in a PFF file are converted to an Exodus file with
;           one dataset per file
;
; CATEGORY:
;       Analysis & Visualization
;
; CALLING SEQUENCE:
;       FLD2EXO [, OUTFILE] [, INFILE]
;
; OPTIONAL INPUTS:
;       OUTFILE: File name for the output Exodus file
;                The default will be used if OUTFILE is undefined or a number.
;                The default file name depends on keywords.
;                IF ALL_SNAPS is set, the file name will be "NAME.exo" where
;                  NAME is the name of the snapshot - OUTFILE is ignored.
;                  (First 32 characters in dataset comment)
;                IF ALL_SNAPS is not set, the name will be qssnp.exo.
;                In all cases, if the name exists, the name will be padded with 
;                  a number to avoid eliminating previous files.
;                  For example if qssnp.exo exists then the outfile is qssnp_001.exo
;
;       INFILE:	 Data file containing the PFF snapshots.
;                If INFILE is undefined and STRUCTURES is undefined, the default is
;                  qssnp.pff.  If this file does not exist, DIALOG_PICKFILE is called.
;                If "Bounding Box" is the first dataset type, it is skipped.
;                If the Application dataset type is 1 or the pff type is 6 (IFL) the first 
;                  dataset is skipped.
;                Enter a number, for example, zero, to go to a widget to select the file.
;	
; KEYWORD PARAMETERS:
;       STRUCTURES: This is a list of structure array numbers to be converted to 
;                an exodus file or a string that will provide a list of structure array
;                numbers when passed to STRVALID.  The structures are assumed to be from a
;                single QS snapshot command and are in time order.
;                Multiple strings (an array of strings) may not be passed.
;
;       ALL_SNAPS: All snapshots in INFILE are converted to a single exodus file.
;                OUTFILE is ignored if ALL_SNAPS is set.
;                The output file names are the name of the snapshot plus ".exo"
;                For example: dataset comment = "e:phinorm1"
;                             goes into file = "e_phinorm1.exo"
;                File name will be limited to letters, numbers, period, and underscore. 
;                Byte values = 46 or (48 to 57) or (65 to 90) or 95 or (97 to 122)
;                Underscore replaces all other values.
;                As noted above, if a file already exists with the selected outfile,
;                "_001" is added to the name to generate a unique file name.
;
;       DATASETS: Array of one or more strings that define the datasets that are to
;                be written to OUTFILE from INFILE.
;                If DATASETS is not present, a widget will be provided to pick the 
;                  datasets for OUTFILE if the dataset contains more than 1 dataset.
;                  If the file contains only one dataset, it will be written.  
;                  All DATASETS written to a single file muct have the same spacial
;                  coordinates and the same output times.
;                Note: Get_Match will be used.
;                NOTE: Vectors are saved as three vectors with _x, _y, and _z added to 
;                      the three components
;                Note: Type, size, range, and locv structure tags are checked to insure
;                      that all datasets have the same spatial/temporal coordinates.
;                      The number of time steps must also be equal, but the current 
;                      version does not check specific time values very well.
;                      See TDELTA, XDELTA
;        TDELTA:  Fraction of the time range to be used as an error check.
;                 Default:  TDELTA = 1e-4
;                 Values of the time for multiple datasets in a file must agree to 
;                 ALLOWED_VARIANCE= (TN-T1)*TDELTA where times are T1, T2, ... TN
;        XDELTA:  Fraction of the spacial range to be used as an error check.
;                 TOTAL(ABS((Range -RANGE_SAVE)/(RANGE_NORM))) lt XDELTA
;                   where range_norm =  TOTAL(RANGE[1, *, 0]- RANGE[0, *, 0])
;                   and RANGE_SAVE is the range for the first time step of the first dataset
;                 Default: XDELTA = 2E-5
;                      
;
; RESTRICTIONS:
;	Exodus file require the same spatial values and the same time steps for all 
;       variables in a file
;
; PROCEDURE:
;       If STRUCTURES is specified, no file is opened.
;         If not, a PFF file is opened.
;       If ALL_SNAPS is set, every dataset in the PFF file will be written to 
;         a separate Exodus file.
;       If DATASETS is set, these snapshots will be written to a single.
;         If DATASETS is not set, a widget will allow the user to select the snapshots
;         that they want written to OUTFILE.
;     For the first time step in a file:
;       Determine the unique points in the problem and the connectivity.
;       The first time step for the structure is used for the grid and 
;         for the dataset name.  Subsequent structures are checked for number
;         of blocks, size and limits.
;       If cylindrical geometry, convert to cartesian coordinates.
;       Set up the Exocus header with variable names, number of blocks, etc.
;       Output the coordinates and connectivity
;       Step through the datasets and write the data for each time steps.
;       
; EXAMPLE:
;       Convert all datasets in a file to be selected to exodus files.
;            FLD2EXO, 0, 0, /all
;
;       Convert all datasets in qssnp.pff to exodus files.
;            FLD2EXO, 0, 'qssnp.pff', /ALL
;        or
;            FLD2EXO, /ALL
;            ; Since INFILE is undefined, the default is qssnp.pff- if it exists
;             
;       Convert the datasets 'e' in a file to be selected to a file, para_in.exo
;       Assume the the file only contains snapshots called 'e'
;            FLD2EXO, 0, 'para_in.exo'
;
;       Convert qssnp.pff to qssnp.exo for dataset "e:phcb"
;            FLD2EXO, dataset = 'e:phcb'
;
;       Convert all pff datasets containing "phinorm" to an exodus file dataout.exo
;            FLD2EXO, 'dataout.exo', dataset ='phinorm'
;
;       Convert structure arrays, 100 to 175 to an exodus file, out.exo
;            FLD2EXO, 'out.exo', struct = 100+indgen(76)  
;
; MODIFICATION HISTORY:
; 	Written by:	L. P. Mix, Jr., February 20, 2009
;
;-

 pro FLD2EXO, fileoutin, filein, structures = strarrayin, all_snaps = all_snaps, $
            datasets= datasets, tdelta = tdelta, xdelta = xdelta

on_error,1
@write_exo_header_com.dat
  
  fileout = (n_elements (fileoutin) eq 0) ? 'qssnp.exo' : fileoutin
  if size(/type, fileout) lt 5 then fileout= 'qssnp.exo'
  fileout = strtrim(fileout, 2)
  if strlen(fileout) lt 2 then fileout= 'qssnp.exo'

  if n_elements(tdelta) eq 0 then tdelta = 1.0e-4
  if n_elements(xdelta) eq 0 then xdelta = 2.0e-5

  if n_elements (strarrayin) gt 0 then begin
    lcopenpff =0
    n = n_elements(strarrayin)
    if (n gt 1) and (size(/type, strarrayin) eq 7) then begin
      print,  '% FLD2EXO: Only a single string is accepted by the keyword STRUCTURES' 
      help,  strarrayin
      print, strarrayin
      return
    endif
    strarray = strvalid(strarrayin, /valid, /quiet, np = np)
    if np lt n then begin
      print, '% FLD2EXO: Some structures are not valid'
      print, '     The following will be written:', strarray
    endif
    com = strcomment(strarray[0])
    ok = strpos(com, '- t=')
    name = strtrim (strmid(com, 0, ok), 2)
    nsig = 1
  endif else begin
    if n_elements (filein) eq 0 then fname = 'qssnp.pff' else fname = filein
    if size(fname, /type) eq 7 then begin
      info = file_info(fname)
      if (info.exists + info.read) ne 2b then begin
        print, '% FLD2EXO: File does not exist with read permission'
        print, '           File Name = ', fname
        print, '           File Exists =', info.exists
        print, '           Read Permission =', info.read
        fname = ''
      endif
    endif
    openpff, fname, ndset = ndset, error = error
    if error ne 0 then begin
      Print, '% FLD2EXO: Unable to open file.'
      return
    endif
    lcopenpff = 1
    get_dir, comment= comment, type = type, number = ndset, /no_trim
    lcbb = 0
    if strpos(type[0], 'Bounding Box') ge 0 then begin
      comment = comment[1:*]
      ndset = ndset-1
      lcbb = 1
    endif else begin
      readhdr, 1, /quiet, pfftyp = pfftyp, aptype = aptype
      if pfftyp eq 6  or aptype eq 1 then begin
        comment = comment[1:*]
        ndset = ndset-1
        lcbb=1
      endif
    endelse
    if ndset eq 0 then begin 
      print, '% FLD2EXO: No valid datasets in this file.'
      if lcopenpff then closepff
      return
    endif
    com1 = strtrim(strmid(comment, 0, 32), 2)
    com1 = strlowcase(com1)
    com2 = strtrim(strmid(comment, 32), 2)
    com2 = strlowcase(com2)
    unq = uniq(com1, sort(com1))
    nunq = n_elements(unq)
    nsig = nunq
    names =  com1[unq]
    if keyword_set(all_snaps) then datasets = names else $
      if n_elements(datasets) gt 0 then begin
      nsig = n_elements(datasets)
      i = 0
      nlist = 0L
      for j = 0L, nsig-1 do begin
        ok = get_match(datasets[j],  names , nelem = nelem)
        if nelem gt 0 then begin
          list = (nlist eq 0) ? ok : [list, ok]
          nlist += nelem
        endif else print, '% FLD2EXO: No Match found for: ' + datasets[j]
      endfor
      if nlist eq 0 then begin
        print, '% FLD2EXO: No Match found for value of datasets'
        print, '           No file written'
        if lcopenpff then closepff
        return
      endif
      list = list[uniq (list, sort(list))]
      datasets = names[list]
      nsig = n_elements(list)
    endif else begin
      if nunq gt 1 then begin
        instr= ['Pick the signals to write to Exodus file', $
              'Signals must have same coordinates and times']
        sel = make_array(/long, value = 0, nunq)
        wselect_item, names, selected = sel, instruct = instr
        ok = where (sel eq 1, nsig)
        if nsig eq 0 then begin
          print, '% FLD2EXO: No signals selected --- RETURNING'
          if lcopenpff then closepff
          return
        endif
        datasets = names[ok]
      endif else begin
        nsig = 1
        datasets = names[0]
      endelse
    endelse   
  endelse

; number of files to write  
  nfile = keyword_set(all_snaps) ? nsig : 1
  work = get_maxstr() -1
; number of signals per file
  nsigpf = keyword_set(all_snaps) ? 1 : nsig
for ifile = 0, nfile-1 do begin
    clabel = datasets[ifile]
    if keyword_set(all_snaps) then begin
      clab = clabel
      byclab = byte(strupcase(clab))
      ok = where ((byclab eq 46b) or (byclab eq 95b) or $
          ((byclab ge 65b) and (byclab le 90b)) or $
          ((byclab ge 48b) and (byclab le 57b)), comp = comp, ncomp = ncomp)
      if ncomp gt 0 then begin
        clab = byte(clab)
        clab[comp]= 95B
        clab = string(clab)
      endif
      fileout = clab + '.exo'
    endif
    fileout = find_newfile(fileout, /quiet)
    if lcopenpff then begin
      ok = get_match ( clabel, com1, nel = nel)
      readstr, work, ok[0]+1+lcbb
    endif else work = strarray[0]
    s2i, s, work
    if s.type ne 1 then begin
      print, '% FLD2EXO: Only type 1 structures are allowed.'
      if lcopenpff then closepff
      return
    endif
    if size (s, /type) lt 8 then begin
      print, '% FLD2EXO: Structure must be defined'
      if lcopenpff then closepff
      return
    endif

    if s.ndim ne 3 then begin
      print, '% FLD2EXO: This version requires 3D data'
      if lcopenpff then closepff
      return
    endif
;   save parameters for file to test if more than one dataset per file.
    ss = s.size
    size_save = ss
    range_save = s.range
    range_norm =  total(s.range[1, *, 0]- s.range[0, *, 0])
    locv_save = s.locv
    nblock = s.block
    loc_nodes = make_array(nblock+1, value =0L)
    loc_elem = loc_nodes
;
    nvect = s.nvect
    for i = 1L, nblock do loc_nodes[i]=ss[0,i-1]*ss[1,i-1]*ss[2,i-1]+loc_nodes[i-1]
    for i = 1L, nblock do loc_elem[i]= $
      ((ss[0,i-1]-1) > 1L)*((ss[1,i-1]-1) > 1L)*((ss[2,i-1]-1) > 1L)+loc_elem[i-1]
    nodes = make_array(3, loc_nodes[nblock])
    if s.two eq 0 then begin
      lcshape = 'HEX'
      nconnect = 8L
    endif else begin
      lcshape = 'SHELL'
      nconnect = 4L
    endelse
    blo_type =make_array(nblock, value = lcshape)
    connect = make_array(nconnect, loc_elem[nblock], /long)
    for nb = 1L, nblock do begin
      x= get_array(s, 1, nb)
      y= get_array(s, 2, nb)
      z= get_array(s, 3, nb)
      inode = loc_nodes[nb-1]
      nx = ss[0, nb-1]
      ny = ss[1, nb-1]
      nz = ss[2, nb-1]
      nxy = nx*ny
      for iz = 0, nz-1 do for iy = 0, ny-1 do for ix = 0, nx-1 do begin
        nodes[0, inode] =[x[ix], y[iy], z[iz]]
        inode += 1
      endfor
      inode = loc_elem[nb-1]
      if s.two eq 0 then begin
        xcon = [0, 1, 1, 0, 0, 1, 1, 0]
        for iz = 0, nz - 2 do begin
          zcon = iz*nxy + loc_nodes[nb-1] + [0, 0, 0, 0, nxy, nxy, nxy, nxy]
          for iy = 0, ny - 2 do begin
            ycon = iy*nx + [0, 0, nx, nx, 0, 0, nx, nx] +zcon
            for ix = 0, nx - 2 do begin
              connect[0, inode] = ix + xcon + ycon
              inode += 1
            endfor
          endfor
        endfor
      endif else begin
        if s.two eq 1 then begin
          xcon = [0, 1, 1, 0]
          for iz = 0, nz-2 do begin
            zcon = iz*ny +loc_nodes[nb-1] +xcon +[0, 0, nxy, nxy]
            for iy = 0, ny-2 do begin
              connect[0, inode]= zcon + iy
              inode += 1
            endfor
          endfor
        endif else if s.two eq 2 then begin
          xcon = [0, 1, 1, 0]
          for iz = 0, nz-2 do begin
            zcon = iz*nxy +loc_nodes[nb-1] +[0, 0, nxy, nxy] +xcon
            for ix = 0, nx-2 do begin
              connect[0, inode]= zcon + ix
              inode += 1
            endfor
          endfor
        endif else begin
          xcon = [0, 1, 1, 0]
          for iy = 0, ny-2 do begin
            zcon = iy*nx +loc_nodes[nb-1] +xcon +[0, 0, nx, nx]
            for ix = 0, nx-2 do begin
              connect[0, inode]= zcon + ix
              inode += 1
            endfor
          endfor
        endelse
      endelse
    endfor
    get_uniq_nodes, nodes, newcoord, ind
    if s.spare[0] eq 2 then begin
      ctheta = reform(cos(newcoord[1, *]))
      stheta = reform(sin(newcoord[1, *]))
      newcoord[1, *] = newcoord[0, *]*stheta
      newcoord[0, *] = newcoord[0, *]*ctheta
      ctheta =0
      stheta =0
    endif

    newcoord = transpose(newcoord)
    ss = size(newcoord)
    num_nodes = ss[1]
    undefine, cdfid
    cdfid = ncdf_create(fileout, clobber = clobber)
    if size(/type, cdfid) ne 3 then begin
      print, '% FLD2EXO: Unable to open NCDF file'
      if lcopenpff then closepff 
      return
    endif
    title = 'IDL/ExodusII: '+systime(0)
  
    num_elem= loc_elem[1:nblock]-loc_elem[0:nblock-1]
    
    if nsigpf gt 1 then begin
      nvart = make_array(nsigpf, value = 0)
      nvart[0] = nvect
      for iv = 1, nsigpf-1 do begin
        ok = where (datasets[iv] eq com1, nok)
        readhdr, ok[0]+1+lcbb, pfftype = pfftype, /quiet
        nvart[iv] = (pfftype eq 4) ? 3 : 1
      endfor
      nvar = round(total(nvart))
    endif else begin
      nvar = nvect
      nvart = nvar
    endelse
    
    varname = make_array(nvar, /string)
    i = 0L
    for iv = 0, nsigpf-1 do begin
      if nvart[iv] eq 1 then varname[i] = datasets[ifile+iv] else begin
        varname[i] = datasets[ifile+iv]+'_x'
        varname[i+1] = datasets[ifile+iv]+'_y'
        if nvart[iv] eq 3 then varname[i+2] = datasets[ifile+iv]+'_z'
      endelse
      i += nvart[iv]
    endfor

    write_exo_header, cdfid, nblock = nblock, nnode = num_nodes, nelem= num_elem, $
      blo_type = blo_type, title = title, numnodevar = nvar, numelemvar=0, ier = ier
    if ier ne 0 then begin
      print,  '% FLD2EXO: Error writing Exodus header.'
      if lcopenpff then closepff 
      return
    endif

    ncdf_varput, cdfid, coordxid, newcoord[*, 0]
    ncdf_varput, cdfid, coordyid, newcoord[*, 1]
    ncdf_varput, cdfid, coordzid, newcoord[*, 2]
    coord_names = make_array(3, /string)

    coord_names[0] = 'X axis'
    coord_names[1] = 'Y axis'
    coord_names[2] = 'Z axis'
;   ind += 1
;    print, ' IND range: ',  min(ind, max = m), m
    ncdf_varput, cdfid, coordnamid, coord_names
    ncdf_varput, cdfid, nodenamid, varname
;   help, varname & print, varname
    for i = 1L, nblock do $
      ncdf_varput, cdfid, connectid[i-1], ind[connect[*, loc_elem[i-1]:loc_elem[i]-1]]+1L
    if lcopenpff eq 1 then ok = where(names[0] eq com1, ntimes) else $
      ntimes = n_elements(strarray)
;   before we go to the trouble to write everything check to determine if all 
;   datasets groups have the same number of time steps.

    if nsigpf gt 1 then begin
      ntim= make_array(nsigpf, /long)
      ntim[0] = ntimes
      for i = 1, nsigpf-1 do begin
        ok  = where(varname[i] eq com1, ntime)
        ntim[i] = ntime
      endfor
      if (max(ntim, min = m)-m) gt 0 then begin
        print, '% FLD2EXO: Number of times vary for datasets in the same file'
        print, '  Datasets: ', names
        print, '  Number of times: ',  ntim
        if lcopenpff eq 1 then closepff
        return
      endif
    endif
  loopvar =0L
  times = make_array(ntimes)
; Loop over different signals in the file
  for loop = 0, nsigpf-1 do begin
    if loop eq 1 then delta = (times(ntimes-1) - times[0]) * tdelta
    variable_name = datasets[loop + ifile]
    if lcopenpff then str = where(variable_name eq com1) + lcbb +1 
;   Loop over times for each signal
    for nts = 0, ntimes-1  do begin
      if lcopenpff then readstr, work, str[nts]  else xfrstr, strarray[nts], work
      s2i, s, work
;     Check the parameters      
      if total(abs(s.range - range_save))/range_norm gt xdelta then begin
        print,  '% FLD2EXO: Range does not match'
        help, loop, nts
        if lcopenpff then closepff
        return
      endif
      
      if total(abs(s.size - size_save)) gt 0 then begin
        print,  '% FLD2EXO: Range does not match'
        print,  ' size: ', s.size
        print,  ' size_save: ', size_save
        help, loop, nts
        if lcopenpff then closepff
        return
      endif
      if total(abs(s.locv - locv_save)) gt 0 then begin
        print,  '% FLD2EXO: LOCV does not match'
        print,  ' locv: ', s.locv
        print,  ' locv_save: ', locv_save
        help, loop, nts
        if lcopenpff then closepff
        return
      endif
      c = s.clab
      jj = strpos(c, 't=')
      reads, strmid(c, jj+2), time
      if loop eq 0 then times[nts] = time else $
        if abs(times[nts] - time)  gt delta then begin
          print,  '% FLD2EXO: TIMES do not match'
          print,  ' Times: ', times
          print,  ' Time: ', time
          help, loop, nts
          if lcopenpff then closepff
          return
      endif
      nvar = s.nvect
      farray = make_array(num_nodes, nvar )
      if nvar eq 1 then farray [ind, nvar-1] = get_array(s, -1, /all) else begin
        farray[ind, 0] = get_array(s, -1, /all)
        farray[ind, 1] = get_array(s, -2, /all)
        if nvar eq 3 then farray[ind, 2] = get_array(s, -3, /all)
      endelse
;  print ,loop, time, nvar, nts & hak, /mes
      if loop eq 0 then ncdf_varput, cdfid, timewholeid, time, offset = (nts)
      for i = 0, nvar-1 do ncdf_varput, cdfid, nodevarid[i+loopvar], farray[*, i], offset = [0, nts]
    endfor
    loopvar += nvar
  endfor
  ncdf_close, cdfid
endfor
;   print, times
    if lcopenpff eq 1 then closepff, 0
    end
