; ------------------------------------------------------------------------
; $Id: get_nodes.pro,v 1.2 2007/06/06 19:40:41 lpmix Exp $
; ------------------------------------------------------------------------
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
;+
; NAME:
;       GET_NODES
;
; PURPOSE:
;       This function returns the node number(s) for nodes closest to a point.
;
; CALLING SEQUENCE:
;
;       NODES = GET_NODES(POSITION [, TIME])
;
; OUTPUT:
;       NODES:  Node numbers from the file.  NODES equal -1 if no nodes available.
;               Nodes are numbered from 1 to NUM_NODES.
;
; INPUTS:
;       POSITION: Spatial position where closest node is requested.
;       TIME:     Optional input to specify a time other than the first time.
;                 The position at initial time is stored in a common block 
;                 and is much faster to access.  If INDEX is set, TIME is the 
;                 time step which must be in the range 1 to N_TIMES.
;
; KEYWORD PARAMETERS:
;       INDEX:  If set, TIME is the time index from 1 to N_TIMES.
;       VALUES: Actual spatial position of the node(s) returned.
;               Size is [Num_nodes, ndim]
;       NNODES: Number of Nodes returned.
;               Default is 1 - Closest Node to Position.
;       RADIUS:	If present and not zero, this specifies a radius for finding
;               nodes.  All nodes with a position less than RADIUS from 
;               POSITION are returned.
;       The following are ignored if RADIUS is set.
;       XDelta: Range of X-values to use in returning nodes.
;               Values returned have X such that:
;               POSITION[0]-0.5*Xdelta LE X LE POSITION[0]+0.5*Xdelta
;       YDelta: Range of Y-values to use in returning nodes.
;               Values returned have Y such that:
;               POSITION[0]-0.5*Ydelta LE Y LE POSITION[0]+0.5*Ydelta
;       ZDelta: Range of Z-values to use in returning nodes.
;               Values returned have Z such that:
;               POSITION[0]-0.5*Zdelta LE Z LE POSITION[0]+0.5*Zdelta
;
; RESTRICTIONS:
;       Current Exodus II file is searched.  Use SETEXO to set desired file.
;
; EXAMPLE:
;       Find the node number and true position (val) of the node closest to 
;       the location: [.1, .2, .1].  Save the node in NN.
;         nn = GET_NODES( [.1, .2, .1], value = val )
;
;       Find the node number and true position (val) of the node closest to 
;       the location: [.1, .2, .1] at a time of 100e-9.  Save the node in N100.
;         n100 = GET_NODES( [.1, .2, .1], 100.0e-9, value = val )
;       Note:  No interpolation is done.  Time closest to 100e-9 us used.
;
;       Find the node numbers and true position (val) of the node within 5 cm of 
;       the location: [.1, .2, .1] at a time of 100e-9.  Save the nodes in N05.
;         n05 = GET_NODES( [.1, .2, .1], 100.0e-9 , r= 0.05, val = val)
;
;       Find the node numbers and true position (val) of the nodes within 5 cm of 
;       the location: x = 0.1 at a the initial time.  Store the numbers in xp1
;         xp1 = GET_NODES( [.1, 0, 0], x= 0.05, val= val)
;
; MODIFICATION HISTORY:
; 	Written by:	L. P. Mix, October 6, 2005
;
;-

  Function GET_NODES, position, tvalue, values = values, nnodes= nnodes, $
              radius = radius, Xdelta = Xdelta, ydelta = ydelta, $
              zdelta = zdelta, index = index

  on_error, 1
; add common block
@exoduscom.dat
;
; check to see if EXODUS has been initialized
;
  nodes = -1
  nnodes = 0
  if n_elements(init_exo) eq 0 then begin
    print, '% READ_EXO: No Exodus files have been opened.'
    return, nodes 
  endif
  if exo_id lt 0 then begin
    print, '% GET_NODES: No Exodus files are open.'
    return, nodes
  endif
  
  if n_params() lt 1 then begin
    print, '% Format: Nodes = GET_NODES( POSITION [, TVALUE] [, /INDEX] '
    print,'       [, VALUES = values] [, NNODES = nnodes] [, RADIUS = radius]'
    print,'       [, XDELTA = xdelta] [, YDELTA = ydelta] [, ZDELTA = zdelta])'
    return, nodes
  endif 

  if n_elements(tvalue) eq 0 then coord = coord_node else $
    coord= read_exo(/grid, tvalue, index= index)
  if n_elements(coord) lt num_nodes then begin
    Print, '% GET_NODES: Unable to obtain nodes.'
    return, nodes
  endif
  ss = size(coord)
  ndim = num_dim[exo_id]
  if (ss[0] ne 2) || (ss[1] ne num_nodes) || (ss[2] ne ndim) then begin
    print, '% GET_NODES: Coordinates are the wrong size'
    help, coord, num_nodes, ndim
    return, nodes
  endif
  if n_elements(position) ne NDIM then begin
    print, '% GET_NODES: Position must have ', strtrim(ndim, 2), ' elements'
    help, position
    return, nodes
  endif
  distance = make_array(num_nodes, ndim)
  distance[*, 0] = (coord[*, 0] - position[0])^2
  distance[*, 1] = (coord[*, 1] - position[1])^2
  if ndim gt 2 then $
    distance[*, 2] = (coord[*, 2] - position[2])^2
  
  if n_elements(radius) gt 0 then begin
    min_d  = radius[0]^2
    distan = total(distance, 2)
    nodes = where(distan le min_d, nnodes)
    if nnodes eq 0 then begin
      print, '% GET_NODES: No nodes within the required distance'
      print, '             Closest node returned.'
      junk = min(distan, nodes)
      nnodes = 1
    endif
    values =reform(coord[nodes, *])
    return, nodes+1
  endif  
  nok = 0
  lcmin = 0
  lcx = [0, 0, 0]
  if n_elements(xdelta) gt 0 then begin
    lcx[0] = 1
    ok = where(distance[*, 0] lt xdelta[0]^2, nok)
    if nok eq 0 then lcmin = 1
  endif else ok = lindgen(num_nodes)
  if (lcmin eq 0) and (n_elements(ydelta) gt 0) then begin
    lcx[1] = 1
    oky = where(distance[ok, 1] lt ydelta[0]^2, nok)
    if nok eq 0 then lcmin = 1 else ok = ok[oky]
  endif
  if (lcmin eq 0) and (n_elements(zdelta) gt 0) then begin
    lcx[2] = 1
    okz = where(distance[ok, 2] lt zdelta[0]^2, nok)
    if nok eq 0 then lcmin = 1 else ok = ok[okz]
  endif
  if (lcmin eq 1) || (total(lcx) eq 0) then begin
    junk = min(total(distance, 2), nodes)
    nnodes = 1
    values = reform(coord[nodes, *])
    return, nodes+1
  endif
  values = reform(coord[ok, *])
  nnodes = nok
  return, nodes+1
  end
