; ------------------------------------------------------------------------ 
; $Id: openexo.pro,v 1.4 2008/02/27 22:47:13 lpmix Exp $
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
;+
;  Open a Exodus file.  The current file pointer is set to this file id.
;  format: OPENEXO [, File] [, Fileid ] [, NDSET =ndset] [, WRITE = write]
;                  [, PATH = path] [, FILTER = filter] [, HELP = help]
;                  [, FILEID = fileid] [, QUIET = quiet] [, /VERBOSE]
;  where:
;    File   - Name of the Exodus file to be opened.  If file is zero, 
;               Dialog_Pickfile will be used to obtain the file.
;             The default file extension is ".exo". 
;             If file is omitted or undefined, PICKFILE will be used to select
;             a file. The initial path in PICKFILE is the current directory or
;             the PATH to the last file selected.
;             NOTE: PATH and FILTER may be used to set the initial path and
;                   filter for PICKFILE.
;             NOTE: FULFNAM procedure is used to handle environment variables
;                   See FULFNAM for details
;    FileID - user specified file id 
;             If fileid is specified, it will be used as the file id.  If 
;             fileid is not provided or is zero, a value will be 
;             provided; default value is the first unused integer greater
;             than 0.  If fileid is provided but has been used for another 
;             file the program will inform the user and terminate.
;    WRITE  - Set to open the file for both read and write
;    HELP   - Print a help text
;    PATH   - Path to the current file 
;             Default is the current directory in the first call to OPENEXO
;             but the default path is changed to the final path selected is 
;             retained as the default in the next call to OPENEXO.
;    FILTER - File filter - The default may be set in USERQSINIT; if undefined
;             the default is '*'.  The default filter is set only with
;              USERQSINIT or SETDEFAULT.
;              USERQSINIT or SETDEFAULT.
;    NDSET  - Number of datasets 
;             For Exodus II: Array of [# of global, # of node, # of cell]
;             For WDF array: Number of WDF arrays
;    QUIET  - Only print error messages
;    VERBOSE- Print file information
;
;  Examples:
;    Open a Exodus file with the default file id
;      openexo
;    Open a Exodus file, 'users/test', with the default file id
;      openpff, '/users/test'
;    Open a file to be selected from a menu with a file id of 14
;      OPENEXO, '', 14
;  or  OPENEXO, 0, 14
;    Get help
;      OPENEXO, /h
;-
;
  pro openexo, p1, p2,  path = path, filter = filter, ierror = ierror, $
             fileid = fileid, ndset = ndset, help = help, quiet = quiet, $
             write = write, verbose = verbose
on_error, 1

;  Common Blocks
@exoduscom.dat
COMMON saveexopicker, oldfilter, oldpath

 if n_elements(init_exo) eq 0 then init_exodus
;
; check on the number of parameters
;
 nparams = n_params()
 ierror = 1
 if nparams eq 0 and keyword_set(help) then begin
print, '% Format: OPENEXO [, File] [, Fileid ] [, /NoLoad] '
print, '                  [, PATH = path] [, FILTER = filter]'
print, '                  [, NDSET = ndset] [, FILEID = fileid]'
print, '                  [, QUIET = quiet] [, VERBOSE = verbose]'

   return
 endif
;
; check to see if exodus has been initialized
;
      if (nparams eq 0) || (size(p1, /type) lt 7) || (strlen(strtrim(p1, 2)) eq 0) then $
         p1 = '' else p1 = strtrim(p1, 2)
      if strlen(p1) eq 0 then begin
        IF (N_ELEMENTS(PATH) EQ 0 and n_elements(oldpath) eq 0) THEN BEGIN
          cd, current = dir
          if init_exo eq  1 then path = dir+'/' else path = dir
        endif else if n_elements(path) eq 0 then path = oldpath
        if n_elements(filter) eq 0 and n_elements(oldfilter) eq 0 then $
          filter = '*.exo' else if n_elements(filter) eq 0 then filter = oldfilter
        p1 = dialog_pickfile(/read, path = path, get_path = oldpath, filter = filter)
        if strlen(p1) lt 1 then return
        nparams = 1
      endif 
;
; user specified file id
      if n_elements (p2) eq 0 then begin
        if n_elements(fileid) gt 0 then p2 = long(fileid) else p2 = 0L
      endif else p2 = long(p2)
      if init_exo eq 1 then begin
;
;       get variable substitution path name for unix systems
;
        temp = p1
        fulfnam, temp, p1
      endif
      p1 = file_expand_path(p1)
      if !version.os eq 'linux' then resolve_it, p1
      NDSET = 0L
      oldid = exo_id
      info = file_info(p1)
      if (not info.exists) then begin
        print, '% OPENEXO: File does not exist'
        print, '          ', p1
        P1 =  dialog_pickfile(/must_exist, /read)
        if strlen(p1) lt 1 then begin
          print, '% OPENEXO: No File was Selected - RETURNING'
          return
        endif
        info = file_info(filename)
      endif
      if keyword_set(write) && (info.write eq 0) then begin
        print, '% OPENEXO: Unable to WRITE to file.  Settign WRITE=0'
        write = 0
      endif
      exo_id = ncdf_open(p1, write = write)
      if exo_id lt 0  then begin
        print, '% OPENEXO: Invalid file: ', p1
        exo_id = oldid
        return
      endif 
;     save the exo_id in FEXO_ID.
;     EXO_ID is returned as an index into FEXO_ID
;     Save the value of exo_id 
      fid = exo_id
      save_exofile_id

      if exo_id lt 0 or exo_id ge max_exofile then begin
        ncdf_close, fid
        print, '% OPENEXO: Unable to open file, Too many files open', exo_id
        exo_id = oldid
        return
      endif
      if p2 eq 0 then begin
        p2 = 1
        ok = where (array_exoid(*, 0) eq p2 and array_exoid(*,1) ne 0 , nok)
        while nok gt 0 do begin
          p2 = p2+1
          ok = where (array_exoid(*, 0) eq p2 and array_exoid(*,1) ne 0, nok)
        endwhile 
      endif else begin
        ok = where (array_exoid(*, 0) eq p2 and array_exoid(*,1) ne 0, nok)
        if nok gt 0 then begin
          print, '% OPENEXO: Unable to open file with that fid.'
          print, '        Either close other file or choose anothr fid'
          ncdf_close, fid 
          fcdf_id(exo_id) = -1
          exo_id = oldid
          return
        endif
      endelse
      array_exoid(exo_id, 0) = p2
      array_exoid(exo_id, 1) = long((systime(1)*10) mod 65536) > 1
      array_exoid(exo_id, 2) = (keyword_set(write)) ? 1:0

      file_name(exo_id) = p1
      glob = ncdf_inquire(fid)
      nattr = glob.ngatts
      attr = make_array(nattr, /string)
      for i = 0, nattr-1 do attr(i) = ncdf_attname(fid, i, /global)
      attr = strlowcase(attr)
      ok = where(attr eq 'title', nok)
      if nok eq  1 then begin
        ncdf_attget, fid, attr(ok(0)), title, /global
        title = strtrim(title, 2)
        file_title(exo_id) = title
      endif else begin
        if nok eq 0 then $
          print, '% OpenEXO: No Title found in the attributes'
        Print, '% OpenEXO: Title set to NULL'
        print, attr
        title = ''
        file_title(exo_id) = ''
      endelse
      ok = where(attr eq 'file_size', nok)
      if nok eq 0 then API_VERSION[exo_id] =0.0 else begin
        ncdf_attget,  fid, 'file_size', value, /global
        api_version[exo_id] = value
      endelse
      
;     the next few lines are probably not needed.
      ok = where(attr eq 'pfidl_type', nok)
      if nok eq 0 then begin
        exo_type(exo_id) = 1
        type = 1
      endif else begin
        ncdf_attget, fid, attr(ok(0)), type, /global
        cdf_type(exo_id) = type
      endelse
      if keyword_set(quiet) then verbose = 0
      dimname = make_array(glob.ndims, /string)
      dimvalue = make_array(glob.ndims, /long)
;     get the names of the dimensional variables
      for i = 0, glob.ndims-1  do begin
        ncdf_diminq, fid, i, name, dimval
        dimname(i) = strlowcase(strtrim(name, 2))
        dimvalue(i) = dimval
      endfor
;
      if type eq 1 then begin
;       Exodus II 
;
        ndset = make_array(3, /long)
        if not keyword_set(quiet) then print, $
          'Exodus II File: ', strtrim(file_name(exo_id), 2), ' opened with ', $
           'ID = ', Strtrim(p2, 2)
        if keyword_set(verbose) then print, ' Title = ', title

        ok = where(dimname eq  'num_glo_var', nok)
        if nok eq 0 then num_glo_var(exo_id) = 0 else begin
          num_glo_var(exo_id) = dimvalue(ok(0))
          ndset(0) = dimvalue(ok(0))
          if keyword_set(verbose) then print, ' Number of Global Variables = ', $
            strtrim(dimvalue(ok(0)), 2)
        endelse

        ok = where(dimname eq 'num_elem_var', nok)
        if nok eq 0 then num_elem_var(exo_id) = 0 else begin
          num_elem_var(exo_id) = dimvalue(ok(0))
          ndset(2) = dimvalue(ok(0))
          if keyword_set(verbose) then print, ' Number of Cell Variables = ', $
            strtrim(dimvalue(ok(0)), 2)
        endelse

        ok = where(dimname eq 'num_nod_var', nok)
        if nok eq 0 then num_nod_var(exo_id) = 0 else begin
          num_nod_var(exo_id) = dimvalue(ok(0))
          ndset(1) = dimvalue(ok(0))
          if keyword_set(verbose) then print, ' Number of Nodal Variables = ', $
            strtrim(dimvalue(ok(0)), 2)
        endelse

        ok = where(dimname eq 'num_el_blk', nok)
        if nok eq 0 then num_elem_blk(exo_id) = 0 else begin
          num_elem_blk(exo_id) = dimvalue(ok(0))
          if keyword_set(verbose) then print, ' Number of Blocks = ', $
            strtrim(dimvalue(ok(0)), 2)
        endelse

        ok = where(dimname eq 'num_dim', nok)
        if nok eq 0 then num_dim(exo_id) = 0 else begin
          num_dim(exo_id) = dimvalue(ok(0))
          if keyword_set(verbose) then print, ' Number of Dimensions = ', $
            strtrim(dimvalue(ok(0)), 2)
        endelse
        
        load_exo_data, dimname, dimvalue, verbose = verbose

      endif else if type eq 2 then begin
;       Waveform data
        num_nod_var(exo_id) = 0
        num_elem_var(exo_id) = 0
        varid = ncdf_varid(fid, 'numb_dataset')
        if varid ge 0 then begin
          ncdf_varget, fid, varid, value
          num_glo_var(exo_id) = value
        endif else begin
          print, '% OPENEXO: Number of datasets variable not found'
          print, '          Contact PFIDL adminsistrator'
          num_glo_var(exo_id) = 0
        endelse
        load_exo_data, dimname, dimvalue, verbose = verbose
      endif
;     ndset = Array of [# of global, # of node, # of cell]
      ndset= [num_glo_var(exo_id), num_nod_var(exo_id), num_elem_var(exo_id)]
      ierror = 0
      end
