; ------------------------------------------------------------------------
; $Id: par2exo.pro,v 1.4 2011/08/09 15:23:39 lpmix Exp $
; ------------------------------------------------------------------------
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
;+
; NAME:
;       PAR2EXO
;
; PURPOSE:
;       Two options exist:
;       1. Convert a series of QS particle datasets to an exodus file
;          Only 1 time series may be included in each file
;       2. Convert an arbitrary series of vertex datasets to an exodus file
;          The times are specified in the input.
;       READPAR is used to read the datasets
;       The exodus format will allow visualization with ParaView or Ensight
;
; CATEGORY:
;       Analysis & Visualization
;
; CALLING SEQUENCE:
;       PAR2EXO [, OUTFILE] [, INFILE]
;
; OPTIONAL INPUTS:
;       OUTFILE: File name for the output Exodus file
;                The default will be used if OUTFILE is undefined or a number.
;                The default file name will be the name of the particle dataset with
;                  spaces replaced by an underscore.
;                In all cases, if the name exists, the name will be padded with 
;                  a number to avoid eliminating previous files.
;                  For example if killed_electrons.exo exists then the outfile is 
;                  killed_electrons_001.exo
;
;       INFILE:	 Data file containing the PFF snapshots.
;                If INFILE is undefined, the default is qspar.pff.
;                If this file does not exist, DIALOG_PICKFILE is called.
;                If "Bounding Box" is the first dataset type, it is skipped.
;                If the Application dataset type is 1 or the pff type is 6 (IFL) the first 
;                  dataset is skipped.
;                Enter a number (for example, zero) to go to a widget to select the file.
;	
; KEYWORD PARAMETERS:
;       RAWDATA: If present, the data is assumed to be an arbitrary set of vertex data.
;                RAWDATA is not used for QUICKSILVER outputs, but for "RAW" vertex datasets.
;                If RAWDATA is not set and N_ELEMENTS(TIME) eq 0 then the data is assumed to 
;                be particle datasets from a QuickSilver simulation.
;       If RAWDATA is set or if TIME has a value then the following apply:
;         TIME:    If present, the time for the dataset(s).  RAWDATA is set to 1.
;                  Default:  TIME is taken from the dataset comment if RAWDATA is missing or zero
;                            TIME is 0.0 if RAWDATA is set      
;                            If TIME is an array, the values will be used for successive datasets.
;         DELTAT:  TIME increment per dataset for RAWDATA =1
;         ALLDATA: If set and RAWDATA = 1, then all datasets will be read
;
;       The following keyword apply regardless of the value of RAWDATA.
;       DATASETS: A string that defines the datasets that are to be written to OUTFILE
;                  from INFILE.  If this is an array, only the first will be used if 
;                  datasets is a string.  This is ignored if RAWDATA is set and ALLDATA is set.
;                If RAWDATA is not set, only the first 32 characters in the dataset
;                  comment will be matched.  If RAWDATA is set the entire dataset comment
;                  will be used.
;                If RAWDATA is set, then if DATASETS is an array of numbers, those datasets will be used.
;                  (DATASETS must be numbers in the range of 1 to number_of_datasets.
;                If DATASETS is not present, a widget will be provided to pick the 
;                  datasets for OUTFILE if the dataset contains more than 1 dataset.
;                  If the file contains only one dataset, it will be written.  
;                Note: Get_Match will be used for string matches.
;                NOTE: Vectors are saved as three vectors with _x, _y, and _z added to 
;                      the three components.
;       NPARTICLE: Maximum number of particles saved at each time step.
;                Default: Maximum number of particles for all time steps.
;                Note: Charge of invalid particles set to zero
;       ELECTRON: If set, normalized particle energy is multiplied by the
;               electron rest energy- See note below.
;               If ELECTRON (/E) is set, the fifth variable will be scaled by 510999.1e-6 
;                 the label
;       PROTON: If ELECTRON is set, this parameter is ignored.
;                If ELECTRON is not set, then the normalized energy is scaled 
;                by 931.49432*PROTON.  If PROTON is input as zero, it is changed to 1
;       Note: If ELECTRON or PROTON are input, the variable will be ENERGY (MeV). 
;             If not, then REL_ENER (Energy/Rest Mass (MeV))
;       Note: If CHARGE (Q) is not available, it will be added with a value of 1 for valid
;             particles and a value of 0 for no particles.
;
; RESTRICTIONS:
;       Only one series of datasets may be stored in each file
;
; PROCEDURE:
;       PFF file is opened
;       All datasets comments are read
;         If RAWDATA is not set the comments are divided into the first 32 characters
;         and the last 32 characters.
;       If DATASETS is set, the selected snapshots will be written to a single file.
;         If DATASETS is not set, a widget will allow the user to select the particle
;         datasets that they want written to OUTFILE.
;       If cylindrical geometry, convert to cartesian coordinates.
;       If the first spare word is -3 then it will be changed to the cartesian flag (1).
;       The first time step has all coordinates set to zero.  DISPLX, DISPLY, and 
;         DISPLZ are used to indicate actual particle positions.
;       Set up the Exocus header with variable names, number of blocks, etc.
;         (If charge is not present, it will be the only variable with a value of 1 or 0)
;       Output the coordinates ([0, 0, 0] for all particles)
;       Step through the dataset series and write the data for each time step.
;         READPAR is used to read the datasets.  See LHELP on READPAR for information 
;         on the variables (attributes).
;       The same number of particles are written for each time, but invalid particles have a 
;         charge of zero.  The data filter can be used to eliminate particles with zero charge.
;       
; EXAMPLE:
;       Convert all datasets that contain 'killed_electrons' in a file to be selected to exodus files
;            PAR2EXO, 0, 0, data= 'killed_electrons', /electron
;
;       Convert selected datasets in qspar.pff to exodus file 'diode_electron.exo'
;            PAR2EXO, 'diode_electron.exo', /electron
;        or
;            PAR2EXO, 'diode_electron.exo', 'qspar.pff', /electron
;       Since INFILE is undefined, the default is qspar.pff- if it exists
;             
;       Convert the lithium particles to an exodus file, "out.exo"
;            PAR2EXO, 'out.exo', dataset='Lithium', pro=6.941
;
;       Convert the vertex data in 'pbin3_1000.pff' to an exodus file, 'pbin3.exo'
;            PAR2EXO, 'pbin3.exo', 'pbin3_1000.pff', /RAW, /ALL
;
; MODIFICATION HISTORY:
; 	Written by:	L. P. Mix, Jr., March 17, 2009
;                       Added RAWDATA keyword and associated mechanics, LPM, 1/18/2010.
;
;-

 pro PAR2EXO, fileoutin, filein, electrons = electrons, protons = protons, nparticle = nparticle, $
            datasets = datasetsin, time = timein, deltat = deltat, rawdata = rawdata, alldata = alldata

on_error,1
@write_exo_header_com.dat
  
  if n_elements (filein) eq 0 then fname = 'qspar.pff' else fname = filein
  if size(fname, /type) eq 7 then begin
    info = file_info(fname)
    if (info.exists + info.read) ne 2b then begin
      print, '% PAR2EXO: File does not exist with read permission'
      print, '           File Name = ', fname
        print, '           File Exists =', info.exists
        print, '           Read Permission =', info.read
      fname = ''
    endif
  endif
  openpff, fname, ndset = ndset, error = error
  if error ne 0 then begin
    Print, '% PAR2EXO: Unable to open file.'
    return
  endif
  get_dir, comment= comment, type = type, number = ndset, /no_trim
  lcbb = 0
  if strpos(type[0], 'Bounding Box') ge 0 then begin
      comment = comment[1:*]
    ndset = ndset-1
    lcbb = 1
  endif else begin
    readhdr, 1, /quiet, pfftyp = pfftyp, aptype = aptype, namefile = namefile
    if pfftyp eq 6  or aptype eq 1 then begin
        comment = comment[1:*]
      ndset = ndset-1
        lcbb=1
    endif
  endelse
  if ndset eq 0 then begin 
    print, '% PAR2EXO: No valid datasets in this file.'
    closepff
    return
  endif
if keyword_set(rawdata) or n_elements(timein) gt 0 then begin
  rawdata =1
  if n_elements(timein) eq 0 then time = 0.0 else time = timein
  if keyword_set(alldata) then list = 1+lindgen(ndset) else $
    if n_elements(datasetin) gt 0 then begin
      datasets = datasetsin
      if size(/type, datasets) lt 4 then begin
        ok = where (datasets ge 1 and datasets le ndset, nok)
        if nok gt 0 then list = datasets[ok] else begin
          print, '% PAR2EXO: No valid datasets numbers specified'
          print, '           No file written'
          closepff
        endelse
      endif else begin
        list = get_match(datasets[0],  comment , nelem = nlist, regexp =0)
        if nlist eq 0 then begin
          print, '% PAR2EXO: No Match found for value of dataset: '+strtrim(datasets, 2)
          print, '           No file written'
          closepff
          return
        endif
      endelse
    endif else begin
        instr= ['Pick the signals to write to Exodus file', $
                'Signals must have same coordinates and times']
        wselect_item, comment, selected = sel, instruct = instr, exclusive =0
        list = where (sel gt 0, nlist)
        if nlist eq 0 then begin
          print, '% PAR2EXO: No datasets selected'
          print, '           No file written'
        endif
        list += 1
    endelse
    nt =  n_elements(time)
    nlist = n_elements(list)
    if nt lt nlist then begin
      if n_elements(deltat) eq 0 then deltat = 1
      time = [time, time[nt-1] + (findgen(nlist-nt)+1)*deltat]
    endif
    clabel = get_bare_filename(namefile)
endif else begin
  emphasis = ' -- t ='
  nemphasis = strlen (emphasis)

  ip = strpos(comment, emphasis)
  ok = where (ip gt 0, nok)
  if nok gt 0 then begin
    com1 = strarr(nok)
    com2 = com1
    for i = 0, nok-1 do com1[i] = strlowcase(strtrim(strmid(comment[ok[i]],0,ip[ok[i]]), 2))
    for i = 0, nok-1 do com2[i] = strlowcase(strtrim(strmid(comment[ok[i]],ip[ok[i]]), 2))
  endif else begin
    com1 = strtrim(strmid(comment, 0, 32), 2)
    com1 = strlowcase(com1)
    com2 = strtrim(strmid(comment, 32), 2)
    com2 = strlowcase(com2)
  endelse
  unq = uniq(com1, sort(com1))
  nunq = n_elements(unq)
  nsig = nunq
  names =  com1[unq]
  if n_elements(datasetsin) gt 0 then begin
    datasets=datasetsin[0]
    i = 0
    list = get_match(datasets,  names , nelem = nlist, regexp = 0)
    if nlist eq 0 then begin
      print, '% PAR2EXO: No Match found for value of dataset: '+strtrim(datasets, 2)
      print, '           No file written'
      closepff
      return
    endif
    if nlist gt 1 then begin
      print, '% PAR2EXO: No unique match found for value of dataset: '+strtrim(datasets, 2)
      for i = 0, nlist-1 do print, '     '+ names[list[i]]
      print, ' No file written'
      closepff
      return
    endif

    datasets = names[list[0]]
    nsig = 1
  endif else begin
    if nunq gt 1 then begin
      instr= ['Pick the signals to write to Exodus file', $
            'Signals must have same coordinates and times']
      sel = 0
      wselect_item, names, selected = sel, instruct = instr, /exclusive
      ok = sel
      nsig =1
      datasets = names[ok]
    endif else begin
      nsig = 1
      datasets = names[0]
    endelse
    list = get_match(datasets, com1, nelem = ndset, regexp = 0)+1+lcbb
    clabel = datasets[0]
  endelse   
endelse
; number of files to write  
  nfile = 1
  nsig = ndset
  dsok = list
; datasets to write
  readpar, s, list[0] 
  if size (s, /type) lt 8 then begin
    print, '% PAR2EXO: Structure must be defined'
    closepff
    return
  endif
  if s.type ne 2 then begin
    print, '% PAR2EXO: Only type 2 structures are allowed.'
    closepff
    return
  endif
  lc_coord = s.spare[0]
  if lc_coord eq -3 then lc_coord = 1
  if lc_coord gt 2 then begin
    print,  '% PAR2EXO: This version only handles cartesian & cylindrical coordinates'
    closepff
    return
  endif else lc_coord =1
  npart = make_array(nsig, val =0L)
  natt = npart
  for i = 0, nsig-1 do begin
    readhdr, list[i], iextra = iextra, pfftype = pfftype, /quiet
    if pfftype ne 5 then begin
      print, '% PAR2EXO: This version requires Vertex (particle) datasets.'
      help, pfftype
      help, list & print, list
      closepff
      return
    endif

    if iextra[0] ne 3 then begin
      print, '% PAR2EXO: This version requires 3D data'
      closepff
      return
    endif
    natt[i] = iextra[1]
    npart[i] = iextra[2]
  endfor
  mxatt = max(natt, min = mnatt)
  if mxatt-mnatt ne 0 then begin
    print, '% PAR2EXO: This the number of attributes to be the same for all datasets'
    print, mnatt, mxatt, natt
    closepff
    return
  endif
  mxpart = max(npart)
  nnode = (n_elements(nparticle) eq 0) ? mxpart : nparticle[0]


; number of signals per file
  ifile = 0
  if n_elements(fileoutin) eq 0 then begin
    byclab = byte(strupcase(clabel))
    ok = where ((byclab eq 46b) or (byclab eq 95b) or $
      ((byclab ge 65b) and (byclab le 90b)) or $
      ((byclab ge 48b) and (byclab le 57b)), comp = comp, ncomp = ncomp)
         
    ok2 = where (byclab eq 32b, noksp)
    if ncomp+ noksp gt 0 then begin
      clab = byte(clabel)
      if ncomp gt 0 then clab[comp]= 95B
      if noksp gt 0 then clab[ok2]= 95B
      clab = string(clab)
    endif else clab = clabel
    fileout = clab + '.exo'
  endif else fileout = fileoutin
  fileout = find_newfile(fileout, /quiet)

;
  coord_names = bytarr(33, 3)
  coord_names[0:5, 0] = byte('X axis')
  coord_names[0:5, 1] = byte('Y axis')
  coord_names[0:5, 2] = byte('Z axis')
  nvect = s.natt
  varname = make_array(3+nvect, /string)
  varname[0: 2] = ['DISPLX', 'DISPLY', 'DISPLZ']
  nam = tag_names(s)
  ok = get_match('V?LAB', nam, nel = nel, regexp = 0)
  nel = (nel < nvect)
  for i = 0, nel-1 do varname[3+i] = (s.(ok[i]))[0]
  ok = where ('Pi' eq varname, nok)
  if nok gt 0 then varname[ok[0]] = 'Px'
  ok = where ('Pj' eq varname, nok)
  if nok gt 0 then varname[ok[0]] = 'Py'
  ok = where ('Pk' eq varname, nok)
  if nok gt 0 then varname[ok[0]] = 'Pz'
  ok = where('Q' eq varname, nok)
  if nok eq 0 then begin
    varname= [varname, 'Q']
    lc_q = 1
  endif else lc_q = 0
  num_elem = nnode
  num_nodes = nnode
  newcoord = make_array(nnode, value = 0.0)
  dlconnect = 1L+lindgen(nnode)
  block_type ='SPHERE'
  numelemvar = 0
  nblock = 1
  cnames = 'Particles'
  title = 'IDL/ExodusII: '+systime(0)
  cdfid = ncdf_create(fileout, clobber = clobber)
  print, '% PAR2EXO: Writing Exodus File: '+fileout
  nnodevar = nvect +3L+lc_q
  write_exo_header, cdfid, nblock = nblock, nnode = num_nodes, nelem= num_elem, $
    blo_type = block_type, title = title, numnodevar = nnodevar, numelemvar= numelemvar
  ncdf_varput, cdfid, coordxid, newcoord
  ncdf_varput, cdfid, coordyid, newcoord
  ncdf_varput, cdfid, coordzid, newcoord
  ncdf_varput, cdfid, ebnameid, cnames
  ncdf_varput, cdfid, coordnamid, coord_names
  ncdf_varput, cdfid, connectid[0], reform(1L+lindgen(nnode), 1, num_elem)

  display = temporary(newcoord)

  ncdf_varput, cdfid, nodenamid, varname
  for nts = 0, ndset-1 do begin
    if nts gt 0 then readpar, s, dsok[nts] 
    c = s.clab
    if keyword_set(rawdata) then tim = time[nts] else begin
      jj = strpos(c, 't =')
      reads, strmid(c, jj+3), tim
    endelse
    ncdf_varput, cdfid, timewholeid, tim, offset = (nts)   
    nv = (s.nvert < nnode)
    if nv lt nnode then display[nv:*]=0.0
    if lc_coord eq 1 then begin
      display [0: nv-1] = (s.x1)[0: nv-1] 
      ncdf_varput, cdfid, nodevarid[0], display, offset = [0, nts]

      display [0: nv-1] = (s.x2)[0: nv-1] 
      ncdf_varput, cdfid, nodevarid[1], display, offset = [0, nts]
    endif else if lc_coord eq 2 then begin
      display[0: nv-1] = (s.x1*cos(s.x2))[0: nv-1] 
      ncdf_varput, cdfid, nodevarid[0], display, offset = [0, nts]

      display [0: nv-1] = (s.x1*sin(s.x2))[0: nv-1] 
      ncdf_varput, cdfid, nodevarid[1], display, offset = [0, nts]
    endif
      
    display [0: nv-1] = (s.x3)[0: nv-1] 
    ncdf_varput, cdfid, nodevarid[2], display, offset = [0, nts]
    if nvect  gt 0 then begin
      display [0: nv-1] = (s.V1)[0: nv-1]
      ncdf_varput, cdfid, nodevarid[3], display, offset = [0, nts]
      if nvect  gt 1 then begin
        display [0: nv-1] = (s.V2)[0: nv-1]
        ncdf_varput, cdfid, nodevarid[4], display, offset = [0, nts]
        if nvect  gt 2 then begin
          display [0: nv-1] = (s.V3)[0: nv-1]
          ncdf_varput, cdfid, nodevarid[5], display, offset = [0, nts]
          if nvect  gt 3 then begin
            display [0: nv-1] = (s.V4)[0: nv-1]
            ncdf_varput, cdfid, nodevarid[6], display, offset = [0, nts]
            if nvect  gt 4 then begin
              display [0: nv-1] = (s.V5)[0: nv-1] 
              ncdf_varput, cdfid, nodevarid[7], display, offset = [0, nts]
              if nvect  gt 5 then begin
                display [0: nv-1] = (s.V6)[0: nv-1]
                ncdf_varput, cdfid, nodevarid[8], display, offset = [0, nts]
                if nvect  gt 6 then begin
                  display [0: nv-1] = (s.V7)[0: nv-1]
                  ncdf_varput, cdfid, nodevarid[9], display, offset = [0, nts]
                  if nvect  gt 7 then begin
                    display [0: nv-1] = (s.V8)[0: nv-1]
                    ncdf_varput, cdfid, nodevarid[10], display, offset = [0, nts]
                  endif
                endif
              endif
            endif
          endif
        endif
      endif
    endif
    if lc_q then begin
      display [0: nv-1] = 1.0
      ncdf_varput, cdfid, nodevarid[nnodevar-1], display, offset = [0, nts]
    endif
  endfor
  ncdf_close, cdfid
;
  closepff, 0
  end
