; ------------------------------------------------------------------------ 
; $Id: read_exo.pro,v 1.4 2008/02/27 22:47:14 lpmix Exp $
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
 Function read_exo, dataset, tvalue, index = index, ierror = ierror, $
          time_index = tindex, grid = grid, node = nodein, clabel = clabel, $
          number = number, position = position, vector = vector
;
;  number   - the node or element number
;  position - the position 
;

;+
;  Name: READ_EXO
;
;  Purpose:
;        Reads a Exodus dataset into an IDL array.  
;        This is a utility function.
;      Note: Cell or element data will be left as element or cell data.
;      Note: IDL arrays are returned with RE_EXO for a node or element history
;
;  Format: 
;
;    DATA = READ_EXO( [ DATASET,] TVALUE [, /INDEX] [, IERROR = ierror]
;                     [, TIME_INDEX = time_index] [, /GRID] [, CLABEL = clab]
;                     [, NODE = NODE] [, VECTOR = vector] )
;
;  where:
;    DATA      - IDL array with the requested data array.
;    DATASET   - indicates a Exodus dataset name or a search string for dataset
;                names.  Only node and cell data may be read.
;                DATASET numbers are number shown in DIREXO.
;                If GRID is set, must contain the TVALUE.
;        See: GET_DSET (PFIDL_1D) for more details of the search.
;    TVALUE    - Time for the data.  If INDEX is set this is an index number.
;                Valid index numbers are 1 to number of time values.
;                The closest time is returned; no interpolation is performed.
;                If GRID is set and DISPLX, DISPLY, DISPLZ are missing, then
;                TVALUE is ignored.
;    INDEX     - If INDEX is set then TVALUE goes from 1 to the maximum number of
;                times.
;    IERROR    - Returned 0 if no error and valid data
;    TIME_INDEX- This is returned with the time value index from 1 to the number
;                Times.  The actual time is given by TIME = (GET_TIME())[Time_index -1]
;    GRID      - If set, a node array will be returned.  If DISPLX, DISPLY, DISPLZ
;                are missing, the coordinates are for the first time step.
;    CLABEL    - Comment label for the data.  It includes the time and filename.
;                Format: "DATASET, t=time - Filename=filename"
;    NODE      - Node number or Element number for dataset vs time.
;                If set, a one-dimensional vector is returned.
;                If NODE is a vector with NDIM elements, then the closest element
;                or node will be returned.
;                Actual values are returned in the keywords, NUMBER and POSITION.
;
;  Examples:
;    Read dataset 34 into array, data, at time step 5.
;      data = read_exo( 34, 5, /ind)
;
;    Read dataset with 'KE' into array, energy, at time = 1.36 ns.
;      energy = read_exo('KE', 1.36e-9)
;
;  MODIFICATION HISTORY:
;       Written by:     L. P. Mix, October 4, 2005
;
;-
      on_error, 1
;     add common block
@exoduscom.dat
;
;     check to see if EXODUS has been initialized
;
      ierror = 1
      if n_elements(init_exo) eq 0 then begin
        print, '% READ_EXO: No Exodus files are open.'
        return, 0
      endif
;       
      if exo_id  lt 0 then begin
        print, '% READ_EXO: Exodus file must be opened'
        return, 0
      endif
;
      nparam = n_params()
      if (nparam lt 1 and not keyword_set(grid)) then begin
 print,'% Format: DATA = READ_EXO( [ DATASET, ]  TVALUE [, /INDEX] [, /GRID]'
 print,'       [, IERROR = ierror] [, TIME_INDEX= time_index] [, CLABEL = clabel])'
 print,'       [, NODE = node]'
	return, 0
      endif

;     exodus file id
      fid = fexo_id(exo_id)
      apiversion = api_version[exo_id]

;
      if lexo_valid ne array_exoid(exo_id, 1) then load_exo_data
      if exo_type(exo_id) eq 1 then begin
        nblo = num_elem_blk[exo_id]
        ntime = n_elements(time_whole)
        ndim = num_dim[exo_id]
        if keyword_set(grid) then begin
;         BEGIN GRID
          if lc_displace eq 1 and n_elements(dataset) eq 0 then begin
             print, '% READ_EXO: TVALUE must be provided to calculate node offset.'
             wok_notok, lcquit, $
               ['Time Value Must be Provided', 'Do you wish to select a time value?'], $
               'Time missing', $
               menu=['Continue - Select Time', 'QUIT - ReEnter Command']
             if lcquit gt 0 then return, 0
             t = get_exotimes()
             wselect_item, string(t), sel = sel, /ex
             tvalue = sel+1
             index = 1
          endif else tvalue = dataset[0]
          if n_elements(coord_node) eq num_nodes*ndim then data = coord_node else begin
            if apiversion ge api_change then begin
              varid = ncdf_varid(fid, 'coordx')
              if varid ge 0 then ncdf_varget, fid, varid, coordx else begin
                print, '% LOAD_CDF_DATA: Unable to read COORDX'
                return, 0
              endelse
              num_nodes = n_elements(coordx)
              varid  = ncdf_varid(fid, 'coordy')
              if varid ge 0 then ncdf_varget, fid, varid, coordy else begin
                print, '% LOAD_CDF_DATA: Unable to read COORDY'
                return, 0
              endelse
              if num_dim eq 3 then begin
                  varid  = ncdf_varid(fid, 'coordz')
                if varid ge 0 then ncdf_varget, fid, varid, coordz else begin
                  print, '% LOAD_CDF_DATA: Unable to read COORDZ'
                  return, 0
                endelse
                coord_node = reform([coordx, coordy, coordz], num_nodes, 3)
              endif else coord_node = reform ([coordx, coordy], num_nodes, 2)
            endif else begin
              varid = ncdf_varid(fid, 'coord')
              if varid ge 0 then ncdf_varget, fid, varid, coord_node else begin
                print, '% LOAD_CDF_DATA: Unable to read COORD'
                return, 0
              endelse
              num_nodes = (size(coord_node))[1]
            endelse            
            data = coord_node
          endelse
          if lc_displace gt 0 then begin
            if keyword_set(index) then begin
              ind = tvalue -1 
              if (ind lt 0) || (ind ge ntime) then begin
                print, '% READ_EXO: Time value (DATASET) must be in the range of [1,', $
                  strtrim( ntime,2)+ ']'
                return, 0
              endif
            endif else ok = min(time_whole-tvalue, /absolute, ind)
            TIME_value = time_whole[ind]
            tindex = ind+1
            tstr = strtrim(time_value, 2)
            if apiversion ge api_change then begin
              varid = ncdf_varid(fid, displx)
              if varid ge 0 then begin
                ncdf_varget, fid, varid, dx, offset = [0, ind], count = [num_nodes, 1]
                data[*, 0]= (lc_vesey) ? reform(dx) : reform(dx)+data[*, 0]
                varid = ncdf_varid(fid, disply)
                ncdf_varget, fid, varid, dy, offset = [0, ind], count = [num_nodes, 1]
                data[*, 1]= reform(dy)+data[*, 1]
                if ndim eq 3 then begin
                  varid = ncdf_varid(fid, displz)
                  ncdf_varget, fid, varid, dz, offset = [0, ind], count = [num_nodes, 1]
                  data[*, 2]= reform(dz)+data[*, 2]
                endif
              endif else begin
                print, '% READ_EXO: Cannot find DISPLX; Contact Paul Mix'
                return, 0
              endelse
            endif else begin
              varid = ncdf_varid(fid, 'vals_nod_var')
              if varid ge 0 then begin
                ncdf_varget, fid, varid, dx, offset = [0, displx, ind], $
                  count = [num_nodes, 1, 1]
                data[*, 0]= (lc_vesey) ? reform(dx) : reform(dx)+data[*, 0]
                ncdf_varget, fid, varid, dy, offset = [0, disply, ind], $
                count = [num_nodes, 1, 1]
                data[*, 1]= reform(dy)+data[*, 1]
                if ndim eq 3 then begin
                  ncdf_varget, fid, varid, dz, offset = [0, displz, ind], $
                    count = [num_nodes, 1, 1]
                data[*, 2]= reform(dz)+data[*, 2]
                endif
              endif else begin
                print, '% READ_EXO: Cannot find "vals_nod_var"; Contact Paul Mix'
                return, 0
              endelse
            endelse
          endif else begin
            tstr = strtrim(time_whole[0], 2)           
            tindex = 1
          endelse
          clabel = 'Node Coordinates, T='+tstr
          ierror = 0
          return, data
        endif
;       END GRID
        if (n_elements(dataset) eq 0) then begin
          print,'% READ_EXO: Dataset must be defined'
          return, 0
        endif
        if (n_elements(tvalue) eq 0) && (n_elements(nodein) eq 0) then begin
          print, '% READ_EXO: TVALUE must be provided to calculate node offset.'
          wok_notok, lcquit, $
            ['Time Value Must be Provided', 'Do you wish to select a time value?'], $
            'Time missing', $
            menu=['Continue - Select Time', 'QUIT - ReEnter Command']
          if lcquit gt 0 then return, 0
          t = get_exotimes()
          wselect_item, string(t), sel = sel, /ex
          tvalue = sel+1
          index = 1
        endif
        if n_elements(tvalue) eq 0 then lcnode = 1 else lcnode = 0
          
        nmax = num_elem_var(exo_id) + num_nod_var(exo_id) 
        if nmax eq 0 then begin
          print, '% READ_EXO: No node or element arrays in this Exodus file'
          return, 0
        endif
        if (n_elements(dataset) eq 0)  then begin
          print,'% READ_EXO: Dataset value must be defined'
          return, 0
        endif
        
        if ( lcnode eq 0) && (n_elements(tvalue) eq 0) then begin
          print,'% READ_EXO: Either NODE or TVALUE must be defined'
          return, 0
        endif
        if n_elements(dataset) gt 1 and not keyword_set(vector) then begin
           print,'% READ_EXO: Only 1 dataset may be specified unless VECTOR is set.'
           help, dataset
           return, 0
        endif 
        siz= [ num_nod_var[exo_id],num_elem_var[exo_id]]
        searchstr = make_array(total(siz), /string)
        if siz[0] gt 0 then searchstr[0:siz[0]-1]=name_nod_var  
        if siz[1] gt 0 then searchstr[siz[0]:total(siz)-1]= name_elem_var
        get_exo_string, dataset, searchstr, map, nmap, nmax, ier, lcstring
        if ier ne 0 then begin
 print,'% READ_EXO: Valid datasets not found. Please change the dataset specification'
           help, dataset
           print, searchstr
           print, name_node_var
           return, 0
        endif
        if (nmap eq 0) || (nmap gt 1 and not keyword_set(vector)) then begin
          print,'% READ_EXO: One and only 1 dataset must be specified unless vector is set.'  
          if nmap eq 0 then begin
            Print,'           No datasets found with specifier: ', dataset
            return, 0
          endif
          print,'           The datasets with this specifier "', dataset, '", are:'
          tempstr =  [name_nod_var, name_elem_var]
          print, '  No.  Dataset Comment'
          for i=0L, nmap-1 do print,format='(i4,3x, a)',map(i),tempstr(map(i)-1)
          return, 0
        endif 
        if keyword_set (lcnode) then begin
;       read data at one node
          node = nodein 
          nt = n_elements(time_whole)
          if map[0] le num_nod_var(exo_id) then begin
;           NODE data
            if n_elements(node) eq 1 then begin
              if node lt 1 or node gt num_nodes then begin
                print, '% READ_EXO: Node must be greater than 1 and'
                print, '            Less/Equal to: ', strtrim(num_nodes, 2)
                return, 0
              endif
              number = node
              position = coord_node[node-1, *]
            endif else if ndim eq n_elements(node) then begin
              distance = (coord_node[*, 0]-node[0])^2
              if num_dim[exo_id] gt 1 then $
                distance = distance + (coord_node[*, 1] - node[1])^2
              if num_dim[exo_id] gt 2 then $
                distance = distance + (coord_node[*, 2] - node[2])^2
              junk = min(distance, node)
              position = coord_node[node, *]
              node = node+1
              number = node
            endif else begin
              print, '% READ_EXO: Number of node elements must be a scalar'
              print, '            or be equal the dimension'
              help, node
              return, 0
            endelse
            if apiversion le api_change then begin
              nname = make_array(nmap, value = 'vals_nod_var')
              for i = 0, nmap-1 do nname = nname[i]+strtrim(map[i], 2)
              varmap= make_array(nmap, /long)
              for i = 0, nmap-1 do varmap[i] = ncdf_varid(fexo_id[exo_id], nname[i])
              if min(map[0:nmap-1]) ge 0 then begin
                for i = 0, nmap- 1 do begin
                  ncdf_varget, fid, varmap[i], data, offset = [number-1, 0], count = [1, nt]
                  if nmap gt 1 then begin
                    if i eq 0 then begin
                      data2 = data*data
                      clabel = 'Amplitude of '+ name_nod_var(map[0]-1)
                    endif else begin
                      data2 += data*data
                      clabel = clabel+', '+name_nod_var(map[i]-1)
                    endelse
                  endif else clabel = name_nod_var(map[0]-1)
                endfor
                if nmap gt 1 then data = sqrt(temporary(data2))
                clabel = clabel +' - Node='+strtrim (number, 2)
              endif else begin
                print, '% READ_EXO: No Node array variables found in Exodus II data'
                print, '                 Number of Nodal Variables set to zero.'
                return, 0
              endelse
            endif else begin
              varid = ncdf_varid(fexo_id[exo_id], 'vals_nod_var')
              if varid ge 0 then begin
                for i = 0, nmap- 1 do begin
                  ncdf_varget, fid, varid, data, offset = [number-1, map[i]-1, 0], $
                    count = [1, 1, nt]
                  if nmap gt 1 then begin
                    if i eq 0 then begin
                      data2 = data*data
                      clabel = 'Amplitude of '+ name_nod_var(map[0]-1)
                    endif else begin
                      data2 += data*data
                      clabel = clabel+', '+name_nod_var(map[i]-1)
                    endelse
                  endif else clabel = name_nod_var(map[0]-1)
                endfor
                if nmap gt 1 then data = sqrt(temporary(data2))
                clabel = clabel +' - Node='+strtrim (number, 2)
              endif else begin
                print, '% READ_EXO: No Node array variables found in Exodus II data'
                print, '                 Number of Nodal Variables set to zero.'
                return, 0
              endelse
            endelse
          endif else begin
;           ELEMENT data
            if n_elements(node) eq 1 then begin
              if node lt 1 or node gt num_elem then begin
                print, '% READ_EXO: Node must be greater than 1 and'
                print, '            Less/Equal to: ', strtrim(num_elem, 2)
                return, 0
              endif
              number = node
              get_element_centers, coord_node, elem
              position = elem[node-1, *]
            endif else if ndim eq n_elements(node) then begin
              number = get_elem(node, values = position)
            endif else begin
              print, '% READ_EXO: Number of node elements must equal the dimension'
              help, node
              return, 0
            endelse
            map = map-num_nod_var(exo_id)
            map = map
            for i = 0, nmap-1 do begin
              vname = 'vals_elem_var'+strtrim(map[i], 2)+'eb' 
              ok = where (number gt loc_V, nok)
              inod = number- loc_v[ok[nok-1]] -1
              varid = ncdf_varid(fid, vname+strtrim(nok, 2))
              if varid ge 0 then begin
                ncdf_varget, fid, varid, data, offset = [ inod, 0], count = [1, nt]
                if nmap gt 1 then begin 
                  if i eq 0 then begin
                    data2 = data*data
                    clabel = 'Amplitude of '+name_elem_var[map[i]-1] 
                  endif else begin
                    data2 += data*data
                    clabel = clabel+', '+name_elem_var[map[i]-1]
                  endelse
                endif else  clabel = name_elem_var[map[i]-1] 
              endif else begin
                print,  '% READ_EXO: Unable to obtain VARID for ', $ 
                   vname+strtrim(nok, 2)
                return, 0
              endelse
            endfor
            ierror = 0
            if nmap gt 1 then data = sqrt(temporary(data2))
            clabel = clabel +' - Element '+strtrim (number, 2)
          endelse
          ierror = 0
          data = reform(data, /overwrite)
          clabel = clabel +', ('+strtrim(position[0],2)+', '+ strtrim(position[1],2)
          clabel = clabel + ((ndim eq 2) ? ')' :  ', ' + strtrim(position[2],2)+ ')')
          return, data
        endif       
;       END reading node data
        if keyword_set(index) then begin
          ind = tvalue -1 
          if (ind lt 0) || (ind ge ntime) then begin
            print, '% READ_EXO: Time value (DATASET) must be in the range of [1,', $
              strtrim( ntime,2) + ']'
            return, 0
          endif
        endif else ok = min(time_whole-tvalue, /absolute, ind)
        TIME_value = time_whole[ind]
        tindex = ind+1
        tstr = strtrim(time_value, 2)
        if map[0] le num_nod_var(exo_id) then begin
          if apiversion ge api_change then begin
            nodename =make_array(nmap, value ='vals_nod_var')
            for i = 0, nmap-1 do nodename[i] =nodename[i]+strtrim(map[i],2)
            varmap = make_array(nmap, /long)
            for i = 0, nmap-1 do varmap[i] = ncdf_varid(fexo_id[exo_id],nodename[i]) 
            if min(map[0:nmap-1]) ge 0 then begin
              for i = 0, nmap-1 do begin
                ncdf_varget, fid, varmap[i], data, offset = [0, ind], count = [num_nodes, 1]
                if nmap gt 1 then begin
                  if i eq 0 then begin
                    data2 = data*data
                    clabel = 'Amplitude of '+ name_nod_var(map[0]-1)
                  endif else begin
                    data2 += data*data
                    clabel = clabel+', '+name_nod_var(map[i]-1)
                  endelse
                endif else clabel = name_nod_var(map[0]-1)
              endfor
              if nmap gt 1 then data = sqrt(temporary(data2))
              clabel = clabel + ', T='+tstr
              ierror = 0
              return, reform (data)
            endif else begin
              print, '% READ_EXO: No Node array variables found in Exodus II data'
              print, '                 Number of Nodal Variables set to zero.'
              print, ' VARID:', map[0:nmap-1]
              print, ' Nodenames: ', nodename
              num_nod_var(exo_id) = 0
              return, 0
            endelse
          endif else begin
            varid = ncdf_varid(fexo_id[exo_id], 'vals_nod_var')
            if varid ge 0 then begin
              for i = 0, nmap-1 do begin
                ncdf_varget, fid, varid, data, offset = [0, map[i]-1, ind], $
                  count = [num_nodes, 1, 1]
                if nmap gt 1 then begin
                  if i eq 0 then begin
                    data2 = data*data
                    clabel = 'Amplitude of '+ name_nod_var(map[0]-1)
                  endif else begin
                    data2 += data*data
                    clabel = clabel+', '+name_nod_var(map[i]-1)
                  endelse
                endif else clabel = name_nod_var(map[0]-1)
              endfor
              if nmap gt 1 then data = sqrt(temporary(data2))
              clabel = clabel + ', T='+tstr
              ierror = 0
              return, reform (data)
            endif else begin
              print, '% READ_EXO: No Node array variables found in Exodus II data'
              print, '                 Number of Nodal Variables set to zero.'
              num_nod_var(exo_id) = 0
              return, 0
            endelse
          endelse
        endif
;
;       element data
;       tslice = array for time slice
        tslice = make_array(num_elem)
        map= map-num_nod_var(exo_id)
        for i = 0, nmap-1 do begin
          vname = 'vals_elem_var'+strtrim(map[i], 2)+'eb'
; time slice
          for j = 1, nblo do begin
            varid = ncdf_varid(fid, vname+strtrim(j, 2))
            ncdf_varget, fid, varid, data, offset = [0, ind], count = [size_v[j-1], 1]
            tslice[loc_v[j-1]:loc_v[j]-1]= reform(data)
          endfor
          if nmap gt 1 then begin
            if i eq 0 then begin
              data2 = tslice*tslice
              clabel = 'Amplitude of '+name_elem_var[map[i]-1]
            endif else begin
              data2 += tslice*tslice
              clabel = clabel+', '+name_elem_var[map[i]-1]
            endelse
          endif else  clabel = name_elem_var[map[i]-1]
        endfor
        if nmap gt 1 then tslice = sqrt(temporary(data2))
        clabel = clabel +', T='+tstr
        ierror = 0
        return, tslice
     endif else if exo_type(exo_id) eq 2 then print, $
        '% READ_EXO: This is wdf data - use RE_EXO'
        
      end
