; ------------------------------------------------------------------------
; $Id: surf2exo.pro,v 1.1 2010/02/15 17:58:10 lpmix Exp $
; ------------------------------------------------------------------------
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
;+
; NAME:
;       SURF2EXO
;
; PURPOSE:
;       Convert a series of QS surface datasets to an exodus file
;       The exodus format will allow visualization with ParaView or Ensight
;       A group of PFF datasets with identical space/time parameters
;         are converted to a Exodus file
;
; CATEGORY:
;       Analysis & Visualization
;
; CALLING SEQUENCE:
;       SURF2EXO [, OUTFILE] [, INFILE] [, DATASETS]
;
; OPTIONAL INPUTS:
;       OUTFILE: File name for the output Exodus file
;                If undefined or zero, the default will be surface_000X.exo where "000X"
;                  is the string version of the surface number in I4.4 format.
;
;       INFILE:	 Data file containing the PFF snapshots.
;                If INFILE and STRUCTURES are both either undefined or zero, the default
;                  is qssnp.pff.  If this file does not exist, DIALOG_PICKFILE is called.
;                If "Bounding Box" is the first dataset type or the Application dataset
;                  type is 1, the first dataset is skipped.
;                Enter a nonzero number to go to a widget to select the file.
;
;       DATASETS: Array of one or more strings or numbers that define the datasets that
;                  are to be written to OUTFILE from INFILE.  
;                If DATASETS is not present, a widget will be provided to pick the 
;                  datasets for OUTFILE if infile contains more than 1 unique dataset.
;                  All "Surface Topology" datasets are ignored.
;                  If the file contains only one dataset, it will be written.  
;                  All DATASETS written to a single file muct have the same spacial
;                  coordinates and the same output times.
;                Note: Get_Match will be used.
;                NOTE: Vectors are saved as three vectors with _x, _y, and _z added to 
;                      the three components
;                Note: All datasets must have the same time steps and the same surface
;                      topology.
;                Note: See TDELTA for tolerance on the times.  No tolerance on topology.
;                Note: STRCOMPRESS will be used to eliminate extra spaces.  Only the first 
;                      30 characters are used for conversion to Exodus format.
;	
; KEYWORD PARAMETERS:
;       STRUCTURES: This is a list of structure array numbers to be converted to 
;                an exodus file or a string that will provide a list of structure array
;                numbers when passed to STRVALID.  The structures are assumed to be from a
;                single QS snapshot command and are in time order.
;                Multiple strings (an array of strings) may not be passed.
;
;       ALL_SNAPS: All snapshots in INFILE are converted to a single exodus file.
;                OUTFILE is ignored if ALL_SNAPS is set.
;                The output file names are the name of the snapshot plus ".exo"
;                For example: dataset comment = "e:phinorm1"
;                             goes into file = "e_phinorm1.exo"
;                File name will be limited to letters, numbers, period, and underscore. 
;                Byte values = 46 or (48 to 57) or (65 to 90) or 95 or (97 to 122)
;                Underscore replaces all other values.
;                As noted above, if a file already exists with the selected outfile,
;                "_001" is added to the name to generate a unique file name.
;
;       DATASETS: Keyword option for parameter, DATASETS.  If present, this keyword is ignored.
;                If the parameter is missing, then this keyword is used.
;
;       TDELTA:  Fraction of the time range to be used as an error check.
;                 Default:  TDELTA = 1e-4
;                 Values of the time for multiple datasets in a file must agree to 
;                 ALLOWED_VARIANCE= (TN-T1)*TDELTA where times are T1, T2, ... TN
;
; RESTRICTIONS:
;	Exodus file require the same spatial values and the same time steps for all 
;       variables in a file
;
; PROCEDURE:
;       If STRUCTURES is specified, no file is opened.
;         If not, a PFF file is opened.
;       If ALL_SNAPS is set, every dataset in the PFF file will be written to 
;         a separate Exodus file.
;       If DATASETS is set, these snapshots will be written to a single.
;         If DATASETS is not set, a widget will allow the user to select the snapshots
;         that they want written to OUTFILE.
;     For the first time step in a file:
;       Determine the unique points in the problem and the connectivity.
;       The first time step for the structure is used for the grid and 
;         for the dataset name.  Subsequent structures are checked for number
;         of blocks, size and limits.
;       If cylindrical geometry, convert to cartesian coordinates.
;       Set up the Exocus header with variable names, number of blocks, etc.
;       Output the coordinates and connectivity
;       Step through the datasets and write the data for each time steps.
;       
; EXAMPLE:
;       Convert all datasets in a file to be selected to exodus files.
;       Note: All datasets must have the same topology and same time steps.
;            SURF2EXO, 0, 0, /all
;
;       Convert all datasets in qssnp.pff to exodus files.
;            SURF2EXO, 0, 'qssnp.pff', /ALL
;        or
;            SURF2EXO, /ALL
;            ; Since INFILE is undefined, the default is qssnp.pff- if it exists
;
;       Convert datasets [3, 4, 5] in a file, selected by dialog pickfile to an exodus
;         file, feed_surface.exo.
;            SURF2EXO, 'feed_surface.exo', 1, [3, 4, 5]
;
;       Convert the datasets 'e' in a file to be selected to a file, para_in.exo
;       Assume the the file only contains snapshots called 'e'
;            SURF2EXO, 'para_in.exo', 0
;
;       Convert qssnp.pff to the surface1.exo for dataset "e:phcb"
;            SURF2EXO, 'surface1.exo', dataset = 'e:phcb'
;
;       Convert all pff datasets containing "phinorm" to an exodus file dataout.exo
;            SURF2EXO, 'dataout.exo', dataset ='phinorm'
;
;       Convert structure arrays, 100 to 175 to an exodus file, out.exo
;            SURF2EXO, 'out.exo', struct = 100+indgen(76)  
;
; MODIFICATION HISTORY:
; 	Written by:	L. P. Mix, Jr., February 10, 2010
;
;-

 pro SURF2EXO, fileoutin, filein, pdatasets, structures = strarrayin, all_snaps = all_snaps, $
            datasets= datasetsin, tdelta = tdelta

on_error,1
@write_exo_header_com.dat
  
  if (size(/type, fileoutin) eq 7) then fileout = strtrim(fileoutin[0], 2)

  if n_elements(tdelta) eq 0 then tdelta = 1.0e-4
  if n_elements (filein) eq 0 then fname = 'qssnp.pff' else begin
    sizfil = size(/type, filein)
    if (sizfil lt 5) && (filein eq 0) then fname = 'qssnp.pff'
    if sizfil eq 7 then fname = strtrim(filein, 2)
  endelse
  if size(fname, /type) eq 7 then begin
    info = file_info(fname)
    if (info.exists + info.read) ne 2b then begin
      print, '% SURF2EXO: File does not exist with read permission'
      print, '           File Name = ', fname
      print, '           File Exists =', info.exists
      print, '           Read Permission =', info.read
      fname = ''
    endif
  endif
  openpff, fname, ndset = ndset, error = error
  if error ne 0 then begin
    Print, '% SURF2EXO: Unable to open file.'
    return
  endif
  lcopenpff = 1
  get_dir, comment= comment, type = type, number = ndset, /no_trim
  lcbb = 0
  if strpos(type[0], 'Bounding Box') ge 0 then begin
    comment = comment[1:*]
    ndset = ndset-1
    lcbb = 1
  endif else begin
    readhdr, 1, ier, /quiet, pfftyp = pfftyp, aptype = aptype
    if  aptype eq 1 then begin
      comment = comment[1:*]
      ndset = ndset-1
      lcbb=1
    endif
  endelse
  if ndset eq 0 then begin 
    print, '% SURF2EXO: No valid datasets in this file.'
    if lcopenpff then closepff
    return
  endif

  com1sav = strtrim(strmid(comment, 0, 32), 2)
  com1 = strlowcase(com1sav)
  com2 = strtrim(strmid(comment, 32), 2)
  com2 = strlowcase(com2)
  unq = uniq(com1, sort(com1))
  nunq = n_elements(unq)
  nsig = nunq
  names =  com1[unq]
  offset = lcbb+1
 
  if keyword_set(all_snaps) then datasets = names else begin
    if n_elements(pdatasets) gt 0 then datasets = pdatasets else $
      if n_elements(datasetsin) gt 0 then datasets = datasetsin
    nsig = n_elements(datasets)
    if (nsig gt 0) then begin
      sd = size (/type, datasets)
      if sd gt 0 and sd lt 4 then $
        datasets = (nsig eq 1) ? com1[datasets[0]-offset] : $
        com1[datasets[uniq(datasets, sort(datasets))]-offset]
      i = 0
      nlist = 0L
      for j = 0L, nsig-1 do begin
        ok = get_match(datasets[j],  names , nelem = nelem)
        if nelem gt 0 then begin
          list = (nlist eq 0) ? ok : [list, ok]
          nlist += nelem
        endif else print, '% SURF2EXO: No Match found for: ' + datasets[j]
      endfor
      if nlist eq 0 then begin
        print, '% SURF2EXO: No Match found for value of datasets'
        print, '           No file written'
        if lcopenpff then closepff
        return
      endif
      list = list[uniq (list, sort(list))]
      datasets = names[list]
      nsig = n_elements(list)
    endif  
  endelse
  if (nsig eq 0) then begin
    instr= ['Pick the signals to write to Exodus file', $
            'Signals must have same coordinates and times']
    sel = make_array(/long, value = 0, nunq)
    wselect_item, names, selected = sel, instruct = instr
    ok = where (sel eq 1, nsig)
    if nsig eq 0 then begin
      print, '% SURF2EXO: No signals selected --- RETURNING'
      if lcopenpff then closepff
      return
    endif
    datasets = names[ok]
  endif
; Check that the datasets all have the same number of time values
  ntim_sig= make_array(nsig, val =0L)
  for i = 0, nsig-1 do begin
    ok = get_match('^'+datasets[i]+'$', com1, nelem = neltot)
    notok =get_match('Surface Topology', type[ok+lcbb],nelem = nnok)
    ntim_sig[i] = neltot-nnok
  endfor
  ok = where (ntim_sig gt 0, nelem)
  if nelem le 0 then begin
    print, '% SURF2EXO: Unable to find data datasets for '+datasets[0]
    help, nelem, neltot, nnok
    if lcopenpff then closepff
    return
  endif
  if (max(ntim_sig[ok], min = m) - m ne 0) then begin
    print, '% SURF2EXO:  Some signals have a different number of time steps'
    for i = 0, nelem-1 do print, datasets[ok[i]], ntim_sig[ok[i]]
    if lcopenpff then closepff
    return
  endif 
  if nsig ne nelem then datasets= datasets[ok]
  nsig = nelem
  nelem = m
  nsig_dataset = make_array(nsig,value =0L)
  data_out = make_array(nelem, nsig, /long)
  times = make_array(nelem)
  for i = 0, nsig-1 do begin
    ok = get_match('^'+datasets[i]+'$', com1, nelem = neltot)
    if neltot lt nelem then begin
      print, '% SURF2EXO: Unable to find ' +strtrim(nelem, 2)+ 'data datasets for '+datasets[i]
      help, neltot, i
      return
    endif
    dataok = where(type[ok+lcbb] ne 'Surface Topology', nok)
    if nok eq 0 then begin
      print, '% SURF2EXO: Unable to find data datasets for '+datasets[i]
      help,  nok, nelem
      print, type[ok+lcbb]
      if lcopenpff then closepff
      return
    endif
    data_out[*, i] = ok[dataok]+offset
    readhdr, data_out[0, i], ier, aptype = aptype, pfftype = pfftype, /quiet
    if ier ne 0 then begin
      print, '% SURF2EXO: Error reading header.'
      print, 'Comment: '+names[data_out[0,i]]
      if lcopenpff then closepff
      return
    endif
    if pfftype ne 6 then begin
      print, '% SURF2EXO: Surface dataset must be an integer-float list.'
      print, 'Comment: '+names[data_out[0,i]]
      if lcopenpff then closepff
      return
    endif
    if aptype lt 200 or aptype gt 210 then begin
      print, '% SURF2EXO: Surface dataset have an application dataset type'
      print, '            in the range 201 to 210'
      print, 'Comment: '+names[data_out[0,i]]
      if lcopenpff then closepff
      return
    endif
    readifl, iarray, flist, farray, data_out[0, i], clab = clab
    if i eq 0 then begin
      timestamp = iarray[0]
      surfacenumber = iarray[1]
      topology= strtrim(iarray[0], 2) + strtrim(string(format='(i4.4)', surfacenumber), 2)
      for j = 0, nelem-1 do begin
        c = com2[data_out[j, 0] -offset]
        jj = strpos(c, 't=')
        if jj lt 0 then begin
          Print, '% SURF2EXO: Unable to find time value for dataset: '+strtrim(data_out[j, 0],2)
          if lcopenpff then closepff
          return
        endif
        reads, strmid(c, jj+2), tim
        times[j] = tim
      endfor
;     make sure times are unique - could have two sets of signals with same name
      junk = times[uniq(times, sort(times))]
      if n_elements(junk) ne nelem then begin
        print, '% SURF2EXO: Times must be unique for each dataset.'
        help, junk, times
        if lcopenpff then closepff
        return
      endif
      deltatime = (times[nelem-1]-times[0])*tdelta
;     check for the topology
      data_topology = get_match('surface topology '+topology, com2, nelem= nelem)
      if nelem gt 0 then data_topology = data_topology[0] else begin
        print, '% SURF2EXO: Unable to find Topology dataset'
        print, '  Search string: Surface Topology '+topology
        if lcopenpff then closepff
        return
      endelse
    endif else begin
      topol =  strtrim(iarray[0], 2) + strtrim(string(format='(i4.4)', iarray[1]), 2)
      if topol ne topology then begin
        print, '% SURF2EXO: All datasets must have the same topology'
        print, 'Comment: '+names[data_out[0,i]]
        print, 'Dataset Comment: '+clab
        print, 'IARRAY: ', iarray[0:3]
        print, 'Dataset Topology: '+topol
        print, 'First Topology: '+topology
        if lcopenpff then closepff
        return
      endif
      for j = 0, nelem-1 do begin
        c = com2[data_out[j, i] -offset]
        jj = strpos(c, 't=')
        if jj lt 0 then begin
          print, '% SURF2EXO: Unable to find "t=" signature'
          return
        endif
        reads, strmid(c, jj+2), tim
        if abs(times[j] - tim) gt deltatime then begin
          print, '% SURF2EXO: Time does not match the first dataset time'
          print, 'Comment: '+com1[data_out[0,i]- offset]
          print, 'Time Base : Time = '+strtrim (times[j], 2)+' : '+ strtrim(tim, 2)
          if lcopenpff then closepff
          return
        endif
      endfor
    endelse
    nsig_dataset[i] = iarray[3]
    datasets[i] = strmid(strcompress(com1sav[data_out[0, i] - offset]), 0, 30)
  endfor
  readhdr, data_topology[0]+offset, ier, aptyp = aptyp, clab = top_clab, /quiet
  if (ier ne 0) || (aptyp ne 200) then begin
    print, '% SURF2EXO: Unable to read topology dataset for '+topology
    help, ier, aptyp, top_clab
    if lcopenpff then closepff
    return
  endif
  readifl, top_iarray, top_flist, top_farray, data_topology+offset
  ndim = top_iarray[3]
  coord = top_iarray[2]
  topol =  strtrim(top_iarray[0], 2) + strtrim(string(format='(i4.4)', top_iarray[1]), 2)
  if topol ne topology then begin
    print, '% SURF2EXO: Topology data does not match dataset comment.'
    print, '            Dataset: '+strtrim(data_topology, 2)
    print, '            Comment: '+clab
    print, '            Search string: '+topology
    print, '            First two integers: '+strtrim(top_iarray[0], 2)+',  '+strtrim(top_iarray[1], 2)
    if lcopenpff then closepff
    return
  endif
  if ndim ne 3 then begin
    print, '% SURF2EXO: This version requires 3D data'
    if lcopenpff then closepff
    return
  endif

; number of files to write  
  nfile = 1
; number of signals
  nsignals = total(nsig_dataset)
  varname = make_array(nsignals, /string)
      
  k = 0L
  for i = 0, nsig-1 do begin
    if nsig_dataset[i] eq 1 then varname[k++] = datasets[i] else begin
      varname[k++] = datasets[i]+'_x'
      varname[k++] = datasets[i]+'_y'
      if nsig_dataset[i] eq 3 then varname[k++] = datasets[i]+'_z'
    endelse
  endfor
  if n_elements(fileout) eq 0 then $
      fileout = 'surface_'+strtrim(string(format='(i4.4)', surfacenumber), 2)+'.exo'
  fileout = find_newfile(fileout, /quiet)
;
  num_nodes = n_elements(top_farray)/ndim
  newcoord = transpose(reform(top_farray, 3, num_nodes))
  if coord eq 2 then begin
    ctheta = reform(cos(newcoord[*, 1]))
    stheta = reform(sin(newcoord[*, 1]))
    newcoord[*, 1] = newcoord[*, 0]*stheta
    newcoord[*, 0] = newcoord[*, 0]*ctheta
    ctheta =0
    stheta =0
  endif
  lcshape = 'SHELL'
  nconnect = 4L
  dlconnect = top_iarray[4:*]
  num_elem = n_elements(dlconnect)/nconnect
  dlconnect = reform (dlconnect, nconnect, num_elem)
  undefine, cdfid
  cdfid = ncdf_create(fileout, clobber = clobber)
  if size(/type, cdfid) ne 3 then begin
    print, '% SURF2EXO: Unable to open NCDF file'
    help, fileout
    if lcopenpff then closepff 
    return
  endif
  print, '% SURF2EXO: Data written to file: '+fileout
  title = 'IDL/ExodusII: '+systime(0)
  write_exo_header, cdfid, nblock = 1L, nnode = num_nodes, nelem= num_elem, $
    blo_type = lcshape, title = title, numnodevar = nsignals, numelemvar=0, ier = ier
  if ier ne 0 then begin
    print,  '% SURF2EXO: Error writing Exodus header.'
    if lcopenpff then closepff 
    return
  endif

  ncdf_varput, cdfid, coordxid, newcoord[*, 0]
  ncdf_varput, cdfid, coordyid, newcoord[*, 1]
  ncdf_varput, cdfid, coordzid, newcoord[*, 2]
  coord_names = make_array(3, /string)

  coord_names[0] = 'X axis'
  coord_names[1] = 'Y axis'
  coord_names[2] = 'Z axis'
  ncdf_varput, cdfid, coordnamid, coord_names
  ncdf_varput, cdfid, nodenamid, varname
  ncdf_varput, cdfid, connectid[0], dlconnect

; Write the datasets
  loopvar =0L
  ntimes = n_elements(times)
; Loop over different signals in the file
  for loop = 0, nsig-1 do begin
;   Loop over times for each signal
    nvar = nsig_dataset[loop]
    for nts = 0, ntimes-1  do begin
      if lcopenpff then  readifl, iarray, flist, farray, data_out[nts, loop]
      farray = reform(farray, num_nodes, nvar )
      if loop eq 0 then ncdf_varput, cdfid, timewholeid, times[nts], offset = (nts)
      for i = 0, nvar-1 do ncdf_varput, cdfid, nodevarid[i+loopvar], farray[*, i], offset = [0, nts]
    endfor
    loopvar += nvar
  endfor
  ncdf_close, cdfid
; print, times
  if lcopenpff eq 1 then closepff, 0
  end
