; ------------------------------------------------------------------------
; $Id: get_element_centers.pro,v 1.2 2007/06/06 19:40:52 lpmix Exp $
; ------------------------------------------------------------------------
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
;+
; NAME:
;       GET_ELEMENT_CENTERS
;
; PURPOSE:
;       Calculate the mean coordinates of nodes 
;
; CALLING SEQUENCE:
;
;       GET_ELEMENT_CENTERS, COORD,  ELEMENT_CENTERS [, TRIANGLES = TETS]
;
; where:
;       COORD:    Node coordinates, [Num_nodes, ndim]
;                 If COORD does not have the correct number of elements, the 
;                 values for the initial time step are used.
;       Element_Centers : Average value of the nodes for each problem element
;                         Dimensions are [Num_elements, NDim]
;           Note: Number of problem elements is less than or equal
;                 to the number of tets or triangles.
;       Triangles:Values of the nodes for each element
;                 Size is [ndim+1,  Num_Elements]
;                 TRIANGLE values are zero based, ie,  valuse are [0: num_nodes-1]
;           Note: Num_elements may be larger than the problem 
;                 if some of the problem elements are not tets
;                 or triangles.
;
; EXAMPLE:
;       Find the element centers and the node connectivity for the current file
;       Use the coordinates at the initial time step and store the centers in centers.
;         and return these values in COORD.
;         COORD = 0
;         GET_ELEMENT_CENTERS, coord, centers, tri = tri
;       
;
; MODIFICATION HISTORY:
;       Written by:     L. P. Mix, October 7, 2005
;
;-
;
  pro get_element_centers, coord, elem, triangles = tets

  on_error, 1
; add common block
@exoduscom.dat
;
  elem =0
  if n_elements(init_exo) eq 0 then begin
    print, '% GET_ELEMENT_CENTERS: No Exodus files have been opened.'
    return
  endif
  if exo_id lt 0 then begin
    print, '% GET_ELEMENT_CENTERS: No Exodus files are open.'
    return
  endif

  fid = fexo_id[exo_id]
  nblo = num_elem_blk[exo_id]
  ndim = num_dim[exo_id]
  lctets = (arg_present(tets)) ? 1 : 0
  if n_elements(coord) ne  num_nodes* ndim then coord = coord_node
  tets = -1
  elem = -1
  if nblo lt 1 then begin
    print, '% GET_ELEMENT_CENTERS: No blocks in this dataset.'
    return
  endif
  elem = make_array(loc_v[nblo], ndim)
  itet = 0
  if (lctets) then begin
    tets = make_array(ndim+1, loc_v[nblo],  /long)
    mxtet = loc_v[nblo]
  endif else tets = -1
  for i = 1, nblo do begin
    varid = ncdf_varid(fid, 'connect'+strtrim(i, 2))
    ncdf_varget, fid, varid, connect
    ss = size(connect)
    connect = connect-1
    nnode= ss[1]
    tmp = coord[connect, *]
    tmp = reform (tmp, nnode, size_V[i-1], ndim)
    elem[loc_v[i-1]:loc_v[i]-1, *] = total(tmp, 1)/nnode
    if lctets then begin
      if nnode gt ndim +1 then begin
        if ndim eq 2 then begin
           tri = make_triangles (connect)
        endif else tri = make_array[ndim+1, size_V[i-1]]
      endif else tri = temporary(connect)
      ss = size(tri)
      if ss[2]+itet gt mxtet then begin
        mxtet = mxtet+ mxtet
        tets = reform (tets, n_elements(tets), /over)
        tets = [tets, tets]
        tets = reform(tets, ndim+1, mxtet, /over)
      endif
      tets[*, itet: itet+ss[2]-1] = tri
      itet = itet+ss[2]
    endif
  endfor
  connect = 0
  if lctets && (itet lt mxtet) then tets = tets[*, 0:itet-1]
  end
