; ------------------------------------------------------------------------
; $Id$
; ------------------------------------------------------------------------
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
;+
; NAME:
;       CALC_SPH_RESPONSE
;
; PURPOSE:
;       Calculates the actual response for a dataset.  Using the response curve
;       generated by MAKE_SPH_RESPONSE, and the measured spacing of the the two
;       energy peaks on the film, this routine determines the Bragg
;       angle or distance required to match the data and does a full calculation of
;       the spherical spectrometer for these values.  The spectrometer values 
;       calculated for the fit are saved as the default values for SPH_SPEC.
;
;       This routine is the first of three procedures used to interpret the data
;       from a spherical spectrometer.  These three routines are: SPH_RESPONSE, 
;       MAKE_SPH_RESPONSE, and CALC_SPH_RESPONSE.
;
; ****  Sph_Response - Make a series of wdf arrays of with different Bragg angles,
;       or different distances.  Each WDF array is energy or xray wavelength vs 
;       position on the film.  Either or both waveforms may be saved.  A given
;       geometry is calculated and then the position of the source is varied to
;       determine the effect on the WDF array.  These waveforms may be saved to
;       a PFF file for future calculations, but take only seconds to generate.
;
; ****  Make_SPH_Response - Take the series of wdf arrays generated by SPH_RESPONSE
;       and generate a WDF array with separation vs angle or distance.  To generate
;       this waveform, we assume that two known lines are measured with the spherical
;       spectrometer.  This routine generates a response curve so that a measurement
;       of the separation in MM of the two lines on the calibration will predict an
;       angle or distance for the spherical spectrometer.
;       The syntax is:`
;          PRO MAKE_SPH_RESPONSE, WDF, E1, E2, WDF_Array
;       Where E1 and E2 are the wavelengths or energies of the two calibration 
;       energies, WDF is the array number for the response, and WDFAR are the WDF
;       arrays generated by the procedure SPH_RESPONSE.
;
; ****  Calc_Sph_Response - Calculates the actual response for a dataset.  Using
;       the response curve generated by MAKE_SPH_RESPONSE, and the measured spacing
;       of the the two energy peaks on the film, this routine determines the Bragg
;       angle or distance required to match the data and does a full calculation of
;       the spherical spectrometer for these values.  The spectrometer values 
;       calculated for the fit are saved as the default values for SPH_SPEC.
;
; CATEGORY:
;       Analysis
;
; CALLING SEQUENCE:
;
;       CALC_SPH_RESPONSE, WD_RESPON, SPACING, WD_OUTPUT
;
; INPUTS:
;       WD_RESPOND: WDF array number for the spacing of the two calibration versus
;               The Bragg angle or the Source Distance.
;       SPACING: Measured spacing of the two calibration wavelengths on the film
;       WD_Output: WDF array number for the wavelength vs position and the photon
;               energy vs position for the parameters corresponding to the observed
;               spacing of the data.  If WD_Output is not valid or undefined, it is 
;               set by SPH_SPEC.
;	
; KEYWORD PARAMETERS:
;       OUTPUT: Desired output arrays.  (See SPH_SPEC)
;               DEFAULT: Output = [12, 13]
;       SAVEIT: If set to zero, the parameters used to calculate WD_OUTPUT are 
;               not saved by SPH_SPEC
;       NPOINTS: Number of points used for the calculation.
;               Default:  NPOINTS = 2001
;
; OUTPUTS:
;       The energy and wavelength vs film position are typically output, but the
;       uers may specify other outputs through the output keyword.
;
; PROCEDURE:
;	The measured separation of the calibration peaks is used to determine the 
;       best fit to the data.  The response curve comment is then used with the 
;       measured distance or angle to calculate the parameters for the data.
;
; EXAMPLE:
;       Use the response curve in WDF array 55 to determine the best fit to the 
;       spherical spectrometer.   Calculate the energy dispersion, and magnification
;       of the data and store the curves starting in WDF array 61 when the measured
;       separation was 22.5 mm 
;          CALC_SPH_RESPONSE, 55, 22.5, 61, output = [8, 13]
;
; MODIFICATION HISTORY:
; 	Written by:	L. P. Mix, Jr., March 9, 2011
;
;
;-

PRO calc_sph_response, sph_resp, spacing, wdfout, output = output, saveit = saveit, $
     npoints = npoints

on_error, 1
npar = n_params()
if npar eq 0 then begin
  print,'% Format: CALC_SPH_RESPONSE, SPH_RESP, SPACING, WDFOUT [, OUTPUT = output]'
  print,'          [, SAVEIT = saveit] [, NPOINTS = npoints]'
  return
endif

if n_elements(sph_resp) eq 0 then begin
  print, '% CALC_SPH_RESPONSE: Spherical Spectrometer response curve must be provided'
  return
endif else sph_resp = sph_resp[0]
wd = wdvalid(sph_resp, /valid, np = np)
if np ne 1 then begin
  print, '% CALC_SPH_RESPONSE: Response curve must be a valid WDF array.'
  return
endif
wdinfo, sph_resp, ymin = ymin, ymax = ymax, xlabel = xlabel
uplabel = strupcase(xlabel[0])
ok1 = strpos(uplabel, 'DISTANCE')
ok2 = strpos(uplabel, 'ANGLE')

lcangle = (ok2 ge 0) ? 1 : 0
lcdist  = (ok1 ge 0) ? 1 : 0
help, lcangle, lcdist

saveit = (n_elements(saveit) eq 0) ? 1 : saveit[0]
npoints = (n_elements(npoints) eq 0) ? 2001 : round(npoints[0])
output = (n_elements(output) eq 0) ? [12, 13] : round(output)

if n_elements(spacing) eq 0 then begin
  print, '% CALC_SPH_RESPONSE: Peak spacing must be provided'
  return
endif else spacing = spacing[0]
if spacing lt ymin or spacing gt ymax then begin
  print, '% CALC_SPH_RESPONSE:  SPACING must be in the range: ['+strtrim (ymin, 2)+', '+strtrim(ymax, 2)+']'
  print, '                      Spacing Value: '+strtrim (spacing, 2)
  return
endif

value = (get_time (sph_resp, spacing))[0]

comm = wdcomment(sph_resp)
search = 'Crystal_Radius='
lsearch = strlen(search)
k = strpos(comm, search)
if k le 0 then begin
  print, '% CALC_SPH_RESPONSE:  Unable to determine spherical radius'
  return
endif
comm = strmid(comm, k+lsearch)
reads, comm, radius

search = '2D_spacing='
lsearch = strlen(search)
k = strpos(comm, search)
if k le 0 then begin
  print, '% CALC_SPH_RESPONSE:  Unable to determine crystal 2D spacing'
  return
endif
comm = strmid(comm, k+lsearch)
reads, comm, cry2d

search = 'Order='
lsearch = strlen(search)
k = strpos(comm, search)
if k le 0 then begin
  print, '% CALC_SPH_RESPONSE:  Unable to diffraction order'
  return
endif
comm = strmid(comm, k+lsearch)
reads, comm, order

if lcangle then begin
  search = 'Source_distance='
  lsearch = strlen(search)
  k = strpos(comm, search)
  if k le 0 then begin
    print, '% CALC_SPH_RESPONSE:  Unable to determine source distance'
    return
  endif
  comm = strmid(comm, k+lsearch)
  reads, comm, distance
  theta = value
endif else begin
  search = 'Source_angle='
  lsearch = strlen(search)
  k = strpos(comm, search)
  if k le 0 then begin
    print, '% CALC_SPH_RESPONSE:  Unable to determine source angle'
    return
  endif
  comm = strmid(comm, k+lsearch)
  reads, comm, theta
  distance = value
endelse 

search = 'Film_Pos(X,Y)=('
lsearch = strlen(search)
k = strpos(comm, search)
if k le 0 then begin
  print, '% CALC_SPH_RESPONSE:  Unable to film position'
  return
endif
film = fltarr(2)
comm = strmid(comm, k+lsearch)
reads, comm, film
search = 'Film_Angle(Rad)='
lsearch = strlen(search)
k = strpos(comm, search)
if k le 0 then begin
  print, '% CALC_SPH_RESPONSE:  Unable to determine film angle'
  return
endif
comm = strmid(comm, k+lsearch)
reads, comm, ang
film =[film, ang]
sph_spec, wdinit= wdfout, radius = radius, output =output, cry2d = cry2d, order = order, $
 distance = distance, theta = theta, film = film, gui=0, saveit = saveit, npoints = npoints
END
