; ------------------------------------------------------------------------
; $Id$
; ------------------------------------------------------------------------
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
;+
; NAME:
;       GET_NTOF_PARAM
;
; PURPOSE:
;       Returns a structure with NTOF parameters 
;
; CATEGORY:
;       Diagnostics
;
; CALLING SEQUENCE:
;       STRUCTURE = GET_NTOF_PARAM (DETECTOR, LOS)
;
; OUTPUTS:
;       STRUCTURE: NTOF structure with the following tag names:
;                  TIME: Center
;                  T_ERROR: Value of SIGMA for COEFF[1]
;                  WIDTH: FWHM 
;                         FWHM = 2*SQRT(2*ALOG(2))*COEFF[2]
;                  WIDTH_ERROR: Error in the FWHM
;                         ERROR = (SQRT(2*ALOG(2))*(COEFF[2]+SIGMA[2]) - 
;                                  SQRT(2*ALOG(2))*(COEFF[2]-SIGMA[2]))
;                  ENERGY: Ion thermal energy (See procedure below.)
;                          ENERGY(WIDTH)
;                  ENERGY_ERROR: (ENERGY( 2*SQRT(2*ALOG(2))*(COEFF[2]+SIGMA[2]) - 
;                                ENERGY( 2*SQRT(2*ALOG(2))*(COEFF[2]-SIGMA[2])) / 2
;                                Where the Energy function is defined in PROCEDURE
;                  AREA:  AREA_PEAK = SQRT(2*!PI) * COEFF[0] * COEFF[2]
;                  If an error is encountered, ZERO (0) is returned.
; INPUTS:
;       DETECTOR:  Detector number (1, 2, 3, ...)
;       LOS:       Line-of-Sight number (1, 2, ...)
;       (see SHOWNTOF)
; KEYWORD PARAMETERS:
;       DEUTERIUM: If set, get the neutron pulse timing for D-D reactions.
;       TRITIUM:   If set, get the neutron pulse timing for D-T reactions.
;           Note:  If DEUTERIUM is present then:
;                  If DEUTERIUM is zero, then TRITIUM =1
;                  If DEUTERIUM is not zero, then TRITIUM =0
;            ELSE  If TRITIUM is present then:
;                  If TRITIUM is zero, then DEUTERIUM =1
;                  If TRITIUM is not zero, then DEUTERIUM =0
;            ELSE  DEUTERIUM = 1 & TRITIUM = 0
;       DISTANCE:  Detector distance
;       NUMBER_LOS: Number of LOS
;       NAME:      Detector name
;       DETECTOR_NUMBER: Number of detectors on the LOS
;       LS_FIT:    Least square fit parameters
;       RESPONSE:  Assumed response time for the detector
;
; EXAMPLE:
;       Determine timing parameters for DD reactions from detector 1 on LOS 2
;         TIMING = GET_NTOF_PARAM (1, 2, /DEU)
;
; MODIFICATION HISTORY:
; 	Written by:	L. P. Mix, Jr., August 9, 2007
;
;-

Function get_ntof_param, detector, los, deuterium = deuterium, tritium = tritium, distance = distance, $
  response = response, number_los= number_los, name= name, detector_number= detector_number, ls_fit = ls_fit

@ntof_common.dat

on_error, 2

if n_elements(ntof_init) eq 0 then begin
  print, '% GET_NTOF_PARAM:  No data is stored.'
  return, 0
endif
if n_elements (detector) eq 0 or n_elements(LOS) eq 0 then begin
  print, '% GET_NTOF_PARAM: Detector and LOS must be provided'
  return, 0
endif
  
if (detector lt 1) || (detector gt max_detect) then begin
  print, '% GET_NTOF_PARAM: Valid detector number must be provided'
  print, '                  Detector must be in the range of 1 to MAX_DETECT'
  return, 0
endif
if (los lt 1) || (los gt numb_los) then begin
  print, '% GET_NTOF_PARAM: Valid LOS number must be provided.'
  print, '                  LOS must be in the range of 1 to NUMB_LOS'
  return, 0
endif
idet = detector -1
ilos = los -1

if n_elements(deuterium) gt 0 then begin
  lc_deuterium = keyword_set(deuterium)
endif else if N_elements(tritium) gt 0 then begin
  lc_deuterium = not keyword_set(tritium)
endif else lc_deuterium = 1

t = (keyword_set(lc_deuterium)) ? dd_time[idet, ilos] : dt_time[idet, ilos]

if arg_present(distance) then distance = detect_dist[idet, ilos]
if arg_present(response) then response = detect_resp[idet, ilos]
if arg_present(name) then name = detect_name[idet, ilos]
if arg_present(ls_fit) then ls_fit = lsfit_param
if arg_present(number_los) then number_los = numb_los
if arg_present(detector_number) then detector_number = numb_detect[ilos]
return, t
END
