; ------------------------------------------------------------------------
; $Id$
; ------------------------------------------------------------------------
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
;+
; NAME:
;       ION_TEMP
;
; PURPOSE:
;       Calculates the ion temperature from the width of the neutron pulse.
;
; CATEGORY:
;       Diagnostics
;
; CALLING SEQUENCE:
;       Temperature = ION_TEMP ( WDF )
;
; OUTPUTS:
;       Temperature - Temperature calculated from the width of the neutron signal
;
; INPUTS:
;       WDF:    Waveform array number or name with the neutron pulse.  
;
; KEYWORD PARAMETERS:
;   These keywords are defined in the documentation for GAUSSFIT
;       NTERMS:    The number of terms sent to GAUSSFIT
;                  Default:  NTERMS = 4
;                  If ESTIMATES is present, then NTERMS = N_ELEMENTS(ESTIMATES)
;       CHISQ:     Reduced chi-square goodness-of-fit statistic
;       ESTIMATES: Initial estimates for the fit coefficients
;                  Default:  If NTERMS = 4 and estimates is undefined:
;                  Estimates[0] = mx(WDF, /absolute, time = t) - Estimates[3]
;                  Estimates[1] = t - st ; where st is the start of the waveform
;                  Estimates[2] = 10
;                  Estimates[3] = average of 3 ns around beginning of waveform
;       COEFF:     Coefficients of the fit
;       SIGMA:     1-sigma estimate of error in COEFF
;                  The sigma values are added to the coefficients and new values of
;                  FWHM and ENERGY are calculated.  The difference between the values
;                  from COEFF and COEFF+SIGMA are considered the errors.
;   These keywords are specific to ION_TEMP
;       AREA_PEAK: Area of the neutron peak assumnig a Gaussian.
;                  AREA_PEAK = SQRT(2*!PI) * COEFF[0] * COEFF[2]
;       STRUCTURE: If set a structure is returned with the following tags:
;                  where the definitions are also keywords
;                  TIME: Center
;                  T_ERROR: Value of SIGMA for COEFF[1]
;                  WIDTH: FWHM 
;                         FWHM = 2*SQRT(2*ALOG(2))*COEFF[2]
;                  WIDTH_ERROR: Error in the FWHM
;                         ERROR = 2*SQRT(2*ALOG(2))*(COEFF[2]+SIGMA[2]) -FWHM
;                  ENERGY: Ion thermal energy (See procedure below.)
;                          ENERGY(WIDTH)
;                  ENERGY_ERROR: ENERGY( 2*SQRT(2*ALOG(2))*(COEFF[2]+SIGMA[2]) -ENERGY(FWHM)
;                                Where the Energy function is defined in PROCEDURE
;                  AREA:  AREA_PEAK = SQRT(2*!PI) * COEFF[0] * COEFF[2]
;       PLOTIT:    Plot the experimental data and the Gaussian fit
;       OUT:       If OUT is a valid WDF array, the fit will be saved in this array
;       CENTER:    Center value for the Gaussian = COEFF[1]  
;       FWHM:      Full width at half maximum for the Gaussian
;                    FWHM = 2*SQRT(2*ALOG(2))*COEFF[2]
;       DEUTERIUM: If set, assume the neutron pulse is from D-D reactions.
;       TRITIUM:   If set, assume the neutron pulse is from D-T reactions.
;           Note:  If DEUTERIUM is present then:
;                  If DEUTERIUM is zero, then TRITIUM =1
;                  If DEUTERIUM is not zero, then TRITIUM =0
;            ELSE  If TRITIUM is present then:
;                  If TRITIUM is zero, then DEUTERIUM =1
;                  If TRITIUM is not zero, then DEUTERIUM =0
;            ELSE  DEUTERIUM = 1 & TRITIUM = 0
;
;        DISTANCE: Distance from the target to the detector
;                  The value is taken from the neutron common block
;                  SEE:  SET_NEUTRON_PARAM
;        WINDOW:   Array of times, Left = min(Window) & Right = max(Window)
;                  If window is a scalar equal to 1, then ZOOMW will be called to allow the
;                  user to window the neutron pulse.
;           Note:  Left and Right are ignored if Window is present with two or 
;                  more values
;        LEFT:     Start time for the fit
;        RIGHT:    End time for the fit
;        RESPONSE: Width of the pulsed time response of the detector.
;                  Measured pulse width is decreased by this amount according to:
;                    WIDTH = SQRT( FWHM^2 - RESPONSE^2 )
;                  DEFAULT: RESPONSE = Detector_Response 
;                  SEE:  NTOF
;        NOSAVE:   If set, value will not be saved.
;        LCQUIT:   Returns a value of 1 if user does not store the values in the 
;                  NTOF common block.
;
; SIDE EFFECTS:
;        GET_MAXWDF() is used as a work array
;
; COMMON BLOCKS:
;        NTOF_common: Neutron detector common block
;                     Parameters may be set or shown using SET_NEUTRON_PARAM
;
; PROCEDURE:
;       The waveform is fit with a gaussian and the width is used to estimate
;       the ion temperature.  The paper of T. J. Murphy, et. al. 
;       (Rev Sci Instrum, Vol 72, January 2001, page 850, eq 3.4)
;       The width DT is determined by the ion temperature and distance 
;       according to the following:
;         DT = 0.778 * SQRT(Ti) * Distance (for DD neutrons)
;    or   Ti = (DT/0.778/Distance)^2
;         DT = 0.122 * SQRT(Ti) * Distance (for DT neutrons)
;    or   Ti = (DT/0.122/Distance)^2
;
;       where DT is in ns, Ti in keV and Distance in m.
;
; EXAMPLE:
;       Determine the ion temperature for a DD neutron pulse in WDF array 6
;         TI = ION_TEMP ( 6 )
;
;       Determine the ion Temperature of a DT neutron pulse between [2.789e-06, 2.895e-06]
;       in wavefore array 7.
;         TI = ION_TEMP(7, window = [2.789e-06, 2.895e-06], /TRITIUM)
;
;
; MODIFICATION HISTORY:
; 	Written by:	L. P. Mix, Jr., June 26, 2007
;
;-

Function ION_TEMP, wdfin, deuterium = deuterium, tritium = tritium, distance = distance, $
  window = wind, left = left, right = right, response = response, fwhm = fwhmax, $
  structure = structure, nosave = nosave, ignore = ignore, $
  center = center, nterms = nterms, chisq = chisq, estimates = estimates, coeff= coeff, $
  sigma = sigma, plotit = plotit, out = out, lcquit = lcquit, area_peak = gaus_area

@ntof_common.dat

on_error, 1

if n_elements(ntof_init) eq 0 then ntof
                                                                                       
if n_elements(wdfin) eq 0 then begin
  print, '% ION_TEMP: WDF array must be specified'
  return, 0
endif else wdf = wdfin[0]
lcquit = 1
wdf = (wdvalid(wdf, /valid, /quiet))[0]

if wdf lt 0 then begin
  print, '% ION_TEMP: WDF array must be a valid WDF array number or name'
  return, 0
endif

work = get_maxwdf()
xfr, wdf, work
if (n_elements(wind) eq 1) && wind[0] eq 1 then $
  zoomw, work, instructions = 'Pick the neutron peak beginning/end', $
  xcur = wind
if n_elements(wind) ge 2 then lim, work, wind else begin
  if n_elements(left) gt 0 or n_elements(right) gt 0 then  begin
    limits, work, xrange = wind
    if n_elements(left) gt 0 then wind[0]= min(left)
    if n_elements(right) gt 0 then wind[1]= max(right)
    lim, work, wind
  endif
endelse 
; Determine the waveform - side or bottom, 1 or 2
  clab = wdcomment(wdf)
  nlos = numb_los
  lc_los = -1
  for i = 0 , nlos-1 do $
    for j = 0, numb_detect[i]-1 do begin
      if (get_match(detect_name[j, i], clab) ge 0) then begin
        lc_los = i
        lc_detect = j
      endif 
  endfor
  if (lc_los lt 0) then begin
    if (keyword_set(ignore)) then begin
      lc_los= 0
      lc_detect = 1
    endif else begin
      print, '% ION_TEMP: Please initialize NTOF parameters'
      print, '            Use SET_NEUTRON_PARAM'
      print, '            ', CLAB,' does not match known names'
      for i = 0, nlos-1 do print, '            ', detect_name[*, i]
      return, 0
    endelse
  endif

dist = (n_elements(distance) gt 0) ? distance[0] : detect_dist[lc_detect, lc_los]

if n_elements(response) eq 0 then response = detect_resp[lc_detect, lc_los]

if n_elements(deuterium) gt 0 then begin
  fudge = keyword_set(deuterium) ? 0.778 : 0.122
endif else if N_elements(tritium) gt 0 then begin
  fudge = keyword_set(tritium) ? 0.122 : 0.778 
endif else fudge =  0.778 

lc_deuterium = (fudge gt 0.5) ? 1 : 0

nel =  n_elements (estimates)
if nel gt 2 then begin
  nterms = nel
  estim = estimates
endif else if n_elements(nterms) eq 0 then nterms = 4

w2i, x, y, work
if (nel eq 0) && (nterms eq 4) then begin
  st = min(x, max = en)
  bl = ave(wdf, left = st-1.5e-9, right = st+1.5e-9, /quiet)
  w = 10.
  m = mn(wdf, lef= st, right = en , time= t, /quiet)
  estim = [m - bl,(t-st)*1e9, w, bl]
endif
x = x*1e9
yfit = gaussfit((x-x[0]), y, coeff, nterms=nterms, sigma = sigma, chisq = chisq, $
       estimates= estim)

; 

fw = 2d0*sqrt(2*alog(2))*coeff[2]
fwp = 2d0*sqrt(2*alog(2))*(coeff[2]+sigma[2])
fwm = 2d0*sqrt(2*alog(2))*coeff[2]-sigma[2]
if response gt 0 then begin
  res2 = response*response
  fwpc  = sqrt(fwp*fwp - res2)
  fwmc  = sqrt(fwm*fwm - res2)
  fw_cor= sqrt(fw*fw - res2) 
endif else begin
  fw_cor= fw
  fwpc  = fwp
  fwmc  = fwm
endelse
fwhmax = fw
ti = fw_cor/fudge/dist
ti = ti*ti

tip = (fwpc/fudge/dist)^2
tim = (fwmc/fudge/dist)^2
center = coeff[1]+x[0]

gaus_area = abs(sqrt(!pi+!pi)*coeff[0]*coeff[2])

if n_elements(plotit) eq 0 then plotit  = 1    
if keyword_set (plotit) then $
  plo, work, /over, /color, leg = [.1, .6]

if n_elements(out) gt 0 then begin
  out = out[0]
  if out le 0 or out gt get_maxwdf() then out = work
endif else if keyword_set(plotit) then out = work
if n_elements(out) gt 0 then $
  i2w, x*1e-9, yfit, out, clab = 'Gaussian fit to '+ wdcomment(work)
if keyword_set(plotit) then begin
  plo, /no, out, /over, /color
  oplot, [1,1]*center*1e-9 , !y.crange
endif
t = {time:float(center),  t_error: float(sigma[1]), width: float(fw_cor), $
      width_error:float(fwpc-fwmc)*0.5, energy:float(ti) , $
      energy_error: float(tip-tim)*0.5, AREA: float(gaus_area)}

ti = keyword_set(STRUCTURE) ? t : ti
if keyword_set(ignore) then  return, ti

if keyword_set(lc_deuterium) then begin
  ans = (keyword_set(nosave)) ? 'N' :  ''
  while strlen(ans) eq 0 do begin
    print, 'Do you want to save this Deuterium fit? (Yes or No) '
    read, ans
    ans = strupcase(strtrim(ans, 2))
  endwhile 

  if (strmid(ans, 0, 1) eq 'N') then begin
    lcquit = 1
    return, ti
  endif else lcquit = 0
;  yes_no_ques, lcquit, 'Do you want to save this Deuterium fit?', 'Save Fit?', $
;      menu=['  S a v e  D a t a  ', '  R e D o  F i t  ']
;  if lcquit eq 1 then return, ti
  dd_time[lc_detect, lc_los] = t
  if los_tlb[lc_los] gt 0 then begin
    widget_control, dd_field[0, lc_detect, lc_los], set_value = center
    widget_control, dd_field[1, lc_detect, lc_los], set_value = fw_cor
    widget_control, dd_field[2, lc_detect, lc_los], set_value = t.energy
  endif
  if dd_time[0, lc_los].time gt 0 and dd_time[1, lc_los].time gt 0 then begin
    distanc = abs(detect_dist[1, lc_los]-detect_dist[0, lc_los])
    fit_velocity[0, lc_los]= distanc/abs(dd_time[1, lc_los].time-dd_time[0, lc_los].time)*1e9
    fit_energy[0, lc_los]= energ(fit_velocity[0, lc_los])*1e-6
    bang_time[0, lc_los] =dd_time[0, lc_los].time - detect_dist[0, lc_los]/ $
      fit_velocity[0, lc_los]*1e9
    if los_tlb[lc_los] gt 0 then begin
      widget_control, fit_ener_field[0, lc_los], set_value =  fit_energy[0, lc_los]
      widget_control, dd_bang_field[0, lc_los], set_value =  bang_time[0, lc_los]
    endif
  endif   
endif else begin
  ans = (keyword_set(nosave)) ? 'N' :  ''
  while strlen(ans) eq 0 do begin
    print, 'Do you want to save this Tritium fit? (Yes or No) '
    read, ans
    ans = strupcase(strtrim(ans, 2))
  endwhile
  if (strmid(ans, 0, 1) eq 'N') then begin
    lcquit = 1
    return, ti
  endif else lcquit = 0

;  yes_no_ques, lcquit, 'Do you want to save this Tritium fit?', 'Save Fit?', $
;      menu=['  S a v e  D a t a  ', '  R e D o  F i t  ']
;  if lcquit eq 1 then return, ti
  dt_time[lc_detect, lc_los] = t

  if los_tlb[lc_los] gt 0 then begin
    widget_control, dt_field[0, lc_detect, lc_los], set_value = center
    widget_control, dt_field[1, lc_detect, lc_los], set_value = fw_cor
    widget_control, dt_field[2, lc_detect, lc_los], set_value = t
  endif
  if dt_time[0, lc_los].time gt 0 and dt_time[1, lc_los].time gt 0 then begin
    distanc = abs(detect_dist[1, lc_los]-detect_dist[0, lc_los])
    fit_velocity[1, lc_los]= distanc/abs(dt_time[1, lc_los].time-dt_time[0, lc_los].time)*1e9
    fit_energy[1, lc_los]= energ(fit_velocity[1, lc_los])*1e-6
    bang_time[1, lc_los] =dt_time[0, lc_los].time - detect_dist[0, lc_los]/ $
        fit_velocity[1, lc_los]*1e9
    if los_tlb[lc_los] gt 0 then begin
      widget_control, fit_ener_field[1, lc_los], set_value =  fit_energy[1, lc_los]
      widget_control, dt_bang_field[0, lc_los], set_value =  bang_time[1, lc_los]
    endif
  endif

endelse
  
lcquit = 0
return, ti

END
