; ------------------------------------------------------------------------
; $Id$
; ------------------------------------------------------------------------
;
; Copyright 2005 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;

Pro read_ndata, sample = test, redo = redo, filter = filter, lineout = lineout, $
                output = str_out

;+
; NAME:
;       READ_NDATA
;
; PURPOSE:
;       Read TIFF files and analyze image to extract optical fiber streak data
;         to determine focused electron beam size.
;
; CATEGORY:
;       Data Analysis
;
; CALLING SEQUENCE:
;
;       READ_NDATA
;
; KEYWORD PARAMETERS:
;       FILTER: Unless FILTER is specified equal to zero, each fiber is 
;                 filtered in time with a Fourier filter.
;		The fiber signal is converted to a waveform array vs pixels.
;               The signal is stored in WFIL = GET_MAXWDF()-1
;               The amplitude is filtered with the following:
;                 FILT, WDF, DELTA = FILTER
;
;               Default: IF N_ELEMENTS(FILTER) eq 0 THEN FILTER = 50
;                        FILTER =  (FILTER = 0) ? 0 : (FILTER > 5)
;       LINEOUT:If set, the values of the flat field from the lineout will be 
;               used for normalization.  If undefined or zero, the flatfield
;               will be integrated over the width of the image.
;       OUTPUT: If a valid structure number, output will be stored in this structure.
;               Note:  Normal output structure (GET_MAXSTR()-2) is not modified.
;       REDO:   If specified, current TIFF images will be processed again.
;               The IMAGE structure is in GET_MAXSTR() -7
;               The Flat-Field structure is in GET_MAXSTR() - 5
;       SAMPLE: If set, used for analysis of sample data which is coded in this
;               routine.
;
; OUTPUTS:
;       The normalized image is in GET_MAXSTR() - 2 and in OUTPUT (if provided)
;       Lineouts and intermediate results are stored beginning in GET_MAXWDF()-12
;
; PROCEDURE:
;  0.  If REDO is set, go to #6
;  1.  Procedure assumes 84 fibers
;         Spatial position is:
;         Xspace = [FINDGEN(18), FINDGEN(48)*0.5+ 18, 18+24.5+FINDGEN(18)]
;         The fibers are not uniformly spaced due to ithe inclusion of timing 
;         fibers and background fibers.
;  2.  Read the 4 TIFF images consisting of:
;         Data Image
;         Data Background
;         Flat-Field Image
;         Flat-Field Background
;  3.  Filter the raw images with a MEDIAN filter of width 3 pixels.
;  4.  Subtract the background and store the data and flat-field images
;  5.  Rotate the images to align the fiber streaks with the pixels coordinates.
;  6.  Take a lineout from the data and flat field images
;  7.  Determine the location of the first fiber or the fiber number for the 
;      for the first fiber in the image.  Bad timing can cause the first fiber 
;      to be some other number than 1.  
;  8.  Using the fiber number and location (item 7) for the first observable fiber,
;      determine the exact fiber location for in the Data and flat-field images
;      from the lineouts in item 6.
;  9.  Separate the Data image into NFIB strips and sum over the width of each strip.
;      NFIB is the number of fibers found in step 8.
; 10.  Calculate a normalization to account for the transmission efficiency of each
;      fiber.  If the keyword, LINEOUT, is set, calculate the normalization by
;      summing the lineout over each fiber to obtain NFIB normalizations.  
;      If LINEOUT is zero or missing sum the flat-field image over all times and 
;      over the width of each fiber
; 11.  Divide the data from step 9 by the normalizations in step 10 and store the 
;      image in (GET_MAXSTR()-2)
;
; EXAMPLE:
;      Read the tiff files from data and background shots and calculate an image.
;      Save the image in structure 51.
;        READ_NDATA, output = 51
;
;      Analyze the data above using a lineout for data normalization.
;      Save the image in structure 52.
;        READ_NDATA, OUTPUT = 52, /REDO, /LINEOUT
;     
; MODIFICATION HISTORY:
; 	Written by:	L. P. Mix, Jr., August 6, 2004
;	April, 2006     Added help data, other changes.
;
;-

; If lineout is set, use lineout for the normalization.

; filter if set will use a low pass filter on the data from each fiber
; if filter lt 5 then filter = 5
; if filter not specified then filter = 50
  if n_elements (filter) eq 0 then filter = 50
  if filter lt 5 then filter = 5
;   The value of filter is used as the value of delta
;     in the filt routine.
;
; 
; Threshold = default = 10%
; Redo, if set, use stored images 
;               Assume that the data is in get_maxstr()-8
;               Assume the focus file is in get_maxstr()-7
;
on_error, 1

if n_elements(threshold) eq 0 then threshold = 0.075
if threshold gt 1.0 then threshold = 0.075

; Modified Streak camera Channels
xarray = findgen(85) * 350 + 700
xarray[9:17] = xarray[9:17]+650
xarray[18:41] = xarray[18:41]+1300
xarray[42:65] = xarray[42:65]+1950
xarray[66:74] = xarray[66:74]+2600
xarray[75:83] = xarray[75:83]+3250
; add an extra
numb_fiber = 84 ; 18+ 48 + 18
xarray[84]= xarray[83]+ 35000

xspace = [findgen(18), findgen(48)*0.5+ 18, 18+24.5+findgen(18)]

; Comb
comb = make_array(5)
comb[0]= total(xarray[8:9])*0.5
comb[1]= total(xarray[17:18])*0.5
comb[2]= total(xarray[41:42])*0.5
comb[3]= total(xarray[65:66])*0.5
comb[4]= total(xarray[74:75])*0.5
; background
backgr = make_array(2)
backgr[0]= 350
backgr[1]= xarray[83]+350

; scale for images (microns/pixel)
Xscale = 18.044
xdata = xarray/xscale
xdata = xdata -xdata[0]

; work arrays for wdf and images

work = get_maxwdf()-12
wstr = get_maxstr()-8

if keyword_set(redo) then begin
  img = wstr+1
  flt = wstr+3
  wstr = wstr+4
  datafile = strcomment(img)
  focfile = strcomment(flt)
endif else begin

  if keyword_set(test) then begin
    datafile = '/remote/lpmix/consult/sportil/bump/45a.TIF'
    backfile = '/remote/lpmix/consult/sportil/bump/44back.TIF'
    focfile = '/remote/lpmix/consult/sportil/bump/44a.TIF'
    ff_backfile = '/remote/lpmix/consult/sportil/bump/44back.TIF'
  endif else begin
    datafile = dialog_pickfile(default_extension = '.tif', get_path= oldpath, $
      /must_exist, Title= 'Pick the DATA file')
    backfile = dialog_pickfile(default_extension = '.tif', get_path= oldpath, $
       /must_exist, Title= 'Pick the Background file', path = oldpath)
    Focfile = dialog_pickfile(default_extension = '.tif', get_path= oldpath, $
       /must_exist, Title= 'Pick the focus file', path = oldpath)
    ff_backfile = dialog_pickfile(default_extension = '.tif', get_path=oldpath, $
       /must_exist, Title= 'Pick the focus background file', path = oldpath)
    if strlen(ff_backfile) lt 1 then begin
      Print, '% READ_DATA: No background file selected for the Focus File'
      print, '             Background for data file will be used'
      ff_backfile= backfile
    endif
  endelse

; Read the data
  img = read_tiff(datafile)
  bkg = read_tiff(backfile)
  ff = read_tiff(focfile)
  ff_bkg = read_tiff(ff_backfile)

; filter the data
  print, ' Filtering the data'
  print, ' Image data range = [ ', strtrim(min(img, max = m), 2), $
    ', ', strtrim (m, 2), ']'
  img = median(img, 3)
  print, ' Image data range after filter = [ ', strtrim(min(img, max = m), 2), $
    ', ', strtrim (m, 2), ']'
  print, ' Background data range = [ ', strtrim(min(bkg, max = m), 2), $
    ', ', strtrim (m, 2), ']'
  bkg = median(bkg, 3)
  print, ' Background data range after filter = [ ', $
    strtrim(min(bkg, max = m), 2), ', ', strtrim (m, 2), ']'
  print, ' Focus data range = [ ', strtrim(min(ff, max = m), 2), $
    ', ', strtrim (m, 2), ']'
  ff = median(ff, 3)
  print, ' Focus data range after filter = [ ', strtrim(min(ff, max = m), 2), $
    ', ', strtrim (m, 2), ']'
  print, ' Focus background data range = [ ', strtrim(min(ff_bkg, max = m), 2), $
    ', ', strtrim (m, 2), ']'
  ff_bkg = median(ff_bkg, 3)
  print, ' Focus background data range after filter = [ ', $
    strtrim(min(ff_bkg, max = m), 2), ', ', strtrim (m, 2), ']'
; Save the image
  img = (img - bkg)
  arr2str, wstr, float(img) , clab = 'Image - '+datafile, file = datafile
  img = wstr & wstr = wstr+1
  alignstr, img, wstr, angle= angle , width = 10./2000, yr= [500, 1500]
  img = wstr
  wstr = wstr+1
  ff = (ff-ff_bkg)
  arr2str, wstr, float(ff) , clab = 'Flat-Field - '+focfile, file = Focfile
  flt = wstr & wstr = wstr+1
; plotstr, flt, cn=2
  alignstr, flt, wstr, angle= angle , width = 10./2000, yr= [500, 1500]
  flt= wstr
  wstr = wstr+1
endelse

imag = get_array(img, -1)
flat = get_array(flt, -1)
ss = size(imag) 
sf = size(flat) 

delwdf, work, /quiet

print, '*** Select the signal region ***'
while (wdvalid(work, /valid, /quiet))[0] lt 0 do $
  linstr, img, work, /vert, wid = 120
filt, work, work+1, d = 8.0

delwdf, work+2, /quiet
while (wdvalid(work+2, /valid, /quiet))[0] lt 0 do $
  linstr, flt, work+2, /vert, wid = 120
filt, work+2, work+3, d = 8.0


st = make_array(128, /long)
en = st
stf = st
enf = st
middle = Make_array(128, /long, value = 65536)
middlef = middle
imag_val = middle
flat_val = middle

nfib = 0L

; Width of fiber = 9
wfiber= 9.5
skip = 11.5
nex  = 20
bnex = 55
hwid = wfiber/2
finish = 0
maxx = sf[2] -31
ans = ''
yes_no_ques, lcno, 'Is the first channel visible?','Fiber Check'
star = 1
if lcno ne 0 then $
  read,  prompt='Enter the first visible data channel (1 to 84): ', star
if star lt 1 or star gt 84 then begin
  print, '% READ_NDATA: The first visible line may not have a value of ', star
  print, '              Use "READ_NDATA, /REDO" to rerun'
  return
endif

w2i, xwor, ywor, work+1
n4= n_elements(xwor)/4
n50 = n4/12.5 > 5
yb = ywor[0:n4]

ok = where (yb gt max(yb)/4, nok)
if nok gt 0 then zoomrange = [xwor[ok[0]-n50], xwor[ok[0]+n50]] else $
  zoomrange = [xwor[n50], xwor[n4-n50]]

zoomw, work+1, xc = xc, zoomrange = zoomrange, instructions = $
  'Use right mouse button in right window to indicate the first data channel'

mid = xc[0]
offset = mid -xdata[star-1]
idata = xdata+offset

lef = mid - skip
ri = mid+skip
junk = mx(work+1, lef= lef, ri= ri, /quiet, min = mm, time = nt)

while ri le maxx do begin
  peak = junk
  time = round(nt)
  m = min(abs(idata-nt), jmid)
;
  middle[nfib] = time
  st[nfib] = time - hwid
  en[nfib] = time + hwid  
  imag_val[nfib] = jmid

  fudge = nt -idata[jmid]
  if abs(fudge) gt 2 then begin
    idata = idata +fudge
    fudge =0
  endif
  nfib= nfib+1
  if nfib eq numb_fiber then break
  jmid = jmid+1
  newmid = idata[jmid]+fudge
  lef = newmid - skip
  ri =  newmid + skip
  repeat begin
    junk = mx(work+1, lef= lef, ri= ri, min= mm, /quiet, time = nt) 
    if (nt-lef lt 2.1) || (ri-nt lt 2.1) then junk =0.0
    ri = ri + nex
  endrep until junk gt 0.4*peak || ri gt maxx
endwhile

i2w, middle[0:nfib-1], 60, clab = 'Middle for Images'
image = make_array(ss[2])
for i = 0, nfib-1 do image[st[i]:en[i] ] = 0.980
i2w, image, work+4, clab = 'Image Fibers'
nfibd = nfib
; now do the same for the flat field
yes_no_ques, lcno, 'Is the first channel visible?','Fiber Check'
star = 1
if lcno ne 0 then $
  read,  prompt='Enter the first visible data channel (1 to 84): ', star
if star lt 1 or star gt 84 then begin
  print, '% READ_NDATA: The first visible line may not have a value of ', star
  print, '              Use "READ_NDATA, /REDO" to rerun'
  return
endif
w2i, xwor, ywor, work+3
n4= n_elements(xwor)/4
n50 = n4/12.5 > 5
yb = ywor[0:n4]

ok = where (yb gt max(yb)/4, nok)
if nok gt 0 then zoomrange = [xwor[ok[0]-n50], xwor[ok[0]+n50]] else $
  zoomrange = [xwor[n50], xwor[n4-n50]]

zoomw, work+3, xc = xc, zoomrange = zoomrange, instructions = $ 
  'Use right mouse button in right window to indicate the first data channel'
mid = xc[0]
offset = mid -xdata[star-1]
idata = xdata+offset

if imag_val[0] gt 0 then mid = idata[imag_val[0]]
lef = mid-skip
ri = mid +skip
nfib =0
junk = mx(work+3, lef= 0, ri= ri, min = mm, /quiet, time = nt)
nok =0
while ri le maxx do begin
  peak = junk
  time = round(nt)
  m = min(abs(idata-time), jmid)
;
; we need one background channel for every data channel
; jmid will be at least as large as imag_val because of the 
; way the middle is set.
;
  if jmid gt imag_val[nfib] then begin
    delt = idata[jmid] - nt
    jmid = imag_val[nfib]
    nt = idata[jmid] -delt
    time = round(nt)
  endif
  middlef[nfib] = time
  stf[nfib] = time - hwid
  enf[nfib] = time + hwid
  flat_val[nfib] = jmid
  fudge = nt -idata[jmid]
  if abs(fudge) gt 2 then begin
    idata = idata +fudge
    fudge =0
  endif
  nfib= nfib+1
  jmid = jmid+1

  
  if nfib eq nfibd then break
  newmid = idata[imag_val[nfib]] + fudge
  lef = newmid - skip
  ri =  newmid + skip

  repeat begin
    junk = mx(work+3, lef= lef, ri= ri, min = mm, /quiet, time = nt)
    if (nt-lef lt 2.1) || (ri-nt lt 2.1) then junk =0.0
    ri = ri + nex
  endrep until junk gt 0.4*peak || ri gt maxx
endwhile

image = make_array(sf[2])
for i = 0, nfib-1 do image[stf[i]:enf[i] ] = 0.99
i2w, image, work+5, clab= 'Flat Field Fibers'
i2w, middlef[0:nfib-1], 61, clab= 'Middle for focus'
nfibf = nfib
;help, nfibd
;help, nfibf
; check data and focus for the same number of points
; and see if they are identical

if nfibf ne nfibd then begin
  print, '% READ_NDATA: Number of fibers is different'
  help, nfibf, nfibd
  print, '              Please contact Paul Mix if this is not resolved'
  return
endif

; stf = stf[0:nfib-1]
; enf = enf[0:nfib-1]
; st = st[0:nfib-1]
; en = en[0:nfib-1]

; print, st, en, stf, enf
; Convert fiber positions to spatial positions
;
values = xspace[imag_val[0:nfib-1]]
image = make_array(ss[1], nfib)
for i = 0, nfib-1 do image[*, i] = total(imag[*, st[i]:en[i]], 2)
if keyword_set(filter) then begin
  wfil = get_maxwdf()-1
  for i = 0, nfib-1 do begin
    i2w, [0, 1], image[*, i], wfil
    filt, wfil, d= filter
    image[*, i] = getyf(wfil)
  endfor
endif
ffield = make_array(sf[1], nfib)
if keyword_set(lineout) then begin
  yy = getyf(work+3)
  nor = make_array(nfib)
  for i = 0, nfib-1 do nor[i] = total(yy[ stf[i]:enf[i]])
  for i = 0, nfib-1 do ffield[*, i] = nor[i]
  i2w, nor, 41
endif else begin
  for i = 0, nfib-1 do ffield[*, i] = total(flat[*, stf[i]:enf[i]], 2)
  nor = total(ffield, 1)
  i2w, nor, 42
endelse
arr2str, wstr, image, get_array(img, 1), values,  $
  xlab = 'Distance', ylab ='Fiber Position (mm)', $
    clab = 'Fiber Data - ' + datafile
arr2str, wstr+1, ffield, get_array(flt, 1), values,  $
  xlab = 'Distance', ylab ='Fiber Position (mm)', $
    clab = 'Flat Field Data - ' + focfile
for i = 0, nfib-1 do image[*, i] = image[*, i]/nor[i]
arr2str, wstr+2, image, get_array(img, 1), values,  $
  xlab = 'Distance', ylab ='Fiber (mm)', $
    clab = 'Normalized Fiber Data - ' + datafile
if keyword_set(str_out) then $
  if (str_out gt 0) && (str_out le get_maxstr()) then xfrstr, wstr+2, str_out
i2w, nor, work+6, clab='Flat Field Total vs Fiber', xlab= 'Fiber', ylab = 'Value'
i2w, values,nor, work+7, /xydata, clab='Flat Field Total vs Position', $
 xlab= 'Positon', ylab = 'Value'

end
