; ------------------------------------------------------------------------
; $Id$
; ------------------------------------------------------------------------
;
; Copyright 2005 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;

Pro read_ndata, sample = test, redo = redo, filter = filter, lineout = lineout, $
                output = str_out

;+
; NAME:
;       READ_NDATA
;
; PURPOSE:
;       Read TIFF files and analyze image to extract optical fiber streak data
;         to determine focused electron beam size.
;
; CATEGORY:
;       Data Analysis
;
; CALLING SEQUENCE:
;
;       READ_NDATA
;
; KEYWORD PARAMETERS:
;       FILTER: Unless FILTER is specified equal to zero, each fiber is 
;                 filtered in time with a Fourier filter.
;		The fiber signal is converted to a waveform array vs pixels.
;               The signal is stored in WFIL = GET_MAXWDF()-1
;               The amplitude is filtered with the following:
;                 FILT, WDF, DELTA = FILTER
;
;               Default: IF N_ELEMENTS(FILTER) eq 0 THEN FILTER = 50
;                        FILTER =  (FILTER = 0) ? 0 : (FILTER > 5)
;       LINEOUT: If a positive number, the values of the flat field from the lineout
;               will be used for normalization.
;               If undefined or less than or equal to zero, the flat field image will
;               be integrated veritcally and the final value of the integral will be used
;               to determine the range of data to be used for normalization. 
;               This vertical integral is VERT_INT
;               a. If lineout is zero or less than -0.75 then the values of the flat field 
;                  corresponding to values of the VERT_INT greater than 0.1*max(VERT_INT)
;                  will be used.
;               b. If lineout is less than zero and greater than -0.75, then the domain will 
;                  be determined by the domain of values of VERT_INT greater than 
;                  abs(LINEOUT)*Max(VERT_INT).
;             
;       OUTPUT: If a valid structure number, output will be stored in this structure.
;               Note:  Normal output structure (GET_MAXSTR()-2) is not modified.
;       REDO:   If specified, current TIFF images will be processed again.
;               The IMAGE structure is in GET_MAXSTR() -7
;               The Flat-Field structure is in GET_MAXSTR() - 5
;       SAMPLE: If set, used for analysis of sample data which is coded in this
;               routine.
;
; OUTPUTS:
;       The normalized image is in GET_MAXSTR() - 2 and in OUTPUT (if provided)
;       Lineouts and intermediate results are stored beginning in GET_MAXWDF()-12
;
; PROCEDURE:
;  0.  If REDO is set, go to #6
;  1.  Procedure assumes 84 fibers
;         Spatial position is:
;         Xspace = [FINDGEN(18), FINDGEN(48)*0.5+ 18, 18+24.5+FINDGEN(18)]
;         The fibers are not uniformly spaced due to ithe inclusion of timing 
;         fibers and background fibers.
;  2.  Read the 4 TIFF images consisting of:
;         Data Image
;         Data Background
;         Flat-Field Image
;         Flat-Field Background
;  3.  Filter the raw images with a MEDIAN filter of width 3 pixels.
;  4.  Subtract the background and store the data and flat-field images
;  5.  Rotate the images to align the fiber streaks with the pixels coordinates.
;  6.  Take a lineout from the data and flat field images
;  7.  Determine the location of the first fiber or the fiber number for the 
;      for the first fiber in the image.  Bad timing can cause the first fiber 
;      to be some other number than 1.  
;  8.  Using the fiber number and location (item 7) for the first observable fiber,
;      determine the exact fiber location for in the Data and flat-field images
;      from the lineouts in item 6.
;  9.  Separate the Data image into NFIB strips and sum over the width of each strip.
;      NFIB is the number of fibers found in step 8.
; 10.  Calculate a normalization to account for the transmission efficiency of each
;      fiber.  See LINEOUT for description.
; 11.  Divide the data from step 9 by the normalizations in step 10 and store the 
;      image in GET_MAXSTR()-2 and in OUTPUT (if provided and valid).
;
; EXAMPLE:
;      Read the tiff files from data and background shots and calculate an image.
;      Save the image in structure 51.
;        READ_NDATA, output = 51
;
;      Analyze the data above using a lineout for data normalization.
;      Save the image in structure 52.
;        READ_NDATA, OUTPUT = 52, /REDO, /LINEOUT
;     
; MODIFICATION HISTORY:
; 	Written by:	L. P. Mix, Jr., August 6, 2004
;	April, 2006     Added help data, other changes.
;
;-

; If lineout is set, use lineout for the normalization.

; filter if set will use a low pass filter on the data from each fiber
; if filter lt 5 then filter = 5
; if filter not specified then filter = 50
  if n_elements (filter) eq 0 then filter = 50
  if filter lt 5 then filter = 5
;   The value of filter is used as the value of delta
;     in the filt routine.
;
; 
; Threshold = default = 10%
; Redo, if set, use stored images 
;               Assume that the data is in get_maxstr()-8
;               Assume the focus file is in get_maxstr()-7
;
on_error, 1

if n_elements(threshold) eq 0 then threshold = 0.075
if threshold gt 1.0 then threshold = 0.075

; Modified Streak camera Channels
xarray = findgen(85) * 350 + 700
xarray[9:17] = xarray[9:17]+650
xarray[18:41] = xarray[18:41]+1300
xarray[42:65] = xarray[42:65]+1950
xarray[66:74] = xarray[66:74]+2600
xarray[75:83] = xarray[75:83]+3250
; add an extra
numb_fiber = 84 ; 18+ 48 + 18
xarray[84]= xarray[83]+ 35000

xspace = [findgen(18), findgen(48)*0.5+ 18, 18+24.5+findgen(18)]

; Comb
comb = make_array(5)
comb[0]= total(xarray[8:9])*0.5
comb[1]= total(xarray[17:18])*0.5
comb[2]= total(xarray[41:42])*0.5
comb[3]= total(xarray[65:66])*0.5
comb[4]= total(xarray[74:75])*0.5
; background
backgr = make_array(2)
backgr[0]= 350
backgr[1]= xarray[83]+350

; scale for images (microns/pixel)
Xscale = 18.044
Xscale = 18.5
xdata = xarray/xscale
xdata = xdata -xdata[0]

; work arrays for wdf and images

work = get_maxwdf()-12
wstr = get_maxstr()-8

if keyword_set(redo) then begin
  img = wstr+1
  flt = wstr+3
  wstr = wstr+4
  datafile = strcomment(img)
  focfile = strcomment(flt)
endif else begin

  if keyword_set(test) then begin
    datafile = '/remote/lpmix/151a.TIF'
    backfile = '/remote/lpmix/153background.TIF'
    focfile = '/remote/lpmix/155flat.TIF'
    ff_backfile = '/remote/lpmix/153background.TIF'
  endif else begin
    datafile = dialog_pickfile(default_extension = '.tif', get_path= oldpath, $
      /must_exist, Title= 'Pick the DATA file')
    backfile = dialog_pickfile(default_extension = '.tif', get_path= oldpath, $
       /must_exist, Title= 'Pick the Background file', path = oldpath)
    Focfile = dialog_pickfile(default_extension = '.tif', get_path= oldpath, $
       /must_exist, Title= 'Pick the focus file', path = oldpath)
    ff_backfile = dialog_pickfile(default_extension = '.tif', get_path=oldpath, $
       /must_exist, Title= 'Pick the focus background file', path = oldpath)
    if strlen(ff_backfile) lt 1 then begin
      Print, '% READ_DATA: No background file selected for the Focus File'
      print, '             Background for data file will be used'
      ff_backfile= backfile
    endif
  endelse

; Read the data
  img = read_tiff(datafile)
  bkg = read_tiff(backfile)
  ff = read_tiff(focfile)
  ff_bkg = read_tiff(ff_backfile)

; filter the data
  print, ' Filtering the data'
  print, ' Image data range = [ ', strtrim(min(img, max = m), 2), $
    ', ', strtrim (m, 2), ']'
  img = median(img, 3)
  print, ' Image data range after filter = [ ', strtrim(min(img, max = m), 2), $
    ', ', strtrim (m, 2), ']'
  print, ' Background data range = [ ', strtrim(min(bkg, max = m), 2), $
    ', ', strtrim (m, 2), ']'
  bkg = median(bkg, 3)
  print, ' Background data range after filter = [ ', $
    strtrim(min(bkg, max = m), 2), ', ', strtrim (m, 2), ']'
  print, ' Focus data range = [ ', strtrim(min(ff, max = m), 2), $
    ', ', strtrim (m, 2), ']'
  ff = median(ff, 3)
  print, ' Focus data range after filter = [ ', strtrim(min(ff, max = m), 2), $
    ', ', strtrim (m, 2), ']'
  print, ' Focus background data range = [ ', strtrim(min(ff_bkg, max = m), 2), $
    ', ', strtrim (m, 2), ']'
  ff_bkg = median(ff_bkg, 3)
  print, ' Focus background data range after filter = [ ', $
    strtrim(min(ff_bkg, max = m), 2), ', ', strtrim (m, 2), ']'
; Save the image
  img = (img - bkg)
  arr2str, wstr, float(img) , clab = 'Image - '+datafile, file = datafile
  img = wstr & wstr = wstr+1
  alignstr, img, wstr, angle= angle , width = 10./2000, yr= [500, 1500]
  img = wstr
  wstr = wstr+1
  ff = (ff-ff_bkg)
  arr2str, wstr, float(ff) , clab = 'Flat-Field - '+focfile, file = Focfile
  flt = wstr & wstr = wstr+1
; plotstr, flt, cn=2
  alignstr, flt, wstr, angle= angle , width = 10./2000, yr= [500, 1500]
  flt= wstr
  wstr = wstr+1
endelse

imag = get_array(img, -1)
flat = get_array(flt, -1)
ss = size(imag) 
sf = size(flat) 

delwdf, work, /quiet

print, '*** Select the signal region ***'
while (wdvalid(work, /valid, /quiet))[0] lt 0 do $
  linstr, img, work, /vert, wid = 120
filt, work, work+1, d = 8.0

delwdf, work+2, /quiet
while (wdvalid(work+2, /valid, /quiet))[0] lt 0 do $
  linstr, flt, work+2, /vert, wid = 120
filt, work+2, work+3, d = 8.0


st = make_array(128, /long)
en = st
stf = st
enf = st
middle = Make_array(128, /long, value = 65536)
middlef = middle
imag_val = middle
flat_val = middle

nfib = 0L

; Width of fiber = 9
wfiber= 8.5
skip = 9.5
hwid = round(wfiber/2)

ans = ''
yes_no_ques, lcno, 'Is the first Flat-Field channel visible?','Fiber Check'
star = 1
if lcno ne 0 then $
  read,  prompt='Enter the first Flag-Field Fiber channel (1 to 84): ', star
if star lt 1 or star gt 84 then begin
  print, '% READ_NDATA: The first visible line may not have a value of ', star
  print, '              Use "READ_NDATA, /REDO" to rerun'
  return
endif

w2i, xwor, ywor, work+1
n4= n_elements(xwor)/4
n50 = n4/12.5 > 5
yb = ywor[0:n4]

ok = where (yb gt max(yb)/4, nok)
if nok gt 0 then zoomrange = [xwor[ok[0]-n50], xwor[ok[0]+n50]] else $
  zoomrange = [xwor[n50], xwor[n4-n50]]
xc = 0
while xc[0] eq 0 do $
  zoomw, work+3, xc = xc, zoomrange = zoomrange, instructions = $
  'Use right mouse button in right window to indicate the first data channel'

mid = round(xc[0])
cur = star-1
idata = xdata +mid
fudge= 0
nfib = 0
last_junk = 0.00
last_time = mid
while cur lt numb_fiber do begin
  mid = idata[cur] +fudge
  lef = mid - skip
  ri = mid + skip
  junk = mx(work+3, lef= lef, ri= ri, /quiet, min = mm, time = nt)
  if (nt -lef lt 2.2) || (ri -nt lt 2.2) then $
    junk = (get_value(work+3, mid))[0]
  if junk gt 0.25*last_junk then begin
    flat_val[nfib] = cur
    time = round(nt)
    middle[nfib] = ((nt-lef) lt 2.2 || (ri-nt) lt 2.2) ? mid: time
    stf[nfib] = Middle[nfib] - hwid
    enf[nfib] = middle[nfib] + hwid
    offset = middle[nfib] -idata[cur]
    fudge = (fudge+offset)*0.5
    offset = fudge
    nfib = nfib+1
    last_junk = junk
  endif
  cur = cur+1
endwhile
flat_val = flat_val[0:nfib-1]
middle = middle[0:nfib-1]
stf = stf[0:nfib-1]
enf = enf[0:nfib-1]

i2w, middle[0:nfib-1], 60, clab = 'Middle for Flat Field'
image = make_array(sf[2])
for i = 0, nfib-1 do image[stf[i]:enf[i] ] = 0.980
i2w, image, work+4, clab = 'Flat-Field Fibers'

; now do the same for the data fibers

yes_no_ques, lcno, 'Is the first DATA channel visible?','Fiber Check'
star = 1
if lcno ne 0 then $
  read,  prompt='Enter the first visible data channel (1 to 84): ', star
if star lt 1 or star gt 84 then begin
  print, '% READ_NDATA: The first visible line may not have a value of ', star
  print, '              Use "READ_NDATA, /REDO" to rerun'
  return
endif
w2i, xwor, ywor, work+1
n4= n_elements(xwor)/4
n50 = n4/12.5 > 5
yb = ywor[0:n4]

ok = where (yb gt max(yb)/4, nok)
if nok gt 0 then zoomrange = [xwor[ok[0]-n50], xwor[ok[0]+n50]] else $
  zoomrange = [xwor[n50], xwor[n4-n50]]
xc = 0
while xc[0] eq 0 do $
  zoomw, work+1, xc = xc, zoomrange = zoomrange, instructions = $ 
  'Use right mouse button in right window to indicate the first data channel'
mid = xc[0]
offset = mid -xdata[star-1]
idata = xdata+offset
cur = star-1
iff = where (flat_val ge cur, ncur)
if n_elements(flat_val) gt nok then begin
  flat_val = flat_val[iff]
  stf = stf[iff]
  enf = enf[iff]
  middle = middle[iff] 
endif
icur = 0
if cur lt flat_val[0] then cur = flat_val[0]
offset = idata[cur] - middle[icur]
fudge = 0
while icur lt ncur do begin
  mid = middle[icur] +offset
  lef = mid - skip
  ri = mid + skip
  junk = mx(work+1, lef= lef, ri= ri, /quiet, min = mm, time = nt)
  if (nt -lef lt 2.2) || (ri -nt lt 2.2) then begin
    st[icur] = stf[icur]+round(offset)
    en[icur] = enf[icur]+round(offset)
  endif else begin
    time = round(nt)
    st[icur] = time-hwid
    en[icur] = time +hwid
    offset = nt - middle[icur]
    fudge = (fudge+offset)*0.5
    offset = fudge
  endelse
  icur += 1
endwhile

image = make_array(ss[2])
for i = 0, nfib-1 do image[st[i]:en[i] ] = 0.99
i2w, image, work+5, clab= 'Data Fibers'
i2w, middlef[0:nfib-1], 61, clab= 'Middle for data'
;
nfib = ncur
values = xspace[flat_val[0:nfib-1]]
nor = make_array(nfib)
if (keyword_set(lineout)) && (lineout gt 0) then begin
  yy = getyf(work+3)
  for i = 0, nfib-1 do nor[i] = total(yy[ stf[i]:enf[i]])
  i2w, values, nor, work+8, clab = 'Normalization', /xydata
endif else begin
 if n_elements(lineout) eq 0 then lineout = 0.1 else $
   lineout = (lineout lt -0.75) ?  0.1 : abs(lineout)
 flat_tot = total (flat, 2)
 ok = where (flat_tot gt max(flat_tot)*lineout, nok)
 flat = flat[ok[0]: ok[nok-1], *]
 for i = 0, nfib-1 do nor[ i] = total(total(flat[* , stf[i]:enf[i]], 2))
  i2w, values, nor, work+8, clab = 'Normalization', /xydata
endelse

image = make_array(ss[1], nfib)

for i = 0, nfib-1 do image[*, i] = total(imag[*, st[i]:en[i]], 2)/nor[i]

if keyword_set(filter) then begin
  wfil = get_maxwdf()-1
  for i = 0, nfib-1 do begin
    i2w, [0, 1], image[*, i], wfil
    filt, wfil, d= filter
    image[*, i] = getyf(wfil)
  endfor
endif
arr2str, wstr+2, image, get_array(img, 1), values,  $
  xlab = 'Distance', ylab ='Fiber (mm)', $
    clab = 'Normalized Fiber Data - ' + datafile
if keyword_set(str_out) then $
  if (str_out gt 0) && (str_out le get_maxstr()) then xfrstr, wstr+2, str_out
i2w, values,nor, work+7, /xydata, clab='Flat Field Total vs Position', $
 xlab= 'Positon', ylab = 'Value'

end
