; ------------------------------------------------------------------------ 
; $Id: read_pattern_data.pro,v 1.1 2007/04/19 15:16:49 lpmix Exp $
; ------------------------------------------------------------------------ 
;
; Copyright 2005 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;

 pro read_pattern_data, wdf, file, alldata = alldata

;
;+
; 
; NAME: READ_PATTERN_DATA
;
; Purpose:  Read an antenna patter file
;
; Format: READ_PATTERN_DATA, WDF [, FILE] [, ALLDATA = alldata]
;
;      Where:
;      WDF     - Initial WDF array number.
;                This must be present and valid.  Additional WDF arrays
;                will be added, if necessary, to read the entire file.
;      FILE    - The file name
;                If it is missing, DIALOG_PICKFILE will be called
;      ALLDATA - If present, the real, imaginary and amplitude of E-Theta
;                and E-Phi will be saved.  The default is ALLDATA = 0; 
;                only the Amplitude of  E-Theta and E-Phi will be saved.
;
; Technique:
;   The pattern and frequency are read followed by the data.
;   Current version assumes a single PHI or THETA value.
;   If PATTERN is missing, the previous PATTERN is used.
;   
; Example:
;   Read the data in pattern.dat and store the wdf arrays beginning in WDF 
;   array, 40.
;     READ_PATTERN_DATA, 40
;
;   Written: LPMix, February 1, 2007
;-

 on_error, 1
 if n_elements(wdf) eq 0 then begin
   Print, '% READ_PATTERN_DATA: First WDF array number must be provided'
   return
 endif
 maxwdf = get_maxwdf()
 if wdf le 0 or wdf gt maxwdf then begin
   print, '% READ_PATTERN_DATA: WDF must be in the range 1 to '+ $
             strtrim(get_maxwdf(), 2)
   return
 endif
 if n_elements(file) eq 0 then $
   fname = dialog_pickfile(/must_exist, /read) else fname = file
 if strlen(fname) lt 1 then begin
   print, '% READ_PATTERN_DATA: No File was Selected - RETURNING'
   return
 endif
 info = file_info(fname)
 if (info.exists + info.read) ne 2b then begin
   print, '% READ_PATTERN_DATA: File does not exist with read permission'
   print, '           File Name = ', fname
   print, '           File Exists =', info.exists
   print, '           Read Permission =', info.read
   return
 endif
 openr,unit,fname, /get_lun, error = error
 if error ne 0 then begin
   print, '% READ_PATTERN_DATA: Unable to open file, Error=', error
   return
 endif
 str = ''
 on_ioerror, badfile
 pattern = ''
 frequency = 0.00
 nphi = 0L
 buffer = (keyword_set(alldata))? 6 : 2
 ntheta = 0L
 readf, unit, str
 while not eof(unit) do begin
   while strpos(str, 'Phi') lt 0 do begin
     if strpos(str, 'Pattern') ge 0 then begin
       pattern = strtrim(strmid(str, 7), 2)
     endif else if strpos(str, 'Frequency') ge 0 then begin
       reads, strmid(str, 9), frequency
     endif else print, str
     readf, unit, str
   endwhile
   for i = 0, 1 do readf, unit, str
   readf, unit, nphi, ntheta
   ntot = nphi*ntheta
   if ntot eq 0 then begin
     print, '% READ_PATTERN_DATA: Total number of points is zero'
     help, nphi, ntheta
     return
     free_lun, unit
     return
   endif
   arr = make_array(6, ntot)
   readf, unit, arr
   if nphi eq 1 then begin
     x = arr[1, *]
     xlab= 'Theta'
     cc = strtrim(string(arr[0,0], format='(f8.1)'),2)
     clab = ' - Pattern='+pattern+';;Phi='+cc
   endif else begin
     x = arr[0, *]
     xlab= 'Phi'
     cc = strtrim(string(arr[1,0], format='(f8.1)'),2)
     clab = ' - Pattern='+pattern+';;Theta='+ cc
   endelse
   cc = strtrim(string(frequency, format='(e12.2)'),2)
   clab = clab+';;Frequency='+ cc
   x = reform (x)
   if n_elements(x) ne ntot then begin
     print, '% READ_PATTERN_DATA: Total number of points is wrong'
     help, nphi, ntheta, x
     return
     free_lun, unit
     return
   endif
   re = reform(arr[2, *])
   im = reform(arr[3, *])
   if wdf gt maxwdf - buffer then begin
     maxwdf = maxwdf + 100
     set_maxwdfarray, maxwdf
   endif
   if keyword_set(alldata) then begin
     i2w, x, re, wdf, /xydata, clab = 'Real E-Theta'+clab, xlab = xlab
     wdf +=1
     i2w, x, im, wdf, /xydata, clab = 'Imag E-Theta'+clab, xlab = xlab
     wdf +=1
   endif
   i2w, x, sqrt(re*re+im*im), wdf, /xydata, clab = 'E-Theta'+clab, xlab = xlab
   wdf +=1
   re = reform(arr[4, *])
   im = reform(arr[5, *])
   if keyword_set(alldata) then begin
     i2w, x, re, wdf, /xydata, clab = 'Real E-Phi'+clab, xlab = xlab
     wdf +=1
     i2w, x, im, wdf, /xydata, clab = 'Imag E-Phi'+clab, xlab = xlab
     wdf +=1
   endif
   i2w, x, sqrt(re*re+im*im), wdf, /xydata, clab = 'E-Phi'+clab, xlab = xlab
   arr = 0
   wdf +=1
   if not eof(unit) then readf, unit, str
 endwhile
 free_lun, unit
 return
badfile:
  print, '% READ_PATTERN_DATA: Error reading file'
  if n_elements(unit) gt 0 then free_lun, unit
end
