; ------------------------------------------------------------------------
; $Id$
; ------------------------------------------------------------------------
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
;+
; NAME:
;       READNTOF
;
; PURPOSE:
;       Read NTOF waveforms from a PFF data file
;
; CATEGORY:
;       Analysis
;
; CALLING SEQUENCE:
;       READNTOF [, WDF]
;
; OPTIONAL INPUTS:
;       WDF:      First WDF array for the NTOF data
;                 WDF must be in the range of 1 to (GET_MAXWDF()-10)
;                 DEFAULT: WDF = 1
;
; KEYWORD PARAMETERS:
;       SHOT_NUM: Shot number for he shot.
;                 If present, this is the same as
;                    NTOF, SHOT_NUM
;       NARRAY:   Array indicating the number of datasets read for each LOS
;       PLOTIT:   If set, plot the waveforms.
;       QUIET:    Do not print dataset information
;       IERROR:   Error parameter -- Value = 0 if no error
;
; OUTPUTS:
;       The designated waveforms are stored.
;
; COMMON BLOCKS:
;       NTOF_COMMON_BLOCK:  This common block is used to determine shot number
;
; PROCEDURE:
;       The PFF file is opened if it is not already open and the waveforms
;       are read.  If the file was opened, then it is closed.
;
; EXAMPLE:
;       
;       Read neutron time-of-flight signals for the current shot.
;         READNTOF
;
;       Read data for the current shot into WDF array 100
;         READNTOF, 100
;
; MODIFICATION HISTORY:
;       Written by:     L. P. Mix, Jr., July 16, 2007
;-
;  
PRO  READNTOF, WDF, NARRAY = narray, PLOTIT = plotit, $
     QUIET = quiet, SHOT_NUM = shot_num, ierror = ierror

on_error,1
;
; Common block
@ntof_common.dat

ierror = 1
if n_elements(ntof_init) eq 0 then begin
  if n_elements(shot_num) eq 0 then begin
        Print, '% READNTOF: Valid shot number must be provided'
        print, '  Format:  READNTOF [, WDF] , SHOT_NUM= shot_num'
        return
  endif
  ntof, shot_num
endif
; help, shot_num, shot_number
narray = replicate(0, numb_los) 

if shot_number lt 10 then begin
  Print, '% READNTOF: Valid shot number must be provided'
  print, '            (NTOF, SHOT_NUMBER)'
  return
endif

shot = shot_number
showpff, filename = filename, ufid = ufid, number = number, /quiet
if number gt 0 then begin
  file = 'pbfa2z_'+strtrim(shot, 2)+'.pff'
  ok = get_match(file, filename, nel = nel)
endif else nel = 0

okclose = nel
if nel eq 0 then openzz, shot, ier = ier else setpff, ufid[ok[0]], ier

if ier ne 0 then begin
  ierror = ier
  if nel eq 0 then $
    print, '% READNTOF: Unable to open file for shot: '+strtrim(shot, 2) $
    else $
    print, '% READNTOF: Unable to set PFF file to : '+strtrim(ufid[ok[0]], 2)
  return
endif

wdf = (n_elements(wdf) eq 0) ? 1 : wdf[0]

if (wdf lt 1) or (wdf ge get_maxwdf()-10) then begin
  print, '% READNTOF: WDF must be in the range 1 to '+strtrim(get_maxwdf()-10, 2)
  return
endif
  
detect_wdf[*]=0

n=0
wdfa = wdf
for i = 0, numb_los-1 do begin
  for j = 0, numb_detect[i]-1 do begin
    re, wdfa, detect_name[j, i], /sort, /all, narray = narr
    if narr gt 0 then detect_wdf [0, j, i]= wdfa+indgen(narr)
    wdfa = wdfa+narr
    
  endfor
  narray[i] = (i gt 0) ? (wdfa-narray[i-1]) : (wdfa - wdf)
endfor

if okclose eq 0 then closepff

n = total(narray)
if n eq 0 then begin
  print,  '% READNTOF: No Arrays were read'
  return
endif
ierror = 0
if keyword_set(quiet) then return

whelp, wdf + indgen(n), full =0
end
