; ------------------------------------------------------------------------
; $Id$
; ------------------------------------------------------------------------
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
;+
; NAME:
;       SPH_RESPONSE
;
; PURPOSE:
;       Generates a series of waveforms for variations in angle or distance 
;       for the spherical spectrometer.  The inputs are almost the same as
;       for SPH_SPEC in PFIDL_1D and the defaults use the same commmon block.
;       For all of the waveforms the film is fixed in position relative to
;       the spherical crystal.
;
;       This routine is the first of three procedures used to interpret the data
;       from a spherical spectrometer.  These three routines are: SPH_RESPONSE, 
;       MAKE_SPH_RESPONSE, and CALC_SPH_RESPONSE.
;
; ****  Sph_Response - Make a series of wdf arrays of with different Bragg angles,
;       or different distances.  Each WDF array is energy or xray wavelength vs 
;       position on the film.  Either or both waveforms may be saved.  A given
;       geometry is calculated and then the position of the source is varied to
;       determine the effect on the WDF array.  These waveforms may be saved to
;       a PFF file for future calculations, but take only seconds to generate.
;
; ****  Make_SPH_Response - Take the series of wdf arrays generated by SPH_RESPONSE
;       and generate a WDF array with separation vs angle or distance.  To generate
;       this waveform, we assume that two known lines are measured with the spherical
;       spectrometer.  This routine generates a response curve so that a measurement
;       of the separation in MM of the two lines on the calibration will predict an
;       angle or distance for the spherical spectrometer.
;       The syntax is:`
;          PRO MAKE_SPH_RESPONSE, WDF, E1, E2, WDFar
;       Where E1 and E2 are the wavelengths or energies of the two calibration 
;       energies, WDF is the array number for the response, and WDFAR are the WDF
;       arrays generated by the procedure SPH_RESPONSE.
;
; ****  Calc_Sph_Response - Calculates the actual response for a dataset.  Using
;       the response curve generated by MAKE_SPH_RESPONSE, and the measured spacing
;       of the the two energy peaks on the film, this routine determines the Bragg
;       angle or distance required to match the data and does a full calculation of
;       the spherical spectrometer for these values.  The spectrometer values 
;       calculated for the fit are saved as the default values for SPH_SPEC.
;
; CATEGORY:
;	Analysis
;
; CALLING SEQUENCE:
;
;       SPH_RESPONSE 
; 
; KEYWORD PARAMETERS:
;     Most keywords used by SPH_SPEC are useable.
;     SPH_RESPONSE unique keywords
;       TDELTA:  The variation of theta about the primary source angle or Bragg angle
;                For all theta variations, the position of the film is fixed.
;                Default: TDELTA = 2.5 ; Degrees
;       DDELTA:  The variation of distance from the crystal to the source 
;                For all distance variations, the position of the film is fixed.
;         Note:  DDELTA is ignored if TDELTA is present
;                If both DDELTA and TDELTA are missing, then default TDELTA is used.
;       NDELTA:  Number of response curves calculated for each TDELTA/DDELTA
;                NDELTA response curves are calculated for each positive and negative
;                TDELTA/DDELTA so a total of (1+ 2*NDELTA) responses are calculated.
;                NDELTA should be greater than 2 (A total of 5 responses)
;                DEFAULT:  NDELTA = 25 
;       PFF_FILE: If present, response waveforms are written to the PFF file.
;
;     SPH_SPEC keywords also used by SPH_RESPONSE
;     To see default values use: "SPH_SPEC, /SHOW_default
;     NOTE:  VERBOSE =0, PLOTIT = 0, GUI = 0, WRITECSV = 0 for the TDELTA/DDELTA calculations
;            Only the center calculation receives these keywords.
;       WDINIT:  First storage location for WDF arrays
;                If undefined or not valid, WDINIT = GET_MAXWDF()- 2*(ndelta+1)
;                If necessary the maximum number of waveforms is increased.
;                OUTPUT values are defined below.
;       WAVELENGTH:  If set, then output = 12
;             NOTE:  OUTPUT is ignored if WAVELENGTH is set.
;       OUTPUT:  If present, this indicates the desired output waveform.
;                OUTPUT should be a scalar; only the first value is used.
;                DEFAULT: OUTPUT = 13  ;; Photon Energy vs Film Position
;                ONLY OUTPUT = 12 or OUTPUT = 13 are allowed in this version
;                These wareforms are:
;               12.  Wavelength vs Film Position
;               13.  Photon Energy vs Film Position
;       DATABASE: If present, read a CSV file with geometry data for spectrometers.
;                This information is stored in a common block with the default values.
;       GEOMETRY: If present, use a geoometry label from the spectrometer database.
;                Input keyword values will ignored and replaced by values from the database.
;       GUI:     If set, use a gui to set spectrometer parameters.
;                Default:  GUI =0 if any of the following specified:
;                  RADIUS, CRY2D, ORDER, DISTANCE, THETA, FILM, GEOMETRY
;                Note: If GEOMETRY does not select a unique data then GUI = 1
;                Note: Input values can be changed by the GUI.
;                Note: If the source is moved, the film position should be adjusted if 
;                      the standard position is desired (HIT the STANDARD POSITION button)
;       RADIUS:  Radius of the spherical crystal in mm
;       CRY2D:   Crystal 2D spacing (angstroms)
;                Twice the crystal plane spacing
;       ORDER:   Diffraction order, N where N*Lambda = 2D*SIN(THETA)
;                Default: ORDER = 1
;       LENGTH:  Length of crystal in mm.  Length gt 10.
;       DISTANCE: Distance to the source in mm from the center of the crystal.
; ***  Source Angle:  The next 3 keywords are redundant - only one is required.
;       THETA:   Angle to the source in degrees from the center of the crystal.
;                This is the Bragg angle for the xray striking the center of the crystal
;                Note:  If TCO is given, then THETA = 90-TCO
;       TCO:     Angle between the crystal normal and the source in degrees.
;                The Bragg angle, THETA, is (90-TCO)
;                Note:  If THETA is present, TCO is ignored.
;       ENERGY:  Energy of XRAYS at X=0 on the crystal (keV)
;                This is used to calculate the Bragg angle at X=0 (THETA).
;                This keyword is ignored if either THETA (BRAGG Angle) or TCO are 
;                specified.                
;                  THETA = ASIN(PHYCONSTF(/AELAMBDA)/ENERGY*ORDER/CRY2D) *180d0/!dpi
;                NOTE:  ORDER and CRY2d should be provided.  If not, the saved values are 
;                       used.  If GUI or GEOMETRY is set, the source angle may be changed to a 
;                       different energy by these keywords.
; ***  End Source Angle ***
;       NPOINTS: Number of discrete rays used for the calculation.
;                Note: An odd number of points is always used.
;                      NPOINTS = (LONG(NPOINTS)/2)*2+1 > 11
;                 Default: NPOINTS = 1001L
;       FILM:    If film has 3 or more values, these values are used to position the 
;                  film plane.  FILM[0] and FILM[1] are X and Y locations of one point on the 
;                  film plane.  FILM[2] is the angle in radians of the film plane to the 
;                  horizontal axis or the arctangent of the slope of the film.
;                If the input value is undefined, then FILM = FILM-DEFAULT  (See SET_DEFAULT)
;                If film has less than 3 elements the film is put in the standard position
;                  and these values are returned.  If X0 and Y0 are the source positions, 
;                  then FILM[0] = 0.0, FILM[1] = RADIUS, FILM[2] = ATAN(Y0 - RADIUS, X0)
;                Units are MM for FILM[0:1] and RADIANS for FILM[2].
;                Note:  The film position is considered standard if the slope to the source 
;                       equals tan(film[2]) to a relative error of less than 1e-5.
;       PLOTIT:  If set, plot the geometry.  This may be set with "/SET_DEFAULT"
;                Default:  PLOTIT = PLOTIT_SAVE unless VERBOSE is set on the command line
;                If KEYWORD_SET(VERBOSE)  then PLOTIT = 1
;                The routine PLOT_SPEC may also be used to plot the geometry after running 
;                SPH_SPEC:
;                  PLOT_SPEC, XRANGE = xrange, YRANGE = yrange, SKIP = skip
;                  XRANGE and YRANGE are the extent of the plot and SKIP is a skip factor 
;                  for the different rays.  Default is that all NPOINT rays are plotted.
;       TITLE:   Title for plot.   Default is the parameters for the calculation
;       XRANGE:  If present, the extent of the XRange for the plot
;       YRANGE:  If present, the extent of the YRange for the plot
;       WRITECSV: If set, write the parameters to a CSV file.  The current calculation is to
;                 appended the structure and written at the end of the file.  Unless WRITECSV
;                 is a valid file name, WRITECSV will overwrite the original file.
;       IDENT:    This keyword is ignored unless WRITECSV is set.
;                 This is set equal to a string or a number (which is converted to a string).
;                 This is the value in the first column of the CSV file.
;                
; OUTPUTS:
;       For the central calculation:
;       A plot of the spectrometer is provided.
;       Construction values are provided in PARAMETERS and printed to the terminal.
;       Waveforms of the spectrometer parameters are produced (See OUTPUT keyword to specify.)
;       The output waveforms include the following:
;      12.  Wavelength vs Film Position
;      13.  Photon Energy vs Film Position
;
; PROCEDURE:
;       1.  Calculate the central source position and save all of the parameters
;           SPH_SPEC is used for the calculation
;       2.  If WRITECSV is set, the central ray calculation will be written to the csv file.
;       3.  Using the parameters for the central position, the angle or distance is varied
;           with the film fixed and the desired waveform (12 or 13) is stored for each position
;       4.  If a PFF file is specified, then waveforms are output.
;
; EXAMPLE:
;       Calculate the dispersion and plot the xrays from source to film with the default values.
;       Store the waveforms starting in WDF array 31.  Generate a scan in angle with 71 angles
;       on 3 degrees either side of the design configuration.
;         SPH_RESPONSE, wdinit  = 31, NDELTA = 35, TDELTA =3
;
; MODIFICATION HISTORY:
; 	Written by:	L. P. Mix, Jr., February 16, 2011
;
;
;-

PRO SPH_RESPONSE, wdinit = wdinit, radius= rc, cry2d = cry2d, Distance = distance, $
          film = film, Theta = theta, order = order, verbose = verbose, $
          npoints = npoints, plotit = plotit, length = length, wavelength = wavelength, $
          gui = gui, geometry = geometry, database = database, $
          tco = tco, energy = energy, writecsv = writecsv, ident = ident, saveit = saveit, $
          output = output, title = title, xrange = xrange, yrange = yrange, $
          pff_file = pfffile, Tdelta = tdelta, Ddelta = ddelta, ndelta = ndelta

on_error, 1
common sph_spec_common, rc_save, cry2d_save, distance_save, film_save, theta_save, order_save, $
   truncate_save, verbose_save, npoints_save, plotit_save, length_save, database_save, db_keys
common sph_spec_plot_common, x0, y0, xc, yc, xf, yf, xmid, ymid, clab_save, $
          rowlandx, rowlandy
film_err = 1e-5
if n_elements(rc_save) eq 0 then begin
  rc_save = 150.0d0
  cry2d_save = 2.749
  order_save = 1L
  length_save = 50.0d0
  distance_save = 300d0
  theta_save = 46.0d0
  npoints_save = 2001L
  film_save = 0
  truncate_save = 0
  verbose_save =1
  plotit_save = 1
  database_save =0
endif

maxwdf = get_maxwdf()
ndelta = (n_elements(ndelta) eq 0) ?  25L : ndelta[0] 
if n_elements(wdinit) eq 0 then wdinit = maxwdf- 2*(ndelta+1L)
if wdinit lt 0 then wdinit = 2
if wdinit +2*(ndelta+1L) gt maxwdf then set_maxwdfarray, wdinit+2*(ndelta+1L)
if n_elements(tdelta) gt 0 then begin
  tdelta = float(tdelta[0])
  lctheta = 1
endif else begin
  lctheta =  (n_elements(ddelta)  gt 0) ? 0 : 1
  if lctheta eq 1 then tdelta = 2.5 else ddelta = float(ddelta[0])
endelse
if lctheta then ddelta = 0.0 else tdelta = 0.0
if keyword_set(wavelength) then output = 12    
if n_elements(output) eq 0 then output = 13 else begin
  output = output[0]
  if output lt 12 or output gt 13 then output = 13
endelse

sph_spec, wd = wdinit+ndelta, rad= rc, cry2d = cry2d,  distance = distance, film = film, $
   theta = theta,  gui = gui, output = output, VERBOSE =verbose, PLOTIT = plotit, $
   TRUNCATE =0, writecsv = writecsv, ident = ident, order = order, npoints = npoints, $
   length = length, geometry = geometry, database = database, energy = energy, $
   title = title, xrange = xrange, yrange = yrange

np = ndelta +ndelta +1L
theta_arr = (lctheta eq 0) ? make_array(np, value = theta) : $
        theta- tdelta + dindgen(np)*(tdelta/ndelta)
dist_arr = (lctheta eq 1) ? make_array(np, value = distance) : $
        distance- ddelta + dindgen(np)*(ddelta/ndelta)

if n_elements (pfffile) gt 0 then  begin
  createpff, pfffile[0], error = ier
  lcpff = (ier eq 0) ? 1 : 0
endif else lcpff =0

for i = 0, np-1 do $
  sph_spec, wd = wdinit+i, rad= rc, cry2d = cry2d,  distance = dist_arr[i], film = film, $
    theta = theta_arr[i], verbose =0, plotit = 0, gui = 0, output = output, truncate =0, $
    order = order, npoints = npoints, length = length
if lcpff then for i = 0, np-1 do wri, wdinit+i, /full
end
