; ------------------------------------------------------------------------ 
; $Id: dirpdb.pro,v 1.6 2011/05/09 17:46:02 lpmix Exp $
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
 pro dirpdb, p1, p2, type = type, ndset = ndset, truncate = truncate, $
      ndim = ndim, dim = dim,  variable = names, quiet = quiet, line=line,  $
      page = page, fileid = fileid, full = full, help = help, $
      contents = contents, reset = reset, filename= filename

;+
;
;     this routine returns information about variables in a pdb file
;
; Format:
;     DIRPDB,  P1 [, P2] [, NDSET = ndset] [, /FULL] [, PAGE = page]
;                 [, TYPE = type] [, NDIM = ndim]  [, DIM = dim]
;                 [, PRINTIT = printit] [, LINE= LINE] [, VARIABLE = variable]
;                 [, /CONTENTS] [, /RESET] [, FILEID = fileid] [, /QUIET]
;
; where:
;
;   P1 and P2 have several possible configurations
;       DIRPDB,  start [, NUMBER]
;       DIRPDB,  <Search_string>
;      where:
;        start    - first dataset number or an array of dataset numbers 
;        number   - if start is a vector, then number is ignored
;                   if start is a scalar, then variables are 
;                     variables = variables(start+ indgen(number)
;        SEARCH_STRING - Search string for the variable name
;                        See GET_MATCH for details of the search.
;   FILEID    - File id for the variable information
;               Default is the current file
;   VARIABLE  - Variable or dataset names
;   NDSET     - Number of datasets 
;   FULL      - If set, full variable description will be printed
;   PAGE      - Number of lines in the directory
;               Default = directorypage (24)
;   TYPE      - Dataset type 
;   Filename  - Name of the file
;   NDIM      - Number of dimensions
;   DIM       - Minimum and maximum values for each dimension 
;               array [2, 5, ndset]
;   QUIET     - If set, no output
;   TRUNCATE  - If set, Delete source information.
;               Default: Truncate = 1
;   CONTENTS  - If set and the files contains a dataset with "CONTENTS", 
;               this dataset will be printed.
;               LINE contains the actual data.
;   HELP      - Print a help message
;   RESET     - If present and not zero, this opens a new directory widget.
;   LINE      - Text output array
; 
; Example:
;    Print the variable information for variable 5 for the current dataset.
;       dirpdb, 5
;
;    Print the variable information for variable 5 to 8 for the current dataset.
;       dirpdb, 5, 8
;
;    Print the variable information for all variables beginning with src
;       dirpdb, '^src'
;
; Original version:  July 29, 1999, lpm
;-
;
@pdbcom.dat
@qscom.dat
      common dirpdb_common, base, list, text1, oldpage, truncate_save
;     start the process
compile_opt defint32

      on_error, 1
;
      if n_elements (qsplotinit) eq 0 then qsinit
      if n_elements (truncate_save) eq 0 then begin
        truncate_save =1
        base  = 0
        text1 = 0
        text2 = 0
        list  = 0
        oldpage = directorypage > 4
      endif
      if n_elements (page) eq 0 then page = oldpage > 4 else page = page > 4
      if n_elements (truncate) gt 0 then truncate_save=truncate $
        else truncate = truncate_save
      nparams = n_params()
      if keyword_set (help) then begin
        print,'% Format: DIRPDB,  P1 [, P2] [, NDSET = ndset] [, /FULL]'
        print,'          [, TYPE = type] [, NDIM = ndim]  [, DIM = dim]'
        print,'          [, TRUNCATE = truncate] [, VARIABLE = variable]'
        print,'          [, FILEID = fileid] [, /quiet]'
        print,'          [, FILENAME = filename] [, PAGE = page] [, LINE = line]'
        return
      endif
      if nparams eq 0 then p1= '*'
      if n_elements(initpdb) eq 0 then initpdb = 0
      if initpdb eq 0 then begin
        print, '% DIRPDB:  No PDB files are open.'
        return
      endif
      if curfid lt 0 then begin
        print, '% DIRPDB:  Current file id is not defined.  Open a pdb file.'
        return
      endif

      if n_elements(fileid) gt 0 then begin
        ok = where (fileid[0] eq ufid, nok)
        if nok gt 0 then begin
          ifid = ok[0]
        endif else begin
          print, '% DIRPDB: ', strtrim(fileid[0], 2), ' is not a valid fileid.'
          showpdb
          return
        endelse
      endif else begin
        ifid = curfid
      endelse
      str = *file_info[ifid*max_pdb_file]
      pid = str.pds_fileid
      names = str.names
      filename = str.file
      nname = n_elements(names)

      if keyword_set(contents) then begin
        iname = get_match('contents', names, nel = nel)
        if nel eq 0 then begin
          print, '% DIRPDB: No match to datasets for specification: CONTENTS'
          return
        endif else if nel gt 1 then begin
          Print, '% DIRPDB: Multiple match to datasets for specification: CONTENTS'
          for i = 0, nel-1 do print, names[iname[i]]
          return
        endif
        if keyword_set (reset) then base = 0
        line = read_pdb(iname[0]+1)
        line = strsplit(line, string(10B), /extract, count = count)
        if count gt 0 then  $
          widget_directory, base, page, list, line, base_title = 'PDB Contents', $
            xsize = xsize, text1 = text1, value1 = filename, text2 = text2, $
            value2 ='Directory      Variable Name' $
        else print,'% DIRPDB: No data in the contents file.'
        return
      endif
 
      if size(p1, /type) eq 7 then $
        iname = get_match(p1, names, nel= nel) $
      else begin
        p3 = long (p1)
        nel = n_elements (p3)
        if ((nel eq 1) and (n_elements(p2) gt 0)) then begin
          if long(p2[0]) gt 0 then p3 = p3+lindgen(long(p2[0]))
        endif
;       convert to a 0 base index
        iname = p3 - 1
        ok = where(iname ge 0 and iname lt nname, nel)
        if nel gt 0 then iname = iname(ok)
      endelse
      if nel eq 0 then begin
        print, '% DIRPDB: No match to datasets for specification: ', p1
        return
      endif
      ndset = nel
      type = str.type[iname]
      ndim = str.ndim[iname]
      famstr = *family_info[ifid]
      dim = famstr.dimensions[*,*,iname]
      names = names[iname]

      if keyword_set(truncate) then for i = 0, ndset-1 do begin
        iat = strpos(names(i), '@')
        if iat gt 0 then names(i) = strmid(names(i), 0, iat)
      endfor
      if keyword_set (quiet) then return
      if keyword_set (reset) then base = 0
      if keyword_set(full) then begin

        line = make_array (3*ndset, /string)
        j = 0
        if strlen(strtrim(iname[0], 2)) gt 0 then iinit = 0 else iinit =1
        for i = iinit, nel-1 do begin
          ssz =  1+dim[1, *, i] -dim[0, *, i]
          sss = ssz[0]
          for iii = 1, n_elements(ssz)-1 do sss = sss*ssz[iii]
          line[j] = string(iname[i]+1, format='(i4)') + $
             ',     Variable Name: ' + names[i] 
          line[j+1] = '       Type: '+ type[i]+  ',    Size: '+ $
              strtrim(sss, 2)+ $
             ',     # of Dimensions :' + strtrim (ndim[i], 2)
          j = j+2
          if ndim[i] gt 0 then begin
             strout = '        Dimensions: ( '+strtrim(dim[0,0, i], 2) +': ' + $
               strtrim(dim[1,0, i], 2)  
             if ndim[i] gt 1 then for jj = 1,(ndim[i]-1) do $
               strout=strout+', '+strtrim(dim[0,jj, i], 2) +': ' +  $
                 strtrim(dim[1,jj, i], 2)  
             strout= strout+')'
             line[j] = strout
             j = j+1
          endif
        endfor
        j = (j-1)> 0
        line = line[0:j]
        page = -page
        widget_directory, base, page, list, line, base_title = 'PDB Directory', $
          xsize = xsize, text1 = text1, value1 = filename

      endif  else begin
        line = names
        for i = 0, nel-1 do $
          line[i] = string(iname[i]+1, names[i], format='(i6, t15, a)') 
        page = -page
        widget_directory, base, page, list, line, base_title = 'PDB Directory', $
          xsize = xsize, text1 = text1, value1 = filename, text2 = text2, $
          value2 ='Dataset      Variable Name'
      endelse
      oldpage = page
      return
      end
