; ------------------------------------------------------------------------ 
; $Id: readpdb.pro,v 1.5 2007/06/06 19:44:56 lpmix Exp $
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;

 Pro  readpdb, wdfa, datasetin, Tvalue, fileid = fileid, void_frac= void_frac, $ 
      structure = structure, node_values = node_values, matrix = matrix, $
      ierror = ierror, index = index, double = double, grid = grid
;
;+
;  Reads a PDB dataset into a structure array.  Procedure options are explained
;  below.
;  Note: Cell or element data will be moved to the nodes.  All data except 
;        
;        
;
;  Format:  READPDB, STRUCT/WDF [, DATASET] [, TVALUE] [, /INDEX]
;             [, FILEID = fileid]  [, GRID = grid] [, /DOUBLE] [, /MATRIX]
;             [, /STRUCTURE ] [, /NODE_VALUES] [, VOID_FRAC = void_frac]
;             [, IERROR = Ierror]
;  where:
;    STRUCT/WDF- Structure Array number or WDF array if TVALUE has 2 elements
;                One dimensional data is stored versus time in a wdf array.
;                For scalar data, the value is returned in STRUCT/WDF.
;                If an multiple arrays are read then WDF may be an array.
;   NOTE: Dataset must be provided unless grid is set.
;    dataset   - indicates a PDB dataset number or 
;                a search string for dataset comments.  
;                (Not required if GRID is set.)
;        See: GET_DSET for more details
;    TVALUE    - Time value for the dataset.
;                Default: All time values will be read.
;                If TVALUE has 2 elements these are interpreted as [K,L] values.
;                  K, L are from 2 to number of zones in each direction.
;                  Use XY2KL or CURPDB to get KL values.
;    INDEX     - If ZVALUE is given and index is set then the value of ZVALUE 
;                is assumed to be a grid index in the Time array. If INDEX 
;                is set then TVALUE goes from 1 to the maximum number of
;                time steps.
;    DOUBLE    - If data is double precision, the data will be saved as double 
;                precision (8-byte).  Default is to save as a float (4-byte).
;    MATRIX    - If set, a matrix is returned in the structure.
;                This can be useful if the dataset was returned as a series of 
;                WDF arrays.
;                See also: READ_PDB to return the data as an idl variable.
;    GRID      - If set, a node array will be returned.
;    FILEID    - File id of the PDB file containing the data
;                Caution: Even if the read is not successful, the current file
;                         id may be changed to FILEID.
;    STRUCTURE - if set a structure will be returned in STRUCT
;                For wdf arrays, time and amplitude are returned in an array.
;    NODE_VALUES - If set, the values are left on the nodes.
;               Default: Move all node values to the cell and distribute them on
;                     the nodes.  Node values are then the average of the values
;                     on the 4 sides of the nodes.
;                     Exception: ut@history, vt@history are left on the nodes.
;    VOID_FRAC - Variable is ignored unless the dataset is 4-dimensional. 
;                Variable is assumed to have 2 spatial dimensions and a fourth
;                dimension corresponding to time.
;                VOID_FRAC varies from 1 to the number of elements in the third
;                dimension.  The READPDB procedure will return only the slice 
;                corresponding to this slice in the third dimension.
;                Note:  Use READ_PDB to get the entire 4-dimensional array.
;                Example:  If F= F[NZ, NR, NV, NT], where NZ, NR and NT are the 
;                spatial and temporal dimensions, then VOID_FRAC = 2 will read
;                FF = F[*, *, 1, *] (Since IDL uses zero based indices) 
;                If TVALUE is provided, then a two dimensional array is returned.
;    IERROR    - Returns 0 if no errors
;
;  Examples:
;    Read dataset 34 into structure 1 from file with file id of 4.
;      readpdb, 1, 34, fil = 4
;
;    Read dataset with 'KE' into structure 2
;      readpdb, 2, 'KE'
;-
      on_error, 1
;     add common block
@pdbcom.dat
@qsstrcom.dat
@com.dat
;
      if n_elements(initwdf) eq 0 then wdfinit

;
;     check to see if pdb and structures have been initialized
;
      if n_elements(initpdb) eq 0 then init_pdb
      if n_elements (structinit) eq 0 then qsstruct_init
;
;
      ierror =1 
      nparam = n_params()
      if (nparam lt 1 ) then begin
 print,'% Format: READPDB, STRUCT/WDF [, DATASET] [, TVALUE] [, /INDEX]'
 print,'    [, FILEID = fileid]  [, GRID = grid] [, /DOUBLE]'
 print,'    [, /STRUCTURE ] [, VOID_FRAC = void_frac] [, IERROR = Ierror]'
	return
      endif
;
      if initpdb  le 0 then begin
        print, '% READPDB: No netPDB files are open.'
        return
      endif

      If curfid lt 0 then begin
        print, '% READPDB:  No valid PDB files are open.'
        return
      endif

      ierror = 0
      if n_elements (fileid) gt 0 then setpdb, fileid[0] , ier = ierror
      if ierror ne 0 then begin
        print, '% READPDB: Unable to set file pointer:' , fileid[0]
        showpdb
        return
      endif

      if keyword_set (grid) then begin
        if not keyword_set(structure) then begin
          if n_elements(wdfa) eq 0 then begin
            print, '% READPDB: Structure array must be provided'
            return
          endif
          wdfa = get_save_str(wdfa[0], 0)
          if wdfa(0) eq 0 then begin
            print,'% READPDB: Valid structure array value must be provided'
            return
          endif
          wdfar = wdfa(0)
          if wdfar lt 1 or wdfar gt maxstructure then begin
            print, '% READPDB: Grid structure must be greater than 0 and </=', $
              strtrim(maxstructure, 1)
            return
          endif
        endif
;       now get the parameters 
        str = read_pdb_grid(double = double)
        
        ss = size(str, /type)
        if ss eq 8 then ierror = 0 else return
        if keyword_set(structure) then wdfa = temporary(str) else $
                    i2s, str, wdfa, /no_copy
        return
      endif
      if n_elements (datasetin) eq 0 then begin
        print, '% READPDB: Dataset name or number must be provided.'
        return
      endif else dataset = datasetin
      str = read_pdb_data(dataset, tvalue, lctype, node_values = node_values, $
                         matrix=matrix, ierror = ierror, index = index, $
                         double = double, clab = clab, void_frac = void_frac) 
      if ierror ne 0 or lctype eq -99 then begin
        print, '% READPDB: Error reading data. Dataset: ', dataset
;        help, /str, str 
        return
      endif
      if lctype gt 0 and not keyword_set(matrix) then begin
        if n_elements (str) lt 5 then begin
          print, '% READPDB: Bad WDF array returned'
          return
        endif
        if not keyword_set(structure) then begin
          if n_elements(wdfa) eq 0 then begin
            print, '% READPDB: WDF array must be provided'
            return
          endif
          wdfar = get_save_array(wdfa[0], 0)
          if wdfar[0] eq 0 then begin
            print, '% READPDB: Valid wdf array value must be provided'
            return
          endif
          if wdfar eq wdfa[0] then wdfar = wdfa
          nwdf = n_elements (wdfar)
          if nwdf lt lctype then $
            wdfar = [wdfar, wdfar[nwdf-1]+1+indgen(lctype-nwdf)]
          if max(wdfar) gt maxwdfarray then set_maxwdfarray, max(wdfar)
          for  i = 0, lctype-1 do $
            put_wdfarray, [reform(str(*, 0)), reform (str(*, i+1))],  $
              wdfar[i], /xydata, /no_copy, $
              xlab = 'Time', Ylab='Value', clab= clab + ' - ' + strtrim (i+1,2)
          wdfa = wdfar [0:lctype-1]
        endif else wdfa = temporary(str)
      endif else if lctype lt 0 then begin
        wdfa = str
      endif else begin
        if not keyword_set(structure) then begin
          if n_elements(wdfa) eq 0 then begin
            print, '% READPDB: Structure array must be provided'
            return
          endif
          wdfa = get_save_str(wdfa[0], 0)
          wdfar = wdfa(0)
          if wdfar eq 0 then begin
            print,'% READPDB: Valid structure value must be provided'
            return
          endif
          if wdfar lt 1 or wdfar gt maxstructure then begin
     print,'% READPDB: Structure value must be greater than 0 and </=', $
              strtrim(maxstructure, 1)
            return
          endif
;       now get the parameters
        endif else begin
            wdfa = temporary(str)
             return
        endelse
        if keyword_set (structure) then return

        i2s, str,  wdfar, /no_copy
        wdfa = wdfar
      endelse
      return
      end
