;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
 Pro slicekl, str, kval, lval, time, out = out
;+
; Name: SliceKL
; Purpose: Slice a structure along a K or L line.
; 
; Format: SLICEKL, Structure, kval, lval, time, out = out.
;     where:
;        Structure - the type 9 structure to be sliced.
;        Note: The following, KVAL, LVAL, TIME, range from 1 to the total number
;        KVAL      - the K-Value index to be sliced
;                    Set to zero to have as independent variable
;        LVAL      - the L-Value index to be sliced
;                    Set to zero to have as independent variable
;        TIME      - the Time index to be sliced
;                    Set to zero to have as independent variable
;            NOTE:  Tindex will return closest index for a given time
;        OUT       - Output Waveform Array. Default = 1
;                    Waveform will be the structure parameter vs Kvalue, Lvalue, or Time
;
; Example:
;    Get the waveform array for Temperatures vs Time for kvalue equal 2, and Lvalue = 3
;    Temperatures are in structure 4.  Store the result in wdf array, 6.
;        SLICEKL, 4, 2, 3, 0, out = 6
;
;    Get the waveform array for density vs k-value at an lvalue of 4 and a time
;      step of 100.  Assume density is in structure 5.  Store the waveform in 5
;        SLICEKL, 5, 0, 4, 100, out = 5
; 
;-
on_error, 1

;  Common block
@qsstrcom.dat
@qscom.dat

   npar = n_params()
   if npar eq 0 then begin
     print,'% Format:  SliceKL, STR,  KVAL, LVAL, TIME, OUT = out'
     return
   endif
   if npar lt 4 then begin
      print,'% SliceKL: Structure, Kval, Lval and Time  must be defined.'
      return
   endif
      if n_elements(str) eq 0 then begin
        print, '% SliceKL:  Structure must be defined'
        return
      endif else a = str
      if n_elements (structinit) eq 0 then qsstruct_init
      if n_elements (qsplotinit) eq 0 then qsinit
;
      b = size(a)
      if b(b(0)+1) ne 8 then begin
        if a gt 0 and a le maxstructure then begin
          s2i, a, a
          b = size (a)
          if b(b(0)+1) ne 8 then begin
 print, '% SliceKL: First parameter must be a structure or a array number'
            return
          endif
        endif  else begin
 print, '% SliceKL: First parameter must be a structure or array number'
          return
        endelse
      endif
      if (a.type ne 9 ) then begin
        print, '% SliceKL: Only type 9 structures may be used.'
        help, a.type
        return
      endif

      if n_elements (kval) gt 0 then aa = kval[0]-1 else begin
        print, '% SliceKL: K-value must be defined'
        return
      endelse

      if n_elements (lval) gt 0 then bb = lval[0]-1 else begin
        print, '% SliceKL: L-value must be defined'
        return
      endelse

      if n_elements (time) gt 0 then cc = time[0]-1 else begin
        print, '% SliceKL: Time-value must be defined'
        return
      endelse
      kmax = a.size[0]
      lmax = a.size[1]
      tmax = a.ntime 

      if kval[0] eq 0 then begin
        if bb lt 0 or bb ge lmax then begin
          print, '% SliceKL: L-value must be in the range [1, ', $
             strtrim (lmax,2), ']'
          return
        endif
        if cc lt 0 or cc ge tmax then begin
          print, '% SliceKL: Time must be in the range [1, ', $
             strtrim (tmax,2), ']'
          return
        endif
        x = a.x1[*, cc]
        y = a.v1[*, cc]
        x = reform (x, kmax, lmax, /overwrite)
        y = reform (y, kmax, lmax, /overwrite)
        x = x(*, lval)
        y = y(*, lval)
        xlab = a.x1lab
        ylab = a.clab
        clab = A.clab+', Lval = '+strtrim (lval,2)+', Time = '+ $
           strtrim(a.x4[cc], 2)
      endif else if lval eq 0 then begin
        if aa lt 0 or aa ge kmax then begin
          print, '% SliceKL: K-value must be in the range [1, ', $
             strtrim (kmax,2), ']'
          return
        endif
        if cc lt 0 or cc ge tmax then begin
          print, '% SliceKL: Time must be in the range [1, ', $
             strtrim (tmax,2), ']'
          return
        endif
        x = a.x2[*, cc]
        y = a.v1[*, cc]
        x = reform (x, kmax, lmax, /overwrite)
        y = reform (y, kmax, lmax, /overwrite)
        x = x(kval, *)
        y = y(kval, *)
        xlab = a.x2lab
        ylab = a.clab
        clab = A.clab+', Kval = '+strtrim (kval,2)+', Time = '+ $
           strtrim(a.x4[cc], 2)
      endif else if time eq 0 then begin
        if aa lt 0 or aa ge kmax then begin
          print, '% SliceKL: K-value must be in the range [1, ', $
             strtrim (kmax,2), ']'
          return
        endif
        if bb lt 0 or bb ge lmax then begin
          print, '% SliceKL: L-value must be in the range [1, ', $
             strtrim (lmax,2), ']'
          return
        endif
        x = a.x4
        ind = aa+bb*kmax
        y = reform (a.v1[ind, *])
        xlab = a.x4lab
        ylab = a.clab
        clab = A.clab+', Kval = '+strtrim (kval,2)+', Lval = '+strtrim (lval,2)
      endif else begin
        print, '% SliceKL: Either K-value, L-value or Time must be zero.'
        return
      endelse
      if n_elements(out) eq 0 then out = 1
      if out lt 1 or out gt get_maxwdf() then out = 1
      i2w, x, y, /xydata, out, clab=clab, ylab=ylab, xlab=xlab, file=a.file
      return
      end
