; ------------------------------------------------------------------------ 
; $Id: get_pds_ids.pro,v 1.3 2007/06/06 19:50:16 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
 function get_pds_ids, filehandle, segments = segment, helpme = helpme, $
   statesegments = statesegments, topology = topology, ier = ier


; Copyright (c) 1998, Sandia National Laboratories  All rights reserved.
;       Unauthorized reproduction prohibited.
;+
; NAME:
;      GET_PDS_IDS
;
; PURPOSE:
;      Inquire the ID's of a particular dataset type
;      Uses: PDS_INQ_IDS
;
; CALLING SEQUENCE:
;
;       IDs  = GET_PDS_IDS([ File_Handle] [, IER = ier] , /Segment, 
;                    /Topology, /Statesegments [, /HELPme] )
;
; WHERE:
;        IDs          - ID's of the desired parameters
;        File_Handle  - File_handle of the PDS file
;                       Default: Current file
;        IER          - Error Parameter
;                       Returns 0 if successful
;        HELPme       - If set, a one user will be given a help message.
;
;  FLAGS: One or more must be set.
;      Note:   If multiple valid flags are set, only the first one found will
;              be returned in the following order:
;
;       SEGMENT      - If set, inquire about segment IDs 
;       TOPOLOGY     - If only TOPOLOGY is set, inquire the topology segment IDs 
;       TOPOLOGY+STATESEGMENT - If both set, inquire the state segments 
;                               relative to the current topology.
;       STATESEGMENT - If only STATESEGMENT is set, inquire the statesegment IDs
;
; EXAMPLE:
;       Get the id's of the state segments of the current file.
;         ids = get_pds_ids(/state)
;
; MODIFICATION HISTORY:
; 	Written by:	L. P. Mix, August 14, 1998
;-

@pds_datacom.dat
      on_error, 1

@pds_info_types.dat
      ier = 0L
      if keyword_set (helpme) then begin
print, '% Format: IDs  = GET_PDS_IDS([ File_Handle] [, IER = ier] , /Segment,'
print, '                 /Topology, /Statesegments [, /HELPme] )'
        return, 0
      endif

      IDS = -1L
      if n_elements(initpds) eq 0 then initpds = 0
      if initpds eq 0 then init_pds
      if initpds lt 0 then begin
        print, '% GET_PDS_IDS: This installation does not support PDS'
        return, ids
      endif
      if n_elements(filehandle) eq 0 then fh = pds_id else begin
        if filehandle lt 0 then fh = pds_id else fh = long(filehandle)
      endelse 

      type = -1L
      if keyword_set(segment) then begin
        type = PDS_SEG
      endif else if keyword_set(topology) then begin
        type = PDS_TP
        if keyword_set(statesegments) then  type = type + PDS_ST
      endif else if keyword_set(statesegments) then begin
        type = PDS_ST
      endif
      if type lt 0 then begin
        print, '% GET_PDS_IDS: Type value must be provided'
        return, ids
      endif
      ier = 1L
      nids = inquire_pds(fh, pdstype = type, ier = ier )
      if ier ne 0 then begin
        Print, '% GET_PDS_IDS: Error inquiring number of IDs.'
        print, '          File_Handle, Type:', fh, type
        return, ids
      endif
      if nids eq 0 then begin
        print, '% GET_PDS_IDS: No IDs available for these parameters', type
        return, ids
      endif
      ier = 1L
      ids = make_array(/long, nids, value = 0L)
      junk = call_external(PDS_EXE, 'pdsp_inq_ids_', fh, type, ids, ier)
      if ier ne 0 then $
        print, '% GET_PDS_IDS: Error in obtaining parameter', type
      return, ids
      end
