; ------------------------------------------------------------------------ 
; $Id: get_pds_names.pro,v 1.3 2007/06/06 19:50:16 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
 function get_pds_names, filehandle, variables = var, datafiles = datafiles, $
   information = information, segment = segment, ier = ier

; Copyright (c) 1998, Sandia National Laboratories  All rights reserved.
;       Unauthorized reproduction prohibited.
;+
; NAME:
;      GET_PDS_NAMES
;
; PURPOSE:
;      Return a list of names associated with a specified PDS type.
;      Uses: PDS_INQ_NAMES
;
; CALLING SEQUENCE:
;
;       NAMES  = GET_PDS_NAMES([ File_Handle] [, IER = ier] [, /HELPme]
;               /Variables, /DataFiles, /Information, /Segment )
;
; WHERE:
;        NAMES        - Names of the desired parameters
;        File_Handle  - File_handle of the PDS file
;                       Default: Current file
;        IER          - Error Parameter
;                       Returns 0 if successful
;        HELPme       - If set, a one user will be given a help message.
;
;  FLAGS: One or more must be set.
;      Note:   If multiple valid flags are set, only the first one found will
;              be returned in the following order:
;
;       INFORMATION  - If only INFORMATION is set, return the info names for the
;                      whole file.
;       INFORMATION+SEGMENT - If both set, return the info names for the current
;                      segment
;       INFORMATION+VARIABLE - If both set, return the info names for the current
;                      variable
;       VARIABLE     - If set, return the variable names
;       DATAFILES    - If set, return the datafile extensions
;
; EXAMPLE:
;       Get the name of the variables in the current file.
;         names  = get_pds_ids(/var)
;
; MODIFICATION HISTORY:
;       Written by:     L. P. Mix, August 14, 1998
;-

;
;
@pds_datacom.dat
      on_error, 1

@pds_info_types.dat
      value = ''

      if keyword_set (helpme) then begin
        IER = 0
print, '% Format: '
print,'       NAMES  = GET_PDS_NAMES([ File_Handle] [, IER = ier] [, /HELPme]'
print,'               /Variables, /DataFiles, /Information, /Segment )'
        return, value
      endif
      ier = -1L
      if n_elements(initpds) eq 0 then initpds = 0
      if initpds eq 0 then init_pds
      if initpds lt 0 then begin
        print, '% GET_PDS_NAMES: This installation does not support PDS'
        return, value
      endif
      if n_elements(filehandle) eq 0 then fh = pds_id else begin
        if filehandle lt 0 then fh = pds_id else fh = long(filehandle)
      endelse 

      type = -1L
      nnames = 0L
      if keyword_set(information) then begin
        type = PDS_INFO
        if keyword_set(segment) then  type = type + PDS_SEG  else $
          if keyword_set(var) then type = type +PDS_VAR
      endif else if keyword_set(var) then begin
        type = PDS_VAR
      endif else if keyword_set(datafiles) then begin
        type = PDS_DTF
      endif
      if type lt 0 then begin
        print, '% INQUIRE_PDS: Type value must be provided'
        return, value
      endif
      ier = 1L
      nnames = inquire_pds(pdstype = type )
      if nnames eq 0 then begin
        print, '% GET_PDS_NAMES: No names available for these parameters', type
        return, value
      endif
      value = make_array(/byte, pds_name_size*nnames, value = 32B)
      sarray = make_array(/string, nnames) 
      junk = call_external(PDS_EXE, 'pdsp_inq_names_', fh, type, value, ier)
      if ier ne 0 then $
        print, '% GET_PDS_NAMES: Error in obtaining parameter', typ $
      else for i = 0, nnames-1 do  $
          sarray[i]= strtrim(value[i*pds_name_size:(i+1)*pds_name_size-1], 2)
      return, sarray
      end
