; ------------------------------------------------------------------------ 
; $Id: comp_geo.pro,v 1.3 2007/06/06 19:59:46 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
;+
; NAME:
;       COMP_GEO
;
; PURPOSE:
;       Compare geometry files.
;
; CALLING SEQUENCE:
;       Format:
;
;       COMP_GEO, PFF1, PFF2 [, UNIT = unit] [, ERROR = error] [, FULL = full]
;
;
; INPUTS:
;       PFF1    File ID for first PFF file
;       PFF2    File ID for second PFF file
;   Note: Routine assumes first dataset is the bounding box, second is the grid
;         and third to end are the conductors to be compared.
;   Note: If the number of blocks and grid values do not match then the 
;         comparison is terminated.
;
; KEYWORD PARAMETERS:
;       UNIT    UNIT number if output is to a file.
;       ERROR   This can be a scalar or a vector.  If a vector, the first value
;               will be used for grids and the second for conductors.
;               GRIDS:
;               If set, two grid values will be equal if they differ by less 
;                 than RELERROR * grid spacing
;                 where REKERROR = error/100 (ie error is taken as a percent)
;               CONDUCTORS:
;               If set, two conductor values will be equal if they differ by
;                 less than RELERROR /(2^17)*(max(x)-min(x)) 
;                 where x represents xi, xj, and xk for each dataset.
;       FULL    All differences will be printed.  ERROR will be used to
;                 determine if a difference exists.
;
; OUTPUTS:
;       Output file differences between two geometry files.
;
; EXAMPLE:
;
;       Compare files with file id's 1 and 3 and write output to unit.
;          openw, unit, /get, 'difference.dat'
;          comp_geo, 1, 3, unit = unit
;          free_lun, unit
;
; MODIFICATION HISTORY:
; 	Written by:     Paul Mix, June 6, 1997
;-
 pro comp_geo, pff1, pff2, unit = unit, error = error, full = full

on_error, 1
      nparams = n_params()
      if nparams lt 2 then begin
        print, '% COMP_GEO: Two PFF file IDs are required for input.'
        print, 'Format: COMP_GEO, PFF1, PFF2 '
        print, '              [, UNIT = unit] [, ERROR = error] [, FULL = full]'
        return 
      endif 

      if n_elements(pff1) eq 0 or n_elements(pff2) eq 0 then begin
        print, '% COMP_GEO: PFF file ids must be defined.'
        return 
      endif 

      pff_info, /quiet, fname = fname, ndset = ndset, dpoint = dpoint, $
        current = current,fid = fileid 
      ok1 = where (fileid eq pff1, nok)
      if nok eq 0 then begin
        print,'% COMP_GEO: PFF1 is not a valid File ID= ', strtrim(pff1, 2)
        return
      endif else ok1 = ok1(0)
      ok2 = where (fileid eq pff2, nok)
      if nok eq 0 then begin
        print,'% COMP_GEO: PFF2 is not a valid File ID= ', strtrim(pff2, 2)
        return
      endif else ok2 = ok2(0)
      if ndset (ok1) ne ndset(ok2) then lcsamend = 1 else lcsamend =0
      ndb = ndset(ok2)
      ndset = ndset(ok1)
      if ndset lt 3 then lcsmallnd = 1 else lcsmallnd = 0
      lctot = lcsmallnd + lcsamend
      if n_elements(unit) gt 0 then lcunit = 1 else lcunit = 0
      if lcunit eq 0 then begin
        print, ' Comparison of the two geometries'
        print, '   Dataset 1 = ', fname(ok1)
        print, '   Dataset 2 = ', fname(ok2)
        if lctot gt 0 then begin
          if lcsamend gt 0 then print, $
            '% COMP_GEO: Files must have the same number of datasets:'
          if lcsmallnd gt 0 then print, $
            '% COMP_GEO: Files must have at least 3 datasets'
          print, ndset, ndb
          return
        endif
        print, ' '
      endif else begin
        printf, unit, ' Comparison of the two geometries'
        printf, unit, '   Dataset 1 = ', fname(ok1)
        printf, unit, '   Dataset 2 = ', fname(ok2)
        if lctot gt 0 then begin
          if lcsamend gt 0 then printf, unit, $
            '% COMP_GEO: Files must have the same number of datasets:'
          if lcsmallnd gt 0 then printf, unit, $
            '% COMP_GEO: Files must have at least 3 datasets'
          printf, unit, ndset, ndb
          return
        endif
        printf, unit, ' '
      endelse
      if n_elements (error) eq 0 then begin
        gerr = 0.0
        cerr = 0.0
      endif else begin
        gerr = (error(0) *0.01) > 0.0
        if n_elements(error) gt 1 then cerr = error(1) else $
          cerr = error(0)
        cerr = (cerr/131072.0) >0
      endelse

      readifl, a1,b1, c1, 1, file= pff1
      readifl, a2,b2, c2, 1, file= pff2

      average = (b1+b2)*0.5
      diff = b1-b2
      ok = where(average eq 0.0, nok)
      aver = average
      if nok gt 0 then aver(ok) = 1.0
      rel_diff = diff/aver
      if lcunit eq 0 then begin
        print, ' Bounding box'
        print, '   Average value:', average
        print, '      Difference:', diff
        print, '   Relative Diff:', rel_diff
        print, ' '
      endif else begin
        printf, unit, ' Bounding box'
        printf, unit, '   Average value:', average
        printf, unit, '      Difference:', diff
        printf, unit, '   Relative Diff:', rel_diff
        printf, unit, ' '
      endelse

      readgrd, g1, 2, file= pff1
      readgrd, g2, 2, file= pff2
      if g1.block ne g2.block then begin
        if lcunit eq 0 then begin
          print, ' Grid Blocks'
          print, '   First Grid has ', strtrim(g1.block, 2), ' blocks'
          print, '   Second Grid has ', strtrim(g2.block, 2), ' blocks'
          print, ' Comparison Terminated'
        endif else begin
          printf, unit, ' Grid Blocks'
          printf, unit, '   First Grid has ', strtrim(g1.block, 2), ' blocks'
          printf, unit, '   Second Grid has ', strtrim(g2.block, 2), ' blocks'
          printf, unit, ' Comparison Terminated'
        endelse
        return
      endif else nblo = g1.block
      diff = g1.size - g2.size
      mxdiff= max(diff, min = mndiff)
      if mxdiff ne 0  or mndiff ne 0 then begin
        if lcunit eq 0 then begin
          print, ' Grids Have Different Number of Elements'
          for i = 0, nblo-1 do begin
          print, '   First file, Block ', strtrim(i, 2), ':', g1.size(*, i)
          print, '   Second file, Block ', strtrim(i, 2), ':', g2.size(*, i)
          endfor
          print, ' Comparison Terminated'
        endif else begin
          printf, unit, ' Grids Have Different Number of Elements'
          for i = 0, nblo-1 do begin
          printf, unit, '   First file, Block ', strtrim(i, 2), ':', g1.size(*, i)
          printf, unit, '   Second file, Block ', strtrim(i, 2), ':', g2.size(*, i)
          endfor
          printf, unit, ' Comparison Terminated'
        endelse
        return
      endif
      blost = make_array(nblo, /str, value = 'Block ')
      for i = 0, nblo-1 do blost(i) = blost(i)+strtrim(i+1, 2)
      xstr = ['X1 Grid', 'X2 Grid', 'X3 Grid'] 
; loop over grid blocks
      for ib = 1, nblo do begin
; loop over spatial dimensions
        for isp = 1, 3 do begin
          a = get_array(g1, isp, ib)
          nela = n_elements(a)
          b = get_array(g2, isp, ib)
          ave_x = (a+b)*0.5
          diff_x=  a-b       
          del_x =[ ave_x(1:nela-1)- ave_x(0:nela-2), 0]
          del_x(nela-1) = del_x(nela-2)
          reldiff = abs(diff_x/del_x)
          max_diff = max(reldiff)
          
          if (max_diff lt gerr) then begin
            if lcunit eq 0 then begin
              print, blost(ib-1), ', ', xstr(isp-1), ': ', $
                'First and second datasets are identical'
            endif else begin
              printf, unit, blost(ib-1), ', ', xstr(isp-1), ': ', $
                'First and second datasets are identical'
            endelse
          endif else begin
            if lcunit eq 0 then begin
              print, blost(ib-1), ', ', xstr(isp-1), ': ', $
               'Grids differ by ', strtrim(max_diff,2), ' of a cell'
              if keyword_set(full) then begin
                ok = where(reldiff ge gerr, nok)
                print, 'Grid values with differences over ', strtrim(gerr, 2)
                print, a(ok)
                print, 'Corresponding grid differences'
                print, diff_x(ok)
                print, 'Corresponding relative errors'
                print, reldiff(ok)
              endif
            endif else begin
              printf, unit, blost(ib-1), ', ', xstr(isp-1), ': ', $
               'Grids differ by ', strtrim(max_diff,2), ' of a cell'
              if keyword_set(full) then begin
                ok = where(reldiff ge gerr, nok)
                printf, unit, 'Grid values with differences over ', strtrim(gerr, 2)
                printf, unit, a(ok)
                printf, unit, 'Corresponding grid differences'
                printf, unit, diff_x(ok)
                printf, unit, 'Corresponding relative errors'
                printf, unit, reldiff(ok)
              endif
            endelse
          endelse
        endfor
      endfor
     
; now for conductors
      for idset = 3, ndset do begin
        readcon, c1, idset, file= pff1
        readcon, c2, c1.clab, file= pff2
        diff = c1.size - c2.size
        mxdiff= max(diff, min = mndiff)
        if mxdiff ne 0  or mndiff ne 0 then begin
          if lcunit eq 0 then begin
            print, 'Dataset ', strtrim(idset, 2), ': ', c1.clab
            print, ' Conductors Have Different Number of Planes'
            print, '   First file: ', c1.size(*)
            print, '   Second file: ', c2.size(*)
            print, ' Comparison Terminated for this dataset'
          endif else begin
            printf, unit, 'Dataset ', strtrim(idset, 2), ': ', c1.clab
            printf, unit, ' Conductors Have Different Number of Planes'
            printf, unit, '   First file: ', c1.size(*)
            printf, unit, '   Second file: ', c2.size(*)
            printf, unit, ' Comparison Terminated for this dataset'
          endelse
        endif else begin
          if lcunit eq 0 then begin
            print, 'Dataset ', strtrim(idset, 2), ': ', c1.clab
          endif else begin
            printf, unit, 'Dataset ', strtrim(idset, 2), ': ', c1.clab
          endelse
          mxx = max(c1.plane, min = mnx)
          cerror = cerr*(mxx - mnx)
          if c1.size(0) gt 0 then begin
;         non-normal planes
            diff = c1.slant -c2.slant
            mdiff = max(abs(diff))
            if mdiff lt cerror then begin
              if lcunit eq 0 then begin
                print, ' Slant Surfaces are identical'
              endif else begin
                printf, unit, ' Slant Surfaces are identical'
              endelse
            endif else begin
              if lcunit eq 0 then begin
                print, 'Slant Surfaces: ', $
                 'Maximum Differences is ', strtrim(mdiff,2)
                if keyword_set(full) then begin
                  print, 'Conductor values and difference over ', $
                     strtrim(cerror, 2)
                  for i = 0, c1.size(0)-1 do begin
                    if (max(abs(diff(*, *, i))) gt cerror) then begin
                      print, ' Plane = ', strtrim(c1.n0(i), 2)
                      print, c1.slant(*, *, i), format = '(3g16)'
                      print, diff(*, *, i), format = '(3g16)'
                    endif
                  endfor
                endif
              endif else begin
                printf, unit, 'Slant Surfaces: ', $
                 'Maximum Differences is ', strtrim(mdiff,2)
                if keyword_set(full) then begin
                  printf, unit, 'Conductor values and difference over ', $
                     strtrim(cerror, 2)
                  for i = 0, c1.size(0)-1 do begin
                    if (max(abs(diff(*, *, i))) gt cerror) then begin
                      print, ' Plane = ', strtrim(c1.n0(i), 2)
                      printf, unit, c1.slant(*, *, i), format = '(3g16)'
                      printf, unit, diff(*, *, i), format = '(3g16)'
                    endif
                  endfor
                endif
              endelse
            endelse
          endif
          if c1.size(1) gt 0 then begin
;         X normal planes
            diff = c1.plane(*,*, c1.n1) -c2.plane(*,*, c2.n1)
            mdiff = max(abs(diff))
            if mdiff lt cerror then begin
              if lcunit eq 0 then begin
                print, ' X Normal Surfaces are identical'
              endif else begin
                printf, unit, ' X Normal Surfaces are identical'
              endelse
            endif else begin
              if lcunit eq 0 then begin
                print, 'X Normal Surfaces: ', $
                 'Maximum Differences is ', strtrim(mdiff,2)
                if keyword_set(full) then begin
                  print, 'Conductor values and difference over ', $
                     strtrim(cerror, 2)
                  for i = 0, c1.size(1)-1 do begin
                    if (max(abs(diff(*, *, i))) gt cerror) then begin
                      print,' Plane = ', strtrim(c1.n1(i), 2)
                      print, c1.plane(0, *, c1.n1(i)), format = '(3g16)'
                      print, c1.plane(1, *, c1.n1(i)), format = '(3g16)'
                      print, diff(0, *, i), format = '(3g16)'
                      print, diff(1, *, i), format = '(3g16)'
                    endif
                  endfor
                endif
              endif else begin
                printf, unit, 'X Normal Surfaces: ', $
                 'Maximum Differences is ', strtrim(mdiff,2)
                if keyword_set(full) then begin
                  printf, unit, 'Conductor values and difference over ', $
                     strtrim(cerror, 2)
                  for i = 0, c1.size(1)-1 do begin
                    if (max(abs(diff(*, *, i))) gt cerror) then begin
                      printf, unit, c1.plane(0, *, c1.n1(i)), format = '(3g16)'
                      printf, unit, c1.plane(1, *, c1.n1(i)), format = '(3g16)'
                      printf, unit, diff(0, *, i), format = '(3g16)'
                      printf, unit, diff(1, *, i), format = '(3g16)'
                    endif
                  endfor
                endif
              endelse
            endelse
          endif
          if c1.size(2) gt 0 then begin
;         Y normal planes
            diff = c1.plane(*,*, c1.n2) -c2.plane(*,*, c2.n2)
            mdiff = max(abs(diff))
            if mdiff lt cerror then begin
              if lcunit eq 0 then begin
                print, ' Y Normal Surfaces are identical'
              endif else begin
                printf, unit, ' Y Normal Surfaces are identical'
              endelse
            endif else begin
              if lcunit eq 0 then begin
                print, 'Y Normal Surfaces: ', $
                 'Maximum Differences is ', strtrim(mdiff,2)
                if keyword_set(full) then begin
                  print, 'Conductor values and difference over ', $
                     strtrim(cerror, 2)
                  for i = 0, c1.size(2)-1 do begin
                    if (max(abs(diff(*, *, i))) gt cerror) then begin
                      print,' Plane = ', strtrim(c1.n2(i), 2)
                      print, c1.plane(0, *, c1.n2(i)), format = '(3g16)'
                      print, c1.plane(1, *, c1.n2(i)), format = '(3g16)'
                      print, diff(0, *, i), format = '(3g16)'
                      print, diff(1, *, i), format = '(3g16)'
                    endif
                  endfor
                endif
              endif else begin
                printf, unit, 'Y Normal Surfaces: ', $
                 'Maximum Differences is ', strtrim(mdiff,2)
                if keyword_set(full) then begin
                  printf, unit, 'Conductor values and difference over ', $
                     strtrim(cerror, 2)
                  for i = 0, c1.size(2)-1 do begin
                    if (max(abs(diff(*, *, i))) gt cerror) then begin
                      printf, unit,' Plane = ', strtrim(c1.n2(i), 2)
                      printf, unit, c1.plane(0, *, c1.n2(i)), format = '(3g16)'
                      printf, unit, c1.plane(1, *, c1.n2(i)), format = '(3g16)'
                      printf, unit, diff(0, *, i), format = '(3g16)'
                      printf, unit, diff(1, *, i), format = '(3g16)'
                    endif
                  endfor
                endif
              endelse
            endelse
          endif
          if c1.size(3) gt 0 then begin
;         Z normal planes
            diff = c1.plane(*,*, c1.n3) -c2.plane(*,*, c2.n3)
            mdiff = max(abs(diff))
            if mdiff lt cerror then begin
              if lcunit eq 0 then begin
                print, ' Z Normal Surfaces are identical'
              endif else begin
                printf, unit, ' Z Normal Surfaces are identical'
              endelse
            endif else begin
              if lcunit eq 0 then begin
                print, 'Z Normal Surfaces: ', $
                 'Maximum Differences is ', strtrim(mdiff,2)
                if keyword_set(full) then begin
                  print, 'Conductor values and difference over ', $
                     strtrim(cerror, 2)
                  for i = 0, c1.size(3)-1 do begin
                    if (max(abs(diff(*, *, i))) gt cerror) then begin
                      print,' Plane = ', strtrim(c1.n3(i), 2)
                      print, c1.plane(0, *, c1.n3(i)), format = '(3g16)'
                      print, c1.plane(1, *, c1.n3(i)), format = '(3g16)'
                      print, diff(0, *, i), format = '(3g16)'
                      print, diff(1, *, i), format = '(3g16)'
                    endif
                  endfor
                endif
              endif else begin
                printf, unit, 'Z Normal Surfaces: ', $
                 'Maximum Differences is ', strtrim(mdiff,2)
                if keyword_set(full) then begin
                  printf, unit, 'Conductor values and difference over ', $
                     strtrim(cerror, 2)
                  for i = 0, c1.size(3)-1 do begin
                    if (max(abs(diff(*, *, i))) gt cerror) then begin
                      printf, unit, ' Plane = ', strtrim(c1.n3(i), 2)
                      printf, unit, c1.plane(0, *, c1.n3(i)), format = '(3g16)'
                      printf, unit, c1.plane(1, *, c1.n3(i)), format = '(3g16)'
                      printf, unit, diff(0, *, i), format = '(3g16)'
                      printf, unit, diff(1, *, i), format = '(3g16)'
                    endif
                  endfor
                endif
              endelse
            endelse
          endif
        endelse
      endfor
      end
