; ------------------------------------------------------------------------ 
; $Id: copypff.pro,v 1.4 2009/03/04 19:33:15 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
;+
; NAME:
;       COPYPFF
;
; PURPOSE:
;       Copy PFF datasets from one file to the end of another 
;       This routine uses a PFF utility and is much faster than using read
;       and write in PFIDL. 
;
; CATEGORY:
;       PFF Utility
;
; CALLING SEQUENCE:
;
;       COPYPFF, FID_1, FID_2 [, START] [, STOP] [, SKIP]
;  or
;       COPYPFF, FID_1, FID_2 , DATASET
;
; INPUTS:
;       FID_1:  File ID of the file to copy FROM, 0 (zero) indicates current FID
;       FID_2:  File ID of the file to copy TO, 0 (zero) indicates current FID
;           NOTE: FID_1 must not equal FID_2
;           NOTE: File ID's are shown with the command PFF_INFO and SHOWPFF
;       START:  First dataset number to copy
;               If START is a string, then get_dir will be used to obtain
;                  DATASET.  STOP and SKIP are ignored.
;               If START is missing then all datasets will be copied.
;               If START is (< 1 or undefinded) then START = 1
;               If START > NDSPFF(FID_1) then no datasets copied.
;       NOTE: If START is an array, see DATASET below.  
;             STOP and SKIP are ignored. 
;       STOP:   Last dataset number to copy
;               IF STOP is undefined, then STOP = START and only 1 dataset 
;                 is copied.
;               If STOP < START then STOP = NDSPFF(FID_1)
;       SKIP:   Skip factor 
;       DATASET: An array of valid datasets will be copied from FID_1 to FID_2.
;               Values of DATASET (< 1 or > NDSPFF(FID_1)) are ignored.
;
; KEYWORD PARAMETERS:
;       DELETE: Dataset number or array of dataset numbers to not copy
;       SORT:   Dataset comments are sorted and datasets transferred in the
;               sorted order.  Equal values will stay in the original order.
;               The standard PFF dataset directory comment is used.  This 
;                 comment string is limited to 64 characters in the present 
;                 PFF version.
;               If is a scalar and SORT > 1 then sort on STRMID(comment, SORT-1)
;               If SORT has more than one elements then sort on the string
;                   STRMID(comment, (SORT[0]-1)>0, SORT[1])
;               that is, SORT[0] is the first character to use 
;                  and    SORT[1] is the length of the string to use
;               If SORT[1] le 0 then it is ignored and all remaining 
;                  characters are used.  
;               NOTE:  ASTRO_LIB routine BSORT is used to avoid rearranging equal values
;
; OUTPUTS:
;       Copies the datasets to the indicated file.
;
; EXAMPLE:
;       Copy file id 1 to file id 2
;       COPYPFF, 1, 2
;
;       Copy dataset 5 from file id 1 to file id 2
;       COPYPFF, 1, 2, 5
;
;       Copy from file id 1 to file id 2 datasets from 5 to the end of file
;       COPYPFF, 1, 2, 5, 0
;
;       Copy file 1 to file 2 datasets 6 and every 10th dataset
;       COPYPFF, 1, 2, 6, 0, 10
;
;       Copy file id 3 to file id 2 and delete every 11th dataset starting 
;         with dataset 6
;       COPYPFF, 3, 2, delete= 6+lindgen(ndspff(3)/11)*11
;
;       Copy file id 3 to file id 4 all the datasets with beginning with "xrd"
;       COPYPFF, 3, 4, '^xrd'
;
;       Copy file id 3 to file id 4 with the comments sorted.
;       COPYPFF, 3, 4, /sort
;
;       Copy file id 3 to file id 4 with the comments sorted using first 5
;         characters.
;       COPYPFF, 3, 4, sort=[1,5]
;
;       Copy file id 3 to file id 4 with the comments sorted beginning with 
;         the 11th character in the dataset comment string.
;       COPYPFF, 3, 4, sort = 11
;
; MODIFICATION HISTORY:
;       Written by:     Paul Mix, 1/31/01
;-
 Pro copypff, fidain, fidbin, startin, stpin, skpin, $
       delete= delete, sort = sortflag
 on_error, 1

common pffcom, initpff, pff_exe
   nparams = n_params()
   if nparams eq 0 then begin
     print, '% Format: COPYPFF, FID_1, FID_2 [, START] [, STOP] [, SKIP]'
     print, '                   [, SORT = sort] [, DELETE = delete]'
     print, '       or'
     print, '          COPYPFF, FID_1, FID_2 , DATASET [, DELETE = delete]'
     return
   endif
   if nparams lt 2 then begin
     print, '% COPYPFF: Input and Output file id''s must be specified'
     return
   endif
;
;     check to see if pff has been initialized
;
   if (n_elements(initpff) eq 0) then initpff=0
   if (initpff eq 0) then begin
     print, '% COPYPFF: No PFF files have been opened'
     PRINT, '           Please open files before the copy.'
     return
   endif
   showpff, /quiet, ufid=fid, fil = fname, number = nfile, current= curfid
;   pff_info, /quiet, fid=fid, fname = fname, nfile = nfile, current= current
   if nfile lt 2 then begin
   print, '% COPYPFF: Two files must be open to copy datasets.'
     PRINT, '         Please open files before the copy.'
     return
   endif
   if n_elements(fidain) eq 0 or n_elements(fidbin) eq 0 then begin
     print, '% COPYPFF: File id''s must be defined'
     return
   endif
;   ok = where (current ne 0, nok)
;   curfid = long(fid[ok[0]])
   if fidain eq 0 then fida = curfid else fida = long(fidain[0])
   if fidbin eq 0 then fidb = curfid else fidb = long(fidbin[0])
   if fida eq fidb then begin
     print, '% COPYPFF: Input and Output files must be different.'
     print, '           Input: ',strtrim(fida, 2), ', Output: ',strtrim(fidb,2)
     return
   endif
   oka = where (fida eq fid, noka)
   if noka eq 0 then begin
     print, '% COPYPFF: Input file id is not valid. FID =' , fida
     print, '      FID    File'
     for i = 0, nfile-1 do print, fid[i], fname[i]
     return
   endif
     
   okb = where (fidb eq fid, nokb)
   if nokb eq 0 then begin
     print, '% COPYPFF: Output file id is not valid. FID =' , fidb
     print, '      FID    File'
     for i = 0, nfile-1 do print, fid[i], fname[i]
     return
   endif
   ndsa = ndspff(fida)

   if nparams eq 2 then begin
     start = 1L
     stpin = ndsa
     skpin = 1L
     nparams =5
   endif else if n_elements (startin) eq 0 then begin
     print, '% COPYPFF:  If START is present, it must be defined.'
     return
   endif else start = startin
   if size(start, /type) eq 7 then begin
     get_dir, fida, start[0], dataset=dataset, comment=comment, $
           number = number, /no_trim
     start = dataset
   endif else if n_elements(start) gt 1 then begin
     dataset = long(start)
     ok = where (dataset gt 0 and dataset le ndsa, nok)
     number = nok
     if nok eq 0 then begin
       print, '% COPYPFF: No valid datasets'
       return
     endif
     number = nok
     dataset = dataset[ok]
     start = dataset
     if nok eq 1 then stp = dataset
     if keyword_set (sort) and nok gt 1 then begin
       get_dir,  dataset=dataset, comment=comment, $
           number = number, /no_trim
       dataset= start
       comment = comment [start-1]
       number = nok
     endif
   endif else start = long(start)
   if n_elements(start) gt 1 then lcarray = 1 else lcarray = 0  
   if lcarray eq 0 then begin
     if start lt 1 then start = 1L
     if start gt ndsa then begin
       print, '% COPYPFF: Start must be less than the number of datasets: ', $
                 strtrim(ndsa, 2)
       return
     end
     if n_elements (stpin) eq 0 then stp = start else stp = long(stpin[0])
     if stp lt start then stp = ndsa else if stp gt ndsa then stp = ndsa 
     if n_elements(skpin) eq 0 then skp = 1L else skp = long(skpin[0])
     get_dir, fida, start,stp, dataset=dataset, comment=comment, $
           number = number, /no_trim
     if skp ne 1 then begin
       number = 1L+ (number-1)/skp
       indx = lindgen(number)*skp
       dataset = dataset [indx]
       comment = comment [indx]
     endif
   endif
   if keyword_set (sortflag) then begin
     st = long(sortflag[0]-1) > 0
     if n_elements (sortflag) gt 1  then n = long(sortflag[1]) else n = -1
     if n le 0 and st gt 0 then for i = 0, number-1 do $
        comment[i] = strmid(comment[i], st) else if n gt 0 then $
        for i = 0, number-1 do comment[i] = strmid(comment[i], st, n)
     for i = 0, number-1 do comment[i] = comment[i]+string(i, format='(i5.5)')
     a = bsort(comment)
     dataset = dataset[a]
     lcarray =1
   endif
   if keyword_set (delete) then begin
     ok = where(delete gt 0, nok)
     if nok lt n_elements(delete) then delete = delete[ok]
     for i = 0, nok-1 do begin
       j = where(dataset eq delete[i], nok)
       if nok gt 0 then dataset[j]=0
     endfor
     ok = where (dataset gt 0, nok) 
     if nok eq 0 then begin
       print, '% COPYPFF: No valid datasets after delete'
       return
     endif

     if nok lt n_elements(dataset) then begin
       lcarray = 1
       dataset = dataset[ok]
       number = nok
     endif
   endif
   if lcarray eq 1 then begin
     start = long (dataset)
     nstart = long (number)
     skp = 1L
     stp = 0L
   endif else nstart = 1L
      ier = 0L

      if initpff eq 1 then begin
; HP
        junk=call_external(PFF_EXE,'pff_slcopy', fida, fidb, $
           start, nstart, stp, skp, ier)

      endif else if initpff eq 2 then begin
; Sun
        junk=call_external(PFF_EXE,'pff_slcopy_', fida, fidb, $
           start, nstart, stp, skp, ier)

      endif else if initpff eq 4 then begin
; WIN
        junk=call_external(PFF_EXE,'_PFF_SLCOPY@8', fida, fidb, $
           start, nstart, stp, skp, ier)
      endif 
      if ier ne 0 then print, '% COPYPFF: Error with dataset copy'
END
   
