; ------------------------------------------------------------------------ 
; $Id: createpff.pro,v 1.6 2010/04/13 16:46:12 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
  pro createpff, p1, p2, fileid = fileid, strict = strict, error = err, $
    delete = delete
;+
; Create a PFF file for output. Current file pointer is set to this file.
;   format:  CREATEPFF  [, filename]  [, fileid ] [, FILEID = fid]
;                       [, /STRICT]  [, /DELETE] [, ERROR = error]
;   where:
;      filename - name of the file to be created
;                 Default value is 'PFFdefault.pff'
;                 If the file exists, it will be expanded to a unique name
;                    unless STRICT is set.
;      STRICT   - If set and the file exists, CREATEPFF will terminate.
;                 Default: STRICT=0; Filename will be expanded to a unique name.
;      DELETE   - If set and filename exists, the old file will be deleted.
;                 Caution:  You will not be asked if you are sure!
;      fileid   - user specified file id 
;      fid      - Alternate user specified file id. If fileid is present
;                then fid is ignored.
;      error    - Returns the error flag
;
;      NOTE:    If fileid (or fid) is specified, it will be used as the file id.
;               If fileid is not provided or is zero, a value will be 
;                 provided; default value is the first unused integer >/= 1.
;                 If fileid is provided but has been used for another 
;                 file, the program will inquire if the user wishes to 
;                 close the old file, to specify a new file id, or to accept 
;                 the default.
;
;  Examples:
;     Create a PFF file, "test" with a default file id
;       createpff, 'test'
;     Creates a PFF file, "TEST.DAT" with a file id of 66
;       createpff, `test.dat', 66
;     Create a PFF file, "PFFdefault.pff" with a default file id
;       createpff
;-
; common block
      common pffcom, initpff, pff_exe
;
on_error, 1
;
;     check to see if pff has been initialized
;
      if (n_elements(initpff) eq 0) then init_pff else $
        if (initpff eq 0) then init_pff

      if initpff eq 0 then begin
        print, 'PFF libraries are not supported by this Operating System'
        return
      endif
        
;
;     check on the number of parameters
;
      nparams = n_params()
;
;     be sure we have a file name
;
      if (nparams eq 0) then nparams = 1
;      get_string, p1 
      if (n_elements(p1) gt 0) then p1= p1[0] else p1= 0L
;
;     be sure p1 is a character string
;
      if (size(/type, p1) ne 7) then p1 = 'PFFdefault'
;
      vers = strupcase(strmid(!version.os_family, 0, 3))
      if (vers eq 'UNI') then begin
;       get system variable substitution for unix systems
        temp = p1
        FULFNAM, temp, p1
      endif
      p1 = file_expand_path(p1)
      if !version.os eq 'linux' then resolve_it, p1
      info = file_info(p1)
      if info.exists gt 0  then begin
        if keyword_set(delete) then begin
          file_delete, p1, /quiet
          info = file_info(p1)
          if info.exists gt 0 then begin
             print, '% CREATEPFF: Unable to delete file'
             print, '       FILE: ', p1
             return
          endif
        endif else begin
          if keyword_set(strict) then begin
            print, '% CREATEPFF: File already exists, use OPENPFF.'
            print, '       FILE: ', p1
            return
          endif
          p1= find_newfile(p1)
        endelse
      endif
;
      if (n_elements(p2) eq 0) then begin
        if n_elements(fileid) gt 0 then p2 = fileid(0) else p2 = 0L
      endif else p2 = p2(0)
      get_pffinteger, p2 
;
;     for the shared library version we need byte arrays
;
      lstr = 256L
      if strlen(p1) ge lstr then begin
        Print, '% CREATEPFF: File name must be less than 256 characters'
        err=1
        return
      endif
      err = 0L
      str2byt, p1, p3, lstr
;
      if initpff eq 1 then begin
; HP
        junk = call_external (PFF_EXE, 'pff_slcreate', p3, p2, err, lstr)
      endif else if initpff eq 2 then begin
; Sun
        junk = call_external (PFF_EXE, 'pff_slcreate_', p3, p2, err, lstr)
      endif else if initpff eq 3 then begin
; VMS
        p3 = string (p3)
        junk = call_external (PFF_EXE, 'pff_create', p3, p2, err)
      endif else if initpff eq 4 then begin
; WIN
        junk = call_external (PFF_EXE, '_PFF_SLCREATE@8', p3, p2, err, lstr)
      endif else if initpff eq 5 then begin
; MAC
        junk = call_external (PFF_EXE, 'pff_slcreate', p3, p2, err, lstr)
      endif 
;      print, strtrim(p3, 2), ' opened with a file id of ', strtrim(p2, 2)
      fileid = p2
      return
      end
