; ------------------------------------------------------------------------ 
; $Id: openpff.pro,v 1.10 2010/05/26 13:43:33 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
  pro openpff, p1, p2, error = error, path = path, filter = filter, $
              repair = repair, overwrite = overwrite, skip_repair = skip_repair, $
              filename = filename, fileid = fileid, ndset = ndset, help = help
;+
;  Open a PFF file.  The current file pointer is set to this file id and 
;  the dataset pointer for this file is set to 1.
;  format: OPENPFF [, File] [, Fileid ] [, Error = error] [, NDSET =ndset]
;                  [, PATH = path] [, FILTER = filter] [, HELP = help]
;                  [, FILENAME = filename] [, FILEID = fileid]
;  where:
;    File   - name of the PFF file to be opened.  The default file extension
;             is ".PFF".  Both upper and lower case versions of the file name
;             will be tried.
;             If file is omitted or undefined or is not a string,
;             DIALOG_PICKFILE will be used to select a file.
;             The initial path in PICKFILE is the current directory or
;             the PATH to the last file selected.
;             NOTE: PATH and FILTER may be used to set the initial path and
;                   filter for PICKFILE.
;             NOTE: FULFNAM procedure is used to handle environment variables
;                   See FULFNAM for details
;             Note: If FILE is already open, the setpff will be used to set the
;                   pointer to that file.  The FILEID will be set to the current
;                   FILEID of the open file.  SETDIRPFF will be called to set the
;                   dataset pointer to 1.
;    Fileid - user specified file id 
;             (If fileid is specified, it will be used as the file id.  If 
;             fileid is not provided or is zero, a value will be 
;             provided; default value is the first unused integer greater
;             than 0.  If fileid is provided but has been used for another 
;             file, the program will inquire if the user wishes to 
;             close the old file, to specify a new fid, or to accept 
;             the default.  If positional parameter is valid, keyword will 
;             be ignored.
;    Error  - Error parameter
;             If this error is zero, no error encountered.
;             If no file specified for open then error = -13
;             If Error = -99, file not found
;             If Error = n > -99, error is a pff library error code defined
;               by the particular routine where the error occurred.
;               For example, file has no PFF header.
;    help   - Print a help text
;    PATH   - Path to the current file 
;             Default is the current directory in the first call to OPENPFF but
;             the default path is changed to the final path selected is 
;             retained as the default in the next call to openpff.
;    FILTER - File filter - The default may be set in USERQSINIT; if undefined
;             the default is '*'.  The default filter is set only with
;              USERQSINIT or SETDEFAULT.
;    NDSET  - Number of datasets
;    REPAIR - If set and file is not opened due to a framing error, REPAIR_PFF will
;             be called to attempt to repair the file after moving the file to 
;             /tmp or C:\TEMP.
;             NOTE:  These file copies will accumulate in /tmp or C:\TEMP unless 
;             overwrite is set.
;    OVERWRITE - Ignored unless REPAIR is set.
;             If REPAIR is set the file will be renamed to a unique name unless
;             overwrite is set.  This is important if multiple repaired files are
;             open simultaneously.
;    FILENAME - Name of the file opened.
;              Filename is also available with:
;                get_dir, 0, 1, filename= filename
;
;  Examples:
;    Open a PFF file with the default file id
;      openpff
;    Open a PFF file, 'users/test', with the default file id
;      openpff, '/users/test'
;    Open a file to be selected from a menu with a file id of 14
;      openpff, ' ', 14
;    Open 'test' or 'test.pff' with a file id of 14
;      openpff, 'test', 14
;    Determine keywords and parameters for openpff
;      openpff, /h
;
;-
;
      on_error, 1
      common pffcom, initpff, pff_exe
      COMMON savepicker, oldfilter, oldpath
;; @files_common.dat
;
;     check on the number of parameters
;
      nparams = n_params()
      if nparams eq 0 and keyword_set(help) then begin
print, '% Format: OPENPFF [, File] [, Fileid ] [, Error = error]'
print, '                  [, PATH = path] [, FILTER = filter]'
print, '                  [, NDSET = ndset] [, FILEID = fileid]'

        return
      endif
;
;     check to see if pff has been initialized
;
      if (n_elements(initpff) eq 0) then init_pff else $
        if (initpff eq 0) then init_pff
      if initpff eq 0 then begin
        print, 'PFF libraries are not supported by this Operating System'
        return
      endif
      filename = ''
;
;     be sure we have a file name
;
      error = -13
      lcexists = 0
      if (nparams eq 0) then begin
        p1 = ''
        nparams = 1
      endif else if n_elements(p1) eq 0 then begin
        p1 = ''
      endif else if (size(p1, /type) ne 7) then begin
        p1 = ''
      endif else  p1 = strtrim(p1, 2)
      cd, current = current
      psep = path_sep()
      if strlen(p1) lt 1 then begin
        IF (N_ELEMENTS(PATH) EQ 0 and n_elements(oldpath) eq 0) THEN BEGIN
          if initpff lt 3 then path = current+psep else path = current
        endif else if n_elements(path) eq 0 then path = oldpath
        if n_elements(filter) eq 0 and n_elements(oldfilter) eq 0 then $
          filter = '*' else if n_elements(filter) eq 0 then filter = oldfilter
        p1 = dialog_pickfile( path = path, get_path =tempath, filter = filter, $
          title= 'Select PFF file to read', /must_exist)
        p1 = strtrim(p1, 2)
        if strlen(p1) lt 1 then return
        lcexists = 1
        if n_elements(tempath) gt 0 then oldpath = tempath
      endif
;
      vers = strupcase(strmid(!version.os_family, 0, 3))
      if (vers eq 'UNI') then begin
        lcunix = 1
;       get system variable substitution for unix systems
        temp = p1
        fulfnam, temp, p1
      endif else lcunix =0
      p1 = file_expand_path(p1)
      if !version.os eq 'linux' then resolve_it, p1
      if lcexists eq 0 then begin
        pfile = p1
        valid = file_info(pfile)
        if valid.exists eq 0 then begin
;         Check for case independence
          dir = file_dirname(pfile)
          ls, dir+psep+'*', curr = filename, /quiet
          ok = get_match(pfile, filename, nel=nel, casese=0, regexp =0)
          if nel gt 0 then begin
            if nel gt 1 then begin
              ok2 = get_match(pfile+'.pff', filename, nel=nel2, casesen=0, regexp =0)
              if nel2 eq 0 or nel2 gt 1 then begin
                print, '% OPENPFF: Unable to determine file to open'
                if nel2 gt 0 then begin
                  print, '  Possible files are:'
                  for i = 0, nel2-1 do print, filename[ok2[i]]
                endif
                return
              endif
              p1 = filename[ok2[0]]
            endif else p1 = filename[ok[0]]
          endif else begin
            print, '% OPENPFF: Unable to find file.'
            print, 'File: '+p1
            return
          endelse
        endif
      endif
      valid = file_info(p1)
      if (valid.directory) then begin
        print, 'OPENPFF: You cannot open a directory as a PFF file.'
        print, 'File: '+p1
        return
      endif
      if valid.read eq 0 then begin
        print, '% OPENPFF:  You do not have read access'
        print, 'File: '+p1
        return
      endif
      showpff,  current= current, ufid = ufid, number = number,filename = filename, /quiet
      if number gt 0 then begin
        ok = get_match(p1, filename, nel=nel, casesen=lcunix, regexp =0)
        if nel gt 0 then begin
          print, '% OPENPFF: File opened with FID = '+strtrim (ufid[ok[0]], 2)
          print, '  File: '+p1
          p2 = ufid[ok[0]]
          setpff, p2
          fileid =p2
          setdirpff, 1
          error = 0
          ndset = ndspff()
          return
        endif         
      endif
;
; user specified file id
;
      if n_elements (p2) eq 0 then begin
        if n_elements(fileid) gt 0 then p2 = long(fileid) else p2 = 0L
      endif else p2 = long(p2)
      lstr = 256L
      error = 0L
      NDSET = 0L
      if strlen(p1) ge lstr then begin
        Print, '% OPENPFF: File name must be less than 256 characters'
        return
      endif 
      str2byt, p1, p3, lstr
;
      if initpff eq 1 then begin
; HP 
        junk = call_external(PFF_EXE, 'pff_slopen', p3, p2, lstr, ndset, error)
      endif else if initpff eq 2 then begin
; Sun
        junk = call_external(PFF_EXE, 'pff_slopen_', p3, p2, lstr, ndset, error)
      endif else if initpff eq 3 then begin
; VMS
        file = string(p3)
        junk = call_external(PFF_EXE, 'pff_open', file, p2, ndset, error)
      endif else if initpff eq 4 then begin
; WIN
       junk = call_external(PFF_EXE, '_PFF_SLOPEN@8', p3, p2, lstr, ndset, error)
      endif else if initpff eq 5 then begin
; MAC 
       junk = call_external(PFF_EXE, 'pff_slopen', p3, p2, lstr, ndset, error)
      endif

      if error eq 0 then begin
        fileid = p2
        filename = strtrim(p3, 2)
      endif else begin
          filename = strtrim(p3, 2)
          if keyword_set(skip_repair)  then begin
            print, '% OPENPFF: Repair failed'
            return
          endif
          if not keyword_set(repair) then begin
            ans = ''
            read, prompt =' Would you like to repair this file (y or n): ', ans
            if strlowcase(strmid(ans, 0, 1)) ne 'y' then return
          endif
          file = file_basename(filename)
          name = strupcase(strmid(!version.os, 0, 3))
          dire = (name eq 'WIN') ? 'C:\TEMP\' : '/tmp/'
          file = dire + file
          if keyword_set(overwrite) then file_copy, filename, file, /overwrite else begin
            file = find_newfile(file, /quiet)
            file_copy, filename, file
          endelse
          repair_pff, file, error = error
          openpff, file, /skip_repair
      endelse
      end
