; ------------------------------------------------------------------------ 
; $Id: pff_diag.pro,v 1.4 2010/04/13 17:40:04 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
 pro pff_diag, filin, debug = debug, verbose = verbose, inquire = inquire, $
        helpme = helpme, quiet = quiet
;
;+
;     This routine sets pff diagnostic options
;
;     format: pff_diag [, FILE] [, DEBUB = debug] [, VERBOSE = verbose]
;                     [, INQUIRE = inquire] [, /HELPME] [, /QUIET]
; 
;     where
;       FILE    - Used only if debug is set.
;                 If debug is a string, FILE is ignored.
;       DEBUG   - If set, DEBUG diagnostics are toggled
;                 File must be provided unless debug is a string
;       VERBOSE - If set, VERBOSE diagnostics are toggled
;       INQUIRE - Returns a two element vector with the value of 0 or 1 depending
;                 on whether verbose and debug are set.
;                 INQUIRE(0) = 0 if VERBOSE is not set
;                            = 1 if VERBOSE is set
;                 INQUIRE(1) = 0 if DEBUG is not set
;                            = 1 if DEBUG is set
;       HELPME  - If set, returns the command format
;       QUIET   - If set, no output to the terminal unless errors are present.
;
;     Example
;       Determine the status of PFF diagnostics
;          pff_diag
;   or     pff_diag, inquire = inquire
;
;       Toggle VERBOSE diagnostics 
;          pff_diag, /verbose
;
;       Toggle DEBUG diagnostics with output to go to '/tmp/debug.out'
;       Return the status of the diagnostics in stat
;          pff_diag, '/tmp/debug.out', /debug, inquire = stat
;   or     pff_diag, debug = '/tmp/debug.out', inquire = stat
; 
;       Turn off debug diagnostic.
;       (Note: If debug is off, a message will be printed but debug will remain
;              off since no file is provided.)
;          pff_diag, /debug
;
; MODIFICATION HISTORY:
;       Written by:     LPMix, October 27, 1998
;
;-
 on_error, 1
      common pffcom, initpff, pff_exe
      common pff_diag_common, debug_flag, verbose_flag
;
;     check to see if pff has been initialized
;
      if (n_elements(initpff) eq 0) then initpff=0
      inquire = [-1L, -1L]
      if (initpff eq 0) then begin
        print, '% PFF_DIAG: No PFF files are open'
        return
      endif
      if n_elements(debug_flag) eq 0 then begin
        verbose_flag = 1L
        debug_flag = 0L
      endif
;
      if keyword_set(helpme) then begin
print, '% Format: PFF_DIAG [, FILE] [, DEBUB = debug] [, VERBOSE = verbose]'
print, '                [, INQUIRE = inquire] [, /HELPME] [, /QUIET]'
        return
      endif
;
      filename = ''
      flag = 0L
      if keyword_set (verbose) then flag = 1L
      if keyword_set (debug) then begin
        flag = flag+2L
        ss = size(debug)
        if ss[ss[0]+1] eq 7 then filename  = strtrim(debug[0], 2) else $
          if n_elements(filin) gt 0 then filename  = strtrim(filin[0], 2) 
      endif
      lfname = strlen(filename)
      if ((debug_flag eq 0) and (lfname eq 0) and (flag gt 1)) then begin
        print, '% PFF_DIAG: Valid filename must be provided if DEBUG is set'
        return
      endif
      lcverb = 0L
      lcdbg  = 0L
      ier = 0L
      if lfname eq 0 then begin
        fil= make_array(8, /byte, value = 32B)
        lfname = 8L
      endif else begin
        if initpff lt 3 then begin
          temp = filename
          fulfnam, temp, filename
          lfname = strlen(filename)
          if lfname eq 0 then begin
;           error in fulfnam
            filename = temp
            lfname = strlen(filename)
          endif
        endif
        lfname = ((lfname-1) /8 +1)*8L
        if lfname ge 256 then begin
          print, '% PFF_DIAG: Debug file name must be less than 256 characters'
          return
        endif
        str2byt, filename, fil,lfname 
      endelse
      if initpff eq 1 then begin
; HP
        junk=call_external(PFF_EXE,'pff_slsvrb', flag, lcverb, lcdbg, $
             fil, lfname, ier )
      endif else if initpff eq 2 then begin
; Sun
        junk=call_external(PFF_EXE,'pff_slsvrb_', flag, lcverb, lcdbg, $
             fil, lfname, ier )
      endif else if initpff eq 4 then begin
; WIN
        junk=call_external(PFF_EXE,'_PFF_SLSVRB@8', flag, lcverb, lcdbg, $
             fil, lfname, ier )
      endif else if initpff eq 5 then begin
; MAC
        junk=call_external(PFF_EXE,'pff_svrb', flag, lcverb, lcdbg, $
             fil, lfname, ier )
      endif else if initpff eq 3 then begin
; VMS
        fnamef = string (fil)
        junk=call_external(PFF_EXE,'pff_slsvrb', flag, lcverb, lcdbg, $
             fil, ier )
      endif
      If ier ne 0 then $
        print, '% PFF_DIAG: Error with PFF diagnostic toggle, IER =', $
        strtrim(ier, 2)
      inquire = [lcverb, lcdbg]
      verbose_flag = lcverb
      debug_flag = lcdbg
      if keyword_set(quiet) then return
 
      if lcverb eq 0 then print, '% PFF_DIAG: VERBOSE mode is NOT set' else $
        print, '% PFF_DIAG: VERBOSE mode is set'
      if lcdbg eq 0 then print, '% PFF_DIAG: DEBUG mode is NOT set' else $
        print, '% PFF_DIAG: DEBUG mode is set'
      return
      end
