; ------------------------------------------------------------------------ 
; $Id: pff_precision.pro,v 1.3 2007/06/06 19:59:47 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
 Pro pff_precision, pid, full_precision = fullprecision, $
        new_files = newfiles, all_files = allfiles, helpme = helpme, $
        show_precision = showprecision, PRECISION = precision, quiet = quiet
;+
; NAME:
;       PFF_PRECISION
;
; PURPOSE:
;       Set the floating point precision of one or more PFF files
;       or
;       Set the floating point precision of all new files opened
;
;  Note: This procedure has no effect on reading PFF data.  Date is read at 
;        the precision at which it was written.
;  Note: Precision may be set on a dataset by dataset basis.
;  Note: READHDR will indicate the precision which floating point numbers have
;        been written.
;
; CALLING SEQUENCE:
;       PFF_PRECISION  [, FILE_ID] [, /FULL_PRECISION] [, /HELPME]
;                      [, /NEW_FILES] [, /ALL_FILES] [, /SHOW_PRECISION]
;                      [, PRECISION = precision] [, /QUIET]
;
; WHERE:
;       FILE_ID        - PFF File_ID   If fileid  is missing or 0, the current
;                        PFF file is used the unless NEW_FILES or ALL_FILES
;                        is set.
;       HELP_ME        - Print a quick description of the command.  
;                        No other action is taken.
;     If SHOW_PRECISION is zero or missing then:
;       FULL_PRECISION - If set, indicated file(s) will write 32-bit IEEE 
;                        floating point numbers.
;                        If omitted or zero, floating point numbers are 
;                        linearly mapped to 2 byte integers.
;       NEW_FILES      - If set, and FILE_ID is missing or zero, a PFF global
;                        default will be set which controls the floating 
;                        precision for subsequently opened files.  The 
;                        precision of files already open is unchanged.
;       ALL_FILES      - If set, and FILE_ID is missing or zero, a PFF global
;                        default will be set which controls the floating 
;                        precision for all files, including those already open
;                        and those which are yet to be opened.
;       PRECISION      - A two dimensional array [2, *] where
;                        PRECISION[0, *] = file ID (File ID = 0 indicates the 
;                                          PFF global default value)
;                        PRECISION[1, *] = Previous Precision Flag Value.
;                                          Valid values are:
;                                          0 - Default scaled 2 byte array
;                                          1 - 32-bit IEEE arrays
;     If SHOW_PRECISION is set then:
;       NEW_FILES      - If set, and FILE_ID is missing or zero, the PFF global
;                        default precision flag will be returned which controls
;                        the floating precision for subsequently opened files.
;       ALL_FILES      - If set, and FILE_ID is missing or zero, the value of
;                        the global default precision flag and the precision
;                        flag for each of the open files is returned.
;       PRECISION      - A two dimensional array [2, *] where
;                        PRECISION[0, *] = file ID (File ID = 0 indicates the 
;                                          PFF global default value)
;                        PRECISION[1, *] = Current Precision Flag.
;                                          Valid values are:
;                                          0 - Default scaled 2 byte array
;                                          1 - 32-bit IEEE arrays
;       QUIET          - If set, no output to the terminal.  All data is
;                        returned in the PRECISION variable.
;
; EXAMPLE:
;       Set the precision to full precision for all pff files.
;         PFF_PRECISION, /full_precision, /all_files
;       Set the precision to full precision for file with fileid =8
;         PFF_PRECISION, 8, /full_precision
;       Set the current file to full precision
;         PFF_PRECISION, /full_precision
;       Return the current file to default precision
;         PFF_PRECISION
;      or PFF_PRECISION, 0, full_precision=0
; MODIFICATION HISTORY:
;       Written by:     L. P. Mix, November 29, 2000
;-

on_error, 1
      common pffcom, initpff, pff_exe
      mxfils = 24L
;
      if keyword_set(helpme) then begin
        print, $
         '% Format: PFF_PRECISION  [, FILE_ID] [, /FULL_PRECISION] [, /HELPME]'
        print, '              [, /NEW_FILES] [, /ALL_FILES] [, /SHOW_PRECISION]'
        print, '              [, PRECISION = precision] [, /QUIET]'
        return
      endif
;
;     check to see if pff has been initialized
;
      if (n_elements(initpff) eq 0) then init_pff else $
        if (initpff eq 0) then init_pff
      if initpff eq 0 then begin
        print, $
          '% PFf_PRECISION: PFF libraries are not supported by this computer'
        return
      endif
; 
;     File ID - Default = 0
;
      if n_elements(pid) eq 0 then fid = 0L else fid = long (pid)
; 
      if fid eq 0 then begin
        if keyword_set(allfiles) then FLAG=3L else $
          if keyword_set(newfiles) then FLAG=1L else FLAG =0L
      endif else FLAG = 0L
      precision = make_array(2, mxfils, /long, value =0L)
      nfiles =0L
;  use setfile as a flag
;     setfile = -1 means get precision
;     setfile = 0  means set to the default precision (2 bytes)
;     setfile = 1  means set to the full precision (4byte IEEE)
      if keyword_set(showprecision) then setfile = -1L else $
      if keyword_set(fullprecision) then setfile = 1L else setfile =0L
; 
      if initpff eq 1 then begin
; HP
        junk=call_external(PFF_EXE,'pff_slprecision', fid, setfile, flag, $
             precision, mxfils, nfiles)
      endif else if initpff eq 2 then begin
; Sun
        junk=call_external(PFF_EXE,'pff_slprecision_', fid, setfile, flag, $
             precision, mxfils, nfiles)
      endif else if initpff eq 4 then begin
; WIN
        junk=call_external(PFF_EXE,'_PFF_SLPRECISION@8', fid, setfile, flag, $
             precision, mxfils, nfiles)
      endif else if initpff eq 5 then begin
; MAC
        junk=call_external(PFF_EXE,'pff_slprecision', fid, setfile, flag, $
             precision, mxfils, nfiles)
      endif
      if nfiles eq 0 then begin
        precision = -1
        if keyword_set(quiet) then return
        if setfile lt 0 then print, $
          '% PFF_PRECISION: Unable to determine precision' else  $
          print, '% PFF_PRECISION: Unable to set precision' 
        return
      endif
      precision = precision[*, 0:nfiles-1]
;     We are finished unless Show_precision is set      
      if setfile ge 0 then return
      if keyword_set (quiet) then return
      prec=['2 Bytes', '4 Bytes']
      print, '  FID    Current Precision'
      if precision[0,0] eq 0 then print, $
             ' Default - ', prec[precision[1, 0]] else $
      print, precision[0, 0], prec[precision[1, 0]], format ='(i5, 6x, a)'
      if nfiles gt 1 then for i = 1, nfiles-1 do $
        print, precision[0, i], prec[precision[1, i]], format ='(i5, 6x, a)'
        
 end
