; ------------------------------------------------------------------------ 
; $Id: re.pro,v 1.7 2010/09/20 18:00:48 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
 Pro  re, wdfa, dataset, alldata=alldata, narray=narray, block=block, $
         fileid=fidin, xydata = xydatain, sortit = sortit, pmes =pmes
;+
;  Reads a PFF dataset into a WDF array.  Procedure options are explained
;  below.
;
;  Format:  RE, wda  [, dataset] [, NARRAY = narray] [, ALL = all] 
;             [, BLOCK = block] [, FILEID = fileid] [XYDATA = xydata]
;             [, /SORTIT] [, /PMES]
;  where:
;    wda -     WDF array number or an array of numbers
;    dataset - indicates a PFF dataset number, an array of dataset numbers,
;              or is a search string for dataset comments.  
;              If dataset is missing or zero, then it is set to the 
;              current dataset.
;        Note: A '*' value of dataset will return all datasets.
;        See: GET_DSET for more details
;    alldata - nonzero value indicates all datasets with the search 
;              string are to be read into WDF arrays.  If all is 
;              missing or zero, only the elements of wda will be filled.
;    fileid -  file id of the PFF file containing the data
;    xydata -  Must be set to zero to store nonuniform data as uniform
;              All uniform datasets will be stored as WDF arrays. 
;              If data is not read from UF1 or UF3 dataset, 
;              then data will be stored as XY data unless xydata = 0.
;    narray -  number of datasets read into WDF arrays
;    block -   block of data in the dataset - Currently ignored
;              default is 1 
;    sortit -  If SORTIT is set then dataset comments will be sorted.
;    pmes   -  If set, points larger than 5e29 are deleted and the data is 
;              stored as XY data.  This is only checked for UF1 and the
;              first vector in NGD data.
;
; the parameters wda and dataset can take the following combination of values
; to produce the correspondint actions:
;  wda          dataset         action
; array(N)  zero or missing   Read current dataset and (N-1) successive
;                             datasets into the array(N) WDF arrays.
; integer      array(M)       Read array(M) datasets and store in WDF array
;                             integer and the following (M-1) WDF arrays.
; array(N)     array(M)       M must equal N; read the array(M) datasets
;                             and store in the array(N) WDF arrays.
; integer      string         Read dataset with string into WDF array, integer.
;             (all = 0)       If more than one array with string, look for
;                             string followed by spaces at the beginning of
;                             the dataset comment.
;            (all not zero)   Read all datasets with string into WDF array,
;                             integer and successive WDF arrays.
; array(N)     string         Read all datasets with string into WDF arrays,
;                             array(N).  Number of array elements must equal
;                             number of datasets with string in the dataset
;                             comment.If string occurs in only 1 dataset,
;                             then read that dataset and the next (N-1) 
;                             datasets into WDF arrays, array(N).
;  Examples:
;    Read dataset 34 into array 1 from file with file id of 4.
;      re, 1, 34, fil = 4
;    Read into WDF array 2,..., 10, datasets 34,..., 42 from the current file
;      re, indgen(9)+2, 34
;  or: re, 2, indgen(9)+34
;    Read into WDF arrays beginning with 2 all datasets beginning with "mspin" 
;    in the current file and indicate the number of arrays in the variable,
;    nread. 
;      re, 2,  '^mspin' , n=nread, /all 
;    Read into WDF arrays [2, 5, 8, 11] the datasets [3, 4, 1, 2] from file
;    with id of 2
;      re, [2,5,8,11], [3,4,1,2], file= 2
;    Read into WDF arrays [2,5,8, 11] the datasets having the string "vtot".
;    Note:  vtot must occur in either 1 array or 4 arrays.
;      re,[2,5,8,11], 'vtot'
;-
;     add common block
@com.dat
      common pffcom, initpff, pff_exe
;
;     check to see if pff has been initialized
;
      on_error, 1
      if (n_elements(initpff) eq 0) then initpff = 0
      if initpff eq 0 then begin
        print, '% RE: No PFF files are open'
        return
      endif
;
      narray = 0
      if n_elements(initwdf) eq 0 then wdfinit
      if (n_params() lt 1 ) then begin
        print,'% Format: RE, wda  [, dataset] [, NARRAY = narray] [, ALL = all]'
        print,'         [XYDATA = xydata] [, BLOCK = block] [, FILEID = fileid]'
	return
      endif
      wdfa = get_save_array(wdfa, 0)
      if wdfa(0) eq 0 then begin
        print,'wdf array value must be provided'
        return
      end
      wdfar = wdfa(*)
      if wdfar(0) lt 1 or wdfar(0) gt maxwdfarray then begin
        print, 'wdf array must be greater than 0 and </=', maxwdfarray
        return
      endif
      if n_elements (xydatain) eq 0 then xydata = 0 else xydata = xydatain
      if n_elements (alldata) eq 0 then alldata = 0
      if n_elements (dataset) eq 0 then dataset = 0
      if n_elements (block) eq 0 then block = 1
      if n_elements (fidin) eq 0 then fid = 0L else fid = long(fidin[0])
      blo = block
      pid = long(fid)
      nwdf = n_elements(wdfar)
      if alldata eq 0 then nmax = nwdf else nmax = maxwdfarray
      get_dset_string, dataset, fid, map, nmap, nmax, ier, lcstring
      if ier ne 0 then begin
         print, 'No datasets found. Please change the dataset specification'
         return
      endif
;
      if alldata eq 0 and nmap ne nwdf and lcstring gt 0 then begin
        print, ' The number of WDF arrays must equal the number of '+ $
           'datasets with the string:'
        print, '             ' , dataset, ' -- Current datasets are'
        dirpff, dataset
        return
      endif else if lcstring lt 0 then begin
        alldata = 1
        nmax = maxwdfarray
      endif
      if nwdf gt 1 and nmap gt 1 then begin
        if nwdf ne nmap then begin
          print, ' Number of wdf arrays is ', nwdf
          print, ' Number of datasets is ', nmap
          print, ' These must be equal if both are greater than 1'
          return
        endif
      endif else begin
        if nwdf gt 1 then begin
          if map(0) gt 0 then map = map(0) + indgen(nwdf) else $
              map= make_array(nwdf, /int)
        endif
        if nmap gt 1 then wdfar = wdfar(0)+indgen(nmap)
      endelse
      narray = max([nwdf, nmap]) < maxwdfarray
      nread = 0
      ok = where (wdfar gt 0 and wdfar le maxwdfarray, nok)
      if nok ne narray then begin
        print, '% RE: Some WDF arrays are not valid.'
        print, '   Only ', strtrim(nok,2) , ' arrays will be read'
        narray = nok
      endif
      if keyword_set(sortit) then begin
        map = map[0:nok-1]
        get_dir, fid, /no_trim, comment = comment
        comment = comment[map-1]
        map = map[sort(comment)]
      endif
      for i = 0, narray-1 do begin
        data = map(i)
        wdf = wdfar(ok(i))
        readhdr,data,ier,fileid=fid, pfftyp = pfftyp, $
           dlength = dlength, tlabel= tlabel, clabel=clabel, $
           aptype= aptype, namefile = filename, iextra = iextra, /quiet
;
        if ier ne 0 or n_elements(pfftyp) eq 0 then begin
          print, '%RE: Error reading dataset header - dataset= ',strtrim(data,2)
        endif else begin
          if (pfftyp eq 2) then begin
;           if the number of blocks is not 1 then skip
            if iextra(0) eq 1 then $
              readuf1, array, data, fileid=fid, pfftyp = pfftyp, $
              dlength = dlength, tlabel= tlabel, clabel=clabel, $
              xlabel = xlabel, ylabel = ylabel, aptype = aptype, $
              namefile = filename, stringlen= stringlen, iextra = iextra, $
              /skip, /wdf else begin
              print, '%RE: Number of blocks must be 1 for this procedure'
              array = 0
            endelse
            if n_elements (array) lt 4 then $
              print, '%RE: Error reading dataset ', strtrim(data,2) else begin
              nread = nread+1
              wdfcomment(wdf-1)= clabel
              wdffile(wdf-1) = filename
              wdfxlabel(wdf-1) = xlabel
              wdfylabel(wdf-1) = ylabel
              if keyword_set (pmes) then begin
                np = iextra[1]
                okpm = where (array[2:*] lt 5.0e29 and array[2:*] gt -5.0e29, nok)
                if nok eq np then put_wdfarray, array, wdf, xydata =0, /no_copy $
                  else begin
                  if nok gt 0  then begin
                    pmes_x = (array[0]+array[1]*findgen(np))[okpm]
                    pmes_y = (array[2:*])[okpm]
                    if nok lt 3 then begin
                      pmes_x= ([pmes_x, pmes_x[nok-1]+1 +findgen(2)])[0:2]
                      pmes_y= ([pmes_y, 0, 0])[0:2]
                    endif
                    put_wdfarray, [pmes_x, pmes_y], wdf, xydata =1, /no_copy
                  endif else begin
                    print,  '%RE: Error reading dataset ', strtrim(data,2), $
                      ' - No valid PMES data'
                    delwdf, wdf
                    nread = nread-1
                  endelse
                endelse
              endif else put_wdfarray, array, wdf, xydata =0, /no_copy

            endelse
          endif else if pfftyp eq 7 then begin
            readngd, array, data, fileid=fid, pfftyp = pfftyp, $
              dlength = dlength, tlabel= tlabel, clabel=clabel, $
              namefile = filename, stringlen= stringlen, iextra = iextra, $
              /skip 
            if array.ndim ne 1 then begin
              print,'%RE: Error reading dataset ', strtrim(data,2)
              print,'     Must be 1 dimensional data not ',strtrim(array.ndim,2)
              return
            endif
            if array.spare(1) eq 1 then begin
              xarr = array.x1
              xlabel = array.x1lab
            endif else if array.spare(1) eq 2 then begin
              xarr = array.x2
              xlabel = array.x2lab
            endif else if array.spare(1) eq 3 then begin 
              xarr = array.x3
              xlabel = array.x3lab
            endif else begin
              xarr = array.x1
              xlabel = array.x1lab
            endelse

            if array.nvect gt 1 then lab1 = array.v1lab + '-' + clabel $
              else lab1 = clabel

            nread = nread+1
            if keyword_set (pmes) then begin
                xarr_orig = xarr
                np = n_elements(xarr)
                okpm = where (array.v1 lt 5.0e29 and array.v1 gt -5.0e29, nok)
                if nok lt np then begin
                  if nok gt 0  then begin
                    xarr = xarr[okpm]
                    array.v1 = (array.v1)[okpm]
                    if nok lt 3 then begin
                      pmes_x= ([xarr, xarr[nok-1]+1 +findgen(2)])[0:2]
                      pmes_y= ([array.v1, 0, 0])[0:2]
                    endif
                  endif else begin
                    print,  '%RE: Error reading dataset ', strtrim(data,2), $
                      ' - No valid PMES data'
                    delwdf, wdf
                    nread = nread-1
                    xarr =0
                  endelse
                endif
            endif
            if n_elements(xarr) gt 1 then $
              put_wdfarray, [xarr, array.v1], wdf, /xydata, /no_copy, $
                clab = lab1, xlab = xlabel, ylabel = array.v1lab, file = filename
            
            if array.nvect gt 1 then begin
              xarr = xarr_orig
              nn = n_elements(wdfar)
              wdfar = [wdfar, wdfar(nn-1)+1+indgen(array.nvect)]
              i = i+1
              wdf = wdfar(i) < maxwdfarray
              put_wdfarray, [xarr, array.v2], wdf, /xydata, /no_copy, $
                clab = array.v2lab + '-' + clabel, xlab = xlabel, $
                ylabel = array.v2lab, file = filename
              if array.nvect gt 2 then begin
                i = i+1
                wdf = wdfar(i) < maxwdfarray
                put_wdfarray, [xarr, array.v3], wdf, /xydata, /no_copy, $
                  clab = array.v3lab + '-' + clabel, xlab = xlabel, $
                  ylabel = array.v3lab, file = filename
              endif
            endif
          endif else if pfftyp eq 3 then begin
            readnf3, array, data, fileid=fid, pfftyp = pfftyp, $
              dlength = dlength, tlabel= tlabel, clabel=clabel, $
              namefile = filename, stringlen= stringlen, iextra = iextra, $
              xarray = xx,  blabel = blabel, xlabel = xlabel,  $
              /skip, /wdfarray
            put_wdfarray, [xx, array], wdf, /xydata, /no_copy, $
             clab =  clabel, xlab = xlabel, $
             ylabel = blabel , file = filename
            nread = nread+1

          endif else begin
            readpff, pid, image, space, data, blo, blabel = blabel, $
              atlabel = atlab, xlabel = xlabel, file = file, clabel = clabel, $
              pff= pfftyp
            b = size(image)
            if b(0) ne 1 then $
              print, 'Array is not an wdf array - please use readstr,' $
                + ' dataset =', data  else begin
              npm = b(1)-1
              if pfftyp eq 1 then begin
; if uniform, store as uniform
                dt = (space(npm) -space(0))/npm
                space = space(0)
                image = [space, dt, image]
                put_wdfarray, image, wdf, clabel = clabel, file= file, $
                   xlabel = xlabel, ylabel= blabel, xydata = 0
              endif else begin
                if n_elements(xydatain)  eq 0 then xydat= 1 else xydat = xydatain
                if pfftyp eq 7 then blabel = atlab(0)
                if xydat eq 0 then begin
                  i2w, space, image, wdf, clabel = clabel, file= file, $
                      xlabel = xlabel, ylabel= blabel, xydata = 0
                endif else begin
                  image= [space(0:npm), image(0:npm)]
                  put_wdfarray, image, wdf, clabel = clabel, file= file, $
                     xlabel = xlabel, ylabel= blabel, xydata = 1
                endelse
              endelse
              nread = nread+1
            endelse
          endelse
        endelse
      endfor
      narray = nread
      return
      end
