; ------------------------------------------------------------------------ 
; $Id: readhdr.pro,v 1.4 2010/04/13 18:29:49 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
 Pro readhdr, dataset, ier, fileid = fidin,pfftype = pfftype, aptype = aptype, $
         version = version, dlength = dlength, resint = resint, $
         tlabel = tlabel, ntype  = ntype, iextra = iextra, nostrip = nostrip, $
         clabel = clabel, ncomment = ncomment, stringlen= stringlen, $
         namefile = filename, precision = precision, quiet = quiet
;
;+
;
;  Reads the header of a pff dataset and prints the parameters.
;  Procedure options are explained below.
;
;  Limit to the number of characters is set by work arrays in pff_slgethdr.f
;
;  WARNING: Because dataset will be modified, do not use a loop index
;                 for dataset.
;   
;  Format:  READHDR, dataset, ier, fileid = fid, 
;             pfftype = pfftype, aptype= aptype,
;             version = version, dlength = dlength, resint = resint, 
;             tlabel = tlabel, ntype = ntype, extraint = extraint, 
;             clabel = clabel, ncomment = ncomment, namefile = filename,  
;             stringlen= stringlen, nostrip = nostrip, 
;             precision = precision, quiet = quiet
;
;  where:
;    dataset - indicates a PFF dataset number 
;              or is a search string for dataset comments.  
;              Dataset may also be a string or string variable of two or more 
;              characters or a single character.  See GET_DSET for a better
;              description of the string options.
;              If dataset is missing or zero, then it is set to the 
;              current dataset.
;     Caution: Because dataset will be modified, do not use a loop index
;                 for dataset.
;    ier     - Error indicator
;
;  Keywords:
;    fileid  - File id of the PFF file containing the dataset
;            - Default is the current file
;    pfftype - PFF dataset type
;            1-scalar data on a uniform 3-D grid (UF3),
;            2-scalar data on a uniform 1-D grid (similar to WDF data) (UF1),
;            3-scalar data on a non-uniform 3-D grid (NF3),
;            4-vector data on a non-uniform grid (NV3),
;            5-m x n vertex data - m-dimensional vertices with n-attributes 
;                per vertex (VTX),(m, n, nvert)
;            6-integer list and a two float lists (IFL) (nfl, nfa, ni),
;            7-n-dimensional vectors on a region of m-dimensional space (NGD)
;                (m, n, nvert),
;            8-nonuniform 3D grid (NG3).
;            9-scalar data on a non-uniform 3-D grid (NI3),
;    aptype  - Application dataset type
;    version - Dataset version number
;    dlength - Dataset length in 16 bit words
;    resint  - Number of non-default reserved integers
;    tlabel  - Dataset type label - more than 2 spaces will be replaced 
;              by ' - ' unless NOSTRIP is set 
;    ntype   - Number of characters in the type label
;    clabel  - Dataset comment label - more than 2 spaces will be replaced 
;              by ' - ' unless NOSTRIP is set
;    ncomment- Number of characters in the comment label
;    namefile- Up to 255 characters describing the file
;    iextra  - Array of extra integers describing the size of the dataset
;               For UF3 or NF3 or NI3 or NV3 or NG3 data
;                 iextra(0) = number of blocks
;                 iextra(1) = number of X-values in data
;                 iextra(2) = number of Y-values in data
;                 iextra(3) = number of Z-values in data
;                 iextra(4) = number of words in field or each vector
;                 iextra(5) = number of spare words in each block
;               For UF1 data
;                 iextra(0) = number of blocks
;                 iextra(1) = number of X-values in first block
;                 iextra(2) = number of words read (header and length info)
;                 iextra(3) = number of spare words in each block
;               For Vertex data
;                 iextra(0) = vertex dimensions
;                 iextra(1) = number of attributes
;                 iextra(2) = number of vertices
;                 iextra(3) = number of spare words in each block
;               For IFL data
;                 iextra(0) = Float array flag
;                 iextra(1) = Length of Float-List
;                 iextra(2) = Length of Integer array
;                 iextra(3) = Length of Float array
;               For NGD data
;                 iextra(0) = spatial dimensions (m)
;                 iextra(1) = number of vector dimensions
;                 iextra(2:m+1) = number of grid points for each
;                                 dimension is space
;                 iextra(m+2) = number of spare words in each block
;
;  stringlen - Currently ignored
;              Maximum of 256 characters for type label
;              Maximum of 2048 characters for comment label
;              Maximum of 192 characters for filename
;                ( Values are stored in PFF_STRINGS.COM )
;                ( Parameters are set in PFF_SLGETHDR by Paul Mix (505)845-7493)
;
;    nostrip - If set, the clabel and tlabel will be returned as read.
;              Otherwise, multiple spaces will be replaced with ' - '
;    precision - PFF dataset precision.  PRECISION Values:
;               0 - Data stored as a 2-byte integer with offset and scale factor
;               1 - Data stored as a 4-byte real
;
;    quiet   - If present and not zero, do not print any data
;              This is ignored if the dataset comment or dataset type labels 
;              are larger than the available buffer.
;  Examples:
;    Read dataset 34 header in file id 4 and store the dataset comment in com
;      readhdr, 34, fil=4, c = com
;    Read header of the next dataset of the current file
;      readhdr
;-
;
@pff_strings.com
      on_error, 1
      common pffcom, initpff, pff_exe
      common readhdr_com, pfftyplabels
      if (n_elements(initpff) eq 0) then initpff = 0
      ier = -98
      if initpff eq 0 then begin
        print, '% READHDR: No PFF files are open'
        return
      endif
      if n_elements (fidin) gt 0 then fid = long(fidin[0]) else fid= 0L
      n = ndspff(fid)
      If n eq 0  then begin
        print, '% READHDR: No PFF datasets in this file'
        ier = -99
         return
      endif
      if n_elements(pfftyplabels) eq 0 then begin
        pfftyplabels= make_array(9, /string)
        pfftyplabels(0) = 'Uniform Scalar 3-D (UF3)'
        pfftyplabels(1) = 'Uniform Scalar 1-D (UF1)'
        pfftyplabels(2) = 'Non-Uniform Scalar Data (NF3)'
        pfftyplabels(3) = 'Non-Uniform Vector Data (NV3)'
        pfftyplabels(4) = 'Vertex Data (Particles) (VTX)'
        pfftyplabels(5) = 'Integer-Float List (IFL)'
        pfftyplabels(6) = 'N-Dim Vector, M-Dim Space (NGD)'
        pfftyplabels(7) = 'Non-Uniform 3-D Grid (NG3)'
        pfftyplabels(8) = 'Non-Uniform Integer Data (NI3)'
      endif
;  initialize parameters
      ier = 0L
      pfftype = 0L
      aptype = 0L
      version = 0L
      dlength = 0L
      resint = 0L
      precision = 0L
      lstrc = 256L 
      clabel = make_array(lstrc, /byte, value = 32b)
      lstrt = 64L
      tlabel = make_array(lstrt, /byte, value = 32b)
      lstrf = 256L
      filename = make_array(lstrf, /byte, value = 32b)
      ncomment = 0L
      ntype =0L
;
      if n_elements (dataset) eq 0 then dataset = 0L
;
      pid = fid
      blo = 1L
      nmap = 0L
;
      iextra = make_array(24, /long)
      nmax = 1L
      get_dset_string, dataset,fid,  map, nmap, nmax, ier
      if ier ne 0 then begin
        print, '% READHDR: Please change the PFF dataset specification'
        return
      endif
;
      direct = map(0)
      
;
      if n_elements (direct) gt 0 then get_pffinteger,direct else direct= 0L
;
;     Limit to the number of characters is set by work arrays in pff_slgethdr.f
      if initpff eq 1 then begin
; HP
        junk = call_external(PFF_EXE,'pff_slgethdr', direct, pid, ier,pfftype, $
          aptype, version, dlength, resint, tlabel,ntype, clabel,ncomment, $
          filename, precision, iextra, lstrt, lstrc, lstrf)
        if ntype gt lstrt or ncomment gt lstrc then begin
          lstrt = ((ntype+1)/2)*2L < max_tlabel
          lstrc = ((ncomment+1)/2)*2L < max_clabel
          clabel = make_array(lstrc, /byte, value = 32b)
          tlabel = make_array(lstrt, /byte, value = 32b)
        junk = call_external(PFF_EXE,'pff_slgethdr', tlabel, $
          clabel, lstrt, lstrc )
        endif
            
      endif else if initpff eq 2 then begin
; Sun
        junk = call_external(PFF_EXE,'pff_slgethdr_',direct, pid,ier, pfftype, $
          aptype, version, dlength, resint, tlabel,ntype, clabel,ncomment, $
          filename, precision, iextra, lstrt, lstrc, lstrf)
        if ntype gt lstrt or ncomment gt lstrc then begin
          lstrt = ((ntype+1)/2)*2L < max_tlabel
          lstrc = ((ncomment+1)/2)*2L < max_clabel
          clabel = make_array(lstrc, /byte, value = 32b)
          tlabel = make_array(lstrt, /byte, value = 32b)
        junk = call_external(PFF_EXE,'pff_slgethdr_', tlabel, $
          clabel, lstrt, lstrc)
        endif
      endif else if initpff eq 4 then begin
; WIN
       junk = call_external(PFF_EXE,'_PFF_SLGETHDR@8',direct, pid,ier, pfftype, $
          aptype, version, dlength, resint, tlabel,ntype, clabel,ncomment, $
          filename, precision, iextra, lstrt, lstrc, lstrf)
        if ntype gt lstrt or ncomment gt lstrc then begin
          lstrt = ((ntype+1)/2)*2L < max_tlabel
          lstrc = ((ncomment+1)/2)*2L < max_clabel
          clabel = make_array(lstrc, /byte, value = 32b)
          tlabel = make_array(lstrt, /byte, value = 32b)
        junk = call_external(PFF_EXE,'_PFF_SLGETHDR@8', tlabel, $
          clabel, lstrt, lstrc)
        endif
      endif
;
      if ier eq 0 then dataset = direct else begin
        if ier eq -2 then showpff
        return
      endelse
;
;      tlabel = strtrim(tlabel, 2)
;      clabel = strtrim(clabel, 2)
      tlabel = strmid(tlabel, 0, ntype)
      clabel = strmid(clabel, 0, ncomment)
      if not keyword_set(nostrip) then begin
        tlabel = strip_spaces(tlabel)
        clabel = strip_spaces(clabel)
      endif
      filename = strtrim(filename, 2)
      if ntype gt lstrt then print, ' Dataset type label contained ' + $
        string(strtrim(ntype, 2)) +' characters - only ' + $
        string(strtrim(lstr, 2)) +' characters returned'
      if ncomment gt lstrc then print, ' Dataset comment contained ' + $
        string(strtrim(ncomment, 2)) +' characters - only ' + $
        string(strtrim(lstr, 2)) +' characters returned'
;
      if n_elements (quiet) eq 0 then quiet =0
      if quiet ne 0 then return
;
      print, '   Header information for dataset ', strtrim(string(direct), 2)
      if pid eq 0 then print, '      from the current PFF file' else $
         print, '      from PFF file with file id ', strtrim(string(pid), 2)
      print, '    File Name: ', filename
      print, '    Comment label: ', clabel      
      print, '    Type label: ', tlabel      
      print, ' PFF dataset type - ', strtrim (string(pfftype),2), ' or ', $
            pfftyplabels(pfftype-1)
      print, ' PFF application dataset type - ',strtrim (string(aptype),2)
      print, ' PFF version number - ',strtrim (string(version), 2)
      print, ' PFF dataset length - ',strtrim (string(dlength), 2), $
             ' 16 bit words'
      datasize = (precision eq 0) ? ' 16 bit words' : ' 32 bit words'
      Print, '     Floating arrays are stored as', datasize
      print, ' Number of Non-Default reserved integers - ', $
                strtrim (string(resint), 2)
      if pfftype eq 1 or pfftype eq 3 or pfftype eq 4 $
        or pfftype eq 8 or pfftype eq 9 then begin
;       UF3, NF3, NV3, NG3 or NI3 
        print, ' Number of Blocks - ', strtrim(iextra(0), 2)
        print, ' Number of X-values - ', strtrim(iextra(1), 2)
        print, ' Number of Y-values - ', strtrim(iextra(2), 2)
        print, ' Number of Z-values - ', strtrim(iextra(3), 2)
        print, ' Number of Field data values - ', $
                 strtrim(iextra(4), 2)
        print, ' Number of spare integers - ', strtrim(iextra(5), 2)
      endif else if pfftype eq 5 then begin
;       Vertex Data
        print, ' Vertex Dimension - ', strtrim(iextra(0), 2)
        print, ' Number of Attributes - ', strtrim(iextra(1), 2)
        print, ' Number of Vertices - ', strtrim(iextra(2), 2)
        print, ' Number of spare integers - ', strtrim(iextra(3), 2)
      endif else if pfftype eq 2 then begin
;       UF1 data
        print, ' Number of Blocks - ', strtrim(iextra(0), 2)
        print, ' Number of X/Y-values - ', strtrim(iextra(1), 2)
        print, ' Number of spare integers - ', strtrim(iextra(3), 2)
      endif else if pfftype eq 7 then begin
;       NGD data
        print, ' Number of spatial dimensions - ', strtrim(iextra(0), 2)
        print, ' Number of vector dimensions - ', strtrim(iextra(1), 2)
        for i = 0, iextra(0)-1 do print, ' Number of values in dimension ', $
          strtrim(i+1, 2), ' - ', strtrim(iextra(2+i), 2)
        print, ' Number of spare integers - ', strtrim(iextra(i+2), 2)
      endif else if pfftype eq 6 then begin
;       IFL data
        if iextra(0) eq 0 then $
          print,' Number of values in the float array - 0' $
        else $
          print, ' Number of values in the float array is - ', $
           strtrim(iextra(3), 2)
        print,' Number of values in the float list is - ', strtrim(iextra(1), 2)
        print,' Number of values in the integer list is - ', $
          strtrim(iextra(2), 2)
      endif 
;
      return
      end
