; ------------------------------------------------------------------------ 
; $Id: readnf3.pro,v 1.10 2010/03/04 14:54:26 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
 Pro readnf3, farray, datasetin, fileid = fid, stringlen= stringlen, $
             array = array, xarray = x, yarray = y, $
             zarray= z, xlabel = xlabel, ylabel = ylabel, $
             zlabel = zlabel, blabel = blabel, tlabel= tlabel, $
             clabel = clabel, namefile = filename, pfftyp = pfftyp,  $
             locx = locx, locv = locfa, ignore = ignore, $
             spare = ispare, apptyp = tapp, skip = skip, $
             dlength = dlength, iextra = iextra, wdfarray = wdfarray

;
;+
;
;  Reads a field or charge density into an IDL structure.
;  Procedure options are explained below.
;  Current version limited to 20 blocks by the dimensions in pff_slrnf3
;  PFF dataset type must be 3: NF3 = Scalar data on a nonuniform grid
;
;  Format:  READNF3, STRUCT  [, dataset] [, FILEID = fileid]
;                    [, STRINGLEN = stringlen] [, ARRAY = array]
;                    [, XARRAY= xarray] [, YARRAY= yarray]
;                    [, ZARRAY= zarray] [APPTYP = apptyp]
;                    [, LOCX = locx] [, LOCV = locv]
;                    [, CLABEL = clabel] [, TLABEL = tlabel]
;                    [, XLABEL = xlabel] [, YLABEL = ylabel] 
;                    [, ZLABEL = zlabel] [, BLABEL = blabel]
;                    [, NAMEFILE = namefile] [, PFFTYP = pfftyp]
;                    [, SPARE = spare] [, IGNORE = ignore]
;                    [, WDFARRAY = wdfarray]
;  where:
;    STRUCT  - IDL structure name
;              If not a valid variable, data is stored in GET_MAXSTR()-1
;    dataset - indicates a PFF dataset number 
;              or is a search string for dataset comments.  
;              If dataset is missing or zero, then it is set to the 
;              current dataset.
;        Note: If dataset is a string, a leading `^' indicates that 
;              the string begins at the start of the dataset comment.
;              String must select a single dataset.
;    fileid -  file id of the PFF file containing the data
;    ignore -    If set, read the P2D data as written.
;                This only applies to P2D data when the third spare word
;                has a value of 1, 2, or 3.
;                Data from the poison solver in QS is stored as X-Y data.
;                The data plane may be in any orientation and the third spare
;                word indicates the true normal direction (1=X, 2=Y, and 3=Z)
;                If ignore is set, Z will be the default normal direction.
;                P2D data is also reduced in size to eliminate the ghost
;                or buffer cells.
;         NOTE:  To keep all data and exactly as written into the NF3 dataset,
;                set ignore to negative. 
;    stringlen - Currently Ignored
;                Maximum number of characters in the spatial labels and the
;                field labels.  Maximum of 64 characters each for each block. 
;    array     - If array is present and not zero, then STRUCT will be
;                the array corresponding to block = array.
;                If array is less than 1 then array = 1
;                If array is greater than the number of blocks, array = nblock
;                If array is present and zero, it will contain the vector array
;    xarray    - Values for the first dimension
;    yarray    - Values for the second dimension
;    zarray    - Values for the third dimension
;    apptyp    - Application dataset type
;    SPARE     - Spare (5, nblock) spare array for each block
;    WDFARRAY  - If set, return ordinate data in STRUCT and abscissa data 
;                in XARRAY  
;                If Yarray and Zarray have more than 1 element/block, then 
;                  print a warning if wdfarray is positive
;                Valid data is only STRUCT, Xarray, Xlabel, Clabel, Tlabel, 
;                  Blabel, NameFile, PFFTYP, APPTYP
;                Note: Blabel is the y Axis label
;  x,y,z,blabel- Labels for x, y, z and block
;
;  Structure Labels
;    type  - 1
;    ndim  - number of spatial dimensions
;    nvect - number of vector dimensions
;    block - number of blocks 
;    two   - two dimensional missing dimension
;            this is 0 for 3d data, 1 for yz data, 2 for zx data, 3 for xy data
;    slice - value of the missing dimension, if known. Default is zero.
;    size  - array of the size of the spatial dimensions
;            size = size(ndim, block)
;    range - (2, ndim, nblock+1) two dimensional array of the max and min for
;            each block.  Block (0) has overall values.
;    order - (ndim, nblock) Ordering of blocks in each dimension by initial 
;            value of the block, in units of block -1
;    locx  - array of the location of the first point for each block
;            spatial array- locx = locx(ndim, block+1)
;            (first spatial dimension for block m is x and is defined by
;             x = x1(locx(0, m-1): locx(0, m)-1) )
;    locv  - array of the location of the first point for each block
;            vector array- locv = locv(block+1)
;            (first vector array for block m is v and is defined by
;            v = reform (v1(locv(m-1):locv(m)-1),size(*,m-1))
;    x1, x2, ...x(ndim) - spatial arrays
;    v1, v2, ...v(nvect) - vector arrays
;    x1lab, x2lab, ...x(ndim)lab - spatial labels
;    v1lab, v2lab, ...v(nvect)lab - vector labels, Vector label in the case of
;                  NGD data and block label in the case of vector and 3d data
;    tlab  - type label
;    clab  - dataset comment
;    file  - file data read from
;    spare - integer array (5)
;  Note: If block > 1 spatial and vector arrays are reformed to linear 
;        arrays and stored in the respective arrays.
;  Examples:
;    Read dataset 34 in file id 4 into structure ch5
;      readnf3, ch5, 34, fil=4
;    Read current dataset of the current file into structure efldstart
;      readnf3, efldstart
;    Read the current dataset into an array image, and put the x and y 
;      coordinates into xar and yar.
;      readnf3, image, /arr, xar = xar, yar = yar
;-
;
      on_error, 1
;
      common pffcom, initpff, pff_exe
@pff_strings.com
;
;     check to see if pff has been initialized
;
      if n_elements(initpff) eq 0 then initpff = 0
      if (initpff eq 0) then begin
        print, '% READNF3: File must be opened before reading data'
        return
      endif
;
;     check on the number of parameters
;
;debug
; tstart = systime(1)

      nparams =  n_params()
      slice = 0.0
      if (nparams lt 1 ) then begin
        print,'% Format: READNF3, structure  [, dataset] [, FILEID = fileid]'
        print,'        [, STRINGLEN = stringlen]  [, ARRAY = array]'
        print,'        [, XARRAY= xarray] [, YARRAY= yarray]'
        print,'        [, ZARRAY= zarray] [, ZLABEL = zlabel]'
        print,'        [, LOCX = LOCX] [, LOCV = LOCV] '
        print,'        [, CLABEL = clabel] [, TLABEL = tlabel]'
        print,'        [, XLABEL = xlabel] [, YLABEL = ylabel]'
        print,'        [, NAMEFILE = namefile] [, PFFTYP = pfftyp]'
        return
      endif
      lcstructure = 0
      farray = 0.0
      if not arg_present(farray) then  begin
        print, '% READNF3: Warning data cannot be returned in variable'
        if keyword_set(array) then begin
          print, '           Unable to return array'
          return
        endif else $
          print, '           Data will be stored in structure '+strtrim (get_maxstr()-1, 2)
        lcstructure = 1
      endif

      if n_elements (fid) eq 0 then fid = 0L
      pid = long(fid)
      if n_elements (datasetin) eq 0 then dataset = 0L else $
        dataset = datasetin(0)
      if keyword_set(skip) then begin
        if n_elements(dlength) ne 1 or n_elements (iextra) lt 8 or $
        n_elements(pfftyp) ne 1 then message, 'Error with keyword SKIP'
      endif else begin
        nmap = 0L
        nmax = 1L
        get_dset_string, dataset,pid,  map, nmap, nmax, ier
        if ier ne 0 then message, ' Please change the dataset specification'
        if nmap gt 1 then begin
          Print,'% READNF3: Please change the dataset specification to get a single dataset.'
          help, dataset  
          print, 'Datasets: '
          for i = 0, nmap-1 do print, map[i]
          return
        endif
        dataset = map[0]
        readhdr,dataset,ier,fileid=pid, pfftyp = pfftyp, $
           dlength = dlength, tlabel= tlabel, clabel=clabel, $
           namefile = filename, iextra = iextra, /quiet
        if ier ne 0 or n_elements(pfftyp) eq 0 then begin
          print, ' Error reading dataset header'
          return
        endif
      endelse
      if (pfftyp  ne 3 ) then begin
        print, '% READNF3: Dataset type '+strtrim(pfftyp, 2) + $
            'cannot be read with READNF3 (TYPE=3)'
        return
      endif

; READNF3

;               For UF3 or NF3 or NI3 or NV3 or NG3 data
;                 iextra(0) = number of blocks
;                 iextra(1) = number of X-values in data
;                 iextra(2) = number of Y-values in data
;                 iextra(3) = number of Z-values in data
;                 iextra(4) = number of words read (header and length info)
;                 iextra(5) = number of spare words in each block
;
      iextra  = long(iextra)
      mblo = iextra(0) < max_block
      mdim = iextra(4)
      farray = make_array(mdim)
      nx = make_array(mblo, /long)
      ny = make_array(mblo, /long)
      nz = make_array(mblo, /long)
      mx = iextra(1)
      my = iextra(2)
      mz = iextra(3)
      x = make_array(mx)
      y = make_array(my)
      z = make_array(mz)
      
      locx = make_array(mblo+1, /LONG)
      locy = make_array(mblo+1, /LONG)
      locz = make_array(mblo+1, /LONG)
      locfa = make_array(mblo+1, /LONG)
      lenfa = make_array(mblo, /LONG)

      tapp = 0L
      nblo = 0L
      ier = 0L
      ispare = make_array(5, mblo, /long)

      stringlen = max_str_len
      lxstr = stringlen
      xlabel = make_array(lxstr*mblo, /byte, value = 32B)
      ylabel = make_array(lxstr*mblo, /byte, value = 32B)
      zlabel = make_array(lxstr*mblo, /byte, value = 32B)
      blabel = make_array(lxstr*mblo, /byte, value = 32B)
      llstr = 40L
      clab = make_array(llstr, /byte, value = 32B)
      tlab = make_array(llstr, /byte, value = 32B)

      xoff10 = make_array(mblo, value = 0L, /long)
      foff10 = make_array(mblo, value = 0L, /long)
;debug
;  tbr = systime(1)
;  print, 'before read data', tbr -tstart, tbr-tnow


      if  initpff eq 1 then begin
; HP
        junk = call_external(PFF_EXE,'pff_slrnf3', pid, dataset, mblo,  $
                mx, my, mz,  mdim, tapp, tlab, clab, nblo, nx, ny, nz, $
                ispare, x, y, z, xoff10, $
                xlabel, ylabel, zlabel, blabel, locx, locy, locz, locfa, $
                lenfa, farray, foff10, ier, lxstr, llstr)

      endif else if initpff eq 2 then begin
; Sun
        junk = call_external(PFF_EXE,'pff_slrnf3_', pid, dataset, mblo, $
                mx, my, mz,  mdim, tapp, tlab, clab, nblo, nx, ny, nz, $
                ispare, x, y, z, xoff10, $
                xlabel, ylabel, zlabel, blabel, locx, locy, locz, locfa, $
                lenfa, farray, foff10, ier, lxstr, llstr)

      endif else if initpff eq 4 then begin
; WIN
        junk = call_external(PFF_EXE,'_PFF_SLRNF3@8', pid, dataset, mblo, $
                mx, my, mz,  mdim, tapp, tlab, clab, nblo, nx, ny, nz, $
                ispare, x, y, z, xoff10, $
                xlabel, ylabel, zlabel, blabel, locx, locy, locz, locfa, $
                lenfa, farray, foff10, ier, lxstr, llstr)
      endif
;debug
;  tar = systime(1)
;  print, 'after read data', tar -tstart, tar-tbr
      if ier ne 0 then begin
        print, 'READNF3: Error reading dataset, IER = ',strtrim(ier,2)
        farray = 0
        return
      endif
      if max(xoff10, min = mm) ne 0 or mm ne 0 then begin
        print,'WARNING - Power of 10 offset for the grid exceeds computer range'
        print, 'Grid offsets - ', xoff10
      endif
      if max(foff10, min = mm) ne 0 or mm ne 0 then begin
        print,'WARNING - Power of 10 offset for the data exceeds computer range'
        print, 'Data offsets - ', foff10
      endif
      if nblo gt mblo then begin
        Print, ' Problem with the number of blocks, contact Paul Mix.'
        farray = 0
        return
      endif
      if keyword_set (wdfarray) then begin
        xarray = x
        xlabel = strtrim(xlabel[0:lxstr-1], 2)
        blabel = strtrim(blabel[0:lxstr-1], 2)
        if (total(ny) +total(nz) gt mblo+mblo)  and wdfarray gt 0 then print, $
          '% READNF3: Data does not appear to be waveform data', ny, nz
        return
      endif
      locfa[mblo] = locfa[mblo-1]+lenfa[mblo-1]
      locx[mblo] = locx[mblo-1]+ nx[mblo-1]
      locy[mblo] = locy[mblo-1]+ ny[mblo-1]
      locz[mblo] = locz[mblo-1]+ nz[mblo-1]
      if locfa[0] ne 0 then locfa= locfa-locfa[0]
      if locx[0] ne 0 then locx= locx-locx[0]
      if locy[0] ne 0 then locy= locy-locy[0]
      if locz[0] ne 0 then locz= locz-locz[0]
      if keyword_set(ignore) then begin
        if ignore lt 0 then isp = 0 else $
          if (ispare[2] gt 0) && (ispare[2] le 3) then isp = 3 $
            else isp = ispare[2]
      endif else isp = ispare[2]
      if (isp gt 0 and isp le 3) then begin
;       this is normal coordinate direction for the data.
        nam = strmid(clabel,0, 4) 
        lnam = strlen(clabel)
        if isp eq 1 then begin
          if nam eq 'Epox' then begin
            clabel= 'Ej Field'+ strmid(clabel, 4, lnam)
            remove_extra, 1, 0, nblo, farray, x, y, locfa, locx, locy, $
              lenfa, nx, ny
          endif else if nam eq 'Epoy' then begin
            clabel='Ek Field'+ strmid(clabel, 4, lnam)
            remove_extra, 2, 0, nblo, farray, x, y, locfa, locx, locy, $
              lenfa, nx, ny
          endif
          temp = temporary(nz)
          nz = temporary(ny)
          ny = temporary(nx)
          nx = temporary(temp)
          temp = temporary(z)
          z = temporary(y)
          y = temporary(x)
          x = temporary(temp)
          temp = temporary(locz)
          locz = temporary(locy)
          locy = temporary(locx)
          locx = temporary(temp)
        endif else if isp eq 2 then begin
;         Ynormal need to do a transpose
          if nam eq 'Epox' then begin
            clabel= 'Ek Field'+ strmid(clabel, 4, lnam)
            remove_extra, 1, 1, nblo, farray, x, y, locfa, locx, locy, $
              lenfa, nx, ny
          endif else if nam eq 'Epoy' then begin
            clabel='Ei Field'+ strmid(clabel, 4, lnam)
            remove_extra, 2, 1, nblo, farray, x, y, locfa, locx, locy, $
              lenfa, nx, ny
          endif else begin
;           do the transpose
            for i = 0, nblo-1 do begin
              ib = locfa(i)
              ie = locfa(i+1)-1
              farray (ib:ie) = transpose( $
                reform (farray[ib:ie], nx(i), ny(i), /overwrite))
            endfor
          endelse
          temp = temporary(nz)
          nz = temporary(nx)
          nx = temporary(ny)
          ny = temporary(temp)
          temp = temporary(z)
          z = temporary(x)
          x = temporary(y)
          y = temporary(temp)
          temp = temporary(locz)
          locz = temporary(locx)
          locx = temporary(locy)
          locy = temporary(temp)
        endif else if isp eq 3 then begin
          if nam eq 'Epox' then begin
            clabel= 'Ei Field'+ strmid(clabel, 4, lnam)
            remove_extra, 1, 0, nblo, farray, x, y, locfa, locx, locy, $
              lenfa, nx, ny
          endif else if nam eq 'Epoy' then begin
            clabel='Ej Field'+ strmid(clabel, 4, lnam)
            remove_extra, 2, 0, nblo, farray, x, y, locfa, locx, locy, $
              lenfa, nx, ny
          endif
        endif
      endif
;
;    pfftype - PFF dataset type 3: NF3
;
;    If array is gt 0 return only the array for the proper block
     if keyword_set(array)  then begin
       array = array(0) > 1
       if array gt nblo then begin
         array = nblo
         print, '%READNF3 - Improper block number in ARRAY - Block ', $
                 strtrim(nblo, 2), ' data returned'
       endif
       arraym = array-1
;      Farray is correct if we only have 1 block
       if nblo gt 1 then farray = farray(locfa(arraym):locfa(array)-1)
       farray = reform(farray, nx(arraym), ny(arraym), nz(arraym), /overwrite)
       x= x(locx(arraym):locx(array)-1)
       y= y(locy(arraym):locy(array)-1)
       z= z(locz(arraym):locz(array)-1)
       xlabel = strtrim(xlabel(arraym*lxstr:array*lxstr-1), 2)
       ylabel = strtrim(ylabel(arraym*lxstr:array*lxstr-1), 2)
       zlabel = strtrim(zlabel(arraym*lxstr:array*lxstr-1), 2)
       blabel = strtrim(blabel(arraym*lxstr:array*lxstr-1), 2)
       return
     endif
     nxt = round (total (nx))
     nyt = round (total (ny))
     nzt = round (total (nz))
     if nxt  eq mblo then begin
       two = 1L
       slice = x(0)
     endif else if nyt  eq mblo then begin
       two = 2L
       slice = y(0)
     endif else if nzt  eq mblo then begin
       two = 3L
       slice = z(0)
     endif else begin
       two = 0L
       slice = 0.0
     endelse

     x1 = make_array(nxt)
     x2 = make_array(nyt)
     x3 = make_array(nzt)
     x4 = 0.0


     range = make_array(2, 3, mblo+1)
     for i = 0, mblo-1 do begin
       range (0, 0, i+1) = min(x(locx(i): locx(i+1)-1), max = mm)
       range (1, 0, i+1) = mm
       range (0, 1, i+1) = min(y(locy(i): locy(i+1)-1), max = mm)
       range (1, 1, i+1) = mm
       range (0, 2, i+1) = min(z(locz(i): locz(i+1)-1), max = mm)
       range (1, 2, i+1) = mm
     endfor
     locx = transpose(reform([locx, locy, locz], mblo+1, 3))
     range (0, 0, 0) = min(range(0, 0, 1:mblo))
     range (0, 1, 0) = min(range(0, 1, 1:mblo))
     range (0, 2, 0) = min(range(0, 2, 1:mblo))
     range (1, 0, 0) = max(range(1, 0, 1:mblo))
     range (1, 1, 0) = max(range(1, 1, 1:mblo))
     range (1, 2, 0) = max(range(1, 2, 1:mblo))

     ssize = make_array(3, mblo, /long)
     ssize(0, *)= nx
     ssize(1, *)= ny
     ssize(2, *)= nz

     x1lab = make_array(mblo, /string)
     x2lab = make_array(mblo, /string)
     x3lab = make_array(mblo, /string)
     x4lab = ''
     v1lab = make_array(mblo, /string)
     v2lab = ''
     v3lab = ''
     v4lab = ''
     v5lab = ''
     v6lab = ''
     v2= 0.0
     v3= 0.0
     v4= 0.0
     v5= 0.0
     v6= 0.0

     for i = 0, mblo-1 do begin
       x1lab[i] = strtrim(xlabel[i*lxstr:(i+1)*lxstr-1], 2)
       x2lab[i] = strtrim(ylabel[i*lxstr:(i+1)*lxstr-1], 2)
       x3lab[i] = strtrim(zlabel[i*lxstr:(i+1)*lxstr-1], 2)
       v1lab[i] = strtrim(blabel[i*lxstr:(i+1)*lxstr-1], 2)
     endfor
     if mblo gt 1 then begin
       order = make_array(3, mblo, /long)
       for i = 0, 2 do order (i, *) = sort(range(0,i, 1:mblo))
     endif else order = 1L

; If we were overly concerned we could reduce size of farray.
; This is probably not worth the time for most data
     farray = {type:1L, ndim:3L, nvect: 1L, block :mblo, two: two, $
              slice: slice, size: ssize, range: range, $
              order: order, locx:locx, locv: locfa, $
              x1:x, x2:y, x3:z, x4:x4,  $
              v1:farray, v2:v2, v3:v3, v4:v4, $
              v5:v5, v6:v6,x1lab : x1lab, x2lab :x2lab, $
              x3lab : x3lab, x4lab :x4lab,v1lab : v1lab, v2lab :v2lab, $
              v3lab : v3lab, v4lab :v4lab,v5lab : v5lab, v6lab :v6lab, $
              file : filename, spare : ispare, tlab : tlabel, clab:clabel }
     if lcstructure then i2s, farray, get_maxstr()-1, /no_copy
     end
