; ------------------------------------------------------------------------ 
; $Id: readngd.pro,v 1.7 2010/03/03 22:57:02 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
 Pro readngd, array, datasetin, fileid = fid, stringlen= stringlen, $
              tlabel= tlabel, clabel = clabel, $
             namefile = filename, pfftyp = pfftyp,  $
             skip = skip, dlength = dlength, iextra = iextra, $
             wdfarray = wdfarray, xzdata = xzdata

;
;+
;
;  Reads a field or charge density into an IDL structure.
;  Procedure options are explained below.
;  PFF dataset type must be 7 NGD = N-Dimensional Vector on 
;      M-spatial dimensions 
;  Note - TWO QUICK data is stored as 3D data
;         The spatial coordinates are assumed to be the two coordinates
;            following the normal direction in a right hand sense.
;            ie, If ispare(1) = 2 then the coordinates are ZX.
;  Note - If ispare(1) is 2 and two dimensions are passed 
;          or 
;         If ispare(1) is 2 and three dimensions are passed with the second
;            being degenerate 
;         then the field data will be transposed.
;  Format:  READNGD, structure  [, dataset] [, FILEID = fileid]
;                   [, WDFARRAY = wdfarray] [, STRINGLEN = stringlen]
;                   [, XZDATA = xzdata]
;  where:
;    structure - IDL structure name
;              If not a valid variable, data is stored in GET_MAXSTR()-1
;    dataset - indicates a PFF dataset number 
;              or is a search string for dataset comments.  
;              If dataset is missing or zero, then it is set to the 
;              current dataset.
;        Note: If dataset is a string, a leading `^' indicates that 
;              the string begins at the start of the dataset comment.
;    xzdata - If set, then no transpose will be done for ispare(1) =2
;             Recall from above that the data is assumed to be zx data if
;             the second spare value is 2.
;    fileid -  file id of the PFF file containing the data
;    stringlen - Currently Ignored
;                Maximum number of characters in the spatial labels and the
;                field labels.  Maximum of 64 bytes for each label.
;    wdfarray - If keyword is set (not equal to zero), return a wdf structure.
;
;  Structure Labels
;    type  - 1
;    ndim  - number of spatial dimensions
;    nvect - number of vector dimensions
;    block - number of blocks 
;    two   - two dimensional missing dimension
;            this is 0 for 3d data, 1 for yz data, 2 for zx data, 3 for xy data
;    slice - value of the missing dimension, if known. Default is zero.
;    size  - array of the size of the spatial dimensions
;            size = size(ndim, block)
;    range - (2, ndim, nblock+1) two dimensional array of the max and min for
;            each block.  Block (0) has overall values.
;    order - (ndim, nblock) Ordering of blocks in each dimension by initial 
;            value of the block, in units of block -1
;    locx  - array of the location of the first point for each block
;            spatial array- locx = locx(ndim, block+1)
;            (first spatial dimension for block m is x and is defined by
;             x = x1(locx(0, m-1): locx(0, m)-1) )
;    locv  - array of the location of the first point for each block
;            vector array- locv = locv(block+1)
;            (first vector array for block m is v and is defined by
;            v = reform (v1(locv(m-1):locv(m)-1),size(*,m-1))
;    x1, x2, ...x(ndim) - spatial arrays
;    v1, v2, ...v(nvect) - vector arrays
;    x1lab, x2lab, ...x(ndim)lab - spatial labels
;    v1lab, v2lab, ...v(nvect)lab - vector labels, Vector label in the case of
;                  NGD data and block label in the case of vector and 3d data
;    tlab  - type label
;    clab  - dataset comment
;    file  - file data read from
;    spare - integer array (5)
;  Note: If block > 1 spatial and vector arrays are reformed to linear 
;        arrays and stored in the respective arrays.
;  Examples:
;    Read dataset 34 in file id 4 into structure ch5
;      readfld, ch5, 34, fil=4
;    Read next dataset of the current file into structure efldstart
;      readfld, efldstart
;-
;
      on_error, 1
;
      common pffcom, initpff, pff_exe
;
@pff_strings.com
      stringlen = max_str_len
;
;     check to see if pff has been initialized
;
      if n_elements(initpff) eq 0 then initpff = 0
      if (initpff eq 0) then begin
        print, '% READNGD: File must be opened before reading data'
        return
      endif
;
;     check on the number of parameters
;
      nparams =  n_params()
      slice = 0.0
      npar = 7
      if (nparams lt 1 ) then begin
        print,'% Format: READNGD, structure  [, dataset] [, FILEID = fileid]'
        print,'             [, STRINGLEN = stringlen]'
        return
      endif

      lcstructure = 0
      if not arg_present(array) then  begin
        print, '% READNGD: Warning data cannot be returned in variable'
        print, '           Data will be stored in structure '+strtrim (get_maxstr()-1, 2)
        lcstructure = 1
      endif
      array = 0.0

      if n_elements (fid) eq 0 then fid = 0L
      pid = long(fid)
      if n_elements (datasetin) eq 0 then dataset = 0L else $
        dataset = datasetin(0)
      if keyword_set(skip) then begin
        if n_elements(dlength) ne 1 or n_elements (iextra) lt 8 or $
        n_elements (pfftyp) ne 1 then message, 'Error with keyword SKIP'
      endif else begin
        nmap = 0L
        nmax = 1L
        get_dset_string, dataset,pid,  map, nmap, nmax, ier
        if ier ne 0 then message, ' Please change the dataset specification'
;
        if nmap gt 1 then begin
         Print,'% READNGD: Please change the dataset specification to get a single dataset.'
          help, dataset
          print, 'Datasets: '
          for i = 0, nmap-1 do print, map[i]
          return
        endif

        dataset = map(0)
;
        readhdr,dataset,ier,fileid=pid, pfftyp = pfftyp, $
           dlength = dlength, tlabel= tlabel, clabel=clabel, $
           namefile = filename, iextra = iextra, /quiet
;
        if ier ne 0 or n_elements(pfftyp) eq 0 then begin
          print, ' Error reading dataset header'
          return
        endif
      endelse
      if (pfftyp  ne 7 ) then begin
        print, '% READNGD: Dataset type '+strtrim(pfftyp, 2) + $
            ' cannot be read with READNGD'
        return
      endif
; READNGD
;               For NGD data
;                 iextra(0) = spatial dimensions (m)
;                 iextra(1) = number of vector dimensions
;                 iextra(2:m+1) = number of grid points for each
;                                 dimension is space
;                 iextra(m+2) = number of spare words in each block

      mdim = long (iextra(0)) < max_block
      ndim = long (iextra(1)) < max_block
      ssize = long (iextra(2: mdim+1))
      
      xdim = ssize(0)
      if mdim gt 1 then for i = 1, mdim-1 do xdim = xdim+ssize(i)
      vdim = ssize(0)
      if mdim gt 1 then for i = 1, mdim-1 do vdim = vdim*ssize(i)
      sprdim = long (iextra(mdim +2))
      ier = 0L
      app = 0L
      spare = make_array(sprdim>5, /long)
      lxstr = stringlen
      xlabel = make_array(lxstr*mdim, /byte, value = 32B)
      lvstr = stringlen
      vlabel = make_array(lvstr*ndim, /byte, value = 32B)
      llstr = 40L
      clab = make_array(llstr, /byte, value = 32B)
      tlab = make_array(llstr, /byte, value = 32B)
      m = 0L
      n = 0L
      nx = ssize
      nvp = 0L
      nspare = 0L
      ioff = make_array(max([mdim, ndim]),/long)
      xarr = make_array(xdim)
      if ndim eq 0 then begin
        print, 'Cannot read this dataset with no vector components.'
        return
      endif
      varr = make_array(vdim,ndim)
      voff10 = 0L
      xoff10 = 0L
      if  initpff eq 1 then begin
; HP
        junk = call_external(PFF_EXE,'pff_slrngd', pid, dataset, mdim, ndim, $
                xdim, vdim, sprdim, app, tlab, clab, m, n, nx, nvp, nspare, $
                spare, xlabel, vlabel, xarr, xoff10, varr, voff10, ioff, ier, $
                llstr, lxstr, lvstr)

      endif else if initpff eq 2 then begin
; Sun
        junk = call_external(PFF_EXE,'pff_slrngd_', pid, dataset, mdim, ndim, $
                xdim, vdim, sprdim, app, tlab, clab, m, n, nx, nvp, nspare, $
                spare, xlabel, vlabel, xarr, xoff10, varr, voff10, ioff, ier, $
                llstr, lxstr, lvstr)

      endif else if initpff eq 4 then begin
; WIN
        junk = call_external(PFF_EXE,'_PFF_SLRNGD@8', pid, dataset, mdim, ndim, $
                xdim, vdim, sprdim, app, tlab, clab, m, n, nx, nvp, nspare, $
                spare, xlabel, vlabel, xarr, xoff10, varr, voff10, ioff, ier, $
                llstr, lxstr, lvstr)

      endif

;
;    pfftype - PFF dataset type
;            7-n-dimensional vectors on a region of m-dimensional space (NGD)
;                (m, n, nvert),
;
     x1lab = ''
     x2lab = ''
     x3lab = ''
     x4lab = ''
     x5lab = ''
     x6lab = ''
     x1 = 0.0
     x2 = 0.0
     x3 = 0.0
     x4 = 0.0
     x5 = 0.0
     x6 = 0.0
     slice = 0.0
     if mdim ge 3 then begin
       x1lab = strtrim(xlabel(0:lxstr-1), 2)
       x2lab = strtrim(xlabel(lxstr:2*lxstr-1), 2)
       x3lab = strtrim(xlabel(2*lxstr:3*lxstr-1), 2)
       x1 = xarr(0:nx(0)-1)
       x2 = xarr(nx(0): nx(0)+nx(1)-1)
       x3 = xarr(nx(0)+nx(1): nx(0)+nx(1)+nx(2)-1)
       if mdim ge 4 then begin
         x4 = xarr(nx(0)+nx(1)+nx(2): nx(0)+nx(1)+nx(2)+nx(3)-1)
         x4lab = strtrim(xlabel(3*lxstr:4*lxstr-1), 2)
         if mdim ge 5 then begin
           x5 = xarr(nx(0)+nx(1)+nx(2)+nx(3): nx(0)+nx(1)+nx(2)+nx(3)+nx(4)-1)
           x5lab = strtrim(xlabel(4*lxstr:5*lxstr-1), 2)
           if mdim ge 6 then begin
             x6 = xarr(nx(0)+nx(1)+nx(2)+nx(3)+nx(4): nx(0)+nx(1)+nx(2)+nx(3)+nx(4)+nx(5)-1)
             x6lab = strtrim(xlabel(5*lxstr:6*lxstr-1), 2)
           endif
         endif
       endif
       ok = where (nx(0:mdim-1) eq 1, nok)
       if (nok gt 0) && (mdim le 3) then begin
         two = ok(0)+1
         if two eq 1 then slice = x1(0) else $
           if two eq 2 then slice = x2(0) else $
             if two eq 3 then slice = x3(0) else slice = x4(0)
       endif else two = 0L
       ssize = nx
     endif else if mdim eq 2 then begin
       mdim = 3
       if spare(0) eq 1 then templabel= ['X', 'Y', 'Z'] else $
         if spare(0) eq 2 then templabel= ['R', 'Theta', 'Z'] else $
           if spare(0) eq 3 then templabel= ['R', 'Theta', 'Phi'] else $
             templabel= ['X', 'Y', 'Z']
       if spare (1) eq 1 then begin
         x2lab = strtrim(xlabel(0:lxstr-1), 2)
         x3lab = strtrim(xlabel(lxstr:2*lxstr-1), 2)
         x1lab = templabel(0)
         x2 = xarr(0:nx(0)-1)
         x3 = xarr(nx(0): nx(0)+nx(1)-1)
         ssize = [1, nx]
         two  = 1L
       endif else if spare (1) eq 2 then begin
         x3lab = strtrim(xlabel(0:lxstr-1), 2)
         x1lab = strtrim(xlabel(lxstr:2*lxstr-1), 2)
         x2lab = templabel(1)
         x3 = xarr(0:nx(0)-1)
         x1 = xarr(nx(0): nx(0)+nx(1)-1)
         ssize = [nx(1), 1, nx(0)]
         two = 2
       endif else begin
         two = 3
         x1lab = strtrim(xlabel(0:lxstr-1), 2)
         x2lab = strtrim(xlabel(lxstr:2*lxstr-1), 2)
         x3lab = templabel(2)
         x1 = xarr(0:nx(0)-1)
         x2 = xarr(nx(0): nx(0)+nx(1)-1)
         ssize = [nx, 1]
       endelse
     endif else begin
;      This is for lineouts and is not really supported.
       if spare(1) eq 1 then begin
         x1lab = strtrim(xlabel(0:lxstr-1), 2)
         x1 = xarr(0:nx(0)-1)
         two = 2L
       endif else if spare(1) eq 2 then begin
         x2lab = strtrim(xlabel(0:lxstr-1), 2)
         x2 = xarr(0:nx(0)-1)
         two = 1L
       endif else if spare(1) eq 3 then begin
         x3lab = strtrim(xlabel(0:lxstr-1), 2)
         x3 = xarr(0:nx(0)-1)
         two = 1L
       endif else begin
;         print, 'I do not understand this output. Putting data into x1'
         x1lab = strtrim(xlabel(0:lxstr-1), 2)
         x1 = xarr(0:nx(0)-1)
         two = 2L
       endelse
     endelse
     v1lab = ''
     v2lab = ''
     v3lab = ''
     v4lab = ''
     v5lab = ''
     v6lab = ''
     v1 = 0.0
     v2 = 0.0
     v3 = 0.0
     v4 = 0.0
     v5 = 0.0
     v6 = 0.0
     if ndim ge 1 then begin
       if spare(2) eq 2 then begin
         v2 = varr(*, 0)
         v2lab = strtrim(vlabel(0:lvstr-1), 2)
       endif else if spare(2) eq 3 then begin
         v3 = varr(*, 0)
         v3lab = strtrim(vlabel(0:lvstr-1), 2)
       endif else begin
         v1 = varr(*, 0)
         v1lab = strtrim(vlabel(0:lvstr-1), 2)
       endelse
       if ndim ge 2 then begin
         if spare(3) eq 1 then begin
           v1 = varr(*, 1)
           v1lab = strtrim(vlabel(lvstr:2*lvstr-1), 2)
         endif else if spare(3) eq 3 then begin
           v3 = varr(*, 1)
           v3lab = strtrim(vlabel(lvstr:2*lvstr-1), 2)
         endif else begin
           v2 = varr(*, 1)
           v2lab = strtrim(vlabel(lvstr:2*lvstr-1), 2)
         endelse
         if ndim ge 3 then begin
           if spare(4) eq 1 then begin
             v1 = varr(*, 2)
             v1lab = strtrim(vlabel(2*lvstr:3*lvstr-1), 2)
           endif else if spare(4) eq 2 then begin
             v2 = varr(*, 2)
             v2lab = strtrim(vlabel(2*lvstr:3*lvstr-1), 2)
           endif else begin
             v3 = varr(*, 2)
             v3lab = strtrim(vlabel(2*lvstr:3*lvstr-1), 2)
           endelse
           if ndim ge 4 then begin
             v4 = varr(*, 3)
             v4lab = strtrim(vlabel(3*lvstr:4*lvstr-1), 2)
             if ndim ge 5 then begin
               v5 = varr(*, 4)
               v5lab = strtrim(vlabel(4*lvstr:5*lvstr-1), 2)
               if ndim ge 6 then begin
                 v6 = varr(*, 4)
                 v6lab = strtrim(vlabel(5*lvstr:6*lvstr-1), 2)
               endif
             endif
           endif
         endif
       endif
     endif
;
     if m gt 1 then begin
;      Transpose the data if y is the degenerate dimension
     if (spare(1) eq 2) and (not keyword_set(xzdata)) and $
           ((m eq 2) or (m eq 3 and nx(1) eq 1)) then begin
       if m eq 3 then begin
            nx(1) = nx(0)
            nx(0) = nx(2)
       endif
       npt = nx(0)*nx(1)
       if ndim ge 1 then begin
         v1 = reform(transpose(reform(v1, nx(0), nx(1), /overwrite)), $
                 npt, /overwrite)
         if ndim ge 2 then begin
           v2 = reform(transpose(reform(v2, nx(0), nx(1), /overwrite)), $
                   npt, /overwrite)
           if ndim ge 3 then begin
             v3 = reform(transpose(reform(v3, nx(0), nx(1), /overwrite)), $
                     npt, /overwrite)
             if ndim ge 4 then begin
               v4 = reform(transpose(reform(v4, nx(0), nx(1), /overwrite)), $
                       npt, /overwrite)
               if ndim ge 5 then begin
                 v5 = reform(transpose(reform(v5, nx(0), nx(1), /overwrite)), $
                         npt, /overwrite)
                 if ndim ge 6 then begin
                   v6 = reform(transpose(reform(v6, nx(0), nx(1), /overwrite)), $
                           npt, /overwrite)
                 endif
               endif
             endif
           endif
         endif
       endif 
     endif 
     endif
      locx = [0*ssize, ssize]
      locv = [0, vdim]
      range = make_array(2, mdim, 2)
      range(0, 0, *) = min(x1, max = mx)
      range(1, 0, *) = mx
      if mdim gt 1 then begin
        range(0, 1, *) = min(x2, max = mx)
        range(1, 1, *) = mx
        if mdim gt 2 then begin
          range(0, 2, *) = min(x3, max = mx)
          range(1, 2, *) = mx
          if mdim gt 3 then begin
            range(0, 3, *) = min(x4, max = mx)
            range(1, 3, *) = mx
          endif
        endif
      endif
      order = 1L
      array = {type:1L, ndim:mdim, nvect: ndim, block :1L, two: two, $
              slice: slice, size: ssize, range: range, $
              dx1:0.0, dx2:0.0, dx3:0.0, dx4:0.0, dx5:0.0, dx6:0.0, $
              order: order, locx:reform(locx, mdim, 2), $
              locv: locv, x1:temporary(x1), x2:temporary(x2), $
              x3:temporary(x3), x4:temporary(x4), x5:temporary(x5), $
              x6:temporary(x6), v1:temporary(v1), $
              v2:temporary(v2), v3:temporary(v3), v4:temporary(v4), $
              v5:temporary(v5), v6:temporary(v6),x1lab : x1lab, x2lab :x2lab, $
              x3lab : x3lab, x4lab :x4lab, x5lab : x5lab, x6lab :x6lab, $
              v1lab : v1lab, v2lab :v2lab, $
              v3lab : v3lab, v4lab :v4lab,v5lab : v5lab, v6lab :v6lab, $
              file : filename, spare : spare, tlab : tlabel, clab:clabel }
      if lcstructure then i2s, array, get_maxstr()-1, /no_copy
      end
