; ------------------------------------------------------------------------ 
; $Id: readpar.pro,v 1.10 2011/06/07 20:54:31 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
 Pro readpar, array, datasetin, fileid = fid, electron = electron, $
            proton = proton, stringlen= stringlen, $
            tlabel= tlabel, clabel = clabel, nptot= nptot, $
            namefile = filename, pfftyp = pfftyp,  $
            w1 = w1, w2= w2, w3 = w3, w4 = w4, w5= w5, w6= w6, $
            radius = radius, quiet = quiet, old_version = old_version, $
            cyl2cart = c2c, analyze_step = analyze_step, min_step = min_step, $
            skip = skip, dlength = dlength, iextra = iextra
;
;+
;
;  Reads a particle distribution into an IDL structure.
;  Procedure options are explained below. PFF dataset type must be VTX
;    Note: Particle types:
;    Type 20 - Particle position
;    Type 21 - Particle position and charge
;    Type 23 - Particle position and momentum
;    Type 24 - Particle position, momentum and charge
;    Type 25 - Particle position, momentum, charge and an integer attribute
;    Type 26 - Particle trajectory data (position, momentum, charge and 
;                particle number) or (position, momentum and particle number)
;                (Particle number is always in attribute 6)
;    Type 27 - Particle position, momentum, charge and kill time
;    Type 28 - Particle position, momentum, charge, kill time, and an integer
;    Type 29 - 1D Dynaid particle data
;
;
;  Format:  READPAR, structure  [, dataset] [, FILEID = fileid]
;                   [, ELECTRON= ELECTRON] [, PROTON = PROTON] [, NPTOT = nptot]
;                   [, ANALYZE_STEP = analyze_step] [, MIN_STEP = min_step]
;                   [, W1= WINDOW1] [, W2= WINDOW2] [, W3= WINDOW3]
;                   [, W4= WINDOW4] [, W5= WINDOW5] [, W6= WINDOW6]
;                   [, CYL2CART = cyl2cart] [, stringlen = stringlen]
;                   [, RADIUS = radius] [, /QUIET]
;  where:
;    structure - IDL structure name
;              If not a valid variable, data is stored in GET_MAXSTR()-1
;    dataset - indicates a PFF dataset number 
;              or is a search string for dataset comments.  
;              If dataset is missing or zero, then it is set to the 
;              current dataset.
;        Note: If dataset is a string, a leading `^' indicates that 
;              the string begins at the start of the dataset comment.
;        Note: If dataset is a string, it must refer to a single dataset.
;    fileid -  file id of the PFF file containing the data
;    stringlen - Currently ignored
;                Currently 64 characters for each label
;    ELECTRON  - If set, normalized particle energy is multiplied by the
;                electron rest energy- See note below.
;    PROTON    - If electron is set, this parameter is ignored.
;                If electron is not set, then the normalized energy is multiplied
;                by the rest energy of the proton if PROTON is 1 or by the rest energy
;                of an atomic mass unit times PROTON.  If PROTON is input as zero, it is 
;                changed to 1.
;    nptot     - Total number of particles read from the dataset before window.
;    analyze_step- If the application dataset type is 26 (tracer particles),
;                then the number of particles versus number of time steps 
;                will be calculated and printed.  The data will be stored 
;                in maxwdfarray.
;                If analyze_step is less than zero, then detailed statistics
;                  will be printed.  Otherwise groups will be printed for more 
;                  than 5 vertex points.
;    min_step  - If the application dataset type is 26 (tracer particles),
;                then only those particles with at least min_step intervals will
;                be saved. If min_step is greater than zero, then analyze
;                statistics will also be printed.
;                Note: MIN_STEP intervals requires MIN_STEP+1 values.
;    W1, W2, ...W6  - Each is an array of 3 or more elements.
;                 After filtration by minstep the data is further 
;                 windowed in real space or attribute space
;                 Last two values in the array are the minimum and maximum
;                 values of the window parameter; the first value(s) are the
;                 spatial or attribute parameters to be used for the window.
;        for example to window all spatial components to [0, 1] use 
;                 w1 =[1, 2, 3, 0, 1]
;        NOTE:  No provisions are made for PFF rounding.
;        NOTE:  Energy has not been calculated. Attributes are as stored in file.
;   QUIET      - If set, do not print warning messages for the W1, ... W6 windowing
;   CYL2CART   - If set and EXTRA(0) eq 2 then the x1 and x2 dimensions will
;                be converted to cartesian coordinates using the following:
;                  x1' = x1 * cos(x2)
;                  x2' = x1 * sin(x2)
;   RADIUS     - If set, 
;                Number of dimensions is set to 4 and a 4th coordinate is 
;                  x4 = SQRT(x1*x1 + x2*x2)
;                  X4Lab = 'Radius'
;            Note:  To plot particles as radius vs Z from cartesian data 
;                   in dataset ds:
;                        readstr, 1, ds, /readius
;                        plotstr, 1, 3, 4                   
;                   To overlay conductors from structure 11 in the Z-X plane:
;                        plotstr, 11, 3, 1, 0, /ignore, /overplot
;   OLD_VERSION- If set, KE will always be in attribute 5
;                If not set, KE will be put in the last attribute.
;
; NOTE: If the number of attributes is three or larger, an additional attribute 
;       will be added called 'KE/rest_mass' or KE in this note.
;       If OLD_VERSION is set then KE will be in attribute 5 and 
;       additional attributes will be moved up 1 position; otherwise
;       KE will be in the last attribute.
;       In all cases the KE attribute is given by the following:
;          bg2 = (Px^2 +Py^2 +Pz^2)/c^2
;          KE  = SQRT(bg2 + 1) -1   ;  KE/Rest_Mass
;       where the P's refer to the attributes for Px, Py, Pz and 
;          c = SOL() = PHYCONSTF(/C) =  299792458.0D0 = "speed of light"
;       The actual attribute number depends on the dataset type and the value of OLD_VERSION.
;       If electron (/electron)is present with ANY value 
;        then KE = KE * PHYCONSTF(/EENERGY) *1e-6 and the label is 'Electon KE (MeV)'
;       If PROTON is present and 1 then
;           KE = KE * PHYCONSTF(/PENERGY)*1e-6
;       (NOTE: /PROTON means PROTON= 1 and PROTON = 0 is changed to PROTON = 1)
;       or if PROTON is not equal to 1 then KE=KE*PROTON*PHYCONSTF(/AENERGY)* 1.0e6
;           In both cases the label is 'Ion KE (MeV)'
;           Current values for electron, proton and amu energies (eV) are:
;             510998.91, 9.3827201e+08, 9.3149403e+08
;
;  Structure Labels
;    type  - 2
;    ndim  - number of spatial dimensions
;    natt  - number of attribute dimensions
;    block - number of blocks (=1)
;    nvert - number of vertices 
;    two   - two dimensional missing dimension
;            this is 0 for 3d data, 1 for yz data, 2 for zx data, 3 for xy data
;    x1, x2, ...x(ndim) - spatial arrays
;    v1, v2, ...v(nvect) - attribute arrays
;    x1lab, x2lab, ...x(ndim)lab - spatial labels
;    v1lab, v2lab, ...v(nvect)lab - vector labels, Vector label in the case of
;                  NGD data and block label in the case of vector and 3d data
;    tlab  - type label
;    clab  - dataset comment
;    file  - file data read from
;    spare - integer array (5)
;  Examples:
;    Read dataset 34 in file id 4 into structure ch5
;      readpar, ch5, 34, fil=4
;    Read next dataset of the current file into structure partstart
;      readpar, partstart
;-
;
      on_error, 1
;
      common pffcom, initpff, pff_exe
@pff_strings.com
      stringlen = max_str_len
;
;     check to see if pff has been initialized
;
      if n_elements(initpff)eq 0 then initpff = 0
      if (initpff eq 0) then begin
        print, '% READPAR: File must be opened before reading data'
        return
      endif
;
;     check on the number of parameters
;
      nparams =  n_params()
;
;     get integers for the write
;
      if nparams eq 0 then begin
        print,'% Format: READPAR, structure  [, dataset] [, FILEID = fileid]'
        print,'      [, ELECTRON= electron] [, PROTON = proton] [, NPTOT=nptot]'
        print,'      [, ANALYZE_STEP = analyze_step] [, MIN_STEP = min_step]'
        print,'      [, W1= WINDOW1] [, W2= WINDOW2] [, W3= WINDOW3]'
        print,'      [, W4= WINDOW4] [, W5= WINDOW5] [, W6= WINDOW6]'
        print,'      [, /CYL2CART] [, STRINGLEN = stringlen]'
        return
      endif
      array = 0.0
      lcstructure = 0
      if not arg_present(array) then  begin
        print, '% READPAR: Warning data cannot be returned in variable'
        print, '           Data will be stored in structure '+strtrim (get_maxstr()-1, 2)
        lcstructure = 1
      endif
      if n_elements (fid) eq 0 then fid = 0L
      pid = long(fid)
      if n_elements(datasetin) eq 0 then dataset = 0L else dataset=datasetin(0)
      if keyword_set(skip) then begin
        if n_elements(dlength) ne 1 or n_elements (iextra) lt 8 or $
        n_elements (pfftyp) ne 1 then message, 'Error with keyword SKIP'
      endif else begin
        nmap = 0L
        nmax = 1L
        if size(/type, dataset) eq 7 then begin
          get_dset_string, dataset,pid,  map, nmap, nmax, ier
          if ier ne 0 then message, ' Please change the dataset specification'
          if nmap gt 0 then begin
            print, '% READPAR:  Dataset string must provide a unique dataset'
            print, '      Please change the dataset specification to get a single dataset.'
            help, dataset
            print, 'Datasets: '
            for i = 0, nmap-1 do print, map[i]
            return
          endif
          dataset = map(0)
        endif
;
        readhdr,dataset,ier,fileid=pid, pfftyp = pfftyp, $
          dlength = dlength, tlabel= tlabel, clabel=clabel, $
          namefile = filename, iextra = iextra, /quiet
;
        if ier ne 0 or n_elements(pfftyp) eq 0 then begin
          print, ' Error reading dataset header'
          return
        endif
      endelse
      if pfftyp ne 5 then begin
        print, ' Dataset ', dataset,' has a pff type of ', pfftyp
        print, ' This dataset type is not supported with READPAR'
        return
      endif
;     READVTX
      mdim = long (iextra(0)) < max_block
      ndim = long (iextra(1)) < Max_block
      vdim = mdim*iextra(2)
      adim = ndim*iextra(2)
      vert = make_array(vdim>1)
      attr = make_array(adim>1)
      ier = 0L
      m = 0L
      n = 0L
      nv = 0L
      nvr = 0L
      app = 0L
      spare = make_array(iextra(3), /long)
      lvstr = stringlen
      vlabel = make_array(lvstr*mdim, /byte, value = 32B)
      lastr = stringlen
      alabel = make_array(lastr*ndim>1, /byte, value = 32B)
      llstr = 40L
      clab = make_array(llstr, /byte, value = 32B)
      tlab = make_array(llstr, /byte, value = 32B)
      voff10 = 0.0
      aoff10 = 0.0
      if  initpff eq 1 then begin
; HP
        junk = call_external(PFF_EXE,'pff_slrvtx', pid, dataset, mdim, ndim, $
                vdim, adim, m, n, nv, nvr, app, tlab, clab, spare, $
                vlabel, alabel, vert, voff10, attr, aoff10, ier, $
                llstr, lvstr, lastr)
      
      endif else if initpff eq 2 then begin
; Sun
        junk = call_external(PFF_EXE,'pff_slrvtx_', pid, dataset, mdim, ndim, $
                vdim, adim, m, n, nv, nvr, app, tlab, clab, spare, $
                vlabel, alabel, vert, voff10, attr, aoff10, ier, $
                llstr, lvstr, lastr)

      endif else if initpff eq 4 then begin
; WIN
        junk = call_external(PFF_EXE,'_PFF_SLRVTX@8', pid, dataset, mdim, ndim, $
                vdim, adim, m, n, nv, nvr, app, tlab, clab, spare, $
                vlabel, alabel, vert, voff10, attr, aoff10, ier, $
                llstr, lvstr, lastr)
      endif
      nptot = nv
      if voff10 ne 0 or aoff10 ne 0 then begin
        print,'WARNING - Power of 10 offset for the data exceeds computer range'
        print, 'Vertex offset - ', strtrim(voff10, 2), ', Attribute offset - ', $
                strtrim(aoff10, 2)
      endif
      if nv ne nvr then  print, 'Not all vertex values returned', nv, nvr
      vert = reform(vert, m, nv, /overwrite)
;  change vert to a (nv,  m) array
      vert = transpose(temporary(vert))
      if ndim gt 0 then attr = reform(attr,  nv, ndim, /overwrite)
;     vlabel = string (vlabel)
;     alabel = string (alabel)
;
      nvm = nv-1
;     if these are trajectories, see if analysis or deletion are necessary
;      if app eq 25 or app eq 26 then begin
       if app eq 26 then begin
        if n_elements(min_step) eq 0 then min_step = 0
        if n_elements(analyze_step) eq 0 then analyze_step = 0
        if keyword_set (min_step) or keyword_set(analyze_step) then begin
;         calculate the pointer array for the number of particles
          if ndim eq 6 then nn = attr(0:nvm, 5) $
            else if ndim eq 5 then nn = attr(0:nvm, 4) $
            else if ndim eq 4 then nn = attr(0:nvm, 3) $
            else if ndim eq 3 then nn = attr(0:nvm, 2) $
            else if ndim eq 2 then nn = attr(0:nvm, 1) else $
             nn = attr(0:nvm, 0)
          nx = n_elements(nn)
          dn = round (nn(1: nx-1)- nn(0:nx-2))
          ok = where (dn gt 0, nok)
          if nok eq 0 then loc = [0, nx] else loc =[0, ok+1, nx]
          nloc = n_elements(loc)
          dloc =loc(1:nloc-1)-loc(0:nloc-2)
          maxdl = max(dloc)
          nn = make_array(/long , maxdl)
          dloc = dloc-1
          for i = 0, nloc-2 do nn(dloc(i))= nn(dloc(i))+1
          if  (analyze_step ne 0) or (min_step gt 0) then begin
            if analyze_step lt 0 then begin
              print, ' Vertex Points   # of Particles'
              for i = 0L, maxdl-1 do $
                if nn(i) gt 0 then print, i+1, nn(i)
            endif else begin
              print, '      Vertex Points     # of Particles'
              j = 1L
              kold = 0L
              istart =1L
again:        for i = istart, 5 do begin
                k = i*j
                if k le maxdl then begin
                  val= long(total(nn(kold:k-1)))
                  print, kold+1, ' to', k,  val
                  kold = k
                endif else begin
                  if kold lt maxdl-1  then begin
                    k = maxdl
                    val= long (total(nn(kold:k-1)))
                    print, kold+1, ' to', k,  val
                  endif
                  goto, finished
                endelse
              endfor
              istart = 2L
              j = j*5L
              goto, again
            endelse
          endif
finished:
          print, total(nn(0:maxdl-1))
          if keyword_set(analyze_step) then i2w, $
            findgen(maxdl)+1., nn(0:maxdl-1),get_maxwdf(), $
            clab = 'Distribution of Trajectory Length', $
            xlab = 'Number of vertex points', ylab = 'Number of particles', $
            file = filename
          nx = abs(round(min_step))
          if nx gt 0 then begin
            ndelete = long(total(nn(0:nx-1)))
            print, ndelete, ' trajectories have been deleted'
            if ndelete gt 0 then begin
              nv = 0L
              print, min(loc, max = mm), mm
              for i = 0L, nloc-2 do begin
                if dloc(i) ge nx then begin
;               move the data
                  vert(nv:nv+dloc(i), *) = vert(loc(i):loc(i+1)-1, *)
                  attr(nv:nv+dloc(i), *) = attr(loc(i):loc(i+1)-1, *)
                  nv = nv+dloc(i)+1
                endif
              endfor
            endif
          endif
        endif
      endif

;
; check the windows
;
      nelw = n_elements(w1)
      if nelw gt 2 then begin
        minw = min(w1(nelw-2:nelw-1), max = maxw)
        w = w1 (0:nelw-3)
        for i = 0, nelw-3 do begin
          window_it, w(i), minw, maxw, vert, attr, nv, mdim, ndim, error = 0.0, quiet = quiet
          if nv eq 0 then  begin
            if not keyword_set(quiet) then print, '% READPAR: No particles after W1 filter'
            return
          endif
        endfor
      endif

      nelw = n_elements(w2)
      if nelw gt 2 then begin
        minw = min(w2(nelw-2:nelw-1), max = maxw)
        w = w2 (0:nelw-3)
        for i = 0, nelw-3 do begin
          window_it, w(i), minw, maxw, vert, attr, nv, mdim, ndim, error = 0.0, quiet = quiet
          if nv eq 0 then begin
            if not keyword_set(quiet) then print, '% READPAR: No particles after W2 filter'
            return
          endif
        endfor
      endif

      nelw = n_elements(w3)
      if nelw gt 2 then begin
        minw = min(w3(nelw-2:nelw-1), max = maxw)
        w = w3 (0:nelw-3)
        for i = 0, nelw-3 do begin
          window_it, w(i), minw, maxw, vert, attr, nv, mdim, ndim, error = 0.0, quiet = quiet
          if nv eq 0 then begin
            if not keyword_set(quiet) then print, '% READPAR: No particles after W3 filter'
            return
          endif
        endfor
      endif

      nelw = n_elements(w4)
      if nelw gt 2 then begin
        minw = min(w4(nelw-2:nelw-1), max = maxw)
        w = w4 (0:nelw-3)
        for i = 0, nelw-3 do begin
          window_it, w(i), minw, maxw, vert, attr, nv, mdim, ndim, error = 0.0, quiet = quiet
          if nv eq 0 then  begin
            if not keyword_set(quiet) then print, '% READPAR: No particles after W4 filter'
            return
          endif
        endfor
      endif

      nelw = n_elements(w5)
      if nelw gt 2 then begin
        minw = min(w5(nelw-2:nelw-1), max = maxw)
        w = w5 (0:nelw-3)
        for i = 0, nelw-3 do begin
          window_it, w(i), minw, maxw, vert, attr, nv, mdim, ndim, error = 0.0, quiet = quiet
          if nv eq 0 then begin
            if not keyword_set(quiet) then print, '% READPAR: No particles after W5 filter'
            return
          endif
        endfor
      endif

      nelw = n_elements(w6)
      if nelw gt 2 then begin
        minw = min(w6(nelw-2:nelw-1), max = maxw)
        w = w6 (0:nelw-3)
        for i = 0, nelw-3 do begin
          window_it, w(i), minw, maxw, vert, attr, nv, mdim, ndim, error = 0.0, quiet = quiet
          if nv eq 0 then begin
            if not keyword_set(quiet) then print, '% READPAR: No particles after W6 filter'
            return
          endif
        endfor
      endif

      nvm = nv-1

      x2 = 0.0
      x3 = 0.0
      x4 = 0.0
      x2lab = ''
      x3lab = ''
      x4lab = ''
      if m gt 0 then begin
        x1 = vert(0:nvm, 0) 
        x1lab = strtrim(vlabel(0:lvstr-1), 2)
        if m gt 1 then begin
          x2 = vert(0:nvm, 1)
          x2lab = strtrim(vlabel(lvstr:2*lvstr-1), 2)
          if m gt 2 then begin
            x3 = vert(0:nvm, 2)
            x3lab = strtrim(vlabel(2*lvstr:3*lvstr-1), 2)
            if m gt 3 then begin
              x4 = vert(0:nvm, 3)
              x4lab = strtrim(vlabel(3*lvstr:4*lvstr-1), 2)
            endif
          endif
        endif
      endif else begin
        x1 = 0.0
        x1lab = ''
      endelse
;     
      if keyword_set(c2c) then begin
        if spare(0) eq 2 then begin
          co = cos(x2)
          si = sin(x2)
          x2 = x1*si
          x1 = temporary(x1)*co
          x1lab = 'x'
          x2lab = 'y'
          spare(0) = 1
        endif else print, $
      '% READPAR: Spare keyword does not indicate cylindrical coordinates: ', $
        strtrim(spare(0), 2)
      endif
;
      v2 = 0.0
      v3 = 0.0
      v4 = 0.0
      v5 = 0.0
      v6 = 0.0
      v7 = 0.0
      v8 = 0.0
      v2lab = ''
      v3lab = ''
      v4lab = ''
      v5lab = ''
      v6lab = ''
      v7lab = ''
      v8lab = ''
      if n gt 0 then begin
        v1 = attr(0:nvm, 0)
        v1lab = strtrim(alabel(0:lastr-1), 2)
        if n gt 1 then begin
          v2 = attr(0:nvm, 1)
          v2lab = strtrim(alabel(lastr:2*lastr-1), 2)
          if n gt 2 then begin
            v3 = attr(0:nvm, 2)
            v3lab = strtrim(alabel(2*lastr:3*lastr-1), 2)
            if n gt 3 then begin
              v4 = attr(0:nvm, 3)
              v4lab = strtrim(alabel(3*lastr:4*lastr-1), 2)
              if n gt 4 then begin
                v5 = attr(0:nvm, 4)
                v5lab = strtrim(alabel(4*lastr:5*lastr-1), 2)
                if n gt 5 then begin
                  v6 = attr(0:nvm, 5)
                  v6lab = strtrim(alabel(5*lastr:6*lastr-1), 2)
                  if n gt 6 then begin
                    v7 = attr(0:nvm, 6)
                    v7lab = strtrim(alabel(6*lastr:7*lastr-1), 2)
                    if n gt 7 then begin
                      v8 = attr(0:nvm, 7)
                      v8lab = strtrim(alabel(7*lastr:8*lastr-1), 2)
                    endif
                  endif
                endif
              endif
            endif
          endif
        endif
      endif else begin
        v1 = 0.0
        v1lab = ''
      endelse
;
      attr = 0
      mx = max(abs(vert))
      space = 0
      if (max(x1,min = mn)-mn)/mx lt 6e-5 then begin
        two = 1L
      endif else if (max(x2,min = mn)-mn)/mx lt 6e-5 then begin
        two = 2L
      endif else if (max(x3,min = mn)-mn)/mx lt 6e-5 then begin
        two = 3L
      endif else two = 0L
; ndim = number of attributes
      if (app ge 23 and app lt 29) then begin
;       We have momentum as the first 3 attributes
;       
        if (ndim eq 3) and keyword_set(old_version) then begin
          v4lab = 'Dummy ARRAY = 0.0'
          v4 = v3*0.0
        endif        
; If Time histories with 4 attributes - assume px, py, pz and history number.
        if keyword_set(old_version) then begin
          if ndim eq 6 then begin
            v7 = v6
            v7lab = v6lab
            ndim = 7
          endif
          if ndim ge 5 then begin
            v6 = temporary(v5)
            v6lab = v5lab
            if ndim eq 5 then ndim = 6
          endif else if ndim eq 4 and app eq 26 then begin
;         Duplicate particle number into 6 for trajectory datasets
            ndim = 6
            v6 = v4
            v6lab = v4lab
            ndim = 6
          endif else ndim = 5L
        endif else ndim = ndim+1

;       calculate (beta*gamma)^2
        bg2 = (v1*v1+v2*v2+v3*v3)/ (sol()*sol())
        bg2 = bg2/(sqrt(bg2+1.0)+1.0)
        if n_elements(electron) gt 0 then begin
          bg2 = bg2 * (phyconstf(/eenergy) *1.0e-6)
          velab = 'Electon KE (MeV)'
        endif else if n_elements (proton) gt 0 then begin
          if proton eq 0 then proton = 1
	  if proton eq 1 then  bg2 = bg2* (phyconstf(/penergy)*1.0e-6) $
          else  bg2 = bg2* (proton*phyconstf(/aenergy) *1.0e-6)
          velab  = 'Ion KE (MeV)'
        endif else velab = 'KE/rest_mass' 
        if keyword_set(old_version) then begin
          v5 = temporary(bg2)
          v5lab = temporary(velab)
        endif else begin
          if ndim eq 4 then begin
            v4 = temporary(bg2)
            v4lab = temporary(velab)
          endif else if ndim eq 5 then begin
            v5 = temporary(bg2)
            v5lab = temporary(velab)
          endif else if ndim eq 6 then begin
            v6 = temporary(bg2)
            v6lab = temporary(velab)
          endif else if ndim eq 7 then begin
            v7 = temporary(bg2)
            v7lab = temporary(velab)
          endif else if ndim eq 8 then begin
            v8 = temporary(bg2)
            v8lab = temporary(velab)
          endif
        endelse
      endif
      if keyword_set (radius) then begin
        x4 = sqrt(x1*x1+x2*x2)
        mdim = 4L
        x4lab = 'Radius' 
      endif
      array = {type:2L, ndim:temporary(mdim), nvert:temporary(nv), $
              natt :temporary(ndim) ,block: 1L, $
              two:temporary(two),  x1:temporary(x1), x2:temporary(x2), $
              x3:temporary(x3), x4:temporary(x4), v1:temporary(v1), $
              v2:temporary(v2), v3:temporary(v3), v4:temporary(v4), $
              v5:temporary(v5), v6:temporary(v6), v7:temporary(v7), $
              v8:temporary(v8), x1lab :temporary(x1lab), $
              x2lab : temporary(x2lab), x3lab : temporary(x3lab), $
              x4lab : temporary(x4lab), v1lab : temporary(v1lab), $
              v2lab : temporary(v2lab), v3lab : temporary(v3lab), $
              v4lab : temporary(v4lab), v5lab : temporary(v5lab), $
              v6lab : temporary(v6lab), v7lab : temporary(v7lab), $
              v8lab : temporary(v8lab), file : filename,  $
              spare : temporary(spare), tlab : tlabel, clab: clabel }
      if lcstructure then i2s, array, get_maxstr()-1, /no_copy
      end
