; ------------------------------------------------------------------------ 
; $Id: readvtx.pro,v 1.6 2010/03/04 15:16:32 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
 Pro readvtx, attr, datasetin, fileid = fid, stringlen= stringlen, $
             xarray = x1, yarray = x2, tarray = x4, x4label = x4lab, $
             zarray= x3, xlabel = x1lab, ylabel = x2lab, $
             ndim = m, nattr = n,  conductor = conductor, $
             zlabel = x3lab, alabel = atlabel, tlabel= tlabel, $
             clabel = clabel, namefile = filename, pfftyp = pfftyp,  $
             spare = spare, aptype = app, skip = skip, $
             dlength = dlength, iextra = iextra

;
;+
;
;  Reads a field or charge density into an IDL structure.
;  Procedure options are explained below.
;  Dataset must be type 5, PFFVTX
;
;  Format:  READVTX, ATTRIBUTES  [, dataset] [, FILEID = fileid]
;                    [, STRINGLEN = stringlen] 
;                    [, NDIM = ndim]  [, NATTR = nattr]
;                    [, XARRAY= xarray] [, YARRAY= yarray]
;                    [, ZARRAY= zarray] [, TARRAY= tarray]
;                    [, APTYPE = aptype] [, ALABEL = alabel]
;                    [, CLABEL = clabel] [, TLABEL = tlabel]
;                    [, XLABEL = xlabel] [, YLABEL = ylabel] 
;                    [, ZLABEL = zlabel] [, X4Label = x4label]
;                    [, NAMEFILE = namefile] [, PFFTYP = pfftyp]
;                    [, SPARE = spare] [, CONDUCTOR = conductor]
;  where:
;    ATTRIBUTES  - IDL Attribute Array size= (NVERT, NATTR)
;              This must be a valid variable
;    dataset - indicates a PFF dataset number 
;              or is a search string for dataset comments.  
;              If dataset is missing or zero, then it is set to the 
;              current dataset.
;        Note: If dataset is a string, a leading `^' indicates that 
;              the string begins at the start of the dataset comment.
;              Dataset string must refer to only 1 dataset
;    fileid -  file id of the PFF file containing the data
;    stringlen - Currently Ignored
;                Maximum number of characters in the spatial labels and the
;                field labels.  Maximum of 64 for each for each block
;    Ndim      - Number of spatial values
;    NATTR     - Number of attributes for each vertex
;    xarray    - Values for the first dimension
;    yarray    - Values for the second dimension
;    zarray    - Values for the third dimension
;    tarray    - Values for the fourth dimension
;    aptype    - Application dataset type
;    Alabel(*) - Labels the attributes
;    Clabel    - Comment Label
;    TLABEL    - Type Label
;    SPARE     - Spare (5) spare array for each block
;    XLabel    - Label for XARRAY
;    YLabel    - Label for YARRAY
;    ZLabel    - Label for ZARRAY
;    X4Label    - Label for Tarray (not TLABEL because of type label)
;    NAMEFILE  - Filename
;    APTYPE    - Application dataset type
;    Conductor - If set, vertex data is returned in ATTRIBUTES 
;                Conductor = conductor(ndim, nvert)
;  Examples:
;    Read dataset 34 in file id 4 into array ch5, put x and y values
;      into xar and yar
;      readvtx, ch5, 34, fil=4, xar = xar, yar = yar
;-
;
      on_error, 1
;
      common pffcom, initpff, pff_exe
;
@pff_strings.com
      stringlen = max_str_len
;
;     check to see if pff has been initialized
;
      if n_elements(initpff) eq 0 then initpff = 0
      if (initpff eq 0) then begin
        print, '% READVTX: File must be opened before reading data'
        return
      endif
;
;     check on the number of parameters
;

      nparams =  n_params()
      slice = 0.0
      farray = 0.0
      if (nparams lt 1 ) then begin
  print,'% Format: READVTX, Attributes  [, dataset] [, FILEID = fileid]'
  print,'        [, STRINGLEN = stringlen]  [, NDIM = ndim] [, NAttr = nttr]'
  print,'        [, XARRAY= xarray] [, YARRAY= yarray] [, ZARRAY= zarray]'
  print,'        [, TARRAY= tarray] [, X4LABEL = x4label]'
  print,'        [, XLABEL= xlabel] [, YLABEL= ylabel] [, Zlabel= Zlabel]'
  print,'        [, ALABEL = alabel] [, CLABEL = clabel] [, TLABEL = tlabel]'
  print,'        [, SPARE = spare] [, NAMEFILE = namefile] [, APTYPE = APtype]'
        return
      endif

      if n_elements (fid) eq 0 then fid = 0L
      pid = long(fid)
      if n_elements (datasetin) eq 0 then dataset = 0L else $
        dataset = datasetin(0)
      if keyword_set(skip) then begin
        if n_elements(dlength) ne 1 or n_elements (iextra) lt 8 or $
        n_elements(pfftyp) ne 1 then message, 'Error with keyword SKIP'
      endif else begin
        nmap = 0L
        nmax = 1L
        attr = 0
        if not arg_present(attr) then begin
          print, '% READVTX: Warning data cannot be returned in variable ATTRIBUTES'
          print, '           Only keyword variables will be returned.'
        endif
        get_dset_string, dataset,pid,  map, nmap, nmax, ier
        if ier ne 0 then message, ' Please change the dataset specification'
;
        if nmap gt 1 then begin
          Print,'% READVTX: Please change the dataset specification to get a single dataset.'
          help, dataset
          print, 'Datasets: '
          for i = 0, nmap-1 do print, map[i]
          return
        endif
        dataset = map(0)
        readhdr,dataset,ier,fileid=pid, pfftyp = pfftyp, $
           dlength = dlength, tlabel= tlabel, clabel=clabel, $
           namefile = filename, iextra = iextra, aptyp = app,  /quiet
        if ier ne 0 or n_elements(pfftyp) eq 0 then begin
          print, ' Error reading dataset header'
          return
        endif
      endelse
      if (pfftyp  ne 5 ) then begin
        print, '% READVTX: Dataset type '+strtrim(pfftyp, 2) + $
            'cannot be read with READVTX'
        return
      endif

; READVTX
;                 iextra(0) = vertex dimensions
;                 iextra(1) = number of attributes
;                 iextra(2) = number of vertices
;                 iextra(3) = number of spare words in each block

;
      mdim = long (iextra(0)) < max_block
      ndim = long (iextra(1)) < max_block
      vdim = mdim*iextra(2)
      adim = ndim*iextra(2)
      vert = make_array(vdim>1)
      attr = make_array(adim>1)
      ier = 0L
      m = 0L
      n = 0L
      nv = 0L
      nvr = 0L
      app = 0L
      spare = make_array(iextra(3), /long)
      lvstr = stringlen
      vlabel = make_array(lvstr*mdim, /byte, value = 32B)
      lastr = stringlen
      alabel = make_array(lastr*ndim>1, /byte, value = 32B)
      llstr = 40L
      clab = make_array(llstr, /byte, value = 32B)
      tlab = make_array(llstr, /byte, value = 32B)
      voff10 = 0.0
      aoff10 = 0.0
      if  initpff eq 1 then begin
; HP
        junk = call_external(PFF_EXE,'pff_slrvtx', pid, dataset, mdim, ndim, $
                vdim, adim, m, n, nv, nvr, app, tlab, clab, spare, $
                vlabel, alabel, vert, voff10, attr, aoff10, ier, $
                llstr, lvstr, lastr)

      endif else if initpff eq 2 then begin
; Sun
        junk = call_external(PFF_EXE,'pff_slrvtx_', pid, dataset, mdim, ndim, $
                vdim, adim, m, n, nv, nvr, app, tlab, clab, spare, $
                vlabel, alabel, vert, voff10, attr, aoff10, ier, $
                llstr, lvstr, lastr)

      endif else if initpff eq 4 then begin
; WIN
        junk = call_external(PFF_EXE,'_PFF_SLRVTX@8', pid, dataset, mdim, ndim, $
                vdim, adim, m, n, nv, nvr, app, tlab, clab, spare, $
                vlabel, alabel, vert, voff10, attr, aoff10, ier, $
                llstr, lvstr, lastr)

      endif
      if voff10 ne 0 or aoff10 ne 0 then begin
        print,'WARNING - Power of 10 offset for the data exceeds computer range'
        print,'Vertex offset - ', strtrim(voff10, 2), ', Attribute offset - ', $
                strtrim(aoff10, 2)
      endif
      if nv ne nvr then  print, 'Not all vertex values returned', nv, nvr
      vert = reform(vert, m, nv, /overwrite)
      
      if ndim gt 0 then attr = reform(attr,  nv, ndim, /overwrite)
;     vlabel = string (vlabel)
;     alabel = string (alabel)
;
      x2 = 0.
      x3 = 0.
      x4 = 0.
      x2lab = ''
      x3lab = ''
      x4lab = ''
      if keyword_set (conductor) then begin
         attr = vert
         if m lt 2 then begin
print, '% READVTX: Cannot have conductor data with less than 2 dimensions'
           x1lab = 'Bad Conductor X'
           x2lab = 'Bad Conductor Y'
           return
         endif
         x1lab = strtrim(vlabel(0:lvstr-1), 2)
         x2lab = strtrim(vlabel(lvstr:2*lvstr-1), 2)
         if m gt 2 then $
            x3lab = strtrim(vlabel(2*lvstr:3*lvstr-1), 2)
           
         return
      endif

;  change vert to a (nv,  m) array
      vert = transpose(temporary(vert))
      nvm = nv-1
      if m gt 0 then begin
        x1 = vert(0:nvm, 0)
        x1lab = strtrim(vlabel(0:lvstr-1), 2)
        if m gt 1 then begin
          x2 = vert(0:nvm, 1)
          x2lab = strtrim(vlabel(lvstr:2*lvstr-1), 2)
          if m gt 2 then begin
            x3 = vert(0:nvm, 2)
            x3lab = strtrim(vlabel(2*lvstr:3*lvstr-1), 2)
            if m gt 3 then begin
              x4 = vert(0:nvm, 3)
              x4lab = strtrim(vlabel(3*lvstr:4*lvstr-1), 2)
            endif
          endif
        endif
      endif else begin
        x1 = 0.0
        x1lab = ''
      endelse
      if n gt 0 then atlabel = make_array(6, /str, val = '')
      if n gt 0 then begin
        atlabel(0) = strtrim(alabel(0:lastr-1), 2)
        if n gt 1 then begin
          atlabel(1) = strtrim(alabel(lastr:2*lastr-1), 2)
          if n gt 2 then begin
            atlabel(2) = strtrim(alabel(2*lastr:3*lastr-1), 2)
            if n gt 3 then begin
              atlabel(3) = strtrim(alabel(3*lastr:4*lastr-1), 2)
              if n gt 4 then begin
                atlabel(4) = strtrim(alabel(4*lastr:5*lastr-1), 2)
                if n gt 5 then begin
                  atlabel(5) = strtrim(alabel(5*lastr:6*lastr-1), 2)
                endif
              endif
            endif
          endif
        endif
      endif
      return
      end
