; ------------------------------------------------------------------------ 
; $Id: repair_pff.pro,v 1.5 2010/04/13 18:32:27 lpmix Exp $
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
  pro repair_pff, p1,  error = error, ndset = ndset, help = help
;+
;  NAME: REPAIR_PFF
;
;  Repair a PFF DIRECTORIED file that has been improperly closed.
;  User must have write permission.
;  This routine attempts to rebuild the directory from the header info 
;    in each dataset, one dataset at a time.  If it cannot reconstruct
;    the entire directory, it truncates the file after the last dataset
;    for which a directory entry was successfully loaded.  It then closes
;    the file, rewriting the directory if necessary.
;
;  Format: 
;     REPAIR_PFF [, File] [, Error = error] [, NDSET =ndset] [, /HELP]
;
;  where:
;    File   - name of the PFF file to be repaired.  The default file extension
;             is ".PFF". 
;             If file is omitted or undefined or is not a string,
;             PICKFILE will be used to select a file.
;             The initial path in PICKFILE is the current directory or
;             the PATH to the last file selected.
;             NOTE: PATH and FILTER may be used to set the initial path and
;                   filter for PICKFILE.
;             NOTE: FULFNAM procedure is used to handle environment variables
;                   See FULFNAM for details
;    NDSET  - Number of datasets successfully recovered from the file
;    Error  - Error parameter
;             -3,   Problem calling the repair procedure
;             -2,   File repaired with potential loss of data
;             -1,   File repaired with no loss of data
;              0,   File not in need of repair
;              1,   No Available PFF File ID's
;              2,   No Available Logical Units
;              3,   Error Opening File
;              4,   File opened for read is empty
;              5,   File has no PFF header
;    HELP   - If set, print a help text
;
;  Examples:
;    Repair a PFF file
;      REPAIR_PFF
;
;    Determine keywords and parameters for REPAIR_PFF
;      REPAIR_PFF, /help
;-
;
      on_error, 1
      common pffcom, initpff, pff_exe
      COMMON savepicker, oldfilter, oldpath
;
;     check on the number of parameters
;
      error = -3
      nparams = n_params()
      if nparams eq 0 and keyword_set(help) then begin
print, '% Format: REPAIR_PFF [, File] [, NDSET = ndset] [, Error = error]'
        return
      endif
;
;     check to see if pff has been initialized
;
      if (n_elements(initpff) eq 0) then init_pff else $
        if (initpff eq 0) then init_pff
      if initpff eq 0 then begin
        print, 'PFF libraries are not supported by this Operating System'
        return
      endif
;
;     be sure we have a file name
;
      if (nparams eq 0) then begin
        p1 = ''
        nparams = 1
      endif else if n_elements(p1) eq 0 then begin
        p1 = ''
      endif else if (size(p1, /type) ne 7) then begin
        p1 = ''
      endif else  p1 = strtrim(p1, 2)
      cd, current = current
      if strlen(p1) lt 1 then begin
        p1 = dialog_pickfile(/must_exist,  title= 'Select PFF File to Repair')
        if strlen(p1) lt 1 then begin
          print, '% REPAIR_PFF:  No File was Selected - RETURNING'
          return
        endif
      endif
;
      vers = strupcase(strmid(!version.os_family, 0, 3))
      if (vers eq 'UNI') then begin
;       get system variable substitution for unix systems
        temp = p1
        FULFNAM, temp, p1
      endif 
      p1 = file_expand_path(p1)
      if !version.os eq 'linux' then resolve_it, p1
      info = file_info(p1)
      if (info.exists + info.read + info.write) ne 3b then begin
         print, '% REASC: File does not exist with read/write permission'
         print, '         File Name = ', p1
         print, '         File Exists =', info.exists
         print, '         Read Permission =', info.read
         print, '         Write Permission =', info.write
         return
      endif
      lstr = 256L
      error = 0L
      NDSET = 0L
      if strlen(p1) ge lstr then begin
        Print, '% REPAIR_PFF: Length of file must be less than 255 characters'
        return
      endif 
      str2byt, p1, p3, lstr
;
      if initpff eq 1 then begin
; HP 
        junk = call_external(PFF_EXE, 'pff_slrepair', p3, lstr, ndset, error)
      endif else if initpff eq 2 then begin
; Sun
        junk = call_external(PFF_EXE, 'pff_slrepair_', p3, lstr, ndset, error)
      endif else if initpff eq 4 then begin
; WIN
       junk = call_external(PFF_EXE, '_PFF_SLREPAIR@8', p3, lstr, ndset, error)
      endif
      error_list = ['File repaired with potential loss of data', $
               'File repaired with no loss of data', $
               'File not in need of repair', 'No Available PFF File ID''s', $
               'No Available Logical Units', 'Error Opening File', $
               'File opened for read is empty', 'File has no PFF header']
      print, 'File: ', p1
      print, '      ', error_list[error+2]
      end
