; ------------------------------------------------------------------------ 
; $Id: wriifl.pro,v 1.6 2007/06/06 19:59:48 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
  pro wriifl,  iarrayin, flist, farray, clabel = clabel, tlabel = tlabel, $
      ap = apptyp, fileid = fid, full_precision = full_precision , ierror = ier

;+
;  WRIIFL
;  Write an integer float list to a PFF file.
;  Format:  WriIFL, iarray [, flist] [, FARRAY] [, TLABEL = tlabel]
;           [, CLABEL = clabel] [, AP = apptype] [, FILEID = fileid]
;           [, FULL_PRECISION = full_precision] [, IERROR = ierror]
; 
;  where:
;    iarray - integer array to be written  
;          or a type 5 or type 101 structure
;          or if n_params()eq 1 and iarray is a scalar then 
;             a structure array number.
;    flist  - float list to be written
;    farray - floating array
;    tlabel - Dataset type label - a maximum of 256 characters are written
;    clabel - Dataset comment - a maximum of 256 characters are written
;             CLABEL is put after the SHOT, LOS and DATE in a type 101 structure
;    ap - application dataset type (User parameter) (default = -3)
;             AP = 101 if type 101 structure is used
;    fileid - file id. If it is omitted or zero, the current file is used.  If 
;             present, the current file pointer will be set to this file.
;    FULL_PRECISION - If defined, FARRAY will be written with the indicated
;                     precision.  If not defined, FARRAY will be written with
;                     the default precision.
;           FULL_PRECISION =0   : Array written as a scaled 2 byte array
;           FULL_PRECISION not 0: Array written as a 32-bit IEEE array
;           Type 101 structures are stored at full precision unless 
;             full_precision = 0 is specified
;
;    Note:  Text strings are limited to 240 characters for the write command.
;    Ierror  - Error flag.  If zero, file successfully written.
;
;  Examples:
;     Write the integers representing the load and source type, and the 
;         float parameters [a, b, c, d, e, f] as an ifl dataset.
;     WRIIFL, [loadtyp, sourtyp], [a, b, c, d, e, f] , $
;         t= 'Run Parameters', c = 'Float list: t0, z0, Vmax, tmax, alp, c', $
;         ap = -3
;
;-
;     this routine writes a ifl dataset to a pff file
;
      on_error, 1
;
; define max_tlabel for length of the type label.
; current version also limits clabel to the same value
@pff_strings.com

      common pffcom, initpff, pff_exe
      nparams = n_params()
      if nparams eq 0 then begin
        print, '% Format:  WRIIFL, iarray [, flist] [, FARRAY]'
        print, '       [, TLABEL = tlabel] [, CLABEL = clabel] [, AP = apptype]'
        print, '       [, FILEID = fileid] [, FULL_PRECISION = full_precision]'
        return
      endif
;
;     check to see if pff has been initialized
;
      if (n_elements(initpff) eq 0) then initpff =0
      if (initpff eq 0) then begin
        print, '% WRIIFL: File must be opened or created before writing data'
        return
      endif
;
;     check on the number of parameters
;
;
;     must have 1 parameters
;
      if n_elements(fid) eq 0 then fid = 0L else fid = long(fid(0))
;
;     insure that integer array is defined
;
      ni = n_elements(iarrayin)
      if (ni eq 0) then begin
        print,'% WRIIFL: Integer Array/Structue is not defined'
        return
      endif
      if ni gt 1 then iarray = long(iarrayin) else begin
        if size (iarrayin[0], /type) eq 8 then str = iarrayin[0] else  $
          if nparams eq 1 then s2i, str, long(iarrayin[0])
        if size (str, /type) ne 8 then begin
          print, '% WRIIFL: Integer Array or Structure must be provided.'
          return
        endif
        ok = where(tag_names(str) eq 'TYPE', nok)
        if nok ne 1 then begin
          print, '% WRIIFL: First arguement is not a valid PFIDL structure'
          return
        endif
        valid = [5, 101]
        ok = where (str.type eq valid, nok)
        if nok eq 0 then begin
          print, '% WRIIFL: Structures must be IFL structures: ' , valid
          print, '          Structure passed of type: ',strtrim(str.type, 2)
          return
        endif
        if ok[0] eq 0 then begin
;         Type 5 structure
          iarray= long(str.iarray)
          flist = str.flist
          farray = str.farray
          if n_elements(clabel) eq 0 then clabel = str.clab 
          if n_elements(tlabel) eq 0 then tlabel = str.tlab
          ssize = str.size
        endif else begin
;         Type 101 structure
          if n_elements(full_precision) eq 0 then full_precision=1 
          if n_elements (tlabel) eq 0 then tlabel = 'XRD Configuration'
          if n_elements (clabel) gt 0 then comment = clabel[0] else $
            comment = ''
          clabel = 'Shot'+string (str.shot, format='(i5)') + $
                 ', LOS '
          los = strupcase(strtrim(str.los, 2))
          if strlen(los) eq 0 then los = '   '
          date = thin_date()
          clabel = clabel + los +';;'+date+', '+str.comment +' - ;;'
          ndet = str.ndet[0]
          if ndet eq 0 then begin
            print, '% WRIIFL: NDET = 0 cannot write as a type 101 dataset'
            return
          endif
          for i = 0, ndet-1 do $
            clabel = clabel+ (str.signal)[i]+';;'+ (str.cathode)[i]+';;' $
                  + (str.calibration)[i]+';;'+ (str.filter)[i]+';;' 
          iarray = long([ndet, str.source])
          ni = 2L
          farray = [str.diameter[0:ndet-1], str.energy_bin[0:ndet]]
          if str.source eq 0 then val=str.src_area else val = str.aper_ht
          flist = [str.dist, val, str.pinch_ht, str.elevation, str.az_ang]
          apptyp = 101
        endelse
      endelse 
          
      nfl = n_elements (flist)
      if nfl eq 0 then begin
        nfl =1L
        flist = 0.0
      endif else flist = float(temporary(flist))
      nfa = n_elements(farray)
      if nfa eq 0 then farray = 0.0 else farray =  float(temporary(farray))
;
      if n_elements(apptyp) eq 0 then apptyp = -3L else apptyp = long(apptyp)
      if n_elements (tlabel) eq 0 then tlabel = 'Int/Float List'
      if n_elements (clabel) eq 0 then clabel = ''
      llabstr = long(max([16L,((strlen(tlabel)/4+1)*4),((strlen(clabel)/4+1)*4)])< max_tlabel)
      str2byt, tlabel, btemp0, llabstr
      str2byt, clabel, btemp1, llabstr
      if ((n_elements(full_precision) gt 0) and (nfa gt 0)) then begin
        pflag = keyword_set(full_precision[0])
        ier = 0
        if fid ne 0 then setpff, fid, ier
        if ier ne 0 then begin
          print, '% WRIIFL: Invalid File ID, NO data written'
          return
        endif
        pff_precision, 0, full = pflag, prec = prec
        if (prec[1] eq pflag) then lcprecision = 1 else lcprecision = 0
      endif else lcprecision =0

;
      ier = 0L
      if initpff eq 1 then begin
; HP
        junk=call_external(PFF_EXE, 'pff_slwifl', fid, apptyp, $
          btemp0, btemp1, ni, nfl, nfa, iarray, flist, farray, ier, llabstr)
      endif else if initpff eq 2 then begin
; Sun
        junk=call_external(PFF_EXE, 'pff_slwifl_', fid, apptyp, $
          btemp0, btemp1, ni, nfl, nfa, iarray, flist, farray, ier, llabstr)
      endif else if initpff eq 4 then begin
; WIN
        junk=call_external(PFF_EXE, '_PFF_SLWIFL@8', fid, apptyp, $
          btemp0, btemp1, ni, nfl, nfa, iarray, flist, farray, ier, llabstr)
      endif else if initpff eq 5 then begin
; MAC
        junk=call_external(PFF_EXE, 'pff_slwifl', fid, apptyp, $
          btemp0, btemp1, ni, nfl, nfa, iarray, flist, farray, ier, llabstr)
      endif
;
      if lcprecision gt 0 then pff_precision, full = prec[1]

      if ier ne 0 then $
         Print,'% WRIIFL: Error writing dataset, ier = ', strtrim(ier,2)

      return
      end
