; ------------------------------------------------------------------------ 
; $Id: writeifl.pro,v 1.7 2009/03/27 17:26:13 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
  pro writeifl, fid, iarray,flist, farray, clabel = clabel, tlabel = tlabel, $
      ap = apptyp, full_precision = full_precision
;+
;  WRITEIFL
;  Write an integer float list to a PFF file.
;  See also WriIFL
;  Both version are present for backward compatibility
;
;  Format:  WRITEIFL, fileid, iarray , flist [, FARRAY] [, TLABEL = tlabel]
;       [, CLABEL = clabel] [, AP = apptype] [, FULL_PRECISION = full_precision]
; 
;  where:
;    fileid - file id. If it is omitted or zero, the current file is used.  If 
;             present, the current file pointer will be set to this file.
;    iarray - integer array to be written
;    flist  - float list to be written
;    farray - floating array
;    tlabel - Dataset type label - a maximum of 256 characters are written
;    clabel - Dataset comment - a maximum of 256 characters are written
;    ap - application dataset type (User parameter) (default = -3)
;    FULL_PRECISION - If defined, FARRAY will be written with the indicated
;                     precision.  If not defined, FARRAY will be written with
;                     the default precision.
;           FULL_PRECISION =0    Array written as a scaled 2 byte array
;           FULL_PRECISION not 0 Array written as a 32-bit IEEE array
;    Note:  Text strings are limited to 256 characters for the write command.
;           (Defined in PFF_STRINGS.COM)
;  Examples:
;     Write the integers representing the load and source type, and the 
;         float parameters [a, b, c, d, e, f] as an ifl dataset.
;     WRITEIFL, 0, [loadtyp, sourtyp], [a, b, c, d, e, f] , $
;         t= 'Run Parameters', c = 'Float list: t0, z0, Vmax, tmax, alp, c', $
;         ap = -3
;
;-
;     this routine writes a ifl dataset to a pff file
;
      common pffcom, initpff, pff_exe
@pff_strings.com
;
;     check to see if pff has been initialized
;
      if (n_elements(initpff) eq 0) then initpff =0
      if (initpff eq 0) then begin
        print, '% WRITEIFL: File must be opened or created before writing data'
        return
      endif
;
;     check on the number of parameters
;
      nparams = long( n_params())
      if nparams eq 0 then begin
        print, '% Format:  WRITEIFL, fileid, iarray [, flist] [, FARRAY]'
        print, '       [, TLABEL = tlabel] [, CLABEL = clabel] [, AP = apptype]'
        print, '       [, FULL_PRECISION = full_precision]'
        return
      endif
;
;     must have two parameters
;
      if (nparams lt 2) then begin
        print ,' Must have a valid fid or 0 and an integer array to write'
        return
      endif
      if n_elements(fid) eq 0 then fid = 0L
      fid = long (fid(0))
;
;     insure that image is defined
;
      ni = n_elements(iarray)
      if (ni eq 0) then begin
        print,' Integer Array is not defined'
        return
      endif else iarray = long(temporary(iarray))
      nfl = n_elements (flist)
      if nfl eq 0 then begin
        nfl =1L
        flist = 0.0
      endif else flist = float(temporary(flist))
      nfa = n_elements(farray)
      if nfa eq 0 then farray = 0.0 else farray =  float(temporary(farray))
;
      if n_elements(apptyp) eq 0 then apptyp = -3L else apptyp = long(apptyp)
      if n_elements (tlabel) eq 0 then tlabel = 'Int/Float List'
      if n_elements (clabel) eq 0 then clabel = ''

      if ((n_elements(full_precision) gt 0) and (nfa gt 0)) then begin
        pflag = keyword_set(full_precision[0])
        ier = 0
        if fid ne 0 then setpff, fid, ier
        if ier ne 0 then begin
          print, '% WRITEIFL: Invalid File ID, NO data written'
          return
        endif
        pff_precision, 0, full = pflag, prec = prec
        if (prec[1] eq pflag) then lcprecision = 1 else lcprecision = 0
      endif else lcprecision =0

      llabstr = $
      (max([16L,((strlen(tlabel)/4+1)*4),((strlen(clabel)/4+1)*4)])< max_tlabel)
      llabstr = long(llabstr)

      str2byt, tlabel, btemp0, llabstr
      str2byt, clabel, btemp1, llabstr

;
      ier = 0L
      if initpff eq 1 then begin
; HP
        junk=call_external(PFF_EXE, 'pff_slwifl', fid, apptyp, $
          btemp0, btemp1, ni, nfl, nfa, iarray, flist, farray, ier, llabstr)
      endif else if initpff eq 2 then begin
; Sun
        junk=call_external(PFF_EXE, 'pff_slwifl_', fid, apptyp, $
          btemp0, btemp1, ni, nfl, nfa, iarray, flist, farray, ier, llabstr)
      endif else if initpff eq 4 then begin
; WIN
        junk=call_external(PFF_EXE, '_PFF_SLWIFL@8', fid, apptyp, $
          btemp0, btemp1, ni, nfl, nfa, iarray, flist, farray, ier, llabstr)

      endif 
;
      if lcprecision gt 0 then pff_precision, full = prec[1]

      if ier ne 0 then $
         Print,'% WRITEIFL: Error writing dataset, ier = ', strtrim(ier,2)
      end
