; ------------------------------------------------------------------------ 
; $Id: writepff.pro,v 1.4 2007/06/06 19:59:48 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
  pro writepff, fid, image,space, tlabel = tlab, clabel = clab, $
      xlabel = xlab, ylabel = ylab, zlabel = zlab, $
      blabel = blab, label = lab, ap = apptyp,spare = ispare
;+
;  WRITEPFF
;  Write an IDL array to a PFF file.  Only 1-D, 2-D, and 3-D data may be
;  written in the present verision of the interface.  The variables are
;  the same as for the read command.  Nonuniform 1-D data and all 2-D data
;  will be written as 3-D data at the present time.  No interactive 
;  input is supported.  Datasets are added sequentially to the end of the file.
;  Format:  WRITEPFF, fileid, image [, space] [, TLABEL = tlabel]
;           [, CLABEL = clabel] [, BLABEL = blabel] [, XLABEL = xlabel]
;           [, YLABEL = ylabel] [, ZLABEL = zlabel] [, LABEL = label]
;           [, AP = apptype] [, SPARE = ispare]
;  where:
;    fileid - file id. If it is omitted or zero, the current file is used.  If 
;             present, the current file pointer will be set to this file.
;    image, space - arrays containing the dataset information defined 
;                   in READPFF for the same variables.
;                   If `space' is not provided, a default grid, beginning at 
;                   zero (0.0) with a spacing of 1.0 units  is used for each 
;                   dimension.
;    tlabel - dataset type label - the first 16 characters are stored 
;             in the directory
;    clabel - dataset comment - the first 64 characters are stored 
;             in the directory
;    blabel - title or label for this block
;    xlabel - first dimension label for this block
;    ylabel - second dimension label for this block
;    zlabel - third dimension label for this block
;    label - array of all the above labels in the order given 
;            (label(0) = tlabel, label(1) = clabel, etc.)
;            If `LABEL' is provided, all other label keywords will be 
;            appended to the front of the string in the `LABEL' array.
;    ap - application dataset type (User parameter) (default = -3)
;    spare - spare integer array for the application (User specified 
;            interger iarray(5))
;    Note:  Text strings are limited to 240 characters for the write command.
;  Examples:
;  Write an IDL array, picture, to the end of the current file.  Since no
;  spatial array is provided, a default grid, beginning at zero, with an
;  increment of 1.0 will be provided for each dimension of `picture'.
;    writepff, 0, picture
;  Write an IDL  array, imag, with spatial array, grid, to the end of the file 
;  having a file id of 23.  Use the strings provided for the various labels.
;    writepff, 23, imag, grid, t='k-alpha image', $
;         c = `Unfiltered image in quadrant 1',  $
;         x= `Distance - Microns', y= `Distance - Microns', b= `Film Density'
;  Write the IDL array, image, with spatial array, space, to the end of
;  the current file.  Use the character array, lab, for the labels and 
;  comments.  The clabel string will be appended to the beginning of the
;  contents of lab(1).
;    writepff , 0, image, space, l= lab,  $
;        c= `Filtered image & subtracted bkgrd of 0.233'
;-
;     this routine writes a pff file
;     only 1d and 3d data types are supported
;
      common pffcom, initpff, pff_exe
;
;     check to see if pff has been initialized
;
      on_error, 1
      if (n_elements(initpff) eq 0) then initpff =0
      if (initpff eq 0) then begin
        print, '% WRITEPFF: File must be opened or created before writing data'
        return
      endif
;
;     check on the number of parameters
;
      nparams = long( n_params())
;
;     must have two parameters
;
      if (nparams lt 2) then begin
        print ,' Must have a valid fid or 0 and an image array to write'
        return
      endif
      fid = long (fid)
;
;     insure that image is defined
;
      np = n_elements(image)
      if (np eq 0) then begin
        print,' Image is not defined'
        return
      endif
      c = size (image)
      if (nparams gt 2) then nsp = n_elements(space) else nsp = 0
      ny= 0L
      nz= 0L
      if (c(0) le 1) then begin
        ndim = 1L
        if c(0) eq 1 then nx = long(c(1)) else nx = 1L
        if (c(c(0)+1) ne 4) then image = float(image)
      endif else begin
        ndim = c(0)
        nx = long(c(1))
        ny = long(c(2))
        if (ndim eq 3) then begin
          nz = long(c(3)) 
          if (c(4) ne 4) then image = float(image)
        endif else begin        
          nz = 1L
          if (c(3) ne 4) then image = float(image)
        endelse
      endelse
      if (nsp gt 0) then begin
        b = size(space)
        if (b(0) ne 1) then begin
          space = space (*,0,0,0)
          nsp = n_elements (space)
        endif
      endif
;
;     generate an array spa with extra slots for possible use in the write
;
      if (nparams eq 2) then spa = 0.0 else begin 
        if (nsp eq 0) then nparams = 2L else begin
          totsp =nx+ny+nz+2
          spa = make_array(totsp)
          mm = min([nsp-1, totsp-3])
          spa(0:mm) = float(space(0:mm))
;         for i = 0L, (min([nsp-1, totsp-3])) do spa(i)= float(space(i))
        endelse
      endelse
;     set the application dataset type to -3 (the default) if not specified
      if (n_elements(apptyp) eq 0) then apptyp = -3L
;     
      if (n_elements(apptyp) gt 1) then tempa = apptyp(0) else $
        tempa = long(apptyp)
      if (n_elements(ispare) eq 0) then ispare = make_array(5, /long)
      if (n_elements(ispare) lt 5) then begin
        nel = n_elements(ispare)
        temp = make_array (5, /long)
        for i = 0, nel-1 do temp(i) = ispare(i)
      endif else       temp = long(ispare)
      blank = string( replicate(32b, 240))
      tempstr= make_array(6, /string, value = blank)
      if (n_elements (lab) gt 0L) then begin
        nel = n_elements (lab)
        m = nel < 6
        for i = 0, m-1 do tempstr(i) = strmid(lab(i)+tempstr(i),0,240)
      endif 
      if (n_elements (tlab) gt 0) then  $
          tempstr0=strmid(tlab+tempstr(0),0,240) else tempstr0 = tempstr(0)
      if (n_elements (clab) gt 0) then  $
          tempstr1=strmid(clab+tempstr(1),0,240) else tempstr1 = tempstr(1)
      if (n_elements (blab) gt 0) then  $
          tempstr2=strmid(blab+tempstr(2),0,240) else tempstr2 = tempstr(2)
      if (n_elements (xlab) gt 0) then  $
          tempstr3=strmid(xlab+tempstr(3),0,240) else tempstr3 = tempstr(3)
      if (n_elements (ylab) gt 0) then  $
          tempstr4=strmid(ylab+tempstr(4),0,240) else tempstr4 = tempstr(4)
      if (n_elements (zlab) gt 0) then  $
          tempstr5=strmid(zlab+tempstr(5),0,240) else tempstr5 = tempstr(5)
;
      llabstr = 240L
      if initpff ne 3 then begin
        str2byt, tempstr0, btemp0, llabstr
        str2byt, tempstr1, btemp1, llabstr
        str2byt, tempstr2, btemp2, llabstr	
        str2byt, tempstr3, btemp3, llabstr
        str2byt, tempstr4, btemp4, llabstr
        str2byt, tempstr5, btemp5, llabstr
      endif
;
      ier = 0L
      if initpff eq 1 then begin
; HP
        junk=call_external(PFF_EXE, 'pff_slwrite', nparams, fid, image, $
          nx, ny, nz, spa, tempa, temp, btemp0, btemp1, btemp2, $
          btemp3, btemp4, btemp5, ndim, ier, llabstr)
      endif else if initpff eq 2 then begin
; Sun
        junk=call_external(PFF_EXE, 'pff_slwrite_', nparams, fid, image, $
          nx, ny, nz, spa, tempa, temp, btemp0, btemp1, btemp2, $
          btemp3, btemp4, btemp5, ndim, ier, llabstr)
      endif else if initpff eq 4 then begin
; WIN
        junk=call_external(PFF_EXE, '_PFF_SLWRITE@8', nparams, fid, image, $
          nx, ny, nz, spa, tempa, temp, btemp0, btemp1, btemp2, $
          btemp3, btemp4, btemp5, ndim, ier, llabstr)
      endif
;
      return
      end
