; ------------------------------------------------------------------------
; $Id: cablcomp.pro,v 1.2 2007/06/06 20:03:44 lpmix Exp $
; ------------------------------------------------------------------------
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
;+
; NAME:
;       CABLCOMP
;
; PURPOSE:
;       Correct the measured signal for cable degredations
;
; CALLING SEQUENCE:
;       CABLCOMP, WDF1, WDF2 [, WDF3] [, /PLOTIT] [, /ZOOMIT]
;
; INPUTS:
;       WDF1:   Measured degraded signal
;       WDF2:   Calculated response function from GET_CABLCOMP
;               NOTE:  WDF2 should be on the same time base as WDF1
;
; OPTIONAL INPUTS:
;       WDF3:   Storage location for the calculated signal
;               DEFAULT:  WDF3 = WDF1
;	
; KEYWORD PARAMETERS:
;       PLOTIT: Plot WDF1 and WDF3, the cable compensated version of WDF1
;               If KEYWORD_SET(ZOOMIT), then PLOTIT = 1
;       ZOOMIT: If set, use ZOOMW to view the plot.  
;
; OUTPUTS:
;       The waveform is corrected for the cable attenuations
;
; SIDE EFFECTS:
;	The original data (WDF1) is stored in Get_MaxWDF().
;
; PROCEDURE:
;       The procedure uses the time domain convolution described by Bill Boyer
;       (SAND87-3072).  The technique is similar to the XDAMP routine
;       XI_cabcomp.  Modified for speed increase.
;
; EXAMPLE:
;	Calculate the signal response expected for a signal in WDF array 5 using 
;       the calculated response in WDF array 44.  Store the result back in array 5
;       and plot the measured and corrected signal.
;
;          CABLCOMP, 5, 44, /plot
;
; MODIFICATION HISTORY:
; 	Written by:	L. P. Mix, Jr., December 7, 2006
;                       From XDAMP routine XI_cabcomp
;                       See SAND87-3072, 1987
;                       Computer Compensation for Cable Signal Degradations
;                         by W. B. Boyer
;-
; Parts of this routine taken from the XDAMP routine XI_cabcomp
; Copyright (c) 2006, Sandia Corporation. The United States Government
; retains a nonexclusive license in this software as prescribed in
; AL 88-1 and AL 91-7. Export of this program may require a license from
; the United States Government.
;++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
;  procedure XI_cabcomp

;  Procedure to compensate cables by convolving ENTERED array with a second
;  waveform assumed to be a compensator. 
;  Maintains number of points in signal array.
;------------------------------------------------------------------------------
PRO CABLCOMP, WDF1, WDF2, WDF3, plotit = plotit, zoomit = zoomit
@com.dat
on_error, 1
  if n_elements(initwdf) eq 0 then wdfinit
  nparams = n_params()
  if nparams eq 0 then begin
    print, '% Format: CABLCOMP, WD1, WD2 [, WD3]'
    return
  endif
  if nparams lt 2 then begin
    print,'% CABLCOMP: Add must have at least 2 parameters'
    return
  endif
;
  if n_elements(wdf1) eq 0 or  n_elements(wdf2) eq 0 then begin
    print, '% CABLCOMP: Parameters must be defined'
    return
  endif
  if wdf1 lt 1 or wdf1 gt maxwdfarray then begin
    print, '% CABLCOMP: WDF1 must be greater than 0 and </=', maxwdfarray
    return
  endif
  n1 = wdfnp(wdf1-1)
  if n1 lt 2 then begin
    print, '% CABLCOMP: No data in the first array'
    return
  endif
  if wdf2 lt 1 or wdf2 gt maxwdfarray then begin
    print, '% CABLCOMP: WDF2 must be greater than 0 and </=', maxwdfarray
    return
  endif
  n2 = wdfnp(wdf2-1)
  if n2 lt 2 then begin
    print, '% CABLCOMP: No data in the second array'
    return
  endif
  if keyword_set(plotit) || keyword_set(zoomit) then xfr, wdf1, maxwdfarray
;*** get the two waveforms
  get_wdfarray, array1, wdf1
  get_wdfarray, array2, wdf2
  if nparams gt 2 then begin
    if wdf3[0] gt 0 and wdf3[0] le maxwdfarray then save = wdf3[0] $
      else save = wdf1
  endif else save = wdf1

  n1 = n_elements(array1)-2
  n2 = n_elements(array2)-2
  nmin = min([n1, n2])
  n12 = n1+n2
  n1m = n1 -1 
  n2m = n2-1
  x0 = array1[0]
  dx = array1[1]
  ratio = dx/array2[1]
  y1 = double(array1[2:*])
  y2 = double(reverse(array2[2:*]))
  if abs(ratio-1.0) gt 0.001 then y2 *=ratio
  temp1 = y1-y1[0]
  t = systime(1)
  result = make_array(n1, /double)
  for i = 0, n1m do begin
    ovlp = MIN([i+1, n12-i, nmin])-1   ;amount of array overlap
    j11 = 0 > (i-n2m)                      ;start index of first (0 to n1-1)
    j22 = (n2m-i) > 0           ;end index of second (0 to n2-1)
    result[i] = TOTAL(temp1[j11:j11+ovlp]*y2[j22:j22+ovlp], /dou)
  endfor
; print, systime(1)-t
  if wdf1 ne save then xfr, wdf1, save
  if size(/type, array1) lt 5 then result = float(result)
  puty, result*dx+y1[0], save
  if keyword_set(plotit) || keyword_set(zoomit) then begin
     if keyword_set(zoomit) then Zoomw, [save, maxwdfarray], /color else $
       plo,  [save, maxwdfarray], /color, /over
  endif
  END

