; ------------------------------------------------------------------------
; $Id: calcl.pro,v 1.1 2009/09/02 16:18:24 lpmix Exp $
; ------------------------------------------------------------------------
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
;+
; NAME:
;       CALCL
;
; PURPOSE:
;       Calculate the inductance for a set of cylindrically symmetric
;       transmission lines symetric around the Z axis.
;
; CATEGORY:
;       Pulsed Power Design
;
; CALLING SEQUENCE:
;       Inductance = CALCL(WD1, WD2, A, B)
;
; OUTPUT:
;       INDUCTANCE: Is the calculated inductance between the R/Z
;               conductors described by WD1 and WD2 assuming cylindrical
;               symmetry around the Z-axis. The units are nH.
;
; INPUTS:
;       WD1:	Waveform array describing the anode conductor
;               Dimensions should be centimeters unless inches or meters is set
;       WD2:    Waveform array describing the cathode conductor
;               Dimensions should be centimeters unless inches or meters is set
;    NOTE:   Waveforms parameters will be switched to insure R(WD1) > R(WD2)
;            The radii of the two conductors must never cross!
;            WDEDIT may be used to generate these conductors or to modify them
;
; OPTIONAL INPUTS:
;       A:      Lower limit for inductance integral
;               Default: Minimum Z value
;       B:      Upper limit for inductance integral
;               Default: Maximum Z value
;
; KEYWORD PARAMETERS:
;       RZ:      If set, Radius is along the X axis and Z, the axis of 
;                cylindrical symmetry, is along the Y axis.
;                DEFAULT:  RZ=1
;       ZR:      If set: Z, the axis of cylindrical symmetry, is along the
;                X axis and Radius is along the Y axis.
;                Default: ZR=0
;       Note: If RZ is specified, ZR is ignored.
;             If RZ is set to 0, then ZR =1 
;       SIMPSON: If set QSIMP will be used for the integration
;                DEFAULT: SIMPSON =0 
;       ROMBERG: If set QROMB will be used for the integration
;                This is generally faster than SIMPSON
;                Default: ROMBERG = 0
;       HISTOGRAM: If set, a histogram integration will be used
;                  This will not be iterative.  
;                  HISTOGRAM is the default if neither SIMPSON nor ROMBERG are set.
;                  If HISTOGRAM eq 1 then bins will be set to 0.1 mm (0.01 cm or 0.004 inch)
;                  If HISTOGRAM lt 1 then bins will be set to a width of 
;                    HISTOGRAM 
;                  Default:  If ROMBERG and SIMPSON are zero or undefined then 
;                            HISTOGRAM = 1 if it is undefined.
;                  The integration uses an extended Simpson's rule integration.
;       VERBOSE: If set, print out the inductance and time information
;       DOUBLE:  If set, do calculation in double precision
;                Default: DOUBLE = 1
;       EPSILON: Desired fractional accuracy for QSIMP and QROMB.
;                Default: EPSILON = 4e-9 for ROMBERG and 1e-8 for SIMPSON
;       KROMB:   Passed as K to QROMB
;       INCHES:  If set, conductor dimensions in inches
;       METERS:  If set, conductor dimensions in meters
;                Meters is not set if INCHES is set
;       BREAK_POINTS: Optional list of Z-values to break the integral.  These
;                should be picked where there is a significant change in the 
;                derivative of the gap.  These will be added to points found
;                and will be used regardless of the value of MAX_BREAK.
;       MAX_BREAK: Maximum number of break points.
;                Data from a simulation may contain a large number of small steps
;                They will be generally ignored as they will exceed MAX_BREAK
;                Default: MAX_BREAK = 10
;       PLOTIT:  Plot the geometry and the the integral break points
;
; OUTPUTS:
;       This function returns the inductance for the anode and cathode 
;       geometries provided.
;
; PROCEDURE:
;       This routine calculates the integral from Zmin to Zmax of the 
;	equation: 2 ALOG(Router/Rinner) dZ.  
;       If the Z range of the two curves are not equal the maximum combined
;       Z range will be calculated with a linear interpolation of the 
;       deficient curve to the maximum / minimum Z value.
;       The integral is broken at points of constant Z if there are fewer 
;       than breakmax of these values on the anode and cathode.  
;       If there are more than breakmax we assume that each step represents a small 
;       perturbation to the gap and a single integration will suffice.
;       In all cases, BREAK_POINTS specified are used.
;
; EXAMPLE:
;       Calculate the inductance between two conductors.  One conductor's 
;       Z values vs R are stored in array 3 and the other's, in array 4
;       Assume radius is along the X-axis and Z is along the y-axis.
;         INDUCT = CALCL(3, 4) ; Histogram integration with 0.01 cm steps
;         INDUCT = CALCL(3, 4, /Romberg) ; Romberg integration with QROMB and EPSILON = 4e-9
;
;       Calculate the inductance of a coaxial transmission line with the 
;       radius as a function of length in WDF 31 and 41.  
;         INDUCT = CALCL(31, 41, /ZR)
;       Note: This will be done in double precision with Histogram integration.
;       To see a geometry plot, time duration, and each integral result:
;         INDUCT = CALCL(31, 41, /ZR, /plotit, /verbose)
;
;       Do the above problem with the dimensions in inches over the Z-values of [0, 24] inches
;          INDUCT = CALCL(31, 41, 0, 24, /ZR, /plotit, /verbose, /inches)
;
; MODIFICATION HISTORY:
; 	Written by:	Eduardo M Waisman
;
;	June 11, 2009   Modified for PFIDL by L Paul Mix
;			Modified integration scheme and supplemented by
;                       Simpson (QSIMP) and Romberg (QROMB) integration.
;                       Changed integration implementation to use a more
;                       efficient lookup and integration function.  Broke 
;                       integration at constant Z-values and know changes
;                       in slope.
;-

function tl_induct_calc, x
common tl_induct_calc_common, z_outer, r_outer, z_inner, r_inner
xi = findex(z_outer, x)
lout = interpolate(r_outer, xi)

xi = findex(z_inner, x)
lin = interpolate(r_inner, xi)
return, alog (lout/lin)
end

function CALCL, WDF1, WDF2, AIN, BIN, simpson = simps, romberg = romberg, kromb = kromb, $
    double = doubl, histogram = histo, epsilon = epsilon, verbose = verbose, plotit = plotit, $
    rz = rzin, zr =zr, inches = inches, meters = meters, MAX_BREAK = breakmax, $
    BREAK_POINTS = breakpoints

common tl_induct_calc_common, z_outer, r_outer, z_inner, r_inner

;
on_error, 1
;     include the common block
@com.dat
;
; see if we are initialized
  if n_elements(initwdf) eq 0 then begin
    wdfinit
    print, '% CALCL: No valid WDF arrays'
    return, 0
  endif
;
  npar = n_params()
  if npar eq 0 then begin
    print, '% Format: CALCL, WD1, WD2 [, A] [, B] [, SIMPSON = simpson] [, ROMBERG = romberg]'
    print, '          [, KROMB = kromb] [, DOUBLE = double] [, HISTOGRAM = histogram]'
    print, '          [, EPSILON = epsilon] [, VERBOSE = verbose] [, RZ = rz]'
    print, '          [, ZR =zr] [, INCHES = inches] [, METERS = meters]'
    print, '          [, BREAK_POINTS = breakpoints] [, BREAK_MAX = breakmax]'
    return, 0
  endif
  if npar lt 2 then begin
    print,  '% CALCL:  Two waveform parameters are required'
    return, 0
  endif
;
  if (n_elements(wdf1) eq 0) || (n_elements(wdf2) eq 0) then begin
    print,  '% CALCL:  Two waveform arrays must be defined'
    return, 0
  endif

  wd1 = wdvalid(wdf1, /valid)
  if WD1 lt 0 then begin
    print, '% CALCL:  Valid waveform arrays must be provided'
    print, '          WD1 is not valid: '+strtrim ( wd1, 2)
    return, 0
  endif
  wd2 = wdvalid(wdf2, /valid)
  if WD2 lt 0 then begin
    print, '% CALCL:  Valid waveform arrays must be provided'
    print, '          WD2 is not valid:', wd2
    return, 0
  endif
    
  rz = (n_elements(RZIN) gt 0) ? rzin[0] : -9999
  if (n_elements(zr) gt 0) && (rz eq -9999) then $
    rz = (keyword_set(zr)) ? 0 : 1 else rz = 1
  
  limits, WD1, xrange=xr1, yrange=yr1
  limits, WD2, xrange=xr2, yrange=yr2

; Calculate the middle of the region of overlap
;  to determine inner and outer conductor
; Get arrays for the conductor coordinates.
  if rz gt 0 then begin
; RZ Data
    midp = (max([yr1[0], yr2[0]]) + min([yr1[1], yr2[1]]))*0.5
    r1 = get_time(wd1, midp)
    r2 = get_time(wd2, midp)
    if r1 gt r2 then begin
;     WD1 is the outer conductor
      w2i, r_outer, z_outer, wd1
      w2i, r_inner, z_inner, wd2
    endif else begin
;     WD2 is the outer conductor
      w2i, r_outer, z_outer, wd2
      w2i, r_inner, z_inner, wd1
    endelse
  endif else begin
; ZR data
    midp = (max([xr1[0], xr2[0]]) + min([xr1[1], xr2[1]]))*0.5
    r1 = get_value(wd1, midp)
    r2 = get_value(wd2, midp)
    if r1 gt r2 then begin
;     WD1 is the outer conductor
      w2i, z_outer, r_outer, wd1
      w2i, z_inner, r_inner, wd2
    endif else begin
;     WD2 is the outer conductor
      w2i, z_outer, r_outer, wd2
      w2i, z_inner, r_inner, wd1
    endelse
  endelse
  nouter = n_elements(z_outer)
  ninner = n_elements(z_inner)

; get z increasing
  if z_outer[0] gt z_outer[nouter-1] then begin
    z_outer = reverse (z_outer)
    r_outer = reverse (r_outer)
  endif
  if z_inner[0] gt z_inner[ninner-1] then begin
    z_inner = reverse (z_inner)
    r_inner = reverse (r_inner)
  endif
  
; match the ends
  if z_inner[0] lt z_outer[0] then begin
    valadd= z_outer[0]
    z_outer = [z_inner[0], z_outer]
    r_outer = [r_inner[0], r_outer]
    nouter++
  endif else if z_inner[0] gt z_outer[0] then begin
    valadd= z_inner[0]
    z_inner = [z_outer[0], z_inner]
    r_inner = [r_outer[0], r_inner]
    ninner++
  endif

  if z_inner[ninner-1] gt z_outer[nouter-1] then begin
    z_outer = [z_outer, z_inner[ninner-1]]
    r_outer = [r_outer, r_inner[ninner-1]]
    valadd = [valadd, z_outer[nouter-1]]
    nouter++
  endif else if z_inner[ninner-1] lt z_outer[nouter-1] then begin
    z_inner = [z_inner, z_outer[nouter-1]]
    r_inner = [r_inner, r_outer[nouter-1]]
    valadd = [valadd, z_inner[ninner-1]]
    ninner++
  endif
  binwidth= 0.01
  if keyword_set(plotit) then begin
    plot, [r_inner, r_outer], [z_inner, Z_outer], /nodata, xtitle ='Radius', Ytitle='Z'
    oplot, r_inner, z_inner, color =4
    oplot, r_outer, z_outer, color = 2
  endif
  dzo = z_outer[1:nouter-1] -z_outer[0:nouter-2]
  dzi = z_inner[1:ninner-1] -z_inner[0:ninner-2]
  if min(dzo) lt 0 then begin
    print, '% CALCL: Outer Z Values must be in increasing or decreasing order'
    return, 0
  endif
  if min(dzi) lt 0 then begin
    print, '% CALCL: Inner Z Values must be in increasing or decreasing order'
    return, 0
  endif
  if n_elements(doubl) eq 0 then doubl =1
  doubl = keyword_set(doubl)
; Scale only Z since R is a ratio in the inductance formula
  if keyword_set(inches) then k= (doubl) ? 2.54d0 : 2.54 else $
    if keyword_set(meters) then k= (doubl) ? 100.d0 : 100.0 else $
    k= (doubl) ? 1.0d0 : 1.0
  if k ne 1 then begin
    z_inner = k*z_inner
    z_outer = k*z_outer
    valadd = k*valadd
  endif
  if (doubl) then begin
    r_inner = double(r_inner)
    r_outer = double(r_outer)
    if k eq 1 then begin
      z_inner = double(z_inner)
      z_outer = double(z_outer)
    endif
  endif
  Vals = make_array(2, double= doubl)
  a = (n_elements (ain) eq 0) ? z_outer[0] : (k*ain > z_outer[0])
  b = (n_elements (bin) eq 0) ? z_outer[nouter-1] : (k*bin < z_outer[nouter-1])
  if a ge b then begin
    print, '% CALCL: Lower integral value must be less than the upper value'
    print, '  Values: ', a, b
    return, 0
  endif 
  vals = [a, b]
  breakmax = (n_elements(breakmax) gt 0) ? breakmax[0] : 10
  if n_elements (valadd) gt 0 then vals = [vals, valadd]
  if n_elements (breakpoints) gt 0 then vals = [vals, breakpoints*k]
  vals_basic = vals
  ok = where (dzo eq 0.0, nok)
  if nok gt 0 then vals = [vals, z_outer[ok]]
  ok = where (dzi eq 0.0, nok)
  if nok gt 0 then vals = [vals, z_inner[ok]]
  vals = vals[ uniq(vals, sort(vals))]
  nvals = n_elements(vals)
  ok = where ((vals ge a) and (vals le b), nok)
  if nok ne nvals then begin
    vals = vals[ok]
    nvals = nok
  endif
  if nvals gt breakmax then begin
    vals = temporary(vals_basic)
    vals = vals[ uniq(vals, sort(vals))]
    nvals = n_elements(vals)
    ok = where ((vals ge a) and (vals le b), nok)
    if nok ne nvals then begin
      vals = vals[ok]
      nvals = nok
    endif
  endif
;
; eliminate similar values
;
  dz = vals[1:nvals-1] - vals[0:nvals-2]
  ok = where (dz lt 1e-6*(b-a), nok)
  if nok gt 0L then begin
    i = 0L
    while i lt nok do begin
      add = 1L
      while ((i+add lt nok) && (ok[i] +add eq ok[i+add])) do add++
      vals[ok[i]: ok[i]+add] = total(vals[ok[i]: ok[i]+add])/ (add+1)
      i = i+add
    endwhile
    vals = vals[uniq(vals)]
    nvals = n_elements(vals)
    dz = vals[1:nvals-1] - vals[0:nvals-2]
  endif

  if keyword_set(plotit) then for i = 0, nvals-1 do $
    oplot, !x.crange, vals[i]*[1,1], color = 3

  Value = (doubl) ? 0.0d0 : 0.0
  
  if keyword_set(simps) then mode = 2 $
    else if keyword_set(romberg) then mode = 3 else begin
    mode = 1
    nbin = make_array(nvals-1, /long)
    if n_elements(histo) eq 0 then histo = 1
    if histo gt 1 then nbin[*]=histo else begin
      if histo lt 1  then binwidth = histo
      nbin = ceil(dz/binwidth)
    endelse
    nbin = ((nbin mod 2) + nbin) > 4L
    dz = dz/nbin
  endelse
  if n_elements (epsilon) gt 0 then eps = epsilon[0] else $
    eps = (mode eq 2) ? 1e-8 : 0.4e-8
  tbegin = systime(1)
  if keyword_set(verbose) then begin
    print, 'Integral cuts: ', vals
  endif
 
  for i = 0, nvals-2 do begin
    a = vals[i]
    b = vals[i+1]
    oldvalue = value
    if keyword_set(verbose) then print, 'Step '+strtrim(i+1, 2) +' Integral limits', a, b
    if mode eq 1 then begin
      if keyword_set(verbose) then print, '   Number of bins: '+ Strtrim(nbin[i], 2)
      ends = [a+dz[i]/2.0, b-dz[i]/2.0]
      fends = 1.5*tl_induct_calc(ends)
      nbin2 = (nbin[i]-2)/2
      xodd = 0.5d0+dindgen(nbin2 +1)
      xeven = 1.0d0+dindgen(nbin2)
      fun1 = tl_induct_calc(vals[i] +2.0d0*dz[i]*xodd)
      fun1[0] = 0.5d0*fun1[0] & fun1[nbin2] = 0.5d0*fun1[nbin2]
      fun2 = tl_induct_calc(vals[i] +2.0d0*dz[i]*xeven)
      value = value+ total([fends, fun1, 2d0*fun2], /double)*(4.0d0*dz[i]/3d0)
    endif else begin
      if mode eq 2 then $
        value = value + 2.0d0*qsimp('tl_induct_calc', a, b, double = doubl, eps = eps) $
      else value = value + $
        2.0d0*qromb('tl_induct_calc', a, b, double = doubl, eps = eps, k= kromb)
    endelse
    if keyword_set(verbose) then print,  $
      '   Interval Inductance: '+strtrim(value -oldvalue, 2)+', Total Inductance: '+strtrim(Value, 2)
  endfor
  if keyword_set(verbose) then begin
    print, 'Results:'
    print, 'Computation Time = ', systime(1)-tbegin
    print, 'The vacuum inductance of your TL is '+strtrim(value, 2) +' nH'
  endif
  return, value
end
