; ------------------------------------------------------------------------
; $Id: cdb.pro,v 1.4 2010/09/17 21:43:00 lpmix Exp $
; ------------------------------------------------------------------------
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
;+
; NAME:
;       CDB
;
; PURPOSE:
;       This routine allows the working directory to be changed to a previous
;       directory.  This routine works with CDF and CDS to facilitate
;       directory navigation.  Two directory stacks are maintained with 
;       these three routines to allow forward and backward navigation similar
;       to a browser.  This routine accesses the backward stack.
;
; CATEGORY:
;       Utility
;
; CALLING SEQUENCE:
;
;       CDB [, NUMBER] [, /SHOW_STACK] [, /CLEAR_STACK] [, /QUIET]
;
; OPTIONAL INPUT:
;       Number      - Optional directory number
;                   If a number, this is the entry number in the past stack
;                     If NUMBER = -1, then the first directory (last directory
;                     in the back_stack) will be used.
;                   If a string, this string is compared to entries in the 
;                     past stack and the first entry found is used.  In the 
;                     case of more than one match, a warning is printed.
;                   Default: NUMBER = 1 
;                   (Directory at the top of the "Past" directory stack.)
;             Note: IF SHOW_STACK or CLEAR_STACK are set, NUMBER is ignored
;
; KEYWORD PARAMETERS:
;       SHOW_STACK  - Show the "PAST" stack of directories
;       CLEAR_STACK - Empty the "PAST" stack of directories
;       QUIET       - Do not print the current directory
;
; PROCEDURE:
;       The procedure changes the working directory to the indicated 
;       PAST directory stack value.  Before changing the current directory is 
;       stored on the "Future" directory stack.  
;
; EXAMPLE:
;       Change to the previous directory back in time.
;         CDB
;
;       Change directory to the 3 directory back
;         CDB, 3
;
;       Show all directories in the PAST stack.
;         CDB, /show
;
;       Change to the first directory put on the directory stack.
;         CDB, -1
;
; MODIFICATION HISTORY:
; 	Written by:	L. P. Mix, December 3, 2003
;-

Pro  CDB, dirin, show_stack= show_stack, clear_stack = clear_stack, quiet=quiet

COMMON CD_DIRECTORY_STACK, Back_STACK, NBACK, Forward_stack, nforward
on_error, 1		; Return to caller on error

  if n_elements(nback) eq 0 then begin
    nback=0L
    nforward = 0l
  endif
  if keyword_set(show_stack) then begin
    pwd
    if nback eq 0 then print, ' Past stack is empty.' else begin
      print, ' '
      print, ' Past Directories - Use CDB to retrieve'
      print, ' Position - Directory'
      for i = 1, nback do print, i, ' - ', back_stack[i-1]
    endelse
    return
  endif
  if keyword_set(clear_stack) then begin
    nback=0L
    back_stack=''
    return
  endif
    
  if nback eq 0 then begin
    print, '% CDB: No values are in the PAST directory stack
    return
  endif
  if n_elements(dirin) eq 0 then dir = 1 else dir = dirin[0]
  if (size(dir, /type) gt 5) then begin
    dir = strtrim(dir,2)
    ok = get_match(dir, back_stack, nel = nok)
    if nok eq 0 then begin
      print, '% CDF: No match for search string: ', dir
      cdb, /sho
      return
    endif else if nok gt 1 then begin
      print, '% CDF: Multiple matches for search string: ', dir
      print, '  Matches are:'
      for i = 0, nok-1 do print,'  ',  back_string[ok[i]]
    endif
    dir = ok[0]
  endif else dir = round(dir)-1
  if dir eq -2 then dir = nback-1
  if dir gt (nback-1) then begin
    print, '% CDB: Cannot change ', strtrim(dir+1,2), ' directories'
    cdb, /show
    return
  endif
  
  pwd, current = current, /quiet
  cd, back_stack[dir]
  if not keyword_set(quiet) then pwd
  if dir eq 0 then begin
    nback = nback -1
    if nforward eq 0 then forward_stack=[current] else $
      forward_stack=[current, forward_stack]
    nforward = nforward+1
    if nback eq 0 then back_stack = '' else back_stack= back_stack[1:*]
    return
  endif
  move = [current , back_stack[0:dir-1]]
  dir = dir+1
  ii = rotate(indgen(dir), 5)
  nback = nback -dir 
  if nforward eq 0 then forward_stack = move[ii] else $
     forward_stack=[move[ii], forward_stack]
  nforward = nforward+dir
  if nback eq 0 then back_stack= '' else back_stack= back_stack[dir:*]
  end
