; ------------------------------------------------------------------------
; $Id: cdf.pro,v 1.4 2010/09/17 21:43:00 lpmix Exp $
; ------------------------------------------------------------------------
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
;12345678901234567890123456789012345678901234567890123456789012345678901234567890
;+
; NAME:
;       CDF
;
; PURPOSE:
;       This routine allows the working directory to be changed to a previous
;       directory.  This routine works with CDB and CDS to facilitate
;       directory navigation.  Two directory stacks are maintained with 
;       these three routines to allow forward and backward navigation similar
;       to a browser.  This routine accesses the forward stack.
;
; CATEGORY:
;       Utility
;
; CALLING SEQUENCE:
;
;       CDF [, NUMBER] [, /SHOW_STACK] [, /CLEAR_STACK] [, /QUIET]
;
; OPTIONAL INPUT:
;       Number      - Optional directory number or a search string
;                   If a number, this is the entry number in the forward stack
;                     If NUMBER = -1, then the last directory (last directory 
;                     in the forward_stack) will be used.
;                   If a string, this string is compared to entries in the 
;                     forward stack and the first entry found is used.  In the 
;                     case of more than one match, a warning is printed.
;                   Default: NUMBER = 1 
;                   (Directory at the top of the "Forward" directory stack.)
;             Note: IF SHOW_STACK or CLEAR_STACK are set, NUMBER is ignored
;
; KEYWORD PARAMETERS:
;       SHOW_STACK  - Show the "FOWARD" stack of directories
;       CLEAR_STACK - Empty the "FOWARD" stack of directories
;       QUIET       - Do not print the current directory
;
; PROCEDURE:
;       The procedure changes the working directory to the indicated 
;       FOWARD directory stack value.  Before changing the current directory is 
;       stored on the "PAST" directory stack.  
;
; EXAMPLE:
;       Change to the next directory in time.
;         CDF
;
;       Change directory to the 3 directory forward
;         CDF, 3
;
;       Show all directories in the FUTURE stack.
;         CDF, /show
;
;       Change to the last directory added to the stack.
;         cdf, -1
;
; MODIFICATION HISTORY:
; 	Written by:	L. P. Mix, December 3, 2003
;-

Pro  CDF, dirin, show_stack= show_stack, clear_stack = clear_stack, quiet=quiet

COMMON CD_DIRECTORY_STACK, Back_STACK, NBACK, Forward_stack, nforward
on_error, 1		; Return to caller on error

  if n_elements(nback) eq 0 then begin
    nback=0L
    nforward = 0l
  endif
  if keyword_set(show_stack) then begin
    pwd
    if nforward eq 0 then print, ' Forward stack is empty.' else begin
      print, ' '
      print, ' Forward Directories - Use CDF to retrieve'
      print, ' Position - Directory'
      for i = 1, nforward do print, i, ' - ', forward_stack[i-1]
    endelse
    return
  endif
  if keyword_set(clear_stack) then begin
    nforward=0L
    forward_stack=''
    return
  endif
    
  if nforward eq 0 then begin
    print, '% CDF: No values are in the FORWARD directory stack'
    return
  endif
  if n_elements(dirin) eq 0 then dir = 1L else dir = dirin[0]
  if (size(dir, /type) gt 5) then begin
    dir = strtrim(dir, 2)
    ok = get_match(dir, forward_stack, nel = nok)
    if nok eq 0 then begin
      print, '% CDF: No match for search string: ', dir
      cdf, /sho
      return
    endif else if nok gt 1 then begin
      print, '% CDF: Multiple matches for search string: ', dir
      print, '  Matches are:'
      for i = 0, nok-1 do print,'  ',  forward_string[ok[i]]
    endif   
    dir = ok[0]
  endif else dir = round(dir)-1  
  if dir eq -2 then dir = nforward-1
  if dir gt (nforward-1) then begin
    print, '% CDF: Cannot change ', strtrim(dir+1,2), ' directories'
    cdf, /show
    return
  endif
  
  pwd, current = current, /quiet
  cd, forward_stack[dir]
  if not keyword_set(quiet) then pwd
  if dir eq 0 then begin
    nforward = nforward -1
    if nback eq 0 then back_stack=[current] else $
      back_stack=[current, back_stack]
    nback = nback+1
    if nforward eq 0 then forward_stack= '' else forward_stack= forward_stack[1:*]
    return
  endif
  move = [current , forward_stack[0:dir-1]]
  dir = dir+1
  ii = rotate(indgen(dir), 5)
  nforward = nforward -dir 
  if nback eq 0 then back_stack = move[ii] else $
     back_stack=[move[ii], back_stack]
  nback = nback+dir
  if nforward eq 0 then forward_stack= '' else forward_stack= forward_stack[dir:*]
  end
