; ------------------------------------------------------------------------
; $Id: cds.pro,v 1.4 2010/09/17 21:43:00 lpmix Exp $
; ------------------------------------------------------------------------
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
;+
; NAME:
;       CDS
;
; PURPOSE:
;       This routine allows the working directory to be changed using a 
;       widget interface.  This routine works with CDF and CDB to facilitate
;       directory navigation.  Two directory stacks are maintained with 
;       these three routines to allow forward and backward navigation similar
;       to a browser.
;
; CATEGORY:
;       Utility
;
; CALLING SEQUENCE:
;
;       CDS [, DIRECTORY] [, /SHOW_STACK] [, /CLEAR_STACK] [, /QUIET]
;
; OPTIONAL INPUT:
;       DIRECTORY - Optional directory
;             Note: IF SHOW_STACK or CLEAR_STACK are set, DIRECTORY is ignored
;                   If present, DIALOG_PICKFILE will not be called.
;                     The working directory will be changed and the present
;                     directory will be placed on the directory stack.
;                   Default: DIRECTORY = DIALOG_PICKFILE(/DIRECTORY, /MUST_EXIST)
;
; KEYWORD PARAMETERS:
;       SHOW_STACK  - Show the stack of directories
;       CLEAR_STACK - Empty the stack of directories
;       QUIET       - Do not print the current directory
;
; EFFECT:
;
; PROCEDURE:
;       The procedure determines if the directory is valid and changes 
;       the working directory.  Before changing the current directory is 
;       stored on the "Past" directory stack.  The "Forward" directory stack
;       is cleared.
;
; EXAMPLE:
;       Change directory to '/remote/data'
;         CDS, '/remote/data'
;
;       Change directory to a directory to be selected from a widget
;         CDS
;
;       Show all directories in the stack.
;         CDS, /show
;
; MODIFICATION HISTORY:
; 	Written by:	L. P. Mix, December 3, 2003
;-

Pro  CDS, dir, show_stack= show_stack, clear_stack = clear_stack, quiet=quiet

COMMON CD_DIRECTORY_STACK, Back_STACK, NBACK, Forward_stack, nforward
on_error, 1		; Return to caller on error

  if n_elements(nback) eq 0 then begin
    nback=0L
    nforward = 0l
  endif
  if keyword_set(show_stack) then begin
    pwd
    print, ' '
    if nback eq 0 then print, ' Past stack is empty.' else begin
      print, ' Past Directories - Use CDB to retrieve'
      print, ' Position - Directory'
      for i = 1, nback do print, i, ' - ', back_stack[i-1]
    endelse
    print, ' '
    if nforward eq 0 then print, ' Forward stack is empty.' else begin
      print, ' Forward Directories - Use CDF to retrieve'
      print, ' Position - Directory'
      for i = 1, nforward do print, i, ' - ', forward_stack[i-1]
    endelse
    return
  endif
  if keyword_set(clear_stack) then begin
    nback=0L
    nforward = 0L
    back_stack=''
    forward_stack=''
    return
  endif
    

  if n_elements(dir) eq 0 then begin
    dir = DIALOG_PICKFILE(/DIRECTORY, /MUST_EXIST)
    if strlen(dir) eq 0 then return
  endif else begin
    dir = strtrim (dir, 2)
    d_valid = file_test(dir[0], /dir)
    if (not d_valid) then begin
      print, '% CDS: Valid directory must be provided: ', dir[0]
      return
    endif
  endelse   
  pwd, current = current, /quiet
  if nforward gt 0 then begin
    nforward = 0L
    forward_stack=''
  endif
  if nback eq 0 then back_stack=[current] else back_stack=[current, back_stack]
  nback = nback+1
  cd, dir[0]
  if not keyword_set(quiet) then pwd
  end
