; ------------------------------------------------------------------------ 
; $Id: cha.pro,v 1.4 2007/06/06 20:03:45 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
 pro Cha, wdfarray, clabel = clab, ylabel= ylab, xlabel = xlab, $
        scale = scale, file = file, append = append, aclabel= aclab, $
        axlab = axlab, aylab = aylab,  quiet = quiet, $
        npoints= np, tzero = tinit, deltat= dt, adt = adt, mdt = mdt
;
;+
; NAME:
;       CHA
;
; PURPOSE:
;       Change the prameters of a WDF array
; CATEGORY:
;       Utility
; CALLING SEQUENCE:
;       CHA, WD1, 
; INPUTS:
;       WD1:      WDF number or an array of numbers of WDF arrays to be changed
;                 or a search string
; KEYWORD PARAMETERS:
;       CLABEL    Replace the dataset comment for array number WD1 
;                 with the string or string variable specified
;       XLABEL    Replace the dataset x-axis label for array number WD1
;                 with the string or string variable specified
;       YLABEL    Replace the dataset y-axis label for array number WD1
;                 with the string or string variable specified
;       APPEND    If set, append the string in CLABEL, XLABEL or YLABEL to
;                 the corresponding label with an intervening space unless
;                 append is a string. 
;       ACLABEL, AXLABEL, AYLABEL - Same as append but only apply to the comment
;                 xlabel or ylabel respectively.
;       NPOINTS   Change the number of points to the specified number --
;                 SEE SIDE EFFECTS
;       DELTAT    Change the interval between points to the specified value --
;                 SEE SIDE EFFECTS
;       TZERO     Change the initial time of the first data point --
;                 the effect is similar to the TSH command
;       ADT       Change the interval between points to the specified value 
;                 and do not change the number of points or their values
;       MDT       Multiply the all abscissa values by the specified value
;                 and do not change the number of points or their value
;       SCALE     Multiply the amplitude by a scale factor.  This is identical
;                 to mul, wd1, 0, scale.
;                 or to set the amplitude to a function of x or y.
;                 example: to set the amplitude to (sin(x)/y) or a user 
;                          specified function use: 
;                   scale = 'sin(x)/y'.
;                   scale = 'My_Function(x, y)' 
;                 Note: The user is required to insure valid data.
;       QUIET     Don't give WARNINGS.
;
;       FILE      Replace the file name with the value in file
; OUTPUTS:
;       NONE    The parameters are stored in the WDF common block
; COMMON BLOCKS:
;       COM.DAT  WDF common block 
; SIDE EFFECTS:
;       If NPOINTS is changed then DELTAT will be changed to allow the data
;         to cover the same interval.
;       If DELTAT is changed then NPOINTS wlll be changed to allow the data
;         to cover the same interval.
;       In both cases:
;          Linear interpolation will be used to calculate new values
;          If the number of points is decreased, then a boxcar filter with 
;            width = ((deltat ratio) +1) is used to smooth the data.
;       Note: If X-Y data is stored in the waveform array, it will be converted
;              to uniformly space abscissa values if npoints, deltat,
;              or adt is specified.
; RESTRICTIONS:
;       WD1 array must be defined and have valid data
; PROCEDURE:
;       See SIDE EFFECTS.  For the other parameters, the new values are 
;          stored in the WDF common block.
; EXAMPLES:
;       Change the comment label of array 4 to inclued the shot number 
;         at the end. Assume shot is in variable shot.
;               Shot= 5791
;               CHA, 4, c= shot, /ac
;   < OR >      CHA, 4, c= shot, /ap ; since only one label is changed
;       Similar to the example above, but use ' - ' as a separator
;               Shot= 5791
;               CHA, 4, c= shot, ac= ' - '
;   < OR >      CHA, 4, c= shot, apppend = ' - '
;       Change the dataset comment or label to 'Machine voltage (MV)' for
;          array 2 and change the amplitude from volts to megavolts
;               CHA, 2, c = 'Machine voltage (MV)', scale = 1e-6
;       Change the abscissa units of arrays 2 through 9 to ns from s.
;               CHA, indgen(8)+2, x = 'Time (ns)' , m = 1e9
;       Change the abscissa units of arrays 2 through 9 to 1.0, change the 
;               abscissa label to 'Time Steps', and change the initial abscissa
;               value to 0.0.
;               CHA, indgen(8)+2, x = 'Time Steps' , ad= 1, tzero = 0.0
; MODIFICATION HISTORY:
;      Original version, L. P. Mix, May 15, 1992
;-
;
on_error, 1
;     include the common block
@com.dat
;
;     see if we are initialized
      if n_elements(initwdf) eq 0 then wdfinit
;
;
;     determine if wdfarray is good
;
      npar = n_params()
      if npar eq 0 then begin
     print, '% Format: CHA, WDF_ARRAY [, CLABEL= clabel] [, SCALE = scale]'
     print, '        [, YLABEL = ylagel] [, XLABEL = xlabel]'
     print, '        [, APPEND = append] [, ACLABEL = aclabel]'
     print, '        [, AXLABEL = axlabel] [, AYLABEL = aylabel]'
     print, '        [, NPOINTS = npoint] [, TZERO = tinit]'
     print, '        [, DELTAT = dt] [, MDT = mdt] [ADT = adt]'
     print, '        [, FILE = file]'
        return
      endif
      nelwdf = n_elements(wdfarray)
      if nelwdf eq 0 then begin
        print, '% CHA: WDF array must be defined'
        return
      endif
      a = size(wdfarray)
      if a(a(0)+1) eq 7 then begin
        if a(0) ne 0 then print, '% CHA: Only the first string is used'
        wdf = wdfarray
        if strtrim(wdf(0), 2) eq '*' then wdf = wdvalid() else $
          wdf = wdvalid(wdf(0))
        if wdf(0) lt 1 then begin
          print, '% CHA: No arrays found using the search string: ', wdf(0)
          return
        endif
        wdfarray = temporary(wdf)
      endif
      apc=0
      apx=0
      apy=0

   if keyword_set(aclab) then begin
      apc = 1 
      apc_st= ' '
      a = size (aclab(0))
      if a(1) eq 7 then apc_st = aclab(0)
   endif else if keyword_set(append) then begin
      apc = 1
      apc_st= ' '
      a = size (append(0))
      if a(1) eq 7 then apc_st = append(0)
   endif
   if keyword_set(axlab) then begin
      apx = 1
      apx_st= ' '
      a = size (axlab(0))
      if a(1) eq 7 then apx_st = axlab(0)
   endif else if keyword_set(append) then begin
      apx = 1
      apx_st= ' '
      a = size (append(0))
      if a(1) eq 7 then apx_st = append(0)
   endif
   if keyword_set(aylab) then begin
      apy = 1
      apy_st= ' '
      a = size (aylab(0))
      if a(1) eq 7 then apy_st = aylab(0)
   endif else if keyword_set(append) then begin
      apy = 1
      apy_st= ' '
      a = size (append(0))
      if a(1) eq 7 then apy_st = append(0)
   endif
   for iarray = 0, nelwdf-1 do begin
      wdf = wdfarray(iarray)
      if wdf lt 1 or wdf gt maxwdfarray then begin
        print, 'wdfarray must be greater than 0 and </=', maxwdfarray
        goto, endarray
      endif 
;     determine if we have valid data
      nel =  wdfnp(wdf-1)
      if nel lt 2 then begin
        Print,' No valid data in wdf array =' , wdf
        goto, endarray
      endif
      if n_elements(file) gt 0 then wdffile(wdf-1)=strtrim(file, 2)
      if n_elements(clab) gt 0 then begin
        if apc eq 0 then wdfcomment(wdf-1)= clab else $
          wdfcomment(wdf-1) =wdfcomment(wdf-1)+apc_st+strtrim(clab, 2)
      endif
      if n_elements(xlab) gt 0 then begin
        if apx eq 0 then wdfxlabel(wdf-1)= xlab else $
          wdfxlabel(wdf-1)= wdfxlabel(wdf-1) + apx_st + strtrim(xlab, 2)
      endif
      if n_elements(ylab) gt 0 then begin
        if apy eq 0 then wdfylabel(wdf-1)= ylab else $
          wdfylabel(wdf-1)= wdfylabel(wdf-1) + apy_st + strtrim(ylab, 2)
      endif
      if ((n_elements(tinit) eq 0) and (n_elements(np) eq 0) and $
      (n_elements(dt) eq 0) and ( n_elements(adt) eq 0) and $
      (n_elements(mdt) eq 0) and (n_elements(scale) eq 0) ) then goto, endarray
;
;     we need to modify array elements
;
      if (n_elements(np) eq 0) and $
        (n_elements(dt) eq 0) and ( n_elements(adt) eq 0) then begin
        get_wdfarray, array, wdf, /xy
        if n_elements(array) eq nel+2 then xydata = 0 else xydata=1
        if n_elements(mdt) gt 0 then begin
          if xydata eq 0 then array(0:1) = mdt(0)*array(0:1) else $
             array(0:nel-1) = mdt(0)*array(0:nel-1)
        endif
        if n_elements(tinit) gt 0 then begin
          if xydata eq 0 then array(0) = tinit(0) else $
            array(0:nel-1) = array(0:nel-1) - (array(0)-tinit(0))
        endif
        if n_elements(scale) gt 0 then begin
          ss = size(scale(0))
          if ss(1) eq 7 then begin
            if xydata eq 0 then begin
              x = findgen(nel)*array(1)+ array(0)
              y = array(2:*)
              ok = execute('array(2:*) = '+scale(0))
            endif else begin
              x = array(0:nel-1)
              y = array(nel:nel-1+nel)
              ok = execute('array(nel:*) = '+scale(0))
            endelse
            x = 0
            y = 0
          endif else begin
            if xydata eq 0 then array(2:*) = scale(0)*array(2:*) else $
            array(nel:*) = scale(0)*array(nel:*)
          endelse
        endif
        put_wdfarray, array, wdf, xydata = xydata, /no_copy
        goto, endarray
      endif 
      get_wdfarray, array, wdf, /xy
      if n_elements(array) eq nel+2 then xydata = 0 else xydata=1

      if n_elements(tinit) gt 0 then begin
          if xydata eq 0 then array(0) = tinit(0) else $
            array(0:nel-1) = array(0:nel-1) - (array(0)-tinit(0))
      endif

;     note np and dt cannot both be changed!
      if (n_elements(np) gt 0) or (n_elements(dt) gt 0) then begin
        if xydata eq 1 then begin
          if not keyword_set(quiet) then begin
            print, '% CHA: X-Y DATA is being changed to UNIFORM DATA.'
            print, '       WDF array = ', strtrim (wdf)
          endif
          space = array(0:nel-1)
          image = array(nel:nel+nel-1)
          i2w, space, image, delta, /get_delta, endpoint = endpoint
          start = space(0)
          old_del = delta
          nel= round((endpoint-start)/delta)
          nels = n_elements(space)
          stopp = (start+nel*delta)*1.000002
          nel = nel+1
        endif else begin
          start = array(0)
          delta = array(1)
          old_del = delta
          nel= n_elements(array)-2
          nels = nel
          image = array(2:*)
          space = findgen(nel)*array(1)+array(0)
          stopp = space(nel-1)*1.000002
        endelse
        if n_elements(np) gt 0 then $
          if np gt 1 then delta=delta*(nel-1)/(np-1)
        if n_elements(dt) gt 0 then if dt gt 0 then delta = dt
        if delta gt  1.5*old_del then $
          array(2:*) =smooth(array(2:*), long(delta/old_del +1.))
        put2uniform, space, image, nels, start, delta, stopp, array
        if n_elements(dt) eq 0 then begin
          if n_elements(array) gt np+2L then array = temporary(array(0:np+1L))
        endif
      endif
      if n_elements (adt) gt 0 then array(1) = adt
      if n_elements (mdt) gt 0 then begin
        if mdt eq 0 then mdt = 1
        array(0:1) = mdt*array(0:1)
      endif
      if n_elements(scale) ne 0 then array(2:*) = array(2:*) * scale(0)
      put_wdfarray, array, wdf, /no_copy
endarray:
      endfor
      return
      end
