; ------------------------------------------------------------------------ 
; $Id: com.pro,v 1.12 2010/08/18 18:58:18 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
 Pro  Com, wdf1, wdf2, base = base, bslope = bslope, amp = amp, $
        bline = bline, tshift = tshift, leg = leg, noleg = noleg, $
        errlim = errlim, initdel = initdel, quiet = quiet, out = out, $
        title = title, sdev = sdd , rsdev = sdr, correlate = correl, $
        left = left, right = right, iterations = iteration, $
        lcharsize = lcharsize, charsize = charsize, $
        only_tshift = only_tshift, weight = weightin, _extra = extrastuff
;+
;  Compares two WDF arrays and attempts to find amplitude and time shifti
;  values which minimize the differences (least squares) between the arrays.
;  The fit of arrays, A and B, is of the form:
;       A(t) = amp*B(t-tshift) + bline(1)*t + bline(0).
;  The default comparison assumes bline(0) and bline(1) are zero.
;  Fit assumes continuous data.  Non-uniform waveforms are first put on a uniform
;  abscissa spacing.  Both waveforms are then placed on the same uniform spacing 
;  using the minimum spacing.
;  Following the fit, the left and right hand sides of this equation are
;  plotted.
;  NOTE: The two maximum WDF arrays are used for the plots if OUT is not specified.
;        The next WDF array is used to return the actual values used in weight.
;
;  FORMAT:  COM, wd1, wd2 [, /BASE] [, /BSLOPE] [, AMP = amp] 
;    [, SDEV = sdev] [, RSDEV = RSDEV] [, BLINE = bline]
;    [, TSHIFT = tshift]   [,LEG = leg][, /NOLEG] [, OUT = out]
;    [, ERRLIM = errlim]  [, INITDEL = initdel] [, QUIET = quiet]
;    [, CORRELATE = correlate] [, TITLE = title] [, /ONLY_TSHIFT]
;    [, LEFT = left] [, RIGHT = right] [, ITERATION = iteration]
;    [, WEIGHT = WEIGHT] [, LCHARSIZE = lcharsize] [, _EXTRA = extrastuff]
;  where:
;    wd1, wd2 - WDF array numbers for the arrays to be compared
;    base - nonzero value indicates a bline(0) will be allowed to  
;           vary from zero
;    bslope - nonzero value indicates a bline(1) and bline(0) will 
;             be allowed to  vary from zero
;    amp - returned with the calculated value of amp
;    bline - returned with the calculated value of bline
;    amp - returned with the calculated value of tshift
;    sdev - Standard deviation 
;           SDEV = SQRT( TOTAL(WEIGHT * (U-V)^2) /TOTAL(WEIGHT))
;           where:
;             U are values from WD1 (Stored in out[0])
;             V are values scaled from WD2 (Stored in out[1])
;             WEIGHT - 1 or user provided value
;    rsdev - Standard deviation relative to the peak amplitude
;    leg - a two element vector specifying the location for the 
;          best fit parameters on the plot as a fraction of the 
;          plot range.  Default value is [0.04, 0.94]
;    noleg - nonzero value indicates that the no best fit
;            parameters will be printed on the plot.  If this 
;            keyword is zero or omitted, the best fit parameters 
;            will be output to the graphics window using the 
;            values of leg.
;    errlim - error limit for convergence; default is 3.125% of the
;             minimum point spacing
;    out - The fitted arrays and weight are stored in the following:
;             if out is a one element array - out = [out, out+1, out+2]
;             if out is a two element array - out = [out[0], out[1], get_maxwdf()-2]
;             if out is not specified or invalid -
;                out= [-1, 0, -2]+ get_maxwdf()
;           If weight is specified, it is stored in out[2]
;    initdel - initial delta for the iteration.  
;              Second array is shifted this amount. (tsh, wd2, initdel)
;              Default is to perform cross correlation of the two arrays
;     quiet  - If present and not zero, no output will be made.
;    correlate - Correlation coefficient between the two arrays
;     title  - If present this is used for the plot title 
;    LEFT    - If present, the waveforms will have this as a minimum abscissa
;              value.  Windowing is done before the fit.
;              Default:  All data is used
;    RIGHT   - If preseht, the waveforms will have this as a maximum abscissa
;              value.  Windowing is done before the fit.
;              Default:  All data is used
;    
;    weight  - If present, the comparison will be weighted by weight.
;              If weight values do not overlap WD1 or if the defined value
;                is LE 0, the value of weight is set to 1.00
;        NOTE: Present version only allows weights with no baseline slope.
;              (ie. bline(1) = 0.0)
;              Weight must be a valid wdf array or -1, -2, -101, or -102
;              If weight = -1, then weight = sqrt(wd1>0.0)
;              If weight = -2, then weight = (wd1>0.0)
;              If weight = -101, then weight = 1./sqrt(wd1>0.0)
;              If weight = -102, then weight = 1./(wd1>0.0)
;              if Weight eq 0 then weight = 1.0
;              Default:  Weight = 1.0
;              If specified, actual data is in get_maxwdf-2.
;
;     ONLY_TSHIFT - Use amplitude factor of 1.0 and shift the time.
;                   If BASE is set, the baseline will be shifted.
;     LCHARSIZE - Character size for the fit parameters
;                 To change the legend for the plo command, you must use:
;                   PLO, /set , lcharsize = value
;                 before calling the COM command.
;     _extra - Any keyword understood by the plo command
;  Examples:
;    Compare array 1 to array 2 with no baseline adjustment
;      com, 1, 2
;    Compare array 1 to array 2 with sloping baseline and store the amplitude
;    in r, the time shift in s, the baseline parameters in basepar.
;    Position the legend in the lower right corner. 
;      com, 1, 2, /bs, a=r, t=s,  bl=basepar,  leg = [0.7, 0.4]
;-
;  Common blocks
;
@com.dat
;
      on_error, 1
;
      if n_elements(initwdf) eq 0 then wdfinit
      nparams = n_params()
      iteration = 0
      if nparams lt 2 then begin
      print,'% Format:  COM, wd1, wd2 [, /BASE] [, /BSLOPE] [, AMP = amp]'
      print,'           [, SDEV = sdev] [, RSDEV = RSDEV]'
      print,'           [, BLINE = bline] [, TSHIFT = tshift] [,LEG = leg]'
      print,'           [, /NOLEG] [, ERRLIM = errlim] [, /QUIET]'
      print,'           [, INITDEL = initdel] [, OUT = out] [, WEIGHT = weight]'
      print,'           [, TITLE = title]  [, _extra = extrastuff]'
        return
      endif
;
      if n_elements(wdf1) eq 0 or  n_elements(wdf2) eq 0 then begin
        print, 'Parameters to COM must be defined'
        return
      endif
      if wdf1 lt 1 or wdf1 gt maxwdfarray then begin
        print, 'wdf array must be greater than 0 and </=', maxwdfarray
        return
      endif
;
      np1 = wdfnp(wdf1-1)
      if np1 eq 0 then begin
        print, 'No data in the first array'
        return
      endif
;
      if wdf2 lt 1 or wdf2 gt maxwdfarray then begin
        print, 'wdf array must be greater than 0 and </=', maxwdfarray
        return
      endif
      np2 =  wdfnp(wdf2-1)
      if np2 eq 0 then begin
        print, 'No data in the second array'
        return
      endif
      get_wdfarray, array1, wdf1
      get_wdfarray, array2, wdf2
;
      amp = 1.0
      tshift = 0.0
      bline = make_array(2)
      sdd = 0.0
      sdr = 0.0
      ier = 0L
      np=0
      if n_elements(weightin) eq 0 then weight = 0 else begin
        if weightin le 0 then begin
          weight = weightin
          if weight lt -102 then weight = 0
          if weight lt -2 and weight gt -101 then weight = 0
        endif else begin
          weight = wdvalid (weightin, /quiet, /valid, np= np)
          if np gt 1 then begin
  Print, '% COM: Unique description must be provided for WEIGHT', weight
            return
          endif
          if np eq 0 then weight = 0
        endelse
      endelse
      if n_elements(base) eq 0 then base = 0
      if n_elements(out) eq 0 then $
        out = [maxwdfarray-1, maxwdfarray, maxwdfarray-2] else begin
        if n_elements(out) eq 1 then out= [out, out+1, out+2]
        if min(out, max = mmm) lt 1 or mmm gt maxwdfarray then $
          out =[maxwdfarray-1, maxwdfarray,  maxwdfarray-2]
        if n_elements(out) eq 2 then out = [out, maxwdfarray-2]
      endelse
      
      if n_elements(bslope) eq 0 then bslope = 0
      if bslope ne 0 then ifit = 2L else $
        if base ne 0  then ifit = 1L else ifit = 0L
      if weight ne 0 and ifit gt 1 then begin
        winformation, ['Weight can not be used with sloping baseline', $
                 'Slope set to zero']
                    
        ifit = 1L
      endif
      if n_elements(left) gt 0 or n_elements(right) gt 0 then begin
        if n_elements(left) eq 0 then begin
          ist1 = 2
          ist2 = 2
        endif else begin
          st = left[0]  
          if st le array1[0] then ist1 = 2 else $
            ist1 = 2+ round((st-array1[0])/array1[1])
          if st le array2[0] then ist2 = 2 else $
            ist2 = 2+ round((st-array2[0])/array2[1])
        endelse
        nel1 =  n_elements(array1)-3
        nel2 =  n_elements(array2)-3
        e1 = nel1*array1[1] +array1[0]
        e2 = nel2*array2[1] +array2[0]
        if n_elements(right) eq 0 then begin
          iend1 = nel1+2
          iend2 = nel2+2
        endif else begin
          en = right[0]  
          if en ge e1 then iend1 = nel1+2 else $
            iend1 = 2+ round((en-array1[0])/array1[1])
          if en ge e2 then iend2 = nel2+2 else $
            iend2 = 2+ round((en-array2[0])/array2[1])
        endelse
        if ist1 gt 2 then array1[0] = array1[0]+array1[1]*(ist1-2)
        if ist2 gt 2 then array2[0] = array2[0]+array2[1]*(ist2-2)
        if iend1-ist1 lt 2 then begin
          print, '% COM:  Less than 3 points for array 1'
          print, '  Start and end index values:', ist1, iend1
          return
        endif
        if iend2-ist2 lt 2 then begin
          print, '% COM:  Less than 3 points for array 1'
          print, '  Start and end index values:', ist2, iend2
          return
        endif
        array1 = [array1[0:1], array1[ist1:iend1]]
        array2 = [array2[0:1], array2[ist2:iend2]]

      endif
      comidl, arrayu,arrayv,array1,array2,amp,tshift,bline,sdd,sdr,ifit, $
        ier, initdel = initdel, errlim = errlim, correl = correl, $
        only_tshift = only_tshift,  weight=weight
      amp = float(amp)
      tshift = float(tshift)
      bline= float(bline)
      sdd = float(sdd)
      sdr = float(sdr)
      if n_elements (leg) gt 1 then legp= leg(0:1) else legp = [0.04, 0.94]
      name1 = strtrim(strmid(wdfcomment[wdf1-1], 0, 12),2)
      name2 = strtrim(strmid(wdfcomment[wdf2-1], 0, 12),2)
      npv = n_elements(arrayv)-2
      npu = n_elements(arrayu)-2
      if npu lt 3 or npv lt 3 then begin
        amp = 0.0
        tsh = 0.0
      endif
      if (npu gt 2 and npv gt 2 ) then begin
        if n_elements(title) eq 0 then $
          title = 'Comparison of '+ name1 + ' and '+ name2
        i2w, arrayu[0:1], arrayu[2:*], out[0], clabel = wdfcomment[wdf1-1], $
         xlabel = wdfxlabel(wdf1-1), ylabel = wdfylabel(wdf1-1), $
         file = wdffile (wdf1-1)
        i2w, arrayv[0:1], arrayv[2:*], out[1], clabel = wdfcomment[wdf2-1], $
         xlabel = wdfxlabel(wdf2-1), ylabel = wdfylabel(wdf2-1), $
         file = wdffile (wdf2-1)
        if n_elements(weight) gt 4 then i2w, weight[0:1], weight[2:*], $
          out[2], clabel= 'Weights', xlabel = wdfxlabel(wdf1-1), $
          ylabel = 'Amplitude'
        if keyword_set (quiet)  then return
        if n_elements(lcharsize) eq 0 then lcharsize = 1.0

        plo, out[0:1], /over, _extra = extrastuff, nextpos = nextpos, $
          title= title, leg = legp, charsize = charsize
;
        if !p.charsize ne 0 then charsize = charsize *!p.charsize
        legch = charsize*lcharsize
        s = make_array(9,/string)
        s[0] = string ('Scale Factor =', amp)
        s[1] = string ('Time Shift =', tshift)
        if ifit eq 0 then s[2] = 'Baseline Mode =     None' else $
        if ifit eq 1 then s[2] = 'Baseline Mode =    Level' else $
                          s[2] = 'Baseline Mode =    Slope'
        i = 3
        if ifit gt 0 then begin
          s[i] = string('Baseline Level =', bline(0))
          i = i+1
        endif
        if ifit eq 2 then begin
          s[i] = string('Baseline Slope =', bline(1))
          i = i+1
        endif
        s[i] = string('Std Deviation =', sdd )
        i = i+1
        s[i] = string('Norm Std Dev =', sdr * 100, ' %')
        i = i+1
        s[i] = string('Correlation =', correl)
        i = i+1
        s[i] = string('# of Iterations =', ier)
        iteration = ier
        print, 'FIT PARAMETERS'
        for j = 0, i do print,'  ', s[j]
        if keyword_set(noleg) then return 
        legendp, [' ', 'FIT PARAMETERS', s[0:i]], position = nextpos, /data, $
            charsize = legch
      endif else print, ' Less than 3 points in the fit'
      end
