; ------------------------------------------------------------------------ 
; $Id: comnt.pro,v 1.6 2010/08/18 15:09:11 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
 Pro  Comnt, wdf1, wdf2, amp = amp, tshift = tshift, leg = leg, $
       noleg = noleg, quiet = quiet, out = out, clabel = clab, $
       title = title, sdev = sdd , rsdev = sdr, correlate = correl, $
       ylog = ylog, weights = weights,  _extra = extrastuff
;+
;  NAME:
;        COMNT
;
;  PURPOSE:
;     Calculates the amplitude multiplier for the second waveform to minimize
;       standard deviation between two WDF arrays.  No time shift is allowed.
;     If the two waveforms are not on the same time base, linear interpolation
;       is used to populate the abscissa values.
;     The standard deviation that is minimized is 
;       SD = sqrt(total ((WD1 - Amp*WD2)**2 * Weight)/total(Weight) ) linear
;       SD = exp(sqrt(Total((alog(wd1)-alog(wd2)-alog(Amp))**2*Weight)/ 
;               total(weight)))
;       Note: For the log case the %-Standard Deviation is (SD-1)*100.
;             If the amplitude is multiplied and divided by SD, then about
;             68% of the data should fall between the two values.
;       where Weight is a user specified weight (Default = 1)
;         and N is the number of points.
;       If log is set, WD1 and WD2 are replaced by alog(wd1) and alog(amp*wd2)
;         in the equation above.  
;     By default the result is plotted.
;     WD1 and WD2 are not modified. 
;    
;  FORMAT:  COMNT, wd1, wd2  [, OUT = out] [, AMP = amp] [, TSHIFT = tshift]
;     [,LEG = leg][, /NOLEG] [, OUT = out] [, /YLOG] [, WEIGHTS = weights]
;     [, SDEV = sdev] [, RSDEV = RSDEV] [, QUIET = quiet] [, CLABEL = clabel]
;     [, CORRELATE = correlate] [, TITLE = title] [, _EXTRA = extrastuff]
;
;  where:
;    wd1, wd2 - WDF array numbers for the arrays to be compared
;               or a unique name for the wdf arrays.
;               WD2 is moved to out for the calculations.
;               xfr, wd2, out
;    out    - The modified values of wd2 are stored in out
;             Default value = get_maxwdf()
;    amp    - returned with the calculated value of amp that minimizes the
;             standard deviation
;    Weights- Waveform array number of weights used for the least squares fit.
;             Default - Weights = 1.0 for all points
;             If Weights is not on the same time scale as WD1, then get_value()
;             will be used to interpolate to the WD1 abscissa values.  
;             Missing values are 1.0d0
;    leg    - a two element vector specifying the location for the 
;             best fit parameters on the plot as a fraction of the 
;             plot range.  Default value is [0.04, 0.94]
;    noleg  - nonzero value indicates that the no best fit parameters will 
;             be printed on the plot.  If this keyword is zero or omitted,
;             the best fit parameters will be output to the graphics window
;             using the values of leg.
;    tshift - if present and not zero wd2 is time shifted
;             tsh, out, tshift
;    ylog   - If set, the fit will use log values for the fit.
;             ie WD1 becomes ALOG(WD1) and Amp*WD2 becomes ALOG(Amp*WD2)
;                in the least square fit equation for SD.
;             If Ylog >0 and Ylog < 0.5 then the range of the two arrays
;               will be limited to Ylog*max(y) where max(y) is the maximum value
;               in each array.
;             If ylog > .5 or < 0 then, the range will be limited to 8 orders 
;               of magnitude
;    sdev   - Standard deviation - See SD in the PURPOSE above.
;              
;    rsdev  - Standard deviation relative to the peak amplitude (mx(wd1)) in
;             the case of a linear scale and (SD-1)*100 in the case of a log
;             fit
;    quiet  - If present and not zero, no output will be made.
;    correlate - Correlation coefficient between the two arrays
;                correlate = correlate(wd1, out) or the logs if ylog is set.
;    title  - If present this is used for the plot title 
;    clabel - If present this is used for the comment label for the scaled waveform.
;    _extra - Any keyword understood by the plo command
;
;  Technique:
;             The standard deviation, SD, is give by:
;               SD = SQRT(Total((y1 - AMP*y2)^2)/nel)
;               where Y1 and Y2 are the amplitudes of WD1 and WD2 on the same 
;               time base and nel is the number of points
;             Taking the derivative with respect to the amplitude, we calculate
;               AMP = TOTAL(Y1*Y2) / TOTAL(Y2*Y2)
;
;  Examples:
;    Compare array 1 to array 2 with no baseline adjustment
;      comnt, 1, 2
;
;    Compare array 3 to array 6 with a log scale
;      comnt, 3, 6, /ylog
;
;    Compare array 2 to array 3 with weights in array 9
;      comnt, 2, 3, weights = 9
;     
;    Compare 'xrd1' to array 4
;      comnt, 'xrd1', 4
;
;-
;  Common blocks
;
@com.dat
;
      on_error, 1
      sdd = 0
      sdr = 0
      correl = 0
      amp = 0

      if n_elements(initwdf) eq 0 then wdfinit
      nparams = n_params()
      if nparams lt 2 then begin
        print,'% Format:  COMNT, wd1, wd2 [, OUT = out] [, AMP = amp]'
        print,'           [, TSHIFT = tshift] [, OUT = out] [, /YLOG]'
        print,'           [,LEG = leg] [, /NOLEG] [, /QUIET] [CLABEL = clabel]'
        print,'           [, SDEV = sdev] [, RSDEV = RSDEV] [, TITLE = title]'
        print,'           [, CORRELATE = correlate] [, _extra = extrastuff]'
        return
      endif
;
      if n_elements(wdf1) eq 0 or  n_elements(wdf2) eq 0 then begin
        print, '% COMNT: Parameters must be defined'
        return
      endif
      wd1 = wdvalid(wdf1, /valid, /quiet, npoint = npoint)
      if npoint ne 1 then begin
        if npoint eq 0 then print, $
          '% COMNT: Not a valid waveform array: ', strtrim (wdf1, 2) $ 
        else print, '% COMNT: Multiple values found for: ', strtrim (wdf1, 2), $
          ', Values= ', wd1
        return
      endif
;
      wd2 = wdvalid(wdf2, /valid, /quiet, npoint = npoint)
      if npoint ne 1 then begin
        if npoint eq 0 then print, $
          '% COMNT: Not a valid waveform array: ', strtrim (wdf2, 2) $ 
        else print, '% COMNT: Multiple values found for: ', strtrim (wdf2, 2), $
          ', Values= ', wd2
        return
      endif
;
      if n_elements(out) eq 0 then out = maxwdfarray else out = out[0]
      if out lt 1 or out gt maxwdfarray then out = maxwdfarray
      xfr, wd2, out

      if n_elements (tshift) eq 0 then tshift = 0.0 else tshift = tshift[0]
      if tshift ne 0.0 then tsh, out, tshift 
      
;     calculate the start and end points
      max1 = wdfmaxx(wd1-1)
      max2 = wdfmaxx(out-1)

      ist = max(wdfminx([wd1, out]-1))
      iend = min([max1, max2])
      if iend -ist le 0 then begin
        print, '% COMNT: No overlap in the data. ', wdf1, wdf2, ist, iend
        return
      endif

      get_wdfarray, array1, wd1, /xydata
      get_wdfarray, array2, out, /xydata

      nel = wdfnp(wd1-1)
      np2 =  wdfnp(out-1)

      t1 = size(array1, /type)
      t2 = size(array2, /type)
      if t1 lt t2 then begin
        if t2 eq 4L then array1 = float(array1) else $
          if t2 lt 4L then array1 = long(array1) else $
          if t2 eq 5L then array1 = double(array1) else array1=complex(array1)
      endif
      match_only = 1
      sortx = 1
      get_common_time, array1, array2, max1, max2, nel, np2, $
           xydata, match_only = match_only, sortx = sortx
      if n_elements(array1) lt 4 then begin
        print, '% COMNT: No comparison due to lack of overlap'
        return
      endif

      if xydata eq 0 then begin
        y1 = array1(2:*)
        y2 = array2(2:*)
        xval = array1[0]+array1[1]*findgen(nel)
      endif else begin
        y1 = array1(nel:*)
        y2 = array2(nel:*)
        xval = array1[0:nel-1]
      endelse

      if n_elements(weights) eq 0 then lcwei = 0  else begin
        lcwei = 1
        wei = wdvalid (weights[0], /valid, npoint = npoint, /quiet)
        if npoint ne 1 then begin
          if npoint eq 0 then print, $
            '% COMNT: Not a valid waveform array for WEIGHTS: ', $
               strtrim (weights[0], 2) $
          else print, '% COMNT: Multiple values found for WEIGHTS: ', $
             strtrim (weights[0], 2), ', Values= ', wei
          return
        endif
        weigh = get_value(wei, xval, missing = 1.0)
      endelse
      if keyword_set(ylog) then begin
        if ylog lt 0 or ylog ge 0.5 then begin
          y1 = y1 > 1e-8*max1
          y2 = y2 > 1e-8*max2
        endif else begin
          y1 = y1 > ylog*max1
          y2 = y2 > ylog*max2
        endelse
        ly1 = alog(y1)
        ly2 = alog(y2)
        if lcwei eq 0 then begin
          ty1 = total(ly1, /double)
          ty2 = total(ly2, /double)
; solve for the log of the amplitude
          amp = (ty1-ty2)/nel
          sdd = ly1-ly2 - amp
          sdd = exp(sqrt(total(sdd*sdd, /double)/nel))
        endif else begin
          ty1 = total(ly1*weigh, /double)
          ty2 = total(ly2*weigh, /double)
          tw = total (weigh, /double)
; solve for the log of the amplitude
          amp = (ty1-ty2)/tw
; calculate log(y1) - log(amp*y2)
          sdd = ly1-ly2 - amp
          sdd = exp(sqrt(total(sdd*sdd*weigh, /double)/tw))
        endelse
        amp = exp(amp)
        y2 = amp*y2
        correl= correlate(y1, y2)
        sdr = sdd-1.0
      endif else begin
        if lcwei eq 0 then begin
          ty1 = total(y1*y2, /double)
          ty2 = total(y2*y2, /double)
          amp = ty1/ty2
          y2 = amp*y2
          sdd = y1-y2
          sdd = sqrt(total(sdd*sdd, /double)/(nel))
        endif else begin
          ty1 = total(y1*y2*weigh, /double)
          ty2 = total(y2*y2*weigh, /double)
          tw = total (weigh, /double)
          amp = ty1 /ty2
          y2 = amp*y2
          sdd = y1-y2
          sdd = sqrt(total(sdd*sdd*weigh, /double)/tw)
        endelse
        sdr = sdd/max(abs(y1))
        correl= correlate(y1, y2)
      endelse
      if keyword_set(xydata) then array2[nel:*] = y2 else array2[2:*] = y2
      
      if n_elements (leg) gt 1 then legp= leg(0:1) else legp = [0.04, 0.94]
      name1 = strtrim(strmid(wdfcomment(wd1-1), 0, 12),2)
      name2 = strtrim(strmid(wdfcomment(out-1), 0, 12),2)
      if n_elements(title) eq 0 then $
        title = 'Comparision of '+ name1 + ' vs '+ name2
      if n_elements(clab) eq 0 then clab = $
        'Scaled Value of '+wdfcomment(out-1)+' - Amplitude_scale='+strtrim(amp,2) $
        else clab = strtrim(clab[0], 2)

      xlab = wdfxlabel(wd1-1)
      ylab = wdfylabel(wd1-1)
      file = wdffile(wd1-1)

      put_wdfarray, array2, out, xydata = xydata, clab = clab, $
        file = file, xlab = xlab, ylab= ylab, /no_copy
      array2 = 0
      if keyword_set (quiet)  then return
      plo, [wd1,out], /over, _extra = extrastuff, nextpos = nextpos, $
          title= title, leg = legp, xrange = xr, ylog = ylog, nchar = 28
;
      s = make_array(5,/string)
      s(0) = 'COMNT: Calculation'
      s(1) = 'Amplitude Factor ='+strtrim( amp, 2)
      s(2) = 'Std Deviation = '+ strtrim(sdd ,2)
      s(3) = 'Norm Std Dev = '+ strtrim(sdr * 100,2)+ ' %'
      s(4) = 'Correlation = '+strtrim( correl, 2)
      print, 'FIT PARAMETERS'
      for j = 0, 4 do print,'  ', s(j)
      if keyword_set(noleg) then return 
      legendp, [' ', 'FIT PARAMETERS', s(0:4)], position = nextpos, /data
      end
