; ------------------------------------------------------------------------ 
; $Id: comte.pro,v 1.4 2010/08/18 18:59:36 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
 Pro  Comte, wdf1, wdf2, base = base,  amp = amp, $
        tshift = tshift, leg = leg, noleg = noleg, rdelta = rdelta, $
        quiet = quiet, out = out, delta = delta, scale = scalein,  $
        title = title, sdev = sdd , rsdev = sdr, correlate = correl, $
         _extra = extrastuff
;+
;  Calculates the standard deviation between two WDF arrays 
;    This is designed for comparison of theoretical and experimental data.
;
;    The amplitude of the second waveform is determined by first filtering
;      the second waveform with a low pass filter, ie.,
;      filt, wdf2, out, delta = 0.361* fwhm(wdf2, /quiet, base=0)
;      mx(out) is adjusted to equal mx(wd1) and the standard deviation (SD)
;      is calculated using the x values from the waveform with the fewest
;      number of points (np) in the region of overlap.  The SD is calculated:
;          SD = sqrt( total (wd1-wd2)^2/(np-1))
;      NOTE:  SD is calculated using actual waveform values and not the 
;             filtered values.
;
;    WD1 and WD2 are not modified. 
;    
;  FORMAT:  COMTE, wd1, wd2 [, BASE = base] [, TSHIFT = tshift] [, AMP = amp] 
;     [, SDEV = sdev] [, RSDEV = RSDEV] [,SCALE = scale] [, QUIET = quiet]
;     [,LEG = leg][, /NOLEG] [, OUT = out] [, DELTA = delta] 
;     [, RDELTA = rdelta] [, CORRELATE = correlate] [, TITLE = title] 
;     [, _EXTRA = extrastuff]
;
;  where:
;    wd1, wd2 - WDF array numbers for the arrays to be compared
;               WD1 expected to be theory with low noise.
;               WD2 is expected to be experiment
;               WD2 is moved to out for the calculations.
;               xfr, wd2, out
;    base   - if present and not zero the the following is performed:
;             sub, out, 0, base
;    tshift - if present and not zero wd2 is time shifted
;             tsh, out, tshift
;    scale  - If scale is specified then wd2 will be scaled by this value
;             mul, out, 0, scale
;             If scale is not specified or zero then delta or rdelta will
;             be used to filter the data and the resultant amplitude 
;             used to scale wd2 to wd1.
;    delta  - if specified wd2 will be filtered with a delta given by delta.
;             ie filt, out, del= delta
;             Default = 0.0
;    rdelta - If delta is zero, then wd2 will be filtered with a delta given
;             by rdelta*fwhm(wd2, b=0, /quiet)
;             ie filt, out, del= rdelta*fwhm(wd2, b=0, /quiet)
;             Default = 0.361
;             The default value was chosen to give an 0.1% change in amplitude
;             for a gaussian pulse.
;             If RDELTA = 0 and DELTA = 0 or missing, then no filtering
;             will be performed.
;    amp    - returned with the calculated value of amp so that
;             mx(out) = mx(wd1)
;    sdev   - Standard deviation 
;    rsdev  - Standard deviation relative to the peak amplitude (mx(wd1))
;    leg    - a two element vector specifying the location for the 
;             best fit parameters on the plot as a fraction of the 
;             plot range.  Default value is [0.04, 0.94]
;    noleg  - nonzero value indicates that the no best fit
;             parameters will be printed on the plot.  If this 
;             keyword is zero or omitted, the best fit parameters 
;             will be output to the graphics window using the 
;             values of leg.
;    out    - The modified values of wd2 are stored in out
;             Default value = get_maxwdf()
;    quiet  - If present and not zero, no output will be made.
;    correlate - Correlation coefficient between the two arrays
;     title  - If present this is used for the plot title 
;     _extra - Any keyword understood by the plo command
;  Examples:
;    Compare array 1 to array 2 with no baseline adjustment
;      comte, 1, 2
;
;    Compare array 3 to array 6 with a filter given by 0.5 of the fwhm.
;      comte, 3, 6, rdel=0.5
;-
;  Common blocks
;
@com.dat
;
      on_error, 1
      sdd = 0
      sdr = 0
      correl = 0
      amp = 0
;
      if n_elements(initwdf) eq 0 then wdfinit
      nparams = n_params()
      if nparams lt 2 then begin
        print,'% Format:  COMTE, wd1, wd2 [, BASE = base] [, TSHIFT = tshift] '
        print,'           [, AMP = amp] [, SDEV = sdev] [, RSDEV = RSDEV]'
        print,'           [,LEG = leg] [, /NOLEG] [, OUT = out] [, /QUIET] '
        print,'           [, DELTA = delta] [, RDELTA = rdelta]'
        print,'           [, CORRELATE = correlate]'
        print,'           [, TITLE = title]  [, _extra = extrastuff]'
        return
      endif
;
      if n_elements(wdf1) eq 0 or  n_elements(wdf2) eq 0 then begin
        print, '% COMTE: Parameters must be defined'
        return
      endif
      if wdf1 lt 1 or wdf1 gt maxwdfarray then begin
        print,'% COMTE: WDF array 1 must be greater than 0 and </=',maxwdfarray
        return
      endif
;
      np1 = wdfnp(wdf1-1)
      if np1 eq 0 then begin
        print, '% COMTE: No data in the first array'
        return
      endif
;
      if wdf2 lt 1 or wdf2 gt maxwdfarray then begin
        print,'% COMTE: WDF array 2 must be greater than 0 and </=',maxwdfarray
        return
      endif
      np2 =  wdfnp(wdf2-1)
      if np2 eq 0 then begin
        print, '% COMTE: No data in the second array'
        return
      endif
;
      if n_elements(out) eq 0 then out = maxwdfarray
      if out lt 1 or out gt maxwdfarray then out = maxwdfarray
      
      xfr, wdf2, out
      if n_elements(base) eq 0 then base = 0.0
      if base ne 0.0 then sub, out, 0, base
 
      if n_elements(tshift) eq 0 then tshift = 0.0
      if tshift ne 0.0 then tsh, out, tshift 
      
      if n_elements(scalein) eq 0 then scale = 0.0 else scale = scalein(0)
      if scale gt 0.0 then mul, out, 0, scale else begin
        if n_elements(delta) eq 0 then  delta = 0.0
        if delta le 0 then begin
          if n_elements (rdelta) eq 0 then rdelta = 0.361
          if rdelta eq 0.0 then rdelta = 0.361
          delta = fwhm(out, b=0, /quiet)*rdelta
        endif
        filt, out, delta = delta
        scale = wdfmax(wdf1-1)/wdfmax(out-1)
        xfr, wdf2, out
        if base ne 0.0 then sub, out, 0, base
        if tshift ne 0.0 then tsh, out, tshift
      endelse
      amp = scale
      mul, out, 0, scale
;     calculate the start and end points
      ist = max(wdfminx([wdf1, out]-1))
      iend = min(wdfmaxx([wdf1, out]-1))
      if iend -ist le 0 then begin
        print, '% COMTE: No overlap in the data. ', wdf1, wdf2, ist, iend
        return
      endif
      w2i, x1, y1, wdf1
      w2i, x2, y2, out
      ok1= where (x1 ge ist and x1 le iend, nok1) 
      ok2= where (x2 ge ist and x2 le iend, nok2) 
      
      if nok1 le nok2 then begin
        np = nok1
        y1 = y1(ok1)
        y2 = get_value(out, x1(ok1))
        xr = [x1(0), x1(nok1-1)]
        correl=correlate(y1,y2)
      endif else begin
        np = nok2
        y2 = y2(ok2)
        y1 = get_value(wdf1, x2(ok2))
        xr = [x2(0), x2(nok2-1)]
        correl=correlate(y1,y2)
      endelse
      
      sdd = sqrt(total((y1-y2)^2)/((np-1)> 1))
      sdr = sdd/ wdfmax(wdf1-1)
      amp = float(amp)
      tshift = float(tshift)
      sdd = float(sdd)
      sdr = float(sdr)
      if n_elements (leg) gt 1 then legp= leg(0:1) else legp = [0.04, 0.94]
      name1 = strtrim(strmid(wdfcomment(wdf1-1), 0, 12),2)
      name2 = strtrim(strmid(wdfcomment(wdf2-1), 0, 12),2)
      if keyword_set (quiet)  then return
      if n_elements(title) eq 0 then $
          title = 'Comparison of '+ name1 + ' and '+ name2
       plo, [wdf1,out], /over, _extra = extrastuff, nextpos = nextpos, $
          title= title, leg = legp, xrange = xr
;
      s = make_array(7,/string)
      s(0) = 'COMTE: Calculation'
      s(1) = 'Scale Factor ='+strtrim( amp, 2)
      s(2) = 'Time Shift ='+strtrim( tshift,2)
      s(3) = 'Baseline Level = '+strtrim( base,2)
      s(4) = 'Std Deviation = '+ strtrim(sdd * 100,2)+ ' %'
      s(5) = 'Norm Std Dev = '+ strtrim(sdr * 100,2)+ ' %'
      s(6) = 'Correlation = '+strtrim( correl, 2)
      print, 'FIT PARAMETERS'
      for j = 0, 6 do print,'  ', s(j)
      if keyword_set (noleg) then return 
      legendp, [' ', 'FIT PARAMETERS', s(0:6)], position = nextpos, /data
      end
