; ------------------------------------------------------------------------ 
; $Id: d2e.pro,v 1.6 2010/12/09 18:31:27 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
  pro d2e, wd1, wd2 , Calibration = cal_curve, print_it= print_it
;+
; NAME:
;       D2E
;
; PURPOSE:
;       Convert Densities in a step wedge to an Density vs Exposure Vida file.
;
;
; CALLING SEQUENCE:
;
;       D2E, WD1, WD2, calibration = calibration, /print_it
;
;       WD1 - Step wedge data vs bin number
;       Wd2 - Output Density vs Exposure, Default = wd1+1
;       Calibration - number or name for the calibration curve
;             1 or 'G'  = Green Light Calibration, 11/99, 16 erg/cm2
;             2 or 'B'  = Blue Light Calibration, 11/99, 52 erg/cm2
;             9 or 'A'  = Read Alternative Calibration from a file.
;                         Dialog Pickfile will be used to select a file
;             Default = 1
;             Note: Exposure value is not case sensitive.
;                   In current version, only the first character is checked.
;                   For example, case 2 is given by any of the following:
;                        2, 'B', 'b', 21, 23456, 'Bread', 'Blue', 'bad_guy'
;                   and Alternative is given by any of the following:
;                        9, 'A', 'aa', 9999 
;                   and anything that is not case 2 or Alternative is case 1 
;              Format for File is:
;                String Identifier
;               (1)  Exposure for upper right hand strip
;               (2)  Exposure for next upper strip to the left 
;                   etc
;               (22) Exposure for lower right hand strip 
;               (23) Exposure for next lower strip to the left 
;                   etc.
;                Quit when the data noise becomes excessive 
;                  Usually about 34 to 38 values.
;        PRINT_IT - If set the exposure table is printed.
; 
; EXAMPLE:
;       Calculate store the Density/exposure data for Green step wedge data in 
;	arrays 1 and store the result in 3.
;         d2e, 1, 3
;
;
; MODIFICATION HISTORY:
;       Written by:     L. P. Mix, April 24, 2000
;-


on_error, 1
     common saved2e_picker, oldfilter, oldpath
if n_elements(cal_curve) eq 0 then cal_curve = 1
cal_curve = strupcase(strmid(strtrim(cal_curve, 2), 0, 1))
if cal_curve eq 'B' or cal_curve eq '2' then begin
  calval = 'Blue' 
  exposure =[43.25172, 30.61987, 21.67721, 15.34629, 11.1174, 7.691364, $
             5.5719,  3.854813, 2.792565, 1.976985, 1.367739, 0.968285, $
             0.66989, 0.474246, 0.33574 , 0.237686, 0.164438, 0.113764, $ 
             0.080538, 0.055719, 0.040365, $
             0.232275, 0.164438, 0.116413, 0.082414, 0.059704, 0.041305, $
             0.029923, 0.020702, 0.014997, 0.010617, 0.007345, 0.005200, $
             0.003598, 0.002547, 0.001803, 0.001276, 0.000883]
endif else if cal_curve eq 'A' or cal_curve eq '9' then begin
  if n_elements(oldpath) eq 0 then cd, current = path else path = oldpath
  filen = dialog_pickfile(/must_exist, title='Select a file for Film Exposure', $
           path = path, get_path = newpath)
  if strlen(filen) eq 0 then return
  oldpath = newpath
  resolve_it, filen
  openr, unit, filen, /get_lun
  calval=''
  readf, unit, calval
  calval = strtrim(calval,2)
  exposure = make_array(42)
  j =0
  on_ioerror, badnum

  while (not eof(unit) and j lt 42)  do begin
    readf, unit, expos
    exposure[j] = expos
    j = j+1
  endwhile
  badnum:
  free_lun, unit
  exposure = exposure[0:j-1]
endif else begin
  calval = 'Green' 
  exposure =[13.93542, 9.86552, 6.984253, 4.944473, 3.500419, 2.421698, $
             1.79523, 1.270925, 0.920704, 0.651808, 0.450941, 0.284525, $
             0.226006, 0.16000, 0.11591,  0.08019 , 0.058092, 0.039275, $
             0.028452, 0.019684 , 0.013935, $
             0.098655, 0.069843, 0.049445, 0.035004, 0.024781, 0.017144, $
             0.012709, 0.008997, 0.006518, 0.004614, 0.003192, 0.002014, $
             0.001600, 0.001133, 0.000821, 0.000568, 0.000411]
endelse
etot = n_elements(exposure)

if keyword_set (print_it) then begin
 print, Calval, ' Calibration Data'
 print, '  Step       Exposure '
 for i = 0, etot-1 do print, i+1, exposure[i], format='(i6, g14.7)'
 return
endif

if (n_elements (wd1) eq 0) then begin
  print, '% D2E: Waveform array must be defined.'
  return
endif 


wda = wdvalid(wd1, /valid, np=npa, /quiet)

if npa lt 1 then begin
  print, '% D2E: Waveform array must contain valid data'
  return
endif 

if n_elements(wd2) eq 0 then wdb = wda+1 else wdb = wd2
if wdb lt 1 or wdb gt get_maxwdf() then wdb = wda+1

w2i, x, y, wda
nyt = n_elements(y) 

x = round (x)
nx = n_elements(x)
ok = where (x gt 0 and x le etot, nok)
if nok eq 0 then begin
  print, '% D2E: No valid data for the step wedge bins'
  return
endif else if nok lt nx then begin
  x = x[ok]
  y = y[ok]
endif

; bins go from 1 and index values start at zero
x = exposure[x-1]
a = sort(y)
x = x[a]
y = y[a]

i2w, y, x, wdb, /xy, xlabel = 'Density', ylabel = 'Exposure', $
  clabel = 'Step Wedge '+strtrim(calval,2)+' D/E Curve'
end
