; ------------------------------------------------------------------------ 
; $Id: decon.pro,v 1.6 2010/09/20 14:10:32 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
 pro  decon, wdf1, wdf2, wdf3, filter = filter, plot = plot, zero = zero
;
;+
; NAME:
;       DECON
;
; PURPOSE:
;       Deconvolve a measured signal using a measurement response function
;
; CALLING SEQUENCE:
;
;       DECON, WD_SIGNAL, WD_RESPONSE [, WD_DECON]
;
; INPUTS:
;       WD_SIGNAL:   Waveform array number of the signal
;       WD_RESPONSE: Waveform array number of the response 
;
; OPTIONAL INPUTS:
;       WD_DECON:    Waveform array number of the deconvolved signal
;                    If missing, signal will be returned in WD_SIGNAL
;	
; KEYWORD PARAMETERS:
;       FILTER:	     Filter parameter.  If set, filter the fft of the response
;                    function.
;       ZERO:        Fraction of the maximum response assumed to be valid.
;                    If zero is less than 0.0 or greater than 0.001, it is assumed
;                    missing.
;                    DEFAULT:  ZERO = 1.0e-7
;       PLOT:        If set, plot the filtered response
;
; OUTPUTS:
;       Deconvolved signal.  No time shifts are applied.
;
; RESTRICTIONS:
;       Response should have at least 6 points when put on the same sweep 
;       rate as the signal.
;
; PROCEDURE:
;       The following steps are taken.
;       1.  Put the response on the same time base as the signal if necessary.
;       2.  Make an array (N) equal to the length of the response function plus 
;           the length of the signal plus additional values to make the total length
;           a power of two.  If NR, NS are the length of the response and signal
;           then the new arrays have the following number of elements:
;             NNEW = (NR +NS +1) > 32
;             NNEW is increased so that it has only factors of 2, 3, or 5.
;       3.  Take the fft of the new response array of length N. (FFT_R)
;       4.  Find the maximum amplitude of the FFT and set all modes to complex(1,0)
;           that have an amplitude less than ZERO of the maximum amplitude.
;              (ZERO default = 1.0e-7)
;       5.  If filter is set, the FFT_R is changed:
;             FFT_R = FFT_R *(1 + .005*FILTER*(MAX_FR/ABS(FFT_R))^2) /(1+ 0.005*FILTER)
;       6.  The deconvolution, WDF_DCON, is calculated according to:
;             WDF_DCON = FLOAT(FFT(FFT(WDF_SIGNAL, -1)/FFT_R*AVERAGE(WDF_RESP), 1))
;
; EXAMPLE:
;       Assume a measured signal in waveform array 12 and a response function 
;       in waveform 4.  Save the deconvolution of the signal in WDF array 22.
;          DECON, 12, 4, 22, /filter
;       To compare the forward calculation to this result.
;          CONV, 22, 4, 23 & plo, [12, 23]
;
; MODIFICATION HISTORY:
; 	Written by:	L. P. Mix
;	February 7, 2007  Update help file and document the procedure
;-
;
@com.dat
on_error, 1
;
      if n_elements(initwdf) eq 0 then wdfinit
      nparams = n_params()
      if nparams lt 2 then begin
        print,' DECON must have at least 2 parameters'
        return
      endif
;
      if n_elements(wdf1) eq 0 or  n_elements(wdf2) eq 0 then begin
        print, 'Parameters to DECON must be defined'
        return
      endif
      if wdf1 lt 1 or wdf1 gt maxwdfarray then begin
        print, 'wdf array must be greater than 0 and </=', maxwdfarray
        return
      endif
;
      if nparams gt 2 then save = get_save_array(wdf3, wdf1) else save = wdf1
;
      np1 = wdfnp(wdf1-1)
      if np1 eq 0 then begin
        print, 'No data in the first array'
        return
      endif
;
      if wdf2 lt 1 or wdf2 gt maxwdfarray then begin
        print, 'wdf array must be greater than 0 and </=', maxwdfarray
        return
      endif
      np2 =  wdfnp(wdf2-1)
      if np2 eq 0 then begin
        print, 'No data in the second array'
        return
      endif
      get_wdfarray, array1, wdf1
      get_wdfarray, array2, wdf2
;     get the actual count in case one of the arrays is XY data
      np1 = n_elements(array1) -2
      n = n_elements(array2) -2
;     put array 2 (Response) on same time base as the signal (1) 
      if array1[1] ne array2[1] then begin
        dtratio = array2[1]/array1[1]
        n = long ( (np2-1) *dtratio +1.00)
        if (n lt 6 ) then print,'% DECON:  Warning - Response array has only',n, ' elements'
        ivalue = findgen(n)/dtratio
        array2 = interpolate (array2(2:np2+1), ivalue, /grid)
      endif else array2= array2[2:*]
;
;     nshift = long(n/2)
      nzero = (n + 1 + np1) > 32
      nzero = f23(nzero, /five)
;
;      nzero = 2L ^ ceil(alog(nzero)/alog(2))
      array3 = make_array(nzero)
      array4 = make_array(nzero)

      array3(0) = array1(2:*)
      array4(0) = array2
;     array4= shift (array4 , -nshift)
      array4 = fft(array4, -1)
      a4 = abs(array4)
      ma4 = max(a4)
      if n_elements(zero) eq 0 then zero = 1.0e-7
      if zero gt .001 or zero lt 0 then zero = 1.0e-7
      ok = where (a4 lt zero* ma4, nok)
;     a = where (a4 eq 0)
      if nok gt 0 then begin
        array4(ok) = complex(1,0)
        print , '% DECON: Warning --  some elements ofthe response FFT are zero'
        print, '         '+strtrim(nok, 2)+' modes will be set to one.'
        a4 = 1.0/abs(array4)
      endif else a4 = 1.0/a4
      if (n_elements(filter) ne 0) then begin 
        temp  = array4* (1+ 0.005*filter* ma4*ma4*a4*a4) / (1.0+0.005*filter)
        if keyword_set(plot) then begin
          atemp = temporary(a4)
          a4 = 1.0/abs(temp)
          ymin = min(a4)
          ymax = max(atemp)
          plot, a4, yr = [ymin, ymax], /ytyp, /nodata, title='Filtered response; Old response'
          oplot, a4, color = 1
          oplot, atemp, col=2 & hak, /mes
          atemp = 0
        endif 
        i2w, array1[0:1], float(fft(temp, 1)), get_maxwdf(), clab = 'Filtered Response'
        array4 = temporary(temp)
      endif 
      if keyword_set(plot) then $
        plot, a4, /ytype, tit = 'Transform vs mode'
      if nok gt 0 then array4(ok) = complex(1.0, 0.0)
;
      array3 = fft(fft(array3, -1)/array4*(total(array2)/nzero),1)
;     array3 = float(shift(array3, -nshift))
      array3 = float(array3)
;
      array1(2:np1+1) = array3(0:np1-1)
      if n_elements (save) eq 0 then save = wdf1
      if save lt 1 or save gt maxwdfarray then save = wdf1
      put_wdfarray, array1, save, /no_copy
      wdf3 = save
      wdffile(save-1) = wdffile(wdf1-1)
      wdfcomment(save-1) = wdfcomment(wdf1-1)
      wdfxlabel(save-1)= wdfxlabel(wdf1-1)
      wdfylabel(save-1)= wdfylabel(wdf1-1)
      a4 = 0
      array2 = 0
      array3 = 0
      array4 = 0
      sp=0
      im=0
      end
