; ------------------------------------------------------------------------ 
; $Id: droop.pro,v 1.5 2007/06/06 20:03:46 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
 pro droop, wdfin, value, wdfout, clabel = clab, ylabel= ylab, xlabel = xlab
;+
; NAME:
;	DROOP
;
; PURPOSE:
;       This procedure makes a correction for integrator droop and magnetic
;         field diffusion.  The calculation involves the following calculation.
;  Note: The math is based on the same assumptions as in XDAMP.
;       The implementation is linear in the number of points and is more than 10 
;         times faster than XDAMP at 1000 points and more than 200 times faster
;         at 20000 points.
;
; CALLING SEQUENCE:
;       DROOP, WDFin, E_FOLD [, WDFout]
;             [, CLABEL = clabel] [, XLABEL = xlabel] [, YLABEL = ylabel]
;
; INPUTS:
;       WDFin:  WDF array number of the array to be corrected.
;       E_Fold: Time constant to use in correcting the data
;               Note: A negative value will decrease the signal
;                     A positive value will increase the signal
;
; OPTIONAL INPUT:
;       WDFout: Optional output WDF array. If missing or undefined, then 
;                 the corrected array will be stored in WDFin
;	
; KEYWORD PARAMETERS:
;       CLABEL    Replace the dataset comment for the output array
;                 with the string or string variable specified
;       XLABEL    Replace the dataset x-axis label for the output array
;                 with the string or string variable specified
;       YLABEL    Replace the dataset y-axis label for the output array
;                 with the string or string variable specified
;
; PROCEDURE:
;       All droop corrections are performed on uniformly spaced data. If the
;       data is not uniform it is put on a uniform spacing using linear 
;       interpolation. YNEW is the droop corrected value for YOLD.
;       If dt is the point spacing and n is the number of Y values, then 
;          const = dt/e_fold
;          scale = (dt * e_fold)/(e_fold - dt)
;          exp_fun = exp(-(1+findgen(n-1))*const)
;    and
;       ynew(0) = yold(0)*scale/dt
;       for i = 1, n-1 do ynew(i) = yold(i)*scale/dt + 
;            scale/e_fold * total(exp_fun(0:i-1)*reverse(ynew(0:i-1)))
;        
; EXAMPLE:
;       Correct the data in wdf array 1 for a 300 ns integrator and store
;         in wdf array 11.
;       DROOP, 1, 300e-9, 11       
;
; MODIFICATION HISTORY:
; 	Written by:     Paul Mix, June 4, 1997
;-
@com.dat
on_error, 1

;
      if n_elements(initwdf) eq 0 then wdfinit
      nparams = n_params()
      if nparams lt 1 then begin
        if nparams gt 0 then print,'% DROOP: Procedure must have two parameters'
        print, '% Format: DROOP, WDFin, E_fold [, WDFout] [, CLABEL = clabel]'
        print, '            [, XLABEL = xlabel] [, YLABEL = ylabel]'
        return
      endif
;
      if n_elements(wdfin) eq 0 then begin
        print, '% DROOP: WDFin must be defined'
        return
      endif
      if n_elements(value) eq 0 then begin
        print, '% DROOP: E_fold time must be defined'
        return
      endif
      wdf = wdfin(0)
      if wdf lt 1 or wdf gt maxwdfarray then begin
        print, 'wdf array must be greater than 0 and </=', maxwdfarray
        return
      endif
;
      get_wdfarray, array, wdf
      dt = array(1)
      if abs(value) lt dt then begin
        print, '% DROOP: E_fold is smaller than the point spacing'
        Print, '         No compensation will be done.
        return
      endif
      n = n_elements(array)-2
      temp = array/dt
      efold = 1./value
      const = efold*dt
      
      expd =exp(-(1.0d0*const))
      ccon  = (1.-const)/dt
      array(2) = temp(2)/ccon
      dummy = expd *array(2)
      for k = 3L, n+1 do begin
        array(k) = (temp(k)+efold*dummy)/ccon
        dummy = (dummy+ array(k))*expd
      endfor
      if nparams eq 3 then save = get_save_array(wdfout, wdf) else save=wdf
      wdfout = save
      

      put_wdfarray, array, save, /no_copy
      wdffile(save-1) = wdffile(wdf-1)
      if n_elements(clab) ne 0 then wdfcomment(save-1) = clab else begin
        cdef = 'Droop Corr='+strtrim(value,2)
        if strpos(wdfcomment(wdf-1), ' - ') gt 0 then $
            cdef = ';;'+cdef else cdef = ' - '+cdef
        wdfcomment(save-1) = wdfcomment(wdf-1)+ cdef
      endelse
      if n_elements(xlab) ne 0 then wdfxlabel(save-1) = xlab else $
        wdfxlabel(save-1)= wdfxlabel(wdf-1)
      if n_elements(ylab) ne 0 then wdfylabel(save-1) = ylab else $
        wdfylabel(save-1)= wdfylabel(wdf-1)
     end
