; ------------------------------------------------------------------------
; $Id$
; ------------------------------------------------------------------------
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
;+
; NAME:
;       EDIST
;
; PURPOSE:
;       Calculate the energy distribution for a pulse give the current and voltage
;         waveforms for the shot
;
; CATEGORY:
;       Analysis
;
; CALLING SEQUENCE:
;
;       EDIST, WDF_Voltage, WDF_Current [, WDF_EDIST ]
;
; INPUTS:
;       WDF_Voltage:  WDF waveform number for the voltage or a search string for
;                     the WDF comment
;       WDF_Current:  WDF waveform number for the current or a search string for
;                     the WDF comment.
;                     Only positive current is used.
;       WDF_EDIST: Coulombs vs Voltage for the input voltage and current
;                     DEFAULT: WDF_EDIST = GET_MAXWDF() -2
;
; KEYWORDS:
;       NPOINTS:      Number of points for the distribution calculation.
;                     NPOINTS = NPOINTS > 256
;                     DEFAULT:  NPOINTS = 2048
;       NSEC:         If set, horizontal sweep in NSEC (1e-9 sec)
;                     DEFAULT:  NSEC =0
;       ABSOLUTE:     If set, Current = ABS(Current)
;                     Default:  
;                     Current = (max(Current) gt 0) ? (current > 0.0) : (-current > 0)
;       FILTER:       Size of box car filter to average the cumulative average.  
;                     Must be less than the number of points/15
;                     Default: Default = 21
;                     Note:  Final number of points is about (NPOINTS / 0.75 / FILTER)
;
; OUTPUT:
;       The energy distribution in coulombs/eV vs Voltage.
;       NOTE:  Integral of the output is the total charge in the pulse.
;
; PROCEDURE:
;       1. Determine region of overlap
;       2. Put to uniform time bases with at least NPOINTS values
;       3. Adjust the current according to the value of ABSOLUTE
;       4. Put current and voltage on a uniform time.
;       5. Interpolate to NPOINTS values
;       6. Remove multiple voltage values
;       7. Smooth and interpolate to fewer values
;
; EXAMPLE:
;       Determine the energy distribution of particles with voltage from WDF array 1 
;       current from WDF array 3 and store the distribution of Coulombs vs Energy
;       in WDF array 11
;         EDIST, 1, 3, 11
;
; MODIFICATION HISTORY:
; 	Written by:	L. P. Mix, April 30, 2008
;
;-

PRO edist, wdf1, wdf2, wdf3, npoints = npoints, filter = filter, $
        NSEC = nsec, ABSOLUTE = absolute

on_error, 1

npar = n_params()
if npar lt 2 then begin
  print, '% EDIST:  Voltage and current waveform must be provided'
  return
endif
wd1 = wdvalid(wdf1, /valid, np = np)

if np ne 1 then begin
  stuff =(np gt 1) ? 'must provide a single WDF array' : 'WDF array must be provided'
  print, '% EDIST:  Voltage '+stuff
  if np gt 1 then for i = 1, np do print, i, ': ', wdcomment(wd1[i-1])
  return
endif else wd1 = wd1[0]

wd2 = wdvalid(wdf2, /valid, np = np)

if np ne 1 then begin
  stuff =(np gt 1) ? 'must provide a single WDF array' : 'WDF array must be provided'
  print, '% EDIST:  Current '+stuff
  if np gt 1 then for i = 1, np do print, i, ': ', wdcomment(wd2[i-1])
  return 
endif else wd2 = wd2[0]

if wdf3 lt 1 or wdf3 gt get_maxwdf() then wdf3 = get_maxwdf()-2

get_wdfarray, array1, wd1
get_wdfarray, array2, wd2
wdinfo, [wd1, wd2], npoints = np, xmax = xmax
nel = np[0]
get_common_time, array1, array2, xmax[0], xmax[1], nel, np[1], xydata
if n_elements(filter) eq 0 then filter = 21
hfilt = filter/2
filter = hfilt+hfilt+1
npoints = (n_elements (npoints) eq 0) ? 2048 : (npoints > 256)

if xydata eq 0 then begin
  dt = array1[1]
  nel = n_elements(array1)-2
  if nel lt npoints then begin
    ratio = ((nel-1.0)/(npoints-1.0))
    xx = findgen(npoints)* ratio
    vol= interpolate(array1[2:*], xx)
    cur= interpolate(array2[2:*], xx)
    dt = dt * ratio
    nel = npoints
  endif else begin
    vol = array1[2:*]
    cur = array2[2:*]
  endelse
endif else begin
  nel = n_elements(array1)/2
; assume xx is sorted
  xx = array1[0:nel-1] - array1[0]
  vol = array1[nel:*]
  cur = array2[nel:*]

;  help, vol, npoints
  dt = xx[nel-1]/(npoints-1)
  print, nel & help, xx
  x = findgen(npoints)*dt
  ix = findex(xx, x)
  vol = interpolate(vol, ix)
  cur = interpolate(cur, ix)
  nel = npoints
endelse

if keyword_set(absolute) then cur = abs(cur) else begin
  if (max(cur, min = mncur) gt -mncur) then cur = (cur > 0) else $
    cur = (-cur > 0)
endelse
ij = sort (vol)
lcuniq =0
vol= vol[ij]
cur = cur[ij]
cur = total(cur, /cumulative, /double)
i2w, vol, cur, 200, /xy
mindev = (vol[nel-1]-vol[0])*2/nel
minv = vol[0]
derv = vol[1:nel-1] -vol[0:nel-2]
while min(abs(derv), ij) lt mindev do begin
  cur[ij] = -1
  ok = where (cur ge 0, nel)
  cur = cur[ok]
  vol = vol[ok]
  derv = vol[1:nel-1] -vol[0:nel-2]
endwhile
if vol[0] gt minv then begin
  vol = [minv, vol]
  cur = [0, cur]
  nel = nel+1
endif else cur[0]=0
; Range of 
delv = vol[nel-1] -vol[0]

i2w, vol, cur, 201, /xy

; plot, vol, cur , title = 'voltage vs current' & hak, /mes
w = (filter < nel/15)
if w gt 2 then curt = smooth(cur, w )
np = round(nel/(filter*0.75))
dv = delv/(np-1.0)
vnew = vol[0]+findgen(np) *dv
ij = findex(vol, vnew)
vv = interpolate(vol, ij)
cc = interpolate(cur, ij)
scal = keyword_set(nsec) ? 1e-9*dt : dt

i2w, [vol[0], dv], cc*scal, wdf3, clab = 'Energy Spectrum', xlab = 'Energy(eV)', ylab='Coul/eV'
; oplot, vv, cc & hak, /mes
dif, wdf3

END
