; ------------------------------------------------------------------------ 
; $Id: fftw.pro,v 1.8 2010/09/20 14:10:32 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
 Pro  FFTW, wdf1in, wdf2, wdf3, wdf4, wdf5, fast = fast, limit = limit, $
            positive = positive, double = double, degree=degree, time_Window = timew, $
            hanning_filter = hanning_filter, mode = mode, delta = delta
;+
;     Format: FFTW, WDIN, WDA [, WDT] [, WDR] [, WDI] [, /positive] 
;           [, /degree] [, /double] [, /mode] [, Time_Window = time_window]
;           [Hanning_filter = hanning_filter]
;
;     Find the FFT of an wdf array and store the components in other wdf
;          arrays.  GET_MAXWDF() is used for a work array if TIME_WINDOW is provided.
;     Restrictions: wdin and one other wdf array numbers must be valid.
;                   zeros may be used for unused wdf arrays.
;      FAST     To avoid problems with long FFT's the number of points
;                will be changed to speed the fft.
;                Fast = 0 Do not modify the data
;                Fast = 1 Modify the data by changing the spacing
;                Fast = 2 Modify the data by adding extra points to connect to
;                         the first value in the waveform
;                Default: FAST = 0 
;                Note: /fast will speed up calculations for all waveforms
;                      with the number of points > ~500 and a prime number.
;                If FAST lt 0 or FAST gt 2, then FAST = 1
;      LIMIT    Indicates the number of points which may be added
;                This is approximate.
;                If NP = total number of points and NEL is the original number:
;                LIMIT <= 0  Change NP to a power of 2 or 3 times a power of 2.
;                LIMIT [0,1] Change NP such than (NP-NEL)< LIMIT*NEL
;                LIMIT > 1   Change NP such than (NP-NEL)< LIMIT
;                Default: LIMIT = 0.03
;      DOUBLE    If set, do in double precision.
;      TIME_WINDOW An array of two or more elements to set the time window for the FFT.
;                GET_MAXWDF() is used as a work array with the routine, LIM. 
;      HANNING_FILTER - If set, a hanning filter will be applied to the data.
;                If Hanning_filter is ge 1 or less than 0.5 then 
;                      Alpha = 0.5 else Alpha = Hanning_filter
;      WDA     - the absolute value or magnitude of the FFT
;      WDT     - the phase angle, theta = atan(imaginary, real)
;             Note: Phase angel in radians unless degree is specified.
;      WDR     - the real part of the complex FFT 
;      WDI     - The imaginary part of the complex FFT
;      POSITIVE - If present and not zero, then the zero frequency
;                 is unchanged, but all positive frequencies are multiplied 
;                 by 2 and only positive frequencies are returned.
;      DEGREE  -  If set, angle returned in degrees.
;      MODE    - If present and not zero, the abscissa is the mode number
;                only positive modes are returned and nonzero mode amplitudes
;                are multiplied by 2
;      DELTA   - Increment of the FFT
;                This number multiplied by the abscissa of mode data should 
;                provide the frequency abscissa values
;                  DELTA = 1.0/(NP * DELTAT)
;     Example:
;       calculate the real and imaginary components of the fft of array
;       2 and store the result in arrays 10 and 11
;         fftw, 2, 0,0,10, 11
;       calculate the amplitude of the fft of array 3 and store the positive
;       frequency components in array 4. Print the peak.
;         fftw, 3, 4, /pos & print, mx(4)
;       calculate the amplitude and phase of WDF array 1 into arrays 2 and 3
;       store the phase in degrees; find the maximum positive frequency.
;         fftw, 1, 2, 3, /deg & a = mx(2,L=0,/quiet,time=freq) & print, freq
;       
;-
@com.dat
;
      on_error, 1
      if n_elements(initwdf) eq 0 then wdfinit
      nparams = n_params()
      if nparams lt 2 then begin
        print, '% FFTW: Routine must have 2 parameter'
        return
      endif
;
      if n_elements(wdf1in) eq 0 then begin
        print, '% FFTW: Parameter must be defined'
        return
      endif
      if wdf1in lt 1 or wdf1in gt maxwdfarray then begin
        print, 'wdf array must be greater than 0 and </=', maxwdfarray
        return
      endif
      wdf1 = wdf1in
      np = wdfnp(wdf1-1)
      if np lt 3 then begin
        print, '% FFTW: No valid data in the array - '+strtrim(wdf1, 2)
        return
      endif
      if n_elements(timew) gt 1 then begin
        wdf1 = get_maxwdf() 
        xfr, wdf1in, wdf1
        lim, wdf1, timew, ier = ier
        if ier[0] ne 0 then begin
          print, '% FFTW: Error limiting the waveform - '+strtrim(wdf1in, 2)
          print, '        Time window =[ '+strtrim (min(timew, max = m), 2)+', '+ $
                strtrim(m, 2)+' ]'
          return
        endif
        np = wdfnp(wdf1-1)
        if np lt 3 then begin
          print, '% FFTW: No valid data after the limit. WDF = '+strtrim(wdf1in, 2)
          print, '        Time window =[ '+strtrim (min(timew, max = m), 2)+', '+ $
                strtrim(m, 2)+' ]'
          return
        endif
      endif
      wdf2 = get_save_array(wdf2, 0) 
      if nparams gt 2 then wdf3 = get_save_array(wdf3, 0) else wdf3 = 0
      if nparams gt 3 then wdf4 = get_save_array(wdf4, 0) else wdf4 = 0
      if nparams gt 4 then wdf5 = get_save_array(wdf5, 0) else wdf5 = 0
;
      if wdf2 lt 0 or wdf1 gt maxwdfarray then begin
        print, 'wdf array for output must be greater than 0 and </=', maxwdfarray
        return
      endif
;
      if wdf3 lt 0 or wdf1 gt maxwdfarray then begin
        print, 'wdf array for real must be greater than 0 and </=', maxwdfarray
        return
      endif
;
      if wdf4 lt 0 or wdf1 gt maxwdfarray then begin
        print, 'wdf array for imaginary be greater than 0 and </=', maxwdfarray
        return
      endif
      if wdf2 eq 0 and wdf3 eq 0 and wdf4 eq 0 and wdf5 eq 0 then begin
        print, 'No valid output arrays have been specified for FFTW' 
        print, '  ** RETURNING WITH NO OPERATION **'
        return
      endif
;
      if n_elements (mode) eq 0 then mode = 0
;
      if mode ne 0 then positive = 1
      if n_elements(positive) eq 0 then positive = 0
        
      get_wdfarray, array, wdf1
      if keyword_set (fast) then begin
        if fast lt 0 or fast gt 2 then fast = 1
        if n_elements (limit) eq 0 then limit = 0.03
        get_fast_fft, array, fast, limit
      endif
      np = n_elements(array)-2
      if keyword_set(hanning_filter) then begin
        h = hanning_filter[0]
        if h lt 0.5 or h ge 1 then $
          y  = fft(hanning(np)*array[2:*], -1, double = double) else $
          y  = fft(hanning(np, alpha = h)*array[2:*], -1, double = double)
      endif else y  = fft(array(2:*), -1, double = double)
      deltat = 1.0/(np*array[1])
      delta = deltat
      if mode eq 0 then begin
        if positive eq 0 then begin
          cent = (np/2)+1
          time =  (cent-np)*deltat
          y =  shift (y, -cent)
        endif else begin
          time = 0.0
          y = y(0:np/2)
          y(1:*) = y(1:*)*2
        endelse
        delt = deltat
        xlab = 'Frequency'
      endif else begin
        delt = 1.0
        time = findgen(1+np/2)
        xlab = 'Mode number (1 = '+strtrim(string(deltat), 2)+')'
        y = y(0:np/2)
        y(1:*) = y(1:*)*2
      endelse
      time = [time, delt]
      if wdf2 ne 0 then $
        i2w, time,float(abs(y)),xl=xlab,y='Spectrum Amplitude', $
          c = 'Amplitude of the FFT of '+wdfcomment(wdf1-1), wdf2
      if wdf3 ne 0 then  begin
        angle =  atan(float(imaginary(y)), float(y))
        if keyword_set(degree) then begin
          angle = angle / !dtor
          ylab = 'Phase Angle (Degrees)'
        endif else ylab = 'Phase Angle (Radians)'
        i2w, time,angle, xlab = xlab, y= ylab ,  $
          c = 'Phase Angle of the FFT of '+wdfcomment(wdf1-1), wdf3
      endif
      if wdf4 ne 0 then $
        i2w, time, float(y),xl= xlab,  y='Real Spectrum', $
          c = 'Real Component of the FFT of '+wdfcomment(wdf1-1), wdf4
      if wdf5 ne 0 then $
        i2w, time, Imaginary(y), xl=xlab, y='Imaginary Spectrum',  $
          c = 'Imaginary Component of the FFT of '+wdfcomment(wdf1-1), wdf5
      array = 0
      end
