; ------------------------------------------------------------------------ 
; $Id: filt.pro,v 1.3 2007/06/06 20:03:46 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
 Pro  FILT, wdf1, wdf2, frequency = freq, atten = filter,  FAST = fast, $
      order = order, delta = delta, type = type, mode = mode, plot = plot, $
      width = widthin, band = fwidth, hp = hp, br = br, bp = bp, $
      low = low, high = high, limit = limit
;
;
;+
; NAME:
;       FILT
;
; PURPOSE:
;       General purpose filter utility
;       (FILT calls LOP, HIP, BAP, or BAR for actual filtering)
; CATEGORY:
;       Filter
;
; CALLING SEQUENCE:
;       FILT, WD1 [, WD2][, TYPE=TYPE][, ATTEN=ATTEN][,ORDER = ORDER] $
;            [, MODE = MODE] [,WIDTH = WIDTH][, /PLOT] $
;            [, FREQUENCY = FREQ][, LOW = LOW][, HIGH = HIGH] $
;            [, DELTA = DELTA][, BAND=BAND] [, /HP] [,/BP] [, /BR] $
;            [, FAST = fast] [, LIMIT = limit]
; INPUTS:
;       WD1      WDF array number of the array to be filtered
;                WD1 has NP points with spacing DT
;                NOTE:  If WD1 is x-y data, it will be changed to uniform.
; OPTIONAL INPUT PARAMETERS:
;       WD2      WDF array number for storage of the filtered array
; KEYWORD PARAMETERS:
;       FAST     To avoid problems with long FFT's the number of points 
;                will be changed to speed the fft.
;                Fast = 0 Do not modify the data
;                Fast = 1 Modify the data by changing the spacing
;                Fast = 2 Modify the data by adding extra points to connect to
;                         the first value in the waveform 
;                Default: FAST = 2
;       LIMIT    Indicates the number of points which may be added
;                This is approximate.
;                If NP = total number of points and NEL is the original number:
;                LIMIT <= 0  Change NP to a power of 2 or 3 times a power of 2.
;                LIMIT [0,1] Change NP such than (NP-NEL)< LIMIT*NEL
;                LIMIT > 1   Change NP such than (NP-NEL)< LIMIT
;                Default: LIMIT = 0.03 
;       TYPE     Type of filter desired. Options are Butterworth, Exponential,
;                User supplied, and Ideal ('BUT', 'EXP', 'USER', 'IDEAL').
;                Only the first 3 letters are checked. Default is 'BUT'
;       ATTEN    Frequency domain filter function.  ATTEN = FILTER
;                in the equations below.  Must be supplied with
;                type = 'USER' and have same number of values as dataset WD1.
;                CAUTION:  SET FAST = 0 for user mode.
;       ORDER    Order of the filter. Default is 2
;       MODE  or
;       FREQUENCY  or
;       DELTA    These three keywords specify the center frequency of the 
;                filter and are related by the following:
;                MODE = (NP*DT)/DELTA = FREQ*NP*DT.  Only one is required.
;                If none are provided, then default is MODE = 12.
;       WIDTH  or
;       BAND  or 
;       HIGH and LOW
;                These keywords specify the width of band rejection and band
;                pass filters and are related by the following:
;                WIDTH  = BAND*NP*DT = (HIGH-LOW)*NP*DT 
;                If HIGH and LOW are specified then MODE = (HIGH+LOW)/(2*NP*DT)
;                If none are specified, then default is WIDTH = 0.8*MODE
;       PLOT     If present, the frequency domain filter function is plotted
;                If PLOT is a two element array, then this is used as XRANGE
;                  in the plot command.
;                If PLOT is a one element array and negative then the filter
;                  function will be saved in -PLOT if it is a valid WDF array.
;                  Otherwise the filter function will be saved in MAX_WDF_ARRAY.
;                  Note: MAX_WDF_ARRAY = get_maxwdf()
;       /HP      Indicates a high pass filter, default is low pass filter
;       /BR      Indicates a band reject filter, default is low pass filter
;       /BP      Indicates a band pass filter, default is low pass filter
; OUTPUTS:
;       NONE     The filtered array is stored in a WDF array in the common block
; OPTIONAL OUTPUT KEYWORD PARAMETERS:
;       ATTEN    If TYPE is not 'USER', ATTEN contains the freq domain
;                filter function.
; COMMON BLOCKS:
;       com.dat  WDF common block
; RESTRICTIONS:
;       If type is 'USER', ATTEN must be provided
; PROCEDURE:
;       The Fourier transform of WD1 is taken and multiplied by a frequency
;       domain filter function.  The inverse Fourier transform is taken, and
;       the real values stored in WD2 (or WD1 if WD2 is not provided).
;       The filter function is defined as follows:
;       Assume WD1 has NP points, ^ indicates an exponent, and DIST is an
;       array of NP points defined as:
;       DIST = [0, 1, 2, 3, 4, 5,..., NP/2, ...,5, 4, 3, 2, 1]
;     The low pass filter is defined by the following:
;       Butterworth - FILTER = 1/(1 + (DIST / MODE) ^ (2 * ORDER))
;       Exponential - FILTER = EXP ( - (( DIST / MODE) ^ ORDER))
;       Ideal - FILTER = 1 for DIST < MODE, 0 for DIST > MODE
;     The high pass filter is defined by the following:
;       Butterworth - FILTER = 1/(1 + ( MODE / DIST ) ^ (2 * ORDER))
;       Exponential - FILTER = EXP ( - (( MODE / DIST ) ^ ORDER))
;       Ideal - FILTER = 0 for DIST < MODE, 1 for DIST > MODE
;     The band type filter is defined by the following where:
;       Butterworth - FILTER = 1/(1 +( DIST*WIDTH/(DIST^2-MODE^2))^(2 * ORDER))
;       (for band pass: FILTER = 1.0 - FILTER)
;       Exponential - FILTER =
;                     EXP( - (( (DIST^2-MODE^2) / (DIST*WIDTH) ) ^ (2*ORDER) ))
;       (for band reject: FILTER = 1.0 - FILTER)
;       Ideal - FILTER = 0 for DIST < MODE - WIDTH/2 or DIST > MODE + WIDTH/2
;                                             = 1, OTHERWISE
;       (for band reject: FILTER = 1.0 - FILTER
;       NOTE:
;       For exponential and butterworth band type frequency, the true center
;       mode, CM, is given by CM = MODE * SQRT (1 + (WIDTH/(2*MODE))^2).
;       For Butterworth filters, a DELTA equal to the full width at half 
;       maximum of a gaussian pulse will only attenuate the peak several percent
; EXAMPLES:
;       Filter array 1 with a low pass Butterworth filter with a delta 
;       of 15 ns and store the result in array 3.
;                FILT, 1, 3, d= 15e-9
;       Filter array 1 with a 100 MHz low pass exponential filter.
;                FILT, 1, f= 100e+6, t = 'exp'
;       Filter array 2 with a low pass filter with the 3db set for 12 cycles.
;                FILT, 2
;       Filter array 3 with a band pass Butterworth filter with 3db points 
;       at 10 and 100 MHz; store the filtered array in 4.
;                FILT, 3, 4, /bp, lo= 1e7, hi= 1e8
;       Filter array 2 with a band reject filter to reduce modes 30 to 60.
;       Plot the filter function and return it in FARRAY.
;                FILT, 2, /br, m = 45, w= 30, /plot, atten = farray
;       NOTE:
;       To look at the modes of an array, n, use the following commands:
;                fftf, n, y & plot, abs(y) 
;             or fftf, n, y & plot, abs(y), xr=[0, 100} ! for modes 0 to 100
; MODIFICATION HISTORY:
;      Original version, L. P. Mix, May 18, 1992
;
;-
;
@com.dat
;
      on_error, 1
;
      if n_elements(initwdf) eq 0 then wdfinit
      nparams = n_params()
      if nparams eq 0 then begin
        print,'% Format: FILT, WDF1 [, WDF2] [, TYPE = type] [, LIMIT = limit]'
        print,'          [, ATTEN = filter] [, ORDER = order] [, FAST = fast] '
        print,'          [, DELTA = delta] [, MODE = mode] [, FREQUENCY = freq]'
        print,'          [, WIDTH = width] [, LOW = low] [, HIGH = high]'
        print,'          [, BAND = fwidth] [, /PLOT ] [, /HP] [, /BR] [, /BP]'
        return
      endif
;
      if n_elements(wdf1) eq 0 then begin
        print, 'Parameters to FILT must be defined'
        return
      endif
      if wdf1 lt 1 or wdf1 gt maxwdfarray then begin
        print, 'wdf array must be greater than 0 and </=', maxwdfarray
        return
      endif
;
      if nparams gt 1 then save = get_save_array(wdf2, wdf1) else save = wdf1
;
      nel = wdfnp(wdf1-1)
      if nel eq 0 then begin
        print, '% FILT: No valid data in the first array'
        return
      endif
      if n_elements(order) gt 0 then order = order > 0.01 $
           else order = 2  
;
      get_wdfarray, array, wdf1
;     if X-Y data nel is not correct
      nelsave = n_elements(array)
      nel = nelsave-2
;
      if n_elements(type) gt 0 then begin
        type = strtrim(type, 2)
        type = strupcase(type)
        type = strmid(type, 0, 3)
        case type of 
  'BUT' : filtertype = 'BUTTER'
  'EXP' : filtertype = 'EXPON'
  'IDE' : filtertype = 'IDEAL'
  else  : filtertype = 'USER'
        endcase
      endif else filtertype = 'BUTTER'
      if filtertype eq 'USER' then begin
        if n_elements (filter) ne nel then begin
          print, 'Filter has wrong number of elements, filter = ' , $
            n_elements(filter)
          print,'  Array has elements = ', nel
          return
        endif
      endif
      if (filtertype eq 'USER') then begin
        if n_elements(fast) eq 0 then fast = 0 
        if (fast ne 0) then begin
          fast = 0
          print, '% FILT:  FAST must equal 0 for USER filtertype'
        endif
      endif else begin
        if n_elements(fast) eq 0 then fast = 2 else $
          if ((fast lt 0) or (fast gt 2)) then fast = 2
        if fast gt 0 then begin
          if n_elements(limit) eq 0 then limit = 0.03
          get_fast_fft, array, fast, limit
          nel = n_elements(array)-2
        endif
      endelse
      total = nel* array(1)
      if n_elements (delta) gt 0 then delta = abs (delta) > 0.1*array(1)
      if n_elements (mode) gt 0 then bin = mode > 1 $
        else if n_elements(delta) gt 0  then $
        bin = total/delta  else if n_elements(freq) gt 0 then $
        bin = abs(freq)*total else bin = 12
      if keyword_set (bp) or keyword_set(br) then begin
        if n_elements (low) gt 0 and n_elements(high) gt 0 then begin
          bin = (high + low)*0.5*total
          width = abs (high-low)*total
        endif else if n_elements (fwidth) gt 0 then width = fwidth*total else $
        if n_elements (widthin) gt 0 then width = widthin else width= 0.8 * bin
        if keyword_set (bp)  then bap, wdf1, save, mode=bin,width = width, $
           plot = plot, atten = filter, type = filtertype,order = order, $
           wdf_array = array else $
          bar, wdf1, save, mode=bin, width= width, plot= plot, atten= filter, $
          type = filtertype, order = order,  wdf_array = array
      endif else begin
        if keyword_set(hp) then hip,wdf1,save,mode=bin,plot=plot, $
        atten = filter, type = filtertype, order = order,  $
        wdf_array = array  else $
        lop, wdf1, save, mode=bin, plot= plot, atten = filter, $
          type = filtertype,order = order, wdf_array = array
      endelse
;
      if fast eq 2 then array = array(0:nelsave-1)
      put_wdfarray, array, save, /no_copy
      wdf2 = save
      wdfcomment(save-1)= wdfcomment(wdf1-1)
      wdffile(save-1)= wdffile(wdf1-1)
      wdfxlabel(save-1)= wdfxlabel(wdf1-1)
      wdfylabel(save-1)= wdfylabel(wdf1-1)

      return
      end
