; ------------------------------------------------------------------------ 
; $Id: fit_temp.pro,v 1.5 2007/06/06 20:03:47 lpmix Exp $
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
;+
; NAME:
;       Fit_Temp
;
; PURPOSE:
;       Fit spectrometer output to a blackbody spectrum
;       Units are (W/m2/eV) vs eV
;       
;       See FIT_Spec for a similar code with Angstroms as the Abscissa
;          (units for FIT_Spec are  (W/cm2/Angstrom/Steradian) vs Angstroms)
;
; CATEGORY:
;       Target analysis
;
; CALLING SEQUENCE:
;       FIT_TEMP , WDF [, TEMPERATURE = temperature] 
;          [, EMISSIVITY = emissivity] [, ITER = iter]
;          [, WEIGHTS = weights] [, ERRORS = errors]
;          [, MIN_ERROR = min_error] [, /PLOTIT]
;          [, SIGMA = sigma] [, CHISQ = chisq]
;          [, ITMAX = itmax] [, TOLERANCE = tolerance] 
;          [, SAVE = save]  [, QUIET = quiet] [, Verbose = verbose]
;          [, /STATISTICAL] [, /Tonly] [, /Eonly]
;          
;
; INPUTS:
;       WDF:    Input data to be fit
;               Units are (W/m2/eV) vs eV
;	
; KEYWORD PARAMETERS:
;      Temperature: Fit value of the temperature.  If a value is present, 
;                   it will be used as an initial estimate.
;      Emissivity : Fit value of the emissivity.  If a value is present
;                   it will be used as an initial estimate.
;      ITER:     Number of iterations performed
;      WEIGHTS:  WDF array of the error estimate for each point or
;                  an array of weight values.  The wdf array or the vector
;                  must cover the domain of the input data.
;      ERRORS:   Ignored if WEIGHTS has valid data
;                These are expected to be fractional estimates of the error.
;                For example if the data has a 5% error then ERRORS = 0.05.
;                Errors may have several formats: Errors may be a
;                WDF array of the error estimate for each point or 
;                the assumed error as a fraction or an array of error values.
;                These errors are used to calculate weights.
;                In the curve fit, actual weights are 1/(Error_Value*Y_value)^2
;                unless STATISTICAL is set(for instrumental or Gaussian weights)
;                If STATISTICAL is set, weights are  1/(Error_Value*Y_value)
;                for statistical or Poisson weights.
;                Y_value are the ordinates for the input data.
;      STATISTICAL : See ERRORS above
;      MIN_ERROR: Minimum value for error 
;                Default = 1e-3
;      PLOTIT:   If set, fit will be plotted.
;      POISSON:  If errors are specified, set the weights to 1/(Error Value)
;      Sigma:    Standard Deviation for the coefficients for each material
;                CURVEFIT values are scaled by SQRT(CHISQ)
;                If both temperature and emissivity are fit,
;                   SIGMA[0] - emissivity
;                   SIGMA[1] - temperature
;      Chisq:    Value of Chi-squared from CURVEFIT
;                NOTE:  Data is normalized to 1.0
;      ITMAX:    Maximum number of iterations
;                Default = 1000
;      Tolerance:Change in Chi-squared which ends fit
;                Minimum = 1.0E-16,  Maximum = 0.01
;                Default = 1e-15 
;      SAVE:     WDF array for the fit.  Default is GET_MAXWDF()
;      Quiet:    If set, terminal output will be minimum.
;      Verbose:  If set, terminal output will be maximum.
;      Tonly:    If set only, the data is fit to the temperature only.
;                Emissivity must contain a valid value.
;      Eonly:    If set only, the data is fit to the Emissivity only.
;                Temperture must contain a valid value.
;      
;
; OUTPUTS:
;       The fit data is stored in the maximum wdf array unless SAVE is set.
;
; PROCEDURE:
;       Data is fit to an equation of the form:
;         data = K * Blackbody_Spectrum(T) 
;       where K is the emissivity and T is the temperature..
;
; EXAMPLE:
;       Fit SRD data which is in WDF array, 5
;         FIT_TEMP, 5
;
; MODIFICATION HISTORY:
; 	Written by:     L. P. Mix, June 11, 2001
;-
;
; NAME:
;       TEMP_fit_function
;
; PURPOSE:
;       Calculates fit parameters for common block variables
;
; CALLING SEQUENCE:
;       Called by Curvefit
;       TEMP_fit_funct, x, a, f, pder
;       X is x values
;       A is Coefficients
;       F is the function value
;       PDER is the partial derivative with respect to A
;
; MODIFICATION HISTORY:
; 	Written by:	L. P. Mix, April 17, 2001
;

PRO temp_fit_t_function, x, a, f, pder
 COMPILE_OPT hidden

on_error, 1
 common temp_fit_funct_common, xval, wdf, fsave, esave, tsave, yave
;
nc = n_elements(a)
nelx = n_elements (x)
lccalc = 0
; convert to meters

   energ = x > 0.025
   efrac = energ/ a[0]
   ok =  where (efrac gt 0.9* alog((machar()).xmax), nok)
   ok2 = where (efrac le 1.1* alog((machar()).eps), nok2)
   if nok gt 0 then efrac[ok]=1.0
   if nok2 gt 0 then efrac[ok2]=1.0
   pder = exp(Efrac)
   power = (pder -1.0d0)
;  if units are set then switch to W/cm2/A/sr from W/m2/m
;  divide by 1e4 for area
;  divide by 1e10 for A
;  divide by !pi for Sr
;
   pder = pder*efrac/power/a[0]
   f = esave*1.583476669924d+08/yave * ((energ^3)/power)
   if nok gt 0 then f[ok]=0.0
   if nok2 gt 0 then f[ok2]=0.0
   pder = f*pder
   end

PRO temp_fit_e_function, x, a, f, pder
on_error, 1
 COMPILE_OPT hidden

 common temp_fit_funct_common, xval, wdf, fsave, esave, tsave, yave
;
nelx = n_elements (x)

if n_elements(fsave) ne nelx then begin
   energ = x > 0.025
   efrac = energ/tsave
   ok =  where (efrac gt 0.9* alog((machar()).xmax), nok)
   ok2 = where (efrac le 1.1* alog((machar()).eps), nok2)
   if nok gt 0 then efrac[ok]=1.0
   if nok2 gt 0 then efrac[ok2]=1.0
   power = (exp(efrac) -1.0d0)
;  if units are set then switch to W/cm2/A/sr from W/m2/m
;  divide by 1e4 for area
;  divide by 1e10 for A
;  divide by !pi for Sr
;
   fsave = (1.583476669924d+08/yave) * ((energ^3)/power)
   if nok gt 0 then fsave[ok]=0.0
   if nok2 gt 0 then fsave[ok2]=0.0
endif
   pder = fsave
   f = a[0]*pder
   end

PRO temp_fit_function, x, a, f, pder
 COMPILE_OPT hidden
on_error, 1
 common temp_fit_funct_common, xval, wdf, fsave, esave, tsave, yave
;common filter_fit_funct_common
;

   energ = x
   efrac = energ/ a[1]
   ok =  where (efrac gt 0.9* alog((machar()).xmax), nok)
   ok2 = where (efrac le 1.1* alog((machar()).eps), nok2)
   if nok gt 0 then efrac[ok]=1.0
   if nok2 gt 0 then efrac[ok2]=1.0

   pder = exp(Efrac)
   power = (pder -1.0d0)
;  if units are set then switch to W/cm2/A/sr from W/m2/m
;  divide by 1e4 for area
;  divide by 1e10 for A
;  divide by !pi for Sr
;
   pder = pder*efrac/a[1]/power
   f = ( 1.583476669924d+08/yave) * ((energ^3)/power)
   if nok gt 0 then f[ok]=0.0
   if nok2 gt 0 then f[ok2]=0.0
   ff = f*a[0]
   pder = [f,  ff*pder]
   pder= reform(pder, n_elements(x), 2, /over)
   f = temporary(ff)
   end


 PRO fit_temp, WDFarr, SAVE = save,  ITER = iter, errors = errors, $
          temperature = temperature, emissivity = emissivity, $
          Tonly = tonly, eonly = eonly,  statistical = statistical, $
          WEIGHTS = weights, SIGMA = sigma, CHISQ = chisq, plotit= plotit, $
          ITMAX = itmax, TOLERANCE = tolerance, FILEID = fid, $
          QUIET = quiet, Verbose = verbose, min_error = min_error
on_error, 1

 common temp_fit_funct_common, xval, wdf, fsave, esave, tsave, yave
 fsave = 0
 npar = N_PARAMS ()
 if npar eq 0 then begin
   print, '% Format: FIT_TEMP , WDF [, TEMPERATURE = temperature]
   print, '          [, EMISSIVITY = emissivity] [, ITER = iter]
   print, '          [, WEIGHTS = weights] [, SIGMA = sigma] [, CHISQ = chisq]
   print, '          [, ITMAX = itmax] [, TOLERANCE = tolerance] [, /PLOTIT]
   print, '          [, SAVE = save]  [, QUIET = quiet] [, Verbose = verbose]
   print, '          [, ERRORS = errors] [, /STATISTICAL] [, /Tonly] [, /Eonly]
   return 
 endif

 if n_elements(save) eq 0 then save = get_maxwdf()
 if save lt 1 or save gt get_maxwdf() then save = get_maxwdf()
 
 if n_elements (wdfarr) eq 0 then begin
   print, '% FIT_TEMP: Waveform must be specified.'
   return 
 endif
 if wdfarr lt 1 or wdfarr gt get_maxwdf() then begin
   print, '% FIT_TEMP: Waveform array must be gt 1 and less than ', $
     strtrim (get_maxwdf(), 2)
   return
 endif


 if keyword_set(verbose) then print, 'Fit to waveform: ', wdcomment(wdfarr)

 if n_elements(min_error) eq 0 then min_error = 1e-3 

 w2i, xval, yval, wdfarr
 np = n_elements(yval)

 if keyword_set (weights) then begin
   if n_elements(weights) eq 1 then begin
     if weights lt 1 then begin
       print, '% FIT_TEMP: WDF array for weights must be gt 1
       return
     endif else if weights ge 1 and weights le get_maxwdf() then begin
       val = long(weights)
       w2i, dummy, wei,val
       if not check_equal(dummy, xval) then wei= get_value(val, xval)
       if n_elements(wei) ne np then begin
         print, '% FIT_TEMP: Weights must have the same number of elements'
         print, '             as the data - Waveform, ', strtrim(val,2)
         return
       endif
       
     endif else begin
        print, '% FIT_TEMP: Invalid value for weights', weights
        return
     endelse
   endif else begin
     if n_elements(weights) ne np then begin
       print, '% FIT_TEMP: Weights must have the same number of elements'
       print, '               as the data'
       return
     endif
     wei = weights
   endelse

 endif else if keyword_set (errors) then begin
   if n_elements(errors) eq 1 then begin
     if errors lt 1 then begin
       wei = errors 
     endif else if errors le get_maxwdf() then begin
       val = long(errors)
       w2i, dummy, wei,val 
       if not check_equal(dummy, xval) then wei = get_value(val, xval)
       if n_elements(wei) ne np then begin
         print, '% FIT_TEMP: Errors WDF array must have the same number of '
         print, '            elements as the data - Waveform, ', strtrim(val,2)
         return
       endif
     endif else begin
        print, '% FIT_TEMP: Invalid value for errors', errors
        return
     endelse
   endif else begin
     if n_elements(errors) ne np then begin
       print, '% FIT_TEMP: Weights must have the same number of elements'
       print, '            As the data'
       return
     endif
     wei = errors
   endelse 
   wei = 1.0/(abs((wei > min_error)*yval))
   if not keyword_set (statistical) then wei = wei*wei
 endif else wei = make_array(np, value = 1.0)

 if keyword_set (tonly) then begin
   if n_elements (emissivity) eq 0 then begin
     print, '% FIT_TEMP: Emissivity must be defined if Tonly is set'
     return
   endif
   funct = 'temp_fit_t_function'
   esave = emissivity
   if n_elements(temperature) gt 0 then coef = temperature[0] else coef = 200.
 endif else if keyword_set (eonly) then begin
   if n_elements (temperature) eq 0 then begin
     print, '% FIT_TEMP: Temperature must be defined if Eonly is set'
     return
   endif
   funct = 'temp_fit_e_function'
   tsave = Temperature
   if n_elements(emissivity) gt 0 then coef = emissivity[0] else coef = 0.5
 endif else begin
   funct = 'temp_fit_function'
   coef= make_array(2)
   if n_elements(emissivity) gt 0 then coef[0]=emissivity[0] else coef[0]=0.5
   if n_elements(temperature) gt 0 then coef[1]=temperature[0] else coef[1]=200.0
 endelse
 xvalue = xval
 yvalue = yval
; yave = total(yval, /double)/n_elements(yval)
 yave = 1d0
; yvalue = yvalue/yave
; wei = wei*yave

 if n_elements(tolerance) eq 0 then tol = 1e-15 else $
     tol = (abs(tolerance) < 0.01)> 1.0e-16
 maxpass = 3
 logtol = alog(tol+tol)/(maxpass -1)
 tol = [exp((1+indgen(maxpass-1))*logtol), tol]
 iter = 0
 if keyword_set(verbose) then print, 'Initial Coefficient =', coef
 for i=0, maxpass-1 do begin
   result=curvefit(xvalue, yvalue, wei, coef, sigma, chisq=chisq, $
     function_name= funct ,iter=iteri,itmax=500,tol=tol[i], /double)
   iter = iteri+iter
   if keyword_set(verbose) then begin
     print, 'Tolerance =' ,tol[i]
     print, 'Coefficient =', coef
     print, 'Standard Deviation =', sigma
     print, 'Chi-Squared =', chisq, ',  Iterations =', iteri
   endif
 endfor
 if keyword_set(tonly) then temperature = coef[0] else begin
     if not keyword_set(eonly ) then temperature = coef[1]
     emissivity = coef[0]
 endelse
 sigma =  sigma*sqrt(chisq)

 if keyword_set (verbose) then begin
   print, 'Final value of coefficients:'
   print, ' Emissivity =', emissivity
   print, ' Temperature =', temperature
   print, ' Sigma =', sigma
 endif
   
 i2w, xvalue, result*yave, save, clabel= 'Temperature Fit, E = ' $
   + strtrim(emissivity, 2) +', Temp = '+ strtrim(temperature, 2), $
   xlabel = 'Wavelength (A)', Ylabel='W/cm2/sr/A' , /xydata

 if keyword_set (verbose) or keyword_set(plotit) then begin
   plo, save, /over, /color
   plo, wdfarr, /no, /color, /over
   
 endif
 
END
