; ------------------------------------------------------------------------ 
; $Id: fwhm.pro,v 1.6 2009/09/24 20:04:25 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
 Function  fwhm,wdf, base = base, left = leftin, right = rightin, $
               first = first, second = second, peak = peak, quiet = quiet, $
               value = h_m_value, maximum = maximum, minimum = minimum
;+
;     this procedure calculates the full with at half maximum of a wdf array
;       By setting a keyword, other fractions can be found.
;     the maximum absolute value is selected as the peak
;     the return value is the full width at half maximum or minimum depending
;        on the absolute magnitude of the waveform values.
;     NOTE: Data is not filtered
;     CAUTION: X-Y data is sorted before fwhm is applied.
;              Strange results will occur if XY data is used which is not a 
;              function; ie more than one y value for an x value.
;     format:
;        WIDTH = FWHM (WDF, BASE = base, LEFT = left, RIGHT = right, 
;              FIRST = first, SECOND = second, PEAK = peak, QUIET = quiet, 
;              VALUE = value, MAXIMUM = maximum, MINIMUM = minimum)
;
;     WIDTH  -  fwhm of the waveform
;               zero is returned if a problem is encountered with input
;               -1   is returned if only first value is found.
;               -2   is returned if only second value is found.
;     WDF is the array number
;     BASE is the baseline
;              If quiet is not set and BASE is undefined then BASE 
;                  is input using the cursor
;        NOTE: If QUIET is set, then default is BASE=0
;     LEFT, RIGHT is the left boundary of the curve and right the right
;       left and right should be used to look for a local extrema over a 
;       a limited range of data
;       If LEFT has two or more elements and RIGHT is missing or if RIGHT has two or
;         more elements and LEFT is missing, VALUES will be taken as the range; ie
;             LEFT = MIN(VALUES)  &  RIGHT = MAX(VALUES)
;         where VALUES is the specified vector.
;       If left is greater than the maximum value then it is set to the 
;         minimum.
;       If right is less then the minimum value then it is set to the maximum.
;
;     FIRST and SECOND are the abscissa values for the half maxima values
;       If only one is found, first = second
;     VALUE is the value of the half maximum as a fraction of the peak
;       Default: VALUE = 0.5
;       For example to find the width at 1/4 (25%) of the maximum, VALUE=0.25
;       VALUE = (VALUE > .01) < 0.99
;     PEAK is the maximum deflection from the baseline unless maximum or
;        minimum is set.
;     The following keywords can be used to specify a maximum or minimum.
;       If both or neither are set, the default is to take the maximum absolute
;       deflection from the baseline.
;     MAXIMUM - If set, Peak is the maximum positive deflection
;     MINIMUM - If set, Peak is the maximum negative deflection
;
;     
;     if quiet is nonzero, terminal output is eliminated
;
; EXAMPLES:
;     
;     Calculate the fwhm of waveform 2 which has a baseline offset that you want
;       to remove and store the result in WID.
;       BLIN, 2
;       WID = FWHM(2, B=0)  ; or WID = FWHM(2, /QUIET) ; or WID = FWHM(2, BASE=0) 
;
;     Calculate the fwhm of waveform 4 which has a baseline of zero.
;     Since the curve has multiple peaks, use ZOOMW to pick the peak of interest.
;     Store the result in WID1
;       ZOOMW, 4, XCUR = XC ; pick the two points to define the peak of interest
;       WID1 = FWHM(4, B=0, R= XC) ; or WID1 = FWHM(4, /QUIET, L= XC)
;
;     Example:
;  First: Let
;       x = findgen(100)
;       y = sin(x*(!pi*0.5/99)) & i2w, x, y, 1
;       print, fwhm(1, first= f, second = s,  /quiet) , f, s
;     will return:
;       -1      33.0000      33.0000
;
;  Second: Let
;       if  x= findgen(100) & y = sin(x*(!pi/99)) & i2w, x, y, 2
;     then
;       print, fwhm(2, first= f, second = s,  /quiet) , f, s
;     will return: 
;       66.0000      16.5000      82.5000
; or
;       print, fwhm(2, first= f, second = s,right = 50, /quiet) , f, s
;     -1      16.5000      16.5000
; or
;       print, fwhm(2, first= f, second = s,left= 45 , /quiet) , f, s
;     -2      82.5000      82.5000
; 
;        NOTE: Results are the same if waveform is negative; ie
;              i2w, x, -y, 2
;              had been used for the examples
;-
@com.dat
;
      on_error, 1
      if n_elements(initwdf) eq 0 then wdfinit
      nparams = n_params()
      if nparams eq  0 then begin
 print,'% Format: WIDTH = FWHM (WDF, BASE = base, LEFT = left, RIGHT = right,'
 print,'          FIRST = first, SECOND = second, PEAK = peak, QUIET = quiet,'
 print,'          MAXIMUM = maximum, MINIMUM = minimum)'
        return, 0.0
      endif
;
      if n_elements(wdf) eq 0 then begin
        print, 'Parameter to FWHM must be defined'
        return, 0.0
      endif
      if wdf lt 1 or wdf gt maxwdfarray then begin
        print, 'wdf array must be greater than 0 and </=', maxwdfarray
        return, 0.0
      endif
;
      nel = wdfnp(wdf-1)
      if nel eq 0 then begin
        print, 'No valid data in the wdf array'
        return, 0.0
      endif
      start = wdfminx(wdf-1)
      stopp = wdfmaxx(wdf-1)
      nl = n_elements(leftin)
      nr = n_elements(rightin)
      if max([nl, nr], min = m) ge 2 and m eq 0 then begin
        if nl ge 2 then right = max(leftin, min = left) else $
           right = max(rightin, min = left)
      endif else begin
        if nl gt 0 then left = leftin(0) else left = start
        if nr gt 0 then right = rightin(0) else right = stopp
      endelse
      if (left lt start) or (left ge stopp) then left =start
      if (right le left) or (right gt stopp) then right = stopp
;     
      if n_elements(quiet) eq 0 then quiet = 0
      get_wdfarray, array, wdf, /xydata
;     need to get the number of elements in case this is x-y data
      if  n_elements(array) eq nel+2 then begin
; uniform data
        x0 = array(0)
        dx = array(1)
        y = temporary(array(2:nel+1))
        lcuniform = 1
      endif else begin
; non-uniform data
        y = array(nel: nel+nel-1)
        x = temporary(array(0:nel-1))
        ord = sort(x)
        x = x(ord)
        y = y(ord)
        lcuniform = 0
      endelse
      if quiet eq 0 then begin
        plo, wdf, xr = [left, right], eng = 0
        if n_elements(base) eq 0 then begin
          print, ' Put cursor on the baseline position'
          cursor, xx , base
        endif else base = base (0)
        oplot, [left, right], [base, base]
        print, 'Baseline is ', strtrim(base, 2)
      endif else if (n_elements (base) eq 0) then base = 0.0
      if lcuniform eq 0 then begin
        ok = where (x ge left and x le right, nok)
        if nok gt 1 then begin
          lb = ok(0)
          rb = ok(nok-1)
        endif
      endif else begin
        lb = long((left - x0)/dx +0.000001)
        rb = long((right - x0)/dx +0.000001)
        nok = rb-lb+1
      endelse
      if nok lt 2 then begin
        print,'%FWHM: Full width cannot be determined with less than two points'
        return, 0
      endif
      lcd = 1
      if (keyword_set(minimum) xor keyword_set(maximum)) then begin
        if keyword_set(minimum) then begin
          peak = - min (y(lb: rb)-base, imax)
          y = -y
          lcd = -1.
        endif else peak = max (y(lb: rb)-base, imax)
      endif else begin
; either both are set or both are not set
        peak = max (y(lb: rb)-base, imax)
        peak2 = - min (y(lb: rb)-base, imax2)
        if peak lt peak2 then begin
          peak = peak2
          imax = imax2
          y = -y
          lcd = -1.
        endif
      endelse
      if keyword_set(h_m_value) then $
        hmvalue = (h_m_value > 0.01) < 0.99 else hmvalue = 0.5
      halfmax = peak*hmvalue+lcd*base
      if lcuniform eq 0 then xmval = x(lb+imax) else $
          xmval = (lb+imax)*dx+x0
      if quiet eq 0 then begin
        print, 'Maximum deflection from baseline is ',strtrim( peak,2),  $
           ' at x = ', strtrim(xmval, 2)
        oplot, [left, right], lcd*[halfmax, halfmax]
      endif
      i = long(imax + lb -1)
      if i lt 0 then lcmin = 0 else begin
        while y(i) gt halfmax and i gt lb do i= i-1
        if (i le lb and y(i) gt halfmax) then lcmin = 0 else lcmin = 1
      endelse
      j = long(imax +lb+1)
      if j gt nel-1 then lcmax = 0 else begin
        while y(j) gt halfmax and j lt rb do j= j+1
        if (j ge rb and y(j) gt halfmax) then lcmax = 0 else lcmax = 1
      endelse
      if lcmin gt 0 then begin
        if y[i] eq y[i+1] then begin
;          if values are equal - average
           if lcuniform eq 0 then $
            first = (x(i)+x(i+1))*0.5 else begin
            i = i + 0.5
            first = i*dx+x0
          endelse
        endif else begin
          if lcuniform eq 0 then $
            first = x(i)+(halfmax-y(i))*(x(i+1)-x(i))/(y(i+1)- y(i)) else begin
            i = i + (halfmax-y(i))/(y(i+1)- y(i))
            first = i*dx+x0
          endelse
        endelse
        if lcmax eq 0 then begin
          second = first
          return, -1
        endif
      endif
      if lcmax gt 0 then begin
        if y[j-1] eq y[j] then begin
;          if values are equal - average
          if lcuniform eq 0 then $
            second = (x(j)+x(j-1))*0.5 else begin
            j = j -0.5
            second = j*dx+ x0
          endelse
        endif else begin
          if lcuniform eq 0 then $
            second = x(j)+(halfmax-y(j))*(x(j-1)-x(j))/(y(j-1)- y(j)) else begin
            j = j - (halfmax-y(j))/(y(j-1)- y(j))
            second = j*dx+ x0
          endelse
        endelse
        if lcmin eq 0 then begin
          first = second
          return, -2
        endif
      endif 
      if lcmax eq 0 and lcmin eq 0 then begin
        first = 0
        second =0
        return, 0
      endif
      if quiet eq 0 then $
        print, string(100*hmvalue, format ='(f5.1)') $
         +' % of maximum occurs at (', strtrim(first, 2), ', ' , $
           strtrim(second, 2),')'
      if lcuniform eq 0 then return, (second-first) else  return,  (j-i)*dx
      end
