; ------------------------------------------------------------------------ 
; $Id: gausd.pro,v 1.7 2011/12/01 21:38:49 lpmix Exp $
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
;+
; NAME:
;       GAUSD
;
; PURPOSE:
;       Return a normal distribution in an IDL array or a wdf array.
;       Function of the form:
;       Y = K * exp(-0.5*((X - X0)/SIG)^2)
;       K is chosen such that TOTAL(Y) = 1.0
;           or Total(y)*DX =1.0
;       Note: FWHM = SQRT(-2 * ALOG(0.5) * SIG^2)
;
; CALLING SEQUENCE:
;
;       GAUSD [, WDF]
;
;
; REQUIRED INPUTS: 
;       WDF     - If set, return the gaussian in  WDF array, WDF.
;                 Default:  WDF = GET_MAXWDF()
;
; KEYWORD Parameters:
;       CENTER  - Center of the gaussian.
;                 Default: CENTER = 0.0
;       SIGMA   - Variance of the distribution
;                 Default: SIG = 1
;                 NOTE:  FWHM = SQRT(-2*ALOG(0.5)*SIGMA^2)
;       FWHM    - Full width half max of the normal distribution
;                 This is ignored if SIGMA is set.
;       RANGE   - Range of data in units of SIG
;                 Default is symmetric function.
;                 If N_elements(RANGE) eq 1 then 
;                   RANGE =[-ABS(RANGE), ABS(RANGE)] 
;                 Default = [-3.5, 3.5]
;                 Range must include the center.
;       DX      - Point spacing in X
;                 Default: DX = ((Range[1]-Range[0])*SIG)/npoints
;       Double  - If set, do calculation in DOUBLE precision.
;       NPoints - Number of points estimate.
;                 NPOINTS is ignored if DX is specified
;                 Default is 501.
;       Normal  - If set, Normalization is integral = 1
;                 Default: Total(y) = 1
;       CLABEL  - Comment label.  Default is the gaussian parameters
;       HELPME  - If set, give a help comment 
;
; OUTPUTS:
;       Requested gaussian will be stored in a WDF array
; 
; SIDE EFFECTS:
;       If WDF is zero or omitted, the gaussian is stored in GET_MAXWDF().
;
; EXAMPLES:
;	Plot the default gaussian
;         GAUSD & plo, get_maxwdf()
;
;       Store a gaussian with width sigma of 10 ns in wdf array 5
;         gausd, 5, sig =10e-9
;
; MODIFICATION HISTORY:
;       Original version: May 28, 2004, L. P. Mix
;-
;
PRO GAUSD, WDF, CENTER = center, SIGMA = sigma, RANGE = range, DX = dx, $
              DOUBLE = double, NPOINTS = npoints, CLABEL = clabel, $
              NORMAL = normal, FWHM = fwhm, HELPME = helpme
;
;Return to caller if an error occurs
on_error,1  
;
;
      if keyword_set(helpme) then begin
        print, '% Format:  GAUSD, WDFIN, CENTER = center, SIGMA = sigma,' 
        print, '             RANGE = range, DX = dx, DOUBLE = double,'
        print, '             NPOINTS = npoints, CLABEL = clabel,'
        print, '             FWHM= fwhm, /Normal'
        return
      endif
      work = get_maxwdf()
      nparams = n_params()
;
      
      if keyword_set(wdf) then wdf = get_save_array(wdf, work) $
        else wdf = work
      if keyword_set(double) then sigd = 1.0d0 else sigd = 1.0
      if (n_elements(sigma) eq 0) and (n_elements(fwhm) gt 0) then begin
        fwhm = (size(/type, fwhm) lt 4) ? float(fwhm[0]) : fwhm[0]
        sigma = sqrt(fwhm^2/(-8*alog(0.5)))
      endif 
      if n_elements(sigma) eq 0 then sig = sigd else sig = abs(sigma[0])
      if n_elements(range) eq 0 then range = [-3.5, 3.5] else $
        if n_elements (range) eq 1 then range = [-1.0, 1.0]*abs(range) $
          else range = float([min(range, max=m), m])
      if range[0] gt 0 then range[0] = 0.0
      if range[1] lt 0 then range[1] = 0.0
      if range[1]-range[0] eq 0 then begin
        print, '% GAUSD:  Range must be greater than zero.'
        print, '          Range set to [-3.5, 3.5]'
        range =  [-3.5, 3.5]
      endif
      if keyword_set(double) then cend = 0.0d0 else cend = 0.0
      if n_elements(center) eq 0 then center = cend else center = center[0]
      if size (/type, center) lt 4 then center = float(center)
      ran = range*sig
      if n_elements (dx) eq 0 then begin
        if n_elements(npoints) gt 0 then np= npoints[0] > 11 else np = 501L
        dx = (ran[1]-ran[0])/ (np-1)
      endif else dx = dx[0]
      npm = round(-ran[0]/dx)
      x0 = center -npm*dx
      npp = round(ran[1]/dx)
      np = npm+npp+1
      
      if keyword_set(double) then begin
        if npm gt 0 then ii = rotate(dindgen(npm+1), 5) else ii = 0.0d0
        if npp gt 0 then ii = [ii, dindgen(npp)+1.0d0]
      endif else begin
        if npm gt 0 then ii = rotate(findgen(npm+1), 5) else ii = 0.0
        if npp gt 0 then ii = [ii, findgen(npp)+1.0]
      endelse
      if keyword_set(double) then half = -0.5d0 else half = -0.5
      ii = (half*dx*dx/sig/sig)*ii*ii
      
      if np ne n_elements(ii) then print, 'Internal error '

      mach = machar(double=double)
      eps = alog(mach.xmin)+2
      ok = where (ii  gt eps, nok)
      if nok eq np then y = exp(ii) else begin
        y = make_array(np, double = double)
        y[ok] = exp(ii)
      endelse
      if keyword_set (normal) then nn = dx*total(y,/double) else nn = total(y,/double)
      if not keyword_set(double) then nn = float(nn)
      y = y/nn
      if n_elements (clabel) gt 0 then clab = clabel[0] else $
        clab='Gaussian Dist - Center=' +strtrim(center, 2) + $
        ';;Sigma='+strtrim(sig, 2)
      put_wdfarray, [x0, dx, y], wdf, clab = clab, file = '', $
           xlab = '', ylab = 'Amplitude'
     end
