; ------------------------------------------------------------------------
; $Id: get_cablcomp.pro,v 1.2 2007/06/06 20:03:47 lpmix Exp $
; ------------------------------------------------------------------------
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
;+
; NAME:
;       GET_CABLCOMP
;
; PURPOSE:
;       Use the response of a cable to use in correcting measured signals
;
; CALLING SEQUENCE:
;       GET_CABLCOMP, WDF1, WDF2 [, WDF3] [, /PLOTIT] [, /ZOOMIT]
;           [, TRUNCATION = truncation] [, RECORD_LENGTH = reclen]
;
; INPUTS:
;       WDF1:   True signal waveform
;               (Signal directly into the recorder)
;       WDF2:   Degraded signal recorded after passing through a cable or 
;               other filter
;
; OPTIONAL INPUTS:
;       WDF3:   WDF array for the calculated response
;               If WDF3 is not provided or invalid, then WDF3 = get_maxwdf()-1
;	
; KEYWORD PARAMETERS:
;       PLOTIT: Plot WDF1, WDF2 and the cable compensated version of WDF2
;               If KEYWORD_SET(ZOOMIT), then PLOTIT = 1
;       ZOOMIT: If set, use ZOOMW to view the plot.  
;       RECORD_LENGTH: Length of the response 
;               DEFAULT:  RECORD_LENGTH =  250*ROUND(N2/250.)
;                         where N2 = N_ELEMENTS(WDF2)
;       TRUNCATION: Level at which data is truncated.
;               DEFAULT:  TRUNDATION = 0.5
;        NOTE:  The technique is recursive and can grow exponentially
;               for unstable initial conditions.  TRUNCATION must be 
;               adjusted to provide a stable solution.
; OUTPUTS:
;       A response waveform is (see WDF3) generated to allow this the 
;       compensation to be applied to actual data.
;
; OPTIONAL OUTPUTS:
;       A plot of the input signal and the compensated signal is generated.
;
; PROCEDURE:
;       The procedure uses the time domain convolution described by Bill Boyer
;       (SAND87-3072).  The technique is similar to the XDAMP routine
;       XI_gcabcomp.  Modified for speed increase.
;
; EXAMPLE:
;       Calculate a compensation function for a step pulse stored in WDF array 7,
;       and a degraded pulse stored in WDF array 8.  Plot 7, 8 and the cable
;       compensated version of waveform 8.
;          GET_CABLCOMP, 7, 8, /PLOT
;
; MODIFICATION HISTORY:
; 	Written by:	L. P. Mix, Jr., December 20, 2006
;                       From XDAMP routine XI_cabcomp
;-
; Parts of this routine taken from the XDAMP routine XI_gcabcomp
; Copyright (c) 2006, Sandia Corporation. The United States Government
; retains a nonexclusive license in this software as prescribed in
; AL 88-1 and AL 91-7. Export of this program may require a license from
; the United States Government.
;++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
;  procedure XI_gcabcomp
;
;  Procedure to deconvolves two arrays using serial deconvolution to generate
;  a cable compensator. The entered array is the "true" response,
;  the second array is the response through the cable (or whatever).
;  Generates a new array with the name COMP_ prepended to the signal name.
;  User is prompted for toe truncation fraction and the target length of
;  the compensator (in case the signal was shortened by earlier processing).
;------------------------------------------------------------------------------
PRO GET_CABLCOMP, WDF1, WDF2, WDF3, plotit = plotit, TRUNCATION = truncation, $
       RECORD_LENGTH = reclen, ZOOMIT = zoomit

@com.dat
on_error, 1
  if n_elements(initwdf) eq 0 then wdfinit
  nparams = n_params()
  if nparams eq 0 then begin
    print, '% Format: GET_CABLCOMP, WD1, WD2 [, WD3]'
    return
  endif
  if nparams lt 2 then begin
    print,'% GET_CABLCOMP: Add must have at least 2 parameters'
    return
  endif
;
  if n_elements(wdf1) eq 0 or  n_elements(wdf2) eq 0 then begin
    print, '% GET_CABLCOMP: Parameters must be defined'
    return
  endif
  if wdf1 lt 1 or wdf1 gt maxwdfarray then begin
    print, '% GET_CABLCOMP: WDF1 must be greater than 0 and </=', maxwdfarray
    return
  endif
  n1 = wdfnp(wdf1-1)
  if n1 lt 2 then begin
    print, '% GET_CABLCOMP: No data in the first array'
    return
  endif
  if wdf2 lt 1 or wdf2 gt maxwdfarray then begin
    print, '% GET_CABLCOMP: WDF2 must be greater than 0 and </=', maxwdfarray
    return
  endif
  n2 = wdfnp(wdf2-1)
  if n2 lt 2 then begin
    print, '% GET_CABLCOMP: No data in the second array'
    return
  endif
;*** get the two waveforms
  get_wdfarray, array1, wdf1
  get_wdfarray, array2, wdf2
  dt1 = array1[1]
  dt2 = array2[1]
  if nparams gt 2 then begin
    if wdf3[0] gt 0 and wdf3[0] le maxwdfarray then save = wdf3[0] $
      else save = maxwdfarray-1
  endif else save = maxwdfarray-1
  n1 = n_elements(array1)-2
  n2 = n_elements(array2)-2
  n1m = n1 -1
  ratio = dt1/dt2
  y1 = temporary(array1[2:*])
;*** must have same timebase sampling interval
  if abs(ratio-1.0) gt 0.001 then begin
;   put response on the same time base as signal
    n2old = n2
    n2 = round(n2/ratio)
    posx = findgen(n2)*ratio
    ix = (floor(posx) > 0) < (n2old-2)
    jx = ix+1
    frac = posx-ix
    y2 = temporary(array2[2:*])
    y2 = y2[ix] *(1.0-frac) +y2[jx]*frac
  endif else y2= array2[2:*]
  n2m = n2-1
  reclen_def =  250*ROUND(n2/250.)
; Check the parameters:
  trunc_def = 0.50
  If n_elements(truncation) eq 0 then trunc= trunc_def else trunc= truncation[0]
  if trunc lt 0.05 or trunc gt 0.95 then begin
    print,'% GET_CABLCOMP: Truncation fraction must be in the range, [0.05, 0.95]: ' $
      + strtrim(trunc, 2)
    return
  endif 
  If n_elements(reclen) eq 0 then reclen= 5000 else reclen = round(reclen[0])
  if reclen lt 10 then begin
    print,'% GET_CABLCOMP: Record length must be greater than 9: ' $
      + strtrim(reclen, 2)
    return
  endif 

  temp1 = make_array(reclen, /double)
  temp2 = make_array(reclen, /double)

;*** the following should never happen with two positive pulses...
  max1 = MAX(y1, MIN=min1)
  IF (ABS(min1) GT ABS(max1)) THEN BEGIN
    y1 = -y1
    max1 = MAX(y1, MIN=min1)
  ENDIF
  max2 = MAX(y2, MIN=min2)
  IF (ABS(min2) GT ABS(max2)) THEN BEGIN
    y2 = -y2
    max2 = MAX(y2, MIN=min2)
  ENDIF

;*** shift and fill end of vector with last value out to reclen
crs1 = (WHERE(y1 GE trunc*max1, nok))[0]
if nok ge reclen then temp1[0] = y1[crs1:crs1+reclen-1] else begin
  temp1[0] = y1[crs1:*]
  temp1[n1-crs1:*]  = y1[n1-1]
endelse
;*** shift and fill end of vector with last value out to reclen
crs2 = (WHERE(y2 GE trunc*max2, nok))[0]
if nok ge reclen then temp2[0] = y2[crs2:crs2+reclen-1] else begin
  temp2[0] = y2[crs2:*]
  temp2[n2-crs1:*]  = y2[n2-1]
endelse
t = systime(1)
;*** do the deconvolution
comp = make_array(reclen, /double)
temp1 = temp1/dt1
comp[0] = temp1[0]/temp2[0]
;*** reverse result inline
FOR i = 1, reclen-1 DO $
   comp[reclen-i-1] = (temp1[i]-TOTAL(temp2[1:i]*comp[reclen-i:*], /double))/temp2[0]
comp = reverse(comp)
; print, systime(1)-t
;*** now fix units
temp_name = (strpos(wdcomment(wdf2), ' - ') gt 0) ? $
     'COMP_'+wdcomment(wdf2)+';;TRUNCATION=' +strtrim(trunc, 2) $
     +';;RECLEN='+strtrim(reclen,2) : $
     'COMP_'+wdcomment(wdf2)+' - TRUNCATION=' +strtrim(trunc, 2) $
     +';;RECLEN='+strtrim(reclen,2)

i2w, [0, array1[1]], comp, save, clab = temp_name, xlab = wdfxlabel([wdf1-1])
if keyword_set(plotit) || keyword_set(zoomit) then begin
   cablcomp, wdf2, save, maxwdfarray
   if keyword_set(zoomit) then Zoomw, [wdf1, wdf2, maxwdfarray], /color else $
     plo,  [wdf1, wdf2, maxwdfarray], /color, /over
endif 
END 
