; ------------------------------------------------------------------------ 
; $Id: get_fcup.pro,v 1.3 2007/06/06 20:03:47 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
 pro get_fcup, vcorin, fcup, pout, distance = dist, ionmass = ionmass, $
       nsecond = nsec, highvolt = highvolt, lowvolt = lowvolt, $
       charge = charge, output= output, limit = limit, smooth= smoo
;
;+
; NAME: GET_FCUP
; 
; Description:
;       Propagate an ion pulse to a target assuming that they are
;       moving with a velocity corresponding to a specified voltage
;       pulse.  The energy versus arrival time is calculated for 
;       Protons by default.  The charge collector signals are then
;       transported back to the anode and the currents modified to relect
;       the rising or falling voltages using the ratio of the arrival times.
;       If the slope of the arrival time changes slope it is set to zero.
;       The transit time is smoothed using a 5 point boxcar by default.
;       Present version limits the response to the high energy portion.
;       Note: The voltage pulse is propagated forward. The measured current
;             is then corrected for the transit time or projected back in time.
;   Note: This is based on get_vprop
;       Velocity is assumed to be given by:
;         velocity = sqrt( 2.0 * 1.6022e-19/ 1.6726e-27 * Vc*Q/M)
;           where M = 1, 12, 16 for protons, carbon and oxygen, respectively, 
;                 Q is the charge, and Vc is the acceleration voltage
; Format:
;      GET_FCUP, Vcor, Fcup, OUTPUT, [, Nsecond = nsec] [, distance = distance]
;                      [, ionmass = ionmass] [, CHARGE = charge ]
;                      [, output= output] [, deltat = deltat] [, SMOOTH = smooth]
;                      [, limit = limit] [, /HighVolt] [, /LowVolt]
;
;    where
;        vcor      wdf array for the corrected voltage or a search string
;                  XY data is sorted - Process is terminated if multiple
;                    voltages at the same time.
;        fcup      wdf array(s) for the charge collectors
;                  or a string for the search
;        Output    If present and a valid wdf number, this will
;                  be used in place of the keyword parameter.
;                  If output is a single number and fcup is several then
;                  successive arrays will be used.
;                  Output may be an array if fcup is an array.
;                  This parameter of the keyword must be provided.
;        nsecond   abscissa units for vcor
;                  default is nsecond = 1, abscissa in nanoseocnds
;        distance  ion propagation distance in meters
;                  default distance = 0.3 m
;        ionmass   Ion mass in amu. Default = 1
;                  IONMASS > 1
;        charge    Ion charge - Default = 1; Charge > 1
;       Note: Only one ion mass and one charge can be used.
;        smooth    Smoothing for the transit time.
;                  Default = 5
;        output    output WDF arrays. Ignored if positional parameter is present
;        limit     Voltage cut; eliminate voltages below this value.
;                  Default is 10% of peak
;                  0 < limit < 0.9 
;        HighVolt  If set only the high voltage portion of the arrival time
;                  will be saved. 
;                  This is the default.
;        LowVolt   NOT CURRENTLY ALLOWED.
;                  If set only the low voltage portion of the arrival time
;                  will be saved. This is ignored if HighVolt is set.
;                  Default is to return the high energy arrival time curve.
;        Example
;           Calculate the current of protons measured at a charge collector
;             30 cm away.
;           Assume the energy is in wdf array 3, the fcup in 4,  and store 
;           the result in 10
;             get_fcup, 3, 4, out = 10 
; < or >      get_fcup, 3, 4, ion=1, char=1, /high,  out = 10, dist = 0.30
;
;           Calculate the currrent of Li+ ions measured at at charge collectors
;             15 cm away using 'VLIDJJ' as the corrected voltage and put 
;             the results beginning in array 40.  The charge collectors are in
;             'Fcup_1c' , 'fcup_2c', ...
;  
;             Get_fcup, 'vlidjj', 'fcup_*c', dist =.15, out=40, ion=7
;
;           Calculate the current of protons at the anode from a diagnostic
;           40 cm away. Assume the energy is in wdf array 3 and store the 
;           results beginning in 40. Use all wdf arrays with 'fcup' in the
;           comment.  Return only the high voltage branch of the arrival 
;           time curve.  Limit results to voltages over 25% of the peak. 
;             get_fcup, 3, 'fcup', dist= .40, /high, ion=1, limit= 0.25, out=40
;-
@com.dat
on_error, 1
      vcor = wdvalid(vcorin, /val, np= np)
      if np eq 0 then begin
        Print, '% Get_FCup: Voltage Array must be provided'
        return
      endif
      if np gt 1 then begin
        print, '% Get_Fcup: Multiple values for Vcor:', vcor
        print, '            First dataset will be used:', wdcomment(vcor(0))
      endif
      vcor = vcor(0)
      get_wdfarray, vc, vcor, /xy
      nelv = n_elements(vc)-2
      if nelv eq wdfnp(vcor-1) then begin
        lcuniform = 1
        t0 = vc(0)
        dt = vc(1)
        vc = vc(2:*)
      endif else begin
        lcuniform = 0
        nelv = nelv/2 + 1
        t = vc(0: nelv-1)
        vc = vc(nelv:*)
        dt = t(1:nelv-1)-t(0:nelv-2)
        if min(dt) lt 0 then begin
          a = sort (t)
          t = t(a)
          vc = vc(a)
          dt = t(1:nelv-1)-t(0:nelv-2)
          if min(dt) eq 0.0 then begin
            print, '% Get_Fcup: Cannot calculate with multiple voltages at a time'
            return
          endif
        endif
      endelse
      ss = size(fcup) 
      nfc = 0
      if ss(ss(0)+1) eq 7 then begin
        fc = wdvalid(fcup(0), npoint = nfc)
      endif else if ss(ss(0)+1) gt 0 then fc = wdvalid(fcup, /valid, npoint= nfc)
      if nfc eq 0 then begin
        print, '% Get_Fcup: Fcup array(s) must be defined'
        return
      endif
      if n_elements (limit) eq 0 then limit = 0.1 else limit = limit > 0
      limit = limit(0) < 0.9
      vmax = max(vc, ivm)
      vlim = limit*vmax
      if ivm eq 0 then i = 0 else begin
        i = ivm-1
        while vc(i) gt vlim and i gt 0 do i = i-1
        if vc(i) lt vlim then i = i+1
      endelse
      if ivm eq (nelv-1) then j = (nelv-1) else begin
        j = ivm+1
        while vc(j) gt vlim and j lt (nelv-1) do j = j+1
        if vc(j) lt vlim then j = j-1
      endelse
      nok = j-i+1
      if nok lt nelv then begin
        vc = vc(i:j)
        if lcuniform eq 1 then t0 = t0+i*dt else begin
          t  =  t(i:j)
          dt  =  dt(i:j-1)
        endelse
        nelv = nok
      endif
      ivm = ivm -i
      if n_elements(ionmass) eq 0 then ionmass = 1.0 $
        else ionmass = float(ionmass(0))
      if n_elements(charge) eq 0 then charge = 1.0 
      ionrat = ionmass/charge(0)
      
      k = 2.0 * 1.6022e-19/ 1.6726e-27/ionrat
;     veloc = sqrt(vc*k)
      if n_elements(dist) eq 0 then dist = 0.3
      if n_elements (nsec) eq 0 then nsec =1 
      trtime = sqrt((dist*dist/k)/vc)
      if nsec ne 0 then begin
        trtime = trtime*1e9 
        xlabel = 'Time (ns)'
      endif else xlabel = 'Time (s)'
      if n_elements (smoo) eq 0 then smoo = 5
      if smoo gt 1 then trtime= smooth(trtime, smoo)
      if keyword_set(highvolt) then mode = 1 else begin
        if keyword_set(lowvolt) then mode = 2 else mode = 1
      endelse
      if mode eq 2 then begin
        mode = 1
        print, 'WARNING: Only high voltage portion will be calculated.'
      endif
      clab = ', M='+strtrim(round(ionmass), 2)+', Q='+strtrim(round(charge),2)
      nelo = n_elements (pout)
      if nelo eq 0 then begin
        nelo = n_elements (output)
        if nelo eq 0 then begin
          Print, '% GET_FCUP: Output arrays must be specified.'
          return
        endif
      endif else output = pout
      if nelo lt nfc then output = [output, 1+output+indgen(nfc-nelo)]

      ok = where(output le 0 or output gt maxwdfarray, nok)
      if nok ne 0 then begin
        print,  '% GET_FCUP: Some output arrays are not valid.'
        print, ' Arrays:', output
        Print, ' Not valid:', output(ok)
        return
      endif
;  calculate arrival times
      if lcuniform eq 1 then begin
        trtime = t0+findgen(nelv)*dt+trtime
;       tmin = min(trtime, i)
;  determine tmin by going from the peak - Start at peak voltage
        dtr =trtime(1:nelv-1)-trtime(0:nelv-2)
        i = ivm
        while (i gt 0 and dtr(i) gt 0) do i = i-1
        if mode eq 1 then begin
          if dtr(i) le 0.0 then i =i+1
          t0 = t0+i*dt
          trtime = trtime(I:*)
          nelv = n_elements(trtime)
          if nelv lt 2 then begin
            Print, '% Get_Fcup: Less than two transit time points'
            return
          endif
          dtr =(dtr(i:*)/(dt +dt)) > 0.0
          dtr = [2*dtr(0), dtr(0:nelv-3)+dtr(1:nelv-2), 2*dtr(nelv-2)]
        endif else begin
          if i lt 2 then begin
            Print, '% Get_Fcup: Less than two transit time points'
            return
          endif
          trtime = trtime(0:i)
          nelv = n_elements(trtime)
          dtr =((trtime(0:nelv-2)-trtime(1:nelv-1))/(dt +dt)) > 0.0
          dtr = [2*dtr(0), dtr(0:nelv-3)+dtr(1:nelv-2), 2*dtr(nelv-2)]
        endelse
      endif else begin
        trtime = t+trtime
;       tmin = min(trtime, i)
;  determine tmin by going from the peak - Start at peak voltage
        dtr =trtime(1:nelv-1)-trtime(0:nelv-2)
        i = ivm 
        while (i gt 0 and dtr(i) gt 0) do i = i-1
        if mode eq 1 then begin 
          if dtr(i) le 0.0 then i =i+1
          t= t(i:*)
          dt= dt(i:*)
          trtime = trtime(I:*)
          nelv = n_elements(trtime)
          if nelv lt 2 then begin
            Print, '% Get_Fcup: Less than two transit time points'
            return
          endif
          dt = [2*dt(0), dt(0:nelv-3)+dt(1:nelv-2), 2*dt(nelv-2)]
          dtr = dtr(i:*) > 0.0
          dtr = [2*dtr(0), dtr(0:nelv-3)+dtr(1:nelv-2), 2*dtr(nelv-2)]/dt
        endif else begin
          if i lt 2 then begin
            Print, '% Get_Fcup: Less than two transit time points'
            return
          endif
          trtime = trtime(0:i)
          t = t(0:i)
          dt = dt(0:i)
          nelv = n_elements(trtime)
          dt = [2*dt(0), dt(0:nelv-3)+dt(1:nelv-2), 2*dt(nelv-2)]
          dtr =(trtime(0:nelv-2)-trtime(1:nelv-1)) > 0.0
          dtr = [2*dtr(0), dtr(0:nelv-3)+dtr(1:nelv-2), 2*dtr(nelv-2)]/dt
        endelse
      endelse
      file = wdffile(vcor-1)
      for i = 0, nfc-1 do begin
        y = get_value(fc(i), trtime)
        y = y*dtr
        if lcuniform eq 1 then $
          put_wdfarray, [t0,dt, y], output(i), clab= wdcomment(fc(i))+clab, $
            xlab=xlabel, ylabel='Current', file = file  else $

          put_wdfarray, [t, y], output(i), clab= wdcomment(fc(i))+ clab, $
             /xy, xlab=xlabel, ylabel='Current', file = file 
      endfor
;
;     return
      end
