; ------------------------------------------------------------------------ 
; $Id: get_iprop.pro,v 1.4 2011/11/15 22:53:11 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
 pro get_iprop, vcorin, ipropin, pout, distance = dist, ionmass = ionmass, $
       nsecond = nsec, charge = charge, output= output, limit = limit, $
         smooth= smoo
;
;+
; NAME: GET_IPROP
; 
; Description:
;       Propagate an ion pulse to a charge collector assuming that they are
;       moving with a velocity corresponding to a specified voltage
;       pulse.  The current versus arrival time is calculated for 
;       Protons by default.
;  IMPORTANT NOTE:
;       In all cases charge is distributed later in time so the result will be 
;         shifted by a half time step.  That is, the current at a time t1
;         is distributed at the charge collector from the arrival time of t1 
;         to the arrival time of t2, where t2 is the next data point in the 
;         original data.
;          
;       Velocity is assumed to be given by:
;         velocity = sqrt( 2.0 * 1.6022e-19/ 1.6726e-27 * Vc*Q/M)
;           where M = 1, 12, 16 for protons, carbon and oxygen, respectively, 
;                 Q is the charge, and Vc is the acceleration voltage
; Format:
;      GET_IPROP, Vcor, current, OUTPUT, [, /Nsecond ] [, distance = distance]
;                     [, ionmass = ionmass] [, CHARGE = charge ]
;                     [, output= output] [, deltat = deltat] [, SMOOTH = smooth]
;                      [, limit = limit]
;
;    where
;        vcor      wdf array for the corrected voltage or a search string
;                  XY data is sorted - Process is terminated if multiple
;                    voltages at the same time.
;        current   wdf array(s) for the charge collectors
;                  or a string for the search
;        Output    If present and a valid wdf number, this will
;                  be used in place of the keyword parameter.
;                  This parameter or the keyword must be provided.
;        nsecond   abscissa units for vcor
;                  default is nsecond = 1, abscissa in nanoseocnds
;        distance  ion propagation distance in meters
;                  default distance = 0.3 m
;        ionmass   Ion mass in amu. Default = 1
;                  IONMASS > 1
;        charge    Ion charge - Default = 1; Charge > 1
;       Note: Only one ion mass and one charge can be used.
;        smooth    Smoothing for the transit time.
;                  Default = 5
;        output    output WDF arrays. Ignored if positional parameter is present
;        limit     Voltage cut; eliminate voltages below this value.
;                  Default is 10% of peak
;                  0 < limit < 0.9 
;
;        Example
;           Calculate the current of protons measured at a charge collector
;             30 cm away.  Assume the energy is in wdf array 3, the current 
;              in 4,  and store the result in 10
;             get_iprop, 3, 4, out = 10 
; < or >      get_iprop, 3, 4, ion=1, char=1, out = 10, dist = 0.30
;
;           Calculate the currrent of Li+ ions measured at at charge collectors
;             15 cm away using 'VLIDJJ' as the corrected voltage and put 
;             the results beginning in array 40.  The current is in the array
;             itot.
;  
;             Get_iprop, 'vlidjj', 'itot', dist =.15, out=40, ion=7
;
;-
@com.dat
on_error, 1
      npar = n_params()
      if npar eq 0 then begin
        print, '% Format:   GET_IPROP, Vcor, current, OUTPUT, [, /Nsecond ]'
        print, '              [, distance = distance] [, ionmass = ionmass]'
        print, '              [, CHARGE = charge ] [, output= output]'
        print, '              [, deltat = deltat] [, SMOOTH = smooth]'
        print, '              [, limit = limit]'
        return
      endif

      vcor = wdvalid(vcorin, /val, np= np)
      if np eq 0 then begin
        Print, '% Get_IProp: Voltage Array must be provided'
        return
      endif
      if np gt 1 then begin
        print, '% Get_IProp: Multiple values for Vcor:', vcor
        print, '            First dataset will be used:', wdcomment(vcor(0))
      endif
      vcor = vcor(0)
      get_wdfarray, array1, vcor, /xy
      nel = wdfnp(vcor-1)
      nelv = nel
      ss = size(ipropin, /type) 
      if ss gt 0 then iprop = ipropin
      nfc = 0
      if ss eq 7 then begin
        fc = wdvalid(iprop[0], npoint = nfc)
      endif else if ss gt 0 then fc = wdvalid(iprop, /valid, npoint= nfc)
      if nfc eq 0 then begin
        print, '% Get_IProp: Current array must be defined'
        return
      endif
      get_wdfarray, array2, fc[0], /xy
      nelb = wdfnp(fc[0]-1)
        max1 = wdfmaxx(vcor-1)
        max2 = wdfmaxx(fc[0]-1)


      get_common_time, array1, array2, max1, max2, nel, nelb, $
        xydata, match_only = 0, sortx = sortx
      if n_elements(array1) lt 4 then begin
        print, '% Get_IProp: No values can be returned due to lack of overlap'
        return
      endif

      if xydata eq 0 then begin
        vc = array1[2:*]
        ic = array2[2:*]
        dt = array1[1]
        nel = n_elements(vc)
        t= array1[0]+findgen(nel)*dt
      endif else begin
        vc = array1[nel:*]
        ic = array2[nel:*]
        t = array1[0:nel-1]
        dt = t[1:nel-1]-t[0:nel-2]
        dt = [dt[0], (dt[1:nel-2]+dt[0:nel-3])*0.5, dt[nel-2]]
      endelse
      nelv = nel
      if n_elements (limit) eq 0 then limit = 0.1 else limit = limit > 0
      limit = limit(0) < 0.9
      vmax = max(vc, ivm)
      vlim = limit*vmax
      if ivm eq 0 then i = 0 else begin
        i = ivm-1
        while vc(i) gt vlim and i gt 0 do i = i-1
        if vc(i) lt vlim then i = i+1
      endelse
      if ivm eq (nelv-1) then j = (nelv-1) else begin
        j = ivm+1
        while vc(j) gt vlim and j lt (nelv-1) do j = j+1
        if vc(j) lt vlim then j = j-1
      endelse
      nok = j-i+1
      if nok lt nelv then begin
        vc = vc[i:j]
        ic = ic[i:j] 
        t = t[i:j] 
        if xydata ne 0 then dt  =  dt(i:j)
        nelv = nok
      endif
      ivm = ivm -i
      if n_elements(ionmass) eq 0 then ionmass = 1.0 $
        else ionmass = float(ionmass(0))
      if n_elements(charge) eq 0 then charge = 1.0 
      ionrat = ionmass/charge(0)
      
      k = 2.0 * 1.6022e-19/ 1.6726e-27/ionrat
;     veloc = sqrt(vc*k)
      if n_elements(dist) eq 0 then dist = 0.3
      if n_elements (nsec) eq 0 then nsec =1 
      trtime = sqrt((dist*(dist/k))/vc)
      if nsec ne 0 then begin
        trtime = trtime*1e9 
        xlabel = 'Time (ns)'
      endif else xlabel = 'Time (s)'
      if n_elements (smoo) eq 0 then smoo = 5
      if smoo gt 1 then trtime= smooth(trtime, smoo)
      clab = ', M='+strtrim(round(ionmass), 2)+', Q='+strtrim(round(charge),2)
      nelo = n_elements (pout)
      if nelo eq 0 then begin
        nelo = n_elements (output)
        if nelo eq 0 then begin
          Print, '% GET_IProp: Output arrays must be specified.'
          return
        endif
      endif else output = pout

      ok = where(output le 0 or output gt maxwdfarray, nok)
      if nok ne 0 then begin
        print,  '% GET_IProp: Some output arrays are not valid.'
        print, ' Arrays:', output
        Print, ' Not valid:', output(ok)
        return
      endif
      output = output[0]
;  calculate arrival times
      trtime = trtime+t
      tmin = min(trtime, i)
      tmax = max(trtime, j)
      newdt = min(dt)
      t0 = floor(tmin)
;
;     get the bin in which the particle arrives and add up the charge
; 
      arriv = (trtime-t0)/newdt
      arriv =[arriv, arriv[nelv-1]-arriv[nelv-2]+arriv[nelv-1]]
      jarriv = long(arriv)
      nnew = max(jarriv) + 4
      tnew = t0 +newdt*findgen(nnew)
      
      deljarr = jarriv[1:nelv]-jarriv[0:nelv-1]
      okneg = where (deljarr lt 0, nokneg)
      deljarr = abs(deljarr)
      flag = make_array(nelv, value =0)
      if nokneg gt 0 then flag[okneg]=1
      charg = make_array(nnew)
;
;    put charge into bins
;
  q = ic*dt
  for i = 0, nelv-1 do begin
    if deljarr[i] eq 0 then charg[jarriv[i]]=charg[jarriv[i]]+q[i] else begin
      qq = q[i]/(arriv[i+1]-arriv[i])
      if flag[i] eq 0 then begin
        charg[jarriv[i]] = charg[jarriv[i]] + qq*(ceil(arriv[i])-arriv[i])
        charg[jarriv[i+1]] = charg[jarriv[i+1]] + $
          qq * (arriv[i+1]-floor(arriv[i+1]))
        if deljarr[i] gt 1 then $
          charg[jarriv[i]+1: jarriv[i]+deljarr[i]-1]= $
            charg[jarriv[i]+1: jarriv[i]+deljarr[i]-1] + qq
      endif else begin
; negative sweep
;       qq = -qq but handle this in the equations
        charg[jarriv[i+1]] = charg[jarriv[i+1]] - $
          qq*(ceil(arriv[i+1])-arriv[i+1])
        charg[jarriv[i]] = charg[jarriv[i]] - qq*(arriv[i]-floor(arriv[i]))
        if deljarr[i] gt 1 then $
          charg[jarriv[i+1]+1: jarriv[i]-1]= $
            charg[jarriv[i+1]+1: jarriv[i]-1] - qq
      endelse
    endelse
  endfor

      charg = charg/newdt
      file = wdffile(vcor-1)
      put_wdfarray, [t0, newdt,charg], output, clab= wdcomment(iprop)+clab, $
            xlab=xlabel, ylabel='Current', file = file
;
      
      end
