; ------------------------------------------------------------------------ 
; $Id: get_match.pro,v 1.7 2011/05/09 16:26:57 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
 Function Get_Match, namein, strarrayin, nelements = nelements, $
       regexpression = regexpression, setdefault = setdefault, $
       showdefault = showdefault, $
       structure = structure, wdfarray = wdfarray, casesen = casesen

;+
;    From a string array return the array elements which contain a specified
;      string.  This may be used for WDVALID, STRVALID, and GET_WDF.
;      Regular expression matching may be used rather than the default 
;        matching defined below. 
;
;    format:
;        array_numbers = GET_MATCH(string , STRING_ARRAY, 
;           RegExpression = regexpression, /SetDefault, /ShowDefault, 
;           NELEMENTS =nelements, WDFARRAY = wdfarray, STRUCTURE = structure, 
;           CASESEN = casesen )
;        where:
;          array_number - an array of integers representing the array elements
;                         which contain STRING
;                         If no matches are found then -1 is returned.
;                         If keywords wdfarray or structure then the array
;                            numbers are returned.
;
;          String       - search string, if it is an array only the first
;                         element is used.
;                         string = '', '$', '^', '^$, '*' 
;                           will return all possible elements
;                          
;                Note: String may end in blanks.
;                      for example: 'cat ' is different from 'cat'
;                      the latter would allow not only'cat dog' but also
;                        'cat1', 'cat2'...
;                      All trailing and leading blanks are stripped from the
;                        string_array.  Therefore 'cat ' would not find 
;                        'cat        ' in string_array; the correct string
;                        would be: 'cat$'
;                 The following special characters can be used:
;                 *  - Wild card indicating 0 to n characters
;                      To use a * in a search string use "\*"
;                 ?  - Wild card indicating 1 and only 1 characters
;                      To use a ? in a search string use "\?"
;                      Note: '^?'
;                 ^  - If this is the first character, indicates that
;                      the string begins the dataset comment.
;                      All other times a ^ is taken literally.
;                      To begin a search string with a literal ^ use:
;                          "*^" or "\^"
;                      All search strings are assumed to begin with "*" unless
;                      a "^" is present. 
;                 $  - If this is the last character, indicates that the
;                      string ends the non-blank portion of the dataset comment.
;                      All other times a $ is taken literally. 
;                      Note: The above words "non-blank".  A search string
;                            of the form 'cat     $' will be simply 'cat$' 
;                      To end a search string with a literal $ use:
;                          "\$"
;                      All search strings are assumed to end with "*" unless
;                      a "$" is present.
;          string_array  - String array.  Each element of string array is
;                          searched for STRING.  If not present, then WDFarray
;                          or STRUCTURE must be set.
;                          Note: string_array = strtrim(string_array, 2) but
;                                the input value is not modified.
;          RegExpression - If set, the IDL STREGEX function will be used rather
;                          than the above procedure.
;                          Default: RegExpression =0
;
;            Caution:  All PFF directory searches will use the scheme described
;                      under the string variable above and the technique 
;                      discussion below and not a regular expression search.
;
;          SetDefault    - If set, The current value of RegExpression and 
;                          CaseSen will become the default values.
;                             DEFAULT:
;                               RegExpression = 0
;                               CaseSen       = 0
;                          No matching is performed; -1 is returned.
;          ShowDefault   - Show the current settings for Set parameters.
;                          No matching is performed; -1 is returned.
;          Nelements     - Number of array elements for which a match is found.
;          WDFARRAY      - If set, WDF dataset comments will be used for 
;                          STRING_ARRAY. If WDFARRAY has two elements, they
;                          will be taken as the bounds for the search.
;          STRUCTURE     - If set, structure array dataset comments will be used
;                          for STRING_ARRAY. If STRUCTURE has two elements, they
;                          will be taken as bounds for the search.
;          CASESEN       - If present and not zero, then search is case
;                          sensitive.  Default is CASESEN=0 (not case sensitive)
;    Example:
;        List the wdf arrays beginning with M and ending in 8
;            print, get_match('^m*8$', /wdf)
;        List the wdf arrays beginning with M and ending in 8 and containing
;          a total of 7 characters
;            print, get_match('^m?????8$', /wdf)
;        Print the elements of titles which contain the letters msp 
;          and end in cc
;            print, titles(get_match('msp*cc$', titles))
;        Set regular expressions as the default matching technique
;        Print the current values.
;            junk = get_match(/set, regex= 1, case= 0, /show)
;
;    Technique:
;         1. All input strings with '*?' are converted to '?*'
;         2. Search string is moved to a work string.
;         3. LENWORK is used as a pointer array to the first character of each
;            substring.
;         4. LENSTR is a length indicator for the length of each substring.
;            EXCEPT LENSTR is negative if a ? is found.  The negative value
;            of LENSTR is the number of successive ? and thus the number of
;            characters to skip.
;         5. If a * follows a ? then more characters can be skipped. Lenwork 
;            is set negative if an * follows a ?, otherwise it is positive
;         6. Leading * are ignored so '*^' is the same as '\^'
;         7. Blanks in string_array are only meaningful if they have non-blank
;            characters on both sides.
;-
    on_error, 1
;
@com.dat
@qsstrcom.dat
common get_match_common, regex_save_flag, case_sens_flag 
compile_opt defint32
nelements= 0L
;
    if keyword_set(wdfarray) and n_elements(initwdf) eq 0 then return, -1
    if keyword_set(structure) and n_elements(structinit) eq 0 then return, -1
;
    if n_elements (regex_save_flag) eq 0 then begin
      regex_save_flag = 0
      case_sens_flag = 0
    endif
    lcquit = 0
    if keyword_set(setdefault) then begin
      if n_elements(regexpression) gt 0 then regex_save_flag = regexpression[0]
      if n_elements(casesen) gt 0 then case_sens_flag = casesen[0]
      lcquit = 1
    endif
    if keyword_set(showdefault) then begin
      print, 'GET_MATCH Default Values'
      print, ' Regular Expression Flag = ', strtrim( regex_save_flag,2)
      if keyword_set(regex_save_flag) then print, $
      '  STREGEX will be used for all comparisons' else $
      Print, '  Simple string search will be used.'

      if keyword_set(case_sens_flag) then print, $
      '  String comparison is Case Sensitive' else print, $
      '  String comparison is NOT Case Sensitive'
      lcquit = 1
    endif
    if lcquit eq 1 then return, -1
    if n_elements(namein) eq 0 then begin
      print, '% GET_MATCH: Search string must be defined'
      return, -1 
    endif else name = namein[0]
    if size(/type, name) ne 7 then begin
      print, '% GET_MATCH: Search string (NAME) must be a string'
      help, name
      return, -1
    endif
    
    if n_elements(casesen) eq 0 then casesen = case_sens_flag else $
        casesen = fix(casesen[0])
    if n_elements(regexpression) eq 0 then regexpression = regex_save_flag $
        else regexpression = regexpression[0]
;   str = strtrim(name, 2)
    str = name
    nels = strlen(str)
  if keyword_set(regexpression) then lcall = 0 else begin
    if nels eq 0 or (nels eq 1 and (str eq '*' or str eq '^' or str eq '$')) $
      then lcall = 1 else begin
      lcall = 0
      istr = 0
;     lchat-Indicates first character is a hat
      lchat = 0
;     lcdollar-Indicates the search string is terminated with a $
      lcdollar = 0
;     lcesc-Indicates the last character was a \
      lcesc = 0
      lenwork = make_array(nels+1, /int)
      lenstr = make_array(nels+1, /int)
      i = 0
      hat = (byte('^'))(0)
      dollar = (byte ('$'))(0)
      space = (byte(' '))(0)
      star = (byte('*'))(0)
      ques = (byte('?'))(0)
      slash = (byte('\'))(0)
      strb = byte(str +'*')
      if nels gt 1 then begin
        if ((strmid(str, 0, 2) eq '^*') or (strb(0) eq star)) then i=1
        ii = i
        iwork = strpos(str, '*?', ii)
        while (iwork gt 0) do begin
          if strb(iwork-1) ne slash then $
            strput,str, '?*', iwork else ii = iwork+1
          iwork = strpos(str, '*?', ii)
        endwhile
      endif
      iwork = 0
      strb = byte(str +'*')
      while (i lt nels) do begin
        if strb[i] eq star then begin
          if (lcesc gt 0) then strput,str, '*', iwork-1 else begin
            if (iwork-lenwork[istr] gt 0) then begin
              lenstr(istr)= iwork-lenwork[istr]
              lenwork[++istr] = iwork
            endif else if (lenstr[istr] lt 0) then begin
              if istr eq 0 then lchat = 0 else begin
                lenwork(istr) = -iwork
                lenwork[++istr] = iwork
              endelse
            endif
          endelse
        endif else if (strb[i] eq ques) then begin
          if (lcesc gt 0) then strput,str, '?', iwork-1 else begin
            if (iwork-lenwork(istr) gt 0) then begin
              lenstr[istr]= iwork-lenwork[istr]
              lenwork[++istr] = iwork
              lenstr[istr]= -1
            endif else lenstr[istr] -= 1
          endelse
        endif else if (((i eq 0) or ((i eq 1) and (lcesc gt 0))) $
          and (strb[i] eq hat)) then begin
          if (lcesc gt 0) then strput,str, '^', iwork-1 else lchat = 1
        endif else if ((i eq nels-1) and (strb[i] eq dollar)) then begin
          if (lcesc gt 0) then strput,str, '$', iwork-1 else begin
            if iwork gt 0 then $
            while ((strb[iwork-1+lchat] eq space)and(iwork gt lenwork[istr])) do iwork -= 1
            if ((iwork gt lenwork[istr]) or (lenstr[istr]lt 0)) then lcdollar = 1
          endelse
        endif else begin
          if (lenstr[istr] lt 0) then lenwork[++istr] = iwork
          strput,str, string(strb[i]), iwork++
        endelse
        lcesc = (strb[i] eq slash) ? 1 : 0
        i += 1
      endwhile
      if (lenstr[istr] lt 0) then lenwork[++istr] = iwork else begin
        lenstr[istr] = iwork-lenwork[istr]
        if (lenstr[istr] ne 0) then lenwork[++istr] = iwork
      endelse
      istr -= 1
      nels = iwork
      if nels le 0  then begin
        if istr lt 0 then lcall = 1 else lcall= (lenstr[istr] lt 0) ? 0 : 1 
      endif else str = strmid(str,0,nels)
      if casesen eq 0 then str = strlowcase(str)
    endelse
  endelse
    nelements = n_elements (strarrayin)
    if nelements gt 0 then begin
      strarray = strarrayin
      if lcall ne 0 then return, indgen(nelements)
      sb = 0
      se = nelements-1
      lcunit = 0
    endif else begin
      lcunit = 1
      if keyword_set (wdfarray) then begin
        if n_elements(wdfarray) gt 1 then begin
          sb = min(wdfarray(0:1), max = se) 
          if sb lt 1 then sb = 0 else sb = sb-1
          if sb gt maxwdfarray then sb = 0
          if se gt maxwdfarray then se = maxwdfarray-1 else se = se-1
          if se le 0 then se = maxwdfarray-1
        endif else begin
          sb = 0
          se = maxwdfarray-1
        endelse
        nelements  = se-sb+1
        if lcall ne 0 then begin
           ok = where(wdfnp(sb:se) ge 2, nelements)+1 + sb
           if nelements eq 0 then ok = -1
           return, ok
        endif
        strarray = wdfcomment
      endif else if keyword_set(structure) then begin
        if n_elements(structure) gt 1 then begin
          sb = min(structure(0:1), max = se)
          if sb lt 1 then sb = 0 else sb = sb-1
          if sb gt maxstructure then sb = 0
          if se gt maxstructure then se = maxstructure-1 else se = se-1
          if se le 0 then se = maxstructure-1
        endif else begin
          sb = 0
          se = maxstructure-1
        endelse
        nelements  = se-sb+1
        if lcall ne 0 then begin
          ok = where(lcvalidstr(sb:se) gt 0, nelements)+1 + sb
          if nelements eq 0 then ok = -1
          return, ok
        endif
        strarray = comment
      endif else begin
        nelements = 0
        print, ' % GET_MATCH: String array must be provided'
        return, -1
      endelse
    endelse
    match = make_array(/int, nelements)
    nelements= 0
  if keyword_set(regexpression) then begin
    if casesen eq 0 then fold = 1 else fold = 0
    for ids = sb, se do begin
      sc = strarray(ids)
      find = stregex(sc, str, fold_case=fold, /boolean)
      if find  then begin
        match(nelements) = ids
        nelements= nelements+1
      endif
    endfor
  endif else begin
    if casesen eq 0 then strarray = strlowcase(strtrim(strarray, 2)) else $
      strarray = strtrim(strarray, 2)
    for ids = sb, se do begin
      sc = strarray(ids)

      find = 1
      iwork = 0
      i = 0l
      isave = -1
      lcexact= 0
      strl = strlen(sc)
;     find = 1 so far so good
;     find = -1 no match
;     find = -2 no match but maybe we can look further down the string.
      while( find ge 0 and i le istr) do begin
        if ((lchat gt 0) and (i eq 0)) then begin
; ......  Initial hat(^)
;         if we don't find the string there is no match.
;           Note: ^* and ^?* are not allowed (latter is really *?)
          if lenstr(0) lt 0 then begin 
            iwork = iwork-lenstr(i)
            if iwork gt strl then find = -1 else begin
              if istr eq 0 and lcdollar gt 0 and iwork lt strl then find =-1
            endelse 
            lcexact = 1
          endif else begin
            find = strpos(sc, strmid(str,lenwork(i), lenstr(i)), 0)
            if find ne 0 then find = -1 else begin
              iwork = lenstr[i]
              if (find eq 0 and (i eq istr) and (lcdollar gt 0)) then begin
                if (strl gt iwork) then find = -1
              endif
            endelse
          endelse
        endif else if ((lcdollar gt 0) and (i eq istr) and (lenstr(i) gt 0)) $
          then begin
; ......  Trailing dollar signal after a string
          lcesc = strl - lenstr(i)
          if (lcesc lt iwork) then find = -1 else begin
            if ((lcexact gt 0) and (lcesc gt iwork)) then find = -2 else $
            find = strpos(sc, strmid(str, lenwork(i), lenstr (i)), lcesc)
          endelse
        endif else begin
;         everything else
          if (lenstr(i) lt  0) then begin
; .....     We have one or more ?
            iwork = iwork -lenstr(i)
; ......... Check on the non-blank length of the string
            if (iwork gt strl) then find = -1
            if (lenwork(i) gt 0) then lcexact = 1
            if lcdollar gt 0 and i eq istr and iwork lt strl then find = -2
          endif else begin
            if lcexact eq 0 then begin
              isave = i
              iwsave = iwork
            endif
;           normal search unless we had a previous ?
            find= strpos(sc, strmid(str, lenwork(i), lenstr (i)), iwork) 
            if find ge 0 then begin
              if ((lcexact ne 0) and (find gt iwork)) then find = -1
              if find ge 0 then iwork = find+lenstr(i) else find = -2
            endif 
            lcexact = 0
          endelse
        endelse
        if ((find eq -2) and (isave ge 0)) then begin 
          i = isave
          iwork = iwsave +1
          find = (iwork gt strl) ? -1 : 1
          lcexact = 0
        endif else i += 1
      endwhile
      if find ge 0 then match[nelements++] = ids
    endfor
  endelse
  match = (nelements gt 0) ? match(0:nelements-1)+ lcunit : -1
  return, match
  end
