; ------------------------------------------------------------------------
; $Id: get_prob.pro,v 1.4 2010/04/21 16:35:23 lpmix Exp $
; ------------------------------------------------------------------------
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
;+
; NAME:
;       GET_PROB
;
; PURPOSE:
;	This function returns the values normal order statistic medians 
;       corresponding to a given fraction.
;       (This function is the inverse of the IDL: Gauss_PDF)
;
; CALLING SEQUENCE:
;	Var = GET_PROB (Fractions)
;       Where:
;       VAR is the value form which a random variable X has has a probability 
;         of FRACTIONS of being less than or equal to VAR if X is distributed 
;         in a standard Gaussian (normal) distribution with a mean of 0.0 and
;         a variance of 1.0
;
; INPUTS:
;       Fractions - Probability that a normal random variable is less than
;                   the variance.
;                   Fractions must be > 0 and < 1.0
;                   Out of range values will be set to 0 or 1.
; OPTIONAL KEYWORD:
;       NPoints   - If specified, Interpolation array will have this number 
;                   of points.  Default = 4096, Minimum = 512
;       MAX_Value - If specified, Interpolation array will go to this value.
;                   Default = 5, Minimum = 1.0
; COMMON BLOCKS:
;       PROB_GAUSS : Used to store the probability distribution function
;
; PROCEDURE:
;       Linear interpolation is used to determine the variance.  A table of
;       4096 values distributed over the interval [0, 5] is used for the 
;       interpolation.
;       The user may specify a different array size with the NPOINTS keyword.
;
; EXAMPLE:
;       Determine the values required to have the central half of the data.
;         print, GET_PROB([.25, .75])
;
; MODIFICATION HISTORY:
; 	Written by:	L. P. Mix, February 25, 2003
;-

Function get_prob, fractin, npoints = npoints, max_value = max_value
on_error, 1
common PROB_GAUSS_COMM, PVAR, PFRAC 


if n_elements(fractin) eq 0 then begin
  PRINT, '% GET_PROB: Fractions must be defined'
  return, 0.0
endif
fract = (fractin > 0.0) < 1.0
if n_elements (NPOINTS) eq 0 then np = 4096 else np = npoints[0] > 512
if n_elements (max_value) eq 0 then maxvalue = 5.0 else $
  maxvalue = max_value[0] > 0.5
if n_elements (pvar) lt np then begin
  pvar = findgen(np)*(maxvalue/float(np-1))
  pfrac = gauss_pdf(pvar)
endif

ok = where(fract lt 0.500, nok)
if nok gt 0 then fract[ok] = 1.0-fract[ok]

var = interpolate(pvar, findex(pfrac, fract))
if nok gt 0 then var[ok] = -var[ok]
return, var

END
