; ------------------------------------------------------------------------ 
; $Id: get_vprop.pro,v 1.3 2007/06/06 20:03:47 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
 pro get_vprop, vcor, distance = dist,ionmass = ionmass, nsecond = nsec, $
       charge = charge, output= output, limit = limit, deltat = deltat, $
       highvolt = highvolt, lowvolt = lowvolt
       
;
;+
; NAME: GET_VPROP
; 
; Description:
;       Propagate an ion pulse to a target assuming that they are
;       moving with a velocity corresponding to a specified voltage
;       pulse.  The energy versus arrival time is calculated for 
;       Protons, Carbon, and Oxygen by default. 
;       Velocity is assumed to be given by:
;         velocity = sqrt( 2.0 * 1.6022e-19/ 1.6726e-27 * Vc*Q/M)
;           where M = 1, 12, 16 for protons, carbon and oxygen, respectively, 
;                 Q is the charge, and Vc is the acceleration voltage
; Format:
;      GET_VPROP, Vcor [, Nsecond = nsec] [, distance = distance]
;                      [, ionmass = ionmass] [, CHARGE = charge ]
;                      [, output= output] [, deltat = deltat]
;                      [, limit = limit] [, /HighVolt] [, /LowVolt]
;
;    where
;        vcor      wdf array for Vcor limited to peak
;        nsecond   abscissa units for vcor
;                  default is nsecond = 1, abscissa in nanoseocnds
;        distance  ion propagation distance in meters
;                  default distance = 0.3 m
;        ionmass   Ion mass in amu. Default = 1
;                  IONMASS > 1
;        charge    Ion charge - Default = 1; Charge > 1
;            NOTE: IonMass and charge must have the same number of elements or
;                  a single element. for example:
;        ionMass= [1, 4, 12, 16], charge = 2       is ok  (single Charge)
;        ionMass= 16, charge = [1,2,3,4]           is ok  (single Mass)
;        ionMass= [1,4,12,12], charge = [1,1,1,2]  is ok  (same Mass & Charge)
;        ionMass= [12, 16], charge = [1,2,3,4]    is not ok (not same or single)
;        output    output WDF arrays
;                  default are vcor+1, vcor+2, ....
;        limit     Voltage cut; eliminate voltages below this value.
;                  Default is 10% of peak
;                  0 < limit < 0.9 
;        HighVolt  If set only the high voltage portion of the arrival time
;                  will be saved. Data saved as uniform data with spacing deltat.
;        LowVolt   If set only the low voltage portion of the arrival time
;                  will be saved. This is ignored if HighVolt is set.
;                  Data saved as uniform data with spacing deltat. 
;                  Default is to return the entire arrival time curve.
;        Deltat    Minimum Time spacing. Used only if HIGHVOLT or LOWVOLT is set
;                  Default is average initial spacing*0.2
;                  If Deltat eq 0, then data will be saved exactly.
;        Example
;           Calculate the arrival time of protons at a target 30 cm away.
;           Assume the energy is in wdf array 3 and store the result in 10
;           Return all possible voltages.
;             get_vprop, 3, ion = 1, out = 10 
;
;           Calculate the arrival time of protons at a target 40 cm away.
;           Assume the energy is in wdf array 3 and store the result in 4
;           Return only the high voltage branch of the arrival time curve.
;           limit results to voltages over 25% of the peak
;             get_vprop, 3, dist= .40, /high, ion=1, limit = 0.25 
;-
on_error, 1
      w2i, t, vc, vcor
      nelv = n_elements (vc)
      if n_elements (deltat) eq 0 then deltat=(max(t,min= mm)-mm)*0.2/(nelv-1)
      if n_elements (limit) eq 0 then limit = 0.1 else limit = limit > 0
      limit = limit(0) < 0.9
      vmax = max(vc, ivm)
      vlim = limit*vmax
      if ivm eq 0 then i = 0 else begin
        i = ivm-1
        while vc(i) gt vlim and i gt 0 do i = i-1
        if vc(i) lt vlim then i = i+1
      endelse
      if ivm eq (nelv-1) then j = (nelv-1) else begin
        j = ivm+1
        while vc(j) gt vlim and j lt (nelv-1) do j = j+1
        if vc(j) lt vlim then j = j-1
      endelse
      nok = j-i+1
      if nok lt nelv then begin
        vc = vc(i:j)
        t  =  t(i:j)
        nelv = nok
      endif
      nelm = n_elements(ionmass)
      if nelm  eq 0 then begin
        ionmass = 1.0
        nelm = 1
      endif else ionmass = float(ionmass) > 1.0

      nelc = n_elements(charge)
      if nelc eq 0 then begin
        charge = make_array(nelm, value = 1)
        nelc = nelm
      endif else charge = charge > 1

      if nelc ne nelm then begin
        if nelc lt nelm then begin
          if nelc gt 1 then begin
            print, '%GET_VPROP - Number of charge values must equal' $
         +' number of mass values'
            print, '# Mass-', strtrim(nelm, 2), '# Charge-',strtrim(nelc, 2)
            return
          endif
          ionmass = ionmass/charge(0)
        endif else begin
          if nelm gt 1 then begin
            print, '%GET_VPROP - Number of mass values must equal' $
         +' number of charge values'
            print, '# Mass-', strtrim(nelm, 2), '# Charge-',strtrim(nelc, 2)
            return
          endif
          ionmass = ionmass(0)/charge
        endelse
      endif else ionmass = ionmass/charge

      nelm = n_elements(ionmass)
      
      k = 2.0 * 1.6022e-19/ 1.6726e-27
      vprop = make_array(nelv, nelm)
        
      for i = 0, nelm-1 do vprop(*, i)= sqrt((k/ionmass(i))*vc)

      if n_elements(dist) eq 0 then dist = 0.3
      if n_elements (nsec) eq 0 then nsec =1 
      if nsec ne 0 then dst = dist*1e9 else dst = dist
      if keyword_set(highvolt) then mode = 1 else begin
        if keyword_set(lowvolt) then mode = 2 else mode = 0
      endelse
      clab = make_array(nelm, /string)
      for i = 0, nelm-1 do clab(i) = 'Mass/Q-'+strtrim(ionmass(i), 2)
      nelo = n_elements (output)
      if n_elements (output) eq 0 then begin
        output = vcor+1
        nelo=1
      endif
      if nelo lt nelm then output = [output, 1+output+indgen(nelm-nelo)]

;  calculate new times
      for i = 0, nelm-1 do begin
        if mode eq 0 then i2w, t+dst/vprop(*, i), vc, output(i), $
         clab= clab(i), /xy, xlab='Time (ns)',ylab='Voltage' else  begin
         ttt = t + dst/vprop(*, i)
         tmin = min(ttt, imin)
         if deltat eq 0 then begin
           if mode eq 1 then i2w,ttt(imin:*), vc(imin:*),output(i), $
             clab= clab(i), xlab='Time (ns)',ylab='Voltage', /xy $
           else i2w,ttt(0:imin), vc(0:imin), output(i), $
             clab= clab(i), xlab='Time (ns)',ylab='Voltage', /xy
         endif else begin
           if mode eq 1 then i2w,ttt(imin:*), vc(imin:*),output(i),eps=1.0e-6, $
             clab= clab(i), xlab='Time (ns)',ylab='Voltage' , delta= deltat $
           else i2w,ttt(0:imin), vc(0:imin), output(i), delta= deltat,  $
             clab= clab(i), xlab='Time (ns)',ylab='Voltage', eps= 1.0e-6
         endelse
        endelse
      endfor
;
;     return
      end
