; ------------------------------------------------------------------------ 
; $Id: hisplt.pro,v 1.16 2010/12/09 18:49:04 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
 pro hisplt, filenamein, outfile, outfile = out_file, wdfsave = wdfsave, $
       merge = merge, pff_write = pff_write, single = single, ierror = ierror, $
       dump_number = dumpnum, write_all = write_all,  group_part = group_part, $
       full_precision = full_precision, save_sets = save_sets, $
       count = count, nwdf = nwdf, npff = npff, $
       quiet = quiet, allsave = allsave, uniform = uniform, multiple = multiple
;
;  allsave, dump_number, full_precision, merge, outfile, pff_write,
;  save_sets, single, wdfsave, write_all
;+
;
;       HISPLT
;
; PURPOSE:
;       Read an Alegra unformatted history file or a series of history files.
;       Store the data in a PFF file.  Data from multiple files is stored using
;       only one time.  If data exists in more than one file then, the data from
;       the latest file is used.  If MERGE is set, all data points are saved.
;
; CALLING SEQUENCE:
;
;       HISPLT [, IN_FILE] [, OUTFILE] [, OUT_FILE = out_file]
;              [, FULL_PRECISION = full_precision] [, /ALLSAVE]
;              [, SAVE_SETS = save_sets] [, WDFSAVE = wdfsave]
;              [, MERGE = merge] [, PFF_WRITE= pff_write] [, /Group_Part]
;              [, /SINGLE] [, /DUMP_NUMBER] [ , MULTIPLE = multiple]
;              [, IERROR = ierror] [, /QUIET] [, UNIFORM = uniform]
;
; PARMAETERS:
;
;       IN_FILE        - Input file - If not valid, then use pickfile.
;                        Multiple files - The number of arrays, particles and 
;                        materials is checked but array names are assumed to be
;                        the same in all files.
;       OUTFILE        - Output file
;                        Default:  OUTFILE = INFILE_BARE+'.pff'
;                        where INFILE_BARE is IN_FILE without any suffix
;       OUT_FILE       - Output file; this is ignored if OUTFILE is specified.
;       SAVE_SETS      - String or array of strings which are used to select
;                        datasets to store in wdf arrays.
;                        NOTE:  GET_MATCH is used and CASESEN =0 is a keyword for strings
;                        If SAVE_SETS is zero, then no datasets will be saved to
;                        a WDF array.
;                        If SAVE_SETS is one, then the user will be prompted 
;                        for the arrays to save to a WDF array.
;                        Default:  SAVE_SETS = 0
;                        If PFF_WRITE is not zero, then only those datasets 
;                        written to a file can be saved.
;       WDFSAVE        - First dataset for saving data to wdf arrays.
;                        Ignored if SAVE_SETS is zero or missing.
;                        Additional WDF arrays will be created if necessary to
;                        store the datasets.
;                        DEFAULT: WDFSAVE = 1
;       MERGE          - If set, merge all data keeping multiple times.
;       FULL_PRECISION - If set, PFF file will be written in full precision.
;                        Default is to store with 16 bit precision.
;       ALLSAVE        - All datasets that are written to a PFF file will 
;                        be stored in WDF arrays
;                        If no datasets are written to a PFF file, all 
;                        datasets will be stored in WDF arrays.
;       WRITE_ALL      - Write all datasets to a PFF file.
;                        If set, PFF_WRITE = '*'
;       PFF_WRITE      - If PFF_WRITE is a string or an array of strings, then
;                        only those arrays will be saved to the pff file.
;                        NOTE:  GET_MATCH is used and CASESEN =0 is a keyword for strings
;                        If PFF_WRITE is zero, then no datasets will be written
;                        to the PFF file.
;                        If PFF_WRITE is one, then the user will be prompted 
;                        for the arrays to write to the PFF file.
;                        DEFAULT: PFF_WRITE=0
;       NWDF           - Number of WDF arrays saved
;       NPFF           - Number of PFF datasets saved
;       Group_Part     - If set, group all variables of a particle together
;                      - Default: Group_Part = 1
;       SINGLE         - If set, read only the selected file.
;       MULTIPLE       - If set, read all of the files found without user interaction
;                        If not set, the user selects the desired files from a widget.
;                        DEFAULT:  MULTIPLE =0
;       DUMP_NUMBER    - If set, each dataset will be stored as a function of 
;                        the dump number.  The abscissa will range from zero to
;                        the number of output time steps.
;                 Note:  This allows data to be stored as a uniform dataset and
;                        reduces file size by a factor of 2.
;       UNIFORM        - If the data is known to be uniform, this keyword will
;                        save the data using TIME[1]-TIME[0] as DT.
;                        If UNIFORM is not equal to 1, then DT = UNIFORM
;       IERROR         - Returns zero unless an error
;                        Returns -1 if search string is not found
;                        Returns 1 for most other errors
;       QUIET          - Suppress extra messages
;
; PROCEDURE:  
;       Read the data and store it as required.
;       Naming convention for particle data is 'NAME.P00x'
;       Naming convention for material data is 'NAME.M00x'
;
;
; EXAMPLES:
;       Convert all data in a selected file or a series of selected files to a 
;       WDF arrays file.
;          HISPLT, /al
;
;       Convert data in a selected file or a series of flies to a PFF file.  Use
;       a widget to select the files to put into wdf arrays and to save to the 
;       pff file.  
;          HISPLT, /write, /save, /pff
;
;       Convert data from the series 'dyn.his' to a pff file and save the datasets
;       of pressure and density beginning in wdfarray 5
;          HISPLT, 'dyn.his', save=['pressure', 'density'], wdf=5, /pff
;
;-
;
on_error, 1
;
@com.dat
;
common savehisplt_picker,  oldfilter, oldpath
;
;     see if we are initialized
      if n_elements(initwdf) eq 0 then wdfinit
;
      ierror=0
;
  if n_elements (filenamein) eq 0 then filename=0 else filename = filenamein[0]
;
  if n_elements(oldpath) eq 0 then cd, current = oldpath
  path = oldpath
  if size(filename , /type) ne 7 then begin
    filename =  dialog_pickfile(/must_exist, /read, path = path, get_path = newpath)
    if strlen(filename) lt 1 then begin
      print, '% HISPLT: No File was Selected - RETURNING'
      ierror = 1
      return
    endif
    oldpath = newpath
  endif
  info = file_info(filename)
  if (not info.exists) then begin
    print, '% HISPLT: File does not exist'
    print, '          ', filename
    
    filename =  dialog_pickfile(/must_exist, /read, path = path, get_path = newpath)
    if strlen(filename) lt 1 then begin
      print, '% HISPLT: No File was Selected - RETURNING'
      ierror = 1
      return
    endif
    oldpath = newpath
    info = file_info(filename)
  endif
  if (not info.read) then begin
    print, '% HISPLT: File is not readable by the user'
    print, '          ', filename
    ierror = 1
    return
  endif
  filename = info.name
  if n_elements(group_part) eq 0 then group_part =1
  if keyword_set(single) then begin 
    files = filename
    c =1
  endif else begin
    ok = strpos(filename, '.his', /reverse_search) +4
    if ok lt 4 then begin
      ok = strpos(filename, '.', /reverse_search)+1
      if ok lt 1 then filename = filename+'*' else $
        filename = strmid(filename, 0, ok)+'*'
      files = file_search(filename, count = c)
      if c gt 0 then ok = where (strpos(files, '.pff') lt 0, c)
      if c eq 0 then begin
;       This should never happen with other checks.
        print, '% HISPLT: File does not exist'
        print, '          ', filename
        return
      endif
      files = files[ok]
    endif else begin
      files = filename
      filename = strmid(filename, 0, ok)
      files1 = file_search(filename, count = c1)
      filename = filename + '_'
      files2 = file_search(filename+'*', count = c2)
      if c1 gt 0 then files= [files, files1]
      if c2 gt 0 then files= [files, files2]
      files = files[uniq(files, sort(files))]
      ok = where (strpos(files, filename) ge 0, nok, comp = comp, ncomp= ncomp)
      if nok gt 0 then begin
        ii = long(strmid(files[ok], strlen(filename)))
        k = sort(ii)
        if ncomp gt 0 then files = [files[comp], files[ok[k]]] else $
          files = files[ok[k]]
      endif
      c = n_elements(files)
    endelse
    if c gt 1 and not keyword_set(multiple) then begin
      ok = get_array_var(files, /select_only, title='Select Files to Convert')
      if ok[0] ge 0 then files= files[ok] else begin
        print, '% HISPLT: No Files Selected'
        ierror = 1
        return
      endelse
;      files = files[ok]
      c = n_elements(files)
    endif
  endelse
  if n_elements (outfile) eq 0 then $
    if n_elements (out_file) ne 0 then outfile= out_file[0] else outfile =''
  outfile = strtrim(outfile[0], 2) 
  ok1 = strpos(files[0], path_sep(), /reverse_search) +1
  ok2= strpos(files[0], '.', ok1)
  print_start = ok1
  if ok2 lt ok1 then file_body = strmid(files[0], ok1) else $
        file_body = strmid(files[0], ok1, ok2-ok1)
  if strlen(outfile) eq 0 then begin
    if ok2 gt 0 then outfile= strmid(files[0], 0, ok2)+'.pff' else $
      outfile = files[0]+'.pff'
  endif
  if n_elements(pff_write) eq 0 then pff_write = 0
  if keyword_set(full_precision) then full_precision=1 else full_precision=0
  if keyword_set(write_all) then pff_write = '*'
  lc_pff_file_open = 0
  if keyword_set(pff_write) then begin
    outfile = find_newfile(outfile, /quiet)
    createpff, outfile, error = ier
  
    if ier ne 0 then begin
      print, '% HISPLT: Error opening PFF file.'
      print, '          File: ', outfile
      ierror =1
      return
    endif 
    lc_pff_file_open = 1
    print, '% HISPLT: PFF file opened.'
    print, '          File: ', outfile
    pff_precision, 0, FULL_PRECISION = full_precision
  endif else write_all = 0
;
  if n_elements(wdfsave) eq 0 then wdfsave = 1 else wdfsave = wdfsave[0]
  if keyword_set (allsave) then save_sets = 1
  if n_elements(save_sets) eq 0 then save_sets = 0
  wdfsave = get_save_array (wdfsave, get_maxwdf())
  wdf_add = 1000
  if wdfsave eq get_maxwdf() then set_maxwdfarray, wdfsave+wdf_add
    
  clabel = ' - '
  if keyword_set (merge) then merg = 1 else merg = 0

  file_times = make_array(2*c)
  ntimes = make_array(c, /long)
  for i = 0 , c-1 do begin
    filename = files[i]
    clabel = clabel + filename +';;'
    if i eq 0 then begin
      read_hisplt, filename, time, dt, cpu, cycle, $
        tday, wallc, jobid, code, point_hist, point_var, point_unit, $
        mat_hist, mat_var, mat_unit, glob_hist, glob_var, glob_unit, $
        pnt_id, mat_id, point_type, ntime, $
        nupts, numat, nupvar, numvar, nugvar, title, quiet = quiet
      ntimes[i] = ntime 
      file_times[2*i:2*i+1] = [time[0], time[ntime-1]]
      np_vec = [nupts, numat, nupvar, numvar, nugvar]
      mxtime = ntime
    endif else begin
      read_hisplt, filename, time0, dt0, cpu0, cycle0, $
        tday0, wallc0, jobid0, code0, $
        point_hist0, point_var0, point_unit0, $
        mat_hist0, mat_var0, mat_unit0, $
        glob_hist0, glob_var0, glob_unit0, pnt_id0, mat_id0, $
        point_type0, ntime0, nupts0, numat0, nupvar0, $
        numvar0, nugvar0, title0, quiet = quiet
      ntimes[i] = ntime0
      file_times[2*i:2*i+1] = [time0[0], time0[ntime0-1]]
      vec = [nupts0, numat0, nupvar0, numvar0, nugvar0]
    endelse
    if i gt 0 then begin
      if min(vec-np_vec, max = mm) ne 0 or mm ne 0 then begin
        print, '% HISPLT:  Multiple files must have the same number of variables'
        print, '  Initial vectors:', np_vec
        print, '  Current file: ', filename
        print, '  Current vectors:', vec
        ierror =1
        return
      endif 
      if keyword_set (merg) then nn = 0L else begin
        ok = where (time ge time0[0], nn)
        if nn gt ntime/2 then begin
         print, '% HISPLT: Caution over half of current data is replaced with'
          print, '          File: ', filename
        endif
      endelse
      ntime = ntime -nn
      if i eq 1 then begin
        mxtime = ntime + ntime0
        delmax = 256L
        while delmax lt mxtime do delmax = delmax + 128
        if c gt 2 then begin
          mxt = 512L
          while mxt lt mxtime do mxt = mxt+256
          mxtime = mxt
        endif
        if nupvar*nupts gt 0 then begin
          tmp = make_array(nupvar, nupts, mxtime)
          tmp[*,*, 0:ntime-1]= point_hist[*,*,0:ntime-1]
          tmp[*,*, ntime:ntime+ntime0-1] = point_hist0
          point_hist = temporary(tmp)
        endif
        if numvar*numat gt 0 then begin
          tmp = make_array(numvar, numat, mxtime)
          tmp[*,*, 0:ntime-1]= mat_hist[*,*,0:ntime-1]
          tmp[*,*, ntime:ntime+ntime0-1] = mat_hist0
          mat_hist = temporary(tmp)
        endif
        if nugvar gt 0 then begin
          tmp = make_array(nugvar, mxtime)
          tmp[*, 0:ntime-1]= glob_hist[*,0:ntime-1]
          tmp[*, ntime:ntime+ntime0-1] = glob_hist0
          glob_hist = temporary(tmp)
        endif 
        tmp = make_array(mxtime)
        tmp[ 0:ntime-1]= time[0:ntime-1]
        tmp[ ntime:ntime+ntime0-1] = time0
        time = temporary(tmp)
        tmp = make_array(mxtime)
        tmp[ 0:ntime-1]= dt[0:ntime-1]
        tmp[ ntime:ntime+ntime0-1] = dt0
        dt = temporary(tmp)
        tmp = make_array(mxtime)
        tmp[ 0:ntime-1]= cpu[0:ntime-1]
        tmp[ ntime:ntime+ntime0-1] = cpu0
        cpu = temporary(tmp)
        tmp = make_array(mxtime)
        tmp[ 0:ntime-1]= cycle[0:ntime-1]
        tmp[ ntime:ntime+ntime0-1] = cycle0
        cycle = temporary(tmp)
        ntime = ntime + ntime0
      endif else begin
        nt = ntime + ntime0
        if nt gt mxtime then begin
          if i eq c-1 then mxtime = nt else begin
            mxt = mxtime + delmax
            while mxt lt nt do mxt = mxt +delmax
            mxtime = mxt
          endelse
          if nupvar*nupts gt 0 then begin
            tmp = make_array(nupvar, nupts, mxtime)
            tmp[*,*, 0:ntime-1]= point_hist[*,*,0:ntime-1]
            point_hist = temporary(tmp)
          endif
          if numvar*numat gt 0 then begin
            tmp = make_array(numvar, numat, mxtime)
            tmp[*,*, 0:ntime-1]= mat_hist[*,*,0:ntime-1]
            mat_hist = temporary(tmp)
          endif
          if nugvar gt 0 then begin
            tmp = make_array(nugvar, mxtime)
            tmp[*, 0:ntime-1]= glob_hist[*,0:ntime-1]
            glob_hist = temporary(tmp)
          endif
          tmp = make_array( mxtime)
          tmp[ 0:ntime-1]= time[0:ntime-1]
          time = temporary(tmp)
          tmp = make_array( mxtime)
          tmp[ 0:ntime-1]= dt[0:ntime-1]
          dt = temporary(tmp)
          tmp = make_array( mxtime)
          tmp[ 0:ntime-1]= cpu[0:ntime-1]
          cpu = temporary(tmp)
          tmp = make_array( mxtime)
          tmp[ 0:ntime-1]= cycle[0:ntime-1]
          cycle = temporary(tmp)
        endif
        if nupvar*nupts gt 0 then point_hist[*,*, ntime:nt-1] = point_hist0
        if numvar*numat gt 0 then mat_hist[*,*, ntime:nt-1] = mat_hist0
        if nugvar gt 0 then glob_hist[*, ntime:nt-1] = glob_hist0
        time[ ntime:nt-1] = time0
        dt[ ntime:nt-1] = dt0
        cpu[ ntime:nt-1] = cpu0
        cycle[ ntime:nt-1] = cycle0
        ntime = nt
      endelse
    endif
  endfor
  if ntime lt  3 then begin
    print, '% HISPLT: Inadequate data - Must have 3 time values.'
    ierror =1
    return
  endif

;
; Select PFF Variables
;
  nvar = nugvar+nupvar+numvar
  nok = nvar
  ok = lindgen(nvar)
  var = make_array(nvar, /string)

  if nugvar gt 0 then var[0:nugvar-1] = glob_var
  if nupvar gt 0 then var[nugvar:nugvar+nupvar-1] =  point_var+'.P'
  if numvar gt 0 then var[nugvar+nupvar:nvar-1] = mat_var+'.M'

  if not keyword_set(write_all) and keyword_set(pff_write) then begin
    if size(pff_write, /type) eq 7 then lc_inquire = 0 else lc_inquire = 1
    if lc_inquire eq 1 and keyword_set (pff_write) then begin
      ok = get_array_var(var, /select_only, $
        title='Select Variables to Write to PFF file')
      nok = n_elements(ok)
    endif else if lc_inquire eq 0 then begin
      maxok = 2048L > nvar
      npw = n_elements(pff_write)
      ok = make_array(maxok)
      nok = 0
      for i = 0, npw-1 do begin
        n = get_match(pff_write[i], var, nelem = nelem, casesen=0)
        if nelem gt 0 then begin
          if nok+nelem gt maxok then begin
            ok = [ok, ok]
            maxok = maxok+maxok
          endif
          ok[nok:nok+nelem-1] = n
          nok = nok+nelem
        endif
      endfor
      if nok gt 0 then begin
        ok = ok[0:nok-1]
        ok = ok[uniq (ok , sort(ok))]
        nok = n_elements(ok)
      endif
    endif
    if nok eq 0 then begin
      print, '% HISPLT:  No variables selected for output to PFF file.'
      ok = get_array_var(var, /select_only, $
          title='Select Variables to Write to PFF file')
      nok = n_elements(ok)
    endif
  endif

  okwrite = ok
  oksave = ok
  nokwrite = nok
;
; Select save Variables
;
  work = get_maxwdf()
  if keyword_set(save_sets) then $
   lc_search = (size(save_sets, /type) eq 7) ?  1 : 0 else lc_search = 0
  if keyword_set(allsave) then begin
    noksave = n_elements(oksave)
    oksave = [oksave, -1]
  endif else if lc_search eq 0 and keyword_set(save_sets) then begin
    ok= get_array_var(var[oksave], /select_only, $
        title='Select Variables to Save to WDF array')
    oksave = [oksave[ok], -1]
    noksave = n_elements(oksave) -1
  endif else if keyword_set(save_sets) then begin
    maxok = 2048L > nvar
    ok = make_array(maxok, /long)
    npw = n_elements(save_sets)
    nok = 0
    for i = 0, npw-1 do begin
      n = get_match(save_sets[i], var[okwrite], nelem = nelem, casesen= 0)
      if nelem gt 0 then begin
        if nok+nelem gt maxok then begin
          ok = [ok, ok]
          maxok = maxok+maxok
        endif
        ok[nok:nok+nelem-1] = n
        nok = nok+nelem
      endif
    endfor
    if nok gt 0 then begin
      ok = ok[0:nok-1]
      ok = ok[uniq (ok , sort(ok))]
      nok = n_elements(ok)
    endif else begin
      print,  ' *---------*---------*---------*---------*--------*'
      print,  ' % HISPLT: No variables match the save string'
      print,  '           Quitting with no action'
      if (lc_pff_file_open) then $
        print, '          PFF file opened" ', outfile
      ierror = -1
      return
    endelse
    oksave = [okwrite[ok], -1]
    noksave = nok
  endif else begin
    oksave = -1
    noksave = 0
  endelse
  if noksave lt nvar then lc_save_head = 0 else lc_save_head =1
;
; added a point to oksave to avoid index out of range on the checks
;
; help, oksave, noksave
  if not keyword_set(pff_write) then begin
    nokwrite = noksave
    if nokwrite gt 0 then okwrite = oksave[0:noksave-1]
    allsave = 1
  endif

  if nokwrite gt 0 then begin
    okglob = where (okwrite lt nugvar, nokglob)
    if nokglob gt 0 then begin
      okglob = okwrite[0:nokglob-1]
      if nokglob lt nokwrite then okwrite = okwrite[nokglob:*] 
    endif
  endif else nokglob = 0

  okwrite = okwrite - nugvar
  nokwrite = nokwrite -nokglob
  if nokwrite gt 0 then begin
    okpoint = where (okwrite lt nupvar, nokpoint)
    if nokpoint gt 0 then begin
      okpoint = okwrite[0:nokpoint-1]
      if nokpoint lt nokwrite then okwrite = okwrite[nokpoint:*]
    endif
  endif else nokpoint = 0

  okwrite = okwrite -nupvar
  nokwrite = nokwrite -nokpoint

  if nokwrite gt 0 then begin
    okmat = okwrite
    nokmat = nokwrite
  endif else nokmat = 0


; np_vec = nupts, numat, nupvar, numvar, nugvar
  
  if keyword_set(dumpnum) then delta = 1 else begin
    delta = 0
    dumpnum = 0
  endelse
  if keyword_set(uniform) then begin
    if uniform eq 1 then delta = time[1]-time[0] else delta = uniform
    if delta eq 0.0 then delta = time[2]-time[1]
  endif
  ntimesp = long (ntimes/32768)
  ntimep =  long (ntime/32768)
  ntime_out = ntime-32768*ntimep
  ntimes_out = ntimes-32768*ntimesp

  np_vec = [c, np_vec, ntimep,ntime_out, ntimesp, ntimes_out, nokglob, $
        nokpoint, nokmat, full_precision, dumpnum, merg ]
if not keyword_set(quiet) then begin
  print, ' Number of points: ', strtrim(nupts,2)
  print, ' Number of materials: ', strtrim(numat,2)
  print, ' Number of point variables: ', strtrim(nupvar,2)
  print, ' Number of material variables: ', strtrim(numvar,2)
  print, ' Number of global variables: ', strtrim(nugvar,2)
  print, ' File         Number of times, initial time, final time'
  for i = 0, c-1 do print, strmid(files[i], print_start), $
     ' ', ntimes[i], file_times[2*i:2*i+1]
  if keyword_set(delta) then $
    print, ' Data defined uniform time with DT: ', strtrim(delta, 2)
endif
  if keyword_set(pff_write) then $
    wriifl, np_vec, file_times, tlabel='HISPLT File', $
      clabel =  file_body+clabel, ap = 104
  if ntime lt mxtime then begin
;        if nupvar*nupts gt 0 then point_hist[*,*, ntime:nt-1] = point_hist0
;        if numvar*numat gt 0 then mat_hist[*,*, ntime:nt-1] = mat_hist0
;        if nugvar gt 0 then glob_hist[*, ntime:nt-1] = glob_hist0
;
    if nupvar*nupts gt 0 then point_hist = temporary(point_hist[*,*,0:ntime-1])
    if numvar*numat gt 0 then mat_hist = temporary(mat_hist[*,*,0:ntime-1])
    if nugvar gt 0 then glob_hist = temporary(glob_hist[*,0:ntime-1])
    time = temporary(time[ 0:ntime-1])
    dt = temporary(dt[ 0:ntime-1])
    cpu = temporary(cpu[ 0:ntime-1])
    cycle = temporary(cycle[ 0:ntime-1])
  endif

  if work eq wdfsave and keyword_set(save_sets) then begin
    set_maxwdfarray, work+wdf_add
    work = get_maxwdf()
  endif
  work_save = work
  if keyword_set(save_sets) then work = wdfsave

  if keyword_set(delta) then begin
    xlab = (keyword_set(dumpnum)) ? 'Dump Cycle' : 'Time (s)'
    put_wdfarray, [0, delta, time], work, clabel = 'Time', $
      xlab = xlab, ylab = 'Time (s)', /no_copy, file = files[0]
    if keyword_set(pff_write) then wri, work
    if keyword_set(save_sets)  then begin
      if  lc_save_head gt 0 then work = work+1
      if work eq work_save then begin
        set_maxwdfarray, work+wdf_add
        work_save = get_maxwdf()
      endif
    endif
    put_wdfarray, [0, delta, dt], work, clabel = 'Time Step', $
      xlab = xlab, ylab = 'Time Step (s)', /no_copy, file = files[0]
    if keyword_set(pff_write) then wri, work
    if keyword_set(save_sets)  then begin
      if  lc_save_head gt 0 then work = work+1
      if work eq work_save then begin
        set_maxwdfarray, work+wdf_add
        work_save = get_maxwdf()
      endif
    endif
    put_wdfarray, [0, delta, cpu], work, clabel = 'CPU Time', $
      xlab = xlab, ylab = 'CPU Time (s)', /no_copy, file = files[0]
    if keyword_set(pff_write) then wri, work
    if keyword_set(save_sets)  then begin
      if  lc_save_head gt 0 then work = work+1
      if work eq work_save then begin
        set_maxwdfarray, work+wdf_add
        work_save = get_maxwdf()
      endif
    endif
    put_wdfarray, [0, delta, cycle], work, clabel = 'Cycle Number', $
      xlab = xlab, ylab = 'Cycle', /no_copy, file = files[0]
    if keyword_set(pff_write) then wri, work
    if keyword_set(save_sets)  then begin
      if  lc_save_head gt 0 then work = work+1
      if work eq work_save then begin
        set_maxwdfarray, work+wdf_add
        work_save = get_maxwdf()
      endif
    endif
  endif else begin
    xlab = 'Time (s)'
    put_wdfarray, [time, dt], work, clabel = 'Time Step', $
      xlab = xlab, ylab = 'Time Step (s)', /xydata, /no_copy, file = files[0]
    if keyword_set(pff_write) then wri, work
    if keyword_set(save_sets)  then begin
      if  lc_save_head gt 0 then work = work+1
      if work eq work_save then begin
        set_maxwdfarray, work+wdf_add
        work_save = get_maxwdf()
      endif
    endif
    put_wdfarray, [time, cpu], work, clabel = 'CPU Time', $
      xlab = xlab, ylab = 'CPU Time (s)', /xydata, /no_copy, file = files[0]
    if keyword_set(pff_write) then wri, work
    if keyword_set(save_sets)  then begin
      if  lc_save_head gt 0 then work = work+1
      if work eq work_save then begin
        set_maxwdfarray, work+wdf_add
        work_save = get_maxwdf()
      endif
    endif
    put_wdfarray, [time, cycle], work, clabel = 'Cycle Number', $
      xlab = xlab, ylab = 'Cycle', /xydata, /no_copy, file = files[0]
    if keyword_set(pff_write) then wri, work
    if keyword_set(save_sets)  then begin
      if  lc_save_head gt 0 then work = work+1
      if work eq work_save then begin
        set_maxwdfarray, work+wdf_add
        work_save = get_maxwdf()
      endif
    endif
  endelse
;
; Global Variables
;
  isave = 0
  if keyword_set(save_sets) then begin
    if not keyword_set(allsave) then begin
      wdfsave = work
      work = work_save
    endif
  endif
  
  if nokglob gt 0 then begin
    okwrite = temporary(okglob)
    for i = 0, nokglob-1 do begin
      if not keyword_set(allsave) and oksave[isave] eq okwrite[i] then begin
        work_save = work
        work = wdfsave
        lcsave = 1 
      endif else lcsave = 0
      if keyword_set(delta) then $
        put_wdfarray, [0, delta, reform (glob_hist[okwrite[i], *])], work, $
          clabel =  glob_var[okwrite[i]], xlab = xlab, /no_copy, $
          ylab = glob_var[okwrite[i]]+ ' ('+glob_unit[okwrite[i]]+')',  $
          file = files[0] $
      else put_wdfarray, [time, reform (glob_hist[okwrite[i], *])], work, $
          clabel =  glob_var[okwrite[i]], xlab = xlab, /no_copy, /xydata, $
          ylab = glob_var[okwrite[i]]+ ' ('+glob_unit[okwrite[i]]+')', $
          file = files[0]
      if keyword_set(pff_write) then wri, work
      if (keyword_set (allsave) or lcsave eq 1) then begin
        isave = isave+1
        work = work +1
        if work eq work_save then begin
          set_maxwdfarray, work+wdf_add
          work_save = get_maxwdf()
        endif
        if lcsave eq 1 then begin
          wdfsave = work
          work = work_save
        endif
      endif
    endfor
  endif

  oksave = oksave - nugvar

  if nokpoint gt 0 and nupvar*nupts gt 0 then begin
    okwrite = temporary(okpoint)
    ffor = ceil(alog10(nupts)) > 3
    sfor = strtrim(ffor, 2)
    ffor = '(I'+sfor+'.'+sfor+')'
    plabel = make_array(nupts, value = '')
    xp = make_array(nupts)
    yp = make_array(nupts)
    zp = make_array(nupts)

    for j = 0, nupts-1 do plabel[j] = '.P'+string(j, format= ffor)
    i = get_match('XPOSITION', point_var, nelem = nelem, casesen=0)
    if nelem eq 1 then xp = reform(point_hist[i, *, 0]) else $
       print, '% HISPLT:  Unable to determine point X-position.'
    i = get_match('YPOSITION', point_var, nelem = nelem, casesen=0)
    if nelem eq 1 then yp = reform(point_hist[i, *, 0]) else $
       print, '% HISPLT:  Unable to determine point Y-position.'
    i = get_match('ZPOSITION', point_var, nelem = nelem, casesen=0)
    if nelem eq 1 then zp = reform(point_hist[i, *, 0]) else $
       print, '% HISPLT:  Unable to determine point Z-position.'
    for i = 0,  nupts-1 do plabel[i]= plabel[i] + ' - ['+  $
      strtrim(xp[i], 2)+', '+strtrim(yp[i], 2)+', '+strtrim(zp[i], 2)+'] '

    if keyword_set (group_part) then begin
      sav_isave = isave
      for j = 0, nupts-1 do begin
        for i  = 0, nokpoint-1 do begin
          if i eq 0 then isave = sav_isave
          if not keyword_set(allsave) and oksave[isave] eq okwrite[i] then begin
            work_save = work
            work = wdfsave
            lcsave = 1
          endif else lcsave = 0
          clab =  point_var[okwrite[i]]
          ylab =  point_var[okwrite[i]]+' ('+point_unit[okwrite[i]]+')'
          if keyword_set(delta) then $
            put_wdfarray, [0, delta, reform (point_hist[okwrite[i], j, *])], work, $
              ylab = ylab, file = files[0], xlab = xlab, /no_copy, $
              clab = clab+plabel[j] $
          else put_wdfarray, [time, reform (point_hist[okwrite[i], j, *])], $
              work, ylab = ylab, file = files[0], xlab = xlab, $
              /no_copy, /xydata, $
              clab = clab+plabel[j]
          if keyword_set(pff_write) then wri, work
          if (keyword_set (allsave) or lcsave eq 1) then begin
            work = work +1
            if work eq work_save then begin
              set_maxwdfarray, work+wdf_add
              work_save = get_maxwdf()
            endif
          endif
          if (keyword_set (allsave) or lcsave eq 1) then isave = isave+1
          if lcsave eq 1 then begin
            wdfsave = work
            work = work_save
          endif
        endfor
      endfor

    endif else begin
      for i = 0, nokpoint-1 do begin
        if not keyword_set(allsave) and oksave[isave] eq okwrite[i] then begin
          work_save = work
          work = wdfsave
          lcsave = 1     
        endif else lcsave = 0
        clab =  point_var[okwrite[i]]
        ylab =  point_var[okwrite[i]]+' ('+point_unit[okwrite[i]]+')'
        for j = 0, nupts-1 do begin
  
          if keyword_set(delta) then $
            put_wdfarray, [0, delta, reform (point_hist[okwrite[i], j, *])], work, $
              ylab = ylab, file = files[0], xlab = xlab, /no_copy, $
              clab = clab+plabel[j] $
          else put_wdfarray, [time, reform (point_hist[okwrite[i], j, *])], $
              work, ylab = ylab, file = files[0], xlab = xlab, $ 
              /no_copy, /xydata, $
              clab = clab+plabel[j]
          if keyword_set(pff_write) then wri, work
          if (keyword_set (allsave) or lcsave eq 1) then begin
            work = work +1
            if work eq work_save then begin
              set_maxwdfarray, work+wdf_add
              work_save = get_maxwdf()
            endif
          endif
        endfor
        if (keyword_set (allsave) or lcsave eq 1) then isave = isave+1
        if lcsave eq 1 then begin
          wdfsave = work
          work = work_save
        endif
      endfor
    endelse
  endif
  oksave = oksave - nupvar
  if (nokmat gt 0) and (numvar*numat gt 0) then begin
    okwrite = temporary(okmat)
    plabel = make_array(numat, value = '')
    ffor = ceil(alog10(numat)) > 2
    sfor = strtrim(ffor, 2)
    ffor = '(I'+sfor+'.'+sfor+')'
    for j = 0, numat-1 do plabel[j] = '.M'+string(j, format= ffor)

    for i = 0, nokmat-1 do begin
      if (not keyword_set(allsave) and (oksave[isave] eq okwrite[i])) then begin
        work_save = work
        work = wdfsave
        lcsave = 1
      endif else lcsave = 0
      clab =  mat_var[okwrite[i]]
      ylab =  mat_var[okwrite[i]]+' ('+mat_unit[okwrite[i]]+')'
      for j = 0, numat-1 do begin
        if keyword_set(delta) then $
          put_wdfarray, [0, delta, reform (mat_hist[okwrite[i], j, *])], work, $
            ylab = ylab, file = files[0], xlab = xlab, /no_copy, $
            clab = clab+plabel[j] $
        else put_wdfarray, [time, reform (mat_hist[okwrite[i], j, *])], $
            work, ylab = ylab, file = files[0], xlab = xlab, $
            /no_copy, /xydata, $
            clab = clab+plabel[j]
        if keyword_set(pff_write) then wri, work
        if (keyword_set (allsave) or lcsave eq 1) then begin
          work = work +1
          if work eq work_save then begin
            set_maxwdfarray, work+wdf_add
            work_save = get_maxwdf()
          endif
        endif
      endfor
      if (keyword_set (allsave) or lcsave eq 1) then isave = isave+1
      if lcsave eq 1 then begin
        wdfsave = work
        work = work_save
      endif
    endfor
  endif
  count = work - wdfsave
  nwdf = count
  if keyword_set(pff_write) then npff = ndspff(0) else npff =0L
end
