; ------------------------------------------------------------------------ 
; $Id: hisplt_hdr.pro,v 1.3 2007/06/06 20:03:48 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
 Pro hisplt_hdr,  fid, fileid = fileid, tlabel = tlabel, $
              clabel = clabel, aptype= aptype, namefile = filename, $
              precision = precision, files = files, nfiles = nfiles, $
              npoint = npoint, nmaterial = nmaterial, npointvar = npointvar, $
              nmatvar = nmatvar, nglobvar = nglobvar, ntimes = ntimes, $
              nfiletime = nfiletime, times = times, merge = merge,  $
              time_history = time_history, quiet = quiet, ier = ier, $
              short_files = short_files, path_files = path_files
;
;+
;
;  Reads the header (first) dataset of a pff file written from a HISPLT file.
;  The HISPLT parameters are printed unless QUIET is set.
;  Procedure options are explained below.
;   
;  Format:  HISPLT_HDR, fileid, FILEID = fileid, TLABEL = tlabel,
;             CLABEL = clabel, APTYPE= aptype, NAMEFILE = filename,
;             PRECISION = precision, NFILES = nfiles, FILES = files,
;             NPOINT = npoint, NMATERIAL = nmaterial, NPOINTVAR = npointvar, 
;             NMATVAR = nmatvar, NGLOBVAR = nglobvar, NTIME = ntime, 
;             NFILETIME = nfiletime, TIMES = times, MERGE = merge, 
;             TIME_HISTORY = time_history, QUIET = quiet,
;             SHORT_FILES = short_files, PATH_FILES = path_files, IER = ier
;
;  where:
;    fileid  - File id of the PFF file containing the dataset
;            - Default is the current file
;  Keywords:
;
;    FILEID  - File id of the PFF file containing the dataset
;              Ignored if fileid is not zero
;    TLABEL  - Dataset type label 
;    CLABEL  - Dataset comment label - Core name of the hisplt file(s).
;    APTYPE  - Application dataset type
;    NAMEFILE- Up to 160 characters describing the file
;    PRECISION- If set (precision = 1) then file is written in full (4 byte) 
;              precision.  If precision = 0, the file is written in standard
;              (2 byte) precision.
;    NFILES  - Number of files which have been combined for the current file.
;    FILES   - Files which have been combined for the current file.
;    NPOINT  - Number of points 
;    NMATERIAL- Number of materials
;    NPOINTVAR- Array of 2 elements.  
;              First is the number of point-variables in the PFF file.
;              Second is the number of point-variables in the HISPLT file.
;    NMATVAR - Array of 2 elements.  
;              First is the number of material-variables in the PFF file.
;              Second is the number of material-variables in the HISPLT file.
;    NGLOBVAR- Array of 2 elements.  
;              First is the number of point-variables in the PFF file.
;              Second is the number of point-variables in the HISPLT file.
;    NTIME   - Number of time-steps in the pff file
;    NFILETIME- Number of time-steps in each HISPLT file
;    TIMES   - Problem times for data in each of the HISPLT files.
;              TIMES = TIMES[2, NFILES] where:
;                TIMES[0, *] are the intial times for each HISPLT file
;                TIMES[1, *] are the final times for each HISPLT file
;    MERGE   - Flag for processing of the HISPLT files.
;              MERGE = 1; all times are kept for every HISPLT file
;              MERGE = 0; Times from later files overwrite times from previous
;                files if there is overlap.
;    TIME_HISTORY - If set, each dataset has time as the abscissa.
;                   If not set, each dataset is versus the dump indexi
;                   Dump_index = [0,1,2,3,4,5...]
;    PRECISION - PFF dataset precision.  PRECISION Values:
;               0 - Data stored as a 2-byte integer with offset and scale factor
;               1 - Data stored as a 4-byte real
;
;    QUIET   - If present and not zero, do not print any data
;              This is ignored if the dataset comment or dataset type labels 
;              are larger than the available buffer.
;    SHORT_FILES - FILES without the path (if it is present).
;    PATH_FILES  - Path for SHORT_FILES 
;    IER      - Error flag, zero indicates no error
; 
;  Examples:
;    Read file information from the current file.
;      HISPLT_HDR
;-
;
on_error, 1
      common pffcom, initpff, pff_exe
      common readhdr_com, pfftyplabels
      if (n_elements(initpff) eq 0) then initpff = 0
      ier = -98
      if initpff eq 0 then begin
        print, '% HISPLT_HDR: No PFF files are open'
        return
      endif
      if n_elements(fid) eq 0 then begin
        if n_elements (fileid) gt 0 then fid = long(fileid[0]) else fid = 0L
      endif else fid = long(fid[0])
      n = ndspff(fid)
      If n eq 0  then begin
        print, '% HISPLT_HDR: No PFF datasets in this file'
        ier = -99
        return
      endif
      readhdr, /quiet , 1, pfftype = pfftype, aptype = aptype, $
        clabel = clabel, tlabel = tlabel, namefile = filename, $
        /nostrip
      if pfftype ne 6 or strpos(tlabel, 'HISPLT') lt 0 then begin
        print, '% HISPLT_HDR: PFF file is not a HISPLT file.'
        print, '        File: ', filename
        return
      endif
;  initialize parameters
      readifl, iarray, flist, farray, 1
      st = strpos(clabel, ' - ') 
      if st gt 0 then begin
        files = strmid(clabel, st+3 )
        files = strsplit(files, ';;', /extract)
        ok1 = strpos(files[0], path_sep(), /reverse_search) +1
        if ok1 gt 0 then begin
          path_files = strmid(files[0], 0,ok1)
          short_files = strmid(files, ok1)
        endif  else begin
          path_files = ''
          short_files = files
        endelse
        clabel = strmid(clabel, 0, st)
      endif else begin
        files = ''
        short_files = ''
        path_files = ''
      endelse
      
      nfiles = iarray[0]
      npoint = iarray[1]
      nmaterial = iarray[2]
      ntime = iarray[6]*32768 +iarray[7]
      ix = 8+nfiles
      nfiletime = iarray[8:ix-1]*32768 +iarray[ix:ix+nfiles-1]
      ix = ix+nfiles
      nglobvar = [iarray[ix],iarray[5]] 
      npointvar = [iarray[ix+1],iarray[3]] 
      nmatvar = [iarray[ix+2],iarray[4]] 
      precision = iarray[ix+3]
      timehistory = iarray[ix+4] ? 0L: 1L
      merge = iarray[ix+5]
      times = temporary(flist)
      if n_elements(times) eq nfiles*2 then $
        times = reform(times, 2, nfiles) else $
        print, '% HISPLT_HDR: Error reading file times'
      ier = 0L
      if keyword_set(quiet) then return
      print, ' Data from file: ', filename
      print, ' Tlabel: ' , tlabel
      print, ' Clabel: ' , clabel
      print, ' Number of files: ', strtrim(nfiles,2)
       print, ' Number of points per dataset: ', strtrim(ntime,2)

  print, ' Number of points: ', strtrim(npoint,2)
  print, ' Number of materials: ', strtrim(nmaterial,2)
  print, ' Number of global variables: ', strtrim(nglobvar[0],2), $
         ' from total of ', strtrim(nglobvar[1],2)
  print, ' Number of point variables: ', strtrim(npointvar[0],2), $
         ' from total of ', strtrim(npointvar[1],2)
  print, ' Number of material variables: ', strtrim(nmatvar[0],2), $
         ' from total of ', strtrim(nmatvar[1],2)
  if keyword_set (precision) then ppp = ' full (4 Byte) ' else $
           ppp = ' PFF (2 Byte) '
  print, ' PFF file written with', ppp, 'precision.'
  if keyword_set (timehistory) then $
     print, ' Datasets written as time-histories' else $
     Print, ' Datasets written as a function of the dump index.'
  if keyword_set(merge) then print, $
  ' All data was merged into datasets; multiple times may be present' $
     else print, ' Datasets are single-valued vs time.'
  if strlen (path_files) gt 1 then $
    print, ' File path for HISPLT files: ', path_files

  print, ' File         Number of times, initial time, final time'
  for i = 0, nfiles-1 do print, short_files[i], nfiletime[i], $
             times[2*i:2*i+1]

      end
