; ------------------------------------------------------------------------ 
; $Id: i2w.pro,v 1.8 2011/01/21 17:21:44 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
  pro i2w, space_in, image_in, wdf, clabel = clab, file = file, $
      xlabel = xlab, ylabel = ylab, epsilon = epsilon, delta = deltain, $
      xydata = xydata, lowres= lowres, get_delta = delta_only, $
      endpoint = endpoint
;+
; format:  i2w [, space], image, wdf, clabel = clab, file = file, $
;      xlabel = xlab, ylabel = ylab, epsilon = epsilon, delta = delta, $
;      xydata = xydata, /lowres, get_delta= get_delta
;     this procedure converts two idl arrays to an wdf array.
;     space is the x array  or a two element array of x0 and dx
;       Default = [0.0, 1.0]
;       Space is required for XY data.
;     image is the y array
;     wdf is the wdf array number
;  Note: the following strings will be stripped of spaces.
;     clabel is dataset comment (used for title of plots)
;     ylabel is y-axis (ordinate)  or block label
;     xlabel is x-axis (abscissa) label
;     file is file or source of these dataset values
;     epsilon is the presumed accuracy of the values.  If two abscissa values
;            closer than epsilon in relative accuracy ie. (xj -xi)/max (abs(x))
;            is less than epsilon then they are assumed equal
;              DEFAULT is 1/2^16 = 1.5e-5
;     delta is the spacing of the uniformly spaced array
;     NOTE:  If the data is sparse then a small value of delta should be 
;            specified to insure that all features are resolved.
;     xydata if present and not zero, indicates the x and y arrays are to 
;            be stored exactly as passed.  If they are used for add, subtract, 
;            etc, then they are frequently converted to a uniform timebase.
;     lowres is set to reduce the assumed resolution of the input data by
;            a factor of 15. This is form ascii data with only 6 digits input.
;     get_delta if present and not zero then space and image will be sorted, 
;                duplicate values averaged and a delta calculated and returned
;                in wdf
;     Note: delta only is ignored for XY data
;  Caution: Space and image must be actual variables for return values to
;                 be stored when "GET_DELTA" is set.
;  Correct - i2w, space, image, delta, /get_delta
;  WRONG!  - i2w, array(0:np-1), array(np:*), delta, /get_delta
;                
; NOTE:
;     image is assumed to be a function and space is sorted.
;     multiple values of image at a given space value are averaged.
;     if Space is not uniformly spaced then the data will be linearly
;     interpolated using an interval, delta, equal to the minimum point
;     spacing in xarray or producing the maximum of 30000 points or the number
;     of points in xarray. If delta is spacified then its value will be used.
; Note: If image is already on a uniform time base, then the CHA
;           command will be used to insure a point spacing of DELTA.
;-
;     endpoint is last abscissa value
;     include the common block
@com.dat
;
      on_error, 1
;     see if we are initialized
      if n_elements(initwdf) eq 0 then wdfinit
      npar= n_params()
      if npar eq 0 then begin
        print,'% Format: I2W [, X-array] , Y-array, WDF_Array_number, '
print,'         xlabel = xlab, ylabel = ylab, epsilon = epsilon, delta = delta,'
print,'         clabel = clab, file = file,'
print,'         xydata = xydata, /lowres, /get_delta'
        return
      endif
      if keyword_set(delta_only) then wdf = 1
      if n_elements(xydata) eq 0 then xydata = 0
      if xydata ne 0 then begin
        i2xy, space_in, image_in, wdf, clabel = clab, file = file, $
          xlabel = xlab, ylabel = ylab
        return
      endif
      if npar eq 1  then begin
        print, '% I2W: Two or more parameters must be specified '
	return
      endif 
;     move input variables to work arrays
      if npar eq 2 then begin
        nel = n_elements(space_in)
        if nel gt 2 then image = space_in else begin
          print, '% I2W - Ordinate parameter must have more than 2 elements'
          return
        endelse
        lcdoit =1
        nelsp = 2
        space = [0,1]
        wdf = get_save_array(image_in, 0)
      endif else begin
        nel = n_elements(image_in)
        if nel gt 2 then image = image_in else begin
          print, '% I2W - Ordinate parameter must have more than 2 elements'
          return
        endelse
        wdf = get_save_array(wdf, 0)
        nelsp = n_elements(space_in)
        if nelsp gt 2 then begin
          space = space_in
          lcdoit = 0
        endif else begin
          lcdoit = 1
          if nelsp eq 2 then space = space_in(0:1) else space = [0, 1]
          nelsp = 2
        endelse
      endelse
;     set arrays to the same size
;     determine if wdf array is good
;
      if wdf eq 0 then begin
        print, '% I2W:  Waveform data array must be specified.'
        print, '        WDF array must be greater than 0 and </=', maxwdfarray
	print, ' All existing arrays have data'
        return
      endif

;     remove any unused dimensions
;
      space = reform (space)
      image = reform (image)
      n = size(space)
      m = size(image)
;     Make sure the arrays are good
;
      if (n(0) ne 1) or (m(0) ne 1) then begin
        print, 'x and y arrays must be one dimensional to convert to WDF array'
         return
      endif
    if lcdoit eq 1 then begin
      wdfarray= [space, image]
    endif else begin
      if (nelsp lt nel) then begin
        print,'%I2W: X array must be at least as large as the Y array'
         print,'   Xarray size, Yarray size = ', n(1), m(1)
        return
      endif
      if nel lt nelsp then space= temporary(space (0:nel-1))
      
;     calculate a uniform spacing and determine if it fits the data
      delta = (space[nel-1] - space[0])/float(nel-1)
      diff = space[1:nel-1] - space[0:nel-2]
      maxdiff = max(diff, min=mindiff)
      if (mindiff lt 0.0) then begin
;     need to sort the data
        ind = sort (space)
        space = space[ind]
        image = image[ind]
        ind = 0
        delta = (space[nel-1] - space[0])/float(nel-1)
        diff = space[1:nel-1] - space[0:nel-2]
        maxdiff = max(diff, min=mindiff)
      endif
      maxx = float(space[nel-1]-space[0])

      if n_elements(epsilon) eq 0 then epsilon = min([1.5258789e-5, 0.5/nel])
;     need to eliminate multiple x values
; use the criteria of nel /2^22 with a minimum of 1/2^15 
; calculate an effective nel to account for large offsets
; if lowres is set then allow a factor of 15 greater change in delta
      if n_elements (lowres) eq 0 then lowres = 1 else lowres = (lowres eq 0) ?  1 : 15
; see if almost uniform with an offset.
;  make sure that we don't have some small spacing which we desire
      tempd = (n_elements(deltain) gt 0) ? deltain[0] : maxdiff
      if ((maxdiff-mindiff) gt max([abs(space[0]), abs(space[nel-1]), $
        max([nel, 128])*delta ])*lowres/4194304.) and $
        (mindiff/maxx lt epsilon and mindiff lt 0.5*tempd) then begin
        ok = where (diff/maxx lt epsilon, ind)
        if ind gt 0 then begin
          i = 0L
          while i lt ind do begin
            add = 1L
            while ((i+add lt ind) && (ok[i] +add eq ok[i+add])) do add++
            space[ok[i]: ok[i]+add] = total(space[ok[i]: ok[i]+add])/ (add+1)
            image[ok[i]: ok[i]+add] = total(image[ok[i]: ok[i]+add])/ (add+1)
            i = i+add
          endwhile
        endif
        kk = uniq(space)
        space = space[kk]
        image = image[kk]
        nel = n_elements(space)
        if nel le 2 then begin
          print, '% I2W: Number of elements must be greater than 2'
          return
        endif
        diff = space[1:nel-1] - space[0:nel-2]
        ok = where (diff/maxx lt epsilon, ind)
        if ind ne 0 then begin
          print, '% I2W: Problem with the removal of duplicate X values'
          return
        endif 
        delta = (space[nel-1] - space[0])/float(nel-1)
        maxdiff = max(diff, min=mindiff)
      endif
; use the criteria of nel /2^22 with a minimum of 1/2^15 
; calculate an effective nel to account for large offsets
; if lowres is set then allow a factor of 10 greater change in delta
      if (maxdiff-mindiff) lt max([abs(space(0)), abs(space(nel-1)), $
        max([nel, 128])*delta ])*lowres/4194304. then begin
;       uniform array
        if keyword_set(delta_only) then begin
          space_in = space(0) + delta*findgen(nel)
          image_in = temporary(image[0:nel-1])
          wdf = delta
          endpoint = space_in[nel-1]
          return
        endif
        if m(2) ne 6 then wdfarray = make_array(nel+2, /float) $
               else wdfarray = make_array(nel+2, /complex)
        wdfarray(0) = space[0]
        wdfarray(1) = delta
        wdfarray(2:*) = image[0:nel-1]
        put_wdfarray, wdfarray, wdf, /no_copy
        if n_elements(clab) gt 0 then wdfcomment[wdf-1]= strtrim(clab,2) else  $
          wdfcomment[wdf-1] = '  '
        if n_elements(file) gt 0 then wdffile[wdf-1]= strtrim(file,2) else  $
          wdffile(wdf-1) = '  '
        if n_elements(xlab) gt 0 then wdfxlabel(wdf-1)= strtrim(xlab, 2) else  $
          wdfxlabel(wdf-1) = '  '
        if n_elements(ylab) gt 0 then wdfylabel(wdf-1)= strtrim(ylab, 2) else  $
          wdfylabel(wdf-1) = '  '
        if n_elements (deltain) gt 0 then cha, wdf, delta = deltain
        return
      endif
      maxnp = max([nel, 30000])
      if n_elements (deltain) gt 0 then delta = deltain else $
        delta = mindiff > (space(nel-1) - space(0))/float(maxnp)
      start = space(0)
      stop = space(nel-1)
;     insure that delta preserves the start and end points
      ndelta =long ( 1.00001* (stop -start) / delta)
      if (stop gt start+ndelta*delta + epsilon*maxx) then $
        delta = (stop-start)/float(ndelta+1)
      if keyword_set(delta_only) then begin
        space_in = temporary(space)
        image_in = temporary(image)
        wdf = delta
        endpoint = stop
        return
      endif
      put2uniform, space, image, nel, start,delta, stop, wdfarray
      image = 0
      space = 0
    endelse

    put_wdfarray, wdfarray, wdf, /no_copy
    if n_elements(clab) gt 0 then wdfcomment(wdf-1)= strtrim(clab,2) else  $
      wdfcomment(wdf-1) = '  '
    if n_elements(file) gt 0 then wdffile(wdf-1)= strtrim(file,2) else  $
      wdffile(wdf-1) = '  '
    if n_elements(xlab) gt 0 then wdfxlabel(wdf-1)= strtrim(xlab, 2) else  $
      wdfxlabel(wdf-1) = '  '

    if n_elements(ylab) gt 0 then wdfylabel(wdf-1)= strtrim(ylab, 2) else  $
      wdfylabel(wdf-1) = '  '
    end
