; ------------------------------------------------------------------------ 
; $Id: int.pro,v 1.5 2010/09/20 14:16:19 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
 Pro  Int, wdf1, wdf2 , moment = moment, keep= keep, $
    clabel = clab, ylabel= ylab, xlabel = xlab, double = double
;+
;
;   Integrate an wdf array
;     This procedure uses a simple double precision integration scheme.
;     Result is stored as a float unless DOUBLE is set.
;
;   FORMAT: Int, wdf1 [, wdf2] [, MOMENT = moment] [, KEEP= keep]
;           [, clabel = clab] [, ylabel= ylab] [, xlabel = xlab] 
;           [, double = double]
;   where:
;     wdf1, wdf2 - WDF array numbers
;             If wd2 is present, then wd2 = INT(wd1) else wd1 = INT(wd1)
;     MOMENT - Allows higher order integrals - moment may be real or integer
;              but x must be gt 0 if x is not a positive integer 
;              default is moment = 0, gives int (y dx)
;              moment = 1 gives int(y x dx)
;              moment = 2 gives int(y x^2 dx)
;              CAUTION: if x is nanoseconds, x^n can underflow!
;     KEEP   - If set, then the integration will will keep the same X values. 
;              See description below.
;              Default: KEEP = 0 ; if uniform data
;                       KEEP = 1 ; if xy data.
;     DOUBLE - If set, the data is stored as double precision.
;
;   Description:
;     An integral is performed on the input waveform array.
;     The value of KEEP determines the type of integral.
;   In the discussion that follows:
;     The original WDF values are Y at locations X.
;       There are NP values of X and Y
;     The integral values are Yi at locations Xi 
;     The original spacing between points is DX
;     DX[i] = X[i+1] - X[i]
;
;     KEEP equals zero  (See keyword description above for default value)
;       The number of points is increased by 1 and the first point is set to zero
;       at a location dx/2 before the first point.
;       Xi and Yi have (NP+1) values
;         Yi[0] = 0.0
;         Xi[0] = X[0] - DX[0]*O.5
;       The next point is given by:
;         Yi[1] = Y[0] + DX[0]*Y[0] 
;         Xi[1] = X[0] + DX[0]*O.5
;       Or in general:
;         Yi[i+1] = Y[i] + DX[i]*Y[i] 
;         Xi[i+1] = X[i] + DX[i]*O.5
;         
;       For MOMENT not equal to 0, X-values are given by the stored values.
;       That is, for MOMENT = M:
;         Yi[i+1] = Y[i] + DX[i]*Y[i]*(X[i]^M)
;
;     KEEP is not zero (See keyword description above for default value)
;       The integral values are at the current X values. 
;       Xi and Yi have NP values
;         Yi[0] = 0.0
;         Xi[0] = X[0] 
;       The next point is given by:
;         Yi[1] = Y[0] + DX[0]*(Y[0] +Y[1])*0.5
;         Xi[1] = X[1] 
;       Or in general:
;         Yi[i+1] = Y[i] + DX[i]*(Y[i]+Y[i+1])*0.5
;         Xi[i+1] = X[i+1] 
;
;       For MOMENT not equal to 0, X-values are given by the mid-point of 
;          input X-values.
;       That is, for MOMENT = M:
;         Yi[i+1] = Y[i] + DX[i]*(Y[i]+Y[i+1])*0.5*((X[i]+X[i+1])*0.5)^M
;
;   Examples:
;     Integrate array 2 and store the result in array 7
;       int, 2, 7
;     Integrate array 1 and store the result in array 1
;       int, 1
;     Calculate the first moment of array 1 and store the result in array 1
;       int, 1, moment = 1
;-
@com.dat
 on_error, 1
;
      if n_elements(initwdf) eq 0 then wdfinit
      nparams = n_params()
      if nparams lt 1 then begin
        print, 'INT must have 1 parameter'
        return
      endif
;
      if n_elements(wdf1) eq 0 then begin
        print, 'Parameter to INT must be defined'
        return
      endif
      if wdf1 lt 1 or wdf1 gt maxwdfarray then begin
        print, 'wdf array must be greater than 0 and </=', maxwdfarray
        return
      endif
;
      if nparams gt 1 then save = get_save_array(wdf2, wdf1) else save = wdf1
;
      np = wdfnp(wdf1-1)
      if np eq 0 then begin
        print, 'No data in the first array'
        return
      endif
;
      if n_elements(moment) eq 0 then moment = 0
      get_wdfarray, array1, wdf1, /xy
      if (n_elements(array1) eq np+2) then begin
; Uniform DATA
        xydata = 0
        if n_elements (keep) eq 0 then keep = 0
        if keyword_set(keep) then begin
          array2= make_array(np+2, double = double)
          array1(2:np) = (array1(2:np)+array1(3:np+1))*0.5d0
          if moment ne 0 then begin
            x = (dindgen(np-1)*array1(1)+ $
             (array1[0]+array1[1]*0.5d0))^moment
            array2[3:np+1] = total(array1[2:np]*x, /double, /cumulative)
          endif else $
            array2[3:np+1] = total(array1[2:np], /double, /cumulative)
          array2(0) = array1(0)
        endif else  begin
          array2= make_array(np+3, double = double)
          if moment ne 0 then begin
            x = (dindgen(np)*array1(1)+array1(0))^moment
            array2[3:np+2] = total(array1[2:np+1]*x, /double, /cumulative)
          endif else $
            array2[3:np+2] = total(array1[2:np+1], /double, /cumulative)
          array2(0) = array1(0) - array1(1)*0.5
        endelse
        array2(1) = array1(1)
        array2(3:*) = array2(3:*)*array1(1)
      endif else begin
; X-Y DATA
        xydata = 1
        if n_elements (keep) eq 0 then keep = 1
        np2=np+np
        if keyword_set(keep) then begin
          array2= make_array(np2, double = double)
          array1(np:np2-2) = (array1(np:np2-2)+array1(np+1:np2-1))*0.5d0
          dx = array1(1:np-1)-array1(0:np-2)
          if moment ne 0 then begin
            x = (0.5*(array1(1:np-1)+array1(0:np-2)))^moment
            array2[np+1:np2-1]=total(array1[np:np2-2]*x*dx,/double,/cumulative)
          endif else $
            array2[np+1:np2-1]=total(array1[np:np2-2]*dx,/double,/cumulative)
          array2(0:np-1) = array1(0:np-1)
        endif else  begin
          array2= make_array(np2+2, double=double)
          dx0= array1(1)-array1(0)
          dxnp = array1(np-1)-array1(np-2)
          array2(0) = array1(0) - dx0*0.5
          array2(1:np) = [(array1(0:np-2)+array1(1:np-1))*0.5, $
                     array1(np-1) + 0.5*dxnp]
          dx = array2(1:np)-array2(0:np-1)
          if moment ne 0 then begin
            x = (array1(0:np-1))^moment
            array2[np+2:np2+1]=total(array1[np:np2-1]*x*dx,/double,/cumulative)
          endif else $
            array2[np+2:np2+1]=total(array1[np:np2-1]*dx,/double,/cumulative)
        endelse
      endelse
;
      if n_elements (save) eq 0 then save = wdf1
      if save lt 1 or save gt maxwdfarray then save = wdf1
      put_wdfarray, array2, save, /no_copy, xydata= xydata
      wdf2 = save
      wdffile(save-1) = wdffile (wdf1-1)
      if n_elements(clab) ne 0 then wdfcomment(save-1) = clab else $
        wdfcomment(save-1) = wdfcomment(wdf1-1)
      if n_elements(xlab) ne 0 then wdfxlabel(save-1) = xlab else $
        wdfxlabel(save-1)= wdfxlabel(wdf1-1)
      if n_elements(ylab) ne 0 then wdfylabel(save-1) = ylab else $
        wdfylabel(save-1)= wdfylabel(wdf1-1)
      array1=0
      sp=0
      im=0
      end
