; ------------------------------------------------------------------------ 
; $Id: iprop.pro,v 1.3 2007/06/06 20:03:49 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
 pro iprop, vdiode, idiode, vth, ith, vtl, itl, itotal, $
     mass = mass, distance = distance, smooth = smoo
;+
;   IPROP
;
;   Procedure propagates current to a target and separates the target 
;     current into a high energy and low energy branch.  The total current
;     at the target is also calculated.
;   Warning:
;          The WDF arrays: MAXWDFARRAY and MAXWDFARRAY-1, are used as 
;          scratch areas to calculate the total current. (81 and 82 for current
;          version (10/2/95) 
;          The Arrival Time vs Time and Actual scale vs Time will be stored
;            in these two arrays before leaving.
;
;  1. First I and V are put on a common uniform timebase
;  2. Voltage is smoothed and limited to 8% of peak
;  3. Vdiode is translated to the target.
;      k = 2.0 * 1.602177-19/ 1.66054e-27
;      velocity of particle = sqrt(k/mass*vdiode)
;      for a proton mass = 1.0073
;            carbon mass = 12.0000 
;  4. Arrival time derivatives are calculated and limited to two branches.
;  5. Absolute value of time derivatives is calculated and smoothed with
;     a three point boxcar.
;  6. Current is translated to the target.
;  7. Total current is calculated.
;
;
;   format: iprop, vdiode [, idiode] [, vth] [, ith] [, vtl] [, itl] [, itotal]
;          [, mass = mass] [, distance = distance] [, smooth = smooth]
;
;   where:
;      vdiode   - WDF array with diode voltage (volts vs nanoseconds)
;      idiode   - WDF array with diode current (amps vs nanoseconds)
;               - Default = vdiode + 1
;      vth      - WDF array with high energy target voltage
;                 Default = idiode + 1
;      ith      - WDF array with high energy target current
;                 Default is vth+1
;      vtl      - WDF array with low energy target voltage
;                 Default is ith+1
;      itl      - WDF array with low energy target current
;                 Default is vtl+1
;      itotal   - WDF array with total target current
;                 Default is itl+1
;      mass     - Atomic Mass in AMU, Default = 1.007276 = proton
;      distance - Distance to target, Default = 0.3 meters
;      smooth   - Number of points used to smooth the voltage, Default = 5
;                 Set to 0 to get no smoothing
;-
;
@com.dat
;
on_error, 1
;
     k = 2.0 * 1.602177e-19/ 1.66054e-27
     threshold = 0.08
     nparams = n_params()
     if nparams eq 0 then begin
      print, '% Format: IPROP, vdiode [, idiode] [, vth] [, ith] [, vtl] [, itl]'
      print, '          [, itotal] [, mass = mass] [, distance = distance] '
      print, '           [, smooth = smooth]'
     endif
     if nparams lt 1 then begin
       print, ' IPROP must be called with at least 1 parameters.'
       return
     endif
     if n_elements(vdiode) eq 0 then begin
       print, 'WDF array with the diode voltage must be defined'
       return
     endif
     if vdiode lt 1 or vdiode gt maxwdfarray then begin
        print, 'VDIOODE array must be greater than 0 and </=', maxwdfarray
        return
     endif
     if n_elements(idiode) eq 0 then idiode = vdiode +1
     if idiode gt maxwdfarray then begin
        print, 'IDIOODE array must be less than', maxwdfarray
        return
     endif

     if wdfnp(vdiode-1) eq 0 then begin
       print, 'No valid data in VDIODE array', vdiode
       return
     endif
     if wdfnp(idiode-1) eq 0 then begin
       print, 'No valid data in IDIODE array', idiode
       return
     endif

     if n_elements (vth) eq 0 then vth = idiode +1
     if vth lt 1 or vth gt maxwdfarray then begin
        print, 'VTH array must be greater than 0 and </=', maxwdfarray
        return
     endif
     if n_elements (ith) eq 0 then ith = vth +1
     if ith lt 1 or ith gt maxwdfarray then begin
        print, 'ITH array must be greater than 0 and </=', maxwdfarray
        return
     endif
     if n_elements (vtl) eq 0 then vtl = ith +1
     if vtl lt 1 or vtl gt maxwdfarray then begin
        print, 'VTL array must be greater than 0 and </=', maxwdfarray
        return
     endif
     if n_elements (itl) eq 0 then itl = vtl +1
     if itl lt 1 or itl gt maxwdfarray then begin
        print, 'ITL array must be greater than 0 and </=', maxwdfarray
        return
     endif
     if n_elements (itotal) eq 0 then itotal = itl +1
     if itotal lt 1 or itotal gt maxwdfarray then begin
        print, 'ITOTAL array must be greater than 0 and </=', maxwdfarray
        return
     endif

     get_wdfarray, array1, vdiode, /xydata
     get_wdfarray, array2, idiode, /xydata
     nela = wdfnp(vdiode-1)
     nelb = wdfnp(idiode-1)
     max1 = wdfmaxx(vdiode-1)
     max2 = wdfmaxx(idiode-1)
     get_common_time, array1, array2, max1, max2, nela, nelb
     if n_elements(array1) lt 4 then begin
       print, 'No values will be generated due to lack of overlap in V/I'
       return
     endif

     if n_elements(smoo) eq 0 then smoo = 5 else smoo = abs(smoo)
     if n_elements(mass) eq 0 then mass = 1.007276 else mass = abs(mass)
     if n_elements(distance) eq 0 then distance=0.3 else distance=abs(distance)

     t0 = array1(0)
     dt = array1(1)
     if smoo gt 0 then vd = smooth(temporary(array1(2:*)), smoo)
     id = temporary(array2(2:*))
     np = n_elements(vd)
     vmax = max(vd, ivmax)
     if ivmax ge np-2 then begin
       print, 'Voltage pulse is incomplete - No values will be generated'
       return
     endif
     vthresh = threshold*vmax
     if ivmax gt 0 then begin
       ok = where (vd(0:ivmax-1) lt vthresh, nok)
       if nok gt 0 then oks = ok(nok-1) else oks = 0
     endif else oks = 0
     ok = where (vd(ivmax+1:*) lt vthresh, nok)
     if nok gt 0 then okend = ok(0)+ivmax+1 else okend = np-1
     np = okend-oks+1
     newpeak = ivmax -oks
     vd = temporary(vd(oks:okend))
     id = temporary(id(oks:okend))
     t0 = temporary(t0) + oks*dt
  plot, t0+findgen(np), vd , title = 'Voltage' & hak, /mes
;    get the answer in nanoseconds
     dist = distance*1e9
     ttime = findgen(np)*dt

     time = sqrt(dist*dist*mass/k/vd)+ttime
     dtime = time(1:np-1)-time(0:np-2)
     dtima = abs(dtime)
     dtimave = [dtima(0), (dtima(0:np-3)+dtima(1:np-2))*0.5, dtima(np-2)]
     itarget = dt*id /dtimave
     ttarget = time +t0
     time = t0 + ttime
     xtitle = 'Time (ns)'
     vtitle = 'Voltage (V)'
     ititle = 'Current (A)'
;  start at peak voltage and look for a reversal
     okfinal = where(dtime(newpeak:*) le 0, nok)
     if nok eq 0 then okfin = np-1 else okfin = okfinal(0)+ newpeak

     oks = where (dtime(0:newpeak) le 0 , nok)
     if nok eq 0 then begin
;      We only have an increasing arrival time
       delwdf, vtl
       delwdf, itl
       print, ' Voltage produces only a monoenergetic beam at the target'
       istart = 0
       nplower = 0
     endif else begin
       istart = oks(nok-1)
       oks = where (dtime(0:istart-1) ge 0 , nok)
       if nok gt 0 then ibeg = oks(nok-1)+1 else ibeg = 0
       nplower = istart-ibeg
       if nplower gt 1 then begin
         itarget(istart+1) =  itarget(istart+1)*0.5
         i2w, ttarget(ibeg:istart+1), itarget(ibeg:istart+1), itl, /xydata, $
            xlab = xtitle, ylab = ititle, clab = 'Low Energy Current'
         i2w, ttarget(ibeg:istart+1), vd(ibeg:istart+1), vtl, /xydata, $
            xlab = xtitle, ylab = vtitle, clab = 'Low Energy Voltage'

       endif else begin
         delwdf, vtl
         delwdf, itl
         nplower = 0
         print, ' Less than 2 points in the lower branch.'
       endelse
       totalcharge = total(id(ibeg:okfin))*dt*1e-9
       print, 'Total charge on target is ', strtrim(totalcharge, 2), ' coulombs'
     endelse
     i2w, ttarget(istart+1:okfin), vd(istart+1:okfin), vth,  /xydata, $
            xlab = xtitle, ylab = vtitle, clab = 'High Energy Voltage'
     i2w, ttarget(istart+1:okfin), itarget(istart+1:okfin), ith,  /xydata, $
            xlab = xtitle, ylab = ititle, clab = 'High Energy Current'
     if nplower eq 0 then xfr , ith , itotal else begin
       xfr, ith, maxwdfarray
       xfr, itl, maxwdfarray-1
       if wdfmaxx(ith-1) gt wdfmaxx(itl-1) then $
           newend, maxwdfarray-1, wdfmaxx(ith-1) $
         else newend, maxwdfarray, wdfmaxx(itl-1)
       add, maxwdfarray, maxwdfarray-1, itotal, clab = 'Total Target Current', $
         ylab = ititle
     endelse
     i2w, time(ibeg:okfin), ttarget(ibeg:okfin), maxwdfarray-1, $
         xlab = xtitle, clab = 'Target Arrival Time', ylab = 'Target Time'
     i2w, time(ibeg:okfin), dtimave(ibeg:okfin), maxwdfarray, $
         xlab = xtitle, clab = 'Average Arrival Time Derivative', $
         ylab = 'Average Time Derivative'
     return
     end
