; ------------------------------------------------------------------------ 
; $Id: legendp.pro,v 1.3 2011/07/20 14:08:23 lpmix Exp $
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
 Pro legendp, stringin, linestyle= linestyle, psymbol = psymbol, color= color, $
          position = positin, device = device, data = data, normal = normal, $
          relative = relative, thick = thick, mspace = mspace, $
          fill=symfill, symsize=symsize, charsize = charsize, $
          offset = ioffset, nextpos = nextpos, box = box, cbox = cbox, $
          right = right, left = left, lcskip = lcskip, space = space, $
          bfill = bfill, margin = margin, bthick= bthickin, $
          lenline = lenline, cxyouts = cxyouts_in
;
;+
;   Produce a legend for a plot in PFIDL
;   This routine was renamed from LEGEND in 2010 when IDL 8.0 was released
;   format: legendp, string, linestyle = linestyle, psymbol = psymbol, 
;           color = color, position = position, /device, /data, /normal,
;           /relative, mspace = mspace, thick = thick, nextpos = nextpos,
;           offset = offset, box = box, cbox = cbox, bfill = bfill,
;           bthick = bthick, cxyouts = cxyouts
;   where
;         STRING    - String array with the legend text for each line in the 
;                     legend.
;         LINESTYLE - Array of integers indicating the linestyle for each line
;                     Linestyle may be omitted for text-only legends.
;                     Default is 0, solid line. Other values are the same as IDL
;                     -1, No LINE; 0, Solid; 1, Dotted; 2, Dashed; 3, Dash-Dot;
;                     4, Dash-Dot-Dot-Dot; 5, Long Dashes
;                     Linestyle should have one value for all curves or one
;                       value for each value of string. If there are fewer 
;                       values, then they will be recycled.
;         PSYMBOL   - Array of integers indicating the symbol for each line
;                     Default is 0, no symbol.  If there are fewer values
;                     of psymbol than curves then they will be recycled.
;                     Values are the same as IDL (abs(pysmbol) is used).
;                     1, +; 2, *; 3, period(.); 4, Diamond; 5, Triangle;
;                     6, Square; 7, X; 8, UserDefined.
;         COLOR     - Color for symbols and lines for each line of the legend
;                     Default is !p.color (white for X and Black for PostScript)
;                     If there are fewer values of color than curves then they
;                     will be recycled.
;                     Values are: 1, red; 2, green; 3, blue; 4, magenta;
;                     5, orange; 6, cyan; 7, yellow; 8, white; 0, black
;         CXYOUTS   - Color of the legend printing
;                     If CXYOUTS is an array, it will be cycled.
;                     If LEFT or RIGHT is set, then !p.color will be used for
;                     axis information if CXYOUTS is an array.
;         POSITION  - Position on the plot in units determined by keywords.
;                     Default is data coordinates if position is specified.
;                     If POSITION is undefined then default is at a relative 
;                        position of [0.04, 0.94] of the plotting area.
;         /device, /data, /normal, /relative  - only one may be specified.
;                     Determines the coordinates for the position keywork
;                     Data refers to the plot coordinates (Default)
;                     Relative is relative the actual grid 
;                     Device is in device coordinates
;                     Normal is normalized space coordinates
;         Mspace    - Multiplier for the spacing between lines
;         Thick     - parameter sent to oplot
;         Charsize  - parameter sent to xyouts; if not specified, set to 
;                     1/max(!p.multi(1:2)-1)>1
;         FILL      - Indicates filled symbols - only for symbols 11 and higher
;                     (see PLO, MKSYMBOL)
;         SYMSIZE   - Symbol size multiplier
;         NEXTPOS   - Next position for the legend in data coordinates
;                     Use this keyword for successive calls to legend
;         OFFSET    - Use to offset the count on the symbols, colors, linestyles
;                     Default = 0
;         BOX       - Draw a box around the legend if set.
;                     If box is undefined and 
;                       Cbox is set or Bfill is set or bthick is set 
;                         then BOX = 1
;                   - If Box has 4 elements, they will be taken as the position
;                     of the box.  Format is:
;                     BOX = [xmin, ymin, xmax, ymax
;                     Units are the same as POSITION
;                     Otherwise the size will be estimated.
;                     The color will be specified by CBOX
;         CBOX      - Color of the BOX. Default is !P.COLOR
;         BFILL     - Fill the box with the specified color.
;                     Note to erase an area: BFILL = !p.background
;         BTHICK    - Thickenss of the lines for the box.
;         MARGIN    - Margin for box.  Default = [2, 1] in units of characters
;                     (ie, !D.X_CH_SIZE and !D.Y_CH_SIZE) 
;         LEFT      - If set and greater than zero, print, "LEFT AXIS"
;                     If LEFT is a string, use this string in the legend.
;         RIGHT     - If set and greater than zero,  print, "RIGHT AXIS"
;                     If LCSKIP is set, skip a line before writing.
;                     If RIGHT is a string, use this string in the legend.
;         LCSKIP    - If LCSKIP is set, skip a line before printing RIGHT AXIS string.
;                     LCSKIP can be any real number (0.5, 1, 1.5, 2, etc)
;         LENLINE   - Length of the line in the legend in units of character width
;                     Default: LENLINE = 5 
;         SPACE     - Length of the space between the line and the characters in
;                     units of character width
;                     Default: SPACE = 2.5
;         Example:
;           Label curves with numbers in the upper left hand corner of the grid
;           legendp, sindgen(10), pos= [0.04, 0.94], /relative, lin = 1, $ 
;                    color = [2, 3, 4, 5, 6, 7, 1]
;
;           Put a box around the words, 'Experimental Data', with a thickness
;           of 2 and color yellow at [2, .4] in data units
;                legendp, 'Experimental Data', pos=[2, .4], cbox =7, bthi=2
;
;           Same as above put put a background color of 4 and use a green print
;                legendp, 'Experimental Data', pos=[2, .4], cbox =7, bthi=2, $
;                  bfill = 4,cxy = 2
;-
  on_error, 1
;
@symbol_list.pro
      nel = n_elements(stringin)
      if nel eq 0 then begin
        print,'% Format: LEGEND, STRING_ARRAY, [, LINESTYLE = linestyle]'
        print,'    [, PSYMBOL = psymbol] [, COLOR = color] [, MSPACE = mspace]'
        print,'    [, /DEVICE] [, /DATA] [, /RELATIVE] [, /NORMAL]'
        print,'    [, POSITION = position] [, CHARSIZE= charsize] [, LCSKIP = lcskip]'
        print,'    [, THICK = thick] [, NEXTPOS = nextpos] [,OFFSET = offset]'
        print,'    [, BOX = box] [, CBOX = cbox] [, BFILL = bfill]'
        print,'    [, MARGIN= margin] [, CXYOUTS = cxyouts]'
        return
      endif
;
      space = (n_elements(space) eq 0) ? 2.5 : float(space[0])
      lenline = (n_elements(lenline) eq 0) ? 5.0 : float(lenline[0])

      s = size(stringin)
      if s[s[0]+1] ne 7 then $
        message, 'Parameter to LEGEND must be a sting array'
      xcrange = !x.crange
      ycrange = !y.crange
      if !x.type eq 1 then xcrange = 10^xcrange
      if !y.type eq 1 then ycrange = 10^ycrange
      minran = convert_coord(xcrange(0), ycrange(0), /data, /to_device)
      maxran = convert_coord(xcrange(1), ycrange(1), /data, /to_device)
      xrang = maxran(0)- minran(0)
      yrang = maxran(1)- minran(1)
      if n_elements (charsize) eq 0 then charsize = 1.0
      crowd = max ((!p.multi)(1:2))
      if crowd gt 2 then chars = 0.6 *charsize  else chars = charsize
      chy = !d.y_ch_size*chars
      chx = !d.x_ch_size*chars
      offset = 0.4*chy
      deltay = 1.5*chy
      if n_elements(box) eq 0 and (keyword_set(cbox) or n_elements(bfill) $
        or keyword_set(bthickin)) then box = 1 
      if keyword_set (box) then begin
        nelbox= n_elements(box)
        if not keyword_set(device) and not keyword_set(relative) and not $
          Keyword_set(normal) then data = 1
        if nelbox eq 4 then begin
          bpos = box
          if not keyword_set(device) then begin
            bpos[0:1] = (convert_coord(bpos(0), bpos(1), data = data, $
                 device = device, relative = relative, /to_device))[0:1]
            bpos[2:3] = (convert_coord(bpos(2), bpos(3), data = data, $
                 device = device, relative = relative, /to_device))[0:1]
          endif
          bposition= [[bpos[0], bpos[2], bpos[2], bpos[0], bpos[0]], $
                bpos[1], bpos[1], bpos[3], bpos[3], bpos[1]]
          bposition = reform(bposition, 5,2, /overwrite)
        endif 
      endif else box = 0

      if n_elements(mspace) ne 0 then begin
        deltay = deltay*abs(mspace)
        offset = offset*abs(mspace)
      endif

      cxyouts = (n_elements(cxyouts_in) eq 0) ? !p.color : cxyouts_in
      cxyhead =(n_elements(cxyouts) eq 1) ? cxyouts[0] : !p.color
      if n_elements(cxyouts) lt nel then $
        while (N_elements(cxyouts) lt nel) do cxyouts=[cxyouts, cxyouts]
      if n_elements (symfill) eq 0 then begin
         symfill = 0
         nsymfill = 1
      endif else nsymfill = n_elements (symfill)
      if n_elements (symsize) eq 0 then begin
         symsize  = 1
         nsymsize = 1
      endif else nsymsize = n_elements (symsize)
      if n_elements (positin) ge 2 then begin
        position = positin
        if keyword_set (device) then begin
          position = position 
        endif else if keyword_set(normal) then begin
          position = CONVERT_COORD(position(0),position(1), /normal, /to_device)
        endif else if keyword_set(relative) then begin
          position = position(0:1) *[xrang, yrang] +minran
        endif else $
          position = CONVERT_COORD(position(0), position(1), /data, /to_device)
      endif else position = [0.04, 0.94]*[xrang, yrang] + minran
;
      ncolor = n_elements(color)
      if ncolor eq 0 then color = !p.color else loadxyct, /quiet, /freeze
      ncolor = ncolor > 1
      nsymbol = n_elements(psymbol)
      if nsymbol gt 0 then begin
        a = size(psymbol)
        if a[a[0]+1] eq 7 then begin
          nel = a[a[0]+2]
          temp = make_array(nel, /int)
          for i = 0, nel-1 do begin
            s = strupcase(strcompress(psymbol(i), /remove_all))
            l = strlen(s)
            if l eq 1 then begin
              if s eq 'X' then s = 'BIGX' else $
              if s eq '+' then s = 'PLUS' else $
              if s eq 'O' then s = 'CIRCLE'
              l = strlen(s)
            endif
            if l eq 0 then temp(i) = -11 else begin
              templist = strmid(symbol_list, 0,l)
              ok = where (s eq templist, nok )
              if nok eq 0 then begin
                print, 'A valid symbol string must be specified - ', s
                return
              endif else if nok eq 1 then temp(i) = ok else begin
                print, ' Multiple symbols specified, using ' , s
                print, symbol_list[ok]
                temp(i) = ok(0)
                print, '   Using a symbol of ', symbol_list[ok[0]]
              endelse
            endelse
          endfor
          psymbol = temp+11
        endif
      endif
      if nsymbol eq 0 then psymbol = 0 else psymbol = abs (psymbol)
      nsymbol = nsymbol > 1
      nlinest = n_elements(linestyle)
      if nlinest eq 0 then linestyle = -1
      nlinest = nlinest > 1
      pthick = (!p.thick eq 0.0) ? 1:!p.thick
      if n_elements(thick) eq 0 then thick=!p.thick else $
        thick=abs(thick)*pthick
;
      if n_elements(ioffset) eq 0 then ioffset = 0
      x = position[0]
      y = position[1]
      if keyword_set (box) then begin
        if n_elements(margin) lt 2 then margin = [2,1]
        if n_elements(cbox) eq 0 and (n_elements(bfill) eq 0 or $
           keyword_set(bthickin)) then cbox= !p.color
      endif
      if n_elements(lcskip)eq 0 then lcskip = 0
      if keyword_set(left) or keyword_set(right) then begin
        lcheader = 1
        if keyword_set(left) then begin
          ss = size(left, /type)
          if ss eq 7 then sstr = strtrim(left, 2) else begin
            if left gt 0 then sstr = 'Left Axis' else lcheader = 0
          endelse
        endif else begin
          ss = size(right, /type)
          if (ss eq 7) then sstr = strtrim(right, 2) else begin
            if right gt 0 then sstr = 'Right Axis' else lcheader = 0
          endelse
        endelse
        lctit = lcheader + lcskip
      endif else lctit =0
;     only skip if we have a header
      if keyword_set(lctit) then y = y-lcskip*deltay
      if max (linestyle) lt 0 then begin
        if max(psymbol) gt 0 then xc = x+lenline*0.5*chx else xc = x
        if keyword_set (box) and n_elements(bfill) gt 0 then begin
          if n_elements(box) eq 4 then $
            polyfill, bposition[*, 0], bposition[*, 1], color = bfill $
          else begin
            x3 = convert_coord(xc, y-offset, /device, /to_data)
            wid=0
            for i = ioffset, ioffset+nel-1 do begin
               xyouts, x3(0), x3(1), stringin(i-ioffset), charsize=chars, $
               width = w, color = cxyouts[i- ioffset]
               wid = w > wid
            endfor
            wid = !d.x_size*wid
            x0= x-margin[0]*chx
            x1= xc+wid+margin[0]*chx
            y1= y-offset+(margin[1]+0.875)*chy
            y0= y-offset-(lctit+nel-1)*deltay-margin[1]*chy
            polyfill, [x0, x1, x1, x0, x0], [y0, y0, y1, y1, y0], /dev, $
              color = bfill
          endelse
        endif
        if keyword_set(lcheader) then begin
          x3 = convert_coord(xc, y-offset, /device, /to_data)
          xyouts, x3(0), x3(1), sstr, charsize=chars*1.1,  color = cxyhead, align = 0.5
          y = y -deltay
        endif
        yy = y
        wid = 0
        for i = ioffset, ioffset+ nel-1 do begin
          x1 = convert_coord(x, y, /device, /to_data)
          x3 = convert_coord(xc, y-offset, /device, /to_data)
          psym = psymbol(i mod nsymbol)
          if psym gt 10 then begin
            psym = psym - 11
            mksymbol, psym, fill = symfill(i mod nsymfill)
            psym = 8
          endif
          col = color (i mod ncolor)
          if psym gt 0 then $
          plots, x1(0), x1(1), symsize=1.1*chars, psym=psym, color = col, $
           thick = thick
          xyouts, x3(0), x3(1), stringin(i-ioffset), charsize=chars, $
            width = w , color = cxyouts[i- ioffset]
          wid = w > wid
          y = y -deltay
        endfor
        if keyword_set(box) and n_elements(cbox) gt 0 then begin
          if n_elements(bthickin) eq 0 then bthick = 1 else bthick = bthickin
          bthick = bthick*pthick
          x0= x-margin[0]*chx
          x1= xc+wid*!d.X_SIZE+margin[0]*chx
          y1= yy-offset+(margin[1]+.875)*chy
          y0= yy-offset-(lctit+nel-1)*deltay-margin[1]*chy
          plots, [x0, x1, x1, x0, x0], [y0, y0, y1, y1, y0], /dev, $
            color = cbox, thick= bthick
        endif
      endif else begin
        xf = x + lenline*chx
        xc = xf+space*chx
        if keyword_set(lcheader) then begin
          x3 = convert_coord(xc, y-offset, /device, /to_data)
          xyouts, x3(0), x3(1), sstr, charsize=chars*1.1,  color = cxyhead, align = 0.5
          y = y -deltay
        endif

        if keyword_set (box) and n_elements(bfill) gt 0 then begin
          if n_elements(box) eq 4 then $
            polyfill, bposition[*, 0], bposition[*, 1], color = bfill, /dev $
          else begin
            x3 = convert_coord(xc, y-offset, /device, /to_data)
            wid=0
            
            for i = ioffset, ioffset+nel-1 do begin
               xyouts, x3(0), x3(1), stringin(i-ioffset), charsize=chars, $]
               width = w, color = cxyouts[i- ioffset]
               wid = w > wid
            endfor
            wid_save = wid
            wid = !d.x_size*wid

            x0= x-margin[0]*chx
            x1= xc+wid+margin[0]*chx
            y1= y-offset+(margin[1]+0.875)*chy
            y0= y-offset-(lctit+nel-1)*deltay-margin[1]*chy
            polyfill, [x0, x1, x1, x0, x0], [y0, y0, y1, y1, y0], /dev, $
              color = bfill
          endelse
        endif
        yy = y
        wid = 0

        for i = ioffset, ioffset+nel-1 do begin
          x1 = convert_coord(x, y, /device, /to_data)
          x2 = convert_coord(xf, y, /device, /to_data)
          x3 = convert_coord(xc, y-offset, /device, /to_data)
          ll = linestyle (i mod nlinest)
          psym = psymbol(i mod nsymbol)
          if psym gt 10 then begin
            psym = psym - 11
            mksymbol, psym, fill = symfill(i mod nsymfill)
            psym = 8
          endif
          col = color (i mod ncolor)
          ss = symsize (i mod nsymsize)
          if ss eq 0. then ss = 1
          if ll ge 0 then $
          oplot,[x1(0), x2(0)], [x1(1), x2(1)], line=ll, psym=-psym, $
            color=col, thick=thick, /noclip , symsize = ss* chars $
          else if psym gt 0 then $
            plots, (x1(0)+x2(0))/2.0, x1(1),symsize=1.5*ss*chars,psym=psym, $
              color = col, thick = thick
          xyouts,x3(0),x3(1),stringin(i-ioffset), charsize=chars, $
            wid= w, color = cxyouts[i- ioffset]
          wid = w > wid
          y = y-deltay
        endfor
        if keyword_set(box) and n_elements(cbox) gt 0 then begin
          if n_elements(bthickin) eq 0 then bthick = 1 else bthick = bthickin
          bthick = bthick*pthick
          x0= x-margin[0]*chx
          x1= xc+wid*!d.X_SIZE+margin[0]*chx
          y1= yy-offset+(margin[1]+0.875)*chy
          y0= yy-offset-(lctit+nel-1)*deltay-margin[1]*chy
          plots, [x0, x1, x1, x0, x0], [y0, y0, y1, y1, y0], /dev, $
            thick = bthick, color = cbox
        endif
      endelse

      nextpos =  convert_coord(x, y, /device, /to_data)
      return
      end
