; ------------------------------------------------------------------------ 
; $Id: mid.pro,v 1.3 2007/06/06 20:03:49 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
 Function  mid, wdf, left = left, right = right, ierr = ierr,area=area, $
                  offset = offset, inquire = inquire, plotit = plotit, $
                  cylindrical = cylindrical
;+
;     NAME: MID
;     
;     This function calculates the center of a wdf array according to the
;         equation:
;          center = total(x*dx*y) / total(dx*y)
;      If keyword cylindrical is set then mid returns:
;          center = total(x*x*dx*y) / total(x*dx*y)
;     Procedure:
;         Get x, y and calculate dx. 
;           DX calculated as average dx on either side if not uniform.
;           End values of DX are the end values. 
;         Get offset
;         Calculate MID = TOTAL(x*dx*(y-offset))/TOTAL(dx*(y-offset))
;         TOTAL is the idl function.
;     Format:
;        center = MID(wdf [, LEFT = left] [, RIGHT=right] [, /PLOTIT] [, /QUIET]
;           [, /INQUIRE] [, OFFSET = offset] [, AREA = area] [, IERR = ierr])
;           [, /cylindrical]
;    where
;     CENTER   -  Center point defined above
;                 If abs(area) is less than 10e-2*xrange*yrange/2)
;                   a warning will be issued. (where xrange = right-left
;                   and yrange = max(y)-min(y))
;                 Perhaps less baseline or other modification will provide
;                   a better answer.
;     WDF      -  WDF array number or name of a wdf array
;     LEFT     -  Minimum X value to consider
;                 If LEFT is outside the data range, it is set to the minimum.
;     RIGHT    -  Maximum X value to consider
;                 If RIGHT is outside the data range, it is set to the maximum.
;     INQUIRE  -  If set, waveform will be plotted and user asked to indicate
;                 the baseline.
;     OFFSET   -  If inquire is set, the baseline will be returned in offset.
;                 If inquire is not set, then the value in offset will be 
;                   subtracted from the ordinate values. Default = 0.0
;     AREA     -  Value of Total(dx*y). For some shapes this is an area;
;                   for others, an integral.
;     PLOT     -  If set the waveform, offset, and median will be plotted.
;     QUIET    -  If set, will not print the area warning.
;     IERR     -  Set to 0 if data is valid, -1 if not valid
;                 Set to 1 if area fails the above criteria.
;     Example:
; 
;       Find the center of the following:
;       x = findgen(100) & y = x*exp(-x/20)  & i2w, x, y, 1
;        center = mid(1, /pl)
;
;      Find the center of mass for waveform 4, using the average of the 
;        data from the start to 50 ns as the offset. Calculate the mid point
;        using data between 50 and 150 ns.
;        cm = mid(4, left=50e-9, ri= 150e-9, off= ave(4, r= 50e-9, /qui))
;      
;      Find the median of waveform 'lineout 3' and plot the result.
;      Set an offset using a widget.
;        median = mid('lineout 3', /plot, /inquire)
;
;      MODIFICATION HISTORY:
;       Written by:     Paul Mix, April 1, 1998
;-
@com.dat
;
      on_error, 1
      ierr = -1
      if n_elements(initwdf) eq 0 then wdfinit
      nparams = n_params()
      if nparams lt 1 then begin
print,'% Format: CENTE =MID(WDF, LEFT=left, RIGHT=right, /INQUIRE, VALID=valid)
        return, 0.0
      endif
;
      if n_elements(wdf) eq 0 then begin
        print, '% MID: WDF array to FWHM must be defined'
        return, 0.0
      endif
      a = size(wdf)
      if a(a(0)+1) eq 7 then begin
        if a(0) ne 0 then print, '% PLO: Only the first string is used'
        if strtrim(wdf(0), 2) eq '*' then wdf = wdvalid() else $
          wdf = wdvalid(wdf(0))
        if wdf(0) lt 1 then begin
          print, '% PLO: No arrays found using the search string: ', wdf(0)
          return, 0.0
        endif
      endif
      ok = where(wdf ge 1 and wdf le maxwdfarray, nok)
      if nok eq 0 then begin
        print, '% MID: WDF array must be greater than 0 and </=', maxwdfarray
        print, '     Value =', wdf
        return, 0.0
      endif
      if nok gt 1 then begin
        print, '% MID: Single WDF array must be provided.'
        print, '     Values =', wdf
        return, 0.0
      endif
;     wdf = wdf(0)
      nel = wdfnp(wdf-1)
      if nel eq 0 then begin
        print, '% MID: No valid data in the wdf array ', strtrim (wdf, 2)
        return, 0.0
      endif
      start = wdfminx(wdf-1)
      stopp = wdfmaxx(wdf-1)
      if n_elements(left) gt 0 then left = left(0) else left=start
      if (left lt start) or (left ge stopp) then left=start
      if n_elements(right) gt 0 then right = right(0) else right=stopp
      if (right le left) or (right gt stopp) then right=stopp
      if n_elements(quiet) eq 0 then quiet = 0
      if keyword_set(inquire) then blin, wdf, /no_save, offset=offset else $
        if n_elements(offset)eq 0 then offset = 0.0
      get_wdfarray, array, wdf, /xydata
;     need to get the number of elements in case this is x-y data
      if  n_elements(array) eq nel+2 then begin
; uniform data
        x0 = array(0)
        dx = array(1)
        x = findgen(nel)*dx+x0
        y = array(2:nel+1)
        lcuniform = 1
      endif else begin
; non-uniform data
        y = array(nel: nel+nel-1)
        x = array(0:nel-1)
        dx = make_array(nel)
        dx(1:nel-1) = x(1:nel-1)-x(0:nel-2)
        dx(0) = dx(1)
        dx(1:nel-2) = (dx(1:nel-2) + dx(2:nel-1))*0.5
        lcuniform = 0
      endelse
      if offset ne 0.0 then y = temporary(y) - offset
      if right lt stopp or left gt start then begin
        ok = where ((x ge left and x le right), nok)
        if nok eq 0 then begin
          Print, '% MID: No data in the interval:', left, right
          return, 0.0
        endif
        x = x(ok)
        y = y(ok)
        if lcuniform eq 0 then dx = dx(ok)
      endif
      y = y*dx
      if keyword_set(cylindrical) then y = y*x
      area = total(y)
      if area eq 0.0 then begin
        print, '% MID: Area is zero. Cannot calculate center of mass'
        return , 0.0
      endif
      if abs(area) lt 0.5e-2*(left-right)*(max(y, min= mm)-mm) then begin
        ierr = +1
         print,' Area =',area
        if not keyword_set(quiet) then Winformation, $
         ['% MID: Area is less than 0.5% of the possible area', $
         '         Try reducing the width of the window.']
      endif else ierr = 0
      center = total(x*y)/area
      if keyword_set(plotit) then begin
        plo, wdf, xr = [left, right], eng=0
        if offset ne 0 then $
          oplot, !x.crange, [offset, offset], lin=2, col = 2
        oplot, [center, center], !y.crange, col = 7, thick = 2
      endif
      return, center
      end
