; ------------------------------------------------------------------------
; $Id: minlin.pro,v 1.1 2009/09/02 16:19:39 lpmix Exp $
; ------------------------------------------------------------------------
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
;+
; NAME:
;       MinLin
;
; PURPOSE:
;       This procedure calculates the midgap and electrode r,z curves corresponding
;       to optimal (minimum inductance) axisymmetric transmission lines with end 
;       points(r1,0) (r2,lz) and radially dependent AK gaps w(r1)=w1 and w(r2)=w2
;       For certain cases when the extrapolated w(0) is "too negative" no solution exists
;       This calculations assumes a transmission line with cylindrical symmetry and 
;       coordinates (R, Z) for all theta.
;
; CATEGORY:
;       Pulsed Power
;
; CALLING SEQUENCE:
;       MINLIN, RADIUS_1, RADIUS_2, WIDTH_1, WIDTH_2, LENGTH
;
; INPUTS:
;       RADIUS_1:  Radius(cm) at Z = 0.0 
;       RADIUS_2:  Radius(cm) at Z = LENGTH 
;       NOTE:  If RADIUS_1 EQ RADIUS_2, then WIDTH_2 = WIDTH_1
;       WIDTH_1:   Anode/Cathode gap(cm) at Radius_1 and Z = 0.0
;       WIDTH_2:   Anode/Cathode gap(cm) at Radius_2 and Z = LENGTH
;       LENGTH:    Axial Length(cm) (in dimension Z) of the power feed.
;       Note:  If all 5 are not provided, a widget  provided.
;
; OPTIONAL INPUTS:
;       All inputs are optional.  
;	
; KEYWORD PARAMETERS:
;       INCHES:    If set, all dimensions in inches
;       
;       WDF:       First waveform array  (DEFAULT: WDF = 1)
;                   WDF      Zmid(cm) vs Radius_Midgap(cm)
;                   WDF+1    Zcath(cm) vs Radius(cm)
;                   WDF+2    Zanode(cm) vs Radius(cm)
;                   WDF+3    Lmin(nH) vs Radius(cm)
;                   WDF+4    Radius(cm) vs Zmid(cm) (Same as 1 but single valued)
;                   WDF+5    Radius of Curvature(cm) vs Radius(cm)
;                   WDF+6    Radial coordinate midgap curve center(cm) vs Radius_Midgap(cm)
;                   WDF+7    Z-coordinate midgap curve center(cm) vs Radius_Midgap(cm)
;                   WDF+8    Lmin(nH) vs Zmid(cm)
;                   WDF+9    Int(dz*(dr/dz)^2)
;                   WDF+10   Arclenght(z) vs Z
;
;       CONE:      First WDF array value for equivalent cone inductance.
;                  If CONE is missing or zero, these will not be generated
;                  Note:  Conical transmission line will be have ends perpendicular 
;                         to the midplane and not truncated at the axial extent.
;                  Default: CONE =0 
;                  CONE      zconecath
;                  CONE+1    zconeanod
;                  CONE+2    zconemid
;                  CONE+3    gapcone
;
;       ERROR:     Error indicator
;                  Returns 0 if no error and 1 if and error.
;                  My be used in parameter searches to MINLIN with:
;                    if error then begin
;                      (process error)
;                    endif
;                  A frequent problem is when w0 is less than zero.
;                  (See equation 5 of referenced paper under PROCEDURE)
;       TOLERANCE: Relative tolerance for the solution.
;                  (Xmax-Xmin)/(Xmax+Xmin) < TOLERANCE
;                  Default: TOLERANCE = 2.0E-11
;       DELTAVAR:  Initial fraction for the iteration.
;                  Default: DELTAVAR = 0.0005
;       ARCLENGTH: Arc Length of the midplane of the transmission line
;       INDUCTANCE:Inductance for the optimal minimum inductance configuration
;                  INDUCTANCE = CALCL(WDF+1, WDF+2)
;            Note: To within the approximations of this calculation, 
;                  GET_VALUE(wdf+8, LENGTH) provides the inductance.
;       SIMPLE:    Inductance for a conical or cylindrical feed.
;                  SIMPLE = CALCL( CONE, CONE+1)
;       VERBOSE:   If set, information will be sent to the screen.
;       TRUNCATE:  If set, do not label waveforms with geometry information.
;                  NOTE:  PLO, /TRUNCATE will plot a WDF arry without showing 
;                         the geometry information, but the information will be
;                         available to the user.
; 
; RESTRICTIONS:
;	Some values will not converge when the extrapolation of the gap width to 
;       Radius = 0 is negative.  
;
; PROCEDURE:
;	The procedure was developed by Eduardo Waissman for the equations in the 
;       paper: "Minimal Inductance for axisymmetric transmission lines with radially
;       dependent anode cathode gap"
;       The root to the equations is estimated by searching for a zero crossing using 
;       (1+ N*DELTAVAR) as a multiplier for the minimum  obtain the next point.
;       If the slope of the solution is not suggesting a root (zero crossing) is near
;       the factor n is increased by a factor of up to 100 to speed the solution.
;       After an initial location for the zero crossing is determined, the root is 
;       refined by factors of two until the desired tolerance is achieved. 
;
; EXAMPLE:
;	Calculate the minimum inductance configuration for a transmission line.
;         MINLIN
;         Note:  A widget will be provided to obtain radius, gap and length information.
;
;       Calculate the minimum inductance for a 1 meter height transmission line with 
;       a radius of 4 cm at center and 100 cm at the end and with gaps of 0.3 to 0.8 cm.
;       Plot the anode and cathode surfaces.
;         MINLIN, 4, 100, .3, .8, 100
;         PLO, 101, 2, /truncate, /over
;       Note: This examples assumes that set_maxwdfarray has been called to set the maximum to 
;             a larger to allow for WDF arrays 100 to 110.
;
; MODIFICATION HISTORY:
; 	Written by:	Eduardo M Waisman, July 7, 2009
;
;       July 8, 2009    Modified by L Paul Mix for inclusion into PFIDL
;                       Basic technique was not modified but a variable step size
;                       was included to speed up some calculations and a relative error
;                       implemented.  Equations modified to allow for vectorization.
;                       Slight style changes.
;
;-
  Function emw_square,x
  on_error, 1
  compile_opt idl2, hidden
  common emw_aleph,alpha,A,sqA,AsqA
  return, sqrt(((A*x+2.d0*alpha)*x+1.d0) > 1.0d-12)
  end

  Function emw_loga,x
  on_error, 1
  compile_opt idl2, hidden
  common emw_aleph,alpha,A,sqA,AsqA
  return,  -(alpha/AsqA)*alog((2.d0*(A*x+alpha+sqA*emw_square(x))) > 1.0d-10)
  end

  Function emw_invsin,x
  on_error, 1
  compile_opt idl2, hidden
  common emw_aleph,alpha,A,sqA,AsqA
  return , (asin((-A*x-alpha)< 1.0d0))*(alpha/AsqA)
  end


Function emw_delta,flag
  on_error, 1
  compile_opt idl2, hidden
  common emw_input,bettaold,betta,deltav,deltavold,deltavmax,rho0,k
  common emw_results, diff, index, solve, rgraph
  common emw_geom,r1,r2,lz,w1,w2,pi,rho1,w0, error, kequal
  common emw_aleph,alpha,A,sqA,AsqA
  common emw_limites,x1,x2,xmax

  if flag eq 0 then begin
    x1=r1/rho0
    A=-1.d0
    sqA=1.d0
    AsqA=1.d0
    v1=emw_square(x1)/A+emw_invsin(x1)
    x2= (solve eq 1) ? rgraph/rho0 : r2/rho0
    v2=(emw_square(x2)/A)+emw_invsin(x2)
    deltav=v2-v1
    deltavmax=-v1
    if solve eq 1 then return, deltav
    if deltavmax lt lz/rho0 then deltav=-v2-v1
    return, lz-rho0*deltav
  endif
  
  if flag eq 1 then begin
    if abs(alpha-1.d0) lt 1.e-6 then alpha=1.000001d0
    rho0=alpha*w0/k
    A=alpha*alpha-1.d0
    sqA=sqrt(abs(A))
    AsqA=abs(A*sqA)
    x1=r1/rho0
    x2= (solve eq 1) ? rgraph/rho0 : r2/rho0

    if alpha gt 1.d0 then begin
      v1= emw_square(x1)/A + emw_loga(x1)
      v2= emw_square(x2)/A + emw_loga(x2)
      deltav=v2-v1
    endif else begin
;up to jump1 is the 1>alpha>0 case
      v1= emw_square(x1)/A + emw_invsin(x1)
      invmax = (alpha/AsqA)*!dpi2
      xmax=1.d0/(1.d0-alpha)
      v2=emw_square(x2)/A + emw_invsin(x2)
      deltav=v2-v1
      if solve eq 1 then return, deltav
      deltavmax=invmax-v1
      if deltavmax lt lz/rho0 then deltav=2.d0*invmax-v2-v1
    endelse
    diff= (solve eq 1) ? deltav : lz-rho0*deltav
    return,diff
  endif

  if flag eq -1 then begin
    alpha=-betta
    A=alpha^2-1.d0
    sqA=sqrt(abs(A))
    AsqA=abs(A*sqA)
    rho0=alpha*w0/k
    x1=r1/rho0
    v1=(emw_square(x1)/A) + emw_loga(x1)
    x2= (solve eq 1) ? rgraph/rho0 : r2/rho0
    v2=(emw_square(x2)/A) + emw_loga(x2)
    deltav=v2-v1
    if (index eq -1) && (deltav lt lz/rho0) then begin
      flag=100
      if keyword_set(verbose) then help, deltav, lz, rho0
      print,'% DELTA:  No solution w0<0 case'
    endif
    diff= (solve eq 1) ? deltav : lz-rho0*deltav
    return,diff
  endif
  print, '% EMW_DELTA: Bad flag value'
  flag = 100
end


PRO MINLIN, R1in, R2in, W1in, w2in, Length, inches = inches, test = test, $
    tolerance = tolerance, WDF = wdf, CONE = cone, deltavar = deltavar, $
    inductance = inductance, simple = simple, verbose= verbose, $
    truncate = truncate, arclength = arclength, error = error

on_error, 1

compile_opt  idl2
  common emw_input,bettaold,betta,deltav,deltavold,deltavmax,rho0,k
  common emw_results, diff, index, solve, rgraph
  common emw_geom,r1,r2,lz,w1,w2,pi,rho1,w0
  common emw_aleph,alpha,A,sqA,AsqA
  common emw_limites,x1,x2,xmax


  defsysv, '!dpi2', exists = i
  
  if i eq 0 then defsysv,  '!dpi2', !dpi*0.5d0
  error = 1
  if keyword_set (test) then begin
    if n_elements (r1in) eq 0 then r1in = 1.5d0
    if n_elements (r2in) eq 0 then r2in = 1.5d0
    if n_elements (w1in) eq 0 then w1in = 0.4d0
    if n_elements (w2in) eq 0 then w2in = 0.4d0
    If n_elements (length) eq 0 then length = 8.0d0
  endif else if ((n_elements (r1in) eq 0) || (n_elements (r2in) eq 0) || $
    (n_elements (w1in) eq 0) || (n_elements (w2in) eq 0) || $
    (n_elements (length) eq 0)) then begin
    instr='Enter Values: R1 < R2 and W1 < W2' 
    labels =['Radius 1', 'Radius 2', 'Width 1', 'Width 2', 'Axial Length']
    get_minlin_param, lcquit, labels, r1in, r2in, w1in, w2in, length, instr = instr
    if lcquit then return 
  endif
  if keyword_set(inches) then begin
    r1 = 2.54d0 *r1in[0]
    w1 = 2.54d0 *w1in[0]
    r2 = 2.54d0 *r2in[0]
    w2 = 2.54d0 *w2in[0]
    lz = 2.54d0 *length[0]
  endif else begin
    r1 = double(r1in[0])
    w1 = double(w1in[0])
    r2 = double(r2in[0])
    w2 = double(w2in[0])
    lz = double(length[0])
  endelse
  if r1 gt r2 then begin
    print, '% MINLIN:  R2 must be greater than or equal R1'
    return
  endif
  if w1 gt w2 then begin
    print, '% MINLIN:  W2 must be greater than or equal W1'
    return
  endif
  delr = r2-r1
  delrr = delr/r2
  delw  =w2-w1
  k = (delrr gt 1.d-3) ? delw/delr : 0.0d0
  w0= (w1-k*r1) 
  if abs(w0) lt 1e-3 then w0 = 1.0d-3 ; minimum gap 10 microns
  vmax=0.d0
  f0=abs(w0)
  xmax=0.d0
  solve=0
  if k eq 0.0 then begin
    if w1 ne w2 then print, '% MINLIN: WIDTH_2 set equal to WIDTH_1'
    w2 = w1
  endif
; flag=-1 is the w0 negative cas, flag=1 is the alpha>0 case 
  flag= (w0 lt 0.0) ? -1 : 1
  
  rho0min=0.d0
  if k eq 0 then begin
    flag = 0
    alpha=0.d0
    betta=0.d0
    bettamin = 0.0d0
    xmax=1.d0
    z0=(lz/2.d0)+(r2^2-r1^2)/(2.d0*lz)
    rhomin=-1.d-4+sqrt(z0^2+r1^2)
    if rhomin lt r2 then rhomin=r2
    diffold=0.d0
  endif else if flag gt 0 then begin
    bettamin=r2*k/w2
    betta=bettamin   ;mininum value of betta for w0>0
    alpha=betta
  endif else begin
;   flag = -1
    bettamin=(w1-w0)/w1
    betta=bettamin
    rhomin=bettamin*f0/k
    alpha=-betta
  endelse
; loop over either betta or rho0 to find curve that has lz axial length
  nstep = 1000
  iterval = make_array(nstep+2)
  iterxval = iterval
  dvar = (n_elements(deltavar) gt 0) ? abs(deltavar[0]) : 0.0005d0
  tol = (n_elements(tolerance) gt 0) ? abs(tolerance[0]) : 2.0d-11
  dvar_init = dvar
  bettaold=bettamin
  rho0old=rho0min
  deltavold=0.d0
  eps = 1.0d0
  index = -1
  del = 1.0
  for j=0,nstep do begin
    betta=bettamin*eps
    alpha=betta*flag
    if flag eq 0 then rho0=rhomin*eps
    index=j-1
    diff=emw_delta(flag)
    if flag eq 100 then begin
      Print, '% MINLIN: No Solution for these parameters'
      return
    endif
    if index eq -1 then diffold=diff
    signo=diff*diffold
    if signo lt 0.d0 then break
    if index gt 0 then del =((abs(diff*del/(diff-diffold))/5.0) > 1) < 1000
    diffold=diff
    bettaold=betta
    deltavold=deltav
    rho0old=rho0
    iterxval[j] = eps
    iterval[j] = diff
    eps += dvar*del
    index +=index
  endfor
  if j ge nstep then begin   ;no convergence
    print,'% MINLIN: Could not find a root in nstep:',nstep
    return
  endif
  if  keyword_set(verbose) then begin
    print, 'Iterations for rough answer:', j
    if j gt 2 then $
      i2w, iterxval[0:j-1], iterval[0:j-1], /xydata, get_maxwdf()-2
  endif
  xleft=rho0old
  xright=rho0
  dleft=diffold
  dright=diff
  for j=1,100 do begin
    if abs((xleft-xright)/(xleft+xright)) lt tol then break
    rho0=0.5d0*(xright+xleft)
    alpha=rho0*k/w0
    betta=alpha*flag
    drho0=emw_delta(flag)
    signo=dleft*drho0
    if signo lt 0.d0 then begin
      xright=rho0
      dright=drho0
    endif else begin
      xleft=rho0
      dleft=drho0
    endelse
  endfor
  if keyword_set(verbose) then print, 'Iterations for phase 2:', j
  if j gt 100 then begin
    print,'% MINLIN: No root was found bisection'
    return
  endif
  error = 0
  rho0=0.5d0*(xleft+xright)
  alpha=rho0*k/f0
  if flag eq -1 then betta=rho0*k/f0
  diffinal=emw_delta(flag)
  solve=1
  alpha=betta*flag
  if keyword_set(verbose) then begin
    print,' r1, r2, w1, w2, lz: ',  r1, r2, w1, w2, lz 
    print,'alpha,rho,w0,k,bettamin,diffinal',alpha,rho0,w0,k,bettamin,diffinal
  endif
  cadd = keyword_set(truncate) ? '' : $
    ' -  r1,r2,w1,w2,lz:'+strtrim(r1,2)+','+strtrim(r2,2)+','+ $
    strtrim(w1,2)+','+ strtrim(w2,2)+','+ strtrim(Lz,2)
; end solution best values
; now prepare for graphs
  r1lim=r1
  r2max=rho0*xmax
  x2=r2/rho0
  r2lim= (betta lt 1.0d0) ? r2max : r2
  x1lim=r1lim/rho0
  x2lim=r2lim/rho0

  xc1=1.d0+alpha*x1lim
  xcout=1.d0+alpha*x2lim    ;sinfi=x/(1+alpha*x)
  xc2=1.d0+alpha*x2
  argfi1= (x1lim/xc1) < 1.0d0
  argfiout= (x2lim/xcout) < 1.0d0
  argfi2= (x2/xc2) < 1.0d0
  fi1=asin(argfi1)
  fiout=asin(argfiout)
  fi2=asin(argfi2)
  deltfi=0.001d0
  m0=0
  mextra=0
  
  if betta lt 1.d0 then begin
    m0=round((!Dpi2-fi1)/deltfi)
    mextra=1+round((!Dpi2-fi2)/deltfi)
    ngraph=1+round((fi2-fi1)/deltfi)
    rgraph=rho0*x2
    diff=emw_delta(flag)
    if lz/rho0 gt deltavmax then ngraph=m0+mextra+1
  endif else begin
    ngraph= (1+round((fi2-fi1)/deltfi)) > 5001L
    deltfi=(fi2-fi1)/(ngraph-1L)
  endelse
  ngraphp = ngraph +1
  fi= fi1 + dindgen(ngraphp)*deltfi
  sfi = sin(fi)
  cfi = cos(fi)
  xcm = 1.0/(1.0d0 -alpha*sfi)
  x = sfi*xcm
  rgraph = x*rho0
  xm =x
  vgraph=emw_delta(flag)
  zm= vgraph
  if betta lt 1.0d0 then begin
    iii = floor(m0) > 0
    if iii lt n_elements(zm) then zm[iii:*]=2.*deltavmax-vgraph[iii:*]
  endif
  radcurv=rho0*(1.d0+alpha*x)^2
  i2w, x, zm, 155
  rcenter=-rho0*alpha*x^2
  zcenter=rho0*zm+radcurv*cfi
  gap=(1.d0+alpha*x)*w0
  xca=xcm+gap/(2.d0*rho0)
  xcc=xcm-gap/(2.d0*rho0)
  if flag eq -1 then xca=xcm-gap/(2.d0*rho0)
  if flag eq -1 then xcc=xcm+gap/(2.d0*rho0)
  xa=xca*sfi
  xc=xcc*sfi
  za=zm-cfi*gap/(2.d0*rho0)
  zc=zm+cfi*gap/(2.d0*rho0)
  deltLnew=[0.0, 2.d0*deltfi*f0*((1.d0+alpha*x))^2/sfi]
  deltL=0.5*(deltLnew[0:ngraph-1] + deltlnew[1:ngraph])
  Lmin = [0, total(deltL, /cum, /double)]

;WDF DESCRIPTION: ALL LENGTHS IN CM, INDUCTANCES IN NH
;            1  zmidgap  vs rmidgap
;            2  zcathode vs rcathode 
;            3  zanode   vs ranode   
;            4  L        vs rmidgap  
;            5  rmidgap  vs zmidgap            same as 1 but it is singled valued
;            6  radius of curvature of midgap curve vs radius
;            7  radial coordinate of the midgap curve center at rmidgap 
;            8  axial  coordinate of the midgap curve center at rmidgap 
;            9  L        vs zmidgap  like 4 but as a function of zmidgap
;           10  arclength vs zmidgap

ww =  (n_elements(wdf) gt 0) ? wdf[0] : 1
ww = (ww > 1) < (get_maxwdf() -12)
xmrh = xm*rho0
zmrh = zm*rho0
put_wdfarray,[xmrh,zmrh],ww,/xydata,clab='zmid(cm)'+cadd,ylab='z(cm)',xlab='r(cm)'
put_wdfarray,[xc*rho0,rho0*zc],ww+1,/xydata,clab='zcath(cm)'+cadd,ylab='z(cm)',xlab='r(cm)'
put_wdfarray,[xa*rho0,rho0*za],ww+2,/xydata,clab='zanode(cm)'+cadd,ylab='z(cm)',xlab='r(cm)'
put_wdfarray,[xmrh,Lmin],ww+3,/xydata,clab='Lmin(nH)'+cadd,ylab='L(nH)',xlab='r(cm)'
put_wdfarray,[zmrh,xmrh],ww+4,/xydata,clab='rm(cm)'+cadd,ylab='rm(cm)',xlab='zm(cm)'
put_wdfarray,[xmrh,radcurv],ww+5,/xydata,clab='radcurv(cm)'+cadd,ylab='radcurv',xlab='r(cm)'
put_wdfarray,[xmrh,rcenter],ww+6,/xydata,clab='rcenter(cm)'+cadd,ylab='x(cm)',xlab='rcenter(cm)'
put_wdfarray,[xmrh,zcenter],ww+7,/xydata,clab='zcenter(cm)'+cadd,ylab='x(cm)',xlab='zcenter(cm)'
put_wdfarray,[zmrh,Lmin],ww+8,/xydata,clab='Lmin(nH)'+cadd,ylab='L(nH)',xlab='zm(cm)'
Lout=get_value(ww+8,lz)

;integrate dz*(dr/dz)^2
arclen= ww+9
dif, ww+4, arclen
xfr, arclen, arclen+1
yr = getyf(arclen)
yr = yr*yr
puty, yr, arclen
puty, sqrt(1+yr), arclen+1
int,arclen+1, c='arclenght(z)'+cadd,xlab='zmid(cm)',ylab='lenght(cm)', /double
int,arclen,c='int(dz*(dr/dz)^2)'+cadd,/double
last=get_value(arclen,lz)
Lth=2.d0*(w0/rho0)*(lz+last)
;end inducance check
;calculate the arc-length
arclength=get_value(arclen+1,lz)
if arg_present(inductance) || keyword_set(verbose) then inductance = calcl(ww+1, ww+2)

;calculate inductance of TL with straight line mid-gap segment
;quantities for TL with straight line mid-gap segment
lstraight = 0.0
lcoax = 0.0
  if delrr gt 1.d-3 then begin
    ds=sqrt(1.d0+(lz/delr)^2)
    Lstraight=2.d0*ds*(k*delr+w0*alog(r2/r1))
  endif else Lcoax=2.d0*lz*alog((r1+0.5*w1)/(r1-0.5*w1))

  Ldiskcoax=(w2-w1)+(w0*alog((r2+w2/2.d0)/r1))+(lz-w2/2.d0)*alog((r2+w2/2.d0)/(r2-w2/2.d0))
  Ldiskcoax += Ldiskcoax


  Lcoax=2.d0*lz*alog((r1+0.5*w1)/(r1-0.5*w1))

  if flag eq -1 then Lout=-Lout
  if flag eq -1 then Lth=-Lth
  if keyword_set(verbose) then begin
    print,'Lminout,Lequation:',Lout,Lth
    print,'flag=0 equal gap,=-1 w0 negative, =1 w0 positive w2>w1, flag = ',strtrim(flag, 2)
    print,'FOLLOWS: arc-lenght of midgap curve, equivalent disk-coax combination'
    print,'and inductance of straight equivalent cone inductance, value of the '
    print,'midgap optimal curve calculated two different ways'
    print,'total length midgap',arclength
    print,'Ldiskcoax,Lstraight,LCoax:',Ldiskcoax,Lstraight, Lcoax
    Print,'Minimum Inductance:', inductance
  endif
  if not keyword_set(cone) then return

  cone = (cone[0] > 1) < (get_maxwdf() - 6)

;make a straight cone
;if r1=r2 the straight cone is a coaxial line
if delrr lt 1.d-3 then begin
  ncone = 256
  nconem = ncone-1
  dzcoax=lz/(nconem)
  z=dzcoax*findgen(ncone)

  rconecath=make_array(ncone, value=r1-0.5d0*w1)
  rconeanod=make_array(ncone, value=r1+0.5d0*w1)
  zconecath= z
  zconeanod= z
  zconemid= z
  rconemid=make_array(ncone, value=r1)
  gapcone=make_array(ncone, value=w1)
  if arg_present(simple) then simple = 2.0d0*lz*alog(rconeanod[0]/rconecath[0])
endif else begin
  ncone=501
  nconem = ncone-1
  slope=lz/(r2-r1)
  drcone=(r2-r1)/(ncone-1.d0)
  sfi=slope/sqrt(1.d0+slope^2)
  cfi=sqrt(1.d0-sfi^2)
  r=r1+ findgen(ncone)*drcone
  z=slope*(r-r1)
  w=w1+(r-r1)*(w2-w1)/(r2-r1)
  rconecath=r-0.5d0*w*sfi
  rconeanod=r+0.5d0*w*sfi
  zconecath=z+0.5d0*w*cfi
  zconeanod=z-0.5d0*w*cfi
  zconemid=z
  rconemid=r
  gapcone=w
endelse

put_wdfarray,[rconecath,zconecath],cone,/xydata,clab='Cathode cone',ylab='zconecath',xlab='r'
put_wdfarray,[rconeanod,zconeanod],cone+1,/xydata,clab='Anode cone',ylab='zconeanod',xlab='r'
put_wdfarray,[rconemid,zconemid],cone+2,/xydata,clab='Cone Midplane',ylab='zconemid',xlab='r'
put_wdfarray,[rconemid,gapcone],cone+3,/xydata,clab='Cone gap',ylab='gapcone',xlab='r'
if arg_present(simple) && (delrr lt 1.0e-3) then simple = calcl(cone, cone+1)

end
