; ------------------------------------------------------------------------ 
; $Id: mx.pro,v 1.5 2010/03/12 22:58:00 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
 function  mx, wdf1,time = time, left = start, right = stopt, range = range,  $
           min = minn, quiet = quiet, ier = ier, soft = soft, degree = degree, $
           absolute = absolute, tminimum = tmin
;+
;  Function which returns the maximum, or local maximum, of a WDF array and
;  optionally the abscissa value at which it occurs.  Optional parameters
;  allow the specification of a limited portion of the array.
;  XY data is converted to uniform data if SOFT is set
;
;  format: MAX_VALUE = MX(WD1 [, TIME = time] [, LEFT  = left] [, RIGHT = right]
;                      [, RANGE = range] [, MIN = min] [, TMINIMUM = tmin]
;                      [, SOFT = soft] [, DEGREE = degree] 
;                      [, ABSOLUTE = absolute] [, QUIET = quiet])
;  where:
;    MAX_VALUE - Maximum value of WDF array between on the specified interval
;                INF and NAN are ignored.
;    WD1       - WDF array number
;    TIME      - Abscissa value for which the local maximum value occurs;
;                for multiple maxima, this is the first
;    LEFT, RIGHT - Optional abscissa value to specify the left and right
;                abscissa values used to determine the local maximum
;                If absent, the default values are the first and last data
;                points, respectively
;    MIN       - Minimum value on the specified interval
;                INF and NAN are ignored.
;    TMINIMUM  - Abscissa value at which the local minimum occurs.
;    SOFT      - if presetn a fit to the data will be performed to determine
;                the maximum
;                If soft is less than 1 then the fit will be over points 
;                within (soft*mx) of the peak. Default = 0.96
;          Note: Last two waveform arrays are used if SOFT is set.
;                get_maxwdf() has peak values
;                get_maxwdf()-1 has fitted values.
;    RANGE     - Actual range of the abscissa values.
;    DEGREE    - Degree of the fit. DEFAULT = 2 
;                IGNORED unless Soft ne 0
;    ABSOLUTE  - If set the absolute keyword will be set in the "MAX" command
;                (See MAX in the IDL Reference Guide)
;                ABSOLUTE is ignored with the soft keyword.
;    IER       - error flag 0 = no error, 1 if data is not in the range
;    quiet     - If nonzero, no values will be printed to the terminal
;
;  Note:   The IDL function MAX is used to determine max_value and time.
;
;  Examples:
;    Print the maximum value of array 2, between 10 and 1000
;      Print, MX(2, l = 10, r = 1000)
;
;    Calculate the maximum value of array 3 and store it in bot and the time
;    at which it occurs in btime; suppress terminal output
;      bot =  MX(3, time = btime, /quiet)
;
;-
@com.dat
;
on_error, 1
      ier = 1
      value = 0.0
;
      if n_elements(initwdf) eq 0 then wdfinit
      nparams = n_params()
      if nparams eq 0 then begin
        print, '% MX: Waveform array must be provided.'
        return, value
      endif
;
      if n_elements(wdf1) eq 0 then begin
        print, '% MX: WDF parameter must be defined'
        return, value
      endif
      if wdf1 lt 1 or wdf1 gt maxwdfarray then begin
        print, '% MX: WDF array must be greater than 0 and </=', maxwdfarray
        print, '      Value = ', wdf1
        return, value
      endif
;
      np = wdfnp(wdf1-1)
      if np eq 0 then begin
        print, '% MX: No data in the array'
        return, value
      endif
;
      if keyword_set (soft) then get_wdfarray, array, wdf1 $
        else get_wdfarray, array, wdf1, /xy
      if n_elements (array) eq np + np then begin
        if n_elements (start) eq 0 then ist = wdfminx(wdf1-1) else ist = start
        if n_elements (stopt) eq 0 then ilst = wdfmaxx(wdf1-1) else ilst = stopt
        ok = where (array(0:np-1) ge ist and array(0:np-1) le ilst, nok)
        if nok eq 0 then begin
          if not keyword_set(quiet) then begin
           print, '% MX: START and STOP for XY data are not compatable'
           print, '      Maximum for the entire WDF array ', strtrim(wdf1, 2), $
             ' is returned'
          endif
           ok = indgen(np)
        endif else ier = 0
        value = max(array(np+ok), min = minn, jvalue, subscript_min = tmin, $
                 absolute = absolute, /nan)
        jvalue = ok[jvalue]
        time = array[jvalue]
        tmin = ok[tmin]
        tmin = array[tmin]
        range = [min(array[ok], max= mm), mm]
      endif else begin
        np = n_elements(array)-2
        if n_elements (start) eq 0 then ist = 0L else $
           ist = long((start-array(0))/array(1)) > 0L
        if n_elements (stopt) eq 0 then ilst = np-1 else $
           ilst = long((stopt-array(0))/array(1) +1.00001) < np-1
        if ist ge ilst then begin
          if not keyword_set(quiet) then begin
           print, '% MX: START and STOP are not compatable'
           print, '      Minimum for the entire WDF array ', strtrim(wdf1, 2), $
                 ' is returned'
          endif
           ist = 0
           ilst = np-1
        endif else ier = 0
        absol = (keyword_set(absolute)) ? 1:0
        if keyword_set(soft) then absol = 0
        value = max(array[ist+2L:ilst+2L], jvalue, min = minn, $
                subscript_min = tmin, absolute = absol, /nan)

        jvalue = jvalue+ist
        time = array[0] + array(1) * jvalue
        tmin = tmin +ist
        tmin = array[0] + array(1) * tmin
        range = array[0]+array[1]*[ist, ilst]
        if keyword_set (soft) then begin
          if soft lt 1 and soft gt 0 then rat = soft else rat = 0.96
          if keyword_set(degree) then deg = abs(degree) else deg = 2
          ok = where(array(ist+2L:ilst+2L) ge rat*value, nok) 
          if nok lt 3 then $
            print, '% MX: More than 2 points required for a soft maximum' $
          else begin
            np = ok(nok-1)-ok(0)
            put_wdfarray, [(ist+ok[0])*array[1]+array[0], array[1], $
                 array[2L+ok[0]+ist: 2L+ok[nok-1]+ist]], maxwdfarray
            wdfit, maxwdfarray, maxwdfarray-1, degree = deg, /quiet
            get_wdfarray, array, maxwdfarray-1
            value = max(array[2:*], jvalue)
            time = jvalue*array(1)+array(0)
            jvalue = -1
          endelse
        endif 
      endelse
      if not keyword_set(quiet) then begin
        print, ' Maximum value of array ', strtrim(wdf1,2), ' is ' , $
          strtrim(value, 2)
        print, '   on the interval from ', strtrim(range[0], 2), $
           ' to ', strtrim(range[1], 2)
        print, ' Maximum occurs at ', strtrim(time,2), ' (array element is: ', $
            strtrim(jvalue, 2),')'
      endif

      array=0
      return, value
      end
