; ------------------------------------------------------------------------ 
; $Id: newend.pro,v 1.5 2010/09/17 21:20:35 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
 Pro  newend, wdfar, value, delta = deltain, fftopt = fftopt, five = five, $
      sigmoid = sigmoid
;
;+
;  This procedure adds zero value points to a WDF array to extend its
;  domain to a specified value.  If the array's domain already includes
;  the specified endpoint, no action is taken.
;  The first point added will be equal to the point spacing for uniform data
;     and to half the minimum spacing for xy data
;  Note:  To add zero values to an array before the beginning of the data,
;         use the TSH command. 
;  Note:  If data is stored as XY-data, CHECK_UNIFORM will be used to see if it 
;         can be changed to uniform data without interpolation.
;
;  format: 
;    NEWEND, WD1, VALUE, DELTA = delta, FFTOPT = fftopt, FIVE=five, 
;                          SIGMOID = sigmoid
;  where:
;    WD1    - WDF array number or an array of wdf array numbers
;    VALUE  - designated final abscissa value
;    DELTA  - ignored unless XY data. If XY data this will be twice the spacing
;             for additional points if required.
;             If delta is not specified, then NEWEND will set delta to the
;                minimum of the minimum current spacing or 1e-4*max(x)
;    FFTOPT - If set:
;             1. Convert the waveform to uniform if it is not uniform
;                If it is approximately uniform, use the value returned by CHECK_UNIFORM.
;                If it is not uniform, uses I2W to put on a uniform spacing.
;             2. Use VALUE as an approximate number.  Add points to insure the 
;                total number of points, can be factored by powers of 2 and 3.
;    FIVE   - Ignored unless FFTOPT is set.
;             The total number of points will be set to a number factored by powers
;             of 2, 3, and 5.
;    SIGMOID- Ignored unless FFTOPT is set.
;             The zero values are replaced by a sigmoid function of the form:
;               Sig(t) = Amplitude/(1+exp(-t))
;             Where:
;               T   = normalized abscissa with a range of [-5,5] 
;               Amplitude = Half amplitude of the difference in the first and last points
;                           of the waveform.
;
;  Examples:
;
;  Extend the data in array 1 by adding zero values to an abscissa value
;  of 200e-9
;    NEWEND, 1, 200e-9
;
;  Extend the data in array 2, presently with values between abscissa
;  values of 65e-9 to 180e-9, to cover the domain [0, 200e-9] by adding
;  zero values at both ends
;    TSH, 2
;    NEWEND, 2, 200e-9
;
;  Extend the data in array 5 to 1 microsecond to increase the resolution of 
;    an fft.
;
;    NEWEND, 5, 1E-6, /FFT
;  
;-
;
;     include the common block specification
;
@com.dat
;
      on_error, 1
      if n_elements(wdfar) eq 0 then begin
        print, 'wdf array must be specified'
        return
      end
      if n_elements (value) eq 0 then begin
        print,'Final time must be specified'
        return
      endif
      nelwdf = n_elements(wdfar)
    for iii = 0, nelwdf-1 do begin
      wdf = wdfar(iii)
      if wdf lt 1 or wdf gt maxwdfarray then begin
        print, 'wdf array must be greater than 0 and </=', maxwdfarray
        goto , endshift
      endif
      nel = wdfnp(wdf-1)
      if nel lt 2 then begin
        print, ' wdf array', wdf, ' does not have valid data'
        array = 0 
        goto, endshift
      endif
      if wdfmaxx(wdf-1) lt value then begin
        get_wdfarray, array, wdf, /xy
        if n_elements(array) eq nel+nel then begin
          check_uniform, array[0:nel-1], dx, dxrange = dxrange
          if dx gt 0 then array=[array[0], dx, array[nel:*]] else $
          if keyword_set(fftopt) then begin
            if dxrange[0] lt 0.0 then begin
              space = array[0:nel-1]
              image = array[nel:*]
              a = sort(space)
              space = space[a]
              image = image[a]
              check_uniform, space, dx, dxrange
              if dx gt 0 then array=[space[0], dx, image]
            endif else begin
              space = array[0:nel-1]
              image = array[nel:*]
            endelse
            if dx eq 0.0 then begin
              i2w, space, image, dx, /get_delta, endpoint = endpoint
              nel = n_elements(space)
              start = space[0]
              put2uniform, space, image, nel, start,dx, endpoint, array
              nel = n_elements(array)-2
            endif
          endif
        endif
;     calculate the number of points required
        if n_elements(array) eq nel+nel then begin
          if n_elements(deltain) eq 0 then $
            delta = min(abs(array(1:nel-1)-array(0:nel-2))) > $
                 (1.0e-4*abs(wdfmaxx(wdf-1))) 
          arr = [array(0: nel-1), array(nel-1)+0.5*delta, value, $
                   array(nel: nel+nel-1), 0.0, 0.0]
          put_wdfarray, arr, wdf, /xy, /no_copy
        endif else begin
          np = long((value-array(0))/array(1)+ 1.6)
          if keyword_set (fftopt) then np = f23(np, five= five)
          if np gt nel then begin
            arr = make_array (np+2)
            arr(0:nel+1) = array
            if keyword_set(sigmoid) then begin
              blin = arr[nel+1]
              amp = arr[2] - blin 
              nn = (np -nel)
              t = (findgen(nn) +(1.0-0.5*(nn+1)))*(10.0/(nn+1))
              arr[nel+2] = blin + amp/(1.0+exp(-t))
            endif
            put_wdfarray, arr, wdf, /no_copy
          endif
        endelse
      endif
      array = 0
  endshift:
    endfor
      return
      end
