; ------------------------------------------------------------------------ 
; $Id: pe2pff.pro,v 1.5 2007/06/06 20:03:50 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
  Pro pe2pff, filein,  image,space, tlabel = tlab, clabel = clab, $
      xlabel = xlab, ylabel = ylab, blabel = blab, header = header, $
      npoints = npoints, delta = delta,  outfile = outfile, delete = delete, $
      filter = filter, scale = scale, swap = swap, ximage = ximage, $
      xheader = xheader, pheader = pheader, path = path, $
      destination = dest, plotit = plotit
              
;+
;    convert a Perkin Elmer file to a pff file
;             This routine uses read_pe
;
;    format pe2pff [, File_Name] [, IMAGE] [, SPACE]
;            [, TLABEL = tlabel] [, CLABEL = clabel] [, BLABEL = blabel]
;            [, XLABEL = xlabel] [, YLABEL = ylabel] [HEADER = header]
;            [, NPOINTS = npoints] [, DELTA = delta] [, OUTFILE = outfile]
; (input)    [, FILTER = filter] [, SCALE = scale] [SWAP = swap]
;            [, PATH = path] [, DELETE = delete]
;            [, XIMAGE = ximage] [, XHEADER = xheader] [PHEADER = pheader]
;            [, DESTINATION = dest] [, PLOTIT = plotit]   
;
;     where 
;        File_Name - the name of the file with or without an extension
;                    if no name is specified, PICKFILE will be used
;    image -    array containing the image file
;    space -    array containing the dataset x and y arrays. Use
;                  SP2XYZ, image, space, x, y
;               to obtain x and y (SPACE = [X, Y])
;               Note: MicroD offsets are ignored.  Both X and Y start at (0, 0)
;    tlabel -   Dataset type label
;               Default is the value of "SETVAL"
;                 (Specular Density, Transmission or Intensity)
;    clabel - dataset comment label
;             Default is File Name // Comment1 // Comment2 // ScanTime
;    blabel - title or label for this block
;             Default is Specular Density, Transmission or Intensity
;               depending on the value of "SETVAL"
;    xlabel - horizontal dimension label for this block
;             Default is 'X (microns)'
;    ylabel - Verticle dimension label for this block
;             Default is 'Y (microns)'
;    header - Complete header for the PE dataset
;    npoints- Array of points for each dimension
;    delta  - Array of microns/step for each dimension
;    outfile- File name without path information
;    filter - Optional filter. If specified this parameter must have at least
;               two values which are taken as the maximum and minimum of the
;               scanned data.  If three parameters are given, the third is taken
;               as the default value given to all data which falls outside the
;               specified range.  Default value is [0, 5000, -1]
;               That is all data less than 0 and greater than 5000 will be set
;               to -1.  With the default scaling of 800, this corresponds to
;               values between 0 and 6.25
;    scale  - Parameter used to scale the integer counts.
;             Default is 1/800 = 1.25e-3
;    swap   - This keyword is set by default according to the following:
;             Windows and linux: SWAP =0
;             UNIX: SWAP = 1
;             If SWAP = 0, then the bytes in IMAGE will not be swapped.
;             Note: VAX data must be swapped.
;    path   - Directory path to the desired file
;             This is the same default value as used in OPENPFF
;    delete  - If set, delete the Perkin Elmer files
;    ximage  - Extension for the image file. Default is .img
;    xheader - Extension for the header file. Default is .hdr
;    pheader - If the keyword is set, the header will be printed to the screen
;
;    destination - Path of the destination file
;                     If no destination is specified the file will be
;                     written in the same directory as the PE file
;    plotit    - if set, a crude plot will be made of the image
;
;     Example
;     pe2pff, 'S6411n5j' ; read file S6411n5j from current directory
; or 
;     pe2pff             ; read a file to be determined form PICKFILE
; or  pe2pff, /pheader, 'S6411n5j.Hdr' ; read S6411n5j and print header
;     pe2pff, /plot     ; read a file to be determined and plot it
;-
;
 on_error, 1
;
      if n_elements(swap) eq  0 then begin
        no_swap = ['linux', 'Win32']
        swap =( where(!version.os eq no_swap) ge 0) ? 0:1
      endif
      read_pe, filein, image, space, tlabel = tlab, clabel = clab, $
        xlabel = xlab, ylabel = ylab, blabel = blab, header = header, $
        npoints = npoints, delta = delta,  outfile = outfile, $
        filter = filter, scale = scale, swap = swap, ximage = ximage, $
        xheader = xheader, pheader = pheader, path = path, $
        file_one = file_one, file_two = file_two, destination = dest
;
      if strlen(filein) lt 1 then return
      nel_im = n_elements(image)
      if (nel_im lt 4) then return
      psep = path_sep()
      if n_elements(dest) eq 0 then fileout = filein + '.pff' else begin
        l = strlen(dest)
        ii = strpos(dest, psep, l-1)
        if ii lt 0 then fileout = dest+psep+outfile + '.pff' else $
           fileout = dest+outfile + '.pff'
      endelse
      print, ' Writing PFF file: ', fileout
      print, '   Image size:', npoints
      fileout = find_newfile(fileout)
      print, 'Open file: ', strtrim(fileout, 2)
      createpff, fileout, error = err
      if err ne 0 then begin
        print, '% PE2PFF: Error opening PFF file.'
        return
      endif 

      writepff, 0, image, space, tlabel = tlab, clabel = clab, $
        xlabel = xlab, ylabel = ylab, blabel = blab
      nds = ndspff(0)
      if nds ne 1 then $
        print, '% PE2PFF: Error writing dataset to PFF file'
      closepff
      if keyword_set(plotit) then begin
         loadxyct
         contour, image, lev = findgen(4)+1.2,  c_col= [3, 1, 2, 7]
      endif
      if keyword_set(delete) then begin
        if nds ne 1 then  $
          print, '% PE2PFF: Number of datasets not equal to 1', nds  else begin
;     Get the total length in bytes
          openr,unit, /get,  fileout, error = err
          if err ne 0 then begin
            print, '% PE2PFF: Error re-opening PFF file.'
            return
          endif
          total = fstat(unit)
          total = total.size
          if total lt 2L*nel_im then begin
            print, '% PE2PFF: PFF file too small for the image'
            print, '          Data files not deleted'
            return
          endif
          spawn, '/bin/rm '+file_one
          spawn, '/bin/rm '+file_two
        endelse
      endif
      return
      end
