; ------------------------------------------------------------------------
; $Id: phyconst.pro,v 1.1 2009/11/03 17:06:50 lpmix Exp $
; ------------------------------------------------------------------------
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
;+
; NAME:
;       PHYCONST
;
; PURPOSE:
;       This procedure returns physical constants as keywords.
;       A related function, PHYCONSTF, returns the constants to the function call.
;
; CALLING SEQUENCE:
;
;       PHYCONST 
;
; INPUTS:
;       No input parameters are required
;
; KEYWORD PARAMETERS:
;    Note: The following values may not be exact if the experimental values of the 
;          constants are changed.
;
;       C:       Speed of Light
;          NOTE: Function SOL also returns this value
;                C = SOL()
;                C = 299792458.0D0 ; meters/second
;       SOLIGHT: Same as C 
;
;       Q:       Elementary Charge
;                Q = 1.602176487d-19 ; Coulomb
;       ECHARGE: Same as Q
;
;       ERADIUS: Classical electron radius 
;                ERADIUS = Fine_Structure_Const^2 * Bohr_Radius
;                ERADIUS = 2.8179402894d-15  ; meters
;
;       JPLANCK: Planck's constant in Joule*Second
;                JPLANCK = 6.62606896d-34  ; j s
;
;       EPLANCK: Planck's constant in eV *Second
;                EPLANCK = JPLANCK / Q ; eV s
;
;       MU0:     Magnetic constant
;                MU0 = !dpi * 4.0d-7
;
;       EPSILON: Electric constant
;                EPSILON = 1.d0/(MU0*C*C)
;
;       EMASS:   Electron Mass (kg)
;                EMASS = 9.10938215d-31  ; kg
;
;       EENERGY: Electron Rest Energy (eV)
;                EENERGY = EMASS* (C*C/Q)  ; eV
;
;       PMASS:   Proton Mass (kg)
;                PMASs = 1.672621637d-27  ; kg
;
;       PENERGY: Proton Rest Energy (eV)
;                EENERGY = PMASS* (C*C/Q)  ; eV
;
;       NMASS:   Neutron Mass (kg)
;                NMASS = 1.674927211d-27  ; kg
;
;       NENERGY: Neutron Rest Energy (eV)
;                NENERGY = NMASS* (C*C/Q)  ; eV
;
;       AMASS:   AMU Mass (kg)
;                AMASS = 1.660538782d-27  ; kg
;       AMU:     AMU Mass (kg)
;                AMU = 1.660538782d-27  ; kg
;
;       AENERGY: AMU Rest Energy (eV)
;                AENERGY = AMASS* (C*C/Q)  ; eV
;
;       DMASS:   Mass (kg)
;                DMASS = 3.34358320D-27  ; kg
;
;       DENERGY: Deuteron Rest Energy (eV)
;                DENERGY = DMASS* (C*C/Q)  ; eV
;
;       ELAMBDA: Energy * Wavelength (eV * m)
;                ELAMBDA = EPLANCK * C  ; eV m
;       EALAMBDA: Energy * Wavelength (eV * Angstrom)
;                EALAMBDA = 1d10 * ELAMBDA
;
;       BOLTZMANN: Boltzmann's Constant (J/Deg_K)
;                  BOLTZMANN = 1.3806504d-23  ; J / Deg_K
;       EBOLTZMANN: Boltzmann's Constant (eV/Deg_K)
;                   EBOLTZMANN = BOLTZMANN/Q   ; eV / Deg_K
;
;       AVOGADRO: Avogadro's number
;                 AVOGADRO = 6.02214179d23  ; number / mole
;
;       HELPME:   Returns a hint on the format of the command
;
;
; OUTPUTS:
;       Keywords return the values of the desired physical constants
;
; PROCEDURE:
;
;       Values are returned taken from the July, 2007 issue of Physics Today
;       These are the recommended values of the fundamental physical constants
;         for 2006.  In general the basic value is taken from the physical
;         tabular values, but derived values are calculated.  
;       The following are taken from the table all other values are calculated:
;          Speed of Light (exact) (m/s)
;          Elementary Charge (coulomb)
;          Planck's Constant (J*s)
;          Classical Electron Radius (m)
;          Electron, Proton, Neutron, Deuteron, and  1 AMU Rest Mass (kg)
;          Boltzmann constant (J / Deg_K)
;          Avogadro number
;
; EXAMPLE:
;       Obtain the value of the electron rest mass and classical electron radius.
;       Store these values in variables, emass and erad.
;         PHYCONST, EMASS = emass, ERADIUS = erad
;
; MODIFICATION HISTORY:
; 	Written by:	L. P. Mix, Jr., October 1, 2009
;	July, 2010	
;-

 Pro phyconst, c = spol2, solight = spol, jplanck = jplanck, eplanck = eplanck, $
       q = qcharge, echarge = charge, eradius = eradius, mu0 = mu0, epsilon = epsilon, $
       emass = emass, pmass = pmass, nmass = nmass, amu = amu, elambda= elambda, $
       avogadro = avogadro, boltzmann = boltzmann, eboltzmann = eboltzmann, $
       eenergy = eenergy, penergy = penergy, nenergy = nenergy, aenergy = aenergy, $
       amass = amass, dmass = dmass, denergy = denergy, ealambda = ealambda, helpme = helpme

on_error, 1


if keyword_set (helpme) then begin
  print, '% Format: PHYCONST, C = sol, SOLIGHT = sol, JPLANCK = jplanck, EPLANCK = eplanck,'
  print, '     Q = charge, ECHARGE = charge, ERADIUS = eradius,'
  print, '     MU0 = mu0, EPSILON = epsilon,'
  print, '     EMASS = emass, PMASS = pmass, NMASS = nmass, AMU = amu, AMASS = amu,'
  print, '     DMASS = dmass, ELAMBDA = elambda, EALAMBDA = ealambda, AVOGADRO = avogadro,'
  print, '     EENERGY = eenergy, PENERGY = penergy, NENERGY = nenergy, AENEERGY =aenergy, '
  print, '     DENERGY = denergy, BOLTZMANN = boltzmann, EBOLTZMANN = eboltzmann,'
  print, '     /HELPME'
  return
endif
@physical_constants.dat

; duplicate names
  spol2 = spol
  qcharge = charge
  amass = amu

; Planck constant in eV*s
  eplanck = jplanck/charge  ; eV s

; Magnetic constant mu0 = 4e-7*pi
  mu0 = 4.0d-7*!dpi

; Electric constant = 1/ (mu0*c^2)
  epsilon = 1.d0/(mu0*spol*spol)

; Conversion of mass to energy in eV
; c*c/q
  convert = spol*spol/charge

; Electron Rest Mass (eV)
  eenergy = emass*convert  ; eV

; Proton Rest Mass (eV)
  penergy = pmass*convert ; eV
 
; Neutron Rest Mass (eV)
  nenergy = nmass*convert  ; eV
 
; AMU Rest Mass (eV)
  aenergy = amu*convert  ; eV

; Deuteron Rest Energy
  denergy = dmass* convert  ; eV

; Energy * Wavelength (eV*m)
  elambda = eplanck *spol  ; eV*m
  ealambda = 1.0d10 * elambda  ; eV * Angstrom

; Boltzmann constant
  eboltzmann = boltzmann/charge ; eV / Deg_K

end   
