; ------------------------------------------------------------------------
; $Id: phyconstf.pro,v 1.2 2010/01/22 18:04:21 lpmix Exp $
; ------------------------------------------------------------------------
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
;+
; NAME:
;       PHYCONSTF
;
; PURPOSE:
;       This function returns a physical constant.
;       A related procedure, PHYCONST, returns the all constants as keywords.
;
; CALLING SEQUENCE:
;
;       VALUE = PHYCONSTF() 
;
; INPUTS:
;       No input parameters are required
;
; KEYWORD PARAMETERS:
;       Note: Only one keyword is processed.
;       Note: If no keywords are present, a help message is printed.
;    Note: The following values may not be exact if the experimental values of the 
;          constants are changed.
;
;       C:       Speed of Light
;          NOTE: Function SOL also returns this value
;                C = SOL()
;                C = 299792458.0D0 ; meters/second
;       SOLIGHT: Same as C 
;
;       Q:       Elementary Charge
;                Q = 1.602176487d-19 ; Coulomb
;       ECHARGE: Same as Q
;
;       ERADIUS: Classical electron radius 
;                ERADIUS = Fine_Structure_Const^2 * Bohr_Radius
;                ERADIUS = 2.8179402894d-15  ; meters
;
;       JPLANCK: Planck's constant in Joule*Second
;                JPLANCK = 6.62606896d-34  ; j s
;
;       EPLANCK: Planck's constant in eV *Second
;                EPLANCK = JPLANCK / Q ; eV s
;
;       MU0:     Magnetic constant
;                MU0 = !dpi * 4.0d-7
;
;       EPSILON: Electric constant
;                EPSILON = 1.d0/(MU0*C*C)
;
;       EMASS:   Electron Mass (kg)
;                EMASS = 9.10938215d-31  ; kg
;
;       EENERGY: Electron Rest Energy (eV)
;                EENERGY = EMASS* (C*C/Q)  ; eV
;
;       PMASS:   Proton Mass (kg)
;                PMASs = 1.672621637d-27  ; kg
;
;       PENERGY: Proton Rest Energy (eV)
;                EENERGY = PMASS* (C*C/Q)  ; eV
;
;       NMASS:   Neutron Mass (kg)
;                NMASS = 1.674927211d-27  ; kg
;
;       NENERGY: Neutron Rest Energy (eV)
;                NENERGY = NMASS* (C*C/Q)  ; eV
;
;       AMASS:   AMU Mass (kg)
;                AMASS = 1.660538782d-27  ; kg
;       AMU:     AMU Mass (kg)
;                AMU = 1.660538782d-27  ; kg
;
;       AENERGY: AMU Rest Energy (eV)
;                AENERGY = AMASS* (C*C/Q)  ; eV
;
;       DMASS:   Mass (kg)
;                DMASS = 3.34358320D-27  ; kg
;
;       DENERGY: Deuteron Rest Energy (eV)
;                DENERGY = DMASS* (C*C/Q)  ; eV
;
;       ELAMBDA: Energy * Wavelength (eV * m)
;                ELAMBDA = EPLANCK * C  ; eV m
;       EALAMBDA: Energy * Wavelength (eV * Angstrom)
;                EALAMBDA = 1d10 * ELAMBDA
;
;       BOLTZMANN: Boltzmann's Constant (J/Deg_K)
;                  BOLTZMANN = 1.3806504d-23  ; J / Deg_K
;       EBOLTZMANN: Boltzmann's Constant (eV/Deg_K)
;                   EBOLTZMANN = BOLTZMANN/Q   ; eV / Deg_K
;
;       AVOGADRO: Avogadro's number
;                 AVOGADRO = 6.02214179d23  ; number / mole
;
; OUTPUTS:
;       The physical constant is returned by the function
;
; PROCEDURE:
;
;       Values are returned taken from the July, 2007 issue of Physics Today
;       These are the recommended values of the fundamental physical constants
;         for 2006.  In general the basic value is taken from the physical
;         tabular values, but derived values are calculated.  
;       The following are taken from the table all other values are calculated:
;          Speed of Light (exact) (m/s)
;          Elementary Charge (coulomb)
;          Planck's Constant (J*s)
;          Classical Electron Radius (m)
;          Electron, Proton, Neutron, Deuteron, and  1 AMU Rest Mass (kg)
;          Boltzmann constant (J / Deg_K)
;          Avogadro number
;
; EXAMPLE:
;       Obtain the value of the electron rest mass in the variable ME0
;         ME0 = PHYCONSTF( /EMASS) 
;
; MODIFICATION HISTORY:
; 	Written by:	L. P. Mix, Jr., October 1, 2009
;	July, 2010	
;-

 Function phyconstf, c = spol2, solight = kspol, jplanck = kjplanck, eplanck = keplanck, $
       q = kqcharge, echarge = kcharge, eradius = keradius, mu0 = kmu0, epsilon = kepsilon, $
       emass = kemass, pmass = kpmass, nmass = knmass, amu = kamu, elambda= kelambda, $
       avogadro = kavogadro, kboltzmann = kboltzmann, keboltzmann = keboltzmann, $
       eenergy = keenergy, penergy = kpenergy, nenergy = knenergy, aenergy = kaenergy, $
       amass = kamass, dmass = kdmass, kdenergy = kdenergy, ealambda = kealambda

on_error, 1

@physical_constants.dat
if keyword_set(kspol) || keyword_set(spol2) then return, spol
if keyword_set(kqcharge) || keyword_set(kcharge) then return, charge
if keyword_set(kamass) || keyword_set(kamu) then return, amu

if keyword_set(kjplanck) then return, jplanck
if keyword_set(keplanck) then return, jplanck/charge

if keyword_set(keradius) then return, eradius
if keyword_set(kmu0) then return,  4.0d-7*!dpi
if keyword_set(kepsilon) then return, 1.d0/(4.0d-7*!dpi*spol*spol)

if keyword_set(kemass) then return, emass
if keyword_set(kpmass) then return, pmass
if keyword_set(knmass) then return, nmass
if keyword_set(kdmass) then return, dmass

if keyword_set(kboltzmann) then return, boltzmann
if keyword_set(keboltzmann) then return, boltzmann/charge
if keyword_set(kavogadro) then return, avogadro

if keyword_set(kelambda) then return, jplanck/charge*spol
if keyword_set(kealambda) then return, jplanck/charge*spol*1.0d10

; Conversion of mass to energy in eV
; c*c/q
  convert = spol*spol/charge

if keyword_set(keenergy) then return, emass*convert
if keyword_set(kpenergy) then return, pmass*convert
if keyword_set(knenergy) then return, nmass*convert
if keyword_set(kaenergy) then return, amu*convert
if keyword_set(kdenergy) then return, dmass*convert

  print, '% Format: PHYCONSTF, C = sol, SOLIGHT = sol, JPLANCK = jplanck, EPLANCK = eplanck,'
  print, '     Q = charge, ECHARGE = charge, ERADIUS = eradius,'
  print, '     MU0 = mu0, EPSILON = epsilon,'
  print, '     EMASS = emass, PMASS = pmass, NMASS = nmass, AMU = amu, AMASS = amu,'
  print, '     DMASS = dmass, ELAMBDA = elambda, EALAMBDA = ealambda, AVOGADRO = avogadro,'
  print, '     EENERGY = eenergy, PENERGY = penergy, NENERGY = nenergy, AENEERGY =aenergy, '
  print, '     DENERGY = denergy, BOLTZMANN = boltzmann, EBOLTZMANN = eboltzmann,'

end   
