; ------------------------------------------------------------------------ 
; $Id: poly_roots.pro,v 1.3 2007/06/06 20:03:50 lpmix Exp $
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
;+
; Function Poly_ROOTS
;   Returns the real roots of a quadratic or cubic equation of the form:
;   
;      A*X^3 + B*X^2 + C*X + D = 0.0
; or   A*X^2 + B*X^1 + C = 0.0
; or   Coef[0] + Coef[1]*X + Coef[2]*x^2 + Coef[3]*X^3 = 0
;
; Format:
;
;  Poly_Roots = Poly_ROOTS(A, B, C [, D] [, /verbose] [, /REAL_ONLY] [, COEF = coef])
; or 
;  Poly_Roots = Poly_ROOTS(Coef, [, /verbose] [, /complex])
;
; PARAMETERS:
;
;   Poly_ROOTS  - Roots of the polynomial
;                 Only real roots are returned if REAL_ONLY is set.
;                 In general, Poly_Roots is a complex array.
;   A, B, C, D  - Coefficients of a polynomial as noted below:
;                 A*X^3 + B*X^2 + C*X + D = 0.0
;                 For a quadratic function, D must be absent.         
;   COEF        - Coefficient array of a function.
;                 Same definition as C in the IDL function: POLY(X, C)
;
;  NOTE:  print, poly(Poly_Roots, coef) 
;         should return very small values.
; Keywords:
;
;   VERBOSE    - if set, print the values
;   REAL_ONLY  - Ignored unless roots are complex
;                If set, only real roots are returned.
;   COEF       - Same as COEF parameter.
;
; Modification history:  Original version: 4 October 2006, L. Paul Mix
;-

 function poly_roots, ain, bin, cin, din, verbose = verbose, coef = coef, REAL_only = realonly
;
on_error, 1
lcquad = 0
npar = n_params()
if npar eq 1 then begin
  nela = n_elements(ain)

  if nela le 2 then begin
    print, '% Poly_ROOTS: Array must have three values to solve for roots.'
    return, 0
  endif else begin
    a = double(ain)
    if (nela ge 4) && (ain[3] eq 0.0) then nela = 3
    if nela eq 3 then begin
      c= a[0]
      b= a[1]
      a= a[2]
      coef = a[0:2]
      lcquad = 1
    endif else begin
      coef = a[0:3]
      a3 = a[0]/a[3]
      a2 = a[1]/a[3]
      a1 = a[2]/a[3]
      lcquad = 0
    endelse
  endelse
endif else if npar lt 3 then begin
  print, '% Poly_ROOTS: A minimum of three parameters are required'
  return, 0
endif else if npar eq 3 then begin
  if (n_elements(ain) eq 0) || (n_elements(bin) eq 0) || (n_elements(cin) eq 0) then begin
    print, '% Poly_ROOTS: The three parameters must be defined'
    return, 0
  endif
  a = double(ain[0])
  b = double(bin[0])
  c = double(cin[0])
  coef = [c, b, a]
  lcquad = 1
endif else begin
  if (n_elements(ain) eq 0) || (n_elements(bin) eq 0) || (n_elements(cin) eq 0) $
    || (n_elements(din) eq 0) then begin
    print, '% Poly_ROOTS: The four parameters must be defined'
    return, 0
  endif
  a = double(ain[0])
  a1 = double(bin[0])/a
  a2 = double(cin[0])/a
  a3 = double(din[0])/a
  coef= [a3, a2, a1, 1.0]*a
  lcquad = 0
endelse

if lcquad eq 0 then begin
;
  Q = (a1*a1-3*a2)/9.0d0
  R = (2.0d0*a1^3 -9.0d0*a1*a2+27.0d0*a3)/54D0
  Q3 = Q*Q*Q
 
  if (Q3-R*R) ge 0 then begin
    theta = acos(R/sqrt(q3))/3.0d0
    x = make_array(3, /double)
    x[0] = -2.0d0*sqrt(q)*cos(theta) - a1/3.0d0
    fudge = 2.0d0*!dpi/3.0d0
    x[1] = -2.0d0*sqrt(q)*cos(theta+fudge) - a1/3.0d0
    x[2] = -2.0d0*sqrt(q)*cos(theta+fudge+fudge) - a1/3.0d0
  endif else begin
    t = (R ge 0.0) ? -1.0d0 : 1.0d0
    stuff = (sqrt(r*r-q3)+ abs(r))^(1.0d0/3d0)
    x = t*( stuff +q/stuff) - a1/3.0d0
    a = 1
    b= a1+x
    c= a2 +x*b
    residual = a3 + x*c
    if keyword_set(verbose) then help, residual
    if keyword_set(realonly) then begin
      print, '% Poly_ROOTS: Cubic equation has complex roots.'
    endif else begin
      z = temporary(x)
      x = make_array(3, /dcomplex)
      x[0]= dcomplex(z, 0.0d0)
      b4ac = b*b-4.0d0*a*c
      z = sqrt(dcomplex(b4ac, 0.0d0))
      x[1:2] = [(-b+z), -(b+z)]/ (a+a)
    endelse
  endelse
endif else begin
  b4ac = b*b-4.0d0*a*c
  if b4ac lt 0 then begin
    if keyword_set (realonly) then begin
      Print, '% QUADRATIC: Quadratic coefficients have complex roots.'
      return, 0
    endif else begin
      z = sqrt(dcomplex(b4ac, 0.0d0))
      x = [(-b+z), -(b+z)]/ (a+a)
    endelse
  endif else begin
    b4ac = sqrt(b4ac)
    q =  (b lt 0) ?  -0.5d0*(b-b4ac) : -0.5d0*(b+b4ac)
    x =  [q/a, c/q]
  endelse
endelse
 
if keyword_set(verbose) then begin
 if lcquad eq 0 then begin
   print, 'Real roots to the cubic equation  A*X^3 + B*X^2 + C*X + D = 0.0'
   print, '    Where A = ', strtrim (coef[3], 2)
   print, '    Where B = ', strtrim (coef[2], 2)
   print, '    Where C = ', strtrim (coef[1], 2)
   print, '    Where D = ', strtrim (coef[0], 2)
 endif else begin
   print, 'Real roots to the quadratic equation  A*X^2 + B*X^1 + C = 0.0'
   print, '    Where A = ', strtrim (coef[2], 2)
   print, '    Where B = ', strtrim (coef[1], 2)
   print, '    Where C = ', strtrim (coef[0], 2)
 endelse
 if n_elements(x) eq 1 then $
   print, 'Root = ', strtrim (x[0],2) else if n_elements(x) eq 2 then $
   print, 'Roots = ',strtrim (x[0],2)+', '+strtrim (x[1],2) else $
   print, 'Roots = ',strtrim (x[0],2)+', '+strtrim (x[1],2)+', '+strtrim (x[2],2)
endif
 return , x 
 end
