; ------------------------------------------------------------------------ 
; $Id: read_pe.pro,v 1.7 2010/12/09 16:46:44 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
  Pro read_pe, fileinput, image,space, tlabel = tlab, clabel = clab, $
      xlabel = xlab, ylabel = ylab, blabel = blab, header = header, $
      npoints = npoints, delta = delta,  outfile = outfile, $
      filter = filter, scale = scale, swap = swap, ximage = ximage, $
      xheader = xheader, pheader = pheader, path = path, $
      file_one = file_one, file_two= file_two, destination = dest
;
;+
;  Read a Perkin Elmer formatted dataset into IDL. 
;
;  Format: READ_PE , File_Name , image , space 
;            [, TLABEL = tlabel] [, CLABEL = clabel] [, BLABEL = blabel]
;            [, XLABEL = xlabel] [, YLABEL = ylabel] [HEADER = header]
;            [, NPOINTS = npoints] [, DELTA = delta] [, OUTFILE = outfile]
; (input)    [, FILTER = filter] [, SCALE = scale] [SWAP = swap] 
;            [, PATH = path]
;            [, XIMAGE = ximage] [, XHEADER = xheader] [PHEADER = pheader]
;  where:
;    File_Name - Name of the Perkin Elmer file with or without an extension.
;                Note: Any extensions will be removed and 
;                      extensions of .hdr and .img  will be assumed.
;                (See XHEADER and XIMAGE below.)
;               filein will be modified to provide a complete path name
;    image -    array containing the image file
;    space -    array containing the dataset x and y arrays. Use
;                  SP2XYZ, image, space, x, y 
;               to obtain x and y (SPACE = [X, Y])
;               Note: MicroD offsets are ignored.  Both X and Y start at (0, 0)
;    tlabel -   Dataset type label
;               Default is the value of "SETVAL"
;                 (Specular Density, Transmission or Intensity)
;    clabel - dataset comment label
;             Default is File Name // Comment1 // Comment2 // ScanTime
;    blabel - title or label for this block
;             Default is Specular Density, Transmission or Intensity
;               depending on the value of "SETVAL"
;    xlabel - horizontal dimension label for this block
;             Default is 'X (microns)'
;    ylabel - Verticle dimension label for this block
;             Default is 'Y (microns)'
;    header - Complete header for the PE dataset
;    npoints- Array of points for each dimension
;    delta  - Array of microns/step for each dimension
;    outfile- File name without path information
;    filter - Optional filter. If specified this parameter must have at least
;               two values which are taken as the maximum and minimum of the
;               scanned data.  If three parameters are given, the third is taken
;               as the default value given to all data which falls outside the
;               specified range.  Default value is [0, 5000, -1]
;               That is all data less than 0 and greater than 5000 will be set
;               to -1.  With the default scaling of 800, this corresponds to 
;               values between 0 and 6.25
;    scale  - Parameter used to scale the integer counts.
;             Default is 1/800 = 1.25e-3
;    swap   - This keyword is set by default according to the following:
;             Windows and linux: SWAP =0
;             UNIX: SWAP = 1
;             If SWAP = 0, then the bytes in IMAGE will not be swapped.
;    path   - Directory path to the desired file
;             This is the same default value as used in OPENPFF
;    ximage  - Extension for the image file. Default is .img
;    xheader- Extension for the header file. Default is .hdr
;    pheader - If the keyword is set, the printer will be printed to the screen
;
;    EXAMPLE
;        read the file S6424n4f and retrieve all the parameters for writing 
;        the file as a pff file.  Print the header.
;            read_pe, 'S6424n4f', image, space, tlab = tlab, clab = clab
;              blab = blab, xlab = xlab, ylab = ylab, /ph
;
;        Note: to write the above data first insure a PFF file is open or
;              if necessary create the file and then write it:
;            createpff, 'S6424n4f.pff'
;            writepff, 0, image, space, tlab = tlab, clab = clab
;              blab = blab, xlab = xlab, ylab = ylab
;
;
;    written 1/ 16/ 95 by lpm
;-
; 
     common savepe_picker, oldfilter, oldpath
     on_error, 1
     ss = size(fileinput)
     image = 0
     if ss( ss(0)+1) ne 7 then begin
       if n_elements(oldpath) eq 0 then cd, current = path else path = oldpath
       p1 = dialog_pickfile(/must_exist, /read, path = path, $
         filter = ['*.hdr;*.HDR', '*.*'], get_path = newpath)
       if strlen(p1) lt 1 then begin
         Print, 'Input file name must be provided'
         return
       endif
       oldpath = newpath
       filein = strtrim(temporary(p1), 2)
     endif else filein = strtrim (fileinput, 2)
     resolve_it, filein
     if n_elements(fileinput) eq 0 then fileinput=filein
     ii = strpos(filein, '.', /reverse_search)
     if ii gt 0 then filein = strmid(filein, 0, ii)
     if strlen(filein) lt 1 then begin
       Print, '% READ_PE: Input file must have at least one character'
       return
     endif
     if n_elements(xheader) eq 0 then xheader = '.hdr'
     xheader = strtrim(xheader, 2)
     ii = strpos(xheader, '.')
     if ii lt 0 then head = filein+'.'+xheader else head = filein+xheader 
     file_one = head
     openr, unit, /get_lun, head, error = err
     if err ne 0 then begin
       print, '% READ_PE: ', !ERR_STRING
       image = 0
       return
     endif
     a = fstat(unit)
     b = make_array(/byt, a.size)
     readu, unit, b
     free_lun, unit
     ok = where (b lt 32, nok)
     if nok lt 22 then begin
       Print, 'Error reading header.' , a.size, ' bytes have been read'
       return
     endif
     if ok(nok-1) ne a.size-1 then begin
       nok = nok+1
       ok = [ok, a.size-1]
     endif
     header = make_array(nok, /string)
     st = 0
     n = 0
     for i = 0, nok-1 do begin
       if ok(i) gt st+1 then begin
         header(n) =strtrim( string(b(st:ok(i)-1)), 2)
         n = n+1
       endif
       st = ok(i) +1
     endfor
     header = header (0:n-1)
     nelhead = n
     if keyword_set(pheader) then for i = 0, n-1 do print, header(i)
     i = 0
     while (strpos(header(i), 'IDENTIFICATION') lt 0) do i = i+1
     i = execute(header(i))
     i = 1
     while (strpos(header(i), 'NXP') lt 0) do i = i+1
     nx = long(strmid(header(i), 9, 20))
     i = i+1
     while (strpos(header(i), 'NYP') lt 0) do i = i+1
     ny = long(strmid(header(i), 9, 20))
     i = i+1
     while (strpos(header(i), 'DELTAX') lt 0) do i = i+1
     deltax = long(strmid(header(i), 9, 20))
     i = i+1
     while (strpos(header(i), 'DELTAY') lt 0) do i = i+1
     deltay = long(strmid(header(i), 9, 20))
     delta = [deltax, deltay]
     npoints = [nx, ny]
     space = [findgen(nx)*deltax, findgen(ny)*deltay]
     i = i+1
     while (strpos(header(i), 'FILE REC LEN') lt 0) do i = i+1
     record = strmid(header(i), 14, 20)
;    first calculate the number of lines/record
     nl = record/nx
;    now calculate the number of values /line
     nline =  record/nl
     if nl*nline ne record then begin
       print,' Record length and number of lines are not consistent'
       print,' NX, Num_Line, Number, Record',fix(nx),fix(nl),fix(nline),record
       return
     endif
     i = i+1
     while (strpos(header(i), 'SETVAL') lt 0) do i = i+1
     setval = strmid(header(i), 27, 1)
     if setval eq 'D' then blab = 'Specular Density' else begin
        if setval eq 'T' then  blab = 'Transmission' else blab = 'Intensity'
     endelse
     tlab = blab
     i = 1
     while (strpos(header(i), 'SCAN TIME') lt 0) do begin
       i = i+1
       if (i eq nelhead) then  begin
         clab =   identification  
         goto, nocomments
       endif
     endwhile
;    add a few extra characters
     sctime = strmid(header(16), 18, 22)
     ii  =  strpos (sctime ,"'"  )
     if ii gt 0 then sctime = strmid(sctime, 0, ii)
     commt1 = strmid(header(20), 8, 80)
     ii  =  strpos (commt1 ,"'"  )
     if ii gt 0 then commt1 = strmid(commt1, 0, ii)
     commt2 = strmid(header(21), 8, 80)
     ii  =  strpos (commt2 ,"'"  )
     if ii gt 0 then commt2 = strmid(commt2, 0, ii)
     if nok gt 0 then commt2 = strmid(commt2, 0, ok(0))
     clab = filein
     l = strlen(clab)
     ii = strpos(clab, path_sep(), /reverse_search)
     if ii ge 0 then begin
       l = l-ii-1
       clab_old = clab
       clab = strmid(clab, ii+1)
     endif
     outfile = clab
;    
     if n_elements(dest) eq 0 then $
        dest=(ii ge 0) ? strmid(clab_old, 0, ii+1) : '.'+path_sep()
     clab = clab + ': '+ strtrim(commt1, 2)+ ': '+ strtrim(commt2, 2) + $
           ': ' + sctime
nocomments:
     xlab = 'X (microns)'
     ylab = 'Y (microns)'
     xline_read = make_array(nline, /int)
     if n_elements(ximage) eq 0 then ximage= '.img'
     ii = strpos(ximage, '.')
     if ii lt 0 then head = filein+'.'+ximage else head = filein+ximage
     file_two = head
     openr, unit, /get_lun, head, error = err
     if err ne 0 then begin
       print, '% READ_PE: ', !ERR_STRING
       image = 0
       return
     endif
     image = make_array( nx, ny)
     if n_elements(swap) eq 0 then begin
       no_swap = ['linux', 'Win32']
       swap =( where(!version.os eq no_swap) ge 0) ? 0:1
     endif
     if n_elements (filter) eq 2 then filter = [filter, -1] else begin
       if n_elements (filter) eq 0 then filter = [0, 5000, -1] else filter =0
     endelse
     if nline lt nx then begin
       print, 'Error in calculating points per line. (nx, nline)', nx, nline
       return
     endif
     if n_elements(scale) eq 0 then scale = 1.25e-3
     for i = 0, ny-1 do begin
       readu, unit, xline_read
       if swap ne 0 then byteorder, xline_read
       xline = xline_read[0:nx-1]
       if n_elements(filter) lt 3 then nok =0 else begin
         ok = where (xline lt filter(0) or xline gt filter(1), nok)
         if nok gt 0 then begin
           print, nok, ' values of the image set to ', strtrim(filter(2), 2)
           print, '   with a filter range of', filter(0:1),' at y = ',strtrim(i,2)
           xline(ok) = filter(2)
         endif
       endelse
       image[0, i] = xline * scale 
     endfor
     free_lun, unit
     return
     end
