; ------------------------------------------------------------------------ 
; $Id: readalab.pro,v 1.2 2010/12/09 16:45:05 lpmix Exp $
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
 Pro readalab, fnamein, wdfin, narray = narray, wdfarray = wdfarray
;
;+
; NAME:
;       READALAB
; PURPOSE:
;       This procedure reads a LabView ASCI file 
; CALLING SEQUENCE:
;       READALAB, Filename [, WDF]
;       READALAB, WDF
;       READALAB
;
; INPUTS:
;       Filename: Character string containing the filename
;                 If missing DIALOG_PICKFILE will be used to determine the file.
;                 If Filename is present and equal to zero, DIALOG_PICKFILE
;                   will be used to determine file
;       WDF: WDF array number for the first dataset or an array of numbers.
;            Following datasets will be in consecutive WDF arrays.
;            If necessary, SET_MAXWDFARRAY will be used to increase the 
;              number of WDF arrays available.
;            DEFAULT:  WDF = 1
; KEYWORDS:
;       NARRAY:    Number of arrays read
;       WDFARRAY:  First WDF array number or an array of WDF numbers
;                  Ignored if WDF is present
;
; Technique:
;       Read data from file using the ASCII keywords:
;       (Data is tab delimited)
;       Channels:  Number of channels
;       Samples:   Number of samples
;       Date:      Date of data
;       Time:      Time of data
;       X_Dimension: X_Label
;       Y_Unit_Label: Y Label
;       X0:        Initial X value
;       Delta_X:   Point spacing
;    Read Channels Names in Columns, followed by data
;
; MODIFICATION HISTORY:
;       Original version, L. P. Mix, October 28, 2003
; 
;   Example:
;       Read a LabView ascii file to be selected from a widget
;         READALAB
;
;       Read a Sandia LabView binary file to be selected from a widget 
;         and store the waveforms beginning in wdfarray 101
;         READALAB, 101
;-
;
     on_error, 1
common savelab_common, oldfilter, oldpath
;    check format
     nparams = n_params()
     narray = 0L
     if ((n_elements (fnamein) eq 0) or (nparams eq 0)) then begin
        fname = 0
        wdf = (n_elements(wdfarray) gt 0) ? wdfarray : 1L
     endif else begin
        if size(/type, fnamein) eq 7 then begin
          fname= fnamein[0]
          if n_elements(wdfin) gt 0 then wdf = long(wdfin) else $
            wdf =  (n_elements(wdfarray) gt 0) ? wdfarray : 1L
        endif else begin
          fname = 0
          if nparams eq 2 then wdf = long(wdfin) else wdf = long(fnamein)
        endelse
     endelse
     if size(/type, fname) lt 7 then begin
       if n_elements(oldpath) eq 0 then cd, current = path else path = oldpath
       fname = dialog_pickfile(/must_exist, /read, path = path, get_path= newpath)
       if (strlen(fname) lt 1) then begin
         print, '% READALAB: No File was Selected - RETURNING'
         return
       endif
       oldpath = newpath
     endif
;
     vers = strupcase(strmid(!version.os_family, 0, 3))
     if (vers eq 'UNI') then begin
;      get system variable substitution for unix systems
       temp = fname
       FULFNAM, temp, fname
     endif
     fname = file_expand_path(fname)
     if !version.os  eq 'linux' then resolve_it, fname
     info = file_info(fname)
     if (info.exists + info.read) ne 2b then begin
        print, '% READALAB: File does not exist with read permission'
        print, '           File Name = ', fname
        print, '           File Exists =', info.exists
        print, '           Read Permission =', info.read
        return
     endif

     openr, unit, fname, /get_lun, error = ier
     if ier lt 0 then begin
       Print, '% READALAB: Error opening file'
       print, '  Filename: ', fname
       print, !Error_state.msg
       return
     endif

     junkstr = '' 
     narray =0
     finish =0
     while not eof(unit) and (finish eq 0) do begin
       readf, unit, junkstr
       sar = strsplit(junkstr, string(9B), /extract, /preserve)
       key = strupcase(sar[0])
       if strpos(key, 'TIME') ge 0 then begin
         time = strtrim(sar[1], 2)
         ok = strpos(time, '.')
         if ok gt 0 then time = strmid(time, 0, ok)
       endif else if strpos(key, 'DATE') ge 0 then begin
         date = strtrim(sar[1], 2)
       endif else if strpos(key, 'CHANNELS') ge 0 then begin
         narray = long(sar[1])
       endif else if strpos(key, 'SAMPLES') ge 0 then begin
         if narray eq 0 then goto, input_error
         npoint = long(sar[1:narray])
       endif else if strpos(key, 'Y_UNIT_LABEL') ge 0 then begin
         if narray eq 0 then goto, input_error
         ylabel = strtrim(sar[1:narray], 2)
       endif else if strpos(key, 'X_DIMENSION') ge 0 then begin
         if narray eq 0 then goto, input_error
         xlabel = strtrim(sar[1:narray], 2)
       endif else if strpos(key, 'X0') ge 0 then begin
         if narray eq 0 then goto, input_error
         xinit = float(sar[1:narray])
       endif else if strpos(key, 'DELTA_X') ge 0 then begin
         if narray eq 0 then goto, input_error
         delta = float(sar[1:narray])
       endif else if strpos(key, 'END_OF_HEADER') ge 0 then begin
         if narray gt 0 then finish =1
       endif

     endwhile
     if narray eq 0 then goto, input_error
     readf, unit, junkstr
     sar = strsplit(junkstr, string(9B), /extract, /preserve)
     clab = (strpos(strupcase(sar[0]), 'X_VALUE') ge 0) ? $
            strtrim(sar[1:narray],2) :  strtrim(sar[0:narray-1],2)
     minval = min(npoint, max = maxval)
     yarr = make_array(narray, maxval)
     datetime=' - DATETIME='+date+':'+time
     npointm = npoint -1
     if minval eq  maxval then readf, unit, yarr else begin
       readf, unit, yarr[*, 0:minval-1]
       for i = minval, maxval-1 do begin
         readf, unit, junkstr
         sar = strsplit(junkstr, string(9B), /extract, /preserve)
         ok = where (npointm ge i, nok)
         if nok gt 0 then yarr[ok, i]= sar[ok]
       endfor
     endelse
     wdf = (wdf > 1L) 
     nwdf = n_elements(wdf)
     if nwdf ge narray then wdf = wdf [0:narray-1] $
       else wdf = [long(wdf), max(wdf)+1+lindgen(narray-nwdf)]
     nwdf = narray
     mxwdf = max(wdf)
     if mxwdf gt (get_maxwdf()-2) then set_maxwdfarray, mxwdf+32
     yarr = transpose(yarr)
     for i = 0, narray-1 do $
       put_wdfarray, [xinit[i], delta[i], yarr[0:npointm[i], i]], wdf[i], $
       file = fname, xlabel = xlabel[i], ylabel = ylabel[i], clabel = clab[i]+datetime
     return
input_error:
     free_lun, unit
     print, '% READALAB:  Error reading input'
     end
