; ------------------------------------------------------------------------ 
; $Id: readasc.pro,v 1.10 2010/06/10 21:08:14 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
 Pro readasc, fname,nvar,ax,bx,label = name, formatin = formatin, np= np, $
               LIMIT = limit, skip = skip, error = error, endnumber= endnumber, $
               type = typein, double = doublevar, complex = complexvar, $
               dcomplex = dcomplexvar, stringvar = stringvar, matrix = matrix
;
;+
;    read an ascii file
;    format is:  readasc, fname,nvar,x,y, label=label, np=np, /limit,
;                formatin=format, type = type, skip = skip, error = error, 
;                endnumber= endnumber, /double, /complex, /dcomplex
;    where 
;      fname   - the file name or a unit number
;                if a file name then the file is opened and closed at the end
;                if a number then the file is read directly
;                  if unit is equal to zero then pickfile will be used 
;                  to determine a file
;                Default: FNAME = 0
;      nvar    - the number of variables in a record
;                form is x, y1, y2, y3, ... 
;      x       - abscissa values - dimensioned x(np)
;                If keyword, MATRIX, is set, XP will be a matrix with
;                   dimensions [nvar, np].
;      y       - ordinate values - dimensioned y(np, nvar-1)
;                Undefined if MATRIX is set
;      label   - string array of the names of the yi's
;      np      - number of data values read 
;                if np is zero or undefined then the program will attempt 
;                  to read 8192 values
;                otherwise, the program will attempt to read np values
;                in all cases np is returned with the actual number read
;      LIMIT   - If set, the number of values read will be limited to NP
;                Ignored if FORMATIN is 1 or 3
;      formatin - input format of the data
;              0 (DEFAULT) only data values in the file maximum of np values
;              1  number of points followed by data values; this value
;                 overrides the values set by np
;              2  Nvar-1 labels (stored in name) followed by data values
;              3  number of points followed by nvar-1 labels then data values
;      type    - Variable type to be returned
;                Default is 4 (float)
;                Values are:
;                4  Float
;                5  Double precision Float
;                6  Complex
;                7  String
;                9  Double precision Complex
;                If type is not equal to one of these values then type = 4
;      DOUBLE  - If set, type = 5
;      COMPLEX - If set, type = 6
;      DCOMPLEX- If set, type = 9
;      STRINGVAR - If set, type = 7
;      Note:  Double, complex and dcomplex keywords are checked after type
;             and will change any previous values.
;      skip    - If the file has several lines of header information which
;                must be skipped, set skip equal to this number of lines.
;                (DEFAULT: SKIP = 0)
;            Note: In general blank lines in the data are skipped.
;      error   - If no errors occur, error = 0
;                Not zero if an open error or no data read.
;      endnumber - If set, reading stops when this X-value is reached.
;                If two or more values, second value must also be ENDNUMBER
;                That is both X and Y[0], if present, must be equal to ENDNUMBER
;                NOTE:  Not allowed if type eq 7
;      MATRIX  - If set, return a matrix with dimensions [nvar, np]
;
;    Read an array from unit 2 and store in array x. Assume format = 1
;       READASC, 2, 1, x, format = 1
;    Read 2 arrays of data from a file selected by pickfile and store it in
;    arrays x and y.  Assume format = 2: label followed by data to EOF.
;       READASC, 0, 2, x,y , format= 2, lab= lab
;    Read 5 arrays from file 'PFIDL.dat', into arrays a and b and label = lab
;    Assume data format 3.
;       READASC, 'PFIDL.dat', 5,  a, b, form = 3, label = lab
;    Read 4 arrays from file, 'testdata', into arrays R and S. Assume 
;    data has no header but only 4 columns of data.
;       READASC, 'testdata', 4, R, S, form = 0
;-
;
     on_error, 1
     error = 0
     nparams = n_params()
      if nparams eq 0 then begin
        print, '% Format: READASC, FNAME_UNIT, NVAR, X [, Y] [, NP = np]'
        print, '           [, LABEL = label] [FORMATIN = format][, TYPE = type]'
        print, '           [, /DOUBLE] [, /COMPLEX] [, /DCOMPLEX] [, /STRINGVAR]'
        print, '           [, ENDNUMBER = endnumber] [, SKIP = skip]'
        return
      endif
;    check format
      if n_elements (formatin) eq 0 then format = 0 else format = fix(formatin)
      if format lt 0 or format gt 3 then format = 0
      if nparams lt 3 then begin
        print, '% READASC: 3 parameters are required: Filename, NVAR, X'
        return
      endif
      fname = (n_elements(fname) eq 0) ? 0 : fname[0]
      if n_elements (nvar) eq 0 then message, $
        '% READASC: Number of variables must be specified'
      b = size(fname)
      if n_elements(typein) eq 0 then type = 4 else begin
        type = typein[0]
        ok = where([4, 5, 6, 9] eq type, nok)
        if nok eq 0 then type =4
      endelse
      if keyword_set(doublevar) then type = 5
      if keyword_set(complexvar) then type = 6
      if keyword_set(dcomplexvar) then type = 9
      if keyword_set(stringvar) then type = 7
      ax = 0
      by = 0
      if b(1) eq 7 then begin
        info = file_info(fname)
        if (info.exists + info.read) ne 2b then begin
           print, '% READASC: File does not exist with read permission'
           print, '           File Name = ', fname
           print, '           File Exists =', info.exists
           print, '           Read Permission =', info.read
           return
        endif
        openr,unit,fname, /get_lun, error = error
        if error ne 0 then begin
          print, '% READASC: Unable to open file, Error=', error
          return
        endif
        lcfile = 1
      endif else begin
        unit = fname
        if unit eq 0 then begin
          fname = dialog_pickfile(/must_exist, /read)
          if strlen(fname) lt 1 then begin
            print, '% READASC: No File was Selected - RETURNING'
            return
          endif
          lcfile = 1
          openr,unit,fname, /get_lun, error = error
          if error ne 0 then begin
            print, '% READASC: Unable to open file, Error=', error
            return
          endif
        endif else lcfile = 0
      endelse
      if n_elements (np) eq 0 then np = 0
      if np le 0 then np = 8192L
      maxnp = long(np)
      junkstr = '' 
      if n_elements (skip) eq 0 then skip = 0
      if n_elements (endnumber) gt 0 then begin
        if type eq 7 then begin
print,'% READASC: Present version will not allow STRING type with endnumbers'
          return
        endif
        lcend = 1
        endnumber = endnumber[0]
      endif else lcend = 0
      on_ioerror, noskipdata
      if skip gt 0 then for i = 1, skip do readf, unit, junkstr
      if format eq 1 or format eq 3 then readf, unit, maxnp
      ax =make_array(maxnp, type  = type)
; Time to start reading
      if nvar eq 1 then begin
        i=0L
        x = make_array(1, type = type)
        while (not eof(unit)) and i lt maxnp do begin
          on_ioerror, baddata
          readf,unit,x
          if keyword_set(lcend) then if (x eq endnumber) then goto, baddata
          ax(i) = x
          i = i+1
          if (i eq maxnp) && (~ keyword_set(limit)) && $
            ((format and 1) eq 0) then begin
            maxnp = maxnp+maxnp
            ax = [ax, ax]
          endif
        endwhile
baddata:
      endif else begin
        name = make_array (nvar-1, /string)
        if format  gt 1 then begin
          for i=0, nvar-2 do begin
            string = ''
            readf, unit, string
            name(i) = string
          endfor
        endif
        if keyword_set (matrix) or (type eq 7) then begin
          ax = make_array ( nvar,maxnp, type = type)
          x = (type eq 7) ? '' : make_array (nvar, type = type)
          i=0L
          while ((not eof(unit)) and i lt maxnp) do begin
            on_ioerror, badmatrix
            readf,unit,x
            if keyword_set(lcend) then if (x eq endnumber) then goto, badmatrix
            if type eq 7 then begin
              val = strsplit(x,  /extract, regex=0, count = nel)
              if nel ne nvar then begin
                 print, '% READASC: String elements missing.', nel, nvar
                 print, val 
                 nel = nel < nvar
              endif
              if nel gt 0 then ax(*, i) = val[0:nel-1] else i = i-1
            endif else ax(*, i) = x
            i = i+1
            if (i eq maxnp) && (~ keyword_set(limit)) && $
               ((format and 1) eq 0) then begin
              ax = reform (ax, maxnp*nvar, /overwrite)
              ax = [ax, ax]
              maxnp = maxnp+maxnp
              ax = reform (ax, nvar, maxnp, /overwrite)
            endif
          endwhile
badmatrix:
          if i gt 0 then ax = ax[*, 0:i-1]
          if not keyword_set(matrix) then begin
            tmp = temporary(ax)
            ax = reform(tmp[0, *])
            bx = reform(tmp[1:*, *])
            tmp =0
          endif 
        endif else begin
          bx = make_array ( nvar-1,maxnp, type = type)
          x = make_array(1, type = type)
          y = make_array (nvar-1, type = type)
          i=0L
          while ((not eof(unit)) and i lt maxnp) do begin
            on_ioerror, badeof
            readf,unit,x, y
            if keyword_set(lcend) then if ((x eq endnumber) and (y[0] eq endnumber)) $
               then goto, badeof
            ax(i) = x[0]
            bx(*, i) = y
            i = i+1
            if (i eq maxnp) && (~ keyword_set(limit)) && $
               ((format and 1) eq 0) then begin
              ax = [ax, ax]
              bx = reform (bx, maxnp*(nvar-1), /overwrite)
              bx = [bx, bx]
              maxnp = maxnp+maxnp
              bx = reform (bx, nvar-1, maxnp, /overwrite)
            endif
          endwhile
        endelse
      endelse
badeof:
      if lcfile eq 1 then  free_lun, unit
      if i eq 0 then begin
        np = 0
        error = -1
        print, '% READASC: Error reading file -- No data found'
        ax = 0
        bx = 0
        return
      endif
      np = i
      if keyword_set(matrix) then return
      ax= ax(0:i-1)
      if nvar gt 2 then begin
        bx = reform(bx(*, 0:i-1))
        if nvar gt 2 then bx = transpose(bx)
      endif else if nvar gt 1 then bx = bx[0:i-1]
      return
noskipdata:
      print, '% READASC: Unable to read skip lines -- probably end of file'
      return
      end
