; ------------------------------------------------------------------------
; $Id: readdita.pro,v 1.2 2007/06/06 20:03:51 lpmix Exp $
; ------------------------------------------------------------------------
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
;+
; NAME:
;	READDITA
;
; PURPOSE:
;
;       This procedure reads a DITABIS format file for the IP plate data and outputs a
;       psl (photo stimulable level) dose equivelent file.  The data for the
;       conversion and the image parameters are read from the Fuji .PCB or .INF
;       header file.  The image data itself must be in a .IMG file of the same 
;       name as the header file.
;
; CALLING SEQUENCE:
;
;       READDITA, STRUCTURE, [, FILE] [, /WINDOW_IMAGE]
;           [/STRUCTURE] [, /IMAGE] [, /WRITETIFF] [, /REVERSEimage]
;           [, CLABEL = clabel] [, XLABEL = xlabel] [, YLABEL = ylabel]
;           [, TLABEL = tlabel] [, SWAPENDIAN = swapendian] [, /VERBOSE]
;           [, /KEEP_AXIS] [, XRANGE = xrange] [, YRANGE = yrange]
;           [, HEADER = header]
;
; INPUTS:
;       STRUCTURE  - Structure Array Number or variable if STRUCTURE 
;                    or IMAGE is set.
;       FILE       - DITABIS file name
;                    If zero of undefined, DIALOG_PICKFILE will be used.
;
; KEYWORD PARAMETERS:
;       WINDOW_IMAGE - If set, Ditabis image is windowed before it is stored in a
;                    structure and returned.  
;                    Default:  WINDOW_IMAGE = (N_ELEMENTS(IMAGE) gt 4e7) ? 1 : 0
;       KEEP_AXIS  - If set keep the original coordinates for the image.
;                    DEFAULT:  Set initial X and Y values to zero
;       XRANGE     - Minimum and Maximum X values for the windowed image.
;       YRANGE     - Minimum and Maximum Y values for the windowed image.
; 
;       STRUCTURE  - If set and if image is not set, the structure is returned in
;                    the variable STRUCTURE
;       IMAGE      - If set, the DITABIS image is returned in the variable STRUCTURE
;       WRITETIFF  - If set, DITABIS is written to a tiff file.
;                    If WRITETIFF is a string, WRITETIFF will be the name of the
;                      TIFF file.
;                    Default: TIFF_FILE = FILE(without extension)+'.tif'
;       REVERSEimage - If not specified, image will be reversed.
;                    IDL has a convention from bottom to top; TIFF stores 
;                    image top to bottom.  Image will be reversed unless REVERSE
;                    DEFAULT:  REVERSE = 1
;             Note:  WRITETIFF will write the original image regardless of this
;                    keyword value.
;       SWAPENDIAN- If set, this keyword will determine if bytes are swapped.
;                    Default:
;                      BYTE_ORDER, IMAGE, /SWAP_IF_LITTLE_ENDIAN
;       CLABEL     - Dataset comment for the structure
;                    DEFAULT: CLABEL = FILE
;       XLABEL     - Abscissa comment for the structure
;                    DEFAULT: XLABEL = 'X'
;       YLABEL     - Ordinate comment for the structure
;                    DEFAULT: YLABEL = 'Y'
;       TLABEL     - Type label for the structure
;                    DEFAULT: TLABEL = 'DITABIS Image'
;       VERBOSE    - Print the header information 
;                    Default: VERBOSE = 1
;       HEADER     - If ARG_PRESENT(HEADER) the header information will be 
;                    returned in the variable, HEADING.
;                    If HEADING is set, ARG_PRESENT is false, the header
;                    information will be added to the comment label.  The 
;                    header will be separated by ' - ' so that truncate will 
;                    eliminate this information from plots.  
;                      USE:  SHELP, STR, /Comment to see the comment
; OUTPUTS:
;       This procedure returns the dose equivelent file from the DITABIS scanner
;       data in a PFIDL structure or an array.  This array may be written to a 
;       TIFF file.
;
; PROCEDURE:
;       Read the header information at the beginning of the file to get image size.
;       Read the image into an unsigned integer, scale the image, and store in a
;         PFIDL structure. 
;       Window the image if it is large.
;       The scaling information is from the header:
;          image = image/gain
;          image = ((xresoluion)^2) * (4000/Sensitivity) * $
;                  (10^(Latitude*(float(image)/gradLevels-0.5))
;
; EXAMPLE:
;       Read shot_670.img into structure 1.
;         READDITA, 1, 'shot_670.img'
;
;       Read an image to be selected into structure 2.
;         READDITA, 2
;
; MODIFICATION HISTORY:
;       Original Version:  L. Paul Mix     April 16, 2005
;       DITABIS format information added   
;                          
;-


PRO readdita, STR, FILE, CLABEL = clabel, verbose = verbose, $
       swapendian = swapendian, WINDOW_IMAGE = window_image, $
       XLABEL = xlabel, YLABEL = ylabel, TLABEL = tlabel, IMAGE= image, $
       STRUCTURE = structure, REVERSEimage = reverseimag, WRITETIFF = writetiff, $
       KEEP_AXIS = keep_axis, xrange = xrange, yrange = yrange, header = header
common saveditapicker, oldfilter, oldpath
;
on_error, 1
;
      Big_image = 4.0e7 ; 160e6 bytes
      nparams = n_params()
      if nparams eq  0 then begin
        print, '% Format:', $
           '  READDITA, STRUCTURE  [, FILE] [, /WRITETIFF] [, /WINDOW_IMAGE]'
        print, '   [, SWAPENDIAN = swapendian] [, REVERSEimage = reverseimage]'
        print, '   [, XLABEL = xlabel] [, YLABEL = ylabel] [, TLABEL = tlabel]'
        print, '   [, CLABEL = clabel] [, /IMAGE] [, /STRUCTURE]'
        print, '   [, KEEP_AXIS = keep_ axis] [, XRANGE = xrange] [, YRANGE = yrange]'
        return
      endif
      if not keyword_set(image) and not keyword_set(structure) then begin
        if n_elements(str) eq 0 then begin
          print, '% READDITA: Structure array number must be defined'
          return
        endif
        strsave = get_save_str(str, get_maxstr())
      endif
      if n_elements(reverseimag) eq 0 then reverseimag = 1
      if n_elements(file) eq  0 then fname = 0 else fname = strtrim(file[0], 2)
      b = size(fname)
      if b(1) ne 7 then begin
        if n_elements(oldpath) eq 0 then cd, current = path else path = oldpath
        p1 = dialog_pickfile(/must_exist, /read, path = path, $
          filter = ['*.ip*;*.IP*', '*.*'], get_path = oldpath)
        if strlen(p1) lt 1 then begin
          print, '% READDITA: No File was Selected - RETURNING'
          return
        endif
        fname = strtrim(temporary(p1), 2)
      endif
      resolve_it, fname
      info = file_info(fname)
      if (info.exists + info.read) ne 2b then begin
         print, '% READDITA: File does not exist with read permission'
         print, '         File Name = ', fname
         print, '         File Exists =', info.exists
         print, '         Read Permission =', info.read
         ier = -2
         return
      endif
      ii = strpos(fname, '.', /reverse_search)
      if ii gt 1 then fnamet = strmid(fname, 0, ii) else begin
        Print, '% READDITA: File must have an extension'
        return
      endelse
      tiff_file = fnamet+'.tif'
      if n_elements(verbose) eq 0 then verbose = 1
;
      if keyword_set(verbose) then print, 'Processing Header: ', fname

      openr, unit, fname, /get_lun
      head = bytarr(2048)
      readu, unit, head
      ok = where(head le 31, nok)
      ind = 0
      if  (arg_present(header)) then begin
        lc_header = 1
        lc_clab = 0
      endif  else begin
        lc_header = 0
        lc_clab = (keyword_set(header)) ? 1:0
      endelse
      header = make_array(256, /string)
      ih = 0
      st=0
      while ind lt nok do begin
        if ok[ind] gt st+1 then begin
          header[ih]= strtrim(head[st:ok[ind]-1], 2)
          ih +=1
        endif
        st = ok[ind]+1
        ind += 1
      endwhile
      header = header[0:ih-1]
      yscale = 0.0
      xscale = 0.0
      nx = 0L
      ny = 0L
      byte_len= 0
      head_len = 0
      gain = 0
      if keyword_set(verbose) then for i = 0, ih-1 do print, header[i]
      LINES = ['XRESOLUTION', 'YRESOLUTION', 'XPIXEL', 'YPIXEL', 'BYTE PER PIXEL', $
              'HEADER', 'GAIN']
      Variable = ['yscale', 'xscale', 'ny', 'nx', 'byte_len', 'head_len', 'gain']
      nvar = n_elements(variable)
      for i = 0, nvar-1 do begin
        ok = get_match('^'+lines[i], header, regexp=0, nel = nel)
        if nel eq 0 or nel gt 1 then begin
          print, '% READDITA: Problem finding '+lines[i]
          return
        endif
        if i lt 2 then begin
          j = (strpos(header[ok], '('))[0]
          je = (strpos(header[ok], ')'))[0]
          If (j gt 0) and (je gt 0) then $
            svar = strmid(header[ok], j+1, je-j-1) else begin
            print, '% READDITA: Problem finding "(" or ")" with '+lines[i]
            return
          endelse
        endif else begin
          j = strpos(header[ok], '=')
          if j gt 0 then svar = strmid(header[ok], j+1) else begin
            print, '% READDITA: Problem finding "=" with '+lines[i]
            return
          endelse
        endelse
        a = execute('reads, svar, '+variable[i])
      endfor
      if min([yscale, xscale, ny, nx, byte_len, head_len, gain]) le 0 then begin
        Print,  '% READDITA: One or more parameters has not been provided.'
        for i = 0, nvar-1 do junk= execute('lines[i]=strtrim('+variable[i]+', 2)')
        for i = 0, nvar-1 do print, i+1,'. ', variable[i]+' = '+lines[i]
        return
      endif
      if keyword_set(verbose) then begin
        for i = 0, nvar-1 do junk= execute('lines[i]=strtrim('+variable[i]+', 2)')
        for i = 0, nvar-1 do print, i+1,'. ', variable[i]+' = '+lines[i]
      endif
      if head_len ne 2048 then begin
        print, '% READDITA: Current version set for 2048'
        print, '            Contact Paul Mix 845-7493'
        return
      endif
      
      if byte_len eq 2 then siz = 12 else if byte_len eq 4 then siz = 13 else begin
        print, '% READDITA: Unexpected byte length: ', strtrim(byte_len, 2)
        return
      endelse
      str = make_array(nx, ny, type = siz)
      readu, unit, str
      free_lun, unit
      if keyword_set(verbose) then print, 'Processing Image: ', fname
      if n_elements(swapendian) eq 0 then byteorder,str, /swap_if_big_endian $
         else if keyword_set(swapendian) then byteorder, str
      if n_elements(reverseimage) eq 0 then reverseimage= 1
      if n_elements(window_image) eq 0  then $
        lc_wi = (nx*ny gt big_image) ? 1:0 else lc_wi = keyword_set(window_image)
      if lc_wi then begin
        nxw = nx/10
        nyw = ny/10
        nxwa = 10L*nxw
        nywa = 10L*nyw
        arrs =  rebin(smooth(str[0:nxwa-1, 0:nywa-1], 11), /sample, nxw, nyw)
        xar =  5 +10*findgen(nxw)
        yar =  5 +10*findgen(nyw)
        smax = get_maxstr()
        arr2str, smax, float(arrs), xar, yar, dx= 10, dy = 10
        winstr, smax, save = smax-1 
        strinfo, smax-1, xrange = xrange, yrange = yrange
        xi = (xrange[0]-10) > 0L
        yi = (yrange[0]-10) > 0L
        xj = (xrange[1]+10) < (nx-1)
        yj = (yrange[1]+10) < (ny-1)
        str = str[xi:xj, yi:yj]
      endif else begin
        xi = 0L
        yi = 0L
      endelse


;     Scale the data
      str = float(str)

      if keyword_set(writetiff) then begin
         if size(/type, writetiff) eq 7 then tiff_file = writetiff
         WRITE_TIFF, tiff_file, str, /float
      endif

      if keyword_set(reverseimag) then begin
        ss = size(str)
        str= reverse(str, ss[0], /overwrite)
      endif

;     If only image is requested then return
      if keyword_set(image) then return
      s = size(str)
      nx = s[1]
      ny = s[2]
      ndim = 2
;     go from microns to mm
      dx = xscale* 0.001
      dy = yscale* 0.001
      if keyword_set(keep_axis) then begin
        x = (Xi +findgen(nx))*dx
        y = (Yi +findgen(ny))*dy
      endif else begin
        x = findgen(nx)*dx
        y = findgen(ny)*dy
      endelse
      if n_elements(clabel) eq 0 then clabel = fname
      if keyword_set(lc_clab) then begin
        clabel = clabel +' - '+header[0]
        for i = 1, ih-1 do clabel = clabel+';;'+header[i]
      endif 
      if n_elements(xlabel) eq 0 then xlabel = 'X (mm)'
      if n_elements(ylabel) eq 0 then ylabel = 'Y (mm)'
      if n_elements(tlabel) eq 0 then tlabel = 'DITABIS Image'
      arr2str, str, /str, str, x, y, dx = dx, dy= dy, clab = clabel, $
         XLABEL = xlabel, YLABEL = ylabel, TLABEL = tlabel, file = fname
      if keyword_set(structure) then return
      i2s, str, strsave, /no_copy
      str = strsave
end
