; ------------------------------------------------------------------------
; $Id: readepson.pro,v 1.1 2011/05/19 13:36:53 lpmix Exp $
; ------------------------------------------------------------------------
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
;+
; NAME:
;       READEPSON
;
; PURPOSE:
;       This routine reads an Epson tiff file and converts it to a PFIDL structure.
;       The spatial dimensions are added if the scan rate is known.
;
; CATEGORY:
;       Data input
;
; CALLING SEQUENCE:
;
;       READEPSON, STR, File
;
; INPUTS:
;       STR:    Structure array number for the Epson image or a structure containing
;               the Epson scanner image.  If STR is undefined or not valid, then 
;               GET_MAXSTR()-1 will be used.
;               If keyword STRUCTURE is set and STR is not a valid argument, then 
;               GET_MAXSTR()-1 will be used.
;       File:   Name of the TIFF file to be read
;               File_Name may also be a search string if it is unique.
;               Dialog_Pickfile is called if no file can be determined.
;
; KEYWORD PARAMETERS:
;       SCAN_Resolution:  Scan resolution for the image.
;               If scan_resolution equals 1 then image will be in pixels.
;               Typical values are 9600 DPI/PPI (Dots per inch, Pixels/inch.
;       MMETERS: If set, save image in millimeters.
;               Default: If MICRONS is not set, save the image in millimeters.
;       MICRONS: If set, save image in microns.
;               If microns is set, save raw image in microns using the delta values
;               provided by the SCAN_Resolution.  MMETERS is ignored.
;       REVERSEIMAGE: If not set to zero, image will be reversed.
;               IDL has a convention from bottom to top; TIFF stores 
;               image top to bottom.  Image will be reversed unless REVERSE
;               is set to zero (ie. REVERSE = 0)
;               DEFAULT:  REVERSE = 1
;       STRUCTURE: If set return the image structure in STR.  If STR is not a
;               a valid argument, the data will be returned in structure array
;               number = STR or in GET_MAXSTR()-1
;       SCALE:  If set, scale the image by scale.
;       FILENAME: If present, this keyword returns the name of the file actually read
;       IERROR: Returns ZERO if no errors.
;
; OUTPUTS:
;       The Epson image is stored in the structure array or structure indicated
;       by the first parameter, STR.
;
; PROCEDURE:
;       READTIFF is called to read the data and the appropriate scaling is
;       applied to the axis.
;
; EXAMPLE:
;       Read the data from an Epson scanner file, 'mydata_4800_3.tif', and 
;       store it in structure array 6
;         READEPSON, 6, 'mydata_4800_3.tif'
;
; MODIFICATION HISTORY:
; 	Written by:	L. P. Mix, Jr., February 22, 2011
;
;-

PRO READEPSON, str, file, scan_resolution = scan_resolution, mmeters =mmeters, $
       microns = microns, reverseimage = reverseimage, structure = structure, $
       scale = scale, filename = filename, ierror = ierror
on_error, 1
  ierror = 1
  nparams = n_params()
  if nparams eq  0 then begin
    print, '% Format: ' + $
     'READEPSON, STR, FILENAME, SCAN_RESOLUTION = scan_resolution,'
    print, '           MMETERS =mmeters, MICRONS = microns, SCALE = scale,'
    print, '           REVERSEIMAGE = reverseimage, /STRUCTURE' 
    return
  endif
  if keyword_set(structure) then lcstr = (arg_present(str)) ? 1 : 0 else lcstr =0
  if lcstr eq 0 then begin
    if n_elements(str) eq 0 then str = get_maxstr()-1 else $
      if ((str le 0) or (str gt get_maxstr())) then str = get_maxstr()-1
  endif
; read the tiff file
  readtiff, tiffstr, file, /structure, filename = filename
  if size(/type, tiffstr) ne 8 then begin
    print, '% READEPSON:  Unable to read TIFF file'
    return
  endif
  print, 'Epson file read: '+filename
  if n_elements (scan_resolution) eq 0 then read, scan_resolution, $
    prompt='Enter the scan resolution: '
  if scan_resolution eq 0 then scan_resolution = 1.0

  dx1 = 25.4/scan_resolution
  if keyword_set (microns) then dx1 *=1e3
  tiffstr.x1lab = keyword_set(microns) ? 'X distance (microns)' : 'X distance (mm)'
  tiffstr.x2lab = keyword_set(microns) ? 'Y distance (microns)' : 'Y distance (mm)'
  tiffstr.dx1 *= dx1
  tiffstr.dx2 *= dx1
  tiffstr.x1 *= dx1
  tiffstr.x2 *= dx1
  tiffstr.range *= dx1
  if n_elements(scale) gt 0 then tiffstr.v1 *= scale
  if lcstr then str = temporary(tiffstr) else i2s, tiffstr, str, /no_copy
  ierror = 0
END
