; ------------------------------------------------------------------------ 
; $Id: readhist.pro,v 1.13 2010/12/09 17:57:30 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
 pro readhist, wdfin, dataset, alldata = alldata, narray = narray, mdt = mdt, $
         ndataset = ndataset, filename = p1, ignore = ignore, helpme = helpme, $
         verbose = verbose, strict = strict, quiet = quiet
;+
; NAME:
;       READHIST
;
; PURPOSE:
;       Read a unformatted history file from QUICKSILVER
;
; CALLING SEQUENCE:
;
;       READHIST, WDFARAY [, DATASET] [, /ALLDATA]
;                [, NDATASET = ndataset ] [, NARRAY = narray]
;                [, FILENAME = filename] [, MDT = mdt] [, /HELPME] 
;
; OPTIONAL INPUTS:
;       WDFARRAY  - WDF array number or an array of numbers.
;                   Default = 1
;                   If more arrays are selected than contained in WDFARRAY,
;                   then sequential numbers will be used.
;                   If WDFARRAY is an array then MAXWDFARRAY will not be
;                   increased so some arrays could remain unsaved if a
;                   sufficient number of arrays are not available.  
;                   If WDFARRAY is a scalar or undefined, MAXWDFARRAY will
;                   be increased to insure that all arrays are read.
;                   If increased, MAXWDFARRAY will be an even number at least
;                   two larger than necessary.
;
;       DATASET   - Dataset number or an array of dataset numbers 
;                or Search string or an array of search strings.  Result of the
;                   search strings will be sorted each dataset read only once.
;                   If dataset is missing and ALLDATA is not set then datasets
;                   are selected from a widget list.  GET_MATCH is used with
;                   default settings for string values.
;                   If DATASET = '' or '*' then ALLDATA will be set.
;       STRICT    - If set, this applies only to string dataset values:
;                   For string data:
;                     If there is more than one dataset for each string, a warning
;                     will be printed.
;                     If there is less than one dataset for each string, a warning 
;                     will be printed.
;                   DEFAULT:  STRICT = (size(/type, DATASET) eq 7) ? 1 : 0
;       ALLDATA   - If set, the entire file will be stored. DATASET is ignored.
;       NDATASET  - Total number of datasets in the file
;       NARRAY    - Number of datasets read into wdf arrays
;       IGNORE    - READHIST expects an integer number of 512 byte records
;                   If IGNORE is set, READHIST truncates the data to an integer
;                   number of records.
;                   Default: IGNORE =1
;       FILENAME  - Name of the file.  Default: FILENAME ='qcks.his'
;                   If qcks.hist is missing or filename =0, then 
;                     DIALOG_PICKFILE will be used.
;       MDT       - Optional time scale factor.  The time or abscissa values
;                   are multiplied by this number. Default: MDT =1
;       QUIET     - If set, do not print informational messages.
;       HELPME    - If set, print a help message.
;
; OUTPUTS:
;       The selected WDF arrays will be stored.
;
; RESTRICTIONS:
;       This will read files generated on the Intel and UNIX machines but
;       has not be verified on MAC's and DEC machines, but should also work
;       there.
;
; PROCEDURE:
;      1. Open the file and determine if byte swapping is required, the total
;         length of the file, and the total length of the header.
;      2. Close and reopen and read the string portion of the header.
;      3. Close and reopen the file and read the data.
;      4. Determine the desired datasets.
;      5. Store the data.
;
;     Single precision file format:
;           Word 0:                    Fortran Record Length
;           Word 1:                    NHISTW  (# of history signals)
;           Word 2:                    TIMSTP  (simulation time step)
;           Words 3-5:                 SKPHIS(1-3)  (history do-list)
;                     (t0 = SKPHIS(1)*TIMSTP, dt = SKPHIS(3)*TIMSTP)
;           Word 6:                    PLNAM  (# of characters in title)
;           Words 7-(6+NHISTW*PLNAM)   ASCII format of NAMHIS(1-NHISTW)
;           Additional    Floating point data to the first -1
;      Double precision file format:
;           Word 0:                    Fortran Record Length
;           Word 1a:                   Negative number flag for real*8
;           Word 1b:                   NHISTW  (# of history signals)
;           Word 2a, 2b:               TIMSTP  (simulation time step)
;                                      (real*8)
;           Words 3 to end are unchanged, but the floats are real*8
;
; EXAMPLE:
;       Read the data from qcks.his and store in WDF arrays beginning at 4
;          READHIST, 4, /all
;
;       Read the data from a file determine from a GUI and store in arrays 
;       beginning in WDF array 1.
;          READHIST
;
;       Read the data from a file, picked from DIALOG_PICKFILE, with the 
;       string 'current' and '^v' and store the arrays beginning in WDF array 1.
;          READHIST, 1, ['current', '^v'], file =0
;
; MODIFICATION HISTORY:
;       Written by:     LPMix, October 1, 1998
;       Modified to provide double precision: November 28, 2005
;-
on_error, 1
      common pffcom, initpff, pff_exe
common savehist_picker, oldfilter, oldpath
@com.dat

;     see if we are initialized
      if n_elements(initwdf) eq 0 then wdfinit

      if (n_elements(initpff) eq 0) then init_pff else $
        if (initpff eq 0) then init_pff
      if initpff eq 0 then begin
        print, 'PFF libraries are not supported by this Operating System'
        return
      endif
;
;     Fortran record length, recl_f, and the number of bytes, recl_fb
      RECL_F = 512L
      RECL_FB = RECL_F*4L
;     UNIX or C record length, recl_u, and the number of bytes, recl_ub
      RECL_U = RECL_F+2L
      RECL_UB = RECL_U*4L
      narray = 0

      if keyword_set(helpme) then begin
        print, '% Format: READHIST, WDFARAY [, DATASET] [, /ALLDATA]'
        print, '               [, NARRAY = narray] [, FILENAME = filename]'
        print, '               [, MDT = mdt] [, /HELPME]'
        return
      endif

      if n_elements(initwdf) eq 0 then wdfinit
      npar = n_params()
      if n_elements (wdfin) eq 0 then wdf = 1 else wdf = wdfin
      vers = strupcase(strmid(!version.os_family, 0, 3))

      if n_elements (p1) eq 0 then begin
        p1 = file_search('qcks.his', count = c)
        if c gt 0 then p1 = p1[0]
      endif else begin
        file = p1[0]
        if size(file, /type) eq 7 then begin
          if (vers eq 'UNI') then begin
;           get system variable substitution for unix systems
            temp = p1
            FULFNAM, temp, p1
          endif
          p1 = file_search(file, count = c)

          if c eq 0 then $
            print, '% READHIST: Unable to locate file: ', strtrim(file, 2) $
          else p1 = p1[0]
        endif else c=0
      endelse
      if c eq 0 then p1 ='' else p1 = strtrim(p1(0), 2)
      if (strlen(p1) lt 1) then begin
        if n_elements(oldpath) eq 0 then cd, current = path else path = oldpath 
        p1 = dialog_pickfile(/must_exist, title= 'Select History file to read', $
             path = path, get_path = newpath)
        p1 = strtrim(p1, 2)
        if strlen(p1) lt 1 then return
        oldpath=newpath
      endif

      if (vers eq 'UNI') then begin
;       get system variable substitution for unix systems
        temp = p1
        FULFNAM, temp, p1
      endif
      p1 = file_expand_path(p1)
      if !version.os eq 'linux' then resolve_it, p1
;
      openr, unit, /get, p1, error = error

      if error ne 0 then return
      if not keyword_set(quiet) then print, '% READHIST: Reading from file ',p1
;
;     read the record length and the number of datasets/double precision flag
      val = make_array(7, /long)
      readu, unit, val
      if val[0] ne recl_fb then begin
         orig = val[0]
         swap = 1
         byteorder, val, /lswap
      endif else swap = 0
      if val[0] ne recl_fb then begin
        Print, '% READHIST: This is not a valid history file:'
        print,   'Original record:', orig , ', Swapped:', val(0)
        free_lun, unit
        return
      endif
      if val[1] lt 0 then begin
        dp_flag= 1
        len_head = 8
        ndset = val[2]
        ndataset = ndset
      endif else begin
        dp_flag= 0
        len_head = 6
        ndset = val[1]
        ndataset = ndset
      endelse

;     Get the total length of the file in bytes
      totalstr = fstat(unit)
      totalsize = totalstr.size

;     Number of records = recl
      recl = totalsize/recl_ub

      if n_elements (ignore) eq 0 then ignore =1
      if (recl*recl_ub ne totalsize) then begin
          Print,'% READHIST: File does not contain an integer number of records'
          print,'     ', totalsize, recl, recl_ub, double(totalsize)/recl_ub
          if (not keyword_set(ignore)) then return
      endif
      nwords = totalsize/4-7
      val = make_array(nwords, /long)
      readu, unit, val
;     we only need to look in the last record
      if recl lt 2 then begin
        ok = where(val eq -1, nok)+6
      endif else ok = where (val(nwords-recl_u+1:*) eq -1L, nok)
;     ok(0) is relative to the last data ie ok(0) = 0 means that the 
;     entire last record is -1's
      if nok eq 0 then begin
        lccheck =1
        ok = recl_f
      endif else ok = ok(0)
      free_lun, unit

;     Reopen with the correct swap set and read the dataset labels
;     Use POINT_LUN if we need to go back to the beginning.
      openr, unit, /get, p1, swap_end= swap 

;     Read record length, number of datasets, and perhaps dp_flag
      val = make_array(2+dp_flag, /long)
;     Simulation time step
      timstp = (dp_flag) ? 0.0d0 : 0.0
;     T0 multiplier (number of time steps for the initial time)
      t0 = 0L
;     i is a dummy  (number of time steps for end - generally unknown or wrong)
      i = 0L
;     DT multiplier (number of time steps between outputs)
      dt = 0L
;     Number of characters/comment label
      nstr =0L
      readu, unit, val, timstp, t0, i, dt, nstr
;
;     Reset file to the beginning.
      point_lun, unit, 0
;
;     Convert to real time by scaling to the simulation time step
      t0 = t0*timstp
      dt = dt*timstp
      if (n_elements (mdt)gt 0) && (mdt ne 0.0) then begin
        t0 = t0*mdt
        dt = dt*mdt
      endif
;
      stuff = make_array(recl_u, /long)
      labels = make_array(ndset, /string)
      imax = nstr*ndset
      lab = make_array(imax, /byte)
      i = 0
      j = len_head
      while i lt imax  do begin
        readu, unit, stuff
        m = recl_f -j
        if (m+i) gt imax  then  m = imax-i 
        last = j+m
        lab(i:i+m-1) = byte(stuff[j+1:last])
        j = 0
        i = i+m
      endwhile
      head = make_array(last, /long)
      f = make_array((recl_f-last)>1, /float)
      
      for i = 0, ndset -1 do $
        labels(i)=strtrim((lab(i*nstr:i*nstr+nstr-1)), 2)

      if keyword_set (verbose) then begin
        print, '   Datasets'
        for i = 0, ndset -1 do print, i+1, '.  ',labels(i)
      endif
;     Reset file to the beginning.
      point_lun, unit, 0
;     get the size of the header
      h_length = len_head+nstr*ndset
      start = h_length mod RECL_F 
      hrec = h_length/RECL_F 
      if hrec gt 0 then begin
        junk = make_array(RECL_U , /long)
        for i = 1, hrec do readu, unit, junk
      endif
      tnp = (recl-1)*recl_f+ok-h_length
      np = tnp/ndset
      if np eq 0 then begin
         print, '% READHIST: File contains no data. Total Points:', tnp
         print, '     File contains the following datasets:'
         for i = 0, ndset-1 do print, i+1, '.  ', labels(i)
         return
      endif
      if keyword_set(lccheck) then tnp  = np*ndset else $
      if np*ndset gt tnp then begin
        print, '% READHIST: Problem with the number of points to read:'
        print, '      ',  tnp, np, ndset, nstr, ndset, ok
        free_lun, unit
        return
      endif
      if keyword_set(dp_flag) then np = np/2
      tnp = np*ndset
      data  = make_array(tnp, double = dp_flag)
      junk = make_array(start+1 , /long)
      n = RECL_F-start
      if hrec+1 eq recl then n = n < (ok(0)-start)     
      irec = 0L
      if keyword_set(dp_flag) then begin
        n = n/2
        RECL_F = RECL_F/2
      endif
      stuff= make_array(n, double = dp_flag)
      readu, unit, junk, stuff, irec
      data(0:n-1) = stuff(0:n-1)
      data_rec =  (dp_flag) ? (RECL_U-2)/2 : (RECL_U-2)
      stuff = make_array( data_rec, double = dp_flag)
      j = RECL_F < (tnp-n)
      while n lt tnp do begin
        readu, unit, irec, stuff, irec
        data(n:n+j-1) = stuff(0:j-1)
        n = n+j
        if n+j ge tnp then begin
          j = tnp-n
          if j gt 0 then stuff = make_array(j, double = dp_flag)
        endif
      endwhile
      free_lun, unit
      data = reform (data, ndset, np, /overwrite)
      data = transpose(data)
      if np lt 3 then begin
        print, '% READHIST: Less than 3 data points/waveform.'
        print, '            This cannot be stored successfully in a WDF array'
        print, ' The following datasets and data have been read:'
        for i = 0, ndset -1 do print, string(i, format='(i4)'), '   ' , $
           labels(i), data(*, i)
        print, ' This data will be stored as a linear array in wdfarray: ', $
            strtrim(wdf(0), 2)
        put_wdfarray,[0, 1, reform(data, ndset*np, /overwrite)], wdf(0), $
          clab = 'Data from '+p1
        narray = 1L
        return
      endif

      arr = -1
      if ((not keyword_set(alldata)) and (size(dataset,/type) eq 7)) then begin
        if ((dataset[0] eq '') or (dataset[0] eq '*')) then alldata =1
      endif

      if keyword_set(alldata) then arr = indgen(ndset) $
        else if n_elements(dataset) gt 0 then begin
        s = size(dataset)
        if s[s[0]+1] eq 7 then begin
          if n_elements(strict) eq 0 then strict = 1
          nalldata = s[s[0]+2]
          nelalldata = make_array(nalldata, val=0)
          arr = get_match (dataset[0], labels, nel = nel)
          nelalldata [0] = nel
          if keyword_set (strict) and nel ne 1 then $
            print, '% READHIST: '+strtrim(nel, 2)+' values found for string: '+dataset[0]
          if nalldata gt 1 then for i = 1, nalldata-1 do begin
            new = get_match (dataset[i], labels, nel = nnew)
            nelalldata[i] = nnew
            if keyword_set (strict) and nnew ne 1 then $
              print, '% READHIST: '+strtrim(nnew, 2)+' values found for string: '+dataset[0]
            if nnew gt 0 then $
              arr = (nel eq 0) ? new : [arr, new]
          endfor
          nel = total(nelalldata)
          if max(nelalldata) gt 1 then begin
            if keyword_set(strict) then print, '% READHIST: Order of string datasets will change'
            arr = arr(uniq(arr, sort(arr)))
            ndset = n_elements(arr)
          endif else ndset = nel
        endif else begin
          dataset = long (dataset)
          ok = where (dataset gt 0 and dataset le ndset, ndset)
          if ndset gt 0 then arr = dataset[ok]-1
        endelse 
      endif
      nwdf = n_elements(wdf)
      if arr[0] lt 0 then begin
        if ndataset gt nwdf then $
          wdf =[wdf, wdf[nwdf-1]+1+lindgen(ndataset-nwdf)]
        arr = get_array_var(labels, wdf)
        if arr(0) lt 0 then begin
          print, '% READHIST: Unable to save any variables'
          return
        endif
        ndset = n_elements(arr)
        nwdf = ndset
      endif
      if nwdf lt ndset then begin
        if nwdf eq 1 then wdf = wdf[0]+indgen(ndset) else $
          wdf =[wdf, wdf[nwdf-1]+1+lindgen(ndset-nwdf)]
      endif
      maxwdf = max(wdf)
      if  ((maxwdf gt maxwdfarray) and (n_elements (wdfin) le 1)) then $
        set_maxwdfarray, (maxwdf+2L+ (maxwdf mod 2)) $
      else begin
        ok = where (wdf gt 0 and wdf le maxwdfarray, nwdf)
        wdf = wdf(ok)
      endelse
      
      if ndset gt nwdf then begin
        print, '% READHIST: Not enough WDF arrays to read all the data', ndset
        print, '            Only ', strtrim(nwdf, 2), ' arrays will be stored'
        ndset = n_elements(wdf)
      endif
;
      for i = 0, ndset-1 do $
        put_wdfarray, [t0, dt, reform(data[*, arr[i]])], wdf[i], /no_copy, $
          clabel = labels[arr[i]], xlabel = 'Time', ylabel = 'Amplitude', $
          file = p1
      narray = ndset
      end
