; ------------------------------------------------------------------------ 
; $Id: readpe.pro,v 1.9 2010/12/09 17:05:41 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
  Pro readpe, STR, fileinput, image,space, tlabel = tlab, clabel = clab, $
      xlabel = xlab, ylabel = ylab, blabel = blab, header = header_sav, $
      npoints = npoints, delta = delta,  outfile = outfile,  filter = filter, $
      scale = scale, swap = swap, ximage = ximage, $
      xheader = xheader, pheader = pheader, path = path, setval = setval, $
      xrange = xrange, yrange = yrange, microns = microns, $
      mmeters = mmeters, file_one = file_one, file_two= file_two
;
;+
;  Read a Perkin Elmer formatted dataset into an PFIDL structure or read
;  a LLNL L-Division PDS file.   
;  
;
;  Format: READPE , STR [, File_Name] [ , image] [ , space ]
;            [, TLABEL = tlabel] [, CLABEL = clabel] [, BLABEL = blabel]
;            [, XLABEL = xlabel] [, YLABEL = ylabel] [HEADER = header]
;            [, NPOINTS = npoints] [, DELTA = delta] [, OUTFILE = outfile]
;            [, XRANGE = xrange] [, YRANGE = yrange] [, SETVAL = setval]
; (input)    [, FILTER = filter] [, SCALE = scale] [SWAP = swap] 
;            [, PATH = path] [, MICRONS = microns] [, MMETERS = mmeters]
;            [, XIMAGE = ximage] [, XHEADER = xheader] [PHEADER = pheader]
;            [, FILE_ONE = file_one] [, FILE_TWO = file_two]
;  where:
;    STR    - Structure array number for the PE image 
;             Unused structure will be selected or get_maxstr()
;    File_Name - Name of the Perkin Elmer file with or without an extension.
;                File_Name may also be a search string if it is unique.
;                For example: File_Name= '*f4*' if f4 is a unique search string.
;                Note: Any extensions will be removed and 
;                      extensions of .hdr and .img  will be assumed.
;               (See XHEADER and XIMAGE below.)
;               filein will be modified to provide a complete path name
;               If XHEADER and XIMAGE are omitted, readpe will search for .hdr 
;                 and .img without regard to case.
;    image -    array containing the image file
;    space -    array containing the dataset x and y arrays. Use
;                  SP2XYZ, image, space, x, y 
;               to obtain x and y (SPACE = [X, Y])
;               Note: MicroD offsets are ignored.  Both X and Y start at (0, 0)
;    tlabel -   Dataset type label
;               Default is the value of "SETVAL"
;                 (Specular Density, Transmission or Intensity)
;    clabel - dataset comment label
;             Default is File Name // Comment1 // Comment2 // ScanTime
;    blabel - title or label for this block
;             Default is Specular Density, Transmission or Intensity
;               depending on the value of "SETVAL"
;    xlabel - horizontal dimension label for this block
;             Default is 'X (mm)'
;    ylabel - Verticle dimension label for this block
;             Default is 'Y (mm)'
;    header - Complete header for the PE dataset
;    npoints- Array of points for each dimension
;    delta  - Array of microns/step for each dimension
;             Valid only for uniform data
;    outfile- File name without path information
;    xrange - Range of X-values
;    yrange - Range of Y-values
;    setval - Data type D = Density, T= Transmission, E = Exposure
; Following are input parameters - values used are returned.
;    filter - Optional filter. If specified this parameter must have at least
;               two values which are taken as the maximum and minimum of the
;               scanned data.  If three parameters are given, the third is taken
;               as the default value given to all data which falls outside the
;               specified range.  Default value if SETVAL = 'D' is [0, 5000, -1]
;               That is all data less than 0 and greater than 5000 will be set
;               to -1.  With the default scaling of 800, this corresponds to 
;               density values between 0 and 6.25
;               Filter = 0 if setval not equal to density
;               If many values are filtered, the SWAP parameter may be wrong.
;    scale  - Parameter used to scale the integer counts.
;             Default is 1/800 = 1.25e-3 if SETVAL = D
;             Default is 1 otherwise.
;    swap   - This keyword is set by default according to the following:
;             Windows and linux: SWAP =0
;             UNIX: SWAP = 1
;             If SWAP = 0, then the bytes in IMAGE will not be swapped.
;             If SWAP = 1, then the bytes in IMAGE will be swapped.
;             Scans from a PC must be swapped if read on a SUN, HP or SGI.
;    path   - Directory path to the desired file
;             This is the same default value as used in OPENPFF
;    MMETERS- If set, save image in millimeters.
;             Default: See MICRONS below
;    MICRONS- If set, save image in microns.
;             If microns is set, save raw image in microns using the delta values
;             provided in the HDR file.  MMETERS is ignored.
;             If MICRONS is not set, save the image in millimeters.
;             If MMETERS is set save the image in millimeters.
;             If MMETERS is not set and polynomial scaling is specified, the 
;             image is stored using the polynomial scaling otherwise the image
;             will be stored in millimeter units.
;    ximage - Extension for the image file.
;    xheader- Extension for the header file.
;    pheader - If the keyword is set, the printer will be printed to the screen
;    file_one - Header file
;    file_two - Image file
;
;    EXAMPLE
;        read the file S6424n4f into structure 3
;            READPE, 3, 'S6424n4f'
;
;        Read a file into structure 5; select the file with DIALOG_PICKFILE
;            READPE, 5
;
; MODIFICATION HISTORY:
;    Written by:       L. P. Mix, 2/29/00
;    March 1, 2005     Added L-Division extensions
;    
;-
; 
     on_error, 1
common savepe_picker, oldfilter, oldpath

     if n_elements(fileinput) eq 0 then filein = 0 else filein = fileinput
     ss = size(filein)
     image = 0
     psep = path_sep()
     if ss( ss(0)+1) ne 7 then begin
       if n_elements(oldpath) eq 0 then begin
         cd, current = dir
         path = dir+psep
       endif
       if n_elements(oldpath) eq 0 then cd, current = path else path = oldpath
       p1 = dialog_pickfile(/must_exist, /read, path=path, get_path=newpath, $
           filter = ['*.hdr;*.HDR', '*.*'] )
       if strlen(p1) lt 1 then begin
         Print, 'Input file name must be provided'
         return
       endif
       oldpath = newpath
       filein = temporary(p1)
     endif
     if n_elements(str) eq 0 then str = get_maxstr() else $
       str = get_save_str(str[0], get_maxstr())

     filein = strtrim (filein, 2)
     ii = strpos(filein, '.', /reverse_search)
     if ii gt 0 then filein = strmid(filein, 0, ii)
     if strlen(filein) lt 1 then begin
       Print, 'Input file must have at least one character'
       return
     endif
     fils = file_search(filein  +'*', count = nfils)
     if nfils lt 2 then begin
       print, '% READPE:  Less than two files found with the name: ', filein
       return
     endif
     if n_elements(xheader) eq 0 then begin
       ok = get_match(strlowcase(filein+'.hdr'), strlowcase(fils), nel= nok)
       if nok eq 0 then begin
         print, '% READPE: Unable to find file: ', filein+'.hdr'
         return
       endif
       if nok gt 1 then begin
          print, '% READPE: Multiple files found: ', fils[ok]
          print, '  Please specify XHEADER'
          return
       endif 
       head = fils[ok[0]]
     endif else begin
       xheader = strtrim(xheader, 2)
       ii = strpos(xheader, '.')
       if ii lt 0 then head = filein+'.'+xheader else head = filein+xheader 
       ok = get_match(head, strlowcase(fils), /casesen, nel= nok)
       if nok eq 1 then head = fils[ok[0]] else begin
         if nok eq 0 then print,'% READPE: No files found with search string: ', $
             head else print,'% READPE: Multiple files found with search string: ', $
             head
         return
       endelse
     endelse
     file_one = head
     openr, unit, /get_lun, head, error = err
     if err ne 0 then begin
       print, '% READ_PE: ', !ERR_STRING
       print, '         Header File: ', head
       image = 0
       return
     endif
     a = fstat(unit)
     b = make_array(/byt, a.size)
     readu, unit, b
     free_lun, unit
     ok = where (b eq 39b, nok)
     if nok gt 0 then b[ok] = 32b
     ok = where (b lt 32, nok)
     if nok lt 22 then begin
       Print, 'Error reading header.' , a.size, ' bytes have been read'
       return
     endif
    
     if ok(nok-1) ne a.size-1 then begin
       nok = nok+1
       ok = [ok, a.size-1]
     endif
     fheader = make_array(nok, /string)
     st = 0
     n = 0
     for i = 0, nok-1 do begin
       if ok[i] gt st+1 then begin
         fheader[n] =strtrim(string(b(st:ok(i)-1)), 2)
         n = n+1
       endif
       st = ok(i) +1
     endfor
     fheader = fheader[0:n-1]
     if arg_present(header_sav) then header_sav = fheader
     nelhead = n
;
     pds_keywords =['IDENTIFICATION','SIGNX', 'SIGNY', 'DELTAX', 'DELTAY', $
       'AXSCAN', 'FILE REC LEN', 'SETVAL', 'XLEFT', 'XRIGHT', 'YBOTTOM', $
       'XLABEL', 'YLABEL', 'XUNITS', 'YUNITS', 'XFITORIGIN', 'YFITORIGIN', $
       'XFITLABEL', 'YFITLABEL', 'XFITUNITS', 'YFITUNITS', 'LINES DONE', $
       'SCAN TIME', 'FILTER', 'COMNT1', 'COMNT2' ]
;    Keywords of length less than or equal to 4.
     pds_4ch_keyword =['NXP', 'NYP', 'XPOS', 'YPOS', 'YTOP', 'XFIT', 'YFIT']
     
     if keyword_set(pheader) then for i = 0, n-1 do print, fheader(i)
     for i = 0, n-1 do begin
       pb = strpos(fheader[i], '=')
       if pb lt 0 then begin
         if strtrim(strupcase(fheader[i]), 2) eq 'END' then break
       endif else begin
;        Contains an equal sign "="
         keywrd =  strtrim(strupcase(strmid(fheader[i], 0, pb)), 2)
         pb = pb+1
         keylen = strlen(keywrd)

         pend = strpos(fheader[i], '/')
         fheader[i] = (pend gt 0) ? strmid(fheader[i], pb, pend-pb) : $
             strmid(fheader[i], pb)
         if keylen le 4 then begin
           ok = (where (keywrd eq pds_4ch_keyword, nok))[0]
           if nok eq 1 then begin
             if ok eq 0 then nx = long(fheader[i]) else $
               if ok eq 1 then ny = long(fheader[i]) else $
               if ok eq 2 then xpos = long(fheader[i]) else $
               if ok eq 3 then ypos = long(fheader[i]) else $
               if ok eq 4 then ytop = double(fheader[i]) else $
               if ok eq 5 then begin 
                 parts = strsplit(fheader[i], ',', /extract, count= count) 
                 xorder = fix(parts[0])
                 if xorder ne count-2 then begin
                   print, '% READPE:  Error reading XFIT keyword'
                   print, '  Header, Count:', fheader[i], count
                   return
                 endif
                 xfit = double(parts[1:*])
               endif else if ok eq 6 then  begin
                 parts = strsplit(fheader[i], ',', /extract, count= count)
                 yorder = fix(parts[0])
                 if yorder ne count-2 then begin
                   print, '% READPE:  Error reading YFIT keyword'
                   print, '  Header, Count:', fheader[i], count
                   return
                 endif
                 yfit = double(parts[1:*])
             endif
           endif else $
             print, '% READPE: Error finding short fheader keyword: ', keywrd, nok
         endif else begin
           ok = (where (keywrd eq pds_keywords, nok))[0]
           if nok eq 1 then begin
             if ok eq 0 then identification = strtrim(fheader[i], 2) else $
               if ok eq 1 then signx = strtrim(fheader[i], 2) else $
               if ok eq 2 then signy = strtrim(fheader[i], 2) else $
               if ok eq 3 then deltax = long(fheader[i]) else $
               if ok eq 4 then deltay = long(fheader[i]) else $
;              skip AXSCAN - Scan Axis
               if ok eq 6 then record = long(fheader[i]) else $
               if ok eq 7 then setval = strtrim(fheader[i], 2) else $
               if ok eq 8 then xleft = double(fheader[i]) else $
               if ok eq 9 then xright = double(fheader[i]) else $
               if ok eq 10 then ybottom = double(fheader[i]) else $
               if ok eq 11 then xlabel = strtrim(fheader[i], 2) else $
               if ok eq 12 then ylabel = strtrim(fheader[i], 2) else $
               if ok eq 13 then xunits = strtrim(fheader[i], 2) else $
               if ok eq 14 then yunits = strtrim(fheader[i], 2) else $
               if ok eq 15 then xfitorigin = double(fheader[i]) else $
               if ok eq 16 then yfitorigin = double(fheader[i]) else $
               if ok eq 17 then xfitlabel = strtrim(fheader[i], 2) else $
               if ok eq 18 then yfitlabel = strtrim(fheader[i], 2) else $
               if ok eq 19 then xfitunits = strtrim(fheader[i], 2) else $
               if ok eq 20 then yfitunits = strtrim(fheader[i], 2) else $
               if ok eq 21 then lines_done = long(fheader[i])
               if ok eq 22 then sctime = strtrim(fheader[i], 2)
               if ok eq 23 then filter = strtrim(fheader[i], 2)
               if ok eq 24 then comnt1 = strtrim(fheader[i], 2)
               if ok eq 25 then comnt2 = strtrim(fheader[i], 2)
                 
           endif else $
             print, '% READPE: Error finding long header keyword: ', keywrd, nok
         endelse
       endelse
     endfor
     if n_elements(nx) eq 0 || n_elements(ny) eq 0 then begin
       print, '% READPE: Bad header file - NX, NY'
       help, nx, ny
       return
     endif
     if n_elements(deltax) eq 0 || n_elements(deltay) eq 0 then begin
       print, '% READPE: Bad header file - DELTAX, DELTAY'
       help, deltax, deltay
       return
     endif
     npoints = [nx, ny]
     if n_elements(record) eq 0 then begin
       print, '% READPE: Bad header file - "File Rec Len"'
       return
     endif
     if n_elements(setval) eq 0 then begin
       print, '% READPE: Bad header file - SETVAL'
       return
     endif
     if setval eq 'D' then begin
       blab = 'Specular Density'
       if n_elements(scale) eq 0 then scale = .00125
       if n_elements(filter) eq 0 then filter = [0, 5000, -1]
     endif else begin
       if setval eq 'T' then  blab = 'Transmission' else blab = 'Intensity'
       if n_elements(scale) eq 0 then scale = 1.0
     endelse
     if n_elements(filter) eq 2 then filter = [filter, -1]
     tlab = blab
     ii = strpos(head, '/', /reverse_search)
     outfile = strmid(head, ii+1)
     clab = outfile
     stuff = ' - '
     if n_elements(identification) gt 0 then $
       if (filein ne identification &&  strlen(identification) gt 0) then $
         stuff = stuff+identification

     if n_elements(sctime) gt 0 then begin
       if strlen(stuff) gt 3 then stuff = stuff+':Scan time='+sctime else $
         stuff = stuff +'Scan time='+sctime
       if n_elements(comnt1) gt 0 then stuff = stuff+':'+strcompress(comnt1)
       if n_elements(comnt2) gt 0 then stuff = stuff+':'+strcompress(comnt2)
     endif   
     clab = clab + stuff
     nline = record/2

     if n_elements(ximage) eq 0 then begin
       ok = get_match(strlowcase(filein+'.img'), strlowcase(fils), nel= nok)
       if nok eq 0 then begin
         print, '% READPE: Unable to find file: ', filein+'.hdr'
         return
       endif
       if nok gt 1 then begin
          print, '% READPE: Multiple files found: ', fils[ok]
          print, '  Please specify XHEADER'
          return
       endif  
       head = fils[ok[0]]
     endif else begin
       xheader = strtrim(xheader, 2)
       ii = strpos(xheader, '.')
       if ii lt 0 then head = filein+'.'+xheader else head = filein+xheader
       ok = get_match(head, strlowcase(fils), /casesen, nel= nok)
       if nok eq 1 then head = fils[ok[0]] else begin
         if nok eq 0 then print,'% READPE: No files found with search string: ', $
             head else print,'% READPE: Multiple files found with search string: ', $
             head 
         return
       endelse
     endelse
;    Image file
     file_two = head
;
     openr, unit, /get_lun, head, error = err
     if err ne 0 then begin
       print, '% READPE: ', !ERR_STRING
       print, '         Image File: ', head
       image = 0
       return
     endif
     if n_elements(swap) eq 0 then begin
       no_swap = ['linux', 'Win32']
       swap =( where(!version.os eq no_swap) ge 0) ? 0:1
     endif
     
     if nline lt nx then begin
       print, 'Error in calculating points per line. (nx, nline)', nx, nline
       return
     endif
     stat = fstat(unit)
     if stat.size lt record*ny then begin
       print, '% READPE:  Image file is too small for this header file'
       print, '  File size: ', strtrim(stat.size, 2), ', Image size: ', $
         strtrim(record*ny, 2)
       return
     endif else if (stat.size gt record*ny+128) then begin
       hhead = bytarr(128)
       readu, unit, hhead
       print, 'Header information: ', string(hhead[2:*])
     endif
     image_read= intarr(nline, ny)
     readu, unit, image_read
     free_lun, unit
     image = image_read[0:nx-1, *]
     if swap ne 0 then byteorder, image
     if n_elements(filter) lt 3 then nok =0 else begin
       ok = where (image lt filter(0) or image gt filter(1), nok)
       if nok gt 0 then begin
         print, nok, ' values of the image set to ', strtrim(filter(2), 2)
         print, '   with a filter range of', filter(0:1)
         image[ok] = filter[2]
       endif
     endelse
     z=0
     if keyword_set(microns) then begin
       xlab = 'X(microns)'
       ylab = 'Y(microns)'
       delta = [deltax, deltay]
       x = findgen(nx)*deltax 
       y = findgen(ny)*deltay
     endif else begin
       if n_elements(xleft) gt 0 and n_elements(xright) gt 0 then begin
         deltax = (xright[0]-xleft[0])/(nx-1)
         x = xleft[0] + deltax* findgen(nx) 
       endif else begin 
         deltax = deltax* 1.0E-3 ; convert microns to mm
         x = findgen(nx)*deltax
       endelse
       if n_elements(ytop) gt 0 and n_elements(ybottom) gt 0 then begin
         deltay =  (ytop[0]-ybottom[0])/(ny-1) 
         y = ybottom[0] + deltay * findgen(ny) 
       endif else begin
         deltay = deltay *1e-3 ; convert microns to mm
         y = findgen(ny)*deltay
       endelse
       if n_elements(xfit) gt 0 and not keyword_set(mmeters) then begin
         x = poly(x, xfit )
         deltax = 0
         if n_elements(xfitlabel) gt 0 then begin
           if n_elements(xfitunits) gt 0 then begin
             if strpos(xfitlabel, xfitunits) lt 0 then $
               xfitlabel= xfitlabel+'('+xfitunits+')'
           endif
           xlab= xfitlabel
         endif else xlab = 'X- Poly fit'
       endif else begin
         if n_elements (xlabel) gt 0 then begin
           if n_elements(xunits) gt 0 then begin
             if strpos(xlabel, xunits) lt 0 then $
                 xfitlabel= xlabel+'('+xunits+')'
           endif
           xlab= xlabel
         endif else xlab = 'X(mm)'
       endelse
       if (n_elements(yfit) gt 0) and not keyword_set(mmeters) then begin
         y = poly(y, yfit)
         deltay = 0
         if n_elements(yfitlabel) gt 0 then begin
           if n_elements(yfitunits) gt 0 then begin
             if strpos(yfitlabel, yfitunits) lt 0 then $
               yfitlabel= yfitlabel+'('+yfitunits+')'
           endif
           ylab= yfitlabel
         endif else ylab = 'Y- Poly fit'
       endif else begin
         if n_elements (ylabel) gt 0 then begin
           if n_elements(yunits) gt 0 then begin
             if strpos(ylabel, yunits) lt 0 then $
                 ylabel= ylabel+'('+yunits+')'
           endif
           ylab= ylabel
         endif else ylab = 'Y(mm)'
       endelse
       if (x[nx-1] - x[0]) lt 0 then begin
         x = reverse (x)
         image = reverse(image, 1)
       endif
       if (y[ny-1] - y[0]) lt 0 then begin
         y = reverse (y)
         image = reverse(image, 2)
       endif
     endelse
     delta = [deltax, deltay]
     space = [X, Y]
     xrange = [min(x, max = mm), mm]
     yrange = [min(y, max = mm), mm]
     image = (scale ne 1) ? float(scale)*image : float(image)
     arr2str, str, image, x, y, z, clabel = clab, $
       xlabel = xlab, ylabel = ylab, file = file_one
     return
     end
