; ------------------------------------------------------------------------ 
; $Id: readsbf.pro,v 1.2 2010/12/09 18:01:12 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
 Pro readsbf, filename, WDF, np = np, ierror = ier
;
;+
; NAME:
;       READSBF
;
; PURPOSE:
;
;       Read a standard Sandia Binary Format file into a wdf array
;       This format is output from Labview for Z and other DAS files.
;
; CALLING SEQUENCE:
;
;       READSBF [, FILENAME] [, WDF]
;
; INPUTS:
;       FILENAME: The file name
;                 If this is not a string, DIALOG_PICKFILE will be used to 
;                 select the file
;       WDF:      WDF array number
;                 DEFAULT:  WDF = GET_MAXWDF()-1
;
; KEYWORD PARAMETERS:
;       NP:       Number of points
;       IERROR:   Error Flag 
;                 If zero, no error occurred
; OUTPUTS:
;       The data in the file will be stored in a WDF array.
;
; PROCEDURE:
;       The required file is opened and read and the waveform stored in a 
;       WDF array  
;       The file is an unformatted binary file in XDR format.
;       The order of the data is NP, DeltaT, Tinitial, Ydata
;       where NP is the number of points in Ydata and DeltaT is the time
;             spacing between points and Tinitial is the initial time.
;
; EXAMPLE:
;       Read NTOF_1 and store in WDF array 6
;       READSBF, 'NTOF_1', 6
;
; MODIFICATION HISTORY:
;       Written by:     L. P. Mix, Jr., February 19, 2010
;
; 
;-
;
     on_error, 1
@com.dat
common savesbf_picker, oldfilter, oldpath
     if n_elements(initwdf) eq 0 then wdfinit
;    check format
     ier = -1
     np = 0L
     fname = (n_elements(filename) eq 0) ?  0 : filename[0]
     b = size(fname, /type)
     if b eq 7 then begin
       vers = strupcase(strmid(!version.os_family, 0, 3))
       if (vers eq 'UNI') then begin
;      get system variable substitution for unix systems
         temp = fname
         FULFNAM, temp, fname
       endif
       fname = file_expand_path(fname)
       if !version.os eq 'linux' then resolve_it, fname
       info = file_info(fname)
       if (info.exists + info.read) ne 2b then begin
          print, '% READSBF: File does not exist with read permission'
          print, '           File Name = ', fname
          print, '           File Exists =', info.exists
          print, '           Read Permission =', info.read
          ier = -2
          return
       endif
       openr,unit,fname, /xdr, /get_lun, error = error
     endif else begin
       if n_elements(oldpath) eq 0 then cd, current = path else path = oldpath
       fname = dialog_pickfile(/must_exist, /read, path = path, get_path= newpath)
       if strlen(fname) lt 1 then begin
         print, ' No File was Selected - RETURNING'
         return
       endif
       oldpath = newpath
       if !version.os eq 'linux' then resolve_it, fname
       openr,unit,fname, /get_lun, /xdr, error = error
       IF (error NE 0) then begin
         Print, '% READSBF:  Unable to open file: '+strtrim(fname, 2)
         PRINT, !ERROR_STATE.MSG 
         return
       endif
     endelse
     clab = file_basename(fname)    
     ier =2
     if n_elements(wdf) eq 0 then wdf = maxwdfarray -1
     if wdf lt 1 or wdf gt maxwdfarray then begin
       print, '% READSBF:  WDF must be in the range of 1 to ' +strtrim(maxwdfarray,2)
       wdf  = maxwdfarray -1
       print, '            WDF set to : '+strtrim(wdf, 2)
     endif
     np = 0.0
     deltat = 0.0
     tinit = 0.0
     readu, unit, np, deltat, tinit
     np = round(np)
     ydata = fltarr(np, /nozero)
     readu, unit, ydata
     free_lun, unit
     i2w, [tinit, deltat], ydata, wdf, clab = clab, xlab = 'Time(s)', $
       ylab = 'Amplitude', file = fname
     ier =0
end
