; ------------------------------------------------------------------------
; $Id: readspe.pro,v 1.1 2011/06/10 17:25:54 lpmix Exp $
; ------------------------------------------------------------------------
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
;+
; NAME:
;       READSPE
;
; PURPOSE:
;       Read a Princeton Instruments "SPE" file and store it in a PFIDL 
;         structure array.  Optionally the image or the PFIDL structure may
;         be returned.  The SPE file format has a 4100 byte header which is 
;         converted to a structure.
;       Multi-frame images are stored as 3d with Z = Frame number.
;       When the SPE file has no specific calibration in the X or Y direction
;         the axis is just pixel units.
;
; CALLING SEQUENCE:
;  Format:  READSPE, STRUCTURE  [, FILE] [, DX = dx] [, DY = dy] 
;           [, SCALE = scale] [, CLABEL = clabel] [, XLABEL = xlabel]
;           [, COMMENTS = comments] [, EXPOSURE = exposure]
;           [, YLABEL = ylabel] [, TLABEL = tlabel] [, REVERSE = reverse]
;           [, FILENAME = filename] [, /BARE_NAME]
;           [, /IMAGE] [, /STRUCTURE] [, /INTEGER]
;
; INPUTS:
;       STRUCTURE  - Structure Array Number or variable if STRUCTURE 
;                    or IMAGE is set.
;                    Default:  STRUCTURE = GET_MAXSTR()-1
;       FILE       - SPE file name
;                    If zero of undefined, DIALOG_PICKFILE will be used.
;
; KEYWORD PARAMETERS:
;       DX         - X increment for linear X scaling
;                    If DX le 0, then DX = 1.0
;                    DEFAULT: Use X_Calibration from header
;       DY         - Y increment for linear Y scaling
;                    If DY le 0, then DY = 1.0
;                    DEFAULT: Use Y_Calibration from header
;       SCALE      - Intensity Scale Factor
;                    DEFAULT: SCALE = 1
;       CLABEL     - Dataset comment for the structure.  If present, BARE_NAME and 
;                    TRUNCATE are ignored.
;                    DEFAULT: CLABEL = FILE + " - DATE="+header.date+';;COMMENTS='+comments
;       BARE_NAME  - If set, strip the path and extension from the filename for CLABEL
;                    If not set, use "FILE_EXPAND_PATH(FILENAME)"
;       TRUNCATE   - If set, do not append the date to the filename.
;       EXPOSURE   - Exposure in seconds = HEADER.EXP_SEC
;       COMMENTS   - Comments = HEADER.Comments
;       XLABEL     - Abscissa comment for the structure
;                    DEFAULT: XLABEL = HEADER.XLABEL
;                                 or = 'Pixel' if the X calibration is missing
;       YLABEL     - Ordinate comment for the structure
;                    DEFAULT: YLABEL = HEADER.YLABEL
;                                 or = 'Pixel' if the Y calibration is missing
;       VLABEL     - Desired vector or data label
;                    DEFAULT: VLABEL = HEADER.DLABEL
;                                 or = 'Intensity' if the header value is missing
;       TLABEL     - Type label for the structure
;                    DEFAULT: TLABEL = 'SPE Image'
;       IMAGE      - If set, the SPE image is returned in the variable STRUCTURE
;       STRUCTURE  - If set and if image is not set, the structure is returned in
;                    the variable STRUCTURE
;       FILENAME   - Filename actually read
;       INTEGER    - If set, save the image as an integer array
;                    By default, integer arrays are saved as floats for future calculations
;       REVERSEIMAG- If not set to zero, image will be reversed.
;                    IDL has a convention from bottom to top; SPE stores 
;                    image top to bottom.  Image will be reversed unless REVERSE
;                    is set to zero (ie. REVERSE = 0)
;                    DEFAULT:  REVERSE = 1
;       HEADER     - SPE files contain a 4100 byte header with various data parameters
;                    If a valid arguement exists, the header is reeturned as a structure.
;       SETDEFAULT - Set the default for the following parameters:
;                     BARE_NAME   = 0
;                     TRUNCATE    = 0
;                     REVERSEIMAG = 1
;       SHOWDEFAULT- Show the default settings.
;	
; OUTPUTS:
;       A SPE image is stored as a PFIDL structure array or returned as an 
;       image or as a structure.  Multi-frame image are stored as 3D images with
;       the frame as the third dimension.
;
; RESTRICTIONS:
;       The procedure assumes header version 2.
;
; PROCEDURE:
;       This procedure uses a procedure written by Mark Rivers, University of Chicago, 
;       to read the file and then converts the data into a PFIDL structure.
;
; EXAMPLE:
;       Read the SPE file 'Shot_753.spe' into structure array 4.
;         READSPE, 4, 'Shot_753.spe'
;
;       Read the SPE file 'Shot_777.spe' into structure 2 and store the header into
;       the variable, head777.
;         READSPE, 2, 'Shot_777.spe', HEADER = head777
;
;       Read an SPE file selected with a browser into structure 3.
;         READSPE, 3
;
; MODIFICATION HISTORY:
;       Written by:     L. P. Mix, Jr., June 8, 2011
;
;-

PRO readspe,  STR, FILE, DX = dx, DY = dy, SCALE = scale, CLABEL = clabel, $
       XLABEL = xlabel, YLABEL = ylabel, TLABEL = tlabel, IMAGE= image, $
       COMMENTS = comments, EXPOSURE = exposure, $
       FILENAME = filename, HEADER = header, VLABEL = vlabel, $
       BARE_NAME = barename, STRUCTURE = structure, REVERSEimag = reverseimag, $
       integer = integerarr, SETDEFAULT = setdefault, showdefault = showdefault

on_error, 1

; Common block
@qsstrcom.dat

common READSPE_COMMON, barename_save, truncate_save, reverseimag_save

      if n_elements (structinit) eq 0 then qsstruct_init
;
      if n_elements(truncate_save) eq 0 then begin
        truncate_save = 0
        barename_save = 0
        reverseimag_save = 1
      endif
      if keyword_set(setdefault) then begin
        if n_elements(reverseimag) gt 0 then reverseimag_save = reverseimag
        if n_elements(barename) gt 0 then barename_save = bare_name
        if n_elements(truncate) gt 0 then truncate_save =truncate
      endif
      if keyword_set(showdefault) then begin
        print,  'Default parameters for the READSPE command are:'
         print, '    TRUNCATE    = ' + strtrim (truncate_save, 2)
         print, '    BARE_NAME   = ' + strtrim (barename_save, 2)
         print, '    REVERSEIMAG = ' + strtrim (reverseimag_save, 2)
      endif
      nparams = n_params()
      if nparams eq 0 and (keyword_set (setdefault) or keyword_set(showdefault)) $
        then return

      if nparams eq  0 then begin
        print, '% Format:', $
           '  READSPE, STRUCTURE  [, FILE] [, DX = dx] [, DY = dy]'
        print, '   [, SCALE = scale] [, CLABEL = clabel] [, /BARE_NAME]'
        print, '   [, COMMENTS = comments] [, EXPOSURE = exposure]'
        print, '   [, XLABEL = xlabel] [, YLABEL = ylabel] [, TLABEL = tlabel]'
        print, '   [, HEADER = header] [, VLABEL = vlabel] [, /REVERSEIMAG]'
        print, '   [,FILENAME = filename] [, /IMAGE] [, /STRUCTURE] [, /INTEGER]'
        return
      endif
      if not keyword_set(image) and not keyword_set(structure) then begin
        if n_elements(str) eq 0 then begin
          print, '% READSPE: Structure array number must be defined'
          return
        endif
        if size(/type, str) gt 3 then begin
          print, '% READSPE: Structure must be an integer or long'
          help, str
          return
        endif
        strsave = get_save_str(str, get_maxstr())
      endif else if not arg_present(str) then begin
        print, '% READSPE: STRUCTURE must be a variable to return a value'
        if keyword_set(image) then print, '           IMAGE is set'
        if keyword_set(structure) then print, '           STRUCTURE is set'
        return
      endif
      if n_elements(reverseimag) eq 0 then reverseimag = reverseimag_save
      if n_elements(truncate) eq 0 then truncate = truncate_save
      if n_elements(barename) eq 0 then barename = barename_save
      if n_elements(file) eq  0 then fname = 0 else fname = file[0]
      b = size(fname)
      if b(1) eq 7 then begin
        resolve_it, fname
        info = file_info(fname)
        if (info.exists + info.read) ne 2b then begin
           print, '% READSPE: File does not exist with read permission'
           print, '         File Name = ', fname
           print, '         File Exists =', info.exists
           print, '         Read Permission =', info.read
           return
        endif
      endif else begin
        fname = dialog_pickfile(/must_exist, /read)
        if strlen(fname) lt 1 then begin
          print, '% READSPE: No File was Selected - RETURNING'
          return
        endif
        resolve_it, fname
      endelse
      read_princeton, fname, str, header = header, date = date, $
         x_calibration=x_calibration, y_calibration= y_calibration, $
         comments=comments, exposure=exposure, dx = dxp, dy = dyp, $
         xlabel = xlabelp, ylabel = ylabelp
      if n_elements(str) lt 10 then begin
        print, '% READSPE: Invalid image returned.'
        help, str
        return
      endif
      filename =  file_expand_path(fname)
      if keyword_set(reverseimag) then str = reverse(str, 2, /overwrite)
      if n_elements(scale) gt 0 then str = str*scale[0]
;     If only image is requested then return
      if keyword_set(image) then return
      ss = size(str)
      nframes = (ss[0] eq 3) ? ss[3] : 1
      nx = ss[1]
      ny = ss[2]
      if n_elements(clabel) eq 0 then begin
        clabel = keyword_set(barename) ? get_bare_filename(fname) : filename
        if not keyword_set(truncate) then begin
           clabel = clabel + ' - DATE=' + date 
           ok = where (strlen(comments) gt 0, nok)
           if nok gt 0 then begin
             clabel = clabel+';;Comments='
             for i = 0, nok -1 do clabel = clabel+strtrim(comments[ok[i]], 2)+';;'
           endif
        endif
      endif

      v1 = keyword_set(integerarr) ? round(temporary(str)) :  float(temporary(str))
      if n_elements(tlabel) eq 0 then tlabel = 'SPE Image'
      if n_elements(xlabel) eq 0 then xlabel = xlabelp
      if n_elements(ylabel) eq 0 then ylabel = ylabelp
      if n_elements(vlabel) eq 0 then vlabel = header.dlabel
      if n_elements(dx) eq 0 then begin
        dx = dxp 
        x = x_calibration
      endif else x = findgen(nx)*dx[0]
      if n_elements(dy) eq 0 then begin
        dy = dyp
        y = y_calibration
      endif else y = findgen(ny)*dy[0]
      if nframes eq 1 then $
        arr2str, str, /str, v1, x, y, dx = dx[0], dy= dy[0], $
           clab = clabel, XLABEL = xlabel, YLABEL = ylabel, TLABEL = tlabel, $
           v1label = vlabel, file = filename  $
      else $
        arr2str, str, /str, v1, x, y, 1.+findgen(nframes), dx = dx[0], dy= dy[0], $
           dz = 1.0, clab = clabel, XLABEL = xlabel, YLABEL = ylabel, TLABEL = tlabel, $
           v1label = vlabel, file = filename 
      if keyword_set(structure) then return
      i2s, str, strsave, /no_copy
      str = strsave

END
