; ------------------------------------------------------------------------ 
; $Id: readtk2.pro,v 1.3 2007/06/06 20:03:52 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
 Pro readtk2, fnamein, wdfin, data= data, label = label, $
       npoints= np, narray = narray
;
;+
;    read an tkdas data file
;       Code assumes 2 datasets per file
;
;    format is:  readtk [, fname] [, WDF] [, data = data] [, label=label] 
;                       [, npoints =np] [, Narray = narray]
;    where 
;      fname   - the file name
;                if a file name then the file is opened and closed at the end
;                If fname is zero or omitted then PICKFILE will be used to get
;                  the file name.
;      NOTE: If only one parameter is given and the parameter is a number, 
;            then ReadTK assumes that it is a WDF number.
;      WDF     - WDF array number or an array of numbers.
;                Default is 1  
;                If WDF is less than 0 and n_elements(data) > 0 then no WDF
;                  arrays will be stored.
;      DATA    - Input data data = data (npoints+2, narray)
;                (Initial time and delta time are first two points.)
;      label   - string array of the names of the arrays
;      npoints - number of data values read for each dataset 
;      narray  - number of arrays read - present version = 2
; EXAMPLES:
;    Read a TKDAS file and store the data beginning in WDF array 1.
;       READTK2, 0
;   or  READTK2, 1 
;    Note: Default WDF array is 1 but procedure must be called with at
;          least 1 parameter.
; 
;    Read a TKDAS file and store the arrays starting in wdfarray 10
;       READTK2, 10 
;   or  READTK2, 0, 10
;  
;    Read TKDAS file 'DAT1234' and store the data in WDF array 1
;       READTK2, 'DAT1234', 1
;   or  READTK2, 'DAT1234'         ; since WDF = 1 is default
;
;    Read a TKDAS file and store in arrays [1, 3, 5, 7, 9, 11]
;       READTK2, 1+2*indgen(6)
;    Note: If there are more than 6 waveforms, additional waveforms will be
;            stored in 12, 13, ....
;          If there are less than 6 waveforms, no data will be stored in the
;            additional WDF arrays.
;
;    Read the data from a TKDAS file into an array DATA and store data beginning
;      in wdf array 1.
;         READTK2, 0, data = data
;    To only return the array without storing data in WDF arrays then:
;         READTK2, -1, data = data
;-
;
     on_error, 1
;    check format
     nparams = n_params()
      if nparams eq 0 then begin
        print, '% Format: READTK2 [, FNAME] [, WDF] [, DATA = data]
        print, '           [, LABEL = label] [NPOINTS = np] [, NARRAY = narray]'
        return
      endif
      if n_elements (fnamein) eq 0 then fnamein = 0
      fname = fnamein(0)
      b = size(fname)
      if b(1) eq 7 then begin
        openr,unit,fname, /get_lun
        lcfile = 1
      endif else begin
        fname = dialog_pickfile(/must_exist, /read)
        if strlen(fname) lt 1 then begin
          print, '% READTK2: No File was Selected - RETURNING'
          return
        endif
        openr,unit,fname, /get_lun
        lcfile = 0
      endelse
      if n_elements (label) eq 0 then label = ['V1', 'V2']
      junkstr = '' 
;     skip the header and find the beginning of the actual data
      skip = 1
      while skip gt  0 do begin
        readf, unit, junkstr
        parser, junkstr,ignore='=', name, narray, /stringarray
        if name(0) eq 'Number' then skip = 0
      endwhile
      np = long(name(narray-1))
      readf, unit, junkstr
      parser, junkstr,ignore='=', name, narray, /stringarray
      t0 = float (name(narray-1))
      readf, unit, junkstr
      parser, junkstr,ignore='=', name, narray, /stringarray
      dt = float (name(narray-1))
      narray = 2
      maxnp = np+2
      data = make_array (maxnp, narray)
      data(0, *) = t0
      data(1, *) = dt
      ax = make_array(narray, np)
      readf,unit,ax
      free_lun, unit
      ax = transpose(ax)
      for i = 0, narray-1 do data(2:*, i) = ax(*, i)
      if nparams eq 1 then begin
        if lcfile eq 1 then wdft = 1+indgen(narray) $
          else wdft = fnamein
      endif else begin
        if n_elements (wdfin) eq 0 then wdft = 1 else wdft = wdfin
      endelse
      if wdft(0) ge 0 then begin
        maxwdf = get_maxwdf()
        ok = where (wdft gt 0 and wdft le maxwdf, nwdf)
        if nwdf eq 0 then wdf = 1+indgen(narray) else begin
          wdf = wdft(ok)
          if nwdf lt narray then $
            wdf = [wdf, wdf(nwdf-1) + indgen(narray-nwdf)+1]
        endelse
        if wdf(narray-1) gt maxwdf then set_maxwdfarray, wdf(narray-1)
        for i = 0, narray-1 do $ 
          put_wdfarray, data(0:(((np(i)+1)>3)), i), wdf(i), $
          file = fname, xlabel = 'Time (s)', ylabel = 'Volts', clabel = label(i)
      endif
      return
;
      end
