; ------------------------------------------------------------------------ 
; $Id: readtqhis.pro,v 1.5 2010/12/09 17:18:48 lpmix Exp $
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
 pro readtqhis, wdfin, dataset, alldata = alldata, narray = narray, mdt = mdt, $
         ndataset = ndataset, $
         filename = p1, ignore = ignore, helpme = helpme, verbose = verbose
;+
; NAME:
;       READTQHIS
;
; PURPOSE:
;       Read a unformatted history file from TWOQUICK
;
; CALLING SEQUENCE:
;
;       READTQHIS, WDFARAY [, NDATASET = ndataset ] [, NARRAY = narray]
;                [, FILENAME = filename] [, MDT = mdt] [, /HELPME] 
;
; OPTIONAL INPUTS:
;       WDFARRAY  - WDF array number or an array of numbers.
;                   Default = 1
;                   If more arrays are selected than contained in WDFARRAY,
;                   then sequential numbers will be used.
;                   If increased, MAXWDFARRAY will be an even number at least
;                   two larger than necessary.
;       NDATASET  - Total number of datasets in the file
;       NARRAY    - Total number of WDF arrays populated
;       FILENAME  - Name of the file.  Default: FILENAME ='tqhis.dat'
;                   If tqhis.dat is missing or filename =0, then 
;                     DIALOG_PICKFILE will be used.
;       MDT       - Optional time scale factor.  The time or abscissa values
;                   are multiplied by this number. Default: MDT =1
;                   Default time step equal 1.
;       HELPME    - If set, print a help message.
;
; OUTPUTS:
;       The selected WDF arrays will be stored.
;
; RESTRICTIONS:
;       This will read files generated on the Intel and UNIX machines but
;       has not be verified on MAC's and DEC machines, but should also work
;       there.
;       Current version assumes less than 8000 history variables.
;
; PROCEDURE:
;      1. Open the file and determine if byte swapping is required.
;      2. Close and reopen the file and read the data.
;      3. Determine the desired datasets.
;      4. Store the data.
;
;
; EXAMPLE:
;       Read the data from qcks.his and store in WDF arrays beginning at 4
;          READTQHIS, 4, /all
;
;       Read the data from a file determine from a GUI and store in arrays 
;       beginning in WDF array 1.
;          READTQHIS
;
;       Read the data from a file, picked from DIALOG_PICKFILE, with the 
;       string 'current' and '^v' and store the arrays beginning in WDF array 1.
;          READTQHIS, 1, ['current', '^v'], file =0
;
; MODIFICATION HISTORY:
;       Written by:     LPMix, April 4, 2003
;-
on_error, 1
      common pffcom, initpff, pff_exe
@com.dat
common savetqhis_picker, oldfilter, oldpath

      ndataset = 0L
      narray =0L
;     see if we are initialized
      if n_elements(initwdf) eq 0 then wdfinit

      if (n_elements(initpff) eq 0) then init_pff else $
        if (initpff eq 0) then init_pff
      if initpff eq 0 then begin
        print, 'PFF libraries are not supported by this Operating System'
        return
      endif
;

      if keyword_set(helpme) then begin
        print, '% Format: READTQHIS, WDFARAY [, DATASET] [, /ALLDATA]'
        print, '               [, NARRAY = narray] [, FILENAME = filename]'
        print, '               [, MDT = mdt] [, /HELPME]'
        return
      endif

      if n_elements(initwdf) eq 0 then wdfinit
      npar = n_params()
      if n_elements (wdfin) eq 0 then wdf = 1 else wdf = wdfin
      vers = strupcase(strmid(!version.os_family, 0, 3))

      if n_elements (p1) eq 0 then begin
        p1 = file_search('tqhis.dat', count = c)
        if c gt 0 then p1 = p1[0]
        if !version.os eq 'linux' then resolve_it, p1
      endif else begin
        file = p1[0]
        if size(file, /type) eq 7 then begin
          if (vers eq 'UNI') then begin
;           get system variable substitution for unix systems
            temp = p1
            FULFNAM, temp, p1
            if !version.os eq 'linux' then resolve_it, p1
          endif
          p1 = file_search(file, count = c)

          if c eq 0 then $
            print, '% READTQHIS: Unable to locate file: ', strtrim(file, 2) $
          else p1 = p1[0]
        endif else c=0
      endelse
      if c eq 0 then p1 ='' else p1 = strtrim(p1(0), 2)
      if (strlen(p1) lt 1) then begin
        if n_elements(oldpath) eq 0 then cd, current = path else path = oldpath 
        p1 = dialog_pickfile(/must_exist, title= 'Select History file to read', $
           path = path, get_path = newpath)
        p1 = strtrim(p1, 2)
        if strlen(p1) lt 1 then return
        oldpath = newpath
        if !version.os eq 'linux' then resolve_it, p1
      endif

      if (vers eq 'UNI') then begin
;       get system variable substitution for unix systems
        temp = p1
        FULFNAM, temp, p1
      endif
      p1 = file_expand_path(p1)
      if !version.os eq 'linux' then resolve_it, p1
;
      openr, unit, /get, p1, error = error

      if error ne 0 then return
      print, '% READTQHIS: Reading from file ',p1

      i = 0L
      readu, unit, i
      if i gt 32768 then begin
        swap = 1
        byteorder, i, /lswap
        if i gt 32768 then begin
          print, '% READTQHIS: Unknown file format
          free_lun, unit
          return
        endif
        ii = i/4
        j = make_array(ii+1, /long)
        readu, unit, j
        j = j[ii]
        byteorder, j, /lswap
        if j ne i then begin
          print, '% READTQHIS: File format is not proper', i, j
          free_lun, unit
          return
        endif
      endif else begin
        swap = 0
        ii = i/4
        j = make_array(ii+1, /long)
        readu, unit, j
        j = j[ii]
        if j ne i then begin
          print, '% READTQHIS: File format is not proper', i, j
          free_lun, unit
          return
        endif
      endelse
;
;     Convert to real time by scaling to the simulation time step
;     Must have mdt -- if not use output index.
      t0 = 0.0
      dt = 1.0
      if n_elements (mdt)gt 0 then begin
        if mdt ne 0.0 then dt = mdt
        xlabel = 'Time'
      endif else xlabel = 'History Number'
      free_lun, unit
;
      openr, unit, /get, p1, swap_end= swap
      ndset = ii
      ndataset = ndset
      np = 0L
      maxnp = 8192L
      labels = make_array(ndset, /string)
      for i = 0, ndset-1 do labels[i] = 'Array_'+strtrim(i+1, 2) 

      rec = 0L
      ii = 0L
      rec2 = 0L
      dat = make_array(ndset)

      data = make_array(maxnp*ndset)
      on_ioerror, badnum 

      while (not eof(unit))  do begin
        readu, unit, rec, dat, rec2
        if np eq maxnp then begin
          data =[data, make_array(ndset*maxnp)]
          maxnp= maxnp+maxnp
        endif
        data [ii] = dat
        ii = ii +ndset
        np = np+1
      endwhile
badnum:

      free_lun, unit
      if np lt 3 then begin
        print, '% READTQHIS: Less than 3 points read: NP =', np
        print, '             Points read:
        for i = 0, np-1 do print, data[i*ndset:(i+1)*ndset-1]
        return
      endif
      data = data(0:np*ndset-1)
      data = reform (data, ndset, np, /overwrite)
      nwdf = n_elements(wdf)
      if nwdf  eq 0 then begin
        wdf = 1
        nwdf = 1
      endif
      if nwdf lt ndset then begin
        if nwdf eq 1 then wdf = wdf[0]+indgen(ndset) else $
          wdf =[wdf, wdf[nwdf-1]+1+lindgen(ndset-nwdf)]
      endif
      maxwdf = max(wdf)
      if maxwdf gt maxwdfarray then $
        set_maxwdfarray, (maxwdf+2L+ (maxwdf mod 2)) 
      data = transpose(data)
      
      for i = 0, ndset-1 do $
        put_wdfarray, [t0, dt, reform(data[*, i])], wdf[i], /no_copy, $
          clabel = labels[i], xlabel = xlabel, ylabel = 'Amplitude', $
          file = p1
      narray = ndset
      end
