; ------------------------------------------------------------------------
; $Id: readunr.pro,v 1.1 2011/06/17 18:24:36 lpmix Exp $
; ------------------------------------------------------------------------
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
;+
; NAME:
;       READUNR
;
; PURPOSE:
;       Read a University of Nevada Reno ASCII file.
;       The file format for each dataset begins as shown below.
;       The first two numbers are the density and temperature.
;       The fourth line contains the number of data points.
;       Each file may contain multiple datasets.
;      The first few lines from a file are below:
;     0.20E+22  190.00 13. 11.  101 0.19430000E+04 0.19830000E+04 501 0.05 1 0 1 2 0
;     0.2149 0.0000D+00 0.1000D+01  11.0000  11.0000  16.5000
;      0
;     401
;    0.19430000E+04  0.409862E-05
;    0.19431000E+04  0.414191E-05
;    < an additional 399 lines of XY data >
;
; CALLING SEQUENCE:
;
;       READUNR, WDF [, FILE]
;
; INPUTS:
;       WDF:    Initial WDF array number
;
; OPTIONAL INPUTS:
;       FILE:   File containing the UNR datasets
;               If missing, DIALOG_PICKFILE will be called.
;	
; KEYWORD PARAMETERS:
;       NWDF:      Return the number of datasets read
;       TRUNCATE:  If set, do not add density, temperature information
;
; OUTPUTS:
;       All datasets in the file are stored in WDF arrays
;
; PROCEDURE:
;       REASC is used to read each dataset.
;
; EXAMPLE:
;       Read the datasets in the file, "lpdb_abs.out17_heg", store the 
;       datasets starting in wdf array, 21, and the number of datasets in n
;         READUNR, 21, "lpdb_abs.out17_heg", nwdf = n
;
; MODIFICATION HISTORY:
; 	Written by:	L. P. Mix, Jr., June 10, 2011
;
;-

pro readunr, wdf, filename, truncate = truncate, nwdf = nwdf
on_error, 1
  npar = n_params()
  if npar eq 0 then begin
     print, '% Format: READUNR, WDF [, FILE] [, NWDF = nwdf]'
     print, '          [, TRUNCATE = truncate]'
     return
  endif
  if n_elements (wdf) eq 0 then begin
    Print, '% READUNR:  Initial WDF array number must be specified'
    return
  endif
  if n_elements(filename) eq 0 then begin
    filename = dialog_pickfile()
    if strlen(filename) lt 1 then begin
      print, '% READUNR:  No File was Selected - RETURNING'
      return
    endif
  endif else begin
    vers = strupcase(strmid(!version.os_family, 0, 3))
    if (vers eq 'UNI') then begin
;     get system variable substitution for unix systems
      temp = filename
      FULFNAM, temp, filename
    endif
    if !version.os eq 'linux' then resolve_it, filename
  endelse
  filename = file_expand_path(filename)
  info = file_info(filename)
  if (info.exists + info.read) ne 2b then begin
     print, '% REASC: File does not exist with read permission'
     print, '         File Name = ', filename
     print, '         File Exists =', info.exists
     print, '         Read Permission =', info.read
     ier = -2
     return
  endif
  wdfini = wdf
  openr, unit, /get_lun, filename, error = ier
  if ier ne 0 then begin
    print, !err_string
    print, '% READUNR:  Error opening file: '+filename
    return
  endif
  on_ioerror, endit
  while not eof(unit) do begin
   readf, unit, x, y
   if eof(unit) then break
   reasc, unit, formatin = 1, 1, wdf, skip = 2, /xydata
   if not keyword_set(truncate) then $
     lab, wdf, ' - Density='+strtrim(x,2)+';;Temp='+strtrim(y,2), /append
   wdf += 1
  endwhile
endit: nwdf = wdf - wdfini
  free_lun, unit
end
