; ------------------------------------------------------------------------
; $Id: readwfm.pro,v 1.4 2010/12/09 17:50:38 lpmix Exp $
; ------------------------------------------------------------------------
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
;+
; NAME:
;      
;
; PURPOSE:
;       Read a WFM Tektronix format file.  Based on Tektronix programmers
;       guide.
;
; CALLING SEQUENCE:
;       READWFM, FILENAME [, WDF]
;
; INPUTS:
;       FILENAME: File name
;               If FILENAME is missing or equal to zero, DIALOG_PICKFILE
;               will be called.
;       WDF:    Waveform array number to store the data.
;               If missing, GET_MAXWDDF()-1 will be used.
;	
; KEYWORD PARAMETERS:
;       RAW_DATA: Show the raw data
;                 If RAW_DATA lt 0, save the data in WDF array = abs(RAW_DATA)
;       EXP_DIM1: Structure for Explicit dimension
;       EXP_DIM2: Structure for Explicit dimension
;       IMP_DIM2: Structure for Implicit dimension
;       IMP_DIM2: Structure for Implicit dimension
; 
; OUTPUTS:
;       Store curve values in a WDF array
;
; RESTRICTIONS:
;       Limited testing
;
; PROCEDURE:
;       Read the waveform data and store it in a wdf array
;
; EXAMPLE:
;       Read TEK040.wfm into wdf array 33
;         READWFM, 'TEK040.wfm', 33
;
;       Read a file to be selected into WDF array 6
;         READWFM, 0, 6
;
; MODIFICATION HISTORY:
; 	Written by:	L. P. Mix, Jr., September 17, 2007
;-

PRO READWFM, fname, wdf, ierror = ier, exp_dim1 = exp_dim1, exp_dim2 = exp_dim2, $
         imp_dim1 = imp_dim1,  imp_dim2 = imp_dim2, raw_data = raw_data , helpme = helpme
  
;
     on_error, 1
@com.dat
common savewfm_picker, oldfilter, oldpath

     if n_elements(initwdf) eq 0 then wdfinit
;    check format
     ier = -1
     if keyword_set(helpme) then begin
        print, '% Format: READWFM [, FNAME] [, WDF] [, IERROR = ier]'
        print, '     [, EXP_DIM1 = exp_dim1]  [, EXP_DIM2 = exp_dim2]'
        print, '     [, IMP_DIM1 = imp_dim1]  [, IMP_DIM2 = imp_dim2]'
        return
     endif
      if n_elements(fname) eq 0 then fname = 0
      fname = fname[0]
      b = size(fname, /type)
      if b eq 7 then begin
        vers = strupcase(strmid(!version.os_family, 0, 3))
        if (vers eq 'UNI') then begin
;       get system variable substitution for unix systems
          temp = fname
          FULFNAM, temp, fname
        endif
        fname = file_expand_path(fname)
        if !version.os eq 'linux' then resolve_it, fname
        info = file_info(fname)
        if (info.exists + info.read) ne 2b then begin
           print, '% READWFM: File does not exist with read permission'
           print, '         File Name = ', fname
           print, '         File Exists =', info.exists
           print, '         Read Permission =', info.read
           ier = -2
           return
        endif
        openr,unit,fname, /get_lun, error = ier
      endif else begin
        if n_elements(oldpath) eq 0 then cd, current = path else path = oldpath
        fname = dialog_pickfile(/must_exist, filter = ['*.wfm;*.WFM','*.*'],  $
                /read, path = path, get_path= newpath)
        if strlen(fname) lt 1 then begin
          print, ' No File was Selected - RETURNING'
          return
        endif
        oldpath = newpath
        if !version.os eq 'linux' then resolve_it, fname
        openr,unit,fname, /get_lun, error= ier
      endelse
      filename = get_bare_filename(fname)
      if n_elements(wdf) eq 0 then wdf = maxwdfarray-1
      if ier ne 0 then begin
        print, '% READWFM:  Error opening file: ', fname
        return
      endif
      if wdf lt 1 or wdf gt maxwdfarray then begin
        print, '% READWFM:  WDF array must be in the range [ 1, ' + $
              strtrim (maxwdfarray, 2)+' ]'
        return
      endif

 b = intarr(2)
 readu, unit, b
 swap_end = (b[0] gt 0) ? 0 : 1
 free_lun, unit
 openr, unit, /get, fname, swap_endian = swap_end
 a = bytarr(10)
 readu, unit, a
 version = string(a[2:9])
 reads, strmid(version, 5), vers_numb
 
 number_of_digits = 1b
 readu, unit, number_of_digits
 numb_bytes_to_EOF = 0L
 readu, unit, numb_bytes_to_EOF
 numb_of_bytes_per_point =0b
 readu, unit, numb_of_bytes_per_point
 byte_offset = ulong(0)
 readu, unit, byte_offset
; Horizontal and Vertical Zoom information - not used
 junk = bytarr(20)
 readu, unit, junk

 waveform_label=bytarr(32)
 readu, unit, waveform_label
 waveform_label=string(waveform_label)
 n = ulong(0)
 readu, unit, n
 size_of_header = uint(0)
 readu, unit, size_of_header
 set_type = bytarr(4)
 readu, unit, set_type
 wfm_cnt = ulong(0)
 readu, unit, wfm_cnt
 junk = bytarr(24)
 readu, unit, junk

 wfm_update_spec = ulong(0)
 imp_dim_ref = ulong(0)
 exp_dim_ref = ulong(0)
 readu, unit, wfm_update_spec, imp_dim_ref, exp_dim_ref
 data_type = bytarr(4)
 skip = bytarr(16)
 readu, unit, data_type, skip
 curve_ref = ulong(0)
 readu, unit, curve_ref
 num_req_fast_fr = ulong(0)
 num_acq_fast_fr = ulong(0)
 readu, unit, num_req_fast_fr,  num_acq_fast_fr
; optional for :WFM#002 and higher 
;
 summary_frame_type = uint(0)
 if vers_numb ge 2 then  readu, unit, summary_frame_type
;
 pix_display_format = bytarr(4)
 readu, unit, pix_display_format
 Pix_max_value = ulong64(0)
 readu, unit, pix_max_value
 junk =  (version eq ':WFM#003') ? 1d0 : ulong(0)
 exp_dim1 ={scale: 1d0, offset: 1d0, size:ulong(0), units: bytarr(20), extent_min:1.0d0, $
         extent_max:1d0, resolution:1d0, ref_point:1d0, format: bytarr(4), $
         storage: bytarr(4), nvalue: 0L, over_range: 0L, under_range:0L, high_range: 0L, $
         low_range: 0L, user_scale: 1d0, user_units:bytarr(20), USER_offset: 1d0, $
         pt_density: junk, href: 1d0, trig_delay: 1d0}
 exp_dim2 = exp_dim1
 readu, unit, exp_dim1
 point_lun, -unit, position
 readu, unit, exp_dim2
 point_lun, -unit, position

 imp_dim1 ={scale:1d0, offset:1d0, dsize: ulong(0), units: bytarr(20), extend_min: 1d0, $
           extend_max: 1d0, resolution: 1d0, ref_point: 1d0, spacing: ulong(0), $
           user_scale: 1d0, user_units: bytarr(20), user_offset: 1d0, $
           point_density: junk, href: 1d0, trig_delay: 1d0}
 imp_dim2 = imp_dim1
 readu, unit, imp_dim1
 readu, unit, imp_dim2
 point_lun, -unit, position
 time_base1={real_pt_spac :ulong(0), sweep: bytarr(4), type_base1 : bytarr(4)}
 time_base2 = time_base1
 readu, unit, time_base1, time_base2

 point_lun, -unit, position
 real_pt_offset = ulong(0)
 tt_offset = 1d0
 frac_sec = 1d0
 gmt_sec = 1L
 readu, unit,  real_pt_offset, tt_offset, frac_sec ,gmt_sec

 state_flag= 0L
 typ_check_sum = bytarr(4)
 check_sum = 0
 start_offset = ulong(0)
 data_start_offset  = ulong(0)
 readu, unit, state_flag, typ_check_sum, check_sum, start_offset, data_start_offset

 point_lun, -unit, position
 post_start_offset = ulong(0)
 Post_stop_offset = ulong(0)
 end_of_curve_offset = ulong(0)
 readu, unit,  post_start_offset, Post_stop_offset, end_of_curve_offset
 curve_size = (post_start_offset - data_start_offset) / numb_of_bytes_per_point
 
 offset = byte_offset +data_start_offset*numb_of_bytes_per_point
 ityp = numb_of_bytes_per_point
 nop = curve_size
 point_lun, unit, offset
 array = make_array(nop, type = ityp)
 readu, unit, array
 if keyword_set (raw_data) then begin
   plot, array
   mm = min(array, max = mmm)
   print, '% READWFM: Minimum data:', mm
   print, '% READWFM: Maximum data:', mmm
   if raw_data lt 0 then begin
     ww = abs(round(raw_data))
     if (ww gt 0) && (ww le maxwdfarray) then i2w, array, ww
   endif
 endif
 point_lun, -unit, position
 free_lun, unit
 okover = where (array ge exp_dim1.over_range, nokover)
 okunder = where (array le -exp_dim1.over_range, nokunder)
 array = array*exp_dim1.scale+exp_dim1.offset
; time = findgen(nop)*imp_dim1.scale+imp_dim1.offset
 ylab = string(exp_dim1.units)
; xlab = string(imp_dim1.units)
 xlab = 'time (s)'
 t = [imp_dim1.offset, imp_dim1.scale]

 i2w, t, array, wdf, clab = filename, xlab = xlab, ylab = ylab, file = fname
 
END
