; ------------------------------------------------------------------------ 
; $Id: reasc.pro,v 1.21 2011/06/10 17:15:45 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
 Pro reasc, fname, narray, WDF, formatin = formatin, np = np, skip = skip, $
     csv = csv, xydata = xydata, endnumber = endnumber, sortx = sortx, $
     delimiter = delimiter, ierror = ier, limit = limit
;
;+
;    read an ascii file into wdf arrays
;    format is:  reasc, fname,narray,WDF, formatin=format, skip = skip, /csv, $
;                  xydata = xydata, endnumber = endnumber, sortx = sortx, $
;                  delimiter = delimiter, ierror = ierror
;    where 
;      fname   - the file name or a unit numberr
;                if a file name then the file is opened and closed at the end
;                if a number then the file is read directly
;                  if unit is equal to zero then pickfile will be used
;                  to determine a file
;      narray  - the number of arrays in the file 
;            for formatin = 0,1,2,3 -
;                each data record is of the form: x, y1, y2, y3, ..., y(narray) 
;                Note: This is one less than the number used for the procedure
;                 READASC because a single array can be read with READASC.
;            for formatin = 4 -
;                Each dataset is separate. 
;                  If narray is less than 1, then all data in the file will 
;                    be read if possible.  The labels are read and the number of 
;                    of columns are determined from the labels.
;                  Example:
;                      reasc, 'a1820.dat', -1, 1, formatin=4
;                      will store each dataset starting in wdf array 1
;                  If WDF is an array, then narray = n_elements(wdf)
;                Only MAXWDFARRAY datasets may be read.
;      WDF     - First WDF array or an array of NARRAY WDF arrays
;                WDF must be either a single value or an array of 
;                narray values
;      formatin - input format of the data
;               - if formatin lt 0 then first value in the file is read as NARRAY
;                   and formatin = abs(formatin)
;              0  only data values in the file maximum of np values
;                 See keyword LIMIT to read a certain number of points.
;              1  number of points followed by data values; this value
;                 overrides the values set by np
;              2  Narray label followed by data values
;              3  (DEFAULT) number of points followed by narray labels 
;                 then data values
;              4  EXCEL format
;                 (narray+1) labels in one line followed by (narray+1) columns:
;                  xlabel y1label y2label ... ynarray_label
;                  x, y1, y2, y3, ..., y(narray)
;              5  XDamp ascii format:
;                    dataset comment/title
;                    xlabel
;                    ylabel
;                    number of points = np
;                    xvalue yvalue (repeated np times) 
;                NOTE:  Some versions have "WAVEFORM" preceeding these values.
;                       This line will be automatically skipped.
;                Multiple datasets may be put into a file
;      np      - Number of points which are read
;      LIMIT   - If FORMATIN = 0, then LIMIT will specify the number of points.
;                Default is to read all points in the file until End-Of-File or ENDNUMBER
;      skip    - If the file has several lines of header information which
;                must be skipped, set skip equal to this number of lines.
;                (DEFAULT: SKIP = 0)
;      xydata  - If set then the data will be left on a nonuniform grid.
;      endnumber - If defined, reading will stop when this number is reached.
;                  Endnumber must be in both X and Y[0].
;      CSV       - If set, the comma will be the delimiter for the EXCEL array
;                  labels. If CSV is set then delimiter is ignored.
;                  NOTE:  If the labels contain spaces, CSV must be set.
;      delimiter - If present, this will be used as delimiter for the EXCEL
;                  array labels
;      sortx     - If set, Sort X values before saving.
;                  Ignored unless XYDATA is set.
;      ierror    - Zero if no error
;                - -1 if no file selected
;                - -2 if file does not exist or no read access
;                -  1 if error reading file
;                -  2 error in WDF arrays
;      If file contains labels, they are stored in the dataset comment
;    Example:
;    Read an array from unit 2 and store in wdf array 12. Assume format = 1
;       REASC, 2, 1, 12, format = 1
;    Read an unknown number of arrays from file tqdata.dat in format 2 with 
;       the number of waveforms as the first number and plot them.
;       REASC, 'tqdata.dat', n, 1, form=-2
;       plo, 1, n, /over
;    Read waveform data from a file selected by pickfile and store it in
;    WDF array 10.  Assume format = 2: label followed by data to EOF.
;       REASC, 0, 1, 10 , format= 2
;    Read 5 arrays from file 'PFIDL.dat', into WDF arrays 
;    2,3,4,5,6.  Assume data format 3.
;       REASC, 'PFIDL.dat', 5, 2
;    Read 3 arrays from file, 'testdata', into WDF arrays 3, 6, 9. Assume 
;    data has no header but only 4 columns of data.
;       REASC, 'testdata', 3, [3,6,9], form = 0
;-
;
     on_error, 1
@com.dat
common saveasc_picker, oldfilter, oldpath
     if n_elements(initwdf) eq 0 then wdfinit
;    check format
     ier = -1
      nparams = n_params()
      if nparams eq 0 then begin
        print, '% Format: REASC, FNAME_UNIT, NARRAY, WDF [, NP = np]'
        print, '     [, /xydata]  [, FORMATIN = format]  [, SKIP = skip]'
        print, '     [, ENDNUMBER = endnumber]'
        return
      endif
      if n_elements (formatin) eq 0 then formatin = 3
      if formatin lt 0 then narray = 0
      if formatin gt 5 then formatin = 3
      if n_elements(fname) eq 0 then begin
        print, ' File name must be specified by name or unit number'
        return
      endif
      np = 0L
      fname = fname[0]
      b = size(fname)
      if b[1] eq 7 then begin
        lcfile = 1
        vers = strupcase(strmid(!version.os_family, 0, 3))
        if (vers eq 'UNI') then begin
;       get system variable substitution for unix systems
          temp = fname
          FULFNAM, temp, fname
        endif
        fname = file_expand_path(fname)
        if !version.os eq 'linux' then resolve_it, fname
        info = file_info(fname)
        if (info.exists + info.read) ne 2b then begin
           print, '% REASC: File does not exist with read permission'
           print, '         File Name = ', fname
           print, '         File Exists =', info.exists
           print, '         Read Permission =', info.read
           ier = -2
           return
        endif
        openr,unit,fname, /get_lun
      endif else begin
        unit = round(fname)
        if unit eq 0 then begin
          if n_elements(oldpath) eq 0 then cd, current = path else path = oldpath 
          fname = dialog_pickfile(/must_exist, /read, path = path, get_path= newpath)
          if strlen(fname) lt 1 then begin
            print, ' No File was Selected - RETURNING'
            return
          endif
          oldpath = newpath
          lcfile = 1
          if !version.os eq 'linux' then resolve_it, fname
          openr,unit,fname, /get_lun
        endif else lcfile = 0
      endelse
      if lcfile eq 1 then filename = file_expand_path(fname) else begin
        fs = fstat(unit)
        filename = file_expand_path(fs.name)
      endelse
      ier =2
      if n_elements(wdf) eq 0 then begin
        print,'WDF array value must be provided'
        print, '% Format: REASC, FNAME_UNIT, NARRAY, WDF '
        return
      end
      wdfar = wdf[*]
      if min(wdfar) lt 1 or max(wdfar) gt maxwdfarray then begin
        print, 'wdf array must be greater than 0 and </=', maxwdfarray
        if lcfile eq 1 then free_lun, unit
        return
      endif
      if n_elements(narray) eq 0 then begin
        print, ' Number of arrays must be specified'
        if lcfile eq 1 then free_lun, unit
        return
      endif
      if n_elements (skip) eq 0 then skip = 0
      junkstr = ''
      ier =1
      if skip gt 0 then for i = 1, skip do readf, unit, junkstr
      if formatin lt 0 then begin
        narray = 0L
        readf, unit, narray
        narray = round(narray)
        formatin = -(formatin)
      endif
      if (narray lt 1) and (formatin ne 4) then begin
        print,  '% REASC: NARRAY must be greater than zero.'
        if lcfile eq 1 then free_lun, unit
        return
      endif 
      nel = n_elements(wdfar)
;      if n_elements (xydata) eq 0 then xydata = 1
    if formatin le 4 then begin
      if n_elements (endnumber) gt 0 then begin
        lcend = 1
        endnumber = endnumber[0]
      endif else lcend = 0
;
      ier = 2
      if formatin eq 4 then begin
        str = ''
        readf, unit, str
        if keyword_set(csv) then delimiter = ','
        if n_elements (delimiter) eq 0 then $
            xlab = strsplit(str,  /extract, regex=0, count = nel) else $
            xlab = strsplit(str, strmid(delimiter[0], 0, 1), /extract, $
              count = nel, /preserve, regex=0)
;        parser, str, xlab, nel, /stringarray
        if narray lt 0 then narray = nel-1
        if narray lt 1 then begin
          print, '% REASC: NARRAY must be greater than zero for EXCEL data.'
          if lcfile eq 1 then free_lun, unit
          return
        endif
        if nel lt narray+1 then begin
          print, '% REASC: Missing labels'
          help, nel, narray
          help, str, xlab
          if lcfile eq 1 then free_lun, unit
          return
        endif
;        okc = get_match(',$', xlab, nelements =nokc)
;       Strip off trailing comma if comma delimited
;        if nokc gt 0 then $
;          for i = 0, nokc-1 do $
;            xlab[okc[i]] = strmid(xlab[okc[i]], 0, strlen(xlab[okc[i]])-1)
      endif
      nel = n_elements(wdfar)
      if nel eq 1 and narray gt 1 then wdfar=indgen(narray)+wdfar(0) else begin
        if nel gt narray then print, $
            ' Warning: only ', strtrim(narray, 2),' arrays will be filled'
        if nel lt narray then begin
          print, ' Only ', strtrim(string(nel), 2), $
          ' arrays have been provided for ', strtrim(string(narray), 2),' arrays'
          print, ' Please try again'
          if lcfile eq 1 then free_lun, unit
          return
        endif
      endelse
      if  max(wdfar) gt maxwdfarray then begin
        print, 'wdf array must be  </=', maxwdfarray
        if lcfile eq 1 then free_lun, unit
        return
      endif
      maxnp = 0L
      on_ioerror, badnp
      if formatin eq 1 or formatin eq 3 then readf, unit, maxnp
      if formatin eq 0 and n_elements(limit) gt 0 then maxnp = abs(limit[0])
      if maxnp le 0 then begin
        np = 4096L
        lcnp = 1
        maxnp = np 
      endif else lcnp = 0
; Time to start reading
      name = make_array (narray, /string)
      if formatin eq 4 then begin
        name = xlab[1:narray]
        xlab = xlab[0]
      endif else if formatin  gt 1 then begin
        for i=0, narray-1 do begin
          string = ''
          readf, unit, string
          name(i) = string
        endfor
      endif 
      if formatin eq 0 or max(strlen(name)) eq 0 then begin
        fs = fstat(unit)
        fnam = get_bare_filename(fs.name)
        name[0] = fnam
        for i = 1, narray-1 do name[i]= fnam +'-'+strtrim(i, 2)
      endif
      ax =make_array(maxnp)
      bx = make_array (narray, maxnp)
      y = make_array (narray)
      i=0L

      on_ioerror, badnum

      while (not eof(unit))  do begin
        readf,unit,x, y
        if keyword_set(lcend) then if ((x eq endnumber) and (y[0] eq endnumber)) $
           then goto, badnum
        if i eq maxnp-1 then begin
          if lcnp eq 0 then begin
            ax(i) = x
            bx(*, i) = y
            i += 1L
            goto, badnum
          endif
          bx = reform (bx, maxnp*narray,/overwrite)
          maxnp = maxnp+maxnp
          bx =[bx,bx]
          bx = reform (bx, narray, maxnp, /overwrite)
          ax =[ax, ax]
        endif
        ax(i) = x
        bx(*, i) = y
        i += 1L
      endwhile
badnum:
      if lcfile eq 1 then free_lun, unit
      if i eq 0 then begin
        np = 0
        print, ' Error reading file -- No data found'
        ier =1
        return
      endif
      np = i
      npm= np-1
      ax= ax(0:npm)
      bx = temporary(bx(*, 0:npm))
      if keyword_set(xydata) and keyword_set(sortx) then begin
        sx = sort(ax)
        ax = ax[sx]
        bx = bx[*, sx]
      endif
      bx = transpose (bx)

; convert to wdf array using the lowres option
      for i = 0, narray-1 do $
         i2w, ax, bx[0:npm, i], wdfar[i], c = name[i], /lowres, $
           xydata = xydata, file = filename, xlab = xlab
      ier =0
      return
    endif
;
;     damp format for ascii file
;     comment, x-label, y-label, npoints, data
;
      ier =2
      if nel eq 0 then begin
        print, 'WDF array number must be specified for at least first array'
        return
      endif
      if nel eq 1 then begin
        if narray gt 1 then wdfar=indgen(narray)+wdfar(0) else begin
          narray = maxwdfarray - wdfar(0)+1
          wdfar=indgen(narray)+wdfar(0)
        endelse  
      endif else narray = nel
      if (max(wdfar, min = mm) gt maxwdfarray) or (mm lt 1) then begin
        print, 'wdf array must be >0 and </=', maxwdfarray
        print, 'Specified values are:'
        print, wdfar
        return
      endif
      ndset = 0L
      lcend = 0L
damp_st:
      comment = ' '
      xlabel  = ' '
      ylabel  = ' '
      np = 0L
      if ( not eof(unit)) then readf, unit, comment else goto, returnxdamp
      if comment eq 'WAVEFORM' then $
        if ( not eof(unit)) then readf, unit, comment else goto, returnxdamp
      if ( not eof(unit)) then readf, unit, xlabel else goto, returnxdamp
      if ( not eof(unit)) then readf, unit, ylabel else goto, returnxdamp
      on_ioerror, badnp
      readf, unit, np
      xx = make_array(np)
      yy = make_array(np)
; Time to start reading
      i=0L
      while ((not eof(unit)) and i lt np) do begin
        on_ioerror, baddnum
        readf,unit,x, y
        xx(i) = x
        yy(i) = y
        i += 1
      endwhile
; convert to wdf array using the lowres option
      if keyword_set(xydata) and keyword_set(sortx) then begin
        sx = sort(xx)
        xx = xx[sx]
        yy = yy[sx]
      endif

      i2w, xx, yy ,wdfar(ndset), clab = comment, /lowres, $
          xlab = xlabel, ylabel = ylabel, xydata = xydata, $
          file = filename
      ndset += +1
      if ndset lt narray then goto, damp_st
returnxdamp:
      if lcfile eq 1 then free_lun, unit
      ier =0
      return

baddnum:
      print, ' Error reading file for dataset: ', comment
      print, i, ' Values found, Expected ', strtrim(np, 2)
      if lcfile eq 1 then free_lun, unit
      ier =1
      return

badnp:
      if lcfile eq 1 then free_lun, unit
       print, ' Error reading number of points -- Check format of data '
      ier =1
      end
