; ------------------------------------------------------------------------
; $Id: rgb2true.pro,v 1.2 2010/09/17 21:20:35 lpmix Exp $
; ------------------------------------------------------------------------
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
;+
; NAME:
;       RGB2TRUE
;
; PURPOSE:
;       Convert a indexed color image with an RGB lookup table to a true color image.
;
; CATEGORY:
;       Utility
;
; CALLING SEQUENCE:
; 
;       TRUE_COLOR_IMAGE = RGB2TRUE (INDEXED_IMAGE [, RED, GREEN, BLUE] )
;  or 
;       TRUE_COLOR_IMAGE = RGB2TRUE (INDEXED_IMAGE [, COLOR_TABLE] )
;
; INPUTS:
;       INDEXED_IMAGE :	Byte array of the indexed image with dimensions W x H
;                 where W and H are the width and heigth of the image in pixels.
;               Values in the range of zero to 255.
;               Note: Gray scale images do not have a color table
;                     Images without a color table are rejected in this version
;               Maximum value must be less than the number of elements in 
;                 RED, GREEN, and BLUE
;    NOTE: All parameters except INDEXED_IMAGE are optional if keyword, COLOR_TABLE
;            or keywords, RED, GREEN, and BLUE, are present
;       RED:    Byte array with RED intensity for the indexed colors
;               Values in the range of zero to 255.
;               Must have at least one more value than the maximum of INDEXED_IMAGE
;       GREEN:  Byte array with GREEN intensity for the indexed colors
;               Values in the range of zero to 255.
;               Must have at least one more value than the maximum of INDEXED_IMAGE
;       BLUE:   Byte array with BLUE intensity for the indexed colors
;               Values in the range of zero to 255.
;               Must have at least one more value than the maximum of INDEXED_IMAGE
;       COLOR_TABLE:  Some image readers provide a color table rather than individual 
;               colors in the colortable.  COLOR_TABLE = BYTARR [ncolors, 3]
;               COLOR_TABLE = reform([RED, GREEN, BLUE], n_elements(r), 3)
;	
; KEYWORD PARAMETERS:
; NOTE: The first 4 keywords are ignored if two parameters are present.
;       If COLOR_TABLE is present, RED, GREEN and BLUE are ignored.
;       COLOR_TABLE: Keyword option for COLOR_TABLE parameter
;                    If the keyword is a 1-D array then it must have 3*N elements
;                    and it is assumed = [RED, GREEN, BLUE] 
;       RED:    Keyword option for RED parameter
;       GREEN:  Keyword option for GREEN parameter
;       BLUE:   Keyword option for BLUE parameter
;       ERROR:  If no error occurs, ERROR = 0
;       SWAP:   If set, put the first value in the RGB arrays to: (255 - original_value)
;               If SWAP is negative, set the first value in the RGB arrays to ABS(SWAP)
;               SWAP must be greater than -256
;       INDEX:  If set and in the range of 1 to 255, SWAP will be applied to this index 
;                 in the RGB arrays.
;       FAST:   If set, the indexed image will be converted to a true color image of the
;                 form, [WIDTH, HEIGHT, 3]
;               For some arrays, this results in a factor of two speedup.
;               Default: Output = TRUE_COLOR_IMAGE [3, WIDTH, HEIGHT]
;       HISTOGRAM: If set and a valid WDF array number, save the histograme in the
;                 WDF array specified by HISTOGRAM.  The X axis will go from 0 to 255.
;
; OUTPUTS:
;       TRUE_COLOR_IMAGE: True color image of the indexed image.
;       Dimensions are [3, WIDTH, HEIGHT] if FAST is not set or 
;         [WIDTH, HEIGHT, 3] if FAST is set.
;
; PROCEDURE:
;       1.  Verify consistent input parameters
;       2.  Use HISTOGRAM to determine image values
;       3.  Use the RGB values and histogram to generate the RED, GREEN and BLUE images
;
; EXAMPLE:
;       Read a indexed PNG file with a color table and convert it to a true color image
;       Plot the image
;       Assume the PNG file is in the current directory in a file, picture.png
;         IMAG = READ_PNG ( 'PICTURE.PNG', RED, GREEN, BLUE)
;         TC_IMAG = RGB2TURE(IMAG, RED, GREEN, BLUE)
;         TV, TC_IMAG, /TRUE
; or 
;         IMAG = READ_PNG ( 'PICTURE.PNG', RED, GREEN, BLUE)
;         TC_IMAG = RGB2TURE(IMAG, RED, GREEN, BLUE, /FAST)
;         TV, TC_IMAG, TRUE = 3
;
;       Convert an indexed image, A, with color table given by R, G, B, to a true color
;        image and store the result in AT; store the histogram in WDF array, 61.
;         AT =  RGB2TURE(A, R, G, B, HIST = 61)
; or
;         AT =  RGB2TURE(A, COLO= [R, G, B], HIST = 61)
;
;       Assume that the above image shows a black background and the histogram shows the highest
;         value in the zero bin.  Convert the image to ATS with the background swapped.
;         ATS =  RGB2TURE(A, R, G, B, HIST = 61, /SWAP)
;
; MODIFICATION HISTORY:
; 	Written by:    L. P. Mix, Jr., March 30, 2010
;
;-

function  rgb2true, v, rin, gin, bin, fast = fast, swap = swap, error = error, $
       index = index, histogram = hist, color_table = coltab, red = red, green= green, blue = blue
;
on_error, 1
npar = n_params()
error = 1
if npar eq 0 then begin
  print, '% FORMAT:  IMAGE = RGB2TRUE( IMAGE, RED, GREEN, BLUE [, FAST = fast]'
  print, '   [, SWAP = swap] [, ERROR = error] [, INDEX = index] [, HISTOGRAM = hist]'
  print, '   [, COLOR_TABLE = coltab] [, RED = red] [, GREEN= green] [, BLUE = blue]'
  return, 0
endif
;
n = n_elements(v)
if n eq 0 then begin
  Print, '% RGB2TRUE: Image is undefined'
  return, 0
endif
s = size(v)
if (s[0] ne 2) || (s[3] ne 1)  then begin
  print, '% RGB2TRUE: Input image array must be a two dimensional byte array.'
  image = v
  help, image
  image = 0
  return, image
endif
if npar eq 1 then begin
  if keyword_set(coltab) then begin
    sct = size(coltab)
    if ((sct[0] eq 2) && (sct[1] ge 2) && (sct[2] eq 3)) then begin
      rin = coltab[*, 0] 
      gin = coltab[*, 1] 
      bin = coltab[*, 2] 
    endif else begin
      ncoltab = n_elements(coltab)
      if ncoltab mod 3 eq 0 then rin = reform(coltab, ncoltab/3, 3, /over) else begin
        print, '% RGB2TRUE: Inadequate keyword color_table.'
        help, coltab
        return, 0
      endelse
    endelse
  endif else if (n_elements(red) gt 2) && (n_elements(green) gt 2) && $
    (n_elements(blue) gt 2) then begin
    rin = red
    gin = green
    bin = blue
  endif else begin
    print, '% RGB2TRUE: Color information must be provided.'
    return, 0
  endelse
endif
sct = size(rin)
if sct[0] eq 2 then begin
  if (sct[1] ge 2) && (sct[2] eq 3) then begin
    mnct = sct[1]
    r = reform(rin[*, 0])  
    g = reform(rin[*, 1])  
    b = reform(rin[*, 2])  
  endif else begin
    print, '% RGB2TRUE: Inadequate color table.'
    help, rin
    return, 0
  endelse
endif else begin
  nr = n_elements(rin)
  ng = n_elements(gin)
  nb = n_elements(bin)
  mnct = min([nr, ng, nb])
  if mnct lt 2 then begin
    print, '% RGB2TRUE: Inadequate number of elements in the R, G, and B arrays'
    print, '            Sizes: '+strtrim(nr,2)+', '+strtrim(ng,2)+', '+strtrim (nb, 2)
    return, 0
  endif 
  r = rin & b = bin & g = gin
endelse
mxv = max(v, min = mnv)
if (mnv lt 0) || (mxv ge mnct) then begin
  print, '% RGB2TRUE: Bad Image array'
  print, '    Range of image data is ['+strtrim(mnv,2)+', '+strtrim(mxv,2)+']'
  Print, '    Color table size = '+strtrim(mnct, 2)
  return, 0
endif


if keyword_set(swap) then begin
  if keyword_set(index) then begin
    if (index gt 0) && (index lt mnct) then ind = index[0] else ind = 0
  endif else ind =0
  if (swap lt 0) && (swap ge -255) then begin
    r[ind]= -swap
    g[ind]= -swap
    b[ind]= -swap
  endif else begin  
    r[ind]= 255B-r[ind]
    g[ind]= 255B-g[ind]
    b[ind]= 255B-b[ind]
  endelse
endif
ind = bytarr(n)
h = histogram (v, reverse_indices =  ri)
if n_elements(hist) gt 0 then begin
  hist = hist[0]
  if (hist gt 0) && (hist le get_maxwdf()) then $
    i2w, h, hist, clab= 'Image Histogram', xLab = 'Byte Value'
endif
ok = where (h gt 0, nok)
for i = 0, nok-1 do ind[ri[ri[ok[i]]: ri[ok[i]+1]-1]] = ok[i]
if keyword_set(fast) then begin
  x = r[ind]
  y = g[ind]
  z = b[ind]
  ind =0
  error =0
  return, reform([x, y, z], s[1], s[2], 3, /over)
endif
vt = bytarr(3, n)
vt[0, *] = r[ind]
vt[1, *] = g[ind]
vt[2, *] = b[ind]
ind =0
error =0
return, reform(vt, 3, s[1], s[2], /over)
end
