; ------------------------------------------------------------------------ 
; $Id: rmbase.pro,v 1.3 2007/06/06 20:03:53 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
 Pro  rmbase, wdf1, wdf2,first,last,base=base,xr=xr, $
            quiet = quiet, noplot=lcplot, tsh=tsh
;
;+
; NAME:
;       RMBASE
;
; PURPOSE:
;       Remove a complex baseline offset from a set of waveform data 
;       Store the result in the initial WDF array
; CATEGORY:
;       Math 
; CALLING SEQUENCE:
;       RMBASE, WD1, WD2, FIRST, LAST
; INPUTS:
;       WD1       WDF array number of the dominant part of the baseline
;       WD2       WDF array number of the complete baseline
;       FIRST     First WDF array number of the composite signal
; OPTIONAL INPUT:
;       LAST      Last WDF array number of the composite signal.  If omitted,
;                 only the first array is done.
; KEYWORD INPUT:
;       BASE      If present, specifies a uniform baseline used for comparison
;       XR        If present, specifies the range of X-values plotted 
;       QUIET     If present and not zero then the comparisons and each 
;                 subtracted waveform are not plotted
;       NOPLOT    same as quiet (If either is set, no data is plotted.)
;       TSH       If present, specifies the waveforms are to be shifted to the
;                 timing of WD1 as determined by the comparison
; OUTPUTS:
;       NONE    The subtracted signal is stored in the same WDF array
;               in the common block
; COMMON BLOCKS:
;       COM.DAT  WDF common block 
; SIDE EFFECTS:
;       WDF arrays maxwdfarray-1 and maxwdfarray are used as scratch arrays
; PROCEDURE:
;       The COM procedure is used to calculate the amplitude and time shift 
;       be used between WD1 and the composite array
;       WD2 is then scaled and shifted and subtracted from the composite array
; EXAMPLES:
;       WDF array 2 has a large x-ray spike between 10 and 50 ns which is 
;       recorded on a detector.  WDF arrays 3 through 10 in addition to the 
;       x-ray signal have ion signals.  Separate the ion and x-ray signals.
;               xfr, 2, 1
;               lim, 1, 10e-9, 50e-9
;               rmbase, 1, 2, 3, 10
; MODIFICATION HISTORY:
;      Original version, L. P. Mix, May 19, 1992
;-
;
@com.dat
;
      if n_elements(initwdf) eq 0 then wdfinit
      nparams = n_params()
      if nparams lt 3 then begin
        print,' RMBASE must have at least 3 parameters'
        return
      endif
;
      if (n_elements(wdf1) eq 0 or  n_elements(wdf2) eq 0 or $
        n_elements(first) eq 0)then begin
        print, 'Parameters to RMBASE must be defined'
        return
      endif
      if n_elements (last) eq 0 then last = first
      
      if wdf1 lt 1 or wdf1 gt maxwdfarray then begin
        print, 'wdf array must be greater than 0 and </=', maxwdfarray
        return
      endif
      if wdf2 lt 1 or wdf2 gt maxwdfarray then begin
        print, 'wdf array must be greater than 0 and </=', maxwdfarray
        return
      endif
      if first  lt 1 or first gt maxwdfarray then begin
        print, 'wdf array must be greater than 0 and </=', maxwdfarray
        return
      endif
      if last lt first or last gt maxwdfarray then begin
        print, 'Last wdf array must be greater than',first,' and </=', $
           maxwdfarray
        return
      endif
;
      if keyword_set(lcplot) then lcplot = 1 else lcplot = 0
      if keyword_set(quiet) then lcplot = 1
      if wdfnp(wdf1-1) eq 0 then begin
        print, 'No data in the first array'
        return
      endif
;
      if wdfnp(wdf2-1) eq 0 then begin
        print, 'No data in the second array'
        return
      endif
      if n_elements(base) eq 0 then base = 0
      for i = first , last do begin
        if wdfnp(i-1) ne 0 then begin
          if base eq 0 then $
          com, wdf1, i, quiet=lcplot, amp = amp, bline = bline, tshift=tshift $
           else $
          com, wdf1, i, quiet=lcplot, amp=amp, bline=bline, tshift=tshift, /base
          xfr, wdf2, maxwdfarray
          tsh, maxwdfarray, tshift
          xfr , i, maxwdfarray-1
          lab , maxwdfarray-1 , 'Signal+Backgr'
          lab, maxwdfarray,'Backgr'
          if amp ne 0 then div, maxwdfarray, 0, amp else mul, maxwdfarray, 0, amp
          if base ne 0 and amp ne 0 then begin
             add , maxwdfarray-1, 0 , (bline(0)/amp)
             add , i, 0 , (bline(0)/amp)
          endif
          sub, i, maxwdfarray
          if n_elements(tsh) gt 0 then if (tsh ne 0) then $
              tsh,[i,maxwdfarray,maxwdfarray-1], -tshift
          if lcplot eq 0 then begin
            hak, /mes
            plo, [i, maxwdfarray, maxwdfarray-1], /ov, xr= xr
            if (i ne last ) then hak, /mes
          endif
          if amp eq 0 then print, 'Comparison failed for array',i
        endif else print, ' No valid data in array', i
      endfor
      return
      end
