; ------------------------------------------------------------------------ 
; $Id: sbf2pff.pro,v 1.1 2010/03/03 18:26:23 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
;+
; NAME:
;       SBF2PFF
;
; PURPOSE:
;
;       Read a standard Sandia Binary Format file into a wdf array
;
; CALLING SEQUENCE:
;
;       SBF2PFF [, DIRECTORY] [, OUTFILE]
;
; INPUTS:
;       DIRECTORY: The directry name which contains the files for conversion
;                 If this is not a string, DIALOG_PICKFILE will be used to 
;                 select the directory
;       OUTFILE:  String for the output file name
;                 OUTFILE may contain path information.
;                 If OUTFILE is missing the the default is used.
;                 DEFAULT:  OUTFILE = DIRECTORY +'.pff'
;                 where DIRECTORY is the directory containing the files.
;                 Only the current directory is used without path informaton.
;
; KEYWORD PARAMETERS:
;       OUTPATH:  Path for OUTFILE
;       FILTER:   Filter for the output datasets
;                 GET_MATCH is used to select only those datasets specified.
;       NDSET:    Number of datasets
;       IERROR:   Error Flag 
;                 If zero, no error occurred
; OUTPUTS:
;       The datasets in the directory will be stored in a pff file.
;
; PROCEDURE:
;       The required OUTFILE is opened.
;       A list of all files in the specified directory is determined.
;       Each File is read and the dataset is stored in (GET_MAXWDF() -1)
;
; EXAMPLE:
;       Convert the files in the directory, 'pbfa2z01234' and store the result in 
;       pbfa2z01234.pff in /home/lpmix
;       SBF2PFF, 'pbfa2z01234', out = '/home/lpmix'
; <or>  SBF2PFF, 'pbfa2z01234', '/home/lpmix/pbfa2z01234.pff'
;
;       Convert the files in the directory, '/data/pbfa01234', to PFF file
;         '/home/lpmix/ntof01234.pff' that begin with 'ntof'
;       SBF2PFF, '/data/pbfa01234', '/home/lpmix/ntof01234.pff', filter = '^NTOF'
;
; MODIFICATION HISTORY:
;       Written by:     L. P. Mix, Jr., February 19, 2010
;-
;
 Pro sbf2pff, filename, outfile, outpath = outpath, filter = filter, ndset = ndset, $
     ierror = ier
;
     on_error, 1
@com.dat
     if n_elements(initwdf) eq 0 then wdfinit
;    check format
     ier = -1
     np = 0L
     fname = (n_elements(filename) eq 0) ?  0 : filename[0]
     sep = path_sep()
     b = size(fname, /type)
     if b eq 7 then begin
       vers = strupcase(strmid(!version.os_family, 0, 3))
       if (vers eq 'UNI') then begin
;      get system variable substitution for unix systems
         temp = fname
         FULFNAM, temp, fname
       endif
       fname = file_expand_path(fname)
       if !version.os eq 'linux' then resolve_it, fname
       info = file_info(fname)
       if (info.exists + info.read + info.directory) ne 3b then begin
          print, '% SBF2PFF: Directory does not exist with read permission'
          print, '           File Name = ', fname
          print, '           File Exists =', info.exists
          print, '           Read Permission =', info.read
          print, '           Directory =', info.directory
          ier = -2
          return
       endif
     endif else begin
       pick_title = 'Please select a directory to convert'
       fname = dialog_pickfile(/must_exist,/directory, title = pick_title)
       if strlen(fname) lt 1 then begin
         print, ' No Directory was Selected - RETURNING'
         return
       endif
       if !version.os eq 'linux' then resolve_it, fname
     endelse
     l = strlen(fname)
     i = strpos(fname, sep, /reverse_search)
     if i lt l-1 then fname = fname + sep
     ls, fname+'*',  /quiet, current = current, nfiles = nelem
     if nelem eq 0 then begin
       print,  '% SBF2PFF: No file found with search string: '
       print,  '       "'+fname+'*"'
       return
     endif
     if n_elements(filter) gt 0 then begin
       filter = filter[0]
       files = file_basename(current)
       ok = get_match(filter, files, nelem= nelem)
       if nelem gt 0 then current= current[ok] else begin
         print,  '% SBF2PFF: No files found with filter string: '
         print,  '       "'+filter+'"'
         return
       endelse
     endif
     if n_elements(outfile) gt 0 then output = outfile[0] else begin
       l = strlen(fname)
       output = strmid(fname, 0, l-1)
       i  = strpos(output, sep, /reverse_search)
       output = strmid(output, i+1)+'.pff'
     endelse
     if n_elements(outpath) eq 0 then outpath = '' else begin
       outpath = outpath[0]
       l = strlen(outpath)
       i = strpos(outpath, sep, /reverse_search)
       if i lt l-1 then outpath= outpath+sep
     endelse
     output = outpath+output
     createpff, output, error = ier
     if ier ne 0 then begin
       print,  '% SBF2PFF: Unable to open output PFF file: '
       print,  '       "'+output+'*"'
       return
     endif
     saveit = maxwdfarray-1
     for i = 0, nelem-1 do begin
       readsbf,current[i], saveit
       wri, saveit
     endfor
     closepff
end
