; ------------------------------------------------------------------------
; $Id: sph_spec.pro,v 1.10 2011/03/25 20:42:59 lpmix Exp $
; ------------------------------------------------------------------------
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
;+
; NAME:
;       SPH_SPEC
;
; PURPOSE:
;       Calculate the central rays on a spherical spectrometer
;       Calculations assume a spherical crystal.  The center of the crystal is assumed 
;         to be at y=0 with the X-axis parallel to the surface/crystal planes and the
;         positive Y-axis is perpendicular and going away from the crystal.
;
; CATEGORY:
;       Analysis
;
; CALLING SEQUENCE:
;
;       SPH_SPEC
;	
; KEYWORD PARAMETERS:
;       WDINIT:  First storage location for WDF arrays
;                If undefined or not valid, WDINIT = GET_MAXWDF()-1-N_ELEMENTS(OUTPUT)
;                NOTE:  The middle photon from the source to the center of the crystal
;                       is stored in GET_MAXWDF().
;                OUTPUT values are defined below.
;       OUTPUT:  If present, this indicates the desired output waveforms.
;                OUTPUT may be a scalar or a vector.  
;                If OUTPUT is 0 (ZERO), no waveforms are output.
;                DEFAULT: OUTPUT = 1+indgen(13) ; all 13 waveforms listed below are saved
;                These wareforms are:
;                1.  Crystal Surface
;                2.  Crystal Bragg Angle vs Crystal X
;                3.  X-ray Wavelength vx Crystal X
;                4.  Energy vs Crystal X
;                5.  Film Position
;                6.  Source-Xtal Dist vs Wavelength
;                7.  Xtal-Film Dist vs Wavelength
;                8.  Magnification vs Film Position
;                9.  Magnification vs Wavelength
;               10.  Magnification vs Photon Energy
;               11.  Angle to Normal vs Film Position
;               12.  Wavelength vs Film Position
;               13.  Photon Energy vs Film Position
;               14.  Middle ray
;               15.  Transverse Magnification vs Crystal-X 
;       DATABASE: If present, read a CSV file with geometry data for spectrometers.
;                This information is stored in a common block with the default values.
;                The value may be set with "SET_DEFAULT" in the IDL startup.
;                If DATABASE is not a string, then DIALOG_PICKFILE will be called.
;                SHOW_DEFAULT may be used to see the CSV database file name.
;       GEOMETRY: If present, use a geoometry label from the spectrometer database.
;                Input keyword values will ignored and replaced by values from the database.
;                If GEOMETRY is valid, values not present in the DATABASE, will be set to
;                the "SAVED" values.
;                See:  SPH_SPEC, /SHOW_DEFAULT to see these values 
;                 and:  SPH_SPEC, /SET_DEFAULT to set these values or /SAVEIT
;       GUI:     If set, use a gui to set spectrometer parameters.
;                Default:  GUI =0 if any of the following specified:
;                  RADIUS, CRY2D, ORDER, DISTANCE, THETA, FILM, GEOMETRY
;                Note: If GEOMETRY does not select a unique data then GUI = 1
;                Note: Input values can be changed by the GUI.
;                Note: If the source is moved, the film position should be adjusted if 
;                      the standard position is desired (HIT the STANDARD POSITION button)
;       SET_DEFAULT:  Set the default spectrometer values.
;                The following values may be set:
;                Crystal
;                  RADIUS - Default = 150d0
;                  CRY2D  - Default = 6.687
;                  ORDER  - Default = 1
;                  LENGTH`- Default = 50.0
;                  WIDTH  - Default = 30.0
;                Source
;                  Distance - Default = 300d0
;                  THETA    - Default = 34.0d0
;                  NPOINTS  - Default = 1001L
;                Detector
;                  FILM     - Default = 0
;                Output
;                  PLOTIT   - Default = 1
;                  TRUNCATE - Default = 0
;                  VERBOSE  - Default = 1
;       SAVEIT:  This keyword, if set, is similar to SET_DEFAULT, but the parameters saved
;                are the parameters of the calculation and are independent of the input 
;                values.
;       SHOW_DEFAULT: If set, show the default values for the keywords listed 
;                under SET_DEFAULT.
;       CALCULATE: If set, do the spherical calculations.  This is ignored unless
;                either SET_DEFAULT or SHOW_DEFAULT is set.
;                Default: CALCULATE = 0
;       RADIUS:  Radius of the spherical crystal in mm
;       CRY2D:   Crystal 2D spacing (angstroms)
;                Twice the crystal plane spacing
;                Values from D. B Sinars, et al.
;                Ge 422
;                  CRY2d = 2.31
;                Ge 220
;                  CRY2d = 4
;                HOPG
;                  CRY2d = 3.36
;                Mica I
;                  CRY2D = 19.9149
;                  Higher orders, 2 to 8, have effective spacing of:
;                  (9.9575, 6.6383, 4.9787, 3.9830, 3.3192, 2.8450, 2.4894)
;                Quartz 1010 
;                  CRY2D = 8.512
;                Quartz 1011 
;                  CRY2D = 6.687
;                Quartz 1012 
;                  CRY2D = 4.564
;                Quartz 1120
;                  CRY2D = 4.912
;                Quartz 1122
;                  CRY2d = 3.636
;                Quartz 2020
;                  CRY2d = 4.246
;                Quartz 2023
;                  CRY2d = 2.749
;                Quartz 2131
;                  CRY2d = 3.082
;                Quartz 2243
;                  CRY2d = 2.030
;                Quartz 3140
;                  CRY2d = 2.3604
;                Quartz 5052
;                  CRY2d = 1.625
;       ORDER:   Diffraction order, N where N*Lambda = 2D*SIN(THETA)
;                Default: ORDER = 1
;       LENGTH:  Length of crystal in dispersion direction (mm).  
;                Minimum Length = 1
;                DEFAULT:  LENGTH = 50.0 
;       WIDTH:   Width of the crystal in the imaging direction (mm)
;                Minimum Width = 1
;                DEFAULT:  WIDTH = 30.0
;       DISTANCE: Distance to the source in mm from the center of the crystal.
;  Source Angle:  The next 3 keywords are redundant - only one is required.
;       THETA:   Angle to the source in degrees from the center of the crystal.
;                This is the Bragg angle for the xray striking the center of the crystal
;                Note:  If TCO is given, then THETA = 90-TCO
;       TCO:     Angle between the crystal normal and the source in degrees.
;                The Bragg angle, THETA, is (90-TCO)
;                Note:  If THETA is present, TCO is ignored.
;       ENERGY:  Energy of XRAYS at X=0 on the crystal (keV)
;                This is used to calculate the Bragg angle at X=0 (THETA).
;                This keyword is ignored if either THETA (BRAGG Angle) or TCO are 
;                specified.                
;                  THETA = ASIN(PHYCONSTF(/AELAMBDA)/ENERGY*ORDER/CRY2D) *180d0/!dpi
;                NOTE:  ORDER and CRY2d should be provided.  If not, the saved values are 
;                       used.  If GUI or GEOMETRY is set, the source angle may be changed to a 
;                       different energy by these keywords.
;  End Source Angle ***
;       NPOINTS: Number of discrete rays used for the calculation.
;                Note: An odd number of points is always used.
;                      NPOINTS = (LONG(NPOINTS)/2)*2+1 > 11
;                 Default: NPOINTS = 1001L
;       FILM:    If film has 3 or more values, these values are used to position the 
;                  film plane.  FILM[0] and FILM[1] are X and Y locations of one point on the 
;                  film plane.  FILM[2] is the angle in radians of the film plane to the 
;                  horizontal axis or the arctangent of the slope of the film.
;                If the input value is undefined, then FILM = FILM-DEFAULT  (See SET_DEFAULT)
;                If film has less than 3 elements the film is put in the standard position
;                  and these values are returned.  If X0 and Y0 are the source positions, 
;                  then FILM[0] = 0.0, FILM[1] = RADIUS, FILM[2] = ATAN(Y0 - RADIUS, X0)
;                Units are MM for FILM[0:1] and RADIANS for FILM[2].
;                Note:  The film position is considered standard if the slope to the source 
;                       equals tan(film[2]) to a relative error of less than 1e-5.
;       PLOTIT:  If set, plot the geometry.  This may be set with "/SET_DEFAULT"
;                Default:  PLOTIT = PLOTIT_SAVE unless VERBOSE is set on the command line
;                If KEYWORD_SET(VERBOSE)  then PLOTIT = 1
;                The routine PLOT_SPEC may also be used to plot the geometry after running 
;                SPH_SPEC:
;                  PLOT_SPEC, XRANGE = xrange, YRANGE = yrange, SKIP = skip
;                  XRANGE and YRANGE are the extent of the plot and SKIP is a skip factor 
;                  for the different rays.  Default is that all NPOINT rays are plotted.
;       TITLE:   Title for plot.   Default is the parameters for the calculation
;       XRANGE:  If present, the extent of the XRange for the plot
;       YRANGE:  If present, the extent of the YRange for the plot
;       TRUNCATE: If set, do not provide calculation parameters in the dataset comment.
;                 Note: "PLO, /TRUNCATE" will eliminate these values from plots.
;                 Default is to provide the parameters.  For example: Comment =
;                 "X-ray Wavelength vx Crystal X - Crystal_Radius=250.000;;2D_spacing=6.68700;;
;                 Source_distance=920.000;;Souce_angle=30.7300;;Film_Pos(X,Y)=
;                 (0.0000000,250.00000);;Film_Angle(Rad)=0.26524130"
;       PERPENDICULAR:  Film position to place the film perpendicular to the 
;                primary ray which goes through the X=0 coordinate of the crystal.  This is a 
;                returned value based on the value of FILM and THETA.
;                The X and Y locations are the position where the film is hit in the current 
;                calculation.  The angle is (90 - theta) * !pi/180
;          NOTE: PERPENDICULAR[0:1] are the X, Y locations for the primary ray on the film.
;       VERBOSE:  If set, print the calculational parameters to the terminal window.
;          Note:  The calculated construction parameters are returned in keyword: PARAMETERS
;                 If PLOTIT is not defined, PLOTIT = 1 ;; Draw the plot
;       PARAMETERS: Angles and distances for the spectrometer (Units are DEGREES and MM)
;                 The three element vector has the following:
;                 1.  Angle between the vertical at X= 0 and the vector from the center of the 
;                     Rowland circle and the X=0 impact point on the film. (Degrees)
;                 2.  Angle between the vector from the center of the Rowland circle to the X=0
;                     point on the film and the plane of the film. (Degrees)
;                 3.  The distance between the center of the Rowland circle and the X=0 point 
;                     on the film.  (mm, divide by 25.4 to get inches)
;                     In the spectrometer code of Dan Sinars, these are designated: 
;                       FRO, A1, and RF
;               Note: The difference between A1 and FRO is the slope of the film (FILM[2], but 
;                           the units are not the same.)
;                 4.  Approximate size of waist of the xrays.  This is defined as the arc length 
;                     between the intersection of the highest and lowest energy xrays and the X=0
;                     xray.  WAIST = Radius * Theta
;                 5.  Radius of the arc in item 4.
;                 6/7. X and Y values of the intersection of the highest and lowest energy xrays.
;                 8/9. X and Y values of the center energy/wavelength of the film
;                 10. Distance from the low energy xray to the center wavelength.
;                 11. Distance from the high energy xray to the center wavelength.
;                 12. Approximate transverse size of X-rays in the transverse (imaging) direction
;       WRITECSV: If set, write the parameters to a CSV file.  The current calculation is to
;                 appended the structure and written at the end of the file. 
;       IDENT:    This keyword is ignored unless WRITECSV is set.
;                 IDENT = N_ELEMENTS(IDENT) EQ 0 ? THIN_DATE() : STRTRIM(IDENT[0], 2)
;                 This is the value in the first column of the CSV file.
;                
; OUTPUTS:
;       A plot of the spectrometer is provided.
;       Construction values are provided in PARAMETERS and printed to the terminal.
;       Waveforms of the spectrometer parameters are produced (See OUTPUT keyword to specify.)
;       The output waveforms include the following:
;       1.  Crystal Surface
;       2.  Crystal Bragg Angle vs Crystal X
;       3.  X-ray Wavelength vx Crystal X
;       4.  Energy vs Crystal X
;       5.  Film Position
;       6.  Source-Xtal Dist vs Wavelength
;       7.  Xtal-Film Dist vs Wavelength
;       8.  Magnification vs Film Position
;       9.  Magnification vs Wavelength
;      10.  Magnification vs Photon Energy
;      11.  Angle to Normal vs Film Position
;      12.  Wavelength vs Film Position
;      13.  Photon Energy vs Film Position
;      14.  Middle ray
;      15.  Transverse Magnification vs Crystal X
;           Multiply this waveform by the crystal width to calculate the transverse extent
;           of the xrays at the waist.
;
; PROCEDURE:
;       1.  Calculate the source position in X, Y coordinates
;       2.  Divide the crystal X positions into NPOINT positions over a length of 50mm 
;       3.  Calculate the Y crystal position at each X.  X-rays will be propagated
;           from the source to each X,Y position on the crystal.
;       4.  Calculate the source to crystal distance, Ax, at each point X.
;       5.  Calculate the angle for the diffracted x-ray.
;       6.  Solve for the intercept of the diffracted ray with the film plane.
;           and the corresponding distances from the crystal to the film and the 
;           distance along the film from the designated point on the film.
;       7.  Calculate the Bragg angle and thus the energy/wavelength for each x-ray.
;       8.  Calculate the angle to the normal of the film for each x-ray.
;       9.  Calculate the transverse dimensions of the waist as a magnification.
;           The magnificaion is the:
;           (crystal to film distance(b) - distance from the waist to crystal)/b
;           This calculations assumes a point source.
;      10.  Draw the geometry
;      NOTE:
;           X-ray waist calculations are approximate.  Size of the waist is calculated as
;           R*Delta_Theta.  Transverse dimensions assume a  point source and calculate the 
;           the geometric reduction in crystal width in going to the waist area.
;
; KEYWORDS For EXCEL Database Columns
; Parameters which SPH_SPEC searches are listed below.  The first column can be called
;   anything; the first column is always used for identifier.
;   GET_MATCH uses "^Sph*Rad" to find the spherical radius of the crystal, etc.
;       1.  * = Geometry
;       2.  ^Sph*Rad = Spherical Radius of Crystal
;       3.  Spac = 2D Spacing of Crystal
;       4.  Order = Diffraction order
;       5.  Length = Crystal Length
;       6.  Width = Crystal Width
;       7.  S*Dist = Source Distance
;       8.  S*Ang = Source Angle (Bragg Angle at X=0)
;       9.  ^Film*PX = X position of a point on the film plane
;      10.  ^Film*PY = Y position of a point on the film plane (8 & 9 are the same point!)
;      11.  F*Ang = Film angle in radians
;      12.  S*Pos = Is Film in Standard Position (YES, NO)
;      13.  Lambda-0 = Wavelength of the photon that comes from the crystal center (X=0)
;      14.  Energy-0 = Energy of the photon that comes from the crystal center (X=0)
;      15.  TCO = Angle between the crystal normal at its center and the source.
;                 TCO = 90 - Bragg Angle
;      16.  FRO = Angle of the vector from the vertical at X=0 and the vector from the 
;                 center of the Rowland Circle to the film "Center" (where X=0 xray strikes film)
;      17.  A1 = Angle between the film and the vector from the film center to the Rowland
;                circle center.  Note: FRO - A1 = Film Angle
;      18.  RF = Distance from the center of the Rowland circle to the film
;      19.  ^Waist$ = Approximate size of the narrowest portion of the xrays between the crystal
;                     and the film.  This occurs where high and low energy X-rays cross.
;      20.  ^Waist*R = Distance of the WAIST from the crystal center
;      21.  Waist*X = X value of the Waist point where high and low energy Xrays cross
;      22.  Waist*Y = Y value of the Waist point where high and low energy Xrays cross
;      23.  Trans* = Transverse extent of the xrays at the waist
;      24.  Cent*X = X location where the Xray from X=0 on the crystal strike the film.
;      25.  Cent*Y = Y location where the Xray from X=0 on the crystal strike the film.
;      26.  Lo*Dist = Distance along the film to the lowest energy Xray
;      27.  Hi*Dist = Distance along the film to the highest energy Xray
;      28.  N*POINTS = Number of points for the calculation
;      29.  MIN*Energy = Minimum energy hitting the film
;      30.  MAX*ENERGY = Maximum energy hitting the film
;      31.  Date = Date of the calculation
;
;
; EXAMPLE:
;       Calculate the dispersion and plot the xrays from source to film with the default values.
;         SPH_SPEC, wdinit  = 31 
;
;       Calculate the dispersion and plot the xrays from source to film with the default values,
;       but change the source angle to 34.5 degrees.  Start the output at wdf array 51.
;         SPH_SPEC, wdinit  = 51, theta = 34.5
;       
;       Calculate the dispersion of a spherical spectrometer.
;         SPH_SPEC
;
;       Calculate the dispersion of a spherical spectrometer using the CSV data from the 
;       row, "Z23", save the 'Energy vs  Film Position in WDF array 55.  Use 5000 points in
;       the calculation.  Do not plot the geometry.
;         SPH_SPEC, GEOMETRY = "Z23", OUTPUT = 12, WD = 55, PLOTIT =0, NPOINTS = 5000
;         (Note:  An odd number of points are always used; NPOINTS = 5001)
;
; MODIFICATION HISTORY:
; 	Written by:     L. P. Mix, Jr., December 3, 2010
;       January, 2011   Added OUTPUT keyword and CSV support.
;-


pro plot_spec, xrange = xrange, yrange = yrange, plotit = plotit, title = title, skip = skip 

common sph_spec_plot_common, x0, y0, xc, yc, xf, yf, xmid, ymid, clab_save, $
         rowlandx, rowlandy
on_error, 1
if n_elements (x0) eq 0 then begin
  Print, '% PLOT_SPEC: The routine, SPH_SPEC, must be run before running PLOT_SPEC.'
  return
endif
if n_elements (plotit) eq 0 then begin
  plotit = get_maxwdf()
  if (wdvalid(plotit, /valid, /quiet))[0]  gt 0 then $
    i = get_match('MIDDLE RAY', wdcomment(plotit), nel = nel) $
  else nel =0
  if nel eq 0 then i2w, /xydata, xmid, ymid, plotit, clab = 'Middle ray - '+clab_save
endif
title = (n_elements(title) eq 0) ? clab_save : title[0]

xx = (n_elements(xrange) lt 2) ? [min([x0, xc, xf], max = m), m]* 1.02 : xrange[0:1]
yy = (n_elements(yrange) lt 2) ? [min([y0, yc, yf], max = m), m]* 1.02 : yrange[0:1]
if keyword_set(plotit) then $
  plo, plotit, xrange = xx, yrange = yy, /asp, /xst, /yst, ov =0, title = title
n = n_elements (xc)
jm = n/2
mkcircle
if n_elements(skip) eq 0 then skip = 1
skip = round (skip) > 1
plots, x0, y0, psym = 8
if skip eq 1 then $
  for i = 0, n-1 do $
    oplot, [x0, xc[i], xf[i]], [y0, yc[i], yf[i]] else $
  for i = 0, n-1, skip do $
    oplot, [x0, xc[i], xf[i]], [y0, yc[i], yf[i]]
oplot, [0,0], !y.crange
oplot, xf, yf, color = 1, thick = 2
oplot, xc, yc, color = 2
oplot, [xf[jm], x0], [yf[jm], y0], col =6
oplot, xmid, ymid, color = 5
oplot, rowlandx, rowlandy, color = 4
end
pro sph_spec, wdinit = wdinit, radius= rc, cry2d = cry2d, Distance = distance, film = film, $
          perpendicular = perp, Theta = theta, order = order, verbose = verbose, $
          truncate = truncate, npoints = npoints, plotit = plotit, length = length, $
          parameters = param, set_default = set_default, show_default = show_default, $
          calculate = calculate, gui = gui, geometry = geometry, database = database, $
          tco = tco, energy = energy, writecsv = writecsv, ident = ident, saveit = saveit, $
          width = width, output = output, title = title, xrange = xrange, yrange = yrange
          
on_error,1
common sph_spec_plot_common, x0, y0, xc, yc, xf, yf, xmid, ymid, clab_save, $
          rowlandx, rowlandy
@sph_spec_common.dat
;common sph_spec_common, rc_save, cry2d_save, distance_save, film_save, theta_save, $
;   order_save, truncate_save, verbose_save, npoints_save, plotit_save, length_save, $
;   database_save, width_save, db_keys
;   handle the default parameters
;if n_elements(rc_save) eq 0 then begin
;  rc_save = 150.0d0
;  cry2d_save = 6.687
;  order_save = 1L
;  length_save = 50.0d0
;  width_save = 30.0d0
;  distance_save = 300d0
;  theta_save = 34.0d0
;  npoints_save = 1001L
;  film_save = 0
;  truncate_save = 0
;  verbose_save =1
;  plotit_save = 1
;  database_save =0
; database column headings for column 2 to the end.
;  db_keys =['^SPH*RAD', 'SPAC', 'ORDER', 'LENGTH', 'S*DIST', 'S*ANG', '^FILM*PX',  $
;    '^FILM*PY', 'F*ANG', 'S*POS', 'LAMBDA-0', 'ENERGY-0', 'TCO','FRO', 'A1', 'RF', $
;     '^WAIST$', '^WAIST*R', 'WAIST*X', 'WAIST*Y', 'CENT*X', 'CENT*Y', 'LO*DIST', $
;     'HI*DIST', 'N*POINTS', 'MIN*ENERGY', 'MAX*ENERGY', 'TRANS*', 'DATE', 'WIDTH']
;endif

;
film_err = 1e-5
phyconst, ealam = ealamda ; (eV* Angstrom)
if n_elements(database) gt 0 then readcsv, database_save, database
if n_elements(theta) eq 0 then begin
  if n_elements(tco) gt 0 then theta = 90d0-double(tco[0]) else $
    if n_elements(energy) gt 0 then begin
;   We need values but we do not want to save them
    if n_elements(cry2d) eq 0 then cry2d = cry2d_save
    theta = 180d0/!dpi*asin(ealamda/energy[0]*order/cry2d)
  endif
endif

if keyword_set(set_default) || keyword_set(show_default) then begin
  if keyword_set(set_default) then begin
    if n_elements(rc) gt 0 then rc_save =double(rc[0])
    if n_elements(cry2d) gt 0 then cry2d_save = double(cry2d[0])
    if n_elements(order) gt 0 then order_save = order[0]
    if n_elements(Length) gt 0 then length_save = double(length[0]) > 1.0
    if n_elements(width) gt 0 then width_save = double(width[0]) > 1.0
    if n_elements(distance) gt 0 then distance_save = double(distance[0])
    if n_elements(theta) gt 0 then theta_save = double(theta[0])
    if n_elements(npoints) gt 0 then npoints_save = npoints[0]
    if n_elements(film) gt 0 then film_save = film
    if n_elements(PLOTIT) gt 0 then plotit_save = plotit[0]
    if n_elements(truncate) gt 0 then truncate_save = truncate[0]
    if n_elements(verbose) gt 0 then verbose_save = verbose[0]
  endif
  if keyword_set(show_default) then begin
    print, 'Default parameters for the PLO command are:'
    print, '    RADIUS   = ' , strtrim (rc_save, 2)
    print, '    CRY2D    = ' , strtrim (cry2d_save, 2)
    print, '    ORDER    = ' , strtrim (order_save, 2)
    print, '    LENGTH   = ' , strtrim (length_save, 2)
    print, '    WIDTH    = ' , strtrim (width_save, 2)
    print, '    DISTANCE = ' , strtrim (distance_save, 2)
    print, '    THETA    = ' , strtrim (theta_save, 2)
    print, '    NPOINTS  = ' , strtrim (npoints_save, 2)
    print, '    FILM     = ' , strtrim (film_save, 2)
    print, '    PLOTIT   = ' , strtrim (plotit_save, 2)
    print, '    TRUNCATE = ' , strtrim (truncate_save, 2)
    print, '    VERBOSE  = ' , strtrim (verbose_save, 2)
    if size(/type, database_save ) eq 8 then $
      print, '    DATABASE File = '+database_save.file_name  $
    else print, '    No Database available'

  endif
  if not keyword_set (calculate) then return
endif
;

if n_elements(gui) eq 0 then begin
  if (n_elements(rc) eq 0) && (n_elements(cry2d) eq 0) && (n_elements(order) eq 0) $
    && (n_elements(distance) eq 0) && (n_elements(theta) eq 0) && $
    (n_elements(film) eq 0) then gui = 1 else gui = 0
  if n_elements(geometry) gt 0 and size(/type , database_save) eq 8 then gui = 0
endif
maxwdf = get_maxwdf()
noutput_max = 15
if n_elements(output) eq 0 then output = 1+indgen(noutput_max)
output = round (output)
output = output[uniq(output, sort(output))]
nelout = n_elements(output)
ok = where (output gt 0 and output le noutput_max, nok)
if nok gt 0 and nok ne nelout then begin
  nelout = nok
  output = output[ok]
endif
wdinit = (n_elements(wdinit) eq 0) ? maxwdf-nok-1  : long(wdinit[0])
if wdinit lt 1 or wdinit gt maxwdf-nok-1 then wdinit = maxwdf-nok-1

if n_elements(geometry) gt 0 and size(/type , database_save) eq 8 then begin
  geometry = geometry[0]
  if size(/type, geometry) lt 4 then ok = where (geometry eq database_save.c0, nok) $
    else ok = get_match( geometry,  database_save.c0, nel = nok)
  if nok ne 1 then begin
    print, '% SPH_SPEC:  Unable to locate a unique key for: "'+strtrim (geometry, 2)+'"'
    print, '             Number of values found: '+ strtrim (nok, 2)
    if nok gt 1 then for kk = 0, nok-1 do print,kk, '   ', (database_save.c0)[ok[kk]]
    gui = 1
    undefine, geometry
  endif else begin
    i = ok[0]
    tags = database_save.tag_names
    get_sph_database_values, i, rc, cry2d, order, length, width, distance, theta, $
        npoints, film, verbose = verbose
    if min(film) lt -0.8e30 then film = 0
  endelse
endif else if n_elements(geometry) gt 0 then begin
  gui = 1
  print, '% SPH_SPEC: Database must be defined to use geometry keyword'
endif
; Crystal Radius
rc = (n_elements(rc) eq 0) ? rc_save : double(rc[0])  ; mm 
; Crystal 2*D spacing (angstroms)
cry2d =  (n_elements(cry2d) eq 0) ? cry2d_save: double(cry2d[0])
; Distance to the source
a0 = (n_elements(distance) eq 0) ? distance_save : double(distance[0])
; Angle to source
theta =  (n_elements(theta) eq 0) ? theta_save : double(theta[0])
if (n_elements(film) eq 0) then film = film_save
if (n_elements(truncate) eq 0) then truncate = truncate_save
if (keyword_set(verbose))  and (n_elements(plotit) eq 0) then plotit = 1
verbose =  (n_elements(verbose) eq 0) ? verbose_save : verbose[0]
plotit = (n_elements(plotit) eq 0) ? plotit_save : plotit[0]
len = (n_elements(length) eq 0) ? length_save : double(length[0]) > 1.0
width = (n_elements(width) eq 0) ? width_save : double(width[0]) > 1.0
order = (n_elements(order) eq 0) ? order_save : (order[0] > 1)
np = (n_elements(npoints) eq 0) ? npoints_save : Long(npoints[0]) > 11L
if n_elements(film) lt 3 then begin
  Thetar = theta *!dpi/180.0d0
  stheta = sin(thetar)
  ctheta = cos(thetar)
  y0 = a0*stheta
  x0 = a0*ctheta
  film = [0, rc, atan(y0-rc, x0)]
endif
quit = 0
if keyword_set(gui) then get_sph_spec_params, $
    wdinit, rc, cry2d, len, width, order, a0, theta, film, geometry = geometry, quit = quit
if quit ne 0 then return

;
; return AO to Distance to return to caller
distance = a0
if keyword_set(verbose) then begin
  print, ' Crystal Radius (mm) =' + strtrim(rc, 2)
  print, ' Crystal Length (mm) =' + strtrim(len, 2)
  print, ' Crystal Width (mm) =' + strtrim(width, 2)
  print, ' Crystal 2D Space (Angs) =' + strtrim(cry2d, 2)
  print, ' Diffraction order =' +strtrim(order, 2)
  print, ' Source Distance (mm) =' + strtrim(a0, 2)
  print, ' Source Bragg Angle (deg)=' + strtrim(theta,2)
  print, ' Film Point (mm) (X, Y) ='+strtrim(Film[0],2)+ ', '+strtrim(film[1],2)
  print, ' Film Angle (Radian) ='+strtrim( film[2], 2)
  print, ' Film Slope (dy/dx) ='+strtrim(tan(film[2]), 2)
endif

if keyword_set(truncate) then clab = '' else begin
  clab = ' - Crystal_Radius='+strtrim(rc, 2)
  clab = clab +';;2D_spacing='+strtrim(cry2d, 2)
  clab = clab +';;Order='+strtrim(order, 2)
  clab = clab +';;Source_distance='+strtrim(a0, 2)
  clab = clab +';;Source_angle='+strtrim(theta, 2)
  clab = clab +';;Film_Pos(X,Y)=('+strtrim(film[0], 2)+','+strtrim(film[1],2)+')'
  clab = clab +';;Film_Angle(Rad)='+strtrim(film[2], 2)
endelse

; Theta in radians
Thetar = theta *!dpi/180.0d0
; Film is perpendicular with slope equal to complement of angle of incident 
perp = [0.0, 0.0, !dpi*0.5d0-thetar]
stheta = sin(thetar)
ctheta = cos(thetar)
y0 = a0*stheta
x0 = a0*ctheta
lamda0 = cry2d*stheta  ;  Angstroms
if n_elements(film) lt 3 then film = [0, rc, atan(y0-rc, x0)]
rowang = findgen(361) *(!pi/180)
Rowland_r = rc*0.5
rowlandx =  cos(rowang) * Rowland_r
rowlandy =  sin(rowang) * Rowland_r +  Rowland_r
;
; Generate a grid 
mid = np/2L
npm = (np/2)*2
np = npm+1

len2 = len*0.5
xc = lindgen(np) * (len/npm) -len2
xRfrac = xc/rc
tmp = sqrt((1.0d0-xrfrac)*(1.0d0+xrfrac))
; YC = Y position on Crystal
yc =  rc * (1.0d0 - tmp)
; angle of crystal planes
phic =  atan(xrfrac/tmp)
thetax = atan((y0-yc)/(x0-xc)) - phic
lamdax = cry2d * sin(thetax) / order
enerfilm = ealamda/lamdax
ok = where(1 eq output, nok)
if nok gt 0 then i2w, xc, yc, wdinit, /xydata, xlab = 'Crystal X (mm)', $
     ylab ='Crystal Y (mm)', clab = 'Crystal Surface'+clab
wdinit += nok
ok = where(2 eq output, nok)
if nok gt 0 then i2w, xc, thetax, wdinit, /xydata, xlab = 'Crystal X (mm)', $
     ylab = 'Angle-radians', clab = 'Crystal Bragg Angle vs Crystal X' +clab 
wdinit += nok
ok = where(3 eq output, nok)
if nok gt 0 then i2w, xc, lamdax, wdinit, /xydata, xlab = 'Crystal X (mm)', $
     ylab = 'Wavelength (Ang)', clab = 'X-ray Wavelength vx Crystal X' +clab 
wdinit += nok
ok = where(4 eq output, nok)
if nok gt 0 then i2w, xc, enerfilm, wdinit, /xydata, xlab = 'Crystal X (mm)', $
     ylab = 'Photon Energy (eV)', clab = 'Energy vs Crystal X'+clab 
wdinit += nok

; source to crystal
ax = sqrt((x0-xc)^2+ (y0-yc)^2)

; film
; angle going to film from (xc, yc) = 180 +phic - thetax = 180 -(thetax -phic)
angle = thetax - phic

; sin(180 -a) =  sin(a)
sangle = sin(angle)
; cos(180 -a) = -cos(a)
cangle = -cos(angle)

; solve the intersection of the film vector with the scattered vector
; xc + dcf*cangle = film[0] +film_d*cfilm
; yc + dcf*sangle = film[1] +film_d*sfilm
; where dcf is the distance from the crystal to the film
;       film_d is the distance in the film direction

d = xc - film[0]
e = yc - film[1]


sfilm = sin(film[2])
cfilm = cos(film[2])
film_d = (sangle* d - cangle*e)/ (cfilm*sangle - sfilm*cangle)
film_d0 = film_d[mid]
dcf = (d*sfilm-e*cfilm)/(cangle *sfilm - sangle*cfilm)

xf = film[0] + film_d*cfilm
yf = film[1] + film_d*sfilm
;
ok = where(5 eq output, nok)
wdfilm = wdinit
if nok gt 0 then i2w, xf, yf, wdinit, /xydata, xlab = 'X (mm)', ylab = 'Y (mm)', $
     clab = 'Film Position'+clab
wdinit += nok
;
xf0 = xf[mid]
yf0 = yf[mid]

;
; add the film points to the PERPENDICULAR keyword.
; Calculate film position using slope and point
perp[0:1] = [xf0, yf0]
xd = xf -xf0
yd = yf -yf0

;mag = bx/ax
bxarr = sqrt((xf-xc)^2 +(yf-yc)^2)
mag =  bxarr/ax
ok = where(6 eq output, nok)
if nok gt 0 then i2w, lamdax, ax, wdinit, /xydata, xlab = 'Wavelength (Ang)', $
     ylab = 'Distance (mm)', clab = 'Source -Xtal Dist vs Wavelength'+clab
wdinit += nok
ok = where(7 eq output, nok)
if nok gt 0 then i2w, lamdax, bxarr, wdinit, /xydata, xlab = 'Wavelength (Ang)', $
     ylab = 'Distance (mm)', clab = 'Xtal-Film Dist vs Wavelength'+clab
wdinit += nok
;
; Set film distance relative to the middle ray 
film_d =film_d -film_d0
ok = where(8 eq output, nok)
if nok gt 0 then i2w, film_d, mag, wdinit, /xydata, xlab = 'Film Distance (mm)', $
     ylab = 'Magnification', clab = 'Magnification vs Film Position'+clab
wdinit += nok
ok = where(9 eq output, nok)
if nok gt 0 then i2w, lamdax, mag, wdinit, /xydata, xlab = 'Wavelength (Ang)', $
     ylab = 'Magnification', clab = 'Magnification vs Wavelength'+clab
wdinit += nok
ok = where(10 eq output, nok)
if nok gt 0 then i2w, enerfilm, mag, wdinit, /xydata, xlab = 'Energy (eV)', $
     ylab = 'Magnification', clab = 'Magnification vs Photon Energy'+clab
wdinit += nok

; Angle to normal
; film vector = fv = cfilm  i + sfilm  j
; xray vector = xv = cangle i + sangle j
; dot product = fv dot xv = cos (angle)
normal =abs(  90.0 -acos(cfilm *cangle +sfilm*sangle) *(180.0/!pi))
ok = where(11 eq output, nok)
if nok gt 0 then i2w, film_d, normal, wdinit, /xydata, xlab = 'X (mm)', $
     ylab = 'Angle(deg)', clab = 'Angle to Normal vs Film Position'+clab
wdinit += nok
ok = where(12 eq output, nok)
if nok gt 0 then i2w, film_d, lamdax, wdinit, /xydata, xlab = 'X (mm)', $
     ylab = 'Wavelength (A)', clab = 'Wavelength vs Film Position'+clab
wdinit += nok
ok = where(13 eq output, nok)
minener = min(enerfilm, max = maxener)
if nok gt 0 then i2w, film_d, enerfilm, wdinit, /xydata, xlab = 'X (mm)', $
     ylab = 'Energy (eV)', clab = 'Photon Energy vs Film Position'+clab
wdinit += nok
ok = where(14 eq output, nok)
i2w, [x0, 0.0, xf0], [y0, 0.0, yf0],maxwdf, /xydata, $
     xlab = 'X-Distance (mm)', ylab = 'Y-Distance (mm)', clab = 'Middle ray'+clab
if nok gt 0 then xfr, maxwdf, wdinit
wdinit += nok
;
; Calculate the parameters for the spectrometer
ok  = where(15 eq output, nok)
if arg_present(param) or keyword_set(verbose)  or keyword_set(writecsv) $
  or (nok gt 0) then begin
  param = make_array(12, /double)
  fdy = yf0-rowland_r
  FRO =ATAN(-xf0, fdy)
  rf = sqrt(xf0*xf0 +fdy*fdy)
; distance from film to top of Rowland circle
  ff0  = sqrt((rc-yf0)^2 +xf0*xf0)
  a1 = !dpi/2 -asin(rowland_r*sin(FRO)/ff0)
  param[0:1] = [fro, a1]*180d0/!dpi
  param[2] = rf
  param[7:8] = [xf0, yf0]
  param[9] =   film_d[0] 
  param[10] = - film_d[npm]
  d = xc[0] -xc[npm]
  e = yc[0] -yc[npm]
  dh = (sangle[0]*d -cangle[0]*e) /(cangle[npm]*sangle[0] -sangle[npm]*cangle[0])
  dl = -(sangle[npm]*d -cangle[npm]*e) /(cangle[0]*sangle[npm] -sangle[0]*cangle[npm])
; 
; Intersection of high and low energy rays
  param[5:6] = [ xc[0]+dl*cangle[0],  yc[0]+dl*sangle[0]]
; calculate size at the crossing for a point source
; crystal to crossing

  dxx = sqrt ((xc - param[5])^2 + (yc -param[6])^2)
  mtrans = (bxarr-dxx)/bxarr
  maxtran = max(mtrans)
  param[11] = maxtran*width
  if nok gt 0 then i2w, xc, mtrans, wdinit, /xydata, xlab = 'Crystal X (mm)', $
     ylab = 'Transverse Magnification', $
     clab = 'Transverse Magnification vs Crystal Position'+clab

; Radius to intersection 
  param[4] = sqrt(param[5]^2+param[6]^2)
  waist_theta = atan(param[6], param[5])
  thetadr = !dpi -thetar
  param[3] = (waist_theta - thetadr) * param[4]
  fslope = tan(film[2])
  enermid = enerfilm[mid]
  Standard = ( abs ((y0 -yf0) /(x0-xf0) - fslope)/fslope lt film_err) ? "YES" : "NO"
  if keyword_set(verbose) then begin
    print, ' Center Wavelength = '+strtrim(lamdax[mid],2)+' Angstroms'
    print, ' Center Energy = '+strtrim(enermid,2)+' eV'
    print, ' TCO = '+strtrim (90.0d0-theta, 2) +' Deg'
    print, ' FRO = '+strtrim (param[0], 2) +' Deg'
    print, ' A1 = '+strtrim (param[1], 2) +' Deg'
    print, ' Film to Rowland Center(RF) = '+strtrim (param[2], 2) +' mm'
    print, ' Beam Waist = '+ strtrim(param[3], 2)+' mm'
    print, ' Waist to Crystal Center = '+ strtrim(param[4], 2)+' mm'
    print, ' Intersection of High/Low energy Xrays = ['+strtrim (param[5], 2)+ $
         ', '+strtrim(param[6], 2)+']'
    print, ' Max Transverse waist beam size = '+strtrim(param[11], 2)
    print, ' Film Center = ['+strtrim (xf0, 2)+', '+strtrim(yf0, 2)+']'
    print, ' Film Center to Low Energy = ' + strtrim(param[9], 2)+' mm'
    print, ' Film Center to High Energy = ' + strtrim(param[10], 2)+' mm'
    print, ' Minimum Energy = ' + strtrim(minener, 2)+' eV'
    print, ' Maximum Energy = ' + strtrim(maxener, 2)+' eV'
    if standard eq 'YES' then print, ' Film is in the standard position.' else $
        print, ' Film is NOT in the standard position.'
  endif
endif
if keyword_set(writecsv) and (size(/type, database_save) eq 8) then begin
  openw, unit, /get, /append, database_save.file_name
  ncol= database_save.ncol
  cols = make_array(ncol, value ='')
  cols[0] = (n_elements (ident)  eq 0) ? thin_date() : strtrim(ident, 2)
  tags =  database_save.tag_names
  rr = get_match(db_keys[0], tags, nel = nel)
  if nel eq 1 then cols[rr[0]] = strtrim(rc, 2) else if keyword_set(verbose) then $
    print, '% SPH_SPEC: Unable to fine unique column for search string: '+db_keys[0]
  rr = get_match(db_keys[1], tags, nel = nel)
  if nel eq 1 then cols[rr[0]] = strtrim(cry2d, 2) else if keyword_set(verbose) then $
    print, '% SPH_SPEC: Unable to fine unique column for search string: '+db_keys[1]
  rr = get_match(db_keys[2], tags, nel = nel)
  if nel eq 1 then cols[rr[0]] = strtrim(order, 2) else if keyword_set(verbose) then $
    print, '% SPH_SPEC: Unable to fine unique column for search string: '+db_keys[2]
  rr = get_match(db_keys[3], tags, nel = nel)
  if nel eq 1 then cols[rr[0]] = strtrim(len, 2) else if keyword_set(verbose) then $
    print, '% SPH_SPEC: Unable to fine unique column for search string: '+db_keys[3]
  rr = get_match(db_keys[4], tags, nel = nel)
  if nel eq 1 then cols[rr[0]] = strtrim(distance, 2) else if keyword_set(verbose) then $
    print, '% SPH_SPEC: Unable to fine unique column for search string: '+db_keys[4]
  rr = get_match(db_keys[5], tags, nel = nel)
  if nel eq 1 then cols[rr[0]] = strtrim(theta, 2) else if keyword_set(verbose) then $
    print, '% SPH_SPEC: Unable to fine unique column for search string: '+db_keys[5]
  rr = get_match(db_keys[6], tags, nel = nel)
  if standard eq 'YES' then begin
    if nel eq 1 then cols[rr[0]] = strtrim(0.0, 2) else if keyword_set(verbose) then $
      print, '% SPH_SPEC: Unable to fine unique column for search string: '+db_keys[6]
    rr = get_match(db_keys[9], tags, nel = nel)
    if nel eq 1 then cols[rr[0]] = '"YES"' else if keyword_set(verbose) then $
      print, '% SPH_SPEC: Unable to fine unique column for search string: '+db_keys[9]
  endif else begin
    if nel eq 1 then cols[rr[0]] = strtrim(film[0], 2) else if keyword_set(verbose) then $
      print, '% SPH_SPEC: Unable to fine unique column for search string: '+db_keys[6]
    rr = get_match(db_keys[7], tags, nel = nel)
    if nel eq 1 then cols[rr[0]] = strtrim(film[1], 2) else if keyword_set(verbose) then $
      print, '% SPH_SPEC: Unable to fine unique column for search string: '+db_keys[7]
    rr = get_match(db_keys[8], tags, nel = nel)
    if nel eq 1 then cols[rr[0]] = strtrim(film[2], 2) else if keyword_set(verbose) then $
      print, '% SPH_SPEC: Unable to fine unique column for search string: '+db_keys[8]
    rr = get_match(db_keys[9], tags, nel = nel)
    if nel eq 1 then cols[rr[0]] = '"NO"' else if keyword_set(verbose) then $
      print, '% SPH_SPEC: Unable to fine unique column for search string: '+db_keys[9]
  endelse
  rr = get_match(db_keys[10], tags, nel = nel)
  if nel eq 1 then cols[rr[0]] = strtrim(lamdax[mid], 2) else if keyword_set(verbose) then $
    print, '% SPH_SPEC: Unable to fine unique column for search string: '+db_keys[10]
  rr = get_match(db_keys[11], tags, nel = nel)
  if nel eq 1 then cols[rr[0]] = strtrim(enermid, 2) else if keyword_set(verbose) then $
    print, '% SPH_SPEC: Unable to fine unique column for search string: '+db_keys[11]
  rr = get_match(db_keys[12], tags, nel = nel)
  if nel eq 1 then cols[rr[0]] = strtrim((90d0-theta), 2) else if keyword_set(verbose) then $
    print, '% SPH_SPEC: Unable to fine unique column for search string: '+db_keys[12]
  rr = get_match(db_keys[13], tags, nel = nel)
  if nel eq 1 then cols[rr[0]] = strtrim(param[0], 2) else if keyword_set(verbose) then $
    print, '% SPH_SPEC: Unable to fine unique column for search string: '+db_keys[13]
  rr = get_match(db_keys[14], tags, nel = nel)
  if nel eq 1 then cols[rr[0]] = strtrim(param[1], 2) else if keyword_set(verbose) then $
    print, '% SPH_SPEC: Unable to fine unique column for search string: '+db_keys[14]
  rr = get_match(db_keys[15], tags, nel = nel)
  if nel eq 1 then cols[rr[0]] = strtrim(param[2], 2) else if keyword_set(verbose) then $
    print, '% SPH_SPEC: Unable to fine unique column for search string: '+db_keys[15]
  rr = get_match(db_keys[16], tags, nel = nel)
  if nel eq 1 then cols[rr[0]] = strtrim(param[3], 2) else if keyword_set(verbose) then $
    print, '% SPH_SPEC: Unable to fine unique column for search string: '+db_keys[16]
  rr = get_match(db_keys[17], tags, nel = nel)
  if nel eq 1 then cols[rr[0]] = strtrim(param[4], 2) else if keyword_set(verbose) then $
    print, '% SPH_SPEC: Unable to fine unique column for search string: '+db_keys[17]
  rr = get_match(db_keys[18], tags, nel = nel)
  if nel eq 1 then cols[rr[0]] = strtrim(param[5], 2) else if keyword_set(verbose) then $
    print, '% SPH_SPEC: Unable to fine unique column for search string: '+db_keys[18]
  rr = get_match(db_keys[19], tags, nel = nel)
  if nel eq 1 then cols[rr[0]] = strtrim(param[6], 2) else if keyword_set(verbose) then $
    print, '% SPH_SPEC: Unable to fine unique column for search string: '+db_keys[19]
  rr = get_match(db_keys[20], tags, nel = nel)
  if nel eq 1 then cols[rr[0]] = strtrim(xf0, 2) else if keyword_set(verbose) then $
    print, '% SPH_SPEC: Unable to fine unique column for search string: '+db_keys[20]
  rr = get_match(db_keys[21], tags, nel = nel)
  if nel eq 1 then cols[rr[0]] = strtrim(yf0, 2) else if keyword_set(verbose) then $
    print, '% SPH_SPEC: Unable to fine unique column for search string: '+db_keys[21]
  rr = get_match(db_keys[22], tags, nel = nel)
  if nel eq 1 then cols[rr[0]] = strtrim(param[9], 2) else if keyword_set(verbose) then $
    print, '% SPH_SPEC: Unable to fine unique column for search string: '+db_keys[22]
  rr = get_match(db_keys[23], tags, nel = nel)
  if nel eq 1 then cols[rr[0]] = strtrim(param[10], 2) else if keyword_set(verbose) then $
    print, '% SPH_SPEC: Unable to fine unique column for search string: '+db_keys[23]
  rr = get_match(db_keys[24], tags, nel = nel)
  if nel eq 1 then cols[rr[0]] = strtrim(np, 2) else if keyword_set(verbose) then $
    print, '% SPH_SPEC: Unable to fine unique column for search string: '+db_keys[24]
  rr = get_match(db_keys[25], tags, nel = nel)
  if nel eq 1 then cols[rr[0]] = strtrim(minener, 2) else if keyword_set(verbose) then $
    print, '% SPH_SPEC: Unable to fine unique column for search string: '+db_keys[25], rr
  rr = get_match(db_keys[26], tags, nel = nel)
  if nel eq 1 then cols[rr[0]] = strtrim(maxener, 2) else if keyword_set(verbose) then $
    print, '% SPH_SPEC: Unable to fine unique column for search string: '+db_keys[26], rr
  rr = get_match(db_keys[27], tags, nel = nel)
  if nel eq 1 then cols[rr[0]] = strtrim(param[11], 2) else if keyword_set(verbose) then $
    print, '% SPH_SPEC: Unable to fine unique column for search string: '+db_keys[27], rr
  rr = get_match(db_keys[29], tags, nel = nel)
  if nel eq 1 then cols[rr[0]] = strtrim(width, 2) else if keyword_set(verbose) then $
    print, '% SPH_SPEC: Unable to fine unique column for search string: '+db_keys[29], rr
  
  rr = get_match(db_keys[28], tags, nel = nel)
  bdate = strsplit(thin_date(), /extract, count = nbd)
  if nbd eq 4 then mydate = bdate[0]+' '+bdate[1]+','+bdate[3]+' '+bdate[2] else $
       mydate= thin_date()
  if nel eq 1 then cols[rr[0]] = '"'+mydate+'"' else begin
    if keyword_set(verbose) then $
      print, '% SPH_SPEC: Unable to fine unique column for search string: '+db_keys[28]
    cols[n-1] = cols[n-1]+',"'+mydate+'"'
  endelse
  line = '"'+cols[0]+'"'
  for i = 1, ncol-1 do line = line+','+cols[i]

  printf, unit, line
  free_lun, unit
  readcsv, database_save, database_save.file_name
endif
if keyword_set(plotit) then begin
  ip = strpos(clab, ' - ')
  clab_save = strmid(clab, ip+3)
  xmid = [x0, 0.0, xf0]
  ymid = [y0, 0.0, yf0]
  plot_spec, xrange = xrange, yrange = yrange
endif
if keyword_set(saveit) then begin
  rc_save = rc
  cry2d_save = cry2d
  order_save = order
  length_save = len
  width_save = width
  distance_save = distance
  theta_save = theta
  npoints_save = np
  film_save = film
endif

end
