; ------------------------------------------------------------------------
; $Id: stats.pro,v 1.3 2007/06/06 20:03:54 lpmix Exp $
; ------------------------------------------------------------------------
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
;+
; NAME:
;       STATS
;
; PURPOSE:
;       This routine prints or returns statistical values about an IDL array.
;
; CATEGORY:
;       Statistics
;
; CALLING SEQUENCE:
;       STATS, ARRAY
;
; INPUTS:
;       ARRAY:	Array for which statistics are desired.
;               Minimum of 4 points required
;
; KEYWORD PARAMETERS:
;       QUIET:	If set, no output to the terminal
;       NPOINTS: Number of values
;       FINITE:  Number of valid values
;       VALID:   Linear array of valid values
;       NAN:     Number of values that are not a number
;       INFINITE: Number of values that are +/- infinity
;       ZERO:    Number of points that are zero
;
;   Following refer only to finite values:
;       MAXIMUM: Maximum value in the array.
;       MINIMUM: Minimum value in the array.
;       PEAK:    Maximum absolute value.
;       AVERAGE: Average value = TOTAL/NPOINTS
;       STANDARD: Standard deviation of the data
;       MEDIAN:  The middle value.  Half of the points are less than the 
;                median and half are greater than the median.
;       BQUARTILE: Value for which 25% are less than and 75% are greater than.
;       TQUARTILE: Value for which 75% are less than and 25% are greater than.
;       LOWER:   Lower estimate of the distribution.  Values below this should be 
;                considered as outliers.  
;                LOWER = BQUARTILE - 1.5*(TQUARTILE - BQUARTILE)
;       UPPER:   Upper estimate of the distribution.  Values above this should be 
;                considered as outliers.  
;                LOWER = TQUARTILE + 1.5*(TQUARTILE - BQUARTILE)
;
; OUTPUTS:
;       Statistics for the array are printed and returned in keywords.
;
; RESTRICTIONS:
;       Array of 4 or more elements must be provided.
;
; PROCEDURE:
;       NAN's and INF's are removed from the array and values calculated.
;
; EXAMPLE:
;       Obtain the statistics of the Y-values in WDF array: 5
;         STATS, GETYF(5)
;
; MODIFICATION HISTORY:
; 	Written by:	L. P. Mix, Jr., September 19, 2006
;-

PRO STATS, array, quiet = quiet, npoints = npoints, finite = nfin, $
   valid = valid, nan= nan_val, infinite = infinite, ZERO = zero, $
   maximum = maximum, minimum = minimum, peak = peak, $
   average = average, median = median_val, standard = standard, $
   bquartile = bquartile, tquartile = tquartile, lower = lower, upper = upper

on_error, 1
  npoints = n_elements(array)
  if npoints eq 0 then begin
    print, '% Format: STATS, ARRAY [, QUIET = quiet] [, NPOINTS = npoints]'
    print, '          [, FINITE = finite] [, VALID = valid] [, NAN= nan]'
    print, '          [, INFINITE = infinite] [, ZERO = zero] [, MAXIMUM = maximum]'
    print, '          [, MINIMUM = minimum] [, PEAK = peak] [,  AVERAGE = average]'
    print, '          [ , MEDIAN = median] [, STANDARD = standard]'
    print, '          [, BQUARTILE = bquartile] [, TQUARTILE = tquartile]'
    print, '          [, LOWER = lower] [, UPPER = upper]'
    return
  endif
  if npoints lt 4 then begin
    print, '% STATS: Less than 4 points in array.'
    help, array
    print, 'Values = ', array
    return
  endif

  valid = where (finite(array), nfin, complement = badarr, ncomplement = nbad)
  if nbad gt 0 then begin
    bad = array[badarr]
    ok = where(finite (bad, /nan) , nan_val)
    ok = where(finite (bad, /inf) , infinite)
    if nfin gt 0 then arr = array[valid]
  endif else begin
    arr = array
    infinite = 0
    nan_val = 0
  endelse
  if nfin eq 0 then begin
    if keyword_set(quiet) then return
    help, array
    print, 'Total number of points = ', strtrim(npoints, 2)
    print, 'Number of finite points = ', strtrim(nfin, 2)
    print, 'Number of infinite points = ', strtrim(infinite, 2)
    print, 'Number of NaN points = ', strtrim(nan_val, 2)
    return
  endif

  arr = arr[sort(arr)]
  minimum = arr[0]
  maximum = arr[nfin-1]
  if (minimum le 0) && (maximum ge 0) then $
    ok = where (array eq 0.0, zero) else zero = 0L
  peak = max(abs([minimum, maximum]))
  standard = sqrt((moment(arr, /Double, Maxmoment=2))[1])
  average = total(arr, /double)/nfin
  iadd = (nfin mod 2)
  nfin2 = nfin/2
; If an odd number of points, take the middle point.
; If an even number of points, take the average of the two middle points
  median_val = (iadd eq 1) ? arr[nfin2] : (arr[nfin2-1]+arr[nfin2])*0.5
; Go 1/2 point above the 1/4 point
  nfin_2 = nfin+2
  iadd = (nfin_2 mod 4) 
  nfin4 = nfin_2/4 -1
  bquartile =  ((4-iadd)*arr[nfin4] + iadd* arr[nfin4+1])*0.25
  nfin_32 = 3*nfin+2
  nfin4= nfin_32/4 -1
  iadd = (nfin_32 mod 4)
  tquartile = ((4-iadd)*arr[nfin4] + iadd* arr[nfin4+1])*0.25
  delta = (tquartile-bquartile)*1.5
  lower = bquartile - delta
  upper = tquartile + delta
  
  if keyword_set(quiet) then return
  help, array
  print, 'Total number of points = ', strtrim(npoints, 2)
  print, 'Number of finite points = ', strtrim(nfin, 2)
  print, 'Number of infinite points = ', strtrim(infinite, 2)
  print, 'Number of NaN points = ', strtrim(nan_val, 2)
  print, 'Number of zero values = ', strtrim(zero, 2)

  print, 'Maximum = ', strtrim(maximum, 2)
  print, 'Minimum = ', strtrim(minimum, 2)
  print, 'Peak Amplitude = ', strtrim(peak, 2)
  print, 'Average Value = ', strtrim(float(average), 2)
  print, 'Standard Deviation = ', strtrim(standard, 2)
  print, 'Median Value = ', strtrim(median_val, 2)
  print, 'Bottom Quartile = ', strtrim(bquartile, 2)
  print, 'Top Quartile = ', strtrim(tquartile, 2)
  print, 'Lower Data Estimate = ', strtrim(lower, 2)
  print, 'Upper Data Estimate = ', strtrim(upper, 2)

END
