; ------------------------------------------------------------------------ 
; $Id: stopping.pro,v 1.3 2007/06/06 20:03:54 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
 Pro stopping, zion,  ein, eout, thick, nmat, press, edel, ier, aion = aion, $
             quiet = quiet, ion_name = ion_name, foil_mat = foil_mat
;+ 
;  Calculate the ion range using the formulation of J.Maenchen
;
;   format: stopping, zion,  ein, eout, thick, mat [, press] [, edel] [, ier]
;            [, aion = aion]
;            [, quiet = quiet] [, ion_name = ion_name] [, foil_mat = foil_mat]
;     where:
;          zion    - z of the ion
;          ein     - input energy (Mev) (may be an array)
;          eout    - output energy (Mev) (may be an array)
;          thick   - foil thickness (microns) (may be an array)
;          mat     - z of the foil or one of several compounds listed below:
;         MAT = 0    VACUUM
;         MAT = 1,92 elements with Z = MAT
;         MAT = 101  polyethylene: C-H2,  rho = 0.92
;         MAT = 102  mylar:  C10-H8-O4,  rho = 1.395
;         MAT = 103  lithium fluoride:  Li-F,  rho = 2.601
;         MAT = 104  teflon: C-F2,  rho = 2.20
;         MAT = 105  kimfol: H14-C16-O3,  rho = 1.21
;         MAT = 106  kapton: C22-H10-N2-O4,  rho = 1.42
;         MAT = 107  parylene-n:  C8-H8,  rho = 1.11
;         MAT = 108  parylene-c:  C8-H7-Cl,  rho = 1.28
;         MAT = 109  parylene-d:  C8-H6-Cl2,  rho = 1.418
;         MAT = 110  erbium deuteride:  Er-D2,  rho = 8.64(used H in derivation)
;         MAT = 111  brass: Cu62-Zn35-Pb3,  rho = 8.49
;         MAT = 112  monel: Fe-Ni67-Cu32,  rho = 8.83
;         MAT = 113  inconel: Cr19-Fe18-Ni53-Mo3-Nb5,  rho = 8.19
;         MAT = 114  LB4: Li0.4-B1.8-O2.9, rho = 2.2
;         MAT = 115  LB5: Li0.2-B1.8-O2.8, rho = 2.2
;          press   - pressure if MAT is a gas; default = 1 torr
;          edel    - energy loss (ein-eout)
;          ier     - error parameter, number of iterations or problem if neg.
;          aion    - Ion mass in AMU. If not set, average atomic mass will be
;                    used for the stopping.
;          quiet   - if set, no output.
;                    Default: list the results
;          ion_name- Name of the ion
;          foil_mat- Name of the foil material
;
;   Discussion:
;         One of the parameters (ein, eout, thick) must be negative.
;         The negative parameter will be determined.
;         If any of the parameters (ein, eout, thick) are arrays then
;            the other parameters will be made into arrays of equal length
;            using the first value to pad arrays if needed.
;         Only 1 value of zion and nmat are allowed
;
;   Example
;         Determine the range of Li in Au at 1, 5 and 10 MeV
;           STOPPING, 3, [1,5,10], 0, -1, 79
;    or 
;           thick = -1 & STOPPING, 3, [1,5,10], 0, thick, 79
;          Note: The value of THICK will be changed to an array of thichnesses.
;-         
on_error, 1
      common pffcom, initpff, pff_exe
;
@com.dat
      if (n_elements(initpff) eq 0) then init_pff else $
        if (initpff eq 0) then init_pff
      if initpff eq 0 then begin
        print, '% STOPPING: Stopping calculations are not supported '
        return
      endif
      npar = N_params()
      if npar lt 5 then begin
        print, ' 5 parameters are required:
 print,'% Format: STOPPING, zion, ein(Mev), eout(Mev), thick(micron), nmat, '
 print,'          [, pressure] [, edel] [, ier] [, /quiet]'
 print,'          [, ion_name = ion_name] [, foil_mat = foil_mat] 
        return
      endif
   nein = n_elements(ein) 
   neout = n_elements(eout) 
   nth = n_elements(thick) 
   if (min([n_elements(zion),nein,neout,nth,n_elements(nmat)]) le 0) then begin
     print, ' First 5 parameters must have valid data'
     return
   endif
   zion = long(zion(0))
   if (zion lt 1 or zion gt 92) then begin
     print, ' ZION must be in the range [1, 92]'
     return
   endif
   nmat = long(nmat(0))
   if ((nmat lt 1 or nmat gt 115) or (nmat gt 92 and nmat lt 101)) then begin
     print, ' NMAT must be in the range [1, 92] or [101, 115]'
     return
   endif
   if n_elements(press) eq 0 then press = 1.0 else press = float(press>0.0)
   lstr = 12L
   ion_name = make_array(12, /byte, value = 32B)
   foil_mat = make_array(12, /byte, value = 32B)
   if ((ein(0) lt 0.0) and (eout(0) ge 0.0) and (thick(0) ge 0.0)) then begin
     nval = max([neout, nth])
     ein= make_array(nval, value = -1.0)
     eout = float(eout > 0.0)
     if neout lt nval then $
     eout=[eout, make_array(nval-neout, value = eout(0))]
     thick = float(thick > 0.0)
     if nth lt nval then $
     thick=[thick, make_array(nval-nth, value = thick(0))]
     edel = make_array(nval)
   endif else if ((eout(0) lt 0.0) and (thick(0) ge 0.0) and $
       (ein(0) ge 0.0)) then begin
     nval = max([nein, nth])
     eout= make_array(nval, value = -1.0)
     ein = float(ein > 0.0)
     if nein lt nval then $
     ein=[ein, make_array(nval-nein, value = ein(0))]
     thick = float(thick > 0.0)
     if nth lt nval then $
     thick=[thick, make_array(nval-nth, value = thick(0))]
     edel = make_array(nval)
   endif else if ((ein(0) ge 0.0) and (eout(0) ge 0.0) and $
       (thick(0) lt 0.0)) then begin
     nval = max([neout, nein])
     thick= make_array(nval, value = -1.0)
     eout = float(eout > 0.0)
     if neout lt nval then $
     eout=[eout, make_array(nval-neout, value = eout(0))]
     ein = float(ein > 0.0)
     if nein lt nval then $
     ein=[ein, make_array(nval-nein, value = ein(0))]
     edel = ein-eout
   endif else begin
     print, 'The first element of one parameter(ein, eout, thick) must be neg.'
     return
   endelse
   if n_elements(aion) eq 0 then ion_mass = 0.0 else ion_mass = float(aion)
   ierr = make_array(nval, /long)
   if initpff eq 1 then begin
; HP

     junk = call_external(PFF_EXE,'sl_foil_stopping', ein, eout, thick, edel, $
          nval, press, nmat, zion, ion_mass, ion_name, foil_mat, ierr, lstr) 
     ion_name = string(ion_name)
     foil_mat = string(foil_mat)
   endif else if initpff eq 2 then begin
; Sun
     junk = call_external(PFF_EXE,'sl_foil_stopping_', ein, eout, thick, edel, $
          nval, press, nmat, zion, ion_mass, ion_name, foil_mat, ierr, lstr) 
     ion_name = string(ion_name)
     foil_mat = string(foil_mat)
   endif else if initpff eq 4 then begin
; WIN
     junk = call_external(PFF_EXE,'_SL_FOIL_STOPPING@8',ein, eout, thick, edel, $
          nval, press, nmat, zion, ion_mass, ion_name, foil_mat, ierr, lstr)
     ion_name = string(ion_name)
     foil_mat = string(foil_mat)
   endif else if initpff eq 5 then begin
; MAC
     junk = call_external(PFF_EXE,'sl_foil_stopping', ein, eout, thick, edel, $
          nval, press, nmat, zion, ion_mass, ion_name, foil_mat, ierr, lstr) 
     ion_name = string(ion_name)
     foil_mat = string(foil_mat)
   endif else if initpff eq 3 then begin
; VMS
     ion_name = string(ion_name)
     foil_mat = string(foil_mat)
     junk = call_external(PFF_EXE,'foil_stopping', ein, eout, thick, edel, $
          nval, press, nmat, zion, ion_mass, ion_name, foil_mat, ierr) 
     
   endif
   if keyword_set(quiet) then return
   print, ' Stopping of ', strtrim(ion_name, 2), ' in ', strtrim(foil_mat)
   ok = where (nmat eq [1,2,7,8,9,10,17, 18, 36, 54, 86], nok)
   if nok gt 0 then print, ' (pressure = ', strtrim(press, 2),')'
   print, 'Input Energy  Output Energy   Thickness     Ener.Loss     Error param'
   print, '       (MeV)          (MeV)     (Micron)        (MeV)     (# of iter)'
   for i = 0, nval -1 do print, ein(i), eout(i), thick(i), edel(i), ierr(i)
   return
   end
