; ------------------------------------------------------------------------ 
; $Id: sum.pro,v 1.12 2011/11/16 21:16:00 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
 Pro  sum, wdfarray, ncur, save, plotit= plotit, clabel = clab, ylabel= ylab, $
          xlabel = xlab,  standard = standard, sdeviation = sd, $
          ncurves = ncurves, average = average, querry = querry, $
          sactual = sactual, actual = actual, all = all, weights = weight, $
          errors = error, discrete = discrete, statistical = statistical, $
          quiet = quiet

;
;+
; NAME:
;       SUM
;
; PURPOSE:
;       Take the sum of multiple WDF arrays into a new WDF array. Optionally
;         calculate the average of these curves.  
;       NOTE: If no overlap exists with the first curve, the data will be
;             ignored in the sum but the data will be included in determining
;             the denominator.
; CATEGORY:
;       Mathematics
; CALLING SEQUENCE:
;       SUM, WDA, NCURVE [,SAVE] [,/PLOTIT][,/AVERAGE][, NCURVES=NCURVES]
;                   [,/QUERRY] [, STANDARD = STANDARD] [, SDeviation = SD]
;                   [, SACTUAL = sactual] [, ACTUAL = actual]
;                   [, WEIGHTS = weights] [, ERRORS = errors]
; INPUTS:
;       WDA:      WDF array number or an array of WDF array numbers or a 
;                 search string.
;       NCURVE:   Number of curves to be plotted or the destination for the
;                 total of the arrays.
;                 If WDA is a single WDF array number, then WDA and the
;                   following (NCURVE-1) arrays will be totaled.  NCURVE is a 
;                   required input parameter.
;                 If WDA is an array of WDF arrays, then NCURVE is optional.
;                   If present, NCURVE will be interpreted as SAVE and the 
;                   third parameter is ignored.
; OPTIONAL INPUT:
;       SAVE:     WDF array number for the sum or average of the WDF arrays.
;                 The total is always stored in the last WDF array unless
;                   average is calculated and stored in that array
; KEYWORDS:
;       PLOT:     If present and not zero, plot the total.
;       AVERAGE:  If present and not zero, average the curves (see NOTE: above)
;                 AVERAGE will be set if DISCRETE, ERRORS or WEIGHTS are set.
;       STANDARD: If AVERAGE is set and STANDARD is not equal to zero, the 
;                 standard deviation from the average is calculated and stored
;                 in MAXWDFARRAY-1. If STANDARD is less than zero, the stand
;                 standard deviation will be stored in abs(standard). 
;                   standard = sqrt(sum(y^2)-n*(average^2)/(n-1))
;                 If ERRORS or WEIGHTS are set, then 
;                   standard = SQRT(1./TOTAL(WEIGHT))
;                 If average has only one value, then STANDARD = 0.0
;       SD        same as standard. Note: SD and Standard are resolved by:
;                     If keyword_set(sd), then standard = sd
;       NCURVES:  The number of curves used for the TOTAL and the 
;                 denominator in calculating the average.  This includes
;                 all arrays in the range with valid data.  (see NOTE above)
;       ALL:      If set, all curves will be added and averaged using entire
;                 domain of all curves.
;                 If ALL is not set, processing only in the area of overlap.
;                 Note: If ALL is set and STANDARD is set, then STANDARD is 
;                       defined equal to zero where n=1 unless errors
;                       are provided..
;                 If DISCRETE is set, then ALL is set.
;                 If ALL is set, then AVERAGE =1 unless it is defined = 0.
;       QUERRY:   If present and not zero, the user will indicate the 
;                   arrays to be summed or averaged.
;       CLABEL:   Replace the dataset comment for array number SAVE
;                 with the string or string variable specified
;       XLABEL:   Replace the dataset x-axis label for array number SAVE
;                 with the string or string variable specified
;       YLABEL:   Replace the dataset y-axis label for array number SAVE
;                 with the string or string variable specified
;       WEIGHTS:  WDF array of the weight for each for each point
;                 If weights is set and does not cover the domain of the data,
;                 the closest value will be used for continuous data.
;                 The number of WDF arrays must be the same as the
;                 number of waveforms to be summed or averaged.
;       ERRORS:   WDF arrays with the errors for each point.
;                 This keyword is not checked if WEIGHTS is set.
;                 If ERRORS is set and does not cover the domain of the data,
;                 the closest value will be used for continuous data.      
;                 ERRORS are the standard deviations for each point in the 
;                 waveform.  These errors are used to calculate weights.
;                 If STATISTICAL is not set,  weights are 1/(Error_Value)^2
;                 (for instrumental or Gaussian weights).
;                 If STATISTICAL is set, weights are  1/Error_Value
;                 (for statistical or Poisson weights).
;                 ERRORS must be greater than zero.  Values less than or equal
;                 to zero are set equal to 1E-3*MIN(Positiver_Values_of_ERRORS)
;       STATISTICAL : See ERRORS above
;       DISCRETE: If set, consider the WDF arrays data points as discrete elements.
;                 If DISCRETE is set, ALL and AVERAGE are set absolutely.
;                 Missing areas of a curve will not contribute to the average; ie,
;                 no interpolation across missing regions will be performed.
;                 The range of abscissa values for all waveforms is determined and the bin
;                 size determined for the SUM/AVERAGE array.
;                 If DISCRETE is positive, then the minimum spacing for each dataset
;                   is determined.  The SUM/AVERAGE array will have a bin size equal
;                   to 1.05 * MINIMUM (point spacing of all arrays)*DISCRETE
;                 If DISCRETE is negative, the SUM/AVERAGE array will have a gin size
;                   spacing of ABS(DISCRETE)
;                 Each waveform is processed sequentially.
;                 Every data point will be located in a bin; if multiple data points for a
;                   given waveform are in the same bin, they will be averaged and added to
;                   the bin.  No interpolation is used; points are either in or out of a bin.
;                 Bins without data are ignored.
;                 Bins in SUM/AVERAGE array with no data will be dropped, and the remaining
;                   bins will be divided by the number of waveformws which contributed to 
;                   the SUM/AVERAGE array in that bin.
;                 WEIGHTS and ERRORS are ignored for this option.
;                 The standard deviation will be set to zero if only one waveform contributes
;                   to a bin average.
;                 The number of curves is stored in (get_maxwdf()-1)
;       ACTUAL:   WDF arrays used in the sum/average
;       SACTUAL:  WDF arrays used in the sum/average as a string with spaces between values.
;       QUIET:    If set, do not print information messages.
; OUTPUTS:
;       NCURVES   Number of WDF arrays which have valid data
;       SUM     - Total of the designated arrays is stored in a WDF array.
; COMMON BLOCKS:
;       COM.DAT  WDF common block
; RESTRICTIONS:
;       WD1 array must be defined and have valid data 
;       The last WDF array will be used as a scratch array to calculate the sum
;       NCURVE may be changed 
; PROCEDURE:
;       If WDA is a single value:
;            WDA and the following NCURVE-1 datasets are added together and 
;            the result is stored into SAVE or the last WDF array if SAVE is
;            not provided.
;       If WDA is an array:
;            Each WDF array indicated by the elements in WDA is added together
;            and the total is stored in NCURVE or the last WDF array if 
;            NCURVE is not provided.
;       If DISCRETE is set:
;            The minimum spacing for each waveform is determined.
;            The value of bin size is then determined according to the value of DISCRETE
;              (1.05 * MINIMUM_SPACING*DISCRETE, or -DISCRETE)
;            Each waveform is then processed and points in a given bin are added to the 
;              sum in that bin.  If multiple points for a waveform are in the same bin,
;              they are averaged.  No interpolation is used and each waveform can add 
;              only zero or one point to a bin total.
; EXAMPLES:
;       Add WDF arrays 2 and 3 and store the result in WDF array 8.
;               SUM, 2, 2, 8
;      <or>     SUM, [2, 3], 8
;      <or>     ADD, 2, 3, 8
;
;       Add arrays 1, 2, 3 and store the average result in array 8.  Assume the data is
;         discrete and you wish to combine all points that are to an array that
;         has a spacing of 5 units.
;           SUM, 1, 3, 8, discrete = -5
;
;       Add arrays 2 through 7 and store the result in array total. Total
;         must be in the range [1, 82] or be undefined. Store standard deviation
;         in WDF array 30
;               SUM, 2, 6, total, stand = -30
;
;       Average selected arrays beginning with 'MOCAM2' and store the result
;         in array 65. Calculate the standard deviation and plot a summary.
;               SUM, /average, 'mocam2', /qu, 65, sd = -68
;               add, 65, 68, 66, cl='SD-Max'
;               sub, 65, 68, 67, cl='SD-Min'
;               plo, 'mocam2', /ov, /co & plo, 65, 3, /ov, /co, /no
;
;       Add arrays 12 through 28 and store the result in array 8
;               SUM, 12, 28-11, 8
;
;       Add arrays 2, 4, 5, 6, 12, 15, plot the sum and store the result in
;         array 8
;               SUM, [2, 4, 5, 6, 12, 15], 8, /plot
;
;       Add arrays 2 through 7, calculate the average and store the average
;         in array 7 with the sum in array 42 and the number of arrays in narr.
;               SUM, 2, 6, 7, /average, n = narr
;
;       Add WDF the selected arrays 2 through 17, plot the total and 
;         store the result in WDF array 42.
;               SUM, 2, 17-2+1 , /plot, /querry
;
;       Note the "Select ALL" button can be used to select all the arrays 
;         and then a few arrays can be deselected.
;
; MODIFICATION HISTORY:
;      Original version, L. P. Mix, December 15, 1992
;      March 15, 2010:   LPM, Modified handling of discrete data.
;-
; include wdf common blocks
;
@com.dat
on_error, 1
;
   common wselect_com, querrylist
;
;  determine if wdfarray is good
;
   if n_elements(initwdf) eq 0 then wdfinit

   nparams = n_params()
   if nparams eq 0 then begin
print, '% Format: SUM, WDA, NCURVE [,SAVE] [,/PLOT][,/AVERAGE][, NCURVES=NCURVES]'
print, '                [,/QUERRY] [, STANDARD = STANDARD] [, SDeviation = SD]'
     return
   endif
   if n_elements(wdfarray) eq 0 then begin
     print, '% SUM: WDF array must be specified'
     return
   endif
   
   a = size(wdfarray)
   if a(a(0)+1) eq 7 then begin
     if a(0) ne 0 then print, '% SUM: Only the first string is used'
     wdf = wdfarray[0]
     if strtrim(wdf, 2) eq '*' then wdf = wdvalid() else $
       wdf = wdvalid(wdf)
     if wdf[0] lt 1 then begin
       print, '% SUM: No arrays found using the search string: ', wdfarray[0]
       return
     endif
   endif else wdf = wdfarray
   nel =  n_elements(wdf)
   if nel eq 1 then begin
     if n_elements (ncur) eq 0 then begin
       print, '% SUM: Number of WDF arrays must be specified'
       return
     endif
     if ncur eq 0 or ncur lt 0 then begin
       print, '% SUM: Number of WDF arrays must be > zero'
       return
     endif
     lcsave = 2
     wdfp = make_array(ncur, /int, /index)+wdf
   endif else begin
     wdfp = wdf
     if n_elements(ncur) gt 0 then save = ncur
     lcsave = 1
   endelse
   ncurves = n_elements(wdfp)
   j=0
   if n_params() gt lcsave then save= get_save_array(save, maxwdfarray) $
     else save = maxwdfarray
   if lcsave eq 1 then ncur = save
;     use a scratch array for the sum 
   saveperm = save & save = maxwdfarray
   for i = 0, ncurves-1 do begin
;       first check for a valid array
     if  wdfp[i] lt 1 or wdfp(i) gt maxwdfarray then begin
       print, '% SUM: WDF array must be greater than 0 and </=', maxwdfarray
     endif else if (wdfnp(wdfp[i]-1) lt 2) then $
       print, '% SUM: No data in the wdf array =', wdfp[i] else wdfp[j++] = wdfp[i]
   endfor
   ncurves = j
   if j eq 0 then begin
     print, '% SUM: No valid data for the total' 
     return
   endif
   wdfp= temporary(wdfp(0:j-1))
   if n_elements (querry) eq 0 then querry = 0
   if querry ne 0 then begin
     list = wdfcomment [wdfp-1]
     wselect_item, list
     j = total (Querrylist)
     if j eq 0 then begin
       print, '% SUM: No valid data for the total'
       return
     endif
     a = where (querrylist ne 0)
     wdfp= wdfp[a]
   endif
   if not keyword_set(quiet) then print, '% SUM: Processing arrays' , wdfp
   n= n_elements(wdfp)
   ncurves = n
   if n lt 2 then begin
     print, '%SUM: Less than 2 arrays; no processing performed'
     return
   endif
   actual = wdfp
   sactual = strtrim(wdfp[0], 2)
   for i = 1, n-1 do sactual = sactual+' '+strtrim(wdfp[i], 2)
   wdfpm = wdfp-1
   if keyword_set(discrete) then all = 1
   if keyword_set(all) then begin
     if n_elements(average) eq 0 then average = 1
   endif
   if keyword_set(all) then begin
     mint = min(wdfminx(wdfpm))
     maxt = max(wdfmaxx(wdfpm))
   endif else begin
     mint = max(wdfminx(wdfpm))
     maxt = min(wdfmaxx(wdfpm))
     if maxt le mint then begin
       print, '% SUM: Arrays cannot be put on a common time base.'
       print, '       Minimum end, Maximum start: ' $
         + strtrim (maxt, 2) + ', '+strtrim (mint, 2)
       return
     endif
   endelse
   if keyword_set(sd) then standard = sd
   if keyword_set(standard) then ss = (standard gt 0) ? maxwdfarray-1 : $
         get_save_array(-standard, maxwdfarray-1)
   if keyword_set(weight) then begin
     If n_elements (weight) ne n then begin
       print, '% SUM: Cannot calculate SUM'
       print, '       Number of Weights:', n_elements(weight)
       print, '       Number of Waveforms:', n
       return
     endif
     ok = where(weight gt 0 and weight lt maxwdfarray, nok)
     if nok ne n then begin
       print,'% SUM: Bad values of weight'
       print,'       ', weight
       return
     endif
     lcweight =1
   endif else if keyword_set(error) then begin
     If n_elements (error) ne n then begin
       print, '% SUM: Cannot calculate SUM'
       print, '       Number of ERROR:', n_elements(error)
       print, '       Number of Waveforms:', n
       return
     endif
     ok = where(error gt 0 and error lt maxwdfarray, nok)
     if nok ne n then begin
       print,'% SUM: Bad values of error:'
       print,'       ', error
       return
     endif
     if min(wdfnp[error-1]) lt 3 then begin
       print,'% SUM: Some ERROR arrays have no data'
       return
     endif
     lcweight = -1
   endif else lcweight = 0
   if keyword_set(lcweight) then average = 1
   if keyword_set(discrete) then begin
;    all = 1, average = 1
     if discrete lt 0 then delta = float(-discrete[0]) else begin
       delta = 1.0e35
       for i = 0L, n-1 do begin
         xx = getxf(wdfp[i])
         nxx = n_elements(xx)
         dx = min(abs(xx[1L:nxx-1]-xx[0L:nxx-2]))
         delta = (delta le dx) ? delta : dx
       endfor
       delta = 1.05*delta*discrete[0]
     endelse
     nux = round((maxt-mint)/delta +1.0)
     uniqx = mint+0.5*delta +findgen(nux)*delta
     averag = make_array(nux, value = 0.0d0, /double)
     lcweight = 0
     y2= make_array(nux, value = 0.0d0)
     weig = make_array(nux, value = 0.0d0)
     for i = 0L, n-1 do begin
       ival = lonarr(nux)
       ydata = dblarr(nux)
       w2i, x, ydatain, wdfp[i]
       indin = round((((x-uniqx[0])/delta) > 0.0) < (nux-1))
       nx = n_elements(indin)
       for ii = 0, nx-1 do ival[indin[ii]] +=1
       for ii = 0, nx-1 do ydata[indin[ii]] += ydatain[ii]
       indy = where (ival gt 0, nok)
       if nok gt 0 then begin
         ydata = ydata[indy]/ival[indy]
         averag[indy] += ydata
         weig[indy] += 1d0
         y2[indy] += ydata*ydata
       endif
     endfor
;    check to see if all of the data has weights if they are required
     ok = where (weig gt 0 , nok)
     if nok lt 3 then begin
       Print, '% SUM: Unable to calculate sum with less than 3 points'
       help, delta, nok
       return
     endif
     averag= averag[ok]
     weig = weig[ok]
     y2 = y2[ok]
     uniqx = uniqx[ok]
     put_wdfarray, [uniqx, weig], get_maxwdf()-1, /xydata, /no_copy, $
             file = '', clab = 'Number of WDF Arrays in Average', $
             xlab = wdfxlabel[wdfp[0]-1], ylab =  'Number of WDF Arrays'

     put_wdfarray, [uniqx, averag], save, /xydata, file = '', $
         clab = 'Total of WDF Arrays', $
         xlab = wdfxlabel[wdfp[0]-1], ylab =  wdfylabel[wdfp[0]-1]
     averag = averag/weig
     if keyword_set(standard) then begin
       std_dev = sqrt(((y2- weig*averag^2)/((weig -1.0d0)> 1.0)) > 0.0)
       oksd = where (weig eq 1, noksd)
       if noksd gt 0 then std_dev[oksd] = 0.0
       put_wdfarray, [uniqx, std_dev], ss, /xydata, $
         clab = 'Standard Deviation', /no_copy, $
         xlab= wdfxlabel[wdfpm[0]], ylab = wdfylabel[wdfpm[0]]
     endif
     put_wdfarray, [uniqx, averag], saveperm, /xydata, /no_copy, $
             file = '', clab = 'Average of WDF Arrays', $
             xlab = wdfxlabel[wdfp[0]-1], ylab =  wdfylabel[wdfp[0]-1]     
     save = saveperm
     if n_elements(clab) ne 0 then wdfcomment(save-1) = clab
     if n_elements(xlab) ne 0 then wdfxlabel(save-1) = xlab
     if n_elements(xlab) ne 0 then wdfxlabel(save-1) = ylab
     if keyword_set(plotit) then plo, save
     return
   endif
;  Continuous waveforms   
   del = make_array(n)
   for i = 0, n-1 do begin
     get_wdfarray, a,wdfp(i), /xy
     np = wdfnp(wdfpm(i))
     if n_elements (a) eq np+2 then del(i) = a(1) else begin
       i2w, a(0:np-1), a(np:*), delta, /get_delta
       del(i) = delta
     endelse
   endfor
   delta = min(del)
   if delta eq 0 then begin
     print, '% SUM: Cannot use a delta of zero'
     return
   endif
   nf = n
   nptm =round((maxt-mint)/delta)
   npt = nptm+1
;  Calculate delta to end on mint/maxt
   delta = (maxt-mint)/(npt-1)
;  Define weight array and total array
   weig = (lcweight eq 0) ? make_array(npt, value= 0L) : $
     make_array(npt, value = 0.0d0)
   averag = make_array(npt,value = 0.0d0)
   xarr = mint+findgen(npt)*delta
   if keyword_set(standard) then tarr2 = make_array(npt, /double)
   if keyword_set (all) then begin
     for i = 0, nf-1 do begin
       get_wdfarray, a, wdfp(i), /xy
       st = wdfminx[wdfpm[i]]
       en = wdfmaxx[wdfpm[i]]
       ist = ceil((st-mint)/delta)
       ien = floor((en-mint)/delta)
       nn = ien-ist+1
       np = wdfnp[wdfpm[i]]
       if np+2 eq n_elements(a) then begin
;        uniform data
         offset = (ist*delta - st+mint) > 0.0
         sp = offset/a[1] +findgen (nn)*(delta/a[1])
         arr2 = interpolate(a[2:*], sp )
       endif else begin
         xx = a(0:np-1)
         yy = a(np:*)
         s = sort(xx)
         xx = xx(s)
         yy = yy(s)
         st = (mint+ist*delta) > st
         put2uniform, xx, yy, np, st, delta, en, arr2, /yonly
       endelse
       ien = (ist+n_elements(arr2)-1) 
       if ien gt nptm then begin
         arr2=arr2[0:ien-ist]
         ien = ien-1
       endif
       if lcweight eq 0 then begin
         averag[ist:ien] += arr2
         weig[ist:ien] += 1
         if keyword_set(standard) then tarr2[ist:ien] += arr2*arr2
       endif else begin
;        most users will not have weight or error - keep coding short
         ww = (lcweight lt 0) ? error[i] : weight[i]
         get_wdfarray, a, ww, /xy
         np = wdfnp[ww-1]
         if np+2 eq n_elements(a) then begin
;        Uniform data
           x = a[0]+findgen(np)*a[1]
           yy = a[2:*]
         endif else begin
;        XY data
           jj = sort(a[0:np-1])
           x = (a[0:np-1])[jj]
           yy = (a[np:*])[jj]
         endelse
         wy = interpolate(yy, findex(x,xarr[ist:ien]))                  
         ok = where (wy le 0 , nok, comp = comp)
         if nok gt 0 then wy[ok] = 1e-3 * min(wy[comp])
         if lcweight lt 0 then wy = (keyword_set(statistical)) ? 1./wy : 1./(wy*wy)
         averag[ist:ien] += arr2*wy
         weig[ist:ien] += wy
       endelse
     endfor
   endif else begin
     for i = 0, n-1 do begin
       get_wdfarray, a, wdfp(i), /xy
       np = wdfnp(wdfpm(i))
       if np+2 eq n_elements(a) then begin
         if (np eq npt) and (a(0) eq mint) and (a(1) eq delta) then $
           arr2 = a(2:*) else begin
           sp = (mint-a(0))/a(1) + findgen(npt)*(delta/a(1))
           spi = long(sp)
           spx = sp-spi
           delarr = [a(3:np+1)-a(2:np), 0.0]
           arr2 = a(spi+2)+spx*delarr(spi)
         endelse
       endif else begin
         xx = a(0:np-1)
         yy = a(np:*)
         s = sort(xx)
         xx = xx(s)
         yy = yy(s)
         put2uniform, xx, yy, np, mint, delta, maxt, arr2, /yonly
       endelse
       if lcweight eq 0 then begin
         averag += arr2
         weig += 1
         if keyword_set(standard) then tarr2 += arr2*arr2
       endif else begin
;        most users will not have weight or error - keep coding short
         ww = (lcweight lt 0) ? error[i] : weight[i]
         get_wdfarray, a, ww, /xy
         np = wdfnp[ww-1]
         if np+2 eq n_elements(a) then begin
;          Uniform data
           x = a[0]+findgen(np)*a[1]
           yy = a[2:*]
         endif else begin
;          XY data
           jj = sort(a[0:np-1])
           x = (a[0:np-1])[jj]
           yy = (a[np:*])[jj]
         endelse
         wy = interpolate(yy, findex(x,xarr))
         if lcweight lt 0 then wy = (keyword_set(statistical)) ? 1./wy : 1./(wy*wy)
         averag += arr2*wy
         weig += wy
       endelse
     endfor
   endelse
;  check to see if all of the data has weights if they are required
   ok = where(weig gt 0, nok, ncomp = ncomp)
   if nok eq 0 then begin
     print, '% SUM: All weights are zero'
     return
   endif
   if ncomp gt 0 then begin
     averag = averag[ok]
     weig = weig[ok]
     xarr = xarr[ok]
     if n_elements(tarr2) gt 2 then tarr2= tarr2[ok]
     xydata = 1
   endif else xydata = 0

   if keyword_set(standard)   then begin
     if lcweight eq 0 then begin
       okzp = where (weig eq 1, nokzp)
       nm = double(weig-1) > 1.0d0
       tarr2 =(tarr2-averag*averag/weig)/nm
       tarr2 = sqrt(tarr2 > 0.0d0)
       if nokzp gt 0 then tarr2[okzp] = 0.0
     endif else tarr2 =  sqrt(1./weig)
     ss = (standard gt 0) ?  maxwdfarray-1 : $
          get_save_array(-standard, maxwdfarray-1)
     tarr2 = (ncomp gt 0)? [xarr, float(tarr2)] : [mint, delta, float(tarr2)]
     put_wdfarray, tarr2, ss, clab = 'Standard Deviation', /no_copy, $
         xlab= wdfxlabel[wdfpm[0]], ylab = wdfylabel[wdfpm[0]], xydata = xydata
   endif
   tarr = (ncomp gt 0)? [xarr, float(averag)] : [mint, delta, float(averag)]
   put_wdfarray, tarr,save, file = '', clab = 'Total of WDF Arrays', xydata = xydata, $
          /no_copy, xlab = wdfxlabel[wdfp[0]-1], ylab =  wdfylabel[wdfp[0]-1]

   if keyword_set(average) then begin
     average_val = averag/weig
     averag = (ncomp gt 0) ? [xarr, float(average_val)] : [mint, delta, float(average_val)]
     put_wdfarray, averag,saveperm, file = '', clab = 'Average of WDF Arrays', $
          /no_copy, xlab = wdfxlabel[wdfp[0]-1], ylab =  wdfylabel[wdfp[0]-1], $
          xydata = xydata
   endif else if save ne saveperm then xfr, save, saveperm
   save = saveperm
   if n_elements(clab) ne 0 then wdfcomment(save-1) = clab
   if n_elements(xlab) ne 0 then wdfxlabel(save-1) = xlab
   if n_elements(xlab) ne 0 then wdfxlabel(save-1) = ylab
   if keyword_set(plotit) then plo, save
 end
