; ------------------------------------------------------------------------ 
; $Id: tlparam.pro,v 1.4 2010/09/17 21:23:38 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
 Pro tlparam, wdfa1, wdfb1, defn, wdfa2, wdfout, dz, $
              res=res, fmax=fmax, quiet=quiet, GHz=GHz, MHz=MHz
;
;+
; NAME: 
;	TLPARAM
; PURPOSE:
;	This procedure computes and (optionally) displays 
;	transmission line parameters as a function of frequency from 
;	voltage/current/power time histories.
; CATAGORY:
;	Math
; CALLING SEQUENCE:
;	TLPARAM, Wdf1a, Wdf1b, Defn, Wdf2a, Wdfout, Distance
; INPUTS:
;    First define the inpedance using the first 3 parameters:
;    Impedance is Z = V/I = V^2/P = P/I^2
;    Any combination of V, I and P may be used and indicated in Defn
;	Wdf1a: voltage, current or power time history at position #1
;	Wdf1b: voltage, current or power time history at position #1
;	Defn: definition of impedance: 'iv', 'vi', 'pv', 'vp', 'pi', 'ip'
;             Alternatively, 1, 2, 3, 4, 5, 6 may be used.
;             where I represents current, V is voltage, adn P is power
;             and the two elements are in Wdf1a and Wdf1b respectively
;             Default = 'IV'
;	Wdf2a: voltage, current or power time history at position #2
;              Must be the same parameter as in Wdf1a.
;	Wdfout: WDF array number to store transmission line parameters
;               Output will be stored in this WDF array and the next 6 arrays
;               (see Outputs)
;	Distance: Distance in meters between Position #1 and Position #2
; KEYWORD PARAMETERS:
;	Res: maximum desired frequency resolution
;            Default value is the previous value if this routine
;            has be run in the current session or 1/(np*dt) where
;            np is the number of points and dt is the point spacing.
;	Fmax: maximum frequency component to compute
;            Default value is the previous value if this routine
;            has be run in the current session or the value 
;            min(1024*res, 0.5/dt) where dt is the point spacing
;       Note: Res and Fmax are approximate and will be used as guides.
;             dt will be modified to produce good values for the FFT
;	Quiet: if present and non-zero, the parameters are not displayed
;	GHz: if present and non-zero, use frequency units of GHz
;	MHz: if present and non-zero, use frequency units of MHz
; OUTPUTS:
;	Wdfout  : Re(Z0) (Real part of the impedance)
;	Wdfout+1: Im(Z0) (Imaginary part of the impedance)
;	Wdfout+2: alpha = Re(gamma)
;	Wdfout+3: beta  = Im(gamma)
;	Wdfout+4: ereff = (beta*c/omega)**2
;	Wdfout+5: PUL L = Z0*beta/omega
;	Wdfout+6: PUL C = beta/omega*Z0
; MODIFICATION HISTORY:
;	Original version, M. F. Pasik, March 18, 1994
;	Modified for different impedance definitions, March 21, 1994
;	Changed method for computing phase, M. F. Pasik, April 29, 1994
;	Added FFT common block/keywords, M. F. Pasik, May 16, 1994
;	Added quiet keyword, M. F. Pasik, June, 9, 1994
;	Fixed bug in the computation of alpha, M. F. Pasik, June 22, 1994
;	Improved user interface, M. F. Pasik, August 5, 1994
;	Modified method for computing phase, M. F. Pasik, August 9, 1994
;	Added some checks, modified some mechanics, L. P. Mix, February 15, 1995
;-
;
;  common block variables for FFT parameters
;
;  fftres: maximum desired frequency response resolution (Hz)
;  fftmax: maximum desired frequency response component (Hz)
;
;
common fftpar, fftres, fftmax
;
;
if n_params() eq 0 then begin
  print, '% Format: TLPARAM, Wdf1a, Wdf1b, DEFN, Wdf2a, Wdfout, distance'
  print, '          [, RES = res ]  [, FMAX = fmax ] [, /QUIET ]'
  print, '          [, /GHz ] [, /MHz ]'
  return
endif
on_error, 1

fact=1.0
if keyword_set(GHz) then fact=1e-9
if keyword_set(MHz) then fact=1e-6

c = sol()
mu0 = 4.0e-7*!pi
eps0 = 1.0/(mu0*c^2)

;
;  set-up the required time histories
;
 if n_elements(wdfa1) eq 0 or n_elements(wdfb1) eq 0  $
   or n_elements(wdfa2) eq 0 or n_elements(wdfout) eq 0 then begin
   print, 'Input and Output waveforms must be defined'
   return
 endif
 if n_elements(dz) eq 0 then begin
   Print, 'Distance between measurements must be defined'
   return
 end
 if dz le 0 then begin
   print, 'Distance between measurements must be >= zero'
   return
 endif
 maxwdf = get_maxwdf()
 if wdfa1 le 0 or wdfa1 gt maxwdf then begin
   print, 'WDF array must be >zero and less than ', strtrim(maxwdf, 2)
   return
 endif
 if wdfa2 le 0 or wdfa2 gt maxwdf then begin
   print, 'WDF array must be >zero and less than ', strtrim(maxwdf, 2)
   return
 endif
 if wdfb1 le 0 or wdfb1 gt maxwdf then begin
   print, 'WDF array must be >zero and less than ', strtrim(maxwdf, 2)
   return
 endif
 if wdfout le 0 or wdfout gt maxwdf-6 then begin
   print, 'WDF array must be >zero and less than ', strtrim(maxwdf-6, 2)
   return
 endif

xfr, wdfa1, wdfout+2
xfr, wdfa2, wdfout+3

if n_elements(defn) eq 0 then defn = 'iv'
a = size(defn)
if a(a(0)+1) ne 7 then begin
 defn = round (defn)
 if defn gt 0 and defn lt 7 then begin
   defn = strmid('  ivvipvvppiip', 2*defn, 2)
 endif else begin
   print, 'Impedance must be defined'
   return
 endelse
endif

case strupcase(defn) of
;
;  voltage-current definition of impedance (a1 = v1, b1 = i1)
;
  'VI' : begin
           xfr, wdfa1, wdfout
           xfr, wdfb1, wdfout+1
         end

;
;  current-voltage definition of impedance (a1 = i1, b1 = v1)
;
  'IV' : begin
           xfr, wdfa1, wdfout+1
           xfr, wdfb1, wdfout
         end

;
;  power-voltage definition of impedance (a1 = p1, b1 = v1)
;
  'PV' : begin
           xfr, wdfa1, wdfout+1
           mul, wdfb1, wdfb1, wdfout
         end

;
;  voltage-power definition of impedance (a1 = v1, b1 = p1)
;
  'VP' : begin
           mul, wdfa1, wdfa1, wdfout
           xfr, wdfb1, wdfout+1
         end

;
;  power-current definition of impedance (a1 = p1, b1 = i1)
;
  'PI' : begin
           xfr, wdfa1, wdfout
           mul, wdfb1, wdfb1, wdfout+1
         end

;
;  current-power definition of impedance (a1 = i1, b1 = p1)
;
  'IP' : begin
           mul, wdfa1, wdfa1, wdfout+1
           xfr, wdfb1, wdfout
         end
;
;  unknown
;
  else : begin
           print, 'TLPARAM: unknown impedance definition ', defn
           return
         end

endcase

;
;  pad the time histories for the FFT to get the desired resolution
;
if n_elements(fftres) eq 0 then fftres = 0
if N_elements(res) gt 0 then fftres=res/fact 

if n_elements(fftmax) eq 0 then fftmax = 0
if N_elements(fmax) gt 0 then fftmax=fmax/fact 

get_wdfarray, array0, wdfout
get_wdfarray, array1, wdfout+1
get_wdfarray, array2, wdfout+2
get_wdfarray, array3, wdfout+3

np0 = n_elements(array0) -2
np1 = n_elements(array1) -2
np2 = n_elements(array2) -2
np3 = n_elements(array3) -2

startx= max([array0(0), array1(0), array2(0), array3(0)], min = startm)
deltax= max([array0(1), array1(1), array2(1), array3(1)], min = deltam)

if fftres eq 0 then fftres =1.0/ max ([ np0*array0(1), $
      np1*array1(1), np2*array2(1), np3*array3(1)])
if fftmax eq 0 then fftmax = min ([1024*res, 0.5/deltax])


if (deltax-deltam)/deltax lt 1e-5 then deltax = 0.5*(deltax+deltam)
 if startx ne 0 then begin
if (startx-startm)/startx lt 1e-5 then startx = 0.5*(startx+startm)
 endif

range0 = np0*array0(1) - startx + array0(0)
range1 = np1*array1(1) - startx + array1(0)
range2 = np2*array2(1) - startx + array2(0)
range3 = np3*array3(1) - startx + array3(0)
range = max([range0, range1, range2, range3, 1.0/fftres])

; Set a target dt that is about twice the desired response unless 
; it is smaller than then input value

targetdt = max([deltax, .35/fftmax])
npout = range/targetdt
npout = 2L ^ (long(alog(npout)*1.4427+1.01))
;  range = npout*targetdt
  targetdt = range/(npout-1)
dtratio = round(targetdt/deltam+1)
dummy = make_array(dtratio+1)
stopp = range+startx


if range gt range0 then begin
   image = [array0(2:*), dummy]
   nel = np0+dtratio +1
   space = findgen(nel)*array0(1)+array0(0)
   space(np0+dtratio)= stopp+targetdt
endif else begin
   space = findgen(np0)*array0(1)+array0(0)
   image = array0(2:*)
   nel = np0
endelse
if targetdt gt 2.*array0(1) then $
       image  =  smooth(image, targetdt/array0(1))
array0 = 0
put2uniform, space, image, nel, startx, targetdt,stopp,array0, /yonly


if range gt range1 then begin
   image = [array1(2:*), dummy]
   nel = np1+dtratio +1
   space = findgen(nel)*array1(1)+array1(0)
   space(np1+dtratio)= stopp+targetdt
endif else begin
   space = findgen(np1)*array1(1)+array1(0)
   image = array1(2:*)
   nel = np1
endelse
if targetdt gt 2.*array1(1) then $
       image  =  smooth(image, targetdt/array1(1))
array1 = 0
put2uniform, space, image, nel, startx, targetdt,stopp,array1, /yonly


if range gt range2 then begin
   image = [array2(2:*), dummy]
   nel = np2+dtratio +1
   space = findgen(nel)*array2(1)+array2(0)
   space(np2+dtratio)= stopp+targetdt
endif else begin
   space = findgen(np2)*array2(1)+array2(0)
   image = array2(2:*)
   nel = np2
endelse
if targetdt gt 2.*array2(1) then $
       image  =  smooth(image, targetdt/array2(1))
array2 = 0
put2uniform, space, image, nel, startx, targetdt,stopp,array2, /yonly


if range gt range3 then begin
   image = [array3(2:*), dummy]
   nel = np3+dtratio +1
   space = findgen(nel)*array3(1)+array3(0)
   space(np3+dtratio)= stopp+targetdt
endif else begin
   space = findgen(np3)*array3(1)+array3(0)
   image = array3(2:*)
   nel = np3
endelse
if targetdt gt 2.*array3(1) then $
       image  =  smooth(image, targetdt/array3(1))
array3 = 0
put2uniform, space, image, nel, startx, targetdt,stopp,array3, /yonly


;
;  generate the frequency domain data
;

df = 1.0/range
freq = df*indgen(npout)
omega = 2.0*!pi*freq

num = fft(array0, -1)
denom = fft(array1, -1)
a1 = fft(array2, -1)
a2 = fft(array3, -1)

;
;  compute and display the transmission line parameters
;
ok = where (abs(denom) eq 0, nok)
if nok gt 0 then denom(ok) = 1.0
z0 = num/denom
if nok gt 0 then begin
  z0(ok) = 0.0
  print, 'The following frequencies had a resonance:'
  print, freq(ok)
endif

npidl = fftmax/df
z0 = z0(0:npidl)
freq = freq(0:npidl)*fact
i2w, freq, float(z0), wdfout, clabel='Z0 (' + strupcase(defn) + ')'

i2w, freq, imaginary(z0), wdfout+1, $
     clabel='Z0 (' + strupcase(defn) + ') imaginary'

a1 = a1(0:npidl)
a2 = a2(0:npidl)
temp = a1/a2
alpha = alog(abs(temp))/dz
;alpha = float(alog(temp))/dz
;phase = atan(imaginary(temp),float(temp))
phase = imaginary(alog(temp))
m=0
for j=1L,npidl do begin
  phase(j) = phase(j) + m*2.0*!pi
  if ( phase(j) lt phase(j-1) ) and $
     ( abs(phase(j)-phase(j-1)) gt (0.5*!pi) ) then begin
    m = m + 1
    phase(j) = phase(j) + 2.0*!pi
  endif 
endfor
pbeta = phase/dz

omega = omega(0:npidl)
i2w, freq, alpha, wdfout+2, clabel='alpha'

i2w, freq, pbeta, wdfout+3, clabel='beta'

i2w, freq(1:*), ((c*pbeta(1:*))/omega(1:*))^2, wdfout+4, $
     clabel='ereff'

i2w, freq(1:*), float(z0(1:*)*pbeta(1:*)/omega(1:*)), wdfout+5, $
     clabel='PUL L'

if nok gt 0 then begin
  ok2 = where (ok le npidl, nok)
  if nok gt 0 then z0(ok(ok2)) = 1.0
endif

i2w, freq(1:*), float(pbeta(1:*)/(omega(1:*)*z0(1:*))), wdfout+6, $
      clabel='PUL C'

if not keyword_set(quiet) then begin
  pmsave = !p.multi
  !p.multi = [0, 2, 2, 0, 0]
  plo, [wdfout,  wdfout+1], /ov, /nsym, col=1, title='Characteristic Impedance'
  plo, [wdfout+2,wdfout+3], /ov, /nsym, col=1, title='Propagation Constant'
  plo, wdfout+5
  plo, wdfout+6
  !p.multi = pmsave
endif

return
end


