; ------------------------------------------------------------------------
; $Id: veloc.pro,v 1.6 2010/09/17 21:23:38 lpmix Exp $
; ------------------------------------------------------------------------
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
;+
; NAME:
;       VELOC
;
; PURPOSE:
;       Return the velocity of a particle from its kinetic energy
;       See: ENERG for energy from velocity.
;
; CALLING SEQUENCE:
;       VELOCITY = VELOC(Energy [, MASS] [, /RELATIVISTIC]
;                       [, /PROTON] [, /DEUTERON] 
;                       [, /ELECTRON] [, /NEUTRON])
;
; OUTPUT:
;       VELOCITY: Particle velocity (meters/second)
; INPUTS:
;       Energy:	  Particle kinetic energy in eV or an array of energies. 
;       MASS:     Particle mass in AMU.
;                 1 AMU = 1.66053873e-27 kg
;                         931494013 eV (Energy equivalent)
;                 DEFAULT: Mass = 1
;       Note: Values above are approximate.  PHYCONST is used.
; KEYWORD PARAMETERS:
;       RELATIVISTIC: If set, use relativistic calculation.
;                     Generally if ENERGY is less than 1e-6 of rest mass,
;                     classical calculations are used.
;                     (Beta (= V/C) less than 0.001414)
;       DOUBLEPRECISION: If set, return a double precision number.
;
;   NOTE: The following keywords are NOT checked if MASS is present.
;       Once a keyword is found, further checking is stopped.
;      Note: Values below are approximate.  PHYCONST is used.
;       PROTON:   If set, assume a proton - MASS = 1.00727646688D0
;       DEUTERON: If set, assume a deuteron - MASS = 2.01355321271D0
;       NEUTRON:  If set, assume a neutron - MASS = 1.00866491578D0
;       ELECTRON: If set, assume an electron - MASS = 5.485799110D-4
;
; OUTPUTS:
;       Particle velocity for the specified energies.
;
; RESTRICTIONS:
;       Energies should be greater than 0.0.
;
; PROCEDURE:
;       The velocity is calculated using
;      For: ENERGY < 1e-6*MASS(eV)
;         Velocity = SQRT(ENERGY *2*1.602176462d-19/MASS(kg))
;      For: ENERGY >  1e-6*MASS(eV)
;         Velocity = SQRT((MASS(eV)+ MASS(eV)+ Energy)*ENERGY)/(1.0+ENERGY/MASS(eV))/MASS(eV)*SOL()
;
;         The Momentum, P, is equal to Gamma*Beta*Mass
;         Where Gamma = (Energy+MASS)/MASS
;               Beta = Velocity/(Speed of Light)
;               
;         These equations approximately use:
;            Mass = 1.66053873 kg/AMU
;                 = 931.494013D6 eV/AMU
;      The actual values are from the routine PHYCONST in PFIDL_1D.
;
; EXAMPLE:
;      Print the velocity of a neutron with an energy of 2.45 Mev
;        print, VELOC(2.45e6, /n)
;
;      Make a waveform array in 22 with velocity vs energy for protons.
;        ener = 10^(findgen(121)/20)
;        v = VELOC(ENER, /P)
;        i2w, ener, v, 22, /xydata, clab = 'Velocity vs Energy', $
;           xlab = 'Energy (eV)', ylab = 'Velocity (m/s)'
;
; MODIFICATION HISTORY:
; 	Written by:	L. P. Mix, Jr., April 10, 2007
;-

Function VELOC, energy, mass, relativistic = relativistic, proton = proton, $
       deuteron = deuteron, electron = electron, neutron = neutron , $
       doubleprecision = doubl

; constants
on_error, 1
; common block
  common Velocity_common, solc, pmass, dmass, nmass, emass, wmass, mcc, const
  if n_elements(solc) eq 0 then begin
    phyconst, sol = solc, amass = wmass, aenergy = mcc, echarge = charge, $
      penergy = pmass, denergy = dmass, nenergy = nmass, eenergy = emass
    const = 2d0*charge/wmass
  endif
 nel = n_elements(energy)
 if nel eq 0 then begin
   print, '% VELOC:  Energy must be defined'
   return, 0.0
 endif
 vel = (nel gt 1) ? make_array(nel, value =0.0d0, /double) : 0.0d0
; rest mass
 if n_elements(mass) eq 0 then begin
   if keyword_set(proton) then mo = pmass else $
   if keyword_set(deuteron) then mo = dmass else $
   if keyword_set(neutron) then mo = nmass else $
   if keyword_set(electron) then mo = emass else mo = mcc
   mass = mo/mcc
 endif else mo = mass[0]*mcc
 
 if keyword_set(relativistic) then begin
   ok = findgen(nel)
   nok = nel
   ncomp = 0
 endif else ok = where (energy gt 1e-6*mo, nok, comp = comp, ncomp= ncomp)
 if nok gt 0 then vel[ok] = SQRT((mo+mo+energy[ok])*energy[ok])/(1.0+energy[ok]/mo)/mo*solc
 if ncomp gt 0 then vel[comp] = sqrt(const/mass[0]*energy[comp])
 if keyword_set(doubl) then return, vel
 return, float(vel)
 end
