; ------------------------------------------------------------------------ 
; $Id: w2i.pro,v 1.5 2010/12/08 17:56:55 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
 Pro  W2I, space, image, wdf, clabel = clab, file = file, dx = dx, $
      npoints = npoints, xlabel = xlab, ylabel = ylab, structure = structure
;
;+
; NAME:
;       W2I
;
; PURPOSE:
;       Convert a WDF array to IDL arrays: X and Y
;        or 
;       Convert a WDF array to an IDL Structure
;
; CATEGORY:
;       Utility
;
; CALLING SEQUENCE:
;       W2I, X, Y, N
;       or
;       W2I, ARRAY, N
;       or 
;       W2I, STR, N, /STRUCTURE
;
; INPUTS:
;       N:      WDF array number,
;               N must be between 1 and MaxWDFarray.
;
; OUTPUTS:
;       X:      IDL array variable with X-axis values
;       Y:      IDL array variable with Y-axis values
;       ARRAY:  IDL array of the form [X0, DX, Yarray]
;               Note:  If WDF array is not uniform, the data will be put
;                      on a uniform set of abscissas.
;                      No other keywords will be checked.
;       STR:    IDL structure variable
;               
;
; KEYWORD OUTPUT PARAMETERS:
;       STRUCTURE: If set, all other keywords are ignored.
;                  If the number of points is zero, then zeros will be
;                  returned for all structure labels.
;                  No error will be sent unless N is out of range.
;                  Structure labels are:
;                  type    - 6 type for WDF arrays
;                  ndim    - 1 dimension
;                  block   - 1 Number of blocks
;                  range   - 2 element array with the minimum and maximum
;                            abscissa values
;                  yrange  - 2 element array with the minimum and maximum
;                            ordinate values
;                  np      - number of points
;                            zero indicates no data in this array
;                  x0      - Initial point 
;                  dx      - Spacing for uniform data; -1 for x-y data
;                  x       - Abscissa values
;                  y       - Ordinate values
;                  xlab   - Abscissa label
;                  ylab   - Ordinate label
;                  file    - File from which the data originated
;                  clab    - Dataset comment label
;                  tlab    - Dataset type label
;
;       CLABEL:  Returns WDF dataset comment or title
;       XLABEL:  Returns WDF dataset X-axis label
;       YLABEL:  Returns WDF dataset Y-axis label
;       DX:      Returns 0 for nonuniform data
;                Returns the spacing for uniform data
;                If the data is stored as X-Y data, then
;                  DX = GET_DX(WDF) 
;                Value of zero indicates nonuniform data
;       NPOINTS: Returns the number of elements in the X and Y arrays
;       FILE:    Returns the name of the file for these dataset values
;
; RESTRICTIONS:
;       WDF array must be defined and have valid data unless the 
;       keyword, STRUCTURE, is set. 
;
; PROCEDURE:
;       X and Y values are returned for WDF dataset N
;       or a stucture is returned.
;
; EXAMPLE:
;       Convert WDF dataset 2 to arrays: RADIUS and TIME.  Save the dataset 
;       axis labels in TLAB and RLAB.
;               W2I, TIME, RADIUS, Xlab = TLAB, Ylab= RLAB, 2
;       Get all the information about waveform array 23.
;               w2i, str, 23 , /s
; 
; MODIFICATION HISTORY:
;      Original version, L. P. Mix, May 15, 1992
;      Added structure keyword, 12/15/94
;      Added array variable, 4/21/97
;
;-
;
;     add common block
@com.dat
      if n_elements(initwdf) eq 0 then wdfinit
      npar = n_params()
      if npar lt 2 then begin
        print,'% Format: W2I, X-array, Y-array, WDF_Array_number, '
print,'         xlabel = xlab, ylabel = ylab, clabel = clab, file = file'
print,'  or       W2I, ARRAY , WDF_Array_number'
print,'  or       W2I, Str, WDF_Array_number, /Structure'
        return
      endif
      if npar eq 2 then begin
        if n_elements(image) gt 0 then wdf = image else begin
          print,'% W2I: WDF array must be specified'
          return
        endelse
        if n_elements(wdf) gt 1 then begin
          Print, '% W2I: Multiple values of WDF given; first value will be used'
          wdf= wdf(0)
        endif
      endif else if n_elements(wdf) eq 0 then begin
        print,'% W2I: WDF array must be specified'
        return
      endif else wdf = wdf(0)
      if wdf lt 1 or wdf gt maxwdfarray then begin
        print, '% W2I: WDF array must be greater than 0 and </=', maxwdfarray
        return
      endif
      nel = wdfnp(wdf-1)
      npoints = nel
      if keyword_set(structure) then begin
        if nel lt 2 then begin
          space = {type:6L, ndim:1L, range:[0.0, 0.0], yrange:[0.0, 0.0], $
                 np:0L, block:1L, loc:[0, nel], x0:0.0, dx:0.0, x:0.0, $
                 y:0.0, xlab:'', ylab:'', file: '', clab:'', tlab:''}
        endif else begin
          get_wdfarray, array, wdf, /xy
          if n_elements(array) eq nel+nel then begin
            space = array[0:nel-1]
            image = array[nel:*]
            dx =  get_dxp(wdf)
            if dx eq 0 then dx = -1
          endif else begin
            space = findgen(nel)*array[1]+array[0]
            image = array[2:*]
            dx = array(1)
          endelse
          x0 = space(0)
          array = 0
          space = {type:6L, ndim:1L, range:[x0, wdfmaxx(wdf-1)], $
                 yrange:[wdfmin(wdf-1), wdfmax(wdf-1)], $
                 np:nel, block:1L, loc:[0, nel], x0:x0, dx:dx, $
                 x:temporary(space), $
                 y:temporary(image), xlab:wdfxlabel(wdf-1), $
                 ylab:wdfylabel(wdf-1), file: wdffile(wdf-1), $
                 clab:wdfcomment(wdf-1), tlab:'' }
        endelse
        image= wdf
        return
      endif
      if nel lt 2 then begin
        print, ' WDF array '+strtrim(wdf,2)+ ' does not have valid data'
        space = 0 
        image = 0
        return
      endif
      if npar eq 2 then begin
        get_wdfarray, space, wdf
        return
      endif 
      get_wdfarray, array, wdf, /xy
      if n_elements(array) eq nel+nel then begin
        space = array[0:nel-1]
        image = array[nel:*]
        dx = get_dxp(wdf)
      endif else begin
        space = findgen(nel)*array[1]+array[0]
        image = array[2:*]
        dx = array[1]
      endelse
      clab = wdfcomment[wdf-1]
      file = wdffile [wdf-1]
      xlab = wdfxlabel[wdf-1]
      ylab = wdfylabel[wdf-1]
      end
