; ------------------------------------------------------------------------ 
; $Id: wderr.pro,v 1.9 2010/07/21 19:38:17 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
;
Pro WDERR, WDF1, WDF2, WDF3, Width = width, NPOINTS = npoints, $
      DELTA = delta, RELATIVE = relative, SETDEFAULT = set, $
      XERROR = xerror, WDXerror = wdxerror, SHOWDEFAULT = show, $
      HEIGHT = heightin, MATCH = match, GRID = grid, _extra = extrastuff
;+
; NAME:
;       WDERR
;
; PURPOSE:
;	Plot error bars over a previously drawn plot.
;       Note: get_maxwdf() is used as a work array.
;
; CALLING SEQUENCE:
;       WDERR, WDdata, relative = rel_err
;       WDERR, WDdata, WDerror
;
;	WDERR, WDdata, WDLow, WDHigh
;
; INPUTS:
;       WDdata  WDF array number of the data curve
;       WDerror WDF array number of the error curve
;               Error bars extend from (WDdata-WDerror) to (WDdata+WDerror)
;       WDLow:  Negative error estimate for the data
;       WDHigh: Positive error estimate for the data
;               Error bars extend from (WDdata-WDLow) to (WDdata+WDHigh)
;
; KEYWORD Parameters:
;	WIDTH:  The width of the error bar ends.  The default is 1% of data plot width.
;               DEFAULT: WIDTH = 0.01
;       HEIGHT: The height of the horizontal error bar ends.
;               This is ignored if only Y-error bars are drawn. That is, HEIGHT is 
;               ignored unless XERROR or WDXError is present.
;               The height is a fraction of the height of the data plot height.  
;               If HEIGHT is less than or equal to 0 then HEIGHT = -HEIGHT * WIDTH
;               DEFAULT: HEIGHT = WIDTH
;       GRID:   If set the WIDTH and HEIGHT will be set as a fraction of the grid.
;               If not set, the WIDTH and HEIGHT are set based on the extent of the curve
;               in the grid.
;               Default:  GRID = 1
;       NPOINTS:Number of error bars. Default is 18 (see DELTA)
;       MATCH:  Match the points in WDdata
;       DELTA:  Spacing of the error bars.  If npoints is greater than zero,
;               then delta will be calculated from NPOINTS.  If NPOINTS is 
;               undefined or <= 0 then DELTA will be used as the spacing.
;               If !x.type ne 0 then DELTA = 0.0
;               If DELTA is larger than the range of the data then it is set to 0.
;               If DELTA is undefined or 0.0 then 18 error bars will be generated.
;               Note: DELTA is secondary to NPOINTS.  If NPOINTS is provided, 
;               then DELTA is ignored.
;       RELATIVE: If set, then WDerror, WDLow, and WDHigh are considered to be
;               the relative error if these parameters are defined.
;               Error bars extend from WDdata*(1.0-WDerror)) to 
;                                      (WDdata*(1.0+WDerror))
;          or
;               Error bars extend from (WDdata*(1.0-WDLow)) to 
;                                      (WDdata*(1.0+WDHigh))
;          or   If there is only 1 parameter then:
;               If RELATIVE is between 0 and 1, then a constant error 
;                 equal to RELATIVE will be used.
;                 Error bars extend from (WDdata*(1.0-RELATIVE)) to
;                                        (WDdata*(1.0+RELATIVE))
;       SETDEFAULT     If set, values of NPOINTS and WIDTH will be saved.
;                      NOTE: Save value of WIDTH = abs(WIDTH)
;                      NOTE: Save value of NPOINTS = abs(NPOINTS) > 1
;       SHOWDEFAULT    If set, show default values of NPOINTS and WIDTH
;       XERROR: Draw Horizontal error bars with from X-Xerror to X+Xerror
;       WDXerror: This is a WDF array number or a search string used to 
;                 determine XERROR as a function of X.  
;                 If two values are specified, the first is used for the negative
;                 XERROR and the second as the positive XERROR.
;                 Horizontal error bars are drawn using the current value of 
;                 in WDXERROR from X-Xerror to X+Xerror.
;           Note: This keyword is ignored if XERROR is not zero
;       _EXTRA: Any valid keyword in call to OPLOT
;
; OUTPUTS:
;	None.
;
; SIDE EFFECTS:
;	An overplot is produced.
;       Last WDF array is used as a work array.
;
; EXAMPLES:
;	To plot symmetrical error bars where for WDF array 3 using error 
;         estimates in WDF array 4.
;           plo, 3               ; plot the data
;           wderr, 3, 4        ; overplot the error bars
;
;       To plot error bars using Y-errors from WDF array 61 and X-errors
;         from WDF array 51 on waveform in WDF 9
;            plo, 9
;            wderr, 9, 61, WDXerror = 51
;
; MODIFICATION HISTORY:
;       Original version, January 29, 1998
;         Based on IDL routine ERRPLOT
;       Added new keywords, GRID and HEIGHT, LPM, July 21, 2010
;-
on_error,2                      ;Return to caller if an error occurs
;
@com.dat
;
 common plot_grid_last, yeng_scale, xeng_scale
;
 common wderror_common, npoints_save, width_save
;
      lcset = 0
      if n_elements(npoints_save) eq 0 then begin
        npoints_save = 18
        width_save = 0.01
      endif
      if keyword_set (set) then begin
        if n_elements(npoints) gt 0 then npoints_save = abs(npoints(0)) > 1
        if n_elements(width) gt 0 then width_save = abs(width(0))
        lcset = 1
      endif
      if keyword_set(show) then begin
        print, 'Default Error Bar Width = ' , strtrim(width_save, 2)
        print, 'Default Number of Error Bars = ' , strtrim(npoints_save, 2)
        lcset = 1
      endif
      nparams = n_params()
      if lcset gt 0 and nparams lt 2 then return

      if nparams eq 0 then begin
        print, '% Format: WDerr, WDdata, WDerror [, WIDTH = width]'
        print, '             [, HEIGHT = height] [, GRID = grid]'
        print, '             [, NPOINTS = npoints] [, DELTA = delta]'
        print, '             [, XERROR = xerror] [, WDXerror = wdxerror]' 
        print, '             [, /RELATIVE] [, _EXTRA = extrastuff]'
        print, '  <or>    WDerr, WDdata, WDLow, WDHigh [, WIDTH = width] '
        print, '             [, HEIGHT = height] [, GRID = grid]'
        print, '             [, NPOINTS = npoints] [, DELTA = delta]'
        print, '             [, XERROR = xerror] [, WDXerror = wdxerror]' 
        print, '             [, /RELATIVE] [, _EXTRA = extrastuff]'
        return
      endif
      if keyword_set(relative) then begin
        if relative lt 1.0 and relative gt 0.0 then lcrel = 1 else lcrel = 0
        nparams = nparams+lcrel
      endif else lcrel = 0
      if nparams lt 2 then begin
        print,'% WDERR: Original waveform and error must be specified '
        return
      endif else nparams = nparams-lcrel
      if n_elements(grid) eq 0 then grid = 1
;
      if n_elements(wdf1) eq 0 or  (n_elements(wdf2)+lcrel) eq 0 then begin
        print, '% WDERR: Waveform arrays must be defined'
        return
      endif
      if wdf1 lt 1 or wdf1 gt maxwdfarray then begin
        print, '% WDERR: WDF1 must be greater than 0 and </=', maxwdfarray
        return
      endif else if wdfnp(wdf1-1) eq 0 then begin
        print, '% WDERR: WDF1 has no data'
        return
      endif
        
      if lcrel eq 0 then begin
        if wdf2 lt 1 or wdf2 gt maxwdfarray then begin
          print, '% WDERR: WDF2 must be greater than 0 and </=', maxwdfarray
          return
        endif else if wdfnp(wdf2-1) eq 0 then begin
          print, '% WDERR: WDF2 has no data'
          return
        endif
      endif 

      if nparams gt 2 then begin
        if n_elements(wdf3) eq 0 then begin
          print, '% WDERR: Waveform array WDHigh must be defined'
          return
        endif
        if wdf3 lt 1 or wdf3 gt maxwdfarray then begin
          print, '% WDERR: WDF3 must be greater than 0 and </=', maxwdfarray
          return
        endif else if wdfnp(wdf3-1) eq 0 then begin
          print, '% WDERR: WDF3 has no data'
          return
        endif
      endif
      if n_elements(width) eq 0 then width = width_save
      if width lt 0.0 then width = width_save
      if n_elements(delta) eq 0 then delta = 0.0
      if !x.type ne 0 then delta = 0.0
      if n_elements(npoints) eq 0 then npoints = 0
      if keyword_set(match) then begin
        w2i, xvaltrue,yval, wdf1
        valid = n_elements(xvaltrue)
        npoints = valid
        xxmin = min(!x.crange, max = xxmax)
        if !x.type ne 0 then begin
          xxmax = 10.^xxmax
          xxmin = 10.^xxmin
        endif
        if not keyword_set(grid) then begin
          xxmin = max([xxmin, wdfminx(wdf1-1)*xeng_scale])
          xxmax = min([xxmax, wdfmaxx(wdf1-1)*xeng_scale])
        endif
        range = (!x.type eq 0) ? xxmax-xxmin : xxmax/xxmin
        if xxmax le xxmin then begin
          print, '% WDERR: Calculated extent of the error is zero.'
          return
        endif
      endif else if delta le 0.0 or npoints gt 0 then begin
        GET_SYMBOL_POINTS, WDF1, Xvaltrue, Yval, NPOINTS=npoints,valid=valid, $
          range = range, grid = grid
      endif else begin
;       delta gt 0 (!x.type = 0)
        xxmin = min(!x.crange, max = xxmax)
;       to calculate the number of points, we need the extent of the data
        xxmin = max([xxmin, wdfminx(wdf1-1)*xeng_scale])
        xxmax = min([xxmax, wdfmaxx(wdf1-1)*xeng_scale])
        if xxmax le xxmin then begin
           print, '% WDERR: Calculated extent of the error is zero.'
           return
        endif
        npoints = round ((xxmax-xxmin)/(delta*xeng_scale))
        GET_SYMBOL_POINTS, WDF1, Xvaltrue, Yval, NPOINTS=npoints, valid=valid, $
           range = range, grid = grid
      endelse
      
      if valid eq 0 then begin
        print, '% WDERR: No valid points.'
        return
      endif
      
      yval = yval *yeng_scale
      xval = xvaltrue *xeng_scale
      if (!x.type eq 0)  then range = range *xeng_scale
      lchoriz = 0
      if keyword_set (xerror) then begin
        lchoriz = 1
        xerr = abs(xerror)*xeng_scale
        left = xval -xerr
        right = xval +xerr
      endif else if keyword_set(wdxerror) then begin
        wdx = WDVALID(wdxerror, /valid)
        if wdx[0] gt 0 then begin
          nwdx =  n_elements (wdx)
          if nwdx gt 2 then begin
 print, '% WDERR: WARNING multiple arrays for X-error. First two will be used.'
            print, wdcomment (wdx)
          endif 
          xerr = get_value(wdx[0], xvaltrue, /match)*xeng_scale
          left = xval -xerr
          if nwdx gt 1 then xerr = get_value(wdx[1], xvaltrue, /match)*xeng_scale
          right = xval +xerr
          lchoriz = 1
        endif
      endif

      if nparams eq 2 then begin
        yerr = get_value(wdf2, xvaltrue, /match)*yeng_scale
        if keyword_set(relative) then begin
          up = yval*(1.0+yerr)
          down = yval*(1.0-yerr)
        endif else begin
          up = yval+yerr
          down = yval-yerr
        endelse 
      endif else if nparams gt 2 then begin
        yerr = get_value(wdf2, xvaltrue, /match)*yeng_scale
        yup = get_value(wdf3, xvaltrue, /match)*yeng_scale
        if keyword_set(relative) then begin
          up = yval*(1.0+yup)
          down = yval*(1.0-yerr)
        endif else begin
          up = yval+yup
          down = yval-yerr
        endelse 
      endif else begin
         up   = yval *(1+relative)
         down = yval *(1-relative)
      endelse
      if !y.type eq 1 then yr0 = (10^!y.crange) else yr0 = !y.crange
      del = 0.05*(yr0[1]-yr0[0])
      yr = yr0+[-del, del]
      if !x.type eq 1 then xr0 = (10^!x.crange) else xr0 = !x.crange
      del = 0.05*(xr0[1]-xr0[0])
      xr =xr0+[-del, del]
;
      if width eq 0.0 then begin
        down = down > yr0[0]
        up = up < yr0[1]
        for i= 0, npoints-1 do oplot,  $
          [xval(i), xval(i)], [down(i), up(i)], _extra = extrastuff 
      endif else begin
        down = down > yr[0]
        up = up < yr[1]
        if !x.type eq 0 then begin
          wid =  range*(width/2.) 
          x0 = xval -wid
          x1 = xval +wid
        endif else begin
          wid = range^(width/2.)
          x0 = xval /wid > xr[0]
          x1 = xval *wid < xr[1]
        endelse
        for i = 0, npoints-1 do oplot, _extra= extrastuff, $
          [x0(i), x1(i),xval(i), xval(i), x0(i), x1(i)], $
          [down(i), down(i), (down(i)> yr0[0]), (up[i]< yr0[1]), up(i), up(i)]
      endelse
      if lchoriz eq 0 then return
;
      if n_elements(heightin) eq 0 then height = width else height = heightin[0]
      if height le 0 then height = -height*width
      if height eq 0.0 then begin
        left = left > xr0[0]
        right = right < xr0[1]
        for i= 0, npoints-1 do oplot,  $
          [left(i), right(i)], [yval(i), yval(i)], _extra = extrastuff 
      endif else begin
        xxmin = min(!y.crange, max = xxmax)
        if !y.type ne 0 then begin
            xxmax = 10.^xxmax
            xxmin = 10.^xxmin
        endif
        if not keyword_set(grid) then begin
          xxmin = max([xxmin, wdfmin(wdf1-1)*yeng_scale])
          xxmax = min([xxmax, wdfmax(wdf1-1)*yeng_scale])
        endif
        range = (!y.type eq 0) ? xxmax-xxmin : xxmax/xxmin
        
        left = left > xr[0]
        right = right < xr[1]
        if !y.type eq 0 then begin
          wid =  range*(height/2.)
          x0 = yval -wid
          x1 = yval +wid
        endif else begin
          wid = range^(height/2.)
          x0 = yval /wid > yr[0]
          x1 = yval *wid < yr[1]
        endelse
        for i = 0, npoints-1 do oplot, _extra= extrastuff, $
          [left(i), left(i), left(i) > xr0[0], right(i) < xr0[1], right(i), right(i)], $
          [x0(i), x1(i),yval(i), yval(i), x0(i), x1(i)]
      endelse
      end
