; ------------------------------------------------------------------------ 
; $Id: wriasc.pro,v 1.8 2009/06/17 15:52:45 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
 Pro WRIASC, fname, WDFin,narray, formatout=format, delimiter = delimiter, $
     vida = vida, keep_x = keep_x, append = append, error_keep = keep_error, $
     xout=xout, yout=yout, internal=internal, xydata = xydata, $
     missing = missing, npoint = npoint
;
;+
;    Write a number of WDF arrays to an ascii file or idl arrays
;    File format is determined by the user.
;    The arrays are limited to the region of overlap and put on an common 
;       time base using the minimum point spacing of the arrays.
;    format is:  WRIASC, fname, WDF [, narray] [, formatout=format]
;                    [, XOUT = xout] [, YOUT = yout] [, INTERNAL = internal]
;                    [, NPOINT = npoint] [/VIDA] [, /KEEP_X]
;                    [, DELIMITER = delimiter] [, XYDATA = xydata]
;                    [, APPEND = append] [, ERROR_KEEP = error_keep]
;                    [, MISSING = missing]
;    where
;      fname   - the file name 
;                If fname = 0 then fname ='PFIDLdefault.dat'
;      WDF     - First WDF array or an array of WDF arrays
;                WDF may be a single search string, or 
;                WDF be a single value or an array of WDF values
;      narray  - the number of arrays to be written to the file
;                if present and greater than 0 then WDF(0) and then next
;                narray-1 datasets will be written to a file
;      formatout - output format of the data
;              0  only data values in the file
;              1  number of points followed by data values
;              2  Narray labels followed by data values
;              3  (DEFAULT) number of points followed by narray labels 
;                 then data values
;              4  EXCEL spreadsheet  
;                 NARRAY+1 labels are written on the first line.
;                 x value followed by narray Y values are on the following lines
;              5  VIDA - Only 1 waveform may be output
;      xout, yout - X and Y values written to the file 
;      xydata   - indicates that the data is not to be writen as a wdf array
;                 Note: Only one array can be written. NPOINT is ignored.
;                   DEFAULT = 1     If only one array is written
;                             0     Otherwise
;       NOTE:  Use KEEP_X to write multiple arrays with the X values of the
;              first dataset
;      VIDA     - If set, Formatout is set to 5
;      internal - if present and not zero, the x and y values will be put into
;                 arrays xout and yout only and no file will be written.
;      npoint   - maximum number of points to be output per waveform
;                 Npoint must be greater than 3 
;                 If n_elements(wdf) = 1 and narray is missing or 1 then
;                   WDF will not be put on a uniform time base.
;                 Default = 4096
;      delimiter- character string delimiter between output values 
;                 Default= ' '
;      keep_x   - If set, WRIASC will keep only x values which are the same 
;                 as the first dataset.  Data will be written as XY data.
;                 If data does not match, a warning will be issued but output 
;                 will continue.  If points vary by less than ERROR_KEEP of the
;                 spacing, they will be considered equal.
;                 Matching details if keep_X is positive:
;                 1-If the number of points are the same, the difference in the 
;                   X-arrays is compared to the error_keep value.  If all of the 
;                   values are the considered equal, the processing continues.
;                 2-If the number of points are different or if some of the X arrays
;                   do not match, then the following steps are performed:
;                     The initial X array is sorted
;                     The current X array is limited to the domain of the initial array
;                     The current X array is matched to the first and all values within 
;                       error_keep are stored
;                     In the case of multiple values Y-values at the same X-value, the 
;                       last value is the one written to the file.
;                 Matching details if keep_X is negative:
;                 1-If the number of points are the same, the difference in the
;                   X-arrays is compared to the error_keep value.  If all of the
;                   values are the considered equal, the processing continues.
;                 2-If the number of points are different or if some of the X arrays
;                   do not match, then the ordinates of the second array will be 
;                   linearly interpolated to the values in the first array.  Missing
;                   values will be set to "MISSING".
;      MISSING  - If keep_x is set missing values will be set to this value.
;                 Default:  Missing = -99999.00
;      error_keep - Acceptable difference in X values if KEEP_X is set.
;                 Actual error for each point is (dx*error_keep) where
;                   dx is the absolute difference between successive x values.
;                 Default:  ERROR_KEEP = 0.08
;      append   - If set, the file will be opened to append
;    Example:
;    Write 5 arrays into file 'PFIDLdefault.dat' from  WDF arrays 
;    2,3,4,5,6.  Assume data format 3.
;       WRIASC, 0, 2, 5 
; <or>  WRIASC, 0, [2,3,4,5,6]
;    Write 3 arrays to file, 'testdata' from  WDF arrays 3, 6, 9. Assume 
;    data format to have no header but only 4 columns of data.
;       WRIASC , 'testdata', [3,6,9], form = 0
;
;    Put arrays 3, 4, 5 on a common time base and put the amplitudes into AMP
;    and the times into TIME
;       WRIASC, 0, [3,4,5], x= time, y = amp, /internal
;-
;
     on_error, 1
@com.dat
     xout = 0
     yout = 0
     if n_elements(missing) eq 0 then missing = -99999.0
     if n_elements(keep_error) eq 0 then keep_error = 0.08 else $
        keep_error = abs(keep_error[0]) 

      if n_elements(initwdf) eq 0 then wdfinit
;    check format
     nparams = n_params()
     if nparams eq 0 then begin
        print, '% Format:  WRIASC, fname, WDF [, narray] [, formatout=format]'
        print, '               [, XOUT = xout] [, YOUT = yout] [, /VIDA] [, /KEEP_X]'
        print, '               [, INTERNAL = internal] [, NPOINT = npoint]'
        print, '               [, DELIMITER = delimiter] [, XYDATA = xydata]'
        return
     endif

      if keyword_set(vida) then format = 5
      if n_elements (format) eq 0 then format = 3
      if format lt 0 or format gt 5 then format = 3
      if format eq 5 then begin
        vida = 1
        format = 4
      endif 
      b = size(fname)
      if b(b(0)+1) ne 7 then begin
        if fname eq 0 then fname = 'PFIDLdefault.dat' else begin
          print, 'FNAME, the input file name, must be a string or zero'
          return
        endelse
      endif
;     make sure we don't corrupt input data
      if n_elements(wdfin) gt 0 then  wdf = wdfin else begin
        print, '% WRIASC: WDF array(s) must be specified'
        return
      endelse
      a = size(wdf)
      if a(a(0)+1) eq 7 then begin
        if a(0) ne 0 then print, '% WRIASC: Only the first string is used'
        if strtrim(wdf(0), 2) eq '*' then wdf = wdvalid() else $
          wdf = wdvalid(wdf(0))
        if wdf(0) lt 1 then begin
          print, '% WRIASC: No arrays found using the search string: ', wdf(0)
          return
        endif
      endif
      nel = n_elements(wdf)
      if nel eq 0 then begin
        print,'% WRIASC: WDF array value must be provided'
        return
      endif else if n_elements (narray) gt 0 then begin
        if narray gt 0 then wdf = wdf(0)+ indgen(narray)
      endif
      if keyword_set (xydata) then wdf = wdf(0)
      wdfar = wdf
      if min(wdfar) lt 1 or max(wdfar) gt maxwdfarray then begin
        print, 'wdf array must be greater than 0 and </=', maxwdfarray
        return
      endif
      nel = n_elements(wdfar)
      ok = where (wdfnp(wdfar-1) gt 0, nok)
      if nok eq 0 then begin 
        print, 'No valid data in specified arrays'
        return
      endif
      wdfar = wdfar(ok)
      if nok eq 1 then xydata = 1
      if keyword_set(vida) then begin
        xydata = 1
        nok = 1
        wdfar = wdfar[0]
      endif
      minx =max( wdfminx(wdfar-1))
      maxx =min( wdfmaxx(wdfar-1))
      if minx ge maxx then begin
        print, 'No region of overlap for these datasets'
        return
      endif
      if n_elements(npoint) eq 0 then npoint = 4096 else npoint=abs(npoint) >3
      if keyword_set(xydata) then begin
        w2i, xout, yout, wdfar(0)
        if keyword_set (vida) then begin
          a = sort(xout)
          xout = xout[a]
          yout = yout[a]
        endif
        maxnp = n_elements(xout)
      endif else begin
;       check to see if we are going to keep the X values of the first dataset.
        delta = maxx -minx
        if keyword_set(keep_x) then begin
          xout = getxf(wdfar[0])
          np = n_elements(xout)
          dd = abs([xout[1:np-1]-xout[0:np-2], 1])*keep_error
          dd[np-1] = dd[np-2]
          eps = min(dd)
          yout = make_array(np, nok)
          yout[*, 0]=getyf(wdfar[0])
          for i = 1, nok-1 do begin 
            n = wdfnp(wdfar[i]-1)
            w2i, xt, yt, wdfar[i]

            lcmatch = 1
            if n eq np then begin
              ok = where (abs(xout-xt) gt dd, nook)
              if nook eq 0 then yout[0, i]=yt else lcmatch = 0
            endif else lcmatch =0
            if lcmatch eq 0 then begin
              yy = make_array(np, value = missing)
;             Not a 1 to 1 match
              if keep_x gt 0 then begin
                print, '% WRIASC: Some of the values in waveform ', $
                strtrim (wdfar[i], 2), ' do not match exactly'
                if n_elements(xout_sort) lt np then begin
                  xsort = sort(xout)
                  if max(xsort-lindgen(np)) eq 0 then begin
                    lcsort=0
                    xrang= [xout[0]-dd[0], xout[np-1]+dd[np-1]]
                  endif else begin
                    unsort = sort(xsort)
                    xout_sort = xout[xsort]
                    xrang= [xout_sort[0]-dd[0], xout_sort[np-1]+dd[np-1]]
                    lcsort=1
                  endelse
                endif
                ok = where (xt gt xrang[0] and xt lt xrang[1] , nook)
                if nook gt 0 then begin
                  xt = xt[ok]
                  yt = yt[ok]
                  if lcsort eq 0 then realind = findex(xout, xt) $
                    else realind = findex(xout_sort, xt)
                  intind = round(realind)
                  ook = where (abs(realind-intind) le keep_error, nook)
                  if nook gt 0 then yy[intind[ook]] = yt[ook]
                  if lcsort ne 0 then yy= yy[unsort]
                endif
              endif else begin
                a = sort (xt)
                xt = xt[a]
                yt = yt[a]
                ok = where(xout gt (xt[0]-eps) and xout lt (xt[n-1]+eps), nook)
                if nook gt 0 then $
                  yy[ok]= interpolate(yt, findex(xt, xout[ok]))
              endelse
              yout[0, i]= yy
            endif
          endfor  
          maxnp = np
        endif else begin
;         first determine the minimum point spacing
          deltamin = float(delta)/(npoint-1)
          i = 0
          while delta gt deltamin  and i lt nok do begin
            get_wdfarray, array, wdfar(i), /xy
            nel = wdfnp(wdfar(i)-1)
            if n_elements(array) eq nel + nel then $
              delta = min([abs(array(1:nel-1)-array(0:nel-2)), delta]) $
            else delta = min([delta, array(1)])
            i = i+1
          endwhile
          if delta lt deltamin then delta = deltamin
          maxnp = long((maxx - minx)/delta+1.0005)
          xout = minx+findgen(maxnp)*delta
          yout = make_array(maxnp, nok)
          for i = 0, nok-1 do begin
            get_wdfarray, array, wdfar(i), /xy
            nel = wdfnp(wdfar(i)-1)
            if n_elements(array) eq nel + nel then begin
              put2uniform, array(0:nel-1), array(nel:nel+nel-1), nel, $
                  minx, delta, maxx, ar, /yonly
            endif else begin
              put2uniform, findgen(nel)*array(1)+array(0), array(2:*), nel, $
                  minx, delta, maxx, ar, /yonly
            endelse
            yout(*, i) = ar
          endfor
        endelse
      endelse
      if keyword_set(internal) then return
      
      if n_elements (delimiter) eq 0 then delimiter = ' ' else $
        delimiter = string(delimiter(0))
      ndelim = strlen(delimiter)
      if ndelim eq 0 then begin
        ndelim =1
        delimiter = ' '
      endif
      recl = nok*(14+ndelim)+18
      openw,unit,fname, /get_lun, width = recl, append = append
; Time to start writing
      if format eq 1 or format eq 3 then printf, unit, maxnp
      if format eq 4 then begin
        fmstr = '(a, '+strtrim(nok, 2)+'("'+delimiter+'", a))'
        tmp = make_array(nok+1, /string)
        if keyword_set(vida) then $
          tmp[1] = strcompress(wdfylabel[wdfar[0]-1]) $
          else for i = 0, nok-1 do $
          tmp[i+1]= strcompress(wdfcomment[wdfar[i]-1])
        sp = (byte(' '))[0]
        und = (byte('_'))[0]
        for i = 1, nok do begin
          bt = byte(tmp[i])
          uk = where (bt eq sp, nuk)
          if nuk gt 0 then bt[uk] = und
          tmp[i] = strtrim(bt, 2)
          if strlen(tmp[i]) eq 0 then tmp[i]= 'DataSet_'+strtrim(i, 2)
        endfor
        tmp[0] = '  '+strcompress(wdfxlabel[wdfar[0]-1], /remove_all)
        if strlen (strtrim(tmp[0], 2)) eq 0 then tmp[0] = ' Time' 
        printf, unit, format = fmstr, tmp
        tmp = 0
      endif else if format  gt 1 then begin
        for i=0, nok -1 do printf, unit, strtrim(wdfcomment(wdfar(i)-1), 2)
      endif
      fmstr = '(g14.7, '+strtrim(nok, 2)+'("'+delimiter+'", g14.7))'
      for i = 0L, maxnp-1 do $
        printf, unit, format = fmstr, xout(i), reform(yout(i, *))
      if keyword_set(vida) then printf, unit, format = fmstr, -999 , -999
      free_lun, unit
      return
      end
