; ------------------------------------------------------------------------ 
; $Id: writehdf.pro,v 1.4 2007/06/06 20:03:56 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
;+
; NAME:
;       WRITEHDF
;
; PURPOSE:
;       Write XDAMP formatted HDF files.
;       This routine is based on the copyrighted xdamp procedure, XI_savefile,
;         written by William P. Ballard as part of the Xdamp package.
;         Ref. SAND98-0907, Version 3.123
;
; CALLING SEQUENCE:
;       Format:
;         WRITEHDF [, FILENAME] [, WDF = WDF] [, STRUCTURE = structure]
;           [, TITLE = title] [, HEADER = header] [, NOTES = notes]
;           [, NWDF = nwdf] [, NSTR = nstr] [, HELP = help] [, IERROR = ierror]
;           [, OUTFILE = outfile] [, AUDIT = audit] [, /batch]
;
; OPTIONAL INPUTS:
;       FILENAME - Name of the HDF file to be opened.
;                  The default file extension is ".hdf"  
;                  If the specified file does not have an extension, this 
;                  extension will be added to the filename.
;                  If file is omitted or undefined or is not a string,
;                  the current directory will be the name of the file.
;                  A unique name will be selected.
;	
; KEYWORD PARAMETERS:
;       WDF     - An array of WDF array numbers or a search string for the 
;                 waveforms to be saved to the HDF file.
;                 WDF = 0 indicates no waveforms to be written
;                 Default = '*' 
;       STRUCTURE - An array of structure array numbers or a search string for
;                 the images to be saved to the HDF file.
;                 Structure = 0 indicates no images to be written
;                 Default = '*'
;       TITLE   - Shot title
;       DATE    - Shot date
;       HEADER  - Shot header information; may be a string array.
;       NOTES   - Shot notes if they are available; may be a string array.
;       NWDF    - Number of waveforms written
;       NSTR    - Number of images written
;       HELP    - If set, a help message is printed.
;       IERROR  - Returns zero if no error.
;       OUTFILE - Output file name
;       AUDIT   - String array to be written with each waveform
;                 One element of the array is written with each waveform.
;                 If the number of elements in AUDIT is less than the number of 
;                 waveforms, then the audit will be be value be defined by the 
;                 following:
;                   audit[i]= audit[ i mod n_elements(audit)]
;                 If AUDIT is undefined, the file name for the waveform will be 
;                 written.
;                 In the case of structures, the file name will always be written.
;       BATCH   - If set, no widgets will be used
;
;
; OUTPUTS:
;       Data will be written to a HDF file
;
; PROCEDURE:
;       This is a modified version of the XDAMP procedure XI_savefile.
;
; EXAMPLE:
;       Write all data to the file example.hdf.
;         WRITEHDF, 'example.hdf'
;
;       Write all waveforms beginning with 'ip' to a file of the same name as
;       the current directory.
;         WRITEHDF, '^ip', str =0
;
; MODIFICATION HISTORY:
; 	Written by:     L. P. Mix, May 14, 2002
;                  based on routine by W. P. Ballard.
;-

PRO WRITEHDF , P1, WDF = WDF, STRUCTURE = struct, TITLE = title, date = date, $
      HEADER = fheader, NOTES = fnotes, NWDF = nwdf, NSTR = nstr, help = help, $
      IERROR = ierror, outfile = outfile, audit = auditin, batch = batch


on_error, 1
      ierror = -1
;
      nparams = n_params()
      if nparams eq 0 and keyword_set(help) then begin
print, '% Format: WRITEHDF [, FILENAME] [, WDF = WDF] [, STRUCTURE = structure]'
print, '              [, TITLE = title] [, HEADER = header] [, NOTES = notes]'
print, '              [, NWDF = nwdf] [, NSTR = nstr] [,/HELP] [, DATE = date]'
print, '              [, IERROR = ierror] [, AUDIT = audit] [, /BATCH]'
        return
      endif
      if (nparams eq 0) then begin
        filename = ''
        nparams = 1
      endif else if n_elements(p1) eq 0 then begin
        filename = ''
      endif else if (size(p1, /type) ne 7) then begin
        filename = ''
      endif else  filename = strtrim(p1, 2)
      cd, current = current
      
      if strlen(filename) lt 1 then begin
        ok = strpos(current, path_sep(), /reverse_search)
        if ok ge 0 then filename = strmid(current, ok+1) else filename =''
        if strlen(filename) eq 0 then begin
          if not keyword_set(batch) then begin
            get_tqstring, lcquit, 'Enter name w/o extension', name, $
              title ='HDF file name'
            filename = strtrim(name[0], 2)
            if lcquit then return
          endif
          if strlen(filename) eq 0 then begin
            print, '% WRITEHDF: No output file'
            return
          endif
        endif 
      endif
      ok = strpos(filename, path_sep(), /reverse_search)
      okp = strpos(filename, '.', /reverse_search)
      if okp le ok then filename = filename+'.hdf'
      outfile = find_newfile(filename, /quiet)
      lcwdf = 1
      if n_elements (wdf) eq 0 then wdf = '*' else begin
        if n_elements(wdf) eq 1 and size(wdf, /type) lt 6 then begin
          if wdf eq 0 then lcwdf = 0
        endif
      endelse
      if lcwdf then wdfout = wdvalid(wdf, np= nwdf, /valid, /quiet) else $
        nwdf = 0L
      lcstr = 1
      if n_elements (struct) eq 0 then struct = '*' else begin
        if n_elements(struct) eq 1 and size(struct, /type) lt 6 then begin
          if struct eq 0 then lcstr = 0
        endif
      endelse
      if lcstr then strout = strvalid(struct, np= nstr, /valid, /quiet) else $
        nstr = 0L
 
      if nstr+nwdf eq 0 then begin
        print, '% WRITEHDF: No output for the HDF file.'
        return
      endif

      if n_elements(title) eq 0 then title = outfile
      if n_elements(fheader) eq 0 then fheader = ''
      if n_elements(fnotes) eq 0 then fnotes = ''
      if n_elements(date) eq 0 then date = systime()
      if not keyword_set(batch) then begin
        get_tqstring, lcquit, 'Enter title ', title, $
              title ='HDF file TITLE'
        if lcquit then return
        title = strtrim(title[0], 2)

        get_tqstring, lcquit, 'Enter date ', date, $
              title ='HDF file date'
        if lcquit then return
        date = strtrim(date[0], 2)

        get_tqstring, lcquit, 'Enter header ', fheader, $
              title ='HDF file HEADER'
        if lcquit then return
        fheader = strtrim(fheader[0], 2)

        get_tqstring, lcquit, 'Enter file notes', fnotes, $
              title ='HDF file NOTES'
        if lcquit then return
        fnotes = strtrim(fnotes[0], 2)
      endif

;*** open a new HDF file, get the file handle
      hdf_handle = HDF_OPEN(outfile, /WRITE, /CREATE)
      if hdf_handle lt 0 then begin
        print, '% WRITEHDF:  Unable to open file: ', Outfile
        return
      endif

;
;  Internal routine that actually writes a file in HDF format
;------------------------------------------------------------------------------
  

;*** declare variables
XI_fields = ["NAME", "ANOTE", "AUDIT", "VUNITS", "HUNITS", $
             "TMIN",  "TMAX",  "DATA"]
XI_ftype =  ["BYTE", "BYTE",  "BYTE",  "BYTE",   "BYTE", $
             "FLOAT", "FLOAT", "FLOAT"]
nfields = N_ELEMENTS(XI_fields)

XI_ifields = ["NAME", "ANOTE", "AUDIT", "XUNITS", "YUNITS", "ZUNITS", $
             "XPTS", "YPTS", "XMIN",  "XMAX",  "YMIN", "YMAX", "MATRIX"]
XI_iftype =  ["BYTE", "BYTE",  "BYTE",  "BYTE",   "BYTE",   "BYTE", $
             "INT", "INT", "FLOAT", "FLOAT", "FLOAT", "FLOAT", "MATRIX"]
nifields = N_ELEMENTS(XI_ifields)

;*** create a vgroup id, attach it and set the name
vg_header = HDF_VG_ATTACH(hdf_handle, -1, /WRITE)
HDF_VG_SETINFO, vg_header, NAME="XDAMP HEADER"

;*** do another for the data
vg_data = HDF_VG_ATTACH(hdf_handle, -1, /WRITE)
HDF_VG_SETINFO, vg_data, NAME="XDAMP DATA"

;*** first write shot date and shot title
vds = HDF_VD_ATTACH(hdf_handle, -1, /WRITE)
HDF_VD_FDEFINE, vds, "SHOT DATE", /BYTE
if strlen (date) eq 0 then date = thin_date()
HDF_VD_WRITE, vds, "SHOT DATE", STRTRIM(date, 2)
HDF_VD_INSERT, vg_header, vds
HDF_VD_DETACH, vds
vds = HDF_VD_ATTACH(hdf_handle, -1, /WRITE)
HDF_VD_FDEFINE, vds, "SHOT TITLE", /BYTE
if strlen(title) eq 0 then title = outfile
HDF_VD_WRITE, vds, "SHOT TITLE", STRTRIM(title, 2)
HDF_VD_INSERT, vg_header, vds
HDF_VD_DETACH, vds

;*** now write in header stuff
FOR i = 0, N_ELEMENTS(fheader)-1 DO BEGIN
   temp = STRTRIM(fheader(i))
   IF (STRLEN(temp) GT 0) THEN BEGIN
      vds = HDF_VD_ATTACH(hdf_handle, -1, /WRITE)
      HDF_VD_FDEFINE, vds, "HEADER INFO", /BYTE
      HDF_VD_WRITE, vds, "HEADER INFO", temp
      HDF_VD_INSERT, vg_header, vds
      HDF_VD_DETACH, vds
   ENDIF
ENDFOR

;*** now add in the file notes to the header
FOR j = 0, N_ELEMENTS(fnotes)-1 DO BEGIN
   temp = STRTRIM(fnotes(j))
   IF (STRLEN(temp) GT 0) THEN BEGIN
      vds = HDF_VD_ATTACH(hdf_handle, -1, /WRITE)
      HDF_VD_FDEFINE, vds, "FILE NOTES", /BYTE
      HDF_VD_WRITE, vds, "FILE NOTES", temp
      HDF_VD_INSERT, vg_header, vds
      HDF_VD_DETACH, vds
   ENDIF
ENDFOR

   audit_def = "No Audit Trail"
   anote_def = "No Annotation"
   if n_elements(auditin) eq 0 then naudit= 0 else naudit=n_elements(auditin)

;*** now add waveforms if any in memory
IF (nwdf GT 0) THEN BEGIN
   vg_wvfm = HDF_VG_ATTACH(hdf_handle, -1, /WRITE)
   HDF_VG_SETINFO, vg_wvfm, NAME="WAVEFORM"
   limits, wdfout, xrange= xrange
;*** first get a set of vdata ids to put a waveform in
   FOR j = 0, nwdf-1 DO BEGIN
      wd = wdfout[j]
      name = wdcomment (wd)
      st = strpos(name, ' - ')
      if st gt 0 then begin
        anote = strmid(name, st+3)
        name = strmid(name, 0, st)
      endif else anote = anote_def

      get_wdfarray, array, wd, file = file, xlabel = xlab, ylabel = ylab
      if naudit eq 0 then audit = (strlen(file) gt 0)? file : audit_def $
        else audit = auditin[j mod naudit]
;     if strlen(file) gt 0 then audit = file else audit = audit_def
      if strlen (xlab) eq 0 then xlab = 'Time'
      if strlen (ylab) eq 0 then ylab = 'Amplitude'

      FOR i = 0, nfields-1 DO BEGIN
         vds = HDF_VD_ATTACH(hdf_handle, -1, /WRITE)
         CASE XI_ftype(i) OF
            "BYTE":    HDF_VD_FDEFINE, vds, XI_fields(i), /BYTE
            "LONG":    HDF_VD_FDEFINE, vds, XI_fields(i), /LONG
            "FLOAT":   HDF_VD_FDEFINE, vds, XI_fields(i), /FLOAT
         ENDCASE
         CASE XI_fields(i) OF
            "NAME":    HDF_VD_WRITE, vds, "NAME", name
            "ANOTE":   HDF_VD_WRITE, vds, "ANOTE", anote
            "AUDIT":   HDF_VD_WRITE, vds, "AUDIT", audit
            "HUNITS":  HDF_VD_WRITE, vds, "HUNITS", xlab
            "VUNITS":  HDF_VD_WRITE, vds, "VUNITS", ylab
            "TMIN":    HDF_VD_WRITE, vds, "TMIN", xrange[0, j]
            "TMAX":    HDF_VD_WRITE, vds, "TMAX", xrange[1, j]
            "DATA":    HDF_VD_WRITE, vds, "DATA", array[2:*]
            ELSE:
         ENDCASE
         HDF_VG_INSERT, vg_wvfm, vds
         HDF_VD_DETACH, vds
      ENDFOR
   ENDFOR
ENDIF

;*** now build an image description
IF (nstr GT 0) THEN BEGIN
   vg_imag = HDF_VG_ATTACH(hdf_handle, -1, /WRITE)
   HDF_VG_SETINFO, vg_imag, NAME="IMAGE"

   FOR j = 0, nstr-1 DO BEGIN
      wd = strout[j]
      s2i, s, wd
      if (s.two ne 3) and s.ndim gt 2 then begin
        print, '% WRITEHDF: Unable to write structure: ', strtrim(st, 2)
        print, '  Data must be 2 dimensional'
        print, '  Dimensions: ', strtrim(s.ndim, 2)
        print, '  Null Dimension: ', strtrim(s.two, 2)
      endif else if s.dx1 eq 0 or s.dx2 eq 0 then begin
        print, '% WRITEHDF: Unable to write structure: ', strtrim(st, 2)
        Print, '  Data must be uniform'
        print, '  DX - 1: ', strtrim(s.dx1, 2)
        print, '  DX - 2: ', strtrim(s.dx2, 2)
      endif else if (s.type ne 1) or (s.block gt 1) then begin
        print, '% WRITEHDF: Unable to write structure: ', strtrim(st, 2)
        Print, '  Data must be single block, image(type=1) data'
        print, '  Blocks = ', strtrim(s.block, 2)
        print, '  Type = ', strtrim(s.type, 2)
      endif else begin
        name = wdcomment (wd)
        st = strpos(name, ' - ')
        if st gt 0 then begin
          anote = strmid(name, st+3)
          name = strmid(name, 0, st)
        endif else anote = anote_def
        
        if strlen(s.file) gt 0 then audit = s.file else audit = audit_def
        xlab = s.x1lab[0]
        ylab = s.x2lab[0]
        if strlen (xlab) eq 0 then xlab = 'X'
        if strlen (ylab) eq 0 then ylab = 'Y'
        mat = reform(get_array(wd, -1, label = zlab), /overwrite)
        if strlen (zlab) eq 0 then zlab = 'Amplitude'
        ssiz = size(mat)

        if ssiz[0] ne 2 then begin
          PRINT, '% WRITEHDF: Unable to write structure: ", strtrim(st, 2)
          Print, '  Data must be two dimensional
          help, mat
        endif else begin

        FOR i = 0, nifields-1 DO BEGIN
         vds = HDF_VD_ATTACH(hdf_handle, -1, /WRITE)
         CASE XI_iftype(i) OF
            "BYTE":    HDF_VD_FDEFINE, vds, XI_ifields(i), /BYTE
            "INT":     HDF_VD_FDEFINE, vds, XI_ifields(i), /INT
            "LONG":    HDF_VD_FDEFINE, vds, XI_ifields(i), /LONG
            "FLOAT":   HDF_VD_FDEFINE, vds, XI_ifields(i), /FLOAT
            "MATRIX":  HDF_VD_FDEFINE, vds, XI_ifields(i), /FLOAT, $
                       ORDER=ssiz[1]
         ENDCASE
         CASE XI_ifields(i) OF
            "NAME":    HDF_VD_WRITE, vds, "NAME", name
            "ANOTE":   HDF_VD_WRITE, vds, "ANOTE", anote
            "AUDIT":   HDF_VD_WRITE, vds, "AUDIT", audit
            "XUNITS":  HDF_VD_WRITE, vds, "XUNITS", xlab
            "YUNITS":  HDF_VD_WRITE, vds, "YUNITS", ylab
            "ZUNITS":  HDF_VD_WRITE, vds, "ZUNITS", zlab
            "XPTS":    HDF_VD_WRITE, vds, "XPTS", ssiz[1]
            "YPTS":    HDF_VD_WRITE, vds, "YPTS", ssiz[2]
            "XMIN":    HDF_VD_WRITE, vds, "XMIN", s.range[0,0,0]
            "XMAX":    HDF_VD_WRITE, vds, "XMAX", s.range[1,0,0]
            "YMIN":    HDF_VD_WRITE, vds, "YMIN", s.range[0,1,0]
            "YMAX":    HDF_VD_WRITE, vds, "YMAX", s.range[1,1,0]
            "MATRIX":  HDF_VD_WRITE, vds, "MATRIX", mat
                       
            ELSE:
         ENDCASE
         HDF_VG_INSERT, vg_imag, vds
         HDF_VD_DETACH, vds
        ENDFOR
        endelse
      endelse 
   ENDFOR
ENDIF

;*** finished writing stuff, close file
IF (nwdf GT 0) THEN BEGIN
   HDF_VG_INSERT, vg_data, vg_wvfm
   HDF_VG_DETACH, vg_wvfm
ENDIF
IF (nstr GT 0) THEN BEGIN
   HDF_VG_INSERT, vg_data, vg_imag
   HDF_VG_DETACH, vg_imag
ENDIF
HDF_VG_DETACH, vg_data
HDF_VG_DETACH, vg_header
print, '% WRITEHDF: Closing file: ', outfile
HDF_CLOSE, hdf_handle
END
