; ------------------------------------------------------------------------ 
; $Id: wriufo.pro,v 1.3 2010/09/20 14:29:06 lpmix Exp $
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
;
;+
; NAME:	     WRIUFO
; PURPOSE:		
;       IDL routine to write a WDF array to a simple x,y file in UFO format #111.
;       Format is assumed to consist of a 5 line header, followed by
;       N lines of x,y data pairs, concluded with a -999,-999 data pair.
;
; CALLING SEQUENCE: WRIUFO, WDF [, FILENAME] [, FILENAME= fname] [, FORMAT = format] [, /DIALOG]
; INPUTS:
;        WDF      - WDF array number or unique dataset comment to be written to a UFO file
;        FILENAME - Output file name
;                   If missing and fname is missing, then user may enter 
;                   a value for the output file naame.
; KEYWORDS:
;       NOTE: FILENAME and FORMAT may be shortened to FI and FO
;
;       FILENAME  - Ignored if the parameter FILENAME is present
;                   If FILENAME is undefined, then FNAME is used.
;                   If neither the parameter not keyword have a filename, 
;                   then the user is prompted for one.
;       FORMAT    - If specified, this will be used as the format string for the print
;                   Note: No checking is performed.
;                   Default: FORMAT = '(2g16.7)'
;       DIALOG    - If set, use DIALOG_PICKFILE to pick a file name.
;                   Default:  Enter name at the command prompt
; EXAMPLE:
;       Write dataset 4 to UFO file, data_4.ufo
;         WRIUFO, 4, 'data_4.ufo'
;       Write dataset with header '1256*wave' to file, zdata1256_09_09_09_wave.ufo
;         WRIUFO, '1256*wave', 'zdata1256_09_09_09_wave.ufo'
;
; PROCEDURE:
;       The UFO format consists of 5 header lines:
;          '111 = FORMAT #'
;          '2 = # TABLE COLUMNS'
;          '1  1 = ITYPE, IORDER'
;          '0.00E+00  0.00E+00 = ABSTOL, RELTOL'
;          '             X             Y'
;       The header is followed by two columns of X, Y data ending with the last pair:
;         [-999, -999]
;
; MODIFICATION HISTORY:
;       Adapted from W E Nelson's wwrite routine and more recently WUFO.
;       Included in PFIDL, L. P. Mix, 9/10/2009
;       Added DIALOG keyword      6/8/2010 lpm
;
;-

 PRO wriufo, WDFin, filename, filename=fname, format = fmt_str, dialog = dialog

on_error, 1			;return to caller on error
;
NPAR = n_params()
if npar eq 0 then begin
  print, '% FORMAT: WRIUFO, WDF [, FILENAME] [, FILENAME= filename ] [, FORMAT = format]'
  return
endif

wdf = wdvalid(wdfin, /valid, /quiet, np = np)

if np ne 1 then begin
  print, '% FORMAT: WDF must specify a unique valid WDF array'
  help, wdfin
  print, ' WDF: ', wdf
  return
endif

filename = (n_elements(filename) eq 0) ?  '' : strtrim(filename, 2)

if strlen(filename) eq 0 then begin
  if keyword_set(dialog) then begin
    filename=dialog_pickfile(RESOURCE_NAME='Save as:',/write,filter='*.ufo')
    filename = strtrim (filename, 2)
    if (strlen(filename) eq 0) then return
  endif else begin
    i = 0
    while strlen(filename) eq 0 and (i lt 3) do begin
      read,' Enter filename (or Q to Quit): ',filename
      filename = strtrim(filename, 2)
      i += 1
    endwhile
    if i eq 3 then return
    if strlowcase(strmid(filename, 0, 1)) eq 'q' then return
  endelse
endif
if n_elements(fmt_str) eq 0 then fmt_str = '(2g16.7)'
if size(/type, fmt_str) ne 7 then begin
  print, '% WRIUFO: FORMAT string must be a valid string format'
  help, fmt_str
  return
endif

catch, error_status
if Error_status ne 0 then begin 
  Print, '% WRIUFO: This is not a legal format'
  help, fmt_str
  print, 'Error message: ', !ERROR_STATE.MSG
  catch, /cancel
  return
endif

junk = string ([2.0, 5.0], format = fmt_str)
catch, /cancel

openw, unit, filename, /get_lun, error = ier
if ier ne 0 then begin
  print, '% WRIUFO: Unable to open file: ', strtrim(filename, 2)
  print, !ERROR_STATE.MSG 
  return
endif

num_hdr_lines = 5
header = strarr(num_hdr_lines)
header(0) = '111 = FORMAT #'
header(1) = '2 = # TABLE COLUMNS'
header(2) = '1  1 = ITYPE, IORDER'
header(3) = '0.00E+00  0.00E+00 = ABSTOL, RELTOL'
header(4) = '             X             Y'

; first write the header stuff
FOR i=0,num_hdr_lines-1 DO printf, unit, header[i]

; now write data pairs
w2i, xval, yval, wdf[0]
npts = n_elements(xval)

FOR i=0,npts-1 DO printf, unit, FORMAT=fmt_str, xval[i], yval[i]
printf, unit, -999., -999., format ='(2f16.0)'
free_lun, unit
xval = 0 & yval =0
end
