; ------------------------------------------------------------------------ 
; $Id: xcur.pro,v 1.7 2010/09/20 14:29:06 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
;+
; NAME:
;	XCUR 
;
; PURPOSE:
;       Select values at a finite number of points in a window
;       Procedure will continuously display the (x,y) location of the cursor
;         and store the values when the mouse button is depressed.
;       The full screen cursor can be set at a specific location by editing
;          the displayed values of X and Y and hitting a carriage return in
;          the X or Y display.
;
;       Note: XCURW is a similar routine which will snap to waveform data.
;             
;       Note: MX, MN, FWHM and GET_VALUE often provide the desired information
;             more accurately and quicker.
;
; CALLING SEQUENCE:
;	XCUR [, X , Y] [, dx, dy]
;
;       X and Y are the values when the mouse button is depressed.
;         By default, a symbol will be placed on the plot at these values.
;       DX and DY are the delta values between adjacent values of X, Y
;         Note: If data is sorted then dx and dy will be recalculated.
;
; KEYWORD PARAMETERS:
;	SORT:	  If present and not zero, then data will be sorted in X
;                 Default is SORT = 0
;       WDFARRAY: Store X and Y in a WDFarray given by WDFARRAY
;                 If a valid array is not specified, then the data will be
;                 stored in an empty array if one is available or in
;                 MAXwdfarray if no arrays are empty.
;                 Must have more than 2 points.
;	NOSYMBOL: If present and not zero, no symbols will be plotted.
;
; EXAMPLE:
;	Take values off a curve stored in waveform array 1 and plot the points
;         Before plotting sort the points in abscissa values
;      NOTE: XCURW will do this more efficiently.
;          plo, 1
;          xcur, x, y, /sort
;          plot, x, y
;       Note: To refine the y-values use the following:
;          plo, 1
;          xcur, x, y, /sort
;          nel = n_elements(x)
;          if nel gt 0 then for i = 0, nel-1 do y(i) = get_value(1, x(i))
;          plot, x, y
;
;      Take values from a visual average of arrays 1 and 2 and store the points
;        in wdf array 3.
;          plo, 1, 2, /ov, /nl, /ns
;          xcur, /sort, wdf=3
;
; MODIFICATION HISTORY:
; 	Written by:	L. P. Mix, October 7, 1994
;-


PRO xcur_event, Event

 COMPILE_OPT hidden
 on_error, 1

@pfidltq.com
common xcur_common, xwindow, ywindow, ptr, image, coordmin, coordrange,ratio, $
         datamin, datarange, xcrange, ycrange, xdeltaw, ydeltaw, $
         snapx, snapy, tlb_offset 
  WIDGET_CONTROL,Event.Id,GET_UVALUE=Ev

  CASE Ev OF 

  'FIELD10': BEGIN
      widget_control, xwindow, get_value = xx
      widget_control, ywindow, get_value = yy
       if !x.type eq 1 then xx = alog10(xx)
       if !y.type eq 1 then yy = alog10(yy)
      pnt = ([xx, yy]-datamin)/datarange*coordrange+coordmin
      xcur_draw, pnt(0), pnt(1), image, xcrange, ycrange
      END
  'FIELD12': BEGIN
      widget_control, xwindow, get_value = xx
      widget_control, ywindow, get_value = yy
       if !x.type eq 1 then xx = alog10(xx)
       if !y.type eq 1 then yy = alog10(yy)
      pnt = ([xx, yy]-datamin)/datarange*coordrange+coordmin
      xcur_draw, pnt(0), pnt(1), image, xcrange, ycrange
      END
  'BUTTON13': BEGIN
      widget_control, event.top, tlb_get_offset = tlb_offset
      widget_control, event.top, /destroy
      END
  'BGROUP2': BEGIN
        IF Event.Select THEN Sel = 1 ELSE Sel = 0
        CASE Event.Value OF
          0: snapx = sel
          1: snapy = sel
          ELSE: Message,'Unknown button pressed'
        ENDCASE
      END

  'DRAW5': BEGIN
      coord = ([event.x, event.y]-coordmin)/coordrange*datarange+datamin
      if !x.type eq 1 then coord(0) = 10^coord(0)
      if !y.type eq 1 then coord(1) = 10^coord(1)
      erase
      if event.press eq 1 then begin
        widget_control, ptr, get_uvalue = value, /no_copy
        if (value.np)+1 eq value.npmax then begin
          xx = [value.x, value.x]
          dxx = [value.dx, value.dx]
          yy = [value.y, value.y]
          dyy = [value.dy, value.dy]
          value.npmax = value.npmax +value.npmax
          value = {x:xx, y:yy, dx:dxx, dy:dyy, np:np, npmax:value.npmax, $
              nosymbol:value.nosymbol}
        endif
        if snapx ne 0 and snapy ne 0 then begin
          dist = (cond.x-coord(0))^2 + ratio*(cond.y-coord(1))^2
          junk = min(dist, j)
          coord(0) = cond.x(j)
          coord(1) = cond.y(j)
          event.x = (coord(0)-datamin(0))*coordrange(0)/datarange(0)+coordmin(0)
          event.y = (coord(1)-datamin(1))*coordrange(1)/datarange(1)+coordmin(1)
        endif else if snapx ne 0 then begin
          dist = (cond.x-coord(0))^2
          junk = min(dist, j)
          coord(0) = cond.x(j)
          event.x = (coord(0)-datamin(0))*coordrange(0)/datarange(0)+coordmin(0)
        endif else if snapy ne 0 then begin
          dist = (cond.y-coord(1))^2
          junk = min(dist, j)
          coord(1) = cond.y(j)
          event.y = (coord(1)-datamin(1))*coordrange(1)/datarange(1)+coordmin(1)
        endif
        value.x(value.np) = coord(0)
        value.y(value.np) = coord(1)
        if value.np gt 0 then begin
          value.dx(value.np) = coord(0) - value.x(value.np-1)
          value.dy(value.np) = coord(1) - value.y(value.np-1)
          widget_control, xdeltaw, set_value = value.dx(value.np)
          widget_control, ydeltaw, set_value = value.dy(value.np)
        endif
        value.np = value.np+1
        device, copy = image
        if value.nosymbol eq 0 then begin
          plots, event.x, event.y, /device, psym = 8
          old = !d.window
          wset, image[6]
          device, copy = [image[0:5], old]
          wset, old
        endif 
        plots, xcrange, [event.y, event.y], /device
        plots, [event.x, event.x], ycrange, /device
        widget_control, ptr, set_uvalue = value, /no_copy
      endif else begin
        device , copy = image
        if event.y ge ycrange(0) and event.y le ycrange(1) then $
          plots, xcrange, [event.y, event.y], /device
        if event.x ge xcrange(0) and event.x le xcrange(1) then $
          plots, [event.x, event.x], ycrange, /device
;       xcur_draw, event.x, event.y, image, xcrange, ycrange
      endelse
      widget_control, xwindow, set_value = coord(0)
      widget_control, ywindow, set_value = coord(1)
      END
   else: Print, 'No Handler for event ', ev

  ENDCASE
END

pro xcur_draw, x, y, image, xrange, yrange

 COMPILE_OPT hidden
 on_error, 1

;   tv, image
;         image = tvrd()

    device , copy = image
    if y ge yrange(0) and y le yrange(1) then $
      plots, xrange, [y, y], /device
    if x ge xrange(0) and x le xrange(1) then $
      plots, [x, x], yrange, /device
    return
    end


PRO xcur, xx, yy, dxx, dyy, sort= sortxy, nosymbol = nosymbol, $
        wdfarray = wdfarray, snap = snap, GROUP=Group, modal = modal
 
common xcur_common
on_error, 1

@com.dat
      if n_elements(initwdf) eq 0 then wdfinit

@pfidltq.com
  if n_elements(cond) eq 0 then ncon = 0 else begin
        b = size(cond)
        if b(b(0)+1) ne 8 then ncon = 0 else ncon = cond.ncon
      endelse
  if keyword_set(snap) then begin
    if ncon eq 0 then snap = 0 else snap = 1
  endif else snap = 0

  IF N_ELEMENTS(Group) EQ 0 THEN GROUP=0
  if n_elements (sortxy) eq 0 then sortxy = 0
  if n_elements (nosymbol) eq 0 then nosymbol = 0 
  if nosymbol eq 0 then mkcircle
; if nosymbol eq 0 then mksymbol, 'plus', /fill, size = 1.5

  main13 = WIDGET_BASE(GROUP_LEADER=Group, $
      COLUMN=1, modal = modal, $
      MAP=1, $
      TITLE='X - Cursor')

  BASE8 = WIDGET_BASE(MAIN13, $
      row=1, $
      FRAME=2, $
      MAP=1, $
      UVALUE='BASE8')

  FieldVal435 = 0.0

   xwindow = CW_FIELD( BASE8,VALUE=FieldVal435, $
      ROW=1, $
      FLOAT=1, $
      RETURN_EVENTS = 1, $
      TITLE='Cursor X-Value:', $
      UVALUE='FIELD10')

  FieldVal437 = 0.0

  ywindow = CW_FIELD( BASE8,VALUE=FieldVal437, $
      ROW=1, $
      float=1, $
      TITLE='Cursor Y-Value:', $
      RETURN_EVENTS = 1, $
      UVALUE='FIELD12')

        snapx = 0
        snapy = 0
      if snap ne 0 then begin
        Btns442 = [ $
         'X-Snap', $
         'Y-Snap' ]
        BGROUP2 = CW_BGROUP( base8, Btns442, $
          row=1, $
          NONEXCLUSIVE=1, $
          UVALUE='BGROUP2')
      endif
        
; delta x, y values

  BASE9 = WIDGET_BASE(MAIN13, $
      row=1, $
      FRAME=2, $
      MAP=1, $
      UVALUE='BASE9')

  FieldVal436 = 0.0

   xdeltaw = CW_FIELD( BASE9,VALUE=FieldVal436, $
      ROW=1, $
      FLOAT=1, $
      TITLE='Delta X-Value:', $
      UVALUE='FIELD11')

  FieldVal438 = 0.0

  ydeltaw = CW_FIELD( BASE9,VALUE=FieldVal438, $
      ROW=1, $
      float=1, $
      TITLE='Delta Y-Value:', $
      UVALUE='FIELD13')

  BUTTON13 = WIDGET_BUTTON( BASE9, $
      UVALUE='BUTTON13', $
      VALUE='Done')



  oldwin = !d.window
  if oldwin lt 0 then begin
     Print, 'Data must be displayed in the current window'
     return
  endif
  my_wshow, oldwin, iconic =0
  xsize = !d.x_size
  ysize = !d.y_size
; get old values
  xvals = !x.crange
  if !x.type eq 1 then xvals= 10.^xvals
  yvals = !y.crange
  if !y.type eq 1 then yvals= 10.^yvals
  
  coordmin = convert_coord(xvals(0), yvals(0), /data, /to_device)
  coordmin= coordmin(0:1)
  coordmax = convert_coord(xvals(1), yvals(1), /data, /to_device)
  coordmax= coordmax(0:1)
  coordrange = coordmax-coordmin
  datamin =[!x.crange(0), !y.crange(0)]
  datarange = [!x.crange(1), !y.crange(1)] - datamin
  ratio = (datarange(0)/datarange(1))^2
  xcrange = [coordmin(0), coordmax(0)]
  ycrange = [coordmin(1), coordmax(1)]

  window, /free , /PIXMAP  , xsize = xsize, ysize = ysize
  pixwin = !d.window
  image = [0, 0, xsize, ysize, 0, 0, oldwin]
  device, copy  = image
  image[6] = pixwin

  DRAW5 = WIDGET_DRAW( MAIN13, $
      BUTTON_EVENTS=1, $
      MOTION_EVENTS=1, $
      RETAIN=1, $
      UVALUE='DRAW5', $
      XSIZE=xsize, $
      YSIZE=ysize)

  if n_elements(tlb_offset) eq 0 then tlb_offset = [10, 20]
  WIDGET_CONTROL, MAIN13, /REALIZE, tlb_set_xoffset = tlb_offset[0], $
            tlb_set_yoffset = tlb_offset[1]

; Get drawable window index
  WIDGET_CONTROL, DRAW5, GET_VALUE=DRAW5_Id


  wset , draw5_id
  device , copy = image
  ptr= widget_base()
  npmax = 512L
  np = 0L
  xx = make_array(npmax)
  yy = xx
  dxx = xx
  dyy = xx
  value = {npmax:npmax, np:np, x:xx, y:yy, dx:dxx, dy:dyy, nosymbol:nosymbol}
  widget_control, ptr, set_uvalue = value
  XMANAGER, 'xcur', MAIN13
  image = 0
  widget_control, ptr, get_uvalue = value, /no_copy
  widget_control, ptr, /destroy
  wdelete, pixwin
  if oldwin ge 0 then wset, oldwin
  np = value.np
  if np gt 0 then begin
    xx = value.x(0:np-1)
    yy = value.y(0:np-1)
    if sortxy ne 0 then begin
      ok = sort(xx)
      xx = xx(ok)
      yy = yy(ok)
      if np gt 1 then begin
        dxx = xx(1:np-1)-xx(0:np-2)
        dyy = yy(1:np-1)-yy(0:np-2)
      endif else begin
        dxx = 0
        dyy = 0
      endelse
    endif else begin
      if np gt 1 then begin
        dxx = value.dx(1:np-1)
        dyy = value.dy(1:np-1)
      endif else begin
        dxx = 0
        dyy = 0
      endelse
    endelse
    if n_elements(wdfarray) gt 0 and np gt 2 then begin
      if (wdfarray lt 1 or wdfarray gt maxwdfarray) then begin
         print, 'WDFarray must be greater than 0 and <= to ', $
           strtrim(maxwdfarray, 2)
         ok = where (wdfnp lt 2, nok)
         if nok gt 0 then wdfarray = ok(0) else wdfarray = maxwdfarray
         print, 'Data will be stored in array: ', strtrim(wdfarray,2)
      endif
      arr = [xx, yy]
      put_wdfarray, arr, wdfarray, /xydata, clabel = 'Xcur Data', /no_copy
    endif
  endif else begin
    xx = 0
    yy = 0
    dxx = 0.0
    dyy = 0.0
  endelse
END
