; ------------------------------------------------------------------------ 
; $Id: xcurw.pro,v 1.7 2010/09/20 14:29:06 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
;+
; NAME:
;	XCURW
;
; PURPOSE:
;       Select values at a finite number of points in a window from a specified
;         waveform.
;       Procedure will continuously display the (x,y) location of the cursor
;         and store the values when the mouse button is depressed. Unless
;         SNAP = 0 then the values moved to the line between the two nearest 
;         points on the curve.
;       The full screen cursor can be set at a specific location by editing
;          the displayed values of X and Y and hitting a carriage return in
;          the X or Y display.
;       Note: MX, MN, FWHM and GET_VALUE often provide the desired information
;             more accurately and quicker.
;
; CALLING SEQUENCE:
;	XCURW, WDF, [, X , Y] [, dx, dy] , _extra= extrastuff
;
;       WDF   Waveform array from which values to be obtained.
;       X and Y are the values when the mouse button is depressed.
;         By default, a symbol will be placed on the plot at these values.
;       DX and DY are the delta values between adjacent values of X, Y
;         Note: If data is sorted then dx and dy will be recalculated.
;       extrastuff - Any keywords available for the plo command
;
; KEYWORD PARAMETERS:
;       SNAP:     If present and zero, then X and Y will be actual cursor
;                 positions. 
;                 SNAP=1; Snap to X value of waveform at current Y.
;                         If outside the range, go to minimum or maximum Y.
;                 SNAP=2; Snap to Y value of waveform at current X.
;                         If outside the range, go to maximum or minimum X.
;                 SNAP=3; Snap to waveform 
;                 SNAP=4; Snap to nearest point in the waveform.
;                 Default is SNAP=3
;	SORT:	  If present and not zero, then data will be sorted in X
;                 Default is SORT = 0
;       WDFARRAY: Store X and Y in a WDFarray given by WDFARRAY
;                 If a valid array is not specified, then the data will be 
;                 stored in an empty array if one is available or in 
;                 MAXwdfarray if no arrays are empty.
;	NOSYMBOL: If present and not zero, no symbols will be plotted.
;       X/Ysize:  Size of the graphics window
;
; EXAMPLE:
;	Take values off a curve stored in waveform array 1 and plot the points
;         Before plotting sort the points in abscissa values
;          xcurw, 1, x, y, /sort
;          plot, x, y
;
;       Same as first example but store XY in WDF array 2
;          xcurw, 1, /sort, w=2
;          plo, 2
;
; MODIFICATION HISTORY:
; 	Written by:	L. P. Mix, July 7, 1996
;-


PRO xcurw_event, Event
 COMPILE_OPT hidden

on_error , 1
common xcurw_common, xwindow, ywindow, ptr, image, coordmin, coordrange,ratio, $
     datamin, datarange, xcrange, ycrange, xdeltaw, ydeltaw, snapxy, $
     plen, delx, dely, delxn, delyn, x, y, okk, nokk, v2, nel, $
     xscale, yscale, tlb_offset

  WIDGET_CONTROL,Event.Id,GET_UVALUE=Ev

  CASE Ev OF 

  'FIELD10': BEGIN
      widget_control, xwindow, get_value = xx
      widget_control, ywindow, get_value = yy
       if !x.type eq 1 then xx = alog10(xx)
       if !y.type eq 1 then yy = alog10(yy)
      pnt = ([xx, yy]-datamin)/datarange*coordrange+coordmin
      xcurw_draw, pnt(0), pnt(1), image, xcrange, ycrange
      END
  'FIELD12': BEGIN
      widget_control, xwindow, get_value = xx
      widget_control, ywindow, get_value = yy
       if !x.type eq 1 then xx = alog10(xx)
       if !y.type eq 1 then yy = alog10(yy)
      pnt = ([xx, yy]-datamin)/datarange*coordrange+coordmin
      xcurw_draw, pnt(0), pnt(1), image, xcrange, ycrange
      END
  'BUTTON13': BEGIN
      widget_control, event.top, tlb_get_offset = tlb_offset
      widget_control, event.top, /destroy
      END
  'DRAW5': BEGIN
      coord = ([event.x, event.y]-coordmin)/coordrange*datarange+datamin
      erase
      if event.press eq 1 then begin
        widget_control, ptr, get_uvalue = value, /no_copy
        if (value.np)+1 eq value.npmax then begin
          xx = [value.x, value.x]
          dxx = [value.dx, value.dx]
          yy = [value.y, value.y]
          dyy = [value.dy, value.dy]
          value.npmax = value.npmax +value.npmax
          value = {x:xx, y:yy, dx:dxx, dy:dyy, np:np, npmax:value.npmax, $
              nosymbol:value.nosymbol}
        endif
        if snapxy ge 3 then begin
          coord(0) = (coord(0)-datamin(0))*xscale
          coord(1) = (coord(1)-datamin(1))*yscale
          xtemp = coord(0)-x
          ytemp = coord(1)-y
          distp = xtemp*xtemp+ytemp*ytemp
          minpoint= sqrt (min(distp, jminpoint))
;         with snap eq to 4 must go to nearest point 
          if snapxy eq 4 then nok = 0 else begin
            distan= delxn*xtemp(0:nel-2)+delyn*ytemp(0:nel-2)
            if nokk gt 0 then distan(okk)= -1
            perpdist = abs(ytemp(0:nel-2)*delxn-xtemp(0:nel-2)*delyn)
            ok = where(perpdist lt minpoint and $
                 distan ge 0.0 and distan le plen, nok)
          endelse
          if nok eq 0 then begin
            coord(0) = x(jminpoint)
            coord(1) = y(jminpoint)
          endif else begin
            if nok gt 1 then $
              junk = min(perpdist(ok), jj) else jj = 0
              jj = ok(jj)
            coord(0) = distan(jj)*delxn(jj)+ x(jj)
            coord(1) = distan(jj)*delyn(jj)+ y(jj)
          endelse
          coord(0) = coord(0)/xscale+datamin(0)
          coord(1) = coord(1)/yscale+datamin(1)
          event.x = (coord(0)-datamin(0))*coordrange(0)/datarange(0)+coordmin(0)
          event.y = (coord(1)-datamin(1))*coordrange(1)/datarange(1)+coordmin(1)
        endif else if snapxy eq 2  then begin
          if coord(0) le delxn(0) then begin
            junk = min(x,j)
            coord(1) = y(j)
          endif else if coord(0) ge delxn(1) then begin
            junk = max(x,j)
            coord(1) = y(j)
          endif else begin
            if min(delx) lt 0 then $ 
            ok = where ((coord(0) ge x(0:nel-2) and coord(0) lt x(1:nel-1)) or $
                (coord(0) gt x(1:nel-1) and coord(0) le x(0:nel-2)), nok) else $
            ok = where (coord(0) ge x(0:nel-2) and coord(0) lt x(1:nel-1), nok)
            if nok eq 0 then begin
              print, 'Error in XCURW: nok = 0'
              widget_control, ptr, set_uvalue = value, /no_copy
              return
            endif
            ytemp= y(ok)+dely(ok)*(coord(0)-x(ok))/delx(ok)
            distan = abs(ytemp-coord(1))
            junk = min(distan, j)
            coord(1) = ytemp(j)
          endelse
          event.y=(coord(1)-datamin(1))*coordrange(1)/datarange(1)+coordmin(1)
        endif else if snapxy eq 1 then begin
          if coord(1) le delyn(0) then begin
            junk = min(y,j)
            coord(0) = x(j)
          endif else if coord(1) ge delyn(1) then begin
            junk = max(y,j)
            coord(0) = x(j)
          endif else begin
            ok = where ((coord(1) ge y(0:nel-2) and coord(1) lt y(1:nel-1)) or $
                (coord(1) gt y(1:nel-1) and coord(1) le y(0:nel-2)), nok) 
            if nok eq 0 then begin
              print, 'Error in XCURW: nok = 0'
              widget_control, ptr, set_uvalue = value, /no_copy
              return
            endif
            xtemp= x(ok)+delx(ok)*(coord(1)-y(ok))/dely(ok)
            distan = abs(xtemp-coord(0))
            junk = min(distan, j)
            coord(0) = xtemp(j)
          endelse
          event.x = (coord(0)-datamin(0))*coordrange(0)/datarange(0)+coordmin(0)
        endif
        if !x.type eq 1 then coord(0) = 10^coord(0)
        if !y.type eq 1 then coord(1) = 10^coord(1)
        value.x(value.np) = coord(0)
        value.y(value.np) = coord(1)
        if value.np gt 0 then begin
          value.dx(value.np) = coord(0) - value.x(value.np-1)
          value.dy(value.np) = coord(1) - value.y(value.np-1)
          widget_control, xdeltaw, set_value = value.dx(value.np)
          widget_control, ydeltaw, set_value = value.dy(value.np)
        endif
        value.np = value.np+1
        device, copy = image
        if value.nosymbol eq 0 then begin
          plots, event.x, event.y, /device, psym = 8
          old = !d.window
          wset, image[6]
          device, copy = [image[0:5], old]
          wset, old
        endif 
        plots, xcrange, [event.y, event.y], /device
        plots, [event.x, event.x], ycrange, /device
        widget_control, ptr, set_uvalue = value, /no_copy
      endif else begin
        device , copy = image
        if !x.type eq 1 then coord(0) = 10^coord(0)
        if !y.type eq 1 then coord(1) = 10^coord(1)
        if event.y ge ycrange(0) and event.y le ycrange(1) then $
          plots, xcrange, [event.y, event.y], /device
        if event.x ge xcrange(0) and event.x le xcrange(1) then $
          plots, [event.x, event.x], ycrange, /device
;       xcurw_draw, event.x, event.y, image, xcrange, ycrange
      endelse
      widget_control, xwindow, set_value = coord(0)
      widget_control, ywindow, set_value = coord(1)
      END
   else: Print, 'No Handler for event ', ev

  ENDCASE
END

pro xcurw_draw, x, y, image, xrange, yrange
 COMPILE_OPT hidden
 on_error, 1
;   tv, image
;         image = tvrd()

    device , copy = image
    if y ge yrange(0) and y le yrange(1) then $
      plots, xrange, [y, y], /device
    if x ge xrange(0) and x le xrange(1) then $
      plots, [x, x], yrange, /device
    return
    end


  PRO xcurw, wdf, xx, yy, dxx, dyy, sort= sortxy, nosymbol = nosymbol, $
    xrange = xrange, yrange = yrange, snap = snap, GROUP=Group, modal = modal, $
    xsize =xsizein , ysize = ysizein, wdfarray = wdfarray, _extra= extrastuff

 
common xcurw_common
on_error, 1

@draw_defaults.com
@com.dat
      common pffcom, initpff, pff_exe

      if n_elements(initwdf) eq 0 then wdfinit
      nparams = n_params()
      if nparams eq 0 then begin
        print, '% Format: XCURW, WDA, [, X] [, Y] [, DX] [, DY] [, /SORT]'
        print, '  [, XSIZE = xsize ] [, YSIZE = ysize]
        print, '  [, SNAP = snap] [, /NOSYMBOL] [, _EXTRA = extrastuff]'
        return
      endif
;
      if n_elements(wdf) eq 0 then begin
        print, 'Parameters to XCURW must be defined'
        return
      endif
      if wdf lt 1 or wdf gt maxwdfarray then begin
        print, 'WDF array must be greater than 0 and </=', maxwdfarray
        return
      endif
;
      nel = wdfnp(wdf-1)
      if nel lt 2 then begin
        print, 'No data in the WDF array'
        return
      endif
;

  IF N_ELEMENTS(Group) EQ 0 THEN GROUP=0
  if n_elements (sortxy) eq 0 then sortxy = 0
  if n_elements (nosymbol) eq 0 then nosymbol = 0 
  if n_elements (snap) eq 0 then snap = 3 else snap = fix(snap)
  if snap lt 0 or snap gt 3 then snap = 3 
; if nosymbol eq 0 then mksymbol, 'plus', /fill, size = 1.5

  main13 = WIDGET_BASE(GROUP_LEADER=Group, $
      COLUMN=1, modal= modal, $
      MAP=1, $
      TITLE='X - Cursor')

  BASE8 = WIDGET_BASE(MAIN13, $
      row=1, $
      FRAME=2, $
      MAP=1, $
      UVALUE='BASE8')

  FieldVal435 = 0.0

   xwindow = CW_FIELD( BASE8,VALUE=FieldVal435, $
      ROW=1, $
      FLOAT=1, $
      RETURN_EVENTS = 1, $
      TITLE='Cursor X-Value:', $
      UVALUE='FIELD10')

  FieldVal437 = 0.0

  ywindow = CW_FIELD( BASE8,VALUE=FieldVal437, $
      ROW=1, $
      float=1, $
      TITLE='Cursor Y-Value:', $
      RETURN_EVENTS = 1, $
      UVALUE='FIELD12')
        
; delta x, y values

  BASE9 = WIDGET_BASE(MAIN13, $
      row=1, $
      FRAME=2, $
      MAP=1, $
      UVALUE='BASE9')

  FieldVal436 = 0.0

   xdeltaw = CW_FIELD( BASE9,VALUE=FieldVal436, $
      ROW=1, $
      FLOAT=1, $
      TITLE='Delta X-Value:', $
      UVALUE='FIELD11')

  FieldVal438 = 0.0

  ydeltaw = CW_FIELD( BASE9,VALUE=FieldVal438, $
      ROW=1, $
      float=1, $
      TITLE='Delta Y-Value:', $
      UVALUE='FIELD13')

  BUTTON13 = WIDGET_BUTTON( BASE9, $
      UVALUE='BUTTON13', $
      VALUE='Done')



  oldwin = !d.window
  if n_elements(xsizein) eq 0 then xsize = medsize(0) else xsize = xsizein
  if n_elements(ysizein) eq 0 then ysize = medsize(1) else ysize = ysizein

  DRAW5 = WIDGET_DRAW( MAIN13, $
      BUTTON_EVENTS=1, $
      MOTION_EVENTS=1, $
      RETAIN=1, $
      UVALUE='DRAW5', $
      XSIZE=xsize, $
      YSIZE=ysize)

  if n_elements(tlb_offset) eq 0 then tlb_offset = [10, 20]
  WIDGET_CONTROL, MAIN13, /REALIZE, tlb_set_xoffset = tlb_offset[0], $
            tlb_set_yoffset = tlb_offset[1]

; Get drawable window index
  WIDGET_CONTROL, DRAW5, GET_VALUE=DRAW5_Id
  wset , draw5_id
  plo, wdf, xrange = xrange, yrange = yrange, eng = 0, _extra = extrastuff
  if nosymbol eq 0 then mkcircle
  w2i, x, y, wdf
; get old values
  xvals = !x.crange
  if !x.type eq 1 then begin
     xvals= 10.^xvals
     x = alog10(x)
  endif
  yvals = !y.crange
  if !y.type eq 1 then begin
     yvals= 10.^yvals
     y = alog10(y)
  endif

  coordmin = convert_coord(xvals(0), yvals(0), /data, /to_device)
  coordmin= coordmin(0:1)
  coordmax = convert_coord(xvals(1), yvals(1), /data, /to_device)
  coordmax= coordmax(0:1)
  coordrange = coordmax-coordmin
  datamin =[!x.crange(0), !y.crange(0)]
  datarange = [!x.crange(1), !y.crange(1)] - datamin
  ratio = (datarange(0)/datarange(1))^2
  xcrange = [coordmin(0), coordmax(0)]
  ycrange = [coordmin(1), coordmax(1)]
  if snap ge 3 then begin
    xscale = (coordmax(0)-coordmin(0))/datarange(0)
    yscale = (coordmax(1)-coordmin(1))/datarange(1)
    x = (x-datamin(0))*xscale
    y = (y-datamin(1))*yscale
    nel = n_elements(x)
    if snap eq 3 then begin
      delx = x(1:nel-1)-x(0:nel-2)
      dely = y(1:nel-1)-y(0:nel-2)
      plen = sqrt(delx*delx+dely*dely)
      okk = where (plen eq 0,nokk)
      if nokk gt 0 then plen(okk) = 1. 
      delxn = delx/plen
      delyn = dely/plen
    endif
  endif else if snap gt 0 then begin
    delxn = [ min(x, max = mm), mm]
    delyn = [ min(y, max = mm), mm]
    delx = x(1:nel-1)-x(0:nel-2)
    dely = y(1:nel-1)-y(0:nel-2)
  endif 
 
  window, /free , /PIXMAP  , xsize = xsize, ysize = ysize
  pixwin = !d.window
  image = [0, 0, xsize, ysize, 0, 0, draw5_id]
  device, copy  = image
  image[6] = pixwin

  wset , draw5_id
  ptr= widget_base()
  npmax = 512L
  np = 0L
  xx = make_array(npmax)
  yy = xx
  dxx = xx
  dyy = xx
  snapxy = snap
  value = {npmax:npmax, np:np, x:xx, y:yy, dx:dxx, dy:dyy, nosymbol:nosymbol}
  widget_control, ptr, set_uvalue = value
  XMANAGER, 'xcurw', MAIN13
  image = 0
  widget_control, ptr, get_uvalue = value, /no_copy
  widget_control, ptr, /destroy
  if oldwin ge 0 then wset, oldwin
  np = value.np
  wdelete, pixwin
  if np gt 0 then begin
    xx = value.x(0:np-1)
    yy = value.y(0:np-1)
    if sortxy ne 0 then begin
      ok = sort(xx)
      xx = xx(ok)
      yy = yy(ok)
      if np gt 1 then begin
        dxx = xx(1:np-1)-xx(0:np-2)
        dyy = yy(1:np-1)-yy(0:np-2)
      endif else begin
        dxx = 0
        dyy = 0
      endelse
    endif else begin
      if np gt 1 then begin
        dxx = value.dx(1:np-1)
        dyy = value.dy(1:np-1)
      endif else begin
        dxx = 0
        dyy = 0
      endelse
    endelse
    if n_elements(wdfarray) gt 0 and np gt 2 then begin
      if (wdfarray lt 1 or wdfarray gt maxwdfarray) then begin
         print, 'WDFarray must be greater than 0 and <= to ', $
           strtrim(maxwdfarray, 2)
         ok = where (wdfnp lt 2, nok)
         if nok gt 0 then wdfarray = ok(0) else wdfarray = maxwdfarray
         print, 'Data will be stored in array: ', strtrim(wdfarray,2)
      endif
      arr = [xx, yy]
      put_wdfarray, arr, wdfarray, /xydata, xlabel= wdfxlabel(wdf-1), $
      ylabel = wdfylabel(wdf-1), clabel = 'Xcurw of '+wdfcomment(wdf-1), $
      file = wdffile(wdf-1), /no_copy
    endif
  endif else begin
    xx = 0
    yy = 0
    dxx = 0
    dyy = 0
  endelse
END
