; ------------------------------------------------------------------------ 
; $Id: xfr.pro,v 1.4 2007/06/06 20:03:56 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
;
;+
; NAME:
;       XFR
;
; PURPOSE:
;       Copies a WDF array to another WDF array number
;
;
; CALLING SEQUENCE:
;       XFR, WDF1, WDF2 [, NARRAY] [, QUIET = quiet]
;
; INPUTS:
;       WDF1:   WDF array number or an array of WDF array numbers
;       WDF2:   WDF array number or an array of WDF array numbers
;               If NARRAY is not present, WDF2 should have either one 
;               element or the same number of as WDF1.
;               If NARRAY is present, then WDF2, should have either one
;               element or narray elements.
;               If the number of WDF1 elements is larger than the number
;               of WDF2 elements, then sequential elements are added.
;               Copies are done exactly as specified. 
;
; OPTIONAL INPUTS:
;       NARRAY:	If specified, then the input value of WDF1 is replaced
;               by the following array.
;                 WDF1 = WDF1[0]+indgen(naraay)
;
;               Note:
;                 XFR, 1, 2, 10 
;               will make 10 new copies of WDF array 1 in arrays 2 to 11.
;
; KEYWORDS:
;       QUIET:  Informational print statements are deleted when bad data 
;               is encountered.  Bad data is ignored.
;               When no WDF arrays are copied, an informational statement 
;               will always be printed.
;
;               Bad data is where WDF1 is not a valid WDF array or 
;                 where WDF2 is not a valid WDF array.  
;	
; OUTPUTS:
;       The specified WDF arrays are copied to their new locations.
;
; EXAMPLE:
;	Copy WDF array 4 to array 14
;         XFR, 4, 14
;
;       Copy WDF arrays, 1 to 10 to WDF arrays 301, to 310
;         XFR, 1, 301, 10
;
;       Copy WDF arrays, 1 to 3, to arrays, 301, 401, 501
;         XFR, 1, [301, 401, 501], 3
;
;       Copy WDF arrays, 1, 2, 4, 5 and 7 to WDF arrays, 301 to 305.
;         XFR, [1, 2, 4, 5, 7], 301
;
; MODIFICATION HISTORY:
; 	Written by:	L. P. Mix, Jr., about 1990
;	April, 3, 2007  Added NARRAY parameter
;-
;
 Pro  xfr , wdf1ain, wdf2ain, narray, quiet = quiet
;
@com.dat
;
      if n_elements(initwdf) eq 0 then wdfinit
      nparams = n_params()
      if nparams eq 0 then begin
        print, '% XFR, WDF1, WDF2, NARRAY'
        return
      endif
      if nparams lt 2 then begin
        print,'% XFR: XFR must have at least 2 parameters'
        return
      endif
;
      if n_elements(wdf1ain) eq 0 or  n_elements(wdf2ain) eq 0 then begin
        print, '% XFR: Parameters must be defined'
        return
      endif
      wdf1a = wdf1ain
      wdf2a = wdf2ain
      if n_elements(narray) gt 0 then wdf1a = wdf1a[0]+lindgen(narray)
      narray = n_elements(wdf1a)
      n2 = n_elements(wdf2a)
      if n2 lt narray then wdf2a = [wdf2a, wdf2a[n2-1]+lindgen(narray-n2)+1]
      ok = where (wdf1a gt 0 and wdf1a le maxwdfarray, nok, comp = comp, ncomp = ncomp)
      if ncomp gt 0 then begin
        if nok eq 0 then begin 
          print, '% XFR: No valid WDF arrays'
          return
        endif
        if not keyword_set(quiet) then $
          print, '% XFR: Invalid WDF arrays not copied: ', wdf1a[comp]
        wdf1a = wdf1a[ok]
        wdf2a = wdf2a[ok]
      endif
      ok = where(wdfnp[wdf1a-1] gt 0, nok, comp = comp, ncomp = ncomp)
      if ncomp gt 0 then begin
        if nok eq 0 then begin 
          print, '% XFR: No data in any of the valid WDF arrays'
          return
        endif
        if not keyword_set(quiet) then $
          print, '% XFR: No data in some WDF arrays: ', wdf1a[comp]
        wdf1a = wdf1a[ok]
        wdf2a = wdf2a[ok]
      endif
      ok = where (wdf2a gt 0 and wdf2a le maxwdfarray, nok, comp = comp, ncomp = ncomp)
      if ncomp gt 0 then begin
        if nok eq 0 then begin 
          print, '% XFR: No valid WDF destination arrays'
          return
        endif
        if not keyword_set(quiet) then $
          print, '% XFR: Invalid WDF destination arrays: ', wdf2a[comp]
        wdf1a = wdf1a[ok]
        wdf2a = wdf2a[ok]
      endif
      nend = n_elements(wdf1a)
      if ((nend lt narray) && (~ keyword_set(quiet))) then $
        Print, '% XFR: The following arrays will be copied: ', wdf1a
      for i = 0, nend-1 do begin
        wdf1 = wdf1a[i]
        wdf2 = wdf2a[i]
        get_wdfarray, array1, wdf1, /xydata
        xydata = (n_elements(array1) eq 2*wdfnp[wdf1-1]) ? 1 : 0
        put_wdfarray, array1, wdf2, xydata = xydata, /no_copy
        wdfcomment(wdf2-1)= wdfcomment(wdf1-1)
        wdffile(wdf2-1)= wdffile(wdf1-1)
        wdfxlabel(wdf2-1)= wdfxlabel(wdf1-1)
        wdfylabel(wdf2-1)= wdfylabel(wdf1-1)
      endfor
      end
