; ------------------------------------------------------------------------ 
; $Id: xplo.pro,v 1.4 2009/11/03 16:53:07 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
 Pro  xplo, wdfx, wdfy, xrange = xr, yrange = yr, header = header, $
      title= title, nogrid = nogrid,  delta = delta, fill = fill, $
      psymbol = psym, out = out, engineering = engineering, $
      symbol = symbol, linestyle = linestyle,color = color, $
      xtitle = xtitle, ytitle = ytitle, overplot = overplot,  _extra= extrastuff
;
;+
; NAME:
;       XPLO
;
; PURPOSE:
;       Plot a WDF array versus a second WDF array at the same times.
; CATEGORY:
;       Input/Output
; CALLING SEQUENCE:
;      XPLO, WDx , WDy, [, XRange =xrange][, YRange =yrange] [, /HEADER] 
;      [, TITLE =title][, /NOGRID] [, DELTA = delta] [, FILL = fill] 
;      [, PSYMBOL =symbol][, LINESTYLE =linestyle][, COLOR = color] 
;      [, OUT = out] [, ENGINEERING = engineering][, /OVERPLOT] [, /NO_SORT]
;      [, XTITLE = xtitle] [, YTITLE = ytitle]  [, _EXTRA = extrastuff]
; INPUTS:
;       WDX, WDY:      WDF array numbers to be plotted versus each other
; KEYWORD PARAMETERS:
;       COLOR     Indicates the color to be used for the curve.  If /color is 
;                 present the curve will be drawn in red.
;                 The colors are: red(1), green(2), blue(3),
;                 magenta(4), orange(5), cyan(6), yellow(7) and white(8).
;       LINESTYLE Indicates the type of line to be used.  Lines specified by
;                 LINESTYLE are: solid(0), dot(1), dash(2), dot-dash(3),
;                 dot-dot-dot-dash(5), and long dash(5).  Default is 0.
;       /NOGRID   Indicates the curve should be plotted on the exixting grid.
;                 ENGINEERING is ignored if NOGRID is set
;       DELTA     Indicates a symbol is to be plotted on the curve at 
;                 approximate increments given by delta.
;       HEADER    If specified a plot header, consisting of the current date
;                 and the current file name, is displayed in the lower left
;                 corner of the plot.  See the HEADER command to set default
;                 HEADER parameters (character size, spacing, alignment).
;                 DEFAULT is HEADER = 1
;                 NOTE: For X-windows screen dumps, many will want to increase
;                      the character size with the command:
;                      HEADER, /set, charsize = 0.8
;                      (Default is 0.6 of current size)
;       FILL      If set, a filled symbol is used.
;                 (only symbols 11 to 23)
;       PSYMBOL   Indicates the type of symbol to be used.  If delta is zero,
;                 about 40 symbols will be plotted and lines will connect the
;                 points.  Values of SYMBOL are: plus sign(1), 2 (asterisk),
;                 3 (period), 4 (diamond), 5 (triangle), 6 (square), 7 ("X").
;                 Default is 1.
;                 USER Defined is a filled circle.
;                 (If the symbol value is greater than 10 then ll will be
;                 subtracted from the value and MKSYMBOL will be called.)
;                 (See help on SYMBOL_LIST for a description of additional
;                 symbols.)
;                 Note: Symbols may also be specified as names:
;                   PSYM=['sq', 'ci', 'diamond', 'clover']
;                   These will be converted to numbers. [11, 12, 14, 21]
;                   A null or blank string('' or '  ') will result in no symbol.
;       TITLE     string to be used as the title of the plot. 
;       ENGINEERING If set, maximum/minimum values will be in the interval
;                 (1, 1000).  For example, if maximum is 1.0e7 volts and
;                 250 e-9 sec  then the y values will be multiplied by 1e-6
;                 and ' x1e6' will be added to the y label and x values
;                 multiplied by 1e9 and ' x1e-9' added to the x label.
;       XTITLE, YTITLE  String to be used for the X and Y axes.  
;                If ENGINEERING is set, the titles will have the scaling 
;                information attached to the label.
;       XRANGE, YRANGE   two element arrays with the first element equal to the 
;                minimum and the second element, the maximum value of the 
;                x-axis and y-axis, respectively.  These values will be 
;                rounded to "nice" values unless xstyle and/or ystyle have
;                been set.
;       OUT      WDF array of the cross plotted data stored as X-Y data
;       OVERPLOT Plot the data using the existing grid  
;                (NOGRID is set to 1)
;       NO_SORT  If data is XY then it will be sorted unless NO_SORT is set.
;                If NO_SORT is set, then delta is set to zero.
;       _EXTRA
;                Any other valid parameters to the plot command in IDL
;
; OUTPUTS:
;       Designated datasets are plotted.
; COMMON BLOCKS:
;       COM.DAT  WDF common block 
; RESTRICTIONS:
;       WDX, WDY arrays must be defined and have valid data
; PROCEDURE:
;       Put WDX and WDY on a common time base and plot wdy versus wdx.
;       Default axis labels is the dataset comment + the y axis label
;       If delta is defined then plot a symbol at increments of 
;       delta along the curve.  If symbol is defined, then plot about 40 points
;       at uniform spacing along the curve
; EXAMPLES:
;       Plot WDF array 2 versus WDF array 3 and put a symbol every 5 ns.
;               XPLO, 2, 3, delta = 5e-9
;       Plot Density vs Temperature and save the result in wdf array 67.
;           XPLO, 'Density', 'Temperature', out = 67
; MODIFICATION HISTORY:
;      Original version, L. P. Mix, August 15, 1992
;-
; include wdf common blocks
;
@com.dat

@plo_defaults.com

 common plot_grid_last, yeng_scale, xeng_scale
      if n_elements(yeng_scale) eq 0 then begin
        xeng_scale = 1.00
        yeng_scale = 1.00
      endif

;
 on_error, 1

@symbol_list.pro
;  ; Special symbols available with MKSYMBOL
;    and their number and character representation
; #    Name     Character
; 0    Square     SQ
; 1    Circle     CI or O
; 2    Triangle   TR
; 3    Diamond    DI
; 4    Big X      BI or X
; 5    Delta      DE
; 6    Star       ST
; 7    Plus       PL or +
; 8    Hexagon    HE
; 9    Pentagon   PE
; 10   Clover     CL
; 11   PACMAN     PA
; 12   Spiral     SP
;
; symbol_list = ['SQUARE', 'CIRCLE', 'TRIANGLE', 'DIAMOND', 'BIGX', $
;            'DELTA', 'STAR', 'PLUS', 'HEXAGON', 'PENTAGON', $
;            'CLOVER', 'PACMAN', 'SPIRAL']
;
      if n_elements(nel_symbols) eq 0 then nel_symbols = 12
;
;
;     
      if n_elements(psym) ne 0 then symbol = psym
      if n_elements(initwdf) eq 0 then wdfinit
      nparams = n_params()
      if nparams lt 2 then begin
print,'% Format: XPLO, WDx , WDy, [, XRange =xr][, YRange =yr] [, /HEADER]'
print,'          [, TITLE =title][, /NOGRID] [, DELTA = delta] [, FILL = fill]'
print,'          [, PSYMBOL =symbol][, LINESTYLE =linestyle][, COLOR = color]'
print,'          [, OUT = out] [, ENGINEERING = engineer] [, /OVERPLOT]'
print,'          [, _EXTRA = extrastuff]'

        return
      endif
;
      if n_elements (delta) eq 0 then delta = 0 else delta = abs(delta)
;
      wd1 = wdvalid(wdfx, /valid, npoint = np) 
      if np ne 1 then begin
        print, '% XPLO: One and only one dataset must be specified for X axis'
        print, '        WDVALID selectd:', wd1
        return
      endif
      wdfx = wd1[0]
;
      wd1 = wdvalid(wdfy, /valid, npoint = np) 
      if np ne 1 then begin
        print, '% XPLO: One and only one dataset must be specified for Y axis'
        print, '        WDVALID selectd:', wd1
        return
      endif
      wdfy = wd1[0]
;
      if n_elements(symbol) gt 0 then begin
        symbol = symbol(0)
        a = size(symbol)
        if a(a(0)+1) eq 7 then begin
            s = strupcase(strcompress(symbol, /remove_all))
            l = strlen(s)
            if l eq 1 then begin
              if s eq 'X' then s = 'BIGX' else $
              if s eq '+' then s = 'PLUS' else $
              if s eq 'O' then s = 'CIRCLE'
              l = strlen(s)
            endif
            if l eq 0 then temp = 12 else begin
              templist = strmid(symbol_list, 0,l)
              ok = where (s eq templist, nok )
              if nok eq 0 then begin
                print, 'A valid symbol string must be specified - ', s
                return
              endif else if nok eq 1 then temp = ok(0) else begin
                print, ' Multiple symbols specified, using ' , s
                print, symbol_list(ok)
                temp = ok(0)
                print, '   Using a symbol of ', symbol_list(ok(0))
              endelse
            endelse
          psymbol = temp+11
        endif  else psymbol = symbol(0)
      endif  else begin
        psymbol = 0
        if delta gt 0 then psymbol = 8
      endelse
      psymbol = abs(psymbol)

      nel = wdfnp(wdfx-1)
      nelb = wdfnp(wdfy-1)
      get_wdfarray, array1, wdfx, /xydata
      get_wdfarray, array2, wdfy, /xydata
      max1 = wdfmaxx(wdfx-1)
      max2 = wdfmaxx(wdfy-1)
      get_common_time, array1, array2, max1, max2, nel, nelb, xydata
      if n_elements(array1) lt 4 then begin
        print, 'No values will be plotted due to lack of overlap'
        return
      endif
; now we have the arrays on a common time base -- let's plot
      if xydata eq 0 then begin
        t0 = array1(0)
        dt = array1(1)
        array1 = array1(2:*)
        array2 = array2(2:*)
        np = n_elements(array1)
      endif else begin
        np= n_elements(array1)/2
        tt = array1[0:np-1]
        array1= temporary(array1[np:*])
        array2= temporary(array2[np:*])
        if not keyword_set(no_sort) then begin
          dt = tt[1:np-1]-tt[0:np-2]
          if min(dt) lt 0 then begin
            a = sort(tt)
            tt = tt[a]
            array1= array1[a]
            array2= array2[a]
          endif
        endif else delta = 0 
      endelse

      if n_elements (color) ne 0 then loadxyct 
      if n_elements(linestyle) eq 0 then linestyle=0 else linestyle=linestyle
      if linestyle gt 5 then linestyle = 0
      if n_elements(header) eq 0 then header = 1
      if n_elements(fill) eq 0 then fill = 0
      lcheader = 0
      if psymbol gt 8 then begin
        if psymbol gt 10 then begin
          tmpsym  = (abs(psymbol) - 11) < (nel_symbols-1)
          mksymbol, tmpsym, fill = fill
          psymbol = 8
        endif else psymbol = 1
      endif else if psymbol eq 8 then mksymbol, 1, fill = fill
      if n_elements (xr) lt 2 then xrp= [min(array1, max = max1), max1] $
         else xrp = xr[0:1]
      if n_elements (yr) lt 2 then yrp= [min(array2, max = max2), max2] $
         else yrp = yr[0:1]

      if keyword_set(overplot) then nogrid = 1
      if not keyword_set(nogrid)  then over_offset = 0
      xeng_label = ''
      yeng_label = ''
      if keyword_set(nogrid) then begin
        if xeng_scale ne 1.00 then array1 = array1*xeng_scale
        if yeng_scale ne 1.00 then array2 = array2*yeng_scale
      endif else  if keyword_set(engineering) then begin
        get_engr, max(abs(xrp)), xeng_scale, xeng_label
        if xeng_scale ne 1.00 then array1 = array1*xeng_scale
        xrp = xrp * xeng_scale

        get_engr, max(abs(yrp)), yeng_scale, yeng_label
        if yeng_scale ne 1.00 then array2 = array2*yeng_scale
        yrp = yrp * yeng_scale
      endif else begin
        xeng_scale = 1.0
        yeng_scale = 1.0
      endelse
      if n_elements(xtitle) gt 0 then xlab = xtitle(0) else begin
        st   = strpos( wdfcomment(wdfx-1), ' - ')
        if st gt 0 then x1 = strtrim(strmid(wdfcomment(wdfx-1), 0, st), 2) $
             else x1 =  strtrim(wdfcomment(wdfx-1),2)
        st   = strpos( wdfylabel(wdfx-1), ' - ')
        if st gt 0 then x2 = strtrim(strmid(wdfylabel(wdfx-1), 0, st), 2) $
              else x2 =  strtrim(wdfylabel(wdfx-1),2)
        if (strlen(strtrim(x1, 2)) eq 0) or (strlen(strtrim(x2, 2)) eq 0) then $
        xlab = strtrim (x1+x2, 2)  else $
        xlab = x1+' - '+x2
      endelse
      xlabp = xlab+xeng_label

      if n_elements(ytitle) gt 0 then ylab = ytitle(0) else begin
        st   = strpos( wdfcomment(wdfy-1), ' - ')
        if st gt 0 then x1 = strtrim(strmid(wdfcomment(wdfy-1), 0, st), 2) $
            else x1 =  strtrim(wdfcomment(wdfy-1),2)
        st   = strpos( wdfylabel(wdfy-1), ' - ')
        if st gt 0 then x2 = strtrim(strmid(wdfylabel(wdfy-1), 0, st), 2) $
            else x2 =  strtrim(wdfylabel(wdfy-1),2)
        if (strlen(strtrim(x1, 2)) eq 0) or (strlen(strtrim(x2, 2)) eq 0) then $
        ylab = strtrim (x1+x2, 2) else $
        ylab = x1+' - '+x2
      endelse
      ylabp = ylab+yeng_label
      if n_elements(title) gt 0 then $
         plot_title = strtrim(title, 2) $
      else $
         plot_title = ylab+' versus '+xlab
      if not keyword_set(nogrid) then begin
;       draw a grid with labels
        plot, array1, array2, $
        xtitle= xlabp, ytitle = ylabp, title= plot_title, $
          xr = xrp, yr =yrp, /nodata, _extra = extrastuff
        filename = strtrim(wdffile (wdfx-1), 2)
        if strlen(filename) eq 0 then header_text='Date: ' + thin_date() $
          else header_text = ['Date: ' + thin_date(), 'File: ' +filename]
        if header ne 0 then header, header_text
      endif 
      if (n_elements(color) gt 0) then $
          oplot,  array1, array2, color = color , lin = linestyle, _extra = extrastuff $
      else oplot, array1, array2 , lin = linestyle, _extra = extrastuff
      if psymbol gt 0 and delta gt 0 then begin
        if delta gt 0.0 then begin
          if xydata gt 0 then begin
            ndel= ((tt[np-1]-tt[0])/delta)+1
            nsym = floor (ndel)
            tim = (tt[0]+0.5*(ndel-nsym)*delta)+findgen(nsym)*delta
            a = make_array(nsym)
            j = 1L
            for i = 0, nsym-1 do begin
              t = tim[i]
              while (t gt tt[j] and j lt np-1) do j = j+1
              a[i] = j - ((tt[j]-t)/(tt[j]-tt[j-1]) > 0)
            endfor
            a = round(a)
          endif else begin
            nsym = round((np-1)*dt/delta+1.0)
            a = round(findgen(nsym)*(delta/dt) + 0.5) < (np-1)
          endelse
        endif  else a = round(findgen(min([40, np]))*(np/40.)+0.5) < (np-1)
;
        if (n_elements(color) gt 0) then oplot, $
           array1(a), array2(a),color=color,lin=linestyle,psym=psymbol,  $
           _extra = extrastuff else $
        oplot, array1(a), array2(a), lin = linestyle, psym=psymbol, _extra = extrastuff
      endif
      if n_elements(out) ne 0 then begin
        out = out[0]
        if out lt 1 or out gt maxwdfarray then begin
          print, 'OUT array must be greater than 0 and </=', maxwdfarray
          return
        endif
        if xeng_scale ne 1.00 then array1 = array1/xeng_scale
        if yeng_scale ne 1.00 then array2 = array2/yeng_scale
        i2w, array1, array2, out, clabel = plot_title, xlabel = xlab, $
             ylabel = ylab, /xydata
      endif
      array1 = 0
      array2 = 0
      end
