; ------------------------------------------------------------------------ 
; $Id: zoomw.pro,v 1.20 2010/09/20 14:29:06 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
;+
; NAME:
;	ZOOMW
;
; PURPOSE:
;	Provide a easy to zoom display of waveform data.
;       Note: Display will not be changed unless zoom endpoint changes by 
;             more than 3%
;       Note: All inputs are the same as the PLO command.  See PLO for 
;             more details.  
;
; CATEGORY:
;	Graphics 
;
; CALLING SEQUENCE:
;
;	ZOOMW, WDA, NCurve
;
; INPUTS:
;
;       WDA:      WDF array number or an array of WDF array numbers
;
; OPTIONAL INPUT:
;       NCURVE:   Number of curves to be plotted.  If WDA is a single WDF 
;                 array number, then WDA and the following ncurve-1 arrays
;                 will be plotted.  Default = 1
;	
; KEYWORD PARAMETERS:
;       BIG_CURSOR: If set a full screen cursor will be used.
;                 Default: BIG_CURSOR = 0
;       LIMIT:    If set, Waveforms(WDA + indgen(ncurve)) will be limited 
;                 to this range of values
;                 LIMIT uses the LIM command:
;                 LIM, WDA + INDGEN(ncurve), zoomrange[0], zoomrange[1]
;       Maxzoom:  Maximum ZOOM parameter, Default = 100 
;                 A factor of 3 higher magnification is obtainable by
;                 adjusting endpoints on the graph; the display will only
;                 read MAXZOOM
;      Zoomrange: Final value of XRANGE in the right window.
;                 This value can be used with PLO to plot other 
;                   data with the same abscissa values
;                 If Zoomrange has two values on input, these values are the 
;                   initial values of the zoom.
;       X/Ysize:  Size of windows, Default is set by Dualsize
;                 Minimum window size is 100 x 100
;                 (See SET_DRAWSIZE in PFIDL.)
;       CURSOR:   Allows a display of cursor position in the right window.  
;                 A Left button press will center the right window on that 
;                    X-value.
;                 A RIGHT button press will save the point into the keyword
;                    parameters: Xcursor and Ycursor.
;                 Default: Cursor = 1
;                 If cursor = 0, then this function is disabled.
;                 Note: Values will still be displayed in the windows
;       X/YRange: Initial values for xrange and yrange in the left plot.
;       X/YCursor: These values are set with the right mouse button.
;                  Values can be deleted using the delete button.  
;                  Designated points are shown in both windows.
;                  If both XCURSOR and YCURSOR are not valid parameters, 
;                  a warning will be issued when the user attempts to save
;                  a value.
;       NCursor:  Number of values designated in xcursor, ycursor.
;
;         Note: The following 3 keywords are processed in the following 
;               order: UNSET, SetDefault, ShowDefault
;
;       Unset      : Set Cursor and Maxzoom to their default values.
;                    Xsize and Ysize must be set using explicit values in
;                    ZOOMW or Set_DrawSize
;                    Defaults values which are set are:
;                             Cursor  = 1
;                             BIG_CURSOR = 0
;                             MaxZoom = 100
;                             Xsize   = Dualsize[0]
;                             Ysize   = Dualsize[1]
;       SetDefault : Use to set the default value of Cursor, Maxzoom, Xsize
;                    and Ysize
;                  Note: Values in Dualsize will be changed if Xsize and Ysize
;                        are set.
;       Showdefault: Show current default values
;
;      All other keyword parameters are the same as PLO except the following
;           which are set specifically for part or all of the ZoomW command.
;      Legend:	Legend position for the default plot. This is set to -1 for 
;                the Zoom image
;       Header:   Used for the plot header; set to 0 for the zoom image
;	Grid:	  Set to 1
;       OVER:     If Ncurve gt 1 or WDA is an array, then over = 1.
;       NoGrid:   Set to zero
;       XSTYLE:   Set to 1 by default for all images. Original plots will used
;                 specified values of Xstyle or 1 if no value is provided
;       INSTRUCTIONS:  Puts up an instruction widget.
;       INSTR_OFFSET:  Location for the instruction widget
;                      Default OFFSET = 0.5*[Xsize, Ysize]
; Procedure:
;       Sliders allow the maximum and minimum of the Zoomed image to be set.
;       or 
;       Sliders allow the center point and the magnification to be set.
;       or 
;       The center point can be set by clicking on the original image; the
;          maginifcation will be taken from the slider value.
;       A right click in the right window will store the values.
;
;       NOTE:  A right click in the left window will replot the curves.
;
; EXAMPLE:
;       Examine the details of the data in wdf array 3.
;         ZOOMW, 3
;
;       Compare arrays 2, 3, 4, and 5 in detail, put the legend at relative
;         positions of [0.5, 0.5]
;         ZOOMW, 2, 4, /ov, leg =[.5, .5]
;
; MODIFICATION HISTORY:
; 	Written by:       Paul Mix, October 10, 1994
;-

PRO Zoomw_event, Event
compile_opt hidden

 on_error, 1
  common plot_grid_zoom, yeng_zoom_scale, xeng_zoom_scale
  common plot_grid_last, yeng_scale, xeng_scale
  common zoomw_save_common, cursor_save, maxzoom_save, big_cursor_save, $
         window_offset, offset_save
  common zoomw_common, par1, par2, ov, extrastuff, draw1, draw2, $
            fslid1, fslid2, fslid3, fslid4, middle, magnif, xmin, xmax, $
            datamin, datarange, xcrange, ycrange, coordmin, coordrange, $
            datamax, image, xmaxt, xmint, maxzoom, epsilon, xsize, ysize, $
            wxcur, wycur, xcur, ycur, ncur, maxcur, big_cur, bangx, bangy, $
            bangp, lcsavepoints
  WIDGET_CONTROL,Event.Id,GET_UVALUE=Ev
  tolerance = 0.02/alog10(maxzoom)
  CASE Ev OF 

  'FSLID1': BEGIN
      if event.drag eq 0 then begin
        middle = event.value 
        update_zoomw, 0
      endif
      END

  'FSLID2': BEGIN
      if event.drag eq 0 then begin
        magnif = (10.0^(event.value)) > 1.0
        update_zoomw, 0
      endif
      END

  'FSLID3': BEGIN
      if event.drag eq 0 then begin
        xmint = event.value
        if abs(xmin-xmint)/(xmax-xmin) gt tolerance then update_zoomw, 1
      endif
      END

  'FSLID4': BEGIN
      if event.drag eq 0 then begin
        xmaxt = event.value
        if abs(xmaxt-xmax)/(xmax-xmin) gt tolerance then update_zoomw, 1 
      endif
      END

  'DRAW8': BEGIN
      if event.type eq 0 then begin
;       get only button press
        if event.press eq 1 then begin
;         left button
          x = event.x
          middle=(x-coordmin(0))/coordrange(0)*datarange(0)+datamin(0) > datamin(0)
          Middle = middle < datamax(0)
          widget_control, fslid1, set_value = middle
          update_zoomw, 0
        endif else if event.press eq 4 then plot_zoomw_draw8
      endif
      END

  'DRAW9': BEGIN
      if event.type eq 0 or event.type eq 2 then begin
; 0 is button press event and 2 is motion event
        coord = convert_coord(event.x, event.y, /device, /to_data)/ $
           [xeng_scale, yeng_scale]

        widget_control, wxcur, set_value = coord[0]
        widget_control, wycur, set_value = coord[1]
        if keyword_set(big_cur) then begin
          wset, draw2
          plo, par1, par2, grid = 1, over = ov, legend = -1, header = 0, $
            /xstyle,  xrange = [xmin, xmax],  _extra = extrastuff
          if ncur gt 0 then plots, xcur[1:ncur]*xeng_scale, ycur[1:ncur]*yeng_scale, $
            noclip = 0, psym = 8, col = 5
          oplot, !x.crange, [1,1]*coord[1]*yeng_scale
          oplot, [1,1]*coord[0]*xeng_scale, !y.crange
        endif
        if event.type eq 0 then begin
;         button press
          if event.press eq 1 then begin
;           Left button
            middle = (coord[0] > datamin[0]) < datamax[0]
            widget_control, fslid1, set_value = middle
            update_zoomw, 0
          endif else if event.press eq 4 then begin
            if lcsavepoints eq 0 then wwarning, $
              ['No arguements for XCURSOR and YCURSOR','No DATA will be returned'], $
               offset =[xsize, ysize]+offset_save
            ncur = ncur+1
            if ncur ge maxcur then begin
              maxcur = maxcur + 1024L
              xcur = [xcur , make_array(1024)]
              ycur = [ycur , make_array(1024)]
            endif
            xcur[ncur]= coord[0]
            ycur[ncur]= coord[1]
            wset, draw2
            plots, xcur[ncur]*xeng_scale, ycur[ncur]*yeng_scale, $
              psym = 8, col = 5
            wset, draw1
            oldx = !x
            oldy = !y
            oldp = !p
            !x = bangx
            !y = bangy
            !p = bangp
            plots, xcur[ncur]*xeng_zoom_scale, ycur[ncur]*yeng_zoom_scale, $
              psym = 8, col = 5
            !x = oldx
            !y = oldy
            !p = oldp
            wset, draw2
          endif
        endif
      endif
      END

  'DELETE_PT': BEGIN
        if ncur gt 0 then begin
          ncur = ncur-1
          plot_zoomw_draw8
        endif
      END

  'BUTTONQ': BEGIN
      widget_control, event.top, tlb_get_offset = offset
      offset_save = offset + window_offset
      widget_control, event.top, /destroy
      END
       
   ELSE: Print, 'No Handler for event: ', ev
  ENDCASE
END

pro plot_zoomw_draw8
common plot_grid_zoom, yeng_zoom_scale, xeng_zoom_scale
compile_opt hidden
  on_error, 1
  common plot_grid_last, yeng_scale, xeng_scale

  common zoomw_common
  wset, draw1
  device, copy= [0, 0, xsize, ysize, 0, 0, image]
  !x = bangx
  !y = bangy
  !p = bangp
  plots, [xmin, xmin]*xeng_zoom_scale, [datamin[1], datamax[1]]
  plots, [xmax, xmax]*xeng_zoom_scale, [datamin[1], datamax[1]]
  if ncur gt 0 then  plots, xcur[1:ncur]*xeng_zoom_scale, ycur[1:ncur]*yeng_zoom_scale, $
      psym = 8, col = 5
  wset, draw2
  plo, par1, par2, grid = 1, over = ov, legend = -1, header = 0, $
     /xstyle,  xrange = [xmin, xmax],  _extra = extrastuff
  if ncur gt 0 then plots, xcur[1:ncur]*xeng_scale, ycur[1:ncur]*yeng_scale, $
      noclip = 0, psym = 8, col = 5
  end

pro update_zoomw, mode
 compile_opt hidden
  on_error, 1
  common zoomw_common
  tolerance = 0.02/alog10(maxzoom)
  if mode eq 0 then begin
    delta = datarange(0)/magnif
    xmint = (middle -delta*0.5) > datamin(0)
    xmaxt = xmint+delta
    if xmaxt gt datamax(0) then begin
      xmaxt = datamax(0)
      xmint = xmaxt-delta
    endif
    if abs(xmin-xmint)/(xmax-xmin) gt tolerance or $
         abs(xmaxt-xmax)/(xmax-xmin) gt tolerance then begin
      xmax = xmaxt
      xmin = xmint
      plot_zoomw_draw8
      widget_control, fslid3, set_value = [xmin, datamin(0), xmax]
      widget_control, fslid4, set_value = [xmax, xmin, datamax(0)]
    endif 
  endif else begin
    xmin = xmint > datamin(0)
    xmax = xmaxt < datamax(0)
    if xmax lt xmin + epsilon then begin
      xmax = xmin+epsilon
      if xmax gt  datamax(0) then begin
        xmax = datamax(0)
        xmin = xmax - epsilon
      endif
    endif
    magnif = (datarange(0)/(xmax-xmin)) > 1
    plot_zoomw_draw8
    widget_control, fslid1, set_value = (xmin+xmax)*0.5
    widget_control, fslid2, set_value = $
            alog10(magnif< maxzoom)
    widget_control, fslid3, set_value = [xmin, datamin(0), xmax]
    widget_control, fslid4, set_value = [xmax, xmin, datamax(0)]
  endelse
  end
     
PRO zoomw, par1in, par2in, grid = gridin, over = overin,  $
        Legend = legendin, header = headerin, maxzoom = maxzoomin, $
        xsize = xsizein, ysize = ysizein, zoomrange = zoomrange, $
        xstyle = xstylein,  GROUP=Group, _extra = extrastuffin, $
        limit = limit,  modal = modal, instructions = instructions, $
        instr_offset = instr_offset, xrange = xrange, yrange = yrange,  $
        cursor = mycursor, xcursor = xcursor, ycursor = ycursor, $
        ncursor = ncursor, big_cursor = big_cursor, $
        showdefault = showdefault, setdefault = setdefault, unset = unset 
;
;       scroll = scrollin
on_error, 1
@draw_defaults.com
@com.dat
  common pffcom, initpff, pff_exe
  common zoomw_common
  common zoomw_save_common
  common plot_grid_zoom, yeng_zoom_scale, xeng_zoom_scale
  common plot_grid_last, yeng_scale, xeng_scale

; see if we are initialized
  if n_elements(initwdf) eq 0 then wdfinit

  lcaction = 0
  if n_elements(cursor_save) eq 0 or keyword_set(unset) then begin
    if keyword_set(unset) then lcaction = 1
    cursor_save = 1
    maxzoom_save = 100.0
    big_cursor_save = 0
  endif
  if n_elements(offset_save) eq 0 then offset_save=[10,10]
  if keyword_set(setdefault) then begin
    lcaction = 1
    if n_elements(mycursor) gt 0 then cursor_save = mycursor  
    if n_elements(maxzoomin) gt 0 then maxzoom_save = maxzoomin
    if n_elements(xsizein) gt 0 then dualsize[0] = xsizein > 100
    if n_elements(ysizein) gt 0 then dualsize[1] = ysizein > 100
    if n_elements(big_cursor) gt 0 then big_cursor_save = big_cursor
  endif
    
  if keyword_set(showdefault) then begin
    lcaction = 1
    print, ' ZOOMW Defaults:'
    print, ' Default CURSOR Value  = ' , strtrim (cursor_save, 2)
    print, ' Default BIG_CURSOR Value  = ' , strtrim (big_cursor_save, 2)
    print, ' Default MAXZOOM Value = ' , strtrim (maxzoom_save, 2)
    print, ' Default XSIZE Value   = ' , strtrim (dualsize[0], 2)
    print, ' Default YSIZE Value   = ' , strtrim (dualsize[1], 2)
  endif

  if n_params() eq 0 then begin
    if lcaction eq 0 then $
      print,'% Format:  ZOOMW, WDA, NCurve, ... (same as PLO)'
    return
  endif
;   if modal widget do not send a warning
    lcsavepoints = (arg_present(xcursor) or arg_present(ycursor) or keyword_set(modal)) ? 1 : 0
    if n_elements(mycursor) eq 0 then mycursor = cursor_save
    big_cur =  (n_elements(big_cursor) eq 0) ? big_cursor_save : big_cursor
    if keyword_set(mycursor) then mycur = 1 else mycur = 0
    if n_elements (maxzoomin) eq  0 then maxzoom = maxzoom_save  else $
      maxzoom = float(maxzoomin(0))
    if n_elements(xsizein) eq 0 then xsize = dualsize(0) else $
        xsize = xsizein > 100
    if n_elements(ysizein) eq 0 then ysize = dualsize(1) else $
        ysize = ysizein > 100
    if keyword_set(overin) then ov = 1 else ov = 0
    if ov eq 0 then begin
      if n_elements(par2in) eq 0 then begin
        par2 = 1
        if n_elements (par1in) eq 0 then begin
          print, 'Parameter to ZOOMW must be specified'
          return
        endif
        par1 = par1in
      endif else begin 
        par2 =  par2in
        if n_elements(par1in) gt 0 then par1 = par1in else begin
          print, 'First parameter to ZOOMW must be specified'
          return
        endelse
      endelse
      If (n_elements(par1) gt 1) or (par2 gt 1) then ov = 1
    endif else begin
      if n_elements(par1in) eq 0 then begin
          print, 'Parameter to ZOOMW must be specified'
          return
        endif
      par1 = par1in
      if n_elements(par2in) eq 0 then par2 = 1 else par2 = par2in
    endelse
    if n_elements (xstylein) ne 0 then xsty = xstylein(0) else xsty = 1
    if n_elements (headerin) eq 0 then head = 0 else head= headerin
    if n_elements (extrastuffin) gt 0 then extrastuff = extrastuffin else $
       if n_elements(extrastuff) gt 0 then extrastuff = {header:head} 

  IF N_ELEMENTS(Group) EQ 0 THEN GROUP=0


  MAIN13 = WIDGET_BASE(GROUP_LEADER=Group, $
      COLUMN=1, modal = modal, $
      MAP=1, $
      TITLE='Zoom - WDF Arrays')

  BASE2 = WIDGET_BASE(MAIN13, $
      ROW=1, $
      MAP=1, $
      UVALUE='BASE2')

  BaseL = WIDGET_BASE(Base2, $
      FRAME = 3, $
      XPAD = 20, $
      column=1, $
      MAP=1)

  LABEL38 = WIDGET_LABEL( BASEL, $
      VALUE='Pick Center of Data and Magnification')

  BaseLS = WIDGET_BASE(BaseL, $
      FRAME = 1, $
      ROW=1, $
      MAP=1)

      magnif = 2.0
; no longer used
;     if n_elements(scrollin) eq 0 then scroll = 0.05 else scroll = scrollin
;      if maxzoom gt 100 then scroll = scroll/alog(maxzoom)

  FSLID1 = cw_fslider( BASELS, $
      DRAG=1, /edit, $
      TITLE='Center of ZOOM region', $
      UVALUE='FSLID1',Value = 0.5, $
      MAXIMUM = 1.0, MINIMUM = 0.0)

  FSLID2 = cw_fslider( BASELS, $
      DRAG=1, /edit, $
      TITLE='Log of ZOOM Magnification', $
      UVALUE='FSLID2', $
      VALUE=alog10(magnif), $
      MAXIMUM = alog10(maxzoom), MINIMUM = 0.0)


  Baser = WIDGET_BASE(Base2, $
      COLUMN=1, $
      XPAD = 20, $
      frame = 3, $
      MAP=1)

  LABEL39 = WIDGET_LABEL( BASER, $
      VALUE='Select Minimum and Maximum Plot Limits')

  BaseRS = WIDGET_BASE(BaseR, $
      FRAME = 1, $
      ROW=1, $
      MAP=1)

  FSLID3 = cw_fslider( BASERS, $
      DRAG=1, /edit, $
      TITLE='Minimum Abscissa Value', $
      UVALUE='FSLID3',Value = 0.25, $
      MAXIMUM = 1.0,  MINIMUM = 0.0)

  FSLID4 = cw_fslider( BASERS, $
      DRAG=1, /edit, $
      TITLE='Maximum Abscissa Value', $
      UVALUE='FSLID4',Value = 0.75, $
      MAXIMUM = 1.0,  MINIMUM = 0.0)
  ncur = 0
  if keyword_set(mycursor) then begin
    maxcur = 1024L
    xcur = make_array(maxcur)
    ycur = make_array(maxcur)
    mkcircle
  endif
  base_cur_ins  = WIDGET_BASE(MAIN13, $
      column=1, $
      MAP=1, $
      UVALUE='BASE_CUR_INS', FRAME=2)
  LABEL38 = WIDGET_LABEL( base_cur_ins, $
      VALUE=' Right Window Cursor ')

  basecur = WIDGET_BASE(base_cur_ins, $
      ROW=1, $
      MAP=1, $
      UVALUE='BASECUR')

  LABEL38 = WIDGET_LABEL( basecur, $
      VALUE='Left Button to Center, Right to Save, ' $
      + 'Cursor Location: ')

  wxcur = cw_field(basecur,  /row, Title ='X-Value:', /floating, $
          value = 0.0, /frame)
  wycur = cw_field(basecur,  /row, Title ='Y-Value:', /floating, $
          value = 0.0, /frame)
  if keyword_set(mycursor) then $
    delbut = widget_button (basecur, value ='Delete Point', /frame, $
       UVALUE = 'DELETE_PT')


 
  ButtonQ =  WIDGET_BUTTON( Main13, $
      UVALUE='BUTTONQ', $
      VALUE='           Q  U  I  T           ')

  BASE3 = WIDGET_BASE(MAIN13, $
      ROW=1, $
      MAP=1, $
      UVALUE='BASE3')

  oldwin = !d.window
  oldx_save = !x
  oldy_save = !y
  oldp_save = !p

  DRAW8 = WIDGET_DRAW( BASE3, $
      BUTTON_EVENTS=1, $
      RETAIN=1, $
      UVALUE='DRAW8', $
      XSIZE=xsize, $
      YSIZE=ysize)

  DRAW9 = WIDGET_DRAW( BASE3, $
      RETAIN=1, $
      UVALUE='DRAW9', $
      button_events = mycursor, $
      motion_events = 1, $
      sensitive = motion, $
      XSIZE=xsize, YSIZE=ysize)

  WIDGET_CONTROL, MAIN13, /REALIZE, tlb_set_xoffset= offset_save[0], $
          tlb_set_yoffset= offset_save[1]
  widget_control, main13, tlb_get_offset = offset
  window_offset = offset_save - offset 
  ; Get drawable window index

  WIDGET_CONTROL, DRAW8, GET_VALUE=DRAW1

  ; Get drawable window index

  WIDGET_CONTROL, DRAW9, GET_VALUE=DRAW2
  wset,draw1
  catch, error_status 
    if error_status ne 0 then begin
      print, '% ZOOMW: Error in call to PLO'
      print, '% PLO error: ', !error_state.msg
      widget_control, main13, /destroy
      return
    endif
    window,  /pixmap, /free, xsize = xsize, ysize = ysize
    plo, par1, par2, grid= 1, over = ov, legend = legendin, header = head, $
      xstyle = xsty,  _extra = extrastuff, xrange = xrange, yrange = yrange
  catch, /cancel
  image = !d.window
  bangx = !x
  bangy = !y
  bangp = !p
  wset,draw1
  device, copy = [0, 0, xsize, ysize, 0, 0, image]
  if !x.type eq 0 then begin
    gxmin = !x.crange(0)
    gxmax = !x.crange(1)
  endif else begin
    gxmin = 10^!x.crange(0)
    gxmax = 10^!x.crange(1)
  endelse
  if !y.type eq 0 then begin
    gymin = !y.crange(0)
    gymax = !y.crange(1)
  endif else begin
    gymin = 10^!y.crange(0)
    gymax = 10^!y.crange(1)
  endelse

  coordmin = convert_coord(gxmin, gymin, /data, /to_device)
  coordmax = convert_coord(gxmax, gymax, /data, /to_device)
  coordrange = coordmax-coordmin
  xcrange = [coordmin(0), coordmax(0)]
  ycrange = [coordmin(1), coordmax(1)]
  gxmin = gxmin/xeng_scale
  gxmax = gxmax/xeng_scale
  gymin = gymin/yeng_scale
  gymax = gymax/yeng_scale

  datamax =  [gxmax, gymax]
  datamin =  [gxmin, gymin]
  datarange = datamax - datamin
  if n_elements(zoomrange) eq 2 then begin
    if (zoomrange(0)  ge gxmin and zoomrange(1) le gxmax) then begin
      xmin = zoomrange(0)
      xmax = zoomrange(1)
      middle = (xmin+xmax)/2.0
      magnif = datarange(0)/(xmax-xmin)
    endif else begin
      middle = total(!x.crange)/2.0/xeng_scale
      xmin = middle - datarange(0)/(2.0*magnif)
      xmax = middle + datarange(0)/(2.0*magnif)
    endelse
  endif else begin
    middle = total(!x.crange)/2.0/xeng_scale
    xmin = middle - datarange(0)/(2.0*magnif)
    xmax = middle + datarange(0)/(2.0*magnif)
  endelse
  widget_control, fslid1, set_value=[middle, [gxmin, gxmax]]
  widget_control, fslid2, set_value = $
            alog10(magnif< maxzoom)
  widget_control, fslid3, set_value=[xmin, gxmin, xmax]
  widget_control, fslid4, set_value=[xmax, xmin, gxmax]
  plots, [xmin, xmin]*xeng_scale, [gymin, gymax]*yeng_scale
  plots, [xmax, xmax]*xeng_scale, [gymin, gymax]*yeng_scale
  xmint = xmin
  xmaxt = xmax
  wset, draw2
  epsilon = datarange(0)/maxzoom/3.0
  yeng_zoom_scale = yeng_scale
  xeng_zoom_scale = xeng_scale
  plo, par1, par2, grid = 1, over = ov, legend = -1, header = 0, $
        /xstyle,  xrange = [xmin, xmax],  _extra = extrastuff
  if n_elements(instructions) gt 0 then begin
    if n_elements(instr_offset) eq 0 then instr_offset = 0.5*[xsize, ysize]
    wwinstr = winstruction(instructions, offset =instr_offset)
  endif else wwinstr = -1
  empty
  XMANAGER, 'ZOOMW', MAIN13
  zoomrange = [xmin, xmax]
  if keyword_set (limit) then begin
    if par2 gt 1 then par1 = par1+indgen(par2)
    lim, par1, xmin, xmax
  endif
  if ncur gt 0 then begin
    ncursor = ncur
    xcursor = xcur[1: ncur]
    ycursor = ycur[1: ncur]
  endif else ncursor = 0
  widget_control, wwinstr, bad_id = bad
  if bad eq 0 then widget_control, wwinstr, /destroy
  wdelete, image
  if oldwin ge 0 then begin
      wset, oldwin
      !x = oldx_save
      !y = oldy_save
      !p = oldp_save
  endif
END

