; ------------------------------------------------------------------------ 
; $Id: alignstr.pro,v 1.7 2010/01/06 20:49:49 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
 pro alignstr, ain, save, np = np, minbin = minbin, smooth = smoth, $
          width = width, missing = missing, verbose = verbose, $
          angle = angle, slope = slope, save_only = save_only, $
           _extra = extrastuff
;+
; NAME:
;       ALIGNSTR
;
; PURPOSE:
;       Align a streak to an exact horizontal or vertical position by
;       rotating the image
;       NOTE: Regardless of the spatial dimensions, the structure is assumed
;             to have square pixels.
;       NOTE: The two largest WDF arrays are used as work arrays.
;
; CATEGORY:
;       Graphics
;
; CALLING SEQUENCE:
;
;       ALIGNSTR, STR1 [, STR2] [, ANGLE = angle] [, SLOPE = slope]
;         [, MINBIN = minbin]  [, NP = np] 
;         [, SMOOTH = smooth] [, MISSING = missing] 
;         [, WIDTH = width]  [, SAVE_ONLY = save_only]
;         [, /VERBOSE] [_extra= extrastuff]
;
; where: 
;       STR1   - Input Structure
;       STR2   - Output Structure (Rotated structure)
;                Default: STR2 = STR1
;       ANGLE  - Rotation angle - used in call to rot.
;       SLOPE  - Slope of the alignment streak
;       NP     - Number of points used to obtain alignment.
;                Default = min(total/minbin, 25) > 5 where total is total number
;                          of pixels between the limits. 
;       MINBIN - Minimum size of lineouts used to determine feature.
;                Default = (11 < (total/5))> 1  where total is total number
;                          of pixels between the limits.
;                Actual number will always be odd
;       SMOOTH - Size of Boxcar filter
;                Default = 9
;       WIDTH  - Half Width of the feature used for alignment 
;                Center +/- width is used to align the image
;                Default = 2% = .02 
;       MISSING- Value for data missing in the rotation
;                See POLY_2D for description 
;       VERBOSE- See the calculation of FWHM and other diagnostics
;                If negative, show number of bins, width of scan, and width of each bin only
;       SAVE_ONLY - Do not rotate image.  Save the streak to the wdf defined
;                by SAVE_ONLY.  If SAVE_ONLY is set but does not have a valid 
;                WDF array, then use get_maxwdf()-1
;       _EXTRA - Keyword parameters for plotfld (Xrange, Yrange, etc)
;
;	
; OUTPUTS:
;       STR1 is rotated so that the feature is horizontal or vertical
;         The rotated structure is stored in STR2.  If STR1 is a structure
;         array number then STR2 will be also.  If STR1 is a structure, then 
;         so also will be STR2.
;
;       RotStr is used to rotate the structure
;
; PROCEDURE:
;       1. User is asked to indicate top and bottom or left and right extreme
;          of the alignment feature.  All calculations are performed using 
;          bins.  The image dimensions are ignored.
;       2. NP Lineouts are taken
;       3. Lineouts are smoothed.
;       4. Lineouts are limited to indicated point +/- Width.
;       5. FWHM is calculated using a baseline of the average of first and last
;          Width/2
;       6. The average of the leading and falling half width points are used
;          as the peak center.
;       7. These peak values are used with a least squares fit to determine 
;          the amount of rotation required.
;       8. The image is rotated with ROT
;       9. The image is stored back into the structure.
;       
;          
; EXAMPLE:
;       Rotate structure 1 and save aligned structure in 4
;          alignstr, 1, 4
;
;       Calculate the points on a line feature in structure 1 and store in wdf 5.
;          alignstr, 1, save=5
;
; MODIFICATION HISTORY:
; 	Written by:	L. Paul Mix, June 26, 2000
;                       Added save_only keyword, February 3, 2004.
;-
;
; insert a common block
;
@qscom.dat
;  Common block
@qsstrcom.dat
;       return on an error
 on_error, 1
;
      if n_elements (structinit) eq 0 then qsstruct_init
      if n_elements (qsplotinit) eq 0 then qsinit
;
      nparams = n_params()

      if nparams eq 0 then begin
        print,'% Format: ALIGNSTR, STR1 [, STR2] [, NP = np][, WIDTH = width]'
        print,'             [, SMOOTH = smooth] [, MISSING = missing]'
        print,'             [, ANGLE = angle] [, SLOPE = slope]'
        print,'             [, MINBIN = minbin] [_extra= extrastuff]'
        return
      endif

      a = get_valid_structure(ain, 'ALIGNSTR', lcstr = lcstrin, ierror = ier)
      if ier ne 0 then return
;
      if a.type ne 1 then begin
        print, '% ALIGNSTR: Only type 1 image structures may be rotated'
        return
      endif
;
      if nparams gt 1  and lcstrin eq 0 then save = get_save_str(save, ain) 
;
      if a.ndim gt 2 and a.two eq 0 then begin
        print, '% ALIGNSTR: Image must have two dimensions'
        retur
      endif
      plotfld, a, cn=2, _extra = extrastuff
      print, 'Indicate one end of the feature used for calibration '
      cur, x1, y1
      print, 'Indicate the other end of the feature used for calibration '
      cur, x2, y2
      arr = reform ( get_array(a, -1)) 
      ss = size(arr)
      if ss[0] ne 2 then begin
        print, '% ALIGNSTR: Structure must be two dimensional'
        return
      endif

      x = get_array(a, 1)
      y = get_array(a, 2)
      nx = n_elements(x)
      ny = n_elements(y)
      if nx ne ss[1] or ny ne ss[2] then begin
        print, '% ALIGNSTR: X and Y arrays are not the same size as the image'
        print, nx, ny, ss[1], ss[2]
        return
      endif
      yrange = y[ny-1] -y[0]
      xrange = x[nx-1] -x[0]

;     get the lineouts
      swapit = 0
      if abs(y2-y1)/yrange gt abs(x2-x1)/xrange then begin
        lchoriz = 0 
        if y1 gt y2 then swapit = 1
      endif else begin
        lchoriz = 1
        if x1 gt x2 then swapit = 1
      endelse
      if swapit gt 0 then begin
        t = x1 & x1 = x2 & x2 = t
        t = y1 & y1 = y2 & y2 = t
      endif
      
;     convert limits to bins
      junk = min( abs(x1-x), bx1)
      junk = min( abs(x2-x), bx2)
      junk = min( abs(y1-y), by1)
      junk = min( abs(y2-y), by2)
      x1 = float(bx1)
      x2 = float(bx2)
      y1 = float(by1)
      y2 = float(by2)
      
      
      if n_elements(width) eq 0 then width = 0.02 else width = (width > 0) < 1.
      if width eq 0.0 then width = 0.02
      if n_elements(smoth) eq 0 then smoth = 9
      def_mb = 11
      def_npmn = 5
      def_npmx = 25
      if lchoriz eq 0 then begin
;       Vertical feature
        tbin = by2-by1+1
        if tbin lt 5 then begin
          Print, '% ALIGN_STR: WARNING line has limited pixels:', tbin
          print, '            Alignment feature must be longer'
          return
        endif
        if n_elements(minbin) eq 0 then minbin = (def_mb < tbin/def_npmn) > 1 $
          else minbin = minbin > 1
        if n_elements(np) eq 0 then np = round(min([tbin/minbin, 25]))> 5
        bins = round (tbin/float(np)) +1
;       get an odd number of bins to make finding the center y-value easy
        if bins mod 2 eq 0 then bins= bins-1
        binsm = bins-1
        while (ny le (by1+bins*np)) do $
          if bins eq 1 then np = np-1 else bins = bins-2
        if np lt 3 then begin
          print, '% ALIGNSTR: Inadequate data; Bins, np:', bins, np
          print, '            Alignment feature must be longer'
          return
        endif
        data = make_array(nx, np)
        for i = 0, np-1 do data[*, i] = $
          smooth(total(arr[*, (by1+i*bins):(by1+i*bins+binsm)], 2, /double), smoth)
        xx = findgen(np)*bins +(by1+bins/2+1)
        xmid = make_array(np)
        rat = (x2-x1)/(y2-y1)
        for i = 0, np-1 do xmid[i] = (xx[i]-y1)*rat+x1
        wid = width*nx
        widmin = (xmid-wid) > 0
        widmax = (xmid+wid) < (nx-1)
        x = findgen(nx)
      endif else begin
;       Horizontal feature
        tbin = bx2-bx1+1
        if tbin lt 5 then begin
          Print, '% ALIGN_STR: WARNING line has limited pixels:', tbin
          print, '            Alignment feature must be longer'
          return
        endif
        if n_elements(minbin) eq 0 then minbin = (def_mb < tbin/def_npmn) > 1 $ 
          else minbin = minbin > 1
        if n_elements(np) eq 0 then np = round(min([tbin/minbin, 25]))> 5
        bins = round (tbin/float(np)) +1
;       get an odd number of bins to make finding the center y-value easy
        if bins mod 2 eq 0 then bins= bins-1
        binsm = bins-1
        while (nx le (bx1+bins*np)) do $
          if bins eq 1 then np = np-1 else bins = bins-2
        if np lt 3 then begin
          print, '% ALIGNSTR: Inadequate data; Bins, np:', bins, np
          print, '            Alignment feature must be longer'
          return
        endif
        data = make_array(ny, np)
        for i = 0, np-1 do data[*, i] = $
          smooth(total(arr[(bx1+i*bins):(bx1+i*bins+binsm), *], 1, /double), smoth)
        xx = findgen(np)*bins +(bx1+bins/2+1)
        xmid = make_array(np)
        rat = (y2-y1)/(x2-x1)
        for i = 0, np-1 do xmid[i] = (xx[i]-x1)*rat+y1
        wid = width*ny
        widmin = (xmid-wid) > 0
        widmax = (xmid+wid) < (ny-1)
        x = findgen(ny) 
      endelse
      if keyword_set(verbose) && verbose lt 0 then begin
        Print, 'Number of bins - ', strtrim(np, 2)
        print, 'Scan Width - ', strtrim(wid, 2)
        print, 'BIN Width - ', strtrim(bins, 2)
        verbose = 0
      endif
      if keyword_set(verbose) then quiet = 0 else quiet = 1
      yy = xx
      wdf = get_maxwdf()
      wid2 = wid*0.5
      ii = 0
      for i = 0, np-1 do begin
        put_wdfarray, [0.0, 1.0, data[*,i]], wdf
        lim, wdf, widmin[i], widmax[i]
        junk = fwhm(wdf, base=(ave(wdf, right =widmin[i]+wid/2, /Quiet) $
          +ave(wdf, left =widmax[i]-wid/2, /quiet))*0.5, $
          first = ff, second = ss, quiet = quiet)
        if junk gt 0 then begin
          yy[ii]=(ff+ss)*0.5
          xx[ii]= xx[i]
          ii = ii+1
        endif
        if keyword_set (verbose) then begin
          print, ' Width, Middle:',junk, yy[(ii-1)>0], ii-1
          if junk le 0 then junk = 0 else junk = 1
          hak, letter,mes = 'HIT any key to accept, Hit N or n to reject'
          letter = (byte(letter))[0]
          if ((letter eq 78B) or (letter eq 110B)) then ii = ii -junk
        endif
      endfor
      if ii lt 3 then begin
        print, '% Alignstr: Cannot determine slope with ', strtrim(ii, 2), $
          ' points found.'
        return
      endif
      if ii lt np then begin
        yy = yy[0:ii-1]
        xx = xx[0:ii-1]
      endif
      i2w, xx, yy, /xy, wdf, clabel='Feature Trajectory - AlignStr'
      wdfit, wdf, wdf-1, coef = coef
      plo, wdf-1, 2, /ov, /col, /nl, /ns
      slope = coef[1]
      angle = atan(coef[1])/!dtor
      if lchoriz eq 0 then angle = -angle
     
      if keyword_set(save_only) then begin
        save_only = round(save_only[0])
        if save_only lt 1 or save_only gt wdf then save_only = wdf-1
        xfr, wdf, save_only
        print, ' Feature parameters stored in ', strtrim(save_only, 2)
        print, ' Image would have been rotated ', strtrim(angle, 2) , ' Degrees'
      endif else begin
        arr = rot(arr, angle, /interp)
        lab_add =  ' Image rotated ' + strtrim(angle, 2) + ' Degrees'
        put_array, a, arr, -1
        a.clab = a.clab+' -' + lab_add
        print, lab_add
        put_array, a, arr, -1
      endelse
      if nparams eq 1 then begin
        if lcstrin eq 0 then i2s, a, ain, /no_copy else $
          ain  = temporary(a)
      endif else begin
        if lcstrin eq 0 then i2s, a, save, /no_copy else save = temporary(a)
      endelse
END
