; ------------------------------------------------------------------------ 
; $Id: arr2str.pro,v 1.10 2010/08/09 16:49:17 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
 Pro  arr2str, str, image_in, image2_in, image3_in, femesh = femesh, $
        x_in, y_in, z_in, t_in, u_in, v_in, clabel = clabel, xlabel = xlabel, $
        ylabel = ylabel, zlabel = zlabel, x4label = x4label, x5label = x5label, $
        x6label = x6label, tlabel = tlabel, file = file, plotit = plotit, grid = grid, $
        v1label = v1label, v2label = v2label, v3label = v3label, $
        dx = dxin, dy = dyin, dz = dzin, dt = dtin, du = duin, dv = dvin, $
        connectivity = tri, structure_only = structure_only, spare = spare
;+
;   Convert a two, three, or four dimensional array to a field type structure array
;     with one or more vector components
;   Five and Six dimensional data may also be written but this data is not generally
;     supported.
;   Single block data only, but JOINSTR may be used to combine into a single str.
;   Format:
;     ARR2STR, STR, IMAGE, image2, image3, femesh = femesh, plotit = plotit,  
;       x, y, z, t, u, v, dx = dx, dy = dy, dz = dz, dt = dt, du = du, dv = dv, 
;       /structure_only, tlabel = tlabel, clabel = clabel, grid = grid, file = file,
;       xlabel = xlabel,  ylabel=ylabel, zlabel = zlabel, x4label = x4label, 
;       x5label = x5label,  x6label = x6label,
;       v1label = v1label, v2label = v2label, v3label = v3label, 
;       connectivity = connectivity, spare = spare
;
;     where: (REQUIRED)
;          STR    - Structure array number where data is stored
;                   If structure_only is set, STR is a structure generated
;                   from the input data
;          IMAGE  - 2D, 3D, or 4D array (5D and 6D arrays may also be used)
;                   Note:  Use JOINSTR for multiple block data.
;                   Note:  5 and 6 dimensional arrays are for special applications
; (OPTIONAL)
;          image2, image3 - Additional images for a Vector type field structure
;            Note: Image, image2 and image3 must all be the same size
;          x      - Vector of x values, must be as big as first dimension
;          y      - Vector of y values, must be as big as second dimension
;          z      - Vector of z values, must be as big as third dimension
;                   If nx*ny*nz eq nI and (size(image))[0] eq 2, a three dimensional 
;                   structure will be generated with a degenerate dimension, 
;                   where ni = n_elements(image), nx = n_elements(x),
;                     ny = n_elements(y), and nz = n_elements(z).
;          t      - Vector of t values, must be as big as fourth dimension
;          u      - Vector of u values, must be as big as fifth dimension
;          v      - Vector of v values, must be as big as sixth dimension
;      (Note:  The following values will be calculated if not present.)
;          dx     - Spacing of X values
;          dy     - Spacing of Y values
;          dz     - Spacing of Z values
;          dt     - Spacing of T values
;          du     - Spacing of U values
;          dv     - Spacing of V values
; Keywords:
;          GRID   - If set, data is a grid dataset.
;                   Number of dimensions must be 2 or 3. 
;                   Current implementation does not allow FE meshes.
;                   IMAGE is the X array
;                   IMAGE2 is the Y array
;                   IMAGE3 is the Z array if 3 dimensional data.
;                   for example: arr2str, 4, x, y, z , /grid
;                     will return a type 3 dataset with x, y, and z as the 
;                     grid values.
;          FEMesh - If set, Data is from a finite element type, random mesh.
;                   Default: FEMesh = 0; rectilinear mesh.
;                   If set, X, Y are required and only 1 image is allowed.
;                   Image, x, and y are linear arrays.
;              Format: arr2str, /fem, str, image, x, y, etc.
;              NOTE: See CONNECTIVITY
;          CONNECTIVITY: Connectivity of the finite element mesh.  This is 
;                        ignored for rectilinear data.
;                        If Connectivity is not specified, TRIANGULATE will be used.
;          PLOTIT - Ignored unless FEMesh is set. Plots the triangles.
;          clabel - Comment Label
;          tlabel - Type Label
;          xlabel - X-Label
;          ylabel - Y-Label
;          zlabel - Z-Label
;          x4label- T-Label
;          x5label- U-Label
;          x6label- V-Label
;          v1label- Vector 1 label
;          v2label- Vector 2 label
;          v3label- Vector 3 label
;          spare  - spare keyword
;                   Default = [0,0,0,0,0]
;          file   - Associated file 
;          Structure_Only - If set STR is returned as a structure.
;          
;      Example:
;          Make a structure array from array, picture, and put it into 4 
;            arr2str, 4, picture, clab = 'filtered image'
;
;          Make a structure array from arrays, vx, vy and vz with spatial
;            coordinates, xval, yval, zval and store into structure 18
;            ARR2STR, 18, vx, vy, vz, xval, yval, zval, $
;               clab = 'Particle velocity'
;
;-
; insert a common block
;
@qsstrcom.dat
;
      on_error, 1
;
      if n_elements (structinit) eq 0 then qsstruct_init
;
      nparams = n_params()
      if nparams eq 0 then begin
        print,'% Format: ', $
       'ARR2STR, STR, IMAGE [, IMAGE2, IMAGE3] [, X, Y, Z, T, U, V] [, /FEMesh ]'
        print, '    [, CLABEL = clabel] [, TLABEL = tlabel] [, XLABEL= xlabel]'
        print, '    [, YLABEL = ylabel] [, ZLABEL= zlabel] [, X4LABEL= x4label]'
        print, '    [, X5LABEL = x5label] [, X6LABEL= x6label]'
        print, '    [, V1LABEL = v1label] [, V2LABEL= v2label]'
        print, '    [, V3LABEL = v3label] [, FILE= file] [, /GRID]'
        print, '    [, DX = dx, DY = dy, DZ = dz, DT = dt, DU = du, DV = dv] '
        print, '    [, /STRUCTRUE_ONLY]'
        return
      endif
;
      if  not keyword_set (structure_only) then begin
        if n_elements(str) eq 0 then begin
          print,'% ARR2STR: Structure number must be defined'
          return
        endif
        if str lt 1 or str gt maxstructure then begin
          print, '% ARR2STR: Structure array must be greater than 0 and </='+ $
            strtrim(maxstructure, 2)
            return
        endif
      endif
;
      if n_elements (spare) eq 0 then ispare = [0,0,0,0,0] else ispare = spare

      if keyword_set (grid) then begin
        if nparams lt 3 then begin
          print, $
          '% ARR2STR: Procedure must have at least 3 parameters for grid data.'
          return
        endif
        nx =  n_elements(image_in)
        ny =  n_elements(image2_in)
        nz =  n_elements(image3_in)
        if nx eq 0 or ny eq 0 then begin
          print, '% ARR2STR: X and Y grid arrays must be defined.'
          return
        endif
        ndim = 3
        mxbl = 1
        two = 0
        ok = where([nx, ny, nz] lt 2, nok)
        if nok gt 1 then begin
          print, '% ARR2STR: Two arrays must have valid data.'
          return
        endif else if nok gt 0 then two = 1+ok[0]
        if nz eq 0 then x3 = 0.0 else x3 = float(image3_in)
        x2=float(image2_in)
        x1=float(image_in)
        ssize = [nx, ny, nz]
        locx = [0,0,0,ssize]
        locx = reform (locx, 3, 2, /overwrite)
        x1lab = make_array(mxbl, /string, value = ' ')
        x2lab = make_array(mxbl, /string, value = ' ')
        x3lab = make_array(mxbl, /string, value = ' ')
        range = make_array(2, ndim, mxbl+1)
        range[0,0,*] = min(x1, max = mx)
        range[1,0,*] = mx
        range[0,1,*] = min(x2, max = mx)
        range[1,1,*] = mx
        range[0,2,*] = min(x3, max = mx)
        range[1,2,*] = mx
        if n_elements(xlabel) gt 0 then x1lab[0]=xlabel[0]
        if n_elements(ylabel) gt 0 then x2lab[0]=ylabel[0]
        if n_elements(zlabel) gt 0 then x3lab[0]=zlabel[0]
        if n_elements(clabel) gt 0 then clabel= clabel[0] else $
          clabel = 'Grid data'
        if n_elements(tlabel) gt 0 then tlabel= tlabel[0] else $
          tlabel = 'Grid data'
        if n_elements(file) gt 0 then file = file[0] else file = ''
        array ={type:3L, ndim:ndim,  block :mxbl, two:two, range:range, $
              size:ssize, locx : locx, x1:x1, x2:x2, x3:x3, $
              x1lab : x1lab, x2lab :x2lab, x3lab : x3lab, $
              file : file, spare :ispare, tlab : tlabel, clab: clabel}
        if keyword_set(structure_only) then str= temporary(array) else $
          i2s, array, str, /no_copy
        return
      endif

      if nparams lt 2 then begin
        print,'% ARR2STR: Procedure must have at least 2 parameters'
        return
      endif
;
;
      if n_elements(image_in) eq 0 then begin
        print, '% ARR2STR: Image input must be defined'
        return
      endif
;
    if not keyword_set (femesh) then begin
      b = size(image_in)
      image = image_in
      ndim = b(0)
      ntot = b(ndim+2)
      if ndim lt 2 or ndim gt 6 then begin
          Print,'% ARR2STR: Limited to 2D, 3D, 4D, 5D and 6D arrays.'
          return
      endif
;
      b2 = size(image2_in)
      if b2(0) lt 2  then begin
        if n_elements(image2_in) gt 0 then x = image2_in
        if n_elements(image3_in) gt 0 then y = image3_in
        if n_elements(x_in) gt 0 then z = (x_in)
        if n_elements(y_in) gt 0 then t = (y_in)
        if n_elements(z_in) gt 0 then u = (z_in)
        if n_elements(t_in) gt 0 then v = (t_in)
        nvect = 1L
        im2 = 0.0
        im3 = 0.0
      endif else begin
        if b2(0) ne ndim then begin
          print, '%ARR2STR: IMAGE2 must be the same size as IMAGE',ndim, b2(0)
          return
        endif
        if ntot ne b2(b2(0)+2) then begin
          print, $
          '%ARR2STR: IMAGE2 must be the same size as IMAGE',ntot,b2(b2(0)+2)
          return
        endif
        im2 = image2_in[0:ntot-1]
        b2 = size(image3_in)
        if b2(0) lt 2 then begin
          if n_elements(image3_in) gt 0 then x = image3_in
          if n_elements(x_in) gt 0 then y = x_in
          if n_elements(y_in) gt 0 then z = y_in
          if n_elements(z_in) gt 0 then t = z_in
          if n_elements(t_in) gt 0 then u = t_in
          if n_elements(u_in) gt 0 then v = u_in
          nvect = 2L
          im3 = 0.0
        endif else begin
          if b2(0) ne ndim then begin
            print, '%ARR2STR: IMAGE3 must be the same size as IMAGE',ndim, b2(0)
            return
          endif
          if ntot ne b2(b2(0)+2) then begin
            print, $
            '%ARR2STR: IMAGE3 must be the same size as IMAGE',ntot,b2(b2(0)+2)
            return
          endif
          im3 = image3_in[0:ntot-1]
          nvect = 3L
          if n_elements(x_in) gt 0 then x = x_in
          if n_elements(y_in) gt 0 then y = y_in
          if n_elements(z_in) gt 0 then z = z_in
          if n_elements(t_in) gt 0 then t = t_in
          if n_elements(u_in) gt 0 then u = u_in
          if n_elements(v_in) gt 0 then v = v_in
        endelse
      endelse
      nx = n_elements (x)
      ny = n_elements (y)
      nz = n_elements (z)
      if ndim eq 2 && nx*ny*nz eq ntot then begin
        ndim = 3L
        b = [ndim, nx, ny, nz, 4, ntot]
      endif
      nt = n_elements (t)
      nu = n_elements (u)
      nv = n_elements (v)
      if n_elements (dxin) eq 0 then dxin =0
      if n_elements (dyin) eq 0 then dyin =0
      if n_elements (dzin) eq 0 then dzin =0
      if n_elements (dtin) eq 0 then dtin =0
      if n_elements (duin) eq 0 then duin =0
      if n_elements (dvin) eq 0 then dvin =0
      dx1 = 1.0
      dx2 = 1.0
      dx3 = 1.0
      dx4 = 1.0
      dx5 = 1.0
      dx6 = 1.0
      if nx lt b(1) then xarray= findgen(b(1)) else begin
         xarray = float(x(0:b(1)-1))
         if dxin gt 0 then dx1 = dxin else check_uniform, xarray, dx1
      endelse
      
      if ny lt b(2) then yarray= findgen(b(2)) else begin
         yarray = float(y(0:b(2)-1))
         if dyin gt 0 then dx2 = dyin else check_uniform, yarray, dx2
      endelse
      if ndim gt 2 then begin
        if nz lt b(3) then zarray= findgen(b(3)) else begin
          zarray = float(z(0:b(3)-1))
          if dzin gt 0 then dx3 = dzin else check_uniform, zarray, dx3
        endelse
      endif else zarray = 0.0
      if ndim gt 3 then begin
        if nt lt b(4) then tarray= findgen(b(4)) else begin
          tarray = float(t(0:b(4)-1))
          if dtin gt 0 then dx4 = dtin else check_uniform, tarray, dx4
        endelse
      endif else tarray = 0.0
      if ndim gt 4 then begin
        if nu lt b(5) then uarray= findgen(b(5)) else begin
          uarray = float(u(0:b(5)-1))
          if duin gt 0 then dx5 = duin else check_uniform, uarray, dx5
        endelse
      endif else uarray = 0.0
      if ndim gt 5 then begin
        if nv lt b(6) then varray= findgen(b(6)) else begin
          varray = float(v(0:b(6)-1))
          if dvin gt 0 then dx6 = dvin else check_uniform, varray, dx6
        endelse
      endif else varray = 0.0

      locx = make_array(ndim, 2, /long, value = 0L)
      locx[*, 1] = b[1:ndim]
      range = make_array(2, ndim, 2)
      range(0, 0, *) = min(xarray, max = mx)
      range(1, 0, *) = mx
      range(0, 1, *) = min(yarray, max = mx)
      range(1, 1, *) = mx
      if ndim gt 2 then begin
        range(0, 2, *) = min(zarray, max = mx)
        range(1, 2, *) = mx
        if ndim gt 3 then begin
          range(0, 3, *) = min(tarray, max = mx)
          range(1, 3, *) = mx
          if ndim gt 4 then begin
            range(0, 4, *) = min(uarray, max = mx)
            range(1, 4, *) = mx
            if ndim gt 5 then begin
              range(0, 5, *) = min(varray, max = mx)
              range(1, 5, *) = mx
            endif
          endif
        endif
      endif
      if ndim eq 2 then two = 3L else begin
        if b(1) eq 1 then two = 1L else $
          if b(2) eq 1 then two = 2L else $
            if b(3) eq 1 then two = 3L else two = 0L
      endelse

      if n_elements(file) eq 0 then file = '' else file = file(0)
      if n_elements(tlabel) eq 0 then tlab = 'Array Str' else $
        tlab = tlabel(0)
      if n_elements(clabel) eq 0 then clab = 'Array Structure' else $
        clab = clabel(0)
      if n_elements(xlabel) eq 0 then xlab='X Axis' else xlab=xlabel(0)
      if n_elements(ylabel) eq 0 then ylab='Y Axis' else ylab=ylabel(0)
      if ndim ge 3 then begin
        if n_elements(zlabel) eq 0 then zlab='Z Axis' else zlab=zlabel(0)
      endif else zlab = ''
      if ndim ge 4 then begin
        if n_elements(x4label) eq 0 then x4lab='T Axis' else x4lab=x4label(0)
      endif else x4lab = ''
      if ndim ge 5 then begin
        if n_elements(x5label) eq 0 then x5lab='U Axis' else x5lab=x5label(0)
      endif else x5lab = ''
      if ndim ge 6 then begin
        if n_elements(x6label) eq 0 then x6lab='V Axis' else x6lab=x6label(0)
      endif else x6lab = ''
      ssize = b[1:ndim]
      if n_elements(v1label) gt 0 then v1lab=strtrim(v1label(0), 2) $
        else v1lab = ''
      if n_elements(v2label) gt 0 then v2lab=strtrim(v2label(0), 2) $
        else v2lab = ''
      if n_elements(v3label) gt 0 then v3lab=strtrim(v3label(0), 2) $
        else v3lab = ''
      if two eq 1 then slice = xarray(0) else $
        if two eq 2 then slice = yarray(0) else $
          if two eq 3 then slice = zarray(0) else slice = 0.0
      
      if ndim le 4 then $
      tempstr = {type:1L, ndim:ndim, nvect: nvect, block :1L, two: two, $
              size:ssize,range: range, order: 1L, slice:slice , locx:locx, $
              locv: [0L, ntot], x1:xarray, x2:yarray, x3:zarray, x4:tarray, $
              dx1:dx1, dx2:dx2, dx3:dx3, dx4:dx4, $
              v1:image[0:ntot-1], v2:im2, v3:im3, v4:0.0, v5:0.0, v6:0.0, $
              x1lab : xlab, x2lab : ylab, x3lab : zlab, x4lab : x4lab, $
              v1lab : v1lab, v2lab :v2lab, v3lab : v3lab, $
              v4lab : '',v5lab : '', v6lab : '', $
              file : file, spare : ispare, tlab : tlab, clab: clab } $

      else tempstr = {type:1L, ndim:ndim, nvect: nvect, block :1L, two: two, $
              size:ssize,range: range, order: 1L, slice:slice , locx:locx, $
              locv: [0L, ntot], x1:xarray, x2:yarray, x3:zarray, x4:tarray, $
              x5:uarray, x6:varray, dx1:dx1, dx2:dx2, dx3:dx3, dx4:dx4, $
              dx5:dx5, dx6:dx6,  $
              v1:image[0:ntot-1], v2:im2, v3:im3, v4:0.0, v5:0.0, v6:0.0, $
              x1lab : xlab, x2lab : ylab, x3lab : zlab, x4lab : x4lab, $
              x5lab : x5lab,  x6lab : x6lab, $
              v1lab : v1lab, v2lab :v2lab, v3lab : v3lab, $
              v4lab : '',v5lab : '', v6lab : '', $
              file : file, spare : ispare, tlab : tlab, clab: clab }

      if keyword_set(structure_only) then str= temporary(tempstr) else $
        i2s, tempstr, str, /no_copy
      image = 0
      if nvect gt 1 then begin
        im2 = 0 & im3=0
      endif
      return
    endif
;   Finite element mesh
      image = reform(image_in)
      b = size(image)
      ndim = b(0)
      ntot = b(ndim+2)
      if ndim gt 2 then begin
        Print,'% ARR2STR: FEMesh requires a linear or 2D array '
        return
      endif
;
      if (n_elements(image2_in) gt 0)and (n_elements(image3_in) gt 0) then begin
        xarray = reform(image2_in)
        yarray = reform(image3_in)
      endif else begin
        Print,'% ARR2STR: FEMesh requires X and Y to be defined'
        print,'Format:  ARR2STR, STR, Value, X, Y [, CLABEL = clab] ...'
        return
      endelse
      nvect = 1L
      ndim = 2L
      two = 3L
      zarray = 0.0
      nx = n_elements (xarray)
      ny = n_elements (yarray)
      nz = n_elements (zarray)
      if (nx ne ntot) or (ny ne ntot) then begin
        Print,'% ARR2STR: FEMesh requires X and Y to be the same size as image'
        return
      endif

      locx = make_array(ndim, 2, /long)
      locx(*, 1)= ntot
      range = make_array(2, ndim, 2)
      range(0, 0, *) = min(xarray, max = mx)
      range(1, 0, *) = mx
      range(0, 1, *) = min(yarray, max = mx)
      range(1, 1, *) = mx
      if n_elements(file) eq 0 then file = '' else file = file(0)
      if n_elements(tlabel) eq 0 then tlab = 'FE Arr Str' else $
        tlab = tlabel(0)
      if n_elements(clabel) eq 0 then clab = 'FE Array Structure' else $
        clab = clabel(0)
      if n_elements(xlabel) eq 0 then xlab='X Axis' else xlab=xlabel(0)
      if n_elements(ylabel) eq 0 then ylab='Y Axis' else ylab=ylabel(0)
      zlab = ''
      ssize = [ntot, ntot]
      if n_elements(v1label) gt 0 then v1lab=strtrim(v1label(0), 2) $
        else v1lab = ''
      v2lab = ''
      v3lab = ''
      slice = 0.0
      
;     triangulate the data
      if n_elements(tri) lt ndim+1 then begin
        xbar = total(xarray)/n_elements(xarray)
        dxi = 1.0/(max(xarray, min= mm)-mm)
        ybar = total(yarray)/n_elements(yarray)
        dyi = 1.0/(max(yarray, min= mm)-mm)
        triangulate, (xarray-xbar)*dxi, dyi*(yarray-ybar), tri
      endif
      loctri = [0, (size(tri))[2]]
      if keyword_set (plotit) then begin
        plot, xarray, yarray, /nodata
        for i = 0, loctri[1]-1 do begin
          ii = [tri[*, i], tri[0, i]]
          oplot,xarray[ii], yarray[ii]
        endfor
        hak,/mes
      endif
      tempstr = {type: 8L, ndim:2L, nvect: 1L, block:1L, two:3L, $
             ntime:1L,  slice:slice, size: ssize, range: range, order: 0L, $
            locx:locx, locv:[0L, ntot],triangles:tri, loctri:loctri, $
            x1:xarray, x2:yarray, x3:0.0, x4:0.0, v1:image, v2:0.0, v3:0.0, $
            v4:0.0, $
            v5:0.0, v6:0.0, x1lab : xlab, x2lab : ylab, x3lab : '', $
            x4lab : '', v1lab : v1lab , v2lab : '', $
            v3lab : '', v4lab : '',v5lab : '', v6lab : '', $
            file : file, spare : ispare, tlab : tlab, clab:clab}

      if keyword_set(structure_only) then str= temporary(tempstr) else $
        i2s, tempstr, str, /no_copy

      image = 0
      return
      end
