; ------------------------------------------------------------------------ 
; $Id: border.pro,v 1.3 2007/06/06 20:09:33 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
 Pro border, str, side, wda, top = top, plot = plot
;
;+
;    This routine calculates the values along the minimum or maximum dimension
;      of a structure array.  The structure array must be two dimensional or be
;      three dimensional with a degenerate dimension and a single attribute.
;
;    Format:  BORDER, STR, SIDE, WDA, TOP = top, PLOT = plot
;      where:
;         STR    - Field structure or a QS structure array number
;         SIDE   - Indicates the coordinate for which the minimum or maximum 
;                  is desired
;         WDA    - Waveform array to store the values
;         TOP    - If present and not zero, the values of the structure at the 
;                  maximum value of SIDE will be returned.
;        PLOT    - If present and not zero, a plot of the values will be made
;    Example:
;      Store in WDF array 34, the value at minimum r for structure 2
;      Average the second spatial dimension between -1 and 10.
;      Use the second component of the field.
;         tempstr = slice(2, 2, 2, [-1, 10])   )
;         border, tempstr, 1, 34
;-
@qsstrcom.dat
;       return on an error
      on_error, 1
;
      if n_elements (structinit) eq 0 then qsstruct_init
;
;     now get the data  -- check that we have a structure of the proper type
;
      if n_elements(str) eq 0 then begin
        print, 'Structure must be defined'
        return
      endif
;
      if n_elements(wda) eq 0 then begin
        print, 'WDF array must be defined'
        return
      endif
;
      if n_elements(side) eq 0 then begin
        print, $
          ' Coordinate for which a minimum value is desired must be specified'
        return
      endif
;
; change variables to insure we don't corrupt input data
      a = str
      b = size(a)
      if b(b(0)+1) ne 8 then begin
        if a gt 0 and a le maxstructure then begin
          s2i, a, a
          b = size (a)
          if b(b(0)+1) ne 8 then begin
            print, $
   ' First parameter must be a structure or a structure array number'
            return
          endif
        endif  else begin
          print, $
   ' First parameter must be a structure or a valid structure array number'
          return
        endelse
      endif
      if a.type ne 1 then begin
        print, ' Borders may be calculated only with FIELD type structrues'
        return
      endif
      if a.ndim eq 3 then begin
        if side lt 1 or side gt 3 then begin
          Print, ' SIDE must be in the range [1, 3]'
         return
        endif
        if side eq a.two then begin
          print, ' SIDE must not equal the degenerate dimension'
          return
        endif
      endif else begin
        if side lt 1 or side gt 2 then begin
          Print, ' SIDE must be in the range [1, 2]'
          return
        endif
      endelse
      if a.type ne 1 then begin
        print, ' BORDER requires field type structures'
        return
      endif
      if a.two eq 0 and a.ndim gt 2 then begin
        print, ' Please use SLICE to reduce the structure to two dimensions'
        return
      endif
      If (a.nvect gt 1) then begin
        print,' Please use SLICE to reduce the number of vector components to 1'
        return
      endif
      if n_elements (top) eq 0 then top = 0
      if n_elements(a.order) le 1 then blockarray = 1 else begin
        if top eq 0 then blockarray = a.order(side-1, *) +1 else $
          blockarray = 1+reverse(sort(a.range(1, side-1, 1:a.block)))
      endelse
      if a.ndim eq 2 then begin
        xval = 3-side 
        yval = xval
      endif else begin
        xval = 6-a.two-side
        if xval eq 1 then yval =1 else $
          if xval eq 3 then yval = 2 else begin
          if a.two eq 3 then yval = 2 else yval = 1
        endelse
      endelse
      nb = n_elements(blockarray)
      tempx = make_array(1024, nb)
      tempy = make_array(1024, nb)
;
      for ii = 0, nb-1 do begin
        iblock = blockarray(ii)
        xx = get_array (a, xval, iblock)
        cont = reform( get_array (a, -1, iblock))
        if top eq 0 then begin
          if yval eq 1 then yy = cont(*, 0) else yy = cont(0,*)
        endif else begin
          b = size(cont)
          if b(0) ne 2 then begin
            print, 'Attribute array must be two dimensional', ii, b
            return
          endif
          if yval eq 1 then yy = cont(*, b(2)-1) else yy = cont(b(1)-1,*)
        endelse
        yy= reform (yy)
        nx =  n_elements (xx)
        ny =  n_elements (yy)
        if ny ne nx then begin
          print, 'ERROR - XX and YY arrays have different number of elements'
          return
        endif
        deriv = xx(1:nx-1)-xx(0:nx-2)
        mxderv = max(deriv, min = mnderv)
        if mnderv lt 0.0 then print, ' Caution: axis array is not increasing'
        if ii gt 0 then begin
          for jj = 0, ii-1 do begin
;           look for regions of overlap.        
            if nx gt 0 then begin
              aa = where( xx lt tempx(1, jj), naa)
              bb = where( xx gt tempx(round(tempx(0, jj)), jj), nbb)
              cc = 0
;             do not take a single point
              if naa gt 1 then cc = [cc, aa]
              if nbb gt 1 then cc = [cc, bb]
              nx =  n_elements (cc) -1
              if nx gt 1 then begin
                cc = cc(1:*)
                xx = xx(cc)
                yy = yy(cc)
              endif
            endif
          endfor
        endif
        if nx gt 1 then begin
          deriv = xx(1:nx-1)-xx(0:nx-2)
          maxderv = max(deriv, min=minderiv)
          if maxderv gt mxderv then begin
            br = where (deriv gt mxderv, nbreak)
            for i = 0, nbreak-1 do begin
              if nx gt 1 then begin
                if br(i) eq 0 then begin
                  nx = nx-1
                  xx = xx(1:*)
                  yy = yy(1:*)
                  br = br-1
                endif else if br(i) eq nx-2 then begin
                  nx = nx-1
                  xx = xx(0:nx-1)
                  yy = yy(0:nx-1)
                  br = br-1
                endif else if i lt nbreak-1 then begin
                  if br(i+1) eq br(i)+1  then begin
                    nx = nx-1
                    xx =[xx (0:br(i)), xx(br(i)+2:*)]
                    yy =[yy (0:br(i)), yy(br(i)+2:*)]
                    br = br-1
                  endif
                endif
              endif
            endfor
          endif
          if nx gt 1 then begin
            if minderiv lt 0 then print, ' Neg. Delta- Possible error in BORDER'
            delx = 0.0005*abs(minderiv)
            xx(0) = xx(0)+ delx
            xx(nx-1) = xx(nx-1)- delx
            tempy(0:nx, ii) = [nx, yy]
            tempx(0:nx, ii) = [nx, xx]
          endif
        endif
      endfor
      xx = 0.0
      yy = 0.0
      np= 0L
      for ii = 0, nb-1 do begin
        nx = round(tempx(0, ii))
        if nx gt 0 then begin
          xx = [xx, tempx(1:nx,ii)]
          yy = [yy, tempy(1:nx,ii)]
          np = np+nx
        endif
      endfor
      if np eq 0 then begin
        Print, ' ERROR in BORDER, No border values were found.'
        return
      endif
      xx = xx(1:np)
      yy = yy(1:np)
      ok = sort(xx)
      xx = xx(ok)
      yy = yy(ok)
      delx = abs(xx(np-1)-xx(0))* 0.50e-4
      xx = [xx(0) -delx , xx, xx(np-1)+delx]
      yy = [yy(0), yy, yy(np-1)]  
      if keyword_set(plot)  then $
         plot, xx, yy, psym =-4, title = $
        'Border values in the'+string(side)+ 'dimension of'+a.clab
      i2w, xx, yy, wda, /xydata, $
       c= 'Border values in the'+string(side)+ 'dimension of'+a.clab
      return
      end
