; ------------------------------------------------------------------------ 
; $Id: cond_range.pro,v 1.3 2007/06/06 20:09:33 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
 pro cond_range, ain, x, y, z, ier, $
    xrange = xrange, yrange = yrange, zrange= zrange
;
;+
; NAME:
;       COND_RANGE
;
; PURPOSE:
;       Determine conductor extent at a plane in space
; CATEGORY:
;       Utility
; CALLING SEQUENCE:
;       COND_RANGE, A , X , Y , Z [, IER]  [, XRANGE = xrange] 
;           [, YRANGE = yrange] [, ZRANGE = zrange] 
; INPUTS:
;       A     IDL conductor  structure or structure array number (type = 4)
; Note: Only 2 of the following parameters will be passed with valid data.
;       The remaining parameter will be returned with the minimum and maximum
;       values of the conductor.  If no conductors are found 
;
;       X     First coordinate value or range of values where the conductor 
;               extent is desired
;       Y     Second coordinate value or range of values where the conductor 
;               extent is desired
;       Z     Third coordinate value or range of values where the conductor 
;               extent is desired
;     Note: An error tolerance of 2^(-16) is used  in determining whether
;              a plane crosses a conductor, that is,
;           maxp = max(a.plane(1, *, *))
;           minp = min(a.plane(0, *, *))
;           err = 1.52588e-5*( max([maxp-minp, abs(maxp), abs(minp)]))
;       IER   Error parameter
;             0 if maximum gt minimum+err
;             1 if maximum eq minimum
;            -1 if no conductors are found
;
; KEYWORD PARAMETERS:
;       X,Y,ZRANGE - Axis range(standard IDL meaning)
;                Note: If multiple keywords are set, only the first 
;                      detected will be used in order: Xrange, Yrange, Zrange
;
; EXAMPLE:
;       Determine the conductor radial extent for conductor structure 4 at 
;         theta = 0, and z = 4
;       COND_RANGE, 4, r, 0, 4, /x
;       Note: r(0) will be the minimum value of the conductor
;             and r(1) will be the maximum value of the conductor.
;
;
; MODIFICATION HISTORY:
;      Original version, L. P. Mix, August 2, 1993
;-
;
; insert a common block
;
@qsstrcom.dat
;
      on_error, 1
;
      if n_elements (structinit) eq 0 then qsstruct_init
;
;
      npar = n_params()
      if npar eq 0 then begin
        print, '% Format:', $
          'COND_RANGE, A , X, Y, Z, IER, /XRANGE, /YRANGE, /ZRANGE'
        return
      endif
      if npar lt 4 then begin
        print, 'COND_RANGE must be called with 4 or 5 parameters and 1 keyword.'
        print, '% Format:', $
          'COND_RANGE, A , X, Y, Z, IER, /XRANGE, /YRANGE, /ZRANGE'
        return
     endif
;    make sure we don't corrupt input data
;    now get the data  -- check that we have a structure of the proper type
      if n_elements(ain) eq 0 then begin
        print, 'Structure must be defined'
        return
      endif
      a = ain
      b = size(a)
      if b(b(0)+1) ne 8 then begin
        if a gt 0 and a le maxstructure then begin
          s2i, a, a
          b = size (a)
          if b(b(0)+1) ne 8 then begin
            print, ' First parameter must be a structure or a QS array number'
            return
          endif
        endif  else begin
          print, ' First parameter must be a structure or a QS array number'
          return
        endelse
      endif
      if a.type ne 4 then begin
        print, ' Extent determined only for Conductor type structures'
        return
      endif
;
      nx = n_elements (x)
      ny = n_elements (y)
      nz = n_elements (z)
;
      maxp = max(a.plane(1, *, *))
      minp = min(a.plane(0, *, *))
      err = 1.52588e-5*( max([maxp-minp, abs(maxp), abs(minp)]))
;
    if a.size(0) eq 0 then begin
      nok = a.size(4)
      arr = lindgen(nok)
    endif else begin
      arr = [a.n1,a.n2,a.n3] 
      ok = where(arr ge 0, nok)
      if nok gt 0 then arr = arr(ok)  
    endelse
    if keyword_set(xrange) then begin
      if ny eq 0 or nz eq 0 then $
        message, ' Y and Z values must be defined to obtain an X-range'
      if ny eq 1 then begin
        ylow = y-err
        yhig = y+err
      endif else begin
        ylow = min(y(0:1), max = mm)-err
        yhig = mm +err
      endelse
      if nz eq 1 then begin
        zlow = z-err
        zhig = z+err
      endif else begin
        zlow = min(z(0:1), max = mm)-err
        zhig = mm+err
      endelse
      n = where(a.plane(0,1, arr) le yhig and $
                a.plane(1,1, arr) ge ylow and $
                a.plane(0,2, arr) le zhig and $
                a.plane(1,2, arr) ge zlow, np)
      nss = 0
      if a.size(0) gt 0 then begin
        ns = where(a.plane(0,1, a.n0) le yhig and $
                   a.plane(1,1, a.n0) ge ylow and $
                   a.plane(0,2, a.n0) le zhig and $
                   a.plane(1,2, a.n0) ge zlow, nps)
        if nps gt 0 then begin
          err205 = 2.05*err
          x = make_array(2*nps)
          for i = 0, nps-1 do begin
            mm = min(abs(a.slnorm(*, ns(i))), ii)
            if ii eq 0 then begin
;           independent of x
              x(nss:nss+1) = a.slant(0,*, ns(i))
              nss = nss+2
            endif else if ii eq 1 then begin
;           independent of y
              if zhig-zlow lt err205 then begin
                x(nss) = a.slant(0, 0, ns(i)) - a.slnorm(2,ns(i))* $
                  (z(0)-a.slant(2,0,ns(i)))/a.slnorm(0,ns(i))
                nss = nss+1
              endif else begin
                xtemp = a.slant(0, 0, ns(i)) - a.slnorm(2,ns(i)) * $
                  (z(0:1)-a.slant(2,0,ns(i)))/a.slnorm(0,ns(i))
                xtemp = [min(xtemp, max = mm), mm]
                if xtemp(0) ge a.plane(0, 0, (a.n0)(ns(i))) $
                  then x(nss)=xtemp(0) else x(nss)=a.plane(0,0,(a.n0)(ns(i)))
                if xtemp(1) le a.plane(1, 0, (a.n0)(ns(i))) then $
                  x(nss+1)=xtemp(1) else x(nss+1)=a.plane(1,0,(a.n0)(ns(i)))
                nss = nss+2
              endelse
            endif else begin
;           independent of z
              if yhig-ylow lt err205 then begin
                x(nss) = a.slant(0, 0, ns(i)) - a.slnorm(1,ns(i))* $
                  (y(0)-a.slant(1,0,ns(i)))/a.slnorm(0,ns(i))
                nss = nss+1
              endif else begin
                xtemp = a.slant(0, 0, ns(i)) - a.slnorm(1,ns(i)) * $
                  (y(0:1)-a.slant(1,0,ns(i)))/a.slnorm(0,ns(i))
                xtemp = [min(xtemp, max = mm), mm]
                if xtemp(0) ge a.plane(0, 0, (a.n0)(ns(i))) $
                  then x(nss)=xtemp(0) else x(nss)=a.plane(0,0,(a.n0)(ns(i)))
                if xtemp(1) le a.plane(1, 0, (a.n0)(ns(i))) then $
                  x(nss)=xtemp(1) else x(nss)=a.plane(1,0,(a.n0)(ns(i)))
                nss = nss+2
              endelse
            endelse
          endfor
        endif
      endif
      if np gt 0 and nss gt 0 then begin
        x =[ reform(a.plane(*, 0, arr(n)), np+np, /overwrite), x(0:nss-1)]
      endif else if np gt 0 then begin
        x = a.plane(*, 0, arr(n))
      endif else if nss gt 0 then begin
        x = x(0:nss-1)
      endif else begin
        ier = -1
        x = [0.0, 0.0]
        return
      endelse
      x = [min(x, max = mm), mm]
      if x(1)-x(0) le err then ier = 1 else ier = 0
      return

    endif else if keyword_set(yrange) then begin
      if nx eq 0 or nz eq 0 then $
        message, ' X and Z values must be defined to obtain an Y-range'
      if nx eq 1 then begin
        xlow = x-err
        xhig = x+err
      endif else begin
        xlow = min(x(0:1), max = mm)-err
        xhig = mm+err
      endelse
      if nz eq 1 then begin
        zlow = z-err
        zhig = z+err
      endif else begin
        zlow = min(z(0:1), max = mm)-err
        zhig = mm+err
      endelse
      n = where(a.plane(0,0, arr) le xhig and $
                a.plane(1,0, arr) ge xlow and $
                a.plane(0,2, arr) le zhig and $
                a.plane(1,2, arr) ge zlow, np)
      nss = 0
      if a.size(0) gt 0 then begin
        ns = where(a.plane(0,0, a.n0) le xhig and $
                   a.plane(1,0, a.n0) ge xlow and $
                   a.plane(0,2, a.n0) le zhig and $
                   a.plane(1,2, a.n0) ge zlow, nps)
        if nps gt 0 then begin
          err205 = 2.05*err
          y = make_array(2*nps)
          for i = 0, nps-1 do begin
            mm = min(abs(a.slnorm(*, ns(i))), ii)
            if ii eq 1 then begin
;           independent of y
              y(nss:nss+1) = a.slant(1,*, ns(i))
              nss = nss+2
            endif else if ii eq 0 then begin
;           independent of x so use z
              if zhig-zlow lt err205 then begin
                y(nss) = a.slant(1, 0, ns(i)) - a.slnorm(2,ns(i))* $
                  (z(0)-a.slant(2,0,ns(i)))/a.slnorm(1,ns(i))
                nss = nss+1
              endif else begin
                xtemp = a.slant(1, 0, ns(i)) - a.slnorm(2,ns(i)) * $
                  (z(0:1)-a.slant(2,0,ns(i)))/a.slnorm(1,ns(i))
                xtemp = [min(xtemp, max = mm), mm]
                if xtemp(0) ge a.plane(0, 1, (a.n0)(ns(i))) $
                  then y(nss)=xtemp(0) else y(nss)=a.plane(0,1,(a.n0)(ns(i)))
                if xtemp(1) le a.plane(1, 1, (a.n0)(ns(i))) then $
                  y(nss+1)=xtemp(1) else y(nss+1)=a.plane(1,1,(a.n0)(ns(i)))
                nss = nss+2
              endelse
            endif else begin
;           independent of z so use x
              if xhig-xlow lt err205 then begin
                y(nss) = a.slant(1, 0, ns(i)) - a.slnorm(0,ns(i))* $
                  (x(0)-a.slant(0,0,ns(i)))/a.slnorm(1,ns(i))
                nss = nss+1
              endif else begin
                xtemp = a.slant(1, 0, ns(i)) - a.slnorm(0,ns(i)) * $
                  (x(0:1)-a.slant(0,0,ns(i)))/a.slnorm(1,ns(i))
                xtemp = [min(xtemp, max = mm), mm]
                if xtemp(0) ge a.plane(0, 1, (a.n0)(ns(i))) $
                  then y(nss)=xtemp(0) else y(nss)=a.plane(0,1,(a.n0)(ns(i)))
                if xtemp(1) le a.plane(1, 1, (a.n0)(ns(i))) then $
                  y(nss)=xtemp(1) else y(nss)=a.plane(1,1,(a.n0)(ns(i)))
                nss = nss+2
              endelse
            endelse
          endfor
        endif
      endif
      if np gt 0 and nss gt 0 then begin
        y =[ reform(a.plane(*, 1, arr(n)), np+np, /overwrite), y(0:nss-1)]
      endif else if np gt 0 then begin
        y = a.plane(*, 1, arr(n))
      endif else if nss gt 0 then begin
        y = y(0:nss-1)
      endif else begin
        ier = -1
        y = [0.0, 0.0]
        return
      endelse
      y = [min(y, max = mm), mm]
      if y(1)-y(0) le err then ier = 1 else ier = 0
      return


    endif else if keyword_set(zrange) then begin
        if nx eq 0 or ny eq 0 then $
          message, ' X and Y values must be defined to obtain an Z-range'
        if nx eq 1 then begin
          xlow = x-err
          xhig = x+err
        endif else begin
          xlow = min(x(0:1), max = mm)-err
          xhig = mm+err
        endelse
        if ny eq 1 then begin
          ylow = y-err
          yhig = y+err
        endif else begin
          ylow = min(y(0:1), max = mm)-err
          yhig = mm+err
        endelse
        n = where(a.plane(0,0, arr) le xhig and $
                  a.plane(1,0, arr) ge xlow and $
                  a.plane(0,1, arr) le yhig and $
                  a.plane(1,1, arr) ge ylow, np)
      nss = 0
      if a.size(0) gt 0 then begin
        ns = where(a.plane(0,0, a.n0) le xhig and $
                   a.plane(1,0, a.n0) ge xlow and $
                   a.plane(0,1, a.n0) le yhig and $
                   a.plane(1,1, a.n0) ge ylow, nps)
        if nps gt 0 then begin
          err205 = 2.05*err
          z = make_array(2*nps)
          for i = 0, nps-1 do begin
            mm = min(abs(a.slnorm(*, ns(i))), ii)
            if ii eq 2 then begin
;           independent of z
              z(nss:nss+1) = a.slant(2,*, ns(i))
              nss = nss+2
            endif else if ii eq 0 then begin
;           independent of x so use y
              if yhig-ylow lt err205 then begin
                z(nss) = a.slant(2, 0, ns(i)) - a.slnorm(1,ns(i))* $
                  (y(0)-a.slant(1,0,ns(i)))/a.slnorm(2,ns(i))
                nss = nss+1
              endif else begin
                xtemp = a.slant(2, 0, ns(i)) - a.slnorm(1,ns(i)) * $
                  (y(0:1)-a.slant(1,0,ns(i)))/a.slnorm(2,ns(i))
                xtemp = [min(xtemp, max = mm), mm]
                if xtemp(0) ge a.plane(0, 2, (a.n0)(ns(i))) $
                  then z(nss)=xtemp(0) else z(nss)=a.plane(0,2,(a.n0)(ns(i)))
                if xtemp(1) le a.plane(1, 2, (a.n0)(ns(i))) then $
                  z(nss+1)=xtemp(1) else z(nss+1)=a.plane(1,2,(a.n0)(ns(i)))
                nss = nss+2
              endelse
            endif else begin
;           independent of y so use x
              if xhig-xlow lt err205 then begin
                z(nss) = a.slant(2, 0, ns(i)) - a.slnorm(0,ns(i))* $
                  (x(0)-a.slant(0,0,ns(i)))/a.slnorm(2,ns(i))
                nss = nss+1
              endif else begin
                xtemp = a.slant(2, 0, ns(i)) - a.slnorm(0,ns(i)) * $
                  (x(0:1)-a.slant(0,0,ns(i)))/a.slnorm(2,ns(i))
                xtemp = [min(xtemp, max = mm), mm]
                if xtemp(0) ge a.plane(0, 2, (a.n0)(ns(i))) $
                  then z(nss)=xtemp(0) else z(nss)=a.plane(0,2,(a.n0)(ns(i)))
                if xtemp(1) le a.plane(1, 2, (a.n0)(ns(i))) then $
                  z(nss)=xtemp(1) else z(nss)=a.plane(1,2,(a.n0)(ns(i)))
                nss = nss+2
              endelse
            endelse
          endfor
        endif
      endif
      if np gt 0 and nss gt 0 then begin
        z =[ reform(a.plane(*, 2, arr(n)), np+np, /overwrite), z(0:nss-1)]
      endif else if np gt 0 then begin
        z = a.plane(*, 2, arr(n))
      endif else if nss gt 0 then begin
        z = z(0:nss-1)
      endif else begin
        ier = -1
        z = [0.0, 0.0]
        return
      endelse
      z = [min(z, max = mm), mm]
      if z(1)-z(0) le err then ier = 1 else ier = 0
      return
    endif else $
        message, ' Either Xrange, Yrange, or Zrange must be set'
    end
