; ------------------------------------------------------------------------ 
; $Id: cyl2cart.pro,v 1.5 2009/11/03 16:30:37 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
;+
; NAME:
;       CYL2CART
;
; PURPOSE:
;       Convect a cylindrical geometry to a rectilinear geometry.
;       The result will be a type 8, finite element mesh in x and y.
;                        
;       Three vector data will be convected to rectilinear data.
;         The three vectors are assumed to be Vr, Vtheta, and Vz.
;         They are transformed to Vx, Vy, and Vz.
;       Scalar data (single vector) data is not modified.
;
; CALLING SEQUENCE:
;       STR2= CYL2CART( STR1, [, ZVALUE] [, INDEX = index] [, /PSEUDO]
;             [, CLABEL = clabel] [, XLABEL = xlabel] [, YLABEL = ylabel]
;             [, SAVE = save] [, HELPME = helpme] )
;                  
; INPUTS:
;       STR2    Field structure with Cartesian data on a finite element mesh.
;               If SAVE is set, STR2 = 0, that is the following is executed:
;                 i2s, str2, save, /no_copy & return, 0
;               If an error occurs, STR2 = -1.
;         Note: To check for an error in CYL2CART, use the following.
;                 str2 = cyl2cart(str1, zvalue)
;                 if (size(str2, /type) lt 8) then $
;                   if str2 eq -1 then print, 'Error occured in CYL2CART'
;         Note: If a structure is returned or if 0 is returned, then no error.
;       STR1    Structure array number or field structure with Cylindrical data
;               on a rectilinear (r, thets, z) mesh
;       ZVALUE  Slice value in the Z direction.
;               If STR1 is a two dimensional field then this value is ignored
;               If STR1 is a three dimensional field then this value
;               indicates the coordinate value for the fixed dimension in
;               the two dimensional plots. Linear interpolation is used
;               according to the value of ZVALUE. If ZVALUE has two elements
;               the average value over the interval is used.
;               Note: If Zvalue is not valid, then the closest Z value is used.
;         Note: If INDEX keyword is set, the units are taken as array indices
;               which range from 1 to the maximum value. 
;               Caution: Multiple block data has may produce unexpected results.
;         Note: If the two elements of ZVALUE span a block boundary, then only
;               the values in the block containing the midpoint will be used.
;               If the midpoint falls on a block boundary, then the result is
;               probably not what you want!!
;       INDEX   If set, units of ZVALUE are Z indices in each block which can
;               range from 1 to n_elements(z)
;               Default: INDEX = 0
;       XLABEL  X Label Value to be used X1LAB
;               Default: X1LAB = 'X'
;       YLABEL  Y Label Value to be used X2LAB
;               Default: X2LAB = 'Y'
;       CLABEL  Comment value to be used CLABEL
;               Default: CLABEL = 'Cartesian '+str1.clab
;       PSEUDO  If set(/p), a pseudo cylindrical plot is made.
;               (This is useful for plotting timing data from an accelerator
;                 where the lines are arranged in a circle.)
;               This is limited to single block, scalar, 2D data.
;               X is assumed to run from (MIN(X) < 1) to N. 
;               Y is time or some similar quantity
;               The first value will be put on the positive Y axis and the
;               remaining points will be uniformly spaced around the circle.
;               If PSEUDO <= 2 then N = MAX(str1.x1) else N = PSEUDO
;               If PSEUDO has 2 elements then they will be taken as the minimum 
;                 and maximum values.
;       OFFSET  If PSEUDO is set, the radius (Y value) will be reduced by OFFSET.
;               This value will be returned in OFFSET. 
;       SAVE    If set, save STR2 in a structure array.  Valid values are
;               [1, get_maxstr()]  If SAVE is set, 
;                 then save = save if positive and <= get_maxstr()
;                           = first unused structure array number
;                           = get_maxstr() if all arrays are full
;       HELPME  If set, a help message will be printed
;
; EXAMPLE:
;       Convect structure f to fxy at the value of Z = 4.5
;         fxy = cyl2cart(f,4.5)
;
;       Convect structure 5 to a cartesian structure and store the
;       result in structure 45.  Slice at a value of 3
;         junk = cyl2cart(5, 3, s =45)
;
;       Convect structure 5 to a cartesian structure and store the
;       result in structure 45, 46 and 47  for Z-planes 1, 2 and 3.
;         for i = 1, 3 do junk = cyl2cart(5, i,/index save=44+i)
; Note: To get vectors in a plane. Vx component in this example and save in 21
;       To see the plot
;         plotstr, str2, 1, work = 21, cn=2 
;       or to just get the data
;         plotstr, str2, , result = 21, /nf, /quiet
;                      (results in 128 by 128 array)
;                      (change nfarray to get a different number)
;
; MODIFICATION HISTORY:
;      Original version, L. P. Mix, April 7, 1999
;-
;
  Function CYL2CART, STR1, ZVALUE, INDEX = index, PSEUDO = pseudo, $
              CLABEL = clabel, XLABEL = xlabel, YLABEL = ylabel, $
              SAVE = save, HELPME = helpme

  on_error, 1
;
;  Common blocks
@qscom.dat
@qsstrcom.dat
;
      if n_elements (structinit) eq 0 then qsstruct_init
      if n_elements (qsplotinit) eq 0 then qsinit
;
   err = -1
   nparam = n_params()
   if ((nparam eq 0) or (keyword_set(helpme))) then begin
     print,'% Format: ', $
   'STR2= CYL2CART( STR1, [, ZVALUE] [, INDEX = index] [, /PSEUDO]'
     print, '        [, CLABEL = clabel] [, XLABEL = xlabel] [, YLABEL = ylabel]'
     print, '        [, SAVE = save] [, HELPME = helpme] )'
     return, err
   endif
;
;     now get the data  -- check that we have a structure of the proper type
;
   if n_elements(str1) eq 0 then begin
     print, 'Structure must be defined'
     return, err
   endif else a= str1
;  change variables to insure we don't corrupt input data
;  get a save array number if possible
   if n_elements(save) eq 0 then save = 0
   if keyword_set(save) then save = get_save_str(save, maxstructure)
;
   b = size(a, /type)
   if b ne 8 then begin
     if a gt 0 and a le maxstructure then begin
       s2i, a, a
       b = size (a, /type)
       if b ne 8 then begin
         print, '% CYL2CART: ',  $
         ' First parameter must be a structure or a structure array number'
         return, err
       endif
     endif  else begin
       print, '% CYL2CART: ',  $
     ' First parameter must be a structure or a valid structure array number'
       return, err
     endelse
   endif
   if a.type ne 1 then begin
     print, '% CYL2CART: Only type 1 field structures may be convected'
     return, err
   endif

   nvect = a.nvect
   block = a.block
   ndim = a.ndim
   if ndim gt 3 then begin
     print, '% CYL2CART: Only 3 dimensional data may be convected'
     return, err
   endif

   if keyword_set (pseudo) then begin
     if ((block gt 1) or (nvect gt 1) or ((ndim gt 2)and(a.two ne 3))) then begin
       print, '% CYL2CART: PSEUDO is limited to single block, 2D, scalar data.'
       return, err
     endif
     y = a.x1
     if n_elements (pseudo) gt 1 then begin
       ymin = min(pseudo, max = ymax)
       y = float(y < ymax)
     endif else if (pseudo ge 2) then begin
       ymin = (min(y) < 1) 
       ymax = pseudo
       y = float(y < ymax)
     endif else ymin = (min(y, max = ymax) < 1) 
     y = (y-ymin) > 0
     n = (round (ymax -ymin) +1) > n_elements (y)
     y = !pi*0.5- y* (2*!pi/n)
;    add another point for closure
     y = [y, y[0]]
     x = a.x2
     xmax = max(x, min= xmin)
     nx = n_elements(x)
     ny = n_elements(y)
     ntot = nx*ny
     locv  = [0L, ntot]
     locx = locv
     ok = where (x eq 0.0, nok)
     if nok gt 0 then x(ok) = x(ok) + 1e-4*max(x)
     px = 0L
     pt = 0L
     pv = 0L
     cy = cos(y)
     sy = sin(y)
     offset = get_nice_values([xmin, xmax]) 
     if offset[0] ne 0.0 then x = x-offset[0]
     yy = reform(x # sy, ntot, /overwrite)
     xx = reform(x # cy, ntot, /overwrite)
     tri = make_triangles(0, nx, ny)+px
     ntri = (size(tri))(2)
     pt = ntri
     v1 = make_array(nx,ny)
     cont= get_array(a, -1)
     v1(*, 0:ny-2) = transpose(cont)
     v1(*, ny-1) = v1(*, 0) 
     x3=a.slice
     v2 = 0.0 & v3 = 0.0 & v4 = 0.0 & v5 = 0.0 & v6 = 0.0
     loctri = [0L, pt]
     range = make_array(2, ndim, 2)
     if ndim eq 3 then range(*, 2, *)=a.slice
     range(0, 0, *)= min(xx, max = mm)
     range(1, 0, *)= mm
     range(0, 1, *)= min(yy, max = mm)
     range(1, 1, *)= mm
     ssize = [px, px, 1]
;       XLABEL  Value to be used X1LAB
     if n_elements(xlabel) eq 0 then  xlabel = 'X'
     if n_elements(ylabel) eq 0 then  ylabel = 'Y'
     if n_elements(clabel) eq 0 then  clabel = 'Cartesian '+a.clab
     if offset[0] ne 0.0 then $
        clabel = clabel+', offset='+strtrim (offset[0], 2)
;
     a = {type: 8L , ndim:2L, nvect: nvect, block:1L, $
         two:3L, slice: a.slice, size: ssize, range: range, order: 0L, $
         locx:locx, locv: locv, triangles: tri, $
         loctri: loctri, x1:xx, x2:yy, x3:x3, x4:a.x4, $
         v1:v1, v2:v2, v3:v3, v4:v4, v5:v5, v6:v6, $
         x1lab : xlabel, x2lab : ylabel, x3lab : 'Z', $
         x4lab :a.x4lab[0], v1lab : a.v1lab[0] , v2lab : a.v2lab[0], $
         v3lab : a.v3lab[0], v4lab : a.v4lab[0], v5lab : a.v5lab[0], $
         v6lab : a.v6lab[0], file : a.file, spare : a.spare, tlab : a.tlab, $
         clab: clabel  }

     if not keyword_set(save) then return, a
     i2s, str2, save, /no_copy
     return, 0
   endif
   two = 3
;  determine the slice plane
   lcreduce = 0
   if ((ndim gt 2) and (a.two eq 0)) then begin
     lcreduce = 1
     zval = get_array(a, 3, 1)
     nzval = n_elements(zval)
     nz = n_elements(zvalue)
     if nz eq 0 then begin
       print, '% CYL2CART: Zvalue must be defined.'
       return, err
     endif 
     if keyword_set (index) then begin
       zv= round (zvalue) 
       if nz gt 1 then begin
         minz = (min(zv) > 1) < nzval
         maxz = (max(zv) < nzval) > 1
         if minz eq maxz then zv = minz else zv = [minz, maxz]
         x3 =  (zval[minz-1]+zval[maxz-1])*0.5
       endif else x3 = zval[((zv> 1)< nzval)-1]
     endif else begin
       zv = zvalue
       if nz gt 1 then begin
         minz = (min(zv) > a.range[0, 2, 0]) < a.range[1, 2, 0]
         maxz = (max(zv) < a.range[1, 2, 0]) > a.range[0, 2, 0]
         x3 = (minz+maxz)*0.5
         if minz eq maxz then zv = minz else zv = [minz, maxz]
       endif else x3 = (zvalue > a.range[0, 2, 0]) < a.range[1, 2, 0]
     endelse 
   endif else x3 = a.slice
   slice = x3
   ntot = a.size[0, 0]*a.size[1, 0]
   if block gt 1 then for i = 1, block-1 do ntot=ntot+a.size[0,i]*a.size[1,i]
   locv  = [0L, ntot]
   locx = locv
   xx = make_array(ntot)
   yy = make_array(ntot)
   v = make_array(ntot, nvect)
   triangles = make_array(3, 2*ntot, /long)
   px = 0L
   pt = 0L
   pv = 0L
   err = (a.ndim gt 2) ? 1e-5*(a.range(1, 2, 0)-a.range(0, 2, 0)) : 0.01

   for iblo = 1, block  do begin
     if (a.ndim eq 2) || $
       ((x3 ge a.range[0,2,iblo]-err) and (x3 le a.range[1,2,iblo]+err)) then begin
       x = get_array(a, 1, iblo)
       ok = where (x eq 0.0, nok)
       if nok gt 0 then x(ok) = x(ok) + 1e-4*max(x)
       y = get_array(a, 2, iblo)
       if ndim gt 2 then z = get_array(a, 3, iblo)
       nx = n_elements(x)
       ny = n_elements(y)
       nxy = nx*ny
       cy = cos(y)
       sy = sin(y)
       yy(px:px+nxy-1) = reform(x # sy, nxy, /overwrite)
       xx(px:px+nxy-1) = reform(x # cy, nxy, /overwrite)
       tri = make_triangles(0, nx, ny)+px
       ntri = (size(tri))(2)
       triangles(*, pt:pt+ntri-1) = tri
       pt = pt+ntri
       px = px+nxy
       if nvect ge 2 then begin
         cy = replicate (1.0, nx)#cy
         sy = replicate (1.0, nx)#sy
       endif
       vtemp = make_array(nx,ny, nvect)
       for iv = 1, nvect do begin
         cont= get_array(a, -iv, iblo)
; 
         if lcreduce gt 0 then $
           if keyword_set(index) then $
             vtemp(*,*, iv-1) = reduce_iarray(cont, 3, z, zv, 0) $
           else $
             vtemp(*,*, iv-1) = reduce_array(cont, 3, z, zv, 0) $
           else vtemp(*,*, iv-1) = cont
       endfor
;    for nvect eq 2 do the cylindircal transform otherwise just copy the values
       if nvect ge 2 then begin
         v(pv:pv+nxy-1, 0) = reform((vtemp[*,*,0]*cy-vtemp[*,*,1]*sy), nxy)
         v(pv:pv+nxy-1, 1) = reform((vtemp[*,*,0]*sy+vtemp[*,*,1]*cy), nxy)
         if nvect eq 3 then v(pv:pv+nxy-1, 2) = reform((vtemp[*,*,2]), nxy)
       endif else $
         for i = 0, nvect-1 do v(pv:pv+nxy-1, i) =reform((vtemp[*,*,i]), nxy)
       pv = pv+nxy
     endif
   endfor

   if px ne pv then begin
     print, '% CYL2CART: Problem with the numbers ' & help, px, pv, ntot
     return, err
   endif
   v =v(0:pv-1, *)
   xx = xx(0:pv-1)
   yy = yy(0:pv-1)
   v1 = v(*, 0)
   v2 = 0.0 & v3 = 0.0 & v4 = 0.0 & v5 = 0.0 & v6 = 0.0
   if nvect gt 3 then begin
     v4 = v(*, 3)
     if nvect gt 4 then v5 = v(*, 4)
     if nvect gt 5 then v6 = v(*, 5)
   endif 
     
   if nvect gt 1 then v2 = v(*, 1)
   if nvect gt 2 then v3 = v(*, 2)
   v = 0
   
   locx = [0L, px]
   locv = locx
   loctri = [0L, pt]
   range = make_array(2, ndim, 2)
   if ndim eq 3 then range(*, 2, *)=a.slice
   range(0, 0, *)= min(xx, max = mm)
   range(1, 0, *)= mm
   range(0, 1, *)= min(yy, max = mm)
   range(1, 1, *)= mm
   ssize = [px, px, 1]
;       XLABEL  Value to be used X1LAB
   if n_elements(xlabel) eq 0 then  xlabel = 'X'
   if n_elements(ylabel) eq 0 then  ylabel = 'Y'
   if n_elements(clabel) eq 0 then  clabel = 'Cartesian '+a.clab
   if nvect eq 3 then begin
     v1lab = 'X-Component'
     v2lab = 'Y-Component'
     v3lab = 'Z-Component'
   endif else begin
     v1lab = a.v1lab[0]
     v2lab = a.v2lab[0]
     v3lab = a.v3lab[0]
   endelse
;
         a = {type: 8L , ndim:2L, nvect: nvect, block:1L, $
          two:3L, slice: slice, size: ssize, range: range, order: 0L, $
          locx:locx, locv: locv, triangles: triangles(*,0:pt-1), $
          loctri: loctri, x1:xx, x2:yy, x3:x3, x4:a.x4, $
          v1:v1, v2:v2, v3:v3, v4:v4, v5:v5, v6:v6, $
          x1lab : xlabel, x2lab : ylabel, x3lab : 'Z', $
          x4lab : a.x4lab[0], v1lab : v1lab , v2lab : v2lab, $
          v3lab : v3lab, v4lab : a.v4lab[0], v5lab : a.v5lab[0], $
          v6lab : a.v6lab[0], file : a.file, spare : a.spare, tlab : a.tlab, $
              clab: clabel  }

   if not keyword_set(save) then return, a
   i2s, a, save, /no_copy 
   return, 0
   end

