; ------------------------------------------------------------------------ 
; $Id: direction.pro,v 1.2 2009/03/04 19:30:24 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
 Pro  direction, wdf1, wdf2, clabel = clabel, undetermined= undin, $
               zero = zeroin, magnitude = magnitude
;
;+
; NAME:
;       DIRECTION
;
; PURPOSE:
;       Find the direction cosines for a type 1 structure.
;       Structure must have 3 vector components and be in cartesian coordinates.
;       Note: PLOTVEC will plot the direction of vector arrays.
;       Note: Vectors with small amplitude have undetermined direction cosines.
;       Note: Other coordinates can be converted in a future version.  
;             Contact Paul Mix for support.
; CATEGORY:
;       Math 
; CALLING SEQUENCE:
;       DIRECTION, STR1 [, STR2] [, CLABEL = clabel] [, MAGNITUDE = magnitude]
;                     [, UNDETERMINED = undetermined]
; INPUTS:
;      STR1:    structure array number of a field structure with 3 vector components
; OPTIONAL INPUT:
;      STR2:    structure array number for the direction cosines of the vectors 
;               in STR1.  If not specified, the result is stored in STR1
;               DEFAULT: STR2 = STR1
;      CLABEL:  Optional label for the direction cosines structure  array
;               Default: "Direct Cosines for " is prepended to STR1.CLABEL
;      MAGNITUDE: Structure array number to store the magnitude.
;               Default: MAGNITUDE = GET_MAXSTR()-1
;      UNDETERMINED: If magnitude lt ZERO*MAX(Magnitude) direction cosines will 
;               be set to undetermined.
;               Default: UNDETERMINED = 0.0
;      ZERO:    Minimum amplitude for valid calculations.
;               This value is multiplied by the maximum magnitude of the vector.
;               Default: ZERO = 1e-7
;               (Valid values: 0 < ZERO < 0.5 )
; OUTPUTS:
;       Direction cosines for the vector data in STR1
; RESTRICTIONS:
;       STR1 array must be defined and have 3 vector components
;
; PROCEDURE:
;       The number of vectors and coordinate system is checked.
;       The magnitude of the vector is calculated and stored in a structure.
;       The direction cosines are calculated using:
;           STR.V1 = STR.V1 / Magnitude
;           STR.V2 = STR.V2 / Magnitude
;           STR.V3 = STR.V3 / Magnitude
;       where Magnitude = SQRT( V1*V1 +V2*V2 +V3*V3)
;       If Magnitude eq 0.0 then:
;           STR.V1 = STR.V2 = STR.V3 = UNDETERMINED
;
; EXAMPLES:
;       Calculate the direction cosines for structure 2.  Store the cosines
;         in structure 22 and the magnitude in 32.
;         DIRECTION, 2, 22, magnitude = 32
;
; MODIFICATION HISTORY:
;      Original version, L. P. Mix, October 3, 2008
;-
;
;  Common block
@qsstrcom.dat
@qscom.dat
;       return on an error
      on_error, 1
;
      if n_elements (structinit) eq 0 then qsstruct_init
      if n_elements (qsplotinit) eq 0 then qsinit
;
      nparams = n_params()
      if nparams eq 0 then begin
        print,'% FORMAT: DIRECTION, STR1 [, STR2] [, CLABEL = clabel]'
        print,'          [, MAGNITUDE = magnitude] [, UNDETERMINED = undetermined]'
        print,'          [, ZERO = zero] '
        return
      endif
;
      if n_elements(wdf1) eq 0 then begin
        print, '% DIRECTION: Structure array must be defined'
        return
      endif
      array1 = get_valid_structure(wdf1, 'DIRECTION', lcstr=lcstrin, ierror=ier)
      if ier ne 0 then return
      if (array1.type ne 1) then begin
        print, '% DIRECTION:  Type 1 field structures only'
        return
      endif
;     check for cylindrical / spherical coordinates
      If (((array1.spare)[0] eq 2) or ((array1.spare)[0] eq 3)) then begin
        print, '% DIRECTION:  Current version limited to cartesian coordinates'
        return
      endif
;
      nvect = array1.nvect
      if nvect lt 3 then begin
        print, '% DIRECTION: Structure must have at least 3 vector coordinates.'
        return
      endif
        
      nblo = array1.block
      if nblo gt 1 then begin
        print, '% DIRECTION: Present version must be single block.'
        return
      endif
;
      if lcstrin eq 0 then begin
        if n_elements(wdf2) eq 0 then wdf2 = wdf1
        if (wdf2 lt 1) or (wdf2 gt get_maxstr()) then wdf2 = wdf1
      endif
      zero = (n_elements(zeroin) eq 0) ? 1e-7 : zeroin[0] 
      zero = (zero > 0) < 0.5
      UND = (n_elements(undin) eq 0) ? 0.0 : undin[0]
      x1 = array1.x1
      x2 = array1.x2
      x3 = array1.x3
      v1 = array1.v1
      v2 = array1.v2
      v3 = array1.v3
      magnitude = sqrt(v1*v1+v2*v2+v3*v3)
      ok = where (magnitude le (zero*max(magnitude)), nok)
      if nok gt 0 then magnitude[ok] = 1.0
      v1 = v1/magnitude     
      v2 = v2/magnitude     
      v3 = v3/magnitude     
      if nok gt 0 then begin
        v1[ok] = und
        v2[ok] = und
        v3[ok] = und
      endif
      array1.v1 = v1
      array1.v2 = v2
      array1.v3 = v3
      if n_elements(clabel) gt 0 then array1.clab = string(clabel) else $
        array1.clab = 'Direct Cosines for '+ array1.clab
      magnitude = reform (magnitude, n_elements(x1), n_elements(x2), n_elements(x3))
      arr2str, get_maxstr()-1, magnitude, x1, y1, z1, v1lab = 'Magnitude', $
        clabel= 'Magnitude of '+array1.clab, $
        xlab = array1.x1lab, ylab= array1.x2lab, zlab= array1.x3lab
      if lcstrin eq 0 then i2s, array1, wdf2[0] else wdf1 = temporary(array1)
      return
      end
