; ------------------------------------------------------------------------ 
; $Id: editstr.pro,v 1.15 2011/07/12 17:49:20 lpmix Exp $
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
;+
; NAME:
;       EDITSTR 
;
; PURPOSE:
;       Edit a 2-D structure - Specified area is set to the average border value
;         or a user specified value.  The user may also set the value outside a 
;         specified region.
;       Procedure will generate windowed regions and display them. 
;         User may set windowed region to background or a user value.
;         Region may be rectangular, circular or elliptical.
;       A preview gives the window before storage. A preview will be generated
;         whenever the mouse moves in the window and one point has been 
;         selected.  The preview frame may be resized.
;       Note: Some time is required to calculate a preview frame. All events
;             generated while the preview frame is generated are deleted; be
;             sure the mouse clicks are acknowledged before proceeding.
;       Mouse buttons are mapped to widget buttons, cursor must be in the 
;         DRAW area of the main widget for the following mouse commands.
;          Left Button   = Set Value
;             Note: A carriage return in the data field will also set the value.
;          Middle Button = Accept/Save
;             (current designated windowed image will be saved)
;          Right Button  = Reset All Points
;             (All points will be undesignated)
;          Note: If a the last point required to define a curve is selected 
;                more than once, the previous "last point" is replaced.
;       Set Point Buttons: These perform the same function as a <CR> in either
;                          the X or Y windows above the buttons.
;
; CALLING SEQUENCE:
;	EDITSTR, STRin [, STRout] [, X, Y] [, XVALUE = X, YVALUE = Y]
;
;   where:
;       STRin - Structure array number of a two dimensional structure or 
;               a slice from three dimensional structure.
;               This structure will be overwritten if STRout is not specified.
;               The structure should be single block for maximum resolution.
;          NOTE: MaxStructure and MaxStructure-1 are used as work arrays.
;         
;       STRout- Structure array number of a two dimensional structure or 
;                 an output variable.  Default: STRout = STRin
;               NOTE:  If STRout has more than 1 element, it is assumed to be 
;                      the parameter X
;       X, Y - 2N X-values and 2N Y-values to specfy the region for editing
;              N is the number of edit regions
;              If they are present, no gui will be generated and the N regions 
;              will be edited.
;              Note: XValue and YValue are used if they are provided with two or
;                    more values.
;
; KEYWORD PARAMETERS:
;       XVALUE, YVALUE:  See X, Y parameters
;                 NOTE:  XVALUE and YVALUE are used if provided.
;       CIRCLE:   If set, the edit shape will be a circle 
;                 If cirdle is a float, an initial circle at the center of mass will
;                 be drawn
;           Note: If CIRCLE is set, ELLIPSE is ignored.
;       ELLIPSE:  If set, the edit shape will be an ellipse  
;           Note: If CIRCLE is set, ELLIPSE is ignored.
;       INVERTIT: If set set the values outside the shape.
;       EDIT_VALUE: Returns the last value used for the EDIT VALUE
;                   This may be the border value or a user value depending on settings
;       AVERAGE:  Returns the average for the current area selected
;       NOEDIT:   If set, the image will not be changed in batch mode but the AVERAGE 
;                 and EDIT_VALUE are returned.  This only applies to the last set of
;                 points if more than two points are provided.         
;       USER_VALUE: If N_ELEMENTS(USERVALUE) gt 0, then the selected region will 
;                   changed to USER_VALUE[0] rather than the border average.
;                   This value may be edited.
;                   The user value or border average will be displayed dynamically
;                   after two points are selected.
;       KEEP_ASPECT: Keep same aspect ratio in the preview window as in the
;                    primary window.  Default: KEEP_ASPECT = 0
;                    If KEEP_ASPECT is negative, the color map will also be kept.
;             NOTE:  Set the ASPECT keyword in EDITSTR to have the proper 
;                    aspect ratio in the primary window.  (See _EXTRA below.)
;       _EXTRA:     Any valid keyword for plotfld. The array for the windowing
;                   is generated with the following command and this plot is 
;                   used for the image:
;                      PLOTFLD, str, cn=2, _extra = extrastuff
; PROCEDURE:
;       A section of the image is selected.  The user may set the central
;       portion of the selected image to the average around the the exterior or 
;       to a user specified value.
;       Note:  The values specified in the windows are moved to the nearest node.
;              All nodes in the specified region are changed.
;   NOTE: MaxStructure and MaxStructure-1 are used as work arrays.
;    Buttons:
;       Border Value - replace region values with the image border average 
;                      The border average is the pixel adjoining the selected area.
;                      No values are used on sides that include the image border.
;                      For curved areas, only values above and below the area
;                        are used.
;       Surface Fit  - Use SURF_FIT to fit the boundary points to a second order 
;                      polynomial.  Replace the points with this fit.
;       User Value   - replace region values with the user specified value
;       Edit Value   - Actual value to be used.  May be edited in the user mode
;                      In Border mode, the value may be seen by switching
;                      to user mode; after reading the value the mode should be
;                      returned to "Border" mode if Border mode is desired.
;       Rectangle    - Define the edit region using the two points
;       Circle       - Define the edit region with a circle in the middle of the 
;                      two points.  The radius is sqrt(dx*dy)/2 where dx and dy
;                      are the horizontal and verticle spacings of the two points.
;       NOTE:  The average, minimum, and maximum are calculated for the rectangular 
;              region.
;       Ellipse      - Define the region with an ellipse tangent to the rectangle
;                      defined by the two points.
;       NOTE:  The average, minimum, and maximum are calculated for the rectangular 
;              region.
;       Invert       - Change the values outside the selected region.
;       Edit Region  - Change the value in the region.
;       Move Area    - If checked, the center of the region will be moved.
;                      If not checked, POINT_2 will be moved
;       Accept/Save  - Modify the image with the new values.
;       Reset All Points - Unset both the first and second points
;                      All modifications to the array since the last save are erased
;       UNDO         - Eliminate the last "Accept/Save Image" 
;       Done/Exit    - Exit and save the current image.
;
; EXAMPLE:
;	Eliminate a number of marks on images from structure array 3 and
;         store the result in structure array 11.
;           EDITSTR, 3, 11
;
;       Edit structure 4 and save the result in structure 12.  Use a circular
;         edit region and replace all places with a value of .01
;           EDITSTR, 4, 12, user=0.01, /circle
;
;       Determine the average of structure 2 in an area without data and use 
;         the average value of the rectangular region defined byt the two points
;         selected to subtract a background.
;           EDITSTR, 2, average = aver
;           SUBSTR, 2, 0, aver
;
; MODIFICATION HISTORY:
; 	Written by:	L. P. Mix, January 7, 2003
;       Modified: LPM, February 1, 2004
;                 Change edit region shapes, add user value and allow spot motion.
;       Modified: LPM, June 1, 2005
;                 Added batch option and AVERAGE and EDIT_Value keywords
;-


PRO editstr_event, Event
common editstr_data_com, xx, yy, array, arrayold, x, y, dx, dy, $
   xdraw_range, ydraw_range, bordertop, borderbot, borderx, inver,  $
   nx, ny, mode, px, py, qx, qy, strfile, widsav, strdx, STRdy, indx, indy, $
   arr_aver, arr_min, arr_max, levels_in, extra_in, average_val, $
   xsurf, ysurf, vsurf, vfit, extra1, extra2, extra3, extra4

common editstr_widget_com, xwindow1, ywindow1, xwindow2, ywindow2, widwindow, $
   bset1, bset2, image, draw, draw_id, line_id, oldbx, oldby, $
   print_var, oldbp, npselect, base21, base7, clabel, xlabel, ylabel, npwdf,$
   wdfarr, indep_var, ypreview, xpreviewsize, x2label, base22, avewin, $
   lcedval, edvalue, editval, lcshape, lcmoveit, beditval, lcnobatch, $
   minwin, maxwin, tlb_offset, line_offset, xsize, ysize, win_aspect, win_levels

COMPILE_OPT hidden
on_error, 1

  WIDGET_CONTROL,Event.Id,GET_UVALUE=uvalue
  lcdoit =0
  if uvalue eq 'EDITVAL' then begin
    edvalue = event.value
    widget_control, editval, set_value = edvalue
    return
  endif else if uvalue eq 'ESHAPE' then begin
    lcshape = event.value
    lcdoit = 2
  endif else if uvalue eq 'ESOURCE' then begin
    lcedval = event.value
    widget_control, beditval, sensitive = lcedval
    lcdoit = 1
  endif else if uvalue eq 'INVERT_BUTTON' then begin
    inver = event.select
    return
  endif

  if lcdoit eq 0 then begin
    print, '%EDITSTR_EVENT: EVENT UVALUE = ', uvalue
    help, /str, event
    return
  endif
; quit if we have less than 2 points selected
  if (npselect lt 2)  then return
; recalculate the edit values
  widget_control, xwindow1, get_value = px
  widget_control, ywindow1, get_value = py
  widget_control, xwindow1, set_value = px
  widget_control, ywindow1, set_value = py
  widget_control, xwindow2, get_value = qx
  widget_control, ywindow2, get_value = qy
  widget_control, xwindow2, set_value = qx
  widget_control, ywindow2, set_value = qy
  get_editstr_line
  if lcdoit eq 2 then begin
    device, copy = image
    if n_elements(xx) gt 2 then oplot, xx, yy, thick =2, col=7
    plots, px, py, psym = 8, color = 1, symsiz=1.5
    plots, qx, qy, psym = 8, color = 2, symsiz=1.5
  endif

  END

 Pro editstr_doit
common editstr_data_com

common editstr_widget_com
COMPILE_OPT hidden
; 
; Calculate a image of the array between points p and q.
;
 on_error, 1
  widget_control, xwindow1, get_value = px
  widget_control, ywindow1, get_value = py
  widget_control, xwindow1, set_value = px
  widget_control, ywindow1, set_value = py
  widget_control, xwindow2, get_value = qx
  widget_control, ywindow2, get_value = qy
  widget_control, xwindow2, set_value = qx
  widget_control, ywindow2, set_value = qy
; be sure values are floats
   get_editstr_line

 if n_elements(xx) gt 2  then begin
   if line_id  ge 0 then begin
     if strupcase(strmid(!version.os, 0, 3)) ne 'MAC' then begin
       device, window_state = wins
       xsiz = xpreviewsize(0)
       ysiz = xpreviewsize(1)
       if n_elements(wins) lt line_id +1 then begin
         window, /free, xsize= xsiz, ysize= ysiz, title = 'Current Sub-Image', $
            xpos = line_offset[0], ypos= line_offset[1]
         line_id = !d.window
       endif else if wins(line_id) eq 0 then begin
         window, /free, xsize= xsiz, ysize= ysiz, title = 'Current Sub-Image', $
            xpos = line_offset[0], ypos= line_offset[1]
         line_id = !d.window
       endif else begin
         wset, line_id
         device, get_window_position = line_offset
         xpreviewsize[0] = !d.x_size
         xpreviewsize[1] = !d.y_size
         xsiz = xpreviewsize(0)
         ysiz = xpreviewsize(1)
       endelse
     endif else wset, line_id
     okx = where (x ge xdraw_range[0] and x le xdraw_range[1], nokx)
     oky = where (y ge ydraw_range[0] and y le ydraw_range[1], noky)
     if nokx gt 2 and noky gt 2 then begin
       if lcshape eq 0 then begin
         ix1 = okx[0] & ix2 = okx[nokx-1]
         iy1 = oky[0] & iy2 = oky[noky-1]
         mnar = min(array[ix1:ix2, iy1:iy2], max = mxar)
         average_val = total(array[ix1:ix2, iy1:iy2], /double)/(nokx*noky)
       endif else begin
         npix = 0L
;        borderx, bordertop, borderbot
         icol = n_elements(borderx)
         average_val = 0.0d0
         mnar = make_array(icol)
         mxar = make_array(icol)
         for i = 0, icol-1 do begin
           npix = (bordertop[i]-borderbot[i]) +1 +npix
           average_val = average_val + $
             total(array[borderx[i], borderbot[i]:bordertop[i]], /doub)
           mnar[i] = min(array[borderx[i], borderbot[i]:bordertop[i]], max = m)
           mxar[i] = m
         endfor
         mnar = min(mnar)
         mxar = max(mxar)
         average_val = (average_val/npix)
       endelse

       if keyword_set(lcnobatch) then begin
         widget_control, avewin, set_value = average_val
         widget_control, minwin, set_value = mnar
         widget_control, maxwin, set_value = mxar
       endif
       if win_aspect ne 0 then begin
;        win_aspect = (Yrange/ysize)/(Xrange/xsize)
;        pasp = yrange/xrange - for preview
         pasp = (ydraw_range[1] - ydraw_range[0]) / $
            (xdraw_range[1] - xdraw_range[0])
         if win_aspect gt pasp/ysiz*xsiz then begin
           xsiza = xsiz
           ysiza = round(xsiz/win_aspect*pasp)
         endif else begin
           ysiza = ysiz
           xsiza = round(ysiz*win_aspect/pasp)
         endelse
       endif else begin
         xsiza = xsiz
         ysiza = ysiz
       endelse
       indx = [okx[0], okx[nokx-1]]
       indy = [oky[0], oky[noky-1]]
       xpre = xdraw_range[1]-xdraw_range[0]
       xpre = findgen(xsiza)*(xpre/float(xsiza-1))+xdraw_range[0]
       ii =  (okx[0]-1) > 0
       ie =  okx[nokx-1] +1 <(nx-1)
       ix = findex(x[ii:ie], xpre)+ii
       ypre = ydraw_range[1]-ydraw_range[0]
       ypre = findgen(ysiza)*(ypre/float(ysiza-1))+ydraw_range[0]
       ii =  (oky[0]-1) > 0
       ie =  oky[noky-1] +1 <(ny-1)
       iy = findex(y[ii:ie], ypre)+ii
;       ix = findgen(xsiza)*(nokx/float(xsiza-1)) + okx[0]
;       iy = findgen(ysiza)*(noky/float(ysiza-1)) + oky[0]
       im= interpolate(array, ix, iy, /grid)
       erase
       if n_elements(win_levels) eq 2 then $
         tv, bytscl(im, min= win_levels[0], max = win_levels[1], $
               top=!d.table_size-11)+9 $
       else tv, bytscl(im, top= !d.table_size-11)+9

     endif else begin
       erase
       indx = 0
       indy = 0
     endelse
   endif
   wset, draw_id
   !x = oldbx
   !y = oldby
   !p = oldbp
 endif
 device, copy = image
 if n_elements(xx) gt 2 then oplot, xx, yy, thick =2, col=7
 plots, px, py, psym = 8, color = 1, symsiz=1.5
 plots, qx, qy, psym = 8, color = 2, symsiz=1.5
 end

pro get_editstr_line
  common editstr_data_com
  common editstr_widget_com
COMPILE_OPT hidden
on_error, 1 

  xdraw_range = [min([px, qx], max = mm), mm]
  ydraw_range = [min([py, qy], max = mm), mm]
  borderx = (round(findex(x, xdraw_range)) > 0) < (nx-1)
  bordery = (round(findex(y, ydraw_range)) > 0) < (ny-1)
  if lcshape eq 0 then begin
; rectangle
     ival = [0,1,1,0,0]
     jval = [0,0,1,1,0]
     xx = xdraw_range[ival]
     yy = ydraw_range[jval]
     ix1 = borderx[0]
     ix2 = borderx[1]
     iy1 = bordery[0]
     iy2 = bordery[1]
     borderbot = iy1
     bordertop = iy2
     if lcedval lt 2 then begin
;      get the border values
       iax1 = (ix1-1) > 0
       iax2 = (ix2+1) < (nx-1) 
       iay1 = (iy1-1) > 0
       iay2 = (iy2+1) < (ny-1)
       n= 0L
       tot = 0.0
       if iax1 lt ix1 then begin
;        left side
         nadd= iay2-iay1+1
         if lcedval eq 0 then tot = total(array[iax1, iay1:iay2]) $
         else begin 
             xsurf = replicate (x[iax2], nadd)
             ysurf = y[iay1:iay2]
             vsurf = reform(array[iax2, iay1:iay2])
         endelse
         n= nadd
       endif
       if iax2 gt ix2 then begin
;        right side
         nadd = iay2-iay1+1
         if lcedval eq 0 then tot = tot+total(array[iax2, iay1:iay2]) $
         else begin 
           if n eq 0 then begin
             xsurf = replicate (x[iax2], nadd)
             ysurf = y[iay1:iay2]
             vsurf = reform(array[iax2, iay1:iay2])
           endif else begin
             xsurf = [xsurf, replicate (x[iax2], nadd)]
             ysurf = [ysurf, y[iay1:iay2]]
             vsurf = [vsurf, reform(array[iax2, iay1:iay2])]
           endelse
         endelse
         n= n+ nadd
       endif
       if iay1 lt iy1 then begin
;        bottom side
         nadd=  iax2-iax1+1
         if lcedval eq 0 then tot = tot+total(array[iax1:iax2, iay1]) $
         else begin
           if n eq 0 then begin
             xsurf = x[iax1:iax2]
             ysurf = replicate (y[iay1], nadd)
             vsurf = reform(array[iax1:iax2, iay1])
           endif else begin
             xsurf = [xsurf, x[iax1:iax2]]
             ysurf = [ysurf, replicate (y[iay1], nadd)]
             vsurf = [vsurf, reform (array[iax1:iax2, iay1])]
           endelse
         endelse
         n += nadd
       endif
       if iax2 gt ix2 then begin
;        top side
         nadd= iax2-iax1+1
         if lcedval eq 0 then tot = tot+total(array[iax1:iax2, iay2]) $
         else begin
           if n eq 0 then begin
             xsurf = x[iax1:iax2]
             ysurf = replicate (y[iay2], nadd)
             vsurf = reform(array[iax1:iax2, iay2])
           endif else begin
             xsurf = [xsurf, x[iax1:iax2]]
             ysurf = [ysurf, replicate (y[iay2], nadd)]
             vsurf = [vsurf, reform(array[iax1:iax2, iay2])]
           endelse
         endelse
         n += nadd
       endif
       if lcedval eq 0 then begin
         edvalue =  (n gt 0) ?  tot/n : 0.0
         if keyword_set(lcnobatch) then $
           widget_control, editval, set_value = edvalue
       endif else begin
         surf_fit, xsurf, ysurf, vsurf, /quiet
       endelse
     endif    
  endif else begin
    a = (xdraw_range[1]-xdraw_range[0])*0.5
    b = (ydraw_range[1]-ydraw_range[0])*0.5
    if a/dx[0] lt 1 || b/dy[0] lt 1 then return
    center = [total(xdraw_range), total(ydraw_range)]*0.5
    if lcshape eq 1 then begin
;     circle
      r = sqrt(a*b)
      lx = (center[0]-r) > x[0]
      rx = (center[0]+r) < x[nx-1]
      ix = round(findex(x, [lx, rx]))
      borderx = ix[0]+indgen(ix[1]-ix[0]+1)
      x1 = x[borderx]
      xc = x1 - center[0]
      y1 = sqrt((r*r-xc*xc)>0)
      bordertop = round(findex(y, (center[1]+y1) < y[ny-1]))
      borderbot = round(findex(y, (center[1]-y1) > y[0]))
      xx = x[borderx]
      xx= [xx, rotate(xx,2)]
      yy= y[[bordertop, rotate(borderbot,2)]]
;       plots, xx , yy, psym = 8 , col = 3 & hak, /mes
      if lcedval eq 0 then begin
;       get the border values
        iydown = borderbot-1 
        iyup = bordertop+1
        ok = where (iydown ge 0, n)
        tot = (n gt 0) ? total(array[nx*iydown[ok]+borderx[ok]]) : 0.0
        ok = where (iyup lt ny, nok)
        n = n+nok
        tot = tot + ((nok gt 0) ? total(array[nx*iyup[ok]+borderx[ok]]) : 0.0)
        edvalue =  (n gt 0) ?  tot/n : 0.0
        if keyword_set(lcnobatch) then $
          widget_control, editval, set_value = edvalue
      endif else if lcedval eq 1 then begin
;       get the border values
        iydown = borderbot-1
        iyup = bordertop+1
        ok = where (iydown ge 0, n)
        if n gt 0 then begin
          xsurf = x[borderx[ok]]
          ysurf = y[iydown[ok]]
          vsurf = array[nx*iydown[ok] +borderx[ok]]
        endif
        ok = where (iyup lt ny, nok)
        if nok gt 0 then begin
          if n gt 0 then begin
            xsurf = [xsurf , x[borderx[ok]]]
            ysurf = [ysurf, y[iyup[ok]]]
            vsurf = [vsurf, array[nx*iyup[ok] +borderx[ok]]]
          endif else begin
            xsurf = x[borderx[ok]]
            ysurf = y[iyup[ok]]
            vsurf = array[nx*iyup[ok] +borderx[ok]]
          endelse
        endif
        if n+nok gt 6 then surf_fit, xsurf, ysurf, vsurf, /quiet
      endif
    endif else begin
;     ellipse
      lx = (center[0]-a) > x[0]
      rx = (center[0]+a) < x[nx-1]
      ix = round(findex(x, [lx, rx]))
      borderx = ix[0]+indgen(ix[1]-ix[0]+1)
      x1 = x[borderx]
      xc = x1 - center[0]
      a2 = a*a
      y1 = b*sqrt((1-xc*xc/a2)>0)
      bordertop = round(findex(y, (center[1]+y1) < y[ny-1]))
      borderbot = round(findex(y, (center[1]-y1) > y[0]))
      xx = x[borderx]
      xx= [xx, rotate(xx,2)]
      yy= y[[bordertop, rotate(borderbot,2)]]
      if lcedval eq 0 then begin
;       get the border values
        iydown = borderbot-1   
        iyup = bordertop+1
        ok = where (iydown ge 0, n)
        tot = (n gt 0) ? total(array[nx*iydown[ok]+borderx[ok]]) : 0.0
        ok = where (iyup lt ny, nok)
        n = n+nok
        tot = tot + ((nok gt 0) ? total(array[nx*iyup[ok]+borderx[ok]]) : 0.0)
        edvalue =  (n gt 0) ?  tot/n : 0.0
        if keyword_set(lcnobatch) then $
          widget_control, editval, set_value = edvalue
      endif else if lcedval eq 1 then begin
;       get the border values
        iydown = borderbot-1
        iyup = bordertop+1
        ok = where (iydown ge 0, n)
        if n gt 0 then begin
          xsurf = x[borderx[ok]]
          ysurf = y[iydown[ok]]
          vsurf = array[nx*iydown[ok] +borderx[ok]]
        endif
        ok = where (iyup lt ny, nok)
        if nok gt 0 then begin
          if n gt 0 then begin
            xsurf = [xsurf , x[borderx[ok]]]
            ysurf = [ysurf, y[iyup[ok]]]
            vsurf = [vsurf, array[nx*iyup[ok] +borderx[ok]]]
          endif else begin
            xsurf = x[borderx[ok]]
            ysurf = y[iyup[ok]]
            vsurf = array[nx*iyup[ok] +borderx[ok]]
          endelse
        endif
        if n+nok gt 6 then surf_fit, xsurf, ysurf, vsurf, /quiet
      endif
    endelse
  endelse  
  if lcedval eq 2 && keyword_set(lcnobatch) then begin
    widget_control, editval, get_value = edvalue
    widget_control, editval, set_value = edvalue
  endif
  end

 pro editstr_point_event, event
COMPILE_OPT hidden
;
  on_error, 1
  common editstr_data_com
  common editstr_widget_com

  WIDGET_CONTROL,Event.Id,GET_UVALUE=Ev
  CASE EV OF
   'DRAW':  Begin
            if event.press eq 4 then editstr_reset_button else begin
              if event.press eq 2 then begin
                  if (npselect eq 2)  then $
                    edit_save_button_event, event else $
                    winformation, 'Two points must be selected for a window'
              endif else begin
                if event.press eq 1 then begin
                  coord = convert_coord(event.x, event.y, /device, /to_data)
                  if (npselect eq 0)  then begin
                    widget_control, xwindow1, set_value = coord(0) 
                    widget_control, ywindow1, set_value = coord(1) 
                    npselect = 1 > npselect
                    widget_control, draw, draw_motion_events=1, /clear_events
                  endif else begin
                    if npselect eq 2 && lcmoveit gt 0 then begin
                      widget_control, xwindow1, get_value = px
                      widget_control, ywindow1, get_value = py
                      widget_control, xwindow2, get_value = qx
                      widget_control, ywindow2, get_value = qy
                      center = [px+qx, py+qy]*0.5
                      delta = coord[0:1] - center
                      widget_control, xwindow1, set_value = px + delta[0]
                      widget_control, ywindow1, set_value = py + delta[1]
                      widget_control, xwindow2, set_value = qx + delta[0]
                      widget_control, ywindow2, set_value = qy + delta[1]
                      editstr_doit
                      return
                    endif
                    widget_control, xwindow2, set_value = coord(0)
                    widget_control, ywindow2, set_value = coord(1)
                    npselect = 2
                    editstr_doit
                    widget_control, base7, map =1
                    widget_control, draw, /clear_events, draw_motion_events = 0
                  endelse
                endif else if event.release eq 0 then begin
                  coord = convert_coord(event.x, event.y, /device, /to_data)
                  if npselect eq 0 then begin
                    widget_control, xwindow1, set_value = coord(0)
                    widget_control, ywindow1, set_value = coord(1)
                  endif else begin
                    widget_control, xwindow2, set_value = coord(0)
                    widget_control, ywindow2, set_value = coord(1)
                    editstr_doit
                  endelse
                endif else widget_control, draw, /clear_events
              endelse
            endelse
            END
  'XVALUE1': EDITSTR_HANDLE_1
  'YVALUE1': EDITSTR_HANDLE_1
  'BUTTON1': EDITSTR_HANDLE_1
  'XVALUE2': EDITSTR_HANDLE_2
  'YVALUE2': EDITSTR_HANDLE_2
  'BUTTON2': EDITSTR_HANDLE_2
   ELSE:   Begin
           print, ev
           help, /str, event
           END
  ENDCASE
  end

 Pro editstr_handle_1
COMPILE_OPT hidden
   on_error, 1
  common editstr_data_com
  common editstr_widget_com
  widget_control, xwindow1, get_value = px
  widget_control, ywindow1, get_value = py
  widget_control, xwindow1, set_value = px
  widget_control, ywindow1, set_value = py
  if ((npselect eq 0) or (npselect ge 1 and mode gt 0)) then begin
     npselect = 1 > npselect
     if mode gt 0 then begin
       editstr_doit
       widget_control, base7, map =1
       widget_control, draw, /clear_events, draw_motion_events = 0
     endif else $
       widget_control, draw, draw_motion_events = 1, /clear_events
  endif else begin
     if npselect eq 2 then begin 
       editstr_doit
       widget_control, base7, map =1
       widget_control, draw, /clear_events, draw_motion_events = 0
     endif
  endelse
  end

 Pro editstr_handle_2
COMPILE_OPT hidden
   on_error, 1
  common editstr_data_com
  common editstr_widget_com
  widget_control, xwindow1, get_value = px
  widget_control, ywindow1, get_value = py
  widget_control, xwindow2, get_value = qx
  widget_control, ywindow2, get_value = qy
  if (npselect eq 0) then begin
    widget_control, xwindow1, set_value = qx
    widget_control, ywindow1, set_value = qy
    widget_control, xwindow2, set_value = px
    widget_control, ywindow2, set_value = py
    npselect = 1 
    if mode gt 0 then begin
       editstr_doit
       widget_control, base7, map =1
       widget_control, draw, /clear_events, draw_motion_events = 0
    endif else widget_control, draw, /clear_events, draw_motion_events = 1
  endif else begin
    widget_control, xwindow2, set_value = qx
    widget_control, ywindow2, set_value = qy

    npselect = 2
    widget_control, base7, map =1
    editstr_doit
    widget_control, draw, /clear_events, draw_motion_events = 0
  endelse
  end

Pro edit_save_button_event, event, wdf
COMPILE_OPT hidden
  common editstr_data_com
  common editstr_widget_com
  common editstr_edit_button, lcedit 
@qsstrcom.dat
  if n_elements(lcedit) eq 0 then lcedit = 0
  if n_elements(event) eq 0 then lcsave = 1 else begin
    if size(event, /type) eq 8 then begin
      WIDGET_CONTROL,Event.Id,GET_UVALUE=Ev
      if ev eq 'MOVEIT' then begin
        lcmoveit = event.select
        return
      endif
      if ev eq 'EDIT_BUTTON' then lcsave = 0 else lcsave=1
    endif else lcsave = 1
  endelse
  if lcsave eq 0 then begin
;   edit the data
;
    get_editstr_line
    arrayold = array
;   rectangular    
    
    if lcshape eq 0 then begin
      if lcedval ne 1 then begin
        if inver eq 0 then $
          array[borderx[0]: borderx[1], borderbot:bordertop]= edvalue else begin
          if borderx[0] gt 0 then array[0:borderx[0], *] = edvalue
          if borderx[1] lt (nx-1) then array[borderx[1]:(nx-1), *] = edvalue
          if borderbot gt 0 then array[borderx[0]:borderx[1], 0:borderbot] = edvalue
          if bordertop lt (ny-1) then $
            array[borderx[0]:borderx[1], bordertop: (ny-1)] = edvalue
        endelse
;       surface fit (lcedval =1)
      endif else begin
        xfit = x[borderx[0]:borderx[1]]
        yfit = y[borderbot:bordertop]
        surf_fit, xfit, yfit, vfit, /evaluate, /grid, /quiet
        array[borderx[0]: borderx[1], borderbot:bordertop] = vfit  
      endelse
    endif else begin
;   circle/elipse
      nborderm = n_elements(borderx) -1
      if lcedval ne 1 then begin
        if inver eq 0 then begin
          for i = 0, nborderm do $
            array[borderx[i], borderbot[i]:bordertop[i]] = edvalue
        endif else begin
          if borderx[0] gt 0 then array[0:borderx[0], *] = edvalue
          if borderx[1] lt (nx-1) then array[borderx[nborderm]:(nx-1), *] = edvalue
          for i = 0, nborderm do array[borderx[i], 0:borderbot[i]] = edvalue
          for i = 0, nborderm do array[borderx[i], bordertop[i]:(ny-1)] = edvalue
        endelse
      endif else begin
;       surface fit 
        for i = 0, nborderm do begin
          yval =y[ borderbot[i]: bordertop[i]]
          xval =replicate (x[borderx[i]] , n_elements(yval))
          surf_fit, xval, yval, vfit, /eval, /quiet
          array[ borderx[i], borderbot[i]: bordertop[i]] = vfit
        endfor
      endelse
    endelse
    if not keyword_set (lcnobatch) then return
    lcedit = 1
    editstr_doit 
    return
  endif
    put_array, maxstructure-1, array, -1
    lcedit = 0
    wset , image[6]
    plotfld,  maxstructure-1, levels= levels_in, cntour = 2, _extra = extra_in
    wset, draw_id
    editstr_reset_button
  END

pro editstr_undo_button
COMPILE_OPT hidden
  common editstr_data_com
  common editstr_widget_com
  common editstr_edit_button, lcedit
@qsstrcom.dat
  if n_elements(lcedit) eq 0 then lcedit = 0

on_error, 1
  array = arrayold
  put_array, maxstructure-1, array, -1
  lcedit = 0
  wset , image[6]
  plotfld,  maxstructure-1, levels= levels_in, cntour = 2, _extra = extra_in
  wset, draw_id
  editstr_reset_button
end

pro editstr_reset_button
COMPILE_OPT hidden
; Handle the actual reset
  common editstr_data_com
  common editstr_widget_com
  common editstr_edit_button, lcedit 
@qsstrcom.dat
    lcedit = 0
    npselect = 0
    if line_id gt 0 then begin
      wset, line_id
      erase
    endif
    wset, draw_id
    device , copy = image
    !x = oldbx
    !y = oldby
    !p = oldbp
    array = get_array(maxstructure-1, -1)
    widget_control, draw, /clear_events
    widget_control, base7, map = 0
    widget_control, draw, /clear_events, draw_motion_events = 0
    xx =0
  END

pro editreset_button_event, event
COMPILE_OPT hidden
;
; Handle the reset, the undo, and the exit buttons
  common editstr_edit_button, lcedit 
  if n_elements(lcedit) eq 0 then lcedit = 0
; First RESET
  on_error, 1
  common editstr_data_com
  common editstr_widget_com
  if event.value eq 1 then editstr_undo_button  else $
  if event.value eq 0 then editstr_reset_button else $
    if event.value eq 1 then editstr_undo_button else begin
    if npselect eq 2  and lcedit eq 1 then begin
      wsave_nosave, lcquit, [ 'There is a WINDOW defined', $
          'Do you want to save it before EXIT'], 'WINSTR EXIT', $
           group = event.top 
      if lcquit eq 0 then edit_save_button_event
    endif
    if strupcase(strmid(!version.os, 0, 3)) ne 'MAC' then begin
      device, window_state = wins
      xsiz = xpreviewsize(0)
      ysiz = xpreviewsize(1)
      if n_elements(wins) gt line_id then begin
        if wins[line_id] gt 0 then begin
          wset, line_id
          device, get_window_position = line_offset
          xpreviewsize[0] = !d.x_size
          xpreviewsize[1] = !d.y_size
          wdelete, line_id
          line_id = -1
        endif
      endif
    endif
    widget_control, event.top, tlb_get_offset = tlb_offset
    widget_control, event.top, /destroy
  endelse
  end

pro editstr_cleanup, val
COMPILE_OPT hidden
  common editstr_data_com
  common editstr_widget_com
  common editstr_cleanup, oldwin, pixwin, initbx, initby, initbp

  if line_id gt 0 then begin
    device, window_state = wins
    if n_elements(wins) gt line_id then $
        if wins[line_id] gt 0 then wdelete, line_id
  endif
  if oldwin lt 0 then return
  wset, oldwin
  !x = initbx
  !y = initby
  !p = initbp
end

pro editstr_batch, xin, yin, noedit= noedit
  COMPILE_OPT hidden
  common editstr_data_com
  common editstr_widget_com
 
  nxx = n_elements (xin)
  nyy = n_elements (yin)
  nregion = min([nxx, nyy])/2
  nregm = nregion-1
  for ireg = 0, nregm do begin
    px = xin[ireg*2]
    qx = xin[ireg*2+1]
    py = yin[ireg*2]
    qy = yin[ireg*2+1]
    get_editstr_line
    okx = where (x ge xdraw_range[0] and x le xdraw_range[1], nokx)
    oky = where (y ge ydraw_range[0] and y le ydraw_range[1], noky)
    if nokx gt 2 and noky gt 2 then begin
       ix1 = okx[0] & ix2 = okx[nokx-1]
       iy1 = oky[0] & iy2 = oky[noky-1]
       mn = min(array[ix1:ix2, iy1:iy2], max = mx)
       average_val = total(array[ix1:ix2, iy1:iy2], /double)/(nokx*noky)
    endif
    if (ireg lt nregm) || (~ keyword_set(noedit)) then begin
;     Code from edit_save_button_event below.
;     rectangular    
      if lcshape eq 0 then begin
        if inver eq 0 then $
          array[borderx[0]: borderx[1], borderbot:bordertop]= edvalue else begin
          if borderx[0] gt 0 then array[0:borderx[0], *] = edvalue
          if borderx[1] lt (nx-1) then array[borderx[1]:(nx-1), *] = edvalue
          if borderbot gt 0 then array[borderx[0]:borderx[1], 0:borderbot] = edvalue
          if bordertop lt (ny-1) then $
            array[borderx[0]:borderx[1], bordertop: (ny-1)] = edvalue
        endelse
      endif else begin
;     circle/elipse
        nborderm = n_elements(borderx) -1
        if inver eq 0 then begin
          for i = 0, nborderm do $
            array[borderx[i], borderbot[i]:bordertop[i]] = edvalue
        endif else begin
          if borderx[0] gt 0 then array[0:borderx[0], *] = edvalue
          if borderx[1] lt (nx-1) then array[borderx[nborderm]:(nx-1), *] = edvalue
          for i = 0, nborderm do array[borderx[i], 0:borderbot[i]] = edvalue
          for i = 0, nborderm do array[borderx[i], bordertop[i]:(ny-1)] = edvalue
        endelse
      endelse
      put_array, get_maxstr()-1, array, -1
;     Code from edit_save_button_event above.
    endif
  endfor
end

pro editstr, strin, stroutin, xpin, ypin, xvalue = xin, yvalue = yin, $
         xsize = xsizein, ysize= ysizein,  $
         keep_aspect = keep_asp, group_leader = group, modal = modal, $
         ellipse= ellipse, circle = circle, invertit = invertit, $
         average = average, edit_value = edit_value, noedit = noedit, $
         surface_fit = surface_fit, user_value = uservalue, levels = lev, _extra =extrastuff
;
;       return on an error
COMPILE_OPT strictarr
      on_error, 1
@com.dat
   common editstr_data_com
   common editstr_widget_com
      if n_elements(initwdf) eq 0 then wdfinit
   common editstr_cleanup, oldwin, pixwin, initbx, initby, initbp
@qscom.dat
;  Common block
@qsstrcom.dat
;
   common pffcom, initpff, pff_exe
@draw_defaults.com

   if n_elements (structinit) eq 0 then qsstruct_init
   if n_elements (qsplotinit) eq 0 then qsinit
   if n_elements (initpff) eq 0 then init_pff else $
      if initpff eq 0 then init_pff
;
   npar= n_params()
   if npar eq 0 then begin
     print,'% Format: ', $
    'EDITSTR, STRin [, STRout] [, X, Y] [, XVALUE = xvalue] [, YVALUE = yvalue]'
     print,'        [, USER_VALUE = uservalue] [, SURFACE_FIT = surface_fit]'
     print,'        [, ELLIPSE = ellipse] [, CIRCLE = circle] [, AVERAGE = average]'
     print,'        [, XSIZE = xsize]  [, YSIZE= ysize] [, INVERTIT = invert]'
     print,'        [, KEEP_ASPECT = keep_aspect] [, _EXTRA = extrastuff]'
     return
   endif

   if n_elements(preview) eq 0 then preview = -1

   arr = get_valid_structure(strin, 'EDITSTR', ierror = ier, lcstr = lcstr)
   if ier ne 0 then return
   if arr.type ne 1  and arr.type ne 8 then begin
     Print, '%EDITSTR: Structure array must be a field structure'
     return
   endif
   if n_elements(xsizein) eq 0 then begin
     if n_elements(xsize) eq 0 then $
       if initpff gt 3 then xsize = medsize[0]*5/6 else xsize = medsize[0]
   endif else xsize = xsizein
   if n_elements(ysizein) eq 0 then begin
     if n_elements(ysize) eq 0 then $
       if initpff gt 3 then ysize = medsize[1]*5/6 else ysize = medsize[1]
   endif else ysize = ysizein

   pmulti_save=!p.multi
   !p.multi = 0

   oldwin = !d.window

   if oldwin ge 0 then begin
     initbx = !x
     initby = !y
     initbp = !p
   endif
   window, /free , /PIXMAP  , xsize = xsize, ysize = ysize
   pixwin = !d.window
   pmulti_save=!p.multi 
   !p.multi = 0
   if arr.block gt 1 then $
        Print, '%EDITSTR: Structure will be put on a single block'
   if n_elements(extrastuff) gt 0 then extra_in=extrastuff else $
     undefine, extra_in
   if n_elements(lev) gt 0 then win_levels = lev else win_levels=0
   levels_in = win_levels
   plotfld, arr, farray = array, xarray= x, yarray= y, $
          xlabel = xlabel, ylabel = ylabel, levels= win_levels,  $
          nfarray = 256,cntour = 2,  _extra = extra_in
   nx = n_elements(x)
   ny = n_elements(y)
   if (n_elements(array) lt 4) || (nx lt 2) || (ny lt 2) then begin
     wdelete, pixwin
     !p.multi = pmulti_save
     if oldwin ge 0 then  begin
       wset, oldwin
       !x = initbx
       !y = initby 
       !p = initbp
     endif
     return
   endif
   check_uniform, x, strdx
   check_uniform, y, strdy
   ok = where(strpos(tag_names(arr), 'DX') ge 0 , nok)
   if nok gt 0 then begin
     if (strdx gt 0) and (arr.dx1 gt 0) then begin
       if abs((arr.dx1[0]-strdx)/strdx) lt 3.0517578e-05 then $
         strdx = arr.dx1[0] else strdx =0.0
     endif
     if (strdy gt 0) and (arr.dx2 gt 0) then begin
       if abs((arr.dx2[0]-strdy)/strdy) lt 3.0517578e-05 then $
         strdy = arr.dx2[0] else strdy = 0.0
     endif
   endif
   if strdx eq 0.0 then dx = x[1:nx-1]-x[0:nx-2] else dx = strdx
   if strdy eq 0.0 then dy = y[1:ny-1]-y[0:ny-2] else dy = strdy

   mode = 0
   if n_elements (group) eq 0 then group = 0
   clabel = arr.clab
   strfile = arr.file
   arr2str, maxstructure-1, array, x, y, xlabel = xlabel, ylabel = ylabel, $
        clabel = clabel, file = strfile, tlabel= arr.tlab, $
        dx = strdx, dy = strdy, v1lab= arr.v1lab[0], spare = arr.spare
   indx =0
   if n_elements(uservalue) gt 0 then begin
     lcedval = 2
     uservalue = float(uservalue[0])
   endif else begin
     lcedval = (keyword_set(surface_fit)) ? 1 : 0
     uservalue = 0.0
   endelse
   edvalue = uservalue
   if keyword_set(circle) then lcshape = 1 else $
     if keyword_set(ellipse) then lcshape = 2 else lcshape = 0
   if lcshape eq 1 then begin
     if size(/type, circle[0]) gt 3 then begin
       cir_radius = circle[0] * 0.5
       cir_center = get_center(array)
     endif else cir_radius = 0.0
   endif
   inver = keyword_set(invertit) ? 1:0
   nstr = n_elements(stroutin)
   average_val=0.0
   lcbatch =0
   if nstr gt 1 then begin 
     xp = stroutin
     if n_elements(xpin) gt 1 then yp = xpin
   endif else if nstr eq 1 then strout = stroutin
   if n_elements(xin) gt 1 then xp = xin
   npx = n_elements(xp)
   if n_elements(ypin) gt 1 then yp = ypin
   if n_elements(yin) gt 1 then yp = yin
   npy = n_elements(yp)
   if npx gt 1 then begin
     if (npx eq npy) then begin
       xin = xp
       yin = yp 
       lcbatch = 1
     endif else begin
       print, '% EDITSTR: Number of X and Y points must be equal'
       print, '           X and Y points:', nstr, npx
       return
     endelse
   endif
   if lcbatch ne 0 then begin 
;     batch mode
      lcnobatch = 0
      wdelete, pixwin
      editstr_batch, xin, yin, noedit = noedit
; batch mode from end of main routine       
      if lcstr eq 0 then begin
        if npar eq 2 || npar eq 4 then begin
          stroutin = get_save_str(strout, maxstructure-1)
          if stroutin ne (maxstructure-1) then xfrstr, maxstructure-1, stroutin
        endif else xfrstr, maxstructure-1, strin
      endif else begin
        if npar eq 2 || npar eq 4 then begin
          if arg_present(stroutin) then s2i, stroutin,  maxstructure-1, /no_copy else begin
            if arg_present(strin) then  s2i, strin,  maxstructure-1, /no_copy
          endelse
        endif else if arg_present(strin) then strin = temporary(arr)
      endelse
;     end of batch material from the main routine
      average = average_val
      edit_value = edvalue
      return
    endif
    image = [0, 0, xsize, ysize, 0, 0, pixwin]

    lcnobatch = 1
  tlbase = WIDGET_BASE(GROUP_LEADER=Group, $
      COLUMN=1, $
      MAP=1, modal = modal, $
      TITLE='Edit Image')

  BASEINS  = WIDGET_BASE(tlbase, $
      COLUMN = 1, $
      MAP=1, $
      UVALUE = 'BASEINS')

 labelins1 =  WIDGET_LABEL( BASEins, $
      UVALUE='LABEL', $
      VALUE = $
 'Left Mouse = Designate Point,     Middle Mouse = Save,     Right Mouse = Reset')

; Base for the points, image parameters, and edit parameters
  BASE0 = WIDGET_BASE(tlbase, $
      row=1, $
      FRAME=0, $
      MAP=1, $
      UVALUE='BASE0')

; Base for the first point
  BASE1 = WIDGET_BASE(base0, $
      column=1, $
      FRAME=2, $
      MAP=1, $
      event_pro = 'Editstr_Point_Event', $
      UVALUE='BASE1')

; Base for the second point
  BASE2 = WIDGET_BASE(base0, $
      FRAME=2, $
      MAP=1, $
      UVALUE='BASE2')

; Base for the image parameters
  BASE3 = WIDGET_BASE(base0, $
      column=1, $
      FRAME=2, $
      MAP=1, $
      UVALUE='BASE3')

; Base for the edit parameters
  BASE3B = WIDGET_BASE(base0, $
      column=1, $
      FRAME=2, $
      MAP=1, $
      UVALUE='BASE3B')

; Base for the edit shape
  BASE3C = WIDGET_BASE(base0, $
      column=1, $
      FRAME=2, $
      MAP=1, $
      UVALUE='BASE3C')

; Base for the commands
  BASE4 = WIDGET_BASE(tlbase, $
      ROW = 1, $
      MAP=1, $
      UVALUE = 'BASE4')

; Base for the image
  BASE5 = WIDGET_BASE(tlbase, $
      ROW = 1, $ 
      MAP=1, $
      event_pro = 'Editstr_Point_Event', $
      UVALUE = 'BASE5')

; Base for the Edit/Save commands
  BASE7 = WIDGET_BASE(BASE4, $
      ROW = 1, $
      MAP=1, $
      FRAME = 2, $
      event_pro = 'EDIT_SAVE_BUTTON_EVENT', $
      UVALUE = 'BASE7')

; Base for the Reset/Exit commands
  BASE8 = WIDGET_BASE(BASE4, $
      ROW = 1, $
      MAP=1, $
      FRAME = 2, $
      event_pro = 'EditRESET_BUTTON_EVENT', $
      UVALUE = 'BASE8')

      if lcshape eq 1 &&  cir_radius gt 0.0 then begin
        ifloor = floor(cir_center)
        frac = cir_center -ifloor
        itop = (ifloor+1) < ([nx, ny]-1)
        cirx= x[ifloor[0]]*(1.0-frac[0]) + frac[0]*x[itop[0]]
        ciry= y[ifloor[1]]*(1.0-frac[1]) + frac[1]*y[itop[1]]
        px = cirx-cir_radius
        py = ciry-cir_radius
        qx = cirx+cir_radius
        qy = ciry+cir_radius
        mode =0
        npselect = 2
        lcmoveit = 1 
      endif else begin
        px = 0.0
        py = 0.0
        qx = 0.0
        qy = 0.0
        lcmoveit = 0 
        npselect = 0
      endelse
   label1  =  WIDGET_LABEL( BASE1, $
      UVALUE='LABEL', $
      VALUE='First Point')

   xwindow1 = CW_FIELD( BASE1,VALUE=px, $
      ROW=1, xsize = 12, $
      FLOAT=1, $
      RETURN_EVENTS = 1, $
      TITLE='X-Value:', $
      UVALUE='XVALUE1')

   ywindow1 = CW_FIELD( BASE1,VALUE=py, $
      ROW=1, xsize = 12, $
      FLOAT=1, $
      RETURN_EVENTS = 1, $
      TITLE='Y-Value:', $
      UVALUE='YVALUE1')

   bset1 = WIDGET_BASE(BASE1, $
      ROW = 1, $
      MAP=1, $
      UVALUE = 'BSET1')

   set1= WIDGET_BUTTON( Bset1, $
      UVALUE='BUTTON1', $
      VALUE=' Set Point 1 Value ')

  BASE21 = WIDGET_BASE(base2, $
      column=1, $
      MAP=1, $
      event_pro = 'Editstr_Point_Event', $
      UVALUE='BASE21')

   label2  =  WIDGET_LABEL( BASE21, $
      UVALUE='LABEL', $
      VALUE='Second Point')

   xwindow2 = CW_FIELD( BASE21,VALUE=qx, $
      ROW=1, xsize = 12, $
      FLOAT=1, $
      RETURN_EVENTS = 1, $
      TITLE='X-Value:', $
      UVALUE='XVALUE2')

   ywindow2 = CW_FIELD( BASE21,VALUE=qy, $
      ROW=1, xsize = 12, $
      FLOAT=1, $
      RETURN_EVENTS = 1, $
      TITLE='Y-Value:', $
      UVALUE='YVALUE2')

   bset2 = WIDGET_BASE(BASE21, $
      ROW = 1, $
      MAP=1, $
      UVALUE = 'BSET2')

   set2= WIDGET_BUTTON( BSET2, $
      UVALUE='BUTTON2', $
      VALUE=' Set Point 2 Value ')
;
;  Image parameters
   label3  =  WIDGET_LABEL( BASE3, $
      UVALUE='LABEL', $
      VALUE='Image Parameters')

   avewin = CW_FIELD( BASE3,VALUE=0.0, $
      ROW=1, xsize = 12, $
      FLOAT=1, /NOEDIT, $
      TITLE='Average:', $
      UVALUE='AVERAGE')

   minwin = CW_FIELD( BASE3,VALUE=0.0, $
      ROW=1, xsize = 12, $
      FLOAT=1, /NOEDIT, $
      TITLE='Minimum:', $
      UVALUE='MINIMUM')

   maxwin = CW_FIELD( BASE3,VALUE=0.0, $
      ROW=1, xsize = 12, $
      FLOAT=1, /NOEDIT, $
      TITLE='Maximum:', $
      UVALUE='MAXIMUM')

;  Edit parameters
   label3B  =  WIDGET_LABEL( BASE3B, $
      UVALUE='LABEL', $
      VALUE='Edit Value')

   editsrc = CW_BGROUP( BASE3B,['Border Value', 'Surface Fit', 'User Value'], $
      /exclusive, /column, /no_release, $
      frame = 0, set_value = lcedval, UVALUE='ESOURCE')
   
   beditval = widget_base(base3b, sensitive = (lcedval eq 2))
   editval = CW_FIELD( beditval,VALUE=edvalue, $
      column=1, xsize = 12, /return_events,  $
      FLOAT=1, TITLE='Edit value:', $
      UVALUE='EDITVAL')

;  Edit Shape
   label3C  =  WIDGET_LABEL( BASE3C, $
      UVALUE='LABEL', $
      VALUE='Edit Shape')

   editshp = CW_BGROUP( BASE3C,['Rectangle', 'Circle', 'Ellipse'], $
      /exclusive, /column, /no_release, $
      frame = 0, set_value = lcshape, UVALUE='ESHAPE')
   invbase = widget_base (BASE3C, frame = 1 , /nonexclusive)
   invbut = widget_button(invbase, value='Invert', uvalue = 'INVERT_BUTTON', $
     /pushbutton_events, /sensitive, /frame )
    
  base7b = widget_base(base7, /frame, /nonexclusive)
 
  WDFwindow = widget_button(base7b, UVALUE = 'MOVEIT', $
      value = 'Move Region' )

  editb = widget_button(base7, $
      UVALUE= 'EDIT_BUTTON', /no_release, $
      VALUE= ' EDIT REGION ')

  SAVEB= WIDGET_BUTTON( base7, $
      UVALUE='SAVE_BUTTON', /no_release, $ 
      VALUE='Accept / Save Image')

  Btns439 = [ $
    ' Reset All Points ', '   UNDO   ', $
    ' Done  / Exit ' ]
  ard_buttons = CW_BGROUP( base8, Btns439, $
      ROW=1,  /NO_RELEASE, $
      UVALUE='buttons')

  if n_elements(tlb_offset) eq 0 then tlb_offset = [10, 20]
  if n_elements(line_offset) eq 0 then $
       line_offset = [tlb_offset[0]+medsize[0]+30, smallsize[1]]

  xpreviewsize = smallsize
  if n_elements (xpreviewsize) eq 0 then xpreviewsize = smallsize


  if keyword_set(keep_asp) then begin
    win_aspect = !d.x_size *  $
    (!x.window[1]-!x.window[0])/!d.y_size/(!y.window[1]-!y.window[0]) * $
    (!y.crange[1]-!y.crange[0])/(!x.crange[1]-!x.crange[0])
    if keep_asp lt 0 then win_levels = (n_elements (lev) eq 2) ? lev : $
      [min(win_levels, max = mlev), mlev] else win_levels = 0
  endif else begin
    win_aspect = 0
    win_levels = 0
  endelse
  xdraw_range = !x.crange
  ydraw_range = !y.crange
  win_aspect = (keyword_set(keep_asp)) ? !d.x_size *  $
    (!x.window[1]-!x.window[0]) /!d.y_size/ (!y.window[1]-!y.window[0]) * $
    (!y.crange[1]-!y.crange[0])/(!x.crange[1]-!x.crange[0]) : 0
  nx = n_elements(x)
 
  ny = n_elements(y)
  
  check_uniform, x, strdx
  check_uniform, y, strdy
  ok = where(strpos(tag_names(arr), 'DX') ge 0 , nok)
  if nok gt 0 then begin
    if (strdx gt 0) and (arr.dx1 gt 0) then begin
      if abs((arr.dx1[0]-strdx)/strdx) lt 3.0517578e-05 then $
        strdx = arr.dx1[0] else strdx =0.0
    endif 
    if (strdy gt 0) and (arr.dx2 gt 0) then begin
      if abs((arr.dx2[0]-strdy)/strdy) lt 3.0517578e-05 then $
        strdy = arr.dx2[0] else strdy = 0.0
    endif
  endif 
  if strdx eq 0.0 then dx = x[1:nx-1]-x[0:nx-2] else dx = strdx
  if strdy eq 0.0 then dy = y[1:ny-1]-y[0:ny-2] else dy = strdy
  arr2str, maxstructure-1, array, x, y, xlabel = xlabel, ylabel = ylabel, $
        clabel = clabel, file = strfile, tlabel= arr.tlab, $
        dx = strdx, dy = strdy, v1lab= arr.v1lab[0], spare = arr.spare
 
  if nx eq 0 or ny eq 0 then begin
    !p.multi = pmulti_save
    if oldwin lt 0 then return
    wset, oldwin
    !x = oldbx
    !y = oldby
    !p = oldbp
    return
  endif 
  image = [0, 0, xsize, ysize, 0, 0, pixwin]
  mode = (npselect eq 0) ? 1 : 0
  DRAW = WIDGET_DRAW( base5, $
      BUTTON_EVENTS=1, $
      MOTION_EVENTS=mode, $
      RETAIN=1, $
      UVALUE='DRAW', $
      XSIZE=xsize, $
      YSIZE=ysize)
  
  WIDGET_CONTROL, tlbase, /REALIZE, tlb_set_xoffset = tlb_offset[0], $
            tlb_set_yoffset = tlb_offset[1]
  widget_control, base7, map = 0
  widget_control, draw, draw_motion_events = mode
  widget_control, invbut, set_button = inver
; Get drawable window index
  WIDGET_CONTROL, DRAW, GET_VALUE=DRAW_Id
  mksymbol, 'plus', /fill, size = 1.5
  loadxyct, /quiet, /freeze
  if keyword_set(printit) then print_var =1 else print_var = 0
  wset , draw_id
  device , copy = image
  oldbx = !x
  oldby = !y
  oldbp = !p
  if n_elements(preview) gt 1 then begin
    line_id = 1
    ypreview = preview[0:1]
  endif else if preview ne 0 then begin
    line_id = 1
    if preview lt 0 then ypreview = 0 else begin 
      ypreview=[min(array,max=mm), mm]
      delta = (ypreview[1]-ypreview[0])* 3.0517578e-05
      ypreview=ypreview+[delta, -delta]
    endelse
  endif else line_id = -1
  
  if line_id gt 0 then begin
    window, /free, xsize= xpreviewsize[0], ysize = xpreviewsize[1], $
      title = 'Current Windowed Image', xpos=line_offset[0], ypos=line_offset[1]
    line_id = !d.window
    device, get_window_position = line_offset_actual
    window_offset = line_offset-line_offset_actual
  endif else  Window_offset = [0,0]
  wset, draw_id
  !x = oldbx
  !y = oldby
  !p = oldbp
  npwdf = 0
  arr_aver = 0.0
  arr_min = 0.0
  arr_max = 0.0
  if npselect eq 2 then begin
    widget_control, base7, map =1
    widget_control, WDFwindow, /set_button
    editstr_doit
  endif
  wdfarr= make_array(1024, /int, value = 0)
  XMANAGER, 'Editstr', tlbase, cleanup='editstr_cleanup'
  array = 0
; If changese are made below - fix batch mode
  if lcstr eq 0 then begin
    if npar eq 2 || npar eq 4 then begin
      stroutin = get_save_str(strout, maxstructure-1)
      if stroutin ne (maxstructure-1) then xfrstr, maxstructure-1, stroutin
    endif else xfrstr, maxstructure-1, strin
  endif else begin
    if npar eq 2 || npar eq 4 then begin
      if arg_present(stroutin) then s2i, stroutin,  maxstructure-1, /no_copy else begin
        if arg_present(strin) then  s2i, strin,  maxstructure-1, /no_copy 
      endelse
    endif else if arg_present(strin) then strin = temporary(arr)
  endelse
; end of batch material
  arr = 0
  wdelete, pixwin
  !p.multi = pmulti_save
  line_offset = line_offset +window_offset
  average = average_val
  edit_value = edvalue
  if oldwin lt 0 then return
  wset, oldwin
  !x = initbx
  !y = initby
  !p = initbp
END
