; ------------------------------------------------------------------------
; $Id: fe2rec.pro,v 1.1 2009/09/02 16:27:20 lpmix Exp $
; ------------------------------------------------------------------------
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
;+
; NAME:
;       FE2REC
;
; PURPOSE:
;       Convert a finite element (FE) type dataset to a rectilinear dataset.
;       The FE data must have come from a rectilinear mesh, but the data may
;       be in any order.  Each row of data must be of the form: 
;       (Xval, Yval, Value) or (Xval, Yval, Zval, Value) with NPOINT rows.
;
; CATEGORY:
;       Data Conversion
;
; CALLING SEQUENCE:
;
;       FE2REC, Datain, ValueOut, Xout, Yout, Zout
;
; INPUTS:
;       Datain:	A matrix with the input data.  The vector is of the form:
;               DATAIN = DATAIN [ NDIM+1, NPOINTS]
;               Each row of DATAIN contains the spatial values for each dimension
;               and the last row is the field variable at those coordinates.
;               An alternative form is DATAIN = DATAIN [NPOINTS, NDIM+1 ]
;         Note: This form is available directly from the READASC with the 
;               MATRIX keyword.  (READASC, FILENAME, NDIM+1, DATAIN, /MATRIX)
;         Note: If you already have linear vectors, XIN, YIN, VALIN for a
;               two dimensional field:
;                 DATAIN = Reform([xin, yin, valin], n_elements(xin), 3)
;               This will generate an input in the alternative form.
;
; OUTPUTS:
;	ValueOut: The two or three dimensional vector with the field values
;       Xout:   Values for the first spatial dimension
;       Yout:   Values for the second spatial dimension
;       Zout:   Values for the third spatial dimension if three dimensional
;               Otherwise, ZOUT =0
;        Note:  If these variables are not present, the data will be stored in 
;               GET_MAXSTR()-1
;	
; KEYWORD PARAMETERS:
;       NXYZ:	See PROCEDURE.  If the number of values in each dimension are known, NXYZ is 
;               used to specify these dimensions.  If NX, NY and NZ are the number
;               of values in the X, Y and Z dimensions then: NXYZ=[NX, NY, NZ]
;          CAUTION:  If NXYZ is present, the arrays will be mindlessly reformated.
;                    VERBOSE will provide a check.
;       DELTA:	See PROCEDURE.  Default error parameter if spatial coordinates are
;               not exactly equal.  Default: DELTA = 2e-5
;       ABSOLUTE:  See PROCEDURE  If present and greater than zero, then ABSOLUTE 
;               is used to determine if two spatial values are equal.  IF ABSOLUTE is 
;               a scalar, one value will be used for all coordinates.
;       STRUCTURE: If present, ValueOut, Xout, Yout, Zout will be sent to ARR2STR.
;               STURCTURE must be a valid structure number.
;       _EXTRA: Ignored unless STRUCTURE is a valid structure number.  If STRUCTURE is 
;               a valid number, then extra keywords are sent to ARR2STR.  Examples might
;               include, CLABEL, XLABEL, YLABEL, ZLABEL, etc.
;       VERBOSE: If present, details of the X, Y, Z arrays will be printed.
;
; RESTRICTIONS:
;       DATA must be from an actual rectilinear dataset.
;
; PROCEDURE:
;       1. Separate the X, Y and Z arrays and sort the values
;       2a. If NXYZ is specified then calculate the average X, Y and Z values
;       2b. Otherwise, try to separate X, Y and Z assuming values are exact
;           If values have some errors, then use DELTA or ABSOLUTE to provide an
;           error value around each of the coordinate values.
;           The error value is ABSOLUTE if it is present.
;           If not then:
;           ABSOLUTE = DELTA * MAX(DATA_RANGE, ABS(DATA_VALUE)
;           ABSOLUTE is calculated for each coordinate using the actual data.
;           Note:
;           The last coordinate is not checked.  Use VERBOSE if you want to 
;           see the range of each coordinae.
;        3. The X, Y, Z arrays are extracte and the field quantity is reordered 
;           to the appropriate ordering for rectilinear data.  If STRUCTURE is
;           specified, the arrays are stored in a structure.  If variables are not
;           available in the call, the arrays are stored in structure (GET_MAXSTR()-1)
;
; EXAMPLE:
;	Read image parameters from a file, FILE, in X, Y, Value format, and store them 
;       in structure 8, with a comment of 'X-Ray Lineouts'
;         READASC, FILE, 3, data, /matrix
;         FE2REC, DATA, STR = 8, file = file, clab ='X-Ray Lineouts'
;
;       Convert the arrays X, Y and Z which are the spatial coordinates and amplitude of a
;       rectilinear dataset to structure 7
;         FE2REC, Reform([X, Y, Z], n_elements(X), 3), STRUCTURE =7
;
; MODIFICATION HISTORY:
; 	Written by:	L. P. Mix, Jr., August 5, 2009
;
;-

pro FE2REC, Datain, ValueOut, Xout, Yout, Zout, NXYZ = nxyz, DELTA = delta, $
      ABSOLUTE = absolute, STRUCTURE = structure, CLABEL = clabel, $
      VERBOSE = verbose, _extra = extrastuff

on_error, 1

npar = n_params()
if npar eq 0 then begin
  print, '% Format:  FE2REC, Datain, ValueOut, Xout, Yout, Zout, NXYZ = nxyz, '
  print, 'DELTA = delta, ABSOLUTE = absolute, STRUCTURE = structure, _extra = extra'
  return
endif
if n_elements(datain) eq 0 then begin
  print, '% FE2REC: Finite Element data must be provided'
  return
endif

ss = size (datain)
if ss[0] ne 2 then begin
  print, '% FE2REC: Data matrix must be two dimensional'
  return
endif
ndim = min(ss[1:2], j)-1

if (ndim lt 2) or (ndim gt 3) then begin
  print, '% FE2REC: Data must be 2 or 3 dimensional data'
  return
endif

Z = 0L
if j eq 0 then begin
; format datain[ndim+1, npoint]
  tmp = transpose (datain)
  npoint = ss[2]
  x= tmp[*, 0]
  y= tmp[*, 1]
  if ndim eq 2 then value = tmp[*, 2] else begin
    z = tmp[*, 2]
    value = tmp[*, 3]
  endelse
  tmp =0
endif else begin
  npoint = ss[1]
  x= datain[*, 0]
  y= datain[*, 1] 
  if ndim eq 2 then value = datain[*, 2] else begin
    z = datain[*, 2] 
    value = datain[*, 3] 
  endelse
endelse


valueout = 0 & xout = 0 & yout =0 & zout =0 
if n_elements (delta) eq 0 then delta = 2.0e-5

ind = make_array(npoint, /long)

if n_elements(nxyz) eq ndim then begin
  nx = nxyz[0]
  ny = nxyz[1]
  if ndim eq 3 then nz = nxyz[2] else nz = 1
  lcxyz = 1
  if nx*ny*nz ne npoint then begin
    print, '% FE2REC: The product of NXYZ must equal the number of points'
    print, '  NPOINT, NX*NY*NZ:', npoint, nx*ny*nz
    return
  endif
endif else lcxyz = 0


isort = sort(x)
x = x[isort]
jsort= sort(y)
y = y[jsort]
ksort = sort(z)
z = z[ksort]

if lcxyz then begin
; User has provided the size of the rectilinear data
  nyz = ny*nz
  ii = lindgen(nx+1)*nyz
  xu = make_array(nx)
  for i = 0, nx-1 do xu[i] = total(x[ii[i]:ii[i+1]-1])
  if keyword_set(verbose) then begin
    range = make_array(nx)
    for i = 0, nx-1 do range[i] = max(x[ii[i]:ii[i+1]-1], min = m)-m
    print, 'Maximum range of X values is ' + strtrim(max(range), 2)
  endif 
  xu = xu/nyz
  nxz = nx*nz
  jj = lindgen(ny+1)*nxz
  yu = make_array(ny)
  for i = 0, ny-1 do yu[i] = total(y[jj[i]:jj[i+1]-1])
  if keyword_set(verbose) then begin
    range = make_array(ny)
    for i = 0, ny-1 do range[i] = max(y[jj[i]:jj[i+1]-1], min = m)-m
    print, 'Maximum range of Y values is ' + strtrim(max(range), 2)
  endif 
  yu = yu/nxz
  nxy = nx*ny
  kk = lindgen(nz+1)*nxy
  zu = make_array(nz)
  for i = 0, nz-1 do zu[i] = total(z[kk[i]:kk[i+1]-1])
  if keyword_set(verbose) then begin
    range = make_array(nz)
    for i = 0, nz-1 do range[i] = max(z[kk[i]:kk[i+1]-1], min = m)-m
    print, 'Maximum range of Z values is ' + strtrim(max(range), 2)
  endif 
  zu = zu/nxy
endif else begin
; We will determine the size of the rectilinear data
  ii = uniq(x)
  nx = n_elements(ii)
  xu = x[ii]

  jj = uniq(y)
  ny = n_elements(jj)
  yu = y[jj]

  if ndim eq 2 then begin
    nz = 1
    zu = z[0]
    kk = 0
  endif else begin
    kk = uniq(z)
    nz = n_elements(kk)
    zu = z[kk]
  endelse
  
  if (nx*ny*nz ne npoint) then begin
    if n_elements (absolute) gt 0 then begin
      absolute = absolute[0]
      if absolute eq 0 then begin
        print, '% FE2REC: Unable to convert data with ABSOLUTE = 0.0'
        return
      endif
      lcabsolute = 1
    endif else begin
;     scale Delta with MAX(DATA_RANGE, ABS(DATA_VALUE) to get ABSOLUTE
      lcabsolute = 0
      absolute = delta * max(abs([xu[0], xu[nx-1], xu[nx-1]-xu[0]]))
    endelse
    dx = xu[1:nx-1] - xu[0:nx-2]
    ok = where (dx lt absolute, nok)
    if nok gt 0 then begin
      i = 0L
      while i lt nok-1 do begin
        add = 1
        while ((i+add lt nok) && (ok[i] +add eq ok[i+add])) do add++
        ii [ok[i]:ok[i]+add] =ii[ok[i]+add]
        i += add
      endwhile
      if i eq nok-1 then ii[ok[i]:ok[i]+1] =ii[ok[i]+1]
      ii = ii[uniq(ii)]
      nx = n_elements(ii)
      dx = ii[1:nx-1]- ii[0:nx-2]
      if min(dx, max = m) ne m then begin
        print, '% FE2REC: Unable to get uniform X values'
        print, '  Uneven stride, Min/Max '+strtrim (min(dx), 2)+'/' +strtrim(m, 2)
        return
      endif
      nyz = m
      if ndim eq 2 then begin
        ny = m
        if (ny * nx ne npoint) then begin
          print, '% FE2REC: Unable to calculate X / Y values.'
          print, '  Number of X values= ' + strtrim (nx, 2)
          print, '  Number of Y values= ' + strtrim (ny, 2)
          print, '  Number of Z values= ' + strtrim (npoint, 2)
          print, '  NX * NY = ' + strtrim (nx*ny, 2)
          return
        endif
        yu = make_array(ny)
        jj = lindgen(ny+1)*nx
        for i = 0, ny-1 do yu[i] = total(y[jj[i]:jj[i+1]-1])
        yu = yu/nx
        if keyword_set(verbose) then begin
          range = make_array(ny)
          for i = 0, ny-1 do range[i] = max(y[jj[i]:jj[i+1]-1], min = m)-m
          print, 'Maximum range of Y values is ' + strtrim(max(range), 2)
        endif
        zu = 0.0
      endif
      xu = make_array(nx)
      ii = lindgen(nx+1) * nyz
      for i = 0, nx-1 do xu[i] = total(x[ii[i]:ii[i+1]-1])
      xu = xu/nyz
      if keyword_set(verbose) then begin
        range = make_array(nx)
        for i = 0, nx-1 do range[i] = max(x[ii[i]:ii[i+1]-1], min = m)-m
        print, 'Maximum range of X values is ' + strtrim(max(range), 2)
      endif
    endif
    dy = yu[1:ny-1] - yu[0:ny-2]
    if lcabsolute eq 0 then $
        absolute = delta  * max(abs([yu[0], yu[ny-1], yu[ny-1]-yu[0]]))
    ok = where (dy lt absolute, nok) 
    if nok gt 0 then begin
      i = 0L
      while i lt nok-1 do begin
        add = 1
        while ((i+add lt nok) && (ok[i] +add eq ok[i+add])) do add++
        jj [ok[i]:ok[i]+add] =jj[ok[i]+add]
        i += add
      endwhile
      if i eq nok-1 then jj[ok[i]:ok[i]+1] =jj[ok[i]+1]
      jj =jj( uniq(jj))
      ny = n_elements(jj)
      dy = jj[1:ny-1]- jj[0:ny-2]
      if min(dy, max = m) ne m then begin
        print, '% FE2REC: Unable to get uniform Y values'
        print, '  Uneven stride, Min/Max '+strtrim (min(dy), 2)+'/' +strtrim(m, 2)
        return
      endif 
      nxz = m
      jj = lindgen(ny+1)*nxz
      yu = make_array(ny)
      for i = 0, ny-1 do yu[i] = total(y[jj[i]:jj[i+1]-1])
      yu = yu/nxz
      if keyword_set(verbose) then begin
        range = make_array(ny)
        for i = 0, ny-1 do range[i] = max(y[jj[i]:jj[i+1]-1], min = m)-m
        print, 'Maximum range of Y values is ' + strtrim(max(range), 2)
      endif
    endif
    nxy = nx*ny
    if (npoint mod nxy) ne 0 then begin
      print, '% FE2REC: Current values of NX and NY are not consistent with NPOINT'
      print, '  NPOINT, NX, NY, NX*NY, NPOINT mod NX*NY: '+ strtrim(npoint, 2)+', '  $
       +', '+strtrim(nx, 2)+', '+ strtrim(ny, 2)+', '+ strtrim(nxy, 2) $
       +', '+ strtrim(npoint mod nxy, 2)
      return 
    endif
    if nz ne npoint/nxy then begin
      nz = npoint/nxy  
      if nz gt 1 then begin
        zu = make_array(nz)
        kk = lindgen(nz+1)*nxy
        for i = 0, nz-1 do zu[i] = total(y[kk[i]:kk[i+1]-1])
        zu = zu/nxy
        if keyword_set(verbose) then begin
          range = make_array(nz)
          for i = 0, nz-1 do range[i] = max(z[kk[i]:kk[i+1]-1], min = m)-m
          print, 'Maximum range of Z values is ' + strtrim(max(range), 2)
        endif
      endif else begin
        kk = [0, kk+1]
        if keyword_set(verbose) then print, 'Maximum range of Z values is 0.0'
      endelse
    endif else begin
      kk = [0, kk+1]
      if keyword_set(verbose) then print, 'Maximum range of Z values is 0.0'
    endelse
  endif else begin
    ii = [0, ii+1]
    jj = [0, jj+1]
    kk = [0, kk+1]
    if keyword_set (verbose) then print, 'Range of X, Y, & Z values are all zero'
  endelse
endelse
ind = make_array(npoint, /long)
xout = xu
yout = yu
zout = zu

for k = 0, nx-1 do ind[isort[ii[k] : ii[k+1]-1]] += k
for k = 0, ny-1 do ind[jsort[jj[k] : jj[k+1]-1]] += k*nx
for k = 0, nz-1 do ind[ksort[kk[k] : kk[k+1]-1]] += k*nx*ny
a = sort(ind)
valueout = reform(value[a], nx, ny, nz)
maxstr = get_maxstr()
lcstr = (n_elements(structure) eq 0) ? 0 : 1
if lcstr then  $
  structure = ((structure[0] lt 1) || (structure[0] gt maxstr)) ? maxstr-1 : structure[0] $
  else structure = maxstr-1

if lcstr gt 0 then begin
  arr2str, structure, valueout, xout, yout, zout, clabel = clabel,  _extra = extrastuff
  return
endif

if arg_present(valueout) && arg_present(xout)  && arg_present(yout) then begin
  if ndim eq 2 then return
  if arg_present(zout) then return
endif
if n_elements (clabel) eq 0 then CLABEL = 'FE2REC Array'
arr2str, structure, valueout, xout, yout, zout, clabel = clabel, _extra = extrastuff

end
