; ------------------------------------------------------------------------
; $Id: ffts.pro,v 1.5 2010/09/17 20:48:24 lpmix Exp $
; ------------------------------------------------------------------------
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
;+
; NAME:
;       FFTS
;
; PURPOSE:
;       Take the FFT of a 2D structure
;       Optionally, the fft can be taken along the X or Y direction
;
; CALLING SEQUENCE:
;
;       FFTS, STR1in [, STR1out] [, STR2out] 
;
; INPUTS:
;       STR1in:	Structure for the FFT
;               Structure must be two dimensional and uniform in the direction of the fft.
;               Structure may be filtered before the fft.  Actual values passed to the FFT 
;               available in STRFFT
;
; OPTIONAL INPUTS:
;       STR1out: Structure array number for AMPLITUDE or REAL part of FFT
;                This is also saved in get_maxstr()-1
;       STR2out: Structure array number for PHASE or IMAGINARY part of FFT
;                This is also saved in get_maxstr()-2
;
; KEYWORD PARAMETERS:
;       FULL:      Full FFT including positive and negative modes/frequencies.
;                  DEFAULT:  Return only positive modes/frequencies.
;                            No scaling is performed to the calculated values. 
;       STRFFT:    Actual structure used for the fft.  
;                  This is stored in get_maxstr()-3.  If STRFFT has a valid 
;                  structure number greater than zero and less than GET_MAXSTR-3,
;                  the structure is stored in that structure number also.
;                  The size of the saved structure is the same as STR1in, regardless
;                  of the value of FAST, ie, padding is not included.  
;       AMPLITUDE: Return the amplitude of the FFT in STR1out
;                  The magnitude of the FFT is stored in (get_maxstr()-1)
;                  The phase angle of the FFT is stored in (get_maxstr()-2)
;                  If STR2out is a valid structure number, then the phase angle is  
;                  stored in STR2out
;                  If AMPLITUDE is set, REAL is ignored
;                  If REAL is not set, AMPLITUDE = 1 
;       DEGREE:    If set, return the phase angle in DEGREES.
;       REAL:      Return the real part of the FFT in STR1out
;                  The real part of the FFT is stored in (get_maxstr()-1)
;                  The imaginary part of the FFT is stored in (get_maxstr()-2)
;                  If AMPLITUDE is set, REAL is ignored.
;                  If REAL is not set, AMPLITUDE = 1
;                  If STR2out is a valid structure number, then the imaginary part
;                  is stored in STR2out.
;       FAST:      Points will be added to the array to make the number of points
;                  factorable by 2, 3, and 5.
;                  DEFAULT: FAST = 0 
;       MODE:      If set, the abscissa values are mode numbers
;       DELTA:     Abscissa increment of the FFT unless mode is set
;                    DELTA =  1.0/(NP * DELTAT)
;                           where NP is the number of points
;                           and DELTAT is the increment of the original data
;       HORIZONTAL: If set, perform FFT's at constant Y value
;                  Default: HORIZ = 0 ; perform a 2D FFT
;       VERTICAL:  If set, perform FFT's at constant X value
;                  Default: VERT = 0 ; perform a 2D FFT
;       SMOOTHW:   Default width for each fft if horizontal or vertical are set.
;                  SMOOTH(ARRAY, SMOOTHW) will be performed if a 2D FFT
;                  SMOOTH(ARRAY, [1, SMOOTHW]) if a horizontal FFT is performed
;                  SMOOTH(ARRAY, [SMOOTHW, 1]) if a vertical fft is performed
;                  DEFAULT:  SMOOTHW =3 ; (SMOOTHW must be odd)
;                  SMOOTHW = 0 for no average.
;                  IF SMOOTHW gt 0 then SMOOTHW = ((SMOOTHW > 3)/2) *2 + 1
;       POLYFIT:   If present with a value between 1 and 10 and if HORIZONTAL or 
;                  VERTICAL is set, then a least square fit will be performed with 
;                  the number of constants equal to POLYFIT.  If neither is set, 
;                  the keyword PDIRECTION will be checked.  The fit curve will be
;                  subtracted from the data before the FFT.
;                  POLYFIT = 1 ; Fit to a constant
;                  POLYFIT = 2 ; Linear fit 
;                  POLYFIT = 3 ; Quadratic Fit 
;                  etc.
;                  If the FFT is two dimensional, then POLYFIT may have as many values as 
;                  PDIRECTION.  If POLYFIT is a scalar, it is used for all fits.
;                  
;       PDIRECTION: Direction of POLYFIT if neither HORIZONTAL nor VERTICAL is set.
;                  PDIRECTION may be an array; The smoothing will be applied in the 
;                  order of the elements.  (HORIZONTAL = 1, VERTICAL = 2)
;                  PDIRECTION is ignored if HORIZONTAL or VERTICAL are set.  
;                  PDIRECTION =[1,2] would smooth horizontally and then vertically.
;                  NOTE:  IF POLYFIT is set and PDIRECTION is undefined then PDIRECTION = 1 
;                  NOTE:  PDIRECTION determines the number of filter passes on the array, 
;                         regardless of the number of elements in POLYFIT.
;                         For the J element of PDIRECTION, POLYFIT = POLYFIT[ J mod NP]
;                         where NP is the number of elements in POLYFIT.
;       DOUBLE:    If set, perform FFT in double precision.
;
; OUTPUTS:
;       The FFT of the structure is returned.
;       Only positive modes are returned 
;
; PROCEDURE:
;       The array is averaged if necessary and the IDL procedure FFT
;
; EXAMPLE:
;       Select a region of an image and take an fft in the vertical direction
;       and average the data over 11 pixels in the horizontal direction.
;       Assume the original image is in structure 8, store the subimage in 9,
;       and store the amplitude in structure 10
;         WINSTR, 8, save = 9
;         FFTS, 9, 10, SMOOTH = 11, /VERT, /AMPLITUDE
;
; MODIFICATION HISTORY:
; 	Written by:	L. P. Mix, Jr., August 20, 2007
;
;-

PRO FFTS, str1in, str1out, str2out, amplitude = amplitude, real = real, fast = fast, $
         mode = mode, degree = degree, delta = deltat, positive = positive, $
         horizontal = horizontal, vertical = vertical, smoothw = smoothw, full = full, $
         doublepre = doub, polyfit = polyfit, pdirection = pdirection, strfft = strfft

on_error, 1
;  Common block
@qsstrcom.dat
@qscom.dat
;

 if n_elements (structinit) eq 0 then begin
   print, '% FFTS: Structure arrays must be initialized'
   return
 endif

 arr = get_valid_structure(str1in, 'FFTS', ierror = ier)
 if ier ne 0 then return
 if arr.type ne 1 then begin
   print, '% FFTS: Structure array must be a uniform field structure'
   return
 endif

 if arr.ndim eq 3 and arr.two ne 3 then begin
   print, '% FFTS: Structure array must be two dimensional'
   return
 endif
 if arr.block ne 1 then begin
   print, '% FFTS: Structure array must be single block'
   return
 endif
 if n_elements(positive) eq 0 then positive = 1
 if keyword_set (horizontal) then lchor = 1 else lchor=0
 if keyword_set (vertical) then lcvert = 1 else lcvert = 0
 lctot = lchor+lcvert
 if lctot eq 2 then begin
   print, '% FFTS: Horizontal and vertical FFT cannot be simultaneously requested'
   return
 endif else if lctot eq 0 then lctwo = 1 else lctwo = 0

 if keyword_set(amplitude) then amp = 1 $
   else if keyword_set(real) then amp = 0 else amp= 1
 if arr.dx1 eq 0.0 then begin
   check_uniform, arr.x1, dx
   if (dx eq 0) && (lctwo +lchor gt 0) then begin
     print, '% FFTS: Structure must be uniform in the X direction'
     return
   endif
 endif else dx = (arr.dx1)[0]

 if arr.dx2 eq 0.0 then begin
   check_uniform, arr.x2, dy
   if (dy eq 0) && (lctwo +lcvert gt 0) then begin
     print, '% FFTS: Structure must be uniform in the Y direction'
     return
   endif
 endif else dy = (arr.dx2)[0]
 
 if lctwo gt 0 then begin
   if (dx eq 0) || (dy eq 0) then begin
     print, '% FFTS: Structure must have same uniform spacing in X and Y direction'
     return
   endif
 endif
 if n_elements(smoothw) eq 0 then smoothw = 3
 if smoothw ne 0 then smoothw = (2*(smoothw/2)+1) > 3
 array = reform(get_array(arr, -1))
 if smoothw gt 0 then begin
   if lctwo gt 0 then array = smooth(array, smoothw) else $
     if lcvert gt 0 then array = smooth(array, [smoothw, 1]) else $
     array = smooth(array, [1, smoothw])
 endif
 if lcvert eq 1 then dimension = 2 else if lchor eq 1 then dimension = 1 $
   else dimension = 0
 strfftsave = get_maxstr()-3
 i2s, arr, strfftsave
 sarr = size(array)
 if keyword_set(fast) then begin
   ss = sarr
   if lchor gt 0 or lctwo gt 0 then nx = f23(ss[1], /five) else nx = ss[1]
   if lcvert gt 0 or lctwo gt 0 then ny = f23(ss[2], /five) else ny = ss[2]
   temp = temporary(array)
   array = make_array(nx, ny)
   array[0:ss[1]-1, 0:ss[2]-1]= temp
 endif
 ss = size(array) 

 if dimension eq 0 then begin
   npoly = n_elements(polyfit)  
   if npoly gt 0 && (max(polyfit, min = minpoly) lt 11) && (minpoly gt 0) then begin
     if n_elements (pdirection) eq 0 then pdirection = 1
     ok = where (pdirection gt 0 and pdirection lt 3, nok)
     pdirection = (nok eq 0) ? 1 : pdirection[ok]
     nel = n_elements(pdirection)
     for idir = 0, nel-1 do begin
       m = round (polyfit[ idir mod npoly]) > 1
     if pdirection[idir] eq 1 then begin
;      Horizontal FFT (uniform in X)
       if m eq 1 then begin
         av = total(array, 1)/ss[1]
         for i = 0, ss[2]-1 do array[*, i] -= av[i]
       endif else begin
         xx = arr.x1
         for i = 0, ss[2]-1 do begin
           coef = svdfit(xx, reform(array[*, i]), m, /double, yfit = yfit, singular = sing)
           if sing gt 0 then begin
             print, '% FFTS: Horizontal fit at y = '+strtrim((arr.x2)[i], 2)+ ' is singular'
             print, '        Try reducing the value of polyfit'
             print, sing
             return
           endif
           array[*, i] -= yfit
         endfor
       endelse
     endif else begin
;      Vertical FFT (uniform in Y)
       if m eq 1 then begin
         av = total(array, 2)/ss[2]
         for i = 0, ss[1]-1 do array[i, *] -= av[i]
       endif else begin
         xx = arr.x2
         for i = 0, ss[1]-1 do begin
           coef = svdfit(xx, reform(array[i, *]), m, /double, yfit = yfit, singular = sing)
           if sing gt 0 then begin
             print, '% FFTS: Vertical fit at x = '+strtrim((arr.x1)[i], 2)+ ' is singular'
             print, '        Try reducing the value of polyfit'
             return
           endif
           array[i, *] -= yfit
         endfor
       endelse
     endelse
     endfor
   endif else if npoly gt 0 then begin
     print, '% FFTS: Invalid values for POLYFIT, Values must be in the range [1, 10]'
     print, '        Values =', polyfit
     return 
   endif
   if max(ss -sarr) eq 0 then put_array, strfftsave, array, -1 else $
     put_array, strfftsave, array[0:sarr[1]-1, 0:sarr[2]-1], -1
   array = fft(array, -1, double = doub)
 endif else begin
   if keyword_set (polyfit) && (polyfit[0] gt 0) && (polyfit[0] lt 11) then begin
     m  = round (polyfit[0]) > 1
     if lchor eq 1 then begin
;      Horizontal FFT (uniform in X)
       if m eq 1 then begin
         av = total(array, 1)/ss[1]
         for i = 0, ss[2]-1 do array[*, i] -= av[i]
       endif else begin
         xx = arr.x1
         for i = 0, ss[2]-1 do begin
           coef = svdfit(xx, reform(array[*, i]), m, /double, yfit = yfit, singular = sing)
           if sing gt 0 then begin
             print, '% FFTS: Horizontal fit at y = '+strtrim((arr.x2)[i], 2)+ ' is singular'
             print, '        Try reducing the value of polyfit'
             print, sing
             return
           endif
           array[*, i] -= yfit
         endfor
       endelse
     endif else begin
;      Vertical FFT (uniform in Y)
       if m eq 1 then begin
         av = total(array, 2)/ss[2]
         for i = 0, ss[1]-1 do array[i, *] -= av[i]
       endif else begin
         xx = arr.x2
         for i = 0, ss[1]-1 do begin
           coef = svdfit(xx, reform(array[i, *]), m, /double, yfit = yfit, singular = sing)
           if sing gt 0 then begin
             print, '% FFTS: Vertical fit at x = '+strtrim((arr.x1)[i], 2)+ ' is singular'
             print, '        Try reducing the value of polyfit'
             return
           endif
           array[i, *] -= yfit
         endfor
       endelse
     endelse
   endif  
   if max(ss -sarr) eq 0 then put_array, strfftsave, array, -1 else $
     put_array, strfftsave, array[0:sarr[1]-1, 0:sarr[2]-1], -1
   array = fft(array, -1, double = doub, dimension = dimension)
 endelse
 if keyword_set(strfft) then begin
   if size(strfft, /type) lt 5 then begin
     strfft = round(strfft)
     if strfft gt 0 and strfft lt strfftsave then xfrstr, strfftsave, strfft
   endif
 endif
 nx = ss[1]
 ny = ss[2]
 center = [nx/2+1, ny/2+1]
 
 if lchor gt 0 then begin
   dx2 = arr.dx2[0]
   x2 = arr.x2
   ylab = arr.x2lab[0]
   if keyword_set(full) then array = shift(array, -center[0], 0) else $
     array = array[0:nx/2, 0:ny-1]
;  array[1:*, *] = array[1:*, *]*2
   ss = size(array)
   x1 = keyword_set(full) ? (center[0]-ss[1]) + findgen(ss[1]) : findgen(center[0])
   deltat = 1./(nx*dx)
   if keyword_set(mode) then begin
     dx1 = 1.0
     xlab = 'FFT Mode Number (1 = '+strtrim(string(deltat), 2)+')'
   endif else begin
     dx1 = deltat
     x1 = x1*deltat
     xlab = 'FFT frequency'
   endelse
 endif else if lcvert gt 0 then begin
   dx1 = arr.dx1[0]
   x1 = arr.x1
   xlab = arr.x1lab[0]
   if keyword_set(full) then array = shift(array, 0, -center[1]) else $
     array = array[0:ss[1]-1, 0:ss[2]/2]
;  array[*, 1:*] = array[*, 1:*]*2
   ss = size(array)
   x2 = keyword_set(full) ? (center[1]-ss[2]) + findgen(ss[2]) : findgen(center[1])
   deltat = 1./(ny*dy)
   if keyword_set(mode) then begin
     dx2 = 1.0
     ylab = 'FFT Mode Number (1 = '+strtrim(string(deltat), 2)+')'
   endif else begin
     dx2 = deltat
     x2 = x2*dx2
     ylab = 'FFT frequency'
   endelse
 endif else begin
   if keyword_set(full) then array = shift(array, -center) else $
     array = array[0:nx/2, 0:ny/2]
   ss = size(array)
   x1 = keyword_set(full) ? (center[0]-ss[1]) + findgen(ss[1]) : findgen(center[0])
   x2 = keyword_set(full) ? (center[1]-ss[2]) + findgen(ss[2]) : findgen(center[1])
   deltax = 1.0/(nx*dx)
   deltay = 1.0/(ny*dy)
   if keyword_set(mode) then begin
     dx1 = 1.0
     dx2 = 1.0
     xlab = 'FFT Mode Number (1 = '+strtrim(string(deltax), 2)+')'
     ylab = 'FFT Mode Number (1 = '+strtrim(string(deltay), 2)+')'
   endif else  begin
     dx1 = deltax
     dx2 = deltay
     x1 = x1*dx1
     x2 = x2*dx2
     xlab = 'FFT X-frequency'
     ylab = 'FFT Y-frequency'
   endelse
 endelse
 deltat=[dx1, dx2]
 if keyword_set(amp) then begin
   clab1 = 'FFT Amplitude of '+arr.clab
   clab2 = 'FFT Phase of '+arr.clab
   arr2str, get_maxstr()-1, abs(array), x1, x2, dx = dx1, dy = dx2, clab = clab1, $
     xlab = xlab, ylab = ylab, v1lab='FFT Amplitude', file = arr.file
   deg= (keyword_set(degree)) ? !dtor : 1
   arr2str, get_maxstr()-2, atan(float(imaginary(array)), float(array))/deg, $
     x1, x2, dx = dx1, dy = dx2, clab = clab2, $
     xlab = xlab, ylab = ylab, v1lab='FFT Amplitude', file = arr.file
   
 endif else begin
   clab1 = 'Real FFT of '+arr.clab
   clab2 = 'Imaginary FFT of '+arr.clab
   arr2str, get_maxstr()-1, float(array), x1, x2, dx = dx1, dy = dx2, clab = clab1, $
     xlab = xlab, ylab = ylab, v1lab='FFT Amplitude', file = arr.file
   arr2str, get_maxstr()-2, imaginary(array), $
     x1, x2, dx = dx1, dy = dx2, clab = clab2, $
     xlab = xlab, ylab = ylab, v1lab='FFT Amplitude', file = arr.file
 endelse
 if n_elements (str1out) gt 0 then $
   if str1out gt 0 and str1out lt get_maxstr()-2 then xfrstr, get_maxstr()-1, str1out
 if n_elements (str2out) gt 0 then $
   if str2out gt 0 and str2out lt get_maxstr()-2 then xfrstr, get_maxstr()-2, str2out
 
END
