; ------------------------------------------------------------------------ 
; $Id: filtstr.pro,v 1.2 2011/08/18 15:36:52 lpmix Exp $
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
 Pro  FILTSTR, wdf1, wdf2, frequency = freq, atten = filter,  FAST = fast, $
      order = order, delta = delta, type = type, mode = mode, plotit = plotit, $
      width = widthin, band = fwidth, hp = hp, br = br, bp = bp, $
      low = low, high = high, five = five
;
;
;+
; NAME:
;       FILTSTR
;
; PURPOSE:
;       General purpose filter utility for images
;       Initial version will only have low and high pass filter.
;       (FILTSTR calls LOPSTR  or HIPSTR for actual filtering)
; CATEGORY:
;       Filter
;
; CALLING SEQUENCE:
;       FILTSTR, STR1 [, STR2][, TYPE=TYPE][, ATTEN=ATTEN][,ORDER = ORDER]
;            [, MODE = MODE] [,WIDTH = WIDTH][, /PLOTIT] [, FAST = fast]
;            [, FREQUENCY = FREQ][, LOW = LOW][, HIGH = HIGH]
;            [, DELTA = DELTA][, BAND=BAND] [, /HP] [,/BP] [, /BR]
;
; INPUTS:
;       STR1     Structure array number of the array to be filtered
;                Array should have two dimensions and be uniform in both dimensions.
;                STR1 is NX by NY with spacing DX, DY.
;                If structure is not uniform in both directions, it will be put on 
;                a uniform grid.
;
; OPTIONAL INPUT PARAMETERS:
;       STR2     WDF array number for storage of the filtered array
;
; KEYWORD PARAMETERS:
;       FAST     To avoid problems with long FFT's the number of points 
;                will be changed to speed the fft.  Zeros will be added to 
;                to pad the original image to the new size which will have 
;                factors of [2, 3] or [2, 3, 5] depending on keyword, FIVE.
;                Fast = 0 Do not modify the data
;                Fast not 0  Modify the data by adding extra points
;                Default: FAST = 0
;       FIVE     Ignored unless FAST is set.
;                If FAST is set and FIVE is set, then the number of points will 
;                have factors of [2, 3, 5].
;                If FAST is set and FIVE is not set, then the number of points
;                will have factors of only [2,3].
;       TYPE     Type of filter desired. Options are Butterworth, Exponential,
;                User supplied, and Ideal ('BUT', 'EXP', 'USER', 'IDEAL').
;                Only the first 3 letters are checked. Default is 'BUT'
;       ATTEN    Frequency domain filter function.  ATTEN = FILTER
;                in the equations below.  Must be supplied with
;                type = 'USER' and have same number of values as dataset STR.
;                CAUTION:  Do NOT set FAST for user mode.
;                X and Y coordinates should be modes 
;                X = [0, 1, 2, 3, ..., NX/2, .... 2, 1]
;                Y = [0, 1, 2, 3, ..., NY/2, .... 2, 1]
;                  = FINDGEN(N) < (N - FINDGEN(N)) ; where N is number of points
;       PLOTIT   If present, the frequency domain filter function is plotted
;                The filter function will be saved in GET_MAXSTR()-1
;                The filter is saved with abscissa in units of the mode number.
;                If PLOTIT is a two element array, then this is used as XRANGE/YRANGE
;                  in the PLOT command.
;                If PLOTIT is a one element array and negative then the filter
;                  function will be saved in -PLOTIT if it is a valid STR array.
;       ORDER    Order of the filter. Default is 2
;
;       MODE  or
;       DELTA or
;       FREQUENCY These three keywords specify the center frequency of the 
;                filter or the cut-off frequency and are related by the following:
;                MODE = (NP*DT)/DELTA = FREQ*NP*DT.
;                Only one is required, but may have 2 elements for the X and Y
;                directions.  
;                They are checked in the order given: MODE, DELTA, FREQUENCY.
;                The first specified keyword is used and any other keywords are ignored.
;                If none are provided, then default is MODE = 12.
;                If these are a scalar, the same value is used for X and Y directions.
;       /HP      Indicates a high pass filter, default is low pass filter
;
;   Note:  The following are not implemented.
;       WIDTH  or
;       BAND  or 
;       HIGH and LOW
;                These keywords specify the width of band rejection and band
;                pass filters and are related by the following:
;                WIDTH  = BAND*NP*DT = (HIGH-LOW)*NP*DT 
;                If HIGH and LOW are specified then MODE = (HIGH+LOW)/(2*NP*DT)
;                If none are specified, then default is WIDTH = 0.8*MODE
;       /BR      Indicates a band reject filter, default is low pass filter
;       /BP      Indicates a band pass filter, default is low pass filter
; OUTPUTS:
;       NONE     The filtered array is stored in a WDF array in the common block
; OPTIONAL OUTPUT KEYWORD PARAMETERS:
;       ATTEN    If TYPE is not 'USER', ATTEN contains the freq domain
;                filter function.
; RESTRICTIONS:
;       If type is 'USER', ATTEN must be provided
; PROCEDURE:
;       The Fourier transform of STR1 is taken and multiplied by a frequency
;       domain filter function.  The frequency domain filter function is the 
;       product of the filter function for the X and Y directions.  The 
;       inverse Fourier transform is taken, and the real values are stored in
;       STR2 (or STR1 if STR2 is not provided).
;
;       The X and Y filter functions are defined as follows:
;       Assume STR1 has NP points in each direction, ^ indicates an exponent, and 
;       DIST is an array of NP points defined as:
;       DIST = [0, 1, 2, 3, 4, 5,..., NP/2, ...,5, 4, 3, 2, 1]
;     The low pass filter is defined by the following:
;       Butterworth - FILTER = 1/(1 + (DIST / MODE) ^ (2 * ORDER))
;       Exponential - FILTER = EXP ( - (( DIST / MODE) ^ ORDER))
;       Ideal - FILTER = 1 for DIST < MODE, 0 for DIST > MODE
;     The high pass filter is defined by the following:
;       Butterworth - FILTER = 1/(1 + ( MODE / DIST ) ^ (2 * ORDER))
;       Exponential - FILTER = EXP ( - (( MODE / DIST ) ^ ORDER))
;       Ideal - FILTER = 0 for DIST < MODE, 1 for DIST > MODE
;     The band type filter is defined by the following where:
;       Butterworth - FILTER = 1/(1 +( DIST*WIDTH/(DIST^2-MODE^2))^(2 * ORDER))
;       (for band pass: FILTER = 1.0 - FILTER)
;       Exponential - FILTER =
;                     EXP( - (( (DIST^2-MODE^2) / (DIST*WIDTH) ) ^ (2*ORDER) ))
;       (for band reject: FILTER = 1.0 - FILTER)
;       Ideal - FILTER = 0 for DIST < MODE - WIDTH/2 or DIST > MODE + WIDTH/2
;                                             = 1, OTHERWISE
;       (for band reject: FILTER = 1.0 - FILTER
;       NOTE:
;       For exponential and butterworth band type frequency, the true center
;       mode, CM, is given by CM = MODE * SQRT (1 + (WIDTH/(2*MODE))^2).
;       For Butterworth filters, a DELTA equal to the full width at half 
;       maximum of a gaussian pulse will only attenuate the peak several percent
; EXAMPLES:
;       Filter array 1 with a low pass Butterworth filter with a delta 
;       of 15 ns and store the result in array 3.
;                FILTSTR, 1, 3, d= 15e-9
;       Filter array 1 with a 100 MHz low pass exponential filter.
;                FILTSTR, 1, f= 100e+6, t = 'exp'
;       Filter array 2 with a 12 cycle filter in the X-direction and a 3 cycle filter
;       in the Y-direction.
;                FILTSTR, 2, mode = [12, 3]
;       Filter array 2 with a low pass filter with the 3db set for 12 cycles.
;                FILTSTR, 2
;            or
;                FILTSTR, 2, MODE = 12
;       Filter array 2 with a low pass filter with the 3db point set for 20 cycles.  
;       Plot the filter function and return it in FARRAY.
;                FILTSTR, 2, m = 20, /plotit, atten = farray
;
; MODIFICATION HISTORY:
;      Original version, L. P. Mix, February 2, 2009
;
;-
;
@qsstrcom.dat
;
 on_error, 1
;
 if n_elements(structinit) eq 0 then qsstruct_init
 nparams = n_params()
 if nparams eq 0 then begin
   print,'% Format: FILTSTR, STR1 [, STR2] [, TYPE = type]'
   print,'          [, ATTEN = filter] [, ORDER = order] [, FAST = fast] '
   print,'          [, DELTA = delta] [, MODE = mode] [, FREQUENCY = freq]'
   print,'          [, WIDTH = width] [, LOW = low] [, HIGH = high]'
   print,'          [, BAND = fwidth] [, /PLOTIT ] [, /HP]'
;   print,'          [, BAND = fwidth] [, /PLOTIT ] [, /BR] [, /BP]'
   return
 endif
;
 if n_elements(wdf1) eq 0 then begin
   print, 'Parameters to FILTSTR must be defined'
   return
 endif
 arr = get_valid_structure(wdf1 ,'FILTSTR', lcstr=lcstr, ierror = ier)
 if ier then return
;
 save = (nparams gt 1) ? get_save_array(wdf2, wdf1) : wdf1
;
 if n_elements(order) gt 0 then order = order > 0.01 $
      else order = [2, 2]  
 if n_elements(order) lt 2 then order = [1,1]*order[0]
        
;     if X-Y data nel is not correct
 if arr.type ne 1 then begin
   print, '% FILTSTR: Structure array must be a uniform field structure'
   return
 endif
 if arr.ndim eq 3 and arr.two ne 3 then begin
   print, '% FILTSTR: Structure array must be two dimensional'
   return
 endif
 if arr.block ne 1 then begin
   print, '% FILTSTR: Structure array must be single block'
   return
 endif
 if arr.dx1 eq 0.0 then begin
   check_uniform, arr.x1, dx
   if dx eq 0 then begin
     print, '% FILTSTR: Structure must be uniform in the X direction'
     return
   endif
 endif else dx = (arr.dx1)[0]

 if arr.dx2 eq 0.0 then begin
   check_uniform, arr.x2, dy
   if dy eq 0 then begin
     print, '% FILTSTR: Structure must be uniform in the Y direction' 
     return
   endif
 endif else dy = (arr.dx2)[0]
 ss = arr.size[0:1]
;
 if n_elements(type) gt 0 then begin
   type = strtrim(type, 2)
   type = strupcase(type)
   type = strmid(type, 0, 3)
   case type of 
    'BUT' : filtertype = 'BUTTER'
    'EXP' : filtertype = 'EXPON'
    'IDE' : filtertype = 'IDEAL'
     else  : filtertype = 'USER'
   endcase
 endif else filtertype = 'BUTTER'

 if filtertype eq 'USER' then begin
   if n_elements(filter) eq 0 then begin
     print, '% FILTSTR: Filter must be defined for "USER" type'
     return
   endif
   nel = (size (filter))[1:2]
   if total(abs(nel-ss)) ne 0 then begin
     print, 'Filter has wrong number of elements:'
     print,'  Filter has elements = ' , nel
     print,'  Array has elements = ', ss
     return
   endif
 endif
 array= reform(arr.v1, ss)
 if (filtertype eq 'USER') then begin
   if n_elements(fast) eq 0 then fast = 0 
   if (fast ne 0) then begin
     fast = 0
     print, '% FILTSTR:  FAST must equal 0 for USER filtertype'
   endif
 endif else begin
   if n_elements(fast) eq 0 then fast = 0 
   if keyword_set(fast) then begin
     ss_new = [f23(ss[0], five = five), f23(ss[1], five = five)]
     new = make_array(ss_new)
     new[0:ss[0]-1, 0:ss[1]-1] = array
     array = temporary(new)
     ss = ss_new
   endif
 endelse
 totalx = ss[0]*dx
 totaly = ss[1]*dy
 total= [totalx, totaly]
; 
; if n_elements (delta) gt 0 then delta = abs (delta) > 0.1*array(1)
 if n_elements (mode) gt 0 then bin = mode > 1 $
   else if n_elements(delta) gt 0  then $
 bin = total/delta  else if n_elements(freq) gt 0 then $
 bin = abs(freq)*total else bin =  12
 if n_elements (bin) eq 1 then bin = bin*[1, 1]
; if keyword_set (bp) or keyword_set(br) then begin
;  n_elements (low) gt 0 and n_elements(high) gt 0 then begin
;     bin = (high + low)*0.5*total
;          width = abs (high-low)*total
;   endif else if n_elements (fwidth) gt 0 then width = fwidth*total else $
;   if n_elements (widthin) gt 0 then width = widthin else width= 0.8 * bin
;   if keyword_set (bp)  then bap, wdf1, save, mode=bin,width = width, $
;      plot = plotit, atten = filter, type = filtertype,order = order, $
;   wdf_array = array else $
;      bar, wdf1, save, mode=bin, width= width, plot= plotit, atten= filter, $
;          type = filtertype, order = order,  wdf_array = array
; endif else begin
 if keyword_set(hp) then $
   hipstr, array, mode = bin, xarray = xarray, yarray = yarray,  $
        atten = filter, type = filtertype, order = order else $
   lopstr, array, mode = bin, xarray = xarray, yarray = yarray,$
        atten = filter, type = filtertype, order = order
;      endelse
;
 if keyword_set (fast) then array = array[0:arr.size[0]-1, 0:arr.size[1]-1]
 if save ne wdf1 then xfrstr, wdf1, save

 put_array, save, array, -1
 if keyword_set(plotit) then begin
   nx = ss[0]
   nx2 = nx/2
   ny = ss[1]
   ny2 = ny/2
   center = [ nx2+1, ny2+1]
   x = shift(xarray, -center[0])
   x[0:nx -nx2 -2] = -x[0:nx-nx2-2]
   y = shift(yarray, -center[1])
   y[0:ny -ny2 -2] = -y[0:ny-ny2-2]
   sam = get_maxstr()-1
   arr2str, sam, shift (filter, -center), x, y, xlab = 'X-Modes', $
     clab = 'Low Pass Filter Function', ylab = 'Y-Modes'
   if n_elements (plotit) gt 1 then begin
     xrang = plotit[0:1]
     yrang = xrang
   endif else if plotit[0] lt 0 then begin
     sa = round(-plotit[0])
     if (sa gt 0) && (sa le get_maxstr()) then xfrstr, sam, sa
   endif
   plotstr, sam, xrange = xrang, yrange= yran
 endif
 end
