; ------------------------------------------------------------------------ 
; $Id: flipstr.pro,v 1.4 2007/06/06 20:09:34 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
 PRO flipstr, ain, save, clabel = clabel, rotate  = rotat, xflip = xflip, $
     yflip = yflip
;
;+
; NAME:
;	FlipSTR
;
; PURPOSE:
;	Flip or rotate an image
;       NOTE:  AXIS values are unchanged except in the case of a rotation, 
;              The X and Y axis will be switched if necessary.
;              If the axis is not uniform, the values will be flipped.
;              
;
; CALLING SEQUENCE:
;	FLIPSTR, STR, [, SAVE]  [, /XFlip ] [, /YFlip ]
;               [, ROTATE = Rotate_Code]
;
; INPUTS:
;	STR:	The image array to be flipped.  This is a field type structure
;               or structure array number. It must have two dimensions with 
;               uniform point spacing.
;	SAVE:	Structure array number for the flipped image.
;               If str is a structure array number, then the flipped
;                 image will be saved as a structure array number.
;               If str is a structure , then the flipped
;                 image will be saved as a structure.
;               Default: If SAVE is missing, flipped image will be returned
;                 in STR.
;       XFLIP:  Reverse the X-Direction (reverse(str, 1))
;       YFLIP:  Reverse the Y-Direction (reverse(str, 2))
;       ROTATE: Rotation code defined in IDL manual
;               1, 2, 3  : Counterclockwise Rotation by 90, 180 and 270 degrees 
;               4        : Transpose Array
;               5, 6, 7  : Transpose and Counterclockwise Rotation 
;                          by 90, 180 and 270 degrees
;               Note: Rotate = 5 is the same as XFlip
;               Note: Rotate = 7 is the same as YFlip
;               If ROTATE equal 5 or 7, then XFLIP or YFLIP will be used.
;
; OUTPUTS:
;	FLIPSTR returns a flipped version of the input image.
;
; PROCEDURE:
;	REVERSE and ROTATE are used to rotate the image.
;       If rotate equals 1, 3, 4, or 6 then the X and Y AXES are reversed.
;       Only one operation is performed on a call.  XFLIP, YFLIP and ROTATE 
;       are checked in order and the first valid command terminates the program.
;       If the coordinates are uniform, the values will not be changed.
;       If the coordinates are not uniform, the coordinates will be changed
;       according to the equations for x (Y is identical):
;         nx = N_elements(x)
;         x = reverse(x[0]+x[nx-1]-x)
;
; EXAMPLE:
;	Invert structure 1 in the y direction.
;		FLIPSTR,1, /y
;
;       Invert structure 2 in the X direction and store the result in 2
;		FLIPSTR,1, /x, 2
;  
; MODIFICATION HISTORY:
;       Original version: August 16, 2000, lpm
;
;-
;  Common block
@qscom.dat
@qsstrcom.dat

; return on an error
      on_error, 1
;
      if n_elements (structinit) eq 0 then qsstruct_init
      if n_elements (qsplotinit) eq 0 then qsinit
;
      nparams = n_params()
      if nparams eq 0 then begin
        print,'% Format: ', $
       'FLIPSTR, STR [, SAVE] [, /XFLIP] [, /YFLIP]
print,'               [, ROTATE = Rotation_code]
        return
      endif
;
      if n_elements(ain) eq 0 then begin
        print, '% FLIPSTR: Input structure must be defined'
        return
      endif
; change variables to insure we don't corrupt input data

      a = ain
;     if a structure input then set lcstrin
      a = get_valid_structure(ain, 'FLIPSTR', lcstr = lcstrin, ierror = ier)
      if ier ne 0 then return
;
      if a.type ne 1 then begin
        print, '% FLIPSTR: Only standard image structures can be flipped.'
        return
      endif
      if a.block  ne 1 then begin
        print, '% FLIPSTR: Only single block image structures can be flipped.'
        return
      endif
 
      if ((a.ndim eq 3 and a.two ne 3) or (a.ndim eq 4)) then begin
        print, '% FLIPSTR: Only two dimensional image structures can be flipped.'
        return
      endif

      if a.nvect ne 1 then begin
   print, '% FLIPSTR: Only structures with a single vector can be flipped.'
        return
      endif
        
      if n_elements (rotat) eq 0 then rot=0 else rot = round(rotat[0])
      ok = where(strpos(tag_names(a), 'DX') ge 0 , nokdx)
      lcx = 0
      lcy = 0
      if keyword_set (xflip) or rot eq 5 then begin
        v = reform(get_array(a, -1))
        put_array, a, reverse(v, 1), -1
        if (nokdx gt 0) && (a.dx1 gt 0) then lcx = 1
        if lcx eq 0 then begin
          x = get_array (a, 1)
          nx = n_elements(x)
          check_uniform, x, dx
          if dx eq 0 then $
            put_array, a, reverse(x[0]+x[nx-1]-x), 1
        endif
        rot =5
      endif else if keyword_set (yflip) or rot eq 7 then begin
        v = reform(get_array(a, -1))
        put_array, a, reverse(v, 2), -1
        if (nokdx gt 0) && (a.dx2 gt 0) then lcy = 1
        if lcy eq 0 then begin
          y = get_array (a, 2)
          ny = n_elements(y)
          check_uniform, y, dy
          if dy eq 0 then $
          put_array, a, reverse(y[0]+y[ny-1]-y), 2
        endif
        rot = 7
      endif else if rot gt 0 and rot lt 8 then begin
        v = reform(get_array(a, -1))
        v =  rotate(v, rot)
        ok = where(rot eq [1, 3, 4, 6], nok)
        x =  get_array (a, 1)
        nx = n_elements(x)
        y = get_array (a, 2)
        ny = n_elements(y)
        if (nokdx gt 0) then begin
          if (a.dx2 gt 0) then lcy = 1 else begin
            check_uniform, y, dy
            if dy gt 0 then lcy = 1
          endelse
          if (a.dx1 gt 0) then lcx = 1 else begin
            check_uniform, x, dx
            if dx gt 0 then lcx = 1
          endelse
        endif
        if rot eq 1 || rot eq 2 || rot eq 6  then $
          if lcy eq 0 then y =  reverse(y[0]+y[ny-1]-y)
        if rot eq 2 || rot eq 3 || rot eq 6  then $
          if lcx eq 0 then x = reverse(x[0]+x[nx-1]-x)
        if nok gt 0 then $
          arr2str, a, /structure, v, y, x, a.x3, clabel= a.clab, $
           dx = a.dx2, dy = a.dx1, dz= a.dx3, xlabel=a.x2lab, $
           ylabel = a.x1lab, zlabel = a.x3lab, v1lab = a.v1lab, file = a.file $
        else $
          arr2str, a, /structure, v, x, y, a.x3, clabel= a.clab, $
           dx = a.dx2, dy = a.dx1, dz= a.dx3, xlabel=a.x2lab, $
           ylabel = a.x1lab, zlabel = a.x3lab, v1lab = a.v1lab, file = a.file
      endif else begin
        print, '% FLIPSTR: No valid parameters'
        return
      endelse


      if nparams gt 1 then  begin
        if lcstrin eq 0 then begin
          save = get_save_str(save, ain)
          i2s, a, save, /no_copy
        endif else save = temporary(a)
      endif else begin
        if lcstrin eq 0 then i2s, a, ain, /no_copy else ain = temporary(a)
      endelse

END
