; ------------------------------------------------------------------------ 
; $Id: get_str_value.pro,v 1.4 2010/07/22 15:34:12 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
;+
; NAME:
;       GET_STR_VALUE
;
; PURPOSE:
;       Get field values at points 
;
; CATEGORY:
;       Input/Output
; CALLING SEQUENCE:
; (for a type 1 structure)
;       GET_STR_VALUE, A [, FLD] [, KVALUE1] [, VALUE1] [, TRANSPLOT = transplot]
;                  [, INDEX = index] [, KINTEGRATE = kintegrate] 
;                  , XARRAY = xarray [, YARRAY = yarray]
;                  , FARRAY = farray [, NFARRAY = nfarray] [, MISSING = missing]
;
; (for a type 12 structure) (see GET_STR12_VALUE)
;       GET_STR_VALUE, A [, /SELECT ] [, /Node] [, MISSING = missing]
;                  , XARRAY = xarray [, YARRAY = yarray] [, ZARRAY = zarray]
;                  , FARRAY = farray [, NFARRAY = nfarray]
; 
; INPUTS for Type 1 structure:
;       A     IDL field structure (a.type = 1 or 12) or QS array number
;                 
;       FLD   Field or attribute parameter to be plotted.
;       Note: FLD may be positive or negative; FLD = -(abs(FLD))
;       Note: If FLD is an array, the square root of the sum of 
;             the squares will be used.
;       Note: If A has only one attribute (such as an image or a charge density);
;             the this parameter can be omitted.
;       KVALUE1
;             If FLD is a two dimensional field then this value is ignored.
;             If FLD is a three dimensional field then this value
;               indicates the coordinates which are fixed for the two
;               dimensional plots.
;               For 3D plots, x and y are chosen according to:
;                  x= (kvalue1 mod 3)+1 and y = (kvalue1+1) mod 3 + 1
;       VALUE1
;             If FLD is a two dimensional field then this value is ignored
;             If FLD is a three dimensional field then this value
;               indicates the coordinate value for the fixed dimension in
;               the two dimensional plots. Linear interpolation is used
;               according to the value of VALUE1. If VALUE1 has two elements
;               the average value over the interval is used.
;         Note: If INDEX keyword is set, the units are taken as array indices
;               which range from 1 to the maximum value.
;         Note: If the two elements of VALUE1 span a block boundary, then only
;               the values in the block containing the midpoint will be used.
;               If the midpoint falls on a block boundary, then the result is 
;               probably not what you want!!
; KEYWORD PARAMETERS:
;       MISSING - If present, missing values will be set to "MISSING"
;       TRANSPLOT-If present and not zero, interchange ordinates and abscissa
;                 Default is a right hand coordinate system.
;       INDEX   - If present and not zero, then the units of VALUEi will be
;                 taken as indices of of the field array. Default INDEX = 0 
;                 Care should be exercised with multiple block data.
;       KINTEGRATE - If present and not zero, integrate the slice rather than 
;                 average over the extent of value1(0) to value1(1).
;                 The integral will be performed between value1(0) and value1(1)
;                 If integrate = 1, a simple integral
;                 If integrate = 2, a x dx integral
;                 If integrate = 3, a x^2 dx integral
;                 Limitations:
;                     Data must have a single block
;                     Two values of value 1 must exist
;
;       XARRAY, YARRAY - (Type 1) Abscissa and ordinate values for the array, FARRAY
;       XARRAY, YARRAY, ZARRAY - (Type 12) Coordinate values for the array, FARRAY
;                  If MISSING is provided, all values will be returned.
;                  XARRAY may be a [2, N] or [3, N] array with all coordinate 
;                  values in XARRAY
;       FARRAY   - Values of the field at the coordinates(xarray, yarray)
;       NFARRAY  - Elements are 0 or 1 depending on whether FARRAY has valid data.
;       NODE     - Type 12 only - uses nearest node value rather than interpolating 
;                  the tetrahedron.
;                           
; OUTPUTS:
;       FARRAY and NFARRAY are returned for the input values of XARRAY and YARRAY
;
; EXAMPLE:
;       Determine the values of the field in str 4 at the points (x, y) = 
;         (3, 4), (4, 5), (5, 6) for a z value = .35.  Store the values in val.
;         get_str_value, 4, 3, .35, xar= [3, 4, 5], yar = [4,5,6], far = val
;
; MODIFICATION HISTORY:
;      Original version, L. P. Mix, March 16, 1998
;      Added link to get_str12_value, July 15, 2010
;-
;
 Pro get_str_value, ain, field, kvlue1, vlue1, node = node, select = select, $
   kintegrate = kinteg, index = index,  $
   zarray = zarray,  missing = missin, xarray= xarrayin,  $
   yarray= yarray, farray= farray, nfarray = nfarray
;
;  Common block
@qscom.dat
@qsstrcom.dat
      on_error, 1
;
      if n_elements (structinit) eq 0 then qsstruct_init
      if n_elements (qsplotinit) eq 0 then qsinit
      npar = n_params()
      if npar eq 0 then begin
        print,'% Format: ', $
             'GET_STR_VALUE, A [, D] [, KVALUE1] [, VALUE1]'
print,'                [, INDEX = index] [, KINTEGRATE = kintegrate]  '
print,'                , XARRAY = xarray [, YARRAY = yarray]'
print,'                , FARRAY = farray [, NFARRAY = nfarray]'
print,'          < or >'
print,'       GET_STR_VALUE, A [, /SELECT ] [, /Node] [, MISSING = missing]'
print,'                 , XARRAY = xarray [, YARRAY = yarray] [, ZARRAY = zarray]'
print,'                 , FARRAY = farray [, NFARRAY = nfarray]'
        return
      endif

;
;     now get the data  -- check that we have a structure of the proper type
;
      if n_elements(ain) eq 0 then begin
        print, '% GET_STR_VALUE: Structure must be defined'
        return
      endif
;
      if not arg_present(farray) then begin
        Print, '% GET_STR_VALUE: FARRAY must be a simple variable to return values'
        return
      endif
; change variables to insure we don't corrupt input data
      a = get_valid_structure(ain,'GET_STR_VALUE', lcstr=lcstr, ierror = ier)
      if ier ne 0 then return
;
      b = size(a)
      if b(b(0)+1) ne 8 then begin
        print,'% GET_STR_VALUE: First parameter must be a structure or a valid QS array number'
        return
      endif
;
      if a.type eq 12 then begin
         get_str12_value, a, zarray = zarray, select = select, node = node, $
         missing = missin, xarray= xarrayin, yarray= yarray, farray= farray, $
         nfarray = nfarray
         return
      endif
      if a.type ne 1 then begin
        print, '% GET_STR_VALUE: Only type 1 and type 12 field structures may be used'
        a = 0
        return
      endif
;
; get a working array number
;
      if n_elements(field) gt 0 then fld = field
      if n_elements(vlue1) gt 0 then value1 = vlue1 
      if n_elements(kvlue1) gt 0 then kvalue1 = kvlue1 
      if n_elements (index) eq 0 then index = 0
      if n_elements (kinteg) eq 0 then kinteg = 0
;
;     reduce the structure to two dimensions with a single attribute
;
      structinit = -1
      a = slice (a, fld, kvalue1, value1, index=index, integ = kinteg )
      structinit = 1
;     slice makes a one attribute structure
      nelx = n_elements(xarrayin)
      nely = n_elements(yarray)
      if nelx eq 0 or nely eq 0 then begin
        if nelx gt 0 then begin
          if (nelx mod 2) eq 0 then begin
            nely = nelx/2
            xarray = reform (xarrayin, 2, nely)
            yarray = reform(xarray[ 1, *])
            xarray = reform(xarray[ 0, *])
            nelx = nely
          endif else begin
            print, '% GET_STR_VALUE: X and Y array values must be defined'
            help, xarrayin, yarray
            a=0
            return
          endelse
        endif else begin
          print, '% GET_STR_VALUE: X and Y array values must be defined'
          help, xarrayin, yarray
          a=0
          return
        endelse
      endif else xarray = xarrayin
      if  nelx ne nely then begin
        print, '% GET_STR_VALUE: X and Y arrays must have the same number of values'
        return
      endif
      farray = make_array (nelx)
      nfarray = make_array(nelx, /int, value =0)
      fld = -1
      nfld = 1
      block = indgen(a.block)+1
;
;
      if n_elements (transplot) eq 0 then transplot = 0
      if a.ndim eq 2 then begin
        xval = 1 & yval = 2 & value1 = 0
      endif else if a.ndim eq 3 then begin
        if kvalue1 eq 2 then begin
          if transplot eq 0 then transplot = 1 else transplot = 0
          xval = 1 & yval = 3
        endif else begin
          xval = (kvalue1 mod 3) + 1
          yval = ((kvalue1 + 1) mod 3) + 1
        endelse
      endif else begin
        print, '% GET_STR_VALUE: Only 2 and 3 dimensional data is allowed'
        return
      endelse
      if keyword_set (transplot) then begin
         temp= xval & xval= yval & yval= temp
      endif
;
;
;     Calculate the values by looping over blocks
; 
      for ii = 1 ,a.block  do begin
        iblock = ii
        cont = 0
        xrange = a.range(*, xval-1, ii)
        yrange = a.range(*, yval-1, ii)
        ok = where ( (xarray ge xrange(0)) and (xarray le xrange(1)) $
          and (yarray ge yrange(0)) and (yarray le yrange(1)), nok)
        if nok gt 0 then begin
          nfarray(ok) = 1
          cont = reform(get_array(a, fld, iblock), /overwrite)
          if transplot ne 0 then cont = transpose(temporary(cont))
          xx = get_array(a, xval, iblock, lab = xlab)
          yy = get_array(a, yval, iblock, lab = ylab)
          for inok = 0, nok-1 do begin
            stx= xarray[ok[inok]]
            sty= yarray[ok[inok]]
            ix = (where (stx le xx))[0]
            iy = (where (sty le yy))[0]
            if ix gt 0 then ix = ix-1+(stx-xx[ix-1])/(xx[ix]-xx[ix-1])
            if iy gt 0 then iy = iy-1+(sty-yy[iy-1])/(yy[iy]-yy[iy-1])
            farray[ok[inok]]= interpolate(cont, ix, iy)
          endfor
        endif
      endfor
      if (n_elements (missin) gt 0) then begin
        ok = where(nfarray eq 0, nok)
        if nok gt 0 then farray [ok] = missin[0]
      endif
      end
