; ------------------------------------------------------------------------ 
; $Id: get_sweep.pro,v 1.4 2007/06/06 20:09:34 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
 pro get_sweep, ain, wdf, minbin = minbin, smooth = smothin, slope = slope,  $
      horizontal=lchoriz, width = width, deflection = deflection,  $
      manual = manual, _extra = extrastuff, quiet = quiet, median = median_width
;+
; NAME:
;       GET_SWEEP
;
; PURPOSE:
;       Measure the spacing of a comb on the image.
;       Image should be rotated using ALIGNSTR before using this routine.
;       NOTE: The Maximum WDF array is used as a work array.
;               
; CATEGORY:
;       Graphics
;
; CALLING SEQUENCE:
;
;       GET_SWEEP, STR1 [, WDF] [, /HORIZONTAL] [, MINBIN = minbin]
;         [, SMOOTH = smooth] [, WIDTH = width] [, DEFLECTION = deflection]
;         [,/MANUAL] [, QUIET = quiet] [, _extra= extrastuff][, MEDIAN = median]
;
; where: 
;       STR1   - Input Structure
;       WDF    - Output Waveform Array
;                Default: WDF = get_maxwdf()-1
;       HORIZONTAL - IF set, comb is horizontal
;       MINBIN - Minimum size of lineouts used to determine feature size.
;                Default = 0.2 of sweep
;                Actual number will always be odd
;       SLOPE  - If set, spacing will use a derivative method for finding peaks.
;                This is most useful if the background has a large variation, 
;                but this can be noiser.  Default is slope = 1
;       SMOOTH - Size of Boxcar filter, units of pixels
;                If specified, only one calculation
;                Default = 9 
;       MEDIAN - Initial filter to remove some of the film noise.
;                If median less than 3 then no median filter will be done.
;                DEFAULT: MEDIAN = 5
;       WIDTH  - Fraction of the fwhm used for actual sweep calibration
;                Default = 100%
;       MANUAL - Use the cursor to set the position of the comb.
;                This is useful when the frequency of the comb is low.
;       Deflection - Fraction of peak derivative which will qualify for a 
;                    change in slope.
;                    Default = 0.2 for the peak finder.
;       _EXTRA - Keyword parameters for plotfld (Xrange, Yrange, etc)
;       QUIET  - If set, minimal output
;	
; OUTPUTS:
;       Spacing between peaks is stored in a wdf array.
;
; PROCEDURE:
;       0. A median with width MEDIAN is applied to the image
;       1. User is asked to indicate the location of the comb on the image.
;       2-3. If manual is selected the comb is indicated.
;       otherwise:
;       2. A MINBIN wide lineout is taken perpendicular to the comb.
;       3. The fwhm is calculated. 
;       4. Width*FWHM lineout is taken along the comb.
;       5. The lineout is smoothed with a width of SMOOTH
; if slope is not set:
;       6. Peaks larger than deflection are found
;       7. The center of the fwhm is used as the middle of the signal.
;       8. The spacing is calculated
;       9. The median spacing is used to determine if a peak is missed
; if slope is set:
;       6. The derivative is taken.
;       7. The derivative is smoothed with a width of smooth
;       8. Values of the derivative with absolute values greater than
;          DEFLECTION * MAX(Derivative) are found.
;       9. Values of zero crossing are determined.
;      10. Half periods are determined if deflections have opposite signs.
;          Multiple zero crossings are not counted.
;          
; EXAMPLE:
;       Calculate the spacing of a calibration comb stored in image 1.
;          alignstr, 1, 2   ; Rotate the image to insure it is square.
;          get_sweep, 2, 1  ; Store the spacing in wdf array 1
;          plo, 1           ; Look at the spacing
;          ; Calculate the average spacing over the range [10,30]
;          dist = ave(1, left = 10, right = 30)
;          ; If comb is 4 ns then sweep rate is sw 
;          sw = 4e-9/dist & print, 'Sweep Rate =' , sw
;
;       Similar to the above but use a quadratic sweep
;          COMB = 4e-9  ; set the spacing between marks
;          alignstr, 1, 2   ; Rotate the image to insure it is square.
;          get_sweep, 2, 1  ; Store the spacing in wdf array 1
;          wdfit, 1, 2, /plot, degree = 2, coef = coef
;          y = get_array(1, 2) ; retrieve the old y values
;          ny = n_elements(y)
;          dy = [0, y[1:ny-1]-y[0:ny-2]]
;          dy[0]= dy[1]  ; calculate the derivative and pad to have the 
;                        ; same number of points
;          Note: The fit represents the number of y units between markers
;                The new y axis is the integral of old dy * comb/spacing
;                    = TOTAL (dy * comb/poly(y, coef), /cumulative, /double)
;          Calculate the new Y-values
;          yy = comb*total(dy/poly(y, coef), /cum, /double)
;          put_array, work, yy, 2, label= 'Time (s)' ; replace the y-values
;
; MODIFICATION HISTORY:
; 	Written by:  L. Paul Mix, June 28, 2000
;       Modified by:   L. Paul Mix, August 14, 2001
;                      Added peak finder option.
;-
;
; insert a common block
;
@qscom.dat
;  Common block
@qsstrcom.dat
;       return on an error
       on_error, 1
;
      if n_elements (structinit) eq 0 then qsstruct_init
      if n_elements (qsplotinit) eq 0 then qsinit
;
      nparams = n_params()

      if nparams eq 0 then begin
        print,'% Format: ', $
         'GET_SWEEP, STR1 [, WDF] [, /HORIZONTAL] [, MINBIN = minbin]'
        print,'            [, SMOOTH = smooth] [, WIDTH = width] '
        print,'            [, DEFLECTION = deflection]  [, _extra= extrastuff]' 
        return
      endif
      a= get_valid_structure(ain, 'GET_SWEEP', lcstr=lcstr, ierror=ier)
      if ier ne 0 then return

      if a.type ne 1 then begin
        print, '% GET_SWEEP: Only type 1 image structures may be rotated'
        return
      endif
      if a.ndim gt 2 and a.two eq 0 then begin
        print, '% GET_SWEEP: Image must have two dimensions'
        retur
      endif
;
      work = get_maxwdf()
      if n_elements (wdf) eq 0 then wdf = work-1
      wdf = get_save_array(wdf, work-1)

      arr = reform ( get_array(a, -1))
      if n_elements(median_width) eq 0 then median_width = 5
      if median_width gt 2 then arr = median(arr, median_width, /even)
      ss = size(arr)
      if ss[0] ne 2 then begin
        print, '% ALIGNSTR: Structure must be two dimensional'
        return
      endif

      x = get_array(a, 1)
      y = get_array(a, 2)
      nx = n_elements(x)
      ny = n_elements(y)
      if nx ne ss[1] or ny ne ss[2] then begin
        print, '% GET_SWEEP: X and Y arrays are not the same size as the image'
        print, nx, ny, ss[1], ss[2]
        return
      endif
;
    plotfld, a, cn=2, _extra = extrastuff
    if keyword_set(manual) then begin
      if keyword_set (lchoriz) then begin
        print, 'Indicate the bottom side of the time signal'
        cur, x1, y1, /quiet
        print, 'Indicate the top side of the time signal'
        cur, x2, y2, /quiet
;       CONVERT the point to bins
        junk = min( abs(y1-y), my1)
        junk = min( abs(y2-Y), my2)
        my1 = min([my1, my2], max= my2)
        oplot, !x.crange, y1*[1,1]
        oplot, !x.crange, y2*[1,1]
      endif else begin
        print, 'Indicate the left side of the time signal'
        cur, x1, y1, /quiet
        print, 'Indicate the right side of the time signal'
        cur, x2, y2, /quiet
;       convert the point to bins
        junk = min( abs(x1-x), mx1)
        junk = min( abs(x2-x), mx2)
        mx1 = min([mx1, mx2], max= mx2)
        oplot, x1*[1,1], !y.crange
        oplot, x2*[1,1], !y.crange
      endelse
    endif else begin
      print, 'Indicate the center of the time signal'
      cur, x1, y1, /quiet

;     get the lineouts
      
;     convert the point to bins
      junk = min( abs(x1-x), bx1)
      junk = min( abs(y1-y), by1)
      x1 = x[bx1]
      y1 = y[by1]
    endelse

    if n_elements(minbin) eq 0 then minbin = 0.2 else minbin = abs(minbin)
    if n_elements(width) eq 0 then width = 1.0
    if n_elements(deflection) eq 0 then deflection = 0.2
    if n_elements(slope) eq 0 then slope = 1
    if n_elements (smothin) eq 0 then begin
      if keyword_set(slope) then peak = 0 else peak=1
      smoth = 9
    endif else begin
      smoth = smothin > 3
    endelse
    nx = n_elements(x)
    ny = n_elements(y)
    itry = 1
    if keyword_set(manual) then begin
      itry = 1
      if keyword_set (lchoriz) then begin
        yy = smooth(total(arr(*, my1:my2), 2), smoth)
        xx = x
        dx = a.dx1
        x0 = x[0]
      endif else begin
        yy = smooth(total(arr(mx1:mx2, *), 1), smoth)
        xx = y
        dx = a.dx2
        x0 = y[0]
        nx = ny
      endelse
    endif else begin
      if not keyword_set (lchoriz) then begin
;       Vertical feature
        bins = (round(ny*minbin) < ny)/2
        ba = (by1 -bins) > 0 
        bb = (by1 + bins) < ny-1 
        if bb-ba lt 1.5*bins then begin
          ba = (by1 -bins) > 0 
          bb = (by1 + bins) < ny-1 
        endif
        yy = smooth(total(arr(*, ba:bb), 2), smoth)

        bins =  round(nx*0.12)
        ba = (bx1-bins) > 0 
        bb = (bx1-bins/3) > 0
        n1 =bb-ba+1
        bc = (bx1+bins/3) < nx-1
        bd = (bx1+bins) < nx-1
        n2 =bd-bc+1
          
        xb = [ba+findgen(n1), bc +findgen(n2)]
        yb = [yy[ba:bb], yy[bc:bd]]
        i2w, xb, yb, wdf, /xydata 
;         wdfit, wdf, coef = coef, degree = 2, /plot & hak, /mes
        wdfit, wdf, coef = coef, degree = 2, /quiet

        nn = bd-ba+1
        xb= ba +findgen(nn)
        yb = (coef[2]*xb+coef[1])*xb+coef[0]

        put_wdfarray, [ba, 1, yy[ba:bd]- yb], wdf
        junk = fwhm(wdf, base = 0.0, first = ff, second = ss, /quiet)
        if junk eq 0 then begin
          print, '% GET_SWEEP: Unable to find the comb data'
          return
        endif
        center = round((ff+ss)*0.5)
        diff = (ss-ff)*width
        ba = round(center -diff/2)
        bb = round(center +diff/2)
        if itry eq 1 then begin
          oplot, x[ba]*[1,1], !y.crange
          oplot, x[bb]*[1,1], !y.crange
        endif
        yy = smooth(total(arr(ba:bb, *), 1), smoth)
        xx = y
        x0=y[0]
        dx= a.dx2
        nx = ny
      endif else begin
;       Horizontal feature
        bins = (round(nx*minbin) < nx )/2
        ba = (bx1 - bins) > 0 
        bb = (bx1 + bins) < nx-1 
        if bb-ba lt 1.5*bins then begin
          ba = (bx1 - bins) > 0
          bb = (bx1 + bins) < nx-1
        endif
        yy = smooth(total(arr( ba:bb, *), 1), smoth)

        bins =  round(ny*0.12)
        ba = (by1-bins) > 0 
        bb = (by1-bins/2) > 0
        n1 =bb-ba+1
        bc = (by1+bins)/2 < ny-1
        bd = (by1+bins) < ny-1
        n2 =bd-bc+1

        xb = [ba+findgen(n1), bc +findgen(n2)]
        yb = [yy[ba:bb], yy[bc:bd]]
        i2w, xb, yb, wdf, /xydata
        wdfit, wdf, coef = coef, degree = 2, /quiet

        nn = bd-ba+1
        xb= ba +findgen(nn)
        yb = (coef[2]*xb+coef[1])*xb+coef[0]

        put_wdfarray, [ba, 1, yy[ba:bd]- yb], wdf
        junk = fwhm(wdf, base = 0.0, first = ff, second = ss, /quiet)
        if junk eq 0 then begin
          print, '% GET_SWEEP: Unable to find the comb data'
          return
        endif
        center = round((ff+ss)*0.5)
        diff = (ss-ff)*width
        ba = center -diff/2
        bb = center +diff/2
        if itry eq 1 then begin
          oplot, !x.crange, y[ba]*[1,1]
          oplot, !x.crange, y[bb]*[1,1]
        endif
        yy = smooth(total(arr(*, ba:bb), 2), smoth)
        xx = x
        x0=x[0]
        dx=a.dx1
        nx = nx
      endelse
    endelse
    if dx gt 0 then put_wdfarray, [x0, dx, yy], work, $
      Clab = 'Lineout for Timing', ylab='Amplitude', xlab='Distance'  else $
      i2w, xx, yy, work,  Clab = 'Lineout for Timing', $
      ylab='Amplitude', xlab='Distance'
    if dx eq 0 then begin
      get_wdfarray, dummyarr, work
      dx = dummyarr[1]
      dummyarr =0
    endif  
    x = xx   

 if keyword_set(peak) then begin
    smaxy = max(yy, min = sminy)
    ok = where (yy gt deflection*(smaxy-sminy)+sminy, nok)
    if nok eq 0 then begin
       print, '% GET_SWEEP: No valid deltas found'
       return
    endif
    dok =  (ok[1:nok-1]-ok[0:nok-2])
    okbig = where (dok gt 8, nokbig)
    if nokbig eq 0 then begin
       print, '% GET_SWEEP: Maxima not separated by 8 pixels.'
       return
    endif
    shft = nok/(nokbig*3)
    okbig = [okbig, nok-1]
    nokbig = nokbig+1
    xpeak = xx[ok[okbig -shft]]

   sminy = mn(work, left = xpeak[0], right = xpeak[nokbig-1], /quiet)
   dxx = (xpeak[1:nokbig-1]-xpeak[0:nokbig-2])
    
    jj = sort(dxx)
    dxx = dxx[jj]
; median delta spacing of the peaks
    mdx = dxx[nokbig/2]
    hw = 0.6*mdx
    step = make_array (nokbig)
    fw = step
    j =0
;
    if n_elements(quiet) eq 0 then quiet = 1
    mkcircle
    for i = 0, nokbig-1 do begin
      fw[j] = fwhm( work, base = sminy, quiet = quiet, left = xpeak[i]-hw, $
            right= xpeak[i]+hw, first = first , second=second, peak = peak)
      if fw[j] gt 0 then begin
        step[j] =(first+second)*0.5
        if not keyword_set (quiet) then begin
          oplot, [1,1]*step[j], !y.crange, linesty= 2
          plots, step[j], peak+sminy, psym = 8
          hak, /mes
        endif
        if j gt 0 then begin
          if step[j] gt step[j-1]+dx then j = j+1
        endif else j = j+1
      endif
    endfor
    if j lt 2 then begin
       print, '% GET_SWEEP: Only ', strtrim(j,2), ' peaks found'
       return
    endif
    fw = total(fw)/j
    if not keyword_set (quiet) then begin
      plo, work, /ns, xr = [step[0]-2*fw, step[j-1]+2*fw], /xst
      for i = 0, j-1 do oplot, [1,1]*step[i], !y.crange, linesty= 2
    endif
    xstep = (step[1:j-1]+step[0:j-2])*0.5
    step = (step[1:j-1]-step[0:j-2])
    sortstep = step[sort(step)]
    midstep = sortstep[j/2]
    rel = round (step/midstep) > 1
    if max(rel) gt 1 then rstep = step/rel else rstep = step

    i2w, xstep, rstep, wdf, clabel = 'Timing Mark Spacing', $
        xlab = 'Distance', ylab= 'Spacing', /xydata, file = a.file
    return

 endif

;  Differential method:
      fftw, work, wdf, /mode, /positive
      m = mx(wdf, time = mode, left = 8, /quiet)
      filt, work, wdf, /bp, mode= mode, wid= mode
      w2i, xx, yy, wdf
      x = xx
      ny = n_elements(yy)
      nx = ny
      dy = yy[1:nx-1]-yy[0:nx-2]
;      dy = smooth(dy, smoth)
      dxx = (xx[1:nx-1]+xx[0:nx-2])*0.5
      xbin = (xx[nx-1]-xx[0])/(nx-1)
      md = max(abs(dy))*deflection
      okp = where (dy gt md, nokp)
      dokp = okp[1:nokp-1] -okp[0:nokp-2]
      okdp = where (dokp gt 1, nokdp)

      okn = where (dy lt -md, nokn)
      dokn = okn[1:nokn-1] -okn[0:nokn-2]
      okdn = where (dokn gt 1, nokdn)

      yp = make_array(nokdp+1, /long)
      yp[0] = total(okp[0:okdp[0]])/(okdp[0]+1)
      for i = 1, nokdp-1 do $
        yp[i] = total(okp[okdp[i-1]+1:okdp[i]]) /(okdp[i]-okdp[i-1])
      yp[nokdp] = total(okp[okdp[nokdp-1]+1:*])/(nokp-1-okdp[nokdp-1])
      yp = round(yp)

      yn = make_array(nokdn+1, /long)
      yn[0] = total(okn[0:okdn[0]])/(okdn[0]+1)
      for i = 1, nokdn-1 do $
        yn[i] = total(okn[okdn[i-1]+1:okdn[i]]) /(okdn[i]-okdn[i-1])
      yn[nokdn] = total(okn[okdn[nokdn-1]+1:*])/(nokn-1-okdn[nokdn-1])
      yn = round(yn)

      jp = 0
      jn = 0
      missed = -999
      pold = missed
      nold = missed
      xstep =  make_array(nokdp+nokdn)
      step = xstep
      nstep = 0
      last = 0
      while jp lt nokdp and jn lt nokdn do begin
        if yp[jp] lt yn[jn] then begin
          if yp[jp+1] gt yn[jn] then begin
            ok = where ((dy[yp[jp]:yn[jn]-1] gt 0) and  $
              (dy[yp[jp]+1:yn[jn]] le 0), nok)
            if nok eq 1 then begin
              ok = ok[0]+yp[jp]
              pnew = dxx[ok] +dy[ok]/(dy[ok]-dy[ok+1])*(dxx(ok+1)-dxx[ok])
              if pold ne missed and last lt 0 then begin
                step[nstep]= pnew-pold
                xstep[nstep]= (pnew+pold)*0.5
                nstep = nstep+1
              endif
              pold = pnew
            endif else begin
              nold = missed
              pold = missed
            endelse
          endif else begin
            nold = missed
            pold = missed
          endelse
          jp = jp+1
          last = +1
        endif else begin
          if yn[jn+1] gt yp[jp] then begin
            ok = where ((dy[yn[jn]:yp[jp]-1] le 0) and  $
              (dy[yn[jn]+1:yp[jp]] gt 0), nok)
            if nok eq 1 then begin
              ok = ok[0]+yn[jn]
              nnew = dxx[ok] +dy[ok]/(dy[ok]-dy[ok+1])*(dxx(ok+1)-dxx[ok])
              if nold ne missed and last gt 0 then begin
                step[nstep]= nnew-nold
                xstep[nstep]= (nnew+nold)*0.5
                nstep = nstep+1
              endif
              nold = nnew
            endif else begin
              nold = missed
              pold = missed
            endelse
          endif else begin
            nold = missed
            pold = missed
          endelse
          jn = jn+1
          last = -1
        endelse
      endwhile
      if nstep eq 0 then begin
        print, '% GET_SWEEP: No valid deltas found'
        nstep = 3
      endif else if nstep lt 3 then begin
        print, '% GET_SWEEP: Only ', strtrim(nstep, 2), ' valid deltas found'
        nstep = 3
      endif
      xstep = xstep[0:nstep-1]
      step = step[0:nstep-1]
      i2w, xstep, step, wdf, clabel = 'Timing Mark Spacing', $
        xlab = 'Distance', ylab= 'Spacing', /xydata


 END
