; ------------------------------------------------------------------------ 
; $Id: hists.pro,v 1.3 2007/06/06 20:09:34 lpmix Exp $
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
 Pro  HISTS, str, wdf, nbins = nbin, maximum = maximum, minimum = minimum, $
             binsize = binsize, xrange = xrange, plotit = plotit, $
             yrange = yrange, zrange = zrange, vector = vector, $
             reverse_indices = reverse_indices, locations = locations
;
;+
; NAME:
;       HISTS
;
; PURPOSE:
;       Generate a histogram for the waveform array amplitudes.
;         This procedure uses the IDL procedure, HISTOGRAM
;
; CALLING SEQUENCE:
;
;      HISTS, STR [, WDF] [BINSIZE = binsize] [, NBINS = nbins] 
;             [, MAXIMUM = maximum] [, MINIMUM = minimum] [, /PLOTIT ]
;             [, VECTOR = vector]
;             [, XRANGE = xrange] [, YRANGE = yrange] [, ZRANGE = zrange]
;             [, REVERSE_INDICES = reverse_indices] [, LOCATIONS = locations]
;
; 
;
; INPUTS:
;       STR;   Input structure or structure array to be analyzed
;              Structure must be type 1 or type 8
;
; OPTIONAL INPUTS:
;       WDF:   Output WDF array
;               Default:  WDF = GET_MAXWDF()-1
;        Note:  X values are the median values for LOCATIONS
;               IDL histogram uses the minimum value of each bin and these 
;               values are returned in LOCATIONS.
;               Additional Zeros are added to the return from HISTOGRAM to 
;               allow a nice graph with psym= 10
;	
; KEYWORD PARAMETERS:
;       Note:  Some of the defaults are set by HISTS
;
;       BINSIZE: Size of BINS
;       NBINS:  Number of bins
;               Default: NBINS = (NPOINTS/50 + 2) < 1000
;                 where NPOINTS = the number of points in the WDF array
;       MAXIMUM: Maximum value to consider for the histogram.  This value is
;                increased by 1.0e-6 before being passed to histogram.
;          Note: MAXIMUM is ignored if both BINSIZE and NBIN are specified.
;          Default: MAXIMUM = MX(WDF1)
;       MINIMUM: Minimum value to consider for the histogram.
;           Default:  MINIMUM = MN(WDF1)
;       VECTOR:  If the number of vectors is greater than 1, VECTOR will 
;                specify the desired vector.
;                Default: VECTOR = 1
;       PLOTIT:  If set, WDF will be plotted.
;                "plo, wdf, psym= 10"
;   The following "RANGE" keywords apply only to single block data:
;       XRANGE:  If two values are provided, only ordinates whose abscissa are 
;                in the specified range are included in the histogram.
;                At least two values must be provided.
;                If there are less than two valid ordinate values, the histogram 
;                will not be calculated.
;       YRANGE, ZRANGE: Same as XRANGE for the second and third dimensions.
;
; The following are output if a variable is provided.
;       REVERSE_INDICES: See HISTOGRAM for explanation
;           Note:  Multi-block structures will NOT produce valid XYZ indices 
;                  from REVERSE_INDICES, but HISTS will be valid.
;                  Single block structures can use GETDIM to convert the linear
;                  index to XYZ indices.
;       LOCATIONS: BIN boundaries
;                  If H = getyf(WDF2) then H[i] is the number of values, VAL, 
;                  which satisfies: LOCATIONS[i] less/equal V less LOCATIONS[i+1]
;          Note:  LOCATIONS will be returned, but cannot be set.
;
; OUTPUTS:
;       WDF array with the histogram distribution function
;
; OPTIONAL OUTPUTS:
;       REVERSE_INDICES and LOCATIONS may be returned.
;
; PROCEDURE:
;       This procedure uses the IDL procedure: HISTOGRAM
;
; EXAMPLE:
;       Get a histogram of the amplitude of structure array 3 and store it in 33
;          HISTS, 3, 33
;
;       Calculate the histogram of the amplitude of structure array 5 between 
;         X-values of 500E3 and 2E6 with a bin size of 1E5.  Store the result in 75
;         and plot the histogram.
;         HISTS, 5, 75, min = 5e5, bin = 1e5, max = 2e6, /plotit
;
;       Determione the X and Y locations of the values of the structure, intensity,
;       between 10 and 12.
;          hists, intensity,rev = r, min = 10, max = 12, nbin = 1
;          ind = r[r[0]:r[1]-1]  ; get the indices for the first bin
;          xyzloc = getdim(get_array(intensity, -1, 1), ind)
;          
;       Note: xyzloc contains the XYZ locations for the desired intensity. 
;             INTENSITY must be a single block array with no ranges provided.
;       To plot the points over the image:
;          plotstr, intensity, cn=2
;          x = (get_array(intensity, 1)) [zyzloc[0, *]]
;          y = (get_array(intensity, 2)) [zyzloc[1, *]]
;          mksymbol, 'star'
;          plots, x, y, psym=8, color=6
;
; MODIFICATION HISTORY:
;       Written by:     L. P. Mix, Jr., March 30, 2005
;
;-
;
@com.dat
;
      on_error, 1
;     include the common block
;
@qscom.dat
;  Common block
@qsstrcom.dat
;
      if n_elements (structinit) eq 0 then qsstruct_init
      if n_elements (qsplotinit) eq 0 then qsinit
      if n_elements(initwdf) eq 0 then wdfinit
      nparams = n_params()
      if nparams lt 1 then begin
print,'% Format: HISTS, STR [, WDF] [BINSIZE = binsize] [, NBINS = nbins]'
print,'         [, MAXIMUM = maximum] [, MINIMUM = minimum] [, /PLOTIT ]'
print,'         [, XRANGE = xrange] [, YRANGE = yrange] [, ZRANGE= zrange]'
print,'         [, REVERSE_INDICES = reverse_indices] [, LOCATIONS = locations]'
        return
      endif
;
      if n_elements(str) eq 0 then begin
        print, '% HISTS: Parameters must be defined'
        return
      endif
      array = str
      b = size(str)
      if b(b(0)+1) ne 8 then begin
        a = str
        if a gt 0 and a le maxstructure then begin
          s2i, array, a
          b = size (array)
          if b(b(0)+1) ne 8 then begin
    print, '% HISTS: First parameter must be a structure or a QS array number'
            return
          endif
        endif  else begin
          print, ' First parameter must be a structure or a QS array number'
          return
        endelse
      endif else a = 0
      if array.type ne 1 && array.type ne 8 then begin
        print, '% HISTS: Only type 1 or type 8 structures may be histogramed'
        return
      endif

;
      ww = maxwdfarray-1
      if nparams gt 1 then save = get_save_array(wdf, ww) else save = ww
      if save lt 1 or save gt maxwdfarray then save = ww

      if array.nvect gt 1 then begin
        if n_elements(vector) eq 0 then vector = 1
         y = get_array(array, -abs(vector))
      endif else y= get_array(array, -1)
      if array.block eq 1 then begin
        lcrange =0
        if n_elements(xrange) gt 1 then begin
          lcrange = 1
          mnx = min(xrange, max = mxx)
          x = get_array(array, 1)
          ok = where(x ge mnx and x le mxx, nok)
          if nok lt 1 then begin
            print, '% HISTS: Less than one value in given XRANGE'
            print, '         XRANGE = [',strtrim(mnx, 2),',',strtrim(mxx,2), ']'
            return
          endif
          mnxx = min(ok, max = mxxx)
          clab = ' - XRange=['+strtrim(mnx,2)+', '+strtrim(mxx,2)+']'
        endif else begin
          clab =''
          mnxx =0L
          mxxx = array.size[0]-1
        endelse
        if n_elements(yrange) gt 1 then begin
          lcrange = 1
          mnx = min(yrange, max = mxx)
          x = get_array(array, 2)
          ok = where(x ge mnx and x le mxx, nok)
          if nok lt 1 then begin
            print, '% HISTS: Less than one value in given YRANGE'
            print, '         YRANGE = [',strtrim(mnx, 2),',',strtrim(mxx,2), ']'
            return
          endif
          mny = min(ok, max = mxy)
          clab = (strlen(clab) lt  1) ? $
            ' - YRange=['+strtrim(mnx,2)+', '+strtrim(mxx,2)+']' : $
            clab + ': YRange=['+strtrim(mnx,2)+', '+strtrim(mxx,2)+']'
        endif else begin
          mny =0L
          mxy = array.size[1]-1
        endelse
        if n_elements(zrange) gt 1 && array.ndim gt 2 then begin
          lcrange = 1
          mnx = min(zrange, max = mxx)
          x = get_array(array, 3) 
          ok = where(x ge mnx and x le mxx, nok)
          if nok lt 3 then begin
            print, '% HISTS: Less than one value in given ZRANGE'
            print, '         ZRANGE = [',strtrim(mnx, 2),',',strtrim(mxx,2), ']'
            return
          endif
          mnz = min(ok, max = mxz)
          clab = (strlen(clab) lt  1) ? $
            ' - ZRange=['+strtrim(mnx,2)+', '+strtrim(mxx,2)+']' : $
            clab + ': ZRange=['+strtrim(mnx,2)+', '+strtrim(mxx,2)+']' 
        endif else begin
          mnz =0L
          mxz = (array.ndim gt 2) ?  array.size[2]-1 : 0
        endelse 
        if lcrange gt 0 then y = y[mnxx:mxxx, mny:mxy, mnz:mxz]
        if n_elements(y) lt 3 then begin
          print, '% HISTS: Less than three values in given RANGES'
          return
        endif
      endif
      if n_elements(minimum) eq 0 then minimum = min(y)
      fudg = 1.000001
      if n_elements(maximum) eq 0 then maxi = max(y) else maxi = maximum[0]
      maxi = maxi*fudg
      if n_elements(binsize) gt 0 then binsize = abs(binsize[0]) else binsize =0
      if n_elements(nbin) gt 0 then nbin = abs(nbin[0]) else nbin =0
      if binsize gt 0 && nbin gt 0 then maxi = minimum+binsize*nbin else $
        if nbin gt 0 then binsize = (maxi - minimum)/nbin 
      if binsize eq 0 then begin
        nbin = (n_elements(y)/50 +2) < 1000
        binsize = (maxi - minimum)/nbin
      endif
      dis = histogram(y, binsize = binsize, min = minimum, max = maxi, $
          locations = locations, reverse_indices = reverse_indices)
      ndis = n_elements(dis)
      dis = (dis[ndis-1] eq 0)? [0L, dis] : [0L, dis, 0L]
      put_wdfarray, [total(locations[0:1])*0.5 - binsize, binsize, dis], $
           save, /no_copy, xydata =0
      wdf = save
      wdfcomment(save-1)= 'Histogram of '+array.clab + clab
      wdffile(save-1)= array.file
      wdfxlabel(save-1)= $
        (strlen(array.v1lab[0]) gt 0) ? array.v1lab[0] : 'Amplitude'
      wdfylabel(save-1)= 'Number of Values'
      if keyword_set(plotit) then plo, save, psym=10
      end
