; ------------------------------------------------------------------------ 
; $Id: kill_part.pro,v 1.3 2007/06/06 20:09:34 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
  function kill_part, ain, filter = filter, allorbits= allorbits

;+
; function kill_part
; Return the location of killed particles or filter killed particles.
;
; format kill= kill_part(str [, filter = filter] [, ALLORBITS = allorbits] )
;  where: 
;     kill   - a structure of the parameters for killed particles
;     str    - particle structure or structure array number
;     filter - If present, killed particles will be filtered according
;              to the value of filter and the entire trajectories returned in 
;              kill.  The form of filter is similar to w1 in plotpar.
;              FILTER must have 3 elements ie. filter = [A, B, C]
;              where A is the quantity, and B and C are the min and maximum
;                values of the quantity. A positive indicates spatial dimensions,
;                and A negative indicates attribute quantities. 
;     allorbits - If all the particles in str are known to be killed particles
;                 and the user only wants the endpoints, then all orbits 
;                 will select the endpoints of all orbits.
;     NOTE:     Filter is ignored if allorbits is set.
;  example:
;  plot the location of killed particles from the particle structure, str
;     in a z-r plot
;  plotpar, kill_part(str), 3, 1
;  
;  plot the trajectories of all particles which die in the first dimension
;     range [0, .1] in a z-r plot
;       s = kill_part(str, filt=[1, 0, .1])
;       plotpar, s, 3, 1, /follow
;
;  plot the endpoints of the above subset of the particles in r/z
;       plotpar, kill_part(s, /all), 3, 1
;-
;  Structure Labels
;    type  - 2
;    ndim  - number of spatial dimensions
;    natt  - number of attribute dimensions
;    block - number of blocks (=1)
;    nvert - number of vertices 
;    two   - two dimensional missing dimension
;            this is 0 for 3d data, 1 for yz data, 2 for zx data, 3 for xy data
;    x1, x2, ...x(ndim) - spatial arrays
;    v1, v2, ...v(nvect) - attribute arrays
;    x1lab, x2lab, ...x(ndim)lab - spatial labels
;    v1lab, v2lab, ...v(nvect)lab - vector labels, Vector label in the case of
;                  NGD data and block label in the case of vector and 3d data
;    tlab  - type label
;    clab  - dataset comment
;    file  - file data read from
;    spare - integer array (5)
;
      on_error, 1
;
@qscom.dat
;
;  Common block
@qsstrcom.dat
;
      if n_elements (structinit) eq 0 then qsstruct_init
      if n_elements (qsplotinit) eq 0 then qsinit
;
      common pffcom, initpff, pff_exe
;
;     check to see if pff has been initialized
;
;     check on the number of parameters
;
      nparams =  n_params()
;
      if nparams eq 0 then begin
        print,'% Format: kill_str = KILL_PART(structure)'
        return, 0
      endif
;    make sure we don't corrupt input data
;     now get the data  -- check that we have a structure of the proper type
      if n_elements(ain) eq 0 then begin
        print, 'Input Structure must be defined'
        return, 0
      endif
      a = ain
      b = size(a)
      if b(b(0)+1) ne 8 then begin
        if a gt 0 and a le maxstructure then begin
          s2i, a, a
          b = size (a)
          if b(b(0)+1) ne 8 then begin
            print, ' First parameter must be a structure or a QS array number'
            return, 0
          endif
        endif  else begin
          print, ' First parameter must be a structure or a QS array number'
          return, 0
        endelse
      endif
      if a.type ne 2 then begin
        print, ' Only Particle type structures may be plotted'
        return, 0
      endif
      if a.natt ne 6 then begin
        print, ' killed particles must have 6 attributes'
        return, 0
      endif
      np = n_elements(a.v6)
      if np eq 0 then begin
        print, ' Must have particle number in attribute 6'
        return, 0
      endif
      dn = a.v6(1: np-1)- a.v6(0:np-2)
      maxtime = a.v4(np-1)
      ok = where (dn gt 0.5, nok)
      if keyword_set (allorbits) then begin
        nok = nok+1
        ok = [ok, np-1]
      endif else lockill = [-1, ok]
      if nok eq 0 then begin
        print, 'No killed particles in this data'
        return, 0
      endif
      if a.ndim ne 3 then begin
        print, ' Must be 3D data'
        return, 0
      endif
      nelf = n_elements(filter)
      if nelf ne 3 and nelf gt 0 then print, $
         '% KILL_PART: Filter must have 3 elements, no action taken'
   if (not keyword_set (allorbits)) then begin
      if (nelf eq  3) then begin
        if filter(0) gt 3 or filter(0) lt -6 then begin
          Print, '% KILL_PART: Filter(0) must be in the range [-6,3]:', $
            filter(0)
          return, 0
        endif
        if filter(0) eq 1 then val = a.x1(ok) $
          else if filter(0) eq 2 then val = a.x2(ok) $ 
          else if filter(0) eq 3 then val = a.x3(ok) $ 
          else if filter(0) eq -1 then val = a.v1(ok) $ 
          else if filter(0) eq -2 then val = a.v2(ok) $ 
          else if filter(0) eq -3 then val = a.v3(ok) $ 
          else if filter(0) eq -4 then val = a.v4(ok) $ 
          else if filter(0) eq -5 then val = a.v5(ok) $ 
          else if filter(0) eq -6 then val = a.v6(ok)
        ma = min(filter(1:2), max = mb)
        okf = where (val ge ma and val le mb, nokf)
        if nokf eq 0 then begin
          print, '% KILL_PART: No killed particles satisfy this criteria', $
                ' Filter = ', filter 
          return, 0
        endif
        ok = ok(okf)
        okkill = where(a.v4(ok) lt maxtime, nokk)
        if nokk eq 0 then begin
          print, 'No killed particles in this data'
          return, 0
        endif
        okk = okf(okkill)
        ok = make_array(np, /long)
        np= 0
        n = 0
        for i = 0, nokk-1 do begin
          j = okk(i)+1
          n = lockill(j)-lockill(j-1)
          ok(np:(np+n-1))= lindgen(n)+ (lockill(j-1)+1L)
          np = np+n
        endfor
        ok = temporary(ok(0:np-1))
        nokk = np
      endif else begin
        okkill = where(a.v4(ok) lt maxtime, nokk)
        if nokk eq 0 then begin
          print, 'No killed particles in this data'
          return, 0
        endif
        ok = ok(okkill)
      endelse
    endif

      tempstr = {type:2L, ndim:a.ndim, natt:a.natt, block:1L, $
              nvert:6L, two:a.two, x1:a.x1(ok), x2:a.x2(ok), $
              x3:a.x3(ok), x4:0.0, v1:a.v1(ok), v2:a.v2(ok), $
              v3:a.v3(ok), v4:a.v4(ok),v5:a.v5(ok), v6:a.v6(ok), $
              x1lab : a.x1lab, x2lab : a.x2lab, x3lab:a.x3lab, $
              x4lab : a.x4lab, v1lab : a.v1lab, v2lab:a.v2lab, $
              v3lab : a.v3lab, v4lab : a.v4lab, v5lab:a.v5lab, $
              v6lab : a.v6lab, file : a.file, spare : a.spare, $
              tlab : 'killed part', $
              clab: 'Killed particles from ' +a.clab }
      return, tempstr
      end
;  Structure Labels
;    type  - 2
;    ndim  - number of spatial dimensions
;    natt  - number of attribute dimensions
;    block - number of blocks (=1)
;    nvert - number of vertices 
;    two   - two dimensional missing dimension
;            this is 0 for 3d data, 1 for yz data, 2 for zx data, 3 for xy data
;    x1, x2, ...x(ndim) - spatial arrays
;    v1, v2, ...v(nvect) - attribute arrays
;    x1lab, x2lab, ...x(ndim)lab - spatial labels
;    v1lab, v2lab, ...v(nvect)lab - vector labels, Vector label in the case of
;                  NGD data and block label in the case of vector and 3d data
;    tlab  - type label
;    clab  - dataset comment
;    file  - file data read from
;    spare - integer array (5)
;
