; ------------------------------------------------------------------------ 
; $Id: lineout.pro,v 1.4 2010/11/17 20:16:31 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
;+
; NAME:
;       LINEOUT 
;
; PURPOSE:
;       Generate lineouts from any 2 or 3-D structure
;       Procedure will generate a lineout with arbitrary directions and 
;         number of points.
;       Lineouts must have a minimum of 3 values.
;       Distances are slant distances from one end point or X, Y, Z value
;       A single lineout is produced by each call to LINEOUT
;
; CALLING SEQUENCE:
;	LINEOUT, STR , WDF 
;   where:
;       STR   - Structure array number or actual structure
;               Structure must be a type 1 field type structure
;               Note: Data will not be transposed.
;       WDF   - WDF array number to store the lineout. Default is 1.
;               WDF can be set manually.
;         
; REQUIRED KEYWORD PARAMETERS:
;       STARTPOINT: Point for start of the lineout
;                   STARTPOINT must have as many elements as the dimensions of
;                   the structure and be within the data range of the structure
;       DIRECTION:  Direction for the lineout 
;                   Direction cosines for the lineout
;                   (Not required if ENDPOINT is specified)
;       ENDPOINT:   End point for the lineout
;                   (Ignored if DIRECTION is specified)
;
; REQUIRED KEYWORD PARAMETERS:
;       TOTAL_LINE: If set, all points along the lineout path in both directions
;                   are returned.  Minimum X value will be the first point.
;                   Default = 0 ; Only the points between the two points or from
;                     the first point to the end of the data are returned.
;	ABSCISSA:   If set to 1, 2 or 3 then X, Y, or Z will be used as the 
;                   abscissa of the lineout.  
;                   Default is SQRT(DX^2+dy^2+dz^2)
;       NPOINTS:    Number of points.  Default is 2*max(128, nx, ny, nz)
;                   where nx, ny, nz are then total number of points in each
;                   direction.
;       MISSING:    WDF array values are preloaded with this value.  Values
;                   not found retain this value.
;
; EXAMPLE:
;	Take a series of lineouts from structure array 3 and store the lineouts
;         beginning in wdf array 11 from [0,0,0] to [1,1,1].
;           LINEOUT, 3, 11, start =[0,0,0], endpoint=[1,1,1]
;
;       Take a series of lineouts from structure array 4 and store the lineouts
;         beginning in wdf array 11 from [0,0,0] to [1,1,1] but extend the 
;         lineout in both directions to include the total data extent.  Take 
;         1000 points
;           LINEOUT,4,11,start=[0,0,0],endpoint=[1,1,1], /total, np=1000
;      <or> LINEOUT,4,11,start=[0,0,0],direction=[1,1,1]/sqrt(3.),/total,np=1000
;
;
; MODIFICATION HISTORY:
; 	Written by:	L. P. Mix, May 9, 2003
;-


 pro lineout, strin, wdf, startpoint= startpointin, endpoint= endpointin, $
       Direction = direction, total_line = total_line, Abscissa = abscissa, $
       missing = missing, npoints= npoints
;
;     return on an error
      on_error, 1
@com.dat
@qscom.dat
;  Common block
@qsstrcom.dat
;
      if n_elements(initwdf) eq 0 then wdfinit
      if n_elements (structinit) eq 0 then qsstruct_init
      common pffcom, initpff, pff_exe
;
      nparams = n_params()
      if nparams eq 0 then begin
        print,'% Format: ', $
       'LINEOUT, STR, WDF,  STARTPOINT=startpoint, ENDPOINT= endpoint'
        print,'       DIRECTION=direction, [, /TOTAL_LINE] [, MISSING = missing]'
        print,'       [, NPOINTS= npoints] [, ABSCISSA = abscissa]'
        return
      endif

      arr = get_valid_structure(strin, 'LINEOUT', ierror = ier)
      if ier ne 0 then return
      
      if arr.type ne 1 then begin
        Print, '%LINEOUT: Structure array must be a type 1, field structure'
        return
      endif

      if n_elements (wdf) eq 0 then begin
        print,  '%LINEOUT: WDF array must be specified.'
        return
      endif
      if wdf lt 1 or wdf gt maxwdfarray then begin
        print, '%LINEOUT: WDFarray must be greater than zero and less than ' $
              + strtrim(maxwdfarray, 2)
        return
      endif

      ndim = arr.ndim
      if ndim eq 2 then lc2dim =1 else lc2dim =0
      two = arr.two

      clabel = arr.clab
      rang = arr.range
      if n_elements(startpointin) lt (3-lc2dim) then begin
        print, '%LINEOUT: Startpoint must be specified with all dimensions.'
        return
      endif
      dx = rang[1,0,0] -rang[0,0,0]
      dy = rang[1,1,0] -rang[0,1,0]
      if ndim eq 3 then dz = rang[1,2,0] -rang[0,2,0]
      dxerr = 1e-5*dx
      dyerr = 1e-5*dy
      if n_elements(dz) gt 0 then dzerr = 1e-5*dz 

      rang[0, 0, *] = rang[0, 0, *]-dxerr
      rang[1, 0, *] = rang[1, 0, *]+dxerr
      rang[0, 1, *] = rang[0, 1, *]-dyerr
      rang[1, 1, *] = rang[1, 1, *]+dyerr
      if ndim eq 3 then begin
        rang[0, 2, *] = rang[0, 2, *]-dzerr
        rang[1, 2, *] = rang[1, 2, *]+dzerr
      endif

      startpoint =  float(startpointin[0:ndim-1])
      if startpoint[0] ge rang[0, 0, 0] and startpoint[0] le rang[1, 0, 0] and $
        startpoint[1] ge rang[0, 1, 0] and startpoint[1] le rang[1, 1, 0] $
        then begin
        lcok = 1
        if ndim eq 3 then if startpoint[2] lt rang[0, 2, 0] or $
          startpoint[2] gt rang[1, 2, 0] then lcok =0
      endif else lcok =0
      if lcok eq 0 then begin
        print, '%LINEOUT: Startpoint must be in the range of the structure'
        return
      endif

      if n_elements(endpointin) lt (3-lc2dim) and $
          n_elements(direction) lt (3-lc2dim) then begin
        print, '%LINEOUT: Endpoint or direction must be specified.'
        return
      endif
      leng = 0
      if n_elements(endpointin) lt (3-lc2dim) then $
        direct = direction[0:ndim-1] else begin
        endpoint = endpointin[0:ndim-1]
        lengv = endpoint-startpoint
        leng = sqrt(total(lengv*lengv))
        direct = lengv/leng
      endelse
      direction = direct

;     get the values of length to get to the boundary for X, Y, Z
      lengx = (arr.range[*, 0, 0] - startpoint[0])/direct[0]
      lengy = (arr.range[*, 1, 0] - startpoint[1])/direct[1]
;     note one length will be positive one negative 
;     Store the backwards direction in 0 and the forward in 1
      if lengx[0] gt lengx[1] then lengx= [lengx[1], lengx[0]]
      if lengy[0] gt lengy[1] then lengy= [lengy[1], lengy[0]]
      if ndim eq 3 then begin
        lengz = (arr.range[*, 2, 0] - startpoint[2])/direct[2]
        if lengz[0] gt lengz[1] then lengz= [lengz[1], lengz[0]]
      endif else lengz =[0,0]
      if ndim eq 3 then lengt = min([lengx[1], lengy[1], lengz[1]]) $
        else lengt = min([lengx[1], lengy[1] ])
      if leng eq 0 or keyword_set(total_line) or lengt lt leng then begin
        endpoint = startpoint+direct*lengt
      endif
      if ndim eq 3 then leng = max([lengx[0], lengy[0], lengz[0]]) $
        else leng = max([lengx[0], lengy[0]])
      if keyword_set(total_line) or leng gt 0 then begin
        startpoint = startpoint+direct*leng
      endif

      if n_elements(npoints) gt 0 then np = npoints[0] > 3 else np = 2L* $
        max([128, n_elements(arr.x1), n_elements(arr.x2), n_elements(arr.x3)])
      dlength = [endpoint-startpoint]
      px= startpoint[0] + dlength[0]/(np-1)*findgen(np)
      py= startpoint[1] + dlength[1]/(np-1)*findgen(np)
      if ndim eq 3 then $
        pz= startpoint[2] + dlength[2]/(np-1)*findgen(np)
      if n_elements(missing) eq 0 then missing =0.0
      funct = make_array(np, value = missing)
      if n_elements(abscissa) eq 0 then begin
        xval = sqrt(total(dlength*dlength))/(np-1)*findgen(np)
        xlabel = 'Distance'
      endif else begin
        abscissa = abscissa[0]
        if abscissa eq 1 then begin
          xval = px
          xlabel = 'X Value'+(arr.x1lab)[0]
        endif else if abscissa eq 2 then begin
          xval = py
          xlabel = 'Y Value'+(arr.x2lab)[0]
        endif else if abscissa eq 3 then begin
          xval = pz
          xlabel = 'Z Value'+(arr.x3lab)[0]
        endif else begin 
          xval = sqrt(total(dlength*dlength))/(np-1)*findgen(np)
          xlabel = 'Distance'
        endelse
      endelse
      nblo = arr.block
      for iblo = 1, nblo do begin
        if ndim eq 3 then $
        ok = where (px ge rang[0,0,iblo] and px le rang[1,0,iblo] $
          and py ge rang[0,1,iblo] and py le rang[1,1,iblo] $
          and pz ge rang[0,2,iblo] and pz le rang[1,2,iblo], nok) $
        else $
        ok = where (px ge rang[0,0,iblo] and px le rang[1,0,iblo] $
          and py ge rang[0,1,iblo] and py le rang[1,1,iblo], nok)
        if nok gt 0 then begin
          v1 = get_array(arr, -1, iblo)
          fx = findex(get_array(arr, 1, iblo), (px)[ok])
          fy = findex(get_array(arr, 2, iblo), (py)[ok])
          if ndim eq 3 then begin
            fz = findex(get_array(arr, 3, iblo), (pz)[ok])
            vblo = interpolate(v1, fx, fy, fz)
          endif else vblo = interpolate(v1, fx, fy)
          funct[ok] = vblo
        endif
      endfor
      if ndim eq 3 then $
      i2w,xval, funct, wdf, clab = 'Lineout from ('+strtrim(startpoint[0],2)+ $
        ', '+strtrim(startpoint[1], 2)+', '+strtrim(startpoint[2], 2) + $
        ') to (' +strtrim(endpoint[0], 2)+', '+strtrim(endpoint[1], 2) + $
        ', '+strtrim(endpoint[2], 2)+') - '+clabel, file = arr.file, $
        xlabel = xlabel else $
      i2w,xval, funct, wdf, clab = 'Lineout from ('+strtrim(startpoint[0],2)+ $
        ', '+strtrim(startpoint[1], 2) + $
        ') to (' +strtrim(endpoint[0], 2)+', '+strtrim(endpoint[1], 2) + $
        ') - '+clabel, file = arr.file, xlabel = xlabel
END
