; ------------------------------------------------------------------------ 
; $Id: maxstr.pro,v 1.3 2007/06/06 20:09:35 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
 Function  maxstr , strin, vector, xrange=xrange, yrange=yrange, $
    zrange=zrange, location = location, minimum = minval, ier = ier, $
    pmaximum = pmaximum, pminimum = pminimum, helpme = helpme 
;
;+
; NAME:
;       MAXSTR
;
; PURPOSE:
;       Find the maximum of a structure array and the location of that maximum.
;       Current version is for type 1 field structures.
;
; CATEGORY:
;       Math
;
; CALLING SEQUENCE:
;       Field_Max =  MAXSTR (Structure [, Vector]  )
;
; where Field_Maxin is the minimum value of the field. 
;
; INPUTS:
;       Structure - Structure array number or idl field Structure.
; OPTIONAL INPUTS:
;       Vector    - Vector component for the function.
;                   If Vector is an array, the square root of the sum of
;                     the squares will be used.
;                   If Vector is a single value array, ABS will be applied to 
;                     that component. That is if Vector = [2] then 
;                     ABS(V2) will be searched.
;                   If max(Vector) has a value larger than the number of vector
;                     components and dimensions, then Vector = 1+indgen(nvect)
;                     where nvect is the number of vector components
;                   Default: VECTOR = 1 if number of vectors is 1
;                            VECTOR = 99 if number of vectors is > 1.
;	
; KEYWORD PARAMETERS:
;       XRANGE    - Range of the first dimension for the maximum search.
;                   Default:  All X values
;       YRANGE    - Range of the second dimension for the maximum search.
;                   Default:  All Y values
;       ZRANGE    - Range of the third dimension for the maximum search.
;                   Default:  All Z values
;       LOCATION  - Position where maximum occurs
;       PMAXIMUM  - Same as LOCATION
;       MINIMUM   - Maximum val in the range defined.
;       PMINIMUM  - Position where MINIMUM occurs.
;       IER       - Error parameter
;                   Default: IER = 0 ==> No problems 
;
; OUTPUTS:
;       The function returns the Maximum of the field structure.
;
; OPTIONAL OUTPUTS:
;       Minimum and the locations for the maximum and minimum are returned
;
; EXAMPLE:
;       Determine the range of the electric field structure E
;         range = maxstr(E, 4, min= emin) -emin
;
; MODIFICATION HISTORY:
;       Written by:     L. P. Mix, March 12, 1999
;-
ier =1 
if keyword_set(helpme) then begin
  print, '% Format: Field_Max =  MAXSTR (Structure [, Vector]'
  print, '           [, XRANGE= xrange] [, YRANGE= yrange] [, ZRANGE= zrange]'
  print, '           [, LOCATION = location] [, PMINIMUM = pminimum]'
  print, '           [, MINIMUM = minimum] [, PMAXIMUM = pmaximum])'
  return, 0
endif
;

If n_elements (strin) eq 0 then begin
  print, '% MAXSTR:  Structure or structure array number must be provided'
  return, 0
endif
if n_elements (vector) gt 0 then fld = vector

minmaxstructure, strin, fld,  xrange=xrange, yrange=yrange, ier = ier, $
    zrange=zrange, maximum = maxval,  minimum = minval, $
    pmaximum = pmaximum, pminimum = pminimum
   
if ier eq 0 then location = pmaximum
return, maxval
end
