; ------------------------------------------------------------------------ 
; $Id: minmax.pro,v 1.3 2007/06/06 20:09:35 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
function minmax, comp, dsmin, dsmax, dsstp, printit = printit, $
              array = arrayin, _extra= extrastuff

;+
; Name:
;       MINMAX
;
; PURPOSE:
;       Scan in a PFF file in DO List and determine the range of the
;         attirbutes.  All datasets must be of the same type.
; CALLING SEQUENCE:
;       Range = MINMAX( Comp [, DatasetMin] [, DatasetMax] [, DatasetStep]
;           [, ARRAY = array] [, PRINTIT = printit] [, _EXTRA = extrastuff] ) 
;   where:
;       Range:    Vector with the maximum and minimum values.
;                 If comp is an array then range = range(2, n_elements(comp)]
;                 If an error then range = -1
;
;       Comp:     Field component: either 1, 2, or 3 for a single
;                 component, 0 for all components or -1 for magnitude
;                 of a vector
;                 Note: If more than 1 components is desired use 0.
;       The datasets in the current PFF file are scanned in a Do list
;       DatasetMin  \
;       DatasetMax   -   DO list for dataset numbers in file
;       DatasetStep /
;       All parameters have default values.
;       Default Values are : DatasetMin =1
;                            DatasetMax =ndspff()
;                            DatasetMin =1
;         If DatasetMax is less than DatasetMin then DatasetMax = ndspff
;       ARRAY:    Array of dataset numbers
;                 Values allowed are gt 0 and le ndspff()
;                 If invalid values are listed, routine returns a -1
;       Printit:  If set, values will be printed.
;       _EXTRA:   Any valid keywords for READSTR
; Examples:
;       Get the max and min of all the datasets in the current file for the
;          first vector component.
;          val = MINMAX(1)
;
;       Get the max and min of all the datasets in the current file for the
;          magnitude of vector component.
;          val = MINMAX(-1)
;
;       Get the max and min of dataset 5 and for every 3 dataset in the
;         current file for the third vector component and print the values.
;          val = MINMAX(3, 5, 0, 3,/print)
;
;       Get all the datasets with 'rho-e'
;          val = minmax(0, array= get_dset('rho-e'))
;       Note:  DIR command strips spaces to obtain maximum information.
;              DIRPFF, -3 will print the actual comments 3 comments on
;                      either side of the current position.
;              so: 'rho-e', 'rho-p', and 'rho    ' will get different datasets.
;                (Trailing spaces on the search string ARE significant.)
;       
;       Original version by TDPointon. 
;       Rewritten Feb 12, 1998 - lpmix
;-
;
;  Common block
@qsstrcom.dat
;
on_error, 1
;
  if n_elements (structinit) eq 0 then qsstruct_init
;

  if n_elements(comp) eq 0 then begin
    print, '% MINMAX: Field component must be specified'
    print, '% Format: Range = MINMAX( Component, DatasetMin, DatasetMax, '
    print, '                  DatasetStep, /Printit)'
    return, -1
  endif else vec = round (comp(0))
  if n_elements(arrayin) gt 0 then begin
    array = arrayin
    nel = n_elements(array)
    nmax = ndspff()
    if nmax eq 0 then begin
      print, '% MINMAX: Please use SETPFF to set a valid PFF file.'
      return, -1
    endif
    ok = where (array le 0 or array gt ndspff(), nok)
    if nok gt 0 then begin
      Print, '% MINMAX: Invalid array values passed:', array(ok)
      return, -1
    endif
    ind = array(0)
    dsmin = 0
    dsmax = nel-1
    dsstp = 1
  endif else begin
    if n_elements(dsmin) eq 0 then dsmin = 1
    if n_elements(dsmax) eq 0 then dsmax = ndspff()
    if dsmax lt dsmin then dsmax = ndspff()
    if dsmax eq 0 then begin
      print, '% MINMAX: Please use SETPFF to set a valid PFF file.'
      return, -1
    endif
    if n_elements(dsstp) eq 0 then dsstp = 1
    if dsstp le 0 then begin
      print, '% MINMAX: Step size must be greater than zero'
      return, -1
    endif
    ind = dsmin
  endelse
  mm = maxstructure
  readstr, mm, no_copy = -1, structure = s, ind, _extra = extrastuff
  ss = size(s) 
  if ss(ss(0)+1) ne 8 then begin
    Print, '% MINMAX: Unable to read dataset ',strtrim (dsmin, 2), $
      ' into a valid structure'
    return, -1
  endif
  pftype = s.type
  if s.type eq 3 then begin
    Print, '% MINMAX: Not valid with GRID type data. Dataset = ', $
      strtrim(dsmin, 2)
    return, -1
  endif
  if s.type eq 4 then begin
    Print, '% MINMAX: Not valid with CONDUCTOR type data. Dataset = ', $
        strtrim(dsmin, 2)
    return, -1
  endif
  if s.type eq 2 then nv = s.natt else nv = s.nvect
  if nv le 0 then begin
    print, '% MINMAX: No vector components/attributes found'
    return, -1
  endif
  if ((vec gt nv) or (vec lt -1)) then begin
    print, '% MINMAX: Vector component must be in the range [-1, ' $
      +strtrim(nv, 2)+']'
    return, -1
  endif
  if vec lt 0 then result = make_array(2) else result  = make_array(2, nv)

  for i = 1, nv do junk = execute('result(*, i-1) = s.v'+strtrim(i,2)+'(0)')  

  for jj = dsmin, dsmax, dsstp do begin
    if dsmin eq 0 then ind = array(jj) else ind = jj
    tot = 0.0
    stopp = 0

    if ind ne dsmin then begin
      readstr, mm, no_copy = -1, structure = s, ind, _extra = extrastuff
      ss = size(s)
      if ss(ss(0)+1) ne 8 then begin
        Print, '% MINMAX: Unable to read dataset ',strtrim (dsmin, 2), $
          ' into a valid structure'
        return, -1
      endif
      if s.type ne pftype then begin
        print, '% MINMAX: All datasets must be the same data type.'
        print, '       Dataset, Dataset type, First dataset type:', $
            ind, s.type, pftype
        return, -1
      endif

      if s.type eq 2 then nvec = s.natt else nvec = s.nvect

      if nv ne nvec then begin
        print, '% MINMAX: All datasets must have the same number of vectors'
        print, '       Dataset, Number of Vectors,  First dataset number:', $
            ind, nvec, nv
        return, -1
      endif
    endif
    if vec le 1 then begin
      if vec lt 0 then tot = s.v1*s.v1 else begin
        vmin=min(s.v1, max = vmax)
        result(0, 0) = result(0,0) < vmin
        result(1, 0) = result(1,0) > vmax
      endelse
      if (vec eq 1 or nv eq 1) then stopp = 1
    endif
    if stopp eq  0 then begin
      if ((vec le 0) or (vec eq 2)) then begin
        if vec lt 0 then tot = tot + s.v2*s.v2 else begin
          vmin=min(s.v2, max = vmax)
          result(0, 1) = result(0,1) < vmin
          result(1, 1) = result(1,1) > vmax
        endelse
        if (vec eq 2 or nv eq 2) then stopp = 1
      endif
      if stopp eq  0 then begin
        if ((vec le 0) or (vec eq 3)) then begin
          if vec lt 0 then tot = tot + s.v3*s.v3 else begin
            vmin=min(s.v3, max = vmax)
            if vec ge 0 then begin
              result(0, 2) = result(0,2) < vmin
              result(1, 2) = result(1,2) > vmax
            endif
          endelse
          if (vec eq 3 or nv eq 3) then stopp = 1
        endif
        if stopp eq  0 then begin
          if ((vec le 0) or (vec eq 4)) then begin
            if vec lt 0 then tot = tot + s.v4*s.v4 else begin
              vmin=min(s.v4, max = vmax)
              if vec ge 0 then begin
                result(0, 3) = result(0,3) < vmin
                result(1, 3) = result(1,3) > vmax
              endif
            endelse
            if (vec eq 4 or nv eq 4) then stopp = 1
          endif
          if stopp eq  0 then begin
            if ((vec le 0) or (vec eq 5)) then begin
              if vec lt 0 then tot = tot + s.v5*s.v5 else begin
                vmin=min(s.v5, max = vmax)
                if vec ge 0 then begin
                  result(0, 4) = result(0,4) < vmin
                  result(1, 4) = result(1,4) > vmax
                endif
              endelse
              if (vec eq 5 or nv eq 5) then stopp = 1
            endif
            if stopp eq  0 then begin
              if ((vec le 0) or (vec eq 6)) then begin
                if vec lt 0 then tot = tot + s.v6*s.v6 else begin
                  vmin=min(s.v6, max = vmax)
                  if vec ge 0 then begin
                    result(0, 5) = result(0,5) < vmin
                    result(1, 5) = result(1,5) > vmax
                  endif
                endelse
              endif
            endif
          endif
        endif
      endif
    endif
    if vec lt 0 then begin
      vmin = min(sqrt(tot), max = vmax)
      if ind gt dsmin then begin
        result(0)= result(0) < vmin
        result(1)= result(1) > vmax
      endif else begin
        result(0)= vmin
        result(1)= vmax
      endelse
    endif
  endfor
  if vec gt 0 then result = reform(result(*, vec-1))
  if keyword_set(printit) then begin
    if vec gt 0 then begin
      print, 'Range of vector component ', strtrim(vec, 2), ':', result
    endif else if vec lt 0  then begin
      print, 'Range of vector magnitude is:', result
    endif else begin
      for i = 1, nv do $
        print, 'Range of vector component ', strtrim(i, 2), ':', result(*,i-1)
    endelse
  endif
  return, result
  end
