; ------------------------------------------------------------------------ 
; $Id: mulstr.pro,v 1.7 2010/12/08 16:23:45 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
 Pro  mulstr, wdf1, wdf2, wdf3, wdf4, clabel = clabel, master = masterin, $
              cubic = cubic, structure = structure, dot = dot, cross = cross
;
;+
; NAME:
;       MULSTR
;
; PURPOSE:
;       Multiply a field structure array to another field structure array 
;         or to a constant 
;       Store the result in the initial field structure array or 
;         another structure array
;       All attribute or vector quantities are multiplied.
; CATEGORY:
;       Math 
; CALLING SEQUENCE:
;       MULSTR, STR1, STR2,  [, STR3]
;       MULSTR, STR1, 0, VAR, [, STR3]
; INPUTS:
;       STR1:    Structure or structure array number of the first structure
;                The output is the same form as the first structure.
;                If STR1 is an IDL structure and the result is to be saved
;                  as an IDL structure in STR3 then the keyword: STRUCTURE
;                  must be set and STR3 must be a variable.
;       STR2:    Structure or structure array number of the second structure
;  or
;       0, VAR    Variable or constant to multiply the first structure
;                 NOTE: CHASTR may be used.
;                       CHASTR, STR1, SCALE = VAR
; OPTIONAL INPUT:
;       STR3:    Structure variable or structure array number for the sum
;                If not specified the product is stored in STR1 (first array)
;                The product is in the same form as the input (STR1).
; KEYWORDS:
;       The following two keywords must be used only with 2 structures that each are
;       3 dimensional, 3 vector structures.  
;       DOT:     Perform the dot product.
;                The result is a single vector structure - V
;                V = X1*X2 + Y1*Y2 + Z1*Z2
;                Where:
;                  X, Y and Z are Vectors 1, 2 and 3 of Structures 1 and 2
;                  and 1 and 2 refer to the STR1 and STR2 structures.
;                  that is, X1 is the first vector of STR1
;                           X2 is the first vector of STR2
;                           Y1 is the second vector of STR1
;                            etc.
;       CROSS:   Perform the cross product of the two vectors.  The product has 
;                3 vector components.  (Notation is the same as for the dot product.)
;                  Vector1 = Y1*Z2 - Y2*Z1
;                  Vector2 = Z1*X2 - X1*Z2
;                  Vector3 = X1*Y2 - X2*Y1
;                  or in IDL notation:
;                  V = A x B
;                  V[0] = A[1]*B[2]-B[1]*A[2]
;                  V[1] = B[0]*A[2]-A[0]*B[2]
;                  V[2] = A[0]*B[1]-B[0]*A[1]
;       STRUCTURE: Optional keyword which must be present if an idl structure
;                is to be saved in the variable STR3. See example.
;       CLABEL:  Optional label for the combined structures.
;       MASTER:  Optional grid specification for single block, two dimensional
;                data.  Normally this type of data is put onto a uniform grid
;                with a spacing equal to the minimum spacing anywhere for either
;                grid.  If master = 1, the grid STR1 is used for the result;
;                STR2 is interpolated onto the grid of STR1.
;                If master = 2, then STR1 is interpolated onto the grid for
;                STR2 and the result has the same grid as STR2.
;                If MASTER is positive, the master grid will not be changed.
;                If MASTER is negative (or zero), the master grid will be
;                reduced to the region of overlap between STR1 and STR2.
;                For 3-Dimensional or multi-block data MASTER must be 1 or 2.
;                  The data will be put onto the grid of either the first or
;                  second structure array grid.  Missing values are set to zero.
;       CUBIC:   If set a cubic interpolation will be used to match single
;                block data on different grids
; OUTPUTS:
;       NONE    The quotient is stored in a structure array in the common block
; COMMON BLOCKS:
;       QSCOM.DAT QSSTRCOM.DAT
; RESTRICTIONS:
;     If not a single block of data and two dimensions then:
;       STR1 array must be defined and have valid data
;       STR2 or 0 and a Variable must be defined
;     If master is specified, there are no additional restrictions.
;       otherwise:
;     If not a single block of data and two dimensions then:
;       STR1 and STR2 must both be field arrays and have the same number
;         of vector components.
;       The SIZE of STR1 and STR2 must be the same
;       The values of the spatial coordinates must be equal to about 
;       3e-5(ie.- max(abs(difference))/max(value) lt 3.1e-5 )
;       The check on spatial coordinates is done globally rather than 
;         on a block by block basis.
;       If a structure has three dimensions, but is degenerate in one of them
;         that axis need not be equal.
;     If a single block in two dimensions then the data will be interpolated to
;       a common spatial scale using a cubic bilinear interpolation.
; PROCEDURE:
;       If STR1 or STR2 is a type 8 or type 12 structure.
;       If str.v1 has the same number of elements as str.v2 then:
;         MULSTR returns str1.v1*str2.v1 or str1.v1*VAR
;           with the spacial values from STR1.
;         If str1.nvect is lt str2.nvect the unmatched vectors from str2 are
;           ignored.
;       Note:  Type 8 or 12 structures are not checked for spatial values.
;
;     For type 1 structures:
;       If STR2 is specified:
;            Each vector component, of STR1 is multiplied to 
;                each vector component of STR2
;       If 0, VAR is specified:
;            VAR is multiplied to each attribute of STR1
;       In either case, the product is stored in STR3, if specified, or STR1, 
;       if STR3 is not specified.
; EXAMPLES:
;       Multiply structure arrays 2 and 3 and store the result in structure 8
;               MULSTR, 2, 3, 8
;       Multiply the variable OFFSET to structure  array 2 and store the product
;       in structure 8
;               MULSTR, 2, 0, offset, 8
;       Multiply structure arrays 3 and 8 and store the result in structure 10
;       with a dataset comment, 'Total POWER - Current * Voltage'
;               MULSTR, 3, 8, 10, c=  'Total POWER - Current * Voltage'
;       Assume f is an idl structure.
;             MULSTR, f, 2 ; Multiply f to Structure array 2 and return the 
;                     product as an idl structure in variable f.
;             MULSTR, f, 2, 3 ; Multiply f to Structure array 2 and return the
;                     product to structure array 3
;                   Note: A warning will be printed. To avoid the warning use:
;                           i2s, f, 3 & MULSTR, 3, 2
;             MULSTR, f, 2, g, /s ; Multiply f to Structure array 2 and return 
;                           the product as an idl structure in variable g
;                  Note: If g is not a variable then the results will be lost.
;
; MODIFICATION HISTORY:
;      Original version, L. P. Mix, JULY 15, 1992
;-
;
; insert a common block
;
@qscom.dat
;  Common block
@qsstrcom.dat
;       return on an error
      on_error, 1
;
      if n_elements (structinit) eq 0 then qsstruct_init
      if n_elements (qsplotinit) eq 0 then qsinit
;
      nparams = n_params()
      if nparams eq 0 then begin
        print,'% Format: ', $
       'MULSTR, STR1, STR2,  [, STR3] [, CLABEL = clabel]'
        print, '                  [, MASTER = master] [, /CUBIC]'
        print,' or        MULSTR, STR1, 0, VAR, [, STR3] [, CLABEL = clabel]'
        return
      endif
;
      if nparams lt 2 then begin
        print,'% MULSTR: Procedure must have at least 2 parameters'
        return
      endif
;
      if n_elements(wdf1) eq 0 or  n_elements(wdf2) eq 0 then begin
        print, '% MULSTR: Parameters must be defined'
        return
      endif
;
      array1 = get_valid_structure( wdf1, 'MULSTR', lcstr=lcstr, ierror=ier)
      if ier ne 0 then return
      if ((array1.type ne 8) and (array1.type ne 1) and (array1.type ne 12)) then begin
        print, '% MULSTR:  Type 1, Type 8, and Type 12 field structures only'
        return
      endif
;
      st2 = size(wdf2, /type)

      if st2[0] lt 4 then if wdf2 eq 0 then begin
        if n_elements (wdf3) eq 0 then begin
          print,'% MULSTR: Constant is undefined'
          return
        endif
        array1.v1 = array1.v1 * wdf3(0)
        if array1.nvect gt 1 then  array1.v2 = array1.v2 * wdf3(0)
        if array1.nvect gt 2 then  array1.v3 = array1.v3 * wdf3(0)
        if array1.nvect gt 3 then  array1.v4 = array1.v4 * wdf3(0)
        if array1.nvect gt 4 then  array1.v5 = array1.v5 * wdf3(0)
        if array1.nvect gt 5 then  array1.v6 = array1.v6 * wdf3(0)


        if n_elements(clabel) gt 0 then array1.clab = strtrim(clabel, 2)

        if nparams gt 3 then begin
          if lcstr eq 1 then begin
            if (keyword_set(structure) and arg_present(WDF4)) then $
              wdf4 = temporary(array1) else begin
              print, $
              '% MULSTR: Keyword STRUCTURE must be set to save result as a structure'
              print, '          and WDF4 must be a simple variable'
              wdf4= get_save_str(wdf4, get_maxstr())
              print, '% MULSTR: Result will be saved in array ', strtrim(WDF4,2)
              i2s, array1, wdf4, /no_copy
            endelse
          endif else begin
            wdf4 = get_save_str(wdf4, wdf1)
            i2s, array1, wdf4, /no_copy
          endelse
        endif else begin
          if lcstr eq 1 then begin
            if arg_present(WDF1) then wdf1 = temporary(array1) else begin
              print, '% MULSTR: WDF1 must be a simple variable to return a structure'
              wdf4= get_save_str(wdf4, get_maxstr())
              print, '% MULSTR: Result will be saved in array ', strtrim(WDF4,2)
              i2s, array1, wdf4, /no_copy
            endelse
          endif else i2s, array1, wdf1, /no_copy
        endelse
        return
      endif

      array2 = get_valid_structure( wdf2, 'MULSTR', ierror=ier)
      if ier ne 0 then return
      if array2.type eq 8 or array1.type eq 8 or array2.type eq 12 or $
        array1.type eq 12  then begin
        n1 = n_elements(array1.v1)
        n2 = n_elements(array2.v1)
        if n1 ne n2 then begin
          Print, $
            '% MULSTR: Field quantities must have then same number of values'
          return
        endif
;       Check for special multiply options
        if keyword_set(dot) then begin
          if nvect ne 3 then begin
            print, '% MULSTR: DOT keyword requires 3 vectors'
            return
          endif
          array1.nvect = 1
          array1.v1 = array1.v1*array2.v1+array1.v2*array2.v2+array1.v3*array2.v3
          array1.v1lab = 'Dot product'
          array1.v2 = 0
          array1.v3 = 0
        endif else if keyword_set(cross) then begin
          if nvect ne 3 then begin
            print, '% MULSTR: CROSS keyword requires 3 vectors'
            return
          endif
          temp1 = array1.v1
          temp2 = array1.v2
          array1.v1 = temp2*array2.v3 - array1.v3*array2.v2
          array1.v2 = array1.v3*array2.v1 - temp1*array2.v3
          array1.v3 = temp1*array2.v2 - temp2*array2.v1
;                  V[0] = A[1]*B[2]-B[1]*A[2]
;                  V[1] = B[0]*A[2]-A[0]*B[2]
;                  V[2] = A[0]*B[1]-B[0]*A[1]
          
        endif else begin  
          array1.v1 =array1.v1 * array2.v1
          if array1.nvect gt array2.nvect then nvect = array1.nvect $
            else nvect = array2.nvect
          if nvect gt 1 then  array1.v2 = array1.v2 * array2.v2
          if nvect gt 2 then  array1.v3 = array1.v3 * array2.v3
          if nvect gt 3 then  array1.v4 = array1.v4 * array2.v4
          if nvect gt 4 then  array1.v5 = array1.v5 * array2.v5
          if nvect gt 5 then  array1.v6 = array1.v6 * array2.v6
        endelse
;
      endif else begin
        if array1.type ne 1 or array2.type ne 1 then begin
          Print, '% MULSTR: Both structure arrays must be field structures'
          print, '          TYPES =', array1.type, array2.type
          return
        endif
        check_identical, array1, array2, lcident
        if n_elements(masterin) eq 0 then master = 0 else master = masterin
        if (master lt -2 or master gt 2) then master = 1
        if lcident eq 0 and master ne 0 then begin
          if array1.block gt 1 or array2.block gt 1 then begin
            master = abs(master)
            if array1.ndim eq array2.ndim then begin
              if array1.nvect ne array2.nvect then begin
                print, ' Structures must have the same number of vectors.'
                return
              endif
              if master eq 2 then begin
                temp = array2
                array2 = array1
                array1 = temp
              endif else temp = array1
              temp.v1= temp.v1*0.0 + 1.00
              if temp.nvect gt 1 then begin
                temp.v2= temp.v2*0.0+ 1.00
                if temp.nvect gt 2 then begin
                  temp.v3= temp.v3*0.0+ 1.00
                  if temp.nvect gt 3 then begin
                    temp.v4= temp.v4*0.0+ 1.00
                    if temp.nvect gt 4 then begin
                      temp.v5= temp.v5*0.0+ 1.00
                      if temp.nvect gt 5 then begin
                        temp.v6= temp.v6*0.0+ 1.00
                      endif
                    endif
                  endif
                endif
              endif
              ndim = array1.ndim
              if ndim lt 2 or ndim gt 3 then begin
                print, ' Present version is only for 2D and 3D data'
                return
              endif
              nblk = array2.block
              eps = make_array(ndim)
              for i = 0, ndim-1 do begin
                eps(i) = (temp.range(1, i, 0)- temp.range(0, i, 0))* 6.1e-5
              endfor
              ok = where (eps eq 0.0, complement = notok, nok)
              if nok gt 0 then eps[ok] = min(eps[notok])
              if array1.two ne 0 or array2.two ne 0 then begin
                if array1.two ne array2.two then begin
                  print, $
                   ' Degenerate arrays must be degenerate in the same coordinate'
                  print, array1.two,array2.two
                  return
                endif
                for i = 0, array2.block do begin
                   array2.range(1,array1.two-1,i)= $
                     array1.range(1,array1.two-1,0) + 2*eps(array1.two-1)
                   array2.range(0,array1.two-1,i)= $
                     array1.range(0,array1.two-1,0) - 2*eps(array1.two-1)
                endfor
              endif
              if n_elements (cubic) eq 0 then cubic = 0
              for i = 1, array1.block do begin
                x1 = temp.x1(temp.locx(0, i-1): temp.locx(0, i)-1)
                x2 = temp.x2(temp.locx(1, i-1): temp.locx(1, i)-1)
                if ndim gt 2 then $
                  x3 = temp.x3(temp.locx(2, i-1): temp.locx(2, i)-1)
                for j = 1, nblk do begin
                  if ((ndim eq 2 and $
                not (array1.range(1,0,i) le array2.range(0,0,j)+eps(0) $
                  or array1.range(0,0,i)+eps(0) ge array2.range(1,0,j) $
                  or array1.range(1,1,i) le array2.range(0,1,j)+eps(1) $
                  or array1.range(0,1,i)+eps(1) ge array2.range(1,1,j))) $
            or  not (array1.range(1,0,i) le array2.range(0,0,j)+eps(0) $
                  or array1.range(0,0,i)+eps(0) ge array2.range(1,0,j) $
                  or array1.range(1,1,i) le array2.range(0,1,j)+eps(1) $
                  or array1.range(0,1,i)+eps(1) ge array2.range(1,1,j) $
                  or array1.range(1,2,i) le array2.range(0,2,j)+eps(2) $
                  or array1.range(0,2,i)+eps(2) ge array2.range(1,2,j))) $
                  then begin

                    x1b = array2.x1(array2.locx(0, j-1): array2.locx(0, j)-1)
                    x2b = array2.x2(array2.locx(1, j-1): array2.locx(1, j)-1)
                    if ndim eq 3 then $
                      x3b = array2.x3(array2.locx(2, j-1): array2.locx(2, j)-1)
                    if array1.two ne 1 then $
                      okx = where (x1 ge array2.range(0,0,j)-eps(0) and $
                          x1 le array2.range(1,0,j)+eps(0), nokx) else nokx=2
                    if array1.two ne 2 then $
                      oky = where (x2 ge array2.range(0,1,j)-eps(1) and $
                          x2 le array2.range(1,1,j)+eps(1), noky) else noky=2
                    if ndim eq 3 then begin
                      if array1.two ne 3 then $
                      okz = where (x3 ge array2.range(0,2,j)-eps(2) and $
                          x3 le array2.range(1,2,j)+eps(2), nokz) else nokz =2
                      if nokx lt 1 or noky lt 1 or nokz lt 1 then begin
                         print, 'Possible problem in MULSTR',nokx,noky,nokz,i,j
                         return
                      endif
                    endif else if nokx lt 1 or noky lt 1 then begin
                       print, 'Possible problem in MUlSTR',nokx,noky,i,j
                       return
                    endif
                    if array1.two ne 1 then begin
                      x0= okx(0) & xn = okx(nokx-1)
                      if nokx -1 ne xn-x0 then begin
                        print,'Problem with X values not Consecutive',x0,xn,nokx
                        return
                      endif
                    endif
                    if array1.two ne 2 then begin
                      y0= oky(0) & yn = oky(noky-1)
                      if noky -1 ne yn-y0 then begin
                        print,'Problem with Y values not Consecutive',y0,yn,noky
                        return
                      endif
                    endif
                    if ndim eq 3 and array1.two ne 3 then begin
                      z0= okz(0) & zn = okz(nokz-1)
                      if nokz -1 ne zn-z0 then begin
                        print,'Problem with Z values not Consecutive',z0,zn,nokz
                        return
                      endif
                    endif
                    if array1.two ne 1 then $
                      p = get_yarr(x1b, findgen(n_elements(x1b)), x1(x0:xn), $
                             error = 1.5*eps(0))
                    if array1.two ne 2 then $
                      q = get_yarr(x2b, findgen(n_elements(x2b)), x2(y0:yn), $
                             error = 1.5*eps(1))
                    if array1.two ne 3 and ndim gt 2 then $
                      r = get_yarr(x3b, findgen(n_elements(x3b)), x3(z0:zn), $
                             error = 1.5*eps(2))
                    v =reform(array2.v1(array2.locv(j-1): $
                       array2.locv(j)-1), array2.size(*,j-1))
                    if ((ndim eq 2) or $
                      (ndim eq 3 and array1.two ne 0))then begin
                      if ndim eq 3 then begin
                        if array1.two eq 1 then begin
                          p = temporary(q)
                          q = temporary(r)
                          x0 = y0 & xn = yn & y0=z0 & yn= zn
                        endif else if array1.two eq 2 then begin
                          q= temporary(r)
                          y0=z0 & yn= zn
                        endif
                      endif
                      if ndim eq 3 then v = reform(v, /overwrite)
                      newv = interpolate(v, p, q, cubic= cubic, /grid)
                      tempv1 = reform(temp.v1(temp.locv(i-1): $
                         temp.locv(i)-1), temp.size(*,i-1))
                      if ndim eq 3 then tempv1 = reform(tempv1, /overwrite)
                      tempv1(x0:xn, y0:yn) = newv
                      temp.v1(temp.locv(i-1):temp.locv(i)-1) = tempv1
                      if temp.nvect gt 1 then begin
                        v =reform(array2.v2(array2.locv(j-1):array2.locv(j)-1), $
                           array2.size(*,j-1))
                        if ndim eq 3 then  v= reform(v, /overwrite)
                        newv = interpolate(v, p, q, cubic= cubic, /grid)
                        tempv2 = reform(temp.v2(temp.locv(i-1):temp.locv(i)-1), $
                          temp.size(*,i-1))
                        if ndim eq 3 then tempv2 = reform(tempv2, /overwrite)
                        tempv2(x0:xn, y0:yn) = newv
                        temp.v2(temp.locv(i-1):temp.locv(i)-1) = tempv2
                      if temp.nvect gt 2 then begin
                        v =reform(array2.v3(array2.locv(j-1):array2.locv(j)-1), $
                           array2.size(*,j-1))
                        if ndim eq 3 then v = reform(v, /overwrite)
                        newv = interpolate(v, p, q, cubic= cubic, /grid)
                        tempv3 = reform(temp.v3(temp.locv(i-1):temp.locv(i)-1), $
                          temp.size(*,i-1))
                        if ndim eq 3 then tempv3 = reform(tempv3, /overwrite)
                        tempv3(x0:xn, y0:yn) = newv
                        temp.v3(temp.locv(i-1):temp.locv(i)-1) = tempv3
                      if temp.nvect gt 3 then begin
                        v =reform(array2.v4(array2.locv(j-1):array2.locv(j)-1), $
                           array2.size(*,j-1))
                        if ndim eq 3 then  v = reform(v, /overwrite)
                        newv = interpolate(v, p, q, cubic= cubic, /grid)
                        tempv4 = reform(temp.v4(temp.locv(i-1):temp.locv(i)-1), $
                          temp.size(*,i-1))
                        if ndim eq 3 then tempv4 = reform(tempv4, /overwrite)
                        tempv4(x0:xn, y0:yn) = newv
                        temp.v4(temp.locv(i-1):temp.locv(i)-1) = tempv4
                      if temp.nvect gt 4 then begin
                        v =reform(array2.v5(array2.locv(j-1):array2.locv(j)-1), $
                           array2.size(*,j-1))
                        if ndim eq 3 then v= reform(v, /overwrite)
                        newv = interpolate(v, p, q, cubic= cubic, /grid)
                        tempv5 = reform(temp.v5(temp.locv(i-1):temp.locv(i)-1), $
                          temp.size(*,i-1))
                        if ndim eq 3 then tempv5 = reform(tempv5, /overwrite)
                        tempv5(x0:xn, y0:yn) = newv
                        temp.v5(temp.locv(i-1):temp.locv(i)-1) = tempv5
                      if temp.nvect gt 5 then begin
                        v =reform(array2.v6(array2.locv(j-1):array2.locv(j)-1), $
                           array2.size(*,j-1))
                        if ndim eq 3 then v = reform(v, /overwrite)
                        newv = interpolate(v, p, q, cubic= cubic, /grid)
                        tempv6 = reform(temp.v6(temp.locv(i-1):temp.locv(i)-1), $
                          temp.size(*,i-1))
                        if ndim eq 3 then tempv6 = reform(tempv6, /overwrite)
                        tempv6(x0:xn, y0:yn) = newv
                        temp.v6(temp.locv(i-1):temp.locv(i)-1) = tempv6
                      endif ; for gt 5
                      endif ; for gt 4
                      endif ; for gt 3
                      endif ; for gt 2
                      endif ; for gt 1
                    endif else begin
;        3d array
                      newv = interpolate(v, p, q,r, cubic= cubic, /grid)
                      tempv1 = reform(temp.v1(temp.locv(i-1): $
                         temp.locv(i)-1), temp.size(*,i-1))
                      tempv1(x0:xn, y0:yn, z0:zn) = newv
                      temp.v1(temp.locv(i-1):temp.locv(i)-1) = tempv1
                      if temp.nvect gt 1 then begin
                        v =reform(array2.v2(array2.locv(j-1):array2.locv(j)-1), $
                           array2.size(*,j-1))
                        newv = interpolate(v, p, q,r, cubic= cubic, /grid)
                        tempv2 = reform(temp.v2(temp.locv(i-1):temp.locv(i)-1), $
                          temp.size(*,i-1))
                        tempv2(x0:xn, y0:yn, z0:zn) = newv
                        temp.v2(temp.locv(i-1):temp.locv(i)-1) = tempv2
                      if temp.nvect gt 2 then begin
                        v =reform(array2.v3(array2.locv(j-1):array2.locv(j)-1), $
                           array2.size(*,j-1))
                        newv = interpolate(v, p, q, r, cubic= cubic, /grid)
                        tempv3 = reform(temp.v3(temp.locv(i-1):temp.locv(i)-1), $
                          temp.size(*,i-1))
                        tempv3(x0:xn, y0:yn, z0:zn) = newv
                        temp.v3(temp.locv(i-1):temp.locv(i)-1) = tempv3
                      if temp.nvect gt 3 then begin
                        v =reform(array2.v4(array2.locv(j-1):array2.locv(j)-1), $
                           array2.size(*,j-1))
                        newv = interpolate(v, p, q, r, cubic= cubic, /grid)
                        tempv4 = reform(temp.v4(temp.locv(i-1):temp.locv(i)-1), $
                          temp.size(*,i-1))
                        tempv4(x0:xn, y0:yn, z0:zn) = newv
                        temp.v4(temp.locv(i-1):temp.locv(i)-1) = tempv4
                      if temp.nvect gt 4 then begin
                        v =reform(array2.v5(array2.locv(j-1):array2.locv(j)-1), $
                           array2.size(*,j-1))
                        newv = interpolate(v, p, q, r, cubic= cubic, /grid)
                        tempv5 = reform(temp.v5(temp.locv(i-1):temp.locv(i)-1), $
                          temp.size(*,i-1))
                        tempv5(x0:xn, y0:yn, z0:zn) = newv
                        temp.v5(temp.locv(i-1):temp.locv(i)-1) = tempv5
                      if temp.nvect gt 5 then begin
                        v =reform(array2.v6(array2.locv(j-1):array2.locv(j)-1), $
                           array2.size(*,j-1))
                        newv = interpolate(v, p, q, r, cubic= cubic, /grid)
                        tempv6 = reform(temp.v6(temp.locv(i-1):temp.locv(i)-1), $
                          temp.size(*,i-1))
                        tempv6(x0:xn, y0:yn, z0:zn) = newv
                        temp.v6(temp.locv(i-1):temp.locv(i)-1) = tempv6
                      endif ; for gt 5
                      endif ; for gt 4
                      endif ; for gt 3
                      endif ; for gt 2
                      endif ; for gt 1
                    endelse
                  endif
                endfor
              endfor
              if master eq 2 then begin
                array2 = temporary(array1)
                array1 = temporary(temp)
              endif else array2 = temporary(temp)
              lcident = 1
            endif
          endif
        endif 
        if lcident eq 0 then begin
          if (array1.block ne 1) or (array2.block ne 1) then begin
            print, 'Cannot multiply structures.'
            print, '   Multiple block structures of different sizes'
            return
          endif
          if (array1.ndim gt 2 and array1.two eq 0 ) or $
            (array2.ndim gt 2 and array2.two eq 0 ) then begin
            print, 'Cannot multiply structures.'
            print, '   More than two dimensions of different sizes'
            return
          endif
          if array1.nvect ne array2.nvect then begin
            print, 'Cannot multiply structures.'
            print,'  Arrays must have the same number of vector components'
            return
          endif
          if array1.two ne array2.two then begin
            print, 'Cannot multiply structures.'
            print,'  Degenerate dimensions must be equal. Change to a 2D image'
            return
          endif
          xa = array1.x1
          xb = array2.x1
          ya = array1.x2
          yb = array2.x2
          two = array1.two
          if two eq 1 then begin
            xa = ya
            xb = yb
            ya = array1.x3
            yb = array2.x3
          endif else if two eq 2 then begin
            ya = array1.x3
            yb = array2.x3
          endif
          nxa = n_elements (xa)
          nya = n_elements (ya)
          nxb = n_elements (xb)
          nyb = n_elements (yb)
          if ier ne 0 then return
          nx1 = n_elements(x1)
          ny1 = n_elements(y1)
          ntot = nx1*ny1
          ndim = array1.ndim
          tlab = 'Contour'
          ok1 =  strpos(array1.clab, ';;')
          if ok1 gt 0 then clab_1 = strmid(array1.clab,0, ok1[0]) else $
                         clab_1 = array1.clab
          ok2 =  strpos(array2.clab, ';;')
          if ok2 gt 0 then clab_2 = strmid(array2.clab,0, ok2[0]) else $
                         clab_2 = array2.clab
          if array1.file eq array2.file then file=array1.file else file = ''

          if (keyword_set(dot) || keyword_set(cross)) then begin
            if array1.nvect ne 3 then begin
              print, '% MULSTR:  Dot and Cross product require a 3 dimensional structure'
              return
            endif
            v4 = 0
            v5 = 0
            v6 = 0
            va = reform(array1.v1, nxa, nya)
            vb = reform(array2.v1, nxb, nyb)
            becky, a1v1, a2v1, x1, y1, va, xa, ya, vb, xb, yb, master, cubic, ier, $
               default = 1.0
            va = reform(array1.v2, nxa, nya)
            vb = reform(array2.v2, nxb, nyb)
            becky, a1v2, a2v2, x1, y1, va, xa, ya, vb, xb, yb, master, cubic, ier, $
               default = 1.0
            va = reform(array1.v3, nxa, nya)
            vb = reform(array2.v3, nxb, nyb)
            becky, a1v3, a2v3, x1, y1, va, xa, ya, vb, xb, yb, master, cubic, ier, $
               default = 1.0

            if keyword_set(dot) then begin
              clab = 'Dot Product of '+clab_1+' and '+clab_2
              array1.nvect = 1
              v1 = a1v1*a2v1+a1v2*a2v2+a1v3*a2v3
              v1lab = 'Dot product'
              v2 = 0
              v3 = 0
            endif else begin
              v1 = reform(a1v2*a2v3 - a1v3*a2v2, ntot)
              v2 = reform(a1v3*a2v1 - a1v1*a2v3, ntot)
              v3 = reform(a1v1*a2v2 - a1v2*a2v1, ntot)
;                  V[0] = A[1]*B[2]-B[1]*A[2]
;                  V[1] = B[0]*A[2]-A[0]*B[2]
;                  V[2] = A[0]*B[1]-B[0]*A[1]
              clab = 'Cross Product of '+clab_1+' and '+clab_2
            endelse
          endif else begin
          clab = 'Product of '+clab_1+' and '+clab_2
          va = reform(array1.v1, nxa, nya)
          vb = reform(array2.v1, nxb, nyb)
          
          becky, vaa, vbb, x1, y1, va, xa, ya, vb, xb, yb, master, cubic, ier, $
             default = 1.0
          nx1 = n_elements(x1)
          ny1 = n_elements(y1)
          ntot = nx1*ny1
          v1 = reform (vaa*vbb, ntot)
          if array1.nvect gt 1 then begin
            va = reform(array1.v2, nxa, nya)
            vb = reform(array2.v2, nxb, nyb)
            becky,vaa, vbb, x1, y1, va, xa, ya, vb, xb, yb, master, cubic, ier, $
              default = 1.0
            if ier ne 0 then return
            v2 = reform (vaa*vbb, ntot)
          endif else v2 = 0.0
          if array1.nvect gt 2 then begin
            va = reform(array1.v3, nxa, nya)
            vb = reform(array2.v3, nxb, nyb)
            becky,vaa, vbb, x1, y1, va, xa, ya, vb, xb, yb, master, cubic, ier, $
              default = 1.0
            if ier ne 0 then return
            v3 = reform (vaa*vbb, ntot)
          endif else v3 = 0.0
          if array1.nvect gt 3 then begin
            va = reform(array1.v4, nxa, nya)
            vb = reform(array2.v4, nxb, nyb)
            becky,vaa, vbb, x1, y1, va, xa, ya, vb, xb, yb, master, cubic, ier, $
              default = 1.0
            if ier ne 0 then return
            v4 = reform (vaa*vbb, ntot)
          endif else v4 = 0.0
          if array1.nvect gt 4 then begin
            va = reform(array1.v5, nxa, nya)
            vb = reform(array2.v5, nxb, nyb)
            becky,vaa, vbb, x1, y1, va, xa, ya, vb, xb, yb, master, cubic, ier, $
              default = 1.0
            if ier ne 0 then return
            v5 = reform (vaa*vbb, ntot)
          endif else v5 = 0.0
          if array1.nvect gt 5 then begin
            va = reform(array1.v6, nxa, nya)
            vb = reform(array2.v6, nxb, nyb)
            becky,vaa, vbb, x1, y1, va, xa, ya, vb, xb, yb, master, cubic, ier, $
              default = 1.0
            if ier ne 0 then return
            v6 = reform (vaa*vbb, ntot)
          endif else v6 = 0.0
          endelse
          range = array1.range
          if ndim eq 2 then begin
            range(0, 0, *) = x1(0)
            range(1, 0, *) = x1(nx1-1)
            range(0, 1, *) = y1(0)
            range(1, 1, *) = y1(ny1-1)
            locx= reform([0L,0L, nx1, ny1], 2,2)
            ssize = reform([nx1, ny1], 2, 1)
            x2 = y1
            x3 = array1.x3
          endif else if two eq 1 then begin
            ssize = reform( [1, nx1, ny1], 3, 1)
            locx=reform([0L, 0L, 0L, 0, nx1, ny1],3, 2)
            range(0,1,*) = x1(0)
            range(1,1,*) = x1(nx1-1)
            range(0,2,*) = y1(0)
            range(1,2,*) = y1(ny1-1)
            x2 = x1
            x3 = y1
            x1 = array1.x1
          endif else if two eq 2 then begin
            ssize = reform( [nx1, 1, ny1], 3, 1)
            locx=reform([0L, 0L, 0L, nx1, 0, ny1],3, 2)
            range(0,0,*) = x1(0)
            range(1,0,*) = x1(nx1-1)
            range(0,2,*) = y1(0)
            range(1,2,*) = y1(ny1-1)
            x3 = y1
            x2 = array1.x2
          endif else begin
            ssize = reform( [nx1, ny1, 1L], 3, 1)
            locx=reform([0L, 0L, 0L, nx1, ny1, 0L],3, 2)
            range(0,0,*) = x1(0)
             range(1,0,*) = x1(nx1-1)
            range(0,1,*) = y1(0)
            range(1,1,*) = y1(ny1-1)
            x2 = y1
            x3 = array1.x3
          endelse
          v1lab = make_array(1, /string, value = array1.v1lab)
          v2lab = make_array(1, /string, value = array1.v2lab)
          v3lab = make_array(1, /string, value = array1.v3lab)
          v4lab = make_array(1, /string, value = array1.v4lab)
          v5lab = make_array(1, /string, value = array1.v5lab)
          v6lab = make_array(1, /string, value = array1.v6lab)
          x1lab = make_array(1, /string, value = array1.x1lab)
          x2lab = make_array(1, /string, value = array1.x2lab)
          x3lab = make_array(1, /string, value = array1.x3lab)
          x4lab = make_array(1, /string, value = '')

          dx1 = 0.0
          dx2 = 0.0
          dx3 = 0.0
          dx4 = 0.0
          if n_elements(x1) gt 1 then check_uniform, x1, dx1
          if n_elements(x2) gt 1 then check_uniform, x2, dx2
          if n_elements(x3) gt 1 then check_uniform, x3, dx3
          array1 = {type:1L, ndim:ndim, nvect:array1.nvect, block:1L, two:two, $
              size:ssize, range: range, order: 1L, slice:array1.slice, $
              locx:locx, locv : [0, ntot], x1:x1, x2: x2, $
              x3:x3, x4:0.0, v1:v1, v2:v2, v3:v3, v4:v4, v5:v5, v6:v6, $
              dx1: dx1, dx2: dx2, dx3: dx3, dx4: dx4, $
              x1lab :x1lab, x2lab :x2lab, x3lab :x3lab, $
              x4lab :x4lab, v1lab :v1lab, v2lab :v2lab, $
              v3lab :v3lab, v4lab :v4lab, v5lab :v5lab, $
              v6lab :v6lab, file: file, spare: array1.spare, tlab : tlab, $
              clab:clab }
        endif else begin
          ok1 =  strpos(array1.clab, ';;')
          if ok1 gt 0 then clab_1 = strmid(array1.clab,0, ok1[0]) else $
                         clab_1 = array1.clab
          ok2 =  strpos(array2.clab, ';;')
          if ok2 gt 0 then clab_2 = strmid(array2.clab,0, ok2[0]) else $
                         clab_2 = array2.clab
          if array1.file eq array2.file then file=array1.file else file = ''


          if (keyword_set(dot) || keyword_set(cross)) then begin
            if array1.nvect ne 3 then begin
              print, '% MULSTR:  Dot and Cross product require a 3 dimensional structure'
              return
            endif
            if keyword_set(dot) then begin

              array1.clab = 'Dot Product of '+clab_1+' and '+clab_2
              array1.nvect = 1
              array1.v1 = array1.v1*array2.v1+array1.v2*array2.v2+array1.v3*array2.v3
              v1lab = 'Dot product'
              v2 = 0
              v3 = 0
            endif else begin
              a1v1 = array1.v1
              a1v2 = array1.v2
              array1.v1 = a1v2*array2.v3 - array1.v3*array2.v2
              array1.v2 = array1.v3*array2.v1 - a1v1*array2.v3
              array1.v3 = a1v1*array2.v2 - a1v2*array2.v1
;                  V[0] = A[1]*B[2]-B[1]*A[2]
;                  V[1] = B[0]*A[2]-A[0]*B[2]
;                  V[2] = A[0]*B[1]-B[0]*A[1]
              array1.clab = 'Cross Product of '+clab_1+' and '+clab_2
            endelse
          endif else begin
            array1.clab = 'Product of '+clab_1+' and '+clab_2
            array1.v1 = array1.v1 * array2.v1
            if array1.nvect gt 1 then  array1.v2 = array1.v2 * array2.v2
            if array1.nvect gt 2 then  array1.v3 = array1.v3 * array2.v3
            if array1.nvect gt 3 then  array1.v4 = array1.v4 * array2.v4
            if array1.nvect gt 4 then  array1.v5 = array1.v5 * array2.v5
            if array1.nvect gt 5 then  array1.v6 = array1.v6 * array2.v6
          endelse
        endelse
      endelse

      if n_elements(clabel) gt 0 then array1.clab = strtrim(clabel, 2)

      if nparams gt 2 then begin
        if lcstr eq 1 then begin
          if keyword_set(structure)  and arg_present(WDF3) then $
            wdf3 = temporary(array1) else begin
            print, $
            '% MULSTR: Keyword STRUCTURE must be set to save result as a structure'
            print, '          and WDF3 must be a simple variable'
            wdf3= get_save_str(wdf3, get_maxstr())
            print, '% MULSTR: Result will be saved in array ', strtrim(WDF3,2)
            i2s, array1, wdf3, /no_copy
          endelse
        endif else begin
          wdf3 = get_save_str(wdf3, wdf1)
          i2s, array1, wdf3, /no_copy
        endelse
      endif else begin
        if lcstr eq 1 then begin
          if arg_present(WDF1) then wdf1 = temporary(array1) else begin
            print, '% MULSTR: WDF1 must be a simple variable to return a structure'
            wdf3= get_save_str(wdf3, get_maxstr())
            print, '% MULSTR: Result will be saved in array ', strtrim(WDF3,2)
            i2s, array1, wdf3, /no_copy
          endelse
        endif else i2s, array1, wdf1, /no_copy
      endelse
      array2=0
      end
