; ------------------------------------------------------------------------ 
; $Id: pff_regrid.pro,v 1.6 2009/09/02 16:21:19 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
;+
; NAME:		PFF_REGRID
; PURPOSE:
;	do density-to-exposure unfold and regridding of scanned PFF image
;	file, and write out a new (smaller) PFF file containing the regridded
;	exposure values. Used as a preprocessor to deal with high-res scanned 
;	data to avoid bogging down Vida.
; CATEGORY:	image pre-preprocessor (scans)
; CALLING SEQUENCE:
;	PFF_REGRID, INFILE [, FITFILE] [, /ZERO_ORIGIN] [,REGRID=regrid]
;              [, XREGRID=xregrid] [, YREGRID=yregrid] [, OUTFILE = outfile]
;              [, /STRUCTURE] [, VALID = valid] [, DEFAULT_VAL = default_val]
; INPUTS:
;	infile	string containing PFF filename to process. User should also
;		provide the extension (.pff, .PFF, etc)
;               Enter 0 or ' ' to use pickfile.
;               If keyword (STRUCTURE) is set, then INFILE is a structure
;                 or structure array number.
;               Structure must be a single block, uniform, type 1 structure.
;      fitfile	string containing film response filename to use. 
;               Enter 0 or ' ' to use pickfile.
;               Should have been created in WDFIT3
;               Can be pre-loaded with READWD3.
;               Values read will be stored in the WDFIT3 common block.
;               If omitted then current values in the WDFIT3 common block are
;               used.
;               If FITFILE is lt 0 then response is not calculated.
;               and a structure is returned in structure array outfile.
;               (See SCALE_REGRID with C1=[0, 1]
; KEYWORDS:
;  STRUCTURE    If set INFILE is a structure or structure array number.
;  VALID        Two element vector with minimum and maximum values for
;               the scaling.  DATA values outside these limits will be
;               set to the values of default.
;               Default: VALID = [min(data, max = mdata), mdata]
;               (See: SCALE_REGRID
;  ZERO_ORIGIN  if nonzero, specifies that x,y axis values should be offset
;		so that lower left corner of image will be at 0,0.
;    regrid     integer >=1 to specify what factor to regrid data by. Note
;		that idl rebin routine is used - averages points. May lose
;		at most regrid-1 of the uppermost rows or columns of scanned
;		data in this process. Equivalent to specifying x_regrid=regrid
;		and y_regrid=regrid.
;               Default value: REGRID = 5
;   x,yregrid	integers >=1 allowing independent regridding factors 
;		in the x and y dimensions. 
;               Default value: X,YREGRID = REGRID
;   default_val Default values for regrid if infile values are outside the range
;               DEFAULT = [0, value at VALID(1)] 
;   OUTFILE     Output file name or a number.
;               If outfile is a number between 1 and get_maxstr(), then the 
;                 result will be stored in "outfile" as a structure.
;               Default: 
;                 1. Creates a new PFF file in current directory, with 
;                    a '_ur' suffix added to root of INFILE if STRUCTURE =0.
;                 2. Saves the result in GET_MAXSTR()-1 if STRUCTURE ne 0.
;              
; SIDE EFFECTS:
;       If OUTFILE is not a number, then PFF_REGRID creates a new PFF file
;       with the as defined by OUTFILE or a new PFF file in current directory,
;	with a '_ur' suffix added to root of filename
; RESTRICTIONS:
;	User should NOT have a file of the same name as the output file in
;	the working directory! Output filename will be modified.
; EXAMPLE:
;       Fit film response data and unfold a pff file and write it out with
;         a reduction in size of 16 (= 4*4), put (0,0) at lower left corner.
;           revida, 0, 1      ; read a vida output file
;           wdfit3, 1, /ylog  ; fit the data to a polynomial
;           pff_regrid, 0 , regrid = 4, /zero
; 
; MODIFICATION HISTORY:
;	Written by Alex B. Filuk, Sandia, 1/17/95
;	ABF (1/25/96) - mods to use readuf3 instead of readpff
;       Completely rewritten: lpmix 8/6/98
;       Added structure keyword: lpmix 10/9/02
;-

PRO pff_regrid, infile, fitfile, outfile = outfile, structure = structure, $
        regrid=regrid, xregrid=xregrid, yregrid=yregrid, valid = valid,  $
        zero_origin=zero_origin, default_val = default_val
      on_error, 1
      common pffcom, initpff, pff_exe
      COMMON savepicker, oldfilter, oldpath
; Common block
@wdfit3_common.dat

t_start = systime(1)			;get starting system time in sec

;
;     check to see if pff has been initialized
;
      regrid_default = 5
      if (n_elements(initpff) eq 0) then init_pff else $
        if (initpff eq 0) then init_pff
      if initpff eq 0 then begin
        print, 'PFF libraries are not supported by this Operating System'
        return
      endif
;

      npar = n_params()
      if npar eq 0 then begin
print, '% Format: PFF_REGRID, INFILE [, FITFILE]  [, OUTFILE = outfile]'
print, '          [, REGRID= regrid] [, XREGRID= xregrid] [, YREGRID= yregrid]'
print, '          [, /ZERO_ORIGIN] [, DEFAULT_VAL= default_val] [, /STRUCTURE]'
        return
      endif
   if keyword_set(structure) then begin
      if n_elements(infile) eq 0 then begin
        print, '% PFF_REGRID: Structure must be defined'
        return
      endif
      ain = infile
      a = get_valid_structure(ain,'PFF_REGRID', lcstr=lcstr, ierror = ier)
      if ier ne 0 then return
      if a.block ne 1 then begin
        print, '% PFF_REGRID: Structure must have a single block'
        return
      endif
      ok = where(strpos(tag_names(a), 'DX') ge 0 , nok)
      if nok eq 0 then begin
        print, '% PFF_REGRID: Structure must have DX values'
        return
      endif
      dx =0 & dy = 0
      dx = a.dx1
      xar = a.x1
      x0 = xar[0]
      xlab = a.x1lab
      if a.two eq 3 then begin
        dy = a.dx2
        yar = a.x2
        y0 = yar[0]
        ylab = a.x2lab
      endif else if a.two eq 2 then begin
        dy = a.dx3
        yar = a.x3
        y0 = yar[0]
        ylab = a.x3lab
      endif else if a.two eq 1 then begin
        dx = a.dx2
        xar = a.x2
        x0 = xar[0]
        xlab = a.x2lab
        dy = a.dx3
        yar = a.x3
        y0 = yar[0]
        ylab = a.x3lab
      endif else nok =0
      if dx eq 0 or dy eq 0 then nok =0
      if nok eq 0 then begin
        print, '% PFF_REGRID: Structure be an image type structure with DX != 0'
        help, /str, a
        a=0
        return
      endif
      clab = a.clab
      tlab = a.tlab
      ok = where(strpos(tag_names(a), 'BLAB') ge 0 , nok)
      if nok eq 0 then blab = a.v1lab else blab = a.blabel[0]
      dimage = get_array(a, -1)
      filename = a.file
      a=0
   endif else begin
      if n_elements (infile) eq 0 then infile = ''
      infile = infile[0]
      if (size(infile))[1] ne 7 then infile = ''
      openpff, infile, error = error
      filename = infile
      if error ne 0 then begin
	print,' Error in openpff. err=',error
	RETURN
      endif
      print,' Reading input file...'
      readuf3, dimage, /arr, xar=sx, yar=sy, tlabel=tlab, clabel=clab, $
         xlabel=xlab, ylabel=ylab, blabel=blab, x0=x0, dx=dx, y0= y0, dy = dy
      closepff
   endelse
      dimage = reform(dimage, /overwrite)  ;convert to 2dim if possible
      print,' File '+filename+' read in!'
      print,' tlabel: ',tlab
      print,' clabel: ',clab
      print,' blabel: ',blab

      test = size(dimage)
      if test(0) ne 2 then begin
	print,' Error - image file not 2-dimensional data. Check file.'
	RETURN
      endif
      xdim = test(1)
      ydim = test(2)
      npoints = test(4)
      print,npoints,' elements, dimensions are (x,y): ',xdim,ydim
      
; now get film response filename - generated in wdfit3

      if n_elements (lc_valid) eq 0 then lc_valid = 0
      lcfitfile =1
      if n_elements (fitfile ) eq 0 then begin
        if lc_valid eq 0 then begin
          print, '% PFF_REGRID: No valid fit data'
          return
        endif
      endif else begin
        if size(/type, fitfile) le 4 then begin
          if fitfile lt 0 then lcfitfile=0
        endif
        if lcfitfile gt 0 then readwd3, fitfile
      endelse
      if lc_valid eq 0 and lcfitfile gt 1 then begin
        print, '% PFF_REGRID: No valid fit parameters'
        return
      endif

      if n_elements (regrid) eq 0 then regrid = regrid_default
      if (n_elements(xregrid) eq 0) then xregrid = regrid
      if (n_elements(yregrid) eq 0) then yregrid = regrid

      print, ' Image averaged over ', strtrim(xregrid, 2), ' X values.'
      print, ' Image averaged over ', strtrim(yregrid, 2), ' Y values.'

      mmx = xdim/xregrid
      mmy = ydim/yregrid
      print, ' New image size:', mmx, mmy


; create an output filename with _ur suffix on root name
    lcwrite =0
    if size(outfile, /type) gt 4 then begin
      lcwrite = 1
      outfile = strtrim(outfile, 2)
      if strlen(outfile) eq 0 then begin
        outfile = strtrim(filename, 2)
        outb = byte(outfile)
        n = n_elements(outb) -1
        dirsep= (byte(path_sep()))[0]
        extension = (byte('.'))[0]

        while ((outb(n) ne extension and outb(n) ne dirsep) and n gt 0) $
           do n= n-1
        strdef = '_ur'
        if (n eq 0) or (outb(n) eq dirsep) then outfile = outfile+strdef else $
            outfile = string(outb(0:n-1))+strdef+string(outb(n:*))
      endif
      outfile = find_newfile(outfile)
      createpff,outfile, error = err
      print,' Writing new PFF file to '+outfile+'...'
      if err ne 0 then begin
        print, '% PFF_REGRID: Unable to create output file.'
        print, '        File: ', outfile
        return
      endif
    endif else begin
      lcwrite = 0
      if n_elements (outfile) gt 0 then outfile = round(outfile[0]) $
        else outfile = get_maxstr()-1


      if outfile lt 1 or outfile gt get_maxstr() then begin
        print, '% PFF_REGRID: No valid structure number for output'
        if lcfitfile eq 0 then begin
          outfile = get_maxstr()-1
          print, '              Structure returned in: ', strtrim(outfile, 2)
        endif else return
      endif
    endelse
      tlab = 'Unfold-Regrid'
      clab2 = 'Regrid Values: ('+strtrim(xregrid, 2)+ ', ' + $
             strtrim(yregrid, 2) 
      if lcfitfile gt 0 then clab2 = clab2+');; Fitfile: '+ filename_save + $
             ' ;; Input File: '+ filename
      ok = strpos(clab, ':' )
      if ok gt 0 then clab2 = strmid(clab, 0, ok(0))+' - '+clab2+ ' ;; ' $
         +strmid (clab, ok[0]+1) else clab2 = clab + ' - ' + clab2 
      x = [x0, dx]
      y = [y0, dy]
      if n_elements(valid_save) eq 0 then valid_save=0
      if n_elements (valid) eq 0 then valid = valid_save
      if lcfitfile eq 0 then begin 
        if n_elements(default_val) gt 0 then def_v=default_val
        scale_regrid, dimage, x, y, xregrid, yregrid, zero = zero_origin, $
          writepff= lcwrite, tlab = tlab, clab = clab2, xlabel=xlab, ylabel=ylab, $
          blabel=blab, c1 =[0,1], valid = valid, default_val = def_v, ier = ier 
     
      endif else begin
        break = break_save
        if n_elements(default_val) lt 2 then begin
          def_v = [0.0,0.0]
          if nbreak_save eq 0 then def_v(1) = poly( valid[1], c1_save) else begin
            if valid[1] le break[0] then def_v(1) = poly(valid[1], c1_save)
            if nbreak_save eq 1 then begin
              if valid[1] gt break[0] then def_v(1) = poly(valid[1], c2_save)
            endif else begin
              if valid[1] gt break[1] then def_v(1) = poly(valid[1], c3_save) else $
                if valid[1] gt break[0] then def_v(1) = poly(valid[1], c2_save)
            endelse
          endelse
          if keyword_set(lc_log) then def_v(1) = exp(def_v(1))
        endif else def_v = default_val[0:1]
        def_v = float(def_v)
     
        scale_regrid, dimage, x, y, xregrid, yregrid, zero = zero_origin, $
          writepff= lcwrite, tlab = tlab, clab = clab2, xlabel=xlab, ylabel=ylab, $
          blabel=blab, valid = valid, default_val = def_v, ier = ier 
      endelse
      if lcwrite eq 1 then closepff else begin
        ss = size(dimage)
        dx = x[1]
        dy = y[1]
        x = x[0] + findgen(ss[1])*dx 
        y = y[0] + findgen(ss[2])*dy 
        arr2str, outfile, dimage, x, y, dx = dx, dy = dy, file = filename, $
           tlab = tlab, clab = clab2, xlabel=xlab, ylabel=ylab, $
           v1label = blab
      endelse
      dimage = 0
   print,' All finished.'

   print,' Took '+strtrim(systime(1)-t_start,2)+' seconds to run.'

RETURN
END
