; ------------------------------------------------------------------------ 
; $Id: plotblk.pro,v 1.3 2007/06/06 20:09:35 lpmix Exp $
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
 pro plotblk, ain, xrange = xrange, yrange = yrange, zrange= zrange, $
   polarplot = polar, color = color, block= blockin, axis_orientation=axisxyz, $
   showmodels = showmodels, getmodels = getmodels, clearmodels= clearmodels, $
   verbose = verbose, normalize= normalxyz, OFFSET = aoffset, $
   ALENGTH = alength, AALENGTH = aalength, ACHARACTER = acharacter
;
;+
; NAME:
;       PLOTBLK
;
; PURPOSE:
;       Plot conputational grid or block boundaries as a solid model.
; CATEGORY:
;       Input/Output
; CALLING SEQUENCE:
;       PLOTBLK, A [, XRANGE = XRANGE] [, YRANGE = YRANGE] [, ZRANGE = zrange]
;               [, POLARPLOT = polarplot] [, BLOCK = BLOCK] [, COLOR = COLOR]
;               [, SHOWMODELS = showmodels] [, GETMODELS = getmodels]
;               [, CLEARMODELS = clearmodels] [, /AXIS_ORIENTATION] [, /VERBOSE]
;               [, NORMALIZE = normalize] [, OFFSET = offset]
;               [, ALENGTH = alength] [, AALENGTH = aalenght]
;               [, ACHARACTER = acharacter] 
; INPUTS:
;       A        - PFIDL rectilinear grid or field structure
;                  Type = 1 or 3
;           or 
;       A        - Variable to return the model if GET_MODELS is set
;
; KEYWORD PARAMETERS:
;       XRANGE   - Two element vector defining the extent of the object in 
;                  the X direction.  Any part of a block in the range will
;                  be included.
;       YRANGE   - Two element vector defining the extent of the object in 
;                  the Y direction.  Any part of a block in the range will
;                  be included.
;       ZRANGE   - Two element vector defining the extent of the object in 
;                  the Z direction.  Any part of a block in the range will
;                  be included.
;       POLARPLOT - If set, a polar plot will be generated.
;       BLOCK    - The block of data to be plotted. Default is all blocks
;       COLOR    - Indicates the color of the block, Color may be an array 
;                  to sequence over colors.  If colors is an array, the
;                  minimum value is 1 and the maximum is !d.table_size
;                   The colors are: black(0), red(1), green(2), blue(3),
;                   purple(4), orange(5), light blue(6), yellow(7) and white(8).
;                  DEFAULT:  COLOR = 1+indgen(7)
;     ShowModels - If present, all input is ignored and the current stored
;                  models are listed.
;      GetModels - If present, all input is ignored the indicated model is
;                  returned in the first parameter.
;    ClearModels - If present and greater than zero, all models are destroyed.
;                  If negative, the indicated models are destroyed.
;    AXIS_ORIENTATION - If set, X, Y and Z axes will be drawn.
;                  DEFAULT:  Axis_Orientation = 1
;    Verbose     - If set, data range and axis parameters will be provided.
;    NORMALIZE   - If set, data will be normalized in the range [-1, 1]
;                  If NORMALIZE is a 6 element vector, the data will be 
;                  normalized in the ranges specified; that is, the first two
;                  elements will be the first range for the first dimension, etc.
;                  DEFAULT: NORMALIZE = 0
;    The following are passed to ADD_AXIS
;    OFFSET      - Axis offset from the data as a fraction of the data range.
;                  DEFAULT: See ADD_AXIS
;    ALENGTH     - Axis length as a fraction or the data range.
;                  If ALENGTH is a 3 element vector, different values will be 
;                  used for each vector.
;                  DEFAULT: see ADD_AXIS
;    AALENGTH    - Axis length in data coordinates
;                  If AALENGTH is a 3 element vector, these valuse will be used.
;                  If AALENGTH is a scalar, the actual values will be returned. 
;            NOTE: AALENGTH is checked before ALENGTH
;    ACHARACTER  - Character size
;                  Default: See ADD_AXIS
;
; OUTPUTS:
;       Designated blocks are plotted in XOBJVIEW
;       NOTE:  When XOBJVIEW is closed the model remains.
;              USE:  PLOTBLK, /clearmodels
;              Models will be destroyed
; EXAMPLE:
;       Generate a solid model from structure 3
;        PLOTBLK, 3
;
;       Clear all models.
;        PLOTBLK, /clearmodels
;
;       Get model 4 and view it
;        PLOTBLK, a, GetModels=4  &  xobjview, a
;
; MODIFICATION HISTORY:
;      Original version, L. P. Mix, November 24, 2003
;-
;
; insert a common block
;
@qscom.dat
;
;  Common block
@qsstrcom.dat
;
common plotblk_defaults, nbuffer, plotblk_comment, plotblk_models
      on_error, 1
;

      if n_elements (structinit) eq 0 then qsstruct_init
      if n_elements (qsplotinit) eq 0 then qsinit
      if n_elements (plotblk_comment) eq 0 then begin
        nbuffer = 64
;        nbuffer = 4
        plotblk_comment = make_array(nbuffer, /string)
        plotblk_models = ptrarr(nbuffer, /allocate)
        a=0
        for i = 0, nbuffer-1 do *plotblk_models[i]= a
        
      endif
;
      if keyword_set(showmodels) then begin
        for i = 0, nbuffer-1 do begin  
          if obj_valid((*plotblk_models[i])[0]) then $
            print, 'Model ', strtrim(i+1,2), ' - ', plotblk_comment[i] 
        endfor
        return
      endif
;
      if keyword_set(getmodels) then begin
        if getmodels gt 0 and getmodels le nbuffer then begin
          if obj_valid((*plotblk_models[getmodels-1])[0]) then $
            ain =  *plotblk_models[getmodels-1] else begin
            print, '% PLOTBLK: Model ', strtrim(getmodels, 2), $
             'is not a valid model'
            print, '           Use PLOTBLK, /SHOWMODELS to see models'
          endelse
        endif else print, '% PLOTBLK: GetModels must be in the range 1 to ', $
          strtrim(nbuffer,2)
        return
      endif
      if keyword_set(clearmodels) then begin
        if clearmodels[0] gt 0 then jj = indgen(nbuffer)+1 else begin
          jj = -clearmodels
          ok = where (jj gt 0 and jj le nbuffer, nok)
          if nok eq 0 then begin
            print, '% PLOTBLK: No valid model numbers.
            return
          endif
          jj = jj[ok]
        endelse
        nok = n_elements(jj)
        for i = 0, nok-1 do begin
          kk = jj[i]-1
          if obj_valid((*plotblk_models[kk])[0]) then $
            obj_destroy,*plotblk_models[kk]
        endfor
        return
      endif
;
      npar = n_params()
;
      if npar eq 0 then begin
        print,'% Format: PLOTBLK, A [, XRANGE = XRANGE] [, YRANGE = YRANGE]'
        print,'          [, ZRANGE = zrange] [, POLARPLOT = polarplot]'
        print,'          [, BLOCK = BLOCK] [, COLOR = COLOR]
        print,'          [, SHOWMODELS = showmodels] [, GETMODELS = getmodels]
        print,'          [, CLEARMODELS = clearmodels] [, /AXIS_ORIENTATION]
        print,'          [, NORMALIZE = normalize] [, AOFFSET = aoffset]
        print,'          [, ALENGTH = alength] [, AALENGTH = aalenght]
        print,'          [, ACHARACTER = acharacter] [, /VERBOSE]
        return
      endif

;    Determine the next available model
;
     imod = 0
     while((imod lt nbuffer)and(obj_valid((*plotblk_models[imod])[0]))) do imod++
     if imod eq nbuffer then begin
       print, '% PLOTBLK: Unable to store additional models'
       print, '           Use PLOTBLK, /CLEARMODELS to eliminate all old models
       return
     endif
;    make sure we don't corrupt input data
;     now get the data  -- check that we have a structure of the proper type
      if n_elements(ain) eq 0 then begin
        print, 'Structure must be defined'
        return
      endif
      a = ain
      b = size(a)
      if b(b(0)+1) ne 8 then begin
        if a gt 0 and a le maxstructure then begin
          s2i, a, a
          b = size (a)
          if b(b(0)+1) ne 8 then begin
            print, ' First parameter must be a structure or a QS array number'
            return
          endif
        endif  else begin
          print, ' First parameter must be a structure or a QS array number'
          return
        endelse
      endif
      if (a.type ne 3 and a.type ne 1) then begin
        print, '% PLOTBLK: Only Type 3 Grid type structures and Type 1 Field'
        Print, '           structures may be plotted with PLOTBLK'
        print, 'TYPE = ', strtrim(a.type, 2)
        return
      endif
      if n_elements (blockin) eq 0 then block = 1+indgen(a.block) else begin
        ok = where (blockin gt 0 and blockin le a.block, nok)
        if nok eq 0 then begin
          print, '% PLOTBLK: No valid blocks specified
          print, '           All blocks will be plotted
          block = 1+indgen(a.block)
        endif else block = blockin[ok]
      endelse
      nblo = n_elements (block)
      
      if n_elements (xrange) ge 2 then begin
        xr = [min(xrange, max = m), m]
        ok = where (a.range[0, 0, block] lt xr[1] and $
              a.range[1, 0, block] gt xr[0], nok)
        if nok eq 0 then begin 
          print,  '% PLOTBLK: No blocks found for XRANGE: ', xr
          return
        endif
        if nok lt nblo then begin
          block = block[ok]
          nblo = nok
        endif
      endif
      if n_elements (yrange) ge 2 then begin
        yr = [min(yrange, max = m), m]
        ok = where (a.range[0, 1, block] lt yr[1] and $
              a.range[1, 1, block] gt yr[0], nok)
        if nok eq 0 then begin 
          print,  '% PLOTBLK: No blocks found for YRANGE: ', yr
          return
        endif
        if nok lt nblo then begin
          block = block[ok]
          nblo = nok
        endif
      endif
      if n_elements (zrange) ge 2 then begin
        zr = [min(zrange, max = m), m]
        ok = where (a.range[0, 2, block] lt zr[1] and $
              a.range[1, 2, block] gt zr[0], nok)
        if nok eq 0 then begin 
          print,  '% PLOTBLK: No blocks found for ZRANGE: ', zr
          return
        endif
        if nok lt nblo then begin
          block = block[ok]
          nblo = nok
        endif
      endif



      if keyword_set(polar) then begin
        print, '% PLOTBLK: POLAR Plots are not supported
        print, '           Contact Paul Mix for assistance.
        return
      endif
      conn = [4, 0, 1, 2, 3,   4, 4, 7, 6, 5,   4, 3, 2, 6, 7, $
              4, 0, 4, 5, 1,   4, 1, 5, 6, 2,   4, 0, 3, 7, 4]
      part = objarr(nblo)
      space_save = make_array(3, 8)
      if n_elements(color) eq 0 then color = 1+indgen(7) else begin
        ok = where(color gt 0 and color lt !d.table_size, nok)
        if nok eq 0 then color = 1+indgen(7) else color = color[ok]
      endelse
      nelc = n_elements (color)
      
      xr = [min(a.range[0, 0, block]), 0.0 ]
      xr[1] = max(a.range[1, 0, block])
      yr = [min(a.range[0, 1, block]), 0.0 ]
      yr[1] = max(a.range[1, 1, block])
      zr = [min(a.range[0, 2, block]), 0.0 ]
      zr[1] = max(a.range[1, 2, block])
      for ii = 0, nblo-1 do begin
        ip = block[ii]
        clab = 'Block '+strtrim(ip, 2)
        space_save[0, [4, 5, 6, 7]] = a.range[0, 0, ip]
        space_save[0, [0, 1, 2, 3]] = a.range[1, 0, ip]
        space_save[1, [0, 3, 4, 7]] = a.range[0, 1, ip]
        space_save[1, [1, 2, 5, 6]] = a.range[1, 1, ip]
        space_save[2, [0, 1, 4, 5]] = a.range[0, 2, ip]
        space_save[2, [2, 3, 6, 7]] = a.range[1, 2, ip]
        col = getcolor((ii mod nelc) +1)
        bcol = .7*col
        
        part[ii] = obj_new('IDLgrPolygon', space_save, polygons= conn, $
        thick = 1.0,  color = col, bottom = bcol, name=clab, shading = 1)
      endfor
      omodel = obj_new('idlgrmodel')
      omodel -> add, part
      if n_elements(normalxyz) eq 0 then n = 0 else n = normalxyz
      if keyword_set(n) then begin
        if n_elements(n) gt 5 then begin
          px = n[0:1]
          py = n[2:3]
          pz = n[4:5]
        endif else begin
          px = [-1, 1]
          py = [-1, 1]
          pz = [-1, 1]
        endelse
        nxr = normalize(xr, position = px)
        nyr = normalize(yr, position = py)
        nzr = normalize(zr, position = pz)
        for i = 0, nblo-1 do $
          part[i] -> setproperty, xcoord_conv=nxr, ycoord_conv=nyr, $
            zcoord_conv=nzr
      endif  else begin
        px = xr
        py = yr
        pz = zr
      endelse
      if n_elements(axisxyz) eq 0 then ax = 1 else ax = axisxyz
      if keyword_set(ax) then $
        omod= add_axis(omodel, OFFSET = aoffset, $
        ALENGTH = alength, AALENGTH = aalength, ACHARACTER = acharacter, $
        xrange = px, yrange = py, zrange = pz, verbose = verbose)
      
      if obj_valid(omod) then begin
        xobjview, omod 
        *plotblk_models[imod] = omod
        plotblk_comment[imod] = a.clab
      endif else obj_destroy, omodel
      end
