; ------------------------------------------------------------------------
; $Id: plotfe12.pro,v 1.7 2010/03/16 16:34:17 lpmix Exp $
; ------------------------------------------------------------------------
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
;+
; NAME:
;       PLOTFE12
;
; PURPOSE:
;       Converts a 3D finite element structure to a 2D rectilinear structure
;       PLOTSTR is then called on that structure
;
; CATEGORY:
;       Vizualization
;
; CALLING SEQUENCE:
;       PLOTFE12, STR, KVALUE, VALUE [, WORK = work] [, MISSING = missing]
;                  [, _EXTRA = extrastuff]
;        or
;       PLOTFE12, STR, CENTER, ANGLE, /RZ [, WORK = work] [, MISSING = missing]
;                  [, _EXTRA = extrastuff]
;
; INPUTS:
;       STR:    IDL type 12 structure or structure array number
;       KVALUE: Indicates the coordinate which is fixed for the two
;               dimensional plots.
;       VALUE:  The coordinate values for the fixed dimensions in
;               the two dimensional plots.
;       (IF RZ is SET)
;       CENTER: A two element vector or ZERO
;               If CENTER = 0, then CENTER = [0.0, 0.0]
;               This version requires the center in the computational volume
;               unless center = 0.  If center = 0 and the center is not in the 
;               computational volume, the slice will go from the point [0,0] to
;               [xmax, xmax*tan(angle)]
;       ANGLE:  Angle between the X axis and the RZ plane.  ([-!pi/2, !pi/2])
;               Angle is in radians unless degrees is set
;               Angle is arbitrary, but it will be converted to quadrants 1 or 4.
;
; KEYWORD PARAMETERS:
;       MISSING: Keyword sent to QGRID3 for grid points that lie outside the data
;                DEFAULT:  MISSING = 0.0
;       WORK:   Storage location for the 2 dimensional, rectilinear structure.
;               DEFAULT:  WORK = GET_MAXSTR() -1
;       RZ:     If set, assume the data is cylindrical in Cartesian coordinates.
;               Plot R along the X axis and Z along the Y axis unless transplot is set.
;       DEGREES:If set, the RZ angle is in degrees
;       TRANSPLOT: If set, transpose the X and Y axes.
;       _EXTRA:	Any valid keyword for PLOTFLD
;
; OUTPUTS:
;       The output field is plotted.
;       The rectilinear dataset is stored in WORK.
;
; PROCEDURE:
;       The data is put interpolated on a rectilinear mesh using the plot parameters
;       to determine resolution.  The IDL command QGRID3 is used.
;
; EXAMPLE:
;       Plot structure 32 along a plane for y = 0.0
;         PLOTSTR, 32, 2, 0.0
;
;       Plot structure 32 along a plane for y = 0.1 and for xvalues from 0.0 to 0.001
;         PLOTSTR, 32, 2, 0.0, xrange= [0.0, 0.001]
;
; MODIFICATION HISTORY:
; 	Written by:	L. P. Mix, Jr., March 1, 2006
;                       Added RZ keyword, lpm, May 1, 2007
;
;-

 pro plotfe12, strin, kvaluein, valuein, work = work, xrange = xrange, yrange = yrange, $
               overplot = over, transplot = transplotin,  _extra = extrastuff, $
               MISSING = missing, DEGREES = degrees, RZ = rz

on_error, 1
;  Common block
@qscom.dat
@qsstrcom.dat
@com.dat
@plotfld_defaults.com
;
 if n_elements (structinit) eq 0 then qsstruct_init
 if n_elements (qsplotinit) eq 0 then qsinit
;
 structinit = 1
 npar = n_params()
 if npar eq 0 then begin
   print, '% PLOTFE12 Format:'
   print, '      PLOTFE12, STR, KVALUE, VALUE [, WORK = work] [, _EXTRA = extrastuff]'
   print, '        < OR >'
   print, '      PLOTFE12, STR, CENTER, ANGLE, /RZ [, WORK = work] [, _EXTRA = extrastuff]'
   return
 endif
 if npar lt 3 then begin
   print, '% PLOTFE12: Structure, Slice dimension, and slice value must be defined'
   return
 endif
 a = get_valid_structure(strin, 'PLOTFE12', lcstr = lcstr, ierror = ierror)
 if ierror ne 0 then begin
   Print,  '% PLOTFE12: Provide a valid structure and try again.'
   return
 endif
 nk = n_elements(kvaluein)
 if n_elements(valuein) gt 0 then value = valuein
 if nk eq 0  then begin
   if keyword_set(RZ) then $
   Print,  '% PLOTFE12: Center point must be provided' else $
   print, '% PLOTFE12: SLICE dimension must be defined'
   return
 endif else kvalue = kvaluein
 if keyword_set(rz) then begin
   if nk ge 2 then kvalue = kvaluein[0:1] else begin
     if kvaluein[0] eq 0 then kvalue = [0.0, 0.0] else begin
       print, '% PLOTFE12: Center must have two elements or be zero'
       return
     endelse
   endelse
 endif else kvalue = kvaluein[0]

if n_elements (value) eq 0 then begin
  print, '% PLOTFE12: Slice value must be privided'
  return
endif
if n_elements(missing) eq 0 then missing = 0
transplot = keyword_set(transplotin) ? 1 : 0
if keyword_set (rz) then begin
 angle = (keyword_set(degrees))? !dtor*value : value
 xval = 1
 yval = 2
 zval = 3
 xmin = a.range[0, 0, 0]
 xmax = a.range[1, 0, 0]
 ymin = a.range[0, 1, 0]
 ymax = a.range[1, 1, 0]
 zmin = a.range[0, 2, 0]
 zmax = a.range[1, 2, 0]
 xerr = 3.0517578e-05*max(abs([(xmax+xmin)/512., xmax-xmin]))
 yerr = 3.0517578e-05*max(abs([(zmax+zmin)/512., zmax-zmin]))
 kerr = 3.0517578e-05*max(abs([(ymax+ymin)/512., ymax-ymin]))
 while  (angle gt !dpi*0.5 ) do angle -=!dpi
 while  (angle lt -!dpi*0.5 ) do angle +=!dpi

;  Determine intercepts with the computational volume
;  Check in two steps and calculate an xrange and yrange
 xran= make_array(2)
 yran= xran
 lcx = 0
 lcy = 0
 lcrad = 0
 center = kvalue[0:1]
 if center[0] eq 0.0 and center[1] eq 0.0 then begin
   xran[0]= 0.0
   yran[0]= 0.0
   xran[1]= xmax
   yran[1]= xmax*tan(angle)
   lcrad = 1
 endif 
 If (kvalue[0] lt xmin-xerr) or (kvalue[0] gt xmax+xerr) then lcx = 1
 If (kvalue[1] lt ymin-yerr) or (kvalue[1] gt ymax+yerr) then lcy = 1
 if lcx gt 0 and lcrad eq 0 then begin
   Print, '% PLOTFE12: X center must be in the calculational volume'
   return
 endif 
 If lcy gt 0 and lcrad eq 0 then begin
   Print, '% PLOTFE12: Y center must be in the calculational volume'
   return
 endif
 if lcx eq 0 and lcy eq 0 then begin
;  if the center is in the calculational volume we will find the limits
   kvalue[0] = (kvalue[0] > xmin) < xmax
   kvalue[1] = (kvalue[1] > ymin) < ymax
 if abs(angle) le !pi/4 then begin
;  Angle less than 45 degrees
;  check in positive direction
   xran[1] = xmax
   yvalu = (xmax-kvalue[0]) * tan(angle) + kvalue[1]
   if (yvalu le ymax) && (yvalu ge ymin) then yran[1] = yvalu else begin
     yran[1] = (angle lt 0) ? ymin:ymax
     xran[1] = (yran[1]-kvalue[1])*tan(!pi/2-angle)+kvalue[0]
   endelse
;  Now get the negative direction
   xran[0] = xmin
   yvalu = (xmin-kvalue[0]) * tan(angle) + kvalue[1]
   if (yvalu le ymax) && (yvalu ge ymin) then yran[0] = yvalu else begin
     yran[0] = (angle lt 0) ? ymax:ymin
     xran[0] = (yran[0]-kvalue[1])*tan(!pi/2-angle)+kvalue[0]
   endelse
 endif else begin
;  Angle larger than 45 degrees
   yran[1] = (angle lt 0) ? ymin:ymax
   xvalu = (yran[1]-kvalue[1])*tan(!pi/2-angle)+kvalue[0]
   if (xvalu le xmax) && (xvalu ge xmin) then xran[1] = xvalu else begin
     xran[1] = xmax
     yran[1] = (xran[1]-kvalue[0])*tan(angle)+kvalue[1]
   endelse
;  Now get the negative direction
   yran[0] = (angle lt 0) ? ymax:ymin
   xvalu = (yran[0]-kvalue[1])*tan(!pi/2-angle)+kvalue[0]
   if (xvalu le xmax) && (xvalu ge xmin) then xran[0] = xvalu else begin
     xran[0] = xmin
     yran[0] = (xran[0]-kvalue[0])*tan(angle)+kvalue[1]
   endelse
 endelse
 endif
 r1 = -sqrt((xran[0]-kvalue[0])^2+(yran[0]-kvalue[1])^2)
 r2 = sqrt((xran[1]-kvalue[0])^2+(yran[1]-kvalue[1])^2)
 ax = a.x1-kvalue[0]
 ay = a.x2-kvalue[1]
 ax1 = cos(angle)*ax+sin(angle)*ay
 ay1 = cos(angle)*temporary(ay)-sin(angle)*temporary(ax)
 
 if n_elements (xrange) lt 2 then begin
   epsilx = (r2 -r1)*3.0517578e-05
   xrange = [r1-epsilx, r2+epsilx] 
 endif else begin
   xrange = xrange[0:1]
   epsilx = (xrange[1]-xrange[0])*3.0517578e-05
 endelse
 if n_elements (yrange) lt 2 then begin
   epsily = (zmax -zmin)*3.0517578e-05
   yrange = [zmin-epsily, zmax+epsilx] 
 endif else begin
   yrange = yrange[0:1]
   epsily = (yrange[1]-yrange[0])*3.0517578e-05
 endelse
 value  = 0.0
 kvalue = 2
 kmin = min(ay1, max = kmax)
 yval = 3
 kerr = 3.0517578e-05*max(abs([(kmax+kmin)/512., kmax-kmin]))
endif else begin
 if kvalue lt 1 or kvalue gt 3 then begin
  print, '% PLOTFE12: KVALUE (the slice dimension) must be in the range[1, 3]'
  return
 endif
 if n_elements(value) gt 0  then value = value[0] else begin
   print, '% PLOTFE12: SLICE value must be defined'
   return
 endelse
 if kvalue lt 1 or kvalue gt 3 then begin
  print, '% PLOTFE12: KVALUE (the slice dimension) must be in the range[1, 3]'
  return
 endif

 if kvalue eq 2 then begin
   if transplot eq 0 then transplot = 1 else transplot = 0
   xval = 1 & yval = 3
 endif else begin
   xval = (kvalue mod 3) + 1
   yval = ((kvalue + 1) mod 3) + 1
 endelse
 if keyword_set(transplot) then begin
   xnew = yval
   ynew = xval
   xval= xnew
   yval= ynew
 endif else begin
   xnew = xval
   ynew = yval
 endelse
 xmin = a.range[0,xval-1, 0]
 xmax = a.range[1,xval-1, 0]
 ymin = a.range[0,yval-1, 0]
 ymax = a.range[1,yval-1, 0]
 kmin = a.range[0, kvalue-1, 0]
 kmax = a.range[1, kvalue-1, 0]
 xerr = 3.0517578e-05*max(abs([(xmax+xmin)/512., xmax-xmin]))
 yerr = 3.0517578e-05*max(abs([(ymax+ymin)/512., ymax-ymin]))
 kerr = 3.0517578e-05*max(abs([(kmax+kmin)/512., kmax-kmin]))
 if value lt kmin then value = kmin+kerr
 if value gt kmax then value = kmax-kerr
 if n_elements(xrange) lt 2 then begin
    epsilx = (xmax-xmin)*3.0517578e-05
    xrange = [xmin+epsilx, xmax-epsilx]
 endif else begin
    xrange = xrange(0:1)
    epsilx = (xrange(1)-xrange(0))*3.0517578e-05
 endelse
 if n_elements(yrange) lt 2 then begin
    epsily = (ymax-ymin)*3.0517578e-05
    yrange = [ymin+epsily, ymax-epsily]
 endif else begin
    yrange = yrange(0:1)
    epsily = (yrange(1)-yrange(0))*3.0517578e-05
 endelse
endelse
 ss = size(extrastuff)
 if ss(ss(0)+1) eq 8 then begin
   myextra = extrastuff
   tags = tag_names(myextra)
   ok = where(strpos(tags, 'XS', 0) eq 0, nok)
   if nok gt 0 then i = execute('myextra.'+tags(ok(0))+ $
     '= myextra.'+tags(ok(0))+' or 1') else $
     myextra = create_struct(myextra, 'xstyle', (!x.style or 1))

   ok = where(strpos(tags, 'YS', 0) eq 0, nok)
   if nok gt 0 then  begin
     i = execute('myextra.'+tags(ok(0))+ $
       '= myextra.'+tags(ok(0))+' or 1')
   endif else begin
     myextra = create_struct(myextra, 'ystyle', !y.style or 1)
   endelse
 endif else begin
   if !y.style gt 0 then begin
     myextra ={ystyle: (!y.style or 4)}
   endif
   if !x.style gt 0 then begin
     if size(extrastuff, /type) eq 8 then $
       extrastuff = create_struct(extrastuff, 'xstyle', !x.style) $
     else extrastuff ={xstyle: !x.style}
       if size(myextra, /type) eq 8 then $
       myextra = create_struct(myextra, 'xstyle', !x.style or 1) $
       else myextra ={xstyle: !x.style}
   endif
 endelse
 wsave = !d.window
 wnew = -1
 lcchange = 0
 lcnull = 0
 if !d.name eq 'NULL' then begin
   lcnull =1
   osname = strupcase(strmid(!version.os, 0,3))
   newplot = (osname eq 'WIN') ? 'WIN' : 'X'
   set_plot, newplot
   window, /pixmap, /free
   contour, make_array(2,2), xrange, yrange, /nodata, position = position, $
       xrange= xrange, yrange= yrange, xstyle =1, ystyle = 1, $
       xmargin = xmargin, ymargin = ymargin, _extra = myextra
   wnew = !d.window
   xrange_first = !x.crange
   yrange_first = !y.crange
 endif else if not keyword_set(over) && !d.name ne 'PS' && !d.name ne 'PCL' then begin
   multcnt = !p.multi(0)

   if wsave ge 0 then begin
     xsave = !x
     ysave = !y
     zsave = !z
     psave = !p
     window, /pixmap, /free, xsize = !d.x_size, ysize = !d.y_size
   endif else if !d.name ne 'PS' && !d.name ne 'PCL' then $
     window, /pixmap, /free else begin
     ps2x
     lcchange = 1
     window, /pixmap, /free
   endelse

   contour, make_array(2,2), xrange, yrange, /nodata, position = position, $
       xrange= xrange, yrange= yrange, xstyle =1, ystyle = 1, $
       xmargin = xmargin, ymargin = ymargin, _extra = myextra
   wnew = !d.window
   xrange_first = !x.crange
   yrange_first = !y.crange
   if lcchange gt 0 then x2ps
 endif
 xrange_first = !x.crange
 yrange_first = !y.crange

 xpos = round(!x.window*!d.x_size)
 ypos = round(!y.window*!d.y_size)
 xr = !x.crange(1)-!x.crange(0)
 yr = !y.crange(1)-!y.crange(0)
 xsiz = xpos(1)-xpos(0)
 ysiz = ypos(1)-ypos(0)
 xrbig = float(!x.crange[0:1])
 xpxbig = round((xrbig -!x.crange(0))*(xsiz/xr))
 yrbig = float(!y.crange[0:1])
 ypxbig = round((yrbig -!y.crange(0))*(ysiz/yr))
 nxbig = long(xpxbig(1)-xpxbig(0))
 nybig = long(ypxbig(1)-ypxbig(0))
 if !d.name eq 'PS' || !d.name eq 'PCL' then begin
   pden = 50
   maxsiz = 2048
   if !d.x_px_cm gt pden then nxbig = round(nxbig*pden/!d.x_px_cm)
   nxbig = nxbig < maxsiz
   if !d.y_px_cm gt pden then nybig = round(nybig*pden/!d.y_px_cm)
   nybig = nybig < maxsiz
 endif
 dxbig = (xrbig(1)-xrbig(0))/(nxbig-1)
 dybig = (yrbig(1)-yrbig(0))/(nybig-1)
 xarray = xrbig(0) + findgen(nxbig)*dxbig
 yarray = yrbig(0) + findgen(nybig)*dybig
 start = make_array(3)
 delta = make_array(3)
 dimension = make_array(3, /long)
 start[xval-1] = xrbig[0]
 start[yval-1] = yrbig[0]
 start[kvalue-1] = value
 delta[xval-1] = dxbig
 delta[yval-1] = dybig
 delta[kvalue-1] = kerr
 dimension[xval-1] = nxbig
 dimension[yval-1] = nybig
 dimension[kvalue-1] = 2
;  
 if keyword_set(rz) then begin
   arr = qgrid3(ax1, ay1, a.x3, a.v1, a.triangles, delta = delta, $
       dimension = dimension, missing = missing, start = start)
 if wnew ge 0 then wdelete, wnew
 if lcnull then set_plot, 'NULL' else if wsave ge 0 then begin
   wset, wsave
   !p.multi[0]= multcnt
   !x = xsave
   !y = ysave
   !z = zsave
   !p = psave
 endif
 if n_elements (work) eq 0 then work = get_maxstr()-1
 if work lt 1 or work gt get_maxstr()-1 then work = get_maxstr()-1
 xarray = start[0] + findgen(dimension[0])*delta[0]
 zarray = start[2] + findgen(dimension[2])*delta[2]
 arr= reform( arr[*, 0, *] )
 clab = a.clab
 clab = (strpos(clab, ' - ') ge 0) ? clab+';;' : clab+' - '
 clab = clab+'RZ Slice;;CENTER=('+strtrim(center[0], 2)+','
 clab = clab+strtrim(center[1], 2)+');;ANGLE='+strtrim(angle, 2)
 arr2str, work, arr,xarray, zarray,  dx = delta[0], dy = delta[2], $
    file = a.file, xlabel = a.x1lab[0], ylabel = a.x2lab, $
   zlabel = a.x3lab, tlab= a.tlab, clab= clab, spare = [0,0,0,0,0]
; help, work
 plotstr, work, 1,  xrange = xrange, yrange = yrange, $
   overplot = over, transplot = transplotin,  _extra = extrastuff
 return
endif
; KEYWORD - RZ not set
 arr = qgrid3(a.x1, a.x2, a.x3, a.v1, a.triangles, delta = delta, $
       dimension = dimension, missing = missing, start = start)
 if wnew ge 0 then wdelete, wnew
 if lcnull then set_plot, 'NULL' else if wsave ge 0 then begin
   wset, wsave
   !p.multi[0]= multcnt
   !x = xsave
   !y = ysave
   !z = zsave
   !p = psave
 endif
 if n_elements (work) eq 0 then work = get_maxstr()-1
 if work lt 1 or work gt get_maxstr()-1 then work = get_maxstr()-1
 xarray = (dimension[0] eq 2) ? start[0] : start(0) + findgen(dimension[0])*delta[0]
 yarray = (dimension[1] eq 2) ? start[1] : start(1) + findgen(dimension[1])*delta[1]
 zarray = (dimension[2] eq 2) ? start[2] : start(2) + findgen(dimension[2])*delta[2]
 if kvalue eq 1 then arr = arr[0, *, *] else $
   if kvalue eq 2 then arr= arr[*, 0, *] $
   else arr = arr[*, *, 0]
 arr2str, work, arr,xarray, yarray, zarray, dx = delta[0], dy = delta[1], $
   dz = delta[2], file = a.file, xlabel = a.x1lab[0], ylabel = a.x2lab, $
   zlabel = a.x3lab, tlab= a.tlab, clab= a.clab +', Sliced', spare = [0,0,0,0,0]
; help, work
 plotstr, work, 1, kvalue, 1, /index,  xrange = xrange, yrange = yrange, $
   overplot = over, transplot = transplotin,  _extra = extrastuff
 end
