; ------------------------------------------------------------------------
; $Id: plotmat.pro,v 1.6 2007/06/06 20:09:35 lpmix Exp $
; ------------------------------------------------------------------------
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
;+
; NAME:
;       PLOTMAT
;
; PURPOSE:
;       Plot outlines of dielectrics from TwoQuick.
;     Note: This is an overplot which requires a previous PFIDL plot command
;           to set the axis orientation, range, and scale.
;     See also "PLOTCON, /FILL"
;
; CATEGORY:
;       Visualization
;
; CALLING SEQUENCE:
;
;       PLOTMAT [, Dataset] [, Color]
;
; OPTIONAL INPUTS:
;       Dataset: Dataset of the dielectric data or an array of datasets
;                Dielectric data from TWOQUICK is stored as IFL data
;                Default: Dataset = 0 ; current dataset
;       Color:   Color to draw the dielectric
;                Multiple values are cycled. If nc is the number of colors and
;                nd is the dataset number, then the dielectric color is:
;                   dielectric_color = color[nd mod nc]
;                Default: Color = keyword(color) (see below)
;
; KEYWORD PARAMETERS:
;       FILEID: File id of the PFF file
;       SCALE:  If provided all spatial quanties are multiplied by SCALE[0].
;               This allows plots to use mm or cm for plot scaling.
;               Default: SCALE = 1.0
;       XMUL:   Same as scale.  Provided for backward compatibility.
;       LINE_FILL: If set, dielectric is filled with lines.
;               If LINE_FILL not equal to 1 then ORIENTATION = LINE_FILL.
;               POLYFILL is repeated N_Elements(ORIENTATION) Times.
;               IF LINE_FILL equal to 1 then ORIENTATION=[+45, -45]
;               Note: LINE_FILL is set if it is not zero or an array.
;               Default is solid fill or LINE_FILL =0.
;       NOFill  If set, only an outline of the region will be drawn.  The
;               _EXTRA keyword applies to the IDL procedure OPLOT in this case.
;       COLOR:  Color to draw the dielectric - see color above
;               If the parameter COLOR is specified the keyword is ignored.
;               Default:  Color = !p.color
;       XSYM:   Make the conductor symmetric in X
;               If both XSYM and YSYM are set, a quadrant becomes 4 quadrants.
;       YSYM:   Make the conductor symmetric in Y
;               If both XSYM and YSYM are set, a quadrant becomes 4 quadrants.
;       Note:   If XSYM or YSYM less than zero the following quadrangs are plotted.
;               If XSYM lt 0 and YSYM eq 0 then Quadrant = 2
;                            and YSYM lt 0 then Quadrant = 3
;                            and YSYM gt 0 then quadrant = 2 & 3
;               If YSYM lt 0 and XSYM eq 0 then Quadrant = 4
;                            and XSYM lt 0 then Quadrant = 3
;                            and XSYM gt 0 then Quadrant = 3 & 4
;               Table:  Quadrants vs XSYM & YSYM
;
;							Quadrants
;				YSYM	1	2,3		1,4		1,2,3,4
;					0	2		1		1,2
;					-1	3		4		3,4
;
;                     		XSYM		-1		0		1
;
;       _EXTRA: Any valid keyword for POLYFILL (normal mode) or
;               OPLOT (if NOFill is set)
;
; OUTPUTS:
;       Dielectric areas are drawn on the existing plot.
;
; RESTRICTIONS:
;       PLOTMAT requires a previous PFIDL 2D plot command to set the axis
;       orientation, range and scale.
;
; PROCEDURE:
;       The dielectric data is read from the file and each dielectric is drawn.
;       IF X = array of X Values for a dielectric
;      and Y = array of Y Values for a dielectric then
;       if keyword_set (line_fill) then
;         polyfill, X, Y, col= color[nd mod nc], orientation = 45, noclip = 0, $
;             _extra = Extra_keywords
;         polyfill, X, Y, col= color[nd mod nc], orientation =-45, noclip = 0, $
;             _extra = Extra_keywords
;       endif else
;         polyfill, X, Y, col= color[nd mod nc], noclip = 0, $
;             _extra = Extra_keywords
;
; EXAMPLE:
;
;       Plot the dielectrics in dataset 9 with the standard print color.
;         PLOTMAT, 9
;
;       Plot the dielectrics in dataset 9 with color = 4.
;         PLOTMAT, 9, 4
; or
;         PLOTMAT, 9, color = 4
;
;       Plot the dielectrics in dataset 9 with color = 4 and lines at +/-45.
;         PLOTMAT, 9, 4 , /LINE_FILL
;  or
;         PLOTMAT, 9, 4 , LINE_FILL=[45, -45]
;
;       Plot the dielectrics in dataset 9 with color = 4 and lines at 45 and
;       linestyle = 2.
;         PLOTMAT, 9, 4 , LINE_FILL=45, linestyle =2
;
;       Plot the dielectrics in dataset 9 with color = 4 and horizontal lines
;       and linestyle = 2.
;         PLOTMAT, 9, 4 , LINE_FILL=[0], linestyle =2
;
;       Plot the dielectrics in dataset 9 with color = 4 and a dot pattern
;         pat= bytarr(3, 3) & pat[4]= 4
;         PLOTMAT, 9, pattern= pat  ; Note: Color must be specified in pattern
;
; MODIFICATION HISTORY:
; 	Written by:	David B. Seidel, 1/1/2001
;	April 12, 2001  L. P. Mix, 4/12/2001
;                       Modified for inclusion in PFIDL
;       May 24, 2006    L. P. Mix, 5/24/2006
;                       Added XSYM and YSYM keywords to plot symmetric
;                       geometries.
;-


Pro plotmat, ds, col, fileid=fid, line_fill=line, xmul=xmul, scale = scale, $
      color = kwcolor, nofill = nofill, xsym = xsym, ysym = ysym, _extra = extrastuff

@qscom.dat
on_error, 1

if n_elements(ds) eq 0 then ds = 0
nds = n_elements(ds)

if n_elements(col) eq 0 then begin
 if n_elements (kwcolor) eq 0 then color = !p.color else color = kwcolor
endif else color = col
ncol = n_elements(color)

if n_elements(xoldvalue) eq 0 then begin
  print, 'Must have done a previous PFIDL plot command !'
  return
endif

xoff = xoldvalue[0] - 1
yoff = yoldvalue[0] - 1
zoff = 3 - xoff - yoff
zoff3 = zoff+3
zv   = zvalueold[0]
xval = [xoff, xoff+3, xoff+3, xoff, xoff]
yval = [yoff, yoff, yoff+3, yoff+3, yoff]

if n_elements(scale) eq 0 then begin
  if n_elements (xmul) eq 0 then scale = 1.0 else scale = xmul[0]
endif else scale = scale[0]

if keyword_set(line) then begin
  if n_elements(line) eq 1 and line[0] eq 1 then orient= [45, -45] else $
      orient= line
  nor = n_elements(orient)
endif
if keyword_set(xsym) then $
  xsym = (xsym gt 0) ? 1 : -1 else xsym =0
if keyword_set(ysym) then $
  ysym = (ysym gt 0) ? 1 : -1 else ysym =0

for id=0,nds-1 do begin
  col = color(id mod nds)
  readifl,nmat,fl,fa,ds[id],clab=clab,fileid=fid
  if scale ne 1.0 then fa = scale*fa

  for m=0L,nmat[0]-1 do begin
    m6  = 6*m
    if zv ge fa[m6+zoff] and zv le fa[m6+zoff3] then begin
      xx = fa[m6+xval]
      yy = fa[m6+yval]

      if (xsym ge 0) && (ysym ge 0) then begin
;     Quadrant 1
        if keyword_set (nofill) then oplot, xx, yy, col=col, $
          _extra = extrastuff  else begin
          if keyword_set(line) then $
            for i = 0, nor-1 do polyfill,xx,yy,col=col, orientation= orient[i], $
            noclip = 0, _extra = extrastuff $
          else  polyfill,xx,yy,col=col, noclip = 0, _extra = extrastuff
        endelse
      endif
      if keyword_set(xsym) then begin
        xx = -xx
        if ysym ge 0 then begin
;       Quadrant 2
          If keyword_set (nofill) then oplot, xx, yy, col=col, $
            _extra = extrastuff  else begin
            if keyword_set(line) then $
              for i = 0, nor-1 do polyfill,xx,yy,col=col, orientation= orient[i], $
                  noclip = 0, _extra = extrastuff $
              else  polyfill,xx,yy,col=col, noclip = 0, _extra = extrastuff
          endelse
        endif
        if keyword_set(ysym) then begin
;       XSYM and YSYM are set
;       Quadrant 3
          yy = -yy
          If keyword_set (nofill) then oplot, xx, yy, col=col, $
            _extra = extrastuff  else begin
            if keyword_set(line) then $
              for i = 0, nor-1 do polyfill,xx,yy,col=col, orientation= orient[i], $
                noclip = 0, _extra = extrastuff $
            else  polyfill,xx,yy,col=col, noclip = 0, _extra = extrastuff
          endelse
        endif
      endif
      if keyword_set(ysym) && xsym ge 0 then begin
;       Quadrant 4
        if xsym eq 0 then yy = -yy else xx = -xx
        If keyword_set (nofill) then oplot, xx, yy, col=col, $
          _extra = extrastuff  else begin
          if keyword_set(line) then $
            for i = 0, nor-1 do polyfill,xx,yy,col=col, orientation= orient[i], $
              noclip = 0, _extra = extrastuff $
          else  polyfill,xx,yy,col=col, noclip = 0, _extra = extrastuff
        endelse
      endif
    endif
  endfor
endfor
end
