; ------------------------------------------------------------------------ 
; $Id: plotvec.pro,v 1.5 2009/11/05 20:31:16 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
 Pro plotvec, str1in, str2in, xrange = xrange, yrange = yrange, $
    color = color, arrow = arrow, bin= bin, thick = thick, fill = fill, $
    length = length, symsize = symsize, title = title, aspect = aspect, $
    border = border, xborder = xborder, yborder = yborder, zrange = zrange, $
    xstyle = xstyle, ystyle = ystyle, ignore = ignore, header = header, $
    zlog = zlog, setdefault = setdefault, unset = unset, rtheta = rtheta,  $
    showdefault = showdefault, thermometer = thermometer, tparam = tparam, $
     cubic = cubic, $
    transplot = transpt, overplot = overplot, _extra = extrastuff
  
;+
; NAME:
;       PLOTVEC
; PURPOSE:
;       Make a vector plot from two single block field structures
;         or from two attributes in a single block field structure      
;       Note: If aspect ratio of the X and Y ases are not nearly equal
;             then care must be exercised in in interpreting the vectors 
;             as (R, theta).  For example if the x axis goes from 0 to 10
;             and the y axis goes from 0 to .1 then almost all the vectors 
;             will be vertical on this plot.
;             Use the rtheta keyword to get an r-theta plot. That is with
;             equal field components the vector will be at 45 degrees.
; CATEGORY:
;       Graphics
; CALLING SEQUENCE:
;       PLOTVEC, STR1 [, STR2]
;       PLOTVEC, STR1 [, VECTORS]
; INPUTS:
;       STR1:    array or array number of the first field structure array 
;              - this must be two dimensional or three dimensional with 
;                a degenerate dimension and a single block.
;                If STR2 is given then STR1 must be a scalar field.
;                If STR2 is not given then STR1 must have 2 or more components.
;       STR2:    array or array number of field structure to be plotted with 
;                the first structure
;              - this must be two dimensional or three dimensional with 
;                a degenerate dimension, a single block, and have a scalar field.
;  or
;       VECTORS  array of two elements indicating the vector components to be 
;                used for the X and Y components of the vector plot 
;                Default value is the components corresponding to the 
;                nondegenerate dimensions.
;
; KEYWORDS:
;       ASPECT:  If set, aspect ratio is 1:1
;                Default is ASPECT =1
;       CUBIC:   If set, use cubic interpolation
;       XRANGE:  Range of X-values of interest for the plot
;       YRANGE:  Range of Y-values of interest for the plot
;       ZRANGE:  Range of Vector amplitudes of interest
;                Default for XRANGE, YRANGE and ZRANGE is the range of the 
;                  actual data unless ZLOG is set.  If the number of elements
;                  of each is less than 2, the actual values will be returned.
;                If ZLOG is set and ZRANGE has no elements then:
;                  ZRANGE(0) > 1.0e-3* (Maximum(VECTOR) = ZRANGE(1))
;                If ZLOG is set and ZRANGE has one element and that element is 
;                  greater than zero and less than 1 then it will be used to 
;                  scale the minimum value for the log plot
;                  ZRANGE(0) = (ZRANGE)* Maximum(VECTOR)
;                IF ZRANGE has 2 elements then the range will be between these
;                  two values.
;                If ZRANGE has 1 element and is equal to 1 then if the vectors 
;                  are from a single structure, zrange will be set to
;                    ZRANGE = [min(sqrt(v1*v1+v2*v2), max = mm), mm]
;                  where v1 and v2 are the vectros used for the vector plot.
;                  Currently, xrange and yrange are ignored.
;                Alternatively if you are doing:
;                     plotstr, 1, [1,2], cn=2, zrange= zrange
;                     plotvec, 1, [1,2], zrange = zrange 
;                then both plots will have the same min/max scaling.
;       ARROW:   If present and not zero, vectors will be indicated with an
;                arrow. (based on USERLIB routine, VELOVECT)
;                Default is ARROW =1; vectors indicated with a spot and bar.
;       FILL:    Make a solid arrow head.
;       RTHETA:  Make an R-THETA plot so that the angle of the vector depends 
;                on the relative magnitude of the fields and may not be the 
;                actual direction of the field
;       LENGTH:  Length factor in units of the point spacing (see BIN)
;                If length is negative, then all bars/arrows will be the same
;                length given by abs(length). Magnitude can be shown with color
;                Default is length = 1
;                NOTE: If length is negative and the magnitude of the vector is
;                less than 2^(-14) (ie, 6.10e-5) of the maximum amplitude, then
;                the vector will be zero length.
;       BIN:     Number of points in the horizontal and verticle direction.
;                If BIN is a vector then the [BIN(0), BIN(1)] are the number
;                in the horizontal and vericle direction.  If BIN is a scalar,
;                Then that number will be used for the longest direction; the 
;                same bin size will be used for then transverse direction.
;                Default is BIN = 30 .
;       THICK:   Thickness of the arrow or bar used for the vector. 
;                Thick is multiplied by !p.thick> 1
;                Default is 1
;       SYMSIZE: Size of the spot used for the vector origin if ARROW = 0
;                Default is 0.75
;       COLOR:   Array of colors for the vectors lines. 
;                If color is an array those colors will be used to quantify 
;                  the amplitudes
;                If color is not an array, then 
;                  If color = -1 then colors will go from 9 to !d.table_size
;                    uniformly spaced using the existing color table
;                    (if ZLOG is set the applitude will be logarithmic)
;                  If color is positive, that color will be used for all 
;                    contours, [1, 7] are the primary colors, 9 to !d.table_size
;                    depend on the color map.
;                  Color = 0 is the default plot color (ie, white or black)
;                    which is set by the system parameter, !P.COLOR
;                Default is set by SETDEFAULT or QSINIT
;                The colors are: red(1), green(2), blue(3),
;                purple(4), orange(5), light blue(6), yellow(7).
;       TITLE:   Title for the plot; Default is a combination of the attribute
;                or vector labels.
;       TRANSPLOT:If present and not zero, interchange ordinates and abscissa
;                SETDEFAULT can be used to set a default value
;                NOTE: This only switches spatial coordinates.  To switch
;                      vector coordinates, interchange the two vector components
;                      or the two structures.
;       OVERPLOT:If present and not zero, plot the vector plot on the 
;                current plot. Default is overplot =0
;       IGNORE:  If present and not zero, ignore the parameters from the previous
;                plot if this is an overplot.
;       BORDER:  If present and not zero, add a border to XRANGE and YRANGE 
;                given by BORDER*LENGTH*point spacing. XSTYLE and YSTYLE if 
;                not set are set to 1.
;       X(Y)BORDER: Ignored if border is set. This is the same as border but it
;                only applies to the X (or Y) axis 
;       ZLOG:    If present and not zero, a log scale will be used for the color
;                map and magnitude.  Default is ZLOG = 0
;                ZLOG = 0 scaling:
;                  length = (magnitude-level(0))/(level(1)-level(0))*max_length
;                ZLOG = 1 scaling:
;                  replace magnitude with ALOG(magnitude)
;                  replace level with ALOG(level)
;       HEADER   If specified a plot header, consisting of the current date
;                and the current file name, is displayed in the lower left
;                corner of the plot.  See the HEADER command to set default
;                Note: File name is taken from the first structure (STR1)
;                HEADER parameters (character size, spacing, alignment).
;                DEFAULT is HEADER = 1
;                NOTE: For X-windows screen dumps, many will want to increase
;                      the character size with the command:
;                      HEADER, /set, charsize = 0.8
;                      (Default is 0.6 of current size)
;    THERMOMETER- If keyword is set a thermometer will be generated
;                 with the vector plot.
;                 Default IDL margins are [10, 3] for x and [4, 2] for y
;                 If TPARAM does not have a Tag Name beginning with 'CU'
;                 then:
;                 If no tag names begin with 'H' xmargin will be set to
;                   [!x.margin(0), 10]
;                   otherwise ymargin will be set to [8, !y.margin(1)]
;           Note: xmargin or ymargin passed to plotfld will override
;                       these settings.
;           Note: If THERMOMETER is a structure, then TPARAM is ignored
;                 and set equal to THERMOMETER
;       TPARAM  - Structure containing data for the thermometer, Tag names
;                 must be valid for the THERMOMETER procedure.
;
;     SETDEFAULT-If present and not zero, then the following parameters will be
;                 saved and used as the default values in all future calls to PLO
;                 If a keyword is specified, it will always be used rather than
;                 the saved values.
;     UNSET     - If present and not zero, will set all save values of keywords
;                 to their default values.
;                 Normal default values are listed below.
;                 Keyword     Default value
;                 ARROW        1
;                 THICK        1
;                 BIN          [30, 30]
;                 XBORDER      0
;                 YBORDER      0
;                 LENGTH       1.0
;                 HEADER       1
;                 XRANGE       0
;                 YRANGE       0
;                 ZRANGE       1.0e-3
;                 SYMSIZE      0.75
;                 THERMOMETER  0
;     SHOWDEFAULT-Show the present default plot values.
;                
;       _extra:  Any additional keywords understood by the plot command.
;                This keyword is ignored if overplot is not equal to zero.
; OUTPUTS:
;       Vector plot of the designated field values.
; PROCEDURE:
;       This routine is based on VELOVECT.  Two options exist for the vector:
;       1. A spot with a bar going in the direction of the vector.
;       2. An arrow going in the direction of the vector.
;       If length is negative, all arrows/bars will be the same length.
;       If color is -1, then the arrows/bars & spots will be colored according
;         to their length and the existing color map.
;
; EXAMPLES:
;       Structure array 2 has a degenerate middle dimension and 3 components of 
;         the electric field.  Make a vector plot of the first and third 
;         components of the electric field and make a plot of the first and
;         second components of the field.
;               PLOTVEC, 2  (Components default to [3,1])
;               PLOTVEC, 2, [2,1]
;
;       Structure array 2 has the radial component of the electric field and
;       structure array 5 has the azial component of the electric field; the 
;       sturctures are degenerate in the theta direction. Make a vector plot
;       of these field values.
;               PLOTVEC, 5, 2
;       
;       Structure 3 is a 3D, multiblock electric field structure. In the plane 
;       Theta = [0,1], plot the x and y components of the electric in a vector
;       plot with arrows. Use structures 5 and 6 as working sturctures.
;       Note: If only a limited portion is of interest for the vector plot, 
;       use xrange, yrange, or block keywords to limit the region in the result
;       structures.
;             PLOTSTR, 3, 3, 2, [0, 1], cn=2, result =5
;             PLOTSTR, 3, 1, 2, [0, 1], cn=2, result = 6
;             PLOTVEC, 5, 6, /arrow
;       To obtain a background with a contour plot use 
;             PLOTSTR, 3, [1,3], 2, [0, 1], cn=2
;             PLOTVEC, 5, 6, color= 4, /over, /ignore, /arrow, leng = -0.75
;
; MODIFICATION HISTORY:
;      Original version, L. P. Mix, April 29, 1994
;-
;
; insert a common block
;
@qscom.dat
;  Common block
@qsstrcom.dat
;       return on an error
      on_error, 1
;
common plotvec_defaults, HEADER_SAVE, XRANGE_SAVE, YRANGE_SAVE, ZRANGE_SAVE, $
       ARROW_SAVE, BIN_SAVE, THICK_SAVE, XBORDER_SAVE, YBORDER_SAVE, $
       LENGTH_SAVE, SYMSIZE_SAVE, THERMOMETER_SAVE

      if n_elements (structinit) eq 0 then qsstruct_init
      if n_elements (qsplotinit) eq 0 then qsinit
;
      zrange_save_value = 1.0e-3
      if keyword_set(unset) or n_elements(header_save) eq 0 then begin
        HEADER_SAVE = 1
        XRANGE_SAVE = 0
        YRANGE_SAVE = 0
        ZRANGE_SAVE = zrange_save_value
        ARROW_SAVE = 1
        BIN_SAVE   = 30
        THICK_SAVE = 1.0
        XBORDER_SAVE = 0
        YBORDER_SAVE = 0
        LENGTH_SAVE = 1.0
        SYMSIZE_SAVE = 0.75
        THERMOMETER_SAVE = 0
      endif
;
      if keyword_set (setdefault) then begin
        if n_elements(header) gt 0 then header_save = header
        if n_elements(xrange) gt 0 then xrange_save = xrange
        if n_elements(yrange) gt 0 then yrange_save = yrange
        if n_elements(zrange) gt 0 then zrange_save = zrange
        if n_elements(arrow) gt 0 then arrow_save = arrow
        if n_elements(thick) gt 0 then thick_save = thick
        if n_elements(bin) gt 0 then bin_save = bin
        if n_elements(border) gt 0 then begin
          xborder_save = border
          yborder_save = border
        endif
        if n_elements(xborder) gt 0 then xborder_save = xborder
        if n_elements(yborder) gt 0 then yborder_save = yborder
        if n_elements(length) gt 0 then length_save = length
        if n_elements(symsize) gt 0 then symsize_save = symsize
        if n_elements(thermometer) gt 0 then thermometer_save = thermometer
      endif

      if keyword_set(showdefault) then begin
         print, 'Default parameters for the PLOTVEC command are:'
         print, '    HEADER     = ' , strtrim (header_save, 2)
         print, '    XRANGE     = ' , strtrim (xrange_save, 2)
         print, '    YRANGE     = ' , strtrim (yrange_save, 2)
         print, '    ZRANGE     = ' , strtrim (zrange_save, 2)
         print, '    ARROW      = ' , strtrim (arrow_SAVE, 2)
         print, '    BIN        = ' , strtrim (bin_save, 2)
         print, '    THICK      = ' , strtrim (THICK_SAVE, 2)
         print, '    XBORDER    = ' , strtrim (xborder_save, 2)
         print, '    YBORDER    = ' , strtrim (yborder_save, 2)
         print, '    LENGTH     = ' , strtrim (LENGTH_SAVE, 2)
         print, '    SYMSIZE    = ' , strtrim (SYMSIZE_SAVE, 2)
         ss = size(thermometer_save)
         if ss(ss(0)+1) eq 8 then begin
           print, '    THERMOMETER STRUCTURE:'
           help, /str, thermometer_save
           endif else $
         print, '    THERMOMETER= ' , strtrim (THERMOMETER_SAVE, 2)
      endif

      nparams = n_params()

      if nparams eq 0 and (keyword_set (setdefault) or keyword_set(unset) $
                  or keyword_set(showdefault) ) $
        then return
      if nparams lt 1 then begin
        print,'% Format: ', $
        'PLOTVEC, STR1 [, STR2] [, XRANGE = xrange] [, YRANGE = yrange]'
print,'       [, ZRANGE = zrange] [, THERMOMETER = thermometer] '
print,'       [, TPARAM = tparam] [, SHOWDEFAULT = showdefault]'
print,'       [, SETDEFAULT = setdefault] [, UNSET = unset]'
print,'       [, ZLOG = zlog] [, HEADER = header] [, THICK = thick]'
print,'       [, COLOR = color] [, ARROW = arrow] [, BIN = bin] '
print,'       [, LENGTH = length] [, SYMSIZE = symsize] [, TITLE = title] '
print,'       [, TRANSPLOT = transplot] [, OVERPLOT = overplot]'
print,'       [, BORDER = border] [, XBORDER = xborder] [, YBORDER = yborder]'
print,'       [, XSTYLE = xstyle] [, YSTYLE = ystyle] [, ASPECT = aspect]'
print,'       [,  _EXTRA = extrastuff]'
        return
      endif
;
      if n_elements(str1in) eq 0 then begin
        print, 'Parameters to PLOTVEC must be defined'
        return
      endif
      a = str1in
            b = size(a)
      if b(b(0)+1) ne 8 then begin
        if a gt 0 and a le maxstructure then begin
          s2i, a, a
          b = size (a)
          if b(b(0)+1) ne 8 then begin
            print, $
   ' First parameter must be a structure or a structure array number'
            return
          endif
        endif else begin
          print, $
   ' First parameter must be a structure or a valid structure array number'
          return
        endelse
      endif
      if a.type ne 1 then begin
        print, ' Only Field type structures may be plotted with PLOTVEC'
        return
      endif
      if a.two eq 0 and a.ndim ge 3 then begin
        print, ' Structure input to PLOTVEC must have two dimensions or '
        print, '   have 3 dimensions with a degererate dimension.'
        return
      endif
      if a.block gt 1 then begin
        print, ' Structures input to PLOTVEC must have a single block'
        return
      endif
      two = a.two
      if n_elements (transdefault) eq 0 then transdefault = 0
      if n_elements (transpt) eq 0 then transplot = transdefault else $
        transplot = transpt
      if n_elements (overplot) eq 0 then overplot = 0
      if n_elements (aspect) eq 0 then aspect = 1
      if a.ndim eq 2 then begin
        xval = 1 & yval = 2 
      endif else if a.ndim eq 3 then begin
        if two eq 2 then begin
          if transplot eq 0 then transplot = 1 else transplot = 0
          xval = 1 & yval = 3
        endif else begin
          xval = (two mod 3) + 1
          yval = ((two + 1) mod 3) + 1
        endelse
      endif
      xnew = xval
      ynew = yval
;
      if a.ndim eq 4 then begin
        print, ' Current version not valid for a dimension of 4'
        return
      endif
;     check on the values for the third dimension
;     reset the xold and yold for a possible overlay plot
      if n_elements(ignore) eq 0 then ignore = 0
      if overplot eq 0 then begin
        if transplot eq 0 then begin
          xoldvalue = xnew
          yoldvalue = ynew
        endif else begin
          xoldvalue = ynew
          yoldvalue = xnew
          temp = xnew & xnew = ynew & ynew = temp
        endelse
        zplaneold = a.two
        zvalueold = a.slice
      endif else begin
        if ignore eq 0 then begin
        if xnew eq xoldvalue and ynew eq yoldvalue then begin
          if transplot ne 0 then begin
            print, ' The contours will not be transposed for this plot.'
             print, ' Hit "q" to quit; hit any other key to continue'
             key =  (byte (get_kbrd(1)))(0)
             if ((key eq 81) or (key eq 113)) then return
            transplot = 0
          endif
        endif else if xnew eq yoldvalue and ynew eq xoldvalue then begin
          if transplot eq 0 then begin
            print, ' The contours will be transposed for this plot.'
             print, ' Hit "q" to quit; hit any other key to continue'
             key =  (byte (get_kbrd(1)))(0)
             if ((key eq 81) or (key eq 113)) then return
            transplot = 1
          endif
        endif
        endif
        if transplot ne 0 then begin
          temp = xnew & xnew = ynew & ynew = temp
        endif
        if ignore eq 0 then begin
        if xnew ne xoldvalue then begin
          Print, ' WARNING: This overplot requested with X-axis of ', xnew, $
              ', present value is', xoldvalue
          print, ' Hit "q" to quit; hit any other key to continue'
          key =  (byte (get_kbrd(1)))(0)
          if ((key eq 81) or (key eq 113)) then return
        endif
        if ynew ne yoldvalue then begin
          Print, ' WARNING: This overplot requested with Y-axis of ', ynew, $
              ', present value is', yoldvalue
          print, ' Hit "q" to quit; hit any other key to continue'
          key =  (byte (get_kbrd(1)))(0)
          if ((key eq 81) or (key eq 113)) then return
        endif
        endif
      endelse
;
      str1 = a
      if str1.nvect eq 1 then lcb =1  else lcb = 0
      nel = n_elements(str2in)
      if nel eq 0 then begin
        if lcb eq 1 then begin
          print, ' A second vector must be specified'
          print, ' First structure array is a scalar field.'
          return
        endif else begin
          vect = [xnew, ynew]
          if max(vect) gt a.nvect then begin
            print, ' Cannot plot components', vect
            print, ' First structure has only ',strtrim(a.nvect,2),' components'
            return
          endif
        endelse
        lctwo = 0
      endif else if nel eq 1 then begin
        a = str2in
        b = size (a)
        if b(b(0)+1) ne 8 then begin
          if a gt 0 and a le maxstructure then begin
            s2i, a, a
            b = size (a)
            if b(b(0)+1) ne 8 then begin
              print, $
   ' Second parameter must be a structure or a structure array number'
              return
            endif
          endif  else begin
            print, $
   ' Second parameter must be a structure or a valid structure array number'
            return
          endelse
        endif
        if a.type ne 1 then begin
          print, ' Only Field type structures may be plotted with PLOTVEC'
          return
        endif
        if a.two eq xnew or a.two eq ynew then begin
          Print, ' Cannot make a vector plot with a degenerate axis'
          return
        endif
        if a.block gt 1 then begin
          print, ' Structure 2 for PLOTVEC must have a single block'
          return
        endif
        if a.two eq 0 and a.ndim ge 3 then begin
          print, ' Structure2 input to PLOTVEC must have two dimensions or '
          print, '   have 3 dimensions with a degererate dimension.'
          return
        endif
        vect = [1,1]
        str2 = a
        lctwo = 1
      endif else begin
        vect = abs(str2in(0:1))
        if max(vect) gt str1.nvect then begin
          print, ' Cannot plot components', vect
          print, ' First structure has only ',strtrim(str1.nvect,2),' components'
          return
        endif
        lctwo = 0
      endelse
      nel = n_elements (bin)
      if nel eq 0 then bin = bin_save else begin
        if nel eq 1 then bin = bin(0) else bin = bin(0:1)
      endelse
      if n_elements(arrow) eq 0 then arrow = arrow_save
      if n_elements(length) eq 0 then length = length_save
      if n_elements(thick) eq 0 then thick = thick_save
      if n_elements(symsize) eq 0 then symsize = symsize_save
      if n_elements(color) eq 0 then color = fldcolor else color = color
      vector1 = reform (get_array (str1, -vect(0), 1, lab = tlab1))
      if lctwo eq 0 then $
        vector2 = reform(get_array (str1, -vect(1), 1, lab = tlab2)) $
        else vector2 = reform(get_array (str2, -vect(1), 1, lab = tlab2))
      if n_elements(title) eq 0 then $
        title = 'Vector Plot -' + strtrim(tlab1, 2)+' , ' + strtrim(tlab2, 2)
      x1 = get_array(str1, xnew, 1, lab = xlab)
      y1 = get_array(str1, ynew, 1, lab = ylab) 
;
      filename = strtrim(str1.file, 2)
      if lctwo ne 0 then begin
        x2 = get_array(str2, xnew, 1, lab = xlab)
        y2 = get_array(str2, ynew, 1, lab = ylab) 
        xmin = max([str1.range(0,xnew-1,0), str2.range(0,xnew-1,0)])
        xmax = min([str1.range(1,xnew-1,0), str2.range(1,xnew-1,0)])
        ymax = min([str1.range(1,ynew-1,0), str2.range(1,ynew-1,0)])
        ymin = max([str1.range(0,ynew-1,0), str2.range(0,ynew-1,0)])
        if strlen(filename) eq 0 then filename = strtrim(str2.file, 2)
      endif else begin
        xmin = str1.range(0,xnew-1,0)
        xmax = str1.range(1,xnew-1,0)
        ymin = str1.range(0,ynew-1,0)
        ymax = str1.range(1,ynew-1,0)
      endelse
      if keyword_set (transplot) then begin
        vector1 = transpose (vector1)
        vector2 = transpose (vector2)
      endif
      if n_elements (xrange) lt 2 then xrange = [xmin, xmax] else $
         xrange = xrange(0:1)
      if abs(xrange (1)-xrange(0)) lt 1e-5* max(abs(xrange)) then begin
        print, 'Range of X-Values is less than 1e-5 * value - STOPPING'
        return
      endif
      if n_elements (yrange) lt 2 then yrange = [ymin, ymax] else $
         yrange = yrange(0:1)
      if ABS(yrange (1)-yrange(0)) lt 1e-5* max(abs(yrange)) then begin
        print, 'Range of Y-Values is less than 1e-5 * value - STOPPING'
        return
      endif
      xmin = min(float(xrange), max = xmax)
      ymin = min(float(yrange), max = ymax)
      if keyword_set (overplot) then begin
        xmin = max([xmin, !x.crange(0)])
        xmax = min([xmax, !x.crange(1)])
        ymin = max([ymin, !y.crange(0)])
        ymax = min([ymax, !y.crange(1)])
      endif
;
      deltax = xmax - xmin
      deltay = ymax - ymin
      if n_elements(bin) eq 1 then begin
        if deltax gt deltay then bin = round (bin*[1.0, deltay/deltax]) $
           else bin = round (bin*[deltax/deltay, 1.0])
      endif
      bin = bin > 2
      xstep = deltax/bin(0)
      ystep = deltay/bin(1)
      xplot = xmin +xstep*0.5+xstep*findgen(bin(0))
      yplot = ymin +ystep*0.5+ystep*findgen(bin(1))
      p1 = get_yarr(x1, findgen(str1.size(xnew-1, 0)), xplot, error = err1)
      q1 =  get_yarr(y1, findgen(str1.size(ynew-1, 0)), yplot, error= err2)
      if n_elements (cubic) eq 0 then cubic = 0
      if n_elements (p1) lt 2 or n_elements(q1) lt 2 then begin
        print, '% PLOTVEC: IM1, Unable to get two locations in each direction'
        help, p1, q1
        return
      endif
      if keyword_set (zrange) then begin
        if n_elements(zrange) eq 1 and zrange[0] eq 1 then begin
          if lctwo eq 0 then begin
            zrange=[min(sqrt(vector1*vector1+vector2*vector2), max=mm), mm]
            print, '% PLOTVEC: ZRANGE scaled to: [ ' , strtrim(zrange[0],2), $
              ', ', strtrim(zrange[1],2), ']'
          endif
        endif
      endif
      im1 = interpolate( vector1, p1, q1, cubic = cubic, /grid)
      if lctwo eq 0 then  $
        im2 = interpolate(vector2, p1, q1, cubic= cubic, /grid) $
      else begin
        p1 = get_yarr(x2, findgen(str2.size(xnew-1, 0)), xplot)
        q1 =  get_yarr(y2, findgen(str2.size(ynew-1, 0)), yplot)
        if n_elements (p1) lt 2 or n_elements(q1) lt 2 then begin
          print, '% PLOTVEC: IM2, Unable to get two locations in each direction'
          help, p1, q1
          return
        endif
        im2 = interpolate(vector2, p1, q1, cubic= cubic, /grid)
      endelse
      b = size(im2)
      bin = b(1:2)
      if n_elements(border) eq 0 then begin
        if n_elements(xborder) eq 0 then xborder = xborder_save else begin
          if xborder le 0 then xborder = 0 else begin
            if n_elements(xstyle) eq 0 then xstyle = 1
          endelse
        endelse
        if n_elements(yborder) eq 0 then yborder = Yborder_save else begin
          if yborder le 0 then yborder = 0 else begin
            if n_elements(ystyle) eq 0 then ystyle = 1
          endelse
        endelse
        border = max ([xborder, yborder])
      endif else begin
        if border le 0 then border= 0 else begin
          xborder = border
          yborder = border
          if n_elements(xstyle) eq 0 then xstyle = 1
          if n_elements(ystyle) eq 0 then ystyle = 1
        endelse
      endelse
      if n_elements(xstyle) eq 0 then xstyle = 0
      if n_elements(ystyle) eq 0 then ystyle = 0
      if border gt 0 then begin
        xborder = xborder*xstep*abs(length)
        xrange = xrange + [-xborder, xborder]
        yborder = yborder*ystep*abs(length)
        yrange = yrange + [-yborder, yborder]
      endif
      if keyword_set(thermometer) then begin
        ss = size (thermometer)
        if ss(ss(0)+1) eq 8 then tparam = thermometer
      endif
      if overplot eq 0 then begin
        xmargin = !x.margin
        ymargin = !y.margin
        if keyword_set(thermometer) then begin
          xmargin = [!x.margin(0), 10]
          if n_elements (tparam) gt 0 then begin
            b = size(tparam)
            if b(b(0)+1) ne 8 then begin
              winformation, $
              ['TPARAM must be a structure', 'TPARAM will be set to null']
              tparam = {title:''}
            endif else begin
              b = tag_names(tparam)
              ok = where (strmid(b, 0, 2) eq 'CU', nok)
              if nok eq 0 then begin
                ok = where (strmid(b, 0, 1) eq 'H', nok)
                if nok gt 0 then begin
                  ymargin = [8, !y.margin(1)]
                  xmargin = !x.margin
                endif
              endif else xmargin = !x.margin
            endelse
          endif
        endif
        multcnt = !p.multi(0)
        if keyword_set(aspect) then begin
          ss = size(extrastuff)
          if ss(ss(0)+1) eq 8 then begin
            myextra = extrastuff
            tags = tag_names(myextra)
            ok = where(strpos(tags, 'XS', 0) eq 0, nok)
            if nok gt 0 then i = execute('myextra.'+tags(ok(0))+ $
              '= myextra.'+tags(ok(0))+' or 4')
            ok = where(strpos(tags, 'YS', 0) eq 0, nok)

            if nok gt 0 then  begin
              i = execute('myextra.'+tags(ok(0))+ $
              '= myextra.'+tags(ok(0))+' or 4')
              i = execute('extrastuff.'+tags(ok(0))+ $
              '= extrastuff.'+tags(ok(0))+' or ystyle')
            endif else if ystyle gt 0 then  $
              extrastuff = create_struct(extrastuff, 'ystyle', ystyle)

          endif else if ystyle gt 0 then extrastuff ={ystyle:ystyle}
          plot,  x1, y1, /nodata, $
            xrange= xrange, yrange= yrange, xstyle =4, ystyle = 4, $
            xmargin = xmargin, ymargin = ymargin, _extra = myextra
          xpos = round(!x.window*!d.x_size)
          ypos = round(!y.window*!d.y_size)
          xr = !x.crange(1)-!x.crange(0)
          yr = !y.crange(1)-!y.crange(0)
          xsiz = xpos(1)-xpos(0)+1
          ysiz = ypos(1)-ypos(0)+1
          if xr/xsiz ge yr/ysiz then begin
            ysiz = round(yr/xr*xsiz)
            position = [xpos(0), ypos(0), xpos(1), ypos(0)+ysiz]
          endif else begin
            xsiz = round(xr/yr*ysiz)
            position = [xpos(0), ypos(0), xpos(0)+xsiz, ypos(1)]
          endelse
          if multcnt eq 0 then erase
          !p.multi(0) = !p.multi(0)+1
          wxsave = !x.region
          wysave = !y.region
          plot, x1, y1, xrange= xrange, yrange = yrange, $
            title= title,xtit= xlab, ytit = ylab,  $
            pos=get_aspect(xrange, yrange, ymargin=ymargin, xmargin=xmargin), $
            xstyle = xstyle, ystyle = ystyle, _extra = extrastuff, /nodata
          !x.region=wxsave
          !y.region=wysave
        endif else plot, x1, y1, xrange= xrange, yrange = yrange, $
          title= title,xtit= xlab, ytit = ylab, xmargin = xmargin, $
          ymargin = ymargin, $
          xstyle = xstyle, ystyle = ystyle, _extra = extrastuff, /nodata
        if n_elements(header) eq 0 then header = header_save
        if header ne 0 and multcnt eq 0 then begin
          if strlen(filename) eq 0 then header_text='Date: ' + thin_date() $
            else header_text = ['Date: ' + thin_date(), 'File: ' +filename]
          header, header_text
        endif
      endif
        
      mag = sqrt(im1^2 + im2^2)
      if keyword_set(rtheta) then begin
        im1  = im1*((!x.crange(1) -!x.crange(0)) $
                /(!y.crange(1)-!y.crange(0)) $
                * !d.y_size*(!y.window(1)-!y.window(0))  $
                / !d.x_size/(!x.window(1)-!x.window(0)))
      endif 
      maxcol = max(mag, min = mm)
      print, 'Range of the amplitude is: ',strtrim(mm,2),' to ',strtrim(maxcol,2)
      if n_elements(zlog) eq 0 then zlog = 0
      if n_elements(zrange) eq 0 then zrange = zrange_save
      if n_elements (zrange) gt 1 then begin
        zrange = [min(zrange(0:1), max = mm), mm]     
        ok = where (mag ge zrange[0] and mag le zrange[1], nok, comp = comp, ncomp = ncomp)
        mag = mag > zrange(0) < mm
        maxcol = max(mag, min = mm)
      endif else ncomp = 0
      if keyword_set(zlog) then begin
        if n_elements(zrange) eq 1 then begin
          if zrange gt 0 and zrange lt 1.0 then begin
            zrange = [zrange*maxcol, maxcol]
          endif else zrange = [zrange_save_value*maxcol, maxcol]
          mag = mag> zrange(0)
          if mm lt zrange(0) then mm = zrange(0)
        endif
        if mm le 0.0 then begin
          zrange(0) = zrange_save_value*maxcol
          print, ' Minimum cannot be zero for a LOG plot'
          print, ' Minimum set to ', strtrim(zrange(0),2)
          mag = mag> zrange(0)
        endif
        magc = alog(mag/zrange(0))
        mmag = magc/mag
        im1t = im1*mmag
        im2t = im2*mmag
        maxmag = max([max(abs(im1t/xstep)), max(abs(im2t/ystep))])
        if  length gt 0 then begin
          sina = im1t*(length/maxmag) 
          cosa = im2t*(length/maxmag) 
        endif else begin
          magnor = sqrt((im1t/xstep)^2 + (im2t/ystep)^2)
          a = where (magnor lt 6.10e-5*maxmag, nzero)
          if nzero gt 0 then magnor (a) = maxmag
          sina = -length*im1t/magnor
          cosa = -length*im2t/magnor
        endelse

      endif else begin
        if n_elements(zrange) lt 2 then zrange =[mm, maxcol]
        maxmag = max([max(abs(im1/xstep)), max(abs(im2/ystep))])
        if  length gt 0 then begin
          sina = im1*(length/maxmag)
          cosa = im2*(length/maxmag)
        endif else begin
          magnor = sqrt((im1/xstep)^2 + (im2/ystep)^2)
          a = where (magnor lt 6.10e-5*maxmag, nzero)
          if nzero gt 0 then magnor (a) = maxmag
          sina = -length*im1/magnor
          cosa = -length*im2/magnor
        endelse
      endelse
      print, 'Range of the vectors is: ', strtrim(mm, 2), ' to ',strtrim(maxcol,2)

      r = 0.32                      ;length of arrow head
      angle = 22.5 * !dtor          ;Angle of arrowhead
      angle = 15.0 * !dtor            ;Angle of arrowhead
      st = r * sin(angle)             ;sin 22.5 degs * length of head
      ct = r * cos(angle)
      lccolor = 0

;
      mkcircle
      nelc = n_elements(color)
      if nelc eq 1 then begin
        if color eq 0 then begin
          color = !p.color 
          levels = zrange
          clevels = [color, color]
        endif else if color eq -1 then begin
          if keyword_set(zlog) then begin
            color = round((!d.table_size-11)/alog(zrange(1)/zrange(0))* $
               magc) + 9
            levels= findgen(81)*(0.0125*alog(zrange(1)/zrange(0)))
            clevels =  round((!d.table_size-11)/alog(zrange(1)/zrange(0))* $
              levels ) + 9
            levels=exp(alog(zrange(0))+levels)
          endif else begin
            levels = zrange(0) + findgen(81)*(0.0125*(zrange(1)-zrange(0)))
            color = round((!d.table_size-11)/(zrange(1)-zrange(0))* $
               (mag-zrange(0))) + 9
            clevels = round((!d.table_size-11)/(zrange(1)-zrange(0))* $
               (levels-zrange(0))) + 9
          endelse
          lccolor = 1
        endif
      endif else begin
        lccolor = 1
        if keyword_set(zlog) then begin
          index = $
          fix(magc*(nelc/alog(zrange(1)/zrange(0))))<(nelc-1) 
          levels=findgen(81)*(0.0125*alog(zrange(1)/zrange(0)))
          clevels = color( $
          fix(levels*(nelc/alog(zrange(1)/zrange(0))))<(nelc-1))
          levels = exp(alog(zrange(0))+levels)
        endif else begin
          levels = zrange(0) + findgen(81)*(0.0125*(zrange(1)-zrange(0)))
          index = fix((mag-mm)*(nelc/(maxcol-mm)))<(nelc-1)
          clevels = color(fix((mag-mm)*(nelc/(maxcol-mm)))<(nelc-1))
        endelse
        color = color(index)
      endelse

; use xd and yd to scale the arrow heads
; xd, yd are the graphical distance/pixel
       xd=(!x.crange(1)-!x.crange(0))/((!x.window(1)-!x.window(0))*!d.x_size)
       yd=(!y.crange(1)-!y.crange(0))/((!y.window(1)-!y.window(0))*!d.y_size)
;   xd = (xd+yd)*0.5 & yd = xd

      nc = n_elements(color)
      cc = color(0)
      if !p.thick eq 0 then thickp = 1 else thickp = !p.thick
      thickp = thick * thickp
      if arrow eq 0 then begin
        for i = 0 , bin(0)-1 do begin
          x0 = xplot(i)
          for j = 0, bin(1) - 1 do begin
            if ncomp gt 0 then ok = where ((i +j*bin[0]) eq comp, nok) else nok = 0
            if nok eq 0 then begin
              dx = sina(i, j)
              x1 = x0 + dx
              y0 = yplot(j)
              dy = cosa(i, j)
              y1 = y0 + dy
              if lccolor ne 0 then cc = color(i,j)
              plots, x0, y0, psym = 8, color = cc, symsize = symsize
              plots, [x0, x1], [y0, y1], color=cc, thick = thickp 
            endif
          endfor
        endfor
      endif else begin
        for i = 0 , bin(0)-1 do begin
          x0 = xplot(i)
          for j = 0, bin(1) - 1 do begin
            if ncomp gt 0 then ok = where ((i +j*bin[0]) eq comp, nok) else nok = 0
            if nok eq 0 then begin
              dx = sina(i, j)
              x1 = x0 + dx
              y0 = yplot(j)
              dy = cosa(i, j)
              y1 = y0 + dy
              if lccolor ne 0 then cc = color(i,j)
              plots,[x0,x1,x1-(ct*dx/xd-st*dy/yd)*xd, x1, $
                x1-(ct*dx/xd+st*dy/yd)*xd], [y0,y1,y1-(ct*dy/yd+st*dx/xd)*yd, $
                y1,y1-(ct*dy/yd-st*dx/xd)*yd], color = cc, thick = thickp
              if keyword_set (fill) then $
                polyfill , [x1-(ct*dx/xd-st*dy/yd)*xd, x1, $
                x1-(ct*dx/xd+st*dy/yd)*xd, x1-(ct*dx/xd-st*dy/yd)*xd], $
                [y1-(ct*dy/yd+st*dx/xd)*yd,  y1, y1-(ct*dy/yd-st*dx/xd)*yd, $
                y1-(ct*dy/yd+st*dx/xd)*yd], color = cc
            endif
          endfor
        endfor
      endelse
      if keyword_set(thermometer) then $
        thermometer, levels, clevels, ylog = zlog,  _extra = tparam 
      return
      end
