; ------------------------------------------------------------------------ 
; $Id: plotwt.pro,v 1.11 2009/12/16 16:15:14 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
 Pro plotwt, x, y, wt, colors = colorin, psymbol = psym, symsize = symsize, $
        overplot = overplot, zrange= zrange_in, thermometer = thermometer, $
        xrange = xrange, yrange = yrange, xstyle = xstyle, xtype = xtype, $
        ystyle  = ystyle, ytype = ytype, fast = fast, quiet= quiet, $
        xtitle = xtitle, ytitle= ytitle , title = title, aspect = aspect, $
        c_range = c_range_in, left = left, right = right, ztype = ztype, $
        _extra = extrastuff
;+
; NAME:
;       PLOTWT
;
; PURPOSE:
;       Plot two arrays with color weighting
; CATEGORY:
;       Input/Output
; CALLING SEQUENCE:
;       PLOTWT, X, Y, WT, [, PSYMBOL = PSYM] [SYMSIZE = SYMSIZE] 
;           [, OVERPLOT = OVERPLOT] [, COLOR = COLOR] [, TITLE = TITLE]
;           [, XRANGE = XRANGE] [, YRANGE = YRANGE] [YSTYLE = YSTYLE]
;           [, XSTYLE = XSTYLE] [, XTYPE = XTYPE] [, FAST = FAST]
;           [, XTITLE = XTITLE] [, YTITLE = YTITLE]  [, QUIET = quiet]
;           [, THERMOMETER = thermometer] [, ZRANGE = zrange] [, C_RANGE= c_range]
;           [, /LEFT] [, /RIGHT] [, ASPECT = ASPECT] [, ZTYPE = ztype]
;           [, _EXTRA = extrastuff]
; INPUTS:
;       X:        Abscissa values
;       Y:        Ordinate values
; OPTIONAL INPUT:
;       WT:       Weight array used to determine color or the points
;                 Note: If electron charge is used, use -wt.
;                 Default - WT is 1
;                 ZRANGE keywork applies to wt
; KEYWORD PARAMETERS:
;       ZRANGE    Scaling range for WT
;                 If log scaling, the absolute value is used and the log taken.
;       ZTYPE     If set, WT scaling is logarithmic
;                 If negative, use it to determine the minimum for log scaling
;                    M = max(wt)
;                    wt = log (wt > abs(ztype)*M)
;                 If ZTYPE gt 0, then ZTYPE = -(1e-4)
;       THERMOMETER If set a thermometer will be generated.
;                   Ignored if no weight is specified.
;       Note: THERMOMETER must have a non-zero range for either WT or ZRANGE
;       PSYMBOL      Symbol type - Default is 8 or 5 (see FAST)
;       FAST      Indicates a fast plot is desired.  If PSYMBOL is not 
;                 specified and fast is not zero, a triangle will be used.
;       SYMSIZE:  Symbol size multiplier - Default is 0.3
;       OVERPLOT  If not zero, indicates no new grid is to be drawn.
;       COLOR     If wt has fewer elements than the x and y arrays, then
;                 the points will be drawn using color = color. 
;                   The colors are: black(0), red(1), green(2), blue(3),
;                 purple(4), orange(5), light blue(6), yellow(7) and white(8).
;                 If color eq 0 then color = !p.color
;       (X,Y)RANGE, STYLE, TYPE have the standard IDL meaning.
;       TITLE, XTITLE and YTITLE If grid is not zero, these will be used for
;                 axis and the plot labels.
;       C_RANGE:  Two element vector specifying the range of the color table to
;                 use for the color weighted particles.
;                 First number is the lower value, and second is the upper
;                 value expressed as a fraction of full scale.
;                 Default: C_RANGE=[0,1] = [9, !d.table_size-2]
;                 Alternatively, elements of the color table can be specified.
;                 If max(C_RANGE) gt 1 then elements are assumed.
;                 The first number should be ge 9 and the second number should
;                 be le (!d.table_size-2) if not they will be changed.
;       LEFT     Indicates a new plot using the left axis.
;                A single axis will be drawn on the left side of the grid
;                For this plot XMARGIN will be set to
;                   XMARGIN = [!X.margin(0), !X.margin(0)]
;                and YSTYLE = 8.
;                NOTE: If LEFT is set, OVERPLOT is set to 0.
;      RIGHT     Indicates a OVERPLOT making and using the right axis.
;                If RIGHT is present and not zero, then:
;                  A single axis will be drawn on the right side of the existing
;                    grid
;                  The particle data will be overplotted on the existing grid.
;                Before using this command, a plot should be created using
;                  the keyword: /LEFT
;                NOTE: If RIGHT is set then OVERPLOT is set to 1.
;                NOTE: After this command all memory of the original y-axis
;                      scaling is lost
;       QUIET   - If present and not zero,level information will not be printed.
;                 If quiet has a value of 1 (/QUIET or QUIET =1), NO plots
;                 will be make.  If QUIET is not zero and not 1, then plots
;                 will be made, but no output to the terminal.
;                 Default: QUIET = 0
;       ASPECT  - If set, the ordinate and abscissa will have the 
;                   same relative scaling.
;       _EXTRA   Structure sent to the plot command using the _EXTRA keyword
;
; OUTPUTS:
;       Designated datasets are plotted.
; EXAMPLE:
;       Plot Y versus X with Charge used as a weight
;       plotwt, x, y, charge
;
; MODIFICATION HISTORY:
;      Original version,  L. P. Mix, March 5, 1993
;      September 15, 2005 Added ZRange and ZType to scale the amplitude
;      November 20, 2005  P. Leyde modified to draw when wmin = wmax and zrange not specified.
;-
;     common block
      common mkcircle, lcmkcircle, nel_symbols
      
on_error, 1
;
common plot_grid_last, yeng_scale, xeng_scale

      if n_elements(quiet) eq 0 then quiet = 0
      if quiet eq 1 then return
      if n_elements(lcmkcircle) eq 0 then mkcircle
      if lcmkcircle eq 0 then mkcircle
;     make sure we don't corrupt input data
      if n_elements(zrange_in) eq 0 then zrange = 0 else zrange = zrange_in
      if n_elements(ztype) eq 0 then ztype = 0
      if keyword_set (left) or keyword_set(thermometer) then begin
        overplot = 0
        xmargin = n_elements(xmargin) lt 2 ? [!x.margin(0), !x.margin(0)] : xmargin
        if keyword_set(left) then begin
          if n_elements(ystyle) eq 0 then ystyle = 8 else  $
            ystyle =   ystyle or  8
        endif
      endif else xmargin = !x.margin
      if keyword_set (right) then overplot = 1
      nx = n_elements (x)
      ny = n_elements (y)
      nw = n_elements (wt)
      if ztype ne 0  and nw gt 0 then begin
        logmin = (ztype gt 0) ? 1e-4 : -ztype 
        mnwt = max(wt) * logmin
        wt = alog10 (wt > mnwt)
        if n_elements(zrange) gt 1 then zrange = alog10(abs(zrange))
      endif
      if n_elements(zrange) gt 1 then zrange = [min(zrange, max= mxzr), mxzr]
;
      if !x.crange(0) eq !x.crange(1) then begin
        overplot = 0
      endif
      n = min([nx, ny])
      if n eq 0 then begin
        print, '% PLOTWT: X and Y arrays must be defined'
        return
      endif
      if nx ne ny then begin
         x = x(0:n-1)
         y = y(0:n-1)
      endif
      if n_elements (overplot) eq 0 then overplot = 0
      if n_elements(fast) eq 0 then fast = 0
      if n_elements (psym) eq 0 then if fast eq 1 then psym = 5 else psym = 8
      if psym lt 1 or psym gt 8 then psym = 8
      if n_elements (symsize) eq 0 then symsize = 0.3
      if overplot eq 0 then begin
        xeng_scale = 1.0
        yeng_scale = 1.0

        if n_elements(title) eq 0 then title = ''
        if n_elements(xtitle) eq 0 then xtitle = ''
        if n_elements(ytitle) eq 0 then ytitle = ''
        if n_elements (xrange) lt 2 then xrange = [min(x, max= mm), mm]
        if n_elements (yrange) lt 2 then yrange = [min(y, max= mm), mm]
        if n_elements (xstyle) eq 0 then xstyle = 0
        if n_elements (ystyle) eq 0 then ystyle = 0
        if n_elements (xtype) eq 0 then xtype = 0
        if n_elements (ytype) eq 0 then ytype = 0
        if keyword_set (thermometer) then begin
          ss = size (thermometer)
          if ss(ss(0)+1) eq 8 then begin
            tparam = thermometer
            thermometer = 1
          endif
        endif
        if keyword_set (aspect) then begin
;           pos = get_aspect(xrange, yrange, xmargin = xmargin)
;           xstyle = (xstyle or 1)
;           ystyle = (ystyle or 1)
;           plot, x,y, /nodata, xrange = xrange, yrange = yrange, $
;             ystyle = ystyle, xstyle = xstyle, xtype = xtype, $
;             ytype  = ytype, _extra = extrastuff, xmargin = xmargin, $
;             xtitle = xtitle, ytitle= ytitle, title= title , $
;             position = pos

          contour, make_array(2,2), xrange, yrange, /nodata, position = position, $
            xrange= xrange, yrange= yrange, xstyle = (4 or xstyle), ystyle = (4 or ystyle), $
            xmargin = xmargin, ymargin = ymargin,  _extra = extrastuff
          xrange_first = !x.crange
          yrange_first = !y.crange
          xpos = round(!x.window*!d.x_size)
          ypos = round(!y.window*!d.y_size)
          xr = !x.crange(1)-!x.crange(0)
          yr = !y.crange(1)-!y.crange(0)
          xsiz = xpos(1)-xpos(0)
          ysiz = ypos(1)-ypos(0)
          if !d.X_PX_CM eq  !d.Y_PX_CM then PRAT = 1.0 else $
            prat = !d.X_PX_CM /  !d.Y_PX_CM
          prat = 1.0
          xstyle = (xstyle or 1)
          ystyle = (ystyle or 1)
          if xr/xsiz ge yr/ysiz/prat then begin
            ysiz = round(yr/xr*xsiz/prat)
            position = [xpos(0), ypos(0), xpos(1), ypos(0)+ysiz]
          endif else begin
            xsiz = round(xr/yr*ysiz*prat)
            position = [xpos(0), ypos(0), xpos(0)+xsiz, ypos(1)]
          endelse
          wxsave = !x.region
          wysave = !y.region
          !p.multi (0)= !p.multi(0)+1
          contour, make_array(2,2), xrange, yrange, /nodata, position = position, $
            xrange= xrange_first, yrange= yrange_first, /xstyle , /ystyle, $
            xmargin = xmargin, ymargin = ymargin, xtype = xtype, /dev, $
            ytype  = ytype, xtitle = xtitle, ytitle= ytitle, title= title, $
            _extra = extrastuff 
          !x.region=wxsave
          !y.region=wysave
        endif else $
	  plot, x,y, /nodata, xrange = xrange, yrange = yrange, $
            ystyle = ystyle, xstyle = xstyle, xtype = xtype, $
            ytype  = ytype, _extra = extrastuff, xmargin = xmargin, $
            xtitle = xtitle, ytitle= ytitle, title= title
      endif else begin
        nn = n
        if keyword_set(right) then begin
          if n_elements (yrange) lt 2 then yrange = [min(y, max= mm), mm]
          if n_elements(ytitle) eq 0 then ytitle = ''
          axis, /yaxis, /save, !x.crange(1), 0, yrange = yrange, $
              ytitle= ytitle
        endif
        xrc = !x.crange
        if !x.type eq 1 then xrc = 10.0^xrc
        ok= where (x ge xrc(0) and x le xrc(1), n)
        if n eq 0 then begin
          print, '% PLOTWT:  No X-data in the plot range'
          return
        endif
        if n lt nn then begin
          x = x(ok)
          y = y(ok)
          if nw ge nn then begin
            nw = n
            wt = wt(ok)
          endif
          nn = n
        endif
        yrc = !y.crange
        if !y.type eq 1 then yrc = 10.0^yrc
        ok= where (y ge yrc(0) and y le yrc(1), n)
        if n eq 0 then begin
          print, '% PLOTWT:  No Y-data in the plot range'
          return
        endif
        if n lt nn then begin
          x = x(ok)
          y = y(ok)
          if nw ge nn then begin
            nw = n
            wt = wt(ok)
          endif
          nn = n
        endif
      endelse
;
      nelcol =  n_elements (colorin)
      if nelcol eq 1 then if colorin eq 0 then color = !p.color else $
        color =  (nelcol eq 0) ? !p.color : colorin
      if xeng_scale ne 1.0 then x = x*xeng_scale
      if yeng_scale ne 1.0 then y = y*yeng_scale
      if nw lt n then begin
;       print, '%PLOTWT: Weight array must be as large as the x and y arrays'
        if (quiet eq 0 ) then $
        print, '%PLOTWT: Plot arrays with a single color.'
;       Plot using !P.color  or specified color
        loadxyct, /quiet, /freeze
        plots, x, y, psym = psym , symsize = symsize, color = color[0]
        return
      endif
      
      wmin = min (wt, max = wmax)
      loadxyct, /quiet, /freeze
;
      nc10 =!d.table_size-11
      if nc10 lt 8 then begin
        print, '% PLOTWT: less than 20 colors, color set to zero'
        c_range = [!p.color, !p.color]
      endif else if n_elements(c_range_in) gt 1 then begin
        c_range = c_range_in[0:1]
        r_low = min(c_range, max = mm) > 0
        r_high = mm
        if  r_high le 1 then begin
          c_range[0] = 9 + round (r_low*nc10)
          c_range[1] = 9 + round (r_high*nc10)
        endif else begin
          c_range[0] = r_low > 9
          c_range[1] = r_high < !d.table_size -2
        endelse
        if c_range[1]-c_range[0] lt 8 then begin
          Print,'% PLOTWT: less than 8 colors.  C_range increased to 8'
          c_range[0] = (c_range[1]-8) > 9
          c_range[1] = (c_range[0]+8)
        endif
      endif else c_range = [ 9, !d.table_size -2]
      nc_range = c_range[1] - c_range[0]
      nc10 = nc_range

      if wmax eq wmin then begin
        ww = color
        if (quiet eq 0 ) then $
          print, '%PLOTWT: Plot arrays with a single color.'
;       if zrange is set plot in the appropriate color
;       if not set, then use the specified color or normal plot color
        if n_elements(zrange) gt 1 then begin
          thcol = findgen(nc10+1)
          range =float(zrange(1)-zrange(0))
          ww = ((round ((wmax-zrange(0))*(nc10/ range))> 0) < nc10)+c_range[0]
          therm_amp = zrange(0)+ thcol*(range/(nc10-1))
          therm_col = thcol+c_range[0]
        endif else ww = color[0]
      endif  else begin
        if (quiet eq 0 ) then $
        print, ' Range of weights is ', strtrim(wmin,2), ' to ', strtrim(wmax,2)
        if (quiet eq 0 ) && (n_elements(zrange) gt 1) then $
        print, ' Range of colors is ', strtrim(zrange[0],2), ' to ', strtrim(zrange[1],2)
        thcol = findgen(nc10+1)
        if n_elements (zrange) gt 1 then begin
          range =float(zrange(1)-zrange(0))
          ww = round((wt-zrange(0))*(nc10/ range))> 0 
          ww = (ww < nc10)+c_range[0]
          therm_amp = zrange(0)+ thcol*(range/(nc10-1))
        endif else  begin
          range = wmax-wmin
          ww = fix ((wt-wmin)*(nc10/ range) > 0) + c_range[0]
          therm_amp = wmin+ thcol*(range/(nc10-1))
        endelse
        therm_col = thcol+c_range[0]
      endelse
      plots,x,y,psym=psym, symsize=symsize, color=ww

      if keyword_set(thermometer) and n_elements(therm_amp) gt 0 then $
          thermometer, therm_amp, therm_col,_extra = tparam
      end
