; ------------------------------------------------------------------------ 
; $Id: put_array.pro,v 1.9 2010/09/17 20:21:59 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
  Pro put_array, arrayin, new, space, blockin, label = lab, $
           alldata = alldata
;
;+
;   This procedute puts an array into a structure
;   The dimensions of the array must be identical to the existing vector
;      which it replaces.
;   Format: PUT_ARRAY, Structure, Vector, Space [, Block] [, label= lab] 
;                       [, /ALLDATA] 
;   where: Structure - IDL structure name or structure array number
;                      type must be 1, Field
;                                   2, Particle
;                                   3, Grid
;                                   8, Finite Element Time Sliced
;                                   9, Finite Element
;    Note:  Type 8 and 9 data requires that the ALLDATA keyword is set
;           If not set, it will be set silently.
;          Vector- Vector to be inserted into structure.  It must be identical
;                  to the vector it replaces.
;          Space - If greater than zero, indicates spatial dimension desired
;                  If negative, indicates field data or attribute data.
;          Block - Indicates block - not used for particle data
;                  Default: BLOCK = 1
;          Label - Attribute label or block label associated with this data
;                  Default:  lab  = 'Modified value'
;          ALLDATA - Ignored - present for backward compatibility
;                  This parameter is not required. If the number of elements in 
;                  Vector is equal to the number of elements for all blocks, 
;                  ALLDATA will be set automatically. 
;    Examples:
;      Calculate the SIN of the third component of structure 1 for all blocks
;        IM = GET_ARRAY(1, -3, /ALL) 
;        IM = SIN(IM)
;        PUT_ARRAY, 1, IM, -3
;
;      Scale the values in an image in array 2 with polynomial coefficients COEF
;      and store the result in array 3
;        IM = GET_ARRAY(2, -3, /ALL) 
;        IM = POLY(COEF, IM)
;        XFRSTR, 2, 3
;        PUT_ARRAY, 3, IM, -3, LAB= 'Image scaled by fit #2'
;
;-
;          (note in the next version a widget might be used to select array)
      on_error, 1
; common block
@qsstrcom.dat
      if n_params() lt 3 then begin
        if n_params() eq 0 then begin
          print, '% Format: Put_array, Structure, Vector, Space [, Block]'
          print, '          [, LABEL = label] '
        endif else print, $
          '% PUT_ARRAY: Structure, vector, and space variable must be defined'
        return
      endif
      if n_elements(arrayin) gt 0 then array = arrayin else begin
	print,  '% PUT_ARRAY: Structure must be defined'
        return
      endelse
      b = size(array)
      if b(b(0)+1) ne 8 then begin
        a = arrayin
        if a gt 0 and a le maxstructure then begin
          s2i, array, a
          b = size (array)
          if b[b[0]+1] ne 8 then begin
            print, '% PUT_ARRAY: First parameter must be a structure'
            print, '             or a structure array number'
            return
          endif
        endif  else begin
          print, '% PUT_ARRAY: First parameter must be a structure or'
          print, '             a structure array number'
          return
        endelse
      endif else a = 0
      if n_elements (blockin) eq 0 then block = 1 else block = blockin[0]
      if n_elements (lab) eq 0 then lab  = 'Modified value'
      if n_elements (space ) eq 0 then begin
        print, '% PUT_ARRAY: Array component must be defined'
        return
      endif
      nv = n_elements (new)
      if nv eq 0 then begin
        Print, '% PUT_ARRAY: Input vector must be defined'
        return
      endif 
      vector = 0
      if space lt 0 then vector = -space
      valid_types =[1, 2, 3, 8, 9]
      ok = where (array.type eq valid_types, nok)
      if nok eq 0 then begin
        print, '% PUT_ARRAY: Structure type ', strtrim(array.type, 2), $
          ' is not supported'
        print, '             Supported types:', valid_types
        return
      endif
;     Finite element data
      if ((array.type eq 8) || (array.type eq 9)) &&  $
        ( nv ne n_elements(array.x1))  then begin
        print, '% PUT_ARRAY: Only complete ALLDATA arrays may be used with FE data'
        print, '             No change will be made.'
        return
      endif
      if ok[0] ne 1 then begin
;       exclude type 2 structures which are particle datasets
;       that leaves field or grid dataset types
;       
        mblo = array.block
        if block gt mblo or block le 0 then begin
          print,'% PUT_ARRAY: Cannot read block', block, '-- Only', mblo,' blocks'
          return
        endif
        if space gt 0 then begin
          if space gt array.ndim then begin
            print, '% PUT_ARRAY: Structure has only',array.ndim,' spatial dimensions'
            print, '             Array of dimension ',space,' is not possible'
            return
          endif
          if space eq 1 then begin
            if nv eq array.locx(0, block)-array.locx(0, block-1) then begin
              array.x1lab(block-1) = lab
              array.x1(array.locx(0, block-1):array.locx(0, block)-1)= new
              array.range(0, 0, block) = min(new, max = mm)
              array.range(1, 0, block) = mm
              array.range(0,0,0) = min(array.range(0, 0, 1:*))
              array.range(1,0,0) = max(array.range(1, 0, 1:*))
              ok = where(strpos(tag_names(array), 'DX') ge 0 , nok)
              if nok gt 0 then array.dx1 = 0 
            endif else begin
;             see if we can use alldata
              if nv eq n_elements(array.x1) then begin
                if mblo gt 1 then begin
                  for block = 1, mblo do begin
                    xx = new[array.locx(0, block-1):array.locx(0, block)-1, 0]
                    array.range(0, 0, block) = min(xx, max = mm)
                    array.range(1, 0, block) = mm
                  endfor
                endif else begin
                  array.range(0, 0, 1) = min(new, max = mm)
                  array.range(1, 0, 1) = mm
                endelse
                array.x1 = new
                array.x1lab[*] = lab
                array.range(0,0,0) = min(array.range(0, 0, 1:*))
                array.range(1,0,0) = max(array.range(1, 0, 1:*))
                ok = where(strpos(tag_names(array), 'DX') ge 0 , nok)
                if nok gt 0 then array.dx1 = 0
              endif else $
                print, '% PUT_ARRAY: Input vector has a different size; NO CHANGE'
            endelse 
          endif else if space eq 2 then begin
            if nv eq array.locx(1, block)-array.locx(1, block-1) then begin
              array.x2lab(block-1) = lab
              array.x2(array.locx(1, block-1): array.locx(1, block)-1) = new
              array.range(0, 1, block) = min(new, max = mm)
              array.range(1, 1, block) = mm
              array.range(0,1,0) = min(array.range(0, 1, 1:*))
              array.range(1,1,0) = max(array.range(1, 1, 1:*))
              ok = where(strpos(tag_names(array), 'DX') ge 0 , nok)
              if nok gt 0 then array.dx2 = 0 
            endif else  begin
;             see if we can use alldata
              if nv eq n_elements(array.x2) then begin
                if mblo gt 1 then begin
                  for block = 1, mblo do begin
                    xx = new[array.locx(1, block-1):array.locx(1, block)-1, 0]
                    array.range(0, 1, block) = min(xx, max = mm)
                    array.range(1, 1, block) = mm
                  endfor
                endif else begin
                  array.range(0, 1, 1) = min(new, max = mm)
                  array.range(1, 1, 1) = mm
                endelse
                array.x2 = new
                array.x2lab[*] = lab
                array.range(0,1,0) = min(array.range(0, 1, 1:*))
                array.range(1,1,0) = max(array.range(1, 1, 1:*))
                ok = where(strpos(tag_names(array), 'DX') ge 0 , nok)
                if nok gt 0 then array.dx2 = 0 
              endif else print, $
                '% PUT_ARRAY: Input vector has a different size; NO CHANGE'
            endelse
          endif else if space eq 3 then begin
            if nv eq array.locx(2, block)-array.locx(2, block-1) then begin
              array.x3lab(block-1) = lab
              array.x3(array.locx(2, block-1): array.locx(2, block)-1) = new
              array.range(0, 2, block) = min(new, max = mm)
              array.range(1, 2, block) = mm
              array.range(0,2,0) = min(array.range(0, 2, 1:*))
              array.range(1,2,0) = max(array.range(1, 2, 1:*))
              ok = where(strpos(tag_names(array), 'DX') ge 0 , nok)
              if nok gt 0 then array.dx3 = 0
            endif else  begin
;             see if we can use alldata
              if nv eq n_elements(array.x3) then begin
                if mblo gt 1 then begin
                  for block = 1, mblo do begin
                    xx = new[array.locx(2, block-1):array.locx(2, block)-1, 0]
                    array.range(0, 2, block) = min(xx, max = mm)
                    array.range(1, 2, block) = mm
                  endfor
                endif else begin
                  array.range(0, 2, 1) = min(new, max = mm)
                  array.range(1, 2, 1) = mm
                endelse
                array.x3 = new
                array.x3lab[*] = lab
                array.range(0,2,0) = min(array.range(0, 2, 1:*))
                array.range(1,2,0) = max(array.range(1, 2, 1:*))
                ok = where(strpos(tag_names(array), 'DX') ge 0 , nok)
                if nok gt 0 then array.dx3 = 0
              endif else print, $
                '% PUT_ARRAY: Input vector has a different size; NO CHANGE'
            endelse
          endif else if space eq 4 then begin
            if nv eq array.locx(3, block)-array.locx(3, block-1) then begin
              array.x4lab(block-1)= lab
              array.x4(array.locx(3, block-1): array.locx(3, block)-1) = new
              array.range(0, 3, block) = min(new, max = mm)
              array.range(1, 3, block) = mm
              array.range(0,3,0) = min(array.range(0, 3, 1:*))
              array.range(1,3,0) = max(array.range(1, 3, 1:*))
              ok = where(strpos(tag_names(array), 'DX') ge 0 , nok)
              if nok gt 0 then array.dx4 = 0
            endif else   begin
;             see if we can use alldata
              if nv eq n_elements(array.x4) then begin
                if mblo gt 1 then begin
                  for block = 1, mblo do begin
                    xx = new[array.locx(3, block-1):array.locx(3, block)-1, 0]
                    array.range(0, 3, block) = min(xx, max = mm)
                    array.range(1, 3, block) = mm
                  endfor
                endif else begin
                  array.range(0, 3, 1) = min(new, max = mm)
                  array.range(1, 3, 1) = mm
                endelse
                array.x4 = new
                array.x4lab[*] = lab
                array.range(0,3,0) = min(array.range(0, 3, 1:*))
                array.range(1,3,0) = max(array.range(1, 3, 1:*))
                ok = where(strpos(tag_names(array), 'DX') ge 0 , nok)
                if nok gt 0 then array.dx4 = 0
              endif else print, $
                '% PUT_ARRAY: Input vector has a different size; NO CHANGE'
            endelse
          endif
          if a ne 0 then i2s, array, a, /no_copy  else arrayin = array
          return
        endif else begin
          if array.type eq 3 then begin
            print, $
        '% PUT_ARRAY: Error in PUT_ARRAY-attempt to store a vector into GRID data'
            return
          endif
          if array.nvect eq 0 then begin
            print, '% PUT_ARRAY: No vectors available in this structure'
            return
          endif
          if vector lt 1 or vector gt array.nvect then begin
            print, '% PUT_ARRAY: Vector dimension must be less than', array.nvect
            return
          endif
          if (nv ne (array.locv(block)-array.locv(block-1))) then begin
            if nv eq n_elements (array.v1) then alldata =1 else begin
              print, '% PUT_ARRAY: Input vector has a different size; NO CHANGE'
              return
            endelse
          endif else alldata =0
          if vector eq 1 then begin
              if keyword_set(alldata) then begin
                array.v1 = new 
                array.v1lab(*)= lab
              endif else begin
                array.v1lab(block-1)= lab
                array.v1(array.locv(block-1): array.locv(block)-1) = new
              endelse
          endif else if vector eq 2 then begin
              if keyword_set(alldata) then begin
                array.v2 = new 
                array.v2lab(*)= lab
              endif else begin
                array.v2lab(block-1)= lab
                array.v2(array.locv(block-1): array.locv(block)-1) = new
              endelse
          endif else if vector  eq 3 then begin
              if keyword_set(alldata) then begin
                array.v3 = new 
                array.v3lab(*)= lab
              endif else begin
                array.v3lab(block-1)= lab
                array.v3(array.locv(block-1): array.locv(block)-1) = new
              endelse
          endif else if vector eq 4 then begin
              if keyword_set(alldata) then begin
                array.v4 = new 
                array.v4lab(*)= lab
              endif else begin
                array.v4lab(block-1)= lab
                array.v4(array.locv(block-1): array.locv(block)-1) = new
              endelse
          endif else if vector  eq 5 then begin
              if keyword_set(alldata) then begin
                array.v5 = new 
                array.v5lab(*)= lab
              endif else begin
                array.v5lab(block-1)= lab
                array.v5(array.locv(block-1): array.locv(block)-1) = new
              endelse
          endif else if vector eq 6 then begin
              if keyword_set(alldata) then begin
                array.v6 = new 
                array.v6lab(*)= lab
              endif else begin
                array.v6lab(block-1)= lab
                array.v6(array.locv(block-1): array.locv(block)-1) = new
              endelse
          endif
          if a ne 0 then i2s, array, a ,/no_copy else arrayin = array
          return
        endelse
      endif else if array.type eq 2 then begin
;       particle data
        alldata = 1
        if space gt 0 then begin
          if space gt array.ndim then begin
            print, '% PUT_ARRAY: Structure has only',array.ndim,' spatial dimensions'
            print,             ' Array of dimension ',space,' is not possible'
            return
          endif
          if nv ne array.nvert then begin
            print, '% PUT_ARRAY: Input vector has a different size; NO CHANGE'
            return
          endif
          if space eq 1 then begin
            array.x1lab = lab
            array.x1 = new
          endif else if space eq 2 then begin
            array.x2lab = lab
            array.x2 = new
          endif else if space eq 3 then begin
            array.x3lab = lab
            array.x3 = new
          endif else if space eq 4 then begin
            array.x4lab = lab
            array.x4 = new
          endif
          if a ne 0 then i2s, array, a, /no_copy else arrayin = array
          return
        endif else begin
          if array.natt eq 0 then begin
            print, '% PUT_ARRAY: No attributes available in this structure'
            return
          endif
          if vector lt 1 or vector gt array.natt then begin
            print, '% PUT_ARRAY: Attribute dimension must be less than', array.natt
            return
          endif
          if nv ne array.nvert then begin
            print, '% PUT_ARRAY: Input vector has a different size; NO CHANGE'
            return
          endif
          if vector eq 1 then begin
            array.v1lab = lab
            array.v1 = new
          endif else if vector eq 2 then begin
            array.v2lab = lab
            array.v2 = new
          endif else if vector eq 3 then begin
            array.v3lab = lab
            array.v3 = new
          endif else if vector eq 4 then begin
            array.v4lab = lab
            array.v4 = new
          endif else if vector eq 5 then begin
            array.v5lab = lab
            array.v5 = new
          endif else if vector eq 6 then begin
            array.v6lab = lab
            array.v6 = new
          endif else if vector eq 7 then begin
            array.v7lab = lab
            array.v7 = new
          endif else if vector eq 8 then begin
            array.v8lab = lab
            array.v8 = new
          endif
          if a ne 0 then i2s, array, a,/no_copy  else arrayin = array
          return
        endelse
      endif
      end
