; ------------------------------------------------------------------------ 
; $Id: reademph.pro,v 1.4 2007/06/06 20:09:36 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
 Pro reademph, strin, filenamein, filename = fnamein, processors = nproc, $
              ghost = ghost, blocks = bloc
;
;+
;
;  Reads a Emphasis GRID dataset and conductor information into IDL structures.
;
;  Format:  READEMPH [, STRUCTURE] [, FILENAME] [, FILENAME = filename]
;                    [, PROCESSOR = processor] [, /GHOST] [BLOCKS = blocks]
;
;  where:
;    STRUCTURE - QS structure array number or an array of 3 numbers
;    Note: If the parmeter FILENAME is a File ID then Structure must be provided
;                Default = [1, 2, 3, 4]
;                If one number is give, then 
;                Default = structure[0] + indgen(3)
;                STRUCTURE[0] = Grid structure with the full grid values
;                               Blocks are the original problem blocks
;                STRUCTURE[1] = Field structure with the half grid and material
;                               numbers for the field values
;                STRUCTURE[2] = Grid structure with the full grid values
;                               in each decomposed blocks. 
;                               If a serial run, then STRUCTURE[2] is not saved.
;                STRUCTURE[3] = Grid structure with the half-grid values
;                               in each decomposed blocks and the material 
;                               number for the field values
;                               If a serial run, then STRUCTURE[3] is not saved.
;    FILENAME  - Name of file containing the Emphasis/Nevada grid information
;        NOTE:  Structure must be provided is FILENAME is a PFF File ID.
;                FILENAME may be the second parameter or a keywork.
;                If FILENAME parameter is missing or zero, the keyword 
;                  FILENAME is checked.
;                If FILENAME is still not defined, DIALOG_PICKFILE is called.
;                If FILENAME is a number, not equal to zero, then FILENAME is
;                  the file ID of the desired PFF file.  If it equals zero, then
;                  DIALOG_PICKFILE is used to determine the file. 
;                If the file id is specified then the dataset pointer should 
;                  be at a IFL dataset type with a comment "Buffer Planes"
;                  The routine will look for a dataset with this comment, and
;                  continue if only one is found.
;    PROCESSORS - Returns the number of processors
;    BLOCKS    - Returns the processor for each decomposed block
;                BLOCKS = 0 for a single processor problem
;    GHOST     - If set, ghost cells will be included.
;                If GHOST is a vector then the first element applies to 
;                  STR[0] and the second to the material structures, 
;                  STR[1] and STR[2]
;
;  Structure Labels
;    type  - 3
;    ndim  - number of spatial dimensions = 3
;    block - number of blocks 
;    size  - array of the size of the spatial dimensions
;            size = size(ndim, block)
;    two   - two dimensional missing dimension
;            this is 0 for 3d data, 1 for yz data, 2 for zx data, 3 for xy data
;    range - (2, ndim, nblock+1) two dimensional array of the max and min for
;            each block.  Block (0) has overall values.
;    locx  - array of the location of the first point for each block
;            spatial array- locx = locx(ndim, block+1)
;            (first spatial dimension for block m is x and is defined by
;             x = x1(locx(0, m-1): locx(0, m)-1) )
;    x1, x2, x3 - spatial arrays
;    proc  - Processor number if available on parallel jobs
;    x1lab, x2lab, x3lab - spatial labels
;    tlab  - type label
;    clab  - dataset comment
;    PROC  - Block Processor number
;    file  - file data read from
;    spare - integer array (5)
;
;  Examples:
;    Read Emphasis/Nevada grids from 'My_data.pff', into arrays 1,2,3
;      READEMPH, 'My_data.pff'
;    Read Emphasis/Nevada grids from 'your_data.pff', into arrays 1,34 ,300
;      READEMPH, [1, 34, 300], 'My_data.pff'
;    
;-
;
      on_error, 1
      nparams =  n_params()
;      if (nparams lt 1 ) then begin
;  Format:  READEMPH [, Structure] [, FILENAME] [, FILENAME = filename]
;                    [, PROCESSOR = processor] [, /GHOST] [BLOCKS = blocks]

;        print,'% Format: READEMPH [, Structure] [, FILENAME] [, /GHOST]'
;        print,'          [, PROCESSOR = processor] [BLOCKS = blocks]
;        return
;      endif
      nproc = 1L
      bloc = 0L
      PID = 0L
      IER = 0L
      str =0
      if nparams eq 1 then begin
        if size(/type, strin) eq 7 then begin
          filenamein = strin
          str= [1, 2, 3, 4]
        endif 
      endif 
      if n_elements(str) lt 4 then begin
        if n_elements(strin) eq 0 then strin = 1
        if n_elements(strin) eq 1 then str = strin+lindgen(4) else $
          str = strin
        nst= n_elements(str)
        if nst lt 4 then str= [str, str[nst-1]+1+indgen(4-nst)] else $
          str = str[0:3]
        if max(str) gt get_maxstr() then set_maxstrarray, max(str)+16
      endif
      if n_elements (filenamein) eq 0 then begin
        if n_elements(fnamein) eq 0 then fname = 0L else fname = fnamein[0]
      endif else fname = filenamein[0]
      b = size(fname)
      if b(1) eq 7 then begin
        lcfile = 1
        vers = strupcase(strmid(!version.os_family, 0, 3))
        if (vers eq 'UNI') then begin
;       get system variable substitution for unix systems
          temp = fname
          FULFNAM, temp, fname
        endif
        fname = file_expand_path(fname)
        if !version.os eq 'linux' then resolve_it, fname
        info = file_info(fname)
        if (info.exists + info.read) ne 2b then begin
           print, '% REASC: File does not exist with read permission'
           print, '         File Name = ', fname
           print, '         File Exists =', info.exists
           print, '         Read Permission =', info.read
           ier = -2
           return
        endif
        openpff, fname, pid, error = ier
      endif else begin
        pid = round(fname)
        if pid eq 0 then begin
          fname = dialog_pickfile(/must_exist, /read)
          if strlen(fname) lt 1 then begin
            print, ' No File was Selected - RETURNING'
            return
          endif
          lcfile = 1
          if !version.os eq 'linux' then resolve_it, fname
          openpff, fname, pid, error = ier
        endif else lcfile = 0
      endelse
      if ier ne 0 then begin
        print, '% READEMPH: Unable to open the file'
        print, '  File: ', fname
        return
      endif
      setpff, pid, ier
      if ier ne 0 then begin
        print, '% READEMPH: Unable to set file to file id: ', strtrim(pid, 2)
        print, '            File ID: ', strtrim(pid, 2)
        print, '            Use SHOWPFF to see valid file ids'
        if lcfile eq 1 then closepff
        return
      endif
      dataset = 0
      readhdr,dataset,ier,fileid=pid,aptype=aptyp, pfftyp = pfftyp, $
        dlength = dlength, clabel=clabel, namefile=file, tlabel = tlabel, $
        iextra=iextra, /quiet
      if ier ne 0 then begin
        print,'% READEMPH: Error reading PFF header'
        return
      endif

      if strpos(clabel, 'Buffer planes') lt 0 or pfftyp ne 6 then begin
        get_dir, com = com
        ok = where (strpos(com, 'Buffer planes') ge 0, nok)
        if nok ne 1 then begin
          print,  '% READEMPH: Unable to determine first dataset'
          print, nok, ' Datasets with the string "Buffer planes"'
          print,  '   Use SETDIRPFF to set the dataset pointer '
          print,  '     to the desired dataset with "Buffer planes"'
          return
        endif
        setdirpff, ok[0]+1
      endif
;     Get started - Read the Ghost Layer information
      dataset =0 
      readhdr,dataset,ier,fileid=pid,aptype=aptyp, pfftyp = pfftyp, $
        dlength = dlength, clabel=clabel, namefile=file, tlabel = tlabel, $
        iextra=iextra, /quiet
      if ier ne 0 then begin
        print,'% READEMPH: Error reading PFF header'
        return
      endif
      if strpos(clabel, 'Buffer planes') lt 0 or pfftyp ne 6 then begin
        print, '% READEMPH: Error determining the Buffer planes dataset'
        return
      endif
      readifl, iarr, flist, farr
      n = n_elements(iarr)
      if n eq 8 then begin
        nproc = iarr[6]
        nblo = iarr[7]
      endif else nproc = 1L
      nghost = iarr[0:5]
;
;     Read the Full grid 
;
      readhdr, 0,ier,fileid=pid,aptype=aptyp, pfftyp = pfftyp, $
        dlength = dlength, clabel=clabel, namefile=file, tlabel = tlabel, $
        iextra=iextra, /quiet
      if ier ne 0 then begin
        print,'% READEMPH: Error reading full grid pff header'
        return
      endif
      if strpos(clabel, 'QUICKSILVER Grid') lt 0 or pfftyp ne 8 then begin
        print, '% READEMPH: Error determining the full grid dataset'
        return
      endif
;     Read the full grid
      readstr, str[0]
;     check to see if we have read successfully
      if strvalid(str[0], /valid) lt 0 then begin
        print, '% READEMPH: Error reading the full grid'
        return 
      endif
      if n_elements(ghost) eq 0 then ghost = 0
      if n_elements(ghost) eq 1 then begin
        ghost_full = ghost[0]
        ghost_mat = ghost_full
      endif else begin
        ghost_full = ghost[0]
        ghost_mat = ghost[1]
      endelse

      if not keyword_set(ghost_full) then remove_ghost_cells, str[0], nghost
;
;     Read the half-grid coordinates
;
      readhdr, 0,ier,fileid=pid,aptype=aptyp, pfftyp = pfftyp, $
        dlength = dlength, clabel=clabel, namefile=file, tlabel = tlabel, $
        iextra=iextra, /quiet
      if ier ne 0 then begin
        print,'% READEMPH: Error reading full grid pff header'
        return
      endif
      if nproc  eq 1 then begin
        if strpos(clabel, 'Material matrix') lt 0 or pfftyp ne 9 then begin
          print, '% READEMPH: Error determining the full grid dataset'
          return
        endif
        readstr, str[1]
;       check to see if we have read successfully
        if strvalid(str[1], /valid) lt 0 then $
          print, '% READEMPH: Error reading the half-grid grid and materials'
        bloc =0L
        if not keyword_set(ghost_mat) then remove_ghost_cells, str[1], nghost
        if lcfile eq 1 then closepff
        return
      endif
;
;     Finished with single processor read
;     Remainder is multi-processor read
;
      if strpos(clabel, 'QUICKSILVER Dual Grid') lt 0 or pfftyp ne 8 then begin
        print, '% READEMPH: Error determining the half-grid dataset'
        return
      endif
;     Read the half-grid
      readstr, str[1]
;     check to see if we have read successfully
      if strvalid(str[1], /valid) lt 0 then begin
        print, '% READEMPH: Error reading the half grid'
        return 
      endif
      
      if not keyword_set(ghost_mat) then remove_ghost_cells, str[1], nghost
;
;     add a vector to the grid array
;
      s2i, s, str[1]
      nbl = s.block
      slocv = make_array(nbl+1, /long)
      clab = 'Material Matrix'
      for i = 0, nbl-1 do $
        slocv[i+1]=slocv[i]+ s.size[0, i]*s.size[1, i]*s.size[2, i]         
      sv = make_array(slocv[nbl], val=-1, /int)
      v1lab = make_array(nbl, value=clab, /string)
      if nbl gt 1 then begin
        order = make_array(3, nbl, /long)
        for i = 0, 2 do order[i, *]=sort(s.range[0, i, 1:nbl])
      endif else order = 1L
      snew = { type:1L, ndim:s.ndim, nvect: 1L, block :nbl, two: s.two, $
              slice: s.slice, size: s.size, range: s.range, $
              order: order, locx: s.locx, $
              locv: slocv, x1:s.x1, x2:s.x2, x3:s.x3, x4:0.0, $
              v1: sv, v2:0.0, v3:0.0, v4:0.0, v5 :0.0, v6:0.0, $
              x1lab : s.x1lab, x2lab : s.x2lab, $
              x3lab : s.x3lab, x4lab : '' ,v1lab : v1lab, v2lab : ' ', $
              v3lab :  '', v4lab : '',v5lab : '', v6lab : '', $
              file : s.file, spare : s.spare, tlab : 'Dual Grid', $
              clab: clab }
      i2s, snew, str[1], /no_copy
;
;     read the processor information
;
      readhdr, 0,ier,fileid=pid,aptype=aptyp, pfftyp = pfftyp, $
        dlength = dlength, clabel=clabel, namefile=file, tlabel = tlabel, $
        iextra=iextra, /quiet
      if ier ne 0 then begin
        print,'% READEMPH: Error reading block decomposition header'
        return
      endif
      if strpos(clabel, 'Global Block Mapping') lt 0 or pfftyp ne 6 then begin
        print, '% READEMPH: Error determining global block decomposition dataset'
        return
      endif
      readifl, decomp, flist, farr
      if n_elements(decomp) ne nblo*8L then begin
        print, '% READEMPH: Wrong number of elements in the decomposition'
        print, '            (8 elements per block)'
        help, nblo, decomp
        return
      endif
;
;     make arrays for the material information in each processor block
;
      ssiz = make_array(3, nblo, /long)
      range = make_array(2, 3, nblo+1)
      locx = make_array(3, nblo+1, /long, value =0L)
      locv = make_array(nblo+1, /long, value =0L)
;     processor for each block
      ii = 8*indgen(nblo)

      bloc = decomp[ii]
      process = bloc

;     PFIDL blocks begin with 1
      ublock = decomp[ii+1]+1

      ilow = decomp[ii+2]
      jlow = decomp[ii+3]
      klow = decomp[ii+4]
      ihi = decomp[ii+5]
      jhi = decomp[ii+6]
      khi = decomp[ii+7]
; sum of the ghost cells
      sxg = nghost[0]+nghost[1]
      syg = nghost[2]+nghost[3]
      szg = nghost[4]+nghost[5]

      imx = ihi
      jmx = jhi
      kmx = khi
      if keyword_set (ghost) then begin
        imx = imx-ilow+sxg
        jmx = jmx-jlow+syg
        kmx = kmx-klow+szg
      endif else begin
        imx = imx-ilow
        jmx = jmx-jlow
        kmx = kmx-klow
      endelse
      ssiz = reform ([imx+1, jmx+1, kmx+1], nblo, 3)
      ssiz = transpose(ssiz)
      nx = (imx+1L)*(jmx+1L)*(kmx+1L)
      xx = make_array(total(ssiz[0, *]))
      yy = make_array(total(ssiz[1, *]))
      zz = make_array(total(ssiz[2, *]))
      locx[0, 1:nblo] = total(ssiz[0, *], /cum)
      locx[1, 1:nblo] = total(ssiz[1, *], /cum)
      locx[2, 1:nblo] = total(ssiz[2, *], /cum)

      ilow = ilow -  nghost[0]
      jlow = jlow -  nghost[2]
      klow = klow -  nghost[4]
      if keyword_set(ghost) then begin
        ihi = ihi+nghost[1]
        jhi = jhi+nghost[3]
        khi = khi+nghost[5]
      endif else begin
        ihi = ihi-nghost[0]
        jhi = jhi-nghost[2]
        khi = khi-nghost[4]
      endelse

      s2i, sss, str[0]
      for i = 0, nblo-1 do begin
        x = (get_array(str[0], 1, ublock[i]))[ilow[i]:ihi[i]]
        range[0, 0, i+1] = min(x, max = mm)
        range[1, 0, i+1] = mm
        xx[locx[0, i]:locx[0, i+1]-1] = x
        y = (get_array(str[0], 2, ublock[i]))[jlow[i]:jhi[i]]
        range[0, 1, i+1] = min(y, max = mm)
        range[1, 1, i+1] = mm
        yy[locx[1, i]:locx[1, i+1]-1] = y
        z = (get_array(str[0], 3, ublock[i]))[klow[i]:khi[i]]
        range[0, 2, i+1] = min(z, max = mm)
        range[1, 2, i+1] = mm
        zz[locx[2, i]:locx[2, i+1]-1] = z
      endfor
      x1lab = make_array(nblo, value= sss.x1lab[0])
      x2lab = make_array(nblo, value= sss.x2lab[0])
      x3lab = make_array(nblo, value= sss.x3lab[0])
      order = make_array(3, nblo, /long)
      for i = 0, 2 do order[i, *]=sort(range[0, i, 1:nblo])
      clabel = sss.clab+' - Decomposed blocks'
      sss= {type:3L, ndim:3L,  block :nblo, two:sss.two, range:range, $
              size: ssiz, locx : locx, slice: 0.0,  $
              x1:xx, x2:yy, x3:zz, x1lab : x1lab, x2lab :x2lab, $
              x3lab : x3lab, proc:bloc , file : sss.file, $
              spare :sss.spare, tlab : sss.tlab, clab: clabel }
      i2s, sss, str[2]

      ilow = decomp[ii+2]
      jlow = decomp[ii+3]
      klow = decomp[ii+4]
      ihi = decomp[ii+5]-1
      jhi = decomp[ii+6]-1
      khi = decomp[ii+7]-1
; sum of the ghost cells
      sxg = nghost[0]+nghost[1]
      syg = nghost[2]+nghost[3]
      szg = nghost[4]+nghost[5]
      sxgp = sxg+1
      sygp = syg+1
      szgp = szg+1

      imx = ihi
      jmx = jhi
      kmx = khi
      if keyword_set (ghost) then begin
        imx = imx-ilow+sxg
        jmx = jmx-jlow+syg
        kmx = kmx-klow+szg
      endif else begin
        imx = imx-ilow
        jmx = jmx-jlow
        kmx = kmx-klow
      endelse
      ssiz = reform ([imx+1, jmx+1, kmx+1], nblo, 3)
      ssiz = transpose(ssiz)
      nx = (imx+1L)*(jmx+1L)*(kmx+1L)
      xx = make_array(total(ssiz[0, *]))
      yy = make_array(total(ssiz[1, *]))
      zz = make_array(total(ssiz[2, *]))
      vv = make_array(total(nx))
      locv[1:nblo] = total(nx, /cum)
      locx[0, 1:nblo] = total(ssiz[0, *], /cum)
      locx[1, 1:nblo] = total(ssiz[1, *], /cum)
      locx[2, 1:nblo] = total(ssiz[2, *], /cum)

      ilow = ilow -  nghost[0]
      jlow = jlow -  nghost[2]
      klow = klow -  nghost[4]
      if keyword_set(ghost) then begin
        ihi = ihi+nghost[1]
        jhi = jhi+nghost[3]
        khi = khi+nghost[5]
      endif else begin
        ihi = ihi-nghost[0]
        jhi = jhi-nghost[2]
        khi = khi-nghost[4]
      endelse
      if keyword_set (ghost) then stot = ssiz else begin
        stot = reform ([imx+sxgp, jmx+sygp, kmx+szgp], nblo, 3)
        stot = transpose(stot)
        stopp =  reform ([imx+(sxg-nghost[1]), jmx+(syg-nghost[3]), $
             kmx+(szg-nghost[5])], nblo, 3)
        stopp = transpose(stopp)
      endelse
      if keyword_set(ghost) then ntot = nx else $
         ntot =(imx+sxgp)*(jmx+sygp)*(kmx+szgp)

      for i = 0, nblo-1 do begin
        readhdr, 0,ier,fileid=pid,aptype=aptyp, pfftyp = pfftyp, $
          dlength = dlength, clabel=clabel, namefile=file, tlabel = tlabel, $
          iextra=iextra, /quiet 
        if ier ne 0 then begin
          print,'% READEMPH: Error reading material matrix header', i
          return
        endif
        if strpos(clabel, 'Material Matrix') lt 0 or pfftyp ne 6 then begin
          print, '% READEMPH: Error determining material matrix dataset', i
          return
        endif

        readifl, mat, flist, farr
        if n_elements(mat) ne ntot[i] then begin
          print, '% READEMPH: Wrong number of elements in the material matrix'
          help, ntot[i], mat,  i
          return
        endif

        x = (get_array(str[1], 1, ublock[i]))[ilow[i]:ihi[i]]
        range[0, 0, i+1] = min(x, max = mm)
        range[1, 0, i+1] = mm
        xx[locx[0, i]:locx[0, i+1]-1] = x
        y = (get_array(str[1], 2, ublock[i]))[jlow[i]:jhi[i]]
        range[0, 1, i+1] = min(y, max = mm)
        range[1, 1, i+1] = mm
        yy[locx[1, i]:locx[1, i+1]-1] = y
        z = (get_array(str[1], 3, ublock[i]))[klow[i]:khi[i]]
        range[0, 2, i+1] = min(z, max = mm)
        range[1, 2, i+1] = mm
        zz[locx[2, i]:locx[2, i+1]-1] = z
        v = get_array(str[1], -1, ublock[i])
        mat = reform(mat, stot[0, i], stot[1, i], stot[2, i], /overwrite)
        if not keyword_set(ghost) then $
          mat=mat[nghost[0]:stopp[0,i],nghost[2]:stopp[1,i],nghost[4]:stopp[2,i]]
        vv[locv[i]:locv[i+1]-1] = mat
        v[ilow[i]:ihi[i], jlow[i]:jhi[i], klow[i]:khi[i]] = mat
        put_array, str[1], v, -1, ublock[i]
;     
      endfor
      if lcfile eq 1 then closepff
      range [0, 0, 0]= min(range[0, 0, 1:*])
      range [1, 0, 0]= max(range[1, 0, 1:*])
      range [0, 1, 0]= min(range[0, 1, 1:*])
      range [1, 1, 0]= max(range[1, 1, 1:*])
      range [0, 2, 0]= min(range[0, 2, 1:*])
      range [1, 2, 0]= max(range[1, 2, 1:*])
      x1lab = make_array(nblo, value= s.x1lab[0])
      x2lab = make_array(nblo, value= s.x2lab[0])
      x3lab = make_array(nblo, value= s.x3lab[0])
      order = make_array(3, nblo, /long)
      for i = 0, 2 do order[i, *]=sort(range[0, i, 1:nblo])
      clab = 'Material Matrix - Processor blocks'
      v1lab = make_array(nblo, value=clab, /string)
      snew = { type:1L, ndim:3L, nvect: 1L, block :nblo, two: 0, $
              slice: 0.0, size: ssiz, range: range, $
              order: order, locx: locx, $
              locv: locv, x1:xx, x2:yy, x3:zz, x4:0.0, $
              v1: vv, v2:0.0, v3:0.0, v4:0.0, v5 :0.0, v6:0.0, $
              x1lab : x1lab, x2lab : x2lab, $
              x3lab : x3lab, x4lab : '', v1lab : v1lab, v2lab : '', $
              v3lab :  '', v4lab : '',v5lab : '', v6lab : '', $
              file : s.file, spare : s.spare, tlab : 'Dual Grid', $
              clab: clab }
      ss = (n_elements(str) lt 4) ? str[2]+1 : str[3]
      i2s, snew, ss, /no_copy
      s=0
      end
