; ------------------------------------------------------------------------
; $Id: rm_line.pro,v 1.3 2007/06/06 20:09:36 lpmix Exp $
; ------------------------------------------------------------------------
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
;+
; NAME:
;       RM_LINE
;
; PURPOSE:
;       Remove a thin verticle line from an image by replacing it with the 
;       values in neighboring pixels or by a user specified value.
;
; CATEGORY:
;       Image analysis
;
; CALLING SEQUENCE:
;
;       RM_LINE, STRUCTURE_in [, STRUCTURE_out]
;
; INPUTS:
;       STRUCTURE_in:  Input structure with a vertical line to be removed
;          Note:  A copy of STRUCTURE_in is in get_maxstr()
;          Note:  This must be a uniform type 1 structure
;
; OPTIONAL INPUTS:
;       STRUCTURE_out: Output structure with the vertical line removed
;                      If STRUCTURE_out is missing, then output is stored
;                      in STRUCTURE_in
;          Note:  A copy of STRUCTURE_out is in get_maxstr()-1
;
; KEYWORD PARAMETERS:
;      LINEWIDTH: Characteristic width of the line in pixels.
;                 Default: LINEWIDTH = 9 pixels
;      PLOTIT:  If set, edited image will be plotted.
;
;      XCUR:    X-Position of point on the line to be removed.
;      YCUR:    Y-Position of point on the line to be removed.
;         Note: XCUR and YCUR are always returned with the values used.
;               If no specified on input, ZOOMS is used to determine their value
;      PEAK_RATIO: Ratio maximum to minimum value of data that indicates a valid
;                  peak 
;                  Default: PEAK_RATIO = 1.5
;      DERIVATIVE: Ratio of peak derivative to tail value.
;                  Values of the derivative below this are considered background.
;                  Default: DERIVATIVE  = 0.14  (Minimum value = 0.05)
;
; OUTPUTS:
;       Modified image is returned in STURCTURE_out or STRUCTURE_in
;       A copy is stored in get_maxstr()-1 
;
; SIDE EFFECTS:
;	Two maximum values of the structure arrays are modified
;
; RESTRICTIONS:
;	Input structure must be a valid type 1 structure with values of
;       DX1 and DX2.
;
; PROCEDURE:
;       Input image is saved and then smoothed with a width of LINEWIDTH
;       The derivative of the smoothed image in the X-direction is taken
;          and smoothed with a width of 0.5* LINEWIDTH+1
;     The remainder of the operations are performed on each row 
;       The center of the line is determined from the smoothed image
;       The maximum (MXD) and minimum (MND) derivatives are determined.
;       Regions on either side are determined where the derivative is less 
;          than DERIVATIVE*MXD and -DERIVATIVE*MND.
;       The average values of the original image is linearly propagated 
;          across the line
;
; EXAMPLE:
;       Eliminate a line from structure 3 and save in structure 4 and plot.
;         rm_line, 3, 4, /plot
;
;       Eliminate several lines from structure 3 and see the results after 
;       each line is removed.  
;         rm_line, 3, 3, /plotit
;         NOTE: Use the up arrow to remove another line.
;               Use "xfrstr, get_maxstr(), 3" to undo the removal.
;
; MODIFICATION HISTORY:
; 	Written by:	L. P. Mix, Jr., November 1, 2006
;-
;      THRESHOLD:  Threshold value when the slope is considered to be essentially
;                  flat at zero. Default = wsize+2

 pro rm_line, strin, strout, xcur = xcur, ycur = ycur, plotit = plotit, $
             linesize = wsize, peak_ratio= peak_ratio, verbose = verbose, $
             threshold = threshold, derivative = derivative
 on_error, 1

 npar = N_params()
 if npar eq 0 then begin
   print, '% Format: RM_LINE, STRUCTURE_in [, STRUCTURE_out] [, /PLOTIT]'
   print, '          [, LINEWIDTH = linesize] [, XCUR = xcur] [, YCUR = ycur]'
   print, '          [, PEAK_RATIO = peak_ratio] [, DERIVATIVE = derivative]'
   print, '          [, THRESHOLD = threshold] [, SHADOW = shadow]'
   return
 endif 
 if n_elements(strin) eq 0 then begin
   print, '% RM_WIRE: Input structure must be defined'
   return
 endif 
 str = strvalid(strin[0], /valid, npoint= nst)
 if nst ne 1 then begin
   print, '% RM_LINE: Valid input structure must be provided.'
   return
 endif
 if n_elements(xcur) eq 0 || n_elements(ycur) eq 0 then begin
   zooms, str, xcur = xcur, ycur = ycur, mag = 30, npoint= npoint
   if npoint eq 0 then return
   xcur = total(xcur)/n_elements(xcur)
   ycur = total(ycur)/n_elements(ycur)
 endif

 if n_elements(wsize) eq 0 then wsize = 9
 wsize = round (wsize)
 if n_elements(peak_ratio) eq 0 then peak_ratio = 1.5 
 if n_elements(derivative) eq 0 then derivative = 0.14
 ratio = derivative > 0.05
 orig = get_maxstr()
 xfrstr, str, orig
 xfrstr, str, orig-1
 
 origarray= reform(get_array(orig, -1))
 newarray= origarray
 smoarray = smooth(origarray, wsize)
 xar = reform(get_array(orig, 1))
 yar = reform(get_array(orig, 2))
 junk = min(abs(xar-xcur), ist)
 junk = min(abs(yar-ycur), jst)
 strinfo, orig, dx = dx, dy = dy
 nx = n_elements(xar)
 ny = n_elements(yar)
 deriv = make_array(nx-1, ny)
 for i = 0, ny-1 do deriv[*, i] = smoarray[1:*, i]-smoarray[0:nx-2, i]
 deriv = smooth(deriv, wsize/2+1)
 if n_elements(threshold) eq 0 then threshold = wsize+2
;
; minimum number of zero values
 nmin = round(2 *wsize)
 initial_j = jst
 initial_i = ist
 wsize2 = round(wsize*1.8) ; fudge used to look for actual peak on each row.
 wsizeb2 = wsize*0.4
 ia = (ist- wsize2) > 0
 ib = (ist+ wsize2) < (nx-1)
; determine if the line is in emission or absorbtion
 peak = min( smoarray[ia:ib, jst], ismin)
 peak = max( smoarray[ia:ib, jst], ismax)
 dis = abs([ismax, ismin] +(ia -ist))
 junk = min(dis, shadow)
 min_sep = 0.6
 help, shadow
 
 for iloop = 0, 1 do begin
   if iloop eq 0 then begin
     iii = initial_j
     jjj = ny -1
     kkk = 1
   endif else begin
     iii = initial_j-1
     jjj = 0
     kkk = -1
     ist = initial_i
   endelse
 for jst = iii, jjj, kkk do begin 
;  go up first
;  find the peak  
   ia = (ist- wsize2) > 0
   ib = (ist+ wsize2) < (nx-1)
   if keyword_set(shadow) then $
     peak = min( smoarray[ia:ib, jst], istt, max= m) $
   else peak = max( smoarray[ia:ib, jst], istt, min= m)

;  See if we really  have a peak
   if istt lt wsizeb2 || istt gt (ib-wsizeb2) then begin
     if keyword_set(verbose) then print, wsizeb2, istt, ist, jst
     goto, skip
   endif
   istt = istt +ia
   iad = (istt - wsize2) > 0
   ibd = (istt + wsize2) < (nx-2)
   dmax = max(deriv[ia:ib, jst],xdmax) 
   dmin = min(deriv[ia:ib, jst],xdmin) 
   dave = (dmax-dmin)*0.5
   if dmin*dmax ge 0.0 then begin
     if keyword_set(verbose) then print, dmax,dmin, ist, jst
     goto, skip
   endif
;  if (abs(dmax+dmin)/dave gt diff_deriv) then begin
;    if keyword_set(verbose) then print, abs(dmax+dmin)/dave, dmax, dmin, jst
;    goto, skip
;  endif
;  if dmax lt min_deriv ||  dmin gt -min_deriv then begin
;    if keyword_set(verbose) then print, dmin, dmax, jst
;    goto, skip
;  endif
   if keyword_set(shadow) then begin
;     if ((xdmax -xdmin) lt min_sep *wsize) || ((xdmax-xdmin) gt max_sep*wsize) then begin
     if ((xdmax -xdmin) lt min_sep *wsize) then begin
       if keyword_set(verbose) then print,xdmax-xdmin,min_sep*wsize,max_sep*wsize,ist,jst 
       goto, skip
     endif
   endif else begin
;    if ((xdmin -xdmax) lt min_sep *wsize) || ((xdmin-xdmax) gt max_sep*wsize) then begin
     if ((xdmin -xdmax) lt min_sep *wsize) then begin
       if keyword_set(verbose) then print,xdmin-xdmax,min_sep*wsize,max_sep*wsize,ist,jst 
        goto, skip
     endif
   endelse
;  we have a valid peak
   peaks = (keyword_set(shadow)) ? [xdmin, xdmax]+ia : [xdmax, xdmin]+ ia
   junk = min(abs(deriv[peaks[0]:peaks[1], jst]), imid)
;  limit delta to 1.5*wsize
   delta = (peaks[1]-peaks[0]) < 1.5*wsize
   ist = imid+peaks[0]
   st = floor(ist-0.6*delta) > 0
   en = ceil(ist+0.6*delta)
   n = en -st+1
   av= (delta/2) + (delta mod 2)
   avh = av/2 
   sta = ( st-av) > 0
   ena = (en + av)< nx-1
   sta = [sta, st]
   ena = [en, ena]
   avh = [((jst-avh) > 0), ((jst+avh) < (ny-1))]
   nsta = sta[1]-sta[0]+1
   nena = ena[1]-ena[0]+1
   navh = avh[1]-avh[0]+1
   f1 = total(origarray[sta[0]:sta[1], avh[0]:avh[1]])/nsta/navh
   f2 = total(origarray[ena[0]:ena[1], avh[0]:avh[1]])/nena/navh
   v = findgen(n)/(n-1)*(f2-f1)+ f1
   newarray[st:en, jst] = v
 skip: 
 endfor
 endfor
 put_array, orig-1, newarray, -1, label= 'New Array'
 if n_elements(strout) gt 0 then begin
   if (strout[0] gt 0) && (strout[0] le get_maxstr()) then $
     xfrstr, orig-1, strout[0] else strout = orig-1
 endif else strout = orig-1
 if keyword_set(plotit) then plotstr, orig-1, /xst, /yst
; arr2str, 76, deriv, clab = 'deriv'
; arr2str, 77, smoarray, clab= 'smoarray'
 end
