; ------------------------------------------------------------------------ 
; $Id: scangrd.pro,v 1.3 2007/06/06 20:09:37 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
pro Scangrd, ain,  Norm,  dsmin, dsmax, dsstp, $
             con=con, index=index, winnum=winnum, values = values, $
             xsize = xsize, ysize = ysize,  _extra = extrastuff
;+
; PURPOSE:
;       Use XINTERANIMATE to build an animation of a DO-list of slices of 
;        a GRID or CONDUCTOR snapshot
; CALLING SEQUENCE:
;       ScanGRD, STR,  NORM, Start [, Stop] [, Skip]
;               [, CON=CON]  [, INDEX=INDEX ] [, VALUES = values]
;               [, XSIZE = xsize] [, YSIZE = ysize]
;               [, WINNUM=WINNUM ] [, _EXTRA = EXTRASTUFF]
; INPUTS:
;       STR :     Structure array value or field structure
;       NORM:     Plane normal for PLOTGRD
;       Start \  
;       Stop -  DO list for frames in the normal direction.
;       Skip  /  
;       Default Value for Stop is number of Number of planes in Normal direction
;       If Stop le start then stop = number of indices in NORM direction
;       If keyword_set(index) then 
;       If Skip le 0 or undefined then skip = 1
;        
; OPTIONAL INPUT:
;       -none-
; KEYWORDS:
;       CON:    Conductor structure
;       VALUES: If present then START, STOP and SKIP are ignored.
;                 VALUES is an array of normal values.
;       SCALE:  Scale factor for field (QHRE argument)
;       INDEX:  Index of plane (PLOTFLD option), Default =1
;               If index eq 0 then default then skip must be defined.
;       WINNUM: IDL window number
;       XSIZE:  X Size of the window; Default = 448
;       YSIZE:  Y Size of the window; Default = 384
;
; Example
;       Make a movie using structure array 4 starting at -.1 to 1.1 in 
;         increments of .05. Use array 5 to overlay the conductors.
;         Make the z axis as the normal direction and show theta as a polar plot
;        ScanGRD,4,3, -.1, 1.1, 0.01, con =5, /polar
; 
;       Make a movie of structure 5 with a normal direction of 3 at values of
;          Z given by gz.  Overlay a conductor from structure 2
;        SCANGRD, 5, 3, values = gz, con= 2 
;
;       Make a movie of the conductors in structure 2 reflected about theta=0
;       Use the z values from block 1.
;        image, 2, 3, ref = -2
;        scangrd, 3, 3, val = get_array(2, 1, 1), /polar 
;       
;         
;-

on_error, 1
@qsstrcom.dat
;
      if n_elements (structinit) eq 0 then qsstruct_init

nparams = n_params()
if nparams lt 2 then begin
  print, '% FORMAT: ScanGRD, STR, FLD, Normal,  Start [, Stop] [, Skip]'
  print, '        [, con=con] [, scale=scale ] [, index=index ]'
  print, '        [, VALUES = values] [, winnum=winnum ] [, _extra=extra ]'
  return
endif

;
      if n_elements(ain) eq 0 then begin
        print, '% ScanGRD: Structure must be defined'
        return
      endif
; change variables to insure we don't corrupt input data
      a = ain
      b = size(a)
      if b(b(0)+1) ne 8 then begin
        if a gt 0 and a le maxstructure then begin
          s2i, a, a
          b = size (a)
          if b(b(0)+1) ne 8 then begin
            print, $
   '% ScanGRD: First parameter must be a structure or a structure array number'
            return
          endif
        endif  else begin
          print, $
    '% ScanGRD: First parameter must be a structure or a structure array number'
          return
        endelse
      endif
structinit = -1
if n_elements(con)    eq 0 then con = 0
if n_elements(scale)  eq 0 then scale = 1.0
if n_elements(index)  eq 0 then index = 0 else begin
  if index eq 0 then begin
    if n_elements(dsstp) eq 0 then begin
      print, '% ScanGRD: STEP must be defined if index eq 0'
      return
    endif
    if n_elements(dsmax) eq 0 then dsmax = dsmin
    if dsmax le dsmin then dsmax = a.range(1, normal-1, 0)
  endif
endelse

if n_elements(winnum) eq 0 then winnum = 1

  nds = n_elements (values)
If nds eq 0 then begin
  if n_elements(dsmin) eq 0 then begin
    PRINT, '% SCANGRD: Minimum value must be defined'
    return
  endif
  if n_elements(dsmax) eq 0 then dsmax = a.size(normal-1, 0)
  if n_elements(dsstp) eq 0 then dsstp = 1

  if n_elements(dsmin) eq 0 then begin
    print, '% ScanGRD: First dataset must be specified.'
    return
  endif
  if dsmax le dsmin then dsmax = a.size(normal-1, 0)
  nds   = (dsmax - dsmin) / dsstp + 1
  values = dsmin+findgen(nds)*dsstp
endif
  

print, nds, ' Frames in this annimation'
 if nds le 0 then begin
   Print, '% ScanGRD: Number of frames must be > 0'
   return
 endif
if n_elements(xsize) eq 0 then xsize = 448
if n_elements(ysize) eq 0 then ysize = 384


window, winnum, xsiz=xsize, ysiz=ysize

xinteranimate, set = [xsize, ysize, nds]

; Loop over requested frames
f = temporary (a)
frame_num = 0
 if norm eq 1 then begin
    tlab = 'x1 = '
    xx= 2
    yy = 3
 endif else if norm eq 2 then begin
    tlab = 'x2 = '
    xx = 3
    yy = 1
 endif else if norm eq 3 then begin
    tlab = 'x3 = '
    xx = 1
    yy = 2
 endif else begin
   print, '% SCANGRD: Normal direction must be 1, 2 or 3'
   return
 endelse


for i = 0, nds-1 do begin 

  normval = values(i)
  tlabel = tlab+strtrim(normval)
  plotstr, f, xx, yy,  normval,  con=con, index=index, $
           tit=tlabel, _extra = extrastuff
  xinteranimate, frame=frame_num, window=winnum
  frame_num = frame_num + 1
endfor
wdelete, winnum

xinteranimate, /block

;; xinteranimate, /close

return
end
