; ------------------------------------------------------------------------
; $Id: sgolstr.pro,v 1.2 2010/09/17 20:51:05 lpmix Exp $
; ------------------------------------------------------------------------
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
;+
; NAME:
;       SGOLSTR
;
; PURPOSE:
;       This routine utilizes the IDL function SAVGOL to obtain the coefficients
;	of a Savitzky-Golay smoothing filter.  This filter is then applied to a 
;       structure array for a weighted average smoothing.  This filter can also be used
;       calculate derivatives of the input waveform.  The filter is applied to each
;       row or each column depending on the keyword.
;
;       This function is not optimal for smoothing a waveform, but is ideal for
;       optimally calculating numeric derivatives.
;
;       This filter works especially well when the typical peaks of the signal 
;       are narrow. The heights and widths of the curves are generally preserved.
;
; CATEGORY:
;       Mathematics
;
; CALLING SEQUENCE:
;
;       SGOLSTR, STR1, STR2
;
; INPUTS:
;       STR1:    Structure array number for the waveform to be smoothed or differentiated.
;                Structure should be two dimensional and uniform in the direction of the filter.
;                Initial version will interpolate the filter direction to uniform, but the 
;                  structure must be either two dimensional or 3 dimensional with a degenerate 
;                  third dimension.
;       NOTE:  If structure is nonuniform in the filter direction, the data will be changed to uniform data.
; OPTIONAL INPUTS:
;       STR2:    Structure array number for the smoothed data or for the derivative.
;	
; KEYWORD PARAMETERS:
;
;       NOTE: Either Horizontal or Vertical must be set.
;       VERTICAL:   If set, perform filter on each column of the image.
;       HORIZONTAL: If set, perform filter on each row of the image.
;       NOTE:   If both are set, a HORIZONTAL filter is calculated.
;               If neither are set, you must specify a direction.
;
;       CLABEL:  Dataset comment for the saved calculated image.
;                If CLABEL is not specified, then SAVGOL(Left, Right, Order, Degree) 
;                  will be added to the old comment separated by ' - ' or ';;'.
;       XLABEL:  X-AXIS label for the calculated image
;                DEFAULT: XLABEL for STR1
;       YLABEL:	 Y-AXIS label for the calculated image
;                DEFAULT: YLABEL for STR1
;
; NOTE:  The following refer to the coefficients for the filter.
;        The descriptions are from the SAVGOL documentation. (? SAVGOL)
;
; NOTE: If either left or right is set and the other is not, then they will be set equal. 
;       LEFT:   An integer specifying the number of data points to the left of
;               each point to include in the filter.
;               DEFAULT:  LEFT = (n_elements(DEGREE) eq 0) ? 5 : DEGREE
;                         LEFT = LEFT > 1
;       RIGHT:  An integer specifying the number of data points to the right of
;               each point to include in the filter. 
;               DEFAULT:  RIGHT = (n_elements(DEGREE) eq 0) ? 5 : DEGREE
;                         RIGHT = RIGHT > 1
;       Note:   Larger values of Nleft and Nright produce a smoother result at
;               the expense of flattening sharp peaks.
;       ORDER:  An integer specifying the order of the derivative desired.
;               For smoothing, use order 0. To find the smoothed first derivative
;               of the signal, use order 1, for the second derivative, use order 2, etc.
;               DEFAULT: Order =0
;       Note:   Order must be less than or equal to the value specified for DEGREE.
;       DEGREE: An integer specifying the degree of smoothing polynomial. Typical values
;               are 2 to 4. Lower values for DEGREE will produce smoother results but may
;               introduce bias, higher values for DEGREE will reduce the filter bias, but
;               may "over fit" the data and give a noisier result.
;               DEFAULT:  DEGREE = 3 < (NLEFT+NRIGHT+1) 
;       Note:   DEGREE must be less than the filter width (NLEFT + NRIGHT + 1).
;       DOUBLE: Set this keyword to force the computation to be done in double-precision
;               arithmetic.
;               DEFAULT: DOUBLE = (DEGREE gt 4) ? 1 : 0
; OUTPUTS:
;       A smoothed waveform or a derivative of the input waveform
;
; PROCEDURE:
;       The IDL procedure SAVGOL is used to calculate the coefficients of the Savitzky-Golay 
;       filter.  If necessary the coefficients are normalized.  CONVOL is then used to perform 
;       the filtering with keyword set: EDGE_TRUNCATE.
;
; EXAMPLE:
;       Calculate the first derivative of structure 1 using a total of 11 points with a 
;       Savitzky-Golay filter degree of 5 in the Y direction, and store the result in 2
;         SGOLSTR, 1, 2, O = 1, D =5, /V ;;  ORDER = 1 is the first derivative
;      or
;         SGOLSTR, 1, 2, ORDER = 1, LEFT = 5, DEGREE = 5, /VERTICAL ; Since Left specified, right = left
;
;       Calculate the second derivative of waveform 7, with 10 points before and after each
;       point with a filter degree of 6 in the X direction.  Save the derivative in 57.
;         SGOLSTR, 7, 57, RIGHT = 10, ORDER =2, DEGREE =6, /HOR  ; Note since RIGHT=10 and LEFT is BLANK
;                                                         ; DEFAULT is LEFT = RIGHT
;
; MODIFICATION HISTORY:
; 	Written by:	L. P. Mix, January 30, 2009
;
;-

PRO SGOLSTR, WD1, WD2, CLABEL = clab, XLABEL = xlab, YLABEL = ylab, DOUBLE = dpin, $
    LEFT = leftin, RIGHT = rightin,  ORDER = orderin, DEGREE = degreein, $
    VERTICAL = vertical, HORIZONTAL = horizontal

on_error, 1
@com.dat

nparams = n_params()
if nparams eq 0 then begin
  print, '% Format: SGOLSTR, WD1, WD2, CLABEL = clab, XLABEL = xlab, '
  print, '          YLABEL = ylab, LEFT = left, RIGHT = right,  ORDER = order, '
  print, '          DEGREE = degree, DOUBLE = double_precision, /VERTICAL, /HORIZONTAL'
  return
endif

if n_elements(wd1) eq 0 then begin
  print, '% SGOLSTR: Input waveform must be defined'
  return
endif
 
wdf = (STRvalid(wd1[0], /valid, /quiet))[0]
if wdf lt 1 then begin
  print, '% SGOLSTR: Valid WDF array must be present'
  print, '  Unable to fine WDF = ' +strtrim (wd1[0], 2)
  return
endif

; Check parameters
nleft = n_elements(leftin) < 1
nright = n_elements (rightin) < 1
if nleft +nright eq 0 then begin
  left = 5
  right = 5
endif else if nleft +nright eq 1 then begin
  left = (nleft eq 1) ? leftin : rightin
  right = left
endif else begin
  left= leftin[0] > 1
  right= rightin[0] > 1
endelse
order = (n_elements(orderin) eq 0) ? 0: (orderin > 0)
if (n_elements(degreein) eq 0) then begin
  degree = 3 < (left+right +1)
endif else degree = degreein[0]
if (order gt degree) or (degree gt (right+left+1)) then begin
    print, '% SGOLSTR:  Value of parameters are not consistent: '
    print, ' Left, Right, Order, Degree: '
    print, left, right, order, degree
    return
endif

;
;  Get_ARRAY returns a uniform wdf array.
;
S2I, S, WDF
if s.type ne 1 then begin
  print, '% SGOLSTR: Only type 1 structures may be processed at this time'
  return
endif
if s.ndim eq 3 then begin
  if s.two ne 3 then begin
    print, '% SGOLSTR: Structure must be 2D or 3D with a degenerate 3rd dim'
    print, '       NDIM, STR.TWO: ', s.ndim, s.two
    return
  endif
endif else if s.ndim ne 2 then begin
  print, '% SGOLSTR: Structure must be 2D or 3D with a degenerate 3rd dim'
  print, '           Structure dimensions: ', s.ndim
  return
endif
lab = ''
if keyword_set(horizontal) then begin
  lc_hor = 1 
  lab = 'HORIZONTAL;;SAVGOL('
endif else if keyword_set(vertical) then begin
  lc_hor = 0
  lab = 'VERTICAL;;SAVGOL('
endif
if strlen(lab) eq 0 then begin
  print, '% SGOLSTR: Filter direction must be specified.'
  print, '           /VERTICAL (/V) or /HORIZONTAL (/H)'
  return
endif

ok = where(strpos(tag_names(s), 'DX') ge 0 , nokdx)
if lc_hor then begin
  x = s.x1
  ny = s.size[1]
  if nokdx gt 0 then dx = s.dx1 else check_uniform, x, dx
  if nokdx gt 0 then dy = s.dx2 else check_uniform, y, dy
  array = get_array(s, -1)
endif else begin
  x = s.x2
  ny = s.size[0]
  if nokdx gt 0 then dx = s.dx2 else check_uniform, x, dx
  if nokdx gt 0 then dy = s.dx1 else check_uniform, y, dy
  array = get_array(s, -1)
  array = transpose(array)
endelse
nx = n_elements(x)

if dx eq 0 then begin
; be sure x is monotonic
  ddx = x[1:nx-1] - x[0:nx-2]
  if min (ddx) lt 0 then begin
    print, '% SGOLSTR: Filter direction must have a monotonically increasing value'
    return
  endif
  lc_interp = 1
  xx = x
  i2w, xx, xx, dx, /get_delta, endpoint = endpoint
  n = 1L +round((endpoint - xx[0])/dx)
  new_x = xx[0] + findgen(n)*dx
  result = findex(x, new_x)
  x = new_x
  nx = n_elements(x)
endif else lc_interp =0  
new = make_array(nx, ny)
dp = (n_elements(dpin) eq 0) ? (degree gt 4) : dpin[0]
savgolfilter = savgol(left, right, order, degree, doubl = dp)
if order gt 0 then savgolfilter = savgolfilter * (FACTORIAL(order)/(dx^order))

for i = 0, ny-1 do begin
  y = array[*, i]
  if lc_interp then y = interpolate(y, result)
  y = convol(y, savgolfilter, /edge_truncate)
  new[0, i] = y
endfor

if nparams gt 1 then save = get_save_array(wd2, wd1) else save = wd1

if n_elements (clab) eq 0 then begin
  clabo = s.clab
  if strpos(s.clab, ' - ') gt 0 then $
    clab = s.clab+';;'+lab+strtrim(left,2)+','+strtrim(right,2)+','+strtrim(order,2)+','+strtrim(degree,2)+')' $
    else $
    clab = s.clab+' - '+lab+strtrim(left,2)+','+strtrim(right,2)+','+strtrim(order,2)+','+strtrim(degree,2)+')'
endif
if n_elements (xlab) eq 0 then xlab = s.x1lab
if n_elements (ylab) eq 0 then ylab = s.x2lab

if lc_hor then $
  arr2str, save, new, x, s.x2, file = s.file, xlab = xlab, ylab = ylab, clab = clab, dx = dx, dy = s.dx2 $
else $
  arr2str, save, transpose(new), s.x1, x, file = s.file, xlab = xlab, ylab = ylab, clab = clab, dx = dx2, dy = dx 

END
