; ------------------------------------------------------------------------ 
; $Id: slice.pro,v 1.6 2010/12/08 16:30:29 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
 Function slice, ain, fld, kvalue1, value1, kvalue2, value2, $
            vector = vector,  $
            overplot = over, block = block, index = index, integrate = integ
;
;+
; NAME:
;	SLICE
;
; PURPOSE:
;	Reduce a three or four dimensional field structure to two dimensions
;       Note: Four dimensional data is checked for valid data as closely as 3D
;       data.
;
; CATEGORY:
;	Graphics
;
; CALLING SEQUENCE:
;
;	Structure2 = SLICE (Structure1, Field, Kvalue1, Value1, Kvalue2, Value2)
;
;
; INPUTS:
;	Structure1: Input this is a multidimensional field type structure.
;                     or a structure array numbger
;       Structure2: Output this is a two dimensional field structure or 
;                     a multidimensional field structure with degenerate
;                     dimensions.
;       Field:      Field component to be sliced
;       Note: Field may be positive or negative; Field = -(abs(Field))
;       Note: If Field is an array, the square root of the sum of
;             the squares will be used.
;       Note: If A has only one attribute (such as an image or a charge density);
;             the this parameter can be omitted.
;       KVALUE1,2
;             If Field is a two dimensional field then these values are ignored.
;             If Field is a three or four dimensional field then these values
;               indicate the coordinates which are fixed for the two
;               dimensional plots.
;       VALUE1, 2
;             If Field is a two dimensional field then these values are ignored
;             If Field is a three or four dimensional field then these values
;               indicate the coordinate values for the fixed dimensions in
;               the two dimensional plots. Linear interpolation is used
;               according to the value of VALUE1. If VALUE1 has two elements
;               the average value over the interval is used.
;             WARNING: If INDEX is specified, VALUE1 will be changed to spatial
;                      coordinates using the midpoints of the bins.
;
; KEYWORDS:
;       INTEGRATE If present and not zero, integrate the slice.
;                 The integral will be performed between value1(0) and value1(1)
;                 If integrate = 1, a simple integral
;                 If integrate = 2, a x dx integral
;                 If integrate = 3, a x^2 dx integral
;                 Limitations:
;                     Data must have a single block
;                     Two values of value 1 must exist
;       OVERPLOT: If overplot is not zero and kvalue1,2 is undefined, the 
;                 values will be taken from the last plot.
;       BLOCK   : If the field is multi-block, BLOCK can be used to specify
;                 one or more blocks to be sliced.  BLOCK is an array of one
;                 or more values.
;       INDEX   : If present and not zero, then the units of VALUEi will be
;                 taken as indices of of the field array.  Valid values are
;                 1 to the maximum number of values in KVALUEi.
;                 Care should be exercised with multiple block data.
;             WARNING: If INDEX is specified, VALUE1 will be changed to spatial
;                      coordinates using the midpoints of the bins.
;       VECTOR  : If set all vector components will be returned.
;
; OUTPUTS:
;	Structure is modified to produce a two dimensional structure or 
;       a structure with n-dimensions and all but two degenerate
;
; EXAMPLE:
;	Reduce structure f to a two dimensional structure by slicing along
;         the second dimension between 0.0 and 0.4 Save the magnitude of the
;         field vector in structure g.
;       g = SLICE (f, [1,2,3], 2, [0.0, 0.4])
;
;       Assume d is a charge density structure with one attribute.  Take a slice 
;         at z= 0.25 and store in sturcture array 12.
;       i2s, SLICE (d, 3, .25), 12
; <or>  i2s, SLICE(d, 1, 3, .25), 12
;
; MODIFICATION HISTORY:
; 	Written by:	Paul Mix, August 10, 1993
;-
; insert common blocks
;
@qscom.dat
;
@qsstrcom.dat
;
;       return on an error
      on_error, 1
;
      npar = n_params()
      if n_elements (ain) eq 0 then begin
 print, '% Format:', $
' Structure2 = SLICE (Structure1, Field, Kvalue1, Value1, Kvalue2, Value2)'
 print,'    [, OVERPLOT = overplot] [, BLOCK = block] [, INDEX = index]'
 print,'    [, INTEGRATE = integrate]'
        return, 0.0
      endif
      if n_elements (structinit) eq 0 then qsstruct_init
      if structinit lt 0 then lcsave = 0 else lcsave = 1
      if structinit lt 0 then a = temporary(ain) else a = ain 
      structinit = 1
      b = size(a)
      if b(b(0)+1) ne 8 then begin
        if a gt 0 and a le maxstructure then begin
          s2i, a, a
          b = size (a)
          if b(b(0)+1) ne 8 then begin
            print, '% SLICE: ', $
   'First parameter must be a structure or a structure array number'
            return, 0.0
          endif
        endif  else begin
          print, '% SLICE: ', $
   'First parameter must be a structure or a valid structure array number'
          return, 0.0
        endelse
      endif
      if a.type ne 1 then begin
        if a.type eq 8 then begin
          if ((a.ndim eq 2) or ((a.ndim eq 3) and (a.two eq 3)))then begin
            if a.ndim  eq 3 then begin
              kvalue1 = 3
              value1 = a.slice
              if a.x3[0] ne a.slice then a.x3 = a.slice
            endif
            return, a
          endif 
          print, '% SLICE: Only 2 dimensional type 8 structures allowed'
          return, 0.0
        endif
        print, '% SLICE: Only Field type structures may be sliced with SLICE'
        return, 0.0
      endif
;
;     if we already have two dimensional data with a single attribute or 
;       vector quantity on a single block then return 
;
      if n_elements(integ) eq 0 then integ = 0
      if integ lt 0 or integ gt 3 then begin
        print, '% SLICE: Slice integration parameter must be in the range [0, 3]'
        return, 0.0
      endif
;      put in as a fix for 4 dim data
;      a.two = 0 is normal for 3 d data.
;
      if a.two eq 4 then a.two = 0
      ok = where (a.size[*, 0] gt 1, aandim)
      if (aandim eq 2) then begin
        if ((a.nvect eq 1) and (a.block eq 1) and integ eq 0) then begin
          if a.two ne 0 then kvalue1 = a.two else kvalue1 = 3
          value1 = a.slice
          return, a
        endif
      endif
;
; Function slice, ain, fld, kvalue1,value1, kvalue2, value2, $
;   fld, kvalue1,value1, kvalue2, value2 need protection
    if lcsave eq 1 then begin
      if n_elements(value2) gt 0 then value2in = value2
      if n_elements(kvalue2) gt 0 then kvalue2in = kvalue2
      if n_elements(value1) gt 0 then value1in = value1
      if n_elements(kvalue1) gt 0 then kvalue1in = kvalue1
      if n_elements(fld) gt 0 then fldin = fld
    endif


      if npar eq 4 or npar eq 6 then begin
;       if data is from PLOTFLD then we need to check for undefined.
        if n_elements(value2) eq 0 then npar = 5
        if n_elements(kvalue2) eq 0 then npar = 4
        if n_elements(value1) eq 0 then npar = 3
        if n_elements(kvalue1) eq 0 then npar = 2
        if n_elements(fld) eq 0 then npar = 1
      endif
; field is optional if the number of attributes is one.
; help, value2, npar

      if npar eq 3 or npar eq 5 then begin
        if a.nvect eq 1 or keyword_set(vector) then begin
          if n_elements(kvalue2) gt 0 then value2 = kvalue2
          if n_elements(value1) gt 0 then kvalue2 = value1
          if n_elements(kvalue1) gt 0 then value1 = kvalue1
          if n_elements(fld) gt 0 then kvalue1 = fld
          if a.nvect eq 1 then fld = 1 else fld = indgen(a.nvect)+1
        endif
      endif
;
      if a.nvect eq 1 then begin
;     no matter what they entered for a field we will slice the only 1 we have!
        fld = -1
        nfld = 1
      endif else begin
        nfld = n_elements (fld)
        if nfld gt 0 then fld = abs(fld) else fld = 0
        if fld (0) gt a.nvect and fld[0] gt a.ndim then begin
          fld = indgen(a.nvect)+1
          nfld = a.nvect
        endif
        ii = where (fld le a.nvect and fld gt 0, nfld)
        if nfld eq 0 then begin
          Print, '% SLICE: FIELD array must be defined'
          Print, ' The following fields are available:'
          print, ' 1. ', a.v1lab
          if a.nvect gt 1 then print, ' 2. ', a.v2lab
          if a.nvect gt 2 then print, ' 3. ', a.v3lab
          if a.nvect gt 3 then print, ' 4. ', a.v4lab
          if a.nvect gt 4 then print, ' 5. ', a.v5lab
          if a.nvect gt 5 then print, ' 6. ', a.v6lab
          print, ' Please reenter the command and specify the vector desired'
          return, 0.0
        endif else fld = -fld(ii)
        if nfld eq 1 then fld = fld(0)
      endelse
;
;     If we have degenerate data, we must slice there
;     (With 4 dimensional data the logic below should be modified to 
;      see if kvalue2 is cutting along a.two)
      if a.two ne 0  and a.ndim gt 2 then begin
        if n_elements(kvalue1) eq 0 then kvalue1 = a.two
        if n_elements(value1) eq 0 then value1 = a.range(0, kvalue1-1,0)
        if kvalue1 ne a.two then begin
          print, '% SLICE: Structure is degenerate on dimension', a.two
          print, 'Cannot take a slice on dimension', kvalue1
          return, 0.0
        endif
      endif
;
;     if we have nondegenerate 3-d data see if we can use previous values
;
      if n_elements(over) eq 0 then over = 0
      if (over ne 0) then begin
        if (a.two eq 0) then begin
;       use the previous values if undefined.
          if n_elements(kvalue1) eq 0 then kvalue1 = zplaneold
          if n_elements(value1) eq 0 then value1 = zvalueold
        endif
        if n_elements (kvalue1) eq 0 then kvalue1 = zplaneold
        if kvalue1 ne zplaneold then begin
          Print, '% SLICE: WARNING: This overplot requested on plane ', kvalue1
          Print, '         Old plot was on plane', zplaneold
          print, '         Hit "q" to quit; hit any other key to continue'
          key =  (byte (get_kbrd(1)))(0)
          if ((key eq 81) or (key eq 113)) then begin
            return, 0.0
          endif
        endif
      endif
;
      if a.ndim gt 2 and (n_elements(value1) eq 0 or $
        n_elements (kvalue1) eq 0) then begin
        print, '% SLICE: Vector has ' , strtrim(a.ndim, 2), ' Dimensions'
        print, '         Please reenter the command and specify the dimension to'
        print, '         be fixed and the value.'
        return, 0.0
      endif
      if a.ndim gt 3 and (n_elements(value2) eq 0 or $
        n_elements (kvalue2) eq 0) then begin
        print, '% SLICE: Vector has ' , strtrim(a.ndim,2), ' Dimensions'
        print, '         Use SLICE to reduce the number of dimensions to two.'
        return, 0.0
      endif
;
;     reduce the dimensions of the structure to a two dimensional structure
; 
;     kvalue1 = kvalue1(0)
;     kvalue2 = kvalue2(0)

      if a.ndim eq 2 then begin
        xval = 1 & yval = 2
      endif else if a.ndim eq 3 then begin
        if kvalue1 ne 1 then xval =1 else xval = 2
        if kvalue1 ne xval+1 then yval = xval+1 else yval = xval+2
      endif else begin
        if kvalue1 eq kvalue2 then begin
          print, '% SLICE: Two distinct dimensions must be eliminated'
          print, '         Array has 4 dimensions and kvalue1 = kvalue2'
          return, 0.0
        endif
        if kvalue1 gt kvalue2 then begin
          temp = kvalue1 & kvalue1 = kvalue2 & kvalue2= temp
          temp = value1  & value1  = value2  & value2 = temp
        endif
        if kvalue1 ne 1 then begin
          xval =1 
          yval = 9-kvalue1-kvalue2
        endif else begin
          xval = (5 - kvalue2) > 2
          yval = (7 - kvalue2) < 4
        endelse
      endelse
;
      if n_elements(index) eq 0 then index = 0
;
;     check on the values for the third dimension
      if a.ndim gt 2 then begin
        if index eq 0 then begin
          minv = a.range(0, kvalue1-1, 0)
          maxv = a.range(1, kvalue1-1, 0)
          if n_elements (value1) gt 1 then begin
            value1 = float(value1(0:1))
            value1 = [min(value1, max = mxv), mxv]
            if value1(0) lt minv then value1(0) = minv
            if value1(0) gt maxv then value1(0) = maxv
            if value1(1) lt minv then value1(1) = minv
            if value1(1) gt maxv then value1(1) = maxv
            if value1(0) eq value1(1) then value1 = value1(0)
          endif else  begin
            if value1 lt minv then value1 = minv
            if value1 gt maxv then value1 = maxv
          endelse
          zvaluemid = total (value1)/n_elements(value1)
        endif else begin
          maxv = max( a.size (kvalue1-1, *))
          value1 = fix(value1)
          if n_elements (value1) gt 1 then begin
            value1 = value1(0:1)
            value1 = [min(value1, max = mxv), mxv]
            if value1(0) lt 1 then value1(0) = 1
            if value1(0) gt maxv then value1(0) = maxv
            if value1(1) lt 1 then value1(1) = 1
            if value1(1) gt maxv then value1(1) = maxv
            if value1(0) eq value1(1) then value1 = value1(0)
          endif else  begin
            if value1 lt 1 then value1 = 1
            if value1 gt maxv then value1 = maxv
          endelse
        endelse
      endif
;     figure out the blocks
      if n_elements (block) eq 0 then block = 0
      if n_elements (block) eq 1 then begin
        if block lt 0 or block gt a.block then block = 0
;
;       determine the range of the data if we must determine levels
;       and draw a grid
;
        if block gt 0 then ts = block else ts = 1
        if block gt 0 then tf = block else tf = a.block
        nblock = tf-ts+1
        blockarray = indgen(nblock)+ts
      endif else begin
        blockarray = block
        ok = where (blockarray gt 0 and blockarray le a.block, nblock)
        if nblock gt 0 then blockarray= blockarray(ok) else begin
          print, '% SLICE:', block, ' is not a valid block specification'
          print, '         Please try again.'
          return, 0.0
        endelse
        block = indgen(nblock)+1
      endelse
;
; take only those blocks which are appropriate
;
      if a.ndim gt 2 then begin
        error = (a.range[1, kvalue1-1, 0] - a.range[0, kvalue1-1, 0])*1e-5
        if index eq 0 then $
          ok = where (a.range(0, kvalue1-1, blockarray) le zvaluemid+error $
          and a.range(1, kvalue1-1, blockarray) ge zvaluemid-error,nblock) $
        else $
          ok = where (value1(0) le a.size(kvalue1-1, blockarray), nblock)
        if nblock eq 0 then begin
          print, '% SLICE: None  of the blocks of data intercept the plane'
          print, '         value =', value1, ' **** returning ****'
          return, 0.0
        endif
        blockarray= blockarray(ok)
      endif
;
;     check on integration of the slice
;
      if integ gt 0 then begin
        if nblock gt 1 or n_elements(value1) ne 2 then begin
          print, '% SLICE: Slice integration must be performed on a single block'
          print, '         between two different values.'
          return, 0.0
        endif
        if a.ndim lt 3 then begin
          print, '% SLICE: Slice integration must be performed on 3-D data.'
          return, 0.0
        endif
      endif
; initialise arrays
      ndim = a.ndim
      x1 = 0.0 & x2 = 0.0 & x3 = 0.0 & x4 = 0.0
      v1 = 0.0 & v2 = 0.0 & v3 = 0.0 & v4 = 0.0 & v5 = 0.0 & v6 = 0.0
      locx = make_array (/long, ndim, nblock+1)
      locv = make_array (/long, nblock + 1)
      ssize = make_array (ndim, nblock, /long, value = 1L)
      x1lab = make_array(nblock, /string, value = 'Sliced: ' +a.x1lab[0])
      x2lab = make_array(nblock, /string, value = 'Sliced: ' +a.x2lab[0])
      x3lab = make_array(nblock, /string, value = 'Sliced: ' +a.x3lab[0])
      x4lab = make_array(nblock, /string, value = 'Sliced: ' +a.x4lab[0])
      v1lab = make_array(nblock, /string, value = '')
      v2lab = make_array(nblock, /string, value = '')
      v3lab = make_array(nblock, /string, value = '')
      v4lab = make_array(nblock, /string, value = '')
      v5lab = make_array(nblock, /string, value = '')
      v6lab = make_array(nblock, /string, value = '')
      range = make_array(2, ndim, nblock+1)
;
      for ii = 0, nblock-1 do begin
        iblock = blockarray(ii)
;
        xx = get_array (a, xval, iblock, lab = lab)
        nx = n_elements (xx)
        range(0, xval-1, ii+1) = min(xx, max = mx)
        range(1, xval-1, ii+1) = mx
        if xval eq 1 then begin
          x1 = [x1,xx]
          x1lab(ii) = lab
          locx(0, ii+1) = locx(0, ii) + nx
          ssize(0, ii) = nx
        endif else if xval eq 2 then begin
          x2 = [x2, xx]
          x2lab(ii) = lab
          locx(1, ii+1) = locx(1, ii) + nx
          ssize(1, ii) = nx
        endif else if xval eq 3 then begin
          x3 = [x3, xx]
          x3lab(ii) = lab
          locx(2, ii+1) = locx(2, ii) + nx
          ssize(2, ii) = nx
        endif
;
        yy = get_array (a, yval, iblock, lab = lab)
        ny = n_elements (yy)
        range(0, yval-1, ii+1) = min(yy, max = mx)
        range(1, yval-1, ii+1) = mx
        if yval eq 2 then begin
          x2 = [x2,yy]
          x2lab(ii) = lab
          locx(1, ii+1) = locx(1, ii) + ny
          ssize(1, ii) = ny
        endif else if yval eq 3 then begin
          x3 = [x3, yy]
          x3lab(ii) = lab
          locx(2, ii+1) = locx(2, ii) + ny
          ssize(2, ii) = ny
        endif else if yval eq 4 then begin
          x4 = [x4, yy]
          x4lab(ii) = lab
          locx(3, ii+1) = locx(3, ii) + ny
          ssize(3, ii) = ny
        ENDIF
        if keyword_set(vector) and (nfld gt 1) then begin
          new_nvect = nfld
;          help, index, kvalue1
;          print, fld
          for i = 0, nfld-1 do begin
            cont = get_array (a, fld[i], iblock, lab = flab)
;           reduce dimensions if ndim gt 2
            if ndim gt 3 then  begin
              z = get_array(a, kvalue2,iblock)
              if index eq 0 then begin
                cont = reduce_array(cont, kvalue2, z, value2)
              endif else $
                cont = reduce_iarray (cont, kvalue2,z, value2)
            endif
            if ndim gt 2 then begin
              if a.two ne 0 then cont = reform (cont) else begin
                z = get_array(a, kvalue1,iblock)
                if index eq 0 then begin
                  cont = reduce_array(cont, kvalue1, z, value1,integ)
                endif else $
                  cont = reduce_iarray (cont, kvalue1,z, value1, integ)
               endelse
            endif

            ncont = n_elements (cont)
            if ncont ne nx*ny then begin
              help, cont, xx, yy
              print, '% SLICE: Error reading vector value.'
              return, 0.0
            endif
            aa = 'v'+strtrim(i+1,2)
            cmd = aa+' = [' + aa + ',  cont(0:ncont-1)]'
            error = execute(cmd)
            if error eq 0 then begin
              print, '% SLICE: Error Setting vector value; Command: ', cmd
              return, 0.0
            endif
            cmd = aa + 'lab[ii] = flab'
            error = execute(cmd)
            if error eq 0 then begin
              print, '% SLICE: Error Setting vector value; Command: ', cmd
              return, 0.0
            endif
          endfor
          locv(ii+1) = locv(ii)+ncont
        endif else begin
          new_nvect = 1L
          if nfld eq 1 then begin
            cont = get_array (a, fld, iblock, lab = flab)
          endif else begin
            temp = get_array(a, fld(0), iblock, lab= lab)^2
            flab = 'Magnitude of '+lab
            for j = 2, nfld do begin
              temp  = temp + get_array (a, fld(j-1), iblock, lab = lab)^2
              flab=flab+' + '+lab
            endfor
            cont = sqrt(temp)
            temp=0
          endelse
          if ndim gt 3 then  begin
            z = get_array(a, kvalue2,iblock)
            if index eq 0 then begin
              cont = reduce_array(cont, kvalue2, z, value2)
            endif else $
              cont = reduce_iarray (cont, kvalue2,z, value2)
          endif
          if ndim gt 2 then begin
            if a.two ne 0 then cont = reform (cont) else begin
              z = get_array(a, kvalue1,iblock)
              if index eq 0 then begin
                cont = reduce_array(cont, kvalue1, z, value1,integ)
              endif else $
                cont = reduce_iarray (cont, kvalue1,z, value1, integ)
             endelse
          endif

          ncont = n_elements (cont)
          if ncont ne nx*ny then begin
            help, cont, xx, yy
            print, '% SLICE: Error reading vector value.'
            return, 0.0
          endif
          v1 = [v1, cont(0:ncont-1)]
          locv(ii+1) = locv(ii)+ncont
        endelse
      endfor
;
      if a.two ne 0 then two = a.two else two = long(kvalue1)
;     return the true value of value1 used for the slice. 
;     Actually for most arrays the slice extends an additional half cell in 
;       either direction.
      if index ne 0 then begin
        zz = get_array (a, kvalue1, blockarray(0))
        if n_elements (value1) gt 1 then begin
          val1  = value1(0) >1
          val2  = value1(1) < n_elements(zz)
          value1 = [zz(val1-1), zz(val2-1)]
        endif else value1 = zz(value1-1)
        if n_elements(value2) gt 0 then begin
          tt = get_array (a, kvalue2, blockarray(0))
          if n_elements (value2) gt 1 then begin
            val1  = value2(0) >1
            val2  = value2(1) < n_elements(tt)
            value2 = [tt(val1-1), tt(val2-1)]
          endif else value2 = tt(value2-1)
        endif
      endif
      
;
      if a.ndim eq 2 or a.two ne 0 then value1 = a.slice else $
        if n_elements(value1) eq 0 then value1 = a.slice
      slice = total(float(value1))/n_elements(value1)
      if n_elements (value2) gt 0 then $
        slice2 = total(float(value2))/n_elements(value2)

      if a.ndim gt 2 then range (*, kvalue1-1, *) = slice
      if n_elements (x1) gt 1 then x1 = x1(1:*) else begin
        x1 = make_array(nblock, value = slice)
        locx(0, *) = indgen(nblock+1)
      endelse
      if n_elements (x2) gt 1 then x2 = x2(1:*) else begin
        if kvalue1 eq 2 then $
        x2 = make_array(nblock, value = slice) else $
        x2 = make_array(nblock, value = slice2)
        locx(1, *) = indgen(nblock+1)
      endelse
      if ndim gt 2 then begin
        if n_elements (x3) gt 1 then x3 = x3(1:*) else begin
          if kvalue1 eq 3 then $
          x3 = make_array(nblock, value = slice) else $
          x3 = make_array(nblock, value = slice2)
          locx(2, *) = indgen(nblock+1)
        endelse
        if ndim gt 3 then begin
          if n_elements (x4) gt 1 then x4 = x4(1:*) else begin
            x4 = make_array(nblock, value = slice2)
            locx(3, *) = indgen(nblock+1)
          endelse
        endif
      endif
      v1 = v1(1:*)
      if (keyword_set(vector) and (nfld gt 1)) then begin
        v2= v2[1:*]
        if nfld gt 2 then begin
          v3= v3[1:*]
          if nfld gt 3 then begin
            v4= v4[1:*]
            if nfld gt 4 then begin
              v5= v5[1:*]
              if nfld gt 5 then v6= v6[1:*]
            endif
          endif
        endif
      endif
          
; generate the ordering
;
      mxbl = nblock
      for i = 0, ndim -1 do begin
        range(0, i, 0)=min(range(0,i,1:mxbl))
        range(1, i, 0)= max(range(1,i,1:mxbl))
      endfor
      if mxbl gt 1 then begin
        order = make_array(ndim, mxbl, /long)
        for i = 0, ndim-1 do order (i, *) = sort(range(0,i, 1:mxbl))
      endif else order = 1L
      dummy = (nblock eq 1) ? 0.0 : make_array(nblock, value = 0.0)
      a = {type:1L, ndim:ndim, nvect: new_nvect, block :nblock, two: two, $
            slice: slice, size:ssize,range: range, order: order, $
              locx:locx, dx1 : dummy, dx2 : dummy, dx3 : dummy, dx4 : dummy,  $
              locv: locv, x1:x1, x2:x2, x3:x3, x4:x4, v1:v1, v2:v2, $
              v3:v3, v4:v4, v5:v5, v6:v6,x1lab : x1lab, x2lab :x2lab, $
              x3lab : x3lab, x4lab :x4lab,v1lab : v1lab, v2lab :v2lab, $
              v3lab : v3lab, v4lab :v4lab,v5lab : v5lab, v6lab :v6lab, $
              file : a.file, spare : a.spare, tlab : a.tlab, $
              clab: a.clab}
      if lcsave eq 1 then begin
        if n_elements(value2in) gt 0 then value2= value2in
        if n_elements(kvalue2in) gt 0 then kvalue2= kvalue2in
        if n_elements(value1in) gt 0 then value1= value1in
        if n_elements(kvalue1in) gt 0 then kvalue1= kvalue1in
        if n_elements(fldin) gt 0 then fld= fldin
      endif

      return, a
      end
