; ------------------------------------------------------------------------ 
; $Id: sliceall.pro,v 1.2 2007/06/06 20:09:37 lpmix Exp $
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
;+
; NAME:
;	SLICEALL
;
; PURPOSE:
;	Reduce a multi-dimensional field structure to fewer dimensions
;       Up to 6 dimensions are supported.
;       Output array must be less than 4 dimensions
;       Only single block, type 1 structures are supported.
;
; CATEGORY:
;	Graphics
;
; CALLING SEQUENCE:
;
;	Structure2 = SLICEALL (STRUCTURE1, DIMENSION, VALUE, /INDEX, WDF = wdf)
;
; INPUTS:
;	STRUCTURE1: Input this is a multidimensional field type structure.
;                     or a structure array numbger
;       Structure2: Output this is a Multi-dimensional field structure or 
;                   a wdf array number if Structure 1 is reduced to one dimension.
;                   The reduced structure has knowledge of the eliminated
;                   dimensions only through the dataset comment.
;                   All field components are sliced.
;                   If a one-dimensional data structure remains, it will be stored
;                   in a WDF array.  The array numbers are returned in STRUCTURE 2.
;                   If not specified, the arrays are stored in 
;                   (get_maxwdf()-nvect) and successive arrays.
;       DIMENSION:  A dimension or an array of dimensions to be removed.
;                   The dimensions are numbered from 1 to the total number of
;                   dimensions.
;       VALUE:      A value or an array of values for each dimension.
;                   This value will be changed to the nearest bin value.
;          WARNING: If INDEX is specified, VALUE will be changed to spatial
;                   coordinates of the bin.
;
; KEYWORDS:
;       INDEX:      If present and not zero, then the units of VALUE will be
;                   taken as indices of of the spatial array.  Valid values are
;                   1 to the maximum number of values in in that dimension.
;          WARNING: If INDEX is specified, VALUE will be changed to spatial
;                   coordinates of the bin.
;                 
;       VECTOR:     If set all vector components will be returned.
;       WDF:        WDF array number to store the one dimensional arrays.  Ignored
;                   if the number of DIMENSIONS + 1 is not equal to the number of 
;                   spatial dimensions in STRUCTURE1.
; METHOD:  
;       Dimensions are removed to produce a field with reduced dimensions.
;       For example if the field of structure 1 is F1[3, 4, 6, 7, 8, 9]
;       and dimension = [2, 3, 5] the new field will have dimension 
;       F2[3, 7, 9] and the values of the missing dimensions will be stored
;       in the dataset comment only.
;
;       The index value is determined using the MIN function to determine the cut plane.
;          If X is the array of values for the dimension and VALUE is the desired value, 
;          the array is cut along the index K given by:
;            junk = min(X - Value, /absolute, K)
;          If INDEX is set, then K = VALUE-1 (since IDL uses a zero initial offset)
;
; EXAMPLE:
;	Reduce structure 4 (a 3D structure) a two dimensional structure by 
;       slicing along the second dimension at 0.4.
;       Save the sliced structure in g.
;         g = SLICEALL (4, 2, 0.4)
;
;       Save a line out of the 3D structure p in wdf array 8 at y = 4 and z = .4
;         w = sliceall(p, [2, 3], [4, .4], wdf = 8)
;
; MODIFICATION HISTORY:
; 	Written by:	Paul Mix, October 25, 2005.
;-
 Function sliceall, ain, dimension, value, wdf = wdf, index = index
;
; insert common blocks
;
@qscom.dat
;
@qsstrcom.dat
;
;       return on an error
      on_error, 1
;
      npar = n_params()
      if n_elements (ain) eq 0 then begin
 print, '% Format:', $
' Str_2 = SLICEALL (Str_1, DIMENSION, VALUE, /INDEX, WDF = WDF)'
        return, 0.0
      endif
      if n_elements (structinit) eq 0 then qsstruct_init
      a = ain
      b = size(a)
      if b(b(0)+1) ne 8 then begin
        if a gt 0 and a le maxstructure then begin
          s2i, a, a
          b = size (a)
          if b(b(0)+1) ne 8 then begin
            print, '% SLICEALL: ', $
   'First parameter must be a structure or a structure array number'
            return, 0.0
          endif
        endif  else begin
          print, '% SLICEALL: ', $
   'First parameter must be a structure or a valid structure array number'
          return, 0.0
        endelse
      endif
      if a.type ne 1 then begin
        print, '% SLICEALL: Only type 1 field structures may be sliced with SLICEALL'
        return, 0.0
      endif
      if a.block ne 1 then begin
        print, '% SLICEALL: Only single block structures may be sliced with SLICEALL'
        return, 0.0
      endif
      if n_elements(dimension) eq 0 then begin
        print, '% SLICEALL: No dimensions have been specified.  Returning original array.'
        return, a
      endif
      ndim = a.ndim
      nd = n_elements (dimension)
      if ndim -nd gt 4 then begin
        print, '% SLICEALL: This routine must reduce the number of dimensions to at least 4.'
        return, 0.0
      endif
      if nd gt 1 then begin
;       sort and take the unique elements
        isort = sort(dimension)
        dimension = dimension[uniq(dimension[isort])]
        if n_elements(dimension) lt nd then begin
          print,  '% SLICEALL: Unique dimensions must be provided.'
          return, 0.0
        endif
      endif
      if n_elements (value) ne nd then begin
        print, '% SLICEALL: Number of values must be the same as the number of dimensions.'
        return, 0.0
      endif
      value = value[isort]
      labs = ['X', 'Y', 'Z', 'T', 'U', 'V']
      labels = [a.x1lab, a.x2lab, a.x3lab, a.x4lab]
      if ndim gt 4 then labels = [labels, a.x5lab, a.x6lab]
      ok = where(strlen(labels) eq 0, nok)
      if nok gt 0 then labels[ok] = labs[ok]
      clab = a.clab + ' - SLICE={'
;     find if there are any degenerate axes.
      ok = where (a.size eq 1, nok)
      if nok gt 0 then begin
        dimension = [dimension, ok+1]
        value = [value, replicate(0.0, nok)]
        isort = uniq(dimension, sort(dimension))
        dimension = dimension[isort]
        value = value[isort]
        nd = n_elements (dimension)
      endif

      ssize = a.size
      ssize[dimension -1] = 1
      okdim = where( ssize gt 1, newdim)
      if newdim eq 0 then begin
        print, '% SLICEALL: At least one dimension must be left'
        return, 0
      endif
      newsize = ssize[okdim]
      j = make_array(ndim, value = ' *')
      for i = 0, nd-1 do begin
        x = get_array(a, dimension[i])
        if keyword_set(INDEX) then $
          k = ((value[i]-1)> 0) < (n_elements(x)-1) $
        else junk = min(x-value[i], /absolute, k)
        value[i] = x[k]
        j[dimension[i]-1] = ' '+strtrim (k, 2)
        clab = clab+labels[dimension[i]-1]+':'+strtrim(value[i], 2)
        clab = ( i eq (nd-1)) ? (clab+'}') : (clab+', ')
      endfor     
      subs = '['+j[0]
      for i =1, ndim-1 do subs = subs+', '+j[i]
      subs = subs + ']' 
      dimlab = ['xlabel', 'ylabel', 'zlabel', 'tlabel' ]
      dxlab = ['dx', 'dy', 'dz', 'dt']

      x1 = 0 & x2 = 0 & x3 = 0 & x4 = 0 & x5 = 0 & x6 = 0 & x7 = 0

      x1lab = '' & x2lab = '' & x3lab = '' & x4lab = '' & x5lab = '' & x6lab = ''
      x7lab = ''
      dx1 = 0 & dx2 = 0 & dx3 = 0 & dx4 = 0
      str = 0
      nvect = a.nvect
      command = 'arr2str, str, /str,x1, x2, x3, tlab = "Sliceall Field", file = a.file' $
                 + ', clab = clab, v1lab=a.v1lab'
      if nvect gt 1 then command = command+', v2lab=a.v2lab'
      if nvect gt 2 then command = command+', v3lab=a.v3lab' 
      for i = 1, nvect do begin
        v  = get_array(a, -i)
        v = reform(v, a.size, /overwrite)
        junk = execute ('x'+strtrim(i, 2)+' = reform(v'+subs+', newsize, /overwrite)')
      endfor
      if newdim eq 1 then begin
        xx = get_array(a, okdim[0]+1)
        aret = get_maxwdf()-nvect
        clabel = (strlen(a.v1lab[0]) eq 0) ? clab:  a.v1lab+', '+ clab
        i2w, xx, x1, aret, clab = clabel, /xydata
        if nvect gt 1 then begin
          clabel = (strlen(a.v2lab[0]) eq 0) ? clab:  a.v2lab+', '+ clab
          i2w, xx, x2, aret+1, clab = clabel, /xydata
          if nvect gt 2 then begin
            clabel = (strlen(a.v3lab[0]) eq 0) ? clab:  a.v3lab+', '+ clab
            i2w, xx, x3, aret+2, clab = clabel, /xydata
          endif
        endif
        return, aret
      endif
      dxnames = ['dx', 'dy', 'dz', 'dt', 'du', 'dv']
      labnames = ['xlabel', 'ylabel', 'zlabel', 'x4label', 'x5label', 'x6label']

      for i = 1, newdim do begin
        v =get_array(a, okdim[i-1]+1)
        junk = execute('x'+strtrim(i+nvect, 2)+' = v')
        if i+nvect gt 3 then command = commend+', x'+strtrim(i+nvect, 2)
        junk = execute(dxnames[okdim[i-1]] +'= a.dx'+strtrim(i, 2))
        junk = execute(labnames[okdim[i-1]]+'= a.x'+strtrim(i, 2)+'lab')
        command = command +', '+dxnames[okdim[i-1]] +'= '+dxnames[okdim[i-1]]
        command = command +', '+labnames[okdim[i-1]]+'= '+labnames[okdim[i-1]]
      endfor
;     print, command
      junk = execute(command)
      return, str
      end
