; ------------------------------------------------------------------------
; $Id: solid.pro,v 1.7 2009/01/29 17:52:27 lpmix Exp $
; ------------------------------------------------------------------------
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;

pro solid, file, ncon, nodiel=nodiel, color=color, fintensity=fintensity, $
           bintensity=bintensity, trans=trans, axis_add = axis_add, skip = skip, $
           model = model, retrieve = retrieve, bcolor = bcolor, $
           spawn = spawnit, include = include, block = block, change = changein
;
;
;+
; NAME:
;       SOLID
; PURPOSE:
;       This procedure loads datasets from a PFF geometry file and
;       uses PFIDL's solid modeling tools to render a 3D display.
;       The model is stored for future reference and can be retrieved
;
;       See also "PLOTCON, SOLID_MODEL = omodel" ; omodel ne 0
;
; CATEGORY:
;       Solid modeling
;
; CALLING SEQUENCE:
;       SOLID [, FILE]
; or    SOLID , START [, NCON]
;
; INPUTS:
;       File: File name
;        If file name = 0, dialog_pickfile is called.
;        Default: FILE = 'qsgeo.pff'
;                 If no file called 'qsgeo.pff' is present,
;                   DIALOG_PICKFILE is called.
;       Note:  SKIP, NODIEL, INCLUDE keywords apply only to FILE input
;       or
;       START : Array of structure array numbers for the conductors
;               or Initial structure array value
;       NCON  : Number of structure arrays
;               NOTE:  NCON is ignored for START is an array
;               Default: NCON = 1
;       Note:   SKIP, NODIEL, INCLUDE keywords do not apply to specified structures
;
; KEYWORDS PARAMETERS:
;       AXIS_ADD: If set, an axis will be added to the 3D model
;                 If AXIS_ADD is a structure, it will be
;                  passed to ADD_AXIS as an _EXTRA keyword.
;       BLOCK:    If set, the solid model is displayed in a blocking widget
;                 If undefined, then BLOCK defined by the following:
;                       DEFSYSV, "!callable_idl", exists = BLOCK
;       NODIEL:   Do not render dielectric objects (vacuum is always skipped)
;       INCLUDE:  Datasets to include.
;                 Default is datasets 3 to the last dataset
;       SKIP:     Dataset name or number to skip
;                 If skip is a string, it will be used as a search string.
;       COLOR:    Array of colors for the objects
;       BCOLOR:   Array of colors for the rear of the objects
;                 Default:  BCOLOR = COLOR
;       FINTENSITY: Foreground color intensity for each object
;            Note:  FINTENSITY may be a scalar or an RGB triplet
;                   Default:  FINTENSITY = 1.0
;       BINTENSITY: Background color intensity for each object
;                   Default:  BINTENSITY = 0.6
;       TRANS:    Transparency flag for the objects (0 = off, 1 = on)
;       MODEL:    If present, the model is returned
;       RETRIEVE: If set, no pff file will be opened and the last model
;                 returned in MODEL
;       SPAWN:    If set, spawn to another IDL process
;       CHANGE:   If present, CHASTR will be called to modify the structures
;                 before the solid model is generated.  Every structure will
;                 have this change applied.
;                 CHANGE may be a string or a structure.
;                 If CHANGE is a string, it will be converted to a structure 
;                   using EXECUTE.
;                 The structure will be modified by:
;                   CALL_PROCEDURE, 'CHASTR', STR, _EXTRA = CHANGE
;
; TECHNIQUE:
;       Solid model is generated and returned in the MODEL keyword.
;       Two models will be stored in a common block
;       After two models are stored, additional calls to SOLID will result
;       in the destruction of the first solid model, the second model will
;       be moved to the first model location and the current model will be
;       stored.
;       The current model can be retrieved with the RETRIEVE keyword.
;
; Examples:
;       Make a solid model of datasets 3, 4, 6 in the qsgeo.pff file in the
;         current directory.
;         SOLID, include=[3, 4, 6]
;
;       Make a solid model with datasets 3 and 4 and multiply the second 
;         dimension by 0.125, and the first dimension by 100.
;         SOLID, INCLUDE = [3,4], CHANGE = 'MYDIM = 0.125, mxdim = 100'
;      or 
;         SOLID, INCLUDE = [3,4], CHANGE = {MYDIM : 0.125, MXDIM:100}
;
;       Make a solid model of structure array numbers: 3, 4, 5.
;         SOLID, [3, 4, 5]
; or      SOLID, 3, 3
;
; MODIFICATION HISTORY:
;       Original version, M. F. Pasik, April 3, 2000
;       Added color options, TDP, 02/14/02
;       Added common block to save the model, LPM, 06/18/04
;       Added routine to PFIDL_2D, LPM, 06/18/04
;       Added CHANGE keyword from suggestion and work of D B Seidel, 01/28/09
;       Close PFF file when return.
;-
;
common solid_model_common, last_model, current_model
on_error, 1
if keyword_set(retrieve) then begin
  if obj_valid(current_model) then begin
    if arg_present(model) then model = current_model else begin
      print,'% SOLID: Keyword(RETRIEVE) MODEL must be a variable'
      help, model
    endelse
  endif else print, '% SOLID: Keyword(RETRIEVE) No model to return'
  return
endif
lcstr = 0
if (n_params() EQ 0) then begin
  file='qsgeo.pff'
  if (file_info(file)).exists eq 0 then file =0
endif else if  size(/type, file) le 4 then begin
  str = round(file)
  lcstr = (min(str) gt 0) ? 1 : 0
  if not lcstr then print, $
    '% SOLID: Structure must be greater than zero - Select a file'
endif
if lcstr eq 0 then begin
  fname = file[0]
  b = size(fname)
  if b(1) eq 7 then begin
    lcfile = 1
    vers = strupcase(strmid(!version.os_family, 0, 3))
    if (vers eq 'UNI') then begin
;   get system variable substitution for unix systems
      temp = fname
      FULFNAM, temp, fname
    endif
    fname = file_expand_path(fname)
    if !version.os eq 'linux' then resolve_it, fname
    info = file_info(fname)
    if (info.exists + info.read) ne 2b then begin
      print, '% SOLID: File does not exist with read permission'
      print, '         File Name = ', fname
      print, '         File Exists =', info.exists
      print, '         Read Permission =', info.read
      ier = -2
      return
    endif
    openpff, fname, error = error, ndset=nd
  endif else begin
    fname = dialog_pickfile(/must_exist, /read)
    if strlen(fname) lt 1 then begin
      print, '% SOLID: No File was Selected - RETURNING'
      return
    endif
    if !version.os eq 'linux' then resolve_it, fname
    openpff, fname, error = error, ndset=nd
  endelse
  if error ne 0 then begin
    print, '% SOLID: Error opening PFF file'
    print, '   File: ', fname
    return
  endif
endif else begin
  n = n_elements(str)
  if n eq 1 then ncond = (n_elements(ncon) eq 0) ? 1: abs(ncon) $
    else ncond = n
    if ncond gt n then str = str[0]+indgen(ncond)
  jj = strvalid(str, /valid, npoint = nok)
  if nok eq 0 then begin
    print, '% SOLID: No valid datasets specified - RETURNING'
    return
  endif
endelse

keepdiel = (keyword_set(nodiel)) ? 0 : 1

if n_elements(changein) ne 0 then begin
  change = changein
  sav_str = get_maxstr()
  if n_elements(change) ne 1 then begin
    print, '% SOLID: CHANGE keyword must be a scalar string or structure'
    help, change
    if lcstr eq 0 then closepff
    return
  endif
  typ_change = size(change, /type)
  lc_change = 1
  if typ_change eq 7 then begin
    change = byte(change)
    ok = where (change eq  (byte('='))[0], nok)
    if nok eq 0 then begin
      print,  '% SOLID: CHANGE keyword string must be of the form:'
      print,  '   Keyword = value, keyword = value, ..., keyword = value'
      print,  '   or a structure'
      if lcstr eq 0 then closepff
      return
    endif else change[ok]= (byte(':'))[0]
    junk = execute ('change = {'+string(change)+'}')
    if junk eq 0 then begin
      print,  '% SOLID: Unable to convert CHANGE string to a structure'
      if lcstr eq 0 then closepff
      return
    endif
  endif else if typ_change ne 8 then begin
    print, '% SOLID: CHANGE keyword must be a scalar string or structure'
    if lcstr eq 0 then closepff
    return
  endif
endif else lc_change = 0

; color
; Set conductor colors: Default is to just cycle through 1 - 7

fcolor =  (n_elements(color) eq 0) ? 1+indgen(7) : color
ncol = n_elements(fcolor)
if n_elements(bcolor) eq 0 then bcolor = fcolor else begin
  if n_elements (bcolor) lt fcolor then begin
    print, $
  '% SOLID: Number of back colors must match the number of front colors'
   if lcstr eq 0 then closepff
    return
  endif
endelse

; Front intensity

if n_elements(fintensity) eq 0 then begin
  fintens = 1.0 + fltarr(ncol)
endif else if n_elements(fintensity) eq 1 then begin
  fintens = fintensity + fltarr(ncol)
endif else if n_elements(fintensity) eq ncol then begin
  fintens = fintensity
endif else begin
  print,'% SOLID: # elements of fintensity inconsistent with color'
  if lcstr eq 0 then closepff
  return
endelse

; Back intensity

if n_elements(bintensity) eq 0 then begin
  bintens = 0.6 + fltarr(ncol)
endif else if n_elements(bintensity) eq 1 then begin
  bintens = bintensity + fltarr(ncol)
endif else if n_elements(bintensity) eq ncol then begin
  bintens = bintensity
endif else begin
  print,'% SOLID: # elements of bintensity inconsistent with color'
  if lcstr eq 0 then closepff
  return
endelse

; transparency

if n_elements(trans) eq 0 then trans = 0
ntra = n_elements(trans)

if lcstr eq 0 then begin
  if keyword_set(skip) then begin
    if size(skip, /type) eq 7 then get_dir, skip[0], dataset = dskip $
      else dskip= round(skip)
    nskip = n_elements(dskip)
    if nskip gt 1 then $
      dskip = dskip[uniq(dskip, sort(dskip))]
  endif else nskip = 0
  if keyword_set(include) then begin
    if size(include, /type) eq 7 then get_dir, include[0], dataset = incl $
       else incl = include
    nincl = n_elements(incl)
      if nincl gt 1 then $
       incl = incl[uniq(incl, sort(incl))]
  endif else incl = indgen(nd-2)+3
  jj = incl
  if nskip gt 0 then begin
    for n = 0, nskip-1 do begin
      ok = where (dskip[n] eq jj,complement = good, nok)
      if nok gt 0 then begin
        if good[0] ge 0 then jj = jj[good] else begin
          print, '% SOLID:  NO datasets found after skip'
          if lcstr eq 0 then closepff
          return
        endelse
      endif
    endfor
  endif
  nok = n_elements(jj)
  if nok eq 0 then begin
    print, '% SOLID: No valid PFF datasets specified - RETURNING'
    if lcstr eq 0 then closepff
    return
  endif else str = get_maxstr()
endif

nmodel = 0
omodel= obj_new('idlgrmodel')
for nn=0,nok-1 do begin
  n = jj[nn]
  cindex = (n) mod ncol
  fcol   = fix( fintens[cindex] * getcolor((fcolor[cindex])) )
  bcol   = fix( bintens[cindex] * getcolor((bcolor[cindex])) )
  lcnew = 1
  if lcstr eq 0 then begin
    ds = n
    readhdr, ds, pfftype=pfftype, aptype=apptype, clabel=label, /quiet
;
    if (pfftype EQ 5) then begin
;   allow filtering of dielectrics (always omit vacuum)

      if ( ((apptype EQ 4) AND (keepdiel) AND  $
            (strpos(label,'vacuum') EQ -1)) OR $
           (apptype EQ 2) ) then begin

        print, form='(a12,a,4x,3i4,4x,3i4)', label, ': fcol, bcol: ', fcol, bcol
        readstr, str, ds, /solid
      endif else lcnew = 0
    endif else lcnew = 0
  endif else str = n
  if lcnew gt 0 then begin
    if lc_change then begin
      if lcstr then begin
        xfrstr, str, sav_str
        str = sav_str
      endif
      call_procedure, 'chastr', str, _extra = change
    endif
    plotcon_solid, str, ocond, color=fcol, back=bcol

;  Make object semi-transparent

    if obj_valid (ocond) then begin
      if trans[(n) mod ntra] eq 1 then begin
        ostrip = stipple_obj()
        ocond -> setproperty, shading=1, fill_pattern=ostrip[12]
      endif
      nmodel +=1
      omodel -> add, ocond
    endif
  endif
endfor
if lcstr eq 0 then closepff
if nmodel eq 0 then begin
  Print, '% SOLID: No conductors added to model'
  obj_destroy, omodel
  return
endif
if keyword_set(axis_add) then $
  omodel=  (size(axis_add, /type) eq 8) ? $
    add_axis(omodel, _extra= axis_add) : add_axis(omodel)
if arg_present(model) then model= omodel
if keyword_set(spawnit) then begin
  if obj_valid(omodel) then my_xobjview, omodel
endif else begin
  if n_elements(block) eq 0 then $
          defsysv, "!callable_idl", exists = block
  if obj_valid(omodel) then xobjview, omodel, block = block
endelse

if obj_valid(last_model) then obj_destroy, last_model
if n_elements(current_model) gt 0 then last_model = temporary(current_model)
current_model= temporary(omodel)
end
