; ------------------------------------------------------------------------ 
; $Id: split_colors.pro,v 1.3 2007/06/06 20:09:37 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
 pro split_colors, structure, vector, bottom= bottom, file = file, top =top, $
       levels = levels, _extra = extrastuff

;
;+
; NAME:
;       SPLIT_COLORS
;
; PURPOSE:
;       Find the minimum and maximum of a vector and build a compound color
;       table for positive and negative values.
;
; CATEGORY:
;       Graphics
;
; CALLING SEQUENCE:
;       Split_Colors, Structure, Vector, BOTTOM = bottom, TOP = top, 
;                     LEVELS = levels, _extra = extrastuff
;
; INPUTS:
;       Structure - Structure array number or idl field Structure.
;   or  LEVELS    - Minimum and maximum for the compound color table.
;
; OPTIONAL INPUTS:
;       Vector    - Vector component desired if Structure contains more than
;                   one vector
;       Bottom    - Color table value used for values below zero
;                   Default = 7
;       Top       - Color table value used for values above zero
;                   Default = 1
;       File      - File where colors are stored (see LOADCT)
;	
; EXTRA KEYWORD PARAMETERS (Sent to MINSTR):
;       XRANGE    - Range of the first dimension for the minimum search.
;                   Default:  All X values
;       YRANGE    - Range of the second dimension for the minimum search.
;                   Default:  All Y values
;       ZRANGE    - Range of the third dimension for the minimum search.
;                   Default:  All Z values
;       LOCATION  - Position where minimum occurs
;       PMINIMUM  - Same as LOCATION
;       MAXIMUM   - Maximum val in the range defined.
;       PMAXIMUM  - Position where MAXIMUM occurs.
;
; OUTPUTS:
;       The split color table is loaded
;
; OPTIONAL OUTPUTS:
;       If levels is undefined or a scalar, the minimum and maximum values are
;         returned.
;
; EXAMPLE:
;       Load a split color table for the electric field structure E
;         split_colors, e
;
;       Load a split color table for the Z component of the field in structure 4
;       Return the min and max in lev.
;         split_colors, 4, 3, lev = lev
;
;       Load a split color table for a range of [-100, 200]
;         Use color table 7 for the negative values and 1 for the top values
;         split_colors,  lev = [-100, 200], b=7, t =1
;
; MODIFICATION HISTORY:
;       Written by:     L. P. Mix, June 20, 2000
;-
  on_error, 1
@loadxyct_common.dat
;

  nparam = n_params()
  nlev = n_elements(levels)
if (nparam eq 0) and (nlev lt 2) then begin
  print,'% Format: Split_Colors, Structure, Vector, BOTTOM = bottom, TOP = top,'
  print,'             LEVELS = levels, _extra = extrastuff '
  return
endif
;
if nlev lt 2 then begin

  If n_elements (structure) eq 0 then begin
    print,'% Split_colors: Structure or structure array number must be provided'
    return
  endif

  ma = minstr(structure, vector, _extra = extrastuff, max = mb, ier =ier)
  if ier ne 0 then begin
    print, '% SPLIT_COLORS: Error getting Structure limits'
    return
  endif
  levels =[ma, mb]
endif else levels =[min(levels, max=mm), mm]
 
if n_elements(bottom) eq 0 then bottom = 7 else bottom = bottom[0]
if n_elements(top) eq 0 then top = 1 else top = top[0]

if levels[0] ge 0 then begin
  loadct, top, file = file
  print, '% SPLIT_COLORS: Only positive values present'
  return
endif

loadct, bottom, file = file
if levels[1] le 0 then begin
  print, '% SPLIT_COLORS: Only negative values present'
  return
endif

ssiz = !d.table_size

if ssiz lt 12 then begin
  print, '% SPLIT_COLORS: Inadequate number of colors: ', ssiz
  return
endif

nm = ssiz-1
nc = nm-9
ttop = nm-1

nn = round(levels[0]*nc/(levels[0]-levels[1]))
np = nc -nn
tvlct, /get, rb, gb, bb
loadct, top, file = file
tvlct, /get, rt, ggt, bt

r = rt
g = ggt
b = bt
      r(1:8) = [255,   0,   0, 255, 255,   0, 255, 255]
      g(1:8) = [  0, 255,   0,   0, 125, 255, 255, 255]
      b(1:8) = [  0,   0, 255, 255,   0, 255,   0, 255]
if n_elements (invertct_flag) gt 0  then if invertct_flag ne 0 then begin
        c = r[0] & r[0] = r[nm] & r[nm] = c
        c = g[0] & g[0] = g[nm] & g[nm] = c
        c = b[0] & b[0] = b[nm] & b[nm] = c
      endif

nind = ttop- indgen(nn)*nc/(nn-1)
r[9:8+nn]=rb[nind]
g[9:8+nn]=gb[nind]
b[9:8+nn]=bb[nind]

pind = indgen(np)*nc/(np-1)
pind = pind + (ttop -pind[np-1])
r[9+nn:ttop] = rt[pind]
g[9+nn:ttop] = ggt[pind]
b[9+nn:ttop] = bt[pind]

tvlct, r, g, b

end
