; ------------------------------------------------------------------------ 
; $Id: stack_xyz.pro,v 1.3 2007/06/06 20:09:37 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
pro stack_xyz, strarray, nstr, pad = pad, width = width, out = out, $
      xaxis = xaxis, yaxis = yaxis, zaxis = zaxis 
;+
; NAME:
;       STACK_XYZ
;
; PURPOSE:
;       Put a series of Structures together in the X, Y, or Z direction.
;
; CALLING SEQUENCE:
;
;       STACK_XYZ, STRARRAY, NSTR, PAD = pad, WIDTH = WIDTH, OUT = OUT
;         XAXIS = xaxis, YAXIS = yaxis, ZAXIS = zaxis
;
; INPUTS:
;       STRARRAY - Structure array number, array of structure array numbers, 
;                   or string matching a structure comment 
;       NSTR     - If STRARRAY is a single value, then that value and the 
;                  next (NSTR -1) values will be used.
;
; KEYWORD PARAMETERS:
;
;       PAD      - Fraction used to pad between images.
;                  This number with typically be ~0.01 for most applications.
;                  The space will be PAD * width of the added image.
;       WIDTH    - X, Y, or Z increment to use between all images.
;                  If pad is not specified and width is not specified, then
;                  width = 0.01* width of the first image.
;       OUT      - Output structure array number.
;                  Default is (get_maxstr()-1)
;       XAXIS    - Stack along the X axis (This is not checked.)
;       YAXIS    - Stack along the Y axis
;       ZAXIS    - Stack along the Z axis
;         NOTE:  If no axis is specified, then XAXIS =1
;                actual code:
;                if keyword_set(zaxis) then stack_axis =3 else $
;                  if keyword_set(yaxis) then stack_axis =2 else stack_axis = 1
; OUTPUTS:
;       The resulting composite sturcture will be returned.
;
; RESTRICTIONS:
;       Only structures supported by "JOINSTR" can be combined.
;       The maximum structure is used as a work array.
;
; PROCEDURE:
;       In the direction of stacking, coordinate will be will be modified to 
;       be adjacent to the previous structure with a user specified gap.
;       The procedure puts different structures in different blocks.  This 
;       allows the exact coordinates to be maintained.
;
; EXAMPLE:
;       Combine structures 22, 23 in the x direction and store the result in 30
;         STACK_XYZ, 22, 2, out =30
;    or
;         STACK_XYZ, [22, 23] , out =30
;
; MODIFICATION HISTORY:
; 	Written by:	L. P. Mix, April 24, 2002
;-
;
on_error, 1
;
      nparams = n_params()
      if nparams eq 0 then begin
        print,'% Format: ', $
           'STACK_XYZ, STRARRAY, NSTR, PAD = pad, WIDTH = WIDTH,'
        print,'           OUT = OUT, /XAXIS, /YAXIS, /ZAXIS'
        return
      endif
;
;     determine if strarray is specified
;
      nelstr = n_elements(strarray)
      if nelstr eq 0 then begin
        print, '% STACK_XYZ: Structures must be defined'
        return
      endif
;
      a = strarray
      st = size(str, /type)
      if st eq 7 then begin
        a = get_match(a, /struct, nel = nel)
        if nel eq 0 then begin
          print,'% STACK_XYZ: No values found.  Search string: ', strarray
          return
        endif
      endif else begin
        a = strvalid(a, /valid, npoint = nel, /quiet)
        if nel eq 0 then begin
          print,'% STACK_XYZ: No valid values in strarray.
          return 
        endif
      endelse
      if nel eq 1 then begin
        if n_elements(nstr) gt 0 then begin
          a = a + lindgen(nstr)
          a = strvalid(a, /valid, npoint = nel, /quiet)
        endif
      endif
      work = get_maxstr()
      if n_elements(out) eq 0 then out= work-1 else $
        out= get_save_str(out, work-1)
      xfrstr, a[0], out
      if nel eq 1 then return

      if keyword_set(zaxis) then stack_axis =3 else $
      if keyword_set(yaxis) then stack_axis =2 else stack_axis = 1

      xrange = [min(get_array(out, stack_axis, /alldata), max = m), m]
      if n_elements(pad) gt 0  then begin
        pad = pad[0]
        lcrelative = 1
      endif else begin
        lcrelative = 0
        if n_elements(width) eq 0 then $
          width = 0.01*(xrange[1]-xrange[0]) else $
          width = width[0]
      endelse
      
      for i = 1, nel -1 do begin
        xfrstr, a[i], work
        axis_data = get_array(work, stack_axis, /alldata)
        newrange = [min(axis_data, max = m), m]
        if lcrelative eq 0 then start = xrange[1]+ width else $
          start = xrange[1] + pad*(xrange[1]-xrange[0])
        addit = start - newrange[0]
        xrange = [xrange[0], newrange[1]+addit]
        axis_data = axis_data + addit
        put_array, work,axis_data, stack_axis, /alldata
        joinstr, out, work
      endfor
      end
