; ------------------------------------------------------------------------
; $Id: strinfo.pro,v 1.9 2010/09/17 20:51:05 lpmix Exp $
; ------------------------------------------------------------------------
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
;+
; NAME:
;       STRINFO
;
; PURPOSE:
;       This procedure returns the parameters for a structure
;       This is similar if not identical to STRLIMITS
;
; CALLING SEQUENCE:
;       STRINFO, STR [, TYPE = type] [, FILE = file] [, NDIM = ndim]
;            [, CLABEL = clabel] [, TLABEL= tlabel] [, X4LABEL= x4label] 
;            [, XLABEL = xlabel] [, YLABEL= ylabel] [, ZLABEL = zlabel]
;            [, XRANGE = xrange] [, YRANGE = yrange] [, ZRANGE = zrange] 
;            [, TRANGE = trange] [, RANGE = range] [, SIZE = size]
;            [, NBLOCK = nblock] [, NATTR = nattr] [, NVERT = nvert]
;            [, DX = dx] [, DY = dy] [, DZ = dz] [, DT = dt]
;            [, V1 = v1] [, V2 = v2] [, V3 = v3] [, V4 = v4]
;            [, V5 = v5] [, V6 = v6] [, V7 = v7] [, V8 = v8]
;            [, AV1 = av1] [, AV2 = av2] [, AV3 = av3] [, AV4 = av4]
;            [, AV5 = av5] [, AV6 = av6] [, AV7 = av7] [, AV8 = av8]
;
; INPUTS:
;       STR:    Valid structure number, PFIDL structure, or search string 
;               for the structure comment
;	
; KEYWORD PARAMETERS:
;       Note: Default values are 0 except for CLABEL
;       TYPE:   Structure type 
;       FILE:   File name for the data
;       NDIM:   Number of dimensions
;       CLABEL: Structure Comment Label
;       TLABEL: Structure Type Label
;       XRANGE:	Range of X-values, or first dimension
;       YRANGE:	Range of Y-values, or seconc dimension
;       ZRANGE:	Range of Z-values, or third dimension
;       TRANGE: Range of T-values, or fourth dimension
;       XLABEL: Label for X-data, or first dimension
;       YLABEL: Label for Y-data, or second dimension
;       ZLABEL: Label for Z-data, or third dimension
;       X4LABEL: Label for T-data, or fourth dimension
;       RANGE:  Structure RANGE variable with spatial ranges by block
;       NBLOCK: Number of blocks
;       SIZESP: If structure has SIZE as a tag, return the values
;               SIZESP is an array of dimensions for each spatial dimension
;               SIZESP = SIZESP [NDIM, NBLOCK]
;               If SIZE is not a tag name then SIZESP =0
;       NATTR:  Number of attributes or vector components
;       NVERT:  Number of particles in particle datasets
;       DX:     DX for first dimension, ZERO if non-uniform or unknown
;       DY:     DY for second dimension, ZERO if non-uniform or unknown
;       DZ:     DZ for third dimension, ZERO if non-uniform or unknown
;       DT:     DT for fourth dimension, ZERO if non-uniform or unknown
;       V1:     Range of first vector component or attribute
;       V2:     Range of second vector component or attribute
;       V3:     Range of third vector component or attribute
;       V4:     Range of fourth vector component or attribute
;       V5:     Range of fifth vector component or attribute
;       V6:     Range of sixth vector component or attribute
;       V7:     Range of seventh vector component or attribute
;       V8:     Range of eighth vector component or attribute
;       AV1:    Average and Standard Deviation 
;               of first vector component or attribute
;       AV2:    Average and Standard Deviation 
;               of second vector component or attribute
;       AV3:    Average and Standard Deviation 
;               of third vector component or attribute
;       AV4:    Average and Standard Deviation 
;               of fourth vector component or attribute
;       AV5:    Average and Standard Deviation 
;               of fifth vector component or attribute
;       AV6:    Average and Standard Deviation 
;               of sixth vector component or attribute
;       AV7:    Average and Standard Deviation 
;               of seventh vector component or attribute
;       AV8:    Average and Standard Deviation 
;               of eighth vector component or attribute
;
; OUTPUTS:
;       Specified keyword parmeters are returned
;
; PROCEDURE:
;       The specified structure is retrieved and the keyword parameters are 
;       extracted.  The average and standard deviation are calculated 
;         Average = Total(V, /double) / N ; where N is the number of elements in V
;         SDEV    = SQRT ((Total(V^2) - N*AVERAGE^2)/ (N-1))
;
; EXAMPLE:
;       Determine the xrange of structure, 9, and store in XR.
;         STRINFO, 9, XRANGE = XR
;       Determine the average and standard deviation of structure 14, Vector 1
;         STRINFO, 14, AV1 = value
;         AVERAGE = Value[0] & SDEV = Value[1]
;
; MODIFICATION HISTORY:
;       Written by:     Paul Mix, February 23, 2006
;       5/25/06         LPM, Added the AVn keywords
;
;-

Function LPM_AVERAGE_SDEV, V
COMPILE_OPT hidden
 on_error, 1

   N = N_elements(V)
   if n lt 2 then begin
     print, '% LPM_AVERAGE_SDEV: Number of elements less than 2'
     return, 0
   endif
   av = Total(V, /Double)
   sd = sqrt(( Total(v*v, /double)-av*av/n)/(n-1))
   return, [av/n, sd]
end

PRO STRINFO,  STRIN, TYPE = type , file = file, range = totrange, $
             CLABEL = clabel, trange = trange, tlabel = tlabel, $
             XRANGE = xrange, YRANGE = yrange, ZRANGE = zrange, $
             NDIM = ndim, nblock = nblock, sizesp = sizesp, nattr = nattr, $
             XLABEL = xlabel, YLABEL = ylabel, ZLABEL = zlabel, X4LABEL = x4label, $
             nvert = nvert, dx = dx, dy = dy, dz = dz, dt = dt, $
             AV1 = av1, AV2 = av2, AV3 = av3, AV4 = av4, $
             AV5 = av5, AV6 = av6, AV7 = av7, AV8 = av8, $
             V1 = v1, V2 = v2, V3 = v3, V4 = v4, $
             V5 = v5, V6 = v6, V7 = v7, V8 = v8
on_error, 1
    
   if n_params() eq 0 then begin
     print,'% Format:', $
            ' STRINFO, STR [, TYPE = type] [, CLABEL = clabel] [, NDIM = ndim]'
     print, '    [, TLABEL = tlabel] [, FILE = file] [, XRANGE = xrange]'
     print, '    [, YRANGE = yrange] [, ZRANGE = zrange] [, TRANGE = trange]'
     print, '    [, XLABEL = xlabel] [, YLABEL = ylabel] [, ZLABEL = zlabel]'
     print, '    [, X4LABEL = x4label]'
     print, '    [, RANGE = range] [, DX = dx] [, DY = dy] [, DZ = dz]'
     print, '    [, NBLOCK = nblock] [, SIZESP = sizesp] [, NATTR = nattr]'
     print, '    [, DT = dt] [, V1 = v1] [, V2 = v2] [, V3 = v3] [, V4 = v4]'
     print, '    [, V5 = v5] [, V6 = v6] [, V7 = v7] [, V8 = v8] [, NVERT = nvert]'
     print, '    [, AV1 = av1] [, AV2 = av2] [, AV3 = av3] [, AV4 = av4]'
     print, '    [, AV5 = av5] [, AV6 = av6] [, AV7 = av7] [, AV8 = av8]'
     return
   endif
   type =0
   clabel = ''
   tlabel = ''
   file = ''
   xrange = 0
   yrange = 0
   zrange = 0
   trange = 0
   nvert = 0
   v1 = 0
   v2 = 0
   v3 = 0
   v4 = 0
   v5 = 0
   v6 = 0
   v7 = 0
   v8 = 0
   av1 = 0
   av2 = 0
   av3 = 0
   av4 = 0
   av5 = 0
   av6 = 0
   av7 = 0
   av8 = 0
   if n_elements(strin) eq 0 then begin
     print, '% STRINFO: Structure must be defined'
     return
   endif else str = strin
   
   a = get_valid_structure(str, 'STRINFO', lcstr=lcstr, ierror=ier)

   if IER ne 0 then begin
     print, '% STRINFO: Returning immediately'
     return
   endif
   names = tag_names(a)
   clabel = a.clab
   ok = where(strpos(names, 'SIZE') ge 0, nok)
   sizesp = (nok gt 0) ? a.size : 0L
   ok = where(strpos(names, 'TYPE') ge 0, nok)
   if nok gt 0 then begin
     type = a.type
     valid_typ = [1, 2, 3, 4, 7, 8, 9, 10, 11, 12]
     ok = where (type eq valid_typ, nok)
     if nok eq 0 then begin
       print, '% STRINFO: Structure is not a valid type: ', strtrim(type, 2)
       return
     endif
   endif else begin
     print, '% STRINFO: This is not a PFIDL structure'
     return
   endelse

   ok  = where(strpos(names, 'BLOCK') ge 0, nok)
   if nok gt 0 then begin
     nblock = a.block
   endif else nblock = 1
   
   ndim = a.ndim
   if arg_present(file) then begin
     ok = where(strpos(names, 'FILE') ge 0, nok)
     if nok gt 0 then file = a.file
   endif
   if arg_present(tlabel) then tlabel = a.tlab
   if arg_present(dx) then begin
     ok = where(strpos(names, 'DX') ge 0, nok)
     if nok gt 0 then begin
       dx = a.dx1
       dy = a.dx2
       dz = a.dx3
       dt = a.dx4
     endif
   endif
   if arg_present(totrange) then begin
     ok = where(strpos(names, 'RANGE') ge 0, nok)
     if nok gt 0 then totrange = a.range
   endif
   if arg_present(xrange) || arg_present(yrange) || arg_present(zrange) $
     || arg_present(trange) || arg_present(nvert) || arg_present(xlabel) $
     || arg_present(ylabel) || arg_present(zlabel) || arg_present(x4label) then begin
     if type eq 2 then begin
;      particle data
       if arg_present(xrange) then xrange = [min(a.x1, max = m), m]
       if arg_present(yrange) && ndim gt 1 then yrange = [min(a.x2, max= m), m]
       if arg_present(zrange) && ndim gt 2 then zrange = [min(a.x3, max= m), m]
       if arg_present(trange) && ndim gt 3 then trange = [min(a.x4, max= m), m]
       if arg_present(xlabel) then xlabel = a.x1lab
       if arg_present(ylabel) && ndim gt 1 then ylabel = a.x2lab
       if arg_present(zlabel) && ndim gt 2 then zlabel = a.x3lab
       if arg_present(x4label) && ndim gt 3 then x4label = a.x4lab
       nvert = a.nvert
     endif else if type eq 4 then begin
;      conductor data
       if arg_present(xrange) then $
         xrange = [min((a.plane)[*, 0, *], max = m), m]
       if arg_present(yrange) && ndim gt 1 then $
         yrange = [min((a.plane)[*, 1, *], max = m), m]
       if arg_present(zrange) && ndim gt 2 then $
         zrange = [min((a.plane)[*, 2, *], max = m), m]
       if arg_present(xlabel) then xlabel = a.xlab[0]
       if arg_present(ylabel) then ylabel = a.xlab[1]
       if arg_present(zlabel) then zlabel = a.xlab[2]
     endif else begin
       range = (a.range)[*, *, 0]
       if arg_present(xrange) then $
         xrange = [min((range)[*, 0], max = m), m]
       if arg_present(yrange) && ndim gt 1 then $
         yrange = [min((range)[*, 1], max = m), m]
       if arg_present(zrange) && ndim gt 2 then $
         zrange = [min((range)[*, 2], max = m), m]
       if arg_present(trange) && ndim gt 3 then $
         trange = [min((range)[*, 3], max = m), m]
       if arg_present(xlabel) then xlabel = a.x1lab
       if arg_present(ylabel) && ndim gt 1 then ylabel = a.x2lab
       if arg_present(zlabel) && ndim gt 2 then zlabel = a.x3lab
       if arg_present(x4label) && ndim gt 3 then x4label = a.x4lab
     endelse
   endif
   valid_vector =[1, 2, 8, 9, 12]
   ok = where (type eq valid_vector, nok)
   nattr = 0
   if nok eq 0 then return
   if type eq 2 then nv = a.natt else nv = a.nvect
   nattr = nv
   if (arg_present(v1) || arg_present(v2) || arg_present(v3) $
      || arg_present(v4) || arg_present(v5) || arg_present(v6) $
      || arg_present(v7) || arg_present(v8)) then begin
      if arg_present(v1) then v1 =  [min(a.v1, max = m), m]
      if arg_present(v2) && nv gt 1 then v2 =  [min(a.v2, max = m), m]
      if arg_present(v3) && nv gt 2 then v3 =  [min(a.v3, max = m), m]
      if arg_present(v4) && nv gt 3 then v4 =  [min(a.v4, max = m), m]
      if arg_present(v5) && nv gt 4 then v5 =  [min(a.v5, max = m), m]
      if arg_present(v6) && nv gt 5 then v6 =  [min(a.v6, max = m), m]
      if arg_present(v7) && nv gt 6 then v7 =  [min(a.v7, max = m), m]
      if arg_present(v8) && nv gt 7 then v8 =  [min(a.v8, max = m), m]
   endif
   if arg_present(av1) then av1 = LPM_AVERAGE_SDEV(a.v1)
   if nv gt 1 then begin
     if arg_present(av2) then av2 = LPM_AVERAGE_SDEV(a.v2)
     if nv gt 2 then begin
       if arg_present(av3) then av3 = LPM_AVERAGE_SDEV(a.v3)
       if nv gt 3 then begin
         if arg_present(av4) then av4 = LPM_AVERAGE_SDEV(a.v4)
         if nv gt 4 then begin
           if arg_present(av5) then av5 = LPM_AVERAGE_SDEV(a.v5)
           if nv gt 5 then begin
             if arg_present(av6) then av6 = LPM_AVERAGE_SDEV(a.v6)
             if nv gt 6 then begin
               if arg_present(av7) then av7 = LPM_AVERAGE_SDEV(a.v7)
               if nv gt 7 && arg_present(av8) then av8 = LPM_AVERAGE_SDEV(a.v8)
             endif
           endif
         endif
       endif
     endif
   endif
   a=0
END
