; ------------------------------------------------------------------------
; $Id: strlimits.pro,v 1.4 2009/02/13 16:57:07 lpmix Exp $
; ------------------------------------------------------------------------
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
;+
; NAME:
;       STRLIMITS
;
; PURPOSE:
;       This procedure returns the parameters for a structure
;       (See STRINFO for additional parameters.)
;
; CALLING SEQUENCE:
;       STRLIMITS, STR [, TYPE = type] [, FILE = file]
;            [, CLABEL = clabel] [, TLABEL= tlabel]
;            [, XRANGE = xrange] [, YRANGE = yrange] [, ZRANGE = zrange] 
;            [, TRANGE = trange]
;            [, V1 = v1] [, V2 = v2] [, V3 = v3] [, V4 = v4]
;            [, V5 = v5] [, V6 = v6] [, V7 = v7] [, V8 = v8]
;
; INPUTS:
;       STR:    Valid structure number, PFIDL structure, or search string 
;               for the structure comment
;	
; KEYWORD PARAMETERS:
;       Note: Default values are 0 except for CLABEL
;       TYPE:   Structure type 
;       FILE:   File name for the data
;       CLABEL: Structure Comment Label
;       TLABEL: Structure Type Label
;       XRANGE:	Range of X-values, or first dimension
;       YRANGE:	Range of Y-values, or seconc dimension
;       ZRANGE:	Range of Z-values, or third dimension
;       TRANGE: Range of T-values, or fourth dimension
;       V1:     Range of first vector component or attribute
;       V2:     Range of second vector component or attribute
;       V3:     Range of third vector component or attribute
;       V4:     Range of fourth vector component or attribute
;       V5:     Range of fifth vector component or attribute
;       V6:     Range of sixth vector component or attribute
;       V7:     Range of seventh vector component or attribute
;       V8:     Range of eighth vector component or attribute
;
; OUTPUTS:
;       Specified keyword parmeters are returned
;
; PROCEDURE:
;       The specified structure is retrieved and the keyword parameters are 
;       extracted.
;
; EXAMPLE:
;       Determine the xrange of structure, 9, and store in XR.
;         STRLIMITS, 9, XRANGE = XR
;
; MODIFICATION HISTORY:
;       Written by:     Paul Mix, January 20, 2005
;
;-

PRO strlimits,  STRIN, TYPE = type , file = file, $
             CLABEL = clabel, trange = trange, tlabel = tlabel, $
             XRANGE = xrange, YRANGE = yrange, ZRANGE = zrange, $
             V1 = v1, V2 = v2, V3 = v3, V4 = v4, $
             V5 = v5, V6 = v6, V7 = v7, V8 = v8
on_error, 1
    
   if n_params() eq 0 then begin
     print,'% Format:', $
            ' STRLIMITS, STR [, TYPE = type] [, CLABEL = clabel]'
     print, '    [, TLABEL = tlabel] [, FILE = file] [, XRANGE = xrange]'
     print, '    [, YRANGE = yrange] [, ZRANGE = zrange] [, TRANGE = trange]'
     print, '    [, V1 = v1] [, V2 = v2] [, V3 = v3] [, V4 = v4]'
     print, '    [, V5 = v5] [, V6 = v6] [, V7 = v7] [, V8 = v8]'
     return
   endif
   type =0
   clabel = ''
   tlabel = ''
   file = ''
   xrange = 0
   yrange = 0
   zrange = 0
   trange = 0
   v1 = 0
   v2 = 0
   v3 = 0
   v4 = 0
   v5 = 0
   v6 = 0
   v7 = 0
   v8 = 0
   if n_elements(strin) eq 0 then begin
     print, '% STRLIMITS: Structure must be defined'
     return
   endif else str = strin
   
   a = get_valid_structure(str, 'STRLIMITS', lcstr=lcstr, ierror=ier)

   if IER ne 0 then begin
     print, '% STRLIMITS: Returning immediately'
     return
   endif
   names = tag_names(a)
   clabel = a.clab
   ok = where(strpos(names, 'TYPE') ge 0, nok)
   if nok gt 0 then begin
     type = a.type
     valid_typ = [1, 2, 3, 4, 7, 8, 9, 10, 11, 12]
     ok = where (type eq valid_typ, nok)
     if nok eq 0 then begin
       print, '% STRLIMITS: Structure is not a valid type: ', strtrim(type, 2)
       return
     endif
   endif else begin
     print, '% STRLIMITS: This is not a PFIDL structure'
     return
   endelse
   
   ndim = a.ndim
   if arg_present(file) then begin
     ok = where(strpos(names, 'FILE') ge 0, nok)
     if nok gt 0 then file = a.file
   endif
   if arg_present(tlabel) then tlabel = a.tlab
     
   if (arg_present(xrange) || arg_present(yrange) || arg_present(zrange) $
     || arg_present(trange)) then begin
     if type eq 2 then begin
;      particle data
       if arg_present(xrange) then xrange = [min(a.x1, max = m), m]
       if arg_present(yrange) && ndim gt 1 then yrange = [min(a.x2, max= m), m]
       if arg_present(zrange) && ndim gt 2 then zrange = [min(a.x3, max= m), m]
       if arg_present(trange) && ndim gt 3 then trange = [min(a.x4, max= m), m]
     endif else if type eq 4 then begin
;      conductor data
       if arg_present(xrange) then $
         xrange = [min((a.plane)[*, 0, *], max = m), m]
       if arg_present(yrange) && ndim gt 1 then $
         yrange = [min((a.plane)[*, 1, *], max = m), m]
       if arg_present(zrange) && ndim gt 2 then $
         zrange = [min((a.plane)[*, 2, *], max = m), m]
     endif else begin
       range = (a.range)[*, *, 0]
       if arg_present(xrange) then $
         xrange = [min((range)[*, 0], max = m), m]
       if arg_present(yrange) && ndim gt 1 then $
         yrange = [min((range)[*, 1], max = m), m]
       if arg_present(zrange) && ndim gt 2 then $
         zrange = [min((range)[*, 2], max = m), m]
       if arg_present(trange) && ndim gt 3 then $
         trange = [min((range)[*, 3], max = m), m]
     endelse
   endif
   valid_vector =[1, 2, 8, 9, 12]
   ok = where (type eq valid_vector, nok)
   if nok eq 0 then return
   if (arg_present(v1) || arg_present(v2) || arg_present(v3) $
      || arg_present(v4) || arg_present(v5) || arg_present(v6) $
      || arg_present(v7) || arg_present(v8)) then begin
      if type eq 2 then nv = a.natt else nv = a.nvect
      if arg_present(v1) then v1 =  [min(a.v1, max = m), m]
      if arg_present(v2) && nv gt 1 then v2 =  [min(a.v2, max = m), m]
      if arg_present(v3) && nv gt 2 then v3 =  [min(a.v3, max = m), m]
      if arg_present(v4) && nv gt 3 then v4 =  [min(a.v4, max = m), m]
      if arg_present(v5) && nv gt 4 then v5 =  [min(a.v5, max = m), m]
      if arg_present(v6) && nv gt 5 then v6 =  [min(a.v6, max = m), m]
      if arg_present(v7) && nv gt 6 then v7 =  [min(a.v7, max = m), m]
      if arg_present(v8) && nv gt 7 then v8 =  [min(a.v8, max = m), m]
   endif
END
