; ------------------------------------------------------------------------
; $Id: strscale.pro,v 1.1 2011/03/15 20:00:39 lpmix Exp $
; ------------------------------------------------------------------------
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
;+
; NAME:
;       STRSCALE
;
; PURPOSE:
;       Scale the spatial axis or amplitude of a structure using a WDF array.
;       The quantity being scaled is compared to the abscissa of a calibration
;       WDF array.  The corresponding ordinates replace the scaled quantity.
;       (See also SCALE_IMAGE for image scaling.)
;
; CATEGORY:
;       Analysis
;
; CALLING SEQUENCE:
;
;       STRSCALE, STR1, WDF_SCALE, SPACE [, STR_SAVE]
;
; INPUTS:
;       STR1:      Original structure array.  This may be a search string,
;                  a structure array number, or an array of structure array numbers.
;       WDF_SCALE: WDF array with the scaling data.  Abscissa must be in 
;                  increasing or decreasing order.  This is not checked.
;       SPACE:     Structure component to be scaled.
;                  [1, 2, 3, 4, ...] indicate spatial dimensons  (X1, X2, X3, ...)
;                  [-1, -2, -3, -4, ...] indicate field or attribute components.
;                     (V1, V2, V3, ...]
;       STR_SAVE:  Structure array with the rescaled structure values.
;                  If WDF_SAVE is missing or invalid, the result will be
;                  saved in STR1.  If STR_SAVE is a scalar and STR1 is a
;                  vector, successive arrays will be used.  
;	
; KEYWORD PARAMETERS:
;       DEFAULT:   If the component values of STR1 lie outside the domain of 
;                  WDF_SCALE, the ordinate will be set to this value.
;                  DEFAULT:  DEFAULT = ordinate closest to the bad data.
; OUTPUTS:
;       The structure array with the rescaled values is returned.
;
; RESTRICTIONS:
;       WDF_SCALE must have increasing or decreasing abscissa values.
;
; PROCEDURE:
;       FINDEX is used to determine the location of the STR1  values
;       in the WDF_SCALE abscissa values.  INTERPOLATE is used to determine the
;       corresponding ordinates in WDF_SCALE.  The values in STR1 are
;       changed to the ordinate values in WDF_SCALE.
;
; EXAMPLE:
;	Rescale a spectrum from a spectrometer using the wavelength scaling in
;       waveform array, 23, which contains wavelength vs position.  Assume the 
;       image data is in structure array 3.  Store the data in structure, 102.
;         STRSCALE, 2, 23, 1, 102
;
; MODIFICATION HISTORY:
;       Written by:	L. P. Mix, Jr., March 14, 2011
;-

PRO STRSCALE, WDF1, WDF_SCALE, SPACE, WDF_SAVE, default = default
npar = n_params()
if npar eq 0 then begin
  print,'% Format: STRSCALE, STR1, WDF_SCALE, SPACE, [, STR_SAVE] [, DEFAULT = default]'
  return
endif

if npar lt 3 then begin
  print, '% STRSCALE:  The structure array, scaling array and component must be provided'
  return
endif

str = strvalid(WDF1, /valid, np = np)
If np eq 0 then begin
  print, '% STRSCALE:  A valid WDF array must be provided'
  help, WDF1
  return
endif
if n_elements(space) eq 0 then begin
  print, '% STRSCALE:  A valid component of the structure must be requested.'
  return
endif else space = space[0]

scal = wdvalid(WDF_SCALE, /valid, np = nscal)
if nscal ne  1 then begin
  print,  '% YSCALE: A single valid scaling array must be provided, np='+strtrim(nscal,2)
  help, wdf_scale
  return
endif
w2i, xscal, yscal, scal[0]
npscalm = n_elements(xscal)-1
xmin = min(xscal, max = xmax)
if n_elements(wdf_save) gt 0 then begin
  ok = where (wdf_save gt 0 and wdf_save lt get_maxstr(), nok)
  if nok eq 0 then lcsave = 0 else begin
    lcsave = 1
    wdf_save = wdf_save[ok]
;   take the first value and add successive arrays if necessary.
    if nok lt nscal then wdf_save= wdf_save[0]+lindgen(nscal)
    if wdf_save[nscal-1] gt get_maxstr() then set_maxstrarray, wdf_save[nscal -1]+24
  endelse
endif else lcsave =0
if lcsave eq 0 then wdf_save = str

lcdefault =(n_elements(default) eq 0) ? 0 : 1

for ind= 0, np-1 do begin
  arr = get_array(str[ind], space, /alldata) 
  if lcsave then xfrstr, str[ind], wdf_save[ind]
  a = findex(xscal, arr) 
  if lcdefault then ok = where (a lt 0 or a gt npscalm) $
    else a = (a > 0) < npscalm
  yy = interpolate(yscal, a)
  if lcdefault then xx[ok] = default[0]
  put_array, wdf_save[ind] , yy, space, /alldata
endfor
END
