; ------------------------------------------------------------------------
; $Id: surf_fit.pro,v 1.3 2011/07/12 17:50:16 lpmix Exp $
; ------------------------------------------------------------------------
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
;+
; NAME:
;       SURF_FIT
;
; PURPOSE:
;       Fit a set of a two dimensions set of set of amplitudes to a polynomial
;       of the form:
;       Amplitude = A*X^2 + B*Y^2 + C*X*Y + D*X +E*Y + F
;       The fit is determined from a least squares formulism.
;
; CATEGORY:
;       Data Analysis
;
; CALLING SEQUENCE:
;
;       SURF_FIT, X, Y, Amp [, FitAmp]
;
; INPUTS:
;       X:	Abscissa values for the two dimensional amplitude.
;               N_ELEMENTS(X) must equal N_ELEMENTS(AMP) or 
;                 or N_ELEMENTS(AMP)/N_ELEMENTS(Y) or(SIZE(AMP))[1]
;       Y:      Ordinate values for the two dimensional Amplitude.
;               N_ELEMENTS(Y) must equal N_ELEMENTS(AMP) or 
;                 or N_ELEMENTS(AMP)/N_ELEMENTS(X) or SIZE(AMP))[2]
;       AMP:    Amplitude at the corresponding X, Y locations.
;               or if KEYWORD_SET(EVALUATE) then the calculated amplitude
;               at the input X, Y locations
;
; OPTIONAL OUTPUTS:
;       FITAMP: Amplitude at X, Y for the fit parameters.
;               Undefined for EVALUATE = 1
;	
; KEYWORD PARAMETERS:
;       COEFF:	A six element array with the best fit parameters
;               COEFF = [A, B, C, D, E, F]
;       EVALUATE: If set, calculate the amplitude at the X, Y locations.
;               If COEFF is present and contains 6 elements, then these
;               values will be used; if not the previous values stored in 
;               the SURF_FIT common block will be used.
;               The amplitude is calculated using:
;                 AMP = (A*X + C*Y +D)*X + (B*Y+E)*Y + F
;       GRID:   If set and if EVALUATE is set, then X and Y are assumed to be 
;               X and Y locations for a rectilinear grid and AMP will have
;               N_ELEMENTS(X) * N_ELEMENTS(Y) locations.
;               GRID is ignored unless EVALUATE is set.
;       STRUCTURE: If set, store X, Y, and Amplitude in a structure 
;               array.  If STRUCTURE is greater than zero, store it in the 
;               structure array number given by STRUCTURE.  If less than zero,
;               then store it in get_maxstr()-1.
;       SDEV:   Standard Deviation
;               SQRT((AMP-FITAMP)^2/ (NPOINT-1))
;       RSDEV:  Standard Deviation as a fraction of the maximum absolute amplitude
;               This is ignored unless SDEV is set.
;               RSDEV = SDEV / MAX(ABS(AMP)) 
;       QUIET:	If set, the fit parameters are not output to the screen
;
; OUTPUTS:
;       If EVALUATE is not set, the best fit parameters will be calculated for the 
;       the input data and the fit amplitudes calculated if the parameter is 
;       a valid argument.  Goodenss of fit parameters are calculated.
;       If EVALUATE is set, the polynomial surface fit is used to calculate a surface 
;       at the input values of X and Y.
;
; COMMON BLOCKS:
;       SURF_FIT_COMMON:  The coeff array is stored in the common block
;
; PROCEDURE:
;       The IDL procedure SVSOL and SVDC are used to solve the least square's equations.
;
; EXAMPLE:
;	Assume an array of X, Y, and Amplitude.  Determine the coefficients for the
;       least squares polynomial fit to this data.  Store the coefficients in COEF.
;       Return the standard deviation and relative standard deviation in SD and RSD.
;         SURF_FIT, X, Y, AMPLITUDE, COEFF = COEF, SDEV = sd, RSDEV = rsd
;
;       Calculate the amplitude for a rectilinear grid from 0 to 5 in X and 0 to 8 
;       in Y using the fit from the previous example and store the data in structure 12.
;         SURF_FIT, findgen(51)/10, findgen(81)/10, /EVALUATE, /GRID, STRUCTURE =12
;
; MODIFICATION HISTORY:
; 	Written by:	L. P. Mix, Jr., June 14, 2011
;		
;-


 pro surf_fit, x, y, amp, fitamp, coeff = coeff, evaluate = evaluate, grid = grid, $
     structure = structure, sdev = sdev, rsdev = rsdev, quiet = quiet
common SURF_FIT_COMMON, coeff_save
 on_error , 1

 nx =n_elements(x)
 ny =n_elements(y)
 np =n_elements(amp)
 if keyword_set(structure) then begin
   str = (structure gt 0) ? round(structure[0]) : get_maxstr()-1
   str = (str > 1) < get_maxstr()
 endif

 if keyword_set(evaluate) then begin
   if not arg_present(amp) then begin
     print, '% SURF_FIT:  AMP variable must be provided to return values.'
     return
   endif
   if n_elements(coeff) lt 6 then begin
     if n_elements(coeff_save) eq 6 then coeff = coeff_save else begin
       print, '% SURF_FIT:  Coefficient array must be provided or calculated.'
       return
     endelse
   endif
   sdev = 0.0
   rsdev = 0.0
   if keyword_set(grid) then begin
     xx = make_array(nx, ny, /double)
     yy = xx
     for i = 0, nx-1 do xx[i, *] = x[i]
     for j = 0, ny-1 do yy[*, j] = y[j]
   endif else begin
     if nx ne ny then begin
       print, '% SURF_FIT:  Number of X and Y points must be equal unless GRID is set'
       return
     endif
     xx = x
     yy = y
   endelse
   amp = (coeff[0] * XX + coeff[2]*yy+ coeff[3]) * xx + $
          (coeff[1]*yy+coeff[4]) * yy +coeff[5]
   amp = float(amp)  
   if n_elements(str) eq 1 then begin
     clab = 'Surface fit - Coeff='
     for i = 0, 5 do clab = clab + strtrim(coeff[i], 2) + ';;'
     if keyword_set(grid) then arr2str, str, amp, x, y, clab = clab, tlab = 'Surf_fit' $
       else arr2str, str, amp, x, y, clab = clab, tlab = 'Surf_fit', /fem
   endif
   return
 endif
 if (nx ne ny) or (nx ne np) then begin
   print, '% SURF_FIT:  Number of X, Y, and AMP points must be equal for a fit.'
   return
 endif
; AMP is the vector amplitude     
; use Sv etc for total for amp
 sx = total(x, /double)
 sy = total(y, /double)
 x2 = x*x
 x3 = x2*x
 x4 = x3*x
 y2 = y*y
 y3 = y2*y
 y4 = y3*y
 sxy = total(x*y, /double)
 sx2 = total(x2, /double)
 sy2 = total(y2, /double)
 sxy2 = total(x*y2, /double)
 sx2y = total(x2*y, /double)
 sx2y2 = total(x2*y2, /double)
 sx3y = total(x3*y, /double)
 sxy3 = total(x*y3, /double)
 sx3y = total(x3*y, /double)
 sx3 = total(x3, /double)
 sy3 = total(y3, /double)
 sx4 = total(x4, /double)
 sy4 = total(y4, /double)
 matrix = [ [sx4,  sx2y2, sx3y,  sx3,  sx2y, sx2], $
            [sx2y2,  sy4, sxy3,  sxy2, sy3,  sy2], $
            [sx3y,  sxy3, sx2y2, sx2y, sxy2, sxy], $
            [sx3,   sxy2, sx2y,  sx2,  sxy,  sx], $
            [sx2y,  sy3,  sxy2,  sxy,  sy2,  sy], $
            [sx2,   sy2,  sxy,    sx,   sy,  np] ]

 svx2 = total(amp*x2, /double)
 svy2 = total(amp*y2, /double)
 svxy = total(amp*y*x, /double)
 svx  = total(amp*x, /double)
 svy  = total(amp*y, /double)
 sv = total(amp, /double)
 b = [svx2, svy2, svxy, svx, svy, sv]
 svdc,  matrix, w, u, v
 sv = make_array(6, 6, value = 0.0) 
 for i = 0, 5 do sv[i, i] = w[i]

 mat2 =  u ## sv ## transpose(v)

 coeff = svsol( u, w, v, b)
 coeff_save = coeff
 if arg_present(fitamp) or keyword_set(sdev) then begin

   fitamp = (coeff[0] * X + coeff[2]*y+ coeff[3]) * x + $
          (coeff[1]*y+coeff[4]) * y +coeff[5]

   if arg_present(sdev) or arg_present(rsdev) then begin
     sdev = sqrt(total((amp-fitamp)^2/ (np-1), /double))
     rsdev = sdev / max(abs(amp))
   endif
 endif
 if not keyword_set(quiet) then begin
   print, 'Surface fit to:'
   print, '   Amplitude = A*X^2 + B*Y^2 + C*X*Y + D*X +E*Y + F'
   print, 'Parameters:'
   print, coeff
   if n_elements(sdev) gt 0 then print, 'Stand Dev: '+strtrim (sdev, 2)
   if n_elements(rsdev) gt 0 then print, 'Relative Stand Dev: '+strtrim (rsdev, 2)
 endif     
 if n_elements(str) eq 1 then begin
   clab = 'Surface fit - Coeff='
   for i = 0, 5 do clab = clab + strtrim(coeff[i], 2) + ';;'
   if keyword_set(grid) then arr2str, str, amp, x, y, clab = clab, tlab = 'Surf_fit' $
     else arr2str, str, amp, x, y, clab = clab, tlab = 'Surf_fit', /fem
 endif

 
 
 end
