; ------------------------------------------------------------------------ 
; $Id: tindex.pro,v 1.3 2007/06/06 20:09:38 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
;+
; NAME:
;       TINDEX
;
; PURPOSE:
;       Function to return the index for the closest specified time.
;
; CALLING SEQUENCE:
;       IND = TINDEX(STR, TIME, /CDF, /PDB, /Valid)	
;
; INPUTS:
;       STR:  Structure or structure array number of a type 9 structure 
;             Taken as TIME if a keyword is set.
;       TIME: Time at which the closest index is desired.
;       IF no keywords are set, both STR and TIME must be provided.
;       If a keyword is set, STR is assumed to be TIME.
;
; KEYWORD PARAMETERS:
;       VALID:If set, only valid values will be returned.
;             If time less than the initial time or greater than the final time
;             and valid is set, then -1 is returned. 
;            Note: A fudge factor of 1/2^15 of the range is allowed.
;             If valid is omitted, then the closest time is returned.
;       CDF:  If set, Time from the current netCDF file is taken
;             If set, STR is taken as TIME
;       PDB:  If set, Time from the current PDB file is taken
;             If set, STR is taken as TIME
;     Note: If CDF is set, PDB is not checked.
;           If no keywords are set, STR is assumed to be a structure.
;
; OUTPUTS:
;       IND:  INDEX for output time closest to TIME.
;             IND = -1 if an error occurs
;             IND is in the range [1, Ntime]
;
; COMMON BLOCKS:
;       PDB and CDF common blocks are available
;
; RESTRICTIONS:
;       A PDB or netCDF file must be open or a time dependent finite element
;       array must be in memory.
;
; EXAMPLE:
;       Determine the index closest to 11.1 ns from the current cdf file
;          IND = tindex (11.1e-9, /cdf) 
;
;       Determine the index closest to 12.3 ns in structure 4
;          IND = tindex (4, 11.1e-9) 
;
;       Plot the data in structure 3 at about 8ns.
;          PLOTSTR, 3, TINDEX(3, 8e-9), /index
;
; 	Written by:	L. P. Mix, December 22, 1999
;-

 Function TINDEX, strin, time, cdf = cdf, pdb = pdb, valid = valid

 on_error, 1

;
;  Common block
@qscom.dat
@qsstrcom.dat
@com.dat
@netcdfcom.dat
;
;
      if n_elements (initwdf) eq 0 then wdfinit
      if n_elements (structinit) eq 0 then qsstruct_init
      if n_elements (qsplotinit) eq 0 then qsinit
;
      on_error, 1
      npar_kw = keyword_set(cdf) + keyword_set(pdb)
      npar = n_params() + npar_kw
      if npar eq 0 then begin
print,'% Format: TINDEX = TINDEX(IDL_Structure, TIME, /VALID [, /CDF ][, /PDB])'
        return, -1
      endif
      snum = 0
      if npar_kw eq 0 then begin
        if n_elements(strin) eq 0 then begin
          print, '% TINDEX: Structure array must be defined'
          return, -1
        endif else str = strin[0]
        if n_elements (time) eq 0 then begin
          print, '% TINDEX: Time value must be defined.'
          return, -1
        endif
        b = size(str)
        if b[b[0]+1] ne 8 then begin
          if str gt 0 and str lt maxstructure then begin
            snum = str
            s2i, str, snum, /no_copy
            b = size(str)
            if b[b[0]+1] ne 8 then begin
       print, '% TINDEX: First parameter must be a valid structure or number'
              return, -1
            endif
          endif else begin
            print, '% TINDEX: First parameter must be a valid structure number'
            return, -1
          endelse
        endif
        if (str.type ne 9) then begin
          print, '% TINDEX: Structure must be a time dependent, FE structure.
          if snum gt 0 then i2s, str, snum, /no_copy
          return, -1
        endif
        mm = min (abs(str.x4-time), i)
        if keyword_set(valid) then  v1 = min(str.x4, max = v2)
        if snum gt 0 then i2s, str, snum, /no_copy
        if keyword_set (valid) then begin
          eps = (v2-v1)*3.05176e-05
          if time lt (v1-eps) or time gt (v2+eps) then return, -1
        endif
        return, i+1
      endif
      if n_elements (strin) eq 0 then begin
        print, '% TINDEX: First parameter be the desired time'
        return, -1
      endif else time = strin
      if keyword_set (cdf) then begin
        if n_elements(initcdf) eq 0 then begin
          print, '% TINDEX: Open a CDF file'
          return, -1
        endif
;
        if cdf_id  lt 0 then begin
          print, '% TINDEX: No netCDF files are open.'
          return, -1
        endif

        if lcdfvalid ne array_cdfid(cdf_id, 1) then load_cdf_data

        if cdf_type(cdf_id) eq 1 then begin
          mm = min (abs(time_whole-time), i)
          if keyword_set (valid) then begin
            v1 = min(time_whole, max = v2)
            eps = (v2-v1)*3.05176e-05
            if time lt (v1-eps) or time gt (v2+eps) then return, -1
          endif
          return, i+1
        endif
        return, -1
      endif else begin
        print, '% TINDEX: Only type 1 netCDF data supported'
        return, -1
      endelse
;     pdb file
      ;
      if initpdb  le 0 then begin
        print, '% TINDEX: No netPDB files are open.'
        return, -1
      endif

      If curfid lt 0 then begin
        print, '% TINDEX:  No valid PDB files are open.'
        return, -1
      endif
      info = file_info(curfid)
; get the standard stuff
      mm = min (abs(info.time-time), i)
      if keyword_set (valid) then begin
        v1 = min(info.time, max = v2)
        eps = (v2-v1)*3.05176e-05
        if time lt (v1-eps) or time gt (v2+eps) then return, -1
      endif
      return, i+1
      end
