; ------------------------------------------------------------------------ 
; $Id: triflw.pro,v 1.5 2010/09/17 20:51:05 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
;+
; NAME:
;	TRIFL
;
; PURPOSE:
;	Model electromagnetic energy transport in a transmission line
;       Source voltages are either from a WDF array or analytic
;       Loads are Constant Impedance or perveance, analytic falling
;         impedance, analytic rising then falling, impedance from a
;         wdf array, or an imploding plasma load
;       Electron flow in the MITL is assumed to occur by default.
;         Electron flow can be turned for the entire MITL or up to a
;         point on the transmission line.  After flow is turned on, it 
;         will be on until the load.
;
; CALLING SEQUENCE:
;
;       TRIFL [, X ,ZVAC] [, Zflow] [, Transition = TR] [, OUT = out]
;         [,CONVOLUTE = convolute] [, FLOWOUT = flowout]
; <or>  TRIFL [, WD_ZV] [, WD_ZF]  [Transition = TR] [, OUT = out]
;         [,CONVOLUTE = convolute] [, FLOWOUT = flowout]
; <or>  TRIFL [, X, Y]  [Transition = TR] [, OUT = out]
;         [,CONVOLUTE = convolute] [, FLOWOUT = flowout]
;
; REQUIRED INPUTS:
;	None - The user is normally prompted for TRIFL line segments.
;              The parameters which the user must specify are listed in a 
;                window interface.
;
; OPTIONAL INPUT:
;       NOTE:   X values cannot be closer than 1e-5 of the largest value
;               Values closer than this will be averaged.
;               All dimensions are meters/ohms
;       X, Zvac, Zflow - 
;                At least 2 arrays must be provided and have the same number of 
;                  elements where :
;                X - spatial coordinate
;                Zvac - Vacuum Impedance
;                Zflow - Flow impedance (In general, ignored and calculated 
;                  according to the formula:Zflow = Zvac*v/(v+2Me) where
;                  Me is 0.511 kV and v is the peak voltage on the line.
;                  At a convolute Zflow is held constant until then convolute 
;                  region and allow to transition to the downstream value.
;       WD_Zv, WD_Zflow -
;                Waveform array numbers for the vacuum impedance and the 
;                flow impedance. (See Zflow above- largely ignored)
;       NOTE:    Linear interpolation will be used to connect input points.
;
;       X, Y  -  Arrays from the READASC command
;                If Y has 3 values in the second dimension then Tr = Y(*, 2)
;                X  - spatial coordinate
;                Y(*,0) - Zv - Vacuum Impedance
;                Y(*,1) - Zflow  - Flow impedance
;                   (See Zflow above- largely ignored)
;       Transition - A single element or an array indicating the type of 
;                transition between tranmission line segments.  If TR is an 
;                array it must have the same number of elements as X. If
;                waveform arrays are provided, linear interpolation will be
;                used to put the waveforms on a common time scale.
;                Values are:
;                    0 Linear (DEFAULT)
;                    1 Sine^2
;                    2 Log
;        OUT     Starting point for WDF arrays and Structure Arrays
;                Default is OUT = 1
;        CONVOLUTE - An array of values corresponding to the convolute flow 
;                    impedance; a zero value indicates no convolute
;        FLOWOUT   - Flow impedance on the output of a convolute; a zero
;                    value indicates no convolute.
;        Note:   The environment variable "TRIFL_EXE" will be used as the 
;                executable if it is specified.
;
; OUTPUTS:
;	Three files are written: 
;         1. Suffix of ".geom" is an ASCII geometry file for the TRIFL
;         2. Suffix of ".src" is an ASCII source file for the TRIF code
;	  3. Suffix of ".pff" is the calculated data for the TRIFL run
;            The first dataset is an IFL dataset.
;            The integer array has the load type and the electron flow
;               flag.
;            The float list has the source inpedance for the driver and,
;               in the case of a Imploding Plasma Load, it has the following:
;                   Mass/Length, Length, Initial Radius, % to Final Radius, 
;                   Return Radius, L0, Lstart, I0
; The following WDF arrays are populated for OUT=1 and a Imploding Plasma Load:
;       Array No.   Dataset Comment
;              1    Load Voltage vs Time
;              2    Load Current vs Time
;              3    Input Voltage vs Time
;              4    Input Current vs Time
;              5    dL/dt - Change in Inductance vs Time
;              6    Inductance vs Time
;              7    Forward Going Voltage
;              8    Forward Going Power vs Time
;              9    Forward Going Energy vs Time
;             10    Input Energy vs Time
;             11    Output Energy vs Time
;             12    Inductive Load Energy vs Time
;             13    Net Foil Energy vs Time
;             14    Total Efficiency (wdf 13/wdf9 * 100)
; For a resistive load the arrays are:
;       Array No.   Dataset Comment
;              1    Load Voltage vs Time
;              2    Load Current vs Time
;              3    Input Voltage vs Time
;              4    Input Current vs Time
;              5    Forward Going Voltage
;              6    Forward Going Power vs Time
;              7    Forward Going Energy vs Time
;              8    Input Energy vs Time
;              9    Output Energy vs Time
;             10    Total Efficiency (wdf 9/wdf7 * 100)
; The following structure arrays are populated for OUT = 1
;      Array No.   Dataset Comment
;            1     A - 
;            2     V - 
;            3     Q - 
;            4     Ie - 
;            5     Ia - 
;            6     Ic - 
;            7     Qa -
;            8     Qc -
;
; COMMON BLOCKS:
;	The transmission line data is stored in a common block: MITL_common
;
; PROCEDURE:
;	Technique developed by C. W. Mendel with some modifications by L. P. Mix
;       and D. B. Seidel.
;       Each transition line section will need the vacuum impedance as a
;         function of axial impedance.
;
; EXAMPLE:
;		TRIFL
;
; MODIFICATION HISTORY:
; 	Written by:	L. P. Mix, June 28, 1994
;                       Added convolutes, June 7, 1995
;
;-

PRO TRIFL_Event, Event

common MITL_common, str, lcexpert, dxcom, dt, outdt, totalt, zsource, stype, $
         wdfa, rtype, tpeak, ftype, tfall, twidth, umax, ltype, wdfb, $
         rltype, tlpeak, fltype, tlfall, tlwidth, peakz, plmass, plength, $
         pradius, plmin, plret, lcflow, qemit, lcflash, tu, uu, tt, zz
common TRIFL_savefile, oldpath

      common pffcom, initpff, pff_exe
      on_error, 1
      if (initpff eq 0) then init_pff
      if initpff eq 0 then begin
        print, 'PFF libraries are not supported by this Operating System'
        widget_control, event.top, /destroy
        return
      endif


  WIDGET_CONTROL,Event.Id,GET_UVALUE=Ev

  CASE Ev OF

  'BGROUP24': BEGIN
      CASE Event.Value OF
      0: Begin
;         'Button Read Previous Pressed'
         IF (n_elements(oldpath) eq 0) THEN cd, current = path $
           else path = oldpath
         filter = '*.geom'
         file = pickfile(/read, path=path, get_path=oldpath, filter=filter)
         if strlen(file) gt 1 then begin
           openr, unit, /get, file
           inptstr = ''
           readf, unit, inptstr
           maxnp = 64L
           y = make_array(maxnp, 7)
           np = 0
           while (not eof(unit)) do begin
             readf, unit, inptstr
             parser, inptstr, z, npp
             if npp gt 2 then begin
               np = np+1
               if np eq maxnp then begin
                 maxnp = maxnp+maxnp
                 y = [y, y]
               endif
               y(np-1, 0:npp-1) = z
               if npp lt 7 then y(np-1, 6) = lcflow
             endif
           endwhile
           if np gt 0 then $
           add_mitlpnts,np, y(0:np-1, 0), y(0:np-1, 1), lcexpert*y(0:np-1,2),$
              y(0:np-1, 3), y(0:np-1, 4), y(0:np-1, 5), y(0:np-1, 6)

           if str.np gt 1 then begin
             widget_control, event.top, get_uvalue = info
             get_zoom_values, /max, info
             widget_control, info.ptype , get_value = type
             mitl_plot, type, xrange =0
           endif
           close, unit
         endif 
         widget_control, event.top, /clear_events
         End

      1: BEGIN
;          Add a point
           if str.np gt 0 then begin
             x = (str.x)(str.np-1)
             zv = (str.zv)(str.np-1)
             zf = (str.zf)(str.np-1)
             fit = (str.fit)(str.np-1)
             zfcon = (str.zfcon)(str.np-1)
             zfdwn = (str.zfdwn)(str.np-1)
             lcemit = (str.lcemit)(str.np-1)
           endif
           widget_control, event.top, get_uvalue=info, tlb_get_offset=offset
           get_point,x, zv, zf, fit, zfcon, zfdwn, lcemit, lcquit, offset, $
             /modal, expert = lcexpert, group = event.top
           if lcquit eq 0 then $
             add_mitlpnts, 1, x, zv, zf, fit, zfcon, zfdwn, lcemit
           if str.np gt 1 then begin
             get_zoom_values, /max, info
             widget_control, info.ptype , get_value = type
             mitl_plot, type, xrange =0
           endif
         widget_control, event.top, /clear_events
         END
      
      2: Begin
;        Button Delete Line Pressed
         if str.np gt 0 then begin
;          print, ' Indicate the point which you wish to delete'
           
           widget_control, event.top, get_uvalue = info, tlb_get_offset = offset
           win = winstruction (offset = offset, $
             ['    Indicate the point which    ','  ', ' you wish to delete '])
           cursor,x, y, 3
           widget_control, win, /destroy
           miss = min(abs(x-str.x), i)
           delete_mitlpnts, i
           if str.np gt 1 then begin
             widget_control, event.top, get_uvalue = info
             get_zoom_values, /max, info
             widget_control, info.ptype , get_value = type
             mitl_plot, type, xrange =0
           endif
         endif
         widget_control, event.top, /clear_events
         END

      3: BEGIN
;       Print,'Button Edit Line Pressed'
         strold = str
         if str.np gt 0 then begin
           if str.np eq 1 then begin
             x = (str.x)(0)
             zf = (str.zf)(0)
             zv = (str.zv)(0)
             fit = (str.fit)(0)
             zfcon = str.zfcon(0)
             zfdwn = str.zfdwn(0)
             lcemit = str.lcemit(0)
             win = 0 
           endif else begin
;            print, ' Indicate the point which you wish to edit'
             widget_control,event.top,get_uvalue= info,tlb_get_offset = offset
             win = winstruction( offset = offset, $
               ['    Indicate the point which    ','  ', ' you wish to edit '])
             cursor,x, y, 3
             widget_control, win, /destroy
             miss = min(abs(x-str.x), i)
             zf = (str.zf)(i)
             zv = (str.zv)(i)
             x = (str.x)(i)
             fit = (str.fit)(i)
             zfcon = (str.zfcon)(i)
             zfdwn = (str.zfdwn)(i)
             lcemit = (str.lcemit)(i)
              delete_mitlpnts, i, /nofill
           endelse
           widget_control, event.top, get_uvalue=info, tlb_get_offset=offset
           get_point,x, zv, zf, fit, zfcon, zfdwn, lcemit, lcquit, offset, $
             /modal, expert = lcexpert, group = event.top
           if lcquit eq 0 then $
             add_mitlpnts, 1, x, zv, zf, fit, zfcon, zfdwn, lcemit $
              else str = strold
           if str.np gt 1 then begin
             get_zoom_values, /max, info
             widget_control, info.ptype , get_value = type
             mitl_plot, type, xrange =0
           endif
         endif

         widget_control, event.top, /clear_events
         END
      ELSE: Message,'Unknown button pressed'
      ENDCASE
      END
  'BGROUP33': BEGIN
        if str.np gt 1 then begin
          type = event.value
          widget_control, event.top, get_uvalue = info
          get_zoom_values, info, x1, x2
          if x1 eq x2 then $
            mitl_plot, type, xrange = 0 $
          else $
            mitl_plot, type, xrange= [x1, x2]
        endif
      END
  'FSLID41': BEGIN
      if event.drag eq 0 then begin
        if str.np gt 1 then begin
          widget_control, event.top, get_uvalue = info
          widget_control, info.ptype , get_value = type
          get_zoom_values, info, x1, x2
          if x1 eq x2 then $
            mitl_plot, type, xrange = 0 $
          else $
            mitl_plot, type, xrange= [x1, x2]
        endif
      endif
      END

  'FSLID42': BEGIN
      if event.drag eq 0 then begin
        if str.np gt 1 then begin
          widget_control, event.top, get_uvalue = info
          widget_control, info.ptype , get_value = type
          get_zoom_values, info, x1, x2
          if x1 eq x2 then $
            mitl_plot, type, xrange = 0 $
          else $
            mitl_plot, type, xrange= [x1, x2]
        endif
      endif
      END
;  'BUTTON49': BEGIN
;     Print, 'Event for  Plot Data '
;     END
  'BGROUP34': BEGIN
      CASE Event.Value OF
      0: BEGIN
;        Print,'Button      Run / Save      Pressed'
         if str.npos lt 2 then $
           winformation,[ 'Cannot run a simulation with', $ 
             'Less than 2 valid TRIFL locations'] else begin
       
         lcdefault = 1
         if n_elements (dxcom) gt 0 then dx = dxcom
         widget_control, event.top, get_uvalue = info, tlb_get_offset = offset
         get_dxdt, lcquit, file, comment, dx, dt, outdt, totalt, offset
         if lcquit eq 0 then begin
               sol = 0.2997925
               dxsave = dx
               nuarr = n_elements (uu)
               nload = n_elements (zz)
               ratio = dx/dt/sol
               startx = str.x(0)
               stopx = str.x(str.np-1)
               nx = round ((stopx-startx)/dx) +1
               if nx lt 21 then begin
                 winformation, ['A minimum of 20 intervals are required', $
                              'DX changed to provide 20 intervals']
                 dxold = dx
                 dx = (stopx-startx)*0.05
                 dxsave = dx
                 dt = dx/dxold*dt
                 nx = 21
               endif else dx =  (stopx-startx)/(nx-1)
               xcalc = startx + findgen(nx)*dx
               dx = dx/sol
               startx = startx/sol
               stopx = stopx/sol*1.00001
               if dx/dt gt 0.99 then dt = dx/ratio
               outdt = max([outdt, dt])
               nstep = round(outdt/dt)
               nt = round(totalt/dt/nstep+1.0)> 10
               nsp= str.npos
               space = str.pos/sol
               dx2 = 0.5*dx
               put2uniform,space,str.zflow,str.npos,startx,dx2,stopx,zflow ,/y
               ok = indgen(nx)*2
               okh = ok(0:nx-2)+1
               zfh = zflow(okh)
               zflow = zflow(ok)
               put2uniform, space, str.zvac,str.npos,startx,dx2,stopx,zvac,/y 
               zvh = zvac(okh)
               zvac = zvac(ok)
               dspace =  space(1:nsp-1)-space(0:nsp-2)
               dspx  = [0, space(1:nsp-1)+space(0:nsp-2), 0]*0.5
               dspx(0)= dspx(1)-dspace(0)
               dspx(nsp)= dspx(nsp-1)+dspace(nsp-2)
               dzvac = [0.0,(str.zvac(1:nsp-1)-str.zvac(0:nsp-2))/dspace,0.0]*dx
               put2uniform, dspx, dzvac, nsp+1, startx, dx, stopx, dzvacdx, /y
               if n_elements(dzvacdx) gt nx then dzvacdx = dzvacdx(0:nx-1)
               dzfl=[0.0,(str.zflow(1:nsp-1)-str.zflow(0:nsp-2))/dspace,0.0]*dx
               put2uniform, dspx, dzfl, nsp+1, startx, dx, stopx, dzfdx, /y
               if n_elements(dzfdx) gt nx then dzfdx = dzfdx(0:nx-1)
; debug
;              print, 'dzfdx max/min ', max(dzfdx, min = mm), mm
;              print, 'dzvacdx max/min ', max(dzvacdx, min = mm), mm
               typload = long(ltype)

;              perveance I = perv*V*sqrt(V)
;              Z = V/I  means at peak voltage perv = 1/(Z*sqrt(v))
               if typload eq 5 then zz(0) = 1.0/(zz(0:1)*sqrt(umax))
               asnp = make_array(nx, nt, value = 0.0)
               vsnp = make_array(nx, nt, value = 0.0)
               vsnp(0,0) = umax
               qsnp = make_array(nx, nt, value = 0.0)
               isnp = make_array(nx, nt, value = 0.0)
               npload = (nt-1)*nstep+1
               if typload lt 4 then $
                 loadsnp = make_array(npload*4) else $
                 loadsnp = make_array(npload*6)
               dzvacdx = dzvacdx/zvac
;              plot, dzvacdx, tit = 'dzvac/dx * dx/zvac'  
               print, 'dt/dx*c = ',strtrim(dt/dx, 2)
               filez  = find_newfile(file +'.src', 1)
               if strlen(filez) gt 1 then begin
                 openw, unit, /get, filez
    write_mitl_src, unit, dxsave, dt, outdt, totalt, zsource, stype,$
      wdfa, rtype, tpeak, ftype, tfall, twidth, umax, ltype, wdfb, $
      rltype, tlpeak, fltype, tlfall, tlwidth, peakz, plmass, plength, $
       pradius, plmin, plret, lcflow, qemit, lcflash
                 free_lun, unit
               endif
               junk1 = make_array(nx)
               junk2 = make_array(nx)
               junk3 = make_array(nx)
               junk4 = make_array(nx)
               junk5 = make_array(nx)
               junk6 = make_array(nx)
               junk7 = make_array(nx)
               junk8 = make_array(nx)
               junk9 = make_array(nx)
               junk10 = make_array(nx)
               junk11 = make_array(nx)
               junk12 = make_array(nx)
               junk13 = make_array(nx)
               junk14 = make_array(nx)
; junk15, 16, 17, 18 are cathj, anodj, cathjh, anodjh
               junk15 = make_array(nx, value = 0.0)
               junk17 = make_array(nx, value = 0.0)
               ok = where (str.zfcon gt 0.0 and str.zfdwn gt 0.0, nok)
               if nok gt 0 then begin
                 ramp =  [findgen(10), rotate(findgen(9), 2)]+1.0
                 ramph = make_array(20)
                 ramph(0:9) =  findgen(10)+0.5
                 ramph(10:19) = rotate(ramph(0:9), 2)
                 smoother = sin(!pi*0.025*(findgen(20)+1.0))^2
                 smootherh= sin(!pi*0.025*(findgen(20)+0.5))^2
                 smootherd= 0.05*!pi*sin(!pi*0.025*(findgen(19)+1.0))* $
                        cos(!pi*0.025*(findgen(19)+1.0))
                 ramp = ramp/(dx*200.0)
                 ramph = ramph/(dx*200.0)
                 for i = 0, nok-1 do begin
                   xcon = min(abs(str.x(ok(i))- xcalc), imid)
                   ibeg = imid -10 > 0
                   iend = ibeg + 20
                   if iend gt nx-1 then begin
                     iend = nx-1
                     ibeg = nx-21
                   endif
                   zf2 = 1.0/(str.zfcon(ok(i))*str.zfcon(ok(i)))
                   junk17(ibeg:ibeg+19)= junk17(ibeg:ibeg+19)+ $
                     ramph*zf2
                   junk15(ibeg+1:ibeg+19)= junk15(ibeg+1:ibeg+19)+ $
                     ramp*zf2
                   zflow(ibeg+1:ibeg+20) = zflow(ibeg) + $
                      (zflow(ibeg+20) -zflow(ibeg))*smoother
                   zfh(ibeg:ibeg+19) = zflow(ibeg) + $
                      (zflow(ibeg+20) -zflow(ibeg))*smootherh
                   dzfdx(ibeg) = 0.5*dzfdx(ibeg)
                   dzfdx(ibeg+20) = 0.5*dzfdx(ibeg+20)
                   dzfdx(ibeg+1:ibeg+19)=(zflow(ibeg+20)-zflow(ibeg))*smootherd
                 endfor
               endif
;debug
               junk16= junk15
               junk18= junk17
               ok = where (zvac le zflow, nok)
               if nok gt 0 then begin
                 Winformation, 'Zflow reduced to 0.95*Zvac'
                 zflow(ok)= 0.95*zvac(ok)
               endif
               ok = where (zvh le zfh, nok)
               if nok gt 0 then $
                 zfh(ok)= 0.95*zvh(ok)
               if n_elements (qemit) eq 0 then qemitout = 0.0 $
                 else qemitout = float(0.0166782*qemit(0))
               print, 'qemitout- ' ,  qemitout
               junkq  = make_array(nx, value = qemitout)
               fileg = find_newfile(file +'.geom', 1)
               if strlen(fileg) gt 1 then begin
                 openw, unit, /get, fileg, width = 110
               fmstr = '(7g14.7)'

                 printf, unit, $
 ' Z-Position, Vacuum Imped., Flow Imped., FitType, Convolute, Zf-Down, Emit'
                 for i = 0, str.np-1 do printf, unit, format = fmstr, $
                   str.x(i), str.zv(i), str.zf(i), str.fit(i), $
                   str.zfcon(i), str.zfdwn(i), str.lcemit(i)
                 free_lun, unit
               endif
               filep  = find_newfile(file +'.pff')
               a = getenv('TRIFL_EXE')
               if strlen(a) lt 1 then $
                 TRIFL_EXE = PFF_EXE else TRIFL_EXE = a
               memit = max(str.lcemit, min = mm)
;              memit is m value where emission begins
               if memit eq 0 then memit = nx else begin
                 if mm gt 0 then memit = 0L else begin
                   ok = where (str.lcemit gt 0)
                   memit = 1L + round((str.x(ok(0))-str.x(0))/dxsave)
                 endelse
               endelse
;              junkq(0:memit-1)= 1.0e6
               if lcflash ne 0 then memit = - memit
               if lcflow eq 0 then typload = -(typload + 1)
               tempsnp = asnp
             if initpff eq 1 then begin
; HP
               junk = call_external( trifl_exe, $
                   'trifl_sl', nx, dx, nt, dt, nstep, nuarr, $
                  uu, tu, zsource, zvac, dzvacdx, zvh, zflow,  dzfdx, zfh, $
                  nload, zz, tt, typload, memit, $
                  asnp, vsnp, qsnp, isnp, loadsnp, junkq, $
                  junk1, junk2, junk3, junk4, junk5, junk6, junk7, junk8, $
                  junk9, junk10, junk11, junk12, junk13, junk14, $
                  junk15, junk16, junk17, junk18, tempsnp)
                print, min(junkq, max = mm), mm
             endif else if initpff eq 2 then begin
; Sun
               junk = call_external( trifl_exe, $
                   'trifl_sl_', nx, dx, nt, dt, nstep, nuarr, $
                  uu, tu, zsource, zvac, dzvacdx, zvh, zflow,  dzfdx, zfh, $
                  nload, zz, tt, typload, memit, $
                  asnp, vsnp, qsnp, isnp, loadsnp, junkq, $
                  junk1, junk2, junk3, junk4, junk5, junk6, junk7, junk8, $
                  junk9, junk10, junk11, junk12, junk13, junk14, $
                  junk15, junk16, junk17, junk18, tempsnp)
             endif else if initpff eq 4 then begin
; WIN
               junk = call_external( trifl_exe, $
                   '_TRIFL_SL@8', nx, dx, nt, dt, nstep, nuarr, $
                  uu, tu, zsource, zvac, dzvacdx, zvh, zflow,  dzfdx, zfh, $
                  nload, zz, tt, typload, memit, $
                  asnp, vsnp, qsnp, isnp, loadsnp, junkq, $
                  junk1, junk2, junk3, junk4, junk5, junk6, junk7, junk8, $
                  junk9, junk10, junk11, junk12, junk13, junk14, $
                  junk15, junk16, junk17, junk18, tempsnp)
             endif else if initpff eq 5 then begin
; MAC
                junk = call_external( trifl_exe, $
                   'trifl_sl', nx, dx, nt, dt, nstep, nuarr, $
                  uu, tu, zsource, zvac, dzvacdx, zvh, zflow,  dzfdx, zfh, $
                  nload, zz, tt, typload, memit, $
                  asnp, vsnp, qsnp, isnp, loadsnp, junkq, $
                  junk1, junk2, junk3, junk4, junk5, junk6, junk7, junk8, $
                  junk9, junk10, junk11, junk12, junk13, junk14, $
                  junk15, junk16, junk17, junk18, tempsnp)


             endif
               junk1 = 0
               junk2 = 0
               junk3 = 0
               junk4 = 0
               junk5 = 0
               junk6 = 0
               junk7 = 0
               junk8 = 0
               junk9 = 0
               junk10 = 0
               junk11 = 0
               junk12 = 0
               junk13 = 0
               junk14 = 0
               junk15 = 0
               junk16 = 0
               junk17 = 0
               junk18 = 0

               widget_control, event.top, get_uvalue= info
               zvac_arr = zvac#replicate(1.0, nt)
               zf_arr = zflow#replicate(1.0, nt)
               zdiff_arr = (zvac-zflow)#replicate(1.0, nt)
               ia = (asnp + zdiff_arr*isnp)/zvac_arr
               ic = (asnp - zf_arr*isnp)/zvac_arr
               qa = (vsnp +temporary(zdiff_arr)*qsnp)/zvac_arr
               qc =  (vsnp -temporary(zf_arr)*qsnp)/temporary(zvac_arr)
               createpff, filep
               typload = long(ltype)
               iarray= [ltype, lcflow] 
               if ltype eq 4 then begin 
                 flist = [zsource, plmass, plength, $
                  pradius, plmin, plret,zz(0:2)] 
                 clab ='TRIFL Parameters:Zs,mass,Len,r0,%rmin,rret,L0,Lstart,I0'
               endif else begin
                 flist = [zsource] 
                 clab = 'TRIFL Parameters:Zs'
               endelse
                i2w, xcalc,junkq, 31, c = 'qemit'

               writeifl, 0, iarray, flist, c =clab
               i2w, xcalc, zvac, 21, c= 'Vacuum Impedance',  $
                     xlab = 'Distance (m)' , ylab = 'Ohms'
               wri, 21
;              writepff, 0, zvac, xcalc,c= 'Vacuum Impedance',  $
                     xlab = 'Distance (m)' , blab = 'Ohms'
               i2w, xcalc, zflow, 22, c= 'Flow Impedance',  $
                     xlab = 'Distance (m)' , blab = 'Ohms'
               wri, 22
;              writepff, 0, zflow, xcalc,c= 'Flow Impedance',  $
                     xlab = 'Distance (m)' , blab = 'Ohms'
               i2w, tu, uu, 23, c= 'Forward Voltage', $
                     xlab = 'Time (ns)' , blab = 'Voltage (MV)'
               wri, 23
;              writepff, 0, uu, tu,c= 'Forward Voltage', $
                     xlab = 'Time (ns)' , blab = 'Voltage (MV)'
               if typload lt 4 then begin
                 i2w, tt, zz, 24, c= 'Load Impedance',  $
                     xlab = 'Time (ns)' , blab = 'Ohms'
                 wri, 24
               endif
;               if typload lt 4 then writepff, 0, zz, tt,c= 'Load Impedance',  $
;                    xlab = 'Time (ns)' , blab = 'Ohms'
               t = findgen(npload)*dt
               i2w, t, loadsnp(0:npload-1), 25, c = 'Load Voltage vs Time', $
                     xlab = 'Time (ns)' , blab = 'Voltage (MV)'
               wri, 25
                hak, /mes
               writepff, 0,loadsnp(0:npload-1), t, c = 'Load Voltage vs Time', $
                     xlab = 'Time (ns)' , blab = 'Voltage (MV)'
               i2w, t, loadsnp(0:npload-1), info.out, $
                     clab = 'Load Voltage vs Time' , $
                     xlab = 'Time (ns)' , ylab = 'Voltage (MV)', file = filep
               writepff, 0, loadsnp(npload:2*npload-1), t, $
                     clab = 'Load Current vs Time', $
                     xlab = 'Time (ns)' , blab = 'Current (MA)'
               i2w, t,loadsnp(npload:2*npload-1), $
                     clab = 'Load Current vs Time', $
                     xlab = 'Time (ns)' , ylab = 'Current (MA)', info.out+1
               writepff, 0,loadsnp(2*npload:3*npload-1), t, $
                     c = 'Input Voltage vs Time', $
                     xlab = 'Time (ns)' , blab = 'Voltage (MV)'
               i2w, t, loadsnp(2*npload:3*npload-1), info.out+2, $
                     clab = 'Input Voltage vs Time' , $
                     xlab = 'Time (ns)' , ylab = 'Voltage (MV)', file = filep
               writepff, 0, loadsnp(3*npload:4*npload-1), t, $
                     clab = 'Input Current vs Time', $
                     xlab = 'Time (ns)' , blab = 'Current (MA)'
               i2w, t,loadsnp(3*npload:4*npload-1), $
                     clab = 'Input Current vs Time', $
                     xlab = 'Time (ns)' , ylab = 'Current (MA)', info.out+3
               if typload eq 4 then begin
                 writepff, 0, loadsnp(4*npload+1:5*npload-1), t(1:*)-dt/2., $
                     clab = 'dL/dt - Change in Inductance vs Time', $
                     xlab = 'Time (ns)' , blab = 'Inductance Change (nH/ns)'
                 writepff, 0, loadsnp(5*npload:6*npload-1), t, $
                     clab = 'Inductance vs Time', $
                     xlab = 'Time (ns)' , blab = 'Inductance (nH)'
                 i2w, t(1:*)-dt/2, loadsnp(4*npload+1:5*npload-1), info.out+4, $
                     clab = 'dL/dt - Change in Inductance vs Time', $
                     xlab = 'Time (ns)' , ylab = 'Inductance Change (nH/ns)'
                 i2w, t, loadsnp(5*npload:6*npload-1), info.out+5, $
                     clab = 'Inductance vs Time', $
                     xlab = 'Time (ns)' , ylab = 'Inductance (nH)'
                 icount = 6
               endif else icount = 4
               if tu(0) gt 0.0 then  begin
                 tu = [0.0, tu]
                 uu = [0.0, uu]
               endif 
               nel = n_elements (tu)
               if tu(nel-1) lt t(npload-1) then begin
                 tu = [tu, 1.01*t(npload-1)]
                 uu = [uu, 0.0]
               endif
               put2uniform , tu, uu,n_elements(uu),0.0, dt, $
                   t(npload-1),newu, /yonly
               i2w, t, newu, info.out+ icount, c= 'Forward Going Voltage', $
                     xlab = 'Time (ns)' , ylab = 'Voltage (MV)'
               icount = icount +1
               i2w, t, newu*newu/zsource, info.out+ icount, $
                     c= 'Forward Going Power vs Time', $
                     xlab = 'Time (ns)' , ylab = 'Power (TW)'
               int, info.out + icount, info.out +icount+1, $
                  c = 'Forward Going Energy vs Time', $
                     xlab = 'Time (ns)' , ylab = 'Energy  (kJ)'

               icount = icount +1
               wri, info.out+ icount 
               icount = icount +1
               mul ,info.out +2, info.out +3, icount+info.out
               int,  icount+info.out, $
                   c = 'Input Energy vs Time', $
                  xlab = 'Time (ns)' , ylab = 'Energy  (kJ)'
               icount = icount +1
               mul ,info.out , info.out +1, icount+info.out
               int,  icount+info.out, $
                   c = 'Output Energy vs Time', $
                  xlab = 'Time (ns)' , ylab = 'Energy  (kJ)'
               print, ' Load type: ', strtrim(typload, 2)
               if typload eq 4 then begin
                 icount = icount +1
                 i2w, t, 0.5*loadsnp(5*npload:6*npload-1)* $
                  (loadsnp(npload:2*npload-1))^2 ,  icount+info.out, $
                   c = 'Inductive Load Energy vs Time', $
                  xlab = 'Time (ns)' , ylab = 'Energy  (kJ)'
                 icount = icount +1
                 sub, icount-2+info.out, icount-1+info.out, icount+info.out, $
                   c = 'Net Foil Energy vs Time', $
                  xlab = 'Time (ns)' , ylab = 'Energy  (kJ)'
                 plo, info.out +icount -4, 5, /ov, /left, nextpos= nextpos, $
                       xrange = [0,totalt] , psym = 11+indgen(5), $
                       title = 'Energy Summary for TRIFL Run'
                 xfr, info.out+icount, info.out+icount+1
                 cha, info.out+icount+1, scale = 100.
                 div, info.out+icount+1,info.out+ icount-4, $
                    c = 'Total Efficiency', $
                  xlab = 'Time (ns)' , ylab = 'Efficiency (%)'
                  junk = mx(info.out+4, /quiet, time = time)
                  lim, info.out+icount+1, -1., time+3*dt
               endif else begin
                 plo, info.out +icount-2, 3, /ov, /left, nextpos = nextpos, $
                       xrange = [0,totalt] , psym = 11+indgen(3), $
                       title = 'Energy Summary for TRIFL Run'
                 xfr, info.out+icount, info.out+icount+1
                 cha, info.out+icount+1, scale = 100.
                 div, info.out+icount+1, info.out+icount-2, $
                    c = 'Total Efficiency', $
                  xlab = 'Time (ns)' , ylab = 'Efficiency (%)'
               endelse
               wri, info.out+icount+1
;              legendp, 'Efficiency (%)', lin= 0, pos = nextpos,/data, psym=16
     plo, info.out+icount+1, /right, psym = 16, /ov, lname = 'Efficiency (%)'
                print, 'Total efficiency: ', strtrim( $
                      mx(info.out+icount+1, /quiet), 2), ' %'
               t = findgen(nt)*(dt*nstep)
               xlab = 'Distance (m)'
               ylab = 'Time (ns)'
               y = reform(ia(nx-1, *), nt)

               locx = make_array (2, 2, /long)
               locx (*, 0) = 0
               locx (0, 1) = nx
               locx (1, 1) = nt
               ntot = nx*nt
               range = make_array(2, 2, 2)
               range(0, 0, *) = startx
               range(0, 1, *) = 0.0
               range(1, 0, *) = xcalc(nx-1)
               range(1, 1, *) = t(nt-1)
               spare = [0,4,0,0,0]
               ssize = make_array(2, 1)
               ssize (0:*, 0)= [nx, nt]
               locv = [0, ntot]
               clab = ' A - ' + comment
               tlab = 'TRIFL Field'
               writepff, 0, asnp, [xcalc, t], c = clab, $
                   xlab = xlab, ylab = ylab, tlab=tlab,blab='Vector Potential'

               st ={type:1L, ndim:2L, nvect: 1L, block :1L, two: 3L, $
                 slice: 0.0, size: ssize, range: range, order: 1L, $
                 locx:locx, locv: locv, x1:xcalc, x2:t, $
                 x3:0.0, x4:0.0, v1:temporary(asnp), v2:0.0, $
                 v3:0.0, v4:0.0, v5:0.0, v6:0.0,x1lab : xlab, x2lab :ylab, $
                 x3lab : '', x4lab :'',v1lab : 'Vector Potential', v2lab :'', $
                 v3lab : '', v4lab :'',v5lab : '', v6lab :'', $
                 file : filep, spare : spare,  tlab : 'TRIFL Field', clab: clab}
               i2s, st, info.out, /no_copy

               clab = ' Voltage - ' + comment
               tlab = 'TRIFL Field'
               writepff, 0, vsnp, [xcalc, t], c = clab, $
                   tlab=tlab,blab='Voltage', xlab = xlab , ylab = ylab

               st ={type:1L, ndim:2L, nvect: 1L, block :1L, two: 3L, $
                 slice: 0.0, size: ssize, range: range, order: 0L, $
                 connect: 0L, locx:locx, locv: locv, x1:xcalc, x2:t, $
                 x3:0.0, x4:0.0, v1:temporary(vsnp), v2:0.0, $
                 v3:0.0, v4:0.0, v5:0.0, v6:0.0,x1lab : xlab, x2lab :ylab, $
                 x3lab : '', x4lab :'',v1lab : 'Voltage', v2lab :'', $
                 v3lab : '', v4lab :'',v5lab : '', v6lab :'', $
                 file : filep, spare : spare,  tlab : 'TRIFL Field', clab: clab}
               i2s, st, info.out+1, /no_copy

               clab = ' Q, Charge - ' + comment
               writepff, 0, qsnp, [xcalc, t], c = clab, $
                   xlab = xlab , ylab = ylab,tlab=tlab,blab='Charge'

               st ={type:1L, ndim:2L, nvect: 1L, block :1L, two: 3L, $
                 slice: 0.0, size: ssize, range: range, order: 0L, $
                 connect: 0L, locx:locx, locv: locv, x1:xcalc, x2:t, $
                 x3:0.0, x4:0.0, v1:temporary(qsnp), v2:0.0, $
                 v3:0.0, v4:0.0, v5:0.0, v6:0.0,x1lab : xlab, x2lab :ylab, $
                 x3lab : '', x4lab :'',v1lab : 'Charge', v2lab :'', $
                 v3lab : '', v4lab :'',v5lab : '', v6lab :'', $
                 file : filep, spare : spare,  tlab : 'TRIFL Field', clab: clab}

               i2s, st, info.out+2, /no_copy

               clab = ' Ie - ' + comment
               writepff, 0, isnp, [xcalc, t], c = clab , $
                   xlab = xlab , ylab = ylab, tlab=tlab,blab='Electron Flow'

               st ={type:1L, ndim:2L, nvect: 1L, block :1L, two: 3L, $
                 slice: 0.0, size: ssize, range: range, order: 0L, $
                 connect: 0L, locx:locx, locv: locv, x1:xcalc, x2:t, $
                 x3:0.0, x4:0.0, v1:temporary(isnp), v2:0.0, $
                 v3:0.0, v4:0.0, v5:0.0, v6:0.0,x1lab : xlab, x2lab :ylab, $
                 x3lab : '', x4lab :'',v1lab : 'Electron Current', v2lab :'', $
                 v3lab : '', v4lab :'',v5lab : '', v6lab :'', $
                 file : filep, spare : spare,  tlab : 'TRIFL Field', clab: clab}

               i2s, st, info.out+3, /no_copy

               clab = ' Ia - ' + comment
               writepff, 0, ia, [xcalc, t], c = clab, $
                   xlab = xlab , ylab = ylab, tlab=tlab,blab='Anode Current'

               st ={type:1L, ndim:2L, nvect: 1L, block :1L, two: 3L, $
                 slice: 0.0, size: ssize, range: range, order: 0L, $
                 connect: 0L, locx:locx, locv: locv, x1:xcalc, x2:t, $
                 x3:0.0, x4:0.0, v1:temporary(ia), v2:0.0, $
                 v3:0.0, v4:0.0, v5:0.0, v6:0.0,x1lab : xlab, x2lab :ylab, $
                 x3lab : '', x4lab :'',v1lab : 'Anode Current', v2lab :'', $
                 v3lab : '', v4lab :'',v5lab : '', v6lab :'', $
                 file : filep, spare : spare,  tlab : 'TRIFL Field', clab: clab}

               i2s, st, info.out+4, /no_copy

               clab = ' Ic - ' + comment
               writepff, 0, ic, [xcalc, t], c = clab, $
                   xlab = xlab , ylab = ylab, tlab=tlab,blab='Cathode Current'

               st ={type:1L, ndim:2L, nvect: 1L, block :1L, two: 3L, $
                 slice: 0.0, size: ssize, range: range, order: 0L, $
                 connect: 0L, locx:locx, locv: locv, x1:xcalc, x2:t, $
                 x3:0.0, x4:0.0, v1:temporary(ic), v2:0.0, $
                 v3:0.0, v4:0.0, v5:0.0, v6:0.0,x1lab : xlab, x2lab :ylab, $
                 x3lab : '', x4lab :'',v1lab : 'Cathode Current', v2lab :'', $
                 v3lab : '', v4lab :'',v5lab : '', v6lab :'', $
                 file : filep, spare : spare,  tlab : 'TRIFL Field', clab: clab}

               i2s, st, info.out+5, /no_copy

               clab = ' Qa - ' + comment
             
               writepff, 0, qa, [xcalc, t], c = clab, $
                   xlab = xlab , ylab = ylab,tlab=tlab,blab='Anode Charge'

               st ={type:1L, ndim:2L, nvect: 1L, block :1L, two: 3L, $
                 slice: 0.0, size: ssize, range: range, order: 0L, $
                 connect: 0L, locx:locx, locv: locv, x1:xcalc, x2:t, $
                 x3:0.0, x4:0.0, v1:temporary(qa), v2:0.0, $
                 v3:0.0, v4:0.0, v5:0.0, v6:0.0,x1lab : xlab, x2lab :ylab, $
                 x3lab : '', x4lab :'',v1lab : 'Anode Charge', v2lab :'', $
                 v3lab : '', v4lab :'',v5lab : '', v6lab :'', $
                 file : filep, spare : spare,  tlab : 'TRIFL Field', clab: clab}

               i2s, st, info.out+6, /no_copy

               clab = ' Qc - ' + comment

               writepff, 0, qc, [xcalc, t], c = clab, $
                   xlab = xlab , ylab = ylab,tlab=tlab,blab='Cathode Charge'

               st ={type:1L, ndim:2L, nvect: 1L, block :1L, two: 3L, $
                 slice: 0.0, size: ssize, range: range, order: 0L, $
                 connect: 0L, locx:locx, locv: locv, x1:xcalc, x2:t, $
                 x3:0.0, x4:0.0, v1:temporary(qc), v2:0.0, $
                 v3:0.0, v4:0.0, v5:0.0, v6:0.0,x1lab : xlab, x2lab :ylab, $
                 x3lab : '', x4lab :'',v1lab : 'Cathode Charge', v2lab :'', $
                 v3lab : '', v4lab :'',v5lab : '', v6lab :'', $
                 file : filep, spare : spare,  tlab : 'TRIFL Field', clab: clab}

               i2s, st, info.out+7, /no_copy

               clab = ' TEMPSNP '

               st ={type:1L, ndim:2L, nvect: 1L, block :1L, two: 3L, $
                 slice: 0.0, size: ssize, range: range, order: 0L, $
                 connect: 0L, locx:locx, locv: locv, x1:xcalc, x2:t, $
                 x3:0.0, x4:0.0, v1:temporary(tempsnp), v2:0.0, $
                 v3:0.0, v4:0.0, v5:0.0, v6:0.0,x1lab : xlab, x2lab :ylab, $
                 x3lab : '', x4lab :'',v1lab : 'Cathode Charge', v2lab :'', $
                 v3lab : '', v4lab :'',v5lab : '', v6lab :'', $
                 file : filep, spare : spare,  tlab : 'TRIFL Field', clab: clab}

               i2s, st, info.out+8, /no_copy

             widget_control, event.top, /destroy
         endif
         endelse
         END

      1: BEGIN
           Print,'Quit without Saving data'
           widget_control, event.top, /destroy
         END
      ELSE: Message,'Unknown button pressed'
      ENDCASE
      END

  ENDCASE
END


PRO TRIFL, par1, par2, par3, par4,out = out, transition = tr , $
            cwmende = cwmende, GROUP=Group, convolute = convolute, $
            flowout = flowout

common mitl_common
; common block
      common pffcom, initpff, pff_exe
;
;     check to see if pff has been initialized
;
      on_error, 1
      if n_elements(initpff) eq 0 then init_pff else $
        if (initpff eq 0) then init_pff
      if initpff eq 0 then begin
        print, '% TRIFL: TRIFL is not supported by this Operating System'
        return
      endif
  IF n_elements(group) eq 0 THEN group=0
  if n_elements(cwmende) eq 0 then cwmende = 0
  if n_elements(lcflow) eq 0 then lcflow = 1
  if cwmende ne 0 then cwmende = 1
  lcexpert = cwmende
  npar = n_params()
  str = {np:0}
  loadxyct
  if npar gt 0 then begin
    np = n_elements(par1)
    if np lt 2 then begin
      if n_elements(par2) gt 1 then begin
        print, ' TRIFL must be called with 1 or 2 valid waveform arrays.'
        return
      endif
      if n_elements (par2 ) eq 0 then begin
        w2i, x, y, par1
        np = n_elements (x)
        if np  gt 1 then y = reform ([y, 0.0*y], np, 2)
      endif else begin
        wriasc,  0, [par1, par2], /internal, x= x , yout = y
      endelse
    endif else if npar eq 3 then begin
      if np eq n_elements(par2) and np eq n_elements(par3) then begin
        x = par1
        y = (reform([par1, par2], np, 2))
      endif else begin
        print, 'Three input parameters to TRIFL must have the same number', $
             ' of elements'
        return
      endelse
    endif else begin
      b = size(par2)
      if b(1) ne np then begin
        print, ' Length of x and y (or Z) arrays must be equal'
        return
      endif
      x = par1
      if b(0) eq 3 then tr = par2(*, 3)
      if b(0) ge 2 then y = par2(*, 0:1) else y=reform([par2, 0.0*par2],np,2)
    endelse
    if n_elements(tr) eq np then fit=round(tr) else begin
      if n_elements(tr) gt 0 then $
        fit = make_array(np, /int, value = fix(tr(0))) else $
        fit = make_array(np, /int)
    endelse
    if n_elements(convolute) lt np then convolute = make_array(np, val= 0.0) $
      else convolute = abs(convolute(0:np-1)) 
    if n_elements(flowout) lt np then flowout = make_array(np, val= 0.0) $
      else flowout = abs(flowout(0:np-1)) 
    add_mitlpnts, np, float(x), float(y(*, 0)),float(y(*, 1)), fit, convolute, $
       flowout, /nofill
  endif else str = {np:0L, npos:0L}
  
  lcdefault = 1
  result = findfile('*.src', count= count)
  if count gt 1 then begin
    cd , current= current
    deffile = pickfile(path= current, filter= '*.src', $
                   title = 'Pick DEFAULT source File')
    if strtrim(deffile, 2) eq '' then lcdefault = 0
  endif else if count eq 1 then deffile = result(0) $
    else lcdefault = 0
  if lcdefault ne 0 then begin
    openr, unit, /get, deffile
    read_mitl_src, unit, dxcom, dt, outdt, totalt, zsource, stype,$
      wdfa, rtype, $
      tpeak, ftype, tfall, twidth, umax, ltype, wdfb, rltype, tlpeak, $
      fltype, tlfall, tlwidth, peakz, plmass, plength, pradius, plmin, $
      plret, lcflow, qemit, lcflash
    free_lun, unit
  endif
  offset = [0,0]
  get_source, lcquit, tu, uu, zsource, stype, wdfa, rtype, tpeak, $
           ftype, tfall, twidth, umax, offset, lcflow, qemit, lcflash
  if lcquit ne 0 then return
  get_load, lcquit, tt, zz, ltype, wdfb, rltype, tlpeak, fltype, tlfall,$
            tlwidth, peakz, plmass, plength, pradius, plmin, plret, offset
  if lcquit ne 0 then return
  
  MAIN13 = WIDGET_BASE(GROUP_LEADER=Group, $
      COLUMN=1, $
      MAP=1, $
      TITLE='TRIFL Simulation', $
      UVALUE='TRIFL')

  Btns122 = [ $
    'Read Geometry', $
    'Add Points', $
    'Delete Points', $
    'Edit Points' ]
  BGROUP24 = CW_BGROUP( MAIN13, Btns122, $
      ROW=1, $
      FRAME=4, /NO_RELEASE, $
      UVALUE='BGROUP24')

  BASE17 = WIDGET_BASE(MAIN13, $
      COLUMN=1, $
      FRAME=5, $
      MAP=1, $
      UVALUE='BASE17')

  LABEL15 = WIDGET_LABEL( BASE17, $
      UVALUE='LABEL15', $
      VALUE='Display Selection')

  LABEL28 = WIDGET_LABEL( BASE17, $
      UVALUE='LABEL28', $
      VALUE='1 - Vacuum Impedance/Flow Impedance')

  LABEL29 = WIDGET_LABEL( BASE17, $
      UVALUE='LABEL29', $
      VALUE='2 - Flow Impedance, (Flow - Vacuum) Impedance')

  Btns129 = [ $
    '1', $
    '2' ]
  BGROUP33 = CW_BGROUP( BASE17, Btns129, $
      /row, $
      EXCLUSIVE=1, /NO_RELEASE, $
      UVALUE='BGROUP33')

  BASE37 = WIDGET_BASE(BASE17, $
      COLUMN=1, $
      MAP=1, $
      UVALUE='BASE37')

  LABEL38 = WIDGET_LABEL( BASE37, $
      UVALUE='LABEL38', $
      VALUE='Display Data Range')

  BASE40 = WIDGET_BASE(BASE37, $
      ROW=1, $
      MAP=1, $
      UVALUE='BASE40')

  FSLID41 = cw_fslider( BASE40, $
      MAXIMUM=1.00000, $
      MINIMUM=0.00000, $
        /drag,/edit,  $
      TITLE='Minimum Value', $
      UVALUE='FSLID41', $
      VALUE=0.00000)

  FSLID42 = cw_fslider( BASE40, $
      MAXIMUM=1.00000, $
      MINIMUM=0.00000, $
        /drag, /edit, $
      TITLE='Maximum Value', $
      UVALUE='FSLID42', $
      VALUE=1.00000)


; BUTTON49 = WIDGET_BUTTON( BASE37, $
;     UVALUE='BUTTON49', $
;     VALUE=' Plot Data ')


  Btns137 = [ $
    '     Run / Save     ', $
    'Quit without Saving' ]
  BGROUP34 = CW_BGROUP( MAIN13, Btns137, $
      ROW=1, /NO_RELEASE, $
      FRAME=5, $
      UVALUE='BGROUP34')
  if n_elements (out) eq 0 then out = 1 else out = out > 0 

  if n_elements (offset) lt 2  then offset = [10, 10] else $
    offset = (offset(0:1)> 0)
  WIDGET_CONTROL, main13, /REALIZE, tlb_set_xoffset = offset(0), $
       tlb_set_yoffset= offset(1)

  widget_control, bgroup33, set_value = 0
  info = {ptype:bgroup33, zoommin: fslid41, zoommax:fslid42, out : out}
  if str.np gt 1 then begin
    fillmitlstr
    get_zoom_values, info, /maximum
    mitl_plot, 0, xrange = 0
  endif
  widget_control, main13, set_uvalue = info, /no_copy
  XMANAGER, 'TRIFL', MAIN13

END
