; ------------------------------------------------------------------------ 
; $Id: unionstr.pro,v 1.4 2010/12/08 16:32:12 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
 Pro  unionstr, wdf1, wdf2, wdf3, clabel = clabel
;
;+
; NAME:
;       UNIONSTR
;
; PURPOSE:
;       Combine two structures into a composite structure.
;       Both structures should have the same block structure.
;       All vector quantities in the region of overlap are added.
;       Note: If the quantities have identical blocks and grid data, then 
;             the results will be identical to ADDSTR
; CATEGORY:
;       Math 
; CALLING SEQUENCE:
;       UNIONSTR, STR1, STR2,  [, STR3]
; INPUTS:
;       STR1:    Structure or structure array number of a Type 1 structure
;       STR2:    Structure or structure array number of a Type 1 structure
; Note: STR1 and STR2 should have the same block configuration as described 
;       in the procedure.  
;       STR1 and STR2 must both have the same number of dimensions.
; OPTIONAL INPUT:
;       STR3:    array number for the sum, if not specified the 
;                 sum is stored in STR1 (the first array)
;                 The form of STR3 is the same as STR1.
; KEYWORDS:
;       CLABEL:  Optional label for the combined structures.
; OUTPUTS:
;       NONE    The sum is stored in a structure array in the common block
; COMMON BLOCKS:
;       QSCOM.DAT QSSTRCOM.DAT
; RESTRICTIONS:
;       STR1 and STR2 arrays must be defined and have valid data.
;       STR1 and STR2 should have the same block structure.
;       Number of dimensions, NDIM, should be less than 4.
; PROCEDURE:
;       Begin with the first block in STR1 and determine if STR2 has any blocks
;         which overlap this block.  The block will be enlarged to include all 
;         data.  Data from STR1 and STR2 which overlap will be added.
;         A minimum of 4 points are required to add a block.
;       This procedure will be repeated for each block in STR1. 
;       If STR2 has blocks which have not been included in the UNION structure,
;         they will be added.
;       Equal is defined with an allowance of 2^(-15) (3.05176e-05) of the
;         range of the data.
;       Both structures should have the same grid values, but only the total
;         number of values between two grid limits is checked.
; EXAMPLES:
;       Combine structure arrays 2 and 3 and store the result in structure 8
;               UNIONSTR, 2, 3, 8
;
;       Add structure arrays 3 and 8 and store the result in structure 10
;       with a dataset comment, 'Total Charge - Line + Diode'
;               UNIONSTR, 3, 8, 10, c=  'Total Charge - Line + Diode'
; MODIFICATION HISTORY:
;      Original version, L. P. Mix, JULY 5, 2000
;-
;
; insert a common block
;
@qscom.dat
;  Common block
@qsstrcom.dat
;       return on an error
      on_error, 1
;
      if n_elements (structinit) eq 0 then qsstruct_init
      if n_elements (qsplotinit) eq 0 then qsinit
;
      nparams = n_params()
      if nparams eq 0 then begin
        print,'% Format: ', $
       'UNIONSTR, STR1, STR2,  [, STR3] [, CLABEL = clabel] '
        return
      endif

      if nparams lt 2 then begin
        print,'% UNIONSTR: 2 structures must be provided.'
        return
      endif
;
      if n_elements(wdf1) eq 0 or  n_elements(wdf2) eq 0 then begin
        print, '% UNIONSTR: Parameters must be defined'
        return
      endif
;
      array1 = get_valid_structure(wdf1, 'UnionSTR', lcstr=lcstr, ierror=ier)
      if ier ne 0 then return
      if (array1.type ne 1) then begin
        print, '% UNIONSTR:  First parameter must be Type 1 field structure'
        return
      endif
;
      array2 = get_valid_structure( wdf2, 'UnionSTR', ierror=ier)
      if ier ne 0 then return
      if (array2.type ne 1) then begin
        print, '% UNIONSTR:  Second parameter must be Type 1 field structure'
        return
      endif
;
      nblo1 = array1.block
      nblo2 = array2.block
      ndim = array1.ndim
      ndim2 = array2.ndim
      if ndim ne ndim2 then begin
        print, '% UNIONSTR:  Arrays must have the same number of dimensions.'
        return
      endif
      if ndim gt 3 then begin
        print, $
           '% UNIONSTR: Structures must have less than 4 dimensions.'
        return
      endif
      if ndim lt 3 then begin
        ndim = 3
        two = 3
      endif else begin
        two = [array1.two, array2.two]
        ok = where(two gt 0, nok2)
        if nok2 eq 2 then if (array1.two ne array2.two) then begin
          print, $
           '% UNIONSTR: Structures must have the same degenerate dimension.'
          return
        endif
        if nok2 gt 0 then two = two[ok[0]] else two = 0
      endelse
      nvect1 = array1.nvect
      nvect2 = array2.nvect
      nbt = nblo1+nblo2
      nbtp = nbt+1

;     make an array of used blocks
      ublo = make_array(nblo2, value=0)

      locx = make_array(ndim, nbtp, /long)
      locx[*, 0] = 0
      nblo = 0
      locv = make_array(nbtp, /long)
      locv[0] = 0
      v1lab = make_array(nbt, /string, value = array1.v1lab[0])
      v2lab = make_array(nbt, /string, value = array1.v2lab[0])
      v3lab = make_array(nbt, /string, value = array1.v3lab[0])
      v4lab = make_array(nbt, /string, value = array1.v4lab[0])
      v5lab = make_array(nbt, /string, value = array1.v5lab[0])
      v6lab = make_array(nbt, /string, value = array1.v6lab[0])
      x1lab = make_array(nbt, /string, value = array1.x1lab[0])
      x2lab = make_array(nbt, /string, value = array1.x2lab[0])
      x3lab = make_array(nbt, /string, value = array1.x3lab[0])
      x4lab = make_array(nbt, /string, value = '')
     
      mvect = max([nvect1, nvect2])
      mxv = n_elements(array1.v1) + n_elements(array2.v1)
      ssize = make_array(ndim, nbt, /long)
      v1 = make_array(mxv)
      if mvect gt 1 then v2 = make_array(mxv) else v2 = make_array(nbt)
      if mvect gt 2 then v3 = make_array(mxv) else v3 = make_array(nbt)
      if mvect gt 3 then v4 = make_array(mxv) else v4 = make_array(nbt)
      if mvect gt 4 then v5 = make_array(mxv) else v5 = make_array(nbt)
      if mvect gt 5 then v6 = make_array(mxv) else v6 = make_array(nbt)

      r1 = array1.range
      r2 = array2.range
      range = r1

      eps =  max([max(range[*, *, 0])* 2.3841858e-07, $
         max(range[1, *, 0]-range[0, *, 0])/32768])
      r2[0, *, *] = r2[0, *, *]-eps
      r2[1, *, *] = r2[1, *, *]+eps
      range = make_array(2, 3, nbtp)
      for i = 0, nblo1-1 do begin
        iblo = i+1
        x = get_array(array1,1, iblo, lab = xl) 
        y = get_array(array1,2, iblo, lab = yl) 
        if ndim2  eq 3 then $
          z = get_array(array1,3, iblo, lab = zl) else begin
            z=0.0
            zl = ''
        endelse

        
        j = 0
        match = 0
        while ((match eq 0)  and (j lt nblo2)) do begin
          if ublo[j] eq 0 then begin
;           block = (j+1) is has not been used yet
            jblo = j+1
            ok = where(x ge r2[0,0,jblo] and x le r2[1,0,jblo], nokx)
            ok = where(y ge r2[0,1,jblo] and y le r2[1,1,jblo], noky)
            if ndim2 eq 3 then $
              ok = where(z ge r2[0,2,jblo] and z le r2[1,2,jblo], nokz) $
              else nokz = 1
            nover = [nokx, noky, nokz]
            if min(nover) gt 0 then begin
              ok = where (nover eq 1, nok)
               
              if (nok eq 0) or ((nok eq 1) and (ok[0]+1 eq two)) then begin
;               we have a match
                match = 1
                ublo[j]=1
                j = nblo2
              endif
            endif
          endif
          j = j+1
        endwhile
        if match ne 0 then begin
          get_array_union, x, get_array(array2, 1, jblo), xtotal, $
            ind1x, ind2x, ran, ier 
          if ier ne 0 then begin
            Print,'% UNIONSTR: Error getting X overlap:', iblo, jblo
            return
          endif

          get_array_union, y, get_array(array2, 2, jblo), ytotal, $
            ind1y, ind2y, ran, ier 
          if ier ne 0 then begin
            print,'% UNIONSTR: Error getting Y overlap:', iblo, jblo
            return
          endif

          if ndim2 eq 2 then begin
            ztotal = 0.0
            ind1z = [0,0]
            ind2z = [0,0]
            ier =0
            ran =[0.0, 0.0]
          endif else $
            get_array_union, z, get_array(array2, 3, jblo), ztotal, $
              ind1z, ind2z, ran, ier 
          if ier ne 0 then begin
            print,'% UNIONSTR: Error getting Z overlap:', iblo, jblo
            return
          endif
          nx = n_elements(xtotal)
          ny = n_elements(ytotal)
          nz = n_elements(ztotal)
          ssize(*, iblo-1) = [nx, ny, nz]
          locx[*, iblo] = locx[*, iblo-1]+ssize(*, iblo-1)

          var= make_array(nx, ny, nz)
          nvar = n_elements(var)
          locv [iblo] = locv[iblo-1]+nvar
          if iblo eq 1 then begin
            x1 = xtotal
            x2 = ytotal
            x3 = ztotal
          endif else begin
            x1 = [x1, xtotal]
            x2 = [x2, ytotal]
            x3 = [x3, ztotal]
          endelse

          if locv[iblo] gt mxv then begin
            nadd = n_elements(var)
            mxv = mxv+nadd
            v1 = [v1,make_array(nadd)]
            if mvect gt 1 then v2 = [v2,make_array(nadd)]
            if mvect gt 2 then v3 = [v2,make_array(nadd)]
            if mvect gt 3 then v4 = [v2,make_array(nadd)]
            if mvect gt 4 then v5 = [v2,make_array(nadd)]
            if mvect gt 5 then v6 = [v2,make_array(nadd)]
          endif
          var[ind2x[0]:ind2x[1], ind2y[0]:ind2y[1], ind2z[0]:ind2z[1]] = $
            get_array(array2, -1, jblo, lab = ll )
          var[ind1x[0]:ind1x[1], ind1y[0]:ind1y[1], ind1z[0]:ind1z[1]] = $
            var[ind1x[0]:ind1x[1], ind1y[0]:ind1y[1], ind1z[0]:ind1z[1]] $
            + get_array(array1, -1, iblo, lab = ll)
          v1[locv[iblo-1]:locv[iblo]-1] = reform(var,  nvar)
          v1lab[iblo-1] = ll

          if mvect gt 1 then begin
            var[*]=0.0
            if nvect2 gt 1 then $
              var[ind2x[0]:ind2x[1], ind2y[0]:ind2y[1], ind2z[0]:ind2z[1]] =$
              get_array(array2, -2, jblo, lab = ll)
            if nvect1 gt 1 then $
              var[ind1x[0]:ind1x[1], ind1y[0]:ind1y[1], ind1z[0]:ind1z[1]] =$
              var[ind1x[0]:ind1x[1], ind1y[0]:ind1y[1], ind1z[0]:ind1z[1]] $
               + get_array(array1, -2, iblo, lab = ll) 
            v2[locv[iblo-1]:locv[iblo]-1] = reform(var,  nvar)
            v2lab[iblo-1] = ll
            if mvect gt 2 then begin
            var[*]=0.0
            if nvect2 gt 2 then $
              var[ind2x[0]:ind2x[1], ind2y[0]:ind2y[1], ind2z[0]:ind2z[1]] =$
              get_array(array2, -3, jblo, lab = ll)
            if nvect1 gt 2 then $
              var[ind1x[0]:ind1x[1], ind1y[0]:ind1y[1], ind1z[0]:ind1z[1]] =$
              var[ind1x[0]:ind1x[1], ind1y[0]:ind1y[1], ind1z[0]:ind1z[1]] $
               + get_array(array1, -3, iblo, lab = ll)
            v3[locv[iblo-1]:locv[iblo]-1] = reform(var,  nvar)
            v3lab[iblo-1] = ll
            if mvect gt 3 then begin
            var[*]=0.0
            if nvect2 gt 3 then $
              var[ind2x[0]:ind2x[1], ind2y[0]:ind2y[1], ind2z[0]:ind2z[1]] =$
              get_array(array2, -4, jblo, lab = ll)
            if nvect1 gt 3 then $
              var[ind1x[0]:ind1x[1], ind1y[0]:ind1y[1], ind1z[0]:ind1z[1]] =$
              var[ind1x[0]:ind1x[1], ind1y[0]:ind1y[1], ind1z[0]:ind1z[1]] $
               + get_array(array1, -4, iblo, lab = ll)
            v4[locv[iblo-1]:locv[iblo]-1] = reform(var,  nvar)
            v4lab[iblo-1] = ll
            if mvect gt 4 then begin
            var[*]=0.0
            if nvect2 gt 4 then $
              var[ind2x[0]:ind2x[1], ind2y[0]:ind2y[1], ind2z[0]:ind2z[1]] =$
              get_array(array2, -5, jblo, lab = ll)
            if nvect1 gt 4 then $
              var[ind1x[0]:ind1x[1], ind1y[0]:ind1y[1], ind1z[0]:ind1z[1]] =$
              var[ind1x[0]:ind1x[1], ind1y[0]:ind1y[1], ind1z[0]:ind1z[1]] $
               + get_array(array1, -5, iblo, lab = ll)
            v5[locv[iblo-1]:locv[iblo]-1] = reform(var,  nvar)
            v5lab[iblo-1] = ll
            if mvect gt 5 then begin
            var[*]=0.0
            if nvect2 gt 5 then $
              var[ind2x[0]:ind2x[1], ind2y[0]:ind2y[1], ind2z[0]:ind2z[1]] =$
              get_array(array2, -6, jblo, lab = ll)
            if nvect1 gt 5 then $
              var[ind1x[0]:ind1x[1], ind1y[0]:ind1y[1], ind1z[0]:ind1z[1]] =$
              var[ind1x[0]:ind1x[1], ind1y[0]:ind1y[1], ind1z[0]:ind1z[1]] $
               + get_array(array1, -6, iblo, lab = ll)
            v6[locv[iblo-1]:locv[iblo]-1] = reform(var,  nvar)
            v6lab[iblo-1] = ll
            endif ; for V6
            endif ; for V5
            endif ; for V4
            endif ; for V3
          endif ; for V2
          range[*, 0 , iblo] = [min(xtotal, max = mm), mm]
          range[*, 1 , iblo] = [min(ytotal, max = mm), mm]
          range[*, 2 , iblo] = [min(ztotal, max = mm), mm]
          x1lab[iblo-1]= xl
          x2lab[iblo-1]= yl
          x3lab[iblo-1]= zl
        endif else begin
          if ndim2 eq 3 then ssize[*, iblo-1]=array1.size[*, iblo-1] $ 
            else ssize[*, iblo-1]= [reform(array1.size[*, iblo-1]), 1]
          locv[iblo] = locv[iblo-1] +array1.locv[iblo]-array1.locv[iblo-1]
          locx[*, iblo]= locx[*, iblo-1]+ssize[*, iblo-1]
          if locv[iblo] gt mxv then begin
            nadd = array1.locv[nblo1]-array1.locv[iblo-1]
            mxv = mxv+nadd
            v1 = [v1,make_array(nadd)]
            if mvect gt 1 then v2 = [v2,make_array(nadd)]
            if mvect gt 2 then v3 = [v2,make_array(nadd)]
            if mvect gt 3 then v4 = [v2,make_array(nadd)]
            if mvect gt 4 then v5 = [v2,make_array(nadd)]
            if mvect gt 5 then v6 = [v2,make_array(nadd)]
          endif
          if iblo eq 1 then begin
            x1 = x
            x2 = y
            x3 = z
          endif else begin
            x1 = [x1, x]
            x2 = [x2, y]
            x3 = [x3, z]
          endelse
          v1[locv[iblo-1]:locv[iblo]-1] = $
            array1.v1[array1.locv[iblo-1]:array1.locv[iblo]-1]
          v1lab[iblo-1] = array1.v1lab[iblo-1]
          if nvect1 gt 1 then begin
            v2[locv[iblo-1]:locv[iblo]-1] = $
              array1.v2[array1.locv[iblo-1]:array1.locv[iblo]-1]
            v2lab[iblo-1] = array1.v2lab[iblo-1]
            if nvect1 gt 2 then begin
              v3[locv[iblo-1]:locv[iblo]-1] = $
                array1.v3[array1.locv[iblo-1]:array1.locv[iblo]-1]
              v3lab[iblo-1] = array1.v3lab[iblo-1]
              if nvect1 gt 3 then begin
                v4[locv[iblo-1]:locv[iblo]-1] = $
                  array1.v4[array1.locv[iblo-1]:array1.locv[iblo]-1]
                v4lab[iblo-1] = array1.v4lab[iblo-1]
                if nvect1 gt 4 then begin
                  v5[locv[iblo-1]:locv[iblo]-1] = $
                    array1.v5[array1.locv[iblo-1]:array1.locv[iblo]-1]
                  v5lab[iblo-1] = array1.v5lab[iblo-1]
                  if nvect1 gt 5 then begin
                    v6[locv[iblo-1]:locv[iblo]-1] = $
                      array1.v6[array1.locv[iblo-1]:array1.locv[iblo]-1]
                    v6lab[iblo-1] = array1.v6lab[iblo-1]
                  endif ; for v6
                endif  ; for v5
              endif   ; for v4
            endif    ; for v3
          endif     ; for v2
          x1lab[iblo-1]= xl
          x2lab[iblo-1]= yl
          x3lab[iblo-1]= zl
          range[*, 0:ndim2-1, iblo] = array1.range[*, *, iblo]
        endelse
      endfor
      ok = where (ublo eq 0, nok)
      iblo = nblo1
      if nok gt 0 then begin
        for i = 0, nok-1 do begin
          iblo = iblo+1
          jblo = ok[i]+1
          if ndim2 eq 3 then ssize[*, iblo-1]=array2.size[*, jblo-1] $ 
            else ssize[*, iblo-1]= [reform(array2.size[*, jblo-1]), 1]
          locv[iblo] = locv[iblo-1] +array2.locv[jblo]-array2.locv[jblo-1]
          locx[*, iblo]= locx[*, iblo-1]+ssize[*, iblo-1]
          if locv[iblo] gt mxv then begin
            nadd = 2* (locv[iblo]-locv[iblo-1])
            mxv = mxv+nadd
            v1 = [v1,make_array(nadd)]
            if mvect gt 1 then v2 = [v2,make_array(nadd)]
            if mvect gt 2 then v3 = [v2,make_array(nadd)]
            if mvect gt 3 then v4 = [v2,make_array(nadd)]
            if mvect gt 4 then v5 = [v2,make_array(nadd)]
            if mvect gt 5 then v6 = [v2,make_array(nadd)]
          endif
          x1 = [x1, get_array(array2, 1, jblo, lab = xl)]
          x2 = [x2, get_array(array2, 2, jblo, lab = yl)]
          if ndim2 eq 3 then $
          x3 = [x3, get_array(array2, 3, jblo, lab = zl)] else begin
            x3=[x3, 0.0]
            zl = ''
          endelse
          v1[locv[iblo-1]:locv[iblo]-1] = $
            array2.v1[array2.locv[jblo-1]:array2.locv[jblo]-1]
          v1lab[iblo-1] = array2.v1lab[jblo-1]
          if nvect1 gt 1 then begin
            v2[locv[iblo-1]:locv[iblo]-1] = $
              array2.v2[array2.locv[jblo-1]:array2.locv[jblo]-1]
            v2lab[iblo-1] = array2.v2lab[jblo-1]
            if nvect1 gt 2 then begin
              v3[locv[iblo-1]:locv[iblo]-1] = $
                array2.v3[array2.locv[jblo-1]:array2.locv[jblo]-1]
              v3lab[iblo-1] = array2.v3lab[jblo-1]
              if nvect1 gt 3 then begin
                v4[locv[iblo-1]:locv[iblo]-1] = $
                  array2.v4[array2.locv[jblo-1]:array2.locv[jblo]-1]
                v4lab[iblo-1] = array2.v4lab[jblo-1]
                if nvect1 gt 4 then begin
                  v5[locv[iblo-1]:locv[iblo]-1] = $
                    array2.v5[array2.locv[jblo-1]:array2.locv[jblo]-1]
                  v5lab[iblo-1] = array2.v5lab[jblo-1]
                  if nvect1 gt 5 then begin
                    v6[locv[iblo-1]:locv[iblo]-1] = $
                      array2.v6[array2.locv[jblo-1]:array2.locv[jblo]-1]
                    v6lab[iblo-1] = array2.v6lab[jblo-1]
                  endif ; for v6
                endif  ; for v5
              endif   ; for v4
            endif    ; for v3
          endif     ; for v2
          x1lab[iblo-1]= xl
          x2lab[iblo-1]= yl
          x3lab[iblo-1]= zl
          range[*, 0:ndim2-1, iblo] = array2.range[*, *, jblo]
          
        endfor
      endif
      range[0, 0, 0] = min(range[*, 0, 1:*], max = mm)
      range[1, 0, 0] = mm
      range[0, 1, 0] = min(range[*, 1, 1:*], max = mm)
      range[1, 1, 0] = mm
      range[0, 2, 0] = min(range[*, 2, 1:*], max = mm)
      range[1, 2, 0] = mm
      
      v1 = temporary(v1[0: locv[iblo]-1])
      if mvect gt 1 then v2 = temporary(v2[0: locv[iblo]-1])
      if mvect gt 2 then v3 = temporary(v3[0: locv[iblo]-1])
      if mvect gt 3 then v4 = temporary(v4[0: locv[iblo]-1])
      if mvect gt 4 then v5 = temporary(v5[0: locv[iblo]-1])
      if mvect gt 5 then v6 = temporary(v6[0: locv[iblo]-1])
      range = range[*, *, 0:iblo]
      ssize = ssize[*, 0:iblo-1]
      locv = locv[0:iblo]
      locx = locx[*, 0:iblo]
      x1lab = x1lab[0:iblo-1]
      x2lab = x2lab[0:iblo-1]
      x3lab = x3lab[0:iblo-1]

      v1lab = v1lab[0:iblo-1]
      if mvect gt 1 then v2lab = v2lab[0:iblo-1]
      if mvect gt 2 then v3lab = v3lab[0:iblo-1]
      if mvect gt 3 then v4lab = v4lab[0:iblo-1]
      if mvect gt 4 then v5lab = v5lab[0:iblo-1]
      if mvect gt 5 then v6lab = v6lab[0:iblo-1]
      if min(ssize) gt 1 then two = 0
      if two ne 0 then slice = array1.slice else slice = 0.0

      dummy = (iblo eq 1) ? 0.0 : make_array(iblo, value =0.0)
      if n_elements(clabel) gt 0 then clab = string(clabel) else $
         clab = array1.clab
      array1 = {type:1L, ndim:ndim, nvect:mvect, block:iblo, two:two, $
              size:ssize, range: range, order: 1L, slice:slice, $
              locx:locx, locv : locv, x1:x1, x2: x2, x3:x3, x4:0.0, $
              dx1:dummy, dx2:dummy, dx3:dummy, dx4:dummy, $ 
              v1:v1, v2:v2, v3:v3, v4:v4, v5:v5, v6:v6, $
              x1lab :x1lab, x2lab :x2lab, x3lab :x3lab, $
              x4lab :x4lab, v1lab :v1lab, v2lab :v2lab, $
              v3lab :v3lab, v4lab :v4lab, v5lab :v5lab, $
              v6lab :v6lab, file: array1.file, spare: array1.spare, $
              tlab : array1.tlab, clab:clab }

      if nparams gt 2 then begin
        if lcstr eq 1 then wdf3 = temporary(array1) else begin
          wdf3 = get_save_str(wdf3, wdf1) 
          i2s, array1, wdf3, /no_copy
        endelse
      endif else begin
        if lcstr eq 1 then wdf1 = temporary(array1) else $
          i2s, array1, wdf1, /no_copy
      endelse
      array2=0
      return
      end
