; ------------------------------------------------------------------------ 
; $Id: warp_image.pro,v 1.6 2011/03/15 19:59:13 lpmix Exp $
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
;
;+
; NAME:
;       Warp_Image
;
; PURPOSE:
;       Change the dimensions of an image or single block field dataset.
;         Y scaling is a function of X or conversely.
;       Only single block, type 1 (Rectilinear), data is supported.
; CATEGORY: 
;       Utility 
; CALLING SEQUENCE: 
;       Warp_Image, STR
; INPUTS: 
;       STR:      Structure array or Structure Array Number
;
; KEYWORD PARAMETERS:
;   Note: POLY in the descriptions below takes an array X and Coefficients C
;         and calculates: POLY(X,C)= C[0] + X*C[1] + X*X*C[2] + ... +X^(M)*C[M]
;         where M = N_Elements(C)-1
;
;       WDF:      If set MDIM and RDIM are taken as WDF arrays.  The X-values of the
;                 structure are used to determine the Y values of the WDF array.
;                 These ordinates are used in place of the POLY function below.
;       OFFSET:   If present and not zero, this value will be subtracted from the 
;                 the current Y values before the scaling occurs.  This line will
;                 remain horizontal.
;       TRANSPOSE:  Scale X according to the Y value.
;                 Default: Y = Y(F(X))
;       MDIM:     Multiply the current axis values by this coefficient or 
;                 array of coefficients.
;                 Example: 
;                   Y is current set of dimensions
;                   Ynew = Y*POLY(X, Mdim) is the new set of dimensions
;                 If WDF is set, POLY(X, MDIM) ==>> GET_VALUE(MDIM, X)
;           Note: Present version may not handle any function (MDIM values) which
;                 do not preserve the order of YNEW.
;        RDIM:    Reciprocal of the spacing between points vs X.
;                 If MDIM has an element, then RDIM is ignored.
;                 One such value might be the magnification of an image.  By dividing
;                 by the magnification, the size at the source can be interpreted.
;                 Example: 
;                   Y is current set of dimensions
;                   Ynew = Y/POLY(X, RDIM) is the new set of dimensions
;                 If WDF is set, POLY(X, RDIM) ==>> GET_VALUE(RDIM, X)
;       CONSERVE: Preserve the total integral over the image.
;                 Set this if the image represents a film value and a smaller value
;                 of DY represents a higher value at the target.
;       LABEL:    Replace the DIM axis label with this string.
;       Keep_all: Replace STRARRAY with a finite element structure
;                 Default:  Uniform Y scale between Max and Min with same number of points.
; OUTPUTS:
;       Rescaled structure.
;               
; COMMON BLOCKS:
;       QSCOM, QSSTRCOM 
; RESTRICTIONS:
;       STR array must be defined and have valid data
;       DIM must be specified
;       Mdim must be specified.
;
; EXAMPLES:
;       Add .1 + 0.01*X to every Y value in structure 2
;            WARP_Image, 2, mdim =[0.1, 0.01]
;
; MODIFICATION HISTORY:
;      Original version, L. P. Mix, June 1, 2004
;-
; Based on scale image
;
; following are not implemented   
;
;       DDIM:     Calculate the spacing between points by this coefficient or 
;                 array of coefficients.
;                 Example:
;                   X is current set of dimensions
;                   Xave is average bin value with one fewer elements.
;                   Xave[i]= (X[i]+X[i+1])*0.5
;                   dx is the spacing
;                   dx[i] = (X[i+1]-X[i])
;                   DX = poly(Xave, DDIM)
;                   Xnew = [0, total(dx*DX, /cum)
;       NDIM:     Calculate the spacing between points by this coefficient or 
;                 array of coefficients.  This is similar to DDIM but ignores
;                 original position values.
;                 Example: 
;                   X is current set of dimensions with N values
;                   DX = poly(findgen(n-1)+0.5, NDIM)
;                   Xnew = [0, total(DX, /cum)
;
 Pro  warp_image, strarray, transpose = transpose, label = labelin, $
           conserve = conserve, Mdim = Mdim, keep_all = keep_all, $
           offset = offset, WDF = wdf, Rdim = rdim
;                Ddim = ddim, Ndim = ndim
on_error, 1
;     include the common block
;
@qscom.dat
;  Common block
@qsstrcom.dat
;
      if n_elements (structinit) eq 0 then qsstruct_init
      if n_elements (qsplotinit) eq 0 then qsinit
;
      nparams = n_params()
      if nparams eq 0 then begin
        print,'% Format: WARP_IMAGE, STRARRAY [, LABEL= lab]'
        print,'          [, /CONSERVE] [, /TRANSPOSE] [, MDIM= mdim]'
; print, '         [, RDIM = rdim] [, NDIM = ndim] [, DDIM = ddim]'
;
        return
      endif
;
;     determine if strarray is specified
;
      if nparams lt 1 then begin
        print, '% Warp_Image: Structure must be provided'
        return 
      endif
      nelstr = n_elements(strarray)
      if nelstr eq 0 then begin
        print, '% Warp_Image: Structure must be defined'
        return
      endif else strarray= strarray[0]
      
      nm = n_elements(mdim) + n_elements(rdim)
      if nm eq 0 then begin
        print, '% Warp_Image: One set of coefficients must be provided'
        help, mdim, rdim
        return
      endif

      str = get_valid_structure(strarray, 'Warp_Image', lcstr=lcstr, $
        ierror=ier)
      if IER ne 0 then return

      if str.type ne 1 then begin
        print, '% Warp_Image: Only type 1 structures may be scaled.
        return
      endif
      if str.block ne 1 then begin
        print, '% Warp_Image: Only single block structures may be scaled
        return
      endif
  
      if (str.ndim eq 2) or (str.ndim eq 3 and str.two eq 3) then begin
        arr = get_array(str, -1)
        x = get_array(str, 1, lab= labx)
        y = get_array(str, 2, lab= laby)
        if keyword_set(transpose) then begin
          arr = transpose(arr)
          t = y & y = x & x = t
        endif
      endif else begin
        print, '% Warp_Image: Only two dimensional arrays may be plotted.'
        return
      endelse

      nx = n_elements(x)
      ny = n_elements(y)
      if n_elements(offset) gt 0 then y = y-offset[0]
      lcwdf = (n_elements(wdf) gt 0) ? 1 : 0
;     Simple multiply
;     
  if n_elements(mdim) gt 0 then begin
    if lcwdf then begin
      wdf = wdvalid(mdim[0], /valid, np = np)
      if np ne 1 then begin
        print, '% WARP_IMAGE: MDIM must represent a unique WDF array'
        help, mdim, np
        return
      endif
      w2i, xarr, yarr, wdf
      warp = findex(xarr, x)
      warp = (warp > 0) < (n_elements(xarr)-1)
      warp = interpolate(yarr, warp)
    endif else warp = poly (x, mdim)
    yy = warp#y
    dyy = yy[*, 1:ny-1]-yy[*, 0:ny-1]
    if min(dyy) lt 0.0 then begin
        print, '% Warp_Image: Value of MDIM causes a negative derivative'
        print, '       MDIM =', mdim
        Print, '  ************ Structure has not been modified ************'
        return
    endif
  endif else begin
        if lcwdf then begin
      wdf = wdvalid(rdim[0], /valid, np = np)
      if np ne 1 then begin
        print, '% WARP_IMAGE: RDIM must represent a unique WDF array'
        help, rdim, np
        return
      endif
      w2i, xarr, yarr, wdf
      warp = findex(xarr, x)
      warp = (warp > 0) < (n_elements(xarr)-1)
      warp = interpolate(yarr, warp)
    endif else warp = poly (x, rdim)
    yy = (1.0/warp)#y
    dyy = yy[*, 1:ny-1]-yy[*, 0:ny-1]
    if min(dyy) lt 0.0 then begin
        print, '% Warp_Image: Value of MDIM causes a negative derivative'
        print, '       RDIM =', rdim
        Print, '  ************ Structure has not been modified ************'
        return
     endif
  endelse
      if keyword_set(conserve) then begin
        tdyy = yy
        tdyy [*, 0] = dyy[*, 0]
        tdyy [*, ny-1]= dyy[*, ny-2]
        tdyy [*, 1:ny-2] = dyy[*, 0:ny-3]+dyy[*, 1:ny-2]
        dy = y[1:ny-1]-y[0:ny-1]
        tdy = y
        tdy [0] = dy[0]
        tdy [ny-1]= dy[ny-2]
        tdy [1:ny-2] = dy[0:ny-3]+dy[1:ny-2]
        rat = make_array(nx, value = 1.0)
        rat = rat#tdy
        arr = arr/tdyy*rat
      endif
      if keyword_set(keep_all) then begin
        xx = x # make_array(ny, value = 1.0)
        nlab = n_elements (labelin)
        if nlab gt 0 then lab = strtrim(labelin[0], 2)
        ssize = [nx, ny]
        if keyword_set(transpose) then begin
          if nlab gt 0 then str.x1lab = lab
          t = yy & yy = xx & xx = t
          ssize = [ny, nx]
        endif else  if nlab gt 0 then str.x2lab = lab
        range = make_array(2, 2, 2)
        type = 8
        range(0, 0, *) = min(xx, max = mm)
        range(1, 0, *) = mm
        range(0, 1, *) = min(yy,max = mm)
        range(1, 1, *) = mm
        locx = make_array(2, 2, /long)
        locx(*, 1)  =  nx*ny
        locv = [0, nx*ny]
        tri = make_triangles(0, nx, ny)
        loctri = [0, (size(tri))[2]]
        clabel = str.clab +' - Warp_image, MDIM= ['+strtrim(mdim[0],2)
        n = n_elements(mdim)
        for i = 1, n-1 do clabel = clabel +', '+strtrim(mdim[i], 2)
        clabel = clabel + ']'
        str=  {type: type, ndim:2L, nvect:1L, block :1, two:3L, $
                ntime:1, slice:0.0, size: ssize, range:range, $
                order:0L, locx : locx, locv:locv, triangles:tri, $
                loctri:loctri, $
                x1:xx, x2:yy, x3:0, x4:0, v1:arr, v2:0.0, v3:0.0, v4:0.0, $
              v5:0.0, v6:0.0, x1lab : labx, x2lab :laby, x3lab : str.x3lab, $
              x4lab : str.x4lab, v1lab : str.v1lab , v2lab : '', $
              v3lab : '', v4lab : '',v5lab : '', v6lab : '', $
              file : str.file, spare : str.spare, tlab : str.tlab, $
              clab:clabel }
      endif else begin          
        ymin = min(yy, max = ymax)
        dx = (ymax-ymin)/(ny-1)
        ynew = ymin +dx*findgen(ny)
        for i = 0, nx-1 do begin
           xx = findex ( reform(yy[i, *]), ynew)
           arr[i, *]=interpolate(arr[i,*], xx)
           ok = where (xx lt 0 or xx gt ny-1, nok)
           if nok gt 0 then arr[i, ok] = 0.0
        endfor
        if keyword_set(transpose) then begin
          str.x1 = ynew
          str.range[0, 0, *] = ymin
          str.range[1, 0, *] = ymax
          str.dx1 = dx
          arr = transpose(arr)
          str.v1 = arr
        endif else begin
          str.x2 = ynew
          str.range[0, 1, *] = ymin
          str.range[1, 1, *] = ymax
          str.dx2 = dx 
          str.v1 = arr
        endelse
      endelse
      if lcstr eq 1 then strarray = str else i2s, str, strarray, /no_copy
      end
