; ------------------------------------------------------------------------ 
; $Id: winstr.pro,v 1.14 2011/08/18 15:39:35 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
;+
; NAME:
;       WINSTR 
;
; PURPOSE:
;       Window a 2-D structure - Similar to WIN for a waveform
;       Single block data generally uses the actual data for window.
;       Multiple block data is put on a uniform grid with 1 pixel resolution.
;       Procedure will generate windowed regions and display them. 
;       The designated windowed regions will be stored in structure arrays.
;       A preview gives the window before storage. A preview will be generated
;         whenever the mouse moves in the window and one point has been 
;         selected.  The preview frame may be resized.
;       Note: Some time is required to calculate a preview frame. All events
;             generated while the preview frame is generated are deleted; be
;             sure the mouse clicks are acknowledged before proceeding.
;       Mouse buttons are mapped to widget buttons, cursor must be in the 
;         DRAW area of the main widget for the following mouse commands.
;          Left Button   = Set Value
;             Note: A carriage return in the data field will also set the value.
;          Middle Button = Accept/Save
;             (current designated windowed image will be saved)
;          Right Button  = Reset All Points
;             (All points will be undesignated)
;          Note: If a the last point required to define a curve is selected 
;                more than once, the previous "last point" is replaced.
;       Set Point Buttons: These perform the same function as a <CR> in either
;                          the X or Y windows above the buttons.
;
; CALLING SEQUENCE:
;	WINSTR, STR [, X, Y]
;   where:
;       STR   - Structure array number of a two dimensional structure or 
;               a slice from three dimensional structure.
;               This structure will be overwritten if save is not specified.
;               The structure should be single block for best results. Multi-
;               block structures will be put on a uniform 256 by 256 grid.
;               To use an interpolation with N elements use:
;                 PLOTSTR, STR, NFARRAY = N, RESULT = STR2
;                 WINSTR, STR2
;               Note: Data will not be transposed.
;               Note: If STR is a structure, data will be saved in the first
;                     unused structure array or in the maximum.
;         
;    BATCH mode:
;       X, Y  - 2N X-values and 2N Y-values 
;               Windows of arbitrary pairs of points may be obtained in
;               batch mode where N is the number of regions desired
;
; KEYWORD PARAMETERS:
;       WindowVal:  A two by two array with the values of the X and Y values
;                   for the last saved structure.
;                   X values = windowval[*, 0]
;                   Y values = windowval[*, 1]
;                   Note: This parameter may be used for windowing several images to 
;                         the same area.
;       SAVE:       Array of structure array numbers where subregions are 
;                   stored.  
;                   Default is smallest unused structure or maxstructure.
;       LINEAR:     If set and if (MULTIPLY[0] eq 0) then the lineout will be with the
;                     original data and the image will be a log scale.  See PLOTFLD to
;                     see the effect of the MULTIPLY keyword.
;       PREVIEW:    If present and zero, no preview will be generated.
;       STRarray:   Array of Structure array numbers where images are stored.
;                   If no images are stored, then STRarray = 0
;       CPRINT:     If set, print the dataset comment for each window structure 
;                   to the terminal
;       KEEP_ASPECT: Keep same aspect ratio in the preview window as in the
;                   primary window.  Default: KEEP_ASPECT = 0
;                   If KEEP_ASPECT is negative, the color map will also be kept.
;       AVERAGE:    Average value (mean) of the saved array
;       SDEVIATION: Standard deviation of the average.
;                   SDEVIATION = sqrt((Total(X*X)-Total(X)*Total(X)/N)/(N-1))
;                     where N is the number of points
;                   For IDL 8.0 and higher the following is used:
;                   !null=moment(data, maxmoment=2, mean=average, sdev=sdeviation)0
;                   NOTE: Either SDEVIATION or Percent SDEVIATION is shown in window.
;       PERCENT:    If set, the widget shows the STANDARD_DEVIATION / AVERAGE * 100
;       MINIMUM:    Minimum value of the saved array
;       MAXIMUM:    Maximum value of the saved array
;       HISTOGRAM:  WDF for the histogram of the last window.
;                   The windowed data is sent to the PFIDL routine, HISTS.
;                   If HISTOGRAM is not a valid WDF array number, then 
;                   HISTOGRAM = GET_MAXWDF()-1
;                   NOTE: HISTOGRAM is not generated in BATCH mode
;                   NOTE: HISTOGRAM must be defined and not zero to obtain a histogram.
;       HEXTRA:     Structure with keywords for HISTS which is used for the histogram.
;                   This is ignored if HISTOGRAM is undefined or zero.
;                   For example to use 100 bins: HEXTRA = {nbins: 100}
;       _EXTRA:     Any valid keyword for plotfld. The array for the windowing
;                   is generated with the following command and this plot is 
;                   used for the image:
;          PLOTFLD, str, nfarray=256, xarray = x, yarray = y, farray = array, $
;                 nlev = 29,  _extra = extrastuff
;         Note: nfarray is ignored with single block data
; EXAMPLE:
;	Take a series of images from structure array 3 and store the images
;         beginning in structure array 11.
;           WINSTR, 3, save = 11
;
;       Take an image from (0, 0) to (3, 3) of structure 82 and store the result
;         in structure 4
;       
;         WINSTR, 82,save= 4, [0, 3], [0, 3]
;
; MODIFICATION HISTORY:
; 	Written by:	L. P. Mix, July 31, 2000
;-


PRO winstr_event, Event

on_error , 1

  WIDGET_CONTROL,Event.Id,GET_UVALUE=Ev

  print, '%WINSTR_EVENT: EVENT UVALUE = ', Ev 
  help, /str, event
  END

 Pro winstr_doit
common winstr_data_com, xx, yy, array, x, y, dx, dy, xdraw_range, ydraw_range, $
   nx, ny, mode, px, py, qx, qy, strfile, widsav, strdx, STRdy, indx, indy, $
   arr_aver, arr_min, arr_max, arr_stdev, stdev, percent, hextra, windowsavevalue, histwdf
   
common Winstr_widget_com, xwindow1, ywindow1, xwindow2, ywindow2, widwindow, $
   bset1, bset2, wdfwindow, image, draw, draw_id, line_id, hist_id, oldbx, oldby, $
   print_var, oldbp, npselect, base21, base7, clabel, xlabel, ylabel, npwdf,$
   wdfarr, indep_var, ypreview, xpreviewsize, x2label, base22, avewin, $
   minwin, maxwin, sdwin, tlb_offset, line_offset, hist_offset, xsize, ysize, $
   win_aspect, win_levels
; 
; Calculate a image of the array between points p and q.
;
 on_error, 1
  widget_control, xwindow1, get_value = px
  widget_control, ywindow1, get_value = py
  widget_control, xwindow1, set_value = px
  widget_control, ywindow1, set_value = py
  widget_control, xwindow2, get_value = qx
  widget_control, ywindow2, get_value = qy
  widget_control, xwindow2, set_value = qx
  widget_control, ywindow2, set_value = qy
; be sure values are floats
   get_winstr_line

 if n_elements(xx) gt 2  then begin
   if line_id  ge 0 then begin
     if strupcase(strmid(!version.os, 0, 3)) ne 'MAC' then begin
       device, window_state = wins
       xsiz = xpreviewsize(0)
       ysiz = xpreviewsize(1)
       if n_elements(wins) lt line_id +1 or line_id lt 0 then begin
         window, /free, xsize= xsiz, ysize= ysiz, title = 'Current Sub-Image', $
            xpos = line_offset[0], ypos= line_offset[1]
         line_id = !d.window
       endif else if wins(line_id) eq 0 then begin
         window, /free, xsize= xsiz, ysize= ysiz, title = 'Current Sub-Image', $
            xpos = line_offset[0], ypos= line_offset[1]
         line_id = !d.window
       endif else begin
         wset, line_id
         device, get_window_position = line_offset
         xpreviewsize[0] = !d.x_size
         xpreviewsize[1] = !d.y_size
         xsiz = xpreviewsize(0)
         ysiz = xpreviewsize(1)
         if hist_id ge 0 then begin
           if n_elements(wins) lt hist_id+1 or hist_id lt 0 then begin
           window, /free, xsize= xsiz, ysize= ysiz, title = 'Current Windowed Histogram', $
            xpos = hist_offset[0], ypos= hist_offset[1]
           hist_id = !d.window
         endif else if wins(hist_id) eq 0 then begin
           window, /free, xsize= xsiz, ysize= ysiz, title = 'Current Windowed Histogram', $
            xpos = hist_offset[0], ypos= hist_offset[1]
           hist_id = !d.window
           endif else wset, hist_id
           device, get_window_position = hist_offset
         endif
       endelse
     endif else wset, line_id
     okx = where (x ge xdraw_range[0] and x le xdraw_range[1], nokx)
     oky = where (y ge ydraw_range[0] and y le ydraw_range[1], noky)
     if nokx gt 2 and noky gt 2 then begin
       ix1 = okx[0] & ix2 = okx[nokx-1]
       iy1 = oky[0] & iy2 = oky[noky-1]
       mn = min(array[ix1:ix2, iy1:iy2], max = mx)
       if float(!version.release) ge 8.0 then $
       !null = moment(array[ix1:ix2, iy1:iy2], maxmoment=2, mean=av, sdev=stdev, /double) $
       else begin
         av = total(array[ix1:ix2, iy1:iy2], /double)
         stdev = sqrt((total((array[ix1:ix2, iy1:iy2])^2, /double) - av*av/(nokx*noky)) $
               /((nokx*noky)-1))
         av = av/(nokx*noky)
       endelse
       if histwdf gt 0 then begin
         arr2str, get_maxstr(), array[ix1:ix2, iy1:iy2]
         if size(/type, hextra) eq 8 then $
           hists, get_maxstr(), histwdf, _extra = hextra $
         else  hists, get_maxstr(), histwdf
         plo, histwdf, psym = 10
         wset, line_id
       endif
       widget_control, avewin, set_value = av
       val = (keyword_set(percent)) ? stdev/av*100.0 : stdev
       widget_control, sdwin, set_value = val
       widget_control, minwin, set_value = mn
       widget_control, maxwin, set_value = mx 
       if win_aspect ne 0 then begin
;        win_aspect = (Yrange/ysize)/(Xrange/xsize)
;        pasp = yrange/xrange - for preview
         pasp = (ydraw_range[1] - ydraw_range[0]) / $
            (xdraw_range[1] - xdraw_range[0])
         if win_aspect gt pasp/ysiz*xsiz then begin
           xsiza = xsiz
           ysiza = round(xsiz/win_aspect*pasp)
         endif else begin
           ysiza = ysiz
           xsiza = round(ysiz*win_aspect/pasp)
         endelse  
       endif else begin
         xsiza = xsiz
         ysiza = ysiz
       endelse
;       ix = findgen(xsiza)*(nokx/float(xsiza-1)) + okx[0]
;       iy = findgen(ysiza)*(noky/float(ysiza-1)) + oky[0]
       indx = [okx[0], okx[nokx-1]]
       indy = [oky[0], oky[noky-1]]

       xpre = xdraw_range[1]-xdraw_range[0]
       xpre = findgen(xsiza)*(xpre/float(xsiza-1))+xdraw_range[0]
       ii =  (okx[0]-1) > 0
       ie =  okx[nokx-1] +1 <(nx-1)
       ix = findex(x[ii:ie], xpre)+ii

       ypre = ydraw_range[1]-ydraw_range[0]
       ypre = findgen(ysiza)*(ypre/float(ysiza-1))+ydraw_range[0]
       ii =  (oky[0]-1) > 0
       ie =  oky[noky-1] +1 <(ny-1)
       iy = findex(y[ii:ie], ypre)+ii

       im= interpolate(array, ix, iy, /grid)
       erase
       if n_elements(win_levels) eq 2 then $
         tv, bytscl(im, min= win_levels[0], max = win_levels[1], $
               top=!d.table_size-11)+9 $
       else tv, bytscl(im, top= !d.table_size-11)+9
       
     endif else begin
       erase
       indx = 0
       indy = 0
     endelse
   endif
   wset, draw_id
   !x = oldbx
   !y = oldby
   !p = oldbp
 endif
 device, copy = image
 oplot, xx, yy, thick =2, col=7
 plots, px, py, psym = 8, color = 1, symsiz=1.5
 plots, qx, qy, psym = 8, color = 2, symsiz=1.5

 return
 end

pro get_winstr_line
  common winstr_data_com
  common winstr_widget_com
on_error, 1 
   
   xdraw_range = [min([px, qx], max = mm), mm]
   ydraw_range = [min([py, qy], max = mm), mm]
   ival = [0,1,1,0,0]
   jval = [0,0,1,1,0]
   xx = xdraw_range[ival]
   yy = ydraw_range[jval]
 return
 end

 pro winstr_point_event, event
;
  on_error, 1
  common winstr_data_com
  common winstr_widget_com

  WIDGET_CONTROL,Event.Id,GET_UVALUE=Ev
  CASE EV OF
   'DRAW':  Begin
            if event.press eq 4 then winstr_reset_button else begin
              if event.press eq 2 then begin
                  if (npselect eq 2)  then $
                    win_save_button_event, event else $
                    winformation, 'Two points must be selected for a window'
              endif else begin
                if event.press eq 1 then begin
                  coord = convert_coord(event.x, event.y, /device, /to_data)
                  if (npselect eq 0)  then begin
                    widget_control, xwindow1, set_value = coord(0) 
                    widget_control, ywindow1, set_value = coord(1) 
                    npselect = 1 > npselect
                    widget_control, draw, draw_motion_events=1, /clear_events
                  endif else begin
                    widget_control, xwindow2, set_value = coord(0)
                    widget_control, ywindow2, set_value = coord(1)
                    npselect = 2
                    winstr_doit
                    widget_control, base7, map =1
                    widget_control, draw, /clear_events, draw_motion_events = 0
                  endelse
                endif else if event.release eq 0 then begin
                  coord = convert_coord(event.x, event.y, /device, /to_data)
                  if npselect eq 0 then begin
                    widget_control, xwindow1, set_value = coord(0)
                    widget_control, ywindow1, set_value = coord(1)
                  endif else begin
                    widget_control, xwindow2, set_value = coord(0)
                    widget_control, ywindow2, set_value = coord(1)
                    winstr_doit
                  endelse
                endif else widget_control, draw, /clear_events
              endelse
            endelse
            END
  'XVALUE1': wINSTR_HANDLE_1
  'YVALUE1': wINSTR_HANDLE_1
  'BUTTON1': wINSTR_HANDLE_1
  'XVALUE2': wINSTR_HANDLE_2
  'YVALUE2': wINSTR_HANDLE_2
  'BUTTON2': wINSTR_HANDLE_2
  ELSE:   Begin
          print, ev
          help, /str, event
          END
  END
  return
  end

 Pro winstr_handle_1
   on_error, 1
  common winstr_data_com
  common winstr_widget_com
  widget_control, xwindow1, get_value = px
  widget_control, ywindow1, get_value = py
  widget_control, xwindow1, set_value = px
  widget_control, ywindow1, set_value = py
  if ((npselect eq 0) or (npselect ge 1 and mode gt 0)) then begin
     npselect = 1 > npselect
     if mode gt 0 then begin
       winstr_doit
       widget_control, base7, map =1
       widget_control, draw, /clear_events, draw_motion_events = 0
     endif else $
       widget_control, draw, draw_motion_events = 1, /clear_events
  endif else begin
     if npselect eq 2 then begin 
       winstr_doit
       widget_control, base7, map =1
       widget_control, draw, /clear_events, draw_motion_events = 0
     endif
  endelse
  end

 Pro winstr_handle_2
   on_error, 1
  common winstr_data_com
  common winstr_widget_com
  widget_control, xwindow1, get_value = px
  widget_control, ywindow1, get_value = py
  widget_control, xwindow2, get_value = qx
  widget_control, ywindow2, get_value = qy
  if (npselect eq 0) then begin
    widget_control, xwindow1, set_value = qx
    widget_control, ywindow1, set_value = qy
    widget_control, xwindow2, set_value = px
    widget_control, ywindow2, set_value = py
    npselect = 1 
    if mode gt 0 then begin
       winstr_doit
       widget_control, base7, map =1
       widget_control, draw, /clear_events, draw_motion_events = 0
    endif else widget_control, draw, /clear_events, draw_motion_events = 1
  endif else begin
    widget_control, xwindow2, set_value = qx
    widget_control, ywindow2, set_value = qy

    npselect = 2
    widget_control, base7, map =1
    winstr_doit
    widget_control, draw, /clear_events, draw_motion_events = 0
  endelse
  end

Pro win_save_button_event, event, wdf
  common winstr_data_com
  common winstr_widget_com
@qsstrcom.dat
  if n_params() lt 2 then $
    widget_control, wdfwindow, get_value = wdf
  if wdf lt 1 or wdf gt maxstructure then begin
    winformation, ['Structure Array must be in the range', $
                'of 1 to '+strtrim (maxstructure,2)]
  endif else if (n_elements(indx) lt 2) then begin
    winformation, 'Image window must contain at least 3 points'
  endif else begin
    temp = array[indx[0]:indx[1], indy[0]:indy[1]]
    clab = 'Window of '+clabel+': ('+strtrim(px,2)+','+strtrim(py,2)+ $
           ') to ('+strtrim(qx,2)+','+strtrim(qy,2)+')' 
    wdfarr(npwdf) = wdf
    npwdf = npwdf+1
    if npwdf gt 1024 then winformation, $
      [' Maximum number of Windows has been reached', $
       ' Please exit and return before taking additional windows']

    arr2str, wdf, temp, x[indx[0]:indx[1]], y[indy[0]:indy[1]], $
      file = strfile, xlab = xlabel, ylab = ylabel, clab = clab, $
      dx = strdx, dy = strdy 
    windowsavevalue = reform([min([px, qx], max = m), m, min([py, qy], max = mm), mm], 2, 2)
    wdf += 1
    if n_params() eq 2 then begin
      nv = n_elements(temp)
      mn = min(temp, max = mx)
      if float(!version.release) ge 8.0 then $
        !null = moment(temp, maxmoment=2, mean = av, sdev = stdev, /double) else begin
        av = total(temp, /double)
        stdev = sqrt((total(temp^2, /double) - av*av/nv)/(nv-1))
        av = av/nv
      endelse
      arr_aver = [arr_aver, av]
      arr_stdev = [arr_stdev, stdev]
      arr_min = [arr_min, mn]
      arr_max = [arr_max, mx]
      return
    endif
    widget_control, avewin, get_value = val
    arr_aver = [arr_aver, val]
    arr_stdev = [arr_stdev, stdev]
    widget_control, minwin, get_value = val
    arr_min = [arr_min, val]
    widget_control, maxwin, get_value = val
    arr_max = [arr_max, val]
    if print_var ne 0 then print, clab
    widget_control, wdfwindow, set_value = wdf
    winstr_reset_button
  endelse
  END

pro winstr_reset_button
; Handle the actual reset
  common winstr_data_com
  common winstr_widget_com
    npselect = 0
    if line_id gt 0 then begin
      wset, line_id
      erase
    endif
    if hist_id ge 0 then begin
      wset, hist_id
      erase
    endif
    wset, draw_id
    device , copy = image
    !x = oldbx
    !y = oldby
    !p = oldbp
    widget_control, draw, /clear_events
    widget_control, base7, map = 0
    widget_control, draw, /clear_events, draw_motion_events = 0
    xx =0
  END

pro winreset_button_event, event
;
; Handle the reset and the exit buttons
; First RESET
  on_error, 1
  common winstr_data_com
  common winstr_widget_com
  if event.value eq 0 then winstr_reset_button else begin
    if npselect eq 2  then begin
      wsave_nosave, lcquit, [ 'There is a WINDOW defined', $
          'Do you want to save it before EXIT'], 'WINSTR EXIT', $
           group = event.top 
      if lcquit eq 0 then win_save_button_event
    endif
    if strupcase(strmid(!version.os, 0, 3)) ne 'MAC' then begin
      device, window_state = wins
      xsiz = xpreviewsize(0)
      ysiz = xpreviewsize(1)
      if n_elements(wins) gt line_id and line_id ge 0 then begin
        if wins[line_id] gt 0 then begin
          wset, line_id
          device, get_window_position = line_offset
          xpreviewsize[0] = !d.x_size
          xpreviewsize[1] = !d.y_size
          wdelete, line_id
          line_id = -1
        endif
      endif
      if n_elements(wins) gt hist_id and hist_id ge 0 then begin
        if wins[hist_id] gt 0 then begin
          wset, hist_id
          device, get_window_position = hist_offset
          xpreviewsize[0] = !d.x_size
          xpreviewsize[1] = !d.y_size
          wdelete, hist_id
          hist_id = -1
        endif
      endif
    endif
    widget_control, event.top, tlb_get_offset = tlb_offset
    widget_control, event.top, /destroy
  endelse
  end

  pro winstr_batch, arr, wdf, xin, yin, multiply = multiply, $
        linear = linear, _extra = extrastuff
;
@com.dat

@qscom.dat
;  Common block
@qsstrcom.dat
;       return on an error
  common winstr_data_com
  common winstr_widget_com
      on_error, 1
      
      clabel = arr.clab
      strfile = arr.file
      oldwin = !d.window
      initbx = !x
      initby = !y
      initbp = !p
      xsize = 700
      ysize = 560
      npselect = 0
      if n_elements(wdf) eq 0 then wdf =1

      if arr.ndim eq 2  or arr.two eq 3 then begin
         xval =1  & yval = 2
       endif else if arr.two eq 2 then begin
         xval = 3 & yval = 1
       endif else if arr.two eq 1 then begin 
         xval =2  & yval = 3
       endif else begin
         Print, 'Error with getting data(two, ndim): ', arr.two, arr.ndim
         return
      endelse
      if n_elements(multiply) gt 0 and keyword_set(linear) then $
        log_image = (multiply[0] eq 0) ? 1 : 0 else log_image = 0

      if arr.block gt 1 or arr.type ne 1 or keyword_set(log_image) then begin
        window, /free , /PIXMAP  , xsize = xsize, ysize = ysize
        pixwin = !d.window
            Print, '%WINSTR: Structure will be put on a single block'
  if log_image then begin
    plotfld, arr, farray = array, xarray= x, yarray= y, levels= win_levels, $
       nfarray = 256,cntour = 2,  _extra = extrastuff
    plotfld, arr, farray = array1, xarray= x, yarray= y,  $
       nfarray = 256,cntour = 2,  multiply = multiply, _extra = extrastuff
    array1 =0
  endif else plotfld, arr, farray = array, xarray= x, yarray= y, levels= win_levels, $
       nfarray = 256,cntour = 2,  multiply = multiply, _extra = extrastuff, /quiet
        wdelete, pixwin
        if oldwin gt 0 then  begin
          wset, oldwin
          !x = initbx
          !y = initby
          !p = initbp
        endif
        if n_elements (array) lt 4 then return
        dum = get_array(arr, xval, lab = xlabel)
        dum = get_array(arr, yval, lab = ylabel)
         
      endif else begin
        x = get_array(arr, xval, lab = xlabel)
        y = get_array(arr, yval, lab = ylabel)
        array = reform(get_array(arr, -1), /overwrite)
      endelse
      check_uniform, x, strdx
      check_uniform, y, strdy
      ok = where(strpos(tag_names(arr), 'DX') ge 0 , nok)
      if nok gt 0 then begin
        if (strdx gt 0) and (arr.dx1[0] gt 0) then begin
          if abs((arr.dx1[0]-strdx)/strdx) lt 3.0517578e-05 then strdx = arr.dx1[0]
        endif
        if (strdy gt 0) and (arr.dx2[0] gt 0) then begin
          if abs((arr.dx2[0]-strdy)/strdy) lt 3.0517578e-05 then strdy = arr.dx2[0]
        endif
      endif

      nx = n_elements(x)
      ny = n_elements(y)
; 2 points per window
      nelx = n_elements(xin)
      nely = n_elements(yin)
      nlin = min([nelx, nely])/2
      if nlin lt 1 then begin
        Print, '% WINSTR: Window must have at least 2 X values and 2 Y values.'
        return
      endif
      for i = 0, nlin-1 do begin
          px = min(xin[i+i:i+i+1], max = qx)
          py = min(yin[i+i:i+i+1], max = qY)
          okx = where (x ge px and x le qx, nokx)
          oky = where (y ge py and y le qy, noky)
          if nokx lt 3 or noky lt 3 then begin
            print, '% WINSTR: Must have at least 3 values in each direction'
            print, ' Xrange =', px, qx
            print, ' Yrange =', py, qy
          endif else begin
            indx = [okx[0], okx[nokx-1]]
            indy = [oky[0], oky[noky-1]]
            win_save_button_event, 0, wdf
          endelse
      endfor

      return
      end

pro winstr_cleanup, val
  common winstr_data_com
  common winstr_widget_com
  common winstr_cleanup, oldwin, pixwin, initbx, initby, initbp

  if line_id gt 0 then begin
    device, window_state = wins
    if n_elements(wins) gt line_id  and line_id ge 0 then $
        if wins[line_id] gt 0 then wdelete, line_id
  endif
  if hist_id gt 0 then begin
    device, window_state = wins
    if n_elements(wins) gt hist_id and hist_id ge 0 then $
        if wins[hist_id] gt 0 then wdelete, hist_id
  endif
  
  if oldwin lt 0 then return
  wset, oldwin
  !x = initbx
  !y = initby
  !p = initbp
end

pro winstr, strin, xin, yin, cprint = printit, strarray = wdfarray, $
           save = savestrin,  group_leader = group, preview = preview, $
           xsize = xsizein , ysize= ysizein,  keep_aspect = keep_asp, $
           average = average, maximum = maximum, levels = lev, $
           sdeviation = standard, windowval = windowval,  $
           minimum = minimum, modal = modal, multiply = multiply, $
           linear = linear, histogram = hist_wdf, hextra = hextrain, $
           percent = percentin,  _extra =extrastuff

;
;       return on an error
      on_error, 1
@com.dat
  common winstr_data_com
  common winstr_widget_com
      if n_elements(initwdf) eq 0 then wdfinit
  common winstr_cleanup, oldwin, pixwin, initbx, initby, initbp
@qscom.dat
;  Common block
@qsstrcom.dat
;
      common pffcom, initpff, pff_exe
@draw_defaults.com

      if n_elements (structinit) eq 0 then qsstruct_init
      if n_elements (qsplotinit) eq 0 then qsinit
      if n_elements (initpff) eq 0 then init_pff else $
        if initpff eq 0 then init_pff
;
      nparams = n_params()
      if nparams eq 0 then begin
        print,'% Format: ', $
       'WINSTR, STR, [, X, Y] [, /CPRINT] [, SAVE = save_structure]'
        print,'        [, PREVIEW = preview] [, /KEEP_ASPECT]'
        print,'        [, _EXTRA = extrastuff]'
        return
      endif
      if n_elements(preview) eq 0 then preview = -1

      arr = get_valid_structure(strin, 'WINSTR', ierror = ier)
      if ier ne 0 then return
      
      if arr.type ne 1  and arr.type ne 8 then begin
        Print, '%WINSTR: Structure array must be a field structure'
        return
      endif
      if keyword_set(hist_wdf) then begin
        histwdf = $
         ((hist_wdf lt 1) or (hist_wdf gt maxwdfarray)) ? maxwdfarray -1 : hist_wdf
      endif else histwdf =0L
      windowval = 0
      mode = 0
      if n_elements (group) eq 0 then group = 0
      if n_elements(savestrin) gt 0 then savestr = savestrin else $
          savestr = get_save_str(savestr, maxstructure)
      win_levels = (n_elements(lev) gt 1) ? lev : 0
      if nparams gt 2 then begin
        npwdf =0
        wdfarr= make_array(1024, /int, value = 0)
        arr_aver = 0.0
        arr_stdev = 0.0
        arr_min = 0.0
        arr_max = 0.0
        winstr_batch, arr, savestr, xin, yin, linear = linear, $
          multiply = multiply, _extra =extrastuff
        minimum = (n_elements(arr_min) gt 1) ? arr_min[1:*] : 0.0
        maximum = (n_elements(arr_max) gt 1) ? arr_max[1:*] : 0.0
        average = (n_elements(arr_aver) gt 1) ? arr_aver[1:*] : 0.0
        standard = (n_elements(arr_stdev) gt 1) ? arr_stdev[1:*] : 0.0
        if npwdf gt 0 then wdfarray = wdfarr(0:npwdf-1) else wdfarray = 0
        return
      endif

      percent = (n_elements(percentin) eq 0) ? 0 : percentin
      hextra = (n_elements(hextrain) eq 0) ? 0 : hextrain

      clabel = arr.clab
      strfile = arr.file
      indx =0

  tlbase = WIDGET_BASE(GROUP_LEADER=Group, $
      COLUMN=1, $
      MAP=1, modal = modal, $
      TITLE='Window Image')

  BASEINS  = WIDGET_BASE(tlbase, $
      COLUMN = 1, $
      MAP=1, $
      UVALUE = 'BASEINS')

 labelins1 =  WIDGET_LABEL( BASEins, $
      UVALUE='LABEL', $
  VALUE='Left Mouse = Designate Point,     Middle Mouse = Save,     Right Mouse = Reset')

  BASE0 = WIDGET_BASE(tlbase, $
      row=1, $
      FRAME=0, $
      MAP=1, $
      UVALUE='BASE0')

  BASE1 = WIDGET_BASE(base0, $
      column=1, $
      FRAME=2, $
      MAP=1, $
      event_pro = 'Winstr_Point_Event', $
      UVALUE='BASE1')

  BASE2 = WIDGET_BASE(base0, $
      FRAME=2, $
      MAP=1, $
      UVALUE='BASE2')

  BASE3 = WIDGET_BASE(base0, $
      column=1, $
      FRAME=2, $
      MAP=1, $
      UVALUE='BASE3')

  BASE4 = WIDGET_BASE(tlbase, $
      ROW = 1, $
      MAP=1, $
      UVALUE = 'BASE4')

  BASE5 = WIDGET_BASE(tlbase, $
      ROW = 1, $ 
      MAP=1, $
      event_pro = 'Winstr_Point_Event', $
      UVALUE = 'BASE5')

  BASE6 = WIDGET_BASE(BASE4, $
      ROW = 1, $
      MAP=1, $
      FRAME = 2, $
      UVALUE = 'BASE6')

  BASE7 = WIDGET_BASE(BASE4, $
      ROW = 1, $
      MAP=1, $
      FRAME = 2, $
      event_pro = 'WIN_SAVE_BUTTON_EVENT', $
      UVALUE = 'BASE7')

  BASE8 = WIDGET_BASE(BASE4, $
      ROW = 1, $
      MAP=1, $
      FRAME = 2, $
      event_pro = 'WINRESET_BUTTON_EVENT', $
      UVALUE = 'BASE8')

      px = 0.0
      py = 0.0
      qx = 0.0
      qy = 0.0

   label1  =  WIDGET_LABEL( BASE1, $
      UVALUE='LABEL', $
      VALUE='First Point')

   xwindow1 = CW_FIELD( BASE1,VALUE=px, $
      ROW=1, xsize = 15, $
      FLOAT=1, $
      RETURN_EVENTS = 1, $
      TITLE='X-Value:', $
      UVALUE='XVALUE1')

   ywindow1 = CW_FIELD( BASE1,VALUE=py, $
      ROW=1, xsize = 15, $
      FLOAT=1, $
      RETURN_EVENTS = 1, $
      TITLE='Y-Value:', $
      UVALUE='YVALUE1')

   bset1 = WIDGET_BASE(BASE1, $
      ROW = 1, $
      MAP=1, $
      UVALUE = 'BSET1')

   set1= WIDGET_BUTTON( Bset1, $
      UVALUE='BUTTON1', $
      VALUE=' Set Point 1 Value ')

  BASE21 = WIDGET_BASE(base2, $
      column=1, $
      MAP=1, $
      event_pro = 'Winstr_Point_Event', $
      UVALUE='BASE21')

   label2  =  WIDGET_LABEL( BASE21, $
      UVALUE='LABEL', $
      VALUE='Second Point')

   xwindow2 = CW_FIELD( BASE21,VALUE=px, $
      ROW=1, xsize = 15, $
      FLOAT=1, $
      RETURN_EVENTS = 1, $
      TITLE='X-Value:', $
      UVALUE='XVALUE2')

   ywindow2 = CW_FIELD( BASE21,VALUE=py, $
      ROW=1, xsize = 15, $
      FLOAT=1, $
      RETURN_EVENTS = 1, $
      TITLE='Y-Value:', $
      UVALUE='YVALUE2')

   bset2 = WIDGET_BASE(BASE21, $
      ROW = 1, $
      MAP=1, $
      UVALUE = 'BSET2')

   set2= WIDGET_BUTTON( BSET2, $
      UVALUE='BUTTON2', $
      VALUE=' Set Point 2 Value ')

   label3  =  WIDGET_LABEL( BASE3, $
      UVALUE='LABEL', $
      VALUE='Image Parameters')

   avewin = CW_FIELD( BASE3,VALUE=0.0, $
      ROW=1, xsize = 15, $
      FLOAT=1, /NOEDIT, $
      TITLE='Average:', $
      UVALUE='AVERAGE')

   minwin = CW_FIELD( BASE3,VALUE=0.0, $
      ROW=1, xsize = 15, $
      FLOAT=1, /NOEDIT, $
      TITLE='Minimum:', $
      UVALUE='MINIMUM')

   maxwin = CW_FIELD( BASE3,VALUE=0.0, $
      ROW=1, xsize = 15, $
      FLOAT=1, /NOEDIT, $
      TITLE='Maximum:', $
      UVALUE='MAXIMUM')

   sdtit = (keyword_set(percent)) ? '% Std Dev:' : 'Std Dev:'
   sdwin =  CW_FIELD( BASE3,VALUE=0.0, $
      ROW=1, xsize = 15, $
      FLOAT=1, /NOEDIT, $
      TITLE= sdtit, $
      UVALUE='STDEV')

  WDFwindow = CW_FIELD( BASE6,VALUE = savestr[0], $
      ROW=1, $
      integer=1, $
      TITLE='Str Array:', $
      RETURN_EVENTS = 0, $
      XSIZE = 5, $
      UVALUE='WDF')

  SAVEB= WIDGET_BUTTON( base7, $
      UVALUE='SAVE_BUTTON', $
      VALUE=' Accept / Save Image ')

  Btns439 = [ $
    ' Reset All Points ', $
    '   Done  / Exit   ' ]
  ard_buttons = CW_BGROUP( base8, Btns439, $
      ROW=1,  /NO_RELEASE, $
      UVALUE='buttons')

  oldwin = !d.window
  initbx = !x
  initby = !y
  initbp = !p

  pmulti_save=!p.multi 
  !p.multi = 0
  if n_elements(xsizein) eq 0 then begin
    if n_elements(xsize) eq 0 then $
      if initpff gt 3 then xsize = medsize(0)*5/6 else xsize = medsize(0) 
  endif else xsize = xsizein
  if n_elements(ysizein) eq 0 then begin
    if n_elements(ysize) eq 0 then $
      if initpff gt 3 then ysize = medsize(1)*5/6 else ysize = medsize(1) 
  endif else ysize = ysizein
  if n_elements(tlb_offset) eq 0 then tlb_offset = [10, 20]
  if n_elements(line_offset) eq 0 then $
       line_offset = [tlb_offset[0]+medsize[0]+30, smallsize[1]]
  if n_elements(hist_offset) eq 0 then $
       hist_offset = [line_offset[0], line_offset[1]+smallsize[1]]

  xpreviewsize = smallsize
  if n_elements (xpreviewsize) eq 0 then xpreviewsize = smallsize

  npselect = 0

  window, /free , /PIXMAP  , xsize = xsize, ysize = ysize
  pixwin = !d.window
  if arr.block gt 1 then $
        Print, '%WINSTR: Structure will be put on a single block'
  if n_elements(multiply) gt 0 and keyword_set(linear) then $
    log_image = (multiply[0] eq 0) ? 1 : 0 else log_image = 0
  if log_image then begin
    plotfld, arr, farray = array, xarray= x, yarray= y, levels= win_levels, $
       xlabel = xlabel, ylabel = ylabel, nfarray = 256,cntour = 2,  $
       _extra = extrastuff
    plotfld, arr, farray = array1, xarray= x, yarray= y,  $
       xlabel = xlabel, ylabel = ylabel, nfarray = 256,cntour = 2,  $
       multiply = multiply, _extra = extrastuff
    array1 =0
  endif else plotfld, arr, farray = array, xarray= x, yarray= y, levels= win_levels, $
       xlabel = xlabel, ylabel = ylabel, nfarray = 256,cntour = 2,  $
       multiply = multiply, _extra = extrastuff
  if n_elements (array) lt 4 then begin
    wdelete, pixwin
    if oldwin gt 0 then  begin
      wset, oldwin
      !x = initbx
      !y = initby
      !p = initbp
    endif
    return
  endif
  if keyword_set(keep_asp) then begin
    win_aspect = !d.x_size *  $
    (!x.window[1]-!x.window[0])/!d.y_size/(!y.window[1]-!y.window[0]) * $
    (!y.crange[1]-!y.crange[0])/(!x.crange[1]-!x.crange[0])
    if keep_asp lt 0 then win_levels = (n_elements (lev) eq 2) ? lev : $
      [min(win_levels, max = mlev), mlev] else win_levels = 0
  endif else begin
    win_aspect = 0
    win_levels = 0
  endelse
  xdraw_range = !x.crange
  ydraw_range = !y.crange
  win_aspect = (keyword_set(keep_asp)) ? !d.x_size *  $
    (!x.window[1]-!x.window[0]) /!d.y_size/ (!y.window[1]-!y.window[0]) * $
    (!y.crange[1]-!y.crange[0])/(!x.crange[1]-!x.crange[0]) : 0
  nx = n_elements(x)
  ny = n_elements(y)
  
  check_uniform, x, strdx
  check_uniform, y, strdy
  ok = where(strpos(tag_names(arr), 'DX') ge 0 , nok)
  if nok gt 0 then begin
    if (strdx gt 0) and (arr.dx1[0] gt 0) then begin
      if abs((arr.dx1[0]-strdx)/strdx) lt 3.0517578e-05 then strdx = arr.dx1[0]
    endif
    if (strdy gt 0) and (arr.dx2[0] gt 0) then begin
      if abs((arr.dx2[0]-strdy)/strdy) lt 3.0517578e-05 then strdy = arr.dx2[0]
    endif
  endif
 
  if nx eq 0 or ny eq 0 then begin
    !p.multi = pmulti_save
    return
  endif
  dx = x(1:nx-1)-x(0:nx-2)
  dy = y(1:ny-1)-y(0:ny-2)
  image = [0, 0, xsize, ysize, 0, 0, pixwin]

  DRAW = WIDGET_DRAW( base5, $
      BUTTON_EVENTS=1, $
      MOTION_EVENTS=1, $
      RETAIN=1, $
      UVALUE='DRAW', $
      XSIZE=xsize, $
      YSIZE=ysize)
  
  WIDGET_CONTROL, tlbase, /REALIZE, tlb_set_xoffset = tlb_offset[0], $
            tlb_set_yoffset = tlb_offset[1]
  widget_control, base7, map = 0
  widget_control, wdfwindow, set_value = savestr[0]
  widget_control, draw, draw_motion_events = mode
; Get drawable window index
  WIDGET_CONTROL, DRAW, GET_VALUE=DRAW_Id
  mksymbol, 'plus', /fill, size = 1.5
  loadxyct, /quiet, /freeze
  if keyword_set(printit) then print_var =1 else print_var = 0
  npselect = 0
  wset , draw_id
  device , copy = image
  oldbx = !x
  oldby = !y
  oldbp = !p
  if n_elements(preview) gt 1 then begin
    line_id = 1
    ypreview = preview(0:1)
  endif else if preview ne 0 then begin
    line_id = 1
    if preview lt 0 then ypreview = 0 else begin 
      ypreview=[min(array,max=mm), mm]
      delta = (ypreview(1)-ypreview(0))* 3.0517578e-05
      ypreview=ypreview+[delta, -delta]
    endelse
  endif else line_id = -1
  
  if line_id gt 0 then begin
    window, /free, xsize= xpreviewsize(0), ysize = xpreviewsize(1), $
      title = 'Current Windowed Image', xpos=line_offset[0], ypos=line_offset[1]
    line_id = !d.window
    device, get_window_position = line_offset_actual
    window_offset = line_offset-line_offset_actual
    if histwdf gt 0 then begin
      window, /free, xsize= xpreviewsize(0), ysize = xpreviewsize(1), $
      title = 'Current Windowed Histogram', xpos=hist_offset[0], ypos=hist_offset[1]
      hist_id = !d.window
      device, get_window_position = hist_offset_actual
    endif else hist_id = -1
  endif else  Window_offset = [0,0]
  wset, draw_id
  !x = oldbx
  !y = oldby
  !p = oldbp
  npwdf = 0
  arr_aver = 0.0
  arr_stdev = 0.0
  arr_min = 0.0
  arr_max = 0.0
  windowsavevalue = 0
  wdfarr= make_array(1024, /int, value = 0)
  XMANAGER, 'Winstr', tlbase, cleanup='winstr_cleanup'
  array = 0
  if npwdf gt 0 then wdfarray = wdfarr(0:npwdf-1) else wdfarray = 0
  wdelete, pixwin
  arr = 0
  !p.multi = pmulti_save
  line_offset = line_offset +window_offset
  minimum = (n_elements(arr_min) gt 1) ? arr_min[1:*] : 0.0
  maximum = (n_elements(arr_max) gt 1) ? arr_max[1:*] : 0.0
  average = (n_elements(arr_aver) gt 1) ? arr_aver[1:*] : 0.0
  standard = (n_elements(arr_stdev) gt 1) ? arr_stdev[1:*] : 0.0
  windowval = windowsavevalue
  if oldwin lt 0 then return
  wset, oldwin
  !x = initbx
  !y = initby
  !p = initbp
END
