; ------------------------------------------------------------------------ 
; $Id: wrimsd_ext.pro,v 1.3 2007/06/06 20:09:38 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
 Pro WRIMSD_ext, fname, array, x, y, z, dx = dx, dy = dy, dz = dz, $
     clabel = clabel, xlabel = xlabel, ylabel = ylabel, zlabel = zlabel
;
;+
;    Write a structure array to an ascii file
;    This procedure uses only IDL procedures
;    This procedure limited to single block, single vector field data
;    All labels are limited to 80 characters.
;
;    Format:  WRIMSD_ext, fname, ARRAY  [, X] [, Y] [, Z]
;             [, DX = dx] [, DY = dy] [, DZ = dz] [, CLABEL = clabel]
;             [, XLABEL = xlabel] [, YLABEL = ylabel] [, ZLABEL = zlabel] 
;    where 
;      fname   - the file name
;                If fname = 0 then fname ='PFIDLdefault.dat'
;      ARRAY   - Image array
;      X       - X values 
;                If DX > 0 then X = X[0] and used as an initial value
;      Y       - Y values
;                If DY > 0 then Y = Y[0] and used as an initial value
;      Z       - Z values
;                If DZ > 0 then Z = Z[0] and used as an initial value
;      DX      - If uniform data, the point spacing in the X direction
;      DY      - If uniform data, the point spacing in the Y direction
;      DZ      - If uniform data, the point spacing in the Z direction
;      CLABEL  - Comment label
;      XLABEL  - X Label
;      YLABEL  - Y Label
;      ZLABEL  - Z Label
;
;    Example:
;    Write ARRAY into file 'PFIDLdefault.dat'
;       WRIMSD_ext, 0, ARRAY
;
;    Write ARRAY into file 'testdata'
;       WRIMSD_ext , 'testdata', ARRAY
;
;    form of file:
;      dataset comment
;      ndim, lcuniform 
;      x axis label 
;      y axis label
;      z axis label
;      nx, ny, nz
;      x0, y0, z0 |
;      dx, dy, dz |  or x, y, z
;      field data
;
;  where:
;      ndim       = 2 or 3
;      lcuniform  = 1 if uniform, 0 if not uniform
;  X is uniform if max(dx)-min(dx) is less than 1.22e-4 *max(abs(x))
;      axis label = string for axis labels
;                   Z label is omitted if ndim = 2
;      nx, ny, nz = number of elements in each dimension
;                   (NZ omitted if ndim = 2)
;      x0, y0, z0 = initial axis values
;      dx, dy, dz = axis step size
;      x, y, z    = axis values (lcuniform = 0)
;                   dz or Z omitted if ndim = 2
;      field data = image values
;
;   Fortran read:
;       integer lu, ndim, lcuniform, nx, ny, nz, i, j, k
;       real x(:), y(:), z(:)
;       real x0, y0, z0, dx, dy, dz 
;       real image( :, :, :)
;       character*80 xlabel, ylabel, zlabel, title
; c1     format (a)
;       open(file = , unit = lu)
;       read(lu, 1) title
;       read(lu, *) ndim, lcuniform 
;       read(lu, 1) xlabel
;       read(lu, 1) ylabel
;       nz = 1
;       if (ndim .eq. 3) then
;         read(lu, 1) zlabel
;         read(lu, *) nx, ny, nz
;       else
;         read(lu, *) nx, ny
;       endif 
; c      if fortran 90, allocate x, y, z, image
;       if (lcuniform .eq. 0) then
;         read(lu, *) (x(i), i = 1, nx)
;         read(lu, *) (y(i), i = 1, ny)
;         if (ndim .eq. 3) read(lu, *) (z(i), i = 1, nz)
;       else
;         if (ndim .eq. 2) then
;           read(lu, *) x0, y0
;           read(lu, *) dx, dy
;         else
;           read(lu, *), x0, y0, z0
;           read(lu, *) dx, dy, dz
;         endif
; c        can make x, y, z if desired x(i)= (i-1)*dx + x0, etc.
;      endif
;      read(lu, *)(((image(i, j, k),i=1,nx),j=1, ny),z=1,nz)
;      close (lu)
;-
;
on_error, 1
;
      nparams = n_params()
      if nparams eq 0  then begin
        print, '% Format: WRIMSD_ext, Filename,  ARRAY  [, X] [, Y] [, Z]'
        print,'          [, DX = dx] [, DY = dy] [, DZ = dz] [, CLABEL = clabel]'
        print,'          [, XLABEL = xlabel] [, YLABEL = ylabel] [, ZLABEL = zlabel]'
        return
      endif
      if n_elements (fname) eq 0 then begin
        print, '% WRIMSD: File Name must be provided'
        return
      endif
      b = size(fname)
      if b(b(0)+1) ne 7 then begin
        if fname(0) eq 0 then fname = 'PFIDLdefault.dat' else begin
          print, '% WRIMSD: Input file name must be a string or zero'
          return
        endelse
      endif else fname = fname(0)
      nel = n_elements(array)
      if nel eq 0  then begin
        print,'% WRIMSD_ext: Array must be provided'
        return
      endif
      b = size(array)
      ndim = b[0] < 3
      if ndim lt 2 or ndim gt 3 then begin
        print, '% WRIMSD_ext: Only two and three dimensional data can be written'
        return
      endif
      nelx = b[1]
      nely = b[2]
      if ndim eq 3 then nelz = b[3]
      array = reform(array, nel, /overwrite)
;     Check each axis
      if n_elements(xlabel) eq 0 then xlabel = 'X axis'
      If n_elements(dx) eq 0 then dx = 0.0
      if n_elements(x) eq 0 then begin
        print, '% WRIMSD_ext: No X data.  An indexed array will be generated.'
        lcunifx = 1
        if dx eq 0 then dx = 1
        x = findgen(nelx)*dx
      endif else begin
        lcunifx =(dx gt 0.0) ? 1 : 0
        if n_elements(x) lt nelx then begin
          if lcunifx gt 0 then x = x[0]+findgen(nelx)*dx else begin
            print, '% WRIMSD_ext: X array does not have correct number of elements'
            print, n_elements(x) , nelx
            return
          endelse
        endif 
      endelse
      if n_elements(ylabel) eq 0 then ylabel = 'Y axis'
      If n_elements(dy) eq 0 then dy = 0.0
      if n_elements(y) eq 0 then begin
        print, '% WRIMSD_ext: No X data.  An indexed array will be generated.'
        lcunify = 1
        if dy eq 0 then dy = 1
        y = findgen(nely)*dy
      endif else begin
        lcunify =(dy gt 0.0) ? 1 : 0
        if n_elements(y) lt nely then begin
          if lcunify gt 0 then y = y[0]+findgen(nely)*dy else begin
            print, '% WRIMSD_ext: X array does not have correct number of elements'
            print, n_elements(y) , nely
            return
          endelse
        endif
      endelse
      if ndim eq 3 then begin
        if n_elements(zlabel) eq 0 then zlabel = 'Z axis'
        If n_elements(dz) eq 0 then dz = 0.0
        if n_elements(z) eq 0 then begin
          print, '% WRIMSD_ext: No X data.  An indexed array will be generated.'
          lcunifz = 1
          if dz eq 0 then dz = 1
          z = findgen(nelz)*dz
        endif else begin
          lcunifz =(dz gt 0.0) ? 1 : 0
          if n_elements(z) lt nelz then begin
            if lcunifz gt 0 then z = z[0]+findgen(nelz)*dz else begin
              print, '% WRIMSD_ext: X array does not have correct number of elements'
              print, n_elements(z) , nelz
              return
            endelse
          endif
        endelse
      endif else lcunifz = 1

      lcuniform = lcunifx*lcunify*lcunifz

      openw,unit,fname, /get_lun, width = recl
; Time to start writing
;      dataset comment
;      ndim, lcuniform 
;      x axis label 
;      y axis label
;      z axis label
;      nx, ny, nz
;      dx, dy, dz or x, y, z
;      field data
      if n_elements(clabel) eq 0 then clabel = 'WRIMSD label'
      printf, unit, strmid(clabel[0], 0, 80)
      printf, unit, ndim, lcuniform
      printf, unit, strmid(xlabel[0], 0, 80)
      printf, unit, strmid(ylabel[0], 0, 80)
      if ndim eq 3 then begin
        printf, unit, strmid(zlabel[0], 0, 80)
        printf, unit, nelx, nely, nelz
      endif else printf, unit, nelx, nely
      if lcuniform eq 1 then begin
        if ndim eq 2 then begin
          printf, unit, x(0), y(0)
          printf, unit, dx, dy 
        endif else begin
          printf, unit, x(0), y(0), z(0)
          printf, unit, dx, dy, dz
        endelse
      endif else begin
        printf, unit, x
        printf, unit, y
        if ndim eq 3 then printf, unit, z
      endelse
      printf, unit, array
      free_lun, unit
      return
      end
