; ------------------------------------------------------------------------ 
; $Id: xlineout.pro,v 1.5 2010/09/17 20:54:30 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
 Function xlineout, array, y, yvalin, width, median = median, $
       integral= integral
;+
;  Calculate a lineout of a two dimensional array at a given y value and 
;    over a specified width.
;  Format is: line = xlineout(array, yarray, yvalue [, width] [, /MEDIAN] $
;         [, /INTEGRAL )
;    where;
;      LINE    - array of ordinate values for the lineout
;      ARRAY   - two dimensional array
;      YARRAY  - Ordinate values for array
;      YVALUE  - Ordinate value at the center of the lineout
;      WIDTH   - Width over which the values of array are averaged to 
;                obtain the values in LINE. Default is 0.0
;         Note:  Average is spatially weighted 
;                Average = integral (array)/(upper - lower)
;      MEDIAN  - If set and WIDTH > 0 then the median is returned.
;      INTEGRAL- If set and WIDTH > 0 then the integral is returned.
;                Note:  If median is set, INTEGRAL is ignored
;  Example:
;    Obtain a horizontal lineout from ARRAY with ordinate values, Y, at 
;    a value of y= AMAX.
;      line = xlineout(array, y, amax)
;    Note:  Since width is zero, value is simple interpolation.
;
;    Obtain a horizontal lineout from PICTURE with ordinate values, Y, at
;    a value of y= yy(i), averaged over a width of 0.06
;      line = xline(picture, y, yy(i), 0.06)
;-
     on_error, 1
     if n_elements (array) eq 0 then begin
       Print, ' Two dimensional array must be defined'
       return, 0.0
     endif
     b = size (array)
     if b(0) ne 2 then begin
       print, ' Array for lineout must have two dimensions.'
       return, 0.0
     endif
     ny = n_elements(y)
     if ny ne b(2) then begin
       print, ' Ordinate array must be defined and have same ' 
       print, ' number of element as the array'
       return, 0.0
     endif
     if n_elements (yvalin) eq 0 then begin
       print, ' Ordinate value for the lineout must be defined'
       return , 0.0
     endif
     yval = yvalin
     if n_elements (width) eq 0 then width = 0 else width = abs(width)
     width2 = width*0.5
     if (width ne 0) then begin
       yl = yval-width2 > y(0)
       yu = yval+width2 < y(ny-1)
       if yl gt y(ny-1) then return ,reform(array(*, ny-1))
       if yu lt y(0) then return ,reform(array(*,0))
       yval = 0.5*(yu+yl)
       delta = min(abs(y-yl), jlow)
       delta = min(abs(y-yu), jhi)
;
;       if width is less than the local point spacing use width = 0
;       but don't change width because it is a passed parameter
;
        if jhi eq jlow then begin
          jmid = jhi
          width2 = 0
        endif else if y(jhi)-y(jhi-1) ge width then begin
          width2 = 0
          delta = min(abs(y-yval), jmid)
        endif
     endif else delta = min(abs(y-yval), jmid)
     if width2 eq 0 then begin
       if jmid eq 0 and yval le y(0) then return , reform(array(*, 0))
       if jmid eq ny-1 and yval ge y(ny-1) then return,reform(array(*,ny-1))
       if yval ge y(jmid) then begin
         delta = y(jmid+1)-y(jmid)
         r1 = (yval-y(jmid))/delta
         r2 = 1-r1
         return, reform(r2*array(*, jmid)+r1*array(*, jmid+1))
       endif else begin
         delta = y(jmid)-y(jmid-1)
         r1 = (y(jmid)-yval)/delta
         r2 = 1-r1
         return,reform(r2*array(*, jmid)+r1*array(*, jmid-1))
       endelse
     endif else begin
       if keyword_set(median) then begin
         xout = fltarr(b[1])
         for i = 0L, b[1]-1 do xout[i] = median(array[i, jlow:jhi], /even)
         return, xout
       endif else begin
         dx = y(1:ny-1)-y(0:ny-2)
         dx2 = dx/2.
         ygap = y(0:ny-2)+dx2
         w = ygap(jlow)-yl
         wt = w
         a = array(*, jlow)*w
         if jlow +1 lt jhi then begin
           for i = jlow+1, jhi-1 do begin
             w = ygap(i)-ygap(i-1)
             wt = wt + w
             a = array(*, i)*w+a
           endfor
         endif
         w = yu- ygap(jhi-1)
         wt = wt+w
         if keyword_set(integral) then return, reform(array(*, jhi)*w+a)
         return, reform(array(*, jhi)*w+a)/wt
       endelse
     endelse
     end
