; ------------------------------------------------------------------------ 
; $Id: ylineout.pro,v 1.5 2010/09/17 20:54:30 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
 Function ylineout, array, x, xvalin, width, median = median, $
       integral= integral
;+
;  Calculate a lineout of a two dimensional array at a given x value and 
;    over a specified width.
;  Format is: line = ylineout(array, xarray, xvalue [, width] [, /MEDIAN]  $
;         [, /INTEGRAL )
;    where;
;      LINE    - array of ordinate values for the lineout
;      ARRAY   - two dimensional array
;      XARRAY  - Abscissa values for array
;      XVALUE  - Abscissa value at the center of the lineout
;      WIDTH   - Width over which the values of array are averaged to 
;                obtain the values in LINE. Default is 0.0
;         Note:  Average is spatially weighted 
;                Average = integral (array)/(upper - lower)
;      MEDIAN  - If set and WIDTH > 0 then the median is returned.
;      INTEGRAL- If set and WIDTH > 0 then the integral is returned.
;                Note:  If median is set, INTEGRAL is ignored
;  Example:
;    Obtain a horizontal lineout from ARRAY with abscissa values X at a 
;    value of x = AMAX
;      line = ylineout (array, x, amax)
;    Note:  Since width is zero, value is simple interpolation.
;
;    Obtain a horizontal lineout from PICTURE with abscissa values, X, at
;    a value of x= xx(i), averaged over a width of 0.06
;      line = xlineout(picture, x, xx(i), 0.06)
;-
     on_error, 1
     if n_elements (array) eq 0 then begin
       Print, ' Two dimensional array must be defined'
       return, 0.0
     endif
     b = size (array)
     if b(0) ne 2 then begin
       print, ' Array for lineout must have two dimensions.'
       return, 0.0
     endif
     nx = n_elements(x)
     if nx ne b(1) then begin
       print, ' Abscissa array must be defined and have same ' 
       print, ' number of element as the array'
       return, 0.0
     endif
     if n_elements (xvalin) eq 0 then begin
       print, ' Abscissa value for the lineout must be defined'
       return , 0.0
     endif
     xval = xvalin
     if n_elements (width) eq 0 then width = 0 else width = abs(width)
     width2 = width*0.5
     if (width ne 0) then begin
       xl = xval-width2 > x(0)
       xu = xval+width2 < x(nx-1)
       if xl gt x(nx-1) then return, reform(array(nx-1, *))
       if xu lt x(0) then return, reform(array(0, *))
       xval = 0.5*(xu+xl)
       delta = min(abs(x-xl), jlow)
       delta = min(abs(x-xu), jhi)
;
;       if width is less than the local point spacing use width = 0
;       but don't change width because it is a passed parameter
;
        if jhi eq jlow then begin
          jmid = jhi
          width2 = 0
        endif else if x(jhi)-x(jhi-1) ge width then begin
         width2 = 0
         delta = min(abs(x-xval), jmid)
        endif
     endif else delta = min(abs(x-xval), jmid)
     if width2 eq 0 then begin
       if jmid eq 0 and xval le x(0) then return , reform(array(0, *))
       if jmid eq nx-1 and xval ge x(nx-1) then return, reform(array(nx-1, *))
       if xval ge x(jmid) then begin
         delta = x(jmid+1)-x(jmid)
         r1 = (xval-x(jmid))/delta
         r2 = 1-r1
         return, reform(r2*array(jmid,*)+r1*array(jmid+1,*))
       endif else begin
         delta = x(jmid)-x(jmid-1)
         r1 = (x(jmid)-xval)/delta
         r2 = 1-r1
         return, reform(r2*array(jmid,*)+r1*array(jmid-1,*))
       endelse
     endif else begin
       if keyword_set(median) then begin
         xout = fltarr(b[2])
         for i = 0L, b[2]-1 do xout[i] = median(array[jlow:jhi, i], /even)
         return, xout
       endif else begin
         dx = x(1:nx-1)-x(0:nx-2)
         dx2 = dx/2.
         xgap = x(0:nx-2)+dx2
         w = xgap(jlow)-xl
         wt = w
         a = array(jlow,*)*w
         if jlow +1 lt jhi then begin
           for i = jlow+1, jhi-1 do begin
             w = xgap(i)-xgap(i-1)
             wt = wt + w
             a = array(i,*)*w+a
           endfor
         endif
         w = xu- xgap(jhi-1)
         wt = wt+w
         if keyword_set(integral) then  return,reform(array(jhi,*)*w+a)
         return,reform(array(jhi,*)*w+a)/wt
       endelse
     endelse
     end
