; ------------------------------------------------------------------------ 
; $Id: zooms.pro,v 1.8 2011/01/06 20:03:05 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
;+
; NAME:
;	ZOOMS
;
; PURPOSE:
;	Provide a easy to zoom display of image data and to evaluate values.
;         Initial version is only for images with uniform pixels
;         (Type 1 field structures with DX, DY > 0.0)
;       Note: All inputs are the same as the PLOTFLD command.  See PLOTFLD
;             for more details.  
;
; CATEGORY:
;	Graphics 
;
; CALLING SEQUENCE:
;
;	ZOOMS, STR, P1, P2, P3, P4, P5
;
; INPUTS:
;
;       STR:      Structure array number or a 2D field structure.
;                 Note: SLICE can be used to generate a 2D structure.
;                 If this is not a single block structure, it will be plotted 
;                 in a pixmap and the interpolated, single block values
;                 used in zooms.
;                 P1, P2, P3, P4, P5 are used to obtain a 2D array from the 
;                    STR using the command:
;                    PLOTSTR, STR, P1, P2, P3, P4, P5, CN=2, RESULT=get_maxstr()-1, /NFarray
;
; KEYWORD PARAMETERS:
;      MAXZOOM:   Maximum ZOOM parameter in one dimension
;                 Default: MAXZOOM = max(MAX_ZOOM_DEFAULT , min([NX, NY])/8)
;                   where NX and NY are the number of pixels in X and Y.
;      MAGNIFICATION: Final magnification.  If there is a value on input
;                 between 1 and MaxZoom, this value will be the initial 
;                 maginification.  Default = 2
;                 Note:  MAGNIFICATION will caused result in an equal number
;                        of pixels in the X and Y direction.
;                 Size of zoom image in pixels will be NX/magnification where NX
;                   is the number of pixels in the X direction.
;      XZoomRange: Final value of XRANGE; can be used with PLOTSTR to plot other 
;                 data with the same abscissa values.  If there is a value
;                 on input, it will be used as the center of the initial
;                 magnification.  (More than one value will be averaged.)
;                 Default = Image X-Midpoint
;                 Note: Change in XZOOMRANGE must be greater than 0.08*DX
;      YZoomRange: Final value of YRANGE; can be used with PLOTSTR to plot other 
;                 data with the same abscissa values.  If there is a value
;                 on input, it will be used as the center of the initial
;                 magnification.  (More than one value will be averaged.)
;                 Default = Image Y-Midpoint
;                 Note: Change in YZOOMRANGE must be greater than 0.08*DY
;       X/Ysize:  Size of windows, Default is set by Dualsize
;                 Minimum window size is 100 x 100
;                 (See SET_DRAWSIZE in PFIDL.)
;       THERMOMETER:  By default, Thermometer is set.
;                 Right Image: THERMOMETER ={ystyle:1}
;                 Left Image: THERMOMETER ={ystyle:1}
;       ASPECT:   Preserve the aspect ratio.
;                 Default: ASPECT = 1 unless DX/DY > 10 or < 0.1
;       PIXPLOT:  If set, PIXPLOT will be sent to plotstr
;                 NOTE: Keyword PIXELS.  Four (4) characters are required.
;     
; *** Returned Values ***
;       X/YCursor: These values are set with the right mouse button.
;                  Values can be deleted using the delete button.  
;                  Designated points are shown in the right ZOOM window.
;                  If WIDTH is greater than 1 then X and Y are centered in
;                  a pixel.
;                  If NPOINT = 0, then XCURSOR = YCURSOR = -999
;       NPOINT:   Number of Xcursor and Ycursor values
;       VALUE:    Value of the field/image at the points [Xcursor, Ycursor]
;                 averaged over an areas of width WIDTH.
;                 If width is greater than 1, then the location is the center
;                   of a pixel and the sum of the pixels equal to WIDTH in each
;                   direction is totaled in determining VALUE
;                 If width is equal to 1 then the exact location is used and 
;                   VALUE is determined by interpolation to the 4 nearest neighbors.
;       WIDTH:    Array of WIDTH values for each VALUE
;                 If WIDTH is input with a single value, it will be used as the 
;                 initial width.  WIDTH is returned in pixel units; it will be rounded
;                 to an odd number of pixels.  The display is in log10(pixels)
;                 Default: width = 0.005 * minimum([nx, ny]) > 3
;                 where nx and ny are the number of pixels in each direction.
;                 Maximum width is 0.25*minimum([nx, ny]) 
;           NOTE: WIDTH must be a variable if a value is to be returned.
;           Note: Alog10(1) = 0.000000
;                 Alog10(3) = 0.477121
;                 Alog10(5) = 0.698970
;                 Alog10(7) = 0.845098
;                 Alog10(9) = 0.954243
;                  etc
;       PIXELS:   Used only on input.  If set, WIDTH is in pixels.
;                 On output, WIDTH is in pixel units.  (DX and DY may be
;                 unequal, but the average is over a square region in 
;                 pixel units.
;       SDEVIATION: Area weighted standard deviation for each VALUE
;                 MOMENT is used to calculate average and standard deviation
;                   for WIDTH > 1
;                 SDeviation = weighted standard deviation for WIDTH = 1
;                   that is: If VALUE = total(F*V) where F is a weight
;                            and V are the values at the corner of the 
;                            4 nearest values, then 
;                            SDEVIATION=sqrt(total(F*(V-Value)^2)
;
;         Note: The following 3 keywords are processed in the following 
;               order: UNSET, SetDefault, ShowDefault
;
;       UNSET      : Set Cursor and Maxzoom to their default values.
;                    Xsize and Ysize must be set using explicit values in
;                    ZOOMS or Set_DrawSize
;                    Defaults values which are set are:
;                             Cursor  = 1
;                             MaxZoom = 100
;                             Xsize   = Dualsize[0]
;                             Ysize   = Dualsize[1]
;       SETDEFAULT : Use to set the default value of Cursor, Maxzoom, Xsize
;                    and Ysize
;                  Note: Values in Dualsize will be changed if Xsize and Ysize
;                        are set.
;       SHOWDEFAULT: Show current default values
;
;      All other keyword parameters are the same as PLO except the following
;           which are set specifically for part or all of the ZOOMS command.
;       HEADER:   Used for the plot header; set to 0 for the zoom image
;       XSTYLE:   Set to 1 by default for all images.
;       YSTYLE:   Set to 1 by default for all images. 
;       INSTRUCTIONS:  Puts up an instruction widget.
;       INSTR_OFFSET:  Location for the instruction widget
;                      Default OFFSET = 0.5*[Xsize, Ysize]
;
; PROCEDURE:
;       Sliders allow the center point and the magnification to be set.
;       The center point can be set by clicking on the original image
;       The desired values are selected by right clinks in the zoom image or 
;         by setting the value in the field widget.
;       When the cursor is in the right window, the cursor position is displayed.
;          A LEFT button press will center the right window on that X-Y value.
;          A RIGHT button press will save the point into the keyword
;            parameters: Xcursor and Ycursor with the values and standard deviation.
;          VALUE will be determined by averaging a region of the image equal to 
;            the specified SPOT size.
;
; EXAMPLE:
;       Examine the details of the data in wdf array 3.
;         ZOOMS, 3
;
;       Compare arrays 2, 3, 4, and 5 in detail, put the legend at relative
;         positions of [0.5, 0.5]
;         ZOOMS, 2, 4, /ov, leg =[.5, .5]
;
; MODIFICATION HISTORY:
; 	Written by:       Paul Mix, April 8, 2004
;-

 Function mk_zooms_textstr, nvalue
;
  common ZOOMS_common, arr, draw1, draw2, asp, thermo, dx1, dx2, sd, wid, val, $
            fslid1, fslid2, fslid3, fslid4, middle, magnif, xr, yr, nwidth, $
            datamin, datarange, coordmin, coordrange, blklist, extras,  $
            datamax, image, xmaxt, xmint, maxzoom, epsilon, xsize, ysize, $
            wxcur, wycur, wvcur, xcur, ycur, ncur, maxcur, xarray, yarray, nx, ny
; convert last data point to a text string
       on_error, 1
      if n_elements (nvalue) eq 1 then $
        values = [xcur[nvalue], ycur[nvalue], val[nvalue], $
                 wid[nvalue], sd[nvalue]] else values = nvalue
      data = ['X Value', 'Y Value', 'Image Value',  $
                'Spot Size', 'Standard Deviation']
      str = make_array(2, /string)
      
      str[0]=string(format='(a, t16, a, t31, a, t46, a, t61, a)', data)
      str(1)=string(format='(a, t16, a, t31, a, t46, a, t61, a)', $
           strtrim(values, 2))
      return, str
      end

PRO ZOOMS_event, Event
  common ZOOMS_common
 on_error, 1
  WIDGET_CONTROL,Event.Id,GET_UVALUE=Ev
  tolerance = 0.08
  CASE Ev OF 

  'FSLID1': BEGIN
        magnif = (10.0^(event.value)) > 1.0
        update_ZOOMS, 0
      END

  'FSLID2': BEGIN
        temp = event.value
        if abs(temp- middle[0]) gt tolerance *dx1 then begin
          middle[0] = temp
          update_ZOOMS, 0
        endif
      END

  'FSLID3': BEGIN
        temp = event.value
        if abs(temp- middle[1]) gt tolerance *dx2 then begin
          middle[1] = temp
          update_ZOOMS, 0
        endif
      END

  'FSLID4': BEGIN
        temp = round(10.0^(event.value)) > 1
        nwidth= ((temp/2)*2+1) > 1L
        widget_control, fslid4, set_value = alog10(nwidth)
      END

  'DRAW8': BEGIN
      x = [event.x, event.y]
      middle=(x-coordmin)/coordrange*datarange+datamin > datamin
      Middle = middle < datamax
      widget_control, fslid2, set_value = middle[0]
      widget_control, fslid3, set_value = middle[1]
      update_ZOOMS, 0
      END

  'DRAW9': BEGIN
;     type = 0 - Button press
;     type = 2 - Motion
      if event.type eq 0 or event.type eq 2 then begin
        coord = convert_coord(event.x, event.y, /device, /to_data)
        widget_control, wxcur, set_value = coord[0]
        widget_control, wycur, set_value = coord[1]
        icent = ((coord[0:1] -datamin)/[dx1, dx2]) > [0.0, 0.0]
        icent = icent< [nx-1, ny-1]
        if nwidth eq 1 then begin
          ii = floor(icent)
          ie = ii+1 
          if ie[0] eq nx then begin
            ie[0] = ie[0]-1
            ii[0] = ie[0]-1
          endif
          if ie[1] eq ny then begin
            ie[1] = ie[1]-1
            ii[1] = ie[1]-1
          endif
          di = (icent - ii) > [0.0, 0.0]
          de = (ie - icent) > [0.0, 0.0]
          dd = di*de
          
          v = (reform(get_array(arr, -1)))[ii[0]:ie[0], ii[1]:ie[1]]
          vv = [v[0,0], v[1,0], v[0,1], v[1,1]]
          ww = [de[0]*de[1], di[0]*de[1], de[0]*di[1], di[0]*di[1]]
;
          average = total(vv*ww)
          sdev = sqrt(total(((vv-average)^2)*ww))
        endif else begin
          icent = round(icent)
          idel = nwidth/2
          ii = (icent-idel) > 0L
          ie = (ii + (nwidth-1)) < [nx-1, ny-1]
          ii = ie -(nwidth-1)
          v = (reform(get_array(arr, -1)))[ii[0]:ie[0], ii[1]:ie[1]]
          mm = moment(v, sde= sdev)
          average = mm[0]
        endelse
        widget_control, wvcur, set_value = average
        if event.type eq 0 then begin
;         button press
          if event.press eq 1 then begin
;           Left button
            middle = (coord > datamin) < datamax
            widget_control, fslid2, set_value = middle[0]
            widget_control, fslid3, set_value = middle[1]
            update_ZOOMS, 0
          endif else if event.press eq 4 then begin
            ncur = ncur+1
            if ncur ge maxcur then begin
              print, '% ZOOMS: Maximum number of points have been stored'
              ncur = ncur-1
              return
            endif
            if nwidth eq 1 then begin
              xcur[ncur]= coord[0]
              ycur[ncur]= coord[1]
            endif else begin
              xcur[ncur]= xarray[icent[0]]
              ycur[ncur]= yarray[icent[1]]
            endelse       
            val[ncur] = average
            sd[ncur] = sdev
            wid[ncur]=nwidth
            ss = MK_ZOOMS_TEXTSTR (ncur)
            widget_control, wvcur, set_value= val[ncur]
            widget_control, blklist, set_value = ss 
            wset, draw2
            plots, xcur[ncur], ycur[ncur], psym = 8, col =5
            if nwidth gt 1 then begin
              jx = [ii[0], ie[0], ie[0], ii[0], ii[0]]
              jy = [ii[1], ii[1], ie[1], ie[1], ii[1]]
              oplot, xarray[jx]+[-dx1, dx1, dx1, -dx1, -dx1]*0.5,  $
                     yarray[jy]+[-dx2, -dx2, dx2, dx2, -dx2]*0.5
            endif
          endif
        endif
      endif
      END

  'DELETE_PT': BEGIN
        if ncur gt 0 then begin
          NCUR = NCUR-1
          update_zooms, 0
          ss = MK_ZOOMS_TEXTSTR (ncur)
          widget_control, blklist, set_value = ss
        endif
      END

  'SAVE_PT': BEGIN
        widget_control, wxcur, get_value = xx
        widget_control, wycur, get_value = yy
        coord= [xx, yy]
        icent = ((coord[0:1] -datamin)/[dx1, dx2]) > [0.0, 0.0]
        if nwidth eq 1 then begin
          ii = floor(icent)
          ie = ii+1 < [nx, ny]
          if ie[0] eq nx then begin
            ie[0] = ie[0]-1
            ii[0] = ie[0]-1
          endif
          if ie[1] eq ny then begin
            ie[1] = ie[1]-1
            ii[1] = ie[1]-1
          endif
          di = (icent - ii) > [0.0, 0.0]
          de = (ie - icent) > [0.0, 0.0]
          dd = di*de

          v = (reform(get_array(arr, -1)))[ii[0]:ie[0], ii[1]:ie[1]]
          average=v[0,0]*de[0]*de[1]+v[1,0]*di[0]*de[1]+ $
                  v[0,1]*de[0]*di[1]+v[1,1]*di[0]*di[1]
          sdev = sqrt(total((v-average)^2)/3.0)
          print, average, sdev
        endif else begin
          icent = round(icent)
          idel = nwidth/2
          ii = (icent-idel) > 0L
          ie = (ii + (nwidth-1)) < [nx-1, ny-1]
          ii = ie -(nwidth-1)
          v = (reform(get_array(arr, -1)))[ii[0]:ie[0], ii[1]:ie[1]]
          mm = moment(v, sde= sdev)
          average = mm[0]
        endelse
        ncur = ncur+1
        if ncur ge maxcur then begin
          print, '% ZOOMS: Maximum number of points have been stored'
          ncur = ncur-1
          return
        endif
        if nwidth eq 1 then begin
          xcur[ncur]= coord[0]
          ycur[ncur]= coord[1]
        endif else begin
          xcur[ncur]= xarray[icent[0]]
          ycur[ncur]= yarray[icent[1]]
        endelse

        val[ncur] = average
        sd[ncur] = sdev
        wid[ncur]=nwidth
        ss = MK_ZOOMS_TEXTSTR (ncur)
        widget_control, wvcur, set_value= val[ncur]
        widget_control, blklist, set_value = ss
        wset, draw2
        plots, xcur[ncur], ycur[ncur], psym = 8, col = 5
        if nwidth gt 1 then begin
          jx = [ii[0], ie[0], ie[0], ii[0], ii[0]]
          jy = [ii[1], ii[1], ie[1], ie[1], ii[1]]
          oplot, xarray[jx]+[-dx1, dx1, dx1, -dx1, -dx1]*0.5,  $
                 yarray[jy]+[-dx2, -dx2, dx2, dx2, -dx2]*0.5
        endif
        widget_control, wxcur, set_value = xcur[ncur] 
        widget_control, wycur, set_value = ycur[ncur]

      END

  'BUTTONQ': BEGIN
      widget_control, event.top, /destroy
      END
       
   ELSE: Print, 'No Handler for event: ', ev
  ENDCASE
END

pro update_ZOOMS, mode
  common ZOOMS_common
  tolerance = 0.08
  if mode eq 0 then begin
    deltax = nx/magnif - 1.0
    center_pix = [(middle[0]-datamin[0])/dx1, (middle[1] -datamin[1])/dx2]
    zoom_min =  (center_pix -deltax/2) > 0
    zoom_max = zoom_min+deltax < [nx-1, ny-1]
    if zoom_max[0] eq nx-1 then zoom_min[0]=(nx-1-deltax)>0.0
    if zoom_max[1] eq ny-1 then zoom_min[1]=(ny-1-deltax)>0.0

    wset, draw1
    xrn = [zoom_min[0], zoom_max[0]] *dx1 + datamin[0]
    yrn = [zoom_min[1], zoom_max[1]] *dx2 + datamin[1]
    if total (abs(xr-xrn)) lt dx1 *tolerance and $
       total (abs(yr-yrn)) lt dx2 *tolerance then return
    xr = temporary(xrn)
    yr = temporary(yrn)
    xx =  [xr[0], xr[1], xr[1], xr[0], xr[0]]
    yy = [yr[0], yr[0], yr[1], yr[1], yr[0]]

    xx = (xx- datamin[0])/datarange[0] *coordrange[0] +coordmin[0]
    yy = (yy- datamin[1])/datarange[1] *coordrange[1] +coordmin[1]
    device, copy= [0, 0, xsize, ysize, 0, 0, image]
    plots, xx, yy, /device
    wset, draw2
    plotstr, arr, xr = xr, yr = yr, /xstyle,/ystyle, _extra = extras, $
       aspect=asp, pixplot = pixplot, head =0, thermometer = thermo, cn=2, quiet = -1
    if ncur gt 0 then $
         plots, xcur[1:ncur], ycur[1:ncur], psym =8, col=5, noclip=0
  endif
  end

     
PRO ZOOMS, strin, p1, p2, p3, p4, p5, maxzoom = maxzoomin, xsize = xsizein, ysize = ysizein, $
        xzoomrange = xzoomrange, yzoomrange = yzoomrange, $
        magnification = magnification, npoint = npoint, $
        GROUP=Group,  thermometer = thermometer, aspect = aspect, $
        limit = limit,  modal = modal, instructions = instructions, $
        instr_offset = instr_offset, pixels = pixels, pixplot = pixplot,  $
        cursor = mycursor, xcursor = xcursor, ycursor = ycursor, $
        value = value, width = width, sdeviation = sdeviation, $
        showdefault = showdefault, setdefault = setdefault, $
         unset = unset, _extra = extrastuff
       
;
;       scroll = scrollin
on_error, 1
@draw_defaults.com
@com.dat
  common pffcom, initpff, pff_exe
  common ZOOMS_common
  common ZOOMS_save_common, cursor_save, maxzoom_save

; see if we are initialized
@qscom.dat
@qsstrcom.dat
  if n_elements (structinit) eq 0 then qsstruct_init
  if n_elements (qsplotinit) eq 0 then qsinit
  lcaction = 0
  if n_elements(cursor_save) eq 0 or keyword_set(unset) then begin
    if keyword_set(unset) then lcaction = 1
    cursor_save = 1
    maxzoom_save = 100.0
  endif

  if keyword_set(setdefault) then begin
    lcaction = 1
    if n_elements(mycursor) gt 0 then cursor_save = mycursor  
    if n_elements(maxzoomin) gt 0 then maxzoom_save = maxzoomin
    if n_elements(xsizein) gt 0 then dualsize[0] = xsizein > 100
    if n_elements(ysizein) gt 0 then dualsize[1] = ysizein > 100
  endif
    
  if keyword_set(showdefault) then begin
    lcaction = 1
    print, ' ZOOMS Defaults:'
    print, ' Default CURSOR Value  = ' , strtrim (cursor_save, 2)
    print, ' Default MAXZOOM Value = ' , strtrim (maxzoom_save, 2)
    print, ' Default XSIZE Value   = ' , strtrim (dualsize[0], 2)
    print, ' Default YSIZE Value   = ' , strtrim (dualsize[1], 2)
  endif

  if (n_params() eq 0) then begin 
     if (lcaction eq 1) then return
      print,'% Format:  ZOOMS, STR'
      print,'KeyWords:  MaxZoom, Magnification, XZoomRange, YZoomRange'
      print,'           XSize, YSize, Cursor, PIXPLOT'
      print,'           XCursor, YCursor, Value, Width, SDeviation'
      print,'           Pixels, UnSet, SetDefault, ShowDefault'
      return
  endif

  if n_elements(thermometer) eq 0 then thermo ={ystyle:1}
  if n_elements(mycursor) eq 0 then mycursor = cursor_save
  if n_elements(xsizein) eq 0 then xsize = dualsize(0) else $
      xsize = xsizein > 100
  if n_elements(ysizein) eq 0 then ysize = dualsize(1) else $
      ysize = ysizein > 100
  arr = get_valid_structure(strin, 'ZOOMS', ierror = ier)
  if ier ne 0 then return

  if arr.type ne 1 then begin
    Print, '% ZOOMS: Structure array must be a field structure'
    return
  endif
  oldwin = !d.window
  oldx_save = !x
  oldy_save = !y
  oldp_save = !p

  if arr.block gt 1 then begin
    bigwin
    resul = get_maxstr()-1
    delstr, resul
    plotstr, arr, p1, p2, p3, p4, p5, cn=2, result = resul, /aspect, /nf, pixplot = pixplot

    wdelete
    if strvalid(resul, /valid) lt 0 then begin
      print, '% Zooms:  Unable to obtain a 2D structure'
      return
    endif
    print, '% Zooms:  Structure has multiple blocks.'
    print, '          Image put on a single block in structure', resul
    s2i, arr, resul
  endif
   
  ok = where(strpos(tag_names(arr), 'DX') ge 0 , nok)
  lcunif = 0
  if nok gt 0  &&  ((arr.dx1(0) gt 0.0) and (arr.dx2(0) gt 0.0)) then lcunif = 2
  xarray = get_array(arr, 1)
  yarray = get_array(arr, 2)
  nx = n_elements (xarray)
  ny = n_elements (yarray)
  if n_elements (maxzoomin) eq  0 then $ 
    maxzoom = max( min([nx, ny])*0.125, maxzoom_save)  else $
    maxzoom = float(maxzoomin(0))

  if lcunif eq 0 then begin
      dx1 = 1.0
      dx2 = 1.0
      check_uniform, xarray, dx1
      check_uniform, yarray, dx2
      if ((dx1 gt 0.0) and (dx2(0) gt 0.0)) then lcunif = 2
      if lcunif lt 2 then begin
         print,'% ZOOMS: Structure must be uniform in both directions'
         resul = get_maxstr()-1
         delstr, resul
         plotstr, arr, p1, p2, p3, p4, p5, cn=2, result = resul, /aspect, pixplot= pixplot,  /nf
         wdelete
         if strvalid(resul, /valid) lt 0 then begin
           print, '% Zooms:  Unable to obtain a uniform 2D structure'
           return
         endif
         s2i, arr, resul
         dx1 = arr.dx1[0]
         dx2 = arr.dx2[0]
      endif 
      dx1 = dx1[0] & dx2 = dx2[0]
  endif else begin
      dx1 = arr.dx1[0]
      dx2 = arr.dx2[0]
  endelse

  if n_elements(aspect) eq 0 then begin
    if (dx1/dx2 gt 10.0) or (dx1/dx2 lt  0.10) then asp = 0 else asp = 1
  endif
  if n_elements (magnification) eq 0 then magnification =2.0 else $
    if magnification le 1 or magnification gt maxzoom then $
      magnification =2.0
  xrange = [xarray[0], xarray[nx-1]]
  yrange = [yarray[0], yarray[ny-1]]
  nxz =  n_elements(xzoomrange)
  nyz =  n_elements(yzoomrange)
  if nxz eq 0 then xzoomrange = (xarray[0] + xarray[nx-1])*0.5 else begin
    temp = total(xzoomrange)/nxz
    if (temp ge xarray[0] and temp le xarray[nx-1]) then $
       xzoomrange = temp else  xzoomrange = (xarray[0] + xarray[nx-1])*0.5
  endelse
  if nyz eq 0 then yzoomrange = (yarray[0] + yarray[ny-1])*0.5 else begin
    temp = total(yzoomrange)/nyz
    if (temp ge yarray[0] and temp le yarray[ny-1]) then $
       yzoomrange = temp else  yzoomrange = (yarray[0] + yarray[ny-1])*0.5
  endelse

  IF N_ELEMENTS(Group) EQ 0 THEN GROUP=0

  MAIN13 = WIDGET_BASE(GROUP_LEADER=Group, $
      COLUMN=1, modal = modal, $
      MAP=1, $
      TITLE='Zoom - Images Arrays')

  BASE2 = WIDGET_BASE(MAIN13, $
      ROW=1, $
      MAP=1, $
      UVALUE='BASE2')

  BaseL = WIDGET_BASE(Base2, $
      FRAME = 3, $
      XPAD = 20, $
      column=1, $
      MAP=1)

  LABEL38 = WIDGET_LABEL( BASEL, /align_center,  $
      VALUE='Pick Magnification and Center of Zoom')

  BaseLS = WIDGET_BASE(BaseL, $
      FRAME = 1, $
      ROW=1, $
      MAP=1)

      magnif = magnification

  FSLID1 = cw_fslider( BASELS, $
      DRAG=0, /edit, $
      TITLE='Log of ZOOM Magnification', $
      UVALUE='FSLID1', $
      VALUE=alog10(magnif), $
      MAXIMUM = alog10(maxzoom), MINIMUM = 0.0)

  FSLID2 = cw_fslider( BASELS, $
      DRAG=1, /edit, $
      TITLE='X Center of ZOOM', $
      UVALUE='FSLID2',Value = xzoomrange, $
      MAXIMUM = xarray[nx-1], MINIMUM = xarray[0])

  FSLID3 = cw_fslider( BASELS, $
      DRAG=1, /edit, $
      TITLE='Y Center of ZOOM', $
      UVALUE='FSLID3',Value = yzoomrange, $
      MAXIMUM = yarray[ny-1], MINIMUM = yarray[0])

  center_pix = [(xzoomrange -xrange[0])/dx1, (yzoomrange -yrange[0])/dx2]
  middle = [xzoomrange, yzoomrange]
  Baser = WIDGET_BASE(Base2, $
      COLUMN=1, $
      XPAD = 20, $
      frame = 3, $
      MAP=1)

  LABEL39 = WIDGET_LABEL( BASER,  /align_center, $
      VALUE='Select Spot Size')

  BaseRS = WIDGET_BASE(BaseR, $
      FRAME = 1, $
      ROW=1, $
      MAP=1)

  nwidth = 0
  if n_elements(width) eq 1 then $
    nwidth = (keyword_set(pixels))? width:(width/dx1)
  minsize = min([nx, ny])
  maxwidth =  (0.25 * minsize)
  if nwidth lt 1 or nwidth gt maxwidth then begin
    nwidth = 0.005*minsize
    nwidth = round(nwidth)
    nwidth= ((nwidth/2)*2+1) > 3
  endif else  nwidth= ((nwidth/2)*2+1)

  FSLID4 = cw_fslider( BASERS, $
      DRAG=1, /edit, $
      TITLE='Log of Spot Pixel Size', $
      UVALUE='FSLID4',Value = alog10(nwidth), $
      MAXIMUM = alog10(maxwidth),  MINIMUM = 0.0)
  ncur = 0L
  if keyword_set(mycursor) then begin
    maxcur = 1024L
    xcur = make_array(maxcur)
    ycur = make_array(maxcur)
    sd  = make_array(maxcur)
    val  = make_array(maxcur)
    wid  = make_array(maxcur)
    mkcircle
    base_cur_ins  = WIDGET_BASE(MAIN13, $
      column=1, $
      MAP=1, $
      UVALUE='BASE_CUR_INS', FRAME=2)
    LABEL38 = WIDGET_LABEL( base_cur_ins, $
      VALUE=' Right Window Cursor ')

    LABEL38 = WIDGET_LABEL( base_cur_ins, $
      VALUE='Left Button to Center, Right to Save, ' $
      + 'Cursor Location and Value: ', /align_center)


    basecur = WIDGET_BASE(base_cur_ins, $
      ROW=1, $
      MAP=1, $
      UVALUE='BASECUR')

      wxcur = cw_field(basecur,  /col, Title ='X-Value:', /floating, $
           value = 0.0, /frame)
      wycur = cw_field(basecur,  /col, Title ='Y-Value:', /floating, $
           value = 0.0, /frame)
      wvcur = cw_field(basecur,  /col, Title ='Image-Value:', /floating, $
           value = 0.0, /frame)

      delbut = widget_button (basecur, value ='Delete Point', /frame, $
        UVALUE = 'DELETE_PT')
      savbut = widget_button (basecur, value ='Save Point', /frame, $
        UVALUE = 'SAVE_PT')
  endif


    base_values = WIDGET_BASE(MAIN13, $
      row=1, $
      MAP=1, $
      UVALUE='BASE_CUR_INS', FRAME=2)
    LABEL38 = WIDGET_LABEL( base_values, $
      VALUE=' Last Point Values: ')

        str_values = mk_zooms_textstr(make_array(5, value=0.0))
        blklist=WIDGET_LIST( base_values,VALUE=str_values, $
          UVALUE='BLKLIST', $
          YSIZE=2)

  ButtonQ =  WIDGET_BUTTON( Main13, $
      UVALUE='BUTTONQ', $
      VALUE='           Q  U  I  T           ')

  BASE3 = WIDGET_BASE(MAIN13, $
      ROW=1, $
      MAP=1, $
      UVALUE='BASE3')

  DRAW8 = WIDGET_DRAW( BASE3, $
      BUTTON_EVENTS=1, $
      RETAIN=1, $
      UVALUE='DRAW8', $
      XSIZE=xsize, $
      YSIZE=ysize)

  DRAW9 = WIDGET_DRAW( BASE3, $
      RETAIN=1, $
      UVALUE='DRAW9', $
      button_events = mycursor, $
      motion_events = mycursor, $
      sensitive = mycursor, $
      XSIZE=xsize, YSIZE=ysize)

  WIDGET_CONTROL, MAIN13, /REALIZE
  ; Get drawable window index

  WIDGET_CONTROL, DRAW8, GET_VALUE=DRAW1

  ; Get drawable window index

  WIDGET_CONTROL, DRAW9, GET_VALUE=DRAW2
  wset,draw1
  undefine, extras
  if n_elements (extrastuff) gt 0 then extras = extrastuff
  plotstr, arr,/xstyle,/ystyle, aspect = asp, cn=2, pixplot = pixplot, $
       thermometer = thermo, _extra = extras
  window,  /pixmap, /free, xsize = xsize, ysize = ysize
  image = !d.window
  device, copy = [0, 0, xsize, ysize, 0, 0, draw1]
  wset,draw1
  gxmin = !x.crange(0)
  gxmax = !x.crange(1)
  gymin = !y.crange(0)
  gymax = !y.crange(1)

; device coordinates for the grid
  coordmin = convert_coord(gxmin, gymin, /data, /to_device)
  coordmax = convert_coord(gxmax, gymax, /data, /to_device)
; device coordinate range
  coordrange = coordmax-coordmin
; xrange in device coordinates
  dxcrange = [coordmin(0), coordmax(0)]
; yrange in device coordinates
  dycrange = [coordmin(1), coordmax(1)]
  
  datamax =  [gxmax, gymax]
  datamin =  [gxmin, gymin]
  datarange = datamax - datamin
 
  deltax = nx/magnif - 1.0
  zoom_min =  (center_pix -deltax/2) > 0
  zoom_max = zoom_min+deltax < [nx-1, ny-1]
  if zoom_max[0] eq nx-1 then zoom_min[0]=(nx-1-deltax)>0.0
  if zoom_max[1] eq ny-1 then zoom_min[1]=(ny-1-deltax)>0.0
  
  xr = [zoom_min[0], zoom_max[0]] *dx1 + xrange[0]
  yr = [zoom_min[1], zoom_max[1]] *dx2 + yrange[0]

  wset, draw2
  epsilon = datarange(0)/maxzoom/3.0
  yeng_zoom_scale = 1
  xeng_zoom_scale = 1
  plotstr, arr, xr = xr, yr = yr, cn=2, /xstyle,/ystyle, pixplot = pixplot, $
     aspect=asp, head =0, thermometer = thermo, _extra = extras

; check the scaling
  wset , draw1

   xx =  [xr[0], xr[1], xr[1], xr[0], xr[0]]
   yy = [yr[0], yr[0], yr[1], yr[1], yr[0]]

   xx = (xx- datamin[0])/datarange[0] *coordrange[0] +coordmin[0]
   yy = (yy- datamin[1])/datarange[1] *coordrange[1] +coordmin[1]

  plots, xx, yy, /device
;  plo, par1, par2, grid = 1, over = ov, legend = -1, header = 0, $
;        /xstyle,  xrange = [xmin, xmax],  _extra = extrastuff
  if n_elements(instructions) gt 0 then begin
    if n_elements(instr_offset) eq 0 then instr_offset = 0.5*[xsize, ysize]
    wwinstr = winstruction(instructions, offset =instr_offset)
  endif else wwinstr = -1

  ninit = maxcur
  xcursor = make_array(ninit)
  ycursor = make_array(ninit)
  value  = make_array(ninit)
  sdeviation = make_array(ninit)
  width = make_array(ninit)

  XMANAGER, 'ZOOMS', MAIN13
  xzoomrange = xr
  yzoomrange = yr
  magnification = magnif 
  if keyword_set (limit) then begin
    if par2 gt 1 then par1 = par1+indgen(par2)
    lim, par1, xmin, xmax
  endif
  npoint = ncur
  if ncur gt 0 then begin
    value = val[1: ncur]
    sdeviation = sd[1: ncur]
    width = wid[1: ncur]
    xcursor = xcur[1: ncur]
    ycursor = ycur[1: ncur]
  endif else begin
    xcursor = -999
    ycursor = -999
  endelse
  widget_control, wwinstr, bad_id = bad
  if bad eq 0 then widget_control, wwinstr, /destroy
  wdelete, image
  if oldwin ge 0 then begin
      wset, oldwin
      !x = oldx_save
      !y = oldy_save
      !p = oldp_save
  endif
END
