; ------------------------------------------------------------------------ 
; $Id: tqcha.pro,v 1.3 2007/06/06 20:12:26 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
 Pro  tqcha, mxdim = mxd, mydim = myd, madim = mad,help= help,  $
        axdim = axd, aydim = ayd,  aadim = aad, replicate = replicate, $
        zx = zx, zy = zy
;
;
;+
; NAME:
;       TQCHA
;
; PURPOSE:
;       Change the prameters of a TQ common block for conductors or grids
;       Note: Grid units are not changed.  That is, if units are inches,
;             then the grid units will remain "inches".
;       To change grid values use the following:
;         tqhelp, x = xout, dx = dxout, y = yout, dy = dyout
;       to change actual values use the following:
;         tqgrid, xout*mxdim +axdim, dxout*mxdim, mult = newmult , /x
;         tqgrid, yout*mydim +aydim, dyout*mydim,  /y
;       where mxdim, axdim, mydim, and aydim are defined below.
;       NEWMULT is a new scale multiplier.
;
; CATEGORY: 
;       Utility 
; CALLING SEQUENCE: 
;       TQCHA 
; INPUTS: 
;       None  -   TQ common block is modified for Conductors only
; KEYWORD PARAMETERS:
;       MXDIM     Multiply the X conductor values by this value
;       MYDIM     Multiply the Y conductor values by this value
;       MADIM     Multiply BOTH conductor spatial values by this value
;       AXDIM     Add this offset to the X spatial dimension
;       AYDIM     Add this offset to the Y spatial dimension
;       AADIM     Add this offset to BOTH the spatial dimensions
;       REPLICATE If present and not zero, then the specified conductor
;                 will be replicated with the specified shift and scaling
;       ZX        If present and not zero then AXDIM will be set to the the
;                 negative of the minimum conductor X-value
;                 AXDIM will be set to the value
;       ZY        If present and not zero then AYDIM will be set to the the
;                 negative of the minimum conductor Y-value
;                 AYDIM will be set to the value
;       HELP      If present and not zero, a help message will be printed.
;       NOTE:  Multiplication and division are performed according to:
;              Xnew = Xold * MXDIM + AXDIM
; OUTPUTS:
;       NONE    The specified structures are modified and stored in their
;               original locations
; COMMON BLOCKS:
;       PFIDLTQ
; RESTRICTIONS:
;       At least one conductor must be defined.
; EXAMPLES:
;       Shift the entire conductor assembly by 2.5 in x and 3.5 in y
;          TQCHA, ax = 2.5, ay = 3.5
;       Replicate conductor 2, 9 times with a shift in x of 1.2
;          For i = 1, 9 do tqcha, replicate= 2, ax = i*1.2
;       
; MODIFICATION HISTORY:
;      Original version, L. P. Mix, October 21, 1994
;-
;
on_error, 1
;     include the common block
;
@pfidltq.com
;
      if n_elements (cond) eq 0 then begin
        print, 'No conductors defined, TQCHA not possible to execute'
        return
      endif
      if cond.ncon eq 0 then begin
        print, ' Number of conductors is zero, TQCHA is returning'
        return
      endif
;
      if keyword_set (help) then begin
        print,'% Format:', $ 
        ' TQCHA , [, MXDIM= mxd] [, MYDIM= myd] [, MADIM= mad]'
 print, '         [, AXDIM= axd] [, AYDIM= ayd] [, AADIM= aad]'
 print, '         [, HELP = help]'
;
      endif
;
;     determine if strarray is specified
;
;  check the parameters
      if n_elements(mad) ne 0 then begin
        mxd = mad
        myd = mad
      endif
      if n_elements(aad) ne 0 then begin
        axd = aad
        ayd = aad
      endif
;
      if n_elements(mxd) gt 0 then mxd = mxd(0) else mxd = 1.0
;
      if n_elements(myd) gt 0 then myd = myd(0) else myd = 1.0
;
      if n_elements(axd) gt 0 then axd = axd(0) else axd = 0.0
;
      if n_elements(ayd) gt 0 then ayd = ayd(0) else ayd = 0.0
;
      if keyword_set(zx) then begin
        axd = -min(cond.x)
        Print, ' Calculated X-shift of ', strtrim(axd, 2), ' to zero data'
      endif
      if keyword_set(zy) then begin
        ayd = -min(cond.y)
        Print, ' Calculated Y-shift of ', strtrim(ayd, 2), ' to zero data'
      endif
      
; do the deed
;
    if n_elements (replicate) eq 0 then begin
      if mxd ne 1.0 or axd ne 0.0 then cond.x = mxd*cond.x+axd
      if myd ne 1.0 or ayd ne 0.0 then cond.y = myd*cond.y+ayd
      cond.lcfit = 0*cond.lcfit
    endif else begin
      if replicate gt 0 and replicate le cond.ncon then begin
        i = replicate
        a = cond.x(cond.locx(i-1):cond.locx(i)-1)
        b = cond.y(cond.locx(i-1):cond.locx(i)-1)
        if mxd ne 1.0 or axd ne 0.0 then a = mxd*a + axd
        if myd ne 1.0 or ayd ne 0.0 then b = myd*b + ayd
        tqcond, a, b
      endif else print, 'Invalid conductor number for REPLICATE'
    endelse
    return 
    end
