; ------------------------------------------------------------------------ 
; $Id: tqcond.pro,v 1.3 2007/06/06 20:12:26 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
 Pro tqcond, xin, yin, locxin, replace = replace, delete= delete, $
      name = name, reset = reset
;
;+
;   Purpose: Store conductor data in the TQ common block
; Format: TQCOND,x, y, locx, replace = replace, delete= delete, reset = reset
;    where x and y are ordered pairs defining one or more conductors
;          NCON is the number of conductors NCON = n_elements(locx)-1
;          If x and y repreasent more than one conductor then 
;            locx is an array defining the start location for each conductor
;      Note: conductor n has values given by:
;             X = x(locx(n-1):locx(n)-1)
;             Y = y(locx(n-1):locx(n)-1)
;          LOCX is not required if NCON = 1
;          RESET set everything to initial conditions.
;          REPLACE - Use the xy conductor values to replace one or more values
;                    Number of conductors (NCON) must equal n_elements (replace)
;          DELETE  - Delete a conductor. X, Y, and LOCX are ignored
;          NEW     - Use the xy values to add new conductors
;                    NEW is the default operation and is not checked.
;          Conductors are numbered 1, 2, ...
;          Example:
;            Store a dummy conductor in the TQ common block
;                 tqcond, [0, 1], [0, 3]
;            Read data from a dxf file and convert to cm and store in 
;               TQ common block
;                 readdxf, a, b, l, 'data.dxf'
;                 a = a*2.54 & b = b*2.54
;                 tqcond, a, b, l
;
;-
;  Common block
;
@pfidltq.com
;
;
      on_error, 1

      if n_elements(cond) eq 0 then ncon = 0 else begin
        b = size(cond)
        if b(b(0)+1) ne 8 then ncon = 0 else ncon = cond.ncon
      endelse
      if keyword_set (reset) then begin
        cond = 0L
        ncon = 0L
      endif
      if n_elements (delete) gt 0 then begin
        if ncon eq 0 then begin
          print, 'No conductors are defined. Cannot delete a conductor'
          return
        endif
        nd = n_elements (delete)
        del = 0L
        if nd gt 1 then begin
          ok = sort(delete)
          delete = delete(reverse(ok))
        endif
;
        cx = cond.x
        cy = cond.y
        locx = cond.locx
        lcfit = cond.lcfit
        ix = cond.ix
        iy = cond.iy
        loci = cond.loci
        locp = cond.locp
        ipx = cond.iplotx
        ipy = cond.iploty
        nbad = cond.nbad
        badxy = cond.badxy
        locb  = cond.locb
        ixx = 0L
        iyy = 0L
;
        if n_elements (lcfit) lt ncon + 1 then $
          lcfit = make_array(ncon+1, /int, value = lcfit(0))
        if n_elements (nbad) lt ncon + 1 then $
          nbad = make_array(ncon+1, /int, value = nbad(0))
;
        for i = 0, nd-1 do begin
          if delete(i) gt 0 and delete(i) le ncon then begin
            array2pack, delete(i), ncon, locx, 0, cx, ixx, cy, iyy
;     deleted after having problems with deleting and adding
;     loci had incorrect number of elements.
;           if n_elements (loci) eq ncon+1 then begin
;
              ncon= ncon+1
              array2pack, delete(i), ncon, loci, 0, ix, ixx, iy, iyy
;           endif
            ncon= ncon+1
            array2pack, delete(i), ncon, locp, 0, ipx, ixx, ipy, iyy
            ncon= ncon+1
            array1pack, delete(i), ncon, locb, 0, badxy, ixx
            if delete(i) le ncon then begin
              lcfit = [lcfit(0:delete(i)-1), lcfit(delete(i)+1:*)]
              nbad = [nbad(0:delete(i)-1), nbad(delete(i)+1:*)]
            endif
          endif
        endfor
        if ncon gt 0 then begin
          ok = where (lcfit(1:*) gt 0, nok)
          if nok eq ncon then lcfit (0) = 1 else lcfit(0) = 0
          nbad(0) = total(nbad(1:*))
        endif
        cond_names = ''
        cond = {ncon:ncon, lcfit:lcfit, x:cx, y:cy, locx:locx, $
             ix:ix, iy:iy, loci:loci, iplotx:ipx, iploty:ipy, locp:locp, $
             badxy:badxy, locb:locb, nbad:nbad, cond_names: cond_names }
        return
      endif
;
;     must be a replace or an add now
;
      if n_elements(locxin) eq 0 then locxin = [0, n_elements(xin)]
      npx = n_elements(xin) 
      npy = n_elements(yin)
      npl = n_elements(locxin)
      if npx eq 0 or npy eq 0  then begin
        Print, 'X and Y conductors must be specified.'
        return
      endif
      if npx ne npy then begin
        Print, 'X and Y conductors must have the same number of elements'
        return
      endif
      if npl ne 0 then begin
        if max(locxin) gt npx+1 then begin
          print, 'LOCX array must not specify values outside the range of X, Y'
          return
        endif
      endif else begin
        nnew = 1
        locxin=[0L, npx]
      endelse
      rm_dupes, xin,yin, locxin
      npx = n_elements(xin) 
      npy = n_elements(yin)
      npl = n_elements(locxin)
      nnew = npl-1
      if npl lt 2 then begin
        print,'Single points cannot be used'
        return
      endif
      nok = 0
      if n_elements(replace) gt 0 then begin
        ok  = where (replace gt 0 and replace le ncon, nok)
        if nok ne nnew then begin
        print,'Number of replace arrays not equal to the number input',nok,nnew
          return
        endif
        replace = replace (ok)
      endif else replace = ncon+ lindgen(nnew)+1
;
      if ncon gt 0 then begin
        cx = cond.x
        cy = cond.y
        locx = cond.locx
        lcfit = cond.lcfit
        ix = cond.ix
        iy = cond.iy
        loci = cond.loci
        locp = cond.locp
        ipx = cond.iplotx
        ipy = cond.iploty
        nbad = cond.nbad
        badxy = cond.badxy
        locb  = cond.locb
        ixx = 0L
        iyy = 0L
        if n_elements (lcfit) lt ncon + 1 then $
            lcfit = make_array(ncon+1, /int, value = lcfit(0))
        if n_elements (nbad) lt ncon + 1 then $
          nbad = make_array(ncon+1, /int, value = nbad(0))
      endif else begin
        locx = 0L
        loci = 0L
        locp = 0L
        locb = 0L
        lcfit = 0L
        nbad = 0L
        ix = 0L
        iy = 0L
        ipx = 0L
        ipy = 0L
        badxy= 0L
      endelse
;
        if nok ne nnew then begin
          locx = [locx, make_array(nnew, /int, value = locx(ncon))]
          loci = [loci, make_array(nnew, /int, value = loci(ncon))]
          locp = [locp, make_array(nnew, /int, value = locp(ncon))]
          locb = [locb, make_array(nnew, /int, value = locb(ncon))]
          lcfit = [lcfit , make_array(nnew, /int, value = 0)]
          nbad = [nbad , make_array(nnew, /int, value = 0)]
          ncon = ncon+nnew
        endif
        for i = 1, nnew do begin
          ixx = float(xin(locxin(i-1): locxin(i)-1))
          iyy = float(yin(locxin(i-1): locxin(i)-1))
          np= n_elements(ixx)
          array2pack, replace(i-1), ncon, locx, np, cx, ixx, cy, iyy
;
          array2pack,/keep, replace(i-1), ncon, loci, 0, ix, ixx, iy, iyy
          array2pack,/keep, replace(i-1), ncon, locp, 0, ipx, ixx, ipy, iyy
          array1pack,/keep, replace(i-1), ncon, locb, 0, badxy, ixx
        endfor
;
        cond = {ncon:ncon, lcfit:lcfit, x:cx, y:cy, locx:locx, $
             ix:ix, iy:iy, loci:loci, iplotx:ipx, iploty:ipy, locp:locp, $
             badxy:badxy, locb:locb, nbad:nbad }
        return
        end
