; ------------------------------------------------------------------------ 
; $Id: tqgrid.pro,v 1.4 2007/06/06 20:12:26 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
 Pro tqgrid, xin, dxin, gxout, gdxin, inches = inches, mils = mils, $
     cm = cm, multiplier = multip, xaxis = xaxis, yaxis = yaxis, quiet = quiet
;
;+
; format:  tqgrid, x, dx, gx, gdx, inches = inches, mils = mils, cm = cm, $
;  multiplier = multip, xaxis = xaxis, yaxis = yaxis, quiet = quiet
;
; This procedure generates a grid with a continuous first derivative using
; a quadratic generating function.
; X is an array of grid locations which are used to specify the grid interval.
; 
; DX is an array of grid intervals corresponding to X locations.
;     If dx is omitted then gx = x
; GX  is the actual grid
; GDX is the actual grid spacing.
; QUIET - If set, no plots will be drawn
; 
; For a continous first derivative, both X and DX cannot be specified exactly
; everywhere.
; Procedure:
; 1.  All values of x < x(0) are deleted with the corresponding dx values
; 2.  X values are sorted.
; 3.  If all DX are positive: 
;         Start with smallest x and generate grid using an integer number of 
;         grids points and a best effort to match the specified DX values.
;     If one or more negative DX values, then:
;         Use the first negative DX value and the corresponding X value to 
;           generate a grid toward lower values.  In this generation, the 
;           end points are exact, and a best effort to match the specified
;           DX values is made.
;         Generate the grid toward larger values by using an integer number of 
;           grids points and a best effort to match the specified DX values if 
;           dx(i) is positive or vary the endpoint with interval given exactly
;           by the absolute value of dx(i) .
; Example;
;   generate a uniform y grid from 0 to 10 with spacing of .05 and 
;     units of inches.
;     tqgrid, [0, 10],[0.05, 0.05], /inches, /y
;
;   generate a grid which varies from .1 to .01 to .1 in going from 
;     0 to 5 to 10 with units of cm 
;     tqgrid, [0, 5, 10], [.1, .01, .1], /cm, /x 
;
;   generate a grid which has a dx of exactly 0.001 at 4 and is 0.01 near 10
;   and is about 0.01 at 0 for the y axis with units of mils
;     tqgrid, [0, 4, 10], [0.01, -0.001, -0.01], /mils, /y
;    
;-
  on_error, 1
@pfidltq.com
;
@tq_qs_gridparam.com
;common tq_qs_gridparam, grid_param_1, grid_param_2, grid_param_3
;
; Digress monemtarily to set the multiplier
  if KEYWORD_SET(inches) then mult= 0.0254
  if KEYWORD_SET(mils) then mult= 0.0000254
  if KEYWORD_SET(cm) then mult= 0.01
  if KEYWORD_SET(multip) then mult= float(abs(multip))
  if n_elements(mult) eq 0 then mult = 1.0
; see if an axis is specified
;
  if n_elements (yaxis) eq 0 then yaxis = 0
  if n_elements (xaxis) eq 0 then xaxis = 0
  axis = 0
  if yaxis ne 0 then axis = 2
  if xaxis ne 0 then axis = axis +1
  if axis eq 0 or axis eq 3 then begin
    print, '% TQGRID: X or Y axis must be specified'
    return
  endif
;
; if no valid input data then use old data if it exists
;
if n_elements(xin) gt 2 and n_elements(dxin) eq 0 then begin
  ; sort the data
  ok = sort(xin)
  xin = xin(ok)
  gxout = xin
  np = n_elements(xin)
  gdxin = xin[1:np-1]-xin(0:np-2)
  dxin = gdxin
  lcnooutput = 1
endif else begin
  lcnooutput = 0
  if n_elements(xin) lt 2 then begin
    if axis eq 1 then begin
      if n_elements(x) gt 2 then begin
        xin = x
        dxin = dx
        gxout = gx
      endif else begin
        print, '% TQGRID: No grid data for this axis.'
        return
      endelse
    endif else begin
      if n_elements(y) gt 2 then begin
        xin = y
        dxin = dy
        gxout = gy
      endif else begin
        print, '% TQGRID: No grid data for this axis.'
        return
      endelse
    endelse
  endif
; must have the same number of elements in x and dx
  if n_elements (dxin) ne n_elements(xin) then $
    message, 'X and DX must have the same size'
;
  if min(abs(dxin)) eq 0 then message, $
       'Delta for the axis must be greater than zero'
;
; see if any points occur before the first point
  ok = where (xin ge xin(0), nok)
  if nok lt 2 then message, 'No valid input arrays'
  xin = float(xin(ok))
  dxin = float(dxin(ok))
;
; remove all markers
  rm_markers
;
; sort the data
  ok = sort(xin)
  xin = xin(ok)
  dxin = dxin(ok)
;
; check for points too close together
  dd = xin(1:nok-1)-xin(0:nok-2)
  adxin = abs(dxin)
  dxave  = (adxin(1:nok-1)+adxin(0:nok-2))*0.5
; we require at least 2*dxave between x values
  ok = where (dd ge 2*dxave, noks)
  if noks lt 1 then message, 'Points must be spaced by at least the grid size'
  if nok ne noks+1 then begin
;   if points are too close together then keep the first and last points
    ok = 1+ok
    ok(noks-1) = nok-1
    xin = [xin(0), xin(ok)]
    dxin = [dxin(0), dxin(ok)]
  endif
;
; 
  nx = noks + 1
  
  temp = make_array(2*nx + 4, /string)
  tempqs = make_array(nx, /string)
  temp_a = make_array(nx-1, /double)
  temp_b = make_array(nx-1, /double)
  temp_x0 = make_array(nx-1, /double)
  temp_nx = make_array(nx-1, /long)

  tempqs(0)= '#  QUICKSILVER GRID'
  temp(0)='#'
  temp(1)='# Grid generated using the following grid locations and grid spacing'
  temp(3)='#'+ string(xin(0))+ string (dxin(0))
  if axis eq 1 then begin
    print, 'x = ', xin
    print, 'dx = ', dxin
    straxis = '  i  '
  endif else begin
    print, 'y = ', xin
    print, 'dy = ', dxin
    straxis = '  j  '
  endelse
  temp(3+nx) = '#'
  print, '     initial-X       final-X  Number  Delta X0      Delta XN'
  endline = '  &'
; find the start location
  ok = where (dxin lt 0 , nok)
  if nok eq 0 then ok = 0 
  iup = ok(0)
  np = 0
  if ok(0) gt 0 then begin
;   set the initial grid points 
    gxout = [xin(iup)+ dxin(iup), xin(iup)]
    np = 2
    gdxin = abs(dxin(iup))
    for i = iup -1, 0, -1 do  begin
      x0 = xin(i)
      xf = gxout(1)
      dist = xf -x0
      d0 = dxin(i)
      df = gdxin(0)
      n = round(2.*dist/(d0+df))
      d0=dist*2.0/n-df
      B = (df-dist/n)/(n-1)
      A = d0-B
      ind= findgen(n-1)
      xtemp = (b*ind + a)*ind + x0
      ind = indgen(n-1)*2+1
      dtemp = ind*B+A
      if i eq iup-1 then ngr = n else begin
        ngr = n-1
        dist = dist -gdxin(0)
      endelse
      np = np +n-1
      gxout = [xtemp, gxout]
      gdxin = [dtemp, gdxin]
      temp(3+i+1) = '#'+ string(xin(i+1))+ string (dxin(i+1))
      print,  xin(i),xin(i+1),  fix(ngr), d0, df
      if i eq nx-2 then endline = ' ' else endline = '  &'
      temp(nx+4+i+1)= string(ngr) +  string(dtemp(0)*mult) + $
        string(dist*mult)+ endline
      if i lt nx-2 then nqs = n-1 else nqs = n
      tempqs (i+1) = ' GRID   1  '+straxis+ string(x0*mult) +string(nqs) + $
         string(A*mult) + string(B*mult)
      temp_a[i] = a*mult
      temp_b[i] = b*mult
      temp_x0[i] = x0*mult
      temp_nx[i] = nqs

;     print, temp(nx+5)
    endfor
  endif
  maxgx = 2048
  tempgx = make_array (maxgx)
  if np gt maxgx then begin
    maxgx = maxgx + maxgx
    tempgx = [tempgx, tempgx]
  endif
  if np gt 0 then begin
    gxout = [gxout(0:np-1), tempgx (0: maxgx-np)] 
    gdxin = [gdxin(0:np-2), tempgx (0: maxgx+1-np)]
  endif else begin
    gxout = tempgx
    gdxin = tempgx
  endelse
;
; now go in the positive direction
;
  if iup eq 0 then begin
    gxout(0:1)=[xin(0), xin(0)+abs(dxin(0))]
    gdxin(0) = abs(dxin(0))
    np = 2
  endif
;
  if iup lt nx -1 then begin
    for i = iup+1, nx-1 do begin
;
    x0 = gxout(np-2)
    xf = xin(i)
    dist = xf-x0
    d0 =  gdxin(np-2)
    df = abs(dxin(i))
    n = round(2.*dist/(d0+df))
    if dxin(i) lt 0 then $
      xf = n*(d0+df)/2.0 + x0 $
    else $
      df = dist*2./n-d0
    B = (df-d0)/(2*(n-1))
    A = d0-B
    if i eq nx-1 then endline = ' '
    ind = findgen(n-1)+2.0
;
    xtemp = (b*ind + a)*ind 
    ind = (indgen(n-1)+1)*2+1
    dtemp = ind*B+A
    if i eq 1 then begin
      ngr = n
      del = d0
    endif else begin
      ngr = n-1
      del = a + 3.0*b
      dist = xtemp(n-2) - d0
    endelse
    xtemp = xtemp+x0
    temp(3+i) = '#'+ string(xin(i))+ string (dxin(i))
    temp(nx+4+i)= string(ngr) +  string(del*mult) + string(dist*mult)+ endline
    if i eq nx-1 then nqs = n else nqs = n-1
    tempqs (i) = ' GRID   1  '+straxis+ string(x0*mult) +string(nqs) + $
         string(A*mult) + string(B*mult)
    temp_a[i-1] = a*mult
    temp_b[i-1] = b*mult
    temp_x0[i-1] = x0*mult
    temp_nx[i-1] = nqs
    print,  xin(i-1),xin(i),  fix(ngr), d0, df
;    print, temp(nx+5)
    np = np +n -1 
    if np gt maxgx then begin
      maxgx = 2*maxgx
      gxout = [gxout, gxout]
      gdxin = [gdxin, gdxin]
    endif
    gxout(0:np-1) = [gxout(0:np-n), xtemp]
    gdxin(0:np-2) = [gdxin(0:np-n-1), dtemp]
  endfor
  endif
  gxout = gxout(0:np-1)
  gdxin = gdxin(0:np-2)
endelse
  i2w, indgen (np), gxout, 1, clab = 'Grid'
  i2w, indgen (np-1), gdxin, 2, clab = 'Delta X'
;
  print,' Total Number of Points = ',strtrim( np ,2)
  if axis eq 1 then begin
    x = xin
    dx = dxin
    gx = gxout
    gdx = gdxin
    i2w, findgen(np), gx, 71, xlab = 'X Axis Index' , ylab = 'Position', $
      clab = 'X Position vs Index'
    dif, 71, 72, clab = 'First Derivative of X-Grid vs Index', y='Delta'
    dif, 72, 73, clab = 'Second Derivative of X-Grid vs Index', y='Delta'
    div , 73,72, 74, clab='Second Deriv / First Deriv vs Index'
    if not keyword_set(quiet) then plo, 71, 4, g = 4, color = 0, over = 0
    if lcnooutput eq 1 then return
    temp (nx+4)= '  x1grid   1 '+ string(fix(np+1))+'   2 '+string(x(0)*mult)+'   &'
    temp(2)='#    Initial-X      Delta-X  (Spatial Multiplier = '+strtrim (mult, 2)+')'
    x1grid = temp
    qsxgrid = tempqs
    grid_param_1 ={A: temp_a, B: temp_b, x0: temp_x0, n: temp_nx}
    
  endif else begin
    y = xin
    dy = dxin
    gy = gxout
    gdy = gdxin
    i2w, findgen(np), gy, 75, xlab = 'Y Axis Index' , ylab = 'Position', $ 
       clab = 'Y Position vs Index'
    dif, 75, 76, clab = 'First Derivative of Y-Grid vs Index', y='Delta'
    dif, 76, 77, clab = 'Second Derivative of Y-Grid vs Index', y='Delta'
    div , 77,76, 78, clab='Second Deriv / First Deriv vs Index'  
    if not keyword_set(quiet) then plo, 75, 4, g = 4,color = 0, over = 0
    if lcnooutput eq 1 then return
    temp (nx+4)= '  x2grid   1 '+ string(fix(np+1))+'   2 '+string(y(0)*mult)+'   &'
    temp(2)='#    Initial-Y      Delta-Y  (Spatial Multiplier = '+strtrim (mult, 2)+')'
    x2grid = temp
    qsygrid = tempqs
    grid_param_2 ={A: temp_a, B: temp_b, x0: temp_x0, N: temp_nx}
  endelse
  if n_elements(cond) gt 0 then cond.lcfit = 0*cond.lcfit
;print, 'begin'
;for i = 0, 2*nx+3 do print,i, temp(i) 
;print, 'end'
  return
  end
