; ------------------------------------------------------------------------ 
; $Id: tqpotential.pro,v 1.4 2010/09/21 17:49:10 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
 Pro tqpotential, es, phis,ces, niteration = niter, file = file, $
  ei = ei, ej = ej, phi = phi, xgrid = xi, ygrid = xj, charge = charge, $
  xcharge = xcharge, ycharge = ycharge, dielectric = dielectric, $
  xdielectric = xdielectric, ydielectric = ydielectric, cim = cim, $
  qarr = g, epsi = er, potential = pot, energy = energy, edensity=edensity, $
  cmissing = cmissing,  batch_mode = batch_mode, maxiteration = maxiteration,  $
  err_convergence = convergence, rdielectric = rdielectric, bpotential = bpot, $
  boundary = boundary, conductors = conductors, cvalue = cvalue,  $
  rerun = rerun, coordinate= coordinate , quiet = quiet

;
;+
;   Purpose: Calculate the 2-D electrostatic field for a distribution
;            of conductors, dielectrics, and charges. 
;   Format:  tqpotential [, Es] [, Phis] [, CEs] [, niteration = niter]
;                      [, file = file] [, ei = ei] [, ej = ej]
;                      [, phi = phi] [, xgrid = xi] [, ygrid = xj] 
;                      [, edensity = edensity] [, energy = energy]
;                      [, charge = charge] [, xcharge = xcharge]
;                      [, ycharge = ycharge] [, dielectric = dielectric]
;                      [, xdielectric = xdielectric]
;                      [, ydielectric = ydielectric] [, cim = cim]
;                      [, qarr = qarr] [, epsi = epsi] [, potential = pot]
;                      [, err_convergence = convergence] [, maxiteration = maxiteration]
;                      [, boundary=boundary] [, rerun = rerun] [, quiet = quiet]
;    BATCH MODE        [, /batch_ mode] [, cmissing = cmissing] 
;                      [, rdielectric = rdielectric] [, coordinate = coordinate]
;                      [, conductors = conductors] [, Cvalue = cvalue ]
;                      [, boundary=boundary] [, bpotential = bpot]
;   NOTE: Conductors, dielectrics and regions of uniform charge must 
;         must be closed on closed surfaces and fit to a grid with all
;         bad points removed.  
; Exception: Two point conductors are allowed.
;
;   Note: Conductors are grid based; conductor values go to grid points.
;         Slant surfaces are not allowed for dielectrics! Dielectrics are 
;         based cell based and depend on the contents of cell volume; 
;         a diagonal indicates neither an empty or full cell.
;         Note: If conductor 4 is a dielectric and the other conductors have
;               been fit with TQFIT, uset "QSFIT, 4" to refit the dielectric.
;
;   Note: QSFIT produces the same potential distribution as TQFIT since the 
;         the nodes are the same with the two fit procedures.
;
;   NOTE:  To integrate fields to obtain potentials, use the cell centered
;          Electric fields and the waveform INT routine to obtain accurate
;          values (KEEP =0).  The routines "plotstr, /yint" will produce small
;          errors if conductors do not lie on the boundary.
;
;   where:
; The calculated potential and electric fields are stored in the 3 locations 
;   designated by Es, Phis, and CEs.
; Assume a grid with nx and ny full grid values
;        Es     Structure array number for the two components of the 
;               electric field, Default = Maxstructure-2 (80) (size = nx by ny)
;        Phis   Structure array number for the potential,
;               Default = Maxstructure-1  (81) (size = nx by ny)
;        CEs    Structure array number for the cell centered Electic fields
;               Default = Maxstructure-3  (79)  (size = (nx-1) by (ny-1))
;               Note: Integral of a lineout of the CEs should be -Potential
;        Niter  Number of iterations required for convergence
;        File   If specified, output will be sent to a PFF file
;        Ei     Electric field matrix in the i direction
;               This is the first component of structure Es
;        Ej     Electric field matrix in the j direction
;               This is the second component of structure Es
;        NOTE:  Ei and Ej cannot be integrated exactly to produce the potential,
;               PHI, because of the discontinuous nature of the field
;        PHI    Potential matrix (size = nx by ny)
;        Xi     Coordinates for the i direction, X values for Ei, Ej, PHI
;               If RERUN < 1 then Xi are the full grid values for the CIM.
;               Values returned will be different if system is left handed.
;        Xj     Coordinates for the j direction, Y values for Ei, Ej, PHI
;               If RERUN < 1 then Xj are the full grid values for the CIM.
;               Values returned will be different if system is left handed.
;        EDensity (Ei^2 + Ej^2)*epsi*8.8541878e-12/2
;               (size = (nx-1) by (ny-1))
;        Energy Total (edensity*dx*dy) for system = 1
;               Total (edensity*y*dx*dy) for system = 2
;               Total (edensity*x*dx*dy) for system =  5
;               Total (edensity*x^2*dx*sin(y)dy) for system =  5
;               Total (edensity*radius^2*sin(x)*dx*dy) for system = 4
;               Total (edensity*sin(theta)*dx*y^2*dy) for system = 6
;               Note: Future versions will handle different coordinates.
;       *Charge, Xcharge, Ycharge 
;               Charge can be a structure array, array number or a 2D array
;               If a 2D array, then X/Ycharge specify the coordinates
;               These parameters allow for non-uniform charge distributions.
;               Units should be Coul/m^3, X and Y units are the units of the 
;               grid.(cm, inches, ...) Default = 0.0
;       *Dielectric, Xdielectric, Ydielectric
;               Dielectric can be a structure array, array number or a 2D array
;               If a 2D array, then X/Ydielectric specify the coordinates
;               These parameters allow for non-uniform dielectric distributions.
;               Units should be Coul/m^2, X and Y units are the units of the 
;               grid.(cm, inches, ...) Default value = epsilon default.
;        *NOTE: If a structure array is passed, tags v1 will be used as the 
;               quantity and the x and y values will be x1 and x2 unless
;               x and y are specified.
;               For example: charge= 3, xcharge = 3, ycharge = 1 will
;                use structure array 3 as a z-x plot or a z-r plot.
;        CIM    Conductor matrix. The CIM used for the calculation is returned.
;               If RERUN < 0 then the input CIM will be used for all conductors.
;               Values should be integers 1 to N.
;               POT(0:N-1) contains the voltages on these conductors.
;               A CIM must be input with values of Xi and Xj corresponding to
;               the full grid values in the CIM for rerun < 0.
;               IF CIM is not a LONG array it will be changed to a long array.
;        Qarr   Charge matrix.  The charge matrix used for the calculation will
;               be returned.
;               If RERUN < 1 then if CIM and QARR are the same size then QARR
;               will be used as the charge array.
;        epsi   Relative dielectric constant matrix
;               If RERUN < 1 then if CIM and EPSI are the same size then EPSI
;               will be used as the dielectric array.
;        pot    Potential values, Values of the voltage on each conductor
;     MaxIteration:  Maximum number of iterations. Must be > 10
;               Default: MaxIteration = 200
;               If MAXITERATION is set, the default becomes the set value.
;     Err_Convergence: Convergence Criteria for P2D.
;                Must be positive and less than 0.1
;                Note: P2D is performed in Double Precision but iteration is
;                      on the potential. Electric fields are calculated from the
;                      potentials and will have larger errors.
;               Default: Err_Convergence = 3.0e-11 
;               If ERR_CONVERGENCE is set, the default becomes the set value.
;        RERUN  If positive, the previous values for the boundary conditions, 
;               conductors, charge, and dielectric will be used as the default
;               If the keywords: cim, qarr, or epsi contain an array then their
;               contents will be used in the rerun. 
;        NOTE:  If BATCH_MODE is set then RERUN=0
;        NOTE:  The potentials, number of conductors, and boundary conditions
;               may not be changed for rerun > 0.
;               If negative, a CIM, QARR and EPSI array will be used for input.
;        Note:  If coordinates are Left-handed Xi and Xj will be interchanged
;               on return.  Full grid values will be passed in with Xi and Xj.
;               If rerun is less than 0 then a ATHETA calculation is expected.
;               If EPSI and QARR are single elements then a constant charge
;                 and dielectric background is expected.
;                 Default values are EPSI = 1, QARR= 0
;               ATHETA requires a second keyword: "boundary"
;
;     BATCH_MODE: If set, all input will be from the command line.
;               This is similar to ReRun but does not require any of the above
;               parameters.  
;     COORDINATE: Coordinate system for the solution
;               1 Cartesion x-y             -1 Cartesion y-x
;               2 Cartesion y-z             -2 Cartesion z-y
;               3 Cartesion z-x             -3 Cartesion x-z
;               4 Cylindrical z-r           -4 Cylindrical r-z    
;               5 Cylindrical r-phi         -4 Cylindrical phi-r    
;               6 Spherical theta-phi       -6 Spherical phi-theta
;               7 Spherical r-theta         -7 Spherical theta-r
;               8 Spherical phi-r           -8 Spherical r-phi
;                IF RERUN is set: (No Default)
;                     If Coordinate is not specified, a widget will get system.
;                If BATCH_Mode is set: Default: Coordinate =1
;     CMissing - Value of the missing coordinate.  TQPOTENTIAL calculates a 
;                Two dimensional potential.
;                Default - CMissing = 0.0 if Abs(coordinate) not 6 
;                          CMissing = 1.0 if Abs(coordinate) = 6
;     RDielectric: Default relative dielectric.  Dielectric matrix will be 
;                 filled with this value.  Dielectric regions defined in TQEDIT
;                 will replace the default values.
;     Conductors: An array of integers with at least as many elements as 
;                 the number of conductors in tqedit.
;                 (The user can determine the correspondence between numbers
;                  and conductors in TQEDIT by toggling the conductors.
;                  Note: TQEDIT conductors begin with 1;
;                        IDL arrays begin with 0.)
;                 Values must be 0, 1, 2, or 3.
;                 Conductor[i] = 0 - Ignore the conductor
;                              = 1 - Metalic conductor
;                              = 2 - Dielectric region
;                              = 3 - Charge region
;     CValue:     An array of floats with at least as many elements as 
;                 the number of conductors in tqedit.
;                 Value depends on the value of the conductor.
;                 If Conductor[i] = 0 then
;                    CValue[i] is ignored
;                 If Conductor[i] = 1 then
;                    CValue[i] is the Conductor Voltage (Volts)
;                 If Conductor[i] = 2 then
;                    CValue[i] is the Relative dielectric constant
;                 If Conductor[i] = 3 then
;                    CValue[i] is the Charge Density (Coulomb/(m*m))
;     BOUNDARY:  4 element array with the boundary conditions
;                Valid values are:
;                   0 is Periodic
;                   1 is Mirror Boundary
;                   2 is Conducting: This is ignored, unless batch mode.
;                        User must fill boundary cim values with conductor
;                        and set the POT value to the potential for rerun.
;                        This may be set in batch mode
;                The order of the values is:
;                  [Left (min X), Right (Max X), Bottom (Min Y), Top (Max Y)]
;                Any values not 0 or 1 will be set to 2
;                If boundary(0) is 0 then boundary(1) is ignored.
;                If boundary(2) is 0 then boundary(3) is ignored.
;     BPOT:      If boundary[i] eq 2, then BPOT[i] is the boundary potential
;                Ignored unless boundary[i] eq 2.
;                BPOT must have 1 or 4 values.
;                DEFAULT: BPOT = [0,0,0,0] if BPOT has 1 value then 
;                         BPOT = make_array(4, value = float(bpot[0]))
;        quiet  May only be set with rerun > 0.  If present and set, then 
;               informational text will not be printed
;
;   example: 
;        The following steps can be followed to calculate a potential
;        1.  Generate a set of conductor surfaces, dielectric surfaces, or 
;            charge distributions; for example, 
;            tqcond, x, y, loc
;        2.  If required, edit geometry and close all surfaces.
;            tqedit, /cond
;        3.  Generate a grid
;            tqgrid, [0, 100], [.2, .2], /x, /cm
;            tqgrid, [0, 100], [.2, .2], /y
;        4.  Fit the surfaces to the grid and close conductors 
;            tqfit, /close
;        5.  Modify fit values to remove bad points or improve fidelity.
;            tqedit
;        6.  Calculate 2D fields
;            tqpotential
;        7.  Plot results
;            plotstr, 80, [1,2] ; for magnitude of the field
;            plotstr, 81        ; for the potential profile
;        8.  Convert conductors to conductor structures in array, 1
;            TQ2S, 1, /all
;        9.  Overplot conductor
;            PLOTstr, 1, /over, col = 5
;      Alternatively:
;        1.  Obtain a conductor surface and grid from a previous calculation:
;               READSTR, 1, 3, 5 ; where conductors are in 3, 4, 5, 6, 7,8
;               TQREADCON, 1, 3, 1, .1 ; where the third and first dimension
;                                      ; are X and Y and the slice is at 0.1 
;                                      ; in the second dimension
;               READSTR,2,2       ; read the grid from the same file
;               TQREADGRD, 2, 3, 1, .1 ; put the grid into the TQ calculation
;               QSFIT
;               TQEDIT                 ; confirm geometry
;               TQPOTENTIAL
;       Another example:
;
;              xi = findgen(10)/9
;              xj = xi/100
;              cim = make_array(10, 10, /long)
;              cim(*, 0)=1
;              cim(*, 9)=2
;              pot = [0, 1]
;              boundary = [1, 1, 2, 2]
;              tqpotential, 1, 2, 3, cim = cim, xgrid = xi, ygrid = xj, $
;              potential = pot, coordinate = 1, boundary= boundary, rerun = -1
;-
@pfidltq.com
;    common pfidltq, x, dx, y, dy, gx, gy, xinsav, yinsav,zinsav, mult, $
;       gdx, gdy, x1grid, x2grid, x3grid, qsxgrid, qsygrid,  qszgrid, $
;       gxhalf, gyhalf, gzhalf, gxfull, gyfull, gzfull, xyzmult, blocklimits, $
;       xgridtype, ygridtype, nxbin, nybin, $
;       cond, markx, marky, markerstring, lcfit, tqsystem, tqslice, syst_list
;
      common pffcom, initpff, pff_exe

    common tqpotent_com, cim_save, qarr_save, epsi_save, bound_top, pot_top, $
         bound_side, pot_side, pot_save, ncond_save, xi_save, xj_save,  $ 
         conv_save, itrmax_save

    on_error, 1
;
;     check to see if pff has been initialized
;
      if n_elements(initpff) eq 0 then init_pff else $
        if (initpff eq 0) then init_pff
      if initpff eq 0 then begin
        print, '% TQPOTENTIAL: P2D is not supported by this Operating System'
        return
      endif
      if n_elements(itrmax_save) eq 0 then begin
        itrmax_save = 200L
        conv_save = 3.0e-11
      endif
      if (n_elements(maxiteration) gt 0) && (maxiteration[0] gt 10L) then $
        itrmax_save = long(maxiteration[0])
      if (n_elements(convergence) gt 0) && (convergence[0] gt 0) && $
        (convergence[0] lt 0.1) then conv_save = float(convergence[0])
      conv = conv_save
      itrmax = itrmax_save

;   see if conductors are defined
    if keyword_set(batch_mode) then rerun = 0 else $
      if n_elements(rerun) eq 0 then rerun = 0
;
    if keyword_set (rerun) then begin 
      nx = n_elements(xi)
      ny = n_elements(xj)
      if rerun lt 0 then begin
        if nx lt 2 or ny lt 2 then ss = 0 else ss = size (cim)
        if ss(0) eq 2 then begin
          if ss(1) ne nx or ss(2) ne ny then  ss = 0
        endif else ss = 0
        if ss(0) eq 0 then begin
          Print,'% TQPOTENTIAL: X, Y, and CIM must be passed with good values.'
          return
        endif
        if ss(3) ne 3 then cim = long(cim)
      endif else begin
        if (n_elements(xi_save) eq 0) || (n_elements(xj_save) eq 0) then begin
          print, '% TQPOTENTIAL: Coordinates not saved.  RERUN is not possible'
          return
        endif
        if nx eq 0 then xi = xi_save
        if ny eq 0 then xj = xj_save
      endelse
      xxx = xi
      yyy = xj
    endif else begin
      if n_elements(cond) eq 0 then begin
         Print, '% TQPOTENTIAL: No conductors defined' 
         nct=0
      endif else begin
        nct = cond.ncon
        if n_elements (gx) lt 2 or n_elements (gy) lt 2 then begin
            wwarning, '  Use TQGRID to generate a grid   '
            return
        endif
        if n_elements(cond.iy) lt 2 or n_elements(cond.ix) lt 2 then begin
            wwarning, '  Use TQFIT to fit data to the grid   '
            return
        endif
        if total(cond.lcfit) ne n_elements(cond.lcfit) then begin
            wwarning, ' Use TQFIT to fit data to the grid   '
            return
        endif
        get_tqbad
        if cond.nbad(0) ne 0 then begin
          print, ' Bad points must be removed before calling TQFILL.'
          return
        endif
      endelse
      xxx = gx
      yyy = gy
      xran = [min(gx, max = mxx), mxx]
      yran = [min(gy, max = myy), myy]
    endelse
    nx = n_elements(xxx)
    ny = n_elements(yyy)
    if rerun gt 0 then begin
      if n_elements (cim_save) eq 0 then begin
        print, '% TQPOTENTIAL: No CIM data has been saved. - End TQ Potential'
        return
      endif
      if n_elements(cim) eq 0 then cim = cim_save
      ntotal = nx*ny
      if n_elements (cim) ne ntotal then cim = temporary(cim_save)
      if n_elements(cim) ne ntotal then begin
        print, '% TQPOTENTIAL: CIM size is not consistent with Xi,Xj'
        return
      endif 
      if n_elements (g) ne ntotal then g = temporary(qarr_save)
      nel = n_elements (er)
      if nel  gt 0 then begin
        if nel ne 1  and nel ne ntotal then er = temporary(epsi_save)
      endif else er  = temporary(epsi_save)
      pot = pot_save
      ncond = ncond_save
      xvalue = (n_elements(tqslice) gt 0) ? tqslice : 0.0
      system = long (abs(tqsystem)-1)
      if system le 2 then system = 1L else system = system -1
;     Note TQSYSTEM has 3 cartesian options
;          P2D has only one; need to modify system to get is in sync with P2D.
    endif else if rerun lt 0 then begin
      if n_elements(coordinate) eq 0 then coordinate = 1
      tqsystem = round(coordinate)
      system = long(abs(tqsystem)-1L)
      if system le 2 then system = 1L else system = system -1L
      if n_elements(cmissing) eq 0 then begin
        if system eq 4 then cmissing = 1.0 else cmissing = 0.0
      endif
      erdef = 1.000
      mult = 1.000
    endif else begin
;   
      if nct gt 0 then begin
        location = cond.loci
        nix = cond.ix
        niy = cond.iy
      endif
      erdef = 1.000
      if keyword_set(batch_mode) then begin
        if n_elements(coordinate) eq 0 then tqsystem =1L else begin
          tqsystem = long(coordinate)
          system = long (abs(tqsystem)-1)
          if (system lt 0) or (system gt 7) then begin
            print, '% TQPOTENTIAL: Bad coordinate system flag.', tqsystem
            print, '          Coordinate system set to X-Y Cartesian'
            tqsystem = 1L
            system = 0L
          endif
        endelse
        system = long (abs(tqsystem)-1)
        if (n_elements(cmissing) gt 0) then tqslice = cmissing else begin
          if system eq 5 then tqslice = 1.0 else tqslice = 0.0
        endelse
        if n_elements(rdielectric) gt 0 then erdef = abs(rdielectric)
      endif else begin             
        if n_elements (tqsystem) eq 0 or n_elements(tqslice) eq 0 then $
           tqset_system
        system = long (abs(tqsystem)-1)
        if tqsystem gt 0 then direction = 'RIGHT' else direction = 'LEFT'
        message = ['Coordinate system is', syst_list(system), $
               'System is '+direction+' handed']
        get_ics, lcquit, itrmax, conv, erdef, offset, message
;
        if lcquit ne 0 then return
      endelse
      xvalue = tqslice
      if system le 2 then system = 1L else system = system -1
    endelse
;   mult is the coordinate system multiplier
    if system eq 4 then begin
      xv2 = xvalue*xvalue
      xmult = xv2*mult
      ymult = xmult
    endif else if system eq 6 then begin
      sintheta = sin(xvalue)
      xmult = mult*sintheta
      ymult = mult
    endif else begin
      xmult = mult
      ymult = mult
    endelse
    if tqsystem lt 0 then begin
      temp = temporary(xmult) 
      xmult = temporary(ymult)
      ymult = temporary(temp)
    endif
    if not keyword_set(rerun) then begin
      ncond = 0L
      pot = 0.0
      cim = make_array(nx, ny, /long)
      g =  make_array(nx, ny)
      er =  make_array(nx, ny, value = erdef)
      if n_elements(dielectric) gt 0 then $
        fill_array , er,xxx, yyy, dielectric, xdielectric, ydielectric, $
        default = erdef, /dielectric
      if n_elements(charge) gt 0 then $
         fill_array , g,xxx, yyy, charge, xcharge, ycharge, $
          default = 0.0
      lcfoil = 0L
      okfoil = 0L
      temp =  make_array(nx, ny, /int)
      if keyword_set (batch_mode) and nct gt 0 then begin
        if nct gt n_elements(conductors) then begin
           print, '% TQPOTENTIAL: Must have at least one entry in CONDUCTORS '
           print, '               for each conductor in TQEDIT.', nct, $
               n_elements(conductors)
           return
        endif
        tqconduct = conductors[0:nct-1]
        ok = where (tqconduct lt 0 or tqconduct gt 3, nok)
        if nok gt 0 then begin
          print, '% TQPOTENTIAL: Conductors must be in the range [0, 3].'
          return
        endif
        ncval = n_elements (cvalue)
        if ncval eq 0 then cvalue = 0.0
        if ncval le 1 then cvalue = make_array(nct, value=float(cvalue)) else $
          if ncval lt nct then begin
          print, '% TQPOTENTIAL: Value must be provided for each conductor'
          return
        endif
      endif else conductors = make_array(nct, value =1)
      for i = 0, nct-1 do begin
;       skip the conductor if batch mode wants to skip
        if keyword_set (batch_mode) and conductors[i] eq 0 then continue
        if i gt 0 then if nok gt 0 then temp(ok) = 0
        value = 1
        ix = nix(location(i): location(i+1)-1)
        iy = niy(location(i): location(i+1)-1)
        np = n_elements(ix)
        if not keyword_set(batch_mode) then begin
          plot, gx, gy, /nodata, /xst, /yst, xran= xran,yran= yran
          for j = 0, nct-1 do  oplot, gx(nix(location(j):location(j+1)-1)), $
             gy(niy(location(j):location(j+1)-1)), line = 2, color = 4
        endif
        if np eq 2 then begin
          ixmin = min(ix, max = ixmax)
          iymin = min(iy, max = iymax)
          if ixmin eq ixmax or iymin eq iymax then $
          temp(ixmin:ixmax, iymin:iymax) = value  else begin
            if (ix(1)-ix(0)) gt 0 then dx = 1 else dx = -1
            if (iy(1)-iy(0)) gt 0 then dy = 1 else dy = -1
            for i = 0, ixmax-ixmin do temp(ix(0)+dx*i, iy(0)+dy*i) = value
          endelse
          ok = where (temp ne 0, nok)
          if nok gt 0 then lcfoil = 1
          okfoil = [okfoil, ok]
          if not keyword_set(batch_mode) then $
            oplot, gx(ix), gy(iy), thick = 2.5, color = 7
        endif else begin
          if (ix(0) ne ix(np-1) or iy(0) ne iy(np-1)) then begin
            print, ' Conductors must be closed'
            array =0
            return
          endif
          ind2fill,value,ix, iy, np, temp, nx, ny 
          if n_elements(temp) lt 2 then return
          ok = where (temp ne 0, nok)
          if not keyword_set(batch_mode) then $
            contour, temp,xxx, yyy, lev = 0.95, /fill, /over
        endelse
        if keyword_set (batch_mode) then begin
          lcquit = 0
          ltype = conductors[i] - 1
          cond_value = cvalue[i]
        endif else get_cdq, lcquit, ltype, cond_value, offset
        if lcquit ne 0 then return
        if ltype eq 0 then begin
          ncond = ncond+1
          cim(ok)= ncond
          if ncond eq 1 then pot = float(cond_value) else pot= [pot, cond_value]
        endif else if ltype eq 1 then begin
          if np gt 2 then $
            diel2fill, cond_value, ix, iy, np, er, nx, ny else begin
            print, 'Dielectrics must have more than 2 points'
            return
          endelse
          if n_elements(er) lt 2 then return
;         er(ok) = cond_value
        endif else if ltype eq 2 then g(ok) = cond_value
      endfor
    endif else if rerun lt 1 then begin
      ner = n_elements(er)                                         
      if ner lt 2 then begin
        if ner eq 0 then er = make_array(nx, ny, value = 1.000) else $
        er = make_array(nx, ny, value = abs(float(er)))
      endif else if ner ne nx*ny then begin
        print, '% TQPOTENTIAL: Dielectric matrix must be the same size as CIM'
        return
      endif
      neg = n_elements(g)
      if neg  lt 2 then begin
        if neg eq 0 then g = make_array(nx, ny, value = 0.000) else $
        g = make_array(nx, ny, value = float(er))
      endif else if neg ne nx*ny then begin
        print, '% TQPOTENTIAL: Charge matrix must be the same size as CIM'
        return
      endif
      ncond = max(cim)
      pot = float (pot)
      if n_elements (pot) lt ncond then begin
        print,'% TQPOTENTIAL: POT array must have an element for each CIM value'
        return
      endif
      if n_elements(boundary) lt 4 then begin
        print, '% TQPOTENTIAL: BOUNDARY must have 4 values.'
        return
      endif
    endif 

    cim_save = cim
    xi_save = xxx
    xj_save = yyy
    qarr_save = g
    epsi_save = er
    pot_save = pot
    ncond_save = ncond
    bcm = 0L
    bcmb = [0L, 0L]
    bcme = bcmb
    imax = nx+1L
    jmax = ny+1L
    bgla = imax*jmax -1
    bgli = nx
    bglj = ny
    blkdim = 1L
    loc = 0L
    loci = 0L
    locj = 0L
    luout = 6L
    mwachs = 100L
    lwork = 28L*(bgla+1L)+10+mwachs
    work = make_array(lwork+1)
;   lwork = (14L*(bgla+1L)+6L+mwachs)
;   work = make_array(lwork, /double)
;   lwork = lwork*2
;
    mmer = max(er, min= mer)
    if (mmer- mer)/mmer  lt  5.0e-5  then begin
      bgle = 0L
      ere= (mer+mmer)*0.5
    endif else begin
      bgle = bgla
      ere = make_array(imax, jmax, value = erdef)
      ere(1:*, 1:*) = er
      ere(*, 0) = ere(*, 1)
      ere(*, ny) = ere(*, ny-1)
      ere(0, *) = ere( 1, *)
      ere(nx, *) = ere(nx-1 , *)
    endelse
    nblk = 1L
;   boundary
    nneu = 0L
    nper =0L
    perpdi = make_array(10, /long)
    neupdi = make_array(5, /long)
    if (rerun gt 0) then begin
        bound1 = bound_side(0)
        bound2 = bound_side(1)
        pot1 = pot_side(0)
        pot2 = pot_side(1)
    endif else  if rerun lt 0  then begin
        ok = where(boundary lt 0 or boundary gt 1, nok)
        if nok gt 0 then boundary(ok)=2 
        bound1 = boundary(0)
        bound2 = boundary(1)
    endif else  begin
      if keyword_set(batch_mode) then begin
        bound_side = boundary[0:1]
        bound1 = boundary(0)
        bound2 = boundary(1)
        pot_side = bpot[0:1]
        pot1 = pot_side[0]
        pot2 = pot_side[1]
      endif else begin
        get_boundary, lcquit, 1, bound1, bound2, pot1, pot2, offset
        if lcquit ne 0 then return
        bound_side= [bound1, bound2]
        pot_side = [pot1, pot2]
      endelse
    endelse
    bleft = bound1
    bright = bound2
    if bound1 eq 0 then begin
; periodic
      xi = [0.0, xmult*xxx]
      deli = [0.0,xi(2:nx)-xi(1:nx-1), 0.0]
      xi(0) = xi(1)-deli(nx-1)
      deli(0) = deli(nx-1)
      deli(nx) = deli(1)
      if (system lt 6  and tqsystem gt 0)or(system eq 6 and tqsystem lt 0) then $
        perpdi = [1L, ny,1L, +1L, 1, 1L, ny, nx, -1L, 1L] else $
        perpdi = [1L, ny,1L, +2L, 1, 1L, ny, nx, -2L, 1L] 
      nper = nper+1
      for j = 0, ny-1 do cim(0, j) = max([cim(0, j), cim(nx-1, j)])
      cim (nx-1, *) = cim(0, *)
      if n_elements(ere) gt 1 then begin
        ere(0, *)= ere(nx-1, *)     
        ere(nx, *) = ere(1, *)
      endif
    endif else begin
      xi = [0.0, xmult*xxx]
      deli = [0.0,xi(2:nx)-xi(1:nx-1), 0.0]
      xi(0)= xi(1)-deli(1)
      deli(0) = deli(1)
      deli(nx) = deli(nx-1)
      if bound1 eq 2 then begin
        if (rerun ge 0) then begin
          ncond= ncond+1
          cim(0, *) = ncond
          if ncond eq 1 then pot = float(pot1) else pot= [pot, pot1]
        endif
      endif else begin
        nneu = nneu + 1L
        if (system lt 6 and tqsystem gt 0)or(system eq 6 and tqsystem lt 0)then $
          neupdi = [1L, ny, 1L, +1L, 1L] else $
          neupdi = [1L, ny, 1L, +2L, 1L]
      endelse
      if bound2 eq 2 then begin 
        if rerun ge 0 then begin 
          ncond= ncond+1
          cim(nx-1, *) = ncond
          if ncond eq 1 then pot = float(pot2) else pot= [pot, pot2]
        endif
      endif else begin
        nneu = nneu + 1L
        if (system lt 6 and tqsystem gt 0)or(system eq 6 and tqsystem lt 0)then $
          temp = [1L, ny, nx, -1L, 1L] else  $
          temp = [1L, ny, nx, -2L, 1L]
        if nneu eq 1 then neupdi = temp else neupdi = [neupdi, temp]
      endelse
    endelse

;   get the top and bottom values
    if (rerun gt 0) then begin
      bound1 = bound_top(0)
      bound2 = bound_top(1)
      pot1 = pot_top(0)
      pot2 = pot_top(1)
    endif else if rerun lt 0 then begin
      bound1 = boundary(2)
      bound2 = boundary(3)
      bound_top= [bound1, bound2]
    endif else  begin
      if keyword_set(batch_mode) then begin
        bound1 = boundary[2]
        bound2 = boundary[3]
        pot1 = bpot[2]
        pot2 = bpot[3]
        bound_top= [bound1, bound2]
        pot_top = [pot1, pot2]
      endif else begin
        get_boundary, lcquit, 0, bound2, bound1, pot2, pot1, offset
        if lcquit ne 0 then return
        bound_top= [bound1, bound2]
        pot_top = [pot1, pot2]
      endelse
    endelse
    btop = bound2
    bbot = bound1
    if bound1 eq 0 then begin
; periodic
      xj = [0.0, ymult*yyy]
      delj = [0.0,xj(2:ny)-xj(1:ny-1), 0.0]
      xj(0) = xj(1)-delj(ny-1)
      delj(0) = delj(ny-1)
      delj(ny) = delj(1)
      if (system lt 6  and tqsystem gt 0) or $
          (system eq 6 and tqsystem lt 0) then begin
        if nper eq 0 then $
          perpdi = [1L, nx,1L, +2L, 1, 1L, nx, ny, -2L, 1L] else $
          perpdi = [perpdi, 1L, nx,1L, +2L, 1, 1L, nx, ny, -2L, 1L]
      endif else begin
        if nper eq 0 then $
          perpdi = [1L, nx,1L, +1L, 1, 1L, nx, ny, -1L, 1L] else $
          perpdi = [perpdi, 1L, nx,1L, +1L, 1, 1L, nx, ny, -1L, 1L]
      endelse 
      nper = nper+1
      for j = 0, nx-1 do cim(j, 0) = max([cim(j, 0), cim(j, ny-1)])
      cim (*, ny-1) = cim(*, 0)
      if n_elements(ere) gt 1 then begin
        ere(*, 0)= ere(*, ny-1)     
        ere(*, ny) = ere(*, 1)
      endif
    endif else begin
      xj = [0.0, ymult*yyy]
      delj = [0.0,xj(2:ny)-xj(1:ny-1), 0.0]
      xj(0)= xj(1)-delj(1)
      delj(0) = delj(1)
      delj(ny) = delj(ny-1)
      if bound1 eq 2 then begin
        if rerun ge 0 then begin
          ncond= ncond+1
          cim(*, 0) = ncond
          if ncond eq 1 then pot = float(pot1) else pot= [pot, pot1]
        endif
      endif else begin
        if system eq 2 and abs (xj(1)) lt 1e-4*xj(ny) then begin
          xj(1) = 0.0
        endif else begin
          nneu = nneu + 1L
          if (system lt 6  and tqsystem gt 0) or $
            (system eq 6 and tqsystem lt 0) then $
          temp = [1L, nx, 1L, +2L, 1L] else $
          temp = [1L, nx, 1L, +1L, 1L]
          if nneu eq 1 then neupdi = temp else neupdi = [neupdi, temp]
        endelse
      endelse
      if bound2 eq 2  then begin
        if rerun ge 0 then begin
          ncond= ncond+1
          cim(*, ny-1) = ncond
          if ncond eq 1 then pot = float(pot2) else pot= [pot, pot2]
        endif
      endif else begin
        nneu = nneu + 1L
        if (system lt 6  and tqsystem gt 0) or $
          (system eq 6 and tqsystem lt 0) then $
        temp = [1L, nx, ny, -2L, 1L] else $
        temp = [1L, nx, ny, -1L, 1L]
        if nneu eq 1 then neupdi = temp else neupdi = [neupdi, temp]
      endelse
    endelse
    cime = make_array(imax, jmax, /long)
    cime(1:*, 1:*) = cim
    cime(1:*, 0) = cim(*, 0)
    cime(0, 1:*) = cim( 0, *)
    gex= make_array(imax, jmax)
    gex(1:*, 1:*) = g
    gex(1:*, 0) = g(*, 0)
    gex(0, 1:*) = g( 0, *)
    niter = 0L
    itrmax = long(itrmax)
    conv = float(conv)
    ei = make_array(imax, jmax)
    ej = ei
    phi = ei
    ier =0L
    a = getenv('P2D_EXE')
    if strlen(a) lt 1 then p2d_exe = PFF_EXE else P2D_exe = a
    if initpff eq 1 or initpff eq 5 then begin
      if (system lt 6  and tqsystem gt 0) or $
          (system eq 6 and tqsystem lt 0) then begin
        junk = call_external(p2d_exe,  'p2d_sl', bcm, bcmb, bcme, $
           bgla, bgle, bgli, bglj, blkdim, cime, conv, deli, delj, ere, gex, $
           nx, itrmax, ny, loc, loci, locj, luout, lwork, mwachs, nblk, $
           ncond, neupdi, nneu, nper, perpdi, pot, system, xi, xj, $
           work, ei, ej, phi, niter, ier)
      endif else begin
         ei = reform (ei, jmax, imax)
         phi = reform (phi, jmax, imax)
         ej = reform (ej, jmax, imax)
         if n_elements(ere) gt 2 then ere = transpose (ere)
         gex = transpose (gex)
         cime = transpose (cime)
         junk = call_external(p2d_exe, 'p2d_sl', bcm, bcmb, bcme, $
           bgla, bgle, bglj, bgli, blkdim, cime, conv, delj, deli, ere, gex, $
           ny, itrmax, nx, loc, locj, loci, luout, lwork, mwachs, nblk, $
           ncond, neupdi, nneu, nper, perpdi, pot, system, xj, xi, $
           work, ej, ei, phi, niter, ier)
         if tqsystem lt 0 then begin
           temp = temporary(xi)
           xi = temporary(xj)
           xj = temporary(temp)
           temp = nx
           nx = ny
           ny = temp
           temp = temporary(deli)
           deli = temporary(delj)
           delj = temporary(temp)
           cim = transpose (cim)
           if n_elements(ere) gt 2 then er = transpose (er)

         endif else begin
           ej = transpose( ej)
           ei = transpose( ei)
           phi = transpose(phi)
           if n_elements(ere) gt 2 then ere = transpose (ere)
           gex = transpose (gex)
           cime = transpose (cime)
         endelse
      endelse
    endif else if (initpff eq 2) then begin
;     must be sun- determine if version 4 or 5 OS
      if (system lt 6  and tqsystem gt 0) or $
          (system eq 6 and tqsystem lt 0) then begin
        junk = call_external(p2d_exe,  'p2d_sl_', bcm, bcmb, bcme, $
           bgla, bgle, bgli, bglj, blkdim, cime, conv, deli, delj, ere, gex, $
           nx, itrmax, ny, loc, loci, locj, luout, lwork, mwachs, nblk, $
           ncond, neupdi, nneu, nper, perpdi, pot, system, xi, xj, $
           work, ei, ej, phi, niter, ier)
      endif else begin
         ei = reform (ei, jmax, imax)
         phi = reform (phi, jmax, imax)
         ej = reform (ej, jmax, imax)
         if n_elements(ere) gt 2 then ere = transpose (ere)
         gex = transpose (gex)
         cime = transpose (cime)
         junk = call_external(p2d_exe, 'p2d_sl_', bcm, bcmb, bcme, $
           bgla, bgle, bglj, bgli, blkdim, cime, conv, delj, deli, ere, gex, $
           ny, itrmax, nx, loc, locj, loci, luout, lwork, mwachs, nblk, $
           ncond, neupdi, nneu, nper, perpdi, pot, system, xj, xi, $
           work, ej, ei, phi, niter, ier)
         if tqsystem lt 0 then begin
           temp = temporary(xi)
           xi = temporary(xj)
           xj = temporary(temp)
           temp = nx
           nx = ny
           ny = temp
           temp = temporary(deli)
           deli = temporary(delj)
           delj = temporary(temp)
           cim = transpose (cim)
           if n_elements(ere) gt 2 then er = transpose (er)
         endif else begin
           ej = transpose( ej)
           ei = transpose( ei)
           phi = transpose(phi)
           if n_elements(ere) gt 2 then ere = transpose (ere)
           gex = transpose (gex)
           cime = transpose (cime)
         endelse
      endelse
    endif else if (initpff eq 4) then begin
;     Windows
      if (system lt 6  and tqsystem gt 0) or $
          (system eq 6 and tqsystem lt 0) then begin
        junk = call_external(p2d_exe,  '_P2D_SL@8', bcm, bcmb, bcme, $
           bgla, bgle, bgli, bglj, blkdim, cime, conv, deli, delj, ere, gex, $
           nx, itrmax, ny, loc, loci, locj, luout, lwork, mwachs, nblk, $
           ncond, neupdi, nneu, nper, perpdi, pot, system, xi, xj, $
           work, ei, ej, phi, niter, ier)
      endif else begin
         ei = reform (ei, jmax, imax)
         phi = reform (phi, jmax, imax)
         ej = reform (ej, jmax, imax)
         if n_elements(ere) gt 2 then ere = transpose (ere)
         gex = transpose (gex)
         cime = transpose (cime)
         junk = call_external(p2d_exe, '_P2D_SL@8', bcm, bcmb, bcme, $
           bgla, bgle, bglj, bgli, blkdim, cime, conv, delj, deli, ere, gex, $
           ny, itrmax, nx, loc, locj, loci, luout, lwork, mwachs, nblk, $
           ncond, neupdi, nneu, nper, perpdi, pot, system, xj, xi, $
           work, ej, ei, phi, niter, ier)
         if tqsystem lt 0 then begin
           temp = temporary(xi)
           xi = temporary(xj)
           xj = temporary(temp)
           temp = nx
           nx = ny
           ny = temp
           temp = temporary(deli)
           deli = temporary(delj)
           delj = temporary(temp)
           cim = transpose (cim)
           if n_elements(ere) gt 2 then er = transpose (er)
         endif else begin
           ej = transpose( ej)
           ei = transpose( ei)
           phi = transpose(phi)
           if n_elements(ere) gt 2 then ere = transpose (ere)
           gex = transpose (gex)
           cime = transpose (cime)
         endelse
      endelse


    endif else begin
        print, 'This operating system is not supported by PFIDL'
    endelse

    print, 'IER = ', strtrim(ier, 2)
    if ier lt 0 then return
    if ier gt 0 then print, ' Results may not be valid'

    print, ' Number of iterations = ', strtrim(niter, 2)
;
    cimx = cim
    cimy = cim
    ntot = nx*ny
    if n_elements(okfoil) gt 1 then begin
      okfoil = okfoil(1:*)
      ntotx = ntot-ny
      ntoty = ntot - nx
      nf = n_elements(okfoil)
      for i = 0, nf-1 do begin
         if okfoil(i) gt ny  and okfoil(i) lt ntotx then $
          if cimx(okfoil(i) +ny) eq 0 and cimx(okfoil(i)-ny) eq 0 then $
            cimx(okfoil(i)) = 0
         if okfoil(i) gt nx  and okfoil(i) lt ntoty then $
          if cimy(okfoil(i) +nx) eq 0 and cimy(okfoil(i)-nx) eq 0 then $
            cimy(okfoil(i)) = 0
      endfor
    endif
    tei = ei(1:nx, 1:ny)
    tej = ej(1:nx, 1:ny)
    ei = tei
    ej = tej
    ei(1:nx-2, *) = tei(0:nx-3, *)+tei(1:nx-2, *)
    ej(*, 1:ny-2) = tej(*, 0:ny-3)+tej(*, 1:ny-2)
    ok = where (cimx eq 0 , nok)
    if nok gt 0 then ei(ok) = ei(ok)*0.5
;   if nok gt 0 then ei = ei*0.5
    ok = where (cimy eq 0 , nok)
    if nok gt 0 then ej(ok) = ej(ok)*0.5
; now for the edges:
    if bleft eq 0 then begin
;     periodic
      ei(0, *) = tei(0, *)+tei(nx-2, *)
      for j = 0, ny-1 do begin
        if cim(0, j) eq 0 then ei(0, j)= ei(0, j)*0.5  else begin
          if cim(1, j) eq 0 and cim(nx-2, j) eq 0 then ei(0, j)= ei(0, j)*0.5
        endelse
      endfor
      ei(nx-1, *) = ei(0, *)
    endif else begin
      if bleft eq 1 then begin
;     mirror
        for i = 0, ny-1 do if cimx(0, i) eq 0 then ei(0, i) = 0.0 else $
             ei(0, i) = tei(0, i)
      endif else ei(0, *) = tei(0, *)
      if bright eq 1 then begin 
; mirror
        for i = 0, ny-1 do if cimx(nx-1, i) eq 0 then ei(nx-1, i) = 0.0 else $ 
             ei(nx-1, i) = tei(nx-2, i)
      endif else ei(nx-1, *) = tei(nx-2, *)
    endelse
    if bbot eq 0 then begin
;     periodic
      ej(*, 0) = tej(*, 0)+tej(*, ny-2)
      for i = 0, nx-1 do begin
        if cim(i, 0) eq 0 then ej(i, 0)= ej(i, 0)*0.5  else begin
          if cim(i, 1) eq 0 and cim(i, ny-2) eq 0 then ej(i, 0)= ej(i, 0)*0.5
        endelse
      endfor
      ej(*, ny-1) = ej(*, 0)
    endif else begin
      if bbot eq 1 then begin
;     mirror
        for i = 0, nx-1 do if cimy(i, 0) eq 0 then ej(i, 0) = 0.0 else $ 
             ej(i, 0) = tej(i, 0)
      endif else ej(*, 0) = tej(*, 0)
      if btop eq 1 then begin 
; mirror
        for i = 0, nx-1 do if cimy(i, ny-1) eq 0 then ej(i, ny-1) = 0.0 else $ 
             ej(i, ny-1) = tej(i, ny-2)
      endif else ej(*, ny-1) = tej(*, ny-2)
    endelse
    phi = phi(1:nx, 1:ny)
    if system eq 4 then begin
      ei = ei/xvalue
      ej = ej/xvalue
      xi = xi/xv2
      xj = xj/xv2
    endif else if system eq 6 then begin
      xi = xi/sintheta
    endif
;
; calculate energy 
;
;   calculate the centered values of the field
    cei = (tei(0:nx-2, 0:ny-2) + tei(0:nx-2, 1:ny-1))/2.0
    cej = (tej(0:nx-2, 0:ny-2) + tej(1:nx-1, 0:ny-2))/2.0
    deli = deli(1:nx-1) 
    delj = delj(1:ny-1)
    cxi = xi(1:nx-1)+deli/2.
    cxj = xj(1:ny-1)+delj/2.
    if n_elements (ere) lt 2 then $
      edensity = (cei*cei+cej*cej)*(ere *4.4270939e-12) else $
      edensity = (cei*cei+cej*cej)*er(0:nx-2, 0:ny-2) *4.4270939e-12
;   Total (edensity*dx*dy) for system = 1
;   Total (edensity*y*dx*dy) for system = 2
;   Total (edensity*x*dx*dy) for system = 3
;   Total (edensity*x^2*dx*sin(y)dy) for system =  5
;   Total (edensity*radius^2*sin(x)*dx*dy) for system = 4
;   Total (edensity*sin(theta)*dx*y^2*dy) for system = 6
    if system eq 1 then  begin
      energy = edensity*(deli#delj)
      energy = total(energy)
      print, 'Total Electric Field Energy is ', strtrim(energy, 2), ' J/m'
    endif else if system eq 2 then begin 
      energy = edensity*(deli#(cxj*delj))
      energy = total(energy)
      print, 'Total Electric Field Energy is ', strtrim(energy, 2), ' J/radian'
    endif else if system eq 3 then begin
      energy = edensity*((cxi*deli)#delj)
      energy = total(energy)
      print, 'Total Electric Field Energy is ', strtrim(energy, 2), ' J/m'
    endif else if system eq 4 then begin
      energy = edensity*((xv2*sin(cxi)*deli)#delj)
      energy = total(energy)
      print, 'Total Electric Field Energy is ', strtrim(energy, 2), ' J/m'
    endif else if system eq 5 then begin
      energy = edensity*((cxi*cxi*deli)#(delj*sin(cxj)))
      energy = total(energy)
      print, 'Total Electric Field Energy is ', strtrim(energy, 2), ' J/radian'
    endif else if system eq 6 then begin
      energy = edensity*((sintheta*deli)#(cxj*cxj*delj))
      energy = total(energy)
      print, 'Total Electric Field Energy is ', strtrim(energy, 2), ' J/radian'
    endif
    ndim = 2
    mxbl = 1
    locx = make_array (ndim, mxbl+1, /long)
    locx (*, 0) = 0
;
;
; We need to save this stuff in the correct format for 
;    3d data
;
;
;   if abs(tqsystem) eq 1 
    locx (0, 1) = nx
    locx (1, 1) = ny
    range = make_array(2, 2, 2)
    range(0, 0, *) = xi(1)
    range(0, 1, *) = xj(1)
    range(1, 0, *) = xi(nx)
    range(1, 1, *) = xj(ny)
    xlabel = make_array(1, /string)
    ylabel = make_array(1, /string)
    zlabel = make_array(1, /string)
    v3 = 'Potential'
    if system eq 1 then begin
      xlabel(0) = 'X (m)'
      ylabel(0) = 'Y (m)'
      v1 = 'Ex'
      v2 = 'Ey'
      spare = [1, 0, 0, 0, 0]
    endif else if system eq 2 then begin
      xlabel(0) = 'Z (m)'
      ylabel(0) = 'Radius (m)'
      v1 = 'Ez'
      v2 = 'Er'
      spare = [2,0,0,0,0]
    endif else if system eq 3 then begin
      xlabel(0) = 'Radius (m)'
      ylabel(0) = 'Phi (rad)'
      v1 = 'Er'
      v2 = 'Ephi'
      spare = [2,0,0,0,0]
    endif else if system eq 4 then begin
      xlabel(0) = 'Theta (rad)'
      ylabel(0) = 'Phi (rad)'
      v1 = 'Etheta'
      v2 = 'Ephi'
      spare = [3,0,0,0,0]
    endif else if system eq 5 then begin
      xlabel(0) = 'Radius (m)'
      ylabel(0) = 'Theta (rad)'
      v1 = 'Er'
      v2 = 'Etheta'
      spare = [3,0,0,0,0]
    endif else begin
      xlabel(0) = 'Sin(Theta)Phi'
      ylabel(0) = 'Radius (m)'
      v1 = 'Ephi'
      v2 = 'Er'
      spare = [3,0,0,0,0]
    endelse
    ssize = make_array(2, 1)
    ssize (0:*, 0)= [nx, ny]
    locv = [0, ntot]
    clab = 'P2D calculated value - '
    tlab = 'P2D Field'
    if n_elements (es) eq 0 then es = 80
    if n_elements (phis) eq 0 then phis = 81
    if n_elements (ces) eq 0 then ces = 79
    xi = xi(1:*)
    xj = xj(1:*)
    if n_elements(file) ne 0 then begin
      space = [xi, xj] 
      filep  = find_newfile(file )
      createpff, filep
      writepff, 0, ei, space , c = clab+v1, tlab = tlab, $
        xlab = xlab , ylab = ylab
      writepff, 0, ej, space , c = clab+v2, tlab = tlab, $
        xlab = xlab , ylab = ylab
      writepff, 0, phi, space , c = clab+v3, tlab = tlab, $
        xlab = xlab , ylab = ylab
      space = [cxi, cxj] 
      writepff, 0, cei, space , c = clab+' Cell Centered '+v1, tlab = tlab, $
        xlab = xlab , ylab = ylab
      writepff, 0, cej, space , c = clab+' Cell Centered '+v2, tlab = tlab, $
        xlab = xlab , ylab = ylab
    endif else filep = ''

    st2 ={type:1L, ndim:2L, nvect: 2L, block :1L, two: 3L, $
        slice: 0.0, size: ssize, range: range, order: 1L, locx:locx,  $
        locv: locv, x1:xi, x2:xj, x3:0.0, x4:0.0, v1:ei, v2:ej, $
        v3:0.0, v4:0.0, v5:0.0, v6:0.0,x1lab : xlabel, x2lab :ylabel, $
        x3lab : '', x4lab :'',v1lab : v1, v2lab :v2, $
        v3lab : '', v4lab :'',v5lab : '', v6lab :'', $
        file : filep, spare : spare,  tlab : 'P2D Field', clab: clab+v1+', '+v2}
    i2s, st2, es
    st2 ={type:1L, ndim:2L, nvect: 1L, block :1L, two: 3L, $
        slice: 0.0, size: ssize, range: range, order: 1L, locx:locx,  $
        locv: locv, x1:xi, x2:xj, x3:0.0, x4:0.0, v1:phi, v2:0.0, $
        v3:0.0, v4:0.0, v5:0.0, v6:0.0,x1lab : xlabel, x2lab :ylabel, $
        x3lab : '', x4lab :'',v1lab : v3, v2lab :'' , $
        v3lab : '', v4lab :'',v5lab : '', v6lab :'', $
        file : filep, spare : spare,  tlab : 'P2D Field', clab: clab +v3}

    i2s, st2, phis

    locx (0, 1) = nx-1
    locx (1, 1) = ny-1
    range = make_array(2, 2, 2)
    range(0, 0, *) = cxi(0)
    range(0, 1, *) = cxj(0)
    range(1, 0, *) = cxi(nx-2)
    range(1, 1, *) = cxj(ny-2)
    v3 = 'Potential'
    ssize (0:*, 0)= [nx-1, ny-1]
    locv = [0, (nx-1)*(ny-1)]
    st2 ={type:1L, ndim:2L, nvect: 2L, block :1L, two: 3L, $
        slice: 0.0, size: ssize, range: range, order: 1L, $
        locx:locx,  $
        locv: locv, x1:cxi, x2:cxj, x3:0.0, x4:0.0, v1:cei, v2:cej, $
        v3:0.0, v4:0.0, v5:0.0, v6:0.0,x1lab : xlabel, x2lab :ylabel, $
        x3lab : '', x4lab :'',v1lab : v1, v2lab :v2, $
        v3lab : '', v4lab :'',v5lab : '', v6lab :'', $
        file : filep, spare : spare,  tlab : 'P2D Field', $
        clab: 'Cell Centered '+ clab+v1+', '+v2}
    i2s, st2, ces
    work = 0
    return
    end
