; ------------------------------------------------------------------------ 
; $Id: check_uniform.pro,v 1.5 2010/11/18 17:44:59 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
 pro check_uniform , xin, dx, error = errorin, dxrange = delt
;+
; See if a vector x is uniform and return the point spacing if it is.
; 
; Format:
;       CHECK_UNIFORM, X, DX [, ERROR = error] [, DXRANGE = DXrange]
; where:
;    X          - the vector to be checked 
;    DX         - the calculated spacing of x
;                 If X is not uniform then DX = 0.0 
;                 If X is uniform, DX = (max(x)-min(x))/(np-1)
;    ERROR      - Error tolerance for determining if the abscissa is uniform
;                 ERROR is expressed as a fraction of the range of the X data.
;                 If specified ERROR = ERROR > ERROR_MIN
;                 See PROCEDURE below.
;                 Default: ERROR = (3.05176E-5) > ERROR_MIN
;                 NOTE:  ERROR_MIN is defined as follows for RANGE = MAX(X)-MIN(X):
;                        ERROR_MIN =  (MACHAR()).EPS * MAX([RANGE, MAX(ABS(X))]) / RANGE 
;                        A warning will be printed if ERROR < ERROR_MIN
;                 NOTE:  (MACHAR()).EPS is about 1.2e-7 (=1.0/2L^23) for many computers
;                        and  3.05176E-5 = 1./2L^15
;    DXRANGE    - Two element vector = [Min(dx), max(dx)]
;-
on_error, 1
 nelm = n_elements(xin) -1L
 
 dx = 0.0 
 if nelm lt 1 then return
; check for byte, integer, and long arrays
 x = (size(/type , xin) lt 4) ? float(xin) : xin
 dxx = x(1:nelm)-x(0:nelm-1)
 delt = [min(dxx, max = mm), mm]
 if delt(0) le 0 then return
 range = x(nelm)-x(0)
; Relative error -- relative to the range of the data
 error = (n_elements(errorin) eq 0) ? 3.05176E-5 : errorin[0]
; Convert to absolute error
 error *= range
 err_min = (machar()).eps *max(abs([range, x[0], x[nelm]]))
 if error lt err_min then begin
   print, '% CHECK_UNIFORM: Error increased from '+strtrim(error, 2)+' to ' + $
           strtrim(err_min, 2)
   error = temporary(err_min)
 endif
 if (delt[1]-delt[0]) lt error then dx = range/nelm
 end
