; ------------------------------------------------------------------------
; $Id: get_center.pro,v 1.1 2008/06/13 14:59:27 lpmix Exp $
; ------------------------------------------------------------------------
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
;+
; NAME:
;       GET_CENTER
;
; PURPOSE:
;       Find the center of a matrix 
;       Matrix assumed to have uniform rectangular pixels
;       x = total(x*M) / total(M)
;         where M is the matrix value
;
;
; CALLING SEQUENCE:
; 
;       CENTER = GET_CENTER( ARRAY )
;
; INPUTS:
;       ARRAY:  Two dimensional array
;
; OUTPUTS:
;       CENTER: [I, J] center of mass of the array
;               I, J are GE zero and less than NX, NY and are floats.
;
; PROCEDURE:
;       Array = Array[nx, ny]
;       I = total(findgen(nx)#array) / total(Array)
;       J = total(array#findgen(ny)) / total(Array)
;
; EXAMPLE:
;
;	Determine the center of array v1
;         CENTER = GET_CENTER(V1)
;
; MODIFICATION HISTORY:
; 	Written by:	L. P. Mix, Jr., May 2, 2008
;
;-

Function GET_CENTER, array
  COMPILE_OPT  hidden
  if n_elements (array) eq 0 then begin
    print, '% GET_CENTER: Array must be defined'
    return, 0
  endif
  ss = size (array)
  If (ss[0] ne 2) then begin
    print, '% GET_CENTER: Array must have two dimensions'
    return, 0
  endif
  nx = ss[1]
  ny = ss[2]
  ta = total(array, /double)
  i = total(findgen(nx)#array, /double)/ta
  j = total(array#findgen(ny), /double)/ta
  return, float([i, j])
END
