; ------------------------------------------------------------------------
; $Id: get_sph_spec_params.pro,v 1.7 2011/03/28 21:49:28 lpmix Exp $
; ------------------------------------------------------------------------
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
;+
; NAME:
;       GET_SPH_SPEC_PARAMS
;
; PURPOSE:
;       Display a GUI to get the parameters
;
; CATEGORY:
;	Widgets.
;
; CALLING SEQUENCE:
;       GET_SPH_SPEC_PARAMS, WDF, RADIUS, CRY2D, ORDER, DISTANCE, THETA, FILM
;
; INPUTS:
; Note:  Default values are from SPH_SPEC in PFIDL_1D
;       WDF:    Initial WDF array number
;       RADIUS: Spherical radius of crystal (mm)
;       CRY2D:  2D Spherical radius of crystal (mm)
;       ORDER:  Diffraction order
;       DIST:   Source distance
;       THETA:  Source Angel - Degrees
;       Film:   Film position parameters: [X, Y, Angle] of the film 
;               If film has less than 3 parameters, it is set to the default value
;               from SPH_SPEC.  If it still has less than 3 values, then it is 
;               set to the default location for the crystal radius and the source.
;       TLBASE: If not zero, this will be used as the top level base for the widget.
;               NO_BLOCK is set in the call to XMANAGER
; 
; OUTPUTS:
;       All input values are returned with the values entered/modifiec
;       by the user.
;
; SIDE EFFECTS:
;       Input values are modified by the widget.
;
; EXAMPLE:
;       Get values for a spherical spectrometer
;         GET_SPH_SPEC_PARAMS, WDF, Radius, CRY2D, ORDER, DIST, THETA, FILM
;
; MODIFICATION HISTORY:
; 	Written by:	L. P. Mix, Jr., December 23, 2010
;
;-
; 
pro GET_SPH_SPEC_PARAMS_EVENT, event
on_error, 1
common exray_sph_spec_common, tlbase_com
@sph_spec_common.dat

widget_control, tlbase_com, get_uvalue = ptr
info = *ptr
lc = 1
en = info.nval
if event.id eq info.stdbutton then lc = 4
if event.id eq info.stdwid then begin
  i = event.index
  get_sph_database_values, i, rc, cry2d, order, length, width, distance, theta, $
      npoints, film, verbose = verbose
; insure values in case film is "standard position"
  info.values[1] = (n_elements(rc) gt 0) ?  rc : rc_save
  info.values[2] = (n_elements(cry2d) gt 0) ? cry2d : cry2d_save
  info.values[3] = (n_elements(length) gt 0) ? length : length_save
  info.values[4] = (n_elements(width) gt 0) ? width : width_save
  info.values[5] = (n_elements(order) gt 0) ? order : order_save
  info.values[6] = (n_elements(distance) gt 0) ? distance : distance_save
  info.values[7] = (n_elements(theta) gt 0) ? theta : theta_save
  if (min(film) lt -0.8e30) or n_elements(film) lt 3 then begin
    thetar = info.values[7] *!dpi/180.d0
    film =[0, info.values[1], atan(info.values[6]*sin(thetar)-info.values[1], $
       info.values[6]*cos(thetar))]
  endif
  info.values[8] = film[0]
  info.values[9] = film[1]
  info.values[10] = film[2]
  for i = 1, 10 do widget_control, info.widval[i], set_value = info.values[i]
endif else begin
  en = en-lc
  for i = 0, en do begin
    widget_control, info.widval[i] , get_value = val
    widget_control, info.widval[i] , set_value = val
    info.values[i] = val
  endfor
  if lc gt 1 then begin
    k = en+1
    info.values[k] = 0.0  
    widget_control, info.widval[k] , set_value = info.values[k]
    k+= 1
    info.values[k] = info.values[1]
    widget_control, info.widval[k] , set_value = info.values[k]
    k+= 1
    thetar = info.values[7] /!radeg
    info.values[k] = atan(info.values[6]*sin(thetar) -info.values[1], info.values[6]*cos(thetar))
    widget_control, info.widval[k] , set_value = info.values[k]
  endif
endelse
if event.id eq info.quitbut then info.values[info.nval] = 1
*ptr = info
widget_control, tlbase_com,  set_uvalue = ptr
if (event.id eq info.exitbut) || (event.id eq info.quitbut) then $
  widget_control, event.top, /destroy
end


PRO GET_SPH_SPEC_PARAMS, WDF, RADIUS, CRY2D, LENGTH, WIDTH, ORDER, DISTANCE, THETA, $
      FILM, GEOMETRY = geometry, group = group, quit = quit, TLBASE = tlbase

on_error, 1
common exray_sph_spec_common, tlbase_com
@sph_spec_common.dat
no_block =0
if n_elements(tlbase) eq 0 then $
  tlbase = WIDGET_BASE(GROUP_LEADER=Group, $
      COLUMN=1, $
      MAP=1, $
      TITLE='Get Spectrometer Parameters') else no_block = 1

  tlbase_com = tlbase
  WDFbase = widget_base(tlbase, frame =2, /column, /map, uvalue = 'WDFBASE')
  if n_elements(wdf) eq 0 then wdf = 1 else wdf = wdf[0]
  WDFfield = CW_FIELD( wdfbase,VALUE=WDF, $
      ROW=1, $
      LONG=1, $
      RETURN_EVENTS = 1, $
      TITLE='Initial WDF Array:', $
      UVALUE='WDF_Value')
  if size(/type, database_save) eq 8 then begin
    stdbase = WIDGET_BASE(tlbase, $
        COLUMN = 1, $
        MAP=1, $
        UVALUE = 'CRYBASE', frame = 2)
    labelstd =  WIDGET_LABEL( stdbase, UVALUE='LABEL', $
    VALUE='Enter the Standard Index', /align_center)
    std = database_save.c0
    stdwid = Widget_droplist (stdbase, value = std, uvalue= 'STD_VALUE')
  endif else stdwid = -1L


  crybase = WIDGET_BASE(tlbase, $
      COLUMN = 1, $
      MAP=1, frame =2, $
      UVALUE = 'CRYBASE')
  labelcry =  WIDGET_LABEL( crybase, UVALUE='LABEL', $
  VALUE='Enter the CRYSTAL Parameters', /align_center)
  if n_elements(radius) eq 0 then radius = rc_save else radius = radius[0]
  if n_elements(cry2d) eq 0 then cry2D = cry2d_save else cry2d = cry2d[0]
  if n_elements(order) eq 0 then order =order_save else order = order[0]
  cryrad = cw_field(crybase, /return_events, /float, title ='Crystal Radius(mm):', $   
     value = radius, uvalue='CRY_RAD')
  cryspacing = cw_field(crybase, /return_events, /float, title ='Plane Spacing(Ang):', $
     value = cry2d, uvalue='CRY_SPACE')
  crylength = cw_field(crybase, /return_events, /float, title ='Length(Wavelength)(mm):', $
     value = length, uvalue='CRY_LENGTH')
  crywidth = cw_field(crybase, /return_events, /float, title ='Crystal Width(Image)(mm):', $
     value = width, uvalue='CRY_WIDTH')
  cryorder = cw_field(crybase, /return_events, /long, title ='Diffraction Order:', $
     value = order, uvalue='CRY_ORDER')
  
  srcbase = WIDGET_BASE(tlbase, $
      COLUMN = 1, $
      MAP=1, frame =2, $
      UVALUE = 'SRCBASE')
  labelSRC =  WIDGET_LABEL( SRCbase, UVALUE='LABEL', $
  VALUE='Enter the SOURCE Parameters', /align_center)
  if n_elements(Distance) eq 0 then distance = distance_save else distance = distance[0]
  if n_elements(Theta) eq 0 then Theta = theta_save else theta = theta[0]
  srcdist = cw_field(srcbase, /return_events, /float, title ='Source Distance(mm):', $   
     value = distance, uvalue='SRC_DIST')
  srcangle = cw_field(srcbase, /return_events, /float, title ='Bragg Angle(deg):', $ 
     value = theta, uvalue='SRC_ANGLE')

  filmbase = WIDGET_BASE(tlbase, $
      COLUMN = 1, $
      MAP=1, frame =2, $
      UVALUE = 'SRCBASE')
  labelsFILM =  WIDGET_LABEL( filmbase, UVALUE='LABEL', $
  VALUE='Press to obtain standard FILM Parameters', /align_center)
  stdbutton = widget_button(filmbase, uvalue ='STD_BUTTON', /align_center, $
         /sensitive, value ='Standard Film Position')
  labelFILM =  WIDGET_LABEL( filmbase, UVALUE='LABEL', $
  VALUE='Enter the FILM Parameters', /align_center)
  thetar = theta/!radeg
  if n_elements(film) lt 3 then film = film_save
  if n_elements(Film) lt 3 then film = $
    [0, radius, atan(distance*sin(thetar)-radius, distance*cos(thetar))] else $
    film =film[0:2]
  Filmx = cw_field(filmbase, /return_events, /float, title ='Film X(mm):', $
     value = film[0], uvalue='FILM_X')
  Filmy = cw_field(filmbase, /return_events, /float, title ='Film Y(mm):', $
     value = film[1], uvalue='FILM_Y')
  Filmang = cw_field(filmbase, /return_events, /float, title ='Film Angle(Rad):', $
     value = film[2], uvalue='FILM_ANGLE')

quit = 0
if no_block eq 0 then begin
  button_base = WIDGET_BASE(tlbase, $
      ROW = 1, MAP=1, frame =2, UVALUE = 'BUTBASE')
  quitbut = widget_button(button_base, /frame, value = '    QUIT / Abort   ')
  exitbut = widget_button(button_base, /frame, value = '  Finished / Exit  ')
endif else begin
  button_base = 0
  quitbut = 0
  exitbut = 0
endelse
widget_control, tlbase, /realize
nval = 11L
values = [wdf, radius, cry2d, length, width, order, distance, theta, $
           film[0], film[1], film[2],quit]
widval = [wdffield, cryrad, cryspacing, crylength, crywidth, cryorder, srcdist, $
         srcangle, filmx, filmy, filmang]
info = {exitbut:exitbut, quitbut:quitbut, stdwid: stdwid, stdbutton:stdbutton, $
        nval: nval, values: values, widval: widval}
ptr = ptr_new(info)
widget_control, tlbase, set_uvalue = ptr
xmanager, 'get_sph_spec_params', tlbase, no_block = no_block
if no_block eq 1 then return
info= *ptr
wdf = round(info.values[0])
radius = info.values[1]
cry2d = info.values[2]
length = info.values[3]
width = info.values[4]
order = round(info.values[5])
distance = info.values[6]
theta = info.values[7]
film = [info.values[8], info.values[9], info.values[10]]
quit = info.values[11]

  ptr_free, ptr
  values =0
  widval =0
  info = 0
end
