; ------------------------------------------------------------------------ 
; $Id: hipstr.pro,v 1.1 2011/08/18 15:42:07 lpmix Exp $
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
 Pro  HIPSTR, array, atten = filter, xarray = distfunx, yarray = distfuny, $
      order = order, type = type, mode = mode
;
;+
; NAME:
;       HIPSTR
;
; PURPOSE:
;       High pass filter utility; called by FILTSTR
;       (See FILTSTR for additional details)
; CATEGORY:
;       Filter
;
; CALLING SEQUENCE:
;       HIPSTR, ARRAY [, TYPE=TYPE] [, ATTEN=ATTEN][,ORDER = ORDER] $R
;            [, MODE = MODE] [, XARRAY = xarray] [, YARRAY = yarray]
; INPUTS:
;       ARRAY    IDL array to be filtered.
;                Must be two dimensional.
;
; KEYWORD PARAMETERS:
;       TYPE     Type of filter desired ('BUT', 'EXP', 'USER', 'IDEAL').
;                Default is 'BUT'
;       ATTEN    Frequency domain filter function.  Must be supplied with
;                TYPE = 'USER' and have same number of values as ARRAY.
;                If TYPE ne 'USER', contains the actual filter function.
;       ORDER    Order of the filter
;                Default is 2
;       MODE     Center frequency of the low pass filter expressed in 
;                units of mode numbers.  Default is 12
;       XARRAY   X-Mode numbers for FILTER, all positive
;       YARRAY   Y-Mode numbers for FILTER, all positive
; OUTPUTS:
;       NONE     The filtered array is returned in ARRAY
;
; OPTIONAL OUTPUT KEYWORD PARAMETERS:
;       ATTEN    If TYPE is not 'USER', ATTEN contains the freq domain
;                filter function.
; RESTRICTIONS:
;       If type is 'USER', ATTEN must be provided
;
; PROCEDURE:
;       The Fourier transform of ARRAY is taken and multiplied by a frequency
;       domain filter function.  The inverse Fourier transform is taken, and
;       the real values returned in ARRAY.
; MODIFICATION HISTORY:
;      Original version, L. P. Mix, August 16, 2009
;
;-
;
;
      nparams = n_params()
      if nparams lt 1 then begin
        print,' HIPSTR must have at least 1 parameter'
        return
      endif
;
      ss = size (array)
      if (ss[0] ne 2) or (min(ss[1:2]) lt 3) then begin
        print, '% HIPSTR: ARRAY must be two dimensional.'
        print, '          NX and NY must be greater than 2'
        return
      endif
      nx = ss[1]
      ny = ss[2]
     
;
      if n_elements(order) gt 0 then order = abs(order) > 0.01 else $
        order = 2
      orderx = order[0]
      ordery = (n_elements(order) gt 1) ? order[1] : orderx
     
      total = [nx, ny]
      bin =  (n_elements(mode) gt 0) ? ( abs(mode) > 1) : 12
      binx = bin[0]
      biny = (n_elements(bin) gt 1) ? bin[1] : binx

      distfun = findgen(nx)
      distfunx = distfun < (nx - distfun)
      distfunx[0] = 1.0

      distfun = findgen(ny)
      distfuny = distfun < (ny - distfun)
      distfuny[0] = 1.0

      if n_elements(type) gt 0 then begin
        type = strtrim(type, 2)
        type = strupcase(type)
        type = strmid(type, 0, 3)
        case type of 
  'BUT' : filtertype = 'BUTTER'
  'EXP' : filtertype = 'EXPON'
  'IDE' : filtertype = 'IDEAL'
  else  : filtertype = 'USER'
        endcase
      endif else  filtertype = 'BUTTER'

      if filtertype eq 'USER' then begin
        ssf = (size (filter))[1:2]
        if total(abs([nx, ny]-ssf)) gt 0 then begin
          print, '% HIPSTR:  USER filter type requires filter and array to be the same size'
          print, '           Filter is size = ' , ssf
          print, '           Array is size = ', [nx, ny]
          return
        endif
      endif else begin
        
        case filtertype of 
  'BUTTER' : BEGIN
             filterx =1.0/(1.0+(binx/distfunx)^(2*orderx))
             filtery =1.0/(1.0+(biny/distfuny)^(2*ordery))
             END
  'EXPON'  : BEGIN
             filterx = exp((-(binx/distfunx)^orderx))
             filtery = exp((-(biny/distfuny)^ordery))
             END
  'IDEAL'  : BEGIN
             filterx = (distfunx ge binx)
             filtery = (distfuny ge biny)
             END
   else    : BEGIN
             print,'% HIPSTR: Error in call, type = ' , filtertype
             return
             END
        endcase
        filterx[0] = 0.0
        distfunx[0] = 0.0
        filtery[0] = 0.0
        distfuny[0] = 0.0
        fx = make_array(nx, ny)
        fy = make_array(ny, nx)
        for i = 0, ny-1 do fx[0, i] = filterx
        for i = 0, nx-1 do fy[0, i] = filtery
        filter = fx*transpose(fy)
      endelse
;
      array = float(fft(filter * fft(array, -1, /double), 1, /double))
      end
