; ------------------------------------------------------------------------ 
; $Id: image_fld.pro,v 1.4 2007/06/06 19:34:30 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
 Pro image_fld, str1, reflection = reflection, periodic = periodic, $
          again = again, $
          value = value, invert = invert, nperiods = nperiods, CLABEL = clabel
;+
; NAME:
;       image_fld
;
; PURPOSE:
;       Generate an image file from data which has a reflection plane or 
;         has periodic boundaries.
;       The data must be two dimensional or three dimensional with a degenerate
;         dimension and have a single attribute.
;
; CATEGORY:
;       Math 
; CALLING SEQUENCE:
;       IMAGE_FLD, STR1, [, REFLECTION = reflection]
;              [, PERIODIC = PERIODIC] [, VALUE = value] [, /INVERT]
;              [, NPERIODS = NPERIODS] [, CLABEL = clabel] [, /REPEAT]
; INPUTS:
;       STR1:    Image structure array of the first field structure array
; KEYWORDS:
;       REFLECTION : If present must equal to +/- 1, 2, or 3.
;                    + indicates maximum value; - indicates minimum value 
;                    for the reflection.
;                    1, 2, 3 indicates coordinates 1, 2, or 3
;       VALUE      : If a different value is required for the reflection, 
;                    value is used to specify this point. Value must be greater
;                    than the maximum spatial value of the data or less than
;                    the minimum to avoid multiple values at a point.
;                    Xnew = 2*value - Xold
;                    For a periodic image, value is the value of the period.
;                    Xnew  = Xold + value 
;       INVERT     : If present and not zero, multiple the field values by -1.
;       PERIODIC   : If present must equal to +/- 1, 2, or 3.
;                    + indicates maximum value; - indicates minimum value 
;                    for the translation.
;                    1, 2, 3 indicates coordinates 1, 2, or 3
;       NPERIODS   : Number of replications for the periodic data, default = 1
;       AGAIN      : If present and not zero, all parameters will be the same
;                    as the last call to IMAGE_FLD
;       CLABEL:  Optional label for the combined structures.
; OUTPUTS:
;       NONE    The composite image is stored in a structure array 
;                 in the common block
; RESTRICTIONS:
;       STR1 array must be defined and have valid field (or image) data
;       If a structure has three dimensions, one of them must be degenerate.
;       The number of vectors or attributes must be 1, ie scalar field.
;       The data must be single block.
; EXAMPLES:
;       Reflect structure array F2 about the maximum y value 
;               IMAGE_FLD, F2, reflect = 2
;       Reflect structure array F2 about the minimum Z value; invert the field
;               IMAGE_FLD, F2, reflect = -3, /invert
;       Reflect structure array F5 about the r= 0 value 
;               IMAGE_FLD, F5, reflect = 1, value = 0
;       Replicate structure array F3 about the maximum theta value (dimension 2)
;               IMAGE_FLD, F3, period = 2

; MODIFICATION HISTORY:
;      Original version, L. P. Mix, April 21, 1994
;-
;
; insert a common block
;
 common image_com, reflect_sav, period_sav, lcneg_sav, value_sav, nperiod_sav
;
@qscom.dat
;  Common block
@qsstrcom.dat
;       return on an error
      on_error, 1
;
      if n_elements (structinit) eq 0 then qsstruct_init
      if n_elements (qsplotinit) eq 0 then qsinit
;
      nparams = n_params()
      if nparams lt 1 then begin
        print,' IMAGE_FLD must have at least 1 parameters'
        return
      endif
      b = size(str1)
      if  b(b(0)+1) ne 8 then begin
            print, $
            ' IMAGE_FLD: First parameter must be a field structure'
            return
       endif
;
      arr = str1
      if arr.type ne 1 then begin
        print, ' IMAGE_FLD may only be used with field structures'
        return
      endif
      if arr.ndim eq 2 or (arr.ndim eq 3 and arr.two ne 0) then begin
        if arr.nvect ne 1 then begin
          print, 'IMAGE_FLD must be used with a scalar field.'
          return
        endif
      endif else begin
        print, 'IMAGE_FLD can be used only with 2D or degenerate 3D data'
        return
      endelse
      nblo = arr.block
      mxbl = 2*nblo 
      ndim = arr.ndim
      if ndim eq 2 then arr.two = 3
      if n_elements (reflect_sav) eq 0 then again = 0
      if n_elements (again) eq 0 then again = 0
      if again eq 0 then begin
        if not keyword_set(reflection) then reflection = 0
        if not keyword_set(periodic) then periodic = 0
        if n_elements (nperiods) eq 0 then nperiods = 1
        if (reflection eq 0 and periodic eq 0)  then begin
          print, 'Reflection or periodic dimension must be specified'
          return
        endif
      endif else begin
        reflection = reflect_sav
        periodic = period_sav
        if lcneg_sav ne 0 then begin
          reflection = -reflection
          periodic = -periodic
        endif
        value = value_sav
        nperiods = nperiod_sav
      endelse
      if reflection eq arr.two or periodic eq arr.two then begin
        print, 'Reflection/periodic dimension may not be degenerate'
        return
      endif
      range = make_array(2, ndim, mxbl+1)
      range (*, *, 0:nblo) = arr.range
      if reflection ne 0 then begin
        if abs(reflection) lt 1 or abs(reflection) gt 3 then begin
          print,'Reflection keyword must be in the range +/-1 to +/-3'
          return
        endif
        if reflection lt 0 then begin
          reflection = - reflection
          lcneg = 1
        endif else lcneg = 0
        if reflection eq 1 then lcx = 1 else if $
          (reflection eq 2 and arr.two eq 1) then lcx = 1 else lcx = 0
        if n_elements(value) eq 0 then begin
          if lcneg eq 0 then value = range(1, reflection-1, 0) $
           else value = range(0, reflection-1, 0)
        endif else begin
          err = 6.1e-5*max(abs(range(*, reflection-1, 0)))
          if value gt range(0, reflection-1, 0)+err and $
              value lt range(1, reflection-1, 0)-err then begin
            print, $
           'Specified value for reflection must be outside the data range'
            return
          endif
        endelse
        periodic = 0
      endif else begin
        if abs(periodic) lt 1 or abs(periodic) gt 3 then begin
          print,'Periodic keyword must be in the range +/-1 to +/-3'
          return
        endif
        reflection = 0
        if periodic lt 0 then begin
          periodic =-periodic
          lcneg = 1
        endif else lcneg = 0
        if periodic eq 1 then lcx = 1 else if $
          (periodic eq 2 and arr.two eq 1) then lcx = 1 else lcx = 0
        if n_elements (value) eq 0 then begin
          delta = range(1, periodic-1, 0)-range(0, periodic-1, 0)
          value = delta
        endif else begin
          delta = range(1, periodic-1, 0)-range(0, periodic-1, 0)
          err = 6.1e-5*max(abs(range(*, periodic-1, 0)))
          if delta -2.0*err gt value then begin
            print,  $
            ' Value of periodic spacing must be greater than the data range'
            return
          endif
          value = delta
        endelse
      endelse
      
      x1 = arr.x1
      x2 = arr.x2
      if ndim eq 3 then x3 = arr.x3 else x3 = 0.0
      x4 = 0.0
      v1 = arr.v1
      if (size(v1))[0] ne 1 then v1 = reform (v1, n_elements(v1), /overwrite)
      v2 = 0.0
      v3 = 0.0
      v4 = 0.0
      v5 = 0.0
      v6 = 0.0
    if keyword_set(periodic) and nblo eq 1 then begin
      mxbl = 1L
      locx = arr.locx
      locv= arr.locv
      x1lab = arr.x1lab
      x2lab = arr.x2lab
      x3lab = arr.x3lab
      x4lab = arr.x4lab
      v1lab = arr.v1lab
      v2lab = arr.v2lab
      v3lab = arr.v3lab
      v4lab = arr.v4lab
      v5lab = arr.v5lab
      v6lab = arr.v6lab
      ssize = arr.size
      range = arr.range
      order = 0L
      n = ssize(*, 0)
      v1 = reform (v1, n, /overwrite)
      v1 = reform (v1, /overwrite)
      v = temporary(v1)
      if periodic eq 1 then begin
        xt = x1(1:*)
        nx1 = n_elements(xt)
        iy = 6-arr.two-1
        for i = 1, nperiods do x1 = [x1, xt+i*delta]
        nx = n_elements(x1)
        locx(0, 1) = nx
        range(1, 0, *) = x1(nx-1)
        ssize(0, 0) = nx
        ny = ssize(iy-1, 0)
        v1 = make_array(nx*ny)
        locv(1) = nx*ny
        nn = 0
        nx2 = nx1-1
        for i = 0, ny-1 do begin
          v1(nn:nn+nx1) = v(*, i)
          nn = nn+nx1+1
          vt = v(1:*, i)
          for j = 1, nperiods do begin
            v1(nn:nn+nx2) = vt
            nn = nn+nx1
          endfor
        endfor
      endif else if periodic eq 2 then begin
        yt = x2(1:*)
        nx1 = n_elements(yt)
        ix = 6-arr.two-2
        for i = 1, nperiods do x2 = [x2, yt+i*delta]
        nx = n_elements(x2)
        range(1, 1, *) = x2(nx-1)
        ssize(1, 0) = nx
        ny = ssize(ix-1, 0)
        v1 = make_array(nx*ny)
        locx(1, 1) = nx
        locv(1) = nx*ny
        nn = 0
        if ix eq 3 then begin
;         yz data  with replicate in y
; this is the same as xy, xz above
          nx2 = nx1-1
          for i = 0, ny-1 do begin
            v1(nn:nn+nx1) = v(*, i)
            nn = nn+nx1+1
            vt = v(1:*, i)
            for j = 1, nperiods do begin
              v1(nn:nn+nx2) = vt
              nn = nn+nx1
            endfor
          endfor
        endif else begin      
;         xy data with replicate in y
          vt = v(*, 1:*)
          nn = n_elements(vt)
          n0 = n_elements(v)
          v1(0:n0-1) = v
          for i = 0, nperiods-1 do v1(n0+i*nn: n0+nn*(i+1)-1) = vt
        endelse
      endif else begin
        yt = x3(1:*)
        ix = 6-arr.two-3
        for i = 1, nperiods do x3 = [x3, yt + i*delta]
        nx = n_elements(x3)
        range(1, 2, *) = x3(nx-1)
        ssize(1, 0) = nx
        ny = ssize(ix-1, 0)
        v1 = make_array(nx*ny)
        locx(2, 1) = nx
        locv(1) = nx*ny
        nn = 0
;         xy data with replicate in y
        vt = v(*, 1:*)
        nn = n_elements(vt)
        n0 = n_elements(v)
        v1(0:n0-1) = v
        for i = 0, nperiods-1 do v1(n0+i*nn: n0+nn*(i+1)-1) = vt
      endelse
    endif else begin
      locx = make_array(ndim, 2*nblo+1)
      locx(*, 0:nblo)= arr.locx
      lx = arr.locx(*, nblo)
      locv= arr.locv
      lv = arr.locv(nblo)
      x1lab = [arr.x1lab, arr.x1lab]
      x2lab = [arr.x2lab, arr.x2lab]
      x3lab = [arr.x3lab, arr.x3lab]
      x4lab = [arr.x4lab, arr.x4lab]
      v1lab = [arr.v1lab, arr.v1lab]
      v2lab = [arr.v2lab, arr.v2lab]
      v3lab = [arr.v3lab, arr.v3lab]
      v4lab = [arr.v4lab, arr.v4lab]
      v5lab = [arr.v5lab, arr.v5lab]
      v6lab = [arr.v6lab, arr.v6lab]
      range = make_array(2, ndim, mxbl+1)
      range[*,*,0:nblo] = arr.range
      ssize = make_array(ndim, mxbl, /long)
      ssize(*, 0:nblo-1)= arr.size
      if periodic ne 0 then begin
        if nperiods gt 1 then begin
          print, '% IMAGE_FLD: NPERIODS must be 1 for multiblock data.'
          nperiods = 1
        endif
        for iblo = 1, nblo do begin
          if arr.two eq 1 then begin
            x1 =[x1, x1(0)]
            dx1= 1L
          endif else begin
            xt = x1(locx(0,iblo-1):locx(0,iblo)-1)
            dx1 = (locx(0,iblo)-locx(0,iblo-1))
            if periodic eq 1 then begin
              for i = 1, nperiods do x1= [x1, xt+i*delta]
              dx1 = dx1*nperiods
            endif else x1= [x1, xt]
          endelse
          if arr.two eq 2 then begin
            x2 =[x2, x2(0)]
            dx2= 1L
          endif else begin
            xt = x2(locx(1,iblo-1):locx(1,iblo)-1)
            dx2 = (locx(1,iblo)-locx(1,iblo-1))
            if periodic eq 2 then begin
              for i = 1, nperiods do x2= [x2, xt+i*delta]
              dx2 = dx2*nperiods
            endif else x2= [x2, xt]
          endelse
          if ndim eq 3 then begin
            if arr.two eq 3 then begin
              x3 =[x3, x3(0)]
              dx3= 1L
            endif else begin
              xt = x3(locx(2,iblo-1):locx(2,iblo)-1)
              dx3 = (locx(2,iblo)-locx(2,iblo-1))
              if periodic eq 3 then begin
                for i = 1, nperiods do x3= [x3, xt+i*delta]
                dx3 = dx3*nperiods
              endif else x3= [x3, xt]
            endelse
            lx = lx+[dx1, dx2, dx3]
            ssize (*, iblo+nblo-1)= [dx1, dx2, dx3]
            locx(*, iblo+nblo)=lx
          endif else begin
            lx = lx+[dx1, dx2]
            ssize (*, iblo+nblo-1)= [dx1, dx2]
            locx(*, iblo+nblo)=lx
          endelse
          range(0, 0, iblo+nblo) = x1(locx(0,iblo+nblo-1)) 
          range(1, 0, iblo+nblo) = x1(locx(0,iblo+nblo)-1) 
          range(0, 1, iblo+nblo) = x2(locx(1,iblo+nblo-1)) 
          range(1, 1, iblo+nblo) = x2(locx(1,iblo+nblo)-1) 
          if ndim eq 3 then begin
            range(0, 2, iblo+nblo) = x3(locx(2,iblo+nblo-1)) 
            range(1, 2, iblo+nblo) = x3(locx(2,iblo+nblo)-1) 
          endif
          vt =  v1(locv(iblo-1): locv(iblo)-1)
          ltemp = locv(iblo)-locv(iblo-1)
          if nperiods eq 1 then begin
            v1 = [v1, vt]
            lv = lv + ltemp
            locv = [locv, lv]
          endif else begin
            vt = reform (vt, ssize(*, iblo-1))
            vt = reform (vt)
            bsize= size(vt)
            if bsize(0) ne 2 then begin
              print, ' A bad error -- BSIZE(0) ne 2'
              return
            endif
            if lcx eq 1 then begin
              temp = make_array(bsize(1)*nperiods, bsize(2))
              for i = 1, nperiods do temp((i-1)*bsize(1):i*bsize(1)-1,*)=vt
              lv = lv +nperiods*bsize(4)
              locv= [locv, lv]
              v1=[v1,reform (temp, nperiods*bsize(4))]
            endif else begin
              temp = make_array(bsize(1), bsize(2)*nperiods)
              for i = 1, nperiods do temp(*,(i-1)*bsize(2):i*bsize(2)-1)=vt
              lv = lv +nperiods*bsize(4)
              locv= [locv, lv]
              v1=[v1,reform (temp, nperiods*bsize(4))]
            endelse
          endelse
        endfor
      endif else begin
;     reflection
        for iblo = 1, nblo do begin
          if arr.two eq 1 then begin
            x1 =[x1, x1(0)]
            dx1= 1L
          endif else begin
            xt = x1(locx(0,iblo-1):locx(0,iblo)-1)
            dx1 = (locx(0,iblo)-locx(0,iblo-1))
            if reflection eq 1 then $
              xt = rotate(2.*value -xt, 2)
            x1= [x1, xt]
          endelse
          if arr.two eq 2 then begin
            x2 =[x2, x2(0)]
            dx2= 1L
          endif else begin
            xt = x2(locx(1,iblo-1):locx(1,iblo)-1)
            dx2 = (locx(1,iblo)-locx(1,iblo-1))
            if reflection eq 2 then $
              xt = rotate(2.*value -xt, 2)
            x2= [x2, xt]
          endelse
          if ndim eq 3 then begin
            if arr.two eq 3 then begin
              x3 =[x3, x3(0)]
              dx3= 1L
            endif else begin
              xt = x3(locx(2,iblo-1):locx(2,iblo)-1)
              dx3 = (locx(2,iblo)-locx(2,iblo-1))
              if reflection eq 3 then $
                xt = rotate(2.*value -xt, 2)
              x3= [x3, xt]
            endelse
            lx = lx+[dx1, dx2, dx3]
            ssize (*, iblo+nblo-1)= [dx1, dx2, dx3]
            locx(*, iblo+nblo)=lx
          endif else begin
            lx = lx+[dx1, dx2]
            ssize (*, iblo+nblo-1)= [dx1, dx2]
            locx(*, iblo+nblo)=lx
          endelse
          range(0, 0, iblo+nblo) = x1(locx(0,iblo+nblo-1))
          range(1, 0, iblo+nblo) = x1(locx(0,iblo+nblo)-1)
          range(0, 1, iblo+nblo) = x2(locx(1,iblo+nblo-1))
          range(1, 1, iblo+nblo) = x2(locx(1,iblo+nblo)-1)
          if ndim eq 3 then begin
            range(0, 2, iblo+nblo) = x3(locx(2,iblo+nblo-1))
            range(1, 2, iblo+nblo) = x3(locx(2,iblo+nblo)-1)
          endif
          vt =  v1(locv(iblo-1): locv(iblo)-1)
          ltemp = locv(iblo)-locv(iblo-1)
          vt = reform (vt, ssize(*, iblo-1))
          vt = reform (vt)
          bsize= size(vt)
          if bsize(0) ne 2 then begin
            print, ' A bad error -- BSIZE(0) ne 2'
            return
          endif
          if lcx eq 1 then begin
            vt = transpose(rotate(vt, 3))
            if keyword_set(invert) then vt = -vt
            lv = lv +bsize(4)
            locv= [locv, lv]
            v1=[v1,reform (vt, bsize(4))]
          endif else begin
            vt = transpose (rotate(vt, 1))
            if keyword_set(invert) then vt = -vt
            lv = lv +bsize(4)
            locv= [locv, lv]
            v1=[v1,reform (vt, bsize(4))]
          endelse
        endfor
      endelse
;
      npar = 7
      for i = 0, ndim -1 do begin
        range(0, i, 0)=min(range(0,i,1:mxbl))
        range(1, i, 0)= max(range(1,i,1:mxbl))
      endfor
;
      order = make_array(ndim, mxbl, /long)
      for i = 0, ndim-1 do order (i, *) = sort(range(0,i, 1:mxbl))
    endelse
;
      if n_elements(clabel) gt 0 then clab = string(clabel) $
        else clab = arr.clab
      str1 = {type:1L, ndim:ndim, nvect:1L, block:mxbl, two:arr.two, $
              size:ssize, range: range, order: order, slice:arr.slice, $
              locx:locx, locv:locv, x1:x1, x2: x2, $
              x3:x3, x4:x4, v1:v1, v2:v2, v3:v3, v4:v4, v5:v5, v6:v6, $
              x1lab : x1lab,  x2lab : x2lab, x3lab :x3lab, x4lab :x4lab,  $
              v1lab : v1lab, v2lab :v2lab, v3lab:v3lab, $
              v4lab : v4lab, v5lab :v5lab, v6lab:v6lab, $
              file: arr.file, spare:arr.spare, tlab : arr.tlab, $
              clab:clab }
;
      if again eq 0 then begin
        reflect_sav = reflection
        period_sav = periodic
        lcneg_sav = lcneg
        value_sav = value
        nperiod_sav = nperiods
      endif
      return
      end
