; ------------------------------------------------------------------------ 
; $Id: image_par.pro,v 1.3 2007/06/06 19:34:30 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
 Pro image_PAR, str1, str2, reflection = reflection, periodic = periodic, $
          again = again, $
          value = value, invert = invert, nperiods = nperiods, CLABEL = clabel
;+
; NAME:
;       image_con
;
; PURPOSE:
;       Generate an particle file from data which has a reflection plane or 
;         has periodic boundaries.
;
; CATEGORY:
;       Math 
; CALLING SEQUENCE:
;       IMAGE_PAR, STR1, [, REFLECTION = reflection]
;              [, PERIODIC = PERIODIC] [, VALUE = value] [, /INVERT]
;              [, NPERIODS = NPERIODS] [, CLABEL = clabel] [, /REPEAT]
; INPUTS:
;       STR1:     Structure array of the particles
; KEYWORDS:
;       REFLECTION : If present must equal to +/- 1, 2, or 3.
;                    + indicates maximum value; - indicates minimum value 
;                    for the reflection.
;                    1, 2, 3 indicates coordinates 1, 2, or 3
;       VALUE      : If a different value is required for the reflection, 
;                    value is used to specify this point. Value must be greater
;                    than the maximum spatial value of the data or less than
;                    the minimum to avoid multiple values at a point.
;       INVERT     : If present and not zero, multiple the field values by -1.
;       PERIODIC   : If present must equal to +/- 1, 2, or 3.
;                    + indicates maximum value; - indicates minimum value 
;                    for the translation.
;                    1, 2, 3 indicates coordinates 1, 2, or 3
;       NPERIODS   : Number of replications for the periodic data, default = 1
;       AGAIN      : If present and not zero, all parameters will be the same
;                    as the last call to IMAGE
;       CLABEL:  Optional label for the combined structures.
; OUTPUTS:
;       NONE    The composite image is stored in a structure array 
;                 in the common block
; RESTRICTIONS:
;       STR1 array must be defined and have valid field (or image) data
;
; MODIFICATION HISTORY:
;      Original version, L. P. Mix, April 21, 1994
;-
;
; insert a common block
;
 common image_com, reflect_sav, period_sav, lcneg_sav, value_sav, nperiod_sav
;
@qscom.dat
;  Common block
@qsstrcom.dat
;       return on an error
      on_error, 1
;
      if n_elements (structinit) eq 0 then qsstruct_init
      if n_elements (qsplotinit) eq 0 then qsinit
;
      nparams = n_params()
      if nparams lt 1 then begin
        print,' IMAGE_PAR must have 1 parameter'
        return
      endif
      b = size(str1)
      if  b(b(0)+1) ne 8 then begin
            print, $
            ' IMAGE_PAR: First parameter must be a conductor structure'
            return
       endif
;
      arr = str1
      if arr.type ne 2 then begin
        print, ' IMAGE_PAR may only be used with particle structures'
        return
      endif
      nblo = arr.block
      mxbl = nblo 
      ndim = arr.ndim
      if n_elements (reflect_sav) eq 0 then again = 0
      if n_elements (again) eq 0 then again = 0
      if again eq 0 then begin
        if n_elements (reflection) eq 0 then reflection = 0
        if n_elements (periodic) eq 0 then periodic = 0
        if n_elements (nperiods) eq 0 then nperiods = 1
        if (reflection eq 0 and periodic eq 0)  then begin
          print, 'Reflection or periodic dimension must be specified'
          return
        endif
      endif else begin
        reflection = reflect_sav
        periodic = period_sav
        if lcneg_sav ne 0 then begin
          reflection = -reflection
          periodic = -periodic
        endif
        value = value_sav
        nperiods = nperiod_sav
      endelse
      natt = arr.natt
      range = make_array(2, ndim)
      x1 = arr.x1
      x2 = arr.x2
      x3 = arr.x3
      x4 = arr.x4
      nvert = arr.nvert
      v1 = arr.v1
      v2 = arr.v2
      v3 = arr.v3
      v4 = arr.v4
      v5 = arr.v5
      v6 = arr.v6
      range(*, 0) = [min(x1, max=mp),mp]
      range(*, 1) = [min(x2, max=mp),mp]
      if ndim ge 3 then range(*, 2) = [min(x3, max=mp),mp]
      if ndim ge 4 then range(*, 3) = [min(x4, max=mp),mp]
      if reflection ne 0 then begin
        if abs(reflection) lt 1 or abs(reflection) gt ndim then begin
          print,'Reflection keyword must be in the range +/-1 to +/-', $
              strtrim(ndim, 2)
          return
        endif
        if reflection lt 0 then begin
          reflection = - reflection
          lcneg = 1
        endif else lcneg = 0
        err = 6.1e-5*max(abs(range(*, reflection-1)))
        if n_elements(value) eq 0 then begin
          if lcneg eq 0 then value = range(1, reflection-1) $
            else value = range(0, reflection-1)
        endif else begin
          if value gt range(0, reflection-1)+err and $
              value lt range(1, reflection-1)-err then begin
            print, $
            'Specified value for reflection must be outside the data range'
            return
          endif
        endelse
        periodic = 0
      endif else begin
        if abs(periodic) lt 1 or abs(periodic) gt ndim then begin
          print,'Periodic keyword must be in the range +/-1 to +/-', $
              strtrim(ndim, 2)
          return
        endif
        reflection = 0
        if periodic lt 0 then begin
          periodic =-periodic
          lcneg = 1
        endif else lcneg = 0
        err = 6.1e-5*max(abs(range(*, periodic-1)))
        if n_elements (value) eq 0 then begin
          delta = range(1, periodic-1)-range(0, periodic-1)
          value = delta
        endif else begin
          delta = range(1, periodic-1)-range(0, periodic-1)
          if delta -2.0*err gt value then begin
            print,  $
            ' Value of periodic spacing must be greater than the data range'
            return
          endif
          value = delta
        endelse
        pvalue = range(1, periodic-1)
      endelse
      if periodic ne 0 then begin
        nvert = nvert*(1+nperiods)
        for i = 1, nperiods do begin
          if periodic eq 1 then x1 = [x1, arr.x1+value] else x1 = [x1, arr.x1]
          if periodic eq 2 then x2 = [x2, arr.x2+value] else x2 = [x2, arr.x2]
          if ndim ge 3 then begin
            if periodic eq 3 then x3 = [x3, arr.x3+value] else x3 = [x3, arr.x3]
          endif
          if ndim ge 4 then begin
            if periodic eq 4 then x4 = [x4, arr.x4+value] else x3 = [x3, arr.x3]
          endif
          if natt ge 1 then v1 = [v1, arr.v1]
          if natt ge 2 then v2 = [v2, arr.v2]
          if natt ge 3 then v3 = [v3, arr.v3]
          if natt ge 4 then v4 = [v4, arr.v4]
          if natt ge 5 then v5 = [v5, arr.v5]
          if natt ge 6 then v6 = [v6, arr.v6]
        endfor
;
      endif else begin
        nvert = 2*nvert
        if reflection eq 1 then x1=[x1, 2*value-x1] else x1 = [x1, x1]
        if reflection eq 2 then x2=[x2, 2*value-x2] else x2 = [x2, x2]
        if ndim ge 3 then begin
          if reflection eq 3 then x3=[x3, 2*value-x3] else x3 = [x3, x3]
        endif
        if ndim ge 4 then begin
          if reflection eq 4 then x4=[x4, 2*value-x4] else x4 = [x4, x4]
        endif
        if natt ge 1 then v1 = [v1, v1]
        if natt ge 2 then v2 = [v2, v2]
        if natt ge 3 then v3 = [v3, v3]
        if natt ge 4 then v4 = [v4, v4]
        if natt ge 5 then v5 = [v5, v5]
        if natt ge 6 then v6 = [v6, v6]
      endelse
;
      if n_elements(clabel) gt 0 then clab = string(clabel) $
        else clab = arr.clab
;
       str1 = {type:2L, ndim:ndim, nvert: nvert, natt :natt ,block: 1L, $
              two: arr.two,  x1:x1, x2:x2, x3:x3, x4:x4, v1:v1, $
              v2:v2, v3:v3, v4:v4, v5:v5, v6:v6,x1lab : arr.x1lab, $
              x2lab : arr.x2lab, x3lab : arr.x3lab, x4lab : arr.x4lab, $
              v1lab : arr.v1lab, v2lab : arr.v2lab, v3lab : arr.v3lab, $
              v4lab : arr.v4lab, v5lab : arr.v5lab, v6lab : arr.v6lab, $
              file : arr.file, spare : arr.spare, tlab : arr.tlab, clab: clab}
;
      if again eq 0 then begin
        reflect_sav = reflection
        period_sav = periodic
        lcneg_sav = lcneg
        value_sav = value
        nperiod_sav = nperiods
      endif
;
      return
      end
