; ------------------------------------------------------------------------ 
; $Id: read_hisplt.pro,v 1.9 2009/02/13 17:07:11 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;

 pro read_hisplt, filename, time, dt, cpu, cycle, tday, wallc, jobid, code, $
      point_hist, point_var, point_unit, mat_hist, mat_var, mat_unit, $
      glob_hist, glob_var, glob_unit, pnt_id, mat_id, point_type, ntime, $
      nupts, numat, nupvar, numvar, nugvar, title, quiet = quiet

;+
;
;       READ_HISPLT
;
; PURPOSE:
;       Read a unformatted history file from ALEGRA
;
; CALLING SEQUENCE:
;
;       READ_HISPLT, FILENAME, TIME, DT, CPU, CYCLE, TDAY, WALLC, JOBID, CODE, $
;         POINT_HIST, POINT_VAR, POINT_UNIT, MAT_HIST, MAT_VAR, MAT_UNIT, $
;         GLOB_HIST, GLOB_VAR, GLOB_UNIT, MAT_ID, POINT_TYPE, NTIME, $
;         NUPTS, NUMAT, NUPVAR, NUMVAR, NUGVAR, TITLE, QUIET= quiet
;
; PARMAETERS:
;
;       FILENAME   - Name of the hisplt file.
;       TIME       - Array of problem time values for the data
;       DT         - Array of time step values at the corresponding times
;       CPU        - CPU time at the at the corresponding time of the dump
;       CYCLE      - Cycle number of the dump
;       TDAY       - Date at the beginning of the calculation
;       WALLC      - Time of day at the beginning of the calculation
;       JOBID      - Job identification
;       CODE       - Code name
;       POINT_HIST - Array of point history variables (npv, np, nt)
;                    where npv is the number of point variables 
;                          np is the number of points
;                          nt is the number of times
;       POINT_VAR  - Array of point variable names
;       POINT_UNIT - Array of point variable units
;       MAT_HIST   - Array of material history variables (nmv, nm, nt)
;                    where nmv is the number of material variables 
;                          nm is the number of material
;                          nt is the number of times
;       MAT_VAR    - Array of material variable names
;       MAT_UNIT   - Array of material variable units
;       GLOB_HIST  - Array of global variables (ng, nt)
;                    where ng is the number of global variables 
;                          nt is the number of times
;       GLOB_VAR   - Array of global variable names
;       GLOB_UNIT  - Array of global variable units
;       MAT_ID     - Array of numbers associated with each material
;       POINT_TYPE - Array of story point types
;       NTIME      - Number of times read
;                    Value of zero indicates a problem
;       NUPTS      - Number of history points
;       NUMAT      - Number of materials
;       NUPVAR     - Number of point variables
;       NUMVAR     - Number of material variables
;       NUGVAR     - Number of global variables.
;       TITLE      - Calculation title
;       QUIET      - If set, suppress informational print outs
; 
;-
;
on_error, 1

  ntime = 0L
  nupts = 0L
  numat = 0L
  nupvar = 0L
  numvar = 0L
  nugvar = 0L

  info = file_info(filename)
  if (not info.exists) then begin
    print, '% READ_HISPLT: File does not exist'
    print, '               ', filename
    return
  endif
  if (not info.read) then begin
    print, '% READ_HISPLT: File is not readable by the user'
    print, '               ', filename
    return
  endif
  openr, unit, /get_lun, filename, error = ier
  if (ier) then begin
    print, '% READ_HISPLT: Unable to open file'
    print, !err_string
    return
  endif 
  rec = 0L
  rec2 = rec
  tim = 0.0
  dtim = 0.0
  cput = 0.0
  cyc = 0L
  lrec= 8
  readu, unit, rec, tim, dtim, cput, cyc, varpts, varmat, varglb, rec2
  if rec ne rec2 then begin
    rec = long64(0)
    rec2 = rec
    lrec= 16
    free_lun, unit
    openr, unit, /get_lun, filename
    readu, unit, rec, tim, dtim, cput, cyc, varpts, varmat, varglb, rec2
  endif
  if rec ne 28 then begin
    free_lun, unit
    openr, unit, /get_lun, filename, /swap_endian
    readu, unit, rec, tim, dtim, cput, cyc, varpts, varmat, varglb, rec2
    if rec ne 28 then begin
      print, '% READ_HISPLT: First value should be: 28, value = ', rec
      free_lun, unit
      return
    endif
  endif
  if cyc ne -100 or rec ne rec2 then begin
    print, '% READ_HISPLT: First record is not correct:'
    print, 'Flag (-100), 28, 28:', cyc, rec, rec2
    free_lun, unit
    return
  endif
  
  remainder = info.size -lrec -rec
  readu, unit, rec, transfer = count
  remainder = remainder -lrec -rec
  title = make_array(rec, /byte)
  readu, unit, title, rec2, transfer = count
 
  title = strtrim(title, 2)

  nupts = 1L
  numat = 1L
  nupvar = 1L
  numvar = 1L
  nugvar = 1L
  nvarpts = 1L
  nvarmat = 1L
  nvarglb = 1L

  point_hist=0
  cycold =0
  while (not eof(unit))  do begin
    on_ioerror, finish
    readu, unit, rec, tim, dtim, cput, cyc
    if nvarpts gt 0 then  readu, unit, varpts
    if nvarmat gt 0 then  readu, unit, varmat
    if nvarglb gt 0 then  readu, unit, varglb
    readu, unit, rec2 
    remainder = remainder -lrec - rec
    if cyc ge 0 then begin
      if cyc eq 0 || cyc gt cycold then begin
        if ntime eq mxtime or rec ne rec2 then begin
          print, '% READ_HISPLT: NTIME equal MXTIME or bad record'
          print, ' Time, Dt, CPU, cycle: ' ,  tim, dtim, cput, cyc
          help, ntime, mxtime, rec, rec2
          goto, finish
        endif
        if nvarpts gt 0 then point_hist[*, *, ntime]= varpts
        if nvarmat gt 0 then mat_hist[*, *, ntime]= varmat
        if nvarglb gt 0 then glob_hist[*, ntime]= varglb
        time[ntime] = tim
        dt[ntime] = dtim
        cpu[ntime] = cput
        cycle[ntime] = cyc
        ntime = ntime +1
        cycold = cyc
      endif else if (cyc lt cycold) then begin
        print, '% READ_HISPLT: File appears to be corrupt.'
        print, '               Cycle '+strtrim(cyc, 2)+' appears after cycle '+strtrim(cycold, 2)
        return
      endif
    endif else begin
      if cyc eq -101 then begin
        varpts = varpts[0]
        varmat = varmat[0]
        varglb = varglb[0]
        readu, unit, rec
        if rec gt 0 then begin
          ss = make_array(rec, /byte)
          readu, unit, ss, rec2
          remainder = remainder -lrec - rec
          if n_elements(tday) eq 0 then begin
            tday  = string(ss[0:9])
            wallc = string(ss[10:19])
            jobid = string(ss[20:29])
            code  = string(ss[30:*])
          endif
        endif else begin
          print, '% READ_HISPLT:  Size of record for cycle = -101'
          help, rec
          
        endelse
      endif else if cyc eq -102 then begin
        readu, unit, rec, nupts, numat, nupvar, numvar, nugvar, rec2
        remainder = remainder -lrec - rec
        nvarpts = nupvar*nupts
        nvarmat = numvar*numat
        nvarglb = nugvar
        varpts = make_array((nupvar> 1), nupts>1)
        varmat = make_array((numvar> 1), numat>1)
        varglb = make_array(nugvar> 1)
        if rec ne rec2 then begin
          print, '% READ_HISPLT: Size of record for cycle = -102'
          help, rec, rec2
          free_lun, unit
          return
        endif
      endif else if cyc eq -103 then begin
        readu, unit, rec 
        pvarb = make_array(nupvar*16>1, /byte)
        mvarb = make_array(numvar*16>1, /byte)
        gvarb = make_array(nugvar*16>1, /byte)
        punitb = make_array(nupvar*16>1, /byte)
        munitb = make_array(numvar*16>1, /byte)
        gunitb = make_array(nugvar*16>1, /byte)
        pnt_id = make_array(nupts>1, /long)
        mat_id = make_array(numat>1, /long)
        point_type = make_array(nupts>1, /long)
        if nupvar gt 0 then readu, unit, pvarb
        if numvar gt 0 then readu, unit, mvarb
        if nugvar gt 0 then readu, unit, gvarb
        if nupvar gt 0 then readu, unit, punitb
        if numvar gt 0 then readu, unit, munitb
        if nugvar gt 0 then readu, unit, gunitb
        if nupts gt 0 then readu, unit, pnt_id
        if numat gt 0 then readu, unit, mat_id
        if nupts gt 0 then readu, unit, point_type
        readu, unit, rec2
        if rec ne rec2 then begin
          print, '% READ_HISPLT: Size of record for cycle = -103'
          help, rec, rec2
          free_lun, unit
          return
        endif
        remainder = remainder -lrec - rec
        if nupvar gt 0 then begin
          point_var = make_array(nupvar, /string)
          point_unit= make_array(nupvar, /string)
          for i = 0, nupvar-1 do $
            point_var[i] = strtrim(pvarb[i*16:i*16+15], 2)
          for i = 0, nupvar-1 do $
            point_unit[i]= strtrim(punitb[i*16:i*16+15], 2)
        endif else begin
          point_var = ''
          point_unit= ''
        endelse
        if numvar gt 0 then begin
          mat_var= make_array(numvar, /string)
          mat_unit= make_array(numvar, /string)
          for i = 0, numvar-1 do $
            mat_var[i]   = strtrim(mvarb[i*16:i*16+15], 2)
          for i = 0, numvar-1 do $
            mat_unit[i]  = strtrim(munitb[i*16:i*16+15], 2)
        endif else begin
          mat_var = ''
          mat_unit= ''
        endelse
        if nugvar gt 0 then begin
          glob_var= make_array(nugvar, /string)
          glob_unit= make_array(nugvar, /string)
          for i = 0, nugvar-1 do $
            glob_var[i]  = strtrim(gvarb[i*16:i*16+15], 2)
          for i = 0, nugvar-1 do $
            glob_unit[i] = strtrim(gunitb[i*16:i*16+15], 2)
        endif else begin
          glob_var = ''
          glob_unit= ''
        endelse
;        nvarpts = nupvar*nupts
;        nvarmat = numvar*numat
;        nvarglb = nugvar
        if n_elements(point_hist) lt 2 then begin
          blksize = (nvarpts+nvarmat+nugvar)*4L + 24
          mxtime = long(remainder/blksize)
          if nvarpts gt 0 then point_hist = make_array(nupvar, nupts, mxtime) $
            else point_hist = 0.0
          if nvarmat gt 0 then mat_hist = make_array(numvar, numat, mxtime) $
            else mat_hist = 0.0
          if nugvar gt 0 then glob_hist = make_array(nugvar, mxtime) $
            else glob_hist = 0.0
          time  = make_array(mxtime)
          dt    = make_array(mxtime)
          cpu   = make_array(mxtime)
          cycle = make_array(mxtime)

        endif else begin
          if nvarpts gt 0 then begin
          ss = size (point_hist)
          if ss[1] ne nupvar and ss[2] ne nupts then begin
            print, '% READ_HISPLT: Size of POINT_HIST changed'
            help, varpts, point_hist
            free_lun, unit
            return
          endif
          endif 
          if nvarmat gt 0 then begin
          ss = size(mat_hist)
          if ss[1] ne numvar and ss[2] ne numat then begin
            print, '% READ_HISPLT: Size of MAT_HIST changed'
            help, varmat, mat_hist
            free_lun, unit
            return
          endif
          endif
          if nvarglb gt 0 then begin
          ss = size(glob_hist)
          if ss[1] ne nugvar then begin 
            print, '% READ_HISPLT: Size of GLOB_HIST changed'
            help, varglb, glob_hist
            free_lun, unit
            return
          endif
          endif
        endelse
      endif else begin
        print, '% READ_HISPLT: Invalid cycle =' , cyc
        free_lun, unit
        return
      endelse
    endelse
  endwhile
  if not keyword_set(quiet) then print, 'Normal exit : ', filename
FINISH:
  free_lun, unit
;
; help, ntime, mxtime
  if ntime eq mxtime then return
; print, 'Reduce arrays : ntime, mxtime', ntime, mxtime
;
  if nvarpts gt 0 then point_hist = point_hist[*, *, 0:ntime-1]
  if nvarmat gt 0 then mat_hist = mat_hist[*, *, 0:ntime-1]
  if nvarglb gt 0 then glob_hist = glob_hist[*, 0:ntime-1]
  time  = time[0:ntime-1]
  dt    = dt[0:ntime-1]
  cpu   = cpu[0:ntime-1]
  cycle = cycle[0:ntime-1]
  end
