; ------------------------------------------------------------------------ 
; $Id: reduce_array.pro,v 1.3 2007/06/06 19:34:32 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
 Function reduce_array, arrayin,dim, y, yvalin, integ
;+
;  Reduce by one the number of dimensions of an array at a given value or 
;    over a specified range of values, or integrate over a range
;  Note: yvalin must have two values if integ is not equal to zero
;          Integral is defined according to the following:
;          integ(0) = init
;          integ(i) = integ(i-1) + 
;           (A(i)+A(i-1))(x(i)-x(i-1))/2.0 * {((x(i)+x(i-1))/2.0)^(integ-1)}
;            where A(i) is the value of the function at x(i)
;  Format is: 
;        newarray = reduce_array(array,dim, yarray, yvalue [, integ] )
;    where;
;      NEWARRAY- array with reduce number of dimensions
;      ARRAY   - original array
;      DIM     - Dimension to be eliminated
;      YARRAY  - Spatial coordinates for the eliminated dimension
;      YVALUE  - Spatial coordinate at the location for the slice to be taken
;                or two values - a minimum and maximum over which the array 
;                is averaged
;      INTEG - Indicates an integral is to be performed; Default = 0.
;              Values may be 1,2, 3 to indicate if the integral is to be
;                Int(dy), int(y*dy), int(y*y*dy)
;  Example:
;    Eliminate the second dimension from PICTURE. Spatial values are Y and 
;      the slice is to be taken between y = 1.0 and 1.6
;      array  = reduce_array(picture,2, y, [1.0, 1.6])
;-
     array = arrayin
     if n_elements (array) eq 0 then begin
       Print, 'REDUCE_ARRAY: Multi-dimensional array must be defined'
       return, 0.0
     endif
     b = size (array)
     if b[0] eq dim-1 then return, reform (array)
     if b(0) lt dim  then begin
       print,'REDUCE_ARRAY:'
       print,'   Cannot eliminate dimension ',strtrim (dim, 2),$
         ' with a ',strtrim(b(0), 2),' dimensional array'
       return, 0.0
     endif
;    if the array is already degenerate in this dimension then quit
;
     if b(dim) eq 1 then return, reform (array)
     ny = n_elements(y)
     if ny ne b(dim) then begin
       print,'REDUCE_ARRAY:'
       print, ' Ordinate array must be defined and have the same ' 
       print, ' number of element as the array'
       return, 0.0
     endif
     nyval = n_elements (yvalin)
     if nyval eq 0 then begin
       print,'REDUCE_ARRAY:'
       print, ' Ordinate value for the lineout must be defined'
       return , 0.0
     endif
     if n_elements (integ) eq 0 then integ =0
     if integ lt 0 or integ gt 3 then begin
       print, 'REDUCE_ARRAY: Integral flag must be in the range [1,3]'
       print, '              Integral flag set to 1'
       integ = 1
     endif
     if integ gt 0 and b(0) ne 3 then begin
       print, 'REDUCE_ARRAY: Integral flag limited to 3D arrays'
       return, 0.0
     endif
     if dim eq 1 then begin
       i= b(2) & j = b(3) & k = b(1)
     endif else if dim eq 2 then begin
       i= b(1) & j = b(3) & k = b(2)
     endif else begin
       i= b(1) & j = b(2) & k = b(3)
     endelse
;
     yval = yvalin
     if n_elements (yval) eq 1 then width = 0 else begin
       yl = min (yval, max = yu) > y(0)
       yu = yu < y(ny-1)
       if yu le yl then begin
         if integ gt 0 then begin
           if yl gt y(ny-1) then $
             return, reform(select_dim(array,b(0),dim,ny-1))
           if yu lt y(0) then $
             return,  reform(select_dim(array, b(0),dim, 0))
;          return a zero array
         endif else return, make_array(i, j)
       endif
       width = yu-yl
       yval = 0.5 * (yu+yl)
     endelse
     if integ gt 0 then begin
       iymin = min(abs(y-yl), iyl)
       iymax = min(abs(y-yu), iyu)
       if y(iyl) gt yl then iyl= iyl-1>0
       if y(iyu) lt yu then iyu= iyu+1<k
       knew = 1+iyu-iyl
       if dim eq 3 then begin
         array = z_gridinteg(array(*, *, iyl:iyu), y(iyl:iyu), integ-1)
       endif else if dim eq 2 then begin
         temp = make_array(i, j, knew)
         for i = iyl, iyu do temp(*,*,i-iyl) = reform(array(*, i, *))
         array = z_gridinteg(temp, y(iyl:iyu), integ-1)
       endif else begin
         temp = make_array(i, j, knew)
         for i = iyl, iyu do temp(*,*,i-iyl) = reform(array(i, *, *))
         array = z_gridinteg(temp, y(iyl:iyu), integ-1)
       endelse
       d1 = y(iyl+1)-y(iyl)
       d2 = y(iyu)-y(iyu-1)
       r1 = (yl-y(iyl))/d1
       r2 = (y(iyl+1)-yl)/d1
       r3 = (yu-y(iyu-1))/d2
       r4 = (y(iyu)-yu)/d2
       array = r4*array(*,*,knew-2) -r2*array(*,*,0) -r1*array(*,*,1) + $
          r3*array(*,*, knew-1)
       return, reform(array)
     endif
     width2 = width*0.5
     if (width ne 0) then begin
       delta = min(abs(y-yl), jlow)
       delta = min(abs(y-yu), jhi)
;
;       if width is less than the local point spacing use width = 0
;       but don't change width because it is a passed parameter
;
        if y(jhi)-y(jhi-1) ge width then begin
         width2 = 0
         delta = min(abs(y-yval), jmid)
        endif
     endif else delta = min(abs(y-yval), jmid)
;
     if width2 eq 0 then begin
       if jmid eq 0 and yval le y(0) then return, $
             reform(select_dim(array, b(0),dim, 0))
       if jmid eq ny-1 and yval ge y(ny-1) then $
          return, reform(select_dim(array,b(0),dim,ny-1))
       if yval ge y(jmid) then begin
         delta = y(jmid+1)-y(jmid)
         r1 = (yval-y(jmid))/delta
         r2 = 1-r1
         return,reform( r2*select_dim(array, b(0),dim,jmid)+ $
            r1*select_dim(array, b(0),dim, jmid+1))
       endif else begin
         delta = y(jmid)-y(jmid-1)
         r1 = (y(jmid)-yval)/delta
         r2 = 1-r1
         return,reform ( r2*select_dim(array, b(0),dim,jmid)+ $
            r1*select_dim(array, b(0),dim, jmid-1))
       endelse
     endif else begin
       dx = y(1:ny-1)-y(0:ny-2)
       dx2 = dx/2.
       ygap = y(0:ny-2)+dx2
       w = ygap(jlow)-yl
       wt = w
       a = select_dim(array, b(0),dim, jlow)*w
       if jlow +1 lt jhi then begin
         for i = jlow+1, jhi-1 do begin
           w = ygap(i)-ygap(i-1)
           wt = wt + w
           a = select_dim(array, b(0),dim, i)*w+a
         endfor
       endif
       w = yu- ygap(jhi-1)
       wt = wt+w
       return, reform(select_dim(array, b(0),dim,jhi)*w+a)/wt
     endelse
     end
