; ------------------------------------------------------------------------ 
; $Id: x_gridinteg.pro,v 1.4 2007/06/06 19:34:34 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
 Function x_gridinteg, array, xin, init, flagin, reverse = reverse
;
;+
;
;        integrate an array in the x direction and return the integral
;        assume that the array is a grid array with values of the function
;        at the end points 
;          Integral is defined according to the following:
;          integ(0) = init
;          integ(i) = integ(i-1) + (A(i)+A(i-1))(x(i)-x(i-1))/2.0
;            where A(i) is the value of the function at x(i)
;        format: integ = x_gridinteg(array, x, init, flag)
;        where;
;          Array is the two dimensional array to be integrated
;          x     is the array of x values for array
;          init is the initial value of the integral; this is usually 
;               zero unless there is a previous value of the integral
;               If init is a scalar or undefined then init = zero
;          flag  is an integration flag - default is 1
;               integ = integral(array * dx)             flag = 1 
;               integ = integral(array * x dx)           flag = 2 
;               integ = integral(array * x*x dx)         flag = 3 
;             or 
;               is a string executable which can be executed to produce 
;               a function of X. Then 
;                integ = integral(array*g(x) dx)
;               where g(x) = flag
;                 examples are: flag = 'sin(x)'
;                               flag = 'x*sqrt(x)'
;                               flag = 'my_function(x, 4.5)'
;                  (flag may be any executable idl or user function)
;
;               Note: either x or y may be used in the function call but y=x
;              ie: flag = 'sin(x)' gives the same answer as flag = 'sin(y)'
;                  Both x and y will be set to the integration variable
;                  Use chastr to multiply the array by coordinate value.
;          reverse - if set, integrate from larger values
;-
 COMPILE_OPT hidden
 on_error, 2
         if n_elements (array) eq 0 or n_elements (xin) eq 0 then begin
           print, ' X_GRIDinteg - Array and abscissa must be defined'
           return, 0.0
         endif
         a = size (array)
         if a(0) ne 2 or a(1) ne n_elements (xin) then begin
           Print, ' X_GRIDinteg ERROR Array must be two dimensional with same'
           print, ' number of elements in first dimension as x'
           return, 0.0
         endif
         nx = a(1)
;   calculate dx/2
         dx2 = 0.5*(xin(1:nx-1)-xin(0:nx-2))
;    calculate the average value of the function between x values
         aarray= array(0:nx-2, *) + array(1:nx-1, *)
;    check reverse
         if keyword_set (reverse) then rev = nx-1 else rev = 0
;    check on initial value
         if n_elements (init) lt a(2) then array(rev,*)=0 else $
               array(rev,*)=init(0:a(2)-1)
         if n_elements (flagin) eq 0 then flag = 1 else flag = flagin(0)
         ss = size(flag(0))
         if ss(1) eq 7 then begin
           x =  xin(0:nx-2)+dx2
           y = x
           ok  = execute('tempx ='+flag) 
           ntemp = n_elements(tempx)
           if ntemp lt (nx-1) then begin
             print, 'Integral kernel is too short. Additional values will be' $
                     + ' added'
             if ntemp gt 1 then begin
               temp = tempx
               tempx= make_array(nx-1, value = tempx(0))
               tempx(0:ntemp-1) = temp
             endif else tempx= make_array(nx-1, value = tempx(0))
           endif
           if keyword_set (reverse) then $
           for ix=nx-2, 0, -1  do $
              array(ix,*)=array(ix+1,*)-aarray(ix,*)*dx2(ix)*tempx(ix) $
           else for ix=0,nx-2 do $
              array(ix+1,*)=array(ix,*)+aarray(ix,*)*dx2(ix)*tempx(ix)
           x=0 & y =0
           return, array
         endif else if flag gt 3 or flag lt 1 then begin
           PRINT, ' INTEG flag for X_GRIDinteg must be in the range 1, 3'
           print, ' Flag set to one'
           flag = 0
         endif else flag = round(flag)-1
         if flag eq 0 then begin
           if keyword_set (reverse) then $
           for ix=nx-2, 0, -1  do $
               array(ix,*)=array(ix+1,*)-aarray(ix,*)*dx2(ix) $
           else for ix = 0, nx-2  do $
               array(ix+1,*)=array(ix,*)+aarray(ix,*)*dx2(ix)
         endif else begin
           tempx = xin(0:nx-2)+dx2
           if flag gt 1 then tempx = tempx*tempx
           if keyword_set (reverse) then $
           for ix=nx-2, 0, -1  do $
              array(ix,*)=array(ix+1,*)-aarray(ix,*)*dx2(ix)*tempx(ix) $
           else for ix=0,nx-2 do $
              array(ix+1,*)=array(ix,*)+aarray(ix,*)*dx2(ix)*tempx(ix)
         endelse
         return, array
         end
