; ------------------------------------------------------------------------ 
; $Id: rm_nl_skew.pro,v 1.6 2010/08/09 16:41:49 lpmix Exp $
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
;
;+
; NAME:
;       RM_NL_SKEW
; PURPOSE:
;       Proceedure for removing warp in an image
;    Note:  See RM_SKEW for a linear skew.
;       Procedure uses linear interpolation or nearest neighbor or in the skew
;       direction.  The skew is adjusted to provide an average value of zero.
;      
; CATEGORY:
;       VIDA - Image manipulation
; CALLING SEQUENCE:
;       RM_NL_SKEW, STRIN [, STROUT] [, FIDU_VALUES = wdf] $
;       [, COEFFICIENTS = coefficients] [, /XAXIS] 
;       [, CUBIC = cubic] [, SIGMA = sigma] [, POLYGON = polygon]
;       [, DELTA_FRACTION = delta]
;       [, INTERPOLATION = interpolation] [, _EXTRA = ExtraStuff]
; INPUTS:
;       STRIN  - Structure array with the image requiring the camera correction.
; OPTIONAL INPUT PARAMETERS:
;       STROUT - Structure array with the corrected image
;                DEFAULT: STRIN
; KEYWORD PARAMETERS:
;       XAXIS  - If set, the skew correction will be in the X-direction.
;                Default: XAXIS = 0 ; skew correction in the Y-direction vs X.
;       FIDU_VALUES- WDF array with the X and Y values of a fiducial expected
;                to be straight in the corrected image.
;       COEFFICIENTS - Coefficients of the least square fit for location of the
;                fiducial.  COEFFICIENTS is ignored if FIDU_VALUES is present.
;       INTERPOLATION - 0 (DEFAULT) Nearest neighbor interpolation
;                       1 Linear Interpolation
;    The following keywords apply only if FIDU_VALUES is passed.
;       CUBIC  - If set, a cubic spline is used to interpolate fiducial values.
;       SIGMA  - The amount of tension that is applied to the curve. 
;                The default value is 1.0.  If sigma is close to 0, 
;                (0.01, for example) then there is a cubic spline fit.  If sigma
;                is large, (greater than 10) then the fit will be a polynomial
;                interpolation.
;       POLYGON- If cubic is not set, then a polynomial filter will be used
;                with a degree equal to polygon.   
;       If neither CUBIC nor POLYGON are set, then CUBIC = 1
;       QUIET  - This will be passed to fit_data.
;       DELTA_FRACTION- Filter length for the interpolated skew values as a 
;                fraction of the X or Y-axis length.  The routine FILT is used.
;                Keyword DELTA in FILT = DELTA_FRACTION * RANGE
;                  where RANGE is the axis range of STRIN
;                Default: Delta_Fraction = 0.3
;                If Delta_Fraction le 0 then, no filter is applied.
;
; OUTPUTS:
;       Image Skew Corrected Image
; OPTIONAL OUTPUT PARAMETERS:
;       None
; COMMON BLOCKS:
;       None
; SIDE EFFECTS:
;       Completely rewrites the image
; RESTRICTIONS:
;       Assumes the input image has rectangular pixels, single block,
;       Type 1 structure, with slice in the 3 dimension.
; PROCEDURE:
;       If CUBIC or POLYGON is used, the interpolation is made to determine
;         the skew at each pixel.  FIT_DATA is used to perform the interpolation
;       If COEFFICIENTS is specified, the image skew is determined from 
;         those values.
;       The image is then corrected according to the value of INTERPOLATION
;    
;  Example:
;       Correct the skew in image stored in structure array 2 data in
;       wdf array = 4
;         RM_NL_SKEW, 2, fidu=4
;
;       Correct the skew in image stored in structure array 1 from the 
;       calculated coefficients of a least square fit in COEF.
;         RM_NL_SKEW, 1, coefficients = coef
;
; MODIFICATION HISTORY:
;       Replaces a routine in VIDA written by W. E. Nelson, 6/1991
;       and modified by Alex Filuk, 7/1992
;       Rewritten:  L. P. Mix, January 14, 2004
;-
; ******************************************************************************

PRO rm_nl_skew, strin, strout, fidu_values = wdf, $
      coefficients = coef,  xaxis = xaxis,  delta_fraction = delta, $
      cubic = cubic, sigma = sigma, polygon = polygon,  $
      quiet = quiet, interpolation = interpolation
;
@qscom.dat
;
@qsstrcom.dat
;     return on an error
      on_error, 1
;
;
if n_elements (structinit) eq 0 then qsstruct_init
if n_elements (qsplotinit) eq 0 then qsinit
;
 nparams = n_params()
 if nparams eq 0 then begin
   print,'% Format: ', $
     'RM_NL_SKEW, STRIN [, STROUT]  [, FIDU_VALUES = wdf] [, /XAXIS]'
   print,'            [, COEFFICIENTS = coefficients] [, POLYGON = polygon]'
   print,'            [, CUBIC = cubic] [, SIGMA = sigma]'
   print,'            [, DELTA_FRACTION = delta]'
   print,'            [, INTERPOLATION = interpolation]'

   return
 endif
 if n_elements(strin) gt 0 then ain = strin[0]
 a =  get_valid_structure(ain, 'RM_SKEW', lcstr=lcstr, ierror=ier)
 if ier ne 0 then return
 if a.type ne 1 or a.block ne 1 or a.two ne 3 then begin
   print, '% RM_NL_SKEW: Only type 1, single block structures can be corrected'
   print, '           Structure type:', a.type
   print, '           Structure Blocks:', a.block
   print, '           Slice dimension:', a.two
   return
 endif
 ok = where(strpos(tag_names(a), 'DX') ge 0 , nok)
;  Note the following IF-block must be modified if the a.two ne 3.
 if nok gt 0 then begin
   if (a.dx2 eq  0) or (a.dx1 eq 0) then begin
     print, '% RM_NL_SKEW: Uniform PIXELS are assumed.'
     print, '           DX1, DX2:', a.dx1, a.dx2
   endif
 endif else print, '% RM_NL_SKEW: Not able to check PIXEL size'

 if a.type ne 1 then begin
   Print, '% RM_NL_SKEW: Only type 1 structures are permitted.'
   return
 endif

 if a.block ne 1 then begin
   Print, '% RM_NL_SKEW: Only single block structures are permitted.'
   return
 endif


 if (a.ndim gt 2 and a.two eq 0) or (a.ndim gt 3) then begin
   Print, '% RM_NL_SKEW: Only two dimensional structures are permitted'
   return
 endif

 array = reform(get_array(a, -1))
 if keyword_set (xaxis)  then lcxaxis = 1 else lcxaxis = 0
 if a.ndim eq 2 then begin
   xval = 1
   yval = 2
 endif else if a.two eq 2 then begin
   xval = 1
   yval = 3
   if keyword_set(lcxaxis) then lcxaxis = 0  else lcxaxis = 1
 endif else begin
   xval = (a.two mod 3) +1
   yval = ((a.two +1) mod 3)+1
 endelse
 if n_elements (wdf) eq 0 and n_elements (coef) eq 0 then begin
   print, '% RM_NL_SKEW: WDF array or coefficients must be provided.'
   return 
 endif
;
; determine dx in the direction of the skew correction and in the 
;   direction of the variation
; 
 if keyword_set (lcxaxis) then xxval = yval else xxval = xval
 if xxval eq 1 then dx = a.dx1[0] else if xxval eq 2 then dx = a.dx2[0] else $
   dx = a.dx3[0]
 if keyword_set (lcxaxis) then yyval = xval else yyval = yval
 if yyval eq 1 then dy = a.dx1[0] else if yyval eq 2 then dy = a.dx2[0] else $
   dy = a.dx3[0]
 

 xarr = get_array (a, xxval)
 if n_elements (delta) eq 0 then delta =  0.3
 if keyword_set (polygon) then type = 1 else type = 0
 wd = wdvalid(wdf, /valid, /quiet)
 if wd[0] gt 0 then begin
   fit_data, wd[0], xarr, yarr, tension = sigma, degree = polygon, $
          type = type, filter = delta, quiet = quiet
 endif else begin
   if n_elements( coef) gt 1 then yarr = poly (xarr, coef) else begin
     print, '% RM_NL_SKEW: Valid WDF array must be provided or a coef array'
     help, wdf, coef
     return
   endelse
 endelse
 
 miny = min(yarr)
 yarr= yarr-miny
 yarr = yarr/float(dy)
;
; Normalize the shift so the average is zero
;
 yarr = yarr - (total(yarr, /double)/n_elements(yarr))

;  transpose image if not lcxaxis set
 if not keyword_set(lcxaxis) then array = transpose(array)
 ss = size(array)
 nx = ss[1]
 nxm = nx-1
 ny = ss[2]
;
 if keyword_set(interpolation) then begin
;  interpolate
   fill = findgen(nx)
   for i = 0, ny-1 do begin
     val = yarr[i] + fill
     array[0, i] = interpolate(reform(array[*, i]), val)
   endfor
 endif else begin
;  nearest neighbor
   fill = lindgen(nx)
   yarr= round(yarr)
   for i = 0, ny-1 do begin
     val = ((yarr[i] +fill) > 0) < (nx-1)
     array[0, i] = array[val, i]
   endfor
 endelse
 if not keyword_set(lcxaxis) then array = transpose(array)
 put_array, a, array, -1
 if nparams eq 2 then begin
   if lcstr eq 0 then begin
     strout = get_save_str(strout, strin)
     i2s, a, strout, /no_copy
   endif else strout = temporary(a)
 endif else begin
   if lcstr eq 0 then i2s, a, strin, /no_copy else strin = temporary(a)
 endelse
END
