;
; ------------------------------------------------------------------------ 
; $Id: rm_refrac.pro,v 1.3 2007/06/06 20:13:25 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
;
;+
; NAME:
;       RM_REFRAC
; PURPOSE:
;       Proceedure for removing time dispersion due to index of refraction
;         in the fiber optic cable
;       This procedure uses the nearest point interpolation used in RM_REFRAC
; CATEGORY:
;       VIDA - Image manipulation
; CALLING SEQUENCE:
;       RM_REFRAC, STRIN [, STROUT] [, REFRACTION = refrac]
;          [, CURRENT = current] [, LENGTH = length]
;          [, WAVELENGTH = wavelength] [, POSITION = position]
;          [, DISPERSION = dispersion] [, SWEEP = sweep]
;
; INPUTS:
;       STRIN  - Structure array with the image requiring the camera correction.
;                CHA_VIDA should be used on raw data to convert the axis scale
;                  to MM.
; OPTIONAL INPUT PARAMETERS:
;       STROUT - Structure array with the corrected image
;                DEFAULT: STRIN
; KEYWORD PARAMETERS:
;       REFRACTION - File name for the refraction correction
;                    REFRACTION may be a string or a number if it uniquely 
;                      specifies the file
;                    File should be a wdfit3 data file.
;                    If file not found, then the data directory for 
;                      VIDA is searched
;                    Units are Seconds vs nanometers.
;       CURRENT    - If set, "fidmark.mnu" will be written to current directory.
;                    Current directory will be searched for default 
;                    parameters in the fidmark.mnu
;                    If fidmark.mnu does not exist, it will be read from the 
;                    VIDA data directory
;
;       Note: The following image parameters, if unspecified, 
;               must be provided in a widget.
;             Initial value is from "fidmark.mnu" in data directory for the 
;               actual data or in the current directory, depending on the keyword
;               CURRENT, or in the data directory for VIDA (*/pfidl/pfidl/vida/data)
;             Format for this file is:
;              "String data anything"
;             Each line is parsed on spaces.
;             The first string should be a string of characters beginning with:
;                LENGTH, WAVELENGTH, POSITION, DISPERSION, or SWEEP
;                in upper or lower case
;                Any line which does not begin with one of these words is ignored
;             The next word is converted to a float representing one of the 
;                following keywords.  Additional data on a line is ignored.
;             If all five keywords are present, no widget is shown.
;             If any of these keywords are present, the data in "fidmark.nmu"
;               is not used for those keywords.
;             If multiple values exist, the last value read is used for the menu
;       LENGTH     - Lenght of the fiber optic in meters
;       WAVELENGTH - Wavelength of the reference wavelength (Angstroms)
;       POSITION   - Position of the reference wavelength
;       DISPERSION - Streak Camers dispersion (Angstroms/mm)
;       SWEEP      - Camera sweep rate (ns/mm)
;
; OUTPUTS:
;       Streak camera image in units of Angstroms and ns corrected for 
;         wavelength dispersion in the fiber optic.
;       If possible, the parameters for the analysis are written to the shot
;         directory.  If the file exists, it will be opened for append.
; OPTIONAL OUTPUT PARAMETERS:
;       None
; COMMON BLOCKS:
;       None
; SIDE EFFECTS:
;       Completely rewrites the image
; RESTRICTIONS:
;       Camera calibrations must be stored in the data directory or 
;          some other file.  
;       Streak camera dispersion and sweep must be known
;       Current version assumes the input image has square pixels, single block
;       Type 1 structure, single block, with slice in the 3 dimension.
; PROCEDURE:
;       Read the file and calculate time shift for all values of wavelength
;       Convert time shift to pixels with an average offset of zero.
;       for j = 0, ny-1 do new_image[0, j] = image[*, j+yoffset]
;    
;  Example:
;       Correct the skew in image stored in structure array 2 from camera 19
;         RM_REFRAC, 2, camera = 19
;
;       Correct the skew in image stored in structure array 1
;         RM_REFRAC, 1
;
; MODIFICATION HISTORY:
;       Replaces a routine in VIDA written by Kyle Cochran
;         Calculational techinque has changed from Vida but
;         all calibrations are from Kyle Cochran.
;       Original Version of this implementation:  L. P. Mix, Jr. 
;                          February 5, 2003
;-
; ******************************************************************************


PRO rm_refrac, strin, strout, REFRACTION = camera, LENGTH = length, $
          WAVELENGTH = wavelength, POSITION = position,  $
          CURRENT = current_dir, DISPERSION = dispersion, SWEEP = sweep
;
@qscom.dat
;
@qsstrcom.dat
;     return on an error
      on_error, 1
;
;
if n_elements (structinit) eq 0 then qsstruct_init
if n_elements (qsplotinit) eq 0 then qsinit
;
 nparams = n_params()
 if nparams eq 0 then begin
   print,'% Format: ', $
     'RM_REFRAC, STRIN [, STROUT] [, REFRACTION = refrac] 
   print,'           [, LENGTH = length]   [, WAVELENGTH = wavelength]
   print,'           [, POSITION = position] [, DISPERSION = dispersion]
   print,'           [, SWEEP = sweep]
   return
 endif
 if n_elements(strin) gt 0 then ain = strin[0]
 a =  get_valid_structure(ain, 'RM_REFRAC', lcstr=lcstr, ierror=ier)
 if ier ne 0 then return
 if a.type ne 1 or a.block ne 1 or a.two ne 3 then begin
   print, '% RM_REFRAC: Only type 1, single block structures can be corrected
   print, '           Structure type:', a.type
   print, '           Structure Blocks:', a.block
   print, '           Slice dimension:', a.two
   return
 endif
 ok = where(strpos(tag_names(a), 'DX') ge 0 , nok)
 if nok gt 0 then begin
   if (a.dx1 ne a.dx2) or (a.dx1 eq 0) then begin
     print, '% RM_REFRAC: Square uniform pixels are assumed
     print, '           DX1, DX2:', a.dx1, a.dx2
   endif
 endif else print, '% RM_REFRAC: Not able to check for square pixels'
 
 if strpos(strupcase(a.x1lab[0]), 'MICRONS') ge 0 then begin
   print, '% RM_REFRAC: Data appears to be in microns'
   print, '             Use CHA_VIDA to convert to MM'
   print, '          or CHASTR to convert the XLABEL'
   return
 endif

; make a flag array to determine if all data is provided
 lc_data = make_array(5, /int)
 if n_elements(length) eq 0 then length = 0.0 else length = length[0]
 if length gt 0 then lc_data[0]=1

 if n_elements(wavelength) eq 0 then wavelength = 0.0 else $
   wavelength = wavelength[0]
 if wavelength gt 0 then lc_data[1]=1

 if n_elements(position) eq 0 then position = 0.0 else $
   position = position[0]
 if position gt 0 then lc_data[2]=1

 if n_elements(dispersion) eq 0 then dispersion = 0.0 else $
   dispersion = dispersion[0]
 if dispersion gt 0 then lc_data[3]=1

 if n_elements(sweep) eq 0 then sweep = 0.0 else sweep = sweep[0]
 if sweep gt 0 then lc_data[4]=1

; get default values for the image parameters not present in keywords
 fname = a.file
 filename = ''
 datadir = ''
 menu_file =''
 
 if keyword_set(current_dir) then begin
   pwd, current = fidmark_dir, /quiet
   menu_file = fidmark_dir+path_sep()+'fidmark.mnu'
   fid_file = menu_file
 endif  else if strlen (fname) gt 2 then begin
   filename = get_bare_filename(fname, directory = datadir)
   resolve_it, datadir
   ls, datadir+'fidmark.mnu', current = current, /quiet
   if strlen(current[0]) gt 2 then menu_file = current[0]
   fid_file = datadir+'fidmark.mnu'
 endif
 which, '^RM_REFRAC$', /quiet, output = output, noutput = noutput
 if noutput ne 1 then begin
   print, '% RM_REFRAC: Problem finding VIDA data directory.'
   vidadir = ''
 endif else begin
   str = (strsplit(output[0], /extract))[1]
   s = get_bare_filename(str, directory = vidadir)
   vidadir = vidadir+'data/'
   resolve_it, vidadir
   if strlen(menu_file) lt 2 then menu_file = vidadir+'fidmark.mnu'
 endelse    
 menu_val= [LENGTH, WAVELENGTH, POSITION, DISPERSION, SWEEP]
 if total(lc_data) lt 5 then begin
   if strlen(menu_file) gt 1 then begin
     openr, unit, /get, menu_file, error = err
     If err ne 0 then print, '% RM_REFRAC: Error opening menu file' $
     else begin
       str = ''
       valid = ['LENGTH', 'WAVELENGTH', 'POSITION', 'DISPERSION', 'SWEEP']
       nelvalid = n_elements(valid)
       nvm = nelvalid-1
       while (not eof(unit)) do begin
         readf, unit, str
         vals = strsplit(str, /extract)
         key = strupcase(vals[0])
         j = 0
         while strpos(key, valid[j< nvm]) ne 0 and j lt nelvalid do j = j+1
         if j lt nelvalid  then if lc_data[j] ne 1 then $
           menu_val[j]=float(vals[1])
       endwhile
       free_lun, unit
     endelse
   endif
   Labels = ['Fiber Length(m)', 'Fiducial Wavelength(Angstroms)', $
        'Fiducial Position(mm)', 'Wavelength Dispersion (Angstroms/mm)', $
        'Streak Sweep Rate (ns/mm)']
   Instructions ='Enter the values for streak camera image'
   get_analysis_values, lcquit, labels, menu_val, instructions = instructions
   if lcquit eq 1 then return
 endif

; get the x values
  nx = a.size[0]
  ny = a.size[1]
  ok = where(strpos(tag_names(a), 'DX') ge 0 , nok)
  if nok gt 0 then begin
    if a.dx1 gt 0 then begin
      a.dx1[0] = a.dx1[0]*menu_val[3]
      x0 =  (a.x1[0] - menu_val[2])*menu_val[3] + menu_val[1]
      a.x1 = x0 + findgen(nx)*a.dx1[0]
    endif else nok = 0
  endif
  if nok eq 0 then $
    a.x1 =  (a.x1 - menu_val[2])*menu_val[3] + menu_val[1]
  xdata = a.x1
  a.x1lab = 'Wavelength (Angstroms)'

; scale the y values
  ok = where(strpos(tag_names(a), 'DX') ge 0 , nok)
  if nok gt 0 then begin
    if a.dx2 gt 0 then begin
      a.dx2[0] = a.dx2[0]*menu_val[4]
      y0 =  a.x2[0]* menu_val[4]
      a.x2 = y0 + findgen(ny)*a.dx2[0]
      dy_val = a.dx2[0]
    endif else nok = 0
  endif
  if nok eq 0 then a.x2 =  a.x2 *menu_val[4]

  if n_elements(dy_val) eq 0 then dy_val = (a.x2[ny-1]-a.x2[0])/(ny-1)
  
; x-values in Nanometers
;
  xdata = a.x1 *0.1
  a.x1lab = 'Wavelength (Angstroms)'
  a.x2lab = 'Time (ns)'
  a.range[0,0,*] = min(a.x1, max = mx)
  a.range[1,0,*] = mx
  a.range[0,1,*] = min(a.x2, max = mx)
  a.range[1,1,*] = mx



 ls, vidadir+'*.refrac', /quiet, current= current
 nnames = n_elements(current)
 names = make_array (nnames, /string)
 j = strlen (vidadir)
 if strlen(current[0]) gt 0 then begin
   for i = 0, nnames-1 do $
       names[i] = strmid(current[i], j, strlen(current[i])-j-7)
   if n_elements(camera) gt 0 then begin
     it = get_match(strtrim(camera[0], 2), names[0:nnames-1], $
            nelements= nel, regexp =0)
     if nel ne 1 then it = -1 else it = it[0]
   endif else it = -1
 endif
 if it eq -1 and strlen(current[0]) gt 0 then $
   pick_one, names, it, $
     instructions = 'Select the calibration for the REFRAC correction', $
       title='Select REFRAC Correction'
;  Read slope from a file
 readwd3, current[it], xdata = xdata, ydata = ydata, ier = ier
 if ((n_elements(ydata) lt n_elements(xdata)) or (ier ne 0)) then begin
   print, '% RM_REFRAC: Error reading file: ', current[it]
   Print, '% RM_REFRAC: No refraction correction performed'
   help, ier
   return
 endif

 v1 = reform(a.v1, nx, ny, /overwrite)

;  convert seconds to pixels below
;  ydata = ydata/dy_val
;  nearest neighbor
;  offset by 1/2 
   av = (min(ydata, max = m)+m)*0.5
   iy = round((ydata-av)*(menu_val[0]*1e9/dy_val))
   v2 = make_array(nx, ny)
   m = ny-1
   i = lindgen(nx)
;  NO need to limit the array. 
   for j = 0L, m do v2[0, j] = v1[i, j+iy]
 v1 =0
 a.v1 = reform(v2, n_elements(v2), /overwrite)
 v2 = 0
 if nparams eq 2 then begin
   if lcstr eq 0 then begin
     strout = get_save_str(strout, strin)
     i2s, a, strout, /no_copy
   endif else strout = temporary(a)
 endif else begin
   if lcstr eq 0 then i2s, a, strin, /no_copy else strin = temporary(a)
 endelse
;
;  See if we can save the parameters
;
 unit =0
 if strlen (fid_file) gt 1 then begin
   ff = fid_file
   info = file_info(ff)
   if (info.exists and info.write)  then begin
     openw, unit, /get, ff, /append, err = err
     If (err ne 0) then begin
       print,'% RM_REFRAC: Error opening file.
       print,'       file: ', ff
       print, '% RM_REFRAC: Unable to save parameters'
     endif 
   endif else if (info.exists) eq 0 then begin
     openw, unit, /get, ff, err = err
     If (err ne 0) then begin
       print,'% RM_REFRAC: Error opening file.
       print,'       file: ', ff 
       print, '% RM_REFRAC: Unable to save parameters'
     endif
   endif 
   if unit ne 0 then begin
     printf, unit, 'FILENAME:   ', fname
     printf, unit, 'DATETIME:   ', systime(0)
     printf, unit, 'LENGTH:     ', strtrim(menu_val[0], 2), ' (m)
     printf, unit, 'WAVELENGTH: ', strtrim(menu_val[1], 2), ' (Angstroms)
     printf, unit, 'POSITION:   ', strtrim(menu_val[2], 2), ' (mm)
     printf, unit, 'DISPERSION: ', strtrim(menu_val[3], 2), ' (Angstroms/mm)
     printf, unit, 'SWEEP:      ', strtrim(menu_val[4], 2), ' (ns/mm)
     free_lun, unit
   endif
 endif else  print, '% RM_REFRAC: Unable to save parameters'
END
