;
; ------------------------------------------------------------------------ 
; $Id: rm_skew.pro,v 1.5 2007/06/06 20:13:25 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
;
;+
; NAME:
;       RM_SKEW
; PURPOSE:
;       Proceedure for removing warp induced by each camera 
;       Problem is that the constant time slit is not perpendicular to the 
;         constant X (wavelength) position.
;    Note:  RM_NL_SKEW removes an arbitrary skew in an image.
;       Uses POLY_2D for bilinear and cubic interpolations
; CATEGORY:
;       VIDA - Image manipulation
; CALLING SEQUENCE:
;       RM_SKEW, STRIN [, STROUT] [, CAMERA = CAMERA] [, SLOPE = slope]
;       [, INTERPOLATION = interpolation] [, _EXTRA = ExtraStuff]
; INPUTS:
;       STRIN  - Structure array with the image requiring the camera correction.
; OPTIONAL INPUT PARAMETERS:
;       STROUT - Structure array with the corrected image
;                DEFAULT: STRIN
; KEYWORD PARAMETERS:
;       CAMERA - Camera number to be used or a string specifying a correction
;                Camera file has the number of elements for the slope followed
;                  by the values
;       SLOPE  - Slope of the constant time line in units of microns/micron
;                Typically skew correction is linear, but slope may be a vector
;                If slope is more than one element:
;                  correction = SLOPE[0]*X + SLOPE[1]*X*X + etc
;          NOTE: A single value of slope depends on DX = DY
;                Multiple values must be in units of PIXELS
;                  ie units of slope[1] = pixels/pixels^2
;       INTERPOLATION - 0 (DEFAULT) Nearest neighbor interpolation
;                       1 Bilinear Interpolation
;                       2 Cubic convolation interpolation (see POLY_2D)
;             NOTE:  INTERPOLATION is set to zero if slope is not scalar
;       ExtraStuff - Valid keyword to POLY_2D
;                    Ignored unless interpolation > 0
;
; OUTPUTS:
;       Camera Skew Corrected Image
; OPTIONAL OUTPUT PARAMETERS:
;       None
; COMMON BLOCKS:
;       None
; SIDE EFFECTS:
;       Completely rewrites the image
; RESTRICTIONS:
;       Camera calibrations must be stored in the data directory or a slope 
;         specified
;       Assumes the input image has square pixels, single block
;       Type 1 structure, single block, with slice in the 3 dimension.
; PROCEDURE:
;       The slope is of the skewed image is determined.
;    For bilinear or cubic interpolation:
;       The Arrays P and Q are defined as 2x2 arrays.
;       Q[1, 0] = P[0, 1] = 1.0
;       Q[0, 1] = slope
;       Corrected_image = POLY_2D(Distorted_image, P, Q, Interpolation, $
;         _EXTRA = extrastuff)
;    For nearest neighbor:
;       define yoffset = round(findgen(nx)*slope- nx/2*slope)
;       for j = 0, ny-1 do image[0, j] = image[*, ((j+yoffset)>0)< ny-1]
;   Note: Extension to higher order fit should be trivial.
;    
;  Example:
;       Correct the skew in image stored in structure array 2 from camera 19
;         RM_SKEW, 2, camera = 19
;
;       Correct the skew in image stored in structure array 1 from camera
;         with a measured slope of  -0.0292792
;         RM_SKEW, 1, slope =  -0.0292792
;
; MODIFICATION HISTORY:
;       Replaces a routine in VIDA written by Kyle Cochran
;       Original Version:  L. P. Mix, Jr. 
;                          January 31, 2003
;-
; ******************************************************************************


PRO rm_skew, strin, strout, camera = camera, slope = slope, $
      interpolation = interpolation, _EXTRA = ExtraStuff
;
@qscom.dat
;
@qsstrcom.dat
;     return on an error
      on_error, 1
;
;
if n_elements (structinit) eq 0 then qsstruct_init
if n_elements (qsplotinit) eq 0 then qsinit
;
 nparams = n_params()
 if nparams eq 0 then begin
   print,'% Format: ', $
     'RM_SKEW, STRIN [, STROUT] [, CAMERA = CAMERA] [, SLOPE = slope]'
   print,'            [, INTERPOLATION = interpolation] [, _EXTRA = ExtraStuff]'
   return
 endif
 if n_elements(strin) gt 0 then ain = strin[0]
 a =  get_valid_structure(ain, 'RM_SKEW', lcstr=lcstr, ierror=ier)
 if ier ne 0 then return
 if a.type ne 1 or a.block ne 1 or a.two ne 3 then begin
   print, '% RM_SKEW: Only type 1, single block structures can be corrected'
   print, '           Structure type:', a.type
   print, '           Structure Blocks:', a.block
   print, '           Slice dimension:', a.two
   return
 endif
 ok = where(strpos(tag_names(a), 'DX') ge 0 , nok)
 if nok gt 0 then begin
   if (a.dx1 ne a.dx2) or (a.dx1 eq 0) then begin
     print, '% RM_SKEW: Square uniform pixels are assumed'
     print, '           DX1, DX2:', a.dx1, a.dx2
   endif
 endif else print, '% RM_SKEW: Not able to check for square pixels'
 if n_elements(interpolation) eq 0 then interpolation =0
 if n_elements(slope) gt 0 then slope = slope else begin
   which, '^RM_SKEW$', /quiet, output = output, noutput = noutput
   if noutput ne 1 then begin
     print, '% RM_SKEW: Problem finding data directory.'
     return
   endif
   str = (strsplit(output[0], /extract))[1]
   s = get_bare_filename(str, directory = datadir)
   datadir = datadir+'data/'
   resolve_it, datadir
   ls, datadir+'*.skew', /quiet, current= current
   nnames = n_elements(current)
   names = make_array (nnames+1, /string)
   j = strlen (datadir)
   if strlen(current[0]) gt 0 then begin
     for i = 0, nnames-1 do $
       names[i] = strmid(current[i], j, strlen(current[i])-j-5)
     names[nnames] =  'Enter a Slope'
     if n_elements(camera) gt 0 then begin
       it = get_match(strtrim(camera[0], 2), names[0:nnames-1], $
         nelements= nel, regexp =0)
       if nel ne 1 then it = -1 else it = it[0]
     endif else it = -1
   endif
   if it eq -1 then begin
     pick_one, names, it, $
       instructions = 'Select the calibration for the SKEW correction', $
       title='Select SKEW Correction'
   endif
   
   if it lt nnames then begin
;      Read slope from a file
       readasc, current[it], 1, x, formatin=1 , error = error, np = np
       if error ne 0 or np ne 1 then begin
         print, '% RM_SKEW: Error reading file: ', current[it]
         Print, '% RM_SKEW: No skew correction performed'
         help, error, np
         return
       endif
       slope = x
   endif else it = -1

   if it eq -1 then begin
     get_float_value, value, inst= 'Enter Skew Slope or Enter 0.0 to QUIT'
     if value eq 0 then begin
       Print, '% RM_SKEW: No skew correction performed'
       return
     endif
     slope = value[0]
   endif    
 endelse
 if n_elements(slope) gt 1 then interpolation = 0
 v1 = reform(a.v1, a.size[0], a.size[1], /overwrite)

 if interpolation gt 0 then begin
   n = 2
   P = make_array(n, n)
   Q = P
   P[0, 1] = 1.0
   Q[1, 0] = 1.0
   Q[0, 1] = slope
   v2 = POLY_2D(v1, P, Q, Interpolation, _EXTRA = extrastuff)
 endif else begin
;  nearest neighbor
;  offset by 1/2 
   n = n_elements(slope)
   x = findgen(a.size[0])
   iy = x*slope[n-1]
   if n gt 1 then for i = n-2, 0, -1 do $
     iy = (iy + slope[i])*x
   av = (min(iy, max = m)+m)*0.5
   iy = round(iy-av)
   v2 = make_array(a.size[0], a.size[1])
   m = a.size[1]-1
   i = lindgen(a.size[0])
;  NO need to limit the array. 
   for j = 0L, m do v2[0, j] = v1[i, j+iy]
 endelse
 v1 =0
 a.v1 = reform(v2, n_elements(v2), /overwrite)
 v2 = 0
 if nparams eq 2 then begin
   if lcstr eq 0 then begin
     strout = get_save_str(strout, strin)
     i2s, a, strout, /no_copy
   endif else strout = temporary(a)
 endif else begin
   if lcstr eq 0 then i2s, a, strin, /no_copy else strin = temporary(a)
 endelse
END
