; ------------------------------------------------------------------------ 
; $Id: dirxrd.pro,v 1.3 2007/06/29 13:42:43 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
  pro dirxrd, p1,  PAGE = page, reset = reset, font= font, ndset = ndset, $
    xsize=xsize, truncate = truncate, print = print, tail = tail,  $
    QUIET = quiet, comment = comment, line = line, cathode = cathode, $
    filter = filter, detector = detector, terminal = terminal, $
    no_block = no_block
;
;+
;
;     This routine prints a directory of XRD unfold files and optionally returns
;       the values of the directory information in keyword arguements.
;
;     DIRXRD [, P1] [, Page = Page] [, NDSET = ndset] [, TAIL = tail]
;         [, TRUNCATE = truncate] [, /TERMINAL] [, /print] [, /quiet] 
;         [, Reset = reset]  [, Xsize = xsize] [, COMMENT = comment]
;         [, LINE = line] [, /CATHODE] [, /FILTER] [, /DETECTOR]
;
;     where:
;           P1       - an optional search string.
;                      For example: To get all datasets with '891' use:
;                        dirxrd, '891' or dirxrd, 891 
;                      See GET_MATCH for allowed wild cards.
;           PAGE     - the number of lines on the directory
;                      default is 24
;           NDSET    - the number of datasets returned 
;           TAIL     - Return the last 20 datasets
;                      If TAIL gt 1 then return the last TAIL datasets.
;           RESET    - present and not zero, resets the directory
;                      use this to put the current directory in a second window
;                 Note: If the window is still open and you wish to leave it
;                       open, use /reset to put the current command in a second
;                       window.
;                 NOTE: By using the reset qualifier, a window can be 
;                       opened for each file with the filename at the top.
;           XSIZE    - the width of the window in pixels (generally not set)
;           TRUNCATE - if set, the dataset comments will be truncated at ' - '
;                      This is a toggle; If not specified the last value will
;                      be used.  Default value =1 -- truncate
;                   Note:  The dataset comment is limited to 64 characters
;           PRINT    - If set, directory will be sent to terminal
;           QUIET    - If set, no outputs unless an error. 
;           COMMENT  - Dataset comment for each of the datasets
;           Cathode  - Directory of the cathode response file
;           Filter   - Directory of the filter response file
;           Detector - Directory of the combined cathode/filter response file
;
;     READ funtion:
;          The DIR widget can be used to read files.
;          Datasets may be highlighted using the left mouse button.
;          Shift-Left mouse button will select a range of datasets.
;          Control-Left mouse button will toggle individual datasets.
;          If a carriage return is entered in the window or if one of the
;          above selections is performed with a double-click, PFIDL will read
;          selected datasets.
;          All seleced datasets should be either structures or waveforms.
;
;     Example:
;         Get a directory of the Header and return the number of datasets.
;           dirxrd, ndset = ndset
;         Get a directory of the cathodes.
;           dirxrd, /cathode
;
;-
;
      on_error, 1
;
;     common blocks

@qscom.dat

  common pffcom, initpff, pff_exe
;
  common dirxrdcom, base, base_typ, list, oldpage, text1, truncate_save
@xrd_common_data.dat

;

  if n_elements(initxrd) eq 0 then initxrd =0
  if initxrd eq 0 then init_xrd
  if initxrd le 0 then begin
    print, '% DIRXRD: XRD analysis is not supported by this system.'
    return
  endif
  if n_elements(qsplotinit) eq 0 then qsinit
;
  if n_elements (oldpage) eq 0 then oldpage = directorypage
  if n_elements (page) eq 0 then page = oldpage
  page = page >4
;
  if n_elements (truncate_save) eq 0 then truncate_save =1
  if n_elements (truncate) gt 0 then truncate_save = truncate
;
  if n_elements (base) eq 0 then begin
    base = 0
    text1 = 0
    text2 = 0
    base_typ = 0
  endif


  if keyword_set(cathode) then begin
    fid = cfid
    basetyp = 1
    Base_title = 'XRD Cathode Response Directory'
  endif else if keyword_set(filter) then begin
    fid = ffid
    basetyp = 2
    Base_title = 'XRD Filter Response Directory'
  endif else if keyword_set(detector) then begin
    fid = dfid 
    basetyp = 3
    Base_title = 'XRD Detector Response Directory'
  endif else begin
    fid = hfid
    basetyp = 4
    Base_title = 'XRD Diagnostic Headers Directory'
  endelse
      
  if fid eq 0 then begin
    Print, '% DIRXRD: This file has not been opened.
    return
  endif

  if n_elements(p1) eq 0 then p1 = '' else p1 = strtrim(p1, 2)

  get_dir, fid, p1, line = line, filename = filename, number = ndset
  if ndset eq 0 then return

  filename = 'File: '+ filename

  if keyword_set(truncate_save) then begin
    for i = 0, ndset-1 do begin
      ii = strpos(line(i), ' - ', 25)
      if ii gt 0 then line(i) = strmid(line(i), 0, ii)
    endfor
  endif
  if ndset eq 0 then return
  if keyword_set(reset) then base = 0
  if (oldpage ne page) or (base_typ ne basetyp) then page = -page
  base_typ = basetyp
  widget_directory, base, page, list, line, base_title = base_title, $
    xsize = xsize, text1 = text1, value1 = filename,  dirtype =1, $
    text2 = text2, fileid = fileid, no_block = no_block,  $
    value2 = ' #   Data Type       Dataset Comment'

  oldpage = page
  end


