; ------------------------------------------------------------------------ 
; $Id: fit_filter.pro,v 1.5 2010/06/10 22:03:09 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
;+
; NAME:
;       Fit_Filter
;
; PURPOSE:
;       Fit filter calibrations to material curves.
;       Default material curves in .../pfidl/pfidl/xrd/data/filter_material.pff
;       or 
;       Assume that Transmission = exp (-Thickenss*density*material_stopping)
;       Transmissions for multiple materials are the product of each material.
;
; CATEGORY:
;       XRD analysis
;
; CALLING SEQUENCE:
;       FIT_FILTER [, WDF] [, Filter] [, ITER = iter]
;          [, WEIGHTS = weights] [, SIGMA = sigma] [, CHISQ = chisq]
;          [, ITMAX = itmax] [, TOLERANCE = tolerance] [, FILEID = fid]
;          [, SAVE = save]  [, QUIET = quiet] [, Verbose = verbose]
;          [, MINERROR= minerror]  [, LOGFILE = logfile] 
;          [, /SETDEFAULT] [, UNSET = unset] [, SHOWDEFAULT = showdefault]
;          
;
; OPTIONAL INPUTS:
;       WDF:    WDF to store the fit data.
;               Default is get_maxwdf()-1.
;               Get_maxwdf() will be used as a work array.
;               The fit data on the same scale as the original data
;                  is in get_maxwdf()-2
;               The difference between the calibration and fit 
;                  is in get_maxwdf()-3
;               The % difference between the calibration and fit 
;                  is in get_maxwdf()-4
;               The fractional error in the calibration data 
;                  is in get_maxwdf()-5
;               The original data will be stored in Get_maxwdf() 
;	FILTER: File for analysis
;               If omitted, then PICKFILE will be used.
;	
; KEYWORD PARAMETERS:
;      ITER:     Number of iterations performed
;      Weights:  Weights used for the least squares fit.
;                Weights is missing
;                  Default - Weight = 1 
;                Weights is 0 then  Weight = 1.0
;                Weights is 1 (or /weight) - Weight = 1/Yi
;                Weights is 2 - Weight = 1/Yi^2
;                    where Yi = %Standard_Deviation*Transmission
;                If Weights is an array, then the values will be used.
;                If there are fewer values of weights than data points, then
;                  missing values are 1.0d0
;      Sigma:    Standard Deviation for the coefficients for each material
;      Chisq:    Value of Chi-squared
;      ITMAX:    Maximum number of iterations
;                Default = 1000
;      Tolerance:Change in Chi-squared which ends fit
;                Minimum = 1.0E-16,  Maximum = 0.01
;                Default = 1e-15 
;      Fileid:   If material data is to come from a special file, this file must
;                be opened and the file id specified.
;                If fileid is missing or zero, then routine checks opened pff
;                files for a file "filter_material.pff".  If not found then 
;                the routine attempts to open: .../pfidl/pfidl/xrd/data/filter_material.pff
;      SAVE:     If set, data will be written to the default filter file.
;                  (See WRITE_FILTER for default definition.)
;      Quiet:    If set, terminal output will be minimum.
;      Verbose:  If set, terminal output will be maximum.
;      MINerror: Minimum value for Yi in calculating the weights.
;                Default = 1e-6
;      LOGFILE:  Unit for Verbose outputs to a log file.
;                If LOGFILE is a positive number, the unit must be open.
;                If LOGFILE is a string, the file is opened for append.
;                IF LOGFILE is a negative number, open file with same name as
;                   the cathode calibration file with suffix = ".log"
;                If LOGFILE is set, then VERBOSE = 1
;      SETDEFAULT: If set, the following parameters may be set as default.
;                LOGFILE = 0
;                WEIGHTS = 1
;
;      UNSET     If present and not zero, will set all save values of keywords
;                to their default values.
;      SHOWDEFAULT: Show the present default plot values.
;
; OUTPUTS:
;       The calibration data is stored in the maximum wdf array.
;       The fit data is stored in the next lower wdf array and in the array WDF.
;
; OPTIONAL OUTPUTS:
;       If SAVE is set the data is written to the filter file.
;
; SIDE EFFECTS:
;       A minimum of the last 7 wdf arrays are used.  
;       The last 5 are used for the fit and the data.
;       The fractional error in the calibration data is in the next array.
;       One additional wdf is used for each material.
;
; PROCEDURE:
;       Data is read into wdf arrays and CURVEFIT is used to perform the 
;       least-squares fit.
;
; EXAMPLE:
;       Fit some data.
;         FIT_FILTER
;
; MODIFICATION HISTORY:
; 	Written by:     L. P. Mix, April 18, 2001
;-

 PRO fit_filter, WDFarr, Filter_in, ITER = iter, $
          WEIGHTS = weights, SIGMA = sigma, CHISQ = chisq, logfile = logfile, $
          ITMAX = itmax, TOLERANCE = tolerance, FILEID = fid, SAVE = save, $
          QUIET = quiet, Verbose = verbose, minerror = min_error, $
          setdefault= setdefault, unset= unset, showdefault= showdefault

on_error, 1
 common filter_fit_funct_common, xval, wdf, fval
 common fit_filter_default_com, logfile_save, weights_save

 common pffcom, initpff, pff_exe

 work = get_maxwdf()
 if n_elements(wdfarr) eq 0 then wdfarr = work-1 else $
    wdfarr = get_save_array(wdfarr, work-1 )

 if keyword_set(unset)  or n_elements (color_save) eq 0 then begin
   LOGFILE_SAVE = 0
   WEIGHTS_SAVE = 1
 endif
;
 if keyword_set (setdefault) then begin
   if n_elements(logfile) gt 0 then logfile_save = logfile
   if n_elements(weights) gt 0 then weights_save = weights
 endif

 if keyword_set(showdefault) then begin
   print, 'Default parameters for the FIT_CATHODE command are:'
     print, '    LOGFILE      = ' , strtrim (logfile_save, 2)
     print, '    WEIGHTS      = ' , strtrim (weights_save, 2)
 endif

 if n_elements(logfile) eq 0 then logfile = logfile_save
 if n_elements(weights) eq 0 then weights = weights_save

 if n_elements(filter_in) eq 0 then begin
   filter = dialog_pickfile(/must_exist, /read, filter = '*.rsp')
   if strlen(filter) lt 1 then begin
     print, '% FIT_FILTER: No File was Selected - RETURNING'
     return
   endif
 endif else filter = filter_in
 
 error_frac = work-5
 rersp, filter, work, comments = comments, run_number = run_number, $
        thickness = thickness, materials = material, error = error_frac, $
        nmaterial = nmat, ier = ier

 if ier ne 0 then begin
   print,  '% FIT_FILTER: Unable to read filter data.'
   return
 endif

 ok = strpos(filter, path_sep(), /reverse_search) 
 okp = strpos(filter, '.', /reverse_search)

 if okp gt ok then dataname= strmid(filter, ok+1, okp-ok-1) else $
   dataname = strmid(filter, ok+1)

 if keyword_set(logfile) then begin
   slog = size(logfile, /type)
   if slog eq 7 then begin
     openw, logunit, /get, logfile, /append, error = ier
     if ier ne 0 then begin
       print, '% FIT_FILTER: Unable to open file: ', logfile
       print, '               ', !error_state.msg
       return
     endif
     lcopen = 1
   endif else if logfile lt 0 then begin
     if okp gt ok then lgfile = strmid(filter,0,  okp) else lgfile = filter
     openw, logunit, /get, lgfile+'.log', /append, error = ier
     if ier ne 0 then begin
       print, '% FIT_FILTER: Unable to open file: ', lgfile
       print, '               ', !error_state.msg
       return
     endif
     lcopen = 1
   endif else begin
     lcopen = 0
     if slog ne 3 then logunit = round (logfile) else logunit = logfile
     stat = fstat(logunit)
     if stat.open eq 0 then begin
       print, '% FIT_FILTER: Logfile unit is not open: ', logunit
       return
     endif
   endelse
   verbose = 1
 endif else logunit = -1

 if keyword_set(verbose) then begin
   printf, logunit, ' ******************************************************'
   printf, logunit, 'Fit to file: ', filter
   printf, logunit, ' '
 endif


 if n_elements (fid) eq 0 then begin
    showpff, filename = fname, ufid = fid, numb = nfile, /quiet
    pdir = get_pfidl_directory()
    sep = path_sep()
    filt_mat = pdir+'pfidl'+sep+ 'xrd'+sep+'data'+sep+'filter_material.pff'
    if nfile eq 0 then $
      openpff, filt_mat, error = ier $
    else begin
      ok = where (strpos (fname, 'filter_material.pff') ge 0, nok)
      if nok gt 0 then setpff, fid[ok[0]], ier else $
        openpff, filt_mat , error = ier
    endelse
 endif else setpff, fid[0], ier

 if ier ne 0 then begin
   print, '% FIT_FILTER: Unable to open/access material filter file.'
   return
 endif
 if nmat eq 0 then begin
   print, '% FIT_FILTER: Bad File, No Filter Materials read.'
   free_lun, unit
   return
 endif
 if keyword_set(verbose) then begin
   printf, logunit, nmat, ' Materials Found'
   printf, logunit, '  Material     Thickness'
   for i = 0, nmat-1 do printf, logunit, material[i], thickness[i]
 endif 
 w2i, xvalue, yval, work
 w2i, xvalue, percent_error, error_frac
 np = n_elements(yval)


 if  np lt nmat then begin
   print, '% FIT_FILTER: Unable to fit data with: Cal.Pts, Materials:', np, nmat
   printf, logunit, $
   '% FIT_FILTER: Unable to fit data with: Cal.Pts, Materials:', np, nmat
   return
 endif
 if n_elements(min_error) eq 0 then min_error = 1e-6
 if n_elements(weights) eq 0 then $
   if n_elements(percent_error) gt 2 then weights = 2 else weights = 0

 if keyword_set(verbose) then begin
   printf, logunit, nmat, ' Materials Found'
   printf, logunit, '  Material     Thickness'
   for i = 0, nmat-1 do printf, logunit, material[i], thickness[i]
 endif

 if keyword_set (weights) then begin
   if n_elements(weights) gt 1 then begin
     wei = make_array(np, /double, value = 1.0d0)
     n = n_elements(weights) < np
     wei[0:n-1]=weights
   endif else begin
     if n_elements (percent_error) lt 2 then begin
       Print, '% FIT_FILTER: Unable to use weights with no errors'
       help, percent_error
       return
     endif
     err = abs(yval*percent_error) > min_error
     wei =  1./err
     if weights gt 1 then wei = wei*wei
   endelse
 endif else Wei= make_array(np, value = 1.0d0)
 if keyword_set(verbose) then printf, logunit, 'Weights: ', wei
 xcal = xvalue
 ycal = yval
;
if keyword_set(verbose) then begin
   printf, logunit, ' Input data and weights'
   printf, logunit, '       Energy    Error_Fract    Value        Weight'
   for i = 0, np-1 do $
      printf, logunit, xcal[i], percent_error[i], yval[i], wei[i]
endif


 result = strsplit(dataname, '_', /extract)
 n = n_elements(result)
 filter_name = result[0]
 if n gt 1 then filter_id = result[1] else filter_id = '****'
 if n gt 2 then cal_name = result[2] else cal_name = '****'
 xval = xcal
 cha, work, clab = dataname, $
   xlabel = 'Photon Energy (eV)', ylabel = 'Transmission'

; Convert thickness in microns to cm
 thickness = thickness*1e-4
 coef = make_array(nmat, /double)
 density = make_array(nmat)
 wdf = lindgen(nmat) + work-5-nmat
 for i = 0, nmat-1 do begin
   mat = strtrim(material[i],2)
   if strlen(mat) le 2 then mat = '('+mat+')' else mat= mat+'* - '
   re, wdf[i], mat, narray = n
   if n ne 1 then begin
     printf, logunit,'% FIT_FILTER: Unable to read properties for :', mat
     print, '% FIT_FILTER: Unable to read properties for :', mat
     return
   endif

   str = wdcomment(wdf[i])
   ii = strpos(str, 'Density=')+8
   if ii lt 0 then begin
     print, '% FIT_FILTER: This material does not have density information.'
     print, 'Assume a value of 1.00 for material =' , mat 
     rho = 1.00
   endif else begin
     j = strpos(str, ';;',ii)
     if j lt 0 then rho = float(strmid(str, ii)) else $
       rho = float(strmid(str, ii, j-ii))
   endelse 

   density[i] = rho
   coef[i] = rho*thickness[i]
;  print, str, density[i], coef[i], thickness[i]
 endfor

 fval = make_array(np, nmat, /double)
 for i = 0, nmat-1 do fval[*,i] = get_value(wdf[i], xvalue)
 if n_elements(tolerance) eq 0 then tol = 1e-15 else $
     tol = (abs(tolerance) < 0.01)> 1.0e-16
 maxpass = 7
 tolmax = tol
 logtol = alog(tol+tol)/(maxpass -1)
 tol = [exp((1+indgen(maxpass-1))*logtol), tol]
 iter = 0
 fallback = 0
 yvalue = double(ycal)
 if keyword_set(verbose) then print, 'Initial Coefficient =', coef
 coef_old = coef
 for i=0, maxpass-1 do begin
   result=curvefit(xvalue, yvalue, wei, coef, sigma, chisq=chisq, $
     function_name='filter_fit_function',iter=iteri,itmax=500,tol=tol[i])
   if keyword_set(verbose) then begin
     print, 'Tolerance =' ,tol[i]
     print, 'Coefficient =', coef
     print, 'Standard Deviation =', sigma
     print, 'Chi-Squared =', chisq, ',  Iterations =', iteri
   endif
   iter = iteri+iter
   if (iteri gt 500)  then begin
     fallback = fallback+1
     if fallback lt maxpass then begin
       if i lt maxpass-1 then tol[i+1:*] = tol[i:maxpass-2]
       tol[i] = sqrt(tol[i]*tol[(i-1)> 0])
       i = i-1
     endif
     coef = coef_old
   endif else coef_old = coef
 endfor
 if keyword_set(verbose) then print, ' Final Coefficient = ', coef
 if tol[maxpass-1] gt tolmax then begin
   print, ' Tolerance reduced to obtain convergence to:', tol[maxpass-1]
   if logunit gt 0 then printf, logunit, $
      ' Tolerance reduced to obtain convergence to:', tol[maxpass-1]
 endif
 new = fval
 for i = 0, nmat-1 do new[*,i] = new[*, i]*coef[i]
 if nmat gt 1 then  new = total(new, 2)
 if logunit gt 0 then begin
     printf, logunit, 'Tolerance =' ,tol[i]
     printf, logunit, 'Coefficient =', coef
     printf, logunit, 'Standard Deviation =', sigma
     printf, logunit, 'Chi-Squared =', chisq, ',  Iterations =', iteri
 endif

 wdfname = make_array(nmat, /string)
 for i = 0, nmat-1 do begin
  cc  = wdcomment(wdf[i])
  wdfname[i]=strtrim(strmid(cc, 0, strpos(cc, ' - ')), 2)
 endfor  
 if keyword_set (verbose) then begin
   printf, logunit, ' Material, Input and Calculated Thicknesses'
   for i = 0, nmat-1 do $
     printf, logunit, wdfname[i], thickness[i], coef[i]/density[i]
 endif
 for i = 0, nmat-1 do mul, wdf[i], 0, coef[i]
 xfr, wdf[0], wdfarr
 if nmat gt 1 then for i = 1, nmat-1 do add, wdfarr, wdf[i]

 maxexpval= -alog((machar()).xmin)
 y = getyf(wdfarr)
 ok = where (y gt maxexpval, nok)
 if nok gt 0 then y[ok]=0.0
 y = exp(-y)
 if nok gt 0 then y[ok]=0.0
 puty, y, wdfarr

 ok = where (new gt maxexpval, nok)
 if nok gt 0 then new[ok]=0.0
 new = exp(-new)
 if nok gt 0 then new[ok]=0.0
 xfr, work, get_maxwdf()-2
 puty, new, get_maxwdf()-2
 lab, get_maxwdf()-2, append = -1, 'Fit of '
 

 xfr, work, get_maxwdf()-3
 xfr, work, get_maxwdf()-4
 diffval = ycal-new
 puty, diffval, get_maxwdf()-3
 lab, get_maxwdf()-3, 'Difference '
 percentval = ((ycal-new)/ycal)*100
 puty, percentval, get_maxwdf()-4
 cha, get_maxwdf()-4, clab = '% Difference ', ylab ='% Diff'

 label = filter_name+ ', Filter: '+filter_id+', Cal: '+cal_name $
          + ', ' + thin_date(/short)+' - '

 for i = 0, nmat-1 do $
   label = label+ wdfname[i]+'=' + strtrim(coef[i], 2)+';;'
 cha, wdfarr, clabel = label, ylabel = 'Filter Transmission'
 if keyword_set (verbose) then begin
   yr = [mn(work, ma= m, range= xr, /quiet), m]
   plo, wdfarr, /color, /over, linest=[0 , 0], /ns, $
      xr = xr, yr = yr, leg = [.5, .6]
   plo, work, psym = -8, over = 1, /color, /no
   wderr, work, get_maxwdf()-5, /relative, color = 3, Npoint= np

   printf, logunit, $
    ' Energy(eV)   Cal Value    Fit Value   Difference  % Difference'
   for i = 0, n_elements(xcal)-1 do printf, logunit, $
     xcal[i], ycal[i], new[i], diffval[i], percentval[i]
   printf, logunit, ' '
   printf, logunit, ' End of analysis for ', filter
   printf, logunit, ' '
 endif
 if keyword_set(lcopen) then free_lun, logunit
 if keyword_set(save) then write_filter, wdfarr

END
