; ------------------------------------------------------------------------ 
; $Id: make_composite.pro,v 1.7 2010/06/10 21:52:47 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
;+
; NAME:
;       MAKE_COMPOSITE
;
; PURPOSE:
;       Calculates the mass absorption coefficient of a foil
;       Foil may contain multiple materials
;
; CATEGORY:
;       X-ray diagnostic
;
; CALLING SEQUENCE:
;
;       MAKE_COMPOSITE, WDF, MATERIAL, NUMBER
;
; INPUTS:
;       WDF:    WDF array number to store the result
;       MATERIAL:  Array of materials.
;                  Materials may be the name as 'Copper' or symbol 'Cu'
;                  The list is not case sensitive but must be an array of
;                  strings.  Names must be longer than 2 characters but the
;                  length may be 3 or more characters if that defines a unique
;                  element or compound.
;       NUMBER:    Array of material numbers of atoms in each molecule
;                  Number of materials must match the number of thicknesses.
;
; KEYWORD PARAMETERS:
;       FILTER: File ID of the file "filter_material.pff" or the file
;               containing the filter material mass absorption coefficient.
;               Routine will look for this file if it is not already open.
;               In most installations, this keyword is not required unless
;               the user has stored his own filter materials.
;               (See: MAKE_COMPOSITE)
;       MASS_FRACTION: If set, NUMBER is in terms of mass fraction
;
; OUTPUTS:
;       The mass absorption coefficient of the composite is stored in the 
;       specified WDF array.
;
; COMMON BLOCKS:
;       None
;
; RESTRICTIONS:
;       Material must be in the file "filter_material.pff" or in a user
;       specified PFF file which is opened before calling filter_material.
;
; PROCEDURE:
;       The mass absorption coefficient is calculated for the composite material
;
; EXAMPLE:
;       Calculate the filter transmission for a filter of equal atoms of carbon
;         and hydrogen and store the result in wdf 4.
;         MAKE_COMPOSITE, 4, ['c', 'h'], [1,1]
;       Calculate the filter transmission for a filter of 7 atoms copper and 
;         3 atoms zinc for every 10 atoms and store in wdf 5.
;         MAKE_COMPOSITE, 4, ['^copper', 'zn'], [7, 3]
;
; MODIFICATION HISTORY:
;       Written by:     L. P. Mix, March 9, 2001
;       December, 2003  Added help file information
;                       Modified location of filter material file
;-

 pro make_composite, wdf, material, number, filter= fid, mass_fraction = massf
on_error, 1
 npar = n_params()
 if npar eq 0 then begin
   print, '% Format: MAKE_COMPOSITE, WDF [, MATERIALS, NUMBER]'
   print, '             [, FILTER= fid] [, /MASS_FRACTION'
   return
 endif
 nelem = n_elements(material)
 nthick = n_elements(number)
 if nelem ne nthick then begin
   print, '% MAKE_COMPOSITE: Number of materials must equal number of atoms.'
   return
 endif
 if nelem eq 0 then begin
   print, '% MAKE_COMPOSITE: Materials and number of atoms must be defined.'
   return
 endif

 if n_elements (fid) eq 0 then begin
    pff_info, fname = fname, fid = fid, nfile = nfile, /quiet
    if nfile eq 0 then nok = 0 else $ 
      ok = where (strpos (strlowcase(fname), 'filter_material.pff') ge 0, nok)
    if nok gt 0 then setpff, fid[ok[0]], ier else  begin
      which, '^MAKE_FILTER$', /quiet, output = output, noutput = noutput
      if noutput ne 1 then begin
        print, '% MAKE_FILTER: Problem finding data directory.'
        return
      endif

      output = strtrim(output[0], 2)
      str  = strtrim(strmid(output, strpos(output, '  ')), 2)
      sep = path_sep()
      fname = file_dirname(str)+sep+'data'+ sep + 'filter_material.pff'
      info = file_info(fname)
      if (info.exists + info.read) ne 2b then begin
        print,'% MAKE_FILTER: Unable to find filter mass absorption coefficient'
        print,'               Open the file and use the filter keyword.'
        print, '              File attempted and permissions are:'
        print, '              File Name = ', fname
        print, '              File Exists =', info.exists
        print, '              Read Permission =', info.read
        return
      endif
      openpff, fname, error = ier
    endelse
 endif else setpff, fid[0], ier

 if ier ne 0 then begin 
   print, '% MAKE_COMPOSITE: Unable to open/access material filter file.'
   return
 endif
 
 mwdf = get_maxwdf()
 work = mwdf-1
; set default wdf array to the maximum wdf array
;   and the work array to the one below it.
;
 wdf = get_save_array( wdf,  mwdf)
; if wdf equals the work array then move the work array to the maxwdfarray.
 if wdf eq work then work = mwdf
 if n_elements(name) eq 0 then fname = 'Filter' else fname= strtrim(name[0], 2)
 if strlen(fname) eq 0 then fname = 'Filter'
 clabel = fname + ' - '
 atotal = 0.0
 frac = make_array(nelem)
 ename = make_array(nelem, /string)
 for i = 0, nelem-1 do begin
   mat = strtrim(material[i],2)
   if strlen(mat) le 2 then mat = '('+mat+')'
   re, work, mat, narray = n
   if n ne 1 then begin
     print, '% MAKE_COMPOSITE: Unable to read properties for :', mat
     return
   endif
   str = wdcomment(work)
   if keyword_set(massf) then rho = number[i] else begin
     ii = strpos(str, 'AWeight=')+8
     if ii lt 8 then begin
 print,'% MAKE_COMPOSITE: This material does not have atomic weight information.'
       return
     endif
     j = strpos(str, ';;',ii) 

     if j lt 0 then rho= float(strmid(str, ii))*number[i] else $
       rho = float(strmid(str, ii, j-ii))*number[i]
   endelse
   mul, work, 0, rho
   atotal = atotal+rho
   
   if i eq 0 then xfr, work, wdf else add, wdf, work, /sortx
   jj = strpos(str, '(') 
   if jj gt 0 then str = strtrim(strmid(str, 0, jj), 2) else begin
     ii = strpos(str, ' - ') 
     if ii gt 0 then str = strtrim(strmid(str, 0, ii), 2) $
       else str= strtrim(str,2)
   endelse
   ename[i]=str
   frac[i]=rho

 endfor
 frac = frac/atotal
 for i = 0, nelem-1 do clabel = clabel+ename[i]+'='+strtrim(frac[i], 2)+';;'
 div, wdf, 0, atotal
 cha, wdf,  clabel = clabel
 end


