; ------------------------------------------------------------------------ 
; $Id: make_filter.pro,v 1.7 2010/06/10 21:53:43 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
;+
; NAME:
;       MAKE_FILTER
;
; PURPOSE:
;       Calculates the transmission of a foil or stack of foils
;
; CATEGORY:
;       X-ray diagnostic
;
; CALLING SEQUENCE:
;
;       MAKE_FILTER, WDF, MATERIAL, THICKNESS
;
; INPUTS:
;       WDF:    WDF array number to store the result
;       MATERIAL:  Array of materials.
;                  Materials may be the name as 'Copper' or symbol 'Cu'
;                  The list is not case sensitive but must be an array of 
;                  strings.  Names must be longer than 2 characters but the
;                  length may be 3 or more characters if that defines a unique
;                  element or compound.
;       THICKNESS: Array of material thicknesses.
;                  Number of materials must match the number of thicknesses.
;                  Units of thickness are grams/cm^2 unless CM is set.
;                  Density is the density in "filter_material.pff"
;                  If CM is set, THICKNESS = THICKNESS*Density
;	
; KEYWORD PARAMETERS:
;	CM:	If set, thickness is in centimeters.
;       NAME:   If set this is use as the label with the components.
;               Default: NAME = 'Filter'
;       FILTER: File ID of the file "filter_material.pff" or the file 
;               containing the filter material mass absorption coefficient.
;               Routine will look for this file if it is not already open.
;               In most installations, this keyword is not required unless
;               the user has stored his own filter materials.
;               (See: MAKE_COMPOSITE)
;
; OUTPUTS:
;       The transmission of the filter is stored in the specified WDF array.
;
; COMMON BLOCKS:
;	MAKE_FILTER_COMMON:  Machine limit for exponentials
;
; RESTRICTIONS:
;       Material must be in the file "filter_material.pff" or in a user
;       specified PFF file which is opened before calling filter_material.
;       For user generated composite materials, the DENSITY must be provided
;       in the dataset comment if "CM" is set.
;
; PROCEDURE:
;       The mass absorption coefficient is calculated for the composite material
;       The filter is exp(-Mass_absorption_coefficient)
;
; EXAMPLE:
;       Calculate the filter transmission for a 1 micron copper filter and
;         store the result in wdf 4.
;         MAKE_FILTER, 4, 'cu', .0001, /cm
;   or    MAKE_FILTER, 4, 'copper', .0001, /cm
;
;       Calculate the filter transmission for a filter with 0.001 inch Aluminum
;         and 0.1 micron gold and store the result in 5.
;         MAKE_FILTER, 5, ['Al', 'gold'], [.001*2.54, .00001], /cm
;
; MODIFICATION HISTORY:
; 	Written by:	L. P. Mix, March 9, 2001
;	December, 2003  Added help file information
;                       Modified location of filter material file
;-

 pro make_filter, wdf, material, thickness, cm = cm, $
      filter= fid, name = name 
on_error, 1
 common make_filter_common, maxexpval
 npar = n_params()
 if npar eq 0 then begin
   print, '% Format: MAKE_FILTER, WDF , MATERIALS, THICKNESS'
   print, '               [, /CM] [, FILTER= fid] [, NAME = name]'
   return
 endif

 if n_elements(maxexpval) eq 0 then maxexpval= -alog((machar()).xmin)

 nelem = n_elements(material)
 nthick = n_elements(thickness)

 if nelem ne nthick then begin
   print, '% MAKE_FILTER: Number of materials must equal number of thicknesses.'
   return
 endif
 if nelem eq 0 then begin
   print, '% MAKE_FILTER: Materials and thicknesses must be defined.'
   return
 endif
 
 if n_elements (fid) eq 0 then begin
    pff_info, fname = fname, fid = fid, nfile = nfile, /quiet
    if nfile eq 0 then nok = 0 else $
      ok = where (strpos (strlowcase(fname), 'filter_material.pff') ge 0, nok)
    if nok gt 0 then setpff, fid[ok[0]], ier else begin
      which, '^MAKE_FILTER$', /quiet, output = output, noutput = noutput
      if noutput ne 1 then begin  
        print, '% MAKE_FILTER: Problem finding data directory.'
        return
      endif
      output = strtrim(output[0], 2)
      str  = strtrim(strmid(output, strpos(output, '  ')), 2)
      sep = path_sep()
      fname = file_dirname(str)+sep+'data'+ sep + 'filter_material.pff'
      info = file_info(fname)
      if (info.exists + info.read) ne 2b then begin
        print,'% MAKE_FILTER: Unable to find filter mass absorption coefficient'
        print,'               Open the file and use the filter keyword.'
        print, '              File attempted and permissions are:'
        print, '              File Name = ', fname
        print, '              File Exists =', info.exists
        print, '              Read Permission =', info.read
        return
      endif
      openpff, fname, error = ier
    endelse
 endif else setpff, fid[0], ier
 if ier ne 0 then begin 
   print, '% MAKE_FILTER: Unable to open/access material filter file.'
   return
 endif
 
 mwdf = get_maxwdf()
 work = mwdf-1
; set default wdf array to the maximum wdf array
;   and the work array to the one below it.
;
 wdf = get_save_array( wdf,  mwdf)
; if wdf equals the work array then move the work array to the maxwdfarray.
 if wdf eq work then work = mwdf
 if n_elements(name) eq 0 then fname = 'Filter' else fname= strtrim(name[0], 2)
 if strlen(fname) eq 0 then fname = 'Filter'
 clabel = fname + ' - '
 
 for i = 0, nelem-1 do begin
   mat = strtrim(material[i],2)
   if strlen(mat) le 2 then mat = '('+mat+')' else mat= mat+'* - '
   re, work, mat, narray = n
   if n ne 1 then begin
     print, '% MAKE_FILTER: Unable to read properties for :', mat
     return
   endif

   str = wdcomment(work)
   if keyword_set (cm) then begin
     ii = strpos(str, 'Density=')+8
     if ii lt 0 then begin
       print, '% MAKE_FILTER: This material does not have density information.
       return
     endif
     j = strpos(str, ';;',ii) 
     if j lt 0 then rho = float(strmid(str, ii))*thickness[i] else $
       rho = float(strmid(str, ii, j-ii))*thickness[i]
   endif else rho= thickness[i]
   mul, work, 0, rho
   if i eq 0 then xfr, work, wdf else add, wdf, work, /sortx
   ii = strpos(str, ' - ')
   if ii gt 0 then str = strtrim(strmid(str, 0, ii), 2) else str= strtrim(str,2)
   clabel = clabel+str+'='+strtrim(rho, 2)+';;'

 endfor
 y = getyf(wdf)
 ok = where (y gt maxexpval, nok)
 if nok gt 0 then y[ok]=0
 y = exp(-y)
 if nok gt 0 then y[ok]=0.0
 
 puty, y, wdf
 cha, wdf, ylabel = 'Filter Transmission', clabel = clabel
 end


