; ------------------------------------------------------------------------ 
; $Id: print_xrd.pro,v 1.3 2007/06/29 13:42:44 lpmix Exp $ 
; ------------------------------------------------------------------------ 
 pro print_xrd, strin, losin, filename = filename, nodelete = nodelete, $
        append = append, print = printit, destination = destination, $
        terminal = terminal
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
;+
; NAME:
;       PRINT_XRD
;
; PURPOSE:
;       PRINT an XRD shot description.
;
; CALLING SEQUENCE:
;       PRINT_XRD [, SHOT_NUMBER]  [, Line_of_Sight]
;  or 
;       PRINT_XRD [, STRUCTURE]
;
; INPUTS:
;       SHOT_NUMBER   - Existing shot number in the shot_config file.
;       Line_of_Sight - Existing Line_of_sight diagnostic array descriptor.
;                       In general this is a string variable, but it may be a 
;                       number.	 If missing, then SHOT_NUMBER will be used with
;                       DIRXRD to select a SHOT/Line_of_SIGHT.
;       STRUCTURE     - Structure or structure array number of type 101
;                       Initial values are taken from this structure.
;         Note:  If SHOT_NUMBER / STRUCTURE is undefined, then dirxrd will be
;                used to select a Shot/Line_of_Sight
; KEYWORD PARAMETERS:
;       FILENAME      - Name of file to write data.
;                       If print is set and NoDelete is not set, 
;                       the default file is the scratch file used for some 
;                       directory PFIDL functions.
;       APPEND        - if set data will be appended to filename
;       Terminal      - If set, send output to terminal
;                       If less than zero then also print to file
;       PRINT         - If set, file will be printed.
;                       File will be deleted if unless NODELETE is set.
;       NODELETE      - If set filename will not be deleted.
;       DESTINATION   - Valid printer destination for the print file.
;                       This destination will not be saved by LPRINT
;                     Note: Default LPRINT destination must be valid if 
;                           DESTINATION is not provided.
;                           Use SET_PRINTER to set the destination.
;          
;  Structure Labels
;    See XRD_HEADER for description of type 101 Labels
;
; OUTPUTS:
;       Print header information to a file and spool to a printer
;
; EXAMPLE:
;       Print shot 681, los 5
;       print_xrd, 681, 5
;
; MODIFICATION HISTORY:
;       Written by:     L. P. Mix, January 29, 2001
;
;-
;/remote/lpmix/consult/dlfehl/xrd
 
on_error, 1

   common getdircom, junkfilename
   common pffcom, initpff, pff_exe
   common xrd_data_print_comm, str_com, def_filename, nfile
@xrd_common_data.dat
;
;     check to see if pff has been initialized

  if (n_elements(initpff) eq 0) then initpff = 0
  if (initpff eq 0) then init_pff
  if initpff eq 0 then begin
    print, '% PRINT_XRD: PFF libraries are not supported by this System'
    return
  endif

;
  if n_elements(nodelete) eq 0 then begin
    if keyword_set(printit) then nodelete = 0 else nodelete = 1
  endif else nodelete = nodelete[0]
  if n_elements(nfile) eq 0 then begin
     nfile =0L
     def_filename = 'XRD_printfile'
  endif
;
  t = size(strin, /type)
  if t eq 0 then begin
    print, '% PRINT_XRD: Structure must be defined.'
    return
  endif else if t eq 8 then str = strin else begin
    if (initxrd eq 0) then init_xrd
    shot = strin
    if n_elements (losin) eq 0 then los = ''  else los = losin
    if hfid ne 0 and shot gt 0 then begin
       str = 'Shot'+string (shot, format='(i5)') + ', LOS '
       if strlen(los) gt 0 then str = str + strupcase(strtrim(los, 2))+';;'
       get_dir, hfid, str, line=line, dataset = dataset, number = number
       if number eq 1 then readifl, str, /structure, dataset[0], fil=hfid  $
         else if number gt 1 then begin
         pick_one, line, n
         readifl, str, /structure, line[n], fil = hfid
       endif
    endif else begin
      print, '% PRINT_XRD: Unable to find shot header.
      return
    endelse
    if size (str, /type) ne 8 then begin
      print, '% PRINT_XRD: Unable to find shot header structure
      return
    endif
  endelse
;

  if str.type ne 101 then begin
    print, '% PRINT_XRD: Only type 101 structures may be output: ', $
       strtrim (str.type, 2)
    return
  endif
  if keyword_set(terminal) then begin
    print_xrd_actual, str, -1
    if terminal gt 0 then return
  endif
  if n_elements (filename) eq 0 then file = junkfilename else begin
    if strlen(strtrim(filename, 2))eq 0 then file = junkfilename else $
       file = filename
  endelse
   if n_elements(filename) gt 0 then def_filename = strtrim(filename[0], 2) $
     else begin
     if keyword_set(nodelete) then begin
      filename = def_filename+'_'+ strtrim(string(nfile, format='(i4.4)'), 2)
      nfile = nfile +1
      endif else filename = junkfilename
   endelse
   openw, unit, /get_lun, filename, error = err, append = append
      
   if err ne 0 then begin
     print, '% PRINT_XRD:  Unable to open print file
     print, !error_state.msg
     return
   endif
   print_xrd_actual, str, unit

   free_lun, unit
     
   if keyword_set(printit) then $
     lprint, filename, destination = destination, /nosave
   if not keyword_set(nodelete) then file_delete, filename
   end
