; ------------------------------------------------------------------------
; $Id: tgs_unfold.pro,v 1.5 2007/06/29 13:42:44 lpmix Exp $
; ------------------------------------------------------------------------
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
;+
; NAME:
;       TGS_UNFOLD
;
; PURPOSE:
;       Unfolds measured voltages to observed GW/eV/ster at each PIN diode,
;       by solving at lower grating orders at higher energies, then
;       finding the higher order contribution of these intensities at
;       the lower energy PINs (linearly interpolating between the data
;       as necessary). 
;
;       Then fits the unfolded power spectrum to a 1- or 2-Planckian curve,
;          by finding the best fit of kT(s) and area(s).
;
;       Current version is for LOS 5/6 and LOS 17/18.
;
;
; CALLING SEQUENCE:
;
;       TGS_UNFOLD [, SHOT_NUMBER] 
;
; OPTIONAL INPUTS:
;       SHOT_NUMBER - Shot Number
;
; KEYWORD PARAMETERS:
;       RERUN       - If set, dataset beginning with WORK will be read used for 
;                     the analysis.  The dataset comment for WORK must begin with
;                     "tgs"+"LOS Number" ie tgs5, or tgs17 or tgs29
;                     If rerun is set, waveforms are not filtered or baselined unless
;                       specifically requested with the BLINE and FILTER keywords.
;       ZOOMW       - If set, ZOOMW will be used to select waveforms
;       PIN_OFFSET  - Offset of the zero order from the design position.
;                     Default: PIN_OFFSET = 0.0
;                     PIN POSITIONS are:
;                     [15.055,12.043,10.035,8.601,7.526,6.689,6.020, $
;                     5.017, 4.3,   -3.763, -3.345,3.010,2.617,-2.315,-2.007,1.672]+ PIN_OFFSET
;                     ENERGY = 12398.42/4000/sin(atan(abs(xpin)/Slit_To_Detector))
;       KEEP_PARAM  - If set the calculated energies will be used in the calculation.
;                     Default: Keep_param = 1
;       DELTA_T     - If greater than zero, this will be used as the spacing 
;                     for the TGS unfold.
;                     Default:  DELTA_T = 0.5E-9
;       BLINE       - If set, a baseline adjustment will be made to the signals
;                     If bline eq -1, baseline will not be interactive.  The range 
;                     used for the first signal will be used for all of the remaining
;                     signals.
;                     Default: BLINE = 0  : KEYWORD_SET(RERUN)
;                              BLINE = -1 : otherwise (use same range for all waveforms)
;       Filter      - If not equal to zero, filter signals with a fourier 
;                     filter with DELTA = FILTER
;                     DEFAULT: Filter = 0     : KEYWORD_SET(RERUN)
;                              Filter = 1e-9  : otherwise
;       LOS         - Line of sight
;                     LOS options = [ 5, 17, 29]
;                     Default: LOS = 5
;                     Note: LOS 5/6, 17/18, 29/30 are true options.
;                     LOS_TYPE = [0, 1, 2] according to the 3 LOS options
;       HOHLRAUM    - If set, assume emission from an aperature or disk source
;       FRAC_PINCH  - Fraction of the pinch in view
;       THETA       - Viewing angle in the length direction, measured from 
;                     normal to cylinder surface in degrees
;       PHI         - Viewing angle in the azimuthal direction
;                     Not used for a pinch source
;       QE          - Internal quantum efficiency
;                     Default: QE = 1
;                     Sensitivity is QE/3.63
;       MAX_POWER   - Maximum Power
;                     Maximum calculated power
;       FDS_WRITE   - Frequency dependent write flag.
;                     If set, write a frequency dependent file for LASNEX
;       TMAXIMUM    - Time at maximum power
;       TFIT        - Temperature fit parameter - Meaning depends on value of WDFarea
;                     (WDFarea allows a specified area vs time.)
;                     If WDFarea is absent:
;                       1 - Fit to source temperature and area
;                       2 - Fit to two temperature distribution with 2 variable areas
;                     Default: FIT = 2 if WDFarea is undefined or zero.
;                     IF WDFarea is present, then 
;                       TFIT is not set (ie, absent or zero) then the specified area 
;                         will be used with a one temperature fit
;                       TFIT is set then the specified area will be used with temperature A
;                         and a temperature B and area B will also be used.
;       WDFarea     - If this keyword is set, the area will be set and a temperature varied
;                     If WDFarea is a float or a double, then its value will be used as a
;                       constant area.
;                     If WDFarea is an integer or a long and then it will be taken as a 
;                       WDF array number.  The WDF array is assumed to be the area vs time.
;
;                     AREA is in CM^2
;                     Note:  A single temperature is fit unless TFIT is defined.
;                            If KEYWORD_SET(TFIT), a three parameter fit is performed
;                                                 (T1, T2, A2) and A1 from the WDFarea
;       TAinit      - Initial temperature A value used for CURVEFIT
;                     Default = 75
;       AAinit      - Initial Area A value used for CURVEFIT
;                     Default = .18
;       TBinit      - Initial temperature B value used for CURVEFIT
;                     Default = 300
;       ABinit      - Initial Area B value used for CURVEFIT
;                     Default = .001
;       DEADLAYER   - Thickness of SiO2 surface layer in Angstroms
;                     Enter negative value to use tabulated DETSENS
;                     Default: DEADLAYER = 60
;       ABSORBT     - Thickness of the silicon absorbing layer in microns
;                     DEFAULT: SILICON = 15
;       DETSENS     - Detector sensitivity in ASCII file.
;                     First row is ignored - use for comments or leave blank
;                     Second and following records have two columns: energy vs sensitivity
;       GRATEFF     - Grating efficiency
;                     Only unique values for energy are used.
;                     ASCII file NP rows of 12 values where np defined by ENERGY
;                     First two rows are ignored - use for comments or leave blank
;                     First 4 rows are:
;                    "Kirchoff Efficiency Curves for Data from File hs16_data.txt
;
;                     Energy(eV)   m=1/m=0   m=2/m=1   m=0 (%)   m=1 (%)   m=2 (%)   m=3 (%)   m=4
;                     (%)   m=5 (%)   m=6 (%)   m=7 (%)   m=8 (%)
;                       "
;                     The third row is actually wrapped and is used to determine if 
;                     the format has changed.  The number of "%" are counted and used
;                     to determine the number of orders.  The total number of columns 
;                     counted by looking for multiple spaces.
;                     Energy must be first.  
;       SILICON     - File with SILICON optical constants
;                     ASCII File with 3 columns: energy, rn and rk
;                     Default:  SILICON = 'si.txt'
;                     First row is ignored - use for comments or leave blank
;       SAND        - File with Silicon Dioxide optical constants
;                     ASCII File with 3 columns: energy, rn and rk
;                     Default:  SAND = 'si02.txt'
;                     First row is ignored - use for comments or leave blank
;       WORK        - First WDF work array number
;                     Work must be greater than 0 and less than (get_maxwdf()-16)
;                       or it is set to DEFAULT
;                     DEFAULT:  WORK = get_maxwdf()-36
;       OUT         - First Output WDF array
;                     Note: OUT can overwrite some of the WDF work arrays.
;                     Note: If out LE 0 or GT (get_maxwdf()-16) then it is set to default.
;                     Default: OUT = get_maxwdf()-16
;       STRUCT      - First Structure output array.
;                     Default: STRUCT = get_maxstr()-5
;       ENERGY      - Energy of the PINs - Vector of NP values
;                     Note:  NP must be less than or equal to the number of rows in 
;                            GRATEFF and DETSENS
;                     Default:  LOS 5, NP = 15
;                      Energy =[ 250.0, 313.0, 375.0, 438.0, 500.0, 563.0, 625.0, $
;                        750.0, 875.0, 1000.0, 1125.0, 1250.0, 1625.0, 1875.0, 2250.0]
;
;                     LOS 17, NP= 14
;                     Energy =[ 156.250, 187.5, 218.750, 250.0, 281.250, 312.50, 375.0, 437.500, $
;                        500.0, 562.50, 718.750, 812.50, 937.50, 1125.0]
;       
; OUTPUTS:
;       Unfold the pins and calculate power, temperature, and source area.
;       Store in waveforms (various depending on solution type):
;         Temperature 1 and error estimate
;         Area 1 and error estimate
;         Temperature 2 and error estimate
;         Area 2 and error estimate
;         Power 1
;         Power 2
;         Total Power
;         Fit Standard deviation
;         Measured B value at peak power
;         Fit B value at peak power
;       Store in structures (3):
;         Measured spectrum (B-value), calculated spectrum(B-value), diffence
;
; RESTRICTIONS:
;       Top 20 waveforms are available to use in selecting waveforms.
;
; PROCEDURE:
;       The effective solid angle is given by
;       OMEGA = !pi*!pi/frac_pinch/cos(theta) ; PINCH configuration
;             = !pi/cos(theta)/cos(phi)       ; HOHLRAUM configuration
;       Power = OMEGA*AREA*(sigma/!pi)*(T^4)
;               where T is the fit temperature
;
; EXAMPLE:
;
;		F = PICKFILE(/READ, FILTER = ['pro', 'dat'])
;
; MODIFICATION HISTORY:
; 	Written by:	L. P. Mix, February 5, 2004
;	March 2, 2007	Added hohlraum source, fds write, and other functions
;-

pro tgs_unfold, shotnum, LOS = los, frac_pinch = frac_pinch, theta = theta, $
     deadlayer = zdead, qe = qe, absorbt = zdepl, detsens = detsens_f, tfit = tfit, $
     grateff = grateff_f, Silicon = si_f, sand = sio2_f, struct = struct, $
     work = work, out = out, zoomw = zoo, phi = phi, bline = bline, filter = filter,  $
     delta_t = deltat, rerun = rerun, PIN_offset= delz, keep_param= keep_param, $
     max_power = max_pow, TMAXIMUM= t_max, wdfarea = wdfarea, hohlraum = hohlraum, $
     fds_write = fds_write, TAinit = tainit, tbinit = tbinit, aainit = aainit, $
     abinit = abinit

@draw_defaults.com

common tgs_common, e3, area_1
;
on_error, 1
;
 if n_elements(tainit) eq 0 then tainit = 75d0
 if n_elements(tbinit) eq 0 then tbinit = 400d0
 if n_elements(aainit) eq 0 then aainit = 0.18d0
 if n_elements(abinit) eq 0 then abinit = 0.001d0
;

;
; Unfolds measured voltages to observed GW/eV/ster at each PIN diode,
; by solving at lower grating orders at higher energies, then
; finding the higher order contribution of these intensities at
; the lower energy PINs (linearly interpolating between the data
; as necessary).
;
; Then fits the unfolded power spectrum to a 1- or 2-Planckian curve,
; by finding the best fit of kT(s) and area(s).
;
; Get default values
pfidldir = get_pfidl_directory()
sep = path_sep()
if (strpos(pfidldir, sep, /reverse_search) +1 lt strlen(pfidldir)) then $
   pfidldir =pfidldir+sep
datadir = pfidldir+'pfidl'+sep+'xrd'+sep+'data'+sep

if n_elements(los) eq 0 then lostype = 0 else begin
  los = los[0]
  val = [5, 6, 17, 18, 29, 30]
  ok = where (los eq val, nok) 
  lostype = (nok eq 0) ? 0 : ok[0]/2
endelse

; see if everything is already defined.
;
if n_elements(shotnum) gt 0 and n_elements(los) gt 0 and $
  n_elements(frac_pinch) gt 0 and n_elements(theta) gt 0 and $
  n_elements(qe) gt 0 and n_elements(zdead) gt 0 and $
  n_elements(zdepl) gt 0 and n_elements(si_f) gt 0 and $
  n_elements(sio2_f) gt 0 and n_elements(grateff_f) gt 0 then $
    lc_check = 0 else lc_check = 1
if lc_check eq 0 then begin
  if zdead lt 0 then if n_elements(detsens_f) eq 0 then lc_check =1
endif
; if some items are missing, set defaults and check values
if lc_check eq 1 then begin
  if n_elements(shotnum) eq 0 then shotnum = 100
  get_shot_los_param, lcquit, shotnum, lostype
  if lcquit eq 1 then return

; get geometry parameters
  param_files = [ 'param_los5.txt', $
                  'param_los17.txt', $
                  'param_los29.txt' ]
  readasc, datadir+param_files[lostype], 7, x, y, np = np, $
    formatin = 0, skip =1, /string
  x = long(x)
  ok = where(shotnum ge x, nok)
  i = ok[nok-1]
  grating = y[i, 0]
  grating_period = long(y[i, 1])
  diode_aper_x = float(y[i, 2])
  diode_aper_y = diode_aper_x
  slit_width = float(y[i, 3])
  source_slit = double(y[i, 4])
  source_diode = double(y[i, 5])
  grating_diode = source_diode - source_slit
  shunt=50.
;
;  Spectral width of each diode in m=1 spectral order:
;
   dlambda1=diode_aper_x / grating_diode * grating_period
;
;  Collection angle (radians) in spectral direction (x):
;
   radians_x= slit_width/source_slit
;
;  Collection angle (radians) along slit length direction (y):
;
   radians_y=(diode_aper_y/source_diode)
;
;  Solid collection angle (sr):
;
   omega=radians_x*radians_y
;
       C4=dlambda1*omega*shunt

; help, grating , grat_per,  det_wid ,slit_wid,  source2slit,source2det

  if n_elements(frac_pinch) eq 0 then frac_pinch = 0.5
  if n_elements(theta) eq 0 then theta = 12.0
  if n_elements(phi) eq 0 then phi = 0.0
  if n_elements(qe) eq 0 then qe = 1.0
  if n_elements(zdead) eq 0 then zdead = 60.
  if n_elements(zdepl) eq 0 then zdepl = 15.
  if n_elements(si_f) eq 0 then si_f = datadir+'si.txt'
  if n_elements(sio2_f) eq 0 then sio2_f = datadir+'sio2.txt'
  sep = path_sep()
  if n_elements(grateff_f) eq 0 then $
    grateff_f = datadir+'grating_eff'+sep+grating+'_curves.txt' 
  info = file_info(grateff_f)
  if (info.exists + info.read) ne 2b then begin
    print, '% TGS_UNFOLD: File does not exist with read permission'
    print, '         File Name = ', grateff_f
    print, '         File Exists =', info.exists
    print, '         Read Permission =', info.read
    print, ' You must select another file or quit'
  endif

  if n_elements(detsens_f) eq 0 then detsens_f = datadir+'detsens.txt'
  get_tgs_parameters, lcquit, frac_pinch, theta, phi, $
      zdead, zdepl, si_f, sio2_f, detsens_f, grateff_f, hohlraum = hohlraum  
  if lcquit eq 1 then return
endif
; Solid angle divided by PI = omegadpi
; OMEGA = !pi*!pi/frac_pinch/cos(theta) ; PINCH configuration
;       = !pi/cos(theta)/cos(phi)       ; HOHLRAUM configuration
;
  omegadpi =  keyword_set(hohlraum) ? 1.0/cos(theta*!dtor)/cos(phi*!dtor) : $
                                    !pi/frac_pinch/cos(theta*!dtor)
;

 if n_elements(delz) eq 0 then delz= 0.0
 xpin=[15.055,12.043,10.035,8.601,7.526,6.689,6.020, $
     5.017, 4.3,   -3.763, -3.345,3.010,2.617,-2.315,-2.007,1.672]+delz

 e1 = 12398.42/grating_period/sin(atan(abs(xpin)/grating_diode))

 
if n_elements(work) eq 0 then work = get_maxwdf()-36
if (work lt 1) or (work gt (get_maxwdf()-16)) then WORK = get_maxwdf()-36

type_val=[5, 17, 29]
losnum = type_val[lostype]
data = 'tgs'+strtrim(losnum,2)
ldata = strlen(data)
shotnumber = strtrim (shotnum, 2)
rrun =0
if keyword_set(rerun) then begin
;   old versionjjjj
;   arr = get_match(data, /wdf, nel = n)
;   rrun = (n lt 3) ? 0 : 1
  wdf = wdvalid('^'+data, min= work, np = n)
  if n gt 3 then rrun = 1
endif else data = data

if rrun eq 0 then begin
  showpff, filename = filename, number = number, /quiet
  if number gt 0 then ok = $
    get_match(shotnumber, filename[0:number-1], regexp=0, nelements = number)
  if number eq 0 then openzz, shotnumber, ierror = ier else begin
    ier =0
    if number eq 1 then setpff, ok[0]+1 else begin
       pick_one, filename[0:number-1], pick, $
           inst='Pick the desired datafile', title ='Select File'
       setpff, ok[pick]+1
    endelse
  endelse

  if ier ne 0 then begin
    print, '% TGS_UNFOLD: Unable to open data file.'
    print, '              Open data file for shot '+shotnumber+' and call unfold again'
    return
  endif
  
  re, work, data, narray= n, /all, /sortit
  if n eq 0 then begin 
    print, '% TGS_UNFOLD: No datasets found with the string: ', data
    return
  endif
  wdf = work+indgen(n)
endif

; set up for initial 12 magnification  
imag = 12*2

tp = make_array(n)
for i = 0, n-1 do begin
  junk = mx(wdf[i], time =t, /quiet)
  tp[i]=t
endfor
limits, wdf, xr = xrange
; find energies
;
np  = 0
ee = make_array(n)
data_u = strupcase(data)

for i = 0, n-1 do begin
  comm = strupcase(wdcomment(wdf[i]))
  jj = strpos(comm, data_u)
  if jj ge 0 then begin
    ei = float(strmid(comm, ldata+jj))
    dele = min(abs(e1- ei), j)
    ee[np] = e1[j]
    wdf[np] = wdf[i]
    np +=1
  endif
endfor
e1 = ee[0:np-1]
n= np
wdf = wdf[0:np-1]
sum, wdf, get_maxwdf(), /quiet
junk = mx(get_maxwdf(), time = t, /quiet)
xr = [t-15e-9, t+25e-9]
zr = [xr[0]-6e-9, xr[1]+12e-9] 
; zr = [min(xrange, max = m), m]
; zr = median(tp)+[-1.0, 1.0]*(zr[1]-zr[0])/imag
; zoomw, wdf,xcur = xcur, ycur = ycur,maxzoom = 200,  zoomrange = zr, $
;   instructions= 'Indicate the region of interest with the cursor in the right window'
; if n_elements (xcur) lt 2 then begin 
;   Print, '% TGS_UNFOLD: Less than two points selected, entire curves will be plotted.'
;   xr = 0
; endif else xr = [min(xcur, max = m), m]
if n_elements (bline) eq 0 then bline = keyword_set(rerun) ? 0 : -1
;
; baseline correction
if bline ne 0 then begin
   if bline eq -1 then begin
     blin, wdf[0], range = range
     for i = 1, n-1 do sub, wdf[i], 0, ave(wdf[i], lef = range[0], right= range[1], /qu)
   endif else for i = 0, n-1 do blin, wdf[i], range = range
endif
if lostype eq 0 then begin
; Default:  LOS 5
  kT0=40.
;
endif else if lostype gt 0 then begin
; LOS 17
  kT0=25.
endif
np = n_elements(e1)

oldwin = !d.window
if oldwin lt 0 then begin
  window
  oldwin = !d.window
endif

window, title='TGS Detectors', xsize= bigsize[0], ysize = bigsize[1], /free
bigwin= !d.window

plo, wdf, g=n, over=0, chars=2, color = 0, xrange = xr, /eng, /xstyle

if oldwin ge 0 then wset, oldwin

if n_elements (keep_param) eq 0 then keep_param=1

select_waveforms, wdf, values, nwdf = nwdf, zoom = zoo, $
       xrange = xr, param = e1, keep_param = keep_param

wdelete, bigwin

if nwdf le 0 then return

s = sort(e1)
e1 = e1[s]
wdf = values[s]
e3 = e1*e1*e1
waveforms = wdcomment(wdf[0])
for i = 1, nwdf-1 do waveforms = waveforms+','+wdcomment(wdf[i])

if n_elements (filter) eq 0 then filter = ( keyword_set(rerun)) ? 0 : 1.0e-9
if (filter gt 0) then  for i = 0, nwdf-1 do filt, wdf[i], delt=filter

;
  workfunct = 3.63
  ampwatt = 1.0/workfunct
; qe = quantum efficiency 
  sensmax = qe * ampwatt
;
; GRATEFF(ip,m) is the calibrated grating efficiency (%) in order m
; at photon energy m*E1(ip);  for this TGS (#5), the calibrated
;
; ------------------------------------------------------------------------------
    grateffin =  read_grating_eff(grateff_f, ierror = ier, norder = norder)
    if ier ne 0 then begin
      print, '% TGS_UNFOLD: Error reading data file: ', grateff_f
      return
    endif
    egrat = reform(grateffin[0, *])
    ss = size (grateffin)
    ncol = ss[1]
    if (3+norder) ne 12 then $
       print, '% TGS_UNFOLD: Grating efficiency in different size file'
;   skip m=0 mode
    norder = norder-1
    nmodes = norder
    ist = ncol -norder
    energies = (1+indgen(norder))#e1

    result = findex(egrat, energies)
    grateff = make_array(norder, nwdf)
    for i = 0, norder-1 do $
      grateff[i, *] = interpolate (reform(grateffin[ist+i, *]), reform(result[i, *]))
    ok = where (energies gt egrat[ss[2]-1], nok)
    if nok gt 0 then grateff[ok]= 0.0

; Read silicon optical constants from file and calculate sensitivity
;   or read the detector sensitivity
;
  if zdead lt 0.0 or zdepl lt 0.0 then begin
; 
;   Read the detector sensitivity
;
;   Physical photon energy striking PIN diode ip from grating order m
;   is m*E1(ip):
;
;   DETSENS(ip,m) is sensitivity (Amps/Watt) of the PIN diode # ip
;   at photon energy m*E1(ip), which includes the ratio of actual
;   PIN # ip aperture area to nominal aperture area assumed in C3:
;
; ------------------------------------------------------------------------------
    readasc, detsens_f, 2, edetsen, detsensin, skip = 1, formatin = 0, error = ier
    if ier ne 0 then begin
      print, '% TGS_UNFOLD: Error reading file: ', detsens_f
      return
    endif
    energies = (1+indgen(nmodes))#e1
    ee = reform (energies, nmodes*nwdf, /overwrite)
    result = findex(edetsen, ee)
    maxv = edetsen[n_elements(edetsen) -1]
    detsens = interpolate(detsensin, result)
    ok = where (energies gt maxv, nok)
    if nok gt 0 then detsens[ok]= 0.0
;
  endif else begin
    readasc, si_f, 3, x, y, skip = 1, formatin = 0, error = ier
    if ier ne 0 then begin
      print, '% TGS_UNFOLD: Error reading file: ', si_f
      return
    endif
    energy = x
    rnsi = y[*, 0]
    rksi = y[*, 1]
;
; Read silicon dioxide optical constants from file
;
      readasc, sio2_f, 3, x, y, skip = 1, formatin = 0, error = ier
    if ier ne 0 then begin
      print, '% TGS_UNFOLD: Error reading file: ', sio2_f
      return
    endif
    rnsio2 = y[*, 0]
    rksio2 = y[*, 1]
;
; Check for consistant data
;
    if max((energy -x)/max(abs(x))) gt 1e-5 then begin
      print, '% TGS_UNFOLD:  Energy scale for Si and SiO2 are not the same.'
      return
    endif
    nener = n_elements(energy)
    d_energy = energy[1:nener-1]-energy[0:nener-2]
;
; convert the active region to angstroms.
;
    zdepl = zdepl*1.0e4
    
   
    APER = 1.0
    en = (1+findgen(nmodes))#e1
    pi4 = 4*!pi
;
;  Get rnsif, rksif, rnsio2f and rksio2f at the values of energy
;
    en = reform(en, nmodes*nwdf)
    wave = 12398.42/en
    zz = findex(energy, en)
    rnsif = interpolate(rnsi, zz)
    rnsio2f = interpolate(rnsio2, zz)
    rksif = interpolate(rksi, zz)
    rksio2f = interpolate(rksio2, zz)
;
; 0.2755 = Amps/Watt for no dead layer and infinitely thick depletion layer
;
; Tsio2 = transmission through dead (sio2) layer
; Asi = absorption within depletion (si) layer
;
    Tsio2=exp(-pi4*abs(zdead)*rksio2f/wave)
    arg=pi4*zdepl*rksif/wave
    ok = where( arg gt 80, nok)
    if nok gt 0 then arg[ok] = 0.0
    Asi=1.0-exp(-arg)
    if nok gt 0 then Asi[ok] = 1.0

;
;  Resulting detector efficiencies, calculated from transmission through
;  SiO2 dead layer of thickness "dead" and absorption through Si depletion
;  layer of thickness "zdepl", multiplied by assumed constant geometrical
;  aperture APER (no flat-fielding correction) and maximum diode sensitivity
;  "sensmax", replaces tabulated values of detector sensitivity "DETSENS",
;  when user inputs negative value for "dead".
;
    detsens=APER*sensmax*Tsio2*Asi
;
  endelse
  detsens = reform(detsens, nmodes, nwdf, /overwrite)
;
; cfac is = (del Angstroms / del eV)
;
   cfac=12398.43/(E1^2)
   eff = detsens*grateff*0.01
   for i = 1, 4 do  eff[i, *] = eff[i,*]/(i+1)
     b = make_array(nwdf)
     d = make_array(5, nwdf)
     kT40=kT0^4
     itime=0
;       write (2,16)
;   16  format (' Time(nsec)   B1    B2    B3    B4',
;     1 '    B5    B6    B7    B8    B9    B10',
;     2 '   B11   B12   B13   B14   B15'/) 
;       write (3,19)
;   19  format (/' Time(nsec)  kT1(eV)  kT2   A1(mm**2)  A2',
;     1          '   Power1(TW)  Power2  Power  Energy(kJ)')
;   25  continue
;
;  Reading in measured diode voltages, V(ip), from file.
;  For IDL use the values from the waveform
       kJoules=0.
;  Get the region for the unfold.
zoomw, wdf,xcur = xcur, ycur = ycur,maxzoom = 200,  zoomrange = zr, $
   instructions= 'Indicate the region for calculations with the cursor in the right window'

if n_elements(xcur) lt 2 then begin
  time0= xr[0]
  m = xr[1]
endif else  time0 = min (xcur, max = m)      

if (n_elements(deltat) eq 0) then deltat = 0.5e-9
nt = round(((m-time0)/deltat+1 ) > 3)
times = findgen(nt)*deltat+time0

yval = make_array(nt, nwdf)
bval = yval
errval = yval
funval = yval
;

error = make_array(nt)
pow1 = make_array(nt)
pow2 = make_array(nt)
tempa = make_array(nt)
tempb = make_array(nt)
areaa = make_array(nt)
areab = make_array(nt)
 err_aa = make_array(nt)
 err_ab = make_array(nt)
 err_ta = make_array(nt)
 err_tb = make_array(nt)

for i = 0, nwdf-1 do yval[*, i]= get_value(wdf[i], times)
;openw, unit, /get, 'tgs_data.dat'
;for i = 0, nt-1 do printf, unit, times[i], yval[i, *], format='(16e12.5)'
;free_lun, unit

; print, 'cfac ', cfac
if keyword_set(wdfarea) then begin
  if size(/type, wdfarea) gt 3 then begin
    areaval = make_array(n_elements(times), value = wdfarea[0])
    
    lcwdf = 1
  endif else begin
    lcwdf = (wdvalid(wdfarea, /val))[0]
    if lcwdf gt 0 then begin
      ifit = -1
      areaval = get_value(lcwdf, times)
      if min(areaval) le 0 then begin
        ok = where (areaval le 0, nok)
        ar_ave = total(areaval)/(nt-nok)
        print, '% TGS_UNFOLD: '+strtrim (nok, 2)+' Area Values set to '+strtrim(ar_ave, 2)
        areaval[ok]= ar_ave
      endif
    endif else lcwdf = 0
  endelse
  if lcwdf ne 0 then a = keyword_set(tfit) ? [tainit, tbinit, abinit] : tainit
endif else lcwdf = 0
if lcwdf eq 0 then begin
  if n_elements(tfit) eq 0 then tfit = 2
  ifit = tfit
;   a = [kt1opt, kt2opt, a1opt, a2opt]
  if (size(tfit, /type) ge 4) and (tfit gt 0) then begin
    area_1 = tfit
    a = keyword_set(tfit) ? [tainit, tbinit, abinit] : tainit 
    ifit = -1
  endif else a = (tfit eq 1) ? [tainit, aainit] : [tainit, tbinit, aainit, abinit]
endif

; wt  - Weight - Undefined
;kinv=11589.
 kinv=11604.5
; Stephan-Boltzmann constant sigma, in units of TW/cm**2/degK**4/sr
       sigma=5.6704e-24
;
 itmax = 120
 nvar= n_elements(a)
for itime  = 0, nt-1 do begin
v = yval[itime, *]
time = times[itime]
if keyword_set (lcwdf) then area_1 = areaval[itime]
  
; LOS 5 is lostype: 0
;
;  Unfolded (source) intensity at photon energy E1(ip) is
;  set as linear array B(ip), in units of GW/eV/sr.
;  It is the voltage measured at that diode (minus the contributions
;  to the voltage from spectral orders higher than m=1) divided by
;  the first order efficiency EFF(ip,1).
;
;  The contribution to the voltage of diode ip from spectral
;  order m is D(ip,m).  It is calculated as the source intensity
;  B(ip) times the efficiency of spectral order m, following the
;  unfolding of B(ip) at higher photon energies.  So, we start at
;  the highest photon energies, where the contributions from spectral
;  orders higher than m=1 are negligible (and are ignored), and then
;  bootstrap our way to the lower energies.
;
;      When the photon energy m * E1(ip) needed to calculate the higher
;      order contribution to V(ip), is not coincident with the center
;      energy of another diode, a linear interpolation of the nearest
;      adjacent diodes is performed.  The first example of this linear
;      interpolation is the next executable statement, where spectral order
;      m=2 times E1(10) of 1000 eV equals 2000 eV.  As there was no diode
;      centered at E1 of 2000 eV, the contributions from diodes 14 (1875 eV)
;      and 15 (2250 eV) are linearly interpolated, and stored in scalar
;      variable "D2000_2"
;
  calculate_bvalue, v, e1, d, eff, b, c4
;
;  Converting Watts/Angstrom/sr to GWatts/eV/sr
;
       B=1.0d-9 * cfac * B
;       print, (Time-Time0)*1.e9, B
       erropt=1.e20
;  
;  The best fit of the four Planckian parameters (2 kT's and 2 areas)
;  is determined by brute-force search (nested do-loops) in the two
;  temperatures and one of the areas (A1), followed by RMS analytical
;  fit to the second area (A2).
;  
;  kT41 = (kt)**4 of temperature 1
;  kT42 = (kt)**4 of temperature 2
   asave = a
   result = curvefit(e1,b,wt, a, sigma_error, /double, funct='tgs_funct', tol= 1e-4, $
     yerr = yerr, itmax=itmax, chisq = chisq, iter= iter, status = status)
   if status ne 0 then begin
     a = asave 
     sigma_error = a*0
   endif
     
;   if (ifit eq 2) && (a[3] lt 0) && (status eq 0)then begin
;     aa = [asave[0], asave[2]]
;     result = curvefit(e1,b,wt, aa, sigma_error, /double, funct='tgs_funct', tol= 1e-4, $
;       yerr = yerr, itmax=itmax, chisq = chisq, iter= iter, status = status)
;     if status eq 0 then begin
;;       a = [aa[0], asave[1], aa[1], 0.0]
;       sigma_error = [sigma_error[0], 0.0, sigma_error[1], 0.0]
;     endif else begin
;       a = asave
;       sigma = a*0
;     endelse
;   endif
     
;  if itime mod 8 le 1 then print, itime, result, b, yerr, result-b
 bval[itime, *] = b
 errval[itime, *] = b- result
 funval[itime, *] = result
 kt1opt= a[0]
 T1=kT1opt*kinv
 err_ta[itime] = sigma_error[0]
 if nvar eq 1 or nvar eq 3 then begin
   a1opt = area_1
   err_aa[itime] =  0.0
 endif
 if nvar eq 2 then begin
   a1opt= a[1] 
   err_aa[itime] =  sigma_error[1]
 endif else if nvar eq 4 then begin
   kt2opt= a[1]
   T2=kT2opt*kinv
   a1opt= a[2]
   a2opt= a[3]
   err_tb[itime] =  sigma_error[1]
   err_aa[itime] =  sigma_error[2]
   err_ab[itime] =  sigma_error[3]
 endif else if nvar eq 3 then begin
   kt2opt= a[1]
   T2=kT2opt*kinv
   a2opt= a[2]
   err_tb[itime] =  sigma_error[1]
   err_ab[itime] =  sigma_error[2]
 endif
; Calculating total emitted power (of viewed plasma) for optically thin cylinder
;
;       power1=A1opt*sigma/pi*T1**4*pi**2/frac_pinch/cos(theta)
; or    power = omegadpi*A1opt*sigma*T1**4
;       power2= omegadpi*A2opt*sigma*T2**4
;       power=power1+power2

 power1= omegadpi * A1opt*sigma*(T1^4)
 if nvar gt 2 then begin
   power2= omegadpi * A2opt*sigma*(T2^4)
   power=power1+power2
 endif else power = power1
 if(itime gt 1) then kJoules=kJoules+1.e+9*power*(Time-Timelast) 

 pow1[itime]= power1
 areaa[itime]= a1opt
 tempa[itime]= kT1opt
 error[itime]= yerr
 if nvar gt 2 then begin
   pow2[itime]= power2
   areab[itime]= a2opt
   tempb[itime]= kT2opt
 endif

 Timelast=Time

 
 endfor
if n_elements(struct) eq 0 then struct = get_maxstr()-5 
ss = (struct > 1) < (get_maxstr() -5)
c = ' - shot_num='+shotnumber+';;waveforms='+waveforms
arr2str, ss, bval, times, e1, clab= 'Measured Spectrum'+c, $
    ylab = 'Energy(eV)', xlab= 'Time(s)', dx = deltat
if keyword_set(fds_write) then fds_write, ss
arr2str, ss+1, funval, times, e1, clab= 'Calculated Spectrum'+c, $
    ylab = 'Energy(eV)', xlab= 'Time(s)', dx = deltat
arr2str, ss+2, errval, times, e1, clab= 'Spectrum Differences'+c, $
    ylab = 'Energy(eV)', xlab= 'Time(s)', dx = deltat
;deltat = deltat *1e9
xlab = 'Time (s)'
ylabt = 'Temperature (eV)'
ylaba = ' Area (mm^2)'
ylabp = ' Power (TW)'

;deltat = deltat*1e-9
 ptot = (nvar gt 2) ?  pow1+pow2 : pow1
 if n_elements(out) eq 0 then out = get_maxwdf()-16
 if out le 0 or out gt (get_maxwdf()-16) then out = get_maxwdf()-16
 max_pow = max(ptot, imax)
 put_wdfarray, [time0, deltat, tempa], out, clab = 'Temp_1'+c, $ 
      xlab= xlab, ylab = ylabt
 out+=1
 put_wdfarray, [time0, deltat, err_ta], xlabel= xlab, $
      ylab = ylabt, out, clab = 'Fit Error,T1 '+c 
 out+=1
 clab = (nvar gt 2) ? 'Area_1' : 'Source Area'
 put_wdfarray, [time0, deltat, 100*areaa], out, clab = clab+c, $ 
      xlab= xlab, ylab = ylaba
 out+=1
 clab = (nvar gt 2) ? 'Fit Error,Area_1' : 'Fit Error,Area'
 put_wdfarray, [time0, deltat, 100*err_aa], out,  xlabel= xlab, $
      ylab = ylaba, clab = clab+c 
 out+=1
 if nvar gt 2 then begin
   put_wdfarray, [time0, deltat, tempb], out, clab = 'Temp_2'+c, $ 
      xlab= xlab, ylab = ylabt
   out+=1
   put_wdfarray, [time0, deltat, err_tb], out, clab = 'Fit Error,Temp_2 '+c , $ 
      xlabel= xlab, ylab = 'Amplitude' 
   out+=1
   put_wdfarray, [time0, deltat, 100*areab], out, clab = 'Area_2'+c, $ 
        xlabel= xlab, ylab = ylaba
   out+=1
   put_wdfarray, [time0, deltat, err_ab], out, clab = 'Fit Error,Area_2 '+c , $ 
      xlabel= xlab, ylab = ylaba
   out+=1
 endif
; Power 1
 clab = (nvar gt 2) ? 'Power_1' : 'Total Power'
 put_wdfarray, [time0, deltat, pow1], out, clab = clab+c, $ 
      xlab= xlab, ylab = ylabp
 out+=1
 if nvar gt 2 then begin
; Power 2
   put_wdfarray, [time0, deltat, pow2], out, clab = 'Power_2'+c, $ 
      xlab= xlab, ylab = ylabp
   out+=1
   put_wdfarray, [time0, deltat, ptot], out, clab = 'Total Power'+c, $ 
      xlab= xlab, ylab = ylabp
   out+=1
 endif
 put_wdfarray, [time0, deltat, error], out, clab = 'Fit Standard Deviation'+c , $ 
      xlabel= xlab, ylab = 'Amplitude' 
 out+=1
 t_max = times[imax]
 c = ' - shot_num='+shotnumber+';; Max Power= '+strtrim(max_pow,2) + $
       ';; Time = '+ strtrim(t_max, 2) + 's'+';;waveforms='+waveforms
 put_wdfarray, [e1, reform(bval[imax, *])], out, clab = 'Measured Bvalue at peak power'+c, $
      xlabel= 'Energy(eV)', ylab = 'Amplitude', /xydata 
 out+=1
 put_wdfarray, [e1, reform(funval[imax, *])], out, clab = 'Fit Bvalue at peak power'+c , $ 
      xlabel= 'Energy(eV)', ylab = 'Amplitude', /xydata
 
bval = 0

 end
