; ------------------------------------------------------------------------ 
; $Id: ufo.pro,v 1.4 2007/06/29 13:42:44 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
;+
; NAME:
;       UFO
;
; PURPOSE:
;       Unfold XRD data.  The XRD channels are unfolded as a function of time
;       to generate a curve of output power vs time and temperature vs time.
;       The power output vs energy is saved for the highest output power.
;       This routine must be run on Sonic or Luigi
;
; CALLING SEQUENCE:
;       UFO [, SHOT_NUMBER]  [, Line_of_Sight]
;  or 
;       UFO [, STRUCTURE]
;
; INPUTS:
;       SHOT_NUMBER   - Existing shot number in the shot_config file.
;       Line_of_Sight - Existing Line_of_sight diagnostic array descriptor.
;                       In general this is a string variable, but it may be a 
;                       number.	
;       STRUCTURE     - Structure or structure array number of type 101
;                       Initial values are taken from this structure.
;         NOTE:  If a valid shot description is not provided, a directory
;                of the shot configuration file will be provided for selection
; KEYWORD PARAMETERS:
;
;       ReCalculate-Use the current matrix coefficient to calculate the unfold
;       Pinch_Area - Area of pinch for BB equivalent temperature.
;                    Default = 1 cm2 = 1e-4 m2
;                   
;       WDFarray -  Use WDF arrays for the XRD signals, cathode sensitivity,
;                   and filter transmission.
;
;       XRDSignal-  Starting storage location for storing XRD signals read
;                   from the shot data PFF file.
;                   Ignored if WDFarray is set
;                   Default is GET_MAXWDF() - 10*NDet
;                   XRD signals are assumed to have units: Volts into 50 ohms
;                   Selected signals are stored in GET_MAXWDF() - 6*NDet
;       Cathode  -  WDF array number to store the the cathode sensitivities.
;                   Default is next number after the last XRD.
;                   Ignored if WDFarray is set
;                   Copies are placed in GET_MAXWDF() - 5*NDet and above
;                   Cathodes are assumed to have units: Amps/Megawatt
;       Filter   -  WDF array number to store the the filter transmission
;                   Default is next number after the last Cathode
;                   Ignored if WDFarray is set
;                   Copies are placed in GET_MAXWDF() - 4*NDet and above
;                  Detector response is calculated in GET_MAXWDF() - 2*NDet 
;       Time     -  If time is a scalar then the unfold is only performed at
;                   the specified time.  If time is a vector with two elements,
;                   the unfold is performed between time[0] and time[1].
;                   If time has a third element, it is used as a skip factor.
;                   Default is start when the first signal exceeds 4% of peak
;                   ends when the last signal is less than 4% of peak.
;                   Default skip factor is 0.2 ns
;       Ntime    -  Number of time steps.
;                   Ignored if Time has three elements.
;       SaveStr  -  Structure with X-ray energy  as a function of 
;                   time and photon energy.
;                   Default = 1 (First structure)
;       Delta    -  FFT smoothing interval.  Default is 1.5 ns.
;       Width    -  Width used to calculate baseline.
;                   The following is used to handle baseline
;                      Assume signal in n
;                      m = mx(n, time = t)
;                      sub, n, 0, ave(n, right = t-width, /quiet)
;                   Default: Width = 50e-9
;
;       Quiet    -  If set, no output to the screen.
;                   Note:  All parameters must be uniquely defined.
;                          This will generally only work if WDFarray is set.
;
;   Solution Keywords:
;       Coefficient - Coefficient matrix where 
;                     Coefficient x Xrays = Voltages
;       SingleValues - Single values from the SVD of the coefficient matrix
;       Inverse     - Inverse of the coefficient matrix.
;
; INTERACTIONS:
;       If get_maxwdf() is less than 200 then "SET_MAXWDFARRAY, 200" is run, 
;         that is, the maximum number of wdf arrays is set to 200.
;       This routine uses the arrays [(get_maxwdf()-2-Ndet): get_maxwdf()]
;       as work arrays where NDet is the number of detectors being analyzed.
;       If WDFarray is not set UFO uses WDF arrays beginning with 85 to store
;       the XRD waveforms, cathode sensitivities, and filter transmissions.
;       
; OUTPUTS:
;       SaveSTR: Structure with Time/Energy X-ray unfold.
;                Default value = 1
;       WDF = get_maxwdf()-3 : Equivalent Temperature vs Time assuming area
;                              (eV)
;       WDF = get_maxwdf()-2 : Energy output at peak XRD signal.
;                              (W/sr/ev)
;       WDF = get_maxwdf()-1 : Energy output at peak output.
;                              (W/sr/ev)
;       WDF = get_maxwdf()   : Total power output vs time
;                              (Watts)
;
; PROCEDURE:
;       1. Obtain shot header
;       2. Open shot file and read XRD signals
;       3. Read Cathode sensitivity signals
;       4. Read Filter transmisson data
;       5. Subtract baseline from xrd signals, Filter, and determine one to use
;       6. Scale signals by dividing by the square of the diameter
;       6. Calculate Coefficient matrix
;       7. Calculate SVD
;       8. Get time steps
;       9. Calculate spectrum at each time step
;
; EXAMPLE:
;       Unfold data from shot 681, los 5
;       ufo, 681, 5
;
; MODIFICATION HISTORY:
;       Written by:     L. P. Mix, March 22, 2001
;
;-
;/remote/lpmix/consult/dlfehl/xrd
;pro please_select , lcquit, val, string_val, Instructions = instructions

 Pro ufo, shotin, losin, wdfarray = wdfarray, xrdsignal = xrd_data, $
      cathode = cathode, filter = filter, recalculate= recalculate, $
      time = time, ntime = ntime, savestr = savestr, delta = delta, $
      width = width, quiet = quiet, coefficient = coef, $
      singlevalues = singlev, inverse = inverse, pinch_area = pincharea

;
   on_error, 1
@xrd_common_data.dat
@ufo_common_savedata.dat
;
;  fft smoothing interval 
   delta_default = 1.5e-9
;  default start/stop fraction
   start_stop_def = 0.04
;  default skip factor
   skip_factor_def = 0.2e-9
;  check to see if pff has been initialized
   if (n_elements(initxrd) eq 0) then initxrd=0
   if initxrd eq 0 then init_xrd

   if initxrd eq 0 then begin
     print, '% EDIT_HEADER: XRD analysis is not supported by this System'
     return
   endif

   maxwdf = get_maxwdf()
   if maxwdf lt 512 then set_maxwdfarray, 512

   if keyword_set(recalculate) then begin
     if (size(str_save, /type) ne 8) then begin
       print, '% UFO: No valid data has been saved'
       Print, '       Recalculate is not possible'
       return
     endif
     coef = coef_save
     str = str_save
     arr = coef 
     ndet = str.ndet
   endif else begin
;     
;    Calculate a coefficient matrix assuming that we will solve for the power
;    per SR coming out of the pinch.
;
     MAKE_UFO_COEF, shotin, losin, coef = coef, str = str, $
     wdfarray = wdfarray, xrdsignal = xrd_data, $
     cathode = cathode, filter = filter, ierror = ierror
     if ierror ne 0 then begin
       print, '% UFO: Error generating coefficient matrix.'
       return 
     endif
     arr = coef 
     ndet = str.ndet

     SVdc, arr, w, u, v, /double
;    diagnostic 
     print, ' single Values: ' , w

     wp = make_array(ndet, ndet, /double)
     singlev = w
     mxsv = max(w) *1e-8
     for i = 0, ndet-1 do if (w[i] gt mxsv) then wp[i,i]=1.d0/w[i]
 
     inverse = v ## wp ## transpose(u)
     inverse_save = inverse
 
;    product of inverse and coef
     print, ' Product of inverse and coef; should be identity matrix'
     print, inverse ## coef

   endelse

     shot= str.shot
     los = str.los
     comm = str.comment
     dist = str.dist
     stype = str.source
     srcarea = str.src_area
     az_ang = str.az_ang
     Aper_hght = str.aper_ht
     Pinhght = str.pinch_ht
     ele_ang= str.elevation
     signal= str.signal[0:ndet-1]
     cathode= str.cathode[0:ndet-1]
     calibration = str.calibration[0:ndet-1]
     diameter = str.diameter[0:ndet-1]
     filter = str.filter[0:ndet-1]
     energy_bin = str.energy_bin[0:ndet]

 if n_elements (xrd_data) eq 0 then xrd_data = get_maxwdf()-10*ndet
 xrd= make_array (ndet+1, /long)

 if keyword_set(wdfarray) then begin
   xarr = make_array(256, /int)
   j = 0
   for i = 0, ndet-1 do begin
     c = get_match('^'+ signal[i], nele= n, /wdfarray)
     if n eq 0 then begin
        print, '% MAKE_UFO_COEF: Unable to find WDF arraY: ', signal[i]
        return
     endif
     xarr[j:j+n-1]= c
     xrd[i+1] = j+n
     j = j+n
   endfor

 endif else begin
   xrd[0] = xrd_data
   openzz, shot, ierror = ierror  
   if ierror ne 0 then begin
     print, '% UFO: This procedure requires access to data'
     print, '       using OPENZZ.'
     return
   endif
   
   for i = 0, ndet-1 do begin
      re, xrd[i], /all, '^'+ signal[i], narray = n
      if n eq 0 then begin
        closepff
        print, '% MAKE_UFO_COEF: Unable to read PFF dataset: ', signal[i]
        return
      endif
      xrd[i+1] = xrd[i]+n
   endfor
   closepff
   if n_elements (delta) eq 0 then delta = delta_default

   for i = xrd[0], xrd[ndet]-1 do filt, i, delta = delta

   c = make_array (xrd[ndet]-xrd[0] )
   for i = xrd[0], xrd[ndet]-1 do begin
     m = mx(i, time = t, /quieT)
     c[i-xrd[0]] = t
   endfor
   tpeak = median(c, /even)
   if n_elements (width) eq 0 then width = 50e-9
   rang = tpeak +[-width, 1.5*width]
;  subtract baseline
   for i = xrd[0], xrd[ndet]-1 do sub, i, 0, ave(i, right=rang[0], /quiet)
;  limit the data
   for i = xrd[0], xrd[ndet]-1 do lim, i, rang[0], rang[1]
   xarr = indgen(xrd[ndet]-xrd[0])+xrd[0]
   xrd = xrd- xrd[0]
 endelse
 j = -1
 st = get_maxwdf() - 6*ndet
 stsig = st
 for i = 0, ndet-1 do begin
   ii = j+1
   j= xrd [i+1]-1
   if j gt ii then begin
     plo, xarr[ii:j], /over, /color, /nl, /ns, /xst
     please_select, lcquit, n, wdcomment(xarr[ii:j])
     if lcquit gt 0 then return else ii = n+ii
   endif
   xfr, xarr[ii], st+i
 endfor
 if n_elements (rang) eq 0 then begin
   limits, indgen(ndet)+st, xrange = xrange, yrange= yrange
   rang = [min(xrange[0, *]), max(xrange[1,*])]
 endif

; prepend the date to each signal so that they will not be found if rerun.
 thindate =  thin_date()+', '
 lab, stsig+indgen(ndet), append=-1, thindate

 
; Note If volt(t) = array(ntime, ndet)
; then the xray spectrum = inverse ## volt(t)

 nelt = n_elements (time)

 if nelt eq 1 then begin
   nt = 1
   times = time[0]
 endif else if nelt ge 3 then begin
   t0 = (time[0] > rang[0]) < rang[1]
   tf = (time[1] > t0) < rang[1]
   skip = time[2]
   nt = long((tf-t0)/skip)+1
   times = (t0+ Findgen(nt)*skip) < rang[1]
 endif else begin
   if nelt eq 2 then begin 
     t0 = (time[0] > rang[0]) < rang[1]
     tf = (time[1] > t0) < rang[1]
   endif else begin
     stars = make_array(ndet) 
     stops = make_array(ndet) 
     sv = start_stop_def
     for i = 0, ndet-1 do begin
       rt = riset(stsig+i, startvalue = sv, value = v1)  
       ft = fallt(stsig+i, endvalue = sv, value = v2)  
       stars[i] = v1[0] > rang[0]
       stops[i] = v2[1] < rang[1]
     endfor
     print, stars, stops
     t0 = min(stars)
     tf = max(stops)
   endelse
   if n_elements(ntime) eq 0 then skip = skip_factor_def else $
     skip = (tf-t0)/((ntime -1) > 1)
   nt = long((tf-t0)/skip)+1L
   times = (t0+ Findgen(nt)*skip) < rang[1]
  help, t0, tf, nt
 endelse
 voltt = make_array(nt, ndet, /double)
 voltp = make_array(ndet)
 for i = 0, ndet-1 do begin
   junk = mx(stsig+i, time = tmax, /quiet)
   voltt[*, i] = get_value(stsig+i, times) > 0.0d0
   voltp[i] = get_value(stsig+i, tmax) > 0.0d0
   print, i, tmax, voltp[i]
 endfor
;
; Units of xrays and xraysp are MW/eV/sr
;
 xrays = inverse ## voltt
 xraysp= inverse ## voltp

; xrays is in units of total MWatts/eV/sr
; Check the answer to calculate the signals
 voltt_for = coef_save ## xrays
; 

 stforw  = get_maxwdf() - 2*ndeT
 for i = 0, ndet-1 do $
   i2w,times,reform(voltt_for(*,i)), stforw+i, $
   Clabel='Forward Voltage - XRD #'+strtrim(i+1, 2),  $
   Xlab= 'Time (s)', Ylabel = 'Voltage', /xydata
;
;  Delta energy is de
  de = energy_bin[1:ndet]-energy_bin[0:ndet-1]
  IntXrays = make_array(nt, ndet)

; Total power in each bin is INTxrays
  for i = 0, ndet-1 do INTxrays[*, i] = (xrays[*, i])*de[i]

  power = total(intxrays, 2) 
; Xrays is MW/SR /eV 
; Power is total radiated power by the pinch in units of
; Power is in MW/SR
; Use solid_angle to convert to Watts emitted
  solid_angle = 1e6*!pi*!pi
  power = power*solid_angle
  power_pk = total(xraysp*de)*solid_angle
  print , 'Peak Power = ' , max(power), ' Watts'
  print , 'Peak Power at Peak Signal = ', power_pk, ' Watts'

  energy = [0, 0.9999*energy_bin[0:ndet], $
    1.0001*energy_bin[0:ndet-1] ]

inde = sort(energy)
energy = energy[inde]
tmp = make_array(nt, ndet*2+2)
val = 2*(lindgen(ndet)+1)
xraysw = 1e6 *xrays ; xrays in watts/sr/ev
tmp[*, val]   =  xraysw
tmp[*, val+1] =  xraysw
tmpp = make_array(ndet*2+2)

xrayspw = 1e6* xraysp
tmpp[val]   =  xrayspw
tmpp[val+1] =  xrayspw

if n_elements(savestr) gt 0 then sav = get_save_str(SaveSTR, 1) else sav = 1
 arr2str, sav, tmp, Times, energy, xlab = 'Time (ns)', $
   ylab = 'Energy(eV)', clab = 'Power(W/sr/eV) vs Time and Energy'
 
; peak output
  maxenergy = max(power, imax)
  pk = reform(tmp[imax, *])
  tt = strtrim (times[imax]*1e9, 2)+' ns'

 i2w,times,power,Get_maxwdf(),Clabel='Total Output Power', Xlab= 'Time (s)', $
  Ylabel = 'Xray Power-W', /xydata
 
 i2w, energy, pk, get_maxwdf()-1, Clabel = 'Xray Spectrum at '+ tt, $
       Xlab= 'Xray Energy (eV)', Ylabel = 'Power (W/sr/eV)', $
       /xydata 
 i2w, energy, tmpp, get_maxwdf()-2, Clabel = 'Xray Spectrum at Peak Signal', $
       Xlab= 'Xray Energy (eV)', Ylabel = 'Power (W/sr/eV)', $
       /xydata 
if n_elements(pincharea) gt 0 then pa = pincharea[0]> 1e-6 else pa = 1.0e-4
 div, get_maxwdf(), 0, pa, get_maxwdf()-3
 temperature, get_maxwdf()-3, min = energy_bin[0], max = energy_bin[ndet]
 lab, get_maxwdf()-3, /append, ', Assumed area = '+strtrim (pa, 2) + ' Sq.M'

end
