; ------------------------------------------------------------------------
; $Id: henkep_film_resp.pro,v 1.3 2011/10/04 21:13:41 lpmix Exp $
; ------------------------------------------------------------------------
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
;+
; NAME:
;       henkep_film_resp  (from henke_film_resp)
; PURPOSE:
; 	HENKEP_FILM_RESP calculates the intensity of photons incident on 
; 	film type FILM given the film density and the wavelength of the
; 	incident radiation, and the angle of incidence theta.
;
; FORMAT:
;   PHOTONS = HENKEP_FILM_RESP( FILM, WL, DENSITY, THETA, TENSION, MODIFY)
;  Where:
;       Film    Film type
;   	        Film types: -1 - Kodak 101-07
;   	                     0 - DEF
;           	             1 - SB-392 (SB-5)
;   	                     2 - RAR 2492 
;  	                     3 - RAR 2495
;   	                     4 - RAR 2497
;       WL      Wavelength, Angstroms
;       Density Film Density - This may be one or two dimensions
;       Theta   Angle of incidence
;               Default: THETA = !pi/2
;       Tension Tension (alpha) for the spline
;       Modify  Modify = 0 Cubic Spline across all edges
;               Modify = 1 Linear Interpolation across edges
;               Default:  MODIFY = 0
; REFERENCE:
;	J.Opt.Soc.Am.B/Vol.1,No.6/December 1984, Henke et al.
; CATEGORY:
;       Grating Spectroscopy, Vida
; INPUTS:
;       film is the film type
;       wl is the wavelength array
;       density is the corresponding film density array
;       theta is the array of angles of incidence on the film
; OUTPUTS:
;       photons is the output array
; COMMON BLOCKS:
;       none
; MODIFICATION HISTORY:
;       Written by:     L. Paul Mix, September 1, 2004
;                       From the VIDA, D2I, routine by 
;                         William E. Nelson
;                         Science and Engineering Associates
;                         March, 1990
;-

Function  henkep_film_resp, film, wl, density, theta, tension, modify

on_error, 2

  if n_elements(modify) eq 0 then modify = 0
  if n_elements(theta) eq 0 then theta = !pi* 0.5

; How many density points have been passed

  ss = size(density)
  ndim = ss[0]
  if ndim gt 2 then begin
    print, '% HENKEP_FILM_RESP: Present version limited to one and two dimensions'
    help, density
    return, 0
  endif
  if ndim eq 1 then ss[2] = 1
  photons = (ndim eq 1) ? fltarr(ss[1]) : make_array(ss[1],ss[2])
  if film eq 0 then begin
    photold = (ndim eq 1) ? fltarr(ss[1]) : make_array(ss[1],ss[2])
    check = (ndim eq 1) ? fltarr(ss[1]) : make_array(ss[1],ss[2])
  endif
;
; Number of wavelengths passed
;
  nwl = n_elements(wL)
  if nwl ne ss[1] then begin
    print, '% HENKEP_FILM_RESP: Wave length array must be the same length'
    print, '           as the first dimension of DENSITY'
    help, WL
    help, density
    return , 0
  endif

  nophotons = where(density le 0.001, nok)
  if nok gt 0 then density(nophotons) = 0.0

; Calculate absorption coefficients amu0 and amu1
  absorption_coef, wl, amu0, amu1, tension, modify

  if nwl eq 1 then begin
      amu0 = amu0[0]
      amu1 = amu1[0]
  endif

;  i2w, wl, amu0, 201, /xy, clab='amu0 - new'
;  i2w, wl, amu1, 202, /xy, clab='amu1 - new'

  lim = machar()

  IF (film lt 0) THEN BEGIN

; Henke film parameters for Kodak 101-07
      a1 = 1.957
      b1 = 0.3128
      t1 = 2.0

;
; Kodak 101-07
;

;     Calculate Beta1 for Kodak 101-07
;     NOTE: The exponential may result in an underflow, but it does not affect
;          the result on the Solbourne or Sun.
      sintheta = sin(theta)
      arg = -amu1*(t1/sintheta)
      beta1 = make_array(nwl, value = sintheta)
;     The exponential will be subtracted from 1 so ignore very small values
      ok = where (arg gt 1.1*alog(lim.eps), nok)
      if nok gt 0 then beta1[ok] *= (1.0 - exp(arg[ok]))
      bbeta = (b1*Beta1)

;     Two parameter empirical relation for Kodak 101-07

      photons = density < (a1 - 0.001)
      photons = alog(a1/(a1-photons))
      if ndim eq 1 then photons = photons/(bbeta) else $
        for i = 0, ss[2]-1 do photons[*, i] = photons[*, i]/bbeta
;     eqn 7 from henke josaB 1,818 (1984)
      return, photons

  endif else begin

;
; Henke film parameters for DEF, SB-392 and the RAR films
;
;     AgBr volume fraction
      v = [0.40, 0.20, 0.10, 0.20, 0.10]

; Emulsion thickness (micrometer)
;     t = [13.0, 11.3, 7.0, 7.0, 7.0]
;     changed at request of Iain Hall, Jim Bailey
      t = [13.0, 10.0, 7.0, 7.0, 7.0]

; AgBr grain size (micrometer)
      d = [1.6, 1.0, 0.3, 0.3, 0.3]

; Effective emulsion layer thickness (micrometer)
      d0 = [2.0, 1.5, 0.6, 0.6, 0.6]

; Supercoat thickness (micrometer)
      t0 = [1.0, 1.0, 0.3, 0.3, 0.3]

; Empirical film parameter (1/micrometer)
      a = [0.68, 0.285, 0.527, 0.528, 0.414]
;     changed at request of Iain Hall, Jim Bailey
;     a = [0.68, 0.545, 0.527, 0.528, 0.414]

; Empirical film parameter (micrometer^2)
      b = [1.69, 1.41, 0.372, 0.926, 0.454]
;     changed at request of Iain Hall, Jim Bailey
;     b = [1.69, 1.39, 0.327, 0.926, 0.454]

; Polyester thickness for DEF (micometer)
      tb = 185.

; Low energy, film coefficients
      IF (max(wl) GT 13.) THEN BEGIN
          t13    = t		; Emulsion thickness (micrometer)
          t13(1) = 10.0
          a13    = a		; Empirical film parameter (1/micrometer)
          a13(1) = 0.285
          b13    = b		; Empirical film parameter (micrometer^2)
          b13(1) = 1.41
      ENDIF

      ile13 = where(wl le 13., comp = igt13, nok, ncomp = ncomp)

      ; SB-392 or RAR films

      ; Calculate the absorption coefficient amu
      amu = amu0 - (1./d(film))*alog(1. - v(film)*(1. - exp(-(amu1 - amu0)* $
                   d(film))))

      ; Calculate the photon energy dependent factors Alpha and Beta
      alpha = amu/(sin(theta)+amu*d0(film))        ; High energy aproximation

;      IF (min(igt13) ge 0) THEN begin             ; Low energy correction
;        if nwl eq 1 then alpha = alpha/(1. + alpha*d0(film)) $
;        else alpha(igt13) = alpha(igt13)/(1. + alpha(igt13)*d0(film))
;      ENDIF

      betac = (1. - exp(-amu1*d(film)))*exp(-amu0*t0(film)/sin(theta))
      ;
      ; Two parameter equation for THICK (completely absorbing) EMULLSION
      ; response.
      ;
      IF (nok gt 0) THEN begin
        if nwl eq 1 then photons = (exp(alpha*density/a(film)) - 1.)/(b(film)*betac) $
        else begin 
          if ndim eq 1 then $
            photons(ile13) = (exp(alpha(ile13)*density(ile13)/a(film)) - 1.)/$
                               (b(film)*betac(ile13)) else for k = 0, ss[2]-1 do $
            photons(ile13, k) = (exp(alpha(ile13)*density(ile13, k)/a(film)) - 1.)/$
                               (b(film)*betac(ile13))
        endelse
      ENDIF

      IF (ncomp gt 0) THEN begin
        if nwl eq 1 then photons = (exp(alpha*density/a13(film)) - 1.)/(b13(film)*betac) $
        else begin
          if ndim eq 1 then $
            photons(igt13) = (exp(alpha(igt13)*density(igt13)/a13(film)) - 1.)/$
                               (b13(film)*betac(igt13))  else for k = 0, ss[2]-1 do $
            photons(igt13, k) = (exp(alpha(igt13)*density(igt13, k)/a13(film)) - 1.)/$
                               (b13(film)*betac(igt13))
        endelse
      ENDIF

      ; Two parameter equation for THIN EMULLSION response.

    ;  ilt12 = where(wl LT 12.40)
    ;  IF (min(ilt12) ge 0) THEN begin
    ;    if nwl eq 1 then photons = photons / (1. - exp(-amu*t(film)/sin(theta) + $
    ;                                alpha*density/a(film))) $
    ;    else photons(ilt12) = photons(ilt12) / $
    ;      (1. - exp(-amu(ilt12)*t(film)/sin(theta(ilt12)) + $
    ;          alpha(ilt12)*density(ilt12)/a(film)))
    ;  ENDIF

      if ndim eq 1 then $
        photons = photons / (1. - exp(-amu*(t(film)/sin(theta)) + $
                                    alpha*density/a(film))) $
      else for k = 0, ss[2]-1 do $
        photons[*, k] = photons[*, k] / (1. - exp(-amu[*]*(t(film)/sin(theta)) + $
                                    alpha*density[*, k]/a(film)))
           
      photons = photons > 0.

  endelse
  ; Double emulsion response for DEF, uses thin emulsion response as 
  ; starting guess.
  IF (film eq 0) then begin

      ; Different alpha for high energy film 
      ; (see Henke et al "High-Energy x-ray response of photographic films:
      ; models and measurement")

      alpha = amu/sin(theta)

      ; vectorized convergence routine

      epsilon = 0.001

      bb = (amu*(t(film)/sin(theta))) < 84.
      cc = ((amu0*tb + amu*(2*t[film]))/sin(theta)) < 84.
      dd = ((amu0*tb + amu*t[film])/sin(theta))  < 84.
      if ndim eq 1 then  begin
        aa = alpha/a(film)*density

;       Ignore photons less than or equal zero
        nconv = where(photons gt 0., nok)

        while (nok gt 0) do begin
          photold(nconv) = photons(nconv)
          if nwl eq 1 then $
            photons(nconv) = (exp(aa(nconv))  * $
              (1.+b(film)*betac*photold(nconv)*exp(-bb)) * $
              (1.+b(film)*betac*photold(nconv)*exp(-cc)) / $
              (1.+b(film)*betac*photold(nconv)*exp(-dd)) - 1.) / $
              (b(film)*betac) $
          else $
            photons(nconv) = (exp(aa(nconv)) * $
              (1.+b(film)*betac(nconv)*photold(nconv)*exp(-bb(nconv))) * $
              (1.+b(film)*betac(nconv)*photold(nconv)*exp(-cc(nconv))) / $
              (1.+b(film)*betac(nconv)*photold(nconv)*exp(-dd(nconv))) - 1.) / $
              (b(film)*betac(nconv))
        
          check(nconv) = abs(1. - photons(nconv)/photold(nconv))
          nconv = where(check gt epsilon, nok)
        endwhile
      endif else begin
        for k = 0, ss[2]-1 do begin
          aa = alpha/a(film)*density[*, k]

;         Ignore photons less than or equal zero
          nconv = where(photons[*, k] gt 0., nok)
           
          while (nok gt 0) do begin
            photold(nconv, k) = photons(nconv, k)

            if nwl eq 1 then $
              photons(nconv, k) = (exp(aa(nconv))  * $
                (1.+b(film)*betac*photold(nconv, k)*exp(-bb)) * $
                (1.+b(film)*betac*photold(nconv, k)*exp(-cc)) / $
                (1.+b(film)*betac*photold(nconv, k)*exp(-dd)) - 1.) / $
                (b(film)*betac) $
            else $
              photons(nconv, k) = (exp(aa(nconv, k)) * $
                (1.+b(film)*betac(nconv, k)*photold(nconv, k)*exp(-bb(nconv, k))) * $
                (1.+b(film)*betac(nconv, k)*photold(nconv, k)*exp(-cc(nconv, k))) / $
                (1.+b(film)*betac(nconv, k)*photold(nconv, k)*exp(-dd(nconv, k))) - 1.) / $
                (b(film)*betac(nconv, k))

            check(nconv, k) = abs(1. - photons(nconv, k)/photold(nconv, k))
            nconv = where(check[*, k] gt epsilon, nok)
        endwhile
        endfor 
      endelse
  endif
  check =0
  photold =0
  return, photons
end

