; ------------------------------------------------------------------------ 
; $Id: write_cathode.pro,v 1.5 2008/02/27 22:34:44 lpmix Exp $ 
; ------------------------------------------------------------------------ 
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
;+
;  This procedure writes a WDF array to a PFF file.
;  Format: WRITE_CATHODE, WD1 [, FILEID] [, FILEID = FID] [, QUIET = quiet]
;          [, Name = Cathode_Name] [, IDENTIFIER = cathode_ID]
;          [, CALIBRATION = calibration] [, DATE= fit_date]
;             
;  where:
;    WD1    - WDF array number
;    FILEID - PFF file ID (Keyword is ignored if this parameter is present.
;  Keywords:
;    FILEID - PFF file id.  If fileid  is 0, the current PFF 
;             file is used. If fileid is not specified, then the file
;             1. If the unfold routine has been initialized, use the cathode
;                response file.
;             2. Use the file 'cathode_resp.pff' that is open, in the current
;                directory, or in the directory, !dir/../xrd
;             Keyword is ignored if parameter is provided.
;    QUIET  - If set, file will be written with no interaction.
;             Default: quiet = 0
;             Note:  If quiet is set, dataset comment must contain a string:
;             "Cathode: cathode_id," where cathode ID is the cathode id for the
;             cathode.
;    Name        - Name of the cathode
;                  Example: XRD
;    IDENTIFIER  - Cathode identification
;                  Example: C20
;    Calibration - Calibration name or identifier
;                  Example: B0008
;    DATE   - Fit date
;             Example: Jan 17, 2002
;
;  Examples:
;  Write WDF array 1 to the current PFF file.
;    write_cathode, 1 , 0
;  Write WDF array 5 to the current PFF file with a cathode name of C26 and 
;  a calibration identifier of 9906
;    write_cathode, 5, name ='c26', cal=9906, fid = 0
; or
;    write_cathode, 5, 0, name ='c26', cal='9906'
;
;-

 pro write_cathode_event, event
   widget_control, event.id, get_uvalue = ev
   CASE Ev OF

     'RUN/QUIT': BEGIN
        widget_control, event.top, get_uvalue= info
        (*info.ptr).lcquit= event.value
        if event.value eq 0 then begin
          widget_control, info.wcath, get_value = cath
          widget_control, info.wcal, get_value = calib
          cath = strtrim(cath[0], 2)
          calib = strtrim(calib[0], 2)
          if strlen(cath) eq 0 or strlen(calib) eq 0 then begin
            widget_control, event.top, tlb_get_offset=offset
            wwarning, ['Cathode and calibration identifiers must be provided', $
                       'Please insure you have identifiers for both'], $
                        offset = offset +10
            return
          endif
          (*info.ptr).cathid= cath
          (*info.ptr).calib= calib
          widget_control, info.wname, get_value = name
          (*info.ptr).name= strtrim(name[0], 2)
          widget_control, info.wdate, get_value = date
          (*info.ptr).date= strtrim(date[0], 2)
         endif
         widget_control, event.top, /destroy
        END
      else: help, /str, event
   endcase
 end

 pro write_cathode, wdfin, fidin, name = namein, fileid = fileid, quiet=quiet, $
         identifier = cathodeid, calibration= calibration, date = fitdate

on_error, 1
@xrd_common_data.dat
common pffcom, initpff, pff_exe

      if n_params() eq 0 then begin
        print,'% Format:  WRITE_CATHODE, WD1 [, FID] [, FILEID = FID] [/QUIET]'
        print,'             [, Name = cathode_name] [, IDENTIFIER = cathode_id]'
        print,'             [, CALIBRATION = calibration] [, DATE= fitdate]'
        return
      endif

      if (n_elements(initpff) eq 0) then init_pff else $
        if initpff eq 0 then init_pff
      if (initpff eq 0) then begin
        print, '% WRITE_CATHODE: PFF is not supported by this system'
        return
      endif

 showpff, filename = fname, ufid = opfid, numb = nfile, /quiet
 fidmiss = max(opfid)+1
 if n_elements(fidin) gt 0 then fid= fidin[0] else begin
   if n_elements(fileid) gt 0 then fid = fileid[0] else begin
     if n_elements(cfid) gt 0 then begin
       fid = cfid[0]
       ok = where(fid eq opfid, nok)
       if nok eq 0 then fid = fidmiss else begin
         if fname[ok[0]] eq cathode_file then begin
           setpff, fid
           fid = 0
          endif else fid = fidmiss
       endelse
     endif else fid = fidmiss
   endelse
 endelse
 if fid eq  fidmiss then  begin
   ok = where (strpos(fname, 'cathode_resp.pff') ge 0, nok)
   if nok eq 0 then begin
     sep = path_sep()
     new = !dir+sep+'..'+sep+'xrd'+sep+'cathode_resp.pff'
     openpff, new, error = error
     if error ne 0 then begin
       print, '% WRITE_CATHODE: Not able to open file: ', new
       return
     endif
   endif else fid = opfid[ok[0]]
 endif
 if keyword_set(fid) then begin
   setpff, fid, ier
   if ier ne 0 then begin
     print, '% WRITE_CATHODE: Unable to set file id:', fid, ier
     return
   endif
 endif

 wdf = wdvalid(wdfin[0], /valid)
 if wdf[0] gt 0 then wdf = wdf[0] else begin
   print, '% WRITE_CATHODE: Valid waveform array must be provided', wdfin[0]
   return
 endelse

 comm = wdcomment(wdf)

 ok = strpos(comm , ' - ')
 if ok lt 0 then comment ='' else begin
   comment = strtrim(strmid(comm, ok[0]+3), 2)
   comm = strmid(comm, 0, ok)
 endelse
 commu = strupcase(comm)

 ss = strtrim(strsplit(commu, ',', /extract), 2)
 nss = n_elements(ss)
 indss = indgen(nss)

 def_name = ''
 def_cathodeid = ''
 def_cal = ''
 def_fitdate =''
 sstr = 'CATHODE:'
 ok = (where(strpos(ss, sstr) ge 0, nok))[0]
 if nok gt 0 then begin
   def_cathodeid = strtrim(strmid(ss[ok], strpos(ss[ok], sstr)+8), 2)
   indss[ok]=-1
 endif else indss = [indss, 0]


 ok = (where(strpos(ss, 'CAL:') ge 0, nok))[0]
 if nok gt 0 then begin
   def_cal= strtrim(strmid(ss[ok], strpos(ss[ok], 'CAL:')+4), 2)
   indss[ok]=-1
 endif
 isp = ''
 i = 0
; Take the preliminary data as a name
 while indss[i] ge 0 and i lt nss do begin
   def_name = def_name+isp+ss[i]
   i = i+1
   isp = ', '
 endwhile
 isp = ''
 iend = i
 i = nss-1
 while indss[i] ge 0 and i ge 0 do begin
   def_fitdate = ss[i]+isp+def_fitdate
   i = i-1
   isp = ', '
 endwhile

 if n_elements(namein) eq 0 then name =strtrim(def_name, 2) else $
     name = strtrim(namein, 2)
 if n_elements(calibration) eq 0 then calib =strtrim(def_cal, 2)  else $
     calib = strtrim(calibration, 2)
 if n_elements(fitdate) eq 0 then date = strtrim(def_fitdate, 2)  else $
     date = strtrim(fitdate, 2)
 if n_elements(cathodeid) eq 0 then cathid = strtrim(def_cathodeid, 2)  else $
     cathid = strtrim(cathodeid, 2)

 if keyword_set(quiet) then begin
   if strlen(cathid) eq 0 then begin
     print, '% WRITE_CATHODE: Cathode ID must be provided if QUIET is set'
     return
   endif
   if strlen(calib) eq 0 then begin
     print, '% WRITE_CATHODE: Calibration name must be provided if QUIET is set'
     return
   endif
 endif else begin
   TLB = WIDGET_BASE( COLUMN=1, MAP=1, $
      TITLE='Enter Cathode Parameters')
   cinfo = comm
   outcom = comment
   if strlen(comment) gt 0 then begin
;    use a dummy to save the end.
     comm = comment
     x = strlen(comm)
     while x gt 0 do begin
       ok = strpos(comm, ';;')
       if ok gt 0 then cinfo = [cinfo, strmid(comm, 0, ok[0])] else $
         cinfo = [cinfo, strmid(comm, 0)]
       if ok gt 0 then comm = strmid(comm, ok+2) else comm=''
       x = strlen(comm)
     endwhile
   endif
   ncomm = n_elements(cinfo)
   for i = 0, ncomm -1 do ntxt = widget_label(tlb, value= cinfo[i])

   ntxt = widget_label(tlb, value= ' - - - - - - - - - - - - - - - - ')

   wname = cw_field(tlb, value = name, /row, /string, $
      title ='Cathode Name:')
   wcath = cw_field(tlb, value = cathid, /row, /string, $
      title ='Cathode Ident:')
   wcal = cw_field(tlb, value = calib, /row, /string, $
      title ='Calibration Ident:')
   wdate = cw_field(tlb, value = date, /row, /string, $
      title ='Fit Date:')
   bnames = [ $
    '  Write/Save  ', $
    '  Cancel/Quit ' ]
   BGROUP = CW_BGROUP( TLB, bnames, $
      ROW=1, $
      SPACE=100, $
      UVALUE='RUN/QUIT')
   str = {lcquit: 0, name:name, cathid:cathid, calib:calib, date:date }
   ptr = ptr_new(str)
   info = {ptr: ptr, wname:wname,wcath:wcath, wcal:wcal, wdate:wdate}
   widget_control, tlb, /realize
   widget_control, tlb, set_uvalue=info
   xmanager, 'write_cathode', tlb
   lcquit =(*ptr).lcquit 
   name = (*ptr).name
   cathid = (*ptr).cathid
   calib = (*ptr).calib
   date = (*ptr).date
   ptr_free, ptr
   if lcquit ne 0 then return
 endelse
 tmpnam = strtrim(name, 2)
 tmplab = 'Cathode: ' + strtrim(strupcase(cathid), 2)
 get_dir, comment = comment , number = number
 if number gt 0 then begin
   ok = where (strpos(comment, tmplab) ge 0, nok)
   if nok gt 0 then begin
     comment = ['The following datasets are already in the file:', comment[ok]]
     wok_notok, lcquit, comment, 'Cathode Already Exists'
     if lcquit eq 1 then begin
       Print, '% WRITE_CATHODE: Nothing written to the PFF file.'
       return
     endif
   endif
 endif
 tmp = tmplab

 if strlen (calib) gt 0 then tmp= tmp+', Calibration: '+ strtrim(strupcase(calib), 2)
 if strlen (date) gt 0 then tmp= tmp+', Date: '+date
 if strlen(tmpnam) gt 0 then tmp= tmp+', '+tmpnam 
 if strlen(outcom) gt 0 then tmp = tmp+' - '+ outcom

 wri, wdf, clab = tmp , ap = 102, /full, tlabel = 'Cathode Resp'
 end
