; ------------------------------------------------------------------------ 
; $Id: readwd3.pro,v 1.4 2007/06/06 20:03:52 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
  Pro readwd3, file, wdfarray = wdfarray, npoints = npoints, xdata = xdata, $
      xrange = xrange, clabel = clabel, xlabel = xlabel, ylabel = ylabel, $
      ydata = ydata, file = filein, ier = ier
;+
; NAME:
;       READWD3
;
; PURPOSE:
;       Read fit parameters from a WDFIT3 fit file
;
; CALLING SEQUENCE:
;
;       READWD3 [, File_Name] [, WDFARRAY = wdf] [, NPOINTS = npoints]
;         [, XDATA = data] [, YDATA = ydata]
;         [, XRANGE = xrange] [, CLABEL = clabel] [, XLABEL = xlabel]
;         [, YLABEl = ylabel] [ , FILENAME = filename] [, IERROR = ier]
;
; INPUTS:
;       File_Name:  If undefined or zero, PICKFILE will be used
;                   Data will be read into the WDFIT3 common block
;       IERROR:     If no error occurs, zero is returned.
;       WDFARRAY:   If set, fit curve will be stored in the 
;                   specified WDF array, an empty array, or in get_maxwdf().
;                   If WDF is a variable, the value will be returned.
;                   For example: wdf =-1
;                                READWD3, wdf = wdf
;                       Upon return wdf will be a number designating the actual
;                       WDF array where the data is stored.
;                   If XDATA is defined, these values will be used.
;       XDATA:      If present, these data values will be scaled by WDFIT3
;                     curve.  XDATA must have 3 or more elements.
;       YDATA:      If xdata is present, YDATA contains the corrensponding values
; *********************
;       NOTE:
;       The following keywords are ignored unless WDFARRAY is defined.
; *********************
;       NPOINTS:   Number of points used to calculate the response
;                  NPOINTS = NPOINTS > 32
;                  Default: 500
;                  NPOINTS is ignored if XDATA has 3 or more points.
;       XRANGE:    Range of the response curve
;                  Xrange will be adjusted so that only valid fit data is used.
;                  If range is zero, the default value is used.
;                  Default: Valid range of the fit
;       CLABEL:    Dataset comment (used for title of plots)
;                  Default: clabel ="Response - Filename=XXX;;DATETIME=YYY"
;                  where XXX is the file containing the step wedge data
;                        YYY is the date the the fit was calculated
;                  Note: "Plo, /truncate" will terminate the title at " - "
;       XLABEL:    X-axis (abscissa) label
;                  Default: "Film Density"
;       YLABEL:    Y-axis (ordinate) label
;                  Default: "Film Exposure"
;       FILENAME:  File or source of these dataset values
;                  Default: File_Name
;
; EXAMPLE:
;       Read output from a fit and generate a response curve from 0.2 to 4 
;       in WDF array 30.  Use pickfile to get the datafile.
;         READWD3                    ; read the fit parameter
;         data = .2+findgen(500)/499*3.8 
;         scale_regrid, data         ; generate the response
;         put_wdfarray, [.2, 3.8/499, data], 30, clab = 'Response'     
;       <or>
;         READWD3, wdf=30,  clab = 'Response', xrange = [.2, 4]
;
; MODIFICATION HISTORY:
;         Written by:  LPMix, August 10, 1998
;         Modification: L. P. Mix, October 5, 2002
;                         Added WDF option
;-

      on_error, 1
; common block
@wdfit3_common.dat

        ier = 1
        lc_valid = 0
        if n_elements(file ) eq 0 then fitfile = 0 else fitfile = file(0)
        if (size(fitfile))[1] ne 7 then fitfile = ''
        fitfile = strtrim(fitfile, 2)
        if strlen(fitfile) eq 0 then fitfile = $
          dialog_pickfile(/must_exist, title='Pick a film response file to READ')
        fitfile = strtrim(fitfile)
        err = 1
        if strlen(fitfile) gt 0 then openr, unit, /get, fitfile, error = err
        if err ne 0 then begin
          print, '% READWD3: Unable to open file'
          return
        endif
        str = ''
; begin
        readf, unit, str
        n = strlen(str)
        ok = strpos(str, 'WDFIT3:')
        if ok lt 0 then begin
          print, '% READWD3: File has incorrect format: WDFIT3'
          free_lun, unit
          return
        endif
        resp_file = strtrim(strmid(str, ok+7), 2)

;       Read calculated date
        readf, unit, str
;       get the calculated date:
        ok = strpos(str, 'Calculated:')
        if ok lt 0 then begin
          print, '% READWD3: Response file has incorrect format: Calculated:'
          free_lun, unit
          return
        endif
        cal_date= strtrim (strmid(str, ok+11),2)
; 
;       Read valid
        readf, unit, str
        ok = strpos(str, 'Valid:')
        if ok lt 0 then begin
          print, '% READWD3: Response file has incorrect format: Valid:'
          free_lun, unit
          return
        endif
        if n_elements(valid_save) eq 0 then valid_save = make_array(2)
        reads, strmid(str, ok+7), valid_save

;       Read breaks
        readf, unit, str
        ok = strpos(str, 'Break:')
        if ok lt 0 then begin
          print, '% READWD3: Response file has incorrect format: Break:'
          free_lun, unit
          return
        endif
        if n_elements(break_save) eq 0 then break_save = make_array(2)
        reads, strmid(str, ok+7), Break_save

        readf, unit, str
        ok = strpos(str, 'Exponential:')
        if ok lt 0 then begin
        print, '% READWD3: Response file has incorrect format: Exponential:'
          free_lun, unit
          return
        endif
        lc_log =0L
        reads, strmid(str, ok+12), lc_log

        n1 = 0L
        readf, unit, n1
        c1_save = make_array(n1, /double)
        readf, unit, c1_save

        n2 = 0L
        readf, unit, n2
        c2_save = make_array(n2, /double)
        readf, unit, c2_save

        n3 = 0L
        readf, unit, n3
        c3_save = make_array(n3, /double)
        readf, unit, c3_save
        filename_save = fitfile
        if n2 eq 1 then nbreak_save = 0L else begin
          if n3 eq 1 then nbreak_save = 1L else nbreak_save = 2L
        endelse
        degree_save = ([n1, n2, n3]-1L) > 1
        free_lun, unit
        lc_valid = 1
        ier = 0
        file = temporary(fitfile)

;       Check to see if we are to save the data
        if n_elements (xdata) gt 2 then begin
          ydata = xdata
          scale_regrid, ydata
        endif
        if keyword_set (wdfarray) then $
          wdfarray=get_save_array(wdfarray, get_maxwdf()) $
        else return
        if n_elements(filename) eq 0 then filename = file else $
          filename= strtrim(filename[0], 2)
        if n_elements(xlabel) eq 0 then xlabel = 'Film Density'  else $
          xlabel= strtrim(xlabel[0], 2)
        if n_elements(ylabel) eq 0 then ylabel = 'Film Exposure'  else $
          ylabel= strtrim(ylabel[0], 2)
        if n_elements(clabel) eq 0 then clabel = 'Response - Filename=' $
          +resp_file+';;DATETIME='+cal_date  else $
          clabel= strtrim(clabel[0], 2)
        if n_elements(npoints) eq 0 then npoints = 500 else $
          npoints = npoints > 32
        if n_elements(xrange) lt 2 then xrange=valid_save else begin
          xrange= float(xrange[0:1])
          xrange[0]= xrange[0] > valid_save[0]
          xrange[1]= xrange[1] < valid_save[1]
          if xrange[1]-xrange[0] le 0 then xrange = valid_save
        endelse
        if n_elements(xdata) gt 2 then put_wdfarray, $
          [xdata, ydata], wdfarray, /xydata, clabel= clabel, $
          xlabel= xlabel, ylabel = ylabel, file = filename, /no_copy $
        else begin
          del = (xrange[1]-xrange[0])/(npoints-1)
          data = findgen(npoints)*del+xrange[0]
          scale_regrid, data
          put_wdfarray, [xrange[0],del, data], wdfarray, clabel= clabel, $
            xlabel= xlabel, ylabel = ylabel, file = filename, /no_copy
        endelse
        end
