; ------------------------------------------------------------------------
; $Id: combine_cims.pro,v 1.4 2010/01/22 18:16:57 lpmix Exp $
; ------------------------------------------------------------------------
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;

pro combine_cims, s1, s2, s3
;+
; ; NAME:
;        
;
; PURPOSE:
;       Combine two QS CIM data structures into a single structure
;       containing all the parts in the two input structures. 
;
; CALLING SEQUENCE:
;       COMBINE_CIMS, STR1 , STR2 [, STR3]
;
; INPUTS:
;       STR1,STR2- Integer structure array numbers or IDL CIM
;                 structures containing that are to be combined.
;                 Any grid blocks common to both structures (as
;                 determined by the encoded QS block number stored in
;                 the second word of the spare array for each block)
;                 must have the same 3D shape. Part names and numbers
;                 in structure STR1 are preserved. Part names in
;                 structure S2, which must be distinct from the names
;                 in STR1, are also preserved, but STR2 part numbers are
;                 offset by the corresponding number of parts in
;                 STR1. Also note that non-zero elements of STR1 will be
;                 overwritten by corresponding non-zero elements of
;                 STR2.
;
;  OUTPUTS:
;       STR3 -   Integer structure array number or IDL CIM structure
;                in which the combined CIM structure is returned.
;                If both STR1 and STR2 are IDL structures, STR3 will be
;                  a structure if it is a variable (ie, ARG_PRESENT())
;                Otherwise:
;                If STR3 is a valid integer structure array number, 
;                  the structure will be returned there.
;                If STR3 is not a valid array number, then 
;                  the output is stored in:
;                  GET_MAXSTR()-1 if STR3 is not a variable
;                  first free structure if STR3 is a variable
;
; KEYWORD PARAMETERS: none
;
; COMMON BLOCKS: none
;
; EXAMPLE: Given a QS CIM structure "cim1" with two conductors and one
;          material, and whose dataset comment is "CIM ARRAY -
;          cathode=1;;anode=2;;plastic=-1", and a second QS CIM
;          structure "cim2" with the comment "CIM ARRAY -
;          plate=1;;water=-1", construct a new QS CIM structure "cim3"
;          which combines the parts of both cim1 and cim2. Note that
;          the new dataset comment will be "CIM ARRAY -
;          cathode=1;;anode=2;;plate=3;;plastic=-1;;water=-2".
;
;         COMBINE_CIMS, cim1, cim2, cim3
;
; SEE ALSO:
;      ASIS2CIM, MERGE_CIM_PARTS
;
; MODIFICATION HISTORY:
;      Original version, D. B. Seidel, Nov. 5, 2008
;      Modified error handling and other minor changes, LPM, April 14, 2009
;-
;

on_error, 1
maxstruct = get_maxstr()
maxstructm1 = maxstruct-1
free = (strfree(max = maxstructm1, nel = nel))[0]
if nel eq 0 then free = maxstructm1

sout = get_valid_structure(s1, 'COMBINE_CIMS', lcstr = lcstr1, ierr = ier)
if ier then return

si = get_valid_structure(s2, 'COMBINE_CIMS', lcstr = lcstr2, ierr = ier)
if ier then return

if lcstr1 * lcstr2 gt 0 then begin
  if arg_present(s3) then lcstr3 = 1 else lcstr3 = 0
endif else lcstr3 = 0
    
if lcstr3 eq 0 then begin
  t3 = size(/type, s3)
  if (t3 gt 0) and (t3 lt 4) then begin
    s3= s3[0]
;   If I can return the value, use first free structure
; 
    if ((s3 le 0) || (s3 gt maxstruct)) then $
        s3 = (arg_present(s3)) ? free : maxstructm1
  endif else s3 = (arg_present(s3)) ? free : maxstructm1
endif
ndim = sout.ndim
ndim2 = si.ndim
if ndim ne ndim2 then begin
  print, '% COMBINE_CIM: Dimensional mismatch: ', ndim,ndim2
  return
endif

blist1 = reform(sout.spare[1,*])
bsize1 = sout.size
clab1 = sout.clab
nb1 = sout.block
xyzlabs = make_array(/str,ndim)

blist2 = reform(si.spare[1,*])
clab2 = si.clab
nb2 = si.block
bsize2 = si.size
spare2 = si.spare
xyzlabs[0] = si.x1lab[0]
xyzlabs[1] = si.x2lab[0]
xyzlabs[2] = si.x3lab[0]

; print,clab1,blist1
; print,clab2,blist2

cname = ''
mname = ''

parse_cim_str,clab1,nc1,nm1,ctmp,mtmp
if nc1 gt 0 then cname = [cname,ctmp]
if nm1 gt 0 then mname = [mname,mtmp]

parse_cim_str,clab2,nc2,nm2,ctmp,mtmp
if nc2 gt 0 then cname = [cname,ctmp]
if nm2 gt 0 then mname = [mname,mtmp]

nc = nc1 + nc2
nm = nm1 + nm2

if nc gt 0 then cname = cname[1:*]
if nm gt 0 then mname = mname[1:*]

bld_cim_str,clab,nc,nm,cname,mname

; print,clab

blist12 = [ blist1, blist2 ]
a = sort(blist12)
n1 = n_elements(blist1)
n2 = n_elements(blist2)
blist = blist12[a]
nbm = n_elements(blist)-1

both = make_array(n2, /long)
match = make_array(n2, /long)
new = make_array(n2, /long)
nboth = 0L
nnew = 0L
i = 0L
err = 0
while i le nbm do begin
  if i lt nbm then begin
    if blist[i] eq blist[i+1] then begin
      a1 = min(a[i:i+1], max = a2)
      match[nboth] = a1
      both[nboth++] = a2-n1
      if max(abs (bsize1[*, a1] - bsize2[*, a2-n1])) gt 0 then begin
        print,'% COMBINE_CIM: Block dimemsion mismatch: ', a1, a2-n1
        print, blist1[a1], bsize1[*,a1],bsize2[*,a2-n1]
        ++ err
      endif
      i++
    endif else if a[i] gt n1 then new[nnew++]= a[i] - n1
  endif else if a[i] gt n1 then new[nnew++]= a[i] - n1
  i++
endwhile
if nboth gt 0 then begin
  both = temporary(both[0:nboth-1])
  match = temporary(match[0:nboth-1])
endif
if nnew gt 0 then new = temporary(new[0:nnew-1])

if err ne 0 then begin
  print,'% COMBINE_CIM: ' + strtrim(err, 2) + ' errors encountered'
  return
endif

coff = 2*nc1
moff = 2*nm1

for i=0L,nboth-1 do begin
  b1 = match[i]+1
  b2 = both[i]+1
  off1 = sout.locv[b1-1]
  blen = sout.locv[b1] - off1
  cim2 = reform(/over,get_array(s2,-1,b2),blen)
  for k=0,nc2-1 do begin
    look = 2*k + 3
    lfnd = where(cim2 eq look, nok)
    if nok gt 0 then sout.v1[off1+lfnd] = look + coff
  endfor
  for k=0,nm2-1 do begin
    look = 2*k + 2
    lfnd = where(cim2 eq look,nok)
    if nok gt 0 then sout.v1[off1+lfnd] = look + moff
  endfor
  print,'Both cims: ',b1,b2,blist1[match[i]],blist2[both[i]]
  print,off1,blen
endfor

if nnew gt 0 then begin
  blkstr = free
  print,'blkstr = ',blkstr

  for i=0,nnew-1 do begin
    b2 = new[i]+1
    blen = 1 & for k=0,ndim-1 do blen *= bsize2[k,new[i]]
    cim2 = reform(/over,get_array(s2,-1,b2),blen)
    for k=0,nc2-1 do begin
      look = 2*k + 3
      lfnd = where(cim2 eq look, nok)
      if nok gt 0 then cim2[lfnd] += coff
    endfor
    for k=0,nm2-1 do begin
      look = 2*k + 2
      lfnd = where(cim2 eq look,nok)
      if nok gt 0 then cim2[lfnd] += moff
    endfor

    print,'2nd cim only: ',new[i],blist2[new[i]],blen,n_elements(cim2)
    cim2 = reform(/over,cim2,bsize2[*,new[i]])
    cmd = 'arr2str, blkstr, cim2'
    bstr = strcompress(string(b2),/remove_all)
    for k=0,ndim-1 do begin
      kstr = strcompress(string(k+1),/remove_all)
      exe = 'h' + kstr + ' = get_array(s2,' + kstr + ',' + bstr + ')'
      print,exe
      rval = execute(exe)
      cmd = cmd + ', h' + kstr 
    endfor
    cmd = cmd + ', xlab=xyzlabs[0], ylab=xyzlabs[1], zlab=xyzlabs[2],' + $
          'spare=spare2[*,new[i]]'
    print,cmd
    rval = execute(cmd)
    joinstr,sout,blkstr
    ;;shelp,blkstr
  endfor
  delstr,blkstr
endif

sout.clab = clab

if lcstr3 then s3 = sout else i2s, sout, s3

end
