; ------------------------------------------------------------------------ 
; $Id: plotacis.pro,v 1.2 2005/03/09 22:20:19 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2005 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
 pro plotacis, ain, x, y, parts = partsin, color = color, thick = thick, $
   xarray = xarray, yarray= yarray, overplot = overplot, $
    xrange = xrange, yrange = yrange, header= header, $
   showdefault = showdefault, setdefault = setdefault, unset = unset,$
    skip = skip, ignore = ignore, frame = frame, aspect = aspect, $
   xtitle = xtitle, ytitle= ytitle , title = title, linestyle = linestyle, $
   conductor = conductor,  _extra = extrastuff
;
;+
; NAME:
;       PLOTACIS
;
; PURPOSE:
;       Plot conputational grids or block boundaries.
; CATEGORY:
;       Input/Output
; CALLING SEQUENCE:
;       PLOTACIS, A [, X] [, Y] [PARTS = parts]
;            [, XRANGE = XRANGE] [, YRANGE = YRANGE]
;            [, THICK = THICK] [, COLOR = COLOR] [, TITLE = TITLE]
;            [, XTITLE = XTITLE] [, YTITLE = YTITLE] [, SKIP= SKIP]
;            [, LINESTYLE = LINESTYLE] [, IGNORE = ignore]
;            [, XARRAY = XARRAY] [, YARRAY = YARRAY] [, OVERPLOT = overplot]
;            [, CONDUCTOR = CONDUCTOR] [, ASPECT= aspect]
;            [, HEADER = header] [, SETDEFAULT = setdefault] [, UNSET = unset]
;            [, SHOWDEFAULT = showdefault] [, _EXTRA = extrastuff]
; INPUTS:
;       A     IDL ACIS slice structure (type = 7)
;       X     Abscissa value (1, 2, or 3 depending on desired spatial array)
;       Y     Ordinate value (1, 2, or 3 depending on desired spatial array)
;       Note: In the case of a normal slice, x and y will be the other
;             two coordinates.
;       Note: In the case of an overplot, the x and y values will be determined
;             from the previous plot.
; KEYWORD PARAMETERS:
;       PARTS    - A part number or an array of parts.
;                  If zero or missing, all parts will be plotted.
;       OVERPLOT - If not zero, indicates no new grid is to be drawn.
;       THICK    - Indicates line thickness for the grid default is 0.6
;       LINESTYLE - Line type for the grid. An array may be specified to 
;                 sequence over linestyles.  Lines specified by
;                 LINESTYLE are: solid(0), dot(1), dash(2), dot-dash(3),
;                 dot-dot-dot-dash(4), and long dash(5).
;       BLOCK    - The block of data to be plotted. Default is all blocks
;       COLOR    - Indicates the color of the grid, Color may be an array 
;                  to sequence over colors.  If colors is an array, the
;                  minimum value is 1 and the maximum is !d.n_colors
;                   The colors are: black(0), red(1), green(2), blue(3),
;                   purple(4), orange(5), light blue(6), yellow(7) and white(8).
;                  Color = -1 will change colors for each segment
;       SKIP     - The number of grid lines plotted is reduced by this value.
;                  Note: If skip is very large, the blocks will be outlined.
;                        (see FRAME below)
;       ASPECT   - If set, the X and Y axis scaling will be identical.
;       FRAME     - If present and not zero, will outline each block.
;                 Same as skip = a very big number
;       CONDUCTOR - Specifies a conductor structure or a structure array 
;                   number specifying a conductor structure.  If valid, the
;                   conductors will be overplotted. Colors and thicknesses
;                   can be set with setdefault.
;       IGNORE   - If the keyword IGNORE is set, then previous parameters 
;                  on overlay plots will be ignored.
;       TITLE, XTITLE and YTITLE If grid is not zero, these will be used for
;                 axis and the plot labels.  If these are not string arrays, 
;                 the title, xtitle and ytitle used for the plots will be used.
;       XARRAY, YARRAY, are the grid arrays for the first block plotted
;       (X,Y)RANGE, STYLE, TYPE have the standard IDL meaning.
;       HEADER    If specified a plot header, consisting of the current date
;                 and the current file name, is displayed in the lower left
;                 corner of the plot.  See the HEADER command to set default
;                 HEADER parameters (character size, spacing, alignment).
;                 DEFAULT is HEADER = 1
;                 NOTE: For X-windows screen dumps, many will want to increase
;                      the character size with the command:
;                      HEADER, /set, charsize = 0.8
;                      (Default is 0.6 of current size)
;     SETDEFAULT- If present and not zero, then the following parameters will be
;                 saved and used as the default values in all future calls to PLO
;                 If a keyword is specified, it will always be used rather than
;                 the saved values.
;     UNSET     - If present and not zero, will set all save values of keywords
;                 to their default values.
;                 Normal default values are listed below.
;                 Keyword     Default value
;                 HEADER       1
;                 XRANGE       0
;                 YRANGE       0
;                 ASPECT       0
;
;     SHOWDEFAULT-Show the present default plot values.
;       _EXTRA    These parameters are sent to PLOT to generate the grid and 
;                 can have any value appropriate for the plot command including
;                 the following:
;                 (X,Y)RANGE, (X,Y)STYLE, (X,Y)TYPE, POSITION , etc
;                 and all have the standard IDL meaning.
; OUTPUTS:
;       Designated datasets are plotted.
; EXAMPLE:
;       Plot from structure g, space 1 vs space 3 with blue dashed lines
;       plotacis, g, 3, 1, color = 3, linestyle = 2
;
;       Plot from structure g, space 1 vs space 3 with the fisrt line green 
;       and the next 4 red. Print the values of the grid lines to the terminal.
;         plotacis, g, 3,1, color = [2,1,1,1,1] 
;
; MODIFICATION HISTORY:
;      Original version, L. P. Mix, March 25, 1993
;-
;
; insert a common block
;
@qscom.dat
;
;  Common block
@qsstrcom.dat
;
common plotacis_defaults, header_save, xrange_save, yrange_save
      on_error, 1
;
common plot_grid_last, yeng_scale, xeng_scale
;

      if n_elements (structinit) eq 0 then qsstruct_init
      if n_elements (qsplotinit) eq 0 then qsinit
;
;
      if keyword_set(unset) or n_elements(header_save) eq 0 then begin
        HEADER_SAVE = 1
        XRANGE_SAVE = 0
        YRANGE_SAVE = 0
      endif
      if keyword_set (setdefault) then begin
        if n_elements(header) gt 0 then header_save = header
        if n_elements(xrange) gt 0 then xrange_save = xrange
        if n_elements(yrange) gt 0 then yrange_save = yrange
      endif
      if keyword_set(showdefault) then begin
         print, 'Default parameters for the PLO command are:'
         print, '    XRANGE     = ' , strtrim (xrange_save, 2)
         print, '    YRANGE     = ' , strtrim (yrange_save, 2)
         print, '    HEADER     = ' , strtrim (HEADER_SAVE, 2)
      endif
      npar = n_params()
      if npar eq 0 and (keyword_set (setdefault) or keyword_set(unset) ) $
        then return
;
      if npar eq 0 then begin
        print,'% Format:', $
                   ' PLOTACIS, STR [, X] [, Y] [PARTS = parts]'
print,'          [, XRANGE = XRANGE] [, YRANGE = YRANGE] '
print,'          [, THICK = THICK] [, COLOR = COLOR] [, TITLE = TITLE]'
print,'          [, XTITLE = XTITLE] [, YTITLE = YTITLE] [, SKIP = skip]'
print,'          [, LINESTYLE = LINESTYLE] '
print,'          [, XARRAY = XARRAY] [, YARRAY = YARRAY] [, BLOCK = BLOCK]'
print,'          [, SETDEFAULT = setdefault] [, UNSET = unset]'
print,'          [, SHOWDEFAULT = showdefault] [HEADER = header]'
print,'          [, CONDUCTOR = CONDUCTOR] [, OVERPLOT = overplot]'
print,'          [, FRAME = frame] [, IGNORE = ignore] [_EXTRA = extrastuff]'
        return
      endif
;    make sure we don't corrupt input data
;     now get the data  -- check that we have a structure of the proper type
      if n_elements(ain) eq 0 then begin
        print, '%PLOTACIS: Structure must be defined'
        return
      endif
      a = ain
      b = size(a)
      if b(b(0)+1) ne 8 then begin
        if a gt 0 and a le maxstructure then begin
          s2i, a, a
          b = size (a)
          if b(b(0)+1) ne 8 then begin
            print, ' First parameter must be a structure or a QS array number'
            return
          endif
        endif  else begin
          print, ' First parameter must be a structure or a QS array number'
          return
        endelse
      endif
      if a.type ne 7 then begin
        print, ' Only ACIS type structures may be plotted with PLOTACIS'
        return
      end
      if n_elements(overplot) eq 0 then overplot = 0
      if n_elements (ignore) eq 0 then ignore = 0
      if n_elements (xoldvalue)  eq 0 then begin
        xoldvalue = 0.0
        yoldvalue = 0.0
      endif
      if overplot ne 0 then begin
          if n_elements (x) eq 0 then x = xoldvalue
          if n_elements (y) eq 0 then y = yoldvalue
      endif 
      nx = n_elements (x)
      if nx eq 0 then begin
        if a.two ne 0 then begin
          x = (a.two mod 3)+1
          y = ((a.two +1) mod 3)+1
          yt = y(0) 
        endif else begin
          print, '%PLOTACIS: Abscissa coordinate must be defined'
          return
        endelse
      endif
      xt = x(0)
      if xt lt 1 or xt gt 3 then begin
        print, '% PLOTACIS: Abscissa must be one of the three dimensions:',xt
        return
      endif
      if overplot ne 0 and xt ne xoldvalue  and ignore eq 0 then begin
          Print, ' WARNING: This overplot requested with X-axis of ', x, $
              ', present value is', xoldvalue
          hak, /mes 
      endif
      ny = n_elements (y)
      if ny eq 0 then begin
        Print, '%PLOTACIS: Ordinate coordinate must be defined'
        return
      endif
      yt = y(0)
      if overplot ne 0 and yt ne yoldvalue and ignore eq 0 then begin
         Print, ' WARNING: This overplot requested with Y-axis of ', y, $
              ', present value is', yoldvalue
         hak, /mes
      endif

      if !x.crange(0) eq !x.crange(1) then overplot= 0
      xoldvalue = xt
      yoldvalue = yt
      zplaneold = 6-xoldvalue-yoldvalue
      if a.two ne 0 then zvalue = a.slice else $
          zvalue = a.range(*, zplaneold-1, 0)
      print, 'Slice value = ', zvalue
      zvalueold = zvalue
;
      if xt eq 1 then begin
        ax1 =a.x1
        xlab = a.x1lab
      endif else if xt eq 2 then begin
        ax1 = a.x2
        xlab = a.x2lab
      endif else begin
        ax1 = a.x3
        xlab = a.x3lab
      endelse
      if yt eq 1 then begin
        ax2 =a.x1
        ylab = a.x1lab
      endif else if yt eq 2 then begin
        ax2 = a.x2
        ylab = a.x2lab
      endif else begin
        ax2 = a.x3
        ylab = a.x3lab
      endelse
          
      if n_elements (partsin) eq 0 then begin
        xmin= a.range(0, xt-1, 0)
        xmax= a.range(1, xt-1, 0)
        ymin= a.range(0, yt-1, 0)
        ymax= a.range(1, yt-1, 0)
        xx = ax1
        yy = ax2
        loc = a.locx
        nloc = n_elements (loc)-1
      endif else begin
        parts = partsin
        ok = where(parts gt 0 and parts le a.npart, nok)
        if nok eq 0 then begin
          print, '% PLOTACIS: No valid parts specified- 0 to ',strtrim(a.npart,2)
          print, parts
          return
        endif
        maxx = 1024
        xx = make_array(maxx)
        yy = xx
        maxloc = 128
        loc = make_array(maxloc, /long)
        loc(0) = 0L
        nloc = 0
        nx = 0
        parts = parts(ok)
        for i = 0, nok-1 do begin
          n = a.locp(parts(i))-a.locp(parts(i)-1)
          if n gt 0 then begin
            if n+nloc ge maxloc then begin
              add = max([n, maxloc])
              loc = [ loc, make_array(add, /long)]
            endif
            st = a.locp(parts(i)-1)
            en = a.locp(parts(i))
            loc(nloc:nloc+n) = a.locx(st:en)+(loc(nloc)-a.locx(st))
            nnx = loc(nloc+n)-loc(nloc)
            if nnx+nx ge maxx then begin
              add = max([maxx, nnx])
              xx= [xx, make_array(add)]
              yy= [yy, make_array(add)]
              maxx= maxx+add
            endif
            xx(loc(nloc):loc(nloc+n)-1) = ax1(a.locx(st):a.locx(en)-1)
            yy(loc(nloc):loc(nloc+n)-1) = ax2(a.locx(st):a.locx(en)-1)
            nloc = nloc+n
            nx = nx +loc(nloc)-loc(nloc-n)
          endif
        endfor
        if nx gt 0 then begin
          xmin = min(xx, max = xmax)
          ymin = min(yy, max = ymax)
        endif else begin
          print, '% PLOTACIS: No intersection for these parts.
          return
        endelse
      endelse     
        
      if overplot eq 0 then begin
;
        tit = 'ACIS ' + a.clab 
        if n_elements (xtitle) eq 0 then xtitle = 0
        b = size (xtitle)
        if b(b(0)+1) ne 7 then xtitle = xlab else xlab = xtitle
        if n_elements (ytitle) eq 0 then ytitle = 0
        b = size (ytitle)
        if b(b(0)+1) ne 7 then ytitle = ylab else ylab = ytitle
        if n_elements (title) eq 0 then title = 0
        b = size (title)
        if b(b(0)+1) ne 7 then title = tit  else tit = title
       
;
        if n_elements(header) eq 0 then header = header_save
        if overplot ne 0 then header = 0
        if !p.multi(0) ne 0 then header = 0
        if header ne 0 then begin
          header_text='Date: ' + systime(0)
        endif
        if n_elements(xrange) eq 0 then xrange = xrange_save
        if n_elements(yrange) eq 0 then yrange = yrange_save
;
        if n_elements(xrange) lt 2 then begin 
          xrange = [xmin, xmax]
          lcxrange = 0
        endif else begin
          xrange = xrange(0:1)
          lcxrange = 1
        endelse
        if n_elements(yrange) lt 2 then begin
          yrange = [ymin, ymax]
          lcyrange = 0
        endif else begin
          yrange = yrange(0:1)
          lcyrange = 1
        endelse
;       Add a small delta to try to avoid precision problems with PFF which cause
;       strange axis scaling.
        if lcxrange eq 0 then begin
          epsil = (xrange(1)-xrange(0))/32768.
          xrange=[xrange(0)+epsil , xrange(1)-epsil]
        endif
        if lcyrange eq 0 then begin
          epsil = (yrange(1)-yrange(0))/32768.
          yrange=[yrange(0)+epsil , yrange(1)-epsil]
        endif
;       see if we have a horizontal or vertical line
        if xrange(1) le xrange (0) then xrange = [.98, 1.02]*xrange(0)
        if yrange(1) le yrange (0) then yrange = [.98, 1.02]*yrange(0)
;
        yeng_scale = 1.00  &  xeng_scale = 1.00
        if n_elements(xstyle) eq 0 then xstyle =0
        if n_elements(ystyle) eq 0 then ystyle =0
        if n_elements(xtype) eq 0 then xtype =0
        if n_elements(ytype) eq 0 then ytype =0
        if keyword_set(aspect) then begin
          if (xstyle/2)*2 eq xstyle then xstyle = xstyle +1
          if (ystyle/2)*2 eq ystyle then ystyle = ystyle +1
          pos = get_aspect(xrange, yrange)
          plot, xx, yy, xrange = xrange, yrange = yrange, ystyle = ystyle, $
          xstyle = xstyle, ytype = ytype, xtype = xtype, title = title, $
          xtitle = xtitle, ytitle = ytitle, /nodata,pos = pos, _extra= extrastuff

        endif else begin
        plot, xx, yy, xrange = xrange, yrange = yrange, ystyle = ystyle, $
          xstyle = xstyle, ytype = ytype, xtype = xtype, title = title, $
          xtitle = xtitle, ytitle = ytitle, /nodata, _extra= extrastuff
        endelse
;
        if header ne 0 then header, header_text
        xarray = xx
        yarray = yy
        lctwodim = 0
      endif
      xrang = !x.crange
      yrang = !y.crange
      if n_elements (color) eq 0 then if n_elements (grdcolor) gt 0 then $
        color = grdcolor else color = 0
      if min(color) lt -1 or max(color) gt !d.n_colors then begin
        color =0
        print, ' Color must be ge 0 and le ' , !d.n_colors
      endif
      if min(color) lt 8 then loadxyct, /quiet

      ncolor = n_elements (color)
      if ncolor eq 1 and color(0) eq -1 then begin
        ncolor = 7
        color = 1+indgen(7)
      endif
      
      if n_elements (linestyle) eq 0 then linestyle = 0
      if min(linestyle) lt 0 or max(linestyle) gt 5 then begin
        linestyle = 0
        print, ' Linestyle must be ge 0 and less than /equal to 5'
      endif
      nline = n_elements (linestyle)
      if n_elements (thick) eq 0 then if n_elements(grdthick) gt 0 then $
         thick = (!p.thick>1) *grdthick else thick = (!p.thick>1)*0.6
      if n_elements (skip) eq 0 then skip = 1 else skip = long(abs(skip)>1)
      for ii = 1,nloc   do begin
        col = color(ii mod ncolor)
        lin = linestyle (ii mod nline)
        if col eq 0 then oplot,xx(loc(ii-1):loc(ii)-1),yy(loc(ii-1):loc(ii)-1), $
          linestyle = lin, thick = thick else oplot, xx(loc(ii-1):loc(ii)-1), $
          yy(loc(ii-1):loc(ii)-1),linestyle = lin, thick = thick, color = col
      endfor
;
      if n_elements (conductor) gt 0 then begin
        plotcon, conductor, /over
      endif

      return
      end
