; ------------------------------------------------------------------------ 
; $Id: atcurv.pro,v 1.2 2005/03/09 22:21:17 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2005 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
;+
; NAME:
;	ATCURV
;
; PURPOSE:
;	This procedure overplots curves as lines or filled polygons
;	on existing axes.
;
; CATEGORY:
;	ATHETA
;
; CALLING SEQUENCE:
;	ATCURV Ncurvs, X, Y, Locs, Npnts [, CL=cl] [, LS=ls] [, TK=tk] $
;       [, MK=mk] [, FLIP=Flip] [, FILL=Fill] [, _EXTRA=Extra]
;
; INPUTS:
;	Ncurvs:	Number of curves to be plotted.
;	X:	Array of x values for all curves.
;	Y:	Array of y values for all curves.
;	Locs:	Array of beginning locations for curves in X, Y arrays.
;	Npnts:	Array of number of points for each curve.
;
; KEYWORD PARAMETERS:
;	CL:	Single value or array of color indicies for curves.  
;               Default is default plot color.
;	LS:	Single value or array of linestyles for curves.  
;               Default is solid.
;	TK:	Single value or array of line thickness for curves.  
;               Default is 1.0.
;	MK:	Single value or array of marker styles for curves.  
;               Default is no markers.
;	FLIP:	Set this keyword to flip the plot orientation to y-x,
;		Default is x-y.
;	FILL:	Set this keyword to draw curves as filled polygons.
;		Default is curves drawn as outlines.
;	_EXTRA:	Extra keywords to be passed to OPLOT procedure
;
; RESTRICTIONS:
;	"Extra" keywords must be valid keywords for OPLOT procedure.
;
; EXAMPLE:
;	Plot one curve with 25 points on an existing axis; use defaults:
;		atcurv, 1, x, y, 0, 25
;
;	Plot two curves as filled polygons with color indicies specified:
;		atcurv, 2, x, y, [0,24], [25,25], cl=[1,3], /fill
;
; MODIFICATION HISTORY:
; 	Written by:	Rebecca Coats, 9/96.
;-
pro atcurv, nc, x, y, l, n, cl=cl, ls=ls, tk=tk, mk=mk, $
            flip=flip, fill=fill, _extra=extra
;
common at_colr
;
; check for empty arrays
;
if (n_elements(nc) eq 0) then return
if (nc eq 0) then return
if (n_elements(x) eq 0) or (n_elements(y) eq 0) or $
   (n_elements(l) eq 0) or (n_elements(n) eq 0) then return
;
; use default values if not supplied
;
if not keyword_set(flip) then flip = 0
if not keyword_set(fill) then fill = 0
;
ccl = intarr(nc)
if keyword_set(cl) then begin
  if n_elements(cl) eq nc then ccl = cl else ccl(*) = cl(0)
  for m=0, nc-1 do begin
    if ccl(m) eq at_bgc then ccl(m) = !P.BACKGROUND $
    else if ccl(m) eq at_fgc then ccl(m) = !P.COLOR
  endfor
endif else ccl(*) = !P.COLOR
cls = intarr(nc)
if keyword_set(ls) then begin
  if n_elements(ls) eq nc then cls = ls else cls(*) = ls(0)
endif else cls(*) = 0
ctk = intarr(nc)
if keyword_set(tk) then begin
  if n_elements(tk) eq nc then ctk = tk else ctk(*) = tk(0)
endif else ctk(*) = 1.0
cmk = intarr(nc)
if keyword_set(mk) then begin
  if n_elements(mk) eq nc then cmk = mk else cmk(*) = mk(0)
endif else cmk(*) = 0
;
for m=0, nc-1 do begin
  if n(m) eq 0 then return
  lb = l(m) - 1
  lf = lb + n(m) - 1
  if fill then begin
    if flip then begin
      polyfill, y(lb:lf), x(lb:lf), color=ccl(m), noclip=0
    endif else begin
      polyfill, x(lb:lf), y(lb:lf), color=ccl(m), noclip=0
    endelse
  endif else begin
    if flip then begin
      oplot, y(lb:lf), x(lb:lf), color=ccl(m), linestyle=cls(m), $
      thick=ctk(m), psym=cmk(m), _extra=extra
    endif else begin
      oplot, x(lb:lf), y(lb:lf), color=ccl(m), linestyle=cls(m), $
      thick=ctk(m), psym=cmk(m), _extra=extra
    endelse
  endelse
endfor
return
end
