; ------------------------------------------------------------------------ 
; $Id: atfld.pro,v 1.2 2005/03/09 22:21:18 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2005 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
;+
; NAME:
;	ATFLD
;
; PURPOSE:
;	This procedure overplots contours of a field structure.
;
; CATEGORY:
;	ATHETA
;
; CALLING SEQUENCE:
;	ATFLD [, Struct] [, NLEVEL=Nlevel] [, LRANGE=Lrange] $
;       [, LEVELS=level] [, FLIP=Flip] [, CLEAN=Clean] $
;       [, THERMOMETER=Thermometer] [, _EXTRA=Extra]
;
; OPTIONAL INPUTS:
;	Struct:	Structure number of field to be plotted.  Default is 
;		structure 1.
;
; KEYWORD PARAMETERS:
;	NLEVEL:	Number of contour levels to draw.  Default number of
;		levels is 10.
;	LRANGE:	Two element array giving contour range, [lowest, highest],
;		for the plot.  Defaults are field minimum and maximum.
;	LEVELS:	Specific contour levels to draw.  If specified, NLEVEL
;		and LRANGE keywords are ignored.
;	FLIP:	Set this keyword to flip the plot orientation to r: x-axis,
;		z: y-axis.  Default is z: x-axis, r: y-axis.
;	CLEAN:	Set this keyword to plot with no annotation for number 
;		of contours, field minimum and maximum.  Default is 
;               include annotation.
;	THERMOMETER:	
;		Set this keyword to draw a thermometer of contour levels
;               on the plot.  Default is no thermometer.
;	_EXTRA:	Extra keywords to be passed to PLOTFLD procedure.
;
; RESTRICTIONS:
;	ATHETA geometry and field data must be already be loaded in 
;	IDL arrays by previous ATHETA PLOT command.  This procedure
;       overplots contours on an existing plot axis.
;	"Extra" keywords must be valid keywords for PLOTFLD procedure.
;
; EXAMPLE:
;	Draw ATHETA contour plot on an existing axis. Field is in 
;       structure 2:
;		atfld, 2
;
;	Overplot ATHETA contour plot with 20 contour levels:
;		atfld, nlevel=20
;
; MODIFICATION HISTORY:
; 	Written by:	Rebecca Coats, 9/96.
;-
pro atfld, struct, nlevel=nlevel, lrange=lrange, flip=flip, clean=clean, $
   levels=level, thermometer=thermometer, _extra=extra
;
common at_axes
common at_fld
common atheta_plot_com, atmin, atmax, atlevels
;
if n_params() eq 0 then struct = 1
valid = strvalid(struct,/valid,/quiet)
if valid le 0 then return
;
if n_elements(level) gt 0 then begin
  nlevel = n_elements(level)
endif else begin
  if not keyword_set(nlevel) then nlevel = 10
  if keyword_set(lrange) then begin
    delta = (lrange(1) - lrange(0))/(nlevel + 1)
    level = fltarr(nlevel)
    for i=1, nlevel do level(i-1) = i*delta + lrange(0)
  endif else level = 0.
endelse
;
if not keyword_set(flip) then flip = 0
;
if flip then begin
  trans = 1
  plotfld, struct, nlevel=nlevel, levels=level, xr=at_yr, yr=at_xr, $
    /overplot, color=-3, trans=trans, thermometer=thermometer, _extra=extra
endif else begin
  trans = 0
  plotfld, struct, nlevel=nlevel, levels=level, xr=at_xr, yr=at_yr, $
    /overplot, color=-3, trans=trans, thermometer=thermometer, _extra=extra
endelse
if not keyword_set(clean) then begin
  mess = ['contours: ' + string(nlevel), 'min:   ' + string(atmin), $
          'max:   ' + string(atmax)]
  xyouts,[.7, .7, .7],[.92,.90,.88], mess, /normal
endif
;
if at_fmin*at_fmax lt 0 then plotfld, struct, nlevel=1, levels=[0.], $
  /overplot, trans=trans, color=1, thick=3.0, quiet=-1
;
return
end
