; ------------------------------------------------------------------------ 
; $Id: atgeom.pro,v 1.2 2005/03/09 22:21:18 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2005 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
;+
; NAME:
;	ATGEOM
;
; PURPOSE:
;	This procedure plots different parts of the ATHETA geometry
;	individually.  These parts are the axes; ATHETA parts, coilsets
;	and curves; and curves stored as "temporary".
;
; CATEGORY:
;	ATHETA
;
; CALLING SEQUENCE:
;	ATGEOM, Label [, FLIP=Flip] [, FILL=Fill] [, CLEAN=Clean] $
;	[, THERMOMETER=Thermometer] [, OTO=oto] [, RADIUS=Radius] $
;	[, _EXTRA=Extra]
;
; INPUTS:
;	Label:	Label of entity to draw:
;		axes:  set up and draw axes
;		part:  draw ATHETA parts
;		coil:  draw ATHETA coilsets
;		curv:  draw ATHETA curves
;		temp:  draw curves stored as "temporary"
;
; KEYWORD PARAMETERS:
;	FLIP:	Set this keyword to flip the plot orientation to r: x-axis,
;		z: y-axis.  Default is z: x-axis, r: y-axis.
;	FILL:	Set this keyword to draw parts as filled polygons.
;		Default is parts drawn as outlines.
;	CLEAN:	Set this keyword to draw axes with no axes, labels
;		or tick marks.  Default is draw axes, labels and ticks.
;	THERMOMETER:	
;		Set this keyword to leave room for a thermometer of 
;		contour levels on the plot.  Default is no thermometer.
;	OTO:	Set this keyword for a one-to-one aspect ratio
;	RADIUS: Draw coils as wires with radius "radius"
;	_EXTRA:	Extra keywords to be passed to PLOT procedure
;
; RESTRICTIONS:
;	ATHETA geometry data must already be loaded in IDL arrays by 
;	previous ATHETA DRAW command.
;	"Extra" keywords must be valid keywords for PLOT procedure.
;
; EXAMPLE:
;	Draw ATHETA axes and parts only:
;		atgeom, 'axes'
;		atgeom, 'part'
;
;	Draw the curve stored as "temp" on an existing plot:
;		atgeom, 'temp'
;
; MODIFICATION HISTORY:
; 	Written by:	Rebecca Coats, 9/96.
;-
pro atgeom, lbl, flip=flip, fill=fill, clean=clean, $
  thermometer=thermometer, oto=oto, radius=radius, _extra=extra
;
common at_axes
common at_part
common at_coil
common at_curv
common at_temp
;
if n_elements(lbl) eq 0 then return
;
case lbl of
'axes': ataxes, at_xr, at_yr, at_xlabel, at_ylabel, at_title, $
        flip=flip, clean=clean, thermometer=thermometer, oto=oto, _extra=extra
'part': atcurv, part_nc, part_x, part_y, part_l, part_n, $
        cl=part_cl, tk=part_lw, flip=flip, fill=fill
'coil': begin
          if n_elements(radius) eq 0 then $
            atcurv, coil_nc, coil_x, coil_y, coil_l, coil_n, $
            mk=coil_mk, flip=flip $
          else begin
            np = 37
            dth = 2.0*!PI/(np-1)
            th = findgen(np)*dth
            xp = make_array(np,/float)
            yp = make_array(np,/float)
            for n=0, coil_l[coil_nc-1] + coil_n[coil_nc-1] - 2 do begin
              xp = radius*cos(th) + coil_x[n]
              yp = radius*sin(th) + coil_y[n]
              atcurv, 1, xp, yp, 1, np, flip=flip
            endfor
          endelse
        endcase
'curv': atcurv, curv_nc, curv_x, curv_y, curv_l, curv_n, $
        flip=flip
else: atcurv, temp_nc, temp_x, temp_y, temp_l, temp_n, $
        cl=temp_cl, ls=temp_ls, tk=temp_lw, mk=temp_mk, $
        flip=flip
endcase
return
end
