; ------------------------------------------------------------------------ 
; $Id: createcdf.pro,v 1.4 2007/06/06 19:39:20 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
  pro createcdf, p1, p2, fileid = fileid, nlabel= nlabel, ncomment = ncomment, $
             ndset = ndset, Title = Title , clobber = clobber, help = help
;
;+
;   Create a CDF file for waveform output. Current file pointer is set to this
;   file.
;   format:  CREATECDF  [, filename]  [, fileid ] [, fileid = fid]
;                  [, TITLE = title] [, NDSET = ndset]
;                 [, NLABEL = nlabel] [, NCOMMENT = ncomment] [, /HELP]
;
;   where:
;      filename - name of the file to be created
;                 Default value is 'CDFdefault.cdf'
;      fileid   - user specified file id 
;      fid      - Alternate user specified file id. If fileid is present
;                then fid is ignored.
;
;      NOTE:    If fileid (or fid) is specified, it will be used as the file id.
;               If fileid is not provided or is zero, a value will be 
;                 provided; default value is the first unused integer >/= 1.
;                 If fileid is provided but has been used for another 
;                 file, the program inform the user and terminate.
;      TITLE    - String describing the contents of the file.
;                 If blank - User will be prompted.
;                 Non blank string length must be greater than 1.
;                 if /t is entered then the default will be used.
;                 default is systime(0)
;      NDSET    - Maximum number of waveforms to be stored.
;                 NDSET > 16; Default is 128
;      NLABEL   - Maximum length of axis labels  
;                 Nlabel > 4; Default = 12
;      NComment - Maximum length of dataset comment. 
;                 Ncomment > 8; Default is 64
;      CLOBBER  - If set, any existing file will be deleted.
;      HELP     - If set, a prompt will be given
;                 No further action will be taken.
;
;  Examples:
;     Create a CDF file, "test" with a default file id
;       createcdf, 'test'
;     Creates a CDF file, "TEST.DAT" with a file id of 66
;       createpff, `test.dat', 66
;-
;
      on_error, 1
@netcdfcom.dat

;     check to see if cdf has been initialized
;
      if n_elements(initcdf) eq 0 then init_cdf
;
;     check on the number of parameters
;
      if keyword_set(help) then begin
print, '% Format: CREATECDF [, File] [, Fileid ] [, FILEID = fileid]'
print, '              [, TITLE = title] [, NDSET = ndset] [, CLOBBER = clobber]'
print, '              [, NLABEL = nlabel] [, NCOMMENT = ncomment] [, /HELP '
        return
      endif

      nparams = n_params()
;
;     be sure we have a file name
;
      if (nparams eq 0) then begin
        filename = 'CDFdefault.cdf'
      endif else begin
        filename = strtrim(p1, 2)
        if strlen(filename) eq 0 then  filename = 'CDFdefault.cdf'
      endelse
      temp = p1
      if initcdf eq 1 then fulfnam, temp, p1
;
      if (n_elements(p2) eq 0) then begin
        if n_elements(fileid) gt 0 then p2 = fileid(0) else p2 = 0L
      endif else p2 = p2(0)
      oldid = cdf_id
      cdf_id = -1
      cdf_id = ncdf_create(p1, clobber = clobber)

      if cdf_id lt 0 then begin
        print, '% CREATECDF: Create failed.'
        print, '       File already exists or you do not have write permission'
        cdf_id = oldid
        return
      endif

      fid = cdf_id
      save_file_id

      if cdf_id lt 0 or cdf_id ge max_cdffile then begin
        ncdf_close, cdf_id
        print, '% CREATECDF: Unable to open file, Too many files open', cdf_id
        cdf_id = oldid
        return
      endif
      if initcdf eq 1 then begin
;
;       get full path name for unix systems
;
        directory = p1
        get_fulldirname, directory, file
       help, p1, directory, file
        if strlen (directory(0)) eq 0 then begin
          print, ' Unable to open file: ', p1
          return
        endif
        p1 = directory(0) +'/'+ file
      endif

      if p2 eq 0 then begin
;       get a default file id
        p2 = 1
        ok = where (array_cdfid(*, 0) eq p2, nok)
        while nok gt 0 do begin
          p2 = p2+1
          ok = where (array_cdfid(*, 0) eq p2, nok)
        endwhile
      endif else begin
        ok = where (array_cdfid(*, 0) eq p2, nok)
        if nok gt 0 then begin
          print, '% CREATECDF: Unable to open file with that fid.'
          print, '        Either close other file or choose anothr fid'
          ncdf_close, fid
          fcdf_id(cdf_id) = -1
          cdf_id = oldid
          return
        endif
      endelse
      array_cdfid(cdf_id, 0) = p2
      array_cdfid(cdf_id, 1) = fix((systime(1)*10) mod 65536)
      array_cdfid(cdf_id, 2) = 1
;     cannot allow identifier to be zero.
      if  array_cdfid(cdf_id, 1) eq 0 then  array_cdfid(cdf_id, 1) = 1
      file_name(cdf_id) = p1
      ncdf_attput, fid, /global, 'version', 1.0
      ncdf_attput, fid, /global, 'floating_point_word_size', 4L
      if n_elements (title ) eq 0 then begin
        title = ''
        read, prompt=' Enter a Title for this CDF file: ', title
      endif

      title = strtrim(title, 2)
      if strlen (title) le 1 then title = systime (0)
      ncdf_attput, fid, /global, 'title', byte(title)
      ncdf_attput, fid, /global, 'pfidl_type', 2L
      cdf_type(cdf_id) = 2L
      file_title(cdf_id) = title
      file_name(cdf_id) = p1
      print, '% CREATECDF: File opened with FID = ' , strtrim (p2, 2)
      print, ' File: ', strtrim (p1, 2)

      num_glo_var(cdf_id) = 0
      num_nod_var(cdf_id) = 0
      num_elem_var(cdf_id) = 0

;     Define Dimensions
;     Maximum number of datasets
      if n_elements (ndset) eq 0 then ndset = 128 else ndset = ndset > 16
      max_ndsid = ncdf_dimdef (fid, 'max_datasets', ndset)
      max_ndspid = ncdf_dimdef (fid, 'max_datasetp1', ndset+1)

      lfid  = ncdf_dimdef (fid, 'length_farray', /unlimited)
;     Length of the axix labels
      if n_elements(nlabel) eq 0 then nlabel = 12 else nlabel = nlabel > 4
      nlabid = ncdf_dimdef (fid, 'label_length', nlabel)

;     Length of the dataset comments
      if n_elements(ncomment) eq 0 then ncomment=64 else ncomment=ncomment > 64
      ncomid = ncdf_dimdef (fid, 'comment_length', ncomment)

;     Define arrays
      ndset_id = ncdf_vardef(fid, 'numb_dataset')
      locf_id  = ncdf_vardef(fid, 'pointer_array', max_ndspid, /long)
      wdf_id  =  ncdf_vardef(fid, 'wdf_array', lfid, /float)
      np_id = ncdf_vardef(fid, 'numb_of_pnts', max_ndsid, /long)
      x1lab_id =  ncdf_vardef(fid, 'x1_label', [nlabid, max_ndsid], /byte)
      x2lab_id =  ncdf_vardef(fid, 'x2_label', [nlabid, max_ndsid], /byte)
      clab_id =  ncdf_vardef(fid, 'comment_label', [ncomid, max_ndsid], /byte)
      
      
      ncdf_control, fid, /endef
;     define arrays
      num_el_blk = 0L
      ncdf_varput, fid, ndset_id, num_el_blk

      loc_f = make_array(ndset+1, /long)
      ncdf_varput, fid, locf_id, loc_f

      num_dim = 1L
      coord_cell= make_array(ndset, /long)
      ncdf_varput, fid, np_id, coord_cell
      coord_node= ndset
      wdf_array = 0.0

      label = make_array(nlabel, ndset, /byte)
      ncdf_varput, fid, x1lab_id, label
      ncdf_varput, fid, x2lab_id, label
      comment = make_array(nlabel, ndset, /byte)
      ncdf_varput, fid, clab_id, comment
      
      
      x1_label = make_array(ndset, /str)
      x2_label = make_array(ndset, /str)
      name_glo_var = make_array(ndset, /string)
      lcdfvalid= array_cdfid(cdf_id, 1) 
      num_label = nlabel
      num_comment = ncomment
      return
      end
