; ------------------------------------------------------------------------ 
; $Id: dircdf.pro,v 1.6 2007/06/06 19:39:20 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
  pro dircdf, p1,  PAGE = page, reset = reset, font= font, ndset = ndset, $
    xsize = xsize, truncate = truncate, print = print, loc = loc, $
    QUIET = quiet, filename = filename, comment = comment, line = line
;
;+
;
;     This routine prints a directory of netCDF files and optionally returns
;       the values of the directory information in keyword arguements.
;
;     DIRCDF [, P1] [, Page = Page] [, NDSET = ndset]
;         [, TRUNCATE = truncate] [, LOC = loc]
;         [, FILENAME = filename] [, COMMENT = comment] [, LINE = line] 
;         [, Reset = reset]  [, Xsize = xsize] [, /print] [, /quiet]
;
;     where:
;           P1       - an optional search string.
;                      For example: To get all datasets with 'TEMP' use:
;                        dircdf, 'temp' 
;                      See GET_MATCH for allowed wild cards.
;           PAGE     - the number of lines on the directory
;                      default is 24
;           RESET    - present and not zero, resets the directory
;                      use this if the window has been closed
;                  Note: If the window is still open and you wish to leave it
;                        open, use /reset to put the current command in a second 
;                        window.
;                  NOTE: By using the reset qualifier, a window can be 
;                        opened for each file with the filename at the top.
;           NDSET    - the number of datasets returned 
;                      For EXODUS II data NDSET has 3 elements for 
;                        global, nodal and cell based data.
;           XSIZE    - the width of the window in pixels (generally not set)
;           TRUNCATE - if set, the dataset comments will be truncated at ' - '
;                      This is a toggle; If not specified the last value will
;                      be used.  Default value =0 -- no truncate
;           PRINT    - If set, directory will be sent to terminal
;           QUIET    - If set, no outputs unless an error. 
;          FILENAME  - Name of the netCDF file
;          COMMENT   - Dataset comment for each of the datasets
;          LOC       - Pointer into COMMENT for each of the dataset types.
;                      For EXODUS II data.
;                Example:
;                      If ndset(0) gt 0, then global data exists and the 
;                      comments are given by 
;                         global_comments= comment(loc(0):loc(1)-1)
;                      If ndset(1) gt 0, then node data exists and the 
;                      comments are given by 
;                         Node_comments= comment(loc(1):loc(2)-1)
;                      If ndset(2) gt 0, then cell or element data exists and the 
;                      comments are given by 
;                         Cell_comments= comment(loc(2):loc(3)-1)
;                       
;     Example:
;         Get a directory of the current cdf file and the number of datasets.
;           dircdf, ndset = ndset
;         Get a directory of the current file and the cdf file with id = 3
;           with an additional window for this widget 
;           (to see two different files simultaneously)
;           dircdf & dircdf, 3, /reset
;         Print a directory of current file to the terminal
;           dircdf, /p
;
;-
;
      on_error, 1
@netcdfcom.dat
;
   common dircdfcom, base, list, oldpage, text1, truncate_save

      if n_elements(initcdf) eq 0 then init_cdf
;

      if n_elements (truncate_save) eq 0 then truncate_save =0
      if n_elements (truncate) gt 0 then truncate_save = truncate
;
;     check to see if cdf has been initialized
;
      if keyword_set(reset) then base = 0

      if n_elements(initcdf) eq 0 then init_cdf
      if cdf_id lt 0 then begin
        print, '% DIRCDF: No netCDF files are open.'
        return
      endif
      
      if lcdfvalid ne array_cdfid(cdf_id, 1) then load_cdf_data

      if n_elements(fid) eq 0 then fid = array_cdfid(cdf_id, 0)
      if n_elements(p1) eq 0 then p1 = '' else p1 = strtrim(p1, 2)
       
      lp1 = strlen(p1)
      if lp1 eq 1 and p1 eq '*' then lp1 = 0

      ok = where (array_cdfid(*, 0) eq fid and array_cdfid(*, 1) ne 0, nok)
      if nok eq 0 then begin
        print, '% DIRCDF: '+Strtrim(fid, 2)+' is not a valid file ID.'
        return
      endif
      if nok gt 1 then print,  $
           '% DIRCDF: This is not suppose to happen 2 files with same fid'
      id = ok(0)
;
      ndset = [ num_glo_var(id),num_nod_var(id), num_elem_var(id)]
;
      filename = file_name(id)

      narr = total (ndset)
      if narr eq 0 then begin
        comment = ''
        line = ''
        return
      endif
      line  = make_array(narr+6,/string)
      comment = make_array(narr,/string)
      loc = make_array(4, value = 0L)
      i = 0L
      stuff = 0
      if array_cdfid(id, 1) eq lcdfvalid then begin
        if num_glo_var(id) gt 0 then begin
          if cdf_type(id) eq 1 then $
          line(i) = ' *** Global Variables *** ' else $
          line(i) = ' ***** WDF Arrays ***** '
          i = i+1
          line(i) = ' No.   Dataset Comment/Label'
          i = i+1
          stuff =2
          if lp1 gt 0 then ok = get_match(p1, name_glo_var, nel = nel) $
          else begin
            nel = num_glo_var(id)
            ok = indgen(nel)
          endelse
          if nel gt 0 then for j = 0, nel-1 do begin
            line(i) = string(format='(i5,5x, a)', ok(j)+1, name_glo_var(ok(j)))
            comment (i-stuff) = name_glo_var(ok(j))
            i = i+1
          endfor
        endif else nel = 0L
        loc(1)=nel
        ndset(0) = nel
        if narr gt num_glo_var(id)  then begin
          if i gt 0 then begin
            line(i) = '---------------------------------'
            i = i+1
            stuff = stuff+1
          endif

          if cdf_type(id) eq 1 then $
          line(i) = ' *** Node and Elem Variables *** ' else $
          line(i) = ' *** Fields and Image Arrays *** '
          i = i+1
          line(i) = ' No.   Dataset Comment/Label'
          i = i+1
          stuff = stuff+2
          if num_nod_var[id] gt 0 then begin
            if lp1 gt 0 then ok = get_match(p1, name_nod_var, nel = nel) $
            else begin
              nel = num_nod_var(id)
              ok = indgen(nel)
            endelse
            if nel gt 0 then for j = 0, nel-1 do begin
              line(i) = string(format='(i5,5x, a)', ok(j)+1, name_nod_var(ok(j)))
              comment (i-stuff) = name_nod_var(ok(j))
              i = i+1
            endfor
          endif else nel = 0
          loc(2)= loc(1)+nel
          ndset(1) = nel
          joff = num_nod_var(id)+1
          if num_elem_var[id] gt 0 then begin
            if lp1 gt 0 then ok = get_match(p1, name_elem_var, nel = nel) $
            else begin
              nel = num_elem_var(id)
              ok = indgen(nel)
            endelse
            if nel gt 0 then for j = 0, nel-1 do begin
              line(i)=string(format='(i5,5x,a)', ok(j)+joff,name_elem_var(ok(j)))
              comment (i-stuff) = name_elem_var(ok(j))
              i = i+1
            endfor
          endif else nel = 0
            loc(3) = loc(2)+nel
            ndset(2) = nel
          endif else begin
          loc(2:3) = loc(1)
          ndset(1:2) = 0L
        endelse  
      endif else begin
;       need to read the names
        if  num_glo_var(id) gt 0 then begin
          if cdf_type(id) eq 1 then $
          line(i) = ' *** Global Variables *** ' else $
          line(i) = ' ***** WDF Arrays ***** '
          i = i+1
          line(i) = ' No.   Dataset Comment/Label'
          i = i+1
          stuff = 2
          varid = ncdf_varid(id, 'name_glo_var')
          if varid ge 0 then ncdf_varget, id, varid, Name_glo else begin
            print, '% DIRCDF: Error reading global variable names.'
            return 
          endelse
          name_glo = strtrim(name_glo, 2)
          if lp1 gt 0 then ok = get_match(p1, name_glo, nel = nel) $
          else begin
            nel = num_glo_var(id)
            ok = indgen(nel)
          endelse
          if nel gt 0 then for j = 0, nel-1 do begin
            line(i) = string(format='(i5,5x, a)', ok(j)+1, name_glo(ok(j)))
            comment (i-stuff) = name_glo(ok(j))
            i = i+1
          endfor
        endif else nel = 0L
        loc(1) = nel
        ndset(0) = nel
        if narr gt num_glo_var(id) then begin
          if cdf_type(id) eq 1 then $
          line(i) = ' *** Node and Elem Variables *** ' else $
          line(i) = ' *** Fields and Image Arrays *** '
          i = i+1
          line(i) = ' No.   Dataset Comment/Label'
          i = i+1
          stuff = stuff +2
          varid = ncdf_varid(id, 'name_nod_var')
          if varid ge 0 then ncdf_varget, id, varid, Name_nod else begin
            print, '% DIRCDF: Error reading nodal variable names.'
            return 
          endelse
          name_nod = strtrim(name_nod, 2)
          if lp1 gt 0 then ok = get_match(p1, name_nod, nel = nel) $
          else begin
            nel = num_nod_var(id)
            ok = indgen(nel)
          endelse
          if nel gt 0 then for j = 0, nel-1 do begin
            line(i) = string(format='(i5,5x, a)', ok(j)+1, name_nod(ok(j)))
            comment (i-stuff) = name_nod(ok(j))
            i = i+1
          endfor
          loc(2) = loc(1) + nel
          ndset(1) = nel
          joff = num_nod_var(id)+1
          varid = ncdf_varid(id, 'name_elem_var')
          if varid ge 0 then ncdf_varget, id, varid, Name_elem else begin
            print, '% DIRCDF: Error reading element variable names.'
            return 
          endelse
          name_elem = strtrim(name_elem, 2)
          if lp1 gt 0 then ok = get_match(p1, name_elem, nel = nel) $
          else begin
            nel = num_elem_var(id)
            ok = indgen(nel)
          endelse
          if nel gt 0 then for j = 0, nel-1 do begin
            line(i) = string(format='(i5,5x, a)', ok(j)+joff, name_elem(ok(j)))
            comment (i-stuff) = name_glo_var(ok(j))
            i = i+1
          endfor
          loc(3) = loc(2) + nel
          ndset(2) = nel
        endif else begin
          loc(2:3) = loc(1)
          ndset(1:2) = 0L
        endelse
      endelse
;
      line = line(0:i-1)
      comment = comment(0:i-1-stuff)
      if keyword_set(truncate_save) then begin
        for i = 0, ndset-1 do begin
          ii = strpos(line(i), ' - ', 25)
          if ii gt 0 then begin
            line(i) = strmid(line(i), 0, ii)
            comment(i) = strmid(comment(i), 0, ii)
          endif
        endfor
      endif
      if keyword_set(quiet) then return
      nline = i
      if n_elements (oldpage) eq 0 then oldpage = 24
      if n_elements (page) eq 0 then page = oldpage
      page = page >4
;

      if keyword_set(print) then begin
        print, ' CDF Directory: ', filename
        for i = 0, nline-1 do print, line(i)
        return
      endif
      bad = 0
      if n_elements (base) eq 0 then base = 0 else $
        widget_control, base, bad_id = bad
      if bad ne 0 then base = 0
      if base gt 0 then begin
        if oldpage eq page then begin
          widget_control, text1, set_value = filename
          widget_control, list, set_value = line
          return
        endif else begin
          widget_control , base, /destroy
        endelse
      endif
    if n_elements (xsize) eq 0 then begin
      if max(strlen(line)) lt 40 then xsize = 300 else xsize = 0
    endif
    if xsize gt 0 then $
      base = WIDGET_BASE(TITLE = 'CDF Directory' , /column, xsize = xsize) $ 
      else base = WIDGET_BASE(TITLE = 'CDF Directory' , /column) 
; put in the file name
      text1 = widget_text(base, /frame, value= filename, /scrol)
        
;
;     text2 = widget_text(base, $
;          value=  ' No.    Dataset Comment' )
      oldpage = page
      list = widget_list(base, value = line, ysize = page )
      widget_control, base, /realize
      return
      end
