; ------------------------------------------------------------------------
; $Id: make_anim.pro,v 1.3 2007/06/06 19:39:21 lpmix Exp $
; ------------------------------------------------------------------------
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
;+
; NAME:
;       MAKE_ANIM
;
; PURPOSE:
;       This routine makes an array of screen snapshots to be used in an 
;         annimation.  Individual waveforms are stored in WDF arrays to 
;         make detailed information readily available.  This routine is 
;         limited to one dimensional calculations.
;         See also RUN_ANIM, GET_ANIM_WDF
;
; CALLING SEQUENCE:
;       ARRAY = MAKE_ANIM( [FILENAME] )
;       where 
;       ARRAY:  Three or 4 dimensional byte array of the plots.
;               Dimensions are [3, XSIZE, YSIZE, NFRAME] for true color
;                           or [XSIZE, YSIZE, NFRAME] for pseudo-color.
;
; OPTIONAL INPUTS:
;       FILENAME: Filename of the Exodus II file.
;               Default:  Use DIALOG_PICKFILE() to obtain the file.
;
; KEYWORD PARAMETERS:
;       RERUN:  If set, use currently stored data (Do not open and read new data)
;	
;       XSIZE:  X size of the plot
;               Default:  XSIZE = 900
;       YSIZE:  Y size of the plot
;               Default:  YSIZE = 700
;       CELL:   If set, cell data will stay at cell centers.
;               Default:  CELL = 1
;       TESTIT: If set, do only 5 plots and animate those to check the annimation
;               If testit is a number greater than /equal to 2 then do (testit+1) 
;               animation frames.  If TESTIT is set, the command 
;               "RUN_ANIM, ARRAY" is automatically issued after array is calculated.
;       PIXMAP: Generate the plots in memory (pixmaps) rather than on the screen.)
;
; OUTPUTS:
;       If TESTIT is set an animation is generated on a limited number of frames.
;       All animation frames are returned in array.
;       The following variables are defined in the in the commond block:
;         MAKE_COMMON_ANIM
;       The variables may be retrieved with GET_ANIM_WDF.
;       The variables on the left are the first wdf array for the quantities on the
;       right in the table below.  GET_ANIM_WDF can be used to retrieve these values.
;       NTIME is the number of Time Steps.  
;        
;       Variable  Quantity
;       den1      Density
;       temp      Temperature
;       rtemp     Rad-Temp
;       press     Pressure
;       btheta    BTheta
;       mpress    Mag Press = Btheta * Btheta/ (2*mu)
;       jr        J_r
;       jz        J_z
;       vr        R_Velocity
;       vz        Z_Velocity
;       ar        R_Acceleration
;       az        Z_Acceleration
;       econ      E_Cond
;       tcon      Ther_Cond
;       opaca     Opacity_A
;       opacr     Opacity_R
;       zbar      ZBar
;       sound     Sound Sp
;       artvis    Art. Viscosity
;       specheat  Spec Heat
;       ntime     Number of Time Steps
;
; PROCEDURE:
;       All variables are read into PFIDL structures and lineouts at each time 
;         are stored in WDF arrays.
;       PLOTS are then generated for all times and stored in an array which is 
;          returned.
;       The user should copy this to his area to modify the plots.  If fewer plots
;          are requested, !p.multi should be changed.
;       The size of the display determines the size of the animation that may be 
;          run.
;       Cell data is left on the cell center.
;
; EXAMPLE:
;       Generate animation frames from the file 'idot.exo' and store the results in a.
;         a = MAKE_ANIM('idot.exo')
;
;       Same a above but make a short animation to see if the requested plots are
;         satisfactory.  Assume that the data has been read previously.
;         a = MAKE_ANIM(/testit)
;         
;		F = PICKFILE(/READ, FILTER = ['pro', 'dat'])
;
; MODIFICATION HISTORY:
; 	Written by:	L. P. Mix, Jr., March 30, 2005
;                       Based on similar routine by Thomas A Haill, but totally 
;                         modified.
;-

pro read_cdf_2d, dataset, label, wdf, ntime, time, celldata = cell
  if n_elements (cell) eq 0 then cell = 1

  readcdf, str, dataset, cell = cell, /structure
  
  arr = str.v1
  if str.type eq 8 then begin
    s = size(arr)
    nx = s[1]
    ntime = s[2]
    time = reform((str.x2)[0, *])
    x = str.x1
    for i = 0, ntime-1 do i2w, reform(x[*, i]), reform(arr[*, i]), wdf+i, $
      xlabel = 'Radius' , ylabel= label, /xydata, file = str.file, $
      clabel = label+' - Time = '+strtrim (time[i], 2)
    return
  endif
  ntime = 0
; Type 9 structure
  if str.type ne 9 then begin
    print, '% READ_CDF_2D: error reading data'
    help, /str, str
    return
  endif
  time = str.x4
  ntime = n_elements(time)
  xx = str.x1
  
  for i = 0, ntime-1 do begin
    x = xx[*, i]
    y = arr[*, i]
    nx2 = n_elements(x)/2
    s = sort(x)
    x = reform(x[s], 2, nx2)
    x = total(x, 1)*0.5
    y = total( reform(y[s], 2, nx2), 1)*0.5

    i2w, x, y, wdf+i, xlabel = 'Radius' , ylabel= label, /xydata, $
      file = str.file, clabel = label+' - Time = '+strtrim (time[i], 2)
  endfor
  end 

function  make_anim, filename, xsize = xsize, ysize = ysize, cell = celldata, $
      rerun = rerun, testit = testit, pixmap = pixmap
;
;
; IDL> array = make_anim( rerun = rerun)
@make_anim_common.dat

on_error, 1
;---------------------------------------------------------------------
; setup plot
;---------------------------------------------------------------------

  if n_elements (xsize) eq 0 then xsize = 900
  if n_elements (ysize) eq 0 then ysize = 700
  if n_elements (times_saved) eq 0 then rerun = 0

  oldpmulti= !p.multi
  !P.multi    = [0,3,4]  ; clear, columns, rows
  oldcharsize = !p.charsize
  !P.charsize = 1.5
  oldsymsize = !p.symsize
  !P.symsize  = 1.5
; ----------------------------------------------
  datasets =['^density_1', '^temperature_1', '^rad_temperature', $
             '^pressure_1', '^btheta', '^j_r', '^j_z', '^velocity_r', $
             '^velocity_z', '^acceleration_r', '^acceleration_z', '^econ_1$', $
             '^thermal_con_1', '^opacity_a_1', '^opacity_r_1', '^zbar_1', $
             '^sound_speed_1', '^artificial_viscosity', '^specific_heat_vol_1']
  Labels = ['Density', 'Temperature', 'Rad-Temp', 'Pressure', 'BTheta', $
            'J_r', 'J_z', 'R_Velocity', 'Z_Velocity', 'R_Acceleration', $
            'Z_Acceleration', 'E_Cond', 'Ther_Cond', 'Opacity_A', 'Opacity_R', $
            'ZBar', 'Sound Sp', 'Art. Viscosity', 'Spec Heat', $
            'Mag Press', 'NTIME']
  variables = ['den1', 'temp', 'rtemp', 'press', 'btheta', 'jr', 'jz', 'vr', $
               'vz', 'ar', 'az', 'econ', 'tcon', 'opaca', 'opacr', 'zbar', $
               'sound', 'artvis', 'specheat', 'mpress' ]
; ----------------------------------------------

if keyword_set(rerun) then begin
  filename = file_saved
  times = times_saved
  ndata = ndata_saved
  ntime = n_elements(times)
endif else begin
; ----------------------------------------------

  if n_elements(celldata) eq 0 then celldata = 1

  nwdf = get_maxwdf()
  if nwdf lt 4096 then set_maxwdfarray, 4096
  nwdf = get_maxwdf()
  opencdf, filename, ndset = ndset
  if n_elements(ndset) eq 0 then begin
    print, '% MAKE_ANIM:  No datasets were found.'
    return, 0
  endif
; start at 101
  wdf = 101L
  ndata= n_elements(datasets)
  var_arr = make_array(ndata+2, /long)
  for i = 0, ndata - 1 do begin
    read_cdf_2d, datasets[i], labels[i], wdf, ntime, times, celldata = celldata 
    stat = execute(variables[i] +' = wdf')
    var_arr[i] = wdf
    if i eq 0 then if ((ntime*(ndata+1) + 105) gt nwdf) then  $
      set_maxwdfarray, (long((ntime *(ndata+1) + 105-nwdf)/1024)+1)*1024 +nwdf
    print, i+1, ' of ', strtrim(ndata, 2), ', ntime = ', strtrim(ntime, 2)
    if ntime eq 0 then begin
      print, '% MAKE_ANIM: Error reading dataset.'
      return, 0
    endif
    wdf = wdf+ntime
  endfor
  wdinfo, 101, file= filename
  two_mu0 = !pi * 8.e-7
  for i = 0, ntime -1 do begin
    y = getyf(btheta+i)
    i2w, getxf(btheta+i), y*y/two_mu0, wdf +i, xlabel = 'Radius', $
      clabel='Magnetic Press - Time =' + strtrim(times[i], 2), $
      ylabel= 'Magnetic Pressure', /xydata, file = filename
  endfor
  mpress = wdf
  var_arr[ndata]= wdf
  var_arr[ndata+1]= ntime
  wdf = wdf+ntime
; ----------------------------------------------
  closecdf
; ----------------------------------------------
endelse
 nf = round(ntime)
 times_saved = times
 file_saved = filename
 ndata_saved = ndata
; if testit is set, make only 5 frames
 if keyword_set(testit) then begin
   ndelta = ( testit ge 2) ? testit : 4
   dt = nf/ndelta > 1
   if (nf mod dt) eq 0 then dt = (dt -1) > 1
 endif else dt = 1
 if dt ne 1 then nf = 1+ long((nf-1)/dt)
 device, get_visual_depth = depth
 if depth le 8 then begin
   array = make_array(xsize, ysize, nf, /byte)
   truecolor = 0
 endif else begin
   array = make_array(3, xsize, ysize, nf, /byte)
   truecolor = 1
 endelse

 print, "Number of Times  = ", ntime

; ----------------------------------------------

 oldwin = !d.window
 if oldwin ge 0 then begin
   oldx_save = !x
   oldy_save= !y
 endif
 window, xsize = xsize, ysize = ysize, /free, pixmap = pixmap

 mkcircle
 f = 0
 ndata = 0
 work = get_maxwdf()
 nlin = 1
 abs = -1
 nsym = 1
 over = 1
 frame = 0
 for i = 0L, ntime-1, dt  do begin
     stime = strtrim(times[i], 2)
     print, " frame = ", strtrim(i+1, 2), " of ", strtrim(nf, 2), $
      " time = ", stime

; 1 = plus
; 2 = asterisk
; 3 = dot
; 4 = diamond
; 5 = triangle
; 6 = square
; 7 = X
     ;--------------------  Density  --------------------
     plo, den1+i, /ylog, /trun, title = wdcomment(den1+i), yr = yr, $
       over =over, nline = nlin, nsym = nsym, leg = -1, logrange = 1e-5

     ;--------------------  B Field  --------------------
     plo,  btheta+i, /trun, /left, title = wdcomment(btheta+i), over=over, $
       nline = nlin, nsym = nsym
     
     ; over plot current fraction (r*btheta)
     x = getxf(btheta+i)
     i2w, x, x*getyf(btheta+i), work, clab =  'R*BTheta', /xydata, $
       xlabel = 'Radius' , ylabel= 'R*BTheta'
     plo, work  , /right, nline = nlin, nsym = nsym, over= over
     
     ;--------------------  Electrical Conductivity  --------------------
     plo, econ+i, /ylog, /trun, title = wdcomment(econ+i), $
       over= over, nlin = nlin, nsym = nsym, leg = -1, logrange=1e-4

     ;--------------------  Temperature  --------------------
     plo, [temp, rtemp]+i, /truncate, over = over, $
       nlin = nlin, nsym = nsym,  /ylog, logrange = 1e-4, $
       title = 'Temp and Rad Temp, Time= '+stime

     ;--------------------  Current Density  --------------------
     plo, jz+i, over = over, /ylog, /truncate, abs = abs, logrange = 1e-5, $
       title = 'Abs Curr Den, Time= '+stime,  nlin = nlin, nsym = nsym, $
       Ytitle = 'Current Density', /left
;    overplot the positive portion of the data
     plo, /no,  jz+i, /over,  nlin = nlin, nsym = nsym, /trun
     plo, jr+i, over = over, /ylog, abs= abs, logrange = 1e-5, /trun, $
       nlin = nlin, nsym = nsym,  Ytitle = 'Current Density', /right
;    overplot the positive portion of the data
     plo, /no,  jr +i, /over, nlin = nlin, nsym = nsym, /trun

     ;--------------------  Thermal Conductivity  --------------------
     plo, tcon+i, /truncate, /ylog, abs = abs, leg = -1, $
       title = 'Thermal Cond, Time= '+stime, over = over, $
       logrange = 1e-5, nline = nlin, nsym = nsym

     ;--------------------  Pressure  --------------------
     plo, [mpress+i, press+i], /truncate, /ylog, $
       title = 'Pressure, Time= '+stime, /left, over = over, $
       nline = nlin, nsym = nsym, logrange = 1e-5
                                              
;    plot artificial viscosity on the right axis
     plo, artvis+i, /right, over = over, nlin = nlin, nsym = nsym, /ylog, $
       logrange = 1e-4

     ;--------------------  Velocity  --------------------

     plo, vr+i, /trun, /ov, abs = abs, title = 'Abs Velocity, Time= '+stime, $
       nlin = nlin, nsym = nsym, ytit = 'Abs Velocity', /left
     plo, vr+i, /trun, /ov, /no, nlin = nlin, nsym = nsym
     plo, sound+i, /trun, /ov, /no, nlin = nlin, nsym = nsym
     plo, vz+i, /trun, /ov, nlin = nlin, nsym = nsym, abs = abs, /right
     plo, vz+i, /ov, nlin = nlin, nsym = nsym, /no, /truncate
    

     ;--------------------  Opacity  --------------------
     plo, [opaca, opacr]+i, /trun, /ylog, over = over, nlin = nlin, $
       nsym = nsym, title = 'Opacity, Time= '+stime, logrange = 1e-5

     ;--------------------  Zbar  --------------------
     plo, zbar+i,  /trun, /ylog, over = over, nlin = nlin, leg = -1,  $
       nsym = nsym, title = 'Zbar, Time= '+stime, logrange = 1e-5

     ;--------------------  Acceleration  --------------------
     plo, ar+i, /trun, /ov, abs = abs, title = 'Abs Acceleration, Time= '+stime, $
       nlin = nlin, nsym = nsym, ytit = 'Acceleration', /left, /ylog, $
       logrange = 1e-5
;    plot the positive portion
     plo, ar+i, /trun, /ov, /no, nlin = nlin, nsym = nsym
;    Z acceleration
     plo, az+i, /trun, /ov, nlin = nlin, nsym = nsym, abs = abs, /right
;    plot the positive portion
     plo, az+i, /ov, nlin = nlin, nsym = nsym, /no, /truncate

     ;--------------------  Specific Heat  --------------------
     plo, specheat+i, /trun, /ylog, abs = abs, logrange = 1e-5, $
       nlin = nlin, nsym = nsym, over = over, leg = -1, $
       title = 'Specific Heat, Time= '+stime

     ;--------------------  Finish  --------------------
     if i eq 0 then begin
       nwdf = get_maxwdf()
       if ndata*nf gt nwdf then set_maxwdfarray, (ndata*nf/1024 +1)*1024
     endif
     if truecolor eq 1 then $
       array[*,*,*, frame] = tvrd(/true) else $
       array[*,*, frame] = tvrd()
     frame = frame +1
     !p.multi[0] = 0
  endfor
  wdelete
  !p.multi  = oldpmulti
  !p.charsize = oldcharsize
  !p.symsize = oldsymsize
;
  if oldwin ge 0 then begin
     wset, oldwin
     !x = oldx_save
     !y = oldy_save
  endif
  if keyword_set(testit) then run_anim, array
  return , array
  end
