; ------------------------------------------------------------------------ 
; $Id: readcdf.pro,v 1.11 2009/09/02 16:29:02 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
 Pro  readcdf, wdfa, dataset, Zvalue, fileid = fileid, structure = structure, $
     block = block, add = add, error = error, cell_values = cell_values, $
      rectilinear = lcrect_val, index = index, global = global, grid = grid, $
      wdfarray = wdfarray

;             [, ADD= add] [, BLOCK = block] [, RECTILINEAR = rectilinear]
;    BLOCK     - If Cell data only indicated blocks will be read.
;                Default is all blocks
;+
;  Reads a CDF dataset into a structure array.  Procedure options are explained
;  below.
;  Note: Cell or element data will be moved to the nodes unless keyword
;        CELL_VALUES is set.  This data will be stored as multi block data.
;        NODE data will be stored as single block data unless rectilinear
;        is set.
;        
;  Note: If delta_x/x or delta_y/y is less than  error then  data will be
;        Changed to x vs t or y vs t data for Exodus II data 
;        default value is 1.0e-3
;
;  Format:  READCDF, STRUCT [, DATASET] [, ZVALUE] [, /INDEX]
;             [, FILEID = fileid]  [, GRID = grid] [, WDFARRAY = wdfarray]
;             [, CELL_VALUES = cell_values] [, ERROR = error] [, /STRUCTURE ]
;             [, ADD= add] [, RECTILINEAR = rectilinear] [, GLOBAL = global]
;  where:
;    Struct    - structure Array number
;   NOTE: Dataset must be provided unless grid is set.
;    dataset   - indicates a CDF dataset number or 
;                a search string for dataset comments.  
;                (Not required if GRID is set.)
;        See: GET_DSET for more details
;    ZVALUE    - Ignored unless 3D data. If 3D data assumed to be rectilinear
;                in the Z direction.  A slice is taken at Z = ZVALUE
;                In all cases the nearest GRID or CELL value will be used.
;                Default value = first z grid
;    INDEX     - If ZVALUE is used and index is set then the value of ZVALUE 
;                is assumed to be a grid index in the Z direction. If INDEX 
;                is set then ZVALUE goes from 1 to the maximum number of
;                Z planes.
;    GRID      - If set, a node array will be returned.
;                If 3D data then the first zone will be returned unless a 
;                ZVALUE is set.
;                If rectilinear is also set, then a standard PFF grid structure
;                will be returned. 
;    WDFARRAY  - For 1D data
;                In addition to storing a type 8 structure as space vs time, 
;                data will be stored in wdfarrays beginning with WDFARRAY.
;                If necessary, SET_MAXWDFARRAY will be called to store the data.
;    CELL_VALUES - If set, element or cell data will be returned with the cell
;                  centers.  Default - Move cell data to the nodes for finite
;                  element data and leave at the cells for rectilinear data; ie 
;                  Default value: Cell_Values = retilinear
;    fileid    -  file id of the CDF file containing the data
;    STRUCTURE - if set a structure will be returned in STRUCT
;    ADD       - If set then multiple datasets will be added.
;                All datasets must be either nodal or element (cell) datasets.
;    Error     - default value for delta-X/x to change ExodusII cell data to
;                X vs Time data Default value is (max -min)/(|Max|+|min|)<1.0e-3
;                where | | indicates absolute value.
;    RECTILINEAR- If set, attempt to put the data on a rectilinear grid.
;                 In current version, 3D data must be read as rectilinear.
;                 Multiple block data will be read as multiple block data
;                 If the number of dimensions is 3 then the user should 
;                 specify RECTILINEAR =1
;                 Default is RECTILINEAR = 0
;                 If 3d data has only 1 cell then will be read as 2d degenerate.
;     Note: Values are assumed equal if they differ by less than 1/2^14 =(6e-5)
;           Only the initial grid is checked.  A Lagrangian problem which starts
;           with a rectilinear grid can be read as rectilinear data; the grid
;           will be the initial grid for all times.
;     Note: A less sensitive of delta_X /xrange < delta may be used if 
;           RECTILINEAR = DELTA 
;    GLOBAL    - If set to zero, then readcdf will attempt to preserve blocks
;                when reading rectilinear data.
;                If undefined or 1 and if rectilinear is set or ndim = 3 then 
;                READCDF will attempt to generate a global rectilinear grid.
;                If this is not successful then, a block by block rectilinear
;                grid will be attempted.
;                NOTE: Default is a global rectilinear grid.
;
;  Examples:
;    Read dataset 34 into structure 1 from file with file id of 4.
;      readcdf, 1, 34, fil = 4
;
;    Read dataset with 'KE' into structure 2
;      readcdf, 2, 'KE'
;-
      on_error, 1
;     add common block
@netcdfcom.dat
@qsstrcom.dat
;
;     check to see if cdf and structures have been initialized
;
      if n_elements(initcdf) eq 0 then init_cdf
      if n_elements (structinit) eq 0 then qsstruct_init
;
;
      if cdf_id  lt 0 then begin
        print, '% READCDF: No netCDF files are open.'
        return
      endif
;
      nparam = n_params()
      if (nparam lt 1 ) then begin
 print,'% Format: READCDF, Struct, [, DATASET] [, ZVALUE] [, GRID = grid] '
 print,'       [, FILEID = fileid] [, CELL_VALUES = cell_values]'
 print,'       [, ERROR = error] [, /STRUCTURE ] '
	return
      endif
      if keyword_set(structure) then begin
        if not arg_present(wdfa) then begin
          print, '% READ_CDF: Structure must be valid variable'
          return
        endif
      endif else begin
        wdfa = get_save_str(wdfa, 0)
        if wdfa(0) eq 0 then begin
          print,'% READCDF: Structure array value must be provided'
          return
        endif
        wdfar = wdfa(0)
        if wdfar lt 1 or wdfar gt maxstructure then begin
  print, '% READCDF: WDF array must be greater than 0 and </=', maxstructure
          return
        endif
      endelse

      if n_elements (error) eq 0 then error = 1.e-3
      if n_elements (fid) eq 0 then fid = array_cdfid(cdf_id, 0)
;
      if array_cdfid(cdf_id, 0) eq fid then begin
        if lcdfvalid ne array_cdfid(cdf_id, 1) then load_cdf_data
      endif else begin
        ok = where (fid eq array_cdfid(*, 0), nok)
        if nok ne 1 then begin
         print,'% READCDF: ', strtrim(nok,2), ' netCDF files with a fid of ', $
            strtrim(fid, 2)
          print, '         No Data read'
          return
        endif
        cdf_id = ok(0)
        if lcdfvalid ne array_cdfid(cdf_id, 1) then load_cdf_data
      endelse

      if keyword_set(lcrect_val) then begin
        lcrect = 1
        if lcrect_val lt 1 then deltaval =  lcrect_val
      endif else lcrect = 0
      if cdf_type(cdf_id) eq 1 then begin
        parmin = 2
        if keyword_set(grid) then parmin = 1
        if (nparam lt parmin) then begin
          print,'% READCDF: ', strtrim(parmin,2),' parameters are required'
          return
        endif
          nmax = num_elem_var(cdf_id) + num_nod_var(cdf_id) 
        if nmax eq 0 then begin
          print, '% READCDF: No structure arrays in this netCDF file'
          return
        endif
        if n_elements(dataset) eq 0 and (not keyword_set(grid)) then begin
          print,'% READCDF: Dataset must be defined'
          return
        endif
        nblo = num_el_blk
        ndim = num_dim
        if keyword_set(grid) and not keyword_set(lcrect) then begin
          if cdf_type(cdf_id) ne 1 then begin
            print, '% READCDF: Grid information available with EXODUS-II data'
            print, '   cdf_type = ', strtrim(cdf_type(cdfid), 2)
          endif
          x1lab = x1_label
          x2lab = x2_label
          x3lab = x3_label
          x4lab = 'Time'
          x4 = time_whole
          nt = n_elements(x4)
;         Range will be wrong
          type = 10L
          range = make_array(2, ndim, nblo+1)
;
          x1 = reform(coord_node(*,*,0))
          x2 = reform(coord_node(*,*,1))
            
          two = 3L
          zval = 0.0
          x3 = zval
          if ndim eq 3 then begin
;           take the first plane if no zvalue specified
            if n_elements(dataset) ne 0 then zvalue = dataset(0) else begin
              zvalue = 1
              index = 1
            endelse
;           look at initial time
            z = z_array
            nz = n_elements(z)
            if keyword_set(index) then begin
              zind = round(zvalue-1) > 0
              zval = z(zind < (nz-1))
            endif else begin
              if zvalue le z(0) then zval = z(0) else begin
                if  zvalue(0) ge z(nz-1) then zval = z(nz-1) else begin
                  dist = min(abs(z -zvalue), zind)
                  zval = z(zind)
                endelse
              endelse
            endelse
            x3 = make_array(nt, value = zval)
            ssize = [n_elements(x1(*, 0)),n_elements(x2(*, 0)),1L ]
            range(*, 2, *)= zval
          endif else ssize = [n_elements(x1(*, 0)),n_elements(x2(*, 0))]
          clab = 'Grid Data - '+file_title(cdf_id)
          
          tlab = 'netCDF Grid Data'
          locx = 0L
          locv = make_array(nblo+1, value = 0L, /long)
          nlocx = 0L
          for i = 1, nblo do begin
            varid = ncdf_varid(fcdf_id[cdf_id], 'connect'+strtrim(i, 2))
            if varid ge 0 then begin
              ncdf_varget, fcdf_id[cdf_id], varid, connect
              connect = connect-1
              ss = size(connect)
              if ss[0] eq 1 then ss=[2, ss[1], 1, ss[2], ss[3]]
              if ndim eq 3 then begin
                jj = 0
                s2 = ss(2)
                s1 = ss(1)/2
                s1m = s1-1
                del = 2*z_array_err
                connz = make_array(s1, s2 , /long)
                for ii = 0L, ss(2)-1 do begin
                  ok = where(abs(coord_node(connect(*, ii), 0, 2) - zval) $
                        lt del, nok)
                  if nok gt 0 then begin
                    if nok ne 4 then begin
                       print, '% READCDF: Bad Z Block', i, ii, nok, ok
                       return
                    endif
                    tt = total(ok)
                    if tt lt 22 then begin
                      if ok(0) eq 0 then begin
                        if tt eq 6 then ok = [0, 3, 2, 1] else $
                          if tt eq 10 then  ok =[ 0, 1, 5, 4] else $
                            ok= [0, 4, 7, 3]
                       endif else if tt eq 14 then ok = [1 ,2, 6, 5] else $
                         ok = [2, 3, 7, 6]
                    endif 
                    connz (*, jj)= connect(ok, ii)
                    jj = jj+1 
                  endif 
                endfor
                connect = temporary(connz(*, 0:jj-1))
                ss = size(connect)
                if ss[0] eq 1 then ss=[2, ss[1], 1, ss[2], ss[3]]
              endif
              range(0, 0, i) = min(x1(connect), max = mm)
              range(1, 0, i) = mm
              range(0, 1, i) = min(x2(connect), max = mm)
              range(1, 1, i) = mm
              if i eq 1 then conn= reform(connect, ss[ss(0)+2], /overwrite) $
                  else conn = [conn, reform(connect, ss[ss(0)+2], /overwrite)] 
              pp = lindgen(ss(2)+1)*ss(1)+ locx[nlocx]
              locx = [locx, pp(1:*)]
              nlocx = nlocx+ss(2)
              locv(i) = nlocx
            endif else begin
              print, '% READCDF: Error reading ', 'connect'+strtrim(i, 2)
              return
            endelse
          endfor
          range(0, 0, 0)= min(range(0, 0, 1:nblo))
          range(1, 0, 0)= max(range(1, 0, 1:nblo))
          range(0, 1, 0)= min(range(0, 1, 1:nblo))
          range(1, 1, 0)= max(range(1, 1, 1:nblo))
  
          spare = make_array(5, val = 0)
          wdfa = {type: type, ndim:ndim, block:nblo, two:two, $
               ntime:nt,  slice: 0.0, size:ssize, range: range, $
                order: 0L, locx:locx, locv: locv, $
                x1:x1, x2:x2, x3:x3, x4:x4, connect:conn, $
                x1lab : x1lab, x2lab :x2lab, x3lab : x3lab, $
                x4lab : x4lab, file : file_name(cdf_id), spare : spare, $
                tlab : tlab, clab:clab   }
          if keyword_set (structure) then return
          i2s, wdfa, wdfar, /no_copy
          wdfa = wdfar
          return
        endif

        ntime = n_elements(time_whole)
;       if ndim eq 3 then lcrect = 1 
        if n_elements (global) eq 0 then global = 1
;       if we have 3 dimensions then we must have rectilinear data
;       if (ndim  gt 2)  and  (lcrect ne 0) and $
        if  (lcrect ne 0) and $
          (n_elements(x1_array)+n_elements(x2_array) lt 5) then begin
          get_rectilinear, global, ier, delta= deltaval
          if ier ne 0 and global eq 1 then begin
            global = 0
            get_rectilinear, global, ier, delta= deltaval
          endif
          if ier eq 0 then lc_global = global else lc_global = -1
        endif 
        if lcrect ne 0 and lc_global lt 0 then begin
          Print, '% READCDF: Unable to generate rectilinear data.'
          return
        endif
        slice = 0.0
        if keyword_set (grid) then begin

          if ndim eq 2 then two = 3L else two = 0L
          if lc_global ne 0 then nblo = 1L
          clab = 'Rectilinear Grid Data - '+file_title(cdf_id)
          tlab = 'netCDF Grid Data'

          wdfa = {type:3L, ndim:ndim, block:nblo, size:rec_size, $
            two:two, slice:slice, range:rec_range, locx:loc_x, $
            x1: x1_array, x2: x2_array, x3: x3_array, $
            x1lab: x1_label, x2lab: x2_label, x3lab: x3_label, $
            tlab: tlab, clab:clab, file:file_title(cdf_id), $
            spare: make_array(5, value = -3)}
          if keyword_set (structure) then return
          i2s, wdfa, wdfar, /no_copy
          wdfa = wdfar
          return
        endif
        if num_nod_var[cdf_id] + num_elem_var[cdf_id] eq 0 then begin
          print, '% READCDF: No element or nodal datasets.'
          return
        endif
        if num_nod_var[cdf_id] gt 0 then begin
          vnames = name_nod_var
          if num_elem_var[cdf_id] gt 0 then vnames = [vnames,  name_elem_var]
        endif else if num_elem_var[cdf_id] gt 0 then vnames = name_elem_var
        get_ncdf_string, dataset, vnames, map, nmap, nmax, ier, lcstring
      
        if ier ne 0 then begin
 print,'Valid datasets not found. Please change the dataset specification'
           return
        endif
;
        if nmap ne 1 and not keyword_set(add) then begin
          print,'% READCDF: Only 1 dataset may be specified. The '  $
             + 'datasets with this specifier are:'
          tempstr =  [name_nod_var, name_elem_var]
           print, '  No.  Dataset Comment'
          for i=0L, nmap-1 do print,format='(i4,3x, a)',map(i),tempstr(map(i)-1)
          return
        endif else if nmap ne 1 then begin
          maxm = max(map(0:nmap-1), min = minm)
          if maxm gt num_nod_var(cdf_id) and minm le num_nod_var(cdf_id) $
            then begin
       print,'% READCDF: Add keyword must be used with the same type variable'
            print, '      Variables should be either cell or nodal variables'
            print, '      Bad datasets with this specifier are:'
            tempstr =  [name_nod_var, name_elem_var]
            print, '  No.  Dataset Comment'
            for i=0L, nmap-1 do print,format='(i4,3x, a)',map(i),tempstr(map(i))
            return
          endif
        endif

        data =max(map)
        if data gt nmax then begin
          print, '% READCDF: Invalid data value returned from GET_NCDF_STRING'
          Print, '           Value = ', strtrim (data, 2)
          return 
        endif
        apiversion = api_version[cdf_id]
        triangles = 0L
        loctri = 0L
        type = 9L
        nt = n_elements(time_whole)
        x1lab = x1_label
        x2lab = x2_label
        x3lab = x3_label
        x4lab = 'Time'
        x3 = 0
        x4 = time_whole
        two = 3L
        if map(0) le num_nod_var(cdf_id) then begin
          if (n_elements (node_array) lt 4)  and (apiversion lt api_change) then begin
            varid = ncdf_varid(fcdf_id[cdf_id], 'vals_nod_var')
            if varid ge 0 then begin
              ncdf_varget, fcdf_id[cdf_id], varid, node_array
            endif else begin
              print, '% READCDF: No Node array variables found in Exodus II data'
              print, '                 Number of Nodal Variables set to zero.'
              num_nod_var(cdf_id) = 0
              return
            endelse
          endif
;
          clab = name_nod_var(map(0)-1) 
          if apiversion gt api_change then begin
            varid = ncdf_varid(fcdf_id[cdf_id],'vals_nod_var'+strtrim(map[0],2))
            ncdf_varget, fcdf_id[cdf_id], varid, vv
          endif else vv = reform(node_array(*, map(0)-1, *))
          if nmap gt 1 then begin
            if apiversion gt api_change then begin
              for i = 1L, nmap-1 do begin
                varid = ncdf_varid(fcdf_id[cdf_id],'vals_nod_var'+strtrim(map[i],2))
                ncdf_varget, fcdf_id[cdf_id], varid, val
                vv = vv + val
              endfor
            endif else for i = 1L, nmap-1 do vv = vv+reform(node_array(*, map(i)-1, *))
            for i = 1, nmap-1 do clab = clab+' + '+ name_nod_var(map(i)-1)
          endif
;         standard node data
          if lcrect eq 0 then begin
            x1 = reform(coord_node(*,*,0))
            if ndim gt 1 then x2 = reform(coord_node(*,*,1)) else $
              x2 = time_whole
            locx = [0, num_nodes ]
            locv = locx

            if ndim eq 3 then begin
;             currently must be planar if 3 dimensions.
              z = z_array
              nz = n_elements(z)
              if n_elements(zvalue) eq 0 then begin
                zvalue = 1
                index = 1
              endif else  zvalue = zvalue(0)
              if keyword_set(index) then begin
                zind = round(zvalue-1) > 0
                zval = z(zind < (nz-1))
              endif else begin
                if zvalue le z(0) then zval = z(0) else begin
                  if  zvalue ge z(nz-1) then zval = z(nz-1) else begin
                    dist = min(abs(z -zvalue), zind)
                    zval = z(zind)
                  endelse
                endelse
              endelse
              x3 = make_array(nt,value = zval) 
              zz = reform(coord_node(*,0,2))
              ok = where ((zz gt zval-z_array_err ) and $
                 (zz lt zval+z_array_err), nok)
              if nok lt 9 then begin
                print, '% READCDF: Less than 9 nodes found with plane', zval
                return
              endif
              x1 = x1(ok, *)
              x2 = x2(ok, *)
              vv = vv(ok, *)
              ssize = [nok, nok] 
              slice = zval
            endif else begin
              ssize = [num_nodes, num_nodes]
              slice = 0.0
            endelse
            two = 3L
            nblo = 1
            range = make_array(2,ndim,2)
            range(0, 0, *)= min(x1, max = mm)
            range(1, 0, *)= mm
            if ndim eq 1 then begin
              ndim = 2
              type = 8L
              two = 3L
              x2lab = 'Time'
              nx2 = (size(x1))[1]
              ssize = [nx2, nt]
              x2 = transpose(time_whole# make_array(nx2,value = 1.))
              x4 = 0.0
              x4lab = ''
              nblo = 1
              range = make_array(2, 2, 2)
              range(0, 0, *) = min(x1, max = mm)
              range(1, 0, *) = mm
              range(0, 1, *) = min(time_whole,max = mm)
              range(1, 1, *) = mm
              temp =0
              if keyword_set(wdfarray) then begin
                if wdfarray lt 0 then begin
                  print, '% READCDF:  WDFARRAY is ignored'
                  print, '            WDFARAAY must be greater than zero'
                endif else begin
                  wdfarray = round (wdfarray) > 1L
                  if wdfarray + nt ge get_maxwdf() then $
                    set_maxwdfarray, wdfarray +nt+ 8
                  tit= make_array(nt, /string)
                  for i = 0, nt-1 do tit[i] = clab+' - Time=' $
                     +strtrim(time_whole[i], 2)
                  for i = 0, nt-1 do i2w, x1[*, i], vv[*, i], wdfarray+i, $
                    /xydata, clab = tit[i], xlabel = x1lab, ylab= 'Amplitude', $
                    file = file_name(cdf_id)
                endelse
              endif
              nt = 1
            endif else begin
              range(0, 1, *)= min(x2, max = mm)
              range(1, 1, *)= mm
              if ndim eq 3 then begin
                range(0, 2, *)= zval
                range(1, 2, *)= zval
              endif
            endelse
          endif else begin 
;           rectilinear data
            type = 1L
            x1 = x1_array
            x2 = x2_array
            if lc_global ne 0 then nblo = 1
            two = 0L
            if ndim eq 2 then begin
              x3 = reform(time_whole#replicate(1.0, nblo), ntime*nblo)
              ndim = 3L
              x3lab = 'Time'
            endif else begin
              x3 = x3_array
              x4 = reform(time_whole#replicate(1.0, nblo), ntime*nblo)
              ndim = 4L 
            endelse
            ssize = rec_size
            range = rec_range
            locx = loc_X
            vt= make_array(n_elements(node_order), nt)
            for i=0L, nt-1 do vt(*, i) = vv(node_order, i)
            vv = temporary(vt)
            vv = reform(vv, n_elements(vv),/overwrite)
            locv = loc_farray
          endelse
          tlab = 'netCDF Node Variable'
          triangles = tri_angles
          loctri = loc_triangles
        endif else begin
;
;         cell or element type data
;
          map = map - num_nod_var(cdf_id)
          maptrim = strtrim(map, 2)
          if max(map) gt num_elem_var(cdf_id) then begin
      print,'% READCDF: No element (cell) variables with this dataset number ', $
            strtrim(map+ num_nod_var(cdf_id),2)
            return
          endif
          nt = ntime
          clab = name_elem_var(map(0)-1)
          if nmap gt 1 then $
            for i = 1, nmap-1 do clab = clab+' + '+ name_elem_var(map(i)-1)
          nvt = loc_f(nblo)
          vt = make_array(nvt, nt)
          locv= loc_f
          for i = 1, nblo do begin
            for ii = 0, nmap-1 do begin
              var = 'vals_elem_var'+maptrim(ii)+'eb'+strtrim(i, 2)
              varid =  ncdf_varid(fcdf_id[cdf_id], var)
              if varid ge 0 then begin
                ncdf_varget, fcdf_id[cdf_id], varid, vtempjj
              endif else begin
          print,'% READCDF: No element array variables found in Exodus II data'
                print,'         for data:', var
                return
              endelse
              if ii eq 0 then vtemp = temporary(vtempjj) else $
                vtemp = vtemp +vtempjj
            endfor
            vt(locv(i-1):locv(i)-1, *) = vtemp

          endfor 
          vtemp = 0
          if n_elements(cell_values) eq 0 then cell_values = lcrect
          if keyword_set(cell_values) then begin
            if lc_global eq 0 then nblo = num_el_blk else nblo = 1
            if lcrect ne 0 then begin
              x1 = x1_carray
              x2 = x2_carray
              if ndim eq 2 then begin
                x3 = reform(time_whole#replicate(1.0, nblo), ntime*nblo)
                x3lab = 'Time'
                ndim = 3L
              endif else begin
                x3 = x3_carray
                x3lab = x3_label
                x4 = reform(time_whole#replicate(1.0, nblo), ntime*nblo)
                x4lab = 'Time'
                ndim = 4L
              endelse
              locx = loc_xc
              locv = loc_fcarray
              ssize = rec_csize
              range = rec_crange
              vv = make_array(nvt, nt)
              locv = loc_fcarray
              for i=0L, nt-1 do vv(*, i) = vt(cell_order, i)
              if max(ssize(0, *)) eq 1 then begin
                two = 1L 
                slice = x1(0)
              endif else if max(ssize(1, *)) eq 1 then begin
                two = 2L
                slice = x2(0)
              endif else if max(ssize(2, *)) eq 1 then begin
                two = 3L
                slice = x3(0)
              endif else two = 0L
              type = 1L
            endif else begin
              two = 3L
              nblo = num_el_blk

              x1 = reform(coord_cell(*,*,0))
              if ndim gt 1 then x2 = reform(coord_cell(*,*,1))
              locv = loc_f

              z = z_carray
              nz = n_elements (z)
              vv = temporary(vt)
              if nz eq 1 then ndim = 2
              if ndim eq 3 then begin
                x3 = reform(coord_cell(*,0,2))
                if n_elements(zvalue) eq 0 then begin
                  zvalue = 1
                  index = 1
                endif else zvalue = zvalue(0)
                if keyword_set(index) then begin
                  zind = round(zvalue-1) > 0
                  zval = z(zind < (nz-1))
                endif else begin
                  if zvalue le z(0) then zval = z(0) else begin
                    if  zvalue ge z(nz-1) then zval = z(nz-1) else begin
                      dist = min(abs(z -zvalue), zind)
                      zval = z(zind)
                    endelse
                  endelse
                endelse
                ii = 0
                eps = z_array_err*4
                nn = 0
                loc = locv

                for i = 1, nblo do begin
                  ok = where (x3(locv(i-1):locv(i)-1) ge zval-eps and $
                    x3(locv(i-1):locv(i)-1) le zval+eps, nok)
                  if nok gt 0 then begin
                    nn = nn+1
                    x1(ii:ii+nok-1, *) =x1(locv(i-1)+ok, *)
                    x2(ii:ii+nok-1, *) =x2(locv(i-1)+ok, *)
                    vv(ii:ii+nok-1, *) =vv(locv(i-1)+ok, *)
                    loc(nn) = loc(nn-1)+nok
                    ii = ii+nok
                  endif
                endfor
                locv = temporary(loc(0:nn))  
                nblo = nn
              endif else zval = 0.0
              x3 = zval
              x3lab = ''
              locx = make_array(/long, ndim, nblo+1, value =0L)
              clab = name_elem_var(map(0)-1)
              ssize = make_array(ndim, nblo, /long)
              range = make_array(2, ndim, nblo+1)
              for i = 1, nblo do begin
                locx(*, i) = locv(i)
                range(0, 0, i) = min(x1(locv(i-1):locv(i)-1), max = mm)
                range(1, 0, i) = mm
                if num_dim gt 1 then begin
                  range(0, 1, i) = min(x2(locv(i-1):locv(i)-1), max = mm)
                  range(1, 1, i) = mm
                endif
                ssize (*, i-1) = locx(*,i)-locx(*,i-1)
              endfor
              range(0, 0, 0) = min(range(0, 0, 1:*))
              range(1, 0, 0) = max(range(1, 0, 1:*))
              if num_dim gt 1 then  begin
                range(0, 1, 0) = min(range(0, 1, 1:*))
                range(1, 1, 0) = max(range(1, 1, 1:*))
              endif
            endelse
          endif else begin
;           Cell data put onto the nodes.
            vv = make_array(num_nodes, nt, value = 0.0)
            nhit = make_array(/int, num_nodes)
            ioff = 0
            for iblo = 1, nblo do begin
              varid = ncdf_varid(fcdf_id[cdf_id], 'connect'+strtrim(iblo, 2))
              if varid ge 0 then begin
                ncdf_varget, fcdf_id[cdf_id], varid, connect
                connect = connect-1
                ss = size (connect)
;               np = n_elements(connect(0, *))
;               number of cells in this block
                if ss[0] eq 1 then np = 1 else np = ss[2]
                
;               nnodes =  n_elements(connect(*, 0))
;               number of nodes per cell in this block
                nnodes = ss[1]
                for j = 0L, np-1 do begin
                  temp = vt(ioff+j,*)
                  for i= 0L, nnodes-1 do $
                  vv(connect(i, j), *) = vv(connect(i,j), *) + temp
                  nhit(connect(*,j)) = nhit(connect(*, j))+1
                endfor
              endif else begin
                print,'% READCDF: No connection array found in Exodus II data'
                print,'         for block:', iblo  
                return
              endelse
              ioff = ioff+np
            endfor
            ok = where (nhit gt 1, nok)
            if nok gt 0 then $
              for i = 0L, nok-1 do vv(ok(i), *) = vv(ok(i), *)/nhit(ok(i))

;           rectilinear data

            if lcrect ne 0 then begin
              x1 = x1_array
              x2 = x2_array
              if ndim eq 2 then begin
                x3 = time_whole
                x3lab = 'Time'
                ndim = 3L
              endif else begin
                x3lab = x3_label
                x3 =  x3_array
                x4 = time_whole
                x4lab = 'Time'
                ndim = 4L
              endelse
                    
              locx = loc_x
              locv = loc_farray
              ssize = rec_size
              range = rec_range
              nvt = n_elements(node_order)
              vt = make_array(nvt, nt)
              locv = loc_farray
              for i=0L, nt-1 do vt( *, i) = vv(node_order, i)
              vv = temporary(vt)
              two = 0L
              type = 1L
              if lc_global eq 0 then nblo = num_el_blk else nblo = 1
            endif else begin
              range = make_array(2, ndim, nblo+1)
              x1 = reform(coord_node(*,*,0))
              if num_dim gt 1 then $
                x2 = reform(coord_node(*,*,1)) else x2 = time_whole
              ssize = [n_elements(x1(*, 0)),n_elements(x2(*, 0))]
              if ndim eq 3 then begin
;               take the first plane if no zvalue specified
                if n_elements(zvalue) eq 0 then begin
                  zvalue = 1
                  index = 1
                endif else zvalue = zvalue(0)
;               look at initial time
                z = z_array
                nz = n_elements(z)
                if keyword_set(index) then begin
                  zind = round(zvalue-1) > 0
                  zval = z(zind < (nz-1))
                endif else begin
                  if zvalue le z(0) then zval = z(0) else begin
                    if  zvalue(0) ge z(nz-1) then zval = z(nz-1) else begin
                      dist = min(abs(z -zvalue), zind)
                      zval = z(zind)
                    endelse
                  endelse
                endelse
                ssize = [n_elements(x1(*, 0)),n_elements(x2(*, 0)),1L ]
                loctri = make_array(/long, nblo+1)
                for i = 1, nblo do begin
                  varid = ncdf_varid(fcdf_id[cdf_id], 'connect'+strtrim(i, 2))
                  if varid ge 0 then begin
                    ncdf_varget, fcdf_id[cdf_id], varid, connect
                    connect = connect-1
                    ss = size(connect)
                    if ss[0] eq 1 then ss=[2, ss[1], 1, ss[2], ss[3]]
                    jj = 0
                    s2 = ss(2)
                    s1 = ss(1)/2
                    s1m = s1-1
                    del = z_array_err+z_array_err
                    connz = make_array(s1, s2 , /long)
                    for ii = 0L, ss(2)-1 do begin
                      ok = where(abs(coord_node(connect(*, ii), 0, 2) - zval) $
                            lt del, nok)
                      if nok gt 0 then begin
                        if nok ne 4 then begin
                           print, '% READCDF: Bad Z Block', i, ii, nok, ok
                           print, ' Contact lpmix@sandia.gov to add WEDGES'
                           print, '  or call (505) 845-7493'
                           return
                        endif
                        tt = total(ok)
                        if tt lt 22 then begin
                          if ok(0) eq 0 then begin
                            if tt eq 6 then ok = [0, 3, 2, 1] else $
                              if tt eq 10 then  ok =[ 0, 1, 5, 4] else $
                                ok= [0, 4, 7, 3]
                           endif else if tt eq 14 then ok = [1 ,2, 6, 5] else $
                             ok = [2, 3, 7, 6]
                        endif
                        connz (*, jj)= connect(ok, ii)
                        jj = jj+1
                      endif
                    endfor
                    connect = temporary(connz(*, 0:jj-1))
; need an if jj gt 0 command if we have some blocks which are not for all z                    
                    if i eq 1 then begin
                      triangles = make_triangles(connect )
                      range(0,0, i) = min(x1(triangles, 0), max = mm)
                      range(1,0, i) = mm
                      range(0,1, i) = min(x2(triangles, 0), max = mm)
                      range(1,1, i) = mm
                      ss = size(triangles)
                      loctri(1) = ss(2)
                      if nblo gt 1 then triangles = $
                        reform (triangles, ss(4), /overwrite)
                    endif else begin
                      tri= make_triangles(connect )
                      range(0,0, i) = min(x1(tri, 0), max = mm)
                      range(1,0, i) = mm
                      range(0,1, i) = min(x2(tri, 0), max = mm)
                      range(1,1, i) = mm
                      ss = size(tri)
                      loctri(i) = loctri(i-1)+ss(2)
                      triangles= [ triangles, $
                         reform(tri, ss(4), /overwrite)]
                    endelse
                  endif
                endfor

                if nblo gt 1 then triangles = reform(triangles, 3, $
                          loctri(nblo), /overwrite )

                range(0, 0, 0) = min(range(0, 0, 1:nblo))
                range(1, 0, 0) = max(range(1, 0, 1:nblo))
                range(0, 1, 0) = min(range(0, 1, 1:nblo))
                range(1, 1, 0) = max(range(1, 1, 1:nblo))
                nblo = nblo
              endif else begin
                triangles = tri_angles
                loctri = loc_triangles
                locx = make_array(2, 2, /long)
                range = make_array(2, 2, 2)
                range(0, 0, *) = min(x1(ok), max = mm)
                range(1, 0, *) = mm
                range(0, 1, *) = min(x2(ok), max = mm)
                range(1, 1, *) = mm
                nblo = 1
              endelse
              two = 3L
              locx = make_array(2, 2, /long)
              if ndim eq 3 then range(*, 2, *)= zval
              locx(*, 1)  =  num_nodes
              locv = [0, num_nodes]
            endelse
          endelse

; see if the problem is quisi 1D
          scalx = abs(range(1,0,0))+abs(range(0,0,0))
          if scalx eq 0.0 then scalx = 1.0
          scaly = abs(range(1,1,0))+abs(range(0,1,0))
          if scaly eq 0.0 then scaly = 1.0
       if num_dim eq 1 then begin
           ndim = 2
           type = 8L
            two = 3L
            x2lab = 'Time'
            nx2 = (size(x1))[1]
            ssize = [nx2, nt]
            x2 = transpose(time_whole# make_array(nx2,value = 1.))
            x4 = 0.0
            x4lab = ''
            nblo = 1
            range = make_array(2, 2, 2)
            range(0, 0, *) = min(x1, max = mm)
            range(1, 0, *) = mm
            range(0, 1, *) = min(time_whole,max = mm)
            range(1, 1, *) = mm
            temp =0
            if keyword_set(wdfarray) then begin
              if wdfarray lt 0 then begin
                print, '% READCDF:  WDFARRAY is ignored'
                print, '            WDFARAAY must be greater than zero'
              endif else begin
                wdfarray = round (wdfarray) > 1L
                if wdfarray + nt ge get_maxwdf() then $
                  set_maxwdfarray, wdfarray +nt+ 8
                tit= make_array(nt, /string)
                for i = 0, nt-1 do tit[i] = clab+' - Time=' $
                   +strtrim(time_whole[i], 2)
                for i = 0, nt-1 do i2w, x1[*, i], vv[*, i], wdfarray+i, $
                  /xydata, clab = tit[i], xlabel = x1lab, ylab= 'Amplitude', $
                  file = file_name(cdf_id)
              endelse
            endif
            nt = 1
       endif else if num_dim eq 2 then begin
          if (range(1,0,0)-range(0,0,0))/scalx lt error then begin
            type = 8L
            two = 3L
            nx1 = (size(x1))[1]
            x1 = x2
            x2 = transpose(time_whole#  make_array(nx1,value = 1.0))
            ssize = [nx1, nt]
            nblo = 1
            range = make_array(2, 2, 2)
            range(0, 0, *) = min(x1, max = mm) 
            range(1, 0, *) = mm 
            range(0, 1, *) = min(time_whole,max = mm)
            range(1, 1, *) = mm
            temp = 0
            nt = 1
            x4 = 0.0
            x4lab = ''
            x2lab = 'Time'
          endif else if (range(1,1,0)-range(0,1,0))/scaly lt error then begin
            type = 8L
            two = 3L
            x2lab = 'Time'
            nx2 = (size(x2))[1]
            ssize = [nx2, nt]
            x2 = transpose(time_whole# make_array(nx2,value = 1.))
            t = 1
            x4 = 0.0
            x4lab = ''
            nblo = 1
            range = make_array(2, 2, 2)
            range(0, 0, *) = min(x1, max = mm) 
            range(1, 0, *) = mm 
            range(0, 1, *) = min(time_whole,max = mm)
            range(1, 1, *) = mm
            temp = 0
            nt = 1
          endif
       endif
        tlab = 'netCDF Elem Var'
        endelse
        spare = make_array(5, val = 0)
        if n_elements (two) eq 0 then begin
          two = 3L
          print, 'ERROR: TWO undefined'
          print, 'type, two:', type, two
        endif

        wdfa = {type: type, ndim:ndim, nvect: 1L, block:nblo, two:two, $
             ntime:nt,  slice: slice, size: ssize, range: range, $
              order: 0L, locx:locx, locv: locv, triangles: triangles, $
              loctri: loctri, $
              x1:x1, x2:x2, x3:x3, x4:x4, v1:vv, v2:0.0, v3:0.0, v4:0.0, $
              v5:0.0, v6:0.0, x1lab : x1lab, x2lab :x2lab, x3lab : x3lab, $
              x4lab : x4lab, v1lab : 'Amplitude' , v2lab : '', $
              v3lab : '', v4lab : '',v5lab : '', v6lab : '', $
              file : file_name(cdf_id), spare : spare, tlab : tlab, $
              clab:clab +' - ' + file_title(cdf_id)  }
        if keyword_set (structure) then return
        i2s, wdfa, wdfar, /no_copy
        wdfa = wdfar
      endif else if cdf_type(cdf_id) eq 2 then begin
            print, 'This is wdf data'
      endif
      return
      end
