; ------------------------------------------------------------------------ 
; $Id$
; ------------------------------------------------------------------------ 
;
; Copyright 2005 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
 Pro tline, wdf, ain, tvaluein, index = index, sortit = sortit, $
          clabel = clab, xlabel = xlab, ylabel = ylab
;
;+
; NAME:
;	TLINE
;
; PURPOSE:
;	Reduce a four dimensional field structure to three dimensions
;       Must be 4 dimensional, single block, single vector, type 1 data.
;
; CATEGORY:
;	Graphics
;
; CALLING SEQUENCE:
;
;	TLINE, WDF, Structure, tvalue [, /INDEX] [, /SORTIT]
;         [, CLABEL = clabel] [, XLABEL = xlabel] [, YLABEL = ylabel]
;
;
; INPUTS:
;   WDF:      WDF array number to save the lineout
;   Structure: Two dimensional structure of position and time
;   TVALUE:   Time value the ordinate value
;             Tvalue before time begins will be set to initial time
;             Tvalue after the end of data will be set the the final value
;   INDEX:    If present and not zero, then the units of TVALUE will be
;             taken as indices of of the field array.  Valid values are
;             1 to the maximum number of values in the Y-array
;   SORTIT:   If set, Sort the X values
;   CLABEL:   Comment label for the waveform array
;   XLABEL:   X label for the waveform array
;   YLABEL:   Y label for the waveform array
;
; OUTPUTS:
;	Values at constant time will be saved.
;
; EXAMPLE:
;	Store the values at the initial time in wdfarray 9 from structure 3.
;         TLINE, 9, 3, 1, /index
;
; MODIFICATION HISTORY:
; 	Written by:	Paul Mix, September 21, 2004
;-
; insert common blocks
;
@qscom.dat
@com.dat
;
@qsstrcom.dat
;
;       return on an error
      on_error, 1
;
;     check to see if wdf have been initialized
;
      if n_elements(initwdf) eq 0 then wdfinit
      npar = n_params()
      if npar eq 0 then begin
        print, '% Format: TLINE, WDF, Structure, tvalue [, /INDEX]'
        return
      endif
      if npar lt 3 then begin
        print, '% TLINE: WDF, structure and time value must be provided'
        return
      endif
      if n_elements (wdf) eq 0 then begin
        print, '% TLINE: WDF array must be defined'
        return
      endif
      
      if n_elements (ain) eq 0 then begin
        print, '% TLINE: Structure must be defined'
        return
      endif
      if n_elements (tvaluein) eq 0 then begin
        print, '% TLINE: Time value muse be defined'
        return
      endif else tvalue = tvaluein
        
      a = get_valid_structure(ain, 'RoyRect', lcstr= lcstr, ierror = ier)
      if ier ne 0 then return
      if a.type ne 8 or a.ndim ne 2 then begin
        Print, '% TLINE:  Structure must be a two dimensional, type 8 structure.'
        return
      endif

; Pro TLINE, wdf, ain, tvalue, index = index

      x1 = a.x1
      time = reform((a.x2)[0, *])
      v1 = a.v1
      sx = size(x1)
      sv = size(v1)
      if (max(abs(sx-sv)) ne 0) then begin
        print, '% TLINE: X and V must have the same dimension'
        return
      endif
      nt= sx[2]
      nx= sx[1]
      if keyword_set(index) then begin
;       subtract 1 to get to index values
        tvalue = round(tvalue) -1
        tvalue = (tvalue > 0) < (nt-1)
        xx = x1[*, tvalue]
        yy = v1[*, tvalue]
        time = time[tvalue]
      endif else begin
        tvalue = (tvalue > time[0]) < time[nt-1]
        ii = findex(time, tvalue)
        bas = floor(ii)
        incr = ii - bas
        if incr gt 0.9990000 then begin
;       assume exact if less than 1e-3 from a point
          incr =0.0
          bas = bas+1
        endif
        if incr le 1.0e-3 then begin
          time = time[bas]
          xx = x1[*, bas]
          yy = v1[*, bas]
        endif else begin
          time = time[bas] + incr*(time[bas+1]-time[bas])
          xx = x1[*, bas] + incr*(x1[*, bas+1]-x1[*, bas])
          yy = v1[*, bas] + incr*(v1[*, bas+1]-v1[*, bas])
        endelse
      endelse
      if n_elements(clab) eq 0 then $
        clab = 'Time='+strtrim(time, 2)+ ' - ' + a.clab
      if n_elements(xlab) eq 0 then xlab = a.x1lab
      if n_elements(ylab) eq 0 then ylab = a.x2lab
      if keyword_set(sortit) then begin
        i = sort(xx)
        xx = xx[i]
        yy = yy[i]
      endif
      a=0
      i2w, xx, yy, wdf, /xydata, clab = clab, xlab= xlab, ylab= ylab

      end
