; ------------------------------------------------------------------------
; $Id: putcsv.pro,v 1.1 2011/01/07 18:22:39 lpmix Exp $
; ------------------------------------------------------------------------
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
;+
; NAME:
;       PUTCSV
;
; PURPOSE:
;       Put a value into a column of data in a CSV structure
;
; CALLING SEQUENCE:
;
;       PUTCSV, STRUCTURE, DATA, [, COL_VALUE] [, KEY] [, IER = ier] )
;
; INPUTS:
;
;       STRUCTURE: CSV Structure into which which a column value is to be placed
;       DATA:      Value to be put into the structure
;       COL_VALUE: Column number (1 to NCOL) or a string used to search the
;                  column headings.  The first column cannot be changed.
;                  If misssing, a widget will be provided to select the column.
;                  If multiple columns are found, a widget will be used to select a s
;                    single column
;       KEY:       Search string or integer to designate the desired row in the 
;                  spreadsheet.  
;                  If the number of rows in the CVS STRUCTURE is 1, then KEY is ignored.
;                  If KEY is an INTEGER or LONG, IDL will look for equal values.
;                  If KEY is a STRING, GET_MATCH will be used to find matches.  
;                  If the first column is a string, KEY must be a string.
;                  If STRUCTURE has only one row, then KEY may be zero or any value.
;                  If missing or undefined and there is more than one row in STRUCTURE, 
;                  a widget will be provided to select the row.
;
; KEYWORD PARAMETERS:
;
;       IER:       If no error occurs, IER = 0
;
; RESTRICTIONS:
;       If STRUCTURE has more than one row, then KEY must be provided.
;       VALUE must be a scalar integer or long, or VALUE must be a string that 
;       provides a unique column identification.  
;
; EXAMPLE:
;       Set the "X-ray Pulse Width" to 11.3 in the current structure, S1445, which
;       has only a single row.
;         PUTCSV, s1445, 11.3, "X-ray Pulse Width"
;
;       Set the 'Energy Output' to 1.6 in row with key 1658 in structure, S1600.
;         PUTCSV, S1600, 1.6, 'Energy Output', 1658
;
; MODIFICATION HISTORY:
; 	Written by:	L. P. Mix, Jr., June 22, 2009
;	July, 1994    
;
;-

PRO PUTCSV, structure, datain, valuein, keyin, ier = ier

on_error, 1

@csvcommon.dat

ier =1
if n_elements(initcsv) eq 0 then init_csv

if size(structure, /type) ne 8 then begin
  print, '% PUTCSV: CSV structure must be defined.'
  return
endif
tags = tag_names(structure)

ok = get_match('CVS_TYPE', tags, nel = nel)
if nel ne 1 then begin
  print, '% PUTCSV: This structure is not a CVS_STRUCTURE'
  return
endif
if structure.CVS_TYPE ne 200 then begin
  print, '% PUTCSV: Expected CVS_TYPE of 200; value = '+strtrim(structure.cvs_type, 2)
  return
endif

instr = 'Pick the column that you wish returned'
tags = structure.tag_names
ncol = structure.ncol
nrow = structure.nrow

siz_key = size(/type, keyin)
invalid_key = [0, 0, 1, 1, 0]
typecode =[ 2, 3, 4, 5, 7 ]

if ncol gt 1 then begin
  nelv = n_elements(valuein)
  if nelv eq 0 then begin
;   use dircsv to get the units included if they are available
    dircsv, structure, line = line, /quiet, column = column
    pick_one, line, pick, instr=instr, Title = 'GETCSV Column Selection'
    value = column[pick]
  endif else value = valuein[0]
  
  typ_val = size(/type, value)
  ok = where(typ_val eq typecode, nok)
  if (nok eq 0) || (invalid_key[ok[0]]) then begin
    print, '% PUTCSV: VALUE must be integer, long, or string'
    help, value
    return
  endif

  if ok[0] gt 2 then begin
;   use dircsv to get the units included if they are available
    dircsv, structure, value[0], line = line, /quiet, column = column
    if n_elements(line) gt 1 then begin
      pick_one, line, pick, instr=instr, Title = 'PUTCSV Column Selection'
      value = column[pick]
    endif else value = column[0]
  endif else begin
    if (value[0] lt 1) || (value[0] gt ncol) then begin
      print, '% PUTCSV: VALUE must be in the range [1, '+strtrim(ncol, 2) + ']'
      return
    endif
  endelse
  rr = value[0]-1
endif

keys = structure.C0
keystype = size (keys, /type)
data = structure.(rr)
if n_elements(data) eq 1 then begin
   structure.(rr) = datain
   ier =0
   return
endif
if n_elements(keyin) gt 0 then begin
  typ_key = size(/typ, keyin)
  ok = where (typ_key eq typecode, nok)
  if (nok eq 0) || (invalid_key[ok[0]]) then begin
    print, '% PUTCSV: KEY must be integer, long, or string'
    help, key
    return
  endif
  if typ_key lt 7 and keystype eq 7 then begin
    print, '% PUTCSV: KEY - Cannot have an integer key for a string'
    help, keyin
    return
  endif else key = keyin[0]
  if typ_key lt 7 then ok = where (key eq keys, nok) else begin
    vart = (keystype eq 7) ? keys : strtrim(keys, 2)
    ok = get_match(key, vart, nel = nok)
  endelse
  if nok eq 1 then  begin
    data[ok[0]] = datain
    structure.(rr) = data
    ier = 0
    return
  endif

  if nok eq 0 then begin
    print, '% PUTCSV: No values found for key='+string(key)
    return
  endif
endif else ok = lindgen(nrow)
 instr = 'Pick the row that you wish returned'
 if keystype eq 7 then $
   pick_one, keys[ok], pick, instr=instr, Title = 'PUTCSV Row Selection' else $
   pick_one, strtrim(keys[ok], 2), pick, instr=instr, Title = 'GETCSV Row Selection'
 ok = ok[pick]
 data[ok[0]] = datain
 structure.(rr) = data
 ier = 0
END
