; ------------------------------------------------------------------------
; $Id: con2exo.pro,v 1.1 2009/04/02 19:38:01 lpmix Exp $
; ------------------------------------------------------------------------
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
;+
; NAME:
;       CON2EXO
;
; PURPOSE:
;       Convert a series of QS conductor datasets to an exodus file
;       The exodus format will allow visualization with ParaView or Ensight
;
; CATEGORY:
;       Analysis & Visualization
;
; CALLING SEQUENCE:
;       CON2EXO [, OUTFILE] [, INFILE]
;
; OPTIONAL INPUTS:
;       OUTFILE: File name for the output Exodus file
;                The default will be used if OUTFILE is undefined or a number.
;                Default: qsgeo.exo
;                In all cases, if the name exists, the name will be padded with 
;                  a number to avoid eliminating previous files.
;                  For example if qsgeo.exo exists then the outfile is qsgeo.exo
;
;       INFILE:  Data file containing the PFF geometry.
;                If INFILE is undefined the default is qsgeo.pff
;                If this file does not exist, DIALOG_PICKFILE is called.
;                Application Dataset Type = 1 and Type = 5 are always skipped.
;                Type 1 is Bounding Box
;                Type 5 is Grid
;       
; KEYWORD PARAMETERS:
;
;       SKIP:    Strings or numbers specifying datasets to be skipped.
;                Dataset comments containing "VACUUM" are always skipped.
;       NODIELECTRIC:  If set, application dataset type 4 datasets are skipped.
;       DATASETS: Array of one or more numbers or strings that define the datasets that
;                are to be written to OUTFILE from INFILE.
;                NOTE:  GET_MATCH is used to match comments to strings
;                If DATASETS are numbers they must be in the range 1 to number of datasets
;                If DATASETS is set, SKIP and NODIELECTRIC are ignored.
;                Default: All datasets except, bounding box, grid, and vacuum are written.
;       SELECT:  If set, use a widget to select the desired conductors.
;
; PROCEDURE:
;       A PFF file is opened.
;       If DATASETS is set, these snapshots will be written to an exodus file .
;       If DATASETS is not set, and SELECT is set, then a widget will be presented
;         for the user to select the desired datasets. 
;       If DATASETS is not set and SELECT is not set, then all conductor datasets except
;         VACUUM and BOUNDING BOX will be written depending on the value of NODIELECTRIC & SKIP.
;
;       All planes are read and the XYZ points are converted to a unique list.
;       If cylindrical geometry, convert to cartesian coordinates.
;       Each conductor is written as sequential blocks in the file.
;       
; EXAMPLE:
;       Convert all datasets in a qsgeo.pff file an exodus file, qsgeo.exo.
;            CON2EXO
;
;       Convert datasets 3, 4, 5, and 9 in qsgeo.pff to mitl.exo
;            CON2EXO, 'mitl.exo', datasets = [3, 4, 5, 9]
;             
;       Convert selected datasets to qsgeo.exo
;            CON2EXO, /select
;
;       Convert all conductor geometries except vacuum and dielectrics to qsgeo.exo
;         Skip dataset 8 
;            CON2EXO, /nodiel, skip = 8
;
; MODIFICATION HISTORY:
;       Written by:     L. P. Mix, Jr., February 20, 2009
;
;-

 pro CON2EXO, fileoutin, filein, skip = skip, select = select, datasets = datasets, $
          nodielectric = nodiel

  on_error,1
@write_exo_header_com.dat

  fileout = (n_elements (fileoutin) eq 0) ? 'qsgeo.exo' : fileoutin
  if size(/type, fileout) lt 5 then fileout= 'qsgeo.exo'
  fileout = strtrim(fileout, 2)
  if strlen(fileout) lt 2 then fileout= 'qsgeo.exo'
  fileout = find_newfile(fileout, /quiet)

  if n_elements (filein) eq 0 then fname = 'qsgeo.pff' else fname = filein
  if size(fname, /type) eq 7 then begin
    info = file_info(fname)
    if (info.exists + info.read) ne 2b then begin
      print, '% CON2EXO: File does not exist with read permission'
      print, '           File Name = ', fname
      print, '           File Exists =', info.exists
      print, '           Read Permission =', info.read
      fname = ''
    endif
  endif
  openpff, fname, ndset = ndset, error = error
  if error ne 0 then begin
    Print, '% FLD2EXO: Unable to open file.'
    return
  endif
  info = file_info(fname)

  undefine, dsets
  lcdset = 0
  get_dir, comment = comment, number = ndset, /no_trim
  if n_elements(datasets) gt 0 then begin
    if size(datasets, /type) eq 7 then begin
;     get the names of the conductors
      comment = strtrim (comment, 2)
      ndata = n_elements(datasets)
      for i = 0, ndata-1 do begin
        ok = get_match(datasets[i], comment, nel= nel)
        if nel gt 0 then dsets = (n_elements(dsets) eq 0) ? ok : [dsets, ok]
      endfor
      if (n_elements(dset) eq 0) then begin
        print, '% CON2EXO: Unable to match DATASETS to any of the file dataset comments.'
        print, 'Comments: ', comment
        print, 'Datasets: ', datasets
        closepff
        return
      endif
      dset += 1
    endif else begin
      datasets = round(datasets)
      ok = where (datasets gt 0 and datasets le ndset, nok)
      if nok gt 0 then dsets = ok+1 else begin
        print, '% CON2EXO: No valid datasets designated'
        print, ' Datasets must be in the range of 1 to '+ strtrim(ndset, 2)
        print, 'Datasets: ', datasets
        closepff
        return
      endelse
    endelse
    dsets = dsets[sort(dsets)]
    dsets = dsets[uniq(dsets)]
    lcdset = 1
  endif else if keyword_set(select) then begin
    instr= ['Pick the conductors to write to Exodus file', $
            'Signals must have same coordinates and times', $
            'Bounding Box will not be written']
    sel = make_array(/long, value = 0, ndset)
    wselect_item, comment, selected = sel, instruct = instr
    ok = where (sel eq 1, nsig)
    if nsig eq 0 then begin
      print, '% CON2EXO: No signals selected --- RETURNING'
      closepff
      return
    endif
    dsets = ok+1
    lcdset = 1
  endif else dsets = 1+lindgen(ndset)   
  nds = n_elements(dsets)
  ncon = 0L
  for i = 0, nds-1 do begin
    readhdr, dsets[i], /quiet, pfftype = pfftype , aptype = aptype, tlab = tlab, clab = clab
;   take care of bounding box and grids by taking only vertex data
    if (pfftype eq 5) and ((aptype eq 2) or (aptype eq 4)) then begin
;     can only handle conductor and dielectric vertex datasets
      if lcdset eq 1 then  dsets[ncon++] = dsets[i] else begin
        ok = get_match('vacuum', clab, nel = nel)
        if keyword_set (skip) then ok2 = where (dsets[i] eq skip, nok2) else nok2 = 0
        if (nel + nok2) eq 0 then begin
          if aptype eq 2 then dsets[ncon++] = dsets[i] else  $
            if not (keyword_set(nodiel)) then dsets[ncon++] = dsets[i]
        endif
      endelse
    endif
  endfor
  if ncon gt 0 then dset = dsets[0:ncon-1] else begin
    print, '% CON2EXO: No conductors available'
    print, dsets
    closepff
    return
  endelse
  
  loc_planes = make_array(ncon+1, value =0L)
  glob_planes = make_array(3, info.size/6)
  cnames = make_array(33, ncon, /byte )
  for i = 0L, ncon-1 do begin
    readvtx, plane, dset[i], /cond , spare = spare, clab = clab
    cnames[0,i]=byte(strtrim (clab, 2))
    ss = size(plane)
    if (ss[0] ne 2) or (ss[1] ne 3) then begin
      print, '% CON2EXO: Conductor plane is not correct'
      help, plane
      closepff
      return
    endif
    if ss[2] lt 4 then begin
      print, '% CON2EXO: Unable to read dataset: ', dset[i]
      help, plane
      closepff
      return
    endif
    loc_planes[1+i] = loc_planes[i]+ss[2]
;    plane = reform(plane, ss[4], /overwrite)
    
    glob_planes[0, loc_planes[i]] = plane 
  endfor
  closepff
  
  nsides = loc_planes[ncon]
  glob_planes = temporary(glob_planes[*, 0:loc_planes[ncon]-1])
  get_uniq_nodes, glob_planes, newcoord, ind

  if spare[0] eq 2 then begin
    ctheta = reform(cos(newcoord[1, *]))
    stheta = reform(sin(newcoord[1, *]))
    newcoord[1, *] = newcoord[0, *]*stheta
    newcoord[0, *] = newcoord[0, *]*ctheta
    ctheta =0
    stheta =0
  endif

  nplanes = nsides/4
 
  newcoord = transpose(newcoord)
  ss = size(newcoord)
  num_nodes = ss[1]
  loc_elem=loc_planes/4L
  num_elem = loc_elem[1:ncon]-loc_elem[0:ncon-1]
  
  dlconnect = reform (ind, 4, nplanes, /overwrite) +1L
  block_type = make_array(ncon, value = 'SHELL', /string)
  title = 'IDL/ExodusII: '+systime(0)
  cdfid = ncdf_create(fileout, clobber = clobber)
  write_exo_header, cdfid, nblock = ncon, nnode = num_nodes, nelem= num_elem, $
    blo_type = block_type, title = title, numnodevar =0, numelemvar=0
  ncdf_varput, cdfid, coordxid, newcoord[*, 0]
  ncdf_varput, cdfid, coordyid, newcoord[*, 1]
  ncdf_varput, cdfid, coordzid, newcoord[*, 2]
  coord_names = bytarr(33, 3)
  
  coord_names[0:5, 0] = byte('X axis')
  coord_names[0:5, 1] = byte('Y axis')
  coord_names[0:5, 2] = byte('Z axis')
  ncdf_varput, cdfid, ebnameid, cnames
  ncdf_varput, cdfid, coordnamid, coord_names
  for i = 1, ncon do $
    ncdf_varput, cdfid, connectid[i-1], dlconnect[*, loc_elem[i-1]:loc_elem[i]-1]

  ncdf_close, cdfid
  print, 'Following conductors written: ' 
  for i = 0, ncon-1 do print, string (i+1), ' - ', strtrim (cnames[*, i], 2)
  end
