; ------------------------------------------------------------------------
; $Id: get_elem.pro,v 1.2 2007/06/06 19:40:41 lpmix Exp $
; ------------------------------------------------------------------------
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
;+
; NAME:
;       GET_ELEM
;
; PURPOSE:
;       This function returns the element number(s) for elements closest to a point.
;
; CALLING SEQUENCE:
;
;       ELEM = GET_ELEM(POSITION [, TIME])
;
; OUTPUT:
;       ELEM:   Element or cell numbers from the file.
;               ELEM equal -1 if no element information is available.
;               Nodes are numbered from 1 to NUM_ELEM.
;
; INPUTS:
;       POSITION: Spatial position where closest element is requested.
;                 Element position is the average position of the nodes that
;                 form the element.
;       TIME:     Optional input to specify a time other than the first time.
;                 The position at initial time is stored in a common block 
;                 and is much faster to access.  If INDEX is set, TIME is the 
;                 time step which must be in the range 1 to N_TIMES.
;
; KEYWORD PARAMETERS:
;       INDEX:  If set, TIME is the time index from 1 to N_TIMES.
;       VALUES: Actual spatial position of the element(s) returned.
;               Values = [Num_elements, ndim]
;       NELEM:  Number of elements returned.
;               Default is 1 - Closest Node to Position.
;       RADIUS:	If present and not zero, this specifies a radius for finding
;               elements.  All elements with a position less than RADIUS from 
;               POSITION are returned.
;       TETS:   If present, tets associated with each element are returned.
;               Size is [4, num_elements]  If some elements are not tets, this
;               request will return -1.
;       The following are ignored if RADIUS is set.
;       XDelta: Range of X-values to use in returning elements.
;               Values returned have X such that:
;               POSITION[0]-0.5*Xdelta LE X LE POSITION[0]+0.5*Xdelta
;       YDelta: Range of Y-values to use in returning elements.
;               Values returned have Y such that:
;               POSITION[0]-0.5*Ydelta LE Y LE POSITION[0]+0.5*Ydelta
;       ZDelta: Range of Z-values to use in returning elements.
;               Values returned have Z such that:
;               POSITION[0]-0.5*Zdelta LE Z LE POSITION[0]+0.5*Zdelta
;
; RESTRICTIONS:
;       Current Exodus II file is searched.  Use SETEXO to set desired file.
;
; EXAMPLE:
;       Find the element number and true position (val) of the element closest to 
;       the location: [.1, .2, .1].  Save the element in NN.
;         nn = GET_ELEM( [.1, .2, .1], value = val )
;
;       Find the element number and true position (val) of the element closest to 
;       the location: [.1, .2, .1] at a time of 100e-9.  Save the element in N100.
;         n100 = GET_ELEM( [.1, .2, .1], 100.0e-9, value = val )
;       Note:  No interpolation is done.  Time closest to 100e-9 us used.
;
;       Find the element numbers and true position (val) of the element within 5 cm of 
;       the location: [.1, .2, .1] at a time of 100e-9.  Save the element in N05.
;         n05 = GET_ELEM( [.1, .2, .1], 100.0e-9 , r= 0.05, val = val)
;
;       Find the element numbers and true position (val) of the element within 5 cm of 
;       the location: x = 0.1 at a the initial time.  Store the numbers in xp1
;         xp1 = GET_ELEM( [.1, 0, 0], x= 0.05, val= val)
;
; MODIFICATION HISTORY:
; 	Written by:	L. P. Mix, October 6, 2005
;
;-

  Function GET_ELEM, position, tvalue, values = values, nelem= nelem, $
              radius = radius, Xdelta = Xdelta, ydelta = ydelta, $
              zdelta = zdelta, index = index

  on_error, 1
; add common block
@exoduscom.dat
;
; check to see if EXODUS has been initialized
;
  elements = -1
  nelem = 0
  if n_elements(init_exo) eq 0 then begin
    print, '% GET_ELEM: No Exodus files have been opened.'
    return, elements 
  endif
 if exo_id lt 0 then begin
    print, '% GET_ELEM: No Exodus files are open.'
    return, nodes
  endif

  if n_params() lt 1 then begin
    print, '% Format: Nodes = GET_ELEM( POSITION [, TVALUE] [, /INDEX] '
    print,'       [, VALUES = values] [, NELEM = nelem] [, RADIUS = radius]'
    print,'       [, XDELTA = xdelta] [, YDELTA = ydelta] [, ZDELTA = zdelta])'
    return, elements
  endif 

  if n_elements(tvalue) eq 0 then coord = coord_node else $
    coord= read_exo(/grid, tvalue, index= index)
  if n_elements(coord) lt num_nodes then begin
    Print, '% GET_ELEM: Unable to obtain nodes.'
    return, elements
  endif
   
  ss = size(coord)
  ndim = num_dim[exo_id]
  if (ss[0] ne 2) || (ss[1] ne num_nodes) || (ss[2] ne ndim) then begin
    print, '% GET_ELEM: Coordinates are the wrong size'
    help, coord, num_nodes, ndim
    return, elements
  endif
; 
; Calculate elements centers.
;
  Get_element_centers, coord, element_centers
  coord = temporary(element_centers)
;

  if n_elements(position) ne NDIM then begin
    print, '% GET_ELEM: Position must have ', strtrim(ndim, 2), ' elements'
    help, position
    return, elements
  endif
  distance = make_array(num_elem, ndim)
  distance[*, 0] = (coord[*, 0] - position[0])^2
  distance[*, 1] = (coord[*, 1] - position[1])^2
  if ndim gt 2 then $
    distance[*, 2] = (coord[*, 2] - position[2])^2
  
  if n_elements(radius) gt 0 then begin
    min_d  = radius[0]^2
    distan = total(distance, 2)
    elements = where(distan le min_d, nelem)
    if nelem eq 0 then begin
      print, '% GET_ELEM: No elements within the required distance'
      print, '             Closest node returned.'
      junk = min(distan, elements)
      nelem = 1
    endif
    values =reform(coord[elements, *])
    return, elements+1
  endif  
  nok = 0
  lcmin = 0
  lcx = [0, 0, 0]
  if n_elements(xdelta) gt 0 then begin
    lcx[0] = 1
    ok = where(distance[*, 0] lt xdelta[0]^2, nok)
    if nok eq 0 then lcmin = 1
  endif else ok = lindgen(num_elem)
  if (lcmin eq 0) and (n_elements(ydelta) gt 0) then begin
    lcx[1] = 1
    oky = where(distance[ok, 1] lt ydelta[0]^2, nok)
    if nok eq 0 then lcmin = 1 else ok = ok[oky]
  endif
  if (lcmin eq 0) and (n_elements(zdelta) gt 0) then begin
    lcx[2] = 1
    okz = where(distance[ok, 2] lt zdelta[0]^2, nok)
    if nok eq 0 then lcmin = 1 else ok = ok[okz]
  endif
  if (lcmin eq 1) || (total(lcx) eq 0) then begin
    junk = min(total(distance, 2), elements)
    nelem = 1
    values = reform(coord[elements, *])
    return, elements+1
  endif
  values = reform(coord[ok, *])
  nelem = nok
  return, elements+1
  end
