; ------------------------------------------------------------------------
; $Id: readexo.pro,v 1.9 2009/11/05 20:29:32 lpmix Exp $
; ------------------------------------------------------------------------
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
PRO  readexo, str, dataset, tvalue, index = index, ierror = ierror, twodim = twodim, $
          time_index = tindex, grid = grid, clabel = clabelin, onedim = oned, $
          cell_values = cell_values,  rectilinear = rectilinear, rz_one = rzone, $
          epsilon = epsilon, structure_only = structure_only, vector = vector
;
;+
;  Name: READEXO
;
;  Purpose:
;        Reads a Exodus dataset into a PFIDL structure or WDF array.
;      Note: Cell or element data can be left as element or cell data.
;            Default is it convert element data to node data.
;
;  Format:
;
;    READEXO, STRUCTURE, [ DATASET,] TVALUE [, /INDEX] [, IERROR = ierror]
;                     [, TIME_INDEX = time_index] [, /GRID] [, CLABEL = clab]
;                     [, ONEDIM = onedim] [, /CELL_VALUES] [, RZ_ONE = rz_one]
;                     [, RECTILINEAR = rectilinear] [, /STRUCTURE_ONLY]
;                     [, EPSILON = epsilon] [, VECTOR = vector]
;
;  where:
;    STRUCTURE - Structure array number for structure data.
;                WDF array number for pseudo-1D data if ONEDIM is set.
;    NOTE: Dataset must be provided unless grid is set.
;   
;    DATASET   - indicates a Exodus dataset name or a search string for dataset
;                names or a Exodus dataset number.  Only node and cell data may be read.
;                DATASET numbers are number shown in DIREXO.
;                DATASET must be omitted if GRID is set.
;                  Format: READEXO, TVALUE, /GRID, ....
;                Use REEXO to read data at one node for all times.
;                If GRID is set, must contain the TVALUE.
;           See: GET_DSET (in PFIDL_1D) for more details of the search.
;                IF DATASET is an array or VECTOR is set, the square root of the sum
;                of the squares will be stored.
;    TVALUE    - Time for the data.  If INDEX is set this is an index number.
;                Valid index numbers are 1 to number of time values.
;                The closest time is returned; no interpolation is performed.
;                If GRID is set and DISPLX, DISPLY, DISPLZ are missing, then
;                TVALUE is ignored.
;          Note: To obtain the node/element value vs time at one node, use REEXO
;    INDEX     - If INDEX is set then TVALUE goes from 1 to the maximum number of
;                times.
;    IERROR    - Returned 0 if no error and valid data
;    TIME_INDEX- This is returned with the time value index from 1 to the number
;                Times.  The actual time is given by TIME = (GET_TIME())[Time_index -1]
;    GRID      - If set, a node array will be returned.  If DISPLX, DISPLY, DISPLZ
;                are missing, the coordinates are for the first time step.
;    CLABEL    - Comment label for the data.  If missing the comment label from 
;                READ_EXO will be used.  The default includes the time and filename.
;                Format: "DATASET, t=time - Filename=filename"
;    ONEDIM    - If set and the data is two-dimensional cell data, the data will be 
;                put into a wdf array and stored in wdfarray, structure.  
;                Node data is averaged over top and bottom values.
;                Element data is put at the cell center.
;          Note: ONEDIM is ignored if number of dimensions is gt 2.
;    TWODIM    - If set, save data as two dimensional if possible.
;                If a dataset is degenerate in one dimension, then the default is
;                  TWODIM = 1
;                The data will be stored in a type 8 structconnecture with the two valid 
;                dimensions in X and Y.
;    CELL_VALUES - If set, element or cell data will be returned with the cell
;                centers.  Default - Move cell data to the nodes for finite
;                element data and leave at the cells for rectilinear data; ie 
;                Default value: Cell_Values = (RZ_One || Rectilinear)
;    RZ_One    - If set, assume a one dimensional one cylindrical dataset.
;                If RZ_ONE = 1 store the data as R vs Z
;                The radius will be averaged between the top and bottom of each cell
;                Data is assumed to lie in planes in the R-Z dimension if the number of 
;                  node values is equal to the number of Z values times the number of 
;                  X values.  For NODE data, the top and bottom nodes are averaged.
;                If the number of node values is less than this product, then the data 
;                  is stored as a finite element mesh.
;                No action is taken for grid data
;    RECTILINEAR-  NOT CURRENTLY IMPLEMENTED
;                If set, attempt to put the data on a rectilinear grid.
;                Multiple block data will be read as multiple block data
;                If the number of dimensions is 3 then the user should 
;                specify RECTILINEAR =1
;                Default is RECTILINEAR = 0
;                If 3d data has only 1 cell then will be read as 2d degenerate.
;    Note: Values are assumed equal if they differ by less than EPSILON
;          Only the initial grid is checked.  A Lagrangian problem which starts
;          with a rectilinear grid can be read as rectilinear data; the grid
;          will be the initial grid for all times.
;    Note: A less sensitive of delta_X /xrange < delta may be used if 
;          RECTILINEAR = DELTA 
;    DOUBLE_PREC - If set, save data as double precision
;    EPSILON   - Default error parameter = 1./16384 * datarange
;    STRUCTURE_ONLY- If set, a structure will be returned in STRUCTURE
;    VECTOR    - If set, multiple dataset values will be expected.  The square 
;                root of the sum of the squares will be stored.
;
;  Examples:
;    Read dataset 34 into array 33, at time step 5.
;      READEXO, 33, 34, 5, /ind
;
;    Read dataset with 'KE' into structure array 4, at time = 1.36 ns.
;      READEXO, 4, 'KE', 1.36e-9
;
;    Read the magnitude of the magnetic field into structure 5 at time step 5.
;      READEXO, 5, '^B_', 5, /index, /vector
;
;  MODIFICATION HISTORY:
;       Written by:     L. P. Mix, November 4, 2005
;       March 14, 2006  Added keyword VECTOR (lpm)
;       November 5, 2009 Added keyword TWODIM
;
;-
      on_error, 1
;     add common block
@exoduscom.dat
;
;     check to see if EXODUS has been initialized
;
      ierror = 1
      if n_elements(init_exo) eq 0 then begin
        print, '% READ_EXO: No Exodus files are open.'
        return
      endif
;       
      nparam = n_params()
      if nparam eq 0 then begin
 print,'% Format: READEXO, STRUCT, DATASET, TVALUE [, /INDEX] [, /GRID]'
 print,'       [, TIME_INDEX= time_index] [, CLABEL = clabel] [, /TWODIM]'
 print,'       [, /CELL_VALUES] [, RECTILINEAR = rectilinear]'
        return
      endif

      if exo_id  lt 0 then begin
        print, '% READEXO: Exodus file must be opened'
        return
      endif
      if keyword_set(structure_only) then begin
        if not arg_present(str) then begin
          print, '% READ_EXO: Structure must be valid variable'
          return
        endif
      endif else begin
        if arg_present(str) then begin
          if keyword_set(oned) then str = get_save_array(str, get_maxwdf()) $
            else str = get_save_str(str, get_maxstr())
        endif else begin
          if keyword_set(oned) and ((str lt 1)  or (str gt get_maxwdf())) then begin
  print, '% READEXO: WDF array must be greater than 0 and </=', get_maxwdf()
            return
          endif else if str lt 1 or str gt get_maxstr() then begin
  print, '% READEXO: Structure array must be greater than 0 and </=', get_maxstr()
            return
          endif
        endelse
      endelse
 
      if n_elements(epsilon) eq 0 then epsilon = 1.0/16384
;
;     exodus file id
      fid = fexo_id(exo_id)
;
      if lexo_valid ne array_exoid(exo_id, 1) then load_exo_data
      if exo_type(exo_id) eq 1 then begin
        nblo = num_elem_blk[exo_id]
        ntime = n_elements(time_whole)
        ndim = num_dim[exo_id]
        two = (ndim eq 3) ? 0 : 3
        if keyword_set(grid) then begin
          if lc_displace eq 1 and n_elements(dataset) eq 0 then begin
             print, '% READEXO: TVALUE must be provided to calculate node offset.'
             wok_notok, lcquit, $
               ['Time Value Must be Provided', 'Do you wish to select a time value?'], $
               'Time missing', $
               menu=['Continue - Select Time', 'QUIT - ReEnter Command']
             if lcquit gt 0 then return
             t = get_exotimes()
             wselect_item, string(t), sel = sel, /ex
             tvalue = sel+1
             index = 1
             datasett = tvalue
          endif else datasett= dataset

          nodes = read_exo(datasett, tvalue, index = index, ierror = ier, $
             time_index = tindex, clabel = clabel, /grid)
          if ier ne 0 then begin
            print, '% READEXO: Unable to read grid values'
            return
          endif
          tlab = 'Exodus Grid Data'
          if n_elements(clabelin) gt 0 then clab = clabelin else clab= 'Grid - '+clabel
          ss = size(nodes)
          block =  num_elem_blk[exo_id]
          locv = loc_v
          loctri = 0L
;         Tindex is the time index which goes from 1 to ntime.
          x4 = time_whole[tindex-1]
          x4lab = 'Time'
          spare = make_array(5, val = 0) 
          type = 10L
          x1 = nodes[*, 0]
          x2 = nodes[*, 1]
          x3 = (ndim ge 3) ? nodes[*, 2] : 0.0
          connect = get_exo_connect(nodes, locx, range, loctri, /grid)

          ssize = make_array(ndim)
          ssize[*]= ss[1]
          tempstr = {type: 10L, ndim:ndim, block:block, two:two, $ 
               ntime:1,  slice: 0.0, size:ssize, range: range, $
                order: 0L, locx:locx, locv: locv, $
                x1:x1, x2:x2, x3:x3, x4:x4, connect:connect, $ 
                x1lab : x1_label, x2lab :x2_label, x3lab : x3_label, $
                x4lab : x4lab, file : file_name(exo_id), spare : spare, $
                tlab : tlab, clab:clab }
           if keyword_set(structure_only) then str = temporary(tempstr) $
             else i2s, tempstr, str, /no_copy
          return
        endif
        nmax = num_elem_var(exo_id) + num_nod_var(exo_id) 
        if nmax eq 0 then begin
          print, '% READEXO: No node or element arrays in this Exodus file'
          return
        endif
;       get the quantity value
        ndset = n_elements (dataset)
        if ndset eq 0 then begin
          print, '% READEXO: Dataset must be defined'
          return
        endif
        rval = 0.0
        cl = make_array(ndset, /string)
        if n_elements (index) gt 0 then indx = index[0]
        for ii = 0, ndset-1 do begin
          rvalue = read_exo(dataset[ii], tvalue, index= indx, ierror = ierror, $
            time_index = tindex, clabel = clabelexo, vector = vector)
          if ierror ne 0 then begin
            print, '% READEXO: Unable to read dataset:', dataset[ii]
            return
          endif
          cl[ii] = clabelexo
          if ndset gt 1 then begin
            if n_elements(tvalue) eq 0 then begin
              tvalue = tindex 
              indx = 1
            endif
            rval = rval+rvalue*rvalue
          endif
        endfor
        vlabel = 'Amplitude'
        if ndset gt 1 then begin
          rvalue = sqrt(rval)
          if n_elements(clabel) eq 0 then begin
            clabel = 'Magnitude of '
            for i =0, ndset-1 do begin
              if i gt 0 then clabel = clabel + '+'
              st = strpos(cl[i], ', ')
              tmp = (st gt 0) ? strtrim(strmid(cl[i], 0, st), 2) : strtrim(cl[i], 2)
              clabel = clabel+tmp
            endfor
            clabel = clabel+', T='+strtrim(get_exotimes(tindex), 2)
          endif
        endif else if n_elements(clabel) eq 0 then clabel = clabelexo
;
        nodes =  read_exo(tindex, /index, /grid, ierror = ierror)
        if ierror ne 0 then begin
          print, '% READEXO: Unable to read Nodes'
          return
        endif
        if n_elements (rvalue) eq num_nodes then lcnodes = 1 else lcnodes = 0
        if keyword_set(oned) && (ndim eq 2) then begin
          get_element_centers, nodes, centers
          xx = centers[*, 0]
          yy = centers[*, 1]
;         check y first
          yrange = [min(nodes[*, 1], max = m), m]
          yerror = (yrange[1]-yrange[0])*epsilon
          xrange = [min(nodes[*, 0], max = m), m]
          xerror = (xrange[1]-xrange[0])*epsilon
          if (max(yy, min= m) -m) lt yerror then begin
;           All Y values are equal
            if n_elements(rvalue) eq  num_elem then begin
;             Cell or element data            
;             only an X variation
              sx = sort(xx)
              rvalue = rvalue[sx]
              xvalue = xx[sx]
            endif else begin
;             Node data
              sx = sort(xx)
              rvalue = rvalue[sx]
              xvalue = xx[sx]
              rvalue = reform(rvalue, 2, num_nodes/2)
              xvalue = reform(xvalue, 2, num_nodes/2)
              rvalue = total(rvalue, 1)*0.5
              xvalue = total(xvalue, 1)*0.5
            endelse
            xlabel = x1_label
          endif else if (max(xx, min= m) -m) lt xerror then begin
;           All X values are equal
            if n_elements(rvalue) eq  num_elem then begin
;             Cell or element data            
;             only an X variation
              sx = sort(yy)
              rvalue = rvalue[sx]
              xvalue = yy[sx]
            endif else begin
;             Node data
              sx = sort(yy)
              rvalue = rvalue[sx]
              xvalue = yy[sx]
              rvalue = reform(rvalue, 2, num_nodes/2)
              xvalue = reform(xvalue, 2, num_nodes/2)
              rvalue = total(rvalue, 1)*0.5
              xvalue = total(xvalue, 1)*0.5
            endelse
            xlabel = x2_label
          endif else begin
            print, '% READEXO: This is not One Dimensional data.'
            return
          endelse
          i2w, xvalue, rvalue, /xydata, wdf, clabel = clabel, xlabel = xlabel, $
            ylabel = vlabel, file= file_name[exo_id]
          ierror = 0
          return
        endif
;       We could worry about rectilinear, but not now.
;       set a few things
;
        tlab = (lcnodes) ? 'Exodus Node Var' : 'Exodus Elem Var'
        if  keyword_set(rzone) then begin
          if (lcnodes) eq 0 then begin
;            CELL data
             get_element_centers, nodes, elemvalue
             nodes = temporary(elemvalue)
             rr = sqrt(nodes[*, 0]^2 + nodes[*, 1]^2)  
             zz = nodes[*, 2]
             a = sort(zz)
             zzs = zz[a]
             zorder = uniq(zzs)
             zval = zzs[zorder]
             rr = rr[a]
             rorder  = uniq(rr, sort(rr))
             nr =n_elements(rorder)
             rvalue = rvalue[a]
             endval = zorder[0]
             offset = endval+1
             nz = n_elements (zval)
             if nr *nz gt n_elements(rvalue) then begin
               print, $
               '% READEXO:  Number of R and Z values exceeds the number of cell values'
               print,'            A FE structure will be saved'
               lc_fe =1 
             endif
             val = make_array(offset, nz, type = size(/type, rvalue))
           
             if n_elements(rvalue) ge nz*offset then for i = 0L, nz-1 do begin
               ra = rr[i*offset:i*offset+endval]
               ind = sort(ra)
               if i eq 0 then R= ra [ind]
               val[*, i]=(rvalue[i*offset:i*offset+endval])[ind]
             endfor
          endif else begin
;           NODE data
             rr = sqrt(nodes[*, 0]^2 + nodes[*, 1]^2)
             zz = nodes[*, 2]
             a = sort(zz)
             zzs = zz[a]
             zorder = uniq(zzs)
             zval = zzs[zorder]
             rr = rr[a]
             xorder = uniq(nodes[*, 0], sort(nodes[*, 0]))
             nx =n_elements(xorder)
             rvalue = rvalue[a]
             endval = zorder[0]
             offset = endval+1
             nz = n_elements (zval)
             if nx *nz gt n_elements(rvalue) then begin
              print, $
              '% READEXO:  Number of X and Z values exceeds the number of cel values'
               print,'            A FE structure will be saved'
               lc_fe =1
             endif
             offset2 = offset/2
             val = make_array(offset2, nz, type = size(/type, rvalue))
             ind1 = 2*lindgen(offset2)
             ind2 = ind1+1
             if n_elements(rvalue) ge nz*offset then for i = 0L, nz-1 do begin
               ra = rr[i*offset:i*offset+endval]
               ind = sort(ra)
                
               if i eq 0 then R= ra[ind[ind1]]+ra[ind[ind2]]
;              average values
               vv = rvalue[i*offset:i*offset+endval]
               val[*, i]= vv[ind[ind1]]+vv[ind[ind2]]
             endfor
             val = val*0.5d0
             r = r*0.5d0 

          endelse
            
          arr2str, str, val, r, zval, clab = clabelexo, tlab = tlab, $
            xlab = 'Radius', ylab = 'Z-axis', $
            file=file_name(exo_id), structure_only = structure_only

          if keyword_set(lc_fe) then arr2str, get_maxstr(), rvalue, rr, zzs, $
            clab = clabelexo, tlab = tlab, xlab = 'Radius', ylab = 'Z-axis', $
            /femesh, file=file_name(exo_id)
          return
        endif
        tri = 0
        loctri = 0
        block = 1L
        if (lcnodes eq 0) && (keyword_set(cell_values)) then begin
;       Element data
          get_element_centers, nodes, elemvalue
          tri = 0
          loctri = 0
          block = 1L
          nodes = temporary(elemvalue)
          locx = make_array(ndim, 2, /long)
          locx(*, 1)= num_elem
          range = make_array(2, ndim, 2)
          for i = 0, ndim-1 do begin
            range[0, i, *] = min(nodes[*, i], max = mm)
            range[1, i, *] = mm
          endfor
          if n_elements(elem_connect) gt 4 then connect = elem_connect $
            else begin
            if ndim eq 2 then begin
              triangulate, nodes[*, 0], nodes[*, 1], connect 
            endif else begin
              qhull, nodes[*, 0], nodes[*, 1], nodes[*, 2], connect, /delaunay 
            endelse
          endelse
          ss = size(connect)
          if ss[0] eq 1 then ss=[2, ss[1], 1, ss[2], ss[3]]

          loctri = [0l, ss[2]]
          ss = size(nodes)
        endif else begin
          if lcnodes eq 0 then begin
             elem2node, rvalue, nodevalue
           rvalue = temporary(nodevalue)
          endif
          connect = get_exo_connect(nodes, locx, range, loctri)
          ss = size(nodes)
        endelse

        x4 = time_whole[tindex-1]
        ss = size (nodes)
        ssize = make_array(ndim, value = ss[1])
        locv= [0L, ss[1]]        
        v1lab = 'Amplitude'
        if ndim eq 2 then begin
;         type 8 structure
          two = 3L
          sst = size(connect)
          if sst[0] eq 1 then sst=[2, sst[1], 1, sst[2], sst[3]]
          slice = 0.0
          ssb = size(range)
          nb = ssb[3]-1
          if nb gt 1 then begin
            tmp = range[*, *, 0]
            range = make_array(2, 2, 2)
            for i = 0, 1 do range[*, *, i] = tmp
          endif
          if sst[0] eq 2 then loctri=[0, sst[2]]
            tempstr = {type: 8L, ndim:2L, nvect: 1L, block:1L, two:3L, $
            ntime:1L,  slice:slice, size: ssize, range: range, order: 0L, $
            locx:locx, locv:locv, triangles:connect, loctri:loctri, $
            x1:nodes[*, 0], x2:nodes[*, 1], x3:0.0, x4:x4, $
            v1:rvalue, v2:0.0, v3:0.0, v4:0.0, $
            v5:0.0, v6:0.0, x1lab : x1_label, x2lab : x2_label, x3lab : '', $
            x4lab : '', v1lab : v1lab , v2lab : '', v3lab : '', v4lab : '', $
            v5lab : '', v6lab : '', file : file_name[exo_id], $
            spare:[0,0,0,0,0], tlab:tlab, clab:clabel}
          if keyword_set(structure_only) then  str= temporary(tempstr) $
             else i2s, tempstr, str, /no_copy
        endif else begin
;         type 12 structure
;
          if n_elements(twodim) eq 0 then twodim = 1
          slice = 0.0
          if range[1, 0, 0] - range[0,0,0] eq 0 then begin
            two = 1
            slice = range[0,0,0]
          endif else if range[1, 1, 0] - range[0,1,0] eq 0 then begin
            two = 2
            slice = range[0,1,0]
          endif else if range[1, 2, 0] - range[0,2,0] eq 0 then begin
            two = 3
            slice = range[0,2,0]
          endif
          if two ne 0 and keyword_set(twodim) then begin
;           type 8 structure
            tmp = make_array(2, 2)
            two = long(two)
            if two eq 1 then begin
              x3 = nodes[0, 0]
              x1 = nodes[*, 1]
              x2 = nodes[*, 2]
              addit = 'X-Slice='
              tmp = range[*, 1:2, 0]
              x1lab = x2_label
              x2lab = x3_label
              x3lab = x1_label
              ssize = ssize[1:2]
            endif else if two eq 2 then begin
              x3 = nodes[0, 1]
              x1 = nodes[*, 0]
              x2 = nodes[*, 2]
              addit = 'Y-Slice='
              tmp[*, 0] = range[*,0 , 0]
              tmp[*, 1] = range[*,2 , 0]
              x1lab = x1_label
              x2lab = x3_label
              x3lab = x2_label
              ssize = ssize[[0, 2]]
            endif else begin
              x3 = nodes[0, 2]
              x1 = nodes[*, 0]
              x2 = nodes[*, 1]
              addit = 'Z-Slice='
              tmp = range[*,0:1 , 0]
              x1lab = x1_label
              x2lab = x2_label
              x3lab = x3_label
              ssize = ssize[0:1]
            endelse
            range = make_array(2,2,2)
            for i = 0, 1 do range[*, *, i] = tmp
            tri = make_triangles(connect)
            loctri= [0, n_elements(tri)]
            clabel = clabel+' - '+addit+strtrim(x3, 2)
            tempstr = {type: 8L, ndim:2L, nvect: 1L, block:1L, two:two, $
              ntime:1L,  slice:slice, size: ssize, range: range, order: 0L, $
              locx:locx, locv:locv, triangles:tri, loctri:loctri, $
              x1:x1, x2:x2, x3:x3, x4:x4, $
              v1:rvalue, v2:0.0, v3:0.0, v4:0.0, $
              v5:0.0, v6:0.0, x1lab : x1lab, x2lab : x2lab, x3lab : x3lab, $
              x4lab : '', v1lab : v1lab , v2lab : '', v3lab : '', v4lab : '', $
              v5lab : '', v6lab : '', file : file_name[exo_id], $
              spare:[0,0,0,0,0], tlab:tlab, clab:clabel}
            if keyword_set(structure_only) then  str= temporary(tempstr) $
               else i2s, tempstr, str, /no_copy
            return
          endif
;
;         this structure definition may need to be changed to "block:nblo"
;
          tempstr = {type: 12L, ndim:ndim, block:1L, two:two, $
              ntime:1,  slice: 0.0, size:ssize, range: range, nvect: 1L, $
              order: 0L, locx:locx, locv: locv, triangles: connect, loctri: loctri, $
              x1:nodes[*, 0], x2:nodes[*, 1], x3:nodes[*, 2], x4:x4, $
              v1:rvalue, v2:0.0, v3:0.0, v4:0.0, v5:0.0, v6:0.0, $
              v1lab : v1lab , v2lab : '', v3lab : '', v4lab : '', $
              v5lab : '', v6lab : '', x1lab : x1_label, $
              x2lab :x2_label, x3lab : x3_label, $
              x4lab : '', file : file_name(exo_id), spare:[0,0,0,0,0], $
              tlab : tlab, clab:clabel }
          if keyword_set(structure_only) then  str= temporary(tempstr) $
             else i2s, tempstr, str, /no_copy
        endelse
      endif else if exo_type(exo_id) eq 2 then $
        print, '% READEXO: This is wdf data - use REEXO'
        
      end
