; ------------------------------------------------------------------------ 
; $Id: reexo.pro,v 1.2 2007/06/06 19:40:42 lpmix Exp $
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
PRO  reexo, wdf, dataset, ierror = ierror, node = node, alldata = alldata, $
          number = number, narray = narray, position = position
;+
;  Name: REEXO
;
;  Purpose:
;        Reads a Exodus dataset into a WDF array.  
;        This reads global data vs time or the value at a node or cell vs time.
;
;  Format: 
;
;    REEXO, WDFARRAY, DATASET [, IERROR = ierror] [, NODE = node] [, /ALLDATA]
;              [, NUMBER = number] [, POSITION = position] [, NARRAY = narray]
;
;  where:
;    WDFARRAY  - WDFarray number for the data
;    DATASET   - Dataset number or dataset name 
;                If NODE is not specified, this is a global dataset name or number
;                If NODE is specified, this is a node or element (cell) dataset 
;                  name or number.
;                DATASET numbers are number shown in DIREXO
;                If DATASET is a search string equal to '' then all
;                datasets are read.
;           See: GET_DSET (in PFIDL_1D) for more details of the search.
;    IERROR    - Returned 0 if no error and valid data
;    NODE      - This is the node or element (cell) number desired.  Values range
;                from 1 to the total number or may be the coordinates of the node 
;                or cell center.  When reading global variables this is omitted.
;    ALLDATA   - If set, all specified data is read.
;                This is only checked with a finite length search string.
;    NARRAY    - Number of datasets returned.
;
;    NOTE:  The following apply only when NODE is specified.
;    NUMBER    - Node number or Element number of the returned data.
;    POSITION  - Actual Coordinates of the returned data.
;
;  Examples:
;    Read global dataset 34 into array 33
;      REEXO, 33, 34
;
;    Read all dataset with 'KE' into WDF array 4 and following.
;      REEXO, 4, 'KE', /ALLDATA
;
;    Read DISPLX data from node 967 into wdf array 7
;      REEXO, 7, 'DISPLX', node = 967
;
;    Read DISPLX data from node at [.1, 2] into wdf array 47
;      REEXO, 47, 'DISPLX', node = [.1, 2]
;
;  MODIFICATION HISTORY:
;       Written by:     L. P. Mix, January 19,  2005
;
;-
      on_error, 1
;     add common block
@com.dat
@exoduscom.dat
;     see if we are initialized
      if n_elements(initwdf) eq 0 then wdfinit

      ierror = 1
      nparam = n_params()
      if nparam eq 0 then begin
 print,'% Format: REEXO, WDF, DATASET, [, NARRAY = narray]'
 print,'            [, IERROR = ierror] [, NODE = node] [, /ALLDATA]'
 print,'            [, NUMBER = number] [, POSITION = position]'
;           
        return
      endif
;
;     check to see if EXODUS has been initialized
;
      if n_elements(init_exo) eq 0 then begin
        print, '% REEXO: No Exodus files are open.'
        return
      endif
;       
      if exo_id  lt 0 then begin
        print, '% REEXO: Exodus file must be opened'
        return
      endif
      if n_elements(wdf) eq 0 then begin
        print, '% REEXO: WDF array number must be provided'
        return
      endif
      if n_elements(dataset) eq 0 then begin
        print, '% REEXO: DATASET name or number must be provided'
        return
      endif
      wdfar = wdf
      if min(wdfar, max = m) lt 1 or m gt maxwdfarray then begin
        print, '% REEXO: Valid WDF array number must be provided'
        print, '         Values must be in the range [ 1, ', $
          strtrim(maxwdfarray, 2), ']'
        return
      endif
      nwdf = n_elements(wdfar)
           
;     exodus file id
      fid = fexo_id(exo_id)

; Read global data
  if n_elements (node) eq 0 then begin
;   Global data
    If num_glo_var[exo_id] eq 0 then begin
      print,'% REEXO: No GLOBAL variables in this file'
      return
    endif

    dtyp = size(/type, dataset)
    IF dtyp eq 7 THEN BEGIN
      IF n_elements(dataset)gt 1 then begin
        print, '% REEXO: Dataset must be a single string.'
        print, '         Only the first string will be used'
      endif
      if strlen(dataset[0]) eq 0 then begin
        map = 1+indgen(num_glo_var[exo_id])
        nelem = num_glo_var[exo_id]
      endif else begin
        map = get_match(dataset[0], name_glo_var, nelem = nelem)+1
        if nelem eq 0 then begin
          print, '% REEXO: No match for search string: ', dataset[0]
          return
        endif
      endelse

    endif else begin
      map = where(dataset ge 1 and dataset le num_glo_var[exo_id], nelem)
      if nelem eq 0 then begin
        print, '% REEXO: No valid datasets specified'
        return
      endif
      map= dataset[map]
    endelse
    if nelem gt nwdf then wdf = [wdf, wdf[nwdf-1]+1+indgen(nelem-nwdf)]
    if max (wdf) gt maxwdfarray then begin
      print, '% REEXO: Maximum number of WDF arrays will be increased'
      set_maxwdfarray, max(wdf)+16
    endif
    varid = ncdf_varid(fid, 'vals_glo_var')
    if varid ge 0 then begin
      ncdf_varget, fid, varid, wdf_array
      wdf_array= transpose(wdf_array)
    endif else begin
      print, '% REEXO: No WDF array variables found in Exodus II data'
      print, '                 Number of global Variables set to zero.'
      num_glo_var(exo_id) = 0
      return
    endelse
    for i = 0, nelem-1 do begin
      data = map[i]-1
      put_wdfarray,[time_whole,wdf_array(*,data)], wdf[i], /xy, clab = $
        name_glo_var(data)+' - '+file_title(exo_id), file=file_name(exo_id), $
        xlab = 'Time (s)', /no_copy
    endfor
    wdf_array=0
    ierror = 0
    return
  endif
  value = read_exo(dataset, node = node, position = position, number = number, $
          clabel = clab, ierror = ier)
  if ier ne 0 then return
  put_wdfarray,[time_whole, value], wdfar[0], /xy, clab = clab,  $
        file=file_name(exo_id), xlab = 'Time (s)', /no_copy

  end
