; ------------------------------------------------------------------------
; $Id: set_block_value.pro,v 1.1 2010/02/15 17:57:27 lpmix Exp $
; ------------------------------------------------------------------------
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
;+
; NAME:
;       SET_BLOCK_VALUE
;
; PURPOSE:
;       Set the field value to a constant in a block
;
; CALLING SEQUENCE:
;
;       SET_BLOCK_VALUE, STR, BLOCK, VALUE
;
; INPUTS:
;       STR:	Structure array number; Must be a type 12 structure 
;       BLOCK:  Block or blocks to be changed 
;               (Value must be in the range 1 to NBLOCK, unless keyword, ID is set)
;               (If ID is set, the value is the assigned block ID.)
;       VALUE:  Value to set the field for the specified blocks
;               If block is an array, VALUE may be an array.
;               If N_ELEMENTS(VALUE) < N_ELEMENTS(BLOCK), the last value is replicated.
;
; KEYWORD PARAMETERS:
;       ID:     If set, the BLOCK values are the block ID's
;               DEFAULT: ID = 1
;
; OUTPUTS:
;       The structure is returned with the specified block field values set to value
;
; SIDE_EFFECTS: The original structure is eliminated
;
; PROCEDURE:
;       The exodus file from which the structure was read is opened and the connectivity
;       for the specified blocks is read.  Unique nodes are determined and those field 
;       values are set to VALUE.
;
; EXAMPLE:
;
;       Set blocks with user id's equal to 89 and 90 to zero for structure, S1 and plot.
;         SET_BLOCK_VALUE, S1, [89, 90], 0.0
;         PLOTSTR, S1, 3, 76E-6, C_RAN= [40, 300], LEV = [2, 17000], /ASPECT
;
;       Set the field in output blocks 2 to 8 to -1
;         SET_BLOCK_VALUE, S1, FINDGEN(7)+2, -1.0, ID =0
;
;       Set the field in output blocks 2 to 8 to [-2, -3, -4, -5, -6, -7, -8]
;         SET_BLOCK_VALUE, S1, FINDGEN(7)+2, -(FINDGEN(7)+2), ID =0
;
; MODIFICATION HISTORY:
; 	Written by:	L. P. Mix, Jr., December 18, 2009
;-
;	July, 1994	
;

PRO SET_BLOCK_VALUE, structure, block, valuein, ID = keyid
;
@qscom.dat
;  Common block
@qsstrcom.dat
;  return on an error
   on_error, 1
;
   if n_elements (structinit) eq 0 then qsstruct_init
   if n_elements (qsplotinit) eq 0 then qsinit
;
   nparams = n_params()
   if nparams eq 0 then begin
     print,'% Format: ', $
    'SET_BLOCK_VALUE, STRUCTURE, BLOCK, VALUE, ID = keyid'
     return
   endif
;
   if nparams ne 3 then begin
     print,'% SET_BLOCK_VALUE: Procedure must have 3 parameters'
     return
   endif
;
   if (n_elements(structure) eq 0) || (n_elements(structure) eq 0) || $
      (n_elements(structure) eq 0) then begin
     print, '% SET_BLOCK_VALUE: All 3 parameters must be defined'
     return
   endif
;
   str = get_valid_structure(structure, 'SET_BLOCK_VALUE', lcstr=lcstr, ierror=ier)
   if str.type ne 12 then begin
      print, '% SET_BLOCK_VALUE: Only Type 12 structures allowed'
      return
   endif
   undefine, id
   id = ncdf_open(str.file)
   if (n_elements(id) eq 0) then begin
     print, '% SET_BLOCK_VALUE:  Unable to open Exodus file'
     return
   endif
   n = n_elements(block)
   nv = n_elements(valuein)
   value = (nv lt n) ? [valuein, make_array(n -nv, value = valuein[nv-1])] : valuein
   if (n_elements(keyid) eq 0) or keyword_set(keyid) then begin
     blo = make_array(n, /long)
     varid = ncdf_varid(id, 'eb_prop1')
     if varid lt 0 then begin
       print, '% SET_BLOCK_VALUE: Unable to obtain EB_PROP1 for block ids'
       return 
     endif
     ncdf_varget, id, varid, blockid
     for i = 0, n-1 do blo[i] = (where(block[i] eq blockid))[0]
     if min(blo) lt 0 then begin
       print, '% SET_BLOCK_VALUE: Unable to find all block ids.'
       print, ' BLOCK IDs:', blockid
       print, ' Input values:', block
       return
     endif
     blo = blo+1
   endif else blo = block
   blo = round (blo)
   for j = 0, n-1 do begin
     varid = ncdf_varid(id, 'connect'+strtrim(blo[j],2))
     ncdf_varget, id, varid, nodes
     uval = nodes[uniq(nodes, sort(nodes))]-1
     str.v1[uval] = value[j]
   endfor
   ncdf_close, id

   if lcstr eq 1 then structure = temporary(str) else i2s, str, structure, /no_copy

END
