; ------------------------------------------------------------------------
; $Id: elem2node.pro,v 1.3 2009/09/02 16:34:28 lpmix Exp $
; ------------------------------------------------------------------------
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
;+
; NAME:
;       ELEM2NODE
;
; PURPOSE:
;       Calculate the element values average to the node.
;       The values of all elements which touch a node are averaged to 
;         determine a node value.
;
; CALLING SEQUENCE:
;
;       ELEM2NODE, Element_Value, Node_Value  
;           [, TRIANGLES = triangles, LOCTRI = loctri]
;
; where:
;       Element_Value: The value of a quantity at each element
;                      If the number of elements is not correct, than only connection
;                      information will be returned.
;       Node_Value   : The average value of the quantity obtained by averaging
;                      the element value for every element that touches the node.
;       Triangles:     Values of the nodes for each element
;                      Size is [ndim+1,  Num_Elements]
;                      TRIANGLE values are zero based, ie,  valuse are [0: num_nodes-1]
;       LOCTRI:        Pointer array to each block in TRIANGLES.
;           Note: Num_elements may be larger than the problem  
;                 if some of the problem elements are not tets
;                 or triangles.  For example for QUADS in a 2 D problem, there is a
;                 a factor of 2 more triangles than quads.
; EXAMPLE:
;       Calculate the value of the electric field at each node and return the 
;       connectivity information in the variable, TRI
;         ELEM2NODE, ELEC_X, NodeEX, tri = tri, loctri = loctri
;       
; MODIFICATION HISTORY:
;       Written by:     L. P. Mix, November 12, 2005
;
;-
;
  pro ELEM2NODE, v_elem, v_node

  on_error, 1
; add common block
@exoduscom.dat
;
  v_node =0
  if n_elements(init_exo) eq 0 then begin
    print, '% ELEM2NODE: No Exodus files have been opened.'
    return
  endif
  if exo_id lt 0 then begin
    print, '% ELEM2NODE: No Exodus files are open.'
    return
  endif
  nblo = num_elem_blk[exo_id]
  if nblo lt 1 then begin
    print, '% ELEM2NODE: No blocks in this dataset.'
    return
  endif

  fid = fexo_id[exo_id]
  ndim = num_dim[exo_id]
  v_node = make_array(num_nodes, value = 0.0d0)
  i_node = make_array(num_nodes, value = 0L)
  ielem = 0
  if  (n_elements (v_elem) ne  num_elem) then begin
    print, '% ELEM2NODE: Element value must have NUM_ELEM values'
    help, v_elem, num_elem
    return
  endif
  for i = 1, nblo do begin
    varid = ncdf_varid(fid, 'connect'+strtrim(i, 2))
    ncdf_varget, fid, varid, connect
    connect = connect-1
    ss = size(connect)
    if ss[0] eq 1 then ss=[2, ss[1], 1, ss[2], ss[3]]
;   nnode is number of nodes per connect block
;   nelemb is the number of connection blocks in this block
    nnode = ss[1]
    nelemb = ss[2]
    for j = 0L, nelemb-1 do $
        v_node[connect[*, j]] += v_elem[j+ielem]
    for j = 0L, nelemb-1 do i_node[connect[*, j]] += 1
    ielem = ielem+nelemb
  endfor
 
  ok = where (i_node eq 0, nok)
  if nok gt 0 then i_node[ok] = 1
  v_node = v_node/i_node
  end
