; ------------------------------------------------------------------------
; $Id: get_exo_connect.pro,v 1.3 2009/09/02 16:38:09 lpmix Exp $
; ------------------------------------------------------------------------
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
;+
; NAME:
;       GET_EXO_CONNECT
;
; PURPOSE:
;       Get the connection array from the file
;
; CALLING SEQUENCE:
;
;       CONNECT = GET_EXO_CONNECT(NODES, LOCX, RANGE, loctri )
;
; where:
;       CONNECT      : List of nodes for each cell
;       NODES        : List of all nodes for the problem
;       LOCX         : Pointer array to the first node in each cell
;       RANGE        : Spatial range for each dimension
;       LOCTRI       : Pointer to first triangle in each block
;
; EXAMPLE:
;       Obtain the connection array for the current file
;           CONNECT = GET_EXO_CONNECT( LOCX )
;       
; MODIFICATION HISTORY:
;       Written by:     L. P. Mix, November 30, 2005
;
;-
;
 PRO Cube2Tet_exo, connect, nnode, nshapes, nval
 COMPILE_OPT hidden
 on_error, 1
; 
; must be called with either 4, 5, 6, or 8 nodes per element
;
 quad = [1,2,4,5,5,8,6,4,5,6,2,4,2,6,3,4,6,7,3,4,8,7,6,4]-1
 wedg = [4,6,5,3,1,4,2,3,2,4,5,3]-1
 pyr = [1,2, 4, 5, 2, 3, 4, 5]-1
 if nnode eq 4 then begin
;  tets
   nval = n_elements(connect) 
   return
 endif
 conn = temporary(connect)
 if nnode eq 6 then begin
;  Wedge
   connect = make_array(4L*nshapes*3)
   nval = 12
   nvalm= 11
   st = 0L
   for i = 0L, nshapes-1 do begin
     connect[st:st+nvalm] = (conn[*, i])[wedg]
     st = st + nval
   endfor
   nshapes = nshapes*3
 endif else if nnode eq 5 then begin
;  Pyramid
   connect = make_array(4L*nshapes*2)
   nval = 8
   nvalm= 7
   st = 0L
   for i = 0L, nshapes-1 do begin
     connect[st:st+nvalm] = (conn[*, i])[pyr]
     st = st + nval
   endfor
   nshapes = nshapes*2
 endif else begin
;  Quads
   connect = make_array(4L*nshapes*6)
   nval = 24
   nvalm= 23
   st = 0L
   for i = 0L, nshapes-1 do begin
     connect[st:st+nvalm] = (reform(conn[*, i]))[quad]
     st = st + nval
   endfor
   nshapes = nshapes*6
 endelse
 nval = n_elements(connect)
 end
;
  Function GET_EXO_CONNECT, nodes, locx, range, loctri, grid = grid
; 
; IF Grid is set, return actual connections.
;

  on_error, 1
; add common block
@exoduscom.dat
;
  if n_elements(init_exo) eq 0 then begin
    print, '% GET_EXO_CONNECT: No Exodus files have been opened.'
    return, 0
  endif
  if exo_id lt 0 then begin
    print, '% GET_EXO_CONNECT: No Exodus files are open.'
    return, 0
  endif
  nblo = num_elem_blk[exo_id]
  if nblo lt 1 then begin
    print, '% GET_EXO_CONNECT: No blocks in this dataset.'
    return, 0
  endif
  ndim = num_dim[exo_id]
  if n_elements (nodes) lt  ndim*num_nodes then begin
    print, '% GET_EXO_CONNECT: No node data passed'
    return, 0
  endif
  locx = 0L
  last_locx = 0L
  range = make_array(2, ndim, nblo+1)
  fid = fexo_id(exo_id)
  loctri = make_array(nblo+1, value =0L)

  for i = 1, nblo do begin
    varid = ncdf_varid(fid, 'connect'+strtrim(i, 2))
    ncdf_varget, fid, varid, connect
    connect = connect-1
    ss = size(connect)
    if ss[0] eq 1 then ss=[2, ss[1], 1, ss[2], ss[3]]
;   ss[1] is number of nodes per connect block
;   ss[2] is the number of connection blocks in this block
    num_nod = ss[1]
    num_conn = ss[2]
    nval = ss[ss[0]+2]
;   Total number of elements is ss[4]
;    nconn = ss[4]
    for j = 0, ndim-1 do begin
      range[0, j, i] =  min(nodes[connect, j], max = mm)
      range[1, j, i] = mm
    endfor
    if keyword_set(grid) then begin
      connect = reform(connect, ss[ss[0]+2], /overwrite)
      conn = (i eq 1) ?  connect : [conn, connect]
      pp = lindgen(ss[2]+1)*ss[1] + last_locx
      locx = [locx, pp[1:*]]
      last_locx  += nval
    endif else begin
      if ss[1] ne ndim+1 then begin
        if ndim eq 3 then begin
          cube2tet_exo, connect, num_nod, num_conn, nval
          loctri[i]= loctri[i-1]+num_conn
        endif else begin
          connect = make_triangles(connect)
          ss = size(connect) 
          if ss[0] eq 1 then ss=[2, ss[1], 1, ss[2], ss[3]]
          loctri[i] = loctri[i-1]+ss[2]
          connect = reform(connect, ss[ss[0]+2], /overwrite)
          nval = ss[4]
          nshapes = ss[2]
        endelse
      endif else begin
        connect = reform(connect, ss[ss[0]+2], /overwrite)
        loctri[i] = loctri[i-1] + num_conn
      endelse
      conn = (i eq 1) ?  connect : [conn, connect]
      pp = last_locx + lindgen(ss[2]+1)*(ndim+1)
      last_locx  += nval
      locx = [locx, pp[1:*]]
    endelse
  endfor
  if not keyword_set(grid) then $ 
    conn = reform(conn, ndim+1, loctri[nblo])
  for j = 0, ndim-1 do range[0, j, 0] = min(range[0, j, 1:*])
  for j = 0, ndim-1 do range[1, j, 0] = max(range[1, j, 1:*])
  return, conn
  end
