; ------------------------------------------------------------------------
; $Id$
; ------------------------------------------------------------------------
;
; Copyright 2005 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
;+
; NAME:
;       D2F
;
; PURPOSE:
;       Converts film density to photons/micron^2 or keV/micron^2
;       Density vs Energy or Density vs Wavelength is input
;
; CATEGORY:
;       Analysis
;
; CALLING SEQUENCE:
;
;       D2F, WDFin [, WDFout] [, FILM = film] [, TENSION = tension]
;          [, ENERGY = ENERGY] [, MODIFY = modify]
;
; INPUTS:
;       WDFin   Waveform array number for the input dataset
;               Unless ENERGY is set units are Density vs Wavelength(A)
;       WDFout  Waveform array number for the output dataset
;               Output abscissa units are the same as the input units
;               Default: WDFout = WDFin  ; original data will be lost
;	
; KEYWORD PARAMETERS:
;       FILM    Film type
;               0 - Kodak 101-07
;               1 - DEF
;               2 - SB-392 (SB-5)
;               2 - RAR 2492
;               4 - RAR 2495
;               5 - RAR 2497
;               Default:  FILM = 1
;       ENERGY  Indicates input data is a function of energy rather than 
;               wavelength
;               Energy = 1 ==>> Abscissa units are eV
;               Energy = 3 ==>> Abscissa units are keV
;       TENSION Tension parameter to the spline function 
;       MODIFY  Interpolation scheme at edges
;               0 - Cubic Spline across all edges
;               1 - Linear Interpolation across all edges but spline where data
;
; OUTPUTS:
;       The output is the photon flux from the theory of Henke
;
; PROCEDURE:
;       The VIDA routine of William E. Nelson, Science and Engineering Associates
;       has been modified for this PFIDL procedure.
;
; EXAMPLE:
;       Convert the density versus energy(ev) in waveform array 3 to photon flux
;         D2F, 3, /energy
;
; MODIFICATION HISTORY:
; 	Written by:	L. Paul Mix, September 1, 2004
;                       From the VIDA, D2I, routine by 
;                         William E. Nelson
;                         Science and Engineering Associates
;                         March, 1990
;
;-
Pro D2F, wdfin, wdfout, film = filmin, tension = tension, energy = energy, $
          modify = modify
;
on_error, 1
;
if n_params() eq 0 then begin
  print,'% Format: D2F, WDFIN, WDFOUT [, FILM = film] [, TENSION = tension]'
  print, '         [, ENERGY = energy]'
  return
endif
if n_elements(tension) eq 0 then tension = 1.0
if n_elements(modify) eq 0 then modify = 0.0

IF n_elements(filmin) eq 0 then Film = 1 else film = filmin

; Film Response
film_types =['101-07', 'DEF', 'SB-5', 'RAR 2492', 'RAR 2495', 'RAR 2497']
IF (film lt 0) or (film gt 5) THEN BEGIN
    PRINT, '% D2F:  Bad film type.'
    print,'         Film=0: 101-07'
    print,'         Film=1: DEF'
    print,'         Film=2: SB-5'
    print,'         Film=3: RAR 2492'
    print,'         Film=4: RAR 2495'
    print,'         Film=5: RAR 2497'
    RETURN
endif else  film_type = film_types[film]


wdf = wdvalid(wdfin, /valid, npoint = npoint)
if npoint eq 0 then begin
   print, '% Format: WDF must be a valid waveform array'
   return
endif
if n_elements (wdfout) gt 0 then wdfout = get_save_array(wdfout[0], wdf) $
   else wdfout = wdf

film = film - 1

ang=!pi/2.0

w2i, x, y, wdf
if keyword_set(energy) then begin
  if energy eq 3 then x= 12.398419/x $
     else x =  12398.419/x
endif 

Flux = henke_film_resp(film, x, y, ang, tension)

if wdfout ne wdf then xfr, wdf, wdfout
puty, flux, wdfout
cha, wdfout, ylab = 'Flux(photons/um^2)'
lab, wdfout, append =-1, 'Henke Cal of '
end
