; ------------------------------------------------------------------------ 
; $Id: pe_cal.pro,v 1.1.1.1 2002/11/12 23:05:26 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2005 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
;+
; NAME:
;       PE_CAL
;
; PURPOSE:
;       Obtain Calibration Data from the PE Scanning Microdensitometer
;       Code is specific to a 21 step calibration wedge with two areas
;       of calibration data
;
; CALLING SEQUENCE:
;       PE_CAL [, FILENAME] [, WIDTH = width] [, UP = up] [, DOWN = down]
;                 [, MANUAL = manual][, AVERAGE = average] [, OUTPUT = output]
;                 [, VALID_PEAK = valid_peak] [, STRUCTURE = structure]
;                 [, NPMIN = npmin] [, /SD] [, CALIBRATION = calibration]
;                 [, WORK = work] [, FILTER = filter] [, /ROTATE ]
;  where:
;       FILENAME - Filename of a header or image file.
;     
; KEYWORDS:
;       STRUCTURE - If a valid sturcture, then FILENAME is ignored and 
;                   the data is taken from structure.
;       ROTATE - If set, image will be rotated 180 degrees
;       WIDTH  - Width of the lineout 
;                Default 1500 Microns
;       UP     - Distance to the middle of the Upper scan from the 
;                middle of the lower set of calibration data
;                Default = 1000 + WIDTH/2 ( = 1750 with default width)
;       DOWN   - Distance to the middle of the Lower scan from the 
;                middle of the lower set of calibration data
;                Default = 1000 + WIDTH/2 ( = 1750 with default width)
;       MANUAL - If set, use cursor to set top and bottom scans 
;                If set, UP and DOWN are ignored.
;       Average- Fraction of the step used to obtain an average
;                Default = 0.7
;       npmin  - Minimum number of points around a step to ignore
;                Default = 5 points.
;       OUTPUT - Output waveform arrays corresponding to the bottom and top
;                lineouts vs bin number and the Density vs Exposure curve.
;                Default if n_elments(output) eq 0 = [1, 2]
;                Default if n_elments(output) eq 1 = [output,output+1]
;       Valid_peak - Derivative change must exceed Valid_peak * standard_dev
;                    Default value = 2
;       SD     - If set, standard deviation is stored in the next array 
;                Standard deviation = sqrt( total(y-ave)^2/ (n-1))
;       CALIBRATION - Calibration curve number or name for the film 
;                     and step-wedge. 
;                  See D2E for values. ('Blue', 'Green', 1, 2)
;                  NOTE:  Use D2E, output[0], wdf, calibration = calibration 
;                         to use the data with another calibration curve.
;       WORK   - Work array; Default = get_maxstr()-1
;       FILTER - Set for noisy signals. If negative, filt will be called with 
;                DELTA = -FILTER
;                 
; This version is set for the following parameters:
;  Contact Paul Mix to change
;       Fudge_factor = 500 microns
;       
; PROCEDURE:
;       1. Read data file
;       2. Take lineouts
;       3. Use upper lineout to determine bins.
;          Smooth and take derivative 
;       4. Calculate average bin value
;       5. Store outputs in Output
; OUTPUTS:
;       Return todays date in the form: Month day, year
;
; RESTRICTIONS:
;       No restrictions.
;
; Structure Arrays get_maxstr()-1 and get_maxstr() contain the image data.
; Waveform arrays 71 to 74 are used for calculations.
; Waveform arrays 75, 76 have the lineout values vs film position
;
; EXAMPLE:
;       PE_cal 
;
; MODIFICATION HISTORY:
; 	Written by: Paul Mix, March 14, 2000
;-
; pe_cal
 pro pe_cal, filename, width = width, up = up, down = down, $
        average = average, output = output, valid_peak = valid, $
        structure = structure, sd = stdev, npmin = npmin, manual= manual, $
        rotate = rotate180, calibration = calibration, filter = filterit
on_error, 1
;
; Default Parameters
 bottom_fudge = 500.
 bottom_fudge = 50.
if n_elements (npmin) eq 0 then npmin=5
 bottom_width = 97750.
 bottom_width = 400.
 
 width_default = 1500.
 width_default = 300.
; width of each filter step (microns)
 step_width = 5000.
 step_width = 300.
; Median filter width in microns
  filt_width = step_width  *0.6 

 up_default = 1000.
 down_default = 1000.
 up_default = 100.
 down_default = 100.
 average_default = 0.7
 if n_elements(valid) gt 0 then valid_peak=valid[0] > 1 else valid_peak = 2
 if n_elements(work) eq 0 then work = get_maxstr()-1
 if work lt 1 or work ge get_maxstr() then  work = get_maxstr()-1
 str =0
 if keyword_set (structure) then begin
   s = size(structure, /type)
   if s lt 5 then begin
     str = long(structure)
     junk = strvalid(str, /valid, /quiet, np=np)
     if (np ne 1) then begin
       print, '% PE_CAL: Structure is not valid: ' , strtrim(str, 2)
       return
     endif
     if str ne work then xfrstr, str, work
     s2i, s, work
   endif else begin
     str =work
     if s eq 8 then i2s, structure, str  else begin
       print, '% PE_CAL: Structure is not valid'
       return
     endelse
     s = structure
   endelse
   if s.type ne 1  then begin
       print, '% PE_CAL: Only type 1 structures may be used for calibration 
       return
   endif else begin
     xrange = s.range[*, 0, 0]
     yrange = s.range[*, 1, 0]
     filename = s.file
   endelse 
   s =0
 endif else if n_elements(filename) eq 0 then begin
   filename = dialog_pickfile(/read, /must_exist)
   if strlen(filename) lt 5 then begin
     Print, '% PE_CAL: No valid file selected'
     return 
   endif
 endif
 file_name = filename
 if str eq 0 then begin
   delstr, work
   readpe, work, filename, xrange = xrange, yrange = yrange
   if strvalid(work, /valid) lt 0 then begin
     print, '% PE_CAL: Error reading data'
     return
   endif
 endif

 if keyword_set(rotate180) then begin
   a = reform(get_array(work, -1))
   a = rotate(a, 2)
   put_array, work, a, -1
 endif
 filename = strtrim(filename, 2)
 print, 'Process file: ', filename

 vers = strupcase(strmid(!version.os, 0, 3))
 if (vers eq 'WIN') then begin
   dirsym = '\'
 endif else if (vers eq 'MAC') then begin
   dirsym= ':'
 endif else  dirsym= '/'
 m = strlen(filename)
 ii = strpos (filename, dirsym, /reverse_search)
 if ii gt 0 then fcom= strmid(filename, ii+1, m) else fcom = filename
 ii = strpos (fcom, '.')
 if ii gt 0 then fcom = strmid(fcom, 0, ii)

 x = get_array(work, 1)
 dx = (max(x, min = mm)-mm)/(n_elements(x)-1)
 filt_wid = long(filt_width/dx)
 if filt_wid mod 2 eq 0 then filt_wid = filt_wid -1
help, filt_wid, filt_width, dx

 if n_elements (width) eq 0 then width = width_default

 if keyword_set(manual) then begin
   plotstr, work, cn=2
   print, 'Indicate the lineout center for the lower steps'
   cur, xlow, ylow
   print, 'Indicate the lineout center for the upper steps'
   cur, xhi, yhi
   if ylow gt yhi then begin
     t = yhi & yhi = ylow & ylow = t
   endif
   y = (ylow+yhi)*0.5
   up = yhi -y
   down = ylow -y
 endif else begin
   plotstr, work, /quiet, /yline, width = (total(xrange)+[-width, width])*0.5, $
        out = 71, cn=2
   smo, 71, 72, /med, wid = filt_wid
   dif, 72
   smo, 72
   aver = ave(72, sd= sd, /quiet)
   m = mx (72, time = time, /quiet)
   if abs(m) lt valid_peak*sd then begin
     print, '% PE_CAL: Unable to find top edge of bottom step wedge'
     return
   endif
   m = fwhm(72, left=time-bottom_fudge, right=time+bottom_fudge, $
       first=h1, second=h2, base = 0.0, /quiet)
   y= 0.5*(h1+h2) 
   if n_elements (up) eq 0 then up = up_default+width/2
   if n_elements (down) eq 0 then down = -down_default-width/2 else down= -down
 endelse

 if n_elements (average) eq 0 then average = average_default
 if average lt 0 or average gt 1 then average = average_default
 buffer = (1.0-average)*0.5
   neo = n_elements (output)
 if neo eq 0 then output= [1, 2] else begin
   if neo eq 1 then output =[output, output+1]
 endelse

 plotstr, work, cn=2, width = width, xline = y+[down, up], out=71, /pl

 smo , 71, 73, wid= filt_wid, /med
 smo , 72, 74, wid= filt_wid, /med
 dif, 73
 dif, 74
; fix for bad data
 ran = mx(73)
 lim, 73, /yaxis, -ran, ran*1.1
 ran = mx(74)
 lim, 74, /yaxis, -ran, ran*1.1
 
   
 av3 = ave(73,sd = sd3,/quiet)
 av4 = ave(74,sd = sd4,/quiet)
 sd = sd3+sd4


 if keyword_set(filterit) then begin
   if filterit gt 0 then smo_new = .3*step_width else smo_new = -filterit
   filt, 73, D = smo_new
   filt, 74, d = smo_new
   bottom_fudge=  smo_new
   sd = min([0.2*mx(73, /quiet),  0.2*mx(74, /quiet), sd*0.5])
 endif
;
   
 val1 = make_array(32)
 val2 = val1
 sd1 = val1
 sd2 = val1
 xx = val1
 ii = 0

 yf = getyf(74)
 ok = where (yf gt sd, nok)
 if nok eq 0 then begin
   print, '% PE_CAL: Unable to find a step on the top lineout
   return
 endif
 if nok gt 75  and not  keyword_set (filterit) then begin
   print, '% PE_CAL: Additional filtering added.
   smo_new = .3*step_width
   filt, 73, D = smo_new
   filt, 74, d = smo_new
   bottom_fudge=  smo_new
   sd = min([0.2*mx(73, /quiet),  0.2*mx(74, /quiet), sd*0.5])
   yf = getyf(74)
   ok = where (yf  gt sd, nok)
   if nok eq 0 then begin
     print, '% PE_CAL: Unable to find a step on the top lineout after filtering
     return
   endif
 endif
 i = 0
 xf = (getxf(74))[ok]
 ok = where (xf gt step_width*0.5, nok)
 if nok eq 0 then begin
   print, '% PE_CAL: Unable to find a step on the top lineout beyond the start
   return
 endif
 first = xf[ok[0]]

;  calculate the first step
 m = fwhm(73, left=first-bottom_fudge, right=first+bottom_fudge, $
     first=b1, second=b2, base = 0.0, /quiet, peak = bpeak)
 if ((m le 0) or (bpeak lt sd)) then begin
     bpeak = 0.0
     bp = 0.0
 endif else bp = b1+b2  

 m = fwhm(74, left=first-bottom_fudge, right=first+bottom_fudge, $
     first=t1, second=t2, base = 0.0, /quiet, peak = tpeak)
 if ((m le 0) or (tpeak lt sd)) then begin
     tpeak = 0.0
     tp = 0.0
 endif else tp = t1+t2
 if bpeak+tpeak lt sd then xx[0] = xrange[1] else $
    xx[0] = (bpeak*bp +tpeak*tp)/(tpeak+bpeak)*0.5

;  see if the bottom step is there.

  m = fwhm(73, first=b1, second=b2, base = 0.0, /quiet, peak = peak, /minimum)
  if peak lt sd then bottom1 = -1e6 else bottom1 = (b1+b2)*0.5
 
 bpeak = 0.00 
 b1 = 0
 b2 = 0
 sdb =0
 sw = step_width
 fw = bottom_fudge
 plast = xrange[1]-fw
 ind = 1
 ss = xx[0]+ sw
 while ((ss lt plast) and (ind lt 32)) do begin
   m = fwhm(73, left=ss-fw, right= ss+fw, first=b1, second=b2, $
      base = 0.0, /quiet, peak = bpeak, /maximum)
   if ((m le 0) or (bpeak lt sd)) then begin
     bpeak = 0.0
     bp = 0.0
   endif else bp = b1+b2  
   m = fwhm(74, left=ss-fw, right= ss+fw, first=t1, second=t2, $
      base = 0.0, /quiet, peak = tpeak, /maximum)
   if ((m le 0) or (tpeak lt sd)) then begin
     tpeak = 0.0
     tp = 0.0
   endif else tp = t1+t2
   if bpeak+tpeak lt sd then ss = xrange[1] else begin
     ss = (xx[ind] = (bpeak*bp +tpeak*tp)/(tpeak+bpeak)*0.5)
     ind = ind+1
   endelse
   ss = ss+sw
 endwhile
   xval = xx
   iv = 0
   dxmin = npmin*dx
   dxmin2 = dxmin+dxmin
   ww = xx[1]-xx[0]
   down= y+down & up = y+up
   hwidth = 0.5*width
   boxwidth = [-hwidth, -hwidth, hwidth, hwidth, -hwidth]
   if xx[0] gt dxmin2 then begin
     if (xx[0] -xrange[0]) gt (buffer+average)*ww then begin
       x2 = xx[0]-buffer*ww
       x1 = xx[0]-(buffer+average)*ww
     endif else begin
       x1 = xrange[0]
       x2 = xx[0]-dxmin
     endelse
     val1[iv] = ave(72, sd = sdt, left = x1, right=x2, /quiet)
     sd1[iv] = sdt
     polyfill, [x1, x2, x2, x1, x1], up+boxwidth, /line_fill, orient = 45
     oplot, [x1, x1], [-hwidth, hwidth]+up
     oplot, [x2, x2], [-hwidth, hwidth]+up
     xval[iv]=(x1+x2)*0.5
     if bottom1  gt x1 and bottom1 lt xx[0] then $
       x2 = min([bottom1, xx[0]])-dxmin
     val2[iv] = ave(71, sd = sdb, left = x1, right=x2, /quiet)
     sd2[iv] = sdb
     polyfill, [x1, x2, x2, x1, x1], down+boxwidth, /line_fill, orient = 45
     oplot, [x1, x1], [-hwidth, hwidth]+down
     oplot, [x2, x2], [-hwidth, hwidth]+down
     iv = iv+1
   endif 
   indm = ind-1
   if ((xx[indm]+ dxmin2) gt xrange[1]) then indm= indm-1
   bav = (buffer+average)
   for i = 0, indm do begin
     if i lt indm then ww = xx[i+1]-xx[i]
     x1 = xx[i] + buffer*ww
     x2 = (xx[i] + bav*ww) < xrange[1]
     val1[iv] = ave(72, sd = sdt, left = x1, right=x2, /quiet)
     polyfill, [x1, x2, x2, x1, x1], up+boxwidth, /line_fill, orient = 45
     oplot, [x1, x1], [-hwidth, hwidth]+up
     oplot, [x2, x2], [-hwidth, hwidth]+up
     sd1[iv] = sdt
     xval[iv]=(x1+x2)*0.5
;    Check for the bottom filter
     if bottom1 gt xx[i] and bottom1 lt xx[i+1] then $
       x2 = min([bottom1, xx[i+1]-dxmin])
     val2[iv] = ave(71, sd = sdb, left = x1, right=x2, /quiet)
     polyfill, [x1, x2, x2, x1, x1], down+boxwidth, /line_fill, orient = 45
     oplot, [x1, x1], [-hwidth, hwidth]+down
     oplot, [x2, x2], [-hwidth, hwidth]+down
     sd2[iv] = sdb
     iv = iv+1
   endfor
 ivm = iv-1
 if ivm lt 2 then begin
   print, '% PE_CAL: At least three steps are required for waveform arrays.'
   print, '          Steps found: ', strtrim(iv, 2), ' Steps'
   return
 endif 
 ivm = ivm < 20
 xval = xval[0:ivm]
 val1 = val1[0:ivm]
 val2 = val2[0:ivm]
 lchak = 1
 if ivm eq 20 then top = 1 else begin
   polyfill, [x1, x2, x2, x1, x1] , up -hwidth*[-1,-1, 1,1,-1], col=4
   top = 0l
   print, 'Please enter the value for the indicated bin [ 1 to ', $
     strtrim(21-ivm, 2), ']:'
   read, top
   lchak =0
 endelse
 if top lt 1 or top gt 21-ivm then begin
   print, '% D2E: Value of the top step must be in the range of 1 to ', $
     strtrim(21-ivm, 2)
   Print, '       Value of the top step is set to 1'
 endif

 btval = top +findgen(ivm+1)
 bbval = btval + 21
 xxval = [btval, bbval]
 dval = [rotate(val1, 2), rotate(val2, 2)]

 xlab =  'Bin number'

 put_wdfarray, [xxval,dval], /xydata, output[0], $
   xlabel = xlab, ylab ='Density',  file= file_name, $
   clabel = fcom + '  -  '+ today() + $
        ', MicroD Bin Calibration Data: '  + filename 

 d2e, output[0], output[1] , calibration = calibration
 lab, output[1], fcom + '  -  '+ wdcomment(output[1])+', '+today()+':'+filename
 cha, output[1], file = file_name

 
 if keyword_set(stdev) then begin
   next = output[1]+1
   dval = [rotate(sd1[0:ivm], 2), rotate(sd2[0:ivm], 2)]

   put_wdfarray, [xxval,dval], /xydata, next, $
     xlabel = 'Bin number', $
     clabel = fcom + '  -  '+ today() + ', Standard Deviation: '  + filename
 endif
 mkcircle, /fill
 if keyword_set(lchak) then hak,/mes
 plo, [72, 71], /ov, /nl, /ns, /color, ytit = 'Density', $
   lname=['Top Cal Data', 'Bottom Cal Data']
 oplot, psym= 8, xval, val1, color = 3
 oplot, psym= 8, xval, val2, color = 4
 i2w, xval, val1, 75, clab = 'Top Data', $
   xlab = 'Position(microns)', ylab = 'Density'
 i2w, xval, val2, 76, clab = 'Bottom Data', $
   xlab = 'Position(microns)', ylab = 'Density'

end

