; ------------------------------------------------------------------------
; $Id$
; ------------------------------------------------------------------------
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
;+
; NAME:
;       MAKE_SPH_RESPONSE
;
; PURPOSE:
;
;       Take the series of wdf arrays generated by SPH_RESPONSE
;       and generate a WDF array with separation vs angle or distance.  To generate
;       this waveform, we assume that two known lines are measured with the spherical
;       spectrometer.  This routine generates a response curve so that a measurement
;       of the separation in MM of the two lines on the calibration will predict an
;       angle or distance for the spherical spectrometer.
;
; ****  Sph_Response - Make a series of wdf arrays of with different Bragg angles,
;       or different distances.  Each WDF array is energy or xray wavelength vs 
;       position on the film.  Either or both waveforms may be saved.  A given
;       geometry is calculated and then the position of the source is varied to
;       determine the effect on the WDF array.  These waveforms may be saved to
;       a PFF file for future calculations, but take only seconds to generate.
;
; ****  Make_SPH_Response - Take the series of wdf arrays generated by SPH_RESPONSE
;       and generate a WDF array with separation vs angle or distance.  To generate
;       this waveform, we assume that two known lines are measured with the spherical
;       spectrometer.  This routine generates a response curve so that a measurement
;       of the separation in MM of the two lines on the calibration will predict an
;       angle or distance for the spherical spectrometer.
;       The syntax is:`
;          PRO MAKE_SPH_RESPONSE, WDF, E1, E2, WDF_ARRAY
;       Where E1 and E2 are the wavelengths or energies of the two calibration 
;       energies, WDF is the array number for the response, and WDF_ARRAY are the
;       WDF arrays generated by the procedure SPH_RESPONSE.
;
; ****  Calc_Sph_Response - Calculates the actual response for a dataset.  Using
;       the response curve generated by MAKE_SPH_RESPONSE, and the measured spacing
;       of the the two energy peaks on the film, this routine determines the Bragg
;       angle or distance required to match the data and does a full calculation of
;       the spherical spectrometer for these values.  The spectrometer values 
;       calculated for the fit are saved as the default values for SPH_SPEC.

;
; CATEGORY:
;       Analysis
;
; CALLING SEQUENCE:
;
;       MAKE_SPH_RESPONSE, WDF, E1, E2, WDFARR
;
; INPUTS:
;       WDF:    WDF array number for the response.
;               Default: WDF = GET_MAXWDF()-1
;       E1:     One energy for the spherical spectrometer calibration
;       E2:     Another, second, energy for the spherical spectrometer calibration.
;       NOTE:   E1 and E2 are the energies/wavelength of the two calibration lines.
;               Units must be the same as the Y-axis of the WDFARR arrays.
;       WDFARR: WDF arrays with the response of the spherical spectrometer
;               for a series of Bragg angles or source distances.
;               If WDFARR is an two element array, then it contains the first and
;               last wdf array where the spherical spectrometer responses are stored.
;               If WDFARR has more than 2 elements, we assume that that this is a list
;               wdf arrays.
;               For example, assume the responses are in arrays 21 to 71.
;                 WDFARR = [21, 71] <or> WDFARR = 21+indgen(51) 
;               would provide identical results.
;
; OUTPUTS:
;       A WDF array is generated with separation vs angle or distance.
;
; RESTRICTIONS:
;       The two energies must be in the range of the data in WDFARR
;       All of the waveforms in WDFARR must have the same spectrometer parameters.
;
; PROCEDURE:
;       1.  Determine the waveforms which contain values for both E1 and E2.
;       2.  Determine the distance between E1 and E2 for each of the valid waveforms.
;       3.  Save the separations vs Theta or Distance in a waveform array.
;
; EXAMPLE:
;
;	Calculate a response for energies 5898.75 and 6403.84 for the waveforms in
;       WDF arrays 101 to 151, and store the response curve in WDF array, 100.
;           MAKE_SPH_RESPONSE, 100, 5898.75, 6403.84, [101, 151]
;
; MODIFICATION HISTORY:
;           Written by:  L. P. Mix, Jr., March 30, 2005
;
;-

PRO make_sph_response, wdf, e1, e2, wdfarr

on_error, 1
npar = n_params()
if npar eq 0 then begin
  print,'% Format: MAKE_SPH_RESPONSE, WDF, E1, E2, WDF_ARRAY'
  return
endif

If n_elements(E1) eq 0 or n_elements(e2) eq 0 then begin
  print, '% MAKE_SPH_RESPONSE:  The two calibration lines must be defined'
  help, e1, e2
  return
endif

If n_elements(wdfarr) lt 2 then begin
  print, '% MAKE_SPH_RESPONSE:  WDFARR must have 2 or more elements.'
  help, WDFARR
  return
endif

okw= wdvalid(wdfarr,/valid, np = nwdf)
if nwdf lt 2 then begin
  print, '% MAKE_SPH_RESPONSE: WDFARR contains '+strtrim(nokw,2)+' valid waveform arrays.'
  print, okw, nwdf
  return
endif
if nwdf eq 2 then begin
  nwdf = 1+max(okw, min = m) -m 
  arr = m + lindgen(nwdf)
endif else arr = okw

; determine if we are scanning in theta or in distance.
emin = min ([e1, e2], max = emax)
wdf = (n_elements(wdf) eq 0) ? get_maxwdf()-1 : wdf[0]
del = (emax-emin)*0.001

wdinfo, arr, ymin = ymin, ymax = ymax
ok = where(((ymin -del le emin) and (ymax+del ge emax)), nok)
if nok eq 0 then begin
  print, '% MAKE_SPH_RESPONSE: E1 and E2 are not in the range of he selected waveforms'
  return
endif
if nok lt nwdf then begin
    nwdf = nok
    arr = arr[ok]
endif


xx = fltarr(nwdf)
yy = fltarr(nwdf)
comm = wdcomment(arr)
search = ';;Source_angle='
lsearch = strlen(search)
for i = 0, nwdf-1 do $
  yy[i] = abs(get_time(arr[i], e1) - get_time(arr[i], e2))
val =0.0
param = strmid(comm[0], strpos(comm[0], ' - Crystal'))
for i = 0, nwdf-1 do begin
  substring = strmid(comm[i], strpos(comm[i], search)+lsearch)
  reads,substring, val
  xx[i] = val
endfor  
if max(xx , min= m) -m lt 0.001*xx[0] then begin
  search = ';;Source_distance='
  lsearch = strlen(search)
  for i = 0, nwdf-1 do begin
    substring = strmid(comm[i], strpos(comm[i], search)+lsearch)
    reads,substring, val
    xx[i] = val
  endfor  
  lcdist = 1
endif else lcdist =0
j = sort(xx)
xx = xx[j]
yy = yy[j]
unit = (e1 gt 400) ? ' eV' : ' Ang'
if lcdist eq 0 then $  
  i2w, xx, yy, wdf, /xydata, xlab ='Source Angle(deg)', ylab ='Separation (mm)', $
    clab = 'Separation of '+strtrim(emin, 2)+' and '+strtrim(emax, 2)+unit+param $
else i2w, xx, yy, wdf, /xydata, xlab ='Source Distance(mm)', ylab ='Separation (mm)', $
    clab = 'Separation of '+strtrim(emin, 2)+' and '+strtrim(emax, 2)+unit+param 
end
