; ------------------------------------------------------------------------
; $Id$
; ------------------------------------------------------------------------
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
;+
; NAME:
;       NTOF
;
; PURPOSE:
;       This sets up a common block to store neutron time of flight parameters
;       for a shot and a widget to display parameters.  This version is set for
;       2 lines-of-sight(LOS) and 2 detectors per LOS.  These may be easily changed.
;
; CATEGORY:
;       Analysis
;
; CALLING SEQUENCE:
;       NTOF [, SHOT_NUM]
;
; OPTIONAL INPUTS:
;       SHOT_NUM:  Shot number for the Z shot to be analyzed
;                  If this is missing or no special data exists, the value will
;                  be stored and default parameters stored in the common block.
;                  SHOT_NUM must be greater than 0.  All calculated parameters
;                  will be set to zero.
;	
; KEYWORD PARAMETERS:
;       RESET:     Rest parameters to default parameters.  Shot number is not changed.
;       SHOT_NUM:  Shot number for he shot.  This is stored in the common block,
;                  but no special parameters are loaded.
;                  SHOT_NUM must be greater than 0
;       READ:      Read NTOF signals for the current shot.
;                  Waveforms are read into WDF arrays beginning with ABS(READ)
;                  DEFAULT:  READ =1
;                  (see also READNTOF)
; Following keywords assume that there are neutron detectors on side and in basement
;
;       BNAMES:    Bottom detector names
;                  DEFAULT: BNAMES =['NTOFBM1', 'NTOFBM2']
;       SNAMES:    Sided detector names 
;                  DEFAULT: SNAMES =['NTOFM1', 'NTOFM3']
;       DETECTOR_RESPONSE:  Detector response in ns (1E-9 Sec)
;                  DEFAULT: DETECTOR_RESPONSE = 6.75 
;       BDISTANCE: Bottom detector distance in meters
;                  Must have the same number of elements as BNAMES
;                  DEFAULT: BDISTANCE =[7.30, 8.09]
;       SDISTANCE: Side detector distance in meters
;                  Must have the same number of elements as SNAMES
;                  DEFAULT: SDISTANCE =[7.419, 8.386]
;       
; OUTPUTS:
;       NTOF common block is modified and widgets showing values are generated.
;
; COMMON BLOCKS:
;       NTOF_COMMON_BLOCK:  This common block is defined and modified
;
; EXAMPLE:
;       Load the neutron time of flight parameters for shot 1721
;         NTOF, 1721
;
;       Read neutron time-of-flight signals for shot 1673
;         NTOF, 1673, /READ
;
;       Read data for the current shot into WDF array 100
;         NTOF, /READ
;          or
;         READNTOF, 1
;
; MODIFICATION HISTORY:
; 	Written by:	L. P. Mix, Jr., June 29, 2007
;-
;  
pro ntof_event, event
;
@ntof_common.dat
    on_error, 1
    if tag_names(event, /structure_name) eq 'WIDGET_KILL_REQUEST' then begin
       ok = where (los_tlb eq event.top, nok)
       if nok gt 0 then begin
         los_tlb[ok] = 0
         widget_control, event.top, tlb_get_offset = offset
         dir_offset[*, ok] = offset
       endif
       widget_control, event.top, /destroy
       return
     endif
end

PRO  NTOF, SHOT, SHOT_NUM = shot_num, BNAMES = bnames, $
     SNAMES = snames,  DETECTOR_RESPONSE =  detector, $
     SDISTANCE = sdistance, BDISTANCE = bdistance, readdata = readdata

on_error,1
;
; Parameters:
;   Time:  Peak of the gaussian
;   Dtime: FWHM of the pulse 
;          FWHM = 2D0*SQRT(2*ALOG(2))*A[2]
;                 Where a is parameter from GaussFit
;

@ntof_common.dat

VALID = [-1] 

if (n_elements(ntof_init) eq 0) or keyword_set (reset) then begin
  if n_elements(shot_number) eq 0 then shot_number = 1
  ntof_init =1
  numb_los = 2
  name_los = ['BOTTOM', 'SIDE']
  numb_detect =[2, 2]
  max_detect = max(numb_detect)
; t = {time:0.0, t_error:0.0, width:0.0, width_error:0.0, energy:0.0, 
;        energy_error:0.0, area:0.0}
;   Time is time for the peak
;   t_error is an estimate of the error in time.
;   width is the fwhm of the peak
;   width_error is an estimate of the error in the width
;   energy is the thermal width of the interacting particles.
;   energy_error is an estimate of the error in the ion energy.
;   area is the area of eh gaussian
  t = {time:0.0, t_error:0.0, width:0.0, width_error:0.0, energy:0.0, energy_error:0.0, area:0.0}
  dd_time=replicate (t, max_detect, numb_los)
  dt_time=replicate (t, max_detect, numb_los)
; Detector Distance:
  detect_dist = make_array(max_detect, numb_los)
  B_DISTANCE =[7.30, 8.09]
  S_DISTANCE =[7.419, 8.386]
  detect_dist[0, 0]= B_distance
  detect_dist[0, 1]= S_distance
  
; Detector response in NS
  DETECT_RESP = 6.75
  detect_resp = replicate(detect_resp, max_detect, numb_los)
; Detector Names:
  detect_name=  make_array(max_detect, numb_los, /string)
  detect_wdf=  make_array(5, max_detect, numb_los, /long)
  B_NAMES =['NTOFBM1', 'NTOFBM2']
  S_NAMES =['NTOFM1', 'NTOFM3']
  detect_name[0, 0]= B_names
  detect_name[0, 1]= S_names
  fit_velocity = replicate(0.0, 2, numb_los)
  fit_energy = replicate(0.0, 2, numb_los)
  bang_time = 0.0
  los_tlb = make_array(numb_los, /long)
  dir_offset = make_array(2, numb_los, /long)
  dir_offset = make_array(2, numb_los, /long)
  dir_offset[1, *]= 20
  for i = 0, numb_los-1 do dir_offset[0, i] = 10 + i*300
  bang_time = make_array(2, numb_los)
  lsfit_param = replicate(0.0, 3, 3)
endif
; help, shot_num, shot
if n_elements(readdata) eq 0 then readdata = 1
; help, shot
lc_reset = 0
if n_elements(shot) gt 0 then begin
  shot_number = shot > 1
  ok = where (shot eq valid, nok)
  if nok gt 0 then begin
    print, ' Custom Initialization'
  endif
  lc_reset = 1
endif else if n_elements(shot_num) gt 0 then shot_number = shot_num > 1


if n_elements(bnames) gt 0 then detect_name[0, 0] = bnames
if n_elements(snames) gt 0 then detect_name[0, 1] = snames

if n_elements(bdistance) gt 0 then detect_dist[0, 0] = bdistance
if n_elements(sdistance) gt 0 then detect_dist[0, 1] = sdistance
if n_elements(detector) gt 0 then detector_response[*]= abs(detector[0])


if lc_reset eq 1 then begin
  max_detect = max(numb_detect)
  t = {time:0.0, t_error:0.0, width:0.0, width_error:0.0, energy:0.0, energy_error:0.0, area:0.0}
  dd_time=replicate (t, max_detect, numb_los)
  dt_time=replicate (t, max_detect, numb_los)
  fit_velocity = replicate(0.0, 2, numb_los)
  fit_energy = replicate(0.0, 2, numb_los)
  bang_time = replicate(0.0, 2, numb_los)
  lsfit_param = replicate(0.0, 3, 3)
  detect_wdf=  make_array(5, max_detect, numb_los, /long)
endif

if n_elements(los_tlb) gt 0 then for i = 0, n_elements(los_tlb)-1 do $
  if los_tlb(i) gt 0 then widget_control, los_tlb[i], /destroy
los_tlb = make_array(numb_los, /long)
dd_field = make_array(3, max_detect, numb_los, /long)
dt_field = make_array(3, max_detect, numb_los, /long)
dd_ener_field = make_array(max_detect, numb_los, /long)
dd_bang_field = make_array(max_detect, numb_los, /long)
dt_bang_field = make_array(max_detect, numb_los, /long)
fit_ener_field = make_array(2,max_detect, numb_los, /long)
for i = 0, numb_los-1 do begin
  los_tlb[i] = widget_base(title= name_los[i]+' LOS', /column, $
      /tlb_kill_request_events)
  for j = 0, numb_detect[i]-1 do begin
;   each detector on the LOS
      base = widget_base(los_tlb[i], /column, frame = 2)
      lab = widget_label(base, /align_center, $
        value = detect_name[j, i])
      lab = widget_label(base, /align_center, $
        value = 'Distance='+strtrim(detect_dist[j, i], 2))
;     once for dd neutrons and once for dt neutrons
      base0 = widget_base(base, /row)
      base_dd = widget_base(base0, /column)
      lab_dd = widget_label(base_dd, value= 'DD Neutrons', /align_center)
      dd_field[0, j, i] = cw_field(base_dd, /float, value = dd_time[j, i].time, $
        title = 'Peak Time(ns):')
      dd_field[1, j, i] = cw_field(base_dd, /float, value = dd_time[j, i].width, $
        title = 'Peak Width(ns):')
      dd_field[2, j, i] = cw_field(base_dd, /float, value = dd_time[j, i].energy, $
        title = 'Ion Energy(keV):')

      base_dt = widget_base(base0, /column)
      lab_dt = widget_label(base_dt, value= 'DT Neutrons')
      dt_field[0, j, i] = cw_field(base_dt, /float, value = dt_time[j, i].time, $
        title = 'Peak Time(ns):')
      dt_field[1, j, i] = cw_field(base_dt, /float, value = dt_time[j, i].width, $
        title = 'Peak Width(ns):')
      dt_field[2, j, i] = cw_field(base_dt, /float, value = dt_time[j, i].energy, $
        title = 'Ion Energy(keV):')
  endfor
  if numb_detect[i] gt 1 then begin
    Sum_base = widget_base(los_tlb[i], /column, /frame)
    sum = widget_label(sum_base, value = 'Neutron TOF Energies / Bang Time')
    data_base = widget_base(sum_base, /row)
    data_ddb = widget_base(data_base, /column)
    data_ddt = widget_base(data_base, /column)
;   this needs to be fixed if more than 2 detectors
    distanc = abs(detect_dist[1, i]-detect_dist[0, i])
    if dd_time[0, i].time gt 0 and dd_time[1, i].time gt 0 then $
      fit_velocity[0, i]= distanc/abs(dd_time[1, i].time-dd_time[0, i].time) $ 
      else fit_velocity [0, i] = 0.0
    if dt_time[0, i].time gt 0 and dt_time[1, i].time gt 0 then $
      fit_velocity[1, i]= distanc/abs(dt_time[1, i].time-dt_time[0, i].time) $ 
      else fit_velocity [1, i] = 0.0
    fit_energy[*,i]= energ(fit_velocity[*, i])
    fit_ener_field[0, i] = cw_field(data_ddb, value = fit_energy[0, i], $
      title = 'DD Energy(MeV):')
    if fit_velocity[0, i] gt 0.0 then $
      ddb = dd_time[0, i].time - detect_dist[0,i]/fit_velocity[0, i]*1e9 else ddb = 0.0
    dd_bang_field[0, i] = cw_field(data_ddb, value = ddb, $
      title = 'DD Bang Time(ns):')

    fit_ener_field[1, i] = cw_field(data_ddt, value = fit_energy[1, i], $
      title = 'DT Energy(MeV):')
    if fit_velocity[1, i] gt 0.0 then $
      dtb = dt_time[0, i].time - detect_dist[0, i]/fit_velocity[1, i]*1e9 else dtb = 0.0
    dt_bang_field[0, i] = cw_field(data_ddt, value = dtb, $
      title = 'DT Bang Time(ns):')
    bang_time[*, i]=[ddb, dtb]
  endif
  widget_control, los_tlb[i], /realize, tlb_set_xoffset = dir_offset[0, i], $
            tlb_set_yoffset = dir_offset[1,i]
  xmanager, 'ntof', los_tlb[i], /no_block
endfor
if keyword_set(readdata) then readntof, readdata
end
