; ------------------------------------------------------------------------
; $Id$
; ------------------------------------------------------------------------
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
;+
; NAME:
;       NTOF_FIT
;
; PURPOSE:
;       Fit NTOF data with velocity and bang time as parameters
;
; CATEGORY:
;       Data analysis
;
; CALLING SEQUENCE:
;
;       NTOF_FIT [, TYPE]
;
; INPUTS:
;       TYPE:	If specified indicates the type of fit
;               TYPE values are bit wise but only the first set bit is utilized.
;               Type = 1 - Fit DD neutron data
;                          Parameters: T0, Energy_DD
;               Type = 2 - Fit DT neutron data
;                          Parameters: T0, Energy_DT
;               Type = 4 - Fit DD and DT neutron data
;                          Parameters: T0, Energy_DD, Energy_DT
;               Type = 8 - Not Implemented
;                          Fit DD and DT neutron data with different energies
;                            for each Line-of-sight.
;               where T0 is the bang time.
;               Default:  If keywords are set, ignore TYPE
;                         If no keywords are set, TYPE = 4.
;	
; KEYWORD PARAMETERS:
;       WDF:        WDF array number for the fit data
;                   Default:  WDF = GET_MAXWDF() - 3
;                             WDF = WDF < (GET_MAXWDF() -3)
;       BANG_TIME:  Calculated time for neutron creation (ns)
;       DD_ENERGY:  Energy corresponding to the best fit of DD neutrons
;                   for the requested fit (eV)
;       DT_ENERGY:  Energy corresponding to the best fit of DT neutrons 
;                   for the requested fit (eV)
;       PLOTIT:     Plot the data and the fit.
;       QUIET:      If set, do not print fit values.
;   TYPE Keywords: (These keywords are ignored if TYPE is provided.)
;       DEUTERIUM:  Same as TYPE = 1 
;       TRITIUM:    Same as TYPE = 2
;       ALL:        Same as type = 4
;
; OUTPUTS:
;       Best fit neutron energies and creation times are returned and printed
;
; OPTIONAL OUTPUTS:
;       Plots of the data and  the fit are provided
;
; COMMON BLOCKS:
;       The neutron time of flight data is taken from the NTOF common block.
;
; RESTRICTIONS:
;	Neutron peaks must be specified for at least 2 detectors for the same
;       neutron type
;
; PROCEDURE:
;       WDFIT is used for for type 1 and type 2 fits.
;       Derived formulas are used for type 4 fits.
;       The error is defined as:
;          ERROR = TOTAL((TDD[i]-T0-D[i]/VDD)^2) + TOTAL((TDT[i]-T0-D[i]/VDT)^2)
;          where T0, VDD and VDT are fit parameters
;                TDD and TDT are times for detector i for dd and dt neutrons
;                D is the distance to detector i
;
; EXAMPLE:
;	Calculate the average neutron energy for dd neutrons and store in EDD
;         NTOF_FIT, 1, dd_energy = EDD, /plotit
;
; MODIFICATION HISTORY:
; 	Written by:	L. P. Mix, Jr., July 20, 2007
;
;-

 Pro NTOF_FIT, type, WDF= wdf, bang_time= bangtime, dd_energy = ddenergy, $
               dt_energy = dtenergy, plotit = plotit, quiet = quiet, $
               deuterium = deuterium, tritium = tritium, all = all
;
on_error, 1
@ntof_common.dat
;
 if n_elements(ntof_init) eq 0 then begin
   print, '% NTOF_FIT: NTOF must be initialized and peaks defined'
   return
 endif
 lc_deut = 0
 lc_trit = 0
 dtenergy = 0.0
 ddenergy = 0.0
 bangtime = 0.0
 maxwdf = get_maxwdf()
 if n_elements(wdf) eq 0 then wdf = maxwdf-3 else wdf = wdf <  (maxwdf-3)
 if (wdf le 0) then wdf = maxwdf-3

 if n_elements(type) gt 0 then begin
   if type le 0 then $
     print, '% NTOF_FIT: TYPE parameter must be greater than zero' $
     else if (type mod 2) gt 0 then lc_deut = 1 $
     else if (type/2 mod 2) gt 0 then lc_trit = 1 $
     else if (type/4 mod 2) gt 0 then begin
     lc_trit = 1
     lc_deut = 1
   endif
 endif
 if lc_trit eq 0 and lc_deut eq 0 then begin
   if keyword_set(deuterium) then lc_deut = 1 else $
   if keyword_set(tritium) then lc_trit = 1 else $
   if keyword_set(all) then begin
     lc_trit = 1
     lc_deut = 1
   endif
 endif
 if lc_trit eq 0 and lc_deut eq 0 then begin
   if not keyword_set(quiet) then $
   print, '% NTOF_FIT: No valid type set.  TYPE = 4.'
   lc_trit = 1 
   lc_deut = 1
 endif
 npd = 0
 npt = 0
 if lc_deut eq 1 then begin
   np = 0
   xd = make_array(n_elements(dd_time))
   yd = xd
   
   for i = 0, numb_los-1 do for j = 0, numb_detect[i]-1 do begin
     if dd_time[j, i].time gt 0 then begin
       yd[np]= dd_time[j, i].time
       xd[np]= detect_dist[j,i]
       np = np+1
     endif
   endfor
   if np lt 2 then lc_deut = 0 else begin
     if n_elements(xd) gt np then begin
       xd = xd[0:np-1]  
       yd = yd[0:np-1]  
     endif
     npd = np
     ii = sort(xd)
     xd = xd[ii]
     yd = yd[ii]
   endelse
 endif
 if lc_trit eq 1 then begin
   np = 0
   xt = make_array(n_elements(dt_time))
   yt = xt

   for i = 0, numb_los-1 do for j = 0, numb_detect[i]-1 do begin
     if dt_time[j, i].time gt 0 then begin
       yt[np]= dt_time[j, i].time
       xt[np]= detect_dist[j,i]
       np = np+1
     endif
   endfor
   if np lt 2 then lc_trit = 0 else begin
     if n_elements(xt) gt np then begin
       xt = xt[0:np-1]
       yt = yt[0:np-1]
     endif
     npt = np
     ii = sort(xt)
     xt = xt[ii]
     yt = yt[ii]
   endelse
 endif
 if (lc_deut eq 0) && (lc_trit eq 0) then begin
   print,  '% NTOF_FIT: Must have at least 2 deuterium or tritium points to fit.'
   return
 endif
 if npd + npt lt 4 then begin
   Print, '% NTOF_FIT: Fit requires a minimum of 4 points.'
   return
 endif

 if lc_deut eq 1 and lc_trit eq 0 then begin
   if npd le 2 then begin
      print,  '% NTOF_FIT: Least square fit cannot be done with less than 3 points.'
      return
   endif else begin
     ydave = total(yd)/npd
     coef = svdfit(xd, yd-ydave, 2, yfit = yfit, singular = singular, sigma = sigma)
     ddenergy = energ(1.0e9/coef[1], /neu)
     dd_error =  energ(1.0e9/(coef[1]-sigma[1]), /neu)- ddenergy
     bangtime = coef[0] + ydave
     yfit = yfit+ydave
;    must have 3 or more points to save
     i2w, xd, yfit, wdf, clab= 'Deuterium Fit', xlab='Distance(m)', ylab = 'Time(ns)', /xydata
     i2w, xd, yd, maxwdf, clab= 'Deuterium data', xlab='Distance(m)', ylab = 'Time(ns)', /xydata
     if keyword_set(plotit) then plo, [maxwdf, wdf], /over, /color, /nl, psym = [11, 12]
   endelse
   lsfit_param[0, 0:1] = [bangtime, ddenergy]
   if not keyword_set(quiet) then begin
     print, ' Fit Parameters from deuterium data'
     print, '   Neutron Energy: '+strtrim(ddenergy*1e-6, 2)+' MeV'
     print, '   Estimated fit error: '+strtrim(dd_error*1e-6, 2)+' MeV'
     print, '   Creation time: '+strtrim(bangtime, 2)+' ns'
     print, '   Creation time error: '+strtrim(sigma[0], 2)+' ns'
   endif
   return
 endif
 if lc_deut eq 0 and lc_trit eq 1 then begin
   if npt le 2 then begin
     print,  '% NTOF_FIT: Least square fit cannot be done with less than 3 points.'
     return
   endif else begin
     ytave = total(yt)/npt
     coef = svdfit(xt, yt-ytave, 2, yfit = yfit, singular = singular, sigma = sigma)
     dtenergy = energ(1.0e9/coef[1], /neu)
     dt_error =  energ(1.0e9/(coef[1]-sigma[1]), /neu)- dtenergy
     bangtime = coef[0] +ytave
     yfit = yfit +ytave
     i2w, xt, yfit, wdf, clab= 'Tritium Fit', xlab='Distance(m)', ylab = 'Time(ns)', /xydata
     i2w, xt, yt, maxwdf, clab= 'Tritium data', xlab='Distance(m)', ylab = 'Time(ns)', /xydata
     if keyword_set(plotit) then plo, [maxwdf, wdf], /over, /color, /nl, psym = [11, 12]
   endelse
   if not keyword_set(quiet) then begin
     print, ' Fit Parameters from tritium data'
     print, '   Neutron Energy: '+strtrim(dtenergy*1e-6, 2)+' MeV'
     print, '   Estimated fit error: '+strtrim(dt_error*1e-6, 2)+' MeV'
     print, '   Creation time: '+strtrim(bangtime, 2)+' ns'
     print, '   Creation time error: '+strtrim(sigma[0], 2)+' ns'
   endif
   lsfit_param[1, *]=[bangtime, 0.0, dtenergy]
   return
 endif
; Fit both Deuterium and Tritium data
 nt = npd+npt
 ydtave = (total(yd)+total(yt))/nt
 yd = yd - ydtave
 yt = yt - ydtave
 sxd = total(xd)
 syd = total(yd)
 sxt = total(xt)
 syt = total(yt)
 sxyd = total(xd*yd)
 sxyt = total(xt*yt)
 sxd2 = total(xd*xd)
 sxt2 = total(xt*xt)
 tb = syd +syt
 bangtime = (syd+ syt-sxyd*sxd/sxd2-sxyt*sxt/sxt2)/(nt-sxd^2/sxd2-sxt^2/sxt2)
 kd =(sxyd-bangtime *sxd)/sxd2
 kt =(sxyt-bangtime *sxt)/sxt2
 kd =(sxyd-bangtime *sxd)/sxd2
 bangtime = bangtime+ydtave
 ddenergy = energ(1.0e9/kd, /neu)
 dtenergy = energ(1.0e9/kt, /neu)
 lsfit_param[2, *]=[bangtime, ddenergy, dtenergy]
 if not keyword_set(quiet) then begin
       print, ' Fit Parameters from deuterium and tritium data'
       print, '   DD Neutron Energy: '+strtrim(ddenergy*1e-6, 2)+' MeV'
       print, '   DT Neutron Energy: '+strtrim(dtenergy*1e-6, 2)+' MeV'
       print, '   Creation time: '+strtrim(bangtime, 2)+' ns'
 endif
 yfitd = bangtime+ kd*xd
 yfitt = bangtime+ kt*xt
 lcplot =keyword_set( plotit)
 if npd gt 2 then begin
   i2w, xd, yfitd, wdf,  clab= 'Deuterium Fit', xlab='Distance(m)', ylab = 'Time(ns)', /xydata
   i2w, xd, yd+ydtave, maxwdf-1, clab= 'Deuterium data', xlab='Distance(m)', ylab = 'Time(ns)', /xydata
 endif else lcplot = 0
 if npt gt 2 then begin
   i2w, xt, yfitt, wdf+1,  clab= 'Tritium Fit', xlab='Distance(m)', ylab = 'Time(ns)', /xydata
   i2w, xt, yt+ydtave, maxwdf, clab= 'Tritium data', xlab='Distance(m)', ylab = 'Time(ns)', /xydata
 endif else lcplot = 0
 if keyword_set(lcplot) then plo, [wdf, wdf+1, maxwdf-1, maxwdf], /over, /color, /nl, psym = [11, 12]
 
end
