; ------------------------------------------------------------------------ 
; $Id: createpdb.pro,v 1.4 2007/06/06 19:44:55 lpmix Exp $
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
  pro createpdb, p1in, p2, error = error, fileid = fileid
;+
;  Open a PDB file.  The current file pointer is set to this file id and 
;  the dataset names are stored.  Same as openpdb with mode = 2 or 'WRITE'
;
;  format: CREATEPDB , File [, Fileid ] [, Error = error] [, FILEID = fileid]
;  where:
;    File   - name of the PDB file to be opened.  
;             NOTE: FULFNAM procedure is used to handle environment variables
;                   See FULFNAM for details
;             NOTE: FIND_NEWFILE is used to insure a new file.
;    Fileid - user specified file id 
;             (If fileid is specified, it will be used as the file id.  If 
;             fileid is not provided or is zero, a value will be 
;             provided; default value is the first unused integer greater
;             than 0.  If fileid is provided but has been used for another 
;             file, the program will provide a default value.
;             If positional parameter is valid, keyword will be ignored.
;    Error  - Error parameter
;             If this error is zero, no error encountered.
;             If Error = -99, file not found
;
;  Examples:
;    Create a PDB file, 'JUNK.pdb' with the default file id
;      createpdb, 'junk.pdb'
;-
;
;
      on_error, 1
;
@pdbcom.dat
; PDB common block
; 
; initpdb   - Variable used to indicate if initialization has occurred and then
;            the machine type.
; pdb_exe   - Dynamically loaded executable for PDB/PACT routines.
;
;    The following two parameters are set in init_pdb 
; Maxfid    - Maximum number of file families
; Max_pdb_file - Maximum number of PDB files for a given file family
;             for example:
;                Assume p00, p01, p02
;             p is the file family and 3 is the number of files in the family
;
; Curfid    - Index of the current family of files
;
; ufid      - Array of user file id's
;             ufid = make_array(/long, maxfid)
;             ufid[curfid] is the current file id
; n_pdbfile - number of files for a given ufid.
;             file_info goes from curfid*max_pdb_file to 
;                    curfid*max_pdb_file + n_pdbfile-1
; family_info-pointer to a structure for a given family.
;             Tags for the structure are:
;             file       - Initial file
;             nfile      - Number of files
;             size       - Size of space array [nz, nr]         
;             ntime      - Number of times
;             loct       - Pointer to times for each file - loct[nfile+1]
;                          loct[0]=0, loct[nfile] = nt
;                            file m contains time loct[ 
;             time       - Time array
;             dimensions - Dimensions for the total array
;             lcparam[nfile] - 15 if file contains zt(1), rt(2), ireg(4) 
;                                  and time(8) data
; file_info - pointer to a structure for a given file.
;             Tags for the structure are:
;             file       - Filename
;             pds_fileid - C stream pointer for pdb
;             ndset      - Number of datasets
;             names      - Variable names (String array)
;             length     - Array of the number of elements for each variable.
;             type       - Variable type (String array)
;             ndim       - Number of dimensions
;             source
;             dimensions - array of the dimensions
;             zt         - Z-value
;             rt         - Radius value
;             ireg       - Region information (material numbers)
;             time       - Array of time values 
;             source     - data source
;                          (for example zt@history has history as source)
;             good       - regions where values are good
;                          =-1 if all good 
;
; common pdbcommon, initpdb, pdb_exe, maxfid, max_pdb_file, ufid, curfid, $
;      n_pdbfile, family_info, file_info
;
      COMMON savepdbpicker, oldfilter, oldpath
;
;     check on the number of parameters
;
      nparams = n_params()
      if nparams eq 0  then begin
print, '% Format: CREATEPDB [, File] [, Fileid ] [, Error = error]'
print, '              [, FILEID = fileid] 

        return
      endif
;
;     check to see if pdb has been initialized
;
      if (n_elements(initpdb) eq 0) then init_pdb else $
        if (initpdb eq 0) then init_pdb
      if initpdb eq 0 then begin
        print, 'PDB libraries are not supported by this Operating System'
        return
      endif
;
;     be sure we have a file name
;
      error = -13
      nparams = nparams > 1
      if (size(p1in, /type) ne 7) then p1 = '' else p1 = strtrim(p1in, 2)
      cd, current = current
      if strlen(p1) lt 1 then begin
        IF (N_ELEMENTS(PATH) EQ 0 and n_elements(oldpath) eq 0) THEN BEGIN
          if initpdb lt 3 then path = current+'/' else path = current
        endif else if n_elements(path) eq 0 then path = oldpath
        if n_elements(filter) eq 0 and n_elements(oldfilter) eq 0 then $
          filter = '*' else if n_elements(filter) eq 0 then filter = oldfilter
        p1 = dialog_pickfile( path = path, get_path =tempath, filter = filter, $
          title= 'Select PDB file to read')
        p1 = strtrim(p1, 2)
        if strlen(p1) lt 1 then return
        if n_elements(tempath) gt 0 then oldpath = tempath
      endif
;
      mode = 2

      vers = strupcase(strmid(!version.os, 0, 3))
      if (vers eq 'WIN') then begin
        if strpos(p1, '\') lt 0 then p1 = current+'\'+p1
      endif else if (vers eq 'MAC') then begin
        if strpos(p1, ':') lt 0 then p1 = current+p1
      endif else begin
        temp = p1
        FULFNAM, temp, p1
;
;       get full path name for unix systems
;
        directory = p1
        get_fulldirname, directory, file
        if strlen (directory(0)) eq 0 then begin
          print, ' Unable to open file: ', p1
          return
        endif
        p1 = directory(0) +'/'+ file
      endelse
      
      p1 = find_newfile(p1, /quiet)

; find the next slot for a file
      next_fid = (where (ufid eq 0))[0]
      if next_fid lt 0 then begin 
  print, '% CREATEPDB: Maximum number of files are open. Maximum =', maxfid
        print, '       No available file ids. Close a file.
        return
      endif
      lstr = 192L
      error = 0L
  
      if strlen(p1) gt lstr then begin
        Print, ' Error in CREATEPDB, length of file greater than ' $
         , strtrim (string(lstr), 2) , ' characters'
        Print, ' Contact PFIDL administrator, or use fewer characters'
        print, 'File: ', p1
        return
      endif 
      ffiles = p1
      nfiles = 1l
;   Now open the files
; user specified file id
    if n_elements (p2) eq 0 then begin
      if n_elements(fileid) gt 0 then p2 = long(fileid[0]) else p2 = 0L
    endif else p2 = long(p2[0])
    if p2 eq 0 then p2= 1+next_fid
    ok = where (p2 eq ufid, nok)
    while (nok gt 0) do begin
      p2 = p2+1
      ok = where (p2 eq ufid, nok)
    endwhile
    curfid = next_fid
    offset = curfid*max_pdb_file
    loct = make_array(/long, nfiles+1)
    maxt = 0L
    nz0 = 0L
    nr0 = 0L
    nt = 0L
    ifile = offset
    initial_file = ffiles[0]
    lcparam = make_array(nfiles, /int)
    i = 0L
    open_pdb_doit, p1, offset, mode, ntimes, times, ssize, lcpar, error=error
    if error ne 0 then begin
        print, '% CREATEPDB: Error reading file ', p1
        print, '      Error: ', strtrim (error,2)
        return
    endif
    nz0 = 0
    nr0 = 0

    lcparam[i] = lcpar
      j = 1l
      loct[j]= loct[i]+0
      if loct[j] gt maxt then begin
        maxt= maxt+maxt
        ttime=[ttime, ttime]
      endif
      if ntimes gt 0 then ttime[loct[i] : loct[j]-1]= times
    nt = 0
    ttime = 0
    print, '% CREATEPDB: File opened with a FID = ', strtrim(p2,2)
    print, ' File: ', initial_file
    print, ' Total of ', strtrim(nfiles, 2), ' files'
    n_pdbfile[curfid]=nfiles
    ftimem = 0
    ndim = 0
    type = 0
    dim = 0

    nttimem = nt-1
    str = {file:initial_file, nfile: nfiles, size:ssize, ntime: nt, $
           loct:loct, time :ttime , dimensions:dim, lcparam:lcparam  }
    ptr_free, family_info[curfid]
    family_info[curfid] = ptr_new(str, /no_copy)
    ufid (curfid) = p2
    ntimes = 0
    times = 0.0
    str = {file:initial_file, pds_fileid: lcpar, ndset: 0L, names:'', $
          length:0l, type:'', ndim:0L, source:'', dimensions:0, $
         zt: 0L, rt:0L, ireg: 0L, time: 0.0, good:0L }
    ifile = offset*curfid
    ptr_free, file_info[ifile]
    file_info [ifile]= ptr_new(str, /no_copy)
 
    return
    end
