; ------------------------------------------------------------------------ 
; $Id: openpdb.pro,v 1.8 2009/03/31 14:04:35 lpmix Exp $
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;

  pro openpdb, p1in, p2, error = error, path = path, filter = filter, $
                mode = modein, fileid = fileid, ndset = ndset, help = help, $
                names = names, single = single, multiple = multiple, $
                all_files = all_files
;+
;  Open a PDB file.  The current file pointer is set to this file id and 
;  the dataset names are stored.
;  format: OPENPDB [, File] [, Fileid ] [, Error = error] [, NDSET =ndset]
;                  [, /ALL_FILES] [, /MULTIPLE]
;                  [, PATH = path] [, FILTER = filter] [, HELP = help]
;                  [, NAMES = names] [, MODE = mode] [, FILEID = fileid]
;  where:
;    File   - name of the PDB file to be opened.  
;             If file is omitted or undefined or is not a string,
;             then PICKFILE will be used to select a file.
;             The initial path in PICKFILE is the current directory or
;             the PATH to the last file selected.
;             
;             NOTE: PATH and FILTER may be used to set the initial path and
;                   filter for PICKFILE.
;             NOTE: FULFNAM procedure is used to handle environment variables
;                   See FULFNAM for details
;    Fileid - user specified file id 
;             (If fileid is specified, it will be used as the file id.  If 
;             fileid is not provided or is zero, a value will be 
;             provided; default value is the first unused integer greater
;             than 0.  If fileid is provided but has been used for another 
;             file, the program will provide a default value.
;             If positional parameter is valid, keyword will be ignored.
;    Mode   - Open mode: Read Only, Write (Create), Append (READ/WRITE) 
;             (Default = Read Only)
;             Mode may be a number (1, 2 or 3) or a string.
;                (Read, Write, Append)
;             If a string is used only the first letter will be checked.
;             If the first letter is not a r, w, or a then mode = 1 (READ_ONLY).
;    ALL_FILES - If set, consider all files as a set.
;             Files must be of the format: String + 1, 2 or 3 numbers
;    MULTIPLE - Same as ALL_FILES.  (Consistent with HISPLT keyword)
;             (Note: /A is the same as /MU)
;    Error  - Error parameter
;             If this error is zero, no error encountered.
;             If Error = -99, file not found
;    help   - Print a help text
;    PATH   - Path to the current file 
;             Default is the current directory in the first call to OPENPDB but
;             the default path is changed to the final path selected is 
;             retained as the default in the next call to openpdb.
;    FILTER - File filter - The default may be set in USERQSINIT; if undefined
;             the default is '*'.  The default filter is set only with
;              USERQSINIT or SETDEFAULT.
;    NDSET  - Number of datasets
;    NAMES  - A string array of dataset names
;    SINGLE - If set, a single file will be opened.
;
;  Examples:
;    Open a PDB file with the default file id
;      openpdb
;    Open a PDB file, 'users/test', with the default file id
;      openpdb, '/users/test'
;    Open a file to be selected from a menu with a file id of 14
;         openpdb, ' ', 14
; <or>    openpdb, 0, 14
; <or>    openpdb, file= 14
;    Determine keywords and parameters for openpdb
;      openpdb, /h
;-
;
;
;
; PDB common block
; 
; initpdb   - Variable used to indicate if initialization has occurred and then
;            the machine type.
; pdb_exe   - Dynamically loaded executable for PDB/PACT routines.
;
;    The following two parameters are set in init_pdb 
; Maxfid    - Maximum number of file families
; Max_pdb_file - Maximum number of PDB files for a given file family
;             for example:
;                Assume p00, p01, p02
;             p is the file family and 3 is the number of files in the family
;
; Curfid    - Index of the current family of files
;
; ufid      - Array of user file id's
;             ufid = make_array(/long, maxfid)
;             ufid[curfid] is the current file id
; n_pdbfile - number of files for a given ufid.
;             file_info goes from curfid*max_pdb_file to 
;                    curfid*max_pdb_file + n_pdbfile-1
; family_info-pointer to a structure for a given family.
;             Tags for the structure are:
;             file       - Initial file
;             nfile      - Number of files
;             size       - Size of space array [nz, nr]         
;             ntime      - Number of times
;             loct       - Pointer to times for each file - loct[nfile+1]
;                          loct[0]=0, loct[nfile] = nt
;                            file m contains time loct[ 
;             time       - Time array
;             dimensions - Dimensions for the total array
;             lcparam[nfile] - 15 if file contains zt(1), rt(2), ireg(4) 
;                                  and time(8) data
; file_info - pointer to a structure for a given file.
;             Tags for the structure are:
;             file       - Filename
;             pds_fileid - C stream pointer for pdb
;             ndset      - Number of datasets
;             names      - Variable names (String array)
;             length     - Array of the number of elements for each variable.
;             type       - Variable type (String array)
;             ndim       - Number of dimensions
;             source
;             dimensions - array of the dimensions
;             zt         - Z-value
;             rt         - Radius value
;             ireg       - Region information (material numbers)
;             time       - Array of time values 
;             source     - data source
;                          (for example zt@history has history as source)
;             good       - regions where values are good
;                          =-1 if all good 
@pdbcom.dat
;
; common pdbcommon, initpdb, pdb_exe, maxfid, max_pdb_file, ufid, curfid, $
;      n_pdbfile, family_info, file_info
;
on_error, 1
      COMMON savepdbpicker, oldfilter, oldpath
;
;     check on the number of parameters
;
      nparams = n_params()
      if nparams eq 0 and keyword_set(help) then begin
print, '% Format: OPENPDB [, File] [, Fileid ] [, Error = error]'
print, '              [, MODE = mode] [, PATH = path] [, FILTER = filter]'
print, '              [, NDSET = ndset] [, FILEID = fileid] [, /SINGLE]'
print, '              [, /ALL_FILES] [, /MULTIPLE]'

        return
      endif
;
;     check to see if pdb has been initialized
;
      if (n_elements(initpdb) eq 0) then init_pdb else $
        if (initpdb eq 0) then init_pdb
      if initpdb eq 0 then begin
        print, 'PDB libraries are not supported by this Operating System'
        return
      endif
;
;     be sure we have a file name
;
      error = -13
      nparams = nparams > 1
      if (size(p1in, /type) ne 7) then p1 = '' else p1 = strtrim(p1in, 2)
      cd, current = current
      if strlen(p1) lt 1 then begin
        IF (N_ELEMENTS(PATH) EQ 0 and n_elements(oldpath) eq 0) THEN BEGIN
          if initpdb lt 3 then path = current+'/' else path = current
        endif else if n_elements(path) eq 0 then path = oldpath
        if n_elements(filter) eq 0 and n_elements(oldfilter) eq 0 then $
          filter = '*' else if n_elements(filter) eq 0 then filter = oldfilter
        p1 = dialog_pickfile( path = path, get_path =tempath, filter = filter, $
          title= 'Select PDB file to read')
        p1 = strtrim(p1, 2)
        if strlen(p1) lt 1 then return
        if n_elements(tempath) gt 0 then oldpath = tempath
      endif
;
      vers = strupcase(strmid(!version.os, 0, 3))
      if (vers eq 'WIN') then begin
        if strpos(p1, '\') lt 0 then p1 = current+'\'+p1
      endif else if (vers eq 'MAC') then begin
        if strpos(p1, ':') lt 0 then p1 = current+p1
      endif else begin
        temp = p1
        FULFNAM, temp, p1
;
;       get full path name for unix systems
;
        directory = p1
        get_fulldirname, directory, file
        if strlen (directory(0)) eq 0 then begin
          print, ' Unable to open file: ', p1
          return
        endif
        p1 = directory(0) +'/'+ file
      endelse

; find the next slot for a file
      next_fid = (where (ufid eq 0))[0]
      if next_fid lt 0 then begin 
        print, '% OPENPDB: Maximum number of files are open. Maximum =', maxfid
        print, '       No available file ids. Close a file.'
        return
      endif

      if n_elements (modein) eq 0 then mode = 1L else begin
        mode = modein
        if (size(mode, /type) eq 7) then begin
          mode = strupcase(strmid(strtrim(mode, 1), 0, 1))
          mode = (where (mode eq ['R', 'W', 'A']) > 0)[0] + 1L
        endif else begin
          mode = long(mode[0])
          if ((mode lt 1) or (mode gt 3)) then mode = 1L
        endelse
      endelse
      lstr = 192L
      error = 0L
  
      if strlen(p1) gt lstr then begin
        Print, ' Error in OPENPDB, length of file greater than ' $
         , strtrim (string(lstr), 2) , ' characters'
        Print, ' Contact PFIDL administrator, or use fewer characters'
        print, 'File: ', p1
        return
      endif 
      dummy = (file_search(p1))[0]
      lendum = strlen(dummy)
      if lendum gt 0 then ffiles = dummy else begin
        print, '% OPENPDB: Unable to locate file: ', p1
        return
      endelse
      
      if keyword_set (single) then ffiles = ffiles[0] else begin
        nnum = 0
        ind = strmid(dummy, lendum-1, 1)
        if ((byte(ind))[0] ge 48 and (byte(ind))[0] le 57) then begin
          nnum = 1
          ind = strmid(dummy, lendum-2, 1)
          if ((byte(ind))[0] ge 48 and (byte(ind))[0] le 57) then begin
            nnum = 2
            ind = strmid(dummy, lendum-3, 1)
            if ((byte(ind))[0] ge 48 and (byte(ind))[0] le 57) then nnum = 3
          endif
        endif
        if nnum eq 0 then  ffiles = ffiles[0] else begin
          if nnum eq 1 then addit = '[0-9]' else $
            addit = (nnum eq 2) ? '[0-9][0-9]' : '[0-9][0-9][0-9]'
          ffiles = file_search(strmid(dummy, 0, lendum-nnum)+addit, $
             /mark_directory, /test_read)
        endelse
      endelse 
      nfiles = n_elements (ffiles)
      if ((nfiles gt 1) and ((not keyword_set(multiple)) and  $
        (not keyword_set(all_files)))) then begin
        ok = get_array_var(ffiles, /select, title = 'Select Files')
        nfiles = n_elements (ok)
        if ok(0) lt 0 then begin
          print, '% OPENPDB: No files selected for opening.'
          error = -3
          return
        endif
        if nfiles lt n_elements(ffiles) then ffiles = ffiles[ok]
      endif
;   Now open the files
; user specified file id
    if n_elements (p2) eq 0 then begin
      if n_elements(fileid) gt 0 then p2 = long(fileid[0]) else p2 = 0L
    endif else p2 = long(p2[0])
    if p2 eq 0 then p2= 1+next_fid
    ok = where (p2 eq ufid, nok)
    while (nok gt 0) do begin
      p2 = p2+1
      ok = where (p2 eq ufid, nok)
    endwhile
    curfid = next_fid
    offset = curfid*max_pdb_file
    loct = make_array(/long, nfiles+1)
    maxt = 0L
    nz0 = 0L
    nr0 = 0L
    nt = 0L
    ifile = offset
    initial_file = ffiles[0]
    lcparam = make_array(nfiles, /int)
    for i = 0, nfiles-1 do begin
      p1 = ffiles[i]
      open_pdb_doit, p1, i+offset, mode, ntimes, times, ssize, lcpar, error=error

      if error ne 0 then begin
        print, '% OPENPDB: Error reading file ', p1
        return
      endif
      if n_elements (ssize) eq 0 then begin
        nz0 = 0
        nr0 = 0
      endif else begin
        nz0 = ssize[0]
        nr0 = ssize[1]
      endelse

      if i eq 0 then begin
        maxt = (ntimes+1)*nfiles > 128L
        ttime = make_array(maxt)
      endif
      lcparam[i] = lcpar
      j = i+1
      loct[j]= loct[i]+ntimes
      while (loct[j] gt maxt) do begin
        maxt= maxt+maxt
        ttime=[ttime, ttime]
      endwhile
      if ntimes gt 0 then ttime[loct[i] : loct[j]-1]= times
    endfor
    loct = loct[0:j]
    nt = loct[j]
    ttime = temporary(ttime[0:loct[j]-1])
    print, '% OPENPDB: File opened with a FID = ', strtrim(p2,2)
    print, ' File: ', initial_file
    print, ' Total of ', strtrim(nfiles, 2), ' files'
    n_pdbfile[curfid]=nfiles
    dim = *file_info[offset]
    ftimem = n_elements(dim.time)-1
    ndim = dim.ndim
    type = dim.type
    dim = dim.dimensions

    nttimem = nt-1
    if nfiles gt 1 then begin
      ok = where (ndim eq 1 and type ne 'char', nok)
      if nok gt 0 then begin
        okc= where ( (dim[1, 0, ok]-dim[0, 0, ok]) eq ftimem, nok)
        if nok gt 0 then dim[1, 0, ok[okc]] = dim[0, 0, ok[okc]] + nttimem
      endif
      ok = where (ndim eq 2, nok)
      if nok gt 0 then begin
        okc= where ( (dim[1, 1, ok]-dim[0, 1, ok]) eq ftimem, nok)
        if nok gt 0 then dim[1, 1, ok[okc]] = dim[0, 1, ok[okc]] + nttimem
      endif
      ok = where (ndim eq 3, nok)
      if nok gt 0 then begin
        okc= where ( (dim[1, 2, ok]-dim[0, 2, ok]) eq ftimem, nok)
        if nok gt 0 then dim[1, 2, ok[okc]] = dim[0, 2, ok[okc]] + nttimem
      endif
      ok = where (ndim eq 4, nok)
      if nok gt 0 then begin
        okc= where ( (dim[1, 3, ok]-dim[0, 3, ok]) eq ftimem, nok)
        if nok gt 0 then dim[1, 3, ok[okc]] = dim[0, 3, ok[okc]] + nttimem
      endif
    endif
    str = {file:initial_file, nfile: nfiles, size:ssize, ntime: nt, $
           loct:loct, time :ttime , dimensions:dim, lcparam:lcparam  }
    ptr_free, family_info[curfid]
    family_info[curfid] = ptr_new(str, /no_copy)
    ufid (curfid) = p2
 
    return
    end
