; ------------------------------------------------------------------------ 
; $Id: read_pdb.pro,v 1.7 2010/04/13 16:16:55 lpmix Exp $
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;

 function read_pdb , array, ier, fileid = fileid, ijklist = ijklist,$
      dimensions = dimensions,  type = type

;+
;  Reads a PDB dataset into an IDL array.
;  This is intended as a utility function.
;
;  Format:
;     ARRAY =  READ_PDB( Dataset# [, IER] [, FileID = FileID]
;                      [, TYPE = type]  [, IJKLIST = ijklist])
;  where:
;      ARRAY    - IDL array of the stored values
;                 The following types are supported:
;                   CHAR, SHORT, DOUBLE, FLOAT, INTEGER, LONG
;      Dataset# - Dataset 
;                 This is the number of the dataset sorted alphabetical
;                   corresponding to the dir command.
;                 or a unique string corresponding to the desired dataset.
;      IER      - If zero, read successful.
;      FileID   - PDB file id
;      TYPE     - String for the dataset type being returned
;                 valid types are integer, float, double, and char
;      IJKLIST  - Do indices for i, j and k dimensions 
;                 3 elements per dimension: start, stop, skip
;-
;
@pdbcom.dat

;     start the process
      on_error, 1
;
      nparams = n_params()
      ier =1L
      if nparams eq 0 then begin
        print,'% Format: '
        print,'      Array = READ_PDB( Dataset#, IER, fileid = fileid)'
        return, 0
      endif
      if n_elements(initpdb) eq 0 then initpdb = 0
      if initpdb eq 0 then begin
        print, '% READ_PDB:  No PDB files are open.'
        return, 0
      endif
      

      if n_elements(array) eq 0 then begin
        print, '% READ_PDB:  Dataset number or name must be presented.'
        return, 0
      endif else array= array[0]
      
      If curfid lt 0 then begin
        print, '% READ_PDB:  No valid PDB files are open.'
        return, 0
      endif
      ier =0L
      if n_elements (fileid) gt 0 then setpdb, fileid[0] , ier = ier
      if ier ne 0 then begin
        print, '% READ_PDB: Unable to set file pointer:' , fileid[0]
        showpdb 
        return, 0
      endif

      offset = curfid*max_pdb_file
      info = *file_info[offset]
      fam_info = *family_info[curfid]
;     array is the dataset designation
      atype = size (array, /type)
      nfile = fam_info.nfile
      names = info.names
      if atype le 4 then begin
        ind = round(array[0])-1
        nel = n_elements (names)
        if ind lt 0 or ind ge nel then begin
          print, '% READ_PDB:  Dataset number must be in the range of 1 to ', $
            strtrim(nel, 2)
          return, 0
        endif
      endif else begin
        arrayspec = array
        ind = get_match(arrayspec, names, nel= nel)
        if nel ne 1 then begin
          arrayspec = '^'+array+'$'
          ind = get_match(arrayspec, names, nel= nel)
          if nel ne 1 then begin
            print,'% READ_PDB: Unable to find a unique string for string: ', arrayspec
            if nel eq 0 then print, 'No Matches found' else begin
              print, '   String matches are (dataset - name):'
              for i = 0, nel -1 do print, ind[i], ' - ' , names[ind[i]]
            endelse
            return, 0
          endif
        endif
        ind = ind[0]
      endelse
      ndim = info.ndim [ind]
      lname = 256L
      buff = 0L
      buffi = 0L

;   ***************************
    nijk = n_elements(ijklist)
    if (nijk lt 3*ndim) or (ndim lt 3) then begin
;     Read the entire dataset      
      for ifile = 0, nfile -1 do begin
        if ifile gt 0 then begin
          info = *file_info[offset+ifile]
          names = info.names
          if atype lt 4 then begin
            nel = n_elements (names)
            if ind lt 0 or ind ge nel then begin
            print, '% READ_PDB:  Dataset number must be in the range of 1 to ', $
                strtrim(nel, 2)
              return, 0
            endif
          endif else begin
            arrayspec = '^'+array+'$'
            ind = get_match(arrayspec, names, nel= nel)
            if nel ne 1 then begin
          print,'% READ_PDB: Unable to find a unique string for string: ', arrayspec
              if nel eq 0 then print, 'No Matches found' else begin
                print, '   String matches are (dataset - name):'
                for i = 0, nel -1 do print, ind[i], ' - ' , names[ind[i]]
              endelse
            return, 0
            endif else ind = ind[0]
          endelse
          ndim = info.ndim[ind]
        endif
        vartype = info.type[ind]
        valid =  ['char', 'double', 'float', 'integer', 'long', 'short']
        ok = where (vartype[0] eq valid,  nok)
        if nok eq 0 then begin
          print, '% READ_PDB:  Only the following dataset types are supported:'
          print, '   [char, double, float, integer, long, short], Dataset type: ',  $
                 vartype
          return, 0
        endif
        pid = info.pds_fileid
        p1 = info.file
        if pid le 0 then $
          pdb_open_only, p1, lstr, mode, ndset, pid, error
        len = info.length[ind]
        dim = info.dimensions[*, *, ind]

        if ok[0] eq 0 then buffer = bytarr(len+1) $
          else if  ok[0]  eq 1 then buffer = dblarr(len) $
          else if  ok[0]  eq 2 then buffer = fltarr(len) $
          else if  ok[0]  eq 3 then buffer = lonarr(len) $
          else if  ok[0]  eq 4 then buffer = lonarr(len) $
          else if  ok[0]  eq 5 then buffer = intarr(len)
        if ((ifile eq 0) and (nfile gt 1))then begin
          fdim = fam_info.dimensions[*, *, ind]
          fdel = (fdim(1, *)-fdim(0, *)) + 1
          lbuf = fdel[0]
          for i = 1, n_elements(fdel)-1 do lbuf *= fdel[i]
          buff = make_array(lbuf, value = buffer[0])
          buffmax = lbuf
          dimensions= fdel
        endif
        
        str2byt, names[ind], varb, 256
      
        ier = 0L
        if initpdb eq 1 then begin
; HP
            junk = call_external(PDB_EXE, 'pdb_read', $
              pid, varb, lname, buffer, ier)
        endif else if initpdb eq 2 then begin
; Sun
            junk = call_external(PDB_EXE, 'pdb_read_', $
              pid, varb, lname, buffer, ier)
        endif else if initpdb eq 4 then begin
; WIN
           junk = call_external(PDB_EXE, '_PDB_READ@8', $
              pid, varb, lname, buffer, ier)
        endif

        if ier ne 0 then ier = 0L else ier =1L
        if ok(0) eq 0 then begin
          buffer = strmid(buffer, 0, dim[1, 0]) 
          if pid gt 0 then close_pdb, pid, ier
          info.file = 0L
          ier = 0
          return, buffer
        endif else if n_elements (buffer) eq 1 and ifile eq 0 and nfile eq 1 then begin
; change
;       old version which did not work with a single element in the first file
;       endif else if n_elements (buffer) eq 1 and ifile eq 0 then begin
          buffer = buffer[0]
          if pid gt 0 then close_pdb, pid, ier
          info.file = 0L
          ier = 0
          return, buffer
        endif else if (nfile eq 1) then begin
          if max(dim(0, *)) gt 1 then begin
            print, '% READ_PDB: Some dimensions do not begin at 1'
            print, dim(0, *)
          endif
          del = (dim(1, *)-dim(0, *)) + 1
          buffer = reform(reform(buffer, del,/overwrite))
          if pid gt 0 then close_pdb, pid, ier
          info.file = 0L
          ier = 0
          return, buffer
        endif
        buff[buffi:buffi+len-1] = buffer
        buffi = buffi+len 
        if buffi eq buffmax then begin
          buffer = reform(buff, fdel, /overwrite)
          if pid gt 0 then close_pdb, pid, ier
          info.file = 0L
          ier = 0
          return, buffer
        endif
        if pid gt 0 then close_pdb, pid, ier
        info.file = 0L
      endfor
;     we should never get here - with multiple files we should exit when buffi=buffmax
      buffer = reform(reform(buff, fdel, /overwrite))
      if pid gt 0 then close_pdb, pid, ier
      info.file = 0L
      ier = 0
      return, buffer
    endif
    ifile = 1
    tval = ijklist[nijk-3]-1
    while ( tval ge fam_info.loct[ifile] ) do ifile = ifile +1
      ifile = ifile -1
      ijklist[nijk-3: nijk-2] = tval - fam_info.loct[ifile]+1
      info=*file_info[offset+ifile]
      names = info.names
      atype = size (array, /type)
      vartype = info.type[ind]
      ok = where (vartype[0] eq ['char', 'double', 'float', 'integer'], nok)
        
      if nok eq 0 then begin
        print, '% READ_PDB:  Only the following dataset types are supported:'
        print, '   [char, double, float, integer], Dataset type: ', vartype
        return, 0
      endif
      type = vartype
      dim = info.dimensions[*, *, ind]
      len = (info.length)[ind]
      ndim = (info.ndim[ind])[0]
      ijklist = long (ijklist)
;      pid = info.pds_fileid
      pid = info.pds_fileid
      p1 = info.file
      if pid le 0 then $
        pdb_open_only, p1, lstr, mode, ndset, pid, error
      slen = 32L
      str2byt, vartype, typebyte , slen
      str2byt, names[ind], varb, 256
      len = 1L
      subdim = make_array(ndim, value = 1L)
      for i = 0L , ndim-1 do $
        subdim[i] = ((ijklist[3*i+1]-ijklist[3*i]+1l)/ijklist[3*i+2]) > 1L
      for i = 0L , ndim-1 do len = len*subdim[i]
        
      if ok[0] eq 0 then buffer = bytarr(len+1) $
        else if  ok[0]  eq 1 then buffer = dblarr(len) $
        else if  ok[0]  eq 2 then buffer = fltarr(len) $
        else if  ok[0]  eq 3 then buffer = lonarr(len)

      ier = 0L
      if initpdb eq 1 then begin
; HP
            junk = call_external(PDB_EXE, 'pdb_read', $
              pid, varb, lname, slen, typebyte,  buffer, ijklist, ier)
      endif else if initpdb eq 2 then begin
; Sun
            junk = call_external(PDB_EXE, 'pdb_read_', $
              pid, varb, lname, slen, typebyte,  buffer, ijklist, ier)
      endif else if initpdb eq 4 then begin
; WIN
           junk = call_external(PDB_EXE, '_PDB_READ@8', $
              pid, varb, lname, slen, typebyte,  buffer, ijklist, ier)
      endif
      if ier ne 0 then ier = 0L else ier = 1L
      if ok(0) eq 0 then buffer = strmid(buffer, 0, dim[1, 0]) $
      else if ok[0] ne 0 and ndim gt 0 then begin
        if max(dim(0, *)) gt 1 then begin
             print, '% READ_PDB: Some dimensions do not begin at 1'
             print, dim(0, *)
        endif
        del = (dim(1, *)-dim(0, *)) + 1
        buffer = reform(reform(buffer,subdim,/overwrite))
      endif else if n_elements(buffer) eq 1 then buffer = buffer[0]
      if pid gt 0 then close_pdb, pid, ier
      info.file = 0L
      ier = 0
      return, buffer
      end
