; ------------------------------------------------------------------------ 
; $Id: read_pdb_data.pro,v 1.5 2007/06/06 19:45:14 lpmix Exp $
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
 Function read_pdb_data, dataset, tvalue, lctype, node_values = node_values, $
          ierror = ier, index = index, double = double, clabel = clab, $
          matrix = matrix, no_truncate = no_truncate, VOID_FRAC = void_frac
;+
;  Utility function to read a PDB data structure.
;
;  Format: structure = READ_PDB_DATA(dataset, tvalue, lctype, 
;                  node_values = node_values, CLABEL = clabel, $
;                  ierror = ierror, index = index,  double = double)
;   where 
;    dataset   - indicates a PDB dataset number or
;                a search string for dataset comments.
;                (Not required if GRID is set.)
;        See: GET_DSET for more details
;    TVALUE    - Time value for the dataset.
;                Note: Only single time values will be read.  Read all times and
;                      use plotfe9 to interpolate data between output times.
;                Default: All time values will be read.
;    LCTYPE     -1 Scalar
;                0 structure data
;                n Waveform data (n waveforms)
;              -99 big problem!
;    NODE_VALUES - If set, the values are left on the nodes.
;                Default: Move all node values to the cell and distribute them on
;                     the nodes.  Node values are then the average of the values
;                     on the 4 sides of the nodes.
;                     Exception: ut@history, vt@history are left on the nodes.
;    INDEX     - If TVALUE is given and index is set then the value of TVALUE
;                is assumed to be a grid index in the Time array. If INDEX
;                is set then TVALUE goes from 1 to the maximum number of
;                time steps.
;    DOUBLE    - If data is double precision, the data will be saved as double
;                Default is to change X and Y values to float
;    VOID_FRAC - Variable is ignored unless the dataset is 4-dimensional.
;                Variable is assumed to have 2 spatial dimensions and a fourth
;                dimension corresponding to time.
;                VOID_FRAC varies from 1 to the number of elements in the third
;                dimension.  The READPDB procedure will return only the slice 
;                corresponding to this slice in the third dimension.
;                Note:  Use READ_PDB to get the entire 4-dimensional array.
;                Example:  If F= F[NZ, NR, NV, NT], where NZ, NR and NT are the 
;                spatial and temporal dimensions, then VOID_FRAC = 2 will read
;                FF = F[*, *, 1, *] (Since IDL uses zero based indices) 
;                If TVALUE is provided, then a two dimensional array is returned.
;    CLABEL    - Comment label for WDF arrays
;      
;  All other parameters are contained in the common block.
;-
      on_error, 1
;     add common block
@pdbcom.dat
@qsstrcom.dat
;
;     check to see if pdb and structures have been initialized
;
      if n_elements(initpdb) eq 0 then init_pdb
      if n_elements (structinit) eq 0 then qsstruct_init
;
;
      ier = 1
      lctype = -99
      If curfid lt 0 then begin
        print, '% READ_PDB_DATA:  No valid PDB files are open.'
        return, 0
      endif

      if n_elements (dataset) eq 0 then begin
        print, '% READ_PDB_DATA:  Dataset must be specified.'
        return, 0
      endif
;
      offset = curfid*max_pdb_file
      info = *file_info[offset]
      fam_info = *family_info[curfid]
      dtyp = size(dataset, /type)
      if dtyp eq 7 then begin
        dtyp = get_match(dataset[0], info.names, nel= nel)
        if nel ne 1 then begin
    print, '% READ_PDB_DATA:  Unique dataset name must be provided.', dataset[0]
          for i = 0, nel-1 do print, dtyp[i]+1, ' - ', info.names[dtyp[i]]
          return, 0
        endif else dtyp = dtyp[0]
      endif else dtyp = long(dataset[0])-1
      if (dtyp lt 0) or (dtyp gt (info.ndset-1)) then begin
        print, '% READ_PDB_DATA:  Invalid dataset number:', dtyp
        print, ' Must be in the range of 1 to', info.ndset
        return, 0
      endif
       
; get the standard stuff 
      if (fam_info.lcparam[0] and 8) eq 8 then begin
        time = fam_info.time
        if not keyword_set(double) then time = float(time)
; ntime
        ntime = fam_info.ntime
      endif else begin
        ntime = 0
        time = 0
      endelse 
      ndim = info.ndim[dtyp]
      dim = info.dimensions[*,*, dtyp]
      clab =  info.names[dtyp]
      if ndim le 1 then begin
        dataset = info.names[dtyp]

        yval = read_pdb(info.names[dtyp], ier)
        if ier ne 0 then begin 
          print, '% READ_PDB_DATA: Error reading ', info.names[dtyp]
          help , yval , ier

          return, 0
        endif
        nyval = n_elements (yval)
        if nyval eq 1 then lctype=-1 else lctype = 1
        if info.type[dtyp] eq 'char'  then return, yval
        if info.type[dtyp] eq 'double' and not keyword_set(double) then $
          yval = float(yval)
        if lctype lt 0 then return, yval
        if (ntime eq nyval) then  yval = [time, yval] else $
          yval =[lindgen(nyval), yval]
        yval = reform(yval, nyval, 2, /overwrite)
        ier = 0
        return, yval
      endif
      
;
;     str = {file:p1, pds_fileid: error, ndset: ndset, datasets:names, $
;       length: totlen, type:type, ndim: ndim, source:source, $
;       dimensions: reform(dim, 2, 3, ndset, /overwrite)}

; x1 data, x2 data, and region data

    if (fam_info.lcparam[0] and 7) eq 7 then begin
      get_pdb_rzreg, x1, x2, ireg, ier, double = double
      if ier ne 0 then begin 
        print, '% READ_PDB_DATA: Error zt, rt and ireg histories'
        return, 0
      endif

      ss = size(x1)
      if ss(0) eq 3 then x1 = reform(x1, ss[1]*ss[2], ss[3], /overwrite)
      ss = size(x2)
      if ss(0) eq 3 then x2 = reform(x2, ss[1]*ss[2], ss[3], /overwrite)
      ss = size(ireg)
      if ss(0) lt 3 or ss[3] lt ntime then ireg = ireg[*,*,0]
    endif
;
      if n_elements (tvalue) eq 0 then begin
;       read everything 
;       dimensions keyword can be used to determine if leading dimensions have
;       been eliminated.
;       Currently yval will return with a leading dimension of 1.
;
        yval = read_pdb(info.names[dtyp], dimensions = dimensions, ier)
        ssy = size(yval)
        if ssy[0] eq 4 && keyword_set(void_frac) then begin
;         IDL index begins at zero
          vind = void_frac-1
          if vind ge 0 && vind lt ssy[3] then begin
            yval = reform(yval[*,*,vind,*])
            ssy = size(yval)
          endif
        endif
        if info.type[dtyp] eq 'double' and not keyword_set(double) then $
          yval = float(yval)
        if max(abs(ssy[1:2]- fam_info.size)) gt 0 or ssy[0] eq 4 then begin
          if ssy[0] ne 2 then begin
            if (strpos(strupcase (clab), 'FL') ge 0 ) and $
               ssy[3] eq ntime then begin
               arr2str, str, yval, 1.0+findgen(ssy[1]), 1.0+findgen(ssy[2]) , $
                 time, clab = clab, Xlab = 'Flux Surface', ylab = 'Energy Bin', $
                 zlab = 'Time', file = info.file, /structure_only
               lctype = 0
               yval =0
               ier = 0
               return, str
            endif
            if (strpos(strupcase (clab), 'KA') ge 0 ) and $
               ssy[4] eq ntime then begin
               arr2str, str, yval, 1.0+findgen(ssy[1]), 1.0+findgen(ssy[2]) , $
                 1+findgen(ssy[3]), time, clab = clab, $
                    Xlab = 'Z-Position', Ylab = 'R-Position', $
                    zlab = 'Energy Bin', x4label = 'Time',  $
                    file = info.file, /structure_only
               lctype = 0
               ier = 0
               yval =0
               return, str
            endif
            if ssy[0] eq 3 and ssy[3] eq ntime then begin
              arr2str, str, yval, 1.0+findgen(ssy[1]), 1.0+findgen(ssy[2]) , $
                 time, clab = clab, Xlab = 'Z-Position', Ylab = 'R-Position', $
                 zlab = 'Time', file = info.file, /structure_only
               lctype = 0
               ier = 0
               yval =0
               return, str
            endif

            print, '% READ_PDB_DATA: Unknown datatype'
            help , yval
            print, '  Please contact Paul Mix'
            lctype =0
            arr2str, str, yval, 1.0+findgen(ssy[1]), 1.0+findgen(ssy[2]) , $
            file = info.file, /structure_only, clab = clab
             lctype = 0
             yval =0
            ier = 0
            return, str
          endif
          
        lctype = ssy[1]
        if not keyword_set(matrix) then begin
          str = make_array(ssy[1]+1, ssy[2])
          str[1:*, *] = yval
          if ssy[2]  eq fam_info.ntime then begin
            str[0, *] = fam_info.time
          endif else if ssy[2]  eq n_elements(x1) then begin
            str[0, *] = x1
          endif else if ssy[2]  eq n_elements(x2) then begin
            str[0, *] = x2
          endif else str[0, *] = findgen(ssy[2])
          str = transpose (str)
        endif else arr2str, str, yval, /structure
          ier = 0
          return, str
      endif
        if n_elements (node_values) eq 0 then begin
          nodes = [ 'ut@history', 'vt@history']
          ok = where (info.names[dtyp] eq nodes, node_values)
        endif

;       get the triangles if we can
        s = size(ireg) 
        if s[0] eq 3 and s[3] eq ntime then begin
          tri = 0L
          loctri= [0, 1L]
          if keyword_set (node_values) then tlab = 'PACT Node Data' else $
            tlab = 'PACT Cell Data'

        endif else begin
          imat =  ireg(*, *, 0)
          make_pact_triangles, imat, tri
          if keyword_set (node_values) then tlab = 'PACT Node Data' else begin
            tlab = 'PACT Cell Data'
            cell2node_3d, yval, ireg, nodes
            yval = temporary(nodes)
          endelse
          ss = size(tri)
          loctri = [0L, ss[2]]
        endelse
        ss = size(yval)
        nreg =  ss[1]*ss[2]
        if ss(0) eq 3 then $
          yval = reform(yval, nreg, ss[3], /overwrite) $
        else if ss[2] ne Ntime then $
          yval = reform (yval, nreg, /overwrite) $
        else nreg = ss[1]
        
        ndim = 2
        block =1 
        type = 9L
        range = make_array(2, 2, 2)
        range(0, 0, *) = min(x1[tri], max = mm)
        range(1, 0, *) = mm
        range(0, 1, *) = min(x2[tri], max = mm)
        range(1, 1, *) = mm
;
        two = 3L
        zval = 0.0
        x3 = 0.0
        ss = size(ireg)
        ssize = ss(1:2)
        spare = make_array(5, val = 0)
        str = {type:type, ndim:ndim, block: block, two: two, ntime:ntime, $
             nvect: 1L, slice: 0.0, size:ssize, triangles:tri, loctri:loctri, $
             range:range, order : 0L, v1: yval, v2:0.0, v3:0.0, v4:0.0, $
             v5:0.0, v6:0.0, iregion: ireg, v1lab:'Amplitude', v2lab : '', $
              v3lab : '', v4lab : '',v5lab : '', v6lab : '', $
             locx: [0, nreg], locv:[0, nreg], x1:x1, x2:x2, x3:x3, x4: time, $
             x1lab:'Z value', x2lab:'R value', x3lab:'', x4lab: 'Time', $
             file: info.file, tlab:tlab, clab:clab, spare: spare }
        lctype = 0
        ier = 0
        return, str
      endif

      tslice = tvalue[0]
;
      dim = reform(info.dimensions[*,*, dtyp])
      if n_elements(tslice) eq 1 then begin
;       TSLICE with two or more elements is not currently supported.
;       
        print, '% READ_PDB_DATA: TSLICE must have only 1 element'
        print, '%                Only the first will be used'
        tslice = tslice[0]
        help, tslice
      endif
      if keyword_set(index) then begin 
        if tslice lt 1 or tslice gt ntime then begin
          print, '% READ_PDB_DATA: Bad index for tslice.'
          print, '       Index should be in the range 1 to ',strtrim(ntime,1)
          return ,0
        endif
        ind = tslice
      endif else begin
        ind = min(abs(time-tslice), nok)
        ind= ind(0)
        if abs(time(ind)-tslice) /max(time) gt 1e-3 then print, $
           '% READ_PDB_DATA: Time slice taken at ',strtrim(time[ind],1)
        ind= ind+1
      endelse
      indm = ind-1
      
      ndim = info.ndim [dtyp]
      if ndim eq 4 then begin
        if n_elements(void_frac) eq 0 then void_frac = 0
        if ((void_frac le 0) || (void_frac gt dim[1,2])) then begin 
          print, '% READ_PDB_DATA: Void_FRAC must be specified to read 4-D data'
          print, '                 with a time slice.'
          return, 0
        endif
      endif
      ijklist = make_array(3*ndim, /long)
      
      
      ijklist[0:1] = dim[*, 0]
      ijklist[2] = 1L
      ijklist[3:4] = dim[*, 1 ]
      ijklist[5] = 1L
      if ndim eq 4 then begin
        ijklist[6] = void_frac 
        ijklist[7:8] = dim[1, 2]
        ii = 9
      endif else ii = 6
      ijklist[ii] = ind 
      ijklist[ii+1: ii+2] = ntime
      dtype = info.type[dtyp]
      yval = read_pdb(info.names[dtyp], ier, type= dtype, $
             ijklist= ijklist)
      nreg = n_elements(yval)
      if info.type[dtyp] eq 'double' and not keyword_set(double) then $
         yval = float(yval)
      if n_elements (node_values) eq 0 then begin
        nodes = [ 'ut@history', 'vt@history']
        ok = where (info.names[dtyp] eq nodes, node_values)
      endif
;     get the triangles
      ireg =ireg[*,*,0]
      imat = ireg
;
      make_pact_triangles, imat, tri, bad = bad
      if keyword_set (node_values) then tlab = 'PACT Node Data' else begin
        tlab = 'PACT Cell Data'
        cell2node, yval, ireg, nodes, bad = bad
        yval = temporary(nodes)
      endelse
      ntime = 1L
      x4 = time[indm]
      ss = size(tri)
      loctri = [0L, ss[2]]
      ndim = 2
      block =1
      type = 8L
      range = make_array(2, 2, 2)
      x1 = x1(*,indm) 
      x2 = x2(*,indm) 
      range(0, 0, *) = min(x1, max = mm)
      range(1, 0, *) = mm
      range(0, 1, *) = min(x2, max = mm)
      range(1, 1, *) = mm
      clab = clab + ': Time = '+strtrim(x4,2)
      two = 3L
      zval = 0.0
      x3 = 0.0
      ss = size(ireg)
      ssize = ss(1:2)
      spare = make_array(5, val = 0)
      str = {type:type, ndim:ndim, block: block, two: two, ntime:1L, $
           nvect: 1L, slice: x4, size:ssize, triangles:tri, loctri:loctri, $
           range:range, order : 0L, v1: yval, v2:0.0, v3:0.0, v4:0.0, $
           v5:0.0, v6:0.0, iregion: ireg, v1lab:'Amplitude', v2lab : '', $
            v3lab : '', v4lab : '',v5lab : '', v6lab : '', $
           locx: [0, nreg], locv:[0, nreg], x1:x1, x2:x2, x3:x3, x4: x4, $
           x1lab:'Z value', x2lab:'R value', x3lab:'', x4lab: 'Time', $
           file: info.file, tlab:tlab, clab:clab, spare: spare }
      lctype = 0
      return, str
      
      end
