; ------------------------------------------------------------------------ 
; $Id: xy2kl.pro,v 1.5 2008/01/24 22:28:09 lpmix Exp $
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;

 function xy2kl, xin, yin, str, time, index = index, quiet=quiet, scalar=scalar

;+
;   Name: XY2KL
;  
;   Returns the cell K and L values for input X and Y spatial positions
;
;   NOTE: This feature may be accessed from the plotgrd command.
;   Note: PDB indices are assumed to start at 1 ie [1,1] is the lower left corner
;
;   PROCEDURE:
;         This routine looks for the cell with the closest midpoint and 
;         then checks to determine if the point is in that cell.
;         If the point is not in the closest cell then the routines checks to
;         see if the closest cell is in a bow tie. 
;   
;   Format:
;       POINT = XY2KL(X, Y, STR [, TIME] [, /QUIET] [, INDEX = index] 
;     where
;        POINT      - K and L coordinates for the cell containing X and Y
;        X, Y       - Spatial locations
;        STR        - Grid structure used for the current plot.
;        TIME       - If defined then used to determine the time of the grid
;                     with the values in X and Y.
;                     Default is TIME=1, /INDEX
;        INDEX      - If set, Time is taken as an index =[1, 2, ...N_time]
;                     If time = zero or integer=1 then time index is 1
;        QUIET      - If set, no output to the terminal
;        scalar     - returned with the scalar node index in the range [1,nnodes]
;    Example:
;        Print the location of a point (3.5, .3) in K and L coordinates for the 
;        initial grid location. Structure with the grid data is 12
;           print,  XY2KL(3.5, 0.3, 12)
;-
 common plot_grid_last, yeng_scale, xeng_scale
on_error,1


;  Common block
@qsstrcom.dat
@qscom.dat

      npar = n_params()
      if npar eq 0 then begin
        print,'% Format: POINT = XY2KL (X, Y, STR [, TIME] [, /QUIET] [, /INDEX]'
        return, -1
      endif
      if npar lt 3 then begin
        print,'% Format: X, Y and Grid structure must be defined.'
        return, -1
      endif

      if n_elements(str) eq 0 then begin
        print, '% XY2KL: Grid structure must be defined'
        return, -1
      endif else a = str
      if n_elements (structinit) eq 0 then qsstruct_init
      if n_elements (qsplotinit) eq 0 then qsinit
;
      b = size(a)
      if b(b(0)+1) ne 8 then begin
        if a gt 0 and a le maxstructure then begin
          s2i, a, a
          b = size (a)
          if b(b(0)+1) ne 8 then begin
 print, '% XY2KL: First parameter must be a structure or a array number'
            return, -1
          endif
        endif  else begin
 print, '% XY2KL: First parameter must be a structure or array number'
          return, -1
        endelse
      endif
      if (a.type ne 11 ) then begin
        print, '% XY2KL: Only PDB Grid type structures may be used'
        help, a.type
        return, -1
      endif

      if n_elements (xin) gt 0 then aa = xin[0] else begin
        print, '% XY2KL: X-value must be defined'
        return, -1
      endelse

      if n_elements (yin) gt 0 then bb = yin[0] else begin
        print, '% XY2KL: Y-value must be defined'
        return, -1
      endelse

      if n_elements (time) eq 0 then begin
        time = 1
        index = 1
        if not keyword_set(quiet) then print, $
          '% XY2KL: First output time step will be used for the grid'
      endif else  begin
        zval = time
        if keyword_set(index) then begin
          time = round(time)-1
          if (time lt 0) or (time ge a.ntime) then begin
            print, '% XY2KL: Invalid Time index:', time
            return, -1
          endif
        endif else begin
          mm = min(abs(a.x4-zval), ii)
          if (abs(mm)/(a.x4[a.ntime-1]-a.x4[0]) gt 1e-4) then begin
            print, '% XY2KL: Invalid Time value', zvalueold, a.x4[ii]
            return, -1
          endif
          time = ii
        endelse
      endelse
      nx = a.size(0)
      ny = a.size(1)
      nxm = nx-1
      nym = ny-1
      region = reform (a.region[*,0])
      ii = -1
      j = 0
      if not keyword_set(quiet) then print,' Point Selected (X, Y):', aa, bb
      ax1 = reform (a.x1(*, time))
      ax2 = reform (a.x2(*, time))
      conn = a.connect
      nc = n_elements(conn)
      nc4 = nc/4
      rdist = make_array(nc4)
      ele = lindgen(4)
      for n = 0L, nc4-1 do rdist[n] = $
        (0.25*total(ax1[conn[4*n+ele]])-aa)^2 + (0.25*total(ax2[conn[4*n+ele]])-bb)^2
      mindist = min(rdist, ival)
      icon = conn[4*ival+ele]
      ix = min(icon mod nx)
      iy = min(icon/nx)
      ii = -1
      ar = [icon, icon[0]]
      px = ax1[ar] 
      py = ax2[ar]
      x1 = px[0:3]-aa
      x2 = px[1:4]-aa
      y1 = py[0:3]-bb
      y2 = py[1:4]-bb
      dp = X1*X2 + Y1*Y2       ; Dot-product
      cp = X1*Y2 - Y1*X2       ; Cross-product
      theta = Atan(cp,dp)
;     total should be zero or 2pi
      IF (Abs(Total(theta)) GT !PI) THEN begin
        ii = ix+1
        jj = iy+1
      endif
      if ii lt 0 then begin
        if not keyword_set(quiet) then begin
          print, '% XY2KL: Unable to locate cell.'
          print, '  Is the point valid or is the cell a bow tie??'
        endif
        return, -1
      endif
      if not keyword_set(quiet) then print,'  Cell selected (I, J):', ii, jj
      scalar = iy*nx + ii
      return, [ii, jj]
      end
