; ------------------------------------------------------------------------ 
; $Id: get_pds_info.pro,v 1.3 2007/06/06 19:50:16 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
 function get_pds_info, par1, par2,  variable = variable, $
   information = information, segment = segment, ier = ier

; Copyright (c) 1998, Sandia National Laboratories  All rights reserved.
;       Unauthorized reproduction prohibited.
;+
; NAME:
;      GET_PDS_INFO
;
; PURPOSE:
;      Return a list of names associated with a specified PDS type.
;      Uses: PDS_INQ_INFO
;
; CALLING SEQUENCE:
;
;       INFO  = GET_PDS_INFO([ File_Handle, ] NAME [, IER = ier] [, /HELPme]
;               /Variables, /Segment , /Information)
;
; WHERE:
;        INFO         - Requested Info parameter.
;        File_Handle  - File_handle of the PDS file
;                       Default: Current file
;        NAME         - Name of the info entry
;        IER          - Error Parameter
;                       Returns 0 if successful
;        HELPme       - If set, a one user will be given a help message.
;
;  FLAGS: One or more must be set.
;      Note:   If multiple valid flags are set, only the first one found will
;
;       SEGMENT      - If set, return the info entry for the current segment
;       VARIABLE     - If set, return the info entry for the current variable
;       INFORMATION  - Ignored. If SEGMENT and VARIABLE are not set then the
;                      the info entry for the file is returned.
;                      
;
; EXAMPLE:
;       Get the info 'title' for the current file.
;         title  = get_pds_info('title')
;
; MODIFICATION HISTORY:
;       Written by:     L. P. Mix, August 27, 1998
;-

;
;
@pds_datacom.dat
      on_error, 1

@pds_info_types.dat
      value = 0
      npar = n_params()
      if npar eq 0  or keyword_set(helpme) then begin
        IER = 0
print, '% Format: '
print,'    INFO  = GET_PDS_INFO([ File_Handle, ] NAME [, IER = ier] [, /HELPme]'
print,'              [ /Variables, /Information, /Segment] )'
        return, value
      endif
      ier = -1L
      if n_elements(initpds) eq 0 then initpds = 0
      if initpds eq 0 then init_pds
      if initpds lt 0 then begin
        print, '% GET_PDS_INFO: This installation does not support PDS'
        return, value
      endif

      if n_elements (par1) eq 0 then begin
        print, '% GET_PDS_INFO: First parameter must be defined'
        return, value
      endif

      if npar eq 1 then begin
        fh = long(pds_id)
        name = par1
      endif else begin
        if n_elements (par2) eq 0 then begin
          print, '% GET_PDS_INFO: Name parameter (par2) must be defined'
          return, value
        endif
        name = par2
        fh = long(par1)
      endelse
      if size(name, /type) ne 7 then begin
        print,  '% GET_PDS_INFO: String name must be provided'
        return, value
      endif

      if keyword_set(segment) then  type =  PDS_SEG  else $
          if keyword_set(variable) then type = PDS_VAR else type = 0L
      name = strtrim(name, 2)
      lstr = strlen(name)
      name = [byte(name), 0B]
      typsav = type
      count = 0L
      value 0L
      junk = call_external(PDS_EXE, 'pdsp_inq_info_', fh, name, lstr, $
        type, count, ier)

      if ier ne 0 then begin
        print, '% GET_PDS_INFO: Error getting info data =' , ier
        return, value
      endif
      if count eq 0 then begin
        print, '% GET_PDS_INFO: No info values available'
        return, value
      endif
      
      type = type mod 256
      lcstring = 0
      if type eq PIO_CHAR  then begin
        value = make_array(/byte, count+1, value = 32B)
        lcstring = 1
      endif else if type eq PIO_SHORT then begin
        value = make_array( count, value = 0)
      endif else if type eq PIO_INT then begin
        value = make_array( count, value = 0L, /long)
      endif else if type eq PIO_FLOAT then begin
        value = make_array( count, value = 0.0)
      endif else if type eq PIO_DOUBLE then begin
        value = make_array( count, value = 0.0D0, /double)
      endif else begin
        ier = -1L
        print, '% GET_PDS_INFO: Unknown data type', type
        return, 0
      endelse

      junk = call_external(PDS_EXE, 'pdsp_inq_info_', fh, name, lstr, $
        typsav, count, value, ier)
     
      if ier ne 0 then $
        print, '% GET_PDS_INFO: Error in obtaining parameter', typ  else $
      if lcstring ne 0 then value = string(value)
      
      return, value
      end
