; ------------------------------------------------------------------------ 
; $Id: get_pds_var.pro,v 1.3 2007/06/06 19:50:16 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
 function get_pds_var, par1, par2, segment = segment, $
          file_handle = filehandle, topology = topology, $
          getTC = getTC, statesegments = statesegments, ier = ier
;
; Copyright (c) 1998, Sandia National Laboratories  All rights reserved.
;       Unauthorized reproduction prohibited.
;+
; NAME:
;      GET_PDS_VAR
;
; PURPOSE:
;      Read a PDS variable from a specified segment
;      Uses: PDS_SGET_VAR
;
; CALLING SEQUENCE:
;
;       VAR = GET_PDS_VAR ([FILE_HANDLE,] VARIABLE [, FILE_HANDLE = file_handle]
;             [, SEGMENT = segment] [, /TOPOLOGY] [, /STATESEGMENTS]
;             [, IER = ier])
;
; WHERE:
;        VAR          - Requested variable value
;        FILE_HANDLE  - PDS file ID
;                       Default: File_Handle = current file
;                       Note: If two parameters are present then the keyword
;                             file_handle is ignored.
;        VARIABLE     - Variable ID or a string.
;        SEGMENT      - Desired segment. Default value = -1
;                       If segment is >+ zero, then TOPOLOGY and STATESEGMENT 
;                       are ignored.
;        TOPOLOGY     - If set, set to topology segment for requested time index
;                       Default: set to a state segment
;        STATESEGMENT - Ignored.  If TOPOLOGY is not set, then statesegment will
;                       be set. 
;        IER          - Error Parameter
;                       Returns 0 if successful
;                               -1 if unsuccessful or if help is selected
;
; EXAMPLE:
;       Read the variable 'veloc' from the current segment
;          vel = get_pds_var('veloc')
;         
;
; MODIFICATION HISTORY:
;       Written by:     L. P. Mix, August 26, 1998
;-

@pds_datacom.dat
      on_error, 1

@pds_info_types.dat

 
 ier = -1L
 npar = n_params()
 if npar eq 0 then begin
   print, '% FORMAT:
   print, '   VAR = GET_PDS_VAR ( [FILE_HANDLE,] VARIABLE [, /TOPOLOGY]'
   print, '              [, /STATESEGMENTS] [, FILE_HANDLE = file_handle]'
   print, '              [, SEGMENT = segment] [, IER = ier]'
   return, 0
 endif

 if n_elements(initpds) eq 0 then initpds = 0
 if initpds eq 0 then begin
   Print, '% GET_PDS_VAR: No PDS files are open'
   return, 0
 endif

 if initpds lt 0 then begin
   print, '% GET_PDS_VAR: This installation does not support PDS'
   return, 0
 endif

 if n_elements (par1) eq 0 then begin
   print, '% GET_PDS_VAR: First parameter must be defined'
   return, 0
 endif
 if npar eq 2 then begin
   if n_elements (par2) eq 0 then begin
     print, '% GET_PDS_VAR: Second parameter must be defined'
     return, 0
   endif
 endif

 if npar eq 1 then begin
   if keyword_set(filehandle) then fh = long(filehandle[0]) $
       else fh = long(pds_id)
   name  = par1[0]
 endif else begin
   fh = long(par1[0])
   name = par2[0]
 endelse

 if n_elements(segment) eq 0 then segment = -1L
 if segment ge 0  then begin
   set_pds_segment,fh, segment, topology = topology, ier = ier
   if ier ne 0 then begin
     print, '% GET_PDS_VAR: Error setting segment'
     return, 0
   endif
 endif

 if size(name, /type) eq 7 then begin
    name=get_pds_varid(name, ier = ier)
    if ier ne 0 then begin
      print,  '% GET_PDS_VAR: Error getting variable ID.'
      return, 0L
    endif
 endif else name=long(name)

 nbyte = get_pds_var_size (fh, name, ier = ier)
 if ier ne 0 then begin
   print, '% GET_PDS_VAR: Error getting size of variable.'
   return, 0
 endif

 ier = 1L
 nvar = total(nbyte)/4
 if nvar le 0 then begin
   print, '% GET_PDS_VAR: Variable has zero size'
   return,0
 endif
 if n_elements(gettc) eq 0 then gettc = 4
 if gettc eq 3 then variable = make_array(nvar, /long)  $
    else if gettc eq 5 then variable = make_array(nvar, /double ) $
    else variable = make_array(nvar, /float)
 junk= call_external(PDS_EXE, 'pdsp_sget_var_',fh, name, variable, ier)

 
 if ier ne 0 then $
    print, '% GET_PDS_VAR: Error getting PDS variable', name
 return, variable
 end
