; ------------------------------------------------------------------------ 
; $Id: get_pds_var_size.pro,v 1.3 2007/06/06 19:50:17 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
 Function get_pds_var_size, par1, par2, file_handle=fhin, ier=ier, $
       helpme=helpme 
;
; Copyright (c) 1998, Sandia National Laboratories  All rights reserved.
;       Unauthorized reproduction prohibited.
;+
; NAME:
;      GET_PDS_VAR_SIZE
;
; PURPOSE:
;      Determine the number of processors which write a variable in a PDS
;      segment.  Zero means the variable was not written in the segment
;      Uses: PDS_INQ_VAR_SIZES
;
; CALLING SEQUENCE:
;
;       N_BYTES   = GET_PDS_VAR_SIZE ([FILE_HANDLE,] VARID
;               [, FILE_HANDLE = file_handle] [, IER = ier] [, /HELPme] )
;
; WHERE:
;        N_BYTES       - Variable Size
;                       Zero means variable is not in this segment.
;        FILE_HANDLE  - PDS file ID
;                       Default: File_Handle = current file
;                       Note: If two parameters are present then the keyword
;                             file_handle is ignored.
;        VARID        - Variable handle
;                       If VARID is a string, then it will be converted to a 
;                       VARID number.
;        IER          - Error Parameter
;                       Returns 0 if successful
;                               -1 if unsuccessful or if help is selected
;        HELPme       - If set, a one user will be given a help message.
;
; EXAMPLE:
;       Get the size in bytes for the variable, 3, in the current segment
;         num  = get_pds_var_size(3)
;
; MODIFICATION HISTORY:
;       Written by:     L. P. Mix, August 27, 1998
;-


;
;
@pds_datacom.dat
      on_error, 1

@pds_info_types.dat
  ier = -1L
  if keyword_set (helpme) then begin
    print, '% Format: N_BYTES   = GET_PDS_VAR_SIZE ([FILE_HANDLE,] VARID'
    print, '       [, FILE_HANDLE = file_handle] [, IER = ier] [, /HELPme] )'
    return, 0
  endif
  npar = n_params()
  if n_elements(par1) eq 0 then begin
    print, '% GET_PDS_VAR_SIZE: Variable name/ID must be provided'
    return, -1
  endif
  if npar eq 1 then begin
    name = par1[0]
    if n_elements(fhin) eq 0 then fh= pds_id else fh = long (fhin[0])
  endif else begin
    fh = long(par1[0])
    if n_elements(par2) eq 0 then begin
print, '% GET_PDS_VAR_SIZE: Variable id must be provided after the file ID'
      return, -1
    endif
    name = par2[0]
  endelse
  if size(name, /type) eq 7 then begin
    name=get_pds_varid(name, ier = ier)
    if ier ne 0 then begin
      print,  '% GET_PDS_VAR_SIZE: Error getting variable ID.'
      return, 0L
    endif
  endif else name=long(name)
    
; 
  nproc = -1L
  junk= call_external(PDS_EXE, 'pdsp_inq_var_n_size_',fh, name, nproc, ier)

  if ier ne 0 then begin
    print, '% GET_PDS_VAR_SIZE: Error obtaining the number of processors'
    return, 0
  endif
  ier = -1L
  if nproc eq 0 then begin
    print, '% GET_PDS_VAR_SIZE: Variable not present in this segment'
    return, 0
  endif

  bsizes = make_array(/long, nproc)
  junk = call_external(PDS_EXE, 'pdsp_inq_var_sizes_',fh, name, bsizes, ier)

  if ier ne 0 then begin
    print, '% GET_PDS_VAR_SIZE: Error obtaining the variable size'
    return, 0
  endif

  return, bsizes
  end
