; ------------------------------------------------------------------------ 
; $Id: inquire_pds.pro,v 1.3 2007/06/06 19:50:17 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;

 Function inquire_pds, filehandle, variables = var, segments = segment, $
   datafiles = datafiles, statesegments = statesegments, topology = topology, $
   information = information, dimension = dimension, pdstype = pdstype, $
   helpme= helpme, ier = ier

;  pdstype used by pfidl directly

;+
; NAME:
;      INQUIRE_PDS
;
; PURPOSE:
;      Retrieve the value of the specified PDS parameter. 
;      Uses: PDS_INQ_PARAM
;
; CALLING SEQUENCE:
;
;       PARAM_VAL = INQUIRE_PDS([ File_Handle] [, IER = ier] , /VARIABLES,
;                    /SEGMENTS, /DATAFILES, /STATESEGMENTS , /TOPOLOGY,
;                    /INFORMATION, /DIMENSION, [, /HelpMe] )
;
; WHERE:
;        PARAM_VAL    - Number of parameters specified by flags
;                       Set to 0 if an error or HelpMe is set
;        File_Handle  - File_handle of the PDS file
;                       Default: Current file
;        IER          - Error Parameter, returned 0 if no error
;                       Returned -1 if an error or if HelpMe is set
;        HelpMe       - If set, a one user will be given a help message.
;
; FLAGS: One or more must be set.
;      Note:   If multiple valid flags are set, only the first one found will
;              be returned in the following order:
;       DIMENSION    - If set, return the problem dimensions.
;       TOPOLOGY     - If only TOPOLOGY is set, return the number of topology
;                      segments.
;       TOPOLOGY+STATESEGMENT - If both set, return the number of state segments
;                      associated with the current topology segment.
;       INFORMATION  - If only INFORMATION is set, return the number of info 
;                      entries for the file.
;       INFORMATION+SEGMENT- If both INFORMATION and SEGMENT are set, return the
;                      number of info entries for the current segment.
;       INFORMATION+VARIABLES- If both INFORMATION and VARIABLES are set, return
;                      the number of info entries for the current variable.
;       STATESEGMENT - If only STATESEGMENT is set, return the number of 
;                      statesegments
;       VARIABLES    - If set, return the number of variables
;       SEGMENTS     - If set, return the number of segments
;       DATAFILES    - If set, return the number of datafiles
;
; EXAMPLE:
;       Get the number of variables in the current pds file
;         n_variables = inquire_pds(/variable)
;
; MODIFICATION HISTORY:
;       Written by:     L. P. Mix, August 14, 1998
;-


@pds_datacom.dat
      on_error, 1

@pds_info_types.dat
      ier = -1L
      value = 0L
      if keyword_set(helpme) then begin
print, '% FORMAT:'
print, '  PARAM_VAL = INQUIRE_PDS([ File_Handle] [, IER = ier] , /VARIABLES,'
print, '              /SEGMENTS, /DATAFILES, /STATESEGMENTS , /TOPOLOGY,'
print, '              /INFORMATION, /DIMENSION, [, /HELPme] )'
        return, -1
      endif
      if n_elements(initpds) eq 0 then initpds = 0
      if initpds eq 0 then init_pds
      if initpds lt 0 then begin
        print, '% INQUIRE_PDS: This installation does not support PDS'
        return, value
      endif
      if n_elements(filehandle) eq 0 then fh = pds_id else begin
        if filehandle[0] lt 0 then fh = pds_id else fh = long(filehandle[0])
      endelse 

      type = -1L
      if keyword_set(pdstype) then begin
        type = pdstype
      endif else if keyword_set(dimension) then begin
        type = PDS_DIM
      endif else if keyword_set(topology) then begin
        type = PDS_TP
        if keyword_set(statesegments) then type = type + PDS_ST
      endif else if keyword_set(information) then begin
        type = PDS_INFO
        if keyword_set(segment) then type = type + PDS_SEG else $
          if keyword_set(var) then type = type +PDS_VAR
      endif else if keyword_set(statesegments) then begin
        type = PDS_ST
      endif else if keyword_set(var) then begin
        type = PDS_VAR
      endif else if keyword_set(segment) then begin
        type = PDS_SEG
      endif else if keyword_set(datafiles) then begin
        type = PDS_DTF
      endif
      if type lt 0 then begin
        print, '% INQUIRE_PDS: Type value must be provided'
        return, value
      endif
      ier = 1L
      junk = call_external(PDS_EXE, 'pdsp_inq_param_', fh, type, value, ier)
      if ier ne 0 then begin
        print, '% INQUIRE_PDS: Error in obtaining parameter', type
        value =0L
      endif
      return, value
      end
