; ------------------------------------------------------------------------
; $Id: get_surf_hist.pro,v 1.1 2010/03/03 18:43:08 lpmix Exp $
; ------------------------------------------------------------------------
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
;+
; NAME:
;       GET_SURF_HIST
;
; PURPOSE:
;       Return a history waveform array of node parameters for a given surface.
;       Quantities include MAXIMUM, MINIMUM, AVERAGE, STANDARD DEVIATION.
;       This routine is much more efficient if all waveforms are requested in
;       a single call rather than multiple calls for a single parameter.
;
; CATEGORY:
;       UTILITY
;
; CALLING SEQUENCE:
;
;       GET_SURF_HIST, SURFACE_DATASET
;
; INPUTS:
;       SURFACE_DATASET:  PFF dataset number or unique string to designate the
;               surface dataset.  Datasets can be retrieved with GET_UNIQUE.
;               Only the first 32 characters of the comment label are checked.
;               If SURFACE_DATASET is zero or undefined, a widget will be presented.
;	
; KEYWORD PARAMETERS:
;       FILEID: File ID for the PFF file
;               DEFAULT:  FILEID = 0  (Current file)
;       NODE_ARRAY:  Array of nodes will be used for the calculations
;               DEFAULT:  If missing or negative, ALL nodes are used.
;               Note:  The node values begin with 0 for the first node.
;                      The values from GET_SURF_NODES may be used directly.
;       VECTOR: Specifies the interpretation of the field quantities.
;               VECTOR = ABS(VECTOR) > 1
;               If VECTOR is an array, the square root of the sum of
;               the squares will be used.
;               If FLD is a single value array, ABS will be applied.
;               That is, if FLD =[1] then the absolute value of the first 
;               field quantity is used.
;               If FLD is a single value larger than the number of vector
;               components, then FLD = 1+indgen(nvect)
;               where nvect is the number of vector components
;               If the surface data has only one attribute such as temperature or 
;                 charge density then this parameter can be omitted.
;       SURFACEID: Surface ID for the surface.  This is returned as a string but may be 
;               converted to an unsigned 64bit integer with ULONG64
;       MAXIMUM: If set the time history of the maximum vector is returned.
;               If set the WDF array will be stored in WDF array MAXIMUM ***
;               NOTE: MAXIMUM will always be in WDF = 1 if /Maximum is entered
;       XMAX:   X-Coordinate of the maximum vector (First coordinate)
;               If set the WDF array will be stored in WDF array XMAX ***
;       YMAX:   Y-Coordinate of the Maximum vector (Second coordinate)
;               If set the WDF array will be stored in WDF array YMAX ***
;       ZMAX:   Z-Coordinate of the Maximum vector (Third coordinate)
;               If set the WDF array will be stored in WDF array ZMAX ***
;       COORDINATES:  If set, XMAX, YMAX, and ZMAX are returned.
;               NOTE:  The IDL MAX function is used.  If two or more nodes have the 
;                      same value, only the first is used.
;               If set the WDF array will be stored in WDF array specified
;                 by coordinates ***
;               If XYZ-MAX are also set, they will overwrite the WDF values.
;       MINIMUM: If set the time history of the minimum vector is returned.
;               If set the WDF array will be stored in WDF array MINIMUM ***
;       AVERAGE: If set the time history of the average vector is returned.
;               If set the WDF array will be stored in WDF array AVERAGE ***
;               MOMENT is used to calculate the mean.
;       SDEVIATION: If set the time history of the standard deviation of the
;                 vector is returned.  
;               MOMENT is used to calculate the variance.
;               If set the WDF array will be stored in WDF array SDEVIATION ***
;
;       ***     Waveform arrays are determined by the following scheme.
;               If all keywords specify unique WDF array number, then they are used.
;               If there is any duplication, the following scheme is used.
;               The WDF arrays are organized in the following order:
;                 [MAXIMUM, XMAX, YMAX, ZMAX, MINIMUM, AVERAGE, SDEVIATION]
;               Any quanties not requested are eliminated.
;               The WDF arrays are sorted and the order preserved for equal values.
;               The quantity farthest to the left is stored in that location and 
;                 successive quanties are stored in successive arrays regardless of
;                 keyword value.
;               For example, the command contained the following:
;                          /MAX, /MIN, /AVERAGE
;               Then, MAXIMUM would be in 1, MINIMUM in 2, and AVERAGE in 3.
;               If the number number of WDF arrays need to be increased, they will be 
;               increased by a maximum of 16 wdf arrays.
;
; OUTPUTS:
;       The requested WDF arrays are stored.
;
; EXAMPLE:
;       Find the time history of the maximum value of magnitude of the magnetic field E
;       and store the result in WDF array 22
;         GET_SURF_HIST, 'E', vector = 4, MAXIMUM = 22
;  <OR>   GET_SURF_HIST, 'E', vector = [1, 2, 3], MAXIMUM = 22
;
;       Calculate the time history of the temperature, TEMP, averaged over the nodes in 
;       variable, POST, and the maximum of these values.  Store the results in WDF arrays
;       33 and 34, respectively.
;         GET_SURF_HIST, 'TEMP',  NODE = POST, MAXIMUM = 34, AVERAGE= 33
;
;       Same as above, but store the maximum in 1 and the average in 2.
;         GET_SURF_HIST, 'TEMP', NODE = POST, /MAXIMUM, /AVERAGE  ;; See *** above
;  <OR>   GET_SURF_HIST, 'TEMP', NODE = POST, MAXIMUM =1, AVERAGE =2
;
;       Same as above but return the coordinates of the maximum in wdf arrays, 11, 12, and 13
;         GET_SURF_HIST, 'TEMP', NODE = POST, MAXIMUM =1, AVERAGE =2, COORD =[11, 12, 13]
;
; MODIFICATION HISTORY:
;       Written by:	L. P. Mix, Jr., February 17, 2010
;       July, 1994	
;
;-
 pro get_surf_hist, datasetin, fileid = fileid, coordinates = coordinates, $
             surfaceid = surfaceid, node_array = nodearray, vector = vectorin, $
             maximum = maximum, minimum = minimum, xmax = xmax, ymax = ymax, $
             zmax = zmax, average = average, sdeviation = sdeviation
 on_error, 1
 maxinc = 16
 nelem = 0L
 ier =0L
 if n_elements(fileid) gt 0 then if fileid[0] ne 0 then setpff, fileid[0], ier
 if ier ne 0 then begin
   print, '% GET_SURF_HIST: Error setting file id to: '+strtrim(fileid[0], 2)
   return
 endif
 nelem = 0
 get_dir, comment= comment, type = type, number = ndset, /no_trim
 clab = strtrim(strmid(comment, 0, 32), 2) 
 oktype =  get_match('Surface Topology', type, nelem = noktype)
 okbox =  get_match('Bounding Box', type, nelem = nokbox)
 ind = lindgen(ndset)+1
 if noktype gt 0 then ind[oktype] =-1 else begin
   print, '% GET_SURF_HIST: No datasets with "Surface Topology" dataset type'
   return
 endelse
 if nokbox gt 0 then ind[okbox] = -1
 okdata = where (ind ge 0 , nokdata)
 if nokdata eq 0 then begin
   print, '% GET_SURF_HIST: No datasets with surface data'
   return
 endif
 ind = ind[okdata]
 clabdata = clab[okdata]
 for i = 0, nokdata-1 do begin
   c = strpos(clabdata[i], '    ') < 32
   if c gt 0 then clabdata[i] = strtrim(strmid(clabdata[i], 0, c), 2) else $
       clabdata[i] = strtrim(strmid(clabdata[i], 0, 32), 2)
 endfor
help, ind, clabdata
print, ind, clabdata
 if n_elements(datasetin) gt 0 then dataset = datasetin[0] else dataset = 0
 if size(/type, dataset) lt 4 && dataset gt 0 then begin
   dset = where (dataset eq ind, ndset)
   if ndset eq 0 then begin
     print, '% GET_SURF_HIST: No surface dataset with dataset number '+strtrim(dataset)
     return
   endif else begin
     ok = get_match('^'+clabdata[dset[0]]+'$', clabdata, nelem = nelem)
     if nelem gt 0 then select = ind[ok]
   endelse
 endif else if size(/type, dataset) eq 7 then begin 
   ok = get_match('^'+strtrim(dataset, 2)+'$', clabdata, nelem = nelem)
   if nelem gt 0 then select = ind[ok]
 endif
help, select
print, select
 if nelem eq 0 then begin
   instr= 'Pick the dataset for the desired surface history'
   cuniq = clabdata[uniq(clabdata, sort(clabdata))]
   wselect_item, cuniq, select = select, /exclusive
   ok = get_match('^'+cuniq[select]+'$', clabdata, nelem = nelem)
   if nelem gt 0 then select = ind[ok]
 endif
 if nelem lt 3 then begin
   print, '% GET_SURF_HIST:  WDF arrays require at least 3 points.'
   return
 endif
 readhdr, select[0], ier, aptype = aptyp, /quiet, clabel = outclab
 if ier ne 0 then begin
   print, '% GET_SURF_HIST: Error reading dataset: '+strtrim(select[0], 2)
   return
 endif
 if (aptyp lt 201) || (aptyp gt 210) then begin
   print, '% GET_SURF_HIST: Wrong dataset type:  '+strtrim(aptyp, 2)
   return
 endif
 c = strpos(outclab, '    ') < 32
 if c gt 0 then outclab = strtrim(strmid(outclab, 0, c), 2) else $
     outclab = strtrim(strmid(outclab, 0, 32), 2)

 readifl, iarray, flist, farray, select[0], clab = clab
 surfaceid = strtrim(iarray[0], 2) + strtrim(string(format='(i4.4)', iarray[1]), 2)
 nvect = iarray[3]
 nnodevec = n_elements(farray)/nvect
 ss = size(vectorin)
 lcabsolute = (ss[0] gt 0) ? 1:0
 if nvect gt 1 then begin
   if n_elements(vectorin) eq 0 then begin
     print, '% GET_SURF_HIST:  This dataset contains '+strtrim(nvect, 2)+ ' vectors'
     print, '                  The VECTOR must be specified to indicate preferences'
     return
   endif
   vector = (abs(vectorin) > 1)
   nelvect = n_elements(vector)
   if nelvect eq 1 then vect=(abs(vector[0]) gt nvect) ? (1+indgen(nvect)) : (vector[0] > 1) $
     else  vect = vector
 endif else vect = 1
 vect = (vect > 1) < nvect  
 nelvect = n_elements(vect)
; convert vect to an index
 vect -=1
 topology = get_match('Surface Topology '+surfaceid, comment[oktype], nelem= nel)
 if nel eq 0 then begin
   print,  '% GET_SURF_HIST: Unable to find surface topology for dataset'
   print,  '      Surface Topology: '+surfaceid
   return
 endif
 topology = oktype[topology[0]]+1
 readifl, topiarray, topflist, topfarray, topology
 ndim = topiarray[3]
 nnodes = n_elements(topfarray)/ndim
 if nnodes ne nnodevec then begin
   print, '% GET_SURF_HIST: Topology dataset and data dataset have different number of nodes'
   print, '               ', nnodes, nnodevec
   return
 endif
 coord = reform (topfarray, 3, nnodes, /overwrite)
 nelnodearr = n_elements (nodearray)
 if nelnodearr gt 0 then begin
   if (min(nodearray, max = mm) lt 0) || (mm ge nnodes) then begin
     print, '% GET_SURF_HIST: Invalid node list'
     print, '     Maximum/Minimum values: ', mm, min(nodearray)
     print, '     Number of nodes: ', nelnodearr
     return
   endif
 endif
 if nelnodearr gt 0 then coord = coord[*, nodearray]
 wdf = lonarr(7)
 if keyword_set(maximum) then wdf[0] = maximum[0]
 if keyword_set(minimum) then wdf[4] = minimum[0]
 if keyword_set(coordinates) then begin
   nc= n_elements(coordinates)
   if nc eq 1 then wdf[1:3] = coordinates[0] else $
     if nc eq 2 then wdf[1:3] = [coordinates, 1] else wdf[1:3] = coordinates[0:2]
 endif
 if keyword_set(xmax) then wdf[1] = xmax
 if keyword_set(ymax) then wdf[2] = ymax
 if keyword_set(zmax) then wdf[3] = zmax
   
 if keyword_set (average) then wdf[5] = average
 if keyword_set (sdeviation) then wdf[6] = sdeviation
 lcminmax = (keyword_set(maximum) || keyword_set(coordinates) || keyword_set(minimum) || $
   keyword_set(xmax) || keyword_set(ymax) || keyword_set(zmax)) ? 1 : 0
 lcave =(keyword_set (average) || keyword_set (sdeviation)) ? 1 : 0 
 ok = where(wdf ne 0, nok)
 if nok eq 0 then begin
   print, '% GET_SURF_HIST: At least 1 time history must be selected.'
   return
 endif
 maxwdf = get_maxwdf()
 wd = abs(wdf[ok])
 srt = bsort(wd)
 uwd = wd[uniq(wd, srt)]
 if nok gt n_elements(uwd) then begin
   wds = wd[srt]
   for i = 1, nok-1 do if wds[i-1] ge wds[i] then wds[i] = wds[i-1]+ 1
   wd[srt] = wds
 endif
 wdf[ok] = wd
 if max(wdf) ge maxwdf then begin
   m = maxwdf
   if m gt maxwdf + maxinc then begin
     print, '% GET_SURF_HIST: Maximum WDF array needed is '+strtrim (m, 2)
     print, '                 Please increase the number of arrays or '
     print, '                 change the storage locations.'
     return
   endif
   set_maxwdfarray, maxwdf+maxinc
 endif
;
 if lcminmax gt 0 then begin
   maxval = fltarr(nelem, /nozero)
   if wdf[4] gt 0 then minval = fltarr(nelem, /nozero)
   if wdf[1] gt 0 then xpval = fltarr(nelem, /nozero)
   if wdf[2] gt 0 then ypval = fltarr(nelem, /nozero)
   if wdf[3] gt 0 then zpval = fltarr(nelem, /nozero)
 endif
 if lcave gt 0 then begin
   aveval = fltarr(nelem, /nozero) 
   if wdf[6] gt 0 then begin
     sdval = fltarr(nelem, /nozero)
   endif
 endif
 times = fltarr(nelem, /nozero)
 for i = 0, nelem-1 do begin

;  could skip next two statements for i = 0 
   readifl, iarray, flist, farray, select[i], clab = clab
   surfaceident = strtrim(iarray[0], 2) + strtrim(string(format='(i4.4)', iarray[1]), 2)
   if surfaceident ne surfaceid then begin
     print, '% GET_SURF_HIST: Surface identification do not agree between the following:'
     print, comment[select[0]-1]
     print, comment[select[i]-1]
     return
   endif
   farray = reform(farray, nnodes, nvect, /overwrite)
   if nelnodearr gt 0 then farray = farray[nodearray, *]
   if nvect gt 1 then begin
     if nelvect gt 1 then begin
       f = (farray[*, vect[0]])^2
       for j = 1, nelvect-1 do f += (farray[*, vect[j]])^2
       farray = sqrt(temporary(f))
     endif else begin
       farray = farray[*, vect[0]]
       if lcabsolute gt 0 then farray = abs(farray)
     endelse
   endif else if lcabsolute then farray = abs(farray)
;  get the time
   jj = strpos(clab, 't=')
   if jj lt 0 then begin
     Print, '% GET_SURF_HIST: Unable to find time value for dataset: '+strtrim(select[i],2)
     help, clab
     return
   endif
   reads, strmid(clab, jj+2), tim
   times[i] = tim
   if lcminmax gt 0 then begin
     maxval[i] = max(farray, min = mm, j)
     if wdf[4] gt 0 then minval[i] = mm
     if wdf[1] gt 0 then xpval[i] = coord[0, j]
     if wdf[2] gt 0 then ypval[i] = coord[1, j]
     if wdf[3] gt 0 then zpval[i] = coord[2, j]
   endif
   if lcave gt 0 then begin
     if wdf[6] eq 0 then aveval[i] = total(farray, /double)/n_elements (farray) else begin
       value = moment(farray, maxmoment = 2)
       sdval[i] = sqrt(value[1])
       aveval[i] = value[0]
     endelse
   endif
 endfor
 help, times, maxval
 print, times
 xl ='Time(s)'
 if wdf[0] gt 0 then i2w, times, maxval, wdf[0], $
    /xydata, clab = outclab+' - Maximum value', xlab = xl
 if wdf[1] gt 0 then i2w, times, xpval, wdf[1], $
    /xydata, clab = outclab+' - XMax value', xlab = xl
 if wdf[2] gt 0 then i2w, times, ypval, wdf[2], $
    /xydata, clab = outclab+' - YMax value', xlab = xl
 if wdf[3] gt 0 then i2w, times, zpval, wdf[3], $
    /xydata, clab = outclab+' - ZMax value', xlab = xl
 if wdf[4] gt 0 then i2w, times, minval, wdf[0], $
    /xydata, clab = outclab+' - Minimum value', xlab = xl
 if wdf[5] gt 0 then i2w, times, aveval, wdf[0], $
    /xydata, clab = outclab+' - Average value', xlab = xl
 if wdf[6] gt 0 then i2w, times, sdval, wdf[0], $
    /xydata, clab = outclab+' - Std Deviation', xlab = xl
END
