; ------------------------------------------------------------------------
; $Id: get_surf_nodes.pro,v 1.1 2010/03/03 18:44:26 lpmix Exp $
; ------------------------------------------------------------------------
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
;+
; NAME:
;       GET_SURF_NODES
;
; PURPOSE:
;       Return a list of nodes fitting spacial specifications
;
; CATEGORY:
;       UTILITY
;
; CALLING SEQUENCE:
;
;       GET_SURF_NODES, NODE_ARRAY [, SURFACE_DATASET]
;
; OPTIONAL INPUTS:
;       SURFACE_DATASET:  PFF dataset number or unique string to designate the
;               topology dataset.  Dataset must have a type label of 
;               "Surface Topology".  Only the first 32 characters of the comment label
;               are checked.
;               If SURFACE_DATASET is zero or undefined, the following are done:
;                 If SURFACEID is a string or 64 bit integer, the value will be used to 
;                   select the dataset
;                 If SURFACEID is not valid, then a widget will be presented.
;               If SURFACE_DATASET is a REAL(4) or DOUBLE(5) array with two dimensions and
;               the first dimension is 3 then surface dataset is assumed to be the coordinates
;               of the nodes for the surface dataset.
;
; OUTPUTS:
;       NODE_ARRAY:  Array of nodes which meet the spatial requirements of 
;               XRANGE, YRANGE, ZRANGE
;               Note:  Node list begins at ZERO so that it may be used as a subscript
;               Note:  If NODE_ARRAY is not a valid variable, the procedure will exit.
;	
; KEYWORD PARAMETERS:
;       FILEID: File ID for the PFF file
;               DEFAULT:  FILEID = 0  (Current file)
;       NELEMENTS: Number of nodes
;       XRANGE: Two element vector with the minimum and maximum values of the First coordinate
;       YRANGE: Two element vector with the minimum and maximum values of the Second coordinate
;       ZRANGE: Two element vector with the minimum and maximum values of the Third coordinate
;       COORDINATES:  If valid, the coordinates of the NODE_ARRAY nodes are returned.
;               COORDINATES = FLTARR[NDIM, NELEMENTS]
;       SURFACEID: Surface ID for the surface.  This is returned as a string but may be 
;               converted to an unsigned 64bit integer with ULONG64(SURFACEID)
;               Note: If SURFACEID is input and valid, it will not be changed.
;               Note: If IARRAY is the integer array in the surface topology dataset, then:
;                 SURFACEID = strtrim(IARRAY[0], 2) + strtrim(string(format='(i4.4)', IARRAY[1]), 2)
;                 or
;                 SURFACEID = IARRAY[0]*10000LL + IARRAY[1]
;       EXACT:  If set exact ranges will be used.
;               DEFAULT: EXACT = 0
;               A buffer of 1/32768 of the range of the values will be used for the filter
;
; OUTPUTS:
;       The nodes meeting the required spatial constraints.
;
; EXAMPLE:
;       Find the nodes for surface 2 datasets of the current PFF file for radius < 0.1
;         GET_SURF_NODES, LIST, '2', XRANGE =[0, 0.1]
;       NOTE:  The value must be a string or the routine assumes it is a dataset number.
;
;       Find the nodes of Surface (2) and print the coordinates of the selected values 
;       in the interval xrange, yrange, and zrange.
;         GET_SURF_NODES, LIST, '(2)', XRANGE =xrange, YRANGE =yrange, ZRANGE = zrange, $
;                  COORDINATE = coord, NELEMENTS = nelem
;         FOR i = 0, NELEM-1 do print, list[i], coord[*, i]
;
;       Find the nodes of the surface in dataset 4 in file with FILEID of 3 with 
;       XRANGE=[0, 1], YRANGE=[0, 1], and ZRANGE=[-0.1, 0.1]
;         GET_SURF_NODES, LIST, 4, FILEID = 3, XRANGE=[0,1],YRANGE=[0,1],ZRANGE=[-0.1,0.1]
;
;       Find the nodes for a surface to be selected from a widget in the range Z > 0.1
;         GET_SURF_NODES, NODES, ZRANGE = [0.1, 10]  ; assume 10 is larger than all Z
;
; MODIFICATION HISTORY:
; 	Written by:	L. P. Mix, Jr., February 17, 2010
;	July, 1994	
;
;-
 pro get_surf_nodes, list, datasetin, fileid = fileid, xrange = xrange, yrange = yrange, $
             zrange = zrange, nelements = nelem, coordinates = coordinates, $
             surfaceid = surfaceidin, exact = exact
 on_error, 1
 
 if not arg_present(list) then begin
   print, '% GET_SURF_NODES: NODE_ARRAY must be a valid variable to return the list'
   return
 endif
 
 nelem = 0L
 list = -1 
 ier =0L
 if n_elements(datasetin) gt 0 then dataset = datasetin[0] else dataset = 0
 ss = size(datasetin)
 if (ss[0] eq 2) && (n_elements(dataset) gt 12) && (ss[1] eq 3) && ((ss[3] eq 4) $
   || (ss[3] eq 5)) then begin
   ndim = 3
   nnodes = ss[2]
   farray = temporary(dataset)
 endif else begin
   if n_elements(fileid) gt 0 then if fileid[0] ne 0 then setpff, fileid[0], ier
   if ier ne 0 then begin
     print, '% GET_SURF_NODES: Error setting file id to: '+strtrim(fileid[0], 2)
     return
   endif
   select = -1
   get_dir, comment= comment, type = type, number = ndset
   clab = strtrim(strmid(comment, 0, 32), 2) 
   oktype =  get_match('Surface Topology', type, nelem = nok)
   if nok eq 0 then begin
     print, '% GET_SURF_NODES: No datasets with "Surface Topology" dataset type'
     return
   endif
   if size(/type, dataset) lt 4 && dataset gt 0 then begin
    if dataset lt ndset then $
     if type [dataset-1] eq 'Surface Topology' then select = dataset-1
   endif else if size(/type, dataset) eq 7 then begin 
     ok = get_match(dataset, clab[oktype], nelem = nelem)
     if nelem eq 1 then select = oktype[ok[0]]
   endif
   if select lt 0 then begin
     ss = size(/type, surfaceidin) 
     if ss gt 0 then begin 
       surfaceid = surfaceidin
       if ss eq 7 then begin
         search = 'Surface Topology '+strtrim(surfaceid, 2)
         ok = get_match (search, comment[oktype], nelem= nelem)
         if nelem gt 0 then select = oktype[ok[0]]
       endif else if ss gt 13 then begin
         search = 'Surface Topology '+strtrim(surfaceid, 2)
         ok = get_match (search, comment[oktype], nelem= nelem)
         if nelem gt 0 then select = oktype[ok[0]]
       endif
       if select lt 0 then ss = 0
     endif
   endif
   if select lt 0 then begin
     instr= 'Pick the dataset with the desired surface'
     wselect_item, clab[oktype], select = select, /exclusive
     select = oktype[select]
   endif
; increment select to be a dataset
   select +=1
   readifl, iarray, flist, farray, select, clab = clab, ierror = ier
   if ier ne 0 then begin
     print, '% GET_SURF_NODES: Error reading dataset: '+strtrim(select, 2)
     return
   endif
   if ss eq 0 then $
     surfaceidin = strtrim(iarray[0], 2) + strtrim(string(format='(i4.4)', iarray[1]), 2)
   ndim = iarray [3]
   nnodes = n_elements(farray)/ndim
   farray = reform(farray, ndim, nnodes, /over)
 endelse
 if keyword_set(exact) then begin
   xdel = 0.0
   ydel = 0.0
   zdel = 0.0
 endif else begin
   xdel  = (max(farray[0, *], min = m) -m)/32768.
   ydel  = (max(farray[1, *], min = m) -m)/32768.
   if ndim gt 2 then zdel  = (max(farray[2, *], min = m) -m)/32768.
 endelse
 nelem = nnodes
 ind = lindgen(nnodes)
 if n_elements(xrange) gt 1 then begin
   xmin = min(xrange, max = xmax)
   ok = where((farray[0, *] ge xmin-xdel) and (farray[0, *] le xmax+xdel), nelem)
   if nelem eq 0 then begin
     print, '% GET_SURF_NODES: No datasets found(X)'
     return
   endif
   farray = farray[*, ok]
   ind = ind[ok]
 endif

 if n_elements(yrange) gt 1 then begin
   ymin = min(yrange, max = ymax)
   ok = where((farray[1, *] ge ymin-ydel) and (farray[1, *] le ymax+ydel), nelem)
   if nelem eq 0 then begin
     print, '% GET_SURF_NODES: No datasets found (Y)'
     return
   endif
   farray = farray[*, ok]
   ind = ind[ok]
 endif

 if (n_elements(zrange) gt 1) && (ndim gt 2) then begin
   zmin = min(zrange, max = zmax)
   ok = where((farray[2, *] ge zmin-zdel) and (farray[2, *] le zmax+zdel), nelem)
   if nelem eq 0 then begin
     print, '% GET_SURF_NODES: No datasets found (Z)'
     return
   endif
   farray = farray[*, ok]
   ind = ind[ok]
 endif
 list = temporary(ind)
 if ARG_PRESENT(coordinates) then coordinates = temporary(farray)
 farray = 0
END
