; ------------------------------------------------------------------------
; $Id: get_unique.pro,v 1.1 2010/03/03 18:46:24 lpmix Exp $
; ------------------------------------------------------------------------
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
;+
; NAME:
;       GET_UNIQUE
;
; PURPOSE:
;       Return a list of unique names in columns 1 to 32 in a PFF file
;       Optionally print this list
;       Note: Bounding Box is not included.
;
; CATEGORY:
;       UTILITY
;
; CALLING SEQUENCE:
;
;       GET_UNIQUE, UNIQUE_LABELS
;
; OUTPUTS:
;       UNIQUE_LABELS:  String array with the unique dataset comments in
;               columns 1 to 32.  
;	
; KEYWORD PARAMETERS:
;       FILEID: File ID for the PFF file
;               DEFAULT:  FILEID = 0  (Current file)
;
;       NELEMENTS: Number of unique labels
;
;       TRUNCATE: If set, all the comment will be truncated at the first 
;               occurance of more than 3 spaces.
;               DEFAULT:  TRUNCATE = 1
;
;       INCLUDE_BB: If set, include the bounding box.
;
;       PRINTIT:  If set, print the list.
;
; OUTPUTS:
;       The unique dataset names in the first 32 columns are returned in
;       a string array.
;       Optionally, these may be printed.
;
; EXAMPLE:
;       Find the unique datasets of the current PFF file.
;         GET_UNIQUE, LIST
;
;       Find the unique datasets of the PFF file with FILEID of 3 and print
;       the results.
;         GET_UNIQUE, LABELS, FILEID = 3, /PRINTIT
;
; MODIFICATION HISTORY:
; 	Written by:	L. P. Mix, Jr., February 17, 2010
;	July, 1994	
;
;-

 pro get_unique, list, fileid = fileid, truncate = truncate, printit= printit, $
              nelements = nelem, include_bb = includebb

 on_error, 1
 
 nelem = 0L
 list = ''
 ier =0L
 if n_elements(fileid) gt 0 then if fileid[0] ne 0 then setpff, fileid[0], ier
 if ier ne 0 then begin
   print, '% GET_UNIQUE: Error setting file id to: '+strtrim(fileid[0], 2)
   return
 endif
 get_dir, comment= comment, type = type, number = ndset
 if not keyword_set(includebb) then begin
   ok = get_match('bounding box', type, nel= nel)
   if nel gt 0 then begin
     if nel eq 1 and ok[0] eq 0 then comment= comment[1:*] else begin
       all = lonarr(ndset)
       all[ok] = -1
       ok = where (all eq 0,nok)
       if nok gt 0 then comment = comment[ok]
     endelse
     ndset -= nel
   endif
 endif
 if ndset eq 0 then begin
   print, '% GET_UNIQUE: No datasets found'
   return
 endif
 if (n_elements(truncate) eq 0) || keyword_set(truncate) then begin
   for i = 0, ndset-1 do begin
     c = strpos(comment[i], '    ') < 32
     if c gt 0 then comment[i] = strtrim(strmid(comment[i], 0, c), 2) else $
       comment[i] = strtrim(strmid(comment[i], 0, 32), 2)
   endfor
 endif else for i = 0, ndset-1 do comment[i] = strtrim(strmid(comment[i], 0, 32), 2)
 list = comment[ uniq(comment, sort(comment))]
 nelem= n_elements(list)
 if keyword_set (printit) then begin
   print, 'Unique Labels:'
   for i = 0, nelem-1 do print, i+1, '.  ', comment[i]
 endif 
END
