; ------------------------------------------------------------------------ 
; $Id: pff_info.pro,v 1.4 2010/04/13 18:28:19 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
 pro pff_info, fidin, fname = fname, ndset = ndset, dpoint = dpoint, $
          current = current, quiet = quiet, nfile = nfile, fid = fileid
;
;+
;     This routine gets pff file information 
;
;     format: pff_info [, FID] [, fname = fname] [, ndset = ndset]
;                     [, dpoint = dpoint] [, current = current] 
;                     [, nfile = nfile]  [,fid = fid] [, quiet = quiet]
; 
;     where
;       FID     - file id of pff file for which information is desired
;                 If FID is undefined or zero, data for all file is returned.
;                 NOTE: Only the positional parameter, FID, will be used to 
;                       specify pff files. (see second example)
;                 FID (positional parameter) is not modified.
;       fname   - array of file names (maximum of 255 characters for each)
;       fid     - keyword used to return an array of file ids.
;       ndset   - Number of datasets in each file
;       dpoint  - Dataset pointer location
;       current - integer array = 0 for all files except CURRENT file = 1 
;                 or if an error Current = -1 if error on the PFF_INFO
;       For example: current = [0, 0,-1, 1] indicates:
;                              First two files had good data but are not current
;                              Third file had a problem with PFF_INFO
;                              Fourth file is current file
;       Nfile   - Integer, number of open files
;       quiet   - If present and not zero, then no data will be printed
;
;
;     Example
;       Determine the number of open files and other pff parameters
;          pff_info
;
;       Get arrays of the fids and the file names for all open files in 
;          arrays fid and name, but print nothing to the screen.
;          (Since the positional parameter is undefined, all file information
;           is returned.)
;            pff_info, fid = fid, fname = fname, /quiet
;-
 on_error, 1
      common pffcom, initpff, pff_exe
;
;     check to see if pff has been initialized
;
      nfile = 0L 
      if (n_elements(initpff) eq 0) then initpff=0
      if (initpff eq 0) then begin
        print, '% PFF_INFO: No PFF files are open'
        return
      endif
;
;     check if the FID is defined
;
      if n_elements (fidin) eq 0 then fid = 0L else fid = long(fidin)
      if fid(0) ne 0 then begin
         fileid = long(fid)
         next = -1L
      endif else begin
         fileid = 1L
         next = 0L
      endelse
      nfidar = n_elements(fileid)
      fid = fileid(0)
      i = 0L
      ndsetf = 0L
      dpointf = 0L
      currentf = 0L
      nfile = 0L 
      ier = 0L
;
AGAIN:
      lstr = 256L
      fnamef = make_array(lstr, /byte)
;   use next as a flag 
;   if next = -1 then fid contains a valid file id
;   if next ge 0 then fid represents pff file id (1-mxfils)
;      if next eq 0 then first valid file is taken and next returns with
;         the next valid pff file id or -1 if all are gone.
      if initpff eq 1 then begin
; HP
        junk=call_external(PFF_EXE,'pff_slgetinfo', fid, fnamef, ndsetf, $
             dpointf, currentf, next, nfile, ier, lstr)
      endif else if initpff eq 2 then begin
; Sun
        junk=call_external(PFF_EXE,'pff_slgetinfo_', fid, fnamef, ndsetf, $
             dpointf, currentf, next, nfile, ier, lstr)
      endif else if initpff eq 4 then begin
; WIN
        junk=call_external(PFF_EXE,'_PFF_SLGETINFO@8', fid, fnamef, ndsetf, $
             dpointf, currentf, next, nfile, ier, lstr)
      endif else if initpff eq 5 then begin
; MAC
        junk=call_external(PFF_EXE,'pff_slgetinfo', fid, fnamef, ndsetf, $
             dpointf, currentf, next, nfile, ier, lstr)
      endif else if initpff eq 3 then begin
; VMS
        fnamef = string (fnamef)
        junk=call_external(PFF_EXE,'pff_getinfo', fid, fnamef, ndsetf, $
             dpointf, currentf, next, nfile, ier)
      endif
      if nfile eq 0 then begin
        print, '% PFF_INFO - No pff files are open'
        return
      endif
      if i eq 0 then begin
        if next lt 0 then nfids = nfidar else nfids = nfile
        if nfids gt 1 then begin
          current= make_array(nfids, /long)
          ndset= make_array(nfids, /long)
          dpoint= make_array(nfids, /long)
          current= make_array(nfids, /long)
          fname= make_array(nfids, /string)
          if n_elements(fileid) ne nfids then fileid = make_array(nfids, /long)
        endif
      endif
      if ier ne 0 then begin
        print, '% PFF_INFO: ERROR : FID = ', strtrim(fid)
        currentf = -1L
      endif
      if nfids gt 1 then begin
        current(i) = currentf
        fname(i) = strtrim(fnamef, 2)
        ndset(i) = ndsetf
        dpoint(i) = dpointf
        fileid(i) = fid
      endif else begin
        current = currentf
        fname = strtrim(fnamef, 2)
        ndset = ndsetf
        dpoint = dpointf
        fileid = fid
      endelse
      i=i+1
      if i lt nfids then begin
        if next lt 0 then fid = fileid(i) else fid = next
        goto, again
      endif
      if not keyword_set (quiet)  then begin
        print, ' Current  FID  #-Dsets  Dpoint  File_Name'
        cc = make_array(/string, nfids, value = '       ')
        ok = where(current eq 1, nok)
        if nok gt 0 then cc(ok) = ' ====>>'
        for i=0,nfids-1 do print,cc(i),fileid(i),ndset(i),dpoint(i),fname(i), $
        format = '(a7, i5, i8,i8, 3x, a)'
      endif
      fid = fileid
      return
      end
