; ------------------------------------------------------------------------ 
; $Id: readcon.pro,v 1.8 2010/03/03 20:37:32 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
 Pro readcon, array, datasetin, ndset, fileid = fid, clabel = clabel_in, $
     solid_model = solid_model,  vert_add= vert_add
;
;123456789;123456789;123456789;123456789;123456789;123456789;123456789;123456789
;+
;
;  Reads a CONDUCTOR dataset into an IDL structure.
;  Procedure options are explained below. PFF dataset type must be VTX
;
;  Format:  READCON, structure  [, dataset] [, number]
;             [, FILEID = fileid] [, CLABEL = clabel]
;             [, SOLID_MODEL = solid_model] [, VERT_ADD = vert_add]
;  where:
;    structure - IDL structure name
;                If not a valid variable, data is stored in GET_MAXSTR()-1
;    dataset - indicates a PFF dataset number 
;              or is a search string for dataset comments.  
;              If dataset is missing or zero, then it is set to the 
;              current dataset.  If dataset is an array then all datasets 
;              will be read into the same structure.
;    number  - If present, then a total of number datasets will be read and 
;              stored in the structure
;        Note: If dataset is a string, a leading `^' indicates that 
;              the string begins at the start of the dataset comment.
;    fileid -  file id of the PFF file containing the data
;    clabel -  If CLABEL has a length gt 0, it will be used as the COMMENT label
;              Note: CLABEL = ' ' will result in a blank value for clabel
;                    CLABEL = '' will result in the default value
;    solid_model - If set store the planes for the geometry with the 
;              connectivity information
;              The following structure labels are added when SOLID_MODEL is set
;              space : polygon coordinates 
;                      Each conductor is a series of planar polygons.
;                      In QS each polygon is a rectangle
;              Connectivity : Connectivity description required to make an 
;                      IDLgrPolygon object
;              loc_con : Pointer array to the beginning and end of the polygons
;                      which are normal to no axis, and those normal to the 
;                      X, Y and Z axis.
;    Vert_add - Additional vertices to add to the model
;               Current version requires 4 points (rectangular planes)
;
;  Structure Labels
;    type  - 4
;    ndim  - number of spatial dimensions = 3
;    block - number of blocks = 1
;    size  - number of planes according to the following
;            size (0) = number not normal to any axis
;            size (1) = number normal to x
;            size (2) = number normal to y
;            size (3) = number normal to z
;            size (4) = Total number of planes
;    plane(2,3,size(4))
;           - array of the min and max in each dimension 
;    n0, n1, n2, n3 array of indices with planes normal to no axis, x, y,or z axis
;    slant(3,2,size(0)) - opposite corners of a slant surface 
;                         if vect(0:3) are read, slant = [vect(0), vect(2)]
;    slnorm(3, size(0)) - unit slant normal from point slant(*, 0, i)
;                         unit cross product of vector(0, 1) X vector(0, 3)
;                         where vector(0, 1) = vect(1)-vect(0) and 
;                               vector(0, 3) = vect(3)-vect(0)
;    xlab(3)- spatial labels
;    tlab  - type label
;    clab  - dataset comment
;    file  - file data read from
;    spare - integer array (5)
;
;  Examples:
;    Read dataset 3 in file id 4 into structure c1
;      readcon, c1, 3, fil=4
;    Read next dataset of the current file into structure anode
;      readcon, anode
;    read datasets 3, 4, 5 into structure c2
;      readcon, c2, 3, 3 
; (or) readcon, c2, [3,4,5]
;-
;
;
@qscom.dat
;
       forward_function  make_planes
; compile_opt strictarr
      on_error, 1
      lcstructure = 0
      if not arg_present(array) then  begin
        print, '% READCON: Warning data cannot be returned in variable'
        print, '           Data will be stored in structure '+strtrim (get_maxstr()-1, 2)
        lcstructure = 1
      endif
      array = 0.0
      if (n_params() lt 1 ) then begin
        print, ' Structure must be specified'
	return
      endif
      if n_elements (datasetin) eq 0 then dataset=0 else dataset=datasetin
      if n_elements (ndset) eq 0 then ndset = 0
      if n_elements (fid) eq 0 then fid = 0
      nd = n_elements (dataset)
      if nd gt 1 then begin
        if ndset eq 0 then print, 'Number of dataset to be read set to', nd
        ndset = nd
      endif else ndset = abs(ndset)
      nd = ndset > nd
      blo = 1L
      pid = long(fid)
      nmap = 0L
      if ndset gt 0 then nmax = 1 else nmax = 100
      get_dset_string, dataset,pid,  map, nmap, nmax, ier
      if ier ne 0 then message, ' Please change the dataset specification'
;
      dataset = map
;
      p = lindgen(4)
; make a dummy plane
      nslant = 0L
      mxslant = 4096L
      slant = make_array(mxslant*6L)
      vecnormal = make_array(mxslant*3L)
      tot = 0L
      if nd gt  n_elements(dataset)  then dataset = indgen(nd)+ dataset[0]
      nd = n_elements(dataset)
    for ii = 0, nd-1 do begin 
      direct = dataset[ii]
      readvtx, fileid = pid, space, direct, namefile = file, $
         pfftyp =pfftyp, aptype = aptype, spare = spare, /conductor, $ 
         xlabel = xlabel, ylabel = ylabel, zlabel = zlabel, clabel = clab, $
         tlabel = tlab
;
      if n_elements (pfftyp) eq 0 then begin
        print, '% READCON: Error reading dataset'
        return
      endif
      if pfftyp ne 5 then begin
        print,'% READCON: PFF dataset type must be 5 - type VTX: ', pfftyp
        return
      endif
      b = size (space)
      if b[0] ne 2 or b[1] ne 3 then begin
        print, '%READCON: Error reading dataset', direct
        Print,'     Space array must have two dimensions with 3 elements in the first'
        Print,'     Size of space is', size(space)
        return
      endif
      ndim = 3L
;     only add one time - check ii
      if n_elements(vert_add) gt 0 and ii eq 0 then begin
        ss = size(vert_add) 
        if ss[0] ne b[0] then begin
          print, '% READCON: Size of the vertex array is incorrect.'
          help, space, vert_add
          return
        endif
        space = [reform(space, b[b[0]+2], /overwrite), $
               reform (vert_add, ss[ss[0]+2] , /overwrite)]
        space = reform(space, ss[1], ss[2]+b[2])
        b = size(space)
      endif
      if ii eq 0 then begin
        xlab = [xlabel[0], ylabel[0], zlabel[0]]
        spar = spare
        clabel = clab[0]
;       spar[1] = 4
      endif else clabel = clabel + ', '+ clab
      totinc  =b[2]/4
      vtotal = 4*totinc
      if b[2] ne vtotal then begin
        print, '% READCON: Problem with the conductor dataset: ', $
          strtrim(direct, 2)
        print, '           '+strtrim (b[2]-vtotal,2)+ $
         ' Extra vertex values will be deleted' 
        b[2] = vtotaL
        b[4] = vtotal*3
      endif
      if tot eq 0 then begin
         plane = make_array(6*totinc)
         normal = make_array(totinc, /int)
      endif else begin
         plane = [plane , make_array(6*totinc)]
         normal = [normal, make_array(totinc, /int)]
      endelse
      maxsp = max(space, min= minsp)
;     err = 1.52588e-5*max([maxsp-minsp, abs(maxsp), abs(minsp)])
      err = 7.62939e-06*max([maxsp-minsp, abs(maxsp), abs(minsp)])
      for j = 0L, b[2]-1, 4 do begin
        minx = min(space[0, j+p], max = mx)
        miny = min(space[1, j+p], max = my)
        minz = min(space[2, j+p], max = mz)
        k = (tot+j/4)
        k6 = 6*k
; rather than taking the cross product - look for equal values.
        plane[k6:k6+5] = [minx, mx,miny, my, minz, mz]
        if(mx - minx) lt err then normal[k] = 1 else $
        if(my - miny) lt err then normal[k] = 2 else $
        if(mz - minz) lt err then normal[k] = 3 else begin
          if nslant+1 gt mxslant then begin
            mxslant = mxslant+mxslant
            slant = [slant, slant]
            vecnormal= [vecnormal, vecnormal]
          endif
          slant[nslant*6] = [space[*, j], space[*, j+2]]
;
;    slnorm[3, size[0]] - unit slant normal from point slant(*, 0, i)
;                         unit cross product of vector(0, 1) X vector(0, 3)
;                         where vector(0, 1) = vect(1)-vect(0) and 
;                               vector(0, 3) = vect(3)-vect(0)
;    store as vecnormal
;
          veca = space[*, j+1]- space[*, j]
          vecb = space[*, j+3]- space[*, j]
          vecaxb = [veca[1]*vecb[2]-veca[2]*vecb[1], $
                   veca[2]*vecb[0]-veca[0]*vecb[2], $
                   veca[0]*vecb[1]-veca[1]*vecb[0]]
          vecnormal[nslant*3] = [vecaxb/sqrt(total(vecaxb*vecaxb))]
          nslant = nslant+1
          normal[k] = 0
        endelse
      endfor
      tot = tot+totinc
;
      if keyword_set(solid_model) then begin
        nelsp= n_elements(space_save)
        if nelsp lt 3 then space_save = temporary(space) else begin
          ssp = size(space_save)
          if ssp[0] eq 2 then $
            space_save = reform (space_save, ssp[4], /overwrite)
          space_save = [space_save, reform(space, b[4], /overwrite)]
          space = 0
        endelse
      endif else space = 0
    endfor
      tot = long(tot)
      if tot eq 0 then begin
        Print, '% READCON: Problem reading the conductor data, No planes found'
        return
      endif
      nel = n_elements(space_save)
;
      nsize = make_array(5, /long)
      plane = reform (plane, 2, 3, tot, /overwrite)
      nsize [4] = tot
      n0 = where (normal eq 0, ns)
       nsize[0] = ns
      n1 = where (normal eq 1,ns)
       nsize[1]= ns
      n2 = where (normal eq 2,ns)
       nsize[2] = ns
      n3 = where (normal eq 3, ns)
       nsize[3] = ns
      
      if keyword_set(solid_model) then begin
        nel = n_elements(space_save)
        space_save = reform (space_save,3, nel/3, /overwrite)
        connectivity = make_planes(space_save, n0, n1, n2, n3, loc_con)
      endif else begin
        space_save = 0
        connectivity=0
        loc_con=0
      endelse
      if nslant gt 0 then begin
        slant = temporary(slant[0:(6*nslant-1)])
        slant = reform (slant,3,2,nsize[0], /overwrite)
        vecnormal = temporary(vecnormal[0:(3*nslant-1)])
        vecnormal = reform (vecnormal,3,nsize[0], /overwrite)
      endif else begin
        slant = 0.0
        vecnormal = 0.0
      endelse
      if n_elements(clabel_in) gt 0 && strlen(clabel_in) gt 0 then $
        clabel = strtrim(clabel_in,2)
      array = {type:4L, ndim:ndim,  block :1L, $
              size:nsize, plane : plane, slant : slant, slnorm:vecnormal, $
              n0 :n0, n1:n1, n2:n2, n3: n3, xlab : xlab, $
              space: space_save, connectivity : connectivity, loc_con:loc_con, $
              file : file, spare : spar, tlab : tlab[0], clab: clabel}
      if lcstructure then i2s, array, get_maxstr()-1, /no_copy
      end
