; ------------------------------------------------------------------------ 
; $Id: readfld.pro,v 1.7 2010/03/05 14:52:45 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
 Pro readfld, array, datasetin, fileid = fid, stringlen= stringlen, $
       get_grid = grid, tlabel= tlabel, clabel = clabel, $
       finite_element = finite_element, namefile = filename, $
       pfftyp = pfftyp,  rectilinear = rectilinear, error = error, $
       skip = skip, dlength = dlength, iextra = iextra, _extra = extrastuff

;
;+
;
;  Reads a field or charge density into an IDL structure.
;  Procedure options are explained below. PFF dataset type must be 
;  7 N-Dimensional Vector on M-spatial dimensions or 3-Dimensional 
;  vector or scalar data.
;  Note - TWO QUICK data is stored as 3D data
;  Format:  READFLD, structure  [, dataset] [, FILEID = fileid]
;           [, Finite_Element= finite_element] [, RECTILINEAR = rectilinear]
;           [, GET_GRID = get_grid] [, STRINGLEN = stringlen] [, ERROR = error]
;
;  where:
;    structure - IDL structure name
;                If not a valid variable, data is stored in GET_MAXSTR()-1
;    dataset - indicates a PFF dataset number 
;              or is a search string for dataset comments.  
;              If dataset is missing or zero, then it is set to the 
;              current dataset.
;        Note: If dataset is a string, a leading `^' indicates that 
;              the string begins at the start of the dataset comment.
;    fileid -  file id of the PFF file containing the data
;    stringlen - Currently ignored
;                Maximum number of characters in the spatial labels and the
;                field labels. Currently 64 characters.
;    finite_element - If set mach2 data will be stored as finite element data.
;    get_grid       - If finite element, reads the grid
;    rectilinear- Set to 1 if data is rectilinear 
;    If old vertex format and range of x, y differ by 200, linear interpolation.
;    ERROR    - Error parameter used to determine if Vertex data is rectilinear.
;               Default value = 1.55E-5 of data range. 
;
;  Structure Labels
;    type  - 1
;    ndim  - number of spatial dimensions
;    nvect - number of vector dimensions
;    block - number of blocks 
;    two   - two dimensional missing dimension
;            this is 0 for 3d data, 1 for yz data, 2 for zx data, 3 for xy data
;    slice - value of the missing dimension, if known. Default is zero.
;    size  - array of the size of the spatial dimensions
;            size = size(ndim, block)
;    range - (2, ndim, nblock+1) two dimensional array of the max and min for
;            each block.  Block (0) has overall values.
;    order - (ndim, nblock) Ordering of blocks in each dimension by initial 
;            value of the block, in units of block -1
;    locx  - array of the location of the first point for each block
;            spatial array- locx = locx(ndim, block+1)
;            (first spatial dimension for block m is x and is defined by
;             x = x1(locx(0, m-1): locx(0, m)-1) )
;    locv  - array of the location of the first point for each block
;            vector array- locv = locv(block+1)
;            (first vector array for block m is v and is defined by
;            v = reform (v1(locv(m-1):locv(m)-1),size(*,m-1))
;    x1, x2, ...x(ndim) - spatial arrays
;    v1, v2, ...v(nvect) - vector arrays
;    x1lab, x2lab, ...x(ndim)lab - spatial labels
;    v1lab, v2lab, ...v(nvect)lab - vector labels, Vector label in the case of
;                  NGD data and block label in the case of vector and 3d data
;    tlab  - type label
;    clab  - dataset comment
;    file  - file data read from
;    spare - integer array (5)
;  Note: If block > 1 spatial and vector arrays are reformed to linear 
;        arrays and stored in the respective arrays.
;  Examples:
;    Read dataset 34 in file id 4 into structure ch5
;      readfld, ch5, 34, fil=4
;    Read next dataset of the current file into structure efldstart
;      readfld, efldstart
;-
;
      on_error, 1
;
      common pffcom, initpff, pff_exe
;
;     check to see if pff has been initialized
;
@pff_strings.com

      rectilinear = 1
      if n_elements(initpff) eq 0 then initpff = 0
      if (initpff eq 0) then begin
        print, '% READFLD: File must be opened before reading data'
        return
      endif
;
;     check on the number of parameters
;
      stringlen = max_str_len
      nparams =  n_params()
      slice = 0.0
      npar = 7
      if (nparams lt 1 ) then begin
        print,'% Format: READFLD, structure  [, dataset] [, FILEID = fileid]'
        print,'             [, /GET_GRID]'
        return
      endif
      lcstructure = 0
      if not arg_present(array) then  begin
        print, '% READFLD: Warning data cannot be returned in variable'
        print, '           Data will be stored in structure '+strtrim (get_maxstr()-1, 2)
        lcstructure = 1
      endif
      array = 0.0

      if n_elements (fid) eq 0 then fid = 0L
      pid = long(fid)
      if n_elements (datasetin) eq 0 then dataset=0L else dataset=datasetin(0)
      if keyword_set(skip) then begin
        if n_elements(dlength) ne 1 or n_elements (iextra) lt 8 or $
        n_elements (pfftyp) ne 1 then message, 'Error with keyword SKIP'
      endif else begin
        nmap = 0L
        nmax = 1L
        get_dset_string, dataset,pid,  map, nmap, nmax, ier
        if ier ne 0 then message, ' Please change the dataset specification'
;
        if nmap gt 1 then begin
         Print,'% READFLD: Please change the dataset specification to get a single dataset.'
          help, datasetin  
          print, 'Datasets: '
          for i = 0, nmap-1 do print, map[i]
          return
        endif

        dataset = map(0)
;
        readhdr,dataset,ier,fileid=pid, pfftyp = pfftyp, version = version, $
           dlength = dlength, tlabel= tlabel, clabel=clabel, $
           namefile = filename, iextra = iextra, /quiet
;
        if ier ne 0 or n_elements(pfftyp) eq 0 then begin
          print, ' Error reading dataset header'
          return
        endif
      endelse
      triangles = 0
      loc_triangles = 0
      if (pfftyp  eq 7 ) then begin
        readngd, array, dataset, fileid=pid, pfftyp = pfftyp, $
           dlength = dlength, tlabel= tlabel, clabel=clabel, $
           namefile = filename, stringlen= stringlen, iextra = iextra, $
           /skip
        if size(/type, array) eq 8 then $
          if lcstructure then i2s, array, get_maxstr()-1, /no_copy
        return
      endif else if (pfftyp  eq 4 ) then begin
        readnv3, array, dataset, fileid=pid, pfftyp = pfftyp, $
           dlength = dlength, tlabel= tlabel, clabel=clabel, $
           namefile = filename, stringlen= stringlen, iextra = iextra, $
           /skip
        if size(/type, array) eq 8 then $
          if lcstructure then i2s, array, get_maxstr()-1, /no_copy
        return
      endif else if (pfftyp  eq 1 ) then begin
        readuf3, array, dataset, fileid=pid, pfftyp = pfftyp, $
           dlength = dlength, tlabel= tlabel, clabel=clabel, $
           namefile = filename, stringlen= stringlen, iextra = iextra, $
           /skip
        if size(/type, array) eq 8 then $
          if lcstructure then i2s, array, get_maxstr()-1, /no_copy
        return
      endif else if (pfftyp  eq 3 ) then begin
        readnf3, array, dataset, fileid=pid, pfftyp = pfftyp, $
           dlength = dlength, tlabel= tlabel, clabel=clabel, $
           namefile = filename, stringlen= stringlen, iextra = iextra, $
           /skip, _extra = extrastuff
        if size(/type, array) eq 8 then $
          if lcstructure then i2s, array, get_maxstr()-1, /no_copy
        return
      endif else if (pfftyp  eq 9 ) then begin
        readni3, array, dataset, fileid=pid, pfftyp = pfftyp, $
           dlength = dlength, tlabel= tlabel, clabel=clabel, $
           namefile = filename, stringlen= stringlen, iextra = iextra, $
           /skip
        if size(/type, array) eq 8 then $
          if lcstructure then i2s, array, get_maxstr()-1, /no_copy
        return
      endif else if (pfftyp eq 5) then begin
        readvtx, attr, dataset,  fileid=pid, pfftyp = pfftyp, $
           dlength = dlength, tlabel= tlabel, clabel=clabel, $
           namefile = filename, stringlen= stringlen, iextra = iextra, $
           ndim = ndim, nattr = nattr, xlabel = x1lab, ylabel = x2lab, $
           zlabel = x3lab, xarray = x1, yarray = x2, zarray = x3,  $
           tarray = x4, x4lab = x4lab, alabel = alabel, spare = spare, $
           aptype = aptype, /skip

        if ((ndim lt 2) or (nattr lt 1)) then begin
           print,'% READFLD: Field dataset must have 2 dimensions and ' $
             , '1 attribute:', ndim, nattr
           array = 0
           return
        endif
        if ndim lt 3 then begin
          two = 3L
          x3 = 0.0
          x3lab = ''
          if ndim lt 4 then begin
            x4 = 0.0
            x4lab = ''
          endif
        endif else two = 0L
        if nattr lt 1 then begin
          print, '% READFLD: Vertex field data must have 1 or more attribute.'
          return
        endif

        v1 = attr(*, 0)
        v2 = 0
        v3 = 0
        v4 = 0
        v5 = 0
        v6 = 0
        if nattr gt 1 then begin
          v2 = attr(*, 1)
          if nattr gt 2 then begin
            v3 = attr(*, 2)
            if nattr gt 3 then begin
              v4 = attr(*, 3)
              if nattr gt 4 then begin
                v5 = attr(*, 4)
                if nattr gt 5 then v6 = attr(*, 4)
              endif
            endif
          endif
        endif
        clabel_save = clabel
        if n_elements(error) eq 0 then error = 1.55e-5
        if version gt 0 then error = -error
        get_rect_xyz,  x, nx, y, ny, z, nz, x1, x2, x3, order, delta = error, $
            ier, /quiet
        ntot = nx*ny*nz

        if (ier ne 0) and (version lt 1) and not keyword_set(finite_element) $
          then begin
          if ndim eq 2 then rang = max([x1, x2], min=m)-m else $
          rang = max([x1, x2, x3], min = m)-m 

          rx =  max(x1, min = m)-m 
          ry =  max(x2, min = m)-m 
          if ndim eq 2 then rz = rang else rz =  max(x3, min = m)-m 

;          something might be broken.
;          last half of this was added 2/22
;            changed my mind - delete the next line
;            and added a bunch of stuff.
;         if (max(order) eq ntot-1) and (n_elements(v1) ge (ntot-1)) then ier=0
          nvert = n_elements(x1)
          limit =   200
;         We will only interpolate if greater than a factor of 200 is lost
          errx = rang/rx
          erry = rang/ry
          errz = rang/rz
          if (errx gt limit) or (erry gt limit) or (errz gt limit) $
            and ndim eq 2  then begin
            lc = strlen(clabel)
            ok = strpos(clabel, ';;', 0)
            nb=0
            if ok gt 0 then begin
              if (ok+2) lt lc then bs = strmid(clabel, ok+2,lc) else bs = ''
              lc = strlen(bs)
              if lc gt 0 then begin
                ss = make_array(ndim, 32, /long)
                ds = make_array(ndim, /long)
                while lc gt 0 do begin
                  reads, bs, ds
                  ss(*, nb) = ds
                  nb = nb+1
                  ok = strpos(bs, ';;', 0)
                  if ok+2 lt lc then bs = strmid(bs, ok+2,lc) else bs = ''
                  lc = strlen (bs)
                endwhile
              endif
            endif 
            tot = 0L
            if nb gt 0 then begin
              np = make_array(nb, /long)
              for i = 0, nb-1 do begin
                n= ss[0, i]*ss[1, i]
                if ndim gt 2 then n = n*ss[2, i]
                np[i] = n
                tot = tot+n
              endfor
            endif
            
            if tot eq nvert  and ndim eq 2 THEn begin
              rang_err = rang/32000
              ii = 0L
              for i = 0, nb-1 do begin
                if errx gt limit then begin
                  xx = x1[ii: (ii+np[i]-1)]
                  xx = reform(xx, ss[0, i], ss[1, i], /overwrite)
                  xt = reform(xx[*, 0])
                  equal = 1
                  j = 1L
; see if rectilinear data
                  while equal eq 1 and j lt  ss[1, i]-1 do begin
                    if max(abs(xx[*, j]-xt)) gt rang_err then equal = 0
                    j = j+1
                  endwhile
                  if equal gt 0 then begin
                    order = (ss[0, i]-2) < 5
                    xfit = findgen(ss[0, i])/(ss[0, i]-1)
                    junk = svdfit(xfit, xt/rx, order, yfit = xt,  $
                       /double)
                    xt = xt*rx
                    for j = 0, ss[1, i]-1 do xx[*, j] = xt
                    x1[ii: (ii+np[i]-1)] = xx
                  endif
                endif
                if erry gt limit then begin
                  xx = x2[ii: (ii+np[i]-1)]
                  xx = reform(xx, ss[0, i], ss[1, i], /overwrite)
                  xt = reform(xx[0, *])
                  equal = 1
                  j = 1L
; see if rectilinear data
                  while equal eq 1 and j lt  ss[0, i]-1 do begin
                    if max(abs(xx[j, *]-xt)) gt rang_err then equal = 0
                    j = j+1
                  endwhile
                  if equal gt 0 then begin
                    order = (ss[1, i]-2) < 5
                    xfit = findgen(ss[1, i])/(ss[1, i]-1)
                    junk = svdfit(xfit, xt/rx, order, yfit = xt,  $
                       /double)
                    xt = xt*rx
                    for j = 0, ss[0, i]-1 do xx[j, *] = xt
                    x2[ii: (ii+np[i]-1)] = xx
                  endif
                endif
                ii = ii +np[i]
              endfor 

              get_rect_xyz,  x, nx, y, ny, z, nz, x1, x2, x3, order, $
                  delta = -error, ier, /quiet

            endif 
          endif
        endif
        if keyword_set (grid) then begin
;         set attributes to zero - we don't need them.
          attr = 0
          if ier eq 0  and not keyword_set (finite_element) then begin
            mblo = 1
            itype = 1L
            triangles = 0L
            loc_triangles = 0L
            x1 = x
            x2 = y
            x3 = z
            ssize = [nx, ny, nz]
            ndim = 3
             
            locx =make_array(ndim, mblo+1, /long, value =0L)
            locx(*, 1)= ssize
            range = make_array(2, ndim, mblo+1)
            range(0, 0, *) = min(x, max = mx)
            range(1, 0, *) = mx
            range(0, 1, *) = min(y, max = mx)
            range(1, 1, *) = mx
            range(0, 2, *) = min(z, max = mx)
            range(1, 2, *) = mx
            x1lab = make_array (mblo, value = x1lab)
            x2lab = make_array (mblo, value = x2lab)
            x3lab = make_array (mblo, value = x3lab)

            array = {type:3L, ndim:ndim,  block :mblo, two:two, range:range, $
              size:ssize, locx : locx,  x1:x1, x2:x2, x3:x3, $
              x1lab : x1lab, x2lab :x2lab, x3lab : x3lab, $
              file : filename, spare :spare, tlab : tlabel, clab: clabel}
          endif else begin
            nx = n_elements(x1)
            if ndim gt 2 then begin
              print, '% READFLD: Only two dimensional finite element data may be read'
              return
            endif
            tlab = 'Mach2 Grid Data'
            type = 10L
            nx = n_elements(x1) 
            locx= make_array(2, /long)
            ii = strpos(clabel, 't =')
            if ii lt 0 then dii = 2 else dii =3
            if ii lt 0 then ii = strpos(clabel, 't=')
            if ii gt 0 then begin
              jj = strpos(clabel, ' ', ii+5)
              if jj lt 0 then jj = strpos(clabel, ',', ii+5) 
              if jj lt 0 then jj = strpos(clabel, ';', ii+5) 
              if jj gt 0 then x4 = float(strmid(clabel, ii+dii, jj-ii+dii))
            endif else x4 = 0.0
                  
            lc = strlen(clabel)
            ok = strpos(clabel, ';;', 0)
            if ok gt 0 then begin
              if (ok+2) lt lc then bs = strmid(clabel, ok+2,lc) else bs = ''
              lc = strlen(bs)
              clabel = strtrim(strmid(clabel, 0,ok), 2)
              if lc gt 1 then begin
                ssize = make_array(ndim, 128, /long)
                locx = 0L
                loc_triangles = make_array(128, /long)
                mblo = 0
                ii = make_array(ndim, /long)
                range=make_array(2, ndim, 128)
                while lc gt 1 do begin
                  reads, bs, ii
                  ssize(*, mblo)=ii
                  mblo = mblo +1
                  if mblo eq 1 then begin
                    triangles = make_triangles(0, ii(0), ii(1), /quad)
                    ss = size(triangles)
                    triangles = reform(triangles, 4*ss(2), /overwrite)
                    offset = ii(0)*ii(1)
                    offinit = 0L
                  endif else begin
                    tri = make_triangles(0, ii(0), ii(1), /quad)+offset
                    offinit= offset
                    offset =offset + ii(0)*ii(1)
                    ss = size(tri)
                    triangles = [triangles, reform(tri, 4*ss(2), /overwrite)]
                  endelse

                  pp = lindgen(ss(2)+1)*4 
                  if mblo gt 1 then pp = pp + locx[loc_triangles[mblo-1]-1]
                  locx = [locx, pp(1:*)]
                  loc_triangles(mblo) = n_elements(locx)
                  range(0, 0, mblo) = min(x1(offinit:offset-1), max = mx)
                  range(1, 0, mblo) = mx
                  range(0, 1, mblo) = min(x2(offinit:offset-1), max = mx)
                  range(1, 1, mblo) = mx
                  ok = strpos(bs, ';;', 0)
                  if ok gt 0 then begin
                    if (ok+2) lt lc then bs = strmid(bs, ok+2,lc) else bs=''
                      lc = strlen(bs)   
                  endif else lc = 0
                endwhile
                range = range(*, *, 0:mblo)
                ssize = ssize(*, 0:mblo-1)
                loc_triangles = loc_triangles(0:mblo)
                range(0, 0, 0) = min(range(0, 0, 1:mblo))
                range(1, 0, 0) = max(range(1, 0, 1:mblo))
                range(0, 1, 0) = min(range(0, 1, 1:mblo))
                range(1, 1, 0) = max(range(1, 1, 1:mblo))
              endif 
            endif else begin
              print, '% READFLD: Unable to read grid data'
              return 
            endelse


            array = {type: type, ndim:ndim, block:mblo, two:two, $
               ntime:1L,  slice: 0.0, size:ssize, range: range, $
                order: 0L, locx:locx, locv: loc_triangles, $
                x1:x1, x2:x2, x3:x3, x4:x4, connect:triangles, $
                x1lab : x1lab, x2lab :x2lab, x3lab : x3lab, $
                x4lab : x4lab, file : filename, spare : spare, $
                tlab : tlab, clab:clabel   }
          endelse
          if size(/type, array) eq 8 then $
            if lcstructure then i2s, array, get_maxstr()-1, /no_copy
          return
        endif
        if ier eq 0  and not keyword_set (finite_element) then begin
          mblo = 1
          itype = 1L
          triangles = 0L
          loc_triangles = 0L
          x1 = x
          x2 = y
          x3 = z
          if ndim eq 2 then ssize = [nx, ny, 1] else ssize = [nx, ny, nz]
          locx = make_array(3, mblo+1, /long)
          locx(*, 1) = ssize
          ntot = nx*ny*nz
          locv = [0, ntot]
          v = temporary(v1)
          v1 = make_array(ntot)
          v1(order) = v
          if nattr gt 1 then begin
          v = temporary(v2)
          v2 = make_array(ntot)
          v2(order) = v
          if nattr gt 2 then begin
            v = temporary(v3)
            v3 = make_array(ntot)
            v3(order) = v
            if nattr gt 3 then begin
              v = temporary(v4)
              v4 = make_array(ntot)
              v4(order) = v
              if nattr gt 4 then begin
                v = temporary(v5)
                v5 = make_array(ntot)
                v5(order) = v
                if nattr gt 5 then begin
                  v = temporary(v6)
                  v6 = make_array(ntot)
                  v6(order) = v
                endif
              endif
            endif
          endif
          endif
          v=0
        endif else if aptype eq 15 and not keyword_set(finite_element) then begin
          lc = strlen(clabel)
          ok = strpos(clabel, ';;', 0)
          if ok gt 0 then begin
            if (ok+2) lt lc then bs = strmid(clabel, ok+2,lc) else bs = ''
            lc = strlen(bs)
            clabel = strtrim(strmid(clabel, 0,ok), 2)
            if lc gt 1 then begin
              ssize = make_array(3, 128, /long)
              locx = make_array(3, 128, /long)
              locv = make_array(128, /long)
              mblo = 0
              ii = make_array(ndim, /long)
              locxt =0L
              xx = (x1)
              yy = (x2)
              zz = (x3)
              ier = 0
              lcmove = 0
              while lc gt 1 and ier eq 0 do begin
                reads, bs, ii
                ntot = ii(0) *ii(1)
                if ndim eq 3 then ntot = ntot*ii(2)
                xt = xx(locxt:locxt+ntot-1)
                yt = yy(locxt:locxt+ntot-1)
                locxt = locxt+ntot
                if ndim eq 2 then zt = 0.0 else $
                  zt = zz(locxt:locxt+ntot-1)
                get_rect_xyz,  x, nx, y, ny, z, nz, xt, yt, zt, $
                   order, ier, delta = error, /quiet
                if ier eq 0 then begin
                  ssize(*, mblo)=[nx, ny, nz]
                  locx(*, mblo+1) = locx(*, mblo)+[nx, ny, nz]
                  mblo = mblo +1
                  xx(locx(0, mblo-1):locx(0, mblo)-1) = x
                  yy(locx(1, mblo-1):locx(1, mblo)-1) = y
                  if ndim eq 3 then zz(locx(2, mblo-1):locx(2, mblo)-1) = z
                  ntot = nx*ny*nz
                  order = sort(order)
                  locv(mblo) = locv(mblo-1)+ntot
                  lcmove = 1
                  if mblo eq 1 then vv1 = v1
                  v1(locv(mblo-1):locv(mblo)-1) = vv1(order+locv(mblo-1))
                  if nattr gt 1 then begin
                    if mblo eq 1 then vv2 = v2
                    v2(locv(mblo-1):locv(mblo)-1) = vv2(order+locv(mblo-1))
                    if nattr gt 2 then begin
                      if mblo eq 1 then vv3 = v3
                      v3(locv(mblo-1):locv(mblo)-1) = vv3(order+locv(mblo-1))
                      if nattr gt 3 then begin
                        if mblo eq 1 then vv4 = v4
                        v4(locv(mblo-1):locv(mblo)-1) = vv4(order+locv(mblo-1))
                        if nattr gt 4 then begin
                          if mblo eq 1 then vv5 = v5
                          v5(locv(mblo-1):locv(mblo)-1) = vv5(order+locv(mblo-1))
                          if nattr gt 5 then begin
                            if mblo eq 1 then vv6 = v6
                            v6(locv(mblo-1):locv(mblo)-1)=vv6(order+locv(mblo-1))
                          endif
                        endif
                      endif
                    endif
                  endif
                endif
                ok = strpos(bs, ';;', 0)
                if ok gt 0 then begin
                  if (ok+2) lt lc then bs = strmid(bs, ok+2,lc) else bs=''
                  lc = strlen(bs)
                endif else lc = 0
              endwhile
              vv1 = 0
              vv2 = 0
              vv3 = 0
              vv4 = 0
              vv5 = 0
              vv6 = 0
              if ier eq 0 then begin
                ssize = ssize(0:ndim-1, 0:mblo-1)
                locx = locx(0:ndim-1, 0:mblo)
                locv = locv(0:mblo)
                x1 = xx(0 :locx(0, mblo)-1)
                x2 = yy(0 :locx(1, mblo)-1)
                if ndim gt 2 then x3 = zz(0 :locx(2, mblo)-1) else begin
                  x3 = make_array(mblo, value= x3)
                  locxt = make_array(3, mblo+1)
                  locxt[0:ndim-1,0:mblo] = locx
                  locxt[2, *] = lindgen(mblo+1)
                  locx = temporary(locxt)
                endelse
                itype = 1L
                triangles = 0L
                loc_triangles = 0L
              endif else begin
                itype = 8L
                if lcmove gt 0 then begin
;                 refill the v# arrays
                  v1 = attr(*, 0)
                  v2 = 0
                  v3 = 0
                  v4 = 0
                  v5 = 0
                  v6 = 0
                  if nattr gt 1 then begin
                    v2 = attr(*, 1)
                    if nattr gt 2 then begin
                      v3 = attr(*, 2)
                      if nattr gt 3 then begin
                        v4 = attr(*, 3)
                        if nattr gt 4 then begin
                          v5 = attr(*, 4)
                          if nattr gt 5 then v6 = attr(*, 4)
                        endif
                      endif
                    endif
                  endif
                endif
              endelse
            endif else itype = 8L
          endif else itype = 8L
        endif else itype = 8L
        attr = 0
        if ((aptype eq 15) and (itype eq 8)) then begin
          rectilinear = 0
          clabel = temporary(clabel_save)
          lc = strlen(clabel)
          ok = strpos(clabel, ';;', 0)
          if ok gt 0 then begin
            if (ok+2) lt lc then bs = strmid(clabel, ok+2,lc) else bs = ''
            lc = strlen(bs)
            clabel = strtrim(strmid(clabel, 0,ok), 2)
            if lc gt 1 then begin
              ssize = make_array(ndim, 128, /long)
              loc_triangles = make_array(128)
              mblo = 0
              ii = make_array(ndim, /long)
              while lc gt 1 do begin
                reads, bs, ii
                ssize(*, mblo)=ii
                mblo = mblo +1
                if mblo eq 1 then begin
                  triangles = make_triangles(0, ii(0), ii(1))
                  ss = size(triangles)
                  loc_triangles(mblo) = ss(2)
                  triangles = reform(triangles, 3*ss(2), /overwrite)
                  offset = ii(0)*ii(1)
                endif else begin
                  tri = make_triangles(0, ii(0), ii(1))+offset
                  offset =offset + ii(0)*ii(1)
                  ss = size(tri)
                  loc_triangles(mblo) = ss(2)+loc_triangles(mblo-1)
                  triangles = [triangles, reform(tri, 3*ss(2), /overwrite)]
                endelse
                ok = strpos(bs, ';;', 0)
                if ok gt 0 then begin
                  if (ok+2) lt lc then bs = strmid(bs, ok+2,lc) else bs=''
                  lc = strlen(bs)   
                endif else lc = 0
              endwhile
              ssize = ssize(*, 0:mblo-1)
              loc_triangles = loc_triangles[0:mblo]
              locx = make_array(ndim, mblo+1, /long)
              locx(0, 1:mblo) = ssize(0, 0:mblo-1)*ssize(1, 0:mblo-1)
              if ndim gt 2 then locx(0, 1:mblo) = $
                  locx(0, 1:mblo)*ssize(2, 0:mblo-1)
              for i = 1, mblo  do locx(0, i) = locx(0, i)+locx(0,i-1)
              locx(1, *)= locx(0,*)
              if ndim gt 2 then locx(2, *)= locx(0:*)
              locv = reform(locx(0, *))     
              triangles = reform (triangles, 3, loc_triangles(mblo), /overwrite)
            endif else begin
              mblo = 1L
              locx = make_array(ndim, mblo+1, /long)
              locx(*, 1) = n_elements(x1)
              locv = [0, n_elements(v1)]
              ssize = make_array(ndim, value = n_elements(x1))
            endelse
          endif else mblo = -1L
        endif else if itype eq 8 then mblo = -1L
        if mblo lt 0 then begin
          rectilinear = 0
          mblo = 1L
          locx = make_array(ndim, mblo+1, /long)
          locx(*,1) =  n_elements(x1)
          locv = locx
;          if n_elements(ssize) eq 0 then $
           ssize = make_array(ndim, value = n_elements(x1))
        endif
        if itype eq 1 then range = make_array(2, 3, mblo+1) else $
          range = make_array(2, ndim, mblo+1)
        for i = 0, mblo-1 do begin
          range (0, 0, i+1) = min(x1(locx(0,i): locx(0,i+1)-1), max = mm)
          range (1, 0, i+1) = mm
          range (0, 1, i+1) = min(x2(locx(1,i): locx(1,i+1)-1), max = mm)
          range (1, 1, i+1) = mm
          if ndim gt 2 then begin
            range (0, 2, i+1) = min(x3(locx(2,i): locx(2,i+1)-1), max = mm)
            range (1, 2, i+1) = mm
          endif
        endfor
        range (0, 0, 0) = min(range(0, 0, 1:mblo))
        range (0, 1, 0) = min(range(0, 1, 1:mblo))
        range (1, 0, 0) = max(range(1, 0, 1:mblo))
        range (1, 1, 0) = max(range(1, 1, 1:mblo))
        if ndim gt 2 then begin
          range (0, 2, 0) = min(range(0, 2, 1:mblo))
          range (1, 2, 0) = max(range(1, 2, 1:mblo))
        endif
        if mblo gt 1 and rectilinear gt 0 then begin
          x1lab = make_array (mblo, value = x1lab)
          x2lab = make_array (mblo, value = x2lab)
          if ndim gt 2 then x3lab = make_array (mblo, value = x3lab)
          if itype eq 1 then ndim = 3L
          array = {type: itype, ndim:ndim, nvect: nattr, block :mblo, two:two, $
              slice: 0.0, size: ssize, range: range, triangles:triangles, $
              order: 0L, locx:locx, locv: locv, loctri:loc_triangles, $
              x1:x1, x2:x2, x3:x3, x4:x4, v1:v1, v2:v2, v3:v3, v4:v4, $
              v5:v5, v6:v6, x1lab : x1lab, x2lab :x2lab, x3lab : x3lab, $
              x4lab : x4lab, v1lab : make_array(mblo, value =alabel(0)), $
              v2lab : make_array(mblo, value =alabel(1)), $
              v3lab : make_array(mblo, value =alabel(2)), $
              v4lab : make_array(mblo, value =alabel(3)), $
              v5lab : make_array(mblo, value =alabel(4)), $
              v6lab : make_array(mblo, value =alabel(5)), $
              file : filename, spare : spare, tlab : tlabel, clab:clabel }
        endif else  begin
          if itype eq 1 then ndim = 3L
          array = {type: itype, ndim:ndim, nvect: nattr, block :mblo, $
              two:two, $
              slice: 0.0, size: ssize, range: range, triangles:triangles, $
              order: 0L, locx:locx, locv: locv, loctri:loc_triangles, $
              x1:x1, x2:x2, x3:x3, x4:x4, v1:v1, v2:v2, v3:v3, v4:v4, $
              v5:v5, v6:v6, x1lab : x1lab, x2lab :x2lab, x3lab : x3lab, $
              x4lab : x4lab, v1lab :alabel(0), v2lab :alabel(1), $
              v3lab : alabel(2), v4lab :alabel(3),v5lab : alabel(4), $
              v6lab :alabel(5), $
              file : filename, spare : spare, tlab : tlabel, clab:clabel }
        endelse
        if size(/type, array) eq 8 then $
          if lcstructure then i2s, array, get_maxstr()-1, /no_copy
        return
      endif

      blo = 1L
      b = size (dataset)
      nmap = 0L
; 
      nmax = 1L
      get_dset_string, dataset,pid,  map, nmap, nmax, ier
      if ier ne 0 then message, ' Please change the dataset specification'
;
      if nmap gt 1 then begin
       Print,'% READFLD: Please change the dataset specification to get a single dataset.'
        help, datasetin  
        print, 'Datasets: '
        for i = 0, nmap-1 do print, map[i]
        return
      endif

      direct = map(0)
;
STARTREAD:    
      readpff, pid, image, space, direct, blo, label = lab, $
         file = file, pff= pfftyp, np = np, mxbl= mxbl, spare = spare

;     sort field will separate spacial data and vector data for 
;     pfftypes uf3, nf3, nv3, ngd 
;
     b = size (image)
     if b(0) lt 2 then begin
        print, ' Error reading dataset', direct
        print, ' Image array does not have 2 dimensions'
        return
      endif
;
;    pfftype - PFF dataset type
;            1-scalar data on a uniform 3-D grid (UF3),
;            2-scalar data on a uniform 1-D grid (similar to WDF data) (UF1),
;            3-scalar data on a non-uniform 3-D grid (NF3),
;            4-vector data on a non-uniform grid (NV3),
;            5-m x n vertex data - m-dimensional vertices with n-attributes
;                per vertex (VTX),(m, n, nvert)
;            6-integer list and a two float lists (IFL) (nfl, nfa, ni),
;            7-n-dimensional vectors on a region of m-dimensional space (NGD)
;                (m, n, nvert),
;            8-nonuniform 3D grid (NG3).
;
      if pfftyp ne 1 and pfftyp ne 3 and pfftyp ne 4 and pfftyp ne 7 then begin
        print, ' Dataset ', direct,' has a pff type of ', pfftyp
        print, ' This dataset type is not supported with READFLD'
        return
      endif
      sort_field, image, space, pfftyp, ndim, natt, np, lv, $
        aa, bb, cc, dd, ee, ff, gg, hh, ii, jj, ier

;
; Now correct the field for two dimensional data.
      if spare(2) ne (-3) and spare(2) ne 0 and spare(2) ne 1 then begin
         if spare (2) eq 2 then begin
           bbb = temporary(ee)
           ccc = temporary(ff)
           aaa = temporary(gg)
           l8 = temporary(lab(7))
           l9 = temporary(lab(8))
           l7 = temporary(lab(9))
; spare (2) must equal 3
         endif else if spare(2) eq 3 then begin
           ccc = temporary(ee)
           aaa = temporary(ff)
           bbb = temporary(gg)
           l9 = temporary(lab(7))
           l7 = temporary(lab(8))
           l8 = temporary(lab(9))
         endif else begin
           PRINT,' Error may have occurred in readfld.(spare = ', spare(2)
           l7 = temporary(lab(7))
           l8 = temporary(lab(8))
           l9 = temporary(lab(9))
           aaa = temporary(ee)
           bbb = temporary(ff)
           ccc = temporary(gg)
         endelse
         ee = temporary(aaa)
         ff = temporary(bbb)
         gg = temporary(ccc)
         lab(7) = temporary(L7)
         lab(8) = temporary(L8)
         lab(9) = temporary(L9)
      endif
;     if 2D data change to 3D data
      if ndim eq 2  and spare(1) gt  0  and spare(1) lt 4 then begin
        if spare(0) eq 1 then templabel= ['X', 'Y', 'Z'] else $
          if spare(0) eq 2 then templabel= ['R', 'Theta', 'Z'] else $
            if spare(0) eq 3 then templabel= ['R', 'Theta', 'Phi'] else $
              templabel= ['X', 'Y', 'Z']
        ndim = 3L
;       if spare(1) equal 1 then the data is y-z
        if spare(1) eq 1 then begin
          np = [1, np]
          cc = temporary(bb)
          bb = temporary(aa)
          aa = 0.
          lab(5) = temporary(lab(4))
          lab(4) = temporary(lab(3))
          lab(3) = templabel(0)
        endif else if spare(1) eq 2 then begin
;       if spare(1) equal 2 then the data is z-x
          npt = np(0)*np(1)
          ee = reform(transpose(reform(ee, np(0), np(1))), npt, /overwrite)
          if n_elements (ff) eq npt then $
            ff = reform(transpose(reform(ff, np(0), np(1))), npt, /overwrite)
          if n_elements (gg) eq npt then $
            gg = reform(transpose(reform(gg, np(0), np(1))), npt, /overwrite)
          if n_elements (hh) eq npt then $
            hh = reform(transpose(reform(hh, np(0), np(1))), npt, /overwrite)
          if n_elements (ii) eq npt then $
            ii = reform(transpose(reform(ii, np(0), np(1))), npt, /overwrite)
          if n_elements (jj) eq npt then $
            jj = reform(transpose(reform(jj, np(0), np(1))), npt, /overwrite)
          np = [np(1), 1, np(0)]
          cc = temporary(aa)
          aa = temporary(bb)
          bb = 0.
          lab(5) = temporary(lab(3))
          lab(3) = temporary(lab(4))
          lab(4) = templabel(1)
        endif else begin
;       if spare(1) equal 3 then the data is x-y
          np = [np, 1]
          lab(5) = templabel(2)
        endelse
      endif
;
      if blo eq 1 then begin
        locx = [0*np, np]
        locv = [0, lv]
        size = np
        lvold = lv
        npold = np
      endif else begin
        npold = np+npold
        lvold = lv+lvold
        locx = [temporary(locx), npold]
        locv = [temporary(locv), lvold]
        size = [temporary(size), np]
      endelse
      if blo eq 1 then   range = make_array(2, ndim, mxbl+1)
      range(0, 0, blo) = min(aa, max = mx)
      range(1, 0, blo) = mx
      range(0, 1, blo) = min(bb, max = mx)
      range(1, 1, blo) = mx
      if ndim gt 2 then begin
        range(0, 2, blo) = min(cc, max = mx)
        range(1, 2, blo) = mx
        if ndim gt 3 then begin
          range(0, 3, blo) = min(dd, max = mx)
          range(1, 3, blo) = mx
        endif
      endif
      if blo eq 1 then begin
        x1lab = make_array(mxbl, /string, value = ' ')
        x2lab = make_array(mxbl, /string, value = ' ')
        x3lab = make_array(mxbl, /string, value = ' ')
        x4lab = make_array(mxbl, /string, value = ' ')
        v1lab = make_array(mxbl, /string, value = ' ')
        v2lab = make_array(mxbl, /string, value = ' ')
        v3lab = make_array(mxbl, /string, value = ' ')
        v4lab = make_array(mxbl, /string, value = ' ')
        v5lab = make_array(mxbl, /string, value = ' ')
        v6lab = make_array(mxbl, /string, value = ' ')
        x1 = temporary(aa)
        x2 = temporary(bb)
        x3 = temporary(cc)
        x4 = temporary(dd)
        v1 = temporary(ee)
        v2 = temporary(ff)
        v3 = temporary(gg)
        v4 = temporary(hh)
        v5 = temporary(ii)
        v6 = temporary(jj)
      endif else begin
        x1 = [temporary(x1), temporary(aa)]
        x2 = [temporary(x2), temporary(bb)]
        x3 = [temporary(x3), temporary(cc)]
        x4 = [temporary(x4), temporary(dd)]
        v1 = [temporary(v1), temporary(ee)]
        v2 = [temporary(v2), temporary(ff)]
        v3 = [temporary(v3), temporary(gg)]
        v4 = [temporary(v4), temporary(hh)]
        v5 = [temporary(v5), temporary(ii)]
        v6 = [temporary(v6), temporary(jj)]
      endelse
      x1lab(blo-1) = lab(3)
      x2lab(blo-1) = lab(4)
      x3lab(blo-1) = lab(5)
      x4lab(blo-1) = lab(6)
; use space label if we are dealing with nf3, nv3, uf3 data
;
     if pfftyp lt 7 then begin
;
        blab = strtrim(lab(2), 2)
        lenblab = strlen(blab)
         if lenblab eq 0 and mxbl gt 1 then begin
           blab = 'Block '+strtrim(blo, 2)
           lenblab = strlen(blab)
         endif
        if pfftyp eq 4 then begin
          if lenblab gt 0 then begin
            blab = blab + '-'
        if spare (0) le 1 or spare(0) gt 3 then begin
          v1lab(blo-1) = blab + 'X-Component'
          v2lab(blo-1) = blab + 'Y-Component'
          v3lab(blo-1) = blab + 'Z-Component'
        endif else if spare (0) eq 2 then begin
          v1lab(blo-1) = blab + 'R-Component'
          v2lab(blo-1) = blab + 'Theta-Component'
          v3lab(blo-1) = blab + 'Z-Component'
        endif else if spare (0) eq 3 then begin
          v1lab(blo-1) = blab + 'R-Component'
          v2lab(blo-1) = blab + 'Theta-Component'
          v3lab(blo-1) = blab + 'Phi-Component'
        endif
           endif else begin
        if spare (0) le 1 or spare(0) gt 3 then begin
          v1lab(blo-1) = 'X-Component'
          v2lab(blo-1) = 'Y-Component'
          v3lab(blo-1) = 'Z-Component'
        endif else if spare (0) eq 2 then begin
          v1lab(blo-1) = 'R-Component'
          v2lab(blo-1) = 'Theta-Component'
          v3lab(blo-1) = 'Z-Component'
        endif else if spare (0) eq 3 then begin
          v1lab(blo-1) = 'R-Component'
          v2lab(blo-1) = 'Theta-Component'
          v3lab(blo-1) = 'Phi-Component'
        endif 
           endelse
        endif else begin
          v1lab(blo-1) = blab
          v2lab(blo-1) = blab
          v3lab(blo-1) = blab
        endelse
        
        v4lab(blo-1) = lab(10)
        v5lab(blo-1) = lab(11)
        v6lab(blo-1) = lab(12)
        if ndim eq 2 then two = 3L else begin
          spare (1) = 4
          if np(0) lt 2 then begin
            two = 1L
            slice = x1(0)
          endif else if np(1) lt 2 then begin
            two = 2L
            slice = x2(0)
          endif else if np(2) lt 2 then begin
            two = 3L
            slice = x3(0)
          endif else two = 0L
        endelse
      endif else begin
        two = spare (1)
        if two gt 3 then two = 0L
        v1lab(blo-1) = lab(7)
        v2lab(blo-1) = lab(8)
        v3lab(blo-1) = lab(9)
        v4lab(blo-1) = lab(10)
        v5lab(blo-1) = lab(11)
        v6lab(blo-1) = lab(12)
      endelse
; increment the pointer arrays    
      if blo lt mxbl then begin
        direct = -1
        blo = blo+1
        goto, startread
      endif
; generate the ordering
;
      for i = 0, ndim -1 do begin
        range(0, i, 0)=min(range(0,i,1:mxbl))
        range(1, i, 0)= max(range(1,i,1:mxbl))
      endfor
;
      if mxbl gt 1 then begin
        order = make_array(ndim, mxbl, /long)
        for i = 0, ndim-1 do order (i, *) = sort(range(0,i, 1:mxbl))
      endif else order = 1L
      array = {type:1L, ndim:ndim, nvect: natt, block :mxbl, two: two, $
              slice: slice, size: reform(size,ndim,mxbl), range: range, $
              order: order, locx:reform(locx,ndim, mxbl+1), $
              locv: locv, x1:temporary(x1), x2:temporary(x2), $
              x3:temporary(x3), x4:temporary(x4), v1:temporary(v1), $
              v2:temporary(v2), v3:temporary(v3), v4:temporary(v4), $
              v5:temporary(v5), v6:temporary(v6),x1lab : x1lab, x2lab :x2lab, $
              x3lab : x3lab, x4lab :x4lab,v1lab : v1lab, v2lab :v2lab, $
              v3lab : v3lab, v4lab :v4lab,v5lab : v5lab, v6lab :v6lab, $
              file : filename, spare : spare, tlab : lab(0), clab: lab(1)}
      if size(/type, array) eq 8 then $
        if lcstructure then i2s, array, get_maxstr()-1, /no_copy
      end
