; ------------------------------------------------------------------------ 
; $Id: readifl.pro,v 1.8 2010/03/04 16:51:26 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
 Pro readifl,iarray,flist,farray,datasetin, structure=structure, $
       fileid=fid, tlabel = tlabel, clabel = clabel, apptyp = apptyp, $
       namefile = filename, pfftyp = pfftyp,  skip = skip, $
       dlength = dlength, ierror = ierror, iextra = iextra, npoints = np
;
;+
;    READIFL
;
;  Reads an integer float list into three arrays
;  Procedure options are explained below. PFF dataset type must be IFL
;
;  Format:  READIFL, iarray, flist, farray [, dataset] [, FILEID = fileid]
;               [, NAMEFILE = file] [, TLABEL= tlab] [, CLABEL= clab]
;               [, APPTYP = apptyp] [, IERROR = ierror] [, NPOINTS = npoints]
;     or    READIFL, STR, /STRUCTURE [, dataset] [, FILEID = fileid]
;               [, NAMEFILE = file] [, TLABEL= tlabel] [, CLABEL= clabel]
;               [, APPTYP = apptyp] [, IERROR = ierror] [, NPOINTS = npoints]
;  where:
;    iarray  - integer array
;    flist   - Float List
;    farray  - Float array
;    str     - Structure of type = 5
;    Structure Tags are:
;            Type    - 5
;            Size    - Array of 3 longs with the size of the integer array,
;                      float list and float array respectively.
;            iarray  - Integer array
;            flist   - Float list
;            farray  - Floating array
;            tlabel  - Dataset type label
;            clabel  - Dataset comment
;            file    - File containing the data
;            spare   - Application Dataset type
;
;    dataset - indicates a PFF dataset number
;              or is a search string for dataset comments.
;              If dataset is missing or zero, then it is set to the
;              current dataset.
;        Note: If dataset is a string, a leading `^' indicates that
;              the string begins at the start of the dataset comment.
;              The dataset string must provide a single dataset.
;    fileid -  file id of the PFF file containing the data
;    NPOINTS - Number of points in each array.
;              NPOINTS[0] - Number of points in the integer array
;              NPOINTS[1] - Number of points in the float list
;              NPOINTS[2] - Number of points in the float array
;    TLabel -  Dataset Type Label
;    CLabel -  Dataset Comment
;    NameFile - File name
;    AppTyp -  Appllication Dataset Type
;    IERROR -  Error flag. Returns 0 if no error
;
;
;  Examples:
;    Read dataset 34 in file id 4 into structure LISTS
;      readifl, LISTS, 34, fil=4, /structure
;    Read next dataset of the current file into structure dd
;      readifl, dd, /structure
;-
;

      on_error, 1
;
      common pffcom, initpff, pff_exe
;
;     check to see if pff has been initialized
;
      ierror = 1
      if n_elements(initpff) eq 0 then initpff =0
      if (initpff eq 0) then begin
        print, '% READIFL: File must be opened before reading data'
        return
      endif
;
;     check on the number of parameters
;

      npar=  n_params()
      if (npar eq 0 ) then begin
 print, '%Format: READIFL, STR, /STRUCTURE [, dataset] [, FILEID = fileid]'
 print, '             [, NAMEFILE = file] [, TLABEL= tlabel] [, CLABEL= clabel]'
 print, '             [, APPTYP = apptyp] [, IERROR = ierror]'

 print, '  or:    READIFL, iarray, flist, farray, [, dataset] [, FILEID = fileid]'
 print, '             [, NAMEFILE = file] [, TLABEL= tlabel] [, CLABEL= clabel]'
 print, '             [, APPTYP = apptyp] [, IERROR = ierror]'
        return
      endif

      if n_elements (fid) eq 0 then fid = 0L
      pid = long(fid)
      iarray = 0
      if keyword_set(structure) then begin
        lcstructure = 0
        if not arg_present(iarray) then  begin
          print, '% READIFL: Warning data cannot be returned in variable'
          print, '           Data will be stored in structure '+strtrim (get_maxstr()-1, 2)
          lcstructure = 1
        endif
        if n_elements(flist) eq 0 then dataset=0L else dataset=flist[0]
      endif else begin
        if n_elements(datasetin) eq 0 then dataset=0L else dataset=datasetin[0]
      endelse
      dataset = long(dataset)
      if keyword_set(skip) then begin
        if n_elements(dlength) ne 1 or n_elements (iextra) lt 8 or $
        n_elements(pfftyp) ne 1 then message, 'Error with keyword SKIP'
      endif else begin
        nmap = 0L
        nmax = 1L
        get_dset_string, dataset,pid,  map, nmap, nmax, ier
        if ier ne 0 then message, ' Please change the dataset specification'
;
        if nmap gt 1 then begin
         Print,'% READIFL: Please change the dataset specification to get a single dataset.'
          help, dataset 
          print, 'Datasets: '
          for i = 0, nmap-1 do print, map[i]
          return
        endif
        dataset = map(0)
        readhdr,dataset,ier,fileid=pid, pfftyp = pfftyp, $
           dlength = dlength, tlabel= tlabel, clabel=clabel, $
           namefile = filename, iextra = iextra, aptyp = apptyp, /quiet
        if ier ne 0 or n_elements(pfftyp) ne 1  then begin
          print, '% READIFL: Error reading dataset header', ier
          ierror = ier
          return
        endif
      endelse
      if (pfftyp  ne 6 ) then begin
        print, '% READIFL: Dataset type '+strtrim(pfftyp, 2) + $
            ' cannot be read with READIFL'
        return
      endif

; READIFL

;               For IFL data
;                 iextra(0) = Float array flag
;                 iextra(1) = Length of Float-List
;                 iextra(2) = Length of Integer array
;                 iextra(3) = Length of Float array
;
      iextra  = long(iextra)
      idim = iextra[2]> 1L
      fldim = iextra[1]> 1L
      offdim = fldim
      fadim = iextra[3] > 1L
      ni = 0L 
      nfl = 0L 
      nfa = 0L
      tapp = 0L
      llstr = 40L
      clab = make_array(llstr, /byte, value = 32B)
      tlab = make_array(llstr, /byte, value = 32B)

      ier = -99L

      iarray = make_array(idim, /long, value =0L)
      flist = make_array(fldim)
      floff = make_array(fldim, /long, value =0L)
      farray = make_array(fadim)
      foff10 = 0L
;*deck pff_rifl
;*********1*********2*********3*********4*********5*********6*********7**

      if  initpff eq 1 then begin
; HP
        junk = call_external(PFF_EXE,'pff_slrifl', pid, dataset,   $
               idim, fldim, fadim, offdim, ni, nfl, nfa, tapp, tlab, clab, $
               iarray, flist, floff, farray, foff10, ier, llstr)

      endif else if initpff eq 2 then begin
; Sun

        junk = call_external(PFF_EXE,'pff_slrifl_', pid, dataset, $
               idim, fldim, fadim, offdim, ni, nfl, nfa, tapp, tlab, clab, $
               iarray, flist, floff, farray, foff10, ier, llstr)

      endif else if initpff eq 4 then begin
; WIN
        junk = call_external(PFF_EXE,'_PFF_SLRIFL@8', pid, dataset, $
               idim, fldim, fadim, offdim, ni, nfl, nfa, tapp, tlab, clab, $
               iarray, flist, floff, farray, foff10, ier, llstr)

      endif
;debug
      if ier ne 0 then begin
        print, 'READIFL: Error reading dataset, IER = ',strtrim(ier,2)
        farray = 0
        ierror = ier
        return
      endif
      if max(floff, min = mm) ne 0 or mm ne 0 then begin
print,'WARNING - Power of 10 offset for the float list exceeds computer range'
        print, '  FLOAT LIST offset - ', strtrim(floff, 2)
      endif
      if (foff10 ne 0 ) then begin
print,'WARNING - Power of 10 offset for the float array exceeds computer range'
        print, '  FLOAT array offset - ', strtrim(foff10, 2)
      endif
      if (iextra[1] ne nfl) or (iextra[2] ne ni)or (iextra[3] ne nfa) then begin
        PRINT, '% READIFL: Problem with readhdr.'
        print, ' IEXTRA: ', iextra[1:3]
        print, ' READIFL values: ', nfl, ni, nfa
      endif
      np = [ni, nfl, nfa]
      ierror = 0L
      if not keyword_set(structure) then return
      if tapp eq 101 then begin
        cnew = strtrim(clabel)
        j = strpos(cnew, 'Shot')+4
        k = strpos(cnew, ', LOS')
        shot = long(strmid(cnew, j, k-j))
        j = k+5
        k = strpos(cnew, ';;', j)
        los = strtrim(strmid(cnew, j, k-j), 2)
        j = k+2
        k = strpos(cnew, ', ', j)
        date = strmid(cnew, j, k-j)
        j = k+2
        k = strpos(cnew, ' - ;;', j)
        clab = strtrim(strmid(cnew, j, k-j), 2)
        j = k+5
        n = 0
        ndet = iarray[0]
        source = iarray[1]
        Signal = make_array(ndet, /string)
        cathode = make_array(ndet, /string)
        calibration = make_array(ndet, /string)
        filter = make_array(ndet, /string)
        while n lt ndet do begin
          k = strpos(cnew, ';;', j)
          signal[n] = strmid (cnew, j, k-j)
          j = k+2
          k = strpos(cnew, ';;', j)
          cathode[n] = strmid (cnew, j, k-j)
          j = k+2
          k = strpos(cnew, ';;', j)
          calibration[n] = strmid (cnew, j, k-j)
          j = k+2
          k = strpos(cnew, ';;', j)
          filter[n] = strmid (cnew, j, k-j)
          j = k+2
          n = n+1
        endwhile
        iarray = {type: 101, shot: shot, los:los, date:date, comment: clab, $
          ndet:ndet, dist : flist[0], source: source, src_area: flist[1], $
          pinch_ht: flist[2], aper_ht: flist[1], elevation: flist[3], $
          az_ang: flist[4], signal: signal, cathode: cathode, $
          calibration:calibration, filter:filter, diameter:farray[0:ndet-1], $
          energy_bin: farray[ndet: ndet+ndet]}

      endif else $
       iarray = {type:5L, Size:np, Iarray:iarray, flist:flist, $
         farray:farray, tlab: tlabel, clab: clabel, spare:tapp, file: filename}
       if lcstructure then i2s, iarray, get_maxstr()-1, /no_copy
      end
