; ------------------------------------------------------------------------ 
; $Id: readng3.pro,v 1.7 2010/03/03 22:52:07 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
 Pro readng3, farray, datasetin, fileid = fid, stringlen= stringlen, $
              xarray = x, yarray = y, zarray= z, $
              xlabel = x1lab, ylabel = x2lab, zlabel = x3lab, $
              blabel = blab, tlabel= tlabel, clabel = clabel, $
              namefile = filename, pfftyp = pfftyp,  $
              spare = ispare, apptyp = tapp, skip = skip, $
              locx = locx, dlength = dlength, iextra = iextra 
;
;+
;
;  Reads a field or charge density into an IDL structure.
;  Procedure options are explained below.
;  Current version limited to 24 blocks by the dimensions in pff_rw.inc
;  PFF dataset type must be 8: NG3 = Nonuniform 3D grid
;
;  Format:  READNG3, STRUCT  [, dataset] [, FILEID = fileid]
;                   [, STRINGLEN = stringlen]
;                   [, XARRAY= xarray] [, YARRAY= yarray]
;                   [, ZARRAY= zarray] [APPTYP = apptyp]
;                   [, CLABEL = clabel] [, TLABEL = tlabel]
;                   [, XLABEL = xlabel] [, YLABEL = ylabel] 
;                   [, ZLABEL = zlabel] [, BLABEL = blabel]
;                   [, NAMEFILE = namefile] [, PFFTYP = pfftyp]
;                   [, LOCX = locx] [, SPARE = spare]
;  where:
;    STRUCT  - IDL structure name
;              If not a valid variable, data is stored in GET_MAXSTR()-1
;    dataset - indicates a PFF dataset number 
;              or is a search string for dataset comments.  
;              If dataset is missing or zero, then it is set to the 
;              current dataset.
;        Note: If dataset is a string, a leading `^' indicates that 
;              the string begins at the start of the dataset comment.
;    fileid -  file id of the PFF file containing the data
;    stringlen - Currently Ignored
;                Maximum number of characters in the spatial labels and the
;                field labels. Currently 64
;    xarray    - Values for the first dimension
;    yarray    - Values for the second dimension
;    zarray    - Values for the third dimension
;    apptyp    - Application dataset type
;    SPARE     - Spare (mspare, nblock) spare array for each block where 
;                mspare is the maximum number of spare words for each block
;  x,y,z,blabel- Labels for x, y, z and block
;
;  Structure Labels
;    type  - 3
;    ndim  - number of spatial dimensions
;    block - number of blocks 
;    two   - two dimensional missing dimension
;            this is 0 for 3d data, 1 for yz data, 2 for zx data, 3 for xy data
;    slice - value of the missing dimension, if known. Default is zero.
;    size  - array of the size of the spatial dimensions
;            size = size(ndim, block)
;    range - (2, ndim, nblock+1) two dimensional array of the max and min for
;            each block.  Block (0) has overall values.
;    order - (ndim, nblock) Ordering of blocks in each dimension by initial 
;            value of the block, in units of block -1
;    locx  - array of the location of the first point for each block
;            spatial array- locx = locx(ndim, block+1)
;            (first spatial dimension for block m is x and is defined by
;             x = x(locx(0, m-1): locx(0, m)-1) )
;    x1lab, x2lab, ...x(ndim)lab - spatial labels
;    proc  - Processor number if available.  Default = 0
;    tlab  - type label
;    clab  - dataset comment
;    file  - file data read from
;    spare - integer array (5)
;  Note: If block > 1 spatial and vector arrays are reformed to linear 
;        arrays and stored in the respective arrays.
;  Examples:
;    Read dataset 34 in file id 4 into structure ch5
;      readng3, ch5, 34, fil=4
;    Read current dataset of the current file into structure efldstart
;      readng3, efldstart
;    Read the current dataset into an array image, and put the x and y 
;      coordinates into xar and yar.
;      readng3, image, /arr, xar = xar, yar = yar
;-
;
      on_error, 1
;
      common pffcom, initpff, pff_exe
@pff_strings.com
      stringlen = max_str_len

;
;     check to see if pff has been initialized
;
      if n_elements(initpff) eq 0 then initpff =0
      if (initpff eq 0) then begin
        print, '% READNG3: File must be opened before reading data'
        return
      endif
;
;     check on the number of parameters
;
;debug
; tstart = systime(1)
      nparams =  n_params()
      slice = 0.0
      if (nparams lt 1 ) then begin
        print,'% Format: READNG3, Structure  [, dataset] [, FILEID = fileid]'
        print,'        [, STRINGLEN = stringlen] [APPTYP = apptyp]'
        print,'        [, XARRAY= xarray] [, YARRAY= yarray]'
        print,'        [, XLABEL = xlabel] [, YLABEL = ylabel]'
      print,'        [, ZARRAY= zarray] [, ZLABEL = zlabel] [, BLABEL = blabel]'
        print,'        [, CLABEL = clabel] [, TLABEL = tlabel]'
        print,'        [, NAMEFILE = namefile] [, PFFTYP = pfftyp]'
        print,'        [, LOCX = LOCX] [, SPARE = spare]'
      return
      endif

      lcstructure = 0
      if not arg_present(farray) then  begin
        print, '% READNG3: Warning data cannot be returned in variable'
        print, '           Data will be stored in structure '+strtrim (get_maxstr()-1, 2)
        lcstructure = 1
      endif
      farray = 0.0
      if n_elements (fid) eq 0 then fid = 0L
      pid = long(fid)
      if n_elements (datasetin) eq 0 then dataset = 0L else $
        dataset = datasetin(0)
      if keyword_set(skip) then begin
        if n_elements(dlength) ne 1 or n_elements (iextra) lt 8 or $
        n_elements(pfftyp) ne 1 then message, 'Error with keyword SKIP'
      endif else begin
        nmap = 0L
        nmax = 1L
        get_dset_string, dataset,pid,  map, nmap, nmax, ier
        if ier ne 0 then message, ' Please change the dataset specification'
;
        if nmap gt 1 then begin
          Print,'% READNG3: Please change the dataset specification to get a single dataset.'
          help, dataset  
          print, 'Datasets: '
          for i = 0, nmap-1 do print, map[i]
          return
        endif

        dataset = map(0)
        readhdr,dataset,ier,fileid=pid, pfftyp = pfftyp, $
           dlength = dlength, tlabel= tlabel, clabel=clabel, $
           namefile = filename, iextra = iextra, /quiet
        if ier ne 0 or n_elements(pfftyp) ne 1  then begin
          print, '% READNG3: Error reading dataset header', ier
          return
        endif
      endelse
      if (pfftyp  ne 8 ) then begin
        print, '% READNG3: Dataset type '+strtrim(pfftyp, 2) + $
            ' cannot be read with READNG3'
        return
      endif

; READNG3

;               For UF3 or NF3 or NI3 or NV3 or NG3 data
;                 iextra(0) = number of blocks
;                 iextra(1) = number of X-values in data
;                 iextra(2) = number of Y-values in data
;                 iextra(3) = number of Z-values in data
;                 iextra(4) = number of words in field or each vector
;                 iextra(5) = number of spare words in each block
;
      iextra  = long(iextra)
      mblo = iextra(0) < max_block

      nx = make_array(mblo, /long)
      ny = make_array(mblo, /long)
      nz = make_array(mblo, /long)
      mx = iextra(1)
      my = iextra(2)
      mz = iextra(3)
      x = make_array(mx)
      y = make_array(my)
      z = make_array(mz)
      
      locx = make_array(mblo+1, /LONG)
      locy = make_array(mblo+1, /LONG)
      locz = make_array(mblo+1, /LONG)

      tapp = 0L
      nblo = 0L
      MSPARE = long(iextra[5]> 1)
      nspare =0L
      ispare = make_array(mspare, mblo, /long)
      ier = -99L

      lxstr = stringlen
      xlabel = make_array(lxstr*mblo, /byte, value = 32B)
      ylabel = make_array(lxstr*mblo, /byte, value = 32B)
      zlabel = make_array(lxstr*mblo, /byte, value = 32B)
      blabel = make_array(lxstr*mblo, /byte, value = 32B)
      llstr = 40L
      clab = make_array(llstr, /byte, value = 32B)
      tlab = make_array(llstr, /byte, value = 32B)

      xoff10 = make_array(mblo, /long, value =0L)
;debug
;  tbr = systime(1)
;  print, 'before read data', tbr -tstart, tbr-tnow
;*deck pff_rng3
;*********1*********2*********3*********4*********5*********6*********7**
;      subroutine pff_rng3(fid, direct, mblo, mx, my, mz, mspare,
;     &     apptyp, tlabel, clabel, nblo, nx, ny, nz, nspare, ispare,
;     &     xarray, yarray, zarray, xoff10,
;     &     xlabel, ylabel, zlabel, blabel,
;     &     locx, locy, locz, ier)



      if  initpff eq 1 then begin
      ; HP
        junk = call_external(PFF_EXE,'pff_slrng3', pid, dataset, mblo,  $
                mx, my, mz,  mspare, tapp, tlab, clab, nblo, nx, ny, nz, $
                nspare, ispare, x, y, z, xoff10, $
                xlabel, ylabel, zlabel, blabel, locx, locy, locz,  $
                ier, lxstr, llstr)

      endif else if initpff eq 2 then begin
; Sun

        junk = call_external(PFF_EXE,'pff_slrng3_', pid, dataset, mblo, $
                mx, my, mz,  mspare, tapp, tlab, clab, nblo, nx, ny, nz, $
                nspare, ispare, x, y, z, xoff10, $
                xlabel, ylabel, zlabel, blabel, locx, locy, locz, $
                ier, lxstr, llstr)

      endif else if initpff eq 4 then begin
; WIN
        junk = call_external(PFF_EXE,'_PFF_SLRNG3@8', pid, dataset, mblo, $
                mx, my, mz,  mspare, tapp, tlab, clab, nblo, nx, ny, nz, $
                nspare, ispare, x, y, z, xoff10, $
                xlabel, ylabel, zlabel, blabel, locx, locy, locz, $
                ier, lxstr, llstr)

      endif else if initpff eq 5 then begin
; MAC
        junk = call_external(PFF_EXE,'pff_slrng3', pid, dataset, mblo,  $
                mx, my, mz,  mspare, tapp, tlab, clab, nblo, nx, ny, nz, $
                nspare, ispare, x, y, z, xoff10, $
                xlabel, ylabel, zlabel, blabel, locx, locy, locz, $
                ier, lxstr, llstr)

      endif
;debug
;  tar = systime(1)
;  print, 'after read data', tar -tstart, tar-tbr
      if ier ne 0 then begin
        print, 'READNG3: Error reading dataset, IER = ',strtrim(ier,2)
        farray = 0
        return
      endif
      if max(xoff10, min = mm) ne 0 or mm ne 0 then begin
        print,'WARNING - Power of 10 offset for the data exceeds computer range'
        print, 'Space offset - ', strtrim(xoff10, 2)
      endif
      if nblo gt mblo then begin
        Print, ' Problem with the number of blocks, contact Paul Mix.'
        farray = 0
        return
      endif
;
;    pfftype - PFF dataset type 8: NG3
;
;    need round because total is always a float
     nxt = round (total (nx))
     nyt = round (total (ny))
     nzt = round (total (nz))
     if nxt  eq mblo then begin
       two = 1L
       slice = x(0)
     endif else if nyt  eq mblo then begin
       two = 2L
       slice = y(0)
     endif else if nzt  eq mblo then begin
       two = 3L
       slice = z(0)
     endif else begin
       two = 0L
       slice = 0.0
     endelse

     locx(mblo) = locx(mblo-1)+ nx(mblo-1)
     locy(mblo) = locy(mblo-1)+ ny(mblo-1)
     locz(mblo) = locz(mblo-1)+ nz(mblo-1)
     if locx(0) ne 0 then locx= locx-locx(0)
     if locy(0) ne 0 then locy= locy-locy(0)
     if locz(0) ne 0 then locz= locz-locz(0)


     range = make_array(2, 3, mblo+1)
     for i = 0, mblo-1 do begin
       range (0, 0, i+1) = min(x(locx(i): locx(i+1)-1), max = mm)
       range (1, 0, i+1) = mm
       range (0, 1, i+1) = min(y(locy(i): locy(i+1)-1), max = mm)
       range (1, 1, i+1) = mm
       range (0, 2, i+1) = min(z(locz(i): locz(i+1)-1), max = mm)
       range (1, 2, i+1) = mm
     endfor
     locx = transpose(reform([locx, locy, locz], mblo+1, 3))
     range (0, 0, 0) = min(range(0, 0, 1:mblo))
     range (0, 1, 0) = min(range(0, 1, 1:mblo))
     range (0, 2, 0) = min(range(0, 2, 1:mblo))
     range (1, 0, 0) = max(range(1, 0, 1:mblo))
     range (1, 1, 0) = max(range(1, 1, 1:mblo))
     range (1, 2, 0) = max(range(1, 2, 1:mblo))

     ssize = make_array(3, mblo, /long)
     ssize(0, *)= nx
     ssize(1, *)= ny
     ssize(2, *)= nz

     x1lab = make_array(mblo, /string)
     x2lab = make_array(mblo, /string)
     x3lab = make_array(mblo, /string)
     blab = make_array(mblo, /string)

     for i = 0, mblo-1 do begin
       x1lab(i) = strtrim(xlabel(i*lxstr:(i+1)*lxstr-1), 2)
       x2lab(i) = strtrim(ylabel(i*lxstr:(i+1)*lxstr-1), 2)
       x3lab(i) = strtrim(zlabel(i*lxstr:(i+1)*lxstr-1), 2)
       blab(i) =  strtrim(blabel(i*lxstr:(i+1)*lxstr-1), 2)
     endfor
; If we were overly concerned we could reduce size of farray.
; This is probably not worth the time for most data
      farray = {type:3L, ndim:3L,  block :mblo, two:two, range:range, $
              size: ssize, locx : locx, slice: slice,  $
              x1:x, x2:y, x3:z, x1lab : x1lab, x2lab :x2lab, $
              x3lab : x3lab, proc: 0, file : filename, spare :ispare, $
              tlab : tlabel, clab: clabel }

      if lcstructure then i2s, farray, get_maxstr()-1, /no_copy
      end
