; ------------------------------------------------------------------------
; $Id: readsurf.pro,v 1.1 2010/03/03 18:40:04 lpmix Exp $
; ------------------------------------------------------------------------
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
;+
;
;  Reads a SURFACE dataset into an IDL structure.
;  Procedure options are explained below. PFF dataset type must be VTX
;
;  Format:  READSURF, structure  [, dataset] [, FILEID = fileid] [, CLABEL = clabel]
;                     [, IERROR = ierror] [, /STRUCTURE]
;  where:
;    structure - IDL variable or structure array number
;                If this is not a valid variable or a structure number, data will be
;                stored in GET_MAXSTR().  Variable must not be a valid integer structure
;                number if a structure is desired.
;    dataset - indicates a PFF dataset number 
;              or is a search string for dataset comments.  
;              If dataset is missing or zero, then it is set to the 
;              current dataset.  If dataset is an array, the first value 
;              will be used.
;              DATASET must refer to ajj PFF dataset type between 200 and 209
;    number  - If present, then a total of number datasets will be read and 
;              stored in the structure
;        Note: If dataset is a string, a leading `^' indicates that 
;              the string begins at the start of the dataset comment.
;    fileid -  file id of the PFF file containing the data
;    clabel -  If CLABEL has a length gt 0, it will be used as the COMMENT label
;              Note: CLABEL = ' ' will result in a blank value for clabel
;                    CLABEL = '' will result in the default value
;    STRUCTURE - If set, return a structure
;    IERROR -  Error parameter - Returns zero if no error
;  Structure Labels
;    TYPE  - 14 for type 200
;          - 15 for type 201 to 209
;    NDIM  - number of spatial dimensions = 3
;    BLOCK - number of blocks = 1
;    NNODES- Number of nodes
;    NPLANE- Number of planes
;    NVECT - Number of field values (V1, V2, V3)
;            For PFF Application dataset type = 200 = 0
;            For Application dataset types 201 to 209 = 1, 3 for scalars and vectors
;    RANGE - array of the min and max in each dimension 
;            Dimensions = (2,ndim,2)
;    CONNECTIVITY - connectivity array 
;            CONNECTIVITY = LONARR[4, nplane]
;            Index goes from 1 to NNODES
;    LOC   - Pointer array to the first plane in each block
;            For this data -- LOC = [0, nplane]
;    XYZ   - Node spatial values - XYZ = FLTARR[NDIM, NNODES]
;    V1    - First Vector Quantity or scalar quantity
;    V2    - Second Vector Quantity
;    V3    - Third Vector Quantity
;    X1LAB - Label for first dimension
;    X2LAB - Label for second dimension
;    X3LAB - Label for third dimension
;    TLAB  - type label
;    CLAB  - dataset comment
;    FILE  - file data read from
;    SPARE - integer array (5)
;            SPARE[0] = Coordinate system flag
;            SPARE[1] = Timestamp
;            SPARE[2] = Surface number
;            SPARE[3] = Number of Dimensions
;            SPARE[4] = Number of Vectors
;
;  Examples:
;
;    Read dataset 3 in file id 4 into structure 5
;      readstr,  5, 3, file = 4
;      readsurf, 5, 3, file = 4
;    
;    Read dataset 3 in file id 4 into structure c1
;      readsurf, c1, 3, fil=4
;
;      
; MODIFICATION HISTORY:
;       Written by:     L. P. Mix, February 12, 2010
;-
;
; see /remote/lpmix/consult/mfpasik/mk_poly_field_object.pro
;
 Pro readsurf, array, datasetin, fileid = fid,  aptype=aptype, pfftype = pfftype,$
        dlength = dlength,iextra=iextra, skip = skip, clabel = clabel_in, $
        ierror = ierror, structure = structure
@qscom.dat
;
      
      on_error, 1
      lcstructure = 0
      maxst = get_maxstr()
      ierror = 0
      if keyword_set(structure) then begin
        if not arg_present(array) then begin
           print, '% READSURF: Warning data cannot be returned in variable'
           print, '            Reenter command without keyword_set (STRUCTURE)'
           return
        endif
      endif else begin
        if n_elements (array) gt 0 then begin
          if size(/type, array) lt 4 then if (array[0] gt 0) and (array[0] le maxst) $
          then lcstructure = array
        endif
  
        if lcstructure eq 0 then begin
          if arg_present(array) then array = 0.0 else begin
            print, '% READSURF: Warning data cannot be returned in variable'
            lcstructure = maxst
            print, '           Data will be stored in structure '+strtrim (maxst, 2)
          endelse
        endif
      endelse
      array = 0.0
      if n_elements (datasetin) eq 0 then dataset=0 else dataset=datasetin[0]
      if n_elements(fid) gt 0 then setpff, fid[0], ierror
      if ierror ne 0 then begin
        print,  '% READSURF: Unable to set the File ID to FID'
        help, fid
        return
      endif
      if keyword_set(skip) then begin
        if n_elements(dlength) ne 1 or n_elements (iextra) lt 8 or $
        n_elements (pfftype) ne 1 then message, 'Error with keyword SKIP'
      endif else begin
        if size(/type, dataset) eq 7 then begin
          nmap = 0L
          nmax = 100
          get_dset_string, dataset, 0L, map, nmap, nmax, ierror
          if ierror ne 0 then message, '% READSURF: Please change the dataset specification'
          if nmap gt 1 then begin
            Print, '% READSURF: Please change the dataset specification to get a single dataset.'
            help, datasetin
            print, 'Datasets: '
            for i = 0, nmap-1 do print, map[i]
            return
          endif
          dataset = map[0]
        endif
        readhdr, dataset, ierror,  aptype = aptype, pfftype = pfftype, /quiet
      endelse
;
      if ierror ne 0 then begin
        print, '% READSURF: Error reading header.'
        print, 'Dataset: '+strtrim(dataset, 2)
        return
      endif
      if pfftype ne 6 then begin
        print, '% READSURF: Surface dataset must be an integer-float list.'
        print, 'Dataset: '+strtrim(dataset, 2)
        return
      endif
      if aptype lt 200 or aptype gt 210 then begin
        print, '% READSURF: Surface dataset have an application dataset type'
        print, '            in the range 200 to 210'
        print, 'Dataset: '+strtrim(dataset, 2)
        return
      endif
      readifl, iarray, flist, farray, dataset, namefile = file, tlabel = tlab, $
           clabel = clabel, ierror = ier, npoints = np, apptyp = apptyp
;
      if ier ne 0 then begin
        print, '% READSURF: Error reading IFL dataset.'
        print, 'Dataset: '+strtrim(dataset, 2)
        return
      endif
;
      strtyp = (apptyp eq 200) ? 14L : 15L
      if n_elements(clabel_in) gt 0 then clab = strtrim (clabel_in[0], 2)
      iextra = lonarr(5, /nozero)
      v1 =   0.0 & v2 = 0.0 & v3=0.0
      if aptype gt 200 then begin
;       A field value
;       Integer list has [Time_stamp, Surface_#, coordinate_flag, Nvector]
        if np[0] ne 4 then begin
          print, '% READSURF: Wrong number of integers for dataset: '+strtrim(dataset, 2)
          help, iarray
          return
        endif
        timestamp = iarray[0]
        surfacenumber = iarray[1]
        topology = strtrim(timestamp, 2) + strtrim(string(format='(i4.4)', surfacenumber), 2)
        nvect = iarray[3]
        nnodes = np[2]/nvect
        if nvect *nnodes ne np[2] then begin
          print, '% READSURF: Wrong number of Vector elements:'
          help, nvect, nnodes, farray
          print, 'NVECT * NNODES = '+strtrim (nvect * nnodes, 2)
          return
        endif
        farray = reform (farray, nnodes, nvect, /over)
        v1= farray[*, 0]
        if nvect gt 1 then begin
          v2 = farray[*, 1]
          if nvect gt 2 then v3 = farray[*, 2]
        endif
        farray = 0
        get_dir, comment= comment, type = type, number = ndset, /no_trim
        com2 = strtrim(strmid(comment, 32), 2)
        com2 = strlowcase(com2)
        data_topology = get_match('surface topology '+topology, com2, nelem= nelem)
        if nelem gt 0 then data_topology = data_topology[0] else begin
          print, '% READSURF: Unable to find Topology dataset'
          print, '  Search string: Surface Topology '+topology
          return
        endelse
        if type[data_topology] ne 'Surface Topology' then begin
          print, '% READSURF:  Topology dataset has the wrong type label'
          help,  type[data_topology]
          return
        endif
        dataset = data_topology +1
        readifl, iarray, flist, farray, dataset, namefile = file, tlabel = tlab, $
           clabel = clab, ierror = ier, npoints = np, apptyp = apptyp
        if ier ne 0 then begin
          print, '% READSURF: Error reading topology IFL dataset.'
          print, 'DATASET: ', dataset
          return
        endif
        if apptyp ne 200 then begin
          print, '% READSURF: Topology dataset must have application equal to 200'
          help, apptyp
          return
        endif
;       Integer list has [Time_stamp, Surface_#, coordinate_flag, Nvector]
        if np[0] lt 4 then begin
          print, '% READSURF: Wrong number of integers for surface dataset: '+strtrim(dataset, 2)
          help, iarray
          return
        endif
        if iarray[0] ne timestamp or iarray[1] ne surfacenumber then begin
          print, '% READSURF: Dataset comment and integer array do not agree.'
          print, '  TimeStamps: ', timestamp, iarray[0]
          print, '  SurfaceNumber: ', surfacenumber, iarray[1]
          return
        endif
        if nnodes ne np[2]/3 then begin
          print, '% READSURF: Wrong number of nodes in the topology dataset'
          print, '  NNODES: '+strtrim(nnodes, 2), np[2]/3
          return
        endif
      endif else nvect = 0L
      if iarray[3] ne 3 then begin
        print, '% READSURF: Topology must be 3D data'
        print, '  [TimeStamp, Surface, Coordinate_Sys, NDim]'
        print, '  Integer array: ', IARRAY[0:3]
        return
      endif
      iextra[0] = iarray[2]
      iextra[1] = iarray[0]
      iextra[2] = iarray[1]
      iextra[3] = iarray[3]
      iextra[4] = nvect
      nnodes = np[2]/3
      
      farray = reform(farray, 3, nnodes, /overwrite)
      connectivity = iarray[4:*]
      nplane = np[0]/4-1
      connectivity = reform(connectivity, 4, nplane, /overwrite)
;
      range = fltarr(2, 3, 2)
      for i = 0, 2 do begin
        range[0, i, *] = min(farray(*, i), max = m)
        range[1, i, *] = m
      endfor
      if iextra[0] eq 1 then begin
        xlab = 'X axis'
        ylab = 'Y axis'
        zlab = 'Z axis'
      endif else begin
        xlab = 'Radius'
        ylab = 'Azimuth'
        zlab = 'Z axis'
      endelse
if strtyp eq 14 then $
      array = {type:strtyp, ndim:3L,  block :1L, nnodes: nnodes, nplane: nplane, $
              xlab: xlab, ylab: ylab, zlab: zlab, range: range, $
              XYZ: farray, connectivity : connectivity, loc : [0,nplane],  $
              file : file, spare : iextra, tlab : tlab[0], clab: clabel} else $
      array = {type:strtyp, ndim:3L,  block :1L, nnodes: nnodes, nplane: nplane, $
              nvect : nvect, xlab: xlab, ylab: ylab, zlab: zlab, range: range, $
              XYZ: farray, connectivity : connectivity, loc : [0,nplane],  $
              v1:v1, v2 :v2, v3:v3, $ 
              file : file, spare : iextra, tlab : tlab[0], clab: clabel}
      if keyword_set(structure) then return
      if lcstructure gt 0 then i2s, array, lcstructure, /no_copy
      end
