; ------------------------------------------------------------------------ 
; $Id: readuf1.pro,v 1.6 2010/03/04 15:41:24 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
 Pro readuf1, farray, datasetin, fileid = fid, stringlen= stringlen, $
              xlabel = xlabel, ylabel = ylabel, $
              blabel = blabel, tlabel= tlabel, $
              aptype = aptype, ispare = ispare, $
              clabel = clabel, namefile = filename, pfftyp = pfftyp,  $
              wdfarray = wdfarray, skip = skip, dlength = dlength, $
              iextra = iextra

;
;+
;
;  Reads a time history into an IDL structure.
;  Procedure options are explained below.
;  Current version limited to 512 blocks by the dimensions in pff_slruf1
;  PFF dataset type must be 1 UF1 = Scalar data on a 1D uniform grid
;
;  Format:  READUF1, STRUCTURE  [, dataset] [, FILEID = fileid]
;                    [, STRINGLEN = stringlen]
;                    [, CLABEL = clabel] [, TLABEL = tlabel]
;                    [, XLABEL = xlabel] [, YLABEL = ylabel] 
;                    [, BLABEL = blabel] 
;                    [, APTYPE = aptype] [, ISPARE = ispare]
;                    [, WDFarray = wdfarray] [, NAMEFILE = namefile] 
;  where:
;    STRUCT  - IDL structure name or WDF array (see WDFarray)
;              This must be a valid variable             
;    dataset - indicates a PFF dataset number 
;              or is a search string for dataset comments.  
;              If dataset is missing or zero, then it is set to the 
;              current dataset.
;        Note: If dataset is a string, a leading `^' indicates that 
;              the string begins at the start of the dataset comment.
;    fileid -  file id of the PFF file containing the data
;    stringlen - Currently Ignored
;                Maximum number of characters in the spatial labels and the
;                field labels.  Maximum of 64 characters for each block.
;    tlabel  - Dataset type label
;    clabel  - Dataset comment label
;    xlabel  - Dataset xlabel for each block
;    blabel, ylabel - Dataset block label for each block
;    aptype  - Application dataset type
;    ispare  - spare array
;    NAMEFILE - Name of the PFF file
;    WDFarray - If keyword is not zero, STRUCT =[X0, DX, Yarray] where
;               X0 is the initial X value, DX is the X spacing and 
;               YARRAY are the ordinate values.
;               Only single block data may be returned as a WDFarray.
;               If WDFARRAY is zero or omitted then the data will be returned
;               as a structure with the following labels:
;
;  Structure Labels
;                  type    - 6 type for WDF arrays
;                  ndim    - 1 dimension
;                  block   - 1 Maximum number of blocks
;                  range   - 2 element array with the minimum and maximum
;                            abscissa values
;                  yrange  - 2 element array with the minimum and maximum
;                            ordinate values
;                  np      - number of points
;                            zero indicates no data in this array
;                  x0      - Initial point
;                  dx      - Spacing for uniform data; -1 for x-y data
;                  x       - Abscissa values
;                  y       - Ordinate values
;                  xlab    - Abscissa label
;                  ylab    - Ordinate label
;                  locv    - Array of the location of the first point for each
;                            block
;                  file    - File from which the data originated
;                  clab    - Dataset comment label
;                  tlab    - Dataset type label
;                  ispare  - Spare array for the first block
;
; If multiple blocks exist, then some of these will be arrays
;
;  Examples:
;    Read dataset 34 in file id 4 into structure ch5
;      readuf1, ch5, 34, fil=4
;    Read current dataset of the current file into structure efldstart
;      readuf1, efldstart
;    Read the current dataset into a WDFarray image
;      readuf1,  image, /wdf
;-
;
      on_error, 1
;
      common pffcom, initpff, pff_exe
@pff_strings.com
      stringlen = max_str_len
;
;     check to see if pff has been initialized
;
      If n_elements(initpff) eq 0 then initpff = 0
      if (initpff eq 0) then begin
        print, '% READUF1: File must be opened before reading data'
        return
      endif
;
;     check on the number of parameters
;
;debug
; tstart = systime(1)

      nparams =  n_params()
      slice = 0.0
      farray = 0.0
      if (nparams lt 1 ) then begin
        print,'% Format: READUF1, STRUCTURE  [, dataset] [, FILEID = fileid]'
        print,'        [, STRINGLEN = stringlen]'
        print,'        [, CLABEL = clabel] [, TLABEL = tlabel]'
        print,'        [, XLABEL = xlabel] [, YLABEL = ylabel]'
        print,'        [, BLABEL = blabel] [, NAMEFILE = namefile]'
        print,'        [, APTYPE = aptype] [, ISPARE = ispare]'
        print,'        [, WDFarray = wdfarray]'
        return
      endif
      if not arg_present(farray) then begin
        print, '% READUF1: Warning - structure cannot be returned in STRUCTURE'
        print, '           Please provide a variable.'
        return
      endif
      if n_elements (fid) eq 0 then fid = 0L
      pid = long(fid)
      if n_elements (datasetin) eq 0 then dataset = 0L else $
        dataset = datasetin(0)
      if keyword_set(skip) then begin
        if n_elements(dlength) ne 1 or n_elements (iextra) lt 8 or $
        n_elements (pfftyp) ne 1 then message, 'Error with keyword SKIP'
      endif else begin
        nmap = 0L
        nmax = 1L
        get_dset_string, dataset,pid,  map, nmap, nmax, ier
        if ier ne 0 then message, ' Please change the dataset specification'
;
        if nmap gt 1 then begin
          Print,'% READUF1: Please change the dataset specification to get a single dataset.'
          help, dataset
          print, 'Datasets: '
          for i = 0, nmap-1 do print, map[i]
          return
        endif

        dataset = map(0)
        readhdr,dataset,ier,fileid=pid, pfftyp = pfftyp, $
           dlength = dlength, tlabel= tlabel, clabel=clabel, $
           aptype = aptype, namefile = filename, iextra = iextra, /quiet
;
        if ier ne 0 or n_elements(pfftyp) eq 0 then begin
          print, ' Error reading dataset header'
          return
        endif
      endelse
      if (pfftyp  ne 2 ) then begin
        print, '% READUF1: Dataset type '+strtrim(pfftyp, 2) + $
            'cannot be read with READUF1'
        return
      endif
; READUF1
;               For UF1
;                 iextra(0) = number of blocks
;                 iextra(1) = number of X-values in first block
;                 iextra(2) = number of words read (header and length info)
;                 iextra(3) = number of spare words in each block
      iextra  = long(iextra)
      mblo = iextra(0) < 512
      if keyword_set (wdfarray) and mblo gt 1 then begin
        Print, '%READUF1: Multiple block UF1 arrays cannot be read with' + $
                       'WDFARRAY keyword set.'
        return
      end
;
      if mblo eq 1 then mdim = iextra(1) $
        else mdim = dlength -iextra(2)
      farray = make_array(mdim)
      nx = make_array(mblo, /long)
      x0 = make_array(mblo, /float)
      dx = make_array(mblo, /float)
      locfa = make_array(mblo+1, /LONG)
      tapp = 0L
      nblo = 0L
      ier = 0L
      ispare = make_array(5, mblo, /long)
      lxstr = long(stringlen)
      xlabel = make_array(lxstr*mblo, /byte, value = 32B)
      blabel = make_array(lxstr*mblo, /byte, value = 32B)
      llstr = 40L
      clab = make_array(llstr, /byte, value = 32B)
      tlab = make_array(llstr, /byte, value = 32B)

      foff10 = make_array(/long, mblo, value = 0L)
      goff10 = make_array(/long, mblo, value = 0L)

      if  initpff eq 1 then begin
; HP   NOTE: tlab and clab are not returned
        junk = call_external(PFF_EXE,'pff_slruf1', pid, dataset, mblo, mdim, $
             tapp, tlab, clab, nblo, nx, ispare, x0, dx, goff10, xlabel, blabel, $
             locfa,  farray, foff10, ier, lxstr, llstr)

      endif else if initpff eq 2 then begin
; Sun
        junk = call_external(PFF_EXE,'pff_slruf1_', pid, dataset, mblo, mdim, $
                tapp, tlab, clab, nblo, nx, ispare, $
                x0, dx, goff10, xlabel,  blabel, $
                locfa,  farray, foff10, ier, lxstr, llstr)

      endif else if initpff eq 4 then begin
; WIN
        junk = call_external(PFF_EXE,'_PFF_SLRUF1@8', pid, dataset, mblo, mdim, $
                tapp, tlab, clab, nblo, nx, ispare, $
                x0, dx, goff10, xlabel,  blabel, $
                locfa,  farray, foff10, ier, lxstr, llstr)
      endif

      if ier ne 0 then begin
        print, 'READUF1: Error reading dataset, IER = ',strtrim(ier,2)
        farray = 0
        return
      endif
      if max(abs(foff10)) ne 0 or max(abs(goff10)) ne 0 then begin
        print,'WARNING - Power of 10 offset for the data exceeds computer range'
        print,'  Data X offsets - ' , goff10(0:mblo-1) 
        print,'  Data Y offsets - ' , foff10(0:mblo-1) 
      endif
      if nblo gt mblo then begin
        Print, ' Problem with the number of blocks, contact Paul Mix.'
        farray = 0
        return
      endif
;
;    pfftype - PFF dataset type 2: UF1
;
      if keyword_set(wdfarray) then begin
        xlabel = strtrim(xlabel(0:lxstr-1), 2)
        blabel = strtrim(blabel(0:lxstr-1), 2)
        ylabel = blabel
        farray = [x0(0), dx(0), farray]
        ispare = reform (ispare)
        return
      endif
;     set up structures
      if mblo eq 1 then begin
        xlabel = strtrim(xlabel(0:lxstr-1), 2)
        blabel = strtrim(blabel(0:lxstr-1), 2)
        ylabel = blabel
        ispare = reform (ispare)
        nx = nx(0)
        x0 = x0(0)
        dx = dx(0)
        range = [x0, x0+(nx-1)*dx]
        yrange = [min(farray(0:nx-1), max = mm), mm]
        x1 = x0 + findgen(nx)*dx
        farray = {type:6L, ndim:1L, range:range, yrange: yrange, $
                 np:nx, block:1L, locv:[0, nx], x0:x0, dx:dx, $
                 x:x1, y:farray, xlab: xlabel, ylab: blabel, $
                 file: filename, clab:clabel, tlab:tlabel, $
                 ispare:ispare }
      endif else begin
        locfa(mblo) = locfa(mblo-1)+nx(mblo-1)
        locfa = locfa-1
        range = make_array(2, mblo+1)
        nxt = round (total (nx))
        x1 = make_array(nxt)
        range = transpose (reform ([0.0,x0, 0.0,x0+dx*(nx-1)], mblo+1, 2))
        range(0, 0)= min(x0)
        range(1, 0)= max(range(1, 1:mblo))
        yrange = make_array(2, mblo+1)
        yrange(*, 0) = [ min(farray, max = mm), mm]
        xlab = make_array(mblo, /string)
        blab = make_array(mblo, /string)
        for i = 0, mblo-1 do begin
          x1(locfa(i):locfa(i+1)-1) = x0(i) + dx(i)*(nx(i)-1)
          xlab(i) = strtrim(xlabel(i*lxstr:(i+1)*lxstr-1), 2)
          blab(i) = strtrim(blabel(i*lxstr:(i+1)*lxstr-1), 2)
          yrange(*, i+1)=[min(farray(locfa(i):locfa(i+1)-1), max = mm), mm]
        endfor
        farray = {type:6L, ndim:1L, range:range, yrange: yrange, $
                 np:nx, block:mblo, locv:locfa, x0:x0, dx:dx, $
                 x:x1, y:farray, xlab: xlab, ylab: blab, $
                 file: filename, clab:clabel, tlab:tlabel, $
                 ispare:ispare }
      endelse
      end
