; ------------------------------------------------------------------------ 
; $Id: wri.pro,v 1.6 2010/03/12 22:49:50 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
 Pro  wri, wdfin, fileid = fid, clabel = clabel, ylabel = ylabel, $
         xlabel= xlabel , tlabel = tlabel, ap = ap, $
         full_precision = full_precision
;
;+
;  This procedure writes a WDF array to a PFF file.  
;  Format: WRI, WD1 [, FILEID = FID] [, CLABEL = CLABEL]  [,  YLABEL = YLABEL]
;             [, XLABEL = XLABEL] [, TLABEL = tlabel] [, AP = apptype]
;             [, FULL_PRECISION = full_precision]
;  where:
;    wd1 - WDF array number
;    fileid - PFF file id.  If fileid  is missing or 0, the current PFF 
;             file is used.
;    tlabel - Dataset type label. If not specified, tlabel = 'WDF Time History'
;             Dataset type string limited to 256 characters.
;    clabel - dataset comment.  If not specified, the WDF comment is used.
;             Dataset comment string limited to 2048 characters.
;    xlabel - dataset x-axis label.  If not specified, the WDF x-axis 
;             label is used.  Label string limited to 64 characters.
;    ylabel - dataset block label (y-axis label).  If not specified, 
;             then the y-axis label is used as the block label.
;             Label string limited to 64 characters.
;    AP     - application dataset type (User parameter)
;        (default is 3 - the Quicksilver dataset type for time history data)
;    FULL_PRECISION - If set, FARRAY will be written with the indicated
;                     precision.  If not set, FARRAY will be written with
;                     the default precision.  
;                     If the data contains NaN or Inf numbers, then it will be 
;                     written as FULL_PRECISION.
;           FULL_PRECISION =0    Array written as a scaled 2 byte array
;           FULL_PRECISION not 0 Array written as a 32-bit IEEE array
;
;  Examples:
;  Write WDF array 1 to the current PFF file.
;    wri, 1
;  Write WDF array 3 to the PFF file with a file id of 4.
;    wri, 3, f = 4
;  Write WDF array 1 to the PFF file with a file id of 1 and use the dataset 
;  comments and block labels indicated.
;    wri, 1, f= 1,c='Calculated Diode Power, Shot 4443', y='Power-W'
;-   
;
;     add common block
@com.dat
@pff_strings.com
;
      common pffcom, initpff, pff_exe
      on_error, 1
;
      if (n_elements(initpff) eq 0) then initpff=0
      if (initpff eq 0) then begin
        print, '% WRI: File must be opened or created before writing data'
        return
      endif
;
      if n_params() eq 0 then begin
        print, '% Format: WRI, WD1 [, FILEID = FID] [, CLABEL = CLABEL]'
        print, '           [,  YLABEL = YLABEL] [, XLABEL = XLABEL] '
        print, '           [, TLABEL = tlabel] [, AP = apptype]'
        print, '           [, FULL_PRECISION = full_precision]'
        return
      endif
      if n_elements(wdfin) eq 0 then begin
        print,'WDF array value must be provided'
        return
      endif
      wdf = wdfin(0)
      if wdf lt 1 or wdf gt maxwdfarray then begin
        print, 'WDF array must be greater than 0 and </=', maxwdfarray
        return
      endif
      if wdfnp(wdf-1) lt 2 then begin
         print, 'No valid data in WDF array ', strtrim(wdf,2)
        return
      endif
      if n_elements (fid) eq 0 then fid = 0L else fid = long (fid(0))
      if n_elements(tlabel) eq 0 then tlabel = 'WDF Time History'
      if n_elements(clabel) eq 0 then clabel=wdfcomment(wdf-1)
      if n_elements(ylabel) eq 0 then ylabel=wdfylabel(wdf-1)
      if n_elements(xlabel) eq 0 then xlabel=wdfxlabel(wdf-1)
      lstrt = ((strlen(tlabel)/4 +1)*4) < max_tlabel
      lstrc = ((strlen(clabel)/4 +1)*4) < max_clabel
      lstrx = ((strlen(xlabel)/4 +1)*4) < max_str_len
      lstry = ((strlen(ylabel)/4 +1)*4) < max_str_len
      tblabel =  string( replicate(32b, lstrt))
      cblabel =  string( replicate(32b, lstrc))
      xblabel =  string( replicate(32b, lstrx))
      yblabel =  string( replicate(32b, lstry))
      if initpff ne 3 then begin
        str2byt, tlabel, tblabel, lstrt
        str2byt, clabel, cblabel, lstrc
        str2byt, xlabel, xblabel, lstrx
        str2byt, ylabel, yblabel, lstry
      endif
;
;     application dataset type for Quicksilver time-history data is 3
      if n_elements (ap) eq 0 then atype = 3L else atype = long(ap)
      nx = long(wdfnp(wdf-1))
      
      get_wdfarray, array, wdf, /xydata
      nblock = 1L
      ier = 0L
      if n_elements(array) eq nx+2 then lcuniform = 1L else lcuniform =0L
      offset = (lcuniform) ? 2 : nx
      ok = where (~finite(array[offset: *]), nok)
      if (nok gt 0) then full_precision =1
      if (n_elements(full_precision) gt 0) then begin
        pflag = keyword_set(full_precision[0])
        ier = 0
        if fid ne 0 then setpff, fid, ier
        if ier ne 0 then begin
          print, '% WRI: Invalid File ID, NO data written'
          return
        endif
        pff_precision, 0, full = pflag, prec = prec
        if (prec[1] eq pflag) then lcprecision = 1 else lcprecision = 0
      endif else lcprecision =0

      ispare = make_array(5, /long , value = -3L)
;
      if size(array, /type) ne 4 then array = float(array)
      if initpff eq 1 then begin
; HP
        junk=call_external(PFF_EXE, 'pff_slwf1',  fid, array, $
          nx, nblock, tblabel, cblabel, xblabel, yblabel, lcuniform, ispare, $
          atype, ier, lstrt, lstrc, lstrx, lstry)
      endif else if initpff eq 2 then begin
; Sun
        junk=call_external(PFF_EXE, 'pff_slwf1_',  fid, array, $
          nx, nblock, tblabel, cblabel, xblabel, yblabel, lcuniform, ispare, $
          atype, ier, lstrt, lstrc, lstrx, lstry)
      endif else if initpff eq 4 then begin
; WIN
        junk=call_external(PFF_EXE, '_PFF_SLWF1@8',  fid, array, $
          nx, nblock, tblabel, cblabel, xblabel, yblabel, lcuniform, ispare, $
          atype, ier, lstrt, lstrc, lstrx, lstry)
      endif else if initpff eq 5 then begin
; MAC
        junk=call_external(PFF_EXE, 'pff_slwf1',  fid, array, $
          nx, nblock, tblabel, cblabel, xblabel, yblabel, lcuniform, ispare, $
          atype, ier, lstrt, lstrc, lstrx, lstry)
      endif else if initpff eq 3 then begin
; VMS
        junk=call_external(PFF_EXE, 'pff_slwf1',  fid, array, $
          nx, nblock, tblabel, cblabel, xblabel, yblabel, lcuniform, ispare, $
          atype, ier)
      endif
;
      if lcprecision gt 0 then pff_precision, full = prec[1]
      if ier ne 0 then $
        Print,'% WRI: Error writing WDF array ',strtrim(wdf, 2),ier
      return
      end
