; ------------------------------------------------------------------------ 
; $Id: wristr.pro,v 1.12 2010/08/24 21:57:58 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
 pro wristr, ain, fid, clabel = clabel, fileid = fileid, tlabel = tlabel, $
             mask_bits = mskbits, shift_bits= shbits, ngd= ngd,   $
             aptype = apptyp, nf3 = nf3, full_precision = full_precision
;+
; WRISTR
; 
; CAUTION: This routine has few seat belts or airbags
;          It might work but don't be suprised if it doesn't
;
; For ASCII Write of an image see WRIMSD
;
; LIMITATIONS
;          ONLY: Grid, Particle, Field and Vector data are supported.
;          Current version does not specifically support 3D uniform data. 
;            Only single block uniform data (images) will use UF3.
;            Multi-block uniform data will be output as (nf3) nonuniform data. 
;            Default dataset types for single block data is NGD and UF3
;            NGD limited to 6 spatial dimensions and 6 vector dimensions
;                    but this is easily changed.
;            Multi block is written as NF3 or NV3
;            Integer data is written as NI3 
;            Integer data outside this range is automatically saved as a LONG  
;              if it is outside the range of an integer.
;
; Format: WRISTR, STR [, FID] [, CLABEL = clabel] [, TLABEL = TLABEL]
;                [, FILEID = FID] [, APTYPE = aptype] [, NF3] [, /NGD]
;                [, FULL_PRECISION = full_precision]
;                [, MASK_BITS = mskbits] [, SHIFT_BITS = shbits]
;       where
;       STR    - structure array number or a structure
;       FID    - file id of pff file.  Default is fid = 0 -- ie the current file
;                Note: if both FID and FILEID are specified, FID is used.
;       TLABEL - Type-label for dataset.  Default is TLAB is used
;       CLABEL - Comment label for dataset. Default is the comment label:CLAB
;       APTYPE - application dataset type :
;                Caution: If a normal type is used, when the data is read the
;                energy calculation might mess things up for particles!
;                Default values are 15 and 27 for fields and particles.
;       Only the first 60 characters are used for labels.
;       NF3    - If set, the field data will be written as NF3
;       NGD    - If set, the field data will be written as NGD.
;                (limited to single block field data)
; FULL_PRECISION - If defined, FARRAY will be written with the indicated
;                  precision.  If not defined, FARRAY will be written with
;                  the default precision.
;           FULL_PRECISION =0    Array written as a scaled 2 byte array
;           FULL_PRECISION not 0 Array written as a 32-bit IEEE array
;
;  The following only apply only to NI3 data
;    MASK_BITS -  # of bits from input word to be written to dataset
;    SHIFT_BITS-  # of bits to be shifted in input word before applying
;              - If mskbits is zero, the supplied input array is simply
;                truncated to 16-bits and all integers in the array must be in
;                the range -32768 to 32767.  shbits is ignored.
;              - If mskbits is nonzero, the value written to the file is shifted
;                and masked from the supplied input iarray, i.e.,
;                      ival = mod( iarray/(2**shbits),2**mskbits)
;                Note: In this mode, the supplied values in iarray must be
;                      non-negative.
;
; Example:
;       Write structure 34 to the current file
;         wristr, 34
;       Write structure F to the file with file id = 3
;         wristr, F, fid = 3
;  or     wristr, f, 3
;- 
;
; pff_slwstr uses the number of variables as a flag
; 29 variables = UF3
; 28 variables = NF3
; 30 variables = NV3
; 17 variables = NGD
; 16 variables = VTX
;
;  Common block
@qsstrcom.dat
;
      common pffcom, initpff, pff_exe
;
;       return on an error
      on_error, 1
;
@pff_strings.com
      stringlen = max_str_len
;
;     check to see if pff has been initialized
;
      if n_elements(initpff) eq 0 then initpff = 0
      if (initpff eq 0) then begin
        print, '% WRISTR: File must be opened or created before writing data'
        return
      endif
;
      if n_elements (structinit) eq 0 then qsstruct_init
;
      npar = n_params()
      if npar eq 0 then begin
        print,'% Format: ', $
            'WRISTR, STR [, FID] [, CLABEL = clabel]'
        print, '           [, FULL_PRECISION = full_precision]'
        return
      endif
;
;     now get the data  -- check that we have a structure of the proper type
;
      if n_elements(ain) eq 0 then begin
        print, 'Structure must be defined'
        return
      endif
; change variables to insure we don't corrupt input data
      a = ain
      b = size(a)
      if b(b(0)+1) ne 8 then begin
        if a gt 0 and a le maxstructure then begin
          s2i, a, a
          b = size (a)
          if b(b(0)+1) ne 8 then begin
            print, $
   ' First parameter must be a structure or a structure array number'
            return
          endif
        endif  else begin
          print, $
   ' First parameter must be a structure or a valid structure array number'
          return
        endelse
      endif
      valid = [1,2,3] 
      ok = where (a.type eq valid, nok)
      if nok eq 0  then begin
        print, ' Only Grid, Field, and Particle type structures may be written'
        return
      endif

      if n_elements(fid) eq 0 then begin
        if n_elements(fileid) eq 0 then fid = 0L else fid = long(fileid[0])
      endif else fid = long(fid[0])

      if n_elements(clabel) gt 0 then begin
        b = size(clabel(0))
        if b(1) ne 7 then clab = a.clab else clab = strtrim(clabel, 2)
      endif else clab = a.clab

      if n_elements(tlabel) gt 0 then begin
        b = size(tlabel(0))
        if b(1) ne 7 then tlab = a.tlab else tlab = strtrim(tlabel, 2)
      endif else tlab = a.tlab

      lstr = max_str_len
      lcstr = (strlen (clab)/4 +2L)*4L < max_clabel

      if n_elements(apptyp)eq 0 then begin
        if a.type eq 3 then tapp = 5L else $
        if a.type eq 1 then tapp = 15L else tapp = 27L
      endif else tapp = long(apptyp)
      str2byt,  tlab, st, lstr
      str2byt,  clab, sc, lcstr

; **********************************************************************
; FIELDS
  if a.type eq 1 then begin

      if a.ndim gt 3 and a.block gt 1 then begin
          print, ' Only 3 Dimensional Field Multi-Block data may be written'
          return
      endif

      mxbl = long(a.block)
      vsize = size(a.v1)
      vtyp = vsize(vsize(0)+1)

; vtyp = 1, 2, 3 indicates byte, integer, or long

      if ((mxbl eq 1) and (vtyp gt 3) and (not keyword_set(nf3))) then begin
;     use UF3 or NGD data type for all single block data
        ispare = long(a.spare)
        nspare = n_elements (ispare)
        m  = long(a.ndim)
;       if a.two eq 3 and m eq 3 then m = 2L
;       Above commented out to allow for writing our the slice dimension.
;
        n  = long(a.nvect)
        if ((m gt 6) or (n gt 6)) then begin
          print,'%WRISTR: Present version limited to 6, for ndim and nvect', $
               m, n
          return
        endif
        nx = long(a.size(0:m-1,  0))
        mg = long(nx(0))
        if m gt 1 then for i = 1, m-1 do mg = long(mg*nx(i))
        xlabel = make_array(lstr*m, /byte)
        str2byt, a.x1lab(0), temp, lstr
        xlabel (0:lstr-1) = temp
        ok = where(strpos(tag_names(a), 'DX') ge 0 , nok)
        lcunif = 0
;  for single block, scalar, uniform 2d and 3d data, use uf3 unless NGD is set
        if nok gt 0 and m gt 1 and m lt 4 and n eq 1 $
           and not keyword_set(ngd) then begin
          if ((a.dx1(0) gt 0.0) and (a.dx2(0) gt 0.0)) then lcunif = 2
          if a.two eq 3 and a.dx3 eq 0 then a.dx3 = 1.0
          if ((m gt 2) and  (a.dx3(0) gt 0.0)) then lcunif = lcunif+1
          if m ne lcunif then lcunif = 0 else begin
;           insure that ispare has 5 elements
            if nspare lt 5 then ispare = [ispare, 0L, 0L, 0L, 0L, 0L]
            if m eq 2 then kmax = 1L else kmax = nx[2]
            jmax = nx[1]
            imax = nx[0]
            x0 = float(a.x1[0])
            y0 = float(a.x2[0])
            z0 = float(a.x3[0])
            dx = float(a.dx1[0])
            dy = float(a.dx2[0])
            dz = float(a.dx3[0])
            xlabel = strarr2byt(a.x1lab, lstr, mxbl)
            ylabel = strarr2byt(a.x2lab, lstr, mxbl)
            zlabel = strarr2byt(a.x3lab, lstr, mxbl)
            blabel = strarr2byt(a.v1lab, lstr, mxbl)
            low = 1L
            locb = long(a.locv) +low
            idlist = make_array(3, mxbl, /long, value = low)
            jdlist = idlist
            kdlist = idlist
            if low ne 1 then begin
              idlist(2, *) =1L
              jdlist(2, *) =1L
              kdlist(2, *) =1L
              idlist(1, *) = imax - (1L -low)
              jdlist(1, *) = jmax - (1L -low)
              kdlist(1, *) = kmax - (1L -low)
            endif else  begin
              idlist(1, *) = imax
              jdlist(1, *) = jmax
              kdlist(1, *) = kmax
            endelse
            farray = float(a.v1)
            ier =0L
            if (n_elements(full_precision) gt 0) then begin
              pflag = keyword_set(full_precision[0])
              ier = 0
              if fid ne 0 then setpff, fid, ier
              if ier ne 0 then begin
                print, '% WRISTR: Invalid File ID, NO data written'
                return
              endif
              pff_precision, 0, full = pflag, prec = prec
              if (prec[1] eq pflag) then lcprecision = 1 else lcprecision = 0
            endif else lcprecision =0
;
; fid, tapp, st, sc, mxbl, ispare, x0, dx, y0, dy, z0, dz, 
; xlabel, ylabel, zlabel, blabel, locb, low, imax, jmax, kmax,
; idlist, jdlist, kdlist, farray, ier
;
;       add a dummy variable for UF3 
          lcdummy = 1L
;       UF3 Write
            if initpff eq 1 then begin
; HP
              junk=call_external(PFF_EXE, 'pff_slwstr', $
               fid, tapp, st, sc, mxbl, ispare, x0, dx, y0, dy, z0, dz, $
               xlabel, ylabel, zlabel, blabel, locb, low, imax, jmax, kmax, $
               idlist, jdlist, kdlist, farray, ier, lcdummy, lstr, lcstr)

            endif else if initpff eq 2 then begin
; Sun
              junk=call_external(PFF_EXE, 'pff_slwstr_', $
               fid, tapp, st, sc, mxbl, ispare, x0, dx, y0, dy, z0, dz, $
               xlabel, ylabel, zlabel, blabel, locb, low, imax, jmax, kmax, $
               idlist, jdlist, kdlist, farray, ier, lcdummy, lstr, lcstr)

            endif else if initpff eq 4 then begin
; WIN
              junk=call_external(PFF_EXE, '_PFF_SLWSTR@8', $
               fid, tapp, st, sc, mxbl, ispare, x0, dx, y0, dy, z0, dz, $
               xlabel, ylabel, zlabel, blabel, locb, low, imax, jmax, kmax, $
               idlist, jdlist, kdlist, farray, ier, lcdummy, lstr, lcstr)

            endif else if initpff eq 5 then begin
; MAC
              junk=call_external(PFF_EXE, 'pff_slwstr', $
               fid, tapp, st, sc, mxbl, ispare, x0, dx, y0, dy, z0, dz, $
               xlabel, ylabel, zlabel, blabel, locb, low, imax, jmax, kmax, $
               idlist, jdlist, kdlist, farray, ier, lcdummy, lstr, lcstr)

            endif else if initpff eq 3 then begin
; VMS
              PRINT, '% WRISTR: NOT supported'
            endif

            if lcprecision gt 0 then pff_precision, full = prec[1]
            if ier ne 0 then print, '% WRISTR - Error writing UF3 dataset'
            return
          endelse
        endif 
        if lcunif eq 0 then begin
          x = make_array(round(total(nx)))
          x(0: nx(0)-1) = a.x1(0:nx(0)-1)
          if m gt 1 then begin
            str2byt, a.x2lab(0), temp, lstr
            xlabel (lstr:2*lstr-1) = temp
            loc = nx[0]+nx[1]
            x[nx[0]: loc-1] = a.x2[0:nx[1]-1]
            if m gt 2 then begin
              str2byt, a.x3lab(0), temp, lstr
              xlabel (2*lstr:3*lstr-1) = temp
              loc2 = loc+nx(2)
              x(loc: loc2-1) = a.x3(0:nx(2)-1)
              if m gt 3 then begin
                str2byt, a.x4lab(0), temp, lstr
                xlabel (3*lstr:4*lstr-1) = temp
                loc3 = loc2+nx(3)
                x[loc2: loc3-1] = a.x4(0:nx(3)-1)
                if m gt 4 then begin
                  str2byt, a.x5lab(0), temp, lstr
                  xlabel [4*lstr:5*lstr-1] = temp
                  loc4 = loc3+nx[4]
                  x[loc3: loc4-1] = a.x5[0:nx[4]-1]
                  if m gt 5 then begin
                    str2byt, a.x6lab[0], temp, lstr
                    xlabel[5*lstr:6*lstr-1] = temp
                    loc5 = loc4+nx[5]
                    x[loc4: loc5-1] = a.x6[0:nx[5]-1]
                  endif
                endif
              endif
            endif
          endif
        endif
        v = make_array(n*mg)
        vlabel = make_array(lstr*n, /byte)
        str2byt, a.v1lab(0), temp, lstr
        vlabel (0:lstr-1) = temp
        v(0:mg-1)= a.v1(0:mg-1)
        if n gt 1 then begin
          str2byt, a.v2lab(0), temp, lstr
          vlabel (lstr:2*lstr-1) = temp
          v(mg: 2*mg-1) = a.v2(0:mg-1)
          if n gt 2 then begin
            str2byt, a.v3lab(0), temp, lstr
            vlabel (2*lstr:3*lstr-1) = temp
            v(2*mg: 3*mg-1) = a.v3(0:mg-1)
            if n gt 3 then begin
              str2byt, a.v4lab(0), temp, lstr
              vlabel (3*lstr:4*lstr-1) = temp
              v(3*mg: 4*mg-1) = a.v4(0:mg-1)
              if n gt 4 then begin
                str2byt, a.v5lab(0), temp, lstr
                vlabel (4*lstr:5*lstr-1) = temp
                v(4*mg: 5*mg-1) = a.v5(0:mg-1)
                if n gt 4 then begin
                  str2byt, a.v6lab(0), temp, lstr
                  vlabel (5*lstr:6*lstr-1) = temp
                  v(5*mg: 6*mg-1) = a.v5(0:mg-1)
                endif
              endif
            endif
          endif
        endif
        ier = 0L
        if (n_elements(full_precision) gt 0) then begin
          pflag = keyword_set(full_precision[0])
          ier = 0
          if fid ne 0 then setpff, fid, ier
          if ier ne 0 then begin
            print, '% WRISTR: Invalid File ID, NO data written'
            return
          endif
          pff_precision, 0, full = pflag, prec = prec
          if (prec[1] eq pflag) then lcprecision = 1 else lcprecision = 0
        endif else lcprecision =0
;       Write NGD data
        if initpff eq 1 then begin
; HP
          junk=call_external(PFF_EXE, 'pff_slwstr', fid, tapp, st, sc, $
           m, n, nx, mg, nspare, ispare, xlabel, vlabel, x, v, ier, lstr, lcstr)

        endif else if initpff eq 2 then begin
; Sun
          junk=call_external(PFF_EXE, 'pff_slwstr_', fid, tapp, st, sc, $
           m, n, nx, mg, nspare, ispare, xlabel, vlabel, x, v, ier, lstr, lcstr)

        endif else if initpff eq 4 then begin
; WIN
          junk=call_external(PFF_EXE, '_PFF_SLWSTR@8', fid, tapp, st, sc, $
           m, n, nx, mg, nspare, ispare, xlabel, vlabel, x, v, ier, lstr, lcstr)

        endif

        if lcprecision gt 0 then pff_precision, full = prec[1]
        if ier ne 0 then print, '% WRISTR - Error writing NGD dataset'
        return
      endif

; multiple block data or integer data
; use nf3, ni3 and nv3 pff dataset types

      n_isp = 5*mxbl
      if n_isp eq n_elements(a.spare)  then ispare = long(a.spare) else begin
        ispare = make_array(5, mxbl, /long )
        for i = 0, mxbl-1 do ispare(*, i)  = a.spare(0:4)
      endelse
      x = float(a.x1)
      y = float(a.x2)
      if a.ndim eq 2 then z = make_array(mxbl, value = 0.0) else $
        z = float(a.x3)
;     maximum number of characters is 720
;     for this version we will assume 60 =lstr characters per label
;
      xlabel = strarr2byt((a.x1lab), lstr, mxbl)
      ylabel = strarr2byt(a.x2lab, lstr, mxbl)
      zlabel = strarr2byt(a.x3lab, lstr, mxbl)
      blabel = strarr2byt(a.v1lab, lstr, mxbl)
  
      low = 1L
      locx = long(a.locx(0, *)) +low
      locy = long(a.locx(1, *)) +low
      if a.ndim ge 3 then locz = long(a.locx(2, *)) +low $
           else locz = lindgen(mxbl)
      locb = long(a.locv) +low
      imax = long(a.size(0, *))
      jmax = long(a.size(1, *))
      if a.ndim ge 3 then kmax = long(a.size(2, *)) $
           else kmax = make_array(mxbl, /long, value = 1L)
      idlist = make_array(3, mxbl, /long, value = low)
      jdlist = idlist
      kdlist = idlist
      if low ne 1 then begin
        idlist(2, *) =1L
        jdlist(2, *) =1L
        kdlist(2, *) =1L
        idlist(1, *) = imax - (1L -low)
        jdlist(1, *) = jmax - (1L -low)
        kdlist(1, *) = kmax - (1L -low)
      endif else  begin
        idlist(1, *) = imax
        jdlist(1, *) = jmax
        kdlist(1, *) = kmax
      endelse
      if a.nvect eq 1 then farray = a.v1 else begin
        vx  = float(a.v1)
        vy  = float(a.v2)
        if a.nvect ge 3 then vz  = float(a.v3) else vz = 0.0*vy
      endelse
      if (n_elements(full_precision) gt 0) then begin
        ier =0L
        pflag = keyword_set(full_precision[0])
        if fid ne 0 then setpff, fid, ier
        if ier ne 0 then begin
          print, '% WRISTR: Invalid File ID, NO data written'
          return
        endif
        pff_precision, 0, full = pflag, prec = prec
        if (prec[1] eq pflag) then lcprecision = 1 else lcprecision = 0
      endif else lcprecision =0
      if vtyp gt 3 then begin
        ier = 0L
      if a.nvect eq 1 then begin
        farray = float (farray)
        if initpff eq 1 then begin
; HP
          junk=call_external(PFF_EXE, 'pff_slwstr', fid, tapp, st, sc, $
            mxbl, ispare, x, y, z, xlabel, ylabel, zlabel, blabel, $
            locx, locy, locz, locb, low, imax, jmax, kmax, $
            idlist, jdlist, kdlist, farray, ier, lstr, lcstr)
        endif else if initpff eq 2 then begin
; Sun
          junk=call_external(PFF_EXE, 'pff_slwstr_', fid, tapp, st, sc, $
            mxbl, ispare, x, y, z, xlabel, ylabel, zlabel, blabel, $
            locx, locy, locz, locb, low, imax, jmax, kmax, $
            idlist, jdlist, kdlist, farray, ier, lstr, lcstr)
        endif else if initpff eq 4 then begin
; WIN
          junk=call_external(PFF_EXE, '_PFF_SLWSTR@8', fid, tapp, st, sc, $
            mxbl, ispare, x, y, z, xlabel, ylabel, zlabel, blabel, $
            locx, locy, locz, locb, low, imax, jmax, kmax, $
            idlist, jdlist, kdlist, farray, ier, lstr, lcstr)
        endif else if initpff eq 5 then begin
; MAC
          junk=call_external(PFF_EXE, 'pff_slwstr', fid, tapp, st, sc, $
            mxbl, ispare, x, y, z, xlabel, ylabel, zlabel, blabel, $
            locx, locy, locz, locb, low, imax, jmax, kmax, $
            idlist, jdlist, kdlist, farray, ier, lstr, lcstr)
        endif
        if ier ne 0 then print, '%WRISTR - Error writing NF3 dataset'
        if lcprecision gt 0 then pff_precision, full = prec[1]
        return
      endif else begin
        if initpff eq 1 then begin
; HP
          junk=call_external(PFF_EXE, 'pff_slwstr', fid, tapp, st, sc, $
            mxbl, ispare, x, y, z, xlabel, ylabel, zlabel, blabel, $
            locx, locy, locz, locb, low, imax, jmax, kmax, $
            idlist, jdlist, kdlist, vx, vy, vz, ier, lstr, lcstr)
        endif else if initpff eq 2 then begin
; Sun
          junk=call_external(PFF_EXE, 'pff_slwstr_', fid, tapp, st, sc, $
            mxbl, ispare, x, y, z, xlabel, ylabel, zlabel, blabel, $
            locx, locy, locz, locb, low, imax, jmax, kmax, $
            idlist, jdlist, kdlist, vx, vy, vz, ier, lstr, lcstr)
        endif else if initpff eq 4 then begin
; WIN
          junk=call_external(PFF_EXE, '_PFF_SLWSTR@8', fid, tapp, st, sc, $
            mxbl, ispare, x, y, z, xlabel, ylabel, zlabel, blabel, $
            locx, locy, locz, locb, low, imax, jmax, kmax, $
            idlist, jdlist, kdlist, vx, vy, vz, ier, lstr, lcstr)
        endif else if initpff eq 5 then begin
; MAC
          junk=call_external(PFF_EXE, 'pff_slwstr', fid, tapp, st, sc, $
            mxbl, ispare, x, y, z, xlabel, ylabel, zlabel, blabel, $
            locx, locy, locz, locb, low, imax, jmax, kmax, $
            idlist, jdlist, kdlist, vx, vy, vz, ier, lstr, lcstr)

        endif
        if ier ne 0 then print, '%WRISTR - Error writing NV3 dataset'
        if lcprecision gt 0 then pff_precision, full = prec[1]
        return
      endelse
    endif
    if (vtyp lt 4) then begin
      ier = 0L
      if (n_elements(full_precision) gt 0) then begin
        pflag = keyword_set(full_precision[0])
        ier = 0
        if fid ne 0 then setpff, fid, ier
        if ier ne 0 then begin
          print, '% WRISTR: Invalid File ID, NO data written'
          return
        endif
        pff_precision, 0, full = pflag, prec = prec
        if (prec[1] eq pflag) then lcprecision = 1 else lcprecision = 0
      endif else lcprecision =0
        farray = long(farray)
        mskbits = 0L
        shbits =0L
        mspare =5L
        nspare = 5L
      if initpff eq 1 then begin
; HP
        junk=call_external(PFF_EXE, 'pff_slwni3', fid, tapp, st, sc, $
          mxbl, mspare, nspare, ispare, x, y, z, xlabel, ylabel, zlabel, blabel, $
          locx, locy, locz, locb, low, imax, jmax, kmax, $
          idlist, jdlist, kdlist, mskbits, shbits, farray, ier, lstr, lcstr)
      endif else if initpff eq 2 then begin
; Sun
        junk=call_external(PFF_EXE, 'pff_slwni3_', fid, tapp, st, sc, $
          mxbl, mspare, nspare, ispare, x, y, z, xlabel, ylabel, zlabel, blabel, $
          locx, locy, locz, locb, low, imax, jmax, kmax, $
          idlist, jdlist, kdlist, mskbits, shbits, farray, ier, lstr, lcstr)
      endif else if initpff eq 4 then begin
; WIN
        junk=call_external(PFF_EXE, '_PFF_SLWNI3@8', fid, tapp, st, sc, $
          mxbl, mspare, nspare, ispare, x, y, z, xlabel, ylabel, zlabel, blabel, $
          locx, locy, locz, locb, low, imax, jmax, kmax, $
          idlist, jdlist, kdlist, mskbits, shbits, farray, ier, lstr, lcstr)
      endif
;
      if lcprecision gt 0 then pff_precision, full = prec[1]
      if ier ne 0 then Print,'% WRISTR: Error writing NI3 dataset'
      return
    endif
  endif
; **********************************************************************
  if a.type eq 2 then begin
; PARTICLES
;     fid, tapp, st, sc are already defined.
;   
      m = long(a.ndim)
      n = long(a.natt)
      if m gt 4 or n gt 6 then begin
        print, 'This version limited to 4 spatial and 6 attribute dimensions'
        return
      endif
      
      nvert = long(a.nvert)
      spare = make_array(5 , /long , value = -3L)
      spare (0:n_elements(a.spare)-1) = long(a.spare)
      nvm = nvert-1

      vert = make_array(m>1, nvert)
      if m gt 0 then begin
        xlabel = make_array(m, /string)
        xlabel(0) = a.x1lab
        vert(0, *) = a.x1(0:nvm)
        if m gt 1 then begin
          xlabel(1) = a.x2lab
          vert(1, *) = a.x2(0:nvm)
          if m gt 2 then begin
            vert(2, *) = a.x3(0:nvm)
            xlabel(2) = a.x3lab
            if m eq 4 then begin
              xlabel(3) = a.x4lab
              vert(3, *) = a.x4(0:nvm)
            endif
          endif
        endif
        if initpff ne 3 then $
          xlabel = strarr2byt(xlabel, lstr, m)
      endif else xlabel = 0L

      attr = make_array(nvert, n>1)
      if n gt 0 then begin
        alabel = make_array(n, /string)
        alabel(0) = a.v1lab
        attr(*, 0) = a.v1(0:nvm)
        if n gt 1 then begin
          attr(*, 1) = a.v2(0:nvm)
          alabel(1) = a.v2lab
          if n gt 2 then begin
            attr(*, 2) = a.v3(0:nvm)
            alabel(2) = a.v3lab
            if n gt 3 then begin
              attr(*, 3) = a.v4(0:nvm)
              alabel(3) = a.v4lab
              if n gt 4 then begin
                attr(*, 4) = a.v5(0:nvm)
                alabel(4) = a.v5lab
                if n gt 5 then begin
                  attr(*, 5) = a.v6(0:nvm)
                  alabel(5) = a.v6lab
                endif
              endif
            endif
          endif
        endif
        if initpff ne 3 then $
          alabel = strarr2byt(alabel, lstr, n)
      endif else alabel = 0L

      if (n_elements(full_precision) gt 0) then begin
        pflag = keyword_set(full_precision[0])
        ier = 0L
        if fid ne 0 then setpff, fid, ier
        if ier ne 0 then begin
          print, '% WRISTR: Invalid File ID, NO data written'
          return
        endif
        pff_precision, 0, full = pflag, prec = prec
        if (prec[1] eq pflag) then lcprecision = 1 else lcprecision = 0
      endif else lcprecision =0

      nv = nvert
      ier =0L
      if initpff eq 1 then begin
; HP
        junk=call_external(PFF_EXE, 'pff_slwstr', fid, tapp, st, sc, $
          m, n, nvert, spare, xlabel, alabel, nv, vert, attr, $
          ier, lstr, lcstr)
      endif else if initpff eq 2 then begin
; Sun
        junk=call_external(PFF_EXE, 'pff_slwstr_', fid, tapp, st, sc, $
          m, n, nvert, spare, xlabel, alabel, nv, vert, attr, $
          ier, lstr, lcstr)
      endif else if initpff eq 4 then begin
; WIN
        junk=call_external(PFF_EXE, '_PFF_SLWSTR@8', fid, tapp, st, sc, $
          m, n, nvert, spare, xlabel, alabel, nv, vert, attr, $
          ier, lstr, lcstr)
      endif else if initpff eq 5 then begin
; MAC
        junk=call_external(PFF_EXE, 'pff_slwstr', fid, tapp, st, sc, $
          m, n, nvert, spare, xlabel, alabel, nv, vert, attr, $
          ier, lstr, lcstr)
      endif else if initpff eq 3 then begin
; VMS
        junk=call_external(PFF_EXE, 'pff_slwstr', fid, tapp, st, sc, $
          m, n, nvert, spare, xlabel, alabel, nv, vert, attr, $
          ier )
      endif
;
      if ier ne 0 then print, '%WRISTR - Error writing vertex dataset'
      if lcprecision gt 0 then pff_precision, full = prec[1]

      return
   endif

; GRIDs
      mxbl = long(a.block)
      ispare = long (a.spare)
      xlabel = strarr2byt((a.x1lab), lstr, mxbl)
      ylabel = strarr2byt(a.x2lab, lstr, mxbl)
      zlabel = strarr2byt(a.x3lab, lstr, mxbl)
      blabel = strarr2byt(sindgen(mxbl), lstr, mxbl)
      x = float(a.x1)
      y = float(a.x2)
      z = float(a.x3)

      low = 1L
      locx = long(a.locx(0, *)) +low
      locy = long(a.locx(1, *)) +low
      locz = long(a.locx(2, *)) +low
      imax = long(a.size(0, *))
      jmax = long(a.size(1, *))
      kmax = long(a.size(2, *))
      idlist = make_array(3, mxbl, /long, value = low)
      jdlist = idlist
      kdlist = idlist
      idlist(1, *) = imax
      jdlist(1, *) = jmax
      kdlist(1, *) = kmax
      s = size(ispare)
      if ((s[0] lt 2) and (mxbl gt 1)) then $
        ispare = replicate(ispare, mxbl)
      mspare = n_elements(ispare)
      if mxbl gt 1 then mspare= mspare/mxbl
      nspare = mspare
        if (n_elements(full_precision) gt 0) then begin
          pflag = keyword_set(full_precision[0])
          ier = 0
          if fid ne 0 then setpff, fid, ier
          if ier ne 0 then begin
            print, '% WRISTR: Invalid File ID, NO data written'
            return
          endif
          pff_precision, 0, full = pflag, prec = prec
          if (prec[1] eq pflag) then lcprecision = 1 else lcprecision = 0
        endif else lcprecision =0
        ier = 0L
        if initpff eq 1 then begin
; HP
          junk=call_external(PFF_EXE, 'pff_slwng3', fid, tapp, st, sc, $
           mxbl, mspare, nspare, ispare, x, y, z, $
           xlabel, ylabel, zlabel, blabel, locx, locy, locz, low, $
           imax, jmax, kmax, idlist, jdlist, kdlist, ier, lstr, lcstr)

        endif else if initpff eq 2 then begin
; Sun
          junk=call_external(PFF_EXE, 'pff_slwng3_', fid, tapp, st, sc, $
           mxbl, mspare, nspare, ispare, x, y, z, $
           xlabel, ylabel, zlabel, blabel, locx, locy, locz, low, $
           imax, jmax, kmax, idlist, jdlist, kdlist, ier, lstr, lcstr)

        endif else if initpff eq 4 then begin
; WIN
          junk=call_external(PFF_EXE, '_PFF_SLWNG3@8', fid, tapp, st, sc, $
           mxbl, mspare, nspare, ispare, x, y, z, $
           xlabel, ylabel, zlabel, blabel, locx, locy, locz, low, $
           imax, jmax, kmax, idlist, jdlist, kdlist, ier, lstr, lcstr)

        endif
     end
