; ------------------------------------------------------------------------ 
; $Id: wrivtx.pro,v 1.5 2007/06/06 19:59:48 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
 Pro wrivtx, attr, fileid = fid,  $
             xarray = x1, yarray = x2, tarray = x4, x4label = x4lab, $
             zarray= x3, xlabel = x1lab, ylabel = x2lab, $
             zlabel = x3lab, alabel = atlabel, tlabel= tlabel, $
             clabel = clabel, spare = spare, aptype = tapp, $
             full_precision = full_precision
;
;+
;
;  Write a field or charge density into an IDL structure.
;  Procedure options are explained below.
;  Dataset must be type 5, PFFVTX
;
;  Format:  WRIVTX [, ATTRIBUTES] [, FILEID = fileid]
;                   [, XARRAY= xarray] [, YARRAY= yarray]
;                   [, ZARRAY= zarray] [, TARRAY= tarray]
;                   [, APTYPE = aptype] [, ALABEL = alabel]
;                   [, CLABEL = clabel] [, TLABEL = tlabel]
;                   [, XLABEL = xlabel] [, YLABEL = ylabel] 
;                   [, ZLABEL = zlabel] [, X4Label = x4label]
;                   [, FULL_PRECISION = full_precision]
;                   [, SPARE = spare]
;  where:
;    ATTRIBUTES  - IDL Attribute Array size= (NVERT, NATTR)
;                  If dataset has no attributes, enter nothing or zero.
;                  If valid, the first dimension should have the same size
;                    as Xarray.
;    fileid -  file id of the PFF file containing the data
;    stringlen - Currently Ignored
;                Maximum number of characters in the spatial labels and the
;                field labels.  Currently 64 for each for each block
;        
;   Note: The following 4 spatial arrays must have the same number of elements
;            or they will be assumed to not contain valid data. 
;         The number of elements in XARRAY is taken as NVERT
;         The number of spatial arrays with the number of elements equal 
;            to NVERT is taken as NDIM (the number of dimensions).
;         
;    xarray    - Values for the first dimension 
;    yarray    - Values for the second dimension
;    zarray    - Values for the third dimension
;    tarray    - Values for the fourth dimension
;            
;    aptype    - Application dataset type
;    Alabel(*) - Labels the attributes
;                Must have the same number of elements as the second dimension
;                  of ATTRIBUTES.
;    Clabel    - Comment Label
;    TLABEL    - Type Label
;    SPARE     - Spare (5) spare array for each block. Default = -3
;    XLabel    - Label for XARRAY
;    YLabel    - Label for YARRAY
;    ZLabel    - Label for ZARRAY
;    X4Label    - Label for Tarray
;    FULL_PRECISION - If defined, FARRAY will be written with the indicated
;                     precision.  If not defined, FARRAY will be written with
;                     the default precision.
;           FULL_PRECISION =0    Array written as a scaled 2 byte array
;           FULL_PRECISION not 0 Array written as a 32-bit IEEE array
;  Examples:
;    Write arrays into a vertex dataset
;      Wrivtx, 0,xar= x, yar = y, zar = z 
;
;    Read a vertex dataset from dataset 10 and write it back out.
;   readvtx, a, 10, ndim = ndim, natt= nat, xa= xa,ya= ya, za = za, ta = ta, $ 
;   xl = xl, ylab = yl, zlab = zl, x4 = x4, alab = al, clab= clab, tlab=tlab, $
;    spare= spare, name = name, aptyp=aptyp
;   Do some stuff  - open another file if necessary
;   wrivtx, a, xa= xa,ya= ya, za = za, ta = ta, $
;   xl = xl, ylab = yl, zlab = zl, x4 = x4, alab = al, clab= clab, tlab=tlab, $
;   spare= spare, aptyp=aptyp
;
;-
;
      on_error, 1
;
      common pffcom, initpff, pff_exe
;
@pff_strings.com
      stringlen = max_str_len
;
;     check to see if pff has been initialized
;
      if n_elements(initpff) eq 0 then initpff = 0
      if (initpff eq 0) then begin
        print, '% WRIVTX: File must be opened or created before writing data'
        return
      endif
;
;     check on the number of parameters
;

      nparams =  n_params()
      if ((nparams lt 1) and (n_elements(x1) eq 0)) then begin
  print,'% Format: WRIVTX [, Attributes] [, FILEID = fileid]'
  print,'        [, XARRAY= xarray] [, YARRAY= yarray] [, ZARRAY= zarray]'
  print,'        [, TARRAY= tarray] [, X4LABEL = x4label]
  print,'        [, XLABEL= xlabel] [, YLABEL= ylabel] [, Zlabel= Zlabel]'
  print,'        [, ALABEL = alabel] [, CLABEL = clabel] [, TLABEL = tlabel]'
  print,'        [, SPARE = spare] [, APTYPE = APtype]'
  print,'        [, FULL_PRECISION = full_precision]'
        return
      endif

      nvert = n_elements(x1)
      if nvert lt 3 then begin
        Print, '% WRIVTX: Vertex data must have at lease 3 points'
        return
      endif
      if n_elements(x2) ne nvert then begin
        ndim = 1L
        vert = x1
      endif else if n_elements(x3) ne nvert then begin
        ndim = 2L
        vert = transpose(reform([x1, x2],nvert, ndim))
      endif else if n_elements(x4) ne nvert then begin
        ndim = 3L
        vert = transpose(reform([x1, x2, x3],nvert, ndim))
      endif else begin
        ndim = 4L
        vert = transpose(reform([x1, x2, x3, x4],nvert, ndim))
      endelse
      vert = float(vert)

       
      nela = n_elements(attr)
      if nela lt nvert then nattr = 0L else begin
        nattr = nela/nvert
        if nattr*nvert ne nela then begin
          Print, '% WRIVTX: Number of attributes must be equal to nvert*nattr'
          return
        endif
      endelse
           
      if nela eq 0 then attr = 0.0 else attr = float(attr)
      if n_elements (fid) eq 0 then fid = 0L
      pid = long(fid)
;
      
      lastr = stringlen
      if nattr eq 0 then alabel = '        ' else begin
        if n_elements(atlabel) eq 0 then begin
          alabel = make_array(nattr*lastr, /byte, value = 32B)
        endif else begin
          alabel = make_array(nattr*lastr, /byte, value = 32B)
          loc = lindgen(nattr+1)*lastr
          for i = 1, nattr do begin
            str2byt, atlabel(i-1), temp, lastr
            alabel(loc(i-1):loc(i)-1) = temp
          endfor
        endelse
      endelse
   
      tempp = make_array(4, val=0)
      if n_elements(x1lab) eq 0 then x1lab= '' else tempp(0) = strlen(x1lab)
      if n_elements(x2lab) eq 0 then x2lab= '' else tempp(1) = strlen(x2lab)
      if n_elements(x3lab) eq 0 then x3lab= '' else tempp(2) = strlen(x3lab)
      if n_elements(x4lab) eq 0 then x4lab= '' else tempp(3) = strlen(x4lab)
      lvstr = stringlen
      vlabel = make_array(ndim*lvstr, /byte, value = 32B)
      loc = indgen(ndim+1)*lvstr
      if tempp(0) gt 0 then begin
        str2byt, x1lab, temp, lvstr
        vlabel(loc(0):loc(1)-1)= temp
      endif
      if ((ndim gt 1) and (tempp(1) gt 0)) then begin
        str2byt, x2lab, temp, lvstr
        vlabel(loc(1):loc(2)-1)= temp
      endif
      if ((ndim gt 2) and (tempp(2) gt 0)) then begin
        str2byt, x3lab, temp, lvstr
        vlabel(loc(2):loc(3)-1)= temp
      endif
      if ((ndim gt 3) and (tempp(3) gt 0)) then begin
        str2byt, x4lab, temp, lvstr
        vlabel(loc(3):loc(4)-1)= temp
      endif
;
      if n_elements(spare) eq 0 then spare = make_array(5, value = -3L) $
        else spare = long(spare)
;     if spare doesn't have enough values, then add some.
      if n_elements(spare) lt 5 then spare = [spare, make_array(5, value= -3L)]
      if n_elements(clabel) eq 0 then clabel = ''
      if n_elements(tlabel) eq 0 then tlabel = ''
;    tlabel is assumed to be the same length as the vertex labels
      str2byt, tlabel, st, lvstr
      temp = strlen(clabel) >8L
      lcstr = (temp/4L +1)*4L  < max_clabel
      str2byt, clabel, sc, lcstr
      nv = nvert
      if n_elements (tapp) eq 0 then tapp = 20L + nattr else tapp = long(tapp)
      ier = 0L

      if (n_elements(full_precision) gt 0) then begin
        pflag = keyword_set(full_precision[0])
        ier = 0
        if fid ne 0 then setpff, fid, ier
        if ier ne 0 then begin
          print, '% WRIVTX: Invalid File ID, NO data written'
          return
        endif
        pff_precision, 0, full = pflag, prec = prec
        if (prec[1] eq pflag) then lcprecision = 1 else lcprecision = 0
      endif else lcprecision =0

      if initpff eq 1 then begin
; HP
        junk=call_external(PFF_EXE, 'pff_slwvtx', pid, tapp, st, sc, $
          ndim, nattr, nvert, spare, vlabel, alabel, nv, vert, attr, $
          ier, lvstr, lastr, lcstr)
      endif else if initpff eq 2 then begin
; Sun
        junk=call_external(PFF_EXE, 'pff_slwvtx_', pid, tapp, st, sc, $
          ndim, nattr, nvert, spare, vlabel, alabel, nv, vert, attr, $
          ier, lvstr, lastr, lcstr)
      endif else if initpff eq 4 then begin
; WIN
        junk=call_external(PFF_EXE, '_PFF_SLWVTX@8', pid, tapp, st, sc, $
          ndim, nattr, nvert, spare, vlabel, alabel, nv, vert, attr, $
          ier, lvstr, lastr, lcstr)
      endif
;
      if ier ne 0 then print, '% WRIVTX - Error writing vertex dataset'
      if lcprecision gt 0 then pff_precision, full = prec[1]

      return
      end
