; ------------------------------------------------------------------------ 
; $Id: activate.pro,v 1.3 2007/06/06 20:03:44 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
      Function activate, wdv, wdi, wdout, reaction = reaction, charge = charge, $
         print = printit
;+
;   
;	ACTIVATE
;
; PURPOSE:
;	Calculate activation yield from voltage and current waveforms
;       Current and voltage will be put on a uniform time base using
;          linear interpolation
;          (Hooks are there to use xy type data)
;	Cross sections have been provided by Gary Cooper.
;
; CALLING SEQUENCE:
;
;	Response = ACTIVATE (WDV, WDI [, WDOUT] [, REACTION = reaction] 
;                     [, CHARGE = charge] [, PRINT = print] )
;
; INPUTS:
;	WDV:	Waveform data array number for the voltage, units = Volts
;	WDI:	Waveform data array number for the current, units = Ampere
;               Negative values of current are set to zero.
;	WDOUT:	Waveform data array number for the response
;               Default is (Maxwdfarray-1)
;
; KEYWORD PARAMETERS:
;    CHARGE:    Ion charge. Default = 1
;               Voltage will be multiplied by this number and current 
;               divided by this number to obtain particle energy and current.
;    REACTION:	Desired reaction indicated by a number given by the following:
;               1. Li on LiF -> 24Na
;               2. H  on LiF ->  7Be
;               3. H  on  BN ->  7Be
;               4. C  on  BN -> 24Na
;               5. F  on  BN -> 28Mg
;               If not specified a widget will be provided for the selection.
;    PRINT:     Print the yield curve values 
;               Note: These values are also store in Maxwdfarray and may be
;                     plotted with: PLO, get_maxwdf()
;
; OUTPUTS:
;	The number of joules/Bq for the given current, voltage and reaction
;       is returned.  The response as a function of time and energy are in
;       WDOUT (or Maxwdfarray-1) and Maxwdfarray.  
;       Note: If all ions are below threshold then zero is returned.
;        
; PROCEDURE:
;	Limit voltage to >5% of maximum
;       Adjust current and voltage for charge if not 1
;       Calculate current yield for these voltages.
;       Integrate yield and energy to obtain response (joules/Bq)
;
; EXAMPLE:
;	Assume counts= [28, 20, 30] is the activation of 3 shakers.
;       Calculate the energy/cm2 for each if the area of each is .02 cm2
;       Assume voltage in array 2 and current in array 3 and the ions have
;         a charge of 1 and the reaction is protons on LiF.
;		ener = activate(2, 3, r=2)/.02*counts
;               print, ' Shaker   Counts    Energy/cm2'
;               for i = 0, 2 do print, i, counts(i), ener(i)
;       
;       Same as the last example but assume the protons go through a 20 micron
;          aluminum filter(z=13) before hitting the LiF.
;      vout = -9  ; set the unknown quantity as a variable
;      stopping, /quiet, 1, getyf(2)*1e-6, vout, 20 , 13
;      ;Note: the 1 is protons, voltage is scaled to MeV, 20 & 13 are the foil
;      puty, vout*1e6, 2   ; return the attenuated voltage to wdf array 2
;      ; the rest is the same as before:
;		ener = activate(2, 3, r=2)/.02*counts
;               print, ' Shaker   Counts    Energy/cm2'
;               for i = 0, 2 do print, i, counts(i), ener(i)
;
; MODIFICATION HISTORY:
; 	Written by:	L. P. Mix, May 5, 1996
;-
;
@com.dat
;
      on_error, 1

      loc =[0, 9, 22, 36, 43, 47]
      ener =[5., 6.5, 8.0, 11.0, 14.0, 17.0, 20.0, 24.0, 30.0, $
             1.9, 2.0, 3.0, 4.0, 5.0, 6.0, 7.0, 8.0, 9.0, 10.0, 12.0, $
             14.0, 16.0, 1.26, 1.5, 2.0, 3.0, 4.0, 5.0, 6.0, 7.0, 8.0, $
             9.0, 10.0, 12.0, 14.0, 16.0, 12.5, 17.5, 20.0, 22.5, 25.0, $
             30.0, 35.0, 15.0, 25.0, 35.0, 45.0]
      activ = [1.935e-13, 4.94e-12, 1.653e-11, 9.66e-11, 1.9e-10, 2.93e-10, $
             3.15e-10, 4.416e-10, 5.14e-10, 1.0e-15, 2.9e-13, 1.17e-11, $
             2.5e-11, 4.57e-11, 6.15e-11, 7.22e-11, 8.9e-11, 9.4e-11, $
             1.07e-10, 1.18e-10, 1.36e-10, 1.38e-10, 1e-15, 2.57e-13, $
             9.72e-13, 2.13e-12, 3.14e-12, 5.58e-12, 7.3e-12, 9.43e-12, $
             1.02e-11, 1.14e-11, 1.19e-11, 1.28e-11, 1.72e-11, 2.8e-11, $
             6.55e-16, 4.27e-13, 1.74e-12, 4.78e-12, 9.22e-12, 1.8e-11, $
             3.4e-11, 1.6e-14, 1.4e-13, 5.8e-13, 9e-13]
      react = ['Li on LiF -> 24Na', 'H  on LiF ->  7Be', 'H  on  BN ->  7Be', $
               'C  on  BN -> 24Na', 'F  on  BN -> 28Mg']
      nreact = n_elements(react)
      if n_elements (reaction) eq 0 then reaction = 0
      if reaction lt 1 or reaction gt nreact then begin
        get_reaction, react, reaction 
        if reaction lt 1 or reaction gt nreact then return, 0.0
      endif
      i = reaction
      en = ener(loc(i-1):loc(i)-1)
      ac = activ(loc(i-1):loc(i)-1)
      clabel = react(i-1)
      nen = loc(i)- loc(i-1)
      if keyword_set (printit) then begin
        print, '   ', react(i-1)
        print, '      Energy   Yield(Bq/ion)'
        for j = 0, nen-1 do print, en(j), ac(j)
      endif
;
      if n_elements(wdv) eq 0 or  n_elements(wdi) eq 0 then begin
        print, 'Parameters to ACTIVATE must be defined'
        return, 0.0
      endif
      if wdv lt 1 or wdv gt maxwdfarray then begin
        print, 'Voltage wdf array must be greater than 0 and </=', maxwdfarray
        return, 0.0
      endif
      if wdi lt 1 or wdi gt maxwdfarray then begin
        print, 'Current wdf array must be greater than 0 and </=', maxwdfarray
        return, 0.0
      endif
;
      if n_elements (wdout) eq 0 then wdout = maxwdfarray-1
      if wdout lt 1 or wdout gt maxwdfarray then begin
        print, 'WDOUT set to ', maxwdfarray-1
        wdout = maxwdfarray-1
      endif
      nel = wdfnp(wdv-1)      
      nelb = wdfnp(wdi-1)      
      if nel lt 2 then begin
        print, 'No valid data in Voltage waveform'
        return, 0.0
      endif
      if nelb lt 2 then begin
        print, 'No valid data in Current waveform'
        return, 0.0
      endif
      get_wdfarray, array1, wdv
      get_wdfarray, array2, wdi
; set the number of elements in case this was xydata
      nel = n_elements(array1)-2
      nelb = n_elements(array2)-2
;     get_wdfarray, array1, wdv, /xydata
;     get_wdfarray, array2, wdi, /xydata
      max1 = wdfmaxx(wdv-1)
      max2 = wdfmaxx(wdi-1)
      get_common_time, array1, array2, max1, max2, nel, nelb, xydata
      nel = n_elements(array1)-2
      if n_elements(array1) lt 4 then begin
        print, 'No values will be changed due to lack of overlap'
        return, 0.0
      endif
      if n_elements(charge) eq 0 then charge = 1 else charge = abs(charge)
      if xydata eq 0 then begin
        maxv = max(array1(2:*) , imax)
        if imax gt 0 then begin
          ok = where (array1(2:2+imax) lt 0.05*maxv, nok)
          if nok eq 0 then st = 0L else st = ok(nok-1)
        endif else st= 0L
        if imax lt nel-1 then begin
          ok = where (array1(2+imax:*) lt 0.05*maxv, nok)
          if nok eq 0 then sto = nel-1 else sto = ok(0)+imax
        endif else sto= nel-1
        array1(0)= array1(0)+st*array1(1)
        np = sto-st+1
; get x in MV and current in Particles
        x = array1(2+st :2+sto) * (charge*1e-6)
        y = array2(2+st :2+sto)* (array1(1)/(1.602177e-19 *charge)) > 0.0
        energy = total(x*y)*1.602177e-13
        print, 'energy = ', energy, ' Joules'
        If energy le 0.0 then return, 0.0
        xsort = sort(x)
        res = lindgen(np)
        res = res(xsort)
        x = x(xsort)
        ok = where (x ge en(0), nok)
        if nok gt 0 then begin
          if ok(0) gt 0 then x(0:ok(0)-1) =0.0
          j= ok(0)
          maxen = en(nen-1)
          i = 1
startiter:
          if x(j) lt maxen then begin
            while (x(j) gt en(i)) do i = i+1
            x(j) = (x(j)-en(i))/(en(i-1)-en(i))* $
                 (ac(i-1)-ac(i)) + ac(i)
            j = j+1
            if j lt np then goto, startiter
          endif
          if j le np-1 then x(j:np-1) = ac(nen-1)
        endif else return, 0.0
        xsort = sort(res)
        x = x(xsort)
        y = x*y
        xx=findgen(np)*array1(1) + array1(0) 
        i2w, xx, y, wdout, c = clabel +' Yield vs Time', xlab = 'Time (s)', $
            ylab= 'Yield(Bq)'
        counts = total(y)
        if counts gt 0 then resp = energy/counts else resp = 0.0
      endif else begin
         Print, 'Major screw up --- We should not have xy data'
         return, 0.0
      endelse
      i2w, en,ac, maxwdfarray, clab = clabel, xlabel='Energy (MeV)', $
        ylabel='Yield(Bq/ion)', /xydata
      return, resp
      end
