; ------------------------------------------------------------------------ 
; $Id: ave.pro,v 1.3 2007/06/06 20:03:44 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
 function  ave, wdf1, left = start, right = stopt, sd =sd, quiet = quiet, $
            arithmetic_mean = ar_mean
;+
;  Function which returns the average value of a waveform data array.
;  Optional parameters allow the specification of a limited portion
;  of the array.
;  NOTE: To obtain the time dependent average of several waveforms, 
;        see SUM
;  format: AVERAGE = AVE(wd1 [, SD = sd] [, LEFT  = left] [, RIGHT = right]
;                      [, /ARITHMETIC_MEAN ] [, /QUIET ])
;  where:
;    AVERAGE   - average value of the WDF array
;    wd1 - WDF array number
;    sd  - Standard deviation = sqrt( total(y-ave)^2/ (n-1))
;    left, right - optional abscissa value to specify the left and right
;                  abscissa values used to determine the local maximum
;                  if absent, the default values are the first and last data
;                  points, respectively
;    Arithmetic_mean - Ignored unless X-Y Data.
;                      If set, ave =(Total (Y)/ n_elements(y))
;              Default is Arithmetic_mean not set
;              Procedure with not set is:
;                      select valid X values.
;                      sort x, y by x 
;                      integrate (y dx)
;                      ave = int(y dx)/(max(x)-min(x))
;    quiet - if present and nonzero, no values will be printed to the terminal
;  Note:   The IDL function TOTAL is used to calculate the average
;  Examples:
;    Print the average value of array 2 between 10 and 1000
;      Print, AVE(2, l = 10, r = 1000)
;
;    Eliminate a baseline offset in waveform 2 by subtracting the average
;    value of the baseline up to the time the signal starts at 35ns
;      sub, 2, 0, ave(2, r= 35e-9)
;
;    Calculate the average value of array 3 and store it in average;
;    suppress terminal output
;      AVERAGE =  AVE(2, /quiet)
;
;-
@com.dat
;
      on_error, 1
      value = 0.0
;
      if n_elements(initwdf) eq 0 then wdfinit
      nparams = n_params()
      if nparams eq 0 then begin
        print, '% Format: AVE_value = AVE( WDF [, SD = Std_Dev] [, LEFT = left]'
        print, '              [, RIGHT = right] [, QUIET = quiet]' 
        return, value
      endif

      if nparams lt 1 then begin
        print, '% AVE: Function must have 1 parameter'
        return, value
      endif
;
      if n_elements(wdf1) eq 0 then begin
        print, '% AVE: Parameter to AVE must be defined'
        return, value
      endif
      if wdf1 lt 1 or wdf1 gt maxwdfarray then begin
        print, '% AVE: WDF array must be greater than 0 and </=', maxwdfarray
        return, value
      endif
;
      np = wdfnp(wdf1-1)
      if np eq 0 then begin
        print, '% AVE: No data in the array'
        return, value
      endif
;
      get_wdfarray, array, wdf1, /xydata
      if size(array, /type) lt 4 then array = float(array)
      nel = n_elements (array)
    if (nel-2) eq np then begin
      if n_elements (start) eq 0 then ist = 0L else $
         ist = long((start-array(0))/array(1)) > 0L
      if n_elements (stopt) eq 0 then ilst = np-1 else $
         ilst = long((stopt-array(0))/array(1) +1.00001) < np-1
      if ist ge ilst then begin
         print, '% AVE: START and STOP are not compatable:', ist, ilst
         print, ' Maximum for the entire WDF array', wdf1,' is returned'
         ist = 0
         ilst = np-1
      endif
      value = total(array(ist+2L:ilst+2L))/(ilst+1-ist)
      sd = total((array(ist+2L:ilst+2L)-value)^2)/(ilst-ist)
      sd = sqrt(sd)
      if not keyword_set(quiet) then begin
        print, ' Average value of array', wdf1, ' is' , value
        print, '   on the interval from', array(0) + ist*array(1), ' to', $
         array(0)+array(1)*(ilst < (np-1))
        print, ' Standard Deviation is ', sd
      endif
      array=0
      return, value
    endif
;
;     xydata
; 
      if np+np ne nel then begin
        print, '% AVE: Problem with x-y data array', np, nel
        return, value
      endif
      x = array(0:np-1)
      y = array(np:nel-1)
      minx= min(x, max = maxx)
      if n_elements (start) eq 0 then ist = minx else ist = start
      if n_elements (stopt) eq 0 then ilst = maxx else ilst = stopt
      if ist ge ilst then begin
         print, ' START and STOP are not compatable'
         print, ' Average for the entire WDF array', wdf1,' is returned'
         ist = minx
         ilst = maxx
      endif

      ok = where (x ge ist and x le ilst, nok)
      if nok eq 0 then begin
        print, '% AVE: No data in the filtered array'
        return, value
      endif
      x = x(ok)
      y = y(ok)
      if keyword_set(ar_mean) or nok le 2 then begin
        value = total(y)/nok
        sd = total ((y-value)^2)/((nok-1) > 1)
        sd = sqrt(sd)
        if not keyword_set(quiet) then begin
          print, ' Arithmetic mean of array', wdf1, ' is' , value
          print, '   on the interval from', ist, ' to', ilst 
          print, ' Standard Deviation is ', sd
        endif
      endif else begin
        a = sort(x)
        x = x[a]
        y = y[a]
        range= x[nok-1] -x[0]
        del = (x[1:nok-1] - x[0:nok-2])*0.5
        del = [del[0], del[0: nok-3]+del[1:nok-2], del[nok-2]]
        value = total(del*y)/range
        sd = total ((y-value)^2)/((nok-1) > 1)
        sd = sqrt(sd)
        if not keyword_set(quiet) then begin
          print, ' Average value of array', wdf1, ' is' , value
          print, '   on the interval from', ist, ' to', ilst
          print, ' Standard Deviation is ', sd
        endif

      endelse
      array=0
      return, value
      end
