; ------------------------------------------------------------------------ 
; $Id: blackbody.pro,v 1.5 2007/06/06 20:03:44 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
pro blackbody, wdf, temp, energy, totalout = totalout, kelvin = kelvin, $
      wavelength = wavelength, highdensity = highdensity, units = units, $
      minimum = minimum, maximum = maximum, power = power, no_save = nosave
;+
; NAME:
;       BLACKBODY
;
; PURPOSE:
;       Generate a blackbody spectrum of Watts/m2/d(unit) vs unit
;       where unit is typically temperature in eV.
;
; CATEGORY:
;       Analysis
;
; CALLING SEQUENCE:
;   BLACKBODY, WDF, TEMP, ENERGY, TOTALOUT = TOTAL, POWER = power,
;                   /WaveLength, /HighDensity , NO_SAVE= nosave
;                   /Kelvin
;                   MINIMUM = minimum, MAXIMUM = maximum
;
;      Where:
;       WDF   - Waveform array for data storage of the blackbody spectrum.
;               Spectrum will be Watts/m^2/eV
;               Default is first unused wdf array or get_maxwdf().
;            Value ( Watts/m^2/eV ) :
;               Spectrum=1.583476669924e+08*Energy^3/(exp(Energy/Temp)-1.d0)
;            Value ( Watts/m^2/m ) :
;               Spectrum=1.277159597666d+14*((Energy^5)/(exp(Energy/Temp)-1.d0)
;       Temp  - Temperature of the black body emitting surface
;               Units are (eV) unless KELVIN is set
;       Energy- Energy values (eV) for the WDF array or wavelength values if
;               wavelength is set.  
;               Energy is ignored if it has less than 3 elements
;               Minimum value = 0.025
;               Default: Energy = t/40 + 15*Temp*findgen(512)/511
;               If HighDensity is set:
;               Default: Energy = t/100+ 25*Temp*findgen(12288)/12287
;             if wavelength is set
;               Wavelength = 1.2398424385d-06/energy
;                 where energy is defined above
;       TotalOUT - Integrated value of the spectrum over all wavelengths.
;               Total = sigma*Temp^4 (W/m2)
;               where sigma = 1.02830e+09 with Temp in eV.
;       POWER - Power calculated for the energy /wavelength
;               Units are Watts/m^2/(m or eV)
;       KELVIN- If set, TEMP is in degrees Kelvin
;               (11604.448 degrees Kelvin = 1 ev)
;       No_SAVE- Do not save in a wdf array
;       WaveLength - If set, WDF will be the spectrum vs photon wavelength
;                    Spectrum will be Watts/m^2/meter with wavelenght in 
;                    meters unless units is set
;                    WaveLength = 1.2398424385d-06/(Energy > 0.025) (meters)
;           Spectrum=3.741774922d-26/Wavelength^5/(exp(Energy/Temp)-1.d0)
;       UNITS - If set, units are Watts/cm^2/Ang/sr or  Watts/cm^2/eV/sr
;               Depending on whether WAVELENGTH is set
;               We assume that emission into "pi" Sr
;       HighDensity - If set a dense set of default energy values are used.
;                     See Energy above.
;       Minimum - Minimum energy/wavelength (See Energy for defaults.)
;       Maximum - Maximum energy/wavelength (See Energy for defaults.)
; OUTPUTS:
;       WDF array with power spectrum and the total power in keyword TOTAL.
;
; EXAMPLE:
;       Store the black body spectrum for 100 eV source in WDF array 5
;         BLACKBODY, 5, 100 
;
; MODIFICATION HISTORY:
;       Written by:     L. P. Mix, 4/30/01
;-

on_error, 1
;
;     add common block and initialize WDF arrays
;
@com.dat
 if n_elements(initwdf) eq 0 then wdfinit

 nparam = n_params()
 if nparam eq 0 then begin
   print, '% Format: BLACKBODY, WDF, TEMP, ENERGY, TOTAL = total, '
   print, '            /WaveLength, /HighDensity'
   return
 endif

 if nparam lt 2 then begin
   print, '% BLACKBODY: Two parameters are required'
   return
 endif 
 save = get_save_array(wdf, get_maxwdf)
 if n_elements (temp) eq 0 then begin
   print, '% BLACKBODY: Emission temperature must be defined'
   return
 endif else t= abs(temp) > 0.025
 if keyword_set(kelvin) then t = t/11604.448d0
 totalout =  1.028300544d+09*t^4
 if keyword_set (wavelength) then begin
   if n_elements (energy) lt 3 then begin
     if keyword_set (highdensity) then begin
       n = 12288L
       if n_elements(minimum) eq 0 then minimum = 4.959e-08/t
       if n_elements(maximum) eq 0 then maximum = 1.240e-04/t
     endif else begin
       n = 512
       if n_elements(minimum) eq 0 then minimum = 8.2656e-08/t
       if n_elements(maximum) eq 0 then maximum = 4.9594e-05/t
     endelse
     minimum = min([minimum, maximum], max= maximum)
      m1 = 1.2398424385d-06/minimum
      m2= 1.2398424385d-06/maximum

      energ= m1 + (m2- m1)/float(n-1)*findgen(n)
      ener = 1.2398424385d-06/energ
     
   endif else begin
     ener = abs(energy) < 4.959e-05
     energ = 1.2398424385d-06/ener
   endelse
   efrac = energ/t
   ok =  where (efrac gt 0.9* alog((machar()).xmax), nok)
   ok2 = where (efrac le 1.1* alog((machar()).eps), nok2)
   if nok gt 0 then efrac[ok]=1.0
   if nok2 gt 0 then efrac[ok2]=1.0
   power = (exp(Efrac)-1.d0)
;  if units are set then switch to W/cm2/A/sr from W/m2/m
;  divide by 1e4 for area
;  divide by 1e10 for A
;  divide by !pi for Sr
; 
   if keyword_set(units) then power = 4.065325261716d-01 * ((energ^5)/power) $
     else power = 1.277159597666d+14 * ((energ^5)/power)
   if nok gt 0 then power[ok]=0.0
   if nok2 gt 0 then power[ok2]=0.0
   ylabel = 'Power (W/m2/m)'
   if keyword_set (units) then begin
     xlabel = 'Wavelength (A)'
     ener = ener*1e10
     clabel = strtrim (string(t, format='(f6.1)'), 1)+ $
           ' eV Black-Body vs Wavelength(A)'
     ylabel = 'Power (W/cm2/A/Sr)'
   endif else begin
     clabel = strtrim (string(t, format='(f6.1)'), 1)+ $
           ' eV Black-Body vs Wavelength'
     xlabel = 'Wavelength (m)'
   endelse
 endif else begin
 if n_elements (energy) lt 3 then begin
   if keyword_set (highdensity) then begin 
     n = 12288L
     if n_elements(minimum) eq 0 then minimum = t/100.0
     if n_elements(maximum) eq 0 then maximum = 25*t 
   endif else begin
     n = 512
     if n_elements(minimum) eq 0 then minimum = t/40.0
     if n_elements(maximum) eq 0 then maximum = 15*t 
   endelse
   minimum = min([minimum, maximum], max= maximum)
   ener = minimum + (maximum- minimum)/float(n-1)*findgen(n)  
 endif else ener = abs(energy) > 0.025
 
; maxexpval= alog((machar()).xmax)
; minexpval= alog((machar()).eps)
 efrac = ener/t
 ok =  where (efrac gt 0.9* alog((machar()).xmax), nok)
 ok2 = where (efrac le 1.1* alog((machar()).eps), nok2)
 if nok gt 0 then efrac[ok]=1.0
 if nok2 gt 0 then efrac[ok2]=1.0
;  if units are set then switch to W/cm2/eV/sr from W/m2/eV
;  divide by 1e4 for area
;  divide by !pi for Sr

 power = (exp(Efrac)-1.d0)
 if keyword_set(units) then power =  5040.362785781962884d0*((ener^3)/power) $
     else  power = 1.583476669924d+08*((ener^3)/power)

;  power = 1.583476669924d+08*((ener^3)/power)
 if nok gt 0 then power[ok]=0.0
 if nok2 gt 0 then power[ok2]=0.0
 xlabel = 'Energy (eV)'
 if keyword_set(units) then $
   ylabel = 'Power(W/cm2/eV/Sr)' else $
   ylabel = 'Power(W/m2/eV)' 
 clabel = strtrim (string(t, format='(f6.1)'), 1)+ $
           ' eV Black-Body vs Energy'
 endelse

 if keyword_set(nosave) then return
 i2w, ener, power, save, /xydata, xlabel = xlabel, ylabel = ylabel, $
      clabel = clabel
END
