; ------------------------------------------------------------------------ 
; $Id: blin.pro,v 1.9 2010/09/17 21:38:16 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
;+
; NAME:
;       BLIN 
;
; PURPOSE:
;       Subtract a constant value from a waveform
;       Baseline can be set by clicking left mouse button, by entering
;         a value, or by averaging a specified portion of the waveform.
;       Once a baseline is set it will moved only if a different
;         value is selected.  Until it is set the baseline will follow the 
;         cursor.
;       Baseline is accepted with the middle mouse button. Program exits.
;       Selected baseline is reset with the right mouse button in cursor mode.
;       Left and right buttons specify the initial and final portions of the 
;         baseline for the average process.
;       Exit/Quit  button will exit with no baseline.
;       
;
; CALLING SEQUENCE:
;	BLIN, WDF1 [, WDF2] [, OFFSET = offset] [, XRANGE = XRANGE] [,/NO_SAVE]
;       [, SAVE = save] [, XSIZE = xsize] [, YSIZE = ysize] [, /AVERAGE]
;       [, SD = sd] [, RANGE = range] [, _EXTRA = extrastuff]
;
;   where:
;       WDF1  - WDF array number or an array name.
;       WDF2  - WDF array number to store the result.
;               Default WDF1 = WDF2
;
; KEYWORD PARAMETERS:
;	OFFSET: Value of the baseline selected. Default = 0.0
;               If quit is selected, OFFSET = 0.0
;               If average is set:
;               offset = ave(wdf1, left = range[0], right = range[1], sd=sd)
;	Xrange: Keyword passed to the PLO command.
;       SAVE:   Is 0 if NO_SAVE is set or if QUIT is hit
;               Is 1, otherwise
;       NO_SAVE:If set, only value will be returned.
;       XSIZE:  Size in pixels for the draw widget
;       YSIZE:  Size in pixels for the draw widget
;               Default values set in set_drawsize.
;       AVERAGE: If set, the baseline will be determine from the average of the
;                waveform over a specified interval.
;                If not set, the baseline is set visually with a cursor.
;                Default is AVERAGE = 1 for the first time.
;                Subsequent calls will use the value from the previous call.
;       SD    : If average is set (the default), then this is the standard 
;               deviation of the average.  SD=0 for cursor method.
;       RANGE : If average is set (the default), this vector contains the 
;               left and right range for calculating the average baseline.
;               If this is input with two values, these will be used for the initial
;               values of the average range.
;       _EXTRA: Any other valid keywords to the plo command.
;
; EXAMPLE:
;	Select the baseline on waveform 4 and subtract it.
;         BLIN, 4 
;
;       Select the baseline on waveform 5 and the result in waveform 15
;       Plot 0nly form -100 to 50 ns.
;         BLIN, 5, 15, xr = [-100, 50]*1e-9
;
; MODIFICATION HISTORY:
; 	Written by:	L. P. Mix, April 1, 1998
;       Added average option, Feb. 19, 2002
;       Added keywords: range, sd, July 26, 2002
;
;-


PRO baseline_event, Event

 COMPILE_OPT hidden

  common baseline_common, ysave_a, ysave_c, value_a, value_c, method_save

on_error , 1

  WIDGET_CONTROL,Event.Id,GET_UVALUE=Ev
  widget_control, event.top, get_uvalue = info, /no_copy

    CASE Ev OF
  'METHOD': BEGIN
      if event.value eq 0 then begin
;       Cursor Method
        if info.method eq 1 then begin
          widget_control, info.ywind, get_value = ysave_a 
          widget_control, info.ywind, set_value = ysave_c
          widget_control, info.basers, map = 0
          widget_control, info.label, set_value = value_c
          widget_control, info.base7, map =0 
          device , copy = info.image
          widget_control, info.draw, /clear_events, draw_motion_events = 1
          info.mode = 1
        endif 
        info.method = 0
        widget_control, event.top, set_uvalue = info, /no_copy
        return
      endif else begin
        if info.method eq 0 then begin
          widget_control, info.ywind, get_value = ysave_c 
          widget_control, info.ywind, set_value = ysave_a
          widget_control, info.basers, map = 1
          widget_control, info.base7, map =1 
          widget_control, info.label, set_value = value_a
          widget_control, info.draw, /clear_events, draw_motion_events = 0
        endif 
        info.method = 1
      endelse
      baseline_doit, info, event.top
      END

  'FSLID3': BEGIN
      if event.drag eq 0 then begin
        info.value[0] = event.value < $
                      (info.value[1] - (info.tolerance)*3)
        y = ave(info.wdf, left= info.value[0], $
             right= info.value[1], sd = sd, /quiet)
        (*info.ptr).sd = sd 
        widget_control, info.ywind, set_value = y
        widget_control, info.fslid3, set_value =  $
          [info.value[0], info.width[0], info.value[1]]
        widget_control, info.fslid4, set_value =  $
          [info.value[1], info.value[0], info.width[1]]
        
      endif
      baseline_doit, info, event.top
      END

  'FSLID4': BEGIN
      if event.drag eq 0 then begin
        info.value[1] = event.value > $
                  (2*info.tolerance + info.value[0])
        y = ave(info.wdf, left= info.value[0], right= info.value[1], $
          sd = sd, /quiet)
        (*info.ptr).sd = sd
        widget_control, info.ywind, set_value = y
        widget_control, info.fslid3, set_value =  $
          [info.value[0], info.width[0], info.value[1]]
        widget_control, info.fslid4, set_value =  $
          [info.value[1], info.value[0], info.width[1]]
      endif
      baseline_doit, info, event.top
      END
   ELSE: BEGIN
      Print, 'No Handler for event: ', ev
      print, '% BASELINE_EVENT: EVENT UVALUE = ', Ev 
      help, /str, event
      widget_control, event.top, set_uvalue = info, /no_copy
      END
   ENDCASE
  END


 Pro baseline_doit, info, top
 COMPILE_OPT hidden

; 
 on_error, 1
  common plot_grid_last, yeng_scale, xeng_scale

  widget_control, info.ywind, get_value = py
; be sure values are floats
  device, copy = info.image
  if info.method eq 1 then begin
    oplot, info.value[0]*[1,1]*xeng_scale, !y.crange, psym = -8, $
       thick = 1, col =6
    oplot, info.value[1]*[1,1]*xeng_scale, !y.crange, psym = -8, $
       thick = 1, col =6
  endif
  oplot, !x.crange, [py, py]*yeng_scale, psym = -8, thick = 1, col =7 
  widget_control, top, set_uvalue = info, /no_copy
 end

 pro baseline_draw_event, event
 COMPILE_OPT hidden
  common plot_grid_last, yeng_scale, xeng_scale
;
  on_error, 1

  WIDGET_CONTROL,Event.Id,GET_UVALUE=Ev
  CASE EV OF
;   Event in the draw widget
   'DRAW':  Begin
            widget_control, event.top, get_uvalue = info, /no_copy
;           event.press = 4 = right button = reset
            if event.press eq 4 then begin
;             event.press = 4 = right button = reset if method = 0
              if info.method eq 0 then baseline_reset_button, info, event.top $
              else begin
                coord = convert_coord(event.x, event.y, /device, /to_data)
;               right mouse for method = 1
                info.value[1] = (coord[0]/xeng_scale) > $
                  (3*info.tolerance + info.value[0])
                y = ave(info.wdf, left= info.value[0], right= info.value[1], $
                    sd=sd, /quiet)
                (*info.ptr).sd = sd
                widget_control, info.ywind, set_value = y
                widget_control, info.fslid3, set_value =  $
                  [info.value[0], info.width[0], info.value[1]]
                widget_control, info.fslid4, set_value =  $
                  [info.value[1], info.value[0], info.width[1]]
                baseline_doit, info, event.top
                return
              endelse
            endif else begin
              coord = convert_coord(event.x, event.y, /device, /to_data)
;      SAVE      event.press =2 is middle mouse button
              if event.press eq 2 then begin
                if info.mode eq 1 and info.method eq 0 then begin
                    winformation, 'One point must be selected for a baseline'
                    widget_control, event.top, set_uvalue = info, /no_copy
                    return
                endif else begin
                    widget_control, info.ywind, get_value = offset
                    (*info.ptr).offset = offset 
                    (*info.ptr).quit = 0
                    (*info.ptr).method = info.method
                    (*info.ptr).range = info.value
                    widget_control, event.top, /destroy
                endelse
              endif else begin
                if event.press eq 1 then begin
                  if info.method eq 0 then begin
                    info.mode = 0
                    widget_control, info.ywind, set_value = coord(1)/yeng_scale
                    widget_control, info.base7, map =1 
                    widget_control, info.draw,/clear_events,draw_motion_events=0
                    baseline_doit, info, event.top
                  endif else begin
;                   left mouse for method = 1
                    info.value[0] = (coord[0]/xeng_scale) < $
                      (info.value[1] - (info.tolerance)*3)
                    y = ave(info.wdf, left= info.value[0], $
                         right= info.value[1], sd = sd, /quiet)
                    (*info.ptr).sd = sd 
                    widget_control, info.ywind, set_value = y
                    widget_control, info.fslid3, set_value =  $
                      [info.value[0], info.width[0], info.value[1]]
                    widget_control, info.fslid4, set_value =  $
                      [info.value[1], info.value[0], info.width[1]]
                    baseline_doit, info, event.top
                    return
                  endelse
                endif else if event.release eq 0 then begin
                  if info.method eq 1 then begin
                    widget_control, event.top,  set_uvalue = info, /no_copy
                    return
                  endif
                  widget_control, info.ywind, set_value = coord(1)/yeng_scale
                  baseline_doit, info, event.top
                  return
                endif else begin
                  widget_control, info.draw, /clear_events
                  widget_control, event.top, set_uvalue = info, /no_copy
                endelse
              endelse
            endelse
            END
  'YVALUE': BASELINE_HANDLE_1, event
  ELSE:   Begin
          print, ev
          help, /str, event
          END
  END
  END

 Pro baseline_handle_1, event
 COMPILE_OPT hidden
   on_error, 1
  widget_control, event.top, get_uvalue = info, /no_copy
  info.mode = 0
  widget_control, info.draw, /clear_events, draw_motion_events = 0
  widget_control, info.base7, map = 1
  baseline_doit, info, event.top
  end


Pro bsave_button_event, event
  widget_control, event.top, get_uvalue = info
  widget_control, info.ywind, get_value = y
  (*info.ptr).offset = y
  (*info.ptr).method = info.method
  (*info.ptr).quit = 0
  (*info.ptr).range = info.value
  widget_control, event.top, /destroy
  END

pro baseline_reset_button, info, top
 COMPILE_OPT hidden
; Handle the actual reset
    if info.method eq 0 then begin
      info.mode = 1
      device , copy = info.image
      widget_control, info.base7, map = 0
      widget_control, info.draw, /clear_events, draw_motion_events = 1
    endif
    widget_control, top, set_uvalue = info, /no_copy
  END

pro breset_button_event, event
 COMPILE_OPT hidden
;
; Handle the reset and the exit buttons
  on_error, 1
  widget_control, event.top, get_uvalue = info, /no_copy
  if event.value eq 1 then begin
    (*info.ptr).quit = 1
    (*info.ptr).offset = 0.0
    widget_control, event.top, /destroy
  endif else begin
    baseline_reset_button, info, event.top
  endelse
  end

pro blin,  wdfinp, wdfb, group = group, modal = modal, xsize= xsizein, $
           save = lcsave,  ysize= ysizein, offset = offset, sd = sd,  $
           average = average, no_save = no_save, range= range, $
           _extra =extrastuff

;
;       return on an error
      on_error, 1
@com.dat
;
@draw_defaults.com
;
  common plot_grid_last, yeng_scale, xeng_scale
  common pffcom, initpff, pff_exe
  common baseline_common, ysave_a, ysave_c, value_a, value_c, method_save
;
      if n_elements(initwdf) eq 0 then wdfinit
;
      if n_elements (initpff) eq 0 then init_pff else $
        if initpff eq 0 then init_pff
;
      if n_elements(method_save) eq 0 then method_save = 1
      nparams = n_params()
      if nparams eq 0 then begin
        print,'% Format: ', $
       'BLIN, WDFin [, WDFout]  [, OFFSET = offset] [, _EXTRA = extrastuff]'
        return
      endif
      if n_elements(wdfinp) gt 0 then  wdf = wdfinp else begin
        print, '% BLIN: WDF array(s) must be specified'
        return
      endelse
      a = size(wdf)
      if a(a(0)+1) eq 7 then begin
        if a(0) ne 0 then print, '% BLIN: Only the first string is used'
        if strtrim(wdf(0), 2) eq '*' then wdf = wdvalid() else $
          wdf = wdvalid(wdf(0))
        if wdf(0) lt 1 then begin
          print, '% BLIN: No arrays found using the search string: ', wdf(0)
          return
        endif
      endif
      if n_elements(wdf) gt 1 then begin
        PRINT, '% BLIN: Only a single array may be specified.'
        print, '   Values =', wdf
        return
      endif
      ysavea = 0.0
      ysavec = 0.0
      wdf = wdf(0)
      if wdf lt 1 or wdf gt maxwdfarray then begin
        print, '% BLIN: WDF array must be greater than 0 and </=', maxwdfarray
        return
      endif
      if nparams gt 1 then save = get_save_array(wdfb, wdf) else save = wdf
;
      nel = wdfnp(wdf-1)
      if nel lt 2 then begin
        print, '% BLIN: No data in the array -', wdf
        return
      endif

  if n_elements(group) eq 0 then group = 0
  tlbase = WIDGET_BASE(GROUP_LEADER=Group, $
      COLUMN=1, modal = modal, $
      MAP=1, $
      TITLE='Select BaseLine')

  BASEINS  = WIDGET_BASE(tlbase, $
      COLUMN = 1, $
      MAP=1, frame = 1, $
      UVALUE = 'BASEINS')
 value_c = 'Left Mouse = Designate Point,     Middle Mouse = Save,     Right Mouse = Reset'
 value_a = 'Left Mouse = Designate Left Point,     Middle Mouse = Save,     Right Mouse = Designate Right Point'

 if n_elements (average) eq 0 then average = method_save
 if keyword_set(average) then aver = 1 else aver = 0
 val = aver ? value_a : value_c

 labelins1 =  WIDGET_LABEL( BASEins, UVALUE='LABEL', VALUE= val)

  BASE1 = WIDGET_BASE(tlbase, $
      row=1, $
      FRAME=2, $
      MAP=1, $
      UVALUE='BASE1')

  BASE11 = WIDGET_BASE(base1, $
      column=1, $
      FRAME=2, $
      MAP=1, $
      UVALUE='BASE11')

 Names = ['Cursor', 'Average']
 method = aver

 wbutton = cw_bgroup(base11, names, /column, /exclusive, /no_release, $
           label_left='Select Baseline Method', $
           UVALUE = 'METHOD', set_value = method)


  BASE12 = WIDGET_BASE(base1, $
      column=1, $
      FRAME=2, $
      MAP=1, $
      event_pro = 'Baseline_Draw_Event', $
      UVALUE='BASE12')

  mm = mx(wdf, range = rang, /quiet)
  wid = rang[1] - rang[0]
  val = [rang[0], rang[0]+0.125*wid]
  tolerance = .005*wid
  if n_elements(range) ge 2 then begin
    tval =  ([min(range, max = m), m] < rang[1]) > rang[0]
    if (tval[1]-tval[0]) gt 0 then val = temporary(tval)
  endif
   ysave_c = 0.0
   ysave_a = ave(wdf, left = val[0], right= val[1], /quiet)
   yvalue =  aver ? ysave_a : ysave_c

   label1  =  WIDGET_LABEL( BASE12, $
      UVALUE='LABEL', $
      VALUE='Selected Baseline Value')

   ywindow = CW_FIELD( BASE12,VALUE=yvalue, $
      ROW=1, $
      FLOAT=1, $
      RETURN_EVENTS = 1, $
      TITLE='Y - Value:', $
      UVALUE='YVALUE')

  Basers = WIDGET_BASE(Base1, $
      row = 1,  $
      XPAD = 20, $
      frame = 3, $
      MAP=0)

  FSLID3 = cw_fslider( BASERS, $
      DRAG=1, /edit, $
      TITLE='Minimum Average Location', $
      UVALUE='FSLID3',Value =val[0], $
      MAXIMUM = val[1],  MINIMUM = rang[0])

  FSLID4 = cw_fslider( BASERS, $
      DRAG=1, /edit, $
      TITLE='Maximum Average Location', $
      UVALUE='FSLID4',Value = val[1], $
      MAXIMUM = rang[1],  MINIMUM = val[0])

  BASE4 = WIDGET_BASE(tlbase, $
      ROW = 1, $
      MAP=1, $
      UVALUE = 'BASE4')

  BASE7 = WIDGET_BASE(BASE4, $
      ROW = 1, $
      MAP=1, $
      FRAME = 2, $
      event_pro = 'BSAVE_BUTTON_EVENT', $
      UVALUE = 'BASE7')

  SAVEB= WIDGET_BUTTON( base7, $
      UVALUE='SAVE_BUTTON', $
      VALUE=' Accept / Save Baseline ')

  BASE8 = WIDGET_BASE(BASE4, $
      ROW = 1, $
      MAP=1, $
      FRAME = 2, $
      event_pro = 'BRESET_BUTTON_EVENT', $
      UVALUE = 'BASE8')

  Btns439 = [ $
    '   Reset  Points   ', $
    '   EXIT  /  QUIT   ' ]

  ard_buttons = CW_BGROUP( base8, Btns439, $
      ROW=1,  /NO_RELEASE, $
      UVALUE='buttons')

  BASE5 = WIDGET_BASE(tlbase, $
      ROW = 1, $
      MAP=1, $
      event_pro = 'BaseLine_Draw_Event', $
      UVALUE = 'BASE5')


  oldwin = !d.window
  if n_elements(xsizein) eq 0 then begin
    if initpff gt 3 then xsize= bigsize(0) else xsize = bigsize(0) 
  endif else xsize = xsizein
  if n_elements(ysizein) eq 0 then begin
    if initpff gt 3 then ysize = bigsize(1)*5/6 else ysize = bigsize(1) - 50
  endif else ysize = ysizein


  npselect = 0

  window, /free , /PIXMAP  , xsize = xsize, ysize = ysize
  pixwin = !d.window
  plo, wdf, _extra = extrastuff
  image = [0, 0, xsize, ysize, 0, 0, pixwin]

  DRAW = WIDGET_DRAW( base5, $
      BUTTON_EVENTS=1, $
      MOTION_EVENTS=1, $
      RETAIN=1, $
      UVALUE='DRAW', $
      XSIZE=xsize, $
      YSIZE=ysize)

  WIDGET_CONTROL, tlbase, /REALIZE
  widget_control, base7, map = 0
; 
; Get drawable window index
  WIDGET_CONTROL, DRAW, GET_VALUE=DRAW_Id
  mksymbol, 'plus', /fill, size = 1.5
  wset , draw_id
  device , copy = image
;
  if aver eq 1 then begin
    val[0] = val[0] > !x.crange[0]
    val[1] = val[1] < !x.crange[1]
    oplot, val[0]*[1,1], !y.crange* yeng_scale, psym = -8, $
       thick = 1, col =6
    oplot, val[1]*[1,1], !y.crange* yeng_scale, psym = -8, $
       thick = 1, col =6
    widget_control, basers, map = 1
    widget_control, base7, map = 1
  endif else widget_control, draw, draw_motion_events = 1
;
  sd = 0.0
  range = [0.0,0.0]
  ptr = ptr_new({quit:0L, offset:yvalue, method: aver, range: range, SD:sd})
  info = {ptr:ptr, ywind:ywindow, mode:1, base7:base7, draw:draw, $
        label: labelins1, wdf:wdf, $
        image:image, basers:basers, method: method, $
     width:rang, tolerance:tolerance, value: val, fslid3:fslid3, fslid4:fslid4}
  widget_control, tlbase, set_uvalue = info, /no_copy
  empty
  XMANAGER, 'baseline', tlbase
  quit = (*ptr).quit
  offset = (*ptr).offset
  method_save =  (*ptr).method
  if method_save eq 1 then begin
    sd = (*ptr).sd
    range = (*ptr).range
  endif
  ptr_free, ptr 
  wdelete, pixwin
  if oldwin ge 0 then wset, oldwin
  lcsave = 0
  if (quit eq 1) or keyword_set(no_save) then return
  lcsave = 1
; adjust the baseline
  if save ne wdf then xfr, wdf, save
  if offset ne 0.0 then puty, getyf(wdf)-offset, save

END
