; ------------------------------------------------------------------------ 
; $Id: conv.pro,v 1.5 2007/06/06 20:03:45 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
 Pro  conv, wdf1, wdf2, wdf3, idl= idl, sigma = sigma,range = range, $
     _extra = extrastuff
;
;+
; Name:
;        CONV
; Purpose:
;        This procedure produces the convolution of two WDF arrays
; Format:
;   CONV, WDF1, WDF2 [, WDF3] [, /IDL] [, SIGMA = sigma] [, _extra = extrastuff]
;     WDF1   - Input Waveform
;     WDF2   - Response waveform.  WDF2 must be shorter than WDF1
;              If Zero, A gaussian will be assumed and SIGMA must be provided
;     WDF3   - Storage for convolution
;              Default:  WDF3 = WDF1
; Keywords:
;     IDL    - If set the IDL procedure CONVOL is used
;              The default is to multiply the fft of the two waveforms together
;              and to take the inverse fft of the product.  The data is normalized
;              to the response, wdf2.
;     Note the following two keywords are passed to GAUSD if WDF2 = 0.
;              The normal keyword is also set.
;     SIGMA  - If WDF2 eq 0, sigma of a gaussian.
;     Range  - If WDF2 eq 0, range of the gaussian distribution.
;              Default:  Range = [-3.5, 3.5] (units are sigma)
;     _extra - may be used to pass keywords to the convol function.
;
;    Example:
;      Convolve waveform array 10 with a kernel of 2 and store the result in 11
;        conv, 10, 2, 11
;
;      Use the IDL CONVOL function to do the same procedure.
;        conv, 10, 2, 11, /idl, center = 0
;          
;-
on_error, 1
@com.dat
;
      if n_elements(initwdf) eq 0 then wdfinit
      nparams = n_params()
      if nparams lt 2 then begin
        print,'% CONV: CONV must have at least 2 parameters'
        return
      endif
;
      if n_elements(wdf1) eq 0 or  n_elements(wdf2) eq 0 then begin
        print, '% CONV: Two Parameters to CONV must be defined'
        return
      endif
      if wdf1 lt 1 or wdf1 gt maxwdfarray then begin
        print, '% CONV: Wdf array must be greater than 0 and </=', maxwdfarray
        return
      endif
;
      if nparams gt 2 then save = get_save_array(wdf3, wdf1) else save = wdf1
;
      np1 = wdfnp(wdf1-1)
      if np1 eq 0 then begin
        print, '% CONV: No data in the first array'
        return
      endif
;
      lcgaus =1
      if wdf2 eq 0 then wdf2 = maxwdfarray else begin
        if wdf2 lt 1 or wdf2 gt maxwdfarray then begin
          print, '% CONV: Wdf array must be greater than 0 and </=', maxwdfarray
          return
        endif
        np2 =  wdfnp(wdf2-1)
        if np2 eq 0 then begin
          print, '% CONV: No data in the second array'
          return
        endif
        lcgaus = 0
      endelse
      get_wdfarray, array1, wdf1
      if lcgaus eq 1 then $
        gausd, wdf2, sigma = sigma, /nor, range = range, dx = array1[1]
      get_wdfarray, array2, wdf2

;     the next two statements are required if we have xy data

      np1 = n_elements(array1)-2
      np2 = n_elements(array2)-2
;     put array 2 on same time base as 1 
;     get a new array
      if array1[1] ne array2[1] then begin
        n = long ( (np2-1) * array2(1)/array1(1)+1.00)
        if n lt 6 then print,  '% CONV: Warning - Response array has only',n, ' values'
        ivalue = findgen(n)* array1(1)/array2(1)
        array2 = interpolate (array2(2:np2+1), ivalue, /grid)
      endif else array2 = array2(2:*)
      n = n_elements(array2)
      If n_elements (array1) lt n_elements(array2) then begin
        print, '% CONV: Kernel response must be shorter than the array.'
        return
      endif
   if keyword_set(idl) then begin
      array3 = array1[2:*]
      array1[2:*]= convol(array3, array2, total(array2), _extra = extrastuff)
   endif else begin
;     FFT product calculation
      nzero = n + 1 + np1
      nmax = 2L^(ceil(alog(nzero)/alog(2))) > nzero
      if nmax lt 64 then nzero = nmax else begin
        if nzero lt nmax/4*3 then nzero =  nmax/4*3 else nzero = nmax
      endelse
      array3 = make_array(nzero)
      array4 = array3
      array3(0:np1-1) = array1(2:*)
      array4(0:n-1) = array2
      array3 = fft(fft(array3, -1)*fft(array4, -1)*(nzero/total(array2)),1)
      array3 = float(array3)

      array1[2:np1+1] = array3[0:np1-1]

; For RENE  need to also remove the if statement at line 69
;      if np1 gt np2 then $
;        array1[2:np1+1] = array3[0:np1-1] $
;       	else array1 = [array1[0:1], array3[0:np2-1]]
   endelse
      if n_elements (save) eq 0 then save = wdf1
      if save lt 1 or save gt maxwdfarray then save = wdf1
      put_wdfarray, array1, save, /no_copy
      wdf3 = save
      wdffile(save-1) = wdffile(wdf1-1)
      wdfcomment(save-1) = 'Convolute of '+wdfcomment(wdf1-1)
      wdfxlabel(save-1)= wdfxlabel(wdf1-1)
      wdfylabel(save-1)= wdfylabel(wdf1-1)
      array2 = 0
      array3 = 0
      array4 = 0
      sp=0
      im=0
      end
