; ------------------------------------------------------------------------ 
; $Id: digitize.pro,v 1.9 2010/09/20 14:10:32 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
;+
; NAME:
;	DIGITIZE
;
; PURPOSE:
;	Provide a easy way to digitize points on an image
;       Multiple waveforms may be stored.
;       Assumptions are made. See Procedure below.
;
; CATEGORY:
;	Graphics 
;
; CALLING SEQUENCE:
;
;       DIGITIZE, IMAGE [, WDF]
;
; INPUTS:
;       IMAGE:    IMAGE to be digitized.
;                 IMAGE should be a 2 dimensional grey scale array or a 3
;                  dimensional true color array.
;                  If this is an indexed image, color information will be used
;                  to convert it to true color.
;                 True color images should be of the form: [3, w, h] or [w, h, 3]
;                  where w and h are the xsize and ysize of the image.
;                  for example:
;                    image = read_tiff('my_file.tiff', r, g, b)
;                    read_jpeg, 'my_file.jpg', image, color_table
;                 If a color map or the elements of a color map are provided, the
;                   the image is converted to true color using RGB2TRUE
;                 An attempt is made to convert true color images to grey scale
;                   if they are of the form [w, 3, h]. Technique is to use 
;                   New_Image = color_quan(image, true_color, r, g, b, cube=6)
;                   If the results are not satisfactory, please convert to 
;                   a pseudo color image and try again.
;           NOTE:  RGB2TRUE is used to convert to true color if RGB or color table 
;                  values are presented.
;       WDF:      WDF array number for the output.  Default is 1
;	
; KEYWORDS:
;       FLIP   :  If set, flip the image to account for the way that IDL
;                 displays images from the lower left corner vs upper left for 
;                 most other displays.
;                   image = reverse(image, (size(image))[0])
;       ROTATE:   If set, rotate them image.
;                 The following command is used IMAGE = ROTATE(image, rotate)
;                 SEE IDL HELP (? ROTATE) for values.
;       MAXZOOM:  Maximum Zoom ratio.  Default is 20 times input.
;       MAXNP  :  Maximum number of points in a curve > 64
;                 Default = 4096
;       COLOR_TABLE: A [nColors, 3] byte array to convert IMAGE to true color
;     Note:  If COLOR_TABLE is present, RED, GREEN, and BLUE are ignored.
;     Note:  If COLOR_TABLE is present and a 1-D array, then it is assumed to be
;              COLOR_TABLE = [RED, GREEN, BLUE] and it is reformed to an [N, 3] 2D array.
;     Note:  If you need to swap background use SWAP.
;       RED:    Byte array with RED intensity for the indexed colors
;               Values in the range of zero to 255.
;               Must have at least one more value than the maximum of INDEXED_IMAGE
;       GREEN:  Byte array with GREEN intensity for the indexed colors
;               Values in the range of zero to 255.
;               Must have at least one more value than the maximum of INDEXED_IMAGE
;       BLUE:   Byte array with BLUE intensity for the indexed colors
;               Values in the range of zero to 255.
;               Must have at least one more value than the maximum of INDEXED_IMAGE
;       SWAP:   If set, put the first value in the RGB arrays to: (255 - original_value)
;               If SWAP is negative, set the first value in the RGB arrays to ABS(SWAP)
;               SWAP must be greater than -256
;
;  NOTE: XLOG and YLOG must be set as keywords
;       XLOG   :  Set for a log X axis 
;       YLOG   :  Set for a log Y axis
;       XRANGE :  Minimum and Maximum X values used for the Horizontal line
;       YRANGE :  Minimum and Maximum Y values used for the Vertical line
;       BORDER :  Fraction of screen where Zoom Center will automatically reset
;                 Default = 0.1 - Must be in the range [0.01, 0.4]
;       RESET  :  Value to which Zoom Center will be shifted.
;                 Default = 0.8 - Must be in the range [border, 1-border]
;       Example: 
;              For the default values of border and reset:
;                 If x location designated is within .1 of the border, the 
;                 zoom region will be moved by 0.8 of the screen.
; Procedure:
;       Display the image in a magnified screen and use the mouse for picking
;         points.
;       This utility assumes a Cartesian system with square pixels.
;       Log scales must be set with keywords on input 
;       For the Horizontal line: The left most point is associated with the 
;         minimum X value regardless of the order of digitization.
;       For the Vertical line: The bottom most point is associated with the 
;         minimum Y value regardless of the order of digitization.
;       True color images are displayed and, if color map information is provided,
;         indexed images are converted to true color with RGB2TRUE.
;       Steps are:
;       1.  Digitize a horizontal line between two know x values.
;       2.  Calibrate the vertical displacement using either 
;           (a) the horizontal line and 1 additional point
;        or (b) 2 points.
;       3.  Digitize the curve.
;       4.  Save the data in a wdf array.
;       5.  User may "exit" or "return to 3" to digitize an additional curve
;
; EXAMPLE:
;       Digitize an image a and store it in wdf array 3
;         DIGITIZE, a, 3
;
;       Read a JPEG image from the file 'image.jpg' and digitize it.
;       Assume the image is stored with a color table.
;         READ_JPEG, 'image.jpg',IMAGE, colortable
;         DIGITIZE, IMAGE, COLOR_TABLE = colortable
;
;       Read a PNG image from the file picture.png with a rgb color map.
;         READ_PNG, 'picture.png', r, g, b
;         DIGITIZE, im, COLOR_TABLE=[r, g, b], /SWAP
;  < or >
;         DIGITIZE, im,  /SWAP, RED= r, GREEN= g, BLUE = b
;
; MODIFICATION HISTORY:
; 	Written by:       Paul Mix, January 26, 2000
;       Modified:  April 6, 2010 - Added color table information for index images
;-

PRO digitize_event, Event

 COMPILE_OPT hidden

 on_error, 1
  WIDGET_CONTROL,Event.Id,GET_UVALUE=Ev
  widget_control, event.top, get_uvalue = str, /no_copy
  CASE Ev OF 

  'SET_VERT':  BEGIN
               str.vert_opt = event.select
               help, str.vert_opt
               END

  'DELETE_PT': BEGIN
      if (str.npdata gt 0) then begin
        str.npdata = str.npdata -1
        update_digit_draw8, str
      endif else begin
        MESS= ' No Points to Delete '
        winformation, mess
      endelse
      END
  'RESTORE_PT': BEGIN
      if (str.npdata lt str.maxdig ) then begin
        i = str.npdata
        str.npdata = str.npdata +1
        wset,  str.draw1
        plots, str.xpix[i], str.ypix[i], psym = 8, /device, $
          color = !d.table_size-2
        wset, str.draw2
        plots, clip=[0,0,str.xsize, str.ysize], noclip = 0, $
            color = !d.table_size-2, $
          (str.xdata[i]-str.xzrange[0])/str.dx_zoom, $
          (str.ydata[i]-str.yzrange[0])/str.dy_zoom
      endif else begin
        MESS= ' No More Points to ADD '
        winformation, mess
      endelse
      END

  'SAVE_DATA':  BEGIN
            npm = str.npdata-1
            if npm lt 2 then begin
              mess = 'Cannot save waveform with less than 3 points'
              winformation, mess
            endif else begin
              str.npdata = 0
              str.maxdig = 0
              update_digit_draw8, str
              delx = str.xdata[0:npm]-str.x0
              dely = str.ydata[0:npm]-str.y0
              x= str.xscale*(delx*str.costheta+dely*str.sintheta)+str.xinit
              y= str.yscale*(dely*str.costheta-delx*str.sintheta)+str.yinit
              if str.xlog gt 0 then x = exp(x)
              if str.ylog gt 0 then y = exp(y)
              get_wdf_labels,clab, xlab, ylab, file , /modal, group= event.top
              widget_control, str.wdf, get_value= wdf
              i2w, x, y, wdf, clab=clab, ylab=ylab, xlab=xlab, $
                file=file, /xydata
              widget_control, str.wdf, set_value = wdf+1
            endelse
                END

  'H_RESET':  BEGIN
              str.npdata = 0
              update_digit_draw8, str
              END
  'V_RESET':  BEGIN
              str.npdata = 2
              update_digit_draw8, str
              END

  'H_CONT':   BEGIN
      if str.npdata lt 2 then begin
        mess = 'Two Points must be indicated'
        winformation, mess
      endif else if str.dig_mode ne 1 then begin
        mess = ' Internal error.  Should be in Horizontal Mode '
        winformation, mess
      endif else begin
        widget_control, str.hwin1, get_value = x1value
        widget_control, str.hwin2, get_value = x2value
        if x1value gt x2value then begin
          str.x1value = x2value & str.x2value = x1value
          hok = 1
        endif else if x2value gt x1value then begin
          str.x1value = x1value & str.x2value = x2value
          hok = 1
        endif else begin
          hok = 0
          mess= ' X Value #1 must not equal X Value #2 '
          winformation, mess
        endelse
      endelse
      if hok gt 0 then begin
        if str.xdata[0] gt str.xdata[1] then begin
          tmp = str.xdata[0]
          str.xdata[0] = str.xdata[1]
          str.xdata[1] = tmp
          tmp = str.ydata[0]
          str.ydata[0] = str.ydata[1]
          str.ydata[1] = tmp
          tmp = str.xpix[0]
          str.xpix[0] = str.xpix[1]
          str.xpix[1] = tmp
          tmp = str.ypix[0]
          str.ypix[0] = str.ypix[1]
          str.ypix[1] = tmp
        endif
        if str.xlog eq 0  or str.x1value  gt 0 then begin
          str.dig_mode = 2
          digitize_set_mode, str
        endif else begin
          mess =[ ' Logarithmic X axis ', 'Values must be greater than Zero']
          winformation, mess
        endelse
      endif
              END

  'V_CONT':   BEGIN
      if (str.npdata+str.vert_opt) lt 4 then begin
        if (str.vert_opt) eq 0 then mess = 'Two Points must be indicated' $
          else mess = 'One Point must be indicated' 
        mess = [mess, 'for vertical calibration']
        winformation, mess
      endif else if str.dig_mode ne 2 then begin
        mess = ' Internal error.  Should be in Vertical Mode '
        winformation, mess
      endif else begin
        widget_control, str.vwin1, get_value = y1value
        widget_control, str.vwin2, get_value = y2value
        if str.ylog gt 0 and min([y1value, y2value]) le 0 then begin
          mess =[ ' Logarithmic Y axis ', 'Values must be greater than Zero']
          winformation, mess
        endif else begin
          str.y1value = y1value
          str.y2value = y2value
          if y1value eq y2value then begin
            mess= ' Y Value #1 must not equal Y Value #2 '
            winformation, mess
            vok = 0
          endif else begin
          dx = str.xdata[1]-str.xdata[0]
          dy = str.ydata[1]-str.ydata[0]
;         slope of the horizontal line
          angle = atan(dy, dx)
          str.sintheta=sin(angle)
          str.costheta=cos(angle)

          xr = str.costheta *dx + str.sintheta*dy
;         calculate from left end of horizontal line to the first point
          dx = str.xdata[2] -str.xdata[0]
          dy = str.ydata[2] -str.ydata[0]
          yr1 =  str.costheta *dy - str.sintheta*dx
          if str.xlog eq 0 then $
            str.xscale = (str.x2value - str.x1value)/xr else $
            str.xscale = (alog(str.x2value) - alog(str.x1value))/xr
          str.xinit = str.x1value
          if str.xlog gt 0 then str.xinit = alog (str.xinit)
          if str.vert_opt eq 1 then begin
            if str.ylog eq 0 then $
              str.yscale = abs((str.y2value - str.y1value)/yr1) else  $
              str.yscale = abs((alog(str.y2value) - alog(str.y1value))/yr1)
            if yr1 lt 0 then str.yinit = max([str.y1value, str.y2value]) $
              else str.yinit = min([str.y1value, str.y2value])
            if str.ylog gt 0 then str.yinit = alog (str.yinit)
          endif else begin
            dx = str.xdata[3] -str.xdata[0]
            dy = str.ydata[3] -str.ydata[0]
            yr2 =  str.costheta *dy - str.sintheta*dx
            yr = yr2-yr1
            if yr2 lt yr1 then y1 = max([str.y1value, str.y2value], min =y2) $ 
              else y1 = min([str.y1value, str.y2value], max =y2)
            if str.ylog eq 0 then str.yscale = (y2- y1)/yr else $
              str.yscale = (alog(y2) - alog(y1))/yr
            if str.ylog eq 0 then str.yinit =  y2-str.yscale*yr2 else $
              str.yinit = alog (y2)-str.yscale *yr2
          endelse
          str.x0 = str.xdata[0]
          str.y0 = str.ydata[0]
          str.dig_mode = 3
          digitize_set_mode, str
          endelse
        endelse  
      endelse
              END



  'FSLIDM': BEGIN
      if event.drag eq 0 then begin
        str.magnif = event.value
        update_digit_draw8, str
      endif
      END

  'DRAW8': BEGIN
      if ((event.type eq 0) and (event.press eq 1)) then begin
        str.zoomx  = str.xleft[event.x < (str.nxleft-1)]
        str.zoomy  = str.yleft[event.y < (str.nyleft-1)]
        update_digit_draw8, str
      endif
      END

  'DRAW9': BEGIN
      if ((event.type eq 0) and (event.press eq 1)) then begin
        update = 0
        if str.dig_mode eq 3 then begin
          if str.maxnp gt str.npdata then begin
            i = str.npdata
            str.xdata[i] = str.newx[event.x < (str.nxzoom-1)]
            str.ydata[i] = str.newy[event.y < (str.nyzoom-1)]
            str.xpix[i] = round(str.xdata[i] / str.dx_left)
            str.ypix[i] = round(str.ydata[i] / str.dy_left)
            str.npdata = str.npdata+1
            if str.maxdig lt str.npdata then str.maxdig = str.npdata
;          Add the points to the plots
            wset, str.draw1
            plots, str.xpix[i], str.ypix[i], psym = 8, /device, $
              color = !d.table_size-2
            wset, str.draw2
            plots, event.x, event.y, psym = 8, /device, color = !d.table_size-2
;          See if we need to move
            if event.x lt str.borderx then begin
              if (str.newx[0]-str.dx_zoom) gt 0 then begin
                str.zoomx = (str.zoomx - str.xsize*str.reset*str.dx_zoom) > 0
                str.zoomy = str.ydata[i]
                update = 1
              endif
            endif else if event.x gt (str.xsize-str.borderx) then begin
              if str.xzrange[1]+str.dx_zoom lt str.xrange[1] then begin
                str.zoomx = (str.zoomx + str.xsize*str.reset*str.dx_zoom) 
                str.zoomy = str.ydata[i]
                update = 1
              endif
            endif else if event.y lt (str.bordery) then begin
              if (str.newy[0]-str.dy_zoom) gt 0 then begin
                str.zoomy = (str.zoomy - str.ysize*str.reset*str.dy_zoom) > 0
                str.zoomx = str.xdata[i]
                update = 1
              endif
            endif else if event.y gt (str.ysize-str.bordery) then begin
              if str.yzrange[1]+str.dy_zoom lt str.yrange[1] then begin
                str.zoomy = (str.zoomy + str.ysize*str.reset*str.dy_zoom) 
                str.zoomx = str.xdata[i]
                update = 1
              endif
            endif
          endif else begin
            mess=[ 'Maximum number of points has been digitized', $
                   'Either delete points' , $
                   'OR', $
                   'Save these points and use JOINW to combine waveforms']
            winformation, mess
          endelse
        endif else if str.dig_mode eq 2 then begin
          if str.vert_opt + str.npdata lt 4 then begin
              i = str.npdata
              str.xdata[i] = str.newx[event.x < (str.nxzoom-1)]
              str.ydata[i] = str.newy[event.y < (str.nyzoom-1)]
              str.xpix[i] = round(str.xdata[i] / str.dx_left)
              str.ypix[i] = round(str.ydata[i] / str.dy_left)
              str.npdata = str.npdata+1
              if str.maxdig lt str.npdata then str.maxdig = str.npdata
;            Add the points to the plots
              wset, str.draw1
              plots, str.xpix[i], str.ypix[i], psym = 8, /device, $
                color = !d.table_size-2
              wset, str.draw2
              plots, event.x, event.y, psym = 8, /device, color=!d.table_size-2
          endif else begin
              mess =['Required number of points have been set', $
                   'Either delete/reset all points' , $
                   'OR', $
                   'Hit CONTINUE to begin digitizing']
              winformation, mess
          endelse
        endif else begin
           if str.npdata lt 2 then begin
              i = str.npdata
              str.xdata[i] = str.newx[event.x < (str.nxzoom-1)]
              str.ydata[i] = str.newy[event.y < (str.nyzoom-1)]
              str.xpix[i] = round(str.xdata[i] / str.dx_left)
              str.ypix[i] = round(str.ydata[i] / str.dy_left)
              str.npdata = str.npdata+1
              if str.maxdig lt str.npdata then str.maxdig = str.npdata
;            Add the points to the plots
              wset, str.draw1
              plots, str.xpix[i], str.ypix[i], psym = 8, /device, $
                color = !d.table_size-2
              wset, str.draw2
              plots, event.x, event.y, psym = 8, /device, color=!d.table_size-2
          endif else begin
              mess =['Required number of points have been set', $
                   'Either delete/reset all points' , $
                   'OR', $
                   'Hit CONTINUE to begin digitizing']
              winformation, mess
          endelse
        endelse
    
        if (update gt 0 ) then update_digit_draw8, str
      endif
      END

  'BUTTONQ': BEGIN
      if ((str.npdata gt 2) and (str.dig_mode eq 3)) then $
        wsave_nosave, lcquit, [ 'There are digitized points defined', $
          'Do you want to save them before EXIT'], 'DIGITIZE EXIT', $
           group = event.top else lcquit =1
      if lcquit eq 1 then begin
        widget_control, event.top, /destroy
        str =0
        return
      endif
      winformation, ['You may digitize additional points' , $
           '- OR -',    'Press "Save Data" to save']
      END

   ELSE: Print, 'No Handler for event: ', ev
  ENDCASE
  widget_control, event.top, set_uvalue = str, /no_copy
END

pro update_digit_draw8, str
 COMPILE_OPT hidden
  on_error, 1
  npm = str.npdata -1
  wset, str.draw1
  device, copy= [0, 0, str.xsize, str.ysize, 0, 0, str.piximage]
  if npm ge 0 then plots, str.xpix[0:npm], str.ypix[0:npm], $
      psym = 8, /device, color = !d.table_size-2
  if str.dig_mode lt 3 and str.npdata ge 2 then $
    plots, str.xpix[0:1], str.ypix[0:1], /dev, color = !d.table_size-2

  get_scale_image, str.image, str.zoomx, str.zoomy, newim, $
   xzoom, yzoom, str.magnif*str.mag0, str.xsize, str.ysize, $
    xzrange, yzrange, dx_zoom, dy_zoom
    str.newx= xzoom
    str.newy= yzoom
    str.xzrange= xzrange
    str.yzrange= yzrange
    str.dx_zoom= dx_zoom
    str.dy_zoom= dy_zoom
  x= [str.xzrange, rotate(str.xzrange, 2), str.xzrange[0]]*str.mag0
  y= [str.yzrange[0], str.yzrange, rotate(str.yzrange, 2)]*str.mag0
  plots, x, y, color= !d.table_size-1, /dev, thick = 2
  wset, str.draw2
  if str.tru_col eq 0 then tvscl, newim, top= !d.table_size-3 $
    else tv, newim, /true
  if npm lt 0 then return
  ok = where ((str.xdata[0:npm] ge str.xzrange[0]) $
          and (str.xdata[0:npm] le str.xzrange[1]) $
          and (str.ydata[0:npm] ge str.yzrange[0]) $
          and (str.ydata[0:npm] le str.yzrange[1]), nok)
  if nok gt 0 then begin
    x = (str.xdata[ok] -xzrange[0])/str.dx_zoom
    y = (str.ydata[ok] -yzrange[0])/str.dy_zoom
    plots, x, y, psym = 8, /device, color = !d.table_size-2
  endif
  end

pro digitize_set_mode, str
 COMPILE_OPT hidden
  on_error, 1

  if str.dig_mode eq 2 then begin
    widget_control, str.base_1, map = 0
    widget_control, str.base_2, map = 1
  endif else  if str.dig_mode eq 3 then begin
    widget_control, str.base_2, map = 0
    widget_control, str.base_3, map = 1
    str.npdata = 0
    str.maxdig = 0
    wset, str.draw1
    device, copy= [0, 0, str.xsize, str.ysize, 0, 0, str.piximage]

  endif else begin
    mess = [ 'ERROR has occurred', ' PLEASE EXIT ']
    winformation, mess
  endelse
  end

PRO digitize, par1in, par2in, maxzoom = maxzoom, maxnp = maxnp, $
         reset = reset, border = border, xrange = xr, yrange = yr, $
         xlog = xlog, ylog = ylog, true_color = true_color, swap = swap, $
	 flip = flip, xsize = xsizein, ysize  = ysizein, blue = blue, $
         rotate = rotat, color_table = coltab, red = red, green= green,  $
         GROUP=Group, _extra = extrastuffin, modal = modal

on_error, 1
  common pffcom, initpff, pff_exe
@draw_defaults.com
@loadxyct_common.dat
; set_drawsize is used to set the draw sizes
  if n_elements(rrratio) eq 0 then set_drawsize, /init
  if keyword_set(true_color) then print, $
      "% DIGITIZE: TRUE_COLOR keyword is no longer used"
  if n_params() eq 0 then begin
    Print, '%Format: DIGITIZE, IMAGE [, WDF]'
    return
  endif
  if n_elements (par1in) lt 4 then begin
    Print, '% DIGITIZE:  Image must be defined'
    return
  endif
  par1 = par1in
  
  if n_elements(loadct_init) gt 0 then color_save = loadct_init else $
    color_save=5
  ss = size(par1)
  nrgb = 0
  if ss[0] gt 2 then tru_col = 1 else begin
    tru_col =0
    ncoltab = n_elements(coltab)
    if n_elements(coltab) gt 6 then begin
      sscol = size (coltab)
      if sscol[0] eq 2 and sscol[2] eq 3 then begin
        image = rgb2true(par1, coltab, error = error, swap = swap)
        if error eq 0 then par1 = temporary(image)
      endif else if ncoltab mod 3 eq 0 then begin
        coltab = reform(coltab, ncoltab/3, 3)
        image = rgb2true(par1, coltab, error = error, swap = swap)
        if error eq 0 then par1 = temporary(image)
      endif
    endif else begin
      nr = n_elements(red)
      ng = n_elements(green)
      nb = n_elements(blue)
      mnct = min([nr, ng, nb])
      if mnct gt 1 then begin
        image = rgb2true(par1, red, green, blue, error = error, swap = swap)
        if error eq 0 then par1 = temporary(image)
      endif
    endelse
    ss = size(par1)
    if ss[0] gt 2 then tru_col =1
  endelse
  if ss[0] eq 3 then begin
    device , decomposed =0
    loadxyct, 0, /quiet
    loadct, 0, /silent
    tvlct, /get, r, g, b
    ok = where(ss eq 3, nok)
    ok4 = where(ss eq 4, nok4)
    if (nok+nok4)lt 2 then begin
      PRINT, '% DIGITIZE: 2D grey scale or 2D true color image must be provided'
      return
    endif
    if nok eq 1 then begin
      if ok4[0] gt 3 then begin
        PRINT, '% DIGITIZE: 2D grey scale or 2D true color image must be provided'
        help, par1
        return
      endif
      true_color = ok4[0]
      par1= par1[0:2, *, *]
    endif else if ok[1] gt 3 then begin
      PRINT, '% DIGITIZE: 2D grey scale or 2D true color image must be provided'
      help, par1
      return
    endif else begin
      if ok[1] eq 3 then begin
        par1 = bytarr(3, ss[1], ss[2])
        par1[0, *, *] = par1in[*, *, 0]
        par1[1, *, *] = par1in[*, *, 1]
        par1[2, *, *] = par1in[*, *, 2]
        true_color = 1
      endif 
        true_color = ok[1]
    endelse
    if true_color ne 1 then begin
      par1 = color_quan(par1, true_color, r, g, b, cube=6 )
      tru_col = 0
    endif
    ss = size(par1)
  endif
  if keyword_set(flip) then par1 = reverse(par1, 2+tru_col)
  if keyword_set(rotat) then begin
    if tru_col eq 0 then par1 = rotate(par1, rotat) else begin
      a0 = rotate(reform (par1[0,*,*]), rotat)
      sr = size(a0)
      tmp = make_array (3, sr[1], sr[2], type = sr[sr[0]+1])
      tmp[0, *, *]= a0
      for i = 1, 2 do begin
        a0 = rotate(reform(par1[i, *, *]), rotat)
        tmp[i, *, *]= a0
      endfor
      par1 = temporary(tmp)
    endelse
  endif
  mkcircle, /fill
  if n_elements(xsizein) eq 0 then xsize = dualsize(0) else xsize = xsizein
  if n_elements(ysizein) eq 0 then ysize = dualsize(1) else ysize = ysizein

  if n_elements(xr) gt 1 then x1value=min(float(xr),max=x2value) else begin $
    x1value = 0.0
    x2value = 1.0
  endelse
  if n_elements(yr) gt 1 then y1value=min(float(yr),max=y2value) else begin $
    y1value = 0.0
    y2value = 1.0
  endelse
  mag0 = min([float(xsize-1)/(ss[1+tru_col]-1), float(ysize-1)/(ss[2+tru_col]-1)])

  MAIN13 = WIDGET_BASE(GROUP_LEADER=Group, $
      COLUMN=1, modal = modal, $
      MAP=1, $
      TITLE='Digitize Waveform')

  BASE2 = WIDGET_BASE(MAIN13, $
      ROW=1, $
      MAP=1, $
      UVALUE='BASE2')
;
; Base left for the digitizer controls
;
  BaseL = WIDGET_BASE(Base2, $
      FRAME = 3, $
      column=1, $
      MAP=1)

  LABEL38 = WIDGET_LABEL( BASEL, VALUE='Digitizer Controls')

; Make a base for Buttons and the Magnification

  BaseLS = WIDGET_BASE(BaseL, $
      FRAME = 0, $
      ROW=1, $
      MAP=1)
;
; Base for buttons only 

  basebut = widget_base(basels, /column, /map )
  but = widget_button (basebut, value ='Delete Point', /frame, $
        UVALUE = 'DELETE_PT')
  but = widget_button (basebut, value ='Restore Point', /frame, $
        UVALUE = 'RESTORE_PT')

if n_elements (maxzoomin) eq  0 then maxzoom = 20.0/mag0 else $
      maxzoom = float(maxzoomin(0))/mag0
      magnif = 3.0

  FSLIDM = cw_fslider( BASELS, $
      DRAG=1, /edit, /frame, $
      TITLE='ZOOM Magnification', $
      UVALUE='FSLIDM', $
      VALUE=magnif, MAXIMUM = maxzoom, MINIMUM = 1.0)

   if n_elements(par2in) eq 0 then par2 = 1L else par2 = round (par2in)
   par2 = (par2 > 1 )< get_maxwdf()

 saveb = widget_base(baseL, /row, /frame)
   wdfsave= cw_field(saveb,  /row, Title ='WDF Save Array', $
          /long, xsize = 5,  value = par2)
  but = widget_button (saveb, value ='Save Data', /frame, $
            UVALUE = 'SAVE_DATA')


  Baser = WIDGET_BASE(Base2, $
      XPAD = 20, $
      frame = 3, $
      MAP=1)
;
; Three modes for the right side.
;  Horizontal scale and tilt, Vertical scale, and Digitize
;
; Horizontal 
  baser_h = widget_base(baser, /column, map = 1)
  label_h = widget_label( baser_h, $
     Value = 'Digitize a Horizontal Line and Enter Values below')
  baser_hv = widget_base(baser_h, /row ) 
  wx1 = CW_FIELD( baser_hv, value = x1value, $
      ROW=1, /frame, $
      float=1, $
      XSIZE=12, $
      TITLE='X-Value #1:', $
      RETURN_EVENTS = 0, $
      UVALUE='WDF')

  wx2 = CW_FIELD( baser_hv, value = x2value, $
      ROW=1, /frame, $
      float=1, $
      XSIZE=12, $
      TITLE='X-Value #2:', $
      RETURN_EVENTS = 0, $
      UVALUE='WDF')

  baser_hb = widget_base(baser_h, /row)
  hreset = widget_button(baser_hb, /frame, uvalue ='H_RESET', $
        Value = '***   Reset Points   ***')
  hcontinue = widget_button(baser_hb, /frame, uvalue ='H_CONT', $
        Value = '***     Continue     ***')

; Vertical calibration
  baser_v = widget_base(baser, /column, map =0 )
  label_v = widget_label( baser_v, $
     Value = 'Calibrate the Vertical Scale')
  baser_vv = widget_base(baser_v, /row ) 
  holder = widget_base(baser_vv, /col, frame =1)
  wy1 = CW_FIELD( holder, value = y1value, $
      ROW=1, frame =0, $
      float=1, $
      XSIZE=12, $
      TITLE='Y-Value #1:', $
      RETURN_EVENTS = 0, $
      UVALUE='WDF')
  nonex = widget_base(holder, /nonexclu )
  whoriz = widget_button(nonex, Value ='Use Horizontal Line',  $
            UVALUE ='SET_VERT')
;              , /no_release)

  wy2 = CW_FIELD( baser_vv, value = y2value, $
      ROW=1, /frame, $
      float=1, $
      XSIZE=12, $
      TITLE='Y-Value #2:', $
      RETURN_EVENTS = 0, $
      UVALUE='WDF')
  baser_vb = widget_base(baser_v, /row)
  vreset = widget_button(baser_vb,/frame, uvalue ='V_RESET', $
        Value = '***   Reset Points   ***')
  vcontinue = widget_button(baser_vb,/frame, uvalue ='V_CONT', $
        Value = '***     Continue     ***')

; Digitize
  baser_d = widget_base(baser, /column, map =0 )
    label_v = widget_label( baser_d, $
     Value = 'Digitize the Data')
    label_v = widget_label( baser_d, $
     Value = 'Hit "Save Data" to save waveform')
 
  ButtonQ =  WIDGET_BUTTON( Main13, $
      UVALUE='BUTTONQ', $
      VALUE='           Q  U  I  T  /  E  X  I  T         ')

  BASE3 = WIDGET_BASE(MAIN13, $
      ROW=1, $
      MAP=1, $
      UVALUE='BASE3')

  oldwin = !d.window
  oldx_save = !x
  oldy_save = !y
  oldp_save = !p

  DRAW8 = WIDGET_DRAW( BASE3, $
      BUTTON_EVENTS=1, $
      RETAIN=1, $
      UVALUE='DRAW8', $
      XSIZE=xsize, $
      YSIZE=ysize)

  DRAW9 = WIDGET_DRAW( BASE3, $
      BUTTON_EVENTS=1, $
      RETAIN=1, $
      UVALUE='DRAW9', $
      XSIZE=xsize, $
      YSIZE=ysize)

  WIDGET_CONTROL, MAIN13, /REALIZE
  ; Get drawable window index
;
  widget_control, whoriz, /set_button

  WIDGET_CONTROL, DRAW8, GET_VALUE=DRAW1

  ; Get drawable window index

  WIDGET_CONTROL, DRAW9, GET_VALUE=DRAW2
  device , decomposed =0
  loadxyct, 0, /quiet
  loadct, 0, /silent
  tvlct, 255, 255, 0, !d.table_size-1
  tvlct, 0, 255, 0, !d.table_size-2

  wset,draw1
  catch, error_status
    if error_status ne 0 then begin
      print, '% DIGITIZE: Error in call to Get_SCALE_IMAGE'
      widget_control, main13, /destroy
      return
    endif
    window,  /pixmap, /free, xsize = xsize, ysize = ysize
    get_scale_image, par1, 0, 0, newim, xleft, yleft, mag0, xsize, ysize, $
       xrange, yrange, dx_left, dy_left
    if keyword_set(tru_col) then tv, newim, /true else $
      tvscl, newim, top = !d.table_size -3
    piximage = !d.window
  catch, /cancel
 
; save the contents to the draw window
  piximage = !d.window
  wset,draw1
  device, copy = [0, 0, xsize, ysize, 0, 0, piximage]

zoomx = 0.0
zoomy = 0.0
  get_scale_image, par1, zoomx, zoomy, newim, newx, newy, magnif*mag0, $
      xsize, ysize, xZrange, yZrange, dx_zoom, dy_zoom
  nx = n_elements(newx)
  ny = n_elements(newy)
  wset, draw1
;
; Note: Get_Scale_Image does not do exact scaling in both directions.
;       We will assume that they are equal for this plots command.
;
  plots, [newx[0], newx[nx-1], newx[nx-1], newx[0], newx[0]]*mag0, $
    [newy[0], newy[0], newy[ny-1], newy[ny-1], newy[0]]*mag0, $
      color = !d.table_size-1, /dev, thick=2

  wset,draw2
  if keyword_set(tru_col) then tv, newim, /true else $
    tvscl, newim, top = !d.table_size -3
;   tvscl, newim, top = !d.table_size -3

  if n_elements (maxnp) eq 0 then maxnp = 4096 
  maxnp = maxnp > 64
  xdata = make_array(maxnp)
  ydata = make_array(maxnp)
  xpix = make_array(maxnp)
  ypix = make_array(maxnp)
  npdata = 0L
  if n_elements(border) eq 0 then border = 0.1
  if n_elements(reset) eq 0 then reset = 0.8
  border = (border > .01)< 0.40
  reset = (reset > border)< (1.0-border)
  maxdig = 0L
  if keyword_set(xlog) then xlog =1L  else xlog = 0L
  if keyword_set(ylog) then ylog =1L  else ylog = 0L
  data = { maxnp : maxnp, xdata:xdata, ydata:ydata, npdata:npdata, $
         maxdig: maxdig, xpix: xpix, ypix: ypix, Draw1: draw1, draw2: draw2, $
         image: par1, xsize:xsize, ysize:ysize, mag0: mag0, $
         piximage:piximage, zoomx: zoomx, zoomy: zoomy, magnif: magnif, $
         xrange:xrange, yrange:yrange, dx_left: dx_left, dy_left: dy_left, $
         xzrange:xzrange, yzrange:yzrange, dx_zoom: dx_zoom, dy_zoom: dy_zoom, $
         x1value:x1value, y1value:y1value, x2value:x2value, y2value: y2value, $
         xinit: 0.0, yinit: 0.0, x0: 0.0, y0: 0.0, xscale: 1.0, yscale: 1.0, $
         sintheta: 0.0, costheta: 0.0, xlog:xlog, ylog:ylog, $
         vert_opt: 1L, dig_mode : 1L, borderx: border*xsize, $
         bordery: border*ysize, reset: reset, base_1: baser_h, $
         base_2: baser_v, base_3: baser_d, hwin1:wx1, hwin2: wx2, $
         vwin1: wy1, vwin2: wy2, wdf: wdfsave, tru_col: tru_col, $ 
         xleft: xleft, yleft: yleft, newx: newx, newy: newy, $
         nxleft:n_elements(xleft), nyleft:n_elements(yleft), $ 
         nxzoom:n_elements(newx), nyzoom:n_elements(newy) }
;
; Structure Values
; maxnp - Maximum number of points
; Maxdig               - Maximum number of points digitized
; xdata, ydata, npdata - X and Y data and the number of valid points in pixels.
; xpix, ypix           - X and Y data in Pixels for left image
; Draw1, draw2         - Left and right window numbers
; image                - Original image
; tru_col              - set if imabe is true color
; mag0, xsize, ysize   - xsize, ysize and magnification for left image
; Magnif               - Zoom magnification relative to left image
; piximage             - pixmap for the left draw window
; xrange, yrange       - Minimum and maximum pixel values for left image
; dx_left, dy_left     - Increment per pixel for left image
; zoomx, zoomy         - Center of the zoom image
; xZrange, yZrange     - Minimum and maximum pixel values for zoomed image
; dx_zoom, dy_zoom     - Increment per pixel for the zoomed image
; x1value, x2value     - X values used to determine Horiz. line, x scaling
; y1value, y2value     - Y values used to determine Y scaling
; xinit, yinit         - X and Y values corresponding to pixel values X0, Y0
; x0, y0               - Initial pixel location 
; sintheta, costheta   - Sine and cosine of the tilt in the horizontal line
; Xscale, yscale       - Conversion between real values and pixel values
; xlog, ylog           - Indicate log scale - currently not implimented.
; border, reset        - Automatic move parameters. 
; vert_opt             - 0 Indicates two vertical points for y calibration
;                        1 Indicates horizontal line is at one Y value.
; Dig_mode             - Digitizer mode: 1, Horizontal (x) calibration and 
;                        slope; 2, Vertical calibration; and 3, Digitize data
; xleft, yleft         - Pixel values for the left image
; Nxleft, nyleft       - Number of elements in X and Y for the left image 	
; newx, newy           - Pixel values for the zoom image
; Nxzoom, nyzoom       - Number of elements in X and Y for the zoom image 	
;
  widget_control, main13, set_uvalue = data
  XMANAGER, 'DIGITIZE', MAIN13
  wdelete, piximage
  loadxyct, color_save, /quiet
  if oldwin ge 0 then begin
      wset, oldwin
      !x = oldx_save
      !y = oldy_save
      !p = oldp_save
  endif
END
