; ------------------------------------------------------------------------ 
; $Id: div.pro,v 1.4 2010/03/12 22:51:48 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
 Pro  div, wdf1, wdf2, wdf3, wdf4, clabel = clab, dzero = default, $
         ylabel= ylab, xlabel = xlab, match_only = match_only, $
         sortx = sortx, azero = azero, rzero = rzero, $
         master = master,  default = default_master

;
;+
; NAME:
;       DIV
;
; PURPOSE:
;       Divide a WDF array by a constant or a WDF array 
;       Store the result in the initial WDF array or another WDF array
; CATEGORY:
;       Math 
;
; CALLING SEQUENCE:
;       DIV, WD1, WD2 [, WD3]
;       DIV, WD1, 0, VAR, [, WD3]
; INPUTS:
;       WD1:     WDF array number of the first array
;       WD2:     WDF array number of array to be divided into the first array
;  or
;       0, VAR   Variable or constant to be divided into the first array
; OPTIONAL INPUT:
;       WD3:     WDF array number for the quotient, if not specified the 
;                quotient is stored in WD1 (the first array)
; KEYWORD PARAMETERS:
;       DZERO     Value used for Division by zero.
;                 Zero is defined as AZERO + MAX(ABS(WD2))*RZERO
;                 DEFAULT: DZERO = 0.0 
;       AZERO     Absolute Value of zero for the above default value.
;                 Default = 0.0
;       RZERO     Relative value of zero compared to the maximum value.
;                 Default = 2.3841858e-07 = (2^(-22))
;       CLABEL    Replace the dataset comment for array number WD3
;                 with the string or string variable specified
;       XLABEL    Replace the dataset x-axis label for array number WD3
;                 with the string or string variable specified
;       YLABEL    Replace the dataset y-axis label for array number WD3
;                 with the string or string variable specified
;       SORTX     Sort the X values and take all unique values.
;                 Ignored if MASTER is set
;       MATCH_ONLY If one array is XY data and the match is not exact, 
;                  then make a new XY array with only X values in both 
;                  arrays that differ by less than ERR where 
;                  ERR =  (xmax-xmin)/(100*np)]  if match_only ge 0
;                      = - match_only            if match_only lt 0
;                  where np is the number of points in the first array
;                     and [xmin, xmax] is the range of the first data array
;                  (1% of the average point spacing)
;                  The points are selected by:
;                       X2-ERR < X1 < X2+ERR
;                  In the actual implementation, if both arrays are XY then
;                    the X1 values are those of the shorter array and not 
;                    necessarily the first array.
;                  Multiple X2 points which satisfy the inequality are averaged
;       MASTER:    If the abscissa of WD1 does not match the abscissa of WD2 
;                  then the abscissa values in the master array will be used 
;                  exactly in the region of overlap.  If MASTER is negative 
;                  then all of the abscissa values in the master array will 
;                  be used and ordinate values where there is no abscissa will
;                  be set to DEFAULT. If keyword_set(MASTER) and abs(MASTER) is 
;                  not 2, then abs(MASTER) is set to 1.
;       DEFAULT:   Value to be use when MASTER is negative and there is not 
;                  complete overlap of the master curve.
;                  Default:  DEFAULT = 1.0
;
; OUTPUTS:
;       NONE     The quotient is stored in a WDF array in the common block
; COMMON BLOCKS:
;       COM.DAT  WDF common block 
; RESTRICTIONS:
;       WD1 array must be defined and have valid data
;       WD2 or 0 and a Variable must be defined
; PROCEDURE:
;       If WD2 is specified: 
;            WD1 and WD2 are put on a common time base and WD1 divided by 
;            WD2.  If there are less than 2 points of overlap, then 
;            no action is taken.  Linear interpolation is used to put the 
;            arrays on a common abscissa scale using the minimum abscissa 
;            increment.  The quotient is set to DEFAULT for all values of WD2
;            less than val = RZERO*  mx(WD2)+AZERO.
;                2.3841858e-07 = 2.0^(-22)
;               3.8146973e-06 = 1.0/2L^18
;            If the maximum of WD2 is less than 1.0e-30 then all points are
;            set to DEFAULT.
;       If 0, VAR is specified:
;            WD1 is divided by VAR, where VAR is a constant or a variable.
;       In either case, the quotient is stored in WD3, if specified, 
;       or WD1, if WD3 is not specified.
; EXAMPLES:
;       Divide WDF arrays 2 by array 3 and store the result in WDF array 8.
;       and label the result: 'signal minus noise'
;               SUB, 2, 3, 8, c = 'signal minus noise'
;       Divide the WDF array 2 by the variable RATE and store the difference 
;       in WDF array 8.
;               SUB, 2, 0, rate, 8
; MODIFICATION HISTORY:
;      Original version, L. P. Mix, May 15, 1992
;-
;
@com.dat
;
       on_error, 1
;      set quotient to DEFAULT if wdf2 is less than AZERO +RZERO*MX(ABS(WD2))
;      RZERO set to 2^(-22)
      if n_elements(azero) eq 0 then azero = 0.0 else azero = abs(azero[0])
      if n_elements(rzero) eq 0 then rzero = 2.3841858e-07 else $
        rzero = abs(rzero[0])

      if n_elements(default) eq 0 then default = 0.0 else default = default[0]
      if n_elements(initwdf) eq 0 then wdfinit
      nparams = n_params()
      if nparams eq 0 then begin
        print, '% Format: DIV, WD1, WD2 [, WD3] [, CLABEL = clabel]'
        print, '             [, XLABEL = xlabel] [, YLABEL = ylabel]'
        print, '             [, MATCH_ONLY = match_only] [, SORTX = sortx]'
        print, '             [, DZERO = dzero] [, AZERO = azero]'
        print, '             [, RZERO = rzero]'
        print, '             [, MASTER = master] [, DEFAULT = default]    or'
        print, '          DIV, WD1, 0, VAR, [, WD3] [, CLABEL = clabel] '
        print, '              [, XLABEL = xlabel] [, YLABEL = ylabel]'
        return
      endif
      if nparams lt 2 then begin
        print,' DIV must have at least 2 parameters'
        return
      endif
;
      if n_elements(wdf1) eq 0 or  n_elements(wdf2) eq 0 then begin
        print, 'Parameters to DIV must be defined'
        return
      endif
      if wdf1 lt 1 or wdf1 gt maxwdfarray then begin
        print, 'WDF array must be greater than 0 and </=', maxwdfarray
        return
      endif
;
      lcconstant = 0
      if nparams gt 2 then save = get_save_array(wdf3, wdf1) else save = wdf1

;
      nel = wdfnp(wdf1-1)
      if nel lt 2 then begin
        print, 'No data in the first array'
        return
      endif
;
      if wdf2 eq 0 then begin
        if n_elements (wdf3) eq 0 then begin
          print,'Constant is undefined'
          return
        endif
        if wdf3 eq 0.0 then begin
          print, ' Cannot divide by zero!'
          return
        endif
        get_wdfarray, array1, wdf1, /xydata
        t1 = size(array1, /type)
        t2 = size(wdf3, /type)
        if t1 lt t2 then begin
          if t2 eq 4L then array1 = float(array1) else $
            if t2 lt 4L then array1 = long(array1) else $
            if t2 eq 5L then array1 = double(array1) else array1=complex(array1)
        endif
        if n_elements(array1) eq nel +nel then begin
          xydata = 1
          array1(nel:*)= array1(nel:*) / wdf3(0)
        endif else begin 
          xydata = 0
          array1(2:*)= array1(2:*) / wdf3(0)
        endelse
	if nparams gt 3 then save = get_save_array(wdf4, wdf1) else save = wdf1
        lcconstant = 1
      endif else begin 

        if wdf2 lt 1 or wdf2 gt maxwdfarray then begin
          print, 'wdf array must be greater than 0 and </=', maxwdfarray
          return
        endif
        nelb = wdfnp(wdf2-1)
        if nelb lt 2 then begin
          print, 'No data in the second array'
          return
        endif
        get_wdfarray, array1, wdf1, /xydata
        get_wdfarray, array2, wdf2, /xydata
        t1 = size(array1, /type)
        t2 = size(array2, /type)
        if t1 lt t2 then begin
          if t2 eq 4L then array1 = float(array1) else $
            if t2 lt 4L then array1 = long(array1) else $
            if t2 eq 5L then array1 = double(array1) else array1=complex(array1)
        endif
        max1 = wdfmaxx(wdf1-1)
        max2 = wdfmaxx(wdf2-1)
        if n_elements (default_master) eq 0 then def = 1.0 else $
          def = default_master[0]
        if keyword_set(master) then $
          get_master_time, array1, array2, max1, max2, nel, nelb, $
               xydata, master = master, default = def  $
        else $
          get_common_time, array1, array2, max1, max2, nel, nelb, $
               xydata, match_only = match_only, sortx = sortx

        if n_elements(array1) lt 4 then begin
          print, 'No values will be changed due to lack of overlap'
          return
        endif
        offset = (xydata eq 0) ? 2 : nel
        arr = abs(array2(offset:*))
        maxarr = max(arr)
        if maxarr lt  1.0e-30 then begin
          array1(offset:*) = DEFAULT
          print, '% DIV: Maximum value of second array is ', strtrim(maxarr)
          print, '       All values set to ', strtrim(default, 2)
        endif else begin
          maxarr = maxarr* rzero + azero
          p = where (arr lt maxarr, neq0)
          if neq0 gt 0  then array2(p+offset) = 1.0
          array1(offset:*) = array1(offset:*)/array2(offset:*)
          if neq0 gt 0 then begin
            array1(p+offset) = DEFAULT
            print, '% DIV: Division by zero'
            print, neq0, ' elements were set to ', strtrim(default,2)
          endif
        endelse
      endelse
      if n_elements (save) eq 0 then save = wdf1
      if save lt 1 or save gt maxwdfarray then save = wdf1
      put_wdfarray, array1, save, xydata = xydata, /no_copy
      if lcconstant eq 0 then wdf3 = save else wdf4 = save
      if lcconstant eq 0 then begin
        if wdffile(wdf1-1) ne wdffile(wdf2-1) then wdffile(save-1) = '  ' $
           else  wdffile(save-1) = wdffile(wdf1-1)
      endif else wdffile(save-1) = wdffile(wdf1-1)
      if n_elements(clab) ne 0 then wdfcomment(save-1) = clab else $
        wdfcomment(save-1) = wdfcomment(wdf1-1)
      if n_elements(xlab) ne 0 then wdfxlabel(save-1) = xlab else $
        wdfxlabel(save-1)= wdfxlabel(wdf1-1)
      if n_elements(ylab) ne 0 then wdfylabel(save-1) = ylab else $
        wdfylabel(save-1)= wdfylabel(wdf1-1)
      array2=0
      sp=0
      im=0
      end
