; ------------------------------------------------------------------------
; $Id: energ.pro,v 1.7 2010/09/20 14:10:32 lpmix Exp $
; ------------------------------------------------------------------------
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
;+
; NAME:
;       ENERG
;
; PURPOSE:
;       Return the energy of a particle from the velocity
;       See: VELOC for velocity from energy
;
; CALLING SEQUENCE:
;       ENERGY = ENERG(Velocity [, MASS] [, RELATIVISTIC = relativistic]
;                       [, /PROTON] [, /DEUTERON] 
;                       [, /ELECTRON] [, /NEUTRON] [, /DOUBLE] )
;
; OUTPUT:
;       ENERGY:   Particle energy (eV)
; INPUTS:
;       VELOCITY: Particle velocity in m/s or an array of velocities. 
;       MASS:     Particle mass in AMU.
;                 1 AMU = 1.66053873e-27 kg
;                         931494013 eV (Energy equivalent)
;       Note: Values above are approximate.  PHYCONST is used.
;                 DEFAULT: Mass = 1
; KEYWORD PARAMETERS:
;       RELATIVISTIC: Determines when an expansion is used for (GAMMA -1)
;                     RELATIVISTIC must be in the range: [0.05, 0.5]
;                     DEFAULT: RELATIVISTIC = 0.2
;       DOUBLEPRECISION: If set, return a double precision number.
;   NOTE: The following keywords are NOT checked if MASS is present.
;       Once a keyword is found, further checking is stopped.
;      Note: Values below are approximate.  PHYCONST is used.
;       PROTON:   If set, assume a proton - MASS = 1.00727646688D0
;       DEUTERON: If set, assume a deuteron - MASS = 2.01355321271D0
;       NEUTRON:  If set, assume a neutron - MASS = 1.00866491578D0
;       ELECTRON: If set, assume an electron - MASS = 5.485799110D-4
;
; OUTPUTS:
;       Particle energy for the specified velocities.
;
; RESTRICTIONS:
;       VELOCITIES should be greater than 0.0.
;
; PROCEDURE:
;       The velocity is calculated using the relationship:
;         KE = (Gamma -1)*M where M is the rest energy
;            = (sqrt(1/(1-beta^2)) -1) * M
;         Where beta = V/C ie the particle velocity compared to the speed-of-light
;  
;      For: BETA < RELATIVISTIC - an expansion of (Gamma -1) is used.
;           BETA > RELATIVISTIC - the exact calculation is done
;
;         These equations approximately use:
;                 = 931.494013D6 eV/AMU for M
;      The actual values are from the routine PHYCONST in PFIDL_1D.
;
; EXAMPLE:
;      Print the energy of a neutron with a velocity of 2.2e7 m/s
;        print, ENERG(2.2e7, /n)
;
; MODIFICATION HISTORY:
; 	Written by:	L. P. Mix, Jr., April 10, 2007
;-

Function ENERG, velocity, mass, proton = proton, relativistic = relativistic, $
       deuteron = deuteron, electron = electron, neutron = neutron, $
       doubleprecision = doublepre

; constants
on_error, 1
; common block
  common Velocity_common, solc, pmass, dmass, nmass, emass, wmass, mcc, const
  if n_elements(solc) eq 0 then begin
    phyconst, sol = solc, amass = wmass, aenergy = mcc, echarge = charge, $
      penergy = pmass, denergy = dmass, nenergy = nmass, eenergy = emass
     const = 2d0*charge/wmass
 endif
; Relativistic must be in the range (.05, .5)
 if n_elements(relativistic) eq 0 then relativistic = 0.2 else $
   relativistic = (relativistic > 0.05)< 0.5
;  help, relativistic
 
 nel = n_elements(velocity)
 if nel eq 0 then begin
   print, '% ENERG:  Velocity must be defined'
   return, 0.0
 endif
 ener = (nel gt 1) ? make_array(nel, value =0.0d0, /double) : 0.0d0

; rest mass
 if n_elements(mass) eq 0 then begin
   if keyword_set(proton) then mo = pmass else $
   if keyword_set(deuteron) then mo = dmass else $
   if keyword_set(neutron) then mo = nmass else $
   if keyword_set(electron) then mo = emass else mo = mcc
 endif else mo = mass[0]*mcc

 betac = velocity/solc
 ok = where (betac gt relativistic, nok, comp = comp, ncomp= ncomp)
 beta2 = betac*betac
 if nok gt 0 then ener[ok] = 1.0d0/sqrt(1d0-beta2[ok])-1d0 
 if ncomp gt 0 then begin
   beta2 = beta2[comp]
   ener[comp] = $
     (((((((15d0/16d0*beta2+1d0)*13d0/14d0*beta2 +1d0)*  $
     11d0/12d0*beta2 +1d0)*9d0/10d0*beta2+1d0)*7d0/8d0*beta2+1d0)* $
     5d0/6d0*beta2+1d0)*3d0/4d0*beta2+1d0)*1d0/2d0*beta2
 endif
 if keyword_set(doublepre) then return, ener*mo     
 return, float(ener*mo)
 end
