; ------------------------------------------------------------------------ 
; $Id: envelope.pro,v 1.6 2010/09/20 14:10:32 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
 Pro  envelope, wdf1, wdf2, smooth = smoo, plot = plot, fitnp = fitnp, $
  period = wdperiod, pperiod = wdpperiod, nperiod = wdnperiod, $
  maxval = wdmaxval, minval = wdminval
;+
;  This routine calculates the local maximum and minimum of an input WDF array
;  and returns the envelope and the period of the maxima and minima.
;
;  The envelope uses the maxima and the negative of the minima.
;  The period combines the positive and negative periods. 
;  XY data is put on a uniform base for this procedure for convenience. This
;    feature can be changed.
;  Technique:
;  0. Put data on a uniform time base if necessary.
;  1. Calculate the first derivative.
;  2. Filter the derivative with a boxcar filter of width given by smooth.
;  3. Determine the points where the derivative switches sign.
;  4. Determine the average number of points between zero derivatives = M.
;  5. Use FITNP or M/8 points to determine a parabolic fit to the data at each
;     zero derivative point.
;  6. The least square fit results are used to determine maximum/minimum and the
;     center point of the peak.
;  Note: If the parabolic fit has the wrong curvature or does not converge, 
;           no maxima or minima is recorded.
;  OUTPUTS: 
;     The minima are multiplied by -1 and merged with the maxima to 
;             provide the result
;     NOTE: WMAX = MAXWDFARRAY
;     Maxima are stored in WMAX or MAXVAL
;     Minima in array WMAX-1 or MINVAL
;     Period is stored in WMAX-2 or PERIOD
;     Positive period is stored in WMAX-3 or PPERIOD
;     Negative period is stored in WMAX-4 or NPERIOD
;
;     This routine uses the IDL user function, SVDFIT.
;
;     In the case of noisy data singular data may result. 
;
;  Format:  ENVELOPE , wd1 [, wd2] [, SMOOTH = smooth] [, PLOT = plot]
;                 [, FITNP = fitnp]  [, PERIOD = period] [, PPERIOD = pperiod]
;                 [, NPERIOD = nperiod] [, MAXVAL = maxval] [, MINVAL = minval]
;
;  where:
;    wd1, wd2 - WDF array numbers
;               if wd2 is present, the final curve is stored in wd2
;               otherwise the result is stored in wd1
;    smooth   - If present and not zero, the derivative is smoothed using this
;               width. Default is smooth = 25; miminum is 11
;    plot     - If present and not zero, the original curve and the extrema
;               are plotted.
;    fitnp    - If present, this number of points is used to fit the extrema
;               Default value is average zero crossing spacing (M) divided by 8
;               (for a sine wave this will give an error less than 1.0e-4)
;               Unsmoothed data is used for the least squares fit.
;               If FITNP lt 5 then fitnp is returned with the value used.
;    PERIOD   - WDF array for the period, default = WMAX-2
;    PPeriod  - WDF array for the positive period, default = WMAX-3
;    NPeriod  - WDF array for the negative period, default = WMAX-4
;    MAXVAL   - WDF array for the maximum values, default = WMAX-1
;    MINVAL   - WDF array for the minimum values, default = WMAX
;    
;
;  Examples:
;    Fit an envelope to the data points in array 2; return the envelope 
;    to array 3 and calculate the period. Fit the data between 9 and 100 to 
;    an exponential and plot the fit.
;      envelope, 2, 3 & print, 'Period = ', ave(80, /q)
;      lim, 3, 9, 100 & wdfit, 3, 4, /ytyp, /plot
;
;    Fit an envelope to the data in WDF array 4, store the envelope in array
;       9 and plot the original data and the envelope.
;      envelope, 4, 9, /plot
;-
;
@com.dat
;
      on_error, 1
      if n_elements(initwdf) eq 0 then wdfinit
      nparams = n_params()
      if nparams lt 1 then begin
        print,' ENVELOPE must have at least 1 parameter'
        return
      endif
;
      if n_elements(wdf1) eq 0 then begin
        print, 'Parameters to ENVELOPE must be defined'
        return
      endif
      if wdf1 lt 1 or wdf1 gt maxwdfarray then begin
        print, 'WDF array must be greater than 0 and </=', maxwdfarray
        return
      endif
;
      if nparams gt 1 then save = get_save_array(wdf2, wdf1) else save = wdf1
      if save lt 1 or save gt maxwdfarray then save = wdf1
      nel = wdfnp(wdf1-1)
      if nel eq 0 then begin
        print, 'No valid data in the array'
        return
      endif
      get_wdfarray, array, wdf1
      scale = max ([abs(max(array(2:*), min= mm)), abs(mm)])
      y = array(2:*)/scale
      nel = n_elements(y)
      x=findgen(nel)*array(1)+array(0)
      dx = array(1)
      degree = 2
      m = 3
      wdf = 1
      sing = 0
      
      dy = y(1:nel-1)-y(0:nel-2)
;     dydx = dy/dx
      if n_elements (smoo) eq 0 then smoo = 25
      smoo = smoo > 5
      del2 = long(smoo/2)
      del = 1+del2+del2
;    since dx is positive definite - just use dy
      dydx = smooth(dy, smoo)
      dydx1= dydx(0:nel-3)
      dydx2 = dydx(1:nel-2)
      okp = where (dydx1 gt 0.0 and dydx2 le 0.0, npos)
      okn = where (dydx1 lt 0.0 and dydx2 ge 0.0, nneg)
      ndel= 0
      ddel = 0.0
      if npos gt 1 then begin
        ddel = total(okp(1:npos-1)-okp(0:npos-2))
        ndel = npos-1
      endif
      if nneg gt 1 then begin
        ddel = ddel + total(okn(1:nneg-1)-okn(0:nneg-2))
        ndel = ndel + nneg-1
      endif
      if ndel lt 2 then begin
        print, '%ENVELOPE: Cannot determine an envelope with less than 4 points'
        return
      endif
      if n_elements(wdmaxval) eq 0 then wdmaxval = maxwdfarray else $
          wdmaxval = get_save_array(wdmaxval, maxwdfarray)
      if n_elements(wdminval) eq 0 then wdminval = maxwdfarray-1 else $
          wdminval = get_save_array(wdminval, maxwdfarray-1)
      if n_elements(wdperiod) eq 0 then wdperiod = maxwdfarray-2 else $
          wdperiod = get_save_array(wdperiod, maxwdfarray-2)
      if n_elements(wdpperiod) eq 0 then wdpperiod = maxwdfarray-3 else $
          wdpperiod = get_save_array(wdpperiod, maxwdfarray-3)
      if n_elements(wdnperiod) eq 0 then wdnperiod = maxwdfarray-4 else $
          wdnperiod = get_save_array(wdnperiod, maxwdfarray-4)
      per = ddel/ndel
      if n_elements(fitnp) eq 0 then fitnp = round(per/8) +1
      if fitnp lt 5 then fitnp = round(per/8) +1
      del2 = (fitnp/2) > 3
      del = 1+del2+del2
      fitnp = del
      xsvd = findgen(del)
      jlower = del2-1
      jupper = nel-2-del2
      jinc = del2+1
      if npos gt 0 then begin
        np= 0
        j = 0
        yp = make_array(npos)
        xp = yp
        for i = 0, npos-1 do begin
          k = okp (i)
          if k le jupper and k gt j+del then begin
            j=k
;         skip points near the end of the curve
            coef=svdfit (xsvd, y(j-jlower:j+jinc), m,yf= yf, singular=sing) 
            if sing eq 0 then begin
              if coef(2) lt 0 then begin
                xm = -coef(1)*0.5/coef(2)
                if xm ge 0 and xm lt del then begin
                  xp(np) =  xm*array(1) +x(j-1)
                  yp(np) = (coef(2)*xm+coef(1))*xm+coef(0)
                  np = np+1
                endif
              endif  else begin
                plot, x(j-jlower:j+jinc), y(j-jlower:j+jinc)
                hak, mes = 'Wrong Curvature for Max. Hit any key to continue..'
              endelse

            endif  else begin
              plot, x(j-jlower:j+jinc), y(j-jlower:j+jinc)
              hak, mes = 'Singular Fit. Hit any key to continue..'
            endelse
          endif
        endfor
      endif
      if nneg gt 0 then begin
        nn= 0
        j = 0
        yn = make_array(nneg)
        xn = yn
        for i = 0, nneg-1 do begin
          k = okn (i)
          if k le jupper and k gt j+del then begin
            j = k
;         skip points near the end of the curve
            coef =  svdfit (xsvd, y(j-jlower:j+jinc), m, singular = sing) 
            if sing eq 0 then begin
              if coef(2) gt 0 then begin
                xm = -coef(1)*0.5/coef(2)
                if xm ge 0 and xm lt del then begin
                  xn(nn) = xm*array(1)+x(j-1)
                  yn(nn) = (coef(2)*xm+coef(1))*xm+coef(0)
                  nn = nn+1
                endif
              endif  else begin
                plot, x(j-jlower:j+jinc), y(j-jlower:j+jinc)
                hak, mes = 'Wrong Curvature for Min. Hit any key to continue..'
              endelse

            endif  else begin
              plot, x(j-jlower:j+jinc), y(j-jlower:j+jinc)
              hak, mes = 'Singular Fit. Hit any key to continue..'
            endelse

          endif
        endfor
      endif
      if nn eq 1 then nn = 0
      if np eq 1 then np = 0
      if nn gt 0 or np gt 0 then begin
        xt = make_array(np+nn)
        yt = xt
        t =  make_array(np+nn-2)
        period =  make_array(np+nn-2)
      endif
      if n_elements (plot) eq 0 then plot =0
      if plot ne 0 then plo, wdf1, tit= 'Envelope for '+wdfcomment(wdf1-1)
      if nn gt 1 then begin
        xn = xn(0:nn-1)
        yn = yn(0:nn-1)*scale
        t(0:nn-2) = (xn(0:nn-2)+xn(1:nn-1))*0.5
        period(0:nn-2) = (xn(1:nn-1)-xn(0:nn-2))
        xt(0:nn-1) = xn
        yt(0:nn-1) =  -yn
        i2w, xn, yn, maxwdfarray-1, c = $
        'Minimum Envelope for '+wdfcomment(wdf1-1), /xy
        i2w, t(0:nn-2),period(0:nn-2), maxwdfarray-4, c = $
        'Period for Minima '+wdfcomment(wdf1-1), /xy
        if plot ne 0 then plo, maxwdfarray-1, color = 1, /no
      endif else $
          print, ' Less than two points found for negative ENVELOPE'
      if np gt 1 then begin
        xp = xp(0:np-1)
        yp = yp(0:np-1)*scale
        i2w, xp, yp, maxwdfarray, c = $
        'Maximum Envelope for '+wdfcomment(wdf1-1), /xy
        xt(nn:*) =  xp
        yt(nn:*) =  yp
        t((nn-1>0):*) = (xp(0:np-2)+xp(1:np-1))*0.5
        period((nn-1>0):*) = xp(1:np-1)-xp(0:np-2)
        i2w, t((nn-1>0):*),period((nn-1>0):*), maxwdfarray-3, c = $
        'Period for Maxima '+wdfcomment(wdf1-1), /xy
        if nn gt 0 then begin
          ii = sort(xt)
          xt = xt(ii)
          yt = yt(ii)
          ii = sort(t)
          t = t(ii)
          period = period(ii)
        endif

        if plot ne 0 then plo, maxwdfarray, color = 2, /no
      endif else print, ' Less than two points found for positive ENVELOPE'
      if nn+np gt 1 then begin
        i2w, xt, yt, save, c = $
        'Envelope for '+wdfcomment(wdf1-1), /xy
        i2w, t, Period  , wdperiod, c = $
        'Period for '+wdfcomment(wdf1-1), ylab ='Period '+ wdfxlabel(wdf1-1), $
         xlab =  wdfxlabel(wdf1-1), /xy
;
        wdf2 = save
        wdffile(save-1) = '  '
        wdfxlabel(save-1)= wdfxlabel(wdf1-1)
        wdfylabel(save-1)= wdfylabel(wdf1-1)
      endif else print, ' No data for the ENVELOPE'
      array=0
      wdf = 0
      a = 0
      b = 0
      end
