; ------------------------------------------------------------------------ 
; $Id: eq_charge.pro,v 1.3 2007/06/06 20:03:46 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
 function eq_charge, z, q, energy, ier
;+
; Function returns the fraction of a given charge species in equilibrium at
;   a given energy.
; This code is from tables generated by John Maenchen from Marion and Young
;   and applies only to lithium 7, boron 11, carbon 12, nitrogen 14, 
;   oxygen 16, and fluorine 19.
; 
; Format:
;       fraction = eq_charge(z, q, energy [, ier]
;    where:
;          z      - ion z, ie number of protons
;                   must be 3, 5, 6, 7, 8,  or 9 
;                   and mass assumed to be 7, 11, 12, 14, 16, and 19
;          q      - atom net charge, must be greater than or equal to zero
;                   and less than or equal to Z
;          energy - ion kinetic energy or an array of energies
;          ier    - optional error flag,
;                   0 no errors
;                   1 data incomplete or bad
;                  -1 errors present (data out of range)
;                 -99 Element not supported
;          fraction- Charge fraction
;                    -1.0 returned for errors 
;     Examples
;        find the charge fraction lithium(+1) at 5 Mev and store in frac
;          frac = eq_charge (3, 1, 5, ier)
;
;        Plot the charge fraction versus energy for lithium (+2) between
;          zero and 10 MeV
;        x = findgen(201)/20
;        plot, x, eq_charge(3, 2, x, ier)
;
;        Store the charge fractions for lithium in WDF arrays 10 to 13 and 
;          plot them in an overlay plot between 0 and 10 MeV
;    x = findgen(201)/20
;    xlab = 'Energy (MeV)' & ylab='Charge Fraction' & tit=ylab +' vs '+xlab
;    for i = 0, 3 do i2w,x,eq_charge( 3, i, x), $
;       i+10, xlab=xlab, ylab= ylab, cl= 'Charge - '+strtrim(i, 2)
;        plo, 10, 4, /ov, /co, title = tit
;-    
      on_error, 1
      common pffcom, initpff, pff_exe
;     check to see if pff has been initialized
;
      if (n_elements(initpff) eq 0) then init_pff else $
         if initpff eq 0 then  init_pff

      if (initpff eq 0) then begin
        print, '% EQ_CHARGE: PFIDL Routines have not been fully installed.'
        return, -1.0
      endif
;
      if n_elements(z) gt 0 then zout = long(round (z)) else begin
        ier = 1
        print, '%EQ_CHARGE: Ion Z not provided'
        return, -1.0
      endelse

      if n_elements(q) gt 0 then qout = long(round (q)) else begin
        ier = 1
        print, '%EQ_CHARGE: Ion Q not provided'
        return, -1.0
      endelse

      if (z lt 1 or z gt 9) then begin
        ier = -99
        print, $
           '%EQ_CHARGE: Ion Q not valid with Ion Z - Z, Q =', strtrim(z, 2), $
            ', ', strtrim(q, 2)
        return, -1.0
      endif


      nel = n_elements(energy)
      if nel gt 0 then eout = float(energy ) else begin
        ier = 1
        print, '%EQ_CHARGE: Ion energy not provided'
        return, -1.0
      endelse

      if min(eout) lt 0 then begin
        ier = 1
        print, '%EQ_CHARGE: Ion energy must be greater/equal to zero'
        return, -1.0
      endif

      ier = 0L
      fraction = make_array(nel)
      name = strmid(!version.os, 0, 3)

      if initpff eq 1 then begin
; HP
        junk = call_external(PFF_EXE, $
           'slcharg_eq', zout, qout, eout, fraction, nel, ier)
      endif else if (initpff eq 2) then begin
; SUN
        junk = call_external(PFF_EXE, $
           'slcharg_eq_', zout, qout, eout, fraction, nel, ier)
      endif else if (initpff eq 4) then begin
; WIN
        junk = call_external(PFF_EXE, $
           '_SLCHARG_EQ@8', zout, qout, eout, fraction, nel, ier)
      endif else if (initpff eq 5) then begin
; MAC
        junk = call_external(PFF_EXE, $
           'slcharg_eq', zout, qout, eout, fraction, nel, ier)
      endif else begin
        print,' This routine is not supported.'
        ier = -999
      endelse

      return, fraction
      end

