; ------------------------------------------------------------------------ 
; $Id: find_pro.pro,v 1.5 2010/09/20 14:10:32 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
 pro find_pro, name, files = files, nfiles = nfiles, save = save, quiet = quiet
;+
; NAME:
;       FIND_PRO
;
; PURPOSE:
;       Find a procedure in the IDL path.
;       This uses the routine file_search and is not very fast.
;          Can take 10 to 20 seconds to do the search. 
;
; CATEGORY:
;       UTILITY
;
; CALLING SEQUENCE:
;
;       FIND_PRO, NAME [, FILES = files] [, NFILES = nfiles] [, /SAVE]
;                   [, /quiet] [, _EXTRA= extrastuff]
;
;     where:
;       NAME   - Name of the procedure
;                Wild cards may be used.  See IDL help for FILE_SEARCH
;                Name is converted to lower case.
;       FILES  - Files which could be used for this procedure
;       NFILES - Number of files found
;       SAVE   - If set, looks for a save file
;                Default: SAVE = 0
;                Look for NAME.pro
;       QUIET  - If set, no output to the terminal
;       _Extaa - any valid keyword for FILE_SEARCH
;
; OUTPUTS:
;       A list of files is output to the terminal.
;
; OPTIONAL OUTPUTS:
;       The list can be returned as keywords
;
; EXAMPLE:
;       Find the source code for a procedure plotfld
;         FIND_PRO, 'plotfld'
;
; MODIFICATION HISTORY:
;       Written by:     L. P. Mix, June 20, 2002

;-

 on_error, 1
 if n_elements(name) eq 0 then begin
   print, '% FIND_PRO: Procedure name must be provided.'
   return
 endif
 fulnam = strlowcase(strtrim (name[0]))
 if keyword_set (sav) then fulnam= fulnam+'.sav' else fulnam= fulnam+'.pro'
 files = file_search(strsplit(!path, path_sep(/search),  /extract), $
   fulnam, count = nfiles, _extra = extrastuff)

 if nfiles gt 0 then begin
   files = files[uniq(files, sort(files))]
   nfiles = n_elements(files)
 endif
 if keyword_set(quiet) then return
 if nfiles eq 0 then begin
   print, '% FIND_PRO: No files found with name: ', fulnam
 endif else begin
   print, files, format = '(a)'
 endelse
 end
