; ------------------------------------------------------------------------ 
; $Id: fit_data.pro,v 1.6 2010/09/20 14:16:19 lpmix Exp $
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
;+
; NAME:
;       FIT_DATA
;
; PURPOSE:
;       Generate a set of points fit to an input WDF array.
;       Options are a spline fit with adjustable tension or a least square
;         polynomial fit.
;       The spline fit includes a filter option.
;       A gui is used unless QUIET is set.
;
; CALLING SEQUENCE:
;	FIT_DATA, WDF, XARRAY [, YARRAY]
;   where:
;       WDF    - WDF array data array number or a unique string in the 
;                dataset comment
;        XARRAY - Abscissa values where interpolated values of WDF are desired
;                XARRAY will be returned with the values used.
;                XARRAY can be specified two ways:
;             Literal value option
;                XARRAY can be the actual values.  A minimum of 4 values is 
;                  required.  The values will be sorted.
;                  NOTE: Some values may be identical if fewer than 4 values
;                        are desired.
;             Do-list option
;                XARRAY can be an initial point and a final point with an 
;                  optional spacing. If spacing is missing, an array of 
;                  512 points will be generated.
;                  XARRAY will extend from XARRAY[0] to or beyond XARRAY[1].
;                  For example: XARRAY = [0, 1, .3] will result in 
;                  XARRAY=[0, 0.3, 0.6, 0.9, 1.2]
;       YARRAY - Ordinate values interpolated from WDF at XARRAY values
;                XARRAY and YARRAY are stored in get_maxwdf()-1
;
; KEYWORD PARAMETERS:
;       SAVE:    WDF array for the fitted data.
;                DEFAULT:   SAVE = get_maxwdf()
;       TYPE:    Type of fit desired
;                TYPE = 0 ==>> SPLINE fit
;                TYPE = 1 ==>> Least Square fit
;                Default: TYPE = 0
;   SPLINE FIT KEYWORDS:
;	TENSION: Tension for the spline fit.
;                DEFAULT: TENSION = 1
;       FILTER:  If a spline fit, a filter will be applied to the fit data
;                If FILTER LE 0, NO filter is applied.
;                Default:  FILTER = XRANGE/8
;                FILTER is sent to FILT: FILT DELTA = filter
;   LEAST SQUARE FIT KEYWORDS:
;        DEGREE: Degree of polynomial fit (maximum power of X)
;        QUIET:  If set, it will be passed to WDFIT
;
; EXAMPLE:
;	Fit the data in wdf array 11 to points between 100 and 1000 wnameith 
;         to X values with a spacing of 0.015 and store the result in 12
;          FIT_DATA, 11, SAVE = 12, [100, 1000, .015]  
;
; MODIFICATION HISTORY:
; 	Written by:	L. P. Mix, January 13, 2004
;-

 pro fdata_plotit_widget, info
  on_error, 1
  COMPILE_OPT hidden
  wset, info.window
  plo, info.wdf1, psym = -8, /over, /color
  plo, info.save, ns, /nl, /ov, /no, /color, /ns
 end

pro fit_data_event, event
  COMPILE_OPT hidden
  on_error, 1
  widget_control, event.id, get_uvalue = name
  Case name of 
'BUTTONS':  BEGIN
    if event.value eq 1 then begin
      widget_control, event.top, /destroy
      return
    endif
    widget_control, event.top, get_uvalue = info, /no_copy
    if info.type eq 0 then begin
      widget_control, info.tension, get_value = tension
      widget_control, info.filter, get_value = delta
      w2i, x, y, info.wdf1
      yarr = spline(x, y, info.xarray, tension)    
      widget_control, info.tension, set_value = tension
      widget_control, info.filter, set_value = delta
    endif else begin
      widget_control, info.degree, get_value = degree
      wdfit, info.wdf1, info.save, degree = degree, coef = coef, $
         quiet= info.quiet
      widget_control, info.degree, set_value = degree
      yarr = poly(info.xarray, coef)
      delta = 0     
    endelse
    if n_elements (info.dx) gt 1 then $
       put_wdfarray, [info.dx, yarr], info.save $
       else i2w, info.xarray, yarr, info.save, /xydata
    lab, info.save,  'Fit to '+wdcomment(info.wdf1)
    if delta gt 0 then filt, info.save, delta = delta

    fdata_plotit_widget, info

    widget_control, event.top, set_uvalue = info, /no_copy
    END   
ELSE:  Print, 'Unknown event'
END
end

pro type_fdata_widget_event, event
 COMPILE_OPT hidden
;
; Handle type changes
; 
  on_error, 1
  widget_control, event.top, get_uvalue = info, /no_copy
  if event.value eq 0 then begin
    widget_control, info.pbase, map = 0
    widget_control, info.sbase, map = 1
    info.type =0
  endif else begin
    widget_control, info.sbase, map = 0
    widget_control, info.pbase, map = 1
    info.type =1
  endelse
  widget_control, event.top, set_uvalue = info, /no_copy
  end

pro poly_fdata_widget_event, event
  COMPILE_OPT hidden
  on_error, 1
  widget_control, event.top, get_uvalue = info, /no_copy
    widget_control, info.degree, get_value = d
    widget_control, info.degree, set_value = d
  widget_control, event.top, set_uvalue = info, /no_copy
end

pro spline_fdata_widget_event, event
 COMPILE_OPT hidden
  on_error, 1
  widget_control, event.top, get_uvalue = info, /no_copy
    widget_control, info.tension, get_value = t
    widget_control, info.tension, set_value = t
    widget_control, info.filter, get_value = f
    widget_control, info.filter, set_value = f
  widget_control, event.top, set_uvalue = info, /no_copy

end

pro fit_data, wdf, xarray, yarray, type = type, filter = filterin, save = save, $
    degree = degree, tension = tension, delta = delta, quiet = quiet, $
    xsize = xsizein, ysize = ysizein, GROUP_LEADER=group, modal = modal
;
;       return on an error
      on_error, 1
@com.dat
   if n_elements(initwdf) eq 0 then wdfinit
;  Common block
;
@draw_defaults.com
;  common fit_data_common, xsize, ysize
;
   nparams = n_params()
   if nparams eq 0 then begin
     print,'% Format: ', $
       'FIT_DATA, WDF, XARRAY [, YARRAY] [, SAVE = SAVE] [, TYPE = type]'
     print,'       [, FILTER = filter] [, TENSION = tension] [, DELTA = delta]'
     print,'       [, DEGREE = degree] [, QUIET = quiet]'
     return
   endif

   if nparams lt 2 then begin
     print, '% FIT_DATA: WDF array and XARRAY must be provided.'
     return
   endif
   if n_elements(wdf) eq 0 then begin 
     print, '% FIT_DATA: WDF array must be defined.'
     return
   endif 
   wd = wdvalid(wdf[0], /valid, npoint = np)

   if np ne 1 then begin
     print, '% FIT_DATA: WDF must define one and only one array'
     print, '            WDF value: ', wdf
     return
   endif else wd = wd[0]
   
   if n_elements(save) eq 0 then save = maxwdfarray-1 else $
     save = get_save_array(save, maxwdfarray-1)
   nelx = n_elements(xarray)
   dx = 0
   if nelx gt 4 then xarr = xarray[sort(xarray)] else begin
     if nelx lt 2 then begin
       print, '% FIT_DATA: XARRAY must be defined'
       return
     endif
     st = min(xarray[0:1], max = en)
     if nelx eq 3 then dx = xarray[2] else dx = (en-st)/512
     nelx = ceil((en-st)/dx) +1
     xarr = findgen(nelx) * dx +st
     dx = [st , dx]
   endelse
   if n_elements(dx) eq 1 then begin
     check_uniform, x, dx 
     if dx gt 0 then dx= [xarr[0], dx] 
   endif
   nx = n_elements(xarr)
   xrange = xarr[nx-1] -xarr[0]
   if n_elements(group) eq 0 then group =0
         
   tlbase = WIDGET_BASE(GROUP_LEADER=Group, $
      ROW=1, modal = modal, $
      MAP=1, $
      TITLE='FIT_DATA')

  BASEINS  = WIDGET_BASE(tlbase, $
      COLUMN = 1, $
      MAP=1, $
      UVALUE = 'BASEINS')

  DrawBase = WIDGET_BASE(tlbase, $
      COLUMN = 1, $
      MAP=1, $
      UVALUE = 'DRAW')

  BASE0 = WIDGET_BASE(baseins, $
      column=1, $
      FRAME=1, $
      MAP=1, $
      event_pro = 'TYPE_FDATA_WIDGET_EVENT', $
      UVALUE='BASE0')

  label  =  WIDGET_LABEL( BASE0, $
      UVALUE='LABEL', $
      VALUE='Select Fit Type')


     Btns = [ $
    'Spline Fit', $
    'Polynomial' $
     ]
  if n_elements (type) eq 0 then type = 0  else $
    if keyword_set(type) then type = 1 else type = 0

  type_sel = CW_BGROUP( Base0, Btns, $
      COLUMN=1, set_value = type, $
      EXCLUSIVE=1, /NO_RELEASE, $
      UVALUE='TYPE')

  BASE1 = WIDGET_BASE(baseins, $
      FRAME=0, $
      MAP=1, $
      UVALUE='BASE1')

  BBASE = WIDGET_BASE(baseins, $
      FRAME=1, $
      MAP=1, $
      UVALUE='BBASE')

  SBASE = WIDGET_BASE(base1, $
      FRAME=2, $
      MAP=1, $
      column = 1, $
      event_pro = 'spline_fdata_widget_event', $
      UVALUE='BASE2')

  label1  =  WIDGET_LABEL( SBASE, $
      UVALUE='LABEL', $
      VALUE='Spline Fit Parameters')

      if n_elements(tension) eq 0 then tension  = 1
  tension_win= CW_FIELD(SBASE,VALUE= tension, $
      ROW=1, xsize = 15, $
      FLOAT=1, $
      RETURN_EVENTS = 1, $
      TITLE='Tension:', $
      UVALUE='TENSION')

      if n_elements(filterin) eq 0 then Filter = .15*xrange else filter =filterin*xrange 
  filter_win= CW_FIELD(SBASE,VALUE= filter, $
      ROW=1, xsize = 15, $
      FLOAT=1, $
      RETURN_EVENTS = 1, $
      TITLE='Filter Length:', $
      UVALUE='FILTER')

  PBASE = WIDGET_BASE(base1, $
      FRAME=2, $
      MAP=0, $
      column=1, $
      event_pro = 'poly_fdata_widget_event', $
      UVALUE='BASE3')

  label2  =  WIDGET_LABEL( PBASE, $
      UVALUE='LABEL', $
      VALUE='Polynomial Fit Parameters')

  if n_elements(degree) eq 0 then degree = 4 else degree = degree > 1
  degree_win= CW_FIELD( PBASE,VALUE= degree, $
      ROW=1, xsize = 8, $
      long=1, $
      RETURN_EVENTS = 1, $
      TITLE='Poly Degree:', $
      UVALUE='DEGREE')

  Btnsdoit = [ $
    ' Calculate Fit ', $
    '  Done / Exit  ' ]
  doit_buttons = CW_BGROUP( bbase, Btnsdoit, $
      ROW=1,  /NO_RELEASE, $
      UVALUE='BUTTONS')

  oldwin = !d.window
  initbx = !x
  initby = !y
  initbp = !p
  pmulti_save=!p.multi 
  !p.multi = 0
  if n_elements(xsizein) eq 0 then begin
    if n_elements(xsize) eq 0 then xsize = bigsize[0]
  endif else xsize = xsizein
  if n_elements(ysizein) eq 0 then begin
    if n_elements(ysize) eq 0 then ysize = bigsize[1]
  endif else ysize = ysizein
  if n_elements(tlb_offset) eq 0 then tlb_offset = [10, 20]

  if n_elements (xpreviewsize) eq 0 then xpreviewsize = smallsize


  DRAW = WIDGET_DRAW( DrawBase, $
      RETAIN=1, $
      UVALUE='DRAW', $
      XSIZE=xsize, $
      YSIZE=ysize)

  WIDGET_CONTROL, tlbase, /REALIZE, tlb_set_xoffset = tlb_offset[0], $
            tlb_set_yoffset = tlb_offset[1]
if keyword_set(type) then begin
  widget_control, type_sel, set_value = type
  widget_control, pbase, map = 1
  widget_control, sbase, map = 0
endif

; Get drawable window index
  WIDGET_CONTROL, DRAW, GET_VALUE=DRAW_Id
  loadxyct, /quiet, /freeze
  wset, draw_id
  plo, wd, psym = -8, /over, /color 
  qu = (keyword_set(quiet))? 1:0
  info={sbase:sbase, pbase:pbase, bbase:bbase, type: type, quiet:qu, $
        tension: tension_win, filter: filter_win, degree:degree_win, $ 
        xarray:xarr, dx: dx, wdf1:wd, save: save, window:draw_id }

  widget_control, tlbase, set_uvalue = info

  XMANAGER, 'fit_data', tlbase
  if  arg_present(yarray) then yarray = getyf(save)
  xarray = xarr
  info = 0
  if oldwin gt 0 then begin
      wset, oldwin
      !x = initbx
      !y = initby
      !p = initbp
      !p.multi = pmulti_save
  endif
  
END
