; ------------------------------------------------------------------------ 
; $Id: gaus.pro,v 1.6 2011/01/21 17:20:20 lpmix Exp $
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
;+
; NAME:
;       GAUS
;
; PURPOSE:
;       Plot an array of data to determine if the data is from a Gaussian or 
;       normal distribution.  Data in the WDF array is assumed to be value
;       versus sequence number.  This routine is designed to examine a set
;       of data to examine the likelihood that it is from a gaussian 
;       distribution.
;
;       This procedure will make one or all four of the following;
;         Sequence plot: Identical to PLO
;         Lag plot: Plot of Y[i] vs Y[i-1]
;         Histogram Plot: Plot of histogram of the data
;         Probability Plot
;
;   Note: get_maxwdf() and the two wdf arrays below it are used as 
;         work arrays for some plots
;
; CALLING SEQUENCE:
;       GAUS, WDdata [, WDSAVE]
;
; INPUTS:
;       WDdata  - WDF array number of the sequence of data points
;                 or an array of data points
;       WDsave  - WDF array number of the probability plot
;                 This is ignored for other types of plots
;                 This is curve is also stored in get_maxwdf()-2
; KEYWORD Parameters:
;       GRID    - Ignored if a single plot is requested.
;                 If more than one plot is requested, grid specifies the number
;                 of plots per page or the number of plots in the X and Y 
;                 direction.  Default: GRID = 1
;                 If grid eq 2 then !p.multi = [0,2,1]
;                 If grid gt 2 then !p.multi = [0,2,2]
;	SEQUENCE- If set, a sequence plot will be drawn.
;                 Input array or Y-values will be plotted versus sample number.
;                 X values in WDdata are ignored.
;       LAG_PLOT- If set a lag plot will be drawn.  
;                 The default is a lag of 1.
;                 Y[i] will be the ordinate and Y[i-1] is the abscissa
;       HISTOGRAM - If set, a histogram plot will be drawn.  
;                 The number of bins will be odd centered on the data average.
;                 The approximate number of bins will be sqrt(0.5*np) on each
;                 side of the mean value to a value of 3.5*SD.
;                 The mean and standard deviation (SD) of the data is printed.
;                 A gaussian is overplotted with the same area as the histogram.
;                 The histogram and gaussian are stored in the top 3 wdf arrays.
;       PROBABILITY - If set, draw a normal probability plot.
;                 In this plot the experimental data is plotted as the ordinate
;                 and the normal order statistic median is drawn along the 
;                 abscissa.
;                 The uniform order statistic median is determined by ordering
;                 the data and assigning probabilities according to:
;                   M[1] = 1 - M[n]
;                   M[i] = (i - 0.3175)/(n+.365) for i = 2, 3, ..., n-1
;                   M[n] = 1./2^n
;                   where n is the number of points.
;                 The PFIDL routine get_prob() is used with these probabilities
;                 to determine abscissa values.
;        The following keywords apply only to PROBABILITY plots:
;            COEF   - Coeficients for he least square fit from WDFIT
;            RSQUARED - Rsquared for the least square fit from WDFIT.
;            MARKERS  - Probability markers for the plot
;                       Default = [.05, .25, .75, .95]
;
;        ALL_PLOTS - Plot all 4 plots.  For this option, Grid = [2,2]
;        QUIET   - Suppress most output to the terminal.
;        
;        _EXTRA  - Any valid keyword in call to PLOT
;               
; OUTPUTS:
;       Requested plots will be made
;
; EXAMPLES:
;	Plot a probability plot of the measurements in the array x
;       Save the plot data in wdfarray 23
;         GAUS, y, 23, /prob
;
;	Plot all 4 statistical plots on one sheef for the data in wdfarray 4
;         GAUS, 4, grid = 4, /all
;
;       Make a histogram plot of data in array, xx.
;         GAUS, xx, /his
;
;       Make a histogram plot of data in array, xx, and show histogram bins
;         GAUS, xx, /his, psym = 10
;
; MODIFICATION HISTORY:
;       Original version, February 26, 2003
;                         L. P. Mix
;-
;
Pro GAUS, WDF1, WDF2, Sequence = sequence, Lag_plot = lagplot, $
       all_plots= allplots, histogram = histog, probability = probability, $
       grid = grid, coef = coef, rsquared = rsquared, quiet = quiet, $
       markers = markers,  _extra = extrastuff
;
;Return to caller if an error occurs
on_error,1  
;
;
      marker_default = [.05, .25, .75, .95]
      work = get_maxwdf()
      nparams = n_params()
      if nparams eq 0 then begin
        print, '% Format: GAUS, WDdata [, WDSAVE]'
        print, '             [, SEQUENCE = sequence] [, LAG_PLOTS = lag_plots]'
        print, '             [, HISTOGRAM = histogram] [, ALL_PLOTS = all_plots]'
        print, '             [, PROBABILITY = probagility] [, /QUIET]
        print, '             [, COEF = coef] [, RSQUARED = rsquared]
        print, '             [, MARKERS = markers] [, _EXTRA = extrastuff]
        return
      endif

      if nparams lt 1 then begin
        print,'% GAUS: Waveform array or array must be provided.
        return
      endif
;
      if n_elements(wdf1) eq 0 then begin
        print, '% GAUS: Waveform array must be defined'
        return
      endif 
      clab=''
      if n_elements(wdf1) gt 1 then yarr = wdf1 else begin
        wdf1 = wdf1[0]
        a = size(wdf1)
        if a(a(0)+1) eq 7 then begin
          wdf1 = wdvalid(wdf1(0))
          if wdf(0) lt 1 then begin
            print, '% PLO: No arrays found using the search string: ', wdf(0)
            return
          endif else yarr = getyf(wdf1[0])
        endif else begin 
          if wdf1 lt 1 or wdf1 gt work then begin
            print, '% GAUS: WDdata must be greater than 0 and </=', work
            return
          endif 
          if wdvalid(wdf1, /valid) lt 1 then begin
            print, '% GAUS: WDdata does not have valid data:' , wdf1
            return
          endif
          yarr = getyf(wdf1)
        endelse
        clab = wdcomment (wdf1[0])
      endelse
      np = n_elements(yarr)
      if nparams gt 1 then wdf2 = get_save_array(wdf2, work-2) else $
        wdf2 = -1
      if keyword_set(allplots) then begin
        if not keyword_set(sequence) then sequence = 1
        if not keyword_set(lagplot) then lagplot = 1
        if not keyword_set(histog) then histog = 1
        if not keyword_set(probability) then probability = 1
      endif 
      if  not keyword_set(sequence) and  not keyword_set(lagplot) and $
        not keyword_set(histog) and not keyword_set(probability) then begin
        print, '% GAUS: One keyword must be set to specify plot'
        return
      endif
      if n_elements(grid) eq 0 then grid = 1
      oldmulti = !p.multi
      if n_elements(grid) gt 1 then begin
        !p.multi =[0, grid[0], grid[1]] 
        lcmulti =1
      endif else if grid eq 2 then begin
        !p.multi = [0,2,1]
        lcmulti =1
      endif else if grid gt 2 then begin
        !p.multi = [0,2,2]
        lcmulti =1
      endif else lcmulti =0
        

      if keyword_set(lcmulti) then begin
        xdelta = 0.02
        ydelta = 0.025
        ydelt2 = 0.05
        charsiz = 1
      endif else begin
        xdelta = 0.04
        ydelta = 0.05
        ydelt2 = 0.08
        charsiz = 2
      endelse
      lcplot = 0
      yr = [min(yarr, max = m), m]
      if keyword_set(sequence) then begin
        tit = 'Run Sequence Plot'
        m = moment(yarr, sd = sd)
        tit2 ='Median= '+strtrim(m[0],2)+', Std Dev= '+strtrim(sd, 2) 
        if strlen (clab) gt 0 then tit = clab+', '+tit
        plot, 1+findgen(np), yarr, title = tit, ytit ='Value', $
          xtit = 'Measurement Index', xr =[0, np], yr = yr, $
          _extra = extrastuff
        xyouts,  !x.window[0]+xdelta, !y.window[1]-ydelta, /normal, tit2
        lcplot = 1
      endif
     
      if keyword_set(lagplot) then begin
        tit = 'Lag Plot'
        m = moment(yarr, sd = sd)
        tit2 ='Median= '+strtrim(m[0],2)+', Std Dev= '+strtrim(sd, 2) 
        if strlen (clab) gt 0 then tit = clab+', '+tit
        mkcircle
        if lcplot eq 1 and lcmulti eq 0 then hak, /message else lcplot = 1
        plot, yarr[0:np-2], yarr[1:*], title = tit, ytit ='Y[i]', $
          xtit = 'Y[i-1]', xr = yr, yr = yr, psym =8,  _extra = extrastuff
        xyouts,  !x.window[0]+xdelta, !y.window[1]-ydelta, /normal, tit2
      endif
      if keyword_set(histog) then begin
        nhist = 512
        tit = 'Histogram Plot'
        m = moment(yarr, sd = sd)
        tit2 ='Median= '+strtrim(m[0],2)+', Std Dev= '+strtrim(sd, 2) 
        if strlen (clab) gt 0 then tit = clab+', '+tit
        i= findgen(512)
        nbin = round(sqrt(np*0.5)) 
        nbin = nbin+nbin+1
        range = 7*sd
        x0 = m[0]-3.5*sd
        x = findgen(nhist)*(range/(nhist-1))+x0
        dbin = range/nbin
        xh = findgen(nbin)*dbin+(x0+dbin*0.5)
        yhist = histogram(yarr, binsiz = dbin, min = x0, nbins = nbin)
        if lcplot eq 1 and lcmulti eq 0 then hak, /message else lcplot = 1
        plot, xh, yhist, title = tit, ytit ='Number of values', $
          xtit = 'Measured Value', _extra = extrastuff, psym =10
        i2w, xh, yhist, work-2, clab = 'Histogram - '+ tit2
;  area of histogram is np*dbin
        yn = np*dbin/sqrt(2*!pi)/sd*exp(-0.5/sd/sd*(x-m[0])^2)
        loadxyct, /quiet, /freeze
        oplot, x, yn, color = 3
        i2w, x, yn, work-1, clab = 'Gaussian - '+tit2
        xyouts,  !x.window[0]+xdelta, !y.window[1]-ydelta, /normal, tit2
      endif

      if keyword_set(probability) then begin
        tit = 'Normal Probability Plot'
        m = moment(yarr, sd = sd)
        tit2 ='Median= '+strtrim(m[0],2)+', Std Dev= '+strtrim(sd, 2)
        if strlen (clab) gt 0 then tit = clab+', '+tit
        pr = findgen(np)
        pr[np-1] = 0.5^(1.0/np)
        pr[0]= 1.0-pr[np-1]
        pr[1:np-2] = (pr[1:np-2]+0.6824) / (np+0.365)
        yarr = yarr[sort(yarr)]
        pr = get_prob(pr)
        i2w, pr, yarr, work-2, clab = tit+' - '+tit2,  $
          xlab = 'Normal Order Statistic Medians', $
          ylab = 'Ordered Values' 
        wdfit, work-2, work -1, quiet = quiet, rsquared = rsquared, coef = coef
        if lcplot eq 1 and lcmulti eq 0 then hak, /message else lcplot = 1
        plo, work-2, _extra = extrastuff, /truncate, over =0
        plo, work-1, /no, over = 0, color = 3
        if wdf2 gt 1 then xfr, work-2, wdf2
        fitc = 'Fit Parameters: C[0]='+strtrim(coef[0], 2)+', C[1]=' $
               +strtrim(coef[1], 2)
        xyouts,  !x.window[0]+xdelta, !y.window[1]-ydelt2,  fitc, /nor
        xyouts,  !x.window[0]+xdelta, !y.window[1]-ydelta, /normal, tit2
        if n_elements(markers) eq 0 then markers = marker_default
        nmark = n_elements(markers)
        
        ymin = !y.crange[0]
        dy = (!y.crange[1]-!y.crange[0])
        yup = !y.crange[0]+0.04*dy
        dy = (!y.crange[1]-!y.crange[0])*0.01
        xval = get_prob(markers)
        for i = 0, nmark-1 do begin
          oplot, thick = 2, xval[i]*[1,1], [ymin, yup]
          xyouts, xval[i],yup+dy, align= 0.5, chars =charsiz, $
            strtrim(round(100*markers[i]), 2)+'%' 
        endfor
     endif
        
 
     !p.multi = oldmulti
     end
