; ------------------------------------------------------------------------ 
; $Id: get_dset.pro,v 1.4 2010/09/20 14:16:19 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
 Function get_dset, str, fileid = fid, maxdset = maxdset, quiet= quiet, $
               ndset = ndset

;+
; Name: GET_DSET
; 
; PURPOSE:
;       Get PFF dataset numbers for all dataset comments containing
;          a specified string.
;          -1 indicates no dataset found.
;       Note:  Dataset comments in the directory are currently limited to
;              64 characters.  A search string longer than this will result
;              in no matches.
; Format:
;       Map = Get_Dset(STRING [, FILEID = fileid] [, MAXDSET = maxdset]
;                     [, NDSet = ndset] )
; INPUTS:
;       STRING:   Search string 
;                 If STRING = '' or ' ' or '*' then 
;                     Map = 1+lindgen(ndspff(fid))
;                     (Maxdset is ignored)
;                 The following special characters can be used:
;                 *  - Wild card indicating 0 to n characters
;                      To use a * in a search string use "\*"
;                 ?  - Wild card indicating 1 and only 1 characters
;                      To use a ? in a search string use "\?"
;                 ^  - If this is the first character, indicates that
;                      the string begins the dataset comment.
;                      All other times a ^ is taken literally.
;                      To begin a search string with a literal ^ use:
;                          "*^" or "\^"
;                      All search strings are assumed to begin with "*" unless
;                      a "^" is present. 
;                 $  - If this is the last character, indicates that the
;                      string ends the non-blank portion of the dataset comment.
;                      All other times a $ is taken literally.
;                      To end a search string with a literal $ use:
;                          "*$" or "\$"
;                      All search strings are assumed to end with "*" unless
;                      a "$" is present.
; OPTIONAL INPUT:
;       FILEID  : File id - Default is 0 (The current PFF file) 
;       QUIET   : If set, no messages will be printed.
;       MAXDSET : Maximum number of files desired.
;                 If more than MAXDSET files are found, then an additional
;                 search is performed over the returned datasets with the
;                 search string modified to force an exact match; ie.
;                 if not present a "^" will be added to the front of the 
;                 string and a "$" to the end of the string.
;       NDSET   : Number of datasets returned.
;  EXAMPLES: 
;     Find all the datasets with VDIODE in the comment.
;              wdf = get_dset('vdiode')
;     Find all the datasets with the non_blank characters only VDIODE
;              wdf = get_dset('^vdiode$')
;     Find all the datasets with the string VDIODE and at least one 
;       more character at the end
;              wdf = get_dset('vdiode?')
;     Read all the datasets in the current file starting at position 1
;              re, 1, get_dset()
;     or more simply
;              re, 1, ''
;  
;-
; add common block for call_external
;
      common pffcom, initpff, pff_exe
;
      on_error, 1
      ndset = 0L
      if (n_elements(initpff) eq 0) then initpff = 0
      if initpff eq 0 then begin
        if not keyword_set(quiet) then print, $
           '% GET_DSET: No PFF files are open'
        return, -1
      endif
      if n_elements (fid) eq 0 then fid = 0L
      pid = long(fid)
      if n_elements(str) eq 0 then str = ''
      slen = strlen (str)
      if ((slen eq 0) or ((slen eq 1) and (strmid(str, 0, 1) eq '*'))) then begin
        ndset = ndspff(fid)
        return, 1+lindgen(ndset)
      endif
      if n_elements (maxdset) eq 0 then maxdset = ndspff(fid) $
        else maxdset = maxdset > 1
      mmap = long(maxdset)
      map = make_array(maxdset, /long)
      nmap = 0L
;           length of the dstring string and the label strings
            ldst = 128L
            str2byt, str, dstringbyt, ldst
; 
            if initpff eq 1 then begin
; HP
              junk = call_external(PFF_EXE,'slgetdset', pid, slen, $
                 dstringbyt, nmap, map, mmap, ldst)
            endif else if initpff eq 2 then begin
; Sun
              junk = call_external(PFF_EXE,'slgetdset_', pid, slen, $
                 dstringbyt, nmap, map, mmap, ldst)
            endif else if initpff eq 3 then begin
; VMS
              dstringbyt = string (dstringbyt)
              junk = call_external(PFF_EXE,'get_dset', pid, slen, dstringbyt, $
                 nmap, map, mmap)
            endif else if initpff eq 4 then begin
; WIN
              junk = call_external(PFF_EXE,'_SLGETDSET@8', pid, slen, $
                 dstringbyt, nmap, map, mmap, ldst)
            endif else if initpff eq 5 then begin
; MAC
              junk = call_external(PFF_EXE,'slgetdset', pid, slen, $
                 dstringbyt, nmap, map, mmap, ldst)
            endif
            if nmap eq 0 then begin
              if not keyword_set (quiet) then print, $
                '% GET_DSET: No match for string ', str
              return, -1
            endif
;           limit the string to defining nmax datasets
;           if we have more try to find an exact match
            if nmap gt maxdset then begin
              if (strmid(str, 0, 1) ne '^' ) then newdata = '^'+str else $
                  newdata = str
              if (strmid(str, slen-1, 1) ne '$') then $
                newdata = newdata+'$'
              slen2 = strlen(newdata)
              if slen2 gt slen then begin
                slen = slen2
                str2byt, newdata, dstringbyt, ldst
                if initpff eq 1 then begin
; HP
                  junk = call_external(PFF_EXE,'slgetdset', pid, slen, $
                     dstringbyt, nmap, map, mmap, ldst)
                endif else if initpff eq 2 then begin
; Sun
                  junk = call_external(PFF_EXE,'slgetdset_', pid, slen, $
                     dstringbyt, nmap, map, mmap, ldst)
                endif  else if initpff eq 3 then begin
; VMS
                  dstringbyt = string (dstringbyt)
                  junk = call_external(PFF_EXE,'get_dset',pid,slen,dstringbyt, $
                     nmap, map, mmap)
                endif else if initpff eq 4 then begin
; WIN
                  junk = call_external(PFF_EXE,'_SLGETDSET@8', pid, slen, $
                     dstringbyt, nmap, map, mmap, ldst)
                endif else if initpff eq 5 then begin
; MAC
                  junk = call_external(PFF_EXE,'slgetdset', pid, slen, $
                     dstringbyt, nmap, map, mmap, ldst)
                endif
              endif
              if nmap eq 0 or nmap gt maxdset then begin
                if keyword_set(quiet) then return, -1
                print, ' A unique dataset was not found with the string '
                print, '             ' , str, ' -- Current datasets are'
                dirpff, dir
                return, -1
              endif
            endif
            map = temporary(map(0:nmap-1))
            ndset = temporary(nmap)
            return, map
      end
