; ------------------------------------------------------------------------ 
; $Id: get_value.pro,v 1.6 2009/03/25 16:20:35 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
Function  Get_value, wdfin, xin, valid = valid, ivalues = ivalues, $
               quiet = quiet, missing = missing, match_values = match_values, $
               epsilon = eps
;+
; NAME:
;       Get_Value
;
; PURPOSE:
;       This procedure returns the y values of an wdf array at a given x value
;
; CALLING SEQUENCE:
;
;      Value = Get_Value (WDF, X)
;
; INPUTS:
;       WDF      -  wdf array number 
;       X        -  abscissa value or an array of abscissa values
;	
; KEYWORD PARAMETERS:
;      VALID     -  If set, X will be modified to include only values
;                   in the domain of the wdf array.
;      IVALUES   -  values of x for which valid values are found
;                   = -1 if no values are found
;      MISSING   -  Value returned if X is outside the domain.
;                   Default = 0.0
;      QUIET     -  If set, output will be supressed.
;                   NOTE:  No error messages will be printed!!
;                   DEFAULT:  QUIET = 0
;      EPSILON   -  Error limit for Matching values.
;                   EPSILON = Min([3.0517578e-5, 1.0/number_of_points(WDF)]) * $
;                     Max(Abs(WDF Abscissa))
;      MATCH_VALUES- If set and data is XY and X values are not monotonically increasing,
;                      then attempt to match X-values in order.
;                    This keyword is ignored for uniform data or nonuniform data with
;                      increasing values of X.
;                    A perfect match is considered if X values differ by less than 
;                      Epsilon
;                    Default: Match = 0
;                    Match = 0, sorts data and does linear interpolation to the 
;                      specified values.
; OUTPUTS:
;       Function returns the Y values for the specified X values.
;
; PROCEDURE:
;       Simple linear interpolation is used for uniform WDF data.
;       XY data is sorted and multple ordinates at the same abscissa are averaged.
;       If keyword MATCH_VALUES is set, the procedure will attempt to match the 
;         specified X values exactly.
;
; EXAMPLE:
;
;       Find the y values for an array x in WDF array 4 and store in y.
;         y = GET_VALUE(4, x)
;
;       Find the y values for an array xx in WDF array with comment 
;         "Corrected Voltage" and store in yy.
;         y = GET_VALUE('Corrected Voltage', xx)
;
; MODIFICATION HISTORY:
;       Written by:     L. Paul Mix 
;       November 15, 2002 L. Paul Mix
;                       Modified handling of XY data
;-
@com.dat
;
 on_error, 1
      ivalues = -1
      if n_elements(initwdf) eq 0 then wdfinit
      nparams = n_params()
      if nparams eq 0 then begin
        if not keyword_set(quiet) then $
          print, '% Format: y = Get_Value( WDF, X [, /VALID] )'
        return, 0.0
      endif
      if nparams lt 2 then begin
        if not keyword_set(quiet) then $
          print,'% GET_VALUE: Two parameters are required'
        return, 0.0
      endif
;
      nelxin = n_elements(xin)
      if n_elements(wdfin) eq 0 or nelxin eq 0 then begin
        if not keyword_set(quiet) then $
          print, '% GET_VALUE: Parameters must be defined'
        return, 0.0
      endif
      a = size(wdfin, /type)
      if a eq 7 then begin
        wdf = wdfin[0]
        if strtrim(wdf, 2) eq '*' then wdf = (wdvalid())[0] else $
          wdf = (wdvalid(wdf))[0]
        if wdf lt 1 then begin
          if not keyword_set(quiet) then $
       print, '% Get_Value: No arrays found using the search string: ',wdfin(0)
          return, 0.0
        endif
      endif else wdf = wdfin[0]
;
      if wdf lt 1 or wdf gt maxwdfarray then begin
        if not keyword_set(quiet) then $
          print, '% Get_Value: WDF array must be >  0 and </=', maxwdfarray
        return, 0.0
      endif
;
      nel = wdfnp(wdf-1)
      if nel lt 2 then begin
        if not keyword_set(quiet) then $
          print, '% Get_Value: No valid data in the wdf array'
        return, 0.0
      endif
      if n_elements (eps) gt 0 then epsilon= eps[0] > 0 else $
        epsilon = min([3.0517578e-5, 1.0/wdfnp(wdf-1)])* $
           max(abs([wdfminx(wdf-1),wdfmaxx(wdf-1)]))

      okx = where ((xin ge  (wdfminx(wdf-1)-epsilon)) $
         and (xin le (wdfmaxx(wdf-1)+epsilon)), nelx)
      ivalues = okx
      if nelx lt nelxin then begin
        if nelx eq 0  then begin
          if not keyword_set(quiet) then $
            print,'% Get_value: No X values within the domain of the WDF array'
          return, 0.0
        endif
        x = xin(okx) 
        if keyword_set (valid) then xin= x else $
          if not keyword_set(quiet) then $
            print,'% Get_value: Not all X values within the domain of the WDF array'
      endif else x = xin
      if size(/type, x) lt 4 then x = float(x)
      get_wdfarray, array, wdf, /xy 
      np = n_elements(array)-2
      atype = size(array, /type)
      if atype lt 4 then begin
        array= float(array)
        atype = 4L
      endif
      if np eq nel then begin
; uniform data
        a = (x -array(0))/array(1)
        n = long(a) < (nel-2)
        np2 = n+2
        y = array(np2)+ (a-n)*(array(np2+1)- array(np2))
      endif else begin
        xx = array(0:nel-1)
        yy = array(nel:*)
        dimage = xx(1:nel-1)-xx(0:nel-2)
        if ((keyword_set(match_values)) and (min(dimage) le 0)) then begin
;         make an array for the output
          y = make_array(nelx) 
          i = 0
          j = 0
          while ((i lt nelx) and (j lt nel)) do begin
            ok = where (x[i] eq xx[j:*], nok)
            if nok gt 0 then begin
              y[i] = yy[j+ok[0]]
              j = j + ok[0]+1 
              i = i+1
            endif else begin
              ok = where ((x[i]-epsilon lt  xx[j:*]) and $
                   (x[i]+ epsilon gt xx[j:*]), nok)
              if nok gt 0 then begin
                y[i] = yy[j+ok[0]]
                j = j + ok[0]+1
                i = i+1
              endif else i = nelx+1
            endelse
          endwhile

          if i eq nelx then begin
            if keyword_set(valid) or nelx eq nelxin then return, y
            if keyword_set (missing) then missing = (missing[0]) else missing = 0.0
            yy = make_array(nelxin, type = atype, value = missing)
            yy(okx) = y
            return, yy
          endif else $
            if not keyword_set(quiet) then $
              print, '% GET_VALUE: Unable to match X values'
        endif
        if min(dimage) lt 0 then begin
          ind = sort(xx)
          xx = xx(ind)
          yy = yy(ind)
          dimage = xx(1:nel-1)-xx(0:nel-2)
        endif
        ok = where (dimage eq 0, nok, complement = notzero)
        if nok gt 0 then begin
          if nok eq nel-1 then begin
            if not keyword_set(quiet) then $
              print, '% Get_value: Only one unique values in waveform array'
            return, 0.0
          endif 
          xx = xx[[notzero, nel-1]]
          n=2
          av = ok[0]
          np = ok[0]+1
          for ij = 1, nok- 1 do begin
            if ok[ij] eq np then begin
              np = np+1
              n = n+1
              av = [av, ok[ij]]
            endif else begin
              yy[np] = total([yy[av], yy[np]])/n
              n = 2 
              av = ok[ij]
              np = av +1
            endelse
          endfor
          yy[np]  = total([yy[av], yy[np]])/n
          yy =  yy[[notzero, nel-1]]
          nel = n_elements(xx)
;         Dimage should not be zero at this point
;           dimage = xx(1:nel-1)-xx(0:nel-2)
;           ok = where (dimage eq 0, nok, complement = notzero)
        endif
        indx = sort(x)
        a = sort(indx)
        x = x(indx)
        y = interpolate(yy,findex(xx, x))
        if nelx eq 1 then y = y(0) else y = y(a)
      endelse
      array=0
      if keyword_set(valid) or nelx eq nelxin then return, y
      if keyword_set (missing) then missing = (missing[0]) else missing = 0.0
      yy = make_array(nelxin, type = atype, value = missing)
      yy(okx) = y
      return, yy
      end
