; ------------------------------------------------------------------------ 
; $Id: getcolor.pro,v 1.4 2007/06/06 20:03:48 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
;+
; NAME:
;       GETCOLOR
;
; PURPOSE:
;       The purpose of this function is to return the value of a
;       "named" color or numbered color. The result is a vector containing
;       the values of the color triple.
;
; CATEGORY:
;       Graphics, Color Specification.
;
; CALLING SEQUENCE:
;       result = GETCOLOR(color)
;
; REQUIRED INPUTS:
;       COLOR: A string with the "name" of the color or a color index. 
;              Only the first character is checked; valid values are
;               [R, G, B, M, O, C, Y, W] or [1, 2, 3, 4, 5, 6, 7, 8]
;              Eight colors are allowed; the default colors are: 
;              1   RED       [255,   0,   0]
;              2   GREEN     [  0, 255,   0]
;              3   BLUE      [  0,   0, 255]
;              4   MAGENTA   [255,   0, 255]
;              5   ORANGE    [255, 125,   0]
;              6   CYAN      [  0, 255, 255]
;              7   YELLOW    [255, 255,   0]
;              8   WHITE     [150, 150, 150] (actually grey)!
;              These colors may be changed with PUTCOLOR.
;
;              If the string is anything else, a RED color triple is
;              returned.  If an index is provided, then the default is 
;              the closest color.
;
; KEYWORD PARAMETERS:
;       TRUE:  If this keyword is set the specified color triple is returned
;              as a 24-bit integer equivalent. The lowest 8 bits correspond to
;              the red value; the middle 8 bits to the green value; and the
;              highest 8 bits correspond to the blue value. 
;
; COMMON BLOCKS:
;       None.
;
; SIDE EFFECTS:
;       None.
;
; EXAMPLE:
;       To obtain the color values for yellow
;       
;          value = getcolor('y')
;   <or>   value = getcolor(7)
;
;       To do the same thing on a 24-bit color system, type:
;       
;          value = getcolor('y', /true)
;   <or>   value = getcolor(7  , /true)
;       
; MODIFICATION HISTORY:
;       Written by: Paul Mix, January 11, 2000
;                   True color modification written by David Fanning.
;            
;-

;  Following function was written by David Fanning.

FUNCTION COLOR24, number

   ; This FUNCTION accepts a [red, green, blue] triple that
   ; describes a particular color and returns a 24-bit long
   ; integer that is equivalent to that color. The color is
   ; described in terms of a hexidecimal number (e.g., FF206A)
   ; where the left two digits represent the blue color, the 
   ; middle two digits represent the green color, and the right 
   ; two digits represent the red color.
   ;
   ; The triple can be either a row or column vector of 3 elements.
   
ON_ERROR, 1



base16 = [[1L, 16L], [256L, 4096L], [65536L, 1048576L]]

num24bit = 0L

FOR j=0,2 DO num24bit = num24bit + ((number(j) MOD 16) * base16(0,j)) + $
   (Fix(number(j)/16) * base16(1,j))
   
RETURN, num24bit
END ; ************************  of COLOR24  ******************************



FUNCTION GETCOLOR, color, TRUE=truecolor
   
@loadxyct_common.dat
; common loadxyct_common, loadct_init, max_color ,invertct_flag, $
;     color_maximum, color_minimum, line_colors
;
ON_ERROR, 1

 if n_elements(line_colors) lt 20 then putcolor
ss = size(color, /type)
if ((ss eq 0) or (ss gt 7))then begin
;
; DEFAULT value
;
  color  = 1
  ss = size(color, /type)
endif

; set up the color names.
   names  = ['r', 'g', 'b', 'm', 'o', 'c', 'y', 'w']
;  maximum color index
   cimax = n_elements(names)-1

if ss eq 7 then begin
   col = strtrim(color, 2)
   if strlen(col) eq 0 then col = 0 else $
     col = where(strcmp(col, names, 1, /fold_case) eq 1) > 0
endif else col = (round(color-1) < cimax) > 0

      ; Get the color triple.

   r = line_colors[col, 0]
   g = line_colors[col, 1]
   b = line_colors[col, 2]
   returnColor = [r, g, b]
   IF KEYWORD_SET(trueColor) THEN returnColor = COLOR24(returnColor)
   RETURN, returnColor
END
