; ------------------------------------------------------------------------ 
; $Id: gtitle.pro,v 1.3 2007/06/06 20:03:48 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
;+
; NAME:
;       GTITLE
;
; PURPOSE:
;       Write a general title for a plot at the top of the page.
;       User must set the y_margin before making a plot which is to have
;          a general title.
;
; CALLING SEQUENCE:
;
;	GTITLE [, MyTitle] [, /MULTI] [, YMARGIN = ymargin] [, OFFSET = offset]
;            [, CHARSIZE = charsize] [, /SETDEFAULT] 
;
; OPTIONAL INPUTS:
;       MYTITLE:    Title for this plot. This is required if default is not set.
;       MULTI:      If set, the general title is centered on the page.
;                   If not set, the general title is centered over the current
;                     plot window.
;       YMARGIN:    Space above the plot window. Default value for GTITLE is [4,4].
;                   IDL default is [4.000, 2.000] which does not leave room for 
;                   a general title.  On the first call to GTITLE the !y.margin
;                   will be set to [4,4] unless a value is passed.
;             NOTE: This keyword is checked on the first call to gtitle and 
;                   later ignored unless the keyword(SETDEFAULT) is set
;       OFFSET:     Offset from the top of the screen.  Default is 0.1 characters.
;                   Minimum value is .01
;       CHARSIZE:   Character size. Default is 1.5  Minimum value is 0.1
;                   If maximum(!p.multi(1:2)) gt 2 then the size is scaled by 0.5
;       SetDefault: If set then default values of Title, Charsize, and Offset can
;                    be set. The first time GTITLE is called then setdefault is 
;                    assumed to be set.
;                       
; PROCEDURE:
;	XYOUTS is used to print a general title
;
; EXAMPLE:
;       Note: Before making a plot which you wish a general title enter:
;          gtitle   ; sets the y-margin to [4,4] the FIRST time called IF the
;                     margin is set at [4,2]
;   or     !y.margin = [4,4] 
;   or     gtitle, /set, ymargin = [4, 4] ; other parameters may also be set
;
;       Put the title 'X-Ray Analysis' on a plot and make it the default title
;          gtitle, 'X-Ray Analysis', /set
;
;       Print the default title in the center of the plot window.
;          gtitle
;
;       Print the default title in the center of the page
;          gtitle, /mul
;
;       Print the title 'Special Title' with character size = 2
;          gtitle, chars = 2, 'Special Title'
;
; MODIFICATION HISTORY:
; 	Written by:        Paul Mix, June 3, 1997
;-
 pro gtitle,mytitle, multi = multi, ymargin = ymargin, charsize = chars, $
         offset = offset, setdefault= setdefault
  on_error, 1
 common gtitle_common, CHARS_SAVE, OFFSET_SAVE, GENERAL_SAVE
 
 if n_elements (chars_save) eq 0 then begin
   if n_elements (chars) gt 0 then chars_save = chars(0) > 0.1 $
       else chars_save = 1.5
   if n_elements (offset) gt 0 then offset_save = offset(0) > 0.01 $
       else offset_save = 0.1
   if !y.margin(1) eq 2 then begin
     if n_elements (ymargin) gt 1 then !y.margin = ymargin(0:1) $
       else !y.margin(1) = 4.000
   endif
   if n_elements (mytitle) gt 0 then general_save = strtrim(mytitle(0), 2) $
       else begin
       general_save = ''
       return
   endelse
 endif else if keyword_set(setdefault) then begin
   if n_elements (chars) gt 0 then chars_save = chars(0) > 0.1 
   if n_elements (offset) gt 0 then offset_save = offset(0) > 0.01 
   if n_elements (mytitle) gt 0 then general_save = strtrim(mytitle(0), 2)
   if n_elements (ymargin) gt 1 then !y.margin = ymargin(0:1)
 endif
 if !d.window eq -1 and (!d.name eq 'X' or !d.name eq  'MAC' $
       or !d.name eq 'WIN') then return
 
; set a default general title for this directory
 if n_elements(mytitle) eq 0 then title = general_save else $
   title = strtrim(mytitle,2)
 if strlen(title) eq 0 then begin
   Print, '% GTITLE: No title specified'
   return
 endif
 if n_elements(chars) eq 0 then chars = chars_save
 if n_elements(offset) eq 0 then offset = offset_save
 if max(!p.multi(1:2)) gt 2 then charsiz = chars*0.5 else charsiz = chars
 
; write the title
 if keyword_set(multi) then $
     xyouts, .5, 1-!d.y_ch_size*(charsiz*(1.0+offset))/!d.y_size, $
        title, chars = charsiz, /normal, align = 0.5 else $
     xyouts, .5*total(!x.window), $
         1-!d.y_ch_size*(charsiz*(1.0+offset))/!d.y_size, $
        title, chars = charsiz, /normal, align = 0.5

 end
