; ------------------------------------------------------------------------ 
; $Id: header.pro,v 1.4 2010/09/20 14:16:19 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
 Pro header, text, charsize = charsize, space = space, align = alignin, $
        start = start, line = line, title = title, xstart = xstart, $
        ystart = ystart, $
        showdefault = showdefault, setdefault = setdefault, unset = unset
;
;+
; NAME:
;       HEADER
;
; PURPOSE:
;       Print a descriptive header (FOOTER) at the bottom of a graphics
;       window. 
; FORMAT:
;       HEADER [, TEXT] [, CHARSIZE = charsize] [, SPACE = space]
;            [, ALIGN = align] [START = start] [, UNSET = unset] [, LINE=line]
;            [, SHOWDEFAULT = showdefault] [, SETDEFAULT = setdefault]
;            [, /TITLE] [, XSTART = xstart] [, YSTART = ystart]
;  where:
;       TEXT     - optional string array. Typically this will be the date
;                  and file name. DEFAULT ='Date: ' + thin_date()
;                  Note: If SETDEFAULT, SHOWDEFAULT or UNSET is specified 
;                    then DEFAULT value of TEXT is ' '
;       CHARSIZE - Character size for XYOUTS. DEFAULT = 0.6
;       SPACE   - Spacing between lines of text. Default = 1.5*Character Height
;       ALIGN    - Indicates the alignment of the header. 
;                  0 - DEFAULT - Left side of plot window
;                  1 -         - Right side of plot window
;                  0.5 -        - Centered in the plot window
;                  ALIGN is ignored if title is set.
;       START    - Vertical height of bottom line of text in units of 
;                    character height. 
;                  Default = 0.25*Character Height
;                  If title is set, start = 0.5*Character Height
;       LINE     - If set, the header will be placed on a single line with
;                  a separator of ';  ' between elements.
;       UNSET    - If present and not zero, will set all the save values
;                  of keywords to their default values:
;                    CHARSIZE = 0.6 (* Default Value)
;                    SPACE = 1.5 (* Character Height)
;                    ALIGN = 0.0
;                    START = 0.25 (* Character Height)
;                    LINE  = 0    (Each element on a single line)
;       SHOWDEFAULT - Show the present default header keyword values.
;       SETDEFAULT - If present and not zero, then the following parameters will
;                  be saved and used as the default values in all future calls
;                  to HEADER. If a keyword is specified, it will always be used
;                  rather than the saved values.
;       TITLE    - If set, center text over the graph.  Align is set to 0.5
;       XSTART   - X start location in device units
;                  Normally calculated from other parameters
;       YSTART   - Y start location in device units
;                  Normally calculated from other parameters
;       Example:
;           Put the date and file name at the bottom of the current plot
;             STUFF = [ 'Date: ' + thin_date(), 'File: ' + filename]
;             header, stuff
;
;-
common header_defaults, Charsize_save, space_save, align_save, $
            start_save, line_save

on_error, 1
      if keyword_set(unset)  or n_elements (charsize_save) eq 0 then begin
        CHARSIZE_SAVE  = 0.6
        SPACE_SAVE     = 1.5
        ALIGN_SAVE     = 0.0
        START_SAVE     = 0.25
        LINE_SAVE      = 0
      endif
      if keyword_set(setdefault) then begin
        if n_elements(charsize) gt 0 then CHARSIZE_SAVE = charsize(0)
        if n_elements(space) gt 0 then SPACE_SAVE = space(0)
        if n_elements(alignin) gt 0 then ALIGN_SAVE = alignin(0)
        if n_elements(start) gt 0 then START_SAVE = start(0)
        if n_elements(line) gt 0 then LINE_SAVE = line(0)
      endif
      if keyword_set(showdefault) then begin
         print, 'Default parameters for the HEADER command are:'
         print, '    CHARSIZE  = ' , strtrim (CHARSIZE_SAVE, 2)
         print, '    SPACE     = ' , strtrim (SPACE_SAVE, 2)
         print, '    ALIGN     = ' , strtrim (ALIGN_SAVE, 2)
         print, '    START     = ' , strtrim (START_SAVE, 2)
         print, '    LINE      = ' , strtrim (LINE_SAVE, 2)
      endif
      if !d.name eq 'X' and !d.window eq -1 then return
      if n_elements(text) eq 0 then begin
        if (keyword_set (setdefault) or keyword_set(unset) or $
          keyword_set(showdefault)) then return
        text = 'Date: ' + thin_date()
      endif
      if keyword_set(title) then begin
        if n_elements(charsize) eq 0 then charsize = 1.2
        if n_elements(start) eq 0 then START = 0.5
      endif else begin
        if n_elements(charsize) eq 0 then charsize = charsize_save
        if n_elements(start) eq 0 then START = start_save
      endelse
      if n_elements(space) eq 0 then SPACE = space_save
      if n_elements(alignin) eq 0 then ALIGN=align_save else align=alignin
      if n_elements(line) eq 0 then line = line_save
      crowd = max ((!p.multi)(1:2)) -1
      if crowd gt 1 then chars = charsize * 0.6 else chars = charsize
      chy = !d.y_ch_size*chars
      deltay = space*chy
      xsize = !d.x_size
      nel = n_elements (text)-1
      if !p.charsize gt 0 then chars = !p.charsize *chars
      if keyword_set(title) then begin
        ysize = !d.y_size
        align = 0.5
        if n_elements(xstart) eq 0 then xstart=0.5*total(!x.window)*xsize $
            else xstart=xstart[0]
        if n_elements(ystart) eq 0 then ystart=start*chy+!y.window[1]*ysize $
            else ystart=ystart[0]
      endif else begin 
        if n_elements(xstart) eq 0 then xstart=align*xsize else xstart=xstart[0]
        if n_elements(ystart) eq 0 then ystart=start*chy else ystart=ystart[0]
      endelse
      if keyword_set (line) and nel gt 0 then begin
        ttext = text(0)
        for i = 1, nel do ttext = ttext+';  '+ text(i)
        xyouts, xstart, ystart, ttext, align = align, /device, charsize = chars
      endif else $
      for i = 0, nel do xyouts, xstart, ystart+i*deltay, text(nel-i), $
           align = align, /device, charsize = chars
      end

