; ------------------------------------------------------------------------ 
; $Id: histw.pro,v 1.3 2010/01/06 14:34:47 lpmix Exp $
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
 Pro  HISTW, wdf1, wdf2, nbins = nbin, maximum = maximum, minimum = minimum, $
             binsize = binsize, xrange = xrange, plotit = plotit, $
             reverse_indices = reverse_indices, locations = locations
;
;+
; NAME:
;       HISTW
;
; PURPOSE:
;       Generate a histogram for the waveform array amplitudes.
;         This procedure uses the IDL procedure, HISTOGRAM
;
; CALLING SEQUENCE:
;
;      HISTW, WDF1 [, WDF2] [BINSIZE = binsize] [, NBINS = nbins]
;             [, MAXIMUM = maximum] [, MINIMUM = minimum] [, /PLOTIT ]
;             [, XRANGE = xrange]
;             [, REVERSE_INDICES = reverse_indices] [, LOCATIONS = locations]
;
; 
;
; INPUTS:
;       WDF1;   Input WDF array to be analyzed
;
; OPTIONAL INPUTS:
;       WDF2:   Output WDF array
;               Default:  WDF2 = GET_MAXWDF()-1
;        Note:  X values are the median values for LOCATIONS
;               IDL histogram uses the minimum value of each bin and these 
;               values are returned in LOCATIONS.
;               Additional Zeros are added to the return from HISTOGRAM to 
;               allow a nice graph with psym= 10
;	
; KEYWORD PARAMETERS:
;       Note:  Some of the defaults are set by HISTW
;
;       BINSIZE: Size of BINS
;       NBINS:  Number of bins
;               Default: NBINS = (NPOINTS/50 + 2) < 1000
;                 where NPOINTS = the number of points in the WDF array
;       MAXIMUM: Maximum value to consider for the histogram.  This value is
;                increased by 1.0e-6 before being passed to histogram.
;          Note: MAXIMUM is ignored if both BINSIZE and NBIN are specified.
;          Default: MAXIMUM = MX(WDF1)
;       MINIMUM: Minimum value to consider for the histogram.
;           Default:  MINIMUM = MN(WDF1)
;       PLOTIT:  If set, WDF2 will be plotted.
;                "plo, wdf2, psym= 10, nsym =0"
;       XRANGE:  If two values are provided, only ordinates whose abscissa are 
;                in the specified range are included in the histogram.
;                At least two values must be provided.
;                If there are less than two valid ordinate values, the histogram 
;                will not be calculated.
;
; The following are output if a variable is provided.
;       REVERSE_INDICES: See HISTOGRAM for explanation
;       LOCATIONS: BIN boundaries
;                  If H = getyf(WDF2) then H[i] is the number of values, VAL, 
;                  which satisfies: LOCATIONS[i] less/equal V less LOCATIONS[i+1]
;          Note:  LOCATIONS will be returned, but cannot be set.
;
; OUTPUTS:
;       WDF array with the histogram distribution function
;
; OPTIONAL OUTPUTS:
;       REVERSE_INDICES and LOCATIONS may be returned.
;
; PROCEDURE:
;       This procedure uses the IDL procedure: HISTOGRAM
;
; EXAMPLE:
;       Get a histogram of the amplitude of wdf array 3 and store it in 33
;          HISTW, 3, 33
;
;       Calculate the histogram of the amplitude of wdf array 5 between values
;         of 500E3 and 2E6 with a bin size of 1E5.  Store the result in 75
;         and plot the histogram.
;         HISTW, 5, 75, min = 5e5, bin = 1e5, max = 2e6, /plotit
;
; MODIFICATION HISTORY:
;       Written by:     L. P. Mix, Jr., March 30, 2005
;
;-
;
@com.dat
;
      on_error, 1
      if n_elements(initwdf) eq 0 then wdfinit
      nparams = n_params()
      if nparams lt 1 then begin
print,'% Format: HISTW, WDF1 [, WDF2] [BINSIZE = binsize] [, NBINS = nbins]'
print,'         [, MAXIMUM = maximum] [, MINIMUM = minimum] [, /PLOTIT ]'
print,'         [, XRANGE = xrange]'
print,'         [, REVERSE_INDICES = reverse_indices] [, LOCATIONS = locations]'
        return
      endif
;
      if n_elements(wdf1) eq 0 then begin
        print, '% HISTW: Parameters must be defined'
        return
      endif
      if wdf1 lt 1 or wdf1 gt maxwdfarray then begin
        print, '% HISTW: WDF array must be greater than 0 and </=', maxwdfarray
        return
      endif
;
      ww = maxwdfarray-1
      if nparams gt 1 then save = get_save_array(wdf2, ww) else save = ww
      if save lt 1 or save gt maxwdfarray then save = ww
      if wdfnp(wdf1-1) eq 0 then begin
        print, '% HISTW: No valid data in the array'
        return
      endif
      y= getyf(wdf1)
      if n_elements(xrange) gt 1 then begin
        mnx = min(xrange, max = mxx)
        x = getxf(wdf1)
        ok = where(x ge mnx and x le mxx, nok)
        if nok gt 0 then y = y[ok]
        if nok lt 3 then begin
          print, '% HISTW: Less than three values in given XRANGE'
          print, '         XRANGE = [',strtrim(mnx, 2),',',strtrim(mxx,2), ']'
          if nok gt 0 then print, '         Y values = ', y
          return
        endif
        clab = ' - Range=['+strtrim(mnx,2)+', '+strtrim(mxx,2)+']'
      endif else clab =''
      if n_elements(minimum) eq 0 then minimum = min(y)
      fudg = 1.000001
      if n_elements(maximum) eq 0 then maxi = max(y) else maxi = maximum[0]
      maxi = maxi*fudg
      if n_elements(binsize) gt 0 then binsize = abs(binsize[0]) else binsize =0
      if n_elements(nbin) gt 0 then nbin = abs(nbin[0]) else nbin =0
      if binsize gt 0 && nbin gt 0 then maxi = minimum+binsize*nbin else $
        if nbin gt 0 then binsize = (maxi - minimum)/nbin 
      if binsize eq 0 then begin
        nbin = (n_elements(y)/50 +2) < 1000
        binsize = (maxi - minimum)/nbin
      endif
      dis = histogram(y, binsize = binsize, min = minimum, max = maxi, $
          locations = locations, reverse_indices = reverse_indices)
      ndis = n_elements(dis)
      dis = (dis[ndis-1] eq 0)? [0L, dis] : [0L, dis, 0L]
      put_wdfarray, [total(locations[0:1])*0.5 - binsize, binsize, dis], $
           save, /no_copy, xydata =0
      wdf2 = save
      wdfcomment(save-1)= 'Histogram of '+wdfcomment(wdf1-1) + clab
      wdffile(save-1)= wdffile(wdf1-1)
      wdfxlabel(save-1)= wdfylabel(wdf1-1)  
      wdfylabel(save-1)= 'Number of Values'
      if keyword_set(plotit) then plo, save, psym=10, nsym=0
      end
