; ------------------------------------------------------------------------ 
; $Id: joinw.pro,v 1.4 2010/09/17 21:20:34 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
 Pro  joinw, wdf1, wdf2, wdf3, value = value, xydata= xydata, $
      alldata = alldata,  delta = delta
;
;+
; NAME:
;       JOINW
;
; PURPOSE:
;       Join one WDF array to another WDF array
;       Store the result in the initial WDF array or another WDF array
;       Name changed to joinw to avoid IDL conflict in Statistical library
; CATEGORY:
;       Math 
; CALLING SEQUENCE:
;       JOINW, WD1, WD2 [, WD3] [, VALUE = VALUE] [, XYDATA = xydata]
;                     [, DELTA = delta] [, /QUIET] [, /ALLDATA]
; INPUTS:
;       WD1:      WDF array number of the first array
;       WD2:      WDF array number of array to be joined to first array
; OPTIONAL INPUT:
;       WD3:      WDF array number for the composite; if not specified the 
;                 composite is stored in WD1 the first array.
;       ALLDATA:  Ignore the remaining keywords.  All of the data in WD1 and WD2
;                 is combined and sorted in X.  The result is stored as XY DAta
;       VALUE:    If specified, the first array will be terminated at VALUE 
;                   and the other array will be ignored before the abscissa
;                    value of VALUE.
;                 If either array has less than 3 data values, a warning
;                   will be printed unless quiet is set. 
;                 If VALUE is not specified, result will be a combination 
;                   of the two arrays in the region of overlap.  This is the 
;                   same as /ALLDATA unless DELTA is set.
;                   Values will be sorted and stored as XY data unless delta
;                   is set. 
;       XYDATA:   If set data will be stored as xydata. Also if either array
;                 is xydata or if VALUE is not specified, the result is XYDATA
;                 unless delta is set.
;       Delta:    If set, xydata = 0 and data will be put on a uniform time with
;                 a spacing of delta regardless of other items.
; OUTPUTS:
;       NONE    The sum is stored in a WDF array in the common block
; COMMON BLOCKS:
;       COM.DAT  WDF common block 
; RESTRICTIONS:
;       WD1 and WD2 arrays must be defined and have valid data
; EXAMPLES:
;       Join WDF arrays 2 and 3 together and store the result in WDF array 8
;               JOINW, 2, 3, 8
;       Join WDF arrays 3 and 5 using one array up to 1.0 and the other 
;       beyond 1.0 and store the result in WDF array 9
;       in WDF array 8.
;               JOINW, 3, 5, 8, value = 1.0
; MODIFICATION HISTORY:
;      Original version, L. P. Mix, May 15, 1992
;     modified 
;-
;
@com.dat
;
      on_error, 1
      if n_elements(initwdf) eq 0 then wdfinit
      nparams = n_params()
      if nparams lt 2 then begin
print,'% Format: JOINW, WD1, WD2 [, WD3] [, VALUE = value] [, XYDATA = xydata]'
        return
      endif
;
      if n_elements(wdf1) eq 0 or  n_elements(wdf2) eq 0 then begin
        print, 'Parameters to JOIN must be defined'
        return
      endif
      if wdf1 lt 1 or wdf1 gt maxwdfarray then begin
        print, 'wdf array must be greater than 0 and </=', maxwdfarray
        return
      endif
;
      if nparams gt 2 then save = get_save_array(wdf3, wdf1) else save = wdf1
;
       np1 = wdfnp(wdf1-1)
      if np1 eq 0 then begin
        print, 'No data in the first array'
        return
      endif
;
      if wdf2 lt 1 or wdf2 gt maxwdfarray then begin
        print, 'wdf array must be greater than 0 and </=', maxwdfarray
        return
      endif
      np2 = wdfnp(wdf2-1)
      if np2 eq 0 then begin
        print, 'No data in the second array'
        return
      endif
    if keyword_set (alldata) then begin
      w2i, x, y, wdf1
      w2i, xx, yy, wdf2
      x = [x, xx]
      y = [y, yy]
      a= sort(x)
      x = x(a)
      y = y(a)
      i2w, x, y, save, /xy
    endif else begin
      if n_elements (delta) gt 0 then delta = abs(delta(0)) else delta = 0
      if delta gt 0 then xydata = 0
      get_wdfarray, array1, wdf1, /xydata
      nel1 = n_elements(array1)
      get_wdfarray, array2, wdf2, /xydata
      x = make_array(np1+np2)
      y = make_array(np1+np2)
      nel2 = n_elements(array2)
      nelv = n_elements(value)
      if nel1 eq (np1+2) then begin
          delta1 = array1(1)
          if nelv eq 0 then begin
            x(0:np1-1) = findgen(np1) *array1(1)+array1(0)
            y(0:np1-1) = array1(2:*)
            nok1 = np1
          endif else begin
            if value lt array1(0) then nok1 = 0L else begin
              if value ge wdfmaxx(wdf1-1) then nok1 = np1 else $
              nok1 = long((value-array1(0))/array1(1)+1)
              x(0:nok1-1) = findgen(nok1)*array1(1) + array1(0)
              y(0:nok1-1) = array1(2:nok1+1)
            endelse
          endelse
      endif else begin
          delta1 = 0
          if nelv eq 0 then  begin
            x(0:np1-1) = array1(0:np1-1)
            y(0:np1-1) = array1(np1:np1+np1-1)
            nok1 = np1
          endif else begin
            ok = where (array1(0:np1-1) le value, nok1)
            if nok1 gt 0 then begin
              a = sort(array1(ok))
              ok = ok(a)
              x(0:nok1-1) = array1(ok)
              y(0:nok1-1) = array1(np1+ok)
            endif
          endelse
      endelse
      if nel2 eq (np2+2) then begin
          delta2 = array2(1)
          if nelv eq 0 then begin
            nok2 = np2
            x(nok1:*) = findgen(np2)*array2(1)+array2(0)
            y(nok1:*) = array2(2:*)
          endif else begin
            if value gt wdfmaxx(wdf2-1) then nok2 = 0 else begin
              if value lt wdfminx(wdf2-1) then st = 0L  else $
              st = long((value-array2(0))/array2(1)+1)
              nok2 = np2-st>1
              st = np2-nok2
              x(nok1:nok1+nok2-1) = (findgen(nok2)+st)*array2(1) +  array2(0)
              y(nok1:nok1+nok2-1) = array2(2+st:np2+1)
            endelse
          endelse
      endif else begin
        delta2 = 0
        if nelv eq 0 then  begin
          x(np1:*) = array2(0:np2-1)
          y(np1:*) = array2(np2:np2+np2-1)
          nok2 = np2
        endif else begin
          ok2 = where (array2(0:np2-1) gt value, nok2)
          if nok2 gt 0 then begin
            a = sort(array2(ok2))
            ok2= ok2(a)
            x(nok1:nok1+nok2-1) = array2(ok2)
            y(nok1:nok1+nok2-1) = array2(np2+ok2)
          endif
        endelse
      endelse
      nt = nok1+nok2
      if nt gt 2 then begin
        x = temporary(x(0:nt-1))
        y = temporary(y(0:nt-1))
      endif else begin
          print, '% JOINW: Less than 3 points to be joined. NO JOIN will be done.'
          return
      endelse

; no specified value
      if nelv eq 0  then begin
        a= sort(x)
        x = x(a)
        y = y(a)
        if delta eq 0 then i2w, x, y, save, /xy  else $
          i2w, x, y, save, delta= delta
      endif else begin
        if nok1 lt 3 then print, $
         '% JOINW: Less than 3 points for array 1 - number = ', strtrim(nok1, 2)
        if nok2 lt 3 then print, $
         '% JOINW: Less than 3 points for array 2 - number = ', strtrim(nok2, 2)

        if delta eq 0 then begin
          if nok1 gt 0 and nok2 gt 0  then delta = min ([delta1, delta2]) else $
            if nok1 gt 0 then delta = delta1 else delta = delta2
        endif
        if keyword_set(xydata) or delta eq 0.0 then begin
            xydata = 1
            array = [x, y]
        endif else begin  
          xydata = 0
          st = x(0)
          stopp = x(nt-1)
          if nok1 gt 0 and nok2 gt 0 then begin
            put2uniform, x, y, nt,st, delta, stopp, array
          endif else if nok1 gt 0 then begin 
            if ((delta1 gt 0) and (delta eq delta1)) then $
              array = [array1(0:1), y] else $
              put2uniform, x, y, nt, st, delta, stopp, array
          endif else begin
            if ((delta2 gt 0) and (delta eq delta2)) then $
              array = [array2(0:1), y] else $
              put2uniform, x, y, nt, st, delta, stopp, array
          endelse
        endelse
        put_wdfarray, array, save , /no_copy, xydata = xydata
      endelse
    endelse
      wdf3 = save
      wdffile(save-1) = wdffile(wdf1-1)
      wdfcomment(save-1)='Join of '+wdfcomment(wdf1-1)+' and '+wdfcomment(wdf2-1)
      wdfxlabel(save-1)= wdfxlabel(wdf1-1)
      wdfylabel(save-1)= wdfylabel(wdf1-1)
      array1=0
      array2=0
      
      sp=0
      im=0
      return
      end
