; ------------------------------------------------------------------------ 
; $Id: lim.pro,v 1.5 2010/07/14 16:34:20 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
 Pro  lim, wdfi, starti, stopi, yaxis = yaxis, keep = keep, $
          wdf = wdfp, ierror = ier
;+
;  Limit the range or domain of a WDF array.  Limits on the domain 
;  of the function are approximate; the domain is reduced to the first
;  data value with abscissa greater than start and the last data value
;  less than stop.
;  format: LIM, WDA, START [, STOP]  [, /YAXIS] [, /KEEP] [, IERROR = ierror]
;               [, WDF = WDF]
;  where:
;    WDA   - WDF array number, a search string,  or an array of array numbers 
; 
;    START - Lower limit for the array(s)
;            or an array of two or more values.
;            If START is an array, then START = MIN(START) & STOP = MAX(START)
;    STOP  - Upper limit for the array(s) (Ignored if START has 2 or more values.)
;    YAXIS - If nonzero, then the range of the WDF array is modified
;            If zero or missing, then the domain of the WDF array is modified
;    KEEP  - If Y-axis is set, and if keep is greater than zero
;              all values out of range will be set to the nearest extreme.
;            If Y-axis is set, and if keep is less than zero
;              all values out of range will be dropped.
;            If keep is missing or zero, then waveform will not start until the 
;              first valid point, and will end at the last valid point.  Points
;              out of range will be set to the limit.
;    WDF   - Actual waveforms that are filtered 
;    IERROR- Error parameter. If no error, return zero = 0
;            If WDF is an array, IERROR will be an array.
;
;  Examples:
;    Limit array 2 to a range of y-values between 0 and 20. 
;      lim, 2, 0, 20, /y
;    Limit array 2 to abscissa values of 0 to 200 ns.
;      lim, 2, 0, 200e-9
;    Limit arrays 2 to 5 to abscissa values between begin and end
;      lim, [2,3,4,5], begin, end
; or:  lim, indgen(4)+2, begin, end
;    Limit all arrays with the name 'mocam' to points selected by zoomw, 1.
;      zoomw, 1, xcur= xx
;      lim, 'mocam', xx
;
;-
;     include the common block specification
;
@com.dat
;
on_error, 1
;
      ier = 1
      if n_elements(initwdf) eq 0 then wdfinit
      if (n_params() eq 0 ) then begin
        print, '% Format: LIM, WDF, START, STOP [, /YAXIS]'
        return
      endif
      if n_elements(wdfi) eq 0 then begin
        print, '% LIM: WDF array must be specified'
        return
      end
      wdfp = wdvalid(wdfi, /valid, /quiet, npoint= np)
      if np eq 0 then begin
        print, '% LIM: No valid WDF arrays are specified'
        return
      endif
      if np gt 1 then ier = make_array(np, /int, value=1)
      if n_elements (starti) eq 0 then begin
        print, '% LIM: Start values must be specified'
        return
      endif
      if n_elements(starti) gt 1 then start = min(starti, max = stopp) $
        else begin
        if n_elements(stopi) eq 0 then begin
          print, '%LIM: Stop values must be specified'
          return
        endif
        start = min([starti(0), stopi(0)], max = stopp)
      endelse
      narray = n_elements(wdfi)
      if n_elements(keep) eq 0 then keep = 0 else keep = keep[0]
      if n_elements(yaxis) eq 0 then yaxis = 0
      for indval = 0, narray-1 do begin
        wdf = wdfp(indval)
        nel = wdfnp[wdf-1]
        get_wdfarray, array, wdf, /xydata
;  XYDATA
        if n_elements(array) eq nel+nel then begin
          if yaxis eq 0 then begin
            space = array(0:nel-1)
            ok = where(space ge start and space le stopp, nok)
            if nok lt 2 then print, '% LIM: No valid data in the window' $
              else begin
              space = space(ok(0):ok(nok-1))< stopp
              space = space > start
              array = [space, array(nel+ok(0):nel+ok(nok-1))]
              put_wdfarray, array, wdf, /xydata, /no_copy
              ier[indval] =0
              space =0
            endelse
          endif else begin
            if keep gt 0 then begin 
              array[nel:*]=(array[nel:*] < stopp) > start
              put_wdfarray, array, wdf, /xydata, /no_copy
              ier[indval] =0
            endif else begin
              ok = where(array[nel:*] ge start and array[nel:*] le stopp, nok)
              if nok lt 2 then print, '% LIM: No valid data in the window' $
              else begin
                if keep lt 0 then array = [array[ok], array[nel+ok]] $
                else begin
                  image = (array[nel+ok(0):nel+ok(nok-1)]< stopp) > start
                  array = [array[ok(0):ok(nok-1)], image]
                  image=0
                endelse
                put_wdfarray, array, wdf, /xydata, /no_copy
                ier[indval] =0
              endelse
            endelse
          endelse
        endif else begin
;  Waveform Data
          if yaxis eq 0 then begin
            space = array(0)+ findgen(nel)*array(1)
            ok = where(space ge start and space le stopp, nok)
            if nok lt 2 then print, 'No valid data in the window' else begin
              put_wdfarray, [space(ok(0)), array(1), array(2+ok)], wdf, /no_copy
              ier[indval] =0
              space =0
            endelse
          endif else begin
            if keep gt 0 then begin 
              array[2:*]=(array[2:*] < stopp) > start
              put_wdfarray, array, wdf, /no_copy
              ier[indval] =0
            endif else begin
              ok = where(array[2:*] ge start and array[2:*] le stopp, nok)
              if nok lt 2 then print, '% LIM: No valid data in the window' $
              else begin
                space = array(0)+ findgen(nel)*array(1)
                if keep lt 0 then begin
                  put_wdfarray,  [space[ok], array[2+ok]], wdf, /xydata, /no_copy
                  ier[indval] =0
                endif else  begin
                  array=[space[ok[0]],array[1],(array[2+ok[0]:2+ok[nok-1]] < stopp) > start ]
                  put_wdfarray, array, wdf, /no_copy
                  ier[indval] =0
                endelse
                space =0
              endelse
            endelse
          endelse
        endelse
      endfor
      end
