; ------------------------------------------------------------------------
; $Id: line_rm.pro,v 1.3 2007/06/06 20:03:49 lpmix Exp $
; ------------------------------------------------------------------------
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
;+
; NAME:
;       LINE_RM
;
; PURPOSE:
;	This routine removes a portion of a waveform.
;       The intent is to allow line removal from otherwise smooth spectra.
;       The total length and the point spacing will not be modified.
;       See the example below for calculating the area of a line.
;       (WDEDIT can be used for interactive line removal.)
;
; CATEGORY:
;	Analysis
;
; CALLING SEQUENCE:
;       LINE_RM, WDF [, VALUE] [, WIDTH = width] [, SAVE = save]
;  
;    WHERE:
;       WDF     Waveform array number or a search string for a unique array
;       VALUE   Abscissa value for the middle of the removal area.
;               This is optional if WIDTH has two values.
;       WIDTH   Width of the removal area.  
;               If width is negative and less than 1, it will be used as 
;                 a multiplier that is WIDTH = abs(WIDTH)* VALUE
;               If width is positive, it will be the total width of the 
;                 region with value as the middle.
;               If width has two values, VALUE is ignored and the two values 
;                 are used as the lower and upper values for removal.
;               Default:  WIDTH = 0.01*VALUE
;       SAVE    WDF array number for the modified array
;               Default: SAVE = WDF
;
; PROCEDURE:
;       All abscissa values greater than (VALUE - 0.5*WIDTH) and less than
;         (VALUE + 0.5*WIDTH) are deleted.
;       Other abscissa values are not changed.
;       Regardless the value of WIDTH, the first and last points and a
;         minimum of 3 points will be kept.
;
; EXAMPLE:
;       Eliminate data values in a 1% region around 2008 from WDF array 5
;         LINE_RM, 5, 2008
;
;       Eliminate values of waveform 12 between 1000 and 1012.
;         LINE_RM, 12, WIDTH = [1000, 1012]
;       
;       Calculate the area of a spectral line in waveform 11
;       1.  Specify the extent of the line
;           ZOOMW, 11, xcursor = xc
;       2.  Remove the line from the original waveform
;           LINE_RM, 11, SAVE = 12, WIDTH = xc
;       3.  Subtract the background from the line; use the original waveform
;           abscissa values
;           sub, 12, 11, 13, master = 2
;       4.  Calculate the area
;           int, 13 & print, 'Integral of the line =', mx(13)
;        
; MODIFICATION HISTORY:
; 	Written by:     L. P. Mix, August 31, 2004
;-
PRO LINE_RM, wdfin, value, width= width, save = saveitin
;  width_def = 0.01
  width_def_2 = 0.005
  on_error, 1
  npar = n_params()
  if (npar eq 0 ) then begin
        print, '% Format:  LINE_RM, WDF [, VALUE] [, WIDTH = width] [, SAVE = save]'
        return
  endif
  wdf = wdvalid(wdfin, /valid, /quiet, npoint= np)
  if np eq 0 then begin
    print, '% LINE_RM: No valid WDF arrays are specified'
    return
  endif
  if np gt 1 then begin
    print, '% LINE_RM: A unique waveform must be specified.'
    return
  endif else wdf = wdf[0]
  if n_elements (saveitin) gt 0 then begin
    saveit = get_save_array(saveitin[0], wdf)
    if saveit ne wdf then xfr, wdf, saveit  
    wdf = saveit
  endif
  nw = n_elements (width)
  if nw gt 0 then begin
    if nw eq 1 then begin
      if n_elements(value) eq 0 then begin
        print, '% LINE_RM: Center value must be specified'
        return 
      endif else value = value[0]
      if width lt 0 and width gt -1 then width = 0.5*abs(width*value) else $
        width = 0.5*abs (width)
      lo = value - width
      hi = value + width
    endif else begin
      lo = min(width, max = m)
      hi = m
    endelse
  endif else begin
    if n_elements(value) eq 0 then begin
       print, '% LINE_RM: Center value must be specified if width not defined'
       return
    endif else value = value[0]
    lo = (1.0-width_def_2)*value
    hi = (1.0+width_def_2)*value
  endelse
  w2i, x, y, wdf
  np = n_elements(x)
  ok = where (x le lo, nok)
  if nok eq 0 then ok = 0
  okh = where (x ge hi, nok2)
  if nok2 eq 0 then okh = np-1
  if (nok + nok2) lt 3 then ok =[0,1]
  ok = [ok, okh]
  x = x[ok]
  y = y[ok]
  put_wdfarray, [x, y], wdf, /xydata
  

END
