; ------------------------------------------------------------------------ 
; $Id: loadxyct.pro,v 1.8 2010/03/03 17:35:13 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
  pro loadxyct, p1, quiet = quiet, freeze = freeze, setdefault = setdefault, $
             showdefault = showdefault, reset = reset, $
             list_colors = list_colors, current = current, $
             unset= unset, background = background, color = color
;+
;    Loads the colors for an x,y plot
;    Default values may be changed using PUTCOLOR
; format: loadxyct, color_table, quiet = quiet, freeze = freeze, $
;       setdefault = setdefault, showdefault = showdefault, reset = reset, $
;       list_colors = list_colors, CURRENT = current, $
;       unset = unset, background = background, color = color
;
;  where: 
;      COLOR_TABLE - Number or string defining the desired color table
;                    The string is not case sensitive and should have enough
;                    characters to uniquely define the table.
;              Note: Several tables are impossible to uniquely specify by name
;                    (11 and 33), (19 and 20)
;      FREEZE - If set, color table is not modified.
;      CURRENT - If ARG_PRESENT(Current) then return the current color table number
;                This is performed after the default keywords are processed, but
;                no other action is taken (colors are not modified).
;      QUIET  - If set, no messages are printed.
;      SetDefault - Set Initial or final color in the color table
;                   Default - Background = [0,0,0]
;                             Color = [255, 255, 255]
;      ShowDefault - Show background and color defaults
;      Reset/Unset - Return initial and final colors to the defaults
;      LIST_COLORS - list the colors available
;
;   Load a color table and set the bottom 9 colors as shown below
;   If color table is not specified then the last color table specified
;   will be loaded.
;   These 8 colors may be set using the routine PUTCOLOR:
;     PUTCOLOR, colors
;       where colors is colors[8,3]
;
;   Default colors are 
;      1   RED       [255,   0,   0]
;      2   GREEN     [  0, 255,   0]
;      3   BLUE      [  0,   0, 255]
;      4   MAGENTA   [255,   0, 255]
;      5   ORANGE    [255, 125,   0]
;      6   CYAN      [  0, 255, 255]
;      7   YELLOW    [255, 255,   0]
;      8   Grey      [150, 150, 150] 
;   The color tables available in IDL 7.0 are:
;  0-        B-W LINEAR    14-             STEPS    28-         Hardcandy
;  1-        BLUE/WHITE    15-     STERN SPECIAL    29-            Nature
;  2-   GRN-RED-BLU-WHT    16-              Haze    30-             Ocean
;  3-   RED TEMPERATURE    17- Blue - Pastel - R    31-        Peppermint
;  4- BLUE/GREEN/RED/YE    18-           Pastels    32-            Plasma
;  5-      STD GAMMA-II    19- Hue Sat Lightness    33-          Blue-Red
;  6-             PRISM    20- Hue Sat Lightness    34-           Rainbow
;  7-        RED-PURPLE    21-   Hue Sat Value 1    35-        Blue Waves
;  8- GREEN/WHITE LINEA    22-   Hue Sat Value 2    36-           Volcano
;  9- GRN/WHT EXPONENTI    23- Purple-Red + Stri    37-             Waves
; 10-        GREEN-PINK    24-             Beach    38-         Rainbow18
; 11-          BLUE-RED    25-         Mac Style    39-   Rainbow + white
; 12-          16 LEVEL    26-             Eos A    40-   Rainbow + black
; 13-           RAINBOW    27-             Eos B
;
;   Example:
;     Load color table 5
;       LOADXYCT, 5
;    or
;       LOADXYCT, 'STD GAMMA'
;  
;-
on_error, 1

@loadxyct_common.dat

; Loadxyct common block
; common loadxyct_common, loadct_init, max_color ,invertct_flag, $
;     color_maximum, color_minimum, line_colors, color_names
;     
      if n_elements(loadct_init) eq 0 then begin
         loadct, get_names = names
         max_color = n_elements(names)
         color_names = temporary(names)
         invertct_flag = 0
         color_maximum = make_array(1, 3, /byte, value =255b)
         color_minimum = make_array(1, 3, /byte, value =0b)
         loadct_init = 5
         if n_elements(line_colors) eq 0 then putcolor
         a = get_nice_values ([0, 1])
         if !d.name ne 'NULL' then loadct, 5, silent = quiet
; get_nice_values is called to make sure that we set the number of colors
      endif
      if !d.name eq 'NULL'  then return
      if keyword_set(reset) or keyword_set(unset) then begin
         color_maximum = make_array(1, 3, /byte, value =255b)
         color_minimum = make_array(1, 3, /byte, value =0b)
      endif
      if keyword_set(setdefault) then begin
        if n_elements(background) eq 3 then color_minimum(0,*)=byte(background)
        if n_elements(color) eq 3 then color_maximum(0,*) = byte(color)
      endif
      if keyword_set(showdefault) then begin
        print, 'First color_table value is:', $
        color_minimum[0], color_minimum[1], color_minimum[2]
        print, 'Last color_table value is:', $
        color_maximum[0], color_maximum[1], color_maximum[2]
      endif
      if arg_present(current) then begin
        current = loadct_init
        return
      endif
      if keyword_set(list_colors) then begin
        ntables = max_color
        names = color_names
        nlines = (ntables + 2) / 3      ;# of lines to print
        nend = nlines - ((nlines*3) - ntables)
        for i=0, nend-1 do $            ;Print each line
          print, format="(i2,'- ',a17, 3x, i2,'- ',a17, 3x, i2,'- ',a17)", $
            i, names[i], i+nlines, names[i+nlines], i+2*nlines < (ntables-1), $
                names[i+2*nlines < (ntables-1)]
        if (nend lt nlines) then begin
          for i=nend, nlines-1 do $
            print, format="(i2,'- ',a17, 3x, i2,'- ',a17)", $
              i, names[i], i+nlines, names[i+nlines]
        endif
      endif
      if n_elements(p1) eq 0 then p2  = loadct_init else begin
        if size(/type, p1) eq 7 then begin
          ok = get_match(p1[0], color_names, nel = nel)
          if nel eq 1 then p2 = ok[0] else begin
            if nel eq 0 then $
              print, '% LOADXYCT: Unable to get a match with: "'+p1+'"' $
            else begin
              print, '% LOADXYCT: Multiple matches with: "'+p1+'"'
              for i = 0, nel-1 do print, ok[i], ' ', color_names[ok[i]]
            endelse
            print, ' USE: LOADXYCT, /LIST see tables available'
            return
          endelse
        endif else begin
          if (p1 lt 0) || (p1 gt max_color) then p2 = loadct_init else p2 = p1
        endelse
      endelse
      ok = where (!d.name eq ['X', 'WIN', 'MAC'], nok)

;     following for LINUX is due to streaking in 5.5

      if nok gt 0 then begin
        name = strupcase(strmid(!version.os, 0, 3))
        if (name eq 'LIN') then device, decomposed = 0, bypass =1 $ 
           else device, decomposed = 0
      endif
      if keyword_set(freeze) then begin
         tvlct, /get, r, g, b
         r(1:8) = line_colors[*, 0]
         g(1:8) = line_colors[*, 1]
         b(1:8) = line_colors[*, 2]
         tvlct, r, g, b
         return
      endif
      loadct, p2, silent = quiet
      loadct_init = p2
      nm = !d.table_size-1
      tvlct, color_minimum, 0
      tvlct, color_maximum, nm
      if nm lt 10 then begin
        if keyword_set(quiet) then return
        print, '% LOADXYCT: XY Color table will not be loaded'
        Print, '            Insufficient number of colors', !d.table_size
        return
      endif 
      tvlct, /get, r, g, b
      r(1:8) = line_colors[*, 0]
      g(1:8) = line_colors[*, 1]
      b(1:8) = line_colors[*, 2]
      if invertct_flag ne 0 then begin
;        c = r[0] & r[0] = r[nm] & r[nm] = c
;        c = g[0] & g[0] = g[nm] & g[nm] = c
;        c = b[0] & b[0] = b[nm] & b[nm] = c
        if invertct_flag gt 0 then begin
          top = nm-1
          r(9:top) = reverse(r(9:top))
          g(9:top) = reverse(g(9:top))
          b(9:top) = reverse(b(9:top))
        endif
      endif    
      tvlct, r, g, b
      r=0
      g = 0
      b =0
      end
