; ------------------------------------------------------------------------ 
; $Id: mirror.pro,v 1.3 2007/06/06 20:03:49 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
 Pro  mirror, wdf1, wdf2, tmirror = tmirror, join = join
;
;+
; NAME:
;       MIRROR
;
; PURPOSE:
;       Mirror a WDF array about its initial point or a user-specified 
;         abscissa value
;       Store the result in the initial WDF array or another WDF array
;       The number of points is doubled according to the following:
;       New_x = 2*Tmirror -reverse(x)
;       New_y = reverse(y)
; CATEGORY:
;       Math 
; CALLING SEQUENCE:
;       MIRROR, WD1, [, WD2] [, TMIRROR = tmirror]
; INPUTS:
;       WD1:      WDF array number of the first array
; OPTIONAL INPUT:
;       WD2:      WDF array number for the composite; if not specified the 
;                 composite is stored in WD1 the first array.
;       TMIRROR:  Absciaas value for mirror symmetry. 
;                 Default: Minimum abscissa value
;       JOIN:     Join the New_X and New_Y to the old values
;                 Default is JOIN = 0 - Return only the mirror values
;
; OUTPUTS:
;       NONE    The mirror is stored in a WDF array in the common block
; COMMON BLOCKS:
;       COM.DAT  WDF common block 
; RESTRICTIONS:
;       WD1 arrays must be defined and have valid data
; EXAMPLES:
;       Mirror WDF array 2 and store the result in WDF array 8
;               MIRROR, 2, 8
;
;       Mirror array 5 about t = 2.0e-9
;               MIRROR, 5, T=2.0e-9
; MODIFICATION HISTORY:
;      Original version, L. P. Mix, October 20, 1993
;      Modified to add tmirror, November 21, 1994, lpm/dbs
;-
;
@com.dat
;
      on_error, 1
      if n_elements(initwdf) eq 0 then wdfinit
      nparams = n_params()
      if nparams lt 1 then begin
        print, $
          '% Format: MIRROR, WD1 [, WD2] [, TMIRROR = tmirror] [JOIN = join]'
        return
      endif
;
      if n_elements(wdf1) eq 0  then begin
        print, 'Parameter to MIRROR must be defined'
        return
      endif
      if wdf1 lt 1 or wdf1 gt maxwdfarray then begin
        print, 'wdf array must be greater than 0 and </=', maxwdfarray
        return
      endif
;
      if nparams gt 1 then save = get_save_array(wdf2, wdf1) else save = wdf1
;
      np = wdfnp(wdf1-1)
      if np eq 0 then begin
        print, 'No data in the first array'
        return
      endif
;
      if n_elements (save) eq 0 then save = wdf1
      if save lt 1 or save gt maxwdfarray then save = wdf1
      if n_elements (tmirror) eq 0 then tmirror = wdfminx(wdf1-1)
      get_wdfarray, array, wdf1, /xydata
      if n_elements (array) eq np+np then begin
        if keyword_set(join) then begin
          array = [array(0:np-1), array(0:np-1), temporary(array)]
          array(2*np: 3*np-1) = reverse(array(3*np:*))
          array(0:np-1)= tmirror*2-reverse(array(0:np-1))
        endif else begin
          array(np:*) = reverse(array(np:*))
          array(0:np-1) = tmirror*2-reverse(array(0:np-1))
        endelse
        put_wdfarray, array, save, /xydata, /no_copy
      endif else begin
        if keyword_set(join) then begin
          image = [reverse (array(2:*)), array(2:*)]
          space = array(0)+findgen(np)*array(1)
          space = [tmirror*2-reverse(space), space]
          put_wdfarray, [space, image], save, /xydata
          space = 0
          image = 0
        endif else begin  
          array(0) = 2*tmirror-array(0)-(np-1)*array(1)
          array(2:*)= reverse (array(2:*))
          put_wdfarray, array, save, /no_copy
        endelse
      endelse
      wdf3 = save
      wdffile(save-1) = '  '
      wdfcomment(save-1)= $
          'Mirror at '+strtrim (tmirror, 2)+' of '+wdfcomment(wdf1-1) 
      wdfxlabel(save-1)= wdfxlabel(wdf1-1)
      wdfylabel(save-1)= wdfylabel(wdf1-1)
      return
      end
