; ------------------------------------------------------------------------
; $Id: mjpeg_frame.pro,v 1.2 2010/12/09 18:53:14 lpmix Exp $
; ------------------------------------------------------------------------
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
;+
; NAME:
;       MJPEG_FRAME
;
; PURPOSE:
;       This procedure returns frames from a Motion JPEG2000 file or writes 
;       frames to a file.  
;
; CATEGORY:
;       Graphics
;
; CALLING SEQUENCE:
;
;       MJPEG_FRAME [, FILENAME] , FRAMES
;
; INPUTS:
;       FILENAME:  Name of the Motion JPEG2000 file
;                  If missing, DIALOG_PICKFILE will be called.
;                  If there are two parameters and the length of 
;                  FILENAME is no larger than one then 
;                  DIALOG_PICKFILE will be called.
;       FRAMES:    Frame number or numbers 
;                  Frame numbers are zero based and correspond to the 
;                  numbers in MJ2_Player.
;
; KEYWORD PARAMETERS:
;       FRAMES:    If FRAMES is a simple variable, the frames will be returned
;                  in an array.
;                  For a TrueColor image, FRAMES = [3, Width, Height, NFRAMES]
;                  If FRAMES is not a simple variable, then PNG will be set.
;       PNG:       If set, frame will be saved in a PNG file.
;                  If FRAMES is not a simple variable then PNG = 1
;       OUTNAME:   Output file name.  Ignored if PNG is not set.
;                  If PNG is set, and N_ELEMENTS(FRAMES) GT 1 THEN 
;                  "_N" is appended to each file.
;                  If OUTNAME is missing, OUTNAME = FILENAME with an 
;                  extension of ".png"
;                  If OUTNAME has an extension, it will be ignored and ".png" added
;                  Unique file names are always used unless REPLACE is set
;       REPLACE:   If set, output names will not be checked to see if they are 
;                  unique.
;       
; OUTPUTS:
;       If keyword FRAMES is provided, the frames are returned.
;       If keyword PNG is set, a PNG file will be written.
;       If neither keyword is present or set, then PNG will be set.
;
; RESTRICTIONS:
;       User must have write permission to the OUTNAME.
;
; PROCEDURE:
;       Motion JPEG2000 file is opened and data is read.
;
; EXAMPLE:
;       Read frames 0, 10 and 20 into a variable, FF, and send PNG versions to 
;       file "movie" in the current directory.  Pick the file with DIALOG_PICKFILE
;         MJPEG_FRAME, [0, 10, 20], /png, outname ='movie', frames = ff
;
;       Read the first frame of a file selected using DIALOG_PICKFILE, and return
;         it in variable, IMAGE.
;         
;
; MODIFICATION HISTORY:
; 	Written by:	L. P. Mix, Jr., July 23, 2010
;
;-

PRO  MJPEG_FRAME, FILENAME , FRAMES, FRAMES = output, png = png, outname = outname, $
      replace = replace

on_error, 1
common savemjpeg_picker, oldfilter, oldpath

  npar = N_Params()
  if npar eq 0 then begin
    print, '% Format: MJPEG_FRAME [, FILENAME] , FRAMES [, /PNG]'
    print, '     [, FRAMES = frames]  [, OUTNAME = outname]  [, /REPLACE]'
    return
  endif
  sfname = size(/type, filename)
  if npar eq 1 then begin
    if (sfname eq 7 or sfname eq 0) then begin
      print, '% MJPEG_FRAME: Frame or frames must be specified'
      return
    endif
    frames= filename
    nframes = n_elements(frames)
    fname = ''
  endif else begin
;   npar = 2 
    nframes = n_elements(frames)
    if nframes eq 0 then begin
      print, '% MJPEG_FRAME: Frame or frames must be specified in addition to file'
      return
    endif
    if size(/type, frames) gt 3 then begin
      print, '% MJPEG_FRAME: Frames must be an byte, integer or a long.'
      return
    endif
    if sfname eq 0 then fname = '' else fname = filename[0]
  endelse
  if strlen(strtrim(fname, 2)) lt 2 then begin
    if n_elements(oldpath) eq 0 then cd, current = path else path = oldpath
    fname = dialog_pickfile (/must_exist, /read, path = path, get_path= newpath)
    if strlen(fname) lt 1 then begin
      print, '% MJPEG_FRAME: No File was Selected - RETURNING'
      return
    endif
    oldpath = newpath
  endif
  vers = strupcase(strmid(!version.os_family, 0, 3))
  if (vers eq 'UNI') then begin
; get system variable substitution for unix systems
    temp = fname
    FULFNAM, temp, fname
  endif
  fname = file_expand_path(fname)
  info = file_info(fname)
  if (info.exists + info.read) ne 2b then begin
    print, '% MJPEG_FRAME: File does not exist with read permission'
    print, '         File Name = ', fname
    print, '         File Exists =', info.exists
    print, '         Read Permission =', info.read
    return
  endif
  omjpeg = OBJ_NEW('IDLffMJPEG2000', fname, /persistent)
  if not obj_valid(omjpeg) then begin
    print, '% MJPEG_FRAME: Unable to create Motion JPEG2000 object with file:'
    print, '     '+ fname
    return
  endif
  omjpeg -> getproperty, dimension = dim, n_frames = numframes
  nframes = n_elements(frames) 
  ok = where (frames ge 0 and frames lt numframes, nok, complement = complement)
  if nok lt nframes then begin
    frames = frames[ok]
    print, '% MJPEG_FRAME: Some requested frames are not in the file.'
    print, '               File has '+strtrim(numframes, 2)+' frames'
    print, '               Unable to read: ', frames[complement]
    nframes = nok
  endif
  lcoutput = arg_present(output) ? 1 : 0
  if lcoutput then lcpng = keyword_set(png) ? 1 : 0 else lcpng = 1
  if lcpng then begin
    lnum = strlen(strtrim (numframes, 2))
    ls = strtrim(lnum, 2)
    for_lnum = '(i'+ls +'.'+ls+')'
    if n_elements (outname) eq 0 then begin
      lcoutname = 0
      outname = fname
    endif else lcoutname =1
    sep = path_sep()
    nsep = strpos(outname, path_sep(), /reverse_search)
    next = strpos(outname , '.', /reverse_search)
    if next gt nsep then basename = strmid(outname,0, next) else begin
      if strlen(outname) eq nsep +1 then begin
        print, '% MJPEG_FRAME: Invalid output file name'
        print, '      '+ strtrim(outname,2)
        Obj_DESTROY, omjpeg
        return
      endif
      basename = outname
    endelse
  endif 
  m = frames[0]
  result = omjpeg -> getdata(m)
  if lcpng then begin
;   if a name is given and only 1 frame, use given name
;     otherwise add frame number.
    if lcoutname eq 1 and nframes eq 1 then fil = basename+'.png'  else $
    fil = basename+'_'+strtrim (string(m, format = for_lnum), 2)+'.png'
    if not keyword_set(replace) then fil = find_newfile(fil, /quiet)
    openw, unit, /get, fil, /delete, error = ierror
    if ierror ne 0 then begin
      print, '% MJPEG_FRAME: Unable to open the PNG file: '+strtrim(fil, 2)
      print, !ERROR_STATE.MSG 
      Obj_DESTROY, omjpeg
      return
    endif
    free_lun, unit
    write_png, fil, result
  endif
  if lcoutput then begin
    if nframes eq 1 then output = result else begin
      s = size(result)
      dim = [s[1 : s[0]], nframes]
      nelem = s[s[0]+2]
      output = make_array(nelem, nframes, type = s[s[0]+1])
      output [0, 0] = reform (result, nelem, /overwrite)
    endelse
  endif
  if nframes eq 1 then begin
    Obj_DESTROY, omjpeg
    result = 0
    return
  endif
  for i = 1L , nframes-1 do begin
    m = frames[i]
    result = omjpeg -> getdata(m)
    if lcpng then write_png, $
      basename+'_'+strtrim (string(m, format = for_lnum), 2)+'.png', result
    if lcoutput then output[0, i]= reform (result, nelem, /overwrite)
  endfor
; clean up
  result = 0
  if lcoutput then output = reform (output, dim, /overwrite)
  Obj_DESTROY, omjpeg
END
