; ------------------------------------------------------------------------
; $Id: mk_help.pro,v 1.5 2007/06/06 20:03:49 lpmix Exp $
; ------------------------------------------------------------------------
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
;+
; NAME:
;       MK_HELP
;
; PURPOSE:
;	Make LHELP help files for directories in IDL_PATH
;         Default behavior is to not include EXAMPLES directories
;
; CAUTION:  User must have write permission in the pfidl help directory
;             or specify a help path
;
; CALLING SEQUENCE:
;       MK_HELP
;	
; KEYWORD PARAMETERS:
;       LIBS:   Libraries (directories) to be included
;               LIBS is a string array
;               Default is all directories in !PATH except for the
;                 IDL "examples" directories
;                 Examples directories are listed with "_" replacing 
;                 directory separators
;               If LIBS is present only those specified directories
;                 will be generated
;       HELP_PATH: Directory for storing the help files.
;               This allows users to include their routines in LHELP
;               See example below
;       EXCLUDE: Directories to exclude
;                Default ='util$'
;       VERBOSE: List directories as they are processed
;
; OUTPUTS:
;       Help files for LHELP are generated in the PFIDL help file directory.
;       Alternatively, the user may specify a HELP_PATH for storing the files.
;       These files are only for the widget help format; html help is not 
;         currently supported with this routine.
;
; RESTRICTIONS:
;       User must have write permission in the PFIDL help file directory or 
;         specify a HELP_PATH to store the files.
;
; EXAMPLE:
;       Update all help files in IDL_PATH except for the IDL example files.
;         MK_HELP
;
;       Update the PFIDL_1D help files.
;         MK_HELP, LIBS = '1D'     ; assuming that no other directories
;                                  ;   contain this search string.
;
;       Update the PFIDL_1D and PFIDL_2D directories.
;         MK_HELP, LIBS = 'PFIDL_' 
; or      MK_HELP, LIBS = ['PFIDL_1D', 'PFIDL_2D']
;
;	Update EXAMPLES directories.
;         MK_HELP, LIBS = 'examples'
;
;       Generate help files for user routines located in the directory:
;         "/home/username/idl" and its subdirectories.
;       Assume the user has his help files in "/home/username/idl/help_files"
;       (for additional help contact Paul Mix - (505) 845-7493 )
;       User should have the following two lines in his IDL_STARTUP procedure.
;
;  !path = expand_path('+/home/username/idl')+path_sep(/search_path)+!path
;  !help_path = '/home/username/idl/help_files'+path_sep(/search_path)+!help_path
;
;       To generate the help files enter:
;         MK_HELP, LIBS ='username', help_path ='/home/username/idl/help_files'
;       LHELP should now display personal procedure help files.
;
; MODIFICATION HISTORY:
;       Written by:	L. P. Mix, 21 February 2003
;-
pro mk_help, libs = libs, help_path = help_path, exclude = exclude, verbose=verbose
on_error, 1

pfidl_dir = get_pfidl_directory()
if strlen(pfidl_dir) lt 1 then begin
  print, '% MK_HELP:  Unable to determine PFIDL directory'
  return
endif
if getenv('USER') eq 'lpmix' && n_elements(verbose) eq 0 then verbose=1
sep =  path_sep()
path = strsplit(strlowcase(!path), path_sep(/search_path), /extract)
if n_elements(exclude) eq 0 then exclude ='util$'

; determine which files must be rebuilt
nlibs =  n_elements(libs)
npath = n_elements(path)
if nlibs gt 0 then begin
  libs = strlowcase(libs)
  nok = 0
  maxlib = 2*(npath > nlibs)
  oklib = make_array(maxlib)
  for i = 0, nlibs-1 do begin
    ok = get_match(libs[i], path, nel = nel)
    if nel gt 0 then begin
      if nok+nel gt  maxlib then begin
        oklib =[oklib, make_array(maxlib)]
        maxlib = maxlib+maxlib
      endif
      oklib[nok] = ok
      nok = nok+nel
    endif
  endfor
  if nok gt 0 then begin
    oklib = oklib[0:nok-1]
    if nok gt 1 then oklib=oklib[uniq(oklib, sort(oklib))]
  endif else begin
    print,  '% MK_HELP:  No directories exist with search strings'
    for i = 0, nlibs-1 do print, '            ', libs[i]
    return
  endelse
  path = path[oklib]
endif else begin
;
; split path into examples and other
;
  ok = where(strpos(path, 'examples') ge 0, nok, $
         complement=notok, ncomplement = n_notok)
  if n_notok eq 0 then begin
    print, '% MK_HELP:  No directories exist without "examples"'
    print, '  !PATH = ', !path
    return
  endif else nok =n_notok
  path = path[notok]
endelse  
nex = n_elements(exclude)
npath = n_elements(path)

for i = 0, nex -1 do begin
  ok = get_match(exclude[i], path, nel = nel)
  if nel gt 0 then begin
    if npath eq nel then begin
      Print,  '% MK_HELP:  No directories exist after "exclude"'
      return
    endif
    other = indgen(npath)
    other[ok] = -1
    path = path[where (other gt 0, npath)]
  endif
endfor
npath = n_elements(path)

if n_elements(help_path) gt 0 then begin
  help_path = help_path[0]
  if strpos(help_path, sep, /reverse_search, /reverse_offset) ne 0 $
     then help_path = help_path+sep
endif else begin
  pfidl_dir = get_pfidl_directory()
  if strlen(pfidl_dir) eq 0 then begin
    print,  '% MK_HELP:  Unable to determine PFIDL directory'
    return
  endif
  help_path =  pfidl_dir+'help'+sep+'help_files'+sep
endelse

for i = 0, npath-1 do begin
  j = strpos(path[i], sep, /reverse_search)
  direct = strmid(path[i], j+1)
  if keyword_set(verbose) then print, '% Processing: ', path[i]
  mk_library_help, path[i], help_path+direct+'.help'
endfor
end

