; ------------------------------------------------------------------------ 
; $Id: mksymbol.pro,v 1.6 2010/08/26 19:58:48 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
;+
; NAME:		MkSymbol
; PURPOSE:	Make neat little user defined symbols
; CATEGORY:	Plotting/Graphics
;
; CALLING SEQUENCE:	MkSymbol,symbol
;
; INPUTS:
;	Symbol          an integer indicating the desired symbol:
;                       0, Square; 1, Circle; 2, Triangle, 3, Diamond;
;                       4, Big X or X; 5, Delta; 6, Star; 7, Plus;
;                       8, Hexagon; 9, Pentagon; 10, Clover; 
;                       11, PACMAN; 12, Spiral
;                       an alternative option is to enter a string with 
;                       sufficient characters to uniquely identify the symbol.
;			Some possible options are Square,Triangle,Circle,
;			Hexagon,BigX,Clover,Spiral,Star...
;                       DEFAULT: symbol = 1 (CIRCLE)
;                       See SYMBOL_LIST in the help for a complete list.
;
; KEYWORD PARAMETERS:	
;	SIZE	Symbol size      (default=!p.thick)
;	LIST	using LIST=Some_variable will return the vector
;		 of available symbol names in Some_variable      
;	HELP	returns this documentation header
;
; and also the keywords which apply to USERSYM
;	THICK	Line thickness   (default=1)
;	FILL	Fill symbol?     (default=0=no)
;	COLOR   Symbol color 
; EXAMPLES:
;       Define a clover leaf for plot symbol 8.
;         mksymbol, 10   <or>  mksymbol, 'cl'
;       Define a filled pacman for symbol 8
;         mksymbol, 11, /fill   <or>  mksymbol, 'pa', /fill
;
; SIDE EFFECTS:		Calls  USERSYM to load the new symbol
; 
; MODIFICATION HISTORY:     original version
;                            Brian Jackel   August 10 1992
;                            University of Western Ontario
;                           Modified for PFIDL 
;                            L. P. Mix, June 2, 1994
;-

pro MKSYMBOL,symbolin,SIZE=size,LIST=list,HELP=help, thick = thick, _EXTRA=_extra

  ON_ERROR,2

     common mkcircle, lcmkcircle, nel_symbols

  IF KEYWORD_SET(HELP) THEN BEGIN
     doc_library,'mksymbol'
     return
  ENDIF
;
@symbol_list.pro
;
  nelsym = n_elements(symbol_list)
  nel_symbols = nelsym
  LIST= symbol_list
  if n_elements(lcmkcircle) eq 0 then lcmkcircle = 0
  npar = n_params()
  if npar eq 0 then begin
    Print, '% Format: MKSYMBOL, symbol [, SIZE = size] [, LIST = list]'
    print, '           [, HELP = help] [, THICK = thick] [, FILL = fill]'
    print, '           [, COLOR = color]'
    print, '     Symbols are:'
    for i = 0, nelsym-1 do print, i, '  ', list[i]
    return
  endif

  IF not KEYWORD_SET(SIZE) THEN size=1.0
  if n_elements (symbolin) eq 0 then symbol = 1 else symbol= symbolin[0]
  type = size (symbol, /type)
  if type eq 7 then begin
     symbol= STRUPCASE( STRCOMPRESS(symbol,/REMOVE_ALL) )
     lsym = strlen(symbol)
     if lsym lt 1 then begin
       print, 'A valid symbol string must be specified'
       return
     endif
     if lsym eq 1 then begin
       if symbol eq 'X' then symbol = 'BIGX' else $
       if symbol eq 'O' then symbol = 'CIRCLE' else $
       if symbol eq '+' then symbol = 'PLUS' 
       lsym = strlen(symbol)
     endif
     temp = strmid(symbol_list, 0, lsym)
     ok = where (symbol eq temp, nok )
     if nok eq 0 then begin
       print, 'A valid symbol string must be specified'
       return
     endif else if nok eq 1 then symbol = symbol_list(ok) else begin
       print, ' Multiple symbols specified, using ' , symbol
       print, symbol_list[ok]
       symbol = symbol_list[ok[0]]
       print, '   Using a symbol of ', symbol
     endelse
     symbol = symbol[0]
  endif else begin
    symbol = round(symbol)
    if symbol ge 0 and symbol lt nelsym then symbol = symbol_list[symbol] $
       else begin
      print,'MKSYMBOL: Symbol must be in the range of 0 to ',strtrim(nelsym-1,2)
      return
    endelse
  endelse

  CASE symbol OF
  'DIAMOND': BEGIN
               x= size* [0.0,0.8,0.0,-0.8,0.0]
               y= size* [1.2,0.0,-1.2,0.0,1.2]
             END
  'PENTAGON':BEGIN
               theta= findgen(6)/5 * 360.0 * !dtor
               x= 1.0 *size* sin(theta)
               y= 1.0 *size* cos(theta)
             END
  'CLOVER':  BEGIN
               theta= (findgen(49)/48.0 * 360.0 ) * !dtor
               r= ABS(1.0 *size* sin(2.0*theta+!pi/2.))
               x= r * sin(theta)
               y= r * cos(theta)
             END
  'PACMAN':  BEGIN
               theta= (- findgen(41)/50.0*360.0 + 35.0 )*!dtor 
               x= [0.0, 1.0 *size* sin(theta), 0.0]
               y= [0.0, 1.0 *size* cos(theta) ,0.0]
             END
  'SPIRAL':  BEGIN
               theta= findgen(41)/40.0 * 720.0 * !dtor
               r= 1.0 *size* theta / MAX(theta)
               x= r * sin(theta)
               y= r * cos(theta)
             END
  'PLUS':    BEGIN
               x= 0.3 *size* $
                [0.,.7,.7,3.5,4.2,3.5,.7,.7,0,-.7,-.7,-3.5,-4.2,-3.5,-.7, -.7,0]
               y= 0.3 *size* $
              [4.2 ,3.5,.7,.7,0.,-.7,-.7,-3.5,-4.2,-3.5,-.7,-.7,0,.7,.7,3.5,4.2]
             END
  'BIGX':    BEGIN
               x= 0.30 *size* [0,2,3,3,1, 3, 3, 2, 0,-2,-3,-3,-1,-3,-3,-2,0]
               y= 0.30 *size* [1,3,3,2,0,-2,-3,-3,-1,-3,-3,-2, 0, 2, 3, 3,1]
             END
  'CIRCLE':  BEGIN
               n= 17.0
               theta= findgen(n)/(n-1.0) * 360.0 * !dtor
               x= 1.0 *size* sin(theta)
               y= 1.0 *size* cos(theta)
             END
  'SQUARE':  BEGIN
               theta= (findgen(5)/4.0 * 360.0 + 45.0 )*!dtor
               x= 1.2 *size* sin(theta)
               y= 1.2 *size* cos(theta)
             END
  'DELTA':BEGIN
               theta= [180,300,60,180]*!dtor
               x= 1.2 *size* sin(theta)
               y= 1.2 *size* cos(theta)
             END
  'TRIANGLE':BEGIN
               theta= [0,120,240,360]*!dtor
               x= 1.2 *size* sin(theta)
               y= 1.2 *size* cos(theta)
             END
  'STAR':    BEGIN
               theta= [0,36, 72,108, 144,180, 216,252, 288,324,0]*!dtor
               r= 1.1*[1.0,0.4, 1.0,0.4, 1.0,0.4, 1.0,0.4, 1.0,0.4,1.0]
               x= r *size* sin(theta)
               y= r *size* cos(theta)
             END
  'HEXAGON': BEGIN
               theta= [0,60,120,180,240,300,360]*!dtor
               x= 1.0 *size* sin(theta) 
               y= 1.0 *size* cos(theta)
             END
  ELSE: BEGIN
         print, '% MKSYMBOL: something failed.'
         print, '   Available symbols are:'
         print, '        '
         return   ;either with a guessed symbol, or a list of them
        END
  ENDCASE

  if n_elements(thick) eq 0 then thick = !p.thick
  USERSYM,x,y,thick = thick, _EXTRA=_extra
  lcmkcircle = 8
  return
END



