; ------------------------------------------------------------------------ 
; $Id: mn.pro,v 1.5 2010/03/12 22:56:28 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
 function  MN, wdfin, time = time, left = start, right = stopt , $
               max = maxx, quiet = quiet, range = range, ier = ier, $
            absolute = absolute, tmaximum = tmax

;+
;  Function which returns the minimum, or local minimum, of a WDF array and
;  optionally the abscissa value at which it occurs.  Optional parameters 
;  allow the specification of a limited portion of the array.  The maximum
;  in the same interval can be returned.
;  format: MIN_VALUE = MN(WD1 [, TIME = time] [, LEFT  = left] [, RIGHT = right]
;                        [, RANGE = range] [, MAX = maxx] [, TMAXIMUM = tmax]
;                      [, ABSOLUTE = absolute] [, QUIET = quiet])
;  where:
;    MIN_VALUE - Minimum value of WDF array on the specified interval
;                INF and NAN are ignored
;    WD1       - WDF array number
;    TIME      - Abscissa value for which the local minimum value occurs;
;                for multiple minima, this is the first
;    left, right - optional abscissa value to specify the left and right 
;                  abscissa values used to determine the local minimum
;                  if absent, the default values are the first and last data 
;                  points, respectively
;    MAX       - Maximum value of WDF array on the specified interval
;                INF and NAN are ignored
;    TMAXIMUM  - Abscissa value at which the local maximum occurs.
;    RANGE     - Range of abscissa values for the MIN/MAX
;    ABSOLUTE  - If set the absolute keyword will be set in the "MIN" command
;                (See MIN in the IDL Reference Guide)
;    IER       - error flag = 0 if no errors = 1 if data is not in the range
;    quiet - if present and nonzero, no values will be printed to the terminal
;
;  Note:   The IDL function MIN is used to determine min_value and time.
;
;  Examples:
;    Print the minimum and maximum value of array 2, between 10 and 1000
;      Print, MN(2, l = 10, r = 1000, m = mm), mm
;
;    Calculate the minimum value of array 3 and store it in bot and the time
;    at which it occurs in btime; suppress terminal output
;      bot =  MN(2, time = btime, /quiet)
;
;-
@com.dat
;
on_error, 1
      value = 0.0
      ier = 1
      if n_elements(initwdf) eq 0 then wdfinit
      nparams = n_params()
      if nparams eq 0 then begin
        print, '% MN: Waveform array must be defined.'
        return, value
      endif else wdf1 = wdfin[0]
;
      if n_elements(wdf1) eq 0 then begin
        print, '% MN: WDF parameter must be defined'
        return, value
      endif
      if wdf1 lt 1 or wdf1 gt maxwdfarray then begin
        print, '% MN: WDF array must be greater than 0 and </=', maxwdfarray
        print, '      Value = ', wdf1
        return, value
      endif
;
      np = wdfnp(wdf1-1)
      if np eq 0 then begin
        print, '% MN: No data in the array'
        return, value
      endif
;
      get_wdfarray, array, wdf1, /xy
      if n_elements (array) eq np + np then begin
        if n_elements (start) eq 0 then ist = wdfminx(wdf1-1) else ist = start
        if n_elements (stopt) eq 0 then ilst = wdfmaxx(wdf1-1) else ilst = stopt
        ok = where (array(0:np-1) ge ist and array(0:np-1) le ilst, nok)
        if nok eq 0 then begin
          if not keyword_set(quiet) then begin
           print, '% MN: START and STOP for XY data are not compatable'
           print, '      Minimum for the entire WDF array ', strtrim(wdf1, 2), $
                 ' is returned'
          endif
           ok = indgen(np)
        endif else ier = 0
        value = min(array(np+ok), max = maxx, jvalue, subscript_max = tmax, $
                 absolute = absolute, /nan)
        range = [min(array[ok], max= mm, /nan), mm]
        jvalue = ok[jvalue]
        time = array[jvalue]
        tmax = ok[tmax]
        tmax = array[tmax]
      endif else begin
        np = n_elements(array)-2
        if array(1) gt 0 then begin
          if n_elements (start) eq 0 then ist = 0L else $
             ist = long((start-array(0))/array(1)) > 0L
          if n_elements (stopt) eq 0 then ilst = np-1 else $
             ilst = long((stopt-array(0))/array(1) +1.00001) < np-1
        endif else begin
          if n_elements (start) eq 0 then ilst = np-1 else $
             ilst = long((start-array(0))/array(1)) < np-1
          if n_elements (stopt) eq 0 then ist = 0L else $
             ist = long((stopt-array(0))/array(1) +1.00001) > 0L
        endelse
        if ist ge ilst then begin
          if not keyword_set(quiet) then begin
           print, '% MN: START and STOP are not compatable'
           print, '      Minimum for the entire WDF array ', strtrim(wdf1, 2), $
                 ' is returned'
          endif
           ist = 0
           ilst = np-1
        endif else ier = 0
        value = min(array(ist+2L:ilst+2L), jvalue, max = maxx, /nan,  $
           subscript_max = tmax, absolute = absolute)
        jvalue = jvalue + ist
        time = array(0) + array(1) * jvalue
        tmax = tmax+ist
        tmax = array[0] + array(1) * tmax
        range = array[0]+array[1]*[ist, ilst]
      endelse
      if not keyword_set(quiet) Then begin
        print, ' Minimum value of array ', strtrim(wdf1,2), ' is ' , $
          strtrim(value, 2)
        print, '   on the interval from ', strtrim(range[0], 2), $
           ' to ', strtrim(range[1], 2)
        print, ' Minimum occurs at ', strtrim(time,2), ' (array element is: ', $
          strtrim(jvalue, 2),')'
      endif
      array=0
      return, value
      end
