; ------------------------------------------------------------------------ 
; $Id: mul.pro,v 1.3 2007/06/06 20:03:50 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
 Pro  mul, wdf1, wdf2, wdf3, wdf4, clabel = clab, sortx = sortx, $
           ylabel= ylab, xlabel = xlab, match_only = match_only,  $
           master = master,  default = default
;
;+
; NAME:
;       MUL
;
; PURPOSE:
;       Multiply a WDF array by a constant or a WDF array 
;       Store the result in the initial WDF array or another WDF array
; CATEGORY:
;       Math 
; CALLING SEQUENCE:
;       MUL, WD1, WD2 [, WD3]
;       MUL, WD1, 0, VAR, [, WD3]
; INPUTS:
;       WD1:     WDF array number of the first array
;       WD2:     WDF array number of array to multiply the first array
;  or
;       0, VAR   Variable or constant to multiply the first array
; OPTIONAL INPUT:
;       WD3:     WDF array number for the product if not specified the 
;                product is stored in WD1 (the first array)
; KEYWORD PARAMETERS:
;       CLABEL    Replace the dataset comment for array number WD3
;                 with the string or string variable specified
;       XLABEL    Replace the dataset x-axis label for array number WD3
;                 with the string or string variable specified
;       YLABEL    Replace the dataset y-axis label for array number WD3
;                 with the string or string variable specified
;       SORTX     Sort the X values and take all unique values.
;                 Ignored if MASTER is set
;       MATCH_ONLY If one array is XY data and the match is not exact,
;                  then make a new XY array with only X values in both
;                  arrays that differ by less than ERR where
;                  ERR =  (xmax-xmin)/(100*np)]  if match_only ge 0
;                      = - match_only            if match_only lt 0
;                  where np is the number of points in the first array
;                     and [xmin, xmax] is the range of the first data array
;                  (1% of the average point spacing)
;                  The points are selected by:
;                       X2-ERR < X1 < X2+ERR
;                  In the actual implementation, if both arrays are XY then
;                    the X1 values are those of the shorter array and not
;                    necessarily the first array.
;                  Multiple X2 points which satisfy the inequality are averaged
;       MASTER:    If the abscissa of WD1 does not match the abscissa of WD2 
;                  then the abscissa values in the master array will be used 
;                  exactly in the region of overlap.  If MASTER is negative 
;                  then all of the abscissa values in the master array will 
;                  be used and ordinate values where there is no abscissa will
;                  be set to DEFAULT. If keyword_set(MASTER) and abs(MASTER) is 
;                  not 2, then abs(MASTER) is set to 1.
;       DEFAULT:   Value to be use when MASTER is negative and there is not 
;                  complete overlap of the master curve.
;                  Default:  DEFAULT = 1.0
;
; OUTPUTS:
;       NONE     The product is stored in a WDF array in the common block
; COMMON BLOCKS:
;       COM.DAT  WDF common block 
; RESTRICTIONS:
;       WD1 array must be defined and have valid data
;       WD2 or 0 and a Variable must be defined
; PROCEDURE:
;       If WD2 is specified:
;            WD1 and WD2 are put on a common time base and the product
;            calculated.  If there are less than 2 points of overlap, then 
;            no action is taken.  Linear interpolation is used to put the 
;            arrays on a common abscissa scale using the minimum abscissa 
;            increment.
;       If 0, VAR is specified:
;            WD1 is multiplied by VAR, where VAR is a constant or a variable.
;       In either case, the product is stored in WD3, if specified, 
;       or WD1, if WD3 is not specified.
; EXAMPLES:
;       Multiply WDF arrays 2 and 3 and store the result in WDF array 8.
;               MUL, 2, 3, 8, c = 'Power', y = 'Power (W)'
;       Multiply the variable pi to WDF array 2 squared and store the product
;       (areas) in WDF array 8.
;               MUL, 2, 2, 8          ; Square array 2 and store in 8
;               MUL, 8, 0, !pi        ; Multiply by pi
; MODIFICATION HISTORY:
;      Original version, L. P. Mix, May 15, 1992
;-
;
@com.dat
      on_error, 1
;
      if n_elements(initwdf) eq 0 then wdfinit
      nparams = n_params()
      if nparams eq 0 then begin
        print, '% Format: MUL, WD1, WD2 [, WD3] [, CLABEL = clabel]'
        print, '              [, XLABEL = xlabel] [, YLABEL = ylabel]'
        print, '             [, MATCH_ONLY = match_only] [, SORTX = sortx]'
        print, '             [, MASTER = master] [, DEFAULT = default]    or'
        print, '          MUL, WD1, 0, VAR, [, WD3] [, CLABEL = clabel] '
        print, '              [, XLABEL = xlabel] [, YLABEL = ylabel]'
        return
      endif
      if nparams lt 2 then begin
        print,' Mul must have at least 2 parameters'
        return
      endif
;
      if n_elements(wdf1) eq 0 or  n_elements(wdf2) eq 0 then begin
        print, 'Parameters to MUL must be defined'
        return
      endif
      if wdf1 lt 1 or wdf1 gt maxwdfarray then begin
        print, 'wdf array must be greater than 0 and </=', maxwdfarray
        return
      endif
;
      lcconstant = 0
      if nparams gt 2 then save = get_save_array(wdf3, wdf1) else save = wdf1
;
      nel = wdfnp(wdf1-1)
      if nel lt 2 then begin
        print, 'No data in the first array'
        return
      endif
;
      if wdf2 eq 0 then begin
        if n_elements (wdf3) eq 0 then begin
          print,'Constant is undefined'
          return
        endif
        get_wdfarray, array1, wdf1, /xydata
        t1 = size(array1, /type)
        t2 = size(wdf3, /type)
        if t1 lt t2 then begin
          if t2 eq 4L then array1 = float(array1) else $
            if t2 lt 4L then array1 = long(array1) else $
            if t2 eq 5L then array1 = double(array1) else array1=complex(array1)
        endif
        if n_elements(array1) eq nel +nel then begin
          xydata = 1
          array1(nel:*)= array1(nel:*) * wdf3(0)
        endif else begin 
          xydata = 0
          array1(2:*)= array1(2:*) * wdf3(0)
        endelse
	lcconstant = 1
        if nparams gt 3 then save = get_save_array(wdf4, wdf1) else save = wdf1
      endif else begin 
        if wdf2 lt 1 or wdf2 gt maxwdfarray then begin
          print, 'wdf array must be greater than 0 and </=', maxwdfarray
          return
        endif
        nelb = wdfnp(wdf2-1)
        if nelb lt 2 then begin
          print, 'No data in the second array'
          return
        endif
        get_wdfarray, array1, wdf1, /xydata
        get_wdfarray, array2, wdf2, /xydata
        t1 = size(array1, /type)
        t2 = size(array2, /type)
        if t1 lt t2 then begin
          if t2 eq 4L then array1 = float(array1) else $
            if t2 lt 4L then array1 = long(array1) else $
            if t2 eq 5L then array1 = double(array1) else array1=complex(array1)
        endif
        max1 = wdfmaxx(wdf1-1)
        max2 = wdfmaxx(wdf2-1)
        if n_elements (default) eq 0 then default=1.0 else default=default[0]
        if keyword_set(master) then $
          get_master_time, array1, array2, max1, max2, nel, nelb, $
               xydata, master = master, default = default  $
        else $
          get_common_time, array1, array2, max1, max2, nel, nelb, $
               xydata, match_only = match_only, sortx = sortx
        if n_elements(array1) lt 4 then begin
          print, 'No values will be changed due to lack of overlap'
          return
        endif
        if xydata eq 0 then $
          array1(2:*) = array1(2:*) * array2(2:*) $ 
        else array1(nel:*)= array1(nel:*) * array2(nel:*)
      endelse
      if n_elements (save) eq 0 then save = wdf1
      if save lt 1 or save gt maxwdfarray then save = wdf1
      put_wdfarray, array1, save, xydata = xydata, /no_copy
      if lcconstant eq 0 then wdf3 = save else wdf4 = save
      if lcconstant eq 0 then begin
        if wdffile(wdf1-1) ne wdffile(wdf2-1) then wdffile(save-1) = '  ' $
           else  wdffile(save-1) = wdffile(wdf1-1)
      endif else wdffile(save-1) = wdffile(wdf1-1)
      if n_elements(clab) ne 0 then wdfcomment(save-1) = clab else $
        wdfcomment(save-1) = wdfcomment(wdf1-1)
      if n_elements(xlab) ne 0 then wdfxlabel(save-1) = xlab else $
        wdfxlabel(save-1)= wdfxlabel(wdf1-1)
      if n_elements(ylab) ne 0 then wdfylabel(save-1) = ylab else $
        wdfylabel(save-1)= wdfylabel(wdf1-1)
      array2=0
      sp=0
      im=0
      return
      end
