; ------------------------------------------------------------------------ 
; $Id: mytitle.pro,v 1.4 2007/06/06 20:03:50 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
;+
; NAME:
;       MYTITLE
;
; PURPOSE:
;       Write a general title for a plot at the top of the plot.
;
; CALLING SEQUENCE:
;
;	MYTITLE , MyTitle [, OFFSET = offset] [, CHARSIZE = charsize]
;                   [, /BOTTOM] [_extra = extrastuff]
;
; OPTIONAL INPUTS:
;       MYTITLE:    Title for this plot. This is required.
;       OFFSET:     Offset.  This is added to the default position.
;                   Units are normalized units.
;                   If offset is a single value, it will be applied to the 
;                     Vertical position.
;       CHARSIZE:   Character size. Default is 1.2 
;                   If maximum(!p.multi(1:2)) gt 2 then the size is scaled by 0.5
;       BOTTOM:     If set, Text will be put below the plot.
;                   If CHARSIZE is not set, CHARSIZE = 1.0
;       _EXTRA:     Any valid keywords to XYOUTS
;                       
; PROCEDURE:
;	XYOUTS is used to print a general text string.
;       Position is 0.5* total(!x.window) in the horizontal direction
;         and 0.5*(!y.window(0)+!y.region(0)) in the vertical direction (Bottom)
;         or 0.5*(!y.window(1)+!y.region(1)) in the vertical direction (Top)
;
; EXAMPLE:
;       Note: Before writing a title a plot must be made.
;
;       Put the title 'X-Ray Analysis' on a plot and make it the default title
;          mytitle, 'X-Ray Analysis'
;
;       Print the title 'Special Title' with character size = 2
;          mytitle, chars = 2, 'Special Title'
;
; MODIFICATION HISTORY:
; 	Written by:        Paul Mix, October 19, 1998
;-
 pro mytitle,mytit, charsize = chars, offset = offset, bottom = bottom, $
        _extra= extrastuff
  on_error, 1
 
 if !d.window eq -1 and (!d.name eq 'X' or !d.name eq  'MAC' $
       or !d.name eq 'WIN') then return
 
; set a default general title for this directory
 if n_params() eq 0 then begin
   print, '% Format: MYTITLE,  [, MyTitle] [, OFFSET = offset] '
   print, '         [, CHARSIZE = charsize] [, /BOTTOM] [_extra = extrastuff]'
   return
 endif
 if n_elements(mytit) eq 0 then begin
   Print, '% MYTITLE: No title specified'
   return
 endif

 if n_elements(chars) eq 0 then chars = (keyword_set(bottom)) ? 1.0 : 1.2
 if n_elements(offset) eq 0 then offset = 0.0
 if max(!p.multi(1:2)) gt 2 then charsiz = chars*0.5 else charsiz = chars
 xval = 0.5*total(!x.window)
 eps= float(!d.y_ch_size)/!d.y_size
 if keyword_set(bottom) then yval = 0.5*(!y.window(0)+!y.region(0))-1.0*eps $
   else yval = 0.5*(!y.window(1)+!y.region(1)-eps) 
 if  n_elements (offset) eq 2  then begin
       xval = xval +offset(0)
       yval = yval +offset(1)
 endif else yval = yval +offset(0)
  
 xyouts, xval, yval, mytit, align = 0.5, chars= charsiz, /nor, _extra=extrastuff

 end
