; ------------------------------------------------------------------------ 
; $Id: pad.pro,v 1.3 2007/06/06 20:03:50 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
 Pro  Pad, wdfarray, mdt
;
;+
; NAME:
;       PAD
;
; PURPOSE:
;       Decrease the point spacing of a WDF array using a FFT technique.
;       This technique attempts to preserve the frequency content of the 
;       waveform.  
;       Note: If the waveform does not begin and end at the same value of 
;             the amplitude then distortions will be generated at either 
;             extreme.
; CATEGORY:
;       Utility
; CALLING SEQUENCE:
;       PAD, WD1, DDT
; INPUTS:
;       WD1:      WDF number or an array of numbers of WDF arrays to be changed
;       DDT:      Decrease the interval between points by this factor
;                 Default is 2.
; OUTPUTS:
;       NONE    The parameters are stored in the WDF common block
; COMMON BLOCKS:
;       COM.DAT  WDF common block 
; SIDE EFFECTS:
;        If the waveform does not begin and end at the same value of 
;          the amplitude then distortions will be generated at either 
;          extreme.
; RESTRICTIONS:
;       WD1 array must be defined and have valid data
; PROCEDURE:
;       A fft is taken and zeros are added at higher frequencies to bring 
;          the inverse point spacing to the desired value.
; EXAMPLES:
;       Increase the number of points in dataset 22 by a factor of 3
;               PAD, 22, 3
;       Increase the number of points in datasets 1 to 10 by a factor of 4
;               PAD, 1+indgen(10), 4
; MODIFICATION HISTORY:
;      Original version, L. P. Mix, January 13, 1994
;-
;
;     include the common block
@com.dat
;
;     see if we are initialized
      if n_elements(initwdf) eq 0 then wdfinit
;
;
;     determine if wdfarray is good
;
      nelwdf = n_elements(wdfarray)
      if nelwdf eq 0 then begin
        print, ' wdf array must be defined'
        return
      endif
      if n_elements (mdt) eq 0 then ddt = 2 else ddt = abs(mdt) >1
      if ddt eq 1 then begin
        Print, ' PAD is not required if then factor <= to 1'
        return
      endif
   for iarray = 0, nelwdf-1 do begin
      wdf = wdfarray(iarray)
      if wdf lt 1 or wdf gt maxwdfarray then begin
        print, 'wdfarray must be greater than 0 and </=', maxwdfarray
        goto, endarray
      endif 
;     determine if we have valid data
      np = wdfnp(wdf-1)
      if np lt 1 then begin
        Print,' No valid data in wdf array =' , wdf
        goto, endarray
      endif
;      
      get_wdfarray, array, wdf
;     Calculate the actual factor
      npnew = round (ddt*np)
      c = fft(array(2:*), -1)
      cc = make_array(npnew, /complex)
      np2 = np/2
      cc(0:np2) = c(0:np2)
      cc(npnew-np+np2+1:*) = c(np2+1:*)
      cc = float (fft(cc, 1))
      array = [array(0), array(1)*np/npnew, cc(0:npnew-round(ddt))]
      put_wdfarray, array, wdf, /no_copy
endarray:
      endfor
      return
      end
