; ------------------------------------------------------------------------ 
; $Id: plotcube.pro,v 1.4 2007/06/06 20:03:50 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
 Pro Plotcube, normal,zvalue, str, wdfarray = wdfarray, noerase= noerase, $
  reverse = reverse, slice = slice, transpos = transpos, $
  axrotate = axrotate, azrotate = azrotate, edges = edges, $
  ecolor = ecolor, elinsstyle = elinestyle, $
  xstyle = xstyle, ystyle = ystyle, zstyle = zstyle, $
  xrange = xrange, yrange= yrange, zrange= zrange, $
  _extra  = extrastuff

;+
; NAME:
;       PLOTCUBE
;
; PURPOSE:
;       Plot a WDF array or structure array in a 3D cube. Can be used
;          with scalar field arrays or wdf arrays.
;       This routine can also be used to indicate a series of slices of 
;          a 3D shape.
;       The default axis is z verticle, x, horizontal and y into the screen.
;       Note: For postscript !p.font = -1 gives better results usually.
;
; CALLING SEQUENCE:
;
;       PLOT_CUBE, NORMAL, ZVALUE, STR [, REVERSE = reverse ]
;       [, SLICE = slice] [, WDFARRAY = wdfarray] [TRANSPOS= transpos]
;       [, NOERASE = noerase] [, AXROTATE = axrotate] [, AZROTATE = azrotate]
;       [, EDGES = edges] [, EColor = ecolor] [, Elinestyle = elinestyle]
;       [, XRANGE =xrange] [, YRANGE =yrange] [, ZRANGE =zrange]
;       [, XSTYLE = xstyle ] [, YSTYLE = ystyle ] [, ZSTYLE = zstyle ]
;       [, _EXTRA = extrastuff]
;
; where:
;       NORMAL  - An array indicating the normal direction to the plot plane
;                 and the position along the normal for the plot plane.
;                 The normal is indicated by the flags of 1, 2, or 3 for 
;                 X, Y and Z normal. If the number of elements of NORMAL 
;                 is less than ZVALUE, then NORMAL will be replicated to
;                 provide additional values.
;       ZVALUE  - An array indicating the value of the normal plane where
;                 it intercepts the normal axis.
;                 These values are the interval [0, 1] (Normalized units)
;       STR     - An array of structure array numbers or WDF array numbers
;                 to be plotted in each of the normal planes.  STR must have
;                 at least as many values as ZVALUE 
;                 (or 1 value if slice is specified)
;       REVERSE - Plot arrays in reverse order.
;       TRANSPOS- Transpose parameter to go to plotstr.
;                 If not specified, transpos = 1 for y normal and zero otherwise
;                 If specified with n elements then ith plot uses 
;                    transp= transpos(i mod n)
;       SLICE   - If set, the kvalues and slice coordinates will be set 
;                 according to the value of normal, zvalue and range.
;                 NOTE: range must be set in the direction for slice
;              plotstr, str, normal, (zvalue-range(0))/(range(1)-range(0)), ...
;                 Default is slice = 0
;       WDFARRAY- If the keyword is set, the str values will be interpreted as 
;                 as WDF array numbers.  The default is WDFarray = 0.
;       NOERASE - If set the plot will not be erased and the 3 dimensional 
;                 transfrom will not be generated.
;                 For example, the commands shade_surf, surface or 
;                 or plotstr with these keywords can be used to set up the
;                 transform:
;                 plotstr, 1, /save, /shade & plotcube,/noerase, ...
;       AXROTATE- Rotation about the X axis. Default = 30
;       AZROTATE- Rotation about the Z axis. Default = 30
;       EDGES   - An array of integers indicating edges of the reference
;                 cube which are to be drawn. Values are:
;                 10 or 11  the x axis
;                 12        the edge parallel to x axis at y = 1
;                 13        the edge parallel to x axis at z = 1
;                >13        the edge parallel to x axis at y=1 and z = 1
;                 20 or 22  the y axis
;                 21        the edge parallel to y axis at x = 1
;                 23        the edge parallel to y axis at z = 1
;                >23        the edge parallel to y axis at x=1 and z = 1
;                 30 or 33  the z axis
;                 31        the edge parallel to z axis at x = 1
;                 32        the edge parallel to z axis at y = 1
;                >33        the edge parallel to z axis at x=1 and y = 1
;       ECOLOR  - Color of line used to plot edges; Default = !p.color
;                 If ecolor has fewer values than edges,
;                   color values are recycled.
;       ELINESTYLE - Linestyle used to plot edges; Default = 0
;                    If elinestyle has fewer values than edges,
;                      elinestyle values are recycled.
;       XYZ Range - Range of values for the plot
;       XYZ Style - Style values form each axis 
;                   Default is style = 1 if range has two values.
;
;       Any valid additional keywords which can be used with PLOTSTR or PLO
;         can be used.
;
;       PROCEDURE:
;         Set up the 3D transform.
;         Plot the edges if requested.
;         Repeat over plots
;           Rotate to the appropriate normal plane.
;           Use plo or plotstr to generate the image
;
;       Examples:
;         Assume lineouts in wdf arrays 1 to 5 corresponding to locations
;         loc = [1, 3, 4, 5, 7]. Plot these in X-normal planes in reverse order
;           plotcube, /reverse, 1, loc/6.0, indgen(5)+1, /wdf
;         
;         Plot the field magnitude on three faces of a cube from structure 
;           array 3
;           plotcube, [1, 2, 3], [0,0,1], 3,  /slice, $
;               xrange = [0, 5], yrange= [1, 3], zrange= [0, 10]
;         Note: If all three faces come from the same structure array, only
;               one array must be specified
;
;         For a plot similar to the one above with no labels and larger plots:
;           plotcube, [1, 2, 3], [0,0,1], 3,  /slice, $
;               xrange = [0, 5], yrange= [1, 3], zrange= [0, 10], $
;               xticks = 1, yticks = 1, xmargin = [0,0], ymargin = [0,0], $
;               xtickname = [' ', ' '], ytickname = [' ', ' '], title = ' '
;-
 on_error, 1
@com.dat
  npar = n_params()
  if npar eq 0 and not keyword_set (edges) then begin
print,'% Format: PLOTCUBE, NORMAL, ZVALUE, STR [, WDFARRAY = wdfarray]'
print,'         [, AXROTATE = axrotate] [, AZROTATE = azrotate]'
print,'         [, REVERSE = reverse] [, SLICE = slice]'
print,'         [, NOERASE = noerase] [, EDGES = edges]'
print,'         [, EColor = ecolor] [, Elinestyle = elinestyle]'
print,'         [, XRANGE =xrange] [, YRANGE =yrange] [, ZRANGE =zrange]'
print,'         [, XSTYLE = xstyle ] [, YSTYLE = ystyle ] [, ZSTYLE = zstyle ]'
print,'         [, _EXTRA = extrastuff]'
    return
  endif

  if not keyword_set(noerase) then begin
    if n_elements(axrotate) eq 0 then ax = 30. else ax = axrotate 
    if n_elements(azrotate) eq 0 then az = 30. else az = azrotate 
    erase
    scale3, xrange= [0,1], yrang= [0,1], zrang= [0, 1], ax = ax, az = az
  endif
  nele = n_elements (edges)
  if nele gt 0 then begin
    nelc = n_elements(ecolor)
    if nelc eq 0 then begin
      nelc = 1
      ecolor = !p.color
    endif
    nell = n_elements(elinestyle)
    if nell eq 0 then begin
      nell = 1
      elinestyle = 0
    endif
    for i = 0, nele-1 do begin
      ic = ecolor(i mod nelc)
      il = elinestyle(i mod nell)
      if edges(i) lt 20 then begin
        if (edges(i)  le 11) then $
          plots, /nor, [0, 1], [0, 0], [0, 0], /t3d, col= ic, linest = iL $
        else if (edges(i)  eq 12) then $
          plots, /nor, [0, 1], [1, 1], [0, 0], /t3d, col= ic, linest = iL $
        else if (edges(i)  le 13) then $
          plots, /nor, [0, 1], [0, 0], [1, 1], /t3d, col= ic, linest = iL $  
        else plots, /nor, [0, 1], [1, 1], [1, 1], /t3d, col= ic, linest = iL
      endif else if edges(i) ge 30 then begin
        if (edges(i) eq 30 or edges(i) eq 33 ) then  $
          plots, /nor, [0, 0], [0, 0], [0, 1], /t3d, col= ic, linest = iL $
        else if (edges(i)  eq 31) then $
          plots, /nor, [1, 1], [0, 0], [0, 1], /t3d, col= ic, linest = iL $
        else if (edges(i)  le 32) then $
          plots, /nor, [0, 0], [1, 1], [0, 1], /t3d, col= ic, linest = iL $  
        else plots, /nor, [1, 1], [1, 1], [0, 1], /t3d, col= ic, linest = iL
      endif else begin
        if (edges(i) eq 20 or edges(i) eq 22 ) then $
          plots, /nor, [0, 0], [0, 1], [0, 0], /t3d, col= ic, linest = iL $
        else if (edges(i)  eq 21) then $
          plots, /nor, [1, 1], [0, 1], [0, 0], /t3d, col= ic, linest = iL $
        else if (edges(i)  le 23) then $
          plots, /nor, [0, 0], [0, 1], [1, 1], /t3d, col= ic, linest = iL $  
        else plots, /nor, [1, 1], [0, 1], [1, 1], /t3d, col= ic, linest = iL
      endelse
    endfor
  endif
  nnormal = n_elements (normal)
  if nnormal eq 0 then begin
    if nele gt  0 then return
    print, '%PLOTCUBE: Normal directions must be defined.'
    return
  endif

  normal = round (normal)
  ok = where (normal lt 1 or normal gt 3, nok)
  if nok gt 0 then begin
    print, '%PLOTCUBE: Normal values must be in the range of [1, 3]'
    print, '   Normal values:', normal
    return
  endif
 
  nzval = n_elements (zvalue)
  if nzval eq 0 then begin
    print, '%PLOTCUBE: Z Values of the normal planes must be defined'
    return
  endif
  ok = where (zvalue lt 0.000 or zvalue gt 1.000, nok)
  if nok gt 0 then begin 
    print, '%PLOTCUBE: Z Values must be in the range [0.000, 1.000]'
    return
  endif


  if nnormal lt nzval then begin
    temp = make_array(nzval, /long)
    temp(0:nnormal-1) = normal
    for i = nnormal, nzval-1 do temp(i) = normal(i mod nnormal)
    normal = temporary(temp)
  endif else if nnormal gt nzval then  normal = temporary(normal(0:nzval-1))

  nstr = n_elements (str)
  if nstr eq 0 then begin
    print, '%PLOTCUBE: Data structures must be defined'
    return
  endif else begin
    if keyword_set(wdfarray) then begin
      if nstr lt nzval then begin
        print, $
          '%PLOTCUBE: Number of structures must equal number of normal planes'
        return
      endif
      ok = where (str lt 1 or str gt maxwdfarray, nok)
      if nok gt 0 then begin
        print, '%PLOTCUBE: STR values must be valid'
        print, '   STR values:', str
        return
      endif
    endif else begin
      if keyword_set (slice ) and nstr eq 1 then begin
        str = make_array(nzval, value = round (str), /long)
      endif else if nstr ge nzval then begin
        if nstr gt nzval then str = temporary(str(0:nzval-1))
      endif else begin
        print, $
          '%PLOTCUBE: Number of structures must equal number of normal planes'
        return
      endelse
    endelse
  endelse
  if keyword_set(reverse)  and nzval gt 1 then begin
    str = reverse(str)
    normal = reverse(normal)
    zvalue = reverse(zvalue)
  endif
  if n_elements (slice) eq 0 then slice = 0

  if n_elements (xrange) ge 2 then begin
    xrange = xrange(0:1)
    xdelta = xrange(1)-xrange(0)
    if n_elements (xstyle) eq 0 then xstyle = 1
  endif else begin
    xrange = 0.0
    if n_elements (xstyle) eq 0 then xstyle = 0
  endelse

  if n_elements (yrange) ge 2 then begin
    yrange = yrange(0:1)
    ydelta = yrange(1)-yrange(0)
    if n_elements (ystyle) eq 0 then ystyle = 1
  endif else begin
    yrange = 0.0
    if n_elements (ystyle) eq 0 then ystyle = 0
  endelse

  if n_elements (zrange) ge 2 then begin
    zrange = zrange(0:1)
    zdelta = zrange(1)-zrange(0)
    if n_elements (zstyle) eq 0 then zstyle = 1
  endif else begin
    zrange = 0.0
    if n_elements (zstyle) eq 0 then zstyle = 0
  endelse

  oldnorm = 3
; do the plotting
  neltran = n_elements(transpos)
  for i = 0, nzval-1 do begin
    if normal(i) ne oldnorm then begin
      if oldnorm eq 3 then begin
        t3d, /yz
        if normal(i) eq 1 then t3d, /xz
      endif else if oldnorm eq 2 then begin
        if normal(i) eq 1 then t3d, /xz else t3d, /yz
      endif else begin
        t3d, /xz
        if normal(i) eq 3 then t3d, /yz
      endelse
    endif
    oldnorm = normal(i) 

    if oldnorm eq 1 then begin
      xr = yrange
      yr = zrange
      xst = ystyle
      yst = zstyle
      if neltran eq 0 then transp = 0 else transp= transpos(i mod neltran)
      if keyword_set (slice) then begin
        if n_elements (xrange) eq 2 then begin
          kvalue = xrange(0) + zvalue(i)*xdelta
        endif else begin
          if not keyword_set(wdfarray) then begin
            print, '%PLOTCUBE: X Range must be specified if SLICE is set'
            print, '           Continue with kvalue set to zero'
            kvalue = 0.000
          endif
        endelse
      endif
    endif else if oldnorm eq 2 then begin
      xr = xrange
      yr = zrange
      xst = xstyle
      yst = zstyle
      if neltran eq 0 then transp = 1 else transp= transpos(i mod neltran)
      if keyword_set (slice) then begin
        if n_elements (yrange) eq 2 then begin
          kvalue = yrange(0) + zvalue(i)*ydelta
        endif else begin
          if not keyword_set(wdfarray) then begin
            print, '%PLOTCUBE: Y Range must be specified if SLICE is set'
            print, '           Continue with kvalue set to zero'
            kvalue = 0.000
          endif
        endelse
      endif
    endif else begin
      xr = xrange
      yr = yrange
      xst = xstyle
      yst = ystyle
      if neltran eq 0 then transp = 0 else transp= transpos(i mod neltran)
      if keyword_set (slice) then begin
        if n_elements (zrange) eq 2 then begin
          kvalue = zrange(0) + zvalue(i)*zdelta
        endif else begin
          if not keyword_set(wdfarray) then begin
            print, '%PLOTCUBE: Z Range must be specified if SLICE is set'
            print, '           Continue with kvalue set to zero'
            kvalue = 0.000
          endif
        endelse
      endif
    endelse

    if keyword_set (wdfarray) then begin
      plo, str(i), zvalue = zvalue(i), /t3d, /noerase, $
        xrange = xr,xstyle = xst, yrange = yr,ystyle = yst, $
        _extra = extrastuff
    endif else begin
      if keyword_set(slice) then begin
        plotstr, str(i), oldnorm, kvalue, zvalue = zvalue(i), /t3d, /noerase, $
        xrange = xr,xstyle = xst, yrange = yr,ystyle = yst, transp = transp, $
        cn = 1, _extra = extrastuff
      endif else begin
        plotstr, str(i), zvalue = zvalue(i), /t3d, /noerase, $
        xrange = xr,xstyle = xst, yrange = yr,ystyle = yst, transp = transp, $
        cn = 1, _extra = extrastuff
      endelse
    endelse
  endfor
  if oldnorm eq 1 then begin
    t3d, /xz
    t3d, /yz
  endif else if oldnorm eq 2 then t3d, /yz
        
  return
  end
