; ------------------------------------------------------------------------ 
; $Id: putcolor.pro,v 1.2 2007/06/06 20:03:51 lpmix Exp $
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
;+
; NAME:
;       PUTCOLOR
;
; PURPOSE:
;       The purpose of this function is to set the default colors for the 
;       drawing colors which are set by LOADXYCT.
;
; CATEGORY:
;       Graphics, Color Specification.
;
; CALLING SEQUENCE:
;       PUTCOLOR  [, COLORS]  [, VALUE]  [, RESET = reset]
;
; Optional INPUTS:
;       COLORS: An array of R, G, B values to set the drawing colors.
;              Default values for each color are shown below.
;              Eight colors are allowed: 
;              1   RED       [255,   0,   0]
;              2   GREEN     [  0, 255,   0]
;              3   BLUE      [  0,   0, 255]
;              4   MAGENTA   [255,   0, 255]
;              5   ORANGE    [255, 125,   0]
;              6   CYAN      [  0, 255, 255]
;              7   YELLOW    [255, 255,   0]
;              8   WHITE     [150, 150, 150] (actually grey)
;              Colors may be an array, COLORS[8, 3] to set all 8 drawing colors.
;              Colors may be an array of 3 values to set only one of the colors.
;              If COLORS has only 3 values, then the color value must be specified.
;       VALUE: This parameter specifies the color value to be replaced.
;              This parameter is ignored if COLORS has 24 values.
;              This parameter must be greater than zero and less than 9 (1 to 8).
; KEYWORDS:
;       RESET: If set, the default values are set.
;        NOTE: If PUTCOLOR has no valid parameters, the colors will be reset.
; COMMON BLOCKS:
;       The LOADXYCT_COMMON has the default line color set.
;
; SIDE EFFECTS:
;       Default drawing colors will be changed.
;
; EXAMPLE:
;       To set the default colors.
;         PUTCOLOR
;  <or>   PUTCOLOR, /reset
;
;       To set color 5 to [125, 125, 125] and reset all other colors to their
;       default values.      
;         PUTCOLOR, /reset,  [125, 125, 125], 5
;
;       To keep all current drawing colors, but to change color 8 to a lighter
;       grey ([180, 180, 180]).
;         PUTCOLOR, [180, 180, 180], 8
;
; MODIFICATION HISTORY:
;       Written by: Paul Mix, May 10, 2005
;            
;-


pro  PUTCOLOR, colors, value, reset= reset
@loadxyct_common.dat
; Loadxyct common block
; common loadxyct_common, loadct_init, max_color ,invertct_flag, $
;     color_maximum, color_minimum, line_colors
;
   
ON_ERROR, 1
;
; DEFAULT value
;
if (n_elements(colors) eq 0) || (n_elements(line_colors) eq 0) || $
 keyword_set(reset) then line_colors = $
        reform ([ 255,   0,   0, 255, 255,   0, 255, 150, $
                    0, 255,   0,   0, 125, 255, 255, 150, $
                    0,   0, 255, 255,   0, 255,   0, 150], 8, 3)

if n_elements(colors) eq 0 then return 
if n_elements(colors) ge 24 then begin
  line_colors = (reform(line_colors[0:23], 8, 3) > 0) < 255
  return
endif
if n_elements(colors) ge 3 and n_elements(value) eq 1 then begin
  if value[0] lt 1  || value gt 8 then begin
    print, '% PUTCOLOR: Bad value passed for the drawing color value'
    print, '            No action taken.  Value = ', value[0]
  endif else line_colors [value[0]-1, *] = colors[0:2]
  return
endif
print, '% PUTCOLOR: Bad color parameters.'
help, colors
help, value

END
