; ------------------------------------------------------------------------
; $Id: readbmp.pro,v 1.2 2010/12/09 18:03:27 lpmix Exp $
; ------------------------------------------------------------------------
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
;+
; NAME:
;       READBMP
;
; PURPOSE:
;	Read a BMP file and return the image, a structure or store it
;         in a PFIDL structure array
;
; CALLING SEQUENCE:
;  Format:  READBMP, STRUCTURE  [, FILE] [, DX = dx] [, DY = dy] 
;           [, SCALE = scale] [, CLABEL = clabel] [, XLABEL = xlabel]
;           [, YLABEL = ylabel] [, TLABEL = tlabel] [, REVERSE = reverse]
;           [, /IMAGE] [, /STRUCTURE] [, /INTEGER]
;
; INPUTS:
;	STRUCTURE  - Structure Array Number or variable if STRUCTURE 
;                    or IMAGE is set.
;       FILE       - BMP file name
;                    If zero of undefined, DIALOG_PICKFILE will be used.
;
; KEYWORD PARAMETERS:
;       DX         - X increment 
;                    DEFAULT: DX = 1
;       DY         - Y increment  - Ignored if DX is undefined
;                    DEFAULT: DY = DX 
;       SCALE      - Intensity Scale Factor
;                    DEFAULT: SCALE = 1
;       CLABEL     - Dataset comment for the structure
;                    DEFAULT: CLABEL = FILE
;       XLABEL     - Abscissa comment for the structure
;                    DEFAULT: XLABEL = 'X'
;       YLABEL     - Ordinate comment for the structure
;                    DEFAULT: YLABEL = 'Y'
;       TLABEL     - Type label for the structure
;                    DEFAULT: TLABEL = 'BMP Image'
;       IMAGE      - If set, the BMP image is returned in the variable STRUCTURE
;       STRUCTURE  - If set and if image is not set, the structure is returned in
;                    the variable STRUCTURE
;       INTEGER    - If set, save the image as an integer array
;       REVERSEIMAG- If not set, image will not be reversed.
;                    If set IMAGE will be reversed top to bottom.
;                    DEFAULT:  REVERSEIMAG = 0
;       
; OUTPUTS:
;	A BMP image is stored in a structure array or returned either as an 
;       image or a structure.
;
; RESTRICTIONS:
;       The procedure handles standard BMP files.
;       See IDL help for READ_BMP
;
; PROCEDURE:
;       The procedure attempts to use the IDL procedure READ_BMP to read the file.
;       If this is unsuccessful, it attempts to use a special BMP reader.
;
; EXAMPLE:
;       Read the BMP image, "shot2122.bmp" and store the image in structure 2
;         readBMP, 2, 'shot2122.bmp'
;
;	Return the image from  "shot2122.bmp" in variable imag2122.
;         readbmp, imag2122, 'shot2122.bmp'
;
;       Return the image as a structure in variable str from a file to be selected 
;       using DIALOG_PICKFILE
;          readbmp, str, /structure
;
; MODIFICATION HISTORY:
; 	Written by:	L. P. Mix, August 7, 2003
;
;-
;
PRO Readbmp, STR, FILE, DX = dx, DY = dy, SCALE = scale, CLABEL = clabel, $
       XLABEL = xlabel, YLABEL = ylabel, TLABEL = tlabel, IMAGE= image, $
       STRUCTURE = structure, REVERSEimag = reverseimag, integer = integerarr

on_error, 1
 
; Common block
@qsstrcom.dat
common savebmp_picker, oldfilter, oldpath
;
      if n_elements (structinit) eq 0 then qsstruct_init
;
      nparams = n_params()
      if nparams eq  0 then begin
        print, '% Format:', $
           '  READBMP, STRUCTURE  [, FILE] [, DX = dx] [, DY = dy]'
        print, '   [, SCALE = scale] [, CLABEL = clabel]'
        print, '   [, XLABEL = xlabel] [, YLABEL = ylabel] [, TLABEL = tlabel]'
        print, '   [, /IMAGE] [, /STRUCTURE] [, /INTEGER]'
        return
      endif
      if not keyword_set(image) and not keyword_set(structure) then begin
        if n_elements(str) eq 0 then begin
          print, '% READBMP: Structure array number must be defined'
          return
        endif
        strsave = get_save_str(str, get_maxstr())
      endif
      if n_elements(reverseimag) eq 0 then reverseimag = 0
      if n_elements(file) eq  0 then fname = 0 else fname = file[0]
      b = size(fname)
      if b(1) eq 7 then begin
        resolve_it, fname
        info = file_info(fname)
        if (info.exists + info.read) ne 2b then begin
           print, '% READBMP: File does not exist with read permission'
           print, '         File Name = ', fname
           print, '         File Exists =', info.exists
           print, '         Read Permission =', info.read
           ier = -2
           return
        endif
      endif else begin
        if n_elements(oldpath) eq 0 then cd, current = path else path = oldpath
        filter = ['*.bmp;*.BMP', '*.*']
        fname = dialog_pickfile(/must_exist, /read, filter = filter, path = path, get_path= newpath)
        if strlen(fname) lt 1 then begin
          print, '% READBMP: No File was Selected - RETURNING'
          return
        endif
        oldpath = newpath
        resolve_it, fname
      endelse
      if n_elements(clabel) eq 0 then clabel = fname
      str = read_bmp(fname, /RGB)
      if n_elements(str) lt 10 then begin
        print, '% READBMP: Invalid image returned.'
        help, str
        return
      endif
      if keyword_set(reverseimag) then begin     
        ss = size(str)
        str= reverse(str, ss[0], /overwrite)
      endif
      if n_elements(scale) gt 0 then str = str*scale[0]
;     If only image is requested then return
      if keyword_set(image) then return
      s = size (str)
      if s[0] eq 3 then begin
        if s[1] ne 3 then begin
          print, '% READBMP: Cannot return a structure at this time'
          print, '            Returning image in the structure'
          return
        endif
        v1 = reform(str[0, *, *])
        v2 = reform(str[1, *, *])
        v3 = reform(str[2, *, *])
        if not keyword_set(integerarr) then begin
          v1 = float(v1)
          v2 = float(v2)
          v3 = float(v3)
        endif
        nx = s[2]
        ny = s[3]
        ndim = 3
      endif else begin
        ndim = 2
        nx = s[1]
        ny = s[2]
        v1 = keyword_set(integerarr) ? temporary(str) :  float(str)
      endelse
      if n_elements (dx) gt 0 then begin
        x = findgen(nx)*dx[0]
        if n_elements(dy) eq 0 then dy = 1.0
        y = findgen(ny)*dy[0]
        if ndim eq 2 then $       
          arr2str, str, /str, v1, x, y, dx = dx[0], dy= dy[0], $
             clab = clabel, XLABEL = xlabel, YLABEL = ylabel, TLABEL = tlabel $
        else $
          arr2str, str, /str, v1, v2, v3, x, y, dx = dx[0], dy= dy[0], $
            v1lab ='Red', v2lab = 'Green', V3lab = 'Blue', $
             clab = clabel, XLABEL = xlabel, YLABEL = ylabel, TLABEL = tlabel 
      endif else begin
        if ndim eq 2 then $       
          arr2str, str, /str, v1, $
             clab = clabel, XLABEL = xlabel, YLABEL = ylabel, TLABEL = tlabel $
        else $
          arr2str, str, /str, v1, v2, v3, v1lab ='Red', v2lab = 'Green', V3lab = 'Blue', $
             clab = clabel, XLABEL = xlabel, YLABEL = ylabel, TLABEL = tlabel 
      endelse
      if keyword_set(structure) then return
      i2s, str, strsave, /no_copy
      str = strsave
      end
