; ------------------------------------------------------------------------
; $Id: readfuji.pro,v 1.9 2011/10/04 21:14:52 lpmix Exp $ 
; ------------------------------------------------------------------------
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
;+
; NAME:
;	READFUJI
;
; PURPOSE:
;
;       This procedure reads a FUJI format file for the IP plate data and outputs a
;       psl (photo stimulable level) dose equivelent file.  The data for the
;       conversion and the image parameters are read from the Fuji .PCB or .INF
;       header file.  The image data itself must be in a .IMG file of the same 
;       name as the header file.  The units of the output are PSL.
;       (Photo-Stimulated Luminescence)
;
; CALLING SEQUENCE:
;
;       READFUJI, STRUCTURE, [, FILE] [, /PIXELS] [, /MM]
;           [/STRUCTURE] [, /IMAGE] [, /WRITETIFF] [, /REVERSEimage]
;           [, CLABEL = clabel] [, XLABEL = xlabel] [, YLABEL = ylabel]
;           [, TLABEL = tlabel] [, SWAPENDIAN = swapendian] 
;           [, SATURATION = saturation] [, HEADER = header] [, LUT = lut]
;           [, NoConvert = noconvert] [, /VERBOSE]
;
; INPUTS:
;       STRUCTURE  - Structure Array Number or variable if STRUCTURE 
;                    or IMAGE is set.
;       FILE       - FUJI file name
;                    If zero of undefined, DIALOG_PICKFILE will be used.
;
; KEYWORD PARAMETERS:
;       PIXELS     - If set, return structure in pixels.
;                    Default: If pixels is undefined, PIXELS = 0 
;                           UNLESS if MM = 0 then PIXELS = 1
;       MM         - If set, return structure in millimeters.
;                    Default: If PIXELS is set, MM is not checked.
;                             If MM is not defined, MM= 1
;                             If MM = 0, then PIXELS = 1
;       STRUCTURE  - If set and if image is not set, the structure is returned in
;                    the variable STRUCTURE
;       IMAGE      - If set, the FUJI image is returned in the variable STRUCTURE
;       WRITETIFF  - If set, FUJI is written to a tiff file.
;                    If WRITETIFF is a string, WRITETIFF will be the name of the
;                      TIFF file.
;                    Default: TIFF_FILE = FILE(without extension)+'.tif'
;       REVERSEimage - If not set, image will be reversed.
;                    IDL has a convention from bottom to top; TIFF stores 
;                    image top to bottom.  Image will be reversed unless REVERSE = 0
;                    DEFAULT:  REVERSE = 1
;             Note:  WRITETIFF will write the original image regardless of this
;                    keyword value.
;       SWAPENDIAN- If set, this keyword will determine if bytes are swapped.
;                    Default:
;                      BYTE_ORDER, IMAGE, /SWAP_IF_LITTLE_ENDIAN
;       CLABEL     - Dataset comment for the structure
;                    DEFAULT: CLABEL = FILE
;       XLABEL     - Abscissa comment for the structure
;                    DEFAULT: XLABEL = 'X'
;       YLABEL     - Ordinate comment for the structure
;                    DEFAULT: YLABEL = 'Y'
;       TLABEL     - Type label for the structure
;                    DEFAULT: TLABEL = 'FUJI Image'
;       SATURATION - Returns the maximum value that the machine setting can return
;       HEADER     - Returns the header information in a structure.
;                    The tag names and some typical values are:
;                    Tag names: FILETYPE:       STRING        'BAS_IMAGE_FILE'
;                               FILETITLE:      STRING        <FUJI file name>
;                               SCAN_DIMENSIONS STRING          '23*40'
;                               XRESOLUTION     FLOAT           100.000
;                               YRESOLUTION     FLOAT           100.000
;                               GRADLEVELS      FLOAT           65536.0
;                               HEIGHT          FLOAT           2320.00
;                               WIDTH           FLOAT           4040.00
;                               SENSITIVITY     FLOAT           4000.00
;                               LATITUDE        FLOAT           5.00000
;                     Note: GRADLEVELS = 2^BITDEPTH
;       LUT        - Look up table with the conversion between counts and PSL
;       NOCONVERT  - If set, do not convert to PSL units.
;                    The image will be in the range 0 to GRADLEVELS-1
;       VERBOSE    - Print the header information 
;                    Default: VERBOSE = 1
; OUTPUTS:
;       This procedure returns the dose equivelent file from the fuji scanner
;       data in a PFIDL structure or an array.  This array may be written to a 
;       TIFF file.  The units are PSL units (Photo-Stimulated Luminescence).
;       The scanner data is compressed logarithmically to 16 bit integers in the .IMG 
;       to preserve the roughly 4 orders of magnitude dynamic range of the image plate. 
;       PSL data must be stored as a full precision float to preserve the PSL values.
;       The image is stored as a float in the TIFF file; the size of the TIFF file
;       is about the same as full a full precision PFF file, but the spatial sizes
;       and dataset comment and labels are lost.
;
; PROCEDURE:
;       Read the header file to get image size.  Read the image into an unsigned
;         integer, scale the image, and store in a PFIDL structure. 
;       ******************
;   (OLD) The scaling information is from the header file:
;         image = ((xresolution)^2) * (4000/Sensitivity) * $
;                  (10^(Latitude*(float(image)/gradLevels-0.5))
;       *********************
;       The above scaling from the original routine appears to be off by a factor 
;         of 10,000 to be in units of PSL.  READFUJI now uses:
; or      PSL_IMAGE = ((xresolution/100)^2) * (4000/Sensitivity) * $
;                  (10^(Latitude*(float(image)/gradLevels-0.5))
;
; EXAMPLE:
;       Read shot_670.img into structure 1.
;         READFUJI, 1, 'shot_670.img'
;
;       Read an image to be selected into structure 2.
;         READFUJI, 2
;
; MODIFICATION HISTORY:
;       Original Version:  L. Paul Mix     January 5, 2005
;       FUJI format information from a routine by Steve Quillin, June 23 1998  
;       Modified 2/21/2011, L. P. Mix to include a factor of 10,000 in scaling 
;         to convert the .img file to PSL.                          
;-


PRO readfuji, STR, FILE, CLABEL = clabel, verbose = verbose, $
       swapendian = swapendian, mm = mmin, pixels = pixelsin, $
       XLABEL = xlabel, YLABEL = ylabel, TLABEL = tlabel, IMAGE= image, $
       STRUCTURE = structure, REVERSEimage = reverseimag, $
       SATURATION = saturation, header = header, LUT = lut, $
       NOCONVERT = noconvert, WRITETIFF = writetiff
common savefujipicker, oldfilter, oldpath

on_error, 1
;
      nparams = n_params()
      if nparams eq  0 then begin
        print, '% Format:', $
           '  READFUJI, STRUCTURE  [, FILE] [, /MM] [, /PIXELS]'
        print, '   [, SWAPENDIAN = swapendian] [, REVERSEimage = reverseimage]'
        print, '   [, XLABEL = xlabel] [, YLABEL = ylabel] [, TLABEL = tlabel]'
        print, '   [, CLABEL = clabel] [, /IMAGE] [, /STRUCTURE]'
        print, '   [, NOCONVERT = noconvert] [, WRITETIFF = writetif]'
        print, '   [, SATURATION = saturation] [, header = header] [, LUT = lut,]'
        print, '   [, /VERBOSE]'
        return
      endif
      if not keyword_set(image) and not keyword_set(structure) then begin
        if n_elements(str) eq 0 then begin
          print, '% READFUJI: Structure array number must be defined'
          return
        endif
        strsave = get_save_str(str, get_maxstr())
      endif
      if n_elements(reverseimag) eq 0 then reverseimag = 1
      if keyword_set(pixelsin) then mm = 0 $
        else mm = (n_elements(mmin) gt 0) ? mmin[0] : 1
      if n_elements(file) eq  0 then fname = 0 else fname = strtrim(file[0], 2)
      b = size(fname)
      if b(1) ne 7 then begin
        if n_elements(oldpath) eq 0 then cd, current = path else path = oldpath
        p1 = dialog_pickfile(/must_exist, /read, path = path, $
          filter = ['*.img;*.IMG', '*.*'], get_path = newpath)
        if strlen(p1) lt 1 then begin
          print, '% READFUJI: No File was Selected - RETURNING'
          return
        endif
        oldpath = newpath
        fname = strtrim(temporary(p1), 2)
      endif
      resolve_it, fname
      info = file_info(fname)
      if (info.exists + info.read) ne 2b then begin
         print, '% READFUJI: File does not exist with read permission'
         print, '         File Name = ', fname
         print, '         File Exists =', info.exists
         print, '         Read Permission =', info.read
         ier = -2
         return
      endif
      ii = strpos(fname, '.', /reverse_search)
      if ii gt 1 then fname = strmid(fname, 0, ii) else begin
        Print, '% READFUJI: File must have an extension'
        return
      endelse
      if n_elements(clabel) eq 0 then clabel = 'FUJI PSL image - '+fname
      tiff_file = fname+'.tif'
      if n_elements(verbose) eq 0 then verbose = 1
      ls, fname+'.*', current = files

      if n_elements(files) lt 2 then begin
        Print, '% READFUJI: A minimum of 2 files are required for FUJI format'
        print, files
        return
      endif
      fuji = make_array(2, /string, value = '')
      exten = -1
      for i = 0, n_elements(files)-1 do begin
        ext= strmid(files[i], ii+1)
        if strlowcase(ext) eq 'img' then fuji[1] = files[i] else begin
          ok = where( strlowcase(ext) eq ['pcb', 'inf'], nok)
          if nok gt 0 then begin
            exten= ok[0]
            fuji[0]= files[i]
          endif
        endelse
      endfor
      if min(strlen(fuji)) eq 0 then begin
        Print, '% READFUJI: A minimum of 2 files are required:'
        print, '  Header file: ', fuji[0]
        print, '   Image file: ', fuji[1]
        return
      endif     
;
      if keyword_set(verbose) then print, 'Processing Header: ', fuji[0]
      openr, unit, fuji[0], /get_lun
      ss = ''
      on_ioerror, bad_data
      aas = make_array(3, /string, value = ss)
      if exten eq 0 then begin
        while not eof(unit) do begin
          readf, unit, ss
          if keyword_set(verbose) then print, ss
          sl =strpos(ss,'=')
          if sl gt 0 then begin
            ss = strlowcase(ss)
            name = strtrim(strmid(ss, 0, sl), 2)
            value = strmid(ss, sl+1)
            Case name of 
                'height'     : reads, value,height
                'width'      : reads, value,width
                'xresolution': reads, value,xResolution
                'yresolution': reads, value,yResolution
                'gradlevels' : reads, value,GradLevels
                'rotation'   : reads, value,Rotation
                'flipped'    : reads, value,flipped
                'latitude'   : reads, value,Latitude
                'sensitivity': reads, value,Sensitivity
                'iptype'     : reads, value,IPType
                ELSE          :Q=1
            endcase
          endif
        endwhile
        if (rotation eq 0.) or (rotation eq 180.) then begin
           htemp=height
           height=width
           width=htemp
        endif
        bitdepth =0
      endif else begin
        for i = 0, 2 do begin
          readf, unit, ss
          if keyword_set(verbose) then print, ss
          aas[i] = strtrim(ss, 2)
        endfor
        READF,unit,xResolution
        if keyword_set(verbose) then PRINT,'xResolution  = ',xResolution
        READF,unit,yResolution
        if keyword_set(verbose) then PRINT,'yResolution  = ',yResolution
        READF,unit,bitdepth
        if keyword_set(verbose) then PRINT,'bitdepth  = ',bitdepth
        GradLevels = 2L^bitdepth
        if keyword_set(verbose) then PRINT,'GradLevels  = ',GradLevels
        READF,unit,height
        if keyword_set(verbose) then PRINT,'height   = ',height
        READF,unit,width
        if keyword_set(verbose) then PRINT,'width    = ',width
        READF,unit, Sensitivity
        if keyword_set(verbose) then PRINT,'Sensitivity    = ',Sensitivity
        READF,unit,Latitude      
        if keyword_set(verbose) then PRINT,'Latitude    = ',latitude
      endelse
      free_lun, unit
;
      if arg_present(header) then header = {filetype: aas[0], filetitle: aas[1], $
        scan_dimensions:aas[2], xResolution: xResolution, yResolution:yResolution, $
        bitdepth: bitdepth, height: height, width: width, Sensitivity: Sensitivity, $
        Latitude: Latitude, gradlevels: gradlevels}

      if keyword_set(verbose) then print, 'Processing Image: ', fuji[1]
      openr, unit, fuji[1], /get_lun
      str=make_array(height, width, /uint)
      readu, unit, str
      free_lun, unit
      if n_elements(swapendian) eq 0 then byteorder,str, /swap_if_little_endian $
         else if keyword_set(swapendian) then byteorder, str

;     Scale the data
      if (not keyword_set(noconvert)) or arg_present(lut) then begin
        nlut = gradlevels 
        lut = findgen(nlut)
        scale_factor = ((xresolution*1.0e-2)^2) * (4000/Sensitivity)
        lut = scale_factor * (10^(latitude*(temporary(lut)/gradLevels-0.5)))
        lut[0] = 0.0
        str = (keyword_set(noconvert)) ? float(str) : lut[str]
        if keyword_set(verbose) then print, 'Saturation Level: '+strtrim(lut[nlut-1], 2)
      endif else str = float(str)

      if keyword_set(writetiff) then begin
         if size(/type, writetiff) eq 7 then tiff_file = writetiff
         WRITE_TIFF, tiff_file, str, /float
      endif

      if keyword_set(reverseimag) then begin
        ss = size(str)
        str= reverse(str, ss[0], /overwrite)
      endif

;     If only image is requested then return
      if keyword_set(image) then return

      s = size(str)
      nx = s[1]
      ny = s[2]
      ndim = 2
      if mm eq 0 then begin
        dx = 1.0
        dy = 1.0
        x = findgen(nx)
        y = findgen(ny)
      endif else begin
        dx = xResolution/1000.0
        dy = yResolution/1000.0
        x = findgen(nx)*dx
        y = findgen(ny)*dy
      endelse
      if n_elements(xlabel) eq 0 then xlabel = (mm eq 0) ? 'X(Pixels)' : 'X(mm)'
      if n_elements(ylabel) eq 0 then ylabel = (mm eq 0) ? 'Y(Pixels)' : 'Y(mm)'
      if n_elements(tlabel) eq 0 then tlabel = 'FUJI Image'
      arr2str, str, /str, str, x, y, dx = dx, dy= dy, clab = clabel, $
         XLABEL = xlabel, YLABEL = ylabel, TLABEL = tlabel
      if keyword_set(structure) then return
      i2s, str, strsave, /no_copy
      str = strsave
      return
bad_data: free_lun, unit
      print, '% READFUJI: Error reading file.'
end
