; ------------------------------------------------------------------------ 
; $Id: readhdf.pro,v 1.4 2007/06/06 20:03:51 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
;+
; NAME:
;       READHDF
;
; PURPOSE:
;       Read XDAMP formatted HDF files.
;       This routine is based on the copyrighted xdamp procedure, XI_readfile,
;         written by William P. Ballard as part of the Xdamp package.
;         Ref. SAND98-0907
;
; CALLING SEQUENCE:
;       Format:
;         READHDF [, FILENAME] [, WDF = WDF] [, STRUCTURE = structure]
;        [, TITLE = title] [, DATE = date] [, HEADER = header] [, NOTES = notes]
;        [, NWDF = nwdf] [, NSTR = nstr] [, HELP = help]
;        [, FILE_READ = file_read] [, IERROR = ierror]
;
; OPTIONAL INPUTS:
;       FILENAME - Name of the HDF file to be opened.
;                  The default file extension is ".hdf"  
;                  If file is omitted or undefined or is not a string,
;                  PICKFILE will be used to select a file.
;	
; KEYWORD PARAMETERS:
;       WDF     - First WDF array for storing the waveform data
;                 Default = 1
;                 If not enough waveform arrays are available, the number will
;                 be increased to read all the waveform data.  If the number 
;                 of waveform arrays is increased, it is incremented in groups
;                 of 500.
;       STRUCTURE - First structure array for storing the image data
;                 Default = 1
;                 If not enough structure arrays are available, the number will
;                 be increased to read all the image data.  If the number 
;                 of waveform arrays is increased, it is incremented in groups
;                 of 100.
;       TITLE   - Shot title and date if it is available.
;       HEADER  - Shot header information
;       NOTES   - Shot notes if they are available.
;       NWDF    - Number of waveforms read into PFIDL.
;       NSTR    - Number of images read
;       HELP    - If set, a help message is printed.
;       FILE_READ- File opened and read
;       IERROR  - Returns zero if no error.
;
;
; OUTPUTS:
;       Data will be read and stored in waveform and structure arrays.
;
; PROCEDURE:
;       This is a modified version of the XDAMP procedure XI_readfile.
;
; EXAMPLE:
;       Read data from the file example.hdf.
;         READHDF, 'example.hdf'
;
;       Read data from an hdf file to be selected by pickfile.  Store the 
;       waveform data beginning with WDF array 21.
;         READHDF, wdf = 21
; MODIFICATION HISTORY:
; 	Written by:     L. P. Mix, May 14, 2002
;                  based on routine by W. P. Ballard.
;-

PRO READHDF , P1, WDF = WDF, STRUCTURE = struct, TITLE = title, date= date, $
       HEADER = fheader, NOTES = fnotes, NWDF = nwdf, NSTR = nstr, help = help, $
       file_read = filename, IERROR = ierror


on_error, 1
      ierror = -1
;
      nparams = n_params()
      if nparams eq 0 and keyword_set(help) then begin
print, '% Format: READHDF [, FILENAME] [, WDF = WDF] [, STRUCTURE = structure]'
print, '              [, TITLE = title] [, HEADER = header] [, NOTES = notes]'
print, '              [, NWDF = nwdf] [, NSTR = nstr] [, HELP = help]'
print, '              [, IERROR = ierror]'
        return
      endif
      if (nparams eq 0) then begin
        filename = ''
        nparams = 1
      endif else if n_elements(p1) eq 0 then begin
        filename = ''
      endif else if (size(p1, /type) ne 7) then begin
        filename = ''
      endif else  filename = strtrim(p1, 2)
      cd, current = current
      
      if strlen(filename) lt 1 then begin
        filter = '*.hdf'
        filename = dialog_pickfile( filter = filter, $
          title= 'Select HDF file to read', /must_exist)
        filename = strtrim(filename, 2)
        if strlen(filename) lt 1 then begin
          print, '% READHDF: No File was Selected - RETURNING'
          return
        endif
      endif else filename = p1
;
      info = file_info(filename)
      if (not info.exists) then begin
        print, '% READHDF: File does not exist'
        print, '          ', filename
        filename =  dialog_pickfile(/must_exist, /read)
        if strlen(filename) lt 1 then begin
          print, '% READHDF: No File was Selected - RETURNING'
          return
        endif
        info = file_info(filename)
      endif
      if (not info.read) then begin
        print, '% READHDF: File is not readable by the user'
        print, '          ', filename
        return
      endif
      print, '% READHDF: Reading ', filename

      mxwdf = get_maxwdf()
      mxstr = get_maxstr()

      if n_elements(wdf) eq 0 then wdf = 1 else wdf = (wdf[0]>1) < (mxwdf+1)
      if n_elements(struct) eq 0 then struct = 1 else $
            struct = (struct[0]>1) < (mxstr+1)
      wdf = long(wdf)
      struct = long(struct)
      wdf_inc = 500
      str_inc = 100
      nwdf = 0L
      nstr = 0L

      shot_title  = ''
      shot_date = ''
      anote = ''
      audit = ''
      npts = 0L
      delim = ''
      s_buffer = ''
      fname = filename

;*** HDF FORMAT

   hdf_handle = -999
   IF (HDF_ISHDF(filename)) THEN hdf_handle = HDF_OPEN(filename, /READ)

   if hdf_handle lt 0 then begin
     print, '% READHDF:  Unable to open file: ', filename
     if hdf_handle eq -999 then print, '  Not a recognized HDF file'
     return
   endif

;*** be sure not an image file incorrectly selected
   IF (HDF_DFR8_NIMAGES(fname) GT 0) THEN BEGIN
      XI_print, fname+" is an image file."
      HDF_CLOSE, hdf_handle
      RETURN
   ENDIF

;*** find header Vgroup
   vgxh = -1
   REPEAT BEGIN
      t = vgxh
      vgxh = HDF_VG_GETID(hdf_handle, t)
      vid = HDF_VG_ATTACH(hdf_handle, vgxh, /READ)
      HDF_VG_GETINFO, vid, NAME=tname
      HDF_VG_DETACH, vid
   ENDREP UNTIL (tname EQ "XDAMP HEADER" OR vgxh EQ -1)
   IF (vgxh NE -1) THEN BEGIN
      vgxhh = HDF_VG_ATTACH(hdf_handle, vgxh, /READ)

;*** find shot information Vgroup
      vgxi = -1
      inote = 0L
      ihead = 0L
      mxhead = 128
      mxnote = 128
      fheader = make_array(mxhead, /string)
      fnotes = make_array(mxnote, /string)
      REPEAT BEGIN
         t = vgxi
         vgxi = HDF_VG_GETNEXT(vgxhh, t)
         IF (HDF_VG_ISVD(vgxhh, vgxi)) THEN BEGIN
            vid = HDF_VD_ATTACH(hdf_handle, vgxi, /READ)
            HDF_VD_GET, vid, FIELDS=tname
            CASE tname OF
               "SHOT DATE":  k = HDF_VD_READ(vid,shot_date,FIELDS="SHOT DATE")
               "SHOT TITLE": k = HDF_VD_READ(vid,shot_title,FIELDS="SHOT TITLE")
               "HEADER INFO":BEGIN
                                k = HDF_VD_READ(vid, stg, FIELDS="HEADER INFO")
                                if ihead eq mxhead then begin
                                  fheader=[fheader,make_array(mxhead, /string)]
                                  mxhead = mxhead+ mxhead
                                endif
                                fheader(ihead) = STRING(stg)
                                ihead = ihead+1
                             ENDCASE
               "FILE NOTES": BEGIN
                                k = HDF_VD_READ(vid, stg, FIELDS="FILE NOTES")
                                if inote eq mxnote then begin
                                  fnotes = [fnotes, make_array(mxnote, /string)]
                                  mxnote = mxnote+ mxnote
                                endif
                                fnotes(inote) = STRING(stg)
                                inote = inote+1
                             ENDCASE
               ELSE:
            ENDCASE
            HDF_VD_DETACH, vid
         ENDIF
      ENDREP UNTIL (vgxi EQ -1)
      HDF_VG_DETACH, vgxhh

      shot_date = strtrim(shot_date, 2)
      date = shot_date
      shot_title = strtrim(shot_title, 2)
      title = shot_title
      if strlen(shot_date) gt 0 then $
        title = title+';;DATETIME='+shot_date+';;'
      if inote gt 0 then fnotes = fnotes[0:inote-1] else fnotes = ''
      if ihead gt 0 then fheader = fheader[0:ihead-1] else fheader = ''
;
   ENDIF

;*** find data Vgroup
   vgxd = -1
   REPEAT BEGIN
      t = vgxd
      vgxd = HDF_VG_GETID(hdf_handle, t)
      vid = HDF_VG_ATTACH(hdf_handle, vgxd, /READ)
      HDF_VG_GETINFO, vid, NAME=tname
      HDF_VG_DETACH, vid
   ENDREP UNTIL (tname EQ "XDAMP DATA")
   vgxdh = HDF_VG_ATTACH(hdf_handle, vgxd, /READ)

;*** find wavform or image Vgroup
   vgwd = -1
   REPEAT BEGIN
      t = vgwd
      vgwd = HDF_VG_GETNEXT(vgxdh, t)
      IF (HDF_VG_ISVG(vgxdh, vgwd)) THEN BEGIN
         vid = HDF_VG_ATTACH(hdf_handle, vgwd, /READ)
         HDF_VG_GETINFO, vid, NAME=tname
         HDF_VG_DETACH, vid
      ENDIF
   ENDREP UNTIL (tname EQ "WAVEFORM" or tname EQ "IMAGE")
   vgwdh = HDF_VG_ATTACH(hdf_handle, vgwd, /READ)

;*** now loop through the waveforms or images
   id = -1
   id = HDF_VG_GETNEXT(vgwdh, id)
   REPEAT BEGIN
      flag = 0
      REPEAT BEGIN
         vds = HDF_VD_ATTACH(hdf_handle, id, /READ)
         HDF_VD_GET, vds, FIELDS=testfield
         CASE testfield OF
            "NAME":      k = HDF_VD_READ( vds, aname, FIELDS="NAME")
            "ANOTE":     k = HDF_VD_READ( vds, anote, FIELDS="ANOTE")
            "AUDIT":     k = HDF_VD_READ( vds, audit, FIELDS="AUDIT")
            "HUNITS":    k = HDF_VD_READ( vds, hunit, FIELDS="HUNITS")
            "VUNITS":    k = HDF_VD_READ( vds, vunit, FIELDS="VUNITS")
            "TMIN":      k = HDF_VD_READ( vds, tmin,  FIELDS="TMIN")
            "TMAX":      k = HDF_VD_READ( vds, tmax,  FIELDS="TMAX")
            "XUNITS":    k = HDF_VD_READ( vds, xunit, FIELDS="XUNITS")
            "YUNITS":    k = HDF_VD_READ( vds, yunit, FIELDS="YUNITS")
            "ZUNITS":    k = HDF_VD_READ( vds, zunit, FIELDS="ZUNITS")
            "XPTS":      k = HDF_VD_READ( vds, xpts,  FIELDS="XPTS")
            "YPTS":      k = HDF_VD_READ( vds, ypts,  FIELDS="YPTS")
            "XMIN":      k = HDF_VD_READ( vds, xmin,  FIELDS="XMIN")
            "XMAX":      k = HDF_VD_READ( vds, xmax,  FIELDS="XMAX")
            "YMIN":      k = HDF_VD_READ( vds, ymin,  FIELDS="YMIN")
            "YMAX":      k = HDF_VD_READ( vds, ymax,  FIELDS="YMAX")
            "DATA":      BEGIN
                            k = HDF_VD_READ( vds, ary, FIELDS="DATA")
                            flag = 1
                         ENDCASE
            "MATRIX":    BEGIN
                            k = HDF_VD_READ( vds, mat, FIELDS="MATRIX")
                            flag = 1
                         ENDCASE
           ELSE:
         ENDCASE
         HDF_VD_DETACH, vds
         id = HDF_VG_GETNEXT(vgwdh, id)
      ENDREP UNTIL flag

      CASE tname OF
         "WAVEFORM":    BEGIN
            clab = STRTRIM(aname,2)+ ' - '
            if strlen(title) gt 0 then clab = clab +title

            anote = STRTRIM(anote,2)
            IF (anote EQ "No Annotation") THEN anote = ''
            if strlen(anote) gt 0 then clab = clab+'ANOTE='+anote+';;'

            audit = STRTRIM(STRING(audit),2)
            IF (audit EQ "No Audit Trail") THEN audit = ''
            IF (STRLEN(STRCOMPRESS(audit,/REMOVE_ALL)) EQ 0) THEN audit = ''
            if strlen(audit) gt 0 then clab = clab+'AUDIT='+audit+';;'

            npts = N_ELEMENTS(ary)
            xlabel = strtrim (hunit, 2)
            if strlen (xlabel) eq 0 then xlabel = 'Time'
            ylabel = strtrim (vunit, 2)
            if strlen (ylabel) eq 0 then ylabel = 'Amplitude'
            if wdf+nwdf gt mxwdf then begin
              mxwdf = mxwdf+wdf_inc
              set_maxwdfarray, mxwdf
            endif
            put_wdfarray, [tmin, (tmax-tmin)/(npts-1), ary], wdf+nwdf, $
              /no_copy, clabel = clab, xlabel = xlabel, ylabel = ylabel, $
              file = fname 
            nwdf = nwdf+1
;*** Done with waveforms, may be images to read
            IF (id EQ -1) THEN BEGIN
               HDF_VG_DETACH, vgwdh
               vgwd = -1
               REPEAT BEGIN
                  t = vgwd
                  vgwd = HDF_VG_GETNEXT(vgxdh, t)
                  IF (vgwd NE -1 AND HDF_VG_ISVG(vgxdh, vgwd)) THEN BEGIN
                     id = HDF_VG_ATTACH(hdf_handle, vgwd, /READ)
                     HDF_VG_GETINFO, id, NAME=tname
                     HDF_VG_DETACH, id
                  ENDIF
               ENDREP UNTIL (tname EQ "IMAGE" OR id EQ -1 OR vgwd EQ -1)
               IF (vgwd NE -1) THEN BEGIN
                  vgwdh = HDF_VG_ATTACH(hdf_handle, vgwd, /READ)
                  id = -1
                  id = HDF_VG_GETNEXT(vgwdh, id)
               ENDIF ELSE id = vgwd
            ENDIF
            END

         "IMAGE": BEGIN
            clab = STRTRIM(aname,2)+ ' - '
            if strlen(title) gt 0 then clab = clab +title

            anote = STRTRIM(anote,2)
            IF (anote EQ "No Annotation") THEN anote = ''
            if strlen(anote) gt 0 then clab = clab+'ANOTE='+anote+';;'

            audit = STRTRIM(STRING(audit),2)
            IF (audit EQ "No Audit Trail") THEN audit = ''
            IF (STRLEN(STRCOMPRESS(audit,/REMOVE_ALL)) EQ 0) THEN audit = ''
            if strlen(audit) gt 0 then clab = clab+'AUDIT='+audit+';;'
            dx = (xmax-xmin)/(xpts-1)
            dy = (ymax-ymin)/(ypts-1)
            xarr = xmin+dx*findgen(xpts)
            yarr = ymin+dy*findgen(ypts)
            if struct+nstr gt mxstr then begin
              mxstr = mxstr+str_inc
              set_maxstrarray, mxstr
            endif

            arr2str, struct+nstr, mat, xarr, yarr, dx= dx, dy = dy, $
              clab = clab, xlab = strtrim(xunit, 2), ylab = strtrim(yunit, 2), $
              file = fname, v1label = strtrim (zunit, 2)
            nstr = nstr +1
            IF (id EQ -1) THEN HDF_VG_DETACH, vgwdh
            END
      ENDCASE
   ENDREP UNTIL (id EQ -1)

   HDF_CLOSE, hdf_handle
   title = shot_title
   ierror = 0
END
