; ------------------------------------------------------------------------ 
; $Id: readlab.pro,v 1.5 2010/12/09 17:58:56 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
 Pro readlab, fnamein, wdfin, narray = narray
;
;+
; NAME:
;       READLAB
; PURPOSE:
;       This procedure reads a LabView binary file format developed
;         at Sandia National Laboratories.
; CALLING SEQUENCE:
;       READLAB, Filename [, Wdf]
;       READLAB, Wdf
;       READLAB
;
; INPUTS:
;       Filename: Character string containing the filename
;                 If missing DIALOG_PICKFILE will be used to determine the file.
;                 If Filename is present and equal to zero, DIALOG_PICKFILE
;                   will be used to determine file
;       Wdf: WDF array number for the first dataset.
;            Following datasets will be in consecutive WDF arrays.
;            If necessary, SET_MAXWDFARRAY will be used to increase the 
;              number of WDF arrays available.
;            DEFAULT:  WDF = 1
; KEYWORDS:
;       NARRAY:  Number of arrays read
;
; OUTPUT:
;       Wdf: Sandia binary LabView Data in WDF arrays
;
; Technique:
;       Read data from file using the ASCII keywords:
;       Name:  Name of dataset
;       T. Zero: Initial point
;       Delta T: Point spacing
;       Points:  Number of points
;       Data:  Data Values
;
; MODIFICATION HISTORY:
;       Original version, L. P. Mix, October 28, 2003
; 
;   Example:
;       Read a Sandia LabView binary file to be selected from a widget
;         READLAB
;
;       Read a Sandia LabView binary file to be selected from a widget 
;         and store the waveforms beginning in wdfarray 101
;         READLAB, 101
;-
;
     on_error, 1
common saveblab_picker, oldfilter, oldpath
;    check format
     nparams = n_params()
     narray = 0L
     if ((n_elements (fnamein) eq 0) or (nparams eq 0)) then begin
        fname = 0
        wdf = 1L
     endif else begin
        if size(/type, fnamein) eq 7 then begin
          fname= fnamein[0]
          if n_elements(wdfin) gt 0 then wdf = long(wdfin[0]) else wdf =1L
        endif else begin
          fname = 0
          if nparams eq 2 then wdf = long(wdfin[0]) else wdf = long(fnamein[0])
        endelse
     endelse
     if size(/type, fname) lt 7 then begin
       if n_elements(oldpath) eq 0 then cd, current = path else path = oldpath
       fname = dialog_pickfile(/must_exist, /read, path = path, get_path= newpath)
       if (strlen(fname) lt 1) then begin
         print, ' No File was Selected - RETURNING'
         return
       endif
       oldpath = newpath
     endif
;
     vers = strupcase(strmid(!version.os_family, 0, 3))
     if (vers eq 'UNI') then begin
;      get system variable substitution for unix systems
       temp = fname
       FULFNAM, temp, fname
     endif
     fname = file_expand_path(fname)
     if !version.os  eq 'linux' then resolve_it, fname
     info = file_info(fname)
     if (info.exists + info.read) ne 2b then begin
        print, '% READLAB: File does not exist with read permission'
        print, '           File Name = ', fname
        print, '           File Exists =', info.exists
        print, '           Read Permission =', info.read
        return
     endif

     openr, unit, fname, /get_lun, error = ier
     if ier lt 0 then begin
       Print, '% READLAB: Error opening file'
       print, '  Filename: ', fname
       print, !Error_state.msg
       return
     endif

     wdf = (wdf > 1L) < get_maxwdf()
     junkstr = '' 
     clab = ''
     xlab = 'Time (s)'
     ylab = 'Amplitude'
     np =0L
     dt = 0.0
     t0 = 0.0
     while not eof(unit) do begin
       readf, unit, junkstr
       st = 1+strpos(junkstr, ':')
       if st gt 0 then begin
         if strpos(junkstr, 'Name:') ge 0  then begin
           clab = strtrim(strmid(junkstr, st),2)
         endif else if  strpos(junkstr, 'T. Zero:') ge 0 then begin
           reads, strmid(junkstr, st), t0
         endif else if  strpos(junkstr, 'Delta T.:') ge 0 then begin
           reads, strmid(junkstr, st), dt
         endif else if  strpos(junkstr, 'Points:') ge 0 then begin
           reads,  strmid(junkstr, st), np
         endif else if  strpos(junkstr, 'Data:') ge 0 then begin
           if ((np gt 0) and (dt gt 0.0)) then begin
             data = make_array(np, /float)
             readu, unit, data
             data =  swap_endian(data, /swap_if_little_endian) 
             if wdf eq get_maxwdf() then set_maxwdfarray, wdf+64
             put_wdfarray, [t0, dt, data], wdf, file = fname, $
               xlabel = xlab, ylabel = ylab, clabel = clab
             narray = narray+1
             wdf = wdf +1
           endif else begin
             print,'% READLAB: Problem with dataset: ', clab
             Print,'  No data will be stored.'
           endelse
           clab = ''
           np = 0L
           dt = 0.0
           t0=0.0
         endif
       endif 
     endwhile
     free_lun, unit
     end
