; ------------------------------------------------------------------------ 
; $Id: readltd.pro,v 1.1 2011/06/10 17:27:13 lpmix Exp $
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
;+
; NAME:
;       READLTD
;
; PURPOSE:
;       Read HDF5 formatted LTD files.
;       This routine is based on a procedure by Brian Stoltzfus.
;       and adapted to PFIDL with some enhancements.
;       The initial version only reads waveform arrays.
;
; CALLING SEQUENCE:
;       Format:
;         READLTD [, FILENAME] [, WDF] [, WDF = WDF] 
;        [, NWDF = nwdf] [, HELP = help] [, /TRUNCATE]
;        [, FILE_READ = file_read] [, IERROR = ierror]
;
; OPTIONAL INPUTS:
;       FILENAME - Name of the HDF5 LTD file to be opened.
;                  The default file extension is ".hdf"  
;                  If file is omitted or undefined or is not a string,
;                  DIALOG_PICKFILE will be used to select a file.
;       WDF     -  First WDF array for storing the waveform data
;                  Default = 1
;                  If not enough waveform arrays are available, the number will
;                  be increased to read all the waveform data.  If the number 
;                  of waveform arrays is increased, it is incremented in groups
;                  of 500.
;
; KEYWORD PARAMETERS:
;       WDF     - First WDF array for storing the waveform data.  See WDF parameter.
;                 If the parameter exists, then the keyword is ignored.
;       NWDF    - Number of waveforms read into PFIDL.
;       HELP    - If set, a help message is printed.
;       TRUNCATE- If set, use only the signal name as the comment label.  
;                 By default the Shot_Time, Comment, and Description are added to the 
;                 signal name with a " - " separator to allow easy removal from plots.
;       FILE_READ- File opened and read
;       VERBOSE - If set, provide detailed information about data
;       IERROR  - Returns zero if no error.
;
; OUTPUTS:
;       Data will be read and stored in waveform arrays.
;
; PROCEDURE:
;       The HDF5 file 
;
; EXAMPLE:
;       Read data from a file to be selected, and store in WDF 1
;         READLTD
;  < or > READLTD, 0, 1
;  < or > READLTD,  wdf = 1
;  
;       Read data from the file example.hdf and store it starting in WDF array, 1.
;         READLTD, 'example.hdf'
;
;       Read data from an hdf file to be selected by pickfile.  Store the 
;       waveform data beginning with WDF array 21.
;         READLTD, wdf = 21
;  < or > READLTD, 0, 21
;
; MODIFICATION HISTORY:
; 	Written by:     L. P. Mix, June 6, 2011
;       
;-

PRO READLTD , P1, wdf, WDF = WDFKW, NWDF = nwdf, help = helpme, $
       file_read = filename, VERBOSE = verbose, IERROR = ierror, truncate = truncate

on_error, 1
      ierror = -1
;
      nparams = n_params()
      if nparams eq 0 and keyword_set(helpme) then begin
print, '% Format: READLTD [, FILENAME] [, WDF = WDF] [, NWDF = nwdf]'
print, '          [, HELP = help] [, IERROR = ierror] [, VERBOSE = verbose]'
print, '          [, FILE_READ = file_read] [, TRUNCATE = truncate]'
        return
      endif
      if n_elements(wdf) eq 0 then $
        if n_elements(wdfkw) gt 0 then wdf = wdfkw

      if (nparams eq 0) then begin
        filename = ''
        nparams = 1
      endif else if n_elements(p1) eq 0 then begin
        filename = ''
      endif else if (size(p1, /type) ne 7) then begin
        filename = ''
      endif else  filename = strtrim(p1, 2)
      cd, current = current
      
      if strlen(filename) lt 1 then begin
        filter = '*.hdf'
        filename = dialog_pickfile( filter = filter, $
          title= 'Select LTD HDF5 file to read', /must_exist)
        filename = strtrim(filename, 2)
        if strlen(filename) lt 1 then begin
          print, '% READLTD: No File was Selected - RETURNING'
          return
        endif
      endif else filename = p1
;
      info = file_info(filename)
      if (not info.exists) then begin
        print, '% READLTD: File does not exist'
        print, '          ', filename
        filename =  dialog_pickfile(/must_exist, /read)
        if strlen(filename) lt 1 then begin
          print, '% READLTD: No File was Selected - RETURNING'
          return
        endif
        info = file_info(filename)
      endif
      if (not info.read) then begin
        print, '% READLTD: File is not readable by the user'
        print, '          ', filename
        return
      endif

      mxwdf = get_maxwdf()

      if n_elements(wdf) eq 0 then wdf = 1 else wdf = (wdf[0]>1) < (mxwdf+1)
      wdf = long(wdf)
      wdfinit = wdf
      wdf_inc = 500
      nwdf = 0L


;*** HDF5 FORMAT used by LTD

;open file and get the number of folders at the top level (level 1)
  if H5F_IS_HDF5(filename) then file_id=H5F_OPEN(filename) else begin
    print, '% READLTD: File is not an HDF5 file type: '+ filename
    return
  endelse
  if keyword_set(verbose) then  print, '% READLTD: Reading ', filename
  num_level1=h5g_get_num_objs(file_id)

;start level 1 for loop over top levels
  for n=0,num_level1-1 do begin
    temp_name=h5g_get_obj_name_by_idx(file_id,n)
    temp1_id=h5g_open(file_id,temp_name)
    num_subfolders=h5g_get_num_objs(temp1_id)
  
; start level 2 for loop
    for i=0, num_subfolders-1 do begin
      temp_name=h5g_get_obj_name_by_idx(temp1_id,i)
      temp2_id=h5g_open(temp1_id,temp_name)
;     Get the waveform data name, get the id, read the data and save it into a temp struct
      clabel=h5g_get_obj_name_by_idx(temp2_id,0)
      temp_data_id=h5d_open(temp2_id,clabel)
      data=h5d_read(temp_data_id)
      np = n_elements(data)
      if np gt 2 then begin
        tzero = 0
        delta_t = 0
        yunits = ''
        xunits = ''
        append = ' - '
        npts =0L

;       Read the attributes and process them
        num_attrib=h5a_get_num_attrs(temp2_id)
        for j=0, num_attrib-1 do begin
          temp_attrib_id=h5a_open_idx(temp2_id, j)
          temp_attrib_name=h5a_get_name(temp_attrib_id)
          temp_data=h5a_read(temp_attrib_id)
          case temp_attrib_name of
            'color': 
            'comment': append = apend + 'COMMENT='+strtrim(temp_data, 2)+';;'
            'delta_t': delta_t=temp_data
            'descrip': append = apend + 'DESCRIPTION='+strtrim(temp_data, 2)+';;'
            'name': if strupcase(clabel) ne strupcase(temp_data) then clabel = temp_data
            'npts': npts=temp_data
            'shot_time': append = apend + 'SHOT_TIME='+strtrim(temp_data, 2)+';;'
            'tzero':  tzero=temp_data
            'xunits': xlabel = temp_data
            'yunits': ylabel = temp_data
             else: print,'new attribute name'
          endcase
        endfor
        if (size(/type, delta_t) gt 3) and (size(/type, tzero) gt 3) then begin
          if (strlen(append) gt 3) && (not keyword_set(truncate)) then clabel = clabel + appen
          if keyword_set(verbose) then print, 'Waveform '+clabel+' saved in WDF '+strtrim(wdf, 2)
          put_wdfarray, float([tzero, delta_t, data]), wdf, clabel = clabel, xlabel = xlabel, $
            ylabel = ylabel, file = filename, /no_copy
          wdf += 1
          if wdf ge mxwdf then begin
            mxwdf += wdf_inc
            set_maxwdfarray, mxwdf
          endif
        endif
      endif
    endfor
  endfor
  nwdf = wdf - wdfinit  
  wdf = wdfinit
  wdfkw = wdfinit
  if nwdf gt 0 then ierror =0
end
