
; ------------------------------------------------------------------------
; $Id: readprism.pro,v 1.1 2011/06/17 18:24:36 lpmix Exp $
; ------------------------------------------------------------------------
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
;+
; NAME:
;       READPRISM
;
; PURPOSE:
;       Read a PrismPlot 2D ASCII data file.
;       The file format for each dataset begins as shown below.
;       The temperature and density are in the line with Dataset:
;       Each file may contain multiple datasets.
;      The first few lines from a file are below:
; # PrismPlot 2D Data File
; # Format ID = 2
; # 
; # 
; # Dataset: 170 / 3.5e+20 / 0
; # Number of points = 1972
; #        X                Y
;   8.856014e+000   9.997924e-001
;   8.853409e+000   9.997924e-001
;   8.850805e+000   9.997923e-001
;   < followed by 1969 additional xy pairs>
;
; CALLING SEQUENCE:
;
;       READprism, WDF [, FILE]
;
; INPUTS:
;       WDF:    Initial WDF array number
;
; OPTIONAL INPUTS:
;       FILE:   File containing the PrismPlot datasets
;               If missing, DIALOG_PICKFILE will be called.
;	
; KEYWORD PARAMETERS:
;       NWDF:      Return the number of datasets read
;       TRUNCATE:  If set, do not add density, temperature information
;
; OUTPUTS:
;       All datasets in the file are stored in WDF arrays
;
; PROCEDURE:
;       REASC is used to read each dataset.
;
; EXAMPLE:
;       Read the datasets in the file, "alrun3_grid_trans_170_instr850.ppd", store
;       the datasets starting in wdf array, 21, and the number of datasets in n
;         READPRISM, 21, " alrun3_grid_trans_170_instr850.ppd", nwdf = n
;
; MODIFICATION HISTORY:
; 	Written by:	L. P. Mix, Jr., June 10, 2011
;
;-

pro readprism, wdf, filename, truncate = truncate, nwdf = nwdf
on_error, 1
  npar = n_params()
  if npar eq 0 then begin
     print, '% Format: READPRISM, WDF [, FILE] [, NWDF = nwdf]'
     print, '          [, TRUNCATE = truncate]'
     return
  endif
  if n_elements (wdf) eq 0 then begin
    Print, '% READPRISM:  Initial WDF array number must be specified'
    return
  endif
  if n_elements(filename) eq 0 then begin
    filename = dialog_pickfile()
    if strlen(filename) lt 1 then begin
      print, '% READPRISM:  No File was Selected - RETURNING'
      return
    endif
  endif else begin
    vers = strupcase(strmid(!version.os_family, 0, 3))
    if (vers eq 'UNI') then begin
;     get system variable substitution for unix systems
      temp = filename
      FULFNAM, temp, filename
    endif
    if !version.os eq 'linux' then resolve_it, filename
  endelse
  filename = file_expand_path(filename)
  info = file_info(filename)
  if (info.exists + info.read) ne 2b then begin
     print, '% READPRISM: File does not exist with read permission'
     print, '         File Name = ', filename
     print, '         File Exists =', info.exists
     print, '         Read Permission =', info.read
     ier = -2
     return
  endif
  str = ''
  wdfini = wdf
  s1 = 'Dataset:'
  ns1 = strlen(s1)
  s2 = 'Number of points ='
  ns2 = strlen(s2)
  openr, unit, /get_lun, filename, error = ier
  if ier ne 0 then begin
    print, !err_string
    print, '% READPRISM:  Error opening file: '+filename
    return
  endif
  on_ioerror, endit
  while not eof(unit) do begin
    i = -1
    while i lt 0 and not eof(unit) do begin
      readf, unit, str
      i = strpos(str, s1)
    endwhile
    if i lt 0 then goto, endit
    comment = strtrim(strmid(str, i+ns1), 2)
    i = -1 
    while i lt 0 and not eof(unit) do begin 
      readf, unit, str
      i = strpos(str,s2) 
    endwhile
    if i lt 0 then goto, endit
    reads, strmid(str, i+ns2), np
    reasc, unit, formatin = 0, 1, wdf, skip = 1, limit = round(np)
    lab, wdf, ' - '+comment, /append
    wdf += 1
  endwhile
  endit: nwdf = wdf - wdfini
  free_lun, unit
end
