; ------------------------------------------------------------------------ 
; $Id: readsig.pro,v 1.7 2010/12/09 17:14:13 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
 Pro readsig, filename, init, dt, y, np = np, double = double, $
      xlabel = xlabel, ylabel = ylabel, clabel = clabel,  $
      initial = initial, delta_t = deltat
;
;+
; NAME: 
;       READSIG
; PURPOSE:
;       This procedure reads a LLNL SIG uniform format ASCII data file.
;       Present version written entirely in IDL but requires IDL 5.6
; CATEGORY:
;       Customization
; CALLING SEQUENCE:
;       READSIG, Filename, Wdf
;       READSIG, Wdf
;       READSIG, Filename, Init, Dt, Y-values
;       READSIG, Init, Dt, Y_Val
;
; INPUTS:
;       Filename: character string containing the filename
;       Wdf: WDF array number
; OUTPUT:
;       Wdf: SIG data in WDF array
;       INIT: Initial data point
;       DT  : Point spacing 
;       Y_Val : Y-values for the data
; KEYWORDS:
;      NP: Actual number of points read including init and dt.
;  DOUBLE: If set, file will be read in double precision
; INITIAL: Value of initial point
; DELTA_T: Data Spacing
; XLABEL, YLABEL, CLABEL - Only used with 1 or 2 parameters
;          Dataset abscissa, ordinate and comment labels.
;          Default value for XLABEL and YLABEL is ' '.
;          Default value for CLABEL is FILENAME. 
; MODIFICATION HISTORY:
;       Original version, M. F. Pasik, August 14, 1996
;       Modified to include different outputs and to use a Fortran read.8/14/96
;
;   Example:
;       Read data1.sig into idl variables and return the number of points in J
;           j = 0
;           readsig, 'data1.sig', t0, dt, y, np =j
;
;       Read data1.sig into WDF array 5 with labels
;           readsig, 'data1.sig', 5, xlabel = 'History', $
;             Ylabel = '# of Particles' , Clabel = 'QS Particle Creation' 
;-
;
@com.dat
 common pffcom, initpff, pff_exe
 common savesig_picker, oldfilter, oldpath
;
 on_error, 1
;
      if (n_elements(initpff) eq 0) then init_pff else $
        if (initpff eq 0) then init_pff
      if initpff eq 0 then begin
        print, '% READSIG: SIG library is not supported by this Operating System'
        return
      endif
;
      if n_elements(initwdf) eq 0 then wdfinit
 npar = n_params()
if npar EQ 0 then begin
  print, '% Format: READSIG, FILENAME, WDF'
  print, '       or READSIG, WDF'
  print, '       or READSIG, FILENAME, INIT, DT, Y_VAL'
  print, '       or READSIG, INIT, DT, Y_VAL '
  print, '   Keywords: NP, DOUBLE, INITIAL, DELTA_T, XLABEL, YLABEL, CLABEL'
  return
endif


if npar EQ 1 or npar eq 3 then begin
  if npar eq 1 then begin
    if n_elements(filename) eq 0 then begin
      print, '% READSIG: WDF array must be specified'
      return
    endif
    wdf = filename
    npar = 2
  endif
  if n_elements(oldpath) eq 0 then cd, current = path else path = oldpath  
  filename = dialog_pickfile(/must_exist, /read, path = path, get_path= newpath)
  if (strlen(filename) lt 1) then begin
    print, ' No File was Selected - RETURNING'
    return
  endif
  oldpath = newpath
endif else if npar eq 2 then begin
  if n_elements(init) eq 0 then begin
      print, '% READSIG: WDF array must be specified'
      return
  endif
  wdf = init
endif

if npar eq 2 then begin
  if (wdf lt 1 or wdf gt maxwdfarray) then begin
    print, 'WDF array must be greater than 0 and </=', maxwdfarray
    return
  endif 
  filein = filename
endif
;
     vers = strupcase(strmid(!version.os_family, 0, 3))
     if (vers eq 'UNI') then begin
;      get system variable substitution for unix systems
       temp = filename
       FULFNAM, temp, filename
     endif
     if ((npar eq 2) or (npar eq 4)) then filein = filename
     filename = file_expand_path(filename)
     if !version.os  eq 'linux' then resolve_it, filename
     if n_elements (filein) eq 0 then filein = filename
     info = file_info(filename)
     if (info.exists + info.read) ne 2b then begin
        print, '% READSIG: File does not exist with read permission'
        print, '           File Name = ', filename
        print, '           File Exists =', info.exists
        print, '           Read Permission =', info.read
        return
     endif

     np = file_lines(filename)-3
     y = (keyword_set(double)) ?  make_array(np, /double) : make_array(np)
     initial = (keyword_set(double)) ? 0.0d0 : 0.0
     deltat = (keyword_set(double)) ? 0.0d0 : 0.0

     openr, unit, filename, /get_lun, error = ier
     if ier lt 0 then begin
       Print, '% READSIG: Error opening file'
       print, !Error_state.msg
       return
     endif

     on_ioerror, badnp
     readf, unit, initial, deltat, format='(4x, g30)'

     junkstr = ''
     readf, unit, junkstr
     readf, unit, y
     free_lun, unit
   if npar le 2 then begin
     if n_elements(clabel) eq 0 then clabel = filein
     if n_elements(xlabel) eq 0 then xlabel = ' '
     if n_elements(ylabel) eq 0 then ylabel = ' '
     put_wdfarray,[initial, deltat, y],wdf,/no_copy,file=filename, $
       clabel = clabel, xlab=xlabel ,ylab=ylabel, xydata=0
   endif else if npar eq 3 then begin
     filename = initial
     init = deltat
     dt = y
   endif else if npar eq 4 then begin
     init = initial
     dt = deltat
   endif
   return
   badnp: 
     print, '% READSIG: Error reading file'
     free_lun, unit
   end
