; ------------------------------------------------------------------------ 
; $Id: readtek3.pro,v 1.3 2007/06/06 20:03:52 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
 Pro readtek3, fnamein, wdfin, ndim = ndim, group = group,narray = narray, $
              maxnp = maxnp, maxblock = maxblock
;
;+
;    read an ascii data file of the following form:
;    This is a "tek plot" file from mach2
;       new style with data in output one variable at a time.
;       ie all the x, then all the y, ...
; 
; TITLE = "restart file, 1 mm shell"
; VARIABLES = "x", "y", "Ux", "Uy", ...., "Mat#"
; ZONE T= "Block 1", I = 93, J = 161  F= POINT
; data - block 1
; ZONE T= "Block 2", I = 93, J = 161  F= POINT
; data - block 2
; etc
;    format is:  readtek3 [, fname] [, WDF] [, label=label] [maxblock = maxblock]
;                       [, npoints =np] [, Narray = narray] [Maxnp = maxnp]
;    where 
;      fname   - the file name
;                if a file name then the file is opened and closed at the end
;                If fname is zero or omitted then PICKFILE will be used to get
;                  the file name.
;      NOTE: If only one parameter is given and the parameter is a number, 
;            then ReadTek3 assumes that it is a STR number.
;      STR     - Structure array number or an array of numbers.
;                Default is 1  
;      narray  - number of arrays read
;          The following keyword is ignored in this version ie NDIM ==2
;      NDIM    - Number of Dimensions - Default = 2
;      Group   - Grouping of arrays into datasets.
;                Default = [ 1, 1, 1, 1, 1]
;      MaxNP   - Maximum number of points, default current version = 131072
;      Maxblock- Maximum number of blocks, default current version = 8
;
; EXAMPLES:
;    Read a file and store the data beginning in WDF array 1.
;       READTEK3, 0
;   or  READTEK3, 1 
;    Note: Default WDF array is 1 but procedure must be called with at
;          least 1 parameter.
; 
;    Read a file and store the arrays starting in wdfarray 10
;       READTek3, 10 
;   or  READTek3, 0, 10
;  
;    Read file 'DAT1234' and store the data in WDF array 1
;       READTek3, 'DAT1234', 1
;   or  READTek3, 'DAT1234'         ; since WDF = 1 is default
;
;    Read a file and store in arrays [1, 3, 5, 7, 9, 11]
;       READTek3, 1+2*indgen(6)
;    Note: If there are more than 6 waveforms, additional waveforms will be
;            stored in 12, 13, ....
;          If there are less than 6 waveforms, no data will be stored in the
;            additional WDF arrays.
;
;-
;
     on_error, 1
;    check format
     nparams = n_params()
      if nparams eq 0 then begin
        print, '% Format: READTK [, FNAME] [, STR] [, NDIM = ndim] 
        print, '           [, GROUP = group] [, NARRAY = narray]'
        return
      endif
      if n_elements (fnamein) eq 0 then fnamein = 0
      fname = fnamein(0)
      b = size(fname)
      if b(1) eq 7 then begin
        openr,unit,fname, /get_lun
        lcfile = 1
      endif else begin
        fname = dialog_pickfile(/must_exist, /read)
        if strlen(fname) lt 1 then begin
          print, '% READTK: No File was Selected - RETURNING'
          return
        endif
        openr,unit,fname, /get_lun
        lcfile = 0
      endelse
;     if n_elements(ndim) eq 0 then Ndim = 2L else ndim = long(ndim) > 1L
;     ndim = ndim < 3L
      NDIM = 2
      junkstr = '' 
      readf, unit, junkstr
      ll = strlen(junkstr)
      a = strpos(junkstr, 'TITLE =')
      if a lt 0 then begin
        Print, '% READTek3: First line must contain "TITLE ="'
        return
      endif
      junkstr = strmid(junkstr, a+7, ll)
      quotes = 34B
      space = 32B
      junkb = byte (junkstr)
      ok = where (junkb eq quotes, nok)
      if nok gt 0 then junkb(ok) = space
      title  = strtrim (junkb, 2)
      
      readf, unit, junkstr
      ll = strlen(junkstr)
      a = strpos(junkstr, 'VARIABLES =')
      if a lt 0 then begin
        Print, '% READTek3: Second line must contain "VARIABLES ="'
        return
      endif
      comma = 44B   
      junkstr = strmid(junkstr, a+12, ll)
      parser, junkstr,delimiter = comma,ignore = quotes, name, narray,/stringarray
      if narray eq 0 then begin
        Print, '% READTek3: Dataset Labels must be present for each dataset'
        free_lun, unit
        return
      endif
      if min(strlen(name)) eq 0 then begin
        i =0
        for j = 0, narray-1 do begin
          if strlen(name(j)) gt 0 then begin
            name(i) = name(j)
            i = i+1
          endif
        endfor
        narray = i
      endif
      if narray eq 0 then begin
        Print, '% READTek3: Dataset Labels must be present for each dataset'
        free_lun, unit
        return
      endif
      if n_elements (group) eq 0 then group = [1, 1, 1, 1, 1]
      if ndim + total (group) ne narray then print, '% READTek3: Warning' + $
          '-Total of group and ndim do not equal the number of dataset labels'
      np = 0
      maxblock = 8
      maxnp    = 131072L
      locx = make_array(/long,3, maxblock+1) 
      locv = make_array(/long, maxblock+1) 
      array = make_array(maxnp, narray-ndim)
      space = make_array(maxnp, ndim)
      blabel = make_array(maxblock, /str)
      ssize = locx
      range = make_array(2, 3, maxblock+1) 
      nblo= 0
      while ((not eof(unit)) and (nblo lt maxblock)) do begin
        nblo = nblo+1
        readf, unit, junkstr
        a = strpos(junkstr, 'ZONE T=')
        if a lt 0 then begin
          Print, '% READTek3: First line of block must contain "ZONE T="'
          return
        endif
        junkstr = strmid(junkstr, a+7, ll)
        a = strpos(junkstr, 'I=')
        if a lt 0 then begin
          Print, '% READTek3: First line of block must contain "I="'
          return
        endif
        junkstr = strmid(junkstr, a+2, ll)
        a = strpos(junkstr, ',')
        nx = 0L
        ny = 0L
        reads, strmid(junkstr, 0, a), nx
        a = strpos(junkstr, 'J=')
        if a lt 0 then begin
          Print, '% READTek3: First line of block must contain "J="'
          return
        endif
        junkstr = strmid(junkstr, a+2, ll)
        a = strpos(junkstr, 'F', 1)
        reads, strmid(junkstr, 0, a), ny
        ntotal = nx*ny
        np = np+ntotal
        if np gt maxnp then begin
          print,'% READTek3: Maximum number of points exceeded in block', nblo
          print,'            Enter command with a value of maxnp > ', np
          return
        endif
        tmp = make_array(ntotal, narray)
  print,' NBLOCK =', nblo, ', I=', nx, ', J=', ny
        readf,unit, tmp
        locx(0, nblo) = nx+locx(0, nblo-1)
        locx(1, nblo) = ny+locx(1, nblo-1)
        locv(nblo) = locv(nblo-1)+ ntotal
        ssize(*, nblo-1)=[nx, ny, 1]
        space(locx(0, nblo-1):locx(0, nblo)-1, 0) = tmp(0:nx-1, 0)
        range(0, 0, nblo) = min(space(locx(0,nblo-1):locx(0,nblo)-1, 0), max=mm)
        range(1, 0, nblo) = mm
        space(locx(1, nblo-1):locx(1, nblo)-1, 1) = tmp( nx*lindgen(ny), 1)
        range(0, 1, nblo) = min(space(locx(1,nblo-1):locx(1,nblo)-1, 1), max=mm)
        range(1, 1, nblo) = mm
            print , 'Xrange = ', range(*, 0, nblo)
            print , 'Yrange = ', range(*, 1, nblo)
        for i = 2, narray-1 do  $
          array(locv(nblo-1):locv(nblo)-1, i-2) = tmp( *, i)
      endwhile

      free_lun, unit


      locx = locx(*,0: nblo)
      locv = locv(0:nblo)
      x1 = space(0:locx(0, nblo)-1, 0)
      x2 = space(0:locx(1, nblo)-1, 1)
      if ndim eq 2 then begin
        x3 = make_array(nblo)
        locx(2,*) = indgen(nblo+1)
      endif
      
      range(0, 0, 0) = min(range(0, 0:ndim-1, 1:*))
      range(1, 0, 0) = max(range(1, 0:ndim-1, 1:*))
      narray = narray-ndim
      ssize = ssize(*, 0:nblo-1)
      
      if nparams eq 1 then begin
        if lcfile eq 1 then wdft = 1+indgen(narray) $
          else wdft = fnamein
      endif else begin
        if n_elements (wdfin) eq 0 then wdft = 1 else wdft = wdfin
      endelse
        group = group > 1
        group = group < 3
        nelg = n_elements(group )
        tg = total(group)
        if tg lt  narray  then begin
          group = [group, make_array(value= 1, narray-tg)]
          nelg = n_elements(group)
        endif else if tg gt narray then begin
          group = make_array(value= 1, narray)
          nelg = narray
        endif
        maxwdf = get_maxstr()
        ok = where (wdft gt 0 and wdft le maxwdf, nwdf)
        if nwdf eq 0 then wdf = 1+indgen(nelg) else begin
          wdf = wdft(ok)
          if nwdf lt nelg then $
            wdf = [wdf, wdf(nwdf-1) + indgen(nelg-nwdf)+1]
        endelse
        if wdf(nelg-1) gt maxwdf then set_maxstrarray, wdf(nelg-1)
        ivec = 0
        
        x1lab = make_array(nblo, /str, val=name(0))
        x2lab = make_array(nblo, /str, val=name(1))
        x3lab = make_array(nblo, /str, val=' ')
        tlab = 'Ascii Input'
        if nblo gt 1 then begin
          order = make_array(ndim, nblo, /long)
          for i = 0, ndim-1 do order (i, *) = sort(range(0,i, 1:nblo))
        endif else order = 1L
        v1lab= make_array(nblo, /str)
        v2lab= make_array(nblo, /str)
        v3lab= make_array(nblo, /str)

        for i = 0,nelg-1 do begin
          v1 = array(0:locv(nblo)-1, ivec)
          v1lab(*) = name(ivec+ndim)
          clab = v1lab(0)
          ivec = ivec+1
          if group(i) gt 1 then begin
            v2 = array(0:locv(nblo)-1, ivec)
            v2lab(*) = name(ivec+ndim)
            clab = clab+', '+v2lab(0)
            ivec = ivec+1
            if group(i) gt 2 then begin
              v3 = array(0:locv(nblo)-1, ivec)
              v3lab(*) = name(ivec+ndim)
              clab = clab+', '+v3lab(0)
              ivec = ivec+1
            endif else begin
              v3 = 0.0
              v3lab(*) = ''
            endelse
          endif else begin
            v2 = 0.0
            v2lab(*) = ''
            v3 = 0.0
            v3lab(*) = ''
          endelse
          clab = clab+' - '+title

           
      tempstr = {type:1L, ndim:3L, nvect: group(i), block :nblo, two: 3L, $
              size:ssize,range: range, order: order, slice:0.0 , locx:locx, $
              locv: locv, x1:x1, x2:x2, x3:x3, x4:0.0, $
              v1:v1, v2:v2, v3:v3, v4:0.0, v5:0.0, v6:0.0, $
              x1lab : x1lab, x2lab : x2lab, x3lab : x3lab, x4lab :'', $
              v1lab : v1lab, v2lab :v2lab, v3lab : v3lab, $
              v4lab : '',v5lab : '', $
              v6lab : '', file : fname, spare : [0,0,0,0,0], tlab : tlab, $
              clab: clab }

          i2s, tempstr, wdf(i)
        endfor
        return
        end
