; ------------------------------------------------------------------------
; $Id: readtiff.pro,v 1.8 2011/06/10 17:21:32 lpmix Exp $
; ------------------------------------------------------------------------
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
;+
; NAME:
;       READTIFF
;
; PURPOSE:
;	Read a tiff file and return the image, a structure or store it
;         in a PFIDL structure array
;
; CALLING SEQUENCE:
;  Format:  READTIFF, STRUCTURE  [, FILE] [, DX = dx] [, DY = dy] 
;           [, SCALE = scale] [, CLABEL = clabel] [, XLABEL = xlabel]
;           [, YLABEL = ylabel] [, TLABEL = tlabel] [, REVERSE = reverse]
;           [, FILENAME = filename]
;           [, /IMAGE] [, /STRUCTURE] [, /INTEGER]
;
; INPUTS:
;	STRUCTURE  - Structure Array Number or variable if STRUCTURE 
;                    or IMAGE is set.
;       FILE       - TIFF file name
;                    If zero of undefined, DIALOG_PICKFILE will be used.
;       BARE_NAME  - If set, strip the path and extension from the filename for CLABEL
;
; KEYWORD PARAMETERS:
;       DX         - X increment 
;                    DEFAULT: DX = 1
;       DY         - Y increment
;                    DEFAULT: DY = 1 
;       SCALE      - Intensity Scale Factor
;                    DEFAULT: SCALE = 1
;       CLABEL     - Dataset comment for the structure
;                    DEFAULT: CLABEL = FILE
;       XLABEL     - Abscissa comment for the structure
;                    DEFAULT: XLABEL = 'X'
;       YLABEL     - Ordinate comment for the structure
;                    DEFAULT: YLABEL = 'Y'
;       TLABEL     - Type label for the structure
;                    DEFAULT: TLABEL = 'TIFF Image'
;       IMAGE      - If set, the tiff image is returned in the variable STRUCTURE
;       FILENAME   - Filename actually read
;       STRUCTURE  - If set and if image is not set, the structure is returned in
;                    the variable STRUCTURE
;       INTEGER    - If set, save the image as an integer array
;       REVERSEIMAG- If not set to zero, image will be reversed.
;                    IDL has a convention from bottom to top; TIFF stores 
;                    image top to bottom.  Image will be reversed unless REVERSE
;                    is set to zero (ie. REVERSE = 0)
;                    DEFAULT:  REVERSE = 1
;       
; OUTPUTS:
;	A tiff image is stored in a structure array or returned either as an 
;       image or a structure.
;
; RESTRICTIONS:
;       The procedure handles standard tiff files.
;
; PROCEDURE:
;       The procedure attempts to use the IDL procedure READ_TIFF to read the file.
;       If this is unsuccessful, it attempts to use a special tiff reader.
;
; EXAMPLE:
;       Read the tiff image, "shot2122.tiff" and store the image in structure 2
;         readtiff, 2, 'shot2122.tiff'
;
;	Return the image from  "shot2122.tiff" in variable imag2122.
;         readtiff, imag2122, 'shot2122.tiff'
;
;       Return the image as a structure in variable str from a file to be selected 
;       using DIALOG_PICKFILE
;          readtiff, str, /structure
;
; MODIFICATION HISTORY:
; 	Written by:	L. P. Mix, August 7, 2003
;
;-
;2345678901234567890123456789012345678901234567890123456789012345678901234567890
;
PRO ReadTiff, STR, FILE, DX = dx, DY = dy, SCALE = scale, CLABEL = clabel, $
       XLABEL = xlabel, YLABEL = ylabel, TLABEL = tlabel, IMAGE= image, $
       FILENAME = filename, BARE_NAME = barename, $
       STRUCTURE = structure, REVERSEimag = reverseimag, integer = integerarr

on_error, 1
;forward_function reverse, get_save_str, my_read_tiff

 
; Common block
@qsstrcom.dat
;
      if n_elements (structinit) eq 0 then qsstruct_init
;
      nparams = n_params()
      if nparams eq  0 then begin
        print, '% Format:', $
           '  READTIFF, STRUCTURE  [, FILE] [, DX = dx] [, DY = dy]'
        print, '   [, SCALE = scale] [, CLABEL = clabel] [, /REVERSEIMAG]'
        print, '   [, XLABEL = xlabel] [, YLABEL = ylabel] [, TLABEL = tlabel]'
        print, '   [, FILENAME = filename] [, /IMAGE] [, /STRUCTURE]'
        print, '   [, /BARE_NAME] [, /INTEGER]'
        return
      endif
      if not keyword_set(image) and not keyword_set(structure) then begin
        if n_elements(str) eq 0 then begin
          print, '% READTIFF: Structure array number must be defined'
          return
        endif
        strsave = get_save_str(str, get_maxstr())
      endif else if not arg_present(str) then begin
        print, '% READTIFF: STRUCTURE must be a variable to return a value'
        if keyword_set(image) then print, '           IMAGE is set'
        if keyword_set(structure) then print, '           STRUCTURE is set'
        return
      endif

      if n_elements(reverseimag) eq 0 then reverseimag = 1
      if n_elements(file) eq  0 then fname = 0 else fname = file[0]
      b = size(fname)
      if b(1) eq 7 then begin
        resolve_it, fname
        info = file_info(fname)
        if (info.exists + info.read) ne 2b then begin
           print, '% READTIFF: File does not exist with read permission'
           print, '         File Name = ', fname
           print, '         File Exists =', info.exists
           print, '         Read Permission =', info.read
           return
        endif
      endif else begin
        fname = dialog_pickfile(/must_exist, /read)
        if strlen(fname) lt 1 then begin
          print, '% READTIFF: No File was Selected - RETURNING'
          return
        endif
        resolve_it, fname
      endelse
      if n_elements(clabel) eq 0 then $
        clabel = keyword_set(barename) ? get_bare_filename(fname) : fname
      aa = 0
      catch, read_tiff_error
      if  read_tiff_error ne 0 then aa =1
      if aa eq 0 then str = read_tiff(fname) else begin
        catch, /cancel
        str = my_read_tiff(fname)
      endelse
      filename = fname
      if n_elements(str) lt 10 then begin
        print, '% READTIFF: Invalid image returned.'
        help, str
        return
      endif
      if keyword_set(reverseimag) then begin     
        ss = size(str)
        str= reverse(str, ss[0], /overwrite)
      endif
      if n_elements(scale) gt 0 then str = str*scale[0]
;     If only image is requested then return
      if keyword_set(image) then return
      s = size (str)
      if s[0] eq 3 then begin
        if s[1] ne 3 then begin
          print, '% READTIFF: Cannot return a structure at this time'
          print, '            Returning image in the structure'
          return
        endif
        v1 = reform(str[0, *, *])
        v2 = reform(str[1, *, *])
        v3 = reform(str[2, *, *])
        if not keyword_set(integerarr) then begin
          v1 = float(v1)
          v2 = float(v2)
          v3 = float(v3)
        endif
        nx = s[2]
        ny = s[3]
        ndim = 3
      endif else begin
        ndim = 2
        nx = s[1]
        ny = s[2]
        v1 = keyword_set(integerarr) ? temporary(str) :  float(str)
      endelse
      if n_elements(dx) eq 0 then dx = 1.0
      x = findgen(nx)*dx[0]
      if n_elements(dy) eq 0 then dy = 1.0
      y = findgen(ny)*dy[0]
      if ndim eq 2 then $       
        arr2str, str, /str, v1, x, y, dx = dx[0], dy= dy[0], $
           clab = clabel, XLABEL = xlabel, YLABEL = ylabel, TLABEL = tlabel $
      else $
        arr2str, str, /str, v1, v2, v3, x, y, dx = dx[0], dy= dy[0], $
           clab = clabel, XLABEL = xlabel, YLABEL = ylabel, TLABEL = tlabel 
      if keyword_set(structure) then return
      i2s, str, strsave, /no_copy
      str = strsave
      end
