; ------------------------------------------------------------------------ 
; $Id: readtk.pro,v 1.3 2007/06/06 20:03:52 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
 Pro readtk, fnamein, wdfin, data= data, label = name, $
       npoints= np, narray = narray
;
;+
;    read an tkdas data file
;       Code assumes first line contains 1 non-blank label for each column of data
;     Format consists of tab delimited columns of the form:
;     Note: Each Title may not have spaces in this version.
;  title
;  t0
;  delta_t
;  Npoints
;  y1  (at t0)
;  y2
;  y3
;  .
;  .
;  .
;  yN (at t0+(N-1)*delta_t)
;
;    format is:  readtk [, fname] [, WDF] [, data = data] [, label=label] 
;                       [, npoints =np] [, Narray = narray]
;    where 
;      fname   - the file name
;                if a file name then the file is opened and closed at the end
;                If fname is zero or omitted then PICKFILE will be used to get
;                  the file name.
;      NOTE: If only one parameter is given and the parameter is a number, 
;            then ReadTK assumes that it is a WDF number.
;      WDF     - WDF array number or an array of numbers.
;                Default is 1  
;                If WDF is less than 0 and n_elements(data) > 0 then no WDF
;                  arrays will be stored.
;      DATA    - Input data data = data (npoints+2, narray)
;                (Initial time and delta time are first two points.)
;      label   - string array of the names of the arrays
;      npoints - number of data values read for each dataset
;      narray  - number of arrays read
; EXAMPLES:
;    Read a TKDAS file and store the data beginning in WDF array 1.
;       READTK, 0
;   or  READTK, 1 
;    Note: Default WDF array is 1 but procedure must be called with at
;          least 1 parameter.
; 
;    Read a TKDAS file and store the arrays starting in wdfarray 10
;       READTK, 10 
;   or  READTK, 0, 10
;  
;    Read TKDAS file 'DAT1234' and store the data in WDF array 1
;       READTK, 'DAT1234', 1
;   or  READTK, 'DAT1234'         ; since WDF = 1 is default
;
;    Read a TKDAS file and store in arrays [1, 3, 5, 7, 9, 11]
;       READTK, 1+2*indgen(6)
;    Note: If there are more than 6 waveforms, additional waveforms will be
;            stored in 12, 13, ....
;          If there are less than 6 waveforms, no data will be stored in the
;            additional WDF arrays.
;
;    Read the data from a TKDAS file into an array DATA and store data beginning
;      in wdf array 1.
;         READTK, 0, data = data
;    To only return the array without storing data in WDF arrays then:
;         READTK, -1, data = data
;-
;
     on_error, 1
;    check format
     nparams = n_params()
      if nparams eq 0 then begin
        print, '% Format: READTK [, FNAME] [, WDF] [, DATA = data]
        print, '           [, LABEL = label] [NPOINTS = np] [, NARRAY = narray]'
        return
      endif
      if n_elements (fnamein) eq 0 then fnamein = 0
      fname = fnamein(0)
      b = size(fname)
      if b(1) eq 7 then begin
        openr,unit,fname, /get_lun
        lcfile = 1
      endif else begin
        fname = dialog_pickfile(/must_exist, /read)
        if strlen(fname) lt 1 then begin
          print, '% READTK: No File was Selected - RETURNING'
          return
        endif
        openr,unit,fname, /get_lun
        lcfile = 0
      endelse
      junkstr = '' 
      on_ioerror, noskipdata
      readf, unit, junkstr
      parser, junkstr,delimiter = 9B, name, narray, /stringarray
      if narray eq 0 then begin
        Print, '% READTK: Dataset Labels must be present for each dataset'
        free_lun, unit
        return
      endif
      t0 = make_array(narray)
      dt = make_array(narray)
      np = make_array(narray)
      ax = make_array(narray)
      on_ioerror, badeof
          readf,unit,t0
          readf,unit,dt
          readf,unit,np
      np = round(np)
      groups = np(uniq(np, sort(np)))
      ngroup = n_elements(groups)
      delta = groups 
      if ngroup gt 1 then delta(1:*) = groups(1:ngroup-1)-groups(0:ngroup-2)
      maxnp = max(np, min = minnp)+2
      bx = make_array (maxnp, narray)
      bx(0, *) = t0
      bx(1, *) = dt
      i=2L
      groups2 = groups+2
      for  ng = 0, (ngroup-1) do begin
        ok = where (np ge groups(ng), nok)
        ax = temporary(ax(0:nok-1))
        while ((not eof(unit)) and i lt groups2(ng)) do begin
          on_ioerror, badeof
          readf,unit,ax
          bx(i, ok) = ax
          i = i+1
        endwhile
      endfor
badeof:
      free_lun, unit
      if i eq 0 then begin
        np = 0
        print, '%READTK: Error reading file -- No data found'
        return
      endif
      if nparams eq 1 then begin
        if lcfile eq 1 then wdft = 1+indgen(narray) $
          else wdft = fnamein
      endif else begin
        if n_elements (wdfin) eq 0 then wdft = 1 else wdft = wdfin
      endelse
      if wdft(0) ge 0 then begin
        maxwdf = get_maxwdf()
        ok = where (wdft gt 0 and wdft le maxwdf, nwdf)
        if nwdf eq 0 then wdf = 1+indgen(narray) else begin
          wdf = wdft(ok)
          if nwdf lt narray then $
            wdf = [wdf, wdf(nwdf-1) + indgen(narray-nwdf)+1]
        endelse
        if wdf(narray-1) gt maxwdf then set_maxwdfarray, wdf(narray-1)
        for i = 0, narray-1 do $ 
          put_wdfarray, bx(0:(((np(i)+1)>3)), i), wdf(i), $
          file = fname, xlabel = 'Time (s)', ylabel = 'Volts', clabel = name(i)
      endif
      data = temporary(bx(0:maxnp-1, *))
      return
;
noskipdata:
      print, 'Unable to read Dataset Labels.'
      return
      end
