; ------------------------------------------------------------------------ 
; $Id: reens.pro,v 1.6 2009/02/18 17:54:12 lpmix Exp $
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
 Pro reens, fname, WDF, nwdf = nwdf, ARRAY = array, joinseg = joinseg, $
           helpme = helpme
;
;+
;    Read an EnSight XY Plot Data Format
;
;    Format:
;          ReEnS [, FNAME] [, WDF] [, NWDF = nwdf] [, ARRAY = array]
;                [, /HELPME]
;    where 
;      FNAME   - File name
;                If fname is a number, undefined, or omitted, 
;                  Dialog_Pickfile will be used to pick the file.
;                FNAME is returned with the name of the file opened
;      WDF     - First WDF array or an array of WDF arrays
;                WDF may be either a single value or an array of values
;                If additional wdf arrays are needed, sequential values 
;                will be used.  If the maximum number of WDF arrays is
;                reached, the maximum will be incremented to a multiple 
;                of 100.
;                If WDF is missing or invalid, WDF = 1
;      NWDF    - Number of WDF arrays which are read
;      JOINSEG - Join segments into one WDF array.
;                If JOINSEG lt 0, sort the data
;                Default:  If JOINSEG is undefined, then JOINSEG = 1
;      SORT    - Sort abscissa values if JOINSEG is set.
;                Same as JOINSIG = -1 (i.e., JOINSIG is less than zero)
;      ARRAY   - Actual WDF arrays saved.
;      HELPME  - If set, a one line help line will be printed.
;          % Format: REENS [, FNAME] [, WDF] [, NWDF = nwdf] [, ARRAYS = arrays]
;
;    Procedure:
;      Data is read and stored as XY data in WDF arrays.
;      Data must be in the EnSight XY Plot Data Format
;      Read is terminated on an error.
;      If an array has one or two values, additional data points
;        will be added to bring the total to 3 points.
;      If a curve has multiple segments, they will be put into
;        sequential waveforms with segment numbers appended.
;        " - Seg #"
;    Example:
;
;    Read waveform arrays from file voltage.dat and store the data
;       beginning in wdf array 21.
;       REENS, 'voltage.dat', 21 
;
;    Same as the first example, but plot the data 6 to a screen.
;       REENS, 'voltage.dat', 21, array = arr
;       PLO, arr, grid = 6
;    Modification History:
;       Original version, L. P. Mix, December 6, 2002
;
;-
;
     on_error, 1
@com.dat
     if n_elements(initwdf) eq 0 then wdfinit
;    check format
      nparams = n_params()
; ******************************************************************************
      if keyword_set (helpme) then begin
        print,  $
         '% Format: REENS [, FNAME] [, WDF] [, NWDF = nwdf] [, ARRAY = array]'
        return
      endif
      if n_elements(fname) eq 0 then fname =0 else fname = fname(0)
      b = size(fname)
      if b(1) eq 7 then begin
        resolve_it, fname
        info = file_info(fname)
        if (info.exists + info.read) ne 2b then begin
           print, '% REENS: File does not exist with read permission'
           print, '         File Name = ', fname
           print, '         File Exists =', info.exists
           print, '         Read Permission =', info.read
           return
        endif
      endif else begin
        fn = dialog_pickfile(/must_exist, /read)
        if strlen(fn) lt 1 then begin
          print, ' No File was Selected - RETURNING'
          return
        endif
        fname = temporary(fn)
        resolve_it, fname
      endelse
      openr,unit,fname, /get_lun
      filename = fname
      if n_elements(wdf) eq 0 then wdf = 1L
      array = wdf
      ok = where (array gt 0 and array lt maxwdfarray, nok)
      if nok gt 0 then array = array[ok] else array = 1L
      nwdfar = nok > 1L
      nwdf = 0L
      if n_elements(joinseg) eq 0 then joinseg = 1
      if keyword_set(joinseg) && (joinseg lt 0) then sort = 1
      
      ncurve = 0L
      nc = ncurve
on_ioerror, badnum
      clab = ''
      xlab = ''
      ylab = ''
      nsegment =0L
      ns = nsegment
      npoint = 0L
      stat = 'NCURVE'
      readf, unit, ncurve

;
;     Loop over curves
;
      for nc = 1, ncurve do begin
;       reading labels
        stat = 'Labels'
        readf, unit, clab
        readf, unit, xlab
        readf, unit, ylab
;       reading number of segments
        stat = 'Segments'
        readf, unit, nsegment
        for ns = 1, nsegment do begin
          stat = 'Points'
          readf, unit, npoint
          stat = 'Data'
          data = make_array(2, npoint)
          readf, unit, data
          if (keyword_set(joinseg)) and (nsegment gt 1) then begin 
            if ns eq 1 then begin
              xx = reform(data[0, *])
              yy = reform(data[1, *])
            endif else begin
              xx = [xx, reform(data[0, *])]
              yy = [yy, reform(data[1, *])]
            endelse
            if  ns eq nsegment then begin
              if keyword_set (sort)  then begin
                a = sort (xx)
                xx = xx[a]
                yy = yy[a]
              endif
              data = [xx, yy]
              npoint = n_elements(xx)
            endif
          endif   
          if not keyword_set(joinseg) or (ns eq nsegment) then begin
            wd = array[nwdf]
            if wd ge maxwdfarray then $
              set_maxwdfarray, (maxwdfarray/100L+1)*100L
            if ns eq 1 then clabel = clab else begin
              if keyword_set(joinseg) then $
                clabel=clab+' - Seg 1 to '+strtrim(ns,2) else $
                clabel=clab+' - Seg '+strtrim(ns,2) 
            endelse
            if n_elements(data) lt 6 then begin 
              print, '% REENS: Segment data less than 3 points'
              print, data
            endif else begin
              data = reform(transpose(data), 2*npoint)
              put_wdfarray, data, wd, /xydata, /no_copy, clabel=clabel, $
                 xlabel = xlab, ylabel = ylab, file = filename
              nwdf = nwdf+1
            endelse
            if nwdf eq nwdfar then begin
              nwdfar = nwdfar +64
              array = [array , array[nwdf-1]+1+indgen(64)]
            endif
          endif
        endfor ; end of segment loop
      endfor ; end of curve loop
      if nwdf gt 0 then array= array[0:nwdf-1] else array = -1
      free_lun, unit
      return
badnum:
      free_lun, unit
      print, '% REENS: Error reading ', stat
      print, '  Current curve: ', strtrim(nc, 2),' of ',strtrim(ncurve,2)
      print, '  Current comment: ', clab
      print, '  Current segment: ', strtrim(ns, 2),' of ',strtrim(nsegment,2)
      if nwdf gt 0 then array= array[0:nwdf-1] else array = -1
      end
