; ------------------------------------------------------------------------ 
; $Id: riset.pro,v 1.8 2009/09/24 20:04:25 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
;+
; Calculate the rise time of a pulse.
; XY data is converted to a uniform time base
; Maximum deviation from zero is used as the pulse height by default.
; (See FALLT for the fall time.)
; Procedure uses linear interpolation to find the last time that the waveform 
;   crosses start value (typically 10% of the maximum) and the first time that 
;   the waveform crosses the end value (typically 90% of the maximum).
;   This procedure produces the minimum rise time and may underestimate the rise
;   time if the signal is very noisy.  
;
; Format: RISE = RISET( WDF [, STARTVALUE = startvalue] [, ENDVALUE = endvalue]
;                    [, MAXVALUE = maxvalue] [, RIGHT = right]
;                    [, SMOO = smoo] [, VALUE = VALUE] [, MAXIMUM= maxamp ]
;                    [, MAXTIME= maxtime ] [, /POSITIVE] [, /NEGATIVE] [, /PLOTIT )
;
;     where:
;        RISE      - risetime of the pulse between startvalue and endvalue
;                    Rise time is defined as the time from the base line to 
;                    maximum deflection.  The maximum deflection can
;                    be positive or negative unless POSITIVE or NEGATIVE is
;                    set.
;                    0 is returned if an error occurs.
;        WDF       - WDF array for the calculation
;        MAXVALUE  - If this is set, the maximum deflection will be determined by this 
;                    value and the times when (Startvalue*Maxvalue) and (EndValue*MaxValue)
;                    are returned.  As noted above, the minimum rise time is returned.
;                    POSITIVE and NEGATIVE are ignored.
;        LEFT      - If set, only data after LEFT are used. (See NOTE after RIGHT.)
;        RIGHT     - If set, only data before RIGHT are used.
;           NOTE:    If LEFT has two or more elements and RIGHT is missing or if RIGHT has two or
;                    more elements and LEFT is missing, VALUES will be taken as the range; ie
;                        LEFT = MIN(VALUES)  &  RIGHT = MAX(VALUES)
;                    where VALUES is the specified vector.
;        STARTVALUE- Beginning value as a fraction of the peak for rise time.
;                    Default = 0.10
;        ENDVALUE  - End Value as a fraction of the peak for the rise time.
;                    Default = 0.9
;        SMOO      - Width for smoothing of the data
;                    If data is X-Y, it is converted to a uniform time base before filtering.
;                    Smoothed data is stored in get_maxwdf()
;                    SMOO = SMOO > 3
;                    If SMOO le 0, then no filtering.
;                    Default: If SMOO undefined, then smoo = 5
;        VALUE     - Vector with actual times for start and end values
;        MAXIMUM   - Maximum value used to determine start and end value
;        MAXTIME   - Time at maximum value
;        POSITIVE  - If set, only positive deflections are considered.
;        NEGATIVE  - If set, only negative deflections are considered.
;                    If both positive and negative are set, then the
;                      maximum deflection is used. (same as if neither are set)
;        PLOTIT    - If set, plot the original data, the smoothed data and the start 
;                    and end values.
;
; Example:
;     Print, the 10% to 90% rise time of wdfarray 5 and plot the data.
;         PRINT, riset(5, /plot)
;
;     Calculate the 10% to 90% rise time of waveform 4
;     Since the curve has multiple peaks, use ZOOMW to pick the peak of interest.
;     Store the result in RT1
;       ZOOMW, 4, XCUR = XC ; pick the two points to define the rise time of interest
;       RT1 = RISET(4, R= XC) ; or RT1 = RISET(4, L= XC)
;
;        September 17, 2009    Added RIGHT and LEFT keywords
;
;-
;
 function riset, wdf, startvalue = startvalue, endvalue = endvalue, $
          smoo = smoo, value = value, maximum = maxv, maxtime = maxtime, $
          positive = positive, negative = negative, plotit = plotit, $
          maxvalue = maxvalue, right = rightin, left = leftin
@com.dat
on_error, 1
;
      if n_elements(initwdf) eq 0 then wdfinit
      nparams = n_params()
      if nparams lt 1 then begin
        print,'% Format: RISE = RISET(WDF1 [, SMOO = smoo] '
        print,'          [, STARTVALUE = startvalue] [, ENDVALUE = endvalue]' 
        print,'          [, VALUE = value] [, /PLOTIT]'
        print,'          [, MAXIMUM= maxamp ] [, MAXTIME= maxtime])'
        return, 0
      endif
;
      if n_elements(wdf) eq 0 then begin
        print, '%RISET: Parameters to RISET must be defined'
        return, 0
      endif
      if wdf lt 1 or wdf gt maxwdfarray then begin
        print, '%RISET: WDF array must be greater than 0 and </=', maxwdfarray
        return, 0
      endif

     if n_elements(startvalue) eq 0 then startvalue = 0.1 else $
         startvalue = abs(startvalue) < 1.0
     if n_elements(endvalue) eq 0 then endvalue = 0.9 else $
         endvalue = abs(endvalue) < 1.0

     if (startvalue ge endvalue) then begin
        print, '%RISET: Startvalue must be LESS THAN Endvalue'
        return, 0
      endif

     if n_elements(smoo) eq 0 then smoo = 5

     get_wdfarray, array, wdf
     if smoo gt 0 then arr=smooth(array[2:*], (smoo > 3)) else arr=array[2:*]
     i2w, array[0:1], arr, maxwdfarray, clab = 'RISET array - '+wdcomment(wdf)
     lcpos = keyword_set(positive) ?  1 : 0
     lcneg = keyword_set(negative) ? -1 : 0

     start = wdfminx(wdf-1)
     stopp = wdfmaxx(wdf-1)
     nl = n_elements(leftin)
     nr = n_elements(rightin)
     if max([nl, nr], min = m) ge 2 and m eq 0 then begin
       if nl ge 2 then right = max(leftin, min = left) else $
           right = max(rightin, min = left)
     endif else begin
       if nl gt 0 then left = leftin[0] else left = start
       if nr gt 0 then right = rightin[0] else right = stopp
     endelse
     if (left lt start) or (left ge stopp) then left =start
     if (right le left) or (right gt stopp) then right = stopp

     if left gt start then begin
       i = floor ((left -start)/array[1])
       if i gt 0 then begin
         arr = arr[i:*]
         array[0] = array[0]+i*array[1]
       endif
     endif
     narrm =  n_elements(arr)-1
     if  right lt stopp then begin
       narrm = ceil((right - array[0])/array[1]) < narrm
       arr= arr[0:narrm]
     endif
;    Get the maximum and minimums
   if n_elements(Maxvalue) gt 0 then begin
     maxv = maxvalue[0]
     if (maxv lt 0) then begin
       maxv = -maxv
       arr = -temporary(arr)
     endif
     maxtime= array(0)+narrm*array[1]
   endif else begin
     maxv = max(arr, i, min = minv, subscript_min = inv)
     if lcpos + lcneg eq 0 then begin
       if (-minv) gt maxv then begin
         maxv = -minv
         i = inv
         arr = -temporary(arr)
       endif
     endif else if keyword_set (lcneg) then begin
       maxv = -minv
       i = inv
       arr = -temporary(arr)
     endif
     maxtime= array(0)+i*array(1)
   endelse
     en = 1
     if endvalue lt 1 then begin
       en = endvalue*maxv
       ok = where(arr[0:i] ge en, nok)
       if nok eq 0 then begin
         print, '% RISET: No Values above ', strtrim (en, 2),' on the rise(2)'
         return, 0
       endif
       imax = ok[0]
       if imax eq 0 then begin
         print, '%RISET: End Value may not be the first point.'
         return, 0
       endif
       if arr[imax] gt en then $
         imax = imax+(en- arr[imax])/(arr[imax]-arr[imax-1])
     endif else imax = i
     stv = startvalue*maxv

     ok = where(arr[0:i] le stv, nok)
     if nok eq 0 then begin
        print, '% RISET: No Values below ', strtrim (stv, 2),' on the rise(1)'
        return, 0
     endif
     imin = ok[nok-1]
     if imin gt imax then begin
        print, '% RISET: Start value is greater than end value'
        return, 0
     endif

     if (arr[imin] lt stv)   then $
         imin = imin+(stv - arr[imin])/(arr[imin+1]-arr[imin])
     value = array(0)+ [imin, imax]*array(1)

     if keyword_set(plotit) then begin
       xr= [value[0], maxtime]
       xr[0] = xr[0]*0.95 > wdfminx[wdf-1]
       xr[1] = xr[1]*1.055 < wdfmaxx[wdf-1]
       plo, [wdf, maxwdfarray], /over, /color, /ns, /nl, xrange= xr
       del = 0.03*(xr[1]-xr[0])
       dy = 0.03*(!y.crange[1]-!y.crange[0])
       oplot, value[0]+[-del, del], stv*[1,1]
       oplot, value[1]+[-del, del], en*[1,1]
       oplot, value[0] *[1,1], stv+[-dy, dy]
       oplot, value[1] *[1,1], en+[-dy, dy]
     endif
     return, value(1)-value(0)
     end
