; ------------------------------------------------------------------------ 
; $Id: rtp.pro,v 1.3 2007/06/06 20:03:53 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
  Pro  rtp, wdf1, wdf2, wdf3, exact = exact
;
;+
; NAME:
;       RTP
;
; PURPOSE:
;       Raise a WDF array to a power
;       Store the result in the initial WDF array or another WDF array
;         If power is not an integer, then negative values will be set to zero.
; CATEGORY:
;       Math 
; CALLING SEQUENCE:
;       RTP, WD1, VALUE [, WD2]  [, /EXACT]
; INPUTS:
;       WD1:      WDF array number of the first array
;     VALUE:      Power to which the first array is to be raised
; OPTIONAL INPUT:
;       WD2:      WDF array number for the result; if not specified or invalid 
;                 then the sum is stored in WD1 (the first array)
;     EXACT:      If present and not zero, aborts the calculation if WD1 < 0 
;                 and value is a real.  This is ignored if VALUE is an integer.
;                 
; OUTPUTS:
;       NONE    The sum is stored in a WDF array in the common block
; COMMON BLOCKS:
;       COM.DAT  WDF common block 
; RESTRICTIONS:
;       WD1 array must be defined and have valid data a power must be defined
; PROCEDURE:
;       If VALUE is an integer then the result is calculated
;           Result is WD1^VALUE
;       If VALUE is not an integer then the result is calculated
;           Result is exp(VALUE* alog(WD1)) if WD1 > 0
;        or Result is 0.0 if WD1 < 0.0
;
;       In either case Result is stored in WD2, if specified, or WD1.
; EXAMPLES:
;       Raise array 2 to the 1.5 power and store the result in array 3
;       Set any negative values in array 2 to zero
;               RTP, 2, 1.5, 3
;       Raise array 2 to the 0.5 power and store the result in array 3
;       Do not do the calculation if any values are less than zero
;               RTP, 2, 0.5, 3, /exact
;       Raise array 2 to the 3 power and store the result in array 4
;               RTP, 2, 3, 4
; MODIFICATION HISTORY:
;      Original version, L. P. Mix, May 15, 1992
;-
;
@com.dat
;
      if n_elements(initwdf) eq 0 then wdfinit
      nparams = n_params()
      if nparams lt 2 then begin
        print,' RTP must have at least 2 parameters'
        return
      endif
;
      if n_elements(wdf1) eq 0 or  n_elements(wdf2) eq 0 then begin
        print, 'Parameters to RTP must be defined'
        return
      endif
      if wdf1 lt 1 or wdf1 gt maxwdfarray then begin
        print, 'wdf array must be greater than 0 and </=', maxwdfarray
        return
      endif
;
      nel = wdfnp(wdf1-1)
      if nel lt 2 then begin
        print, 'No data in the first array'
        return
      endif
;
      if n_elements(wdf2) eq 0 then begin
        print, ' Value must be defined in RTP'
        return
      endif else value = wdf2(0)
;
      if nparams gt 2 then save = get_save_array(wdf3, wdf1) else save = wdf1
;
      get_wdfarray, array1, wdf1, /xy
;
      if nel+nel eq n_elements(array1) then begin
        xydata = 1
        offset = nel
      endif else begin
        xydata = 0
        offset = 2
      endelse
      val = long(value)
      if value eq val then begin
        array1(offset:*)= array1(offset:*) ^ val
      endif else begin 
        p = where(array1(offset:*) lt 0.0, nok) 
        if nok gt  0 then begin
          if keyword_set (exact) then begin
            print, ' Cannot raise a negative value to a real power'
            print, '  No modification to array', wdf1
            return
          endif else array1(p+offset)= 1.0 
        endif
        array1(offset:*) = array1(offset:*) ^ value
        if nok gt 0 then array1(p+offset) = 0.0
      endelse
      if n_elements (save) eq 0 then save = wdf1
      if save lt 1 or save gt maxwdfarray then save = wdf1
      put_wdfarray, array1, save, xydata = xydata, /no_copy
      wdf3 = save
      wdffile(save-1) = wdffile(wdf1-1)
      wdfcomment(save-1) = wdfcomment(wdf1-1)
      wdfxlabel(save-1)= wdfxlabel(wdf1-1)
      wdfylabel(save-1)= wdfylabel(wdf1-1)
      array2=0
      sp=0
      im=0
      return
      end
