; ------------------------------------------------------------------------
; $Id: sample.pro,v 1.2 2010/08/17 15:44:51 lpmix Exp $
; ------------------------------------------------------------------------
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
;+
; NAME:
;       SAMPLE
;
; PURPOSE:
;       Sample a waveform to extract an approximate number of points or 
;       with a certain spacing.
;
; CATEGORY:
;       Analysis
;
; CALLING SEQUENCE:
;       SAMPLE, WDF1 [, WDF2]
;
;
; INPUTS:
;       WDF1:   WDF array number or name that is to be down sampled.
;
; OUTPUTS:
;       WDF2:   WDF array number for the output
;               If WDF2 is missing, then WDF2 = GET_MAXWDF()
;
; KEYWORD PARAMETERS:
;       NPOINT:	Number of points for WDF2  
;               REDUCE is calculated by N_ELEMENTS(WDF1)/ (NPOINT - 1)              
;               Actual value of the keyword, REDUCE, is ignored.
;       REDUCE:	Skip factor for the sample
;               If NPOINT is present, REDUCE is ignored
;       FILTER: Filter for the data. 
;               For uniform data this will be a simple boxcar filter.  (SMOOTH (Y, F))
;               For nonuniform data, this will be a sum over all points nearest
;                 a point divided by the number of points.
;                DEFAULT:  If FILTER is undefined, no filtering of the data
;                 UNIFORM DATA:  If FILTER LE 2 then F = (((REDUCE-1)/2)*2)+1 
;                                If FILTER GE 3 then F = FILTER
;                                Y_VALUE = SMOOTH(GETYF(WDF1, F))
;                 NON-UNIFORM DATA: If FILTER NE 0 average half of the points between
;                                   successive values. 
;                                   For example assume X = indgen(10) with values at [0, 5, 9]
;                                   If filter is defined and not zero, the value at 5 would be 
;                                   the sum of the values at Y(5) = total(Y[3, 4, 5, 6])/4
;                                   Similarly for the same example but samples at [0, 4, 9] then 
;                                     Y(4)= total(Y[3, 4, 5, 6])/4 (same Y value)
;       DELTAT: This is the maximum allowable time spacing.  This is ignored for uniform data.
;               DEFAULT:  For non-uniform data DELTAT = XRANGE/NPOINT * 4
;       PLOTIT: Plot the two curves, WDF1 and WDF2
;
; RESTRICTIONS
;       REDUCE and NPOINT must be smaller than twice the number of points in WDF1.
;       
; PROCEDURE:
;       
;	You might not need this section for your routine.
;
; EXAMPLE:
;       Sample the data in WDF array 2 to 600 points with a minimum spacing of 1 ns.
;         Store the sampled data in WDF array 12
;         SAMPLE, 2, 12, NPoint = 600, DeltaT = 1e-9
;
; MODIFICATION HISTORY:
; 	Written by:	L. P. Mix, Jr., August 13, 2010
;
;-
 pro sample, wdf1, wdf2, npoint = npoint, reduce = reduce, filter = filter, $
        deltat = deltat, plotit = plotit
;  
@com.dat
 on_error, 1

   if n_elements(wdf1) eq 0 then begin
     print, '% SAMPLE:  WDF array must be defined'
     return
   endif
   wdf = wdvalid(wdf1, /valid, np = np)
   if np eq 0 then begin
     print, '% SAMPLE:  WDF must be a valid array or search string'
     return
   endif
;  This version allows only np =1
   if np gt 1 then begin
     print, '% SAMPLE:  WDF must be a single array number or a unique search string'
     help, wdf
     print, wdf
     return
   endif

   if n_elements(wdf2) eq 0 then wdf2 = get_maxwdf()
   lcnp = (n_elements(npoint) eq 0) ? 0 : 1
   lcreduce = (n_elements(reduce) eq 0) ? 0 : 1
   if lcnp then if npoint le 1 then lcnp = 0
   if lcreduce then if reduce le 2 then lcreduce = 0
   if lcnp+lcreduce eq 0 then begin
     print, '% SAMPLE:  Number of points or skip factor must be provided'
     return
   endif
   if n_elements (deltat) gt 0 then deltat = abs(deltat)
   if n_elements(filter) eq 0 then filter = 1
;   This version is for a single wdf array
;   for i = 0L, np-1 do begin
     i =0
     wd = wdf[i]
     nel = wdfnp[wd-1]
     get_wdfarray, array, wd, /xydata
     xfr, wd, wdf2
     xydata = (n_elements(array) eq (nel+2)) ? 0: 1
     if xydata then begin
       x = array[0:nel-1]
       y = array[nel: *]
       dx = x[1:nel-1] - x[0:nel-2]
       if min(dx) lt 0 then begin
         ss = sort(x)
         x = x[ss]
         y = y[ss]
       endif
       xrange = x[nel-1]-x[0]
       if lcnp then red = nel/(npoint-1) else red = reduce
       newn = round(nel/red+1)
       del = (n_elements(deltat) eq 0) ? xrange*4.0/newn : deltat
       ii = round(lindgen(newn)*red) < (nel-1)
       xt = x[ii]
       dx = xt[1: newn -1]-xt[0: newn-2]
       mindx = min(dx)*0.9
       while (max(dx) gt del) do begin
         add = -1L
         ok = where (dx gt del, nok)
         for j = 0, nok-1 do begin
           ndel = round(dx[ok[j]] /del)
;          idel is the number of x values spaces
           idel = (ii[ok[j]+1] - ii[ok[j]])
;          print, x[ii[ok[j]]], x[ii[ok[j]+1]], newn, idel , ndel
;          if idel is one then we are at the end
           if idel gt 1 and ndel gt 1 then begin
             if idel le 1.5*ndel then aa =ii[ok[j]]+lindgen(idel+1) else $
               aa = ii[ok[j]]+(round(idel*findgen(ndel+1)/ndel))
             ndel = n_elements(aa)
             dxx = x[aa[1:ndel-1]] -x[aa[0:ndel-2]]
             okk = where (dxx gt mindx, nokk)
             if nokk gt 0 then begin
               if nokk eq 1 then add =[add, aa[okk[0]+[0, 1]]] else $
                 if nokk lt (ndel-1) then add = [add, aa[okk], aa[okk+1]] else $
                   add = [add, aa]
             endif
           endif
         endfor
         if n_elements(add) gt 1 then begin
           ii = [ii, add[1:*]]
           ii = ii[uniq(ii, sort(ii))]
           xt = x[ii]
           if n_elements(xt) eq newn then dx =0 else begin
             newn = n_elements(xt)
             dx = xt[1: newn -1]-xt[0: newn-2]
           endelse
         endif else dx = 0
       endwhile
;    put filter stuff here
       if keyword_set(filter) then begin
         ni = n_elements(ii)
         mid = (ii[1:ni-1]+ii[0:ni-2])*0.5
         lmid = floor(mid)
         umid = ceil(mid)
         first = [0, umid]
         second = [lmid, nel-1]
         yy = make_array(ni)
         for i = 0, ni-1 do yy[i] = total(y[first[i]:second[i]])
         yy = yy/(second-first+1)
       endif else yy = y[ii]
       lab, /ap, wdf2, ' - Sampled,SKIP='+strtrim(red, 2)
       put_wdfarray, [xt, yy], wdf2, /xydata, /no_copy
       x= 0 
       y=0
     endif else begin
       x = array[0:1]
       y = array[2:*]
       if lcnp then red = nel/(npoint-1) else red = reduce
       if filter gt 0 then begin
         if filter lt 3 then ff = red else ff = filter
         if ff mod 2 eq 0 then ff +=1
         y = smooth(temporary(y), ff)
       endif
       newn = nel/red + 1
       x[1] = x[1]*red
       y = y[lindgen(newn)*red]
       put_wdfarray, [x, y], wdf2, /no_copy
       lab, /ap, wdf2, ' - Sampled,SKIP='+strtrim(red, 2)
     endelse
     if keyword_set(plotit) then begin
       plo, [ wd,wdf2],  /nl, psym =[0, -8], color =[1, 2], /over
     endif
;   endfor
   
end
