; ------------------------------------------------------------------------
; $Id: sgolay.pro,v 1.2 2010/09/17 21:23:38 lpmix Exp $
; ------------------------------------------------------------------------
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
;+
; NAME:
;       SGOLAY
;
; PURPOSE:
;       This routine utilizes the IDL function SAVGOL to obtain the coefficients
;	of a Savitzky-Golay smoothing filter.  This filter is then applied to a 
;       WDF array for a weighted average smoothing.  This filter can also be used
;       calculate derivatives of the input waveform.
;
;       This function is not optimal for smoothing a waveform, but is ideal for
;       optimally calculating numeric derivatives.
;
;       This filter works especially well when the typical peaks of the signal 
;       are narrow. The heights and widths of the curves are generally preserved.
;
; CATEGORY:
;       Mathematics
;
; CALLING SEQUENCE:
;
;       SGOLAY, WD1, WD2
;
; INPUTS:
;       WD1:    WDF array number for the waveform to be smoothed or differentiated.
;               NOTE:  If WD1 is X-Y data, it will be changed to uniform data.
; OPTIONAL INPUTS:
;       WD2:    WDF array number for the smoothed data or for the derivative.
;	
; KEYWORD PARAMETERS:
;       CLABEL: Dataset comment for the saved calculated waveform.
;               If CLABEL is not specified, then SAVGOL(Left, Right, Order, Degree) 
;                 will be added to the old comment separated by ' - ' or ';;'.
;       XLABEL: X-AXIS label for the calculated waveform
;               DEFAULT: XLABEL for WD1
;       YLABEL:	Y-AXIS label for the calculated waveform
;               DEFAULT: YLABEL for WD1
;
; NOTE:  The following refer to the coefficients for the filter.
;        The descriptions are from the SAVGOL documentation. (? SAVGOL)
;
; NOTE: If either left or right is set and the other is not, then they will be set equal. 
;       LEFT:   An integer specifying the number of data points to the left of
;               each point to include in the filter.
;               DEFAULT:  LEFT = (n_elements(DEGREE) eq 0) ? 5 : DEGREE
;                         LEFT = LEFT > 1
;       RIGHT:  An integer specifying the number of data points to the right of
;               each point to include in the filter. 
;               DEFAULT:  RIGHT = (n_elements(DEGREE) eq 0) ? 5 : DEGREE
;                         RIGHT = RIGHT > 1
;       Note:   Larger values of Nleft and Nright produce a smoother result at
;               the expense of flattening sharp peaks.
;       ORDER:  An integer specifying the order of the derivative desired.
;               For smoothing, use order 0. To find the smoothed first derivative
;               of the signal, use order 1, for the second derivative, use order 2, etc.
;               DEFAULT: Order =0
;       Note:   Order must be less than or equal to the value specified for DEGREE.
;       DEGREE: An integer specifying the degree of smoothing polynomial. Typical values
;               are 2 to 4. Lower values for DEGREE will produce smoother results but may
;               introduce bias, higher values for DEGREE will reduce the filter bias, but
;               may "over fit" the data and give a noisier result.
;               DEFAULT:  DEGREE = 3 < (NLEFT+NRIGHT+1) 
;       Note:   DEGREE must be less than the filter width (NLEFT + NRIGHT + 1).
;       DOUBLE: Set this keyword to force the computation to be done in double-precision
;               arithmetic.
;               DEFAULT: DOUBLE = (DEGREE gt 4) ? 1 : 0
; OUTPUTS:
;       A smoothed waveform or a derivative of the input waveform
;
; PROCEDURE:
;       The IDL procedure SAVGOL is used to calculate the coefficients of the Savitzky-Golay 
;       filter.  If necessary the coefficients are normalized.  CONVOL is then used to perform 
;       the filtering with keyword set: EDGE_TRUNCATE.
;
; EXAMPLE:
;       Calculate the first derivative of waveform 1 using a total of 11 points with a 
;       Savitzky-Golay filter degree of 5, and store the result in 2
;         SGOLAY, 1, 2, O = 1 ;;  ORDER = 1 is the first derivative
;      or
;         SGOLAY, 1, 2, ORDER = 1, LEFT = 5, RIGHT = 5, DEGREE = 5
;
;       Calculate the second derivative of waveform 7, with 10 points before and after each
;       point with a filter degree of 6.  Save the derivative in 57.
;         SGOLAY, 7, 57, RIGHT = 10, ORDER =2, DEGREE =6  ; Note since RIGHT=10 and LEFT is BLANK
;                                                         ; DEFAULT is LEFT = RIGHT
;
; MODIFICATION HISTORY:
; 	Written by:	L. P. Mix, January 26, 2009
;
;-

PRO SGOLAY, WD1, WD2, CLABEL = clab, XLABEL = xlab, YLABEL = ylab, DOUBLE = dpin, $
    LEFT = leftin, RIGHT = rightin,  ORDER = orderin, DEGREE = degreein

on_error, 1
@com.dat

nparams = n_params()
if nparams eq 0 then begin
  print, '% Format: SGOLAY, WD1, WD2, CLABEL = clab, XLABEL = xlab, '
  print, '          YLABEL = ylab, LEFT = left, RIGHT = right,  ORDER = order, '
  print, '          DEGREE = degree, DOUBLE = double_precision'
  return
endif

if n_elements(wd1) eq 0 then begin
  print, '% SGOLAY: Input waveform must be defined'
  return
endif
 
wdf = (wdvalid(wd1[0], /valid, /quiet))[0]
if wdf lt 1 then begin
  print, '% SGOLAY: Valid WDF array must be present'
  print, '  Unable to fine WDF = ' +strtrim (wd1[0], 2)
  return
endif

; Check parameters
nleft = n_elements(leftin) < 1
nright = n_elements (rightin) < 1
if nleft +nright eq 0 then begin
  left = 5
  right = 5
endif else if nleft +nright eq 1 then begin
  left = (nleft eq 1) ? leftin : rightin
  right = left
endif else begin
  left= leftin[0] > 1
  right= rightin[0] > 1
endelse
order = (n_elements(orderin) eq 0) ? 0: (orderin > 0)
if (n_elements(degreein) eq 0) then begin
  degree = 3 < (left+right +1)
endif else degree = degreein[0]
if (order gt degree) or (degree gt (right+left+1)) then begin
    print, '% SGOLAY:  Value of parameters are not consistent: '
    print, ' Left, Right, Order, Degree: '
    print, left, right, order, degree
    return
endif

;
;  Get_ARRAY returns a uniform wdf array.
;
get_wdfarray, array, wdf
y = array[2:*]
dt = array[1]
t0 = array[0]
dp = (n_elements(dpin) eq 0) ? (degree gt 4) : dpin[0]

savgolfilter = SAVGOL(left, right, order, degree, doubl = dp)

if order gt 0 then savgolfilter = savgolfilter * (FACTORIAL(order)/(dt^order))

y = convol(y, savgolfilter, /edge_truncate)

wdinfo, wdf, comment = clabo, xlab = xlabo, ylab= ylabo, file = file
if n_elements (clab) eq 0 then begin
  if strpos(clabo, ' - ') gt 0 then $
    clab = clabo+';;SAVGOL('+strtrim(left,2)+','+strtrim(right,2)+','+strtrim(order,2)+','+strtrim(degree,2)+')' $
    else $
    clab = clabo+' - SAVGOL('+strtrim(left,2)+','+strtrim(right,2)+','+strtrim(order,2)+','+strtrim(degree,2)+')'
endif

if n_elements (xlab) eq 0 then xlab = xlabo
if n_elements (ylab) eq 0 then ylab = ylabo
if nparams gt 1 then save = get_save_array(wd2, wd1) else save = wd1
i2w, [t0, dt], y, save, clab = clab, file = file, xlab = xlab, ylab = ylab

END
