; ------------------------------------------------------------------------
; $Id: sigfit2.pro,v 1.4 2010/09/20 14:16:19 lpmix Exp $
; ------------------------------------------------------------------------
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
;+
; NAME:
;       SIGFIT2
;
; PURPOSE:
;	Fit a waveform array to the sum of two sigmoids of the form:
;  Y = BASE + AMP0/(1 +exp(ALPHA*(X-MIDDLE)))+ AMP1/(1 +exp(BETA*(X-MIDDLE)))
;         where X and Y are the cartesian coordinates of the waveform
;       The sign of alpha will affect the values of BASE and AMP
;
;
; CALLING SEQUENCE:
;       SIGFIT2, WDF [, WDFOUT] [, BASE = base] [, AMPLITUDE = amplitude]
;         [, ALPHA = alpha] [, BETA = beta] [, MIDDLE = middle] [, /PLOTIT]
;         [, TOL = tol] [, ITER = iter] [, ITMAX = itmax]
;         [, CHISQ = chisq] [, INITVAL = initval] [, /VERBOSE]
;         [, WEIGHT = weight] [, SCALE = scale]
;
; INPUTS:
;       WDF:    Waveform array number or comment label for a waveform array
;               that specifies the data for the sigmoid fit.
;       WDFOUT: Waveform array number for the fitted data.
;               Default:  WDFOUT = WDF
;	
; KEYWORD PARAMETERS:
;       BASE:   Fit value for the BASE value of the sigmoid
;               (see  PURPOSE: above)
;       AMPLITUDE: Fit value for the AMPLITUDE value of the sigmoid
;               AMPLITUDE is a two element vector.
;               AMPLITUDE = [AMP0, AMP1]
;               (see  PURPOSE: above)
;       ALPHA:  Fit value for the ALPHA value of the sigmoid
;               (see  PURPOSE: above)
;       BETA:   Fit value for the ALPHA value of the sigmoid
;               (see  PURPOSE: above)
;       MIDDLE:  Fit value for the MIDDLE value of the sigmoid
;               (see  PURPOSE: above)
;       PLOTIT:  If set, the original data and the fit data will be plotted.
;       TOL:     Tolerance input to CURVEFIT
;                Default: TOL = 2e-6
;       ITER:    Number of iterations in CURVEFIT
;       ITMAX:   Maximum number of iterations in CURVEFIT
;                Default: ITMAX = 20
;       CHISQ:   Reduced goodness-of-fit statistic from CURVEFIT
;       INITVAL: Initial values for the CURVEFIT routine
;                This is a vector of 6 values
;                INITVAL =[BASE, AMP0, AMP1, ALPHA, BETA, MIDDLE]
;                Default values calculated:
;                  LIMITS, WDF, XRANGE=xr, YRANGE = yr
;                  BASE = yr[0]
;                  AMPLITUDE = yr[1] - BASE*[1, .2]
;                  ALPHA = SIGN*10/(xr[1]-xr[0])
;                  BETA = 0.2 * ALPHA
;                  MIDDLE = TOTAL(GET_TIME(WDF,BASE+0.5*AMPLITUDE,NP= NP))/NP 
;           Where: SIGN = (DERIVATIVE > 0) ? -1 : +1
;              and DERIVATIVE is the average slope of the WDF array
;       VERBOSE: If set, parameters will be printed to the screen.
;
;  Note:  If WEIGHT is not defined, then SCALE and NORMALIZE_WEIGHT are ignored
;
;       WEIGHT:  If set, this is a valid WDF array number to use as a weight.
;                WEIGHT is an array number or comment label for a WDF array.
;                WEIGHT should have the same number of elements as WDF.  
;                The actual weight is determined by the value of scale.
;           Note: If W are the values of weight for the points in WDF, they may
;                 be stored in a wdf array with the following:
;                   I2W, W, WEIGHT, clabel = 'Calculated Weights'
;       SCALE:   Ignored unless WEIGHT contains a valid WDF array number
;                If WEIGHT is set, SCALE determines how WEIGHT values are used.
;                In the discussion below, assume Y = GETYF(WEIGHT) and 
;                  FIT_WEIGHT is the weight passed to CURVEFIT.
;                IF SCALE is undefined, SCALE = 1
;                  FIT_WEIGHT = Y ^ SCALE
;       NORMALIZE_WEIGHT: If NORMALIZE_WEIGHT is zero then weight is not normalized
;                If NORMALIZE_WEIGHT is missing:
;                   TOTAL(WEIGHT) = NX ; where NX = Number of Points
;                If NORMALIZE_WEIGHT NE 0 then 
;                   TOTAL(WEIGHT) = ABS(NORMALIZE_WEIGHT)
;                 
; OUTPUTS:
;       A waveform array with the sigmoid fit to the data and the
;       parameters for the waveform array.
;
; PROCEDURE:
;       The IDL procedure, CURVEFIT, is used to fit the data with a uniform 
;       weight of 1.0 unless WEIGHT is specified.
;
; EXAMPLE:
;       Fit waveform array, 8, to a sigmoid and store the fitted data in
;         wdf array 9.
;         SIGFIT2, 8, 9
;
; MODIFICATION HISTORY:
;        Written by:    L. Paul Mix, October 12, 2004
;-


pro gsigmoid2, x, a, f, pder
  on_error, 1
; a = [base, amp1, amp2, alpha, beta, mid]
  difer = x-a[5]
  arg1 = a[3]*difer
  arg2 = a[4]*difer
  okpos = where (arg1 gt 80., n80)
  okneg = where (arg1 lt -80., m80)
  if n80 gt 0 then arg1[okpos]=0
  if m80 gt 0 then arg1[okneg]=0
  okpos1 = where (arg2 gt 80., n81)
  okneg1 = where (arg2 lt -80., m81)
  if n81 gt 0 then arg2[okpos1]=0
  if m81 gt 0 then arg2[okneg1]=0
  
  expfac = exp(arg1)
  if n80 gt 0 then expfac[okpos]=exp(80.0)
  if m80 gt 0 then expfac[okneg]=exp(-80.0)

  expfac1 = exp(arg2)
  if n81 gt 0 then expfac[okpos1]=exp(80.0)
  if m81 gt 0 then expfac[okneg1]=exp(-80.0)
  
  denom = 1.0d0+expfac
  denom1 = 1.0d0+expfac1

  f = a[0]+ a[1]/denom + a[2]/denom1
  pder = make_array(n_elements(x), 6, /double)
  pder[*, 0] = 1.0d0
  pder[*, 1] = 1.0/denom
  pder[*, 2] = 1.0/denom1
  temp = a[1]*pder[*, 1] 
  temp1= a[2]*pder[*, 2]
  f = a[0]+ temp + temp1
  temp = temp/denom*expfac
  temp1 = temp1/denom1*expfac1
  pder[*, 3]= -difer*temp
  pder[*, 4]= -difer*temp1
  pder[*, 5]= a[3]*temp + a[4]*temp1
end

pro sigfit2, wdfin, wdfout, base = base, middle = mid, iter = iter, $
    amplitude= amp, alpha= alpha, beta = betaout, tol = tol, plotit = plotit, $
    itmax = itmax, initval = initval, chisq = chisq, verbose=verbose, $
    weight = weight, scale = scale, normalize_weight= norwei

 on_error, 1

npar = n_params()
 if npar eq 0 then begin
   print, '% Format: SIGFIT2, WDFIN, WDFOUT, BASE = base, MIDDLE = mid,'
   print, '          ITER = iter, AMPLITUDE = amp, ALPHA= alpha, BETA =  beta, '
   print, '          PLOTIT = plotit, ITMAX = itmax, INITVAL = initval,'
   print, '          WEIGHT = weight, SCALE = scale, NORMALIZE_WEIGHT = norwei,'
   print, '          TOL = tol, CHISQ = chisq, VERBOSE = verbose'
   return
 endif
 if n_elements(wdfin) eq 0 then begin
   print, '% SIGFIT2:  WDF array must be defined'
   return
 endif
 wdf = wdvalid(wdfin, /valid, /quiet, npoint = npoint)
 if npoint eq 0 then begin
   print, '% SIGFIT2:  WDF must be a valid WDF array'
   return
 endif
 if n_elements(wdfout) eq 0 then wdfout = wdf else $ 
   wdfout = get_save_array(wdfout, wdf)

 w2i, x, y, wdf
 nx = n_elements(x)
 limits, wdf, xrange = xr, yrange= yr
 if n_elements(initval) ge 6 then a = double(initval[0:5]) else begin
   midst = total(yr)/2
   ampst = yr[1]-yr[0]
   basest = yr[0]
   ok = get_time(wdf, midst,np = np)
   if np gt 0 then midst =  total(ok)/np else begin
     print, '% SIGFIT2: Problem with finding the mid point.'
     help, midst, np
     return
   endelse
   alphast = 10.0/(xr[1]-xr[0])
   if y[0] lt y[nx-1] then alphast = -alphast
 
   a = double([basest, ampst, 0.2*ampst, alphast, 0.2*alphast, midst])
; a = [base, amp1, amp2, alpha, beta, mid]
 endelse
 if n_elements(weight) eq 0 then  ww = make_array(nx , value = 1.0) $
 else begin
   wei  = wdvalid(weight, /valid, /quiet, npoint = npw)
   if npw eq 0 then begin
     print, '% SIGFIT2:  WEIGHT must be a valid WDF array'
     help, weight
     return
   endif else if npw gt 1 then begin
     print, '% SIGFIT2:  WEIGHT must specify a unique WDF array'
     help, weight
     return
   endif
   ww = getyf(wei[0])
   if n_elements(scale) eq 0 then scale = 1
   if scale eq 0 then  ww = make_array(nx , value = 1.0) else $
     if scale ne 1 then ww= ww ^ scale 
   if n_elements(norwei) eq 0 then ww = nx/total(ww)*ww else $
     if norwei ne 0 then ww = abs(norwei)/total(ww)*ww
 endelse
; plot, ww & hak, /mes
 if n_elements(tol) eq 0 then tol = 2e-6
 if n_elements (itmax) eq 0 then itmax = 20 else itmax = itmax[0]
 yfit = curvefit(x, y, ww, a, iter = iter, itmax = itmax, tol=tol, $
    /double, function_name = 'gsigmoid2', chisq = chisq, status = status)
 if status ne 0 then begin
   print, '% SIGFIT2: CURVEFIT calculation failed'
   if status eq 1 then print, $
     '% SIGFIT2: Chi-square was increasing without bounds' else print, $
     '% SIGFIT2: Failed to converge in '+strtrim(iter, 2)+' iterations'
   return
 endif
 if keyword_set (plotit) then plo, wdf, /over
 base = a[0]
 amp = [a[1], a[2]]
 alpha = a[3]
 betaout = a[4]
 mid = a[5]
 i2w, x, yfit, wdfout, clab = 'Two-Sigmoid fit to '+wdcomment(wdf), /xydata
 if keyword_set (plotit) then plo, wdfout, /nogrid , /over
 if keyword_set (VERBOSE) then begin
   print, ' Fit converged in ', strtrim(iter, 2), ' iterations'
   print, ' Base: ', strtrim(base, 2)
   print, ' Amplitudes: ', strtrim(amp[0], 2), ', ', strtrim(amp[1], 2)
   print, ' Alpha: ', strtrim(alpha, 2)
   print, ' Beta: ', strtrim(betaout, 2)
   print, ' Middle: ', strtrim(mid, 2)
   print, ' Tol: ', strtrim(tol, 2)
   print, ' CHISQ: ', strtrim(chisq, 2)
 endif
 end
