; ------------------------------------------------------------------------ 
; $Id: smo.pro,v 1.5 2007/06/06 20:03:54 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
 Pro  Smo, wdf1, wdf2, width = widthin, clip = clip , median = median
;
;+
;  Smooth a WDF array using a boxcar or median type filter
;
;  NOTE: Array elements within WIDTH/2 of the ends of the waveform 
;        may not be modified depending on the value of CLIP
;
;  NOTE: Non-Uniform data is filtered without regard to abscissa value.
;
;  Format: SMO, wd1, [, wd2] [, WIDTH = width] [, CLIP = clip] [, /MEDIAN]
;  where:
;     wd1, wd2 - WDF array numbers
;                If wd2 is present, then wd2 = SMO (wd1) 
;                else  wd1 = SMO ( wd1)
;     WIDTH - width of the filter; the default value is 5
;             (width is always odd; if width is even then: width=width+1)
;     CLIP  - Parameter for handling the ends of the array.  If clip =
;          0  Default - average the first and last WIDTH/2 points and 
;             linearly interpolate at the beginning and end of the curve
;         >0  (or /clip) remove the first and last WIDTH/2 points
;         <0  do nothing the the first and last WIDTH/2 points
;     MEDIAN  if set, use a median rather than a boxcar filter
;     
;  Examples:
;  Calculate a box car filter (width = 5) of array 2 and store the result
;  in array 3.
;    smo, 2, 3
;  Calculate a box car filter with width of 11 of array 1 and store the 
;  result in array 5.  Clip the unfiltered points
;    smo, 1, 5, w=11, /c
;-
;
@com.dat
;
      on_error, 1
      if n_elements(initwdf) eq 0 then wdfinit
      nparams = n_params()
      if nparams lt 1 then begin
        print,'% Format: SM0, WDF1 [, WDF2] [, WIDTH = width] [, CLIP = clip]'
        return
      endif
;
      if n_elements(wdf1) eq 0 then begin
        print, '% SMO: Parameters must be defined'
        return
      endif
      if wdf1 lt 1 or wdf1 gt maxwdfarray then begin
        print, '% SMO: WDF array must be greater than 0 and </=', maxwdfarray
        return
      endif
;
      if nparams gt 1 then save = get_save_array(wdf2, wdf1) else save = wdf1
      if save lt 1 or save gt maxwdfarray then save = wdf1
      if wdfnp(wdf1-1) eq 0 then begin
        print, '% SMO: No valid data in the first array'
        return
      endif
      if n_elements(widthin) gt 0 then boxwidth = widthin else boxwidth = 5
      if boxwidth lt 2 then begin
        boxwidth = 5
        print,'% SMO: Width must be greater than 1; set to 5'
      endif
      w2 =  round(boxwidth)/2L
      w2p = w2+1
      boxwidth = w2 + w2 +1L
      get_wdfarray, array1, wdf1, /xydata
      nel = wdfnp(wdf1-1)
      if n_elements(array1) eq nel+2 then begin
        y = array1[2:*]
        array1 = array1[0:1]
        xydata = 0
      endif else begin
        y = array1[nel:*]
        array1 = array1[0:nel-1]
        xydata = 1
      endelse
; filter the y array      
      if keyword_set (median) then y = median(y, boxwidth)  else $
        y = smooth(y, boxwidth)
      if n_elements (clip) eq 0 then clip = 0
; fix the ends if width is greater than 3
      if boxwidth ge 5 && clip ge 0 then begin
        if clip eq 0 then begin
;         interpolate the amplitude to the average of the endpoints
;         average of first part of curve
          ave = total(y[0:w2])/w2p
          y[0:w2-1] = ave + (y[w2]-ave)/w2 * findgen(w2)
;         average of last part of curve
          ave = total(y[(nel-w2p): (nel-1)])/w2p
          y[(nel-w2):(nel-1)] = y[nel-w2p] + (ave-y[nel-w2p])/w2*(findgen(w2)+1)
        endif else begin  ; (clip gt 0)
;         clip the data that was not filtered
          y = y[w2:(nel-w2p)]
          if keyword_set(xydata) then array1 = array1[w2:(nel-w2p)] else $
            array1[0]= array1[0]+w2*array1[1]
        endelse
      endif
      put_wdfarray, [array1, y], save, /no_copy, xydata = xydata
      wdf2 = save
      wdfcomment(save-1)= wdfcomment(wdf1-1)
      wdffile(save-1)= wdffile(wdf1-1)
      wdfxlabel(save-1)= wdfxlabel(wdf1-1)
      wdfylabel(save-1)= wdfylabel(wdf1-1)
      return
      end
