; ------------------------------------------------------------------------ 
; $Id: startvideo.pro,v 1.3 2011/10/04 21:16:31 lpmix Exp $
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
;+
;  Make the object to write a MPEG-4, AVI or other video file.
;  Note: ENDVIDEO must be used to close the file.
;  Note: StartGif may be used to obtain TrueType fonts, wider lines, 
;        and white background.
;
;   NOTE:  I am not a video expert.
;          I have not to done extensive testing of all of the formats and
;          codecs, but if you will tell me which formats and codecs you discover
;          that are better, I will update this procedure.
;
;  Format: STARTVIDEO [, FILE] [, /AVI ] [, /FLASH ] [, /MPEG ] [, /WMP]
;               [, /OVERWRITE] [, FRAMERATE = framerate]
;               [, FORMAT = format] [, CODEC = codec]
;  where:
;    FILE:      Output file name for video output
;               A Unique file will be opened unless OVERWRITE is set.
;               Default is 'idlvideo' plus an extension determined by 
;               the format.  If the file extension is missing, then 
;                 FILE = FILE + "." + strtrim(FORMAT)
;    FRAMERATE: Desired frame rate.
;               Default: FRAMERATE = 2
;    OVERWRITE: If set overwrite an existing file.
;    IERROR:    Returns zero if no problems.
;  NOTE:  The following 4 keywords were selected without extensive
;         testing.  If a better selection is found, please inform
;         Paul Mix, 845-7493.
;    AVI:       If set, FORMAT= 'avi', CODEC = 'mjpeg'
;    FLASH:     If set, FORMAT= 'swf', CODEC = 'flv'
;    MPEG:      If set, FORMAT= 'mp4', CODEC = 'mpeg4'
;    WMP:       If set, FORMAT= 'wmp', CODEC = 'msmpeg4'
;         Note: "avi" is the preferred extension for WMP
;    Note:  The following keywords are ignored if AVI, FLASH, MPEG, or WMP is set . 
;    CODEC:     Desired codec for encoding the movie
;               DEFAULT:  CODEC = 'mpeg4'
;    FORMAT:    Desired format for the file.
;               DEFAULT:  FORMAT = 'mp4'
;   NOTE:  AVI files can be played with QuickTime and xine on Linux.
;          FLASH files can be played in browsers on including Internet Explorer,
;              Mozilla/Firefox, and Google Chrome.
;          MPEG files can be played with xine on Linux.
;          WMP  files can be played with Windows Media Player and xine on Linux.
;          Many other players are available for on the internet.
;
;  Examples:
;  Send the images to a MPEG-4 file called: target.mp4
;; Initialize MPEG-4 Video output
;    startVideo, 'target.mp4', /mpeg, framerate = 2
;; Generate frames
;    for i = 1L, nframe do begin
;      <Make a plot>
;      svideo
;    endfor
;; Close the file
;    endvideo
;;
;  View with xine
;
;  MODIFICATION HISTORY:
;      Original version, L. P. Mix, May 2, 2011
;      June 15, 2011     Added keywords and rewrote after testing options.
;
;-

Pro startvideo,filein, mpeg = mpeg, avi = avi, flash = flash, wmp = wmp, $
           format = formatin, overwrite = overwrite, framerate = framerate,$
           ierror = ierror , codec = codecin
on_error, 1 
compile_opt idl2
@switchvideo.dat
;
;  fileopt = ['', 'mp4', 'avi', 'swf']
   ierror = 1
   if obj_valid(ovid_write) then begin
     print, '% StartVIDEO: Unable to start video file.'
     Print, '         File is already open: ', current_file
     PRINT, '         USE: EndVideo ; to close file '
     return
   endif
   if keyword_set(mpeg) then begin
     format= 'mp4'
     codec = 'mpeg4'
   endif else if keyword_set(avi) then begin
     format= 'avi'
     codec = 'mjpeg'
   endif else if keyword_set(flash) then begin
     format= 'swf'
     codec = 'flv'
   endif else if keyword_set(wmp) then begin
     format= 'wmp'
     codec = 'msmpeg4'
   endif else begin
     format= (n_elements(formatin) gt 0) ? strtrim(formatin[0], 2) : 'mp4'
     codec = (n_elements(codecin) gt 0) ? strtrim(codecin[0], 2) : 'mpeg4'
   endelse
   if n_elements(filein) gt 0 then begin
     file = filein[0]
     barefile = get_bare_filename(file, extension = extension, directory = dir)
     next = strlen(extension)
     if next lt 2  then file = (next eq 1) ? (file + format) : (file + '.' + format)
   endif else file = 'idlvideo.'+ format
   info = file_info(file)
   if n_elements(initial) eq 0 then initial =0
   if info.exists gt 0 and not keyword_set(overwrite) then $
     file = find_newfile(file, /quiet)
   openw, unit, file, /get_lun, error = err
   if err ne 0 then begin
     Print, '% StartVideo: Valid File Name must be provided'
     print, ' Directory must be one with write permission for user'
     return
   endif else begin
     info = file_info(file)
     if info.write eq 0 then begin
       print, '% StartVideo: Unable to write to file: '+file
       return
     endif
   endelse
   free_lun, unit
   ovid_write=IDLffVideoWrite(file, format = ext)
   if obj_valid(ovid_write) then begin
     current_file = file
     init_video = 1
     fr_rate = (n_elements (framerate) eq 0) ? 2 : framerate[0]
     codec_vid = codec
     ierror = 0
   endif  else print, '% StartVIDEO: Unable to create Video Write object'
   end
