; ------------------------------------------------------------------------ 
; $Id: temperature.pro,v 1.3 2007/06/06 20:03:55 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
pro temperature, wdf1, wdf2, minimum = minimum, maximum = maximum, $
      wavelength = wavelength, npoints = np, limits = limits, array = array
; Wavelength is not implemented
;       WaveLength - If set, MINMIMUM and MAXIMUM represent wavelengths in
;                    meters
;                 WaveLength = 1.2398424385d-06/(Energy > 0.025) (meters)
;+
; NAME:
;       TEMPERATURE
;
; PURPOSE:
;       Convert power in Watts/m^2 to an effective temperature.
;       Procedure is iterative with an error limit of 1e-5
;
; CATEGORY:
;       Analysis
;
; CALLING SEQUENCE:
;       TEMPERATURE, WDFin [, WDFout] [, MINIMUM = minimum] 
;           [, MAXIMUM = maximum] [, /LIMITS] 
;           [, /ARRAY] [, NPOINTS = NPOINTS]
;
;      Where:
;       WDFin - Waveform array containing photon power (W/m2)
;               Values must be greater than 1e9.  
;         NOTE: Values less than this are changed to 1e9
;       WDFout- Waveform array equivalent temperature (eV)
;               Default: WDFin
;       MINIMUM - Minimum energy represented by WDFin
;       MAXIMUM - Maximum energy represented by WDFin
;       LIMITS  - Limits on the minimum and maximum energies.
;                 Default: LIMITS = [0.5, 10.0]
;      NOTE:  If minimum is greater than LIMITS[1]*temperature or maximum is 
;             less than LIMITS[0]*temperature, a temperature of zero is returned
;       ARRAY   - If set, then WDFin is an array of one or more elements
;                 The calculated temperatures are stored as an array in WDFout
;       NPOINTS - If specified, the number of points used to calculate the 
;                 spectrum between minimum and maximum.
;                 Default is 1024
; OUTPUTS:
;       WDF array with equivalent temperature
;
; PROCEDURE:
;       An initial temperature is calculated assuming the power is for the 
;       entire spectrum.  This blackbody spectrum is integrated using 
;       1000 points between minimum and maximum.  The integrated power povides
;       an estimate of the observed power to the total blackbody power.
;       A new temperature is calculated and the process is repeated until the 
;       temperature does not change. CALC_TEMP in util is used for this process.
;
; EXAMPLE:
;       WDF array 5 contains total radiated power between 120 eV and 2000 eV.
;       convert it to temperature in array 7.
;         TEMPERATURE, 5, 7, min=120, max = 2000 
;
;
;
; MODIFICATION HISTORY:
;       Written by:     L. P. Mix, 5/18/01
;-

on_error, 1

 nparam = n_params()
 if nparam eq 0 then begin
   print, '% Format: TEMPERATURE, WDFin [, WDFout] [, MINIMUM = minimum]
   print, '                  [, MAXIMUM = maximum] [, NPOINTS = NPOINTS]
   print, '                  [, LIMITS = limits] [, /ARRAY ] [, /WaveLength]'
   return
 endif
 emin = 1.0e9
;
      if n_elements(wdf1) eq 0 then begin
        print, '% TEMPERATURE: Parameter must be defined'
        return
      endif
    if keyword_set(array) then yval = wdf1 else begin
      
      if wdf1 lt 1 or wdf1 gt get_maxwdf() then begin
        print, '% TEMPERATURE: WDF array must be in the range, [ 0, ' + $
          strtrim(get_maxwdf(), 2) + ' ]'
        return
      endif
;
      yval = (getyf(wdf1)) > emin
      if n_params() gt 1 then save = get_save_array(wdf2, wdf1) else save = wdf1
    endelse
;
      if n_elements (minimum) eq 0 then minimum = -1
      if n_elements (maximum) eq 0 then maximum = -1
      if minimum le 0 and maximum le 0 then lc_range = 0 else begin
        if minimum lt 0 then minimum = 0
        if maximum le 0 then lc_range = -1 else lc_range = 1
      endelse
      if minimum eq maximum and lc_range gt 0 then begin
        print, '% TEMPERATURE: Maximum energy must be greater than the minimum'
        return
      endif
;
         
      if lc_range eq 0 then begin
         
        yval = (yval/1.028300544e+09)^.25

      endif else if lc_range lt 0 then begin
        if n_elements(limits) lt 2 then limits = [0.5, 10.0]
       
        nel = n_elements(yval)
        rat = 1.00
        for i = 0, nel-1 do yval[i] = $
         calc_temp(yval[i], rat, minimum, np= np, limits = limits)
      endif else begin
        if n_elements(limits) lt 2 then limits = [0.5, 10.0]
        nel = n_elements(yval)
        rat = 1.00
        for i = 0, nel-1 do yval[i] = $
          calc_temp(yval[i], rat, minimum, maximum, np= np, limits = limits)
      endelse
      if keyword_set(array) then begin
        wdf2 = yval
        return
      endif

      if save ne wdf1 then xfr, wdf1, save
      puty, yval, save
      cha, save, ylabel = 'Temperature(eV)', $
        clab= 'BB Temperature from ' + wdcomment(wdf1)
END
