; ------------------------------------------------------------------------ 
; $Id: tsh.pro,v 1.6 2010/09/17 21:23:38 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
 Pro  tsh, wdfar, valuein, delta = deltain, ignore = ignore
;
;+
;  Performs time shift on a WDF array.  This procedure can be used to 
;  synchronize two arrays or to insure an array begins at zero.
;  Format:  TSH, WD1    [, AMOUNT] [, DELTA = delta] [, /IGNORE]
;  where:
;    wd1 - WDF array number or an array of wdf array numbers
;    amount - time shift.  If amount is zero or omitted, then the 
;             array will start at zero by deleting all points before 
;             zero or by adding zero amplitude points if wd1 begins after zero.
;        Note: XY data will start at zero by adding points if necessary, but then
;              only those points greater than zero will be included, ie, 
;              except for the first point, no additional points will be added to
;              linearly interpolate data to x =0..
;    IGNORE - If set, AMOUNT = 0 will do nothing to WD1.
;    delta  - ignored unless XY data. If XY data this will be twice the spacing
;             for additional points if required. 
;             If delta is not specified, then TSH will set delta to the minimum
;                of the minimum current spacing or 1e-4*max(x)
;      XY data is handled according to:
;            1. If the first X value is within 50% of delta then the first point
;                 will be moved by linear interpolation to zero
;            2. If the first X value is between 1.0 and 0.5 of delta then
;                 a zero value will be added at zero
;            3. If the first X value is greater than delta then a point will
;                 be added at x = x0-0.5*delta and at zero with a y-value of zero
;
;  Examples:
;  Make array 1 begin at an abscissa value of zero.  Add initial zeros
;         if necessary.
;     tsh, 1
;  Time shift (add to the abscissa values of) array 1 the amount, -2.42e-6
;     tsh, 1, -2.42e-6
;-
;
;     include the common block specification
;
@com.dat
;
      on_error, 1
      if n_elements(wdfar) eq 0 then begin
        print, 'wdf array must be specified'
        return
      endif
      if n_elements (valuein) eq 0 then value = 0.0000 else begin
        value = valuein[0]
        if value eq 0.0 and keyword_set (ignore) then return
      endelse
      nelwdf = n_elements(wdfar)
    for iii = 0, nelwdf-1 do begin
      wdf = wdfar(iii)
      if wdf lt 1 or wdf gt maxwdfarray then begin
        print, 'wdf array must be greater than 0 and </=', maxwdfarray
        goto, endshift
      endif
      nel = wdfnp(wdf-1)
      if nel lt 2 then begin
        print, ' wdf array', wdf, ' does not have valid data'
        goto, endshift
      endif
      get_wdfarray, array, wdf, /xy
      if (n_elements(array) ne (nel+2)) then begin
; xy data
        if value eq 0.0 then begin
          if n_elements(deltain) eq 0 then $
            delta = min(abs(array(1:nel-1)-array(0:nel-2))) > $
                 (1.0e-4*wdfmaxx(wdf-1)) $
          else delta= deltain
          ok = where (array(0:nel-1) ge 0.0, nok)
          if nok lt  2 then begin
            print,'WDF array - ',strtrim(wdf,2) , $
               ' does not have but ', strtrim (nok, 2), ' positive data values'
            goto, endshift
          endif else begin
;            print, 'delta' , delta
            if array(0) gt 0.0 then begin
              if array(0) gt delta then begin
;               add points at zero and before the data with zero amplitude
                space = [0.0, array(0)- 0.5*delta, array(ok)]
                image = [0.0, 0.0, array(ok+nel)]
              endif else begin
                if array(0) ge 0.5*delta then begin
;                 add a zero point at zero
                  space = [0.0, array(ok)]
                  image = [0.0, array(ok+nel)]
                endif else begin
;                 the first point is almost at zero so just linearly interpolate
;                   back to zero and add a point at 0.5 delta.
                  if array(1) gt array(0) then $
                    space = [0.0, 0.5* delta, array(ok(1:nok-1))] $
                    else space = [0.5* delta, 0.0, array(ok(1:nok-1))]
                  y1= array(nel)+(space(0)-array(0))*(array(nel+1)-array(nel)) $
                           /(array(1)-array(0))
                  y2= array(nel)+(space(1)-array(0))*(array(nel+1)-array(nel)) $
                           /(array(1)-array(0))
                  image = [y1, y2, array(ok(1: nok-1)+nel)]
                endelse
              endelse
            endif else begin
              if array(ok(0)) gt 0.0 then begin
;               interpolate back to zero 
;               if close to zero, just move first point otherwise add a point
                if array(ok(0)) ge 0.5*delta then begin

                  space = [0.0, array(ok(0:nok-1))]
                  i = ok(0)-1+nel
                  j = ok(0)+nel
                  image = [array(j)+(array(j)-array(i))*array(ok(0))/ $
                     (array(ok(0))-array(ok(0)-1)), array(nel+ok(0:nok-1))]
                endif else begin
                  space = [0.0, array(ok(1:nok-1))]
                  i = ok(0)-1+nel
                  j = ok(0)+nel
                  image = [array(j)+(array(j)-array(i))*array(ok(0))/ $
                     (array(ok(0))-array(ok(0)-1)), array(nel+ok(1:nok-1))]
                endelse
              endif else begin
;             we happened to hit it exactly on zero.
                space = array(ok)
                image = array(nel+ok)
              endelse
            endelse
          endelse
          put_wdfarray, [space, image], wdf, /xy
          space = 0
          image = 0
        endif else begin
          array(0:nel-1) = array(0:nel-1) + value
          put_wdfarray, array, wdf, /xy, /no_copy
        endelse
      endif else begin
;  Uniform data
        if value eq 0.0 then begin
          stop = wdfmaxx(wdf-1)
          if stop lt 0 then begin
            Print, '%TSH: Cannot time shift to zero. No data after zero.'
            print, '      No shift made on array: ', strtrim (wdf, 2)
          endif else begin
            if array(0) gt 0.00 then begin
              nel = nel+2
              space = make_array(nel, /index)*array(1)+(array(0)-2.0*array(1))
              image = make_array(nel, value = 0.0)
              image[2:*] = array(2:*)
              if space(0) gt 0.0 then space(0) = 0.0
            endif else begin
              space = make_array(nel, /index)*array(1)+array(0)
              image = array(2:*)
            endelse
            delta = array(1)
            put2uniform, space, image, nel, value, delta, stop, array
          endelse
        endif else array(0) = array(0) + value
        put_wdfarray, array, wdf, /no_copy
      endelse
  endshift:
    endfor
      return
      end
