; ------------------------------------------------------------------------
; $Id: wdffree.pro,v 1.3 2010/09/08 16:56:00 lpmix Exp $
; ------------------------------------------------------------------------
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
;+
; NAME:
;       WDFFree
;
; PURPOSE:
;       Return free WDF array numbers, that is a list of WDF array numbers
;         with no data associated with them.
;       Note:  USE DELWDF to eliminate data from WDF arrays. 
;
; CALLING SEQUENCE:
;       Free_Wdf = WDFFree(Minimum)
;
; OUTPUTS:
;       FREE_WDF: An array of WDF values which are available
;                 The scalar -1 is returned if none are available.
;
; INPUTS:
;       MINIMUM:    This is the minimum WDF array desired
;                   If present, MINIMUM keyword is ignored
;
; KEYWORD PARAMETERS:
;       NElements:  Number of values in FREE_WDF
;       MINIMUM:    Minimum value of WDF values desired.
;                   MINIMUM must satisfy: MINIMUM = (MIMIMUM > 1) < GET_MAXWDF()
;       MAXIMUM:    Maximum value of WDF values desired.
;                   MAXIMUM must satisfy: MAXIMUM = (MAXIMUM > MINIMUM) < GET_MAXWDF()
;
; OUTPUTS: Array of unused WDF array numbers.
;
; SIDE EFFECTS:
;	NONE
;
; PROCEDURE:
;       WDFFree uses an internal array to determine WDF arrays with no data.
;
; EXAMPLE:
;       Determine arrays that are free between 100 and 200; store in FREE
;         Free = WDFFREE(MINIMUM = 100, MAXIMUM = 200)
;
; MODIFICATION HISTORY:
; 	Written by:	L. P. Mix, Jr., September 12, 2008
;-

Function WDFFREE, a, minimum = minimum, maximum = maximum, nelements = nelements

on_error, 1
@com.dat
;
; Set default value
if n_elements(initwdf) eq 0 then wdfinit
if n_elements(a) gt 0 then a = (a[0]  > 1) < maxwdfarray else begin
  if n_elements(minimum) eq 0 then a = 1 else a = (minimum > 1) < maxwdfarray
endelse
if n_elements(maximum) eq 0 then b = maxwdfarray else b = (maximum[0] > a) < maxwdfarray

answ = where(wdfnp[a-1:b-1] eq 0, nelements) 
answ = (nelements eq 0) ? -1 : answ+a
return, answ

END
