; ------------------------------------------------------------------------ 
; $Id: wdfit.pro,v 1.9 2011/04/28 14:42:22 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
 Pro  wdfit, wdf1in, wdf2, degree = degreein, weight = weight, coefficient = coef, $
            xtype = xtype, ytype = ytype, plot = plot, Quiet = quiet, $
            left = left, right = right, double= double, sda = sda, sdr = sd , rsquared = rr
;+
;  Fit a WDF array to a polynomial using the IDL user function, SVDFIT.
;  Format:  WDFIT, wd1 [, wd2] [, degree = degree] [, WEIGHT =weight]
;                    [, LEFT = left] [, RIGHT = right]
;                    [, COEF =coef], [, XTYPE = xtype] [, YTYPE = ytype]
;                    [, DOUBLE = double] [, QUIET = quiet] [, PLOT = plot]
;                    [, SDR = sd] [, SDA = sda] [, RSQUARED = rsquared]
;  Note: Relative standard deviation is sqrt((y -yfit)^2/(np-1))/max(abs(y))
;        where yfit is the fitted y value and np is the number of points
;  Note: GET_MAXWDF() is used as a work array if plot is set
;  Note: GET_MAXWDF()-1 is used as a work array if LEFT or RIGHT is set
;  where:
;    wd1, wd2 - WDF array numbers
;               if wd2 is present, the final curve is stored in wd2
;               otherwise the result is stored in wd1
;    degree   - the degree of the polynomial fit -- default value is 1
;               i.e. a linear fit (NOTE: for the average value (degree =0)
;               use the AVE routine)
;    weight   - optional weight array for the points -- default is 1
;    LEFT     - If set, it will be used to set the data minimum
;    RIGHT    - If set, it will be used to set the data maximum
;    coef     - coef of the polynominal
;    X[Y]type - If present and not zero, then a log fit in the X[Y] direction
;               is performed.  Only points greater than zero will be used in 
;               the log direction.
;    plot     - If present and not zero, a plot of the fit is performed.
;               Default is plot = 0
;    quiet    - If present and not zero, no output will be sent.
;               Default is quiet = 0
;    double   - If set to zero then the calculation will be done in single
;               precision.  Default is double precision.
;    SDR      - Relative standard deviation
;  Note: Relative standard deviation is sqrt((y -yfit)^2/(np-1))/max(abs(y))
;        where yfit is the fitted y value and np is the number of points
;    SDA      - Absolute standard deviation
;  Note: Absolute standard deviation is sqrt((y -yfit)^2/(np-1))
;        where yfit is the fitted y value and np is the number of points
;    RSQUARED - Correlation coefficient R^2 defined for a linear fit as:
;               Rsquared = (coef[0]*total(y)+coef[1]*total(x*y)- $
;                 (total(y)^2)/np) /(total(y*y)-(total(y)^2)/np)
;               where np is the number of x-y points
;               If not a linear fit then RSQUARED = 1
;
;  Examples:
;    Fit data points in array 2 with a linear fit; return the data to array 2
;      WDfit, 2
;
;    Fit data points in array 2 to a polynomial of the form y = c(j)*x^j 
;       where j goes from 0 to 3 and return the data to array 3;
;      WDfit, 2, 3, d= 3
;-
;
@com.dat
;
      on_error, 1
      if n_elements(initwdf) eq 0 then wdfinit
      nparams = n_params()
      if nparams lt 1 then begin
        print,'% Format: ', $
'WDFIT, wd1 [, wd2] [, degree = degree] [, WEIGHT =weight] [, LEFT = left]'
print,'             [, RIGHT = right], [, XTYPE = xtype] [, YTYPE = ytype]'
print,'             [, COEFFICIENT =coef], [, QUIET = quiet] [, PLOT = plot]'

        return
      endif
;
      if n_elements(wdf1in) eq 0 then begin
        print, 'Parameters to WDfit must be defined'
        return
      endif else wdf1 = wdf1in
      if wdf1 lt 1 or wdf1 gt maxwdfarray then begin
        print, 'wdf array must be greater than 0 and </=', maxwdfarray
        return
      endif
;
      if nparams gt 1 then save = get_save_array(wdf2, wdf1) else save = wdf1
      if save lt 1 or save gt maxwdfarray then save = wdf1
      nel = wdfnp(wdf1-1)
      if nel eq 0 then begin
        print, 'No valid data in the array'
        return
      endif
      if (n_elements(left) + n_elements(right)) gt 0  then begin
        work = maxwdfarray-1
        xfr, wdf1, work
        ll = (n_elements(left) gt 0) ? left[0] : wdfminx(work-1)
        rr = (n_elements(right) gt 0) ? right[0] : wdfmaxx(work-1)
        lim, work, ll, rr
        wdf1 = work
        nel = wdfnp(wdf1-1)
      endif
      get_wdfarray, array, wdf1, /xydata
      nela = n_elements(array)-2
      if nela eq nel then begin 
        a = array(2:*)
        b=findgen(nel)*array(1)+array(0)
        xydata = 0
      endif else begin
        a = array(nel:*)
        b = array(0:nel-1)
        xydata = 1
      endelse
      if n_elements (degreein) gt 0 then degree = abs(degreein[0]) < (nel-1) $
                                  else degree = 1
      m= (degree+1) > 2
      nelw =  n_elements (weight)
      if nelw gt 0 and nelw ne nel then begin
        nelw = 0
        print, ' Weight array must have 1 element for each WDF array element'
        print, ' Weight has', nelw, ' elements and WDF array has',nel
        print, ' Weight of 1 will be used for each data point'
      endif
      if n_elements (xtype) eq 0 then xtype = 0
      if xtype ne 0 then xtype =1
      if n_elements (ytype) eq 0 then ytype = 0
      if ytype ne 0 then ytype =1
      wdf = 1
      sing = 0
      if ytype eq 1 then begin
        oy = where(a gt 0, np)
        if np eq 0 then begin
          print, 'Log y is not possible - No points are greater than zero'
          return
        endif
        aa = alog(a(oy))
        bb = b(oy)
      endif else begin
        aa = a
        bb = b
      endelse
      if xtype eq 1 then begin
        ox = where(b gt 0, np)
        if np eq 0 then begin
          print, 'Log x is not possible - No points are greater than zero'
          return
        endif
        bs = bb(ox)
        bb = alog(bs)
        aa = aa(ox)
      endif else begin
;       scale the x data if linear x 
        xscal = max(abs([wdfmaxx(wdf1-1), wdfminx(wdf1-1)]))*0.5d0
        bb = bb/xscal
      endelse
      if n_elements(double ) eq 0 then double = 1
      if nelw eq 0 then $
        coef = svdfit (bb, aa, m, yfit=wdf, singular=sing, double=double) else $
        coef = svdfit (bb, aa, m, yfit=wdf, weight = weight, singular=sing, $
           double = double)
      if sing ne 0 then retall
      sda = sqrt(total((aa-wdf)^2)/(n_elements(aa)-1))
      sd =  sda / max(abs(aa))
      np = n_elements(aa)
      if (m eq 2) then rr =  (coef[0]*total(aa)+coef[1]*total(bb*aa)- $
           (total(aa)^2)/np) /(total(aa*aa)-(total(aa)^2)/np) else rr = 1
      if ytype eq 1 then wdf = exp(wdf)
      if xtype eq 1 then bb = bs else begin
        bb = bb *xscal 
        for i = 1, degree do coef(i) = coef(i)/(xscal^i)
      endelse
      if n_elements(quiet)eq 0 then quiet =0
      if n_elements(plot) eq 0 then plot = 0
      xsort = sort (bb)
      bb = bb(xsort)
      wdf = wdf(xsort)
      if plot ne 0 then begin
        i2w, bb, wdf, maxwdfarray, c = 'Fit to '+ wdfcomment(wdf1-1), $
            xydata= xydata
        plo, [wdf1, maxwdfarray], /over, /color, /ns, xtype=xtype, $
          ytype=ytype, Title ='Least Square Fit'
        xfr, maxwdfarray, save
      endif else i2w, bb, wdf, save, c = 'Fit to '+ wdfcomment(wdf1-1), $
        xydata = xydata
      if quiet eq 0 then begin
        if xtype eq 0 and ytype eq 0 then begin
          print,' Fit of the form y = c(0) + c(1)*x +c(2)*x^2 + ...'
          print,' Maximum power of x is ',strtrim(string(m-1),2)
          print,' Relative standard deviation is ', sd*100., ' %'
          print,' Coefficients are:'
          for i = 0, m-1 do print, 'C(', strtrim(string(i),2), ') =', coef(i)
        endif else if xtype eq 0 and ytype eq 1 then begin
          print,' Fit of the form y = exp (c(0) + c(1)*x +c(2)*x^2 + ...)'
          print,' Maximum power of x is ',strtrim(string(m-1),2)
          print,' Relative standard deviation is ', sd*100., ' %'
          print,' Coefficients are:'
          for i = 0, m-1 do print, 'C(', strtrim(string(i),2), ') =', coef(i)
        endif else if xtype eq 1 and ytype eq 1 then begin
          print,' Fit of the form log(y) = c(0)+c(1)*log(x)+c(2)*log(x)^2+...)'
          print,' Maximum power of x is ',strtrim(string(m-1),2)
          print,' Relative standard deviation is ', sd*100., ' %'
          print,' Coefficients are:'
          for i = 0, m-1 do print, 'C(', strtrim(string(i),2), ') =', coef(i)
        endif else begin
          print,' Fit of the form y = c(0)+c(1)*log(x)+c(2)*log(x)^2+...)'
          print,' Maximum power of x is ',strtrim(string(m-1),2)
          print,' Relative standard deviation is ', sd*100., ' %'
          print,' Coefficients are:'
          for i = 0, m-1 do print, 'C(', strtrim(string(i),2), ') =', coef(i)
       endelse
      endif
;
      wdf2 = save
      wdffile(save-1) = '  '
      wdfxlabel(save-1)= wdfxlabel(wdf1-1)
      wdfylabel(save-1)= wdfylabel(wdf1-1)
      array=0
      wdf = 0
      a = 0
      b = 0
      end
