; ------------------------------------------------------------------------ 
; $Id: wdsplit.pro,v 1.6 2010/09/03 18:30:50 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
;+
; NAME:
;	WDSPLIT
;
; PURPOSE:
;       Split a waveform into two parts. Either an exponential or a 
;       linear ramp can be selected. The start point and the 
;         a. Zero point for a linear ramp.
;         b. Exp(-3) point for an exponential
;       is selected. The remainder is the difference between the original 
;       waveform and the first part. 
;    Note: Breaks occur only at existing X-values.
;
; CALLING SEQUENCE:
;	WDSPLIT, WDF [, _EXTRA= extrastuff]
; or    WDSPLIT, WDF, WDF2 [, _EXTRA= extrastuff]
; or    WDSPLIT, WDF, WDF1, WDF2 [, _EXTRA= extrastuff]
;
;  where:
;       WDF   Waveform array which is to be split
;       WDF1  Storage location for First Half of WDF
;             Default location is to replace WDF
;       WDF2  Storage location for Second Half of WDF
;             Default location is WDF +1
;  KeyWords:
;       X/YSize: Size of the graphics window
;       LCTYP Type of fit: 0= Linear, 1= Exponential
;       Any keywords use by the PLO command.
;  Note:
;       Mouse buttons are mapped to widget buttons, cursor must be in the
;         DRAW area of the main widget for the following mouse commands.
;          Left Button   = Set Value
;              Note: A carriage return in the data field will also set the value.
;          Middle Button = Accept/Save
;             (current designated lineout will be saved)
;          Right Button  = Reset All Points
;             (All points will be undesignated)
;          Note: If a the last point required to define a curve is selected
;                more than once, the previous "last point" is replaced.
;       Set Point Buttons: These perform the same function as a <CR> in either
;                          the X or Y windows above the buttons.
;
; EXAMPLE:
;	Split array 60 and put the parts in 60 and 61.
;         WDSPLIT, 60
;
;       Split array 40 and put the parts in 40 and 50
;         WDSPLIT, 40, 50
;
;       Split array 45 and put the first and second halves in 50 and 60
;       Do not change 45 and plot only the range [30, 90].
;         WDSPLIT, 45, 50, 60, xrange = [30, 90]
;
; MODIFICATION HISTORY:
; 	Written by:	L. P. Mix, January 3, 1997
;-


PRO wdsplit_event, Event
 COMPILE_OPT hidden
on_error , 1
common wdsplit_common, xwindow, ywindow, image,xa, xb, draw5,  $
       nsel, xcrange , ycrange, lcsel, np, x, y, yy,  np1, np2,x0, dx 

  WIDGET_CONTROL,Event.Id,GET_UVALUE=Ev

  CASE Ev OF 

  'FIELD10': BEGIN
      widget_control, xwindow, get_value = xp
      widget_control, ywindow, get_value = yp
      nsel = nsel>1 
      wdsplit_draw, xp, yp
      widget_control, xwindow, set_value = x(np1)
      END

  'FIELD12': BEGIN
      widget_control, xwindow, get_value = xp
      widget_control, ywindow, get_value = yp
      nsel =  2
      widget_control, draw5, draw_motion_events = 0
      wdsplit_draw, xp, yp
      widget_control, ywindow, set_value = x(np2)
      END

  'SLOPE': BEGIN
       lcsel = event.value
       if nsel gt 0 then begin
         if xb gt xa then begin
           xp=xa & yp = xb 
           wdsplit_draw, xp, yp
         endif
       endif
       END

  'BUTTON11': BEGIN
       wdsplit_reset
       END

  'BUTTON12': BEGIN
      nsel = -1
      widget_control, event.top, /destroy
      END

  'BUTTON13': BEGIN
      wdsplit_exit, event
      END

  'DRAW5': BEGIN
      if event.press  eq 4 then wdsplit_reset else begin
        if event.press eq 2 then wdsplit_exit, event else begin
          coord = convert_coord(event.x, event.y, /device, /to_data)
          if event.press eq 1 then begin
            if nsel eq 0 then begin
              xp = coord(0)
              widget_control, ywindow, get_value = yp
              nsel = 1
              wdsplit_draw, xp, yp
              widget_control, xwindow, set_value = x(np1)
            endif else if nsel ge 1 then begin
              yp = coord(0)
              widget_control, xwindow, get_value = xp
              nsel = 2
              widget_control, draw5, draw_motion_events = 0
              wdsplit_draw, xp, yp
              widget_control, ywindow, set_value = x(np2)
            endif 
          endif else begin
            if nsel eq 0 then begin
              xp = coord(0)
              widget_control, ywindow, get_value = yp
              wdsplit_draw, xp, yp
              widget_control, ywindow, set_value = x(np1)
            endif else if nsel ge 1 then begin
              yp = coord(0)
              widget_control, xwindow, get_value = xp
              wdsplit_draw, xp, yp
              widget_control, ywindow, set_value = x(np2)
            endif 
          endelse
        endelse
      endelse
      END
   else: Print, 'No Handler for event ', ev
  ENDCASE
END

Pro wdsplit_exit, event
 COMPILE_OPT hidden
    common wdsplit_common
    on_error, 1
      if nsel lt 2 then $
        wok_notok,lcquit,'Only '+strtrim(nsel,2)+' point has been designated', $
            'Do you wish to continue the Split?', /modal, group_leader = event.top $
             else lcquit = 1
      if lcquit eq 1 then  widget_control, event.top, /destroy
    return
    end

Pro wdsplit_reset
 COMPILE_OPT hidden
    common wdsplit_common
    on_error, 1
    nsel = 0
    device , copy = image
    widget_control, draw5, draw_motion_events = 1
    return
    end

pro wdsplit_draw, x1, x2
 COMPILE_OPT hidden
    common wdsplit_common
    on_error, 1
    if nsel eq 0 then begin
      if x1 ge xcrange(0) and x1 le xcrange(1) then begin
        device , copy = image
        plots, [x1, x1], ycrange
      endif
    endif else begin
      if x1 ne xa then begin
        xa = x1
        np1 = (round((x1-x0)/dx) < (np-1)) > 0
        yy(0:np1) = y(0:np1)
      endif
      if x2 ge x1 then begin
        if x2 ne xb then begin
          xb = x2
          np2 = (round((x2-x0)/dx) < (np-1)) > 0
        endif
        if nsel ge 1 then begin
          if np2 lt np-1 then yy(np2+1:np-1)= 0.0
          if np1 lt np2 then begin
            if lcsel eq 0 then begin
              ns = np2-np1
              r = findgen(ns+1)*(y(np1)/ns)
              yy(np1:np2)= y(np1)-r
            endif else begin
              n =  np-np1
              r = make_array(n, value=0.0)
              nmax = 1+(np2-np1)*6 < n
              r(0:nmax-1)=exp(-findgen(nmax)*(3.0/(np2-np1)))
              yy(np1:*)=yy(np1)*r
            endelse
          endif
          device , copy = image
          oplot, x,  yy, col = 1
          if nsel gt 1 then oplot, x, (y- yy), col = 2
        endif
      endif else if nsel eq 1 then begin
        if x1 ge xcrange(0) and x1 le xcrange(1) then begin
          device , copy = image
          plots, [x1, x1], ycrange
        endif
      endif
    endelse
    return
    end


  PRO wdsplit, wdf, WDFa, WDFb, xrange = xrange, yrange = yrange, $
      xsize =xsizein , ysize = ysizein, $
     lctyp = lctyp, GROUP=Group, modal = modal, _extra= extrastuff

on_error, 1
common wdsplit_common
common pffcom, initpff, pff_exe
@draw_defaults.com
@com.dat
common plot_grid_last, yeng_scale, xeng_scale

      if n_elements(initwdf) eq 0 then wdfinit
      nparams = n_params()
      if nparams eq 0 then begin
        print, '% Format: WDSPLIT, WDF, [, WDFa] [, WDFb] [, _EXTRA = extrastuff]'
        return
      endif
;
      nsel = 0
      if n_elements(wdf) eq 0 then begin
        print, 'Parameters to WDSplit must be defined'
        return
      endif
      if wdf lt 1 or wdf gt maxwdfarray then begin
        print, 'WDF array must be greater than 0 and </=', maxwdfarray
        return
      endif
;
      nel = wdfnp(wdf-1)
      if nel lt 2 then begin
        print, 'No data in the WDF array'
        return
      endif
;
      save = [wdf, wdf+1]
      if nparams eq 1 then begin
        if save(1) gt maxwdfarray then save = save-1
      endif else if nparams eq 2 then begin
        if wdfa gt 0 and wdfa le maxwdfarray then save(1) = wdfa
      endif else begin
        if wdfa gt 0 and wdfa le maxwdfarray then save(0) = wdfa
        if wdfb gt 0 and wdfb le maxwdfarray then save(1) = wdfb
      endelse

  IF N_ELEMENTS(Group) EQ 0 THEN GROUP=0

  main13 = WIDGET_BASE(GROUP_LEADER=Group, $
      COLUMN=1, $
      MAP=1, modal = modal, $
      TITLE='X - Cursor')
  BASEINS  = WIDGET_BASE(main13, $
      COLUMN = 1, $
      MAP=1, FRAME = 2,  $
      UVALUE = 'BASEINS')

 labelins1 =  WIDGET_LABEL( BASEins, $
      UVALUE='LABEL', $
      VALUE='Left Mouse = Designate Point,     Middle Mouse = Save,  ' $
           +'   Right Mouse = Reset')


  BASE8 = WIDGET_BASE(MAIN13, $
      row=1, $
      FRAME=0, $
      MAP=1, $
      UVALUE='BASE8')

      Btns = [ $
    'Linear', $
    'Exponential']

   SLOPE_sel = CW_BGROUP( Base8, Btns, $
      COLUMN=1, /frame, $
      EXCLUSIVE=1, /RETURN_INDEX, /NO_RELEASE, $
      Label_Top = 'Ramp Function', $
      UVALUE='SLOPE')


  FieldVal435 = 0.0

   xwindow = CW_FIELD( BASE8,VALUE=FieldVal435, $
      column=1, $
      FLOAT=1, /frame, $
      xsize = 12, $
      RETURN_EVENTS = 1, $
      TITLE='Ramp Start X:', $
      UVALUE='FIELD10')

  FieldVal437 = 0.0

  ywindow = CW_FIELD( BASE8,VALUE=FieldVal437, $
      column=1, $
      float=1, /frame, $
      xsize = 12, $
      TITLE='Ramp End X:', $
      RETURN_EVENTS = 1, $
      UVALUE='FIELD12')
        
  Button11 = WIDGET_button( BASE8, $
      UVALUE='BUTTON11', $
      VALUE=' R E S E T ')

  Button12 = WIDGET_button( BASE8, $
      UVALUE='BUTTON12', $
      VALUE=' Quit - No Split ')

  BUTTON13 = WIDGET_BUTTON( BASE8, $
      UVALUE='BUTTON13', $
      VALUE=' D O N E ')



  oldwin = !d.window
  if n_elements(xsizein) eq 0 then xsize = medsize(0) else xsize = xsizein
  if n_elements(ysizein) eq 0 then ysize = medsize(1) else ysize = ysizein

  DRAW5 = WIDGET_DRAW( MAIN13, $
      BUTTON_EVENTS=1, $
      MOTION_EVENTS=1, $
      RETAIN=1, $
      UVALUE='DRAW5', $
      XSIZE=xsize, $
      YSIZE=ysize)

  WIDGET_CONTROL, MAIN13, /REALIZE
  if n_elements (lcsel) eq 0 then lcsel = 0
  if n_elements (lctyp) gt 0 then lcsel= lctyp(0)< 1
  if lcsel lt 0 then lcsel = 0
  
  widget_control, slope_sel, set_value = lcsel

; Get drawable window index
  WIDGET_CONTROL, DRAW5, GET_VALUE=DRAW5_Id
  wset , draw5_id
  plo, wdf, xrange = xrange, yrange = yrange, _extra = extrastuff
  get_wdfarray, array, wdf
; get old values
  xcrange = !x.crange
  ycrange = !y.crange
 
  window, /free , /PIXMAP  , xsize = xsize, ysize = ysize
  pixwin = !d.window
  image = [0, 0, xsize, ysize, 0, 0, draw5_id]
  device, copy  = image
  image[6] = pixwin

  wset , draw5_id
  npmax = 512L
  np = n_elements(array)-2
  if xeng_scale ne 1.000 then begin
    x0 = array(0)*xeng_scale
    dx = array(1)*xeng_scale
  endif else begin
    x0 = array(0)
    dx = array(1)
  endelse
  x = findgen(np)*dx+x0
  if yeng_scale ne 1.000 then $
    y = array (2:*)*yeng_scale $
    else y = array (2:*)
  yy = y
  xa = x0
  xb = x0
  np1= 0L
  np2= 0L
  XMANAGER, 'wdsplit', MAIN13
  image = 0
  if oldwin ge 0 then wset, oldwin
  wdelete, pixwin
  if nsel lt 2  then return
  if yeng_scale ne 1.000 then yy = yy/yeng_scale
  put_wdfarray,[array(0:1), yy], save(0)
  put_wdfarray,[array(0:1), (array(2:*)-yy)], save(1)
  if save(0) ne wdf then begin
    wdfylabel(save(0)-1)= wdfylabel(wdf-1)
    wdfxlabel(save(0)-1)= wdfxlabel(wdf-1)
  endif
  wdfylabel(save(1)-1)= wdfylabel(wdf-1)
  wdfxlabel(save(1)-1)= wdfxlabel(wdf-1)
  comm = wdfcomment(wdf-1)
  array = 0
  wdfcomment(save(0)-1)= 'First Split of '+comm 
  wdfcomment(save(1)-1)= 'Second Split of '+comm 
END
