; ------------------------------------------------------------------------ 
; $Id: wdvalid.pro,v 1.6 2010/09/20 14:29:06 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
   Function wdvalid,stringin, maximum=maximumin, minimum=minimumin, $
       case_sensitive = casesen, npoint= npoint, quiet = quiet, $
       valid = valid
;+
; Name: WDVALID
;
; NOTE: VALID is a flag to determine the mode of this routine.
;
;     If keyword(valid) is not set.
;     This function returns an array of valid WDF array numbers.
;     Minimum , maximum and a search string can be specified.
;     
; or
;     If keyword(valid) is set.
;     This function returns an array of valid WDF array numbers
;     from a proposed array of numbers or a string.
;
;
; Format:
;   array = WDVALID([STRING] , [Maximum = maximum] [,Miminum = minimum]
;           [,Npoint = npoint] [,case_sensitive = case_sensitive])
;           [, VALID = valid] [, QUIET = quiet]
;
;   where:
;    IF VALID is zero or omitted:
;         STRING is a search string. Leading and trailing blanks are ignored.
;                String may be a number. This routine uses:
;                       str = strtrim(string(0), 2)
;                  where str is the actual search string
;                  For example s = wdvalid(180) will return 
;                   a wdf array with a comment of 'BDOT_180'
;                If missing, all valid arrays are returned.
;         Minimum is the minimum value desired, Default = 1
;         Maximum is the maximum value desired, Default = maxwdfarray
;           Maximum  and minimum are sorted.
;         Npoint is the number of values returned.
;         Case_sensitive if set makes the search case sensitive.
;         CAUTION: CASE is a special word so use /c or /ca or /cas or /case_
;                  or more letters but /case will give an error.
;    IF VALID is not zero:
;         STRING  - an array of possible WDF arrays.
;                   Must be a byte, integer, long or a float variable.
;                   Floats are converted to LONG using ROUND
;                   See GET_MATCH for wild card options
;                   A search string.
;                     If an array, only the first element is used.
;         NPOINT  - the number of values returned.
;         QUIET   - if set, not output will be generated.
;                   if not set, messages will be printed if STRING contains
;                     invalid wdf arrays.
;                    Messages of the form if VALID is a string:
;                   "% "+STRTRIM(VALID, 2)+": "+ message
;              or   "% INVALID WDF: " + message
;           where message =
;                WDF array number is UNDEFINED
;         or     WDF array must be a byte, integer, long, or float value 
;         or     WDF array number must be greater than 0 and </= MX
;         or     No data in N WDF array 
;  where MX is the maximum number of arrays and N is the number of arrays.
;
;   For both states of VALID, if NPOINT = 0 then ARRAY = -1
;
;   Example:
;      find all WDF arrays containing data and store in VALS
;          VALS = wdvalid()
;      find arrays with bdot and average them and store the result in 34
;          SUM, wdvalid('bdot'), 34, /average, clabel = 'Average bdots' 
; or       SUM, 'bdot', 34, /average, clable = 'Average Bdots'
;
;      determine if wdin is a valid WDF array with data. Do not print errors.
;          wdgood = wdvalid(wdin, /valid, /quiet, npoint = npoint)
;      Same as above but print an error message beginning with 'MY_ADD_IT'
;          wdgood = wdvalid(wdin, npoint = npoint, valid = 'MY_ADD_IT')
;-
@com.dat
on_error, 1
      npoint = 0
      ok = -1
      if n_elements(initwdf) eq 0 then return, ok
      if keyword_set (valid) then begin
        nel = n_elements (stringin)
        if nel eq 0 then begin
          if not (keyword_set(quiet)) then begin
            if (size(valid(0)))(1) eq 7 then print, '% '+ strtrim(valid(0), 2) $
              + ': WDF array is UNDEFINED' else print, '% INVALID WDF' $
              + ': WDF array is UNDEFINED'
          endif
          return, ok
        endif
        typin = (size(stringin(0)))(1)
        if typin eq 7 then begin
;         str = strtrim(stringin(0), 2)
          str = stringin(0)
          if strlen(str) gt 0 then $
            ok = get_match(str, /wdf, nelement=npoint, casesen=casesen)
          if npoint gt 0  then return, ok
          if not keyword_set(quiet) then begin
            if (size(valid(0)))(1) eq 7 then print, '% '+ strtrim(valid(0), 2) $
              + ': No structures with the string: ' + str $
                else print, '% INVALID WDF' $
              + ': No WDF arrays with the string: ' + str
          endif
          return, -1
        endif else if typin gt 4 then begin
          if not (keyword_set(quiet)) then begin
            if (size(valid(0)))(1) eq 7 then print, '% '+ strtrim(valid(0), 2) $
              + ': WDF array must be a byte, integer, long, or float value' $
               else print, '% INVALID WDF' $
              + ': WDF array must be a byte, integer, long, or float value'
          endif
          return, ok
        endif 
        if typin eq 4 then okin = round (stringin) else okin = long(stringin)
        ok = where(okin gt 0 and okin le maxwdfarray, npoint)
        if npoint lt nel and not keyword_set(quiet) then begin
            if (size(valid(0)))(1) eq 7 then print, '% '+ strtrim(valid(0), 2) $
              + ': WDF array number must be greater than 0 and </= ',  $
               strtrim(maxwdfarray, 2) else print, '% INVALID WDF' $
              + ': WDF array number must be greater than 0 and </= ',  $
               strtrim(maxwdfarray, 2)
            print, nel - npoint, ' WDF arrays are invalid'
        endif
        if npoint eq 0 then return, ok
        if npoint lt nel then begin
          nel = npoint
          okin = okin(ok)
        endif
        ok = where(wdfnp(okin-1) ge 2, npoint)
        if npoint lt nel and not keyword_set(quiet) then begin
            if (size(valid(0)))(1) eq 7 then print, '% '+ strtrim(valid(0), 2) $
              + ': No valid data in ', strtrim(nel-npoint, 2), ' arrays'  $
                else print, '% INVALID WDF' $
              +  ': No valid data in ', strtrim(nel-npoint, 2), ' arrays'  
        endif
        if npoint eq 0 then return, ok
        if npoint lt nel then okin= okin(ok)
        return, okin
      endif
      if n_elements (minimumin) eq 0 then minimum = 0 else $
         minimum = (minimumin[0]-1)> 0
      if n_elements (maximumin) eq 0 then maximum = maxwdfarray-1 else $
         maximum = (maximumin[0] < maxwdfarray)-1
      sb = min([minimum, maximum], max = se)
      ok = where(wdfnp(sb:se) ge 2, npoint)+1 + sb
      if npoint eq 0 then begin
        ok = -1
        return, ok
      endif
      if n_elements(stringin) eq 0 then return, ok
;     str = strtrim(stringin(0), 2)
      str = stringin(0)
      if strlen(str) eq 0 then return, ok
      return, get_match(str, wdf=[sb, se]+1, casesen=casesen, nelem= npoint)
      end
