; ------------------------------------------------------------------------ 
; $Id: wufo.pro,v 1.5 2010/09/20 14:29:06 lpmix Exp $
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
;
;+
; NAME:  WUFO
; PURPOSE:		
;       IDL routine to write in a simple x,y file in UFO format #111.
;       Format is assumed to consist of a 5 line header, followed by
;       N lines of x,y data pairs, concluded with a -999,-999 data pair.
;
; CALLING SEQUENCE:  WUFO, X, Y, FILENAME [, FILENAME= fname] [, FORMAT = format]
; INPUTS:
;        X, Y      Data arrays to be written
;                  X and Y must have the same number of elements
;        FILENAME  Output file name
;                  If missing and fname is missing, then user may enter 
;                  a value for the output file naame.
; KEYWORDS:
;       NOTE: FILENAME and FORMAT may be shortened to FI and FO
;
;       FILENAME:       Ignored if the parameter FILENAME is present
;                       If FILENAME is undefined, then FNAME is used.
;                       If neither the parameter not keyword have a filename, 
;                         then the user is prompted for one.
;       FORMAT:         If specified, this will be used as the format string for the print
;                       Default: FORMAT = '(2g16.7)'
; EXAMPLE:
;       Write XVAL and YVAL UFO file, data_4.ufo
;         WUFO, XVAL, YVAL, 'data_4.ufo'
;       Write arrays XX and YY to file, zdata1256_09_09_09_wave.ufo
;         WUFO, XX, YY, 'zdata1256_09_09_09_wave.ufo'
;

; PROCEDURE:
;       The UFO format consists of 5 header lines:
;          '111 = FORMAT #'
;          '2 = # TABLE COLUMNS'
;          '1  1 = ITYPE, IORDER'
;          '0.00E+00  0.00E+00 = ABSTOL, RELTOL'
;          '             X             Y'
;       The header is followed by two columns of X, Y data ending with the last pair:
;         [-999, -999]
;
;
; MODIFICATION HISTORY:
;	Adapted WENelson's wwrite routine, done by Alex Filuk, 9/18/91.
;	NOTE: this routine is to be kept exactly the same as wufo.pro, which
;	is simply a shorter named version (for typing convenience)
;	A.B. Filuk (11/6/92) - removed initial whitespace at start of first
;		four lines of header, to make format COMPLETELY UFO 111 
;		compatible!
;	A.B. Filuk (1/13/93) - added free_lun to free logical unit!
;       L. P. Mix (9/9/2009) - Minor editorial changes for inclusion in PFIDL
;-

 PRO wufo, xvalues, yvalues, filename, filename=fname, format = fmt_str

on_error, 1			;return to caller on error
;
NPAR = n_params()
if npar eq 0 then begin
  print, '% FORMAT: WUFO, X, Y, FILENAME [, FILENAME= filename ] [, FORMAT = format]'
  return
endif

npts = n_elements(xvalues)
IF n_elements(yvalues) ne npts THEN BEGIN
  print,'% WUFO: X and Y arrays do not have same dimensions.'
  help, xvalues, yvalues
  return
ENDIF

if n_elements(filename) eq 0 then filename = (n_elements(fname) ne 0) ? fname[0] : ''
while strlen(filename) eq 0 do begin
  READ,' Enter filename (or Q to Quit): ',filename
  filename = strtrim(filename, 2)
  if strlowcase(strmid(filename, 0, 1)) eq 'q' then return
endwhile

if n_elements(fmt_str) eq 0 then fmt_str = '(2g16.7)'

if size(/type, fmt_str) ne 7 then begin
  print, '% WUFO: FORMAT string must be a valid string format'
  help, fmt_str
  return
endif

catch, error_status
if Error_status ne 0 then begin
  Print, '% WUFO: This is not a legal format'
  help, fmt_str
  print, 'Error message: ', !ERROR_STATE.MSG
  catch, /cancel
  return
endif

junk = string ([2.0, 5.0], format = fmt_str)
catch, /cancel

openw, unit, filename, /get_lun, error = ier
if ier ne 0 then begin
  print, '% WUFO: Unable to open file: ', strtrim(filename, 2)
  print, !ERROR_STATE.MSG 
  return
endif

num_hdr_lines = 5
header = strarr(num_hdr_lines)
header(0) = '111 = FORMAT #'
header(1) = '2 = # TABLE COLUMNS'
header(2) = '1  1 = ITYPE, IORDER'
header(3) = '0.00E+00  0.00E+00 = ABSTOL, RELTOL'
header(4) = '             X             Y'

; first write the header stuff
FOR i=0,num_hdr_lines-1 DO printf, unit, header(i)

; now write data pairs

FOR i=0,npts-1 DO printf, unit, FORMAT=fmt_str, xvalues[i], yvalues[i]

printf, unit, -999.0, -999.0, FORMAT= '(2f16.0)'
free_lun, unit
end
