; ------------------------------------------------------------------------
; $Id: x2lambda.pro,v 1.2 2011/03/15 19:52:53 lpmix Exp $
; ------------------------------------------------------------------------
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
;+
; NAME:
;       X2LAMBDA
;
; PURPOSE:
;       Convert a waveform array of Amplitude vs X (some abscissa) to 
;       a waveform array of Amplitude vs some other abscissa using a calibration
;       curve of NEW_ABSCISSA vs X.
;
; CATEGORY:
;       Analysis
;
; CALLING SEQUENCE:
;
;       X2LAMBDA, WDATA, WCAL, VALUE, X
;
; INPUTS:
;       WDATA:  WDF array for the waveform to be calibrated.  This may be a valid 
;               WDF array number or a unique search string.
;               If the keyword, SAVE, is not present, the data in WDATA will be 
;               modified to reflect the calibration.
;               For example, this can be a spectrometer output of amplitude vs X
;               The X values are relative to an arbitrary X0.
;       WCAL:   WDF array for the calibration calculation.  This may be a valid 
;               WDF array number or a unique search string.
;               For example, this can be a relation of wavelength or photon energy vs 
;               X position for a specific spectrometer geometry.
;       VALUE:  Some value which can be designated on WDATA array.
;               For example, if WDATA contains a peak vs position that is known 
;               to represent a certain wavelength or energy, then VALUE is that 
;               wavelength or energy.
;       X:      The position that Value occurs the WDATA waveform
;               If X is undefined or not a float, then the user will be asked to
;               select the X-position using a cursor. (SEE MANUAL below)
;               If eithr MANUAL or FWHM is set, then X is ignored on input.
;               In all cases the abscissa used with VALUE for the calibration will
;               be returned in X.
;
; KEYWORD PARAMETERS:
;       SAVE:   If SAVE contains a valid WDF array number, then calibrated waveform 
;               will be stored in SAVE.  If SAVE is a valid variable but not a valid
;               WDF array number, then SAVE is set to the lowest unused WDF array.
;       CALIBRATION: If a valid wdf array number then wavelength or energy vs x-position
;               will be stored.  The actual values used are stored including the
;               effects of the REVERSE keyword.  This curve may be used for other 
;               lineouts or to calibrate the X-axis of the original image.
;                 If CALIBRATION = 55, then 
;                   STRSCALE, STR, 55, 1
;                 will use the calibration curve to change the X-axis.
;               If CALIBRATION is not zero but not a valid WDF array, then it is set
;               to GET_MAXWDF()-1
;       MANUAL: If set, use ZOOMW to select the center of the feature corresponding 
;               to the value, VALUE.
;               CAUTION:  If FWHM is set MANUAL will not be checked.
;               DEFAULT:  If X is not valid and FWHM is not set, then MANUAL =1
;       FWHM:   If set, use FWHM function to determine the middle of a feature.
;               The mean of the 50% points is considered the middle of the feature.
;       REVERSE: If set, reverse the distances in the calibration curve. 
;               This might occur if WDATA were scanned in the reverse direction.
;
; OUTPUTS:
;       The calibrated waveform is returned in WDATA or in SAVE
;
; SIDE EFFECTS:
;	If SAVE is not provided, the modified waveform will be returned in WDATA
;       and WDATA will be lost.
;
; PROCEDURE:
;       1.  Determine the calibration value and X
;       2.  Find the calibration value on the calibration curve (WCAL) and the 
;           corresponding abscissa value.  
;       3.  Use linear interpolation on the offsets from the calibration point to 
;           determine the new abscissa values.  If REVERSE is set, change the sign 
;           of the offsets.
;       4.  Change the abscissa values of the data in WDATA or SAVE, if SAVE is present.
;
; EXAMPLE:
;       Assume that a spectral lineout is in WDF array 2 and you have a calculated
;       waveform of energy vs X in waveform 21.  Assume a calibration point at 3800 eV
;       and use the cursor to pick the X value.  Save the result in 71.
;         X2LAMBDA, 2, 21, 3800, SAVE = 71
;       Note:  Manual selection is the default.
;
;       Assume you observed 71 in the above example, and the peaks seemed to be going 
;       in the wrong energy direction.  Repeat the calculations assuming that the data
;       had been reversed.  Save the inverted waveform in WDF array 72.
;         X2LAMBDA, 2, 21, 3800, SAVE = 72, /REVERSE
;
;       Same as the previous example, but save the result in the lowest unused WDF array.
;         S=0 & X2LAMBDA, 2, 21, 3800, SAVE = S, /REVERSE
;       Note:  A variable must be provided for SAVE with and undefined value or an invalid
;              WDF array number to obtain the lowest unused WDF array number.
;
;       Assume you have a series of 6 lineouts that are from the same scan that you want 
;       to calibrate and they are in WDF arrays 2, 3, 4, 5, 6, 7.  Assume the calibration
;       is in WDF array 21 and you want to calibrate at 3.7 angs.  Save the results in WDF
;       arrays 12 to 17.
;         X= 0 & X2LAMBDA, 2, 21, 3.7, X, SAVE = 12
;         for i = 3, 7 do  X2LAMBDA, i, 21, 3.7, X, SAVE = 10+i
;
; MODIFICATION HISTORY:
; 	Written by:	L. P. Mix, Jr., December 20, 2010
;	July, 1994	
;-
;
 pro x2lambda, wdatain, wspecin, lambdain, xvalue, Manual = manual, FWHM = FWHM, $
      REVERSE = reverseit, save = saveit, calibration = calibration
;
; WDATA  - Amplitude vs X for a spectrometer scan
;          WDATA is converted to Amplitude vs Wavelength
; WSPEC  - Wavelength vs X for a spectrometer
; Lambda  - Wavelength for a single point 
; XValue - Returned value for the calibration wavelength
; MANUAL - If set, use the zoomw procedure to pick the center of the line
; FWHM   - If set, use the FWHM function to select the center of the line.
;          The mean of the 50% points is considered the center
;
on_error, 1
compile_opt strictarr
npar = n_params()
if npar eq 0 then begin
  print,'% Format: X2LAMBDA, WDATA, WSPEC, LAMBDA [, XVALUE]'
  print,'        [, MANUAL = manual]  [, FWHM = fwhm] [, /REVERSE]'
  print,'        [, SAVE = save]'
  return
endif
If npar lt 3 then begin
  print, '% X2LAMBDA:  The data waveform, calibration waveform and'
  print, '             the calibration value must be provided.'
  return
endif

if n_elements(WDATAin) eq 0 then begin
  print, '% X2LAMBDA:  The data waveform is undefined'
  return
endif else wdata = wdatain
if n_elements(WSPECin) eq 0 then begin
  print, '% X2LAMBDA:  The calibration waveform is undefined'
  return
endif else wspec = wspecin
if n_elements(lambdain) eq 0 then begin
  print, '% X2LAMBDA:  The calibration value is undefined'
  return
endif else lambda = lambdain[0]

wdata = wdvalid(wdata, /valid, np = np)
if np ne 1 then begin
  print, '% X2LAMBDA:  WDVALID returned '+strtrim(np, 2)+ ' values for WDATA'
  print, '             Provide a valid, unique WDF data descriptor'
  return
endif else wdata = wdata[0]

wspec = wdvalid(wspec, /valid, np = np)
if np ne 1 then begin
  print, '% X2LAMBDA:  WDVALID returned '+strtrim(np, 2)+ ' values for WSPEC'
  print, '             Provide a valid, unique WDF calibration descriptor'
  return
endif else wspec = wspec[0]
wdinfo, wspec, ymin = ymin, ymax = ymax,  xmin = xmin, xmax = xmax, comment = clab
if lambda lt ymin[0] or lambda gt ymax[0] then begin
  print, '% X2LAMBDA:  Invalid value for the calibration point'
  print, '             Value must be in the range: '+strtrim(ymin,2)+' to ' + $
         strtrim (ymax, 2)
  print, '             Value provide = '+strtrim (lambda, 2)
  print, '             for calibration curve: '+wdcomment(wspec)
  return
endif else xcalibrate = get_time(wspec, lambda)
if keyword_set(manual) or keyword_set(fwhm) then xvalue = 0
; see if X was passed as a parameter
if size(/type, xvalue) lt 4 then begin
  if keyword_set(fwhm) then begin
    zoomw, wdata, maxzoom = 400, xcursor = xval, ncursor = nc, Instruction = $
      'Select the line at '+strtrim(lambda, 2) +' Ang in the right window', $
      zoomrange = zoomrang
    junk = fwhm(wdata, left=zoomrang[0], right=zoomrang[1], first = f, second = s)
    if junk le 0 then begin
      print, '% X2Lambda: Bad choices for fwhm selection.'
      return
    endif else xvalue = (f+s)*0.5
  endif else begin
; We will use the default MANUAL to get XVALUE
    zoomw, wdata, maxzoom = 400, xcursor = xval, ncursor = nc, Instruction = $
     'Select the midpoint for the selected line at '+strtrim(lambda, 2) +' Ang'
    If nc gt 0 then xvalue = total(xval)/nc else begin
      print, '% X2Lambda: When manual is set, a point must be selected'
      return
    endelse
  endelse
endif else xvalue = xvalue[0]
if n_elements (saveit) eq 0 then saveit = 0 else saveit = saveit[0]
if saveit lt 1 or saveit gt get_maxwdf() then begin
  nel =0
  if arg_present(saveit) then saveit = wdffree(nel  = nel) 
  saveit = (nel gt 0) ? saveit[0] : wdata
endif
newx = getxf(wdata)
clab = wdcomment(wdata)+' - Calibrated X using '+clab
; wspec = Lambda vs distance on film
xs = xcalibrate[0] ; xcalibrate = get_time(wspec, lambda)
newx= keyword_set(reverseit) ? (xs +xvalue -newx) : (xs-xvalue + newx)
nnew = n_elements(newx)
ok = where((newx ge xmin) and (newx le xmax), nok)
if nok eq 0 then begin
  print, '% X2Lambda: This spectrometer does not appear to be compatible with this data.'
  return 
endif
newx =  newx[ok]
yy = (getyf(wdata))[ok]
w2i, xspec, yspec, wspec
if keyword_set(calibration) then begin
  if calibration lt 1 or calibration gt get_maxwdf() then calibration = get_maxwdf-1
  xcal = keyword_set(reverseit) ? (xs +xvalue -xspec) : (xs-xvalue + xspec)
  k = sort(xcal)
  i2w, xcal[k], yspec[k], calibration, /xydata, clab ='X Calibration Data - Value=' + $
     strtrim(lambda, 2)+';;Position='+strtrim(xvalue, 2)
endif
xx = interpolate(yspec, findex(xspec, newx))
i2w, xx, yy, saveit, /xydata, clabel = clab, xlabel = 'Wavelength', ylabel = 'Amplitude'

end
