; ------------------------------------------------------------------------
; $Id: xscale.pro,v 1.1 2011/03/15 20:07:44 lpmix Exp $
; ------------------------------------------------------------------------
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
;+
; NAME:
;       XSCALE
;
; PURPOSE:
;       Scale the X axis (abscissa) of a WDF array using a second WDF array.
;       The abscissa fo the first WDF array is compared to the abscissa of 
;       the second array and the corresponding ordinates (Y-values) of the 
;       second array replace the abscissa values.
;
; CATEGORY:
;       Analysis
;
; CALLING SEQUENCE:
;
;       XSCALE, WDF1, WDF_SCALE, WDF_SAVE
;
; INPUTS:
;       WDF1:      Original WDF array.  This may be a search string, a 
;                  WDF array number, or an array of WDF array numbers.
;       WDF_SCALE: WDF array with the scaling data.  Abscissa must be in 
;                  increasing or decreasing order.  This is not checked.
;       WDF_SAVE:  WDF array with the rescaled abscissa values.
;                  If WDF_SAVE is missing or invalid, the result will be
;                  saved in WDF1.  If WDF_SAVE is a scalar and WDF1 is a
;                  vector, successive arrays will be used.  
;	
; KEYWORD PARAMETERS:
;       DEFAULT:   If the abscissa values of WDF1 lie outside the domain of 
;                  WDF_SCALE, the abscissa will be set to this value.
;                  DEFAULT:  DEFAULT = ordinate closest to the bad data.
; OUTPUTS:
;       The WDF array with the rescaled abscissa is returned.
;
; RESTRICTIONS:
;       WDF_SCALE must have increasing or decreasing abscissa values.
;
; PROCEDURE:
;       FINDEX is used to determine the location of the WDF1 abscissa values
;       in the WDF_SCALE abscissa values.  INTERPOLATE is used to determine the
;       corresponding ordinates in WDF_SCALE.  The abscissa values in WDF1 are
;       changed to the ordinate values.
;
; EXAMPLE:
;	Rescale a spectrum with amplitude vs X-position using a calibration
;       waveform array, 23, which contains Energy vs position.  Assume the 
;       data is in WDF array 3.  Store the data in WDF array, 102.
;         XSCALE, 2, 23, 102
;
; MODIFICATION HISTORY:
;       Written by:	L. P. Mix, Jr., March 14, 2011
;-

PRO XSCALE, WDF1, WDF_SCALE, WDF_SAVE, default = default
npar = n_params()
if npar eq 0 then begin
  print,'% Format: XSCALE, WDF1, WDF_SCALE [, WDF_SAVE] [, DEFAULT = default]'
  return
endif

if npar lt 2 then begin
  print, '% XSCALE:  The WDF array and scaling array must be provided'
  return
endif

wdf = wdvalid(WDF1, /valid, np = np)
If np eq 0 then begin
  print, '% XSCALE:  A valid WDF array must be provided'
  help, WDF1
  return
endif

scal = wdvalid(WDF_SCALE, /valid, np = nscal)
if nscal ne  1 then begin
  print,  '% XSCALE: A single valid scaling array must be provided, np='+strtrim(nscal,2)
  help, wdf_scale
  return
endif
w2i, xscal, yscal, scal[0]
npscalm = n_elements(xscal)-1
xmin = min(xscal, max = xmax)
if n_elements(wdf_save) gt 0 then begin
  ok = where (wdf_save gt 0 and wdf_save lt get_maxwdf(), nok)
  if nok eq 0 then lcsave = 0 else begin
    lcsave = 1
    wdf_save = wdf_save[ok]
;   take the first value and add successive arrays if necessary.
    if nok lt nscal then wdf_save= wdf_save[0]+lindgen(nscal)
    if wdf_save[nscal-1] gt get_maxwdf() then set_maxwdfarray, wdf_save[nscal -1]+24
  endelse
endif else lcsave =0
if lcsave eq 0 then wdf_save = wdf

lcdefault =(n_elements(default) eq 0) ? 0 : 1

for ind= 0, np-1 do begin
w2i, xx, yy, wdf[ind]

if lcsave then xfr, wdf[ind], wdf_save[ind]
a = findex(xscal, xx) 
if lcdefault then ok = where (a lt 0 or a gt npscalm) $
  else a = (a > 0) < npscalm
xx = interpolate(yscal, a)
if lcdefault then xx[ok] = default[0]
put_wdfarray, [xx, yy], wdf_save[ind] , /xydata
endfor

END
