; ------------------------------------------------------------------------ 
; $Id: add_axis.pro,v 1.3 2007/06/06 20:09:33 lpmix Exp $
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
;
;+
; NAME:
;       ADD_AXIS
;
; PURPOSE:
;       Add an axis to a model object.
;
; CATEGORY:
;       Graphics
;
; CALLING SEQUENCE:
;       Model = ADD_AXIS( GEOMETRY [, XRANGE = XRANGE] [, YRANGE = YRANGE] 
;               [, ZRANGE = zrange] [, OFFSET = offset]
;               [, ALENGTH = alength] [, AALENGTH = aalenght]
;               [, ACHARACTER = acharacter] [, /VERBOSE] )
; OUTPUTS:
;       Model    - IDLgrModel containing GEOMETRY and the axis
;
; INPUTS:
;       GEOMETRY - IDLgrModel containing the solid or IDLgrPolygon
;                  (Users may suggest other solid types.)
;
; KEYWORD PARAMETERS:
;       XRANGE   - Two element vector defining the extent of the object in 
;                  the X direction.  If this is not input, the values will be
;                  returned.
;       YRANGE   - Two element vector defining the extent of the object in 
;                  the Y direction.  If this is not input, the values will be
;                  returned.
;       ZRANGE   - Two element vector defining the extent of the object in 
;                  the Z direction.  If this is not input, the values will be
;                  returned.
;    Verbose     - If set, data range and axis parameters will be provided.
;    OFFSET      - Axis offset from the data as a fraction of the data range.
;                  DEFAULT: AOPFFSET = 0.05 
;    ALENGTH     - Axis length as a fraction or the data range.
;                  If ALENGTH is a 3 element vector, different values will be 
;                  used for each vector.
;                  DEFAULT: ALENGTH = 0.5
;    AALENGTH    - Axis length in data coordinates
;                  If AALENGTH is a 3 element vector, these valuse will be used.
;                  If AALENGTH is a scalar, greater than zero, this value will
;                    be used for all three axes.
;                  Otherwise, the actual values will be returned.
;            NOTE: AALENGTH is checked before ALENGTH
;    ACHARACTER  - Character size as a function of the data range
;                  Default = max(.05 * Data Range)
;
; EXAMPLE:
;
;       Add a set of axes to the object: oModel
;          oModel = add_axis(oModel)
;
; MODIFICATION HISTORY:
;      Original version, L. P. Mix, December 2, 2003
;-
; 
;  get_poly_range
pro get_poly_range, ain, xr, yr, zr
   npoly =  n_elements(ain)
   if npoly eq 1 then begin
     ain[0] ->  getproperty, xrange = xr, yrange = yr, zrange = zr
     return
   endif
   npoly2= npoly+npoly
   xr = make_array(npoly*2)
   yr = make_array(npoly*2)
   zr = make_array(npoly*2)
   for i = 0, npoly-1 do begin
     ain[i] ->  getproperty, xrange = xxr, yrange = yyr, zrange = zzr
     xr[2*i:2*i+1] = xxr
     yr[2*i:2*i+1] = yyr
     zr[2*i:2*i+1] = zzr
   endfor
   xr = [min(xr, max = m), m]
   yr = [min(yr, max = m), m]
   zr = [min(zr, max = m), m]
end

pro get_poly_objects, moda, pobj, nobj, maxpobj
  nmods =  n_elements(moda)
 for i = 0, nmods-1 do begin
    p = moda[i] -> get(isa='IDLGRPOLYGON',/all)
    if size(p, /type) eq 11 then begin
      n = n_elements(p)
      fn = nobj +n
      if fn gt maxpobj then begin
        maxpobj = maxpobj+512  
        pobj = [pobj, objarr(512)]
      endif
      pobj[nobj :fn-1] = p
      nobj = fn
    endif
    modb = moda[i] -> get(isa='IDLGRMODEL', /all)
    ss = size(modb)
    if ss[ss[0]+1] eq 11 then get_poly_objects,modb, pobj, nobj, maxpobj
 endfor
 end

;
 function add_axis, ain, xrange = xrange, yrange = yrange, zrange= zrange, $
        verbose = verbose, OFFSET = offset, ALENGTH = alength, $
        AALENGTH = aalength, ACHARACTER = acharacter
; insert a common block
;
@qscom.dat
;
;  Common block
@qsstrcom.dat
;
common plotblk_defaults, nbuffer, plotblk_comment, plotblk_models
      on_error, 1
;

      if n_elements (structinit) eq 0 then qsstruct_init
      if n_elements (qsplotinit) eq 0 then qsinit

      offset_def = 0.05
      alength_def = 0.5
      acharacter_def = 0.05

      npar = n_params()
;
      if npar eq 0 then begin
        print,'% Format: Model = ADD_AXIS( GEOMETRY [, XRANGE = XRANGE]'
        print,'               [, YRANGE = YRANGE] [, ZRANGE = zrange]'
        print,'               [, OFFSET = offset] [, ALENGTH = alength]'
        print,'               [, AALENGTH = aalenght]  [, /VERBOSE] 
        print,'               [, ACHARACTER = acharacter] )
        return, 0
      endif
     if n_elements(ain) eq 0 then begin
       Print, '% ADD_AXIS: Input model must be defined
       return, 0
     endif
     if obj_isa(ain[0], 'IDLgrModel') then begin
       a = ain
       if n_elements(xrange) lt 2 or n_elements(yrange) lt 2 or $
         n_elements(zrange) lt 2 then begin
         npoly = 0
         maxpobj = 512L
         poly = objarr(maxpobj)
         get_poly_objects, a, poly, npoly, maxpobj
         poly = poly[0:npoly-1]
         get_poly_range, poly, xr, yr, zr
       endif
     endif else begin
       if obj_isa(ain[0], 'IDLgrPolygon') then begin
         if n_elements(xrange) lt 2 or n_elements(yrange) lt 2 or $
           n_elements(zrange) lt 2 then begin
           get_poly_range, ain, xr, yr, zr
           a = obj_new('idlgrmodel')
           a -> add, ain
         endif
       endif else begin
         Print, '% ADD_AXIS: This version only supports polygons and models.'
         return, ain
       endelse
     endelse

; Set defaults
     if n_elements(offset) eq 0 then offset= offset_def
     if n_elements(acharacter) eq 0 then acharacter = acharacter_def
     if n_elements(alength) eq 0 then alength= alength_def
     
     if n_elements (xrange) ge 2 then xr= [min(xrange, max = m), m] else $
       xrange = xr
     if n_elements (yrange) ge 2 then yr= [min(yrange, max = m), m] else $
       yrange = yr
     if n_elements (zrange) ge 2 then zr= [min(zrange, max = m), m] else $
       zrange = zr
     xrr = xr[1]-xr[0]
     yrr = yr[1]-yr[0]
     zrr = zr[1]-zr[0]
     dist = [xrr, yrr, zrr]
     if n_elements(aalength) gt 0 then begin
       if n_elements(aalength) eq 1 and aalength[0] gt 0 then $
         aalength = [1., 1., 1.]*aalength[0] else $
         if n_elements(aalength) ne 3 then aalength = -9999
     endif else aalength = -9999

     if aalength[0] eq -9999 then begin
       if n_elements(alength) lt 3 then aalength = dist*alength[0] $
         else aalength = alength[0:2] * dist
     endif
     chsiz = max(acharacter*dist)
     chsiz = [1,1]*chsiz
     orig = [xr[0], yr[0], zr[0]] - offset*dist
     endp = orig +aalength
     if keyword_set(verbose) then begin
       print,'X-Axis Length: ', strtrim(aalength[0], 2)
       print,'Y-Axis Length: ', strtrim(aalength[1], 2)
       print,'Z-Axis Length: ', strtrim(aalength[2], 2)
     endif


     ax = obj_new('IDLgrAxis', 0,range=[orig[0], endp[0]], color = $
       getcolor('red'), thick = 3, /notext, location = orig, ticklen =0, /exact)
     ay = obj_new('IDLgrAxis', 1,range=[orig[1], endp[1]], color = $
       getcolor('green'), thick = 3, /notext, location = orig, ticklen =0, /exact)
     az = obj_new('IDLgrAxis', 2,range=[orig[2], endp[2]], color = $
       getcolor('blue'), thick = 3, /notext, location = orig, ticklen =0, /exact)

     tx = obj_new('IDLgrtext', 'X', baseline =[1, 0, 0], align = 0.5, $
           location = [endp[0]+chsiz[0], orig[1], orig[2]], $
           char_dimensions=chsiz, updir=[0,1,0])
     ty = obj_new('IDLgrtext', 'Y', baseline =[0, 1, 0], align = 0.5, $
           location = [orig[0],  endp[1]+chsiz[0], orig[2]], $
           char_dimensions=chsiz, updir=[0,0,1])
     tz = obj_new('IDLgrtext', 'Z', baseline =[0, 0, 1], align = 0.5, $
           location = [orig[0], orig[1], endp[2]+chsiz[0]], $
           char_dimensions= chsiz, updir=[0,1,0])

     a -> add, ax
     a -> add, ay
     a -> add, az
     a -> add, tx
     a -> add, ty
     a -> add, tz

     return, a
     end

