; ------------------------------------------------------------------------ 
; $Id: addfr.pro,v 1.3 2007/06/06 20:09:33 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
 Function addfr, strin, framein, numberin , allframes = allframes
;
;+
; NAME:
;       ADDFR
;
; PURPOSE:
;       Add the frames in a CR-39 digitized image.
;       Input structure must be single block, 3 dimensions.
;       Output structure will be 2 dimensions.
;
; CATEGORY:
;       Math 
;
; CALLING SEQUENCE:
;       STR_Total = ADDFR(STR_IN, /ALLFRAMES)
;       STR_Total = ADDFR(STR_IN, FRAMES)
;       STR_Total = ADDFR(STR_IN, FRAMES [, NFRAMES])
;
; Where:
;       STR_Total - Composite image equal to the sum of the designated frames
;       STR_IN    - Structure or structure array number with the CR-39 images.
;                   This may be any 3 dimensional, single block structure
;       ALLFRAMES - If set, frames 1 to 19 will be added.  If the number of 
;                   frames (size of dimension 3) is not equal to 20, then
;                   the frames from 1 to (size of dimension 3)-1 will be added.
;                   If ALLFRAMES is set, all other parameters are ignored.
;       FRAMES    - Array of frames to be added. These values correspond to 
;                   indices in the third dimension.  For current data these 
;                   are in the range of 1 to 20.
;                   If FRAMES may be a single value.
;       NFRAMES   - Number of frames to be added.
;                   If FRAMES is an array, NFRAMES is ignored.
;                   Default value = 1.
; OUTPUTS:
;       Add structure arrays 2 and 3 and store the result in structure 8
;
; BACKGROUND:
;       CR-39 images are digitized according to the number of tracks with a
;       given size in a bin. Low resolution scans are 300 microns/bin.
;       High resolution scans are 150 microns/bin.
;
;  Frame Number       Track Area (Pixels = 0.299 um^2) / Effective Diam.
;         1                 1 to 2 pixels    /  0.617 to 0.872 microns
;         2                 3 to 5 pixels    /  1.069 to 1.380 microns
;         3                 6 to 10 pixels   /  1.511 to 1.951 microns
;         4                11 to 16 pixels   /  2.046 to 2.468 microns
;         5                17 to 23 pixels   /  2.544 to 2.959 microns
;         6                24 to 32 pixels   /  3.023 to 3.490 microns
;         7                33 to 42 pixels   /  3.544 to 3.999 microns
;         8                43 to 53 pixels   /  4.046 to 4.492 microns
;         9                54 to 65 pixels   /  4.534 to 4.974 microns
;        10                66 to 94 pixels   /  5.013 to 5.982 microns
;        11                95 to 128 pixels  /  6.014 to 6.981 microns
;        12               129 to 168 pixels  /  7.008 to 7.997 microns
;        13               169 to 212 pixels  /  8.021 to 8.984 microns
;        14               213 to 262 pixels  /  9.005 to 9.987 microns
;        15               263 to 378 pixels  / 10.006 to 11.996 microns
;        16               379 to 514 pixels  / 12.012 to 13.989 microns
;        17               515 to 672 pixels  / 14.002 to 15.995 microns
;        18               673 to 851 pixels  / 16.007 to 17.999 microns
;        19               852 to 9999 pixels / 18.010 to  112.832 microns
;        20               Saturation Frame - Total Number of Pixels with tracks
;                                            normalized to the number of pixels
;                                            in the bin times 32768.
;                    (a value of 16384 means half of the pixels contain tracks)
;
; EXAMPLES:
;       Plot the composite of all frames in sturcture 1 except the 
;       saturation frame.
;         plotstr, addfr(1, /al)
;
;       Plot the composite of sturcture 1 frames 3, 4, 5, and 6 
;         plotstr, addfr(1, 3, 4)
; or      plotstr, addfr(1, [3, 4, 5, 6])
;
;       Store the composite of sturcture 1 frames 3, 4, 5, and 6 in sturcture dog
;         dog = addfr(1, 3, 4) 
;
;       Store the composite of sturcture 1 frames 3, 4, 5, and 6 in 
;       structure array 2
;         i2s, addfr(1, 3, 4), 2
;
;       Plot frame 5 of structure 2
;         plotstr, addfr(2, 5)    ; nframes has a default value of 1
;    or   plotstr, addfr(2, 5, 1)
;    or   plotstr, 2, 3, 5, /index
;
; MODIFICATION HISTORY:
;      Original version, L. P. Mix, November 16, 1998
;-
;
; insert a common block
;
@qscom.dat
;  Common block
@qsstrcom.dat
;       return on an error
      on_error, 1
;
      if n_elements (structinit) eq 0 then qsstruct_init
      if n_elements (qsplotinit) eq 0 then qsinit
;
      nparams = n_params()
      if nparams eq 0 then begin
        print,'% Format: STR_Total = ADDFR(STR_IN, FRAMES)'
        print,' or       STR_Total = ADDFR(STR_IN, FRAMES, NFRAMES)'
        print,' or       STR_Total = ADDFR(STR_IN, /ALLFRAMES)'
        return, 0
      endif
;
      if n_elements(strin) eq 0 then begin
        print, '% ADDFR: Input structure must be defined'
        return, 0
      endif
;
      a = strin
      b = size(a)
      if b(b(0)+1) ne 8 then begin
        if a gt 0 and a le maxstructure then begin
          s2i, a, a
          b = size (a)
          if b(b(0)+1) ne 8 then begin
            print, '% ADDFR: ', $
   'First parameter must be a structure or a structure array number'
            return, 0
          endif
        endif  else begin
          print,  '% ADDFR: ',$
   'First parameter must be a structure or a valid structure array number'
          return, 0
        endelse
      endif
      if a.type  ne 1 then begin
        print,'% ADDFR: Only field type structures may be added.'
        return, 0
      endif
      if ((a.ndim  ne 3) or (a.block ne 1) or (a.nvect ne 1)) then begin
print,'% ADDFR: 3 Dimensional, single block scalar structures are required.'
        return, 0
      endif
      nz = a.size (2, 0)
      if nz lt 2 then begin
        print,'% ADDFR: Third dimension must have at least 2 elements'
        return, 0
      endif
      nok = nz-1
      if keyword_set(allframes) then frames = indgen(nz-1) else begin
        if n_elements (framein) eq 0 then begin
          print,'% ADDFR: Frames to add must be specified.
          return, 0
        endif
        frames = framein
        ok = where(frames gt 0 and frames le nz, nok)
        if nok eq 0 then begin
          print,'% ADDFR: Frames must be in the range 1 to ', strtrim(nz,1)
          return, 0
        endif
        frames = frames(ok)
        if nok eq 1 then begin
          if n_elements(numberin) eq 0 then number = 1 else $
            number = numberin(0) >1
          frames = frames(0) + indgen(number)
          ok =  where(frames gt 0 and frames le nz, nok)
          if nok lt number then frames = frames (ok)
        endif
        frames = frames-1
      endelse
      v = get_array(a, -1)
      v1 = reform(v(*, *, frames(0)))
      if nok gt 1 then for i = 1, nok-1 do v1 = v1+reform(v(*, *, frames(i)))
      clab = a.clab
      if strlen(strtrim(clab, 2)) eq 0 then clab = a.tlab
      clab = clab +';; Frames -'

      for i = 0, nok-1 do clab = clab+string(frames(i)+1, format='(i3)')
      locx = a.locx(0:1, *)
      locv = [0L, n_elements(v1)]
      slice = total(frames+1.)/nok
      ok = where (strpos(tag_names(a), 'DX') ge 0 , nok)
      if nok eq  0 then begin
        dx1 = 0.0
        dx2 = 0.0
      endif else begin
        dx1 = a.dx1
        dx2 = a.dx2
      endelse

      array = {type: 1L, ndim:2L, nvect: 1L, block :1L, two:3L, $
              slice: slice, size: a.size(0:1, 0), $
               range: reform(a.range(*, 0:1,*)), triangles:0L, $
              order: 0L, locx:locx, locv: locv, loctri: 0L, $
              x1:a.x1, x2:a.x2, x3:0L, x4:0L, $
              dx1 :dx1, dx2 :dx2, dx3 :0.0, dx4 :0.0, $
               v1:v1, v2:0L, v3:0L, v4:0L, $
              v5:0L, v6:0L, x1lab : a.x1lab, x2lab :a.x2lab, x3lab : '', $
              x4lab : '', v1lab : a.v1lab, v2lab : '', v3lab : '', $
              v4lab : '', v5lab :'',  v6lab : '', $
              file : a.file, spare : a.spare, tlab:a.tlab, clab:clab }
      
    
        a=0
      return, array
      end
