; ------------------------------------------------------------------------
; $Id: comstra.pro,v 1.1 2009/11/24 22:45:28 lpmix Exp $
; ------------------------------------------------------------------------
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
;+
; NAME:
;       COMSTRA (Compare Structure - Alternate)
;
; PURPOSE:
;       Compare two single block two dimensional structures and maximizing 
;       the correlation by shifting the second structure in X and Y.
;       A local optimum is found with an X, Y precision of 0.5 pixels, but 
;       larger errros can be observed if the image data does not have a buffer
;       strip around the main data.  The scaling of X and Y can also be varied
;       if the exact magnification is unknown.
;       SEE ALSO: COMSTR to vary position and image rotation
;
; CATEGORY:
;       Analysis
;
; CALLING SEQUENCE:
;
;       COMSTRA, STR1, STR2  [, STR3]
;
; INPUTS:
;       STR1:   Structure or structure array number of the first structure.
;               This structure must have uniform spacing in the X and Y directions.
;               DX and DY need not be equal.
;       STR2:   Structure or structure array number of the second structure.
;               This structure will be interpolated onto the same grid as STR1 but it 
;               should be uniform in the X and Y directions.
;
; OPTIONAL INPUTS:
;       STR3:   Structure or structure array number for the best fit second 
;               structure.  This will be of the same form as STR2.  
;               If STR3 is not a valid variable or undefined, the result is 
;               stored in GET_MAXSTR()-2
;	
; KEYWORD PARAMETERS:
;       XSHIFT: Value of the X shift which minimizes the difference between 
;               STR1 and STR2
;               CHASTR, STR2, AXDIM = XSHIFT
;       YSHIFT:	Value of the X shift which minimizes the difference between 
;               STR1 and STR2
;               CHASTR, STR2, AYDIM = YSHIFT
;       SCALE:  Value of the scale of STR2 that maximizes the overlap
;               (CHASTR, STR2, MXDIM = SCALE, MYDIM = SCALE)
;       AMPLITUDE: The value of the amplitude scaling of the second array so 
;               that the average value of the two structures over the region of
;               overlap is equal. 
;               (CHASTR, STR2, S1V = AMPLITUDE
;       MAGNIFICATION: If set, allow the magnification to vary between the two images.
;               Default:  MAGNIFICATION = 0  (Adjust only X-Y centering.)
;       SMOOTHE: Value of the width of the boxcar smoothing of both STR1 and
;               STR2.  Default: SMOOTH = 5
;               NOTE:  SMOOTHE has a minimum of 3
;               NOTE:  The command used is: SMOOTH (ARRAY, SMOOTHE, /EDGE_TRUNCATE
;       MEDIAN: If set, a median filter will be used rather the the average(SMOOTH).
;               If median is set, then if (MEDIAN EQ 1) then MEDIAN = 5
;       ITERMAX: Maximum number of iterations
;               Default: ITERMAX = 20
;       ERROR:  Minimum error for which iteration stops  
;               The error is the change in the scale which terminates the iteration
;               Default:  ERROR = 0.0002
;       VERBOSE: If set, print, intermediate results.
;
; OUTPUTS:
;       The second structure is shifted to maximize the correlation of the 
;       first and second structure.
;
; RESTRICTIONS:
;	Structures must be two dimensional, single block with uniform X and
;       Y grids 
;
; PROCEDURE:
;       The two structures are smoothed.
;       The initial scale value is used to put the second structure on the same
;         grid as the first
;       The center-of-mass for the two structures are calculated and these values
;         are used as an initial value for the shift to maximize the correlation.
;       The correlation defined by V1*V2 - Total(V1)*Total(V2)/NP is calculated.
;       The shift and scale are varied to obtain a local maximum for the correlation.
;
; EXAMPLE:
;       Find the X and Y displacements which maximize the correlations of structures
;       1 and 3.  Store the shifted version of structure 3 in structure 12.
;         COMSTRA, 1, 3, 12
;
;       Find the X and Y displacements which maximize the correlations of structures
;       10 and 11.  Store the shifted version of structure 11 in structure 12.  Use no
;       smoothing in the correlation.
;         COMSTRA, 10, 11, 12, SMOOTH=0
;
; MODIFICATION HISTORY:
; 	Written by:	L. P. Mix, Jr., October 12, 2009
;	July, 1994
;
;-
function  lp_corr, v1, dx, dy, newv, nx, ny, indx, indy, nxnew, nynew, amplitude
compile_opt  hidden
  on_error,1
    ist = (indx[0] + dx)
    js = (indy[0] + dy)
    if ist ge 0 then begin
      ileft = ist
      iright = 0
    endif else begin
      ileft = 0
      iright = abs (ist)
    endelse
    if js ge 0 then begin
      jleft = js
      jright = 0
    endif else begin
      jleft = 0
      jright = abs (js)
    endelse
    nnx  = min([nx- ileft, nxnew -iright])-1
    nny  = min([ny- jleft, nynew -jright])-1
    mat = newv[iright: iright+nnx, jright: jright+ nny]
    vmat = v1[ileft: ileft +nnx, jleft: jleft+nny]
    n = n_elements(mat)
    amp1 = total(vmat, /double)
    amp1_2 = total(vmat*vmat, /double)
    amp2 = total(mat, /double)
    amp2_2 = total(mat*mat, /double)
    amplitude = amp1/amp2
    return, (n*total(vmat*mat, /double) - amp1*amp2)/  $
       (sqrt(n*amp2_2 -amp2*amp2)*sqrt(n*amp1_2 -amp1*amp1))
    end

;
PRO comstra, str1, str2, sav, xshift = xshift, yshift = yshift, scale = scale, $
         smoothe = smoo, itermax = maxiter, medianfilter = medianf, error = error, $
         magnification = noscale, verbose = verbose, amplitude = amplitude
@qscom.dat
;  Common block
@qsstrcom.dat
;       return on an error
on_error, 1

;
if n_elements (structinit) eq 0 then qsstruct_init
if n_elements (qsplotinit) eq 0 then qsinit
;
nparams = n_params()
if nparams eq 0 then begin
  print,'% Format: COMSTRA, STR1, STR2,  [, STR3] [, SCALE = scale] [, /NOSCALE]'
  print,'          [, XSHIFT = xshift] [, YSHIFT = yshift] [, ERROR = error]'
  print,'          [, SMOOTH = smooth] [, MEDIANFILTER = medianfilter]'
  print,'          [, MAXITER = maxiter]'
  return
endif
time0 = systime(1)
;
if nparams lt 2 then begin
  print,'% COMSTRA: Procedure must have at least 2 parameters'
  return
endif
;
if n_elements(str1) eq 0 or  n_elements(str2) eq 0 then begin
  print, '% COMSTRA: Structure parameters must be defined'
  return
endif
;
array1 = get_valid_structure(str1, 'COMSTRA', lcstr=lcstr, ierror=ier)
if ier ne 0 then return

array2 = get_valid_structure(str2, 'COMSTRA', lcstr=lcstr, ierror=ier)
if ier ne 0 then return

if (array1.type ne 1 or array2.type ne 1) then begin
  print, '% COMSTRA:  Only Type 1 field structures can be compared'
  return
endif
if (array1.block ne 1) or (array2.block ne 1) then begin
  print, '% COMSTRA:  Only single block field structures can be compared'
  return
endif

if ((array1.ndim ne 2) and (array1.two ne 3)) then begin
  print, '% COMSTRA:  Only two dimensional field structures (1) can be compared'
  return
endif
if ((array2.ndim ne 2) and (array2.two ne 3)) then begin
  print, '% COMSTRA:  Only two dimensional field structures (2) can be compared'
  return
endif

dx = (array1.dx1)[0]
dy = (array1.dx2)[0]

if dx eq 0 then check_uniform, array1.x1, dx
if dy eq 0 then check_uniform, array1.x2, dy

if dx * dy eq 0 then begin
  print, '% COMSTRA:  STR1 must be a uniform image.'
  return
endif


dx2 = array2.dx1
dy2 = array2.dx2

if dx2 eq 0 then check_uniform, array2.x1, dx2
if dy2 eq 0 then check_uniform, array2.x2, dy2

if dx2 *dy2 eq 0 then begin
  print, '% COMSTRA:  STR2 must be a uniform image.'
  return
endif

dx1 = dx[0] & dy1 = dy[0]
dx2 = dx2[0] & dy2 = dy2[0]
strinfo, array1, xrange = rangx, yrange = rangy
strinfo, array2, xrange = xr2, yrange = yr2, clab = clab

x2range = xr2[1] -xr2[0]
y2range = yr2[1]- yr2[0]

if n_elements(maxiter) eq 0 then maxiter = 20
if n_elements(error) eq 0 then error = 0.0002
v1o = get_array(array1, -1)
v2o = get_array(array2, -1)
s1 = size(v1o, /dim)
s2 = size(v2o, /dim)

s1m = s1-1
s2m = s2-1


if keyword_set(medianf) then begin
  if medianf le 1 then medianf = 5
  med = medianf/2
  med = med + med + 1
  smoot = 0
endif else begin
  smoot = (n_elements(smoo) eq 0) ? 5 : (smoo[0] > 3)
  smo2 = smoot/2
  smoot = smo2+smo2+1
  med = 0
endelse

nx = s1[0]
ny = s1[1]
if smoot gt 1 then  v1 = smooth(v1o, smoot, /edge_truncate) else $
   if med gt 1 then v1 = median(v1o, med)
if smoot gt 1 then  v2 = smooth(v2o, smoot, /edge_truncate) else $
   if med gt 1 then v2 = median(v2o, med)

ioffx = (xr2[0] - rangx[0])/dx
ioffy = (yr2[0] - rangy[0])/dy
istartx = ceil(ioffx)
istarty = ceil(ioffy)

deltax = istartx - ioffx
deltay = istarty - ioffy
ioffx = deltax *dx1
ioffy = deltay *dy1

noscale = keyword_set(noscale) ? 0 : 1
quality = make_array(3, /double)
delta =0
change = 1
ntot = nx*ny
scale_delta = 0.02
midscale = 1.
iterscale = 0
error4 =4*error
ddx = [-1, 0, 1]
lcredo = (keyword_set(noscale)) ? 1 : -1
iterscalemax = maxiter
amplitude = make_array(3)
integ1 = total(v1, /double)
intave = integ1/ntot
py1 = total(total(v1, 1, /double)*dindgen(ny), /double)/integ1
px1 = total(total(v1, 2, /double)*dindgen(nx), /double)/integ1
titer = systime(1)
tot = make_array(9, /double)
lcinitialize = 0
while (iterscale lt iterscalemax) and ((change gt error) or (scale_delta gt error4)) do begin
  for iscale = 0, 2 do begin
    if (lcredo lt 0) || (lcredo eq iscale) then begin
      scale = midscale +ddx[iscale]*scale_delta
      newdx = dx2*scale
      newdy = dy2*scale
      nxnew = (floor((x2range-ioffx) /dx1) +1)
      nynew = (floor((y2range-ioffy) /dy1) +1)

      indx = istartx
      indy = istarty
      x2ind = ioffx/newdx + findgen(nxnew)*dx1/newdx
      y2ind = ioffy/newdy + findgen(nynew)*dy1/newdy
      newv = interpolate(v2, x2ind, y2ind, /grid)

      integ2 = total(newv, /double)
      if lcinitialize eq 0 then begin
;       calculate the center of mass for first iteration
        px2 = total(total(newv, 2, /double)*(istartx + dindgen(nxnew)), /double)/integ2
        py2 = total(total(newv, 1, /double)*(istarty + dindgen(nynew)), /double)/integ2
        dx = round(px1 - px2)
        dy = round(py1 - py2)
        lcinitialize +=1
      endif
      xs = (iterscale ge 2)? 16L : 64L
      ys = xs 
      iter = 0
      fracx = 0
      fracy = 0
      tot = make_array(9, /double)
      old = 0.0
      lcagain = 1
      while iter lt maxiter and lcagain gt 0 do  begin
        if (xs eq 1) && (ys eq 1) then lcagain = 0
        if tot[3] eq 0.0 then $
          tot[3] = lp_corr(v1, dx-xs, dy, newv, nx, ny, indx, indy, nxnew, nynew)
        if tot[4] eq 0.0 then $
          tot[4] = lp_corr(v1,dx, dy, newv, nx, ny, indx, indy, nxnew, nynew, ampl)
        amplitude[iscale] = ampl
        if tot[5] eq 0.0 then $
          tot[5] = lp_corr(v1,dx+xs, dy, newv, nx, ny, indx, indy, nxnew, nynew)
        if tot[1] eq 0.0 then $
          tot[1] = lp_corr(v1,dx,dy+ys,newv,nx,ny,indx,indy,nxnew,nynew)
        if tot[7] eq 0.0 then $
          tot[7] = lp_corr(v1,dx,dy-ys,newv,nx,ny,indx,indy,nxnew, nynew)
        maxtot = max(tot, tmax)
        if keyword_set(verbose) then $
          print, fix(iter), fix(tmax), fix(xs), fix(ys), fix(dx), fix(dy), $
            scale, tot[[1, 3, 4, 5, 7]], systime(1)-titer
        if tmax eq 4 then begin
          fracx = 0.5*(tot[3]- tot[5])/(tot[3]-tot[4]+tot[5]-tot[4])
          fracy = 0.5*(tot[7]- tot[1])/(tot[7]-tot[4]+tot[1]-tot[4])
          dx += round(fracx*xs)
          dy += round(fracy*ys)
          quality[iscale] = tot[4]
          tot[*]=0.d0
          if xs gt 1 then xs /= 2
          if ys gt 1 then ys /= 2
          if fracx lt .2 && xs gt 1 then xs /= 2
          if fracy lt .2 && ys gt 1 then ys /= 2
        endif else begin
          lcagain = 1
          dx += xs*ddx[tmax mod 3]
          dy -= ys*ddx[tmax/3]
          if (tmax eq 1) || (tmax eq 7) then begin
            tot[3] = 0.0
            tot[5] = 0.0
            if tmax eq 1 then begin
              tot[7] = tot[4]
              tot[4] = tot[1]
              tot[1] = 0.0
            endif else begin
              tot[1] = tot[4]
              tot[4] = tot[7]
              tot[7] = 0.0
            endelse
          endif else begin
            tot[1] = 0.0
            tot[7] = 0.0
            if tmax eq 5 then begin
              tot[3] = tot[4]
              tot[4] = tot[5]
              tot[5] = 0.0
            endif else begin
              tot[5] = tot[4]
              tot[4] = tot[3]
              tot[3] = 0.0
            endelse
          endelse
        endelse
        iter +=1
      endwhile
if keyword_set(verbose) then print, 'Number of iterations: '+strtrim(iter, 2)
      if iter eq maxiter then begin
        Print, '% COMSTRA:  Comparison did not converge after ' + $
          strtrim(maxiter, 2)+' iterations'
        print, '  XSHIFT = '+ strtrim(dx*dx1, 2)
        print, '  YSHIFT = '+ strtrim(dy*dy1, 2)
        print, '  SCALE = '+ strtrim(midscale, 2)
        return
      endif
      if keyword_set(noscale) then goto, FINISHED
      if keyword_set(verbose) then print, 'iter ',iscale, iter, systime(1)- time0
    endif
  endfor
  if keyword_set(verbose) then print, 'Quality: ', quality
  mm = max(quality, qmax)
  if qmax eq 1 then begin
    delta = 0.5*(quality[0] - quality[2])/(quality[0]-quality[1]+quality[2]-quality[1])
    change = delta*scale_delta
    midscale += change
    scale_delta = (abs(delta) lt 0.1875) ? scale_delta*0.5 : scale_delta*0.25
    lcredo = -1
  endif else begin
    if qmax eq 0 then begin
      quality[1:2] = quality[0:1]
      lcredo = 0
    endif else begin
      quality [0: 1] = quality[1:2]
      lcredo = 2
    endelse
    change = 1
    midscale = midscale + ddx[qmax]*scale_delta
  endelse
  iterscale += 1
endwhile
if iterscale eq iterscalemax then begin
  Print, '% COMSTRA:  Comparison did not converge after ' + $
   strtrim(iterscalemax, 2) + ' iterations'
  print, '  XSHIFT = '+ strtrim(dx*dx1, 2)
  print, '  YSHIFT = '+ strtrim(dy*dy1, 2)
  print, '  SCALE = '+ strtrim(midscale, 2)
  return
endif
FINISHED:
amplitude = amplitude[1]
if keyword_set(verbose) then begin
  print, '  XSHIFT = '+ strtrim(dx*dx1, 2)
  print, '  YSHIFT = '+ strtrim(dy*dy1, 2)
  print, '  SCALE = '+ strtrim(midscale, 2)
  print, '  AMPLITUDE = '+ strtrim(amplitude, 2)
endif
if n_elements(sav) eq 0 then sav = get_maxstr()-2 else $
  if ((sav lt 1) or (sav gt get_maxstr())) then sav =  get_maxstr()-2
xfrstr, str2, sav
xshift = dx*dx1
yshift = dy*dy1

i = strpos(clab, ' - ')
insrt = (i lt 0) ? ' - ' : ';;'
clab = clab+insrt+ 'XShift='+strtrim(xshift, 2)+';;YShift='+strtrim(yshift, 2)+ $
    ';;SCALE='+strtrim(midscale, 2)+';;AMPLITUDE='+strtrim(amplitude, 2)
chastr, sav, ax = xshift, ay = yshift, clab = clab, mx = scale, my = scale, s1v = amplitude
v1 = 0 & v2 = 0 & v1s = 0 & v2s = 0
array1 = 0 & array2 = 0
v2save = 0 
end
