; ------------------------------------------------------------------------ 
; $Id: get_array.pro,v 1.7 2007/06/06 20:09:34 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
;+
;   this function returns an array from a structure
;   Format: get_array(structure, space [, block] [, label= lab] [, /ALLDATA ] )
;   where: 
;          structure - IDL structure name or structure array number
;                      type must be 1, Field
;                                   2, Particle
;                                   3, Grid
;                                   8, Finite Element Time Sliced
;                                   9, Finite Element
;    Note:  Type 8 and 9 data requires that the ALLDATA keyword is set
;           If not set, it will be set silently.
;                                   
;          space - if not zero indicates spatial dimension desired, if 
;                  negative indicates field data and attribute data.
;          block - indicates block - not used for particle data
;          label - attribute label or block label associated with this data
;          alldata - If set all blocks will be returned in a linear array.
;                    If ALLDATA is set, then label will be for block 1.
;                    If ALLDATA is undefined, and the number of blocks is 
;                      greater one, and BLOCK is undefined, then ALLDATA =1
;                    ALLDATA is set for type 8 and type 9 data.
;                    The array for type 9 data is [num_nodes, num_times]
;    example:
;     make a contour plot of field data in structure, fld1
;
;     contour, get_array(fld1,-1), get_array(fld1, 1, l=xl), $
;       get_array(fld1, 2, l= yl), xtit= xl, ytit= yl
;
;     note if fld1 is a 3 dimensional array then the above becomes
;     k = xx     (define k to be the appropriate location)
;     a = get_array(fld1,-1)
;     contour , a(*, k, *) , get_array(fld1, 1, la=xlab), $
;       get_array(fld1, 3,la=ylab), xtit = xlab, ytit = ylab
;-
;
 Function get_array_doit, array, space, block, lab, alldata
      compile_opt  hidden
      on_error, 1
      valid_types =[1, 2, 3, 8, 9]
      ok = where (array.type eq valid_types, nok)
      if nok eq 0 then begin
        print, '% GET_ARRAY: Structure type ', strtrim(array.type, 2), $
          ' is not supported'
        print, '             Supported types:', valid_types
        return, 0.0
      endif
      vector = 0
;     Check for type 8/9 data
;  
      if array.type eq 8 || array.type eq 9 then begin
        alldata = 1
        block = 1
      endif
      if space lt 0 then vector = -space
      if ok[0] ne 1 then begin
;       field or grid dataset type
        nblo = array.block
        if block gt nblo then begin
          print,'% GET_ARRAY: Cannot read block', block, '-- Only', nblo,' blocks'
          return,0.0
        endif
        if space gt 0 then begin
          if space gt array.ndim then begin
            print, '% GET_ARRAY: Structure has only',array.ndim,' spatial dimensions'
            print, '             Array of dimension ',space,' is not possible'
            return,0.0
          endif
          if space eq 1 then begin
            lab = (array.x1lab)(block-1)
            a = array.x1
          endif else if space eq 2 then begin
            lab = (array.x2lab)(block-1)
            a = array.x2
          endif else if space eq 3 then begin
            lab = (array.x3lab)(block-1)
            a = array.x3
          endif else if space eq 4 then begin
            lab = (array.x4lab)(block-1)
            a = array.x4
          endif else if space eq 5 then begin
            lab = (array.x5lab)(block-1)
            a = array.x5
          endif else if space eq 6 then begin
            lab = (array.x6lab)(block-1)
            a = array.x6
          endif
          if ((keyword_set(alldata)) or (nblo eq 1)) then return, a
          a = a(array.locx(space-1, block-1): array.locx(space-1, block)-1)
          return , a
        endif else begin
          if array.type eq 3 then begin
            print,'% GET_ARRAY: Cannot return a vector with GRID data'
            return, 0.0
          endif
          if array.nvect eq 0 then begin
            print, '% GET_ARRAY: No vectors available in this structure'
            return, 0.0
          endif
          if vector lt 1 or vector gt array.nvect then begin
            print, '% GET_ARRAY: Vector dimension must be less than', array.nvect
            return, 0.0
          endif
          if vector eq 1 then begin
            lab = (array.v1lab)(block-1)
            a = array.v1
          endif else if vector eq 2 then begin
            lab = (array.v2lab)(block-1)
            a = array.v2
          endif else if vector  eq 3 then begin
            lab = (array.v3lab)(block-1)
            a = array.v3
          endif else if vector eq 4 then begin
            lab = (array.v4lab)(block-1)
            a = array.v4
          endif else if vector  eq 5 then begin
            lab = (array.v5lab)(block-1)
            a = array.v5
          endif else if vector eq 6 then begin
            lab = (array.v6lab)(block-1)
            a = array.v6
          endif
          if keyword_set(alldata) then return, a
          if nblo gt 1 then $
            a = a(array.locv(block-1): array.locv(block)-1)
          n = array.size(*, block-1)
          return, reform(a, n, /overwrite)
        endelse
      endif else if array.type eq 2 then begin
;       particle data
        if space gt 0 then begin
          if space gt array.ndim then begin
            print, '% GET_ARRAY: Structure has only',array.ndim,' spatial dimensions'
            print, '             Array of dimension ',space,' is not possible'
            return,0.0
          endif
          if space eq 1 then begin
            lab = array.x1lab
            a = array.x1
          endif else if space eq 2 then begin
            lab = array.x2lab
            a = array.x2
          endif else if space eq 3 then begin
            lab = array.x3lab
            a = array.x3
          endif else if space eq 4 then begin
            lab = array.x4lab
            a = array.x4
          endif
          return , a
        endif else begin
          if array.natt eq 0 then begin
            print, '% GET_ARRAY: No attributes available in this structure'
            return, 0.0
          endif
          if vector lt 1 or vector gt array.natt then begin
            print, '% GET_ARRAY: Attribute dimension must be less than', array.natt
            return, 0.0
          endif
          if vector eq 1 then begin
            lab = array.v1lab
            a = array.v1
          endif else if vector eq 2 then begin
            lab = array.v2lab
            a = array.v2
          endif else if vector eq 3 then begin
            lab = array.v3lab
            a = array.v3
          endif else if vector eq 4 then begin
            lab = array.v4lab
            a = array.v4
          endif else if vector eq 5 then begin
            lab = array.v5lab
            a = array.v5
          endif else if vector eq 6 then begin
            lab = array.v6lab
            a = array.v6
          endif else if vector eq 7 then begin
            lab = (array.v7lab)
            a = array.v7
          endif else if vector eq 8 then begin
            lab = (array.v8lab)
            a = array.v8
          endif
          return, a
        endelse
      endif
      end
 Function get_array, arrayin, space, blockin, label = lab, alldata = alldata
      on_error, 1
; common block
@qsstrcom.dat
    if n_params() lt 2 then begin
      print, '% GET_ARRAY: Structure and space variable must be defined'
      return,0.0
    endif
    if n_elements(arrayin) eq 0 then begin
      print, '% GET_ARRAY: Array structure must be defined'
      return,0.0
    endif
    if n_elements (space ) eq 0 then begin
      print, '% GET_ARRAY: Array component must be defined'
      return,0.0
    endif
    lab = ''
    space = round(space)
    array = arrayin
    b = size(arrayin)

    if b(b(0)+1) ne 8 then begin
      if arrayin gt 0 and arrayin le maxstructure then begin
        s2i, array, arrayin
        b = size (array)
        if b(b(0)+1) ne 8 then begin
          print, '% GET_ARRAY: First parameter must be a structure'
          print, '             or a structure array number'
          return, 0.0
        endif
      endif  else begin
        print, '% GET_ARRAY: First parameter must be a structure or'
        print, '             a structure array number'
        return, 0.0
      endelse
    endif
    if n_elements (blockin) eq 0 then begin
      block = 1
      if array.block gt 1 && n_elements(alldata) eq 0 then alldata = 1
    endif else block = blockin[0]
    if keyword_set(alldata) then block =1
    a = get_array_doit(array, space, block, lab, alldata)
    array = 0
    return, a
    end
