; ------------------------------------------------------------------------ 
; $Id: get_str12_value.pro,v 1.1 2010/07/19 14:22:12 lpmix Exp $
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
;+
; NAME:
;       GET_STR12_VALUE
;
; PURPOSE:
;       Get field values at points 
;
; CATEGORY:
;       Input/Output
; CALLING SEQUENCE:
;       GET_STR12_VALUE, A [, /SELECT ] [, /Node] [, MISSING = missing]
;                  , XARRAY = xarray [, YARRAY = yarray] [, ZARRAY = zarray]
;                  , FARRAY = farray [, NFARRAY = nfarray]
; 
; INPUTS:
;       A     IDL field structure (a.type = 12) or structure array number
;       
; KEYWORD PARAMETERS:
;
;       XARRAY, YARRAY, ZARRAY - Coordinates for FARRAY
;                  If SELECT is set, these must be simple variables.
;                  Alternatively XARRAY may be an n x 3 array of points
;       FARRAY   - Values of the field at the coordinates [xarray, yarray, zarray]
;                  Missing values are set to 1e30 > 10* MAX(A.V1) or the MISSING keyword.
;       NFARRAY  - Elements are 0 or 1 depending on whether FARRAY has valid data.
;                  If keyword, MISSING, is present, NFARRAY is 1.
;                  If keyword, MISSING, is not present then NFARRAY has the following values:
;                  NF = 0 if the value is bad
;                  NF = 1 if the value is within the convex hull
;       MISSING  - Value for data that is missing, ie the points are not within the 
;                  convex hull.  Default is MISSING = 1e30 > 10* MAX(A.V1)
;       SELECT   - If set, select values of XARRAY, YARRAY, ZARRAY, FARRAY that are
;                  within the convex hull of the data
;       NODE     - If set, take the nearest node value.
;                  If node gt 0 then distance is relative distance.
;                    The range in each dimension is used to normalize the distance from 
;                    each point to the nodes.
;                  If node lt 0 then distance is absolute distance.
;                  SELECT is ignored and every point returns the nearest node value whether
;                  the point is inside or outside the actual data, i.e., NFARRAY = 1
; OUTPUTS:
;       FARRAY and NFARRAY are returned for the input values of XARRAY, YARRAY, ZARRAY
;       If SELECT is set, invalid points will be deleted.  For this reason, if SELECT
;         is set then XARRAY, YARRAY, and ZARRAY must be simple variables.
;       If NODE is not zero, the value of FARRAY will be determined from the nearest
;         node value without interpolation.
;       If NODE is missing or zero, the value of FARRAY is determined from the routine
;         QGRID3.
; EXAMPLE:
;       Determine the values of the field in str 4 at the points (x, y) = 
;       (3, 4), (4, 5), (5, 6) for a z value = .35.  Store the values in val.
;         get_str12_value, 4,  xar= [3, 4, 5], yar = [4,5,6], zarr=[1,1,1]*0.35, far = val
;  <or>
;         xarray =reform([3, 4, .35, 4, 5, .35, 5, 6, .35], 3, 3)
;         get_str12_value, 4,  xarray= xarray, far = val
;
; MODIFICATION HISTORY:
;      Original version, L. P. Mix, July 8, 2010
;-
;
 Pro get_str12_value, ain, zarray = zarray, select = select, node = node, $
   missing = missin, xarray= xarrayin, yarray= yarray, farray= farray, nfarray = nfarray
;
;  Common block
@qscom.dat
@qsstrcom.dat
      on_error, 1
;
      if n_elements (structinit) eq 0 then qsstruct_init
      if n_elements (qsplotinit) eq 0 then qsinit
      npar = n_params()
      if npar eq 0 then begin
        print,'% Format: GET_STR12_VALUE, A [, /SELECT]'
        print,'       , XARRAY = xarray , YARRAY = yarray, ZARRAY = zarray'
        print,'       , FARRAY = farray [, NFARRAY = nfarray]'
        return
      endif

;
;     now get the data  -- check that we have a structure of the proper type
;
      if n_elements(ain) eq 0 then begin
        print, '% GET_STR12_VALUE: Structure must be defined'
        return
      endif
; change variables to insure we don't corrupt input data
      a = get_valid_structure(ain,'GET_STR12_VALUE', lcstr=lcstr, ierror = ier)
      if ier ne 0 then return
;
      if a.type ne 12 then begin
        print, '% GET_STR12_VALUE: Only Field type 12 structures may be used to get data values'
        a = 0
        return
      endif
;
; get a working array number
;
;
      b = size(a, /type)
      if b ne 8 then begin
        print,'% GET_STR12_VALUE: First parameter must be a structure or a valid QS array number'
        return
      endif
      if not arg_present(farray) then begin
        Print, '% GET_STR12_VALUE: FARRAY must be a simple variable to return values'
        return
      endif
;     slice makes a one attribute structure
      nelx = n_elements(xarrayin)
      nely = n_elements(yarray)
      nelz = n_elements(zarray)
;
      lcxonly = 0
      if nelx eq 0 or nely eq 0 or nelz eq 0 then begin
        if nelx gt 0 then begin
          if (nelx mod 3) eq 0 then begin
            nely = nelx/3
            nelz = nely
            xarray = reform (xarrayin, 3, nely)
            yarray = reform(xarray[ 1, *])
            zarray = reform(xarray[ 2, *])
            xarray = reform(xarray[ 0, *])
            nelx = nely
            lcxonly =1
          endif else begin 
            print, '% GET_STR12_VALUE: X, Y, and Z array values must be defined'
            help, xarrayin, yarray, zarray
            a=0
            return
          endelse
        endif else begin
            print, '% GET_STR12_VALUE: X, Y, and Z array values must be defined'
            help, xarrayin, yarray, zarray
            a=0
            return
        endelse 
      endif else xarray = xarrayin
      if (nelx ne nely) or (nelx ne nelz)then begin
 print, '% GET_STR12_VALUE: X, Y, and Z arrays must have the same number of values'
        return
      endif
      nfarray = make_array (nelx, value =1)
      farray = fltarr(nelx)
      x = a.x1
      y = a.x2
      z = a.x3
      f = a.v1
      dx = a.range[1, 0, 0] - a.range[0, 0, 0]
      dy = a.range[1, 1, 0] - a.range[0, 1, 0]
      dz = a.range[1, 2, 0] - a.range[0, 2, 0]
      if keyword_set(node) then begin
        if node gt 0 then begin
;         Relative distance compared to the entire range
          dx = dx*dx
          dy = dy*dy
          dz = dz*dz
          for i = 0, nelx-1 do begin
            d = (x-xarray[i])^2 /dx+ (y-yarray[i])^2 /dy+(z-zarray[i])^2 /dz 
            junk = min(d, j)
            farray[i] = f[j]
          endfor
        endif else begin
;        absolute distance
          for i = 0, nelx-1 do begin
            d = (x-xarray[i])^2 + (y-yarray[i])^2 +(z-zarray[i])^2
            junk = min(d, j)
            farray[i] = f[j]
          endfor
        endelse
        x = 0 & y= 0 & z = 0 & f = 0 & a = 0
        return
      endif
      delta = [dx, dy, dz]*1e-4
      miss= (n_elements(missin) gt 0) ? missin[0] : (1e30 > 10*max(abs(f)))
      tri = a.triangles
      for i = 0, nelx-1 do begin
        result = qgrid3(x, y, z, f, tri, delta = delta, missing = miss, $
          start= [xarray[i], yarray[i], zarray[i]], dimension = [2,2,2])
        farray[i] = result[0,0,0]
      endfor
      if n_elements(missin) eq 0 then begin
        bad = miss*0.5
        ok = where (farray gt bad, nok)
        if nok gt 0 then nfarray[ok] = 0
      endif
      if keyword_set (select) then begin
        if (arg_present (xarrayin) and arg_present(yarray) and arg_present(zarray)) $
          or arg_present(xarrayin) and lcxonly  then begin
          ok = where (nfarray gt 0, nok)
          if nok eq 0 then begin
            print, '% GET_STR12_VALUE: No valid data'
            print, '                 All variables set to zero.'
            xarrayin =0 & yarray =0 & zarray =0 & farray =0 
          endif else begin
            if nok lt nelx then begin 
              if lcxonly then begin
                xarrayin = make_array(3, nok)
                xarrayin[0, *] = xarray[ok]
                xarrayin[1, *] = yarray[ok]
                xarrayin[2, *] = zarray[ok]
                yarray = 0 & zarray = 0
              endif else begin
                xarrayin = xarray[ok]
                yarray = yarray[ok]
                zarray = zarray[ok]
              endelse
              farray = farray[ok]
            endif
          endelse
        endif else begin
          print, '% GET_STR12_VALUE: Unable to filter data'
          print, '   XArray, YArray, ZArray must be simple variables if SELECT is set.'
        endelse
      endif
      x = 0 & y= 0 & z = 0 & tri = 0 & f = 0 & a = 0
      end
