; ------------------------------------------------------------------------
; $Id: get_str_dx.pro,v 1.4 2011/10/04 21:19:29 lpmix Exp $
; ------------------------------------------------------------------------
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
;+
; NAME:
;       GET_STR_DX
;
; PURPOSE:
;       Return the spacing of points a single block TYPE 1 structure
;	Returns 0 if point spacing of the structure is nonuniform
;       The structure array is not changed
;
; CATEGORY:
;       Analysis
;       Utility
;
; CALLING SEQUENCE:
; 
;       Delta =GET_STR_DX(STRarray [, ERROR = error] [, DXRANGE = dxrange] [, /MODIFY] )
;
;       DELTA:    The calculated spacing of the spatial coordinates for STRarray.
;                 The dimensions are: Delta[NDIM] where ndim is the number of dimensions
;                 If DELTA is zero, spacing is not uniform.
;                 If DELTA is negative, abscissa is decreasing.
;                 If DELTA is positive, abscissa is increasing.
;                 If MODIFY is set, the order will be changed and DELTA is positive.
;
; INPUTS:
;       STRarray: Structure array number, structure, or unique search string for 
;                 a single block type 1 structure.
;       Note:  Some sructure arrays are stored as nonuniform data, but have uniform 
;              spacing between spatial values.
;       Note:  The spacing is considered uniform is the variation in the 
;              spacing is less than an error parameter.
;              (See Procedure and keyword ERROR)
;
; KEYWORD PARAMETERS:
;       ERROR:    Error tolerance for determining if the spatial coordinate is uniform
;                 ERROR is expressed as a fraction of the range of the X data.
;                 If specified ERROR = ERROR > ERROR_MIN
;                 See PROCEDURE below.
;                 Default: ERROR = (3.05176E-5) > ERROR_MIN
;                 Note:  ERROR_MIN is defined as follows for RANGE = MAX(X)-MIN(X):
;                        ERROR_MIN =  (MACHAR()).EPS * MAX([RANGE, MAX(ABS(X))]) / RANGE 
;                        A warning will be printed if ERROR < ERROR_MIN .
;                 Note:  (MACHAR()).EPS is about 1.2e-7 (=1.0/2L^23) for many computers
;                 Note:  PFF files have an accuracy of about 7.6e-6
;                        PFF files have the least significant bit of about 1.5e-5
;                        Thus the default error is twice the least significant bit.
;                 Note:  If the structure already indicates uniform spacing, no 
;                        calculations are performed.  The tag, DXn, indicates spacing. 
;       DXRANGE:  Range of DX-Spacing - a range of zero indicates a single point
;                 The size of DXRANGE = [2, ndim] where ndim is number of dimensions.
;                 The minimum and maximum spacing for each dimension are provided.
;                 For uniform data, these numbers will be equal.
;       MODIFY:   If set, change the array to a uniform array if the X-spacing 
;                 is uniform.  Default is to set DXn to the calculated DELTA value
;                 DEFAULT:  MODIFY =1
;
; OUTPUTS:
;       If the coordinate values for the structure are uniform, the spacing for each
;       dimension is returned.  If the coordinates are not uniform in a particular
;       dimension, then a zero is returned for nonuniform dimensions.  The coordinate 
;       spacing is returned by the function in an array of NDIM values, where NDIM is 
;       the number of spatial dimensions for the structure.
;
; SIDE EFFECTS:
;       DX1, DX2, and DX3 are modified if the coordinates are uniform and MODIFY is 
;       undefined or not zero.
;
; PROCEDURE:
;       If the structure array is stored as uniform data, the spacing is returned.
;       (Uniformity is determined by the presense of a nonzero DXn.)
;       If the structure array is stored as X-Y data, then the spacing is calculated.
;     The procedure for coordinates with a zero value of DXn is:
;       Calculate the maximum and minimum spacing of the coordinate.
;         NELEMENTS = number of points
;         DX is the spacing = X[1, ndim-1] - X[0:ndim-2]
;         DXRange [*, n] = [min(DX), max(DX)]
;       Calculate the range of X
;         RANGE = max(X) - min(X)
;       If (DXRange[1] -DXRange[0]) / RANGE < ERROR, then the waveform is uniform.
;         DELTA = RANGE / (NELEMENTS -1) 
;         DELTA is returned
;
; EXAMPLE:
;       Determine the X-spacing for structure array 4 and store it in DX4
;         DX4 = GET_STR_DX(4)
;
;       Determine if structure array 6 is uniform.
;         if GET_STR_DX(6) gt 0 then print, 'Structure array 6 is uniform'
;
; MODIFICATION HISTORY:
; 	Written by:	L. P. Mix, Jr., September 24, 2010
;
;-

Function GET_STR_DX, STRarray,  ERROR = error, DXRANGE = dxrange, MODIFY = modify
on_error, 1

; Insert common blocks
;
@qscom.dat
@qsstrcom.dat
;
if n_elements (structinit) eq 0 then qsstruct_init
if n_elements (qsplotinit) eq 0 then qsinit

npar = n_params()
delta = 0
dxrange =0
if npar eq 0 then begin 
  print, '% Format: Delta = GET_STR_DX(STRarray [, ERROR = error] '
  print, '                    [, DXRANGE = dxrange] [, MODIFY = modify])'
  return, delta
endif

if n_elements(strarray) eq 0 then begin
  print, '% GET_STR_DX:  Structure array must be defined'
  return, delta
endif
a = get_valid_str (strarray, 'GET_STR_DX', lcstr=lcstr, ierror=ier)
if ier ne 0 then return, delta
if (a.type ne 1) then begin
  print, '% GET_STR_DX:  Type 1 field structures are required'
  return, delta
endif

if a.block ne 1 then begin
  print, '% GET_STR_DX:  Single block field structures are required'
  print, '               Number of blocks: '+strtrim (a.block, 2)
  return, delta
endif
tags = tag_names(a)
ok =  get_match('DX', tags, nelem=nel)
ndim = a.ndim
if nel lt ndim then return, delta

if n_elements(modify) eq 0 then modify = 1

dxrange = make_array(2, ndim, value = -1.0)
delta = make_array(ndim, value = -1.0)
lcrev = make_array(ndim, value = 1)
err0 = ( n_elements (error) eq 0) ? 3.05176E-5 : error[0]
errmach =  (machar()).eps

for idim = 1, ndim do begin
  if idim eq 1 then dx = a.dx1[0] else $
    if idim eq 2 then dx = a.dx2[0] else $
    if idim eq 3 then dx = a.dx3[0] else dx = a.dx4[0]
  if dx gt 0 then delta[idim -1] = dx else begin
    if idim eq 1 then x = a.x1 else $
      if idim eq 2 then x = a.x2 else $
      if idim eq 3 then x = a.x3 else x = a.x4
    np = n_elements(x)
    if np lt 3 then begin
      if np eq 1 then delta[idim-1] = 0 else $
        delta[idim-1] = x[1] - x[0]
    endif
  endelse
  if delta[idim-1] gt -1.0 then dxrange[*, idim-1] = [delta[idim-1], delta[idim-1]] $
    else begin
; NP ge 3 -- need to calculate uniformity  
    npm = np-1
    range = max(x, min = m)-m
    dxx = x[1:npm]- x[0:npm-1]
    dxrange[*, idim-1]  = [min(dxx, max = m), m]
    If dxrange[0, idim-1] lt 0.0  && dxrange[1, idim-1] gt 0 then delta [idim-1] = 0 else begin
      if dxrange[0, idim-1] lt 0.0 then begin
        x = reverse(x)
        dxx = x[1:npm]- x[0:npm-1]
        dxrange[*, idim-1] = [min(dxx, max = m), m]
        lcrev[idim-1] = -1
      endif
      err_d = err0*range
      err_min = errmach * max(abs([range, x[0], x[npm]]))
      if err_d lt err_min then begin
        print, '% GET_STR_DX: Error increased from '+strtrim(err_d, 2)+' to ' + $
           strtrim(err_min,2) + ' for DIM '+strtrim(idim, 2)
        err_d = temporary(err_min)
      endif

      delta[idim-1] = (float(dxrange[1, idim-1] - dxrange[0, idim-1]) lt err_d) ? $
         range/npm : 0.0
    endelse
  endelse
endfor

if keyword_set(modify) then begin
  if delta[0] gt 0.0 then a.dx1 = delta[0]
  if (ndim gt 1) && (delta[1] gt 0.0) then a.dx2 = delta[1]
  if (ndim gt 2) && (delta[2] gt 0.0) then a.dx3 = delta[2]
  if (ndim gt 3) && (delta[3] gt 0.0) then a.dx4 = delta[3]
  if min(lcrev) lt 0 then begin
    v1 = get_array(a, -1)
    if lcrev[0] lt 0 then begin
      a.x1 = reverse(a.x1)
      v1 = reverse(v1, 1)
    endif
    if lcrev[1] lt 0 then begin
      a.x2 = reverse(a.x2)
      v1 = reverse(v1, 2)
    endif
    if (ndim gt 2) && (lcrev[2] lt 0) then begin
      a.x3 = reverse(a.x3)
      v1 = reverse(v1, 3)
    endif    
    if (ndim gt 3) && (lcrev[3] lt 0) then begin
      a.x4 = reverse(a.x4)
      v1 = reverse(v1, 4)
    endif    
    put_array, a, v1, -1
  endif
  if lcstr eq 1 then strarray = temporary(a) else i2s, a, strarray, /no_copy
endif else begin
  a =0
  if min(lcrev) lt 0 then delta *= lcrev
endelse
return, delta
end   
