; ------------------------------------------------------------------------ 
; $Id: get_sweep2.pro,v 1.3 2007/06/06 20:09:34 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
 pro get_sweep2, ain, wdf, minbin = minbin, smooth = smothin, $
      horizontal=lchoriz, width = width, deflection = deflection,  $
      manual = manual, _extra = extrastuff
;+
; NAME:
;       GET_SWEEP2
;
; PURPOSE:
;       Measure the spacing of a comb on the image.
;       Image should be rotated using ALIGNSTR before using this routine.
;       NOTE: The Maximum WDF array-1 is used as a work array unless WDF is 
;             specified.
;               
; CATEGORY:
;       Graphics
;
; CALLING SEQUENCE:
;
;       GET_SWEEP2, STR1 [, WDF] [, /HORIZONTAL] [, MINBIN = minbin]
;         [, SMOOTH = smooth] [, WIDTH = width] [, DEFLECTION = deflection]
;         [, MANUAL] [, _extra= extrastuff]
;
; where: 
;       STR1   - Input Structure
;       WDF    - Output Waveform Array
;                Default: WDF = get_maxwdf()-1
;       HORIZONTAL - IF set, comb is horizontal
;       MINBIN - Minimum size of lineouts used to determine feature size.
;                Default = 0.1 of sweep
;                Actual number will always be odd
;       SMOOTH - Size of Boxcar filter, units of pixels
;                If specified, only one calculation
;                Default = 7 for a first try and do a second try with 
;                          smooth = .375 * (average period)
;       WIDTH  - Fraction of the fwhm used for actual sweep calibration
;                Default = 70%
;       MANUAL - Use the cursor to set the position of the comb.
;                This is useful when the frequency of the comb is low.
;       Deflection - Fraction of peak derivative which will qualify for a 
;                    change in slope.  Default = 0.1
;       _EXTRA - Keyword parameters for plotfld (Xrange, Yrange, etc)
;	
; OUTPUTS:
;       Spacing between peaks is stored in a wdf array.
;
; PROCEDURE:
;       0. A median with width 3 is applied to the image
;       1. User is asked to indicate the location of the comb on the image.
;       2-3. If manual is selected the comb is indicated.
;       otherwise:
;       2. A MINBIN wide lineout is taken perpendicular to the comb.
;       3. The fwhm is calculated. 
;       4. Width*FWHM lineout is taken along the comb.
;       5. The lineout is smoothed with a width of SMOOTH
;       6. The derivative is taken.
;       7. The derivative is smoothed with a width of smooth 
;       8. Values of the derivative with absolute values greater than 
;          DEFLECTION * MAX(Derivative) are found.
;       9. Values of zero crossing are determined.
;      10. Half periods are determined if deflections have opposite signs.
;          Multiple zero crossings are not counted.
;          
; EXAMPLE:
;       Calculate the spacing of a calibration comb stored in image 1.
;          alignstr, 1, 2   ; Rotate the image to insure it is square.
;          get_sweep2, 2, 1  ; Store the spacing in wdf array 1
;          plo, 1           ; Look at the spacing
;          ; Calculate the average spacing over the range [10,30]
;          dist = ave(1, left = 10, right = 30)
;          ; If comb is 4 ns then sweep rate is sw 
;          sw = 4e-9/dist & print, 'Sweep Rate =' , sw
;
; MODIFICATION HISTORY:
; 	Written by:	L. Paul Mix, June 28, 2000
;-
;
; insert a common block
;
@qscom.dat
;  Common block
@qsstrcom.dat
;       return on an error
       on_error, 1
;
      if n_elements (structinit) eq 0 then qsstruct_init
      if n_elements (qsplotinit) eq 0 then qsinit
;
      nparams = n_params()

      if nparams eq 0 then begin
        print,'% Format: ', $
         'GET_SWEEP2, STR1 [, WDF] [, /HORIZONTAL] [, MINBIN = minbin]
        print,'            [, SMOOTH = smooth] [, WIDTH = width] '
        print,'            [, DEFLECTION = deflection]  [, _extra= extrastuff]' 
        return
      endif
      a= get_valid_structure(ain, 'GET_SWEEP2', lcstr=lcstr, ierror=ier)
      if ier ne 0 then return

      if a.type ne 1 then begin
        print, '% GET_SWEEP2: Only type 1 image structures may be rotated'
        return
      endif
      if a.ndim gt 2 and a.two eq 0 then begin
        print, '% GET_SWEEP2: Image must have two dimensions
        retur
      endif
;
      if n_elements (wdf) eq 0 then wdf = get_maxwdf()-1
      wdf = get_save_array(wdf, get_maxwdf()-1)

      arr = reform ( get_array(a, -1))
      arr = median(arr, 3)
      ss = size(arr)
      if ss[0] ne 2 then begin
        print, '% ALIGNSTR: Structure must be two dimensional'
        return
      endif

      x = get_array(a, 1)
      y = get_array(a, 2)
      nx = n_elements(x)
      ny = n_elements(y)
      if nx ne ss[1] or ny ne ss[2] then begin
        print, '% GET_SWEEP2: X and Y arrays are not the same size as the image'
        print, nx, ny, ss[1], ss[2]
        return
      endif
;
      plotfld, a, cn=2, _extra = extrastuff
    if keyword_set(manual) then begin
      if keyword_set (lchoriz) then begin
        print, 'Indicate the bottom side of the time signal'
        cur, x1, y1, /quiet
        print, 'Indicate the top side of the time signal'
        cur, x2, y2, /quiet
;       CONVERT the point to bins
        junk = min( abs(y1-y), my1)
        junk = min( abs(y2-Y), my2)
        my1 = min([my1, my2], max= my2)
        oplot, !x.crange, y1*[1,1]
        oplot, !x.crange, y2*[1,1]
      endif else begin
        print, 'Indicate the left side of the time signal'
        cur, x1, y1, /quiet
        print, 'Indicate the right side of the time signal'
        cur, x2, y2, /quiet
;       convert the point to bins
        junk = min( abs(x1-x), mx1)
        junk = min( abs(x2-x), mx2)
        mx1 = min([mx1, mx2], max= mx2)
        oplot, x1*[1,1], !y.crange
        oplot, x2*[1,1], !y.crange
      endelse
    endif else begin
      print, 'Indicate the center of the time signal'
      cur, x1, y1, /quiet

;     get the lineouts
      
;     convert the point to bins
      junk = min( abs(x1-x), bx1)
      junk = min( abs(y1-y), by1)
      x1 = x[bx1]
      y1 = y[by1]
    endelse

      if n_elements(minbin) eq 0 then minbin = 0.1 else minbin = abs(minbin)
      if n_elements(width) eq 0 then width = 0.7
      if n_elements(deflection) eq 0 then deflection = 0.1
      if n_elements (smothin) eq 0 then begin
        smoth = 7
        try =2
      endif else begin
        try = 1
        smoth = smothin > 3
      endelse
  for itry = 1, try do begin
help, itry, smoth
      nx = n_elements(x)
      ny = n_elements(y)
    if keyword_set(manual) then begin
      if keyword_set (lchoriz) then begin
        yy = smooth(total(arr(*, my1:my2), 2), smoth)
        xx = x
      endif else begin
        yy = smooth(total(arr(mx1:mx2, *), 1), smoth)
        xx = y
        nx = ny
      endelse
    endif else begin
      if not keyword_set (lchoriz) then begin
;       Vertical feature
        bins = ny*minbin < ny
        ba = (by1 -bins/2) > 0 
        bb = (by1 + bins/2) < ny-1 
        yy = smooth(total(arr(*, ba:bb), 2), smoth)
        put_wdfarray, [0,1,yy], wdf
        bins =  round(nx*0.2)
        ba = (bx1-bins) > 0 
        bb = (bx1-bins/2) > 0
        n1 =bb-ba+1
        bc = (bx1+bins/2) < nx-1
        bd = (bx1+bins) < nx-1
        n2 =bd-bc+1
        base = (total(yy[ba:bb]) + total(yy[bc:bd]))/(n1+n2)
        
        junk = fwhm(wdf, base = base, left = ba, right = bd, $
          first = ff, second = ss, /quiet)
        if junk eq 0 then begin
          print, '% ALIGNSTR: Unable to find the comb data'
          return
        endif
        center = round((ff+ss)*0.5)
        diff = (ss-ff)*width
        ba = round(center -diff/2)
        bb = round(center +diff/2)
        if itry eq 1 then begin
          oplot, x[ba]*[1,1], !y.crange
          oplot, x[bb]*[1,1], !y.crange
        endif
        yy = smooth(total(arr(ba:bb, *), 1), smoth)
        xx = y
        nx = ny
      endif else begin
;       Horizontal feature
        bins = nx*minbin < nx 
        ba = (bx1 -bins/2) > 0 
        bb = (bx1 + bins/2) < nx-1 
        yy = smooth(total(arr( ba:bb, *), 1), smoth)
        put_wdfarray, [0,1,yy], wdf
        bins =  round(ny*0.2)
        ba = (by1-bins) > 0 
        bb = (by1-bins/2) > 0
        n1 =bb-ba+1
        bc = (by1+bins)/2 < ny-1
        bd = (by1+bins) < ny-1
        n2 =bd-bc+1
        base = (total(yy[ba:bb]) + total(yy[bc:bd]))/(n1+n2)
          
        junk = fwhm(wdf, base = base, left = ba, right = bd, $
          first = ff, second = ss, /quiet)
        if junk eq 0 then begin
          print, '% ALIGNSTR: Unable to find the comb data'
          return
        endif
        center = round((ff+ss)*0.5)
        diff = (ss-ff)*width
        ba = center -diff/2
        bb = center +diff/2
        if itry eq 1 then begin
          oplot, !x.crange, y[ba]*[1,1]
          oplot, !x.crange, y[bb]*[1,1]
        endif
        yy = smooth(total(arr(*, ba:bb), 2), smoth)
        xx = x
        nx = nx
      endelse
    endelse
      dy = yy[1:nx-1]-yy[0:nx-2]
      dy = smooth(dy, smoth)
      dxx = (xx[1:nx-1]+xx[0:nx-2])*0.5
      xbin = (xx[nx-1]-xx[0])/(nx-1)
      md = max(abs(dy))*deflection

      okp = where (dy gt md, nokp)
      dokp = okp[1:nokp-1] -okp[0:nokp-2]
      okdp = where (dokp gt 1, nokdp)

      okn = where (dy lt -md, nokn)
      dokn = okn[1:nokn-1] -okn[0:nokn-2]
      okdn = where (dokn gt 1, nokdn)

      yp = make_array(nokdp+1, /long)
      yp[0] = total(okp[0:okdp[0]])/(okdp[0]+1) 
      for i = 1, nokdp-1 do $
        yp[i] = total(okp[okdp[i-1]+1:okdp[i]]) /(okdp[i]-okdp[i-1])
      yp[nokdp] = total(okp[okdp[nokdp-1]+1:*])/(nokp-1-okdp[nokdp-1])
      yp = round(yp)

      yn = make_array(nokdn+1, /long)
      yn[0] = total(okn[0:okdn[0]])/(okdn[0]+1) 
      for i = 1, nokdn-1 do $
        yn[i] = total(okn[okdn[i-1]+1:okdn[i]]) /(okdn[i]-okdn[i-1])
      yn[nokdn] = total(okn[okdn[nokdn-1]+1:*])/(nokn-1-okdn[nokdn-1])
      yn = round(yn)

      jp = 0
      jn = 0
      missed = -999
      pold = missed
      nold = missed
      xstep =  make_array(nokdp+nokdn)
      step = xstep
      nstep = 0
      last = 0
      while jp lt nokdp and jn lt nokdn do begin
        if yp[jp] lt yn[jn] then begin
          if yp[jp+1] gt yn[jn] then begin
            ok = where ((dy[yp[jp]:yn[jn]-1] gt 0) and  $
              (dy[yp[jp]+1:yn[jn]] le 0), nok)
            if nok eq 1 then begin
              ok = ok[0]+yp[jp]
              pnew = dxx[ok] +dy[ok]/(dy[ok]-dy[ok+1])*(dxx(ok+1)-dxx[ok])
              if pold ne missed and last lt 0 then begin
                step[nstep]= pnew-pold
                xstep[nstep]= (pnew+pold)*0.5
                nstep = nstep+1
              endif
              pold = pnew
            endif else begin
              nold = missed
              pold = missed
            endelse
          endif else begin
            nold = missed
            pold = missed
          endelse
          jp = jp+1
          last = +1
        endif else begin
          if yn[jn+1] gt yp[jp] then begin
            ok = where ((dy[yn[jn]:yp[jp]-1] le 0) and  $
              (dy[yn[jn]+1:yp[jp]] gt 0), nok)
            if nok eq 1 then begin
              ok = ok[0]+yn[jn]
              nnew = dxx[ok] +dy[ok]/(dy[ok]-dy[ok+1])*(dxx(ok+1)-dxx[ok])
              if nold ne missed and last gt 0 then begin
                step[nstep]= nnew-nold
                xstep[nstep]= (nnew+nold)*0.5
                nstep = nstep+1
              endif
              nold = nnew
            endif else begin
              nold = missed
              pold = missed
            endelse
          endif else begin
            nold = missed
            pold = missed
          endelse
          jn = jn+1 
          last = -1
        endelse
      endwhile
      if nstep eq 0 then begin
        print, '% GET_SWEEP2: No valid deltas found'
        nstep = 3
      endif else if nstep lt 3 then begin
        print, '% GET_SWEEP2: Only ', strtrim(nstep, 2), ' valid deltas found'
        nstep = 3
      endif
      xstep = xstep[0:nstep-1]
      step = step[0:nstep-1]
      i2w, xstep, step, wdf, clabel = 'Timing Mark Spacing', $
        xlab = 'Distance', ylab= 'Spacing', /xydata
      if itry lt try then begin
        old = smoth
        smoth = round(total(step)/(nstep*xbin)*0.375)
        if smoth eq old then return
        print, 'SMOOTH set to ', strtrim (smoth, 2)
      endif
       
  endfor

      return
  END
