; ------------------------------------------------------------------------ 
; $Id: get_valid_str.pro,v 1.5 2010/09/29 13:50:55 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
 Function get_valid_str, str, call_prog, lcstr = lcstr, ierror = ier
;
;+
; NAME:
;       GET_VALID_STR
;
; PURPOSE:
;       Determine if a variable is a valid structure or a valid structure
;         array number and return it to the calling program
;       Error messages can be make to reflect the calling program.
;         
; CATEGORY:
;       Utility
;
; CALLING SEQUENCE:
;
;       Structure = GET_VALID_STR(STR, Call_Prog, LCSTR= lcstr, 
;                                 IERROR = ierror)
;    Where:
;        STRUCTURE - A structure if command is successful
;                    0 if STR is not a structure or a valid structure array 
;                      number.
;        STR       - A structure or a structure array number or a string which
;                    matches uniquely a structure comment.  If STR is a string,
;                    GET_MATCH(STR, /structure) is used.
;        Call_Prog - Name of the calling program.
;                    This will be printed if an error occurs.
;                    Default: Call_Prog = 'GET_VALID_STRUCTURE'
;        LCSTR     - Structure flag if no errors
;                    0 if STR is not a valid structure
;                    Note: 0 is returned if STR is a valid structure array #.
;                    1 if STR is a valid structure
;        IERROR    - Error flag
;                    0 indicates no errors
;                    1 error, STR is undefined or STR is not a structure, or 
;                      STR is not a valid structure array number
; EXAMPLE:
;       Check FIELD and return it as a structure f_str.  
;       Use the variable, lc, to return 1 if FIELD is a structure.
;       If an error occurs, label the command with 'PLOTFLD'
;         f_str = GET_VALID_STR (FIELD, 'PLOTFLD',lcstr = lc, ier = ier)
;
;
; MODIFICATION HISTORY:
;       Written by:     Paul Mix, Feb 20, 2002
;
;-     
 COMPILE_OPT hidden

on_error, 1
ier= 1
lcstr = 0
if n_elements (call_prog) eq 0 then call_prog ='GET_VALID_STR' $
  else call_prog= strupcase(strtrim(call_prog,2))

if n_elements(str) eq 0 then begin
  print, '% '+call_prog+': Structure must be defined'
  return, 0
endif


; if str is a structure input then set lcstrin

a = str
st = size(str, /type)
if st ne 8 then begin
  if st eq 7 then begin 
    a = get_match(a, /struct, nel = nel)
    if nel eq 0 or nel gt 1 then begin
      print,'% '+call_prog+': Search string found ', strtrim(nel, 2), ' values'
      print, a
      return, 0
    endif
  endif
  a = round(a[0])
  if a gt 0 and a le get_maxstr() then begin
    s2i, a, a
    if size(a, /type) ne 8 then begin
      print,'% '+call_prog+': Parameter must be a valid structure number'
      return, 0
    endif
  endif  else begin
    print,'% '+call_prog+': Parameter must be a structure or structure number'
    return, 0
  endelse
endif else lcstr= 1
names = tag_names(a)
junk = get_match('CLAB', names, nel = nel)
if nel eq 1 then junk = get_match('TYPE', names, nel = nel2) else nel2=0
if nel ne 1 and nel2 ne 1 then begin
  print, '% '+call_prog+': Not a valid PFIDL structure'
  help, /str, a
  return, 0
endif
ier= 0
return, a
end
