; ------------------------------------------------------------------------
; $Id: getdim.pro,v 1.5 2007/06/06 20:09:34 lpmix Exp $
; ------------------------------------------------------------------------
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
;+
; NAME:
;       GETDIM
;
; PURPOSE:
;	Convert a linear index as returned from the WHERE command to a set
;       of indices for the location
;       NOTE:  IDL will always allow a linear (one dimensional) index 
;              to access elements of a multi-dimensional array.
;
; CALLING SEQUENCE:
; 
;	IND = GETDIM(ARRAY, INDEX)
;
; PARAMETERS:
;       IND:    Array of indices
;               The first index of IND equals the number of dimensions
;               of the input ARRAY.
;               The second index of IND equals the number of elements in
;               INDEX.
;	ARRAY:  Array for which the indices are desired.
;               or
;               Linear array of dimensions for each coordinate.
;               DIMENSIONS must be set for this option.
;       INDEX:  One dimensional index to be converted to components.
;               INDEX may be an array of one dimensional indices.
;       DIMENSIONS: If set, ARRAY is a linear array of dimensions.
;
; OUTPUTS:
;       The only normal output is the array IND with the multi-dimensional 
;       indices.
;
; PROCEDURE:
;       The procedure is a loop.
;       IND is the array of indices.
;       INDEX is the one dimensional index
;       NDIM is the number of dimensions and SS[i] are the number of
;       elements for the Ith dimension.
;       D=1L
;       for i = 0, ndim -2 do begin
;         IND[i] = INDEX/D mod ss[i+1]
;         D = D * SS[i+1]
;       endfor
;       IND[ndim-1] = INDEX/d
;
; EXAMPLE:
;       Find the coordinates of the maximum value of an array, PICT, and
;       print the coordinates.  X and Y are the coordinates for PICT.
;         maximum = max(PICT, J)
;         IND = getdim(PICT, J)
;         print, 'Maximum of the picture is', maximum, '
;         print, 'Coordinates are: ', X[ind[0]], Y[ind[1]]
;
;       Using the values in the first example: PICT, X, Y
;       Print the X and Y locations where PICT is greater than 5.
;         ok = where (PICT gt 5, nok)
;         if nok gt 0 then begin
;           print, 'PICT is greater than 5 at the following locations'
;           ind = getdim(pict, ok)
;           for i = 0, nok-1 do print, x[0, i], y[1,i]
;         endif
;
;       Find the x and y coordinate indices for array, ARRAY, for location IND.
;         IND = GETDIM(ARRAY, IND)
;           or
;         ss = size(array)
;         if SS[0] gt 0 then IND = GETDIM(SS[1:SS[0]], IND, /DIMENSIONS) else IND=0
;
;
; MODIFICATION HISTORY:
;       Written by:  L. P. Mix, November 16, 2004
;-

Function getdim, array, index, dimensions = dimensions
on_error, 1

npar = n_params()
if npar lt 2 then begin
  print, '% GETDIM: Two parameters are required'
  print, '% FORMAT: IND = GETDIM (ARRAY, INDEX)'
  return, 0
endif
if n_elements(array) eq 0 or n_elements(index) eq 0 then begin
  print, '% GETDIM: Both parameters must be defined'
  print, '% FORMAT: IND = GETDIM (ARRAY, INDEX)'
  return, 0
endif

maxind = max(index)
if keyword_set(dimensions) then begin
  ndim = n_elements(array)
  nele = long(array[0])
  for i = 1, ndim-1 do nele = nele*array[i]
  if (nele le maxind) then begin
    print, '% GETDIM: Index not consistent with array size, return: 0'
    print, '          Size, index: ', strtrim (nele, 2),', ', $
      strtrim((maxind),2)
    return, 0
  endif
  ss = array
endif else begin  
  ss = size(array)
  ndim = ss[0]
  if ndim eq 0 then begin
    print, '% GETDIM: Scalar input, return: 0'
    return, 0
  endif

  if (ss[ndim+2] le maxind) then begin
    print, '% GETDIM: Index not consistent with array size, return: 0'
    print, '          Size, index: ', strtrim (ss[ndim+2], 2),', ', $
      strtrim((maxind),2)
    return, 0
  endif
  ss = ss[1:ndim]
endelse
if ndim eq 1 then begin
  ind = long(index)
  return, ind
endif

ind = make_array(n_elements(index), ndim, /long)
d = 1L
for i = 0, ndim-2 do begin
  ind[*, i] = index/d mod ss[i]
  d = d*ss[i]
endfor
ind[*, ndim-1] = index/d
return, transpose(ind)
END
