; ------------------------------------------------------------------------ 
; $Id: imagep.pro,v 1.1 2010/08/24 22:12:50 lpmix Exp $
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
 Pro imagep, str1in, str2, reflection = reflection, periodic = periodic, $
          value = value, invert = invert, nperiods = nperiods, $
          again = again, CLABEL = clabel
;+
; NAME:
;       image
;
; PURPOSE:
;       Generate an image file from data which has a reflection plane or 
;         has periodic boundaries.
;       The data must be two dimensional or three dimensional with a degenerate
;         dimension and have a single attribute.
;
; CATEGORY:
;       Math 
; CALLING SEQUENCE:
;       IMAGE, STR1 [, STR2] [, REFLECTION = reflection]
;              [, PERIODIC = PERIODIC] [, VALUE = value] [, /INVERT]
;              [, NPERIODS = NPERIODS] [, CLABEL = clabel] [, /AGAIN]
; INPUTS:
;       STR1:    Structure array or array number of the first structure
;                
; OPTIONAL INPUT:
;       STR2:    Array number or structure array for the result,
;                if not specified the image is stored in STR1 (the first array)
;        Note:   If str1 is an array number then str2 will be an array number.
;                If str1 is an array, then str2 will be an array.
; KEYWORDS:
;       REFLECTION : If present must equal to +/- 1, 2, or 3.
;                    + indicates maximum value; - indicates minimum value
;                    for the reflection.
;                    1, 2, 3 indicates coordinates 1, 2, or 3
;       VALUE      : If a different value is required for the reflection, 
;                    value is used to specify this point. Value must be greater
;                    than the maximum spatial value of the data or less than
;                    the minimum to avoid multiple values at a point.
;            NOTE: For particle data, value should be defined.
;       INVERT     : If present and not zero, multiple the field values by -1.
;       PERIODIC   : If present must equal to +/- 1, 2, or 3.
;                    1, 2, 3 indicates coordinates 1, 2, or 3
;                    Current version uses abs(periodic)
;                    Single block periodic will remain single block
;       NPERIODS   : Number of replications for the periodic data, default = 1
;                    Note: NPERIODS =1 for multiple block field data.
;       AGAIN      : If present and not zero, all parameters will be the same
;                    as the last call to IMAGE
;         NOTE: This means all other parameters are ignored
;       CLABEL:  Optional label for the combined structures.
; OUTPUTS:
;       NONE    The composite image is stored in a structure array 
;                 in the common block
; RESTRICTIONS:
;       STR1 array must be defined and have valid data
;       If a structure has three dimensions, one of them must be degenerate.
;       The number of vectors or attributes must be 1, ie scalar field.
; EXAMPLES:
;       Assume a field structure in 2 and conductors in 1; generate a 
;         a reflection about the positive y value in both.
;           IMAGE, SLICE (2, 1), 3, reflect= 2 ; slice will get the r component
;       or 
;           PLOTSTR, 2, 1, /quiet      ; select the r component of the field
;           IMAGE, 82,3, reflect = 2      ; reflect the field
;       and
;           IMAGE, 1, /again          ; reflect the conductors
;       Reflect structure array 2 about the maximum y value and store in 
;         array 3
;               IMAGE, 2, 3, reflect = 2
;       Reflect structure array 2 about the minimum Z value and store in 
;         array 4, and invert the field 
;               IMAGE, 2, 4, reflect = -3
;       Reflect structure array 5 about the r= 0 value and store the result in 
;         array 4
;               IMAGE, 5, 4, reflect = 1, value = 0
;       Replicate structure array 3 about the maximum theta value (dimension 2)
;       and store the result in array 
;               IMAGE, 3, period = 2
;       Replicate structure array f about the maximum r value and store the
;        result in structure, g
;               IMAGE, f, g, period=1
;
; MODIFICATION HISTORY:
;      Original version, L. P. Mix, April 21, 1994
;-
;
; insert a common block
;
@qscom.dat
;  Common block
@qsstrcom.dat
;       return on an error
      on_error, 1
;
      if n_elements (structinit) eq 0 then qsstruct_init
      if n_elements (qsplotinit) eq 0 then qsinit
;
      nparams = n_params()
      if nparams lt 1 then begin
        print,'% Format: ', $
      ' IMAGE, STR1 [, STR2] [, REFLECTION = reflection]'
print,'             [, PERIODIC = PERIODIC] [, VALUE = value] [, /INVERT]'
print,'             [, NPERIODS = NPERIODS] [, CLABEL = clabel] [, /AGAIN]'
        return
      endif
;
      str1= str1in
      if n_elements(str1) eq 0 then begin
        print, 'Parameter to IMAGE must be defined'
        return
      endif
      a = str1
      lcstr = 1
      b = size(a)
      if b(b(0)+1) ne 8 then begin
        if a gt 0 and a le maxstructure then begin
          lcstr = 0
          s2i, a, a
          b = size (a)
          if b(b(0)+1) ne 8 then begin
            print, $
   ' First parameter must be a structure or a structure array number'
            return
          endif
        endif  else begin
          print, $
   ' First parameter must be a structure or a valid structure array number'
          return
        endelse
      endif
;
      b = size(str2)
      if b(b(0)+1) eq 8 then str2 = 0
      save = 0
      if nparams gt 1 and lcstr eq 0 then begin 
          save = get_save_str(str2, str1) 
      endif else if lcstr eq 0 then save = str1 
;
      if a.type eq 1 then begin
        image_fld, a, reflection = reflection, periodic = periodic, $
          value = value, invert = invert, nperiods = nperiods, $
          AGAIN = again, CLABEL = clabel
;
      endif else if a.type eq 2 then begin
        image_par, a, reflection = reflection, periodic = periodic, $
          value = value, invert = invert, nperiods = nperiods, $
          again = again, CLABEL = clabel
;
      endif else if a.type eq 3 then begin
        image_grd, a, reflection = reflection, periodic = periodic, $
          value = value, invert = invert, nperiods = nperiods, $
          again = again, CLABEL = clabel
;
      endif else if a.type eq 4 then begin
        image_con, a, reflection = reflection, periodic = periodic, $
          value = value, invert = invert, nperiods = nperiods, $
          again = again, CLABEL = clabel
      endif else begin
        Print, '% IMAGE: Please contact Paul Mix.(LPMIX@SANDIA.GOV)'
        print, '         IMAGE not complete for grid data'
      endelse
;
      if lcstr eq 0 and save ne 0  then begin
        i2s, a, save , /no_copy
        str2 = save
      endif else begin
        if nparams eq 2 then str2 = temporary(a) else str1in = temporary(a)
      endelse
      return
      end
