; ------------------------------------------------------------------------ 
; $Id: intstr.pro,v 1.3 2007/06/06 20:09:34 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
  Function intstr, ain, fld, kvlue1, vlue1, kvlue2, vlue2, $
               overplot = over, block = block, index = index, smooth = smoo,$
                xinteg = xinteg, yinteg = yinteg, start = start, $
                reverse = reverse
; 
;+
; NAME:
;	INTSTR
;
; PURPOSE:
;	Integrate a two dimensional structure with respect to its first or 
;         second dimension.
;       Three dimensional data will be reduced to a two dimensional structure.
;       Note: Four dimensional data is checked for valid data as closely as 3D
;       data.
;
; CATEGORY:
;	Math
;
; CALLING SEQUENCE:
;
;	Structure2 = INTSTR (Structure1, Field, Kvalue1, Value1, Kvalue2, Value2)
;
;
; INPUTS:
;	Structure1: Input this is a multidimensional field type structure.
;                     or a structure array numbger
;       Field:      Field component to be integrated
;       Note: Field may be positive or negative; Field = -(abs(Field))
;       Note: If Field is an array, the square root of the sum of
;             the squares will be used.
;       Note: If A has only one attribute (such as an image or a charge density);
;             the this parameter can be omitted.
;       KVALUE1,2
;             If Field is a two dimensional field then these values are ignored.
;             If Field is a three or four dimensional field then these values
;               indicate the coordinates which are fixed for the two
;               dimensional plots.
;       VALUE1, 2
;             If Field is a two dimensional field then these values are ignored
;             If Field is a three or four dimensional field then these values
;               indicate the coordinate values for the fixed dimensions in
;               the two dimensional plots. Linear interpolation is used
;               according to the value of VALUE1. If VALUE1 has two elements
;               the average value over the interval is used.
;             WARNING: If INDEX is specified, VALUE1 will be changed to spatial
;                      coordinates using the midpoints of the bins.
;
; KEYWORDS:
;       OVERPLOT: If overplot is not zero and kvalue1,2 is undefined, the 
;                 values will be taken from the last plot.
;       BLOCK   : If the field is multi-block, BLOCK can be used to specify
;                 one or more blocks to be sliced.  BLOCK is an array of one
;                 or more values.
;       INDEX   : If present and not zero, then the units of VALUEi will be
;                 taken as indices of of the field array.  Valid values are
;                 1 to the maximum number of values in KVALUEi.
;                 Care should be exercised with multiple block data.
;             WARNING: If INDEX is specified, VALUE1 will be changed to spatial
;                      coordinates using the midpoints of the bins.
;       XINTEG  - Integrate the field variable with respect to the abscissa 
;                 value (x) according to the following values:
;                 If XINTEG = 0, do nothing.
;                 If XINTEG = 1, integrate f(x, y) dx
;                 If XINTEG = 2, integrate f(x, y) x dx
;                 If XINTEG = 3, integrate f(x, y) x^2 dx
;                 If XINTEG = string, integrate f(x,y)* g(x) dx
;                   dx is the spacing between abscissa values.
;                   and g(x) = string where string is any function of x.
;                 examples are: string = 'sin(x)'
;                               string = 'x*sqrt(x)'
;                               string = 'my_function(x, 4.5)'
;        Note: Either x or y may be used in the function call in the string
;              ie: flag = 'sin(x)' gives the same answer as flag = 'sin(y)'
;                  Both x and y will be set to the integration variable
;                  Use chastr to multiply the array by coordinate value.
;                    for example: integral(f(x,y) *y*g(x) * dx  
;                                 must be done with two calls: intstr and chastr
;              The initial value of the integral is set to 0.0 or 
;                   to the value in the WDF array specified by START.
;          Default: If yinteg and xinteg are both 0 or undefined, then xinteg =1.
;           Note: If two blocks terminate on a third, then the initial value of
;                 of the integral at the block boundary is linearly interpolated
;                 between the two incomming blocks.
;       YINTEG  - Integrate the field variable with respect to ordinate 
;                 value (y) according to the following values:
;                 If YINTEG = 0 or if XINTEG = [1,2,3], do nothing.
;                 If YINTEG = 1, integrate f(x, y) dy
;                 If YINTEG = 2, integrate f(x, y) y dy
;                 If YINTEG = 3, integrate f(x, y) y^2 dy
;                 If XINTEG = string, integrate f(x,y)* g(y) dy
;                   dy is the spacing between abscissa values.
;                   and g(y) = string where string is any function of y.
;                 examples are: string = 'sin(y)'
;                               string = 'y*sqrt(y)'
;                               string = 'my_function(y, 4.5)'
;        Note: Either x or y may be used in the function call in the string
;              ie: flag = 'sin(x)' gives the same answer as flag = 'sin(y)'
;                  Both x and y will be set to the integration variable
;                  Use chastr to multiply the array by coordinate value.
;                    for example: integral(f(x,y) *x*g(y) * dy   
;                                 must be done with two calls: intstr and chastr
;                 The initial value of the integral is set to 0.0 or 
;                   to the value in the WDF array specified by START.
;           Note: If XINTEG ne 0 then YINTEG is ignored.
;           Note: If two blocks terminate on a third, then the initial value of
;                 of the integral at the block boundary is linearly interpolated
;                 between the two incomming blocks.
;       START     If present this is interpreted as initial values at the lower
;                 limit of integration. Start is an integer indicating a WDF 
;                 array.  Values are between 1 and maxwdfarray
;       REVERSE   If present and not zero, the limits of integration are 
;                 reversed. 
;       SMOOTH  - If present and not zero, the two dimensional field will be
;                 smoothed using a boxcar filter.  The number of cells will
;                 be the lowest odd number greater than or equal to the
;                 maximum of [3, SMOOTH]
;
; OUTPUTS:
;	Structure is modified to produce a two dimensional structure or 
;       a structure with n-dimensions and all but two degenerate
;
; EXAMPLE:
;	Reduce structure f to a two dimensional structure by slicing along
;         the second dimension between 0.0 and 0.4 and integrate with respect 
;         to x.  Save the magnitude of the integrated field vector in structure g.
;       g = intstr (f, [1,2,3], 2, [0.0, 0.4], /xint)
;
;       Assume d is a charge density structure with one attribute.  Take a slice 
;         at z= 0.25 and store the integral (y dy) in sturcture array 12.
;       i2s, INTSTR (d, 3, .25, yint = 2), 12
; <or>  i2s, INTSTR(d, 1, 3, .25, yint = 2), 12
; <or>  i2s, INTSTR (d, 3, .25, yint = 'y'), 12
;
;       Integrate structure array 2, a two dimensional structure, with respect to 
;         the first dimension and store the result in st2.
;       st2 = intstr(2, /x)
; <or>  st2 = intstr(2)
;
;       Integrate structure array 2, a two dimensional structure, times the sin
;         of the first dimension with respect to the first dimension and store
;         the result in st2.
;       st2 = intstr(2, xint = 'sin(x)')
; <or>  st2 = intstr(2, xint = 'sin(y)')
;
;
; MODIFICATION HISTORY:
; 	Written by:	Paul Mix, August 20, 1993
;       Added strings for xintet and yinteg, 2/96 lpm
;-
; insert a common block
;
@qscom.dat
;  Common block
@qsstrcom.dat
;
@com.dat
;       return on an error
      on_error, 1
;
      if n_elements(initwdf) eq 0 then wdfinit
;
      if n_elements (structinit) eq 0 then qsstruct_init

      a = get_valid_structure(ain, 'INTSTR', ierror = ier)
      if ier ne 0 then return, 0.0

      if a.type ne 1 then begin
        print, '% INTSTR: Only Field type structures may be integrated.'
        return, 0.0
      endif
;
      if n_elements(block) eq 0 then block = 0
;     if we already have two dimensional data with a single attribute or 
;       vector quantity on a single block then return 
;
      if ((a.ndim eq 2) or (a.two ne 0 and a.ndim eq 3)) and $
        (a.nvect eq 1) and (block(0) eq 0) then a = a else begin
;
        if n_elements(field) gt 0 then fld = field
        if n_elements(vlue1) gt 0 then value1 = vlue1
        if n_elements(kvlue1) gt 0 then kvalue1 = kvlue1
        if n_elements(vlue2) gt 0 then value2 = vlue2
        if n_elements(kvlue2) gt 0 then kvalue2 = kvlue2
;
        if n_elements (block) eq 0 then block = 0
        if n_elements (over) eq 0 then over  = 0
        if n_elements (index) eq 0 then index = 0
        structinit = -1
        a = slice(a, fld, kvalue1, value1, kvalue2, value2, $
          block=block, overplot=over, index=index )
        structinit = 1
        b = size(a)
        if b(b(0)+1) ne 8 then begin
          print, $
     '% INTSTR: First parameter must be a structure or a valid QS array number'
          return, 0.0
        endif
      endelse
      if a.type ne 1 then begin
        print, '% INTSTR: Only Field type structures may be integrated'
        return, 0.0
      endif
;     slice makes a one attribute structure
      fld = -1
      nfld = 1
;
;     Check start
;
      if n_elements (start) eq 0 then start = 0
      if (start gt get_maxwdf() ) or (start le 0) then start=0 else begin
        if wdfnp(start-1) le 0 then start = 0 else begin
          startmin = wdfminx(start-1)
          startmax = wdfmaxx(start-1)
          w2i, xstart, ystart, start
        endelse
      endelse
;
      if n_elements(smoo) eq 0 then smoo = 0
      if not keyword_set(xinteg) and not keyword_set(yinteg) then xinteg = 1
;
      if n_elements(kvalue1) eq 0 then kvalue1 = a.two
;
      nblock = a.block
      if a.ndim eq 2 then begin
        xval = 1 & yval = 2
      endif else if a.ndim eq 3 then begin
        if kvalue1 ne 1 then xval =1 else xval = 2
        if kvalue1 ne xval+1 then yval = xval+1 else yval = xval+2
      endif else begin
; kvalue1 is less than kvalue2 from function slice
        xval = 0
        if kvalue1 ne 1 then xval =1
        if xval eq 0 and kvalue2 ne 2 then xval = 2 $
          else begin
            xval = 3
            yval = 4
        endelse
        if xval ne 3 then begin
          if (kvalue1 ne xval +1 and kvalue2 ne xval+1) then $
          yval = xval+1 else begin
            if (kvalue1 ne xval +2 and kvalue2 ne xval+2) then $
              yval = xval+2 else yval = xval+3
          endelse
        endif
      endelse
      xmin = a.range(0, xval-1, 0)
      xmax = a.range(1, xval-1, 0)
      ymin = a.range(0, yval-1, 0)
      ymax = a.range(1, yval-1, 0)
;     1/2^15 = 3.0517578e-05
      epsilon = 3.0517578e-05
      xerr = (xmax-xmin)*epsilon
      yerr = (ymax-ymin)*epsilon
;
      nconnect=0
      ntconnect = 0
;     calculate x and y connections if nblock > 1
      if nblock gt 1 then begin
        if n_elements(a.order) lt a.ndim*nblock then begin
;         calculate order
          print, ' calculate order'
          order = make_array(ndim, mxbl, /long)
          for i = 0, ndim-1 do order (i, *) = sort(range(0,i, 1:mxbl))
          connect = get_blockconnect(order, a.range, a.two, a.ndim, a.block)
        endif else $
        connect = get_blockconnect(a.order, a.range, a.two, a.ndim, a.block)
        save_con= connect
;       select the connections with the proper plane 
        ii = where (connect(0, *) eq xval-1, nconnect)
        if nconnect gt 0 then xconnect = connect(*, ii)
;       nxconnect is number of connections in the x direction
        nxconnect = nconnect
        jj = where (connect(0, *) eq yval-1, nconnect)
        if nconnect gt 0 then connect = connect(*, jj)
;       nconnect is the number of connections in the y direction
        isave = [ii, jj]
        ok = where (isave gt 0, n_con_save)
;       save_con are the connections with joins in the x and y directions
        if n_con_save gt 0 then save_con = save_con(*, isave(ok))
      endif else n_con_save = 0
      blockarray = 1+ indgen(a.block)
      if nblock gt 1 then begin
        if keyword_set(xinteg)  then begin
          intval = xval
          perpval = yval
          if keyword_set(reverse) then $
            ii = rotate(sort(a.range(1, xval-1, blockarray)), 2) else $
            ii = sort(a.range(0, xval-1, blockarray))
;........ transverse connections (tconnect, ntconnect)
          ntconnect = nconnect
          if ntconnect gt 0 then tconnect = connect
;........ connections along integration path are nconnect and connect
          nconnect = nxconnect
          if nconnect gt 0 then connect = xconnect
          yinteg = 0
        endif else  begin
          intval = yval
          perpval = xval
          if keyword_set(reverse) then $
            ii = rotate(sort(a.range(1, yval-1, blockarray)), 2) else $
            ii = sort(a.range(0, yval-1, blockarray))
          ntconnect = nxconnect
          if ntconnect gt 0 then tconnect = xconnect
        endelse
        blockarray= blockarray(ii)
      endif else begin
        if keyword_set(xinteg)  then begin
          intval = xval
          perpval = yval
        endif else  begin
          intval = yval
          perpval = xval
        endelse
      endelse
      xsize = max(a.size(perpval-1, *))
;
;     make arrays for the connections
      if nconnect gt 0 then begin
        conx = make_array(xsize, nconnect)
        conv = make_array(xsize, nconnect)
        connp = make_array(nconnect)
      endif
      if ntconnect gt 0 then begin
        contx = make_array(2, ntconnect)
        contv = make_array(2, ntconnect)
      endif
;
;     save the extent for each block in the perpendicular direction 
      extent = make_array(2, nblock)
;
    if not keyword_set(reverse)  then begin

      if n_con_save gt 0  and nblock gt 2 then begin
        if keyword_set(xinteg)  then errval = xerr else errval = yerr
;           if two blocks begin at the same point and one has no connection
;             make sure the first one is the one with the connection with a 
;             higher block
        arr = -1
        for j = 0, nblock -2 do begin
          ok = where (save_con(1, *) eq blockarray(j), nok)
          if nok gt 0 then arr = [arr , reform (save_con(2, ok), nok)]
          ok = where (save_con(2, *) eq blockarray(j), nok)
          if nok gt 0 then arr = [arr , reform (save_con(1, ok), nok)]
          if n_elements(arr) gt 1 then repeat begin
            noswap = 1
            for i = j+1, nblock-2 do begin
              if abs(a.range(0, intval-1, blockarray(i)) -  $
                a.range(0, intval-1, blockarray(i+1))) lt  errval then begin
                ok1 = where (blockarray(i+1) eq arr, nok1)
                ok  = where (blockarray(i)   eq arr, nok)
                if nok1 gt 0 and nok eq 0 then begin
                   noswap = 0
                   t = blockarray(i) & blockarray(i) = blockarray(i+1)
                   blockarray(i+1) = t
                endif
              endif
            endfor
          endrep until noswap
        endfor
      endif
;
      for ii = 0,  nblock-1  do begin
        iblock = blockarray(ii)
        cont = reform( get_array (a, fld, iblock, lab = lab))
;       if smoothing is desired - do it!
        if smoo gt 0 then cont = smooth(cont, smoo>3)
;
        xx = get_array (a, xval, iblock)
        yy = get_array (a, yval, iblock)
        ny = n_elements(yy)
        nx = n_elements(xx)
        if keyword_set(xinteg) then begin
          dely = (yy(ny-1)-yy(0))*epsilon
          yst = 0
          extent(*, ii)=[yy(0)-yerr , yy(ny-1)+yerr]
          if start gt 0 and xx(0) lt xmin+xerr then begin
            yst = yy
            yst(0)= yst(0)+dely
            yst(ny-1)=yst(ny-1)-dely
            yst = get_yarr(xstart, ystart, yst, /no_truncate)
            if n_elements(yst) ne ny then begin
              Print, '% INTSTR: Error Getting start position, block', iblock
              yst = make_array(ny, value = 0.0)
            endif
          endif else if nconnect gt 0 or ntconnect gt 0 then begin
            xtemp = 0 & ytemp = 0
;           nicon- number of direct connections
;           nitcon - number of transverse connections
            if nconnect gt 0 then $
            icon = where (connect(2, *) eq iblock, nicon) else nicon = 0
            if ntconnect gt 0 then $
            itcon = where (tconnect(1, *) eq iblock or tconnect(2,*) $
              eq iblock , nitcon)  else nitcon = 0
            if start gt 0 then begin
              ok = where (yy ge startmin and yy le startmax, nystemp)
              if nystemp gt 0 then begin
                ystemp= yy(ok)
                ystemp(0)= ystemp(0)+dely
                ystemp(nystemp-1) = ystemp(nystemp-1) -dely
              endif
            endif else nystemp = 0
            if nicon gt 0 then begin
              npt = connp(icon(0))
              xtemp = [0,conx(0:npt-1,icon(0))]
              ytemp = [0,conv(0:npt-1,icon(0))]
              if nystemp gt 0 then ok = where (ystemp lt conx(0,icon(0))-yerr $
                 or ystemp gt conx(npt-1, icon(0))+yerr, nystemp)
              if nystemp gt 0 then ystemp = ystemp(ok)
              if nicon gt 1 then begin
                for i = 1, nicon-1 do begin
                  npt = connp(icon(i))
                  xtemp = [xtemp, reform(conx(0:npt-1, icon(i)))]
                  ytemp = [ytemp, reform(conv(0:npt-1, icon(i)))]
                  if nystemp gt 0 then ok=where(ystemp lt conx(0,icon(i))-yerr $
                     or ystemp gt conx(npt-1, icon(i))+yerr, nystemp)
                  if nystemp gt 0 then ystemp = ystemp(ok)
                endfor
              endif
            endif
            if nitcon gt 0 then begin
              for i = 0, nitcon-1 do begin
                if tconnect(1,itcon(i)) eq iblock then $
                  tblock = tconnect(2, itcon(i))
                if tconnect(2,itcon(i)) eq iblock then $
                  tblock = tconnect(1, itcon(i))
                nb = (where (tblock eq blockarray))(0)
                if nb lt 0 then begin
                  print,'Error in calculating transverse blocks',nb,tblock,iblock
                  return, 0.0
                endif
;                add both the upper and lower values for the block.
                if nb lt ii then begin
                  xtemp = [xtemp, contx(*,itcon(i))]
                  ytemp  = [ytemp, contv(*,itcon(i))]
                  if nystemp gt 0 then ok=where(abs(ystemp - $
                    contx(0,itcon(i))) -yerr gt 0.0 , nystemp )
                  if nystemp gt 0 then ystemp = ystemp(ok)
                  if nystemp gt 0 then ok=where(abs(ystemp - $
                    contx(1,itcon(i))) -yerr gt 0.0 , nystemp )
                  if nystemp gt 0 then ystemp = ystemp(ok)
                endif
              endfor
            endif
            if nystemp gt 0 then begin
              if ii gt 0 then begin
                i = 0
                while i lt ii and nystemp gt 0 do begin
                  ok = where(ystemp lt extent(0, i) or ystemp gt extent(1, i), $
                         nystemp)
                  if nystemp gt 0 then ystemp = ystemp(ok)
                  i = i+1
                endwhile
              endif
              if nystemp gt 0 then begin
                xtemp = [xtemp, ystemp]
                ystemp =get_yarr(xstart,ystart, ystemp, /no_truncate)
                ytemp = [ytemp, ystemp]
              endif
            endif
            if n_elements(xtemp) gt 1 then begin
              xtemp = xtemp(1:*) & ytemp = ytemp(1:*)
              aa = sort (xtemp)
              xtemp= xtemp(aa)
              ytemp = ytemp(aa)
              if xtemp(0) gt a.range(0, yval-1, iblock) then begin
                xtemp = [a.range(0, yval-1, iblock)-10*yerr, xtemp]
                ytemp = [ytemp(0), ytemp]
              endif
              np = n_elements(xtemp)
              if xtemp(np-1) lt a.range(1, yval-1, iblock) then begin
                xtemp=[xtemp,  a.range(1, yval-1, iblock)+yerr*10]
                ytemp = [ytemp, ytemp(np-1)]
              endif
              yst = yy
              yst = get_yarr(xtemp, ytemp, yy, /no_truncate)
            endif
          endif
          cont = x_gridinteg (cont, xx, yst, xinteg)
          if nconnect gt 0 then begin
            icon = where (connect(1, *) eq iblock, nicon)
            if nicon gt 0 then begin
              nx = n_elements(xx)
              for i  = 0, nicon-1 do begin
                connp(icon(i)) = ny
                conx (0:ny-1,icon(i)) = yy
                conv (0:ny-1,icon(i)) = cont(nx-1,*)
              endfor
            endif
          endif
          if ntconnect gt 0 then begin
            itcon = where (tconnect(1, *) eq iblock or tconnect(2,*) $
              eq iblock , nitcon)
            if nitcon gt 0 then begin
              for i = 0, nitcon-1 do begin
                if tconnect(1,itcon(i)) eq iblock then $
                  tblock = tconnect(2, itcon(i))
                if tconnect(2,itcon(i)) eq iblock then $
                  tblock = tconnect(1, itcon(i))
                nb = (where (tblock eq blockarray))(0)
                if nb lt 0 then begin
                  print,'Error in calculating transverse blocks',nb,tblock,iblock
                  return, 0.0
                endif
                if nb gt ii then begin
                  contx(*, itcon(i)) = a.range (*, yval -1, iblock)
                  contv(*,itcon(i))=[get_y(xx,cont(*,0),a.range(0,xval-1,tblock) $
                    ), get_y(xx,cont(*,ny-1),a.range(0,xval-1,tblock))]
                endif
              endfor
            endif
          endif
;
        endif else if keyword_set(yinteg) then begin
          delx =(xx(nx-1)-xx(0))*epsilon
          yst = 0
          extent(*, ii) = [xx(0)-xerr, xx(nx-1)+xerr]
          if start gt 0 and yy(0) lt ymin+yerr then begin
            yst = xx
            yst(0) = yst(0)+delx
            yst(nx-1) = yst(nx-1)-delx
            yst = get_yarr(xstart, ystart, yst, /no_truncate)
            if n_elements(yst) ne nx then begin
              Print, '% INTSTR: Error Getting start position, block', iblock
              yst = make_array(ny, value = 0.0)
            endif

          endif else if nconnect gt 0  or ntconnect gt 0  then begin
            xtemp = 0 & ytemp = 0
            if nconnect gt 0 then $
            icon = where (connect(2, *) eq iblock, nicon) else nicon = 0
            if ntconnect gt 0 then $
            itcon = where (tconnect(1, *) eq iblock or tconnect(2,*) $
              eq iblock , nitcon) else nitcon = 0
            if start gt 0 then begin
              ok = where (xx ge startmin and xx le startmax, nystemp)
              if nystemp gt 0 then begin
                xxt= xx(ok)
                xxt(0) = xxt(0) + delx
                xxt(nx-1) = xxt(nx-1) -delx
                ystemp= temporary(xxt)
              endif
            endif else nystemp = 0
            if nicon gt 0 then begin
              npt = connp(icon(0))
              xtemp = [0,conx(0:npt-1,icon(0))]
              ytemp = [0,conv(0:npt-1,icon(0))]
              if nystemp gt 0 then ok = where (ystemp lt conx(0,icon(0))-xerr $
                 or ystemp gt conx(npt-1, icon(0))+xerr, nystemp)
              if nystemp gt 0 then ystemp = ystemp(ok)
              if nicon gt 1 then begin
                for i = 1, nicon-1 do begin
                  npt = connp(icon(i))
                  xtemp = [xtemp, conx(0:npt-1, icon(i))]
                  ytemp = [ytemp, conv(0:npt-1, icon(i))]
                  if nystemp gt 0 then ok=where(ystemp lt conx(0,icon(i))-xerr $
                     or ystemp gt conx(npt-1, icon(i))+xerr, nystemp)
                  if nystemp gt 0 then ystemp = ystemp(ok)
                endfor
              endif
            endif
            if nitcon gt 0 then begin
              for i = 0, nitcon-1 do begin
                if tconnect(1,itcon(i)) eq iblock then $
                  tblock = tconnect(2, itcon(i))
                if tconnect(2,itcon(i)) eq iblock then $
                  tblock = tconnect(1, itcon(i))
                nb = (where (tblock eq blockarray))(0)
                if nb lt 0 then begin
                  print,'Error in calculating transverse blocks',nb,tblock,iblock
                  return, 0.0
                endif
                if nb lt ii then begin
                  xtemp = [xtemp, contx(*,itcon(i))]
                  ytemp  = [ytemp, contv(*,itcon(i))]
                  if nystemp gt 0 then ok=where(abs(ystemp - $
                    contx(0,itcon(i))) -xerr gt 0.0 , nystemp )
                  if nystemp gt 0 then ystemp = ystemp(ok)
                  if nystemp gt 0 then ok=where(abs(ystemp - $
                    contx(1,itcon(i))) -xerr gt 0.0 , nystemp )
                  if nystemp gt 0 then ystemp = ystemp(ok)
                endif
              endfor
            endif
            if nystemp gt 0 then begin
              if ii gt 0 then begin
                i = 0
                while i lt ii and nystemp gt 0 do begin
                  ok = where(ystemp lt extent(0, i) or ystemp gt extent(1, i), $
                         nystemp)
                  if nystemp gt 0 then ystemp = ystemp(ok)
                  i = i+1
                endwhile
              endif
              if nystemp gt 0 then begin
                xtemp = [xtemp, ystemp]
                ystemp = get_yarr(xstart, ystart, ystemp, /no_truncate)
                ytemp = [ytemp, ystemp]
              endif
            endif
            if n_elements(xtemp) gt 1 then begin
              xtemp = xtemp(1:*) & ytemp = ytemp(1:*)
              aa = sort (xtemp)
              xtemp= xtemp(aa)
              ytemp = ytemp(aa)
              if xtemp(0) gt a.range(0, xval-1, iblock) then begin
                xtemp = [a.range(0, xval-1, iblock)-10*xerr, xtemp]
                ytemp = [ytemp(0), ytemp]
              endif
              np = n_elements(xtemp)
              if xtemp(np-1) lt a.range(1, xval-1, iblock) then begin
                xtemp=[xtemp,  a.range(1, xval-1, iblock)+xerr*10]
                ytemp = [ytemp, ytemp(np-1)]
              endif
              yst = xx
              yst = get_yarr(xtemp, ytemp, yst, /no_truncate)
            endif
          endif
          cont = y_gridinteg (cont, yy, yst, yinteg)
          if nconnect gt 0 then begin
            icon = where (connect(1, *) eq iblock, nicon)
            if nicon gt 0 then begin
              ny = n_elements(yy)
              for i  = 0, nicon-1 do begin
                connp(icon(i)) = nx
                conx (0:nx-1, icon(i)) = xx
                conv (0:nx-1, icon(i)) = cont(*, ny-1)
              endfor
            endif
          endif
          if ntconnect gt 0 then begin
            itcon = where (tconnect(1, *) eq iblock or tconnect(2,*) $
              eq iblock , nitcon)
            if nitcon gt 0 then begin
            for i = 0, nitcon-1 do begin
              if tconnect(1,itcon(i)) eq iblock then $
                tblock = tconnect(2, itcon(i))
              if tconnect(2,itcon(i)) eq iblock then $
                tblock = tconnect(1, itcon(i))
              nb = (where (tblock eq blockarray))(0)
              if nb lt 0 then begin
                print,'Error in calculating transverse blocks',nb,tblock,iblock
                return, 0.0
              endif
              if nb gt ii then begin
                contx(*, itcon(i)) = a.range (*, xval -1, iblock)
                contv(*,itcon(i))=[get_y(yy,cont(0,*),a.range(0,yval-1,tblock) $
                 ), get_y(yy,cont(nx-1,*),a.range(0,yval-1,tblock))]
              endif
             endfor
            endif
          endif
        endif
        a.v1(a.locv(iblock-1):a.locv(iblock)-1)  =  cont(0:nx*ny-1)
      endfor
      return, a
     endif

;
      if n_con_save gt 0  and nblock gt 2 then begin
        if keyword_set(xinteg)  then errval = xerr else errval = yerr
;           if two blocks begin at the same point and one has no connection
;             make sure the first one is the one with the connection with a
;             higher block
        arr = -1
        for j = 0, nblock -2 do begin
          ok = where (save_con(1, *) eq blockarray(j), nok)
          if nok gt 0 then arr = [arr , reform (save_con(2, ok), nok)]
          ok = where (save_con(2, *) eq blockarray(j), nok)
          if nok gt 0 then arr = [arr , reform (save_con(1, ok), nok)]
          if n_elements(arr) gt 1 then repeat begin
            noswap = 1
            for i = j+1, nblock-2 do begin
              if abs(a.range(1, intval-1, blockarray(i)) -  $
                a.range(1, intval-1, blockarray(i+1))) lt  errval then begin
                ok1 = where (blockarray(i+1) eq arr, nok1)
                ok  = where (blockarray(i)   eq arr, nok)
                if nok1 gt 0 and nok eq 0 then begin
                   noswap = 0
                   t = blockarray(i) & blockarray(i) = blockarray(i+1)
                   blockarray(i+1) = t
                endif
              endif
            endfor
          endrep until noswap
        endfor
      endif
;
; print, 'blocks ', blockarray
      for ii = 0,  nblock-1  do begin
        iblock = blockarray(ii)
        cont = reform( get_array (a, fld, iblock, lab = lab))
;       if smoothing is desired - do it!
        if smoo gt 0 then cont = smooth(cont, smoo>3)
;
        xx = get_array (a, xval, iblock)
        yy = get_array (a, yval, iblock)
        ny = n_elements(yy)
        nx = n_elements(xx)
        if keyword_set(xinteg) then begin
          dely = (yy(ny-1)-yy(0))*epsilon
          yst = 0
          extent(*, ii)=[yy(0)-yerr , yy(ny-1)+yerr]
          if start gt 0 and xx(nx-1) gt xmax-xerr then begin
            yst = yy
            yst(0)= yst(0)+dely
            yst(ny-1)=yst(ny-1)-dely
            yst = get_yarr(xstart, ystart, yst, /no_truncate)
            if n_elements(yst) ne ny then begin
              Print, '% INTSTR: Error Getting start position, block', iblock
              yst = make_array(ny, value = 0.0)
            endif
          endif else if nconnect gt 0 or ntconnect gt 0 then begin
            xtemp = 0 & ytemp = 0
            if nconnect gt 0 then $
            icon = where (connect(1, *) eq iblock, nicon) else nicon = 0
;debug if nicon gt 0 then print, connect(*, icon)
            if ntconnect gt 0 then $
            itcon = where (tconnect(1, *) eq iblock or tconnect(2,*) $
              eq iblock , nitcon)  else nitcon = 0
            if start gt 0 then begin
              ok = where (yy ge startmin and yy le startmax, nystemp)
              if nystemp gt 0 then begin
                yyt= yy
                yyt(0) = yyt(0) + dely
                yyt(ny-1) = yyt(ny-1) -dely
                ystemp= yyt(ok)
              endif
            endif else nystemp = 0
            if nicon gt 0 then begin
              npt = connp(icon(0))
              xtemp = [0,conx(0:npt-1,icon(0))]
              ytemp = [0,conv(0:npt-1,icon(0))]
              if nystemp gt 0 then ok = where (ystemp lt conx(0,icon(0))-yerr $
                 or ystemp gt conx(npt-1, icon(0))+yerr, nystemp)
              if nystemp gt 0 then ystemp = ystemp(ok)
              if nicon gt 1 then begin
                for i = 1, nicon-1 do begin
                  npt = connp(icon(i))
                  xtemp = [xtemp, conx(0:npt-1, icon(i))]
                  ytemp = [ytemp, conv(0:npt-1, icon(i))]
                  if nystemp gt 0 then ok=where(ystemp lt conx(0,icon(i))-yerr $
                     or ystemp gt conx(npt-1, icon(i))+yerr, nystemp)
                  if nystemp gt 0 then ystemp = ystemp(ok)
                endfor
              endif
            endif
            if nitcon gt 0 then begin
              for i = 0, nitcon-1 do begin
                if tconnect(1,itcon(i)) eq iblock then $
                  tblock = tconnect(2, itcon(i))
                if tconnect(2,itcon(i)) eq iblock then $
                  tblock = tconnect(1, itcon(i))
                nb = (where (tblock eq blockarray))(0)
                if nb lt 0 then begin
                  print,'Error in calculating transverse blocks',nb,tblock,iblock
                  return, 0.0
                endif
;                add both the upper and lower values for the block.
                if nb lt ii then begin
                  xtemp = [xtemp, contx(*,itcon(i))]
                  ytemp  = [ytemp, contv(*,itcon(i))]
                  if nystemp gt 0 then ok=where(abs(ystemp - $
                    contx(0,itcon(i))) -yerr gt 0.0 , nystemp )
                  if nystemp gt 0 then ystemp = ystemp(ok)
                  if nystemp gt 0 then ok=where(abs(ystemp - $
                    contx(1,itcon(i))) -yerr gt 0.0 , nystemp )
                  if nystemp gt 0 then ystemp = ystemp(ok)
                endif
              endfor
            endif
            if nystemp gt 0 then begin
              if ii gt 0 then begin
                i = 0
                while i lt ii and nystemp gt 0 do begin
                  ok = where(ystemp lt extent(0, i) or ystemp gt extent(1, i), $
                         nystemp)
                  if nystemp gt 0 then ystemp = ystemp(ok)
                  i = i+1
                endwhile
              endif
              if nystemp gt 0 then begin
                xtemp = [xtemp, ystemp]
                ystemp = get_yarr(xstart, ystart, ystemp, /no_truncate, /sort)
                ytemp = [ytemp, ystemp]
              endif
            endif
            if n_elements(xtemp) gt 1 then begin
              xtemp = xtemp(1:*) & ytemp = ytemp(1:*)
              aa = sort (xtemp)
              xtemp= xtemp(aa)
              ytemp = ytemp(aa)
              if xtemp(0) gt a.range(0, yval-1, iblock) then begin
                xtemp = [a.range(0, yval-1, iblock)-10*yerr, xtemp]
                ytemp = [ytemp(0), ytemp]
              endif
              np = n_elements(xtemp)
              if xtemp(np-1) lt a.range(1, yval-1, iblock) then begin
                xtemp=[xtemp,  a.range(1, yval-1, iblock)+yerr*10]
                ytemp = [ytemp, ytemp(np-1)]
              endif
              yst = yy
              yst = get_yarr(xtemp, ytemp, yst, /no_truncate, /sort)
            endif
          endif
          cont = x_gridinteg (cont, xx, yst, xinteg, /reverse)
          if nconnect gt 0 then begin
            icon = where (connect(2, *) eq iblock, nicon)
            if nicon gt 0 then begin
              nx = n_elements(xx)
              for i  = 0, nicon-1 do begin
                connp(icon(i)) = ny
                conx (0:ny-1,icon(i)) = yy
                conv (0:ny-1,icon(i)) = cont(0,*)
              endfor
            endif
          endif
          if ntconnect gt 0 then begin
            itcon = where (tconnect(1, *) eq iblock or tconnect(2,*) $
              eq iblock , nitcon)
            if nitcon gt 0 then begin
              for i = 0, nitcon-1 do begin
                if tconnect(1,itcon(i)) eq iblock then $
                  tblock = tconnect(2, itcon(i))
                if tconnect(2,itcon(i)) eq iblock then $
                  tblock = tconnect(1, itcon(i))
                nb = (where (tblock eq blockarray))(0)
                if nb lt 0 then begin
                  print,'Error in calculating transverse blocks',nb,tblock,iblock
                  return, 0.0
                endif
                if nb gt ii then begin
                  contx(*, itcon(i)) = a.range (*, yval -1, iblock)
                  contv(*,itcon(i))= $
                     [get_y(xx,cont(*,0),a.range(1,xval-1,tblock)), $
                      get_y(xx,cont(*,ny-1),a.range(1,xval-1,tblock))]
                endif
              endfor
            endif
          endif
;
        endif else  begin
          delx =(xx(nx-1)-xx(0))*epsilon
          yst = 0
          extent(*, ii) = [xx(0)-xerr, xx(nx-1)+xerr]
          if start gt 0 and yy(0) lt ymin+yerr then begin
            yst = xx
            yst(0) = yst(0)+delx
            yst(nx-1) = yst(nx-1)-delx
            yst = get_yarr(xstart, ystart, yst, /no_truncate)
            if n_elements(yst) ne nx then begin
              Print, '% INTSTR: Error Getting start position, block', iblock
              yst = make_array(ny, value = 0.0)
            endif
          endif else if nconnect gt 0  or ntconnect gt 0  then begin
            xtemp = 0 & ytemp = 0
            if nconnect gt 0 then $
            icon = where (connect(1, *) eq iblock, nicon) else nicon = 0
            if ntconnect gt 0 then $
            itcon = where (tconnect(1, *) eq iblock or tconnect(2,*) $
              eq iblock , nitcon) else nitcon = 0
            if start gt 0 then begin
              ok = where (xx ge startmin and xx le startmax, nystemp)
              if nystemp gt 0 then begin
                xxt= xx
                xxt(0) = xxt(0) + delx
                xxt(nx-1) = xxt(nx-1) -delx
                ystemp= xxt(ok)
              endif
              if nystemp gt 0 then ystemp= xx(ok)
            endif else nystemp = 0
            if nicon gt 0 then begin
              npt = connp(icon(0))
              xtemp = [0,conx(0:npt-1,icon(0))]
              ytemp = [0,conv(0:npt-1,icon(0))]
              if nystemp gt 0 then ok = where (ystemp lt conx(0,icon(0))-xerr $
                 or ystemp gt conx(npt-1, icon(0))+xerr, nystemp)
              if nystemp gt 0 then ystemp = ystemp(ok)
              if nicon gt 1 then begin
                for i = 1, nicon-1 do begin
                  npt = connp(icon(i))
                  xtemp = [xtemp, conx(0:npt-1, icon(i))]
                  ytemp = [ytemp, conv(0:npt-1, icon(i))]
                  if nystemp gt 0 then ok=where(ystemp lt conx(0,icon(i))-xerr $
                     or ystemp gt conx(npt-1, icon(i))+xerr, nystemp)
                  if nystemp gt 0 then ystemp = ystemp(ok)
                endfor
              endif
            endif
            if nitcon gt 0 then begin
              for i = 0, nitcon-1 do begin
                if tconnect(1,itcon(i)) eq iblock then $
                  tblock = tconnect(2, itcon(i))
                if tconnect(2,itcon(i)) eq iblock then $
                  tblock = tconnect(1, itcon(i))
                nb = (where (tblock eq blockarray))(0)
                if nb lt 0 then begin
                  print,'Error in calculating transverse blocks',nb,tblock,iblock
                  return, 0.0
                endif
                if nb lt ii then begin
                  xtemp = [xtemp, contx(*,itcon(i))]
                  ytemp  = [ytemp, contv(*,itcon(i))]
                  if nystemp gt 0 then ok=where(abs(ystemp - $
                    contx(0,itcon(i))) -xerr gt 0.0 , nystemp )
                  if nystemp gt 0 then ystemp = ystemp(ok)
                  if nystemp gt 0 then ok=where(abs(ystemp - $
                    contx(1,itcon(i))) -xerr gt 0.0 , nystemp )
                  if nystemp gt 0 then ystemp = ystemp(ok)
                endif
              endfor
            endif
            if nystemp gt 0 then begin
              if ii gt 0 then begin
                i = 0
                while i lt ii and nystemp gt 0 do begin
                  ok = where(ystemp lt extent(0, i) or ystemp gt extent(1, i), $
                         nystemp)
                  if nystemp gt 0 then ystemp = ystemp(ok)
                  i = i+1
                endwhile
              endif
              if nystemp gt 0 then begin
                xtemp = [xtemp, ystemp]
                ystemp = get_yarr(xstart, ystart, ystemp, /no_truncate)
                ytemp = [ytemp, ystemp]
              endif
            endif
            if n_elements(xtemp) gt 1 then begin
              xtemp = xtemp(1:*) & ytemp = ytemp(1:*)
              aa = sort (xtemp)
              xtemp= xtemp(aa)
              ytemp = ytemp(aa)
              if xtemp(0) gt a.range(0, xval-1, iblock) then begin
                xtemp = [a.range(0, xval-1, iblock)-10*xerr, xtemp]
                ytemp = [ytemp(0), ytemp]
              endif
              np = n_elements(xtemp)
              if xtemp(np-1) lt a.range(1, xval-1, iblock) then begin
                xtemp=[xtemp,  a.range(1, xval-1, iblock)+xerr*10]
                ytemp = [ytemp, ytemp(np-1)]
              endif
              yst = xx
              yst = get_yarr(xtemp, ytemp, xx, /no_truncate)
            endif
          endif
          cont = y_gridinteg (cont, yy, yst, yinteg, /reverse)
          if nconnect gt 0 then begin
            icon = where (connect(2, *) eq iblock, nicon)
            if nicon gt 0 then begin
              for i  = 0, nicon-1 do begin
                connp(icon(i)) = nx
                conx (0:nx-1, icon(i)) = xx
                conv (0:nx-1, icon(i)) = cont(*, 0)
              endfor
            endif
          endif
          if ntconnect gt 0 then begin
            itcon = where (tconnect(1, *) eq iblock or tconnect(2,*) $
              eq iblock , nitcon)
            if nitcon gt 0 then begin
            for i = 0, nitcon-1 do begin
              if tconnect(1,itcon(i)) eq iblock then $
                tblock = tconnect(2, itcon(i))
              if tconnect(2,itcon(i)) eq iblock then $
                tblock = tconnect(1, itcon(i))
              nb = (where (tblock eq blockarray))(0)
              if nb lt 0 then begin
                print,'Error in calculating transverse blocks',nb,tblock,iblock
                return, 0.0
              endif
              if nb gt ii then begin
                contx(*, itcon(i)) = a.range (*, xval -1, iblock)
                contv(*,itcon(i))=[get_y(yy,cont(0,*),a.range(1,yval-1,tblock) $
                 ), get_y(yy,cont(nx-1,*),a.range(1,yval-1,tblock))]
              endif
             endfor
            endif
          endif
        endelse
        a.v1(a.locv(iblock-1):a.locv(iblock)-1)  =  cont(0:nx*ny-1)
      endfor
      return, a

     end
