; ------------------------------------------------------------------------ 
; $Id: joinstr.pro,v 1.8 2011/06/06 21:19:01 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
 Pro  joinstr, wdf1, wdf2, wdf3, clabel = clabel
;
;+
; NAME:
;       JOINSTR
;
; PURPOSE:
;       Join a field structure array to another field structure array 
;         (See UNIONSTR to actually add blocks which overlap.)
;         No check is made to verify blocks are consistent.  All of the blocks
;         in each structure are simply combined into a common structure.
;  <or> Join a particle structure to another particle structure by 
;         adding the particles of the second structure to the end of the
;         particles in the first structure.  Each structure must have the same
;         number of dimensions and the same number of attributes.
;         No check is made to verify attributes are consistent.  All of
;         the attributes in each structure are simply combined into a common 
;         structure. If only one of the structures is degenerate, then the 
;         structure will be made 3-D with no degenerate dimensions.
;
; For both fields and particles structures:
;       Store the result in the initial field structure array or 
;         another structure array
;       All attributes or vector quantities are joined.
;       
; CATEGORY:
;       Math 
; CALLING SEQUENCE:
;       JOINSTR, STR1, STR2,  [, STR3]
; INPUTS:
;       STR1:    array number or structure of the first field structure array
;       STR2:    array number or structure of field structure to be joined to 
;                the first structure
; OPTIONAL INPUT:
;       STR3:    array number for the combination, if not specified the 
;                 combination is stored in STR1 (the first array)
; KEYWORDS:
;       CLABEL:  Optional label for the combined structures.
; OUTPUTS:
;       NONE    The sum is stored in a structure array in the common block
; COMMON BLOCKS:
;       COM.DAT  WDF common block 
; RESTRICTIONS:
;       STR1 and STR2 array must be defined and have valid data
;       STR1 and STR2 must both be the same type of structures (Particle / Field)
;       STR1 and STR2 must have the same number of dimensions and 
;         the same number of attributes or vector dimensions.
; PROCEDURE:
;       Blocks of STR1 are combined with the blocks of STR2
;       The combination of blocks is stored in STR3, if specified, or STR1, 
;         if STR3 is not specified.
; EXAMPLES:
;       Join structure arrays 2 and 3 and store the result in structure 8
;               JOINSTR, 2, 3, 8
;       Join structure arrays 3 and 8 and store the result in structure 10
;        with a dataset comment, 'Total Charge - Line + Diode'
;               JOINSTR, 3, 8, 10, c=  'Total Charge - Line + Diode'
; MODIFICATION HISTORY:
;      Original version, L. P. Mix, March 8, 1994
;      Modified to include get_valid_structure()
;      Modified to join particle structures, January 6, 2004
;      Modified to allow different degenerate dimensions for Particle data.
;        June 2, 2011
;-
;
; insert a common block
;
@qscom.dat
;  Common block
@qsstrcom.dat
;       return on an error
      on_error, 1
;
      if n_elements (structinit) eq 0 then qsstruct_init
      if n_elements (qsplotinit) eq 0 then qsinit
;
      nparams = n_params()
      if nparams lt 2 then begin
        print,' JOINSTR must have at least 2 parameters'
        return
      endif
;
      array1 = get_valid_structure( wdf1, 'joinstr', lcstr = lcstr, ierror=ier)
      if ier ne 0 then return
       array2 = get_valid_structure( wdf2, 'joinstr', ierror = ier)
      if ier ne 0 then return

      if array1.type ne 1 or array2.type ne 1 then begin
        if array1.type ne 2 or array2.type ne 2 then begin
          Print, '% JOINSTR: Both structure arrays must be either'
          Print, '           field(1) or particle(2) structures'
          print, '  Structure types: ', strtrim(array1.type, 1), ', ', $
            strtrim(array2.type, 1)
          return
        endif else type = 2
      endif else type = 1
    if type eq 1 then begin
      if array1.ndim ne array2.ndim then begin
        Print,'Both structure arrays must be have the same number of dimensions'
        return
      endif
      if array1.nvect ne array2.nvect then begin
        Print, 'Both structure arrays must have the same number of vector components'
        return
      endif
      if array1.two ne array2.two then begin
        Print, 'Both structure arrays must have the same degenerate dimension'
        return
      endif
      block = array1.block+array2.block
      v1lab = [ array1.v1lab, array2.v1lab]
      v2lab = [ array1.v2lab, array2.v2lab]
      v3lab = [ array1.v3lab, array2.v3lab]
      v4lab = [ array1.v4lab, array2.v4lab]
      v5lab = [ array1.v5lab, array2.v5lab]
      v6lab = [ array1.v6lab, array2.v6lab]
      x1lab = [ array1.x1lab, array2.x1lab]
      x2lab = [ array1.x2lab, array2.x2lab]
      x3lab = [ array1.x3lab, array2.x3lab]
      x4lab = [ array1.x4lab, array2.x4lab]
      x1 = [ array1.x1, array2.x1]
      x2 = [ array1.x2, array2.x2]
      x3 = [ array1.x3, array2.x3]
      x4 = [ array1.x4, array2.x4]
      v1 = [ array1.v1, array2.v1]
      v2 = [ array1.v2, array2.v2]
      v3 = [ array1.v3, array2.v3]
      v4 = [ array1.v4, array2.v4]
      v5 = [ array1.v5, array2.v5]
      v6 = [ array1.v6, array2.v6]
      nb1 = array1.block
      nb2 = array2.block
      two = array1.two
      ndim = array1.ndim
      locx = reform(array1.locx, n_elements(array1.locx))
      locxx = array2.locx(*, 1:*)
      for i = 1, nb2 do locxx(*,i-1) = array2.locx(*, i)+array1.locx(*, nb1)
      locx = [locx, reform(locxx, n_elements(locxx))]
      locx = reform (locx, ndim, block+1)
      locv = [array1.locv, array1.locv(nb1)+array2.locv(1:*)]
      range = reform(array1.range, n_elements(array1.range))
      locxx = array2.range(* , *, 1:*)
      range = [range, reform(locxx, n_elements(locxx))]
      range = reform (range,2, ndim, block+1)
      for i = 0, ndim-1 do begin
        range(0, i, 0)=min(range(0,i,1:block))
        range(1, i, 0)= max(range(1,i,1:block))
      endfor
      if block gt 1 then begin
        order = make_array(ndim, block, /long)
        for i = 0, ndim-1 do order (i, *) = sort(range(0,i, 1:block))
      endif else order = 1L

      size = reform(array1.size, n_elements(array1.size))
      size = [size, reform(array2.size, n_elements(array2.size))]
      size = reform (size, ndim, block)
;
;     take care of the spare array
;
      spare = array1.spare
      ss = size(spare)
      if ss[1] gt 5 then $ 
        print, '% JOINSTR: Spare array 1 will be truncated to 5 elements per block.'
      ncopy = ss[1] < 5 
      newspare = make_array(5, block, /long)
      if ss[0] eq 1 then $
        for i = 0, nb1 -1 do newspare[0, i] = spare[0:ncopy-1] $
        else  $
        newspare[0:ncopy-1, 0:nb1 -1] = spare[0:ncopy-1,0:nb1 -1]
      spare = array2.spare
      ss= size(spare)
      if ss[1] gt 5 then $
        print, '% JOINSTR: Spare array 2 will be truncated to 5 elements per block.'
      ncopy = ss[1] < 5
      nb2 = nb2 < ss[2]
      if ss[0] eq 1 then $
        for i = nb1, block -1 do newspare[0, i] = spare[0:ncopy-1] $
        else  $
        newspare[0:ncopy-1, nb1:block -1] = spare[0:ncopy-1, 0:nb2 -1]

      dx1 = make_array(block, value =0.0)
      dx2 = make_array(block, value =0.0)
      dx3 = make_array(block, value =0.0)
      dx4 = make_array(block, value =0.0)

      array1 = {type:1L, ndim:ndim, nvect:array1.nvect, $
              block:block, two:two, size:size, range: range,$
              order: order, slice:array1.slice, $
              locx:locx, locv:locv, x1:x1, x2: x2, $
              x3:x3, x4:x4, v1:v1, v2:v2, v3:v3, v4:v4, v5:v5, v6:v6, $
              dx1: dx1, dx2: dx2, dx3: dx3, dx4: dx4, $
              x1lab : x1lab,  x2lab : x2lab, x3lab :x3lab, x4lab :x4lab,  $
              v1lab : v1lab, v2lab :v2lab, v3lab:v3lab, $
              v4lab : v4lab, v5lab :v5lab, v6lab:v6lab, $
              file: array1.file, spare: newspare, tlab : array1.tlab, $
              clab:array1.clab }
;
    endif else begin
;     particle data
      if array1.natt ne array2.natt then begin
        print, '% JOINSTR: Number of attributes must be the same to '
        print, '           join structures', array1.natt, array2.natt
        return
      endif
      if array1.ndim ne array2.ndim then begin
        print, '% JOINSTR: Number of dimension must be the same to '
        print, '           join structures', array1.ndim, array2.ndim
        return
      endif
      ttwo = array1.two
      if array1.two ne array2.two then ttwo = 0

      array1 = {type:2L, ndim: array1.ndim, natt: array1.natt, block: 1L, $ 
              nvert: array1.nvert+array2.nvert, $
              two:ttwo, x1: [array1.x1, array2.x1],  $
              x2:  [array1.x2, array2.x2], x3:  [array1.x3, array2.x3], $
              x4:  [array1.x4, array2.x4], v1:  [array1.v1, array2.v1], $
              v2:  [array1.v2, array2.v2], v3:  [array1.v3, array2.v3], $
              v4:  [array1.v4, array2.v4], v5:  [array1.v5, array2.v5], $
              v6:  [array1.v6, array2.v6], v7:  [array1.v7, array2.v7], $
              v8:  [array1.v8, array2.v8], x1lab: array1.x1lab, $
              x2lab : array1.x2lab, x3lab : array1.x3lab, $
              x4lab : array1.x4lab, v1lab : array1.v1lab, $
              v2lab : array1.v2lab, v3lab : array1.v3lab, $
              v4lab : array1.v4lab, v5lab : array1.v5lab, $
              v6lab : array1.v6lab, v7lab : array1.v7lab, $
              v8lab : array1.v8lab, file : array1.file,  $
              spare : array1.spare, tlab : array1.tlab, clab: array1.clab }
    endelse
      if n_elements(clabel) gt 0 then array1.clab = string(clabel)
      if nparams gt 2 then begin
        if lcstr ne 0 then wdf3 = temporary(array1)  else begin
          save = get_save_str(wdf3, wdf1)
          i2s, array1, save, /no_copy
        endelse
      endif else begin
        if lcstr ne 0 then wdf1 = temporary(array1) else $
          i2s, array1, wdf1, /no_copy
      endelse
;
      array2=0
      end
