; ------------------------------------------------------------------------
; $Id: linstr.pro,v 1.28 2011/10/05 14:16:48 lpmix Exp $
; ------------------------------------------------------------------------
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
;+
; NAME:
;       LINSTR
;
; PURPOSE:
;       Generate lineouts from any 2-D structure
;       Single block data generally uses the actual data for the lineout.
;       Multiple block data is put on a uniform grid with 1 pixel resolution.
;       Procedure will generate lineouts and display them. Lineouts use
;         the existing grid in one direction. The X or Y values are used
;         depending on which direction provides the most values between the
;         designated end points. Lineouts must have a minimum of 3 values.
;         Distances are slant distances from one end point.
;       When designated, the lineouts will be stored in WDF arrays.
;       Horizontal and Vertical lineouts require a single point.
;       A preview gives lineout before storage. A preview will be generated
;         whenever the mouse moves in the window and one point has been
;         selected for a general lineout or if no points are selected for a
;         vertical or horizontal lineout.
;       Note: Some time is required to calculate a preview frame. All events
;             generated while the preview frame is generated are deleted; be
;             sure the mouse clicks are acknowledged before proceeding.
;       Mouse buttons are mapped to widget buttons, cursor must be in the
;         DRAW area of the main widget for the following mouse commands.
;          Left Button   = Set Value
;              Note: A carriage return in the data field will also set the value.
;          Middle Button = Accept/Save
;             (current designated lineout will be saved)
;          Right Button  = Reset All Points
;             (All points will be undesignated)
;          Note: If a the last point required to define a curve is selected
;                more than once, the previous "last point" is replaced.
;       Set Point Buttons: These perform the same function as a <CR> in either
;                          the X or Y windows above the buttons.
;
; CALLING SEQUENCE:
;   LINSTR, STR [, WDF] [, X, Y]
;   where:
;       STR   - Structure array number of a two dimensional structure or
;               a slice from three dimensional structure.
;               The structure should be single block for best results. Multi-
;               block structures will be put on a uniform 256 by 256 grid.
;               To use an interpolation with N elements use:
;                 PLOTSTR, STR, NFARRAY = N, RESULT = STR2
;                 LINSTR, STR2, wdf
;               Note: Data will not be transposed.
;       WDF   - Initial WDF array number to store the lineout or an array of values to
;               store the lineouts.  If WDF is a single value, successive WDF numbers
;               will be used for additional lineouts.
;               If WDF is undefined, an array of unused WDF arrays is used.
;               If all arrays are occupied, then wdf = indgen(get_maxwdf()) + 1 
;               and if only one free wdf array exists successive WDF arrays
;               will be made available for lineouts (overwriting some existing data).
;               If two or more values are presented, you will be limited to that number
;               of lineouts.
;               WDF can also be set manually in the widget.  The WDF array determined
;               as noted above will be modified as noted below:
;               If a value is set in the LINSTR widget, it will be used.  If it already
;               exists in the initial WDF array then it is removed.  The next number
;               suggested in the widget is the next higher value in the initial array of 
;               wdf numbers. If no higher numbers exist, then the first unused number 
;               will be used.
;         Note: WDF must be specified if X and Y are specified or it will be set
;                     to the default. No widget is provided.
;       X, Y  - 2N X-values and 2N Y-values if neither horizontal nor verticle
;               is set. Lineouts of arbitrary pairs of points may be obtained
;               directly where N is the number of lineouts.
;   or
;       X     - N values corresponding to Y or X values if horizontal or verticle
;               is set, respectively.
;
; KEYWORD PARAMETERS:
;       XVALUE: This keyword is an alternate way to specify X above.
;               Note: XVALUE must be specified if wdf is not specified.
;               Note: XVALUE is ignored if X is specified.
;       YVALUE: This keyword is an alternate way to specify Y above.
;               Note: YVALUE is ignored if Y is specified.
;       ORDERX:     If set, lineouts are from Minimum X to Maximum X.
;                   Default = 0 ; from first point to second point
;       Horizontal: If present and not zero, then a full width horizontal
;                   lineout is taken. Can also be set manually.
;                   Only one point must be designated.
;       Vertical:   If present and not zero, then a full height vertical
;                   lineout is taken. Can also be set manually.
;                   Only one point must be designated.
;       PEXTRA:     PEXTRA is a structure.  The tags and values in this 
;                     structure are passed to the plot command for the preview
;                     frame.  Structure tags can be any keyword for PLOT.
;       LINEAR:     If set and if (MULTIPLY[0] eq 0) then the lineout will be with the original
;                     data and the image will be a log scale.  See PLOTFLD to see the 
;                     effect of the MULTIPLY keyword.
;       Preview:    If present and zero, no preview will be generated.
;                   If positive, preview will have a fixed scale equal to the
;                     total range of data.
;                   If PREVIEW has two elements, these will be used as the
;                     y-range of the preview frame.
;                   Default is preview = -1 => Auto scale preview.
;        Note: Preview frame may be resized.
;       PXRANGE:    Xrange for the preview frame.
;                   Default is the range of data.
;       Width:      In current version, a horizontal or vertical lineout will
;                   use the actual specified width at each Y or X value.
;                   NOTE:  The indicated width for ROTATE and NORMAL are slightly wider than
;                          for EXACT because the averaging for exact extends 1/2 step on
;                          each side of the indicated width.
;                   Arbitrary Slope Lineout
;                 If EXACT is set:
;                   Generate  grid for the lineout.  If the grid has N grid points in width,
;                   the width of each grid is the width divided by N.  The grid is rotated
;                   and the image is evaluated at each grid point.  This process is faster 
;                   than ROTATE, but large arrays will take a fraction of a second.
;                 If ROTATE is set:
;                   Rotate the image for a horizontal lineout.  The width is exactly as
;                   specified.  This process is very slow, but the rotated structure is
;                   available in GET_MAXSTR()-1.
;                 If EXACT or ROTATE is not set:
;                   The width is modified to
;                      delta_x = W/sin(SlopeAngle)  or
;                      delta_y = W/cos(SlopeAngle)
;                   All values are taken at a fixed value of X or Y.
;       EXACT:      This is ignored for horizontal lineouts, vertical lineouts, or if WIDTH=0.
;                   The basic element of this technique was suggested by Tim Webb.
;                   To obtain the lineout, a secondary grid is generated with spacing equal to 
;                   the minimum of DX, DY of the structure.  The grid may be reduced by the
;                   keyword, GRID. This grid is rotated to the lineout
;                   and the values of the structure are obtained for each grid point.  
;       CUBIC:      If EXACT is set, this value will be passed to INTERPOLATE.
;                   No checking of the value is performed.
;       ROTATE:     ROTATE is ignored for horizontal lineouts, vertical lineouts, or if WIDTH=0.
;                   ROTATE is also ignored if EXACT is set.
;                   If set and WIDTH is greater than zero, the arbitrary slope lineouts 
;                   are rotated to horizontal and the lineout width is exact perpendicular
;                   to lineout direction.
;                   ROTSTR is used to rotate the image and store the rotated image
;                   in "GET_MAXSTR() - 1".  The keyword, GRID, is passed to ROTSTR to 
;                   reduce the size of the rectilinear mesh.
;                   DEFAULT: If ROTATE is undefined, ROTATE =0
;                   NOTE:  Abscissa is the distance from the first point.
;                   NOTE:  ROTATE can take seconds for large arrays.  Better time response 
;                   is obtained with zero width or by rotating the entire structure for
;                   horizontal or verticle lineouts. 
;       GRID:       This is ignored unless EXACT or ROTATE is used.  This keyword is passed
;                   to ROTSTR to reduce the size of the rectilinear mesh and used by EXACT
;                   to set the size of the mesh.
;                   If GRID is undefined, gt 1 or le 0 then GRID = 1.
;                   If GRID is in the range (0:1), then the minimum point spacing is
;                     GRID * MIN([dx, dy])
;       Median:     If set, and Width is not zero, then a median filter
;                   will be taken between lower and upper limits.
;       Integral:   If set, and Width is not zero, then an integral
;                   will be taken between lower and upper limits.
;       SET_DEFAULT: If set, the default values for the following keywords will be set.
;                   These values will be remembered for future calls to LINST.
;                 EXACT:  If present then EXACT_SAVE = EXACT; DEFAULT EXACT_SAVE =0
;                   In future calls to LINSTR then:
;                     If (N_ELEMENTS(EXACT) eq 0) and (N_ELEMENTS(ROTATE) eq 0) then EXACT = EXACT_SAVE
;                 CUBIC:  If present then CUBIC_SAVE = CUBIC
;                   In future calls to LINSTR then:
;                     If (N_ELEMENTS(CUBIC) eq 0) then CUBIC = CUBIC_SAVE
;       SHOW_DEFAULT: If set, show default values for EXACT_SAVE and CUBIC_SAVE.
;       WDFarray:   Array of WDF arrays where lineouts are stored.
;                   If no values are stored, then WDFarray = 0
;       Xonly:      Return the abscissa values only not the length along the
;                   lineout, ie the wdfarray will be amplitude vs x axis value.
;                   Xonly is ignored if delta X/max(X)< 1e-4
;       Yonly:      Return the ordinate values only not the length along the
;                   lineout, ie the wdfarray will be amplitude vs y axis value.
;                   Yonly is ignored if delta Y/max(Y)< 1e-4
;       SLOPEAngle: Angle in degrees of the slope of the last lineout
;                   Result of arc tangent with two arguements.
;       CPRINT:     If set, print the dataset comment for each lineout
;                   to the terminal
;       IFLAG:      If set, the first point will be set to the nearest X or Y
;                     value.  If width is not zero, spatial averaging will
;                     still occur.
;                     IFLAG = 1, First point set to nearest X value
;                     IFLAG = 2, First point set to nearest Y value
;                     IFLAG = 3, First point set to nearest X and Y value
;       PLAST:      Coordinates of first point of the last lineout selected
;       QLAST:      Coordinates of second point of the last lineout selected
;       _EXTRA:     Any valid keyword for plotfld. The array for the lineout
;                   is generated with the following command and this plot is
;                   used for the image:
;          PLOTFLD, str, nfarray=256, xarray = x, yarray = y, farray = array, $
;                 nlev = 29,  _extra = extrastuff
;         Note: nfarray is ignored with single block data
;
; SIDE EFFECTS:
;   If ROTATE is set and the WIDTH is not zero, then GET_MAXSTR()-1 will be used
;   for the rotated image and GET_MAXWDF()-1 will be used for the lineout.
;
; EXAMPLE:
;   Take a series of lineouts from structure array 3 and store the lineouts
;         beginning in wdf array 11.
;           LINSTR, 3, 11
;
;   Take a series of lineouts from structure array 3 and store the lineouts
;         beginning in wdf array 11; Set the preview to fixed scaling.
;           LINSTR, 3, 11, /preview
;     or    LINSTR, 3, 11, /prev
;
;       Take a series of lineouts from structure array 1 and plot the lineouts
;         12 to a page.
;           LINSTR, 1, wdf= wdf
;           plo, wdf, g = 12
;
;       Take a radial lineout from (0, 0) to (3, 3) and store the result
;         in wdfarray 4 from structure 82
;
;         LINSTR, 82, 4, [0, 3], [0, 3]
;
; MODIFICATION HISTORY:
;   Written by:  L. P. Mix, August 27, 1996
;                Added width options to all lineout types, March 16, 2005 LPM
;                Added EXACT and ROTATE keywords, November 11, 2010 LPM
;                Modified selection of WDF array numbers, December 16, 2010 lpm
;-


PRO linstr_event, Event
COMPILE_OPT hidden
on_error, 1

  WIDGET_CONTROL,Event.Id,GET_UVALUE=Ev

  print, '%LINSTR_EVENT: EVENT UVALUE = ', Ev
  help, /str, event
  END

 Pro linstr_doit
COMPILE_OPT hidden
common linstr_data_com, xx, yy, array, x, y, dx, dy, xdraw_range, ydraw_range, $
   nx, ny, mode, px, py, qx, qy, strfile, slopesav, widsav, xline_range, $
   pxran, orderx, iflag_lin, dx1, dx2, xwid, ywid, dxm, dym, medsetting, $
   avesetting, arrc, exactc, gridc, rotatec, deltamin, deltas, cubicc
common linstr_widget_com, xwindow1, ywindow1, xwindow2, ywindow2, widwindow, $
   bset1, bset2, wdfwindow, image, draw, draw_id, line_id, oldbx, oldby, $
   print_var, oldbp, npselect, base21, base7, base33, clabel, xlabel, ylabel, npwdf,$
   wdfarr, indep_var, ypreview, xpreviewsize, x2label, base22, tlb_offset, $
   line_offset, xsize, ysize, avewindow, avemethod, plot_extra
;
; Calculate a lineout of array between points p and q.
;
 on_error, 1
  widget_control, xwindow1, get_value = px
  widget_control, ywindow1, get_value = py
  if keyword_set(iflag_lin) then begin
    if iflag_lin eq 1 or iflag_lin eq 3 then begin
      junk = min(abs(x-px), ijk)
      px = x[ijk]
    endif
    if iflag_lin eq 2 or iflag_lin eq 3 then begin
      junk = min(abs(y-py), ijk)
      py = y[ijk]
    endif
  endif
  widget_control, xwindow1, set_value = px
  widget_control, ywindow1, set_value = py
  if mode eq 0 then begin
    widget_control, xwindow2, get_value = qx
    widget_control, ywindow2, get_value = qy
    widget_control, xwindow2, set_value = qx
    widget_control, ywindow2, set_value = qy
  endif
  widget_control, widwindow, get_value = widsav
  widget_control, widwindow, set_value = widsav
  widget_control, avewindow, get_value = medsetting
  widget_control, avemethod, get_value = avesetting
    
; be sure values are floats
  get_linstr_line

 if n_elements(xx) gt 2  then begin
   if line_id  ge 0 then begin
     if strupcase(strmid(!version.os, 0, 3)) ne 'MAC' then begin
       device, window_state = wins
       xsiz = xpreviewsize(0)
       ysiz = xpreviewsize(1)
       if n_elements(wins) lt line_id +1 then begin
         window, /free, xsize= xsiz, ysize = ysiz, title = 'Current Lineout', $
           xpos = line_offset[0], ypos= line_offset[1]
         line_id = !d.window
       endif else if wins(line_id) eq 0 then begin
         window, /free, xsize= xsiz, ysize = ysiz, title = 'Current Lineout', $
           xpos = line_offset[0], ypos= line_offset[1]
         line_id = !d.window
       endif else begin
         wset, line_id
         device, get_window_position = line_offset
         xpreviewsize[0] = !d.x_size
         xpreviewsize[1] = !d.y_size
       endelse

     endif else wset, line_id
     if n_elements(pxran) gt 1 then xline_range= pxran
     if n_elements(ypreview) eq 2 then plot, xx, yy, yrange = ypreview, $
       xrange = xline_range, _extra = plot_extra else $
       plot, xx, yy, xrange = xline_range,  _extra = plot_extra
   endif
   wset, draw_id
   !x = oldbx
   !y = oldby
   !p = oldbp
 endif
 device, copy = image
 widget_control, draw, /clear_events
 if mode eq 0 then oplot, [px, qx], [py, qy], psym = -8, thick = 2,col = 7 $
   else $
   if mode eq 1 then $
      oplot, !x.crange, [py, py], thick = 2, col =7 $
      else  $
     oplot, [px, px], !y.crange, thick = 2, color = 7
 if widsav gt 0 and n_elements(xx) gt 2 then begin
   if mode eq 1 then begin
;    horizontal lineout
     oplot, !x.crange, [py, py]+widsav*0.5, thick = 1, $
         linestyle = 2, col =7
     oplot, !x.crange, [py, py]-widsav*0.5, thick = 1, $
         linestyle = 2, col =7
   endif else if mode eq 2 then begin
;    vertical lineout
     oplot, [px, px]+widsav*0.5, !y.crange, thick = 1, $
         linestyle = 2, col =7
     oplot, [px, px]-widsav*0.5, !y.crange, thick = 1, $
         linestyle = 2, col =7
   endif else begin
;    arbitrary slope
     oplot, xwid[*, 0], ywid[*, 0], thick = 1, $
         linestyle = 2, col =7
     oplot, xwid[*, 1], ywid[*, 1], thick = 1, $
         linestyle = 2, col =7
   endelse
 endif
 end

pro get_linstr_line
COMPILE_OPT hidden
  common linstr_data_com
  common linstr_widget_com
on_error, 1
 val = medsetting
 exactc = 0 & rotatec = 0
 if avesetting gt 0 then if avesetting eq 1 then exactc = 1 else rotatec = 1
 if mode eq 1 then begin
;  horizontal
   xx = x
   xline_range =xdraw_range
   if val eq 1 then begin
     ii = 0 & im = 1
   endif else if val eq 2 then begin
     ii = 1 & im = 0
   endif
   yy = xlineout(array, y, py, widsav, median = im, integral = ii)
   slopesav = 0.0
 endif else if mode eq 2 then begin
;  vertical
   if val eq 1 then begin
     ii = 0 & im = 1
   endif else if val eq 2 then begin
     ii = 1 & im = 0
   endif
   xx = y
   xline_range =ydraw_range
   yy = ylineout (array, x, px, widsav, median = im, integral = ii)
   slopesav = 90.0

 endif else if keyword_set(exactc) and (widsav gt 0) then begin
   deltax = qx - px
   deltay = qy - py
   slopesav  = atan (deltay, deltax)
   length = sqrt(deltax*deltax + deltay*deltay) 
   cosangle = cos(slopesav)
   sinangle = sin(slopesav)
   nwidth = round(widsav/deltamin)
   nwidth = (nwidth le 1) ? 1 : (nwidth /2L)*2+1L
;   stepy = (nwidth gt 1) ? widsav/(nwidth-1) : 0.0
   stepy = (nwidth gt 1) ? widsav/(nwidth) : 0.0
   n = floor(length/deltamin) > 2
   np = n+1
   stepx = length/n
;  generate the lineout grid
   Xa = make_array(Np, nwidth, /float)
   row = findgen(Np)* stepx
   for i = 0, nwidth-1 do Xa[*, i] = row

   Ya = make_array(Np, nwidth, /float)
   col = (findgen(nwidth) -nwidth/2)* stepy
   for j =0, N do Ya[j, *] = col
;  rotating X and Y to get interpolating arrays IX and IY:
   xx = Xa*cosangle-Ya*sinangle+Px
   yy = Xa*sinangle+Ya*cosangle+Py
   if nwidth eq 1 then begin
     xwid  = [xx[0, 0], xx[n, 0],xx[0, 0], xx[n, 0]]
     ywid  = [yy[0, 0], yy[n, 0],yy[0, 0], yy[n, 0]]
   endif else begin
     xwid = [xx[0, 0], xx[n, 0], xx[0, nwidth-1], xx[n, nwidth-1]]
     ywid = [yy[0, 0], yy[n, 0], yy[0, nwidth-1], yy[n, nwidth-1]]
   endelse
   xwid = reform(xwid, 2, 2, /over)
   ywid = reform(ywid, 2, 2, /over)
   iix = (dx1 eq 0) ? findex(x, xx) : (xx-x[0])/dx1
   iiy = (dx2 eq 0) ? findex(y, yy) : (yy-y[0])/dx2
   averaged_values = interpolate(array, iix, iiy, cubic =cubicc)
   if nwidth gt 1 then begin
     if keyword_set(median) then averaged_values = median(averaged_values, dim =2) else begin
        averaged_values = total(averaged_values, 2, /nan)
        if not keyword_set(integral) then averaged_values /= nwidth
     endelse
   endif
   xx = row
   yy = averaged_values 
   clab = 'Lineout of '+clabel+': ('+strtrim(px,2)+','+strtrim(py,2)+ $
           ') to ('+strtrim(qx,2)+','+strtrim(qy,2)+'): Exact_Width = '+strtrim(widsav, 2)
   if indep_var gt 0 then begin
     if indep_var eq 1 then begin
       if abs(qx - px) gt 1.0e-4*max(abs([px, qx])) then begin
         fac = cos(slopesav)
         stepx *= fac
         tzero = px
         xl = xlabel+'(X-axis)'
         xx = fac*xx + tzero
       endif else begin
         tzero = 0.0
         xl = 'Distance'
       endelse
     endif else begin
;      indep_var =2
       if abs(qy - py) gt 1.0e-4*max(abs([py, qy])) then begin
         fac = sin(slopesav)
         stepx *= fac
         tzero = py
         xl = x2label +'(Y-axis)'
         xx = fac*xx + tzero
       endif else begin
         tzero = 0.0
         xl = 'Distance'
       endelse
     endelse
   endif else begin
     tzero = 0.0
     xl = 'Distance'
   endelse
   i2w, [tzero, stepx], yy, get_maxwdf()-1, clab = clab, xlab = xl, $
     ylab= arrc.v1lab, file = arrc.file
   xline_range = [min(xx, max = mm), mm]
;  want to change the rotation, but rotstr uses negative angle
   slopesav = slopesav * !radeg
 endif else if keyword_set(rotatec) and (widsav gt 0) then begin
   slopesav  = atan (qy-py, qx-px)
;  protect the common values
   points = [px, py, qx, qy]
   rotstr, arrc, slopesav, /radian, arr_rot, center =[px, py], grid = gridc, $
     points = points, /exact
   xex = arr_rot.x1
   yex = arr_rot.x2
   arrayex = get_array(arr_rot, -1)
   if val eq 1 then begin
     ii = 0 & im = 1
   endif else if val eq 2 then begin
     ii = 1 & im = 0
   endif
   eps = (xdraw_range[1] -xdraw_range[0])* 4.76837e-07 
   px0 = points[0]
   points[0:1] =    points[0:1] + [-eps , +eps]  
   ok = where (xex ge points[0] and xex le points[1], nok)
   if nok eq 0 then begin
     xx = 0
     yy = 0
     delwdf , get_maxwdf()-1
     return
   endif
   yrot = total(points[2:3])*0.5
   yy = xlineout(arrayex, yex, yrot, widsav, median = im, integral = ii)
   yy = yy[ok]
   xx = (xex[ok] - px0) > 0
   i2s, arr_rot, get_maxstr()-1, /no_copy
   clab = 'Lineout of '+clabel+': ('+strtrim(px,2)+','+strtrim(py,2)+ $
           ') to ('+strtrim(qx,2)+','+strtrim(qy,2)+'): Rotate_Width = '+strtrim(widsav, 2)
   cc = cos(slopesav) 
   ss = sin(slopesav)
   xl = 'Distance'
   if indep_var gt 0 then begin
     if indep_var eq 1 then begin
       if abs(qx - px) gt 1.0e-4*max(abs([px, qx])) then begin
         xl = xlabel+'(X-axis)'
         xx = cc*xx + px
       endif
     endif else begin
;      indep_var =2
       if abs(qy - py) gt 1.0e-4*max(abs([py, qy])) then begin
         xl = x2label +'(Y-axis)'
         xx = ss*xx + py
       endif
     endelse
   endif 
   i2w, xx, yy, get_maxwdf()-1, /xydat, clab = clab, xlab = xl, $
     ylab= arrc.v1lab, file = arrc.file
   xline_range = [min(xx, max = mm), mm]
   xxwid = [points[0], points[1], points[0], points[1]] - points[0]
   yywid = +0.5*widsav*[1, 1, -1, -1] 
;  want to change the rotation, but rotstr uses negative angle
   xwid = xxwid*cc - yywid*ss+ px
   ywid = yywid*cc + xxwid*ss+ py
   xwid = reform (xwid, 2, 2, /over)
   ywid = reform (ywid, 2, 2, /over)
   slopesav = slopesav * !radeg
 endif else begin
;  arbitrary slope but exact and rotate are not set or  width = 0.0
   slopesav  = atan (qy-py, qx-px)
;  find the number of X values
   if px le qx then begin
     xswitch = 0
     okx = where (x ge px and x le qx, nokx)
   endif else begin
     xswitch = 1
     okx = where (x ge qx and x le px, nokx)
   endelse
;  find the number of x values that have a valid y
   if nokx gt 0 then begin
     if qx eq px then nokx = 0 else begin
       xx = x(okx)
       ytrue = py + (qy-py)/(qx-px)*(xx-px)
       okxx = where (ytrue ge y(0) and ytrue le y(ny-1), nokx)
     endelse
   endif
   if py le qy then begin
     yswitch = 0
     oky = where (y ge py and y le qy, noky)
   endif else begin
     yswitch = 1
     oky = where (y ge qy and y le py, noky)
   endelse
   if noky gt 0 then begin
     if py eq qy then noky = 0 else begin
       yy = y(oky)
       xtrue = px + (qx-px)/(qy-py)*(yy-py)
       okyy = where (xtrue ge x(0) and xtrue le x(nx-1), noky)
     endelse
   endif
;  now we have found the number of values possible in each direction
   if nokx lt 3 && noky lt 3 then begin
     xx = 0.0
     yy = 0.0
     slopesav = slopesav * !radeg
     return
   endif
   if nokx ge noky then begin
;    wid = delta_y value
     wid = ( widsav gt 0) ? abs(0.5*widsav/cos(slopesav)) : 0.0
     slopesav = slopesav * !radeg
     xx = xx(okxx)
     xwid = reform([xx, xx], nokx, 2)
     ytrue = ytrue(okxx)
     okx = okx(okxx)
     xxx = 0
     if  indep_var gt 0 then begin
       if indep_var eq 1 then begin
         if (abs(xx(nokx-1) -xx(0))) gt 1.0e-4*max(abs(xx)) then xxx = xx
       endif else begin
         if (abs(ytrue(nokx-1) -ytrue(0))) gt 1.0e-4*max(abs(ytrue)) $
            then xxx = ytrue
        endelse
     endif
     if (n_elements(xxx) lt 3) then begin
       if xswitch eq 0 or not keyword_set(orderx) then begin
         xx = xx-px
         ty = ytrue-py
       endif else begin
         xx = xx-qx
         ty = ytrue-qy
       endelse
       xxx = sqrt (xx*xx+ty*ty)
     endif
     indy = (findex(y, ytrue) > 0) < (ny-1)
     if widsav eq 0 then begin
       yceil = ceil(indy)
       yfloor = floor(indy)
       yy = array[okx, yfloor]*(yceil-indy) + array[okx, yceil]*(indy-yfloor)
       ok_eq = where (yceil eq yfloor, nok)
       if nok gt 0 then yy[ok_eq] = array[okx[ok_eq], yfloor[ok_eq]]
     endif else begin
       ytop = (ytrue + wid) < y[ny-1]
       ybot = (ytrue - wid) > y[0]

       ywid = reform([ytop, ybot], nokx, 2)
       ytop = findex(y, ytop)
       ybot = findex(y, ybot)
       if val eq 1 then begin
;        median filter
         ytop = round(ytop)
         ybot = round(ybot)
         yy = array[okx, ybot]
         ok = where ((ytop-ybot) gt 0, nok)
         if nok gt 0 then $
           for i = 0, nok-1 do yy[ok[i]] = median(array[okx[ok[i]], ybot[ok[i]]: ytop[ok[i]]])
       endif else begin
;        integral or mean value
         yceil = ceil(indy)
         yfloor = floor(indy)
         yy = array[okx, yfloor]*(yceil-indy) + array[okx, yceil]*(indy-yfloor)
         ok_eq = where (yceil eq yfloor, nok)
         if nok gt 0 then yy[ok_eq] = array[okx[ok_eq], yfloor[ok_eq]]
         w= ytop-ybot
         yy = yy*w
         ytop = ytop+0.5
         ybot = ybot+0.5
         ytfloor = floor(ytop)
         ybfloor = floor(ybot)
         ytceil = ceil(ytop)
         ybceil = ceil(ybot)

         ss = where ((ytfloor-ybfloor) gt 1, nok)
         if nok gt 0 then for j = 0, nok-1 do begin
           i = ss[j]
           wi = (ybceil[i]-ybot[i])*dym[ybfloor[i]]
           wj = (ytop[i]-ytfloor[i])*dym[ytfloor[i]]
           yy[i]= array[okx[i], ybfloor[i]]*wi +  array[okx[i], ytfloor[i]]*wj
           w[i]= wi+wj
           for k = ybfloor[i]+1, ytfloor[i]-1 do $
             yy[i] = yy[i]+array[okx[i], k]* dym[k]
           w[i] = w[i]+total(dym[ybfloor[i]+1: ytfloor[i]-1])
         endfor
         if val eq 0 then yy = yy/w
       endelse
     endelse
   endif else begin
;    wid = delta_x value
     wid = ( widsav gt 0) ? abs(0.5*widsav/sin(slopesav)) : 0.0
     slopesav = slopesav * !radeg
     yy = yy(okyy)
     ywid = reform([yy, yy], noky, 2)
     xtrue = xtrue(okyy)
     oky = oky(okyy)
     xxx = 0
     if  indep_var gt 0 then begin
       if indep_var eq 2 then begin
         if (abs(yy(noky-1) -yy(0))) gt 1.0e-4*max(abs(yy)) then xxx = yy
       endif else begin
         if (abs(xtrue(noky-1) -xtrue(0))) gt 1.0e-4*max(abs(xtrue)) $
            then xxx = xtrue
        endelse
     endif
     if (n_elements(xxx) lt 3) then begin
       if xswitch eq 0 or not keyword_set(orderx) then begin
         xx = xtrue-px
         yy = yy- py
       endif else begin
         xx = xtrue-qx
         yy = yy- qy
       endelse
       xxx = sqrt (yy*yy+xx*xx)
     endif
     indx = (findex(x, xtrue) > 0) < (nx-1)
     if widsav eq 0 then begin
       xceil = ceil(indx)
       xfloor = floor(indx)
       yy = array[ xfloor, oky]*(xceil-indx) + array[xceil, oky]*(indx-xfloor)
       ok_eq = where (xceil eq xfloor, nok)
       if nok gt 0 then yy[ok_eq] = array[xfloor[ok_eq], oky[ok_eq]]
     endif else begin
       xtop = (xtrue + wid) < x[nx-1]
       xbot = (xtrue - wid) > x[0]

       xwid = reform([xtop, xbot], noky, 2)
       xtop = findex(x, xtop)
       xbot = findex(x, xbot)
       if val eq 1 then begin
;        median filter
         xtop = round(xtop)
         xbot = round(xbot)
         yy = array[xbot, oky]
         ok = where ((xtop-xbot) gt 0, nok)
         if nok gt 0 then $
           for i = 0, nok-1 do $
           yy[ok[i]] = median(array[xbot[ok[i]]: xtop[ok[i]],oky[ok[i]]])
       endif else begin
;        integral or mean value
         xceil = ceil(indx)
         xfloor = floor(indx)
         yy = array[xfloor, oky]*(xceil-indx) + array[xceil, oky]*(indx-xfloor)
         ok_eq = where (xceil eq xfloor, nok)
         if nok gt 0 then yy[ok_eq] = array[xfloor[ok_eq], oky[ok_eq]]
         w = xtop-xbot
         yy = w * yy
         xtop = xtop+0.5
         xbot = xbot+0.5
         xtfloor = floor(xtop)
         xbfloor = floor(xbot)
         xtceil = ceil(xtop)
         xbceil = ceil(xbot)


         ss = where ((xtfloor-xbfloor) gt 1, nok)
         if nok gt 0 then for j = 0, nok-1 do begin
           i = ss[j]
           wi = (xbceil[i]-xbot[i])*dxm[xbfloor[i]]
           wj = (xtop[i]-xtfloor[i])*dxm[xtfloor[i]]
           yy[i]= array[xbfloor[i], oky[i]]*wi +  array[xtfloor[i], oky[i]]*wj
           w[i]= wi+wj
           for k = xbfloor[i]+1, xtfloor[i]-1 do $
             yy[i] = yy[i]+array[k, oky[i]]* dxm[k]
           w[i] = w[i]+total(dxm[xbfloor[i]+1: xtfloor[i]-1])
         endfor
         if val eq 0 then yy = yy/w
       endelse
     endelse
   endelse
   xx = temporary(xxx)
   xline_range = [min(xx, max = mm), mm]
 endelse
 end

 pro widstr_point_event, event
COMPILE_OPT hidden
  common linstr_data_com
  common linstr_widget_com

  WIDGET_CONTROL,Event.Id,GET_UVALUE=Ev
  widget_control, widwindow, get_value = widsav
  widget_control, widwindow, set_value = widsav
  if (npselect ge 2) then linstr_doit else $
    if ((npselect ge 1) && (mode gt 0)) then linstr_doit
 end


 pro linstr_point_event, event
;
COMPILE_OPT hidden
  on_error, 1
  common linstr_data_com
  common linstr_widget_com

  WIDGET_CONTROL,Event.Id,GET_UVALUE=Ev
  CASE EV OF
   'DRAW':  Begin
            if event.press eq 4 then linstr_reset_button else begin
              if event.press eq 2 then begin
                if mode eq 0 then begin
                  if npselect eq 2 then save_button_event, event else $
                    winformation, 'Two points must be selected for a lineout'
                endif else begin
                  if npselect ge 1 then save_button_event, event else $
                    winformation, 'One point must be selected for a lineout'
                endelse
              endif else begin
                if event.press eq 1 then begin
                  coord = convert_coord(event.x, event.y, /device, /to_data)
                  if ((npselect eq 0) or (npselect ge 1 and mode gt 0)) then begin
                    widget_control, xwindow1, set_value = coord(0)
                    widget_control, ywindow1, set_value = coord(1)
                    npselect = 1 > npselect
                    if mode gt 0 then begin
                      linstr_doit
                      widget_control, base7, map =1
                      widget_control, draw, /clear_events, $
                        draw_motion_events = 0
                    endif else $
                      widget_control, draw, draw_motion_events=1, /clear_events
                  endif else begin
                    widget_control, xwindow2, set_value = coord(0)
                    widget_control, ywindow2, set_value = coord(1)
                    npselect = 2
                    linstr_doit
                    widget_control, base7, map =1
                    widget_control, draw, /clear_events, draw_motion_events = 0
                  endelse
                endif else if event.release eq 0 then begin
                  coord = convert_coord(event.x, event.y, /device, /to_data)
                  if npselect eq 0 then begin
                    widget_control, xwindow1, set_value = coord(0)
                    widget_control, ywindow1, set_value = coord(1)
                    if mode gt 0 then linstr_doit
                  endif else begin
                    widget_control, xwindow2, set_value = coord(0)
                    widget_control, ywindow2, set_value = coord(1)
                    linstr_doit
                  endelse
                endif else widget_control, draw, /clear_events
              endelse
            endelse
            END
  'XVALUE1': LINSTR_HANDLE_1
  'YVALUE1': LINSTR_HANDLE_1
  'BUTTON1': LINSTR_HANDLE_1
  'XVALUE2': LINSTR_HANDLE_2
  'YVALUE2': LINSTR_HANDLE_2
  'BUTTON2': LINSTR_HANDLE_2
  ELSE:   Begin
          print, ev
          help, /str, event
          END
  END
  return
  end

 Pro linstr_handle_1
COMPILE_OPT hidden
  on_error, 1
  common linstr_data_com
  common linstr_widget_com
  widget_control, xwindow1, get_value = px
  widget_control, ywindow1, get_value = py
  if keyword_set(iflag_lin) then begin
    if iflag_lin eq 1 or iflag_lin eq 3 then begin
      junk = min(abs(x-px), ijk)
      px = x[ijk]
    endif
    if iflag_lin eq 2 or iflag_lin eq 3 then begin
      junk = min(abs(y-py), ijk)
      py = y[ijk]
    endif
  endif

  widget_control, xwindow1, set_value = px
  widget_control, ywindow1, set_value = py
  if ((npselect eq 0) or (npselect ge 1 and mode gt 0)) then begin
     npselect = 1 > npselect
     if mode gt 0 then begin
       linstr_doit
       widget_control, base7, map =1
       widget_control, draw, /clear_events, draw_motion_events = 0
     endif else $
       widget_control, draw, draw_motion_events = 1, /clear_events
  endif else begin
     if npselect eq 2 then begin
       linstr_doit
       widget_control, base7, map =1
       widget_control, draw, /clear_events, draw_motion_events = 0
     endif
  endelse
  end

 Pro linstr_handle_2
COMPILE_OPT hidden
   on_error, 1
  common linstr_data_com
  common linstr_widget_com
  widget_control, xwindow1, get_value = px
  widget_control, ywindow1, get_value = py
  widget_control, xwindow2, get_value = qx
  widget_control, ywindow2, get_value = qy

  if (npselect eq 0) then begin
    if keyword_set(iflag_lin) then begin
      if iflag_lin eq 1 or iflag_lin eq 3 then begin
        junk = min(abs(x-qx), ijk)
        qx = x[ijk]
      endif
      if iflag_lin eq 2 or iflag_lin eq 3 then begin
        junk = min(abs(y-qy), ijk)
        qy = y[ijk]
      endif
    endif
    widget_control, xwindow1, set_value = qx
    widget_control, ywindow1, set_value = qy
    widget_control, xwindow2, set_value = px
    widget_control, ywindow2, set_value = py
    npselect = 1
    if mode gt 0 then begin
       linstr_doit
       widget_control, base7, map =1
       widget_control, draw, /clear_events, draw_motion_events = 0
    endif else widget_control, draw, /clear_events, draw_motion_events = 1
  endif else begin
    widget_control, xwindow2, set_value = qx
    widget_control, ywindow2, set_value = qy

    npselect = 2
    linstr_doit
    widget_control, base7, map =1
    widget_control, draw, /clear_events, draw_motion_events = 0
  endelse
  end

pro line_slope_event, event
COMPILE_OPT hidden
  on_error, 1
  common linstr_data_com
  common linstr_widget_com
  mode = event.value
  if mode eq 0 then begin
    if npselect lt 2 then widget_control, base7, map =0
    widget_control, base33, map = 1
    if npselect eq 0 or npselect eq 2 then begin
      widget_control, draw, /clear_events, draw_motion_events = 0
      if npselect eq 2 then linstr_doit else device, copy = image
    endif else  begin
      widget_control, draw, /clear_events, draw_motion_events = 1
      device, copy = image
    endelse
    widget_control, base21, map =1
  endif else begin
    widget_control, base7, map = npselect
    if npselect ge 1 then begin
      widget_control, draw, /clear_events, draw_motion_events = 0
      linstr_doit
    endif else begin
      widget_control, draw, /clear_events, draw_motion_events = 1
      device, copy = image
    endelse
    widget_control, base21, map =0
    widget_control, base33, map = 0
  endelse
  end


pro line_method_event, event
COMPILE_OPT hidden
  on_error, 1
  common linstr_data_com
  common linstr_widget_com
  avesetting = event.value
  if npselect eq 2 then linstr_doit
  end

Pro save_button_event, event
COMPILE_OPT hidden
  common linstr_data_com
  common linstr_widget_com
@com.dat
  widget_control, wdfwindow, get_value = wdf
  if n_elements(xx) lt 3 then begin
    winformation, 'Lineout must contain at least 3 points'
  endif else begin
    if (keyword_set(exactc) or keyword_set(rotatec)) and (mode eq 0) and $
      (widsav gt 0) then begin
      xfr, maxwdfarray-1, wdf
      clab = wdcomment(wdf)
    endif else begin
      if mode eq 1 and dx1 gt 0.0 then begin
        xydata = 0
        temp = [xx[0], dx1, yy]
      endif else if mode eq 2 and dx2 gt 0.0 then begin
        xydata = 0
        temp = [xx[0], dx2, yy]
      endif else begin
        temp = [xx, yy]
        xydata = 1
      endelse
      if mode eq 0 then begin
        if ((px lt qx) or not keyword_set(orderx)) then $
        clab = 'Lineout of '+clabel+': ('+strtrim(px,2)+','+strtrim(py,2)+ $
             ') to ('+strtrim(qx,2)+','+strtrim(qy,2)+')' else $
        clab = 'Lineout of '+clabel+': ('+strtrim(qx,2)+','+strtrim(qy,2)+ $
             ') to ('+strtrim(px,2)+','+strtrim(py,2)+')'
        if widsav gt 0 then clab = clab+': Width = '+strtrim(widsav, 2)
      endif else begin
        if mode eq 1 then $
        clab = 'Lineout of '+clabel+': Y = '+strtrim(py,2) else $
        clab = 'Lineout of '+clabel+': X = '+strtrim(px,2)
        if widsav gt 0 then clab = clab+': Width = '+strtrim(widsav, 2)
      endelse
      if mode eq 2 then $
        put_wdfarray, temp, wdf, clab = clab, xlab = x2label, ylab = ylabel, $
          file = strfile, xydata = xydata else $
        put_wdfarray, temp, wdf, clab = clab, xlab = xlabel, ylab = ylabel, $
          file = strfile, xydata = xydata
    endelse
    if print_var ne 0 then print, clab
    if wdfarr[npwdf] ne wdf  then begin
      temp = wdfarr[npwdf:*]
      wdfarr[npwdf] = wdf
      wdfarr = wdfarr[0:npwdf]
      npwdf += 1
      ok = where (wdf gt temp, nok, ncomp = ncomp, comp = comp)
      if nok gt 0 then begin
        ok = temp[ok]
        if nok gt 1 then ok = ok[sort(ok)]
      endif
      if ncomp gt 0 then begin
        comp= temp[comp]
        if ncomp gt 1 then comp = comp[sort(comp)]
        if comp[0] eq wdf then begin
          ncomp -=1
          if ncomp gt 0 then comp = comp[1:*]
        endif
      endif
      if ncomp gt 0 then wdfarr = [wdfarr, comp]
      if nok gt 0 then wdfarr = [wdfarr, ok]
    endif else npwdf += 1
    if npwdf ge n_elements(wdfarr) then begin
      winformation, $
        [' Maximum number of lineouts has been reached', $
         ' Please exit and return before taking additional lineouts'] 
      if strupcase(strmid(!version.os, 0, 3)) ne 'MAC' then begin
        device, window_state = wins
        xsiz = xpreviewsize(0)
        ysiz = xpreviewsize(1)
        if n_elements(wins) ge line_id then begin
          if wins(line_id) gt 0 then begin
            wset, line_id
            device, get_window_position = line_offset
            xpreviewsize[0] = !d.x_size
            xpreviewsize[1] = !d.y_size
            wdelete, line_id
          endif
        endif
      endif
      arrc = 0
      widget_control, event.top, tlb_get_offset = tlb_offset
      widget_control, event.top, /destroy
      return
    endif
    wdf = wdfarr[npwdf]
    widget_control, wdfwindow, set_value = wdf
    linstr_reset_button
  endelse
  END

pro linstr_reset_button
; Handle the actual reset
COMPILE_OPT hidden
  common linstr_data_com
  common linstr_widget_com
    npselect = 0
    if line_id gt 0 then begin
      wset, line_id
      erase
    endif
    wset, draw_id
    device , copy = image
    !x = oldbx
    !y = oldby
    !p = oldbp
    widget_control, draw, /clear_events
   widget_control, base7, map = 0
   if mode gt 0 then $
      widget_control, draw, /clear_events, draw_motion_events = 1 $
   else widget_control, draw, /clear_events, draw_motion_events = 0

  END

pro reset_button_event, event
;
; Handle the reset and the exit buttons
COMPILE_OPT hidden
; First RESET
  on_error, 1
  common linstr_data_com
  common linstr_widget_com
  if event.value eq 0 then linstr_reset_button else begin
    if (mode eq 0 and npselect eq 2) or (mode gt 0 and npselect gt 0) then begin
      if n_elements (xx) gt 2 then $
        wsave_nosave, lcquit, [ 'There is a LINEOUT defined', $
          'Do you want to save it before EXIT'], 'LINSTR EXIT', $
           group = event.top else lcquit = 1
      if lcquit eq 0 then save_button_event
    endif
    if strupcase(strmid(!version.os, 0, 3)) ne 'MAC' then begin
      device, window_state = wins
      xsiz = xpreviewsize(0)
      ysiz = xpreviewsize(1)
      if n_elements(wins) ge line_id then begin
        if wins(line_id) gt 0 then begin
          wset, line_id
          device, get_window_position = line_offset
          xpreviewsize[0] = !d.x_size
          xpreviewsize[1] = !d.y_size
          wdelete, line_id
        endif
      endif
    endif
    arrc = 0
    widget_control, event.top, tlb_get_offset = tlb_offset
    widget_control, event.top, /destroy
  endelse
  end
 
  pro linstr_batch, arr, wdf, xin, yin, median = median, integral=integral, $
    multiply = multiply, linear = linear, _extra = extrastuff
COMPILE_OPT hidden
;
@com.dat
@qscom.dat
;  Common block
@qsstrcom.dat
;       return on an error
  common linstr_data_com
  common linstr_widget_com
      on_error, 1

      if mode eq 2 then xlabel = arr.x2lab else xlabel = arr.x1lab
      if arr.ndim eq 3 and arr.two ne 3 then begin
        if arr.two eq 1 then begin
          if mode eq 2 then xlabel = arr.x3lab else xlabel = arr.x2lab
        endif else if mode eq 2 then xlabel = arr.x1lab else xlabel = arr.x3lab
      endif
      ylabel = arr.v1lab
      strfile = arr.file
      oldwin = !d.window
      initbx = !x
      initby = !y
      initbp = !p
      xsize = 700
      ysize = 560
      npselect = 0
      slopesav=0
      if n_elements(multiply) gt 0 and keyword_set(linear) then $
        log_image = (multiply[0] eq 0) ? 1 : 0 else log_image = 0
      if arr.block gt 1 or arr.type ne 1 or keyword_set(log_image) then begin
        window, /free , /PIXMAP  , xsize = xsize, ysize = ysize
        pixwin = !d.window
        if arr.block gt 1 then $
            Print, '%LINSTR: Structure will be put on a single block'
  if log_image then begin
    plotfld, arr, farray = array, xarray= x, yarray= y, iflag=iflag, $
       nfarray = 256,cntour = 2,  _extra = extrastuff
    plotfld, arr, farray = array1, xarray= x, yarray= y, iflag=iflag, $
       nfarray = 256,cntour = 2,  multiply = multiply, _extra = extrastuff
    array1 =0
  endif else plotfld, arr, farray = array, xarray= x, yarray= y, iflag=iflag, $
       nfarray = 256,cntour = 2,  multiply = multiply, _extra = extrastuff
        nx = n_elements(x)
        ny = n_elements(y)
        dx = x(1:nx-1)-x(0:nx-2)
        dxm = [dx[0], (dx[0:nx-3]+dx[1:nx-2])*0.5, dx[nx-2]]
        dy = y(1:ny-1)-y(0:ny-2)
        dym = [dy[0], (dy[0:ny-3]+dy[1:ny-2])*0.5, dy[ny-2]]

        ok = where(strpos(tag_names(arr), 'DX') ge 0 , nok)
        if nok eq 0 then begin
          dx1 = 0
          dx2 = 0
        endif else begin
          if xoldvalue eq 1 then dx1 = arr.dx1[0] else $
            if xoldvalue eq 2 then dx1 = arr.dx2[0] else $
            if xoldvalue eq 3 then dx1 = arr.dx3[0] else dx1 = 0
          if yoldvalue eq 1 then dx2 = arr.dx1[0] else $
            if yoldvalue eq 2 then dx2 = arr.dx2[0] else $
            if yoldvalue eq 3 then dx2 = arr.dx3[0] else dx2 = 0
        endelse

        if oldwin gt 0 then  begin
          wset, oldwin
          !x = initbx
          !y = initby
          !p = initbp
        endif
        wdelete, pixwin
        if n_elements (array) lt 4 then return
      endif else begin
        trans=0
        if arr.ndim eq 2  or arr.two eq 3 then begin
          xval =1  & yval = 2
        endif else if arr.two eq 2 then begin
          xval = 3 & yval = 1 & trans=1
        endif else if arr.two eq 1 then begin
          xval =2  & yval = 3
        endif else begin
          Print, 'Error with getting data(two, ndim): ', arr.two, arr.ndim
          return
        endelse

        ok = where(strpos(tag_names(arr), 'DX') ge 0 , nok)
        if nok eq 0 then begin
          dx1 = 0
          dx2 = 0
        endif else begin
          if xval eq 1 then dx1 = arr.dx1[0] else $
            if xval eq 2 then dx1 = arr.dx2[0] else $
            if xval eq 3 then dx1 = arr.dx3[0] else dx1 = 0
          if yval eq 1 then dx2 = arr.dx1[0] else $
            if yval eq 2 then dx2 = arr.dx2[0] else $
            if yval eq 3 then dx2 = arr.dx3[0] else dx2 = 0
        endelse
        x = get_array(arr, xval)
        y = get_array(arr, yval)
        nx = n_elements(x)
        ny = n_elements(y)
        array = reform(get_array(arr, -1), /overwrite)
        if trans ne 0 then array = transpose(array)
        dx = x(1:nx-1)-x(0:nx-2)
        dxm = [dx[0], (dx[0:nx-3]+dx[1:nx-2])*0.5, dx[nx-2]]
        dy = y(1:ny-1)-y(0:ny-2)
        dym = [dy[0], (dy[0:ny-3]+dy[1:ny-2])*0.5, dy[ny-2]]
      endelse

      if mode eq 0 then begin
; 2 points per lineout
        nelx = n_elements(xin)
        nely = n_elements(yin)
        nlin = min([nelx, nely])/2
        if nlin lt 1 then begin
          Print, '% LINSTR: Lineout must have at least 2 X values and 2 Y values.'
          return
        endif
        if keyword_set(exactc) or keyword_set(rotatec) then arrc = arr
        for i = 0, nlin-1 do begin
          px = xin(i+i)
          py = yin(i+i)
          if keyword_set(iflag_lin) then begin
            if iflag_lin eq 1 or iflag_lin eq 3 then begin
              junk = min(abs(x-px), ijk)
              px = x[ijk]
            endif
            if iflag_lin eq 2 or iflag_lin eq 3 then begin
              junk = min(abs(y-py), ijk)
              py = y[ijk]
            endif
          endif
          qx = xin(i+i+1)
          qy = yin(i+i+1)
          get_linstr_line
          if n_elements(xx) lt 3 then begin
            print, 'Lineout must contain at least 3 points'
          endif else begin
            npwdf += 1
            if (keyword_set(rotatec) or keyword_set(exactc)) and (mode eq 0) $
              and (widsav gt 0) then begin
              xfr, maxwdfarray-1, wdf
            endif else begin
              if ((px lt qx) or not keyword_set (orderx)) then $
               clab = 'Lineout of '+clabel+': ('+strtrim(px,2)+','+strtrim(py,2)+ $
                 ') to ('+strtrim(qx,2)+','+strtrim(qy,2)+')' else $
               clab = 'Lineout of '+clabel+': ('+strtrim(qx,2)+','+strtrim(qy,2)+ $
                 ') to ('+strtrim(px,2)+','+strtrim(py,2)+')'
              temp = [xx, yy]
              if widsav gt 0 then clab = clab+', Width='+strtrim(widsav, 2)
              put_wdfarray, temp, wdf, clab = clab, xlab = 'Distance', ylab = arr.v1lab, $
                file = strfile, /xydata
            endelse
            if npwdf ge n_elements(wdfarr) then begin
              print, 'Maximum number of lineouts has been reached'
              print, '  Please exit and return before taking additional lineouts'
              return
            endif 
            wdf = wdfarr[npwdf]
          endelse
        endfor
      endif else if mode eq 1 then begin
;       horizontal
        nelx = n_elements(xin)
        if nelx lt 1 then begin
          Print, '% LINSTR: Horizontal lineout must have at least one Y value.'
          return
        endif
        xx = x
        for i = 0, nelx-1 do begin
          py = xin(i)
          if iflag_lin eq 2 or iflag_lin eq 3 then begin
            junk = min(abs(y-py), ijk)
            py = y[ijk]
          endif
          yy = xlineout(array,y, py, widsav, median = median, integral = integral)
          if n_elements(xx) lt 3 then begin
            print, 'Lineout must contain at least 3 points'
          endif else begin
            npwdf += 1
            clab = 'Lineout of '+clabel+': Y = '+strtrim(py,2)
            if dx1 gt 0.0 then begin
              xydata = 0
              temp = [xx[0], dx1, yy]
            endif else begin
              temp =[xx, yy]
              xydata = 1
            endelse
            put_wdfarray, temp, wdf, clab = clab, xlab = xlabel, ylab = ylabel, $
              file = strfile, xydata = xydata
            if npwdf ge n_elements(wdfarr) then begin
               print, ' Maximum number of lineouts has been reached'
               print, '    Please exit and return before taking additional lineouts'
               return
            endif
            wdf = wdfarr[npwdf]
          endelse
          slopesav = 0.0
        endfor
      endif else begin
;       Vertical
        nelx = n_elements(xin)
        if nelx lt 1 then begin
          Print, '% LINSTR: Horizontal lineout must have at least one Y value.'
          return
        endif
        xx = y
        for i = 0, nelx-1 do begin
          px = xin(i)
          if iflag_lin eq 1 or iflag_lin eq 3 then begin
            junk = min(abs(x-px), ijk)
            px = x[ijk]
          endif
          yy = ylineout(array, x, px, widsav, median = median, integral = integral)
          if n_elements(xx) lt 3 then begin
            print, 'Lineout must contain at least 3 points'
          endif else begin
            npwdf += 1
            clab = 'Lineout of '+clabel+': X = '+strtrim(px,2)
            if dx2 gt 0.0 then begin
              xydata = 0
              temp = [xx[0], dx2, yy]
            endif else begin
              temp =[xx, yy]
              xydata = 1
            endelse
            put_wdfarray, temp, wdf, clab = clab, xlab = xlabel, ylab = ylabel, $
              file = strfile, xydata = xydata
            if npwdf ge n_elements(wdfarr) then begin
               print, ' Maximum number of lineouts has been reached'
               print, '    Please exit and return before taking additional lineouts'
               return
            endif
            wdf = wdfarr[npwdf]
          endelse
        endfor
        slopesav = 90.0
      endelse
      return
      end

pro linstr, strin, wdfin, xin, yin, horizontal=horizontal, vertical=vertical, $
           xonly = xonly, yonly = yonly, width = width, cprint = printit, $
           preview = preview, wdfarray = wdfarray, group = group, pextra = pextra, $
           xsize = xsizein , ysize= ysizein, orderx = orderxin, median = medianin, $
           plast = plast, qlast = qlast, pxrange = pxrange, iflag = iflag, $
           xvalue = xinput, yvalue = yinput, integral = integralin, grid = grid, $
           exact = exact, slopeangle = slopeangle, modal = modal, rotate = rotatein, $
           cubic = cubic, set_default = set_default, show_default = show_default, $
           multiply = multiply, linear = linear, _extra =extrastuff

;
;       return on an error
on_error, 1
@com.dat
  common linstr_save_data, exact_save, cubic_save, rotate_save
  common linstr_data_com
  common linstr_widget_com
      if n_elements(initwdf) eq 0 then wdfinit
@qscom.dat
;  Common block
@qsstrcom.dat
;
      common pffcom, initpff, pff_exe
@draw_defaults.com
      nparams = n_params()
      lc_set = 0
      if n_elements (exact_save) eq 0 then begin
        exact_save = 0
        cubic_save = 0
        rotate_save = 0
      endif
      if keyword_set (set_default) gt 0 then begin
        if n_elements(exact) gt 0 then exact_save = exact[0]
        if n_elements(cubic) gt 0 then cubic_save = cubic[0]
        if n_elements(rotatein) gt 0 then rotate_save = rotatein[0]
        lc_set =1
      endif
      if keyword_set(show_default) then begin
        print, 'Default Values for the LINSTR command are:'
        print, '   EXACT   = '+strtrim (exact_save, 2)
        print, '   CUBIC   = '+strtrim (cubic_save, 2)
        print, '   ROTATE  = '+strtrim (rotate_save, 2)
        lc_set =1
      endif
      if (nparams eq 0) and (lc_set gt 0)  then return
      if n_elements (structinit) eq 0 then qsstruct_init
      if n_elements (qsplotinit) eq 0 then qsinit
      if n_elements (initpff) eq 0 then init_pff else $
       if initpff eq 0 then init_pff
;
      if n_elements(exact) eq 0 and n_elements(rotatein) eq 0 then exactc = exact_save $
        else if n_elements(exact) eq 0 then exactc = exact[0] $ 
        else if n_elements(rotatein) eq 0 then rotatec = 0  else rotatec = rotatein[0]
      if (n_elements(cubic) eq 0) then cubic = cubic_save
      gridc = n_elements(grid) gt 0 ? grid[0] : 1
      if keyword_set(xonly) then indep_var = 1 else $
      if keyword_set(yonly) then indep_var = 2 else indep_var = 0
      if nparams eq 0 then begin
        print,'% Format: ', $
       'LINSTR, STR, WDF,  [, /HORIZONTAL] [,/ VERTICAL] [, /CPRINT]'
        print,'        [, XVALUE = X] [, YVALUE = Y] [, GRID = grid]'
        print,'        [, SLOPEANGLE= slopeangle] [, PXRANGE = pxrange] '
        print,'        [, /EXACT] [, /ROTATE] [, /OrderX] [ WDFARRAY = wdfarray]'
        print,'        [, WIDTH = width][, PREVIEW = 0] [, IFLAG = iflag]'
        print,'        [, /SET_DEFAULT] [, /SHOW_DEFAULT] [, CUBIC = cubic]'
        print,'        [, _EXTRA = extrastuff]'
        return
      endif
      if n_elements(wdfin) gt 0 then wdf = wdfin
      nwdf = n_elements(wdf)
      if nwdf eq 1 then wdfarr = wdf[0] + indgen(maxwdfarray-wdf[0]+1) else $
        if nwdf eq 0 then wdfarr = wdffree(nelements = nel) else wdfarr = wdf
      nwdf = n_elements(wdfarr)
      if nwdf eq 1 then wdfarr = indgen(maxwdfarray -wdfarr[0] +1)+  wdfarr[0]
      npwdf = 0L
      if min(wdfarr, max =m) lt 1 or m gt maxwdfarray then begin
        winformation, ['WDF Arrays must be in the range', $
                'of 1 to '+strtrim (maxwdfarray,2)]
        return
      endif
      wdf = wdfarr[0]
  
      if n_elements(preview) eq 0 then preview = -1
      if n_elements(pxrange) lt 2 then pxran=0 else pxran= pxrange[0:1]
      if n_elements(orderxin) eq 0 then orderx=0 else orderx= orderxin[0]
      if n_elements(iflag) eq 0 then iflag_lin =0 else iflag_lin = iflag
      arr = get_valid_structure(strin, 'LINSTR', ierror = ier)
      if ier ne 0 then return
      if arr.type ne 1  and arr.type ne 8 then begin
        Print, '%LINSTR: Structure array must be a field structure'
        return
      endif

      if keyword_set(horizontal) then mode = 1 else $
        if keyword_set(vertical) then mode = 2 else mode =0
      if n_elements (group) eq 0 then group = 0
      if n_elements(width) eq 0 then widsav = 0 else widsav = abs(width)
      if n_elements(integralin) eq 0 then integral = 0 else integral = integralin
      if keyword_set(integral) then integral = 2 else integral = 0
      if n_elements(medianin) eq 0 then median = 0 else median = medianin
      if keyword_set(median) then begin
         median = 1
         medsetting = 1
      endif else medsetting = integral
      
      if n_elements(xin) eq 0 then begin
        if n_elements(xvalue) gt 0 then begin
          xin = xvalue
          lcbatch = 1
        endif else lcbatch = 0
      endif else lcbatch = 1
      if n_elements(yin) eq 0 then begin
        if n_elements(yvalue) gt 0 then begin
          yin = yvalue
        endif else lcbatch = mode*lcbatch < 1
      endif
      cubicc = (n_elements(cubic) eq 0) ? 0 : cubic[0]
      clabel = arr.clab
      xlabel = arr.x1lab
      x2label = arr.x2lab
      if arr.ndim eq 3 and arr.two ne 3 then begin
        if arr.two eq 1 then begin
          xlabel = arr.x2lab
          x2label = arr.x3lab
        endif else begin
          xlabel = arr.x3lab
          x2label =arr.x1lab
        endelse
      endif
      if arr.block eq 1 then begin
        deltas = get_str_dx(arr, dxrange = dxrange, /modify)
        deltamin = min(dxrange)
      endif else begin
        deltas =0
        deltamin = 0
      endelse
      if n_elements(grid) gt 0 then if ((grid[0] gt 0) and (grid[0] lt 1.0)) then $
        deltamin = deltamin * grid[0]
      if nparams gt 2 || keyword_set(lcbatch) then begin
        xdraw_range = arr.range[*, 0, 0]
        ydraw_range = arr.range[*, 1, 0]
        if keyword_set (exactc) then avesetting = 1 else $
          if keyword_set(rotatec) then avesetting = 2 else avesetting =0
        linstr_batch, arr, wdf, xin, yin, median = median, integral = integral, $
          multiply = multiply, linear= linear, _extra = extrastuff
        if npwdf gt 0 then wdfarray = wdfarr(0:npwdf-1) else wdfarray = 0
        slopeangle = slopesav
        arr =0
        arrc =0
        return
      endif
      if size(/type, pextra) ne 8 then undefine, plot_extra else plot_extra = pextra

      ylabel = arr.v1lab
      strfile = arr.file

  tlbase = WIDGET_BASE(GROUP_LEADER=Group, $
      COLUMN=1, modal = modal, $
      MAP=1, $
      TITLE='Line-Out')

  BASEINS  = WIDGET_BASE(tlbase, $
      COLUMN = 1, $
      MAP=1, $
      UVALUE = 'BASEINS')

 labelins1 =  WIDGET_LABEL( BASEins, $
      UVALUE='LABEL', $
  VALUE='Left Mouse = Designate Point,     Middle Mouse = Save,     Right Mouse = Reset')

  BASE0 = WIDGET_BASE(tlbase, $
      row=1, $
      FRAME=0, $
      MAP=1, $
      UVALUE='BASE0')

  BASE1 = WIDGET_BASE(base0, $
      column=1, $
      FRAME=2, $
      MAP=1, $
      event_pro = 'Linstr_Point_Event', $
      UVALUE='BASE1')

  BASE2 = WIDGET_BASE(base0, $
      FRAME=2, $
      MAP=1, $
      UVALUE='BASE2')

   BASE22 = WIDGET_BASE(base0, $
      column=1, frame = 2, $
      MAP=1, $
      event_pro = 'Widstr_Point_Event', $
      UVALUE='BASE22')

  BASE3 = WIDGET_BASE(base0, $
      column=1, $
      FRAME=2, $
      MAP=1, $
      event_pro = 'Line_Slope_Event', $
      UVALUE='BASE3')

  BASE33 = WIDGET_BASE(base0, $
      column=1, $
      FRAME=2, $
      MAP=1, $
      event_pro = 'Line_Method_Event', $
      UVALUE='BASE33')

  BASE4 = WIDGET_BASE(tlbase, $
      ROW = 1, $
      MAP=1, $
      UVALUE = 'BASE4')

  BASE5 = WIDGET_BASE(tlbase, $
      ROW = 1, $
      MAP=1, $
      event_pro = 'Linstr_Point_Event', $
      UVALUE = 'BASE5')

  BASE6 = WIDGET_BASE(BASE4, $
      ROW = 1, $
      MAP=1, $
      FRAME = 2, $
      UVALUE = 'BASE6')

  BASE7 = WIDGET_BASE(BASE4, $
      ROW = 1, $
      MAP=1, $
      FRAME = 2, $
      event_pro = 'SAVE_BUTTON_EVENT', $
      UVALUE = 'BASE7')

  BASE8 = WIDGET_BASE(BASE4, $
      ROW = 1, $
      MAP=1, $
      FRAME = 2, $
      event_pro = 'RESET_BUTTON_EVENT', $
      UVALUE = 'BASE8')

      px = 0.0
      py = 0.0
      qx = 0.0
      qy = 0.0

   label1  =  WIDGET_LABEL( BASE1, $
      UVALUE='LABEL', $
      VALUE='First Point')

   xwindow1 = CW_FIELD( BASE1,VALUE=px, $
      ROW=1, xsize = 12, $
      FLOAT=1, $
      RETURN_EVENTS = 1, $
      TITLE='X-Value:', $
      UVALUE='XVALUE1')

   ywindow1 = CW_FIELD( BASE1,VALUE=py, $
      ROW=1, xsize = 12, $
      FLOAT=1, $
      RETURN_EVENTS = 1, $
      TITLE='Y-Value:', $
      UVALUE='YVALUE1')

   bset1 = WIDGET_BASE(BASE1, $
      ROW = 1, $
      MAP=1, $
      UVALUE = 'BSET1')

   set1= WIDGET_BUTTON( Bset1, $
      UVALUE='BUTTON1', $
      VALUE=' Set Point 1 Value ')

  BASE21 = WIDGET_BASE(base2, $
      column=1, $
      MAP=1, $
      event_pro = 'Linstr_Point_Event', $
      UVALUE='BASE21')

   label2  =  WIDGET_LABEL( BASE21, $
      UVALUE='LABEL', $
      VALUE='Second Point')

   xwindow2 = CW_FIELD( BASE21,VALUE=px, $
      ROW=1, xsize = 12, $
      FLOAT=1, $
      RETURN_EVENTS = 1, $
      TITLE='X-Value:', $
      UVALUE='XVALUE2')

   ywindow2 = CW_FIELD( BASE21,VALUE=py, $
      ROW=1, xsize = 12, $
      FLOAT=1, $
      RETURN_EVENTS = 1, $
      TITLE='Y-Value:', $
      UVALUE='YVALUE2')

   bset2 = WIDGET_BASE(BASE21, $
      ROW = 1, $
      MAP=1, $
      UVALUE = 'BSET2')

   set2= WIDGET_BUTTON( BSET2, $
      UVALUE='BUTTON2', $
      VALUE=' Set Point 2 Value ')


   label2  =  WIDGET_LABEL( BASE22, $
      UVALUE='LABEL', $
      VALUE='LineOut Width')

   widwindow = CW_FIELD( BASE22,VALUE=widsav, $
      ROW=1, $
      FLOAT=1, xsize = 12, $
      RETURN_EVENTS = 1, $
      TITLE='Width:', $
      UVALUE='WIDVALUE')

   avewindow = cw_bgroup(base22, ['Average', 'Median', 'Integral'],  $
         /exclusive, /column, set_value = medsetting)

   label3  =  WIDGET_LABEL( BASE3, $
      UVALUE='LABEL', $
      VALUE='Select Slope')

     Btns335 = [ $
    'Arbitrary', $
    'Horizontal', $
    'Vertical' ]
   SLOPE_sel = CW_BGROUP( Base3, Btns335, $
      COLUMN=1, $
      EXCLUSIVE=1, /NO_RELEASE, $
      UVALUE='SLOPE')

  if keyword_set (exactc) then avesetting = 1 else $
    if keyword_set(rotatec) then avesetting = 2 else avesetting =0

  label33 = WIDGET_LABEL( BASE33, $
      UVALUE='LABEL', $
      VALUE='Width Method')
  btns435 =['Normal', 'Exact', 'Rotate']
  avemethod = CW_BGROUP( Base33, Btns435, $
      COLUMN=1, set_value = avesetting, $
      EXCLUSIVE=1, /NO_RELEASE, $
      UVALUE='AVEMETH')


  WDFwindow = CW_FIELD( BASE6,VALUE = WDF, $
      ROW=1, $
      integer=1, $
      TITLE='WDF Array:', $
      RETURN_EVENTS = 0, $
      XSIZE = 5, $
      UVALUE='WDF')

  SAVEB= WIDGET_BUTTON( base7, $
      UVALUE='SAVE_BUTTON', $
      VALUE=' Accept / Save Lineout ')

  Btns439 = [ $
    ' Reset All Points ', $
    '   Done  / Exit   ' ]
  ard_buttons = CW_BGROUP( base8, Btns439, $
      ROW=1,  /NO_RELEASE, $
      UVALUE='buttons')

  oldwin = !d.window
  initbx = !x
  initby = !y
  initbp = !p
  pmulti_save=!p.multi
  !p.multi = 0
  if n_elements(xsizein) eq 0 then begin
    if initpff gt 3  then xsize = medsize(0)*5/6 else xsize = medsize(0)
  endif else xsize = xsizein
  if n_elements(ysizein) eq 0 then begin
    if initpff gt 3 then ysize = medsize(1)*5/6 else ysize = medsize(1)
  endif else ysize = ysizein
  if n_elements(tlb_offset) eq 0 then tlb_offset = [10, 20]
  if n_elements(line_offset) eq 0 then $
       line_offset = [tlb_offset[0]+medsize[0]+30, smallsize[1]]

  if n_elements (xpreviewsize) eq 0 then xpreviewsize = smallsize
  npselect = 0
  slopesav=0
  window, /free , /PIXMAP  , xsize = xsize, ysize = ysize
  pixwin = !d.window
  if arr.block gt 1 then $
        Print, '%LINSTR: Structure will be put on a single block'
  if n_elements(multiply) gt 0 and keyword_set(linear) then $
    log_image = (multiply[0] eq 0) ? 1 : 0 else log_image = 0
  if log_image then begin
    plotfld, arr, farray = array, xarray= x, yarray= y, iflag=iflag, $
       nfarray = 256,cntour = 2,  _extra = extrastuff
    plotfld, arr, farray = array1, xarray= x, yarray= y, iflag=iflag, $
       nfarray = 256,cntour = 2,  multiply = multiply, _extra = extrastuff
    array1 =0
  endif else plotfld, arr, farray = array, xarray= x, yarray= y, iflag=iflag, $
       nfarray = 256,cntour = 2,  multiply = multiply, _extra = extrastuff
  if n_elements (array) lt 4 then begin
    wdelete, pixwin
    if oldwin gt 0 then  begin
      wset, oldwin
      !x = initbx
      !y = initby
      !p = initbp
    endif
    return
  endif
  ok = where(strpos(tag_names(arr), 'DX') ge 0 , nok)
  if nok eq 0 then begin
    check_uniform, x, dx1
    check_uniform, y, dx2
  endif else begin
    if xoldvalue eq 1 then dx1 = arr.dx1[0] else $
      if xoldvalue eq 2 then dx1 = arr.dx2[0] else $
      if xoldvalue eq 3 then dx1 = arr.dx3[0] else dx1 = 0
    if yoldvalue eq 1 then dx2 = arr.dx1[0] else $
      if yoldvalue eq 2 then dx2 = arr.dx2[0] else $
      if yoldvalue eq 3 then dx2 = arr.dx3[0] else dx2 = 0
    check_uniform, x, dxx
    if dx1 eq 0 then dx1 = dxx else dx1 = ((abs((dxx-dx1)/dx1)) lt .01) ? dx1:dxx
    check_uniform, y, dyy
    if dx2 eq 0 then dx2 = dyy else dx2 = ((abs((dyy-dx2)/dx2)) lt .01) ? dx2:dyy
  endelse

  xdraw_range = !x.crange
  ydraw_range = !y.crange
  nx = n_elements(x)
  ny = n_elements(y)
  if nx eq 0 or ny eq 0 then begin
    !p.multi = pmulti_save
    return
  endif
  dx = x(1:nx-1)-x(0:nx-2)
  dxm = (nx gt 2) ? [dx[0], (dx[0:nx-3]+dx[1:nx-2])*0.5, dx[nx-2]] : dx
  dy = y(1:ny-1)-y(0:ny-2)
  dym = (ny gt 2) ? [dy[0], (dy[0:ny-3]+dy[1:ny-2])*0.5, dy[ny-2]] : dy
  image = [0, 0, xsize, ysize, 0, 0, pixwin]

  DRAW = WIDGET_DRAW( base5, $
      BUTTON_EVENTS=1, $
      MOTION_EVENTS=1, $
      RETAIN=1, $
      UVALUE='DRAW', $
      XSIZE=xsize, $
      YSIZE=ysize)

  WIDGET_CONTROL, tlbase, /REALIZE, tlb_set_xoffset = tlb_offset[0], $
            tlb_set_yoffset = tlb_offset[1]
  widget_control, slope_sel, set_value = mode
  widget_control, base7, map = 0
  if mode gt 0 then widget_control, base21, map = 0
  if mode gt 0 then widget_control, base33, map = 0
  widget_control, widwindow, set_value = widsav
  widget_control, wdfwindow, set_value = wdf
  widget_control, draw, draw_motion_events = mode
  
; Get drawable window index
  WIDGET_CONTROL, DRAW, GET_VALUE=DRAW_Id
  mksymbol, 'plus', /fill, size = 1.5
  loadxyct, /quiet, /freeze
  if keyword_set(printit) then print_var =1 else print_var = 0
  npselect = 0
  wset , draw_id
  device , copy = image
  oldbx = !x
  oldby = !y
  oldbp = !p

  if n_elements(preview) gt 1 then begin
    line_id = 1
    ypreview = preview(0:1)
  endif else if preview ne 0 then begin
    line_id = 1
    if preview lt 0 then ypreview = 0 else begin
      ypreview=[min(array,max=mm), mm]
      delta = (ypreview(1)-ypreview(0))* 3.0517578e-05
      ypreview=ypreview+[delta, -delta]
    endelse
  endif else line_id = -1
  if line_id gt 0 then begin
    window, /free, xsize= xpreviewsize(0), ysize = xpreviewsize(1), $
        title = 'Current Lineout', xpos=line_offset[0], ypos=line_offset[1]
    line_id = !d.window
    device, get_window_position = line_offset_actual
    window_offset = line_offset-line_offset_actual
  endif else  Window_offset = [0,0]
  arrc = temporary(arr)
  wset, draw_id
  !x = oldbx
  !y = oldby
  !p = oldbp
  XMANAGER, 'linstr', tlbase
  array = 0
  arrc = 0
  if npwdf gt 0 then wdfarray = wdfarr(0:npwdf-1) else wdfarray = 0
  plast = [px, py]
  qlast = [qx, qy]
  line_offset = line_offset +window_offset
  slopeangle = slopesav
  wdelete, pixwin
  !p.multi = pmulti_save
  if oldwin lt 0 then return
  wset, oldwin
  !x = initbx
  !y = initby
  !p = initbp
END
