; ------------------------------------------------------------------------
; $Id: logstr.pro,v 1.7 2010/08/24 21:59:22 lpmix Exp $
; ------------------------------------------------------------------------
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
Pro LOGSTR, str_in, str_out, minimum = minimum, $
            NATURAL = natural, TIME=timein, $
            scale = scale, absolute = absolute, relative = relative, $
            showdefault = showdefault, setdefault = setdefault, unset = unset
;
;
;+
; NAME:
;       LOGSTR
;
; PURPOSE:
;       Take the log of values in a structure array.
;
; CATEGORY:
;       Utility
;
; CALLING SEQUENCE:
;       LOGSTR, STR_IN [, STR_OUT]
; INPUTS:
;       STR_IN:   Structure array number or structure array to take the logarithm
;                 STR_IN may be a search string for a structure array.
;       STR_OUT:  Structure array number or structure array for the smoothed
;                 result.   If STR_OUT is present but does not have a valid 
;                 value, the default value is (get_maxstr()-1)
;                 If STR_IN is a structure, then 
;                   If arg_present(STR_OUT) then STR_OUT = LOG(STR_IN) else
;                     arg_present(STR_IN) then STR_IN = LOG(STR_IN) else 
;                     LOG(STR_IN) is stored in (get_maxstr()-1)
;                 If STR_IN is a string or number, then LOG(STR_IN) will
;                   be stored in a structure array.
;                   If STR_OUT is present, but undefined then it will be set.
;                   If STR_OUT is defined, then that will be used if valid or
;                     STR_OUT will be set to (get_maxstr()-1)
;                   If STR_OUT is not present, then the result is stored in
;                     STR_IN. 
; KEYWORD PARAMETERS:
;       MINIMUM:  Used to determine the minimum value for the logarithm.
;             Note: In the case of a relative minimum, the maximum is the 
;                   maximum for all times unless TIME is set.
;                 LOG_MIN = MINIMUM
;                   IF (LOG_MIN LT 0.0 or LOG_MIN GE 1) then
;                       MINVAL = ABS(LOG_MIN)
;                   IF (LOG_MIN GT 0 AND  LOG_MIN LT 1) then
;                       MINVAL = LOG_MIN*MAX(abs(field_value))
;                   unless RELATIVE = 0
;       NATURAL:  Natural LOG (ALOG)
;                 IF NATURAL = 0, base 10 log (ALOG10)
;                 DEFAULT: NATURAL = 1
;                 Labels will be LOG for Natural Log and LOG10 for base 10
;       RELATIVE: If set to zero, then MINVAL= ABS(MINIMUM) regardless of value
;                 DEFAULT:  RELATIVE = 1
;                 If MINIMUM in the range(0,1) and RELATIVE = 1, then
;                    MINVAL = LOG_MIN*MAX(abs(field_value))
;       TIME:     Ignored unless type 9 structure
;                 If set, and minimum is in the range (0,1), then 
;                 the minimum will be set differently for each time step.
;       SCALE:    If not equal to zero, values will be multiplied by SCALE 
;                 before the LOG is taken.
;                    Vector = SCALE * Vector
;                 DEFAULT: SCALE = 1
;       ABSOLUTE: If set, the absolute value will be taken before taking the log.
;                 If set, the order is:
;                    Vector = ABS(SCALE*Vector)
;       SHOWDEFAULT: Show the default values
;       UNSET:    Reset the default values
;                 LOG_MIN = 1.0E-4
;                 LOG_TYPE= 1 (NATURAL TYPE)
;       SETDEFAULT: Make the current values the default values.
;
; OUTPUTS:
;       The LOG of the specified structure is returned in STR_IN or STR_OUT
;
; RESTRICTIONS:
;       STR_IN array must be defined and have valid data.
;        NOTE:  Results with multi block data may not be satisfactory
;               along block boundaries.
;
; TECHNIQUE:
;       The data is filtered with respect to the MINIMUM and the appropriate 
;       logarithm is taken
;
; EXAMPLES:
;       Take the natural logarithm of STR 2 and save it in 3
;            LOGSTR, 2, 3
;       Take the natural logarithm of STR 2 and save it back in 2
;            LOGSTR, 2
;       Set the minimum to an absolute value of 1.0e-6
;            LOGSTR, -1.0e-6, /set
;       Take the logarithm of the density and store it into the same structure 
;         array and plot the result
;            LOGSTR, 'density'
;            PLOTSTR, 'density', cn=2
;
; MODIFICATION HISTORY:
;      Original version, L. P. Mix, December 15, 2003
;        Based on routine by R. W. Lemke, "make_log_str"
;-
;
on_error, 1
;     include the common block
;
common LOG_STR_COMMON, LOG_MIN, LOG_TYPE, LOG_RELATIVE

npar = n_params()

if n_elements(log_type) eq 0 or keyword_set(UNSET) then begin
  if n_elements(log_min) eq 0 or keyword_set(unset) then log_min = 1.0e-4
  log_relative = 1
  log_type = 1
endif

if keyword_set(setdefault) then begin
  if n_elements(natural) gt 0 then begin
    if keyword_set(natural) then LOG_TYPE = 1 else LOG_TYPE = 0
  endif
  if n_elements(relative) gt 0 then log_relative = relative
  if n_elements(minimum) gt 0 then begin
    if minimum[0] eq 0 then log_min= 1.0e-4 else log_min = minimum[0]
  endif
endif

if keyword_set(showdefault) then begin
  print, 'LOG_MINIMUM: ', strtrim (log_min, 2)
  if keyword_set(log_type) then tlog= ' (Natural Log)' else $
    tlog = ' (Base 10 Log)'
  print, 'LOG_TYPE: ', strtrim (log_type, 2), tlog
  print, 'RELATIVE: ', strtrim (log_relative, 2)
endif

If (keyword_set(unset) or keyword_set(setdefault) or keyword_set(showdefault)) $
   and (npar eq 0) then return

if npar eq 0 then begin
  print,'% Format: LOGSTR, STR_IN [, STR_OUT] [, MINIMUM = minimum]'
  print,'           [, NATURAL = natural] [, /RELATIVE]'
  print,'            [, /SETDEFAULT] [, /UNSET] [, /SHOWDEFAULT]'
  return
endif 

if n_elements(str_in) eq 0 then begin
  print, '% LOGSTR: Input structure must be defined.'
  return
endif else str= str_in[0]

ss=0
if size(/type, str) lt 8 then begin
  ss = strvalid(str, /valid, /quiet, np= np)
  if np ne 1 then begin
    if np lt 1 then print, '% LOGSTR: STR_IN is not valid' else $
      print, '% LOGSTR: STR_IN has multiple values'
    help, str_in
    return
  endif
  ss = ss[0]
  lc_str = 0
  s2i, str, ss
endif else lc_str = 1

if n_elements(minimum) eq 0 then logmin = log_min else logmin= minimum[0]
if n_elements(relative) eq 0 then relative = log_relative

ok = where (strpos(tag_names(str), 'NVECT') ge 0 , nok)
if nok eq 0 then begin
  print, '% LOGSTR: Unable to find tag name NVERT'
  return
endif
nvect = str.nvect
if nvect gt 6 then begin
  print, '% LOGSTR:  Present version set for 6 vector components.'
  print, '           Only the first 6 will be transformed.'
  nvect = 6
endif

if logmin eq 0 then logmin = log_min
if keyword_set(relative) then begin
  if logmin gt 0 and logmin lt 1 then lc_rel = 1 else lc_rel = 0
endif else lc_rel= 0
logmin = abs(logmin)

if n_elements (natural) eq 0 then natural = log_type

if str.type eq 9  and  keyword_set(timein) then begin
  ntime = str.ntime
  if ntime eq 1 then time = 0 else time = 1*lc_rel
endif else time = 0 

for i = 1, nvect do begin

  if i eq 1 then v= str.v1 else $
    if i eq 2 then v = str.v2 else $
    if i eq 3 then v = str.v3 else $
    if i eq 4 then v = str.v4 else $
    if i eq 5 then v = str.v5 else $
    if i eq 6 then v = str.v6  
  if keyword_set(scale) then v = scale[0] * v
  if keyword_set(absolute) then v = abs(v)
  if time gt 0 then begin
    for i = 0, ntime -1 do begin
      vt = v[*, i]
      if keyword_set(absolute) then vm = logmin*max(vt) else $
        vm = logmin*max(abs(vt))
      ok = where(vt ge vm, complement = notok, ncomplement = nnotok, nok)
      if nok lt nnotok/10 then begin
        print, '% LOGSTR:  Most values below log minimum
        print, '           Minimum:', vm
        print, '           Number Good: ', strtrim(nok, 2)
        print, '           Number Bad : ', strtrim(nnotok, 2)
        print, '           Time step: ' , strtrim(i, 2)
      endif
      if nnotok gt 0 then vt[notok] = vm
      if keyword_set(natural) then vt= alog(vt) else vt = alog10(vt)
      v[0, i]=vt
    endfor
  endif else begin
    if lc_rel eq 1 then begin 
      if keyword_set(absolute) then vmin = logmin*max(v) else $
        vmin = logmin*max(abs(v))
    endif else vmin = logmin
    ok = where(v ge vmin, complement = notok, ncomplement = nnotok, nok)
    if nok lt nnotok/10 then begin
      print, '% LOGSTR:  Most values below log minimum
      print, '           Minimum:', vmin
      print, '           Number Good: ', strtrim(nok, 2)
      print, '           Number Bad : ', strtrim(nnotok, 2)
    endif
    if nnotok gt 0 then v[notok] = vmin
    if keyword_set(natural) then v = alog(v) else v = alog10(v)
  endelse
  if i eq 1 then  str.v1 = v else $
    if i eq 2 then str.v2 = v else $
    if i eq 3 then str.v3 = v else $
    if i eq 4 then str.v4 = v else $
    if i eq 5 then str.v5 = v else $
    if i eq 6 then str.v6 = v
endfor

lc = (keyword_set(natural)) ? 'LOG(' : 'LOG10('

; Look for separators in the comment
;
clab = str.clab
p = strpos(clab, ' - ')
q =  strpos(clab, '@')
if p gt 0 then begin
  clab = lc+strmid(clab, 0, p)+')'+strmid(clab, p) 
endif else if q gt 0 then begin
  clab = lc+strmid(clab, 0, q)+')'+strmid(clab, q) 
endif else  clab = lc+clab+')'
str.clab = clab

if lc_str eq 0 then begin
  if npar eq 2 then begin
    str_out = get_save_str(str_out, get_maxstr()-1)
    i2s, str, str_out, /no_copy 
  endif else  i2s, str, ss, /no_copy
endif else begin
  if npar eq 2 then begin
    if arg_present(str_out) then str_out = temporary(str) else begin
      if arg_present(str_in) then str_in = temporary(str) else $
        i2s, str, get_maxstr-1, /no_copy
    endelse
  endif else if arg_present(str_in) then str_in = temporary(str) else $
        i2s, str, get_maxstr-1, /no_copy
endelse

end

