; ------------------------------------------------------------------------ 
; $Id: minstr.pro,v 1.3 2007/06/06 20:09:35 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
 Function  minstr , strin, vector, xrange=xrange, yrange=yrange, $
    zrange=zrange, location = location, maximum = maxval, ier = ier, $
    pmaximum = pmaximum, pminimum = pminimum, helpme = helpme 
;
;+
; NAME:
;       MINSTR
;
; PURPOSE:
;       Find the minimum of a structure array and the location of that minimum.
;       Current version is for type 1 field structures.
;
; CATEGORY:
;       Math
;
; CALLING SEQUENCE:
;       Field_Min =  MINSTR (Structure [, Vector]  )
;
; where Field_Min is the minimum value of the field. 
;
; INPUTS:
;       Structure - Structure array number or idl field Structure.
; OPTIONAL INPUTS:
;       Vector    - Vector component for the function.
;                   If Vector is an array, the square root of the sum of
;                     the squares will be used.
;                   If Vector is a single value array, ABS will be applied to 
;                     that component. That is if Vector = [2] then 
;                     ABS(V2) will be searched.
;                   If max(Vector) has a value larger than the number of vector
;                     components and dimensions, then Vector = 1+indgen(nvect)
;                     where nvect is the number of vector components
;                   Default: VECTOR = 1 if number of vectors is 1
;                            VECTOR = 99 if number of vectors is > 1.
;	
; KEYWORD PARAMETERS:
;       XRANGE    - Range of the first dimension for the minimum search.
;                   Default:  All X values
;       YRANGE    - Range of the second dimension for the minimum search.
;                   Default:  All Y values
;       ZRANGE    - Range of the third dimension for the minimum search.
;                   Default:  All Z values
;       LOCATION  - Position where minimum occurs
;       PMINIMUM  - Same as LOCATION
;       MAXIMUM   - Maximum val in the range defined.
;       PMAXIMUM  - Position where MAXIMUM occurs.
;       IER       - Error parameter
;                   Default: IER = 0 ==> No problems 
;
; OUTPUTS:
;       The function returns the Minimum of the field structure.
;
; OPTIONAL OUTPUTS:
;       Maximum and the locations for the maximum and minimum are returned
;
; EXAMPLE:
;       Determine the range of the electric field structure E
;         emin = minstr(E, 4, max= emax)
;         range = emax-emin
;
;
; MODIFICATION HISTORY:
;       Written by:     L. P. Mix, March 12, 1999
;-
 ier = 1
if keyword_set(helpme) then begin
  print, '% Format:'
  print, '      Field_Min =  MINSTR (Structure [, Vector]'
  print, '           [, XRANGE= xrange] [, YRANGE= yrange] [, ZRANGE= zrange]'
  print, '           [, LOCATION = location] [, PMINIMUM = pminimum]'
  print, '           [, MAXIMUM = maximum] [, PMAXIMUM = pmaximum])'
  return, 0
endif
;

If n_elements (strin) eq 0 then begin
  print, '% MINSTR:  Structure or structure array number must be provided'
  return, 0
endif

; we lied  - we will leave fld undefined and let minmaxstructure handle it
if n_elements (vector) gt 0 then fld = vector

minmaxstructure, strin, fld,  xrange=xrange, yrange=yrange, ier = ier, $
    zrange=zrange, maximum = maxval,  minimum = minval, $
    pmaximum = pmaximum, pminimum = pminimum



if ier eq 0 then location = pminimum
return, minval
end
