; ------------------------------------------------------------------------
; $Id: penumbral.pro,v 1.2 2011/10/04 21:20:34 lpmix Exp $
; ------------------------------------------------------------------------
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
;+
; NAME:
;       PENUMBRAL
;
; PURPOSE:
;       This routine calculates the image measured using penumbral imaging 
;       for a square aperature.  Two types of filtering are offered:
;       1.  The Savitzky-Golay smoothing filter is used to perform differentiation.
;       2.  A least squares Fourier filter, related to a Wiener filter, is
;           applied after simple differentiation of the image.
;
; CATEGORY:
;       Analysis, Imaging
;
; CALLING SEQUENCE:
;
;       PENUMBRAL, STRUCTURE [, IMAGE_STR]
;
; INPUTS:
;       STRUCTURE:  Structure or structure array number or name
;                   This structure should contain an image of the corner of 
;                   penumbral imaging.  In typical images about 1/4 is bright, 
;                   and about 3/4 is dark due to to the square aperature.
;                   _______
;                   |__|__|  In the figure at the left 3 of the squares will have
;                   |__|__|  little exposure and one will have maximum exposure.
;                   Image must be uniform in each direction.
;
; OPTIONAL INPUTS:
;       IMAGE_STR:  Image structure number for the calculated image. 
;                   If this parameter is missing or not valid, image
;                   is stored in (GET_MAXSTR() -1)
;                   The coordinates of the image are set to zero at the maximum 
;                   value of the image and the image maximum is normalized to 1.
;                   If keyword, ZERO, is set then the rotation is about the maximum
;                   of the image.  Otherwise, the rotation is about the 50% values of
;                   ORIENT scans.
;	
; KEYWORD PARAMETERS:
;       WDF:        Waveform array number for the width of the image as a function
;                   of angle.  The WDF waveform is the AWE spot size vs angle and the
;                   WDF+1 waveform is the full width at half max value with a baseline
;                   of zero.
;                   DEFAULT:  WDF = GET_MAXWDF()-2
;       MAGNIFICATION: Image magnification.  Divide spot by the image magnification.
;                   If MAGNIFICATION = 0 then it is changed to 1.00
;                   DEFAULT:  MAGNIFICATION = 1.00
;       FUDGE:      This factor is used to relate the 25% to 75% points to the AWE
;                   spot size.  The spot is determined by integrating in one direction
;                   and then integrating in the orthogonal direction.  The distance between 
;                   the 25% and 75% points are determined and multiplied by FUDGE to 
;                   determine the AWE spot size.
;                   In all calculations of the AWE spot size the cumulative distribution
;                   is smoothed by FILTER, normalized to the max, sorted, and the 25% and 
;                   75% points are determined and the difference is multiplied by FUDGE.
;                   DEFAULT:  FUDGE = 2.5
;       XSPOT:      The spot size in the X direction determined from the "orient"
;                   scan in the X direction.  (XSPOT = FUDGE *(X(75%) - X(25%)))
;                   Normalization occurs after smoothing.
;                   Note:  Lineout is sorted after smoothing to facilitate the calculation,
;                          but points in the transition region are generally not affected.
;       YSPOT:      The spot size in the Y direction determined from the "orient"
;                   scan in the Y direction.  (YSPOT = FUDGE *(Y(75%) - Y(25%)))
;                   Normalization occurs after smoothing.
;                   Note:  Lineout is sorted after smoothing to facilitate the calculation,
;                          but points in the transition region are generally not affected.
;       SPOT:       Average spot size determined from the penumbral image. 
;                   The image is rotated by 1 degree from 0 to 179 degrees.
;                   If keyword, ZERO, is set then the rotation is about the maximum
;                   of the image.  Otherwise, the rotation is about the 50% values of
;                   ORIENT scans.  If zero is set then PIVOT is also checked.
;                   SPOT is a two element vector.  The first element refers to the FWHM
;                   and the second element is the AWE spot size.
;                   The FWHM is determined using the PFIDL routine FWHM with quiet set.
;                   The AWE spot size is determined as noted below:
;                   At each angle, the image is integrated in X and then in Y as
;                   a function of Y.  The integral is filtered and sorted and then the 25%
;                   and 75% points are determined and the difference is multiplied by FUDGE.
;                   Normalization occurs after smoothing and sorting.
;       SDEV:       Standard Deviation of SPOT calculated by the MOMENT function in IDL for 
;                   IDL version 8 and higher.  This is a two element vector.  The first element
;                   refers to the FWHM and the second element is the AWE spot size.
;       ZERO:       If set, rotate about the maximum of the image to calculate the FWHM and
;                   the AWE spot size..
;                   Default: ZERO =1
;       PIVOT:      Passed to ROT to rotate the image for the FWHM and the AWE spot sizes.
;       FILTER:     Boxcar filter applied to the ORIENT spot sizes and the 180 spot
;                   sizes calculated in 1 degree increments.  FILTER > 3
;                   DEFAULT:  FILTER = 11
;       ORIENT:     Scans are taken at the top, bottom, left and right side 
;                   of the input structure.  Orient specifies the fraction of the 
;                   image to use as an offset from the edge of the image.
;                   Lineout will go from ORIENT to ORIENT+WIDTH
;                   DEFAULT:  ORIENT = 0.02
;       WIDTH:      The width of the orient scan.
;                   DEFAULT:  WIDTH = 0.05 (of the image in each direction)
;       SAVEIT:     If set, save the X, Y, and angular lineouts in WDF arrays SAVEIT 
;                   and the next 181 (361) waveform arrays.  The first two waveforms are
;                   the X and Y spotsizes from the lineouts at the edges of the image.
;                   (see XSPOT and YSPOT) (See OPTSAVE for the options.) 
;                   The next 180 (360) waveforms are the AWE integrals or the lineouts taken at
;                   1 degree rotations.   OPTSAVE specifies the lineouts/integrals to 
;                   be saved.
;                   If the maximum number of waveforms is too small, it will be increased.
;       OPTSAVE:    Ignored unless SAVEIT is set.  Indicates the option for the SAVEIT arrays.
;                   OPTSAVE = 1 - Save the lineouts throuth the peak of the image.
;                   OPTSAVE = 2 - Save the integrals used to calculate the AWE spot size
;                   OPTSAVE = 3 - Save both 1 and 2
;                   Default:  OPTSAVE = 1
;       WINDOW:     Fraction of the XSPOT and YSPOT 25% to 75% distances to use as the 
;                   half width of the final image after differentiation.
;                   This is denoted HALF-WIDTH.
;                   WINDOW should be > 1. but any value to 0.1 are allowed.
;                   DEFAULT:  WINDOW = 6.0
;                   The window is calculated relative to the 50% points in the X and Y 
;                   directions.  The saved image coordinates are adjusted to have [0, 0]
;                   at the maximum value of the image.
;       QUIET:      Supress all informational printouts.  Errors are always printed.
;
; ****  Smoothing -- Either a Fourier (Wiener) type filter or a Savitzky-Golay filter are used
; ****  Wiener / Least Square smothing filter
;       WIENER:     If set use a Least Square Wiener type filter of the differentiation.
;                   DEFAULT:  WIENER = 0
;                     (Use the Savitzky-Golay filter to smooth the differentiation)
;                     (Wiener method is from the work of Gideon Barnea which is based on
;                      Digital Image Processing 2nd ed. by Gonzalez and Woods)
;                     (Real part of the inverse FFT is used for the image with the sign chosen
;                      to provide a positive signal.)
;       WWINDOW:    Approx Size of the data used for the Wiener filter.  By default the 
;                   size is 1024 x 1024.  The maximum size is approximately the size of the
;                   minimum dimension of the original image.  A square window is used
;                   and the size must have only 2 and 3 as factors.  The minimum size is
;                   4 * WINDOWSIZE (see WINDOW above- WINDOWSIZE = WINDOW*HALF-WIDTH)
;                   If WWINDOW is defined then:
;                     WWINDOW = (WWINDOW > 4*WINDOWSIZE) < (Min Size of original image)
;                   If WWINDOW is not defined:
;                     WWINDOW = (4*WINDOWSIZE > 1024) < (Min Size of original image)
;                   Actual size is increased to F23(WWINDOW) 
;       GAMMA:      Parameter for the least squares filter.  Increase for more smoothing.
;                   Default:  GAMMA = 50000 * (wwindow/1024)^2
;       Technique:
;                   Determine size of image for the Wiefer filter.  Calculate an ideal image
;                   of the same size ad the image by setting all values to 0.1 and in one quadrant
;                   of the ideal image to a value of 1; crop the experimental image to the same size
;                   as the ideal image.  Calculate the derivatives of the ideal image and the 
;                   experimental image.  Use a Wiener filter to reduce the noise in the derivative
;                   of the experimental data.  If SX3 is the size of the data array minus 3 then:
;                     1.  pe = [[0,1,0,fltarr(sx3)],[1,-4,1,fltarr(sx3)],[0,1,0,fltarr(sx3)], $
;                              [fltarr(sx3+3,sx3)]]
;                     2.  pe = FFT(pe, -1, /double)
;                     3.  peconjpe = peconjpe=pe*CONJ(pe)
;                     (data and ideal are the derivatives of the data and ideal (pseudo) data)
;                     4.  g = FFT(data,-1,/double)
;                     5.  h = FFT( -ideal, -1, /double)
;                     6.  xxxx = conj(h)
;                     7.  hxxxx=h*xxxx
;                     8.  Wiener = (xxxx/(hxxxx + peconjpe*gammaf)) * g
;                     9.  Filtered_Image = (+/-) SHIFT(REAL_PART(FFT(wien2,1)),SX/2,SY/2)
;                         (SX and SY are the size of the ideal and experimental images.)
;                         The sign of the image is chosen to make the peak positive.
;
; ****  Savitzky-Golay smoothing filter
;       DEGREE:     The degree of polynomial smoothing used to take the derivative.
;                   DEFAULT:  DEGREE =4
;       DOUBLE:     Do derivative calculation in double precision if set.
;                   DEFAULT:  DOUBLE =1
;       LENGTH:     Number of points in each direction used for the smoothing polynomial
;                   used for the derivative calculation.
;                   If length is less than 2 it is a fraction of the number of points
;                   between the 25% and 75% points in the "orient" lineouts.
;                   If length is greater than 2 then it is an absolute number of points.
;                   DEFAULT:  LENGTH = 0.75
; OUTPUTS:
;       The penumbral image, the average spot size, the spot size vs angle are output.
;
; PROCEDURE:
;	1.  Lineouts are taken at a distance ORIENT from each edge with a width
;           of WIDTH.`
;	2.  The orientation of the image is determined, the spot size in the X 
;           and Y direction, and the spot center in each direction are all 
;           determined by multiplying FUDGE times the 25% to 75% width.
;       3.  The penumbral image is determined by taking the derivative of the image
;           in the X and Y directions.  The Savitzky-Golay smoothing filter using the 
;           IDL SAVGOL routine is used to take the derivative or a simple derivative
;           is taken and then smoothed using a Fourier Least Squares filter.
;       4.  The resulting image is cropped to a square equal to WINDOW times the maximum spot
;           size determined in step 2.  This final image is saved in IMAGE_STR.
;       5.  The cropped image is totaled in the X direction and the integral taken
;           in the Y direction and normalized between 0 and 1.  The 25% and 75% points
;           are determined and the spacing multiplied by FUDGE to get the spot size.
;           A vertical lineout is taken through the peak and the FWHM calculated.
;       6.  The image is rotated by a degree and #5 repeated until the range of 0 to 179 
;           degrees has been covered.
;       7.  A WDF array of spot size vs angle is saved in WDF arrays.
;
; EXAMPLE:
;       Calculate the penumbral image from structure 8 and store the resulting image in
;       structure 21.
;         PENUMBRAL, 8, 21
;
; MODIFICATION HISTORY:
; 	Written by:     L. P. Mix, Jr., July 20, 2011
;	July, 2012     
;
;-

PRO PENUMBRAL, str, image_str, WDF = wdfin, FUDGE = fudge, XSPOT = xspot, YSPOT = yspot, $
               SPOT = spot, SDEV = sdev, ORIENT = orient, WIDTH = width, DEGREE = degree, $
               FILTER = filter, DOUBLE = dp, LENGTH = length, magnification = mag, $
               WINDOW = windowsize, saveit = saveitin, QUIET = quiet, WIENER = wiener, $
               OPTSAVE = optsavein, GAMMA = gammaf, wwindow = wwindow, zero = zero, $
               pivot = pivot
;
; Insert common blocks
;
@qscom.dat
;  Common block
@qsstrcom.dat
;       return on an error
on_error, 1
;
      if n_elements (structinit) eq 0 then qsstruct_init
      if n_elements (qsplotinit) eq 0 then qsinit
;
      nparams = n_params()
      if nparams eq 0 then begin
        print, '% Format:  PENUMBRAL, STR [, IMAGE_STR] [, WDF = wdf]'
        print, '          [, FUDGE = fudge] [, XSPOT = xspot] [, YSPOT = yspot]'
        print, '          [, SPOT = spot] [, SDEV = sdev] [, WINDOW = window]'
        print, '          [, ORIENT = orient] [, WIDTH = width]'
        print, '          [, DEGREE = degree] [, /DOUBLE] [, LENGTH = length]'
        print, '          [, MAGNIFICATION = magnification] [, FILTER = filter]'
	print, '          [, WIENER = wiener] [, GAMMA = gamma] [, /QUIET]'
	print, '          [, SAVEIT = saveit] [, OPTSAVE = optsave]'
        return
      endif
;
      if n_elements(str) eq 0 then begin
        print, '% PENUMBRAL:  Parameter must be defined'
        return
      endif

      array = get_valid_structure(str, 'PENUMBRAL', lcstr=lcstr, ierror=ier)
      if ier ne 0 then return
      if (array.type ne 1) then begin
        print, '% PENUMBRAL:  Only type 1 structures may be used'
        return
      endif
      dx = get_str_dx(array)
      if (n_elements(dx) ne 2 and array.two eq 0) || (dx[0] ne dx[1]) || [dx[0] eq 0] $
        then begin
        print, '% PENUMBRAL:  Current version requires square, uniform pixels.'
        return
      endif
      maxstr = get_maxstr()
      maxwdf = get_maxwdf()
      saveit =0
      if keyword_set(saveitin) then begin
        optsave = (n_elements(optsavein) eq 0) ?  1 : optsavein[0]
        if optsave lt 1 or optsave gt 3 then optsave = 1
        if saveitin lt 1 or saveitin gt maxwdf then begin
          print, '% PENUMBRAL:  Invalid value for SAVEIT.  Cannot save waveforms.'
          optsave = 0
        endif else begin
          saveit = saveitin
          total_save = (saveit gt 2) ? 366 : 186
          if (saveit + total_save) gt maxwdf then set_maxwdfarray, saveit+ total_save 
          maxwdf = get_maxwdf()
        endelse           
      endif else optsave = 0
      if (n_elements(image_str) eq 0) || (image_str lt 1) or $
        (image_str gt maxstr) then image_str = maxstr-1
      if (n_elements(wdfin) eq 0) || ((wdfin[0] lt 1) or $
        (wdfin[0] gt maxwdf-2)) then wdf = maxwdf-2 else wdf = wdfin[0]
      if n_elements(dp) eq 0 then dp =1
      if n_elements(orient) eq 0 then orient = 0.02
      if n_elements(width) eq 0 then width = 0.05
      if n_elements(fudge) eq 0 then fudge = 2.50
      if n_elements(degree) eq 0 then degree = 4
      if n_elements(length) eq 0 then length = 0.75
      filter = (n_elements(filter) eq 0) ?  11 : (filter[0] > 3)
      windowsize = (n_elements(windowsize) eq 0) ? 6.0 : (windowsize[0] > .1)
      mag = (n_elements(mag) eq 0) ?  1.00 : (mag[0] > 0)
      if mag eq 0 then mag = 1.00
      savgol_norm = dx[0]
      pixel = dx[0] /mag
      strlimits, array, xrange = xrange, yrange = yrange
      image = reform(get_array(array, -1))
      if ~keyword_set(quiet) then plotstr, array, /aspect, /xst, /yst
      ss = size (image)
      if ss[0] ne 2 then begin
        print, '% PENUMBRAL:  Only 2 dimensional images may be processed.'
        return
      endif
      pos = ss[1:2] * orient
      dx  = ss[1:2] * width
      dx = round(pos+dx)
      pos = round (pos)
      n = dx -pos +1
      i1 = total(image[pos[0]:dx[0], *], 1) /n[0]
      if ~keyword_set(quiet) then begin
        oplot, savgol_norm*pos[0]*[1,1]+xrange[0] , !y.crange
        oplot, savgol_norm*dx[0]*[1,1]+xrange[0] , !y.crange
      endif
      i2 = total(image[*, pos[1]:dx[1]], 2) /n[1]
      if ~keyword_set(quiet) then begin
        oplot, !x.crange, savgol_norm*pos[1]*[1,1]+yrange[0] 
        oplot, !x.crange, savgol_norm*dx[1]*[1,1] +yrange[0] 
      endif
      ind = ss[1] - [dx[0], pos[0]]
      i3 = total(image[ind[0]:ind[1], *], 1)/n[0]
      if ~keyword_set(quiet) then begin
        oplot, savgol_norm*ind[0]*[1,1]+xrange[0] , !y.crange
        oplot, savgol_norm*ind[1]*[1,1]+xrange[0] , !y.crange
      endif
      ind = ss[2] - [dx[1], pos[1]]
      i4 = total(image[*, ind[0]:ind[1]], 2)/n[1]
      if ~keyword_set(quiet) then begin
        oplot, !x.crange, savgol_norm*ind[0]*[1,1]+yrange[0] 
        oplot, !x.crange, savgol_norm*ind[1]*[1,1] +yrange[0] 
      endif
;     Values for the image lineouts
      values = [0.25, 0.5, 0.75]
      sx = ss[1]
      sy = ss[2]
      if (max(i1 , min = m)-m) gt (max(i3, min = mm)-mm) then begin
        lcleft = 1
        i1 = smooth (i1, filter)
        if keyword_set(saveit) then begin
          i2w, [0, pixel], i1, saveit, clab = 'Left Edge, Y-Spot', $
            xlab = 'Distance', ylab = 'Amplitude'
          saveit +=1
        endif
        top = max(i1, i, min = bot, subscript = j)
        k = (i gt j) ? sort(i1) : reverse(sort(i1))
        i1 = (i1[k] -bot) / (top-bot)
        ysp = findex(i1, values)
        yspot_bin = abs(ysp[2]-ysp[0])
        yspot = fudge * pixel * yspot_bin
        lr_ave = total(i3)/sy
      endif else begin
        lcleft =0
        i3 = smooth (i3, filter)
        if keyword_set(saveit) then begin
          i2w, [0, pixel], i3, saveit, clab = 'Right Edge, Y-Spot', $
            xlab = 'Distance', ylab = 'Amplitude'
          saveit +=1
        endif
        top = max(i3, i, min = bot, subscript = j)
        k = (i gt j) ? sort(i3) : reverse(sort(i3))
        i3 = (i3[k] -bot) / (top-bot)
        ysp = findex(i3, values)
        yspot_bin = abs(ysp[2]-ysp[0])
        yspot = fudge * pixel * yspot_bin
        lr_ave = total(i1)/sy
      endelse
      if (max(i2 , min = m)-m) gt (max(i4, min = mm)-mm) then begin
        lcbot = 1
        i2 = smooth (i2, filter)
        if keyword_set(saveit) then begin
          i2w, [0, pixel], i2, saveit, clab = 'Bottom Edge, X-Spot', $
            xlab = 'Distance', ylab = 'Amplitude'
          saveit +=1
        endif
        top = max(i2, i, min = bot, subscript = j)
        k = (i gt j) ? sort(i2) : reverse(sort(i2))
        i2 = (i2[k] -bot) / (top-bot)
        xsp = findex(i2, values)
        xspot_bin = abs(xsp[2]-xsp[0])
        xspot = fudge * pixel * xspot_bin
        tb_ave = total(i4)/sx
      endif else begin
        lcbot =0
        i4 = smooth (i4, filter)
        if keyword_set(saveit) then begin
          i2w, [0, pixel], i4, saveit, clab = 'Top Edge, X-Spot', $
            xlab = 'Distance', ylab = 'Amplitude'
          saveit +=1
        endif
        top = max(i4, i, min = bot, subscript = j)
        k = (i gt j) ? sort(i4) : reverse(sort(i4))
        i4 = (i4[k] -bot) / (top-bot)
        xsp = findex(i4, values)
        xspot_bin = abs(xsp[2]-xsp[0])
        xspot = fudge * pixel * xspot_bin
        tb_ave = total(i2)/sx
      endelse
      maxspot = max([xspot_bin, yspot_bin])
      halfw = round(maxspot*windowsize )
      clabel = array.clab
      clabel = (strpos(clabel, ' - ') gt 0) ? clabel+';;' : clabel+' - '
      
      if keyword_set(wiener) then begin 
        if n_elements (wwindow) eq 0 then wwindow = 1024
        winsize = (wwindow > (4*halfw)) <  f23(min([sx, sy]))
        winsize = f23(winsize)
        gammaf = (n_elements(gammaf) eq 0) ? 50000d0*(winsize/1024.0)^2 : double (gammaf > 0.0)
        if ~keyword_set(quiet) then begin
          print, ' GAMMA  = '+ strtrim (gammaf, 2)
          print, ' Wiener Filter Window size = ' + strtrim(winsize, 2)
          print, ' Original image: ' + strtrim (ss[1],2)+ ' , '+ strtrim(ss[2], 2)
        endif
        winsizem = winsize-1
        winsize2 = winsize/2L
        im = make_array(winsize, winsize, /double)
        im1= make_array(winsize, winsize, value = +0.1)
;       reduce size of the image to WINSIZE x WINSIZE
        if (sx gt winsize) then begin
;         See where the spot is relative to the midpoint
          if xsp[1] lt sx/2  then begin
            ist = (xsp[1] -winsize2) > 0L
            iend = (ist+winsizem) < (sx -1)
          endif else begin
            iend = (xsp[1] + winsize2) < (sx-1)
            ist = (iend -winsizem) > 0
          endelse
          xsp -= ist
          image= image[ist: iend, *]
        endif
        if (sy gt winsize) then begin
          if ysp[1] lt sy/2  then begin
            ist = (ysp[1] -winsize2) > 0L
            iend = (ist+winsizem) < (sy -1)
          endif else begin
            iend = (ysp[1] + winsize2) < (sy-1)
            ist = (iend -winsizem) > 0
          endelse
          ysp -= ist
          image= image[*, ist: iend]
        endif
        ss = size(image)
        sx = ss[1]
        sy = ss[2]
        if lcbot eq 1 and lcleft eq 1 then begin
;         Lower left
          im1[0:winsize2,0:winsize2]=1.
          im[0: ((sx-1) < winsizem), 0: ((sy-1) < winsizem)] = $
            image[0: ((sx-1) < winsizem), 0: ((sy-1) < winsizem)]
          if sx lt winsize then for i = sx, winsizem do im[i, *] = lr_ave
          if sy lt winsize then for i = sy, winsizem do im[*, i] = tb_ave
        endif else if lcbot eq 1 and lcleft eq 0 then begin
;         Lower right
          im1[winsize2: winsizem,0:winsize2]=1.
          if sx le winsize then begin
            st = winsize-sx
            xsp  += st
            sti = 0L
          endif else begin
            st = 0L
            sti = sx -winsize
            xsp = xsp - sti
          endelse
          im[st: winsizem, 0:((sy-1) < winsizem)] = $
            image[sti: (sx-1), 0:((sy-1) < winsizem)]
          if sx lt winsize then for i = 0, st do im[i, *] = lr_ave
          if sy lt winsize then for i = sy, winsizem do im[*, i] = tb_ave
        endif else if lcbot eq 0 and lcleft eq 1 then begin
;         Top left
          im1[0:winsize2,winsize2: winsizem]=1.
          if sy le winsize then begin
            st = winsize-sy
            ysp  += st
            sti = 0L
          endif else begin
            st = 0L
            sti = sy -winsize
            ysp -= sti
          endelse
          im[ 0:((sx-1) < winsizem), st: winsizem] = $
            image[0: ((sx-1)< winsizem), sti:(sy-1)]
          if sx lt winsize then for i = sx, winsizem do im[i, *] = lr_ave
          if sy lt winsize then for i = 0, st do im[*, i] = tb_ave
        endif else begin
;         Top right
          im1[winsize2: winsizem,winsize2:winsizem]=1.
          if sx le winsize then begin
            stx = winsize-sx
            xsp  += stx
            stix = 0L
          endif else begin
            stx = 0L
            stix = sx -winsize
            xsp -= stix
          endelse
          if sy le winsize then begin
            sty = winsize-sy
            ysp  += sty
            stiy = 0L
          endif else begin
            sty = 0L
            stiy = sy -winsize
            ysp  -=  stiy
          endelse
          im[stx: winsizem, sty:winsizem] = $
            image[stix: (sx-1), stiy :(sy-1)]
          if sx lt winsize then for i = 0, stx do im[i, *] = lr_ave
          if sy lt winsize then for i = 0, sty do im[*, i] = tb_ave
        endelse
        nsp = 0
        b = [[1,-1],[-1,1]] 
;        b=fltarr(nsq*2+1,nsq*2+1)
;        if nsq eq 0 then b=[[1,-1],[-1,1]] else $
;          for i=0.,2.*nsq do for j= 0.,2.*nsq do b[i,j]=(i-nsq)*(j-nsq)/nsq^2
        datar=convol_mm(im,b,/clip)
        extra_blur_psf=convol_mm(im1,b,/edge_truncate)
;        fhat = wienersq4(datar, -extra_blur_psf, gamma, gidkernsize=gidkernsize)/
;           (winsize*winsize)
        ss=size(datar)
        asize=ss[1]-3
        pe=FFT([[0,1,0,fltarr(asize)],[1,-4,1,fltarr(asize)],[0,1,0,fltarr(asize)], $
          [fltarr(asize+3,asize)]]  ,-1,/double)
        peconjpe=pe*CONJ(pe)
        g = FFT(datar,-1,/double)
        h = FFT( -extra_blur_psf, -1, /double)
        xxxx = conj(h)
        hxxxx=h*xxxx
        wz=where(hxxxx eq complex(0.,0.,/double)  ,count)
        if count ne 0 then hxxxx[wz]=complex(1.,1.,/double)
        wien2=(xxxx / (hxxxx + peconjpe* gammaf)) * g
        residual=g - h*wien2
        residual=abs(fft(residual ,1))
        rbarsq=total(residual^2)
        if ~keyword_set(quiet) then print, ' RBARSQ = '+ strtrim(rbarsq, 2)
        image=shift(real_part(fft(wien2,1)),ss[1]/2,ss[2]/2)
        if max(image, min = m) lt abs(m) then image = -temporary(image)
        clabel = clabel + 'Filter:Wiener;;Gamma:'+strtrim(gammaf, 2)
      endif else begin
        npt = (length gt 2) ? length : (length * maxspot)
        npt = round (npt)  
        clabel = clabel + 'Filter:Sav-Golay;;Length:'+strtrim(pixel*npt, 2)+ $
           ';;Degree:'+strtrim(degree, 2)
        if ~keyword_set(quiet) then begin
          print, 'Filter Length = '+strtrim(pixel*npt, 2)
          print, 'Filter Degree = '+strtrim(degree, 2)
        endif
        savgolfilter = savgol(npt, npt, 1, degree, doubl = dp) / savgol_norm
        for i = 0, ss[1]-1 do $
          image[i,*] = convol(reform(image[i, *]), savgolfilter, /edge_truncate)
        for i = 0, ss[2]-1 do $
          image[*, i] = convol(reform(image[*, i]), savgolfilter, /edge_truncate)
      endelse
;  get the spot 
; help,npt, xspot, yspot, lcbot, lcleft, xspot_bin, yspot_bin


      rxsp = round(xsp[1])
      rysp = round(ysp[1])
      if (halfw gt rxsp) || (halfw gt rysp) || (halfw + rxsp gt ss[1]) || $
         (halfw + rysp gt ss[2]) then begin
        halfw_old = halfw
        halfw = halfw < min([rxsp, rysp])
        halfw = halfw < (ss[1] -rxsp) -1
        halfw = halfw < (ss[2] -rysp) -1
        if ~keyword_set(quiet) then print, $
          '% PENUMBRAL:  Window size changed from '+strtrim(halfw_old, 2) + ' to ' + $
             strtrim(halfw, 2)
      endif
      i = rxsp + [-halfw, halfw]
      j = rysp + [-halfw, halfw]
      
      image = image[i[0]:i[1], j[0]:j[1]] 
      image /= max(image)
      ss = size(image)
      if ss[1] ne ss[2] then print, 'Image is not square'
;     center saved image at the maximum
      immax = max(image, k)
      ky = k/ss[1]
      kx = k mod ss[1]
      yvalues = (findgen(ss[2])-ky)*pixel
      yyy = [yvalues[0] ,pixel]
      arr2str, image_str, image, tlabel= 'Penumbral Imag', clabel = clabel, $
         (findgen(ss[1])-kx)*pixel, (findgen(ss[2])-ky)*pixel,  dx = pixel, dy = pixel, $
         xlabel = array.x1lab, ylabel= array.x2lab, v1lab ='Penumbral Image'     
      values = [0.25, 0.75]
      aspot = make_array(180)
      fwhmval = make_array(180)
      if n_elements(zero) eq 0 then zero = 1
      addit = (optsave eq 3) ? 180 : 0
      for iang = 0, 179 do begin
        datarotate = keyword_set(zero) ? rot(image, iang, 1.0, kx, ky, /interp, pivot = pivot) : $
           ROT(image, iang, /interp)
;       Center will move unless both pivot and zero are set.
        junk = max(datarotate, k)
        kxx = k mod ss[1]
        i2w, yyy, datarotate[kxx, *], maxwdf, clab = 'Vertical lineout, Rotate '+ strtrim(iang, 2) +' degrees', $
          xlab = array.x2lab
        if (optsave eq 1 or optsave eq 3) then xfr, maxwdf, saveit
        fw = fwhm (maxwdf, /quiet)
        edgescan = total(TOTAL(DataRotate, 1, /double), /cum, /double)
        edgescan = smooth(edgescan, filter, /edge_truncate)
        n = n_elements(edgescan)
        edgescan = edgescan[sort(edgescan)]
        edgescan = edgescan/edgescan[n-1]
        if (optsave eq 2 or optsave eq 3) then begin
          i2w, yyy, edgescan, saveit+addit, $
          clab = 'AWE Vertical Integral, Rotate '+ strtrim(iang, 2) +' degrees' , xlab = array.x2lab
          saveit += 1
        endif else if optsave eq 1 then saveit += 1
        e = findex(edgescan, values)
        aspot[iang] = fudge * abs(e[1]-e[0]) * pixel
        fwhmval[iang] = fw
      endfor
      i2w, [0,1], aspot, wdf, clab = 'AWE Spotsize vs Angle - '+clabel, xlab='Angle-deg', $
        ylab = 'AWE Spot Size(mm)'
      wdf += 1
      i2w, [0,1], fwhmval, wdf, clab = 'FWHM vs Angle - '+clabel, xlab='Angle-deg', $
        ylab = 'FWHM Size(mm)'
      wdf += 1
      release = float(!version.release)
      if (release ge 8.0) then begin
        junk = moment(fwhmval, max =2,  sdev = sdev1, mean  = spot1) 
        junk = moment(aspot, max =2,  sdev = sdev2, mean  = spot2) 
      endif else begin
        spot1 = mean(fwhmval, /double)
        sdev1 = stddev(fwhmval, /double)
        spot2 = mean(aspot, /double)
        sdev2 = stddev(aspot, /double)
      endelse
      spot =[spot1, spot2]
      sdev =[sdev1, sdev2]
      if keyword_set(quiet) then return
      print, 'FWHM Ave Spot size = ' + strtrim(spot[0], 2) +' +/- '+strtrim(sdev[0], 2)
      print, 'AWE Ave Spot size = ' + strtrim(spot[1], 2) +' +/- '+strtrim(sdev[1], 2)
      print, 'AWE X-Spot size = '+strtrim(Xspot, 2)
      print, 'AWE Y-Spot size = '+strtrim(Yspot, 2)
      
end
