; ------------------------------------------------------------------------
; $Id: plotcon.pro,v 1.15 2008/06/24 17:57:12 lpmix Exp $
; ------------------------------------------------------------------------
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
 pro plotcon, ain, x, y,zvalue,  color=color, thick = thick, $
    solid_model = solid_model, pec = pec, zerothick = zerothick, $
    xrange = xrange, yrange = yrange, _extra = extrastuff, $
    showdefault = showdefault, setdefault = setdefault, unset = unset,$
    header= header, ignore = ignore, aspect = aspect, polarplot = polar, $
    xtitle = xtitle, ytitle= ytitle , title = title, overplot = overplot, $
    fill = fill, orientation = orientation, line_fill = line, noplot = noplot, $
    axis_add = axis_add, quiet = quiet, block = block, spawn= spawnit, $
    closed_only = closed_only, reset_conductor = reset_cond, nodata = nodata
;
;+
; NAME:
;       PLOTCON
;
; PURPOSE:
;       Plot conductor geometry
; CATEGORY:
;       Input/Output
; CALLING SEQUENCE:
;       PLOTCON, A [, X] [, Y] [, ZVALUE] [, YRANGE = YRANGE] [, THICK = THICK]
;           [, XRANGE = XRANGE] [, COLOR = COLOR] [, TITLE = TITLE] [, /ASPECT]
;           [, XTITLE = XTITLE] [, YTITLE = YTITLE] [, /SOLID_MODEL] [, /AXIS_ADD]
;           [, POLARPLOT = polarplot] [, SHOWDEFAULT = showdefault]
;           [HEADER = header] [, SETDEFAULT = setdefault] [, UNSET = unset]
;           [, OVERPLOT = OVERPLOT] [, IGNORE = ignore]  [, /ZEROTHICK] [, /PEC]
;           [, FILL = fill] [, /NOPLOT] [, CLOSED_ONLY = closed_only]
;           [, RESET_CONDUCTOR = reset_cond ] [, NODATA = nodata]
;           [, QUIET = quiet] [, BLOCK = block] [, _EXTRA = extrastuff]
; INPUTS:
;       A     IDL conductor  structure (type = 4)
;       X     Abscissa values (1, 2, or 3 x, y, or z coordinates)
;       Y     Ordinate values (1, 2, or 3 x, y, or z coordinates)
;       ZVALUE Value of the coordinate in the third dimension. In the case of
;             two dimensional data, enter 0.
;     Note: An error tolerance of the maximum range of the conductor data
;           maxp = max(a.plane(1, *, *))
;           minp = min(a.plane(0, *, *))
;           err = 1.52588e-5*( max([maxp-minp, abs(maxp), abs(minp)]))
;           used in determining whether a plane crosses a conductor.
;     NOTE: If overplot is not zero, then x and y, zvalue will be determined
;             from he previous plot.
;     NOTE: If the actual plot range exceeds the range specified by xrange or
;           yrange and only one of these values has been specified, then
;           then conductors may appear to begin in space.  That is, if only
;           one plot range is specified (xrange or yrange) then the data is
;           roughly windowed to that specification to determine the appropriate
;           value for the other plot dimension.
;
; KEYWORD PARAMETERS:
;       OVERPLOT - If not zero, indicates no new grid is to be drawn.
;       THICK    - Line Thickness default is 2
;       COLOR    - All lines will be drawn with a single color.
;                   The colors are: black(0), red(1), green(2), blue(3),
;                   purple(4), orange(5), light blue(6), yellow(7) and white(8).
;       TITLE, XTITLE and YTITLE If grid is not zero, these will be used for
;                 axis and the plot labels.  If these are not string arrays,
;                 the title, xtitle and ytitle used for the plots will be used.
;       SOLID_MODEL - If set, a solid model will be shown
;                  Note: Structure must have the solod model information.
;                        SEE Also:  SOLID
;                  Note: See BLOCK and ADD_AXIS keywords
;                  To return the model, use following:
;                    model=1
;                    plotstr, str, solid_model = model
;       AXIS_ADD - Ignored unless SOLID_MODEL is set.  If set, an axis will be
;                  added to the model.  If AXIS_ADD is a structure, it will be
;                  passed to ADD_AXIS as an _EXTRA keyword.
;       BLOCK    - If set, the solid model is displayed in a blocking widget
;                  If undefined, then BLOCK defined by the following:
;                       DEFSYSV, "!callable_idl", exists = BLOCK
;       ASPECT   - If set, the ordinate and abscissa will have the same relative
;                  scale
;       POLARPLOT - If set, a polar plot will be generated.
;                 For the SPARE[0] eq 1, no polar plot is generated
;                 The coordinates for cylindrical and spherical data are as follows:
;                   R     - Radius vector
;                   PHI   - Angle between the radius vector projected on the X-Y
;                           plane and the X axis
;                   Z     - Axis normal to the X-Y plane
;                   THETA - Angle between the Z axis and the Radius vector.
;                 For the SPARE[0] eq 2, the cylindrical data will be converted
;                   to cartesian and plotted; Coordinates=(R, Phi, Z)
;                   X and Y must be 1 and 2 in any order, but they will be
;                     set to 1 and 2 respectively.
;                   The conversions are:
;                     X = R * Cos(Phi)
;                     Y = R * Sin(Phi)
;                 For the SPARE[0] eq 3, the spherical data will be converted
;                   to cartesian and plotted; Coordinates=(R, Theta, Phi)
;                   The conversions to cartesian are:
;                     X = R * Cos(Phi) * Sin(Theta)
;                     Y = R * Sin(Phi) * Sin(Theta)
;                     Z = R * Cos(Theta)
;                   For (X, Y) equal to (1,3), Theta will be set and the abscissa
;                     and ordinates will be X and Y, respectively.
;                   For (X, Y) equal to (1,2), PHI will be set.  The plot will be
;                     rotated to the slice value, and the abscissa and ordinates
;                     will be X and Z, respectively.  Because of the rotation,
;                     (PHI = 0) in the equation for X above.
;
;                   ASPECT will be set unless it is explicitly unset.
;                   Current version is for type 4 data only.
;       X,YRANGE - Axis range(standard IDL meaning)
;       HEADER    If specified a plot header, consisting of the current date
;                 and the current file name, is displayed in the lower left
;                 corner of the plot.  See the HEADER command to set default
;                 HEADER parameters (character size, spacing, alignment).
;                 DEFAULT is HEADER = 1
;                 NOTE: For X-windows screen dumps, many will want to increase
;                      the character size with the command:
;                      HEADER, /set, charsize = 0.8
;                      (Default is 0.6 of current size)
;     SETDEFAULT- If present and not zero, then the following parameters will be
;                 saved and used as the default values in all future calls to PLO
;                 If a keyword is specified, it will always be used rather than
;                 the saved values.
;     UNSET     - If present and not zero, will set all save values of keywords
;                 to their default values.
;                 Normal default values are listed below.
;                 Keyword     Default value
;                 HEADER       1
;                 XRANGE       0
;                 YRANGE       0
;                 COLOR        0 (!p.color)
;                 THICK        2
;
;     SHOWDEFAULT-Show the present default plot values.
;     IGNORE    - If set and an overplot, ignore the previous values.
;                 This includes X, Y, Z, POLARPLOT, etc.
;     ZEROTHICK - If set, zero thickness planes will be plotted.
;     PEC       - If set, zero thickness planes will be plotted.
;                 Results are the same with either zerothick or PEC set.
;     FILL      - If set, fill the conductor/dielectric.
;                 If set, conductor will be stored in the TQPFIDL common block.
;                 No attempt is made to close conductors.  POLYFILL will simply
;                 connect the first and last points.
;                 QSEDIT, /COND may be used to edit these values and
;                 TQPLOTCON can be used to plot these conductors.
;     NOPLOT    - Applies only to the fill keyword.  If set the conductors will
;                 not be plotted but they will be stored in the TQPFIDL structures.
;     CLOSED_ONLY - If set, fill only if all conductors are closed conductors
;                 If any conductors are not closed, ignore fill.  All conductors
;                 will be saved in the TQPFIDL common block.
;     RESET_COND- If set, RESET will be set in the call to TQCOND.
;                 Default:  RESET_COND =1
;     LINE_FILL - If set, dielectric is filled with lines.  Ignored unless
;                 FILL is set.
;                 If LINE_FILL not equal to 1 then ORIENTATION = LINE_FILL.
;                 POLYFILL is repeated N_Elements(ORIENTATION) Times.
;                 IF LINE_FILL equal to 1 then ORIENTATION=[+45, -45]
;                 Note: LINE_FILL is set if it is not zero or an array.
;                 Default is solid fill or LINE_FILL =0.
;     NODATA    - Plot only the axis.
;     ORIENTATION - Ignored unless fill is set.
;                 ORIENTATION is passed to POLYFILL if FILL is set.
;                 This is same as LINE_FILL, but there is no default.
;     QUIET     - If set, do not print warning messages.
;     SPAWN     - If set, spawn XOBJVIEW to another process
;       _EXTRA    additional parameters to the plot command. POSITION,
;                 (X,Y)STYLE, (X,Y)TYPE have the standard IDL meaning.
; OUTPUTS:
;       Designated datasets are plotted.
; EXAMPLE:
;       Plot a TQ, 2D cylindricl structure A, with Z horizontal and R vertical.
;         Use a limits: X = [0,0.064] and Y=  [0, 0.14]
;         PLOTCON, A, 3, 1, xrange = [0,0.064],yrange = [0, 0.14]
;
;       Overplot conductor geometry from structure 4
;         PLOTSTR, 4, /over
;
;       Plot a solid representation of the conductors in dataset 4 and 5 in the current
;       PFF file with an axis
;         READSTR, 23, [4,5], /solid
;         PLOTSTR, 23, /solid, color = [1, 3], /axis
;
;       Plot a filled conductor from structure 32 with Radius horizontal and Z vertical.
;       at an angle of !pi/18
;         PLOTSTR, 32, /reset, 1, 3, !pi/18, /fill
;
;       Plot a filled conductor from structure 32 with Radius horizontal and Z vertical.
;       at an angle of !pi/18 and add similar data from structure 33 and plot the data.
;       NOTE:  PLOTSTR with /NODATA is used to make the grid.
;         PLOTSTR, 32, /reset, 1, 3, !pi/18, /fill ; /RESET clears the TQPFIDL conductor
;         PLOTSTR, 33,  1, 3, !pi/18, /fill ; these conductors are added to the TQPFIDL cond.
;         QSEDIT, /cond ; if necessary for reasonable fill
;         PLOTSTR, 33, 1, 3, !pi/18, /NODATA
;         TQPLOTCON, /FILL
;
;
;       Add the conductor in structure 33 to the above conductor 32 and plot the result.
;       Before plotting check the conductors to make sure that the first and last points
;       are satisfactory.  Make the first 3 conductors red and the next 2 green.
;         PLOTSTR, 33, 1, 3, !pi/18, /fill
;         TQEDIT, /cond
;  <or>   QSEDIT, /cond
;         ; generate a grid
;         PLOTSTR, 33,  1, 3, !pi/18, /NODATA, title = 'Anode and Cathode'
;         TQPLOTCON, /fill, color = [1, 1, 1, 2, 2]
;   Note:  Last two commands may be replaced with the following:
;         TQPLOTCON, /fill, color = [1, 1, 1, 2, 2], $
;             grid ={title:'Anode and Cathode', xtitle:'Radius', ytitle:'Z-Axis'}
;
; MODIFICATION HISTORY:
;      Original version, L. P. Mix, March 5, 1993
;      Modified: L. P. Mix, May 23, 2006
;                Allow for solid fill for conductors.
;-
;
; insert a common block
;
@qscom.dat
;
;  Common block
@qsstrcom.dat
;
common plotcon__defaults, header_save, xrange_save, yrange_save, color_save, $
                  thick_save
      on_error, 1
;
common plot_grid_last, yeng_scale, xeng_scale
      if n_elements (structinit) eq 0 then qsstruct_init
      if n_elements (qsplotinit) eq 0 then qsinit
;
;
      if keyword_set(unset) or n_elements(header_save) eq 0 then begin
        HEADER_SAVE = 1
        XRANGE_SAVE = 0
        YRANGE_SAVE = 0
        COLOR_SAVE = 0
        THICK_SAVE = 2
      endif
      if n_elements (polarplot) eq 0 then polarplot = 0
      if keyword_set (setdefault) then begin
        if n_elements(header) gt 0 then header_save = header
        if n_elements(xrange) gt 0 then xrange_save = xrange
        if n_elements(yrange) gt 0 then yrange_save = yrange
        if n_elements(thick) gt 0 then thick_save = thick
        if n_elements(color) gt 0 then color_save = color
      endif
      if keyword_set(showdefault) then begin
         print, '% PLOTCON: Default parameters for the PLOTCON command are:'
         print, '    XRANGE     = ' , strtrim (xrange_save, 2)
         print, '    YRANGE     = ' , strtrim (yrange_save, 2)
         print, '    HEADER     = ' , strtrim (HEADER_SAVE, 2)
         print, '    COLOR      = ' , strtrim ( COLOR_SAVE, 2)
         print, '    THICK      = ' , strtrim ( THICK_SAVE, 2)
      endif
      npar = n_params()
      if npar eq 0 and (keyword_set (setdefault) or keyword_set(unset) $
         or keyword_set(showdefault) ) $
        then return
;
      if npar eq 0 then begin
        print, '% Format:', $
   'PLOTCON, A [, X] [, Y] [, ZVALUE] [, YRANGE = YRANGE] [, THICK = THICK]'
print,'        [, XRANGE = XRANGE] [, COLOR = COLOR] [, TITLE = TITLE]'
print,'        [, XTITLE = XTITLE] [, YTITLE = YTITLE] [, IGNORE = ignore]'
print,'        [, SETDEFAULT = setdefault] [, UNSET = unset] [, /POLAR]'
print,'        [, SHOWDEFAULT = showdefault] [HEADER = header] [, /PEC] [,/NODATA]'
print,'        [, OVERPLOT = OVERPLOT] [, _EXTRA = extrastuff] [, /ZEROTHICK]'
print,'        [, FILL = fill] [, ORIENTATION = orientation] [, LINE_FILL = line]'
print,'        [, CLOSED_ONLY = closed_only] [, RESET_CONDUCTOR = reset_cond]

        return
      endif

;    make sure we don't corrupt input data
;     now get the data  -- check that we have a structure of the proper type
      if n_elements(ain) eq 0 then begin
        if not keyword_set(quiet) then $
          print, '% PLOTCON: Structure must be defined'
        return
      endif
      if n_elements(header) eq 0 then header = header_save
      if n_elements(xrange) eq 0 then xrange = xrange_save
      if n_elements(yrange) eq 0 then yrange = yrange_save
      if n_elements(color) eq 0 then color = color_save
      if color[0] eq 0 then color = !p.color
      if n_elements(thick) eq 0 then thick = thick_save
      if keyword_set (solid_model) then begin
        b = size(ain)
        if b(b(0)+1) ne 8 then begin
          a = ain
          str=strvalid(a, /valid, np = np)
          if np eq 0 then begin
            if not keyword_set(quiet) then $
            print, '% PLOTCON: Valid structure must be provided: ' , a
            return
          endif
        endif else str = ain
        plotcon_solid, str, odata, xrange= xrange, yrange = yrange, $
           color = color, _extra=extrastuff
        if keyword_set(axis_add) then begin
          if size(axis_add, /type) eq 8 then extrastuff = axis_add
          odata2 = add_axis(odata, _extra= extrastuff)
          odata = temporary(odata2)
        endif
        solid_model = odata
        if keyword_set(spawnit) then begin
          if obj_valid(odata) then xobjview, odata
          return
        endif
        if n_elements(block) eq 0 then $
          defsysv, "!callable_idl", exists = block
        if obj_valid(odata) then xobjview, odata, block = block
        return
      endif
      a = ain
      b = size(a)
      if b(b(0)+1) ne 8 then begin
        if a gt 0 and a le maxstructure then begin
          s2i, a, a
          b = size (a)
          if b(b(0)+1) ne 8 then begin
            if not keyword_set(quiet) then print, $
              '% PLOTCON:  First parameter must be a structure or a QS array number'
            return
          endif
        endif  else begin
          if not keyword_set(quiet) then print, $
            '% PLOTCON:  First parameter must be a structure or a QS array number'
          return
        endelse
      endif
      if a.type ne 4 then begin
        if not keyword_set(quiet) then $
          print, '% PLOTCON:  Only Conductor type structures may be plotted'
        return
      endif
      if n_elements(zvalueold) eq 0 then zvalueold = 0
      if n_elements (overplot) eq 0 then overplot  = 0
      if n_elements (xoldvalue)  eq 0 then begin
        xoldvalue = 0
        yoldvalue = 0
      endif

      if n_elements(ignore) eq 0 then ignore = 0
      if keyword_set(ignore) then ignore = 1 else ignore = 0
      coordinate = a.spare[0]
      if coordinate eq  1 then begin
        if keyword_set(polar) then begin
          if not keyword_set(quiet) then $
          print, '% PLOTCON: Cannot do a polar plot with a cartesian grid'
           return
        endif
      endif

      if n_elements(lctwodim) eq 0 then lctwodim = 0
      if overplot ne 0 then begin
        if n_elements (x) eq 0 then x = xoldvalue
        if x ne xoldvalue and ignore eq 0 then begin
          if not keyword_set(quiet) then begin
           Print, '% PLOTCON: WARNING: This overplot requested with X-axis of ', x, $
                ', present value is', xoldvalue
           print, '           Requested data will be plotted'
           hak, mes = ' Hit any key to continue'
          endif
        endif
        if n_elements(y) eq 0 then y = yoldvalue
        if y ne yoldvalue and ignore eq 0 then begin
          if not keyword_set(quiet) then begin
            Print, '% PLOTCON: WARNING: This overplot requested with Y-axis of ', y, $
                ', present value is', yoldvalue
            print, '           Requested data will be plotted'
          endif
           hak, mes = ' Hit any key to continue'
        endif
        if n_elements (zvalue) eq 0 then zvalue = zvalueold
        if zvalue(0) ne zvalueold(0) and ignore eq 0 then begin
          if not keyword_set(quiet) then begin
          Print, '% PLOTCON: WARNING: This overplot requested value in third dimension'
          Print, ' of ', zvalue, ', present value is', zvalueold
          print, '           Requested data will be plotted'
          endif
            hak, mes = ' Hit any key to continue'
        endif
        if n_elements(polar) eq 0 then polar = polarplot
      endif  else begin
        if n_elements(polar) eq 0 then polar = 0
      endelse
;
      nx = n_elements (x)
      if nx eq 0 then begin
        if not keyword_set(quiet) then $
          Print, '% PLOTCON: X array must be defined'
        return
      endif
      xt = x[0]
      if xt eq 0 then begin
        if not keyword_set(quiet) then $
        Print , '% PLOTCON: Please renter the command and specify a nonzero x parameter'
        return
      endif
      ny = n_elements (y)
      if ny eq 0 then begin
        if not keyword_set(quiet) then $
          Print, '% PLOTCON: Y array must be defined'
        return
      endif
      yt = y[0]
      if yt eq 0 then begin
        if not keyword_set(quiet) then $
        Print , '% PLOTCON: Please renter the command and specify a nonzero y parameter'
        return
      endif
      if xt lt 1 or xt gt 3 then begin
        if not keyword_set(quiet) then begin
          print, '% PLOTCON: Abscissa coordinate value must be between 1 and 3'
          print, '           Present value is ', xt
        endif
        return
      endif
      if yt lt 1 or yt gt 3 then begin
        if not keyword_set(quiet) then begin
          print, '% PLOTCON: Ordinate coordinate value must be between 1 and 3'
          print, '           Present value is ', yt
        endif
        return
      endif
      zt = 6 -xt-yt
      if overplot eq 0 then begin
        if n_elements (zvalue) eq 0 then begin
         if not keyword_set(quiet) then begin
          print, '% PLOTCON: Coordinate value in the third dimension must be specified'
          print, '           If this is Two-D data, enter zero (0).'
         endif
          return
        endif
      endif
;
      if keyword_set(polar) then begin
        if ((xt+yt) ne 3)  and (coordinate eq 2) then begin
          if not keyword_set(quiet) then begin
          print, '% PLOTCON: Cylindrical data:', xt, yt
          print, '           X and Y values must be 1 and 2 for a polar plot'
          endif
          return
        endif else if  ((xt+yt) gt 4)  and (coordinate eq 3) then begin
          if not keyword_set(quiet) then begin
          print, '% PLOTCON: Spherical data:', xt, yt
          print, '           X and Y values must be (1,2) or (1,3) for a polar plot'
          endif
          return
        endif
        xt = min([xt, yt], max = yt)
        if n_elements(aspect) eq 0 then aspect = 1
      endif
      xoldvalue = xt
      yoldvalue = yt
      zt= 6-xt-yt
      zplaneold = zt
      polarplot = polar
      nn = fix (6-xt-yt)
      if n_elements (zvalue) gt 1 then begin
        zvalue = float([min(zvalue, max = mv), mv])
        zvalue(0) = max([zvalue(0), min(a.plane(0, zplaneold-1, *))])
        zvalue(1) = min([zvalue(1), max(a.plane(1, zplaneold-1, *))])
        zvalue=total(zvalue)/n_elements (zvalue)
      endif

      zvalueold = zvalue
      maxp = max(a.plane(1, *, *))
      minp = min(a.plane(0, *, *))
      if keyword_set (pec) then zerothick = 1
      npzero = 0L
      err = 1.52588e-5*( max([maxp-minp, abs(maxp), abs(minp)]))
      if nn eq 3 then begin
        if keyword_set(zerothick) then begin
          arrzero = a.n3
          if arrzero[0] ge 0 then begin
            n= where(a.plane(0,2, arrzero) le zvalue+err and $
                 a.plane(1,2, arrzero) ge zvalue-err, npzero)
            if npzero gt 0 then arrzero= arrzero[n]
          endif
        endif
        arr = [a.n1,a.n2]
        n = where (arr ge 0, np)
        if np gt 0 then begin
          arr = arr(n)
            n = where(a.plane(0,2, arr) le zvalue+err and $
                 a.plane(1,2, arr) ge zvalue-err, np)
            if np gt 0 then arr = arr(n)
        endif
      endif else if nn eq 2 then begin
        if keyword_set(zerothick) then begin
          arrzero = a.n2
          if arrzero[0] ge 0 then begin
            n= where(a.plane(0,1, arrzero) le zvalue+err and $
                 a.plane(1,1, arrzero) ge zvalue-err, npzero)
            if npzero gt 0 then arrzero= arrzero[n]
          endif
        endif
        arr = [a.n1,a.n3]
        n = where (arr ge 0, np)
        if np gt 0 then begin
          arr = arr(n)
            n = where( a.plane(0,1, arr) le zvalue+err and $
               a.plane(1,1, arr) ge zvalue-err, np)
            if np gt 0 then arr = arr(n)
        endif
      endif else begin
        if keyword_set(zerothick) then begin
          arrzero = a.n1
          if arrzero[0] ge 0 then begin
            n= where(a.plane(0,0, arrzero) le zvalue+err and $
                 a.plane(1,0, arrzero) ge zvalue-err, npzero)
            if npzero gt 0 then arrzero= arrzero[n]
          endif
        endif
        arr = [a.n2,a.n3]
        n = where (arr ge 0, np)
        if np gt 0 then begin
          arr = arr(n)
            n = where(a.plane(0 ,0, arr) le zvalue+err and $
               a.plane(1,0, arr) ge zvalue-err, np)
            if np gt 0 then arr = arr(n)
        endif
      endelse
; logic below modified to check for Z value
      if a.size(0) gt 0 then begin
        nslant = a.size(0)
        zslant = reform (a.slant(zt-1, *,*))
        zval = zvalue+[err, -err]
        ok = where((zval[0] ge zslant[0,*] and zval[1] le zslant[1,*]) or $
                   (zval[1] le zslant[0,*] and zval[0] ge zslant[1,*]), nok)
        nslant = nok
        if nslant gt 0 then begin
          xslant = reform (a.slant[xt-1, *, ok])
          yslant = reform (a.slant[yt-1, *, ok])
          zslant = reform (a.slant[zt-1, *, ok])
          slnorm = reform(a.slnorm[*, ok])
          for ind = 0, nok-1 do begin
            if slnorm [zt-1, ind] ne 0 then begin
              frac = (zvalue - zslant[0, ind])/ (zslant[1, ind] - zslant[0, ind])
              if slnorm[xt-1, ind] ne 0 then $
                xslant[*,ind]=frac*(xslant[1,ind] - xslant[0,ind])+ xslant[0, ind]
            if slnorm[yt-1, ind] ne 0 then $
                yslant[*,ind]=frac*(yslant[1,ind] - yslant[0,ind])+ yslant[0, ind]
            endif
          endfor
          if keyword_set(polar) then begin
            xx = temporary(xslant)
            yy = temporary(yslant)
            if coordinate eq 2 then begin
;             Cylindrical
;             xt = 1 and yt = 2 for polar
              xslant = xx * cos(yy)
              yslant = xx * sin(yy)
            endif else begin
;           coordinate eq 3 - Spherical
;           this does not work
;              xx  = reform(a.slant[0, *, ok])
;              if yt eq 3 then begin
;                yy = reform(a.slant[2, *, ok])
;                sth = sin(zvalue)
;                yslant = xx*sin(yy)*sth
;                xslant = xx*cos(yy)*sth
;              endif else begin
;                zz = reform(a.slant[1, *, ok])
;                yslant = xx*cos(zz)
;                xslant = xx*sin(zz)
;              endelse
              print, '% PLOTCON: Slants currently not supported with Spherical'
              return
            endelse
          endif
        endif
      endif else nslant = 0
      if np eq 0  and nslant eq 0 and npzero eq 0 then begin
        if not keyword_set(quiet) then begin
          Print, '% PLOTCON: No conductor available for these x, y, zvalue parameters'
          print, x, y, zvalue
        endif
        return
      endif
      if keyword_set (polar) then begin
        xx = reform(a.plane[*, 0, arr])
        if coordinate eq 2 then begin
;         Cylindrical
          yy = reform(a.plane[ *, 1, arr])
          cy = cos(yy)
          yy = xx * sin(yy)
          xx = temporary(xx) * cy
        endif else begin
;         coordinate eq 3 - Spherical
          if yt eq 3 then begin
            ph = reform(a.plane[*, 2, arr])
            sth = sin(zvalue)
            yy = xx*sin(ph)*sth
            xx = xx*cos(ph)*sth
          endif else begin
            zz = reform(a.plane[*, 1, arr])
            yy = xx*cos(zz)
            xx = xx*sin(zz)
          endelse
        endelse
      endif else begin
        xx = reform (a.plane[*, xt-1, arr])
        yy = reform (a.plane[*, yt-1, arr])
      endelse

      if npzero gt 0 then begin
        st = 2*np
        send = st/2 + 4*NPzero
        if np eq 0 then begin
          xx = make_array(8*npzero)
          yy = make_array(8*npzero)
        endif else begin
          xx = reform(xx, st, /overwrite)
          yy = reform(yy, st, /overwrite)
          xx = [xx, make_array(8*npzero)]
          yy = [yy, make_array(8*npzero)]
        endelse

        for i = 0L, npzero-1 do xx[st+8*i:st+8*i+7] = $
           a.plane[[0, 1, 1, 1, 1, 0, 0, 0], xt-1, arrzero[i]]
        for i = 0L, npzero-1 do yy[st+8*i:st+8*i+7] = $
           a.plane[[0, 0, 0, 1, 1, 1, 1, 0], yt-1, arrzero[i]]
        xx = reform(xx, 2, send, /overwrite)
        yy = reform(yy, 2, send, /overwrite)
        np = np + 4*NPzero
      endif
;
      if overplot ne 0 then header = 0
      if !p.multi(0) ne 0 then header = 0
      if header ne 0 then begin
        filename = strtrim(a.file, 2)
        if (strlen(filename) eq 0) then header_text='Plot: ' + thin_date() $
        else  begin
          ss = file_info(filename)
          if ss.exists eq 1 then begin
            cdate = thin_date(ss.ctime)
            header_text= ['Plot: '+thin_date() +'; Data: ' + cdate,  $
                   'File: ' +filename]
          endif else header_text=['Plot: ' + thin_date(), 'File: ' +filename]
        endelse

      endif
;
      if !p.thick gt 0 then pthick = !p.thick * thick else pthick = thick
      if overplot eq 0 then begin
        yeng_scale = 1.00  &  xeng_scale = 1.00
        if n_elements (xrange) lt 2 then begin
          xrange = [min(xx, max = mx), mx]
          if nslant gt 0 then begin
            xr = [min(xslant,max = mx), mx]
            xrange = [min([xr(0), xrange(0)]), max([xr(1), xrange(1)])]
          endif
          lcxrange = 0
        endif else begin
          xrange = xrange(0:1)
          lcxrange = 1
        endelse
        if n_elements (yrange) lt 2 then begin
          yrange = [min(yy, max = my), my]
          if nslant gt 0 then begin
            yr = [min(yslant, max = my), my]
            yrange = [min([yr(0), yrange(0)]), max([yr(1), yrange(1)])]
          endif
          lcyrange = 0
        endif  else begin
          yrange = yrange(0:1)
          lcyrange = 1
        endelse
;
        if ((lcyrange  ne 0) xor (lcxrange ne 0))   then begin
          if    keyword_set(polar) then begin
;         ignore the slants for this
            if lcyrange eq 0 then begin
              ok = where (xx ge xrange(0) and  xx le xrange(1), nok)
              if nok eq 0 then begin
                if not keyword_set(quiet) then $
                print, '% PLOTCON: No plots exist form this range of X'
                return
              endif
              yrange(0) = min(yy(ok), max = ymax)
              yrange(1) = ymax
            endif else begin
              ok = where (yy ge yrange(0) and  yy le yrange(1), nok)
              if nok eq 0 then begin
                if not keyword_set(quiet) then $
                print, '% PLOTCON: No plots exist form this range of Y'
                return
              endif
              xrange(0) = min(xx(ok), max = xmax)
              xrange(1) = xmax
            endelse

          endif else begin
          ok = where( xx(0,*) le xrange(1) and xx(1,*) ge xrange(0), np)
          if np gt 0 then begin
            xx = xx(*, ok)
            yy = yy(*, ok)
          endif else if nslant eq 0 then begin
            if not keyword_set(quiet) then $
            print, '% PLOTCON: No conductor available for these values of xrange'
            return
          endif
          ok = where( yy(0,*) le yrange(1) and yy(1,*) ge yrange(0), np)
          if np eq 0 and nslant eq 0 then begin
            if not keyword_set(quiet) then $
            print, '% PLOTCON: No conductor available for these values of yrange'
            return
          endif
          xx = xx(*, ok)
          yy = yy(*, ok)
          if nslant gt 0 then begin
            ytemp = reform (a.plane(*, yt-1, a.n0))
            xtemp = reform (a.plane(*, xt-1, a.n0))
            ok= where(xtemp(0,*) lt xrange(1) and xtemp(1,*) gt xrange(0), $
                nslant)
            if nslant gt 0 then begin
              xtemp = xtemp(*, ok)
              ytemp = ytemp(*, ok)
              xslant = xslant(*, ok)
              yslant = yslant(*, ok)
              ok= where(ytemp(0,*) lt yrange(1) and ytemp(1,*) gt yrange(0), $
                nslant)
              if nslant gt 0 then begin
                xslant = xslant(*, ok)
                yslant = yslant(*, ok)
              endif
            endif
          endif
          if nslant eq 0 and np eq 0 then begin
            if not keyword_set(quiet) then $
            Print, '% PLOTCON: No conductor available for these values of plot range'
            return
          endif
          if nslant gt 0 and np gt 0 then begin
            if lcxrange eq 0 then $
              xrange=[min([min(xx, max = mxx), min(xslant, $
               max = msl)]), max([mxx, msl])]
            if lcyrange eq 0 then $
              yrange=[min([min(yy, max = myy), min(yslant, $
               max = msl)]), max([myy, msl])]
          endif else if np gt 0 then begin
            if lcxrange eq 0 then $
              xrange=[min(xx, max=mxx), mxx]
            if lcyrange eq 0 then $
              yrange=[min(yy, max= myy), myy]
          endif else begin
            if lcxrange eq 0 then $
              xrange=[min(xslant, max=msl), msl]
            if lcyrange eq 0 then $
              yrange=[min(yslant, max=msl), msl]
          endelse
          endelse
        endif
        if lcxrange eq 0 then begin
          epsil = (xrange(1)-xrange(0))/32768.
          xrange= [xrange(0)+epsil , xrange(1)-epsil]
        endif
        if lcyrange eq 0 then begin
          epsil = (yrange(1)-yrange(0))/32768.
          yrange= [yrange(0)+epsil , yrange(1)-epsil]
        endif
;
        xlab = a.xlab(xt-1)
        ylab = a.xlab(yt-1)
        if n_elements(xtitle) eq 0 then xtitle = xlab
        b = size (xtitle)
        if b(b(0)+1) ne 7 then xtitle = xlab
        if n_elements(ytitle) eq 0 then ytitle = ylab
        b = size (ytitle)
        if b(b(0)+1) ne 7 then ytitle = lab
        if n_elements(title) eq 0 then title = a.clab
        b = size (title)
        if b(b(0)+1) ne 7 then title = a.clab
;       see if we have a horizontal or vertical line
        if xrange(1) le xrange (0) then xrange = [.98, 1.02]*xrange(0)
        if yrange(1) le yrange (0) then yrange = [.98, 1.02]*yrange(0)
;        now make the grid
        if keyword_set(aspect) then begin
        pos = get_aspect(xrange, yrange)
        plot, xx(*,0), yy(*,0),/nodata, thick = pthick, xtitle = xtitle, $
          title = title, ytitle = ytitle, xrange = xrange, yrange = yrange, $
          position = pos, /xst, /yst, _extra = extrastuff
        endif else $
        plot, xx(*,0), yy(*,0),/nodata, thick = pthick, xtitle = xtitle, $
          title = title, ytitle = ytitle, xrange = xrange, yrange = yrange, $
          _extra = extrastuff
        if header ne 0 then header, header_text
      endif
;
      if keyword_set (nodata) then return
      if color[0] gt 0 and color[0] lt 8 then loadxyct, /quiet, /freeze
      if keyword_set(fill) then begin
        if keyword_set(line) then begin
          if n_elements(line) eq 1 && line[0] eq 1 then orient= [45, -45] else $
            orient= line
        endif else if n_elements(orientation) gt 0 then begin
          fill = 1
          orient = orientation
        endif
        nor = n_elements(orient)
        nxx = n_elements(xx)
        nxs = n_elements(xslant)
        nxfin = (nxx+nxs)/2
        if nxx gt 0 then begin
          xf = reform(xx, nxx, /overwrite)
          yf = reform(yy, nxx, /overwrite)
          if nxs gt 0 then begin
            xf = [xf,  reform(xslant, nxs, /overwrite)]
            yf = [yf,  reform(yslant, nxs, /overwrite)]
          endif
        endif else begin
          xf = reform(xslant, nxs, /overwrite)
          yf = reform(yslant, nxs, /overwrite)
        endelse
        xxx = make_array(nxfin+64)
        yyy = make_array(nxfin+64)
;       IND = 1 means point has been taken
        ind = make_array(2, nxfin, value = 0)
        ind[*, 0]= 1
;       nc is number of points in XXX, YYY
        nc = 2
        xxx[0:1] = xf[0:1]
        yyy[0:1] = yf[0:1]
;       Use lc_up as a flag.  Look in both directions from the initial line segment
;         When no points found LC_UP=-1
        lc_up = 1
        nok = 1
        loc = make_array(256, /long)
        nloc = 0
;       flag for closed conductors
        lc_closed = 1

;     check first X value first
        while nok gt 0  do begin
          ok = where (xxx[nc-1] eq xf, nok)
          if nok gt 0 then $
            ok2 = where((yyy[nc-1] eq yf[ok]) and (ind[ok] eq 0), nok)
          if nok eq 0 then begin
;           no points found
            lc_found =0
            if lc_up gt 0 then begin
              xxx[loc[nloc]: nc-1] = rotate(xxx[loc[nloc]: nc-1], 5)
              yyy[loc[nloc]: nc-1] = rotate(yyy[loc[nloc]: nc-1], 5)
              lc_up = 0
              nok = 1
            endif else lc_up = -1
          endif else lc_found = 1
          if (lc_found gt 0) then begin
;           Take the first match
            ok = ok[ok2[0]]
            okval = ((ok mod 2) eq 0) ? ok+1 : ok-1
            if nc eq n_elements(xxx) then begin
              xxx= [xxx, make_array(1024)]
              yyy= [yyy, make_array(1024)]
            endif
            xxx[nc] = xf[okval]
            yyy[nc] = yf[okval]
            ind[*, okval/2] = 1
;           see if we are back where we started
            if xxx[nc] eq xxx[loc[nloc]] && yyy[nc] eq yyy[loc[nloc]] then lc_up= -2
            nc = nc+1
          endif
          if lc_up lt 0 then begin
;           one conductor complete
            nloc = nloc+1
            if nloc eq n_elements(loc) then loc = [loc, make_array(32, value = 0L)]
            loc[nloc] = nc
            if lc_up eq -1 then lc_closed = 0
            ok = where(ind eq 0, nok)
            if nok gt 0 then begin
              xf = temporary(xf[ok])
              yf = temporary(yf[ok])
              ind = reform(temporary(ind[ok]), 2, nok/2,/overwrite)
              xxx[nc: nc+1] = xf[0:1]
              yyy[nc: nc+1] = yf[0:1]
              ind[*, 0] = 1
              nc = nc+2
              nok = nok -2
              if nok eq 0 then begin
                nloc = nloc+1
                loc[nloc]= nc
                lc_closed = 0
              endif else lc_up = 1
            endif
          endif
        endwhile
        loc = loc[0:nloc]
        xxx = xxx[0:loc[nloc]-1]
        yyy = yyy[0:loc[nloc]-1]
        if n_elements(reset_cond) eq 0 then reset_cond=1
        tqcond, xxx, yyy, loc, reset = reset_cond
        if keyword_set(noplot) then return
        if keyword_set(closed_only) and lc_closed eq 0 then tqplotcon $
          else begin
          if nor gt 0 then tqplotcon, orient= orient, color= color,  $
             fill = fill, _extra = extrastuff else $
              tqplotcon, color = color,/fill,  _extra = extrastuff
        endelse
      endif else begin
        xr = !x.crange
        yr = !y.crange
        xer = (xr[1]-xr[0])/32768
        yer = (yr[1]-yr[0])/32768
        clip_lim= [xr[0]-xer, yr[0]-yer, xr[1]+xer, yr[1]+yer]
        for i = 0, np-1 do $
          oplot, xx(*,i),yy(*, i), thick = pthick, color = color, $
               clip = clip_lim,  _extra = extrastuff
        if nslant gt 0 then $
          for i = 0, nslant-1 do $
            oplot, xslant(*,i), yslant(*,i), thick = pthick, color = color, $
               clip = clip_lim, _extra = extrastuff
;
      endelse
      end
