; ------------------------------------------------------------------------
; $Id: plotfld.pro,v 1.43 2011/07/05 22:06:45 lpmix Exp $
; ------------------------------------------------------------------------
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
;+
; NAME:
;       PLOTFLD
;
; PURPOSE:
;       Plot field variable in a contour, surface, or shaded surface plot.
;       This routine will also integrate field values with respect to the
;         x and/or y values, and take lineouts of the values or integrated
;         field values.
;    Note: LOG PLOTS
;       For log contours the two techniques may be used:
;         1.  PLOTFLD, A, Multiply =0
;             (See the MULTIPLY keyword.)
;         2.  LOGSTR, A, B
;             PLOTFLD, B
;
;       WARNING:  PLOTSTR uses the _extra structure to pass variables. If
;                 data is to be returned, PLOTFLD must be used.
;       WARNING:  The maximum structure array will be used as a working
;                 structure array and will contain the two dimensional data
;                 used for the contour plot unless the keywork WORK provides
;                 a valid working structure number.
;                 The maximum structure array minus 1 is used by the POLARPLOT
;                 keyword to store the converted structure.
;
; CATEGORY:
;       Input/Output
; CALLING SEQUENCE:
;       PLOTFLD, A [, FLD] [, KVALUE1] [, VALUE1] [, KVALUE2] [, VALUE2]
; (general         [, BLOCK = block] [, ASPECT = aspect] [, MULTIPLY = multiply]
; plot parameters) [, ADD = add] [, KINTEGRATE = kintegrate] [HEADER = header]
;                  [, SETDEFAULT = setdefault] [, UNSET = unset]
;                  [, SHOWDEFAULT = showdefault] [, ALWAYS_PLOT = always_plot]
;                  [, INDEX = index] [, SMOOTH = smooth] [, POLARPLOT = polar]
;                  [, FARRAY = farray] [, NFARRAY = nfarray]
;                  [, XARRAY = xarray] [, YARRAY = yarray] [, ZTITLE = ztitle]
;                  [, TITLE = title] [, XTITLE = xtitle] [, YTITLE = ytitle]
;                  [, TRANSPLOT = transplot] [, /TRUNCATE] [, QUIET = quiet]
;                  [, LEFT = left] [, RIGHT = right] [, /RBTHETA]
;                  [, RESULT = result] [, WORK = work] [,NO_COPY = no_copy]
;                  [, XRANGE = xrange] [, YRANGE = yrange] [, ZRANGE = zrange]
;
; (plot types)     [, CNTOUR = contour] [, C_RANGE = crange]
;                  [, C_EXTRA = c_extra] [, IFLAG = iflag]
;                  [, XSURFACE = xsurface] [, PIXPLOT = pixplot]
;                  [, SHADESURF= shadesurf] [, SURFACE = surface]
;                  [, S_EXTRA = S_extra]
;                  [, CSHADESURF = cshadesurf] [, CS_EXTRA = cs_extra]
;                  [, OSHADESURF = oshadesurf] [, /NEGATIVE] [, /FASTPLOT]
;                  [, AXSURF= axsurf] [, AZSURF =azsurf]
; (contour param)  [, OVERPLOT = overplot] [IGNORE = ignore]
;                  [, NLEVEL = nlevel] [, LEVELS = levels]
;                  [, CLIP_VALUES = clip_values]
;                  [, COUTPUT = Contour_Output_WDF_Arrays] [, COLOR = color]
;                  [, XRANGE = xrange] [, YRANGE = yrange]
;                  [, CONDUCTOR = conductor] [, GRID = grid]
; (Color scaling)  [, SCALE_COLOR = scale_color] [, MEAN_COLOR = mean_color]
;                  [, GAMMA_COLOR = gamma_color] [, INVERTCT = invertctable]
; (Thermometer)    [, THERMOMETER = thermometer]
; (integrals)      [, XINTEG = xinteg] [, YINTEG = yinteg] [, START = start]
;                  [, REVERSE = reverse]
; (lineouts)       [, XLINE = xline] [, HLINE = hline]
;                  [, YLINE = yline] [, VLINE = vline] [, NPOINTS = npoints]
;                  [, WIDTH = width] [,PLINE = pline] [, OUT = out]
;                  [, CLINE = cline] [, CZERO = czero]
;                  [, CABSCISSA = cabscissa] [, CMAXIMUM = cmaximum]
; (any stuff)      (any contour keywords -- see IDL manual-- examples include:
;                  FILL, X(Y)TYPE, POSITION, X(Y)MARGIN, THICK, FOLLOW,
;                  DOWNHILL, X(Y)TICKLEN, X(Y)STYLE,  C_ANNOTATION,
;                  C_CHARSIZE, C_LINESTYLE, ....)
;
; INPUTS:
;       A     IDL field structure (a.type = 1) or QS array number
;       FLD   Field or attribute parameter to be plotted.
;       Note: FLD may be positive or negative; for GET_ARRAY: SPACE = -(abs(FLD))
;       Note: If FLD is an array, the square root of the sum of
;             the squares will be used.
;               If FLD =[1] the the absolute value is plotted.
;             If FLD is a single value array, ABS will be applied.
;       Note: If FLD is a single value larger than the number of vector
;             components, then FLD = 1+indgen(nvect)
;                where nvect is the number of vector components
;       Note: If A has only one attribute (such as an image or a charge density);
;             the this parameter can be omitted.
;       KVALUE1, 2
;             If FLD is a two dimensional field then these values are ignored.
;             If FLD is a three or four dimensional field then these values
;               indicate the coordinates which are fixed for the two
;               dimensional plots.
;               For 3D plots, x and y are chosen according to:
;                  x= (kvalue1 mod 3)+1 and y = (kvalue1+1) mod 3 + 1
;               For 4D plots, the lowest index is x than the other is y
;       VALUE1, 2
;             If FLD is a two dimensional field then these values are ignored
;             If FLD is a three dimensional field then these values
;               indicate the coordinate values for the fixed dimensions in
;               the two dimensional plots. Linear interpolation is used
;               according to the value of VALUE1. If VALUE1 has two elements
;               the average value over the interval is used.
;         Note: If INDEX keyword is set, the units are taken as array indices
;               which range from 1 to the maximum value.
;         Note: If the two elements of VALUE1 span a block boundary, then only
;               the values in the block containing the midpoint will be used.
;               If the midpoint falls on a block boundary, then the result is
;               probably not what you want!!
; KEYWORD PARAMETERS:
;
; (General plot keywords)
;
;       BLOCK   - If the field is multi-block, BLOCK can be used to specify
;                 one or more blocks to be plotted/integrated/...
;       ASPECT  - If set, will attempt to scale axis based on xrange and yrange.
;                 xstyle and ystyle are set
;                 If this is a type 1 field structure and:
;                   if ASPECT = "PIXELS" then:
;                     The plot will have X and Y axis lenghts that are
;                     proportional to the number of elements in the image
;                     in the first block (thus this option should only be
;                     used with single block data.)
;                 else if ASPECT = "SQUARE" then the X and Y axes will be the
;                     same length.
;                 Only the first character will be checked; ie "P" or "S"
;                 If ASPECT is set and the first character is not a "P" or "S",
;                 then aspect will be assumed to be 1.
;       POLARPLOT -  If set, a polar plot will be generated.
;                 The type-8 structure procuced by the transform will be stored
;                   in get_maxstr()-1.  Structure will have X and Y or X and Z in
;                   the X1 and X2 dimensions.
;                 The SPARE structure tag is used to determine the coordinate system
;                 For the SPARE[0] eq 1, no polar plot is generated
;                   (Already in Cartesian Coordinates)
;                 For SPARE[0] equal to 2 - CYLINDRICAL coordinates
;                 For SPARE[0] equal to 3 - SPHERICAL coordinates
;
;                 The coordinates for cylindrical and spherical data are as follows:
;                   R     - Radius vector
;                   PHI   - Angle between the radius vector projected on the X-Y
;                           plane and the X axis
;                   Z     - Axis normal to the X-Y plane
;                   THETA - Angle between the Z axis and the Radius vector.
;                 For the SPARE[0] eq 2, the cylindrical data will be converted
;                   to cartesian and plotted; Coordinates=(R, Phi, Z)
;                   X and Y must be 1 and 2 in any order, but they will be
;                     set to 1 and 2 respectively.
;                   The conversions are:
;                     X = R * Cos(Phi)
;                     Y = R * Sin(Phi)
;                 For the SPARE[0] eq 3, the spherical data will be converted
;                   to cartesian and plotted; Coordinates=(R, Theta, Phi)
;             NOTE:  Constant THETA will not result in a planar plot at a fixed
;                    value of Z but the projection of a conical plane at
;                    constant theta projected on the X-Y plane.
;                   The conversions to cartesian are:
;                     X = R * Cos(Phi) * Sin(Theta)
;                     Y = R * Sin(Phi) * Sin(Theta)
;                     Z = R * Cos(Theta)
;                   For (X, Y) equal to (1,3), X and Y will be calculated
;                     according to the equations above and will be plotted.
;                     X, Y equal to (1, 3) implies a slice in the theta dimension.
;                   For (X, Y) equal to (1,2) (Slice in the PHI dimension)
;                     X = R * SIN(X2)
;                     Z = R * COS(X2)
;                   TRANSPLOT will be set to zero.
;                   ASPECT will be set unless it is explicitly unset.
;            NOTE:
;                  In the case of mapped mesh grids:
;                    Assume the corners of the zone to be 1, 2, 3, 4 from lower
;                    left to upper left, counter-clockwise around the cell.
;                    The normal order for triangles is 1, 2, 3, and 1, 3, 4.
;                    The command: junk = make_triangles(/toggle) will change the
;                    triangles to 1, 2, 4 and 2, 3, 4
;
;                  Example:
;                      Replicate a field in theta and plot with conductors.
;                      Assume zval = zval, conductors in 2 and field in 3
;                        plotstr, /quiet, 3, 3, zval
;                        image, 82, 4, refl=2
;                        image, 2, 5,  refl=2
;                        plotstr, 4, /polar, con= 5
;       MULTIPLY- Multiply the field by a constant or a vector
;                 If MULTIPLY(0) eq 0 then MULTIPLY implies a log plot
;                       if MULTIPLY has 2 elements then
;                          field_value = field_value*multiply(1)
;                       FIELD_VALUE = ALOG10(FIELD_VALUE > MINVAL)
;                       LEVEL = ALOG10(LEVEL)
;                 Note: Enter MULTIPLY = [0, -1] to make a log plot of negative
;                       data such as electron charge density.
;                 Note: See also the routine: LOGSTR
;                       LOGSTR will take the LOG of a structure
;       ALWAYS_PLOT- If set, always plot the field.  If the range of the field
;                 is zero, plot with level = value +[-1,+1]
;       ADD     - Add a scalar to the field before plotting
;             Note: If F is the field then (multiply*F + add) will be plotted
;
;             NOTE: LOG PLOTS (MULTIPLY = 0)
;                 If MULTIPLY eq 0, then ADD is used to determine a lower bound.
;                    Default value: ADD = LOG_MIN ; (from LOGSTR)
;                         Initial value (see LOGSTR ) LOG_MIN = 1e-4
;                    The quantity plotted is ALOG10(FIELD_VALUE > MINVAL)
;                           MIN = ADD
;                           IF (MIN LT 0.0 or MIN GE 1) then
;                                MINVAL = ABS(MIN)
;                           IF (MIN GT 0 AND  MIN LT 1) then
;                                MINVAL = MIN*MAX(field_value)
;             NOTE: LOG_MIN is used by LOGSTR.  The default value is the same
;                   for both PLOTFLD and LOGSTR and may be set in LOGSTR
;             NOTE: LOG is always ALOG10 in PLOTFLD.
;
;       KINTEGRATE - If present and not zero, integrate the slice rather than
;                 average over the extent of value1(0) to value1(1).
;                 The integral will be performed between value1(0) and value1(1)
;                 If integrate = 1, a simple integral
;                 If integrate = 2, a x dx integral
;                 If integrate = 3, a x^2 dx integral
;                 Limitations:
;                     Data must have a single block
;                     Two values of value 1 must exist
;       HEADER    If specified a plot header, consisting of the current date
;                 and the current file name, is displayed in the lower left
;                 corner of the plot.  See the HEADER command to set default
;                 HEADER parameters (character size, spacing, alignment).
;                 DEFAULT is HEADER = 1
;                 NOTE: For X-windows screen dumps, many will want to increase
;                      the character size with the command:
;                      HEADER, /set, charsize = 0.8
;                      (Default is 0.6 of current size)
;     SETDEFAULT- If present and not zero, then the following parameters will be
;                 saved and used as the default values in all future calls to
;                 PLOTFLD
;                 If a keyword is specified, it will always be used rather than
;                 the saved values.
;     UNSET     - If present and not zero, will set all save values of keywords
;                 to their default values.
;                 Normal default values are listed below.
;                 Keyword     Default value
;                 HEADER       1
;                 COLOR        FLDCOLOR (-1 typically)
;                 XRANGE       0
;                 YRANGE       0
;                 NLEVEL      20
;                 LEVELS       0
;                 INDEX        0
;                 THERMOMETER  0
;                 ASPECT       0
;                 TRUNCATE     0
;                 CNTOUR       4
;                 ALWAYS_PLOT  0
;                 C_COLOR      [9, !d.table_size-2]
;           NOTE: The following is set in LOGSTR
;                 LOG_DEFAULT  1.0e-4
;           NOTE: CNTOUR = 4  implies CNTOUR = 1 if N_ELEMENTS(A.V1) < 100000
;                                     CNTOUR = 2 if N_ELEMENTS(A.V1) > 100000
;
;     SHOWDEFAULT-Show the present default plot values.
;       INDEX   - If present and not zero, then the units of VALUEi will be
;                 taken as indices of of the field array.  Valid values are
;                 1 to the maximum number of values in KVALUEi.
;                 Care should be exercised with multiple block data.
;       SMOOTH  - If present and not zero, the two dimensional field will be
;                 smoothed using a boxcar filter.  The number of cells will
;                 be the lowest odd number greater than or equal to the
;                 maximum of [3, SMOOTH]
;                 If SMOOTH is less than zero then the Absolute value of
;                 SMOOTH will be used as a median filter with a width equal to
;                 the maximum of [3, abs(SMOOTH)]
;       FARRAY, NFARRAY - For single block data FARRAY is the field quantity
;                 being plotted and NFARRAY is ignored.
;                 For multiple block data, a uniform grid is generated with
;                 NFARRAY by NFARRAY elements. The field quantity is
;                 interpolated to this grid.
;           Note: If nfarray is undefined, then farray will not be returned
;                 unless a plot is made.  If CN=2 then farray has the dimension
;                 of the plot area in pixels.  If CN = 1 then farray will be
;                 a scalar 0 for single block data.  The work array always
;                 contains the actual data plotted.
;           Note: All plots, except contour plots, plot the array, FARRAY.
;                 For multiple block data:
;                 CNTOUR = 1 (/CNTOUR)
;                 For standard contour plots, the default value of NFARRAY
;                 is 128 if NFARRAY is equal to 1, or abs(NFARRAY) > 16.
;                 (That is "/nf" would result in a value NFARRAY = 128)
;                 If NFARRAY is undefined, then NFARRAY = 0 unless SURFACE,
;                 SHADESURF, or XSURFACE is set.
;
;                 CNTOUR = 2 (or 3)
;                 For contour plots using a continuous color scale, FARRAY
;                 is the size of the plot area in PIXELS without scalable pixels.
;                 With scalable pixels the number of elements will be the actual
;                 array if it is larger than (750 x 600). If the array is smaller
;                 than this then the array will be resampled to a density of
;                 about 50 pixels /cm.
;
;       XARRAY, YARRAY - Abscissa and ordinate values for the array, FARRAY
;       TITLE, XTITLE and YTITLE - If these are character strings, they
;                 will be used as the plot title and the axis labels.
;                 If they are not character strings or are undefined (i.e.,
;                 delvar, title, xtitle, ytitle) then they will be returned
;                 with the values actually used for the plots.  The default
;                 TITLE is the dataset comment. The title will be truncated
;                 at a double simicolon (;;) or at the following string:
;                 ';DATETIME='. The default axis labels are the PFF axis
;                 labels read with the data.
;       ZTITLE    If set, will be used for cshade_surf.
;       TRUNCATE -  If set to zero, the full dataset comment will be used as
;                   a title. Default is to not limit the title to the characters
;                   before the first ' - ' in the comment. (TRUNCATE =0)
;                   If truncate is set only caracters before a ' - ' will
;                   be used and only the dataset comment will be used.
;       TRANSPLOT-If present and not zero, interchange ordinates and abscissa
;                 SETDEFAULT can be used to set a default value
;       QUIET   - If present and not zero,level information will not be printed.
;                 If quiet has a value of 1 (/QUIET or QUIET =1), NO plots
;                 will be make.  If QUIET is not zero and not 1, then plots
;                 will be made, but no output to the terminal.
;                 Default: QUIET = 0
;       LEFT     Indicates a new plot using the left axis.
;                A single axis will be drawn on the left side of the grid
;                For this plot XMARGIN will be set to
;                   XMARGIN = [!X.margin(0), !X.margin(0)]
;                and YSTYLE = 8.
;                NOTE: If LEFT is set, OVERPLOT is set to 0.
;                Note: Enter a value of ystyle of 8 plus the desired value
;                      if any other ystyle values are desired.
;                Example:
;                   Make a series of lineouts and overplot them on the contour
;                   plot. Average the second spatial component between [0,1 ]
;                   and take lineouts at .13, .14, .15.
;                 plotstr, 1, 1, 2, [0,1], xl = [0.13,.14,.15], /pl, /left
;                 plo, 1, 3, /over, /right, /ns, /color, thick = 3
;      RIGHT     Indicates a OVERPLOT making and using the right axis.
;                If RIGHT is present and not zero, then:
;                  A single axis will be drawn on the right side of the existing
;                    grid
;                  The field data will be overplotted on the existing grid.
;                Before using this command, a plot should be created using
;                  the keyword: /LEFT
;                NOTE: If RIGHT is set then OVERPLOT and IGNORE are set to 1.
;                NOTE: After this command all memory of the original y-axis
;                      scaling is lost
;       RBTHETA - If set and:
;                   1. structure has 3 field components
;                   2. Sturcture is type 1
;                   3. fld is 2
;                 then use chastr, B_structure, s2v = 1, /space
;                 to convert Btheta to R*Btheta.
;                 Ignored otherwise.
;       RESULT  - Structure array number to be used to store farray,
;                 xarray, yarray, title, xtitle, ytitle.
;                 This array contains a single block structure.
;                 For CN = 2, the array will have the screen dimensions.
;                 Note: If No_Copy is set then result will not be saved.
;                       If result is undefined, then farray will not be saved.
;       WORK    - Structure array number to be used as a working array.
;                 Default is maxstructure. This will have the same number of
;                 blocks as the input array.  If data is integrated, it will contain
;                 the integrated data. 
;                 Note: If NO_COPY is set then work will not be saved.
;       NO_COPY  - If this keyword is set then the plotted data will not
;                 be saved in WORK. Use for very large arrays.
;                 If the number of data elements is greater than 1,000,000
;                   and N0_COPY is not set then NO_COPY =1 for type 1 data
;
; (Plot types)
;
;       NOTE:   - Default plot type is contour plot
;                 If no other plot types are selected, a contour plot will
;                 be drawn by default.
;                 If fill is set, a pixmap will be generated.
;                 If the field contains more than 40000 elements, a pixmap will
;                 be generated.
;       CNTOUR  - Set the default contour type. Each bit indicates a different
;                 type of plot. Multiple plots can be specified.  If no other
;                 plots are specified, then CNTOUR will be set according to:
;                 If PIXPLOT is set, then CNTOUR = (2 or CNTOUR)
;                 If CNTOUR is undefined and N_elements(C_EXTRA) lt 3 then
;                    CNTOUR = CNTOUR_SAVE
;                 If CNTOUR is undefined and N_elements(C_EXTRA) ge 3 then
;                    CNTOUR = 2 or CNTOUR_SAVE
;                 Each bit of CNTOUR specifies a plot.
;                 CNTOUR = 1 or 3
;                 Standard contour plot.
;                (If value is 3, first a scaled color plot then the contours.)
;
;                 CNTOUR = 2 (or 3)
;                 Continuous scaled color plot (Image type plot).
;                 Note:  All colors will be used unless CRANGE is set.
;                 If CNTOUR < 0 CNTOUR = abs (CNTOUR) and default field is zero
;           Note: CNTOUR = 4  implies CNTOUR = 1 if N_ELEMENTS(A.V1) < 100000
;                                     CNTOUR = 2 if N_ELEMENTS(A.V1) > 100000
;           Note: CNTOUR = 2 will cause the entire plot area to be replaced.
;           Note: IF abs(CNTOUR) gt 3 then CNTOUR = CNTOUR_SAVE 
;                 Any data on the screen will be overwritten.
;                 For CNTOUR = 2 background color will be at 1% of the minimum.
;           Note: Do not use CNTOUR = 2 with RIGHT or OVERPLOT set.
;           Note: /NEGATIVE will make the tick marks in the background color
;       C_RANGE:  Two element vector specifying the range of the color table to
;                 use for CNTOUR = 1,2.
;                 First number is the lower value, and second is the upper
;                 value expressed as a fraction of full scale.
;                 If C_RANGE LE 1:
;                   NCOL = !d.table_size -11
;                   (For true color devices, !d.table_size = 256)
;                 If C_RANGE GT 1:
;                 Elements of the color table can be specified.
;                 The smallest number should be greater than 8 and the larger
;                 number should be less than (!d.table_size-1)
;                 If not they are not in this this range, they will be changed.
;                 DEFAULT: C_RANGE=[0,1] = [9, !d.table_size-2]
;       NEGATIVE: If set and either FILL is set or CNTOUR =2, 3 then the
;                 grid tick marks are in the background color.
;                 NOTE: See INVERTCT to generate a negative image.
;       FAST_PLOT: If set and single block data, then data reduced to 100 x 100
;       C_EXTRA - Specifies solid fill regions for CNTOUR = 2 plots.
;                 C_EXTRA must have a minimum of 3xn elements (n >= 1).
;                 If C_EXTRA is defined with 3 or more elements, then
;                 If CNTOUR is undefined then CNTOUR = (2 OR CNTOUR_SAVE).
;                 If CNTOUR is defined to be 1 then C_EXTRA is ignored
;                 C_EXTRA must have 3 or more elements. Format is of the form:
;                   C_extra =[color1, min1, max1, color2, min2, max2, ...]
;                 Where color1 exists between min1 and max1, etc.
;           Note: C_EXTRA is not implemented for other than linear scaling.
;
;           Note: Normal PLOTFLD commmand can be used to make filled regions
;                    without an image type plot.
;                 plotfld, str, color=[color1, -1, color2, -1], cn=1, $
;                      level =[min1, max1,  min2, max2], /fill, nlev = 4
;                 where the levels are in in assending order.  If max1 equals
;                 min2 then the first -1 color can be deleted.
;                 If normal contours are are desired, they must be plotted
;                   using the overplot keyword.
;                 for example: plotfld, str, /over, cn=1
;       IFLAG   - If set to 1, the interpolation for CNTOUR =2 will be done
;                 with rounding to the nearest X value.
;                 If set to 2, the interpolation for CNTOUR =2 will be done
;                 with rounding to the nearest Y value.
;                 If set to 3, the interpolation for CNTOUR =2 will be done
;                 with rounding to the nearest X value and the nearest Y value.
;                 IFLAG = 3 is similar to PIXPLOT
;                 If A surface plot only lines of constant X or Y (1, 2) will
;                    be drawn depending on the value of iflag.
;       SURFACE - If present and not zero, a surface plot of the field is
;                 generated using the interpolated array, FARRAY.
;         Note: The 3D transform is saved.
;                 If IFLAG is set, individual curves in x vs z or y vs z will be
;                   drawn.
;       S_EXTRA - Extra parameters for surface plots
;                 Note: CS_EXTRA can be used for adding keywords for the Surface
;                       plots.
;                 This may provide keywords for the surfact plot or add colored
;                   overlays to a surface plot.
;                 This is a structure.
;                 The following are valid tags:
;                 Spectrum: If set, colors are ignored. (see RAINBOW)
;                 Values: X or Y values for highlighting
;                         If IFLAG is not set, these are assumed to be Y values
;                            ie IFLAG = 2
;                 Colors: Colors for these curves.
;                         Valid values are [0, !d.table_size-1]
;          Note:  Only the first character is checked.
;
;       SHADESURF-If present and not zero, a shaded surface plot of the
;                 field is generated using the interpolated array, FARRAY.
;         Note: The 3D transform is saved.
;       OSHADESURF - If set, an object graphics shade surf is provided.
;       CSHADESURF-If present and not zero, a shaded surface plot of the
;                 field is generated using the interpolated array, FARRAY.
;                 The plot will use shades to color code the amplitude and
;                 overlay a surface plot.
;                 If CSHADESURF is less than 0, then surface plot is deleted.
;         Note: The 3D transform is saved.
;       CS_EXTRA- Structure with Keywords and values which can be used with
;                 SHADE_SURF and SURFACE.
;                 Example to set charsize to 3 use: CS_EXTRA={charsize:3}
;       XSURFACE- If present and not zero, an xsurface plot of the field is
;                 generated which allows rotation and resizing of the data
;                 using the interpolated array, FARRAY.
;       PIXPLOT - If present and not zero, a pixplot is generated.
;                 The coordinates are assumed to
;                   be on the boundary and boundary cells are only 1/2 (1/4 for
;                   the corners) of the size of interior cells.
;                 If dataset type is 8 (Finite element data) then negative pixplot
;                   is the symbol size
;                 Color is determined by the following:
;                    keyword color = -1 (DEFAULT)
;                    color = (Amplitude-FMIN)/(FMAX-FMIN)*(!d.table_size-11)+9
;                    keyword color = -2
;                    color = ((round(abs(amplitude))-1) mod 7)+1
;                 where FMIN and FMAX are the minimum and maximum values.
;                 If PIXPLOT is set then the minimum and maximum level values
;                   are set to fmin, fmax.
;                 Example:
;                   Make a pixplot for data in the range 422784 to 578304.
;                     and add a thermometer.
;                   plotfld, 4, /pix, ther= {ystyle:1, yrange: [4, 6]}, m = 1e-5
;       AXSURF  - X rotation for SURFACE and SHADESURF
;       AZSURF  - Z rtoation for SURFACE and SHADESURF
;
; (Contour plot keywords)
;
;       COUTPUT - If set, this specifies the initial WDF array for the
;                 x-y Pairs which specify the output contours.
;                 Successive wdf arrays are used.
;                 Output is from the CNTOUR=2 data.
;                 If COUTPUT is less than 0 then all data will be saved.
;                 If COUTPUT is greater than 0, then only lines with more than
;                   3 points are saved.
;                 If min(coutput) lt 1 then coutput = abs(coutput) > 1
;                 Output ceases when the maximum WDF array is saved.
;
;                 CNTOUR is set to 3.
;                 If quiet is set, no plot will be generated.
;                 EXAMPLE;
;                   Store the contour from structure 7 for lev = [10, 1000]
;                   in wdf array 40,41
;                      plotstr, 7, cout=40, lev = [10, 1000], nlev=2
;       OVERPLOT- If present and not zero, then the existing grid is to be used.
;                 IF CN=2, then:
;                  overplot > 0: Regions of the current plot with background
;                                values will use the previous values of the plot.
;                  overplot < 0: Regions of the previous plot with background
;                                values will use the new values of the plot.
;                 This option is ignored if any type of surface plots are made.
;                 NOTE:  IF X windows are used, then the data will be read
;                        from the screen for the above options.
;                        Otherwise only the last CN=2 bitmap will be used.
;                 With this option set, previous values of KVALUE1 and VALUE1
;                 are used by default as well as the choice of abscissa and
;                 ordinate coordinates. This option assumes that the previous
;                 plot was from a PLOTFLD, PLOTGRD, PLOTCON, or PLOTPAR
;                 command and attempts use the same parameters for the current
;                 plot.  To use OVERPLOT with an arbitrary grid, set the
;                 IGNORE keyword.  OVERPLOT is set to 1 if RIGHT is set.
;       IGNORE  - This keyword is ignored unless OVERPLOT is set.  If OVERPLOT
;                 is set and IGNORE is set, then all previous plot information
;                 is ignored.  IGNORE is set to 1 if RIGHT is set.
;       NLEVEL  - Number of contour plot levels.
;                 Default = 20.
;       LEVELS  - Array of values for the plot contours. If NLEVEL is specified,
;                 then the values in levels are ignored unless there are exactly
;                 NLEVEL values.
;                 If not then the values of levels are:
;                   LEVELS = minL + (findgen(NLEVEL)+0.5)*(maxL -minL)/(NLEVEL)
;                 where minL and maxL are the minimum and maximum values of
;                   LEVELS unless it has a single value then they are the maximum
;                   and minimum values of the data.
;                 The actual contour levels will be returned in LEVELS.
;                 For image (CNTOUR=2) plots, the levels go from from min to max
;                 of LEVELS.
;           NOTE: If PLOTSTR is used, the levels will NOT be returned because
;                 the EXTRA keyword is used to pass this value in a structure.
;      CLIP_VALUES If set, set any levels below the minimum to the background.
;                 If CLIP_VALUES = -1, then clip all values outside the
;                   range of LEVELS.
;                 Default = 1 ===>> clip only the values below the minimum.
;                 Note: Since CLINE and CLIP_VALUES are both keywords, 4 characters
;                       are required.
;      C_LABELS - Array of integers of value 0 or 1 to indicate if contour lines
;                 are to be labeled. Default is no labels on the contour lines
;                 If "/FOLLOW" is used as a keyword C_label is set to the array
;                 [1,0,1,0,1, ...] and every other  line is labeled.
;                 If C_LABEL= make_array(30, /fix, val= 1), then every contour
;                 is labeled.
;       COLOR   - Array of colors for the contour lines.
;                 If color is an array, then
;                   negative values will be set to !p.background
;                 If color is not an array, then
;                   If color = -1 then colors will go from c_range[0] to
;                     c_range[1] uniformly spaced using the existing color table
;                   If color = -2 then the 7 primary colors will be used to make
;                     a color map.  With 20 levels we will have 3 red, 3 green,
;                     3 blue, 3 purple, 3 orange, 3 light blue, and 2 yellow.
;                   If color = -3 the 7 standard colors will be cycled.
;                   If color is positive, that color will be used for all
;                     contours, [1,7] are the primary colors, 9 to !d.table_size
;                     depend on the color map.
;                   Color = 0 is the default plot color ie white or black
;                 Default is set by SETDEFAULT  or QSINIT
;                 EXCEPTION: If CNTOUR and 3 = 3 then the Default color is:
;                 c_color=[make_array(nlev/2, val = c_range[1]), $
;                          make_array(nlev - nlev/2, val = c_range[0])]
;                 The colors are: red(1), green(2), blue(3),
;                 purple(4), orange(5), light blue(6), yellow(7).
;       XRANGE, YRANGE - Two element arrays indicating the desired axis scaling.
;                 If they are scalar or undefined, the actual data range will
;                 be returned in these variables.
;       ZRANGE  - Range of the field
;       CONDUCTOR-Specifies a conductor structure or structure array number
;                 specifying a conductor structure.  If valid, the
;                 conductors will be overplotted.  Colors and thicknesses
;                 can be set with SETDEFAULT.
;       GRID    - Specifies a grid structure or a structure array number.
;                 If valid, the grid will be overplotted.  Color and
;                 thickness can be set with SETDEFAULT.
;
; (COLOR SCALING for CNTOUR equal to 2)
;    SCALE_COLOR- Type of color scaling used for the false color image
;                 PLOTFLD uses the 256 color index color map but the relation 
;                 between color and amplitude can be changed.
;                 MEAN_VALUE and SIGMA_COLOR (below) are also used with the
;                 LEVELS and C_RANGE keywords described above.
;                 In the discussion assume:
;                   DATA   = Actual data which is either a long or a float
;
;                   LEVELS = [ MIN(LEVELS, MAX = M), M] ; Range of data to scale
;                   DELTA_L = LEVELS[1]- LEVELS [0]
;
;                   C_RANGE = [ MIN(C_RANGE, MAX= MC, MC] ; Range of color table to use
;                   DELTA_C = C_RANGE[1] - C_RANGE[0]
;
;          The following values can be used:
;          Note: SCALE_color can be a number or a string.
;                Only the first 3 characters are checked if it is a string.
;                valid options:'LIN', 'GAM', 'LOG', 'HIS', GAU', 'ASI', 'SQR', 'HIS'
;                     < or >     0     1      2      3     4      5      6      3
;                 
;         LINEAR or 0
;                 DEFAULT - Linear scaling
;                 IMAGE = DELTA_C * (DATA - LEVELS[0]) / DELTA_L + C_RANGE[0])
;         GAMMA or 1
;               - Power law type function.  For GAMMA equal to 1, this is the same
;                 as linear scaling (Value = 0)  GAMMA less than 1 spreads the low values 
;                 and values greater than 1 spread the large values.
;                 IMAGE = (((DATA - LEVELS[0]) / DELTA_L) ^ GAMMA ) * DELTA_C + C_RANGE[0]
;                   (GAMMA_COLOR = GAMMA) (DEFAULT: GAMMA = 1)
;                 GAMMA = GAMMA_COLOR, DEFAULT: GAMMA = 1
;                 Values outside the range are set to the minimum/maximum levels.
;         LOG or 2
;               - A Sigmoid type function but the formula of D F Fanning is used.
;                 IMAGE_F = 1.0 / (1.0 + (MEAN / ((DATA - LEVELS[0]) / DELTA_L)) ^ GAMMA)
;                   (GAMMA = GAMMA_COLOR)  DEFAULT: GAMMA = 4
;                   (MEAN = MEAN_COLOR) DEFAULT = (FMIN+FMAX)/2
;         HISTOGRAM or EQUALIZATION or 3
;                 Histogram equalization to put equal number of points at each color
;                 IMAGE = HIST_EQUAL(DATA, MAXV = LEVELS[1], MINV = LEVELS[0], $
;                         TOP = DELTA_C) + C_RANGE[0]
;         GAUSSIAN or 4
;               - Folds the color table so maximum of color table is in the 
;                 middle of the data with fall to lower color table values at the maximum
;                 and minimum of the data.  
;                 IMAGE = DELTA_C*EXP( 0.5*((DATA - MEAN)/SIGMA)^2))+ C_RANGE[0]
;                   (SIGMA = GAMMA_COLOR) DEFAULT: (FMAX-FMIN) / (2*!PI)
;                   (MEAN = MEAN_COLOR) DEFAULT = (FMIN+FMAX)/2
;         ASINH or 5
;               - This performs an inverse hyperbolic sine function intensity 
;                 transformation on an image.  Based on the routine ASINHSCL
;                 Best results are on image type data which go from zero to some value
;                 with enhancements needed at low values.  The response is similar to a log
;                 response with a softening parameter given by 
;                 Soft = DELTA_L / ((GAMMA - LEVELS[0])  > (1.0e-12 * DELTA_L))
;                 IMI = ((IMAGE - LEVELS[0])/DELTA_L > 0 )  < 1.0
;                 EXT = ASINH ([0, 1] * Soft)
;                 IMAGE = (ASINH (IMI*SOFT) - EXT[0]) *(DELTA_C/(EXT[1]-EXT[0])) +C_RANGE[0]
;         SQRT or 6
;               - Linear scaling on the square root of the amplitude
;    GAMMA_COLOR- Gamma is a parameter used with nonlinear color table mapping
;                 The default value depends on the value of SCALE_COLOR 
;                 This keyword is not used by linear scaling or histogram equalization
;                 types of color scaling( SCALE_COLOR = 0 or 3), but is used by the GAMMA,
;                 LOG, GAUSSIAN, and ARCSINH scalings (SCALE_COLOR = 1, 2, 4, 5).
;    MEAN_COLOR - Mean value used for the nonlinear color table mapping.
;                 The default value depends of the value of SCALE_COLOR
;                 This keyword is used for the LOG(2) and GAUSSIAN(4) values of SCALE_COLOR
;    INVERTCT   _ Invert the color table.  If IMAGE is scaled from C_RANGE[0] to C_RANGE[1]
;                 then, if INVERTCT is set and positive
;                   IMAGE_NEW = 9 + (!D.TABLE_SIZE -2) - IMAGE
;                 or, if INVERTCT is negative then
;                   IMAGE_NEW = C_RANGE[0] + C_RANGE[1] - IMAGE
; (THERMOMETER keywords - used with contour plots)
;       THERMOMETER- If keyword is set a thermometer will be generated with
;                 the contour plot.  If thermometer eq -1, then the xmargin
;                 will be set to [10, 10] but no thermometer will be drawn.
;           Note: If THERMOMETER is a structure, then TPARAM is
;                 set equal to THERMOMETER.  TPARAM is passed to the
;                 THERMOMETER routine.
;                 Default IDL margins are [10, 3] for x and [4, 2] for y
;                 If TPARAM does not have a Tag Name beginning with 'CU'
;                 then:
;                 If no tag names begin with 'H' xmargin will be set to
;                   [!x.margin(0), 10]
;                   otherwise ymargin will be set to [8, !y.margin(1)]
;           Note: xmargin or ymargin passed to plotfld will override
;                       these settings.
;
; (INTEGRAL keywords)
;       XINTEG  - Integrate the field variable with respect to the abscissa
;                 value (x) according to the following values:
;                 If XINTEG = 0, do nothing.
;                 If XINTEG = 1, integrate f(x, y) dx
;                 If XINTEG = 2, integrate f(x, y) x dx
;                 If XINTEG = 3, integrate f(x, y) x^2 dx
;                   dx is the spacing between abscissa values.
;                 The initial value of the integral is set to 0.0 or
;                   to the value in the WDF array specified by START.
;           Note: If the resultant plot has contour lines passing through
;                 conductors, TQCLEAN can be used to improve the appearance
;                 of the plot.
;           Note: If two blocks terminate on a third, then the initial value of
;                 of the integral at the block boundary is linearly interpolated
;                 between the two incomming blocks.
;       YINTEG  - Integrate the field variable with respect to ordinate
;                 value (y) according to the following values:
;                 If YINTEG = 0 or if XINTEG = [1,2,3], do nothing.
;                 If YINTEG = 1, integrate f(x, y) dy
;                 If YINTEG = 2, integrate f(x, y) y dy
;                 If YINTEG = 3, integrate f(x, y) y^2 dy
;                   dy is the spacing between ordinate values.
;                 The initial value of the integral is set to 0.0 or
;                   to the value in the WDF array specified by START.
;           Note: If XINTEG ne 0 then YINTEG is ignored.
;           Note: If two blocks terminate on a third, then the initial value of
;                 of the integral at the block boundary is linearly interpolated
;                 between the two incomming blocks.
;       START     If present this is interpreted as initial values at the lower
;                 limit of integration. Start is an integer indicating a WDF
;                 array.  Values are between 1 and maxwdfarray.  Any blocks
;                 with coordinates in the domain of START will be initialized
;                 unless they are connected directly to another block.  Values
;                 at one point on a boundary are propagated along the entire
;                 boundary and linearly interpolated between adjacent values.
;       REVERSE   If present and not zero, the limits of integration are reversed
;
; (LINEOUT keywords - Values of the LINEOUT coordinates are determined
;       in the following sequence.) For lineouts parallel to an axis
;
;       1.  Two elements in width - These values taken as the max/min value
;       2.  NPOINTS >0, NPOINTS equally spaced lineouts in specified
;           direction(s), spacing is between the specified range --
;           use xrange and yrange to limit the values of the lineouts.
;           If NPOINTS eq N_ELEMENTS(XLINE) or NPOINTS eq N_ELEMENTS(YLINE)
;              those values are used for the X/Y Lineout.
;       3.  XLINE, YLINE is an array of values for the lineout
;      Note: To take a single lineout at one location width must be specified
;            with two values (which may be equal) and /xline or /yline
;            specified to indicate the direction of the lineout.
;         OR NPOINTS = N_ELEMENTS (X/YLINE) then NPOINTS lineout will be taken
;            at the appropriate y or x location specified by XLINE or YLINE with
;            a width specified by WIDTH
;      Note: If a single value is present, it will be adjusted to the closest
;            boundary if the point lies outside the domain of the data.
;            If multiple values are present, only valid points are used.
;       Note: If the width of a lineout spans a block boundary, only values
;             in the block containing the midpoint of the lineout will be used.
;
;       XLINE, HLINE - If present and not zero, a lineout is taken in the X or
;                 horizontal directon.  If HLINE is present then:
;                   XLINE = HLINE
;                 and any xline value is ignored.  A WDF array of field value
;                 vs the abscissa is generated at designated ordinate values.
;                 If np is omitted, lineouts are taken at valid values of XLINE.
;       YLINE, VLINE - If present and not zero, a lineout is taken in the Y or
;                 vertical directon.  If VLINE is present then:
;                   YLINE = VLINE
;                 and any yline value is ignored.  A WDF array of field value
;                 vs the ordinate is generated at designated abscissa values.
;                 If np is omitted, lineouts are taken at valid values of YLINE.
;       NPOINTS - Indicates number of lineouts in the X and/or Y direction.
;                 NOTE: If npoints = n_elements(XLINE) or N_elements(Yline),
;                       xline and/or yline are assumed to equal the desired value
;                       of the lineout.
;                       If x/yline is less than the minimum value then a lineout
;                         is taken at the minimum value.
;                       If x/yline is more than the maximum value then a lineout
;                         is taken at the maximum value.
;       WIDTH   - If a single value, then it is taken as the width of the
;                 lineout.  Default value is 0.
;                 If two values are present, they are taken as the mimimum and
;                 maximum values for obtaining the lineout.
;                 In all cases, the average value between the limits or in the
;                 Width about the designated point is used for the lineout.
;       PLINE   - If present and not zero, plot a dashed line at the position
;                 of the lineouts
;       OUT     - Value of the first WDF array for storing lineouts or an array
;                 of values for the lineouts.  Default is OUT=1
;                 (See W2I in the PFIDL help.)
;   Lineouts along conductors
;       CLINE   - If present and not zero, a lineout along a conductor will be
;                 taken.  The value of cline corresponds to the conductor
;                 number in QSEDIT.  If CLINE is negative lineouts will be taken
;                 of all conductors.  The WDF array generated will be stored in
;                 WDF array number, out, or the next WDF array following the
;                 X and Y lineouts. All waveforms are stored as XY data.
;                 Note: To obtain conductor geometries the TQ suite must be used.
;                       TQEDIT/QSEDIT can be used with the other routines to
;                       generate an arbitrary line.
;                       To use the actual geometry the following can be used.
;                     Read geometry - assume conductors in datasets 3, 4, 5, 6
;                        READSTR, 1, 3, 4
;                     Put the geometry into the TQ suite in Z-R, Z-X orientation
;                     and cut at y or theta = 0:
;                        TQREADCON, 1, 3, 1, 0
;                     Edit the geometry if necessary:
;                        QSEDIT, /cond
;                     Take the lineout along all conductors
;                        PLOTSTR, 2, 1, 2, 0, cline = -1, con = 2
;               Technique:  If dx and dy are less than 2^(-14) to a field point
;                           then that point is used. If not then the four closest
;                           field values are used to calculate a weighted
;                           average.  Values less than 2^(-14)*range are not
;                           used unless CZERO is set. Weight is (dx*dy)^(-1)
;       CABSCISSA - Specifies the independent variable for the lineout.
;                   If CABSCISSA is negative then CABSCISSA = abs(CABSCISSA)
;                 1 - X-value, 2 - Y-value, 3 - Length=sqrt(x^2+y^2)
;                 Default = 1
;       CMAXIMUM - Maximum point spacing. Generally only nodes in QSEDIT will be
;                  used but additional points will be inserted if the spacing
;                  exceeds cmaximum.
;       CZERO    - In general all values less than 6.10352e-5* (range of field)
;                  are ignored.  If CABSCISSA is negative then CZERO is set.
;                  IF CZERO is set, then all values will be used.
;            NOTE: CZERO is not a normal keyword but is set if CLINE eq -1
;
; OUTPUTS:
;       Designated datasets are plotted and WDF arrays generated for lineouts.
; Contour plots:
;  Maximum number of levels is 30.
;  CNTOUR will be used to determine plot type.
;
;   CNTOUR and 1 = 1 (Current Default)
;   Default is 20 levels, maximum is 30 levels.
;   LEVELS, if specified, will be used to set levels.
;   (If NLEVELS is specified and not equal to the number of values in LEVELS
;    then there will be NLEVELS levels between the max and min of LEVELS.)
;
;   CNTOUR and 2 = 2
;   Image will be a continuous gradient of color table from 9 to !d.table_size-2
;   with values corresponding to the 1% and 99% of the range of the data.
;   If LEVELS has two or more values then color 9 is the minimum value and
;   color !d.table_size-2 will be set to the maximum value.
;
;   For example:
; To put red hatching between .3 and .4 on a standard contour plot with levels
; at intervals of 0.1 for structure 3 use with a thermometer:
;    plotstr,3,level=[.3,.4],nl=2,c_orientation=45,color=[1, -1],ther=-1,$
;       /fill, c_thick = 2
;    plotstr,3,levels = findgen(20)/10 , /over, /th
;
; Similarly for solid red fill use:
;    plotstr,3, level=[.3, .4], /fill, color=[1, -1], nl = 2, TH=-1
;    plotstr,3,levels = findgen(20)/10 , /over, /th
;
; To obtain a continuous gradient plot with red between .3 and .4  and
; green between .7 and .8 use:
;
;   plotstr, 3,c_extra=[1, .3, .4, 2, .7, .8]
;
;
; EXAMPLE:
;       Contour plot of the charge density, in structure F, from a 2-D calc.
;         plotfld, f
;       (to add a thermometer with a plot with title Voltage, and tick values of
;            [0, 5, 10]
;         plotfld, f, /thermom, tparam={title:'Voltage', tickvalue:[0,5,10]}
;
;       Contour plot of the charge density, in structure F, from a 3-D
;       calculation taking a slice for the second dimension equal to 0.0
;         plotfld, f,1, 2, 0.0
;  (or)   plotfld, f,   2, 0.0 (since only one attribute, 1 can be omitted)
;
;       Contour plot of the CR-39 track distribution with the third dimension
;       the track size index in structure, C. Average tracks in bins 2 through
;       10.
;        plotfld, c,    3, [2, 10], /index
;  (or)  plotfld, c, 1, 3, [2, 10], /index
;  Note: Add ", m = 9" to plot the total number of tracks (since average is over
;        9 bins (2 to 10)
;
;       Contour plot of the magnitude of the vectors in structure START from
;       TWOQUICK simulation (ie. Only two spatial dimensions).
;         plotfld, start, [1,2,3]
;
;       Contour plot of two dimensional structure array 1 of charge density
;       (ie only 1 attribute)
;         plotfld, 1
;
;       Plot a slice of an image file, and take lineouts in the x and y
;       directions through a feature in the contour plot.  Assume the image
;       is in structure array 4 and an average over k components 3 to 10 is
;       desired. The lineout width is 0.01 and lines indicating lineouts are
;       to be plotted.
;       Note: Plotstr and plotfld may be used interchangably.
;         plotstr, 4, 3, [3, 10]
;         cur, x, y (place cursor over the center of the feature)
;         plotstr, 4, 3, [3, 10], np=1, xline = y, yline = x, wid = 0.01, /pl
;
;       Contour plot of vector 1 of field structure a. Use a value of 0.4
;         for spatial dimension 2 and take lineouts at 10 locations in the
;         first and third dimensions and store the lineouts in WDF arrays 11
;         to 30.  Sequence colors over the contour lines.
;         plotfld, a, 1, 2, 0.4, /xl, /yl, npoint = 10, out = 11, color=-1
;
;       Plot from structure a, vector 3 in a contour plot after smoothing.
;       Set the value of the first dimension to 0.05
;       Sequence linestyles over the contours.  Take a lineout between
;       values of the second dimension of 1 and 2 and store in WDF array 1.
;       (Note: The FOLLOW keyword should be used with multiple line types.)
;       plotfld, a, 3,1,0.05,/smo,c_line = findgen(6),/follow, /yl, wid=[1,2]
; or plotfld, a, 3,1,0.05,/smo,c_line = findgen(6),/follow, yl=1.5, wid=1, np =1
;
;       Make a shaded surface plot from structure END, vector 1, after
;       smoothing, and generate verticle lineouts at intervals of 0.1
;       between 1 and 2.5.  Assume structure END has two spatial components.
;       plotfld, end,1, yline= findgen(16)*0.1+1, /smo, /sha
;
; MODIFICATION HISTORY:
;      Original version, L. P. Mix, March 15, 1993
;      Added nonlinear color table mapping, April 27, 2010
;-
;
FUNCTION ASinh_wland, x
;
; NAME:
;     ASINH
; PURPOSE:
;     Return the inverse hyperbolic sine of the argument
; EXPLANATION:
;     The inverse hyperbolic sine is used for the calculation of asinh
;     magnitudes, see Lupton et al. (1999, AJ, 118, 1406)
;
; CALLING SEQUENCE
;     result = asinh( x)
; INPUTS:
;     X - hyperbolic sine, numeric scalar or vector or multidimensional array
;        (not complex)
; OUTPUT:
;     result - inverse hyperbolic sine, same number of elements as X
;              double precision if X is double, otherwise floating pt.
; METHOD:
;     Expression given in  Numerical Recipes, Press et al. (1992), eq. 5.6.7
;     Note that asinh(-x) = -asinh(x) and that asinh(0) = 0. and that
;     if y = asinh(x) then x = sinh(y).
; REVISION HISTORY:
;     Written W. Landsman                 February, 2001
;     Work for multi-dimensional arrays  W. Landsman    August 2002
;     Simplify coding, and work for scalars again  W. Landsman October 2003
;
COMPILE_OPT hidden
   on_Error, 1

    y = ALog( Abs(x) + SQRT( x^2 + 1.0) )

    index = Where(x LT 0 ,count)
    IF count GT 0 THEN y[index] = -y[index]

    RETURN, y

 END ;-------------------------------------------------------------------------------

  function get_byte_val,cont, fmin, fmax, c_extra, bytvalid, thamp, thcol, $
       modsw, c_range, clip_values, scale_col, gamma_col, mean_col, invertctable
; byte scale the data
  on_error, 1
  nc10 = c_range[1]-c_range[0]
  range = float(fmax-fmin)
;  boff = c_range[0]
if n_elements(scale_col) eq 0 then scale_col =0
if (scale_col le 0) || (scale_col gt 5) then begin
  if scale_col eq 6 then begin
    fmin = sqrt(fmin > 0)
    fmax = sqrt(fmax > 0)
    cont = sqrt(cont > 0)
    range = float(fmax-fmin)
    c_extra =0
  endif  
  if modsw eq 0 then begin
    bytval = round((((cont-fmin)> 0.0)*(nc10/range)) < 16384)+c_range[0] $
          < c_range[1]
    bytval = byte (bytval)
    thcol = findgen(nc10+1)
    thamp = fmin +thcol*(range/nc10)
    thcol = thcol+c_range[0]
    if keyword_set(clip_values)  then begin
       ok = (clip_values gt 0) ? where(cont lt fmin, nok) : $
             where(cont lt fmin or  cont gt fmax, nok)
       if nok gt 0 then bytval(ok) = !p.background
    endif
    nelc = n_elements(c_extra)
    if nelc gt 2 then begin
      nb = nelc/3
      for i = 0, nb-1 do begin
        ok = where(cont ge c_extra(3*i+1) and cont le c_extra(3*i+2), nok)
        col = round (c_extra(3*i))
        if nok gt 0 then begin
          bytval(ok)= byte(col)
          ok= where(thamp ge c_extra(3*i+1) and thamp le c_extra(3*i+2), nok)
          if nok gt 0 then thcol[ok]= col
        endif
        thamp = [thamp, c_extra(3*i+1),c_extra(3*i+2)]
        thcol = [thcol, col, col]
      endfor
      a = sort (thamp)
      thamp = thamp(a)
      thcol = thcol(a)
    endif
  endif else begin
    ss = size(cont)
    if ss(0) ne 2 then bytval = byte(cont*0) else begin
      bytval = make_array(ss(1), ss(2), /byt, value=0B)
      tmp = round(cont)
      ok = where (tmp gt 0, nok)
       if nok gt 0 then bytval(ok) = 1+ (tmp(ok)-1) mod 7
    endelse
    thcol = findgen(nc10)
    thamp = fmin +thcol*(range/(nc10-1))
    tmp = round (thamp)
    thcol = 0*thamp
    ok = where (tmp gt 0, nok)
    if nok gt 0 then thcol(ok) = 1+ (tmp(ok)-1) mod 7
  endelse
  ok = where (bytvalid eq 0, nok)
  if nok gt 0 then bytval(ok) = !p.background
  if scale_col eq 6 then begin
    thamp *= thamp
    fmin *= fmin
    fmax *= fmax
    cont *= cont
  endif
  if not keyword_set(invertctable) then return, bytval
  ok = where (bytval eq !p.background, nok)
  bytval = (invertctable lt 0) ? c_range[0] + c_range[1] - bytval : $
        9 + (!d.table_size -2) - bytval  
  if nok gt 0 then bytval[ok] = !p.background
  thcol =  (invertctable lt 0) ? c_range[0] + c_range[1] - thcol : $
        9 + (!d.table_size -2) - thcol
  return, bytval
endif 

; Gamma scaling
if scale_col eq 1 then begin
  if n_elements(gamma_col) eq 0 then gammacol = 1 else $
    gammacol = gamma_col > 1e-6
  bytval = (float(cont-fmin)/range > 0) < 1.0
  bytval = nc10 * (temporary(bytval)^ gammacol) + c_range[0]
  bytval = byte (round(temporary(bytval)))
  if keyword_set(clip_values)  then begin
     ok = (clip_values gt 0) ? where(cont lt fmin, nok) : $
           where(cont lt fmin or  cont gt fmax, nok)
     if nok gt 0 then bytval(ok) = !p.background
  endif
  
  thcol = findgen(nc10+1)
  thamp = fmin +thcol*(range/nc10)
  thcol = nc10 * ((thcol/nc10)^gammacol) +c_range[0]
  if not keyword_set(invertctable) then return, bytval
  ok = where (bytval eq !p.background, nok)
  bytval = (invertctable lt 0) ? c_range[0] + c_range[1] - bytval : $
        9 + (!d.table_size -2) - bytval  
  if nok gt 0 then bytval[ok] = !p.background
  thcol =  (invertctable lt 0) ? c_range[0] + c_range[1] - thcol : $
        9 + (!d.table_size -2) - thcol
  return, bytval
endif

; Log
if scale_col eq 2 then begin
  if n_elements(gamma_col) eq 0 then exponent = 4 else exponent = $
    (size (/type, gamma_col) lt 4) ? (gamma_col > 1) : (gamma_col > 1e-6)
  if n_elements(mean_col) eq 0 then meancol = 0.5 else $
    meancol = ((((mean_col - fmin)/range) > 0) < 1.0) 
  minval = 1e-3 
  bytval = ((float(cont-fmin)/range) > minval) < 1.0
  bytval = (1.0/ (1.0+ (meancol/temporary(bytval))^ exponent)) * nc10 + c_range[0]
  bytval = byte ((round(temporary(bytval)) > 0) < 255)
  if keyword_set(clip_values)  then begin
     ok = (clip_values gt 0) ? where(cont lt fmin, nok) : $
           where(cont lt fmin or  cont gt fmax, nok)
     if nok gt 0 then bytval[ok] = !p.background
  endif
  thcol = findgen(nc10+1)/nc10 > minval
  thamp = fmin + thcol*range
  thcol = c_range[0] + nc10 * (1.0/ (1.0+ ((meancol/thcol)^ exponent)))
  if not keyword_set(invertctable) then return, bytval
  ok = where (bytval eq !p.background, nok)
  bytval = (invertctable lt 0) ? c_range[0] + c_range[1] - bytval : $
        9 + (!d.table_size -2) - bytval  
  if nok gt 0 then bytval[ok] = !p.background
  thcol =  (invertctable lt 0) ? c_range[0] + c_range[1] - thcol : $
        9 + (!d.table_size -2) - thcol
  return, bytval
endif

; Histogram 
If scale_col eq 3 then begin
;  temp = (cont > fmin) < fmax
;  help, temp, fmin, fmax
  bytval = hist_equal(cont, maxv = fmax, minv = fmin, top = nc10) 
  h = histogram (bytval, reverse_indices = ri)
  bytval = bytval + c_range[0]
  if keyword_set(clip_values)  then begin
     ok = (clip_values gt 0) ? where(cont lt fmin, nok) : $
           where(cont lt fmin or  cont gt fmax, nok)
     if nok gt 0 then bytval(ok) = !p.background
  endif
  thcol = bindgen(nc10+1) + byte(c_range[0])
  thamp = make_array(nc10+1)
  thamp[0] = fmin & thamp[nc10] = fmax
  for i = 1, nc10-1 do thamp[i] = (h[i] eq 0) ? thamp[i-1] : $
    total(cont[ri[ri[i]: ri[i+1]-1]])/(ri[i+1]-ri[i])
  if not keyword_set(invertctable) then return, bytval
  ok = where (bytval eq !p.background, nok)
  bytval = (invertctable lt 0) ? c_range[0] + c_range[1] - bytval : $
        9 + (!d.table_size -2) - bytval  
  if nok gt 0 then bytval[ok] = !p.background
  thcol =  (invertctable lt 0) ? c_range[0] + c_range[1] - thcol : $
        9 + (!d.table_size -2) - thcol
  return, bytval
endif

; Gaussian
if scale_col eq 4 then begin
  if n_elements(gamma_col) eq 0 then sigma = range/(!pi+!pi) else  $
    sigma = (abs(gamma_col) > 1e-6*range) < 10* range
  if n_elements(mean_col) eq 0 then meancol = 0.5 *(fmin+fmax) else $
    meancol = (mean_col > fmin) < fmax
  bytval = (cont > fmin) < fmax
  bytval = nc10 * exp(-0.5 * ((temporary(bytval)-meancol)/sigma)^2) + c_range[0]
  bytval = byte (round(temporary(bytval)))
  if keyword_set(clip_values)  then begin
     ok = (clip_values gt 0) ? where(cont lt fmin, nok) : $
           where(cont lt fmin or  cont gt fmax, nok)
     if nok gt 0 then bytval(ok) = !p.background
  endif
  thamp = findgen(nc10+1) * (range/nc10) + fmin
  thcol =  nc10 * exp(-0.5 * ((thamp-meancol)/sigma)^2) + c_range[0]
  thcol = byte (round(temporary(thcol)))
  if not keyword_set(invertctable) then return, bytval
  ok = where (bytval eq !p.background, nok)
  bytval = (invertctable lt 0) ? c_range[0] + c_range[1] - bytval : $
        9 + (!d.table_size -2) - bytval  
  if nok gt 0 then bytval[ok] = !p.background
  thcol =  (invertctable lt 0) ? c_range[0] + c_range[1] - thcol : $
        9 + (!d.table_size -2) - thcol
  return, bytval
endif

; ArcSinh 
  if n_elements(gamma_col) eq 0 then betas = 3.0 else  $
    betas = gamma_col > 0.0
  nonlinearity = (betas - fmin) > (1e-12 * range)
  nonlinearity = range/nonlinearity
  bytval = ((cont - fmin) > 0  ) < range
  bytval = temporary (bytval)/range
  extrema = ASinh_wland ([0.0, 1.0]*nonlinearity)
  bytval = (ASinh_wland(nonlinearity * temporary(bytval)) -  $
    extrema[0]) *(nc10/(extrema[1]-extrema[0])) + c_range[0]
  bytval = byte (round (temporary(bytval)))
  if keyword_set(clip_values)  then begin
     ok = (clip_values gt 0) ? where(cont lt fmin, nok) : $
           where(cont lt fmin or  cont gt fmax, nok)
     if nok gt 0 then bytval(ok) = !p.background
  endif
  thcol = findgen(nc10+1) /nc10
  thamp = range*thcol+fmin
  thcol = byte(round((ASinh_wland(nonlinearity *temporary(thcol)) - $
    extrema[0]) *(nc10/(extrema[1]-extrema[0])) + c_range[0]))
  if not keyword_set(invertctable) then return, bytval
  ok = where (bytval eq !p.background, nok)
  bytval = (invertctable lt 0) ? c_range[0] + c_range[1] - bytval : $
        9 + (!d.table_size -2) - bytval  
  if nok gt 0 then bytval[ok] = !p.background
  thcol =  (invertctable lt 0) ? c_range[0] + c_range[1] - thcol : $
        9 + (!d.table_size -2) - thcol
  return, bytval
end

  pro make_conductor_ar, cline, cloc, cxarr, cyarr, cmaximum, ncloc
;
; set up arrays with the conductor values
  on_error, 1
        tqhelp, cond= cond
        if n_elements(cond) eq 0 then begin
          Print, 'Cannot take a conductor lineout with data in TQCOND'
          cline = 0
        endif else begin
          b = size(cond, /type)
          if b ne 8 then ncon = 0 else ncon = cond.ncon
          if ncon eq 0 then begin
            Print, 'Cannot take a conductor lineout with no conductors'
            cline = 0
          endif else begin
            if cline lt 0 then cline = 1+indgen(ncon)
            ok = where (cline le ncon+1, nok)
            if nok eq 0 then begin
              print, 'No valid conductors were specified - NO conductor lineouts'
              cline = 0
            endif else begin
              nelcx = max([2* n_elements(cond.x), 16384])
              cxarr = make_array(nelcx)
              cyarr = make_array(nelcx)
              cloc = make_array(nok+1, /long, value = 0L)
              for i = 0, nok-1 do begin
                cloc(i+1) = cloc(i)+ cond.locx(cline(ok(i))) - $
                    cond.locx(cline(ok(i))-1)
                cxarr(cloc(i):cloc(i+1)-1) = cond.x $
                  (cond.locx(cline(ok(i))-1):cond.locx(cline(ok(i)))-1)
                cyarr(cloc(i):cloc(i+1)-1) = cond.y $
                  (cond.locx(cline(ok(i))-1):cond.locx(cline(ok(i)))-1)
              endfor
              ncloc = nok
              if n_elements(cmaximum) gt 0 then begin
                if cmaximum le 0.0 then begin
                  print, 'Keyword CMAXIMUM lt 0 - IGNORED!'
                endif else begin
                  delx = cxarr(1:cloc(nok)-1)- cxarr(0:cloc(nok)-2)
                  dely = cyarr(1:cloc(nok)-1)- cyarr(0:cloc(nok)-2)
                  del = sqrt(delx*delx+dely*dely)
                  if nok gt 1 then for i = 1, nok-1 do del(cloc(i)-1)=0.0
                  ok = where (del gt cmaximum, nok)
                  ntadd = 0
                  if nok gt 0 then for i = 0, nok -1 do begin
                    nadd = long(del(ok(i)-ntadd)/cmaximum)
                    r = (1+findgen(nadd))/(nadd+1)
                    xadd = cxarr(ok(i))+r*(cxarr(ok(i)+1)-cxarr(ok(i)))
                    yadd = cyarr(ok(i))+r*(cyarr(ok(i)+1)-cyarr(ok(i)))
                    if nadd + cloc(ncloc) gt nelcx then begin
                      nelcx = nelcx+nelcx
                      cxarr = [cxarr, cxarr]
                      cyarr = [cyarr, cyarr]
                    endif
                    cxarr(ok(i)+1+nadd:cloc(ncloc)+nadd-1)= $
                       cxarr(ok(i)+1:cloc(ncloc)-1)
                    cyarr(ok(i)+1+nadd:cloc(ncloc)+nadd-1)= $
                       cyarr(ok(i)+1:cloc(ncloc)-1)
                    cxarr(ok(i)+1:ok(i)+nadd) = xadd
                    cyarr(ok(i)+1:ok(i)+nadd) = yadd
                    okcl = where (cloc gt ok(i))
                    cloc (okcl(0):*) = cloc(okcl(0):*)+nadd
                    ntadd = ntadd + nadd
                    if i lt nok-1 then ok(i+1:*) = ok(i+1:*)+nadd
                  endfor
                endelse
              endif
            endelse
          endelse
        endelse
        return
        end
;
  pro save_conductor_line,xlab,flab,a,cnval,cval, cloc,ncloc, nx, $
                       cxarr,cyarr, cline, cabscissa
 on_error, 1
        for i = 0, ncloc-1 do begin
          okx = cnval(cloc(i):cloc(i+1)-1)
          ok = where (okx gt 0, nok)
          if nok eq 0 then print, 'No valid data for conductor ', $
             strtrim(cline(i), 2) $
            else begin
            if cabscissa eq 1 then begin
              xx = (cxarr(cloc(i):cloc(i+1)-1))(ok)
            endif else if cabscissa eq 2 then begin
              xx = (cyarr(cloc(i):cloc(i+1)-1))(ok)
            endif else begin
              xx = (cxarr(cloc(i):cloc(i+1)-1))(ok)
              yy = (cyarr(cloc(i):cloc(i+1)-1))(ok)
              xx = xx(1:nok-1)-xx(0:nok-2)
              xx = xx*xx
              yy = yy(1:nok-1)-yy(0:nok-2)
              yy = yy*yy
              yy = sqrt (xx+temporary(yy))
              xx = make_array(nok)
              for ii = 1L, nok-1 do xx(ii) = xx(ii-1)+ yy(ii-1)
            endelse
            yy = (cval(cloc(i):cloc(i+1)-1))(ok)
            if i+nx le get_maxwdf() then $
              i2w, xx, yy, i+nx, file = a.file, /xydata, clab ='Cond '+ $
              strtrim(cline(i), 2)+ ' Lineout, -'+a.clab,xlab = xlab,ylab= flab $
            else print, 'Cannot save data for conductor ', strtrim(cline(i),2), $
              '- WDF maxarray exceeded'
          endelse
        endfor
        return
       end

 Pro help_plotfld
        print,'% Format: ', $
             'PLOTFLD, A [, FLD] [, KVALUE1] [, VALUE1]  [, KVALUE2] [, VALUE2]'
        print, $
' (general       [, BLOCK = block] [,ASPECT = aspect] [, MULTIPLY = multiply]'
print, $
'                [,ADD = add] [, KINTEGRATE = kintegrate]  [HEADER = header]'
print,'                [, SETDEFAULT = setdefault] [, UNSET = unset]'
print,'                [, SHOWDEFAULT = showdefault] [, ALWAYS_PLOT=always_plot]'
print,' plot parameters)[, INDEX = index] [, SMOOTH = smooth] [,POLARPLOT =polar]'
print,'                [, FARRAY = farray] [, NFARRAY = nfarray]'
print,'                [, XARRAY = xarray] [, YARRAY = yarray]'
print,'                [, TITLE = title] [, XTITLE = xtitle] [, YTITLE = ytitle]'
print,'                [, TRANSPLOT = transplot] [, QUIET = quiet] '
print,'                [, LEFT = left] [, RIGHT = right] [, /RBTHETA]'
print,'                [, RESULT = result] [, WORK = work] [,/NO_COPY]'
print,' (plot types)   [, CNTOUR = contour] [, C_EXTRA = c_extra ] [,IFLAG=iflag]'
print,'                [, /NEGATIVE ] [, /FASTPLOT] [, OSHADESURF = oshadesurf]'
print,'                [, SURFACE = surface]  [, PIXPLOT = pixplot]'
print,'                [, SHADESURF= shadesurf]  [, XSURFACE = xsurface]'
print,'                [, CSHADESURF= cshadesurf] [, CS_EXTRA = cs_extra]'
print,'                [, AXSURF = axsurf] [, AZSURF = azsurf]'
print,' (contour param) [, OVERPLOT = overplot] [IGNORE = ignore]'
print,'                [, NLEVEL = nlevel] [, LEVELS = levels] [, COLOR = color]'
print,'                [, XRANGE = xrange] [, YRANGE = yrange]'
print,'                [, ZRANGE = zrange] [, /COUTPUT]'
print,'                [, CONDUCTOR = conductor] [, GRID = grid]'
print,'                [, SCALE_COLOR = scale_color] [, MEAN_COLOR = mean_color]'
print,'                [, GAMMA_COLOR = gamma_color] [, INVERTCT = invertctable]'
print,' (thermomometer) [, THERMOMETER = thermometer] '
print,' (integrals)    [, XINTEG = xinteg] [, YINTEG = yinteg] [, START = start]'
print,'                [, REVERSE = reverse]'
print,' (lineouts)     [, XLINE = xline] [, HLINE = hline] '
print,'                [, YLINE = yline] [, VLINE = vline] [, NPOINTS= npoints]'
print,'                [, WIDTH = width] [,PLINE = pline] [, OUT = out]'
print,'                [, CLINE = cline] '
print,'                [, CABSCISSA = cabscissa] [, CMAXIMUM = cmaximum]'
print,' (any stuff)  (any contour keywords -- see IDL manual-- examples include:'
print,'                FILL, X(Y)TYPE, POSITION, X(Y)MARGIN, THICK, FOLLOW, '
print,'                DOWNHILL, X(Y)TICKLEN, X(Y)STYLE,  C_ANNOTATION, '
print,'                C_CHARSIZE, C_LINESTYLE, ....)'
   return
   end
;
;*deck
;
 PRO plotfld, ain, field, kvlue1, vlue1, kvlue2, vlue2, add = additin, $
   block = blockin, aspect = aspect, multiply = multin, smooth = smoo, $
   kintegrate = kinteg, index = index, no_copy= no_copy, fill = fill, $
   xarray= xarrayout, yarray= yarrayout, farray= farrayout, nfarray = nfarray, $
   xtitle = xtitle, ytitle= ytitle , title = title, polarplot = polar, $
   transplot = transpt, quiet = quiet, result = result, work = work,  $
   negative = negative, fastplot = fastplot,c_range = c_range_in, s_extra = sextra, $
   surface = surf, shadesurf= shadesurf, cntour= cntour, c_extra= c_extra, $
   xsurface= xsurface, pixplot = pixplot, azsurf=azsurf, axsurf=axsurf, $
   overplot = over, ignore = ignore, nlevel = nlev, showdefault=showdefault, $
   levels= levels, color = color, setdefault = setdefault, unset = unset,$
   thermometer = thermometer,  truncate = truncate, coutput = coutput, $
   xrange = xrange, yrange= yrange, zrange = zrange, conductor = conductor, $
   grid = grid, cshadesurf = cshadesurf, cs_extra = cs_extra, iflag= iflag, $
   xinteg= xinteg, yinteg = yinteg, start = start, reverse = reverse, $
   xline = inxline, yline = inyline, npoints=npline, width = width, $
   pline=pline, out = out, hline = hline, vline = vline, rbtheta = rbtheta, $
   cline = cline, cabscissa = cabscissa, cmaximum = cmaximum, always_plot = always_plot, $
   header= header, left = left, right = right, oshadesurf = oshadesurf, $
   clip_values = clip_values, position = position, scale_color = scale_color, $
   invertct = invertctable, mean_color = mean_col, gamma_color = gamma_col, $
   _extra = extrastuff
;
; alphabetical - keywords
; add, always_plot, aspect, axsurf, azsurf, block, bxsurface, cabscissa, cline,clip_values
; cmaximum, cntour, coutput, c_extra, c_range, color, cshadesurf,
; conductor, farray, grid, header, cs_extra, czero,
; hline, iflag, ignore, index, kintegrate, left, levels, multiply,
; negative,  nfarray, nlevel,no_copy, npoints, oshadesurf, out,
; overplot,pixplot, pline, polarplot,
; quiet, rbtheta, result, reverse, right,
; setdefault, shadesurf, showdefault, smooth, start, surface, thermometer,
; title, tparam, transplot,
; unset, vline, width, work, xarray, xinteg, xline,  xrange, xsurface, xtitle,
; yarray, yinteg, yline, yrange, ytitle
; insert a common block
;
;     return on an error
      on_error, 1
;
common LOG_STR_COMMON, LOG_MIN, LOG_TYPE, LOG_RELATIVE

@qscom.dat
;  Common block
@qsstrcom.dat
;
@plotfld_defaults.com

common left_right_save, old_extrastuff
common plot_grid_last, yeng_scale, xeng_scale
common atheta_plot_com, atmin, atmax, atlevels
common mkcircle, lcmkcircle, nel_symbols
common thermometer_common, therm_amp, therm_col

      if n_elements(lcmkcircle) eq 0 then lcmkcircle = 0
      if lcmkcircle eq 0 then mkcircle
;
      if keyword_set (setdefault) then begin
        if n_elements(color) gt 0 then color_save = color
        if n_elements(header) gt 0 then header_save = header
        if n_elements(xrange) gt 0 then xrange_save = float(xrange)
        if n_elements(yrange) gt 0 then yrange_save = float(yrange)
        if n_elements(nlev) gt 0 then nlevel_save = nlev
        if n_elements(levels) gt 0 then levels_save = float(levels)
        if n_elements(index) gt 0 then index_save = index
        if n_elements(thermometer) gt 0 then thermometer_save = thermometer
        if n_elements(aspect) gt 0 then aspect_save = aspect
        if (n_elements(cntour) gt 0) && (abs(cntour) le 4) then cntour_save = cntour
        if n_elements(truncate) gt 0 then truncate_save = truncate
        if n_elements(pixplot) gt 0 then pixplot_save = pixplot
        if n_elements(always_plot) gt 0 then always_plot_save = always_plot
        if n_elements(c_range_in) gt 1 then begin
          r_low  = min(c_range_in[0:1], max = mm) > 0
          r_high = mm
          nc10 =!d.table_size-11
          if nc10 lt 8 then begin
            print, '% PLOTFLD: less than 20 colors, color set to zero'
            c_range = [!p.color, !p.color]
          endif else begin
            c_range = lonarr(2)
            if  r_high le 1 then c_range = 9+round([r_low, r_high]*nc10) else $
              c_range = [(round(r_low) > 9), (round(r_high) < (!d.table_size -2))]
            if c_range[1]-c_range[0] lt 7 then begin
              Print,'% PLOTFLD: less than 8 colors.  C_range increased to 8'
              c_range[0] = (c_range[1]-7) > 9
              c_range[1] = (c_range[0]+7)
            endif
          endelse
          c_range_save = c_range
        endif
      endif
      if keyword_set(showdefault) then begin
         print, 'Default parameters for the PLOTFLD command are:'
         print, '    COLOR      = ' , strtrim (color_save, 2)
         print, '    C_RANGE    = [' , strtrim (c_range_save[0], 2), $
                ', ', strtrim (c_range_save[1], 2), ']'
         print, '    XRANGE     = ' , strtrim (xrange_save, 2)
         print, '    YRANGE     = ' , strtrim (yrange_save, 2)
         print, '    INDEX      = ' , strtrim (INDEX_SAVE, 2)
         print, '    NLEVEL     = ' , strtrim (nlevel_save, 2)
         print, '    LEVELS     = ' , strtrim (levels_save, 2)
         print, '    HEADER     = ' , strtrim (HEADER_SAVE, 2)
         print, '    ASPECT     = ' , strtrim (ASPECT_SAVE, 2)
         print, '    CNTOUR     = ' , strtrim (CNTOUR_SAVE, 2)
         print, '    TRUNCATE   = ' , strtrim (TRUNCATE_SAVE, 2)
         print, '    PIXPLOT    = ' , strtrim (PIXPLOT_SAVE, 2)
         print, '    ALWAYS_PLOT= ' , strtrim (ALWAYS_PLOT_SAVE, 2)
         ss = size(thermometer_save)
         if ss(ss(0)+1) eq 8 then begin
           print, '    THERMOMETER STRUCTURE:'
           help, /str, thermometer_save
           endif else $
         print, '    THERMOMETER= ' , strtrim (THERMOMETER_SAVE, 2)
      endif
      npar = n_params()
      if npar eq 0 and (keyword_set (setdefault) or keyword_set(unset) $
                  or keyword_set(showdefault) ) $
        then return
      if npar eq 0 then begin
        help_plotfld
        return
      endif
;
      if n_elements (structinit) eq 0 then qsstruct_init
      if n_elements (qsplotinit) eq 0 then qsinit
;
;     now get the data  -- check that we have a structure of the proper type
;

      if n_elements(ain) eq 0 then begin
        print, '% PLOTFLD: Structure must be defined'
        return
      endif
; change variables to insure we don't corrupt input data
      ier = 0
      if (structinit lt 0) then a = temporary(ain) else $
        a = get_valid_structure(ain,'PLOTFLD', lcstr=lcstr, ierror = ier)
      structinit = 1
      if ier ne 0 then return
;
      if n_elements(always_plot) eq 0 then always_plot=always_plot_save
;
;
; get a working array number
      if n_elements(work) eq 0 then work = maxstructure
      if work lt 1 or work gt maxstructure then work = maxstructure
      if n_elements(field) gt 0 then begin
        fld = field
        if max(abs(fld)) gt max([a.nvect,a.ndim])  then fld = 1+indgen(a.nvect)
      endif
      if n_elements(kvlue1) gt 0 then kvalue1 = kvlue1
      if n_elements(vlue1) gt 0 then value1 = vlue1
      if n_elements(kvlue2) gt 0 then kvalue2 = kvlue2
      if n_elements(vlue2) gt 0 then value2 = vlue2
;
      if n_elements (blockin) eq 0 then block = 0 else block = blockin
      if n_elements (ignore) eq 0 then ignore = 0
      if keyword_set (right) then begin
        over = 1
        oldycrange= !y.crange
        ignore = 1
      endif else right =0
      if keyword_set (left) then begin
        over = 0
        if (n_elements(old_extrastuff) gt 0) then old_extrastuff = extrastuff
      endif else begin
        if (n_elements(old_extrastuff) gt 0) then left = temporary(old_extrastuff)
        left = 0
      endelse
      if n_elements (over) eq 0 then over  = 0
      if n_elements (polar) eq 0 then begin
        if over ne 0 then polar = polarplot else polar = 0
      endif
      polarplot = polar
      if n_elements (index) eq 0 then index = index_save
      if n_elements (aspect) eq 0 then begin
        if keyword_set(polar) then aspect = 1 else aspect = aspect_save
      endif
      if n_elements (pixplot) eq 0 then pixplot = pixplot_save
      if n_elements (truncate) eq 0 then truncate = truncate_save
      if ignore ne 0 then overparam = 0 else overparam = over
      if n_elements (kinteg) eq 0 then kinteg = 0
;
;     check for rbtheta scaling
;
      if keyword_set (rbtheta) and a.type eq 1 then begin
        if a.nvect eq 3 then begin
          if fld[0] eq 2 then chastr, a, s2v =1, /space
        endif
      endif
;
;     reduce the structure to two dimensions with a single attribute
;

;zzdebug
;zzt(1) = systime(1)
;zztc(1) = 'Before slice'
      ss = size(fld)
      nfld = ss(ss(0)+2)
      if nfld gt 1 then fld_sav= fld
      structinit = -1
      a = slice (a, fld, kvalue1, value1, kvalue2, value2, $
          block= block, overplot=overparam, index=index, integ = kinteg )
      structinit = 1
      if size(a, /type) ne 8 then return
      if ss[0] gt 0 then a.v1 = abs(a.v1)
;zzdebug
;zzt(2) = systime(1)
;zztc(2) = 'After slice'
;
      if keyword_set(cline) and n_elements(cabscissa) gt 0 then begin
        if cabscissa lt 0 then begin
          czero = 0
          cabscissa = abs(cabscissa)
        endif
      endif
      if n_elements(smoo) eq 0 then smo = 0 else smo = abs(smoo) > 3
      if n_elements(clip_values) eq 0 then clip_values = 1
      if keyword_set (polar) then begin
        polarplot = 1
        cyl2cartesian, a, kvalue1, smooth= smo
        if (size(a))(0) eq 0 then begin
          print, '% PLOTFLD: Error in converting to polar coordinates.'
          return
        endif
        transpt = 0
        if nfld gt 1 then begin
            fld_sav = fld_sav(sort (fld_sav))
            tmp = 'Magnitude of components: '
            for i = 0 , nfld-1 do tmp = tmp+' ' + strtrim(fld_sav(i), 2)
            a.clab = tmp+', '+a.clab
        endif
        if n_elements(xtitle) gt 0 then a.x1lab = xtitle(0)
        if n_elements(ytitle) gt 0 then a.x2lab = ytitle(0)
      endif
      if keyword_set (coutput) then cntour=3

      scale_col = (n_elements(scale_color) eq 0) ? 0 : scale_color[0]
;     options 'LIN', 'GAM', 'LOG', 'HIS', GAU', 'ASI', 'SQR', 'HIS'
      if size(scale_col, /type) eq 7 then begin
        scale_col = strtrim (scale_col, 2)
        if strlen (scale_col) lt 3 then begin
          print, '% PLOTFLD: SCALE_COLOR must have 3 characters or be an integer'
          return
        endif
        types = [ 'LIN', 'GAM', 'LOG', 'EQU', 'GAU', 'ASI', 'SQR', 'HIS']
        scale_col = where (strupcase(strmid(scale_col, 0, 3)) eq types, nok)
        if nok eq 0 then begin
           print, '% PLOTFLD: SCALE_COLOR has an invalid string: ', scale_col
           print, '          Valid strings are:'
           print, '         LINEAR, GAMMA, LOG, HISTOGRAM or EQUALIZATION,'
           print, '         GAUSSIAN, ASINH, SQRT'
           return
        endif
        scale_col = scale_col[0]
        if (scale_col gt 6) then scale_col = 3
      endif
        
;     nc10 is the total number of colors -1
      nc10 =!d.table_size-11
      if nc10 lt 8 then begin
        print, '% PLOTFLD: less than 20 colors, color set to zero'
        c_range = [!p.color, !p.color]
      endif else if n_elements(c_range_in) gt 1 then begin
        c_range = lonarr(2)
        r_low = min(c_range_in[0:1], max = mm) > 0
        r_high = mm
        if  r_high le 1 then c_range = 9+round([r_low, r_high]*nc10) else $
          c_range = [(round(r_low) > 9), (round(r_high) < (!d.table_size -2))]
        if c_range[1]-c_range[0] lt 7 then begin
          Print,'% PLOTFLD: less than 8 colors.  C_range increased to 8'
          c_range[0] = (c_range[1]-7) > 9
          c_range[1] = (c_range[0]+7)
        endif
      endif else c_range = c_range_save
      nc_range = c_range[1] - c_range[0]
      if a.type ne 1 then begin
        if (n_elements (cntour) gt 0) && (abs(cntour) gt 3) then cntour = 2
        if a.type eq 8 then begin
          lcfmin = 1
          if (n_elements(cntour) gt 0) && (cntour lt 0) then begin
            cntour = -cntour
            lcfmin = 0
          endif
          nfld = n_elements(fld)
          if nfld eq 0  then begin
            if a.nvect gt 1 then begin
 print,'% PLOTFLD: Structure contains ', strtrim(a.nvect, 2), ' vector components'
 print,'           Field component must be specified'
              return
            endif
            nfld = 1
            fld = 1
          endif
          if a.nvect eq 1 then  begin
            fld = 1
            nfld = 1
          endif
          if min(fld,max = mm) lt 1 or mm gt a.nvect then begin
            print, '% PLOTFLD: Vector quantities must be in the range, (1, ', $
              strtrim(a.nvect, 2), ')'
            return
          endif
          if nfld gt 1 then begin
            fld = fld(sort (fld))
            tmp = 'Magnitude of components: '
            for i = 0 , nfld-1 do tmp = tmp+' ' + strtrim(fld(i), 2)
            a.clab = tmp+', '+a.clab
            tmp  =   make_array(n_elements(a.v1))
          endif
          for i = 0, nfld-1 do begin
            if fld(i) eq 1 then begin
              if nfld gt 1 then tmp = a.v1*a.v1
            endif else if fld(i) eq 2 then begin
              if nfld gt 1 then tmp = tmp + a.v2*a.v2 else a.v1= a.v2
            endif else if fld(i) eq 3 then begin
              if nfld gt 1 then tmp = tmp + a.v3*a.v3 else a.v1= a.v3
            endif else if fld(i) eq 4 then begin
              if nfld gt 1 then tmp = tmp + a.v4*a.v4 else a.v1= a.v4
            endif else if fld(i) eq 5 then begin
              if nfld gt 1 then tmp = tmp + a.v5*a.v5 else a.v1= a.v5
            endif else if fld(i) eq 6 then begin
              if nfld gt 1 then tmp = tmp + a.v6*a.v6 else a.v1= a.v6
            endif
          endfor
          if nfld gt 1 then a.v1 = sqrt(tmp)
          nfld = 1
          fld = -1
        endif else begin
          print, ' Only Field type structures may be plotted with PLOTFLD'
          a = 0
          return
        endelse
        a.nvect = 1
      endif else begin
;     slice makes a one attribute structure
;       avoid saving multiple copies of large rectilinear data
;
        if ((n_elements(cntour) gt 0) && (abs(cntour) gt 3)) or $
          (n_elements(cntour) eq 0) then begin
          cntour = cntour_save
          if cntour eq 4 then cntour = (n_elements(a.v1) gt 100000) ? 2 : 1
        endif
        if n_elements(no_copy) eq 0 then $
          if n_elements(a.v1) gt 1000000 then no_copy = 1
        fld = -1
        nfld = 1
      endelse
;
      if n_elements (additin) eq 0 then addit = 0.0 else addit = additin(0)
      if n_elements (multin) eq 0 then mult = 1.0 else mult = multin(0)
      if mult eq 0 then begin
        mult = 1.0
        if n_elements (multin) eq 2 then begin
          a.v1 = multin(1)*a.v1
          if n_elements(levels) gt 0 then levels =  multin(1)*levels
        endif
        if n_elements(log_min) eq 0 then log_min = 1.0e-4
        if n_elements(additin) eq 0 then addit = log_min else addit = additin
        if addit eq 0 then addit = 1.0e-4
        if addit gt 0 and addit lt 1.000 then addit = addit *max(a.v1) $
          else addit = abs(addit)
        a.v1 =  alog10(a.v1 > addit)
        if n_elements(levels) gt 0 then levels = alog10(levels > addit)
        addit = 0.0
      endif
      if n_elements (block) eq 1 then begin
        if block lt 0 or block gt a.block then block = 0
;
;       determine the range of the data if we must determine levels
;       and draw a grid
;
        if block gt 0 then ii = block else ii = 1
        if block gt 0 then imx = block else imx = a.block
        nblock = imx-ii+1
        blockarray = indgen(nblock)+ii
      endif else begin
        blockarray = block
        ok = where (blockarray gt 0 and blockarray le a.block, nblock)
        if nblock gt 0 then blockarray= blockarray(ok) else begin
          print, block, ' is not a valid block specification'
          print, ' Please try again.'
          a = 0
          return
        endelse
      endelse
      kkk = 0
      for ib = 0, nblock-1 do begin
        ok = where(a.size[*, blockarray[ib]-1] gt 1, nok)
        if nok gt 1 then begin
          blockarray[kkk] = blockarray[ib]
          kkk += 1
        endif
      endfor
      if kkk eq 0 then begin
        print, '% PLOTFLD: No valid blocks have been specified'
        a=0
        return
      endif
      if kkk lt nblock then begin
        help, kkk, nblock
        nblock = kkk
        blockarray = blockarray[0:nblock-1]
      endif
;
;   Check the plot types and determine if lineouts or integrals are req'd
;
      if n_elements (lcfmin) eq 0 then begin
        lcfmin = 1
        if n_elements (cntour) gt 0 then if cntour lt 0 then begin
          cntour = -cntour
          lcfmin = 0
        endif
      endif
      if keyword_set (pixplot) then begin
        if (a.type eq 1) then begin
          if n_elements(cntour) eq 0 then cntour = 2 else cntour = (2 or cntour)
        endif else if (a.type eq 8) then begin
          if n_elements(cntour) eq 0 then cntour = 4 else cntour = (4 or cntour)
        endif
      endif
      if n_elements(quiet) eq 0 then quiet = 0
      if ( quiet eq 1) then begin
        surf = 0
        shadesurf = 0
        oshadesurf =0
        cshadesurf = 0
        if not keyword_set(coutput) then cntour = 0
        xsurface = 0
        pixplot = 0
        lcbig = 0
      endif else if (not keyword_set(surf) and not keyword_set (shadesurf) $
        and not keyword_set(xsurface ) and not keyword_set (cshadesurf)) $
        and not keyword_set (oshadesurf) then begin

;       with no other plots we will do a contour plot
;       quiet = 0
        lcbig = 0
        if n_elements(cntour) eq 0 then begin
          cntour = cntour_save
          if cntour eq 4 then cntour = (n_elements(a.v1) lt 100000) ? 1 : 2
        endif
        if n_elements(c_extra) ge 3 then cntour = (2 or cntour)
      endif else begin
;       we must have one array to plot for (surface, xsurface or shade_surf)
;       quiet = 0
        lcbig = 1
        if n_elements(nfarray)  eq 0  then nfarray = 128
        if n_elements(cntour) eq 0 then cntour = 0
;       cannot do an overlay plot if we do any type of a surface plot
        if over ne 0 then print, 'OVERPLOT set to zero'
        if right ne 0 then print, 'RIGHT set to zero'
        over  = 0
        right = 0
      endelse
;
      if keyword_set (over) then aspect = 0

;     determine coordinates axes
;     first sort the kvalues
;
      if n_elements (transdefault) eq 0 then transdefault = 0
      if n_elements (transpt) eq 0 then transplot = transdefault else $
        transplot = transpt
      if a.ndim eq 2 then begin
        xval = 1 & yval = 2 & value1 = 0
      endif else if a.ndim eq 3 then begin
        if kvalue1 eq 2 then begin
          if transplot eq 0 then transplot = 1 else transplot = 0
          xval = 1 & yval = 3
        endif else begin
          xval = (kvalue1 mod 3) + 1
          yval = ((kvalue1 + 1) mod 3) + 1
        endelse
      endif else if a.ndim eq 4 then begin
        ok = where (a.size(*, 0) gt 1, nok)
        if nok ne 2 then begin
          print, '% PLOTFLD: 4D data must have two degenerate coordinates.'
          return
        endif
        xval = 1+ok[0]
        yval = 1+ok[1]
      endif else message, 'Only 2, 3 and 4 dimensional data is allowed'
;
      ispare = a.spare
        xnew = xval
        ynew = yval
;     check on the values for the third dimension
;     reset the xold and yold for a possible overlay plot
      if over eq 0  or (over ne 0 and right ne 0)then begin
        if transplot eq 0 then begin
          xoldvalue = xnew
          yoldvalue = ynew
        endif else begin
          xoldvalue = ynew
          yoldvalue = xnew
          temp = xval
          xval= yval & yval = temp
        endelse
        zplaneold = a.two
        zvalueold = value1
      endif else begin
        if ignore eq 0 then begin
; we have an overplot and ignore eq 0
          if xnew eq xoldvalue and ynew eq yoldvalue then begin
            if transplot ne 0 then begin
              print, ' The contours will not be transposed for this plot.'
               print, ' Hit "q" to quit; hit any other key to continue'
               key =  (byte (get_kbrd(1)))(0)
               if ((key eq 81) or (key eq 113)) then return
              transplot = 0
            endif
          endif else if xnew eq yoldvalue and ynew eq xoldvalue then begin
            if transplot eq 0 then begin
              print, ' The contours will be transposed for this plot.'
               print, ' Hit "q" to quit; hit any other key to continue'
               key =  (byte (get_kbrd(1)))(0)
               if ((key eq 81) or (key eq 113)) then return
              transplot = 1
            endif
          endif
        endif
        if transplot ne 0 then begin
          temp = xnew & xnew = ynew & ynew = temp
          temp  = xval
          xval= yval & yval = temp
        endif
        if ignore eq 0 then begin
          if xnew ne xoldvalue then begin
            Print, ' WARNING: This overplot requested with X-axis of ', xnew, $
                ', present value is', xoldvalue
            print, ' Hit "q" to quit; hit any other key to continue'
            key =  (byte (get_kbrd(1)))(0)
            if ((key eq 81) or (key eq 113)) then return
          endif
          if ynew ne yoldvalue then begin
            Print, ' WARNING: This overplot requested with Y-axis of ', ynew, $
                ', present value is', yoldvalue
            print, ' Hit "q" to quit; hit any other key to continue'
            key =  (byte (get_kbrd(1)))(0)
            if ((key eq 81) or (key eq 113)) then return
          endif
        endif
      endelse
;

      if over eq 0 and a.ndim gt 2 then  zvalueold = value1 else $
        if right ne 0 and a.ndim gt 2 then  zvalueold = value1
      if a.type eq 8 then zvalueold = a.slice
;
      if n_elements (start) eq 0 then start = 0
      if keyword_set(xinteg) or keyword_set(yinteg)  then begin
        if transplot eq 0 then $
          a = intstr(a, smoo=smo, xinteg=xinteg, yinteg=yinteg, start=start, $
              reverse = reverse) else $
          a = intstr(a, smoo=smo, xinteg=yinteg, yinteg=xinteg, start=start, $
              reverse = reverse)
;       see if we integrated without error
        b = size(a)
        if b(b(0)+1) ne 8 then begin
          print,' Error in the integrate routine.'
          a = 0
          return
        endif
;       we did the smoothing in the integration so turn it off now.
        smo = 0
      endif
;
;     get the max and min and labels
;
      xmin = min(a.range(0,xval-1, blockarray))
      xmax = max(a.range(1,xval-1, blockarray))
      ymin = min(a.range(0,yval-1, blockarray))
      ymax = max(a.range(1,yval-1, blockarray))
;
; set the error to the delta length in the problem divided by 2^15
; or the machine precision of the absolute length (2^23).
;
      xerr = 3.0517578e-05*max(abs([(xmax+xmin)/512., xmax-xmin]))
      yerr = 3.0517578e-05*max(abs([(ymax+ymin)/512., ymax-ymin]))
;     if the levels are not specified then we need to calculate them
;
      if n_elements(header) eq 0 then header = header_save
      if over ne 0 then header = 0
      if !p.multi(0) ne 0 then header = 0
      if header ne 0 then begin
        filename = strtrim(a.file, 2)
        iii = strpos(a.clab, 'DATETIME=', 0)
        if iii gt 0 then begin
          ii = strpos(a.clab, ';', iii+2)-iii-9
          if ii gt 0 then cdate = strmid(a.clab,iii+9,ii) $
               else cdate = strmid(a.clab,iii+9,20)
          cdate = strtrim(cdate, 2)
          header_text='Plot: ' + thin_date()+'; Data: ' + cdate
          if strlen(filename) gt 0 then $
               header_text = [header_text, 'File: ' +filename]
        endif else begin
          osver = strupcase(strmid(!version.os, 0, 3))
          if (strlen(filename) eq 0) or (osver eq 'MAC') then $
            header_text='Plot: ' + thin_date() $
          else  begin
            ss = file_info(filename)
            cdate=(ss.exists eq 1) ? thin_date(ss.ctime) : thin_date()
            header_text= ['Plot: '+thin_date() +'; Data: ' + cdate,  $
                     'File: ' +filename]
;           header_text = ['Plot: ' + thin_date(), 'File: ' +filename]
          endelse
        endelse
      endif
      if n_elements(xrange) eq 0 then xrange = xrange_save else $
          xrange=float(xrange)
      if n_elements(yrange) eq 0 then yrange = yrange_save else $
          yrange=float(yrange)
      if (n_elements(xrange) lt 2) then begin
         epsilx = (keyword_set(aspect)) ? 0.0 : (xmax-xmin)*3.0517578e-05
         xrange = [xmin+epsilx, xmax-epsilx]
         lcxrange = 0
         cxrange = 0
      endif else begin
         xrange = xrange[0:1]
         epsilx = (xrange(1)-xrange(0))*3.0517578e-05
         lcxrange = 1
      endelse
      if (n_elements(yrange) lt 2) then begin
         epsily = (keyword_set(aspect)) ? 0.0 : (ymax-ymin)*3.0517578e-05
         yrange = [ymin+epsily, ymax-epsily]
         lcyrange = 0
         cyrange = 0
      endif else begin
         yrange = yrange[0:1]
         epsily = (yrange(1)-yrange(0))*3.0517578e-05
         lcyrange = 1
      endelse


      if lcyrange eq 0 xor lcxrange eq 0 then begin
;     if only x or only y have been set then maybe the remaining range
;       has been reduced.  Check to see the extent of the blocks.
        nb= n_elements(blockarray)
        if lcyrange eq 0 then begin
          ok = where (xrange(0) le a.range(1, xval-1, blockarray) and $
              xrange(1) ge a.range (0, xval-1, blockarray), nok)
          if nok eq 0 then begin
            Print, ' No y extent for the data - RETURNING'
            return
          endif
          blockarray = blockarray(ok)
          yrange= [min (a.range(*, yval-1, blockarray), max= m1) , m1]
          cyrange = yrange
        endif else begin
          ok = where (yrange(0) le a.range(1, yval-1, blockarray) and $
              yrange(1) ge a.range (0, yval-1, 1:*), nok)
          if nok eq 0 then begin
            Print, ' No x extent for the data - RETURNING'
            a = 0
            return
          endif
          blockarray = blockarray(ok)
          xrange= [min (a.range(*, xval-1, blockarray), max= m1) , m1]
          cxrange = xrange
        endelse
        nblock = n_elements(blockarray)
        if (nb gt nblock) and (quiet ne -1) then print, $
         '% PLOTFLD: Data available only for the following blocks:', blockarray
      endif
;
;     round the values to avoid ugly axis on plots if one of the ranges
;     has been corrected due to the limitations in the other dimension.
;
      if lcyrange eq 0 and n_elements(cyrange) eq 2 then begin
        epsily = (cyrange(1)-cyrange(0))/32768.
        yrange=[cyrange(0)+epsily , cyrange(1)-epsily]
      endif else epsily = 0.0
      if lcxrange eq 0 and n_elements(cxrange) eq 2 then begin
          epsilx = (cxrange(1)-cxrange(0))/32768.
          xrange=[cxrange(0)+epsilx , cxrange(1)-epsilx]
      endif else epsilx = 0.0
;
;    LINEOUTS
;
      if n_elements (hline) gt 0 then inxline = hline
      n_xline = n_elements (inxline)
      if n_xline eq 0 then xline = 0 else xline = inxline
      if n_elements (vline) gt 0 then inyline = vline
      n_yline = n_elements (inyline)
      if n_yline eq 0 then yline = 0 else yline = inyline

      if n_elements (cline) eq 0 then cline = 0
      if n_elements (cabscissa) eq 0 then cabscissa = 1
      if cabscissa lt 1 or cabscissa gt 3 then cabscissa = 1
      if n_elements(out) eq 0 then out = 1
      if n_elements(width) eq 0 then width = 0 else width = float(width)
      if n_elements (npline) eq 0 then npline =0
      nxline = 0
      nyline = 0
;
      if n_elements(width) ge 2 then begin
        if keyword_set(inxline) then begin
          xline = 1
          yline = 0
          ookk = 1
          minv = a.range(0, yval-1, 0)
          maxv = a.range(1, yval-1, 0)
        endif else if  keyword_set(inyline) then begin
          yline = 1
          xline = 0
          ookk = 1
          minv = a.range(0, xval-1, 0)
          maxv = a.range(1, xval-1, 0)
        endif else ookk = 0
        if ookk ne 0 then begin
          width = width(0:1)
          width = [min(width, max = mxv), mxv]
          if width(0) lt minv then width(0) = minv
          if width(0) gt maxv then width(0) = maxv
          if width(1) lt minv then width(1) = minv
          if width(1) gt maxv then width(1) = maxv
        endif else begin
          print, $
        'WARNING: To obtain a lineout using width directions must be specified'
          print, $
        ' Use /xline parallel to the x axis and /yline for a verticle lineout'
        endelse
        linmid = (width(0)+width(1))*0.5
        widsav = abs (width(0)-width(1))
        if n_xline ne 0 then begin
          xline = linmid
          nxline = 1
        endif
        if n_yline ne 0 then begin
          yline = linmid
          nyline = 1
        endif
      endif else if (npline gt 0) && ((npline eq n_xline) || (npline eq n_yline)) then begin
        if n_xline eq npline  then begin
          nxline = npline
          xline = float(inxline)
          ok = where (xline lt a.range[0, yval-1, 0], nok)
          if nok gt 0 then xline[ok] =  a.range[0, yval-1, 0]
          ok = where (xline gt a.range[1, yval-1, 0], nok)
          if nok gt 0 then xline[ok] =  a.range[1, yval-1, 0]
        endif
        if n_yline eq npline then begin
          nyline = npline
          yline = float(inyline)
          ok = where (yline lt a.range[0, xval-1, 0], nok)
          if nok gt 0 then yline[ok] =  a.range[0, xval-1, 0]
          ok = where (yline gt a.range[1, xval-1, 0], nok)
          if nok gt 0 then yline[ok] =  a.range[1, xval-1, 0]
        endif
        widsav = width
        if nxline eq 0 and nyline eq 0 then begin
          print, $
        'WARNING: To obtain a lineout using NPOINTS=1 XLINE or YLINE must '
          print, $
         '        specify a ordinate or abscissa value for the lineout.'
        endif
      endif else begin
        widsav = width
        npline = abs(npline)
        if npline gt 1 then begin
          if n_xline gt 0 then begin
            nxline = npline
            xline=(findgen(npline)+1.)*(yrange(1)-yrange(0))/(npline+1)+yrange(0)
          endif
          if n_yline gt 0 then begin
            nyline = npline
            yline=(findgen(npline)+1.)*(xrange(1)-xrange(0))/(npline+1)+xrange(0)
          endif
        endif else begin
          if n_xline gt 1 then begin
            nxline = n_xline
            xline = float(inxline)
            ok = where (xline ge a.range[0, yval-1, 0]-yerr and $
               xline le a.range[1, yval-1, 0]+yerr, nxline)
            if nxline gt 0 then $
              xline = (xline[ok]>a.range[0, yval-1, 0]) < a.range[1, yval-1, 0]
          endif
          if n_yline gt 1 then begin
            nyline = n_elements (yline)
            yline = float(inyline)
            ok = where (yline ge a.range[0, xval-1, 0]-xerr and $
               yline le a.range[1, xval-1, 0]+xerr, nyline)
            if nyline gt 0 then $
              yline = (yline[ok]>a.range[0, xval-1, 0]) < a.range[1, xval-1, 0]
          endif
        endelse
      endelse
;
;     see if we can quit
;
      lclineout = cline +nxline+nyline
      if (quiet eq 1) and (lclineout eq 0)  and (a.type eq 1) and $
        (n_elements(nfarray) eq 0) then begin
        if not keyword_set(no_copy) then i2s, a, work, /no_copy
        return
      endif
      if lcbig eq 0 and a.type ne 1 then lcbig =1

;
;     optimize the block ordering for the lineouts
;     this is not absolutely necessary
      if nblock gt 1 and (nxline gt 0) then begin
        ii = sort(a.range(0, xval-1, blockarray))
        blockarray= blockarray(ii)
      endif else if nblock gt 1 and yline(0) gt 0 then begin
        ii = sort(a.range(0, yval-1, blockarray))
        blockarray= blockarray(ii)
      endif
;
;     get the first block elements
;
      if a.type eq 1 then begin
        xx = get_array (a, xval, blockarray(0), lab = xlab)
        nx = n_elements(xx)
        yy = get_array (a, yval, blockarray(0), lab = ylab)
        ny = n_elements(yy)
        cont = reform(get_array(a, fld, blockarray(0), lab = flab), /overwrite)
        if transplot ne 0 then cont = transpose(temporary(cont))
        if mult ne 1.0 then cont = mult *temporary(cont)
        if addit ne 0.0 then cont = addit +temporary(cont)
        if smo gt 0 then begin
          if smo le nx and smo le ny then $
              cont = smooth(temporary(cont), smo)
        endif
      endif
;
;     set some values to zero just in case
      farray = 0
      xarray = 0
      yarray = 0
;zzdebug
;zzt(3) = systime(1)
;zztc(3) = 'Before Tags '

      ok = where(strpos(tag_names(a), 'DX') ge 0 , nok)
      if nok eq 0 then lcunif = 0 else begin
        tot = xval+yval
        if tot eq 3 then begin
;         x and y
          if ((a.dx1(0) gt 0.0) and (a.dx2(0) gt 0.0)) then begin
            lcunif = 1
            if xval eq 1 then begin
              dx = a.dx1
              dy = a.dx2
            endif else begin
              dx = a.dx2
              dy = a.dx1
            endelse
          endif else lcunif = 0
        endif else if tot eq 4 then begin
;         x and z
          if ((a.dx1(0) gt 0.0) and (a.dx3(0) gt 0.0)) then begin
            lcunif = 1
            if xval eq 1 then begin
              dx = a.dx1
              dy = a.dx3
            endif else begin
              dx = a.dx3
              dy = a.dx1
            endelse
          endif else lcunif = 0
        endif else begin
;         y and z
          if ((a.dx2(0) gt 0.0) and (a.dx3(0) gt 0.0)) then begin
            lcunif = 1
            if xval eq 2 then begin
              dx = a.dx2
              dy = a.dx3
            endif else begin
              dx = a.dx3
              dy = a.dx2
            endelse
          endif else lcunif = 0
        endelse
      endelse

      if n_elements(nfarray) eq 0 then nfarray = 0
      if n_elements (thermometer) eq 0 then thermometer = thermometer_save
      wnew =-1

; changed for steve to be sure to get the correct stuff
;      if cntour ne 0 or (lclineout gt 0 and a.type eq 8) then begin
      if (cntour ne 0) or (a.type eq 8) then begin
        xstyle = 0
        ystyle = 0
        xmargin = !x.margin
        ymargin = !y.margin
        if left ne 0 then begin
          xmargin = [!x.margin(0), !x.margin(0)]
          ystyle = 8 or ystyle
        endif else if keyword_set(thermometer) then begin
          ss = size (thermometer)
          if ss(ss(0)+1) eq 8 then begin
            tparam = thermometer
            thermometer = 1
          endif
          xmargin = [!x.margin(0), !x.margin(0)]
          if n_elements (tparam) gt 0 then begin
            b = tag_names(tparam)
            ok = where (strmid(b, 0, 2) eq 'CU', nok)
            if nok eq 0 then begin
              ok = where (strmid(b, 0, 1) eq 'H', nok)
              if nok gt 0 then begin
                ymargin = [8, !y.margin(1)]
                xmargin = !x.margin
              endif
            endif else xmargin = !x.margin
          endif
          if thermometer gt 0 then begin
            therm_amp = 0
            therm_col = 0
            ovamp = 0
            ovcol = 0
          endif
        endif
        ss = size(extrastuff)
;       save extrastuff for overplotting the axis.
        if n_elements (extrastuff) gt 0 then extrastuff_a = extrastuff
        if ss(ss(0)+1) eq 8 then begin
          myextra = extrastuff
          tags = tag_names(myextra)
          ok = where(strpos(tags, 'XS', 0) eq 0, nok)
          if nok gt 0 then i = execute('myextra.'+tags(ok(0))+ $
            '= myextra.'+tags(ok(0))+' or 4') else $
            myextra = create_struct(myextra, 'xstyle', (!x.style or 4))
          ok = where(strpos(tags, 'YS', 0) eq 0, nok)
          if nok gt 0 then  begin
            i = execute('myextra.'+tags(ok(0))+ $
            '= myextra.'+tags(ok(0))+' or 4')
            if ystyle gt 0 then $
              i = execute('extrastuff.'+tags(ok(0))+ $
              '= extrastuff.'+tags(ok(0))+' or ystyle')
          endif else begin
            extrastuff = create_struct(extrastuff, 'ystyle', ystyle or !y.style)
            myextra = create_struct(myextra, 'ystyle', ystyle or !y.style or 4)
          endelse
        endif else begin
          if ystyle gt 0 or !y.style gt 0 then begin
            extrastuff ={ystyle: (ystyle or !y.style)}
            myextra ={ystyle: (ystyle or !y.style or 4)}
          endif
          if !x.style gt 0 then begin
            if size(extrastuff, /type) eq 8 then $
               extrastuff = create_struct(extrastuff, 'xstyle', !x.style) $
            else extrastuff ={xstyle: !x.style}
            if size(myextra, /type) eq 8 then $
               myextra = create_struct(myextra, 'xstyle', !x.style or 4) $
            else myextra ={xstyle: !x.style}
          endif
        endelse
;
        if not keyword_set (over) then begin
          multcnt = !p.multi(0)
          wsave = !d.window
          if quiet eq 1 then begin
            if wsave ge 0 then begin
              xsave = !x
              ysave = !y
              zsave = !z
              psave = !p
            endif
            window, /pixmap, /free
            wnew = !d.window
          endif else wsave = -1
          contour, make_array(2,2), xrange, yrange, /nodata, position = position, $
            xrange= xrange, yrange= yrange, xstyle =4, ystyle = 4, $
            xmargin = xmargin, ymargin = ymargin, _extra = myextra
          xrange_first = !x.crange
          yrange_first = !y.crange
        endif
        xpos = round(!x.window*!d.x_size)
        ypos = round(!y.window*!d.y_size)
        xr = !x.crange(1)-!x.crange(0)
        yr = !y.crange(1)-!y.crange(0)
        xsiz = xpos(1)-xpos(0)
        ysiz = ypos(1)-ypos(0)
        if !d.X_PX_CM eq  !d.Y_PX_CM then PRAT = 1.0 else $
          prat = !d.X_PX_CM /  !d.Y_PX_CM
        prat = 1.0
        if keyword_set(aspect) then begin
          position = 0
          if a.type eq 1 and size(aspect, /type) eq 7 then begin
            if strupcase(strmid(strtrim(aspect,1), 0, 1))  eq 'P' then begin
              if float(nx)/xsiz gt float(ny)/ysiz then begin
                ysiz = round(float(ny)*xsiz/(nx))
                position = [xpos(0), ypos(0), xpos(1), ypos(0)+ysiz]
              endif else begin
                xsiz = round(float(nx)*ysiz/(ny))
                position = [xpos(0), ypos(0), xpos(0)+xsiz, ypos(1)]
              endelse
            endif else if $
              strupcase(strmid(strtrim(aspect,1), 0, 1))  eq 'S' then begin
              xsiz = min([xsiz, ysiz*prat])
              ysiz = xsiz/prat
              position = [xpos(0), ypos(0), xpos(0)+xsiz, ypos(0)+ysiz]
            endif
            if n_elements(position) lt 4 then $
               print, '% PLOTFLD: ASPECT set to 1'
          endif
          if n_elements(position) lt 4 then begin
            if xr/xsiz ge yr/ysiz/prat then begin
              ysiz = round(yr/xr*xsiz/prat)
              position = [xpos(0), ypos(0), xpos(1), ypos(0)+ysiz]
            endif else begin
              xsiz = round(xr/yr*ysiz*prat)
              position = [xpos(0), ypos(0), xpos(0)+xsiz, ypos(1)]
            endelse
          endif
        endif else position = [xpos(0), ypos(0), xpos(1), ypos(1)]
;
        if (cntour and 2) ne 0  or a.type eq 8  then begin
;         put data on a uniform grid with a resolution of 1 pixel
;         Put type 8 data on a uniform grid to save it as type 1
;
          xrbig = float(!x.crange[0:1])
          xpxbig = round((xrbig -!x.crange(0))*(xsiz/xr))
          yrbig = float(!y.crange[0:1])
          ypxbig = round((yrbig -!y.crange(0))*(ysiz/yr))
          if (!d.flags and 1) ne 0 then begin
;           scalable pixels
;           Find the number of pixels in the plot window.
            if a.type eq 1 then begin
              okx = where (xx ge xrbig(0) and xx le xrbig(1), nokx)
              oky = where (yy ge yrbig(0) and yy le yrbig(1), noky)
            endif else begin
              nokx = 0
              noky =0
            endelse
            if nblock eq 1 and nokx ge 750 and noky ge 600 and lcunif eq 1 $
            then begin
              nxbig = nokx
              nybig = noky
              farray = cont
              dxbig = (xrbig(1)-xrbig(0))/(nxbig-1)
              xarray = xrbig(0) + findgen(nx)*dxbig
              dybig = (yrbig(1)-yrbig(0))/(nybig-1)
              yarray = yrbig(0) + findgen(ny)*dybig
            endif else begin
              pden = 50
              if !d.x_px_cm gt pden then nxbig = round(xsiz*pden/!d.x_px_cm) $
                else nxbig = xsiz
              if !d.y_px_cm gt pden then nybig = round(ysiz*pden/!d.y_px_cm) $
                else nybig = ysiz
              dxbig = (xrbig(1)-xrbig(0))/(nxbig-1)
              xarray = xrbig(0) + findgen(nxbig)*dxbig
              dybig = (yrbig(1)-yrbig(0))/(nybig-1)
              yarray = yrbig(0) + findgen(nybig)*dybig
            endelse
          endif else begin
            nxbig = long(xpxbig(1)-xpxbig(0))
            nybig = long(ypxbig(1)-ypxbig(0))
            dxbig = (xrbig(1)-xrbig(0))/(nxbig-1)
            xarray = xrbig(0) + findgen(nxbig)*dxbig
            dybig = (yrbig(1)-yrbig(0))/(nybig-1)
            yarray = yrbig(0) + findgen(nybig)*dybig
          endelse
        endif else nxbig = 0
      endif else nxbig =0
;
;       if no big array for contour then see if we need one for farray
;
      if nxbig eq 0 and ((nfarray gt 0) or  (lcbig gt 0)) then begin
        if nblock eq 1 and a.type eq 1 then begin
          xarray = xx
          yarray = yy
          farray = cont
          nxbig = 0
        endif else begin
          if nfarray eq 0 then nxbig=128 else $
            if nfarray eq 1 then nxbig = 128 else nxbig =abs(nfarray)>16
          nybig = nxbig
          xrbig = fltarr(2)
; Changes below for steve rosenthal x and y ranges are likely not valid
;          xrbig(0) = max([xmin, !x.crange(0)])
;          xrbig(1) = min([xmax, !x.crange(1)])
          xrbig(0) = xmin
          xrbig(1) = xmax
          yrbig = fltarr(2)
;          yrbig(0) = max([ymin, !y.crange(0)])
;          yrbig(1) = min([ymax, !y.crange(1)])
          yrbig(0) = ymin
          yrbig(1) = ymax
          dxbig = (xrbig(1)-xrbig(0))/(nxbig-1)
          xarray = xrbig(0) + findgen(nxbig)*dxbig
          dybig = (yrbig(1)-yrbig(0))/(nybig-1)
          yarray = yrbig(0) + findgen(nybig)*dybig
        endelse
      endif

;     interpolate onto the big array and calculate the range of the data
;zzdebug
;zzt(4) = systime(1)
;zztc(4) = 'Before interpolate '
;
;   a.type =8 is finite element type data
      if n_elements(color) eq 0 then  begin
        if (cntour and 3) eq 3 then begin
          if n_elements (nlev) eq 0 then nlev = nlevel_save
          color = [make_array((nlev/2)>1,val=c_range[1]),  $
                  make_array(nlev - nlev/2, value = c_range[0])]
        endif else color= color_save
      endif
    if a.type eq 8 then begin
      cont = a.v1
      flab = a.clab
      if mult ne 1.0 then cont = mult *temporary(cont)
      if addit ne 0.0 then cont = addit +temporary(cont)
      nblock =1
      if keyword_set(transplot) then begin
        yy = a.x1
        xx = a.x2
        xlab = a.x2lab
        ylab = a.x1lab
      endif else begin
        xx = a.x1
        yy = a.x2
        xlab = a.x1lab
        ylab = a.x2lab
      endelse
      if lcxrange eq 0 and lcyrange eq 0 then fmin=min(cont,max=fmax) else begin
        okx = where (xx ge (xrange(0)-2*epsilx) and xx le (xrange(1)+2*epsilx) $
        and  yy ge (yrange(0)-2*epsily) and yy le (yrange(1)+2*epsily),   nokx)
        if nokx eq 0 then begin
          print, '% PLOTFLD: No data in the specified range'
          if wnew ge 0 then begin
            wdelete, wnew
            if wsave ge 0 then begin
              wset, wsave
              !x = xsave
              !y = ysave
              !z = zsave
              !p = psave
            endif
          endif
          return
        endif
        fmin= min(cont(okx), max = fmax)
      endelse

      if n_elements(levels) gt 1 then fdef = min([fmin, levels]) else $
          fdef = fmin
      if lcfmin eq 0 then fdef = 0.0
      ok = where(strpos(tag_names(a), 'TRIANGLES') ge 0 , nok)
      if nok gt 0 then $
        if n_elements (a.triangles) lt 3 then nok = 0
      if nok eq 0 then begin
        xbar = total(xx)/n_elements(xx)
        dxi = 1.0/(max(xx, min= mm)-mm)
        ybar = total(yy)/n_elements(yy)
        dyi = 1.0/(max(yy, min= mm)-mm)
        triangulate, (xx-xbar)*dxi, dyi*(yy-ybar), tri
      endif else tri= a.triangles
;
; account for a problem with too much precision
;
      mm = max(xx[tri], min= mmm)-mmm
      if (mmm/mm le 0.0) then begin
        ok = where((abs(xx) lt 2.0e-7*mm) and (xx ne 0.0), nok)
        if nok gt 0 then begin
          if not keyword_set(quiet) then begin
            print, '     ',strtrim(nok, 2), ' Values of X set to zero'
            print, '     Range of zeroed values: ', $
              strtrim(min(xx[ok], max = mm),2), $
              ' to ', strtrim(mm, 2)
          endif
          xx[ok] =0.0
        endif
      endif
      mm = max(yy[tri], min= mmm)-mmm
      if (mmm/mm le 0.0) then begin
        ok = where((abs(yy) lt 2.0e-7*mm) and (yy ne 0.0), nok)
        if nok gt 0 then begin
          if not keyword_set(quiet) then begin
            print, '     ',strtrim(nok, 2), ' Values of Y set to zero'
            print, '     Range of zeroed values: ', $
              strtrim(min(yy[ok], max = mm),2), $
              ' to ', strtrim(mm, 2)
          endif
          yy[ok] =0.0
        endif
      endif
;  Limit the error to 5% of the spacing
      xerr = xerr < 0.05 *dxbig
      yerr = yerr < 0.05 * dybig
      farray = trigrid(xx, yy, cont, tri, [dxbig, dybig], $
        [xrbig(0)+xerr, yrbig(0)+yerr,xarray(nxbig-1)-xerr, $
        yarray(nybig-1)-yerr], miss = 1.0e35)
;
      bytval = make_array(nxbig, nybig , val=1B, /byt)
      ok = where (farray ge 9.0e34, nok)
      if nok gt 0 then begin
        bytval(ok)= 0B
        farray(ok) = fdef
      endif
      if smo gt 0 then begin
        if smo le nxbig and smo le nybig then $
          farray = smooth(temporary(farray), smo)
      endif
      if keyword_set(pixplot) and color(0) eq -2 then modsw =1 else modsw = 0
    endif else begin
;     Regular (rectilinear grid) data
;     if we need a big grid then make it
      if (nxbig gt 0) then begin
        farray = make_array(nxbig, nybig)
        bytval = make_array(nxbig, nybig, /byt, val = 0B)
      endif
;     now interpolate onto the single grid and find the amplitude max and min
      for ii = 0 ,nblock -1 do begin
        iblock = blockarray(ii)
;       get the data - if first block we already have it
        if ii gt 0  then begin
          cont = 0
          cont = reform(get_array(a, fld, iblock), /overwrite)
          if transplot ne 0 then cont = transpose(temporary(cont))
          xx = get_array(a, xval, iblock)
          nx = n_elements (xx)
          yy = get_array(a, yval, iblock)
          ny = n_elements (yy)
          if mult ne 1.0 then cont = mult *temporary(cont)
          if addit ne 0.0 then cont = addit +temporary(cont)
          if smo gt 0 then begin
            if smo le nx and smo le ny then $
              cont = smooth(temporary(cont), smo)
          endif
        endif
;       do the interpolation if necessary
        if nxbig gt 0 then begin
          if lcunif eq 0 then begin
;           Non-uniform X and Y axis
            if pixplot gt 0 then  begin
              if xx[0] lt xx[nx-1] then begin
                xfudgs  = xerr + (xx(1)-xx(0))*0.49
                xfudgf  = xerr + (xx(nx-1)-xx(nx-2))*0.49
              endif else begin
                xfudgf  = xerr - (xx(1)-xx(0))*0.49
                xfudgs  = xerr - (xx(nx-1)-xx(nx-2))*0.49
              endelse
              if yy[0] lt yy[ny-1] then begin
                yfudgs  = yerr + (yy(1)-yy(0))*0.49
                yfudgf  = yerr + (yy(ny-1)-yy(ny-2))*0.49
              endif else begin
                yfudgf  = yerr - (yy(1)-yy(0))*0.49
                yfudgs  = yerr - (yy(ny-1)-yy(ny-2))*0.49
              endelse
            endif else begin
              xfudgs  = xerr
              yfudgs  = yerr
              xfudgf  = xerr
              yfudgf  = yerr
            endelse
;           get block min and max
            xbmin = min(xx, max = xbmax) - xfudgs
            xbmax = xbmax + xfudgf
            ybmin = min(yy, max= ybmax) - yfudgs
            ybmax = ybmax + yfudgf
            okx = where(xarray ge xbmin and xarray le xbmax, nokx)
            oky = where(yarray ge ybmin and yarray le ybmax, noky)
            if nokx gt 0 and noky gt 0 then begin
              xtemp = xarray(okx)
              ytemp = yarray(oky)
;              xtemp = get_yarr(xx, findgen(nx), xtemp, $
;                 err=max([xfudgs, xfudgf])+xerr)
;              ytemp = get_yarr(yy, findgen(ny), ytemp, $
;                 err=max([yfudgs, yfudgf])+yerr)
              xtemp = findex(xx, xtemp)
              ytemp = findex(yy, ytemp)
              ix = okx(0)
              ifx = okx(nokx-1)
              iy = oky(0)
              ify = oky(noky-1)
            endif else nokx = 0
          endif else begin
;           Uniform X and Y axis
            if pixplot gt 0 then  begin
              xfudg  = xerr + dx(iblock-1) *0.49
              yfudg  = yerr + dy(iblock-1) *0.49
            endif else begin
              xfudg  = xerr
              yfudg  = yerr
            endelse
            if xx(0) -xfudg le xarray(0) then ix = 0 else $
              ix = long((xx(0)-xfudg-xarray(0))/dxbig)+1
            if yy(0) -yfudg le yarray(0) then iy = 0 else $
              iy = long((yy(0)-yfudg-yarray(0))/dybig)+1
            if xx(nx-1) + xfudg ge xarray(nxbig-1) then ifx = nxbig-1 else $
              ifx = (long((xx(nx-1)+xfudg- xarray(0))/dxbig)+1)< (nxbig-1)
            if yy(ny-1) + yfudg ge yarray(nybig-1) then ify = nybig-1 else $
              ify = (long((yy(ny-1)+yfudg- yarray(0))/dybig)+1)< (nybig-1)
            nokx = ifx-ix+1
            noky = ify-iy+1
            if nokx gt 0 and noky gt 0 then begin
              xtemp =(xarray(ix)-xx(0))/dx(iblock-1) $
                 +findgen(nokx)*(dxbig/dx(iblock-1))
              ytemp =(yarray(iy)-yy(0))/dy(iblock-1) $
                 +findgen(noky)*(dybig/dy(iblock-1))
            endif else nokx = 0
          endelse
          if nokx gt 0 then begin
            if keyword_set(pixplot) then begin
              xtemp = round (xtemp)
              ytemp = round (ytemp)
            endif else begin
              if xtemp (0) lt 0 then xtemp(0) = 0.0
              if ytemp (0) lt 0 then ytemp(0) = 0.0
              if xtemp (nokx -1) gt nx-1 then xtemp(nokx-1) = nx-1
              if ytemp (noky -1) gt ny-1 then ytemp(noky-1) = ny-1
            endelse
            if keyword_set(iflag) then begin
              if iflag eq 1 or iflag eq 3 then xtemp = round(xtemp)
              if iflag eq 2 or iflag eq 3 then ytemp = round(ytemp)
            endif
            farray(ix:ifx, iy:ify) = interpolate(cont, xtemp, ytemp, /grid)
            bytval(ix:ifx, iy:ify) = 1B
          endif
        endif
        if ii eq 0 then fldlab  = flab
        if ii eq 0 then begin
          if lcxrange eq 0 and lcyrange eq 0 then begin
            fmin = min(cont, max = fmax)
          endif else begin
            ix = where(xx ge xrange(0)-2*epsilx and $
              xx le xrange(1)+2*epsilx, nix)
            iy  = where(yy ge yrange(0)-2*epsily and $
              yy le yrange(1)+2*epsily, niy)
            if nix gt 0 and niy gt 0 then begin
              temp = cont(ix(0):ix(nix-1), iy(0):iy(niy-1))
              fmin = min(temp, max = fmax)
            endif
          endelse
        endif else begin
          if lcxrange eq 0 and lcyrange eq 0 then begin
            fmin2 = min(cont, max = fmax2)
            fmin = min([fmin, fmin2])
            fmax = max([fmax, fmax2])
          endif else begin
            ix = where(xx ge xrange(0) and xx le xrange(1), nix)
            iy  = where(yy ge yrange(0) and yy le yrange(1), niy)
            if nix gt 0 and niy gt 0 then begin
              temp = cont(ix(0):ix(nix-1), iy(0):iy(niy-1))
              fmin2 = min(temp, max = fmax2)
              if n_elements(fmin) gt  0 then begin
                fmin = min([fmin, fmin2])
                fmax = max([fmax, fmax2])
              endif else begin
                fmin = fmin2
                fmax = fmax2
              endelse
            endif
          endelse
        endelse
      endfor ; end of for loop over blocks
;     we have made a pixplot with farray so just plot it with tv
      if keyword_set(pixplot) and color(0) eq -2 then modsw =1 else modsw = 0
      pixplot = 0
    endelse    ; for end of rectilinear

;zzdebug
;       zzt(5) = systime(1)
;       zztc(5) = 'After interpolate '

;       make arrays for the lineouts, but first clean up

      if nxline+nyline gt 0 then begin
        totline = nxline+nyline
;       lineouts should fit in an array with following size.
        ntemplinsp = max([n_elements(a.x1),n_elements(a.x2),n_elements(a.x3)])+2
        if a.type eq 8 then ntemplinsp = max([ntemplinsp, nxbig, nybig])
        linsp= make_array(ntemplinsp, totline)
        linv = make_array(ntemplinsp, totline)
        linp = make_array(nblock+1, totline, /long)
        linnp = make_array(nblock, totline, /long)
        linn= make_array(totline, /long)
      endif
;
;     check on lineouts for conductors
;
      if n_elements(fmin) eq 0 then begin
        print, '% PLOTFLD: No field values in the designated range.'
        zrange =0
        return
      endif

      zrange = [fmin, fmax]
      ferr = 6.10352e-5*max(abs([fmax, fmin]))
      if cline ne 0 then begin
        make_conductor_ar, cline, cloc, cxarr, cyarr, cmaximum, ncloc
        if cline(0) gt 0 then begin
          nok = cloc(ncloc)
          cxarr = temporary(cxarr(0:nok-1))
                  cyarr = temporary(cyarr(0:nok-1))
          cval = make_array(nok)
          cnval = make_array(nok, /int)
          if n_elements(czero) eq 0 then czero = ferr else czero = abs(czero)
        endif
      endif

      if n_elements (fmin) eq 0 then begin
        Print, ' No data in the specified range'
         a = 0
          if wnew ge 0 then begin
            wdelete, wnew
            if wsave ge 0 then begin
              wset, wsave
              !x = xsave
              !y = ysave
              !z = zsave
              !p = psave
            endif
          endif
        return
      endif
      nl = n_elements(levels)
      lcfieldzero = 0
      if fmax-fmin le 3.05e-5*(fmax+fmin) then begin
        if keyword_set(always_plot) then begin
          if n_elements(levels) lt 2 then levels=(fmin+fmax)*0.5+[1, -1.]
          fmin = min(levels, max = fmax)
        endif else begin
          print, '% PLOTFLD: Field values have a range of zero.'
          if (nl le 0) || (cntour eq 0) then lcfieldzero = 1
        endelse
      endif
      if nl eq 0 then begin
        levels = levels_save
        nl = n_elements(levels)
      endif else levels = float(levels)
      if n_elements (nlev) eq 0 then nlev = nlevel_save

      if nl gt 0 then begin
          if nl ne nlev then begin
            if nl gt 1 then $
              fsmin = min(levels, max = fsmax)  else begin
              fsmin = fmin
              fsmax = fmax
            endelse
            levels = fsmin+(findgen(nlev)+0.5)*((fsmax-fsmin)/nlev)
          endif else begin
            if nl gt 1 then $
              fsmin = min(levels, max = fsmax)  else begin
              fsmin = fmin
              fsmax = fmax
            endelse
          endelse
      endif else begin
        fsmin = fmin
        fsmax = fmax
      endelse

      if quiet eq 0 then begin
        print, 'Range of the field is:', fmin, ' to' , fmax
        if cntour and 1 gt 0 then begin
          print, ' Contour with ', strtrim(nlev, 2), ' Levels'
          if nlev gt 1 then $
           print, 'First level is:' ,levels(0),', Spacing is', $
             levels(1)-levels(0) $
           else print, 'Contour level is:' ,levels(0)
        endif
      endif
      atlevels = levels
      atmin    = fmin
      atmax    = fmax
      ferr = 6.10352e-5*max(abs([fmax, fmin]))
      if n_elements (color) eq 1 then begin
        if color eq -1 then begin
          color = round(c_range[0]+findgen(nlev)*(nc_range/(nlev-1.0)))
        endif else if color eq -2 then begin
          loadxyct, /quiet, /freeze
          if nlev le 7 then begin
            color= 1+indgen(nlev)
          endif else begin
            color = make_array(nlev, /int, value = 7)
            ncold = 0
            for i = 1L, 6 do begin
              nc = nlev*i/7
              color(ncold:nc-1) = i
              ncold  = nc
            endfor
          endelse
        endif else if color ge 0 then begin
          if color gt 0 and color lt 8 then loadxyct, /quiet, /freeze
          if color eq 0 then color = !p.color
        endif else begin
          color=[1,2,3,4,5,6,7]
          loadxyct, /quiet, /freeze
          ncolor = n_elements(color)
          color = color(indgen(nlev) mod ncolor)
        endelse
      endif else begin
        color = (color)< (!d.table_size -1)
        ok = where (color lt 0, nok)
        if nok gt 0 then color[ok]=!p.background
      endelse
;
      lcextra = 0
      levelsave = levels
      colorsave = color
;
      if !x.crange(0) eq !x.crange(1) then over= 0
      b = size (xtitle)
      if b(b(0)+1) ne 7 then xtitle = xlab else xlab = xtitle (0)
      b = size (ytitle)
      if b(b(0)+1) ne 7 then ytitle = ylab else ylab = ytitle (0)
      b = size (title)
      if b(b(0)+1) ne 7 then begin
        if keyword_set(truncate) then begin
          st = strpos(a.clab, ' - ')
          if st gt 0 then title = strtrim(strmid(a.clab, 0, st), 2) $
            else title = a.clab
        endif else begin
          ii = strpos(a.clab, ';DATETIME=')
          jj = strpos(a.clab, ';;')
          ii = max([ii, jj])
          if ii gt 0 then title = strtrim(strmid(a.clab, 0, ii), 2) $
            else title = a.clab
        endelse
      endif else title = title(0)
      plotit = 1
;zzdebug
;       zzt(6) = systime(1)
;       zztc(6) = 'Before save '

      if n_elements(result) ne 0 or a.type eq 8 then begin
        if a.type eq 8 then begin
          if n_elements(result) eq 0 then result = work
          if keyword_set(no_copy) then result = -1
        endif
        if result gt 0 and result le maxstructure then begin
          nxs = n_elements (xarray)
          nys = n_elements (yarray)
          nfs = n_elements (farray)
;
          check_uniform, xarray, dxar
          check_uniform, yarray, dyar

          locx = make_array(2, 2, /long)
          locx(0, *)= [0, nxs]
          locx(1, *)= [0, nys]
          range = make_array(2, 2, 2)
          range(0, 0, *) = min(xarray, max = mx)
          range(1, 0, *) = mx
          range(0, 1, *) = min(yarray, max = mx)
          range(1, 1, *) = mx
          tlab = 'Contour'
          x1lab = make_array(1, /string, value = xtitle)
          x2lab = make_array(1, /string, value = ytitle)
          v1lab =  make_array(1, /string, value = title )
          if n_elements(nf) eq 0 then nf = n_elements(farray)
          tempstr = {type:1L, ndim:2L, nvect: 1L, block :1L, two: 3L, $
              size:[nxs, nys],range: range, order: 1L, slice:0.0, $
              connect: 0L,  locx:locx, $
              locv: [0L, nf], x1:xarray, x2:yarray, x3:0.0, x4:0.0, $
              v1:farray(0:nf-1), v2:0.0, v3:0.0, v4:0.0, $
              v5:0.0, v6:0.0, dx1 : dxar, dx2:dyar, dx3:0, dx4:0, $
              x1lab : x1lab, x2lab : x2lab, x3lab : '', x4lab :'', $
              v1lab : v1lab, v2lab :'', v3lab : '', v4lab : '',v5lab : '', $
              v6lab : '', file : a.file, spare : [0,0,0,0,0], tlab : tlab, $
              clab:  a.clab}
          i2s, tempstr, result, /no_copy
        endif
      endif
;zzdebug
;       zzt(7) = systime(1)
;       zztc(7) = 'Before plot '

      if n_elements (nfarray) gt 0 then begin
        if nfarray ne 0 then begin
          if a.type eq 8 or a.block gt 1 then begin
            xarrayout = xarray
            yarrayout = yarray
            farrayout = farray
          endif else begin
            xarrayout = xx
            yarrayout = yy
            farrayout = cont
          endelse
        endif
      endif

;     save the structure

      if (a.type eq 1 and not keyword_set(no_copy)) then i2s, a, work


      if a.type eq 8 then begin
          if (cntour and 4) gt 0 then begin
            xpix = temporary(xx)
            ypix = temporary(yy)
            fpix = temporary(cont)
          endif
          xx = temporary (xarray)
          nx = n_elements(xx)
          yy = temporary (yarray)
          ny = n_elements(yy)
          cont = temporary (farray)
      endif
      if keyword_set (coutput) then begin
        if coutput lt 0 then lcall = 1 else lcall = 0
        if min(coutput) lt 1 then coutput = abs(coutput[0]) > 1 else $
          coutput = fix(coutput[0]) > 1
        if coutput gt get_maxwdf() then begin
          print, '% PLOTFLD:  COUTPUT value must be less than get_maxwdf()'
          print, '            Either increase the number of WDF arrays'
          print, '         OR make COUTPUT smaller'
          if wnew ge 0 then begin
            wdelete, wnew
            if wsave ge 0 then begin
              wset, wsave
              !x = xsave
              !y = ysave
              !z = zsave
              !p = psave
            endif
          endif
          return
        endif
        get_contour_lines,cont, xx, yy, coutput, levelsave, clab = a.clab, $
            file = a.file, xlabel = xtitle, ylabel = ytitle , lcall = lcall
        if keyword_set (quiet) then begin
          if wnew ge 0 then begin
            wdelete, wnew
            if wsave ge 0 then begin
              wset, wsave
              !x = xsave
              !y = ysave
              !z = zsave
              !p = psave
            endif
          endif
          return
        endif
      endif
      if quiet ne 1 then begin
        if lcfieldzero eq 1 then begin
          help, cont, xx, yy
          if over eq 0 then  begin
            !p.multi (0)= !p.multi(0)+1
            contour, cont,xx, yy, xrange = xrange, yrange = yrange, $
              xtitle = xtitle, ytitle = ytitle, title = title, _extra=extrastuff
          endif
;         (note: parameters such as x(y)style and x(y)type are ignored)
          levels = levelsave
          color = colorsave
          a = 0
          if wnew ge 0 then begin
            wdelete, wnew
            if wsave ge 0 then begin
              wset, wsave
              !x = xsave
              !y = ysave
              !z = zsave
              !p = psave
            endif
          endif
          return
        endif

        if (keyword_set(shadesurf) or keyword_set(surf) $
            or keyword_set(xsurface) or keyword_set(cshadesurf) $
            or keyword_set(oshadesurf) $
            or (n_elements(cont) gt 20000 and (cntour and 1) gt 0)) $
            and not keyword_set (coutput)  then begin
          if a.type eq 8 or $
            (n_elements(cont) gt 20000 and nblock eq 1) then begin
            if keyword_set(fastplot) then begin
              nr  = 100
              nelx = n_elements(xx)
              nely = n_elements(yy)
              ix = findgen(nr+1)*((nelx-1)/float(nr))
              iy = findgen(nr+1)*((nely-1)/float(nr))
              xarray= interpolate(xx,ix, /grid)
              yarray= interpolate(yy,iy, /grid)
              farr= interpolate(cont,ix,iy, /grid)
            endif else begin
              farr = cont
              xarray = xx
              yarray = yy
            endelse
          endif else farr = farray
        endif

        if keyword_set (shadesurf) or keyword_set(cshadesurf) $
             or keyword_set(oshadesurf) then begin
          yeng_scale=1.00
          xeng_scale=1.00
          oldfont = !p.font
   ;       !p.font = -1
          if n_elements(axsurf) eq 0 then axsurf = 30.0
          if n_elements(azsurf) eq 0 then azsurf = 30.0
          epsil = (xrange(1)-xrange(0))/16384
          okx  = where (xarray ge xrange(0)-epsil and $
             xarray le xrange(1)+epsil, nokx)
          epsil = (yrange(1)-yrange(0))/16384
          oky  = where (yarray ge yrange(0)-epsil and $
             yarray le yrange(1)+epsil, noky)
          multerase = !p.multi(0)
          fsiz = (!p.charsize eq 0) ? 1.5: 1.1*!p.charsize
          if nokx gt 1 and noky gt 1 then  begin
            if keyword_set (shadesurf) then $
              shade_surf,  farr(okx(0):okx(nokx-1), oky(0):oky(noky-1)), $
                xarray(okx(0):okx(nokx-1)), yarray(oky(0):oky(noky-1)), $
                xrange = xrange, yrange= yrange, /save, $
                title = title , xtitle = xtitle, ytitle = ytitle, $
                ax = axsurf, az= azsurf, chars = fsiz, _extra=cs_extra $
            else if  keyword_set(cshadesurf) then begin
              tmp = farr(okx(0):okx(nokx-1), oky(0):oky(noky-1))
              xmarg =!x.margin
              nc10 = nc_range
;              if keyword_set (thermometer) then xmarg[1] = xmarg[0]*0.8
              stmp = bytscl(tmp, min= fxmin, max = fsmax, top=nc10)+c_range[0]
              shade_surf, tmp, shades = stmp, xmargin = xmarg, $
                xarray(okx(0):okx(nokx-1)), yarray(oky(0):oky(noky-1)), $
                xrange = xrange, yrange= yrange, /save, $
                title = title , xtitle = xtitle, ytitle = ytitle, $
                ax = axsurf, az= azsurf, chars = fsiz, _extra=cs_extra
              if cshadesurf gt 0 then $
                surface, tmp, /noerase, thick = 0.6, xst = 4, yst =4, $
                xarray(okx(0):okx(nokx-1)), yarray(oky(0):oky(noky-1)), $
                xrange = xrange, yrange= yrange, /save, $
                title = title , xtitle = xtitle, ytitle = ytitle, $
                ax = axsurf, az= azsurf, chars = fsiz, xmargin = xmarg , $
                zst=4, _extra= cs_extra
              if keyword_set(thermometer) then begin
                 range = float(fsmax-fsmin)- ferr -ferr
                 therm_col = findgen(nc_range)
                 therm_amp = fsmin+ferr +therm_col*(range/(nc_range-1))
                 therm_col= therm_col+c_range[0]
                 if n_elements (tparam) eq 0 then begin
                   if size(thermometer, /type) eq 8 then tparam = thermometer
                 endif
                 thermometer, therm_amp, therm_col,  _extra = tparam
               endif
            endif else begin
              oarr = farr[okx(0):okx(nokx-1),  oky(0):oky(noky-1)]
              oox = xarray(okx(0):okx(nokx-1))
              ooy = yarray(oky(0):oky(noky-1))
              ozrange = [min(oarr, max =m), m]
              nxr = normalize (xrange, position = [-1,1])
              nyr = normalize (yrange, position = [-1,1])
              nzr = normalize (zrange, position = [-1,1])
              nxx = n_elements(oox)
              nyy = n_elements(ooy)
              vert = bytscl(reform(oarr, nxx*nyy), top = 234)+20b
              tvlct, /get, red, green, blue
              opalette = OBJ_NEW('IDLgrPalette', red, green, blue)

              Omod2 = obj_new ('idlgrsurface', oarr, oox, ooy, style = 2, $
                shading = 1, palette=opalette, vert_colors=vert )
              omod2 -> setproperty, xcoord_conv = nxr, ycoord_conv = nyr, $
                zcoord_conv = nzr
              omodel= obj_new('idlgrmodel')
              omodel -> add, omod2

            ofont = obj_new('idlgrfont')
            oxtit = obj_new('idlgrtext', xtitle, font = ofont)
            oytit = obj_new('idlgrtext', ytitle, font = ofont)
            oztit = obj_new('idlgrtext', title, font = ofont)

            pdel = 0.04
            px = xrange[0]-(xrange[1]-xrange[0])*pdel
            py = yrange[0]-(yrange[1]-yrange[0])*pdel
            pz = zrange[0]-(zrange[1]-zrange[0])*pdel
            location =[ px, py, pz]
            oxa = obj_new('idlgraxis', 0, title = oxtit, range = xrange, location = location)
            oya = obj_new('idlgraxis', 1, title = oytit, range = yrange, location = location)
            oza = obj_new('idlgraxis', 2, title = oztit, range = zrange, location = location)

            oxa -> setproperty, xcoord_conv = nxr, ycoord_conv = nyr, zcoord_conv = nzr
            oya -> setproperty, xcoord_conv = nxr, ycoord_conv = nyr, zcoord_conv = nzr
            oza -> setproperty, xcoord_conv = nxr, ycoord_conv = nyr, zcoord_conv = nzr

            omodel -> add, oxa
            omodel -> add, oya
            omodel -> add, oza

            xobjview, omodel, /block
            obj_destroy, omodel
            obj_destroy, opalette
            obj_destroy, oxa
            obj_destroy, oya
            obj_destroy, oza
            obj_destroy, oxtit
            obj_destroy, oytit
            obj_destroy, oztit
            obj_destroy, ofont

            endelse
          endif else begin
            print, ' No valid data for this surface plot'
            return
          endelse
          if header ne 0  and multerase eq 0 then header, header_text
          plotit = 0
          !p.font = oldfont
        endif

        if keyword_set (surf) then begin
          yeng_scale=1.00
          xeng_scale=1.00
          oldfont = !p.font
          !p.font = -1
          if n_elements(axsurf) eq 0 then axsurf = 30.0
          if n_elements(azsurf) eq 0 then azsurf = 30.0
          if plotit eq 0 then hak, /mes  else begin
            epsil = (xrange(1)-xrange(0))/16384
            okx  = where (xarray ge xrange(0)-epsil and $
               xarray le xrange(1)+epsil, nokx)
            epsil = (yrange(1)-yrange(0))/16384
            oky  = where (yarray ge yrange(0)-epsil and $
               yarray le yrange(1)+epsil, noky)
          endelse
          multerase = !p.multi(0)
          fsiz = (!p.charsize eq 0) ? 1.5: 1.1*!p.charsize
          nsval =0
          if nokx gt 1 and noky gt 1 then begin
            if n_elements (sextra) gt 0 then begin
              if size(sextra, /type) eq 8 then begin
                llist = tag_names(sextra)
                if keyword_set (iflag) then iflg = (round(iflag) < 2) > 1 $
                  else iflg = 2
                ok = get_match('^V', llist, regexp=0, nelements = nel)
                if nel ne 1 then begin
                  print, '% PLOTFLD: Unable to highlight values'
                  print, '           Tag Name beginning with V has ', nel, ' values'
                  print, '           Please correct value of S_extra'
                  help, /structure, sextra
                  return
                endif
                svalues = sextra.(ok[0])
                nsval = n_elements(svalues)
                ok = get_match('^C', llist, regexp=0, nelements = nel)
                if nel ne 1 then begin
                  scolors = 1 +indgen(7)
                  nscol =7
                endif else begin
                  scolors = sextra.(ok[0])
                  nscol = n_elements(scolors)
                endelse
                ok = get_match('^S', llist, regexp=0, nelements = nel)
                if nel eq 1 then begin
                  spectrum = sextra.(ok[0])
                  if keyword_set(spectrum) then begin
                    tvlct, /get , rsave, gsave, bsave
                    rainbow, (nsval > 2) < 245
                    scolors = (nsval le 245) ? indgen(nsval > 2) : $
                      round (findgen(nsval)/(nsval -1)*245.)
                    scolors = scolors+10
                    nscol = nsval >2
                  endif
                endif
                sarr = farr(okx(0):okx(nokx-1), oky(0):oky(noky-1))
                sxx = xarray(okx(0):okx(nokx-1))
                syy = yarray(oky(0):oky(noky-1))
                if iflg eq 1 then begin
                  ok = where(svalues le xrange[1] and svalues ge xrange[0], nsval)
                  if nsval eq 0 then begin
                    print, '% PLOTFLD: No values in the XRANGE:', xrange
                    return
                  endif
                  svalues = svalues[ok]
                  svalues =  round(findex(sxx, svalues))
                endif else begin
                  ok = where(svalues le yrange[1] and svalues ge yrange[0], nsval)
                  if nsval eq 0 then begin
                    print, '% PLOTFLD: No values in the XRANGE:', xrange
                    return
                  endif
                  svalues = svalues[ok]
                  svalues =  round(findex(syy, svalues))
                endelse
              endif else sext =0
            endif
            if keyword_set(iflag) then iflg = iflag
            if keyword_set(iflg) then begin
              if n_elements (sarr) eq 0 then begin
                sarr = farr(okx(0):okx(nokx-1), oky(0):oky(noky-1))
                sxx = xarray(okx(0):okx(nokx-1))
                syy = yarray(oky(0):oky(noky-1))
              endif
              if keyword_set(iflag) then ndata =1 else ndata = 0
              surface, sarr, sxx, syy, $
                xrange = xrange, yrange= yrange, /save, nodata = ndata, $
                title = title , xtitle = xtitle, ytitle = ytitle, $
                ax = axsurf, az= azsurf, chars= fsiz, _extra= cs_extra
              if ndata eq 1 then begin
                if iflg eq 1 then begin
                  for i = 0, n_elements(sxx)-1 do plots, $
                    make_array(noky, value= sxx[i]), syy, reform(sarr[i, *]), /t3d
                endif else begin
                  for i = 0, n_elements(syy)-1 do plots, $
                    sxx, make_array(nokx, value= syy[i]), reform(sarr[*, i]), /t3d
                endelse
              endif
            endif else $
              surface, farr(okx(0):okx(nokx-1), oky(0):oky(noky-1)), $
                xarray(okx(0):okx(nokx-1)), yarray(oky(0):oky(noky-1)), $
                xrange = xrange, yrange= yrange, /save, $
                title = title , xtitle = xtitle, ytitle = ytitle, $
                ax = axsurf, az= azsurf, chars= fsiz, _extra= cs_extra
            if nsval gt 0 then begin
              if iflg eq 1 then begin
                for i = 0, nsval-1 do plots, $
                  make_array(noky, value=sxx[svalues[i]]), syy, reform(sarr[svalues[i], *]), $
                    color = scolors[i mod nscol], /t3d
              endif else begin
                for  i = 0, nsval-1 do plots, $
                  sxx, make_array(nokx, value=syy[svalues[i]]), reform(sarr[*, svalues[i]]), $
                    color = scolors[i mod nscol], /t3d
              endelse
              if keyword_set(spectrum) then tvlct, rsave, gsave, bsave
            endif
          endif else begin
            print, ' No valid data for the surface plot'
            return
          endelse
          if header ne 0  and multerase  eq 0 then header, header_text
          plotit = 0
          !p.font = oldfont
        endif

        if keyword_set (xsurface )  then begin
          yeng_scale=1.00
          xeng_scale=1.00
          if plotit eq 0 then hak, /mes
          xsurface, farr, /block
          plotit = 0
        endif
;
; if we have not plotted a graph then we will make a contour plot
;
      endif
;zzdebug
;       zzt(8) = systime(1)
;       zztc(8) = 'After save '

      if cntour eq 0 and lclineout eq 0 then begin
        cont = 0
        a = 0
          if wnew ge 0 then begin
            wdelete, wnew
            if wsave ge 0 then begin
              wset, wsave
              !x = xsave
              !y = ysave
              !z = zsave
              !p = psave
            endif
          endif
        return

      endif

      if plotit eq 0 then hak, /mes
      lcplot = 0
          yeng_scale=1.00
          xeng_scale=1.00
      if right ne 0 then axis, $
         /yaxis, /save, !x.crange(1), 0, yrange = yrange,  ytitle= ytitle, $
         _extra = extrastuff

;;    Check to see if noerase is set
      if size(extrastuff, /type) eq 8 then begin
        ok = where ( strmid(tag_names(extrastuff), 0, 3) eq  'NOE', nok)
        noerase = nok
      endif else noerase = 0
      if cntour ne 0 and quiet ne 1 and not keyword_set(over) then begin
        if header ne 0  and multcnt eq 0 then begin
          if noerase eq 0 then erase
          header, header_text
          multcnt = 1
        endif
      endif
        if ((cntour and 2) gt 0  or (cntour and 4) gt 0)  $
          and quiet ne 1 then begin
          if a.type eq 8 and keyword_set (pixplot) then begin
            if not keyword_set(over) then begin
              wxsave = !x.region
              wysave = !y.region
              !p.multi (0)= !p.multi(0)+1
              contour, make_array(2,2), xrange, yrange,  /nodata, $
                xrange = xrange, yrange = yrange, position = position, /dev, $
                xtitle = xtitle,  ytitle = ytitle, title = title, $
                _extra = extrastuff
              !x.region=wxsave
              !y.region=wysave
            endif
            okx = where ((xpix ge !x.crange(0) and xpix le !x.crange(1)) and $
              (ypix ge !y.crange(0) and ypix le !y.crange(1)),   nokx)
            if nokx gt 0 then begin
              xpix = xpix(okx)
              ypix = ypix(okx)
              zz = fpix(okx)
              nc10 = nc_range
              range = float(fsmax-fsmin)
              cc = round((zz-fsmin)*(nc10/range))+ c_range[0]
              ok = where (zz le fsmin, nok)
              if nok gt 0 then cc(ok) = !p.background
              if pixplot lt 0 then syms = abs(pixplot) else syms = 1.2
              for i = 0L, nokx-1 do $
                 plots, xpix(i), ypix(i), color = cc(i), syms = syms, psym = 8
              if keyword_set(thermometer) then begin
                 therm_col = findgen(nc10)
                 therm_amp = fsmin +therm_col*(range/(nc10-1))
                 therm_col= therm_col+ c_range[0]
              endif
            endif
          endif else  if a.type eq 8 then begin
;           Image plot of finite element data
            wxsave = !x.region
            wysave = !y.region
            bytvl = get_byte_val (cont, fsmin, fsmax, c_extra, bytval, $
              therm_amp, therm_col, modsw, c_range, clip_values, scale_col, $
              gamma_col, mean_col, invertctable )
            if not keyword_set(over) then begin
              tv,bytvl , xpxbig(0)+position(0), ypxbig(0)+position(1), $
                xsiz = xsiz, ysiz = ysiz
              contour, make_array(2,2), xrange, yrange, /noerase, /nodata, $
                xrange = xrange, yrange = yrange, position = position, /dev, $
                xtitle = xtitle,  ytitle = ytitle, title = title, $
                _extra = extrastuff
            endif else begin
              if !d.name eq 'X' then wset, !d.window
              if (!d.n_colors gt !d.table_size) then begin
                TrueColor=1
                tvlct, /get, rr, gg, bb
              endif else TrueColor=0

              if (!d.name eq 'PS' or !d.name eq 'PCL')then old = old_bytvl else $
                old = tvrd(xpxbig(0)+position(0), ypxbig(0)+position(1), $
                   xsiz, ysiz, true = TrueColor)
              if over lt 0 then begin
                if TrueColor eq 0 then $
                  ok = where (old eq !p.background, nok) $
                else begin
                  pback = (!p.background eq 0) ? 0:255
                  ok = where (old[0, *, *] eq rr[pback] $
                         and old[1, *, *] eq gg[pback] $
                         and old[2, *, *] eq bb[pback], nok)
                endelse

                if nok gt 0 then begin
                  if TrueColor eq 0 then begin
                    old(ok) = bytvl(ok)
                    bytvl = temporary(old)
                  endif else begin
                    rold = reform(old[0, *, *])
                    rold[ok] = rr[bytvl[ok]]
                    gold = reform(old[1, *, *])
                    gold[ok] = gg[bytvl[ok]]
                    bold = reform(old[2, *, *])
                    bold[ok] = bb[bytvl[ok]]
                    bytvl = temporary(old)
                    bytvl[0, *, *] = rold
                    bytvl[1, *, *] = gold
                    bytvl[2, *, *] = bold
                  endelse
                  tv,bytvl , xpxbig(0)+position(0), ypxbig(0)+position(1), $
                     xsiz = xsiz, ysiz = ysiz, true = TrueColor
                endif
              endif else begin
                pback  = (!p.background eq 0) ? 0:255
                ok = where (bytvl eq pback, nok)
                if nok gt 0 and truecolor eq 0 then bytvl(ok) = old(ok)
                if truecolor eq 1 and nok gt 0 then begin
                  ss = size(bytvl)
                  tmp = temporary(bytvl)
                  bytvl = make_array(3, ss[1], ss[2])
                  rrnew = rr[tmp]
                  rrnew[ok]=(old[0, *, *])[ok]
                  bytvl [0, *, *] = rrnew
                  rrnew = gg[tmp]
                  rrnew[ok]=(old[1, *, *])[ok]
                  bytvl [1, *, *] = rrnew
                  rrnew = bb[tmp]
                  rrnew[ok]=(old[2, *, *])[ok]
                  bytvl [2, *, *] = rrnew
                  tv, bytvl, xpxbig(0)+position(0), ypxbig(0)+position(1), $
                     xsiz = xsiz, ysiz = ysiz, /true
                endif else tv, bytvl, xpxbig(0)+position(0), ypxbig(0)+position(1), $
                     xsiz = xsiz, ysiz = ysiz
              endelse
            endelse
            old_bytvl = temporary(bytvl)
            !x.region=wxsave
            !y.region=wysave
          endif else begin
;           Image plot of regular images
            wxsave = !x.region
            wysave = !y.region
            bytvl = get_byte_val (farray, fsmin, fsmax, c_extra, bytval, $
              therm_amp, therm_col, modsw, c_range, clip_values, scale_col, $
              gamma_col, mean_col, invertctable)
;           tv,bytvl, xpxbig(0)+position(0), ypxbig(0)+position(1), $
;             xsiz = xsiz, ysiz = ysiz

            if not keyword_set(over) then begin
              tv,bytvl, xpxbig(0)+position(0), ypxbig(0)+position(1), $
                xsiz = xsiz, ysiz = ysiz
              contour, make_array(2,2), xrange, yrange, /noerase, /nodata, $
                xrange = xrange, yrange = yrange, position = position, /dev, $
                xtitle = xtitle,  ytitle = ytitle, title = title, $
                _extra = extrastuff
            endif else begin
              if !d.name eq 'X' then wset, !d.window
              if (!d.n_colors gt !d.table_size) then begin
                TrueColor=1
                tvlct, /get, rr, gg, bb
              endif else TrueColor=0
              if (!d.name eq 'PS' or !d.name eq 'PCL') then old= old_bytvl else $
                old = tvrd(xpxbig(0)+position(0), ypxbig(0)+position(1), $
                   xsiz, ysiz, true = TrueColor)
              if over lt 0 then begin
                if TrueColor eq 0 then $
                  ok = where (old eq !p.background, nok) $
                else begin
                  pback  = (!p.background eq 0) ? 0:255
                  ok = where (old[0, *, *] eq rr[pback] $
                         and old[1, *, *] eq gg[pback] $
                         and old[2, *, *] eq bb[pback], nok)
                endelse
                if nok gt 0 then begin
                  if TrueColor eq 0 then begin
                    old(ok) = bytvl(ok)
                    bytvl = temporary(old)
                  endif else begin
                    rold = reform(old[0, *, *])
                    rold[ok] = rr[bytvl[ok]]
                    gold = reform(old[1, *, *])
                    gold[ok] = gg[bytvl[ok]]
                    bold = reform(old[2, *, *])
                    bold[ok] = bb[bytvl[ok]]
                    bytvl = temporary(old)
                    bytvl[0, *, *] = rold
                    bytvl[1, *, *] = gold
                    bytvl[2, *, *] = bold
                  endelse
                  tv, bytvl, xpxbig(0)+position(0), ypxbig(0)+position(1), $
                     xsiz = xsiz, ysiz = ysiz, true = TrueColor
                endif
              endif else begin
                pback  = (!p.background eq 0) ? 0:255
                ok = where (bytvl eq pback, nok)
                if nok gt 0 and truecolor eq 0 then bytvl(ok) = old(ok)
                if truecolor eq 1 and nok gt 0 then begin
                  ss = size(bytvl)
                  tmp = temporary(bytvl)
                  bytvl = make_array(3, ss[1], ss[2])
                  rrnew = rr[tmp]
                  rrnew[ok]=(old[0, *, *])[ok]
                  bytvl [0, *, *] = rrnew
                  rrnew = gg[tmp]
                  rrnew[ok]=(old[1, *, *])[ok]
                  bytvl [1, *, *] = rrnew
                  rrnew = bb[tmp]
                  rrnew[ok]=(old[2, *, *])[ok]
                  bytvl [2, *, *] = rrnew
                  tv, bytvl, xpxbig(0)+position(0), ypxbig(0)+position(1), $
                     xsiz = xsiz, ysiz = ysiz, /true
                endif else tv, bytvl, xpxbig(0)+position(0), ypxbig(0)+position(1), $
                     xsiz = xsiz, ysiz = ysiz
              endelse
            endelse
            old_bytvl = temporary(bytvl)
            !x.region=wxsave
            !y.region=wysave
          endelse
          lcplot = 1
        endif
        if n_elements (xrange_first) gt 1 then begin
          xrange_second = !x.crange
          yrange_second = !y.crange
          rexx= total(abs(xrange_first-xrange_second))/(xrange_first[1]-xrange_first[0])
          reyy= total(abs(yrange_first-yrange_second))/(yrange_first[1]-yrange_first[0])
          if rexx gt 1 then begin
            print, '% PLOTFLD: Possible error in scaling the plot.'
            print, 'xrange_first =', xrange_first
            print, 'xrange_second =', xrange_second
          endif
          if reyy gt 1 then begin
            print, '% PLOTFLD: Possible error in scaling the plot.'
            print, 'yrange_first =', yrange_first
            print, 'yrange_second =', yrange_second
          endif
        endif
        for ii =0, nblock-1 do begin
          i = blockarray(ii)
          if (ii gt 0 or nblock gt 1)  then begin
            xx = get_array (a, xval, i, lab = lab)
            nx = n_elements(xx)
            yy = get_array (a, yval, i, lab = lab)
            ny = n_elements(yy)
            cont = reform (get_array (a, fld, i, lab = lab), /overwrite)
            if transplot ne 0 then cont = transpose(temporary(cont))
            if mult ne 1.0 then cont = mult *temporary(cont)
            if addit ne 0.0 then cont = addit +temporary(cont)
            if smo gt 0 then begin
              if smo le nx and smo le ny then $
                 cont = smooth(temporary(cont), smo)
            endif
            if nx eq 0 or ny eq 0 then begin
              print, 'We have a problem at contour'
              print, ' X and Y arrays have ', nx, ny, ' elements'
              a = 0
              return
            endif

          endif else if keyword_set(fastplot) and nxline+nyline eq 0 then begin
            cont= temporary(farr)
            xx = temporary(xarray)
            yy = temporary(yarray)
          endif
          if quiet ne 1  and (cntour and 1) gt 0 then begin
            if ii eq 0 then begin
              if n_elements(therm_col) lt 2 then begin
                therm_col = colorsave
                therm_amp = levelsave
              endif else begin
                ovamp = levelsave
                ovcol = colorsave
              endelse
              ntcol = n_elements(therm_col)
              ntamp = n_elements(therm_amp)
              if keyword_set(thermometer) && (ntcol lt ntamp) then $
                while n_elements(therm_col) lt ntamp do $
                  therm_col= [therm_col, therm_col]
              if n_elements(therm_col) gt ntamp then therm_col = therm_col[0:ntamp-1]
            endif
;    changed to eliminate informational print if one of the blocks had no data.
;;            if max(xx, min = mx) lt xrange(0) or mx gt xrange(1) or $
;;              max(yy, min = my) lt yrange(0) or my gt yrange(1) then begin
;;              print, ' No contours for block', i
;;            endif else begin
            if (max(xx, min = mx) ge xrange(0)) && (mx le xrange(1)) &&  $
              (max(yy, min = my) ge  yrange(0)) && (my le yrange(1)) then begin
              if lcplot eq 0  and over eq 0 then begin
                wxsave = !x.region
                wysave = !y.region
                contour, make_array(2,2), xrange, yrange,/noerase, /nodata, $
                  xrange = xrange, yrange = yrange, position = position, /dev, $
                  xtitle = xtitle,  ytitle = ytitle, title = title, $
                  color = !p.color, _extra = extrastuff
                !x.region=wxsave
                !y.region=wysave
                lcplot =1
              endif
              contour, cont, level= levels, xx, yy, fill = fill, $
                /overplot, c_color = color, _extra = extrastuff
            endif
          endif
;
;    LINEOUTS
; mrdougl
; commented out the nx and ny lines but I don't know why??
          nx = n_elements(xx)
          ny = n_elements(yy)
          if nxline gt 0 then for j = 0, nxline-1 do begin
            epline = (yy(ny-1)-yy(0))*6.1035156e-05
            if xline (j) ge (yy(0)-epline) and xline(j) le (yy(ny-1)+epline) $
              then begin
              ylin = xlineout (cont, yy, xline(j), widsav)
              if linn(j) eq 0 then begin
                linsp(0:nx-1, j) = xx
                linv(0:nx-1, j ) = ylin
                linp(1,j) = nx
                linnp(0,j) = nx
              endif else begin
                n = linn(j)
                linnp(n, j) = nx
                linp (n+1, j) = linp(n, j)+nx
                linsp(linp(n,j):linp(n+1, j)-1, j) = xx
                linv (linp(n,j):linp(n+1, j)-1, j) = ylin
              endelse
              linn(j)= linn(j)+1
              if  (n_elements (ylin) lt nx ) then begin
                 print, ' Error in obtaining lineout at', xline(j)
                 a = 0
                 if wnew ge 0 then begin
                   wdelete, wnew
                   if wsave ge 0 then begin
                     wset, wsave
                     !x = xsave
                     !y = ysave
                     !z = zsave
                     !p = psave
                   endif
                 endif

                 return
              endif
            endif
          endfor
          if nyline gt 0 then for j = nxline,nxline+ nyline-1 do begin
            epline = (xx(nx-1)-xx(0))*6.1035156e-05
            if yline(j-nxline) ge (xx(0)-epline) and yline(j-nxline) le $
              (xx(nx-1)+epline) then  begin
              ylin = ylineout (cont, xx, yline(j-nxline), widsav)
              if linn(j) eq 0 then begin
                linsp(0:ny-1, j) = yy
                linv(0:ny-1, j ) = ylin
                linp(1,j) = ny
                linnp(0,j) = ny
              endif else begin
                n = linn(j)
                linnp(n, j) = ny
                linp (n+1, j) = linp(n, j)+ny
                linsp(linp(n,j):linp(n+1, j)-1, j) = yy
                linv (linp(n,j):linp(n+1, j)-1, j) = ylin
              endelse
              linn(j)= linn(j)+1
              if  (n_elements (ylin) lt ny ) then begin
                 print, ' Error in obtaining lineout at', xline(j)
                 a=0
                 if wnew ge 0 then begin
                   wdelete, wnew
                   if wsave ge 0 then begin
                     wset, wsave
                     !x = xsave
                     !y = ysave
                     !z = zsave
                     !p = psave
                   endif
                 endif
                 return
              endif
            endif
          endfor
          if cline(0) gt 0 then for j = 0, ncloc-1 do begin
            xt = cxarr(cloc(j):cloc(j+1)-1)
            yt = cyarr(cloc(j):cloc(j+1)-1)
            ok = where (xt+xerr gt xx(0) and xt-xerr lt xx(nx-1) and $
                  yt+yerr gt yy(0) and yt-yerr lt yy(ny-1), nok)
            if nok gt 0 then for i = 0, nok-1 do begin
              junkx= min(abs(xt(ok(i))-xx), imx)
              if junkx le xerr then  lcx = 0 else begin
              lcx = 1
              if xt(ok(i)) gt xx(imx) then begin
                imx1 = imx
                imx2 = imx+1
                m2 = junkx
                m1 = xx(imx+1)-xt(ok(i))
              endif else begin
                imx2 = imx
                imx1 = imx-1
                m1 = junkx
                m2 = xt(ok(i))-xx(imx-1)
              endelse
            endelse
            junky= min(abs(yt(ok(i))-yy), imy)
            if junky le yerr then  lcy = 0 else begin
              lcy = 1
              if yt(ok(i)) gt yy(imy) then begin
                imy1 = imy
                imy2 = imy+1
                m4 = junky
                m3 = yy(imy+1)-yt(ok(i))
              endif else begin
                imy2 = imy
                imy1 = imy-1
                m3 = junky
                m4 = yt(ok(i))-yy(imy-1)
              endelse
            endelse
            if lcx eq 0 and lcy eq 0 then cvaltmp = cont(imx, imy) $
            else begin
              temp = 0.0
              weight = 0.0
              if lcx eq 0 then begin
                if czero ne 0.0 then begin
                  if abs(cont(imx, imy1)) gt czero then begin
                    temp = m3*cont(imx, imy1)
                    weight = m3
                  endif
                  if abs(cont(imx, imy2)) gt czero then begin
                    temp = temp + m4*cont(imx, imy2)
                    weight = weight + m4
                  endif
                endif
                if weight gt 0.0 then cvaltmp = temp/weight else $
                cvaltmp = (m3*cont(imx, imy1)+m4*cont(imx, imy2))/(m3+m4)
              endif else if lcy eq 0 then begin
                if czero ne 0.0 then begin
                  if abs(cont(imx1, imy)) gt czero then begin
                    temp = m1*cont(imx1, imy)
                    weight = m1
                  endif
                  if abs(cont(imx2, imy)) gt czero then begin
                    temp = temp + m2*cont(imx2, imy)
                    weight = weight + m2
                  endif
                endif
                if weight gt 0.0 then cvaltmp = temp/weight else $
                cvaltmp = (m1*cont(imx1, imy)+m2*cont(imx2, imy))/(m1+m2)
              endif else begin
                if czero ne 0.0 then begin
                  if abs(cont(imx1, imy1)) gt czero then begin
                    temp = m1*m3*cont(imx1, imy1)
                    weight = m1*m3
                  endif
                  if abs(cont(imx1, imy2)) gt czero then begin
                    temp = temp + m1*m4*cont(imx1, imy2)
                    weight = weight + m1*m4
                  endif
                  if abs(cont(imx2, imy2)) gt czero then begin
                    temp = temp + m2*m4*cont(imx2, imy2)
                    weight = weight + m2*m4
                  endif
                  if abs(cont(imx2, imy1)) gt czero then begin
                    temp = temp + m2*m3*cont(imx2, imy1)
                    weight = weight + m2*m3
                  endif
                endif
                if weight gt 0.0 then cvaltmp = temp/weight else $
                cvaltmp = (m1*m3*cont(imx1, imy1)+m2*m3*cont(imx2, imy1) $
                  + m1*m4*cont(imx1, imy2)+m2*m4*cont(imx2, imy2)) $
                  / ((m1+m2)*(m3+m4))
              endelse
            endelse
            if cnval(cloc(j)+ok(i)) eq 0 then cval(cloc(j)+ok(i)) = cvaltmp $
              else cval(cloc(j)+ok(i)) = (cval(cloc(j)+ok(i))*cnval(cloc(j) + $
               ok(i)) + cont(imx, imy))/(cnval(cloc(j)+ok(i))+1)
            cnval(cloc(j)+ok(i))= cnval(cloc(j)+ok(i))+1
          endfor
        endfor
      endfor
; help , noerase
      if ((keyword_set(fill) and ((cntour and 1) eq 1)) or $
         ((cntour and 2) eq 2)) and (noerase eq 0) then begin
         blanknam = make_array(30,value = ' ')
         xcrange = !x.crange
         ycrange = !y.crange
         if !x.type eq 1 then xcrange = 10^xcrange
         if !y.type eq 1 then ycrange = 10^ycrange
          if keyword_set(negative) then gridcol = !p.background else $
            gridcol = !p.color
        axis, xaxis=0, xcrange[0], ycrange[0], xrange = xcrange, /xst, $
            xtickname= blanknam, color = gridcol, _extra = extrastuff_a
        axis, xaxis=1, xcrange[0], ycrange[1], xrange = xcrange, /xst, $
            xtickname= blanknam, color = gridcol, _extra = extrastuff_a
        if keyword_set(left) then begin
           axis, yaxis=0, xcrange[0], ycrange[0], yrange = ycrange, /yst, $
             ytickname= blanknam, color = gridcol, _extra = extrastuff_a
        endif else if keyword_set(right) then begin
           axis, yaxis=0, xcrange[0], oldycrange[0], yrange = oldycrange, $
             ytickname= blanknam, color = gridcol, /yst, _extra = old_extrastuff
           axis, yaxis=1, xcrange[1], ycrange[0], yrange = ycrange, /yst, $
             ytickname= blanknam, color = gridcol, _extra = extrastuff_a
        endif else begin
           axis, yaxis=0, xcrange[0], ycrange[0], yrange = ycrange, /ysty, $
             ytickname= blanknam, color = gridcol, _extra = extrastuff_a
           axis, yaxis=1, xcrange[1], ycrange[0], yrange = ycrange, /ysty, $
             ytickname= blanknam, color = gridcol, _extra = extrastuff_a
         endelse
      endif
      if nxline+nyline gt 0 then for j = 0, nyline+nxline-1 do begin
        if linn(j) eq 0 then begin
          if j lt nxline then begin
           i2w,  [xmin, xmax], [0.0, 0.0], out+j, eps = 3.0e-4, /xydata, $
           xlab = xlab, ylab = flab,clab='y = '+strtrim(string(xline(j)),2)+  $
              ' -'+a.clab, file=a.file
          endif else $
           i2w,  [xmin, xmax], [0.0, 0.0], out+j, eps = 3.0e-4,xlab = ylab, $
           ylab = flab,clab='x = '+strtrim(string(yline(j-nxline)),2)+  $
              ' -'+a.clab, file=a.file, /xydata
        endif else if linn(j) eq 1 then begin
          np = linp(1, j)
          if j lt nxline then begin
           i2w, linsp(0:np-1, j), linv(0:np-1, j), out+j, eps = 3.0e-4, $
           xlab = xlab, ylab = flab,clab='y = '+strtrim(string(xline(j)),2)+  $
              ' -'+a.clab, file=a.file, /xydata
          endif else $
           i2w,linsp(0:np-1,j),linv(0:np-1,j),out+j,eps=3.0e-4,xlab = ylab, $
           ylab = flab,clab='x = '+strtrim(string(yline(j-nxline)),2)+  $
              ' -'+a.clab, file=a.file, /xydata
        endif else begin
          n = linn(j)
          aa = sort (linsp(linp(0:n-1, j), j))
          ntemp = linp(n, j) + 2*n
          tempx = make_array(ntemp)
          tempy = tempx
          np = linp(n, j)
          st= 0
          for g = 0, n-1 do begin
            np = linnp(aa(g), j)
            if g gt 0 then begin
              xx = linsp(linp(aa(g), j):linp(aa(g)+1, j)-1, j)
              yy = linv(linp(aa(g), j):linp(aa(g)+1, j)-1, j)
              if xx(0) gt tempx(st-1)+xerr then begin
                if abs(xx(0) - tempx(st-1)) gt 0.1*(xx(1)-xx(0)) then begin
                  tempx(st) = tempx(st-1)+0.02*(xx(1)-xx(0))
                  tempx(st+1) = xx(0)-0.02*(xx(1)-xx(0))
                  tempy(st:st+1) = 0.0
                  st = st+2
                endif
              endif else begin
                ok = where(xx gt tempx(st-1)-xerr, np)
                if np gt 0 then begin
                  xx = xx(ok)
                  yy = yy(ok)
                endif
              endelse
              if np gt 0 then begin
                tempx(st:st+np-1)= xx
                tempy(st:st+np-1)= yy
              endif
              st = st+np
            endif else begin
              tempx(0:np-1) = linsp(linp(aa(g), j):linp(aa(g)+1, j)-1, j)
              tempy(0:np-1) = linv(linp(aa(g), j):linp(aa(g)+1, j)-1, j)
              st = st+np
            endelse
          endfor
          if j lt nxline then begin
           i2w, tempx(0:st-1), tempy(0:st-1), out+j, eps = 3.0e-4, $
           xlab = xlab, ylab = flab,clab='y = '+strtrim(string(xline(j)),2)+  $
              ' -'+a.clab, file=a.file, /xydata
          endif else $
           i2w, tempx(0:st-1), tempy(0:st-1), out+j, eps = 1.0e-3,xlab = ylab, $
           ylab = flab,clab='x = '+strtrim(string(yline(j-nxline)),2)+  $
              ' -'+a.clab, file=a.file, /xydata
        endelse
      endfor
      nx = out +nxline+nyline

      if cline(0) gt 0 then  begin
        if cabscissa eq 2 then xlab = ylab else if cabscissa eq 3 then $
          xlab = 'Total Distance'
        save_conductor_line, xlab,flab,a,cnval,cval, cloc,ncloc,nx,  $
                cxarr,cyarr,cline,cabscissa
      endif
    if quiet ne 1 then begin
      if n_elements(pline) eq 0 then pline = 0
       if pline ne 0 then widsav2 = widsav*0.5
      if pline ne 0 and nxline gt 0 then begin
        for i = 0, nxline-1 do oplot, !x.crange, [1,1]*xline(i), lin = 2
        if widsav gt 0.0 then begin
        for i = 0, nxline-1 do oplot, !x.crange, [1,1]*xline(i)+widsav2, lin=4
        for i = 0, nxline-1 do oplot, !x.crange, [1,1]*xline(i)-widsav2, lin=4
        endif
      endif
      if pline ne 0 and nyline gt 0 then begin
        for i = 0, nyline-1 do oplot, [1,1]*yline(i), !y.crange, lin = 2
        if widsav gt 0.0 then begin
        for i = 0, nyline-1 do oplot, [1,1]*yline(i)-widsav2, !y.crange, lin=4
        for i = 0, nyline-1 do oplot, [1,1]*yline(i)+widsav2, !y.crange, lin=4
        endif
      endif
      mkcircle
      if pline ne 0 and cline(0) gt 0 then begin
        for i = 0, ncloc -1 do $
          oplot, cxarr(cloc(i):cloc(i+1)-1), cyarr(cloc(i):cloc(i+1)-1), $
          psym = -8, lin = 2, col = 7
      endif
      lctwodim = a.two
      if keyword_set(conductor)  then begin
        plotcon, conductor, /over
      endif
      if n_elements (grid) gt 0 then begin
        plotgrd, grid, /over
      endif
    endif
;zzdebug
;zzt(9) = systime(1)
;zztc(9) = 'Finished '

;zzdt = [0,zzt(1:9)-zzt(0:8)]
;for i = 0,9 do print, i, zzt(i)- zzt(0), zzdt(i), '  ', zztc(i)
;
      levels = levelsave
      color = colorsave
      cont = 0
      a = 0
      farray = 0
      if (keyword_set(thermometer) and (thermometer eq -1)) then thermometer = 0
      if keyword_set(thermometer) and quiet ne 1 then $
        thermometer, therm_amp, therm_col,ovamp = ovamp, $
          ovcol = ovcol, _extra = tparam
      if wnew ge 0 then begin
        wdelete, wnew
        if wsave ge 0 then begin
          wset, wsave
          !x = xsave
          !y = ysave
          !z = zsave
          !p = psave
        endif
      endif

      return
      end
