; ------------------------------------------------------------------------
; $Id: plotsurf.pro,v 1.1 2010/03/03 17:00:49 lpmix Exp $
; ------------------------------------------------------------------------
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
;+
; NAME:
;       PLOTSURF
; PURPOSE:
;       This procedure displays one or more surface structures using XOBJVIEW
;       A model is generated and may be combined with other models.
;       The model is stored for future reference and can be retrieved
;
;       See also SURF2EXO to visualize surfaces with PARAVIEW or Ensight
;
; CATEGORY:
;       Solid modeling
;
; CALLING SEQUENCE:
;       PLOTSURF, STRUCTURE [, FIELD]
;
; INPUTS:
;       STRUCTURE: A surface or surface topology structure or a structure array number
;                  or an array of these structures.
;       FIELD:     Field to be plotted.  This is ignored for topology datasets and for 
;                  surface field structures with a scalar value.
;                  Vector fields require this parameter.
;                  If FIELD is an array, the square root of the sum of
;                  the squares will be used.
;            Note: If FIELD =[1] the the absolute value is plotted.
;                  That is, if FIELD is a single value array, ABS will be applied.
;                  If FLD is a single value larger than the number of vector
;                  components, then FLD = 1+indgen(nvect)
;                  where nvect is the number of vector components
;                  
; KEYWORDS PARAMETERS:
;       AXIS_ADD: If set, an axis will be added to the 3D model
;                 If AXIS_ADD is a structure, it will be
;                  passed to ADD_AXIS as an _EXTRA keyword.
;       BLOCK:    If set, the solid model is displayed in a blocking widget
;                 If undefined, then BLOCK defined by the following:
;                       DEFSYSV, "!callable_idl", exists = BLOCK
;       XRANGE:   X-Range for the nodes.  This uses GET_SURF_NODES
;       YRANGE:   Y-Range for the nodes.  This uses GET_SURF_NODES
;       ZRANGE:   Z-range for the nodes.  This uses GET_SURF_NODES
;       NODES:    List of nodes to include.   This is ignored if XRange or YRange
;                 or ZRange have 2 or more elements.
;                 If missing, all nodes are used unless IGNORE is valid.
;                 These nodes can be obtained from GET_SURF_NODES in the PFF area.
;                 XRANGE, YRANGE, and ZRANGE may also be used.
;                 NODES are zero based. (Range is zero to str.nnodes-1)
;       IGNORE:   List of nodes to not use. This is ignored if nodes has 4 or more
;                 values or the XRANGE or YRange or ZRange are valid.
;                 IGNORE values are zero based. (Range is zero to str.nnodes-1)
;       RETRIEVE: If set, no new model will be generated but the last model will be
;                 returned in MODEL
;       SHININESS:Shininess property in IDLgrPolygon
;                 Default: Undefined
;       SHADING:  Shading value
;                 1 is Gouraud, 0 is flat
;                 Default: SHADING =1
;       **********
;       These  keywords are used for surfact surface datasets
;       LEVELS:   Minimimum and maximum levels for the model
;                 The colors between 10 and 254 are linearly scaled to the amplitude
;                 These values apply to all fields in the model
;                 DEFAULT:  LEVELS = [ MIN(FIELD, max = MM), mm]
;       **********
;       These  keywords are used for surfact topology datasets
;       COLOR:    Array of colors for the objects
;                 These will be cycled through for the topology structures
;                 Default: color = indgen(7)+1
;                 Note: Actual color is modified by F/B INTENSITY
;       FINTENSITY: Foreground color intensity for each object
;            Note:  FINTENSITY may be a scalar or an RGB triplet
;                   If a scalar, COLOR = FINTENSITY*COLOR
;                   Default:  FINTENSITY = 1.0
;       BINTENSITY: Background color intensity for each object
;                   If a scalar, BOTTOM = BINTENSITY*COLOR
;                   Default:  BINTENSITY = 0.6
;                   
;
; TECHNIQUE:
;       Solid model is generated and returned in the MODEL keyword.
;       Two models will be stored in a common block
;       After two models are stored, additional calls to PLOTSURF will result
;       in the destruction of the first solid model, the second model will
;       be moved to the first model location and the current model will be
;       stored.
;       The current model can be retrieved with the RETRIEVE keyword.
;
; Examples:
;       Make a solid model of structure arrays 3, 4, 6.  Scale the field between
;       300 and 2000 for these temperature structures.
;         PLOTSURF, [3, 4, 6], LEVELS =[300, 2000]
;
;       Make a solid model with structure array 3 and scale the amplitude of the
;       x, y, and z electric field components between 0 and 1.0E6
;         
;         PLOTSURF, 3, 4, LEVELS =[0, 1000000]
;    or   PLOTSURF, 3, [1,2,3], LEVELS =[0, 1000000]
;
;       Display the electric and magnetic field amplitudes in a widget
;       Assume that they are in structure arrays 3 and 4
;         omodel =  obj_new('idlgrmodel')
;         PLOTSURF, 3, 4, MODEL = modele
;         omodel -> add, modele
;         PLOTSURF, 4, 4, MODEL = modelb
;         omodel -> add, modelb
;         XOBJVIEW, OMODEL
;
;
; MODIFICATION HISTORY:
;       Original version, L. P. Mix, February 25, 2010
;       Based on routine SOLID.
;
;-
;
pro plotsurf, structure, Field, color=color, fintensity=fintensity, $
           bintensity=bintensity, trans=trans, axis_add = axis_add, $
           model = model, retrieve = retrieve, block = block, nodes = nodes, $
           ignore = ignore, xrange = xrange, yrange = yrange, zrange = zrange, $
           levels = levels
common Surface_model_common, last_model, current_model
on_error, 1
if keyword_set(retrieve) then begin
  if obj_valid(current_model) then begin
    if arg_present(model) then model = current_model else begin
      print,'% PLOTSURF: Keyword(RETRIEVE) MODEL must be a variable'
      help, model
    endelse
  endif else print, '% PLOTSURF: Keyword(RETRIEVE) No model to return'
  return
endif
if (n_params() EQ 0) then begin
  print,  '% PLOTSURF: Structure must be specified'
  return
endif
nels = n_elements(structure)
if nels eq 0 then begin
  print,  '% PLOTSURF: Structure(s) must be defined'
  return
endif
lcstr = 0
if nels eq 1 then begin
  str = structure[0]
  if size(/type, str) eq 8 then begin
    if ((str.type lt 14) || (str.type gt 15)) then begin
      print,  '% PLOTSURF: Only structures with surface parameters are allowed.'
      print,  '            (Structure types 14 or 15)'
      return
    endif
    lcstr = 1
  endif
endif
if lcstr eq 0 then begin
  str = strvalid(structure, /valid, np = np)
  if np ne nels then begin
    print, '% PLOTSURF: Some structures are not valid'
    print, ' Structures input: ', structures
    print, ' Valid Structures: ', str
    return
  endif
endif

; color
; Set conductor colors: Default is to just cycle through 1 - 7
; Do not allow ncol eq 3
nelc = n_elements(color)
fcolor =  (nelc eq 0) ? 1+indgen(7) : color
ncol = n_elements(fcolor)
if ncol eq 3 then begin
  ncol += ncol
  fcolor = [fcolor, fcolor]
endif
mincolor = 10

; Front intensity

if n_elements(fintensity) eq 0 then begin
  fintens = 1.0
endif else if n_elements(fintensity) eq 3 then begin
  fintens = 1.0
  fcolor = fintensity
endif else fintens = fintensity [0]
 
; Back intensity
bintens = 0.6

nmodel = 0
omodel= obj_new('idlgrmodel')
tvlct, /get, red, green, blue

for istr = 0, nels-1 do begin
  if lcstr eq 1 then ss = str else s2i, ss, str[istr]
  lcgood =  ((ss.type lt 14) || (ss.type gt 15)) ? 0 : 1
  if lcgood then begin
;   get the spatial information
    lcfinish = 1
    xyz = ss.xyz
    lcnodelist = 0
    nodelist = -1
    if (n_elements(xrange) gt 1) || (n_elements(xrange) gt 1) $
      || (n_elements(xrange) gt 1) then begin
      get_surf_nodes, nodelist, xyz, xrange = xrange,  yrange = yrange, $
             zrange = zrange, nelements = nelnod
      if nelnod eq 0 then lcfinish = 0
      if ss.nnodes gt nelnod then lcnodelist = 1
    endif else if n_elements(nodes) gt 3 then begin
      ok = where (nodes ge 0 and nodes lt ss.nnodes, nok)
      if nok gt 4 and nok lt ss.nnodes then begin
        nodelist = temporary(ok)
        lcnodelist = 1
      endif
    endif else if n_elements(ignore) gt 0 then begin
      ok = where (ignore ge 0 and ignore lt ss.nnodes, nok)
      if nok gt 0 and nok lt n_elements(ignore) then ignore = ignore[ok]
      if nok gt 0 then begin
        t = lonarr(ss.nnodes)
        t[ok] = -1
        nodelist = where(t eq 0)
        lcnodelist = 1
      endif
    endif
    if lcnodelist && lcfinish then begin
      t = lonarr(ss.nnodes)
      t[nodelist] = 1
      conn = make_array(4, ss.nplane)
      val = t[conn]
      ok = where (max(val, dim=1) gt 0, nok)
      if nok eq 0 then begin
        print, '% PLOTSURF:  Unable to plot selected nodes'
        lcfinish = 0
      endif else begin
        conn = lonarr(5, nok)
        conn[0, *] = 4
        conn[1:4, *]=(ss.connectivity-1)[*, ok]
        if ss.type eq 15 then v1 = ss.v1[ok]
      endelse
    endif else begin
      conn = lonarr(5, ss.nplane)
      conn[0, *] = 4
      conn[1:4, *]= ss.connectivity-1
      if ss.type eq 15 then v1 = ss.v1
    endelse

    if lcfinish then begin
      if ss.spare[0] eq 2 then begin
        ctheta = reform(cos(xyz[1, *]))
        stheta = reform(sin(xyz[1, *]))
        xyz[1, *] = xyz[0, *]*stheta
        xyz[0, *] = xyz[0, *]*ctheta
        ctheta =0
        stheta =0
      endif
      if ss.type eq 15 then begin
        minv1 = min(v1, max = m)
        maxv1 = m
        print, 'Range of the field is [' +strtrim(minv1, 2)+', '+strtrim(maxv1, 2)+']'
        if n_elements(levels) gt 1 then levels =[min(levels, max= m), m] $
           else levels = [minv1, maxv1]
        vv = bytscl(v1, min = levels[0], max = levels[1],  top = 255-mincolor) $
          +byte(mincolor)
        if n_elements(opal) eq 0 then opal = obj_new ('IDLgrPalette', red, green, blue)
        oData = obj_new('idlgrpolygon', xyz, polygons = conn, $
          vert_colors = vv, shading =1, palette = opal, shininess = 30)
      endif else begin
        icolor = (ncol eq 3) ?  fcolor : getcolor(fcolor[istr mod ncol])
               
        oData = obj_new('IDLgrPolygon', xyz, polygons= conn, $
          thick = 1.0, name = ss.clab, color = icolor*fintens, bottom = bintens*icolor, $
          shininess = shininess, shading = 1, style = style)
      endelse    
      omodel ->add , oData
      nmodel += 1
    endif   
  endif
endfor
if nmodel eq 0 then begin
  Print, '% PLOTSURF: No conductors added to model'
  obj_destroy, omodel
  return
endif
if keyword_set(axis_add) then $
  omodel=  (size(axis_add, /type) eq 8) ? $
    add_axis(omodel, _extra= axis_add) : add_axis(omodel)
if arg_present(model) then model= omodel
if n_elements(block) eq 0 then $
          defsysv, "!callable_idl", exists = block
if obj_valid(omodel) then xobjview, omodel, block = block
if obj_valid(last_model) then obj_destroy, last_model
if n_elements(current_model) gt 0 then last_model = temporary(current_model)
current_model= temporary(omodel)
end
