; ------------------------------------------------------------------------ 
; $Id: readraw.pro,v 1.3 2007/06/06 20:09:36 lpmix Exp $
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
 Pro READRAW, fname, str,   helpme = helpme, clabel = clabel, $
       XSIZE = xsize, YSIZE = ysize, integer = int_arr
;
;+
;    Read an image array from a RAW image file
;    Image is stored as two-byte binary integers
;
;    format is:  READRAW [, FNAME] [, STR]  [, /HELPME] 
;                        [, CLabel = clabel] [ /INTEGER]
;                        [,  XSIZE = xsize [, YSIZE = ysize]
;    where 
;      fname   - the file name
;                If fname = 0 or undefined then dialog_pickfile() will 
;                be used to select a file
;      STR     - STR array or STR array number
;                If STR is not provided, then an unused structure array
;                will be used.
;      HELPME  - If set, a help statement will be printed.
;      CLABEL  - If set used as the label for the structure.
;                Default:  'Raw Image - ' + fname
;      INTEGER - If set, save the image as an integer array
;      XSIZE   - Number of pixels in the horizontal direction
;                Default: XSIZE = 2048
;      YSIZE   - Number of pixels in the vertical direction
;                Default: YSIZE = 2000
; PROCEDURE:
;      Open a file and read the data as an integer.  Swap the byte order
;      if working on a little endian computer.
;
;    Example:
;    Read STR array 5 from file 'PFIDLdefault.dat'
;       READRAW, 'PFIDLdefault.dat' , 5
;
;    Resd STR 3 from a file to be selected from Dialog_Pickfile()
;       READRAW , 0, 3
;
; MODIFICATION HISTORY:
;      Original version, L. P. Mix, October 2, 2006
;-
;
on_error, 1
@qscom.dat
;  Common block
@qsstrcom.dat
;
      if n_elements (structinit) eq 0 then qsstruct_init
      if n_elements (qsplotinit) eq 0 then qsinit
;
      nparams = n_params()
      if keyword_set(helpme)   then begin
        print, '% Format: READRAW, Filename, Structure'
        return
      endif
      if n_elements (fname) eq 0 then fname = 0
      b = size(fname)
      if b(b(0)+1) ne 7 then begin
        str = fname
        fname = dialog_pickfile(/must_exist, /read)
        if strlen(fname) lt 1 then begin
            print, '% READRAW: No File was Selected - RETURNING'
            return
        endif
        resolve_it, fname
      endif else begin
        fname = fname(0)
        resolve_it, fname
        info = file_info(fname)
        if (info.exists + info.read) ne 2b then begin
           print, '% READRAW: File does not exist with read permission'
           print, '         File Name = ', fname
           print, '         File Exists =', info.exists
           print, '         Read Permission =', info.read
           return
        endif
      endelse
      openr,unit,fname, /get_lun, error = error, /swap_if_little_endian
      if error ne 0 then begin
        print, '% READRAW:  Error opening file: ', fname
        print, !error_state.msg
        return 
      endif

      str = get_save_str(str, maxstructure)
      
      info = file_info(fname)
      if n_elements(xsize) eq 0 then xsize = 2048L
      if n_elements(ysize) eq 0 then ysize = 2000L
      if info.size lt 2L*xsize*ysize then begin
        print, '% READRAW: File size is too small to use with values of Xsize, Ysize'
        print, '  File Name = ', fname
        print, '  File Size = ', strtrim (info.size, 2)
        print, '  Xsize, Ysize, Image size: ', xsize, ysize, 2L*xsize*ysize
        return
      endif
      image = make_array(xsize, ysize, /integer)
      readu, unit, image
      free_lun, unit
      clabel = (N_elements(clabel) gt 0) ? strtrim(clabel, 2): 'Raw Image - '+fname
;      help, str, image, clabel
      if keyword_set(int_arr) then arr2str, str, image, clabel = clabel $
        else  arr2str, str, float(image), clabel = clabel 
      end
