; ------------------------------------------------------------------------ 
; $Id: readstr.pro,v 1.7 2010/05/26 13:48:11 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
 Pro readstr, str, datasetin,ndset, field= field, particle = particle, $
     conductor = conductor, grid = grid, fileid = fidin, structure=struct, $
     stringlen= stringlen, no_copy = no_copy, clabel = clabel_in, $
     _extra = extrastuff
;+
;
;  Reads a structure dataset into a QS structure array 
;  Procedure options are explained below. 
;  This procedure calls:
;     READCON, READFLD, READPAR, READGRD, READSURF, READNG3, READNGD
;
;  Format: READSTR, STR, [, dataset][, NDSET] [, FILEID = fileid] 
;             [, STRINGLEN = stringlen] [, CLABEL = clabel]
;             [, GRID = GRID] [, PARTICLE = PARTICLE] 
;             [, CONDUCTOR = CONDUCTOR] [, FIELD = FIELD] 
;             [, STRUCTURE = STRUCTURE] [, NO_COPY = no_copy]
;  where:
;    STR     - QS structure array number
;           or an undefined variable - STR will be set to the appropriate
;               structure array number -DELVAR will make a variable undefined
;    dataset - indicates a PFF dataset number 
;              or is a search string for dataset comments.  
;              If dataset is missing or zero, then it is set to the 
;              current dataset.
;     ndset  - used for conductor data to read more than 1 dataset
;        Note: If dataset is a string, a leading `^' indicates that 
;              the string begins at the start of the dataset comment.
;    fileid -  file id of the PFF file containing the data
;    Clabel -  If CLABEL has a length gt 0, it will be used as the COMMENT label
;              Note: CLABEL = ' ' will result in a blank value for clabel
;                    CLABEL = '' will result in the default value
;    Particle - Indicates a particle dataset
;               If the Application dataset type is 20, 21, 23 or 24 
;                 particle will be set
;    Conductor - Indicates a conductor dataset
;                If the Application dataset type is 2, conductor will be set
;    Grid     - Indicates a grid dataset
;               If the Application dataset type is 5 or if PFF dataset type 
;               is NG3 (8), grid will be set
;    Field    - Indicates a field dataset
;               If the Application dataset type is 11, 12, 13, or 14,
;               Field will be set.
;               Field is the DEFAULT.
;    Structure - Structure stored in QS array, STR 
;                Must set no_copy to 0 or -1.
;    No_copy   - If keyword is set, structure is returned as undefined
;                Default value = 1
;                If keyword is equal to -1 then only structure is returned
;  Examples:
;    Read field data in dataset 34 in file id 4 into QS structure array, 5
;      readstr, 5, 34, fil=4
;    Read next dataset of the current file into QS structure array 6. Assume 
;    dataset is particle data.
;      readstr, 6, /p (or if QS or TQ data) readstr, 6
;    Read next dataset of the current file into QS structure array 7. Assume 
;    dataset is particle data and return the sturcture in PAR.
;      readstr,7,/p,structure=par (or if QS or TQ data) readstr,7,structure= par
;    Read conductor datasets 2, 3, 4, 5 into QS structure array 7.
;      readstr, 7, /con, 2, 4 (or if QS or TQ data) readstr, 7, 2, 4
;-
;
;  Common block
@qsstrcom.dat
;
      on_error, 1
;
      if n_elements (structinit) eq 0 then qsstruct_init
;
      npar = n_params()
      if npar lt 1 then begin
 print, '% Format: READSTR, STR, [, dataset][, NDSET] [, FILEID = fileid]'
 print, '    [, GRID = GRID] [, PARTICLE = PARTICLE] [, FIELD = FIELD]'
 print, '    [, CONDUCTOR = conductor] [, STRUCTURE = STRUCTURE]'
 print, '    [, STRINGLEN = stringlen]'
        return 
      endif
      if (npar lt 2 ) then datasetin = 0
      if n_elements (datasetin) eq 0 then dataset = 0 else dataset = datasetin
      if n_elements (fid) eq 0 then fid = 0
      if n_elements (str) eq 0 then begin
        i = 1
        str = 0
        while i gt 0  and i lt maxstructure do begin
          if lcvalidstr(i-1) eq 0  then begin
            str = i
            i = -1
          endif
          i = i+1
        endwhile
        if str eq 0 then begin
          print, 'All structure arrays contain data;', $
            ' specify a value of STR to overwrite existing data'
          return
        endif
      endif
      if str lt 1 or str gt maxstructure then begin
        print, 'QS structure array number must be greater than zero'
        print, '  and less than or equal to', maxstructure 
        return
      endif
      dsetnumber= dataset(0)
;     don't modify dataset but for all others we use dsetnumber to read
;     conductors must allow for multiple values of dataset
      if n_elements (fidin) gt 0 then fid = long(fidin[0]) else fid= 0L
      ierror = 0
      if (fid ne 0) then setpff, fid, ierror
      if ierror ne 0 then begin
        print,  '% READSTR: Unable to set the File ID to FID'
        help, fid
        return
      endif
      readhdr,dsetnumber,ier,aptype=aptyp, pfftyp = pfftyp, $
        dlength = dlength, clabel=clabel, namefile=file, tlabel = tlabel, $
        iextra=iextra, /quiet
      if ier ne 0 then begin
        print,'% READSTR: Error reading PFF header'
        return
      endif

    if pfftyp eq 6 then  begin
      if ((aptyp ge 200) && (aptyp le 210)) then $
        readsurf, /structure, struct, dsetnumber,  aptype=aptyp, pfftyp = pfftyp,$
        dlength = dlength,iextra=iextra, /skip,  _extra = extrastuff else $
      readifl, /structure,struct, dsetnumber
    endif else begin
      if not keyword_set(particle) and not keyword_set(conductor) and $
        not  keyword_set(grid) and not  keyword_set(field) then begin
        if pfftyp eq 7 or pfftyp eq 9 then field = 1 else $
          if aptyp eq 2 then conductor = 1 else $
           if aptyp eq 4 then conductor = 1 else $
            if ((pfftyp eq 8) or (aptyp eq 5)) then grid = 1 else $
              if aptyp ge 11 and aptyp le 19 then field = 1 else $
                if aptyp ge 20 and aptyp le 29 then particle = 1 else $
                  field = 1
      endif
      
      if keyword_set(particle) then $
         readpar, struct, dsetnumber, dlength=dlength, $
         iextra=iextra, clabel = clabel, tlabel = tlabel, pfftyp = pfftyp, $
         namefile = file, stringlen= stringlen, /skip , _extra = extrastuff $
      else if keyword_set(conductor) then $
         readcon,struct,dataset,ndset, _extra = extrastuff $
      else if keyword_set(grid) then begin
         if pfftyp eq 8 then $
           readng3, struct, dsetnumber, dlength=dlength, $
             iextra=iextra, clabel = clabel, tlabel = tlabel, pfftyp = pfftyp, $
             namefile = file, stringlen= stringlen, /skip else  $
           readgrd,struct,dsetnumber, _extra = extrastuff 
      endif else readfld,struct,dsetnumber, _extra = extrastuff
    endelse
    if size(/type, struct)  eq 8 then begin
      if n_elements (no_copy) eq 0 then no_copy = 1
      if n_elements (clabel_in) gt 0 && strlen(clabel_in) gt 0 then $
        struct.clab = strtrim(clabel_in, 2)
      if no_copy eq -1 then return
      i2s, struct, str, no_copy = no_copy 
    endif else print, '% READSTR: Error reading dataset, no structure saved'
    end
