; ------------------------------------------------------------------------
; $Id: rebinstr.pro,v 1.2 2010/11/18 15:42:06 lpmix Exp $
; ------------------------------------------------------------------------
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
;+
; NAME:
;       REBINSTR
;
; PURPOSE:
;       Change the density of points in a 2-dimensional single block structure
;       3 dimensional structures are ok if they are degenerate in the third dimension.
;
; CATEGORY:
;       Analysis, Graphics
;
; CALLING SEQUENCE:
;
;       REBINSTR, STRUCTURE [, SAVE] [, DELTA = delta] [, NPOINT= npoint]
;          [, SMOOTH = smooth] [, CUBIC = cubic], [, XLABEL = xlabel]
;          [, YLABEL = ylabel] [, CLABEL = clabel] 
;
; INPUTS:
;       STRUCTURE: Structure or Structure array number.  This must be a two
;               dimensional, single block structure or a three dimensional 
;               structure with a degenerate 3rd dimension.
;               The coordinates must be either increasing or decreasing.
;
; OPTIONAL INPUTS:
;       SAVE:	This is the storage location for the saved structure.
;		If save is missing or undefined, then SAVE = STRUCTURE.
;               If STRUCTURE is a structure, then SAVE is a structure,
;               but if STRUCTURE is an array number, then save is an array
;               number.  In all cases, the rebinned structure is stored in
;               (get_maxstr() - 1)
;	
; KEYWORD PARAMETERS:
;   NOTE: Either DELTA or NPOINT is mandatory.  If NPOINT is present DELTA is ignored.
;       NPOINT: Number of points.  The range of the data is exactly the same as the 
;               original data.  DELTA = <Original Range> / (NPOINT -1)
;               If N_ELEMENTS(NPOINT) eq 2 then NX = NPOINT[0] & NY = NPOINT[1]
;               If N_ELEMENTS(NPOINT) ne 2 then NX = NY = NPOINT[0].
;                 where NX and NY are the number of points in each dimension.
;       DELTA:	Spacing for the rebin.  If delta is a scalar or has three or more elements,
;               the structure will have equal spacing in both directions equal to DELTA[0].
;               If DELTA has two values they will be taken as DX and DY.  The range of the
;               data will generally be reduced to start at the original location and stop
;               after an integer number of DELTAs.
;       SMOOTH: Bin width for the SMOOTH function.  This may be a scalar or a two
;               element vector.  The smooth keyword is sent directly to the smooth
;               function.  If smooth is undefined then the default values are
;               determined by the change in the number of points.  If the number
;               of points in each direction decreases by less than 1.5, no smoothing
;               will be done.  The default value of SMOOTH is calculated as follows:
;                 NXold, NYold are the original number of points in each direction.
;                 NXnew, NYnew are the new values of points in each direction.
;                 DEFAULT: SMOOTH = ROUND([NXold/NXnew, NYold/NYnew]) > 1
;               Note: SMOOTH = 1 implies no smoothing.
;               Note: If SMOOTH is even, SMOOTH = SMOOTH+1
;       XLABEL: New X-Label for the rebined structure
;       YLABEL: New Y-Label for the rebined structure
;       CLABEL: New Comment Label for the rebined structure
;               Default: Append - "REBIN_DELTA=[DX,DY]" or "REBIN_NPOINT=[NX,NY]"
;                        Where appropriate values are substituted for DX, DY, NX, NY.
;
;   NOTE:  The following keyword is not used directly but passed to INTERPOLATE.
;       CUBIC:  Use CUBIC interpolation.  See IDL help for INTERPOLATE.
;		Value between -1 and 0; -0.5 recommended in the help file for 
;               INTERPOLATE.
;
; OUTPUTS:
;       An interpolated image is returned.
;
; SIDE EFFECTS:
;       If SAVE is not provided, the original data is replaced by the rebinned data.
;       GET_MAXSTR-1 contains the rebinned structure
;
; PROCEDURE:
;       The new grid is determined and INTERPOLATE used to calculate a new image.
;
; EXAMPLE:
;       Rebin structure 5 to have 1400 x 1200 pixels (X x Y) and save the result in 15
;         REBINSTR, 5, 15, NPOINT =[1400, 1200]
;
;       Rebin structure S and store it in T with 20 micron square pixels.
;         REBINSTR, S, T, DELTA = 20e-6       
;
; MODIFICATION HISTORY:
; 	Written by:	L. P. Mix, Jr., September 27, 2010
;
;-

PRO rebinstr, strarray, saveit, DELTA = delta, npoint = npoint, smooth = smo, $
    cubic = cubic, xlabel = xlabel, ylabel = ylabel, clabel = clabel

on_error, 1

; Insert common blocks
;
@qscom.dat
@qsstrcom.dat
;
if n_elements (structinit) eq 0 then qsstruct_init
if n_elements (qsplotinit) eq 0 then qsinit

npar = n_params()
if npar eq 0 then begin
  print, '% Format:  REBINSTR, STRUCT [, SAVEIT] [, DELTA = delta]'
  print, '           [, NPOINT = npoint] [, SMOOTH = smo] [, CUBIC = cubic]'
  print, '           [, XLABEL = xlabel] [, YLABEL = ylabel] [, clabel = clabel]'
  return
endif

if n_elements(strarray) eq 0 then begin
  print, '% REBINSTR:  Structure array must be defined'
  return
endif
a = get_valid_str (strarray, 'REBINSTR', lcstr=lcstr, ierror=ier)
if ier ne 0 then return
if (a.type ne 1) then begin
  print, '% REBINSTR:  Type 1 field structures are required'
  a =0
  return
endif

if a.block ne 1 then begin
  print, '% REBINSTR:  Single block field structures are required'
  print, '               Number of blocks: '+strtrim (a.block, 2)
  a =0
  return
endif

two = a.two
ndim = a.ndim
if ndim gt 2 and a.two ne 3 then begin
  print, '% REBINSTR:  Only Two dimensional structures are permitted.'
  print, '             3D structures must be degenerate in the third dimension.'
  help, ndim, two
  a =0
  return
endif

v1 = get_array(a, -1)
x = get_array(a, 1)
y = get_array(a, 2)
if size(v1, /type) lt 4 then v1 = float(v1)
if size(x, /type) lt 4 then x = float(x)
if size(y, /type) lt 4 then y = float(y)
nx = n_elements(x)
ny = n_elements(y)
; be sure that they are sorted
dx = x[1:nx-1]-x[0:nx-2]
dy = y[1:ny-1]-x[0:ny-2]
; Calculate the range of dx, dy
dxr = [min(dx, max = m), m]
dyr = [min(dy, max = m), m]

if (dxr[1] gt 0 and dxr[0] ge 0) || (dxr[0] lt 0 and dxr[1] le 0) then begin
; axis is either increasing or decreasing.
  if dxr[0] lt 0 then begin
    x = reverse(x)
    v1 = reverse(v1)
  endif
endif else begin
  print, '% REBINSTR: X-coordinate must be either increasing or decreasing.'
  print, 'DX range: '+strtrim (dxr[0], 2)+' to '+strtrim (dxr[1], 2)
  a=0 & v1 = 0
  return
endelse

if (dyr[1] gt 0 and dyr[0] ge 0) || (dyr[0] lt 0 and dyr[1] le 0) then begin
; ayis is either increasing or decreasing.
  if dyr[0] lt 0 then begin
    y = reverse(y)
    v1 = reverse(v1, 2)
  endif
endif else begin
  print, '% REBINSTR: Y-coordinate must be either increasing or decreasing.'
  print, 'DX range: '+strtrim (dyr[0], 2)+' to '+strtrim (dyr[1], 2)
  a=0 & v1 = 0
  return
endelse

nel =  n_elements(npoint)
range = [x[nx-1]-x[0], y[ny-1]-y[0]]
if nel gt 0 then begin
  np = (nel eq 2) ? round(npoint) : [npoint[0], npoint[0]]
  delt =  range / (np -1)
  cadd = (strpos(a.clab, ' - ') gt 0) ? $
    ';;REBIN_NPOINT=['+strtrim(np[0], 2) +','+strtrim(np[1],2)+']' : $
    ' - REBIN_NPOINT=['+strtrim(np[0], 2) +','+strtrim(np[1],2)+']'
endif else begin
  nel = n_elements(delta)
  if nel eq 0 then begin
    print, '% REBINSTR: DELTA must be defined if NPOINT is not defined'
    help, delta, npoint
    a=0 & v1 = 0
    return
  endif
  delt = (nel eq 2) ? delta : [delta[0], delta[0]]
  np = floor (range/delt) +1L
  cadd = (strpos(a.clab, ' - ') gt 0) ? $
    ';;REBIN_DELTA=['+strtrim(delt[0], 2) +','+strtrim(delt[1],2)+']' : $
    ' - REBIN_DELTA=['+strtrim(delt[0], 2) +','+strtrim(delt[1],2)+']'
endelse
xnew = findgen(np[0])*delt[0]
ynew = findgen(np[1])*delt[1]  
nsmo = n_elements(smo)
if nsmo eq 0 then smo =  [float(NX)/np[0], float(NY)/np[1]]  else $
  smo = (nsmo eq 1) ? [smo, smo] : smo[0:1]
smo = round(smo) > 1

if max(smo) gt 1 then v1 = smooth(v1, smo)

print, smo, np, nx, ny

vnew = interpolate(v1, findex(x, xnew), findex(y, ynew),  cubic = cubic, /grid)
xlab = (n_elements(xlabel) gt 0) ? strtrim(xlabel[0],2) : a.x1lab
ylab = (n_elements(ylabel) gt 0) ? strtrim(ylabel[0],2) : a.x2lab
clab = (n_elements(clabel) gt 0) ? strtrim(clabel[0],2) : a.clab + cadd
arr2str, strnew, vnew, xnew, ynew, dx = delt[0], dy = delt[1], clabel = clab, $
    xlabel = xlab, ylabel = ylab, v1label = a.v1lab, spare = a.spare, /structure

i2s, strnew, get_maxstr()-1
if lcstr eq 1 then begin
  if (arg_present(saveit)) then saveit= temporary(strnew)  else $
    strarray = temporary(strnew)
endif else begin
  
  If n_elements (saveit) gt 0 then saveit =  $
   (saveit[0] gt 0 and saveit[0] le get_maxstr()) ? saveit[0] : strarray $
   else  saveit = strarray
   i2s, strnew, saveit, /no_copy
endelse
a=0 & v1 = 0 & vnew = 0
END
