; ------------------------------------------------------------------------ 
; $Id: remsd.pro,v 1.6 2011/06/17 18:26:47 lpmix Exp $
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
 Pro REMSD, fname, str,   helpme = helpme, Vector_label = vector_label
;
;+
;    Read an image array from an ascii file
;    This procedure limited to single block, single vector field data
;    WRIMSD will write this ascii format.
;
;    format is:  REMSD [, fname] [, STR]  [, /HELPME] 
;                        [, Vector_label = vector_label]
;    where 
;      fname   - the file name
;                If fname = 0 or undefined then dialog_pickfile() will 
;                be used to select a file
;      STR     - STR array or STR array number
;                If STR is not provided, then an unused structure array
;                will be used.
;      HELPME  - If set, a help statement will be printed.
;      Vector_label - If set used as an V1Label in the structure.
;
;    Example:
;    Read STR array 5 from file 'PFIDLdefault.dat'
;       REMSD, 'PFIDLdefault.dat' , 5
;
;    Resd STR 3 from a file to be selected from Dialog_Pickfile()
;       REMSD , 0, 3
;
;    form of file:
;      dataset comment
;      ndim, lcuniform 
;      x axis label 
;      y axis label
;      z axis label
;      nx, ny, nz
;      x0, y0, z0 |
;      dx, dy, dz |  or x, y, z
;      field data
;
;  where:
;      ndim       = 2 or 3
;      lcuniform  = 1 if uniform, 0 if not uniform
;  X is uniform if max(dx)-min(dx) is less than 1.22e-4 *max(abs(x))
;      axis label = string for axis labels
;                   Z label is omitted if ndim = 2
;      nx, ny, nz = number of elements in each dimension
;                   (NZ omitted if ndim = 2)
;      x0, y0, z0 = initial axis values
;      dx, dy, dz = axis step size
;      x, y, z    = axis values (lcuniform = 0)
;                   dz or Z omitted if ndim = 2
;      field data = image values
;
;   Fortran read:
;       integer lu, ndim, lcuniform, nx, ny, nz, i, j, k
;       real x(:), y(:), z(:)
;       real x0, y0, z0, dx, dy, dz 
;       real image( :, :, :)
;       character*80 xlabel, ylabel, zlabel, title
; c1     format (a)
;       open(file = , unit = lu)
;       read(lu, 1) title
;       read(lu, *) ndim, lcuniform 
;       read(lu, 1) xlabel
;       read(lu, 1) ylabel
;       nz = 1
;       if (ndim .eq. 3) then
;         read(lu, 1) zlabel
;         read(lu, *) nx, ny, nz
;       else
;         read(lu, *) nx, ny
;       endif 
; c      if fortran 90, allocate x, y, z, image
;       if (lcuniform .eq. 0) then
;         read(lu, *) (x(i), i = 1, nx)
;         read(lu, *) (y(i), i = 1, ny)
;         if (ndim .eq. 3) read(lu, *) (z(i), i = 1, nz)
;       else
;         if (ndim .eq. 2) then
;           read(lu, *) x0, y0
;           read(lu, *) dx, dy
;         else
;           read(lu, *), x0, y0, z0
;           read(lu, *) dx, dy, dz
;         endif
; c        can make x, y, z if desired x(i)= (i-1)*dx + x0, etc.
;      endif
;      read(lu, *)(((image(i, j, k),i=1,nx),j=1, ny),z=1,nz)
;      close (lu)
;-
;
on_error, 1
@qscom.dat
;  Common block
@qsstrcom.dat
;
      if n_elements (structinit) eq 0 then qsstruct_init
      if n_elements (qsplotinit) eq 0 then qsinit
;
      nparams = n_params()
      if keyword_set(helpme)   then begin
        print, '% Format: REMSD, Filename, Structure'
        return
      endif
      if n_elements (fname) eq 0 then fname = 0
      b = size(fname)
      if b(b(0)+1) ne 7 then begin
        fname = dialog_pickfile(/must_exist, /read)
        if strlen(fname) lt 1 then begin
            print, '% REMSD: No File was Selected - RETURNING'
            return
        endif
        resolve_it, fname
      endif else begin
        fname = fname(0)
        resolve_it, fname
        info = file_info(fname)
        if (info.exists + info.read) ne 2b then begin
           print, '% REMSD: File does not exist with read permission'
           print, '         File Name = ', fname
           print, '         File Exists =', info.exists
           print, '         Read Permission =', info.read
           return
        endif
      endelse
      openr,unit,fname, /get_lun, error = error
      if error ne 0 then begin
        print, '% REMSD:  Error opening file: ', fname
        print, !error_state.msg
        return 
      endif

      str = get_save_str(str, maxstructure)
      clab = ''
      
on_ioerror, end_read
      mode = 'Reading the comment'
      readf, unit, clab
      ndim = 0L
      lcuniform = 0L
      mode = 'Reading the dimensions and the uniformity'
      readf, unit, ndim, lcuniform
      if ndim lt 2 or ndim gt 3 then begin
        print, '% REMSD: Only two and three dimensional data can be read'
        free_lun, unit
        return
      endif
      xlabel = ''
      ylabel = ''
      zlabel = ''
      mode = 'Reading Axis Labels'
      readf, unit, xlabel
      readf, unit, ylabel
      if ndim eq 3 then readf, unit, zlabel
      np = make_array(ndim, /long)
      mode = 'Reading the number of points'
      readf, unit, np
      if lcuniform eq 0 then begin
        mode = 'Reading X-Axis Array'
        x = make_array(np[0], /float)
        readf, unit, x
        mode = 'Reading Y-Axis Array'
        y =  make_array(np[1], /float)
        readf, unit, y
        if ndim eq 3 then begin
          mode = 'Reading Z-Axis Array'
          z =  make_array(np[2], /float)
          readf, unit, z
        endif else z = 0.0
        dx = 0
        dy = 0
        dz = 0
      endif else begin
        mode = 'Reading Uniform Axis Initial Values'
        val = make_array(ndim)    
        readf, unit, val
        x0= val[0]
        y0= val[1]
        if ndim eq 3 then z0= val[2]
        mode = 'Reading Uniform Axis Delta Values'
        readf, unit, val
        dx= val[0]
        dy= val[1]
        if ndim eq 3 then dz= val[2]
        x = findgen(np[0])*dx+x0
        y = findgen(np[1])*dy+y0
        if ndim eq 3 then z = findgen(np[2])*dz+z0
      endelse
      if ndim lt 3 then array = make_array(np[0],np[1]) else $
        array = make_array(np[0],np[1],np[2])
      mode = 'Reading Vector Values'
      readf, unit, array
      if n_elements(Vector_label) gt 0 then v1lab = vector_label[0] $
        else  v1lab = 'Amplitude'
      arr2str, str, array, x, y, z, dx=dx, dy=dy, dz=dz, $
        file = fname, xlabel = xlabel, ylabel= ylabel, tlab ='MSD data', $
        clab = clab, zlabel = zlabel, v1label = v1lab
      free_lun, unit
      return
 end_read:
      print, '% REMSD: IO Error'
      print, mode
      free_lun, unit
      end
