; ------------------------------------------------------------------------ 
; $Id: remsd_ext.pro,v 1.3 2007/06/06 20:09:36 lpmix Exp $
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
 Pro REMSD_ext, fname, array, x, y, z, dx = dx, dy = dy, dz = dz, $
     clabel = comment, xlabel = xlabel, ylabel = ylabel, zlabel = zlabel
;
;+
;    Read an image array from an ascii file
;    This procedure uses only IDL procedures
;    This procedure limited to single block, single vector field data
;    WRIMSD will write this ascii format.
;
;    Format:  REMSD_ext , fname, ARRAY  [, X] [, Y] [, Z]
;             [, DX = dx] [, DY = dy] [, DZ = dz] [, CLABEL = clabel]
;             [, XLABEL = xlabel] [, YLABEL = ylabel] [, ZLABEL = zlabel] 
;    where 
;      fname   - the file name
;                If fname = 0 or undefined then dialog_pickfile() will 
;                be used to select a file
;      ARRAY   - Image array
;      X       - X values
;      Y       - Y values
;      Z       - Z values
;      DX      - If uniform data, the point spacing in the X direction
;      DY      - If uniform data, the point spacing in the Y direction
;      DZ      - If uniform data, the point spacing in the Z direction
;      CLABEL  - Comment label
;      XLABEL  - X Label
;      YLABEL  - Y Label
;      ZLABEL  - Z Label
;
;    Example:
;       Read only the image array from the file 'PFIDLdefault.dat'
;       REMSD_ext, 'PFIDLdefault.dat' , image
;
;    Read the image from from a file to be selected from Dialog_Pickfile()
;       REMSD_ext, 0, image
;
;    form of file:
;      dataset comment
;      ndim, lcuniform 
;      x axis label 
;      y axis label
;      z axis label
;      nx, ny, nz
;      x0, y0, z0 |
;      dx, dy, dz |  or x, y, z
;      field data
;
;  where:
;      ndim       = 2 or 3
;      lcuniform  = 1 if uniform, 0 if not uniform
;  X is uniform if max(dx)-min(dx) is less than 1.22e-4 *max(abs(x))
;      axis label = string for axis labels
;                   Z label is omitted if ndim = 2
;      nx, ny, nz = number of elements in each dimension
;                   (NZ omitted if ndim = 2)
;      x0, y0, z0 = initial axis values
;      dx, dy, dz = axis step size
;      x, y, z    = axis values (lcuniform = 0)
;                   dz or Z omitted if ndim = 2
;      field data = image values
;
;   Fortran read:
;       integer lu, ndim, lcuniform, nx, ny, nz, i, j, k
;       real x(:), y(:), z(:)
;       real x0, y0, z0, dx, dy, dz 
;       real image( :, :, :)
;       character*80 xlabel, ylabel, zlabel, title
; c1     format (a)
;       open(file = , unit = lu)
;       read(lu, 1) title
;       read(lu, *) ndim, lcuniform 
;       read(lu, 1) xlabel
;       read(lu, 1) ylabel
;       nz = 1
;       if (ndim .eq. 3) then
;         read(lu, 1) zlabel
;         read(lu, *) nx, ny, nz
;       else
;         read(lu, *) nx, ny
;       endif 
; c      if fortran 90, allocate x, y, z, image
;       if (lcuniform .eq. 0) then
;         read(lu, *) (x(i), i = 1, nx)
;         read(lu, *) (y(i), i = 1, ny)
;         if (ndim .eq. 3) read(lu, *) (z(i), i = 1, nz)
;       else
;         if (ndim .eq. 2) then
;           read(lu, *) x0, y0
;           read(lu, *) dx, dy
;         else
;           read(lu, *), x0, y0, z0
;           read(lu, *) dx, dy, dz
;         endif
; c        can make x, y, z if desired x(i)= (i-1)*dx + x0, etc.
;      endif
;      read(lu, *)(((image(i, j, k),i=1,nx),j=1, ny),z=1,nz)
;      close (lu)
;-
;
on_error, 1
;
      nparams = n_params()
      if nparams lt 2  then begin
        print,'% Format: REMSD_ext , fname, ARRAY  [, X] [, Y] [, Z]'
        print,'          [, DX = dx] [, DY = dy] [, DZ = dz] [, CLABEL = clabel]'
        print,'          [, XLABEL = xlabel] [, YLABEL = ylabel] [, ZLABEL = zlabel]'
        return
      endif
      if n_elements (fname) eq 0 then fname = 0
      b = size(fname)
      if b(b(0)+1) ne 7 then begin
        fname = dialog_pickfile(/must_exist, /read)
        if strlen(fname) lt 1 then begin
            print, '% REMSD: No File was Selected - RETURNING'
            return
        endif
      endif else begin
        fname = fname(0)
        info = file_info(fname)
        if (info.exists + info.read) ne 2b then begin
           print, '% REMSD: File does not exist with read permission'
           print, '         File Name = ', fname
           print, '         File Exists =', info.exists
           print, '         Read Permission =', info.read
           return
        endif
      endelse
      openr,unit,fname, /get_lun, error = error
      if error ne 0 then begin
        print, '% REMSD:  Error opening file: ', fname
        print, !error_state.msg
        return 
      endif
      comment = ''
      
on_ioerror, end_read
      mode = 'Reading the comment'
      readf, unit, comment
      ndim = 0L
      lcuniform = 0L
      mode = 'Reading the dimensions and the uniformity'
      readf, unit, ndim, lcuniform
      if ndim lt 2 or ndim gt 3 then begin

        print, '% REMSD: Only two and three dimensional data can be read'
        free_lun, unit
        return
      endif
      xlabel = ''
      ylabel = ''
      zlabel = ''
      mode = 'Reading Axis Labels'
      readf, unit, xlabel
      readf, unit, ylabel
      if ndim eq 3 then readf, unit, zlabel
      np = make_array(ndim, /long)
      mode = 'Reading the number of points'
      readf, unit, np
      if lcuniform eq 0 then begin
        mode = 'Reading X-Axis Array'
        x = make_array(np[0], /float)
        readf, unit, x
        mode = 'Reading Y-Axis Array'
        y =  make_array(np[1], /float)
        readf, unit, y
        if ndim eq 3 then begin
          mode = 'Reading Z-Axis Array'
          z =  make_array(np[2], /float)
          readf, unit, z
        endif else z = 0.0
        dx = 0
        dy = 0
        dz = 0
      endif else begin
        mode = 'Reading Uniform Axis Initial Values'
        val = make_array(ndim)    
        readf, unit, val
        x0= val[0]
        y0= val[1]
        if ndim eq 3 then z0= val[2]
        mode = 'Reading Uniform Axis Delta Values'
        readf, unit, val
        dx= val[0]
        dy= val[1]
        if ndim eq 3 then dz= val[2]
        x = findgen(np[0])*dx+x0
        y = findgen(np[1])*dy+y0
        if ndim eq 3 then z = findgen(np[2])*dz+z0
      endelse
      if ndim lt 3 then array = make_array(np[0],np[1]) else $
        array = make_array(np[0],np[1],np[2])
      mode = 'Reading Vector Values'
      readf, unit, array
      free_lun, unit
      return
 end_read:
      print, '% REMSD: IO Error'
      print, mode
      free_lun, unit
      end
