; ------------------------------------------------------------------------ 
; $Id: rotstr.pro,v 1.7 2010/11/18 15:39:02 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
 PRO rotstr, ain, anglein, save,  magnification = magnification, center=center, $
    interpolation = interp, pivot = pivot, Missing = missing, exact =exact, $
    grid = grid, ignore = ignore, points = points, radian = radian, dxdy = dxdy
    
;
; Copyright (c) 1982-1998, Research Systems, Inc.  All rights reserved.
;       Unauthorized reproduction prohibited.
; This routine uses the IDL function ROT as the basis for the math.
;   The actual PFIDL routine was written at Sandia National Laboratories.
;
;
;Rotate and magnify an image
;+
; NAME:
;	ROTSTR
;
; PURPOSE:
;	Rotate, magnify or demagnify, and/or translate an image.
;       Based on IDL procedure ROT
;   NOTE:  See FLIPSTR for 90 degree rotations and reflections.
;       Limited to single block, single vector data (but could be changed.)
;       NOTE: Image is interpolated so rather than sequential rotations, 
;             the angles should be summed and one rotation performed.
;       Data will be on a uniform rectilinear grid unless exact is set.
;       If EXACT is set, data will be finite element type data.
;
; CALLING SEQUENCE:
;	ROTSTR, STR, Angle [, SAVE] [, IGNORE = ignore] [, RADIAN = radian]
;               [, EXACT = exact] [, MISSING = missing] [, DXDY = dxdy]
;               [, POINTS = points]
;               [, MAGNIFICATION = magnification] [, CENTER = center]
;		[, GRID = grid] [, INTERPOLATION = interpolation] [, /PIVOT]
;
; INPUTS:
;	STR:	The image array to be rotated.  This is a field type structure
;               or structure array number. It must have two dimensions and be
;               uniform. If it is not uniform, it will be put on a uniform 
;               grid unless EXACT is set.
;               (Array have the same grid spacing in X and in Y.)
;               (Uniform is abs(DX-DY) < 1.5e-5 * range)
;       EXACT:  If set rotation will be exact and the result will not be
;               a rectilinear dataset.  If EXACT is set, IGNORE = 0
;               Only ANGLE, SAVE, CENTER and MAGNIFICATION are used for EXACT.
;               IGNORE is not checked for EXACT.
;       GRID:   Ignored unless exact is set.  
;               If set, puts the data on a uniform rectilinear grid using 
;               the minimum (dx, dy) of the original data.  If grid lt 1, then DX,DY
;               are multiplied by grid.
;       DXDY:   Two element array with DX and DY for the rotated structure if grid is set.
;               These values are returned to the caller and are the actual values used by
;               TRIGRID to put the rotated data on a rectilinear grid or the actual values
;               stored in the structure for the rotation using the POLY_2D function.
;	ANGLE:	Angle of rotation in degrees CLOCKWISE. (Why?,
;		because of an error in the old ROT.)
;               (Note: Angle is based on the coordinates.  If the structure,
;                      does not have square pixels, it will be interpolated
;                      unless exact is set.  Much of the IDL routine ROT is 
;                      used to perform the rotation.   If IGNORE is set,
;                      the angle will be in terms of PIXELS and spacial 
;                      coordinates are ignored.)
;       IGNORE: If set, ignore the spatial dimensions and assume that the angle
;               is measured in pixels and not spatial coordinates.
;       RADIAN: If set ANGLE is in radians, not degrees
;       POINTS: An array of one or more (X, Y) points that are rotated by the transform.
;               These only apply with "EXACT" set.
;               The form is POINTS =[X0, Y0, X1, Y1, X2, Y2, ...]
;               The POINTS are returned as:
;                 [X[X0], X[Y0], X[X1], X[Y1], ..., Y[X0], Y[Y0], Y[X1], Y[X2], ...]
;               NOTE: POINTS is input as X,Y pairs, and is returned as X-values
;                     followed by Y-values.
;	SAVE:	Structure array number for the rotated image.
;               If str is a structure array number, then the rotated
;                 image will be saved as a structure array number.
;               If str is a structure , then the rotated
;                 image will be saved as a structure.
;               Default: If SAVE is missing, rotated image will be returned
;                 in STR.
;
;	MAGNIFICATION: Magnification/demagnification factor.
;               A value of 1.0 = no change, > 1 is magnification
;               and < 1 is demagnification.
;               Generally this is not used. Default: magnification =1.0
;
;	CENTER:	(X, Y) Vector Values for the center of rotation.
;               Default: X = 0.5(xmax-xmin)
;                        Y = 0.5(ymax-ymin)
;               Note: If PIVOT = 0, then specifying a center is equal to 
;                     a translation.
;               Note: PIVOT is ignored if exact is set.
;
;	INTERPOLATION:	Set this keyword for interpolation type. 
;               Default: INTERPOLATION = 1
;               Values:
;		    0  - Nearest neighbor sampling is used.
;                   1  - Bilinear interpolation
;                   2  - "Cubic convolution" interpolation
;                        If this parameter is negative, it specifies the value
;                        of the cubic interpolation parameter as described
;                        in the INTERPOLATE function.
;                        Valid ranges are -1 <= Interp < 0.
;                        If INTERP = 2 then INTERP = -1
;
;      MISSING:	The data value to substitute for pixels in the output image 
;		that map outside the input image.
;               (See POLY_2D)
;      PIVOT: Setting this keyword causes the image to pivot around the point
;		X0, Y0, so that this point maps into the same point in the
;		output image.  If this keyword is set to 0, then the
;		point X0, Y0 in the input image is mapped into the center of
;		the output image.
;               Default: PIVOT = 1
;
; OUTPUTS:
;	ROTSTR returns a rotated, magnified, and translated version of the
;	input image.  Note that the dimensions of the output image are
;	always the same as those of the input image.
;
; PROCEDURE:
;	If exact is not set, the POLY_2D function is used to translate, scale, and
;	rotate the original image following the techniques in the function ROT.
;       If exact is set, each pixel is rotated exactly and the the resulting finite
;       element structure is returned unless GRID is set, and if grid is set, TRIGRID
;       is used to put the finite element points on a rectilinear mesh.
;
; EXAMPLE:
;	Rotate the structure 1, 33 degrees and magnify it 1.5 times.
;       Use bilinear interpolation to make the image look nice.  Store the 
;       result in 1.
;
;		ROTSTR,1, 33, mag =1.5 ; (Bilinear is the default)
;  
;       Same as above but store the rotated image in 5.
;           
;		ROTSTR,1, 33, 5, mag =1.5 ; (Bilinear is the default)
;
;       Display an image and determine a horizontal line using LINSTR  
;       Rotate the image and make the indicated line horizontal.
;       Assume the original image is in 5 and the result to be in 7
;               linstr, 5, slope = slope
;               rotstr, 5, slope, 7
;       NOTE:  If multiple rotations are desired it is better to:
;               rotstr, 5, slope1 + slope2, 7
;              The following requires 2 interpolations (avoid if possible):
;               rotstr, 5, slope1, 7
;               rotstr, 7, slope2
;
;	
; MODIFICATION HISTORY:
;       Original version: May 6, 1998, lpm
;       Using IDL ROT version IDL 5.1 B3
;       October 25, 2010: Added keywords, GRID, IGNORE, EXACT, DXDY, RADIAN, and POINTS
;
;-
;
@qscom.dat
;  Common block
@qsstrcom.dat
;       return on an error
      on_error, 1
;
      if n_elements (structinit) eq 0 then qsstruct_init
      if n_elements (qsplotinit) eq 0 then qsinit
;
      nparams = n_params()
      if nparams eq 0 then begin
        print,'% Format: ', $
       'ROTSTR, STR, Angle [, SAVE] [, MAGNIFICATION = magnification]'
print,'               [, /EXACT] [, GRID = grid] [, CENTER = center] [, MISSING = missing]'
print,'               [, INTERPOLATION = interpolation] [, /PIVOT]'
        return
      endif
      dxdy =0
;
      if n_elements(ain) eq 0 then begin
        print, '% ROTSTR: Input structure must be defined'
        return
      endif
; change variables to insure we don't corrupt input data
;     if a structure input then set lcstrin
      a = get_valid_structure(ain, 'ROTSTR', lcstr = lcstrin, ierror = ier)
      if ier ne 0 then return
;
      if a.type ne 1 then begin
        print, '% ROTSTR: Only standard image structures (type 1) can be rotated.'
        print, 'TYPE = ' + strtrim(a.type, 2)
        return
      endif
      if a.block  ne 1 then begin
        print, '% ROTSTR: Only single block image structures can be rotated.'
        return
      endif
      if nparams lt 2 then begin
        print, '% ROTSTR: Input structure and angle must be provided.'
        return
      endif
      if n_elements (anglein) eq 0 then begin
        print, '% ROTSTR: Angle of rotation must be defined.'
        return
      endif
;     angle in degrees CLOCKWISE.
      angle = (keyword_set(radian)) ? !radeg *anglein : anglein
      theta = (keyword_set(radian)) ? -anglein : -anglein/!radeg 
      
      if ((a.ndim eq 3 and a.two ne 3) or (a.ndim eq 4)) then begin
        print, '% ROTSTR: Only two dimensional image structures can be rotated.'
        return
      endif

      if a.nvect ne 1 then begin
        print, '% ROTSTR: Only structures with a single vector can be rotated.'
        return
      endif

      if n_elements(magnification) eq 0 then  mag=1.0 else $
        mag = magnification(0)
      if mag lt 0 then begin
        print,  '% ROTSTR: Magnification must be be >0. Setting to 1.0'
        mag = 1.0
      endif

      c = cos(theta)*mag		;cos theta * mag
      s = sin(theta)*mag
      nx = n_elements(a.x1)
      ny = n_elements(a.x2)
      if keyword_set(exact) then begin
;       NPOINT is only used for EXACT
        npoint = n_elements(points)
        if npoint gt 1 and not arg_present(points) then begin
          print, '% ROTSTR:  NPOINT must be a variable to return the rotated values.'
          npoint = 0
        endif
        if npoint gt 1 then begin
          npoint = npoint/2
          ind_point = lindgen(npoint)*2
          xpoint = points[ind_point]
          ypoint = points[ind_point+1]
        endif else npoint =0
;       see cyl2cartesian in the util directory to expand to multi block
        x = a.x1
        y = a.x2
        if n_elements (center) gt 1 then begin
          x0= center[0]
          y0= center[1]
        endif else begin
          minx = min(x, max = maxx)
          miny = min(y, max = maxy)
          x0 = (maxx-minx)*0.5
          y0 = (maxy-miny)*0.5
        endelse
        ntot = nx*ny
        xx = (x-x0) # replicate(1.0, ny)
        yy = replicate(1.0, nx) # (y-y0)
        x = reform (xx*c - yy*s, ntot)+x0
        y = reform (yy*c + xx*s, ntot)+y0
        minx = min(x, max = maxx)
        miny = min(y, max = maxy)
        if npoint gt 0 then begin
          xx = (xpoint - x0)
          yy =  (ypoint - y0)
          xpoint = x0+(xx*c -yy*s)
          ypoint = y0+(yy*c +xx*s)
          points = [xpoint, ypoint]
        endif
        tri = make_triangles(0, nx, ny)
        ntri = (size(tri))(2)
        if keyword_set (grid) then begin
          ok = where(strpos(tag_names(a), 'DX') ge 0 , nok)
;         DX might be missing because it was never checked.
          if nok lt 2 then begin
            check_uniform, a.x1, dx1, dxrange = dxx
            check_uniform, a.x2, dx2, dxrange = dyy
            if dx1 le 0 then dx1 = min(dxx)
            if dx2 le 0 then dx2 = min(dyy)
          endif else begin
            dx1= a.dx1(0)
            dx2= a.dx2(0)
            if dx1 le 0 or dx2 le 0 then begin
              check_uniform, a.x1, dx1, dxrange = dxx
              check_uniform, a.x2, dx2, dxrange = dyy
              if dx1 le 0 then dx1 = min(dxx)
              if dx2 le 0 then dx2 = min(dyy)
            endif
          endelse
;
          xerr = 3.0517578e-05*max(abs([maxx, minx, maxx-minx]))
          yerr = 3.0517578e-05*max(abs([maxy, miny, maxy-miny]))
          fudge = ((grid gt 0.0) and (grid lt 1.0)) ? grid : 1.0
          dx1 = min([dx1, dx2])* fudge
          dx2 = dx1
          nx = round ((maxx-minx)/dx1)+1         
          ny = round ((maxy-miny)/dx2)+1         
          ntot = nx*ny
          maxx = (nx-1)*dx1 +minx
          maxy = (ny-1)*dx2 +miny
          xx = minx+findgen(nx)*dx1
          yy = miny+findgen(ny)*dx2
          cont =reform( trigrid(x, y, a.v1, tri, [dx1, dx2], $
             [minx, miny,maxx-xerr, maxy-yerr]), ntot, /overwrite)
;         cont = trigrid(x, y, a.v1, tri, [dx1, dx2], $
;           [minx, miny,maxx-xerr, maxy-yerr])
          
          range = make_array(2, a.ndim, 2)
          range(0, 0, *) = minx
          range(0, 1, *) = miny
          range(1, 0, *) = maxx
          range(1, 1, *) = maxy
          if a.ndim eq 0 then range(*, 2, *) = a.slice
          ntot = nx*ny
          locx = make_array(a.ndim,2, value = 0L, /long)
          locx(0, 1)= nx 
          locx(a.ndim-1, 1) =1
          locx(1, 1)= ny
          if a.ndim eq 2 then ssize= [nx, ny] else ssize= [nx, ny, 1]
          
          a =  {type:1L, ndim:a.ndim, nvect: a.nvect, block :1L, two:a.two, $
              size:ssize,range: range, order: 1L, slice:a.slice , locx:locx, $
              locv: [0L, ntot], x1:xx, x2:yy, x3:a.x3, x4:0.0, $
              dx1:dx1, dx2:dx2, dx3:0.0, dx4:0.0, $
              v1:cont, v2:0.0, v3:0.0, v4:0.0, v5:0.0, v6:0.0, $
              x1lab : a.x1lab, x2lab : a.x2lab, x3lab : a.x3lab, x4lab :'', $
              v1lab : a.v1lab , v2lab :a.v2lab, $
              v3lab : a.v3lab, v4lab : '',v5lab : '', $
              v6lab : '', file : a.file, spare : [0,0,0,0,0], tlab : a.tlab, $
              clab: a.clab +' - Rotated by '+strtrim(angle, 2)+' degrees'}
          dxdy = [dx1, dx2]
        endif else begin
          minx = min(x, max = maxx)
          miny = min(y, max = maxy)
          range = make_array (2, a.ndim, 2)
          range (0, 0, *)  = minx
          range (1, 0, *)  = maxx
          range (0, 1, *)  = miny
          range (1, 1, *)  = maxy
          locx = [0L, ntot]
          locv = locx
          loctri = [0, ntri]
          ssize = [ntot, ntot, 1]
          a = {type: 8L, ndim:a.ndim, nvect: a.nvect, block:1L, $
            two:3L, slice: a.slice, size: ssize, range: range, order: 0L, $
            locx:locx, locv: locv, triangles: tri, $
            loctri: loctri, x1:x, x2:y, x3:a.x3, x4:a.x4, $
            v1:a.v1, v2:a.v2, v3:a.v3, v4:a.v4, v5:a.v5, v6:a.v6, $
            x1lab : 'X', x2lab :'Y', x3lab : 'Z', $
            x4lab :a.x4lab, v1lab : a.v1lab , v2lab : a.v2lab, $
            v3lab : a.v3lab, v4lab : a.v4lab, v5lab : a.v5lab, $
            v6lab : a.v6lab, file : a.file, spare : a.spare, tlab : a.tlab, $
                clab: a.clab +' - Rotated by '+strtrim(angle, 2)+' degrees'}
        endelse
        if nparams  eq 3 then begin
          if lcstrin eq 0 then begin
            save = get_save_str(save, ain)
            i2s, a, save, /no_copy
          endif else save = temporary(a)
        endif else begin
          if lcstrin eq 0 then i2s, a, ain, /no_copy else ain = temporary(a)
        endelse
        return
      endif  
    if keyword_set(ignore) then begin
      lcignore = 1
      dx1 = 1.0
      dx2 = 1.0
      cont  = reform(get_array(a, -1,1), /overwrite)
      ss = size(cont)
      nx = ss[1]
      ny = ss[2]
    endif else begin
      lcignore =0
      ok = where(strpos(tag_names(a), 'DX') ge 0 , nok)
;     DX might be missing because it was never checked.
      if nok lt 2 then begin
        check_uniform, a.x1, dx1, dxrange = dxx 
        check_uniform, a.x2, dx2, dxrange = dyy 
      endif else begin
        dx1= a.dx1(0)
        dx2= a.dx2(0)
        if dx1 le 0 or dx2 le 0 then begin
          check_uniform, a.x1, dx1, dxrange = dxx
          check_uniform, a.x2, dx2, dxrange = dyy
        endif else begin
          dxx = dx1
          dyy = dx2
        endelse
      endelse
      range=max([a.range(1,0,0)-a.range(0,0,0), a.range(1,1,0)-a.range(0,1,0)]) 
      cont  = reform(get_array(a, -1,1), /overwrite)
      if (dx1 le 0) or (dx2 le 0) or (abs(dx1-dx2) gt 1.5e-5 * range) then begin
        lcinterpolate = 1
;       put data on a uniform grid
        delta = min([dxx, dyy])
        if delta le 0 then begin
          print, '% ROTSTR: Error putting data on a uniform grid', delta
          return
        endif
        n = long((a.range(1,0,0)-a.range(0,0,0))/delta)+1L
        xx = a.x1(0) + delta *findgen(n)
        xtemp = get_yarr(a.x1, findgen(nx), xx)
        nx = n
        n = long((a.range(1,1,0)-a.range(0,1,0))/delta)+1L
        yy = a.x2(0) + delta *findgen(n)
        ytemp = get_yarr(a.x2, findgen(ny), yy)
        ny = n
        
        dx1 = delta
        dx2 = delta
;       for a single vector 
        cont= interpolate(cont, xtemp, ytemp, /grid)
      endif else begin
        lcinterpolate = 0
        xx = a.x1
        yy = a.x2
      endelse
      range = make_array(2, a.ndim, 2)
      range(0, 0, *) = xx(0)
      range(0, 1, *) = yy(0)
      range(1, 0, *) = xx(nx-1)
      range(1, 1, *) = yy(ny-1)
      if a.ndim eq 2 then ssize= [nx, ny] else ssize= [nx, ny, 1]
      locx = make_array(a.ndim,2, value = 0L, /long)
      locx(0, 1)= nx 
      locx(a.ndim-1, 1) =1
      locx(1, 1)= ny
    endelse
    ntot = nx*ny

;      if n_elements(missing) eq 0 then missing = 0.0
      if n_elements(magnification) eq 0 then  mag=1.0 else $
        mag = magnification(0)
      if n_elements(interp) eq 0 then interp = 1
      cubic = 0
      if interp lt 0 then cubic = interp > (-1)
      if interp eq 2 then cubic = -1
      if interp eq 1 then interpret = 1 else interpret = 0
;
      if n_elements(center) lt 2 then  begin
        x0= 0.5*(nx-1)
        y0= 0.5*(ny-1)
      endif else begin
        if n_elements (pivot) eq 0 then pivot = 1
        x0 = (center(0)-xx(0))/dx1
        y0 = (center(1)-yy(0))/dx2
      endelse
      if keyword_set(pivot) then begin
        xc = x0
        yc = y0
      endif else begin
        xc= 0.5*(nx-1)
        yc= 0.5*(ny-1)
      endelse
;
      kx = -xc+c*x0-s*y0		;useful constants.
      ky = -yc+s*x0+c*y0
      kk = 1./(1.+s^2/c^2)
;
      cx = kk* [s/c^2*ky+kx/c,s/c^2,1/c,0.] ;x coeff...
      cy = kk * [-s/c^2*kx+ky/c,1/c,-s/c^2,0.] ;y coeff.
      if n_elements (missing) gt 0 then $
        cont= reform(poly_2d(cont,cx,cy, interpret, MISSING = missing, $
              CUBIC=cubic), ntot, /overwrite) $
      else cont= reform(poly_2d(cont,cx,cy, interpret,  $
              CUBIC=cubic), ntot, /overwrite)
    if keyword_set(lcignore) then begin
      put_array, a, cont, -1, 1
      chastr, a, clab = a.clab +' - Rotated by '+strtrim(angle, 2)+' degrees'
    endif else $
      a =  {type:1L, ndim:a.ndim, nvect: a.nvect, block :1L, two:a.two, $
              size:ssize,range: range, order: 1L, slice:a.slice , locx:locx, $
              locv: [0L, ntot], x1:xx, x2:yy, x3:a.x3, x4:0.0, $
              dx1:dx1, dx2:dx2, dx3:0.0, dx4:0.0, $
              v1:temporary(cont), v2:0.0, v3:0.0, v4:0.0, v5:0.0, v6:0.0, $
              x1lab : a.x1lab, x2lab : a.x2lab, x3lab : a.x3lab, x4lab :'', $
              v1lab : a.v1lab , v2lab :a.v2lab, $
              v3lab : a.v3lab, v4lab : '',v5lab : '', $
              v6lab : '', file : a.file, spare : [0,0,0,0,0], tlab : a.tlab, $
              clab: a.clab +' - Rotated by '+strtrim(angle, 2)+' degrees'}
      if a.type eq 1 then dxdy = [a.dx1, a.dx2]
      if nparams  eq 3 then begin
        if lcstrin eq 0 then begin
          save = get_save_str(save, ain)
          i2s, a, save, /no_copy
        endif else save = temporary(a)
      endif else begin
        if lcstrin eq 0 then i2s, a, ain, /no_copy else ain = temporary(a)
      endelse
      
END
;
