; ------------------------------------------------------------------------ 
; $Id: rtpstr.pro,v 1.4 2009/03/30 19:51:24 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
 Pro  rtpstr, wdf1, wdf2, wdf3, exact = exact, clabel = clabel
;
;+
; NAME:
;       RTPSTR
;
; PURPOSE:
;       Raise field structure components to a power
;       Store the result in the initial field structure array or 
;         another structure array
;       All attribute or vector quantities are raised to a power
; CATEGORY:
;       Math 
; CALLING SEQUENCE:
;       RTPSTR, STR1, VALUE, [, STR3] [, /EXACT] [, CLABEL = clabel]
; INPUTS:
;      STR1:    array number of the first field structure array
;      VALUE:   Power to which the first array is raised
; OPTIONAL INPUT:
;      STR3:    array number for the result, if not specified the 
;                 result is stored in STR1 (the first array)
;      EXACT:   If present and not zero, aborts the calculation if STR1 < 0
;                 and value is a real.  This is ignored if VALUE is an integer.
;      CLABEL:  Optional label for the resultant array
; OUTPUTS:
;       NONE    The quotient is stored in a structure array in the common block
; COMMON BLOCKS:
;       QSCOM.DAT, QSSTRCOM.DAT
; RESTRICTIONS:
;       STR1 array must be defined and have valid data
;       VALUE must be defined
; PROCEDURE:
;       If  VALUE is an integer then the result is calculated
;           Result is STR1^VALUE
;           Each attribute, or vector component, of STR1 is raised to the power
;
;       If VALUE is not an integer then the result is calculated
;           Result is exp(VALUE* alog(STR1)) if STR1 > 0
;        or Result is 0.0 if STR1 < 0.0
;
;       In either case RESULT is stored in STR3, if specified, or STR1
; EXAMPLES:
;       Raise structure 2 to the 1.5 power and store the result in
;       structure 3.  Set any negative values in array 2 to zero
;               RTPSTR, 2, 1.5, 3
;       Raise structure 2 to the 0.5 power and store the result in 
;       structure 3. Do not do the calculation if any values are < 0.0
;               RTPSTR, 2, 0.5, 3, /exact
;       Raise structure 2 to the 3 power and store the result back into 2
;               RTPSTR, 2, 3
; MODIFICATION HISTORY:
;      Original version, L. P. Mix, August 15, 1993
;-
;
; insert a common block
;
@qscom.dat
;  Common block
@qsstrcom.dat
;       return on an error
      on_error, 1
;
      if n_elements (structinit) eq 0 then qsstruct_init
      if n_elements (qsplotinit) eq 0 then qsinit
;
      nparams = n_params()
      if nparams lt 2 then begin
        print,' RTPSTR must have at least 2 parameters'
        return
      endif
;
      
      if n_elements(wdf1) eq 0 then begin
        print, '% RTPSTR: Structure array must be defined'
        return
      endif
      array1 = get_valid_structure(wdf1, 'RTPSTR', lcstr=lcstrin, ierror=ier)
      if ier ne 0 then return
      if ((array1.type ne 8) and (array1.type ne 1) $
	 and (array1.type ne 9)) then begin
        print, '% RTPSTR:  Type 1, Type 8, or Type 9 field structures only'
        return
      endif
;
      if n_elements(wdf2) Eq 0 then begin
        print, '% RTPSTR: Power must be defined '
        return
      endif else value = wdf2[0]
      
;
      if n_elements(exact) eq 0 then exact = 0
;
      if array1.type ne 1 then begin
        print, ' RTPSTR may only be used with field structures'
        return
      endif
      if value eq long (value) then begin
        array1.v1 = array1.v1 ^ value
        if array1.nvect gt 1 then  array1.v2 = array1.v2 ^ value
        if array1.nvect gt 2 then  array1.v3 = array1.v3 ^ value
        if array1.nvect gt 3 then  array1.v4 = array1.v4 ^ value
        if array1.nvect gt 4 then  array1.v5 = array1.v5 ^ value
        if array1.nvect gt 5 then  array1.v6 = array1.v6 ^ value
      endif else begin 
        ok = where (array1.v1 lt 0.0,nok)
        if nok gt 0 then begin
          if exact ne 0 then begin 
            print, ' Cannot raise a negative value to a real power'
            print, '  No modification to structure', wdf1
            return
          endif
          array1.v1(ok) = 1.0
          array1.v1 = array1.v1 ^ value
          array1.v1(ok) = 0.0
        endif else array1.v1 = array1.v1 ^ value
        if array1.nvect gt 1 then begin
          ok = where (array1.v2 lt 0.0, nok)
          if nok gt 0 then begin
            if exact ne 0 then begin 
              print, ' Cannot raise a negative value to a real power'
              print, '  No modification to structure', wdf1
              return
            endif
            array1.v2(ok) = 1.0
            array1.v2 = array1.v2 ^ value
            array1.v2(ok) = 0.0
          endif else array1.v2 = array1.v2 ^ value
          if array1.nvect gt 2 then begin
            ok = where (array1.v3 lt 0.0, nok)
            if nok gt 0 then begin
              if exact ne 0 then begin 
                print, ' Cannot raise a negative value to a real power'
                print, '  No modification to structure', wdf1
                return
              endif
              array1.v3(ok) = 1.0
              array1.v3 = array1.v3 ^ value
              array1.v3(ok) = 0.0
            endif else array1.v3 = array1.v3 ^ value
            if array1.nvect gt 3 then begin
              ok = where (array1.v4 lt 0.0, nok)
              if nok gt 0 then begin
                if exact ne 0 then begin 
                  print, ' Cannot raise a negative value to a real power'
                  print, '  No modification to structure', wdf1
                  return
                endif
                array1.v4(ok) = 1.0
                array1.v4 = array1.v4 ^ value
                array1.v4(ok) = 0.0
              endif else array1.v4 = array1.v4 ^ value
              if array1.nvect gt 4 then begin
                ok = where (array1.v5 lt 0.0, nok)
                if nok gt 0 then begin
                  if exact ne 0 then begin 
                    print, ' Cannot raise a negative value to a real power'
                    print, '  No modification to structure', wdf1
                    return
                  endif
                  array1.v5(ok) = 1.0
                  array1.v5 = array1.v5 ^ value
                  array1.v5(ok) = 0.0
                endif else array1.v5 = array1.v5 ^ value
                if array1.nvect gt 5 then begin
                  ok = where (array1.v6 lt 0.0, nok)
                  if nok gt 0 then begin
                    if exact ne 0 then begin 
                      print, ' Cannot raise a negative value to a real power'
                      print, '  No modification to structure', wdf1
                      return
                    endif
                    array1.v6(ok) = 1.0
                    array1.v6 = array1.v6 ^ value
                    array1.v6(ok) = 0.0
                  endif else array1.v6 = array1.v6 ^ value
                endif
              endif
            endif
          endif
        endif
      endelse
      if n_elements(clabel) gt 0 then array1.clab = string(clabel)
      if nparams gt 2 then  begin
        if lcstrin eq 0 then begin
          wdf3 = get_save_str(wdf3, wdf1)
          i2s, array1, wdf3, /no_copy
        endif else wdf3 = temporary(array1)
      endif else begin
        if lcstrin eq 0 then i2s, array1, wdf1, /no_copy $
          else wdf1 = temporary(array1)
      endelse
      return
      end
