; ------------------------------------------------------------------------ 
; $Id: scale_image.pro,v 1.5 2011/03/14 17:11:52 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
 Pro  Scale_image, strarray, dimarray, label = labelin, Mdim = Mdim, $
                Ddim = ddim, Rdim = rdim, Ndim = ndim
;
;+
; NAME:
;       Scale_Image
;
; PURPOSE:
;       Change the dimensions of an image or single block field dataset.
;       Only single block, type 1 (Rectilinear), data is supported.
; CATEGORY: 
;       Utility 
; CALLING SEQUENCE: 
;       Scale_image, STR, DIM
; INPUTS: 
;       STR:      Structure array or Structure Array Number
;       DIM:      Dimension(s) to be changed; Must be in the range 1 to NDIM
;                 where NDIM is the number of dimensions
;
; KEYWORD PARAMETERS:
;   Note: ONE and only ONE of the DIM keywords must be specified.
;   Note: POLY in the descriptions below takes an array X and Coefficients C
;         and calculates: POLY(X,C)= C[0] + X*C[1] + X*X*C[2] + ... +X^(M)*C[M]
;         where M = N_Elements(C)-1
;
;       MDIM:     Multiply the current axis values by this coefficient or 
;                 array of coefficients.
;                 Example: 
;                   X is current set of dimensions
;                   Xnew = POLY(X, Mdim) is the new set of dimensions
;       DDIM:     Calculate the spacing between points by this coefficient or 
;                 array of coefficients.
;                 Example:
;                   X is current set of dimensions
;                   Xave is average bin value with one fewer elements.
;                   Xave[i]= (X[i]+X[i+1])*0.5
;                   dx is the spacing
;                   dx[i] = (X[i+1]-X[i])
;                   DX = poly(Xave, DDIM)
;                   Xnew = [0, total(dx*DX, /cum)
;       NDIM:     Calculate the spacing between points by this coefficient or 
;                 array of coefficients.  This is similar to DDIM but ignores
;                 original position values.
;                 Example: 
;                   X is current set of dimensions with N values
;                   DX = poly(findgen(n-1)+0.5, NDIM)
;                   Xnew = [0, total(DX, /cum)
;       RDIM:     Reciprocal of the spacing between points is given by this 
;                 coefficient or array of coefficients.  
;                   X, Xave, dx are the same as in DDIM
;                   DX  = POLY(Xave, RDIM)
;                   Xnew = [0, total(dx/DX, /cum)
;       LABEL:    Replace the DIM axis label with this string.
;
; OUTPUTS:
;       NONE:     The specified structure is modified
;               
; COMMON BLOCKS:
;       QSCOM, QSSTRCOM 
; RESTRICTIONS:
;       STR array must be defined and have valid data
;       DIM must be specified
;       One of the keywords: Mdim, Ddim, Ndim, or Rdim must be specified.
;
; EXAMPLES:
;       Double the value of the points in the y direction on structure 2 and add
;            1 to the result.
;            Scale_Image, 2, 2, mdim =[1, 2]
;
;       Assume an image was made with a magnification of 1.233 and is in
;       structure, pict.  Convert the dimensions of the image to the source.
;            Scale_Image, pict, [1,2], m=[0,1/1.233], lab = 'Target Dimensions'
;
;       Assume a streak camera image in structure 2 with a light burst at 
;       4 ns intervals.  Change the Y axis to make it time in ns.
;           ; Determine the spacing in the original image and store the 
;           ; period in waveform array 66
;           get_sweep, 2, 66 
;           ; Fit the period and save the coefficients
;           wdfit, 66, 67, coef = coef, /plo, degree = 2
;           ; Scale the Y axis
;           Scale_Image, 2, 2, rdim = coef/4 , lab= 'Time (ns)'
;        Note:  A repeat of get sweep should indicate a period of 4 ns.
;
; MODIFICATION HISTORY:
;      Original version, L. P. Mix, July 11, 2000
;-
;
on_error, 1
;     include the common block
;
@qscom.dat
;  Common block
@qsstrcom.dat
;
      if n_elements (structinit) eq 0 then qsstruct_init
      if n_elements (qsplotinit) eq 0 then qsinit
;
      nparams = n_params()
      if nparams eq 0 then begin
        print,'% Format:', $ 
 ' SCALE_IMAGE, STRARRAY, DIMS [, LABEL= lab] [, MDIM= mdim]'
 print, '         [, RDIM = rdim] [, NDIM = ndim] [, DDIM = ddim]'
;
        return
      endif
;
;     determine if strarray is specified
;
      if nparams lt 2 then begin
        print, '% Scale_Image, Structure and Dimension must be provided'
        return 
      endif
      nelstr = n_elements(strarray)
      if nelstr eq 0 then begin
        print, '% Scale_Image: Structure must be defined'
        return
      endif else strarray= strarray[0]
      
      if n_elements(dimarray) eq 0 then begin
        print, '% Scale_Image: Dimension(s) must be defined'
        return
      endif

      nm = n_elements(mdim)
      nd = n_elements(ddim)
      nr = n_elements(rdim)
      nn = n_elements(ndim)
      okcoef = where ([nm, nd, nr, nn] gt 0, nok)
      if nok eq 0 or nok gt 1 then begin
        print,  $
        '% Scale_Image: One and only one set of coefficients must be provided'
        print, nm, nd, nr, nn
        return
      endif

      str = get_valid_structure(strarray, 'Scale_Image', lcstr=lcstr, $
        ierror=ier)
      if IER ne 0 then return

      if str.type ne 1 then begin
        print, '% Scale_Image: Only type 1 structures may be scaled.'
        return
      endif
      if str.block ne 1 then begin
        print, '% Scale_Image: Only single block structures may be scaled'
        return
      endif
  
      ok = where( dimarray gt 0 and dimarray le str.ndim , nok)

      if nok eq 0 then begin
        print, '% Scale_Image: Dimension must be in the interval [1, ndim]'
        print, dimarray
        return
      endif

      dimarr =  dimarray[ok]

      okcoef = okcoef[0]
      nlab = n_elements (labelin)
      if nlab gt 0 then label = strtrim(labelin[0], 2)
      for idim = 0, nok-1 do begin
        dim = dimarr[idim]
        case okcoef of 
          0 : BEGIN
;             Simple multiply
                x = get_array(str, dim, 1, label = lab)
                if nlab gt 0 then lab = label
                y = make_array(n_elements(x), /double,value=double(mdim[nm-1]))
                if nm gt 1 then for i = nm-2, 0, -1 do Y= Y*X + mdim[i]
; plot, x, y
                put_array, str, y, dim, 1, label = lab
              END
          2 : BEGIN
;             Reciprocal 
                x = get_array(str, dim, 1, label = lab)
                if nlab gt 0 then lab = label
                nx = n_elements (x)
                dx = x[1:nx-1]-x[0:nx-2]
                y = make_array(nx-1, /double,value=double(rdim[nr-1]))
                if nr gt 1 then begin
                  xave = 0.5*(x[0:nx-2]+x[1:nx-1])
                  for i = nr-2, 0, -1 do Y= Y*Xave + rdim[i]
                endif
                y = [0.0 , total(dx/y, /cumulative, /double)]
; plot, x, y
                put_array, str, y, dim, 1, label = lab
              END
          1 : BEGIN
;             Delta
                x = get_array(str, dim, 1, label = lab)
                if nlab gt 0 then lab = label
                nx = n_elements (x)
                dx = x[1:nx-1]-x[0:nx-2]
                y = make_array(nx-1, /double,value=double(ddim[nd-1]))
                if nd gt 1 then begin
                  xave = 0.5*(x[0:nx-2]+x[1:nx-1])
                  for i = nd-2, 0, -1 do Y= Y*Xave + ddim[i]
                endif
                y = [0.0 , total(dx*y, /cumulative, /double)]
; plot, x,y
                put_array, str, x, dim, 1, label = lab
              END
          3 : BEGIN
;             Integer generation function
                x = get_array(str, dim, 1, label = lab)
                if nlab gt 0 then lab = label
                nx = n_elements (x)
                y = make_array(nx-1, /double,value=double(ndim[nn-1]))
                if nn gt 1 then begin
                  ddx = poly(findgen(nx-1)+0.5, ndim)
                  for i = nn-2, 0, -1 do Y= Y*DDX + ndim[i]
                endif
                y = [0, total(y, /cumulative, /double)]
; plot, x, y
                put_array, str, y, dim, 1, label = lab
              END
        endcase
      endfor
      if lcstr eq 1 then strarray = str else i2s, str, strarray, /no_copy
      return
      end
