; ------------------------------------------------------------------------ 
; $Id: scale_regrid.pro,v 1.4 2007/06/06 20:09:36 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
 pro scale_regrid, data, x, y, newx, newy, valid = valid, break= break, $
    c1 = c1in, c2 = c2in, c3 = c3in, zero_origin = zero_origin, $
    exponential = lclogin, writepff = lcwrite, tlabel = tlab, clabel = clab, $
    xlabel = xlab, ylabel = ylab, zlabel = zlab, blabel = blab, fudge = fudge, $
    default_val = default_val, reform_data = reformit, ier = ier
;
;+
; NAME:
;       SCALE_REGRID
;
; PURPOSE:
;       Fortran routine which will scale data according to a polynomial
;       equation with specified coefficients. 
;       Note: See also PFF_REGRID 
;       Optionally the routine will average points to reduce the size of the
;       array.
;       Optionally the routine will output the data to a PFF file.
;       Note: If fit occurs in the same idl session, then parameters 
;             will be taken from the WDFIT3 common block.
;             READWD3 routine will also fill the WDFIT3 common block.
;
; CALLING SEQUENCE:
;      SCALE_REGRID, DATA [, X] [, Y] [, NEWX] [, NEWY]
;       [, BREAK = break]  ,C1 = c1 [, C2 = c2] [, C3 = c3] [, FUDGE = fudge]
;        [, /EXPONENTIAL] [, /WRITEPFF] [, /REFORM_DATA] [, /ZERO_ORIGIN]
;        [, VALID = valid] [, DEFAULT_VAL = default_val]
;        [, TLABEL = tlabel] [, CLABEL = clabel]
;        [, XLABEL = xlabel] [, YLABEL = ylabel] [, ZLABEL = zlabel]
;        [, BLABEL = blabel] [, IER = ier]
; where:
;        DATA      - Array to be scaled.  May be one or two dimensions.
;                    Data must be a float array.
;                    DATA will be be changed.
;                    If DATA = DATA (NX, NY) then the returned value will
;                    be DATA = DATA (NX/NEWX, NY/NEWY)
;                    Note: NY may equal 1
;                    If DATA is not 2 dimensional then newy = 1
;        X, Y      - Optional X and Y parameters for data.  
;                    Two element vector with initial X value and X spacing.
;                    Default: X, Y  = [0, 1] 
;                    These will be modified dependind on NEWX, NEWY.
;        NEWX,NEWY - Used to rescale X or Y 
;                    NEWX points in the X direction and NEWY points in the Y
;                    direction will be averaged to obtain a new average value.
;                    Default: NEWX, NEWY = 1L (No averaging of data)
;        ZERO_ORIGIN - If ZERO_ORIGIN is set then X(0)=0.0 and Y(0) = 0.0
;                    Otherwise:
;                        x(0) = x(0) +x(1)*(newx-1)/2
;                        y(0) = y(0) +y(1)*(newy-1)/2
;        VALID     - Two element vector with minimum and maximum values for
;                    the scaling.  DATA values outside these limits will be
;                    set to the values of default.
;                    Default: VALID = [min(data, max = mdata), mdata]
;        DEFAULT_VAL- If DATA is outside the range set by valid, then these are
;                    the values to which DATA will be set.
;                       If data < valid(0) then data = default_val(0)
;                       If data > valid(1) then data = default_val(1)
;                    Default values are: DEFAULT_VAL = scale_regrid(VALID)
;                       unless EXPONENTIAL is set.
;                    If exponential is set, DEFAULT_VAL(0) = 0.0
;        BREAK     - A vector of one or two elements indicating the positions
;                    where the C1 coefficients go to the C2 coefficients.
;                    The second element, if it exists, is where the C2
;                    coefficients go to the C3 coefficients.
;                    For three regions to the fit, C1, C2 and C3 must each have 
;                    two or more elements and BREAK must have two elements.
;        FUDGE     - Factor used to insure the function is continuous between
;                    polynomial fits. Ignored if N_Elements(c2) lt 2
;                    This is, if data has no breaks fudge is not used.
;                    Default: FUDGE = 0.05 of Valid 
;                    Fudge details:
;                    Segment 1, 2 and 3 (if there are 2 break points) are
;                    modified according to the following:
;                    Let x be the value of image and y the scaled value.
;                      y = poly( x, c1)
;                      range1 = break(1)-valid(1)
;                    if (break(1)-x)/range1 is less than fudge/2 then y is
;                     modified
;                      y = y+delta1 *(1-(break(1)-x)/(fudge*range1))
;                    where delta1= poly(break(1), c2)-poly(break(1),c1)
;                    C2 is also modified.
;       Exponential- If set then after the scaling the data will be exponentiated.
;                    Default: Exponential = 0
;                    If exponential = 0 then data = poly(data, C)
;                    If exponential = 1 then data = exp(poly(data, C))
;        WRITEPFF  - If set, data will be written to the current PFF file.
;                    Default: WRITEPFF = 0
;       REFORM_DATA- If set, data will be reformatted to:
;                       data = data;   if newx = 1, newy = 1
;                 else  data = data(nx/newx, ny/newy)
;                    Default: If WRITEPFF is set, data will not be reformatted.
;                             If WRITEPFF is not set, data will be reformatted.
;      The following are used only if WRITEPFF is set
;        Default value for all these is: '  '
;        TLABEL    - Dataset TYPE label
;        CLABEL    - Dataset COMMENT label
;        XLABEL    - Dataset X label
;        YLABEL    - Dataset Y label
;        ZLABEL    - Dataset Z label
;        BLABEL    - Dataset BLOCK label
;                    
; EXAMPLE:
;       Obtain polynomial scaling of the form:
;          y = 45 + 7x + 3x^2 
;       for 501 x values in the range 0 to 10 and plot the results.
;          image = findgen(501)*(10.0/500)
;          xx = image ; need to save a copy for the plot
;          SCALE_REGRID, image,c1=[45, 7, 3]
;          plot, xx, image
;
; MODIFICATION HISTORY:
; 	Written by:	Paul Mix, August 3, 1998
;-
      on_error, 1
;
;     add common block
@com.dat
;
      common pffcom, initpff, pff_exe
; Common block
@wdfit3_common.dat

;
      if (n_elements(initpff) eq 0) then initpff=0
;
      npar = n_params()
      if npar eq 0 then begin
print, '% Format:  SCALE_REGRID, DATA [, X] [, Y] [, NEWX] [, NEWY] [, VALID]'
print, '         [, BREAK] ,C1 = c1 [, C2 = c2] [, C3 = c3] [, FUDGE = fudge]'
print, '         [, /EXPONENTIAL] [, /WRITEPFF] [, /REFORM_DATA]'
print, '         [, TLABEL = tlabel] [, CLABEL = clabel]'
print, '         [, XLABEL = xlabel] [, YLABEL = ylabel] [, ZLABEL = zlabel]'
print, '         [, BLABEL = blabel] [, IER = ier]'

        return
      endif
      if n_elements (data) eq 0 then begin
        print,'% SCALE_REGRID: Input data must be defined.'
        return 
      endif
      n1 = n_elements(c1in)
      n2 = n_elements(c2in)
      n3 = n_elements(c3in)
      c2 = 0.0
      c3 = 0.0
      nbreak =0L
      if n_elements(lc_valid) eq 0 then lc_valid = 0
      if n1 lt 2 then begin
        if n_elements(c1_save) gt 1  and lc_valid ne 0 then begin
          c1= double(c1_save)
          c2= double(c2_save)
          c3= double(c3_save)
          n1 = n_elements(c1)
          n2 = n_elements(c2)
          n3 = n_elements(c3)
          lclogin = lc_log
          if n_elements(valid) eq 0 then valid = valid_save
          break =break_save 
          nbreak = long(nbreak_save)
        endif else begin
          print,'% SCALE_REGRID: Polynomial fit requires at least 2 elements: C1'
          return
        endelse
      endif else begin
        c1 = double(c1in)
        if n2 gt 1 then begin
          nbreak = 1L
          c2 = double(c2in)
          if n3 gt 1 then begin
            nbreak = 2L
            c3 = double(c3in)
          endif
        endif
      endelse
      ss = size(data)
      if ss[0] ne 2 then newy = 1L
      if ss[0] eq 0 then nx = 1L else nx = ss[1]
      ny = ss[ss[0]+2]/nx
;
      if ss[ss[0]+1] ne 4 then data = float(temporary(data))

      if n_elements(break) lt nbreak then begin
        Print,  '% SCALE_REGRID: Number of break points must be one less than' 
        print,  '                the number of coefficient arrays', $
          nbreak,n_elements(break)
        return
      endif
      if n_elements (break) eq 0 then  break = 0.0 else break = float(break)
        

      if n_elements (x) lt 2 then x = [0.0, 1.0] else x = float(x[0:1])
      if n_elements (y) lt 2 then y = [0.0, 1.0] else y = float(y[0:1])

      if n_elements(newx) eq 0 then newx = 1L else newx = long(newx > 1L) < nx
      if n_elements(newy) eq 0 then newy = 1L else newy = long(newy > 1L) < ny
;
      if n_elements(valid) lt 2 then valid = [min(data, max = mdata), mdata]
      valid = float(valid)

      if keyword_set(lclogin) then lclog = 1L else lclog= 0L

      if keyword_set(lcwrite) then lcwrite = 1L else lcwrite= 0L
      if (initpff eq 0 and lcwrite eq 1 ) then begin
        print,'% SCALE_REGRID: File must be opened or created before writing data'
        return
      endif 
      if initpff eq 0 then init_pff
      if initpff eq 0 then begin
        print,'% SCALE_REGRID: This routine is not supported on this installation'
        return
      endif

      if n_elements(default_val) lt 2  then begin
        if nbreak eq 0 then def_v = poly(valid, c1) else begin
          def_v = [0.0,0.0]
          if valid(0) le break(0) then def_v(0) = poly( valid(0), c1)
          if valid(1) le break(0) then def_v(1) = poly(valid(1), c1)
          if nbreak eq 1 then begin
            if valid(0) gt break(0) then def_v(0) = poly(valid(0), c2)
            if valid(1) gt break(0) then def_v(1) = poly(valid(1), c2)
          endif else begin
            if valid(0) gt break(1) then def_v(0) = poly(valid(0), c3) else $
               if valid(0) gt break(0) then def_v(0) = poly(valid(0), c2)
            if valid(1) gt break(1) then def_v(1) = poly(valid(1), c3) else $
              if valid(1) gt break(0) then def_v(1) = poly(valid(1), c2)
          endelse
        endelse
        if keyword_set(lclogin) then def_v = exp(def_v)
      endif else def_v = default_val

      def_v = float(def_v)

      if newx ne 1L then begin
        x(0) = x[0]+x[1]*(newx-1)*0.5
        x(1) = x[1]*newx
      endif
      
      if newy ne 1L then begin 
        y(0) = y[0]+y[1]*(newy-1)*0.5
        y(1) = y[1]*newy
      endif

      if keyword_set(zero_origin) then begin
        x(0) = 0.0
        y(0) = 0.0
      endif 
      if lcwrite eq 0 then begin
        llab = 2L
        lcom = 2L
        stb = make_array(4, /byte, value = 32B)
        scb = make_array(4, /byte, value = 32B)
        xlabb = make_array(4, /byte, value = 32B)
        ylabb = make_array(4, /byte, value = 32B)
        zlabb = make_array(4, /byte, value = 32B)
        blabb = make_array(4, /byte, value = 32B)
      endif else begin
        if n_elements(clab) eq 0 then clab = 'Unfold Image' else $
           clab = strtrim(clab,2)
        lcom = strlen(clab) +3L
        lcom =   long(lcom -(lcom mod 4)) < 1024L
        str2byt, clab, scb, lcom
        if n_elements(tlab) eq 0 then tlab = 'Unfold Image' else $
          tlab = strtrim(tlab,2)
        if n_elements(xlab) eq 0 then xlab= 'X' else xlab= strtrim(xlab,2)
        if n_elements(ylab) eq 0 then ylab= 'Y' else ylab= strtrim(ylab,2)
        if n_elements(zlab) eq 0 then zlab= 'Z' else zlab= strtrim(zlab,2)
        if n_elements(blab) eq 0 then blab= 'Block-01' else blab= strtrim(blab,2)
        llab = max([strlen(tlab), strlen(xlab), strlen(ylab), strlen(zlab), $
               strlen(blab)]) +3
        llab =   long(llab - (llab mod 4)) < 128L
        str2byt, tlab, stb, llab
        str2byt, xlab, xlabb, llab
        str2byt, ylab, ylabb, llab
        str2byt, zlab, zlabb, llab
        str2byt, blab, blabb, llab
      endelse
      def_v = float(def_v)
      nelf = n_elements (fudge)
      
      if nelf eq 0 then fudge = [0.125, 0.125] else $
        if nelf eq 1 then fudge = float([fudge, fudge]) else $
        fudge = float(fudge)
      ier = 0L

      if initpff eq 1 then begin
; HP
        junk=call_external(PFF_EXE, 'unfold_slregrid',  data, x, y, nx, ny,  $
        newx, newy, valid, def_v,break, nbreak, n1, c1, n2, c2, n3, c3, lclog, $
        lcwrite, fudge, stb, scb, xlabb, ylabb, zlabb, blabb, ier, llab, lcom)

      endif else if initpff eq 2 then begin
; Sun
        junk=call_external(PFF_EXE, 'unfold_slregrid_',  data, x, y, nx, ny,  $
        newx, newy, valid, def_v,break, nbreak, n1, c1, n2, c2, n3, c3, lclog, $
           lcwrite, fudge, stb, scb, xlabb, ylabb, zlabb, blabb, ier, llab, lcom)

      endif else if initpff eq 4 then begin
; WIN
        junk=call_external(PFF_EXE, '_UNFOLD_SLREGRID@8', data, x, y, nx, ny,  $
        newx, newy, valid, def_v,break, nbreak, n1, c1, n2, c2, n3, c3, lclog, $
           lcwrite, fudge, stb, scb, xlabb, ylabb, zlabb, blabb, ier, llab, lcom)

      endif else if initpff eq 5 then begin
; MAC
        junk=call_external(PFF_EXE,  'unfold_slregrid',  data, x, y, nx, ny,  $
        newx, newy, valid, def_v,break, nbreak, n1, c1, n2, c2, n3, c3, lclog, $
           lcwrite, fudge, stb, scb, xlabb, ylabb, zlabb, blabb, ier, llab, lcom)
      endif else if initpff eq 3 then begin
; VMS
        junk=call_external(PFF_EXE, 'unfold_slregrid',  data, x, y, nx, ny,  $
        newx, newy, valid, def_v,break, nbreak, n1, c1, n2, c2, n3, c3, lclog, $
           lcwrite, fudge, tlab, clab, xlab, ylab, zlab, blab, ier)
      endif
;
      if ier ne 0 then Print,'% SCALE_REGRID: Error with UNFOLD_REGRID.'
      if n_elements(reformit) eq 0 then begin
        if keyword_set(lcwrite) then return else reformit = 1
      endif
      if newx eq 1 and newy eq 1 then return
      if not keyword_set(reformit) then return
      data = reform (temporary(data(0:nx*ny-1)), nx, ny)   
      return
      end

