; ------------------------------------------------------------------------ 
; $Id: smostr.pro,v 1.4 2007/06/06 20:09:37 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
 Pro  Smostr, strarray, SavSTR, Width = width, Median = median, $
            xonly = xonly, yonly = yonly, _extra= extrastuff
;
;+
; NAME:
;       SMOSTR
;
; PURPOSE:
;       Smooth the Vectors of a Type 1 field structure
;
; CATEGORY: 
;       Utility 
;
; CALLING SEQUENCE: 
;       SMOSTR, STR_IN [, STR_OUT] 
; INPUTS: 
;       STR_IN:   Structure array number or structure array to be smoothed.
;                 If STR_OUT is specified, then STR_OUT = SMOOTH(STR_IN)
;       STR_OUT:  Structure array number or structure array for the smoothed 
;                 result.  If STR_OUT not is specified, then 
;                 STR_OUT = SMOOTH(STR_IN)
; KEYWORD PARAMETERS:
;       MEDIAN:   If set a median filter is used using the IDL routine MEDIAN.
;                 Default is boxcar filter using the IDL routine SMOOTH.
;       WIDTH:    Width of the filter.  Default = 3
;                 If width is greater than array dimensions, then no action is 
;                 taken.
;       XONLY:    If set, smooth only in the X direction, one row at a time.
;                 Limited to 2 dimensional data
;       YONLY:    If set, smooth only in the Y direction, one column at a time.
;                 Limited to 2 dimensional data
;       _EXTRA:   Any valid keyword for MEDIAN or SMOOTH.
;       
; OUTPUTS:
;       The specified structure is filtered and returned in STR_IN or STR_OUT
;
; RESTRICTIONS:
;       STR_IN array must be defined and have valid data.
;        NOTE:  Results with multi block data may not be satisfactory 
;               along block boundaries.
;
; TECHNIQUE:
;       IDL procedures SMOOTH and MEDIAN are used to filter each block of a 
;       type 1 field structure.  Loops are used.
;
; EXAMPLES:
;       Smooth structure 2 with a boxcar filter.
;            SMOSTR, 2
;       Smooth structure 2 with a boxcar filter of width 7 and store the result
;         in structure 3.
;            SMOSTR, 2, 3, width = 7
;       Smooth structure 2 with a boxcar filter of width 7 and store the result
;         in structure 3. Use Edge Truncation.
;            SMOSTR, 2, 3, width = 7, /edge
;       Smooth structure 2 with a median filter of width 7 and store the result
;         in structure 3.
;            SMOSTR, 2, 3, width = 7, /median
;       Smooth structure density a median filter of width 3 and store the result
;         in structure smodensity.
;            SMOSTR, density, smodensity, /median
;       Smooth structure 2 with a median filter of width 40 in the Y direction
;       Smooth the result with a median filter of width 3 in both directions
;       Store the result in structure 3.
;            SMOSTR, 2, 3, width = 40, /yonly, /median
;            SMOSTR, 3, /med
;       
; MODIFICATION HISTORY:
;      Original version, L. P. Mix, September 18, 2001
;-
;
on_error, 1
;     include the common block
;
@qscom.dat
;  Common block
@qsstrcom.dat
;
      if n_elements (structinit) eq 0 then qsstruct_init
      if n_elements (qsplotinit) eq 0 then qsinit
;
      nparams = n_params()
      if nparams lt 1 then begin
        print,'% Format: SMOSTR, STR_IN [, STR_OUT]
        print,'           [, /MEDIAN] [, WIDTH = width] [, _EXTRA = extrastuff]
        return
      endif
;
;     determine if strarray is specified
;
      nelstr = n_elements (strarray)
      if nelstr eq 0 then begin
        print, '% SMOSTR: Structure must be defined'
        return
      endif
      array = strarray
      b = size(strarray)
      if b(b(0)+1) ne 8 then begin
        a = strarray
        if a gt 0 and a le maxstructure then begin
          s2i, array, a
          b = size (array)
          if b(b(0)+1) ne 8 then begin
    print, '% SMOSTR: First parameter must be a structure or a QS array number'
            return
          endif
        endif  else begin
          print, ' First parameter must be a structure or a QS array number'
          return
        endelse
      endif else a = 0
      if array.type ne 1 then begin
        print, '% SMOSTR: Only type 1 structures may be filtered'
        return
      endif 
      if n_elements(width) eq 0 then width = 3 else width = width > 3
      nblo = array.block
      nvect = array.nvect
      dim = array.nvect
      if dim gt 2 then begin
        xonly = 0
        yonly = 0
      endif
      
      for i = 1, nblo do begin
        for j = 1, nvect do begin
          arr = reform(get_array(array, -j, i, lab = lab))
          ss = size(arr)
          if keyword_set(Xonly) then begin
            if width lt ss[1] then begin
              if keyword_set(median) then for ii = 0L, ss[2]-1 do arr[*,ii] = $
                 median(reform(arr[*, ii]), width, _extra= extrastuff) else $
                 for ii = 0L, ss[2]-1 do $
                arr[*,ii]= smooth(reform(arr[*,ii]), width, _extra= extrastuff) 
              put_array, array, arr, -j, i, lab = lab
            endif
          endif else if keyword_set(Yonly) then begin
            if width lt ss[2] then begin
              if keyword_set(median) then for ii = 0L, ss[1]-1 do arr[ii, *] = $
                 median(reform(arr[ii, *]), width, _extra= extrastuff) else $
                 for ii = 0L, ss[1]-1 do $
                arr[ii,*]= smooth(reform(arr[ii,*]), width, _extra= extrastuff) 
              put_array, array, arr, -j, i, lab = lab
            endif
          endif else if width lt min(ss[1:ss[0]]) then begin
            if keyword_set(median) then $
               arr = median(arr, width, _extra= extrastuff) $
            else arr = smooth(arr, width, _extra= extrastuff)
            put_array, array, arr, -j, i, lab = lab
	  endif
        endfor
      endfor
      if a gt 0 then begin 
        if nparams gt 1 then save = get_save_str(SavSTR, a) else save = a
        i2s, array, save, /no_copy
      endif else $
        if nparams gt 1 then savstr = temporary(array) else $
          strarray = temporary(array)
      end
