; ------------------------------------------------------------------------
; $Id: strfree.pro,v 1.2 2010/09/08 16:57:57 lpmix Exp $
; ------------------------------------------------------------------------
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
;+
; NAME:
;       STRFree
;
; PURPOSE:
;       Return free structure array numbers, that is a list of structure array numbers
;         with no data associated with them.
;       Note:  USE DELSTR to eliminate data from structure arrays. 
;
; CALLING SEQUENCE:
;       Free_STR = STRFree(MINIMUM)
;
; OUTPUTS:
;       FREE_STR: An array of structure values which are available
;                 The scalar -1 is returned if none are available.
;
; INPUTS:
;        MINIMUM:   This is the minimum structure array desired
;                   If present, MINIMUM keyword is ignored
;
; KEYWORD PARAMETERS:
;       NElements:  Number if values in FREE_STR
;       MINIMUM:    Minimum value of structure values desired.
;                   MINIMUM must satisfy: MINIMUM = (MIMIMUM > 1) < GET_MAXSTR()
;       MAXIMUM:    Maximum value of structure values desired.
;                   MAXIMUM must satisfy: MAXIMUM = (MAXIMUM > MINIMUM) < GET_MAXSTR()
;
; OUTPUTS: Array of unused structure array numbers.
;
; PROCEDURE:
;       STR uses an internal array to determine STR arrays with no data.
;
; EXAMPLE:
;       Determine arrays that are free between 10 and 100; store in FREE
;       Return the number of values in nel
;         FREE = STRFREE(MINIMUM = 10, MAXIMUM = 100, NELEMENTS = nel)
;   or    FREE = STRFREE( 10, MAXIMUM = 100, NELEMENTS = nel)
;
; MODIFICATION HISTORY:
; 	Written by:	L. P. Mix, Jr., September 12, 2008
;-

Function STRFREE, a, minimum = minimum, maximum = maximum, nelements = nelements

on_error, 1
@qsstrcom.dat
;
; Set default value
if n_elements(structinit) eq 0 then qsstruct_init
if n_elements(a) gt 0 then a = (a[0]  > 1) < maxstructure else begin
  if n_elements(minimum) eq 0 then a = 1 else a = (minimum > 1) < maxstructure
endelse
if n_elements(maximum) eq 0 then b = maxstructure else b = (maximum > a) < maxstructure
answ = where(lcvalidstr[a-1:b-1] eq 0, nelements)
answ = (nelements eq 0) ? -1 : answ+a
return, answ

END
