; ------------------------------------------------------------------------ 
; $Id: strvalid.pro,v 1.4 2010/09/17 20:51:05 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
 Function strvalid,stringin, maximum=maximum, minimum=minimum, $
       case_sensitive = casesen, npoint= npoint, quiet = quiet, $
       valid = valid
;+
; Name: STRVALID
;
; NOTE: VALID is a flag to determine the mode of this routine.
;
;     if keyword(valid) is not set.
;     This function returns an array of valid structure array numbers.
;     Minimum , maximum and a search string can be specified.
; or
;     if keyword(valid) is set.
;     This function returns an array of valid structure array number
;     from a proposed array of numbers.
;
; Format:
;   array = STRVALID([STRING] , [Maximum = maximum] [,Miminum = minimum]
;           [,Npoint = npoint] [,case_sensitive = case_sensitive])
;           [, VALID = valid] [, QUIET = quiet]
;
;   where:
;    IF VALID is zero or omitted:
;         STRING is a search string. Leading and trailing blanks are ignored.
;                String may be a number. This routine uses:
;                       str = strtrim(string(0), 2)
;                  where str is the actual search string
;                For example s = strvalid(24.50) will return 
;                   a structure with a comment   'Efield, t = 24.50 ns'
;                If missing, all valid arrays are returned.
;                See GET_MATCH for available wild card options.
;         Minimum is the minimum value desired, Default = 1
;         Maximum is the maximum value desired, Default = maxstructure
;           Current version will sort maximum and minimum.
;         Npoint is the number of values returned.
;         Case_sensitive if set makes the search case sensitive.
;         CAUTION: CASE is a special word so use /c or /ca or /cas or /case_
;                  or more letters but /case will give an error.
;    IF VALID is not zero:
;         STRING  - an array of possible structure arrays.
;                   Must be a byte, integer, long or a float variable.
;                   Floats are converted to LONG using ROUND
;                   This array of 1 or more numbers is not optional if 
;                      VALID is set(ie not equal to zero or omitted).
;                   See GET_MATCH for wild card options
;                   or
;                   A search string. If an array, only the first element is used.
;         NPOINT  - the number of values returned.
;         QUIET   - if set, not output will be generated.
;                   if not set, messages will be printed if STRING contains
;                     invalid structure arrays.
;                    Messages of the form if VALID is a string:
;                   "% "+STRTRIM(VALID, 2)+": "+ message
;              or   "% INVALID STR: " + message
;           where message =
;                Structue array number is UNDEFINED
;         or     Structure array must be a byte, integer, long, or float value 
;         or     Structure array number must be greater than 0 and </= MX
;         or     No data in N structure array 
;  where MX is the maximum number of arrays and N is the number of arrays.
;
;   For both states of VALID, if NPOINT = 0 then ARRAY = -1
;
;   Example:
;      find all Structure arrays containing data and store in VALS
;          VALS = strvalid()
;      find Sturcture arrays with 'e in inlet' and print the dataset
;          comments only.
;          SHELP, strvalid('e in inlet'), full = 0
;
;      determine if strin is a valid WDF array with data. Do not print errors.
;          strgood = strvalid(strin, /valid, /quiet, npoint = npoint)
;      Same as above but print an error message beginning with 'MY_ADD_IT'
;          strgood = strvalid(strin, npoint = npoint, valid = 'MY_ADD_IT')
;-
@qsstrcom.dat
on_error, 1
      npoint = 0
      ok = -1
      if n_elements(structinit) eq 0 then return, ok
      if keyword_set (valid) then begin
        nel = n_elements (stringin)
        if nel eq 0 then begin
          if not (keyword_set(quiet)) then begin
            if (size(valid(0)))(1) eq 7 then print, '% '+ strtrim(valid(0), 2) $
              + ': Structure array is UNDEFINED' else print, '% INVALID STR' $
              + ': Structure array is UNDEFINED'
          endif
          return, ok
        endif
        typin = (size(stringin(0)))(1)
        if typin eq 7 then begin
          str = strtrim(stringin(0), 2)
          if strlen(str) gt 0 then $
            ok = get_match(str, /str, nelement=npoint, casesen=casesen)
          if npoint gt 0  then return, ok
          if not keyword_set(quiet) then begin
            if (size(valid(0)))(1) eq 7 then print, '% '+ strtrim(valid(0), 2) $
              + ': No structures with the string: ' + str $
                else print, '% INVALID STR' $
              + ': No structures with the string: ' + str
          endif
          return, -1
        endif else if typin gt 4 then begin
          if not (keyword_set(quiet)) then begin
            if (size(valid(0)))(1) eq 7 then print, '% '+ strtrim(valid(0), 2) $
             + ': Structure array must be a byte, integer, long, or float value' $
               else print, '% INVALID STR: ' $
              + 'Structure array must be a byte, integer, long, or float value'
          endif
          return, ok
        endif 
        if typin eq 4 then okin = round (stringin) else okin = fix(stringin)
        ok = where(okin gt 0 and okin le maxstructure, npoint)
        if npoint lt nel and not keyword_set(quiet) then begin
            if (size(valid(0)))(1) eq 7 then print, '% '+ strtrim(valid(0), 2) $
              + ': Structure array number must be greater than 0 and </= ',  $
               strtrim(maxstructure, 2) else print, '% INVALID STR: ' $
              + 'Structure array number must be greater than 0 and </= ',  $
               strtrim(maxstructure, 2)
            print, nel - npoint, ' Structure arrays are invalid'
        endif
        if npoint eq 0 then return, ok
        if npoint lt nel then begin
          nel = npoint
          okin = okin(ok)
        endif
        ok = where(lcvalidstr(okin-1) gt 0, npoint)
        if npoint lt nel and not keyword_set(quiet) then begin
            if (size(valid(0)))(1) eq 7 then print, '% '+ strtrim(valid(0), 2) $
              + ': No valid data in ', strtrim(nel-npoint, 2), ' arrays'  $
                else print, '% INVALID STR: ' $
              +  'No valid data in ', strtrim(nel-npoint, 2), ' arrays'  
        endif
        if npoint eq 0 then return, ok
        if npoint lt nel then okin= okin(ok)
        return, okin
      endif
      if n_elements (minimum) eq 0 then minimum = 0 else $
         minimum = (minimum(0)-1)> 0
      if n_elements (maximum) eq 0 then maximum = maxstructure-1 else $
         maximum = (maximum(0) < maxstructure)-1
      sb = min([minimum, maximum], max = se)

      ok = where(lcvalidstr(sb:se) gt 0, npoint)+1 + sb
      if npoint eq 0 then begin
        ok = -1
        return, ok
      endif
      if n_elements(stringin) eq 0 then return, ok
      str = strtrim(stringin(0), 2)
      if strlen(str) eq 0 then return, ok
      return, get_match(str, structure=[sb, se]+1, casesen=casesen)
      end
