; ------------------------------------------------------------------------ 
; $Id: t2w.pro,v 1.3 2007/06/06 20:09:38 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
;+
; NAME:
;       T2W
;
; PURPOSE:
;       Convert times in a Time-Dependent Finite-element Structure array 
;         to a WDF array
;       or 
;       Convert times in a netCDF or PDB file to a WDF array
;
; CALLING SEQUENCE:
;       T2W, STR [, WDF]
;
; INPUTS:
;       STR:  Structure or structure array number of a type 9 structure 
;             Taken as WDF if a keyword is set.
;       WDF:  WDF array number to store the function
;             Default: WDF = Get_MaxWDF()
;
; KEYWORD PARAMETERS:
;       CDF:  If set, Time from the current netCDF file is taken
;             If set, STR is taken as WDF 
;       PDB:  If set, Time from the current PDB file is taken
;             If set, STR is taken as WDF 
;     Note: If CDF is set, PDB is not checked.
;           If PDB is set, STR is assumed to be the WDF.
;           If no keywords are set, STR is assumed to be a structure.
;
; OUTPUTS:
;       Store the time data in a WDF array.
;
; COMMON BLOCKS:
;       PDB and CDF common blocks are available
;
; RESTRICTIONS:
;       A PDB or netCDF file must be open or a time dependent finite element
;       array must be in memory.
;
; EXAMPLE:
;       Convert the time array for structure 6 to WDF array #4.
;          T2W, 6, 4
;
;       Store the time values for the current PDB file in the maximum WDF array.
;          T2W, /pdb
;
;       Store the time values for the current netCDF file in WDF array 4
;          T2W, 4, /cdf
;
; MODIFICATION HISTORY:
; 	Written by:	L. P. Mix, December 22, 1999
;-

 PRO T2W, strin, wdf, cdf = cdf, pdb = pdb

 on_error, 1

;
;  Common block
@qscom.dat
@qsstrcom.dat
@com.dat
@netcdfcom.dat
;
;
      if n_elements (initwdf) eq 0 then wdfinit
      if n_elements (structinit) eq 0 then qsstruct_init
      if n_elements (qsplotinit) eq 0 then qsinit
;
      on_error, 1
      npar_kw = keyword_set(cdf) + keyword_set(pdb)
      npar = n_params() + npar_kw
      if npar eq 0 then begin
        print,'% Format: T2W, IDL_Structure, WDF_Array_Number [, /CDF ][, /PDB]'
        return
      endif
      snum = 0
      if npar_kw eq 0 then begin
        if n_elements(strin) eq 0 then begin
          print, '% T2W: Structure array must be defined'
          return
        endif else str = strin[0]
        if n_elements (wdf) ne 0 then begin
          if ((wdf[0] lt 1) or (wdf[0] gt maxwdfarray)) then $
            wdf = maxwdfarray else wdf= wdf[0]
        endif else wdf = maxwdfarray
        b = size(str)
        if b[b[0]+1] ne 8 then begin
          if str gt 0 and str lt maxstructure then begin
            snum = str
            s2i, str, snum, /no_copy
            b = size(str)
            if b[b[0]+1] ne 8 then begin
       print, '% T2W: First parameter must be a valid structure or number'
              return
            endif
          endif else begin
            print, '% T2W: First parameter must be a valid structure number'
            return
          endelse
        endif
        if (str.type ne 9) then begin
          print, '% T2W: Structure must be a time dependent, FE structure.
          if snum gt 0 then i2s, str, snum, /no_copy
          return
        endif
        i2w, findgen(str.ntime)+1, str.x4, wdf, xlab = 'Time Index (1:ntime)', $
          ylab = 'Time', clab = 'Time vs Index - '+str.clab
        if snum gt 0 then i2s, str, snum, /no_copy
        return
      endif
      if keyword_set (cdf) then begin
        if n_elements(initcdf) eq 0 then begin
          print, '% T2W: Open a CDF file'
          return
        endif
;
        if cdf_id  lt 0 then begin
          print, '% T2W: No netCDF files are open.'
          return
        endif

        if lcdfvalid ne array_cdfid(cdf_id, 1) then load_cdf_data

        if cdf_type(cdf_id) eq 1 then begin
          x4 = time_whole
          nt = n_elements(x4)
          if n_elements (strin) ne 0 then begin
            str = strin[0]
            if ((str lt 1) or (str gt maxwdfarray)) then str = maxwdfarray
          endif else str = maxwdfarray

          i2w, findgen(nt)+1, x4, str, xlab = 'Time Index (1:nt)', $
            ylab = 'Time', clab = 'Time vs Index - '+file_title(cdf_id)
;
        endif else print, '% T2W: Only type 1 netCDF data supported'
        return
      endif
;     pdb file
      ;
      if initpdb  le 0 then begin
        print, '% T2W: No netPDB files are open.'
        return
      endif

      If curfid lt 0 then begin
        print, '% T2W:  No valid PDB files are open.'
        return
      endif
      info = file_info(curfid)
; get the standard stuff
      time = info.time
      if not keyword_set(double) then time = float(time)
      ntime = n_elements(time)
      if n_elements (strin) ne 0 then begin
        str = strin[0]
        if ((str lt 1) or (str gt maxwdfarray)) then str = maxwdfarray 
      endif else str = maxwdfarray

      i2w, findgen(ntime)+1, x4, str, xlab = 'Time Index (1:ntime)', $
        ylab = 'Time', clab = 'Time vs Index - '+info.file

      end
