; ------------------------------------------------------------------------ 
; $Id: thermometer.pro,v 1.4 2007/06/06 20:09:38 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
 Pro thermometer, amplitude, col_values, title = title, color= color, $ 
          position = position,  data = data, normal = normal, $
          tickvalue = tickvalue, horizontal = horizontal, txsize = txsize, $
          tysize = tysize, bwhite = bwhite, cursor = cursor,ignore = ignore,  $
          box = box, bfill = bfill,  cbox = cbox, absolute = absolute, $
          ovamp = ovamp, ovcol = ovcol, $
          thick = thick, charsize = charsize, _extra = extrastuff
;+
;   Produce a thermometer type legend for a contour plot
;   Unless the position keyword is used the thermometer will be on the right
;     end or top of the current plot.
;   Internals: The command uses a contour plot with X = [0, 1] and 
;              Y = Amplitude with the c_colors given by col_values, and 
;              FILL = 1 (See Procedure below)
;   The values for the !x and !y variables are saved when this procedure
;              is called and are restored at completion.
;   format: thermometer, amplitude, col_values, title = title, color= color, $ 
;           position = position, /data, /horizontal,  absolute = absolute, 
;           txsize = txsize, tysize = tysize, tickvalue = tickvalue, 
;           cursor = cursor, bwhite = bwhite, ignore = ignore, 
;           box = box, bfill = bfill,  cbox = cbox,
;           thick = thick, charsize = charsize, _extra = extrastuff
;   where
;         AMPLITUDE - Array of values - typically levels of the contour plot
;         Col_Values - Array of color values -
;                     typically levels of the contour plot
;         Note: Amplitude and Col-Values must have the same number of values.
;         Exception  - If Amplitude has 3 values and Col_Values has 2 and 
;                      amplitude(1) < amplitude(0) < amplitude(2) then
;                      a two color thermometer will be drawn with col_values(0) 
;                      below (or to the left of) amplitude(0) and 
;                      col_values(1) above (or to the right of)amplitude(0)
;                      See also /bwhite
;         BWHITE    - If set, col_values is set to [!p.color, !p.background]
;         COLOR     - Color for printing
;                     Default is !p.color (white for X and Black for PostScript)
;                     Values are: 1, red; 2, green; 3, blue; 4, magenta;
;                     5, orange; 6, cyan; 7, yellow; 8, white; 0, black
;         CURSOR    - If set the two opposite corners of the thermometer will 
;                     be set with a cursor.  Normalized position will be 
;                     returned in position.
;         POSITION  - Position on the plot for the thermometer in units
;                     determined by keywords. If position has valid data,
;                     TXsize and TYsize are ignored.
;                     If POSITION is undefined then the default is determined
;                        by the value of txsize and tysize
;                     POSITION will be returned with the values in normal 
;                        coordinates 
;         IGNORE    - Thermometer trys to adjust the title position to fit.
;                     If it moves it to a bad location, use ignore.
;                     If ignore is not equal to 1, then the position will be
;                     moved an amount left or right given by the difference
;                     between ignore and 1
;                     Specifically: If middle is the location of the default
;                     middle of the title, then middle is changed according to
;                          middle = middle +(ignore -1.0)
;         /data     - Determines the coordinates for the position keywork
;                     Data refers to the plot coordinates 
;                     If keyword is not set, then normalized space coordinates
;                     are used.
;            NOTE:  Normalized coordinates are relative to the plot not
;                   then entire screen, ie, x = 1 will be the middle of the
;                   screen or page if !p.multi (1) equals 2
;         ABSOLUTE  - If present and not zero then position is normalized 
;                     coordinates relative to the screen not the last plot
;         /HORIZONTAL If keyword is set, thermometer will be horizontal.
;                     If set, default value of TXsize = [.5, .9] and 
;                     TYSIZE =[.3, .6]*(!y.window(0)-!y.region(0))+!y.region(0)
;                     Default is vertical.
;         TXsize    - If position is not specified, this parameter determines
;                     the horizontal size of the thermometer and optionally the 
;                     positional x values if TXsize is a two element vector.
;                     Size in window units is TXsize*(!x.region(1)-!x.window(1))
;                     Default is TXsize = 0.3
;                     Default position is against the right border.
;                     If two values are given, these are used as the two 
;                     positional x parameters.
;         TYsize     If position is not specified, this parameter determines the
;                     vertical size of the thermometer and optionally the 
;                     positional y values if TYsize is a two element vector.
;                     If a single parameter is given, the height of the
;                     thermometer will be given by:
;                      TYsize*(!y.window(1)-!y.window(0))
;                     Default value is TYsize = 0.6
;                     Default position is vertically centered.
;                     If two values are given, these are used as the two 
;                     positional y parameters.
;         Example:
;                     TXsize and TYsize are two element arrays.
;                     position = [txsize(0), tysize(0), txsize(1), tysize(1)]
;         Thick     - parameter sent to plot, changes the thickness of the frame
;         Charsize  - parameter sent to plot.
;                     default is 1/max(!p.multi(1:2)-1)>1
;         Box       - Draw a box around the thermometer if set.
;                   - If Box has 4 elements, they will be taken as the position
;                     of the box.
;                     Otherwise the size will be estimated.
;                     The color will be specified by CBOX
;         CBOX      - Color of the BOX. Default is COLOR
;         BFILL     - Fill the box with the specified color.
;                     Note to erase an area: BFILL = !p.background
;         Title     - Title for the thermometer. This may be a string array.
;                     Each element is printed on separate lines. 
;                     For a vertical thermometer the title will be centered over
;                     an extension of the Y axis unless it will entend past the
;                     edge of the plot region; otherwise it will be move left or
;                     right to lie with the plot region.
;                     For a horizontal thermometer, the title will be centered
;                     over the thermometer.
;         Tickvalue - Value where thermometer values are to be printed.
;                     use /xstyle for horizontal and /ystyle for normal 
;                     thermometers if exact scaling is desired.
;         extrastuff- Any valid parameters for the plot command
;                     xstyle or ystyle = 1 will force exact scaling
;      Procedure:
;     Set up the position if not specified and:       
;          plot, amplitude, /nodata, position = position, $
;            color = color, _extra = extrastuff, $
;            xtickname =[' ', ' '], xticks= 1, xticklen = 1e-3 
;     Determine the number of number and repeat as required the following:
;          contour, data, [0, 1], amplitude, position= position, $
;            color = color, c_color= col_values, levels = amplitude, $
;            thick = thick
;
;         Example:
;           Add a thermometer to a contour plot of data in structure 23.
;             plotstr, 23, /thermometer 
;
;           Add a thermometer to a E field plot from str 2 and scale the data.
;           Take Er and slice the data at theta = 0.0 and plot 25 levels.
;             plotstr,2, 1, 2, 0.0, /thermometer, m = 1e-8, nl =25, $
;                ther ={title:['Er Field', '(MV/cm)']}, $
;             title= 'Radial Electric Field'
;        or 
;            lev = 0 &  color = -1 
;            plotfld, 2, 1, 2, 0.0, color = color, nl = 25, level= lev, $
;              xmargin= [10, 10], title= 'Radial Electric Field'
;            thermometer, lev*1e-8, color, title =['Er Field', '(MV/cm)']
;
;          Add a thermometer to a plot with a range of [0, 10] and a value
;            of 5.6 and color 1 below 5.6 and color 2 above 5.6; make a two
;            line title of 'Diode Voltage (MV)'; use the cursor to place.
;
;            thermometer, [5.6, 0, 10], [1,2], tit=['Diode','Voltage (MV)'], /cur
;
;          Note: to change to horizontal merely add the ", /horizontal" keyword.
;          To make a black and white thermometer merely change [1, 2] to 
;             [!p.color, !p.background] or set the keyword /bwhite as:
;          thermometer,  [5.6, 0, 10], /bw, tit=['Diode', 'Voltage (MV)'], /cur
;
;          Add a black and white thermometer to a plot which has a value of 
;          volt(i) and ranges from 0 to 5.
;            thermometer, [volt(i), 0, 5], /bw
;-
  on_error, 1
;
      npar = n_params()
      if npar eq 0 then begin
print, '%Format: THERMOMETER, Levels, Col_values [, COLOR = color]'
print,'    [, POSITION = position] [, /DATA] [, /BWHITE] [, /ABSOLUTE]'
print,'    [, TXSIZE = txsize] [, TYSIZE = tysize] [, /HORIZONTAL]'
print,'    [, THICK = thick] [, TITLE = nextpos] [, TICKVALUE = tickvalue]'
print,'    [, BOX = box] [, BFILL = bfill] [, CBOX = cbox] [, IGNORE = ignore]'
print,'    [, POSITION = position] [, CHARSIZE= charsize] [, _EXTRA= extrastuff]'
        return
      endif
;
; default parameters
;
      width_default = 0.2
      length_default = 0.5
      char_default = 0.8
; save current values
      x_str_save = !x
      y_str_save = !y
      p_str_save = !p
;
      nlevels  = n_elements(amplitude)
      if keyword_set(bwhite) then col_values = [!p.color, !p.background]
      ncol = n_elements (col_values)
      if ((nlevels eq 0) or (ncol eq 0)) then begin
        stat= widget_message('The amplitude and color values must be defined', $
           /error)
        return
      end
      if nlevels eq 3 and ncol eq 2 then begin
        if amplitude (0) ge amplitude(1) and amplitude(0) le amplitude(2) $
          then begin
          amp = [amplitude(1), amplitude(2)]
          levels = [amplitude(1), amplitude(0)]
        endif else begin
        stat= widget_message('The amplitude and color values must be defined'+ $
          ' with the same number of values', /error)
          return
        endelse
      endif else if (nlevels ne ncol) then begin
        stat= widget_message('The amplitude and color values must be defined'+ $
          ' with the same number of values', /error)
        return
      end else begin
        amp = [min(amplitude, max = mm), mm]
        ok = sort(amplitude)
        levels = amplitude(ok)
        col_val= col_values(ok)
      endelse
      chy = float(!d.y_ch_size)/!d.y_size
      chx = float(!d.x_ch_size)/!d.x_size
      chxh = 8*chx
;
;     lets take care of positioning
;
      if ((!d.name eq 'PS') and (keyword_set(cursor))) then begin
          cursor = 0
          winformation, ['Cursor may not be used with postscript', $
             'Default positioning will be used.']
      endif
      if keyword_set(cursor) then begin
        print, 'Enter one corner of the color thermometer'
        cursor, x, y, /down, /normal
        mksymbol, 'plus'
        plots, x, y, psym = 8, /normal
        device , set_graphics = 6
        plots, [x, x], !y.region, /normal
        plots, !x.region,[y,y], /normal
        print, 'Enter opposite corner of the color thermometer'
        cursor, xx, yy, /down, /normal
        plots, [x, x], !y.region, /normal
        plots, !x.region,[y,y], /normal
        device , set_graphics = 3
        plots, x, y, psym = 8, /normal, color = !p.background
        data = 0
        position = [ x< xx, y< yy, x > xx, y> yy]
      endif else if n_elements(position) eq 4 then begin
        position = reform(position, 2, 2, /overwrite)
        if not keyword_set(absolute) then begin
          if keyword_set (data) then  begin
            position = convert_coord(temporary(position), /data, /to_normal)
            position = position(0:1, *)
          endif else begin
            position(0,*)=(!x.region(1)-!x.region(0))*position(0,*)+!x.region(0)
            position(1,*)=(!y.region(1)-!y.region(0))*position(1,*)+!y.region(0)
          endelse
        endif
        position = reform(position, 4, /overwrite)
      endif else begin
        ntx = n_elements(txsize)
        if ntx lt 2 then begin
          if keyword_set(horizontal) then begin
            if ntx eq 0 then txsize = length_default
            txsize = txsize*(!x.region(1)-!x.region(0)) 
            txsize= [!x.region(1)-chxh-txsize, !x.region(1)-chxh]
          endif else begin
            if ntx eq 0 then txsize = width_default
            txsize = txsize*(!x.region(1)-!x.window(1))> chx
            txsize = [!x.region(1)-txsize, !x.region(1)] - 1.5*chx
          endelse
        endif else begin
          if keyword_set(data) then begin
            txsize = (txsize-!x.crange(0))/(!x.crange(1)-!x.crange(0))* $
            (!x.window(1)-!x.window(0)) + !x.window(0)
          endif else begin
            txsize = txsize*(!x.window(1)-!x.window(0)) + !x.window(0)
          endelse
        endelse
        ntx =  n_elements(tysize)
        if ntx lt 2 then begin
          if keyword_set(horizontal) then begin
            if ntx eq 0 then tysize = width_default
            tysize = tysize*(!y.window(0)-!y.region(0)) > chx
            tysize= [tysize, tysize + tysize] +(!y.region(0)+chy)
          endif else begin
            if ntx eq 0 then tysize = length_default
            ytot = (!y.window(1)-!y.window(0))
            tysize = tysize*(!y.window(1)-!y.window(0))
            tysize = [(ytot-tysize)*0.5, (ytot+tysize)*0.5] + !y.window(0)
          endelse
        endif else begin
          if keyword_set(data) then begin
            tysize = (tysize-!y.crange(0))/(!y.crange(1)-!y.crange(0))* $
            (!y.window(1)-!y.window(0)) + !y.window(0)
          endif else begin
            tysize = tysize*(!y.window(1)-!y.window(0)) + !y.window(0)
          endelse
        endelse
;
        position = [txsize(0), tysize(0), txsize(1), tysize(1)]
;
      endelse
      if n_elements (charsize) eq 0 then chars =char_default $
        else chars = charsize*char_default
      if n_elements (color) eq 0 then color = !p.color

      neltit = n_elements(title)
      maxstr = 0
      if neltit gt 0 then begin
        tit = title
        for i = 0, neltit-1 do tit(i) = strtrim(title(i), 2)
        maxstr = max(strlen(tit))
        if maxstr gt 0 then begin
          if !p.multi[1] gt 2 or !p.multi[2] gt 2 then chart = chars*0.6 $
             else chart = 1.2*chars
          space = 1.25*chy*chart
          start = position(3)+space*0.5
          ytitle = rotate(findgen(neltit)*space +start, 2)
          middle = (position(0)+position(2))*0.5
     if ( keyword_set(ignore)) then begin
       if ignore ne 1 then middle = middle +ignore -1
     endif else begin
          if !p.font eq 0 then halflength = maxstr*chx*0.6*chars else begin
             mm = make_array(neltit)
             for i = 0, neltit-1 do begin
               xyouts, /nor, .5, .5, chars = -1, width = wid, tit(i)
               mm(i) = wid
             endfor
             halflength = max(mm)*chart *0.5
          endelse
          if middle - halflength lt !x.region(0) then begin
            middle = !x.region(0) + halflength + chx*0.8
            minx = chx*0.8
            maxx = middle +halflength
          endif else if middle + halflength gt !x.region(1) then begin
            middle = !x.region(1) - halflength - chx*0.8
            maxx = !x.region(1)-chx*0.8
            minx = middle -halflength
          endif else begin
            minx = middle -halflength
            maxx = middle +halflength
          endelse 
          middle = make_array(neltit, value = middle)
;           xyouts, middle, ytitle, tit, /nor, chars= chars*1.2, $
;             align = 0.5, color = color
        endelse
      endif
      endif
;
      nelbox = n_elements (box) 
      if nelbox gt 0 then begin
        if nelbox eq 4 then begin
          bpos = box
          bpos = reform(bpos, 2, 2, /overwrite)
          if keyword_set (data) then  begin
            bpos = convert_coord(temporary(bpos), /data, /to_normal)
            bpos = bpos(0:1, *)
          endif else begin
            bpos(0,*) =(!x.region(1)-!x.region(0))*bpos(0,*)+!x.region(0)
            bpos(1,*) =(!y.region(1)-!y.region(0))*bpos(1,*)+!y.region(0)
          endelse
          bpos = reform(bpos, 4, /overwrite)
        endif else if box (0) ne 0 then begin
          if maxstr gt 0 then begin
            bpos = [position(0)-7.5*chx*chars< minx , position(1)-space, $
              position(2)+chx> maxx, position(3) + (neltit+1)*space]
          endif else $
            bpos = [position(0)-7.5*chx*chars , position(1)-chy*chars, $
              position(2)+chx, position(3)+ chy*chars]
        endif else box = 0
      endif
;
      if keyword_set(box) then begin
        if n_elements(bfill) gt 0 then $
          polyfill, [bpos(0), bpos(2), bpos(2), bpos(0), bpos(0)], $
              [bpos(1), bpos(1), bpos(3), bpos(3), bpos(1)], $
              /normal, color = abs(bfill)
        if n_elements(cbox) eq 0 then cbox = color
        plots, [bpos(0), bpos(2), bpos(2), bpos(0), bpos(0)], $
              [bpos(1), bpos(1), bpos(3), bpos(3), bpos(1)], $
              /normal, color = cbox
      endif
      if n_elements(tickvalue) eq 0 then tickvalue = 0
      ticks = n_elements(tickvalue)-1
      nelamp = n_elements(levels)
      !p.multi(0) = !p.multi(0)+1
      
      if keyword_set (horizontal) then begin
        y = [0, 1]
        plot, amp, y, /nodata, position = position, charsize = chars, $
          color = color, xtickv = tickvalue, xticks = ticks, $
          ytickname =[' ', ' '], yticks= 1, yticklen = 1.0e-7, $
          xticklen = 1.0e-7, xrange = [min(amp,max = mmm), mmm], $
           _extra = extrastuff
        crange = !x.crange
        if !x.type ne 0 then levels = alog10 (levels)
        xpos =round( !d.x_size* [position(0), position(2)])
        ypos =round( !d.y_size* [position(1), position(3)])

        xsizp = xpos(1)-xpos(0)+1
        ysizp = ypos(1)-ypos(0)+1
        if (!d.flags and 1) ne 0 then begin
;         scalable pixels
          pden = 50
          if !d.x_px_cm gt pden then xsiz = round(xsizp*pden/!d.x_px_cm) $
                else xsiz = xsizp
          if !d.y_px_cm gt pden then ysiz = round(ysizp*pden/!d.y_px_cm) $
                else ysiz = ysizp
        endif else begin
          xsiz = xsizp
          ysiz = ysizp
        endelse
        yy = findgen(xsiz) *(crange(1)-crange(0))/(xsiz-1)+crange(0)
        ok = where(yy gt levels(0) and yy le levels(nelamp-1), nok)
        ytemp = long(get_yarr(levels, findgen(nelamp), yy(ok)))
        yy(ok) = interpolate(col_val, ytemp, /grid)
        if ok(0) gt 0 then yy(0:ok(0)-1)= !p.background
        if ok(nok-1) lt xsiz-1 then yy(ok(nok-1)+1:xsiz-1) = col_val(nelamp-1)
        yy = byte((round(yy)> 0) < 255)
        image = make_array(/byt, xsiz, ysiz)
        for i = 0, ysiz-1 do image[*, i]=yy
        tv, image, xpos(0), ypos(0) , xsiz = xsizp, ysiz = ysizp

        y = [0, 1]
        !p.multi(0) = !p.multi(0)+1
        plot, amp, y, /nodata, position = position, charsize = chars, $
          color = color, xtickv = tickvalue, xticks = ticks, $
          ytickname =[' ', ' '], yticks= 1, yticklen = 1.0e-7, $
          xticklen = .18, xrange = [min(amp,max = mmm), mmm], $
           _extra = extrastuff
        nelov = n_elements(ovamp)
        if nelov gt 1 then begin
          for i = 0, nelov-1 do $
            oplot, [ovamp(i), ovamp(i)], y, col = ovcol(i)
        endif
      endif else begin
        x = [0, 1]
        plot, x, amp, /nodata, position = position, charsize = chars, $
          color = color, ytickv = tickvalue, yticks = ticks, $
          xtickname =[' ', ' '], xticks= 1, yticklen = 1.0e-7, $
          xticklen = 1.0e-7, yrange =  [min(amp,max = mmm), mmm],$ 
           _extra = extrastuff
        crange = !y.crange
        if !y.type ne 0 then levels = alog10 (levels)
        xpos =round( !d.x_size* [position(0), position(2)])
        ypos =round( !d.y_size* [position(1), position(3)])
        xsizp = xpos(1)-xpos(0)+1
        ysizp = ypos(1)-ypos(0)+1
        if (!d.flags and 1) ne 0 then begin
;         scalable pixels
          pden = 50
          if !d.x_px_cm gt pden then xsiz = round(xsizp*pden/!d.x_px_cm) $
                else xsiz = xsizp
          if !d.y_px_cm gt pden then ysiz = round(ysizp*pden/!d.y_px_cm) $
                else ysiz = ysizp
        endif else begin
          xsiz = xsizp
          ysiz = ysizp
        endelse
        yy = findgen(ysiz) *(crange(1)-crange(0))/(ysiz-1)+crange(0)
        ok = where(yy gt levels(0) and yy le levels(nelamp-1), nok)
        if nok eq 0 then begin
          !x = x_str_save
          !y = y_str_save
          !p = p_str_save
          return
        endif
        ytemp = long(get_yarr(levels, findgen(nelamp), yy(ok)))
        yy(ok) = interpolate(col_val, ytemp, /grid)
        if ok(0) gt 0 then yy(0:ok(0)-1)= !p.background
        if ok(nok-1) lt ysiz-1 then yy(ok(nok-1)+1:ysiz-1) = col_val(nelamp-1)
        yy = byte((round(yy)> 0) < 255)
        image = make_array(/byt, xsiz, ysiz)
        for i = 0, xsiz-1 do image[i, *]=yy
        tv, image, xpos(0), ypos(0), xsiz= xsizp, ysiz = ysizp
        x = [0, 1]
        !p.multi(0) = !p.multi(0)+1
        plot, x, amp, /nodata, position = position, charsize = chars, $
          color = color, ytickv = tickvalue, yticks = ticks, $
          xtickname =[' ', ' '], xticks= 1, yticklen = .18 , $
          xticklen = 1.0e-7, yrange =  [min(amp,max = mmm), mmm], $
          _extra = extrastuff
        nelov = n_elements(ovamp)
        if nelov gt 1 then begin
          for i = 0, nelov-1 do $
            oplot, x, [ovamp(i), ovamp(i)], col = ovcol(i)
        endif
      endelse
    ; axis  overplot
      if keyword_set (horizontal) then begin
      endif else begin
      endelse
; reset current values and plot the title
      !x = x_str_save
      !y = y_str_save
      !p = p_str_save

      if (maxstr gt 0) then xyouts, middle, ytitle, tit, /nor, chars= chart, $
          align = 0.5, color = color
      return
      end

