; ------------------------------------------------------------------------ 
; $Id: totfr.pro,v 1.3 2007/06/06 20:09:38 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
 Function TOTFR, strin, nxin, nyin, area = area, density = density, plotit =pp
;
;+
; NAME:
;       TOTFR
;
; PURPOSE:
;       Add the counts in a CR-39 digitized image.
;       Input structure must be single block, 2 dimensions.
;
; CATEGORY:
;       Math 
;
; CALLING SEQUENCE:
;       Track_Total = TOTFR(STR_IN [, NX] [, NY] [, AREA = area] 
;                            [, DENSITY = density] [, /PLOTIT ])
;
; Where:
;       Track_Total - Total number of tracks in the designated area
;                     If NX is present and > 0, NX*NY values will be returned
;                     If NX is zero or omitted, Track_Total is a scalar value
;       STR_IN    - Structure or structure array number with the CR-39 image.
;                   This may be any 2 dimensional, single block, scalar structure
;       NX        - Number of images in the horizontal direction.
;                   If NX is omitted, zero or negative, then the user will be
;                   asked to designate a single area.
;       NY        - Number of frames in the vertical direction.
;                   Ignored if NX is not positive.
;                   Default value = 1.
;       AREA      - Keyword returned with the area = Sx*Sy*Dx*Dy
;                   Where Sx and Sy are the size in bins in the X and Y 
;                   directions of each image and Dx and Dy are the size of the
;                   bins.  AREA is a scalar quantity.
;       DENSITY   - Average number of Tracks /unit area.  = Track_Total/AREA
;       PLOTIT    - If set, Image will be plotted.
;
; BACKGROUND:
;       CR-39 images are digitized according to the number of tracks with a
;       given size in a bin. Low resolution scans are 300 microns/bin.
;       High resolution scans are 150 microns/bin.
;
;  Frame Number       Track Area (Pixels = 0.299 um^2) / Effective Diam.
;         1                 1 to 2 pixels    /  0.617 to 0.872 microns
;         2                 3 to 5 pixels    /  1.069 to 1.380 microns
;         3                 6 to 10 pixels   /  1.511 to 1.951 microns
;         4                11 to 16 pixels   /  2.046 to 2.468 microns
;         5                17 to 23 pixels   /  2.544 to 2.959 microns
;         6                24 to 32 pixels   /  3.023 to 3.490 microns
;         7                33 to 42 pixels   /  3.544 to 3.999 microns
;         8                43 to 53 pixels   /  4.046 to 4.492 microns
;         9                54 to 65 pixels   /  4.534 to 4.974 microns
;        10                66 to 94 pixels   /  5.013 to 5.982 microns
;        11                95 to 128 pixels  /  6.014 to 6.981 microns
;        12               129 to 168 pixels  /  7.008 to 7.997 microns
;        13               169 to 212 pixels  /  8.021 to 8.984 microns
;        14               213 to 262 pixels  /  9.005 to 9.987 microns
;        15               263 to 378 pixels  / 10.006 to 11.996 microns
;        16               379 to 514 pixels  / 12.012 to 13.989 microns
;        17               515 to 672 pixels  / 14.002 to 15.995 microns
;        18               673 to 851 pixels  / 16.007 to 17.999 microns
;        19               852 to 9999 pixels / 18.010 to  112.832 microns
;        20               Saturation Frame - Total Number of Pixels with tracks
;                                            normalized to the number of pixels
;                                            in the bin times 32768.
;                    (a value of 16384 means half of the pixels contain tracks)
;
; EXAMPLES:
;       Calculate the total number of tracks for the entire structure 1.
;         tracks = totfr(1, 1)
;
;       Calculate the number of tracks in each of 6 images in structure 2 
;       which are arranged in a 3 horizontal by 2 vertical array.
;          tracks = totfr(1, 3, 2)
;        NOTE: tracks = tracks[3,2]
;
;
; MODIFICATION HISTORY:
;      Original version, L. P. Mix, November 16, 1998
;-
;
;
;  Common blocks
@qscom.dat
@qsstrcom.dat
;       return on an error
      on_error, 1
;
      if n_elements (structinit) eq 0 then qsstruct_init
      if n_elements (qsplotinit) eq 0 then qsinit
;
      nparams = n_params()
      if nparams eq 0 then begin
        print,'% Format: Track_Total = TOTFR(STR_IN [, NX] [, NY] '
        print,'            [, AREA = area] [, DENSITY = density] )'
        return, 0
      endif
;
      if n_elements(strin) eq 0 then begin
        print, '% TOTFR: Input structure must be defined'
        return, 0
      endif
;
      area = 0
      density = 0
      a = strin
      b = size(a)
      if b(b(0)+1) ne 8 then begin
        if a gt 0 and a le maxstructure then begin
          s2i, a, a
          b = size (a)
          if b(b(0)+1) ne 8 then begin
            print, '% TOTFR: ', $
   'First parameter must be a structure or a structure array number'
            return, 0
          endif
        endif  else begin
          print,  '% TOTFR: ',$
   'First parameter must be a structure or a valid structure array number'
          return, 0
        endelse
      endif
      if a.type  ne 1 then begin
        print,'% TOTFR: Only field type structures may be added.'
        return, 0
      endif
      if ((a.block ne 1) or (a.nvect ne 1)) then begin
        print,'% TOTFR: Single block scalar structures are required.'
        return, 0
      endif
      if a.ndim gt 2 and a.two ne 3 then begin
        print,'% TOTFR: 2 Dimensional data is required'
        return, 0
      endif
      if n_elements(nxin) gt 0 then begin 
        if nxin le 0 then nx = -1 else nx = nxin
      endif else nx = -1
      v = get_array(a, -1)
      if nx lt 0 then begin
        get_image_region,  a, ix, iy, /snap, /box
        if n_elements(ix) lt 2 then begin
          print,'% TOTFR: Region not defined
          return, 0
        endif
        tot = total (v(ix(0):ix(1), iy(0):iy(1)))
        xdel = ix(1)-ix(0)+1
        ydel = iy(1)-iy(0)+1
        area = xdel*ydel*a.dx1*a.dx2
        density = tot/area
      endif else begin
        if n_elements (nyin) eq 0 then ny = 1 else ny = nyin
        if ny le 0 then ny = 1
        nxt = a.locx(0, 1)
        nyt = a.locx(1, 1)
        xdel = nxt/nx
        ydel = nyt/ny
        area = xdel*ydel*a.dx1*a.dx2
        tot = reform(make_array(nx, ny))
        for i = 0, nx-1 do for j = 0, ny-1 do $
          tot(i,j)= total(v(i*xdel:(i+1)*xdel-1, j*ydel:(j+1)*ydel-1))
        density = tot/area
      endelse
      if keyword_set (pp) then begin
        plotstr, a, cn=2, /xst, /yst
        if nx lt 0 then begin
          x0 = a.x1(ix(0))
          x1 = a.x1(ix(1))
          y0 = a.x2(iy(0))
          y1 = a.x2(iy(1))
          oplot, [x0, x1, x1, x0, x0], [y0, y0, y1, y1, y0], thick = 2, color=7
      
        endif else if nx*ny gt 1 then begin
         if nx gt 1 then $
           for i = 1, nx -1 do $
             oplot, [a.x1(i*xdel), a.x1(i*xdel)], !y.crange, thick = 2, color= 7
         if ny gt 1 then $
           for i = 1, ny -1 do $
             oplot, !x.crange, [a.x2(i*ydel), a.x2(i*ydel)], thick = 2, color= 7
        endif
      endif
      a = 0
      return, tot
      end
