; ------------------------------------------------------------------------ 
; $Id: wrimsd.pro,v 1.5 2007/06/06 20:09:38 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
 Pro WRIMSD, fname, WDF, vida = vida, clabel = clabel
;
;+
;    Write a structure array to an ascii file
;    This procedure limited to single block, single vector field data
;    All labels are limited to 80 characters.
;    format is:  WRIMSD, fname [, STR] [, VIDA = vida] [, CLABEL = clabel]
;    where 
;      fname   - the file name
;                If fname = 0 then fname ='PFIDLdefault.dat' 
;      STR     - STR array or STR array number
;                If STR is not provided, then keyword VIDA must be set.
;      VIDA    - Keyword indicating that VIDA data is to be put into a 
;                structure.
;                See VIDA2STR for values. The following command is called:
;                VIDA2STR, VIDA, STR, /STRUCTURE
;      CLABEL  - Comment label for the output
;    Example:
;    Write STR array 5 into file 'PFIDLdefault.dat'
;       WRIMSD, 0, 5
;
;    Write STR 3 to file, 'testdata' 
;       WRIMSD , 'testdata', 3
;
;    form of file:
;      dataset comment
;      ndim, lcuniform 
;      x axis label 
;      y axis label
;      z axis label
;      nx, ny, nz
;      x0, y0, z0 |
;      dx, dy, dz |  or x, y, z
;      field data
;
;  where:
;      ndim       = 2 or 3
;      lcuniform  = 1 if uniform, 0 if not uniform
;  X is uniform if max(dx)-min(dx) is less than 1.22e-4 *max(abs(x))
;      axis label = string for axis labels
;                   Z label is omitted if ndim = 2
;      nx, ny, nz = number of elements in each dimension
;                   (NZ omitted if ndim = 2)
;      x0, y0, z0 = initial axis values
;      dx, dy, dz = axis step size
;      x, y, z    = axis values (lcuniform = 0)
;                   dz or Z omitted if ndim = 2
;      field data = image values
;
;   Fortran read:
;       integer lu, ndim, lcuniform, nx, ny, nz, i, j, k
;       real x(:), y(:), z(:)
;       real x0, y0, z0, dx, dy, dz 
;       real image( :, :, :)
;       character*80 xlabel, ylabel, zlabel, title
; c1     format (a)
;       open(file = , unit = lu)
;       read(lu, 1) title
;       read(lu, *) ndim, lcuniform 
;       read(lu, 1) xlabel
;       read(lu, 1) ylabel
;       nz = 1
;       if (ndim .eq. 3) then
;         read(lu, 1) zlabel
;         read(lu, *) nx, ny, nz
;       else
;         read(lu, *) nx, ny
;       endif 
; c      if fortran 90, allocate x, y, z, image
;       if (lcuniform .eq. 0) then
;         read(lu, *) (x(i), i = 1, nx)
;         read(lu, *) (y(i), i = 1, ny)
;         if (ndim .eq. 3) read(lu, *) (z(i), i = 1, nz)
;       else
;         if (ndim .eq. 2) then
;           read(lu, *) x0, y0
;           read(lu, *) dx, dy
;         else
;           read(lu, *), x0, y0, z0
;           read(lu, *) dx, dy, dz
;         endif
; c        can make x, y, z if desired x(i)= (i-1)*dx + x0, etc.
;      endif
;      read(lu, *)(((image(i, j, k),i=1,nx),j=1, ny),z=1,nz)
;      close (lu)
;-
;
on_error, 1
@qscom.dat
;  Common block
@qsstrcom.dat
;
      if n_elements (structinit) eq 0 then qsstruct_init
      if n_elements (qsplotinit) eq 0 then qsinit
;
      nparams = n_params()
      if nparams eq 0  then begin
        print, '% Format: WRIMSD, Filename, Structure'
        return
      endif
      if n_elements (fname) eq 0 then begin
        print, '% WRIMSD: File Name must be provided'
        return
      endif
      b = size(fname)
      if b(b(0)+1) ne 7 then begin
        if fname(0) eq 0 then fname = 'PFIDLdefault.dat' else begin
          print, '% WRIMSD: Input file name must be a string or zero'
          return
        endelse
      endif else fname = fname(0)
      nel = n_elements(wdf)
      lcvida = 0
      if nel eq 0  or keyword_set(vida) then begin
        if keyword_set(vida) then begin
          wdf=0
          vida2str, vida, wdf, /structure, clabel = clabel, $
             dx = delx, dy= dely, dz = delz
          lcvida = 1
        endif else begin
          print,'% WRIMSD: Structure array must be provided'
          return
        endelse
      endif
      a = wdf(0)
      b = size(a)
      if b(b(0)+1) ne 8 then begin
        if a gt 0 and a le maxstructure then begin
          s2i, str, a
          b = size (str)
          if b(b(0)+1) ne 8 then begin
            print, $
              '% WRIMSD: No data in structure array ', strtrim(a,2)
            return
          endif
        endif  else begin
          print, '% WRIMSD: Second parameter must be a structure or '
          print, '          a valid structure array number'
          return
        endelse
      endif else str = temporary(a)
      if str.type ne 1 then begin
        print, '% WRIMSD: Only FIELD type (=1) data can be written', str.type
        return
      endif
      ndim = str.ndim
      if ndim lt 2 or ndim gt 3 then begin
        print, '% WRIMSD: Only two and three dimensional data can be written'
        return
      endif
      if str.block ne 1 or str.nvect ne 1 then begin
        print, '% WRIMSD: Only single vector, single block data can be written'
        print, '          Block, vectors:', str.block, str.nvect
        return
      endif
      recl = 80
      lcuniform = 1
      x = str.x1
      y = str.x2
      z = str.x3

      nelx = n_elements(x)
      nely = n_elements(y)
      nelz = n_elements(z)
      if lcvida eq 0 then begin
        if nelx eq 1 then delx = 1 else begin
          delx = x(1:nelx-1)-x(0:nelx-2)
          err = max(abs(x))*1.22070e-4
          if (max (delx, min= mm)- mm) lt err then $
            delx =(x(nelx-1)-x(0))/(nelx-1) else lcuniform = 0
        endelse
        if lcuniform eq 1 then begin
          if nely eq 1 then dely = 1 else begin
            dely = y(1:nely-1)-y(0:nely-2)
            err = max(abs(y))*1.22070e-4
            if (max (dely, min= mm)- mm) lt err then $
              dely =(y(nely-1)-y(0))/(nely-1) else lcuniform = 0
          endelse
        endif
        if lcuniform eq 1 and ndim gt 2 then begin
          if nelz eq 1 then delz = 1 else begin
            delz = z(1:nelz-1)-z(0:nelz-2)
            err = max(abs(z))*1.22070e-4
            if (max (delz, min= mm)- mm) lt err then $
              delz =(z(nelz-1)-z(0))/(nelz-1) else lcuniform = 0
          endelse
        endif
      endif

      openw,unit,fname, /get_lun, width = recl
; Time to start writing
;      dataset comment
;      ndim, lcuniform 
;      x axis label 
;      y axis label
;      z axis label
;      nx, ny, nz
;      dx, dy, dz or x, y, z
;      field data
      if n_elements(clabel) gt 0 then printf, unit, strmid(clabel, 0, 80) $
        else printf, unit, strmid(str.clab, 0, 80)
      printf, unit, str.ndim, lcuniform
      printf, unit, strmid(str.x1lab, 0, 80)
      printf, unit, strmid(str.x2lab, 0, 80)
      if ndim eq 3 then begin
        printf, unit, strmid(str.x3lab, 0, 80)
        printf, unit, nelx, nely, nelz
      endif else printf, unit, nelx, nely
      if lcuniform eq 1 then begin
        if ndim eq 2 then begin
          printf, unit, x(0), y(0)
          printf, unit, delx, dely 
        endif else begin
          printf, unit, x(0), y(0), z(0)
          printf, unit, delx, dely, delz
        endelse
      endif else begin
        printf, unit, x
        printf, unit, y
        if ndim eq 3 then printf, unit, z
      endelse
      printf, unit, str.v1
      free_lun, unit
      return
      end
