; ------------------------------------------------------------------------ 
; $Id: opensaf.pro,v 1.3 2007/06/06 20:10:58 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
   pro opensaf, p1,  error = error,  filename = filename
; pro opensaf, p1, p2, error = error, path = path, filter = filter, $
;              filename = filename, fileid = fileid, ndset = ndset, help = help
;+
;  Open a SAF file.  The current file pointer is set to this file id and
;  the dataset pointer for this file is set to 1.
;
;  format: OPENSAF [, File] [, Fileid ] [, Error = error] [, FILENAME = filename]
;
;  FORMAT BELOW NOT IMPLEMENTED
;  format: OPENSAF [, File] [, Fileid ] [, Error = error] [, NDSET =ndset]
;                  [, PATH = path] [, FILTER = filter] [, HELP = help]
;                  [, FILENAME = filename] [, FILEID = fileid]
;  where:
;    File   - name of the SAF file to be opened.  The default file extension
;             is ".saf".  Both upper and lower case versions of the file name
;             will be tried.
;             If file is omitted or undefined or is not a string,
;             PICKFILE will be used to select a file.
;             The initial path in PICKFILE is the current directory or
;             the PATH to the last file selected.
;             NOTE: PATH and FILTER may be used to set the initial path and
;                   filter for PICKFILE.
;             NOTE: FULFNAM procedure is used to handle environment variables
;                   See FULFNAM for details
;    Error  - Error parameter
;             If this error is zero, no error encountered.
;             If no file specified for open then error = -13
;             If Error = -99, file not found
;             If Error = n > -99, error is a saf library error code defined
;               by the particular routine where the error occurred.
;               For example, file has no SAF header.
;    *********************************
;    ITEMS BELOW are not implemented 
;    Fileid - user specified file id
;             (If fileid is specified, it will be used as the file id.  If
;             fileid is not provided or is zero, a value will be
;             provided; default value is the first unused integer greater
;             than 0.  If fileid is provided but has been used for another
;             file, the program will inquire if the user wishes to
;             close the old file, to specify a new fid, or to accept
;             the default.  If positional parameter is valid, keyword will
;             be ignored.
;    help   - Print a help text
;    PATH   - Path to the current file
;             Default is the current directory in the first call to OPENSAF but
;             the default path is changed to the final path selected is
;             retained as the default in the next call to opensaf.
;    FILTER - File filter - The default may be set in USERQSINIT; if undefined
;             the default is '*'.  The default filter is set only with
;              USERQSINIT or SETDEFAULT.
;    NDSET  - Number of datasets
;    FILENAME - Name of the file opened.
;               Filename is also available with:
;               get_dir, 0, 1, filename= filename
;
;  Examples:
;    Open a SAF file with the default file id
;      opensaf
;    Open a SAF file, 'users/test', with the default file id
;      opensaf, '/users/test'
;    Open a file to be selected from a menu with a file id of 14
;      opensaf, ' ', 14
;    Open 'test' or 'test.saf' with a file id of 14
;      opensaf, 'test', 14
;    Determine keywords and parameters for opensaf
;      opensaf, /h
;-
;
      on_error, 1
      common safcom, initsaf, saf_exe

      COMMON savepicker, oldfilter, oldpath
;
@saf_com.dat
;
; common saf_common, saf_directory_list, saf_filename
;     check on the number of parameters
;
      nparams = n_params()
      if nparams eq 0 and keyword_set(help) then begin
print, '% Format: OPENSAF [, File] [, Fileid ] [, Error = error]'
print, '                  [, PATH = path] [, FILTER = filter]'
print, '                  [, NDSET = ndset] [, FILEID = fileid]'

        return
      endif
;
;     check to see if saf has been initialized
;
      if (n_elements(initsaf) eq 0) then init_saf else $
        if (initsaf eq 0) then init_saf
      if initsaf eq 0 then begin
        print, 'SAF libraries are not supported by this Operating System'
        return
      endif
      if n_elements(filename) eq 0 then filename = '' else $
        filename = strtrim(filename[0], 2)
;
;     be sure we have a file name
;
      error = -13
      if (nparams eq 0) then begin
        p1 = filename
        nparams = 1
      endif else if n_elements(p1) eq 0 then begin
        p1 = filename
      endif else if (size(p1, /type) ne 7) then begin
        p1 = filename
      endif else  p1 = strtrim(p1, 2)
      cd, current = current
      if strlen(p1) lt 1 then begin
        IF (N_ELEMENTS(PATH) EQ 0 and n_elements(oldpath) eq 0) THEN BEGIN
          if initsaf lt 3 then path = current+'/' else path = current
        endif else if n_elements(path) eq 0 then path = oldpath
        if n_elements(filter) eq 0 and n_elements(oldfilter) eq 0 then $
          filter = '*' else if n_elements(filter) eq 0 then filter = oldfilter
        p1 = dialog_pickfile( path = path, get_path =tempath, filter = filter, $
          title= 'Select saf file to read', /must_exist)
        p1 = strtrim(p1, 2)
        if strlen(p1) lt 1 then return
        if n_elements(tempath) gt 0 then oldpath = tempath
      endif
;
      vers = strupcase(strmid(!version.os_family, 0, 3))
      if (vers eq 'UNI') then begin
;       get system variable substitution for unix systems
        temp = p1
        FULFNAM, temp, p1
      endif
      p1 = file_expand_path(p1)
      if !version.os eq 'linux' then resolve_it, p1
      info = file_info(p1)
      if (info.exists + info.read) ne 2b then begin
           print, '% OPENSAF: File does not exist with read permission'
           print, '         File Name = ', p1
           print, '         File Exists =', info.exists
           print, '         Read Permission =', info.read
           return
      endif


; user specified file id
      if n_elements (p2) eq 0 then begin
        if n_elements(fileid) gt 0 then p2 = long(fileid) else p2 = 0L
      endif else p2 = long(p2)

      error = 0L
      p3 = [byte(p1), 0b]
;
      if initsaf lt 3 then begin
; UNIX/Linux
        junk = call_external(saf_exe, 'SAF_open', p3, error)
      endif else if initpff eq 4 then begin
; WIN
       junk = call_external(SAF_EXE, 'SAF_open@8', p3, p2, lstr, ndset, error)
      endif else begin
        Print, '% OPENSAF: This operating system is not supported.'
        return
      endelse
help, error
      if error eq 0 then begin
        saf_filename = p1
        print, ' SAF file opened: ', saf_filename
        fileid = p2
        filename = p1
      endif
      end

