; ------------------------------------------------------------------------ 
; $Id: icem2qs.pro,v 1.3 2007/06/06 20:12:25 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
pro icem2qs, fname, outfile, outfile = koutfile, multiplier = mult, $
             cm = cm, inch = inch, micron = micron, mil = mil
;+
;    Read an ICEM ascii output file and generate an input file for QS
;    Format:
;        ICEM2QS, filename, outfile [, outfile = outfile]
;         [, multiplier= multiplier] [, /CM] [, /INCH] [, /MICRON] [, /MIL]
;    where
;      fname   - The name of a file
;                If fname is not a string, DIALOG_PICKFILE will be used to
;                select a file.
;      outfile - Output file name.  If outfile is present, the keyword is
;                ignored.
;                Default is the bare filename + ".out"
;                A widget is used to confirm the name if no name is specified.
;      multiplier - QuickSilver requires MKS units
;                   If ICEM uses other units a multiplier is required.
;            Note: If multiplier is set, the following are ignored.
;      CM      - If set, multiplier = 1e-2
;      Micron  - If set, multiplier = 1E-6
;      Inch    - If set, multiplier = 2.54E-2
;      Mil     - If set, multiplier = 2.54E-5
;
;     Example:
;       Read a file selected and store the QS input file in 'qs_geo.dat'
;         ICEM2QS, 0, 'qs_geo.dat'
;       or
;         ICEM2QS, out='qs_geo.dat'
;  MODIFICATION HISTORY:
;      Original version, L. P. Mix, December 22, 2002
;-
on_error, 1
      if keyword_set (multiplier) then mult=multiplier else $
        if keyword_set(cm) then mult = 1e-2 else $
          if keyword_set(micron) then mult = 1e-6 else $
            if keyword_set(inch) then mult = 2.54e-2 else $
              if keyword_set(mil) then mult = 2.54e-5 else mult =1.0

      if n_elements(fname) eq 0 then fname =0 else fname = fname(0)
      b = size(fname)
      if b(1) eq 7 then begin
        lcfile = 1  
        resolve_it, fname 
        info = file_info(fname)
        if (info.exists + info.read) ne 2b then begin
           print, '% ICEM2QS: File does not exist with read permission'
           print, '           File Name = ', fname
           print, '           File Exists =', info.exists
           print, '           Read Permission =', info.read
           return
        endif
      endif else begin
        fname = dialog_pickfile(/must_exist, /read)
        if strlen(fname) lt 1 then begin
          print, ' No File was Selected - RETURNING'
        endif
        lcfile = 1 
        resolve_it, fname
      endelse
      filename = fname

      iq = 0
      if n_elements(outfile) eq 0 then begin
        if n_elements(koutfile) ne 0 then outfile = koutfile[0] else begin
          outfile= get_bare_filename(fname) + '.out'
          outfile= find_newfile(outfile)
          iq =1
        endelse
      endif

;  pro icem2qs, fname, outfile, outfile = koutfile, multiplier = mult

on_ioerror, end_read
openr, unit, /get_lun, filename, error = error
if error ne 0 then begin
  print, '% ICEM2QS:  Error opening file: ', filename
  print, !error_state.msg
  return
endif

str= ''
grid_flag = '*GEO_GRILLE'
volume_flag = '*GEO_VOLUME'
nx = 0L
ny =0L
nz =0L
mode ='Looking for '+grid_flag
while strpos(str,grid_flag) lt 0 do readf, unit, str
mode ='Reading grid size'
readf, unit, nx, ny, nz
mode = 'Reading true/false'
readf, unit, str
xarr = make_array(nx)
yarr = make_array(ny)
zarr = make_array(nz)
formatg ='(10(1x, g0.8))'
formatf ='(10(1x, f0.6))'
ff = formatf
mode ='Reading X1 grid'
readf, unit, xarr
;print, xarr, format = ff
mode ='Reading X2 grid'
readf, unit, yarr
;print, yarr, format = ff
mode ='Reading X3 grid'
readf, unit, zarr
;print, zarr, format = ff
if mult ne 1 then begin
  xarr = xarr*mult
  yarr = yarr*mult
  zarr = zarr*mult
endif

mode ='Looking for '+volume_flag
while strpos(str,volume_flag) lt 0 do readf, unit, str

if iq eq 1 then get_tqstring, iq, 'ENTER the File Name:', outfile
if iq eq 1 then begin
  Print, '% ICEM2QS: Quitting.'
  return
endif
 
openw, out_unit, /get, outfile, error= error
if error ne 0 then begin
  print, '% ICEM2QS:  Error opening file: ', outfile
  print, !error_state.msg
  return
endif

mode = 'Reading number of conductors'
nconductor =0L

readf, unit, nconductor
for iconduct =0L, nconductor-1 do begin
  mode = 'Reading number of conductor elements and name for conductor: ' $
    + strtrim (iconduct+1,2)
  nblocks = 0L
  cond_name =''
  readf, unit, nblocks, cond_name
  byt = byte(cond_name)
  ok = where (byt lt 35b, nok)
  if nok gt 0 then byt[ok]=32b
  cond_name = strtrim(byt,2)
  get_icem_cond_info, lcquit, cond_name, cond_type, offset
  if cond_type eq 2 then cond_str = 'Dielectric Conformal ' else  $
     cond_str = 'Conductor Conformal '
  if cond_type eq 1 then name = '  '+cond_name +' -1' else name = '  '+cond_name

  if lcquit eq 0 then begin
    npt = 6L*nblocks
    iconductor = make_array(npt, /long)
    mode = 'Reading conductor blocks i, j, k: '+ cond_name
    readf, unit, iconductor
;            npt-1
    for i = 0L, npt-1, 6 do printf, out_unit, cond_str,   $
      xarr[iconductor[i]],   yarr[iconductor[1+i]], $
      zarr[iconductor[2+i]], xarr[iconductor[3+i]], $
      yarr[iconductor[4+i]], zarr[iconductor[5+i]], name, $
      format='(a, 6g15, a)'
  endif
endfor
free_lun, unit
free_lun, out_unit
return

end_read:
print, '% ICEM2QS: IO Error'
print, mode
free_lun, unit
free_lun, out_unit
end
