; ------------------------------------------------------------------------ 
; $Id: qs_read_grid.pro,v 1.3 2007/06/06 20:12:25 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
 PRO qs_read_grid, fname, xaxis =xaxis, yaxis = yaxis,  _extra = extrastuff, $
     readonly = readonly, values = values, deltas = deltas, multiplier = mult, $
     noscale = noscale, helpme = helpme

;
;+
; Read a portion of a QSGRID generated comment string to regenerate a grid 
; commands.
; Below is an example of a file:
; File must have "Multiplier = "line
;*#
;*# Grid generated using the following grid locations and grid spacing
;*#    Initial-X      Delta-X  (Spatial Multiplier = 0.00100000)
;*#      -16.930000      0.55889499
;*#      -10.430100      0.25359249  Quadratic
;*#      -7.4666800      0.28521122  Quadratic
;*#      -6.0756302      0.17847207  Quadratic
;*#      -4.9217401      0.28308398  Quadratic
;*#      -3.4692800      0.13190460  Quadratic
;*#      -1.7009900      0.26104874  Quadratic
;*#       6.0110698      0.20634882  Quadratic
;*#       8.0273705      0.16025130  Quadratic
;*#       10.684400      0.15234036  Quadratic
;*#       12.348300      0.32305975  Quadratic
;*#       17.572001      0.54755701  Quadratic
;*#
;
; Format:
;    QS_READ_GRID [, FILE_NAME] , /XAXIS or ,/YAXIS , _EXTRA = extrastuff
;                 [, /READONLY] [, VALUES = VALUES] [, DELTAS = DELTAS]
;                 [, MULTIPLIER = MULTIPLIER] [, NoScale = noscale]
;                 [, /HELPME]
;
;   where:
;     FILE_NAME  - The file name
;                  If missing, PICKFILE will be used.
;     /XAXIS   |
;              | - One Keyword must be set to specify the horizontal or vertical
;     /YAXIS   |   axis.
;                  Note: AXES may be passed as an extra parameter
;                  Note: If Xaxis is set, yaxis is set to zero.
;     _EXTRA     - Any other valid keywords for QSGRID.
;
;    /READONLY   - If set, file will only be read.
;     VALUES     - Grid tie points
;     DELTAS     - Grid delta values
;     MULTIPLIER - Grid Spatial multiplier
;     NoScale    - If set the multiplier will not be applied ot the values.
;     HELPME     - If set, a help message will be printed
;
; Side Effect:
;   If readonly is not set, the following command is issued:
;    QSGRID, values*multiplier, deltas*multiplier, xaxis = xaxis, yaxis=yaxis, $
;       multiplier = 1.0, _extra = extrastuff
;   If NoScale is set then the command is: 
;    QSGRID, values, deltas, xaxis = xaxis, yaxis = yaxis, $
;       multiplier = multiplier, _extra = extrastuff
;
; Example:
;   Read a file for the X axis.
;    QS_READ_GRID, /x
;
;-   

on_error, 1
     nparams = n_params()
      if keyword_set(helpme)  then begin
    print, '% Format: QS_READ_GRID [, FILE_NAME], /XAXIS or ,/YAXIS '
    print, '                [, /READONLY] [, VALUES = VALUES] [, DELTAS = DELTAS]'
    print, '                [, MULTIPLIER = MULTIPLIER] [ , _EXTRA = extrastuff]'
        return
      endif
      if keyword_set(xaxis) then yaxis = 0 else begin
        if not keyword_set(yaxis) then begin
          print, '% QS_READ_GRID: Xaxis or Yaxis keyword must be set'
          return
        endif
      endelse

      if n_elements (fname) eq 0 then begin
        fname = dialog_pickfile(/must_exist, /read)
        if strlen(fname) lt 1 then begin
          print, '% QS_READ_GRID: No File was Selected - RETURNING'
          return
        endif
      endif
      openr,unit,fname, /get_lun, error = error
      if error ne 0 then begin
        print, '% QS_READ_GRID: Error opening file: ', fname
        return
      endif
      ok = -1
      str = ''
      search = 'Multiplier ='
      lsearch = strlen(search)
      while (not eof(unit) and ok lt 0) do begin
        readf, unit, str
        ok = strpos(str, search)
      endwhile
      if eof(unit) then begin
        print, '% QS_READ_GRID: "Multiplier =" string not found'
        free_lun, unit
        return
      endif
      okend = strpos (str, ')')
      ok = ok + lsearch
      str = strmid (str, ok, okend-ok)
      mult= 1.0
      reads, str, mult
      np = 0
      while (np lt 3) and (not eof(unit)) do begin
        readf, unit, str
        parser, str, strar, np, /string
      endwhile
      if eof(unit) then begin
        print, '% QS_READ_GRID: No data found'
        free_lun, unit
        return
      endif
      nvmax = 256
      values = make_array(256)
      deltas = make_array(256)
      nval = 0
      val = 0.0
      dx = 0.0
      while np ge 3 do begin
        reads, strar[1], val
        reads, strar[2], dx
        values(nval) = val
        deltas(nval) = dx
        nval = nval+1
        if nval eq nvmax then begin
          values = [values, values]
          deltas = [deltas, deltas]
          nvmax = nvmax+nvmax
        endif
        if eof(unit) then np=0 else begin
          readf, unit, str
          parser, str, strar, np, /string
        endelse
      endwhile
      free_lun, unit
      if nval lt 2 then begin
        print, '% QS_READ_GRID: Less than two data values found.'
        return
      endif
      values = values(0:nval-1)
      deltas = deltas(0:nval-1)
      if keyword_set(readonly) then return
      if keyword_set (NoScale) then $
         qsgrid, values, deltas, xaxis = xaxis, yaxis = yaxis, $
            multiplier = mult, _extra = extrastuff $
      else  $
         qsgrid, values*mult, deltas*mult, xaxis = xaxis, yaxis = yaxis, $
         multiplier = 1.0, _extra = extrastuff
      return
      end
