; ------------------------------------------------------------------------ 
; $Id: qsgrid.pro,v 1.5 2007/06/06 20:12:25 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
 Pro qsgrid, xin, dxin, gxout,  inches = inches, mils = mils, $
     cm = cm, multiplier = multip, xaxis = xaxis, yaxis = yaxis, $
     axes = axes, ratio = ratio, gridtype = gridtype, $
     blocks = blocks, quiet = quiet, helpme = helpme
;
;+
; format:  qsgrid, x, dx, gx, inches = inches, mils = mils, cm = cm, $
;  multiplier = multip, xaxis = xaxis, yaxis = yaxis, ratio = ratio, 
;  gridtype = gridtype, axes = axes, quiet = quiet
;
; This procedure generates a grid with a continuous first derivative of the 
; generating function. Either a quadratic or a geometric generating function 
; can be used.
;
; X is an array of grid locations which are used to specify the grid interval.
; DX is an array of grid intervals corresponding to X locations.
;    Note: All DX values must be less than 1/2 of the corresponding X spacing.
;          (ie a minimum of 2 zones per grid region.)
;     If dx is omitted then gx = x
; GX  is the actual grid
; If X and DX are not specified the last values input according to the values
;      of XAXIS and YAXIS will be used.
; RATIO is the smallest value of the ratio between adjacent cells.
;       The largest value is 1.0/ratio
;       Default is 0.9 
; GRIDTYPE is an integer or an array of integers which correspond to each 
;          grid region.  Default is 0. 
;          If GRIDTYPE is an integer, then GRIDTYPE is set to 
;             MAKE_ARRAY(n_elements(X)-1, value = GRIDTYPE, /int)
;          If GRIDTYPE is smaller than the number of regions, then the last 
;             value will be used for all missing values. 
;          If GRIDTYPE is an array, no elements are changed or deleted as shown
;             in paragraph 4 under Procedure.
;          The meaning of GRIDTYPE is as follows:
;          0  Normal quadratic grid generation scheme.
;         -N  If the first grid region, uniform spacing with N grid spaces.
;             If it connects to another grid region, the upper DX value is set 
;               to the grid spacing/N
;             Negative GRIDTYPE values are set to the max(GRIDTYPE) > 0
;         +N  Use a geometric grid to over the grid region.
;     Note:  If DX on average changes less than 1%/zone then a quadratic grid 
;            is used. Specifically if:
;            max(dx(0), dx(n))/min(dx(0), dx(n)) -1 gt 0.005* dist/(dx(0)+dx(n))
;            then a quadratic grid is used
; Note: If GRIDTYPE  is an array, it will not be modified in QSGRID.
; For a continous first derivative, both X and DX cannot be specified exactly
; everywhere. DX is defined as the derivative of the grid generation function.
; The grid generation function is given by:
;   Quadratic: x(i) = (i*b + a)*i + x(0)
;   Geometric: x(i) = DX/log(R) * (R^i -1) + x(0)
;            Example of GRIDTYPE.
;            assume X = [0, 1, 2, 3], DX=[1,1,1,1], G= -[10, 20, 10]
;            This is would be the same as to: 
;               X= X, DX= [.1, -.1, .05, .1], G = 0
;            explanation: dx(1, 2, 3) are set to the the X-spacing divided by 
;                         the respective G. Because DX(0) is positive, it is 
;                         changed to dx(1) and dx(1) is set negative.
;
;            assume X = [0, 1, 2, 3], DX=[.5,1,1,1], G= [1, -20, -10]
;            This is would be the same as to: 
;               X= X, DX= [.5, .05, -.05, .1], G = [1, 1, 1]
;            explanation: dx(2, 3) are set to the the X-spacing divided by 
;                         the respective G. Because DX(1) is positive, it is 
;                         changed to dx(2) and dx(2) is set negative.
;            Note: Because of the default limit of ratio, this particular 
;                  will have an initial derivative of 0.149679 because that
;                  is as large as it can grow.
;
;             assume X = [0, 1, 2, 3], DX=[-0.1,1,1,1], G= [-10, -20, -10]
;            This is would be the same as to:
;               X= X, DX= [-0.1, .1, .05, .1], G = 0
;            explanation: dx(1, 2, 3) are set to the the X-spacing divided by 
;                         the respective G. Because DX(0) is already negative,
;                         we assume that it is to remain fixed.
;
;             assume X = [0, 1, 2, 3], DX=[-0.1,1,1,1], G= [-10, -20, -10, 1]
;            This is would be the same as to:
;               X= X, DX= [-0.1, .1, .05, .1], G = 1
;            explanation: Same as above, but G is set to 1 because the 
;                         maximum of G is 1.
;
;             assume X = [0, 1, 2, 3], DX=[0.1,1,1,1], G= [-20, -20, -10, 1]
;            This is would be the same as to:
;               X= X, DX= [0.05,-0.05, .05, .1], G = 1
;            explanation: Same as the second example regarding DX but because
;                         dx(0) is positive, it is 
;                         changed to dx(1) and dx(1) is set negative.
;
; BLOCKS  If present, each grid region will be given the indicated block 
;           designation.  If there are more regions than blocks, the last block
;           will be used for all remaining regions. Default is BLOCKS = 1
;  Note:  If BLOCKS is an array, then (like GRIDTYPE) it will NOT be modified
;           in QSGRID.
; AXES    If present, the grid axis for output will be i, j or k depending 
;           on whether AXES is 1, 2, or 3.  Default is 'i' if xaxis is set or 
;           'j' if yaxis is set.
; QUIET   If keyword is set, no plots or informational output will be made
; HELPME  If set, print a help message.
;
; Procedure:
; 1.  All values of x < x(0) are deleted with the corresponding dx values
; 2.  X values are sorted.
; 3.  GRIDTYPE if it has fewer elements than X/DX is made into an array with the
;         proper number of elements.
; 4.  Negative GRIDTYPE values are resolved and any X values which are spaced 
;         closer than twice the corresponding DX values are deleted.
; 5.  If all DX are positive: 
;         Start with smallest x and generate grid using an integer number of 
;         grids points and a best effort to match the specified DX values.
;     If one or more negative DX values, then:
;         Use the first negative DX value and the corresponding X value to 
;           generate a grid toward lower values.  In this generation, the 
;           end points are exact, and a best effort to match the specified
;           DX values is made.
;         Generate the grid toward larger values by using an integer number of 
;           grids points and a best effort to match the specified DX values if 
;           dx(i) is positive or vary the endpoint to obtain in endpoint 
;           interval given exactly by the absolute value of dx(i) .
; Side Effects:
;     The grid will be in WDF array 80
;     The derivative will be in WDF array 81
;     For X grids, the grid, first, second , and ratio of the derivatives
;        are in 71 to 74
;     For Y grids, the grid, first, second , and ratio of the derivatives
;        are in 75 to 78
; Example;
;   generate a uniform y grid from 0 to 10 with spacing of .05 and 
;     units of inches.
;     qsgrid, [0, 10],[0.05, 0.05], /inches, /y
;
;   generate a grid which varies from .1 to .01 to .1 in going from 
;     0 to 5 to 10 with units of cm 
;     qsgrid, [0, 5, 10], [.1, .01, .1], /cm, /x 
;
;   generate a grid which has a dx of exactly 0.001 at 4 and is 0.01 near 10
;   and is about 0.01 at 0 for the y axis with units of mils
;     qsgrid, [0, 4, 10], [0.01, -0.001, -0.01], /mils, /y
;
;   generate a grid which has uniform spacing between 2 and 4 with 50 spaces 
;   increases geometrically at either side of the interval.
;     qsgrid, [0, 2, 4, 6], [1, 1, -1, 1], g= [1, -50, 1], /x
;  or qsgrid, [0, 2, 4, 6], [1, 1, 1, 1], g= [1, -50, 1], /x
;  or qsgrid, [0, 2, 4, 6], [1, .04, -0.04, 1], /g, /x
;   Note: The limits for the first and last regions will be changed to about
;         0.25 because of the default limit on the value of R for the geometric
;         grid generator of 0.9
;   Note: In the first two examples the values of DX(1:2) of 1, -1 are ignored
;         and a uniform mesh of 50 intervals is generated.  Values of DX should 
;         be less than or equal to half the X spacing to prevent elimination. 
;   Note: A geometric grid with abs(dxn/dx0 - 1) less than 0.02*n reverts to a 
;         quadratic grid where n is the number of intervals
; 
;-
@pfidltq.com
@tq_qs_gridparam.com

 common qsgrid_common, xminimum , yminimum, xmaximum, ymaximum
  on_error, 1
;
  npar = n_params()
  if keyword_set(helpme) then begin
    print, '% Format: QSGRID, X, DX, GX, [, /INCHES] [, /MILS] [, /CM]'
    print, '         [,  MULTIPLIER = multiplier] [, /XAXIS] [, /YAXIS] [, /QUIET]'
    print, '         [, RATIO = ratio] [, GRIDTYPE = gridtype] [, AXES = axes]'
    return
  endif
;
; Digress monemtarily to set the multiplier
  if KEYWORD_SET(inches) then mult= 0.0254
  if KEYWORD_SET(mils) then mult= 0.0000254
  if KEYWORD_SET(cm) then mult= 0.01
  if KEYWORD_SET(multiplier) then mult= float(abs(multip))
  if n_elements(mult) eq 0 then mult = 1.0
  axis_lab = ['  i', '  j', '  k']
; see if an axis is specified
;
  axis = 0
  if keyword_set(yaxis)  then axis = 2
  if keyword_set(xaxis) then axis = axis +1
  if axis eq 0 or axis eq 3 then begin
    print, '% QSGRID: X or Y axis must be specified'
    return
  endif
;
; if no valid input data then use old data if it exists
;
  if n_elements (axes) eq 0 then axes = axis else begin
    if axes lt 1 or axes gt 3 then axes = axis else axes = axes(0)
  endelse
if n_elements(xin) gt 2 and n_elements(dxin) eq 0 then begin
; sort the data
  ok = sort(xin)
  xin = xin(ok)
  gxout = xin
  np = n_elements(xin)
  gdxin = xin[1:np-1]-xin(0:np-2)
  dxin = gdxin
  gridtype =0
  nbin = 0
  lcnooutput = 1
endif else begin
  lcnooutput = 0
  if n_elements(xin) lt 2 then begin
    if axes eq 1 then begin
      if n_elements(xinsav) gt 0 then oldst = xinsav else $
        print,  '% QSGRID: No grid data for X axis.'
    endif else if axes eq 2 then begin
      if n_elements(yinsav) gt 0 then oldst = yinsav else $
        print, '% QSGRID: No grid data for Y axis.'
    endif else begin 
      if n_elements(zinsav) gt 0 then oldst = zinsav else $
        print, '% QSGRID: No grid data for Z axis.'
    endelse
    if n_elements(oldst) eq 0 then oldst = {gridtype:0}
    if axis eq 1 then begin
      if n_elements(x) eq 0 then message,  $
        '% QSGRID: No grid data for the horizontal axis'
      xin = x
      dxin = dx 
    endif else begin
      if n_elements(y) eq 0 then message,  $
        '% QSGRID: No grid data for the vertical axis'
      xin = y
      dxin = dy 
    endelse
;   xin = oldst.x
;   dxin = oldst.dx
    if n_elements(gridtype) eq 0 then gridtype = oldst.gridtype
  endif
      
; must have the same number of elements in x and dx
  if n_elements (dxin) ne n_elements(xin) then $
    message, 'X and DX must have the same size'
;
  if min(abs(dxin)) eq 0 then message, $
       'Delta for the axis must be greater than zero'
; see if any points occur before the first point
  ok = where (xin ge xin(0), nok)
  if nok lt 2 then message, 'No valid input arrays'
  xin = double(xin(ok))
  dxin = double(dxin(ok))
;
; remove all markers
  rm_markers
;
; sort the data
  ok = sort(xin)
  xin = xin(ok)
  dxin = dxin(ok)
;
  ngt = n_elements (gridtype)
  nel = nok
  if ngt eq 0 then begin
    gridtype = 0
    ngt = 1
  endif
  if ngt lt nel then begin
    if ngt eq 1 then $
      gridtype = make_array(nel, /int, value = gridtype(0)) else $
      gridtype = [gridtype, make_array(nel-ngt, /int, value = gridtype(ngt-1))]
  endif
;
  ok = where (gridtype(0:nel-2) lt 0, nokg)
  if nokg gt 0 then begin
    dxin(ok+1) = (xin(ok)-xin(ok+1))/gridtype(ok)
    if dxin(ok(0)) gt 0 then dxin(ok(0)) = dxin(ok(0)+1)
    if min(dxin(0:ok(0)+1)) gt 0 then dxin(ok(0)+1) = -dxin(ok(0)+1)
    gridtype(ok) = max(gridtype)>0
  endif
;
; check for points too close together
  dd = xin(1:nok-1)-xin(0:nok-2)
  adxin = abs(dxin)
  dxave  = (adxin(1:nok-1)+adxin(0:nok-2))
; we require at least 0.8*dxave between x values to have at least 2 zones.
  ok = where (dd gt .8*dxave , noks)
  if noks lt 1 then message, 'Points must be spaced by at least the grid size'
  if nok ne noks+1 then begin
    print, ' Some points are too close; these will be removed.'
    print, ' If GRIDTYPE or BLOCKS are arrays, some values may be incorrect'
;   if points are too close together then keep the first and last points
    ok = 1+ok
    ok(noks-1) = nok-1
    xin = [xin(0), xin(ok)]
    dxin = [dxin(0), dxin(ok)]
  endif
  nbl = n_elements(blocks)
  nel = n_elements(xin)
  nbin = make_array(nel, /int)
  if nbl eq 0 then begin
    blocks = 1
    nbl = 1
  endif
  if nbl lt nel-1 then begin
    if nbl eq 1 then blocks = make_array(nel-1, value = blocks(0), /int) else $
    blocks = [blocks, make_array(nel-nbl-1, value = blocks(nbl-1), /int)]
  endif
  maxgridtype = max(gridtype)
;
; 
  nx = noks + 1
  
  temp = make_array(2*nx + 4, /string)
  tempqs = make_array(nx, /string)
  temp_a = make_array(nx-1, /double)
  temp_b = make_array(nx-1, /double)
  temp_x0 = make_array(nx-1, /double)
  temp_nx = make_array(nx-1, /long)

  tempqs(0)= '*  QUICKSILVER Geometric GRID'
  temp(0)='#'
  temp(1)='# Grid generated using the following grid locations and grid spacing'
  temp(3)='#'+ string(xin(0))+ string (dxin(0))
  if not keyword_set(quiet) then begin
    if axis eq 1 then begin
      print, 'x = ', float(xin)
      print, 'dx = ', float(dxin)
      print, 'Grid =', (gridtype)
    endif else begin
      print, 'y = ', float(xin)
      print, 'dy = ', float(dxin)
      print, 'Grid =', (gridtype)
    endelse
  endif
  oldst = {x:xin, dx:dxin, gridtype:gridtype}
  if axes eq 1 then xinsav = temporary(oldst) else $ 
    if axes eq 2 then yinsav = temporary(oldst) else $ 
    if axes eq 3 then zinsav = temporary(oldst)
  straxis = axis_lab(axes-1) 
  temp(3+nx) = '#'
  if not keyword_set(quiet) then print, $
  '    initial-X      final-X    Number   Delta X0     Delta XN     R-1(Geo)'
  endline = '  &'
; take care of the ratio
  if n_elements(ratio) eq 0 then ratio = 0.9
  if ratio gt 1.0 or ratio lt 0 then ratio = 0.9
  ratiomax = 1.d0/ratio
;
; find the start location
;
  ok = where (dxin lt 0 , nok)
  if nok eq 0 then ok = 0
  iup = ok(0)
  np = 0
  if ok(0) gt 0 then begin
;   set the initial grid points 
    gxout = xin(iup)
    np = 1
    a = (abs(dxin(iup)))
    for i = iup -1, 0, -1 do  begin
      x0 = xin(i)
      xf = gxout(0)
      dist = xf -x0
      df = a
      lcgeo = 0
        d0 = dxin(i)
        if max([df,d0],min = mm)/mm-1.D0 lt 0.005*dist/(df+d0) or $
          gridtype(i) le 0 then begin
;
;         quadratic grid
;
;         gridtype(i)= 0
          n = round((dist+dist)/(df+d0))>2
          B = (df - dist/n)/n
          A = (dist+dist)/n-df
          if b ne 0.0 then begin
            peakvalue = -a/(b+b)
            while ((peakvalue gt 0.0) and (n gt peakvalue)) do begin
              n = n-1
              if n lt 2 then begin
                print, ' This set of values cannot be used: Xo, Xn, Do, Dn '
                print, X0, xf, d0, df
                return
              endif
              B = (df - dist/n)/n
              A = (dist+dist)/n-df
              peakvalue = -a/(b+b)
            endwhile
          endif
          ind = dindgen(n)
          xtemp = (B * ind + A)* ind + x0
          ind = ind +0.5d0
          if i eq iup-1 then $
            gxouthalf = (B * ind + A)* ind + x0 $
          else gxouthalf = [(B * ind + A)* ind + x0, gxouthalf]
        endif else begin
;
;         geometric grid
;
          lcgeo = 1
          r = exp ((df-d0)/dist)
          if r lt ratio or r gt ratiomax then begin
            if r lt ratio then r= ratio else  r = ratiomax
            d0 = df-dist*alog(r)
;           dxin(i) = d0
          endif
          n = round(alog(df/d0)/alog(r))
          error = 1.
          coef = df/dist 
          ii = 0
          r = refine_rneg (r, dist, df, n)
          a = df /r^n
          ind= dindgen(n)
          xtemp =  (r^ind - 1.0d0)*(a/alog(r))+ x0
          ind = ind +0.5d0
          if i eq iup-1 then $
            gxouthalf = (r^ind - 1.0d0)*(a/alog(r))+ x0 $
          else gxouthalf = [(r^ind - 1.0d0)*(a/alog(r))+ x0, gxouthalf]
          b = 0.0
        endelse
      np = np +n_elements(xtemp)
      gxout = [xtemp, gxout]
      if lcgeo eq 0 then gtyp = '  Quadratic' else gtyp = '  Geometric'
      temp(3+i+1) = '#'+ string(xin(i+1))+ string (dxin(i+1))+ gtyp
      if not keyword_set(quiet) then begin
        if lcgeo eq 1 then print, float(xin(i)), $ 
        float(xin(i+1)), fix(n), float(a), float(df), float(R-1.0d0) else $
        print, float(xin(i)), float(xin(i+1)), fix(n), float(a), float(df)
      endif
      nbin(i) = n
      if i eq nx-2 then endline = ' ' else endline = '  &'
      if maxgridtype eq 0 then $
        temp(nx+4+i+1)= string(n) +  string((a+b)*mult) + $
        string(dist*mult)+ endline
;     if i lt nx-2 then nqs = n-1 else nqs = n
      if lcgeo eq 0 then $
      tempqs (i+1) = ' GRID  '+ strtrim(blocks(i), 2) +straxis+ $
         string(x0*mult) +string(fix(n)) +string(A*mult) + string(B*mult) else $
      tempqs (i+1) = ' GRID  GEOMETRIC  ' +strtrim(blocks(i), 2) +straxis+ $
         string(x0*mult) + string(fix(n)) + string(A*mult) + string(R-1.0D0) 

      temp_a[i] = a*mult
      if lcgeo eq 0 then  temp_b[i] = b*mult else temp_b[i] = R-1.0d0
      temp_x0[i] = x0*mult
      temp_nx[i] = n
                
;    print, temp(nx+5+i)
    endfor
  endif
  maxgx = 256L
  tempgx = make_array (maxgx, /double)
  if np gt 0 then begin
    gxout = [gxout, tempgx ] 
    gxouthalf = [gxouthalf, tempgx ]
    maxgx = maxgx+np
  endif else begin
    gxout = tempgx
    gxouthalf = temporary(tempgx)
  endelse
;
; now go in the positive direction
;
  if iup eq 0 then begin
    gxout(0)=xin(0)
    np = 1
  endif
  if iup lt nx -1 then begin
    df = abs(dxin(iup))
    for i = iup+1, nx-1 do begin
;
    x0 = gxout(np-1)
    xf = xin(i)
    dist = xf-x0
    if dist lt 0 then begin
        print, 'Dist less than zero - STOP', x0, xf, dist
        return
    endif

    lcgeo = 0
      d0 = df
      A = d0
      df = abs(dxin(i))
      if max([df,d0],min = mm)/mm-1.D0 lt 0.005*dist/(df+d0) or $
        gridtype(i-1) le 0 then begin
;
;       quadratic grid
;
;       gridtype(i-1) = 0
        n = round((dist+dist)/(df+d0))>2
        if dxin(i) gt 0 then begin
          B = (dist/n-d0)/n
          df = (dist+dist)/n - a
        endif else begin
          B = (dF-d0)/(n+n)
        endelse
        if b ne 0.0 then begin
          peakvalue = -a/(b+b)
          while ((peakvalue gt 0.0) and (n gt peakvalue)) do begin
            n = n-1
            if n lt 2 then begin
              print, ' This set of values cannot be used: Xo, Xn, Do, Dn '
              print, X0, xf, d0, df
              return
            endif
            if dxin(i) gt 0 then begin
              B = (dist/n-d0)/n
              df = (dist+dist)/n - a
            endif else begin
              B = (dF-d0)/(n+n)
            endelse
            peakvalue = -a/(b+b)
          endwhile
        endif
        ind = dindgen(n) +1.d0
        while  np gt maxgx -n  do begin
          maxgx = 2*maxgx
          gxout = [gxout, gxout]
          gxouthalf = [gxouthalf, gxouthalf]
        endwhile
        gxout(np: np+n-1) = (B * ind + A)* ind + x0
        ind = ind -0.5d0
        gxouthalf(np-1:n+np-2) = (B * ind + A)* ind + x0
        np = np + n
      endif else begin
;
;       geometric grid
;
        lcgeo = 1
        r = exp ((df-d0)/dist)
        if r lt ratio or r gt ratiomax then begin
          if r lt ratio then r= ratio else  r = ratiomax
          df = d0+dist*alog(r)
        endif
        n = round(alog(df/d0)/alog(r))
        if dxin(i) gt 0 then begin
;         dxin(i) = df
          coef = dist/d0
          ii = 0
          r = refine_rpos(r, dist, d0, n)
          df = d0*(r^n)
        endif else begin
          if df ne -dxin(i) then begin
            print, $
            'Warning - Unable to match DX with allowable ratio'
            print, ' Target DX', -dxin(i), ', Actual DX', df
          endif
;         dxin(i) = df
          R = (df/d0)^(1.0d0/n)
        endelse
        ind= dindgen(n) +1.0d0
        while np gt maxgx -n  do begin
          maxgx = 2*maxgx
          gxout = [gxout, gxout]
          gxouthalf = [gxouthalf, gxouthalf]
        endwhile
        gxout(np: np+n-1) = (r^ind - 1.0d0)*(a/alog(r))+ x0
        ind = ind -0.5d0
        gxouthalf(np-1:n+np-2) = (r^ind - 1.0d0)*(a/alog(r))+ x0
        b = 0.0
        np = np + n
      endelse
    if i eq nx-1 then endline = ' '
    if lcgeo eq 0 then gtyp = '  Quadratic' else gtyp = '  Geometric'
    temp(3+i) = '#'+ string(xin(i))+ string (df)+ gtyp
    if maxgridtype eq 0 then $
      temp(nx+4+i)= string(n) +  string((a+b)*mult) + string(dist*mult)+ endline
;   if i eq nx-1 then nqs = n else nqs = n-1
    if lcgeo eq 0 then $
    tempqs (i) = ' GRID  '+strtrim(blocks(i-1), 2)+straxis+ $
       string(x0*mult) +string(fix(n)) + string(A*mult) + string(B*mult) else $
    tempqs (i) = ' GRID  GEOMETRIC  '+strtrim(blocks(i-1), 2) +straxis+ $
       string(x0*mult) + string(fix(n)) + string(A*mult) + string(R-1.0d0)

      temp_a[i-1] = a*mult
      if lcgeo eq 0 then  temp_b[i-1] = b*mult else temp_b[i-1] = R-1.0d0
      temp_x0[i-1] = x0*mult
      temp_nx[i-1] = n

    if not keyword_set(quiet) then begin
        if lcgeo eq 1 then print, float(x0), $ 
        float(gxout(np-1)), fix(n), float(d0), float(df), float(R-1.d0) else $
        print, float(x0), float(gxout(np-1)), fix(n), float(d0), float(df)
    endif
    nbin(i-1) = n
;    print, temp(nx+5)
  endfor
  endif
  gdxin = float(gxout(1:np-1)-gxout(0:np-2))
  gxout = float(gxout(0:np-1))
  gxouthalf = float(gxouthalf(0:np-2))
endelse
  i2w, findgen (np), gxout, 80, clab = 'Grid'
  i2w, findgen (np-1), gdxin, 81, clab = 'Delta X'
  if min(gdxin) le 0.0 then begin
     print, ' This is not acceptable. Delta must be greater than zero.'
     plo, 80, 2, g = 2
     return
  endif
;
  if not keyword_set(quiet) then $
    print,' Total Number of Points = ',strtrim( np ,2)
  if axis eq 1 then begin
    x = xin
    dx = dxin
    gx = (gxout)
    gdx = (gdxin)
    nxbin = nbin
    xgridtype = gridtype
  endif else begin
    y = xin
    ygridtype = gridtype
    dy = dxin
    nybin = nbin
    gy = gxout
    gdy = gdxin
  endelse
  if axes eq 1 then begin
    i2w, findgen(np), gxout, 71, xlab = 'X Axis Index' , ylab = 'Position', $
      clab = 'X Position vs Index'
    dif, 71, 72, clab = 'First Derivative of X-Grid vs Index', y='Delta'
    dif, 72, 73, clab = 'Second Derivative of X-Grid vs Index', y='Delta'
    div , 73,72, 74, clab='Second Deriv / First Deriv vs Index'
    if not keyword_set(quiet) then plo, 71, 4, g = 4, over = 0, color = 0
    if lcnooutput eq 1 then return

    grid_param_1 ={A: temp_a, B: temp_b, x0: temp_x0, n: temp_nx}
    gxfull = gxout
    gxhalf = temporary(gxouthalf)
    i2w, [findgen(np), findgen(np-1)+0.5], [gxfull, gxhalf], 70, clab = $
        'X-Grid and Half Grid'
    temp(2)='#    Initial-X      Delta-X  (Spatial Multiplier = '+ $
         strtrim (mult, 2)+')'
    if maxgridtype eq 0 then begin
      temp(nx+4)= '  x1grid   1 '+ string(fix(np+1))+'   2 '+ $
         string(xin(0)*mult)+'   &' 
      x1grid = temp
    endif else begin
      temp(nx+4) = '  x1grid   2 '+ string(fix(np+1))+'   2 '
      tx = [gx, gxhalf, 2.0*gxhalf(np-2)-gxhalf(np-3)]*mult
      ntx = np+np
      nt = ntx/5
      resid = ntx mod 5
      if resid gt 0 then x1grid = [temp(0: nx+4), make_array(nt+1, /string)] $
        else x1grid = [temp(0: nx+4), make_array(nt, /string)]
      ii = 0
      for i = nx+5,nx+4+ nt do begin
        x1grid(i-1) =  x1grid(i-1) + '   &'
        x1grid(i) = string(tx(ii))
        for ii = ii+1, ii+4 do x1grid(i) = x1grid(i) + string(tx(ii))
      endfor
      if resid gt 0 then begin
        x1grid(i-1) =  x1grid(i-1) + '   &'
        x1grid(i) = string(tx(ii))
        if resid gt 1 then $
        for ij = ii+1, ii+resid-1  do x1grid(i) = x1grid(i) + string(tx(ij))
      endif
    endelse
    qsxgrid = tempqs
    xminimum = min(abs(dx), max = xmaximum)
    if n_elements(yminimum) eq 0 and n_elements(dy) gt 0 then $
      yminimum = min(abs(dy), max = ymaximum)
  endif else if axes eq 2 then begin
    gyfull = (gxout)
    i2w, findgen(np), gyfull, 75, xlab = 'Y Axis Index' , ylab = 'Position', $ 
       clab = 'Y Position vs Index'
    dif, 75, 76, clab = 'First Derivative of Y-Grid vs Index', y='Delta'
    dif, 76, 77, clab = 'Second Derivative of Y-Grid vs Index', y='Delta'
    div , 77,76, 78, clab='Second Deriv / First Deriv vs Index'  
    if not keyword_set(quiet) then plo, 75, 4, g = 4, over = 0, color = 0
    if lcnooutput eq 1 then return

    grid_param_2 ={A: temp_a, B: temp_b, x0: temp_x0, n: temp_nx}
    gyhalf = temporary(gxouthalf)
    i2w, [findgen(np), findgen(np-1)+0.5], [gyfull, gyhalf], 79, clab = $
        'Y-Grid and Half Grid'
    temp(2)='#    Initial-Y      Delta-Y  (Spatial Multiplier = '+ $
      strtrim (mult, 2)+')'
    if maxgridtype eq 0 then begin
      temp(nx+4)= '  x2grid   1 '+ string(fix(np+1))+'   2 '+ $
         string(xin(0)*mult)+'   &'
      x2grid = temp
    endif else begin
      temp(nx+4) = '  x2grid   2 '+ string(fix(np+1))+'   2 '
      tx = [gy, gyhalf, 2.0*gyhalf(np-2)-gyhalf(np-3)]*mult
      ntx = np+np
      nt = ntx/5
      resid = ntx mod 5
      if resid gt 0 then x2grid = [temp(0: nx+4), make_array(nt+1, /string)] $
        else x2grid = [temp(0: nx+4), make_array(nt, /string)]
      ii = 0
      for i = nx+5,nx+4+ nt do begin
        x2grid(i-1) =  x2grid(i-1) + '   &'
        x2grid(i) = string(tx(ii))
        for ii = ii+1, ii+4 do x2grid(i) = x2grid(i) + string(tx(ii))
      endfor
      if resid gt 0 then begin
        x2grid(i-1) =  x2grid(i-1) + '   &'
        x2grid(i) = string(tx(ii))
        if resid gt 1 then $
        for ij = ii+1, ii+resid-1  do x2grid(i) = x2grid(i) + string(tx(ij))
      endif
    endelse
    qsygrid = tempqs
    yminimum = min(abs(dy), max = ymaximum)
    if n_elements(xminimum) eq 0 and n_elements(dx) gt 0 then $
      xminimum = min(abs(dx), max = xmaximum)
  endif else  begin
    gzfull = (gxout)
    i2w, findgen(np), gzfull, 75, xlab = 'Z Axis Index' , ylab = 'Position', $
       clab = 'Z Position vs Index'
    dif, 75, 76, clab = 'First Derivative of Z-Grid vs Index', y='Delta'
    dif, 76, 77, clab = 'Second Derivative of Z-Grid vs Index', y='Delta'
    div , 77,76, 78, clab='Second Deriv / First Deriv vs Index'
    if not keyword_set(quiet) then plo, 75, 4, g = 4, over = 0, color = 0
    if lcnooutput eq 1 then return

    grid_param_3 ={A: temp_a, B: temp_b, x0: temp_x0, n: temp_nx}
    gzhalf = temporary(gxouthalf)
    i2w, [findgen(np), findgen(np-1)+0.5], [gzfull, gzhalf], 80, clab = $
        'Z-Grid and Half Grid'
    temp(2)='#    Initial-Y      Delta-Y  (Spatial Multiplier = '+ $
      strtrim (mult, 2)+')'
    if maxgridtype eq 0 then begin
      temp(nx+4)= '  x3grid   1 '+ string(fix(np+1))+'   2 '+ $
         string(xin(0)*mult)+'   &'
      x3grid = temp
    endif else begin
      temp(nx+4) = '  x2grid   2 '+ string(fix(np+1))+'   2 '
      tx = [gzfull, gzhalf, 2.0*gzhalf(np-2)-gzhalf(np-3)]*mult
      ntx = np+np
      nt = ntx/5
      resid = ntx mod 5
      if resid gt 0 then x3grid = [temp(0: nx+4), make_array(nt+1, /string)] $
        else x3grid = [temp(0: nx+4), make_array(nt, /string)]
      ii = 0
      for i = nx+5,nx+4+ nt do begin
        x3grid(i-1) =  x2grid(i-1) + '   &'
        x3grid(i) = string(tx(ii))
        for ii = ii+1, ii+4 do x3grid(i) = x3grid(i) + string(tx(ii))
      endfor
      if resid gt 0 then begin
        x3grid(i-1) =  x3grid(i-1) + '   &'
        x3grid(i) = string(tx(ii))
        if resid gt 1 then $
        for ij = ii+1, ii+resid-1  do x3grid(i) = x3grid(i) + string(tx(ij))
      endif
    endelse
    qszgrid = tempqs
    zminimum = min(abs(dx), max = zmaximum)
    if n_elements(xminimum) eq 0 and n_elements(dx) gt 0 then $
      xminimum = min(abs(dx), max = xmaximum)
  endelse
; if not keyword_set(quiet) then begin
;   if n_elements(xminimum) gt 0 then $
;     print, ' Minimum dX = ', strtrim(xminimum, 2), $
;       ',  Maximum dX = ', strtrim(xmaximum, 2)
;   if n_elements(yminimum) gt 0 then $
;     print, ' Minimum dY = ', strtrim(yminimum, 2), $
;       ',  Maximum dY = ', strtrim(ymaximum, 2)
;   if n_elements(xminimum) gt 0 and n_elements(yminimum) gt 0 then begin
;     print, ' Minimum dx/dy: ', strtrim(xminimum/ymaximum, 2), $
;       ', Maximum dx/dy: ', strtrim(xmaximum/yminimum, 2)  
;     TEMP = ' Maximum 2D time step is ' + strtrim( $ 
;     sqrt(1./(1./xminimum/xminimum + 1./yminimum/yminimum))*mult/2.9979246e8, 2)
;     print, temp
;   endif
; endif
  if n_elements(cond) gt 0 then cond.lcfit = 0*cond.lcfit
  return
  end
