; ------------------------------------------------------------------------ 
; $Id: tqclean.pro,v 1.3 2007/06/06 20:12:26 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
 pro tqclean, cond_in, color = color, _extra = extrastuff

;+
;  This routine is intended to erase all areas outside a conductor,
;     but requires user intervention to set up the command.
;  See the example.
;  Format: 
;        Tqclean [, cond_in] [, color = color] [, _extra = extrastuff]
;           where 
;             cond_in : Conductor structure from tqhelp
;                    tqhelp, cond = cond_in
;             color is an optional keyword indicating the color of the 
;                conductor.
;             Default is one color = !p.background
;             If color is an array then each conductor segment will be 
;                a different color given by each element of the color array.
;                Colors will be recycled.
;             Example: If color is 3 then all conductors will be blue.
;                If color = [1, 2, 3] then the first conductor is red, 
;                the second is green and the third is blue.  If there are
;                more than 3 conductors then the colors will be recycled, 
;                ie, the fourth conductor will be red, ....
;  
;  Example:
;     Assume you are plotting field data with the following command:
;         plotstr, 3, 3, 2, 0, /xint, con= 1
;       where you are plotting a plane normal to the second coordinate
;     and you observe the contour lines passing through the conductor.
;     These lines can be erased with the following series of commands
;     where the conductors are assumed to be in structure 1 and the grid
;     in structure 2:
;        tqreadcon, 1, 3, 1, 0, /reset
;        tqedit, /cond
;           in the editor modify the conductors so that they extend to the
;           boundary and are closed or would be closed if first and last 
;           points are connected:
;           for example: figure on left might have conductors on right
;              ___________________             __________
;              |        |         |            |         |
;              | (cond) |         |            |         |
;              |________|         |    ________|         |
;              |         (vac)    |   |                  |
;              |__________________|   |                  |
;
;              change the conductor to a closed block as shown below:
;              __________
;              |        |
;              | (cond) |
;              |________|
;              
;             
;
;     Note: tqsave may be used to save the conductor configuration
;     Now do the plot:
;         plotstr, 3, 3, 2, 0, /xint
;         tqclean
;         plotstr, 1, /over    ; plot the conductors if desired last
;     Note:
;         After the geometry has be set in TQEDIT, TQCLEAN may be called 
;         as many times as desired.
;-
 on_error, 1
@pfidltq.com
 if size(cond_in, /type) ne 8 then con= cond else con = cond_in 
 if n_elements(color) eq 0 then color = !p.background
 nc = n_elements(color)
 loc = con.locx
 for i = 0, con.ncon-1 do $
 polyfill, [con.x[con.locx[i]:(con.locx[i+1]-1)], con.x[con.locx[i]]], $
          [con.y[con.locx[i]:(con.locx[i+1]-1)], con.y[con.locx[i]]], $
           color = color[ i mod nc], noclip =0
 
 tickf = make_array(30, value = ' ')
 plot, con.x, con.y, /nodata, /noerase, xrange= !x.crange, yrange= !y.crange, $
  position=[!x.window[0], !y.window[0], !x.window[1], !y.window[1]], $
  xticknam= tickf, yticknam = tickf, /xst, /yst, _extra = extrastuff
 return
 end
