; ------------------------------------------------------------------------ 
; $Id: comidl.pro,v 1.6 2008/02/26 20:21:42 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
 Pro comidl, u, v, xin, yin, amp, tshft, baslin, sdd, sdr, ifitin, ier, $
     errlim = errlim, initdel = initdel, correl = correl, weight = weight, $
     only_tshift = only_tshift
;+
;
;     format:
;   comidl, u, v, x, y, amp, tshft, baslin, sdd, sdr, ifit, ier, 
;       errlim = errlim, initdel = initdel, correl = correl
;
;     perform a least-square-fit of two data sets
;     least square fit of form -- u(t)=amp*v(t+tshft)+baslin(2)*t+baslin
;
; arguments
;     x and y are two arrays to be fit
;     u is a working array returned with x data used for the fit
;     v is a working array returned with y data used for the fit
;         v data will be scaled by the lqfit value of the amplitude
;     u,v,x,y-- are standard format arrays
;        1st element is initial time
;        2nd element is the time increment between elements
;        3rd, ...  are the values
;     ifit determines the type of fit
;          if ifit = 0, baslin=0.
;          if ifit = 1, baslin is varied to obtain best fit
;          if ifit = 2, baslin assumed to be of form(a + b*time)
;                       a is returned as baslin(1) and b is baslin(2)
;     ier .gt. 100  then sd=0.0 to within computer accuracy
;        number of iterations is ier-100
;      ier .gt. 0 and le. 30   then ier is number of iterations required
;     ier=0 - no fit obtained in 30 iterations or no data overlap
;     ier.lt.0,number of iterations    to point at which no change observed
;                 in standard deviation
;     sdd is standard deviation, sdr=sdd/abs(max(x))
;     correl is the correlation between the two waveforms
;     errlim is the error limit for convergence. Default is 3.125% of the
;            minimum abscissa spacing
;     initdel is the initial delta for the iteration. Default is 5% of the
;            domain.  If this value is not zero, the second array is 
;            shifted by this amount.
;     only_tshift - if set, only shift time with an amplitude multiplier of 1
;                   Only for ifit = 0 or 1
;
;  The technique involves calculating a standard deviation at one location
;     and moving a delta earlier and later in time and calculating the
;     std. deviations at these locations.  The search then estimates the
;     location of the minimum and moves to that location.  If the move is small
;
;     then the value of delta is decreased.  Because of the large number of
;     points and the possibility large offsets in the time base, real*8
;     variables will be used.
;
;  The program is organized as follows:
;     1.  Calculate fit parameters at a point (LABEL = 100)
;     2.  Move to a different point and repeat 1 until we have 3 points.
;     3.  Calculate the minimum.
;     4.  Return to 1 or return depending on the change.
;
;     6/25/93 initial idl version of compare
;- 
;
      on_error, 1
;
;     set the criteria for convergence to 3.125% of the spacing
      if n_elements(errlim) eq 0 then errlim = 3.125e-2 else $
          errlim = abs(errlim)
;     set the minimum number of points to 4
      npmin = 4
;     initialize the values in case we fail
      u = 0.0
      v = 0.0
      amp = 0.0
      baslin= [0.0, 0.0]
      sdd=0.0
      sdr=0.0
      ier=0
      if n_elements(ifitin) eq 0 then ifit = 0 else ifit = round(ifitin)
      if (ifit lt 0 or ifit gt 2) then ifit =0
      if keyword_set(only_tshift) then ifit = 0
;     find the minimum point spacing
      delt = min ([xin(1), yin(1)])
;  Convert X and Y to double precision variables
      if (xin(1) ne delt) then begin
        np = n_elements(xin)-2
        sp = findgen(np)*xin(1)+xin(0)
        im = xin(2:*)
        put2uniform, sp, im, np, xin(0), delt, sp(np-1), x
        x = double(x)
      endif else x = double(xin)
      nptx = n_elements(x)
      if (yin(1) ne delt) then begin
        np = n_elements(yin)-2
        sp = findgen(np)*yin(1)+yin(0)
        im = yin(2:*)
        put2uniform, sp, im, np, yin(0), delt, sp(np-1), y
        y = double(y)
      endif else y = double(yin)
      npty = n_elements(y)
      if n_elements(weight) eq 0 then weight = 0 else weight = weight[0]
      if (ifit eq 2) && (weight ne 0) then begin
        weight = 0
        print, '% COMIDL:  Weight set to zero with SLOPING BASELINE'
      endif
      w=0
      if weight gt 0 then begin
        lcweight = 1
        get_wdfarray, wa, weight
        np = n_elements(wa)-2
        if np lt 2 then lcweight = 0 else begin
          sp = wa(0) +findgen(np)*wa(1)
          spu = xin(0) +findgen(nptx-2)*delt
          wend =  sp(np-1)
          ok = where (spu ge sp(0) and spu le wend, nok)
          if nok eq 0 then lcweight = 0 else begin
            w = make_array(nptx, value = 1.0)
            put2uniform,sp,wa(2:*),np,spu[ok[0]],delt,spu[ok[nok-1]],ww,/yonly
            wok = where (ww le 0.0, wnok)
            if wnok gt 0 then ww(wok)=1.0 
            w[2+ok[0]:2+ok[nok-1]] = ww
            w[0:1] = [xin[0], delt]
          endelse
        endelse
      endif else if weight lt 0 then begin
        w= float(x)
        ok = where (w(2:*) le 0.0, nok)
        if nok gt 0 then w(2+ok)= 1.0
        if weight eq -101 then w(2:*) = 1.0/sqrt(w(2:*))
        if weight eq -102 then  w(2:*) = 1.0/w(2:*)
        if weight eq -1 then w(2:*) = sqrt(w(2:*))
        if weight eq -2 then  w(2:*) = w(2:*)
        lcweight = -1
      endif else lcweight = 0
      weight = w
      errdel = errlim
      errdel2 = 2*errdel
      t = systime(1)
      cross = [0, 1]
      if n_elements(initdel) eq 0 then begin
        nx = nptx -2
        ny = npty -2
        ntot = nx+ny -1
        if ntot ge 1800 then  begin
          thin = long(ntot/600)
          nx = (nptx-2)/thin
          ny = (npty-2)/thin
          nm = ny-1
          ntot= nx+ny-1
          cross = make_array(ntot)
          xx = x(2+lindgen(nx)*thin)
          yy = y(2+lindgen(ny)*thin)
          nmin = min([nx, ny]) -1
          for i = 1L, ntot do begin
            overlap = min([i-1, ntot-i, nmin])
            jx = (0L > (i-ny))
            jy = ((ntot-i)< nm)
            cross(i-1) = total(xx(jx:jx+overlap)*yy(jy -overlap:jy), /double)
          endfor
          a = max(abs(cross), imax)
          initdel = thin*delt*(imax -ny+1)-y(0)+x(0)
        endif else begin
          thin = 1
          cross = make_array(ntot)
          nm = ny-1
          nmin = min([nx, ny]) -1
          for i = 1L, ntot do begin
            overlap = min([i-1, ntot-i, nmin])
            jx = (0 > (i-ny))+2L
            jy = ((ntot-i)< nm)+2L
            cross(i-1) = total(x(jx:jx+overlap)*y(jy -overlap:jy), /double)
          endfor
          a = max(abs(cross), imax)
          initdel = delt*(imax -ny+1)-y(0)+x(0)
        endelse
      endif else begin
        thin = 0
        initdel = initdel(0)
      endelse
;         print,' initdel = ', initdel
      y(0) = y(0)+initdel
;     put initial and final values in terms of elements
      xinit = 0.0
      yinit = (y(0)-x(0))/delt
      tfx = nptx-3
      tfy = yinit + npty-3
      tt = dindgen(max([nptx, npty]))
;     set the initial step to 20 times the minimum spacing
      if thin gt 0 then delta = 2*thin else begin
        delta = min([tfx, tfy]) - max([xinit, yinit])
        delta = delta*0.05 > 5
      endelse
      deltat = delta
      correl = 0.0
      m = 1
      sdp = 1.0e35
      tp= [0.0, 0.0, 0.0]
      sd= [0.0, 0.0, 0.0]
      tpp = 0.0
      iter = 1
      lfinish = 0
start:
      tp(m)=(m-1)* delta+tp(1)
      tm = tp(m)
      ti = max([xinit, yinit-tm])
      tf = min([tfx, tfy-tm])
      if tf-ti lt npmin  then return
      np = long(tf -ti)+1
      tist = long(ti)+2
      tistf = ti+2 -tist
      tistr = 1.0 -tistf
      if tistf lt 3.0e-4  or tistr lt 3.0e-4 then begin
        i= round(ti+2L)
        xx = x(i: i+np-1)
        if lcweight ne 0 then ww = w[i: i+np-1]
      endif else begin
        xx = x(tist:tist+np-1)*tistr + x(tist+1:tist+np)*tistf
        if lcweight ne 0 then $
          ww = w(tist:tist+np-1)*tistr + w(tist+1:tist+np)*tistf
      endelse
      ty = ti+tm-yinit+2L
      tist = long(ty)
      tistf = ty -tist
      tistr = 1.0 -tistf
      if tistf lt 3.0e-4  or tistr lt 3.0e-4 then begin
        i= round(ty)
        yy = y(i: i+np-1)
      endif else begin
        yy = y(tist:tist+np-1)*tistr + y(tist+1:tist+np)*tistf
      endelse
      if lcweight eq 0 then begin
        sxy = total (xx*yy, /double)
        sx2 = total (xx*xx, /double)
        sy2 = total (yy*yy, /double)
        xnpm = float(np)
      endif else begin
        xxw = xx*ww
        sxy = total (xxw*yy, /double)
        sx2 = total (xx*xxw, /double)
        sy2 = total (yy*yy*ww, /double)
        xnpm = total (ww, /double)
      endelse
      if ifit gt 0 then begin
        sx = total(xx, /double)
        sy = total(yy, /double)
        if ifit gt 1 then begin
            t = tt(0:np-1)
            st = np*(np-1)*0.5
            stx = total (t*xx, /double)
            sty = total (t*yy, /double)
            stt = total (t*t, /double)
        endif
      endif
      if lfinish gt 0 then begin
        u = make_array(np+2)
        v = make_array(np+2)
        ti = x(0) + ti*delt
        u(0:1) = [ti, delt]
        v(0:1) = [ti, delt]
        xmax = max(abs(xx))
        u(2:*) = xx
        v(2:*) = yy
      endif
      xover = xnpm
;     xnpm = float(np-1)
      if (ifit eq 0) then begin
;
; amplitude fit
;
        blinp=0.0
        slopep=0.0
        conp = 1.0
        if keyword_set(only_tshift) then begin
          if ifitin eq 1 then begin
            blinp = (total(xx, /double) -  total(yy, /double))/xnpm
            sd[m] = total(((xx-blinp)-yy)^2, /double)/xnpm
          endif else sd[m] = total((xx-yy)^2, /double)/xnpm
        endif else begin
          if sy2 ne 0 then conp= sxy/sy2 
          sd(m) = (sx2-sxy*conp)/xnpm
        endelse
;
      endif else if(ifit eq 1) then begin
;
; uniform baseline shift
;
        slopep=0.0
        conp= (sxy-sx*sy/xover)/(sy2-sy*sy/xover)
        blinp=(sx-conp*sy)/xover
        sd(m) = ((conp*sy2-sxy-sxy)*conp-xover*blinp*blinp+sx2)/xnpm
      endif else begin

;
; the only thing left -- sloping baseline
;
        c1= sy2-sy*sy/xover
        c2= sty-st*sy/xover
        c3= sx2-sx*sx/xover
        c4= stt-st*st/xover
        c5= sxy-sx*sy/xover
        c6= stx-sx*st/xover
        denom=c1*c4-c2*c2
        conp=(c5*c4-c2*c6)/denom
        slopep=(c1*c6-c5*c2)/denom
        blinp=(sx-conp*sy-slopep*st)/xover
        sd(m)= (slopep*(slopep*c4-c6-c6)+conp*(conp*c1-c5-c5+ $
          (slopep*c2)+(slopep*c2))+c3)/xnpm
      endelse
;
; This is the normal way to leave
;     Before we go, need to scale the v plot array and set the constants.
;
      if (lfinish gt 0) then begin
        if (sd(1) ge 0.0) then sdd= sqrt(sd(1)) else sdd= 0.0
        amp = conp
        tshft=tp(1)*delt - initdel
; Scale the second plot array
        if (ifitin eq 0) then v(2:*) = v(2:*)*amp $
          else if (ifitin eq 1) then v(2:*) = v(2:*)*amp + blinp $
          else v(2:*) = v(2:*)*amp + blinp + slopep*findgen(np)
        sdr = sdd
        if ifit eq 2 then begin
          slopep = slopep/delt
          blinp = blinp-slopep*ti
        endif
        baslin= [blinp, slopep]
        xmax = max(abs(u(2:*)))
        if (xmax gt 0.0) then sdr=sdr/xmax
        correl=correlate(u(2:*), v(2:*))
        return
      endif
      if( sd(m) le 0.0) then begin
;
; We have a fit to better than the accuracy of the computer.
;     Get ready to exit
;
        tp(1) = tp(m)
        lfinish = 1
        m=1
        ier = iter + 100
        goto, start
      endif
      sd(m)=sqrt(sd(m)/xnpm)
;
; We now have the fit parameters for this m-value.  If we have all of them,
;     then we look for a minimum; otherwise, calculate the rest of them.
;
      if (m eq 1) then begin
        if(sd(1) gt sdp) then begin
;
; The error on the midpoint is greater than one of the points which
;     we calculated last time.  Take that point as midpoint and reduce delta.
;
          delta=delta*0.5
          sd(1) = sdp
          tp(1) = tpp
        endif
        m=0
;
; calculate parameters at a negative time shift
;
        goto, start
      endif else if (m eq 0) then begin
;
; calculate parameters at a positive time shift
;
        m=2
        goto, start
      endif
;
; we have all the fit parameters calculated at the three different time shifts.
;     now we need to analyze the results.
; First we will find the smallest value of the standard deviation
;     to insure that we don't get any worse.
;
      sdp = min(sd, j)
      tpp = tp(j)
;
; We will now fit the three points to a parabola and look for the extreme.
; Make sure that we are not trying to do a parabolic fit on a straight line.
;
      denom = sd(0)-sd(1)-sd(1)+sd(2)
      if(abs(denom) lt 1.0e-6*sd(1))then begin
;
; The standard deviations are an approximate straight line so check the slope
;
        if (abs(sd(2)-sd(0)) le 1.0e-6*sd(1))then begin
;
; no significant change in the standard deviation so quit
;
          ier = -iter
          m=1
          lfinish = 1
          goto, start
        endif
;
; Finite slope to the standard deviation so take the smallest value
;     and increase the delta to attempt to get to a minimum
;
        sd(1)=sdp
        tp(1) = tpp
; Do not allow delta to become larger than 20% of the overlap
        delta = delta*1.5 < 0.2*xnpm
;
; Do another iteration -- calculate parameters at a negative time shift
;
        m=0
        iter=iter+1
        if (iter gt 30) then begin
;
; Send the latest values of the fit and stop
;
          m=1
          lfinish =1
          goto, start
        endif
        goto, start
      endif
;
; We know that the three std. deviations are not a straight line so
;     calcualte delta time to the minimum (hopefully!!) of the parabola.
; We do not want to take too big a step.  Limit it to 2.0 delta
;
      deltat = delta*0.5*(sd(0)-sd(2))/denom < (delta+delta)
      deltat = deltat > (-(delta + delta))
;
; Evaluate the standard deviation at the point we have chosen using the
;     parabolic fit to the standard deviations as an estimate.
;
      stest=sd(1)+0.25*deltat/delta*(sd(2)-sd(0))
      if(stest gt 1.00002*sdp) then begin
;
; The standard deviations fit is concave downward so take the smallest
;     value and increase the delta to attempt to get to a minimum
;
        sd(1)=sdp
        tp(1) = tpp
;
; Do not allow delta to become larger than 20% of the overlap
;
        delta = delta*1.50 < 0.2*xnpm
;
; Do another iteration -- calculate parameters at a negative time shift
;
        m=0
        iter=iter+1
        if (iter gt 30) then begin
          m=1
          lfinish =1
          ier = 0
          print, ' More than 30 iterations'
        endif
        goto, start
      endif
;
; Are we finished?
; We shall require a small step size with also a small delta
;
      if (abs(deltat) lt errdel and delta le errdel2) then begin
;
; We did it!!
; Calculate the exact parameters at this point and exit
;
        ier = iter
        m=1
        lfinish =1
        tp(1) = tp(1) + deltat
        goto, start
;
      endif else if (abs(deltat) lt 0.5*delta) then begin
;
; reduce delta to the step size or by 1/10 but not less than 2x error limit.
;
        delta= max([abs(deltat),0.1*delta,errdel2])
      endif
      m=1
      tp(1)=tp(1)+deltat
      iter=iter+1
      if (iter gt 30) then begin
        ier = 0
        print, ' More than 30 iterations'
        return
      endif
      goto, start
      end
