; ------------------------------------------------------------------------
; $Id: get_dxp.pro,v 1.1 2010/12/08 17:55:22 lpmix Exp $
; ------------------------------------------------------------------------
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
;+
; NAME:
;       GET_DXP
;       NOTE: This is the same as GET_DX.pro
;             This was added to avoid a conflict with GET_DX in Kyle Cochran's tools
;
; PURPOSE:
;       Return the spacing of a WDF array
;	Returns 0 if the WDF array is nonuniform
;       The WDF array is not changed
;
; CATEGORY:
;       Analysis
;       Utility
;
; CALLING SEQUENCE:
; 
;       Delta =GET_DX(WDFarray [, ERROR = error] [, DXRANGE = dxrange] [, /MODIFY])
;
;       DELTA:   The calculated spacing of the abscissa for WDFarray.
;                If DELTA is zero, spacing is not uniform.
;                If DELTA is negative, abscissa is decreasing.
;                If DELTA is positive, abscissa is increasing.
;                If MODIFY is set, the order will be changed and DELTA is positive.
;
; INPUTS:
;       WDFarray: WDF array number or unique search string
;       Note:  Some WDF arrays are stored as X-Y Data, but have uniform 
;              spacing between X values.
;       Note:  The spacing is considered uniform is the variation in the 
;              spacing is less than an error parameter
;              (See Procedure and keyword ERROR)
;
; KEYWORD PARAMETERS:
;       ERROR:    Error tolerance for determining if the abscissa is uniform
;                 ERROR is expressed as a fraction of the range of the X data.
;                 If specified ERROR = ERROR > ERROR_MIN
;                 See PROCEDURE below.
;                 Default: ERROR = (3.05176E-5) > ERROR_MIN
;                 NOTE:  ERROR_MIN is defined as follows for RANGE = MAX(X)-MIN(X):
;                        ERROR_MIN =  (MACHAR()).EPS * MAX([RANGE, MAX(ABS(X))]) / RANGE 
;                        A warning will be printed if ERROR < ERROR_MIN .
;                 NOTE:  (MACHAR()).EPS is about 1.2e-7 (=1.0/2L^23) for many computers
;                        and  3.05176E-5 = 1./2L^15
;       DXRANGE:  Range of DX-Spacing.  Two elements with the minimum and maximum
;                 spacing.  For uniform data, these numbers will be equal.
;       MODIFY:   If set, change the array to a uniform array if the X-spacing 
;                 is uniform.  DEFAULT: MODIFY = 0
;                 If MODIFY is set, DELTA will be positive and the order reversed.
;
; OUTPUTS:
;       If the abscissa is approximately uniform, the delta for the dataset
;       is returned.  If the abscissa is not uniform, zero is returned.
;
; SIDE EFFECTS:
;       If MODIFY is set and the abscissa is uniform, the array is converted from 
;       an X-Y array to a uniform spacing array.
;
; PROCEDURE:
;       If the WDF array is stored as uniform data, the spacing is returned.
;       If the WDF array is stored as X-Y data, then the spacing is calculated.
;     The procedure for X-Y data is:
;       Calculate the maximum and minimum spacing of the abscissa.
;         NELEMENTS = number of points
;         DX is the spacing.
;         DXRange = [min(DX), max(DX)]
;       Calculate the range of X
;         RANGE = max(X) - min(X)
;       If (DXRange[1] -DXRange[0]) / RANGE < ERROR, then the waveform is uniform.
;         DELTA = RANGE / (NELEMENTS -1) 
;         DELTA is returned
;
; EXAMPLE:
;       Determine the X-spacing for WDF array 4 and store it in DX4
;         DX4 = GET_DX(4)
;
;       Determine if WDF array 6 is uniform.
;         if GET_DX(6) gt 0 then print, 'WDF array 6 is uniform'
;
; MODIFICATION HISTORY:
; 	Written by:	L. P. Mix, Jr., September 23, 2010
;
;-

Function GET_DXP, WDFARRAY,  ERROR = error, DXRANGE = dxrange, MODIFY = modify

on_error, 1
@com.dat
npar = n_params()

delta = 0
if npar eq 0 then begin 
  print, '% Format: Delta = GET_DX(WDFarray [, ERROR = error] '
  print, '                    [/MODIFY] [, DXRANGE = dxrange])'
  return, delta
endif

if n_elements(wdfarray) eq 0 then begin
  print, '% GET_DX:  WDF array must be defined'
  return, delta
endif
wdf = wdvalid(wdfarray[0], npoint = np,/valid)
if np eq 0 then begin
  print, '% GET_DX: Unable to determine a valid WDF array'
  return, delta
endif
wdf = wdf[0]
if np gt 1  then begin
  print, '% GET_DX: WDFarray is not a unique specifier'
  print, '          Only the first will be used: '+strtrim (wdf, 2)
endif
if n_elements(modify) eq 0 then modify = 0
wdfm = wdf-1
np = wdfnp[wdfm]
npm = np -1  

get_wdfarray, array, wdf, /xydata

if n_elements(array) eq np+2 then begin
  delta = array[1]
  dxrange = [Delta, Delta]
  return, delta
endif
 
 x = (size(/type, array) lt 4) ? float(array[0:npm]) : (array[0:npm])
 array = array[np:*]
 range = max(x, min = m)-m
 dxx = x[1:npm]- x[0:npm-1]
 dxrange = [min(dxx, max = m), m]
 lcrev = 0
 if dxrange[0] lt 0.0 && dxrange[1] gt 0 then return, delta
 if dxrange[0] lt 0.0 then begin 
   x = reverse(x)
   array = reverse(array)
   dxx = x[1:npm]- x[0:npm-1]
   dxrange = [min(dxx, max = m), m]
   lcrev= 1
 endif
 err = (n_elements(error) eq 0) ? 3.05176E-5 : error[0]
 err *= range
 err_min = (machar()).eps *max(abs([range, x[0], x[npm]]))
 if err lt err_min then begin
   print, '% CHECK_UNIFORM: Error increased from '+strtrim(err, 2)+' to ' + $
           strtrim(err_min, 2)
   err = temporary(err_min)
 endif
if (dxrange[1] - dxrange[0]) lt err then delta = range/npm
if keyword_set(modify) && (delta gt 0) then $
   put_wdfarray, [x[0], delta, array], wdf, /no_copy else $
   if lcrev then delta = -delta
return, delta 
END
