; ------------------------------------------------------------------------ 
; $Id: get_symbol_points.pro,v 1.9 2010/07/21 20:02:46 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
 pro Get_symbol_points, wdf, x, y, npoints=npoints, valid=valid, xonly=xonly, $
         range = range, spacing = spacing, set_default= set_default, grid = grid
; range is the range of the data or the ratio of the range
;+
; Name:
;      GET_SYMBOL_POINTS
; 
; Purpose:
;      Calculate x values and y values for symbols and error bars.
; Format: 
;      GET_SYMBOL_POINTS, WDF, X, Y [, NPOINTS = npoints] [, VALID = valid]
;                          [ , /Xonly] [, RANGE = range] [, spacing = spacing]
; where 
;      WDF     - WDF array number
;      X,Y     - X and Y points for the symbol
;      NPOINTS - Number of points desired
;                Default value set in WDERR
;      Valid   - Number of points returned
;      Xonly   - If set, return only X values
;      spacing - Offset in the spacing, Default = 0.25
;                Spacing must be in the interval [0, 1]
;      GRID    - If set, range is set to the grid not the data
;                DEFAULT: GRID = 0
;      set_default - If set, the spacing will be stored in the common block
;      From range of data and range of plot determine the range of abscissa 
;      values.
;           delta = range/(npoints)
;           mmod = long (1./spacing)
;           x-values = X-Min + (findgen(npoints)+(wdf mod mmod)*spacing)*delta
;      
;      Original version: April 8, 1998, lpm
;-

 COMPILE_OPT hidden

@com.dat
;
 common wderror_common, npoints_save, width_save
;
 common plot_grid_last, yeng_scale, xeng_scale

 common com_get_symbol_points, spacing_offset
 on_error, 1
      nparams = n_params()
      if n_elements(spacing_offset) eq 0 then spacing_offset = 0.25
      if keyword_set(set_default) then begin
        if n_elements(spacing) gt 0 then spacing_offset = spacing[0]
        if nparams lt 2 then return
      endif
      if n_elements(spacing) eq 0 then spac =  spacing_offset else $
        spac = spacing[0]
      if n_elements(initwdf) eq 0 then wdfinit
      valid = 0
      range = 0
      x = 0
      y = 0
      if n_elements(npoints_save) eq 0 then begin
        npoints_save = 18
        width_save = 0.01
      endif
      if n_elements(grid) eq 0 then grid = 0
      if nparams lt 2 then begin
        print,'% GET_SYMBOL_POINTS: Two parameters are required'
        return
      endif
      if n_elements(wdf) eq 0 then begin
        print, '% GET_SYMBOL_POINTS: Parameter must be defined'
        return
      endif
      if wdf lt 1 or wdf gt maxwdfarray then begin
        print,'% GET_SYMBOL_POINTS: WDF array must be greater than 0 and </=', $
        maxwdfarray
        return
      endif
      if n_elements(npoints) eq 0 then npoints = npoints_save > 1 else $
        if npoints le 0 then npoints = npoints_save >1
      xxmin = min(!x.crange, max = xxmax)
      if !x.type ne 0 then begin
        xxmax = 10.^xxmax
        xxmin = 10.^xxmin
      endif
      if keyword_set(grid) then begin
        xxmin = xxmin/xeng_scale
        xxmax = xxmax/xeng_scale
        range = (!x.type eq 0) ? xxmax-xxmin : xxmax/xxmin
        xxmin = max([xxmin, wdfminx(wdf-1)])
        xxmax = min([xxmax, wdfmaxx(wdf-1)])
      endif else begin
        xxmin = max([xxmin/xeng_scale, wdfminx(wdf-1)])
        xxmax = min([xxmax/xeng_scale, wdfmaxx(wdf-1)])
        range = (!x.type eq 0) ? xxmax-xxmin : xxmax/xxmin
      endelse
      if xxmax le xxmin then begin
        if wdfminx[wdf-1] eq wdfmaxx[wdf-1] and wdfnp[wdf-1] lt npoints then begin
          w2i, x, y, wdf
          valid = n_elements(x)
          return
        endif
        print, '% GET_SYMBOL_POINTS: Calculated extent of the error is zero.'
        return
      endif
      valid = npoints
      mmod = (spac gt .05) ? round(1./spac) : 1L
      get_wdfarray, arr, wdf, /xy
      nel = n_elements(arr)
      if nel eq (wdfnp(wdf-1)+2) then begin
        if !x.type eq 0 then begin
          delta = (xxmax-xxmin)/npoints
          x= (xxmin+(0.1+(wdf mod mmod)*spac+findgen(npoints))*delta) < xxmax
        endif else begin
          ratio = xxmax/xxmin
          pr = ratio^(1.0/npoints)
          x= sqrt(pr)*xxmin*(pr^lindgen(npoints))
        endelse
        if keyword_set(xonly) then begin
          y = 0
          return
        endif
        y = get_value(wdf, x)
      endif else begin
        nel = nel/2
        ok = where( arr(0:nel-1) ge xxmin and arr(0:nel-1)le xxmax , nok)
        if nok gt 1.5*npoints then begin
          skip = float(nok)/npoints
          offset = wdf mod mmod 
          p =(round(findgen(npoints)+ offset*spac)*skip) < (nok-1)
          ok = ok(p)
        endif
        if ok[0] lt 0 then valid = 0 else begin
          x = arr(ok)
          y = arr(ok+nel)
        endelse
      endelse
      if valid gt 0 then valid = n_elements(x)
      npoints = n_elements(x)
      END
