; ------------------------------------------------------------------------ 
; $Id: hip.pro,v 1.4 2011/08/15 21:12:56 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
 Pro  HIP, wdf1, wdf2, atten = filter, $
      order = order, type = type, mode = mode, plot = plot, wdf_array = array
;
;+
; NAME:
;       HIP
;
; PURPOSE:
;       High pass filter utility; called by FILT
;       (See FILT for additional details)
; CATEGORY:
;       Filter
;
; CALLING SEQUENCE:
;       HIP, WD1 [, WD2][, TYPE=TYPE][, ATTEN=ATTEN][,ORDER = ORDER] $
;            [, MODE = MODE] [, /PLOT]     
; INPUTS:
;       WD1      WDF array number of the array to be filtered
; OPTIONAL INPUT PARAMETERS:
;       WD2      WDF array number for storage of the filtered array
; KEYWORD PARAMETERS:
;       TYPE     Type of filter desired ('BUT', 'EXP', 'USER', 'IDEAL').
;                Default is 'BUT'
;       ATTEN    Frequency domain filter function.  Must be supplied with
;                type = 'USER' and have same number of values as dataset WD1.
;       ORDER    Order of the filter
;                Default is 2
;       MODE     Center frequency of the high pass filter expressed in 
;                units of mode numbers.  Default is 12
;       PLOT     If present, the frequency domain filter function is plotted 
;                If PLOT is a two element array, then this is used as XRANGE
;                  in the plot command.
;                If PLOT is a one element array and negative then the filter
;                  function will be saved in -PLOT if it is a valid WDF array.
;                  Otherwise the filter function will be saved in MAX_WDF_ARRAY.
;                  Note: MAX_WDF_ARRAY = get_maxwdf()
; OUTPUTS:
;       NONE     The filtered array is stored in a WDF array in the common block
; OPTIONAL OUTPUT KEYWORD PARAMETERS:
;       ATTEN    If TYPE is not 'USER', ATTEN contains the freq domain
;                filter function.
; COMMON BLOCKS:
;       com.dat  WDF common block
; RESTRICTIONS:
;       If type is 'USER', ATTEN must be provided
; PROCEDURE:
;       The Fourier transform of WD1 is taken and multiplied by a frequency
;       domain filter function.  The inverse Fourier transform is taken, and
;       the real values stored in WD2 (or WD1 if WD2 is not provided).
; MODIFICATION HISTORY:
;      Original version, L. P. Mix, May 18, 1992
;
;-
;
@com.dat
;
      if n_elements(initwdf) eq 0 then wdfinit
      nparams = n_params()
      if nparams lt 1 then begin
        print,' HIP must have at least 1 parameter'
        return
      endif
;
      if n_elements(wdf1) eq 0 then begin
        print, 'Parameters to HIP must be defined'
        return
      endif
      if wdf1 lt 1 or wdf1 gt maxwdfarray then begin
        print, 'wdf array must be greater than 0 and </=', maxwdfarray
        return
      endif
;
      if n_elements(wdf2) eq 0 then save = wdf1 else save = wdf2
      if save lt 1 or save gt maxwdfarray then save = wdf1
      nel = wdfnp(wdf1-1)
      if nel eq 0 then begin
        print, 'No valid data in the first array'
        return
      endif
      lcarray = 1
      if n_elements(array) eq 0 then begin
         get_wdfarray, array, wdf1
         lcarray = 0
      endif

      nel = n_elements(array)-2
      if n_elements(order) gt 0 then order = abs(order) > 0.01 else order = 2
      total = nel* array(1)
      if n_elements(mode) gt 0 then bin = abs(mode) < 2L*nel else $
        bin = 12
      if n_elements(type) gt 0 then begin
        type = strtrim(type, 2)
        type = strupcase(type)
        type = strmid(type, 0, 3)
        case type of 
  'BUT' : filtertype = 'BUTTER'
  'EXP' : filtertype = 'EXPON'
  'IDE' : filtertype = 'IDEAL'
  else  : filtertype = 'USER'
        endcase
      endif else  filtertype = 'BUTTER'
      if filtertype eq 'USER' then begin
        if n_elements (filter) ne nel then begin
          print, 'Filter has wrong number of elements, filter = ' , $
            n_elements(filter)
          print,'  Array has elements = ', nel
          return
        endif
        if n_elements(plot) gt 0 then begin
          distfun = findgen(nel)
          distfun = distfun < (nel - distfun)
        endif
      endif else begin
        distfun = findgen(nel)
        distfun = distfun < (nel - distfun)
        distfun[0] = 1
        case filtertype of 
  'BUTTER' : filter =1.0/(1.0+(bin/distfun)^(2*order))
  'EXPON'  : filter =1.0- exp((-(distfun/bin)^order))
  'IDEAL'  : filter =1.0- (distfun le bin)
   else    : print,'Error in call to HIP, type = ' , filtertype
        endcase
        filter[0] = 0.0
        distfun[0] = 0.0
      endelse
;
      a = fft(array(2:*), -1)
      b = float(fft(a*filter, 1))
      array(2:*) = b
      a = 0
      if keyword_set (plot) then begin
        a = distfun /total
        oldwin = !D.window
        window, /free, title= 'High Pass Filter Function'
        x = shift (a, nel/2)
        x(0:nel/2) = -x(0:nel/2)
        y = make_array(nel+2)
        y(2:*)=shift(filter, nel/2)
        if n_elements(plot) gt 1 then $
          plot, x, y(2:*), xrange= plot(0:1) else plot, x, y(2:*)
        y(0)= x(0)
        y(1)= 1.0/total
        if n_elements(plot) eq 1 and plot(0) lt 0 then begin
          fsav= -fix(plot(0))
          if fsav lt 1 or fsav gt maxwdfarray then fsav = maxwdfarray
        endif else fsav = maxwdfarray
        put_wdfarray, y, fsav, clab = 'High Pass Filter Function', $
          xlab = 'Frequency', ylab = 'Filter', /no_copy
        hak, /mes
        wdelete
        wset , oldwin
      endif      
      if lcarray ne 0 then return

      put_wdfarray, array, save, /no_copy
      wdf2 = save
      wdfcomment(save-1)= wdfcomment(wdf1-1)
      wdffile(save-1)= wdffile(wdf1-1)
      wdfxlabel(save-1)= wdfxlabel(wdf1-1)
      wdfylabel(save-1)= wdfylabel(wdf1-1)
      return
      end
