; ------------------------------------------------------------------------
; $Id: image_con.pro,v 1.5 2007/06/06 19:34:30 lpmix Exp $
; ------------------------------------------------------------------------
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
 Pro image_con, str1, str2, reflection = reflection, periodic = periodic, $
          again = again, $
          value = value, invert = invert, nperiods = nperiods, CLABEL = clabel
;+
; NAME:
;       image_con
;
; PURPOSE:
;       Generate an image file from data which has a reflection plane or
;         has periodic boundaries.
;
; CATEGORY:
;       Math
; CALLING SEQUENCE:
;       IMAGE_CON, STR1, [, REFLECTION = reflection]
;              [, PERIODIC = PERIODIC] [, VALUE = value] [, /INVERT]
;              [, NPERIODS = NPERIODS] [, CLABEL = clabel] [, /REPEAT]
; INPUTS:
;       STR1:    Structure array of the conductor
; KEYWORDS:
;       REFLECTION : If present must equal to +/- 1, 2, or 3.
;                    + indicates maximum value; - indicates minimum value
;                    for the reflection.
;                    1, 2, 3 indicates coordinates 1, 2, or 3
;       VALUE      : If a different value is required for the reflection,
;                    value is used to specify this point. Value must be greater
;                    than the maximum spatial value of the data or less than
;                    the minimum to avoid multiple values at a point.
;       INVERT     : If present and not zero, multiple the field values by -1.
;       PERIODIC   : If present must equal to +/- 1, 2, or 3.
;                    + indicates maximum value; - indicates minimum value
;                    for the translation.
;                    1, 2, 3 indicates coordinates 1, 2, or 3
;       NPERIODS   : Number of replications for the periodic data, default = 1
;       AGAIN      : If present and not zero, all parameters will be the same
;                    as the last call to IMAGE_CON
;       CLABEL:  Optional label for the combined structures.
; OUTPUTS:
;       NONE    The composite image is stored in a structure array
; RESTRICTIONS:
;       STR1 array must be defined and have valid field (or image) data
;
; MODIFICATION HISTORY:
;      Original version, L. P. Mix, April 21, 1994
;-
;
; insert a common block
;
 common image_com, reflect_sav, period_sav, lcneg_sav, value_sav, nperiod_sav
;
@qscom.dat
;  Common block
@qsstrcom.dat
;       return on an error
      on_error, 1
;
      if n_elements (structinit) eq 0 then qsstruct_init
      if n_elements (qsplotinit) eq 0 then qsinit
;
      nparams = n_params()
      if nparams lt 1 then begin
        print,' IMAGE_CON must have at least 1 parameters'
        return
      endif
      b = size(str1)
      if  b(b(0)+1) ne 8 then begin
            print, $
            ' IMAGE_CON: First parameter must be a conductor structure'
            return
       endif
;
      arr = str1
      if arr.type ne 4 then begin
        print, ' IMAGE_CON may only be used with conductor structures'
        return
      endif
      nblo = arr.block
      mxbl = nblo
      ndim = arr.ndim
      if n_elements (reflect_sav) eq 0 then again = 0
      if n_elements (again) eq 0 then again = 0
      if again eq 0 then begin
        if n_elements (reflection) eq 0 then reflection = 0
        if n_elements (periodic) eq 0 then periodic = 0
        if n_elements (nperiods) eq 0 then nperiods = 1
        if (reflection eq 0 and periodic eq 0)  then begin
          print, 'Reflection or periodic dimension must be specified'
          return
        endif
      endif else begin
        reflection = reflect_sav
        periodic = period_sav
        if lcneg_sav ne 0 then begin
          reflection = -reflection
          periodic = -periodic
        endif
        value = value_sav
        nperiods = nperiod_sav
      endelse
      tplane = arr.plane
      tslant = arr.slant
      range = make_array(2, 3)
      range(*, 0) = [min(tplane(*, 0, *), max=mp),mp]
      range(*, 1) = [min(tplane(*, 1, *), max=mp),mp]
      range(*, 2) = [min(tplane(*, 2, *), max=mp),mp]
      if n_elements (tslant) gt 3 then begin
        range(*,0)=[min([range(0,0),min(tslant(0, *, *), max= mm)]), $
             max([range(1,0), mm])]
        range(*,1)=[min([range(0,1),min(tslant(1, *, *), max= mm)]), $
             max([range(1,1), mm])]
        range(*,2)=[min([range(0,2),min(tslant(2, *, *), max= mm)]), $
             max([range(1,2), mm])]
        lcslant = 1
      endif else lcslant = 0
      if reflection ne 0 then begin
        if abs(reflection) lt 1 or abs(reflection) gt 3 then begin
          print,'Reflection keyword must be in the range +/-1 to +/-3'
          return
        endif
        if reflection lt 0 then begin
          reflection = - reflection
          lcneg = 1
        endif else lcneg = 0
        err = 6.1e-5*max(abs(range(*, reflection-1, 0)))
        if n_elements(value) eq 0 then begin
          if lcneg eq 0 then value = range(1, reflection-1) $
            else value = range(0, reflection-1)
        endif else begin
          if value gt range(0, reflection-1)+err and $
              value lt range(1, reflection-1)-err then begin
            print, $
            'Specified value for reflection must be outside the data range'
            return
          endif
        endelse
        periodic = 0
      endif else begin
        if abs(periodic) lt 1 or abs(periodic) gt 3 then begin
          print,'Periodic keyword must be in the range +/-1 to +/-3'
          return
        endif
        reflection = 0
        if periodic lt 0 then begin
          periodic =-periodic
          lcneg = 1
        endif else lcneg = 0
        err = 6.1e-5*max(abs(range(*, periodic-1, 0)))
        if n_elements (value) eq 0 then begin
          delta = range(1, periodic-1)-range(0, periodic-1)
          value = delta
        endif else begin
          delta = range(1, periodic-1)-range(0, periodic-1)
          if delta -2.0*err gt value then begin
            print,  $
            ' Value of periodic spacing must be greater than the data range'
            return
          endif
          value = delta
        endelse
        pvalue = range(1, periodic-1)
      endelse
      npl = arr.size(4)
      if periodic ne 0 then begin
        b = size(tplane)
        ssize = arr.size*(1+nperiods)
        plane = make_array(2, 3, b(3)*(1+nperiods))
        plane (*,*, 0:b(3)-1) = tplane
        for n = 1, nperiods do begin
          tp = tplane
          tp(*, periodic-1, *) = tplane (*, periodic-1, *)+delta*n
          plane(*, *, n*b(3):(n+1)*b(3)-1) = tp
        endfor
        if lcslant gt 0 then begin
          bs = size(tslant)
          slant = make_array(3, 2,bs(3)*(1+nperiods))
          slant (*,*, 0:bs(3)-1) = tslant
          for n = 1, nperiods do begin
            tp = tslant ( periodic-1,*, *)+delta*n
            plane(*, *, n*b(3):(n+1)*b(3)-1) = tp
          endfor
        endif else slant = 0.0000
        n0=arr.n0
        if n0(0) ge 0 then begin
          nt = arr.n0
          for i = 1, nperiods do n0 = [n0, nt+i*npl]
        endif
        n1=arr.n1
        if  n1(0) ge 0 then begin
          nt = arr.n1
          for i = 1, nperiods do n1 = [n1, nt+i*npl]
          if periodic eq 1 then begin
            nel= n_elements (n1)
            for i = 1, nperiods do begin
              if nel gt 0 then begin
                b = pvalue +(i-1)*delta
                ok = where (plane(0, 0, n1) lt b-err or $
                   plane(0,0,n1) gt b+err, nel)
                if nel gt 0 then n1= n1(ok) else n1 = -1
              endif
            endfor
            ssize(1) = nel
          endif
        endif
        n2=arr.n2
        if n2(0) ge 0 then begin
          nt = arr.n2
          for i = 1, nperiods do n2 = [n2, nt+i*npl]
          if periodic eq 2 then begin
            nel= n_elements (n2)
            for i = 1, nperiods do begin
              if nel gt 0 then begin
                b = pvalue +(i-1)*delta
                ok = where (plane(0, 1, n2) lt b-err or $
                   plane(0,1,n2) gt b+err, nel)
                if nel gt 0 then n2= n2(ok) else n2 = -1
              endif
            endfor
            ssize(2) = nel
          endif
        endif
        n3=arr.n3
        if n3(0) ge 0 then begin
          nt = arr.n3
          for i = 1, nperiods do n3 = [n3, nt+i*npl]
          if periodic eq 3 then begin
            nel= n_elements (n3)
            for i = 1, nperiods do begin
              if nel gt 0 then begin
                b = pvalue +(i-1)*delta
                ok = where (plane(0, 2, n3) lt b-err or $
                   plane(0,2,n3) gt b+err, nel)
                if nel gt 0 then n3= n3(ok) else n3 = -1
              endif
            endfor
            ssize(3) = nel
          endif
        endif
;
      endif else begin
        b = size(tplane)
        ssize = arr.size*2
        plane = make_array(2, 3, b(3)*2)
        plane (*,*, 0:b(3)-1) = tplane
        tp = tplane
        tp(0, reflection-1, *) = 2*value -tplane (1, reflection-1, *)
        tp(1, reflection-1, *) = 2*value -tplane (0, reflection-1, *)
        plane(*, *, b(3):2*b(3)-1) = tp
        if lcslant gt 0 then begin
          bs = size(tslant)
          slant = make_array(3, 2,bs(3)*2)
          slant (*,*, 0:bs(3)-1) = tslant
          for n = 1, nperiods do begin
            tp = tslant
            tp( reflection-1,*, *) = 2*value-tslant ( reflection-1,*, *)
;           help, slant, tp,slant[*, *, bs(3):(2*bs(3)-1)]
            slant[*, *, bs(3):(2*bs(3)-1)] = tp
          endfor
        endif else slant = 0.0000
        n = arr.n0
        if n(0) ge 0 then begin
          n0 = [n, n+npl]
          ssize(0) = n_elements (n0)
        endif else n0 = n
        n = arr.n1
        if n(0) ge 0 then begin
          n1 = [n, n+npl]
          if reflection eq 1 then begin
            ok = where (plane(0, 0, n1) lt value-err or $
                  plane(0,0,n1) gt value+err, nel)
            if nel gt 0 then n1= n1(ok) else n1 = -1
            ssize(1) = nel
          endif
        endif else n1 = n
        n = arr.n2
        if n(0) ge 0 then begin
          n2 = [n, n+npl]
          if reflection eq 2 then begin
            ok = where (plane(0, 1, n2) lt value-err or $
                  plane(0,1,n2) gt value+err, nel)
            if nel gt 0 then n2= n2(ok) else n2 = -1
            ssize(2) = nel
          endif
        endif else n2 = n
        n = arr.n3
        if n(0) ge 0 then begin
          n3 = [n, n+npl]
          if reflection eq 3 then begin
            ok = where (plane(0, 2, n3) lt value-err or $
                  plane(0,2,n3) gt value+err, nel)
            if nel gt 0 then n3= n3(ok) else n3 = -1
            ssize(3) = nel
          endif
        endif else n3 = n
      endelse
;
      if n_elements(clabel) gt 0 then clab = string(clabel) $
        else clab = arr.clab
      str1 = {type:4L, ndim:ndim, block:mxbl, $
              size:ssize, plane: plane,  slant:slant, $
              n0:n0, n1:n1, n2:n2, n3:n3, xlab:arr.xlab, $
              file: arr.file, spare:arr.spare, tlab : arr.tlab, $
              clab:clab }
;
      if again eq 0 then begin
        reflect_sav = reflection
        period_sav = periodic
        lcneg_sav = lcneg
        value_sav = value
        nperiod_sav = nperiods
      endif
;
      return
      end
