; ------------------------------------------------------------------------
; $Id: image_grd.pro,v 1.3 2007/06/06 19:34:30 lpmix Exp $
; ------------------------------------------------------------------------
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
 Pro image_grd, str1, reflection = reflection, periodic = periodic, $
          again = again, $
          value = value, invert = invert, nperiods = nperiods, CLABEL = clabel
;+
; NAME:
;       image_grd
;
; PURPOSE:
;       Generate a grid file from data which has a reflection plane or
;         has periodic boundaries.
;
; CATEGORY:
;       Math
; CALLING SEQUENCE:
;       IMAGE_GRD, STR1, [, REFLECTION = reflection]
;              [, PERIODIC = PERIODIC] [, VALUE = value] [, /INVERT]
;              [, NPERIODS = NPERIODS] [, CLABEL = clabel] [, /REPEAT]
; INPUTS:
;       STR1:    Structure array of the conductor
; KEYWORDS:
;       REFLECTION : If present must equal to +/- 1, 2, or 3.
;                    Negative sign is ignored
;                    + indicates maximum value; - indicates minimum value
;                    for the reflection.
;                    1, 2, 3 indicates coordinates 1, 2, or 3
;       VALUE      : If a different value is required for the reflection,
;                    value is used to specify this point. Value must be greater
;                    than the maximum spatial value of the data or less than
;                    the minimum to avoid multiple values at a point.
;       PERIODIC   : If present must equal to +/- 1, 2, or 3.
;                    Negative sign is ignored
;                    + indicates maximum value; - indicates minimum value
;                    for the translation.
;                    1, 2, 3 indicates coordinates 1, 2, or 3
;       NPERIODS   : Number of replications for the periodic data, default = 1
;       AGAIN      : If present and not zero, all parameters will be the same
;                    as the last call to IMAGE_CON
;       CLABEL:  Optional label for the combined structures.
; OUTPUTS:
;       NONE    The composite image is stored in a structure array
; RESTRICTIONS:
;       STR1 array must be defined and have grid data
;
; MODIFICATION HISTORY:
;      Original version, L. P. Mix, April 21, 1994
;-
;
; insert a common block
;
 common image_com, reflect_sav, period_sav, lcneg_sav, value_sav, nperiod_sav
;
@qscom.dat
;  Common block
@qsstrcom.dat
;       return on an error
      on_error, 1
;
      if n_elements (structinit) eq 0 then qsstruct_init
      if n_elements (qsplotinit) eq 0 then qsinit
;
      nparams = n_params()
      if nparams lt 1 then begin
        print,'% IMAGE_GRD:  Must have at least 1 parameters'
        return
      endif
      b = size(str1)
      if  b(b(0)+1) ne 8 then begin
            print, $
            '% IMAGE_GRD: First parameter must be a conductor structure'
            return
       endif
;
      arr = str1
      if arr.type ne 3 then begin
        print, '% IMAGE_GRD: Structure must be a GRID (type = 3) structure'
        return
      endif
      nblo = arr.block
      mxbl = nblo+nblo
      ndim = arr.ndim
      if arr.two eq 0 and ndim eq 3 then begin
         print, '% IMAGE_GRD: Structure must be two dimensional or'
         print, '             3-D with a degenerate dimension'
         return
      endif
      if n_elements (reflect_sav) eq 0 then again = 0
      if n_elements (again) eq 0 then again = 0
      if again eq 0 then begin
        if n_elements (reflection) eq 0 then reflection = 0
        if n_elements (periodic) eq 0 then periodic = 0
        if n_elements (nperiods) eq 0 then nperiods = 1
        if (reflection eq 0 and periodic eq 0)  then begin
          print, '% IMAGE_GRD: Reflection or periodic dimension must be specified'
          return
        endif
      endif else begin
        reflection = reflect_sav
        periodic = period_sav
        if lcneg_sav ne 0 then begin
          reflection = -reflection
          periodic = -periodic
        endif
        value = value_sav
        nperiods = nperiod_sav
      endelse
      range = arr.range
      ar = round (abs(reflection))
      ap = round (abs(periodic))
      if ar gt 0 then begin
        periodic = 0
        if ar gt 3 then begin
     print,'% IMAGE_GRD: Reflection keyword must be in the range +/-1 to +/-3'
          return
        endif
        if reflection lt 0 then lcneg = 1 else lcneg = 0
        err = 6.1e-5*max(abs(range[*, ar-1, 0]))
        if n_elements(value) eq 0 then begin
          if lcneg eq 0 then value = range[1, ar-1, 0] $
            else value = range[0, ar-1, 0]
        endif else begin
          if value gt range(0, ar-1)+err and $
              value lt range(1, ar-1)-err then begin
            print, $
  '% IMAGE_GRD: Specified value for reflection must be outside the data range'
            return
          endif
        endelse
        periodic = 0
      endif else begin
        if ap gt 3 then begin
          print,'% IMAGE_GRD: Periodic keyword must be in the range +/-1 to +/-3'
          return
        endif
        reflection = 0
        if periodic lt 0 then lcneg = 1 else lcneg = 0
        err = 6.1e-5*max(abs(range(*, ap-1, 0)))
        if n_elements (value) eq 0 then begin
          delta = range[1, ap-1, 0]-range[0, ap-1, 0]
          value = delta
        endif else begin
          delta = range[1, ap-1, 0] - range[0, ap-1, 0]
          if delta -2.0*err gt value then begin
            print,  $
  '% IMAGE_GRD: Value of periodic spacing must be greater than the data range'
            return
          endif
          value = delta
        endelse
        pvalue = range[1, ap-1, 0]
      endelse
      if ar eq arr.two or ap eq arr.two then begin
        print, '% IMAGE_GRD: Reflection/periodic dimension may not be degenerate'
        return
      endif
 
      reflection = ar
      periodic = ap

      x1 = arr.x1
      x2 = arr.x2
      if ndim eq 3 then x3 = arr.x3 else x3 = 0.0
    if keyword_set(periodic) and nblo eq 1 then begin
      mxbl = 1L
      locx = arr.locx
      x1lab = arr.x1lab
      x2lab = arr.x2lab
      x3lab = arr.x3lab
      ssize = arr.size
      order = 0L
      n = ssize[*, 0]
      if periodic eq 1 then begin
        xt = x1(1:*)
        nx1 = n_elements(xt)
        for i = 1, nperiods do x1 = [x1, xt+i*delta]
        nx = n_elements(x1)
        locx[0, 1] = nx
        range(1, 0, *) = x1(nx-1)
        ssize[0, 0] = nx
      endif else if periodic eq 2 then begin
        yt = x2(1:*)
        nx1 = n_elements(yt)
        for i = 1, nperiods do x2 = [x2, yt+i*delta]
        nx = n_elements(x2)
        range[1, 1, *] = x2(nx-1)
        ssize(1, 0) = nx
        locx[1, 1] = nx
      endif else begin
;       periodic = 3
        yt = x3(1:*)
        for i = 1, nperiods do x3 = [x3, yt + i*delta]
        nx = n_elements(x3)
        range(1, 2, *) = x3(nx-1)
        ssize(1, 0) = nx 
        locx[2, 1] = nx
      endelse
    endif else begin
;   do periodic and
      locx = make_array(ndim, 2*nblo+1)
      locx(*, 0:nblo)= arr.locx
      lx = arr.locx(*, nblo)
      x1lab = [arr.x1lab, arr.x1lab]
      x2lab = [arr.x2lab, arr.x2lab]
      x3lab = [arr.x3lab, arr.x3lab]
      range = make_array(2, ndim, mxbl+1)
      range[*,*,0:nblo] = arr.range

      ssize = make_array(ndim, mxbl, /long)
      ssize(*, 0:nblo-1)= arr.size
     if periodic ne 0 then begin
        for iblo = 1, nblo do begin
          if arr.two eq 1 then begin
            x1 =[x1, x1(0)]
            dx1 =1L
          endif else begin
            xt = x1(locx(0,iblo-1):locx(0,iblo)-1)
            dx1 = (locx(0,iblo)-locx(0,iblo-1))
            if periodic eq 1 then begin
              for i = 1, nperiods do x1= [x1, xt+i*delta]
              dx1 = dx1*nperiods
            endif else x1= [x1, xt]
          endelse
          if arr.two eq 2 then begin
            x2 =[x2, x2(0)]
            dx2= 1L
          endif else begin
            xt = x2(locx(1,iblo-1):locx(1,iblo)-1)
            dx2 = (locx(1,iblo)-locx(1,iblo-1))
            if periodic eq 2 then begin
              for i = 1, nperiods do x2= [x2, xt+i*delta]
              dx2 = dx2*nperiods
            endif else x2= [x2, xt]
          endelse
          if ndim eq 3 then begin
            if arr.two eq 3 then begin
              x3 =[x3, x3(0)]
              dx3= 1L
            endif else begin
              xt = x3(locx(2,iblo-1):locx(2,iblo)-1)
              dx3 = (locx(2,iblo)-locx(2,iblo-1))
              if periodic eq 3 then begin
                for i = 1, nperiods do x3= [x3, xt+i*delta]
                dx3 = dx3*nperiods
              endif else x3= [x3, xt]
            endelse
            lx = lx+[dx1, dx2, dx3]
            ssize (*, iblo+nblo-1)= [dx1, dx2, dx3]
            locx(*, iblo+nblo)=lx
          endif else begin
            lx = lx+[dx1, dx2]
            ssize (*, iblo+nblo-1)= [dx1, dx2]
            locx(*, iblo+nblo)=lx
          endelse
          range(0, 0, iblo+nblo) = x1(locx(0,iblo+nblo-1))
          range(1, 0, iblo+nblo) = x1(locx(0,iblo+nblo)-1)
          range(0, 1, iblo+nblo) = x2(locx(1,iblo+nblo-1))
          range(1, 1, iblo+nblo) = x2(locx(1,iblo+nblo)-1)
          if ndim eq 3 then begin
            range(0, 2, iblo+nblo) = x3(locx(2,iblo+nblo-1))
            range(1, 2, iblo+nblo) = x3(locx(2,iblo+nblo)-1)
          endif
        endfor
      endif else begin
;     reflection
        for iblo = 1, nblo do begin
          if arr.two eq 1 then begin
            x1 =[x1, x1(0)]
            dx1= 1L
          endif else begin
            xt = x1(locx(0,iblo-1):locx(0,iblo)-1)
            dx1 = (locx(0,iblo)-locx(0,iblo-1))
            if reflection eq 1 then $
              xt = rotate(2.*value -xt, 2)
            x1= [x1, xt]
          endelse
          if arr.two eq 2 then begin
            x2 =[x2, x2(0)]
            dx2= 1L
          endif else begin
            xt = x2(locx(1,iblo-1):locx(1,iblo)-1)
            dx2 = (locx(1,iblo)-locx(1,iblo-1))
            if reflection eq 2 then $
              xt = rotate(2.*value -xt, 2)
            x2= [x2, xt]
          endelse
          if ndim eq 3 then begin
            if arr.two eq 3 then begin
              x3 =[x3, x3(0)]
              dx3= 1L
            endif else begin
              xt = x3(locx(2,iblo-1):locx(2,iblo)-1)
              dx3 = (locx(2,iblo)-locx(2,iblo-1))
              if reflection eq 3 then $
                xt = rotate(2.*value -xt, 2)
              x3= [x3, xt]
            endelse
            lx = lx+[dx1, dx2, dx3]
            ssize (*, iblo+nblo-1)= [dx1, dx2, dx3]
            locx(*, iblo+nblo)=lx
          endif else begin
            lx = lx+[dx1, dx2]
            ssize (*, iblo+nblo-1)= [dx1, dx2]
            locx(*, iblo+nblo)=lx
          endelse
          range(0, 0, iblo+nblo) = x1(locx(0,iblo+nblo-1))
          range(1, 0, iblo+nblo) = x1(locx(0,iblo+nblo)-1)
          range(0, 1, iblo+nblo) = x2(locx(1,iblo+nblo-1))
          range(1, 1, iblo+nblo) = x2(locx(1,iblo+nblo)-1)
          if ndim eq 3 then begin
            range(0, 2, iblo+nblo) = x3(locx(2,iblo+nblo-1))
            range(1, 2, iblo+nblo) = x3(locx(2,iblo+nblo)-1)
          endif
        endfor
      endelse
;
      for i = 0, ndim -1 do begin
        range(0, i, 0)=min(range(0,i,1:mxbl))
        range(1, i, 0)= max(range(1,i,1:mxbl))
      endfor
;
    endelse
;
      if n_elements(clabel) gt 0 then clab = strtrim(clabel, 2) $
        else clab = arr.clab

;     Structure for grid data
      str1 = {type:3L, ndim:3L,  block :mxbl, two:arr.two, range:range, $
              size: ssize, locx : locx, slice: arr.slice,  $
              x1:x1, x2:x2, x3:x3, x1lab : x1lab, x2lab :x2lab, $
              x3lab : x3lab, proc: 0, file : arr.file, spare :arr.spare, $
              tlab : arr.tlab, clab: clab }


;
      if again eq 0 then begin
        reflect_sav = reflection
        period_sav = periodic
        lcneg_sav = lcneg
        value_sav = value
        nperiod_sav = nperiods
      endif
;
      end
