; ------------------------------------------------------------------------ 
; $Id: reduce_iarray.pro,v 1.3 2007/06/06 19:34:32 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
 Function reduce_iarray, arrayin,dim, y, yvalin, integ
;+
;  Reduce by one the number of dimensions of an array at a given value or 
;    over a specified range of values
;  Note: yvalin must have two values if integ is not equal to zero
;          Integral is defined according to the following:
;          integ(0) = init
;          integ(i) = integ(i-1) +
;           (A(i)+A(i-1))(x(i)-x(i-1))/2.0 * {((x(i)+x(i-1))/2.0)^(integ-1)}
;            where A(i) is the value of the function at x(i)
;  Format is: newarray = reduce_iarray(array,dim, yarray, yvalue, integ )
;    where;
;      NEWARRAY- array with reduce number of dimensions
;      ARRAY   - original array
;      DIM     - Dimension to be eliminated
;      YARRAY  - Spatial coordinates for the eliminated dimension
;      YVALUE  - Spatial coordinate at the location for the slice to be taken
;                or two values - a minimum and maximum over which the array 
;                is averaged
;      INTEG   - Indicates if an integral is to be performed; Default = 0.
;              Values may be 1,2, 3 to indicate if the integral is to be
;                Int(dy), int(y*dy), int(y*y*dy)
;  Example:
;    Eliminate the second dimension from PICTURE. Spatial values are Y and 
;      the slice is to be taken between y = 1.0 and 1.6
;      array  = reduce_array(picture,2, y, [1.0, 1.6])
;-
     array = arrayin
     if n_elements (array) eq 0 then begin
       Print, 'REDUCE_IARRAY: Multi-dimensional array must be defined'
       return, 0.0
     endif
     b = size (array)
     if b[0] eq dim-1 then return, reform (array)
     if b(0) lt dim  then begin
       print,'REDUCE_IARRAY:'
       print,'   Cannot eliminate dimension ',strtrim (dim, 2),$
         ' with a ',strtrim(b(0), 2),' dimensional array'
       return, 0.0
     endif
;    if the array is already degenerate in this dimension then quit
;
     ny = b(dim)
     if ny eq 1 then return, reform (array)
     if ny ne b(dim) then begin
       print,'REDUCE_IARRAY:'
       print, ' Ordinate array must be defined and have the same ' 
       print, ' number of element as the array'
       return, 0.0
     endif
;
     if dim eq 1 then begin
       i= b(2) & j = b(3) & k = b(1)
     endif else if dim eq 2 then begin
       i= b(1) & j = b(3) & k = b(2)
     endif else begin
       i= b(1) & j = b(2) & k = b(3)
     endelse
     if n_elements (integ) eq 0 then integ =0
     if integ lt 0 or integ gt 3 then begin
       print, 'REDUCE_IARRAY: Integral flag must be in the range [1,3]'
       print, '              Integral flag set to 1'
       integ = 1
     endif
     if integ gt 0 and b(0) ne 3 then begin
       print, 'REDUCE_IARRAY: Integral flag limited to 3D arrays'
       return, 0.0
     endif
     yval = yvalin
     nyval = n_elements (yvalin)
     if nyval eq 0 then begin
       print,'REDUCE_IARRAY:'
       print, ' Ordinate value for the lineout must be defined'
       return , 0.0
     endif else if n_elements(yval) eq 1 then begin
       yval = yval > 1
       yval = yval < ny
       if integ ne 0 then return, make_array(i, j) else $
         return, reform(select_dim(array, b(0), dim, yval-1))
     endif
     yval = [min(yval, max = mx), mx]
     yvall = yval(0)>1
     yvalu = yval(1) < ny
     nyval = yvalu - yvall
     if nyval lt 0 then return, 0.0
     if integ gt 0 then begin
       if nyval eq 0 then return,  make_array(i, j)
       iyl = yvall-1
       iyu = yvalu-1
       knew = 1+iyu-iyl
       if dim eq 3 then begin
         array = z_gridinteg(array(*, *, iyl:iyu), y(iyl:iyu), integ-1)
       endif else if dim eq 2 then begin
         temp = make_array(i, j, knew)
         for i = iyl, iyu do temp(*,*,i-iyl) = reform(array(*, i, *))
         array = z_gridinteg(temp, y(iyl:iyu), integ-1)
       endif else begin
         temp = make_array(i, j, knew)
         for i = iyl, iyu do temp(*,*,i-iyl) = reform(array(i, *, *))
         array = z_gridinteg(temp, y(iyl:iyu), integ-1)
       endelse
       return , reform(array(*,*,knew-1)-array(*,*,0))
     endif
     new =  reform(select_dim(array, b(0), dim, yvall-1))
     if nyval eq 0 then return, new
     for j = yvall +1 , yvalu do $
       new = new +reform(select_dim(array, b(0), dim, j-1))
     return, new/(nyval+1)
     end
