; ------------------------------------------------------------------------
; $Id: select_waveforms.pro,v 1.6 2010/01/22 18:08:04 lpmix Exp $
; ------------------------------------------------------------------------
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
;+
; NAME:
;       SELECT_WAVEFORMS
;
; PURPOSE:
;	Allow user to select waveforms of interest
;
; CALLING SEQUENCE:
;       SELECT_WAVEFORMS, WDF, WDFSEL
;
; INPUTS:
;       WDF:    Array of WDF arrays or a search string
;               WDVALID is used with VALID keyword to return WDF arrays
;       WDFSEL: ARRAY of selected waveforms
;               If WDFSEL = 0 then no waveforms selected.
;               If WDFSEL = -1 then QUIT was selected.
;	
; KEYWORD PARAMETERS:
;	XRANGE: Plots of the waveform will use this range.
;               Default:  XRANGE = 0
;       ZOOM:   If set, ZOOMW will be used to examine waveforms
;               If not set, PLO will be used to examine waveforms
;               Default: ZOOM =0 
;       TITLE:  Title for the widget
;               Default:  TITLE = 'Select Desired Waveform Arrays'
;       NWDFSEL: Number of waveforms selected
;               If WDFSEL is negative, QUIT was selected.
;       PARAMETER:  Optional editable parameter for each waveform.
;       KEEP_PARAM: If set, only passed parameters may be used.
;               Selection assume the nearest value.
;       LABEL:  Label for editable parameter.  
;               DEFAULT:  LABEL = 'Energy'
; OUTPUTS:
;       WDFSEL is an array of the selected waveform numbers.
;
; EXAMPLE:
;       Select good waveforms from the waveforms with "TGS" in the comment
;       Store the good wdf arrays in array GOOD.
;         SELECT_WAVEFORMS, 'tgs', good
;
;
; MODIFICATION HISTORY:
; 	Written by:	L. P. Mix, June 11, 2004
;-

pro select_wdf_event, event
widget_control, event.top, get_uvalue = info, /no_copy
widget_control, event.id, get_uvalue = uvalue
uval = strmid(uvalue, 0, 1)
case uval of 
;  PLOT data
  'P': Begin
       wd = 0
       reads, strmid(uvalue, 1), wd
       if keyword_set(info.zoom) then $
         zoomw, (info.wdf)[wd], /modal, group= event.top $
       else $
         plo, (info.wdf)[wd], xrange = info.xrange, /xstyle
       widget_control, event.top, set_uvalue = info, /no_copy
       END
;  Select data
  'S': Begin
       wd = 0
       reads, strmid(uvalue, 1), wd
       (*info.ptr)[wd] = event.select
       widget_control, event.top, set_uvalue = info, /no_copy
       END
;  Options
  'O': Begin
       if event.value eq 0 then begin 
;        Set all
         val =(total(*info.ptr) gt info.nwdf/2) ? 0 : 1 
         for i = 0, info.nwdf-1 do (*info.ptr)[i] = val
         for i = 0, info.nwdf-1 do $
           widget_control, info.butt[i], set_button = val
         widget_control, event.top, set_uvalue = info, /no_copy
       endif else if event.value eq 1 then begin
         (*info.ptr)[0] = -1
         widget_control, event.top, /destroy
       endif else begin
         if info.npar gt 0 then begin
           for i = 0, info.npar-1 do begin
             widget_control, info.para[i], get_value =v
             if (info.keep eq 0) then (*info.ptrp)[i] = v $
               else begin
               j = min(abs(info.param-(double(v))[0]), ii)
               (*info.ptrp)[i] = info.param[ii] 
             endelse
           endfor
         endif
         widget_control, event.top, /destroy
       endelse
       END
; Parameters from CW_FIELD
  'F': Begin
       wd = 0
       reads, strmid(uvalue, 1), wd
       if (info.keep eq 0) then (*info.ptrp)[wd] = event.value[0] $
         else begin
         j = min(abs(info.param-event.value[0]), ii)
         (*info.ptrp)[wd] =info.param[ii]
       endelse
       widget_control, info.para[wd], set_value =  (*info.ptrp)[wd]
       widget_control, event.top, set_uvalue = info, /no_copy
       END

; error
   ELSE: Begin
         Print, '% SELECT_WDF_EVENT: Unexpected event'
         Help, uvalue
         help, /str, event
         widget_control, event.top, set_uvalue = info, /no_copy
         end
   ENDCASE
  
end

pro select_waveforms, wdfin, wdfsel, xrange = xrange, zoom = zzz, $
                      title = title, nwdfsel = nwdfsel, parameters = param, $
                      label = label, keep_param = keep_param
on_error, 2 
@draw_defaults.com
nparams = n_params()

if nparams lt 2 then begin
        print,'% SELECT_WAVEFORMS: Two parameters are required'
        return
endif
if n_elements(wdfin) eq 0 then begin
        print,'% SELECT_WAVEFORMS: WDF arrays must be defined'
        return
endif else wdf = wdfin

if not arg_present(wdfsel) then begin
  print,  $
    '% SELECT_WAVEFORMS: Variable for the selected waveforms must be provided'
  return
endif

wd = wdvalid(wdf, /valid, /quiet, npoint = nwdf)
if nwdf eq 0 then begin
  print,  $
    '% SELECT_WAVEFORMS: No valid waveforms for the input value of WDF'
  return
endif

if n_elements(title) eq 0 then TITLE = 'Select Desired Waveform Arrays'
if n_elements(xrange) eq 0 then xrange = 0
zzz = (keyword_set(zzz)) ? 1 : 0
bbase = make_array(nwdf, /long)
butt = make_array(nwdf, /long)
para = make_array(nwdf, /long)
pad = '   '
;  Check on parameters
;
npar = n_elements(param) < nwdf
If npar gt 0 then if (npar lt nwdf) then $
  param = [param, replicate(param[npar-1], nwdf-npar)]
if n_elements(label) eq 0 then label = 'Energy:' else label = label[0]
;
  
tlb = widget_base(Title = title, /col, $
  x_scroll_size = 400, y_scroll_size=bigsize[1] )
for i = 0, nwdf-1 do begin
  bbase[i] = widget_base(tlb, /row, /frame)
  butbase = widget_base(bbase[i], /nonexclusive)
  butt[i] = widget_button(butbase,  value = pad+wdcomment(wd[i])+pad, $
         uvalue = 'S'+ strtrim(i, 2))
  if npar gt 0 then para[i] = cw_field(bbase[i], /return, /row,  $
    title = label, value = param[i], uvalue =  'F'+ strtrim(i, 2))
  plobase  =  widget_base(bbase[i])
  plobutt =  widget_button(plobase, /no_release, value = '  PLOT  ', $
         uvalue = 'P'+ strtrim(i, 2))
endfor
keep = keyword_set(keep_param)? 1:0
base2 = widget_base(tlb, /frame)
b2 = cw_bgroup(base2, ['Select ALL', '  QUIT  ', ' FINISHED '], $
   /row, uval= 'OPTIONS' )
widget_control, tlb, /realize
ptr = ptr_new( make_array(nwdf, value = 1, /int))
ptrp = (npar gt 0)? ptr_new(param) : ptr_new(0)
info = {ptr:ptr, ptrp:ptrp, butt:butt, lcquit:0, nwdf: nwdf, wdf :wd, $
  xrange : xrange, zoom : zzz, para:para, npar:npar, param: ptrp, keep:keep}
widget_control, tlb, set_uvalue = info
for i = 0, nwdf-1 do widget_control, butt[i], set_button=1

xmanager, 'select_wdf', tlb
; val = selected values
   val = *ptr
; valp = parameters
   valp = *ptrp
   ptr_free, ptr, ptrp
if val[0] eq -1 then begin
  wdfsel = -1
  nwdfsel = -1
  return
endif

ok = where(val gt 0, nwdfsel)
wdfsel = (nwdfsel gt 0) ? wd[ok] : 0
if npar gt 0 then param= (nwdfsel gt 0) ? valp[ok]:0
end
