; ------------------------------------------------------------------------ 
; $Id: y_gridinteg.pro,v 1.4 2007/06/06 19:34:34 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
 Function y_gridinteg, array, yin, init, flagin, reverse = reverse
;
;+
;
;        integrate an array in the y direction and return the integral
;        assume that the array is a grid array with values of the function
;        at the end points 
;        format: integ = y_gridinteg(array, y, init, flag)
;        where;
;          Array is the two dimensional array to be integrated
;          y     is the array of y values for array
;          init is the initial value of the integral; this is usually 
;               zero unless there is a previous value of the integral
;               If init is a scalar or undefined then init = zero
;          flag  is an integration - default is 1
;               integ = integral(array * dy)             flag = 1 
;               integ = integral(array * y dy)           flag = 2 
;               integ = integral(array * y*y dy)         flag = 3 
;             or 
;               is a string executable which can be executed to produce 
;               a function of y. Then 
;                integ = integral(array*g(y) dy)
;               where g(y) = flag
;                 examples are: flag = 'sin(y)'
;                               flag = 'y*sqrt(y)'
;                               flag = 'my_function(y, 4.5)'
;                  (flag may be any executable idl or user function)
;
;        Note: Either x or y may be used in the function call but x = y
;              ie: flag = 'sin(x)' gives the same answer as flag = 'sin(y)'
;                  Both x and y will be set to the integration variable
;                  Use chastr to multiply the array by coordinate value.
;          reverse - if set, integrate from larger values
;-
 COMPILE_OPT hidden
 on_error, 2
         if n_elements (array) eq 0 or n_elements (yin) eq 0 then begin
           print, ' Y_GRIDinteg - Array and ordinate must be defined'
           return, 0.0
         endif
         a = size (array)
         if a(0) ne 2 or a(2) ne n_elements (yin) then begin
           Print, ' Y_GRIDinteg ERROR Array must be two dimensional with same'
           print, ' number of elements in second dimension as y'
           return, 0.0
         endif
         ny = a(2)
;   calculate dy/2
         dy2 = 0.5*(yin(1:ny-1)-yin(0:ny-2))
;    calculate the average value of the function between y values
         aarray= array(*, 0:ny-2) + array(*, 1:ny-1)
;    check reverse
         if keyword_set (reverse) then rev = ny-1 else rev = 0
;    check on initial value
         if n_elements (init) lt a(1) then array(*,rev)=0 else $
               array(*,rev)=init(0:a(1)-1)
         if n_elements (flagin) eq 0 then flag = 1 else flag = flagin(0)
         ss = size(flag(0))
         if ss(1) eq 7 then begin
           x =  yin(0:ny-2)+dy2
           y = x
           ok  = execute('tempy ='+flag)
           ntemp = n_elements(tempy)
           if ntemp lt (ny-1) then begin
             print, 'Integral kernel is too short. Additional values will be' $
                     + ' added'
             if ntemp gt 1 then begin
               temp = tempy
               tempy= make_array(ny-1, value = tempy(0))
               tempy(0:ntemp-1) = temp
             endif else tempy= make_array(ny-1, value = tempy(0))
           endif
           if keyword_set (reverse) then $
           for iy=ny-2, 0, -1  do $
               array(*,iy)=array(*,iy+1)-aarray(*,iy)*dy2(iy)*tempy(iy) $
           else for iy=0,ny-2 do $
               array(*,iy+1)=array(*,iy)+aarray(*,iy)*dy2(iy)*tempy(iy)
           x = 0 & y = 0
           return, array
         endif else if flag gt 3 or flag lt 1 then begin
           PRINT, ' INTEG flag for Y_GRIDinteg must be in the range 0, 2'
           print, ' Flag set to zero'
           flag = 0
         endif else flag = round(flag)-1
         if flag eq 0 then begin
           if keyword_set (reverse) then $
           for iy=ny-2, 0, -1  do $
               array(*,iy)=array(*,iy+1)-aarray(*,iy)*dy2(iy) $
           else for iy=0,ny-2 do $
               array(*,iy+1)=array(*,iy)+aarray(*,iy)*dy2(iy)
         endif else begin
           tempy = yin(0:ny-2)+dy2
           if flag gt 1 then tempy = tempy*tempy
           if keyword_set (reverse) then $
           for iy=ny-2, 0, -1  do $
               array(*,iy)=array(*,iy+1)-aarray(*,iy)*dy2(iy)*tempy(iy) $
           else for iy=0,ny-2 do $
               array(*,iy+1)=array(*,iy)+aarray(*,iy)*dy2(iy)*tempy(iy)
         endelse
         return, array
         end
