; ------------------------------------------------------------------------
; $Id: d2fstr.pro,v 1.1 2009/09/02 16:46:54 lpmix Exp $
; ------------------------------------------------------------------------
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
;+
; NAME:
;       D2FSTR
;
; PURPOSE:
;       Converts film density to photons/micron^2 or keV/micron^2
;       Density vs Energy or Density vs Wavelength is input
;       Input is an image with wavelength on the horizontal axis.
;       NOTE: D2F handles waveform data
;
; CATEGORY:
;       Analysis
;
; CALLING SEQUENCE:
;
;       D2FSTR, STRin [, STRout] [, FILM = film] [, TENSION = tension]
;          [, ENERGY = ENERGY] [, MODIFY = modify] [, FOG = fog]
;
; INPUTS:
;       STRin   Structure array number for the input dataset
;               Unless ENERGY is set units are Density vs Wavelength(A) 
;       STRout  Structure array number for the output dataset
;               Output abscissa units are the same as the input units
;               Default: STRout = STRin  ; original data will be lost
;	
; KEYWORD PARAMETERS:
;       FILM    Film type
;               0 - Kodak 101-07
;               1 - DEF
;               2 - SB-392 (SB-5)
;               3 - RAR 2492
;               4 - RAR 2495
;               5 - RAR 2497
;               Default:  FILM = 1
;       ANGLE   Angle of incidence
;               Default: ANGLE = !pi/2
;       ENERGY  Indicates input data is a function of energy rather than 
;               wavelength
;               Energy = 1 ==>> Abscissa units are eV
;               Energy = 3 ==>> Abscissa units are keV
;       TENSION Tension parameter to the spline function 
;       MODIFY  Interpolation scheme at edges
;               0 - Cubic Spline across all edges
;               1 - Linear Interpolation across all edges but spline where data
;       FOG     Film Fog Level 
;               Subtracted from the amplitude
;               DEFAULT: FOG = 0.0
;
; OUTPUTS:
;       The output is the photon flux from the theory of Henke
;
; PROCEDURE:
;       The VIDA routine of William E. Nelson, Science and Engineering Associates
;       has been modified for this PFIDL procedure.
;
; EXAMPLE:
;       Convert the density versus energy(ev) in waveform array 3 to photon flux
;         D2FSTR, 3, /energy
;
; MODIFICATION HISTORY:
; 	Written by:	L. Paul Mix, August 31, 2009
;                       From the VIDA, D2I, routine by 
;                         William E. Nelson
;                         Science and Engineering Associates
;                         March, 1990
;
;-
Pro D2FSTR, wdfin, wdfout, film = filmin, tension = tension, energy = energy, $
          modify = modify, angle= ang, fog = fog
;
on_error, 1
;
if n_params() eq 0 then begin
  print,'% Format: D2FSTR, WDFIN, WDFOUT [, FILM = film] [, TENSION = tension]'
  print, '         [, ENERGY = energy] [, ANGLE = angle] [, FOG= FOG] [, /MODIFY]'
  return
endif
if n_elements(tension) eq 0 then tension = 1.0
if n_elements(modify) eq 0 then modify = 0.0
if n_elements(fog) eq 0 then fog = 0.0

IF n_elements(filmin) eq 0 then Film = 1 else film = filmin

; Film Response
film_types =['101-07', 'DEF', 'SB-5', 'RAR 2492', 'RAR 2495', 'RAR 2497']
IF (film lt 0) or (film gt 5) THEN BEGIN
    PRINT, '% D2FSTR:  Bad film type.'
    print,'         Film=0: 101-07'
    print,'         Film=1: DEF'
    print,'         Film=2: SB-5'
    print,'         Film=3: RAR 2492'
    print,'         Film=4: RAR 2495'
    print,'         Film=5: RAR 2497'
    RETURN
endif else  film_type = film_types[film]


wdf = strvalid(wdfin, /valid, npoint = npoint)
if npoint eq 0 then begin
   print, '% Format: STRin must be a valid structure array'
   return
endif
if n_elements (wdfout) gt 0 then wdfout = get_save_str(wdfout[0], wdf) $
   else wdfout = wdf

film -=  1

if n_elements(ang) eq 0 then ang=!pi/2.0 else ang = ang[0]

x = get_array(wdf, 1)
y = get_array(wdf, -1)

if keyword_set(energy) then begin
  if energy eq 3 then x= 12.398419/x $
     else x =  12398.419/x
endif 
if fog gt 0 then y -= fog

Flux = henkep_film_resp(film, x, y, ang, tension, modify)

if wdfout ne wdf then xfrstr, wdf, wdfout
put_array, wdfout, flux, -1
chastr, wdfout, ylab = 'Flux(photons/um^2)', clab= 'Henke Cal of '+strcomment(wdf)
end
