; ------------------------------------------------------------------------ 
; $Id: edit_header.pro,v 1.3 2007/06/29 13:42:44 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
;+
; NAME:
;       EDIT_HEADER
;
; PURPOSE:
;       View an XRD shot description.
;       Edit or generate a shot description and save to the shot_config file.
;
; CALLING SEQUENCE:
;       EDIT_HEADER [, SHOT_NUMBER]  [, Line_of_Sight]
;  or 
;       EDIT_HEADER [, STRUCTURE]
;
; INPUTS:
;       SHOT_NUMBER   - Existing shot number in the shot_config file.
;       Line_of_Sight - Existing Line_of_sight diagnostic array descriptor.
;                       In general this is a string variable, but it may be a 
;                       number.	
;       STRUCTURE     - Structure or structure array number of type 101
;                       Initial values are taken from this structure.
; KEYWORD PARAMETERS:
;       Structure     - Type 101 structure with final values from the session.
;                       This will be stored in array number GET_MAXSTR().
;       Reset         - If set, provide a blank widget form to generate a shot
;                       description.  If reset is greater than 1, then the form
;                       will contain "reset" detectors
;                       (ie reset =5 means generate a form with 5 detectors)
;       Scratch       - File ID of a scratch file
;                       Default = current pff file
;          
;  Structure Labels
;    type          - 101
;    shot          - Accelerator shot number 
;    LOS           - String descriptor of the diagnostic line of sight (LOS).
;    Date          - String for current date
;    Comment       - String comment about this diagnostic setup
;    CLAB          - Comment Label for structure directory
;    NDet          - number of detectors
;    dist          - Distance from detectors to the source
;    source        - Type of X-ray source: 0 = Hohlraum, 1= Pinch Shot
;    src_area      - Source Area, Ignored for Pinch Shot
;    az_ang        - Azimutha angle between LOS and surface normal
;                    Ignored for Pinch Shot
;    Aper_ht       - Aperature height, Ignored for hohlraum shot
;    Pinch_ht      - Total pinch length, Ignored for hohlraum shot
;    elevation     - Angle between a horizontal plane and the diagnostic LOS
;    Signal[*]     - DAS signal name 
;    cathode[*]    - Cathode identifier
;    calibration[*]- Calibration identifier for cathode
;    diameter[*]   - Cathode active diameter
;    filter[*]     - Filter identifier
;    energy_bin[*] - Energy bins boundaries for the unfold
;
; OUTPUTS:
;       Diagnostic array descriptor structure 
;       This is stored in the maximum structure and optionally returned with 
;       the keyword, STRUCTURE.
;
; EXAMPLE:
;       Edit shot 681, los 5
;       edit_header, 681, 5
;
; MODIFICATION HISTORY:
;       Written by:     L. P. Mix, January 22, 2001
;
;-
;/remote/lpmix/consult/dlfehl/xrd
 
 pro edit_header_event, event
     on_error,1
     WIDGET_CONTROL,Event.Id,GET_UVALUE=Ev
     if size(/type, ev) lt 7 then begin
       help, /str, event
       print, ' No handler for this event'
       widget_control, event.top, /destroy
       return
     endif
      
   case Ev of 

  'Shot_Type': BEGIN
                 if event.select eq 1 then begin
                   widget_control, event.top, get_uvalue = info, /no_copy
                   (*info.ptr).source = event.value
                   widget_control, info.whoh, map = (event.value eq 0)
                   widget_control, info.wpin, map = (event.value eq 1)
                   widget_control, event.top, set_uvalue = info, /no_copy
                 endif
               END
   ELSE: BEGIN
           Print, 'No Handler for event: ', ev
           widget_control, event.top, /destroy
           return
         END
   ENDCASE

 end
 pro edit_header_button_event, event
@xrd_common_data.dat
     on_error,1
     WIDGET_CONTROL,Event.Id,GET_UVALUE=Ev
     widget_control, event.top, get_uvalue = info, /no_copy
     xrd_update_read, info
     hstr = *info.ptr
     
   case Ev of

   'PRINT'   : BEGIN
               print_xrd, hstr
               END

   'DRAFT'   : BEGIN
               ier = 1
               widget_control, event.top, tlb_get_offset = offset
               if info.scr_fid ne 0 then $
                 wriifl, hstr , /full, fil= info.scr_fid, ierror = ier $
               else wwarning,[ 'No Scratch Files Are Open', $
                   'Structure will be saved in GET_MAXSTR()', $
                   'Save with: wriifl, get_maxstr()'], offset = offset+ 10
               if ier eq 0 then $
                 print, '% EDIT_HEADER: Shot header written to scratch file'
               END
   'SAVE'    : BEGIN
               ier = 1
               ok = where (info.fid eq hfid, nok)  
               if hfid eq 0 or nok eq 0 then begin
                 print, '% EDIT_HEADER: Header PFF file has not been found.
               endif else wriifl, hstr, fileid = hfid, /full, ier = ier
               if ier eq 0 then $
                 print, '% EDIT_HEADER: Shot header written to Shot_Config'
               END
   'EXIT'    : BEGIN
               widget_control, event.top, /destroy
               return
               END
   ENDCASE
   widget_control, event.top, set_uvalue = info, /no_copy
 end

 pro xrd_update_read, info, detectors = detectors
   on_error, 1
   for i = 0, (*info.ptr).ndet-1 do begin
     widget_control, info.wsign[i], get_value = a
     (*info.ptr).signal[i]= strupcase(a)
     widget_control, info.wcath[i], get_value = a
     (*info.ptr).cathode[i]= strupcase(a)
     widget_control, info.wcalib[i], get_value = a
     (*info.ptr).calibration[i]= strupcase(a)
     widget_control, info.wfilt[i], get_value = a
     (*info.ptr).filter[i]=  strupcase(a)
     widget_control, info.wdia[i], get_value = a
     (*info.ptr).diameter[i]= a
     widget_control, info.wener[i], get_value = a
     (*info.ptr).energy_bin[i]=a
   endfor
   i = (*info.ptr).ndet
   widget_control, info.wener[i], get_value = a
   (*info.ptr).energy_bin[i]=a
     
   if keyword_set(detectors) then return
   widget_control, info.wshot, get_value = shot 
     (*info.ptr).shot = shot
   widget_control, info.wlos, get_value = los
     los = strupcase(strtrim(los, 2))
     (*info.ptr).los = los
   widget_control, info.wcomment, get_value = comment
     (*info.ptr).comment = comment[0]
   widget_control, info.wdist, get_value = a
     (*info.ptr).dist = a
   widget_control, info.wtype, get_value = a
     (*info.ptr).source = a
   widget_control, info.warea, get_value = a
     (*info.ptr).src_area = a
   widget_control, info.wazang, get_value = a
     (*info.ptr).az_ang = a
   widget_control, info.wpinch, get_value = a
     (*info.ptr).pinch_ht = a
   widget_control, info.waper, get_value = a
     (*info.ptr).aper_ht = a
   widget_control, info.wele, get_value = a
     (*info.ptr).elevation = a
   date  = thin_date()
   clabel = 'Shot'+string (shot, format='(i5)') + ', LOS '
   if strlen(los) eq 0 then los = '   '
   clabel = clabel + los +';;'+date+', '+comment
   (*info.ptr).date = date
   (*info.ptr).clab = clabel
 
 end

 pro xrd_detect_val, info
;  set up the widgets for each detector's values
   on_error, 1
   if (*info.ptr).ndet gt info.maxcol then begin 
     ncol = (*info.ptr).ndet/info.maxcol
     if (*info.ptr).ndet mod info.maxcol gt 0 then ncol = ncol+1
   endif else ncol = 1
   info.ncol = ncol
   ii = 0
   for j = 0, ncol-1 do begin
     info.wbas[j] =  widget_base(info.detval, /column, /map)
     ie = (ii+info.maxcol-1) < ((*info.ptr).ndet - 1)
     for i = ii, ie do begin
; make detector base to put signal name, cathode, filter and diameter info
       db = widget_base(info.wbas[j], /map, /column, /frame)
       txt = widget_label(db, value = 'Detector '+ strtrim(i+1,2)+' Values')
       info.wsign[i] = cw_field(db, value = (*info.ptr).signal[i], /row, $
                 /string, Title = 'DAS Name: ', xsize =10)
       info.wcath[i] = cw_field(db, value = (*info.ptr).cathode[i], /row, $
                 /string, Title = 'Cathode ID: ', xsize =10)
       info.wcalib[i] = cw_field(db, value = (*info.ptr).calibration[i], /row, $
                 /string, Title = 'Calibration ID: ', xsize =8)
       info.wfilt[i] = cw_field(db, value = (*info.ptr).filter[i], /row, $
                 /string, Title = 'Filter ID: ', xsize =10)
       info.wdia[i] = cw_field(db, value = (*info.ptr).diameter[i], /row, $
                 /float, Title = 'Diameter: ', xsize =8)
     endfor
     ii = ii + info.maxcol
   endfor
   info.wener[0]= cw_field(info.wbe, value = (*info.ptr).energy_bin[0], /row, $
                  /float, Title = 'Minimum Energy:', xsize =8)
   for i = 1, (*info.ptr).ndet do $
     info.wener[i]= cw_field(info.wbe, value= (*info.ptr).energy_bin[i], /row, $
                /float, Title = 'Bin '+strtrim(i,2)+' Max Energy:', xsize =8)

   end



 Pro xrd_det_numb_event, event
   widget_control, event.top, get_uvalue = info, /no_copy
   on_error, 1
   ndet = event.index+1
   if ndet ne (*info.ptr).ndet then begin
     oldndet = (*info.ptr).ndet
     widget_control, event.top, update = 0
     xrd_update_read, info, /detectors
     for i = 0, info.ncol-1 do widget_control, info.wbas[i], /destroy
     for i = 0, oldndet do widget_control, info.wener[i], /destroy
     (*info.ptr).ndet = ndet
     xrd_detect_val, info
   endif
   widget_control, event.top, update = 1
   widget_control, event.top, set_uvalue = info, /no_copy
   end

 Pro edit_header, shotin, losin, structure = structure_out, reset = reset, $
      scratch= scr_fid, no_block = no_block

;
   on_error, 1
@xrd_common_data.dat
;
   
;  check to see if pff has been initialized
   if (n_elements(initxrd) eq 0) then initxrd=0
   if (initxrd eq 0) then begin
     lcnew = 1
     pff_info, fid= fid, curr = curr, nfile = nfile, /quiet
     if not keyword_set(scr_fid)  then begin
       if nfile gt 0 then begin
         ok = where (curr gt 0)
         scr_fid= fid[ok[0]]
       endif else scr_fid =0
     endif
     init_xrd
   endif else lcnew = 0
   if initxrd eq 0 then begin
     print, '% EDIT_HEADER: XRD analysis is not supported by this System'
     return
   endif

   max_detector = 12
   maxcol = 3

   pff_info, fid= fid, curr = curr, nfile = nfile, /quiet
   if lcnew eq 0 then begin
     if not keyword_set(scr_fid)  then begin
       if nfile gt 0 then begin
         ok = where (curr gt 0)
         scr_fid= fid[ok[0]]
       endif else scr_fid =0
     endif
   endif
   if nfile eq 0 then fid = 0

   wbas = make_array(max_detector/maxcol+1, /long)
   wsign = make_array(max_detector, /long)
   wcath = make_array(max_detector, /long)
   wcalib = make_array(max_detector, /long)
   wfilt = make_array(max_detector, /long)
   wdia = make_array(max_detector, /long)

   diameter = make_array(max_detector)
   energy_bin = make_array(max_detector+1)
   signal = make_array(max_detector, /string)
   cathode = make_array(max_detector, /string)
   calibration = make_array(max_detector, /string)
   filter = make_array(max_detector, /string)

   if n_elements (shotin) eq 0 then shot = 0 else shot = shotin
   if n_elements (losin) eq 0 then los = ''  else los = losin
   if size(shot, /type) lt 8 then begin
     if hfid ne 0 and shot gt 0 then begin
       str = 'Shot'+string (shot, format='(i5)') + ', LOS '
       if strlen(los) gt 0 then str = str + strupcase(strtrim(los, 2))+';;'
       get_dir, hfid, str, line=line, dataset = dataset, number = number
        
       if number eq 1 then readifl, shot, /structure, dataset[0], fil=hfid  $
         else if number gt 1 then begin
         pick_one, line, n
         readifl, shot, /structure, line[n], fil = hfid
       endif
     endif
   endif
   if size (shot, /type) eq 8 then begin
     str = temporary(shot)
     if str.type ne 101 then begin
   print, '% Edit_Header: Only shot configuration (101) structures may be input.
       return
     endif
     shot= str.shot
     los = str.los
     comm = str.comment
     ndet = str.ndet
     dist = str.dist
     stype = str.source
     srcarea = str.src_area
     az_ang = str.az_ang
     Aper_hght = str.aper_ht
     pinhght = str.pinch_ht
     ele_ang= str.elevation
     signal[0:ndet-1]= str.signal[0:ndet-1]
     cathode[0:ndet-1]= str.cathode[0:ndet-1]
     calibration[0:ndet-1]= str.calibration[0:ndet-1]
     diameter[0:ndet-1] = str.diameter[0:ndet-1]
     filter[0:ndet-1]= str.filter[0:ndet-1]
     energy_bin[0:ndet]= str.energy_bin[0:ndet]

   endif else begin    
     comm = ''
     ndet = 5
     dist = 2237.75
     stype = 1
     srcarea = 90.
     az_ang = 12.0
     aper_hght =  10
     pinhght = 20.
     ele_ang = 11.0
     signal[0:ndet-1] = strupcase(['xrd5?1km', 'xrd5?2_v', 'xrd5?3zn', $
        'xrd5?4be', 'xrd5?5be'])
     cathode[0:ndet-1] = strupcase(['c26', 'j20', 'c60', 'c63', 'c64'])
     calibration[0:ndet-1]='9906'
     diameter[0:ndet-1]= 4.896
     filter[0:ndet-1] = strupcase(['960731-3e', '961125-1h', '990608-1', $
        '960626-1d', '961106-a1'])
     energy_bin[0:ndet] = [137.000, 284, 513, 1020, 1500, 2300]
     
   endelse

   tlb = widget_base(/column, /map, title = 'XRD Shot Description')
   main = widget_base(tlb, /row, /map)
   lb  = widget_base(main, /column, /map)
   rb  = widget_base(main, /column, /map)
   txt = widget_label(lb, value = 'XRD Array Parameters')
   txt = widget_label(rb, value = 'Individual Detector Parameters')
   wshot = cw_field(lb, value = shot, /row, /long, xsize = 6, $
                   Title = 'Shot Number:' )
   wlos  = cw_field(lb, value = los, /row, /long,  $
                   Title = 'Line-of-Sight:', xsize =12)
   wdist = cw_field(lb, value = dist, /row, /float,   $
                   Title = 'Source Distance:', xsize =12)
   wndet = widget_droplist(lb, value =strtrim(1+indgen(max_detector), 2),  $
                   title = 'Number of Detectors:', $
                   event_pro ='xrd_det_numb_event')
   
   txt = widget_label(lb, value = ' *********************** ')
   txt = widget_label(lb, value = 'Source Parameters')

   wtype = cw_bgroup(lb, ['Hohlraum', 'Pinch'] , set_value = stype, $
                 /row, /exclusive, /No_release, Label_top = 'Shot Type', $
                 uvalue ='Shot_Type')

   ww = widget_base(lb)
   whoh = widget_base(ww, map= (stype eq 0), /col)
   wpin = widget_base(ww, map= (stype eq 1), /col)
   warea = cw_field(whoh, value = srcarea, /row, /float,   $
                   Title = 'Source area (mm2):', xsize =8)
   wazang = cw_field(whoh, value = az_ang, /row, /float,   $
                   Title = 'Azimuth angle:', xsize =8)
   waper = cw_field(wpin, value = aper_hght, /row, /float,   $
                   Title = 'Aperature Height:', xsize =8)
   wpinch = cw_field(wpin, value = pinhght, /row, /float,   $
                   Title = 'Pinch Height:', xsize =8)
   wele = cw_field(lb, value = ele_ang, /row, /float,   $
                   Title = 'LOS Elevation:', xsize =8)
;  widget base for the energy cuts
   wbe = widget_base(lb, /col)
   wener = make_array(max_detector+1, /long)
   detval = widget_base(rb, /map, /row, /frame)
   wcomment = cw_field(rb, value = comm, /row, /string, $
                   Title = 'Comment:', xsize = 60)
   bbase =  widget_base(rb, /row, /map, frame =2)
   wbut = widget_button(bbase, value= 'Print to File', $
           event_pro = 'edit_header_button_event', uvalue = 'PRINT')
   wbut = widget_button(bbase, value= 'Save to Scratch File', $
           event_pro = 'edit_header_button_event', uvalue = 'DRAFT')
   wbut = widget_button(bbase, value= 'Save to Shot_Config', $
           event_pro = 'edit_header_button_event', uvalue = 'SAVE')
   wbut = widget_button(bbase, value= 'Quit/Exit', $
           event_pro = 'edit_header_button_event', uvalue = 'EXIT')
   date  = systime()
   date  = strmid(date, 4, 12)+ strmid(date, 19, 5)
   clabel = 'Shot'+string (shot, format='(i5)') + ', LOS '
   llos = strupcase(strtrim(los, 2))
   if strlen(los) eq 0 then llos = '   '
   clabel = clabel + llos +';;'+date+', '+comm
   str = { TYPE: 101, shot: shot, los:los, date:date, comment: comm, $
           Ndet: ndet, dist:dist, source: stype, src_area:srcarea, $
           az_ang:az_ang, aper_ht: aper_hght, pinch_ht: pinhght, $
           elevation:ele_ang, signal:signal,  cathode:cathode, $
           calibration: calibration, diameter:diameter, filter:filter, $
           energy_bin:energy_bin , clab: clabel}
   ptr = ptr_new(str)
   info = { $
          maxcol:maxcol, max_detector: max_detector, $
          ncol: 0L, detval: detval, wbe: wbe, wshot: wshot, wlos: wlos,  $
          wndet:wndet, ndet:ndet, wdist:wdist, wtype:wtype, $
          whoh: whoh, wpin:wpin, warea:warea, wazang:wazang, $
          wele:wele, wener: wener, wpinch:wpinch, waper:waper, $
          wbas:wbas, wsign:wsign, wcath:wcath, wfilt:wfilt, wdia: wdia, $
          wcalib:wcalib, wcomment : wcomment, $
          fid: fid, scr_fid:scr_fid, ptr: ptr}

;  set up individual detector widgets
   xrd_detect_val, info

   widget_control, tlb , /realize
   widget_control, wndet , set_droplist_select = ndet-1
   widget_control, tlb , set_uvalue = info, /no_copy
   xmanager , 'edit_header', tlb, no_block = no_block
   str = *ptr 
   ptr_free, ptr
   i2s, str, 82
   structure_out = temporary(str)
 end
