; ------------------------------------------------------------------------ 
; $Id: print_xrd_actual.pro,v 1.3 2007/06/29 13:42:44 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
 pro print_xrd_actual, str, unit
;+
; NAME:
;       PRINT_XRD_ACTUAL
;
; PURPOSE:
;       PRINT an XRD shot description to a unit.
;
; CALLING SEQUENCE:
;       PRINT_XRD, STRUCTURE, UNIT
;
; INPUTS:
;       STRUCTURE     - Structure or structure array number of type 101
;                       Initial values are taken from this structure.
;       UNIT          - Unit for the print
; KEYWORD PARAMETERS:
;       FILENAME      - Name of file to write data.
;                       If print is set and NoDelete is not set, 
;                       the default file is the scratch file used for some 
;                       directory PFIDL functions.
;       APPEND        - if set data will be appended to filename
;       Terminal      - If set, send output to terminal
;                       If less than zero then also print to file
;       PRINT         - If set, file will be printed.
;                       File will be deleted if unless NODELETE is set.
;       NODELETE      - If set filename will not be deleted.
;       DESTINATION   - Valid printer destination for the print file.
;                       This destination will not be saved by LPRINT
;                     Note: Default LPRINT destination must be valid if 
;                           DESTINATION is not provided.
;                           Use SET_PRINTER to set the destination.
;          
;  Structure Labels
;    See XRD_HEADER for description of type 101 Labels
;
; OUTPUTS:
;       Print header information to a file and spool to a printer
;
; EXAMPLE:
;       Print shot 681, los 5
;       print_xrd, 681, 5
;
; MODIFICATION HISTORY:
;       Written by:     L. P. Mix, January 29, 2001
;
;-
;/remote/lpmix/consult/dlfehl/xrd
 
on_error, 1
;
   t = size(str, /type)
   if t ne 8 then begin
    print, '% PRINT_XRD_ACTUAL: Structure must be defined.'
    return
   endif
   
   if n_elements (unit) eq 0 then begin
     print, '% PRINT_XRD_ACTUAL: UNIT must be defined.'
     return
   endif
;

   if str.type ne 101 then begin
     print, '% PRINT_XRD_ACTUAL: Only type 101 structures may be output: ', $
       strtrim (str.type, 2)
     return
   endif

   printf, unit, ' Shot Number: ', strtrim(str.shot, 2)
   printf, unit, ' Diagnostic Line-of-Sight: ', strtrim(str.los, 2)
   printf, unit, ' Date: ', strtrim(str.date, 2)
   printf, unit, ' Comment: ', strtrim(str.Comment, 2)
   n = str.ndet
   printf, unit
   printf, unit, ' *** Source Parameters ***'
   printf, unit, ' Detector to Source Distance: ', strtrim(str.dist, 2), ' cm'
   if str.source eq 0 then begin
;    Hohlraum Source
     printf, unit, ' Hohlraum Source Area: ' , strtrim(str.src_area, 2), ' mm^2'
     printf, unit, ' Azimuth Angle between LOS and Surface: ' , $
       strtrim(str.Az_ang, 2), ' degrees'
   endif else if  str.source eq 1 then begin
;    Pinch source
     printf, unit, ' Pinch Length Observed: ' , strtrim(str.aper_ht, 2), ' cm'
     printf, unit, ' Total Pinch Length: ' , strtrim(str.Pinch_ht, 2), ' cm'
   endif else begin
     printf, unit, ' % PRINT_XRD_ACTUAl:  Unable to determine source type'
     return
   endelse

   printf, unit, ' LOS Angle to Horizontal: ', $
     strtrim(str.elevation, 2), ' degrees'

   printf, unit
   printf, unit, ' *** Detector Parameters ***
   printf, unit, ' Number of Detectors: ' , strtrim(str.ndet, 2)
   printf, unit, ' Signal Name  Cathode ID  Calibration  Diam(mm)   Filter ID'
   printf, unit, ' -----------  ----------  -----------  --------   ---------'
   for i = 0, n-1 do printf, unit, str.signal[i], str.cathode[i], $
     str.calibration[i], str.Diameter[i], str.filter[i], format = $
     '(a12, a12, a13, f10.3, a12)'
   printf, unit
   printf, unit, ' *** Unfold Energy Bins ***'
   printf, unit, ' Minimum Energy: ', strtrim(str.energy_bin[0], 2), ' eV'
   Printf, unit, ' Maximum Energy For Each Bin'
   printf, unit, ' Bin   Energy(eV)'
   printf, unit, ' ---   ----------
   for i = 1, n do printf, unit, string(i, format='(i3)'), '.', $
      string(str.energy_bin[i], format='(f13.1)')

   end
