; ------------------------------------------------------------------------ 
; $Id: rersp.pro,v 1.6 2008/02/27 22:33:14 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
 pro rersp, filenamein, wdfin, comments = comments, run_number = runn, $
        thickness = thickness, materials = materials, error = error_frac , $
        nmaterial = nmat, Photon_energy = photon, ier = ier

;+
; NAME:
;       ReRSP
;
; PURPOSE:
;       Read a text file used for XRD cathode and filter calibrations.
;       Format:  Skip first line
;                Skip any line with # as first non-blank character
;                Skip that begins with the word "END "
;                Blank lines are ignored.
;                Searches are not case sensitive.
;                Filter files must have material and thickness information
;                  in the file for FIT_FILTER.
;
;                Data is after the following two lines:
;                
;       photon           trans-
;       energy          mission         % error          Run #
;
;                 Data is read to the end of file
;
; CATEGORY:
;       XRD analysis
;
; CALLING SEQUENCE:
;       RERSP, FILENAME [, WDF] 
;
; OPTIONAL INPUTS:
;       FILENAME: File for analysis
;                 If omitted or zero, then PICKFILE will be used.
;                 If a number between 1 and get_maxwdf() it will be interpreted
;                 as WDF and PICKFILE will be used.
;       WDF:      WDF to store the calibration transmission data.
;                 WDF must be provided.
;
; KEYWORD Parameters:
;       COMMENTS:  File header lines
;       NMATERIAL: Number of materials
;       Materials: Materials symbol or name
;       Thickness: Thickness of material in Microns
;       Error:     Fractional error in the transmission expressed as a decimal 
;                  number.  This is stored in a WDF array, specified by error.
;                  If error is not set, the error will be stored in get_maxwdf()
;       Photon_Energy: Array of Photon Energies for the data
;                  Run numbers below correspond to these energies.
;       Run_number:Run number for each energy transmission measurement
;                  Run number is a string.  A string array is returned
;                  
;
; EXAMMPLE:
;       Read a response file using pickfile into wdf array 2.
;         ReRSP, 2
;
;       Read file "Be-Va_000731-BV1_B0008.rsp" into wdf array 6
;         ReRSP, 'Be-Va_000731-BV1_B0008.rsp', 6
;
; FILE FORMAT:
;
; First line is always ignored and assumed to be a comment line.
; Any line with "end" as the first 3 non-blank characters is ignored.
; Any line with "#" as the first non blank character is ignored.
;   The first line and all lines beginning with a # will be saved in a 
;   comment array returned with the comments keyword.
; Any line which is not ignored is assumed to be a material line.
;   Material lines must have at least 2 fields delimited by white space
;   (tabs or spaces).
;   The first field is assumed to be an element symbol or an element name 
;     or a material name.
;   The second field is assumed to be the material thickness in microns.
;   The third field is optional and will be ignored.
; Comments and materials will cease to be processed when the letters "photon" 
;   are the first non-blank characters.
; The word "energy" will be expected to be the first word in the following line.
; The remainder fo the file is assumed to be composed of 4 columns of numbers
;   representing the photon energy, the transmission, the percent error, and 
;   the run number. 
;
; MODIFICATION HISTORY:
;       Written by:     L. P. Mix, December 13, 2001
;       
;-

on_error, 1
 ier = 1
 if n_params() eq 0 then begin
   print, '% Format:  RERSP, FILENAME [, WDF]'
   return
 endif
 ftyp = size (filenamein, /type)
 if ftyp eq 0 then begin
   print, '% RERSP: WDF array must be defined'
   return
 endif
 if ftyp lt 7 then begin
   wdf = long(filenamein)
   if n_elements (wdfin) eq 0 then wdfin = wdf else if wdf gt 0 then begin
;    since they put in a wdf we will assume that they meant to have a filename
     print, '% RERSP: Filename must contain a valid file name.'
     return
   endif
   ftyp = 0
 endif else if ftyp gt 7 then $
   print, '% RERSP: Filename must contain a valid file name or wdf array.'
 work = get_maxwdf()
 if n_elements(wdfin) gt 0 then wdf = wdfin else begin
   Print, '% RERSP: WDF array must be provided' 
   return
 endelse
 if ftyp eq 0 then begin
   filename = dialog_pickfile(/must_exist, /read, filter = '*.rsp') 
   if strlen(filename) lt 1 then begin
     print, '% RERSP: No File was Selected - RETURNING'
     return
   endif
 endif else filename = filenamein
 if wdf lt 1 or wdf gt work then begin
   Print, '% RERSP: Valid WDF array must be provided.'
   return
 endif
 openr,unit,filename, /get_lun, error = ierr
 if ierr ne 0 then return
 ncom = 1L
 nmat = 0L
 mxcom = 128L
 mxmat = 128L
 comments = make_array(mxcom, /string)
 materials = make_array(mxmat, /string)
 thickness = make_array(mxmat)

 str = ''
 readf, unit, str
 comments[0]=strtrim(str, 2)
 readf, unit, str
 str = strlowcase(strtrim(str,2))
 while(strpos(str, 'photon') ne 0 and $
    not eof(unit)) do begin
   if strlen(str) gt  0  then begin
     if strpos(str, '#') eq 0 then begin
       if ncom eq mxcom then begin
         comments=[comments , make_array(mxcom, /string)]
         mxcom = mxcom+mxcom
       endif
       comments[ncom] = strtrim (str, 2)
       ncom = ncom + 1
     endif else if strpos(str, 'end') ne 0 then begin
       mat = strsplit(str, /extract)
       if n_elements(mat) lt 2 then begin
         Print, '% RERSP: Unable to parse the following line:'
         print, str
         free_lun, unit
         return
       endif 
       if mxmat eq nmat then begin
         materials=[materials , make_array(mxmat, /string)]
         thickness=[thickness , make_array(mxmat)]
         mxmat = mxmat+mxmat
       endif
       materials[nmat]= mat[0]
       thickness[nmat]= mat[1]

       nmat = nmat+1
     endif
   endif
   readf, unit, str
   str = strlowcase(strtrim(str,2))
 endwhile
 if nmat gt 0 then begin
   materials = materials[0:nmat-1]
   thickness = thickness[0:nmat-1]
 endif else  begin
   materials = materials[0]
   thickness = thickness[0]
 endelse
 strold = str
 if not eof (unit) then readf, unit, str else begin
   print, '% RERSP: End-of-file before data'
   free_lun, unit
   return
 endelse
 str = strlowcase(strtrim(str,2))
 if (strpos(str, 'energy') ne 0) then begin
   print, '% RERSP: PHOTON was read but ENERGY not read.'
   print, strold
   print, str
   free_lun, unit
   return
 endif
 filnam = strpos(filename, path_sep(), /reverse_search)
 if filnam ge 0 then filnam = strmid(filename, filnam+1) else $
   filnam = filename
 comments = comments[0:ncom-1]
;  Change the following since run numbers are strings. 
  readasc, unit, 4, xvalue, yval, np= np,  formatin= 0, /string
 if strpos(strlowcase(filename), 'xrd') ne 0 then $
   ylab = 'XRD Response(A/MW)' else ylab = 'Transmission'
 free_lun, unit
 if n_elements(xvalue) lt 2 then begin
   print, '% RERSP: Problem reading data'
   return
 endif
 xvalue = float(xvalue)
 photon = xvalue
 runn = reform (yval[*, 2])
 yvalue = float (reform(yval[*,0]))
 i2w, xvalue, yvalue, wdf, /xydata, file = filename,  $
     clab=filnam, xlab = 'Energy(eV)', ylab= ylab
 if keyword_set (error_frac) then begin
   if error_frac eq wdf then error_frac = work else $
     if error_frac lt 0 or error_frac gt work then error_frac = work
;   convert percent to fraction
   err = float(reform(yval[*,1]))/100.
   i2w, xvalue, err, error_frac, /xydata, file = filename, $
     clab='Error - '+filnam, xlab = 'Energy(eV)', ylab= 'Fractional error'
 endif
 ier = 0
 end
