; ------------------------------------------------------------------------ 
; $Id: write_filter.pro,v 1.4 2008/02/27 22:35:29 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
;+
;  This procedure writes a WDF array to a PFF file.
;  Format: WRITE_FILTER, WD1 [, FILEID] [, FILEID = FID] [, QUIET = quiet]
;          [, Name = Filter_Name] [, IDENTIFIER = filterid]
;          [, CALIBRATION = calibration] [, DATE= fit_date]
;             
;  where:
;    WD1    - WDF array number
;    FILEID - PFF file ID (Keyword is ignored if this parameter is present.
;
;  Keywords:
;    FILEID - PFF file id.  If FILEID  is 0, the current PFF 
;             file is used.  If FILEID is not specified, then the file 
;             1. If the unfold routine has been initialized, use the filter     
;                response file.
;             2. Use the file 'filter_resp.pff' that is open, in the current
;                directory, or in the directory, !dir/../xrd
;             Keyword is ignored if parameter is provided.
;    QUIET  - If set, file will be written with no interaction.
;             Default: quiet = 0
;             Note:  If quiet is set, dataset comment must contain a string:
;             "Filter: filter_id, Cal: Calibration" where 
;                Filter ID is the filter id for the filter and  
;                Calibration is a calibration id.
;    NAME   - Filter name or comment
;             Example: 2 micron Kimfol
;    IDENTIFIER - Filter identification
;             Example: 961001-B5
;    CALIBRATION - Calibration identifier
;             Example: B0008
;    DATE   - Fit date
;             Example: Jan 17, 2002
;  
;  Examples:
;  Write WDF array 1 to the current PFF file.
;    write_filter, 1, 0
;  Write WDF array 5 to the current PFF file with a filter name of pary3.5
;    and an id of 990406-4c.
;    write_filter, 5, name ='pary3.5', ID = '990406-4c'
;
;-

 pro write_filter_event, event
   widget_control, event.id, get_uvalue = ev
   CASE Ev OF

     'RUN/QUIT': BEGIN
        widget_control, event.top, get_uvalue= info
        (*info.ptr).lcquit= event.value
        if event.value eq 0 then begin
          widget_control, info.wfilt, get_value = filt
          filt = strtrim(filt[0], 2)
          widget_control, info.wcal, get_value = calib
          calib= strtrim(calib[0], 2)
          if (strlen(filt) eq 0) or (strlen(calib) eq 0) then begin
            widget_control, event.top, tlb_get_offset=offset
            wwarning, ['Filter and Calibration Identifiers must be provided', $
                       'Please insure you have identifiers for both'], $
                        offset = offset +10
            return
          endif
          (*info.ptr).filtid= filt
          (*info.ptr).calib= calib
          widget_control, info.wname, get_value = name
          (*info.ptr).name= strtrim(name[0], 2)
          widget_control, info.wdate, get_value = date
          (*info.ptr).date= strtrim(date[0], 2)
        endif
        widget_control, event.top, /destroy
      END
      else: help, /str, event
   endcase
 end
 
 pro write_filter, wdfin, fidin, name = namein, fileid = fileid, quiet=quiet, $
         identifier = filterid, calibration= calibration, date = fitdate
       
on_error, 1
@xrd_common_data.dat
common pffcom, initpff, pff_exe

      if n_params() eq 0 then begin
        print,'% Format:  WRITE_FILTER, WD1 [, FID] [, FILEID = FID] [/QUIET]
        print,'             [, Name = filter_name] [, IDENTIFIER = filter_id]
        print,'             [, CALIBRATION = calibration] [, DATE= fitdate]'
        return
      endif

      if (n_elements(initpff) eq 0) then init_pff else $
        if initpff eq 0 then init_pff
      if (initpff eq 0) then begin
        print, '% WRITE_FILTER: PFF is not supported by this system'
        return
      endif

 showpff, filename = fname, ufid = opfid, numb = nfile, /quiet
 fidmiss = max(opfid)+1
 if n_elements(fidin) gt 0 then fid= fidin[0] else begin
   if n_elements(fileid) gt 0 then fid = fileid[0] else begin
     if n_elements(ffid) gt 0 then begin
       fid = ffid[0]
       ok = where(fid eq opfid, nok)
       if nok eq 0 then fid = fidmiss else begin
         if fname[ok[0]] eq filter_file then begin
           setpff, fid
           fid = 0
          endif else fid = fidmiss
       endelse
     endif else fid = fidmiss
   endelse
 endelse
 if fid eq  fidmiss then  begin
   ok = where(strpos(fname, 'filter_resp.pff') ge 0, nok)
   if nok eq 0 then begin
     sep = path_sep()
     new = !dir+sep+'..'+sep+'xrd'+sep+'filter_resp.pff'
     openpff, new, error = error
     if error ne 0 then begin
       print, '% WRITE_FILTER: Not able to open file: ', new
       return
     endif
   endif else fid = opfid[ok[0]]
 endif
 if keyword_set(fid) then begin
   setpff, fid, ier
   if ier ne 0 then begin
     print, '% WRITE_FILTER: Unable to set file id:', fid, ier
     return
   endif
 endif

 wdf = wdvalid(wdfin[0], /valid)
 if wdf[0] gt 0 then wdf = wdf[0] else begin
   print, '% WRITE_FILTER: Valid waveform array must be provided', wdfin[0]
   return
 endelse
 
 comm = wdcomment(wdf)

 ok = strpos(comm , ' - ')
 if ok lt 0 then comment ='' else begin
   comment = strtrim(strmid(comm, ok[0]+3), 2)
   comm = strmid(comm, 0, ok)
 endelse
 commu = strupcase(comm)
 
 ss = strtrim(strsplit(commu, ',', /extract), 2)
 nss = n_elements(ss)
 indss = indgen(nss)

 def_name = ''
 def_filterid = ''
 def_cal = ''
 def_fitdate =''
 sstr = 'FILTER:'
 ok = (where(strpos(ss, sstr) ge 0, nok))[0]
 if nok gt 0 then begin
   def_filterid = strtrim(strmid(ss[ok], strpos(ss[ok], sstr)+7), 2)
   indss[ok]=-1
 endif else indss = [indss, 0]

 ok = (where(strpos(ss, 'CAL:') ge 0, nok))[0]
 if nok gt 0 then begin
   def_cal= strtrim(strmid(ss[ok], strpos(ss[ok], 'CAL:')+4), 2)
   indss[ok]=-1
 endif
 isp = ''
 i = 0
 while indss[i] ge 0 and i lt nss do begin
   def_name = def_name+isp+ss[i]
   i = i+1
   isp = ', '
 endwhile
 isp = ''
 iend = i
 i = nss-1
 while indss[i] ge 0 and i ge 0  do begin
   def_fitdate = ss[i]+isp+def_fitdate
   i = i-1
   isp = ', '
 endwhile

 if n_elements(namein) eq 0 then name =strtrim(def_name, 2) else $
     name = strtrim(namein, 2)
 if n_elements(calibration) eq 0 then calib =strtrim(def_cal, 2)  else $
     calib = strtrim(calibration, 2)
 if n_elements(fitdate) eq 0 then date = strtrim(def_fitdate, 2)  else $
     date = strtrim(fitdate, 2)
 if n_elements(filterid) eq 0 then filtid = strtrim(def_filterid, 2)  else $
     filtid = strtrim(filterid, 2)
 
 
 if keyword_set(quiet) then begin
   if strlen(filtid) eq 0  or strlen(calib) eq 0 then begin
     print, '% WRITE_FILTER: Filter ID must be provided if QUIET is set'
     return
   endif 
   if strlen(calib) eq 0 then begin
     print, '% WRITE_CATHODE: Calibration name must be provided if QUIET is set'
     return
   endif
 endif else begin
   TLB = WIDGET_BASE( COLUMN=1, MAP=1, $
      TITLE='Enter Filter Parameters') 
   cinfo = comm
   outcom = comment
   if strlen(comment) gt 0 then begin
;    use a dummy to save the end.
     comm = comment
     x = strlen(comm)
     while x gt 0 do begin
       ok = strpos(comm, ';;')
       if ok gt 0 then cinfo = [cinfo, strmid(comm, 0, ok[0])] else $
         cinfo = [cinfo, strmid(comm, 0)]
       if ok gt 0 then comm = strmid(comm, ok+2) else comm=''
       x = strlen(comm)
     endwhile
   endif
   ncomm = n_elements(cinfo)
   for i = 0, ncomm -1 do ntxt = widget_label(tlb, value= cinfo[i])
 
   ntxt = widget_label(tlb, value= ' - - - - - - - - - - - - - - - - ')
   wname = cw_field(tlb, value = name, /row, /string, $
      title ='Filter Name:')
   wfilt = cw_field(tlb, value = filtid, /row, /string, $
      title ='Filter Ident:')
   wcal = cw_field(tlb, value = calib, /row, /string, $
      title ='Calibration Ident:')
   wdate = cw_field(tlb, value = date, /row, /string, $
      title ='Fit Date:')
   bnames = [ $
    '  Write/Save  ', $
    '  Cancel/Quit ' ]
   BGROUP = CW_BGROUP( TLB, bnames, $
      ROW=1, $
      SPACE=100, $
      UVALUE='RUN/QUIT')
   str = {lcquit: 0, name:name, filtid:filtid, calib:calib, date:date }
   ptr = ptr_new(str)
   info = {ptr: ptr, wname:wname,wfilt:wfilt, wcal:wcal, wdate:wdate}
   widget_control, tlb, /realize
   widget_control, tlb, set_uvalue=info
   xmanager, 'write_filter', tlb
   lcquit =(*ptr).lcquit 
   name = (*ptr).name 
   filtid = (*ptr).filtid 
   calib = (*ptr).calib 
   date = (*ptr).date 

   ptr_free, ptr
   if lcquit ne 0 then return
 endelse
 tmpnam = strtrim(name, 2)
 tmplab = 'Filter: ' + filtid 
 get_dir, comment = comment , number = number
 if number gt 0 then begin
   ok = where (strpos(comment, tmplab) ge 0, nok)
   if nok gt 0 then begin
     comment = ['The following datasets are already in the file:', comment[ok]]
     wok_notok, lcquit, comment, 'Filter Already Exists'
     if lcquit eq 1 then begin
       Print, '% WRITE_FILTER: Nothing written to the PFF file.'
       return
     endif
   endif
 endif

 tmp = tmplab
 if strlen (calib) gt 0 then tmp= tmp+', '+calib
 if strlen (date) gt 0 then tmp= tmp+', '+date
 if strlen (tmpnam) gt 0 then tmp = tmp+ ', '+tmpnam
 if strlen(outcom) gt 0 then tmp = tmp+' - '+ outcom

 wri, wdf, clab = tmp , ap = 103, /full, tlabel = 'Filter Trans'
 end
